/*!
*  filename: ej.web.all.js
*  version : 12.3
*  Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
*/

window.ej = window.Syncfusion = window.Syncfusion || {};

(function ($, ej, undefined) {
    'use strict';

    ej.consts = {
        NamespaceJoin: '-'
    };
    ej.TextAlign = {
        Center: 'center',
        Justify: 'justify',
        Left: 'left',
        Right: 'right'
    };
    ej.Orientation = { Horizontal: "horizontal", Vertical: "vertical" };

    ej.serverTimezoneOffset = 0;

    if (!Object.prototype.hasOwnProperty) {
        Object.prototype.hasOwnProperty = function (obj, prop) {
            return obj[prop] !== undefined;
        };
    }

    String.format = function () {
        var source = arguments[0];
        for (var i = 0; i < arguments.length - 1; i++)
            source = source.replace(new RegExp("\\{" + i + "\\}", "gm"), arguments[i + 1]);

        source = source.replace(/\{[0-9]\}/g, "");
        return source;
    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };
    // Function to create new class
    ej.defineClass = function (className, constructor, proto, replace) {
        /// <summary>Creates the javascript class with given namespace & class name & constructor etc</summary>
        /// <param name="className" type="String">class name prefixed with namespace</param>
        /// <param name="constructor" type="Function">constructor function</param>
        /// <param name="proto" type="Object">prototype for the class</param>
        /// <param name="replace" type="Boolean">[Optional]Replace existing class if exists</param>
        /// <returns type="Function">returns the class function</returns>
        if (!className || !proto) return undefined;

        var parts = className.split(".");

        // Object creation
        var obj = window, i = 0;
        for (; i < parts.length - 1; i++) {

            if (ej.isNullOrUndefined(obj[parts[i]]))
                obj[parts[i]] = {};

            obj = obj[parts[i]];
        }

        if (replace || ej.isNullOrUndefined(obj[parts[i]])) {

            //constructor
            constructor = typeof constructor === "function" ? constructor : function () {
            };

            obj[parts[i]] = constructor;

            // prototype
            obj[parts[i]].prototype = proto;
        }

        return obj[parts[i]];
    };

    ej.util = {
        getNameSpace: function (className) {
            /// <summary>Internal function, this will create namespace for plugins using class name</summary>
            /// <param name="className" type="String"></param>
            /// <returns type="String"></returns>
            var splits = className.toLowerCase().split(".");
            splits[0] === "ej" && (splits[0] = "e");

            return splits.join(ej.consts.NamespaceJoin);
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (ej.util.isNullOrUndefined(value)) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i, t, length = splits.length;

            for (i = 0; i < length; i++) {
                t = splits[i];
                if (i + 1 == length)
                    from[t] = value === undefined ? {} : value;
                else if (ej.isNullOrUndefined(from[t]))
                    from[t] = {};

                from = from[t];
            }

            return start;
        },

        isNullOrUndefined: function (value) {
            /// <summary>Util to check null or undefined</summary>
            /// <param name="value" type="Object"></param>
            /// <returns type="Boolean"></returns>
            return value === undefined || value === null;
        },
        print: function (element) {
            var $div = ej.buildTag('div')
            var elementClone = element.clone();
            $div.append(elementClone);
            var printWin = window.open('', 'pint', "'height=452,width=1024,tabbar=no");
            printWin.document.write('<!DOCTYPE html>');
            var links = $('head').find('link').add("style");
            if (ej.browserInfo().name === "msie") {
                var a = ""
                links.each(function (index, obj) {
                    if (obj.tagName == "LINK")
                        $(obj).attr('href', obj.href);
                    a += obj.outerHTML;
                });
                printWin.document.write('<html><head></head><body>' + a + $div[0].innerHTML + '</body></html>');
            }
            else {
                var a = ""
                printWin.document.write('<html><head>')
                links.each(function (index, obj) {
                    if (obj.tagName == "LINK")
                        $(obj).attr('href', obj.href);
                    a += obj.outerHTML;
                });
                printWin.document.writeln(a + '</head><body>')
                printWin.document.writeln($div[0].innerHTML + '</body></html>')
            }
            printWin.focus();
            setTimeout(function () {
                if (!ej.isNullOrUndefined(printWin.window)) {
                    printWin.print();
                    printWin.location.reload();
                    setTimeout(function () { printWin.close() }, 1000);
                }
            }, 1000);
        },
        ieClearRemover: function (element) {
            var searchBoxHeight = $(element).height();
            element.style.paddingTop = parseFloat(searchBoxHeight / 2) + "px";
            element.style.paddingBottom = parseFloat(searchBoxHeight / 2) + "px";
            element.style.height = "1px";
            element.style.lineHeight = "1px";
        },
        //To send ajax request
        sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.successHandler,
                error: ajaxOptions.errorHandler,
                beforeSend: ajaxOptions.beforeSendHandler,
                complete: ajaxOptions.completeHandler
            });
        },

        buildTag: function (tag, innerHtml, styles, attrs) {
            /// <summary>Helper to build jQuery element</summary>
            /// <param name="tag" type="String">tagName#id.cssClass</param>
            /// <param name="innerHtml" type="String"></param>
            /// <param name="styles" type="Object">A set of key/value pairs that configure styles</param>
            /// <param name="attrs" type="Object">A set of key/value pairs that configure attributes</param>
            /// <returns type="jQuery"></returns>
            var tagName = /^[a-z]*[0-9a-z]+/ig.exec(tag)[0];

            var id = /#([a-z]+[-_0-9a-z]+)/ig.exec(tag);
            id = id ? id[id.length - 1] : undefined;

            var className = /\.([a-z]+[-_0-9a-z ]+)/ig.exec(tag);
            className = className ? className[className.length - 1] : undefined;

            return $(document.createElement(tagName))
                .attr(id ? { "id": id } : {})
                .addClass(className || "")
                .css(styles || {})
                .attr(attrs || {})
                .html(innerHtml || "");
        },
        _preventDefaultException: function (el, exceptions) {
            if (el) {
                for (var i in exceptions) {
                    if (exceptions[i].test(el[i])) {
                        return true;
                    }
                }
            }

            return false;
        },

        //Gets the maximum z-index in the document
        getMaxZindex: function () {
            var maxZ = 1;
            maxZ = Math.max.apply(null, $.map($('body *'), function (e, n) {
                if ($(e).css('position') == 'absolute' || $(e).css('position') == 'fixed')
                    return parseInt($(e).css('z-index')) || 1;
            })
            );
            if (maxZ == undefined || maxZ == null)
                maxZ = 1;
            return maxZ;
        },

        //To prevent default actions for the element
        blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },

        //To get dimensions of the element when its hidden
        getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            if ($hidden) {
                var prop = { visibility: 'hidden', display: 'block' };
                var tmp = [];
                $hidden.each(function () {
                    var temp = {}, name;
                    for (name in prop) {
                        temp[name] = this.style[name];
                        this.style[name] = prop[name];
                    }
                    tmp.push(temp);
                });
                value = /(outer)/g.test(method) ?
                $(element)[method](true) :
               $(element)[method]();

                $hidden.each(function (i) {
                    var temp = tmp[i], name;
                    for (name in prop) {
                        this.style[name] = temp[name];
                    }
                });
            }
            return value;
        },
        //Get triggers when transition End
        transitionEndEvent: function () {
            var transitionEnd = {
                '': 'transitionend',
                'webkit': 'webkitTransitionEnd',
                'Moz': 'transitionend',
                'O': 'otransitionend',
                'ms': 'MSTransitionEnd'
            };

            return transitionEnd[ej.userAgent()];
        },
        //Get triggers when transition End
        animationEndEvent: function () {
            var animationEnd = {
                '': 'animationend',
                'webkit': 'webkitAnimationEnd',
                'Moz': 'animationend',
                'O': 'webkitAnimationEnd',
                'ms': 'animationend'
            };

            return animationEnd[ej.userAgent()];
        },
        //To return the start event to bind for element
        startEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchstart" : "mousedown";
        },
        //To return end event to bind for element
        endEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchend" : "mouseup"
        },
        //To return move event to bind for element
        moveEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? ($.support.hasPointer && !ej.isMobile()) ? "ejtouchmove" : "touchmove" : "mousemove";
        },
        //To return cancel event to bind for element
        cancelEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "touchcancel" : "mousecancel";
        },
        //To return tap event to bind for element
        tapEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "tap" : "click";
        },
        //To return tap hold event to bind for element
        tapHoldEvent: function () {
            return (ej.isTouchDevice() || $.support.hasPointer) ? "taphold" : "click";
        },
        //To check whether its Device
        isDevice: function () {
            if (ej.getBooleanVal($('head'), 'data-ej-forceset', false))
                return ej.getBooleanVal($('head'), 'data-ej-device', this._device());
            else
                return this._device();
        },
        //To check whether its windows web view
        isWindowsWebView: function () {
            return location.href.indexOf("x-wmapp") != -1;
        },
        _device: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        //To check whether its Mobile
        isMobile: function () {
            return ((/iphone|ipod|android|blackberry|opera|mini|windows\sce|palm|smartphone|iemobile/i.test(navigator.userAgent.toLowerCase()) && /mobile/i.test(navigator.userAgent.toLowerCase()))) || (ej.getBooleanVal($('head'), 'data-ej-mobile', false) === true);
        },
        //To check whether its Tablet
        isTablet: function () {
            return (/ipad|xoom|sch-i800|playbook|tablet|kindle/i.test(navigator.userAgent.toLowerCase())) || (ej.getBooleanVal($('head'), 'data-ej-tablet', false) === true) || (!ej.isMobile() && ej.isDevice());
        },
        //To check whether its Touch Device
        isTouchDevice: function () {
            return (('ontouchstart' in window || (window.navigator.msPointerEnabled && ej.isMobile())) && this.isDevice());
        },
        //To get the outerHTML string for object
        getClearString: function (string) {
            return $.trim(string.replace(/\s+/g, " ").replace(/(\r\n|\n|\r)/gm, "").replace(new RegExp("\>[\n\t ]+\<", "g"), "><"));
        },
        //Get the attribute value with boolean type of element
        getBooleanVal: function (ele, val, option) {
            /// <summary>Util to get the property from data attributes</summary>
            /// <param name="ele" type="Object"></param>
            /// <param name="val" type="String"></param>
            /// <param name="option" type="GenericType"></param>
            /// <returns type="GenericType"></returns>
            var value = $(ele).attr(val);
            if (value != null)
                return value.toLowerCase() == "true";
            else
                return option;
        },
        //Gets the Skew class based on the element current position
        _getSkewClass: function (item, pageX, pageY) {
            var itemwidth = item.width();
            var itemheight = item.height();
            var leftOffset = item.offset().left;
            var rightOffset = item.offset().left + itemwidth;
            var topOffset = item.offset().top;
            var bottomOffset = item.offset().top + itemheight;
            var widthoffset = itemwidth * 0.3;
            var heightoffset = itemheight * 0.3;
            if (pageX < leftOffset + widthoffset && pageY < topOffset + heightoffset)
                return "e-m-skew-topleft";
            if (pageX > rightOffset - widthoffset && pageY < topOffset + heightoffset)
                return "e-m-skew-topright";
            if (pageX > rightOffset - widthoffset && pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottomright";
            if (pageX < leftOffset + widthoffset && pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottomleft";
            if (pageX > leftOffset + widthoffset && pageY < topOffset + heightoffset && pageX < rightOffset - widthoffset)
                return "e-m-skew-top";
            if (pageX < leftOffset + widthoffset)
                return "e-m-skew-left";
            if (pageX > rightOffset - widthoffset)
                return "e-m-skew-right";
            if (pageY > bottomOffset - heightoffset)
                return "e-m-skew-bottom";
            return "e-m-skew-center";
        },
        //Removes the added Skew class on the element
        _removeSkewClass: function (element) {
            $(element).removeClass("e-m-skew-top e-m-skew-bottom e-m-skew-left e-m-skew-right e-m-skew-topleft e-m-skew-topright e-m-skew-bottomleft e-m-skew-bottomright e-m-skew-center e-skew-top e-skew-bottom e-skew-left e-skew-right e-skew-topleft e-skew-topright e-skew-bottomleft e-skew-bottomright e-skew-center");
        },
        _touchStartPoints: function (evt, object) {
            if (evt) {
                var point = evt.touches ? evt.touches[0] : evt;
                object._distX = 0;
                object._distY = 0;
                object._moved = false;
                object._pointX = point.pageX;
                object._pointY = point.pageY;
            }
        },
        _isTouchMoved: function (evt, object) {
            if (evt) {
                var point = evt.touches ? evt.touches[0] : evt,
                deltaX = point.pageX - object._pointX,
                deltaY = point.pageY - object._pointY,
                timestamp = Date.now(),
                newX, newY,
                absDistX, absDistY;
                object._pointX = point.pageX;
                object._pointY = point.pageY;
                object._distX += deltaX;
                object._distY += deltaY;
                absDistX = Math.abs(object._distX);
                absDistY = Math.abs(object._distY);
                return !(absDistX < 5 && absDistY < 5);
            }
        },
        //To bind events for element
        listenEvents: function (selectors, eventTypes, handlers, remove, pluginObj, disableMouse) {
            for (var i = 0; i < selectors.length; i++) {
                ej.listenTouchEvent(selectors[i], eventTypes[i], handlers[i], remove, pluginObj, disableMouse);
            }
        },
        //To bind touch events for element
        listenTouchEvent: function (selector, eventType, handler, remove, pluginObj, disableMouse) {
            var event = remove ? "removeEventListener" : "addEventListener";
            var jqueryEvent = remove ? "off" : "on";
            var elements = $(selector);
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                switch (eventType) {
                    case "touchstart":
                        ej._bindEvent(element, event, eventType, handler, "mousedown", "MSPointerDown", "pointerdown", disableMouse);
                        break;
                    case "touchmove":
                        ej._bindEvent(element, event, eventType, handler, "mousemove", "MSPointerMove", "pointermove", disableMouse);
                        break;
                    case "touchend":
                        ej._bindEvent(element, event, eventType, handler, "mouseup", "MSPointerUp", "pointerup", disableMouse);
                        break;
                    case "touchcancel":
                        ej._bindEvent(element, event, eventType, handler, "mousecancel", "MSPointerCancel", "pointercancel", disableMouse);
                        break;
                    case "tap": case "taphold": case "ejtouchmove": case "click":
                        $(element)[jqueryEvent](eventType, handler);
                        break;
                    default:
                        if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)
                            pluginObj["_on"]($(element), eventType, handler);
                        else
                            element[event](eventType, handler, true);
                        break;
                }
            }
        },
        //To bind events for element
        _bindEvent: function (element, event, eventType, handler, mouseEvent, pointerEvent, ie11pointerEvent, disableMouse) {
            if ($.support.hasPointer)
                element[event](window.navigator.pointerEnabled ? ie11pointerEvent : pointerEvent, handler, true);
            else
                element[event](eventType, handler, true);
        },
        _browser: function () {
            return (/webkit/i).test(navigator.appVersion) ? 'webkit' : (/firefox/i).test(navigator.userAgent) ? 'Moz' : (/trident/i).test(navigator.userAgent) ? 'ms' : 'opera' in window ? 'O' : '';
        },
        styles: document.createElement('div').style,
        /**
       * To get the userAgent Name     
       * @example             
       * &lt;script&gt;
       *       ej.userAgent();//return user agent name
       * &lt;/script&gt         
       * @memberof AppView
       * @instance
       */
        userAgent: function () {
            var agents = 'webkitT,t,MozT,msT,OT'.split(','),
            t,
            i = 0,
            l = agents.length;

            for (; i < l; i++) {
                t = agents[i] + 'ransform';
                if (t in ej.styles) {
                    return agents[i].substr(0, agents[i].length - 1);
                }
            }

            return false;
        },
        addPrefix: function (style) {
            if (ej.userAgent() === '') return style;

            style = style.charAt(0).toUpperCase() + style.substr(1);
            return ej.userAgent() + style;
        },
        //To Prevent Default Exception

        //To destroy the mobile widgets
        destroyWidgets: function (element) {
            var dataEl = $(element).find("[data-role *= ejm]");
            dataEl.each(function (index, element) {
                var $element = $(element);
                var plugin = $element.data("ejWidgets");
                if (plugin)
                    $element[plugin]("destroy");
            });
        },
        //Get the attribute value of element
        getAttrVal: function (ele, val, option) {
            /// <summary>Util to get the property from data attributes</summary>
            /// <param name="ele" type="Object"></param>
            /// <param name="val" type="String"></param>
            /// <param name="option" type="GenericType"></param>
            /// <returns type="GenericType"></returns>
            var value = $(ele).attr(val);
            if (value != null)
                return value;
            else
                return option;
        },

        // Get the offset value of element
        getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },

        // Z-index calculation for the element
        getZindexPartial: function (element, popupEle) {
            if (!ej.isNullOrUndefined(element) && element.length > 0) {
                var parents = element.parents(), bodyEle;
                bodyEle = $('body').children();
                if (!ej.isNullOrUndefined(element) && element.length > 0)
                    bodyEle.splice(bodyEle.index(popupEle), 1);
                $(bodyEle).each(function (i, ele) { parents.push(ele); });

                var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                    if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
                }));
                if (!maxZ || maxZ < 10000) maxZ = 10000;
                else maxZ += 1;
                return maxZ;
            }
        },

    };

    $.extend(ej, ej.util);

    // base class for all ej widgets. It will automatically inhertied
    ej.widgetBase = {
        droppables: { 'default': [] },
        resizables: { 'default': [] },

        _renderEjTemplate: function (selector, data, index, prop) {
            var type = null;
            if (typeof selector === "object" || selector.startsWith("#") || selector.startsWith("."))
                type = $(selector).attr("type");
            if (type) {
                type = type.toLowerCase();
                if (ej.template[type])
                    return ej.template[type](this, selector, data, index, prop);
            }
            return ej.template.render(this, selector, data, index, prop);
        },

        destroy: function () {

            if (this._trigger("destroy"))
                return;

            if (this.model.enablePersistence)
                this.persistState();

            try {
                this._destroy();
            } catch (e) { }

            var arr = this.element.data("ejWidgets") || [];
            for (var i = 0; i < arr.length; i++) {
                if (arr[i] == this.pluginName) {
                    arr.splice(i, 1);
                }
            }
            if (!arr.length)
                this.element.removeData("ejWidgets");

            while (this._events) {
                var item = this._events.pop(), args = [];

                if (!item)
                    break;

                for (var i = 0; i < item[1].length; i++)
                    if (!$.isPlainObject(item[1][i]))
                        args.push(item[1][i]);

                $.fn.off.apply(item[0], args);
            }

            this._events = null;

            this.element
                .removeClass(ej.util.getNameSpace(this.sfType))
                .removeClass("e-js")
                .removeData(this.pluginName);

            this.element = null;
            this.model = null;
        },

        _on: function (element) {
            if (!this._events)
                this._events = [];
            var args = [].splice.call(arguments, 1, arguments.length - 1);

            var handler = {}, i = args.length;
            while (handler && typeof handler !== "function") {
                handler = args[--i];
            }

            args[i] = ej.proxy(args[i], this);

            this._events.push([element, args, handler, args[i]]);

            $.fn.on.apply(element, args);

            return this;
        },

        _off: function (element, eventName, selector, handlerObject) {
            var e = this._events, temp;
            if (!e || !e.length)
                return this;
            if (typeof selector == "function") {
                temp = handlerObject;
                handlerObject = selector;
                selector = temp;
            }
            for (var i = 0; i < e.length; i++) {
                var arg = e[i],
                r = arg[0].length && (!handlerObject || arg[2] === handlerObject) && arg[1][0] === eventName && (!selector || arg[1][1] === selector) && $.inArray(element[0], arg[0]) > -1;
                if (r) {
                    $.fn.off.apply(element, handlerObject ? [eventName, selector, arg[3]] : [eventName, selector]);
                    e.splice(i, 1);
                    break;
                }
            }

            return this;
        },

        // Client side events wire-up / trigger helper.
        _trigger: function (eventName, eventProp) {
            var fn = null, returnValue, args, clientProp = {};
            $.extend(clientProp, eventProp)

            if (eventName in this.model)
                fn = this.model[eventName];

            if (fn) {
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }

                if ($.isFunction(fn)) {

                    args = ej.event(eventName, this.model, eventProp);

                    returnValue = fn.call(this, args);

                    // sending changes back - deep copy option should not be enabled for this $.extend 
                    if (eventProp) $.extend(eventProp, args);

                    if (args.cancel || !ej.isNullOrUndefined(returnValue))
                        return returnValue === false || args.cancel;
                }
            }

            var isPropDefined = Boolean(eventProp);
            eventProp = eventProp || {};
            eventProp.originalEventType = eventName;
            eventProp.type = this.pluginName + eventName;

            args = $.Event(eventProp.type, ej.event(eventProp.type, this.model, eventProp));

            this.element.trigger(args);

            // sending changes back - deep copy option should not be enabled for this $.extend 
            if (isPropDefined) $.extend(eventProp, args);

            if (ej.isOnWebForms && args.cancel == false && this.model.serverEvents && this.model.serverEvents.length)
                ej.raiseWebFormsServerEvents(eventName, eventProp, clientProp);

            return args.cancel;
        },

        setModel: function (options, forceSet) {
            // check for whether to apply values are not. if _setModel function is defined in child,
            //  this will call that function and validate it using return value

            if (this._trigger("modelChange", { "changes": options }))
                return;

            for (var prop in options) {
                if (!forceSet) {
                    if (this.model[prop] === options[prop]) {
                        delete options[prop];
                        continue;
                    }
                    if ($.isPlainObject(options[prop])) {
                        iterateAndRemoveProps(this.model[prop], options[prop]);
                        if ($.isEmptyObject(options[prop])) {
                            delete options[prop];
                            continue;
                        }
                    }
                }

                if (this.dataTypes) {
                    var returnValue = this._isValidModelValue(prop, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + prop + " - " + returnValue;
                }
                if (this.model.notifyOnEachPropertyChanges && this.model[prop] !== options[prop]) {
                    var arg = {
                        oldValue: this.model[prop],
                        newValue: options[prop]
                    };

                    options[prop] = this._trigger(prop + "Change", arg) ? this.model[prop] : arg.newValue;
                }
            }
            if ($.isEmptyObject(options))
                return;

            if (this._setFirst) {
                var ds = options.dataSource;
                if (ds) delete options.dataSource

                $.extend(true, this.model, options);
                if (ds) {
                    this.model.dataSource = (ds instanceof Array) ? ds.slice() : ds;
                    options["dataSource"] = this.model.dataSource;
                }
                !this._setModel || this._setModel(options);

            } else if (!this._setModel || this._setModel(options) !== false) {
                $.extend(true, this.model, options);
            }
            if ("enablePersistence" in options) {
                this._setState(options.enablePersistence);
            }
        },

        option: function (prop, value, forceSet) {
            if (!prop)
                return this.model;

            if ($.isPlainObject(prop))
                return this.setModel(prop);

            if (typeof prop === "string") {
                prop = prop.replace(/^model\./, "");
                var oldValue = ej.getObject(prop, this.model);

                if (value === undefined)
                    return oldValue;

                if (prop === "enablePersistence")
                    return this._setState(value);

                if (forceSet && value === ej.extensions.modelGUID) {
                    return this._setModel(ej.createObject(prop, ej.getObject(prop, this.model), {}));
                }

                if (forceSet || ej.getObject(prop, this.model) !== value)
                    return this.setModel(ej.createObject(prop, value, {}), forceSet);
            }
            return undefined;
        },

        _isValidModelValue: function (prop, types, options) {
            var value = types[prop], option = options[prop], returnValue;

            if (!value)
                return true;

            if (typeof value === "string") {
                if (value == "enum") {
                    options[prop] = option ? option.toString().toLowerCase() : option;
                    value = "string";
                }

                if (value === "array") {
                    if (Object.prototype.toString.call(option) === '[object Array]')
                        return true;
                }
                else if (value === "data") {
                    return true;
                }
                else if (value === "parent") {
                    return true;
                }
                else if (typeof option === value)
                    return true;

                return "Expected type - " + value;
            }

            if (option instanceof Array) {
                for (var i = 0; i < option.length; i++) {
                    returnValue = this._isValidModelValue(prop, types, option[i]);
                    if (returnValue !== true) {
                        return " [" + i + "] - " + returnValue;
                    }
                }
                return true;
            }

            for (var innerProp in option) {
                returnValue = this._isValidModelValue(innerProp, value, option);
                if (returnValue !== true)
                    return innerProp + " : " + returnValue;
            }

            return true;
        },

        _setState: function (val) {
            if (val === true) {
                this._persistHandler = ej.proxy(this.persistState, this);
                $(window).on("unload", this._persistHandler);
            } else {
                this.deleteState();
                $(window).off("unload", this._persistHandler);
            }
        },

        persistState: function () {
            var model = copyObject({}, this.model);

            if (this._ignoreOnPersist) {
                for (var i = 0; i < this._ignoreOnPersist.length; i++) {
                    delete model[this._ignoreOnPersist[i]];
                }
            }

            if (this._persistState) {
                model.customPersists = {};
                this._persistState(model.customPersists);
            }

            if (window.localStorage)
                window.localStorage.setItem(this.pluginName + this._id, JSON.stringify(model));
            else if (document.cookie)
                ej.cookie.set(this.pluginName + this._id, model);
        },

        deleteState: function () {
            if (window.localStorage)
                window.localStorage.removeItem(this.pluginName + this._id);
            else if (document.cookie)
                ej.cookie.set(this.pluginName + this._id, model, new Date());
        },

        restoreState: function (silent) {
            var value = null;
            if (window.localStorage)
                value = window.localStorage.getItem(this.pluginName + this._id);
            else if (document.cookie)
                value = ej.cookie.get(this.pluginName + this._id);

            if (value) {
                var model = ej.parseJSON(value);

                if (this._restoreState) {
                    this._restoreState(model.customPersists);
                    delete model.customPersists;
                }

                this.model = $.extend(true, this.model, model);
            }

            if (!silent && value && this._setModel)
                this._setModel(this.model);
        },
        // Get formatted text 
        formatting: function (formatstring, str, locale) {
            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
            locale = Globalize.cultures[locale] ? locale : "en-US";
            var s = formatstring;
            var frontHtmlidx, FrontHtml, RearHtml, lastidxval;
            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            FrontHtml = frontHtmlidx[0];
            RearHtml = lastidxval[1];
            if (typeof (str) == "string" && $.isNumeric(str))
                str = Number(str);
            if (formatstring.indexOf("{0:") != -1) {
                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                var formatVal = toformat.exec(formatstring);
                if (formatVal != null && str != null) {
                    if (FrontHtml != null && RearHtml != null)
                        str = FrontHtml + Globalize.format(str, formatVal[2], locale) + RearHtml;
                    else
                        str = Globalize.format(str, formatVal[2], locale);
                } else if (str != null)
                    str = str;
                else
                    str = "";
                return str;
            } else if (s.startsWith("{") && !s.startsWith("{0:")) {
                var fVal = s.split(""), str = (str || "") + "", strSplt = str.split(""), formats = /[0aA\*CN<>\?]/gm;
                for (var f = 0, f, val = 0; f < fVal.length; f++)
                    fVal[f] = formats.test(fVal[f]) ? "{" + val++ + "}" : fVal[f];
                return String.format.apply(String, [fVal.join("")].concat(strSplt)).replace('{', '').replace('}', '');
            } else if (this.data != null && this.data.Value == null) {
                $.each(this.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });
                return s;
            } else {
                return this.data.Value;
            }
        },
    };

    var iterateAndRemoveProps = function (source, target) {
        for (var prop in source) {
            if (source[prop] === target[prop])
                delete target[prop];
            if ($.isPlainObject(target[prop]) && $.isPlainObject(source[prop]))
                iterateAndRemoveProps(source[prop], target[prop]);
        }
    }

    ej.widget = function (pluginName, className, proto) {
        /// <summary>Widget helper for developers, this set have predefined function to jQuery plug-ins</summary>
        /// <param name="pluginName" type="String">the plugin name that will be added in jquery.fn</param>
        /// <param name="className" type="String">the class name for your plugin, this will help create default cssClas</param>
        /// <param name="proto" type="Object">prototype for of the plug-in</param>

        if (typeof pluginName === "object") {
            proto = className;
            for (var prop in pluginName) {
                var name = pluginName[prop];

                if (name instanceof Array) {
                    proto._rootCSS = name[1];
                    name = name[0];
                }

                ej.widget(prop, name, proto);

                if (pluginName[prop] instanceof Array)
                    proto._rootCSS = "";
            }

            return;
        }

        var nameSpace = proto._rootCSS || ej.getNameSpace(className);

        proto = ej.defineClass(className, function (element, options) {

            this.sfType = className;
            this.pluginName = pluginName;
            this.instance = pInstance;

            if (ej.isNullOrUndefined(this._setFirst))
                this._setFirst = true;

            this["ob.values"] = {};

            $.extend(this, ej.widgetBase);

            if (this.dataTypes) {
                for (var property in options) {
                    var returnValue = this._isValidModelValue(property, this.dataTypes, options);
                    if (returnValue !== true)
                        throw "setModel - Invalid input for property :" + property + " - " + returnValue;
                }
            }

            var arr = (element.data("ejWidgets") || []);
            arr.push(pluginName);
            element.data("ejWidgets", arr);

            for (var i = 0; ej.widget.observables && this.observables && i < this.observables.length; i++) {
                var t = ej.getObject(this.observables[i], options);
                if (t) ej.createObject(this.observables[i], ej.widget.observables.register(t, this.observables[i], this, element), options);
            }

            this.element = element.jquery ? element : $(element);
            this.model = copyObject(true, {}, proto.prototype.defaults, options);
            this.model.keyConfigs = copyObject(this.keyConfigs);

            this.element.addClass(nameSpace + " e-js").data(pluginName, this);

            this._id = element[0].id;

            if (!this.element.attr("tabIndex"))
                this.element.attr("tabIndex", "");

            if (this.model.enablePersistence) {
                this._persistHandler = ej.proxy(this.persistState, this);
                $(window).on("unload", this._persistHandler);
                this.restoreState(true);
            }

            this._init(options);

            if (typeof this.model.keyConfigs === "object" && !(this.model.keyConfigs instanceof Array)) {
                var requiresEvt = false;
                if (this.model.keyConfigs.focus)
                    this.element.attr("accesskey", this.model.keyConfigs.focus);

                for (var keyProps in this.model.keyConfigs) {
                    if (keyProps !== "focus") {
                        requiresEvt = true;
                        break;
                    }
                }

                if (requiresEvt && this._keyPressed) {
                    var el = element, evt = "keydown";

                    if (this.keySettings) {
                        el = this.keySettings.getElement ? this.keySettings.getElement() || el : el;
                        evt = this.keySettings.event || evt;
                    }

                    this._on(el, evt, function (e) {
                        if (!this.model.keyConfigs) return;

                        var action = keyFn.getActionFromCode(this.model.keyConfigs, e.which, e.ctrlKey, e.shiftKey, e.altKey);
                        var arg = {
                            code: e.which,
                            ctrl: e.ctrlKey,
                            alt: e.altKey,
                            shift: e.shiftKey
                        };
                        if (!action) return;

                        if (this._keyPressed(action, e.target, arg, e) === false)
                            e.preventDefault();
                    });
                }
            }
            if (this.pluginName == "ejGrid" && this.model.editSettings.showAddNewRow && this.model.editSettings.editMode == "normal")
                this._startAdd();
            this._trigger("create");
        }, proto);

        $.fn[pluginName] = function (options) {
            var opt = options, args;
            for (var i = 0; i < this.length; i++) {

                var $this = $(this[i]),
                    pluginObj = $this.data(pluginName),
                    isAlreadyExists = pluginObj && $this.hasClass(nameSpace),
                    obj = null;

                if (this.length > 0 && $.isPlainObject(opt))
                    options = ej.copyObject({}, opt);

                // ----- plug-in creation/init
                if (!isAlreadyExists) {
                    if (!options || typeof options === "object") {
                        new proto($this, options);
                    }
                    else {
                        throwError(pluginName + ": methods/properties can be accessed only after plugin creation");
                    }
                    continue;
                }

                if (!options) continue;

                args = [].slice.call(arguments, 1);

                if (this.length > 0 && args[0] && opt === "option" && $.isPlainObject(args[0])) {
                    args[0] = ej.copyObject({}, args[0]);
                }

                // --- Function/property set/access
                if ($.isPlainObject(options)) {
                    // setModel using JSON object
                    pluginObj.setModel(options);
                }

                    // function/property name starts with "_" is private so ignore it.
                else if (options.indexOf('_') !== 0
                    && !ej.isNullOrUndefined(obj = ej.getObject(options, pluginObj))
                    || options.indexOf("model.") === 0) {

                    if (!obj || !$.isFunction(obj)) {

                        // if property is accessed, then break the jquery chain
                        if (arguments.length == 1)
                            return obj;

                        //setModel using string input
                        pluginObj.option(options, arguments[1]);

                        continue;
                    }

                    var value = obj.apply(pluginObj, args);

                    // If function call returns any value, then break the jquery chain
                    if (value !== undefined)
                        return value;

                } else {
                    throwError(className + ": function/property - " + options + " does not exist");
                }
            }
            if (pluginName.indexOf("ejm") != -1)
                ej.widget.registerInstance($this, pluginName, className, proto.prototype);
            // maintaining jquery chain
            return this;
        };

        ej.widget.register(pluginName, className, proto.prototype);
    };

    $.extend(ej.widget, (function () {
        var _widgets = {}, _registeredInstances = [],

        register = function (pluginName, className, prototype) {
            if (!ej.isNullOrUndefined(_widgets[pluginName]))
                throwError("ej.widget : The widget named " + pluginName + " is trying to register twice.");

            _widgets[pluginName] = { name: pluginName, className: className, proto: prototype };
        },
        registerInstance = function (element, pluginName, className, prototype) {
            _registeredInstances.push({ element: element, pluginName: pluginName, className: className, proto: prototype });
        }

        return {
            register: register,
            registerInstance: registerInstance,
            registeredWidgets: _widgets,
            registeredInstances: _registeredInstances
        };

    })());

    ej.widget.destroyAll = function (elements) {
        if (!elements || !elements.length) return;

        for (var i = 0; i < elements.length; i++) {
            var data = elements.eq(i).data(), wds = data["ejWidgets"];
            if (wds && wds.length) {
                for (var j = 0; j < wds.length; j++) {
                    if (data[wds[j]] && data[wds[j]].destroy)
                        data[wds[j]].destroy();
                }
            }
        }
    };

    ej.cookie = {
        get: function (name) {
            var value = RegExp(name + "=([^;]+)").exec(document.cookie);

            if (value && value.length > 1)
                return value[1];

            return undefined;
        },
        set: function (name, value, expiryDate) {
            if (typeof value === "object")
                value = JSON.stringify(value);

            value = escape(value) + ((expiryDate == null) ? "" : "; expires=" + expiryDate.toUTCString());
            document.cookie = name + "=" + value;
        }
    };

    var keyFn = {
        getActionFromCode: function (keyConfigs, keyCode, isCtrl, isShift, isAlt) {
            isCtrl = isCtrl || false;
            isShift = isShift || false;
            isAlt = isAlt || false;

            for (var keys in keyConfigs) {
                if (keys === "focus") continue;

                var key = keyFn.getKeyObject(keyConfigs[keys]);
                if (keyCode === key.code && isCtrl == key.isCtrl && isShift == key.isShift && isAlt == key.isAlt)
                    return keys;
            }
            return null;
        },
        getKeyObject: function (key) {
            var res = {
                isCtrl: false,
                isShift: false,
                isAlt: false
            };
            key = key.split("+");
            for (var i = 0; i < key.length; i++) {
                if (key[i] === "ctrl")
                    res.isCtrl = true;
                else if (key[i] === "shift")
                    res.isShift = true;
                else if (key[i] === "alt")
                    res.isAlt = true;
                else res.code = parseInt(key[i], 10);
            }
            return res;
        }
    };

    ej.getScrollableParents = function (element) {
        return $(element).parentsUntil("body").filter(function () {
            return $(this).css("overflow") != "visible";
        }).add($(window));
    }
    ej.browserInfo = function () {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    browser.culture = {};
                    browser.culture.name = browser.culture.language = navigator.language || navigator.userLanguage;
                    if (typeof (Globalize) != 'undefined') {
                        var oldCulture = Globalize.culture();
                        var culture = (navigator.language || navigator.userLanguage) ? Globalize.culture(navigator.language || navigator.userLanguage) : Globalize.culture("en-US");
                        for (var i = 0; (navigator.languages) && i < navigator.languages.length; i++) {
                            culture = Globalize.culture(navigator.languages[i]);
                            if (culture.language == navigator.languages[i])
                                break;
                        }
                        Globalize.culture(oldCulture);
                        $.extend(true, browser.culture, culture);
                    }
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        return browser;
    };
    ej.eventType = {
        mouseDown: "mousedown touchstart",
        mouseMove: "mousemove touchmove",
        mouseUp: "mouseup touchend",
        mouseLeave: "mouseleave touchcancel",
        click: "click touchend"
    };

    ej.event = function (type, data, eventProp) {

        var e = $.extend(eventProp || {},
            {
                "type": type,
                "model": data,
                "cancel": false
            });

        return e;
    };

    ej.proxy = function (fn, context, arg) {
        if (!fn || typeof fn !== "function")
            return null;

        if ('bind' in fn && context)
            return arg ? fn.bind(context, arg) : fn.bind(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            return fn.apply(context || this, args);
        };
    };

    ej.hasStyle = function (prop) {
        var style = document.documentElement.style;

        if (prop in style) return true;

        var prefixs = ['ms', 'Moz', 'Webkit', 'O', 'Khtml'];

        prop = prop[0].toUpperCase() + prop.slice(1);

        for (var i = 0; i < prefixs.length; i++) {
            if (prefixs[i] + prop in style)
                return true;
        }

        return false;
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };
    var copyObject = ej.copyObject = function (isFirstDefault, target) {
        var start = 2, current, source;
        if (typeof isFirstDefault !== "boolean") {
            start = 1;
        }
        var objects = [].slice.call(arguments, start);
        if (start === 1) {
            target = isFirstDefault;
            isFirstDefault = undefined;
        }

        for (var i = 0; i < objects.length; i++) {
            for (var prop in objects[i]) {
                current = target[prop], source = objects[i][prop];

                if (source === undefined || current === source || objects[i] === source || target === source)
                    continue;
                if (source instanceof Array) {
                    if (i === 0 && isFirstDefault)
                        target[prop] = source.slice();
                    else
                        target[prop] = source.slice();
                } else if (ej.isPlainObject(source)) {
                    target[prop] = current || {};
                    if (isFirstDefault)
                        copyObject(isFirstDefault, target[prop], source);
                    else
                        copyObject(target[prop], source);
                } else
                    target[prop] = source;
            }
        }
        return target;
    };
    var pInstance = function () {
        return this;
    }

    ej.template = {};

    ej.template.render = ej.template["text/x-jsrender"] = function (self, selector, data, index, prop) {
        if (selector.slice(0, 1) !== "#")
            selector = ["<div>", selector, "</div>"].join("");
        var property = { prop: prop, index: index };
        return $(selector).render(data, property);
    }

    ej.isPlainObject = function (obj) {
        if (!obj) return false;

        if (typeof obj !== "object" || obj.nodeType || jQuery.isWindow(obj)) {
            return false;
        }

        try {
            if (obj.constructor &&
                !obj.constructor.prototype.hasOwnProperty("isPrototypeOf")) {
                return false;
            }
        } catch (e) {
            return false;
        }

        var key, ownLast = ej.support.isOwnLast;
        for (key in obj) {
            if (ownLast) break;
        }

        return key === undefined || obj.hasOwnProperty(key);
    };
    var getValueFn = false;
    ej.util.valueFunction = function (prop) {
        return function (value, getObservable) {
            var val = ej.getObject(prop, this.model);

            if (getValueFn === false)
                getValueFn = ej.getObject("observables.getValue", ej.widget);

            if (value === undefined) {
                if (!ej.isNullOrUndefined(getValueFn)) {
                    return getValueFn(val, getObservable);
                }
                return typeof val === "function" ? val() : val;
            }

            if (typeof val === "function") {
                this["ob.values"][prop] = value;
                val.call(this, value);
            }
            else
                ej.createObject(prop, value, this.model);
        }
    };
    ej.util.getVal = function (val) {
        if (typeof val === "function")
            return val();
        return val;
    };
    ej.support = {
        isOwnLast: function () {
            var fn = function () { this.a = 1; };
            fn.prototype.b = 1;

            for (var p in new fn()) {
                return p === "b";
            }
        }(),
        outerHTML: function () {
            return document.createElement("div").outerHTML !== undefined;
        }()
    };

    var throwError = ej.throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };



    ej.extensions = {};
    ej.extensions.modelGUID = "{0B1051BA-1CCB-42C2-A3B5-635389B92A50}";
})(window.jQuery, window.Syncfusion);
(function () {
    $.fn.addEleAttrs = function (json) {
        var $this = $(this);
        $.each(json, function (i, attr) {
            if (attr && attr.specified) {
                $this.attr(attr.name, attr.value);
            }
        });

    };
    $.fn.removeEleAttrs = function (regex) {
        return this.each(function () {
            var $this = $(this),
                names = [],
                attrs = $(this.attributes).clone();
            $.each(attrs, function (i, attr) {
                if (attr && attr.specified && regex.test(attr.name)) {
                    $this.removeAttr(attr.name);
                }
            });
        });
    };
    $.fn.attrNotStartsWith = function (regex) {
        var proxy = this;
        var attributes = [], attrs;
        this.each(function () {
            attrs = $(this.attributes).clone();
        });
        for (i = 0; i < attrs.length; i++) {
            if (attrs[i] && attrs[i].specified && regex.test(attrs[i].name)) {
                continue
            }
            else
                attributes.push(attrs[i])
        }
        return attributes;

    }
    $.fn.removeEleEmptyAttrs = function () {
        return this.each(function () {
            var $this = $(this),
                names = [],
                attrs = $(this.attributes).clone();
            $.each(attrs, function (i, attr) {
                if (attr && attr.specified && attr.value === "") {
                    $this.removeAttr(attr.name);
                }
            });
        });
    };
    $.extend($.support, {
        has3d: ej.addPrefix('perspective') in ej.styles,
        hasTouch: 'ontouchstart' in window,
        hasPointer: navigator.msPointerEnabled,
        hasTransform: ej.userAgent() !== false,
        pushstate: "pushState" in history &&
        "replaceState" in history,
        hasTransition: ej.addPrefix('transition') in ej.styles
    });
    //Ensuring elements having attribute starts with 'ejm-' 
    $.extend($.expr[':'], {
        attrNotStartsWith: function (element, index, match) {
            var i, attrs = element.attributes;
            for (i = 0; i < attrs.length; i++) {
                if (attrs[i].nodeName.indexOf(match[3]) === 0) {
                    return false;
                }
            }
            return true;
        }
    });
})();;
window.ej = window.Syncfusion = window.Syncfusion || {};

(function ($, ej, doc, undefined) {
    'use strict';
	
    ej.DataManager = function (dataSource, query, adaptor) {
          if (!(this instanceof ej.DataManager))
            return new ej.DataManager(dataSource, query, adaptor);

        if (!dataSource)
            dataSource = [];
        adaptor = adaptor || dataSource.adaptor;

        if (typeof (adaptor) === "string") 
            adaptor = new ej[adaptor]();
        var data = [], self = this;

        if (dataSource instanceof Array) {
            // JSON array
            data = {
                json: dataSource,
                offline: true
            };

        } else if (typeof dataSource === "object") {
            if ($.isPlainObject(dataSource)) {
                if (!dataSource.json)
                    dataSource.json = [];
                if (dataSource.table)
                    dataSource.json = this._getJsonFromElement(dataSource.table, dataSource.headerOption);
                data = {
                    url: dataSource.url,
                    insertUrl: dataSource.insertUrl,
                    removeUrl: dataSource.removeUrl,
                    updateUrl: dataSource.updateUrl,
                    crudUrl: dataSource.crudUrl,
                    batchUrl: dataSource.batchUrl,
                    json: dataSource.json,
                    headers: dataSource.headers,
                    accept: dataSource.accept,
                    data: dataSource.data,
                    timeTillExpiration: dataSource.timeTillExpiration,
                    cachingPageSize: dataSource.cachingPageSize,
                    enableCaching: dataSource.enableCaching,
                    requestType: dataSource.requestType,
                    key: dataSource.key,
                    crossDomain: dataSource.crossDomain,
                    jsonp: dataSource.jsonp,
                    dataType: dataSource.dataType,
                    offline: dataSource.offline !== undefined ? dataSource.offline : dataSource.adaptor == "remoteSaveAdaptor" ? false : dataSource.url ? false : true,
                    requiresFormat: dataSource.requiresFormat
                };
            } else if (dataSource.jquery || isHtmlElement(dataSource)) {
                data = {
                    json: this._getJsonFromElement(dataSource),
                    offline: true,
                    table: dataSource
                };
            }
        } else if (typeof dataSource === "string") {
            data = {
                url: dataSource,
                offline: false,
                dataType: "json"
            };
        }

        if (data.requiresFormat === undefined && !ej.support.cors)
            data.requiresFormat = isNull(data.crossDomain) ? true : data.crossDomain;
        if (data.dataType === undefined)
            data.dataType = "json";

        this.dataSource = data;
        this.defaultQuery = query;

        if (data.url && data.offline && !data.json.length) {
            this.isDataAvailable = false;
            this.adaptor = adaptor || new ej.ODataAdaptor();
            this.dataSource.offline = false;
            this.ready = this.executeQuery(query || ej.Query()).done(function (e) {
                self.dataSource.offline = true;
                self.isDataAvailable = true;
                data.json = e.result;
                self.adaptor = new ej.JsonAdaptor();
            });
        }
        else
            this.adaptor = data.offline ? new ej.JsonAdaptor() : new ej.ODataAdaptor();
        if (!data.jsonp && this.adaptor instanceof ej.ODataAdaptor)
            data.jsonp = "callback";
        this.adaptor = adaptor || this.adaptor;
        if (data.enableCaching)
            this.adaptor = new ej.CacheAdaptor(this.adaptor, data.timeTillExpiration, data.cachingPageSize);
        return this;
    };

    ej.DataManager.prototype = {
        setDefaultQuery: function (query) {
            this.defaultQuery = query;
        },
	
        executeQuery: function (query, done, fail, always) {
            if (typeof query === "function") {
                always = fail;
                fail = done;
                done = query;
                query = null;
            }

            if (!query)
                query = this.defaultQuery;

            if (!(query instanceof ej.Query))
                throwError("DataManager - executeQuery() : A query is required to execute");

            var deffered = $.Deferred();

            deffered.then(done, fail, always);
            var args = { query: query };

            if (!this.dataSource.offline && this.dataSource.url != undefined) {
				 var result = this.adaptor.processQuery(this, query);
                if (!ej.isNullOrUndefined(result.url))
                    this._makeRequest(result, deffered, args, query);
                else {
                    nextTick(function () {
                        args = this._getDeferedArgs(query, result, args);
                        deffered.resolveWith(this, [args]);;
                    }, this);
                }
            } else {
                nextTick(function () {
                    var res = this.executeLocal(query);
                    args = this._getDeferedArgs(query, res, args);
                    deffered.resolveWith(this, [args]);
                }, this);
            }

            return deffered.promise();
        },
        _getDeferedArgs: function (query, result, args) {
            if (query._requiresCount) {
                args.result = result.result;
                args.count = result.count;
            } else
                args.result = result;
            args.getTableModel = getTableModel(query._fromTable, args.result, this);
            args.getKnockoutModel = getKnockoutModel(args.result);
            return args;
        },
	
        executeLocal: function (query) {
            if (!this.defaultQuery && !(query instanceof ej.Query))
                throwError("DataManager - executeLocal() : A query is required to execute");

            if (!this.dataSource.json)
                throwError("DataManager - executeLocal() : Json data is required to execute");

            query = query || this.defaultQuery;

            var result = this.adaptor.processQuery(this, query);

            if (query._subQuery) {
                var from = query._subQuery._fromTable, lookup = query._subQuery._lookup,
                    res = query._requiresCount ? result.result : result;

                if (lookup && lookup instanceof Array) {
                    buildHierarchy(query._subQuery._fKey, from, res, lookup);
                }

                for (var j = 0; j < res.length; j++) {
                    if (res[j][from] instanceof Array) {
                        res[j] = $.extend({}, res[j]);
                        res[j][from] = this.adaptor.processResponse(query._subQuery.using(ej.DataManager(res[j][from].slice(0))).executeLocal(), this, query);
                    }
                }
            }

            return this.adaptor.processResponse(result, this, query);
        },

        _makeRequest: function (url, deffered, args, query) {
            var isSelector = !!query._subQuerySelector;

            var fnFail = $proxy(function (e) {
                args.error = e;
                deffered.rejectWith(this, [args]);
            }, this);

            var process = $proxy(function (data, count, xhr, request, actual, aggregates) {
                if (isSelector) return;

                args.xhr = xhr;
                args.count = parseInt(count, 10);
                args.result = data;
                args.request = request;
                args.aggregates = aggregates;
                args.getTableModel = getTableModel(query._fromTable, data, this);
                args.getKnockoutModel = getKnockoutModel(data);
                args.actual = actual;
                deffered.resolveWith(this, [args]);

            }, this);

            var fnQueryChild = $proxy(function (data, selector) {
                var subDeffer = $.Deferred(),
                    childArgs = { parent: args };

                query._subQuery._isChild = true;

                var subUrl = this.adaptor.processQuery(this, query._subQuery, data ? this.adaptor.processResponse(data) : selector);

                var childReq = this._makeRequest(subUrl, subDeffer, childArgs, query._subQuery);

                if(!isSelector)
                    subDeffer.then(function (subData) {
                        if (data) {
                            buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, data, subData);
                            process(data);
                        }
                    }, fnFail);

                return childReq;
            }, this);

            var fnSuccess = proxy(function (data, status, xhr, request) {
                if (xhr.getResponseHeader("Content-Type").indexOf("xml") == -1 && ej.dateParse)
                    data = ej.parseJSON(data);
                var result = this.adaptor.processResponse(data, this, query, xhr, request), count = 0, aggregates = null;
                if (query._requiresCount) {
                    count = result.count;
                    aggregates = result.aggregates;
                    result = result.result;
                }

                if (!query._subQuery) {
                    process(result, count, xhr, request, data, aggregates);
                    return;
                }

                if (!isSelector)
                    fnQueryChild(result);

            }, this);

            var req = $.extend({
                type: "GET",
                dataType: this.dataSource.dataType,
                crossDomain: this.dataSource.crossDomain,
                jsonp: this.dataSource.jsonp,
                cache: true,
                beforeSend: $proxy(this._beforeSend, this),
                processData: false,
                success: fnSuccess,
                error: fnFail
            }, url);

            req = $.ajax(req);

            if (isSelector) {
                var res = query._subQuerySelector.call(this, { query: query._subQuery, parent: query });

                if (res && res.length) {
                    req = $.when(req, fnQueryChild(null, res));

                    req.then(proxy(function (pData, cData, requests) {
                        var pResult = this.adaptor.processResponse(pData[0], this, query, pData[2], requests[0]), count = 0;
                        if (query._requiresCount) {
                            count = pResult.count;
                            pResult = pResult.result;
                        }
                        var cResult = this.adaptor.processResponse(cData[0], this, query._subQuery, cData[2], requests[1]), count = 0;
                        if (query._subQuery._requiresCount) {
                            count = cResult.count;
                            cResult = cResult.result;
                        }

                        buildHierarchy(query._subQuery._fKey, query._subQuery._fromTable, pResult, cResult);
                        isSelector = false;
                        process(pResult, count, pData[2]);

                    }, this), fnFail);
                } else {
                    isSelector = false;
                }
            }

            return req;
        },

        _beforeSend: function (request, settings) {
            this.adaptor.beforeSend(this, request, settings);

            var headers = this.dataSource.headers, props;
            for (var i = 0; headers && i < headers.length; i++) {
                props = [];
                for (var prop in headers[i]) {
                    props.push(prop);
                    request.setRequestHeader(prop, headers[i][prop]);
                }
            }
        },
	
        saveChanges: function (changes, key, tableName) {
            var args = {
                url: tableName,
                key: key || this.dataSource.key
            };

            var req = this.adaptor.batchRequest(this, changes, args);

            if (this.dataSource.offline) {
                return req;
            }

            var deff = $.Deferred();
            $.ajax($.extend({
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (data, status, xhr, request) {
                    deff.resolveWith(this, [this.adaptor.processResponse(data, this, null, xhr, request, changes)]);
                }, this),
                error: function (e) {
                    deff.rejectWith(this, [{ error: e }]);
                }
            }, req));

            return deff.promise();
        },
	
        insert: function (data, tableName) {
            var res = this.adaptor.insert(this, data, tableName);
            if (this.dataSource.offline) {
                return res;
            }

            var deffer = $.Deferred();

            $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                processData: false,
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (record, status, xhr, request) {
                    try {
                        if (ej.isNullOrUndefined(record))
                            record = [];
                        else
                            p.parseJson(record);
                    }
                    catch (e) {
                        record = [];
                    }
                    record = this.adaptor.processResponse(p.parseJson(record), this, null, xhr, request);
                    deffer.resolveWith(this, [{ record: record, dataManager: this }]);
                }, this),
                error: function (e) {
                    deffer.rejectWith(this, [{ error: e, dataManager: this }]);
                }
            }, res));

            return deffer.promise();
        },
	
        remove: function (keyField, value, tableName) {
            if (typeof value === "object")
                value = value[keyField];

            var res = this.adaptor.remove(this, keyField, value, tableName);

            if (this.dataSource.offline)
                return res;

            return $.ajax($.extend({
                type: "POST",
                contentType: "application/json; charset=utf-8",
                beforeSend: $proxy(this._beforeSend, this)
            }, res));
        },
	
        update: function (keyField, value, tableName) {
            var res = this.adaptor.update(this, keyField, value, tableName);

            if (this.dataSource.offline) {
                return res;
            }

            var deffer = $.Deferred();

           $.ajax($.extend({
                contentType: "application/json; charset=utf-8",
                beforeSend: $proxy(this._beforeSend, this),
                success: proxy(function (record, status, xhr, request) {
                    try {
                        if (ej.isNullOrUndefined(record))
                            record = [];
                        else
                            p.parseJson(record);
                    }
                    catch (e) {
                        record = [];
                    }
                    record = p.parseJson(record);
                    deffer.resolveWith(this, [{ record: record, dataManager: this }]);
                }, this),
                error: function (e) {
                    deffer.rejectWith(this, [{ error: e, dataManager: this }]);
                }
           }, res));

           return deffer.promise();
        },

        _getJsonFromElement: function (ds) {
            if (typeof (ds) == "string")
                ds = $($(ds).html());

            ds = ds.jquery ? ds[0] : ds;

            var tagName = ds.tagName.toLowerCase();

            if (tagName !== "table")
                throwError("ej.DataManager : Unsupported htmlElement : " + tagName);

            return ej.parseTable(ds);
        }
    };

    var buildHierarchy = function (fKey, from, source, lookup) {
        var i, grp = {}, t;
        if (lookup.result) lookup = lookup.result;

        if (lookup.GROUPGUID)
            throwError("ej.DataManager: Do not have support Grouping in hierarchy");

        for (i = 0; i < lookup.length; i++) {
            t = grp[lookup[i][fKey]] || (grp[lookup[i][fKey]] = []);

            t.push(lookup[i]);
        }

        for (i = 0; i < source.length; i++) {
            source[i][from] = grp[source[i][fKey]];
        }
    };

    var oData = {
        accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
        multipartAccept: "multipart/mixed",
        batch: "$batch",
        changeSet: "--changeset_",
        batchPre: "batch_",
        contentId: "Content-Id: ",
        batchContent: "Content-Type: multipart/mixed; boundary=",
        changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
        batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
    };
    var p = {
        parseJson: function (jsonText) {
            var type = typeof jsonText;
            if (type === "string") {
                jsonText = JSON.parse(jsonText, p.jsonReviver);
            } else if (jsonText instanceof Array) {
                p.iterateAndReviveArray(jsonText);
            } else if (type === "object")
                p.iterateAndReviveJson(jsonText);
            return jsonText;
        },
        iterateAndReviveArray: function (array) {
            for (var i = 0; i < array.length; i++) {
                if (typeof array[i] === "object")
                    p.iterateAndReviveJson(array[i]);
                else if (typeof array[i] === "string" && !/^[\s]*\[|^[\s]*\{|\"/g.test(array[i]))
                    array[i] = p.jsonReviver("",array[i]);
                else
                    array[i] = p.parseJson(array[i]);
            }
        },
        iterateAndReviveJson: function (json) {
            var value;

            for (var prop in json) {
                if (prop.startsWith("__"))
                    continue;

                value = json[prop];
                if (typeof value === "object") {
                    if (value instanceof Array)
                        p.iterateAndReviveArray(value);
                    else
                        p.iterateAndReviveJson(value);
                } else
                    json[prop] = p.jsonReviver(prop, value);
            }
        },
        jsonReviver: function (field, value) {
            var s = value;
            if (typeof value === "string") {
                var ms = /^\/Date\(([+-]?[0-9]+)([+-][0-9]{4})?\)\/$/.exec(value);
                if (ms)
                    return new Date(parseInt(ms[1]));
                else if (/^(\d{4}\-\d\d\-\d\d([tT][\d:\.]*){1})([zZ]|([+\-])(\d\d):?(\d\d))?$/.test(value)) {
                    value = new Date(value);
                    if (isNaN(value)) {
                        var a = s.split(/[^0-9]/);
                        value = new Date(a[0], a[1] - 1, a[2], a[3], a[4], a[5]);
                    }
                }
            }

            return value;
        },
        isJson: function (jsonData) {
            if(typeof jsonData[0]== "string")
                return jsonData;
            return ej.parseJSON(jsonData);
        },
        isGuid: function (value) {
            var regex = /[A-Fa-f0-9]{8}(?:-[A-Fa-f0-9]{4}){3}-[A-Fa-f0-9]{12}/i;
            var match = regex.exec(value);
            return match != null;
        },
        replacer: function (key, value) {

            if (ej.isPlainObject(value))
                return p.jsonReplacer(key, value);

            if (value instanceof Array)
                return p.arrayReplacer(key, value);

            if (value instanceof Date)
                return p.jsonReplacer("", { val: value }).val;

            return value;
        },
        jsonReplacer: function (key, val) {
            var value;
            for (var prop in val) {
                value = val[prop];

                if (!(value instanceof Date))
                    continue;

                var d = value, unixstamp = +d - (d.getTimezoneOffset() * 60000);
                val[prop] = new Date(unixstamp - (ej.serverTimezoneOffset * 3600000)).toJSON();
            }

            return val;
        },
        arrayReplacer: function (key, val) {

            for (var i = 0; i < val.length; i++) {            
                if (ej.isPlainObject(val[i]))
                    val[i] = p.jsonReplacer("", val[i]);
                else if (val[i] instanceof Date)
                    val[i] = p.jsonReplacer("", { date: val[i] }).date;
            }

            return val;
        }
    };

    ej.isJSON = p.isJson;
    ej.parseJSON = p.parseJson;
    ej.dateParse = true;
    ej.isGUID = p.isGuid;
    ej.Query = function (from) {
        if (!(this instanceof ej.Query))
            return new ej.Query(from);

        this.queries = [];
        this._key = "";
        this._fKey = "";

        if (typeof from === "string")
            this._fromTable = from || "";
        else if (from && from instanceof Array)
            this._lookup = from;

        this._expands = [];
        this._sortedColumns = [];
        this._groupedColumns = [];
        this._subQuery = null;
        this._isChild = false;
        this._params = [];
        return this;
    };

    ej.Query.prototype = {
        key: function (field) {
            if (typeof field === "string")
                this._key = field;

            return this;
        },
	
        using: function (dataManager) {
            if (dataManager instanceof ej.DataManager) {
                this.dataManagar = dataManager;
                return this;
            }

            return throwError("Query - using() : 'using' function should be called with parameter of instance ej.DataManager");
        },
	
        execute: function (dataManager, done, fail, always) {
            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeQuery(this, done, fail, always);

            return throwError("Query - execute() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },
	
        executeLocal: function (dataManager) {
            // this does not support for URL binding
            

            dataManager = dataManager || this.dataManagar;

            if (dataManager && dataManager instanceof ej.DataManager)
                return dataManager.executeLocal(this);

            return throwError("Query - executeLocal() : dataManager needs to be is set using 'using' function or should be passed as argument");
        },
	
        clone: function () {
            var cl = new ej.Query();
            cl.queries = this.queries.slice(0);
            cl._key = this._key;
            cl._isChild = this._isChild;
            cl.dataManagar = this.dataManager;
            cl._fromTable = this._fromTable;
            cl._params = this._params.slice(0);
            cl._expands = this._expands.slice(0);
            cl._sortedColumns = this._sortedColumns.slice(0);
            cl._groupedColumns = this._groupedColumns.slice(0);
            cl._subQuerySelector = this._subQuerySelector;
            cl._subQuery = this._subQuery;
            cl._fKey = this._fKey;
            cl._requiresCount = this._requiresCount;
            return cl;
        },
	
        from: function (tableName) {
            if (typeof tableName === "string")
                this._fromTable = tableName;

            return this;
        },
	
        addParams: function (key, value) {
            if (typeof value !== "function" && !ej.isPlainObject(value))
                this._params.push({ key: key, value: value });
            else if (typeof value === "function")
                this._params.push({ key: key, fn: value });

            return this;
        },
	
        expand: function (tables) {
            if (typeof tables === "string")
                this._expands = [].slice.call(arguments, 0);
            else
                this._expands = tables.slice(0);

            return this;
        },
	
        where: function (fieldName, operator, value, ignoreCase) {
            operator = (operator || ej.FilterOperators.equal).toLowerCase();
            var predicate = null;

            if (typeof fieldName === "string")
                predicate = new ej.Predicate(fieldName, operator, value, ignoreCase);
            else if (fieldName instanceof ej.Predicate)
                predicate = fieldName;
            else
                throwError("Query - where : Invalid arguments");

            this.queries.push({
                fn: "onWhere",
                e: predicate
            });
            return this;
        },
	
        search: function (searchKey, fieldNames, operator, ignoreCase) {
            if (!fieldNames || typeof fieldNames === "boolean") {
                fieldNames = [];
                ignoreCase = fieldNames;
            } else if (typeof fieldNames === "string")
                fieldNames = [fieldNames];

            if (typeof operator === "boolean") {
                ignoreCase = operator;
                operator = null;
            }
            operator = operator || ej.FilterOperators.contains;
            if (operator.length < 3)
                operator = ej.data.operatorSymbols[operator];

            var comparer = ej.data.fnOperators[operator] || ej.data.fnOperators.processSymbols(operator);

            this.queries.push({
                fn: "onSearch",
                e: {
                    fieldNames: fieldNames,
                    operator: operator,
                    searchKey: searchKey,
                    ignoreCase: ignoreCase,
                    comparer: comparer
                }
            });
            return this;
        },
		
        sortBy: function (fieldName, comparer, isFromGroup) {
            var order = ej.sortOrder.Ascending, sorts, t;

            if (typeof fieldName === "string" && fieldName.toLowerCase().endsWith(" desc")) {
                fieldName = fieldName.replace(/ desc$/i, '');
                comparer = ej.sortOrder.Descending;
            }
            if (fieldName instanceof Array) {
                for(var i=0;i<fieldName.length;i++)
                   this.sortBy(fieldName[i],comparer,isFromGroup);
                return this;
            }
            if (typeof comparer === "boolean")
                comparer = !comparer ? ej.sortOrder.Ascending : ej.sortOrder.Descending;
            else if (typeof comparer === "function")
                order = "custom";

            if (!comparer || typeof comparer === "string") {
                order = comparer ? comparer.toLowerCase() : ej.sortOrder.Ascending;
                comparer = ej.pvt.fnSort(comparer);
            }
            if (isFromGroup) {
                sorts = filterQueries(this.queries, "onSortBy");

                for (var i = 0; i < sorts.length; i++) {
                    t = sorts[i].e.fieldName;
                    if (typeof t === "string") {
                        if (t === fieldName) return this;
                    } else if (t instanceof Array) {
                        for (var j = 0; j < t.length; j++)
                            if (t[j] === fieldName || fieldName.toLowerCase() === t[j] + " desc")
                                return this;
                    }
                }
            }

            this.queries.push({
                fn: "onSortBy",
                e: {
                    fieldName: fieldName,
                    comparer: comparer,
                    direction: order
                }
            });

            return this;
        },
		
        sortByDesc: function (fieldName) {
            return this.sortBy(fieldName, ej.sortOrder.Descending);
        },
		
        group: function (fieldName) {
            this.sortBy(fieldName, null, true);

            this.queries.push({
                fn: "onGroup",
                e: {
                    fieldName: fieldName
                }
            });
            return this;
        },
	
        page: function (pageIndex, pageSize) {
            this.queries.push({
                fn: "onPage",
                e: {
                    pageIndex: pageIndex,
                    pageSize: pageSize
                }
            });
            return this;
        },
	
        range: function (start, end) {
            if (typeof start !== "number" || typeof end !== "number")
                throwError("Query() - range : Arguments type should be a number");

            this.queries.push({
                fn: "onRange",
                e: {
                    start: start,
                    end: end
                }
            });
            return this;
        },
	

        take: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Take : Argument type should be a number");

            this.queries.push({
                fn: "onTake",
                e: {
                    nos: nos
                }
            });
            return this;
        },
	
        skip: function (nos) {
            if (typeof nos !== "number")
                throwError("Query() - Skip : Argument type should be a number");

            this.queries.push({
                fn: "onSkip",
                e: { nos: nos }
            });
            return this;
        },
	
        select: function (fieldNames) {
            if (typeof fieldNames === "string")
                fieldNames = [].slice.call(arguments, 0);

            if (!(fieldNames instanceof Array)) {
                throwError("Query() - Select : Argument type should be String or Array");
            }

            this.queries.push({
                fn: "onSelect",
                e: { fieldNames: fieldNames }
            });
            return this;
        },
	
        hierarchy: function (query, selectorFn) {
            if (!query || !(query instanceof ej.Query))
                throwError("Query() - hierarchy : query must be instance of ej.Query");

            if (typeof selectorFn === "function")
                this._subQuerySelector = selectorFn;

            this._subQuery = query;
            return this;
        },
	
        foreignKey: function (key) {
            if (typeof key === "string")
                this._fKey = key;

            return this;
        },
	
        requiresCount: function () {
            this._requiresCount = true;

            return this;
        },
        //type - sum, avg, min, max
        aggregate: function (type, field) {
            this.queries.push({
                fn: "onAggregates",
                e: { field: field, type: type }
            });
        }
    };

    ej.Adaptor = function (ds) {
        this.dataSource = ds;
        this.pvt = {};
		this.init.apply(this, [].slice.call(arguments, 1));
    };

    ej.Adaptor.prototype = {
        options: {
            from: "table",
            requestType: "json",
            sortBy: "sorted",
            select: "select",
            skip: "skip",
            group: "group",
            take: "take",
            search: "search",
            count: "requiresCounts",
            where: "where",
            aggregates: "aggregates"
        },
        init: function () {
        },
        extend: function (overrides) {
            var fn = function (ds) {
                this.dataSource = ds;

                if (this.options)
                    this.options = $.extend({}, this.options);
				this.init.apply(this, [].slice.call(arguments, 0));

                this.pvt = {};
            };
            fn.prototype = new this.type();
            fn.prototype.type = fn;

            var base = fn.prototype.base = {};
            for (var p in overrides) {
                if (fn.prototype[p])
                    base[p] = fn.prototype[p];
            }
            $.extend(true, fn.prototype, overrides);
            return fn;
        },
        processQuery: function (dm, query) {
            // this needs to be overridden
        },
        processResponse: function (data, ds, query, xhr) {
            return data;
        },
        convertToQueryString: function (req, query, dm) {
            return $.param(req);
        },
        type: ej.Adaptor
    };

    ej.UrlAdaptor = new ej.Adaptor().extend({
        processQuery: function (dm, query, hierarchyFilters) {
            var sorted = filterQueries(query.queries, "onSortBy"),
                grouped = filterQueries(query.queries, "onGroup"),
                filters = filterQueries(query.queries, "onWhere"),
                searchs = filterQueries(query.queries, "onSearch"),
                aggregates = filterQueries(query.queries, "onAggregates"),
                singles = filterQueryLists(query.queries, ["onSelect", "onPage", "onSkip", "onTake", "onRange"]),
                params = query._params,
                url = dm.dataSource.url, tmp, skip, take = null,
                op = this.options;

            var r = {
                sorted: [],
                grouped: [],
                filters: [],
                searches: [],
                aggregates: []
            };

            // calc Paging & Range
            if (singles["onPage"]) {
                tmp = singles["onPage"];
                skip = getValue(tmp.pageIndex, query);
                take = getValue(tmp.pageSize, query);
				skip = (skip - 1) * take;
            } else if (singles["onRange"]) {
                tmp = singles["onRange"];
                skip = tmp.start;
                take = tmp.end - tmp.start;
            }

            // Sorting
            for (var i = 0; i < sorted.length; i++) {
                tmp = getValue(sorted[i].e.fieldName, query);

                r.sorted.push(callAdaptorFunc(this, "onEachSort", { name: tmp, direction: sorted[i].e.direction }, query));
            }

            // hierarchy
            if (hierarchyFilters) {
                tmp = this.getFiltersFrom(hierarchyFilters, query);
                if (tmp)
                    r.filters.push(callAdaptorFunc(this, "onEachWhere", tmp.toJSON(), query));
            }

            // Filters
            for (var i = 0; i < filters.length; i++) {
                r.filters.push(callAdaptorFunc(this, "onEachWhere", filters[i].e.toJSON(), query));

                for (var prop in r.filters[i]) {
                    if (isNull(r[prop]))
                        delete r[prop];
                }
            }

            // Searches
            for (var i = 0; i < searchs.length; i++) {
                tmp = searchs[i].e;
                r.searches.push(callAdaptorFunc(this, "onEachSearch", {
                    fields: tmp.fieldNames,
                    operator: tmp.operator,
                    key: tmp.searchKey,
                    ignoreCase: tmp.ignoreCase
                }, query));
            }

            // Grouping
            for (var i = 0; i < grouped.length; i++) {
                r.grouped.push(getValue(grouped[i].e.fieldName, query));
            }

            // aggregates
            for (var i = 0; i < aggregates.length; i++) {
                tmp = aggregates[i].e; 
                r.aggregates.push({ type: tmp.type, field: getValue(tmp.field, query) });
            }

            var req = {};
            req[op.from] = query._fromTable;
            if (op.expand) req[op.expand] = query._expands;
            req[op.select] = singles["onSelect"] ? callAdaptorFunc(this, "onSelect", getValue(singles["onSelect"].fieldNames, query), query) : "";
            req[op.count] = query._requiresCount ? callAdaptorFunc(this, "onCount", query._requiresCount, query) : "";
            req[op.search] = r.searches.length ? callAdaptorFunc(this, "onSearch", r.searches, query) : "";
            req[op.skip] = singles["onSkip"] ? callAdaptorFunc(this, "onSkip", getValue(singles["onSkip"].nos, query), query) : "";
            req[op.take] = singles["onTake"] ? callAdaptorFunc(this, "onTake", getValue(singles["onTake"].nos, query), query) : "";
            req[op.where] = r.filters.length || r.searches.length ? callAdaptorFunc(this, "onWhere", r.filters, query) : "";
            req[op.sortBy] = r.sorted.length ? callAdaptorFunc(this, "onSortBy", r.sorted, query) : "";
            req[op.group] = r.grouped.length ? callAdaptorFunc(this, "onGroup", r.grouped, query) : "";
            req[op.aggregates] = r.aggregates.length ? callAdaptorFunc(this, "onAggregates", r.aggregates, query) : "";
			req["param"] = [];
			
            // Params
            for (var i = 0; tmp = params[i]; i++) {
                if (req[tmp.key]) throwError("ej.Query: Custom Param is conflicting other request arguments");

                if(dm.adaptor instanceof ej.WebMethodAdaptor) {                  
                    var webkey = tmp.key, webvalue = tmp.value,  obj = {};                    
                    if (tmp.fn) 
                        webvalue = tmp.fn.call(query, tmp.key, query, dm);  
                    obj[webkey] = webvalue;
                    req["param"].push(obj);
                }
                else {
                    req[tmp.key] = tmp.value;
                    if (tmp.fn)
                        req[tmp.key] = tmp.fn.call(query, tmp.key, query, dm);
                }                
            }

            // cleanup
            for (var prop in req) {
                if (isNull(req[prop]) || req[prop] === "" || req[prop].length === 0)
                    delete req[prop];
            }

            if (!(op.skip in req && op.take in req) && take !== null) {
                req[op.skip] = callAdaptorFunc(this, "onSkip", skip, query);
                req[op.take] = callAdaptorFunc(this, "onTake", take, query);
            }
            var p = this.pvt;
            this.pvt = {};

            if (this.options.requestType === "json") {
                return {
                    data: JSON.stringify(req),
                    url: url,
                    ejPvtData: p,
                    type: "POST",
                    contentType: "application/json; charset=utf-8"
                }
            }
            tmp = this.convertToQueryString(req, query, dm);
            tmp =  (dm.dataSource.url.indexOf("?")!== -1 ? "&" : "/") + tmp;
            return {
                type: "GET",
                url: tmp.length ? url.replace(/\/*$/, tmp) : url,
                ejPvtData: p
            };
        },
        convertToQueryString: function (req, query, dm) {
            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1)
                return $.param(req);
            return "?" + $.param(req);
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request.ejPvtData || {};
			var groupDs= data.groupDs;
			if (xhr && xhr.getResponseHeader("Content-Type") && xhr.getResponseHeader("Content-Type").indexOf("xml") != -1 && data.nodeType == 9)
                return query._requiresCount ? { result: [], count: 0 } : [];
            var d = JSON.parse(request.data);
            if (d && d.action === "batch" && data.added) {
                changes.added = data.added;
                return changes;
            }
            if (data.d)
                data = data.d;

            if (pvt && pvt.aggregates && pvt.aggregates.length) {
                var agg = pvt.aggregates, args = {}, fn, res = {};
                if (data["count"]) args.count = data.count;
                if (data["result"]) args.result = data.result;
                if (data["aggregate"]) data = data.aggregate;
                for (var i = 0; i < agg.length; i++) {
                    fn = ej.aggregates[agg[i].type];
                    if (fn)
                        res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
                }
                args["aggregates"] = res;
                data = args;
            }

            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups, args = {};
                if (data["count"] != undefined) args.count = data.count;
                if (data["aggregates"]) args.aggregates = data.aggregates;
                if (data["result"]) data = data.result;
                for (var i = 0; i < groups.length; i++){
                    var level=null;
                    if(!ej.isNullOrUndefined(groupDs))
                        groupDs = ej.group(groupDs, groups[i]);
                    data = ej.group(data, groups[i], pvt.aggregates,level,groupDs);
                }
                if (args.count != undefined)
                    args.result = data;
                else
                    args = data;
                return args;
            }
            return data;
        },
        onGroup: function (e) {
            this.pvt.groups = e;
        },
        onAggregates: function (e) {
            this.pvt.aggregates = e;
        },
        batchRequest: function (dm, changes, e) {
            return {
                type: "POST",
                url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                data: JSON.stringify({
                    changed: changes.changed,
                    added: changes.added,
                    deleted: changes.deleted,
                    action: "batch",
                    table: e.url,
                    key: e.key
                })
            };
        },
        beforeSend: function (dm, request) {
        },
        insert: function (dm, data, tableName) {
            return {
                url: dm.dataSource.insertUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    value: data,
                    table: tableName,
                    action: "insert"
                })
            };
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.removeUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    key: value,
                    keyColumn: keyField,
                    table: tableName,
                    action: "remove"
                })
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.updateUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                data: JSON.stringify({
                    value: value,
                    action: "update",
                    keyColumn: keyField,
                    key: value[keyField],
                    table: tableName
                })
            };
        },
        getFiltersFrom: function (data, query) {
            if (!(data instanceof Array) || !data.length)
                throwError("ej.SubQuery: Array of key values required");
            var key = query._fKey, value, prop = key, predicats = [],
                isValues = typeof data[0] !== "object";

            if (typeof data[0] !== "object") prop = null;

            for (var i = 0; i < data.length; i++) {
                value = !isValues ? ej.pvt.getObject(prop, data[i]) : data[i];
                predicats.push(new ej.Predicate(key, "==", value));
            }

            return ej.Predicate.or(predicats);
        }
    });
    ej.WebMethodAdaptor = new ej.UrlAdaptor().extend({
        processQuery: function (dm, query, hierarchyFilters) {
            var obj = ej.UrlAdaptor.prototype.processQuery(dm, query, hierarchyFilters); 
            var data = ej.parseJSON(obj.data), result = {};
            // Param
            if (data.param) {
                for(var i=0; i< data.param.length; i++){
                    var param = data.param[i], key = Object.keys(param)[0];
                    result[key] = param[key];                
                }
            }            
            result["value"] = data;
            return {
                data: JSON.stringify(result),
                url: obj.url,
                ejPvtData: obj.ejPvtData,
                type: "POST",
                contentType: "application/json; charset=utf-8"
            }
        }
    });
    ej.CacheAdaptor = new ej.UrlAdaptor().extend({
        init: function (adaptor, timeStamp, pageSize) {
            if (!ej.isNullOrUndefined(adaptor)) {
                this.cacheAdaptor = adaptor;
            }
            this.pageSize = pageSize;
            this.guidId = ej.getGuid("cacheAdaptor");
            var obj = { keys: [], results: [] };
            if (window.localStorage)
                window.localStorage.setItem(this.guidId, JSON.stringify(obj));
            var guid = this.guidId;
            if (!ej.isNullOrUndefined(timeStamp)) {
                setInterval(function () {
                    var data = ej.parseJSON(window.localStorage.getItem(guid));
                    var forDel = [];
                    for (var i = 0; i < data.results.length; i++) {
                        data.results[i].timeStamp = new Date() - new Date(data.results[i].timeStamp)
                        if (new Date() - new Date(data.results[i].timeStamp) > timeStamp)
                            forDel.push(i);
                    }
                    var d = forDel;
                    for (var i = 0; i < forDel.length; i++) {
                        data.results.splice(forDel[i], 1);
                        data.keys.splice(forDel[i], 1);
                    }
                    window.localStorage.removeItem(guid);
                    window.localStorage.setItem(guid, JSON.stringify(data));
                }, timeStamp);
            }
        },
        generateKey: function (url, query) {
            var sorted = filterQueries(query.queries, "onSortBy"),
                grouped = filterQueries(query.queries, "onGroup"),
                filters = filterQueries(query.queries, "onWhere"),
                searchs = filterQueries(query.queries, "onSearch"),
				pageQuery = filterQueries(query.queries, "onPage"),
                singles = filterQueryLists(query.queries, ["onSelect", "onPage", "onSkip", "onTake", "onRange"]),
                params = query._params;
            var key = url;
            key += singles["onPage"].pageIndex;
            sorted.forEach(function (obj) {
                key += obj.e.direction + obj.e.fieldName;
            });
            grouped.forEach(function (obj) {
                key += obj.e.fieldName;
            });
            searchs.forEach(function (obj) {
                key += obj.e.searchKey;
            });
            for (var filter = 0; filter < filters.length; filter++) {
                var currentFilter = filters[filter];
                if (currentFilter.e.isComplex) {
                    var newQuery = query.clone();
                    newQuery.queries = [];
                    for (var i = 0; i < currentFilter.e.predicates.length; i++) {
                        newQuery.queries.push({ fn: "onWhere", e: currentFilter.e.predicates[i], filter: query.queries.filter });
                    }
                    key += currentFilter.e.condition + this.generateKey(url, newQuery);
                }
                else
                    key += currentFilter.e.field + currentFilter.e.operator + currentFilter.e.value
            }
            return key;
        },
        processQuery: function (dm, query, hierarchyFilters) {
            var key = this.generateKey(dm.dataSource.url, query);
            var cachedItems;
            if (window.localStorage)
                cachedItems = ej.parseJSON(window.localStorage.getItem(this.guidId));
            var data = cachedItems ? cachedItems.results[cachedItems.keys.indexOf(key)] : null;
            if (data != null) {
                return data;
            }
            return this.cacheAdaptor.processQuery.apply(this.cacheAdaptor, [].slice.call(arguments, 0))
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var data = this.cacheAdaptor.processResponse.apply(this, [].slice.call(arguments, 0));
            var key = this.generateKey(ds.dataSource.url, query)
            var obj = {};
            if (window.localStorage)
                obj = ej.parseJSON(window.localStorage.getItem(this.guidId));
            obj.results[obj.keys.push(key) - 1] = { keys: key, result: data.result, timeStamp: new Date(), count: data.count }
            while (obj.results.length > this.pageSize) {
                obj.results.splice(0, 1);
                obj.keys.splice(0, 1);
            }
            window.localStorage.setItem(this.guidId, JSON.stringify(obj));
            return data;
        }
    });
    var filterQueries = function (queries, name) {
        return queries.filter(function (q) {
            return q.fn === name;
        }) || [];
    };
    var filterQueryLists = function (queries, singles) {
        var filtered = queries.filter(function (q) {
            return singles.indexOf(q.fn) !== -1;
        }), res = {};
        for (var i = 0; i < filtered.length; i++) {
            if (!res[filtered[i].fn])
                res[filtered[i].fn] = filtered[i].e;
        }
        return res;
    };
    var callAdaptorFunc = function (obj, fnName, param, param1) {
        if (obj[fnName]) {
            var res = obj[fnName](param, param1);
            if (!isNull(res)) param = res;
        }
        return param;
    };

    ej.ODataAdaptor = new ej.UrlAdaptor().extend({
        options: {
            requestType: "get",
            accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
            multipartAccept: "multipart/mixed",
            sortBy: "$orderby",
            select: "$select",
            skip: "$skip",
            take: "$top",
            count: "$inlinecount",
            where: "$filter",
            expand: "$expand",
            batch: "$batch",
            changeSet: "--changeset_",
            batchPre: "batch_",
            contentId: "Content-Id: ",
            batchContent: "Content-Type: multipart/mixed; boundary=",
            changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
            batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
        },
        onEachWhere: function (filter, requiresCast) {
            return filter.isComplex ? this.onComplexPredicate(filter, requiresCast) : this.onPredicate(filter, requiresCast);
        },
        onPredicate: function (pred, query, requiresCast) {
            var returnValue = "",
                operator,guid,
                val = pred.value,
                type = typeof val,
                field = this._p(pred.field);

            if (val instanceof Date)
                val = "datetime'" + dateSerialize(val) + "'";

            if (type === "string") {
                val = "'" + val + "'";

                if (requiresCast) {
                    field = "cast(" + field + ", 'Edm.String')";
                }
                if (ej.isGUID(val))
                    guid = 'guid';
                if (pred.ignoreCase) {
                    !guid ? field = "tolower(" + field + ")" : field;
                    val = val.toLowerCase();
                }
            }

            operator = ej.data.odBiOperator[pred.operator];
            if (operator) {
                returnValue += field;
                returnValue += operator;
                if (guid)
                    returnValue += guid;
                return returnValue + val;
            }

            operator = ej.data.odUniOperator[pred.operator];
            if (!operator || type !== "string") return "";

            if (operator === "substringof") {
                var t = val;
                val = field;
                field = t;
            }

            returnValue += operator + "(";
            returnValue += field + ",";
            if (guid) returnValue += guid;
            returnValue += val + ")";

            return returnValue;
        },
        onComplexPredicate: function (pred, requiresCast) {
            var res = [];
            for (var i = 0; i < pred.predicates.length; i++) {
                res.push("(" + this.onEachWhere(pred.predicates[i], requiresCast) + ")");
            }
            return res.join(" " + pred.condition + " ");
        },
        onWhere: function (filters) {
            if (this.pvt.searches)
                filters.push(this.onEachWhere(this.pvt.searches, null, true));

            return filters.join(" and ");
        },
        onEachSearch: function (e) {
            if (e.fields.length === 0)
                throwError("Query() - Search : oData search requires list of field names to search");

            var filter = this.pvt.searches || [];
            for (var i = 0; i < e.fields.length; i++) {
                filter.push(new ej.Predicate(e.fields[i], e.operator, e.key, e.ignoreCase));
            }
            this.pvt.searches = filter;
        },
        onSearch: function (e) {
            this.pvt.searches = ej.Predicate.or(this.pvt.searches);
            return "";
        },
        onEachSort: function (e) {
            var res = [];
            if (e.name instanceof Array) {
                for (var i = 0; i < e.name.length; i++)
                    res.push(this._p(e.name[i]));
            } else
                res.push(this._p(e.name) + (e.direction === "descending" ? " desc" : ""));

            return res.join(",");
        },
        onSortBy: function (e) {
            return e.reverse().join(",");
        },
        onGroup: function (e) {
            this.pvt.groups = e;
            return "";
        },
        onSelect: function (e) {
            for (var i = 0; i < e.length; i++)
                e[i] = this._p(e[i]);

            return e.join(',');
        },
        onAggregates: function(e){
            this.pvt.aggregates = e;
            return "";
        },
        onCount: function (e) {
            return e === true ? "allpages" : "";
        },
        beforeSend: function (dm, request, settings) {
            if (settings.url.endsWith(this.options.batch) && settings.type.toLowerCase() === "post") {
                request.setRequestHeader("Accept", oData.multipartAccept);
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.overrideMimeType("text/plain; charset=x-user-defined");
            }

            if (!dm.dataSource.crossDomain) {
                request.setRequestHeader("DataServiceVersion", "2.0");
                request.setRequestHeader("MaxDataServiceVersion", "2.0");
            }
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            if (!ej.isNullOrUndefined(data.d)) {
                var dataCopy = query._requiresCount ? data.d.result : data.d;
                if (!ej.isNullOrUndefined(dataCopy))
                    for (var i = 0; i < dataCopy.length; i++) {
                        !ej.isNullOrUndefined(dataCopy[i].__metadata) && delete dataCopy[i].__metadata;
                    }
            }
            var pvt = request && request.ejPvtData;
            if (xhr && xhr.getResponseHeader("Content-Type") && xhr.getResponseHeader("Content-Type").indexOf("xml") != -1 && data.nodeType == 9)
                return query._requiresCount ? { result: [], count: 0 } : [];
            if (request && this.options.batch && request.url.endsWith(this.options.batch) && request.type.toLowerCase() === "post") {
                var guid = xhr.getResponseHeader("Content-Type"), cIdx, jsonObj;
                guid = guid.substring(guid.indexOf("=batchresponse") + 1);
                data = data.split(guid);
                if (data.length < 2) return;

                data = data[1];
                var exVal = /(?:\bContent-Type.+boundary=)(changesetresponse.+)/i.exec(data);
                data.replace(exVal[0], "");

                var changeGuid = exVal[1];
                data = data.split(changeGuid);

                for (var i = data.length; i > -1; i--) {
                    if (!/\bContent-ID:/i.test(data[i]) || !/\bHTTP.+201/.test(data[i]))
                        continue;

                    cIdx = parseInt(/\bContent-ID: (\d+)/i.exec(data[i])[1]);

                    if (changes.added[cIdx]) {
                        jsonObj = p.parseJson(/^\{.+\}/m.exec(data[i])[0]);
                        $.extend(changes.added[cIdx], this.processResponse(jsonObj));
                    }
                }
                return changes;
            }
            var version = xhr && xhr.getResponseHeader("DataServiceVersion"), count = null, aggregateResult = {};
            version = (version && parseInt(version, 10)) || 2;

            if (query && query._requiresCount) {
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
                if (data.d) data = data.d;
                if (data.__count || data['odata.count']) count = data.__count || data['odata.count'];
            }

            if (version === 3 && data.value) data = data.value;
            if (data.d) data = data.d;
            if (version < 3 && data.results) data = data.results;

            if (pvt && pvt.aggregates && pvt.aggregates.length) {
                var agg = pvt.aggregates, args = {}, fn, res = {};
                for (var i = 0; i < agg.length; i++) {
                    fn = ej.aggregates[agg[i].type];
                    if (fn)
                        res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
                }
                aggregateResult = res;
            }
            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups;
                for (var i = 0; i < groups.length; i++)
                    data = ej.group(data, groups[i], pvt.aggregates);
            }
            return isNull(count) ? data : { result: data, count: count, aggregates: aggregateResult };
        },
        convertToQueryString: function (req, query, dm) {
            var res = [], tableName = req.table || "";
            delete req.table;

            if (dm.dataSource.requiresFormat)
                req["$format"] = "json";

            for (var prop in req)
                res.push(prop + "=" + req[prop]);

            res = res.join("&");

            if (dm.dataSource.url && dm.dataSource.url.indexOf("?") !== -1 && !tableName)
                return res;

            return res.length ? tableName + "?" + res : tableName || "";
        },
        insert: function (dm, data, tableName) {
            return {
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : ''),
                data: JSON.stringify(data)
            }
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "DELETE",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value + ')'
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "PUT",
                url: dm.dataSource.url.replace(/\/*$/, tableName ? '/' + tableName : '') + '(' + value[keyField] + ')',
                data: JSON.stringify(value),
                accept: this.options.accept
            };
        },
        batchRequest: function (dm, changes, e) {
            var initialGuid = e.guid = ej.getGuid(oData.batchPre);
            var url = dm.dataSource.url.replace(/\/*$/, '/' + this.options.batch);
            var args = {
                url: e.url,
                key: e.key,
                cid: 1,
                cSet: ej.getGuid(oData.changeSet)
            };
            var req = "--" + initialGuid + "\n";

            req += "Content-Type: multipart/mixed; boundary=" + args.cSet.replace("--", "") + "\n";

            this.pvt.changeSet = 0;

            req += this.generateInsertRequest(changes.added, args);
            req += this.generateUpdateRequest(changes.changed, args);
            req += this.generateDeleteRequest(changes.deleted, args);

            req += args.cSet + "--\n";
            req += "--" + initialGuid + "--";

            return {
                type: "POST",
                url: url,
                contentType: "multipart/mixed; charset=UTF-8;boundary=" + initialGuid,
                data: req
            };
        },
        generateDeleteRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "DELETE ";
                req += e.url + "(" + arr[i][e.key] + ") HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + this.pvt.changeSet++ + "\n";
                req += oData.batchChangeSetContentType + "\n";
            }

            return req + "\n";
        },
        generateInsertRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "POST ";
                req += e.url + " HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + this.pvt.changeSet++ + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n";
            }

            return req;
        },
        generateUpdateRequest: function (arr, e) {
            if (!arr) return "";
            var req = "";

            for (var i = 0; i < arr.length; i++) {
                req += "\n" + e.cSet + "\n";
                req += oData.changeSetContent + "\n\n";
                req += "PUT ";
                req += e.url + "(" + arr[i][e.key] + ")" + " HTTP/1.1\n";
                req += "Accept: " + oData.accept + "\n";
                req += "Content-Id: " + this.pvt.changeSet++ + "\n";
                req += oData.batchChangeSetContentType + "\n\n";

                req += JSON.stringify(arr[i]) + "\n\n";
            }

            return req;
        },
        _p: function (prop) {
            return prop.replace(/\./g, "/");
        }
    });
    ej.ODataV4Adaptor = new ej.ODataAdaptor().extend({
        options: {
            requestType: "get",
            accept: "application/json;odata=light;q=1,application/json;odata=verbose;q=0.5",
            multipartAccept: "multipart/mixed",
            sortBy: "$orderby",
            select: "$select",
            skip: "$skip",
            take: "$top",
            count: "$count",
            search: "$search",
            where: "$filter",
            expand: "$expand",
            batch: "$batch",
            changeSet: "--changeset_",
            batchPre: "batch_",
            contentId: "Content-Id: ",
            batchContent: "Content-Type: multipart/mixed; boundary=",
            changeSetContent: "Content-Type: application/http\nContent-Transfer-Encoding: binary ",
            batchChangeSetContentType: "Content-Type: application/json; charset=utf-8 "
        },
        onCount: function (e) {
            return e === true ? "true" : "";
        },
        onPredicate: function (pred, query, requiresCast) {
            var returnValue = "",
                val = pred.value,
                isDate = val instanceof Date;               

                returnValue = this.base.onPredicate.call(this, pred, query, requiresCast);

                if (isDate)
                    returnValue = returnValue.replace(/datetime'(.*)'$/, "$1");

            return returnValue;
        },
        onEachSearch: function (e) {
			 var search = this.pvt.search || [];
			 search.push(e.key);
			 this.pvt.search = search;
		},
		onSearch: function (e) {
			 return this.pvt.search.join(" OR ");
		},
        beforeSend: function (dm, request, settings) {
 
        },
        processResponse: function (data, ds, query, xhr, request, changes) {
            var pvt = request && request.ejPvtData;
            if (xhr && xhr.getResponseHeader("Content-Type") && xhr.getResponseHeader("Content-Type").indexOf("xml") != -1 && data.nodeType == 9)
                return query._requiresCount ? { result: [], count: 0 } : [];
            if (request && this.options.batch && request.url.endsWith(this.options.batch) && request.type.toLowerCase() === "post") {
                var guid = xhr.getResponseHeader("Content-Type"), cIdx, jsonObj;
                guid = guid.substring(guid.indexOf("=batchresponse") + 1);
                data = data.split(guid);
                if (data.length < 2) return;

                data = data[1];
                var exVal = /(?:\bContent-Type.+boundary=)(changesetresponse.+)/i.exec(data);
                data.replace(exVal[0], "");

                var changeGuid = exVal[1];
                data = data.split(changeGuid);

                for (var i = data.length; i > -1; i--) {
                   if (!/\bContent-ID:/i.test(data[i]) || !/\bHTTP.+201/.test(data[i]))
                        continue;

                    cIdx = parseInt(/\bContent-ID: (\d+)/i.exec(data[i])[1]);

                    if (changes.added[cIdx]) {
                        jsonObj = p.parseJson(/^\{.+\}/m.exec(data[i])[0]);
                        $.extend(changes.added[cIdx], this.processResponse(jsonObj));
                    }
                }
                return changes;
           }
            var count = null, aggregateResult = {};
            if (query && query._requiresCount)
                if (data['@odata.count']) count = data['@odata.count'];

           data = data.value;
           if (pvt && pvt.aggregates && pvt.aggregates.length) {
               var agg = pvt.aggregates, args = {}, fn, res = {};
               for (var i = 0; i < agg.length; i++) {
                   fn = ej.aggregates[agg[i].type];
                   if (fn)
                       res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
               }
               aggregateResult = res;
           }
            if (pvt && pvt.groups && pvt.groups.length) {
                var groups = pvt.groups;
                for (var i = 0; i < groups.length; i++)
                    data = ej.group(data, groups[i], pvt.aggregates);
            }
            return isNull(count) ? data : { result: data, count: count, aggregates: aggregateResult };
        },
    });
    ej.JsonAdaptor = new ej.Adaptor().extend({
        processQuery: function (ds, query) {
            var result = ds.dataSource.json.slice(0), count = result.length, cntFlg = true, ret, key, agg = {};

            for (var i = 0; i < query.queries.length; i++) {
                key = query.queries[i];
                ret = this[key.fn].call(this, result, key.e, query);
                if (key.fn == "onAggregates")
                    agg[key.e.field + " - " + key.e.type] = ret;
                else
                result = ret !== undefined ? ret : result;

                if (key.fn === "onPage" || key.fn === "onSkip" || key.fn === "onTake" || key.fn === "onRange") cntFlg = false;

                if (cntFlg) count = result.length;
            }

            if (query._requiresCount) {
                result = {
                    result: result,
                    count: count,
                    aggregates: agg
                };
            }

            return result;
        },
        batchRequest: function (dm, changes, e) {
            var i;
            for (i = 0; i < changes.added.length; i++)
                this.insert(dm, changes.added[i]);
            for (i = 0; i < changes.changed.length; i++)
                this.update(dm, e.key, changes.changed[i]);
            for (i = 0; i < changes.deleted.length; i++)
                this.remove(dm, e.key, changes.deleted[i]);
            return changes;
        },
        onWhere: function (ds, e) {
            if (!ds) return ds;

            return ds.filter(function (obj) {
                return e.validate(obj);
            });
        },
        onAggregates: function(ds, e){
            var fn = ej.aggregates[e.type];
            if (!ds || !fn || ds.length == 0) return null;
            return fn(ds, e.field);
        },
        onSearch: function (ds, e) {
            if (!ds || !ds.length) return ds;

            if (e.fieldNames.length === 0) {
                ej.pvt.getFieldList(ds[0], e.fieldNames);
            }

            return ds.filter(function (obj) {
                for (var j = 0; j < e.fieldNames.length; j++) {
                    if (e.comparer.call(obj, ej.pvt.getObject(e.fieldNames[j], obj), e.searchKey, e.ignoreCase))
                        return true;
                }
                return false;
            });
        },
        onSortBy: function (ds, e, query) {
            if (!ds) return ds;
            var fnCompare, field = getValue(e.fieldName, query);
            if (!field)
                return ds.sort(e.comparer);

            if (field instanceof Array) {
                field = field.slice(0);

                for (var i = field.length - 1; i >= 0; i--) {
                    if (!field[i]) continue;

                    fnCompare = e.comparer;

                    if (field[i].endsWith(" desc")) {
                        fnCompare = ej.pvt.fnSort(ej.sortOrder.Descending);
                        field[i] = field[i].replace(" desc", "");
                    }

                    ds = stableSort(ds, field[i], fnCompare, []);
                }
                return ds;
            }
            return stableSort(ds, field, e.comparer, query ? query.queries : []);
        },
        onGroup: function (ds, e, query) {
            if (!ds) return ds;
            var aggQuery = filterQueries(query.queries, "onAggregates"), agg = [];
            if (aggQuery.length) {
                var tmp;
                for (var i = 0; i < aggQuery.length; i++) {
                    tmp = aggQuery[i].e;
                    agg.push({ type: tmp.type, field: getValue(tmp.field, query) });
                }
            }
            return ej.group(ds, getValue(e.fieldName, query), agg);
        },
        onPage: function (ds, e, query) {
            var size = getValue(e.pageSize, query),
                start = (getValue(e.pageIndex, query) - 1) * size, end = start + size;

            if (!ds) return ds;

            return ds.slice(start, end);
        },
        onRange: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.start), getValue(e.end));
        },
        onTake: function (ds, e) {
            if (!ds) return ds;

            return ds.slice(0, getValue(e.nos));
        },
        onSkip: function (ds, e) {
            if (!ds) return ds;
            return ds.slice(getValue(e.nos));
        },
        onSelect: function (ds, e) {
            if (!ds) return ds;
            return ej.select(ds, getValue(e.fieldNames));
        },
        insert: function (dm, data) {
            return dm.dataSource.json.push(data);
        },
        remove: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i;
            if (typeof value === "object")
                value = value[keyField];
            for (i = 0; i < ds.length; i++) {
                if (ds[i][keyField] === value) break;
            }

            return i !== ds.length ? ds.splice(i, 1) : null;
        },
        update: function (dm, keyField, value, tableName) {
            var ds = dm.dataSource.json, i, key = value[keyField];

            for (i = 0; i < ds.length; i++) {
                if (ds[i][keyField] === key) break;
            }

            return i < ds.length ? $.extend(ds[i], value) : null;
        }
    });
    ej.remoteSaveAdaptor = new ej.JsonAdaptor().extend({
        beforeSend: ej.UrlAdaptor.prototype.beforeSend,
        insert: ej.UrlAdaptor.prototype.insert,
        update: ej.UrlAdaptor.prototype.update,
        remove: ej.UrlAdaptor.prototype.remove,
        batchRequest: function (dm, changes, e) {
            var i;
            for (i = 0; i < changes.added.length; i++)
                ej.JsonAdaptor.prototype.insert(dm, changes.added[i]);
            for (i = 0; i < changes.changed.length; i++)
                ej.JsonAdaptor.prototype.update(dm, e.key, changes.changed[i]);
            for (i = 0; i < changes.deleted.length; i++)
                ej.JsonAdaptor.prototype.remove(dm, e.key, changes.deleted[i]);
            return {
                type: "POST",
                url: dm.dataSource.batchUrl || dm.dataSource.crudUrl || dm.dataSource.url,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                data: JSON.stringify({
                    changed: changes.changed,
                    added: changes.added,
                    deleted: changes.deleted,
                    action: "batch",
                    table: e.url,
                    key: e.key
                })
            };
        }
    });
    ej.WebApiAdaptor = new ej.ODataAdaptor().extend({
        insert: function (dm, data, tableName) {
            return {
                type: "POST",
                url: dm.dataSource.url,
                data: JSON.stringify(data)
            };
        },
        remove: function (dm, keyField, value, tableName) {
            return {
                type: "DELETE",
                url: dm.dataSource.url + "/" + value,
                data: JSON.stringify(value)
            };
        },
        update: function (dm, keyField, value, tableName) {
            return {
                type: "PUT",
                url: dm.dataSource.url,
                data: JSON.stringify(value)
            };
        },
        processResponse: function (data, ds, query, xhr, request, changes) {

            var pvt = request && request.ejPvtData;
            if (request && request.type.toLowerCase() != "post") {
                var version = xhr && xhr.getResponseHeader("DataServiceVersion"), count = null, aggregateResult = {};
                version = (version && parseInt(version, 10)) || 2;

                if (query && query._requiresCount) {
                     if (!isNull(data.Count)) count = data.Count;
                }

                if (version < 3 && data.Items) data = data.Items;

                if (pvt && pvt.aggregates && pvt.aggregates.length) {
                    var agg = pvt.aggregates, args = {}, fn, res = {};
                    for (var i = 0; i < agg.length; i++) {
                        fn = ej.aggregates[agg[i].type];
                        if (fn)
                            res[agg[i].field + " - " + agg[i].type] = fn(data, agg[i].field);
                    }
                    aggregateResult = res;
                }
                if (pvt && pvt.groups && pvt.groups.length) {
                    var groups = pvt.groups;
                    for (var i = 0; i < groups.length; i++)
                        data = ej.group(data, groups[i], pvt.aggregates);
                }
                return isNull(count) ? data : { result: data, count: count, aggregates: aggregateResult };
            }
        }
    });
    var getValue = function (value, inst) {
        if (typeof value === "function")
            return value.call(inst || {});
        return value;
    }

    ej.TableModel = function (name, jsonArray, dataManager, modelComputed) {
        if (!instance(this, ej.TableModel))
            return new ej.TableModel(jsonArray);

        if (!instance(jsonArray, Array))
            throwError("ej.TableModel - Json Array is required");

        var rows = [], model, dirtyFn = $proxy(setDirty, this);

        for (var i = 0; i < jsonArray.length; i++) {
            model = new ej.Model(jsonArray[i], this);
            model.state = "unchanged";
            model.on("stateChange", dirtyFn);
            if (modelComputed)
                model.computes(modelComputed);
            rows.push(model);
        }

        this.name = name || "table1";

        this.rows = ej.NotifierArray(rows);
        this._deleted = [];

        this._events = $({});

        this.dataManager = dataManager;

        this._isDirty = false;

        return this;
    };

    ej.TableModel.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },

        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },

        setDataManager: function (dataManager) {
            this.dataManagar = dataManager;
        },

        saveChanges: function () {
            if (!this.dataManager || !instance(this.dataManager, ej.DataManager))
                throwError("ej.TableModel - saveChanges : Set the dataManager using setDataManager function");

            if (!this.isDirty())
                return;

            var promise = this.dataManager.saveChanges(this.getChanges(), this.key, this.name);

            promise.done($proxy(function (changes) {
                var rows = this.toArray();
                for (var i = 0; i < rows.length; i++) {
                    if (rows.state === "added") {
                        rows.set(this.key, changes.added.filter(function (e) {
                            return e[this.key] === rows.get(this.key);
                        })[0][this.key]);
                    }
                    rows[i].markCommit();
                }

                this._events.triggerHandler({ type: "save", table: this });

            }, this));

            promise.fail($proxy(function (e) {
                this.rejectChanges();
                this._events.triggerHandler({ type: "reject", table: this, error: e });
            }, this));

            this._isDirty = false;
        },

        rejectChanges: function () {
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++)
                rows[i].revert(true);

            this._isDirty = false;
            this._events.triggerHandler({ type: "reject", table: this });
        },

        insert: function (json) {
            var model = new ej.Model(json);
            model._isDirty = this._isDirty = true;

            this.rows.push(model);

            this._events.triggerHandler({ type: "insert", model: model, table: this });
        },

        update: function (value) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var row = value, model, key = this.key, keyValue = row[key];

            model = this.rows.array.filter(function (obj) {
                return obj.get(key) === keyValue;
            });

            model = model[0];

            for (var col in row) {
                model.set(col, row[col]);
            }

            this._isDirty = true;

            this._events.triggerHandler({ type: "update", model: model, table: this });
        },

        remove: function (key) {
            if (!this.key)
                throwError("TableModel - update : Primary key should be assigned to TableModel.key");

            var field = this.key;

            var index = -1, model;

            if (key && typeof key === "object") {
                key = key[field] !== undefined ? key[field] : key.get(field);
            }

            for (var i = 0; i < this.rows.length() ; i++) {
                if (this.rows.array[i].get(field) === key) {
                    index = i;
                    break;
                }
            }

            if (index > -1) {
                model = this.rows.removeAt(index);
                model.markDelete();

                this._deleted.push({ model: model, position: index });

                this._isDirty = true;
                this._events.triggerHandler({ type: "remove", model: model, table: this });
            }
        },

        isDirty: function () {
            return this._isDirty;
        },

        getChanges: function () {

            var changes = {
                added: [],
                changed: []
            };
            var rows = this.toArray();
            for (var i = 0; i < rows.length; i++) {
                if (changes[rows[i].state])
                    changes[rows[i].state].push(rows[i].json);
            }

            changes.deleted = ej.select(this._deleted, ["model"]);

            return changes;
        },

        toArray: function () {
            return this.rows.toArray();
        },

        setDirty: function (dirty, model) {
            if (this._isDirty === !!dirty) return;

            this._isDirty = !!dirty;

            this._events.triggerHandler({ type: "dirty", table: this, model: model });
        },
        get: function (index) {
            return this.rows.array[index];
        },
        length: function () {
            return this.rows.array.length;
        },

        bindTo: function (element) {
            var marker = tDiv, template = $(element.html()), rows = this.toArray(), cur;
            if ($.inArray(element.prop("tagName").toLowerCase(), ["table", "tbody"]))
                marker = tTR;

            marker.insertBefore(element);
            element.detach().empty();

            for (var i = 0; i < rows.length; i++) {
                cur = template.clone();
                rows[i].bindTo(cur);
                element.append(cur);
            }

            element.insertAfter(marker);
            marker.remove();
        }
    };

    var tDiv = doc ? $(document.createElement("div")) : {},
        tTR = doc ? $(document.createElement("tr")) : {};

    ej.Model = function (json, table, name) {
        if (typeof table === "string") {
            name = table;
            table = null;
        }
        this.$id = getUid("m");

        this.json = json;
        this.table = table instanceof ej.TableModel ? table : null;
        this.name = name || (this.table && this.table.name);
        this.dataManager = (table instanceof ej.DataManager) ? table : table.dataManagar;
        this.actual = {};
        this._events = $({});
        this.isDirty = false;
        this.state = "added";
        this._props = [];
        this._computeEls = {};
        this._fields = {};
        this._attrEls = {};
        this._updates = {};
        this.computed = {};
    };

    ej.Model.prototype = {
        computes: function (value) {
            $.extend(this.computed, value);
        },
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        set: function (field, value) {
            var obj = this.json, actual = field, prev;
            field = field.split('.');

            for (var i = 0; i < field.length - 1; i++) {
                field = field[0];
                obj = obj[field[0]];
            }

            this.isDirty = true;
            this.changeState("changed", { from: "set" });

            prev = obj[field];

            if (this.actual[field] === undefined && !(field in this.actual))
                this.actual[field] = value; // Complex property ?

            obj[field] = value;

            this._updateValues(field, value);
            this._events.triggerHandler({ type: actual, current: value, previous: prev, model: this });
        },
        get: function (field) {
            return ej.pvt.getObject(field, this.json);
        },
        revert: function (suspendEvent) {
            for (var prop in this.actual) {
                this.json[prop] = this.actual[prop];
            }

            this.isDirty = false;

            if (suspendEvent)
                this.state = "unchanged";
            else
                this.changeState("unchanged", { from: "revert" });
        },
        save: function (dm, key) {
            dm = dm || this.dataManagar;
            key = key || dm.dataSource.key;
            if (!dm) throwError("ej.Model - DataManager is required to commit the changes");
            if (this.state === "added") {
                return dm.insert(this.json, this.name).done(ej.proxy(function (e) {
                    $.extend(this.json, e.record);
                }, this));
            }
            else if (this.state === "changed") {
                return dm.update(key, this.json, this.name);
            }
            else if (this.state === "deleted") {
                return dm.remove(key, this.json, this.name);
            }
        },
        markCommit: function () {
            this.isDirty = false;
            this.changeState("unchanged", { from: "commit" });
        },
        markDelete: function () {
            this.changeState("deleted", { from: "delete" });
        },
        changeState: function (state, args) {
            if (this.state === state) return;

            if (this.state === "added") {
                if (state === "deleted")
                    state = "unchanged";
                else return;
            }

            var prev = state;
            args = args || {};

            this.state = state;
            this._events.triggerHandler($.extend({ type: "stateChange", current: state, previous: prev, model: this }, args));
        },
        properties: function () {
            if (this._props.length)
                return this._props;

            for (var pr in this.json) {
                this._props.push(pr);
                this._updates[pr] = { read: [], input: [] };
            }

            return this._props;
        },
        bindTo: function (element) {
            var el = $(element), ctl, field,
                elements = el.find("[ej-observe], [ej-computed], [ej-prop]"), len = elements.length;

            el.data("ejModel", this);
            var unbindData = { fields: [], props: [], computes: [] };
            for (var i = 0; i < len; i++) {
                ctl = elements.eq(i);

                field = ctl.attr("ej-prop");
                if (field) {
                    this._processAttrib(field, ctl, unbindData);
                }
                field = ctl.attr("ej-observe");
                if (field && this._props.indexOf(field) !== -1) {
                    this._processField(ctl, field, unbindData);
                    continue;
                }

                field = ctl.attr("ej-computed");
                if (field) {
                    this._processComputed(field, ctl, unbindData);
                    continue;
                }
            }
            el.data("ejModelBinding" + this.$id, unbindData);
        },
        unbind: function (element) {
            var tmp, data = {
                props: this._attrEls,
                computes: this._computeEls
            }, isCustom = false;

            if (element) {
                data = $(element).removeData("ejModel").data("ejModelBinding" + this.$id) || data;
                isCustom = true;
            }

            for (var p in this.computed) {
                tmp = data.computes[p], p = this.computed[p];
                if (tmp && p.deps) {
                    this.off(p.deps.join(' '), tmp.handle);
                    if (isCustom)
                        delete this._computeEls[p];
                }
            }
            if (!isCustom)
                this._computeEls = {};

            for (var p in data.props) {
                tmp = data.props[p];
                if (tmp) {
                    this.off(tmp.deps.join(' '), tmp.handle);
                    delete data.props[p];
                    if (isCustom)
                        delete this._attrEls[p];
                }
            }
            if (!isCustom)
                this._attrEls = {};

            if (data.fields && data.fields.length) {
                var len = data.fields.length, ctl, idx, ty;
                for (var i = 0; i < len; i++) {
                    ctl = data.fields[i];
                    $(ctl).off("change", null, this._changeHandler);

                    ty = this.formElements.indexOf(ctl.tagName.toLowerCase()) !== -1 ? "input" : "read";
                    idx = this._updates[ty].indexOf(ctl);
                    if (idx !== -1)
                        this._updates[ty].splice(idx, 1);
                }
            }
        },
        _processComputed: function (value, element, data) {
            if (!value) return;

            var val, deps, safeVal = safeStr(value),
            type = this.formElements.indexOf(element[0].tagName.toLowerCase()) !== -1 ? "val" : "html";

            if (!this.computed[value] || !this.computed[safeVal]) {
                this.computed[safeVal] = {
                    value: new Function("var e = this; return " + value),
                    deps: this._generateDeps(value)
                }
                value = safeVal;
            }

            val = this.computed[value];
            if (!val.get) {
                val.get = function () {
                    val.value.call(this.json);
                }
            }

            deps = val.deps;
            val = val.value;

            this._updateDeps(deps);
            this._updateElement(element, type, val);

            val = { el: element, handle: $proxy(this._computeHandle, this, { value: value, type: type }) };
            this._computeEls[value] = val;
            data.computes[value] = val;

            this.on(deps.join(' '), val.handle);
        },
        _computeHandle: function (e) {
            var el = this._computeEls[e.value];
            if (el && this.computed[e.value])
                this._updateElement(el.el, e.type, this.computed[e.value].value);
        },
        _updateElement: function (el, type, val) {
            el[type](val.call($.extend({}, this.json, this.computed)));
        },
        _updateDeps: function (deps) {
            for (var i = 0; i < deps.length; i++) {
                if (!(deps[i] in this.json) && deps[i] in this.computed)
                    ej.merge(deps, this.computed[deps[i]].deps);
            }
        },
        _generateDeps: function (value) {
            var splits = value.replace(/(^e\.)|( e\.)/g, '#%^*##ej.#').split("#%^*#"),
                field, deps = [];

            for (var i = 0; i < splits.length; i++) {
                if (splits[i].startsWith("#ej.#")) {
                    field = splits[i].replace("#ej.#", "").split(' ')[0];
                    if (field && this._props.indexOf(field) !== -1)
                        deps.push(field);
                }
            }

            return deps;
        },
        _processAttrib: function (value, el, data) {
            var prop, val, res = {};
            value = value.replace(/^ +| +$/g, "").split(";");
            for (var i = 0; i < value.length; i++) {
                value[i] = value[i].split(":");
                if (value[i].length < 2) continue;

                prop = value[i][0].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
                res[prop] = value[i][1].replace(/^ +| +$/g, "").replace(/^'|^"|'$|"$/g, "");
            }
            value = res;
            var deps = [];
            for (prop in value)
                deps.push(value[prop]);

            this._updateDeps(deps);
            this._updateProps(el, value);

            res = getUid("emak");
            val = { el: el, handle: $proxy(this._attrHandle, this, res), value: value, deps: deps };
            el.prop("ejmodelattrkey", res);

            data.props[res] = val;
            this._attrEls[res] = val;

            this.on(deps.join(' '), val.handle);
        },
        _attrHandle: function (res) {
            var el = this._attrEls[res];
            if (el)
                this._updateProps(el.el, el.value);
        },
        _updateProps: function (element, value) {
            var json = this.json, t, c = this.computed;
            for (var prop in value) {
                t = value[prop];
                if (t in json)
                    t = json[t];
                else if (t in c) {
                    t = c[t];
                    if (t) {
                        t = t.value.call($.extend({}, this.json, c));
                    }
                }

                if (!isNull(t)) {
                    element.prop(prop, t);
                }
            }
        },
        _updateValues: function (prop, value) {
            var arr = this._updates[prop];

            if (!arr || (!arr.read && !arr.input)) return;

            this._ensureItems(arr.read, "html", value);
            this._ensureItems(arr.input, "val", value);
        },
        _ensureItems: function (a, type, value) {
            if (!a) return;

            for (var i = a.length - 1; i > -1; i--) {
                if (!a[i].offsetParent) {
                    a.splice(i, 1);
                    continue;
                }
                $(a[i])[type](value);
            }
        },
        _changeHandler: function (e) {
            e.data.self.set(e.data.prop, $(this).val());
        },
        _processField: function (ctl, field, data) {
            var e = { self: this, prop: field }, val = this.get(field);

            data.fields.push(ctl[0]);

            if (this.formElements.indexOf(ctl[0].tagName.toLowerCase()) === -1) {
                ctl.html(val);
                return this._updates[field].read.push(ctl[0]);
            }

            ctl.val(val)
                    .off("change", null, this._changeHandler)
                    .on("change", null, e, this._changeHandler);

            return this._updates[field].input.push(ctl[0]);
        },
        formElements: ["input", "select", "textarea"]
    };

    var safeReg = /[^\w]+/g;
    var safeStr = function (value) {
        return value.replace(safeReg, "_");
    };
    var setDirty = function (e) {
        this.setDirty(true, e.model);
    };

    ej.Predicate = function (field, operator, value, ignoreCase) {
        if (!(this instanceof ej.Predicate))
            return new ej.Predicate(field, operator, value, ignoreCase);

        if (typeof field === "string") {
            this.field = field;
            this.operator = operator;
            this.value = value;
            this.ignoreCase = ignoreCase;
            this.isComplex = false;

            this._comparer = ej.data.fnOperators.processOperator(this.operator);

        } else if (field instanceof ej.Predicate && value instanceof ej.Predicate || value instanceof Array) {
            this.isComplex = true;
            this.condition = operator.toLowerCase();
            this.predicates = [field];
            if (value instanceof Array)
                [].push.apply(this.predicates, value);
            else
                this.predicates.push(value);
        }
        return this;
    };

    ej.Predicate.and = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "and");
    };

    ej.Predicate.or = function () {
        return pvtPredicate._combinePredicates([].slice.call(arguments, 0), "or");
    };

    // Private fn
    var pvtPredicate = {
        _combinePredicates: function (predicates, operator) {
            if (!predicates.length) return undefined;
            if (predicates.length === 1) {
                if (!instance(predicates[0], Array))
                    return predicates[0];
                predicates = predicates[0];
            }
            return new ej.Predicate(predicates[0], operator, predicates.slice(1));
        },

        _combine: function (pred, field, operator, value, condition, ignoreCase) {
            if (field instanceof ej.Predicate)
                return ej.Predicate[condition](pred, field);

            if (typeof field === "string")
                return ej.Predicate[condition](pred, new ej.Predicate(field, operator, value, ignoreCase));

            return throwError("Predicate - " + condition + " : invalid arguments");
        }
    };

    ej.Predicate.prototype = {
        and: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "and", ignoreCase);
        },
        or: function (field, operator, value, ignoreCase) {
            return pvtPredicate._combine(this, field, operator, value, "or", ignoreCase);
        },
        validate: function (record) {
            var p = this.predicates, isAnd, ret;

            if (!this.isComplex) {
                return this._comparer.call(this, ej.pvt.getObject(this.field, record), this.value, this.ignoreCase);
            }

            isAnd = this.condition === "and";

            for (var i = 0; i < p.length; i++) {
                ret = p[i].validate(record);
                if (isAnd) {
                    if (!ret) return false;
                } else {
                    if (ret) return true;
                }
            }

            return isAnd;
        },
        toJSON: function () {
            var predicates, p;
            if (this.isComplex) {
                predicates = [], p = this.predicates;
                for (var i = 0; i < p.length; i++)
                    predicates.push(p[i].toJSON());
            }
            return {
                isComplex: this.isComplex,
                field: this.field,
                operator: this.operator,
                value: this.value,
                ignoreCase: this.ignoreCase,
                condition: this.condition,
                predicates: predicates
            }
        }
    };

    ej.dataUtil = {
        swap: function (array, x, y) {
            if (x == y) return;

            var tmp = array[x];
            array[x] = array[y];
            array[y] = tmp;
        },

        mergeSort: function (jsonArray, fieldName, comparer) {
            if (!comparer || typeof comparer === "string")
                comparer = ej.pvt.fnSort(comparer, true);

            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.mergeSort(jsonArray, fieldName, comparer);
        },

        max: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnDescending);
        },

        min: function (jsonArray, fieldName, comparer) {
            if (typeof fieldName === "function") {
                comparer = fieldName;
                fieldName = null;
            }

            return ej.pvt.getItemFromComparer(jsonArray, fieldName, comparer || ej.pvt.fnAscending);
        },

        distinct: function (json, fieldName, requiresCompleteRecord) {
            var result = [], val, tmp = {};
            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (!(val in tmp)) {
                    result.push(!requiresCompleteRecord ? val : json[i]);
                    tmp[val] = 1;
                }
            }
            return result;
        },

        sum: function (json, fieldName) {
            var result = 0, val, castRequired = typeof getVal(json, fieldName, 0) !== "number";

            for (var i = 0; i < json.length; i++) {
                val = getVal(json, fieldName, i);
                if (!isNaN(val) && val !== null) {
                    if (castRequired)
                       val = +val;
                   result += val;
                }
            }
            return result;
        },

        avg: function (json, fieldName) {
            return ej.sum(json, fieldName) / json.length;
        },

        select: function (jsonArray, fields) {
            var newData = [];

            for (var i = 0; i < jsonArray.length; i++) {
                newData.push(ej.pvt.extractFields(jsonArray[i], fields));
            }

            return newData;
        },

        group: function (jsonArray, field,agg, /* internal */ level,groupDs) {
            level = level || 1;

            if (jsonArray.GROUPGUID == ej.pvt.consts.GROUPGUID) {
                for (var j = 0; j < jsonArray.length; j++) {
                    if(!ej.isNullOrUndefined(groupDs)){
                        var indx = -1;
                        var temp = $.grep(groupDs,function(e){return e.key==jsonArray[j].key});
                        indx = groupDs.indexOf(temp[0]);
                        jsonArray[j].items = ej.group(jsonArray[j].items, field, agg, jsonArray.level + 1,groupDs[indx].items);
                        jsonArray[j].count = groupDs[indx].count;
                    }
                    else{
                        jsonArray[j].items = ej.group(jsonArray[j].items, field, agg, jsonArray.level + 1);
                        jsonArray[j].count = jsonArray[j].items.length;
                    }  
                }

                jsonArray.childLevels += 1;
                return jsonArray;
            }

            var grouped = {}, groupedArray = [];

            groupedArray.GROUPGUID = ej.pvt.consts.GROUPGUID;
            groupedArray.level = level;
            groupedArray.childLevels = 0;
            groupedArray.records = jsonArray;

            for (var i = 0; i < jsonArray.length; i++) {
                var val = getVal(jsonArray, field, i);

                if (!grouped[val]) {
                    grouped[val] = {
                        key: val,
                        count: 0,
                        items: [],
                        aggregates: {},
                        field: field
                    };
                    groupedArray.push(grouped[val]);
					if(!ej.isNullOrUndefined(groupDs)) {
                        var tempObj = $.grep(groupDs,function(e){return e.key==grouped[val].key});
                       grouped[val].count = tempObj[0].count
                    }
                }

                grouped[val].count = !ej.isNullOrUndefined(groupDs) ? grouped[val].count :  grouped[val].count += 1;
                grouped[val].items.push(jsonArray[i]);
            }
            if (agg && agg.length) {

                for (var i = 0; i < groupedArray.length; i++) {
                    var res = {}, fn;
                    for (var j = 0; j < agg.length; j++) {

                        fn = ej.aggregates[agg[j].type];
                        if(!ej.isNullOrUndefined(groupDs)) {
                            var temp = $.grep(groupDs,function(e){return e.key==groupedArray[i].key});
                            if(fn)
                                res[agg[j].field + " - " + agg[j].type] = fn(temp[0].items, agg[j].field);
                        }
                        else{
                            if (fn)
                                res[agg[j].field + " - " + agg[j].type] = fn(groupedArray[i].items, agg[j].field);
                        }

                    }
                    groupedArray[i]["aggregates"] = res;
                }
            }
            return groupedArray;
        },

        parseTable: function (table, headerOption, headerRowIndex) {
            var tr = table.rows, headerRow, headerTds = [], data = [], i;

            if (!tr.length) return [];

            headerRowIndex = headerRowIndex || 0;

            switch ((headerOption || "").toLowerCase()) {
                case ej.headerOption.tHead:
                    headerRow = table.tHead.rows[headerRowIndex];
                    break;
                case ej.headerOption.row:
                default:
                    headerRow = table.rows[headerRowIndex];
                    break;
            }

            var hTd = headerRow.cells;

            for (i = 0; i < hTd.length; i++)
                headerTds.push($.trim(hTd[i].innerHTML));

            for (i = headerRowIndex + 1; i < tr.length; i++) {
                var json = {}, td = tr[i].cells;
                for (var j = 0; j < td.length; j++) {
                    var temp = td[j].innerHTML;
                    if (typeof temp == "string" && $.isNumeric(temp))
                       json[headerTds[j]] = Number(temp);
				    else
                       json[headerTds[j]] = temp;
                }
                data.push(json);
            }
            return data;
        }
    };

    ej.headerOption = {
        tHead: "thead",
        row: "row"
    };

    ej.aggregates = {
        sum: function (ds, field) {
            return ej.sum(ds, field);
        },
        average: function (ds, field) {
            return ej.avg(ds, field);
        },
        minimum: function (ds, field) {
            return ej.min(ds, field)[field];
        },
        maximum: function (ds, field) {
            return ej.max(ds, field)[field];
        },
        truecount: function (ds, field){
            var predicate = ej.Predicate(field, "equal", true);
            return ej.DataManager(ds).executeLocal(ej.Query().where(predicate)).length;
        },
        falsecount: function (ds, field) {
            var predicate = ej.Predicate(field, "equal", false);
            return ej.DataManager(ds).executeLocal(ej.Query().where(predicate)).length;
        },
        count: function (ds, field) {
            return ds.length;
        }

    };
    ej.pvt = {
        filterQueries: filterQueries,
        mergeSort: function (jsonArray, fieldName, comparer) {
            if (jsonArray.length <= 1)
                return jsonArray;

            // else list size is > 1, so split the list into two sublists
            var middle = parseInt(jsonArray.length / 2, 10);

            var left = jsonArray.slice(0, middle),
                right = jsonArray.slice(middle);

            left = ej.pvt.mergeSort(left, fieldName, comparer);
            right = ej.pvt.mergeSort(right, fieldName, comparer);

            return ej.pvt.merge(left, right, fieldName, comparer);
        },

        getItemFromComparer: function (array, field, comparer) {
            var keyVal, current, key, i = 0,castRequired = typeof getVal(array, field, 0) == "string";
            if (array.length)
            while (ej.isNullOrUndefined(keyVal)) {
                keyVal = getVal(array, field, i);
                key = array[i++];
            }
            for (; i < array.length; i++) {
                current = getVal(array, field, i);
                if (ej.isNullOrUndefined(current))
                    continue;
                if (castRequired) {
                    keyVal = +keyVal;
                    current = +current;
                }
                if (comparer(keyVal, current) > 0) {
                    keyVal = current;
                    key = array[i];
                }
            }
            return key;
        },

        quickSelect: function (array, fieldName, left, right, k, comparer) {
            if (left == right)
                return array[left];

            var pivotNewIndex = ej.pvt.partition(array, fieldName, left, right, comparer);

            var pivotDist = pivotNewIndex - left + 1;

            if (pivotDist == k)
                return array[pivotNewIndex];

            else if (k < pivotDist)
                return ej.pvt.quickSelect(array, fieldName, left, pivotNewIndex - 1, k, comparer);
            else
                return ej.pvt.quickSelect(array, fieldName, pivotNewIndex + 1, right, k - pivotDist, comparer);
        },

        extractFields: function (obj, fields) {
            var newObj = {};

            if (fields.length == 1)
                return ej.pvt.getObject(fields[0], obj);

            for (var i = 0; i < fields.length; i++) {
                newObj[fields[i].replace('.', ej.pvt.consts.complexPropertyMerge)] = ej.pvt.getObject(fields[i], obj);
            }

            return newObj;
        },

        partition: function (array, field, left, right, comparer) {

            var pivotIndex = parseInt((left + right) / 2, 10),
                pivot = getVal(array, field, pivotIndex);

            ej.swap(array, pivotIndex, right);

            pivotIndex = left;

            for (var i = left; i < right; i++) {
                if (comparer(getVal(array, field, i), pivot)) {
                    ej.swap(array, i, pivotIndex);
                    pivotIndex++;
                }
            }

            ej.swap(array, pivotIndex, right);

            return pivotIndex;
        },

        fnSort: function (order) {
            order = order ? order.toLowerCase() : ej.sortOrder.Ascending;

            if (order == ej.sortOrder.Ascending)
                return ej.pvt.fnAscending;

            return ej.pvt.fnDescending;
        },

        fnGetComparer: function (field, fn) {
            return function (x, y) {
                return fn(ej.pvt.getObject(field, x), ej.pvt.getObject(field, y));
            }
        },

        fnAscending: function (x, y) {
            if (y === null || y === undefined)
                return -1;

            if (typeof x === "string")
                return x.localeCompare(y);

            if (x === null || x === undefined)
                return 1;

            return x - y;
        },

        fnDescending: function (x, y) {
            if (y === null || y === undefined)
                return 1;

            if (typeof x === "string")
                return x.localeCompare(y) * -1;

            if (x === null || x === undefined)
                return -1;

            return y - x;
        },

        merge: function (left, right, fieldName, comparer) {
            var result = [], current;

            while (left.length > 0 || right.length > 0) {
                if (left.length > 0 && right.length > 0) {
                    if (comparer)
                        current = comparer(getVal(left, fieldName, 0), getVal(right, fieldName, 0)) <= 0 ? left : right;
                    else
                        current = left[0][fieldName] < left[0][fieldName] ? left : right;
                } else {
                    current = left.length > 0 ? left : right;
                }

                result.push(current.shift());
            }

            return result;
        },

        getObject: function (nameSpace, from) {
            if (!from) return undefined;
            if (!nameSpace) return from;

            if (nameSpace.indexOf('.') === -1) return from[nameSpace];

            var value = from, splits = nameSpace.split('.');

            for (var i = 0; i < splits.length; i++) {

                if (value == null) break;

                value = value[splits[i]];
            }

            return value;
        },

        createObject: function (nameSpace, value, initIn) {
            var splits = nameSpace.split('.'), start = initIn || window, from = start, i;

            for (i = 0; i < splits.length; i++) {

                if (i + 1 == splits.length)
                    from[splits[i]] = value === undefined ? {} : value;
                else if (from[splits[i]] == null)
                    from[splits[i]] = {};

                from = from[splits[i]];
            }

            return start;
        },

        getFieldList: function (obj, fields, prefix) {
            if (prefix === undefined)
                prefix = "";

            if (fields === undefined || fields === null)
                return ej.pvt.getFieldList(obj, [], prefix);

            for (var prop in obj) {
                if (typeof obj[prop] === "object" && !(obj[prop] instanceof Array))
                    ej.pvt.getFieldList(obj[prop], fields, prefix + prop + ".");
                else
                    fields.push(prefix + prop);
            }

            return fields;
        }
    };

    ej.FilterOperators = {
        lessThan: "lessthan",
        greaterThan: "greaterthan",
        lessThanOrEqual: "lessthanorequal",
        greaterThanOrEqual: "greaterthanorequal",
        equal: "equal",
        contains: "contains",
        startsWith: "startswith",
        endsWith: "endswith",
        notEqual: "notequal"
    };

    ej.data = {};

    ej.data.operatorSymbols = {
        "<": "lessthan",
        ">": "greaterthan",
        "<=": "lessthanorequal",
        ">=": "greaterthanorequal",
        "==": "equal",
        "!=": "notequal",
        "*=": "contains",
        "$=": "endswith",
        "^=": "startswith"
    };

    ej.data.odBiOperator = {
        "<": " lt ",
        ">": " gt ",
        "<=": " le ",
        ">=": " ge ",
        "==": " eq ",
        "!=": " ne ",
        "lessthan": " lt ",
        "lessthanorequal": " le ",
        "greaterthan": " gt ",
        "greaterthanorequal": " ge ",
        "equal": " eq ",
        "notequal": " ne "
    };

    ej.data.odUniOperator = {
        "$=": "endswith",
        "^=": "startswith",
        "*=": "substringof",
        "endswith": "endswith",
        "startswith": "startswith",
        "contains": "substringof"
    };

    ej.data.fnOperators = {
        equal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) == toLowerCase(expected);

            return actual == expected;
        },
        notequal: function (actual, expected, ignoreCase) {
            return !ej.data.fnOperators.equal(actual, expected, ignoreCase);
        },
        lessthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) < toLowerCase(expected);

            return actual < expected;
        },
        greaterthan: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) > toLowerCase(expected);

            return actual > expected;
        },
        lessthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) <= toLowerCase(expected);

            return actual <= expected;
        },
        greaterthanorequal: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return toLowerCase(actual) >= toLowerCase(expected);

            return actual >= expected;
        },
        contains: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return !isNull(actual) && !isNull(expected) && toLowerCase(actual).indexOf(toLowerCase(expected)) != -1;

            return !isNull(actual) && !isNull(expected) && actual.toString().indexOf(expected) != -1;
        },
        notnull: function (actual) {
            return actual !== null;
        },
        isnull: function (actual) {
            return actual === null;
        },
        startswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).startsWith(toLowerCase(expected));

            return actual && expected && actual.startsWith(expected);
        },
        endswith: function (actual, expected, ignoreCase) {
            if (ignoreCase)
                return actual && expected && toLowerCase(actual).endsWith(toLowerCase(expected));

            return actual && expected && actual.endsWith(expected);
        },

        processSymbols: function (operator) {
            var fnName = ej.data.operatorSymbols[operator];
            if (fnName) {
                var fn = ej.data.fnOperators[fnName];
                if (fn) return fn;
            }

            return throwError("Query - Process Operator : Invalid operator");
        },

        processOperator: function (operator) {
            var fn = ej.data.fnOperators[operator];
            if (fn) return fn;
            return ej.data.fnOperators.processSymbols(operator);
        }
    };

    ej.NotifierArray = function (array) {
        if (!instance(this, ej.NotifierArray))
            return new ej.NotifierArray(array);

        this.array = array;

        this._events = $({});
        this._isDirty = false;

        return this;
    };

    ej.NotifierArray.prototype = {
        on: function (eventName, handler) {
            this._events.on(eventName, handler);
        },
        off: function (eventName, handler) {
            this._events.off(eventName, handler);
        },
        push: function (item) {
            var ret;

            if (instance(item, Array))
                ret = [].push.apply(this.array, item);
            else
                ret = this.array.push(item);

            this._raise("add", { item: item, index: this.length() - 1 });

            return ret;
        },
        pop: function () {
            var ret = this.array.pop();

            this._raise("remove", { item: ret, index: this.length() - 1 });

            return ret;
        },
        addAt: function (index, item) {
            this.array.splice(index, 0, item);

            this._raise("add", { item: item, index: index });

            return item;
        },
        removeAt: function (index) {
            var ret = this.array.splice(index, 1)[0];

            this._raise("remove", { item: ret, index: index });

            return ret;
        },
        remove: function (item) {
            var index = this.array.indexOf(item);

            if (index > -1) {
                this.array.splice(index, 1);
                this._raise("remove", { item: item, index: index });
            }

            return index;
        },
        length: function () {
            return this.array.length;
        },
        _raise: function (e, args) {
            this._events.triggerHandler($.extend({ type: e }, args));
            this._events.triggerHandler({ type: "all", name: e, args: args });
        },
        toArray: function () {
            return this.array;
        }
    };

    $.extend(ej, ej.dataUtil);

    // For IE8
    Array.prototype.forEach = Array.prototype.forEach || function (fn, scope) {
        for (var i = 0, len = this.length; i < len; ++i) {
            fn.call(scope, this[i], i, this);
        }
    };

    Array.prototype.indexOf = Array.prototype.indexOf || function (searchElement) {
        var len = this.length;

        if (len === 0) return -1;

        for (var i = 0; i < len; i++) {
            if (i in this && this[i] === searchElement)
                return i;
        }
        return -1;
    };

    Array.prototype.filter = Array.prototype.filter || function (fn) {
        if (typeof fn != "function")
            throw new TypeError();

        var res = [];
        var thisp = arguments[1] || this;
        for (var i = 0; i < this.length; i++) {
            var val = this[i]; // in case fun mutates this
            if (fn.call(thisp, val, i, this))
                res.push(val);
        }

        return res;
    };

    String.prototype.endsWith = String.prototype.endsWith || function (key) {
        return this.slice(-key.length) === key;
    };

    String.prototype.startsWith = String.prototype.startsWith || function (key) {
        return this.slice(0, key.length) === key;
    };

    if (!ej.support) ej.support = {};
    ej.support.stableSort = function () {
        var res = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16].sort(function () { return 0; });
		for(var i = 0; i < 17; i++){
		    if(i !== res[i]) return false;
		}
        return true;
    }();
    ej.support.cors = $.support.cors;

    if (!$.support.cors && window.XDomainRequest) {
        var httpRegEx = /^https?:\/\//i;
        var getOrPostRegEx = /^get|post$/i;
        var sameSchemeRegEx = new RegExp('^' + location.protocol, 'i');
        var xmlRegEx = /\/xml/i;

        // ajaxTransport exists in jQuery 1.5+
        $.ajaxTransport('text html xml json', function (options, userOptions, jqXHR) {
            // XDomainRequests must be: asynchronous, GET or POST methods, HTTP or HTTPS protocol, and same scheme as calling page
            if (options.crossDomain && options.async && getOrPostRegEx.test(options.type) && httpRegEx.test(userOptions.url) && sameSchemeRegEx.test(userOptions.url)) {
                var xdr = null;
                var userType = (userOptions.dataType || '').toLowerCase();
                return {
                    send: function (headers, complete) {
                        xdr = new XDomainRequest();
                        if (/^\d+$/.test(userOptions.timeout)) {
                            xdr.timeout = userOptions.timeout;
                        }
                        xdr.ontimeout = function () {
                            complete(500, 'timeout');
                        };
                        xdr.onload = function () {
                            var allResponseHeaders = 'Content-Length: ' + xdr.responseText.length + '\r\nContent-Type: ' + xdr.contentType;
                            var status = {
                                code: 200,
                                message: 'success'
                            };
                            var responses = {
                                text: xdr.responseText
                            };

                            try {
                                if (userType === 'json') {
                                    try {
                                        responses.json = JSON.parse(xdr.responseText);
                                    } catch (e) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        //throw 'Invalid JSON: ' + xdr.responseText;
                                    }
                                } else if ((userType === 'xml') || ((userType !== 'text') && xmlRegEx.test(xdr.contentType))) {
                                    var doc = new ActiveXObject('Microsoft.XMLDOM');
                                    doc.async = false;
                                    try {
                                        doc.loadXML(xdr.responseText);
                                    } catch (e) {
                                        doc = undefined;
                                    }
                                    if (!doc || !doc.documentElement || doc.getElementsByTagName('parsererror').length) {
                                        status.code = 500;
                                        status.message = 'parseerror';
                                        throw 'Invalid XML: ' + xdr.responseText;
                                    }
                                    responses.xml = doc;
                                }
                            } catch (parseMessage) {
                                throw parseMessage;
                            } finally {
                                complete(status.code, status.message, responses, allResponseHeaders);
                            }
                        };
                        xdr.onerror = function () {
                            complete(500, 'error', {
                                text: xdr.responseText
                            });
                        };
						if(navigator.userAgent.indexOf("MSIE 9.0") != -1)
							xdr.onprogress = function() {};
                        xdr.open(options.type, options.url);
                        //xdr.send(userOptions.data);
                        xdr.send();
                    },
                    abort: function () {
                        if (xdr) {
                            xdr.abort();
                        }
                    }
                };
            }
        });
    }

    $.support.cors = true;

    ej.sortOrder = {
        Ascending: "ascending",
        Descending: "descending"
    };

    // privates
    ej.pvt.consts = {
        GROUPGUID: "{271bbba0-1ee7}",
        complexPropertyMerge: "_"
    };

    // private utils
    var nextTick = function (fn, context) {
        if (context) fn = $proxy(fn, context);
        (window.setImmediate || window.setTimeout)(fn, 0);
    };

    ej.support.enableLocalizedSort = false;

    var stableSort = function (ds, field, comparer, queries) {
        if (ej.support.stableSort) {
            if(!ej.support.enableLocalizedSort && typeof ej.pvt.getObject(field, ds[0] || {}) == "string" 
                && (comparer === ej.pvt.fnAscending || comparer === ej.pvt.fnDescending)
                && queries.filter(function(e){return e.fn === "onSortBy";}).length === 1)
                return fastSort(ds, field, comparer === ej.pvt.fnDescending);
            return ds.sort(ej.pvt.fnGetComparer(field, comparer));
        }
        return ej.mergeSort(ds, field, comparer);
    };

    var fastSort = function(ds, field, isDesc){
        var old = Object.prototype.toString;
        Object.prototype.toString = (field.indexOf('.') === -1) ? function(){
            return this[field];
        }:function(){
            return ej.pvt.getObject(field, this);
        };
        ds = ds.sort();
        Object.prototype.toString = old;
        if(isDesc)
            ds.reverse();
    }

    var toLowerCase = function (val) {
        return val ? val.toLowerCase ? val.toLowerCase() : val.toString() : (val === 0 || val === false) ? val.toString() : "";
    };

    var getVal = function (array, field, index) {
        return field ? ej.pvt.getObject(field, array[index]) : array[index];
    };

    var isHtmlElement = function (e) {
        return typeof HTMLElement === "object" ? e instanceof HTMLElement :
            e && e.nodeType === 1 && typeof e === "object" && typeof e.nodeName === "string";
    };

    var instance = function (obj, element) {
        return obj instanceof element;
    };

    var getTableModel = function (name, result, dm, computed) {
        return function (tName) {
            if (typeof tName === "object") {
                computed = tName;
                tName = null;
            }
            return new ej.TableModel(tName || name, result, dm, computed);
        };
    };

    var getKnockoutModel = function (result) {
        return function (computedObservables, ko) {
            ko = ko || window.ko;

            if (!ko) throwError("Knockout is undefined");

            var model, koModels = [], prop, ob;
            for (var i = 0; i < result.length; i++) {
                model = {};
                for (prop in result[i]) {
                    if (!prop.startsWith("_"))
                        model[prop] = ko.observable(result[i][prop]);
                }
                for (prop in computedObservables) {
                    ob = computedObservables[prop];

                    if ($.isPlainObject(ob)) {
                        if (!ob.owner) ob.owner = model;
                        ob = ko.computed(ob);
                    } else
                        ob = ko.computed(ob, model);

                    model[prop] = ob;
                }
                koModels.push(model);
            }

            return ko.observableArray(koModels);
        };
    };

    var uidIndex = 0;
    var getUid = function (prefix) {
        uidIndex += 1;
        return prefix + uidIndex;
    };

    ej.getGuid = function (prefix) {
        var hexs = '0123456789abcdef', rand;
        return (prefix || "") + '00000000-0000-4000-0000-000000000000'.replace(/0/g, function (val, i) {
            rand = Math.random() * 16 | 0;
            return hexs[i === 19 ? rand & 0x3 | 0x8 : rand];
        });
    };

    var proxy = function (fn, context) {
        return function () {
            var args = [].slice.call(arguments, 0);
            args.push(this);

            return fn.apply(context || this, args);
        };
    };

    var $proxy = function (fn, context, arg) {
        if ('bind' in fn)
            return arg ? fn.bind(context, arg) : fn.bind(context);

        return function () {
            var args = arg ? [arg] : []; args.push.apply(args, arguments);
            return fn.apply(context || this, args);
        };
    };

    var dateSerialize = function (date) {
        date = JSON.parse(JSON.stringify({ val: date }, p.replacer)).val;
        return date.slice(0);
    };

    ej.merge = function (first, second) {
        if (!first || !second) return;

        Array.prototype.push.apply(first, second);
    };

    var isNull = function (val) {
        return val === undefined || val === null;
    };

    var throwError = function (er) {
        try {
            throw new Error(er);
        } catch (e) {
            throw e.message + "\n" + e.stack;
        }
    };

})(window.jQuery, window.Syncfusion, window.document);;

(function ($, undefined) {

    var $document = $(document);
    // add new event shortcuts
    $.each(("touchstart touchmove touchend " +
		"tap doubletap taphold " +
		"swipe swipeleft swiperight " + "pinch pinchin pinchout pinchstop " +
		"scrollstart scrollstop").split(" "), function (i, name) {

		    $.fn[name] = function (fn) {
		        return fn ? this.bind(name, fn) : this.trigger(name);
		    };

		    // jQuery < 1.8
		    if ($.attrFn) {
		        $.attrFn[name] = true;
		    }
		});

    var isPointer = browserInfo().isMSPointerEnabled,
    isIE11Pointer = browserInfo().pointerEnabled,
    supportTouch = 'ontouchstart' in window,
	scrollEvent = "scroll",
	touchStartEvent = isPointer ? (isIE11Pointer ? "pointerdown" : "MSPointerDown") : (supportTouch ? "touchstart" : "mousedown"),
	touchStopEvent = isPointer ? (isIE11Pointer ? "pointerup" : "MSPointerUp") : (supportTouch ? "touchend" : "mouseup"),
	touchMoveEvent = isPointer ? (isIE11Pointer ? "pointermove" : "MSPointerMove") : (supportTouch ? "touchmove" : "mousemove"),
    touchCancelEvent = isPointer ? (isIE11Pointer ? "pointercancel" : "MSPointerCancel") : (supportTouch ? "touchcancel" : "mouseleave"),
    mouseStartEvent = isPointer ? touchStartEvent : "touchstart mousedown",
    mouseStopEvent = isPointer ? touchStopEvent : "touchend mouseup",
    mouseMoveEvent = isPointer ? touchMoveEvent : "touchmove mousemove",
    browser = browserInfo(),
    isIE9 = ((browser.name == 'msie') && (browser.version == '9.0')) ? true : false;

    function browserInfo() {
        var browser = {}, clientInfo = [],
        browserClients = {
            webkit: /(chrome)[ \/]([\w.]+)/i, safari: /(webkit)[ \/]([\w.]+)/i, msie: /(msie) ([\w.]+)/i,
            opera: /(opera)(?:.*version|)[ \/]([\w.]+)/i, mozilla: /(mozilla)(?:.*? rv:([\w.]+)|)/i
        };
        for (var client in browserClients) {
            if (browserClients.hasOwnProperty(client)) {
                clientInfo = navigator.userAgent.match(browserClients[client]);
                if (clientInfo) {
                    browser.name = clientInfo[1].toLowerCase();
                    browser.version = clientInfo[2];
                    if (!!navigator.userAgent.match(/Trident\/7\./)) {
                        browser.name = "msie";
                    }
                    break;
                }
            }
        }
        browser.isMSPointerEnabled = (browser.name == 'msie') && browser.version > 9 && window.navigator.msPointerEnabled;
        browser.pointerEnabled = window.navigator.pointerEnabled;
        return browser;
    }
    function initiateCustomEvent(obj, eventType, e) {
        var originalType = e.type;
        e.type = eventType;
        $.event.dispatch.call(obj, e);
        e.type = originalType;
    }
    function copyObject(e, origEvent) {
        if (origEvent) {
            for (prop in origEvent) {
                if (!(prop in e)) {
                    e[prop] = origEvent[prop];
                }
            }
        }
    }
    $.event.special.ejtouchmove = {
        setup: function () {
            var thisObj = this,
				$this = $(thisObj);
            $this.bind(touchStartEvent, startMoveHandler);
            $document.bind(touchStopEvent, clearTouchMoveHandlers);
            function clearTouchMoveHandlers() {
               // $this.unbind(touchMoveEvent, moveHandler)
            }
            var coords = {};
            function startMoveHandler(e) {
                if (!(e.which && e.which !== 1)) {
                    var origTarget = e.target,
					origEvent = e.originalEvent;
                    if (isPointer)
                        coords = { x: origEvent.x, y: origEvent.y };
                    $this.bind(touchMoveEvent, moveHandler);
                }
            }
            function moveHandler(e) {
                if (!(e.which && e.which !== 1)) {
                    var origTarget = e.target,
					origEvent = e.originalEvent;
                    copyObject(e, e.originalEvent);
                    if (!isPointer || !coords || (Math.abs(coords.x - origEvent.x) > 10 || Math.abs(coords.y - origEvent.y) > 10 && isPointer))
                        initiateCustomEvent(thisObj, "ejtouchmove", e);
                }
            }
        }
    };
    function touchObj(e) {
        return e.originalEvent.touches ?
					e.originalEvent.touches[0] : isPointer ? e.originalEvent : e;
    }
    // handles swipeup and swipedown
    $.event.special.swipeupdown = {
        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);
            $this.bind(touchStartEvent, function (e) {
                var _startevent = e;
                var data = touchObj(e),
                            startPoint = {
                                time: (new Date).getTime(),
                                coords: [data.pageX, data.pageY],
                                origin: $(e.target)
                            },
                            stopPoint;
                function moveHandler(e) {
                    e.preventDefault();
                    e.stopPropagation();
                    if (!startPoint) return;
                    var data = touchObj(e);
                    stopPoint = {
                        time: (new Date).getTime(),
                        coords: [data.pageX, data.pageY]
                    };
                    if (Math.abs(startPoint.coords[1] - stopPoint.coords[1]) > 10) e.preventDefault();
                }

                $this
                            .bind(touchMoveEvent, moveHandler)
                            .one(touchStopEvent, function (e) {
                                $this.unbind(touchMoveEvent, moveHandler);
                                if (startPoint && stopPoint) {
                                    if (stopPoint.time - startPoint.time < 1000 &&
                                    Math.abs(startPoint.coords[1] - stopPoint.coords[1]) > 30 &&
                                    Math.abs(startPoint.coords[0] - stopPoint.coords[0]) < 75) {
                                        var _addDetails = { time: stopPoint.time - startPoint.time, _isSwipe: true, _isDelta: true, stopPoint: stopPoint };
                                        var _options = _getOptions(e, _addDetails, _startevent);

                                        startPoint.origin
                                        .trigger($.extend(true, { type: "swipeupdown" }, _options))
                                        .trigger($.extend(true, { type: startPoint.coords[1] > stopPoint.coords[1] ? "swipeup" : "swipedown" }, _options));
                                    }
                                }
                                startPoint = stopPoint = undefined;
                            });
            });
        }
    };
    $.event.special.scrollstart = {
        isEnabled: true,
        setup: function () {
            var thisObj = this, $this = $(thisObj), scrolling, timer;
            function trigger(e, scrollState) {
                scrolling = scrollState;
                initiateCustomEvent(thisObj, scrolling ? "scrollstart" : "scrollstop", e);
            }
            $this.bind(scrollEvent, function (e) {
                if (!$.event.special.scrollstart.isEnabled) return;
                if (!scrolling) trigger(e, true);
                clearTimeout(timer);
                timer = setTimeout(function () {
                    trigger(e, false);
                }, 250);
            });
        }
    };

    // also handles doubletap, taphold
    $.event.special.tap = {
        doubleTapThreshold: 500,
        tapholdThreshold: 750,
        canDoubleTap: function (d) {
            return ((getTimeSpan() - d.doubleTapStartTime) <= $.event.special.tap.doubleTapThreshold);
        },
        setup: function () {
            var thisObj = this, $this = $(thisObj), d = $this.data();
            checkMsieTouch($this);
            d.isDoubleTapWait = false;
            d.stopProcess = false;
            d.preTouchend = null;
            d.preTouchstart = null;

            $this.bind(mouseStartEvent, function (event) {
                d = $this.data();
                d.startTime = getTimeSpan();
                if (!d.isDoubleTapWait) d.doubleTapStartTime = d.startTime;
                if (event.type == "touchstart") d.preTouchstart = d.startTime;
                // checked mousedown event arrives within 300'ms after the touchend completes
                if (event.type == "mousedown" && (d.startTime - d.preTouchend < 300 || d.startTime - d.preTouchstart < 30))
                    d.stopProcess = true;
                else d.stopProcess = false;

                var origTarget = event.currentTarget,
					origEvent = event.originalEvent,
					timer;

                function clearTapHandlers() {
                    clearTimeout(timer);

                    $this.unbind(mouseStopEvent, clickHandler);
                    if (isIE9) $document.unbind(mouseStopEvent, clickHandler);
                    $this.unbind(touchCancelEvent, clearTapHandlers);
                    $document.unbind(mouseMoveEvent, touchMoveAction);
                }
                function touchMoveAction(e) {
                    var coor1 = e, coor2 = event;
                    if (isPointer) coor1 = e.originalEvent, coor2 = event.originalEvent;
                    else if (e.type == "touchmove" && event.type == "touchstart")
                        coor1 = e.originalEvent.changedTouches[0],
                        coor2 = event.originalEvent.changedTouches[0];
                    if (!(coor1.pageX == coor2.pageX && coor1.pageY == coor2.pageY))
                        clearTapHandlers();
                }
                function clickHandler(e) {
                    if (e.type == "touchend") d.preTouchend = getTimeSpan();
                    clearTapHandlers();

                    // ONLY trigger a 'tap' event if the startPoint target is
                    // the same as the stopPoint target.
                    if (origTarget === e.currentTarget) {
                        initiateCustomEvent(thisObj, "tap", $.extend(_getBaseOptions(e), {
                            time: getTimeSpan() - d.startTime
                        }));

                        if (d.isDoubleTapWait && $.event.special.tap.canDoubleTap(d)) {
                            d.isDoubleTapWait = false;

                            initiateCustomEvent(thisObj, "doubletap", $.extend(_getBaseOptions(e), {
                                time: getTimeSpan() - d.doubleTapStartTime
                            }));
                        }
                        else {
                            if (d.isDoubleTapWait) {
                                d.isDoubleTapWait = false;
                                d.doubleTapStartTime = d.startTime;
                            }
                            if ($.event.special.tap.canDoubleTap(d)) {
                                d.isDoubleTapWait = true;
                            }
                        }
                    }
                }

                if (!(event.which && event.which !== 1) && !d.stopProcess) {

                    $this.bind(mouseStopEvent, clickHandler);
                    if (isIE9) $document.bind(mouseStopEvent, clickHandler);
                    $this.bind(touchCancelEvent, clearTapHandlers);
                    $document.bind(mouseMoveEvent, touchMoveAction);

                    timer = setTimeout(function () {
                        if (d.isDoubleTapWait) d.isDoubleTapWait = false;
                        initiateCustomEvent(thisObj, "taphold", $.extend(_getBaseOptions(event), {
                            options: origEvent,
                            time: getTimeSpan() - d.startTime
                        }));

                    }, $.event.special.tap.tapholdThreshold);
                }
                else if (d.stopProcess) d.stopProcess = false;
            });
        }
    };    
    $.event.special.swipe = {
        scrollSupression: 10,
        duration: 1000,
        horizontalDistance: 30,
        verticalDistance: 75,
        pointers: window.navigator.msPointerEnabled,
        startPoint: function (e) {
            var data = touchObj(e);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY],
                origin: $(e.target)
            };
        },
        stopPoint: function (e) {
            var data = touchObj(e);
            return {
                time: (new Date()).getTime(),
                coords: [data.pageX, data.pageY]
            };
        },
        handleSwipe: function (startPoint, stopPoint, e, _startevent) {

            if (stopPoint.time - startPoint.time < $.event.special.swipe.duration &&
				Math.abs(startPoint.coords[0] - stopPoint.coords[0]) > $.event.special.swipe.horizontalDistance &&
				Math.abs(startPoint.coords[1] - stopPoint.coords[1]) < $.event.special.swipe.verticalDistance) {

                var _addDetails = { time: stopPoint.time - startPoint.time, _isSwipe: true, _isDelta: true, stopPoint: stopPoint };
                var _options = _getOptions(e, _addDetails, _startevent);

                startPoint.origin.trigger($.extend(true, { type: "swipe" }, _options))
					.trigger($.extend(true,
                    { type: startPoint.coords[0] > stopPoint.coords[0] ? "swipeleft" : "swiperight" }, _options));
            }
        },

        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (e) {
                var startPoint = $.event.special.swipe.startPoint(e),
					stopPoint;
                var _startevent = e;
                $(e.target).data('_dataTouchStart', { event: e, _now: new Date().getTime() });

                function moveHandler(e) {
                    if (!startPoint) return;
                    stopPoint = $.event.special.swipe.stopPoint(e);
                    if (Math.abs(startPoint.coords[0] - stopPoint.coords[0]) > $.event.special.swipe.scrollSupression) e.preventDefault();
                }

                $this.bind(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (e) {
					    $this.unbind(touchMoveEvent, moveHandler);
					    if (startPoint && stopPoint) {
					        $.event.special.swipe.handleSwipe(startPoint, stopPoint, e, _startevent);
					    }
					    startPoint = stopPoint = undefined;
					});
            });
        }
    };
    // also handles pinchin, pinchout
    $.event.special.pinch = {
        distance: function (e) {
            if (e.originalEvent.touches.length < 2) return null;
            return $.event.special.pinch._getdistance(e.originalEvent.touches[0], e.originalEvent.touches[1]);
        },
        _getdistance: function (coor1, coor2) {
            return Math.sqrt((coor1.pageX - coor2.pageX) * (coor1.pageX - coor2.pageX) +
                (coor1.pageY - coor2.pageY) * (coor1.pageY - coor2.pageY));
        },
        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);
            $this.bind(touchStartEvent, function (e) {
                var _startevent = e;
                if (e.originalEvent.touches && e.originalEvent.touches.length >= 2) {
                    var startPoint = $.event.special.pinch.distance(e), stopPoint, minDistance = 5;

                    var _pinchDistance = startPoint, moveEvent;
                    var _options = _getOptions(e, { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent);

                    $(e.target).trigger($.extend(true, { type: "pinch" }, _options));

                    function moveHandler(e) {
                        moveEvent = e;
                        stopPoint = $.event.special.pinch.distance(e) || null;
                        if (startPoint && stopPoint && Math.abs(startPoint - stopPoint) > minDistance) {

                            $(e.target).trigger($.extend(true,
                                { type: startPoint > stopPoint ? "pinchin" : "pinchout" }, _getOptions(e,
                                { _isPinch: true, _pinchDistance: _pinchDistance }, _startevent)));

                            startPoint = stopPoint;
                        }
                    }

                    $this.bind(touchMoveEvent, moveHandler)
                        .one(touchStopEvent, function () {
                            $this.unbind(touchMoveEvent, moveHandler);
                            $(e.target).trigger($.extend(true, { type: "pinchstop" }, _getOptions(moveEvent,
                                { _isPinch: true, _pinchDistance: stopPoint }, _startevent)));
                            startPoint = stopPoint = undefined;
                        });
                }
            });
        }
    };
    //  handles touchdrag, touchdrag
    $.event.special.touchdrag = {

        setup: function () {
            var thisObj = this, $this = $(thisObj);
            checkMsieTouch($this);

            $this.bind(touchStartEvent, function (e) {

                var startPoint = touchObj(e),
					stopPoint;
                var _startevent = e;
                $(e.target).data('_dataTouchStart', { event: e, _now: new Date().getTime() });

                function moveHandler(e) {
                    if (!startPoint) {
                        return;
                    }
                    stopPoint = touchObj(e);

                    if ($.event.special.pinch._getdistance(startPoint, stopPoint) > 5)
                        $(e.target).trigger($.extend(true, { type: "touchdrag" },
                            _getOptions(e, { _isdrag: true, stopPoint: stopPoint, _isDelta: true }, _startevent)));
                }

                $this.bind(touchMoveEvent, moveHandler)
					.one(touchStopEvent, function (e) {
					    $this.unbind(touchMoveEvent, moveHandler);
					    startPoint = stopPoint = undefined;
					});
            });
        }
    };
    function _getBaseOptions(e) {
        var _pointer = isPointer ? e.originalEvent.pointerType : (!e.originalEvent.touches ? "mouse" : "touch")
        var pointerType = (isPointer && isIE11Pointer == undefined) ? (_pointer == 4 ? "mouse" : "touch") : _pointer;
        e["pointerType"] = pointerType;
        if (e.type != "mousedown" && e.type != "mouseup") {
            copyObject(e, e.originalEvent);
        }
        return e;
    }
    function _getOptions(e, _details, _startevent) {
        var _distance, _time, _scale, _iDelta = {}, _startXY, _endXY;
        if (_startevent) {
            var coor;
            if (!_startevent.originalEvent.touches) coor = [_startevent.originalEvent, e.originalEvent];
            else coor = [_startevent.originalEvent.touches[0], e.originalEvent.changedTouches[0]];

            if (_details._isSwipe || _details._isdrag) {
                _distance = $.event.special.pinch._getdistance(coor[0], coor[1]);
                _time = _details.time;
                _endXY = { pageX: _details.stopPoint.pageX, pageY: _details.stopPoint.pageY };
            }
            else if (_details._isPinch) {
                _distance = $.event.special.pinch.distance(e);
                _time = e.timeStamp - _startevent.timeStamp;
                _scale = _details._pinchDistance;
            }
            if (_details._isDelta) {
                _iDelta._dTime = e.timeStamp - _startevent.timeStamp;
                _iDelta._x = coor[1].pageX - coor[0].pageX;
                _iDelta._y = coor[1].pageY - coor[0].pageY;
            }
        }

        return {
            options: e,
            delta: {
                time: _iDelta._dTime || null,
                X: _iDelta._x || null,
                Y: _iDelta._y || null
            },
            distance: _distance,
            scale: _details._isPinch ? _scale : null,
            time: _time,
            velocity: {
                XY: _distance / _iDelta._dTime || null,
                X: _iDelta._x / _iDelta._dTime || null,
                Y: _iDelta._y / _iDelta._dTime || null
            },
            currentPosition: { pageX: _endXY ? _endXY.pageX : null, pageY: _endXY ? _endXY.pageY : null }
        };
    }

    function getTimeSpan() {
        var now = new Date();
        return now.getTime();
    }
    function checkMsieTouch($this) {
        if (isPointer)
            $this.css("-ms-touch-action", "pinch-zoom").css("touch-action", "pinch-zoom");
    }

    $.each({
        scrollstop: "scrollstart",
        doubletap: "tap",
        taphold: "tap",
        swipeleft: "swipe",
        swiperight: "swipe",
        swipedown: "swipeupdown",
        swipeup: "swipeupdown",
        pinchin: "pinch",
        pinchout: "pinch",
        pinchstop: "pinch"
    }, function (event, sourceEvent) {

        $.event.special[event] = {
            setup: function () {
                $(this).bind(sourceEvent, $.noop);
            }
        };
    });

})(jQuery);;
/**
* @fileOverview Plugin to drag the html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    
    ej.widget("ejDraggable", "ej.Draggable", {
        
        element: null,

        
        model: null,
        validTags: ["div", "span", "a"],

        
        defaults: {
            
            scope: 'default', 
            
            handle: null,  
            
            dragArea: false,
            
            clone: false,
            
            distance: 1, 
            
            cursorAt: { top: -1, left: -2 }, 
            
            dragStart: null, 
            
            drag: null, 
            
            dragStop: null, 
            
            destroy: null, 
            
            helper: function () {
                return $('<div class="e-drag-helper" />').html("draggable").appendTo(document.body);
            }
        },

        
        _init: function () {
            this.handler = function () { },
			this.resizables = {},
			helpers = {};
            this._wireEvents();
            this._browser = ej.browserInfo();
            this._isIE8 = this._browser.name == "msie" && this._browser.version == "8.0";
            this._isIE9 = this._browser.name == "msie" && this._browser.version == "9.0";
        },

        _setModel: function (options) {

        },
        
        
        _destroy: function () {
            $(document)
                .unbind(ej.eventType.mouseUp, this._destroyHandler)
                .unbind(ej.eventType.mouseUp, this._dragStopHandler)
                .unbind(ej.eventType.mouseMove, this._dragStartHandler)
                .unbind(ej.eventType.mouseMove, this._dragHandler)
                .unbind("mouseleave", this._dragMouseOutHandler)
                .unbind('selectstart', false);

            

            ej.widgetBase.droppables[this.scope] = null;
            
        },

        _initialize: function (e) {
            var ori = e;
            e.preventDefault();
            e = this._getCoordinate(e);
            this.target = $(ori.currentTarget);
            this._initPosition = { x: e.pageX, y: e.pageY };
            
            $(document).bind(ej.eventType.mouseMove, this._dragStartHandler).bind(ej.eventType.mouseUp, this._destroyHandler);
            if (!this.model.clone) {
                var _offset = this.element.offset();
                this._relXposition = e.pageX - _offset.left;
                this._relYposition = e.pageY - _offset.top;
            }
            $(document.documentElement).trigger(ej.eventType.mouseDown, ori); // The next statement will prevent 'mousedown', so manually trigger it.
           //return false;
        },
        _setDragArea: function () {
            var _dragElement = $(this.model.dragArea)[0]; if (!_dragElement) return;
            var over = ($(_dragElement).css("overflow") != 'hidden');

            this.dragArea = [
                (parseInt($(_dragElement).css("borderLeftWidth"), 10) || 0) + (parseInt($(_dragElement).css("paddingLeft"), 10) || 0),
                (parseInt($(_dragElement).css("borderTopWidth"), 10) || 0) + (parseInt($(_dragElement).css("paddingTop"), 10) || 0),
                (over ? Math.max(_dragElement.scrollWidth, _dragElement.offsetWidth) : _dragElement.offsetWidth) - (parseInt($(_dragElement).css("borderLeftWidth"), 10) || 0) - (parseInt($(_dragElement).css("paddingRight"), 10) || 0) - this.helper.outerWidth() - this.margins.left - this.margins.right,
                (over ? Math.max(_dragElement.scrollHeight, _dragElement.offsetHeight) : _dragElement.offsetHeight) - (parseInt($(_dragElement).css("borderTopWidth"), 10) || 0) - (parseInt($(_dragElement).css("paddingBottom"), 10) || 0) - this.helper.outerHeight() - this.margins.top - this.margins.bottom
            ];
            this.relativeArea = $(this.model.dragArea);
        },
        _dragStart: function (e) {
            var ori = e;
            e = this._getCoordinate(e);
            
            this.margins = {
                left: (parseInt(this.element.css("marginLeft"), 10) || 0),
                top: (parseInt(this.element.css("marginTop"), 10) || 0),
                right: (parseInt(this.element.css("marginRight"), 10) || 0),
                bottom: (parseInt(this.element.css("marginBottom"), 10) || 0)
            };
            this.offset = this.element.offset();
            this.offset = {
                top: this.offset.top - this.margins.top,
                left: this.offset.left - this.margins.left
            };
            this.position = this._getMousePosition(ori);
            var x = this._initPosition.x - e.pageX, y = this._initPosition.y - e.pageY;
            var distance = Math.sqrt((x * x) + (y * y));

            if (distance >= this.model.distance) {
                var dragTargetElmnt = this.model.handle = this.helper = this.model.helper({ sender: ori, element: this.target });

                
                
                

                if (this.model.dragStart) {
                    var currTarget = null;
                    if (ori.type == 'touchmove') {
                        var coor = ori.originalEvent.changedTouches[0];
                        currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                    }
                    else currTarget = ori.originalEvent.target || ori.target;
                    if (this._trigger("dragStart", { event: ori, element: this.element, target: currTarget })) {
                        this._destroy();
                        return false;
                    }
                }
                if (this.model.dragArea)
                    this._setDragArea();
                
                var pos= dragTargetElmnt.offsetParent().offset();
                $(document).unbind(ej.eventType.mouseMove, this._dragStartHandler).unbind(ej.eventType.mouseUp, this._destroyHandler)
                    .bind(ej.eventType.mouseMove, this._dragHandler).bind(ej.eventType.mouseUp, this._dragStopHandler).bind("mouseleave", this._dragMouseOutHandler).bind("selectstart", false);
                ej.widgetBase.droppables[this.model.scope] = {
                    draggable: this.element,
                    helper: dragTargetElmnt.css({ position: 'absolute',  left: (this.position.left-pos.left), top: (this.position.top-pos.top) }),
                    destroy: this._destroyHandler
                }
            }
        },

        _drag: function (e) {
            this.position = this._getMousePosition(e);
            if (this.position.top < 0)
                this.position.top = 0;
            if ($(document).height() < this.position.top)
                this.position.top = $(document).height();
            if ($(document).width() < this.position.left)
                this.position.left = $(document).width();
            var helperElement = ej.widgetBase.droppables[this.model.scope].helper;
            if (this.model.drag) {
                var currTarget = null;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                else currTarget = e.originalEvent.target || e.target;
                this._trigger("drag", { event: e, element: this.target, target: currTarget });// Raise the dragging event
            }
            var element = this._checkTargetElement(e);
            if (!ej.isNullOrUndefined(element)) {
                e.target = e.toElement = element;
                element.object._over(e); 
                this._hoverTarget = element; 
            }
            else if (this._hoverTarget) {
                e.target = e.toElement = this._hoverTarget;
                this._hoverTarget.object._out(e);
                this._hoverTarget = null;
            }
            var helperElement = ej.widgetBase.droppables[this.model.scope].helper;
			var pos= helperElement.offsetParent().offset();			 
            helperElement.css({ left: (this.position.left-pos.left) , top: (this.position.top-pos.top)});
        },

        _dragStop: function (e) {
            if (e.type == 'mouseup' || e.type == 'touchend') 
                this._destroy(e);
            if (this.model.dragStop) {
                var currTarget = null;
                if (e.type == 'touchend') {
                    var coor = e.originalEvent.changedTouches[0];
                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                else currTarget = e.originalEvent.target || e.target;
                this._trigger("dragStop", { event: e, element: this.target, target: currTarget });// Raise the dragstop event
            }
            this._dragEnd(e);
        },
        _dragEnd: function (e) {
            var element = this._checkTargetElement(e);
            if (!ej.isNullOrUndefined(element)) {
                e.target = e.toElement = element;
                element.object._drop(e);
            }
        },

        _dragMouseEnter: function (e) {
            $(document).unbind("mouseenter", this._dragMouseEnterHandler);
            if (this._isIE9)
                this._dragManualStop(e);
            else if (this._isIE8) {
                if (e.button == 0)
                    this._dragManualStop(e);
            }
            else if (e.buttons == 0)
                this._dragManualStop(e);
        },

        _dragManualStop: function (e) {
            if (this.model.dragStop != null)
                this._trigger("dragStop", { event: e, element: this.target, target: e.originalEvent.target || e.target });  // Raise the dragstop event
            this._destroy(e);
        },

        _dragMouseOut: function (e) {
            $(document).bind("mouseenter", this._dragMouseEnterHandler);
        },

        _checkTargetElement:function(e)
        {
            var target = e.target;
            if (this.helper && this._contains(this.helper[0], target)) {
                this.helper.hide();
                target = this._elementUnderCursor(e);
                this.helper.show();
                return this._withDropElement(target);
            }
            return this._withDropElement(target);
        },
        _withDropElement:function(target)
        {
            if (target) {
                dropObj = $(target).data('ejDroppable');
                if (ej.isNullOrUndefined(dropObj)) dropObj = this._checkParentElement($(target));
                if (!ej.isNullOrUndefined(dropObj)) {
                    return $.extend(target, { object: dropObj });
                }
            }
        },
        _checkParentElement: function (element) {
            var target = $(element).closest('.e-droppable');
            if (target.length > 0) {
                dropObj = $(target).data('ejDroppable');
                if (!ej.isNullOrUndefined(dropObj)) return dropObj;
            }
        },
        _elementUnderCursor:function(e){
            if(e.type == "touchmove" || e.type == "touchstart" || e.type == "touchend")
                return document.elementFromPoint(e.originalEvent.changedTouches[0].pageX, e.originalEvent.changedTouches[0].pageY);
            else return document.elementFromPoint(e.pageX, e.pageY);
        },
        _contains:function(parent, child) {
            try {
                return $.contains(parent, child) || parent == child;
            } catch (e) {
                    return false;
                }
        },
        _wireEvents: function () {
            this._on(this.element, ej.eventType.mouseDown, this._initialize);
            this._dragStartHandler = $.proxy(this._dragStart, this);
            this._destroyHandler = $.proxy(this._destroy, this);
            this._dragStopHandler = $.proxy(this._dragStop, this);
            this._dragHandler = $.proxy(this._drag, this);
            this._dragMouseEnterHandler = $.proxy(this._dragMouseEnter, this);
            this._dragMouseOutHandler = $.proxy(this._dragMouseOut, this);
        },
        _getMousePosition: function (event) {
            event = this._getCoordinate(event);
            var pageX = this.model.clone ? event.pageX : event.pageX - this._relXposition;
            var pageY = this.model.clone ? event.pageY : event.pageY - this._relYposition;
            if (this.dragArea) {
                if (this.relativeArea) {
                    var co = this.relativeArea.offset();
                    _area = [this.dragArea[0] + co.left, this.dragArea[1] + co.top, this.dragArea[2] + co.left, this.dragArea[3] + co.top];
                }
                else {
                    _area = this.dragArea;
                }
                if (event.pageX - this.margins.left < _area[0]) pageX = _area[0] + this.margins.left;
                if (event.pageY - this.margins.top < _area[1]) pageY = _area[1] + this.margins.top;
                if (event.pageX - this.margins.left > _area[2]) pageX = _area[2] + this.margins.left;
                if (event.pageY - this.margins.top > _area[3]) pageY = _area[3] + this.margins.top;
            }
            return { left: pageX - this.margins.left - this.model.cursorAt.left, top: pageY - this.margins.top - this.model.cursorAt.top };
        },
        _getCoordinate: function (evt) {
            var coor = evt;
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend")
                coor = evt.originalEvent.changedTouches[0];
            return coor;
        }
    });

})(jQuery, Syncfusion);

/**
* @fileOverview Plugin to drop the html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejDroppable", "ej.Droppable", {
        
        element: null,        
        model: null,
        validTags: ["div", "span", "a"],
        
        defaults: {
            
            accept: null,
            
            scope: 'default',
            
            drop: null,
            
            over: null,
            
            out: null
        },

        
        _init: function () {
            if (this.model.accept) $(this.element).delegate(this.accept, 'mouseup', $.proxy(this._drop, this));
            else $(this.element).bind('mouseup', $.proxy(this._drop, this));
            this._on($(document), 'touchend', this._drop);
            this._mouseOver = false;
        },

        _setModel: function (options) {

        },
        
        
        _destroy: function () {

        },

        _over: function (e) {
            if (!this._mouseOver) {
                this._trigger("mouseover", e);
                this._mouseOver = true;
            }
        },
        _out: function (e) {
            if (this._mouseOver) {
                this._trigger("mouseout", e);
                this._mouseOver = false;
            }
        },
        _drop: function (e) {
            var drag = ej.widgetBase.droppables[this.model.scope];
            var isDragged = !ej.isNullOrUndefined(drag.helper) && drag.helper.is(":visible");
            var area = this._isDropArea(e);
            if (drag && !ej.isNullOrUndefined(this.model.drop) && isDragged && area.canDrop) {
                this.model.drop($.extend(e, { dropTarget: area.target }, true), drag);
            }
        },
        _isDropArea: function (e) {
            // check for touch devices only
            var area = { canDrop: true, target: $(e.target) };
            if (e.type == "touchend") {
                var coor = e.originalEvent.changedTouches[0], _target;
                _target = document.elementFromPoint(coor.pageX, coor.pageY);
                area.canDrop = false;
                var _parents = $(_target).parents();

                for (var i = 0; i < this.element.length; i++) {
                    if ($(_target).is($(this.element[i]))) area = { canDrop: true, target: $(_target) };
                    else for (var j = 0; j < _parents.length; j++) {
                        if ($(this.element[i]).is($(_parents[j]))) {
                            area = { canDrop: true, target: $(_target) };
                            break;
                        }
                    }
                    if (area.canDrop) break;
                }
            }
            return area;
        }
    });

})(jQuery, Syncfusion);

/**
* @fileOverview Plugin to resize the Html elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    
    ej.widget("ejResizable", "ej.resizable", {
        
        element: null,        
        model: null,
        validTags: ["div", "span", "a"],
        
        defaults: {
            
            scope: 'default',
            
            handle: null,
            
            distance: 1,
            
            maxHeight: null,
            
            maxWidth: null,
            
            minHeight: 10,
            
            minWidth: 10,
            
            cursorAt: { top: 1, left: 1 },
            
            resizeStart: null,
            
            resize: null,
            
            resizeStop: null,
            
            destroy: null,
            
            helper: function () {
                return $('<div class="e-resize-helper" />').html("resizable").appendTo(document.body);
            }
        },
        
        _init: function () {
            this.target = this.element;
            this._browser = ej.browserInfo();
            this._isIE8 = this._browser.name == "msie" && this._browser.version == "8.0";
            this._isIE9 = this._browser.name == "msie" && this._browser.version == "9.0";
            if (this.handle != null) {
                $(this.target).delegate(this.handle, ej.eventType.mouseDown, $.proxy(this._mousedown, this))
                .delegate(this.handle, 'resizestart', this._blockDefaultActions);
            }
            else {
                $(this.target).bind("mousedown", $.proxy(this._mousedown, this));                                
            }
            this._resizeStartHandler = $.proxy(this._resizeStart, this);
            this._destroyHandler = $.proxy(this._destroy, this);
            this._resizeStopHandler = $.proxy(this._resizeStop, this);
            this._resizeHandler = $.proxy(this._resize, this);
            this._resizeMouseEnterHandler = $.proxy(this._resizeMouseEnter, this);
        },
        _mouseover: function (e) {
            if ($(e.target).hasClass("e-resizable")) {
                $(e.target).css({ cursor: "se-resize" });
                $(this.target).bind(ej.eventType.mouseDown, $.proxy(this._mousedown, this));
            }
            else {
                $(this.target).unbind(ej.eventType.mouseDown);
                $(this.target).css({ cursor: "" });
            }
        },
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
        _setModel: function (options) {

        },
        _mousedown: function (e) {
            var ori = e;
            e = this._getCoordinate(e);
            this.target = $(ori.currentTarget);
            this._initPosition = { x: e.pageX, y: e.pageY };
            this._pageX = e.pageX;
            this._pageY = e.pageY;

            $(document).bind(ej.eventType.mouseMove, this._resizeStartHandler).bind(ej.eventType.mouseUp, this._destroyHandler);

            $(document.documentElement).trigger(ej.eventType.mouseDown, ori); // The next statement will prevent 'mousedown', so manually trigger it.
            return false;
        },

        _resizeStart: function (e) {
            if ($(e.target).hasClass("e-resizable")) {
                e = this._getCoordinate(e);
                var x = this._initPosition.x - e.pageX, y = this._initPosition.y - e.pageY, _width, _height;
                var distance = Math.sqrt((x * x) + (y * y));
                if (distance >= this.model.distance) {
                    if (this.model.resizeStart != null) 
                        if (this._trigger("resizeStart", { event: e, element: this.target }))  // Raise the resize start event
                            return;
                    var resizeTargetElmnt = this.model.helper({ element: this.target });
                    _width = (e.pageX - this._pageX) + resizeTargetElmnt.outerWidth();
                    _height = (e.pageY - this._pageY) + resizeTargetElmnt.outerHeight();
                    this._pageX = e.pageX;
                    this._pageY = e.pageY;
                    var pos = this.getElementPosition(resizeTargetElmnt);
                    $(document).unbind(ej.eventType.mouseMove, this._resizeStartHandler).unbind(ej.eventType.mouseUp, this._destroyHandler)
                        .bind(ej.eventType.mouseMove, this._resizeHandler).bind(ej.eventType.mouseUp, this._resizeStopHandler).bind("mouseenter", this._resizeMouseEnterHandler).bind("selectstart", false);
                    ej.widgetBase.resizables[this.scope] = {
                        resizable: this.target,
                        helper: resizeTargetElmnt.css({ width: _width, height: _height }),
                        destroy: this._destroyHandler
                    }
                }
            }
        },

        _resize: function (e) {
            var _width, _height, _diff;
            e = this._getCoordinate(e);
            var pos = this.getElementPosition(ej.widgetBase.resizables[this.scope].helper);
            var resizeTargetElmnt = this.model.helper({ element: this.target });
            _width = (e.pageX - this._pageX) + resizeTargetElmnt.outerWidth();
            _height = (e.pageY - this._pageY) + resizeTargetElmnt.outerHeight();
            this._pageX = e.pageX;
            this._pageY = e.pageY;
            if (_width < this.model.minWidth) {
                _diff = this.model.minWidth - _width;
                _width = this.model.minWidth;
                this._pageX = e.pageX + _diff;
            }
            if (_height < this.model.minHeight) {
                _diff = this.model.minHeight - _height;
                _height = this.model.minHeight;
                this._pageY = e.pageY + _diff;
            }
            if (this.model.maxHeight != null && _height > this.model.maxHeight) {
                _diff = _height - this.model.maxHeight;
                _height = this.model.maxHeight;
                this._pageY = e.pageY - _diff;
            }
            if (this.model.maxWidth != null && _width > this.model.maxWidth) {
                _diff = _width - this.model.maxWidth;
                _width = this.model.maxWidth;
                this._pageX = e.pageX - _diff;
            }
            ej.widgetBase.resizables[this.scope].helper.css({ width: _width, height: _height });
            this._trigger("resize", { element: this.target }) // Raise the resize event
        },

        _resizeStop: function (e) {
            if (this.model.resizeStop != null)
                this._trigger("resizeStop", { element: this.target });  // Raise the resize stop event
            if (e.type == 'mouseup' || e.type == 'touchend')
                this._destroy(e);
        },

        _resizeMouseEnter: function (e) {
            if (this._isIE9)
                this._resizeManualStop(e);
            else if (this._isIE8) {
                if (e.button == 0)
                    this._resizeManualStop(e);
            }
            else if (e.buttons == 0)
                this._resizeManualStop(e);
        },

        _resizeManualStop: function (e) {
            if (this.model.resizeStop != null)
                this._trigger("resizeStop", { element: this.target });  // Raise the resize stop event
            this._destroy(e);
        },

        
        _destroy: function (e) {
            $(document)
                .unbind(ej.eventType.mouseUp, this._destroyHandler)
                .unbind(ej.eventType.mouseUp, this._resizeStopHandler)
                .unbind(ej.eventType.mouseMove, this._resizeStartHandler)
                .unbind(ej.eventType.mouseMove, this._resizeHandler)
                .unbind("mouseenter", this._resizeMouseEnterHandler)
                .unbind('selectstart', false);            
            ej.widgetBase.resizables[this.scope] = null;
            
        },

        getElementPosition: function (elemnt) {
            if (elemnt != null && elemnt.length > 0)
                return {
                    left: elemnt[0].offsetLeft,
                    top: elemnt[0].offsetTop
                };
            else
                return null;
        },
        _getCoordinate: function (evt) {
            var coor = evt;
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend")
                coor = evt.originalEvent.changedTouches[0];
            return coor;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ScrollBar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, window, undefined) {
    'use strict';

    ej.widget("ejScrollBar", "ej.ScrollBar", {
        defaults: {

            orientation: "horizontal",

            viewportSize: 0,

            height: 18,

            width: 18,

            smallChange: 57,

            largeChange: 57,

            value: 0,

            maximum: 0,

            minimum: 0,

            buttonSize: 18,

            infiniteScrolling: false
        },
        validTags: ["div"],
        type: "transclude",
        dataTypes: {
            buttonSize: "number",
            smallChange: "number",
            largeChange: "number",
        },
        observables: ["value"],
        value: ej.util.valueFunction("value"),
        _enabled: true,
        content: function () {
            if (!this._content || !this._content.length) {
                if (this.model.orientation === "horizontal") {
                    this._content = this.element.find(".e-hhandle");
                }
                else {
                    this._content = this.element.find(".e-vhandle");
                }
            }
            return this._content;
        },
        _init: function () {
            this.element.addClass("e-widget");
            this._ensureScrollers();
            this.content();
            this._setInitialValues();

        },

        _setInitialValues: function () {
            var xy = "X";
            if (this.model.orientation === ej.ScrollBar.Orientation.Horizontal) {
                this.element.addClass("e-hscrollbar");
            }
            else {
                this.element.addClass("e-vscrollbar");
                xy = "Y";
            }
            if (this.value() !== 0 || this.model.minimum !== 0) {
                if (this.value() < this.model.minimum)
                    this.value(this.model.minimum);
                this["scroll"](this.value(), "none");
            }
        },

        _ensureScrollers: function () {
            var jqVersion = $.fn.jquery, height, width;
            if (this.model.height) {
                this.element.height(this.model.height);
            }
            if (this.model.width) {
                this.element.width(this.model.width);
            }            
            var d2;
            if (!this._scrollData) {
                if (this.model.orientation === "vertical") {
                    this._scrollData = this._createScroller("Height", "Y", "Top", "e-v");
                }
                else {
                    this._scrollData = this._createScroller("Width", "X", "Left", "e-h");
                }
            }
        },

        _setModel: function (option) {
            for (var prop in option) {
                if (prop === "value") {
                    if (this.value()) {
                        this.scroll(this.value(), "none");
                    }
                } else {
                    this.refresh();
                    break;
                }
            }
        },

        _createScroller: function (dimension, xy, position, css) {
            var height;
            var d = {};
            var jqVersion = $.fn.jquery;
            d.dimension = dimension;
            d.xy = xy;
            d.position = position;
            d.css = css;
            d.uDimension = dimension;

            this._calculateLayout(d);
            this._createLayout(d);
            var buttons = this[d.main].find(".e-button");

            this._off(buttons, "mousedown")
                ._on(buttons, "mousedown", { d: d, step: 1 }, this._spaceMouseDown);
            this._off(this[d.scroll], "mousedown")
                ._on(this[d.scroll], "mousedown", { d: d }, this._spaceMouseDown);
            this._off(this[d.handler], "mousedown touchstart")
                ._on(this[d.handler], "mousedown touchstart", { d: d }, this._mouseDown);

            return d;
        },
        _createLayout: function (d) {
            var divString = "<div class='" + d.css + "{0}' style='" + d.dimension + ":{1}px'>{2}</div>";
            var jqVersion = $.fn.jquery;
            var lit = {}, height;
            lit[d.dimension] = d.modelDim;

            var el = ej.buildTag(
                "div." + d.css + "scroll e-box",
                    String.format(divString, "up e-icon e-box e-button", d.buttonSize) +
                    String.format(divString, "handlespace", d.handleSpace,
                        String.format(divString, "handle e-box", d.handle)) +
                    String.format(divString, "down e-icon e-box e-button", d.buttonSize),
                lit
            );

            this.element.append(el);
            jqVersion === "1.7.1" || jqVersion === "1.7.2" ? height = d.uDimension.toLowerCase() : height = "outer" + d.uDimension;
            this[d.handler] = this.element.find("." + d.handler);
            this[d.handler].css("transition", "none");
            this[d.scroll] = this[d.handler].parent();
            this[d.main] = this[d.scroll].parent();
            this[d.main].find(".e-button")["outer" + d.uDimension](d.buttonSize);
        },
        _calculateLayout: function (d) {
            d.scrollDim = "scroll" + d.dimension;
            d.lPosition = d.position.toLowerCase();
            d.clientXy = "page" + d.xy;
            d.scrollVal = "scroll" + d.position;
            d.scrollOneStepBy = this.model.smallChange;
            d.modelDim = this.model[(d.dimension = d.dimension.toLowerCase())];
            d.handler = d.css + "handle";
            d.buttonSize = this.model.buttonSize;
            d.main = d.css + "scroll";
            d.scroll = d.css + "ScrollSpace";
            d.handleSpace = d.modelDim - 2 * d.buttonSize;
            d.scrollable = (this.model.maximum - this.model.minimum);
            var trackLength = this.model.height;
            if (this.model.orientation === "horizontal")
                trackLength = this.model.width;
            d.handle = (this.model.viewportSize / ((this.model.maximum - this.model.minimum) + this.model.viewportSize)) * (trackLength - 2 * this.model.buttonSize);
            var check;
            !ej.isNullOrUndefined(this.model.elementHeight) && typeof this.model.elementHeight === "string" && this.model.elementHeight.indexOf("%") != -1 ? check = true : check = false;
            if (d.handle < 20 && !check) d.handle = 20;
            d.onePx = d.scrollable / (d.handleSpace - d.handle);
            d.fromScroller = false;
            d.up = true;
            d.vInterval = undefined;
        },
        _updateLayout: function (d) {
            this.element.height(this.model.height);
            this.element.width(this.model.width);
            this.element.find("." + d.css + "scroll").css(d.dimension, d.modelDim + "px")
                .find(".e-button").css(d.dimension, this.model.buttonSize).end()
                .find("." + d.css + "handlespace").css(d.dimension, d.handleSpace + "px")
                .find("." + d.css + "handle").css(d.dimension, d.handle + "px");
        },
        refresh: function () {
            this._ensureScrollers();
            if (this.value()) {
                this.scroll(this.value(), "none");
            }
            if (this._scrollData) {
                this._calculateLayout(this._scrollData);
                this._updateLayout(this._scrollData);
            }
        },

        scroll: function (pixel, source, triggerEvent) {
            var dS = this._scrollData;
            if (!triggerEvent) {
                if (this.model.orientation === ej.ScrollBar.Orientation.Horizontal) {
                    if (this._trigger("scroll", { source: source || "custom", scrollData: this._scrollData, scrollLeft: pixel }))
                        return;
                }
                else {
                    if (this._trigger("scroll", { source: source || "custom", scrollData: this._scrollData, scrollTop: pixel }))
                        return;
                }
            }
            this.value(pixel);
            if (this.content().length > 0) {
                if (this.model.orientation === ej.ScrollBar.Orientation.Horizontal)
                    this.content()[0].style.left = (pixel - this.model.minimum) / this._scrollData.onePx + "px";
                else
                    this.content()[0].style.top = (pixel - this.model.minimum) / this._scrollData.onePx + "px";
            }
        },

        _changeTop: function (d, step, source) {
            var start, t;
            if (d.dimension === "height")
                start = this.value();
            else
                start = this.value();
            t = start + step;
            d.step = step;
            if ((d.enableRTL && step < 0) || (step > 0 && !d.enableRTL)) {
                if (d.enableRTL) {
                    if (t < this.model.maximum * -1)
                        t = this.model.maximum * -1;
                }
                else {
                    if (t > this.model.maximum)
                        t = this.model.maximum;
                }
            }
            else {
                if (d.enableRTL) {
                    if (t > this.model.minimum)
                        t = this.model.minimum;
                }
                else {
                    if (t < this.model.minimum)
                        t = this.model.minimum;
                }
            }
            if (t !== start || this.model.infiniteScrolling) {
                this["scroll"](t, source);
            }
            return t !== start;
        },

        _mouseUp: function (e) {
            if (!e.data) return;
            var d = e.data.d;
            e.type === "mouseup" && this[d.handler].removeClass("e-active");
            if (e.type === "mouseup" || e.type === "touchend" || (!e.toElement && !e.relatedTarget && !e.target)) {
                this._off($(document), "mousemove touchmove");
                $(document).off("mouseout mouseup touchend", ej.proxy(this._mouseUp, this));
                d.fromScroller = false;
                this[d.scroll].off("mousemove");
                this[d.handler].off("mousemove").css("transition", "");

                if (e.data.source === "thumb" && !ej.isNullOrUndefined(this.model)) {
                    $.when(this.content()).done(ej.proxy(function () {
                        this._trigger("thumbEnd", { originalEvent: e, scrollData: d });
                    }, this));
                }
            }
            d.up = true;
        },


        _mouseDown: function (down) {

            if (!this._enabled) return;
            var d = down.data.d, value, step = 0;
            d.target = down.target;
            d.fromScroller = true;
            this[d.handler].css("transition", "none");
            var prevY, top = parseInt(this[d.handler].css(d.lPosition)) || 0;
            this._on($(document), "mousemove touchmove", (function (move) {
                move.preventDefault();
                var skip = 1;
                var pageXY = move.type == "mousemove" ? move[d.clientXy] : move.originalEvent.changedTouches[0][d.clientXy];
                if (prevY && pageXY !== prevY) {
                    step = (pageXY - prevY);
                    if (this.model.infiniteScrolling) {
                        top = top + step;
                        d.step = step;
                        if (d.enableRTL ? top > 0 : top < 0) top = 0;
                        if ((top * (d.enableRTL ? -1 : 1)) + d.handle >= d.handleSpace)
                            top = (d.handleSpace - d.handle) * (d.enableRTL ? -1 : 1);
                        value = Math.ceil(top * d.onePx);
                        this["scroll"](value, "thumb");
                    }
                    else {
                        value = step * d.onePx;
                        this._changeTop(d, value, "thumb");
                    }
                    this._trigger("thumbMove", { originalEvent: move, scrollData: d });
                }
                if (skip === 1)
                    prevY = pageXY;
            }));
            this._trigger("thumbStart", { originalEvent: down, scrollData: d });
            $(document).one("mouseup touchend", { d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
        },

        _spaceMouseDown: function (e) {
            if (!e.data || !this._enabled) return;
            var d = e.data.d;
            if (e.which !== 1 || e.target === this[d.handler][0]) return;
            var step = e.data.step ? this.model.smallChange : this.model.largeChange, hTop = e.data.top || this[d.handler].offset()[d.lPosition];
            e[d.clientXy] = e[d.clientXy] || 0;
            if (e[d.clientXy] < hTop) step *= -1;
            d.target = e.target;
            this._changeTop(d, step, step === 3 ? "track" : "button");
            if (e.data.step !== 1) {
                this[d.scroll].mousemove(function () {
                    d.up = true;
                });
            }
            d.up = false;
            d.vInterval = setInterval(ej.proxy(function () {
                if (step < 0 ? hTop + (step / d.onePx) < e[d.clientXy] : hTop + d.handle + (step / d.onePx) > e[d.clientXy])
                    d.up = true;
                if (d.up) {
                    clearInterval(d.vInterval);
                    return;
                }
                this._changeTop(d, step, step === 3 ? "track" : "button");
                e.data ? hTop = e.data.top || this[d.handler].offset()[d.lPosition] : hTop = this[d.handler].offset()[d.lPosition];
            }, this), 150);

            $(document).one("mouseup", { d: d }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d }, ej.proxy(this._mouseUp, this));
        },

        _remove: function () {
            if (this.model.orientation === ej.ScrollBar.Orientation.Horizontal)
                this.element.find(".e-hscroll").remove();
            if (this.model.orientation === ej.ScrollBar.Orientation.Vertical)
                this.element.find(".e-vscroll").remove();
            this._scrollData = null;
            this._content = null;
        },

        _destroy: function () {
            this.element.remove();
        },
    });

    ej.ScrollBar.Orientation = {
        Horizontal: "horizontal",
        Vertical: "vertical"
    };
})(jQuery, Syncfusion, window);;

/**
* @fileOverview Plugin to style the Html Scroller elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/


(function ($, ej, window, undefined) {
    'use strict';

    ej.widget("ejScroller", "ej.Scroller", {

        defaults: {

            height: 250,

            autoHide: false,

            width: 0,

            scrollOneStepBy: 57,

            buttonSize: 18,

            scrollLeft: 0,

            scrollTop: 0,

            targetPane: null,

            scrollerSize: 18,

            enablePersistence: false,

            enableRTL: undefined,

            enableTouchScroll: true,

            enabled: true,

            create: null,

            destroy: null
        },
        validTags: ["div"],
        type: "transclude",

        dataTypes: {
            buttonSize: "number",
            scrollOneStepBy: "number"
        },
        observables: ["scrollTop", "scrollLeft"],
        scrollTop: ej.util.valueFunction("scrollTop"),
        scrollLeft: ej.util.valueFunction("scrollLeft"),

        keyConfigs: {
            up: "38",
            down: "40",
            left: "37",
            right: "39",
            pageUp: "33",
            pageDown: "34",
            pageLeft: "ctrl+37",
            pageRight: "ctrl+39"
        },

        content: function () {
            if (!this._content || !this._content.length || !this._content[0].offsetParent)
                this._content = this.element.children("div").first().addClass("e-content");

            return this._content;
        },
        _setFirst: true,
        _updateScroll: false,

        _init: function () {
            this.element.addClass("e-widget");
            this.content();
            this._browser = ej.browserInfo().name;
            this._eleHeight = this.model.height;
            if (this.model.enableRTL === undefined) {
                this.model.enableRTL = this.element.css("direction") === "rtl";
            }
            this._ensureScrollers();
            if (this.model.enableRTL) {
                this.element.addClass("e-rtl");
                this._rtlScrollLeftValue = this.content().scrollLeft();
            }
            this._on(this.content(), "scroll", this._scroll);
            this.model.targetPane != null && this._on(this.content().find(this.model.targetPane), "scroll", this._scroll);
            if (this.scrollLeft())
                this._setScrollLeftValue(this.scrollLeft());
            if (this.scrollTop())
                this.content().scrollTop(this.scrollTop());

            if (this.model.autoHide) {
                this.element.addClass("e-autohide");
                this._on(this.element, "mouseenter mouseleave", this._scrollerHover);
                this.content().siblings().hide();
            }
            if (this.model.enabled) {
                this.enable();
            }
            else {
                this.disable();
            }
            if (!ej.isNullOrUndefined(this.model.height) && typeof this.model.height === "string" && this.model.height.indexOf("%") != -1) {
                if (!this._vScroll)
                    $(this.content()[0]).height("");
                else {
                    $(window).bind('resize', $.proxy(this._resetScroller, this));
                    this.model.height = this._convertPercentageToPixel(parseInt(this._eleHeight), this.element.parent().height());
                }
            }
        },

        _setScrollLeftValue: function (leftValue) {
            if (this.model.enableRTL) {
                if (ej.browserInfo().name == "mozilla")
                    leftValue = leftValue < 0 ? leftValue : (leftValue * -1);
                else if (!ej.isNullOrUndefined(this._rtlScrollLeftValue) && (ej.browserInfo().name == "chrome" || this._rtlScrollLeftValue > 0))
                    leftValue = leftValue < 0 ? (this._rtlScrollLeftValue + leftValue) : (this._rtlScrollLeftValue - leftValue);
                else
                    leftValue = Math.abs(leftValue);
            }
            this.content().scrollLeft(leftValue);
        },


        _ensureScrollers: function () {
            var jqVersion = $.fn.jquery, height, width, borderLeftWidth, borderRightWidth, paddingLeftWidth, paddingRightWidth;
            if (this.model.height) {
                this.element.height(this.model.height);
            }
            if (this.model.width) {
                this.element.width(this.model.width);
            }            

            this._off(this.content(), "mousedown touchstart");
            if (this.content().length > 0) {
                if (this.isVScroll()) {
                    if (!this._vScrollbar) {
                        this._vScrollbar = this._createScrollbar(ej.ScrollBar.Orientation.Vertical, this.isHScroll());
                    }
                    if (this.model.enableTouchScroll)
                        this._on(this.content(), "mousedown touchstart", { d: this._vScrollbar._scrollData }, this._mouseDownOnContent);
                } else {
                    this._vScrollbar = null;
                    this.element.children(".e-vscrollbar").remove();
                }
                if (this.isHScroll()) {
                    if (!this._hScrollbar) {
                        this._hScrollbar = this._createScrollbar(ej.ScrollBar.Orientation.Horizontal, this.isVScroll());
                    }
                    if (this.model.enableTouchScroll)
                        this._on(this.content(), "mousedown touchstart", { d: this._hScrollbar._scrollData }, this._mouseDownOnContent);
                } else {
                    this._hScrollbar = null;
                    this.element.children(".e-hscrollbar").remove();
                }

                if (!this._vScrollbar && !this._hScrollbar)
                    this.content().css({ width: "auto", height: "auto" });

                if (!(this.element.find(".e-hscroll").length > 0)) {
                    if (this._vScrollbar) {
                        this.content().outerHeight(this.content().outerHeight() - 1);
                    }
                }
                this._hScroll = this.isHScroll(), this._vScroll = this.isVScroll();
                if (this._hScroll || this._vScroll) {
                    this.content().addClass("e-content");
                    jqVersion === "1.7.1" || jqVersion === "1.7.2" ? (height = "height", width = "width") : (height = "outerHeight", width = "outerWidth");
                    this.content()[height](this.element.height() - (this._hScroll && !this.model.autoHide ? this.model.scrollerSize : 0));
                    var rect = this.element.get(0).getBoundingClientRect();
                    rect.width ? rect = rect.width : rect = rect.right - rect.left;
                    borderLeftWidth = isNaN(parseFloat(this.element.css("border-left-width"))) ? 0 : parseFloat(this.element.css("border-left-width"));
                    borderRightWidth = isNaN(parseFloat(this.element.css("border-right-width"))) ? 0 : parseFloat(this.element.css("border-right-width"));
                    paddingLeftWidth = isNaN(parseFloat(this.element.css("padding-left"))) ? 0 : parseFloat(this.element.css("padding-left"));
                    paddingRightWidth = isNaN(parseFloat(this.element.css("padding-right"))) ? 0 : parseFloat(this.element.css("padding-right"));
                    var ElementWidth = rect - (borderLeftWidth + borderRightWidth + paddingLeftWidth + paddingRightWidth);
                    this.content()[width](ElementWidth - (this._vScroll && !this.model.autoHide ? this.model.scrollerSize : 0));
                } else
                    this.content().removeClass("e-content");
            }            
        },
        _resetScroller: function () {
            this.model.height = this._convertPercentageToPixel(parseInt(this._eleHeight), this.element.parent().height());
            this.refresh();
        },
        _convertPercentageToPixel: function (ele, outer) {
            return Math.floor((ele * outer) / 100);
        },

        isHScroll: function () {
            if (this.model.width > 0) {
                var $paneObject = this.content().find(this.model.targetPane);
                if (this.model.targetPane != null && $paneObject.length)
                    return ($paneObject[0].scrollWidth + $paneObject.siblings().width()) > this.model.width;
                else
                    return this.content()[0].scrollWidth > this.model.width;
            }
            return false;
        },

        isVScroll: function () {
            if (!ej.isNullOrUndefined(this.model.height) && typeof this.model.height === "string" && this.model.height.indexOf("%") != -1)
                return this.content()[0].scrollHeight > this.element.height();
            else
                return this.model.height > 0 && this.content()[0].scrollHeight > this.model.height;
        },

        _setModel: function (option) {
            for (var prop in option) {
                if (prop === "enableRTL") {
                    if (option[prop])
                        this.element.addClass("e-rtl");
                    else
                        this.element.removeClass("e-rtl");
                } else if (prop === "scrollLeft") {
                    this._setScrollLeftValue(this.scrollLeft());
                } else if (prop === "scrollTop") {
                    this.content().scrollTop(this.scrollTop());
                } else if (prop === "touchScroll") {
                    if (!this.model.enableTouchScroll)
                        this._off(this.content(), "mousedown touchstart");
                    else {
                        if (this._vScrollbar)
                            this._on(this.content(), "mousedown touchstart", { d: this._vScrollbar._scrollData }, this._mouseDownOnContent);
                        if (this._hScrollbar)
                            this._on(this.content(), "mousedown touchstart", { d: this._hScrollbar._scrollData }, this._mouseDownOnContent);
                    }
                }
                else if (prop === "buttonSize") {
                    if (this._vScrollbar) this._vScrollbar.model.buttonSize = this.model.buttonSize;
                    if (this._hScrollbar) this._hScrollbar.model.buttonSize = this.model.buttonSize;
                    this.refresh();
                }
                else {
                    this.scrollTop(0);
                    this.scrollLeft(0);
                    this.refresh();
                    break;
                }
            }
        },

        _createScrollbar: function (orientation, isOtherScroll) {
            var proxy = this;
            var id, viewportSize, width, height, maximum, value;
            var div = document.createElement("div");
            if (orientation === ej.ScrollBar.Orientation.Vertical) {
                width = this.model.scrollerSize;
                if (!ej.isNullOrUndefined(this.model.height) && typeof this.model.height === "string" && this.model.height.indexOf("%") != -1)
                    height = viewportSize = this.element.height() - (isOtherScroll ? this.model.scrollerSize : 0);
                else
                    height = viewportSize = this.model.height - (isOtherScroll ? this.model.scrollerSize : 0);
                maximum = this.content()[0]["scrollHeight"];
                value = this.scrollTop();
            }
            else {
                width = viewportSize = this.model.width - (isOtherScroll ? this.model.scrollerSize : 0);
                height = this.model.scrollerSize;
                var $pane = this.content().find(this.model.targetPane);
                if (this.model.targetPane != null && $pane.length)
                    maximum = $pane[0]["scrollWidth"] + $pane.parent().width() - $pane.width();
                else
                    maximum = this.content()[0]["scrollWidth"];
                value = this.scrollLeft();
            }
            if (this.element.children(".e-hscrollbar").length > 0)
                $(this.element.children(".e-hscrollbar")).before(div);
            else
                this.element.append(div);
            $(div).ejScrollBar({
                elementHeight: proxy._eleHeight,
                buttonSize: proxy.model.buttonSize,
                orientation: orientation,
                viewportSize: viewportSize,
                height: height,
                width: width,
                maximum: maximum - viewportSize,
                value: value,
                smallChange: this.model.scrollOneStepBy,
                largeChange: 3 * this.model.scrollOneStepBy,
                scroll: ej.proxy(this._scrollChanged, this),
                thumbEnd: ej.proxy(this._thumbEnd, this),
                thumbStart: ej.proxy(this._thumbStart, this),
                thumbMove: ej.proxy(this._thumbMove, this),
            });
            var scrollbar = $(div).ejScrollBar("instance");
            this._off(orientation === ej.ScrollBar.Orientation.Vertical ? this.element : scrollbar[scrollbar._scrollData.scroll], this._browser == "msie" ? "wheel mousewheel" : "mousewheel DOMMouseScroll")
                   ._on(orientation === ej.ScrollBar.Orientation.Vertical ? this.element : scrollbar[scrollbar._scrollData.scroll], this._browser == "msie" ? "wheel mousewheel" : "mousewheel DOMMouseScroll", { d: scrollbar._scrollData }, this._mouseWheel);
            if (orientation === ej.ScrollBar.Orientation.Horizontal) {
                this._scrollXdata = scrollbar._scrollData;
            }
            else
                this._scrollYdata = scrollbar._scrollData;
            if (orientation === ej.ScrollBar.Orientation.Horizontal && this.model.enableRTL) {
                scrollbar._scrollData.enableRTL = true;
            }
            scrollbar._enabled = this.model.enabled;
            return scrollbar;
        },

        _updateScrollbar: function (orientation, isOtherScroll) {
            var scrollbar = orientation === ej.ScrollBar.Orientation.Vertical ? this._vScrollbar : this._hScrollbar;
            if (scrollbar) {
                if (orientation === ej.ScrollBar.Orientation.Vertical) {
                    scrollbar.model.width = this.model.scrollerSize;
                    scrollbar.model.height = scrollbar.model.viewportSize = this.model.height - (isOtherScroll ? this.model.scrollerSize : 0);
                    scrollbar.model.maximum = (this.model.targetPane != null ? this.content().find(this.model.targetPane)[0]["scrollHeight"] : this.content()[0]["scrollHeight"]) - scrollbar.model.viewportSize;
                    scrollbar.model.value = this.scrollTop();
                }
                else {
                    scrollbar.model.width = scrollbar.model.viewportSize = this.model.width - (isOtherScroll ? this.model.scrollerSize : 0);
                    scrollbar.model.height = this.model.scrollerSize;
                    scrollbar.model.maximum = (this.model.targetPane != null ? this.content().find(this.model.targetPane)[0]["scrollWidth"] + (this.content().width() - $(this.model.targetPane).outerWidth()) : this.content()[0]["scrollWidth"]) - scrollbar.model.viewportSize;
                    if (!this.model.enableRTL)
                        scrollbar.model.value = this.scrollLeft();
                }
            }
        },

        _scrollChanged: function (e) {
            this._updateScroll = true;
            if (e.scrollTop !== undefined)
                this.scrollY(e.scrollTop, true, "", e.source);
            else if (e.scrollLeft !== undefined)
                this.scrollX(e.scrollLeft, true, "", e.source);
            this._updateScroll = false;
            var proxy = this;
            $.when(this.content()).done(ej.proxy(function () {
                proxy._trigger("scrollEnd", { scrollData: e });
            }));
        },
        _thumbStart: function (e) {
            this._trigger("thumbStart", e);
        },
        _thumbMove: function (e) {
            this._trigger("thumbMove", e);
        },
        _thumbEnd: function (e) {
            this._trigger("thumbEnd", e);
        },

        refresh: function () {
            this.element.find(">.e-content").removeAttr("style");
            this._ensureScrollers();

            this.content().scrollLeft(this.scrollLeft());
            if ((this.scrollTop() && this._vScrollbar == null) || (this._vScrollbar !== null && this._vScrollbar._scrollData != null && !this._vScrollbar._scrollData.skipChange))
                this.content().scrollTop(this.scrollTop());

            if (this._vScrollbar) {
                this._vScrollbar._scrollData.dimension = "Height";
                this._updateScrollbar(ej.ScrollBar.Orientation.Vertical, this._hScroll);
                this._vScroll && !this._vScrollbar._calculateLayout(this._vScrollbar._scrollData) && this._vScrollbar._updateLayout(this._vScrollbar._scrollData);
            }
            if (this._hScrollbar) {
                this._hScrollbar._scrollData.dimension = "Width";
                this._updateScrollbar(ej.ScrollBar.Orientation.Horizontal, this._vScroll);
                this._hScroll && !this._hScrollbar._calculateLayout(this._hScrollbar._scrollData) && this._hScrollbar._updateLayout(this._hScrollbar._scrollData);
            }
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version == "8.0")
                this.element.find(".e-hhandle").css("left", "0px");
            else
                this.model.targetPane != null && this._on(this.content().find(this.model.targetPane), "scroll", this._scroll);
        },

        _keyPressed: function (action, target) {
            if (!this.model.enabled) return;
            if (["input", "select", "textarea"].indexOf(target.tagName.toLowerCase()) !== -1)
                return true;

            var d, iChar;

            if (["up", "down", "pageUp", "pageDown"].indexOf(action) !== -1) {
                if (this._vScrollbar)
                    d = this._vScrollbar._scrollData;
                iChar = "o";
            } else if (["left", "right", "pageLeft", "pageRight"].indexOf(action) !== -1) {
                if (this._hScrollbar)
                    d = this._hScrollbar._scrollData;
                iChar = "i";
            } else return true;
            if (!d) return true;

            return !this._changeTop(d, (action.indexOf(iChar) < 0 ? -1 : 1) * (action[0] !== "p" ? 1 : 3) * d.scrollOneStepBy, "key");
        },

        scrollY: function (pixel, noAnimation, animationSpeed, source) {
            if (noAnimation) {
                var e = { source: source || "custom", scrollData: this._vScrollbar ? this._vScrollbar._scrollData : null, scrollTop: pixel };
                if (this._trigger("scroll", e))
                    return;                
                pixel = e.scrollTop;
                this.scrollTop(pixel);
                this.content().scrollTop(pixel);
                if (!this._updateScroll && this._vScrollbar)
                    this._vScrollbar["scroll"](pixel, source, true);
                return;
            }
            if (ej.isNullOrUndefined(animationSpeed) || animationSpeed == "")
                animationSpeed = 100;
            this.content().stop().animate({
                scrollTop: pixel
            }, animationSpeed, 'linear');
        },

        scrollX: function (pixel, noAnimation, animationSpeed, source) {
            if (noAnimation) {
                if (this._trigger("scroll", { source: source || "custom", scrollData: this._hScrollbar ? this._hScrollbar._scrollData : null, scrollLeft: pixel }))
                    return;
                var browserName = ej.browserInfo().name;
                if (this.model.enableRTL && browserName != "mozilla") {
                    if (pixel < 0)
                        pixel = Math.abs(pixel);
                    var content = this.model.targetPane != null ? this.content().find(this.model.targetPane)[0] : this.content()[0];
                    if (browserName == "chrome" || browserName == "webkit")
                        pixel = content.scrollWidth - content.clientWidth - pixel;
                }
                this.scrollLeft(pixel);
                if (this.model.targetPane != null)
                    this.content().find(this.model.targetPane).scrollLeft(pixel);
                else
                    this.content().scrollLeft(pixel);
                if (!this._updateScroll && this._hScrollbar)
                    this._hScrollbar["scroll"](pixel, source, true);
                return;
            }
            if (ej.isNullOrUndefined(animationSpeed) || animationSpeed == "")
                animationSpeed = 100;
            this.content().stop().animate({
                scrollLeft: pixel
            }, animationSpeed, 'linear');
        },

        enable: function () {
            var scroller = this.element.find(".e-vscrollbar,.e-hscrollbar,.e-vscroll,.e-hscroll,.e-vhandle,.e-hhandle,.e-vscroll .e-icon,.e-hscroll .e-icon");
            if (scroller.hasClass("e-disable")) {
                scroller.removeClass("e-disable").attr({ "aria-disabled": false });
                this.model.enabled = true;
            }
            if (this._vScrollbar)
                this._vScrollbar._enabled = this.model.enabled;
            if (this._hScrollbar)
                this._hScrollbar._enabled = this.model.enabled;
        },

        disable: function () {
            var scroller = this.element.find(".e-vscrollbar,.e-hscrollbar,.e-vscroll,.e-hscroll,.e-vhandle,.e-hhandle,.e-vscroll .e-icon,.e-hscroll .e-icon");
            scroller.addClass("e-disable").attr({ "aria-disabled": true });
            this.model.enabled = false;
            if (this._vScrollbar)
                this._vScrollbar._enabled = this.model.enabled;
            if (this._hScrollbar)
                this._hScrollbar._enabled = this.model.enabled;
        },

        _changeTop: function (d, step, source) {
            var start = this.model.targetPane != null && d.dimension != "height" ? this.content().find(this.model.targetPane)[d.scrollVal]() : this.content()[d.scrollVal](), t;

            if (d.dimension == "height" && start == 0)
                start = this.scrollTop() != 0 ? this.scrollTop() : 0;
            t = start + step;
            if (!d.enableRTL ? t > d.scrollable : t < d.scrollable) t = d.scrollable;
            if (!d.enableRTL ? t < 0 : t > 0) t = 0;

            if (t !== start) {
                this["scroll" + d.xy](t, true, "", source);
                if (d.xy === "X" && !ej.isNullOrUndefined(this._hScrollbar))
                    this._hScrollbar["scroll"](t, source, true);
                else if (!ej.isNullOrUndefined(this._vScrollbar))
                    this._vScrollbar["scroll"](t, source, true);
            }

            return t !== start;
        },

        _mouseWheel: function (e) {
            if (!e.data || !this.model.enabled) return;
            var delta = 0, data = e.data.d, ori = e;
            e = e.originalEvent;
            if (this._wheelx != 1 && (e.wheelDeltaX == 0 || e.wheelDeltaY == 0))
                this._wheelx = 1;
            if (navigator.platform.indexOf("Mac") == 0 && (this._wheelx == 0)) {
                if (this._browser == "webkit" || this._browser == "chrome")
                    return true;
            }
            if (this._browser == "mozilla")
                e.axis == e.HORIZONTAL_AXIS ? data = this._scrollXdata : this._scrollYdata;
            else if (this._browser == "msie") {
                e.deltaX ? data = this._scrollXdata : this._scrollYdata;
                delta = e.deltaX / 120;
            }
            else if (this._wheelx && e.wheelDeltaX != 0 && e.wheelDeltaY == 0 && this._scrollXdata)
                data = this._scrollXdata;
            if (e.wheelDeltaX == 0) this._wheelx = e.wheelDeltaX;
            if (e.wheelDelta) {
                delta = navigator.platform.indexOf("Mac") == 0 ? -e.wheelDelta / 3 : -e.wheelDelta / 120;
                if (window.opera) {
                    if (parseFloat(window.opera.version, 10) < 10)
                        delta = -delta;
                }
            } else if (e.detail) delta = e.detail / 3;
            if (!delta) return;
            if (this._changeTop(data, delta * data.scrollOneStepBy, "wheel")) {
                e.preventDefault ? e.preventDefault() : ori.preventDefault();
                ori.stopImmediatePropagation();
                ori.stopPropagation();
            }
            else {
                this._trigger("scrollEnd", { originalEvent: e, scrollData: ori });
                this._wheelx = 0;
            }
        },
        _scrollerHover: function (e) {
            if (e.type == "mouseenter" && !this.content().siblings().is(":visible"))
                this.content().siblings().css("display", "block");
            else if (e.type == "mouseleave")
                this.content().siblings().hide();
        },

        _mouseUp: function (e) {
            if (!e.data) return;

            var d = e.data.d;

            if (e.type === "mouseup" || e.type === "touchend" || (!e.toElement && !e.relatedTarget)) {
                this._off($(document), "mousemove touchmove");
                $(document).off("mouseout mouseup touchend", ej.proxy(this._mouseUp, this));
                d.fromScroller = false;
                if (this._mouseMoved === true && e.data.source === "thumb" && !ej.isNullOrUndefined(this.model)) {
                    $.when(this.content()).done(ej.proxy(function () {
                        this._trigger("thumbEnd", { originalEvent: e, scrollData: d });
                    }, this));
                }
            }
            d.up = true;
        },

        _mouseDownOnContent: function (down) {
            if (!this.model.enabled) return;
            var d = down.data.d;
            if (this._trigger("thumbStart", { originalEvent: down, scrollData: d }))
                return;

            if (down.which == 3 && down.button == 2) return;
            d.fromScroller = true;

            var prevY = null, skip = 1, min = 5, direction;

            this._on($(document), "mousemove touchmove", (function (move) {
                move.preventDefault();
                var pageXY = move.type == "mousemove" ? move[d.clientXy] : move.originalEvent.changedTouches[0][d.clientXy];

                if (prevY && pageXY !== prevY) {
                    this._mouseMoved = true;
                    var diff = pageXY - prevY, sTop = this.model[d.scrollVal] - (diff * d.onePx / min);

                    if (skip == 1 && Math.abs(diff) > min) {
                        direction = d.position;
                        skip = 0;
                    }
                    if (skip == 0) prevY = pageXY;

                    if (sTop >= 0 && sTop <= d.scrollable && direction === d.position) {
                        this["scroll" + d.xy](sTop, true, "", "thumb");
                        if (d.xy === "X")
                            this._hScrollbar["scroll"](sTop, "thumb", true);
                        else if (!ej.isNullOrUndefined(this._vScrollbar))
                            this._vScrollbar["scroll"](sTop, "thumb", true);
                        this.content().css("cursor", "pointer");
                        this._trigger("thumbMove", { originalEvent: move, scrollData: d });
                    }
                }
                if (prevY == null) prevY = pageXY;
            }));
            this._mouseMoved = false;
            $(document).one("mouseup touchend", { d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
            $(document).mouseout({ d: d, source: "thumb" }, ej.proxy(this._mouseUp, this));
        },


        _scroll: function (e) {
            var dS = [this._vScrollbar ? this._vScrollbar._scrollData : null, this._hScrollbar ? this._hScrollbar._scrollData : null];

            for (var i = 0; i < 2; i++) {
                var d = dS[i];
                if (!d || d.skipChange) continue;

                if (this.model.targetPane != null && i == 1)
                    d.sTop = this.content().find(this.model.targetPane)[0][d.scrollVal];
                else
                    d.sTop = e.target[d.scrollVal];
                this[d.scrollVal](d.sTop);
                if (d.fromScroller) return;
                if (i === 1) {
                    var content = this.content()[0];
                    if (this._rtlScrollLeftValue && content.scrollWidth - content.clientWidth != this._rtlScrollLeftValue)
                        this._rtlScrollLeftValue = content.scrollWidth - content.clientWidth;
                    d.sTop = (ej.browserInfo().name != "mozilla" && this.model.enableRTL) ? (this._rtlScrollLeftValue == 0 ? (d.sTop * -1) : (d.sTop - this._rtlScrollLeftValue)) : d.sTop;
                    this._hScrollbar["scroll"](d.sTop, "", true);
                } else
                    this._vScrollbar["scroll"](d.sTop, "", true);
            }
        },

        _changevHandlerPosition: function (top) {
            var scrollbar = this._vScrollbar;
            if (scrollbar) {
                top = scrollbar._scrollData != null && top >= scrollbar._scrollData.scrollable ? scrollbar._scrollData.scrollable : top;
                if (scrollbar != null && top >= 0 && top <= scrollbar._scrollData.scrollable)
                    scrollbar[scrollbar._scrollData.handler].css(scrollbar._scrollData.lPosition, (top / scrollbar._scrollData.onePx) + "px");
            }
        },

        _changehHandlerPosition: function (left) {
            var scrollbar = this._hScrollbar;
            if (scrollbar) {
                left = scrollbar._scrollData != null && left >= scrollbar._scrollData.scrollable ? scrollbar._scrollData.scrollable : left;
                if (scrollbar != null && top >= 0 && left <= scrollbar._scrollData.scrollable)
                    scrollbar[scrollbar._scrollData.handler].css(scrollbar._scrollData.lPosition, (left / scrollbar._scrollData.onePx) + "px");
            }
        },

        _destroy: function () {
            this.element.css({ "width": "", "height": "" }).find(".e-vscrollbar,.e-hscrollbar").remove();
            this.content().removeClass("e-content").css({ "width": "", "height": "" });
            $(window).unbind('resize', $.proxy(this._resetScroller, this));
        }
    });
})(jQuery, Syncfusion, window);;
/**
* @fileOverview Plugin to style the Html NavigationDrawer elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejNavigationDrawerBase", "ej.NavigationDrawerBase", {

        defaults: {

            contentId: null,

            cssClass: "",

            direction: "left",

            targetId: null,

            position: "normal",

            enableListView: false,

            listViewSettings: {},

            type: "overlay",

            width: "auto",

            swipe: null,

            open: null,

            beforeClose: null,

            items: []
        },

        dataTypes: {
            direction: "enum",
            type: "enum"
        },

        _renderControl: function () {
            if (this.model.position.toLowerCase() == "fixed")
                this.element.appendTo(this._nbHome);
            if (!this.model.isPaneOpen) {
                this._elementOverlay = ej.buildTag("div#" + this._id + "_Overlay", {}, {}, { "class": this._rootCSS + " " + this._prefixClass + "nb-layout " + this._prefixClass + "nb-overlay" });
                this._elementShadow = ej.buildTag("div#" + this._id + "_shadow", {}, {}, { "class": this._rootCSS + " " + this._prefixClass + "nb-shadow " + "" + this._prefixClass + "nb-type-" + (this.model.type == "slide" ? "slide" : "overlay") });
                this._nbHome.append(this._elementOverlay);
                this._elementOverlay.hide();
                this.element.hide();
            }
            this.element.addClass(this._prefixClass + "user-select" + " " + this.model.cssClass);
            this.element.addClass(this._prefixClass + 'nb-layout ' + (this.model.direction.toLowerCase() == "left" ? this._prefixClass + 'nb-left' : this._prefixClass + 'nb-right'));
            this.element.addClass(this._prefixClass + 'nb-type-' + (this.model.type == "slide" ? "slide" : "overlay"));
            this.element.prepend(this._elementShadow);
            this._maxIndex = ej.getMaxZindex();
            this._parentWidth = this._nbHome.width();
            this._setWidth();
            this._setLayout();
        },

        _transform: function (dist, time, element) {
            var translate = "-" + this._browser + "-transform",
                translateVal = "translateX(" + dist + "px) translateZ(0px)";
            var trans = "-" + this._browser + "-transition-property",
            transVal = "transform",
            transDur = "-" + this._browser + "-transition-duration",
            transDurVal = time + "ms";
            if (element) {
                this.element.addClass("" + this._prefixClass + "nb-animate");
                this.element.css(trans, transVal).css(transDur, transDurVal).css(translate, translateVal);
            }
            else {
                this._nbHome.addClass(this._rootCSS + " " + this._prefixClass + "nb-animate");
                this._nbHome.css(trans, transVal).css(transDur, transDurVal).css(translate, translateVal);
            }
        },

        _show: function () {
            if (!Number(this.element.css("left").match((/\d/g))[0] < this.element.width())) this._setLayout();
            if ($("." + this._prefixClass + "nb-opened").length && $("." + this._prefixClass + "nb-opened")[0] != this.element[0])
                this._prefixClass == "e-m-" ? $("." + this._prefixClass + "nb-opened").ejmNavigationDrawer("close") : $("." + this._prefixClass + "nb-opened").ejNavigationDrawer("close");
            this.element.show();
            ej.listenTouchEvent((this._isTransitionElement() ? this.element : this._nbHome), ej.transitionEndEvent(), this._transitionOpenHandler, false, this);
            this.element.addClass("" + this._prefixClass + "nb-opened");
            this._elementOverlay.css("z-index", this._maxIndex * 2);
            this.element.css("z-index", this._maxIndex * 3);
            if (!(ej.browserInfo().name == "msie" && ej.browserInfo().version <= 9)) {
                this.model.direction == "left" ? this._transform($(this.element).width(), 400, this._isTransitionElement()) : this._transform(-($(this.element).width()), 400, this._isTransitionElement());
            }
            else {
                $(this.element).show().animate({ "left": (this.model.direction == "left" ? "0px" : (this._parentWidth - $(this.element).width())) }, 400, $.proxy(this._transitionOpenEnd, this));
            }
            this._data = { element: this.element };
            this.model.enableListView ? $.extend(this._data, { listview: this._lb }) : null;
            this._elementOverlay.show();
            this._trigger("open", this._data);
        },

        _hide: function () {
            this._elementOverlay.hide();
            ej.listenTouchEvent((this._isTransitionElement() ? this.element : this._nbHome), ej.transitionEndEvent(), this._transitionCloseHandler, false, this);
            this.element.removeClass("" + this._prefixClass + "nb-opened");
            if (!(ej.browserInfo().name == "msie" && ej.browserInfo().version <= 9))
                this._transform(0, 400, this._isTransitionElement());
            else
                $(this.element).animate({ "left": (this.model.direction == "left" ? -($(this.element).width()) : this._nbHome.width()) }, 400, $.proxy(this._transitionCloseEnd, this));
        },
        _clearHomeElement: function () {
            var browSpec = "";
            if (this._browser != "")
                browSpec = '-' + this._browser + '-';
            var translate = browSpec + "transform";
            var trans = browSpec + "transition-property";
            var transDur = browSpec + "transition-duration";
            this._nbHome.css(translate, "").css(transDur, "").css(trans, "");
            this._nbHome.removeClass(this._rootCSS + " " + this._prefixClass + "nb-animate");
        },

        _isOpened: function () {
            return this.element.hasClass("" + this._prefixClass + "nb-opened");
        },
        _isWeb: function () {
            return this._prefixClass == "e-" ? true : false;
        },

        _isOtherOpened: function () {
            return $("." + this._prefixClass + "nb-opened").length;
        },

        _isTransitionElement: function () {
            return this.model.type == "overlay" ? true : false;
        },

        _isOrientationSupported: function () {
            return ("orientation" in window && "onorientationchange" in window);
        },

        _setWidth: function () {
            $(this._nbHome).width() < 500 ? this.element.css("maxwidth", "80%") : this.element.css("maxwidth", "40%");
            if (this.model.width != "auto" && Number(this.model.width) != 0)
                this.element.width(Number(this.model.width));
            this.element.height("100%");
        },

        _setLayout: function () {
            this.element.show();
            if (this.element.is(":visible")) {
                if (this.model.position == "fixed")
                    this.element.css("min-height", window.innerHeight);
                if (this.model.direction.toLowerCase() == "left") {
                    if (!this.model.isPaneOpen) {
                        this.element.css({ left: (-(this.element.width())) });
                    }
                    else {
                        this.element.css("left", this.model.position == "fixed" ? -this.element.width() : 0);
                        this._nbHome.css({ "width": this._nbHome.parent().width() - this.element.width(), "left": this.element.width(), "position": "absolute" });
                    }
                }
                else {
                    if (!this.model.isPaneOpen) {
                        this.element.css({ left: this.model.position.toLowerCase() == "normal" ? this.element.width() : $(this._nbHome).width() });
                    }
                    else {
                        this._nbHome.css({ "width": this._nbHome.parent().width() - this.element.width() });
                        this.element.css("left", this._nbHome.width());
                    }
                }
            }
            if (!this._isOpened() && !this.model.isPaneOpen)
                this.element.hide();
        },

        _destroy: function () {
            this._wireEvents(true);
            if (this._elementOverlay)
                this._elementOverlay.remove();
            if (this._elementShadow)
                this._elementShadow.remove();
            if (this._elementWrapper) this.element.unwrap();
            this.element.removeAttr("style");
            this.element.css("left", "");
            this._destroyEJMPlugin();
        },

        _refresh: function () {
            this._destroy();
            this._load();
            this._renderControl();
            this._renderEJMControls();
            this._createDelegate();
            this._wireEvents();
        },


        _createDelegate: function () {
            //Swipe Events Handler
            this._swipeStartHandler = $.proxy(this._swipeStart, this);
            this._swipeEndHandler = $.proxy(this._swipeEnd, this);
            this._swipeMoveHandler = $.proxy(this._swipeMove, this);
            //Touch Event Handler
            this._touchMoveHandler = $.proxy(this._touchMove, this);
            this._touchEndHandler = $.proxy(this._touchEnd, this);
            //Transition Event
            this._transitionOpenHandler = $.proxy(this._transitionOpenEnd, this);
            this._transitionCloseHandler = $.proxy(this._transitionCloseEnd, this);
            //Target Touch Event Handler
            this._overlayTapHandler = $.proxy(this._elementOverlayTap, this);
            //TargetButton Tap
            this._targetButtonTapHandler = $.proxy(this._targetButtonTap, this);
            // Size Event Handler
            this._onOrientationChangeHandler = $.proxy(this._onOrientationChangeEvent, this);
            this._onResizeEventHandler = $.proxy(this._onResizeEvent, this);
        },

        _wireEvents: function (remove) {
            if (!this.model.isPaneOpen || remove) {
                //Event Binding
                if (this.model.targetId) {
                    if (ej.getCurrentPage)
                        var ele = ej.getCurrentPage();
                    else
                        var ele = $("body");
                    ej.listenTouchEvent(ele.find("#" + this.model.targetId), (this._prefixClass == "e-" ? "click" : ej.endEvent()), this._targetButtonTapHandler, remove, this);
                }
                else {
                    if (!(ej.browserInfo().name == "msie" && ej.browserInfo().version <= 8))
                        ej.listenTouchEvent(this._nbHome, ej.startEvent(), this._swipeStartHandler, remove, this);
                }
                //Ovelay Binding
                ej.listenTouchEvent(this._elementOverlay, ej.endEvent(), this._overlayTapHandler, remove, this);
            }
            //Resize Events
            if (ej.isTouchDevice() && this._isOrientationSupported())
                ej.listenTouchEvent(window, "orientationchange", this._onOrientationChangeHandler, remove, this);
            else
                ej.listenTouchEvent(window, "resize", this._onResizeEventHandler, remove, this);
        },

        _transitionCloseEnd: function () {
            this.element.hide();
            this._clearHomeElement();
            ej.listenTouchEvent((this._isTransitionElement() ? this.element : this._nbHome), ej.transitionEndEvent(), this._transitionCloseHandler, true, this);
        },
        _transitionOpenEnd: function () {
            this.element.show();
            if (this._scrollbar)
                this._scrollbar.ejmScrollPanel("refresh");
            if (this._lbscrollbar)
                this._lbscrollbar.ejmScrollPanel("refresh");
            ej.listenTouchEvent((this._isTransitionElement() ? this.element : this._nbHome), ej.transitionEndEvent(), this._transitionOpenHandler, true, this);
        },

        _targetButtonTap: function (e) {
            e.preventDefault();
            if (!this._isOtherOpened()) this._show();
        },
        _elementOverlayTap: function (e) {
            ej.blockDefaultActions(e);
            this._data = { element: this.element };
            this.model.enableListView ? $.extend(this._data, { listview: this._lb }) : undefined;
            this._trigger("beforeclose", this._data);
            this._hide();
        },

        _swipeStart: function (e) {
            var point = e.changedTouches ? e.changedTouches[0] : e;
            this._startSwipeOffsetX = point.clientX - this._nbHome.offset().left;
            this._startSwipeOffsetActualX = point.clientX;
            this._startSwipeOffsetY = point.clientY;
            this._containerX = this._isTransitionElement() ? this.element.offset().left : this._nbHome.offset().left;
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, false, this);
            ej.listenTouchEvent(this._nbHome, ej.endEvent(), this._swipeEndHandler, false, this);
        },

        _swipeMove: function (e) {
            var point = e.changedTouches ? e.changedTouches[0] : e;
            if (Number(this.element.css("left").match((/\d/g))[0]) < this.element.width()) this._setLayout();
            if (this._startSwipeOffsetActualX && this._startSwipeOffsetActualX > (point.clientX)) { this._swipeDirection = "right"; }
            else { this._swipeDirection = "left"; }
            var allowOpDirection = this._isOpened() ? (this._swipeDirection != this.model.direction.toLowerCase() ? true : false) : false;
            if (!allowOpDirection) {
                if (this._swipeDirection == this.model.direction.toLowerCase() && !this._isOtherOpened()) {
                    ej.listenEvents([this._nbHome, this._nbHome, this._nbHome], [ej.moveEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler], false, this);
                }
                else {
                    ej.listenEvents([this._nbHome, this._nbHome, this._nbHome], [ej.moveEvent(), ej.endEvent()], [this._touchMoveHandler, this._touchEndHandler], true, this);
                }
            }
            else {
                if (this._startSwipeOffsetX && this._isOpened()) {
                    this._relativeX = point.clientX - this._startSwipeOffsetActualX;
                    this._relativeY = point.clientY - this._startSwipeOffsetY;
                    var leftPoint = (this.element.width() + this._relativeX) - (this.model.type.toLowerCase() == "overlay" ? this._nbHome.offset().left : 0);
                    if ($(point.target).hasClass("" + this._prefixClass + "nb-overlay")) {
                        if (ej.getRenderMode && ej.getRenderMode() == "ios7") {
                            this._hide();
                            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, true, this);
                        }
                        else {
                            if (this.model.direction == "left" ? -(this._relativeX) < this.element.width() : this._relativeX < this.element.width())
                                this._transform((this.model.direction == "left" ? leftPoint : -((this.element.width() - this._relativeX))), 0, this._isTransitionElement());
                        }
                    }
                }
            }
        },

        _swipeEnd: function (e) {
            var point = e.changedTouches ? e.changedTouches[0] : e;
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, true, this);
            ej.listenTouchEvent(this._nbHome, ej.endEvent(), this._swipeEndHandler, true, this);
        },

        _touchMove: function (e) {
            var point = e.changedTouches ? e.changedTouches[0] : e;
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._swipeMoveHandler, true, this);
            if (this._isTransitionElement()) { this._containerX = 0; }
            if (this._startSwipeOffsetX && !this._isOpened()) {
                this._relativeX = point.clientX - this._startSwipeOffsetX;
                this._relativeY = point.clientY - this._startSwipeOffsetY;
                if ((this.model.direction == "left" ? this._startSwipeOffsetX < 50 : this._startSwipeOffsetX > this._nbHome.width() - 50) && !(ej.getRenderMode ? ej.getRenderMode() == "ios7" : false)) {
                    this.element.show();
                    var leftPoint = this.model.type.toLowerCase() == "slide" ? 0 : this._nbHome.offset().left;
                    if (this.model.direction == "left" ? this._relativeX - leftPoint < this.element.width() : this._relativeX - leftPoint > -(this.element.width())) {
                        !this._containerX || this.model.targetHome ? this._transform((this._containerX + this._relativeX) - leftPoint, 0, this._isTransitionElement()) : undefined;
                    }
                }
                else {
                    if (ej.getRenderMode && ej.getRenderMode() == "ios7") {
                        if (Math.abs(this._relativeX) > 50)
                            this._show();
                    }
                }
            }
        },

        _touchEnd: function (e) {
            var point = e.changedTouches ? e.changedTouches[0] : e;
            ej.listenTouchEvent(this._nbHome, ej.moveEvent(), this._touchMoveHandler, true, this);
            if (!this._isOpened() && !($.isFunction(ej.isIOS7) ? ej.isIOS7() : false)) {
                if (!this._containerX) {
                    this._relativeX = point.clientX - this._startSwipeOffsetX;
                    this._relativeY = point.clientY - this._startSwipeOffsetY;
                    if (this.model.direction == "left" ? this._startSwipeOffsetX < 50 : this._startSwipeOffsetX > this._nbHome.width() - 50) {
                        if (this._relativeX > 30 || this._relativeX < -30)
                            this._show();
                        else
                            this._hide();
                    }
                }
                else {
                    this._hide();
                }
            }
            this._data = { targetElement: this._nbHome, element: this.element, direction: this._swipeDirection };
            this.model.enableListView ? $.extend(this._data, { listview: this._lb }) : undefined;
            this._trigger("swipe", this._data);
            ej.listenTouchEvent(this._nbHome, ej.endEvent(), this._touchEndHandler, true, this);
            ej.listenTouchEvent(this._nbHome, "mouseleave", this._touchEndHandler, true, this);
        },
        _onOrientationChangeEvent: function (e) {
            var state;
            this.element.show();
            if (this._isOpened() && this.element.is(":visible")) { state = "opned"; } else { state = "clsed" }
            if (window.orientation == 0 || window.orientation == 180 || window.orientation == 90) { window.scrollTo(0, 0); }
            this._transform(0, 0, this._isTransitionElement());
            this.element.hide();
            this.element.removeClass("" + this._prefixClass + "nb-opened");
            this._parentWidth = this._nbHome.width();
            this._clearHomeElement();
            var proxy = this
            setTimeout(function () {
                proxy._setWidth();
                proxy._setLayout();
                if (state == "opned") { proxy._show(); }
            }, 300);
        },

        _onResizeEvent: function (e) {
            var state;
            this.element.show();
            if (this._isOpened() && this.element.is(":visible") && !this.model.isPaneOpen) { state = "opned"; } else { state = "clsed" }
            this._transform(0, 0, this._isTransitionElement());
            this.element.hide();
            this.element.removeClass("" + this._prefixClass + "nb-opened");
            this._clearHomeElement();
            this._parentWidth = this._nbHome.width();
            var proxy = this
            if (this._prefixClass == "e-m-" && this.model.renderMode == "windows")
                setTimeout(function () {
                    proxy._setWidth();
                    proxy._setLayout();
                    if (state == "opned") { proxy._show(); }
                }, 300);
            else {
                proxy._setWidth();
                proxy._setLayout();
                if (state == "opned") { proxy._show(); }
            }
        },

        open: function (e) {
            if (!this.model.isPaneOpen) {
                this._show();
            }
        },

        close: function (e) {
            if (!this.model.isPaneOpen) {
                this._data = { element: this.element };
                this.model.enableListView ? $.extend(this._data, { listview: this._lb }) : undefined;
                this._trigger("beforeClose", this._data);
                this._hide();
            }
        },

        toggle: function (e) {
            if (!this.model.isPaneOpen) {
                this._isOpened() ? this.close() : this.open();
            }
        }

    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ListView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejListViewBase", "ej.ListViewBase", {

        defaults: {
            
            height: null,

            width: null,

            selectedItemIndex: -1,

            enableGroupList: false,
            
            enableAjax: false,
            
            enableCache: false,
            
            enablePersistence: false,
            
            load: null,
            
            loadComplete: null,
            
            ajaxBeforeLoad: null,

            ajaxSuccess: null,

            ajaxError: null,

            ajaxComplete: null,

            ajaxSettings: {
                type: 'GET',
                cache: false,
                async: true,
                dataType: "html",
                contentType: "html",
                url: "",
                data: {}
            },
                                    
            renderTemplate: false,

            templateId: null,

            persistSelection: false,

            preventSelection: false,

            dataSource: [],

            query: null,

            showHeader: false,

            cssClass: "",

            headerTitle: "Title",

            headerBackButtonText: null,

            enableFiltering: false,

            enableCheckMark: false

        },

        observables: ["selectedItemIndex", "dataSource"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),
        dataSource: ej.util.valueFunction("dataSource"),

        _updateModelItems: function () {
            this.model.items = eval(this.model.items);
            var ang_attr = this.model.items;
            if (ang_attr.length) {
                var ul = ej.buildTag("ul.e-m-clearall");
                for (i = 0; i < ang_attr.length; i++) {
                    ang_attr[i].items = [];
                    var ang_li = ej.buildTag("li");
                    if (this.model.items[i].template)
                        ang_li.html(this.model.items[i].template);
                    if (!this.model.items[i].childId && this.model.items[i].href)
                        this.model.items[i].childId = ("page_" + parseInt(Math.random().toFixed(3) * 1000));
                    if (this.model.items[i].renderTemplate) {
                        if (!this._storedTemplate[i]) {
                            if (this.model.items[i].templateId) {
                                var ele = this._tempContent.find('#' + this.model.items[i].templateId).remove();
                                this._storedTemplate[i] = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                            }
                            else
                                this._storedTemplate[i] = this.model.items[i].template;
                            this.model.items[i].templateId = this._storedTemplate[i];
                        }
                    }
                    if (this.model.fieldSettings)
                        this.model.fieldSettings = $.extend(this.defaults.fieldSettings, this.model.fieldSettings);
                    ul.append(ang_li);
                }
                this.element.append(ul)
            }
            else {
                var ul = this.element.find(">ul");
                var groupid = 1;
                for (ulindex = 0; ulindex < ul.length; ulindex++) {
                    this._listitems = $(ul[ulindex]).find(">li");
                    for (index = 0; index < this._listitems.length; index++) {
                        var element = this._listitems[index];
                        if ((ej.getAttrVal(element, "data-ej-primarykey") == null) && ($(element).find("ul").length)) {
                            var primaryKey = Math.round(Math.random() * 100);
                            $(element).attr("data-ej-primarykey", primaryKey);
                        }
                        else
                            primaryKey = "";
                        var groupTitle = ej.getAttrVal($(ul[ulindex]), "data-ej-grouplistitle") ? ej.getAttrVal($(ul[ulindex]), "data-ej-grouplistitle") : "GroupList" + groupid;
                        this.model.items.push(this._itemsObjectCollection($(this._listitems[index]), primaryKey, null, groupTitle));
                        this._nestedListitems = $(this._listitems[index]).find("ul >li");
                        for (index1 = 0; index1 < this._nestedListitems.length; index1++) {
                            var element1 = this._nestedListitems[index1];
                            if ((ej.getAttrVal(element1, "data-ej-primarykey") == null) && ($(element1).find("ul").length)) {
                                var primaryKey = Math.round(Math.random() * 100);
                                $(element1).attr("data-ej-primarykey", primaryKey);
                            }
                            else
                                primaryKey = "";
                            var parentPrimaryKey = ej.getAttrVal($($(element1).parent()).closest("li"), "data-ej-primarykey");
                            this.model.items.push(this._itemsObjectCollection($(this._nestedListitems[index1]), primaryKey, parentPrimaryKey, groupTitle));
                        }
                    }
                    groupid++;
                }
            }
        },

        _load: function () {
            this._orgEle = this.element.clone();
            this._index = 0;
            this._items = [];
            this.model.id = this.element[0].id;
            if (this.model.load)
                this._trigger("load");
            this.model.fieldSettings = eval(this.model.fieldSettings);
            if (this.model.fieldSettings && this.model.query != null) {
                this._dataUrl = this.model.dataSource;
                var proxy = this;
                if (proxy._prefixClass == "e-m-")
                    App.waitingPopUp.show();
                this.dataSource().executeQuery(eval(this.model.query)).done(function (e) {
                    proxy.model.dataSource = e.result;
                    proxy._renderControl();
                    if (proxy._prefixClass == "e-m-")
                        App.waitingPopUp.hide();
                });
            }
            else
                this._renderControl();
        },
        
        
        _renderControl: function () {
            this.element.addClass(this._prefixClass + "parentlv " + this.model.cssClass);
            this._lbEle = ej.buildTag("div." + this._rootCSS);
            this._lbEle.addClass("subpage");
            if (this._prefixClass == "e-m-") {
                this._lbEle.addClass("e-m-" + this.model.renderMode + " e-m-" + this.model.theme);
                if (this.model.ios7.inline)
                    this._lbEle.addClass("e-m-inline")
            }
            this._lContainer = ej.buildTag("div." + this._prefixClass + "list-container#" + this.model.id + "_container", "<div></div>");
            if (this._prefixClass == "e-m-" && this.model.enablePullToRefresh) {
                this._refreshContainer = ej.buildTag("div." + this._prefixClass + "ptr-container");
                this._ptrImgContainer = ej.buildTag("span." + this._prefixClass + "image-container " + this._prefixClass + "pull");
                this._textContainer = ej.buildTag("span." + this._prefixClass + "text-container");
                this._refreshContainer.append(this._ptrImgContainer).append(this._textContainer);
                this._lContainer.find("div:first").append(this._refreshContainer);
                this._textContainer.html(this.model.pullToRefreshSettings.pullText)
            }
            var ul = this.element.find(">ul");
            if (this.model.showHeader) {
                var hdr = this._renderHeader(this.model.id, this.model.showHeaderBackButton, this.model.headerTitle, this.model.headerBackButtonText);
                this._lbEle.prepend(hdr);
            }
            if (this._prefixClass == "e-m-")
                if (this.model.allowScrolling)
                    this._renderScrollPanel(this._lbEle, this.model.id + "_scroll", this.model.id + "_container");
            if (this._hasDataSource() && !this.model.renderTemplate)
                this.element.empty();
            if (this.model.renderTemplate) {
                if (this.model.templateId) {
                    if (this._tempContent.find('#' + this.model.templateId).length) {
                        this._template = this._tempContent.find('#' + this.model.templateId);
                        ej.destroyWidgets(this._template);
                        if (this._template[0].nodeName.toLowerCase() != "script")
                            this._template.remove();
                        this._template = this._template[0].nodeName && this._template[0].nodeName.toLowerCase() == "script" ? ej.getClearString(this._template[0].innerHTML) : this._template[0].outerHTML;
                    }
                }
                else {
                    ej.destroyWidgets(this.element);
                    this._template = this.element.html();
                    this.element.empty();
                }
            }
            if (this.model.renderTemplate && !this._hasDataSource() && !this.model.enableGroupList) {
                this.element.append(this._lbEle.append(this._lContainer));
                if (this._template) {
                    this._lContainer.append(this._template);
                    if (ej.widget.init)
                        ej.widget.init(this._lContainer);
                }
            }
            else if (!this.model.renderTemplate || this.model.renderTemplate && this._hasDataSource() || this.model.renderTemplate && this.model.enableGroupList) {
                this._model_index = 0;
                if (this.model.enableGroupList) {
                    this._lContainer.addClass(this._prefixClass + "grouped");
                    var innerDiv = ej.buildTag("div." + this._prefixClass + "grouplist");
                    ul = this.element.children();
                    if (ul.length || this.dataSource().length) {
                        if (this.dataSource().length)
                            var group = ej.DataManager(this.dataSource()).executeLocal(ej.Query().from(this.dataSource()).group(this.model.fieldSettings['groupID']));
                        else
                            var group = ej.DataManager(this.model.items).executeLocal(ej.Query().from(this.model.items).group(this.model.fieldSettings['groupID']));
                        var length = group.length;
                        var title;
                        for (var i = 0; i < length; i++) {
                            this._items = group[i].items;
                            if (this._hasDataSource()) {
                                ulItem = ej.buildTag('ul', "", {}, { "data-ej-grouplisttitle": group[i].key });
                                this._lbEle.append(ulItem);
                            }
                            else
                                ulItem = ul[i];
                            title = this._hasDataSource() ? group[i].key : ej.getAttrVal($(ulItem), 'data-ej-grouplisttitle', "GroupList" + (i + 1));
                            $(ulItem).attr("data-ej-grouplisttitle", title);
                            var groupdiv = ej.buildTag("div." + this._prefixClass + "groupdiv", ej.buildTag("div." + this._prefixClass + "grouptitle", title));
                            innerDiv.append(groupdiv.append(this._renderListItems($(ulItem).addClass(this._prefixClass + "grouped"))));
                        }
                    }
                    else if (this._template) {
                        ej.destroyWidgets(this._template);
                        innerDiv[0].innerHTML = this._template;
                    }
                    this._prefixClass == "e-m-" ? this._lContainer.find("div:first").append(innerDiv) : this._lContainer.append(innerDiv);
                    if (ej.widget.init)
                        ej.widget.init(innerDiv);
                }
                else {
                    if (this._hasDataSource()) {
                        var ul = ej.buildTag('ul');
                        this._items = this.dataSource();
                        this._items = eval(this._items);
                        for (j = 0; j < this._items.length; j++) {
                            if (this._items[j].href && !this._items[j].childId) {
                                this._items[j].childId = ("page_" + parseInt(Math.random().toFixed(3) * 1000));
                            }
                        }
                    }
                    if (ul.length) {
                        ej.destroyWidgets(ul);
                        this._prefixClass == "e-m-" ? this._lContainer.find("div:first").append(this._renderListItems(ul)) : this._lContainer.append(this._renderListItems(ul));
                    }
                    else if (this._template) {
                        ej.destroyWidgets(this._template);
                        this._lContainer[0].innerHTML = this._template;
                    }
                }
                this.element.prepend(this._lbEle);
                if (this.model.enableFiltering)
                    this._createFilterWrapper(this._lbEle);
                this._lbEle.append(this._lContainer);
                if (ej.widget.init)
                    ej.widget.init(this._lContainer);
                if (this._prefixClass == "e-m-" && (App.angularAppName || ej.angular.defaultAppName))
                    ej.angular.compile(this._lContainer)
                var eLi = this.element.find('li.' + this._prefixClass + 'list');
                eLi.removeEleEmptyAttrs();
                eLi.find('.' + this._prefixClass + 'list-anchor').removeEleEmptyAttrs();
                if (ej.widget.init)
                    ej.widget.init(this._lbEle);
                var ulItems = this.element.find('ul');
                ulItems.find('li:first').addClass(this._prefixClass + "firstitem");
                ulItems.find('li:last').addClass(this._prefixClass + "lastitem");
                this._liEl = this.element.find("li");
                if (this.selectedItemIndex() >= 0 && !this.model.preventSelection && this.model.persistSelection) {
                    if (!ej.getBooleanVal(ulItems[this.selectedItemIndex()], 'data-preventselection', this.model.preventSelection) && ej.getBooleanVal(ulItems[this.selectedItemIndex()], 'data-persistselection', this.model.persistSelection)) {
                        this._currentItem = $(this._liEl[this.selectedItemIndex()]);
                        this._prevItem = this._currentItem;
                        this._currentItem.removeClass(this._prefixClass + "state-default").addClass(this._prefixClass + "state-active");
                    }
                }

                $(this.element.find('.' + this._prefixClass + 'lv-check').parent()).addClass(this._prefixClass + 'lv-checkdiv');
                $(this.element.find('.' + this._prefixClass + 'lv-check').closest('li.' + this._prefixClass + 'list')).addClass(this._prefixClass + 'list-check');
                $(this.element.find('.' + this._prefixClass + 'lv-input').closest('.' + this._prefixClass + 'lv-filter')).addClass(this._prefixClass + 'lv-inputdiv');
                this._wireEvents();
            }
            if (this._prefixClass == "e-m-" && (App.angularAppName || ej.angular.defaultAppName))
                ej.angular.compile(this._lbEle);
            this._setHeightWidth();
            if (this._prefixClass == "e-m-")
                this.element.find("li[data-href]").addClass("e-m-arrow").find("a.e-m-list-anchor").addClass("e-m-fontimage");
            if (this.model.loadComplete)
                this._trigger("loadComplete");
        },

        
        
        _createFilterWrapper: function (element) {
            var fDiv = this._createFilterDiv();
            var fAnchor = ej.buildTag("a." + this._prefixClass + "lv-anchor", ej.buildTag('span.' + this._prefixClass + 'input-btn'), {}, { "Title": "Clear text", "data-role": "none" });
            $(fDiv).append(fAnchor);
            element.append(fDiv);
        },
        
        
        _renderListItems: function (ul) {
            ul.addClass(this._prefixClass + "list-hdr " + this._prefixClass + "clearall");
            this._liItems = ul.find("li");
            if (this._liItems.length && !this._hasDataSource() || this._hasDataSource()) {
                this._renderParentChildTemplate();
                if (!this._hasDataSource() && !this.model.enableGroupList) {
                    this._items = this.model.items;
                }
                var items = this._items;
                var result = this._filterParentChild(items);
                if (result.child.length)
                    this._childRendering(result.child);
                if (result.parent) {
                    ul.empty().html($(this.jsRender).render(result.parent));
                    var ulItems = ul.find('>li');
                    for (var j = 0; j < ulItems.length; j++) {
                        if (result.parent[j] && result.parent[j].attributes)
                            $(ulItems[j]).addEleAttrs(result.parent[j].attributes);
                    }
                }
            }
            if (this._initEJCheckBox)
                this._initEJCheckBox(ul);
            return ul;
        },

        _filterParentChild: function (items) {
            var dataMgr = ej.DataManager(items);
            var childItem = dataMgr.executeLocal(ej.Query().from(items).where(ej.Predicate(this.model.fieldSettings['parentPrimaryKey'], ej.FilterOperators.notEqual, null)).group(this.model.fieldSettings['parentPrimaryKey']));
            var parentItem = dataMgr.executeLocal(ej.Query().from(items).where(ej.Predicate(this.model.fieldSettings['parentPrimaryKey'], ej.FilterOperators.equal, null)));
            return { parent: parentItem, child: childItem };
        },

        _childRendering: function (grouped) {
            var proxy = this;
            if (grouped.length) {
                proxy.element.append($(proxy.jsChildRender).render(grouped));
                $.each(grouped, function (index, element) {
                    if (ej.widget.init)
                        ej.widget.init(proxy.element.find('#child' + element.key));
                    var ul = proxy.element.find('#child' + element.key).find('ul');
                    var li = ul.find('li');
                    k = 0;
                    for (var j = 0; j < li.length; j++) {
                        if (grouped[k].items[j] && grouped[k].items[j].attributes)
                            $(li[j]).addEleAttrs(grouped[k].items[j].attributes);
                    }
                    k++;
                });
            }
        },

        
        
        _renderParentChildTemplate: function () {
            var list = this._renderLists();
            $.views.helpers({ _checkAjaxUrls: this._checkAjaxUrl, _checkImgUrls: this._checkImgUrl, _checkIsChecked: this._checkIsCheck, Object: this, ej: ej });
            //Template for parent item
            this.jsRender = ej.buildTag("script#" + this.model.id + "_Template", "", {}, { "type": "text/x-jsrender" });
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 10)
                this.jsRender[0].text = list[0].outerHTML.replace(/&gt;/g, '>');
            else
                this.jsRender[0].text = list[0].outerHTML.replace(/&gt;/g, '>');
            //Template for child item
            this.jsChildRender = ej.buildTag("script#" + this.model.id + "_ChildTemplate", "", {}, { "type": "text/x-jsrender" });
            var ul = ej.buildTag("ul." + this._prefixClass + "childcontainer " + this._prefixClass + "list-hdr " + this._prefixClass + "clearall");
            list.addClass(this._prefixClass + "childli");
            ul[0].innerHTML = "{{for items}}" + list[0].outerHTML + "{{/for}}";
            var div = ej.buildTag("div." + this._rootCSS + " subpage " + this._prefixClass + "childitem", "", {}, { "id": "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}", "style": "display:none" });
            if (this._prefixClass == "e-m-")
                div.addClass("e-m-" + this.model.renderMode + " e-m-" + this.model.theme);
            var innerdiv = ej.buildTag("div." + this._prefixClass + "list-container", "", {}, { "id": "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}_container" });
            if (this.model.showHeader)
                div.append(this._renderHeader("{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}", true, "Title"));
            if (this.model.enableFiltering)
                this._createFilterWrapper(div);
            div.append(innerdiv.append(ul));
            if (this._prefixClass == "e-m-")
                if (this.model.allowScrolling)
                    this._renderScrollPanel(div, "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}_scroll", "{{if key}}child{{>key}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}_container");
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 10)
                this.jsChildRender[0].text = div[0].outerHTML.replace(/&gt;/g, '>');
            else
                this.jsChildRender[0].innerHTML = div[0].outerHTML.replace(/&gt;/g, '>');
        },
        
        
        _renderChild: function (childId, ignoreWidth) {
            this._currentItem.attr('data-childid', childId);
            if (!this._currentItem.attr('data-childtitle'))
                this._currentItem.attr('data-childtitle', this._currentItem.text());
            this._createListDiv(childId);
            this._div.hide();
            this._container = ej.buildTag("div." + this._prefixClass + "list-container#" + childId + "_container");
            this.element.append(this._div);
            if (this.model.allowScrolling)
                this._renderScrollPanel(this._div, childId + "_scroll", childId + "_container", true);
        },

        _getText: function (element) {
            return $(element).clone()
           .children()
           .remove()
           .end()
           .text();
        },

        
        _checkImgUrl: function (item) {
            var obj = this.getRsc("helpers", "Object");
            var img = this.data[obj.model.fieldSettings.imageUrl];
            if (!$.support.pushstate)
                img = typeof App == "object" ? App.route.makeUrlAbsolute(img, true) : img;
            return "<img src = " + img + " class='" + obj._prefixClass + "list-img " + obj._prefixClass + "rel " + obj._prefixClass + "user-select'/>";
        },

        
        
        _checkAjaxUrl: function () {
            var href = this.data.href;
            var childid = this.data.childId;
            var rendertemp = this.data.renderTemplate;
            var tempid = this.data.templateId;
            var listObj = this.getRsc("helpers", "Object");
            var ej = this.getRsc("helpers", "ej");
            var content = listObj._currentPage(listObj);
            if (href && href.indexOf("#") != -1 && href != "#") {
                if (!listObj._storedContent[childid]) {
                    content.find(href).show();
                    var ele = content.find(href).clone();
                    content.find(href).hide();
                    listObj._storedContent[childid] = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                }
            }
            else {
                if (listObj._storedContent[this.index])
                    listObj._storedContent = ej._pushValue(listObj._storedContent, "", this.index);
            }

        },

        _currentPage: function (obj) {
            return obj._prefixClass == "e-m-" ? ej.getCurrentPage() : $("body");
        },
        
        _checkIsCheck: function (item) {
            return this.data[this.getRsc("helpers", "Object").model.fieldSettings.checked] ? true : false;
        },

        _onTouchStartHandler: function (evt) {
            if (!ej.isDevice() && !ej._preventDefaultException(evt.target, this._preventDefaultException))
                evt.preventDefault && evt.preventDefault();
            if (ej.isWindows && ej.isWindows())
                ej._touchStartPoints(evt, this);
            this._currentItem = $(evt.currentTarget);
            this._scroll = false;
            if (!ej.getBooleanVal(this._currentItem, 'data-preventSelection', this.model.preventSelection))
                this._addSelection();
            if (this.model.renderMode == "windows" && !this.model.windows.preventSkew)
                this._currentItem.addClass(ej.isMobile() ? ej._getSkewClass($(this._currentItem), evt.pageX, evt.pageY) : this._prefixClass + "m-skew-center");
            this._triggerStartEvent(this._returnData());
            ej.listenEvents([this._liEl, (this._prefixClass == "e-m-" && this.model.enablePullToRefresh) ? this.element : this._liEl, this._liEl],
                           [ej.endEvent(), ej.moveEvent(), ej.cancelEvent()],
                           [this._touchEndDelegate, this._touchMoveDelegate, this._touchMoveDelegate], false, this);
            $(window).bind(ej.endEvent() + " MSPointerUp pointerup", this._docClickDelegate);
        },
        
        
        _onTouchMoveHandler: function (evt) {
            this._isMoved = true;
            if (this._prefixClass == "e-m-" && this.model.enablePullToRefresh && !this._isRefreshing) {
                this._listScroll = this.element.find(".e-m-scroll");
                var scrollPos = this._listScroll.ejmScrollPanel("getScrollPosition").y;
                var moved = evt.changedTouches ? evt.changedTouches[0].pageY : evt.pageY;
                if (scrollPos > 0) {
                    if (this.model.refreshThreshold < moved) {
                        this._textContainer.empty().html(this.model.pullToRefreshSettings.releaseText);
                        this._ptrImgContainer.removeClass("rotatereverse").addClass("rotateimage");
                        this._isCrossed = true;
                    }
                    else {
                        if (this._ptrImgContainer.hasClass("rotateimage"))
                            this._ptrImgContainer.removeClass("rotateimage").addClass("rotatereverse");
                        this._textContainer.html(this.model.pullToRefreshSettings.pullText);
                        this._isCrossed = false;
                    }
                }
            }
            else if (!ej.isWindows || (ej.isWindows && !ej.isWindows()) || (ej.isWindows && ej.isWindows() && ej._isTouchMoved(evt, this))) {
                this._scroll = true;
                if (!ej.getBooleanVal(this._currentItem, 'data-persistSelection', this.model.persistSelection))
                    this._removeSelection();
                else if (this._prevItem && this._prevItem[0] != this._currentItem[0])
                    this._removeSelection();
                if (this.model.renderMode == "windows" && !this.model.windows.preventSkew)
                    ej._removeSkewClass(this._currentItem);
                else if (this.model.renderMode != "windows" && !ej.getBooleanVal(this._currentItem, 'data-preventselection', this.model.preventSelection) && this._prevItem && ej.getBooleanVal(this._currentItem, 'data-persistSelection', this.model.persistSelection))
                    this._prevItem.removeClass(this._prefixClass + "state-default").addClass(this._prefixClass + "state-active");
            }
        },
        
        

        _onTouchEndHandler: function (evt) {
            this._isFromAjax = false;
            if (!ej.getBooleanVal(this._currentItem, 'data-persistSelection', this.model.persistSelection))
                this._removeSelection();
            if (this.model.renderMode == "windows" && !this.model.windows.preventSkew)
                ej._removeSkewClass(this._currentItem);
            if (this._scroll) {
                this._setCurrent();
                this._unbindEvents(evt);
                return false;
            }
            else if (!this._scroll) {
                if (this._currentItem.find('.' + this._prefixClass + 'lv-check').length) {
                    var lbCheck = this._currentItem.find('.' + this._prefixClass + 'lv-check');
                    if (this._prefixClass == "e-m-" || (this._prefixClass == "e-" && !$(evt.target).closest('.e-lv-checkdiv').length))
                        this._toggleCheckboxValue(lbCheck);
                }
                if (this._prefixClass == "e-m-")
                    var backButton = this._currentItem.attr('data-childheaderbackbuttontext') == undefined ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header').length ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header').ejmHeader('getTitle') : "Back" : ej.getAttrVal(this._currentItem, 'data-childheaderbackbuttontext');
                else
                    var backButton = this._currentItem.attr('data-childheaderbackbuttontext') == undefined ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header').length ? this._currentItem.closest('.' + this._rootCSS + '').find('.' + this._prefixClass + 'header .' + this._prefixClass + 'htitle').text() : "Back" : ej.getAttrVal(this._currentItem, 'data-childheaderbackbuttontext');
                var urlVal = this._currentItem.attr('data-href');
                if (!this._currentItem.attr('data-navigateUrl')) {
                    var page = this._isInsideNavigation ? this._tempContent.find("[data-ajaxurl='" + this._convertToRelativeUrl(urlVal) + "']") : this.element.find("[data-ajaxurl='" + this._convertToRelativeUrl(urlVal) + "']");
                    if (ej.getBooleanVal(this._currentItem, 'data-loadajax', (this.model.enableAjax && typeof (urlVal) != "undefined" || !typeof (urlVal))) && (!this.model.enableCache || page.length == 0)) {
                        if (page.length)
                            page.remove();
                        if ((this._prefixClass == "e-m-" && this._currentItem.hasClass("e-m-arrow"))|| this._prefixClass != "e-m-")
                                this.loadAjaxContent(urlVal, backButton);
                        this._unbindEvents(evt);
                        if (this._isInsideNavigation && this._nearestND.model.contentId)
                            this._closeNavigation();
                        return;
                    }
                    else if (urlVal && urlVal.indexOf("#") != -1) {
                        if (this._currentPage(this).find('#' + urlVal.replace('#', '')).length) {
                            this._renderChild(ej.getAttrVal(this._currentItem, 'data-childid', "page_" + parseInt(Math.random().toFixed(3) * 1000)));
                            var content = ej.buildTag("div." + this._prefixClass + "content", this._storedContent[this._currentItem.attr('data-childid')]);
                            this._div.append(this._container.append(content));
                            if (ej.widget.init)
                                ej.widget.init(this._div);
                        }
                    }
                    this._updateContent(this._currentItem, backButton);
                    if (this._isInsideNavigation)
                        var close = this._nearestND.model.contentId ? (!this._currentItem.attr("data-childid") || this._currentItem.attr("data-href")) : !(this._currentItem.attr("data-childid") || this._currentItem.attr("data-href"));
                    if (this._isInsideNavigation && close)
                        this._closeNavigation();
                    this._touchEndEventHandler();
                }
                else
                    this._touchEndEventHandler();
                this._prevItem = this._currentItem;
            }
            else if (this.model.renderMode == "windows" && !ej.isMobile())
                this._touchEndEventHandler();
            this._unbindEvents(evt);
        },

        _generateData: function (items, elementId) {
            var collection = ej.DataManager(items).executeLocal(ej.Query().from(this.model.dataSource).where(ej.Predicate(typeof ej.getAttrVal(this._currentItem, "data-id") == "undefined" ? this.model.fieldSettings['text'] : this.model.fieldSettings['id'], ej.FilterOperators.equal, elementId)).group(elementId))[0];
            return collection ? collection.items[0] : [];
        },


        
        _closeNavigation: function () {
            if (this._prefixClass == "e-m-")
                this.element.closest('.' + this._prefixClass + 'nb').ejmNavigationDrawer('close');
            else
                this.element.closest('.' + this._prefixClass + 'nb').ejNavigationDrawer('close');
        },

        
        _setCurrent: function (e) {
            if (this._prevItem && ej.getBooleanVal(this._currentItem, 'data-persistSelection', this.model.persistSelection)) {
                this._currentItem = this._prevItem;
                this._currentItem.removeClass(this._prefixClass + "state-default").addClass(this._prefixClass + "state-active");
            }
        },
        
        _unbindEvents: function (e) {
            if (e && e.target.nodeName && e.target.nodeName.toLowerCase() != "a") {
                
                this._scroll = false;
            }
            ej.listenEvents([this._liEl, this._liEl],
                           [ej.endEvent(), ej.moveEvent(), ej.cancelEvent()],
                           [this._touchEndDelegate, this._touchMoveDelegate, this._touchMoveDelegate], true, this);
        },
        
        
        _addSelection: function () {
            if (!this._scroll) {
                this._currentItem.closest('.' + this._prefixClass + 'list-container').find('.' + this._prefixClass + 'state-active').removeClass(this._prefixClass + "state-active").addClass(this._prefixClass + "state-default");
                this._currentItem.removeClass(this._prefixClass + "state-default").addClass(this._prefixClass + "state-active");
            }
        },
        
        
        _removeSelection: function () {
            var proxy = this;
            proxy._currentItem.removeClass(this._prefixClass + "state-active").addClass(this._prefixClass + "state-default");
        },

        
        _setHeightWidth: function () {
            if (this.model.autoAdjustHeight)
                this.element.height(window.innerHeight);
            else if (this.model.height)
                this.element.height(this.model.height);
            else {
                var hgt = this.element[0].scrollHeight ? this.element[0].scrollHeight : ej.getDimension(this._lbEle.find('.' + this._prefixClass + 'list-container'), 'outerHeight');
                if (this.model.showHeader)
                    hgt = hgt + ej.getDimension(this._lbEle.find('.' + this._prefixClass + 'header'), 'outerHeight');
                if (this.model.enableFiltering)
                    hgt = hgt + ej.getDimension(this._lbEle.find('.' + this._prefixClass + 'lv-inputdiv'), 'outerHeight');
                this.element.height(hgt);
            }
            if (this.model.width)
                this.element.width(this.model.width);
        },
        
        
        _touchEndEventHandler: function () {
            this._triggerEndEvent(this._returnData());
        },
                
        _docClick: function (evt) {
            if (this._isMoved && this._prefixClass == "e-m-" && this.model.enablePullToRefresh && this._isCrossed && !this._isRefreshing) {
                ej.listenEvents([this.element],
                         [ej.moveEvent()],
                         [this._touchMoveDelegate], true, this);
                this._refreshContainer.addClass("refreshing");
                this._ptrImgContainer.removeClass(this._prefixClass + "pull").addClass(this._prefixClass + "loading");
                this._textContainer.html(this.model.pullToRefreshSettings.refreshText);
                this._updateDataSource();
                this._ptrImgContainer.removeClass("rotateimage");
                this._isCrossed = false;
            }
            if (this._scroll) {
                this._setCurrent();
                $(document).unbind(ej.endEvent() + " MSPointerUp pointerup", this._docClickDelegate);
                ej.listenEvents([this._liEl, this._liEl],
                               [ej.endEvent(), ej.moveEvent()],
                               [this._touchEndDelegate, this._touchMoveDelegate], true, this);
                $(window).unbind(ej.endEvent() + " MSPointerUp pointerup", this._docClickDelegate);
            }
        },
        
        
        _popStateNavigation: function (evt, data) {
            if (data.pageUrl && this.model.enableFiltering)
                this._initializeFiltering($("div[data-url='" + data.pageUrl.replace('#', '') + "']"));
        },
        
        
        _anchorClickHandler: function (e) {
            if (this._scroll) {
                ej.blockDefaultActions(e);
                return false;
            }
        },
        
        _onResize: function () {
            var proxy = this;
            setTimeout(function () {
                proxy._setHeightWidth();
            }, ej.isAndroid() ? 200 : 0);
        },
        _onScrollStop: function (e) {
            ej.blockDefaultActions(e);
        },
        
        
        _createDelegates: function () {
            this._anchorClickDelegate = $.proxy(this._anchorClickHandler, this);
            this._keyup = $.proxy(this._onKeyUp, this);
            this._touchStartDelegate = $.proxy(this._onTouchStartHandler, this);
            this._touchEndDelegate = $.proxy(this._onTouchEndHandler, this);
            this._touchMoveDelegate = $.proxy(this._onTouchMoveHandler, this);
            this._resizeDelegate = $.proxy(this._onResize, this);
            this._popStateDelegate = $.proxy(this._popStateNavigation, this);
            this._docClickDelegate = $.proxy(this._docClick, this);
        },
        
        
        _wireEvents: function (remove, items) {
            var listItems = this._liEl || items;
            if (listItems) {
                this._createDelegates();
                var eventType = remove ? "unbind" : "bind";
                if (this.model.autoAdjustHeight) {
                    var evt = !ej.isDevice() && "onorientationchange" in window ? "orientationchange" : "resize";
                    ej.listenEvents([window], [evt], [this._resizeDelegate], remove, this);
                }
                ej.listenEvents([listItems.find('a'), this.element.find('.' + this._prefixClass + 'lv-input'), listItems, listItems],
                ["click", "keyup", ej.startEvent(), ej.cancelEvent()],
                [this._anchorClickDelegate, this._keyup, this._touchStartDelegate, this._touchMoveDelegate], remove, this);
                this._lContainer.on("scrollstop", $.proxy(this._onScrollStop, this));
                $('body')[eventType]('viewpopstate', this._popStateDelegate);
                if (this.model.enableFiltering)
                    this._initializeFiltering(this._lbEle);
            }
        },
        
        
        _initializeFiltering: function (element) {
            this._searchItems = $(element).find("." + this._prefixClass + "list");
            this._emptyFilterTextValue(element);
            element.find('.' + this._prefixClass + 'list[style*="display: none"]').show();
            var selector = "a";
            this._elementText = [];
            for (var i = 0; i < this._searchItems.length; i++) {
                if ($(this._searchItems[i]).find(selector))
                    this._elementText.push($.trim($(this._searchItems[i]).find(selector).html().replace(new RegExp('<[^<]+\>', 'g'), "").toLowerCase()));
                else
                    this._elementText.push("");
            }
        },
        
        
        _onKeyUp: function (evt) {
            for (var i = 0; i < this._searchItems.length; i++) {
                if (this._elementText[i].indexOf(evt.target.value.toLowerCase()) == -1)
                    $(this._searchItems[i]).css("display", "none");
                else
                    $(this._searchItems[i]).css("display", "");
            }
        },
        
        
        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                var setModel = "_set" + prop.charAt(0).toUpperCase() + prop.slice(1);
                if (this[setModel])
                    this[setModel](options[prop]);
                else
                    refresh = true;
            }
            if (refresh)
                this._refresh();
        },
        
        
        _setTheme: function (value) {
            if (value) {
                this.model.theme = value;
                this._lbEle.removeClass("e-m-dark e-m-light e-default").addClass("e-m-" + this.model.theme);
                if (this.model.enableFiltering)
                    this.element.find('.' + this._prefixClass + 'text-input').ejmTextBox('model.theme', this.model.theme);
                if (this.model.showHeader)
                    this._lbEle.find('#' + this.model.id + "_header").ejmHeader('model.theme', this.model.theme);
                if (this.element.find('.' + this._prefixClass + 'childitem').length) {
                    $(this.element.find('.' + this._prefixClass + 'childitem')).removeClass("e-m-dark e-m-light e-default").addClass("e-m-" + this.model.theme);
                    var header = this.element.find('.' + this._prefixClass + 'childitem .' + this._prefixClass + 'header');
                    var proxy = this;
                    header.each(function (index, element) {
                        $(element).ejmHeader('model.theme', proxy.model.theme);
                    });
                }
            }
        },
        
        
        _setDataSource: function (dataSource, fieldSettings) {
            if (this._hasDataSource() && dataSource) {
                if (fieldSettings)
                    this.model.fieldSettings = fieldSettings;
                this._refresh();
            }
        },
        
        
        _hasDataSource: function () {
            return this.dataSource() && this.dataSource().length;
        },
        
        
        _getElement: function (childId) {
            return childId ? this.element.find('#' + childId) : this._lbEle;
        },
        
        
        _isEnable: function (item) {
            return item.hasClass(this._prefixClass + 'state-disabled') ? false : true;
        },
        //To refresh the control
        _refresh: function () {
            this._destroy();
            this.element.addClass(this._rootCSS);
            this._load();
        },
        //To clear the element
        _clearElement: function () {
            this.element.removeAttr("class").removeAttr("style");
            this.element.empty().html(this._orgEle.html());
        },
        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },

        loadAjaxContent: function (urlVal, backButton) {
            var proxy = this;
            this._isFromAjax = true;
            this._renderChild(ej.getAttrVal(this._currentItem, 'data-childid', "page_" + parseInt(Math.random().toFixed(3) * 1000)));
            if (!$.support.pushstate || ej.isWindowsWebView())
                if (proxy._prefixClass == "e-m-")
                    urlVal = App.route.makeUrlAbsolute(urlVal);
            var loadData = { content: proxy._div, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
            if (this.model.ajaxBeforeLoad)
                this._trigger("ajaxBeforeLoad", loadData);
            var ajaxSettings = {
                cache: proxy.model.ajaxSettings.cache,
                async: proxy.model.ajaxSettings.async,
                type: proxy.model.ajaxSettings.type,
                contentType: proxy.model.ajaxSettings.contentType,
                url: ej.isWindowsWebView() ? urlVal : proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal,
                dataType: proxy.model.ajaxSettings.dataType,
                data: proxy.model.ajaxSettings.data,
                "successHandler": function (data) { //Ajax post success event handler
                    var content = ej.buildTag("div." + this._prefixClass + "content", (/<\/?body[^>]*>/gmi).test(data) ? data.split(/<\/?body[^>]*>/gmi)[1] : data || "");
                    proxy._div.append(proxy._container.append(content));
                    if (proxy._prefixClass == "e-m-")
                        proxy._div.attr("data-ajaxurl", App.route.convertToRelativeUrl(urlVal));
                    proxy._updateContent(proxy._currentItem, backButton);
                    if (proxy._prefixClass == "e-m-" && (App.angularAppName || ej.angular.defaultAppName))
                        ej.angular.compile(content);
                    var successData = { content: proxy._div, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxSuccess)
                        proxy._trigger("ajaxSuccess", successData);
                },
                "errorHandler": function (xhr, textStatus, errorThrown) {
                    var errorData = { "xhr": xhr, "textStatus": textStatus, "errorThrown": errorThrown, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxError)
                        proxy._trigger("ajaxError", errorData);
                },
                "completeHandler": function (data) {
                    var completeData = { content: proxy._div, item: proxy._currentItem, index: $(proxy._currentItem).index(), text: $(proxy._currentItem).text(), url: proxy.model.ajaxSettings.url ? proxy.model.ajaxSettings.url : urlVal };
                    if (proxy.model.ajaxComplete)
                        proxy._trigger("ajaxComplete", completeData);
                }
            };
            ej.sendAjaxRequest(ajaxSettings);
        },

        selectItem: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]))) {
                this.setActive(index, childId);
                this._currentItem = $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]);
                this._prevItem = this._currentItem;
                this._onTouchEndHandler();
            }
        },

        setActive: function (index, childId) {
            if (index >= 0) {
                element = this._getElement(childId);
                if (this._isEnable($(element.find('li.' + this._prefixClass + 'list')[index])) && ej.getBooleanVal($(element.find('li.' + this._prefixClass + 'list')[index]), 'data-persistSelection', this.model.persistSelection)) {
                    element.find('li.' + this._prefixClass + 'list.' + this._prefixClass + 'state-active').removeClass(this._prefixClass + 'state-active').addClass(this._prefixClass + 'state-default');
                    this._currentItem = $(element.find('li.' + this._prefixClass + 'list')[index]);
                    this._prevItem = this._currentItem;
                    this._currentItem.removeClass(this._prefixClass + 'state-default').addClass(this._prefixClass + 'state-active');
                }
            }
        },

        deActive: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index])))
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).removeClass(this._prefixClass + 'state-active').addClass(this._prefixClass + 'state-default');
        },

        enableItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).removeClass(this._prefixClass + 'state-disabled');
        },

        disableItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).addClass(this._prefixClass + 'state-disabled');
        },

        removeCheckMark: function (index, childId) {
            element = this._getElement(childId);
            if (index >= 0 && this._isEnable($(element.find('li.' + this._prefixClass + 'list')[index])))
                $(element.find('li.' + this._prefixClass + 'list')[index]).find('.' + this._prefixClass + 'lv-checkdiv').remove();
            else
                element.find('.' + this._prefixClass + 'lv-checkdiv').remove();
        },

        checkItem: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index])))
                this._setCheckboxValue($(this._getElement(childId).find('.' + this._prefixClass + 'lv-check')[index]), true);
        },

        unCheckItem: function (index, childId) {
            if (index >= 0 && this._isEnable($(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index])))
                this._setCheckboxValue($(this._getElement(childId).find('.' + this._prefixClass + 'lv-check')[index]), false);
        },

        checkAllItem: function (childId) {
            var proxy = this;
            this._getElement(childId).find('.' + this._prefixClass + 'lv-check').each(function (index, check) {
                if (proxy._isEnable($(proxy._getElement(childId).find('li.' + proxy._prefixClass + 'list')[index])))
                    proxy._setCheckboxValue($(check), true);
            });
        },

        unCheckAllItem: function (childId) {
            var proxy = this;
            this._getElement(childId).find('.' + this._prefixClass + 'lv-check').each(function (index, check) {
                if (proxy._isEnable($(proxy._getElement(childId).find('li.' + proxy._prefixClass + 'list')[index])))
                    proxy._setCheckboxValue($(check), false);
            });
        },

        getActiveItem: function (childId) {
            return this._getElement(childId).find('li.' + this._prefixClass + 'list.' + this._prefixClass + 'state-active');
        },

        getActiveItemText: function (childId) {
            return this._getElement(childId).find('li.' + this._prefixClass + 'list.' + this._prefixClass + 'state-active').text();
        },

        getItemText: function (index, childId) {
            if (index >= 0)
                return $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).text();
        },

        getCheckedItems: function (childId) {
            if (childId != undefined)
                return this._getElement(childId).find('input.' + this._prefixClass + 'lv-check:checked').closest('li.' + this._prefixClass + 'list');
            else
                return this.element.find('input.' + this._prefixClass + 'lv-check:checked').closest('li.' + this._prefixClass + 'list');
        },

        getCheckedItemsText: function (childId) {
            return $(this.getCheckedItems(childId)).map(function () { return $(this).text(); }).get();
        },

        hasChild: function (index, childId) {
            if (index >= 0)
                return this.element.find('#' + $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).attr('data-childid')).length ? true : false;
        },
        
        isChecked: function (index, childId) {
            if (index >= 0)
                return $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).find('input.' + this._prefixClass + 'lv-check').prop('checked');
        },

        showItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).css('visibility', '');
        },

        hideItem: function (index, childId) {
            if (index >= 0)
                $(this._getElement(childId).find('li.' + this._prefixClass + 'list')[index]).css('visibility', 'hidden');
        },

        show: function (childId) {
            this._getElement(childId).css('visibility', '');
        },

        hide: function (childId) {
            this._getElement(childId).css('visibility', 'hidden');
        },

        addItem: function (list, index, groupid) {
            if (index >= 0) {
                if (!this._hasDataSource()) {
                    this.model.items.splice(index, 0, this._itemsObjectCollection($(list), null, null, groupid));
                    var items = this.model.items;
                }
                else {
                    this.model.dataSource.splice(index, 0, this._itemsObjectCollection($(list), null, null, groupid));
                    var items = this.model.dataSource;
                }
            }
            else
                this._orgEle.children().append(list);
            if (!this.model.enableGroupList)
                var $ul = $(this.element.find("ul:visible"));
            else
                var $ul = $(this.element.find('ul[data-ej-grouplisttitle= ' + groupid + ']'));
            var ele = $(this.jsRender).render(items[index]);
            ej.widget.init($(ele).insertBefore($ul.children()[index]));
            this._processing($ul);
            this._liEl = this.element.find('li.' + this._prefixClass + 'list');
            this._wireEvents();
        },

        _processing: function ($ul) {
            $ul.find("li." + this._prefixClass + "lastitem").removeClass(this._prefixClass + "lastitem");
            $ul.find('li:last').addClass(this._prefixClass + "lastitem");
            eLi = $ul.find('li.' + this._prefixClass + 'list');
            eLi.removeEleEmptyAttrs();
            eLi.find('.' + this._prefixClass + 'list-anchor').removeEleEmptyAttrs();
            $ul.find('.' + this._prefixClass + 'lv-check').parent().addClass(this._prefixClass + 'lv-checkdiv');
            $ul.find('.' + this._prefixClass + 'lv-check').closest('li.' + this._prefixClass + 'list').addClass(this._prefixClass + 'list-check');
            $ul.find('.' + this._prefixClass + 'lv-input').closest('.' + this._prefixClass + 'lv-filter').addClass(this._prefixClass + 'lv-inputdiv');
        },

        removeItem: function (index, childId) {
            if (index >= 0) {
                element = this._getElement(childId);
                var child = $(element.find('li.' + this._prefixClass + 'list')[index]).attr('data-childid');
                if (this.element.find($('#' + child).length))
                    this.element.find($('#' + child)).remove();
                $(element.find('li.' + this._prefixClass + 'list')[index]).remove();
            }
        },

        clear: function () {
            this.element.empty().html();
            this._liEl = this.element.find('li.' + this._prefixClass + 'list');
        },

        getItemsCount: function (childId) {
            return this._getElement(childId).find('li.' + this._prefixClass + 'list').length;
        },

        append: function (dynamicdatasource) {
            var $ul, newListItems;
            if (dynamicdatasource.length) {
                this.model.dataSource = this.model.dataSource.concat(dynamicdatasource);
                if (!this.model.enableGroupList) {
                    $ul = $(this.element.find("ul:visible"));
                    newListItems = this._rendering(dynamicdatasource, $ul);
                    this._processing($ul);
                }
                else {
                    var group = ej.DataManager(dynamicdatasource).executeLocal(ej.Query().from(dynamicdatasource).group('groupID'));
                    for (var i in group) {
                        $ul = this.element.find('ul[data-ej-grouplisttitle= ' + group[i].key + ']');
                        newListItems = this._rendering(group[i].items, $ul);
                        this._processing($ul);
                    }
                }
                this._liEl = this.element.find('li.' + this._prefixClass + 'list');
                this._wireEvents(false, newListItems);
                if (this._prefixClass == "e-m-" && this.element.find(".e-m-scroll").length) {
                    var scroll = this.element.find(".e-m-scroll");
                    this._scrollInst = scroll.ejmScrollPanel("instance");
                    this._scrollInst.refresh();
                }
            }
            this._setHeightWidth();
        },
        getActiveItemData: function () {
            if (this.getActiveItem().attr("data-id"))
                return (this._generateData(this.dataSource().length ? typeof this.dataSource() == "string" ? eval(this.dataSource()) : this.dataSource() : this.model.items, this.getActiveItem().attr("data-id")));
        }
    });
})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Waiting pop elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejWaitingPopup", "ej.WaitingPopup", {

        element: null,

        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        _rootCSS: "e-waitingpopup",


        defaults: {

            showOnInit: false,

            target: null,

            showImage: true,

            htmlAttributes: {},

            cssClass: "",

            text: null,

            template: null,

            create: null,

            destroy: null
        },

        dataTypes: {
            showOnInit: "boolean",
            showImage: "boolean",
            cssClass: "string"
        },

        show: function () {
            this._refreshPanel();
            this.maindiv.css("display", "block");
            this.model.showOnInit = true;
        },

        hide: function () {
            this.maindiv.css("display", "none");
            this.model.showOnInit = false;
        },

        refresh: function () {
            this._refreshPanel();
        },

        _setText: function (text) {
            if (text) {
                if (this.popupText) this.popupText.html(text);
                else {
                    this._generateTextTag(text);
                    this._setContentPosition();
                }
            }
            else if (this.popupText) {
                this.popupText.remove();
                this.popupText = null;
            }
        },

        _showImage: function (boolean) {
            if (boolean) {
                this.popupImage = ej.buildTag("span.e-image");
                if (this.popupText) this.popupImage.insertBefore(this.popupText);
                else this.maindiv.append(this.popupImage);
            }
            else if (this.popupImage) {
                this.popupImage.remove();
                this.popupImage = null;
            }
        },

        _setTemplate: function () {
            var template = this.model.template;
            if (typeof template === "string") template = $(template);
            if (typeof template === "object" && typeof template.css === "function")
                this.templateObj = template;
            else
                this.templateObj = ej.buildTag("div", "", { "text-align": "center" }).append(template);

            this.templateObj.css({ "visibility": "visible", "display": "block" });
            this.maindiv.append(this.templateObj);
        },

        _setTheme: function (skin) {
            this.maindiv.removeClass(this.model.cssClass).addClass(skin);
        },



        _init: function () {
            this._initialize();
            this._render();
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "text": this._setText(options[option]); break;
                    case "cssClass ": this._setTheme(options[option]); break;
                    case "htmlAttributes ": this._addAttr(options[option]); break;
                    case "showOnInit": this._setVisibility(options[option]); break;
                    case "showImage": this._showImage(options[option]); this._setContentPosition(); break;
                    case "target": this.model.target = options[option]; this._setTarget(); this.refresh(); break;
                    case "template":
                        this.maindiv.empty();
                        if (options[option]) {
                            this.model.template = options[option];
                            this._setTemplate();
                        }
                        else {
                            this.model.template = options[option] = null;
                            this._showImage(this.model.showImage);
                            if (this.model.text) this._generateTextTag(this.model.text);
                        }
                        this._setContentPosition();
                        break;
                }
            }
        },
        _setTarget: function () {
            this.targetElement = this.model.target ? $(this.model.target) : this.element;
        },
        _destroy: function () {
            this.maindiv.remove();
        },

        _initialize: function () {
            this.maindiv = null;
            this.popupText = null;
            this.popupImage = null;
            this.templateObj = null;
            this.targetElement = null;
        },


        _render: function () {
            this._setTarget();
            var oldWrapper = $("#" + this.element.context.id + "_WaitingPopup").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.maindiv = ej.buildTag("div.e-waitpopup-pane e-widget " + this.model.cssClass + "#" + this.element[0].id + "_WaitingPopup");
            if (this.model.template) {
                this._setTemplate();
            }
            else {
                this._showImage(this.model.showImage);
                if (this.model.text) {
                    this._generateTextTag(this.model.text);
                }
            }
            $('body').append(this.maindiv);
            this._setVisibility(this.model.showOnInit);
            this._addAttr(this.model.htmlAttributes);
        },

        _refreshPanel: function () {
            this.maindiv.width(this.targetElement.outerWidth());
            this.maindiv.height(this.targetElement.outerHeight());
            this._setPanelPosition();
            this._setContentPosition();
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.maindiv.addClass(value);
                else proxy.maindiv.attr(key, value)
            });
        },


        _setPanelPosition: function () {
            var location = ej.util.getOffset(this.targetElement);
            this.maindiv.css({
                "position": "absolute",
                "left": Math.ceil(location.left) + "px",
                "top": Math.ceil(location.top) + "px",
                "z-index": this._maxZindex() + 1
            });
        },

        _setContentPosition: function () {
            if (this.model.template == null) {
                var textHeight = 0, imgHeight = 0, targetHeight, top = null;
                targetHeight = this.targetElement.outerHeight();
                if (this.popupText) textHeight = this.popupText.outerHeight();
                if (this.popupImage) imgHeight = this.popupImage.outerHeight();

                if (this.popupImage) {
                    top = Math.ceil((targetHeight - (imgHeight + textHeight)) / 2);
                    this.popupImage.css("top", top + "px");
                }
                if (this.popupText) {
                    if (!top) top = Math.ceil((targetHeight - textHeight) / 2);
                    this.popupText.css("top", top + "px");
                }
            }
            else {
                this.templateObj.css({
                    "position": "relative",
                    "left": Math.ceil((this.targetElement.outerWidth() - this.templateObj.outerWidth()) / 2),
                    "top": Math.ceil((this.targetElement.outerHeight() - this.templateObj.outerHeight()) / 2)
                });
            }
        },

        _generateTextTag: function (text) {
            this.popupText = ej.buildTag("div.e-text", text);
            this.maindiv.append(this.popupText);
        },

        _setVisibility: function (showOnInit) {
            if (showOnInit) this.show();
            else this.hide();
        },

        _maxZindex: function () {
            return ej.util.getZindexPartial(this.element);
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejButton", "ej.Button", {

        element: null,

        model: null,
        validTags: ["button", "input"],
        _setFirst: false,

        _rootCSS: "e-button",

        defaults: {

            size: "normal",

            type: "submit",

            height: "",

            width: "",

            enabled: true,

            htmlAttributes: {},

            text: null,

            contentType: "textonly",

            imagePosition: "imageleft",

            showRoundedCorner: false,

            cssClass: "",

            prefixIcon: null,

            suffixIcon: null,

            enableRTL: false,

            repeatButton: false,

            timeInterval: "150",

            create: null,

            click: null,

            destroy: null
        },


        dataTypes: {
            size: "enum",
            enabled: "boolean",
            type: "enum",
            showRoundedCorner: "boolean",
            text: "string",
            contentType: "enum",
            imagePosition: "enum",
            prefixIcon: "string",
            suffixIcon: "string",
            cssClass: "string",
            repeatButton: "boolean",
            enableRTL: "boolean",
            timeInterval: "string",
            htmlAttributes: "data"
        },

        disable: function () {
            this.element.addClass("e-disable").attr("aria-disabled", true);
            this.model.enabled = false;
        },

        enable: function () {
            this.element.removeClass("e-disable").attr("aria-disabled", false);
            this.model.enabled = true;
        },

        _init: function () {
            this._initialize();
            this._render();
            this._controlStatus(this.model.enabled);
            this._wireEvents(this.model.repeatButton);
            this._addAttr(this.model.htmlAttributes);
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else proxy.element.attr(key, value);
                if (key == "disabled" && value == "disabled") proxy.disable();
            });
        },

        _destroy: function () {
            this.element.removeClass(this.model.cssClass + " e-btn e-select e-disable e-corner e-widget");
        },


        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "text":
                        this._setText(options[option]);
                        break;
                    case "prefixIcon":
                        this._setMajorIcon(options[option]);
                        break;
                    case "suffixIcon":
                        this._setMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        break;
                    case "timeInterval":
                        this.model.timeInterval = options[option];
                        break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }
        },


        _setSize: function (val) {
            this.element.removeClass('e-btn-mini e-btn-medium e-btn-small e-btn-large e-btn-normal');
            this.element.addClass("e-btn-" + val);
        },
        _setType: function (val) {
            this.element.prop({ "type": val });
        },

        _setHeight: function (val) {
            this.element.css('height', val);
        },

        _setWidth: function (val) {
            this.element.css('width', val);
        },

        _setText: function (val) {
            if (this.buttonType == "inputButton") {
                this.element.val(val);
            } else {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.element.text(val);
                } else {
                    this.textspan.text(val);
                }
            }
            this.model.text = val;
        },

        _setMajorIcon: function (val) {
            this.majorimgtag.removeClass(this.model.prefixIcon);
            this.majorimgtag.addClass(val);
            this.model.prefixIcon = val;
        },

        _setMinorIcon: function (val) {
            this.minorimgtag.removeClass(this.model.suffixIcon);
            this.minorimgtag.addClass(val);
            this.model.suffixIcon = val;
        },

        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.element.empty();
                this.model.contentType = val;
                this._renderButtonNormal();
            }
        },

        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.ContentType.TextAndImage) && (val != this.model.imagePosition)) {
                this.element.empty();
                this.model.imagePosition = val;
                this._renderButtonNormal();
            }
        },

        _setRTL: function (val) {
            if (val) {
                this.element.addClass("e-rtl");
            } else {
                this.element.removeClass("e-rtl");
            }
        },

        _controlStatus: function (value) {
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },

        _setSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass(skin);
            }
        },

        _initialize: function () {
            if (this.element.is("input")) {
                this.buttonType = "inputButton";
            }
            else if ((this.element.is("a")) || (this.element.is("button"))) {
                this.buttonType = "tagButton";
            }
            else {
                this.element.removeClass("e-button");
            }
            if (this.element.attr("type")) {
                this.model.type = this.element.attr("type");
            }
            else
                this._setType(this.model.type);
            this._timeout = null;
        },


        _render: function () {
            this._setSize(this.model.size);
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
            this._setRTL(this.model.enableRTL);
            this.element.addClass(this.model.cssClass + " e-btn e-select e-widget").attr("role", "button");
            if (this.buttonType == "inputButton") {
                this.element.addClass("e-txt");
                if ((this.model.text != null) && (this.model.text != "")) {
                    this.element.val(this.model.text);
                } else {
                    this.model.text = this.element.val();
                }
            } else { this._renderButtonNormal(); }
            this._roundedCorner(this.model.showRoundedCorner);
            if (this.model.text)
                this.element.attr("aria-describedby", this.model.text);
        },

        _renderButtonNormal: function () {
            if ((this.model.text == null) || (this.model.text == "")) {
                this.model.text = this.element.text();
            }
            this.element.empty();
            /*Image and Text*/
            this.textspan = ej.buildTag('span.e-btntxt', this.model.text);
            if (this.model.contentType.indexOf("image") > -1) {
                this.majorimgtag = ej.buildTag('span.e-icon ' + this.model.prefixIcon);
                this.minorimgtag = ej.buildTag('span.e-icon ' + this.model.suffixIcon);
                this.imgtxtwrap = ej.buildTag('div');
            }

            if (this.model.contentType == ej.ContentType.TextAndImage) {
                switch (this.model.imagePosition) {
                    case ej.ImagePosition.ImageRight:
                        this.imgtxtwrap.append(this.textspan, this.majorimgtag);
                        break;
                    case ej.ImagePosition.ImageLeft:
                        this.imgtxtwrap.append(this.majorimgtag, this.textspan);
                        break;
                    case ej.ImagePosition.ImageBottom:
                        this.majorimgtag.attr("style", "display:inherit");
                        this.imgtxtwrap.append(this.textspan, this.majorimgtag);
                        break;
                    case ej.ImagePosition.ImageTop:
                        this.majorimgtag.attr("style", "display:inherit");
                        this.imgtxtwrap.append(this.majorimgtag, this.textspan);
                        break;
                }
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageTextImage) {
                this.imgtxtwrap.append(this.majorimgtag, this.textspan, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageBoth) {
                this.imgtxtwrap.append(this.majorimgtag, this.minorimgtag);
                this.element.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageOnly) {
                this.imgtxtwrap.append(this.majorimgtag);
                this.element.append(this.imgtxtwrap);
            } else {
                this.element.addClass("e-txt");
                this.element.text(this.model.text);
            }
        },

        _roundedCorner: function (value) {
            value == true ? this.element.addClass('e-corner') : this.element.removeClass('e-corner');
        },

        _wireEvents: function (val) {
            if (val) {
                this._on(this.element, "mousedown", this._btnRepatMouseClickEvent);
                this._on($(document), 'mouseup', this._mouseUpClick);
                this._on(this.element, "keyup", this._btnRepatKeyUpEvent);
                this._on($(document), "keypress", this._btnRepatKeyDownEvent);

            }
            this._on(this.element, "click", this._btnMouseClickEvent);
        },

        _btnMouseClickEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                // here aregument 'e' used in serverside events 
                var args = { status: self.model.enabled, e: e };
                self._trigger("click", args);
            } 
        },

        _btnRepatMouseClickEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                if ((e.button == 0) || (e.which == 1)) {

                    self._timeout = setInterval(function () { self._trigger("click", { target: e.currentTarget, status: self.model.enabled }); }, this.model.timeInterval);
                }
            }
        },

        _mouseUpClick: function (event) {
            clearTimeout(this._timeout);
        },

        _btnRepatKeyDownEvent: function (e) {
            var self = this;
            if (!self.element.hasClass("e-disable")) {
                var args = { status: self.model.enabled };
                if ((e.keyCode == 32) || (e.keyCode == 13)) {
                    self._trigger("click", args);
                }
            }
        },

        _btnRepatKeyUpEvent: function (e) {
            if ((e.keyCode == 32) || (e.keyCode == 13)) {
                clearTimeout(this._timeout);
            }
        },
    });


    ej.ContentType = {
		/**  Supports only for text content only */
		TextOnly: "textonly", 
		/** Supports only for image content only */
		ImageOnly: "imageonly", 
		/** Supports image for both ends of the button */
		ImageBoth: "imageboth", 
		/** Supports image with the text content */
		TextAndImage: "textandimage", 
		/** Supports image with both ends of the text */
        ImageTextImage: "imagetextimage"
    };


    ej.ImagePosition = {
		/**  support for aligning text in left and image in right. */
		ImageRight: "imageright", 
		/**  support for aligning text in right and image in left. */
		ImageLeft: "imageleft",
		/**  support for aligning text in bottom and image in top. */
		ImageTop: "imagetop", 
		/**  support for aligning text in top and image in bottom. */
		ImageBottom: "imagebottom"
    };

    ej.ButtonSize = {
		/**  Creates button with inbuilt default size height, width specified */
		Normal : "normal",
		/**  Creates button with inbuilt mini size height, width specified */
		Mini: "mini", 
		/**  Creates button with inbuilt small size height, width specified */
		Small: "small",
		/**  Creates button with inbuilt medium size height, width specified */
		Medium:"medium", 
		/**  Creates button with inbuilt large size height, width specified */
        Large: "large"
    };

    ej.ButtonType = {
		/**  Creates button with inbuilt button type specified */
		Button : "button",
		/**  Creates button with inbuilt reset type specified */
		Reset: "reset", 
		/**  Creates button with inbuilt submit type specified */
		Submit: "submit"
    };
})(jQuery, Syncfusion);
;
/**
* @fileOverview Calculate engine to perform the calculatin
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.3 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    

    this.__calcQuickextends = function (d, b) {
        for (var p = 0; p < b.length; p++) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() { this.constructor = d; }
        __.prototype = b.prototype;
        d.prototype = new __();
    };
    
	/**
    * @namespace ej
	* @classdesc Custom engine to perfom calculation like excel sheet
	* @class ejCalculate
	* @requires jQuery
	* @requires ej.core.js
	* @requires ej.calculate.js
	* @example 
    * &lt;script&gt;
	* // Create calcEngine with grid data
    * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
	* &lt;/script&gt;
	*/
    CalcEngine = function (parentObject) {
        this.parentObject = parentObject;
        this._useFormulaValues = false;
        this._multiTick = false;
        this._isErrorString = false;
        this._ignoreBracet = false;
        this._libraryFunctions = new HashTable();
        this._customlibraryFunctions = new HashTable();
        this._refreshedCells = new HashTable();
        this._treatStringsAsZero = true;
        this._supportLogicalOperators = false;
        this._isRangeOperand = false;
        this._uniqueStringMarker = String.fromCharCode(127);
        this._isParseArgumentSeparator = false;
        this._isParseDecimalSeparatorChanged = false;
        this._validPrecedingChars = " (+-*/^&<>=";
        this._validFunctionNameChars = "_";
        this.bMARKER = String.fromCharCode(146);
        this.bMARKER2 = this.bMARKER + this.bMARKER;
        this.tic = "\"";
        this._parseDecimalSeparator = '.';
        this.parseDateTimeSeparator = '/';
        this._parseArgumentSeparator = ',';
        this._rightBracket = String.fromCharCode(131);
        this._leftBracket = String.fromCharCode(130);
        this.iFMarker = "qIF" + String.fromCharCode(130);
        this.braceLeft = "{";
        this.braceRight = "}";
        this._braceRightNLeft = ")(";
        this.computedValueLevel = 0;
        this._circCheckList = [];
        this._maximumRecursiveCalls = 100;
        this._sortedSheetNames = null;
        this.trueValueStr = "TRUE";
        this.falseValueStr = "FALSE";
        this.useDatesInCalcs = false;
        this.sheetToken = '!';
        this.namedRanges = null;
        this.undefinednamedRange = null;
        this.undefinedsheetNamedRnages = null;
        this.namerangecellcollection = null;
        this.sheetNamedRangesOriginalNames = null;
        this.sheetNamedRangeCellCollection = null;
        this.sheetDependentNamedRangeCells = null;
        this.namedRangesOriginalNames = null;
        this.namedRangeValues = null;
        this.rangeValues = null;
        this.dependentNamedRangeCells = null;
        this.namedRangesSized = null;
        this._namedRangesNonScoped = null;
        this.char_add = '+';
        this.char_and = 'i';
        this.char_ANDop = String.fromCharCode(140);
        this.char_divide = '/';
        this.char_ELSEop = String.fromCharCode(144);
        this.char_EM = 'r';
        this.char_EP = 'x';
        this.char_equal = '=';
        this.char_greater = '>';
        this.char_greatereq = 'h';
        this.char_IFop = String.fromCharCode(142);
        this.char_less = '<';
        this.char_lesseq = 'f';
        this.char_multiply = '*';
        this.char_noequal = 'p';
        this.char_NOTop = String.fromCharCode(145);
        this.char_or = 'w';
        this.char_ORop = String.fromCharCode(139);
        this.char_subtract = '-';
        this.char_THENop = String.fromCharCode(143);
        this.char_XORop = String.fromCharCode(141);
        this.chartic = "'"[0];
        this._string_and = "&";
        this._string_E = "E";
        this._string_EM = "E-";
        this._string_empty = "";
        this._string_EP = "E+";
        this._string_fixedreference = "$";
        this._string_greatereq = ">=";
        this._string_lesseq = "<=";
        this._string_noequal = "<>";
        this._string_or = "^";
        this.token_add = 'a';
        this.token_and = 'c';
        this.token_ANDop = String.fromCharCode(133);
        this.token_divide = 'd';
        this.token_ELSEop = String.fromCharCode(137);
        this.token_EM = 'v';
        this.token_EP = 't';
        this.token_equal = 'e';
        this.token_greater = 'g';
        this.token_greatereq = 'j';
        this.token_IFop = String.fromCharCode(135);
        this.token_less = 'l';
        this.token_lesseq = 'k';
        this.token_multiply = 'm';
        this.token_noequal = 'o';
        this.token_NOTop = String.fromCharCode(138);
        this.token_or = String.fromCharCode(126);
        this.token_ORop = String.fromCharCode(132);
        this.token_subtract = 's';
        this.token_THENop = String.fromCharCode(136);
        this.token_XORop = String.fromCharCode(134);
        this.tokens = [
            this.token_add, this.token_subtract, this.token_multiply, this.token_divide, this.token_less,
            this.token_greater, this.token_equal, this.token_lesseq, this.token_greatereq, this.token_noequal, this.token_and, this.token_or];
        this._dateTime1900 = new Date(1900, 1, 1, 0, 0, 0);
        this._preserveLeadingZeros = false;
        this._ignoreCellValue = false;
        this._errorStrings = null;
        this._cell = "";
        this._iterationMaxCount = 0;
        this._supportRangeOperands = false;
        this._allowShortCircuitIFs = false;
        this._processUpperCaseFormula = "";
        this._processUpperCaseIvalue = 0;
        this._processUpperCaseSheet = "";
        this._markerChar = '`';
        this._rowMaxCount = -1;
        this._columnMaxCount = -1;
        this._isInteriorFunction = false;
        this._tempSheetPlaceHolder = String.fromCharCode(133);
        this.sheetFamilyID = 0;
        this._supportsSheetRanges = true;
        this._markers = "()+-*/=><.,!";
        this._formulaInfoTable = null;
        this._dependentFormulaCells = null;
        this._dependentCells = new HashTable();
        this._calculatingSuspended = false;
        this._inAPull = false;
        this._useDatesInCalcs = false;
        this._excelLikeComputations = false;
        this._rethrowLibraryComputationExceptions = false;
        this.formulaErrorStrings = [
            "binary operators cannot start an expression",
            "cannot parse",
            "bad library",
            "invalid char in front of",
            "number contains 2 decimal points",
            "expression cannot end with an operator",
            "invalid characters following an operator",
            "invalid character in number",
            "mismatched parentheses",
            "unknown formula name",
            "requires a single argument",
            "requires 3 arguments",
            "invalid Math argument",
            "requires 2 arguments",
            "#NAME?",
            "too complex",
            "circular reference: ",
            "missing formula",
            "improper formula",
            "invalid expression",
            "cell empty",
            "bad formula",
            "empty expression",
            "",
            "mismatched string quotes",
            "wrong number of arguments",
            "invalid arguments",
            "iterations do not converge",
            "Control named '{0}' is already registered",
            "Calculation overflow",
            "Missing sheet"
        ];
        this._parseDateTimeSeparator = '/';
        this._millisecondsOfaDay = 24 * 60 * 60 * 1000;
        this.treat1900AsLeapYear = false;
        this._oaDate = new Date(1899, 11, 30);
        this._saveStringsText = "";
        this._processedCells = [];
        this.ignoreValueChanged = false;
        this._breakedFormulaCells = [];
        this._tempBreakedFormulaCells = [];
        this._lockDependencies = false;
        this._useDependencies = false;
        this._inHandleIterations = false;
        this._inRecalculateRange = false;
        this._useNoAmpersandQuotes = false;
        this._calcID = 0;
        this._operators_cannot_start_an_expression = 0;
        this._reservedWord_AND = 1;
        this._reservedWord_XOR = 2;
        this._reservedWord_IF = 3;
        this._number_contains_2_decimal_points = 4;
        this._reservedWord_ELSE = 5;
        this._reservedWord_NOT = 6;
        this._invalid_char_in_number = 7;
        this._mismatched_parentheses = 8;
        this._unknown_formula_name = 9;
        this._requires_a_single_argument = 10;
        this._requires_3_args = 11;
        this._invalid_Math_argument = 12;
        this._requires_2_args = 13;
        this._bad_index = 14;
        this._too_complex = 15;
        this._circular_reference_ = 16;
        this._missing_formula = 17;
        this._improper_formula = 18;
        this._invalid_expression = 19;
        this._cell_empty = 20;
        this._bad_formula = 21;
        this._empty_expression = 22;
        this._virtual_mode_required = 23;
        this._mismatched_tics = 24;
        this._wrong_number_arguments = 25;
        this._invalid_arguments = 26;
        this._iterations_dont_converge = 27;
        this._calculation_overflow = 29;
        this._already_registered = 28;
        this._missing_sheet = 30;
        this._alwaysComputeDuringRefresh = true;
        this._libraryComputationException = null;
        this._dependencyLevel = 0;
        this._isDisposed = undefined;
        this._forceRefreshCall = false;
        this.grid = this.parentObject;
        this._enableLookupTableCaching = 0 /* None */;
        this._lookupTables = new HashTable();
        this._isIE8 = (ej.browserInfo().name == 'msie' && ej.browserInfo().version == '8.0') ? true : false;

		/**
         * 
		 * @private
         */	 
        this._addFunction = function (name, func) {
            name = name.toUpperCase();
            if (this._libraryFunctions.getItem(name) == undefined) {
                this._libraryFunctions.add(name, func);
                return true;
            }
            return false;
        };
        /**
         * 
		 * @private
         */
		 this._addToFormulaDependentCells = function (s) {
            var cell1 = this.cell;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family.sheetNameToParentObject != null && cell1.indexOf(this.sheetToken) == -1) {
                var token = family.parentObjectToToken.getItem(this.grid);
                cell1 = token + cell1;
            }

            if (!this.getDependentFormulaCells().containsKey(cell1)) {
                this.getDependentFormulaCells().add(cell1, new HashTable());
                this.getDependentFormulaCells().getItem(cell1).add(s, s);
            } else if (!(this.getDependentFormulaCells().getItem(cell1)).containsKey(s)) {
                this.getDependentFormulaCells().getItem(cell1).add(s, s);
            }
        };
        /**
         * 
		 * @private
         */
		 this._arrayRemove = function (array, value) {
            var index = null;

            while ((index = array.indexOf(value)) !== -1)
                array.splice(index, 1);

            return array;
        };
        /**
         * 
		 * @private
         */
		 this._canGetRowIndex = function (s) {
            var i = 0;
            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }
                i++;
            }

            while (i < s.length && this._isLetter(s[i])) {
                i++;
            }

            if (i < s.length) {
                if (this._isDigit(s[i]))
                    return true;
                else
                    return false;
            }
            return false;
        };
        /**
         * 
		 * @private
         */
		 this._checkAddNonScopedNamedRange = function (key) {
            var loc = -1;
            if ((loc = key.indexOf('!')) > -1) {
                var key1 = key.substring(loc + 1);
                if (!this._namedRangesNonScoped.containsKey(key1)) {
                    this._namedRangesNonScoped.add(key1, this.namedRanges.getItem(key));
                }
            }
        };
        /**
         * 
		 * @private
         */
		 this._checkHasCharBeforeNumber = function (tempFormula) {
            var check = false;
            for (var x = tempFormula.length - 1; x > 0; x--) {
                if (this._isLetter(tempFormula[x])) {
                    check = true;
                    break;
                }
            }
            return check;
        };
        /**
         * 
		 * @private
         */
		this._checkIfScopedRange = function (text, scopedRange) {
            scopedRange = "";
            var b = "NaN";
            var id = this.getSheetID(this.grid);
            var sheet = CalcEngine.getSheetFamilyItem(this.grid);

            if (text[0] == this.sheetToken.toString()) {
                var i = text.indexOf(this.sheetToken, 1);
                if (i > 1 && !isNaN(parseInt(text.substring(1, i - 1)))) {
                    //id reset to proper value
                    text = text.substring(i + 1);
                }
            }

            var token = '!' + id.toString() + '!';

            if (sheet.sheetNameToToken == null)
                return b;
            var sheetNameKeys = sheet.sheetNameToToken.keys();
            for (var name = 0; name < sheetNameKeys.length; name++) {
                if (sheet.sheetNameToToken.getItem(sheetNameKeys[name]).toString() == token) {
                    var s = (sheetNameKeys[name] + '!' + text).toUpperCase();
                    if (this.getNamedRanges().containsKey(s)) {
                        scopedRange = (this.getNamedRanges().getItem(s)).toUpperCase();
                        b = scopedRange;
                    }
                }
            }
            return b;
        };
        /**
         * 
		 * @private
         */
		this._checkUnderTolerance = function (d, oldValue) {
            if (Math.abs(oldValue) > this._absoluteZero) {
                return Math.abs((d - oldValue) / oldValue) < this._iterationMaxTolerance;
            } else {
                return Math.abs(d - oldValue) < this._iterationMaxTolerance;
            }
        };
        /**
         * 
		 * @private
        */
		this._combineStack = function (formula, i, _tempStack) {
            var combineString = "";
            if (_tempStack.length == 2) {
                var j = 3;
                var checkString = formula.substring(i);
                while (j > 1) {
                    var popStringValue = _tempStack.pop().toString();
                    combineString = (popStringValue == (this.tic + this.tic)) ? combineString + this.tic : popStringValue + combineString;
                    checkString = (j == 3) ? popStringValue : checkString;
                    j--;
                }
                if (_tempStack[0] == (this.tic + this.tic))
                    _tempStack.pop();
                combineString = (formula.length == i && this._isTextEmpty(checkString.split(this.tic).join(""))) ? combineString + checkString : combineString;

                _tempStack.push(combineString);
            }
            return _tempStack;
        };
        /**
         * 
		 * @private
        */
		this._computeMath = function (args, func) {
            var ans = 0;
            var s1;
            var isDegree = false;

            ////Only work with first arg.
            if (args.length > 0) {
                args = this.splitArgsPreservingQuotedCommas(args)[0];
            }
            var indexArray = [this.getParseArgumentSeparator(), ':'];

            ////parsed formula
            if (args.length > 0 && ((!this._isLetter(args[0]) && args[0] != this.sheetToken && args[0] != this.bMARKER) || (args[0] == 'u' && this._isDigit(args[1]))) && this._indexOfAny(args, indexArray) == -1) {
                ////Swap out unary minus.
                args = (args.split('u').join('-')).split("n").join("");

                var d1 = this._parseDouble(args);

                if (!isNaN(d1)) {
                    ans = func(d1);
                } else {
                    return this.getErrorStrings()[1].toString();
                }
            } else if (args.length > 0 && (args[0] == this.bMARKER || args[0] == 'u' || args[0] == 'n' || this._indexOfAny(args, this.tokens) > -1)) {
                ////parsed formula
                args = args.split('{').join('(');
                args = args.split('}').join(')');
                var rightparam = "";
                var decimalPI = (Math.PI).toString();
                try  {
                    if (args.indexOf(decimalPI) > -1) {
                        s1 = this._substring(args, 1, args.indexOf(decimalPI) - 2);
                        s1 = (this._isTextEmpty(s1)) ? "1" : s1;
                        rightparam = (this._indexOfAny(args, this.tokens) > -1) ? args.split(decimalPI.toString())[1] : "1";
                        isDegree = true;
                    } else
                        s1 = this.computedValue(args);
                } catch (ex) {
                    if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                        throw this.getLibraryComputationException();
                    }

                    return ex;
                }
                if (isDegree) {
                    var leftparam = this.computedValue(s1.toString());
                    if (rightparam.indexOf("180") > -1) {
                        d1 = this._parseDouble(leftparam);
                        if (d1)
                            ans = func(d1 * (Math.PI / 180));
                        else {
                            return this.getErrorStrings()[1].toString();
                        }
                    } else {
                        var temp, d2;
                        temp = this.computedValue(leftparam + rightparam);
                        d2 = this._parseDouble(temp);
                        if (!isNaN(d2))
                            ans = func(d2 * (Math.PI));
                        else {
                            return this.getErrorStrings()[1].toString();
                        }
                    }
                } else {
                    d1 = this._parseDouble(s1);
                    if (!isNaN(d1))
                        ans = func(d1);
                    else {
                        return this.getErrorStrings()[1].toString();
                    }
                }
            } else {
                ////Swap out unary minus.
                args = args.split('u').join('-');
                var cells = this.getCellsFromArgs(args);
                for (var s = 0; s < cells.length; s++) {
                    try  {
                        s1 = this.getValueFromArg(cells[s]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d1 = this._parseDouble(s1);
                        if (!isNaN(d1)) {
                            ans = func(d1);
                        } else
                            return this.getErrorStrings()[1].toString();
                    }

                    break;
                }
            }
            if (this.computeIsErr(ans.toString()) == this.trueValueStr) {
                return this.getErrorStrings()[4].toString();
            }
            return ans.toString();
        };
        /**
         * 
		 * @private
        */
		this._computeInteriorFunctions = function (formula) {
            try  {
                if (this._isTextEmpty(formula)) {
                    return formula;
                }

                this.computeFunctionLevel++;

                ////int q = formula.LastIndexOf('q');
                var q = this._findLastqNotInBrackets(formula);
                while (q > 0) {
                    var last = formula.substring(q).indexOf(this._rightBracket);
                    if (last == -1) {
                        throw this.formulaErrorStrings[this._bad_formula];
                    }
                    this._isInteriorFunction = true;
                    var s = this._substring(formula, q, last + 1);
                    s = this.computedValue(s);
                    if (!(s == "") && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                        var newS = this._substring(s, 1, s.length - 2);
                        if (newS.indexOf(this.tic) != -1) {
                            this._multiTick = true;
                            newS = newS.split(this.tic).join("|");
                        }
                        s = this.tic + newS + this.tic;
                    }
                    s = this._markupResultToIncludeInFormula(s);
                    if (!this._isInteriorFunction)
                        formula = formula.substring(0, q) + s.split(this.tic).join("") + formula.substring(q + last + 1);
                    else
                        formula = formula.substring(0, q) + s + formula.substring(q + last + 1);
                    this._isInteriorFunction = false;
                    q = this._findLastqNotInBrackets(formula);
                }
            } catch (ex) {
                return ex;
            } finally {
                this.computeFunctionLevel--;
            }

            return formula;
        };
        /**
         * 
		 * @private
        */
		this._findAndCheckPrecedingChar = function (copy, op, loc) {
            var loc1 = copy.indexOf(op, loc);
            if (loc1 > 0) {
                while (loc1 > -1 && this.getValidPrecedingChars().indexOf(copy[loc1 - 1]) == -1) {
                    loc1 = copy.indexOf(op, loc1 + 1);
                }
            }
            this._findAndCheckPrecedingCharCopy = copy;
            return loc1;
        };
        /**
         * 
		 * @private
        */
		this._findLastNonQB = function (text) {
            var ret = -1;
            if (text.indexOf(this.bMARKER) > -1) {
                var bracketLevel = 0;
                for (var i = text.length - 1; i >= 0; --i) {
                    if (text[i] == this._rightBracket) {
                        bracketLevel--;
                    } else if (text[i] == this._leftBracket) {
                        bracketLevel++;
                    } else if (text[i] == this.bMARKER && bracketLevel == 0) {
                        ret = i;
                        break;
                    }
                }
            }

            return ret;
        };
        /**
         * 
		 * @private
        */
		this._findLastqNotInBrackets = function (s) {
            var found = -1;
            var lastBracket = false;
            var i = s.length - 1;
            while (i > -1) {
                if (s[i] == 'q' && lastBracket) {
                    found = i;
                    break;
                }

                if (s[i] == this._leftBracket) {
                    lastBracket = true;
                } else if (s[i] == this._rightBracket) {
                    lastBracket = false;
                }

                i--;
            }

            return found;
        };
        /**
         * 
		 * @private
        */
		this._findNextSeparator = function (formula, location) {
            var qCount = 0;
            var found = false;
            while (!found && location < formula.length) {
                if (formula[location] == 'q') {
                    qCount++;
                } else if (formula[location] == this._rightBracket) {
                    qCount--;
                } else if (qCount == 0 && formula[location] == this.getParseArgumentSeparator()) {
                    found = true;
                    location--;
                }

                location++;
            }
		    return location.toString();
		};
        /**
         * 
		 * @private
        */
		this._findNextEndIndex = function (formula, location) {
		    var count = 0;
		    var loc = location;
		    var found = false;
		    while (!found && location < formula.Length) {
		        if (formula[location] == '[') {
		            count++;
		        }
		        else if (formula[location] == ']') {
		            count--;
		            if (count == 0) {
		                found = true;
		            }
		        }
		        location++;
		    }
		    location = location - loc;
		    return location;
		};

        /**
         * 
		 * @private
        */
		this._findNonQB = function (text) {
            var ret = -1;
            if (text.indexOf(this.bMARKER) > -1) {
                var bracketLevel = 0;
                for (var i = 0; i < text.length; ++i) {
                    if (text[i] == this._rightBracket) {
                        bracketLevel--;
                    } else if (text[i] == this._leftBracket) {
                        bracketLevel++;
                    } else if (text[i] == this.bMARKER && bracketLevel == 0) {
                        ret = i;
                        break;
                    }
                }
            }

            return ret;
        };
        /**
         * 
		 * @private
        */
		this._findRightBracket = function (formula, location) {
            var qCount = 0;
            var found = false;
            while (!found && location < formula.length) {
                if (formula[location] == 'q') {
                    qCount++;
                } else if (qCount == 0 && formula[location] == this._rightBracket) {
                    found = true;
                    location--;
                } else if (formula[location] == this._rightBracket) {
                    qCount--;
                }

                location++;
            }

            if (found)
                return location.toString();
            else
                return "NaN";
        };
        /**
         * 
		 * @private
        */
		this._fromOADate = function (doubleNumber) {
            var result = new Date();
            result.setTime((doubleNumber * this._millisecondsOfaDay) + Date.parse(this._oaDate));
            return result;
        };
        /**
         * 
		 * @private
        */
		this._getDateFromSerialDate = function (days) {
            if (this.treat1900AsLeapYear && days > 59) {
                days -= 1;
            }

            return new Date(this._oaDate.setDate(this._oaDate.getDate() + days));
		};


        /**
        * 
        * @private
       */
		var _weekEndType = ["", "6,0", "0,1", "1,2", "2,3", "3,4", "4,5", "5,6", "", "", "", "0", "1", "2", "3", "4", "5", "6"];

        /**
         * 
		 * @private
        */
		this._getDoubleArray = function (range) {
            var d;
            var s1 = "";
            var x = [];

            range = this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cells.length; s++) {
                        s1 = this.getValueFromArg(cells[s]);

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                x.push(Number(d));
                            } else {
                                x.push(0.0);
                            }
                        }
                    }
                } else {
                    s1 = this.getValueFromArg(ranges[r]);
                    d = this._parseDouble(s1);
                    if (!isNaN(Number(d))) {
                        x.push(d);
                    } else {
                        x.push(0.0);
                    }
                }
            }
            return x;
        };
        /**
         * 
		 * @private
        */
		this._getDoubleArrayA = function (range) {
            var d;
            var s1 = "";
            var x = [];

            range = this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cells.length; s++) {
                        s1 = this.getValueFromArg(cells[s]);

                        if (s1.length > 0) {
                            if (s1 == this.trueValueStr || s1== this.falseValueStr) {
                                d = 0.0;
                            } else {
                                d = this._parseDouble(s1);
                                if (isNaN(Number(d))) {
                                    d == 0.0;
                                }
                            }
                            x.push(d);
                        }
                    }
                } else {
                    s1 = this.getValueFromArg(ranges[r]);
                    if (s1 == this.trueValueStr || s1 == this.falseValueStr) {
                        d = 0.0;
                    } else {
                        d = this._parseDouble(s1.toString());
                        if (isNaN(d)) {
                            d == 0.0;
                        }
                    }
                    x.push(d);
                }
            }
            return x;
        };
        /**
         * 
		 * @private
        */
		this._getFormulaArrayBounds = function (currentCell, arrayHeight, arrayWidth) {
            var currentColIndex = this.colIndex(this.cell);
            var currentRowIndex = this.rowIndex(this.cell);

            var formulaText = "";
            var formula = this.getFormulaInfoTable().containsKey(this.cell) ? this.getFormulaInfoTable().getItem(this.cell) : null;
            if (formula != null)
                formulaText = formula.getFormulaText();
            else if (this.parentObject.getValueRowCol == undefined)
                formulaText = this.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex, currentColIndex).toString();
            else
                formulaText = this.parentObject.getValueRowCol(this.getSheetID(this.grid)+1, currentRowIndex, currentColIndex).toString();
            var startRowIndex = 1, startColIndex = 1;
            for (var x = 1; x <= arrayHeight; x++) {
                if ((currentColIndex - x) > 0) {
                    var tempCell = this._getSheetTokenFromReference(this.cell) + RangeInfo.getAlphaLabel(currentColIndex - x) + currentRowIndex.toString();
                    var tempFormulaText = "";
                    var formula1 = this.getFormulaInfoTable().containsKey(tempCell) ? this.getFormulaInfoTable().getItem(tempCell) : null;
                    if (formula1 != null)
                        tempFormulaText = formula1.getFormulaText();
                    else if (this.parentObject.getValueRowCol == undefined)
                        tempFormulaText = this.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex, currentColIndex - x).toString();
                    else
                        tempFormulaText = this.parentObject.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex, currentColIndex - x).toString();
                    startColIndex = currentColIndex - x;
                    if (formulaText != null && tempFormulaText != formulaText) {
                        startColIndex++;
                        break;
                    }
                }
            }
            for (var y = 1; y <= arrayWidth; y++) {
                if ((currentRowIndex - y) > 0) {
                    var tempCell = this._getSheetTokenFromReference(this.cell) + RangeInfo.getAlphaLabel(startColIndex) + (currentRowIndex - y).toString();
                    var tempFormulaText = "";
                    var formula1 = this.getFormulaInfoTable().containsKey(tempCell) ? this.getFormulaInfoTable().getItem(tempCell) : null;
                    if (formula1 != null)
                        tempFormulaText = formula1.getFormulaText();
                    else if (this.parentObject.getValueRowCol == undefined)
                        tempFormulaText = this.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex - y, startColIndex).toString();
                    else
                        tempFormulaText = this.parentObject.getValueRowCol(this.getSheetID(this.grid) + 1, currentRowIndex - y, startColIndex).toString();
                    startRowIndex = currentRowIndex - y;
                    if (formulaText != null && tempFormulaText != formulaText) {
                        startRowIndex++;
                        break;
                    }
                }
            }

            this._getFormulaArrayBoundsfirstRowIndex = startRowIndex;
            this._getFormulaArrayBoundsfirstColIndex = startColIndex;
            this._getFormulaArrayBoundslastRowIndex = startRowIndex + arrayHeight;
            this._getFormulaArrayBoundslastColIndex = startColIndex + arrayWidth;
            return true;
        };
        /**
         * 
		 * @private
        */
		this._getSerialDateFromDate = function (y, m, d) {
            var days = 0;
            if (y < 1900) {
                y += 1900;
            }

           
            var isValidMonth = false;
            while (!isValidMonth) {
                while (m > 12) {
                    y++;
                    m -= 12;
                }
                isValidMonth = true;
                var x = new Date(y, m, 1, -1).getDate();
                while (d > x) {
                    x = new Date(y, m, 1, -1).getDate();
                    m++;
                    d -= x;
                    isValidMonth = false;
                }
                if (d < 1) {
                    m--;
                    x = new Date(y, m, 1, -1).getDate();
                    d = x - d;
                }
            }
            var dateTime = Date.parse(y.toString() + this.getParseDateTimeSeparator() + m.toString() + this.getParseDateTimeSeparator() + d.toString());
            if (!isNaN(dateTime)) {
                days = this._toOADate(new Date(dateTime));
            }
            if (this.treat1900AsLeapYear && days > 59) {
                days += 1;
            }
            return days;
        };
        /**
         * 
		 * @private
        */
		this._getSerialDateTimeFromDate = function (dt) {
            var d = this._toOADate(dt);

            if (this.treat1900AsLeapYear && d > 59) {
                d += 1;
            }

            return d;
        };
        /**
         * 
		 * @private
        */
		this._getSheetTokenFromReference = function (r) {
            var sheet = "";
            if (r.length > 2 && r[0] == this.sheetToken) {
                var i = 1;
                sheet = this.sheetToken;
                while (i < r.length && r[i] != this.sheetToken) {
                    sheet += r[i];
                    i++;
                }

                sheet += this.sheetToken;
            }

            return sheet;
        };
        /**
         * 
		 * @private
        */
		this._getValueComputeFormulaIfNecessary = function (row, col, grd) {
            var alreadyComputed = false;

            var formula = this.getFormulaInfoTable().getItem(this.cell);
            var o;
            if (this.parentObject.getValueRowCol == undefined)
                o = this.getValueRowCol(this.getSheetID(grd) + 1, row, col);
            else
                o = this.parentObject.getValueRowCol(this.getSheetID(grd) + 1, row, col);

            var val = (o != null && !(o.Length === 0)) ? o.toString() : "";

            if (val[val.length - 1] == ("}") && val[0] == ("{")) {
                val = this._substring(val, 1, val.length - 2);
            }

            if ((val == "" && (formula == null || formula == undefined)) || (val != "" && val[0] != this.getFormulaCharacter() && val[val.length - 1] != "%")) {
                if (formula != null && val == formula.getFormulaValue())
                    return formula.getFormulaValue();
                else
                    return val;
            }

            if (val.length > 0 && val[0] == this.getFormulaCharacter()) {
                if (formula != null) {
                    ////will be set later..
                    //// val = formula.FormulaValue;
                    ////val = ComputeFormula(formula.ParsedFormula);
                } else {
                    formula = new FormulaInfo();
                    formula.setFormulaText(o.toString());

                    if (!this.getDependentFormulaCells().containsKey(this.cell)) {
                        this.getDependentFormulaCells().add(this.cell, new HashTable());
                    }

                    var compute = true;
                    try  {
                        formula.setParsedFormula(this.parseFormula(val));
                    } catch (ex) {
                        if (this._inAPull) {
                            val = ex;
                            formula = null;
                        } else {
                            formula.setFormulaValue(ex);
                        }
                        compute = false;
                    }

                    if (compute) {
                        formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                        alreadyComputed = true;
                    }
                    if (formula != null) {
                        formula.calcID = this._calcID;
                        if (!this.getFormulaInfoTable().containsKey(this.cell))
                            this.getFormulaInfoTable().add(this.cell, formula);
                        val = formula.getFormulaValue() != null ? formula.getFormulaValue() : "";
                    }
                }
            }

            if (formula != null) {
                if (this.getUseFormulaValues() || this.getCalculatingSuspended() && (!this._inAPull || alreadyComputed)) {
                    val = formula.getFormulaValue() != null ? formula.getFormulaValue() : "";
                } else if (!alreadyComputed) {
                    if (this._calcID == formula.calcID) {
                        val = formula.getFormulaValue();
                    } else {
                        val = this.computeFormula(formula.getParsedFormula());
                        formula.setFormulaValue(val);
                        formula.calcID = this._calcID;
                    }
                }
                if (this.getTreatStringsAsZero() && val == "") {
                    return "0";
                }
            }

            if (val == "" || val == undefined) {
                val = "";
            }

            var d = this._parseDouble(val.substring(0, val.length - 1));

            if (val[val.length - 1] == ("%") && !isNaN(d)) {
                val = (Number(d) / 100).toString();
            }
            return val;
        };
        /**
         * 
		 * @private
        */
		this._handleEmbeddedEs = function (text) {
            var j = 0;
            while (j > -1 && ((j = text.indexOf(this._string_EP, j)) > -1)) {
                if (this._notInBlock(text, j)) {
                    var left = j;
                    while (left > 0 && (this._isDigit(text[left - 1]) || text[left - 1] == this.getParseDecimalSeparator())) {
                        left--;
                    }

                    if (left != j && (left == 0 || !this._isUpperChar(text[left - 1]))) {
                        var right = j + this._string_EP.length;
                        while (right < text.length && this._isDigit(text[right])) {
                            right++;
                        }

                        if (right != j + this._string_EP.length) {
                            text = text.substring(0, j) + this.char_EP + this._substring(text, j + this._string_EP.length);
                        }
                    }
                }

                j += 1;
            }

            j = 0;
            while (j > -1 && ((j = text.indexOf(this._string_EM, j)) > -1)) {
                if (this._notInBlock(text, j)) {
                    var left = j;
                    while (left > 0 && (this._isDigit(text[left - 1]) || text[left - 1] == this.getParseDecimalSeparator())) {
                        left--;
                    }

                    if (left != j && (left == 0 || !this._isUpperChar(text[left - 1]))) {
                        var right = j + this._string_EM.length;
                        while (right < text.length && this._isDigit(text[right])) {
                            right++;
                        }

                        if (right != j + this._string_EM.length) {
                            text = text.substring(0, j) + this.char_EM + this._substring(text, j + this._string_EM.length);
                        }
                    }
                }

                j += 1;
            }

            j = 0;

            while (j > -1 && ((j = text.indexOf(this._string_E, j)) > -1) && text[0] != this.bMARKER) {
                if (this._notInBlock(text, j)) {
                    var left = j;
                    while (left > 0 && (this._isDigit(text[left - 1]) || text[left - 1] == this.getParseDecimalSeparator())) {
                        left--;
                    }

                    if (left != j && (left == 0 || !this._isUpperChar(text[left - 1]))) {
                        var right = j + this._string_E.length;
                        while (right < text.length && this._isDigit(text[right])) {
                            right++;
                        }

                        if (right != j + this._string_E.length && (left == -1 || !this._isUpperChar(text[left]))) {
                            text = text.substring(0, j) + this.char_EP + this._substring(text, j + this._string_E.length);
                        }
                    }
                }

                j += 1;
            }

            return text;
        };
        /**
         * 
		 * @private
        */
		this._handleIterations = function (formula) {
            this._inHandleIterations = true;
            var oldValue = 0;
            var d = 0;
            var count = 1;
            var first = true;
            while (count < this._iterationMaxCount - 1 && (first || !this._checkUnderTolerance(d, oldValue))) {
                first = false;
                if (!this.getIterationValues().containsKey(this.cell)) {
                    this.getIterationValues().add(this.cell, "0");
                }

                this.getIterationValues().add(this.cell, (formula.getFormulaValue() == "" ? "0" : formula.getFormulaValue()));

                formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                oldValue = d;
                d = this._parseDouble(formula.getFormulaValue());
                if (isNaN(d))
                    d = 0;
                count++;
            }

            ////save the last iteration
            this.getIterationValues.getItem(this.cell, (formula.getFormulaValue() == "" ? "0" : formula.getFormulaValue()));

            this._inHandleIterations = false;
        };
        /**
         * 
		 * @private
        */
		this._indexOfAny = function (text, operators) {
            for (var i = 0; i < text.length; i++) {
                if (operators.indexOf(text[i]) > -1) {
                    return i;
                }
            }
            return -1;
        };
        /**
         * 
		 * @private
        */
		this._initLibraryFunctions = function () {
            this._libraryFunctions = new HashTable();

            ////Hand-coded formulas.
            this._addFunction("SUM", "computeSum");
            this._addFunction("EncodeURL", "computeEncodeURL");

            
            //TEXT FORMULA
            this._addFunction("CHAR", "computeChar");
            this._addFunction("CODE", "computeCode");
            this._addFunction("UNICODE", "computeUniCode");
            this._addFunction("UNICHAR", "computeUniChar");
            this._addFunction("UPPER", "computeUpper");
            this._addFunction("LOWER", "computeLower");
            this._addFunction("LEN", "computeLen");
            this._addFunction("MID", "computeMid");
            this._addFunction("LEFT", "computeLeft");
            this._addFunction("CLEAN", "computeClean");
            this._addFunction("REPT", "computeRept");
            this._addFunction("RIGHT", "computeRight");
            this._addFunction("REPLACE", "computeReplace");
            this._addFunction("EXACT", "computeExact");
            this._addFunction("FIND", "computeFind");
            this._addFunction("TRIM", "computeTrim");
            this._addFunction("SEARCH", "computeSearch");
            this._addFunction("SUBSTITUTE", "computeSubstitute");
            this._addFunction("PROPER", "computeProper");
            this._addFunction("T", "computeT");
            this._addFunction("NUMBERVALUE", "computeNumberValue");
            this._addFunction("CONCATENATE", "computeConcatenate");
            this._addFunction("VALUE", "computeValue");
            this._addFunction("DOLLAR", "computeDollar");
            this._addFunction("FIXED", "computeFixed");

            //Engineering formulas
            this._addFunction("BIN2DEC", "computeBin2Dec");
            this._addFunction("BIN2OCT", "computeBin2Oct");
            this._addFunction("BIN2HEX", "computeBin2Hex");
            this._addFunction("DEC2BIN", "computeDec2Bin");
            this._addFunction("DEC2OCT", "computeDec2Oct");
            this._addFunction("HEX2BIN", "computeHex2Bin");
            this._addFunction("HEX2OCT", "computeHex2Oct");

            // DATE and TIME
            this._addFunction("DATE", "computeDate");
            this._addFunction("DATEVALUE", "computeDatevalue");
            this._addFunction("DAY", "computeDay");
            this._addFunction("DAYS", "computeDays");
            this._addFunction("DAYS360", "computeDays360");
            this._addFunction("EDATE", "computeEDate");
            this._addFunction("EOMONTH", "computeEOMonth");
            this._addFunction("HOUR", "computeHour");
            this._addFunction("ISOWEEKNUM", "computeISOWeeknum");
            this._addFunction("MINUTE", "computeMinute");
            this._addFunction("MONTH", "computeMonth");
            this._addFunction("NETWORKDAYS", "computeNetworkDays");
            this._addFunction("NETWORKDAYS.INTL", "computeNetworkDaysOintl");
            this._addFunction("NOW", "computeNow");
            this._addFunction("SECOND", "computeSecond");
            this._addFunction("TIME", "computeTime");
            this._addFunction("TIMEVALUE", "computeTimevalue");
            this._addFunction("TODAY", "computeToday");
            this._addFunction("WEEKDAY", "computeWeekday");
            this._addFunction("WEEKNUM", "computeWeeknum");
            this._addFunction("WORKDAY", "computeWorkDay");
            this._addFunction("WORKDAY.INTL", "computeWorkDayOintl");
            this._addFunction("YEAR", "computeYear");

            //look and ref
            this._addFunction("ADDRESS", "computeAddress");
            this._addFunction("AREAS", "computeAreas");
            this._addFunction("CHOOSE", "computeChoose");
            this._addFunction("COLUMN", "computeColumn");
            this._addFunction("COLUMNS", "computeColumns");
            this._addFunction("FORMULATEXT", "computeFormulaText");
            this._addFunction("HYPERLINK", "computeHyperlink");
            this._addFunction("HLOOKUP", "computeHLookUp");
            this._addFunction("INDEX", "computeIndex");
            this._addFunction("INDIRECT", "computeIndirect");
            this._addFunction("LOOKUP", "computeLookUp");
            this._addFunction("OFFSET", "computeOffSet");
            this._addFunction("TRANSPOSE", "computeTranspose");



            // Statics
            this._addFunction("LOGNORM.INV", "computeLognormOinv");
            this._addFunction("NORM.INV", "computeNormOinv");
            this._addFunction("NORM.DIST", "computeNormOdist");
            this._addFunction("NORM.S.DIST", "computeNormOsODist");
            this._addFunction("NORM.S.INV", "computeNormOsOInv");
            this._addFunction("PERMUT", "computePermut");
            this._addFunction("PERMUTATIONA", "computePermutationA");
            this._addFunction("STANDARDIZE", "computeStandardize");
            this._addFunction("BINOM.DIST", "computeBinomOdist");
            this._addFunction("CHISQ.INV.RT", "computeChisqOinvOrt");
            this._addFunction("CHISQ.INV", "computeChisqOinv");
            this._addFunction("CHISQ.DIST.RT", "computeChisqOdistOrt");
            this._addFunction("F.DIST", "computeFOdist");
            this._addFunction("GAMMALN", "computeGammaln");
            this._addFunction("CONFIDENCE.NORM", "computeConfidenceOnorm");
            this._addFunction("EXPON.DIST", "computeExponOdist");
            this._addFunction("FISHER", "computeFisher");
            this._addFunction("FISHERINV", "computeFisherInv");
            this._addFunction("GAMMALN.PRECISE", "computeGammalnOPrecise");
            this._addFunction("AVERAGE", "computeAverage");
            this._addFunction("AVERAGEA", "computeAverageA");
            this._addFunction("POISSON.DIST", "computePoissonODist");
            this._addFunction("WEIBULL.DIST", "computeWeiBullODist");
            this._addFunction("F.INV.RT", "computeFOinvOrt");
            this._addFunction("T.DIST", "computeTOdist");
            this._addFunction("MAX", "computeMax");
            this._addFunction("MAXA", "computeMaxa");
            this._addFunction("MEDIAN", "computeMedian");
            this._addFunction("MIN", "computeMin");
            this._addFunction("MINA", "computeMina");
            this._addFunction("PERCENTRANK.INC", "computePercentrankInc");
            this._addFunction("PERCENTILE", "computePercentile");
            this._addFunction("RANK.EQ", "computeRankOEq");
            this._addFunction("COUNT", "computeCount");
            this._addFunction("COUNTA", "computeCounta");
            this._addFunction("DEVSQ", "computeDevsq");
            this._addFunction("F.DIST.RT", "computeFOdistORt");
            this._addFunction("FORECAST", "computeForecast");
            this._addFunction("GEOMEAN", "computeGeomean");
            this._addFunction("HARMEAN", "computeHarmean");
            this._addFunction("INTERCEPT", "computeIntercept");
            this._addFunction("LARGE", "computeLarge");
            this._addFunction("SMALL", "computeSmall");
            this._addFunction("LOGNORM.DIST", "computeLognormOdist");          
            this._addFunction("AVEDEV", "computeAvedev");
            this._addFunction("COUNTBLANK", "computeCountblank"); 
            this._addFunction("STDEV.P", "computeStdevOp");
            this._addFunction("STDEV.S", "computeStdevOS");
            this._addFunction("STDEVA", "computeStdeva");
            this._addFunction("STDEVPA", "computeStdevpa");
            this._addFunction("T.INV", "computeTOinv");
            this._addFunction("VAR.P", "computeVarp");
            this._addFunction("VARA", "computeVara");
            this._addFunction("VARPA", "computeVarpa");
            this._addFunction("CORREL","computeCorrel");
            //this._addFunction("NEGBINOM.DIST", "computeNegbinomODist");
            this._addFunction("PERCENTILE.EXC", "computePercentileExc");
            this._addFunction("PERCENTILE.INC", "computePercentileOInc");
            this._addFunction("TRIMMEAN", "computeTrimmean");
            this._addFunction("RSQ", "computeRsq");
            this._addFunction("PEARSON", "computePearson");
            //60 statics function
            

            // math and Trig
            this._addFunction("ABS", "computeAbs");
            this._addFunction("ACOS", "computeAcos");
            this._addFunction("ACOSH", "computeAcosh");
            this._addFunction("ACOT", "computeAcot");
            this._addFunction("ACOTH", "computeAcoth");
            this._addFunction("ARABIC", "computeArabic");
            this._addFunction("ASIN", "computeAsin");
            this._addFunction("ATAN", "computeAtan");
            this._addFunction("ATAN2", "computeAtan2");
            this._addFunction("CEILING.MATH", "computeCeilingMath");
            this._addFunction("CEILING", "computeCeiling");
            this._addFunction("COMBIN", "computeCombin");
            this._addFunction("COMBINA", "computeCombinA");
            this._addFunction("COS", "computeCos");
            this._addFunction("COSH", "computeCosh");
            this._addFunction("COT", "computeCot");
            this._addFunction("CSC", "computeCsc");
            this._addFunction("CSCH", "computeCsch");
            this._addFunction("DECIMAL", "computeDecimal");
            this._addFunction("DEGREES", "computeDegrees");
            this._addFunction("ISTEXT", "computeIsText");
            this._addFunction("EXP", "computeExp");
            this._addFunction("EVEN", "computeEven");
            this._addFunction("FACT", "computeFact");
            this._addFunction("FACTDOUBLE", "computeFactdouble");
            this._addFunction("FLOOR", "computeFloor");
            this._addFunction("INT", "computeInt");
            this._addFunction("LN", "computeLn");
            this._addFunction("LOG", "computeLog");
            this._addFunction("PI", "computePI");
            this._addFunction("PRODUCT", "computeProduct");
            this._addFunction("SEC", "computeSecant");
            this._addFunction("SERIESSUM", "computeSeriessum");
            this._addFunction("SIN", "computeSin");
            this._addFunction("SINH", "computeSinh");
            this._addFunction("SQRT", "computeSqrt");
            this._addFunction("SUBTOTAL", "computeSubTotal");
            this._addFunction("SUMIF", "computeSumif");
            this._addFunction("TRUNC", "computeTrunc");
            this._addFunction("TAN", "computeTan");
            this._addFunction("LOG10", "computeLogTen");

            //Logic Function

            this._addFunction("AND", "computeAnd");
            this._addFunction("FALSE", "computeFalse");
            this._addFunction("IF", "computeIf");
            this._addFunction("IFERROR", "computeIfError");
           // this._addFunction("IFNA", "computeIfNA");
            this._addFunction("NOT", "computeNot");
            this._addFunction("OR", "computeOr");
            this._addFunction("TRUE", "computeTrue");
            this._addFunction("XOR", "computeXor");

            // Information Function

            this._addFunction("CELL", "computeCell");
            this._addFunction("ERROR.TYPE", "computeErrorType");
            this._addFunction("INFO", "computeInfo");
            this._addFunction("ISBLANK", "computeIsBlank");
            this._addFunction("ISERR", "computeIsErr");
            this._addFunction("ISERROR", "computeIsError");
            this._addFunction("ISEVEN", "computeIsEven");
            this._addFunction("ISFORMULA", "computeIsFormula");
            this._addFunction("ISLOGICAL", "computeIsLogical");
            this._addFunction("ISNA", "computeIsNA");
            this._addFunction("ISNONTEXT", "computeIsNonText");
            this._addFunction("ISNUMBER", "computeIsNumber");
            this._addFunction("ISODD", "computeIsOdd");
            this._addFunction("ISREF", "computeIsRef");
            this._addFunction("ISTEXT", "computeIsText");
            this._addFunction("N", "computeN");
            this._addFunction("NA", "computeNA");
            this._addFunction("SHEET", "computeSheet");
            this._addFunction("SHEETS", "computeSheets");
            this._addFunction("TYPE", "computeType");
            this._addFunction("ROW", "computeRow");
            this._addFunction("ROWS", "computeRows");

        };
        /**
         * 
		 * @private
        */
		this._comb = function (k, n) {
            var top = 1;
            for (var i = k + 1; i <= n; ++i) {
                top = top * i;
            }

            var bottom = 1;
            for (var i = 2; i <= (n - k) ; ++i) {
                bottom = bottom * i;
            }

            return top / bottom;
        };
        /**
         * 
		 * @private
        */
		this._finv = function (p, df1, df2) {
            var mult = Math.exp(this._gammaln((df1 + df2) / 2) - this._gammaln(df1 / 2) - this._gammaln(df2 / 2) + (df1 / 2) * Math.log(df1 / df2));

            var guess = mult;

            var lastIncrement = guess / 2;
            var gi = 0;
            var eps = 1e-7;
            var k = 100;
            var tries = 3;
            var its = 0;
            while (k == 100 && tries > 0) {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k) {
                    its++;
                    gi = 1 - mult * this._fdist(guess, df1, df2);

                    ////Console.WriteLine("{0}: guess {1}  computed {2}  target {3}", its, guess, gi, p);
                    if (Math.abs((gi - p) / p) < eps) {
                        break;
                    }

                    ////greater since looking for 1 - xxxx
                    if (gi > p) {
                        guess = guess + lastIncrement;
                    } else {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0) {
                            lastIncrement = guess / 2;
                        }

                        guess = guess - lastIncrement;
                    }
                }
            }

            if (k == 100) {
                guess = -1;
            }

            return guess;
        };
        /**
         * 
		 * @private
        */
		this._tProbabilityDensity = function (x, k) {
            var a = this._gammaFunction(0.5 * k + 0.5);
            var b = Math.pow(1.0 + (x * x) / k, -0.5 * k - 0.5);
            var c = Math.sqrt(k * Math.PI) * this._gammaFunction(0.5 * k);
            return a * b / c;
        };
        /**
         * 
		 * @private
        */
		this._tCumulativeDensity = function (x, k) {
            if (isNaN(x))
                return NaN;
            if (x == 0)
                return 0.5;
            if (x > 0) {
                var x2 = k / (x * x + k);
                return 1.0 - 0.5 * this._rIBetaFunction(x2, 0.5 * k, 0.5);
            }
            return 1.0 - this._tCumulativeDensity(-x, k);
        };
        /**
         * 
		 * @private
        */
		this._sign = function (p) {
            if (p === 0 || isNaN(p))
                return p;
            return p > 0 ? 1 : -1;
        };
        /**
         * 
		 * @private
        */
		this._var = function (x) {
            var sumx = 0;
            var n = x.length;
            for (var i = 0; i < n; ++i) {
                sumx += x[i];
            }

            sumx = sumx / n;

            var sumx2 = 0;
            var d;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumx2 += d * d;
            }

            return sumx2 / (n - 1);
        };
        /**
         * 
		 * @private
        */
		this._tCumulativeDistributionInverse = function (p, k) {
            var maxIterations= 50;
            var minAccuracy = 0.0001;

            if (p < 0 || p > 1.0)
                throw "Probability must be between 0 and 1";
            if (p == 0)
                return Number.NEGATIVE_INFINITY;
            if (p == 1)
                return Number.POSITIVE_INFINITY;
            if (p == 0.5)
                return 0.0;

            switch (k) {
                case 1:
                    return Math.tan(Math.PI * (p - 0.5));
                case 2:
                    var alpha = 4 * p * (1 - p);
                    return 2 * (p - 0.5) * Math.sqrt(2.0 / alpha);
                case 4:
                    var alpha = 4 * p * (1 - p);
                    var sqrtAlpha = Math.sqrt(alpha);
                    var q = Math.cos((1.0 / 3.0) * Math.acos(sqrtAlpha)) / sqrtAlpha;
                    return this._sign(p - 0.5) * 2 * Math.sqrt(q - 1);
                default:
                    var x;
                    if (k > 6)
                        x = this._standardNormalCumulativeDistributionFunctionInverse(p);
                    else
                        x = this._tCumulativeDistributionInverse(p, 4);

                    for (var i = 0; i < maxIterations; i++) {
                        var p0 = this._tCumulativeDensity(x, k);
                        var absAccuracy = Math.abs(p - p0);
                        if (absAccuracy < minAccuracy)
                            return x;
                        var slope = this._tProbabilityDensity(x, k);
                        x = x + (p - p0) / slope;
                    }
                    throw "Solution did not converge";
            }
        };
        /**
         * 
		 * @private
        */
		this._fdist = function (x, df1, df2) {
            var nPanels = 32;
            var leftside = 0;
            var rightside = x;

            var h = (rightside - leftside) / nPanels;
            var mult = h / 3;

            var sum1 = this._fdensity(leftside, df1, df2) + this._fdensity(rightside, df1, df2);
            var sum4 = 0;
            for (var i = 1; i < nPanels; i += 2) {
                sum4 = sum4 + 4 * this._fdensity(leftside + i * h, df1, df2);
            }

            var sum2 = 0;
            for (var i = 2; i < nPanels; i += 2) {
                sum2 = sum2 + 2 * this._fdensity(leftside + i * h, df1, df2);
            }

            var gd = mult * (sum1 + sum2 + sum4);
            var oldgd = gd;
            var eps = 1e-7;

            var k = 0;

            for (k = 0; k < 10; ++k) {
                nPanels = nPanels * 2;

                sum2 += sum4 / 2;
                sum4 = 0;
                h = (rightside - leftside) / nPanels;
                for (var i = 0; i < nPanels; ++i) {
                    if (i % 2 == 1) {
                        var d = this._fdensity(leftside + h * i, df1, df2);
                        sum4 = sum4 + d;
                    }
                }

                sum4 = 4 * sum4;
                mult = h / 3;
                gd = mult * (sum1 + sum2 + sum4);

                if (Math.abs((gd - oldgd) / oldgd) < eps) {
                    break;
                }

                oldgd = gd;
            }

            return gd;
        };
        /**
         * 
		 * @private
        */
		this._fdensity = function (x, df1, df2) {
            return Math.pow(x, (df1 - 2) / 2) / Math.pow(1 + df1 * x / df2, (df1 + df2) / 2);
        };
        /**
         * 
		 * @private
        */
		this._normaldist = function (x, u, s) {
            var nPanels = 32;
            var leftside;
            var rightside;
            if (x > u) {
                leftside = u - (x - u);
                rightside = x;
            } else {
                leftside = x;
                rightside = u + (u - x);
            }

            var h = (rightside - leftside) / nPanels;
            var mult = h / 3;

            var sum1 = this._normaldensity(leftside, u, s) + this._normaldensity(rightside, u, s);
            var sum4 = 0;
            for (var i = 1; i < nPanels; i += 2) {
                sum4 = sum4 + 4 * this._normaldensity(leftside + i * h, u, s);
            }

            var sum2 = 0;
            for (var i = 2; i < nPanels; i += 2) {
                sum2 = sum2 + 2 * this._normaldensity(leftside + i * h, u, s);
            }

            var gd = mult * (sum1 + sum2 + sum4);
            var oldgd = gd;
            var eps = 1e-7;

            var k = 0;

            for (k = 0; k < 10; ++k) {
                nPanels *= 2;

                sum2 = sum2 + sum4 / 2;
                sum4 = 0;
                h = h = (rightside - leftside) / nPanels;
                for (var i = 0; i < nPanels; ++i) {
                    if (i % 2 == 1) {
                        var d = this._normaldensity(leftside + h * i, u, s);
                        sum4 += d;
                    }
                }

                sum4 = 4 * sum4;
                mult = h / 3;
                gd = mult * (sum1 + sum2 + sum4);

                if (Math.abs((gd - oldgd) / oldgd) < eps) {
                    break;
                }

                oldgd = gd;
            }

            if (x > u) {
                ////Add the left tails.
                gd = gd + (1 - gd) / 2;
            } else {
                ////Return only the left tail.
                gd = (1 - gd) / 2;
            }

            ////Console.WriteLine(k.toString() + "  " + gd.toString());
            return gd;
        };
        /**
         * 
		 * @private
        */
		this._normaldensity = function (x, u, s) {
            return 1 / (Math.sqrt(2 * Math.PI) * s) * Math.exp(-(x - u) * (x - u) / (2 * s * s));
        };
        /**
         * 
		 * @private
        */
		this._standardNormalCumulativeDistribution = function (x) {
            //Approimation based on Abramowitz & Stegun (1964)
            if (x < 0)
                return 1.0 - this._standardNormalCumulativeDistribution(-x);
            var b0 = 0.2316419;
            var b1 = 0.319381530;
            var b2 = -0.356563782;
            var b3 = 1.781477937;
            var b4 = -1.821255978;
            var b5 = 1.330274429;
            var pdf = this._standardNormalProbabilityDensityFunction(x);
            var a = 1.0 / (1.0 + b0 * x);
            return 1.0 - pdf * (b1 * a + b2 * Math.pow(a, 2) + b3 * Math.pow(a, 3) + b4 * Math.pow(a, 4) + b5 * Math.pow(a, 5));
        };
        /**
         * 
		 * @private
        */
		this._normalinv = function (p, u, s) {
            var guess = u;
            if (p < .05) {
                guess = u - 2 * s;
            } else if (p < .5) {
                guess = u;
            } else if (p < .95) {
                guess = u + 2 * s;
            } else {
                guess = u + 5 * s;
            }

            var lastIncrement = guess / 2;
            var gi = 0;
            var eps = 1e-7;
            var k = 100;
            var tries = 3;
            var its = 0;
            while (k == 100 && tries > 0) {
                tries--;
                guess = guess / 2;
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k) {
                    its++;
                    gi = this._normaldist(guess, u, s);
                    if (Math.abs((gi - p) / p) < eps) {
                        break;
                    }

                    if (gi < p) {
                        guess = guess + lastIncrement;
                    } else {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0) {
                            lastIncrement = guess / 2;
                        }

                        guess = guess - lastIncrement;
                    }
                }
            }

            if (k == 100) {
                guess = -1;
            }

            return guess;
        };
		/**
         * 
		 * @private
        */
		this._normalCumulativeDistributionFunctionInverse = function (p, mean, standardDeviation) {
            var x = this._standardNormalCumulativeDistributionInverse(p);
            return standardDeviation * x + mean;
        };
        /**
         * 
		 * @private
        */
		this._standardNormalCumulativeDistributionInverse = function (p) {
            if (p < 0 || p > 1.0)
                throw "Probability must be between 0 and 1";
            if (p == 0)
                return Number.NEGATIVE_INFINITY;
            if (p == 1)
                return Number.POSITIVE_INFINITY;
            if (p == 0.5)
                return 0.0;

            var a1 = -3.969683028665376e+01;
            var a2 = 2.209460984245205e+02;
            var a3 = -2.759285104469687e+02;
            var a4 = 1.383577518672690e+02;
            var a5 = -3.066479806614716e+01;
            var a6 = 2.506628277459239e+00;

            var b1 = -5.447609879822406e+01;
            var b2 = 1.615858368580409e+02;
            var b3 = -1.556989798598866e+02;
            var b4 = 6.680131188771972e+01;
            var b5 = -1.328068155288572e+01;

            var c1 = -7.784894002430293e-03;
            var c2 = -3.223964580411365e-01;
            var c3 = -2.400758277161838e+00;
            var c4 = -2.549732539343734e+00;
            var c5 = 4.374664141464968e+00;
            var c6 = 2.938163982698783e+00;

            var d1 = 7.784695709041462e-03;
            var d2 = 3.224671290700398e-01;
            var d3 = 2.445134137142996e+00;
            var d4 = 3.754408661907416e+00;

            var pLow = 0.02425;
            var pHigh = 1 - pLow;

            var q, x;
            if (0 < p && p < pLow) {
                q = Math.sqrt(-2 * Math.log(p));
                x = (((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
                return x;
            }

            if (pLow <= p && p <= pHigh) {
                q = p - 0.5;
                var r = q * q;
                x = (((((a1 * r + a2) * r + a3) * r + a4) * r + a5) * r + a6) * q / (((((b1 * r + b2) * r + b3) * r + b4) * r + b5) * r + 1);
                return x;
            }

            //if(p_high < p && p < 1)
            q = Math.sqrt(-2 * Math.log(1 - p));
            x = -(((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
            return x;
        };
        /**
         * 
		 * @private
        */
		this._normalProbabilityDensity = function (x, mean, standardDeviation) {
            var sqrtTwoPiInv = 0.398942280401433;
            var z = (x - mean) / standardDeviation;
            return sqrtTwoPiInv * Math.exp(-0.5 * z * z) / standardDeviation;
        };
        /**
         * 
		 * @private
        */
		this._standardNormalProbabilityDensityFunction = function (x) {
            var SqrtTwoPiInv = 0.398942280401433;
            return SqrtTwoPiInv * Math.exp(-0.5 * x * x);
        };
        /**
         * 
		 * @private
        */
		this._standardNormalCumulativeDistributionFunction = function (x) {
            if (x < 0)
                return 1.0 - this._standardNormalCumulativeDistributionFunction(-x);
            var b0 = 0.2316419;
            var b1 = 0.319381530;
            var b2 = -0.356563782;
            var b3 = 1.781477937;
            var b4 = -1.821255978;
            var b5 = 1.330274429;
            var pdf = this._standardNormalProbabilityDensityFunction(x);
            var a = 1.0 / (1.0 + b0 * x);
            return 1.0 - pdf * (b1 * a + b2 * Math.pow(a, 2) + b3 * Math.pow(a, 3) + b4 * Math.pow(a, 4) + b5 * Math.pow(a, 5));
        };
        /**
         * 
		 * @private
        */
		this._standardNormalCumulativeDistributionFunctionInverse = function (p) {
            if (p < 0 || p > 1.0)
                throw "Probability must be between 0 and 1";
            if (p == 0)
                return Number.NEGATIVE_INFINITY;
            if (p == 1)
                return Number.POSITIVE_INFINITY;
            if (p == 0.5)
                return 0.0;

            var a1 = -3.969683028665376e+01;
            var a2 = 2.209460984245205e+02;
            var a3 = -2.759285104469687e+02;
            var a4 = 1.383577518672690e+02;
            var a5 = -3.066479806614716e+01;
            var a6 = 2.506628277459239e+00;

            var b1 = -5.447609879822406e+01;
            var b2 = 1.615858368580409e+02;
            var b3 = -1.556989798598866e+02;
            var b4 = 6.680131188771972e+01;
            var b5 = -1.328068155288572e+01;

            var c1 = -7.784894002430293e-03;
            var c2 = -3.223964580411365e-01;
            var c3 = -2.400758277161838e+00;
            var c4 = -2.549732539343734e+00;
            var c5 = 4.374664141464968e+00;
            var c6 = 2.938163982698783e+00;

            var d1 = 7.784695709041462e-03;
            var d2 = 3.224671290700398e-01;
            var d3 = 2.445134137142996e+00;
            var d4 = 3.754408661907416e+00;

            var pLow = 0.02425;
            var pHigh = 1 - pLow;

            var q, x;
            if (0 < p && p < pLow) {
                q = Math.sqrt(-2 * Math.log(p));
                x = (((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
                return x;
            }

            if (pLow <= p && p <= pHigh) {
                q = p - 0.5;
                var r = q * q;
                x = (((((a1 * r + a2) * r + a3) * r + a4) * r + a5) * r + a6) * q / (((((b1 * r + b2) * r + b3) * r + b4) * r + b5) * r + 1);
                return x;
            }

            //if(p_high < p && p < 1)
            q = Math.sqrt(-2 * Math.log(1 - p));
            x = -(((((c1 * q + c2) * q + c3) * q + c4) * q + c5) * q + c6) / ((((d1 * q + d2) * q + d3) * q + d4) * q + 1);
            return x;
        };
        /**
         * 
		 * @private
        */
		this._chiSquaredProbabilityDensityFunction = function (x, k) {
            var g = this._gammaFunction(0.5 * k);
            var a = 1.0 / (Math.pow(2, 0.5 * k) * g);
            return a * Math.pow(x, 0.5 * k - 1.0) * Math.exp(-0.5 * x);
        };
        /**
         * 
		 * @private
        */
		this._binomdist = function (trials, successes, p) {
            var pPow = 1;
            var pm1 = 1 - p;

            var p1Pow = Math.pow(pm1, trials);

            var dist = 0;
            var cbn = pPow * Math.pow(pm1, trials);
            if (cbn == 0) {
                return NaN;
            }

            for (var i = 0; i <= successes; ++i) {
                dist += cbn;
                cbn = cbn * p / pm1 * (trials - i) / (i + 1);
                if (!isFinite(cbn) || isNaN(cbn)) {
                    dist = NaN;
                    break;
                }
            }

            return dist;
        };
        /**
         * 
		 * @private
        */
		this._gammadensity = function (a, b, x) {
            //// excel version     dist = 1 / (Math.Pow(b, a) * gammaln(a)) * Math.Pow(x, a - 1) * Math.Exp(-x / b);
            //// http://www.itl.nist.gov/div898/handbook/eda/section3/eda366b.htm
            return Math.pow(x / b, a - 1) * Math.exp(-x / b) / (b * Math.exp(this._gammaln(a)));
        };
        this.minValue = -1.79769e+308;
        this.maxValue = 2147483647;
        this._charTable = [this._string_empty, "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"];
        /**
         * 
		 * @private
        */
		this._stdevdotP = function (range) {
            var n = range.length;
            var mean = 0;
            var sigma = 0;
            var count = 0;
            for (var i = 0; i < n; ++i)
                mean = mean + range[i];
            mean = mean / n;
            for (var i = 0; i < n; i++) {
                sigma = sigma + Math.pow(range[i] - mean, 2);
                count++;
            }

            return Math.sqrt(sigma / count);
        };
        /**
         * 
		 * @private
        */
		this._determinant = function (a, k) {
            var s = 1, det = 0;
            var matlength = parseInt(k.toString());
            var b = [matlength, matlength];
            var i, j, m, n, c;
            if (k == 1) {
                return (a[0, 0]);
            } else {
                det = 0;
                for (c = 0; c < k; c++) {
                    m = 0;
                    n = 0;
                    for (i = 0; i < k; i++) {
                        for (j = 0; j < k; j++) {
                            b[i, j] = 0;
                            if (i != 0 && j != c) {
                                b[m, n] = a[i, j];
                                if (n < (k - 2))
                                    n++;
                                else {
                                    n = 0;
                                    m++;
                                }
                            }
                        }
                    }
                    det = det + s * (a[0, c] * this._determinant(b, k - 1));
                    s = -1 * s;
                }
            }

            return (det);
        };
        this._factorialTable = [
            1, 1, 2, 6, 24, 120, 720,
            5040, 40320, 362880, 3628800,
            39916800, 479001600
        ];
        /**
         * 
		 * @private
        */
		this._mean = function (array) {
            return this._mean(array, 1.0);
        };
        /**
         * 
		 * @private
        */
		this._negbinomdensity = function (failures, successes, p) {
            return this._comb(successes - 1, failures + successes - 1) * Math.pow(p, successes) * Math.pow(1 - p, failures);
        };
        /**
         * 
		 * @private
        */
		this._sd = function (x, xbar) {
            var n = x.length;
            xbar = 0;
            for (var i = 0; i < n; ++i) {
                xbar += x[i];
            }

            xbar = xbar / n;
            var sumx2 = 0;
            var d = 0;

            for (var i = 0; i < n; ++i) {
                d = x[i] - xbar;
                sumx2 += d * d;
            }
            if (n == 1)
                return 0;
            return Math.sqrt(sumx2 / (n - 1));
        };
        /**
         * 
		 * @private
        */
		this._betaCumulativeDist = function (x, a, b) {
            if (x < 0 || x > 1)
                throw "x must be between 0 and 1";
            return this._rIBetaFunction(x, a, b);
        };
        /**
         * 
		 * @private
        */
		this._betaProbabilityDens = function (x, a, b) {
            if (x < 0 || x > 1)
                throw "x must be between 0 and 1";

            var B = this._betaFunction(a, b);
            var d = Math.pow(x, a - 1) * Math.pow(1 - x, b - 1) / B;
            return d;
        };
        /**
         * 
		 * @private
        */
		this._critbinom = function (nTrials, p, alpha) {
            var checkval = nTrials;
            var half = nTrials;
            var dist = 1;
            var dist1 = 1;
            do {
                half = half / 2 + 1;
                if (dist >= alpha) {
                    dist1 = this._binomdist(nTrials, checkval - 1, p);
                    if (!isNaN(dist1)) {
                        return this.maxValue;
                    }

                    if (dist1 < alpha && dist1 > 0) {
                        break;
                    }

                    checkval = checkval - half;
                } else {
                    dist1 = this._binomdist(nTrials, checkval + 1, p);
                    if (dist1 >= alpha) {
                        checkval = checkval + 1;
                        break;
                    }

                    checkval = checkval + half;
                }

                dist = this._binomdist(nTrials, checkval, p);
            } while (checkval < nTrials && checkval > 0);

            return checkval;
        };
        /**
         * 
		 * @private
        */
		this._newnormalinv = function (p) {
            var region1 = [
                -3.969683028665376e+01, 2.209460984245205e+02,
                -2.759285104469687e+02, 1.383577518672690e+02,
                -3.066479806614716e+01, 2.506628277459239e+00];

            var region2 = [
                -5.447609879822406e+01, 1.615858368580409e+02,
                -1.556989798598866e+02, 6.680131188771972e+01,
                -1.328068155288572e+01];

            var region3 = [
                -7.784894002430293e-03, -3.223964580411365e-01,
                -2.400758277161838e+00, -2.549732539343734e+00,
                4.374664141464968e+00, 2.938163982698783e+00];

            var region4 = [
                7.784695709041462e-03, 3.224671290700398e-01,
                2.445134137142996e+00, 3.754408661907416e+00];

            var pLow = 0.02425;
            var pHigh = 1 - pLow;
            var q = 0.0;
            var r = 0.0;

            // For lower region:
            if (p < pLow) {
                q = Math.sqrt(-2 * Math.log(p));
                return (((((region3[0] * q + region3[1]) * q + region3[2]) * q + region3[3]) * q + region3[4]) * q + region3[5]) / ((((region4[0] * q + region4[1]) * q + region4[2]) * q + region4[3]) * q + 1);
            }

            // For upper region:
            if (pHigh < p) {
                q = Math.sqrt(-2 * Math.log(1 - p));
                return -(((((region3[0] * q + region3[1]) * q + region3[2]) * q + region3[3]) * q + region3[4]) * q + region3[5]) / ((((region4[0] * q + region4[1]) * q + region4[2]) * q + region4[3]) * q + 1);
            }

            // For central region:
            q = p - 0.5;
            r = q * q;
            return (((((region1[0] * r + region1[1]) * r + region1[2]) * r + region1[3]) * r + region1[4]) * r + region1[5]) * q / (((((region2[0] * r + region2[1]) * r + region2[2]) * r + region2[3]) * r + region2[4]) * r + 1);
        };
        /**
         * 
		 * @private
        */
		this._chiinv = function (p, v) {
            var guess = p;

            var lastIncrement = Number(guess / 2);
            var gi = 0;
            var eps = 1e-7;
            var k = 100;
            var tries = 3;
            while (k == 100 && tries > 0) {
                tries--;
                guess = (guess / 2);
                lastIncrement = guess / 2;

                for (k = 0; k < 100; ++k) {
                    gi = 1 - this._chidist(guess, v);
                    if (Math.abs((gi - p) / p) < eps) {
                        break;
                    }

                    if (gi > p) {
                        guess = guess + lastIncrement;
                    } else {
                        lastIncrement = lastIncrement / 2;
                        if (guess - lastIncrement < 0) {
                            lastIncrement = guess / 2;
                        }

                        guess = guess - lastIncrement;
                    }
                }
            }

            if (k == 100) {
                guess = -1;
            }

            return guess;
        };
        /**
         * 
		 * @private
        */
		this._chidist = function (x, v) {
            var gauss_w = [
                0.09654008851,
                0.09563872008,
                0.09384439908,
                0.09117387870,
                0.08765209300,
                0.08331192423,
                0.07819389579,
                0.07234579411,
                0.06582222278,
                0.05868409348,
                0.05099805926,
                0.04283589802,
                0.03427386291,
                0.02539206531,
                0.01627439473,
                0.00701861001
            ];

            var gauss_x = [
                0.04830766569,
                0.14447196158,
                0.23928736225,
                0.33186860228,
                0.42135127613,
                0.50689990893,
                0.58771575724,
                0.66304426693,
                0.73218211874,
                0.79448379597,
                0.84936761373,
                0.89632115577,
                0.93490607594,
                0.96476225559,
                0.98561151155,
                0.99726386185
            ];

            var gauss_n = 16;
            var ex = v / 2;
            var mult = 1 / (Math.pow(2, ex) * Math.exp(this._gammaln(ex)));
            ex = ex - 1;

            var a = 0;
            var b = x;

            var midPt = (a + b) / 2;
            var radius = (b - a) / 2;

            var sum = 0;

            var offset;
            for (var i = 0; i < gauss_n; ++i) {
                offset = gauss_x[i] * radius;
                sum = Number(sum) + Number(gauss_w[i] * (Math.pow(midPt + offset, ex) * Math.exp(-(midPt + offset) / 2) + Math.pow(midPt - offset, ex) * Math.exp(-(midPt - offset) / 2)));
            }

            sum = Number(mult) * Number(sum) * Number(radius);
            return sum;
        };
        /**
         * 
		 * @private
        */
		this._gammaln = function (x) {
            var gammaAs = [0.918938533204673, 0.000595238095238, 0.000793650793651, 0.002777777777778, 0.083333333333333];
            var y = x;
            var f = 0;
            if (y < 7) {
                f = y;
                y++;
                while (y < 7) {
                    f = f * y;
                    if (y < 7) {
                        y++;
                    }
                }

                f = -Math.log(f);
            }

            var z = this._parseDouble((1 / (y * y)).toString());
            return f + (y - this._parseDouble("0.5")) * Math.log(y) - y + gammaAs[0] + (((-gammaAs[1] * z + gammaAs[2]) * z - gammaAs[3]) * z + gammaAs[4]) / y;
		};
        /**
         * 
		 * @private
        */
		this._pearson = function (x, y, n) {
            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumx2 = 0;
            var sumy2 = 0;
            var d1, d;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                d1 = y[i] - sumy;
                sumxy += d * d1;
                sumx2 += d * d;
                sumy2 += d1 * d1;
            }

            return sumxy / Math.sqrt(sumx2 * sumy2);
        };
        /**
         * 
		 * @private
        */
		this._fProbabilityDensity = function (x, k1, k2) {
            if (k1 <= 0 || k2 <= 0)
                throw "k1 and k2 must be greater than 0.";
            if (x == 0)
                return 0.0;
            var a1 = Math.pow(k1 * x, 0.5 * k1);
            var a2 = Math.pow(k2, 0.5 * k2);
            var a3 = Math.pow(k1 * x + k2, 0.5 * (k1 + k2));
            var a = a1 * a2 / a3;
            var b = this._betaFunction(0.5 * k1, 0.5 * k2);
            var c = x * b;
            return a / c;
        };
        /**
         * 
		 * @private
        */
		this._fCumulativeDensity = function (x, k1, k2) {
            if (k1 <= 0 || k2 <= 0)
                throw "k1 and k2 must be greater than 0.";
            if (x < 0)
                throw "x must be greater than 0.";

            if (x == 0)
                return 0.0;
            var x2 = (k1 * x) / (k1 * x + k2);
            var p = this._rIBetaFunction(x2, 0.5 * k1, 0.5 * k2);
            return Math.min(1, p);
        };
        /**
         * 
		 * @private
        */
		this._rIBetaFunction = function (x, a, b) {
            if (x <= 0.0)
                return 0.0;
            if (x >= 1.0)
                return 1.0;

            if (a % 1.0 == 0 && b % 1.0 == 0 && a + b > 0)
                return this._rIBetaFunction1(x, a, b);
            if (b % 1.0 == 0 && a + b < 172)
                return this._rIBetaFunction2(x, a, b);
            if (a % 1.0 == 0 && a + b < 172)
                return 1.0 - this._rIBetaFunction2(1.0 - x, b, a);
            if (a == 0.5 && b == 0.5)
                return (2.0 / Math.PI) * Math.atan(Math.sqrt(x / (1.0 - x)));
            if (a == 0.5 && b % 0.5 == 0) {
                return 1.0 - this._rIBetaFunction(1.0 - x, b, 0.5);
            }
            if (a % 0.5 == 0 && b == 0.5) {
                if (a < 45) {
                    //CASE B (partial)
                    var s = 0;
                    var n = Math.round(a - 0.5);
                    var logGammaHalf = this._parseDouble(this.computeGammaln("0.5".toString()));
                    for (var i = 0; i < n; i++) {
                        s = s + Math.exp(this._gammaln(i + 1) - this._gammaln(i + 1.5) - logGammaHalf) * Math.pow(x, i);
                    }
                    return this._rIBetaFunction(x, 0.5, 0.5) - Math.sqrt(x * (1 - x)) * s;
                } else {
                    //CASE C (partial)
                    var M = this._betaFunction(a, 0.5);
                    var lambda = Math.sqrt(1.0 - Math.pow(M * Math.sqrt((a - 1.0) / Math.PI) * this._epsilon, 1.0 / (a - 1.0)));

                    //Gaussian weights
                    var weights = [
                        0.066671344308688, 0.14945134915058, 0.21908636251598, 0.26926671931000, 0.29552422471475,
                        0.29552422471475, 0.26926671931000, 0.21908636251598, 0.14945134915058, 0.066671344308688];

                    var mgas = [
                        0.013046735791414, 0.067468316655507, 0.16029521585049, 0.28330230293538, 0.42556283050918,
                        0.57443716949081, 0.71669769706462, 0.83970478414951, 0.93253168334449, 0.98695326420859];

                    var sqrtOneMinusX = Math.sqrt(1 - x);
                    var s = 0;
                    for (var i = 0; i < 10; i++) {
                        s = s + weights[i] * Math.pow(1.0 - Math.pow((lambda - sqrtOneMinusX) * mgas[i] + sqrtOneMinusX, 2), a - 1.0);
                    }
                    var l = Math.exp(this._gammaln(a + 0.5) - this._gammaln(a) - this._gammaln(0.5));
                    return (lambda - sqrtOneMinusX) * l * s;
                }
            }
            if (a % 0.5 == 0 && b % 0.5 == 0) {
                //CASE B and C
                var s = 0;
                var n = Math.round(b - 0.5);
                var logGammaA = this._gammaln(a);
                var xToTheA = Math.pow(x, a);
                for (var i = 0; i < n; i++) {
                    s = s + Math.exp(this._gammaln(a + i + 0.5) - logGammaA - this._gammaln(i + 1.5)) * Math.pow(1 - x, i) * xToTheA;
                }
                var d = this._rIBetaFunction(x, a, 0.5) + Math.sqrt(1 - x) * s;
                return Math.max(0.0, Math.min(1.0, d));
            }
            if (x > 0.5)
                return 1.0 - this._rIBetaFunction(1.0 - x, b, a);

            var ribf = this._iBetaFunction(x, a, b) / this._betaFunction(a, b);
            return Math.max(0.0, Math.min(1.0, ribf));
        };
        /**
         * 
		 * @private
        */
		this._rIBetaFunction1 = function (x, a, b) {
            var s = 0;
            var c = a + b - 1;
            if (c < 21) {
                for (var i = a; i < a + b; i++) {
                    s = s + Math.pow(x, i) * Math.pow(1.0 - x, c - i) / (this._factorial(i) * this._factorial(c - i));
                }
                s = s * this._factorial(c);
            } else {
                for (var i = a; i < a + b; i++) {
                    s = s + Math.pow(x, i) * Math.pow(1.0 - x, c - i) * this._combinations(c, i);
                }
            }

            return Math.max(0.0, Math.min(1.0, s));
        };
        /**
         * 
		 * @private
        */
		this._rIBetaFunction2 = function (x, a, b) {
            if (a + b > 172)
                throw "Cannot currently compute RegularizedIncompleteBetaFunction for a + b > 172";

            var s = 0;
            for (var i = 1; i < b + 1; i++) {
                s = s + Math.pow(1 - x, i - 1) * Math.exp(this._gammaln(a + i - 1) - this._gammaln(i));
            }
            s = s * (Math.pow(x, a) * Math.exp(-this._gammaln(a)));
            return s;
        };
        /**
         * 
		 * @private
        */
		this._iBetaFunction = function (x, a, b) {
            if (x == 0)
                return 0;
            if (x == 1)
                return this._betaFunction(a, b);

            if (x <= 0.9) {
                return this._pIBetaFunction(0, x, a, b);
            }
            if (x <= 0.99) {
                var s1 = this._pIBetaFunction(0, 0.9, a, b);
                var s2 = this._pIBetaFunction(0.90, x, a, b);
                return s1 + s2;
            } else {
                var s1 = this._pIBetaFunction(0, 0.9, a, b);
                var s2 = this._pIBetaFunction(0.9, 0.99, a, b);
                var s3 = this._pIBetaFunction(0.99, x, a, b);
                return s1 + s2 + s3;
            }
        };
        /**
         * 
		 * @private
        */
		this._pIBetaFunction = function (xL, xU, a, b) {
            if (xU == xL)
                return 0.0;
            if (xU < xL)
                return NaN;

            var nSteps = 1000;
            if (a < 1)
                nSteps = 80000;

            var dt = (xU - xL) / nSteps;
            var t = xL + 0.5 * dt;
            var sum = 0;
            for (var i = 0; i < nSteps; i++) {
                sum = sum + Math.pow(t, a - 1) * Math.pow(1.0 - t, b - 1) * dt;
                t = t + dt;
            }
            return sum;
        };
        /**
         * 
		 * @private
        */
		this._betaFunction = function (a, b) {
            if (a + b > 143) {
                if (b > 20)
                    return 2.5066282746310002 * Math.pow(a, a - 0.5) * Math.pow(b, b - 0.5) / Math.pow(a + b, a + b - 0.5);
                return this._gammaFunction(b) * Math.pow(a, -b);
            }
            return this._gammaFunction(a) * this._gammaFunction(b) / this._gammaFunction(a + b);
        };
        /**
         * 
		 * @private
        */
		this._gammaFunction = function (z) {
            //approximation based on Stirling's formula
            //based on approximation from NIST Handbook of Mathematical Function. 2010. Cambridge University Press.
            if (z > 143)
                throw "Cannot currently compute gamma function for z > 143";

            //need z<21 or z<11 becaus Factorial(x) will not work for x>20
            if (z > 0 && z < 21 && z % 1.0 == 0)
                return this._factorial(Math.round(z - 1));
            if (z > 0 && z < 11 && z % 0.5 == 0) {
                var n = z;
                var sqrtPi = 1.77245385090552;
                return sqrtPi * this._factorial(2 * n) / (Math.pow(4, n) * this._factorial(n));
            }

            var sqrtTwoPi = 2.5066282746310002;
            var d = 1.0 + 1.0 / (12 * z) + 1.0 / (288 * z * z) - 139.0 / (51840 * Math.pow(z, 3)) - 571.0 / (2488320 * Math.pow(z, 4)) + 163879.0 / (209018880 * Math.pow(z, 5)) + 5246819.0 / (75246796800 * Math.pow(z, 6));
            var g = Math.pow(z, z - 0.5) * Math.exp(-z) * sqrtTwoPi * d;
            return g;
        };
        /**
         * 
		 * @private
        */
		this._epsilon = 4.94066e-324;
        /**
         * 
		 * @private
        */
		this._factorial = function (n) {
            if (n < 0)
                throw "Factorial not defined for negative n";
            if (n > 20)
                throw "Answer will exceed max long";
            var fact = 1;
            for (var i = n; i > 0; i--)
                fact = fact * i;
            return fact;
        };
        /**
         * 
		 * @private
        */
		this._combinations = function (n, k) {
            var logCombin = this._logCombin(n, k);
            var combin = Math.exp(logCombin);
            return Math.round(combin);
        };
        /**
         * 
		 * @private
        */
		this._logCombin = function (n, k) {
            return this._logFactorial(n) - this._logFactorial(k) - this._logFactorial(n - k);
        };
        /**
         * 
		 * @private
        */
		this._logFactorial = function (x) {
            var lf = 0;
            for (var i = 2; i <= x; i++)
                lf = lf + Math.log(i);
            return lf;
        };
        /**
         * 
		 * @private
        */
		this._isCellReference = function (args) {
            if (args == "") {
                return false;
            }
            args = this.setTokensForSheets(args);
            var sheetToken1 = this._sheetToken(args);
            var containsBoth = false;
            if (sheetToken1 != "") {
                args = args.split(this.sheetToken).join("");
            }

            var isAlpha = false, isNum = false;
            if (args.indexOf(':') != args.lastIndexOf(':')) {
                return false;
            }
            var charArray = (args.split('').join(this.getParseArgumentSeparator())).split(this.getParseArgumentSeparator());
            for (var c = 0; c < charArray.length; c++) {
                if (this._isLetter(charArray[c])) {
                    isAlpha = true;
                } else if (this._isDigit(charArray[c])) {
                    if (charArray[c] === 0)
                        return false;
                    isNum = true;
                } else if (charArray[c] == ':') {
                    if (isAlpha && isNum) {
                        containsBoth = true;
                    }
                    isAlpha = false;
                    isNum = false;
                } else
                    return false;
            }
            if (args.indexOf(":") > -1 && args.indexOf(this.tic) == -1) {
                if (containsBoth && isAlpha && isNum)
                    return true;
                else if (((isAlpha && !isNum) || (!isAlpha && isNum)) && !containsBoth) {
                    return true;
                } else
                    return false;
            }
            if (isAlpha && isNum && args.indexOf(this.tic) == -1) {
                return true;
            }

            return false;
        };
        /**
         * 
		 * @private
        */
		this._isDate = function (dateString) {
            var date = new Date(Date.parse(dateString));
            if (date >= this._dateTime1900) {
                return date;
            } else
                return "NaN";
        };
        /**
         * 
		 * @private
        */
		this._isDigit = function (text) {
            var charCode = text.charCodeAt(0);

            if ((charCode > 47) && (charCode < 58)) {
                return true;
            }
            return false;
        };
        /**
         * 
		 * @private
        */
		this._isHLookupCachingEnabled = function () {
            return ((this._enableLookupTableCaching & 3 /* Both */) != 0) || ((this._enableLookupTableCaching & 2 /* HLOOKUP */) != 0);
        };
        /**
         * 
		 * @private
        */
		this._isLetter = function (c) {
            var cc = c.charCodeAt(0);
            if ((cc >= 0x41 && cc <= 0x5A) || (cc >= 0x61 && cc <= 0x7A)) {
                return true;
            }
            return false;
        };
        /**
         * 
		 * @private
        */
		this._isLetterOrDigit = function (c) {
            var cc = c.charCodeAt(0);
            if ((cc >= 0x30 && cc <= 0x39) || (cc >= 0x41 && cc <= 0x5A) || (cc >= 0x61 && cc <= 0x7A)) {
                return true;
            }
            return false;
        };
        /**
         * 
		 * @private
        */
		this._isLookupCachingEnabled = function () {
            return ((this._enableLookupTableCaching & 3 /* Both */) != 0) || ((this._enableLookupTableCaching & 1 /* VLOOKUP */) != 0) || ((this._enableLookupTableCaching & 2 /* HLOOKUP */) != 0);
        };
        /**
         * 
		 * @private
        */
		this._isOptimizedMatchesEnabled = function () {
            return (this._enableLookupTableCaching & 4 /* OptimizeForMatches */) != 0;
        };
        /**
         * 
		 * @private
        */
		this._isRange = function (range) {
            ////checks if range holds a stand-alone range. Used to id when user inputs
            ////a formula that is just a range. Part of the support for using ranges as
            ////operands of binary operators. This method allows a range to be used with
            ////the formula assignment operator, =
            var isRange = false;
            var i = range.indexOf(':');
            if (i > 1 && i < range.length - 2) {
                ////check left side
                var j = i - 1;

                if (this._isDigit(range[j])) {
                    //// left side must end in digit
                    var needToCheckRightSide = false;
                    j--;
                    while (j > 0 && this._isDigit(range[j])) {
                        j--;
                    }

                    if (this._isLetter(range[j])) {
                        ////letters must come now
                        j--;
                        while (j >= 0 && this._isLetter(range[j])) {
                            j--;
                        }

                        if (j > -1 && range[j] == this._string_fixedreference[0]) {
                            j--;
                        }

                        if (j < 0) {
                            needToCheckRightSide = true; ////might have a range
                        } else {
                            if (range[j] == this.sheetToken) {
                                if (j-- > 1 && range[j] == this.tic[0]) {
                                    needToCheckRightSide = range.substring(0, j - 1).lastIndexOf(this.tic[0]) == 0;
                                } else if (j > 0 && this._isDigit(range[j])) {
                                    needToCheckRightSide = range.substring(0, j).lastIndexOf(this.sheetToken) == 0;
                                }
                            }
                        }
                    }

                    if (needToCheckRightSide) {
                        ////check right side
                        j = i + 1;

                        ////handle possible sheetnames
                        if (j < range.length - 6 && range[j] == this.tic[0]) {
                            j = range.indexOf(this.sheetToken, j + 1);
                            if (j < range.length - 2) {
                                j++;
                            }
                        }

                        if (j < range.length - 2 && range[j] == this._string_fixedreference[0]) {
                            j++;
                        }

                        if (this._isLetter(range[j])) {
                            j++;
                            while (j < range.length - 1 && this._isLetter(range[j])) {
                                j++;
                            }

                            if (this._isDigit(range[j])) {
                                j++;
                                while (j < range.length && this._isDigit(range[j])) {
                                    j++;
                                }

                                isRange = j == range.length;
                            }
                        }
                    }
                }
            }

            return isRange;
        };
        /**
         * 
		 * @private
        */
		this._isUpperChar = function (text) {
            var charCode = text.charCodeAt(0);
            return ((charCode > 64) && (charCode < 91));
        };
        /**
         * 
		 * @private
        */
		this._iisVLookupCachingEnabled = function () {
            return ((this._enableLookupTableCaching & 3 /* Both */) != 0) || ((this._enableLookupTableCaching & 1 /* VLOOKUP */) != 0);
        };
        /**
         * 
		 * @private
        */
		this._mark = function (copy, text, op, token, checkPrecedingChar) {
            var loc = 0;
            if (checkPrecedingChar) {
                loc = this._findAndCheckPrecedingChar(copy, op, loc);
                copy = this._findAndCheckPrecedingCharCopy;
            } else
                loc = copy.indexOf(op);
            var len = op.length;
            while (loc > -1) {
                copy = copy.substring(0, loc) + token + copy.substring(loc + len);
                text = text.substring(0, loc) + token + text.substring(loc + len);
                if (checkPrecedingChar) {
                    loc = this._findAndCheckPrecedingChar(copy, op, loc);
                    copy = this._findAndCheckPrecedingCharCopy;
                } else
                    loc = copy.indexOf(op, loc);
            }
            this._markCopy = "";
            return text;
        };
        /**
         * 
		 * @private
        */
		this._markColonsInQuotes = function (args) {
            var inQuotes = false;
            for (var i = 0; i < args.length; ++i) {
                if (args[i] == this.tic[0]) {
                    inQuotes = !inQuotes;
                } else if (args[i] == ':' && inQuotes) {
                    args = args.split(':').join(this._markerChar);
                }
            }
            return args;
        };
        /**
         * 
		 * @private
        */
		this._markIF = function (copy, text) {
            var locIF = copy.indexOf(this.getReservedWordOperators()[this._reservedWord_IF]);
            var locTHEN = copy.indexOf(this.getReservedWordOperators()[this._reservedWord_THEN]);
            var locELSE = copy.indexOf(this.getReservedWordOperators()[this._reservedWord_ELSE]);

            if (locELSE > -1 && locTHEN > -1) {
                var ifStart = this.getReservedWordOperators()[this._reservedWord_IF].length;
                var iflength = locTHEN - ifStart;
                var thenStart = locTHEN + this.getReservedWordOperators()[this._reservedWord_THEN].length;
                var thenlength = locELSE - thenStart;
                var elseStart = locELSE + this.getReservedWordOperators()[this._reservedWord_ELSE].length;
                var elselength = text.length - elseStart;
                text = "IF((" + this._substring(text, ifStart, iflength) + ")" + this.getParseArgumentSeparator() + "(" + this._substring(text, thenStart, thenlength) + ")" + this.getParseArgumentSeparator() + "(" + this._substring(text, elseStart, elselength) + "))";

                copy = text;
            } else if (locTHEN > -1) {
                var ifStart = 0;
                var iflength = locTHEN;
                var thenStart = locTHEN + this.getReservedWordOperators()[this._reservedWord_THEN].length;
                var thenlength = text.length - thenStart + 1;
                text = "IF((" + this._substring(text, ifStart, iflength) + ")" + this.getParseArgumentSeparator() + "(" + this._substring(text, thenStart, thenlength) + "))";

                copy = text;
            }

            this._markIFCopy = text;
            return text;
        };
        /**
         * 
		 * @private
        */
		this._markLibraryFormulas = function (formula) {
            var rightParens = formula.indexOf(')');
            if (rightParens == -1) {
                formula = this._markNamedRanges(formula);
            } else {
                while (rightParens > -1) {
                    var parenCount = 0;
                    var leftParens = rightParens - 1;
                    while (leftParens > -1 && (formula[leftParens] != '(' || parenCount != 0)) {
                        if (formula[leftParens] == ')') {
                            parenCount++;
                        } else if (formula[leftParens] == ')') {
                            parenCount--;
                        }

                        leftParens--;
                    }

                    if (leftParens == -1) {
                        throw this.formulaErrorStrings[this._mismatched_parentheses];
                    }

                    var i = leftParens - 1;
                    while (i > -1 && (this._isLetterOrDigit(formula[i]) || this._validFunctionNameChars.indexOf(formula[i]) > -1 || formula[i] == this.getParseDecimalSeparator())) {
                        i--;
                    }

                    var len = leftParens - i - 1;
                    if (len > 0 && this.getLibraryFunctions().getItem(this._substring(formula, i + 1, len)) != undefined) {
                        if (this._substring(formula, i + 1, len) == "AREAS")
                            this._ignoreBracet = true;
                        else
                            this._ignoreBracet = false;
                        var s = this._substring(formula, leftParens, rightParens - leftParens + 1);
                        s = this._markNamedRanges(s);
                        formula = formula.substring(0, i + 1) + 'q' + this._substring(formula, i + 1, len) + (s.split('(').join(this._leftBracket)).split(')').join(this._rightBracket) + formula.substring(rightParens + 1);
                    } else if (len > 0) {
                        if (this.unknownFunction != null) {
                            var family = CalcEngine.getSheetFamilyItem(this.grid);
                            var grd = this.grid;
                            var sheet = this._sheetToken(this.cell);
                            var s = this.cell;
                            if (sheet.length > 0) {
                                grd = family.tokenToParentObject.getItem(sheet);
                                s = s.substring(s.lastIndexOf(this.sheetToken) + 1);
                            }

                            var sheetNamekeys = family.sheetNameToParentObject.keys();
                            for (var key = 0; key < sheetNamekeys.length; key++) {
                                if (family.sheetNameToParentObject.getItem(sheetNamekeys[key]) == grd) {
                                    s = sheetNamekeys[key] + this.sheetToken + s;
                                    break;
                                }
                            }

                            var args = new UnknownFunctionEventArgs();
                            args.setMissingFunctionName(this._substring(formula, i + 1, len));
                            args.setCellLocation(s);

                            this.unknownFunction(this, args);
                            if (formula.substring(i + 1, len) == "AREAS")
                                this._ignoreBracet = true;
                            else
                                this._ignoreBracet = false;
                        }
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._unknown_formula_name] + " " + this._substring(formula, i + 1, len);
                        return this.getErrorStrings()[5].toString();

                    } else {
                        var s = "";
                        if (leftParens > 0) {
                            s = formula.substring(0, leftParens);
                        }

                        s = s + '{' + this._substring(formula, leftParens + 1, rightParens - leftParens - 1) + '}';
                        if (rightParens < formula.length) {
                            s = s + formula.substring(rightParens + 1);
                        }

                        s = this._markNamedRanges(s);
                        formula = s;
                    }

                    rightParens = formula.indexOf(')');
                }
            }

            formula = (formula.split('{').join('(')).split('}').join(')');
            return formula;
        };
        /**
         * 
		 * @private
        */
		this._markNamedRanges = function (argList) {
            ////Console.WriteLine("MarkNamedRanges in#{0}#", argList);
            var rightParens = argList.indexOf(')');
            var markers = [')', this.getParseArgumentSeparator(), '}', '+', '-', '*', '/', '<', '>', '=', '&'];

            //  int i = (argList.length > 0 && argList[0] == '(')
            var i = (argList.length > 0 && (argList[0] == '(' || argList[0] == '{')) ? 1 : 0;
            if (argList.indexOf("#N/A") > -1) {
                argList = argList.split("#N/A").join("#N~A");
            }
            if (argList.indexOf("#DIV/0!") > -1) {
                argList = argList.split("#DIV/0!").join("#DIV~0!");
            }
            var end = i;
            if (argList.indexOf("[") == -1 || argList.indexOf("[") > this._indexOfAny(argList.substring(i),markers))
                end = this._indexOfAny(argList.substring(i), markers);
            else
                end = this._findNextEndIndex(argList, end);

            while (end > -1 && end + i < argList.length) {
                var scopedRange = "";
                var s = null;
                if ((this._substring(argList, i, end)).indexOf("[") > -1) {
                    s = this._getTableRange(this._substring(argList, i, end));
                }
                else {
                    s = this.getNamedRanges().getItem(this._substring(argList, i, end));
                }
                if (s != undefined && !(typeof s === 'string'))
                    s = s.getItem(this._substring(argList, i, end));
                if (s == null) {
                    scopedRange = this._checkIfScopedRange(this._substring(argList, i, end));
                    if (!isNaN(scopedRange)) {
                        s = scopedRange;
                    } else if (this._substring(argList, i, end).startsWith(this.sheetToken.toString())) {
                        var iii = this._substring(argList, i, end).indexOf(this.sheetToken, 1);
                        if (iii > 1) {
                            s = this.getNamedRanges().getItem(this._substring(argList.substring(i), iii + 1, end - iii - 1));
                        }
                    }
                }
                if (s != null) {
                    s = s.toUpperCase();
                    s = this.setTokensForSheets(s);
                    s = this._markLibraryFormulas(s); ////recursive call
                }

                if (s != null) {
                    argList = argList.substring(0, i) + s + argList.substring(i + end);
                    i += s.length + 1;
                } else {
                    i += end + 1;
                    while (i < argList.length && !this._isUpperChar(argList[i]) && argList[i] != this.sheetToken) {
                        i++;
                    }
                }
                end = i;// argList.Substring(i).IndexOfAny(markers);
                if (argList.indexOf("[") == -1 || argList.indexOf("[") > this._indexOfAny(argList.substring(i), markers))
                    end = this._indexOfAny(argList.substring(i), markers);
                else
                    end = this._findNextEndIndex(argList, end);
                end = this._indexOfAny(argList.substring(i), markers);
                while (end == 0 && i < argList.length - 1) {
                    i++;
                    end = this._indexOfAny(argList.substring(i), markers);
                }

                if ((end == -1 || argList.substring(i).indexOf("[") >-1) && i < argList.length) {
                    if (argList.substring(i).indexOf("[") > -1) {
                        s = this._getTableRange(argList.substring(i));
                    }
                    else {
                        ////check last part of arglist
                        s = this.getNamedRanges().length > 0 ? this.getNamedRanges().getItem(argList.substring(i)) : s;
                    }
                    if (s == null) {
                        scopedRange = this._checkIfScopedRange(argList.substring(i));
                        if (!isNaN(scopedRange))
                            s = scopedRange;
                    }
                    if (s != null) {
                        s = s.toUpperCase();
                        s = this.setTokensForSheets(s);
                        s = this._markLibraryFormulas(s);
                        if (s != null) {
                            argList = argList.substring(0, i) + s; //// +argList.Substring(i + end);
                            i += s.toString().length + 1;
                        }
                    }

                    end = (i < argList.length) ? this._indexOfAny(argList.substring(i), markers) : -1;
                }
            }
            if (argList.indexOf("#N~A") > -1) {
                argList = argList.split("#N~A").join("#N/A");
            }

            if (argList.indexOf("#DIV~0!") > -1) {
                argList = argList.split("#DIV~0!").join("#DIV/0!");
            }
            return argList;
        };
        /**
         * 
		 * @private
        */
		this._markReserveWords = function (text) {
            var copy = text.toLowerCase();
            var trimStr = (this._isIE8) ? copy.replace(/^\s+|\s+$/g, '') : copy.trim();
            if (trimStr[0] == (this.getReservedWordOperators()[this._reservedWord_IF])) {
                text = this._markIF(copy, text);
                copy = this._markIFCopy;
            }

            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_NOT], this.char_NOTop, true);
            copy = this._markCopy;
            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_OR], this.char_ORop, false);
            copy = this._markCopy;
            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_AND], this.char_ANDop, false);
            copy = this._markCopy;
            text = this._mark(copy, text, this.getReservedWordOperators()[this._reservedWord_XOR], this.char_XORop, false);
            copy = this._markCopy;
            return text;
        };
        /**
         * 
		 * @private
        */
		this._markupResultToIncludeInFormula = function (s) {
            var d3;
            if (s.length > 0 && s[0] == '-' && !isNaN(this._parseDouble(s))) {
                s = "nu" + s.substring(1);
            } else if (s.length > 0 && (s[0] == this.tic[0] || s[0] == this.bMARKER || s[0] == '#')) {
                ////Pass on the string...
            } else if (s[0] == this.trueValueStr || s[0] == this.falseValueStr) {
                ////Pass on the bool...
            } else {
                ////if(double.TryParse(s, NumberStyles.Any, null, out d))
                if (!isNaN(this._parseDouble(s))) {
                    // s = d.toString(); ////defect 8721
                    s = s.split(this.getParseArgumentSeparator()).join(String.fromCharCode(32));

                    s = 'n' + s;
                } else {
                    if (!this._isRange(s)) {
                        s = this.tic + s + this.tic;
                    }
                }
            }
            return s;
        };
        /**
         * 
		 * @private
        */
		this._matchCompare = function (o1, o2) {
            var s1 = o1.toString();
            var s2 = o2.toString();
            var d1 = this._parseDouble(s1), d2 = this._parseDouble(s2);
            if (!isNaN(d1) && !isNaN(d2)) {
                if (d1 == d2)
                    return 0;
                if (d1 > d2)
                    return 1;
                else
                    return -1;
            } else {
                if (s1 == s2)
                    return 0;
                if (s1 > s2)
                    return 1;
                else
                    return -1;
            }
        };
        /**
         * 
		 * @private
        */
		this._matchingRightBracket = function (formula) {
            var ret = -1;
            var loc = 1;
            var bracketLevel = 0;
            var token = this._sheetToken(formula);
            while (ret == -1 && loc < formula.length) {
                if (formula[loc] == this._rightBracket) {
                    if (bracketLevel == 0) {
                        ret = loc;
                    } else {
                        bracketLevel--;
                        if (bracketLevel == 0 && loc == formula.length - 1) {
                            ret = loc;
                        }
                    }
                } else if (formula[loc] == 'q') {
                    var val = loc + 1;
                    if (val < formula.length) {
                        var libFunc = this._processUpperCase(formula, val, token);
                        formula = this._processUpperCaseFormula;
                        val = this._processUpperCaseIvalue;
                        token = this._processUpperCaseSheet;
                        if (libFunc != "" && this.getLibraryFunctions().containsKey(libFunc))
                            bracketLevel++;
                    }
                }

                loc++;
            }

            return ret;
        };
        /**
         * 
		 * @private
        */
		this._notInBlock = function (text, position) {
            var i = text.indexOf(this.bMARKER);
            var inBlock = false;
            while (i > -1 && i < position) {
                inBlock = !inBlock;
                i = text.indexOf(this.bMARKER, i + 1);
            }

            return !inBlock;
        };
        /**
         * 
		 * @private
        */
		this._getTopRowIndexFromRange = function (range) {
		    range = this._markColonsInQuotes(range);
		    var row1, col1;
		    var i = range.indexOf(':');
		    if (i == -1)
		        return this.rowIndex(range);
		    var sheet = "";
		    var j = range.indexOf(this.sheetToken);
		    if (j > -1) {
		        var j1 = range.indexOf(this.sheetToken, j + 1);
		        if (j1 > -1) {
		            sheet = this._substring(range, j, j1 - j + 1);
		            range = range.replace(sheet, "");
		            i = range.indexOf(':');
		        }
		    }
		    row1 = this.rowIndex(range.substring(0, i));
		    return row1;
		};

        /**
         * 
		 * @private
        */
		this._getTableRange = function (text) {
		    text = text.replace(" ", "").toUpperCase();
		    var name = text.replace("]", "").replace("#DATA", "");
		    var tableName = name;
		    if (name.indexOf(this.getParseArgumentSeparator()) > -1) {
		        tableName = name.substring(0, name.indexOf(this.getParseArgumentSeparator())).replace("[", "");
		        name = name.replace("[", "").replace(this.getParseArgumentSeparator(), '_');
		    }
		    var range = "";
		    if (text.indexOf("#THISROW") > -1) {
		        var cellRange = this.getNamedRanges().getItem(name.replace("#THISROW", ""));

		        if (cellRange != undefined && !(typeof cellRange === 'string'))
		            cellRange = cellRange.getItem(name.replace("#THISROW", ""));
		        if (cellRange == null)
		            return cellRange;
		        cellRange = cellRange.toUpperCase();
		        var row = this.rowIndex(this.cell);
		        cellRange = cellRange.replace(this._string_fixedreference, "");

		        cellRange = this.setTokensForSheets(cellRange);
		        var startindex = 0;
		        var rangeTable = this.getNamedRanges().getItem(tableName.Replace("#THISROW", ""));
		        if (rangeTable != undefined && !(typeof rangeTable === 'string'))
		            rangeTable = rangeTable.getItem(tableName.Replace("#THISROW", ""));
		        rangeTable = rangeTable.replace(this._string_fixedreference, "").toUpperCase();
		        rangeTable = this.setTokensForSheets(rangeTable);
		        var topRow = this._getTopRowIndexFromRange(cellRange);
		        range = this.getCellsFromArgs(cellRange)[row - topRow];
		    }
		    else {
		        name = (name[name.length - 1] == "[") ? name.replace("[", "") : name.replace("[", "_");
		        if (name.indexOf("#TOTALS") > -1 || name.indexOf("#ALL") > -1 || name.indexOf("#HEADERS") > -1 || name.indexOf("#DATA") > -1)
		            name = name.replace("#", "");
		        range = this.getNamedRanges().getItem(name);
		        if (range != undefined && !(typeof range === 'string'))
		            range = range.getItem(name);
		        if (range == null)
		            return null;
		    }
		    return range.toUpperCase();;
		};
        /**
         * 
		 * @private
        */
		this._parse = function (formulaText) {
            var text = formulaText;
            if (this._isTextEmpty(text)) {
                return text;
            }
            text = this.getFormulaText(text);

            if (this.getSupportLogicalOperators()) {
                text = this._markReserveWords(text);
            }

            if (this.getFormulaCharacter() != String.fromCharCode(0) && this.getFormulaCharacter() == text[0]) {
                text = text.substring(1);
            }

            ////Special check for single namedrange.
            if (this.getNamedRanges().length > 0) {
                if (text.indexOf("[") > -1) {
                    var namerangeVal = this._getTableRange(text);
                    if (namerangeVal != null)
                        text = namerangeVal;
                }
                if (this.getNamedRanges().getItem(text.toUpperCase()) != undefined) {
                    text = this.getNamedRanges().getItem(text.toUpperCase()).toUpperCase();
                } else {
                    var scopedRange = "";
                    scopedRange = this._checkIfScopedRange(text.toUpperCase());
                    if (!isNaN(scopedRange)) {
                        text = scopedRange.toUpperCase();
                    }
                }
            }

            var formulaStrings = this._saveStrings(text);
            text = this._saveStringsText;
            text = text.split(this.braceLeft).join(this.tic);
            text = text.split(this.braceRight).join(this.tic);

            text = text.split("-+").join("-");
            var i = 0;
            if (text[text.length - 1] != this.bMARKER || this._indexOfAny(text, this.tokens) != (text.length - 2))
                text = text.toUpperCase();

            if (text.indexOf(this.sheetToken) > -1) {
                //replace sheet references with tokens.
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.length > 0) {
                    try  {
                        text = this.setTokensForSheets(text);
                    } catch (ex) {
                        if (this._rethrowExceptions) {
                            throw ex;
                        } else {
                            return ex;
                        }
                    }
                }
            }

            if (this._isRangeOperand) {
                this._isRangeOperand = false;
                return this._getCellFrom(this._parseSimple(text));
            }

            ////next 3 lines moved from above - //keep the tic defect#541
            text = text.split(" ").join("");
            text = text.split("=>").join(">=");
            text = text.split("=<").join("<=");

            try  {
                text = this._markLibraryFormulas(text);
            } catch (ex) {
                if (this._rethrowExceptions) {
                    throw ex;
                } else {
                    return ex;
                }
            }

            ////Look for inner matching and parse pieces without parens with ParseSimple.
            if (!this._ignoreBracet) {
                while ((i = text.indexOf(')')) > -1) {
                    var k = text.substring(0, i).lastIndexOf('(');
                    if (k == -1) {
                        throw this.formulaErrorStrings[this._mismatched_parentheses];
                    }

                    if (k == i - 1) {
                        throw this.formulaErrorStrings[this._empty_expression];
                    }
                    var s = "";
                    if (this._ignoreBracet) {
                        s = this._substring(text, k, i - k + 1);
                    } else
                        s = this._substring(text,k + 1, i - k - 1);
                    text = text.substring(0, k) + this._parseSimple(s) + text.substring(i + 1);
                }
            }

            ////All parens should have been removed.
            if (!this._ignoreBracet && text.indexOf('(') > -1) {
                throw this.formulaErrorStrings[this._mismatched_parentheses];
            }

            var retValue = this._parseSimple(text);

            if (formulaStrings != null && formulaStrings.length > 0) {
                retValue = this._setStrings(retValue, formulaStrings);
            }
            return retValue;
        };
        /**
         * 
		 * @private
        */
		this._parseDouble = function (value) {
            return !isNaN(Globalize.parseInt(value)) ? Number(value.replace(/[^0-9\.]-+/g,"")) * 1 : NaN;
        };
        /**
         * 
		 * @private
        */
		this._parseSimple = function (formulaText) {
            var text = formulaText;

            ////strip leading plus
            if (text.length > 0 && text[0] == '+') {
                text = text.substring(1);
            }

            if (text == "#N/A" || text == "#N~A") {
                return "#N/A";
            } else if (text.indexOf("#N/A") > -1) {
                text = text.split("#N/A").join("#N~A");
            }
            if (text == "#DIV/0!" || text == "#DIV~0!") {
                return "#DIV/0!";
            } else if (text.indexOf("#DIV/0!") > -1) {
                text = text.split("#DIV/0!").join("#DIV~0!");
            }

            text = this._handleEmbeddedEs(text);

            text = text.split(this._string_lesseq).join(this.char_lesseq);
            text = text.split(this._string_greatereq).join(this.char_greatereq);
            text = text.split(this._string_noequal).join(this.char_noequal);
            text = text.split(this._string_fixedreference).join(this._string_empty);
            text = text.split(this._string_or).join(this.char_or);
            text = text.split(this._string_and).join(this.char_and);

            if (text == "") {
                return text;
            }

            var needToContinue = true;

            ////doing things sequentially imposes computation hierarchy for 6 levels
            //// 0. E+ E- //handles exponential notation like 1.2e-1 or 1.2e+1 to 1.2e1
            //// 1. ^ exponentiation
            //// 2. * /
            //// 3. + -
            //// 4. < > = <= >= <>
            //// 5. & concatenation
            //// in each level parsing done left to right
            var expTokenArray = [this.token_EP, this.token_EM];
            var orTokenArray = [this.token_or];
            var mulTokenArray = [this.token_multiply, this.token_divide];
            var addTokenArray = [this.token_add, this.token_subtract];
            var compareTokenArray = [this.token_less, this.token_greater, this.token_equal, this.token_lesseq, this.token_greatereq, this.token_noequal];
            var notTokenArray = [this.token_NOTop];
            var logicTokenArray = [this.token_ORop, this.token_ANDop, this.token_XORop];
            var andTokenArray = [this.token_and];

            var expCharArray = [this.char_EP, this.char_EM];
            var orCharArray = [this.char_or];
            var mulCharArray = [this.char_multiply, this.char_divide];
            var addCharArray = [this.char_add, this.char_subtract];
            var compareCharArray = [this.char_less, this.char_greater, this.char_equal, this.char_lesseq, this.char_greatereq, this.char_noequal];
            var notCharArray = [this.char_NOTop];
            var logicCharArray = [this.char_ORop, this.char_ANDop, this.char_XORop];
            var andCharArray = [this.char_and];

            text = this._parseSimpleOperators(text, expTokenArray, expCharArray);

            text = this._parseSimpleOperators(text, orTokenArray, orCharArray);

            if (needToContinue) {
                text = this._parseSimpleOperators(text, mulTokenArray, mulCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, addTokenArray, addCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, compareTokenArray, compareCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, notTokenArray, notCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, logicTokenArray, logicCharArray);
            }

            if (needToContinue) {
                text = this._parseSimpleOperators(text, andTokenArray, andCharArray);
            }

            if (text.indexOf("#N~A") > -1) {
                text = text.split("#N~A").join("#N/A");
            }

            if (text.indexOf("#DIV~0!") > -1) {
                text = text.split("#DIV~0!").join("#DIV/0!");
            }
            return text;
        };
        /**
         * 
		 * @private
        */
		this._parseSimpleOperators = function (formulaText, markers, operators) {
            var text = formulaText;
            var i;

            ////op is string containing this.char_xxxx's for each operator in this level
            var op = "";
            for (var c = 0; c < operators.length; c++) {
                op = op + operators[c];
            }

            //text = sb.replace(",-", ",u").replace(this._leftBracket + "-", this._leftBracket + "u").replace("=-", "=u").replace(">-", ">u").replace("<-", "<u").replace("/-", "/u").replace("*-", "*u").replace("+-", "+u").replace("--", "-u").replace("w-", "wu").toString();
            text = text.split("---").join("-").split("--").join("+").split(this.getParseArgumentSeparator() + "-").join(this.getParseArgumentSeparator() + "u").split(this._leftBracket + "-").join(this._leftBracket + "u").split("=-").join("=u");
            text = text.split(">-").join(">u").split("<-").join("<u").split("/-").join("/u").split("*-").join("*u").split("+-").join("-").split("--").join("-u").split("w-").join("wu").split(this.tic + "-").join(this.tic + "u").toString();

            ////Get rid of leading pluses.
            text = text.split(",+").join(",").split(this._leftBracket + "+").join(this._leftBracket).split("=+").join("=").split(">+").join(">").split("<+").join("<").split("/+").join("/").split("*+").join("*").split("++").join("+").toString();

            if (text.length > 0 && text[0] == '-') {
                ////Leading unary minus.
                text = "0" + text;
            } else if (text.length > 0 && text[0] == '+') {
                ////Leading plus.
                text = text.substring(1);
            }

            if (this._indexOfAny(text, operators) > -1) {
                while ((i = this._indexOfAny(text, operators)) > -1) {
                    var left = "";
                    var right = "";

                    var leftIndex = 0;
                    var rightIndex = 0;
                    var isNotOperator = this._supportLogicalOperators && text[i] == this.char_NOTop;
                    var j = 0;

                    if (!isNotOperator) {
                        if (i < 1 && text[i] != '-') {
                            throw this.formulaErrorStrings[this._operators_cannot_start_an_expression];
                        }

                        j = i - 1;

                        if (i == 0 && text[i] == '-') {
                            ////Unary minus - block and continue.
                            text = this.bMARKER + "nu" + text.substring(1) + this.bMARKER;
                            continue;
                        } else if (text[j] == this.tic[0]) {
                            ////string
                            var k = text.substring(0, j - 1).lastIndexOf(this.tic);
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            left = this._substring(text, k, j - k + 1); ////Keep the tics.
                            leftIndex = k;
                        } else if (text[j] == this.bMARKER) {
                            var k = this._findLastNonQB(text.substring(0, j - 1));
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }
                            left = this._substring(text, k + 1, j - k - 1);
                            leftIndex = k + 1;
                        } else if (text[j] == this._rightBracket) {
                            ////Library member.
                            var bracketCount = 0;
                            var k = j - 1;
                            while (k > 0 && (text[k] != 'q' || bracketCount != 0)) {
                                if (text[k] == 'q') {
                                    bracketCount--;
                                } else if (text[k] == this._rightBracket) {
                                    bracketCount++;
                                }

                                k--;
                            }

                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            left = this._substring(text, k, j - k + 1);
                            leftIndex = k;
                        } else if (!this._isDigit(text[j]) && text[j] != '%') {
                            while (j >= 0 && (this._isUpperChar(text[j]) || text[j] == '_' || text[j] == '.')) {
                                j--;
                            }

                            left = this._substring(text, j + 1, i - j - 1); ////'n' for number
                            leftIndex = j + 1;
                            var scopedRange = "";

                            if (this.getNamedRanges().containsKey(left) > -1) {
                                left = this._parse(this.getNamedRanges()[left]);
                            } else if (!isNaN(scopedRange = this._checkIfScopedRange(left, scopedRange))) {
                                left = this._parse(scopedRange);
                            } else if (left == this.trueValueStr) {
                                left = 'n' + this.trueValueStr;
                            } else if (left == this.falseValueStr) {
                                left = 'n' + this.falseValueStr;
                            } else {
                                if (this.getRethrowLibraryComputationExceptions())
                                    throw this.formulaErrorStrings[this._invalid_char_in_front_of]  + " " + text[i];
                                return this.getErrorStrings()[5].toString();
                            }
                        } else {
                            var period = false;
                            var percent = false;

                            while (j > -1 && (this._isDigit(text[j]) || (!period && text[j] == this.getParseDecimalSeparator()) || (!percent && text[j] == '%') || text[j] == 'u')) {
                                if (text[j] == this.getParseDecimalSeparator()) {
                                    period = true;
                                } else if (text[j] == '%') {
                                    percent = true;
                                }
                                j = j - 1;
                            }

                            ////Add error check for 2%.
                            if (j > -1 && period && text[j] == this.getParseDecimalSeparator()) {
                                throw this.formulaErrorStrings[this._number_contains_2_decimal_points];
                            }

                            j = j + 1;

                            if (j == 0 || (j > 0 && !this._isUpperChar(text[j - 1]))) {
                                left = 'n' + this._substring(text, j, i - j); ////'n' for number
                                leftIndex = j;
                            } else {
                                ////We have a cell reference.
                                j = j - 1;
                                while (j > -1 && this._isUpperChar(text[j])) {
                                    j = j - 1;
                                }

                                ////include any unary minus as part of the left piece
                                if (j > -1 && text[j] == 'u') {
                                    j = j - 1;
                                }
                                if (j > -1 && text[j] == this.sheetToken) {
                                    j = j - 1;
                                    while (j > -1 && text[j] != this.sheetToken) {
                                        j = j - 1;
                                    }

                                    if (j > -1 && text[j] == this.sheetToken) {
                                        j = j - 1;
                                    }
                                }

                                if (j > -1 && text[j] == ':') {
                                    //// handle range operands
                                    j = j - 1;
                                    while (j > -1 && this._isDigit(text[j])) {
                                        j = j - 1;
                                    }

                                    while (j > -1 && this._isUpperChar(text[j])) {
                                        j = j - 1;
                                    }

                                    if (j > -1 && text[j] == this.sheetToken) {
                                        j--;
                                        while (j > -1 && text[j] != this.sheetToken) {
                                            j--;
                                        }

                                        if (j > -1 && text[j] == this.sheetToken) {
                                            j--;
                                        }
                                    }

                                    j = j + 1;
                                    left = this._substring(text, j, i - j);
                                    left = this._getCellFrom(left);
                                } else {
                                    //// handle normal cell reference
                                    j = j + 1;
                                    left = this._substring(text, j, i - j);
                                }
                                this.updateDependencies(left);
                                leftIndex = j;
                            }
                        }
                    } else {
                        leftIndex = i;
                    }
                    if (i == text.length - 1) {
                        throw this.formulaErrorStrings[this._expression_cannot_end_with_an_operator];
                    } else {
                        j = i + 1;

                        var isU = text[j] == 'u';
                        if (isU) {
                            j++; ////ship for now, but add back later
                        }

                        if (text[j] == this.tic[0]) {
                            ////string
                            var k = text.substring(j + 1).indexOf(this.tic);
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            right = this._substring(text, j, k + 2);
                            rightIndex = k + j + 2;
                        } else if (text[j] == this.bMARKER) {
                            ////Block of already parsed code.
                            var k = this._findNonQB(text.substring(j + 1));
                            if (k < 0) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            right = this._substring(text, j + 1, k);

                            if (isU) {
                                right = right + "nu1m"; ////multiply quantity by -1...
                            }

                            rightIndex = k + j + 2;
                        } else if (text[j] == 'q') {
                            ////library
                            var bracketCount = 0;
                            var k = j + 1;
                            while (k < text.length && (text[k] != this._rightBracket || bracketCount != 0)) {
                                if (text[k] == this._rightBracket) {
                                    bracketCount++;
                                } else if (text[k] == 'q') {
                                    bracketCount--;
                                }

                                k++;
                            }

                            if (k == text.length) {
                                throw this.formulaErrorStrings[this._cannot_parse];
                            }

                            right = this._substring(text, j, k - j + 1);

                            if (isU) {
                                right = 'u' + right;
                            }

                            rightIndex = k + 1;
                        } else if (this._isDigit(text[j]) || text[j] == this.getParseDecimalSeparator()) {
                            var period = text[j] == this.getParseDecimalSeparator();
                            j = j + 1;
                            while (j < text.length && (this._isDigit(text[j]) || (!period && text[j] == this.getParseDecimalSeparator()))) {
                                if (text[j] == this.getParseDecimalSeparator()) {
                                    period = true;
                                }

                                j = j + 1;
                            }

                            if (j < text.length && text[j] == '%') {
                                j += 1;
                            }

                            if (period && j < text.length && text[j] == this.getParseDecimalSeparator()) {
                                throw this.formulaErrorStrings[this._number_contains_2_decimal_points];
                            }
                            right = 'n' + this._substring(text, i + 1, j - i - 1);
                            rightIndex = j;
                        } else if (this._isUpperChar(text[j]) || text[j] == this.sheetToken || text[j] == 'u') {
                            if (text[j] == this.sheetToken) {
                                j++;
                                while (j < text.length && text[j] != this.sheetToken) {
                                    j++;
                                }
                            }
                            j = j + 1;
                            var j0 = 0;
                            while (j < text.length && (this._isUpperChar(text[j]) || text[j] == '_' || text[j] == '.')) {
                                j++;
                                j0++;
                            }

                            var noCellReference = (j == text.length) || !this._isDigit(text[j]);

                            if (j0 > 4) {
                                while (j < text.length && (this._isUpperChar(text[j]) || this._isDigit(text[j]))) {
                                    j++;
                                }

                                noCellReference = true;
                            }

                            while (j < text.length && this._isDigit(text[j])) {
                                j = j + 1;
                            }

                            if (j < text.length && text[j] == ':') {
                                //// handle range operands
                                j = j + 1;
                                if (j < text.length && text[j] == this.sheetToken) {
                                    j++;
                                    while (j < text.length && text[j] != this.sheetToken) {
                                        j = j + 1;
                                    }

                                    if (j < text.length && text[j] == this.sheetToken) {
                                        j++;
                                    }
                                }

                                while (j < text.length && this._isUpperChar(text[j])) {
                                    j = j + 1;
                                }

                                while (j < text.length && this._isDigit(text[j])) {
                                    j = j + 1;
                                }

                                j = j - 1;

                                right = this._substring(text, i + 1, j - i);
                                right = this._getCellFrom(right);
                            } else {
                                //// handle normal cell reference
                                j = j - 1;
                                right = this._substring(text, i + 1, j - i);
                                isU = text[j] == 'u';
                                if (isU) {
                                    right = 'u' + right;
                                }
                            }
                            if (!noCellReference) {
                                this.updateDependencies(right);
                            } else {
                                var scopedRange = "";
                                if (this.getNamedRanges().getItem(right) > -1) {
                                    right = this._parse(this.getNamedRanges()[right]);
                                } else if (!isNaN(scopedRange = this._checkIfScopedRange(right, scopedRange))) {
                                    right = this._parse(scopedRange);
                                } else if (right == this.trueValueStr) {
                                    right = 'n' + this.trueValueStr;
                                } else if (right == this.falseValueStr) {
                                    right = 'n' + this.falseValueStr;
                                } else {
                                    if(this.getRethrowLibraryComputationExceptions())
                                        throw this.formulaErrorStrings[this._invalid_characters_following_an_operator];
                                    return this.getErrorStrings()[5].toString();
                                }
                            }

                            rightIndex = j + 1;
                        } else {
                            throw this.formulaErrorStrings[this._invalid_characters_following_an_operator];
                        }
                    }
                    var p = op.indexOf(text[i]);
                    var s = this.bMARKER + this._zapBlocks(left) + this._zapBlocks(right) + markers[p] + this.bMARKER;
                    if (leftIndex > 0) {
                        s = text.substring(0, leftIndex) + s;
                    }

                    if (rightIndex < text.length) {
                        s = s + text.substring(rightIndex);
                    }

                    s = s.split(this.bMARKER2).join(this.bMARKER.toString());

                    //// s = s.replace(tic+tic, tic); removed forum post http://64.78.18.34/Support/Forums/message.aspx?MessageID=22483
                    text = s;
                }
            } else {
                ////Process left argument.
                var j = text.length - 1;
                if (text[j] == this.bMARKER) {
                    var k = this._findLastNonQB(text.substring(0, j - 1));
                    if (k < 0) {
                        throw this.formulaErrorStrings[this._cannot_parse];
                    }
                } else if (text[j] == this._rightBracket) {
                    ////library member
                    var bracketCount = 0;
                    var k = j - 1;
                    while (k > 0 && (text[k] != 'q' || bracketCount != 0)) {
                        if (text[k] == 'q') {
                            bracketCount--;
                        } else if (text[k] == this._rightBracket) {
                            bracketCount++;
                        }

                        k--;
                    }

                    if (k < 0) {
                        throw this.formulaErrorStrings[this._bad_library];
                    }
                } else if (!this._isDigit(text[j])) {
                    // return "ERROR";
                } else {
                    var period = false;
                    var percent = false;

                    while (j > -1 && (this._isDigit(text[j]) || (!period && text[j] == this.getParseDecimalSeparator()) || (!percent && text[j] == '%'))) {
                        if (text[j] == this.getParseDecimalSeparator()) {
                            period = true;
                        } else if (text[j] == '%') {
                            percent = true;
                        }

                        j = j - 1;
                    }

                    if (j > -1 && period && text[j] == this.getParseDecimalSeparator()) {
                        throw this.formulaErrorStrings[this._number_contains_2_decimal_points];
                    }
                }
                if (text.length > 0 && (this._isUpperChar(text[0]) || text[0] == this.sheetToken)) {
                    ////Check if cell reference.
                    var ok = true;
                    var checkLetter = true;
                    var oneTokenFound = false;
                    for (var k = 0; k < text.length; ++k) {
                        if (text[k] == this.sheetToken) {
                            if (k > 0 && !oneTokenFound) {
                                throw this.formulaErrorStrings[this._missing_sheet];
                                ////break;
                            }

                            oneTokenFound = true;
                            k++;
                            while (k < text.length && this._isDigit(text[k])) {
                                k++;
                            }

                            if (k == text.length || text[k] != this.sheetToken) {
                                ok = false;
                                break;
                            }
                        } else {
                            ////if(!checkLetter && char.IsUpper(text, k))
                            if (!checkLetter && this._isLetter(text, k)) {
                                ok = false;
                                break;
                            }

                            if (this._isLetterOrDigit(text[k]) || text[k] == this.sheetToken) {
                                checkLetter = this._isUpperChar(text[k]);
                            } else {
                                ok = false;
                                break;
                            }
                        }
                    }

                    if (ok) {
                        this.updateDependencies(text); ////cb
                        //needToContinue = false;
                    }
                }
            }
            return text;
        };
        /**
         * 
		 * @private
        */
		this._pop = function (_stack) {
            var o = _stack.pop();

            var s;
            if (o != null) {
                if (o.toString() == this.tic + this.tic) {
                    return NaN;
                } else
                    s = o.toString().split(this.tic).join("");
                if (this.getErrorStrings().indexOf(s) > -1) {
                    this._isErrorString = true;
                    return this.getErrorStrings().indexOf(s);
                }

                ////moved from ComputedValue above as result of DT26064
                if (s[0] == "#" || s == "") {
                    return 0;
                }

                if (s == this.trueValueStr) {
                    return 1;
                } else if (s == this.falseValueStr) {
                    return 0;
                }

                var d = this._parseDouble(s);
                if (!isNaN(d)) {
                    return d;
                } else if (this.getUseDatesInCalculations()) {
                    var dt = this._isDate(s);
                    if (!isNaN(dt))
                        return this._getSerialDateTimeFromDate(dt);
                    else
                        return 0;
                }
            }
            if (s == ""  && this.getTreatStringsAsZero()) {
                return 0;
            }
            else if (o != null && o.toString().length > 0) {
                return Number.NaN;
            }
            return 0;
        };
        /**
         * 
		 * @private
        */
		this._popString = function (_stack) {
            var o = _stack.pop();

            if (o == null) {
                o = "";
            } else if (this.getUseDatesInCalculations()) {
                var dt = this._isDate(o.split(this.tic).join(""));
                if (!isNaN(dt))
                    return this._getSerialDateTimeFromDate(dt);
            }

            return this.removeTics(o.toString());
        };
        /**
         * 
		 * @private
        */
		this._populateNamedRangesNonScoped = function () {
            if (this._namedRangesNonScoped == null) {
                this._namedRangesNonScoped = new HashTable();
            }
            this._namedRangesNonScoped.clear();

            for (var k = 0; k < this.namedRanges.keys().length; k++) {
                this._checkAddNonScopedNamedRange(this.namedRanges.keys()[k]);
            }
        };
        /**
         * 
		 * @private
        */
		this._processUpperCase = function (formula, i, sheet) {
            var s = "";
            while (i < formula.length && this._isUpperChar(formula[i])) {
                s = s + formula[i];
                i = i + 1;
            }

            while (i < formula.length && this._isDigit(formula[i])) {
                s = s + formula[i];
                i = i + 1;
            }
            var firstCell = s;
            var firstIndex = i;
            if (i < formula.length && formula[i] == ':') {
                s = "";
                i = i + 1;
                if (i < formula.length && formula[i] == this.sheetToken) {
                    s = s + formula[i];
                    i = i + 1;

                    while (i < formula.length && formula[i] != this.sheetToken) {
                        s = s + formula[i];
                        i = i + 1;
                    }
                }

                while (i < formula.length && this._isUpperChar(formula[i])) {
                    s = s + formula[i];
                    i = i + 1;
                }

                while (i < formula.length && this._isDigit(formula[i])) {
                    s = s + formula[i];
                    i = i + 1;
                }
            } if (this._supportRangeOperands && firstCell != s) {
                s = sheet + this._getCellFrom(firstCell + ':' + s);
            } else {
                i = (firstCell != s) ? firstIndex : i;
                s = sheet + firstCell;
            }
            sheet = "";
            this._processUpperCaseFormula = formula;
            this._processUpperCaseIvalue = i;
            this._processUpperCaseSheet = sheet;
            return s;
        };
        /**
         * 
		 * @private
        */
		this._saveStrings = function (text) {
            var strings = null;
            var TICs2 = this.tic + this.tic;
            var id = 0;
            var i = -1;
            if ((i = text.indexOf(this.tic)) > -1) {
                while (i > -1 && i < text.length) {
                    if (strings == null) {
                        strings = new HashTable();
                    }

                    var j = (i + 1) < text.length ? text.indexOf(this.tic, i + 1) : -1;
                    if (j > -1) {
                        var key = this.tic + this._uniqueStringMarker + id.toString() + this.tic;
                        if (j < text.length - 2 && text[j + 1] == this.tic[0]) {
                            j = text.indexOf(this.tic, j + 2);
                            if (j == -1) {
                                throw this.formulaErrorStrings[this._mismatched_tics];
                            }
                        }

                        var s = this._substring(text, i, j - i + 1);
                        strings.add(key, s);
                        s = s.split(TICs2).join(this.tic);
                        id++;
                        text = text.substring(0, i) + key + text.substring(j + 1);
                        i = i + key.length;
                        if (i < text.length) {
                            i = text.indexOf(this.tic, i);
                        }
                    } else if (j == -1 && text.indexOf(this.sheetToken) > -1 && i < text.indexOf(this.sheetToken, i)) {
                        var sheetName = "";
                        for (var k = (text.indexOf(this.sheetToken, i)) - 1; k > -1; k--) {
                            if (this.getValidPrecedingChars().indexOf(text[k].toString()) == -1)
                                sheetName = text[k] + sheetName;
                            else
                                break;
                        }
                        if (!(sheetName[0] == "'" && sheetName[sheetName.length - 1] == "'")) {
                            throw this.formulaErrorStrings[this._mismatched_tics];
                        }
                        if (sheetName[0] == "'")
                            sheetName = sheetName.substring(1);
                        if (sheetName[sheetName.length - 1] == "'")
                            sheetName = sheetName.substring(0, sheetName.length - 1);
                        if (this.getSortedSheetNames().indexOf(sheetName.toUpperCase()) == -1) {
                            throw this.formulaErrorStrings[this._missing_sheet];
                        }
                        if ((i + 1) < text.length) {
                            i = text.indexOf(this.tic, i + 1);
                        }
                    } else {
                        throw this.formulaErrorStrings[this._mismatched_tics];
                    }
                }
            }
            this._saveStringsText = text;
            return strings;
        };
        /**
         * 
		 * @private
        */
		this._setStrings = function (retValue, strings) {
            var keys = strings.keys();
            for (var s = 0; s < keys.length; s++) {
                retValue = retValue.split(keys[s]).join(strings.getItem(keys[s]));
            }
            return retValue;
        };
        /**
         * 
		 * @private
        */
		this._sheetToken = function (s) {
            var i = 0;
            var s1 = "";
            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }

                s1 = s.substring(0, i + 1);
            }

            if (i < s.length) {
                return s1;
            }

            throw this.formulaErrorStrings[this._bad_index];
        };
        /**
         * 
		 * @private
        */
		this._splitArguments = function (args, argSeperator) {
            return args.split(argSeperator);
        };
        /**
         * 
		 * @private
        */
		this._stripTics0 = function (s) {
            if (s.length > 1 && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                s = this._substring(s, 1, s.length - 2);
            }

            return s;
        };
        /**
         * 
		 * @private
        */
		this._substring = function (text, startIndex, length) {
            return text.substring(startIndex, length + startIndex);
        };
        /**
         * 
		 * @private
        */
		this._isTextEmpty = function (s) {
            return s == null || s == "";
        };
        /**
         * 
		 * @private
        */
		this._toOADate = function (dateTime) {
            var result = (dateTime.getTime() - Date.parse(this._oaDate)) / this._millisecondsOfaDay;
            return Math.round(result);
        };
        /**
         * 
		 * @private
        */
		this._zapBlocks = function (text) {
            if (text.indexOf(this.bMARKER) > -1) {
                var bracketLevel = 0;
                for (var i = text.length - 1; i > 0; --i) {
                    if (text[i] == this._rightBracket) {
                        bracketLevel--;
                    } else if (text[i] == this._leftBracket) {
                        bracketLevel++;
                    } else if (text[i] == this.bMARKER && bracketLevel == 0) {
                        text = text.substring(0, i - 1) + text.substring(i + 1);
                    }
                }
            }

            return text;
        };
        
		
		/**		
        * Add the custom formuls with function in CalcEngine library 
        * @return boolean
        * @param {string} FormulaName pass the formula name 
        * @param {string} FunctionName pass the custom function name to call 
        * @example 
        * &lt;div id="Grid"&gt;&lt;/div&gt; 
        * &lt;script&gt;
        * // Create Grid
        * $('#Grid').ejGrid({
        *     dataSource: window.gridData
        * });         
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * var sheetID = calcObj.createSheetFamilyID();
        * calcObj.registerGridAsSheet("sheet1", $("#Grid").data("ejGrid"), sheetID);
        * calcObj.addCustomFunction("ADD", "customAdd");
        *    customAdd = function (argsList) {
        *    var splitArgs = argsList.split(calcObj.getParseArgumentSeparator())
        *       var result = 0;
        *       for (var i in splitArgs) {
        *           var s1 = calcObj.getValueFromArg(splitArgs[i]);
        *           result = Number(result) + Number(s1);
        *       }
        *       return result;
        *    }
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.addCustomFunction = function (name, func) {
            name = name.toUpperCase();
            this._addFunction(name, func);
            if (this._customlibraryFunctions[name] == undefined) {
                this._customlibraryFunctions[name] = func;
                return true;
            }
            return false;
        };
        this.UpdateDependentNamedRangeCell = function (key) {
            if (this.getDependentNamedRangeCells()!=null && this.getDependentNamedRangeCells().containsKey(key)) {
                var ht = this.getDependentNamedRangeCells().keys();
                  if (ht != null && ht.length > 0) {
                    for (var s = 0; s<ht.length; s++) {
                        var cell1 = this.getDependentNamedRangeCells().getItem(ht[s]);
                        var i = cell1.indexOf(this.sheetToken);
                        var row, col;
                        var grd = this.grid;
                        if (i > -1) {
                            var family = CalcEngine.getSheetFamilyItem(this.grid);
                           
                            this.grid = family.tokenToParentObject.getItem(cell1.substring(0, i + 3));
                            row = this.rowIndex(cell1);
                            col = this.colIndex(cell1);
                        }
                        else {
                            row = this.rowIndex(cell1);
                            col = this.colIndex(cell1);
                        }
                        this.recalculateRange(RangeInfo.cells(row, col,row,col), this.grid);

                        this.grid = grd;
                    }
                }
            }
        }

        this.RemoveNamedRangeDependency = function (key) {
            if (this.getDependentNamedRangeCells().containsKey(key)) {
                var ht = (Hashtable)(this.getDependentNamedRangeCells()[key]);
                ht.Clear();
                this.getDependentNamedRangeCells().remove(key);
            }
        }
        this.SetNamedRangeDependency = function (key, cell1) {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family.sheetNameToParentObject != null && cell1.indexOf(this.sheetToken) == -1) {
                var token = family.GridModelToToken[this.grid];
                cell1 = token + cell1;
            }

            if (this.getDependentNamedRangeCells()!=null && this.getDependentNamedRangeCells().containsKey(key)) {
                var ht = this.getDependentNamedRangeCells()[key];
                if (!ht.containsKey(cell1)) {
                    ht.add(cell1, "0");
                }
            }
            else {
                var ht1 = new HashTable();
                ht1.add(cell1, "0");
                this.getDependentNamedRangeCells().add(key, cell1.toString());
            }
        }
        /**		
        * Adds a named range to the NamedRanges collection 
        * @return boolean
        * @param {string} Name pass the namedRange's name 
        * @param {string} cellRange pass the cell range of NamedRange
        * @example
        * &lt;div id="Grid"&gt;&lt;/div&gt; 
        * &lt;script&gt;
        * // Create Grid
        * $('#Grid').ejGrid({
        *     dataSource: window.gridData
        * });         
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * var sheetID = calcObj.createSheetFamilyID();
        * calcObj.registerGridAsSheet("sheet1", $("#Grid").data("ejGrid"), sheetID);
        * calcObj.addNamedRange("FIRSTCELL","A1");
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.addNamedRange = function (name, range) {
            name = name.toUpperCase();
            if (this.getNamedRanges().getItem(name) == null) {
                this.getNamedRanges().add(name, range);
                if (!this.getNameRangeValues().containsKey(name)) {
                    this.getNameRangeValues().add(name, range.toString());
                }
                if (this.getUndefinedNamedRanges().containsKey(name) || this.getUndefinedNamedRanges().containsKey(name.toUpperCase()) || this.getUndefinedNamedRanges().containsKey(name.toLowerCase())) {
                    this.SetNamedRangeDependency(name.toUpperCase(), this.cell);
                    this.UpdateDependentNamedRangeCell(name.toUpperCase());
                }
                namedRangesSized = null;
                this._checkAddNonScopedNamedRange(name);
                return true;
            }
            return false;
        };
        /**		
        * Accepts a possible parsed formula and returns the calculated value without quotes.
        * @return adjusted range
        * @param {string} Name pass the cell range to adjust its range
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * var sheetID = calcObj.createSheetFamilyID();
        * calcObj.registerGridAsSheet("sheet1", $("#Grid").data("ejGrid"), sheetID);
        * calcObj.addNamedRange("FIRSTCELL","A1");
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.adjustRangeArg = function (range) {
            if (range.length > 1 && range[0] == this.bMARKER && range[range.length - 1] == this.bMARKER && this._substring(range, 1, range.length - 2).indexOf(this.bMARKER) == -1) {
                range = this.computedValue(range);
            }

            if (range.length > 1 && range[0] == this.tic[0] && range[range.length - 1] == this.tic[0]) {
                range = this._substring(range, 1, range.length - 2);
            }
            return range;
        };
        /**		
        * When a formula cell changes, call this method to clear it from its dependent cells.
        * @param {string} Cell pass the changed cell address
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.clearFormulaDependentCells("A1");
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.clearFormulaDependentCells = function (cell) {
            var ht = this.getDependentFormulaCells().getItem(cell);
            if (ht != null) {
                var keys = ht.keys();
                for (var o = 0; o < keys.length; o++) {
                    var s = keys[o];
                    var al = this.getDependentCells().getItem(s);
                    this._arrayRemove(al, cell);
                    if (al.length == 0) {
                        this.getDependentCells().remove(s);
                    }
                }

                this.getDependentFormulaCells().remove(cell);
            }
        };
        /**		
        *  Call this method to clear whether an exception was raised during the computation of a library function.
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.clearLibraryComputationException();
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.clearLibraryComputationException = function () {
            this._libraryComputationException = null;
        };
        /**		
        * Get the column index from a cell reference passed in.
        * @param {string} Cell pass the cell address
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.colIndex("A1");
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.colIndex = function (s) {
            var i = 0;
            var k = 0;
            s = s.toUpperCase();

            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }

                i++;
            }

            while (i < s.length && this._isLetter(s[i])) {
                var charCode = s[i].charCodeAt(0);
                k = k * 26 + charCode - 64;
                i++;
            }

            if (k == 0) {
                return -1;
            }

            return k;
        };
        /**		
        * Evaluates a parsed formula.
        * @return computed value of formula
        * @param {string} Formula pass the parsed formula
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.computedValue("n10n2a");
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.computedValue = function (formula) {
            if (this._isTextEmpty(formula)) {
                return formula;
            }
            try  {
                this.computedValueLevel++;
                if (this.computedValueLevel > this._maximumRecursiveCalls) {
                    this.computedValueLevel = 0;
                    throw this.formulaErrorStrings[this._too_complex];
                }
                var _stack = [];

                var i = 0;
                _stack.length = 0;
                var sheet = "";
                if (this.getAllowShortCircuitIFs()) {
                    var loc = -1;
                    do {
                        ////code peels all only the outer most IF functions  and relies on a custom IF to handle the proper alternative
                        if (i < formula.length && (i = formula.indexOf(this.iFMarker, i)) > -1) {
                            loc = this._matchingRightBracket(formula.substring(i));
                            if (loc > -1) {
                                var result = "";
                                var nextIfLoc = formula.indexOf(this.iFMarker, loc);

                                do {
                                    loc = i;
                                    var sepLoc = this._findNextSeparator(formula, loc);
                                    if (!isNaN(sepLoc)) {
                                        loc = sepLoc;
                                    }
                                    var funcArgs = this._substring(formula, i + this.iFMarker.length, loc - this.iFMarker.length - 1);
                                    var ifArguments = [];
                                    var argLoc = 0;
                                    var argNo = 0;
                                    while (argLoc < funcArgs.length) {
                                        sepLoc = this._findNextSeparator(funcArgs, argLoc);
                                        if (!isNaN(sepLoc)) {
                                            argLoc = sepLoc;
                                            ifArguments.push(funcArgs.substring(0, argLoc));
                                            funcArgs = funcArgs.substring(Number(argLoc) + 1);
                                            argLoc = 0;
                                        }
                                        else if (argLoc > 0) {
                                            arguments.push(funcArgs.substring(0, argLoc));
                                        }
                                    }
                                    var ifResult = this.getValueFromArg(ifArguments[0]);
                                    if (ifResult == this.trueValueStr)
                                        result = this.getValueFromArg(ifArguments[1]);
                                    else
                                        result = this.getValueFromArg(ifArguments[2]);
                                    nextIfLoc = formula.indexOf(this.iFMarker, loc);
                                }
                                while (formula.indexOf(this.iFMarker) > -1 && nextIfLoc > -1);

                                result = this._markupResultToIncludeInFormula(result)
                                var rightPiece = "";
                                if (i + loc + 1 < formula.length)
                                    rightPiece = formula.substring(i + loc + 1);
                                formula = formula.substring(0, i) + result + rightPiece;
                            }
                        }
                    }
                    while (formula.indexOf(this.iFMarker) > -1 && loc > -1);
                }
                i = 0;
                while (i < formula.length) {
                    if (formula[i] == this.bMARKER) {
                        i = i + 1;
                        continue;
                    }

                    var uFound = formula[i] == 'u';
                    if (uFound) {
                        i++;
                        if (i >= formula.length) {
                            continue;
                        }

                        if (formula[i] == this.bMARKER) {
                            i++;
                        }

                        if (i >= formula.length) {
                            continue;
                        }
                    }

                    ////defect 8729
                    ////************
                    //// The % is the last thing in the cell contents here
                    if (formula[i] == '%' && _stack.length > 0) {
                        var o = _stack[0];
                        var d = this._parseDouble(o);
                        if (!isNaN(d)) {
                            _stack.pop(); ////remove it
                            _stack.push(Number(d) / 100); ////push it back
                        }
                        i = i + 1;
                        continue;
                    }
                    if (formula[i] == this.sheetToken) {
                        sheet = formula[i].toString();
                        i++;
                        while (i < formula.length && formula[i] != this.sheetToken) {
                            sheet += formula[i];
                            i++;
                        }

                        if (i < formula.length) {
                            sheet += formula[i];
                            i++;
                        } else {
                            continue;
                        }
                    }
                    if ((formula.substring(i)).indexOf(this.trueValueStr) == 0) {
                        _stack.push(this.trueValueStr);
                        i += this.trueValueStr.length;
                    } else if (formula.substring(i).indexOf(this.falseValueStr) == 0) {
                        _stack.push(this.falseValueStr);
                        i += this.falseValueStr.length;
                    } else if (formula[i] == this.tic[0] || formula[i] == '|') {
                        if (_stack.length == 3)
                            _stack = this._combineStack(formula, i - 2, _stack);

                        var s = formula[i].toString();
                        i++;
                        while (i < formula.length && formula[i] != this.tic[0]) {
                            s = s + formula[i];
                            i = i + 1;
                        }
                        if (this._multiTick) {
                            s = s.split("|").join(this.tic);
                        }
                        _stack.push(s + this.tic);
                        i += 1;
                    } else if (this._isUpperChar(formula[i])) {
                        ////cell loc
                        var s = this._processUpperCase(formula, i, sheet);
                        formula = this._processUpperCaseFormula;
                        i = this._processUpperCaseIvalue;
                        sheet = this._processUpperCaseSheet;
                        if (uFound) {
                            s = this.getValueFromParentObjectCell(s);
                            var d3;
                            if (!isNaN(this._parseDouble(s))) {
                                d3 = -d3;
                                _stack.push(d3.toString());
                            } else {
                                _stack.push(s);
                            }
                        } else {
                            if (this.getUseFormulaValues() && this.computedValueLevel > 50) {
                                if (this._breakedFormulaCells.indexOf(s) == -1 && this.getUseFormulaValues()) {
                                    var family = CalcEngine.getSheetFamilyItem(this.grid);
                                    var token = this.sheetToken + this.getSheetID(this.grid).toString() + this.sheetToken;
                                    s = token + s;
                                    this._breakedFormulaCells.add(s);
                                    this._tempBreakedFormulaCells.add(s);
                                    break;
                                }
                                _stack.push("FALSE");
                            } else
                                _stack.push(this.getValueFromParentObjectCell(s));
                        }
                    } else if (formula[i] == 'q') {
                        ////library
                        formula = this._computeInteriorFunctions(formula);

                        var ii = formula.substring(i + 1).indexOf(this._leftBracket);
                        if (ii > 0) {
                            var bracketCount = 0;
                            var bracketFound = false;
                            var start = ii + i + 2;
                            var k = start;
                            while (k < formula.length && (formula[k] != this._rightBracket || bracketCount > 0)) {
                                if (formula[k] == this._leftBracket) {
                                    bracketCount++;
                                    bracketFound = true;
                                } else if (formula[k] == this._leftBracket) {
                                    bracketCount--;
                                }

                                k++;
                            }

                            if (bracketFound) {
                                var s = this._substring(formula, start, k - start - 2);
                                var s1 = "";
                                var splits = this.splitArgsPreservingQuotedCommas(s);
                                for (var t = 0; t < splits.length; t++) {
                                    if (s1.length > 0) {
                                        s1 += ",";
                                    }

                                    var j = this._findLastqNotInBrackets(t);
                                    if (j > 0) {
                                        s1 += splits[t].substring(0, j) + this.computedValue(splits[t].substring(j));
                                    } else {
                                        s1 += this.computedValue(splits[t]);
                                    }
                                }

                                formula = formula.substring(0, start) + s1 + formula.substring(k - 2);
                            }

                            var name = this._substring(formula, i + 1, ii);
                            if (name == "AVG" && this._excelLikeComputations) {
                                throw this.formulaErrorStrings[this._bad_index];
                            }
                            if (this.getLibraryFunctions().getItem(name) != undefined) {
                                var j = formula.substring(i + ii + 1).indexOf(this._rightBracket);
                                var args = this._substring(formula, i + ii + 2, j - 1);
                                try  {
                                    var functionName = this.getLibraryFunctions().getItem(name);
                                    var result;
                                    if (this.getCustomLibraryFunctions().getItem(name) != undefined) {
                                        result = eval(functionName)(args);
                                    } else
                                        result = this[functionName](args);
                                    _stack.push(result);
                                } catch (ex) {
                                    if (this.getRethrowLibraryComputationExceptions()) {
                                        this._libraryComputationException = ex;
                                        throw ex;
                                    }

                                    return ex;
                                }

                                i += j + ii + 2;
                            } else {
                                if (this.getRethrowLibraryComputationExceptions()) {
                                    throw this.formulaErrorStrings[this._missing_formula];
                                    return this.getErrorStrings()[5].toString();
                                }

                            }
                        } else if (formula[0] == this.bMARKER) {
                            ////Restart the processing with the formula without library finctions.
                            i = 0;
                            _stack.length == 0;
                            continue;
                        } else {
                            throw this.formulaErrorStrings[this._improper_formula];
                        }
                    } else if (this._isDigit(formula[i]) || formula[i] == 'u') {
                        var s = "";
                        if (formula[i] == 'u' || uFound) {
                            s = "-";
                            if (!uFound) {
                                i++;
                            } else {
                                uFound = false;
                            }
                        }

                        if (i < formula.length && this._isUpperChar(formula[i])) {
                            s = s + this.getValueFromParentObjectCell(this._processUpperCase(formula, i, sheet));
                            formula = this._processUpperCaseFormula;
                            i = this._processUpperCaseIvalue;
                            sheet = this._processUpperCaseSheet;
                        } else {
                            while (i < formula.length && (this._isDigit(formula[i]) || formula[i] == this.getParseDecimalSeparator())) {
                                s = s + formula[i];
                                i = i + 1;
                            }
                        }

                        _stack.push(s);
                    } else {
                        switch (formula[i]) {
                            case '#': {
                                var errIndex = 0;
                                if (formula.indexOf('!') == -1) {
                                    errIndex = formula.indexOf('?') + 1;
                                } else
                                    errIndex = formula.indexOf('!') + 1;
                                _stack.push(this._substring(formula, i, errIndex - i));
                                i = errIndex;
                                break;
                            }

                            case 'n':
                                {
                                    i = i + 1;

                                    var s = "";
                                    if (formula.substring(i).indexOf("Infinity") == 0) {
                                        s = "Infinity";
                                        i += s.length;
                                    } else if (formula.substring(i).indexOf("uInfinity") == 0) {
                                        s = "Infinity";
                                        i += s.length + 1;
                                    } else if (formula.substring(i).indexOf(this.trueValueStr) == 0) {
                                        s = this.trueValueStr;
                                        i += s.length;
                                    } else if (formula.substring(i).indexOf(this.falseValueStr) == 0) {
                                        s = this.falseValueStr;
                                        i += s.length;
                                    } else if (i <= formula.length - 3 && formula.substring(i, 3) == "NaN") {
                                        i += 3;
                                        s = "0";
                                    } else {
                                        if (formula[i] == 'u' || uFound) {
                                            s = "-";
                                            if (!uFound) {
                                                i = i + 1;
                                            } else {
                                                uFound = false;
                                            }
                                        }

                                        while (i < formula.length && (this._isDigit(formula[i]) || formula[i] == this.getParseDecimalSeparator())) {
                                            s = s + formula[i];
                                            i = i + 1;
                                        }
                                        if (i < formula.length && formula[i] == '%') {
                                            i = i + 1;

                                            ////defect 8729
                                            ////**************
                                            if (s == "") {
                                                if (_stack.length > 0) {
                                                    var o = _stack[0];
                                                    var d = this._parseDouble(o.toString());
                                                    if (!isNaN(d)) {
                                                        _stack.pop(); ////remove it
                                                        s = (d / 100).toString(); ////push it back
                                                    }
                                                }
                                            } else {
                                                ////**************
                                                s = (Number(s) / 100).toString();
                                            }
                                        } else if (i < formula.length - 2 && formula[i] == 'E' && (formula[i + 1] == '+' || formula[i + 1] == '-')) {
                                            s += this._substring(formula, i, 4);
                                            i += 4;
                                        }
                                    }
                                    _stack.push(s);
                                }
                                break;
                            case this.token_add:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else
                                        _stack.push((Number(d1) + Number(d)).toString());
                                    i = i + 1;
                                }

                                break;
                            case this.token_subtract:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else {
                                        _stack.push((d1 - d).toString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case this.token_multiply:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else {
                                        _stack.push((d1 * d).toString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case this.token_divide:
                                {
                                    var d = this._pop(_stack);
                                    var x = parseInt(d.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    var d1 = this._pop(_stack);
                                    x = parseInt(d1.toString());
                                    if (!isNaN(x) && this._isErrorString) {
                                        this._isErrorString = false;
                                        return this.getErrorStrings()[x].toString();
                                    }
                                    if (isNaN(d) || isNaN(d1)) {
                                        _stack.push("#VALUE!");
                                    } else if (d == 0) {
                                        _stack.push(this.getErrorStrings()[3]);
                                    } else {
                                        _stack.push((d1 / d).toString());
                                    }
                                    i = i + 1;
                                }

                                break;
                            case this.token_EP:
                                {
                                    var d = this._pop(_stack);
                                    var d1 = this._pop(_stack);
                                    _stack.push((d1 * Math.pow(10, d)).toString());
                                    i = i + 1;
                                }
                                break;
                            case this.token_EM:
                                {
                                    var d = this._pop(_stack);
                                    var d1 = this._pop(_stack);
                                    _stack.push((Number(d1) * Math.pow(10, -Number(d))).toString());
                                    i = i + 1;
                                }
                                break;
                            case this.token_less:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (d1 < d) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s2.toUpperCase().split(this.tic).join("").localeCompare(s1.toUpperCase().split(this.tic).join("")) < 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_greater:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "") {
                                        if (this.getTreatStringsAsZero())
                                            s1 = "0";
                                        else
                                            s1 = s2 + 1;
                                    }

                                    if (s2 == "") {
                                        if (this.getTreatStringsAsZero())
                                            s2 = "0";
                                        else
                                            s2 = s1 + 1;
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) > Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s2.toUpperCase().split(this.tic).join("").localeCompare(s1.toUpperCase().split(this.tic).join("")) > 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_equal:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) == Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("") == s2.toUpperCase().split(this.tic).join("")) ? this.trueValueStr : this.falseValueStr;
                                    }
                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_lesseq:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) <= Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("").localeCompare(s2.toUpperCase().split(this.tic).join("")) <= 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_greatereq:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) >= Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("").localeCompare(s2.toUpperCase().split(this.tic).join("")) >= 0) ? this.trueValueStr : this.falseValueStr;
                                    }
                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_noequal:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    if (s1 == "" && this.getTreatStringsAsZero()) {
                                        s1 = "0";
                                    }
                                    if (s2 == "" && this.getTreatStringsAsZero()) {
                                        s2 = "0";
                                    }
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (!isNaN(d) && !isNaN(d1)) {
                                        val = (Number(d1) != Number(d)) ? this.trueValueStr : this.falseValueStr;
                                    } else {
                                        if (s1 == "#VALUE!" || s2 == "#VALUE!")
                                            val = "#VALUE!";
                                        else
                                            val = (s1.toUpperCase().split(this.tic).join("") != s2.toUpperCase().split(this.tic).join("")) ? this.trueValueStr : this.falseValueStr;
                                    }
                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_and:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = "";
                                    if (_stack.length > 0)
                                        s2 = this._popString(_stack);
                                    if (this.getUseNoAmpersandQuotes()) {
                                        _stack.push(s2 + s1);
                                    } else {
                                        _stack.push(this.tic + s2 + s1 + this.tic);
                                    }
                                    i = i + 1;
                                }
                                break;
                            case this.token_or:
                                {
                                    var d = this._pop(_stack);
                                    var d1 = this._pop(_stack);
                                    _stack.push(Math.pow(d1, d).toString());
                                    i = i + 1;
                                }
                                break;
                            case this.token_ORop:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    if (s2 == "") {
                                        s2 = this.falseValueStr;
                                    } else if (!isNaN(d1)) {
                                        s2 = (Number(s2) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = (s1.toUpperCase().split(this.tic).join("") == this.trueValueStr || s2.toUpperCase().split(this.tic).join("") == this.trueValueStr) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_ANDop:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    if (s2 == "") {
                                        s2 = this.falseValueStr;
                                    } else if (!isNaN(d1)) {
                                        s2 = (Number(s2) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = (s1.toUpperCase().split(this.tic).join("") == this.trueValueStr && s2.toUpperCase().split(this.tic).join("") == this.trueValueStr) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_XORop:
                                {
                                    var s1 = this._popString(_stack);
                                    var s2 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var d1 = this._parseDouble(s2);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    if (s2 == "") {
                                        s2 = this.falseValueStr;
                                    } else if (!isNaN(d1)) {
                                        s2 = (Number(s2) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = ((s1.toUpperCase().split(this.tic).join("") == this.trueValueStr && s2.toUpperCase().split(this.tic).join("") != this.trueValueStr) || (s2.toUpperCase().split(this.tic).join("") == this.trueValueStr && s1.toUpperCase().split(this.tic).join("") != this.trueValueStr)) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            case this.token_NOTop:
                                {
                                    var s1 = this._popString(_stack);
                                    var d = this._parseDouble(s1);
                                    var val;
                                    if (s1 == "") {
                                        s1 = this.falseValueStr;
                                    } else if (!isNaN(d)) {
                                        s1 = (Number(s1) != 0) ? this.trueValueStr : this.falseValueStr;
                                    }

                                    val = (s1.toUpperCase().split(this.tic).join("") == this.falseValueStr) ? this.trueValueStr : this.falseValueStr;

                                    _stack.push(val);
                                    i = i + 1;
                                }
                                break;
                            default: {
                                this.computedValueLevel = 0;
                                if(this.getRethrowLibraryComputationExceptions())
                                    throw this.formulaErrorStrings[this._invalid_expression];
								else
                                    return this.getErrorStrings()[1].toString();
                            }
                        }
                    }
                }
                if (this._checkDanglingStack && _stack.length > 1 && formula.length > 1 && (formula.indexOf(this.bmarker.toString()) != 0 || formula.indexOf(this.bmarker) != (formula.length - 1) || formula.indexOf(this.bmarker2) > -1)) {
                    this.computedValueLevel = 0;
                    return this.formulaErrorStrings[this._improper_formula];
                }

                if (_stack.length == 0) {
                    return "";
                } else {
                    var s = "";
                    var d;
                    var cc = _stack.length;
                    do {
                        s = _stack.pop().toString() + s;
                        return s;
                        cc--;
                    } while(cc > 0 && !(s.indexOf(this.FALSEVALUESTR) > -1 || s.indexOf(this.tRUEVALUESTR) > 1));

                    ////This check moved below in Pop as result of DT26064
                    ////if(_textIsEmpty(s))
                    ////    s = "0"; //Empty is zero in calculations.
                    return s;
                }
            } catch (ex) {
                this.computedValueLevel = 0;
                if (ex.toString().indexOf(this.formulaErrorStrings[this._circular_reference_]) > -1 || (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null)) {
                    if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                        throw this.getLibraryComputationException();
                    }

                    throw ex;
                }

                if (ex.toString().indexOf(this.formulaErrorStrings[this._cell_empty]) > -1) {
                    return "";
                } else {
                    if (this.getRethrowLibraryComputationExceptions()) {
                        this._libraryComputationException = ex;
                        throw ex;
                    }
                    return ex;
                }
            } finally {
                this.computedValueLevel--;
                if (this.computedValueLevel < 0) {
                    this.computedValueLevel = 0;
                }
            }
        };
        /**		
        * Evaluates a parsed formula.
        * @return computed value of formula
        * @param {string} Formula pass the parsed formula
        * @example
        * var calcObj = new CalcEngine($("#Grid").data("ejGrid"));
        * calcObj.computedValue("n10n2a");
        * &lt;/script&gt;
		* @memberof ejCalculate
		* @instance
        */
        this.computeFormula = function (parsedFormula) {
            if (this.getThrowCircularException()) {
                if (this._iterationMaxCount > 0) {
                    if (this.computedValueLevel == 0) {
                        this._circCheckList.length = 0;
                        this._circCheckList.push(this.cell);
                    } else if (this._circCheckList.indexOf(this.cell) > -1) {
                        if (!this.getIterationValues().containsKey(this.cell)) {
                            this.getIterationValues().add(this.cell, "0");
                        }

                        return this.getIterationValues().getItem(this.cell).toString();
                    }
                } else {
                    if (this.computedValueLevel == 0) {
                        this._circCheckList.length = 0;
                    } else if (this._circCheckList.indexOf(this.cell) > -1) {
                        this.computedValueLevel = 0;
                        this._circCheckList.length = 0;
                        throw this.formulaErrorStrings[this._circular_reference_];
                    }

                    this._circCheckList.push(this.cell);
                }
            }

            var s = this.computedValue(parsedFormula);

            if (this.getUseNoAmpersandQuotes() && s.length > 1 && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                s = this._substring(s, 1, s.length - 2);
            }
            return s;
        };
        
        this.dispose = function () {
            if (!this._isDisposed) {
                var i = 0;
                if (this.grid != null) {
                    i = this.getSheetID(this.grid);
                }
                CalcEngine._tokenCount--;
                CalcEngine.sheetFamilyID--;
                if (CalcEngine._sheetFamiliesList != null && CalcEngine._sheetFamiliesList.length > 0) {
                    CalcEngine._sheetFamiliesList[i] = null;
                    CalcEngine._sheetFamiliesList.remove(i);
                }
                if (CalcEngine.modelToSheetID != null && CalcEngine.modelToSheetID.length > 0) {
                    CalcEngine.modelToSheetID[i] = null;
                    CalcEngine._sheetFamiliesList.remove(i);
                }
                if (this._formulaInfoTable != null) {
                    this._formulaInfoTable.clear();
                    this._formulaInfoTable = null;
                }
                if (this._circCheckList != null) {
                    this._circCheckList.length = 0;
                    this._circCheckList = null;
                }
                if (this._dependentCells != null) {
                    this._dependentCells.clear();
                    this._dependentCells = null;
                }
                if (this._dependentFormulaCells != null) {
                    this._dependentFormulaCells.clear();
                    this._dependentFormulaCells = null;
                }

                if (this.getIterationValues() != null) {
                    this.getIterationValues().clear();
                    this.getIterationValues(null);
                }
                if (this._libraryFunctions != null) {
                    this._libraryFunctions.clear();
                    this._libraryFunctions = null;
                }
                if (this._lookupTables != null) {
                    this._lookupTables.clear();
                    this._lookupTables = null;
                }
                this.grid = null;
                if (this.getLibraryFunctions() != null) {
                    this.getLibraryFunctions().clear();
                    this.setLibraryFunctions(null);
                }
                this._isDisposed = true;
            }
        };
        this.getCalcID = function () {
            return this._calcID;
        };
        this.getCellsFromArgs = function (args) {
            args = this._markColonsInQuotes(args);

            var row1;
            var col1;

            var i = args.indexOf(':');
            if (i == -1) {
                args = args.split(this._markerChar).join(':');
                i = args.indexOf(this.getParseArgumentSeparator());
                if (i == -1) {
                    row1 = this.rowIndex(args); ////maybe throw exception
                    col1 = this.colIndex(args); ////maybe throw exception
                    var argsArray = [];
                    argsArray.push(args);
                    return argsArray;
                } else {
                    return this.splitArgsPreservingQuotedCommas(args);
                }
            }

            var sheet = "";
            var argsRet = args;
            var j = args.indexOf(this.sheetToken);
            if (j > -1) {
                var j1 = args.indexOf(this.sheetToken, j + 1);
                if (j1 > -1) {
                    sheet = this._substring(args, j, j1 - j + 1);
                    args = args.split(sheet).join("");
                    i = args.indexOf(':');
                }
            }

            ////check if column only range
            if (i > 0 && this._isLetter(args[i - 1])) {
                var count = (this._rowMaxCount > 0) ? this._rowMaxCount : 50;
                args = args.substring(0, i) + "1:" + args.substring(i + 1) + count.toString();
                i = args.indexOf(':');
            } else if (i > 0 && this._isDigit(args[i - 1])) {
                var k1 = i - 2;
                while (k1 >= 0 && this._isDigit(args[k1])) {
                    k1--;
                }

                if (k1 == -1 || !this._isLetter(args[k1])) {
                    var count = (this._columnMaxCount > 0) ? this._columnMaxCount : 50;
                    args = "A" + args.substring(0, i) + ":" + RangeInfo.getAlphaLabel(count) + args.substring(i + 1);
                    i = args.indexOf(':');
                }
            }

            // Check Whether rowindex can get from the arguments
            var isDigit = this._canGetRowIndex(args.substring(0, i));
            if (!isDigit) {
                // if unable to get index return the arguments
                this._ignoreCellValue = true;
                args = argsRet;
                return this.splitArgsPreservingQuotedCommas(args);
            }
            row1 = this.rowIndex(args.substring(0, i));
            var row2 = this.rowIndex(this._substring(args, i + 1, args.length - i - 1));
            var errorStr;
            if ((!(!(row1 == -1) || (row2 == -1)) == (((row1 == -1) || !(row2 == -1))))) {
                throw this.getErrorStrings()[5].toString();
                // errorStr[0] = this.getErrorStrings()[5].toString();
                // return errorStr;
            }

            col1 = this.colIndex(args.substring(0, i));

            var col2 = this.colIndex(this._substring(args, i + 1, args.length - i - 1));

            if (row1 > row2) {
                i = row2;
                row2 = row1;
                row1 = i;
            }

            if (col1 > col2) {
                i = col2;
                col2 = col1;
                col1 = i;
            }

            var numCells = (row2 - row1 + 1) * (col2 - col1 + 1);
            var cells = [];
            var k = 0;
            for (i = row1; i <= row2; ++i) {
                for (j = col1; j <= col2; ++j) {
                    try  {
                        cells[k++] = sheet + RangeInfo.getAlphaLabel(j) + i.toString();
                    } catch (ex) {
                        continue;
                    }
                }
            }
            return cells;
        };
        this.getFormulaRowCol = function (grd, row, col) {
            var family = CalcEngine.getSheetFamilyItem(grd);
            var s = RangeInfo.getAlphaLabel(col) + row.toString();
            if (family.sheetNameToParentObject != null) {
                var token = family.parentObjectToToken.getItem(grd);
                s = token + s;
            }

            if (this.getFormulaInfoTable().containsKey(s)) {
                var formula = this.getFormulaInfoTable.getItem(s);
                var value = formula.getFormulaText();
                this.getFormulaText(value);
                return value;
            }

            return "";
        };
        this.getFormulaText = function (s) {
            var e = new FormulaParsing(s);
            s = e.getText();
            return s;
        };
        this.getSheetID = function (grd) {
            var family = CalcEngine.getSheetFamilyItem(grd);
            if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.length > 0) {
                var token = family.parentObjectToToken.getItem(grd);
                token = token.split("!").join("");
                var d = this._parseDouble(token);
                if (!isNaN(d)) {
                    return d;
                }
            }

            return -1;
        };
        this.getStringArray = function (s) {
            //ArrayList argList = new ArrayList();
            var argList = [];
            var argStart = 0;
            var inQuote = false;
            for (var argEnd = 0; argEnd < s.length; argEnd++) {
                var ch = s[argEnd];
                if (ch == this.tic[0]) {
                    inQuote = !inQuote;
                } else if (!inQuote && ch == this.getParseArgumentSeparator()) {
                    argList.push(this._substring(s, argStart, argEnd - argStart));
                    argStart = argEnd + 1;
                }
            }

            argList.push(s.substring(argStart));
            return argList;
        };
        this.getValueFromArg = function (arg) {
            var d;
            var tempDate = this._dateTime1900;

            ////handle special characters - probably need to revise...
            ////if(arg.IndexOf("%") > -1)
            ////	Console.WriteLine("special");
            ////arg = arg//.replace(TRUEVALUESTR, TRUEVALUE.toString())
            ////		//.replace(FALSEVALUESTR, FALSEVALUE.toString())
            ////		.replace("%", "");

            if (!isNaN(arg))
                arg = arg.toString();

            if (this._isTextEmpty(arg)) {
                return arg;
            } else if (arg[0] == this.tic[0]) {
                ////string
                var doubleValue;
                tempDate = this._isDate(arg.split(this.tic).join(""));
                if (this.getExcelLikeComputations() && this.getUseDatesInCalculations() && isNaN(this._parseDouble(arg.split(this.tic).join(""))) && !isNaN(tempDate.getDate()) && this._dateTime1900 <= tempDate) {
                    return this._toOADate(tempDate).toString();
                }
                return arg;
            } else if (arg[0] == this.bMARKER || arg[0] == 'q') {
                ////parsed formula
                arg = arg.split('{').join('(');
                arg = arg.split('}').join(')');
                return this.computedValue(arg);
            } else if (arg.length > 1 && arg.substring(0, 2) == "ub") {
                ////parsed formula
                arg = arg.split('{').join('(');
                arg = arg.split('}').join(')');
                var ggg = arg.substring(1);
                ggg = this.computedValue(ggg);
                var d2 = this._parseDouble(ggg);
                if (!isNaN(d2)) {
                    d2 = -d2;
                    return d2.toString();
                }
                return this.computedValue(arg);
            } else {
                if (arg.indexOf("unu") == 0) {
                    arg = "n" + arg.substring(3);
                } else if (arg.indexOf("un") == 0) {
                    arg = "-" + arg.substring(2);
                }

                arg = arg.split('u').join('-');
                if (!this._isUpperChar(arg[0]) && (this._isDigit(arg[0]) || arg[0] == this.getParseDecimalSeparator() || arg[0] == '-' || arg[0] == 'n')) {
                    if (arg[0] == 'n') {
                        arg = arg.substring(1);
                    }

                    d = this._parseDouble(arg);
                    if (!isNaN(d)) {
                        return this._preserveLeadingZeros ? arg : d.toString();
                    } else if (arg.indexOf(this.trueValueStr) == 0 || arg.indexOf(this.falseValueStr) == 0) {
                        return arg;
                    }
                }
            }
            if (this._ignoreCellValue && !(arg.indexOf(this.trueValueStr) == 0 || arg.indexOf(this.falseValueStr) == 0)) {
                this._ignoreCellValue = false;
                return arg;
            }

            var symbolArray = ['+', '-', '/', '*', ')', ')', '{'];

            ////Not a number.
            if ((this._indexOfAny(arg, symbolArray) == -1 && this._isUpperChar(arg[0])) || arg[0] == this.sheetToken) {

                if (((arg != this.trueValueStr && arg != this.falseValueStr))) {
                    var family = CalcEngine.getSheetFamilyItem(this.grid);
                    if (family.sheetNameToParentObject != null && arg.indexOf(this.sheetToken) == -1) {
                        var token = family.parentObjectToToken.getItem(this.grid);
                        arg = token + arg;
                    }
                }
                ////Check for circular reference.
                if (arg == this.cell) {
                    var ht = this.getDependentCells().getItem(arg);
                    if (ht != null && ht.indexOf(arg) > -1) {
                        this._arrayRemove(ht, arg);
                    }
                    if (!this.getDependentFormulaCells().containsKey(this.cell))
                        this.clearFormulaDependentCells(this.cell);
                    throw this.formulaErrorStrings[this._circular_reference_] + arg;
                }

                var s1 = this.getValueFromParentObjectCell(arg);
                if (arg != this.trueValueStr && arg != this.falseValueStr) {
                    d = this._parseDouble(s1.split(this.tic).join(""));
                    if (!this._preserveLeadingZeros && s1.length > 0 && !isNaN(d)) {
                        s1 = d.toString();
                    }

                    this.updateDependencies(arg);
                }
                if (s1 == this.tic + this.tic) {
                    return "NaN";
                }
                var doubleValue;
                tempDate = this._isDate(s1);
                if (this.getExcelLikeComputations() && this.getUseDatesInCalculations() && isNaN(this._parseDouble(s1)) && !isNaN(tempDate.getDate()) && this._dateTime1900 <= tempDate) {
                    s1 = this._toOADate(tempDate);
                }
                return s1;
            }

            ////Must be a formula, so try to parse it and compute.
            arg = arg.split('{').join('(');
            arg = arg.split('}').join(')');
            arg = this._parse(arg);

            d = this._parseDouble(arg.substring(0, arg.length - 1));
            if (arg.indexOf("%") == (arg.length - 1) && !isNaN(d)) {
                arg = (Number(d) / 100).toString();
            }
            if (this.getErrorStrings().indexOf(arg) > -1) {
                return arg;
            }

            return this.computedValue(arg);
        };
        this.getValueFromParentObjectCell = function (cell1) {
            if (cell1 == this.trueValueStr || cell1 == this.falseValueStr) {
                return cell1;
            }

            var i = cell1.lastIndexOf(this.sheetToken);
            var row = 0, col = 0;
            var grd = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (i > -1 && family.tokenToParentObject != null) {
                this.grid = family.tokenToParentObject.getItem(cell1.substring(0, i + 1));
                row = this.rowIndex(cell1);
                col = this.colIndex(cell1);
            } else if (i == -1) {
                var j = 0;
                while (j < cell1.length && this._isLetter(cell1[j])) {
                    j++;
                }
                if (j == cell1.length) {
                    cell1 = cell1.toLowerCase();
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[5].toString();
                } else {
                    row = this.rowIndex(cell1);
                    col = this.colIndex(cell1);
                    if (this.isSheetMember() && family.parentObjectToToken != null) {
                        cell1 = family.parentObjectToToken.getItem(this.grid) + cell1;
                    }
                }
            }

            var saveCell = (this.cell == "" || this.cell == null) ? "" : this.cell;
            this.cell = cell1;

            if (this._iterationMaxCount > 0) {
                if (this._circCheckList.indexOf(this.cell) > -1 && this.computedValueLevel > 0) {
                    this.grid = grd;
                    this.cell = saveCell;

                    if (!this.getIterationValues().containsKey(this.cell)) {
                        this.getIterationValues().add(this.cell, "0");
                    }

                    return this.getIterationValues().getItem(this.cell).toString();
                } else if (this.getIterationValues().containsKey(this.cell) && this.computedValueLevel > 0) {
                    return this.getIterationValues.getItem(this.cell).toString();
                }
            }

            var val = this._getValueComputeFormulaIfNecessary(row, col, this.grid);

            this.grid = grd;
            this.cell = saveCell;
            return val;
        };
        this.getValueFromParentObject = function (grd, row, col) {
            var family = CalcEngine.getSheetFamilyItem(grd);
            var cell1 = (family.parentObjectToToken == null || family.parentObjectToToken.length == 0) ? "" : family.parentObjectToToken.getItem(grd).toString();
            cell1 = cell1 + RangeInfo.getAlphaLabel(col) + row.toString();

            var saveGrid = this.grid;
            var saveCell = this.cell;
            this.cell = cell1;
            this.grid = grd;

            var val = this._getValueComputeFormulaIfNecessary(row, col, grd);
            var tempDate = new Date(Date.parse(val));
            var doubleValue = this._parseDouble(val);
            if (this.getExcelLikeComputations() && this.getUseDatesInCalculations() && isNaN(doubleValue) && !isNaN(tempDate.getDate()) && this._dateTime1900 <= tempDate) {
                val = this._toOADate(tempDate).toString();
            }
            this.grid = saveGrid;
            this.cell = saveCell;
            return val;
        };
        this.getValueFromParentObjectRowCol = function (row, col) {
            return this._getValueComputeFormulaIfNecessary(row, col, this.grid);
        }

        this.handleIteration = function (s, formula) {
            if (this.getFormulaInfoTable().containsKey(s)) {
                this.getFormulaInfoTable.add(s, formula);
            } else {
                this.getFormulaInfoTable.add(s, formula);
            }
            if (this._iterationMaxCount > 0) {
                if (s == this.cell) {
                    this._handleIterations(formula);
                }
            }
            return formula.getFormulaValue();
        };
        this.handleSheetRanges = function (text, family) {
            var sheetLoc = text.indexOf(this.sheetToken);
            var start = 0;
            while (sheetLoc > 0) {
                var colonLoc = this._substring(text, start, sheetLoc).lastIndexOf(':');
                if (colonLoc > -1) {
                    var rightSide = this._substring(text, start + colonLoc + 1, sheetLoc - colonLoc - 1).toUpperCase().split("'").join("");
                    var trimStr = (this._isIE8) ? rightSide.replace(/^\s+|\s+$/g, '') : rightSide.trim();
                    if (family.sheetNameToToken.containsKey(trimStr)) {
                        var leftStart = start + colonLoc - 1;
                        if (this._sortedSheetNames.contains(rightSide)) {
                            leftStart = leftStart - rightSide.length - 1;
                        } else {
                            while (leftStart > 0 && this._markers.indexOf(text[leftStart]) == -1) {
                                leftStart--;
                            }
                        }

                        var leftSide = this._substring(text, leftStart + 1, colonLoc - leftStart + start - 1).toUpperCase().split("'").join("");
                        trimStr = (this._isIE8) ? leftSide.replace(/^\s+|\s+$/g, '') : leftSide.trim();
                        if (family.sheetNameToToken.containsKey(trimStr)) {
                            var rightEnd = sheetLoc + start + 1;
                            if (this._sortedSheetNames.contains(trimStr)) {
                                rightEnd = text.length - 1;
                            } else {
                                while (rightEnd < text.length && this._markers.indexOf(text[rightEnd]) == -1) {
                                    rightEnd++;
                                }
                            }

                            var otherSide = this._substring(text, start + sheetLoc + 1, rightEnd - start - sheetLoc - 1);
                            var left = family.sheetNameToToken.getItem(trimStr).toString();
                            var right = family.sheetNameToToken.getItem(trimStr).toString();

                            ////make sure they are ordered properly
                            if (right < left) {
                                var s = left;
                                left = right;
                                right = s;
                            }

                            var replacement = "";

                            for (var name = 0; name < _sortedSheetNames.length; name++) {
                                var s = family.sheetNameToToken.getItem(this._sortedSheetNames[name]).toString();
                                var sint = parseInt(s.split("!").join(""));
                                var leftint = parseInt(left.split("!").join(""));
                                var rightint = parseInt(right.split("!").join(""));
                                if (sint > leftint && sint < rightint) {
                                    if (replacement.length > 0) {
                                        replacement += this.getParseArgumentSeparator();
                                    }

                                    replacement += name + String.fromCharCode(131) + otherSide;
                                }
                            }

                            text = text.substring(0, leftStart + 1) + replacement + text.substring(rightEnd);
                            start = text.length - rightEnd;
                        } else {
                            start = sheetLoc + start;
                        }
                    } else {
                        start = sheetLoc + start;
                    }
                } else {
                    start = sheetLoc + start;
                }

                sheetLoc = text.substring(start + 1).indexOf(this.sheetToken) + 1;
            }

            text = text.split(String.fromCharCode(131)).join(this.sheetToken);
            return text;
        };
        this.isSheetMember = function () {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            return (family == null || family == undefined) ? false : family.isSheetMember;
        };
        this.parseAndComputeFormula = function (formulaString) {
            var formula = formulaString;
            if (formula.length > 0 && formula[0] == this.getFormulaCharacter()) {
                formula = formula.substring(1);
            }

            if (formula.length > 0 && formula[0] == '+') {
                formula = formula.substring(1);
            }
            if (formula.length > 1 && formula[0] == ',') {
                if (formula[1] >= '0' && formula[1] <= '9')
                    formula = "0" + ',' + formula.substring(1);
            }
            var tempFormula = formula.split(" ").join("");

            for (var i = tempFormula.indexOf("("); i != -1 && i < tempFormula.length; i++) {
                if (i > 0) {
                    if (this._isDigit(tempFormula[i - 1])) {
                        if (!this._checkHasCharBeforeNumber(tempFormula.substring(0, i)))
                            throw (this.formulaErrorStrings[this._bad_formula] + " " + tempFormula.substring(0, i + 1));
                    }
                    var next = tempFormula.substring(i + 1).indexOf('(');
                    if (next == -1) {
                        break;
                    } else {
                        i += next;
                    }
                }
            }
            for (var i = tempFormula.indexOf(")"); i != -1 && i < tempFormula.length - 1; i++) {
                if (this._isDigit(tempFormula[i + 1])) {
                    throw (this.formulaErrorStrings[this._bad_formula] + " " + tempFormula.substring(0, i + 2));
                }
                var next = tempFormula.substring(i + 1).indexOf(')');
                if (next == -1) {
                    break;
                } else {
                    i += next;
                }
            }
            this._multiTick = false;

            var s = this._parse(formula);
            return this.computedValue(s);
        };
        this.parseFormula = function (formula) {
            if (formula.length > 0 && formula[0] == this.getFormulaCharacter()) {
                formula = formula.substring(1);
            }

            if (formula.length > 0 && formula[0] == '+') {
                formula = formula.substring(1);
            }

            this._isRangeOperand = this._supportRangeOperands && this._isRange(formula);
            if (this.getCheckDanglingStack() && (formula.split(" ").join("")).indexOf(this._braceRightNLeft) > -1) {
                this.computedValueLevel = 0;
                return this.formulaErrorStrings[this._improper_formula];
            }
            var trimStr = (this._isIE8) ? formula.replace(/^\s+|\s+$/g, '') : formula.trim();
            return this._parse(trimStr);
        };
        this.getUpdatedValueCell = function (cellRef) {
            var isUseFormulaValueChanged = false;
            this._inAPull = true;
            var grd = this.grid;
            var saveCell = this.cell;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            var s = cellRef.toUpperCase();
            var i;
            if ((i = s.indexOf(this.sheetToken)) == -1 && this.cell != "" && this.cell != undefined) {
                i = this.cell.indexOf(this.sheetToken, 1);
                if (i > -1 && family.tokenToParentObject != null) {
                    s = this.cell.substring(0, i + 1) + s;
                    this.grid = family.tokenToParentObject.getItem(this.cell.substring(0, i + 1));
                }
            } else {
                if (i > 0 && family.sheetNameToToken != null && family.tokenToParentObject != null) {
                    ////need sheet id
                    var token = family.sheetNameToToken.getItem(s.substring(0, i));
                    s = token + s.substring(i + 1);
                    this.grid = family.tokenToParentObject.getItem(token);
                    this.cell = s;
                }
            }

            this.updateCalcID();
            var txt;
            if (!this.getDependentFormulaCells().containsKey(s) && !this.getFormulaInfoTable().containsKey(s)) {
                txt = this.getValueFromParentObjectCell(s);
                while (this._breakedFormulaCells.length == 1) {
                    this.updateCalcID();
                    txt = this.getValueFromParentObjectCell(this._breakedFormulaCells[0].toString());
                    this._breakedFormulaCells.removeAt(0);
                    this.setUseFormulaValues(true);
                }
                if (this.getUseFormulaValues()) {
                    this.isUseFormulaValueChanged = true;
                    this.setUseFormulaValue(false);
                }
                if (this._tempBreakedFormulaCells.length > 0) {
                    for (var p = 1; p <= this._tempBreakedFormulaCells.length; p++) {
                        txt = this.getValueFromParentObjectCell(this._tempBreakedFormulaCells[this._tempBreakedFormulaCells.length - p].toString());
                    }
                    this._tempBreakedFormulaCells.length = 0;
                    txt = this.getValueFromParentObjectCell(s);
                }
                this.setUseFormulaValues(this.isUseFormulaValueChanged);
                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;
                var row = this.rowIndex(s);

                var col = this.colIndex(s);
                if (this.getPreserveFormula()) {
                    var token = this._sheetToken(cellRef);
                    if (token == "") {
                        token = "!" + this.getSheetID(this.grid) + "!";
                        var formula = this.getFormulaInfoTable().getItem(token + s);
                        if (formula != null && formula != undefined) {
                            if (this.parentObject.setValueRowCol == undefined)
                                this.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                            else
                                this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                        }
                    } else if (this.parentObject.setValueRowCol == undefined)
                        this.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                    else
                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                } else if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                else
                    this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);

                this.ignoreValueChanged = saveIVC;
            } else {
                this._processedCells.clear();
                this.updateDependenciesAndCell(s);
                this._processedCells.clear();
                txt = this.getValueFromParentObject(s);
            }

            this.grid = grd;
            this.cell = saveCell;
            this._inAPull = false;

            return txt;
        };
        this.getUpdatedValueRowCol = function (targetSheetID, row, col) {
            var saveGrid = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            var token = this.sheetToken + targetSheetID.toString() + this.sheetToken;
            this.grid = family.tokenToParentObject.getItem(token);
            this._inAPull = true;
            var s = token + RangeInfo.getAlphaLabel(col) + row.toString();

            this.updateCalcID();

            if (!this.getDependentFormulaCells().containsKey(s) && !this.formulaInfoTable.containsKey(s)) {
                var txt = this.getValueFromParentObjectCell(s);
                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;
                if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                else
                    this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, txt, row, col);
                this.ignoreValueChanged = saveIVC;
            } else {
                this._processedCells.clear();
                this.updateDependenciesAndCell(s);
                this._processedCells.clear();
            }

            this.grid = saveGrid;
            this._inAPull = false;
        };
        this.setTokensForSheets = function (text) {
            var family = CalcEngine.getSheetFamilyItem(this.grid);

            if (this.getSupportsSheetRanges()) {
                text = this.handleSheetRanges(text, family);
            }

            var sortedSheetNamesCollection = this.getSortedSheetNames();

            ////family.SheetNameToToken.Keys)
            if (sortedSheetNamesCollection != null) {
                for (var name = 0; name < sortedSheetNamesCollection.length; name++) {
                    var token = family.sheetNameToToken.getItem(sortedSheetNamesCollection[name]);
                    token = token.split(this.sheetToken).join(this._tempSheetPlaceHolder);

                    ////keep the tic defect#541 - TODO: Optimize this process - cache tic names for instance
                    var s = "'" + sortedSheetNamesCollection[name].toUpperCase() + "'" + this.sheetToken;
                    if (text.indexOf(s) == -1) {
                        s = sortedSheetNamesCollection[name].toUpperCase() + this.sheetToken;
                    }

                    text = text.split(s).join(token);

                    //do it again without the tics
                    s = sortedSheetNamesCollection[name].toUpperCase() + this.sheetToken;

                    text = text.split(s).join(token);
                }
            }

            text = text.split(this._tempSheetPlaceHolder).join(this.sheetToken);
            return text;
        };
        this.recalculateRange = function (range, data) {
            this._inRecalculateRange = true;
            for (var row = range.getTop(); row <= range.getBottom(); ++row) {
                for (var col = range.getLeft() ; col <= range.getRight() ; ++col) {
                    if (this.parentObject.getValueRowCol != undefined)
                        this.parentObject.setValueRowCol(this.getSheetID(data) + 1, this.parentObject.getValueRowCol(this.getSheetID(data) + 1, row, col), row, col);
                    else
                        data.setValueRowCol(this.getSheetID(data) + 1, data.getValueRowCol(this.getSheetID(data) + 1, row, col), row, col);
                }
            }
            this._inRecalculateRange = false;
        };
        this.refresh = function (s) {
            ////Don't refresh any cells.
            if (this.getCalculatingSuspended()) {
                return;
            }

            if (this._dependencyLevel == 0) {
                this._refreshedCells.clear();
            }

            if (this.getDependentCells().containsKey(s) && this.getDependentCells().getItem(s) != null) {
                this._dependencyLevel++;
                try  {
                    var family = CalcEngine.getSheetFamilyItem(this.grid);

                    var ht = this.getDependentCells().getItem(s);
                    var save1 = this._lockDependencies;
                    this._lockDependencies = true;
                    for (var o = 0; o < ht.length; o++) {
                        var s1 = ht[o];
                        if (s1 != null) {
                            var grd = this.grid;
                            var sheet = this._sheetToken(s1);
                            if (sheet.length > 0) {
                                this.grid = family.tokenToParentObject.getItem(sheet);
                            }
                            try  {
                                var row = this.rowIndex(s1);
                                var col = this.colIndex(s1);

                                var info = this.getFormulaInfoTable().getItem(s1);
                                if (info != null) {
                                    var save = this.cell;
                                    this.cell = s1;
                                    if (this.getAlwaysComputeDuringRefresh() || info.calcID != this._calcID || info.getFormulaValue() == "") {
                                        info.setFormulaValue(this.computeFormula(info.getParsedFormula()));
                                    }

                                    info.calcID = this._calcID;
                                    this.cell = save;
                                    var saveIVC = this.ignoreValueChanged;
                                    this.ignoreValueChanged = true;
                                    if (this.parentObject.setValueRowCol == undefined)
                                        this.setValueRowCol(this.getSheetID(this.grid) + 1, info.getFormulaValue(), row, col);
                                    else
                                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, info.getFormulaValue(), row, col);
                                    this.ignoreValueChanged = saveIVC;
                                    if (!this._refreshedCells.containsKey(s1)) {
                                        this._refreshedCells.add(s1, 0); ////from below ...a
                                        this.refresh(s1); ////recursive call
                                    }
                                }
                            } catch (ex) {
                                continue;
                            }
                            this.grid = grd;
                        }
                    }

                    this._lockDependencies = save1;
                } finally {
                    if (!this._refreshedCells.containsKey(s)) {
                        this._refreshedCells.add(s, 0);
                    }

                    this._dependencyLevel--;
                    if (this._dependencyLevel == 0) {
                        this._refreshedCells.clear();
                    }
                }
            }
        };
        this.refreshRange = function (range) {
            for (var r = range.getTop(); r <= range.getBottom(); r++) {
                for (var c =range.getLeft(); c <= range.getRight(); c++) {
                    var s = RangeInfo.getAlphaLabel(c) + r.toString();
                    this._dependencyLevel = 0;
                    var family = CalcEngine.getSheetFamilyItem(this.grid);
                    var token = this.sheetToken + this.getSheetID(this.grid).toString() + this.sheetToken;
                    if (family.tokenToParentObject != null && family.tokenToParentObject.contains(token))
                        this.grid = family.tokenToParentObject.getItem(token);

                    s = token + s;
                    this.refresh(s);
                }
            }
        };
        this.registerGridAsSheet = function (refName, model, sheetFamilyID) {
            ////refName = refName.replace(" ", ""); //keep the spaces defect#541
            if (CalcEngine.modelToSheetID != null) {
                // this.wireParentObject();
            }


            if (CalcEngine.modelToSheetID == null) {
                CalcEngine.modelToSheetID = new HashTable();
            }

            if (CalcEngine.modelToSheetID.getItem(model) == null || CalcEngine.modelToSheetID.getItem(model) == undefined) {
                CalcEngine.modelToSheetID.add(model, sheetFamilyID);
            }

            var family = CalcEngine.getSheetFamilyItem(model);
            family.isSheetMember = true;

            var refName1 = refName.toUpperCase();
            if (family.sheetNameToParentObject == null) {
                family.sheetNameToParentObject = new HashTable();
            }

            if (family.tokenToParentObject == null) {
                family.tokenToParentObject = new HashTable();
            }

            if (family.sheetNameToToken == null) {
                family.sheetNameToToken = new HashTable();
            }

            if (family.parentObjectToToken == null) {
                family.parentObjectToToken = new HashTable();
            }

            if (family.sheetNameToParentObject.getItem(refName1) != undefined) {
                var token = family.sheetNameToToken.getItem(refName1);
                family.tokenToParentObject.add(token, model);
                family.parentObjectToToken.add(model, token);
            } else {
                var token = this.sheetToken + CalcEngine._tokenCount.toString() + this.sheetToken;
                CalcEngine._tokenCount++;
                family.tokenToParentObject.add(token, model);
                family.sheetNameToToken.add(refName1, token);
                family.sheetNameToParentObject.add(refName1, model);
                family.parentObjectToToken.add(model, token);
                this.sortedSheetNames = null;
            }
            return refName;
        };

        this.removeFunction = function (name) {
            if (this._customlibraryFunctions.getItem(name) != undefined) {
                this._customlibraryFunctions.remove(name);
            }
            if (this._libraryFunctions.getItem(name) != null) {
                this._libraryFunctions.remove(name);
                return true;
            }
            return false;
        };
        this.removeNamedRange = function (name) {
            name = name.toUpperCase();
            if (this.getNamedRanges().getItem(name) != null) {
                this.getNamedRanges().remove(name);
                this._populateNamedRangesNonScoped();
                return true;
            }
            return false;
        };
        this.rowIndex = function (s) {
            var i = 0;
            var result;

            var isLetter = false;
            if (i < s.length && s[i] == this.sheetToken) {
                i++;
                while (i < s.length && s[i] != this.sheetToken) {
                    i++;
                }

                i++;
            }

            while (i < s.length && this._isLetter(s[i])) {
                isLetter = true;
                i++;
            }
            result = parseInt(s.substring(i));
            if (i < s.length && !isNaN(result)) {
                return result;
            }

            if (isLetter) {
                return -1;
            }
            throw this.formulaErrorStrings[this._bad_index];
        };
        this.splitArgsPreservingQuotedCommas = function (args) {
            if (args.toString().indexOf(this.tic) == -1) {
                return args.toString().split(this._parseArgumentSeparator);
            }

            var formulaStrings = this._saveStrings(args);
            args = this._saveStringsText;
            var results = args.split(this._parseArgumentSeparator);
            var pieces = [];
            for (var s = 0; s < results.length; s++) {
                var s1 = results[s];
                s1 = this._setStrings(s1, formulaStrings); ////replace tokens with original strings
                pieces.push(s1);
            }
            return pieces;
        };
        this.removeTics = function (s) {
            if (s.length > 1 && s[0] == this.tic[0] && s[s.length - 1] == this.tic[0]) {
                if (this._substring(s, 1, s.length - 2).indexOf(this.tic) == -1) {
                    s = this._substring(s, 1, s.length - 2);
                } else if (this._multiTick) {
                    s = this._substring(s, 1, s.length - 2);
                }
            }
            return s;
        };
        this.updateCalcID = function () {
            this._calcID++;
            if (this._calcID == Number.MAX_SAFE_INTEGER) {
                this._calcID = Number.MIN_SAFE_INTEGER + 1;
            }
        };
        this.updateDependencies = function (s) {
            ////TraceUtil.TraceCurrentMethodInfoIf(Switches.FormulaCell.TraceVerbose, this);
            if (this._lockDependencies || !this.getUseDependencies()) {
                return;
            }
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            var cell1 = this.cell;
            if (family.sheetNameToParentObject != null && cell1.indexOf(this.sheetToken) == -1) {
                var token = family.parentObjectToToken.getItem(this.grid);
                cell1 = token + cell1;
            }
            if (family.sheetNameToParentObject != null && s.indexOf(this.sheetToken) == -1) {
                var token = family.parentObjectToToken.getItem(this.grid);
                s = token + s;
            }

            if (!this.getDependentCells().containsKey(s)) {
                var ht1 = [];
                this.getDependentCells().add(s, ht1);

                ht1.push(cell1);

                if (this.getDependentFormulaCells().containsKey(s)) {
                    var ht = this.getDependentFormulaCells()[s];
                    if (ht != null) {
                        var keys = ht.keys();
                        for (var o = 0; o < keys.length; o++) {
                            var s1 = keys[o];
                            var ht2 = this.getDependentCells().getItem(s1);
                            if (ht2 == null) {
                                ht2 = [];
                                this.getDependentCells().add(s1, ht2);
                            }

                            if (ht2.indexOf(cell1) == -1) {
                                ht2.push(cell1);
                            }
                        }
                    }
                }

                ////	else
                this._addToFormulaDependentCells(s);

                if (this.getDependentCells().containsKey(cell1)) {
                    var htCells = this.getDependentCells().getItem(cell1);
                    if (htCells != null && htCells != ht1) {
                        for (var o = 0; o < htCells.length; o++) {
                            var s1 = htCells[o];
                            ht1.push(s1);
                        }
                    }
                }
            } else {
                var ht1 = this.getDependentCells().getItem(s);

                if (ht1.indexOf(cell1) == -1) {
                    ht1.push(cell1);
                }

                this._addToFormulaDependentCells(s);

                ////maybe remove this block ..... not needed if other dependencies working OK
                if (this.getDependentFormulaCells().containsKey(s)) {
                    var ht = this.getDependentFormulaCells().getItem(s);
                    if (ht != null) {
                        var keys = ht.keys();
                        for (var o = 0; o < keys.length; o++) {
                            var s1 = keys[o];
                            var ht2 = this.getDependentCells().getItem(s1);
                            if (ht2 == null) {
                                ht2 = [];
                                this.getDependentCells().add(s1, ht2);
                            }
                            if (ht2.indexOf(cell1) == -1) {
                                ht2.push(cell1);
                            }
                        }
                    }
                }
            }
        };
        this.updateDependenciesAndCell = function (cell1) {
            var grd = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);

            var sheet = this._sheetToken(cell1);
            if (sheet.length > 0) {
                this.grid = family.tokenToParentObject.getItem(sheet);
            }

            if (this.formulaInfoTable.containsKey(cell1)) {
                var formula = this.formulaInfoTable[cell1];
                if (formula.calcID != this._calcID) {
                    var saveCell = this.cell;
                    this.cell = cell1;

                    if (this._iterationMaxCount > 0 && this._circCheckList.indexOf(this.cell) > -1) {
                        this._handleIterations(formula);
                    } else {
                        formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                    }

                    if (this.getDependentCells().containsKey(cell1)) {
                        var ht = this.getDependentCells().getItem(cell1);
                        for (var s = 0; s < ht.length; s++) {
                            var f = this.FormulaInfoTable.getItem(ht[s]);
                            if (f != null && f != undefined) {
                                f.calcID = Number.MIN_SAFE_INTEGER; ////mark as dirty
                            }
                        }
                    }

                    formula.calcID = this._calcID;
                    this.cell = saveCell;
                }

                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;

                var row = this.rowIndex(cell1);
                var col = this.colIndex(cell1);
                if (this.getPreserveFormula()) {
                    if (this.parentObject.setValueRowCol == undefined)
                        this.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                    else
                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaText(), row, col);
                }
                this.ignoreValueChanged = saveIVC;

                this.grid = grd;

                if (this._processedCells.contains(cell1)) {
                    return;
                } else {
                    this._processedCells.add(cell1);
                }
                if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaValue(), row, col);
                else
                    this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, formula.getFormulaValue(), row, col);
                if (this.getDependentFormulaCells().containsKey(cell1)) {
                    var hthash = this.getDependentFormulaCells().getItem(cell1);
                    var keys = hthash.keys();
                    for (var c = 0; c < keys.length; c++) {
                        this.updateDependenciesAndCell(keys[c]); ////recursive call
                    }
                }
            }
        };
        this.valueChanged = function (grid, e) {
            if (this.ignoreValueChanged) {
                return;
            }

            var grd = grid;
            this.grid = grid;

            var computedValueChanged = true;
            var family = CalcEngine.getSheetFamilyItem(grd);
            var s = RangeInfo.getAlphaLabel(e.getColIndex()) + e.getRowIndex().toString();
            if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.length > 0) {
                var token = family.parentObjectToToken.getItem(grd);
                s = token + s;
            }

            if (e.getValue().length > 0 && e.getValue()[0] == this.getFormulaCharacter()) {
                this.cell = s;
                var formula;
                var compute = true;
                if (this.getFormulaInfoTable().containsKey(s)) {
                    formula = this.getFormulaInfoTable().getItem(s);
                    if (e.getValue() != formula.getFormulaText() || formula.getParsedFormula() == null) {
                        formula.setFormulaText(e.getValue());
                        if (this.getDependentFormulaCells().containsKey(this.cell))
                            this.clearFormulaDependentCells(this.cell);
                        try  {
                            formula.setParsedFormula(this.parseFormula(e.getValue()));
                        } catch (ex) {
                            formula.setFormulaValue(ex);
                            compute = false;
                        }
                    }

                    if (compute) {
                        var s1 = this.computeFormula(formula.getParsedFormula());

                        computedValueChanged = (s1 != formula.getFormulaValue()) || this.getForceRefreshCall();

                        formula.setFormulaValue(s1);
                    }

                    formula.calcID = this._calcID;
                } else {
                    formula = new FormulaInfo();
                    formula.setFormulaText(e.getValue());
                    if (!this.getDependentFormulaCells().containsKey(s)) {
                        this.getDependentFormulaCells().add(s, new HashTable());
                    }

                    try  {
                        formula.setParsedFormula(this.parseFormula(e.getValue()));
                    } catch (ex) {
                        formula.setFormulaValue(ex);
                        compute = false;
                    }

                    if (compute) {
                        formula.setFormulaValue(this.computeFormula(formula.getParsedFormula()));
                        if (this.computeIsFormula(formula).toString() == "#NAME?" && !(this.getNamedRanges().containsKey(formula.getFormulaText().split("=")[1].toUpperCase())))
                            this.getUndefinedNamedRanges().add(formula.getFormulaText().split("=")[1].toUpperCase());
                    }

                    formula.calcID = this._calcID;

                    if (this.getFormulaInfoTable().containsKey(s)) {
                        this.getFormulaInfoTable().add(s, formula);
                    } else {
                        this.getFormulaInfoTable().add(s, formula);
                    }
                }

                if (this._iterationMaxCount > 0 && compute && !this._inHandleIterations) {
                    if (s == this.cell) {
                        this._handleIterations(formula);
                    }
                }

                var saveIVC = this.ignoreValueChanged;
                this.ignoreValueChanged = true;
                if (this.parentObject.setValueRowCol == undefined)
                    this.setValueRowCol(this.getSheetID(grd) + 1, formula.getFormulaValue(), e.getRowIndex(), e.getColIndex());
                else
                    this.parentObject.setValueRowCol(this.getSheetID(grd) + 1, formula.getFormulaValue(), e.getRowIndex(), e.getColIndex());
                if (formula != null && formula.getFormulaValue() == "" && e.getValue().startsWith(this.getFormulaCharacter())) {
                    if (this.parentObject.setValueRowCol == undefined)
                        this.setValueRowCol(this.getSheetID(this.grid) + 1, "0", e.getRowIndex(), e.getColIndex());
                    else
                        this.parentObject.setValueRowCol(this.getSheetID(this.grid) + 1, "0", e.getRowIndex(), e.getColIndex());
                }
                this.ignoreValueChanged = saveIVC;
            } else if (!this._inRecalculateRange && this.getFormulaInfoTable().containsKey(s)) {
                this.getFormulaInfoTable().remove(s);
                if (this.getDependentFormulaCells().containsKey(s))
                    this.clearFormulaDependentCells(s);
            }

            if (computedValueChanged && this.getDependentCells().containsKey(s)) {
                this._dependencyLevel = 0;
                this.refresh(s);
            }
        };
        this.getActiveCell = function () {
            return this.cell;
        };
        this.getAllowShortCircuitIFs = function () {
            return this._allowShortCircuitIFs;
        };
        this.setAllowShortCircuitIFs = function (value) {
            this._allowShortCircuitIFs = value;
        };
        this.getAlwaysComputeDuringRefresh = function () {
            return this._alwaysComputeDuringRefresh;
        };
        this.setAlwaysComputeDuringRefresh = function (value) {
            this._alwaysComputeDuringRefresh = value;
        };
        this.getCalculatingSuspended = function () {
            return this._calculatingSuspended;
        };
        this.setCalculatingSuspended = function (value) {
            this._calculatingSuspended = value;
        };
        this.getCheckDanglingStack = function () {
            return this._checkDanglingStack;
        };
        this.setCheckDanglingStack = function (value) {
            this._checkDanglingStack = value;
        };
        this.getCustomLibraryFunctions = function () {
            return this._customlibraryFunctions;
        };
        this.getDependentCells = function () {
            if (this.isSheetMember()) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetDependentCells == null) {
                    family.sheetDependentCells = new HashTable();
                }

                return family.sheetDependentCells;
            } else {
                if (this._dependentCells == null) {
                    this._dependentCells = new HashTable();
                }

                return this._dependentCells;
            }
        };
        this.getDependentFormulaCells = function () {
            if (this.isSheetMember()) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetDependentFormulaCells == null) {
                    family.sheetDependentFormulaCells = new HashTable();
                }

                return family.sheetDependentFormulaCells;
            } else {
                if (this._dependentFormulaCells == null) {
                    this._dependentFormulaCells = new HashTable();
                }

                return this._dependentFormulaCells;
            }
        };
        this.getEnableLookupTableCaching = function () {
            return this._enableLookupTableCaching;
        };
        this.setEnableLookupTableCaching = function (value) {
            this._enableLookupTableCaching = value;
        };
        this.getErrorStrings = function () {
            if (this._errorStrings == null) {
                this._errorStrings = ["#N/A", "#VALUE!", "#REF!", "#DIV/0!", "#NUM!", "#NAME?", "#NULL!"];
            }
            return this._errorStrings;
        };
        this.setErrorStrings = function (value) {
            this._errorStrings = value;
        };
        this.getExcelLikeComputations = function () {
            return this._excelLikeComputations;
        };
        this.setExcelLikeComputations = function (value) {
            this._excelLikeComputations = value;
        };
        this.getForceRefreshCall = function () {
            return this._forceRefreshCall;
        };
        this.setForceRefreshCall = function (value) {
            this._forceRefreshCall = value;
        };
        this.getFormulaCharacter = function () {
            if (CalcEngine._formulaChar == '\0') {
                CalcEngine._formulaChar = '=';
            }
            return CalcEngine._formulaChar;
        };
        this.setFormulaCharacter = function (value) {
            CalcEngine._formulaChar = value;
        };
        this.getFormulaInfoTable = function () {
            if (this.isSheetMember()) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family.sheetFormulaInfoTable == null) {
                    family.sheetFormulaInfoTable = new HashTable();
                }

                return family.sheetFormulaInfoTable;
            } else {
                if (this._formulaInfoTable == null) {
                    this._formulaInfoTable = new HashTable();
                }

                return this._formulaInfoTable;
            }
        };
        this.getPreserveLeadingZeros = function () {
            return this._preserveLeadingZeros;
        };
        this.setPreserveLeadingZeros = function (value) {
            this._preserveLeadingZeros = value;
        };
        this.getIterationMaxCount = function () {
            return this._iterationMaxCount;
        };
        this.setIterationMaxCount = function (value) {
            this._iterationMaxCount = value;
            if (this._iterationMaxCount > 0) {
                this.setThrowCircularException(true);
            }
        };
        this.getIterationMaxTolerance = function () {
            return this._iterationMaxTolerance;
        };
        this.setIterationMaxTolerance = function (value) {
            this._iterationMaxTolerance = value;
        };
        this.getIterationValues = function () {
            if (this._iterationValues == null) {
                this._iterationValues = new HashTable();
            }
            return this._iterationValues;
        };
        this.getLibraryComputationException = function () {
            return this._libraryComputationException;
        };
        this.getLibraryFunctions = function () {
            return this._libraryFunctions;
        };
        this.getLockDependencies = function () {
            return this._lockDependencies;
        };
        this.setLockDependencies = function (value) {
            this._lockDependencies = value;
        };
        this.getMaximumRecursiveCalls = function () {
            return this._maximumRecursiveCalls;
        };
        this.setMaximumRecursiveCalls = function (value) {
            this._maximumRecursiveCalls = value;
        };
        this.getNamedRanges = function () {
            if (this.namedRanges == null) {
                this.namedRanges = new HashTable();
                this._namedRangesNonScoped = new HashTable();
            }
            return this.namedRanges;
        };
        this.setNamedRanges = function (value) {
            this.namedRanges = value;
            this._populateNamedRangesNonScoped();
        };
        this.getUndefinedNamedRanges = function () {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family != null || family != undefined) {
                if (this.undefinedsheetNamedRnages == null) {
                    this.undefinedsheetNamedRnages = new HashTable();
                }
                return this.undefinedsheetNamedRnages;
            }
            else {
                if (this.undefinednamedRange == null) {
                    this.undefinednamedRange = new HashTable();
                }
                return this.undefinednamedRange;
            }
        };
        this.getNameRangeValues = function () {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family != null || family != undefined) {
                if (this.namedRangeValues == null) {
                    this.namedRangeValues = new HashTable();
                }
                return this.namedRangeValues;
            }
            else {
                if (this.rangeValues == null) {
                    this.rangeValues = new HashTable();
                }
                return this.rangeValues;
            }
        };
        this.getNamedRangesOriginalNames = function () {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family != null || family != undefined) {
                if (this.sheetNamedRangesOriginalNames == null) {
                    this.sheetNamedRangesOriginalNames = new HashTable();
                }
                return this.sheetNamedRangesOriginalNames;
            }
            else {
                if (this.namedRangesOriginalNames == null) {
                    this.namedRangesOriginalNames = new HashTable();
                }
                return this.namedRangesOriginalNames;
            }
        };
        this.getNamedRangeCellCollection = function () {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family != null || family != undefined) {
                if (this.sheetNamedRangeCellCollection == null) {
                    this.sheetNamedRangeCellCollection = new HashTable();
                }
                return this.sheetNamedRangeCellCollection;
            }
            else {
                if (this.namerangecellcollection == null) {
                    this.namerangecellcollection = new HashTable();
                }
                return this.namerangecellcollection;
            }
        };
        this.getDependentNamedRangeCells = function () {
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (family != null || family != undefined) {
                if (this.sheetDependentNamedRangeCells == null) {
                    this.sheetDependentNamedRangeCells = new HashTable();
                }
                return this.sheetDependentNamedRangeCells;
            }
            else {
                if (this.dependentNamedRangeCells == null) {
                    this.dependentNamedRangeCells = new HashTable();
                }
                return this.dependentNamedRangeCells;
            }
        };
        this.getParseArgumentSeparator = function () {
            if (this._parseArgumentSeparator == '\0') {
                this._parseArgumentSeparator = ',';
            }
            var seperator = ',';
            if (!this._isParseArgumentSeparator && seperator != this._parseArgumentSeparator) {
                this._parseArgumentSeparator = seperator;
            }
            return this._parseArgumentSeparator;
        };
        this.setParseArgumentSeparator = function (value) {
            this._parseArgumentSeparator = value;
            this._isParseArgumentSeparator = true;
        };
        this.getParseDateTimeSeparator = function () {
            return this._parseDateTimeSeparator;
        };
        this.setParseDateTimeSeparator = function (value) {
            this._parseDateTimeSeparator = value;
        };
        this.getParseDecimalSeparator = function () {
            ////DT 26058 incident
            ////handle problems with threading by dynamically assigning the static values
            if (this._parseDecimalSeparator == '\0') {
                this._parseDecimalSeparator = '.';
            }

            var seperator = '.';
            if (!this._isParseDecimalSeparatorChanged && seperator != this._parseDecimalSeparator) {
                this._parseDecimalSeparator = seperator;
            }

            return this._parseDecimalSeparator;
        };
        this.setParseDecimalSeparator = function (value) {
            this._parseDecimalSeparator = value;
            this._parseDecimalSeparator = true;
        };
        this.getPreserveFormula = function () {
            return this._preserveFormula;
        };
        this.setPreserveFormula = function (value) {
            this._preserveFormula = value;
        };
        this.getReservedWordOperators = function () {
            if (this._reservedWordOperators == null) {
                this._reservedWordOperators = [" or ", " and ", " xor ", "if ", " then ", " else ", "not "];
            }

            return this._reservedWordOperators;
        };
        this.setReservedWordOperators = function (value) {
            this._reservedWordOperators = value;
        };
        this.getRethrowLibraryComputationExceptions = function () {
            return this._rethrowLibraryComputationExceptions;
        };
        this.setRethrowLibraryComputationExceptions = function (value) {
            this._rethrowLibraryComputationExceptions = value;
        };
        this.getRethrowParseExceptions = function () {
            return this._rethrowExceptions;
        };
        this.setRethrowParseExceptions = function (value) {
            this._rethrowExceptions = value;
        };
        this.getRowMaxCount = function () {
            return this._rowMaxCount;
        };
        this.setRowMaxCount = function (value) {
            this._rowMaxCount = value;
        };
        this.getSortedSheetNames = function () {
            if (this._sortedSheetNames == null) {
                var family = CalcEngine.getSheetFamilyItem(this.grid);
                if (family != null && family.sheetNameToToken != null) {
                    this._sortedSheetNames = family.sheetNameToToken.keys();
                    this._sortedSheetNames.sort(); //(new LenComparer());
                }
            }

            return this._sortedSheetNames;
        };
        this.getSupportLogicalOperators = function () {
            return this._supportLogicalOperators;
        };
        this.setSupportLogicalOperators = function (value) {
            this._supportLogicalOperators = value;
        };
        this.getSupportRangeOperands = function () {
            return this._supportRangeOperands;
        };
        this.setSupportRangeOperands = function (value) {
            this._supportRangeOperands = value;
        };
        this.getSupportsSheetRanges = function () {
            return this._supportsSheetRanges;
        };
        this.setSupportsSheetRanges = function (value) {
            this._supportsSheetRanges = value;
        };
        this.getThrowCircularException = function () {
            return this._throwCircularException;
        };
        this.setThrowCircularException = function (value) {
            this._throwCircularException = value;
        };
        this.getTreatStringsAsZero = function () {
            return this._treatStringsAsZero;
        };
        this.setTreatStringsAsZero = function (value) {
            this._treatStringsAsZero = value;
        };
        this.getUseDatesInCalculations = function () {
            return this._useDatesInCalcs;
        };
        this.setUseDatesInCalculations = function (value) {
            this._useDatesInCalcs = value;
        };
        this.getUseDependencies = function () {
            return this._useDependencies;
        };
        this.setUseDependencies = function (value) {
            this._useDependencies = value;
        };
        this.getUseFormulaValues = function () {
            return this._useFormulaValues;
        };
        this.setUseFormulaValues = function (value) {
            this._useFormulaValues = value;
        };
        this.getUseNoAmpersandQuotes = function () {
            return this._useNoAmpersandQuotes;
        };
        this.setUseNoAmpersandQuotes = function (value) {
            this._useNoAmpersandQuotes = value;
        };
        this.getValidPrecedingChars = function () {
            return this._validPrecedingChar + this.getParseArgumentSeparator();
        };
        this.setValidPrecedingChars = function (value) {
            this._validPrecedingChars = value;
        };
        this.getWeekEndType = function () {
            return ["", "6,0", "0,1", "1,2", "2,3", "3,4", "4,5", "5,6", "", "", "", "0", "1", "2", "3", "4", "5", "6"];
        };
        this.computeSum = function (range) {
            var sum = 0, s1, d, adjustRange;
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var key = 0; key < ranges.length; key++) {
                adjustRange = ranges[key];
                if (adjustRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s = 0; s < cellCollection.length; s++) {
                        try  {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join("");
                        } catch (ex) {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1 != "") {
                            d = (s1 == this.trueValueStr) ? true : (s1 == this.falseValueStr) ? false : this._parseDouble(s1);
                            if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            }
                            if (isNaN(d)) {
                                s1 = "";
                            } else {
                                sum = sum + d;
                            }
                        }
                    }
                } else {
                    try  {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join("");
                    } catch (ex) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = (s1 == this.trueValueStr) ? true : (s1 == this.falseValueStr) ? false : this._parseDouble(s1);
                        if (this.getErrorStrings().indexOf(s1) > -1) {
                            return s1;
                        }
                        if (!isNaN(d)) {
                            sum = sum + d;
                        }
                    }
                }
            }
            return sum.toString();
        };

        this.computeDate = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }

            var year = this._parseDouble(args[0]);
            var month = this._parseDouble(args[1]);
            var day = this._parseDouble(args[2]);
            var days = 0;

            if (!isNaN(year) && !isNaN(month) && !isNaN(day)) {
                if (year < 0) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[4].toString();
                    return this.getErrorStrings()[4].toString();
                }
                while (month > 12) {
                    month -= 12;
                    year++;
                }

                days = this._getSerialDateFromDate(year, month, day);
            }
            if (this._excelLikeComputations) {
                var date = this._fromOADate(days);
                if (date.toString() != "Invalid Date") {
                    return date.toLocaleDateString();
                }
            }
            return days.toString();
        };
        this.computeDatevalue = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var dt = new Date(Date.parse(args[0]));
            var dtNum = this._parseDouble(args[0]);
            try  {
                if (!isNaN(dtNum) || dt.toString() == "Invalid Date") {
                    if (dtNum > 0) {
                        dt = this._fromOADate(dtNum);
                    } else {
                        var ranges = argList.split(this.getParseDateTimeSeparator());
                        var argCount = ranges.length;
                        var arg = "";
                        for (var i = 0; i < argCount; ++i) {
                            ranges[i] = this.computedValue(ranges[i]);
                            if (i < argCount - 1)
                                arg += ranges[i] + this.getParseDateTimeSeparator();
                            else
                                arg += ranges[i];
                        }
                        arg = arg.split(this.tic).join(this._string_empty);
                        dt = new Date(Date.parse(arg));
                    }
                }
            } catch (ex) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            if (args[0].indexOf("2001") == -1 && dt.getFullYear() == 2001) {
                dt.setFullYear((new Date(Date.now())).getFullYear());
            }
            var days = this._toOADate(dt);
            if (isNaN(days)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            if (this.treat1900AsLeapYear && days > 59) {
                days += 1;
            }

            return Math.round(days).toString();
        };
        this.computeDay = function (argList) {
            var day = 1;
            var s = this.getValueFromArg(argList).split(this.tic).join("");
            var serialdate = this._parseDouble(s);
            if (!isNaN(serialdate)) {
                if (serialdate < 1) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._invalid_Math_argument];
                    return this.getErrorStrings()[4].toString();
                } else {
                    date = this._getDateFromSerialDate(serialdate);
                    day = date.getDate();
                }
            } else {
                var date = new Date(Date.parse(s));
                if (date.toString() != "Invalid Date") {
                    day = date.getDate();
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._invalid_Math_argument];
                    return this.getErrorStrings()[1].toString();
                }
            }
            return day.toString();
        };
        this.computeDays = function (argList) {
            var argument = argList;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            if (args.length != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getFormulaErrorStrings()[this._wrong_number_arguments];
                return this.getFormulaErrorStrings()[this._wrong_number_arguments];
            }

            var startDateValue = this.getValueFromArg(args[1]);
            var endDateValue = this.getValueFromArg(args[0]);

            var startDateTime, endDateTime;

            var startDateSerial = this._parseDouble(startDateValue.split(this.tic).join(""));
            var endDateSerial = this._parseDouble(endDateValue.split(this.tic).join(""));

            if (!isNaN(startDateSerial)) {
                startDateTime = this._getDateFromSerialDate(startDateSerial);
            } else
                startDateTime = new Date(Date.parse(startDateValue));

            if (!isNaN(endDateSerial)) {
                endDateTime = this._getDateFromSerialDate(endDateSerial);
            } else
                endDateTime = new Date(Date.parse(endDateValue));

            var timeDiff = endDateTime.getTime() - startDateTime.getTime();

            var diffDays = Math.ceil(timeDiff / (1000 * 3600 * 24));
            return diffDays.toString();
        };
        this.computeDays360 = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var method = false;
            var days = 0;
            var dt1 = new Date(Date.parse(this.getValueFromArg(args[0]).split(this.tic).join("")));
            var dt2 = new Date(Date.parse(this.getValueFromArg(args[1]).split(this.tic).join("")));
            var serialdate1 = this._parseDouble(this.getValueFromArg(args[0]).split(this.tic).join(""));
            var serialdate2 = this._parseDouble(this.getValueFromArg(args[1]).split(this.tic).join(""));

            if ((!isNaN(serialdate1) || !isNaN(this._isDate(dt1))) && (!isNaN(serialdate2) || !isNaN(this._isDate(dt2)))) {
                dt1 = (serialdate1 > 0) ? this._getDateFromSerialDate(serialdate1) : dt1;
                dt2 = (serialdate2 > 0) ? this._getDateFromSerialDate(serialdate2) : dt2;
                var flipSign = false;
                var mathodStr = this.falseValueStr;
                if (argCount == 3)
                    mathodStr = this.getValueFromArg(args[2]);
                method = (mathodStr == this.trueValueStr) ? true : false;
                if (dt1.getDate() == 31) {
                    dt1 = new Date(dt1.setDate(dt1.getDate() - 1));
                }

                if (dt2.getDate() == 31 && !method && dt1.getDate() < 30) {
                    dt2 = new Date(dt2.setDate(dt2.getDate() + 1));
                } else if (dt2.getDate() == 31) {
                    dt2 = new Date(dt2.setDate(dt2.getDate() - 1));
                }

                if (dt2 < dt1) {
                    flipSign = true;
                    var t = dt1;
                    dt1 = dt2;
                    dt2 = t;
                }

                days = dt2.getDate() - dt1.getDate();
                days += 30 * (dt2.getMonth() - dt1.getMonth());
                days += 360 * (dt2.getFullYear() - dt1.getFullYear());
                if (flipSign) {
                    days = -days;
                }
            } else {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            return days.toString();
        };
        this.computeEDate = function (argList) {
            var argument = argList;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var StartDateValue = parseInt(startDate);
            var startDateTime = (isNaN(startDate)) ? new Date(Date.parse(startDate)) : this._fromOADate(parseInt(startDate));
            var monthCount = this.getValueFromArg(args[1]).split(this.tic).join("");
            var month = parseInt(monthCount);
            if (startDateTime.toString() == "Invalid Date" && isNaN(month)) {
                try {
                    var today = new Date(Date.now());
                    var first = this._parseDouble(startDate);
                    var second = parseInt(monthCount);
                    if (second < 0 && second < first) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._bad_index];
                        return this.getErrorStrings()[4].toString();
                    }
                    if (first != 0) {
                        var dt = this.dateTime1900;
                        dt = this._fromOADate(first);
                        startDateTime = new Date(dt.setMonth(dt.getMonth() + second));
                    } else {
                        startDateTime = new Date(today.setMonth(today.getMonth() + second));
                        var days = this.computeDays(today.toLocaleDateString() + this.getParseArgumentSeparator() + startDateTime.toLocaleDateString());
                        return days;
                    }
                }
                catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula] + ex;
                    return this.getErrorStrings()[1].toString();
                }
            }
            startDateTime = new Date(startDateTime.setMonth(startDateTime.getMonth() + month));
            if (this.getExcelLikeComputations()) {
                return startDateTime.toLocaleDateString();
            } else {
                return parseInt(this._toOADate(startDateTime)).toString();
            }
        };
        this.computeEOMonth = function (argList) {
            var argument = argList;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var startDateTime = new Date(startDate);
            var month = this._parseDouble(this.getValueFromArg(args[1]).split(this.TIC).join(""));
            if(startDateTime.toString() == "Invalid Date")
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            if (isNaN(month)) {
                try  {
                    var today = new Date(Date.now());
                    var first = parseInt(startDate);
                    month = this._parseDouble(this.getValueFromArg(args[1]).split(this.tic).join(""));
                    if (month < 0 && first <= 0 && month < first) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._bad_index];
                        return this.getErrorStrings()[4].toString();
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[1].toString();
                }
            }
            var dateValue = parseInt(startDate);
            if (!isNaN(dateValue) && startDateTime.getFullYear() > 9999) {
                startDateTime = this._fromOADate(dateValue);
            }
            startDateTime = new Date(startDateTime.setMonth(startDateTime.getMonth() + month));
            var totalDays = (new Date(startDateTime.getFullYear(), startDateTime.getMonth() + 1, 1, -1)).getDate();
            startDateTime = new Date(startDateTime.getFullYear(), startDateTime.getMonth(), totalDays);

            if (this.getExcelLikeComputations())
                return startDateTime.toLocaleDateString();
            else
                return this._getSerialDateTimeFromDate(startDateTime).toString();
        };
        this.computeHour = function (argList) {
            var time;
            var dt = new Date(Date.now());
            argList = this.getValueFromArg(argList);
            argList = argList.split(this.tic).join("");
            dt = new Date(Date.parse(argList));
            var hourValue = parseInt(argList);
            if (hourValue < 0)
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (dt.toString() == "Invalid Date") {
                var argVal = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dt = new Date(Date.parse(argVal));
            }
            if (dt.toString() == "Invalid Date") {
                dt = this._fromOADate(argList);
            }
            if (dt.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            return dt.getHours().toString();
        };
        this.computeISOWeeknum = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var value = this.getValueFromArg(args[0]);
            if (value[0] == (this.tic) && this._isTextEmpty(value.split(this.tic).join(""))) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[1].toString();
            }
            if (this._isTextEmpty(value)) {
                argList = this.computeDatevalue(new Date(1990, 12, 28).toLocaleDateString());
            }

            if (argList.indexOf(":") > -1 || !this._isCellReference(this.getValueFromArg(args[0])) && this.getValueFromArg(this.DateFormatter(args[0])).indexOf(":") > -1) {
                argList = "0";
            }
            var weekDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var date = new Date(weekDate);
            var weekDateOnly = this.DateFormatter(weekDate);
    
            var weekDateTime = new Date(Date.parse(weekDateOnly)), firstDate = this.dateTime1900;
            ;
            if (weekDateTime.toString() == "Invalid Date") {
                try  {
                    var weeknumber = parseInt(weekDateOnly);
                    weekDateTime = this.fromOADate(weeknumber);
                    weekDate = weekDateTime.toLocaleDateString();
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw "Bad Cell reference";
                    return this.getErrorStrings()[1].toString();
                }
            }
            var isoarg = argList + this.getParseArgumentSeparator() + "21";
            var weekno = parseInt(this.computeWeeknum(isoarg));
            return weekno.toString();
        };

       this.DateFormatter= function  (date) {
           var d = new Date(date);
            var day = d.getDate();
            var month = d.getMonth() + 1;
            var year = d.getFullYear();
            if (day < 10) {
                day = + day;
            }
            if (month < 10) {
                month =  month;
            }
            var date = month + "/" + day + "/" + year;

            return date;
        }; 

        this.computeMinute = function (argList) {
            var time;
            var dt = new Date(Date.now());
            argList = this.getValueFromArg(argList);
            argList = argList.split(this.tic).join("");
            dt = new Date(Date.parse(argList));
            if (argList < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (dt.toString() == "Invalid Date") {
                var argVal = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dt = new Date(Date.parse(argVal));
            }
            if (dt.toString() == "Invalid Date") {
                dt = this._fromOADate(argList);
            }
            if (dt.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            return dt.getMinutes().toString();
        };
        this.computeMonth = function (argList) {
            var month = 1;
            var s = this.getValueFromArg(argList);
            var date = new Date(Date.parse(s.split(this.tic).join("")));

            var dateValue = parseInt(s);
            if (!isNaN(dateValue) && date.getFullYear() > 9999) {
                date = this._fromOADate(dateValue);
            }
            if (s < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (date.toString() == "Invalid Date") {
                argList = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                date = new Date(Date.parse(argList));
            }
            if (date.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            if (date.toString() != "Invalid Date") {
                month = date.getMonth() + 1;
            } else {
                var serialdate = this._parseDouble(this.getValueFromArg(argList));
                if (!isNaN(serialdate)) {
                    ////serialdate = serialdate - 1 - ((Treat1900AsLeapYear && serialdate > 59) ?  1 : 0);
                    if (serialdate < 1) {
                        month = 1;
                    } else {
                        date = this._getDateFromSerialDate(serialdate); ////dateTime1900.AddDays(serialdate);
                        month = date.getMonth() +1;
                    }
                } else {
                    return this.formulaErrorStrings[this._invalid_arguments];
                }
            }
            return month.toString();
        };
        this.computeNetworkDays = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var adjustRange, s1;
            var date;
            var holidays = [];

            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var endDate = this.getValueFromArg(args[1]).split(this.tic).join("");
            var startDateTime = new Date(Date.parse(startDate));
            var endDateTime = new Date(Date.parse(endDate));
            var dval = this._parseDouble(startDate);
            if (!isNaN(dval) && startDateTime.getFullYear() > 9999) {
                startDateTime = this._fromOADate(dval);
            }
            if (startDateTime.toString() == "Invalid Date") {
                startDateTime = this._fromOADate(dval);
            }
            if (startDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }
            var dval = this._parseDouble(endDate);
            if (!isNaN(dval) && endDateTime.getFullYear() > 9999) {
                endDateTime = this._fromOADate(dval);
            }
            if (endDateTime.toString() == "Invalid Date") {
                endDateTime = this._fromOADate(dval);
            }
            if (endDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }
            if (argCount == 3) {
                adjustRange = args[2];
                ////is a cellrange
                if (adjustRange.indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(adjustRange);
                    for (var s = 0; s < cells.length; s++) {
                        try  {
                            s1 = this.getValueFromArg(s).split(this.tic).join("");
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1 != "") {
                            date = new Date(Date.parse(s1));
                            if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date < endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                                holidays.push(date);
                            }
                        }
                    }
                } else {
                    var dateArray = this.splitArgsPreservingQuotedCommas(adjustRange.split(this.tic).join(""));
                    for (var i = 0; i < dateArray.length; i++) {
                        try {
                            s1 = this.getValueFromArg(dataArray[i]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            date = new Date(Date.parse(s1));
                            if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date < endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                                holidays.push(date);
                            }
                        }
                    }
                }
            }

            var numberOfWorkingDays = 0;
            var noofDays = parseInt(this.computeDatevalue(endDate)) - parseInt(this.computeDatevalue(startDate));
            var startWeekDay = startDateTime.getDay();
            var endWeekDay = endDateTime.getDay();
            var excesStartDays = startWeekDay - 1;
            var excesEndDays = 7 - endWeekDay;
            noofDays = noofDays + excesStartDays + excesEndDays;

            var noofholidays = parseInt(noofDays / 7) * 2;

            numberOfWorkingDays = noofDays - excesStartDays - excesEndDays - noofholidays + 1 - holidays.length;
            if (endWeekDay == 6 || startWeekDay == 0)
                numberOfWorkingDays--;
            return numberOfWorkingDays.toString();
        };
        this.computeNetworkDaysOintl = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var adjustRange, s1;
            var date;
            var holidays = [];

            if (argCount != 2 && argCount  > 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var endDate = this.getValueFromArg(args[1]).split(this.tic).join("");
            var startDateTime = new Date(Date.parse(this._fromOADate(startDate)));
            var endDateTime = new Date(Date.parse(this._fromOADate(endDate)))
            if (startDateTime.toString() == "Invalid Date") {
                var dval = this._parseDouble(startDate);
                if (!isNaN(dval)) {
                    startDateTime = this.fromOADate(dval);
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[1].toString();
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (endDateTime.toString() == "Invalid Date") {
                var dval = this._parseDouble(endDate);
                if (!isNaN(dval)) {
                    endDateTime = this._fromOADate(dval);
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[1].toString();
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (argCount == 4) {
                adjustRange = args[3];
                adjustRange = this.adjustRangeArg(adjustRange);

                ////is a cellrange
                if (adjustRange.indexOf(':') > -1 || adjustRange.indexOf(',') > -1) {
                    var cells = this.getCellsFromArgs(adjustRange);
                    for (var s = 0; s < cells.length; s++) {
                        try  {
                            s1 = this.getValueFromArg(cells[s]).toString().split(this.tic).join("");
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1 != "") {
                            date = new Date(Date.parse(s1));
                            if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date > endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                                holidays.push(date);
                            }
                        }
                    }
                } else {
                    try  {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join("");
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        date = new Date(Date.parse(s1));
                        if (date.toString() != "Invalid Date" && (holidays.indexOf(date) == -1) && date > startDateTime && date < endDateTime && date.getDay() != 0 && date.getDay() != 6) {
                            holidays.push(date);
                        }
                    }
                }
            }

            var weekend = parseInt(args[2]);
            if ((argCount <=2 && !isNaN(weekend))) {
                weekend = 1;
            } else if(!isNaN(args[2])) {
                weekend = parseInt(args[2]);
            }
        else{ 
            
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var weekendDays = this._splitArguments(this.getWeekEndType()[weekend].toString(), ',');
            var noofDays = parseInt(this.computeDatevalue(endDate)) - parseInt(this.computeDatevalue(startDate)) + 1;
            var netWorkDays = noofDays;
            var j = 1;
            var tempStartDate = startDateTime;
            while (j < noofDays) {
                var dayofweek = tempStartDate.getDay();
                if (weekendDays.indexOf(dayofweek.toString()) >= 0) {
                    netWorkDays--;
                } else if (holidays.indexOf(tempStartDate) >= 0) {
                    netWorkDays--;
                }
                tempStartDate = new Date(tempStartDate.setDate(tempStartDate.getDate() + 1));
                j++;
            }
            return netWorkDays.toString();
        };
        this.computeNow = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            return new Date(Date.now()).toString();
        };
        this.computeSecond = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var time;
            var dt = new Date(Date.now());
            argList = this.getValueFromArg(argList);
            argList = argList.split(this.tic).join("");
            dt = new Date(Date.parse(argList));
            if (argList < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }
            if (dt.toString() == "Invalid Date") {
                var argVal = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dt = new Date(Date.parse(argVal));
            }
            if (dt.toString() == "Invalid Date") {
                dt = this._fromOADate(argList);
            }
            if (dt.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            return dt.getSeconds().toString();
        };
        this.computeTime = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var hour = this._parseDouble(this.getValueFromArg(args[0]));
            var minute = this._parseDouble(this.getValueFromArg(args[1]));
            var second = this._parseDouble(this.getValueFromArg(args[2]));
            var time = 0;
            if (!isNaN(hour) && !isNaN(minute) && !isNaN(second)) {
                time = (3600 * hour + 60 * minute + second) / 86400;
                var dateString = (new Date(Date.now())).toLocaleDateString() + " " + time;
                dt = new Date(Date.parse(dateString));
                if (time < 0) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[4].toString();
                }
                return time.toString();
            }
            else {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
        };
        this.computeTimevalue = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]).split(this.tic).join("");

            argValue = argValue.split(this.tic).join("");
            if (argValue.indexOf(":") > -1) {
                var date = new Date();
                var argValue = date.toLocaleDateString() + " " + argValue;
            }
            var dateTime = new Date(Date.parse(argValue));
            if (dateTime.toString() == "Invalid Date") {
                argList = (new Date(Date.now())).toLocaleDateString() + " " + argList;
                dateTime = new Date(Date.parse(argList));
            }
            if (dateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            var time = (3600 * dateTime.getHours() + 60 * dateTime.getMinutes() + dateTime.getSeconds()) / 86400;
            return time.toString();
        };
        this.computeToday = function (argList) {
            var dt = new Date(Date.now());

            if (this.getExcelLikeComputations()) {
                return dt.toLocaleDateString();
            } else
                return this._toOADate(dt).toString();
        };
       
        this.computeWeekday = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var dateString = this.getValueFromArg(args[0]);

            var dt = new Date(Date.parse(dateString.split(this.tic).join("")));
            var dateValue = parseInt(dateString);
            if (!isNaN(dateValue) && dt.getFullYear() > 9999) {
                dt = this._fromOADate(dateValue);
            }
            args[1] = (argCount == 2) ? args[1] : "1";
            if (dt.toString() == "Invalid Date")
                dt = this._fromOADate(dateValue);
            var return_type = parseInt(this.getValueFromArg(args[1]));
            var day = dt.getDay();

            if (return_type == 1) {
                day += 1;
            } else {
                if (day == 0) {
                    day = 7;
                }

                if (return_type == 3) {
                    day -= 1;
                }
            }
            return day.toString();
        };
        this.computeWeeknum = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var weekDate;
            if (this._fromOADate(this.DateFormatter(this.getValueFromArg(args[0]).split(this.tic).join(""))) == "Invalid Date")

                weekDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            else
                weekDate = this.DateFormatter(this.getValueFromArg(args[0]).split(this.tic).join(""));
            var weeknumber = parseInt(weekDate);
            var d = new Date(weekDate);
            var weekDateTime = new Date(Date.parse(weekDate));
           // weekDateTime = this._fromOADate(weekDateTime);

            if (weekDateTime.toString() == "Invalid Date")
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var weekStart = ["", "0", "1", "", "", "", "", "", "", "", "", "1", "2", "3", "4", "5", "6", "0", "", "", "", "1"];

            var weektype ;
            if (argCount != 2) {
                weektype = 1;
            }
            else
                weektype = this.getValueFromArg(args[1]);
            
            if (!isNaN(weektype)) {
            }
            var weehStartValue = parseInt(weekStart[weektype]);
            var firstDate = new Date(weekDateTime.getFullYear(), 0, 1);
            var firstDay = firstDate.getDate();
            var offset = (firstDay < weehStartValue) ? weehStartValue - firstDay + 1 : firstDay - weehStartValue;
            var dt = this._fromOADate(this._toOADate(firstDate) - offset);
            var noofDays = this._toOADate(weekDateTime) - this._toOADate(dt);
            var dval = Math.floor(noofDays / 7) +1 + weehStartValue;
            return dval.toString();
        };
        this.computeWorkDay = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 2 && argCount > 3) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argString = "";
            var dateValue = 0;
            if (argCount == 3) {
                argString = args[0] + this.getParseArgumentSeparator() + args[1] + this.getParseArgumentSeparator() + "1" + this.getParseArgumentSeparator() + args[2];
            } else
                argString = args[0] + this.getParseArgumentSeparator() + args[1] + this.getParseArgumentSeparator() + "1";
            var value = this.computeWorkDayOintl(argString);

            var dateTime = new Date(Date.parse(this._fromOADate(value).toString()));

            if (isNaN(Date.parse(value)) && parseInt(value)) {
                dateTime = this._fromOADate(value);
            }

            if (dateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            }

            return this._toOADate(dateTime).toString();
        };
        this.computeWorkDayOintl = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount > 4 || argCount < 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var adjustRange, s1;
            var date;
            var holidays = [];

            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var startDateTime = new Date(Date.parse(startDate));

            var temp = this.getValueFromArg(args[1]).split(this.tic).join("");
            var dateval;

            if (!isNaN(Date.parse(startDate)) && parseInt(startDate)) {
                startDateTime = this._fromOADate(startDate);
            }

            if (isNaN(temp)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            } else {
                dateval = this._toOADate(startDateTime) + temp;
            }

            if (startDate.split(this.tic).join("") == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[0].toString();
            } else if (startDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            }
            var totalDays = parseInt(temp.toString());
            if (argCount == 4) {
                adjustRange = args[3];
                adjustRange = this.adjustRangeArg(adjustRange);

                ////is a cellrange
                if (adjustRange.indexOf(':') > -1 || adjustRange.indexOf(',') > -1) {
                    var cells = this.getCellsFromArgs(adjustRange);
                    for (var s = 0; s < cells.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cells[s]).split(this.tic).join("");
                        } catch (ex) {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.formulaErrorStrings[this.bad_formula];
                            return this.getErrorStrings()[4].toString();
                        }

                        if (s1 != "") {
                            var dt = new Date(Date.parse(s1));
                            if (dt.toString() != "Invalid Date" && holidays.indexOf(dt) == -1 && dt > startDateTime) {
                                holidays.push(dt.toString());
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join("");
                    } catch (Exception) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this.bad_formula];
                        return this.getErrorStrings()[4].toString();
                    }

                    if (s1 != "") {
                        var dt = new Date(Date.parse(s1));

                        if (isNaN(Date.parse(s1)) && parseInt(s1)) {
                            dt = this._fromOADate(s1);
                        }

                        if (dt.toString() != "Invalid Date" && holidays.indexOf(dt) == -1 && dt > startDateTime) {
                            holidays.push(dt);
                        }
                    }
                }
            }
            
            var weekend = parseInt(args[2]);
            if (!(argCount > 2 && !isNaN(weekend))) {
                weekend = 1;
            }
            if (weekend == 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var weekendDays = this._splitArguments(_weekEndType[weekend].toString(), ",");
            var j = 0;
            var tempStartDate = startDateTime;
            if (totalDays < 0) {
                while (j > totalDays) {
                    tempStartDate = new Date(tempStartDate.setDate(tempStartDate.getDate() - 1));
                    var dayofweek = tempStartDate.getDay();
                    if (weekendDays.indexOf(dayofweek.toString()) < 0 && holidays.indexOf(tempStartDate) < 0) {
                        j--;
                    }
                }
            } else {
                while (j < totalDays) {
                    tempStartDate = new Date(tempStartDate.setDate(tempStartDate.getDate() + 1));
                    var dayofweek = tempStartDate.getDay();
                    if (weekendDays.indexOf(dayofweek.toString()) < 0 && holidays.indexOf(tempStartDate.toString()) < 0) {
                        j++;
                    }
                }
            }
            return this._toOADate(tempStartDate).toString();
        };
        this.computeYear = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var startDate = this.getValueFromArg(args[0]).split(this.tic).join("");
            var startDateTime = new Date(Date.parse(startDate));
            var dateValue = parseInt(startDate);
            if (!isNaN(dateValue) && startDateTime.getFullYear() > 9999) {
                startDateTime = this._fromOADate(dateValue);
            }
            if (dateValue == 0) {
                return "1900";
            }
            if (dateValue < 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            if (startDate.split(this.tic).join("") == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[0].toString();
            } else if (startDateTime.toString() == "Invalid Date") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this.bad_formula];
                return this.getErrorStrings()[1].toString();
            }

            return startDateTime.getFullYear().toString();
        };


        this.computeChar = function (arg) {
            var ranges = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = ranges.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s = this.getValueFromArg(ranges[0]).split(this.tic).join("");
            var i = this._parseDouble(s);
            if (!isNaN(i) && i > 0 && i < 256) {
                return String.fromCharCode(s);
            }
            if (this.getErrorStrings().indexOf(s) > -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return s;
            }
            return (s[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
        };
        this.computeCode = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]).split(this.tic).join("");
            var num =  parseInt(text);
            if (!isNaN(num))
            {
                return text.charCodeAt(0).toString();
            }
            if (text == null || text == "") {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return (args[0].length > 4) ? this.getErrorStrings()[5].toString() : this.getErrorStrings()[1].toString();
            }
            if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var code = text;

            return code.charCodeAt(0).toString();
        };
        this.computeUniCode = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            if (this._isCellReference(args[0])) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var str = this._parseDouble(text);
            if (text == "invalid expression" || text == null || text == "" || !isNaN(str)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            var unicodeString = text.split(this.tic).join("").charCodeAt(0);
            return unicodeString.toString();
        };
        this.computeUpper = function (args) {

            if (!this._isCellReference(args) && args[0].indexOf(this.tic) == -1)
            {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            return this._stripTics0(this.getValueFromArg(args)).toUpperCase();
        };
        this.computeLower = function (args) {

            if (!this._isCellReference(args) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            return this._stripTics0(this.getValueFromArg(args)).toLowerCase();
        };
        this.computeLen = function (range) {

            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }            
            var s1 = this._stripTics0(this.getValueFromArg(args[0]));

            if(isNaN(parseInt(s1))){
                var dt = new Date(Date.parse(s1));
                if (dt != "Invalid Date") {
                    s1 = this.DateFormatter(s1);
                }
            }
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            return (hasTics ? s1.length - 2 : s1.length).toString();
        };
        this.computeMid = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s1 = args[0];
            s1 = this.getValueFromArg(s1);
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic)
            {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            var s2 = this.getValueFromArg(args[1]);
            var len = this._parseDouble(args[2]);
            s2 = this.getValueFromArg(args[1]);
            var start = Number(s2) + Number(hasTics ? 0 : -1);

            if (args[1].indexOf("#VALUE!") > -1) {
                return "#VALUE!";
            }
            if (Number(start) > s1.length) {
                return "";
            }
            if (start + len > s1.length) {
                s1 = s1.substring(start);
            }
            else
                s1 = this._substring(s1, start, len);
            if (hasTics && s1[0] != this.tic) {
                s1 = this.tic + s1;
            }

            if (hasTics && s1) {
                s1 = s1 + this.tic;
            }

            return this._stripTics0(s1);
        };
        this.computeLeft = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var s1 = this._stripTics0(this.getValueFromArg(args[0]));
            if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            var s2 = (argCount == 2) ? args[1] : "1";
            s2 = this.computedValue(s2);
            var len = Number(s2) + Number(hasTics ? 1 : 0);
            len = (s1.length >= len) ? len : s1.length;
            len = (s1.length >= len) ? len : s1.length;
            if (len < 1) {
                return this.getErrorStrings()[1].toString();
            }
            s1 = s1.substring(0, len);
            if (hasTics && s1[s1.length - 1] != this.tic) {
                s1 = s1 + this.tic;
            }
            if (this.getUseNoAmpersandQuotes() && s1.length > 1 && s1[0] == this.tic[0] && s1[s1.length - 1] == this.tic[0]) {
                s1 = this._substring(s1,1, s1.length - 2);
            }
            return s1;
        };
        this.computeRight = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }           
            s1 = this._stripTics0(this.getValueFromArg(args[0]));
            if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var hasTics = s1[0] == this.tic && s1[s1.length - 1] == this.tic;
            var s2 = (argCount == 2) ? this.getValueFromArg(args[1]) : "1";
            var len = parseInt(s2);
            if (isNaN(len) || len <0 || s2.indexOf("#VALUE!") > -1) {
                return "#VALUE!";
            }
            if (len == 0)
                return "";
            var len = Number(s2) + Number(hasTics ? 1 : 0);
            var start = (s1.length >= len) ? s1.length - len : 0;
            s1 = s1.substring(start);
            if (hasTics && s1[0] != this.tic) {
                s1 = this.tic + s1;
            }

            return s1;
        };
        this.computeReplace = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstText = this._stripTics0(this.getValueFromArg(args[0]));
            var newText = this._stripTics0(this.getValueFromArg(args[3]));
            var resultText = "";
            var startIndex = this._parseDouble(this.getValueFromArg(args[1]));
            var noOfChar = this._parseDouble(this.getValueFromArg(args[2]));
            if ((!this._isCellReference(args[0]) && args[0][0] != this.tic) || (!this._isCellReference(args[3]) && args[3][0] != this.tic)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if (startIndex == 0 && noOfChar == 0)
            {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[1].toString();
            }
            if (!isNaN(startIndex) && !isNaN(noOfChar)) {
                try {
                    var replaceText = "";
                    if (firstText == null || firstText == "") {
                        return newText;
                    }
                    if (firstText.length <= startIndex && startIndex > noOfChar) {
                        resultText = firstText + newText;
                    } else if (firstText.length <= startIndex) {
                        replaceText = firstText;
                        resultText = firstText.split(replaceText).join(newText);
                    } else if (noOfChar == 0) {
                        resultText = newText + firstText;
                    }else {
                        replaceText = this._substring(firstText, startIndex - 1, noOfChar);
                        resultText = firstText.split(replaceText).join(newText);
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[1].toString();
                }
            } else {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[1].toString();
            }

            return resultText;
        };
        this.computeExact = function (range) {
            //this.AdjustRangeArg(ref range);
            var args = this.splitArgsPreservingQuotedCommas(range);
            if (args.length != 2) {
                return this.formulaErrorStrings[this._requires_2_args];
            }

            var s1, s2, d;

            s1 = this.getValueFromArg(args[0]);
            s2 = this.getValueFromArg(args[1]);
            var str1 = this._parseDouble(s1);
            var str2 = this._parseDouble(s2);

            if ((!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) || (!this._isCellReference(args[1]) && args[1].indexOf(this.tic) == -1)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if (!isNaN(str1)) {
                s1 = str1.toString();
            }

            if (!isNaN(str2)) {
                s2 = str2.toString();
            }

            return (s1.split(this.tic).join("") == s2.split(this.tic).join("")) ? "TRUE" : "FALSE";
        };
        this.computeFind = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 2 && args.length != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if ((!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1) || (!this._isCellReference(args[1]) && args[1].indexOf(this.tic) == -1) || (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            ////strip off outside tics (if any) on the evaluated arguments
            var lookFor = this._stripTics0(this.getValueFromArg(args[0]));
            var lookIn = this._stripTics0(this.getValueFromArg(args[1]));
            var start = 1;

            if (args.length == 3) {
                var s3 = this.getValueFromArg(args[2]);
                start = this._parseDouble(s3);
                if (isNaN(start)) {
                    start = 1;
                }
            }

            if (start <= 0 || start > lookIn.length) {
                return "#VALUE!";
            }

            var loc = lookIn.indexOf(lookFor, start - 1);
            if (loc < 0) {
                return "#VALUE!";
            }

            return (Number(loc) + Number(1)).toString();
        };
        this.computeSearch = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var index;
            var startNum = 1;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstStr = this._stripTics0(this.getValueFromArg(args[0])).toUpperCase();
            var secondStr = this._stripTics0(this.getValueFromArg(args[1])).toUpperCase();
            if ((!this._isCellReference(args[0]) && args[0][0] != this.tic) || (!this._isCellReference(args[1]) && args[1][0] != this.tic)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }

            var qusfirstStr = firstStr;
            if (firstStr.indexOf("*") >-1)
            {
                firstStr = firstStr.split("*").join("")[0].toString();
            }
            else if (firstStr.indexOf("?") >-1)
            {
                var subStr = qusfirstStr.substring(0, qusfirstStr.indexOf("?"));
                var strIndex = Number(secondStr.indexOf(subStr));
                while (secondStr.indexOf(qusfirstStr) == -1)
                {
                    strIndex = Number(secondStr.substring(strIndex).indexOf(subStr)) + Number(strIndex) + 1;
                    qusfirstStr = firstStr;
                    while (qusfirstStr.indexOf("?") != -1)
                    {
                        if (secondStr.length <= strIndex)
                            return this.getErrorStrings()[1].toString();
                        qusfirstStr = qusfirstStr.substring(0, qusfirstStr.indexOf("?")) + secondStr[strIndex] + firstStr.substring(qusfirstStr.indexOf("?") + 1);
                        strIndex++;
                    }
                }
                firstStr = qusfirstStr;
            }
            index = Number(secondStr.indexOf(firstStr)) + 1;
            if (argCount == 3) {

                var startNum = parseInt(this.getValueFromArg(args[2]));
                if (startNum == 1 && firstStr[0] == secondStr[0])
                {
                    return startNum;
                }
                if (!isNaN(startNum)) {
                    try {
                        index = Number(secondStr.indexOf(firstStr, startNum)) + 1;
                    } catch (ex) {
                        return this.getErrorStrings()[1].toString();
                    }
                }
            }
            if (index == 0) {
                return this.getErrorStrings()[1].toString();
            }

            return index.toString();
        };
        this.computeSubstitute = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            if (args.length != 3 && args.length != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            ////strip off outside tics (if any) on the evaluated arguments
            var s1 = this._stripTics0(this.getValueFromArg(args[0]));
            var s2 = this._stripTics0(this.getValueFromArg(args[1]));
            var s3 = this._stripTics0(this.getValueFromArg(args[2]));
            if ((!this._isCellReference(args[0]) && args[0][0] != this.tic) || (!this._isCellReference(args[1]) && args[1][0] != this.tic)
                || (!this._isCellReference(args[2]) && args[2][0] != this.tic)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if (args.length == 3) {
                if(s2=="" || s2==null)
                {
                    return s1;
                }
                s1 = s1.split(s2).join(s3);
            } else {
                var s4 = this.getValueFromArg(args[3]);
                if (!this._isCellReference(args[3]) && args[3][0] == this.tic) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[5].toString();
                }
                var d = parseInt(s4)

                if (!isNaN(d)) {
                    var count = d;
                    var loc = -1;
                    while (count > 0 && (loc = s1.indexOf(s2, Number(loc) + 1)) > -1) {
                        count--;
                    }
                    if (count == 0) {
                        s1 = s1.substring(0, loc) + s3 + s1.substring(Number(loc) + Number(s2.length));
                    }
                }
            }
            return  s1;
        };
        this.computeUniChar = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            var code = this._parseDouble(this._stripTics0(text));
            if (isNaN(code) || code <=0) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return (text[0] == this.tic) ? this.getErrorStrings()[5].toString() : this.getErrorStrings()[1].toString();
            }
            var unicharString = String.fromCharCode(text);
            return unicharString.toString();
        };
        this.computeClean = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var result;
            if (args.length != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments].toString();
            }
            var cleanedArg = this.getValueFromArg(args[0]);
            var isLogic = true;
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var boolFlag = (cleanedArg == "TRUE" || cleanedArg == "FALSE") ? true : false;
            if (this.getErrorStrings().indexOf(cleanedArg) > -1)
            {
                return cleanedArg;
            }
            if (cleanedArg.indexOf(this.tic) == -1 && !this._isCellReference(arg) && isNaN(this._parseDouble(cleanedArg)) && !boolFlag) {
                return this.getErrorStrings()[5].toString();
            }
            for (var i = 0; i <= 31; i++) {
                var nonPrintChar = this.computeChar(i.toString());
                if (cleanedArg.indexOf(nonPrintChar) > -1)
                    cleanedArg = cleanedArg.replace(nonPrintChar, "");
            }
            if (cleanedArg[0] == this.tic && cleanedArg[cleanedArg.length - 1] == this.tic) {
                cleanedArg = this._substring(cleanedArg, 1, cleanedArg.length - 1);
            }
            return cleanedArg;
        };
        this.computeTrim = function (args) {
            var s = this._stripTics0(this.getValueFromArg(args));
            var arr = [this.tic, ''];
            s = s.split(this.tic).join("");
            var len = 0;
            if (!this._isCellReference(args) && args[0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            while (s.length != len) {
                len = s.length;
                s = s.split("  ").join(" ");
            }

            return s;
        };
        this.computeRept = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this._stripTics0(this.getValueFromArg(args[0]));
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var repeatTime = 1;
            var value = parseInt(this.getValueFromArg(args[1].split(this.tic).join("")));
            if (value == null || value == "") {
                return "";
            }
            if (value < 0 || isNaN(value)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            var reptString = "";
            for (var i = 0; i < value ; i++) {
                if (reptString.length > 32767) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[1].toString();
                }
                reptString = reptString.concat(text);
            }

            return reptString;
        };
        this.computeProper = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var textValue = this.getValueFromArg(args[0]);
            if (!this._isCellReference(args[0]) && args[0][0] != this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            var text = this._stripTics0(textValue);

            var quoteIndex = text.indexOf("'");

            if (quoteIndex >= 0 && quoteIndex < text.length - 1) {
                text = text.replace(/[']/, " '");
            }
            if (/[^a - zA - Z]/.test(text)) {

                text = text.replace(/\w\S*/g, function (text) {
                    var i = 0;
                    var charCode = (text[i].toString().toUpperCase()).charCodeAt(0);
                    while (!(charCode > 64 && charCode < 91))
                    {
                        i++;
                        if (text[i] != undefined)
                            charCode = (text[i].toUpperCase()).charCodeAt(0);
                        else
                        {
                            i--;
                            charCode = 0;
                        }
                    }
                    return text.substr(0, i) + text.charAt(i).toUpperCase() + text.substr(i + 1).toLowerCase();
                });
            }
            else if (/[a - zA - Z0 - 9] + $/.test(text)) {
                text = text.replace(/[^a-zA-Z0-9_\\]/, "");
            }

            return text;

            if (text == null || text == "") {
                return "";
            }
            text = text[0].toUpperCase() + text.slice(1).split(" '").join("'");
            return text.toString();
        };
        this.computeT = function (args) {
            var elements = 0;
            var arg = this.splitArgsPreservingQuotedCommas(args);
            if (arg.length > 1) {
                for (var r = 0; r < arg.length; r++) {
                    elements++;
                    if (r.indexOf(':') > -1) {
                        --elements;
                        var cells1 = this.getCellsFromArgs(r);
                        for (var s1 = 0; s1 < cells1.length; s1++) {
                            elements++;
                        }
                    }
                }
            }
            if (elements > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s = this.getValueFromArg(args);
            var d;
            var pattern = /[a-zA-Z0-9!#$%&'()*+,/:;<=>?@\^_`{|}~-]/;
            var flag = (s == "TRUE" || s == "FALSE") ? true : false;
            if (!isNaN(this._parseDouble(s)) || flag || !isNaN(this._isDate(s))) {
                return " ";
            } else if (/[a-zA-Z0-9!#$%&'()*+,/:;<=>?@\^_`{|}~-]/.test(s)) {
                s = this._stripTics0(s);
                return s;
            } else {
                return " ";
            }
        };

        this.computeNumberValue = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var decimalseparator = this._parseDecimalSeparator;
            var groupseparator = this._parseArgumentSeparator;
            if (argCount > 3) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this._stripTics0(this.getValueFromArg(args[0]));
            if (argCount > 1) {
                try {
                    var deci = (args.length > 1 && args[2] != null) ? this._stripTics0(this.getValueFromArg(args[1])) : groupseparator;
                    var gro = (args.length > 1 && args[2]!=null) ? this._stripTics0(this.getValueFromArg(args[2])) : decimalseparator;
                    if ((deci == "invalid expression" || gro == "invalid expression") || ((deci == null || deci == "") && (gro == null || gro == ""))) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[1].toString();
                    } else {
                        decimalseparator = this._stripTics0(deci)[0];
                        groupseparator = this._stripTics0(gro)[0];
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (text == "" || text == null) {
                return "0";
            }
            var indexDecimal = text.indexOf(decimalseparator);
            var indexGroup = text.indexOf(groupseparator);
            if ((indexGroup >= indexDecimal) && (indexDecimal != -1 || indexGroup != -1) && args.length>2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            text = text.replace(this.tic, "");
            text = text.replace(" ", "");
            text = text.replace(groupseparator.toString(), "");
            text = text.replace(decimalseparator, this._parseDecimalSeparator);
            var compText = this.computeValue(text);
            if (isNaN(compText) || (text[text.length - 1] == "$")) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            return compText;
        };
        this.computeConcatenate = function (range) {
            var text = "";
            var ar = this.splitArgsPreservingQuotedCommas(range);
            if ((!this._isCellReference(ar[0]) && ar[0].indexOf(this.tic) == -1) || (!this._isCellReference(ar[1]) && ar[1].indexOf(this.tic) == -1)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            for (var r = 0; r < ar.length; r++) {
                var toAppend = ar[r];
                text = text.concat(this._stripTics0(this.getValueFromArg(ar[r])));
            }
            if (text.indexOf("#N/A") > -1) {
                text = "#N/A";
            }
            return text;
        };
        this.computeValue = function (range) {
            var val = "";
            try {
                var args = this.splitArgsPreservingQuotedCommas(range);
                var argCount = args.length;
                if (argCount == 1) {
                    var s1 = args[0];
                    var ticContains = false;
                    var quotation = false;
                    if (s1.indexOf(this.tic) > -1)
                        ticContains = true;
                    s1 = this._stripTics0(this.getValueFromArg(s1));
                    if ((s1 != null || s1 != "") && !this._isCellReference(s1) && ticContains && isNaN(this._parseDouble(s1.split(",").join("").split("$").join("")))) {
                        s1 = s1.split(this.tic).join("");
                        quotation = true;
                    }
                    if (!this._isCellReference(args[0]) && args[0].indexOf(this.tic) == -1 && isNaN(this._parseDouble(s1))) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[5].toString();
                    }
                    if ((s1 == null || s1 == "") && ticContains) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[1].toString();
                    } else if ((s1 == null || s1 == "") && !ticContains) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.formulaErrorStrings[this._invalid_arguments];
                    }
                    if (s1.indexOf(":") > -1)
                    {
                        val = this.computeTimevalue(s1);
                        return val.toString();
                    }
                    if (s1[0] == "$") {
                        s1 = s1.split("$").join("");
                    }
                    if (s1.indexOf(",") > -1) {
                        s1 = s1.split(",").join("");
                    }
                    if (s1[0] == "%" || s1[s1.length - 1] == "%") {
                        s1 = s1.split("%").join("");
                        var per = this._parseDouble(s1);
                        if (!isNaN(per)) {
                            s1 = (per / 100).toString();
                        }
                    }
                    var dt = new Date(Date.parse(s1));
                    var dous = this._parseDouble(s1);
                    if (!isNaN(dous)) {
                        val = dous.toString();
                    } else if (dt.toString() != "Invalid Date") {
                        val = this._toOADate(dt);
                    } else {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        return this.getErrorStrings()[1].toString();
                    }
                } else {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.formulaErrorStrings[this._wrong_number_arguments];
                }
            } catch (ex) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }

            return val.toString();
        };
        this.computeDollar = function (args) {
            var argsArray = this.splitArgsPreservingQuotedCommas(args);
            var s1 = argsArray[0];
            var s2 = "2";
            if (argsArray.length == 2) {
                s2 = argsArray[1];
            }
            if (argsArray.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }           
            s1 = this.getValueFromArg(s1);
            s2 = this.getValueFromArg(s2);
            var number = this._parseDouble(s1);
            var emptyCell = false;
            if ( isNaN(this._parseDouble(s2))) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[5].toString();
            }
            if ((s1 == null || s1 == "") && this._isCellReference(argsArray[0]) && (s1[0] != this.tic && s1[s1.length - 1] != this.tic))
            {
                number=0;
                emptyCell=true;
            }            
            if (isNaN(number) && !emptyCell) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            var decimals = this._parseDouble(s2);
            var round = 0.0;
            if (isNaN(decimals)) {
                decimals = (s2 == "") ? 0 : 2;
            }
            if (decimals > 0) {
                return Globalize.format(number, "c" + decimals);
            }
            else {
                var mult = Math.pow(10, -decimals);
                round = Math.round(number / mult) * mult;
                return Globalize.format(round, "c0");
            }
        };
        this.computeFixed = function (args) {
            var argsArray = this.splitArgsPreservingQuotedCommas(args);
            var s1 = argsArray[0];
            var s2 = "2";
            var s3 = "FALSE";

            var argCount = argsArray.length;
            if (argCount > 1) {
                s2 = argsArray[1];
            }

            if (argCount > 2) {
                s3 = argsArray[2];
            }

            s1 = this.getValueFromArg(s1);
            s2 = this.getValueFromArg(s2);
            s3 = this.getValueFromArg(s3);

            var numbers, decimals;
            numbers = this._parseDouble(s1);
            if (isNaN(numbers)) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
            decimals = this._parseDouble(s2);
            if (isNaN(decimals)) {
                decimals = (s2 == "") ? 0 : 2;
            }
            var no_commas_flag;
            var no_commas;
            no_commas_flag = this._parseDouble(s3);
            if (!isNaN(no_commas_flag)) {
                if (no_commas_flag == 0) {
                    no_commas = false;
                } else {
                    no_commas = true;
                }
            } else {
                if (s3.toUpperCase() == this.falseValueStr) {
                    no_commas = false;
                } else {
                    if (s3.toUpperCase() == this.trueValueStr) {
                        no_commas = true;
                    } else {
                        return "#NAME?";
                    }
                }
            }
            if (no_commas)
            {
                Globalize.culture().numberFormat[","] = "";
            }
            if (decimals > 0) {
                return Globalize.format(numbers, "n" + decimals)
            } else {
                var mult = Math.pow(10, -decimals);
                numbers = Math.round(numbers / mult) * mult;
                return Globalize.format(numbers, "n0");
            }
        };
        this.computeBin2Dec = function (argList) {
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            if (ranges.length > 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            if (!/^[01]{1,10}$/.test(value))
                return this.getErrorStrings()[4].toString();
            try {
                if (value == "")
                    value = "0";

                // Convert binary number to decimal
                var result = parseInt(value, 2);

                // Handle negative numbers
                if (value.length == 10 && value.substring(0, 1) == "1") {
                    return (parseInt(value.substring(1), 2) - 512).toString();
                } else {
                    return result.toString();
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.getErrorStrings()[1].toString();
            }
        };
        this.computeBin2Oct = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var number = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var result = "";
            // Return error if number is not binary or contains more than 10 characters (10 digits)
            if (!/^[01]{1,10}$/.test(number))
                return this.getErrorStrings()[4].toString();

            if (ranges.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.formulaErrorStrings[this._wrong_number_arguments];
            } else if (ranges.length > 1) {
                places = parseInt(this.getValueFromArg(ranges[1]).replace(this.tic, ""));
            }

            // Ignore places and return a 10-character octal number if number is negative
            var value = ranges.toString();
            if (value.length == 10 && value.substring(0, 1) == "1") {
                var i = 10;
                var s = "";
                for (var j = 0; j < 3; j++) {
                    i = i - 3;
                    var rmv = this._substring(value, i ,3)
                    var valResult = this._parseDouble(parseInt(rmv, 2).toString(8));
                    var sum = 0;
                    for (var n = valResult; n > 0; sum += parseInt(n % 10), n = parseInt(n / 10));
                        s = sum.toString() + s;
                }
                return "7777777" + s;
            }

            //try {
            //    if (value.length > 10) {
            //        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
            //            throw this.getLibraryComputationException();
            //        else
            //            return this.getErrorStrings()[4].toString();
            //    } else {
            //        result = parseInt(value, 2).toString(8);
            //        if (ranges.length > 1) {
            //            if (charCount >= result.length && charCount <= 10) {
            //                result = this._padLeft('0', charCount, result);
            //            } else {
            //                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
            //                    throw this.getLibraryComputationException();
            //                else
            //                    result = this.getErrorStrings()[4].toString();
            //            }
            //        }
            //    }
            //} catch (Exception) {
            //    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
            //        throw this.getLibraryComputationException();
            //            result = this.getErrorStrings()[5].toString();
            //    }

            // Ignore places and return a 10-character octal number if number is negative
            if (number < 0) {
                return (1073741824 + number).toString(8);
            }

            // Convert decimal number to octal
            var result = parseInt(number, 10).toString(8);

            // Return octal number using the minimum number of characters necessary if places is undefined
            if (typeof places === 'undefined') {
                return result;
            } else {
                // Return error if places is nonnumeric
                if (isNaN(places)) return '#VALUE!';

                // Return error if places is negative
                if (places < 0) return '#NUM!';

                // Truncate places in case it is not an integer
                places = Math.floor(places);

                // Pad return value with leading 0s (zeros) if necessary (using Underscore.string)
                if (places >= result.length && places <= 10) {
                    while (places- result.length > 0) {
                        result = "0" + result;
                        places--;
                    }
                }
                else
                    if (this.getRethrowLibraryComputationExceptions())
                        throw new this.getErrorStrings()[4].toString();
                    
                return  result;
            }
            //return result;
        };
        this.computeBin2Hex = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var result = "";

            if (ranges.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.formulaErrorStrings[this._wrong_number_arguments];
            } else if (ranges.length > 1) {
                charCount = this._parseDouble(ranges[1]);
            }

            if (value == "1111111111") {
                return value.replace('1', 'F');
            }

            // Ignore places and return a 10-character octal number if number is negative
            if (value.length == 10 && value.substring(0, 1) == "1") {
                return (1073741312 + parseInt(value, 2).toString(8)).toString();
            }

            try {
                if (value.length > 10 || this._parseDouble(value) < 0 || isNaN(value)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                } else {
                    result = parseInt(value, 2).toString(16);
                    if (ranges.length > 1) {
                        if (charCount >= result.length && charCount <= 10) {
                            result = this._padLeft('0', charCount, result);
                        } else {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            else
                                result = this.getErrorStrings()[4].toString();
                        }
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else {
                    if (this.computeIsText(value) == this.trueValueStr)
                        result = this.getErrorStrings()[5].toString();
                    else if (value == "")
                        return "0";
                    else
                        return this.getErrorStrings()[4].toString();
                }
            }

            return result.toUpperCase();
        };
        this.computeDec2Bin = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var range1 = (ranges.length > 1) ? this.getValueFromArg(ranges[1]).replace(this.tic, "") : "10";
            var result = "";
            if (argList == null || argList == "" || ranges.length > 2 || ranges.length < 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            // double d1, d2;
            if (isNaN(this._parseDouble(value)) && isNaN(this._parseDouble(range1))) {
                if (((value != null || value != "") && (this.computeIsText(value) == this.trueValueStr)) || ((range1 != null || range1 != "") && (this.computeIsText(range1) == this.trueValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
                if (((value != null && value != "") && (this.computeIsText(value) == this.falseValueStr)) || ((value != null && value != "") && (this.computeIsText(range1) == this.falseValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
                if ((value == null && value == "") || (range1 == null || range1 == "")) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
            }
            try {
                if (ranges.length > 1) {
                    var value1 = this.getValueFromArg(ranges[1]).replace(this.tic, "");
                    charCount = this._parseDouble(value1);
                }

                if ((this._parseDouble(value) < -512 || this._parseDouble(value) > 511)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                } else {
                    result = (value>>>0).toString(2); //BaseToBase(value, 10, 2);
                    if (!isNaN(this._parseDouble(value)) && this._parseDouble(value) < 0) {
                        result = result.substring(result.length - 10);
                    }
                    if (ranges.length > 1) {
                        if (charCount > 10) {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            return this.getErrorStrings()[4].toString();
                        }
                        if (charCount <= 10)
                            result = this._padLeft('0', charCount, result);
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
            }
            return result;
        };
       
        this.computeDec2Oct = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            var value = this.getValueFromArg(ranges[0]).replace(this.tic, "");
            var range1 = (ranges.length > 1) ? this.getValueFromArg(ranges[1]).replace(this.tic, "") : "10";
            var result = "";
            if (argList == null || argList == "" || ranges.length > 2 || ranges.length < 1) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (isNaN(this._parseDouble(value)) && isNaN(this._parseDouble(range1))) {
                if (((value != null || value != "") && (this.computeIsText(value) == this.trueValueStr)) || ((range1 != null || range1 != "") && (this.computeIsText(range1) == this.trueValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }

                if (this.getErrorStrings().indexOf(value) > -1)
                    return value;
                if (this.getErrorStrings().indexOf(range1) > -1)
                    return range1;

                if (((value != null && value != "") && (this.computeIsText(value) == this.falseValueStr)) || ((value != null && value != "") && (this.computeIsText(range1) == this.falseValueStr))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
                if ((value == null && value == "") || (range1 == null || range1 == "")) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    return this.getErrorStrings()[1].toString();
                }
            }
            try {
                if (ranges.length > 1) {
                    var value1 = this.getValueFromArg(ranges[1]).replace(this.tic, "");
                    charCount = this._parseDouble(value1);
                }

                if ((this._parseDouble(value) < -536870912 || this._parseDouble(value) > 536870911)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                } else if (this._parseDouble(value) < 0) {
                    result = parseInt((value) >>> 0, 10).toString(8);
                    if (!isNaN(this._parseDouble(value)) && this._parseDouble(value) < 0) {
                        result = result.substring(result.length - 10);
                    }
                }
                else {
                    result = parseInt(value, 10).toString(8);
                    if (!isNaN(this._parseDouble(value)) && this._parseDouble(value) < 0) {
                        result = result.substring(result.length - 10);
                    }
                    if (ranges.length > 1) {
                        if (charCount > 10) {
                            if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            return this.getErrorStrings()[4].toString();
                        }
                        if (charCount <= 10)
                            result = this._padLeft('0', charCount, result);
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
            }
            return result;
        };
        this.computeHex2Bin = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            if (argList == this._parseArgumentSeparator.toString()) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[0].toString();
            }
            var value = this.getValueFromArg(ranges[0]);
            var chars = (ranges.length == 2) ? this.getValueFromArg(ranges[1]) : "0";

            if (value != "") {
                if (value[0] != this.tic && !this._isCellReference(value) && isNaN(this._parseDouble(value))) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[5].toString();
                }
            } else if (this._isCellReference(ranges[0]) && value == "") {
                return "0";
            }
            if ((value[0] == this.tic) && (value.split(this.tic).join("") == null || value.split(this.tic).join("") == "")) {
                return "0";
            }
            if (chars[0] == this.tic) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[1].toString();
            } else if (isNaN(this._parseDouble(chars.split(this.tic).join("")))) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[5].toString();
            }
            charCount = this._parseDouble(chars);
            if (charCount < 0 || value[0] == "-") {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }
            var result = "";
            value = value.split(this.tic).join("");
            try {
                for (var i = 0; i < value.length; i++) {
                    // Convert to decimal
                    var output = this._padLeft('0', 4, parseInt(value.charAt(i), 16).toString(2));
                    result += output;
                }
                if (value.split(this.tic).join("") == "FFFFFFFFFF") {
                    return this._parseDouble(value.split('F').join('1'));
                }

                if (result.length > 10 || charCount > 10 || (charCount < result.length && charCount != 0)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                }

                if (charCount >= result.length)
                    result = this._padLeft('0', charCount, result);
                else if (charCount != 0) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                }

                if (!isNaN(this._parseDouble(result)) && (this._parseDouble(result) > 536870911.0)) {
                    if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                        throw this.getLibraryComputationException();
                    else
                        return this.getErrorStrings()[4].toString();
                }
            } catch (ex) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }

            return result;
        };
        
        this.computeHex2Oct = function (argList) {
            var charCount = 0;
            var ranges = this.splitArgsPreservingQuotedCommas(argList);
            if (argList == this._parseArgumentSeparator.toString()) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[0].toString();
            }
            var value = this.getValueFromArg(ranges[0]);
            ranges[0] = ranges[0].split(this.TIC).join("");
            var chars = (ranges.length == 2) ? this.getValueFromArg(ranges[1]) : "0";
            if ((value[0] == this.tic) && (value.split(this.tic).join("") == null || value.split(this.tic).join("") == "")) {
                return "0";
            } else if (value.indexOf(this._parseArgumentSeparator.toString()) > -1) {
                var len = value.indexOf(this._parseArgumentSeparator);
                value = value.split(this.tic).join("").substring(0, len - 1);
            }
            var result = "";
            if (isNaN(this._parseDouble(chars.split(this.tic).join("")))) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[5].toString();
            }
            charCount = this._parseDouble(chars);
            if (charCount < 0 || value[0] == "-") {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }
            if (ranges.length > 2) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else
                    return this.getErrorStrings()[4].toString();
            }

            if (ranges.length > 1)
                charCount = this._parseDouble(ranges[1]);
            try {
                value = value.split(this.tic).join("");
                result = parseInt(value, 16).toString(8);
                if (!isNaN(this._parseDouble(result)) && result.length > 10) {
                    result = result.substring(result.length - 10);
                }
                if (ranges.length > 1) {
                    if (charCount >= result.length && charCount <= 10)
                        result = this._padLeft('0', charCount, result);
                    else {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        else
                            return this.getErrorStrings()[4].toString();
                    }
                    if (!isNaN(this._parseDouble(result)) && (this._parseDouble(result) > 536870911.0)) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        else
                            return this.getErrorStrings()[4].toString();
                    }
                }
            } catch (Exception) {
                if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null)
                    throw this.getLibraryComputationException();
                else {
                    if (this.computeIsText(value) == this.trueValueStr)
                        return this.getErrorStrings()[4].toString();
                    else if (value == "")
                        return "0";
                    else
                        return this.getErrorStrings()[4].toString();
                }
            }
            return result;
        };
        this._padLeft = function (paddingChar, length, stringpad) {
            var strlen = stringpad.length;
            if ((stringpad.length < length) && (paddingChar.toString().length > 0)) {
                for (var i = 0; i < (length - strlen) ; i++)
                    stringpad = paddingChar.toString().charAt(0).concat(stringpad);
            }
            return stringpad;
        };


        this.computeAddress = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var ans = this._string_empty;
            var arg0 = this.getValueFromArg(args[0].split(this.tic).join(this._string_empty));
            var arg1 = this.getValueFromArg(args[1].split(this.tic).join(this._string_empty));
            var row = this._parseDouble(arg0);
            var col = this._parseDouble(arg1);
            if (row < 1 || col < 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }
            var absNum = (argCount > 2) ? this.getValueFromArg(args[2].split(this.tic).join(this._string_empty)) : "1";
            var temp = col;
            var rem = temp % 26;
            temp = parseInt(temp / 26);
            ans = ans + this._charTable[rem];
            ans = ans + this._charTable[parseInt(temp)];
            var arr = (ans.split('').join(',')).split(this.getParseArgumentSeparator());
            arr.reverse();
            var ans1 = arr;
            ans = '$' + ans1 + '$' + arg0;
            if (argCount > 2) {
                switch (absNum) {
                    case "":
                        break;
                    case "1":
                        break;
                    case "2":
                        ans = ans1 + '$' + arg0;
                        break;
                    case "3":
                        ans = '$' + ans1 + arg0;
                        break;
                    case "4":
                        ans = ans1 + arg0;
                        break;
                }
            }
            if (argCount > 3 && (this.getValueFromArg(args[3].split(this.tic).join(this._string_empty)) == "FALSE" || this.getValueFromArg(args[3].split(this.tic).join(this._string_empty)) == "0")) {
                switch (absNum) {
                    case "":
                        ans = "R" + row + "C" + col;
                        break;
                    case "1":
                        ans = "R" + row + "C" + col;
                        break;
                    case "2":
                        ans = "R" + row + "C[" + col + "]";
                        break;
                    case "3":
                        ans = "R[" + row + "]C" + col;
                        break;
                    case "4":
                        ans = "R[" + row + "]C[" + col + "]";
                        break;
                }
            }
            if ((argCount > 4 && this.getValueFromArg(args[4].split(this.tic).join(this._string_empty)) == this._string_empty)) {
                args[4] = this.getValueFromArg(args[4]);
                ans = args[4].split(this.tic).join(this._string_empty) + "!" + ans;
            }
            return ans;
        };
        this.computeAreas = function (arg) {
            if (arg.indexOf(" ") > 0) {
                arg = arg.substring(0, arg.indexOf(" "));
            }
            if (!this._isCellReference(arg) && !this.getNamedRanges().containsKey(arg) && !arg.indexOf(")")) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var args1;
            args1 = this._splitArguments(arg, ')');
            if (args1.length > 2) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.getErrorStrings()[1].toString();
            }
            var argument = args1[0].split("(").join(this._string_empty);
            var args;
            args = this.splitArgsPreservingQuotedCommas(argument);
            var argCount = args.length;
            for (var i = 0; i < args.length; i++) {
                if (!this._isCellReference(args[i])) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.getErrorStrings()[1].toString();
                }
            }
            return args.length.toString();
        };
        this.computeChoose = function (arg) {
            var args;
            args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length < 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var s3 = this.getValueFromArg(args[0]).split(this.tic).join("");
            var indexArgs = this.splitArgsPreservingQuotedCommas(s3);
            s3 = indexArgs[0];
            var loc = parseInt(s3);
            if (isNaN(loc) || loc > args.length - 1 || loc < 1) {
                return "#VALUE!";
            }
            if (this._isInteriorFunction)
            {
                this._isInteriorFunction = !this._isInteriorFunction;
                return args[loc];
            }
            var value = this.getValueFromArg(args[loc]);
            if (value == "")
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[5].toString();
            }
            return value;
        };
        this.computeColumn = function (arg) {
            var rowInd, colInd;
            if (arg == null || arg == this._string_empty) {
                colInd = this.colIndex(this.cell).toString();
                rowInd = this.rowIndex(this.cell).toString();
                return colInd;
            }

            colInd =  this.colIndex(arg).toString();
            rowInd = this.rowIndex(arg).toString();
            if (rowInd <= 0)
            {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[5].toString();
            }
            return colInd;
        };
        this.computeColumns = function (arg) {
            var args;
            args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var val = -1;
            if (this._parseDouble(args[0], val)) {
                return "1";
            } else if ((args[0].indexOf(this.tic) && !args[0].indexOf(";") && !args[0].indexOf(this.getParseArgumentSeparator().toString())) || (!this._isCellReference(args[0]) && !this.getNamedRanges().containsValue(args[0]) && !arg.indexOf(";") && !arg.indexOf(this.getParseArgumentSeparator().toString()))) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            }

            var firstCol, LastCol, totalCols = 1;
            if (arg.indexOf(':') > -1) {
                var cells;
                cells = this.getCellsFromArgs(arg);
                firstCol = this.colIndex(cells[0].toString());
                LastCol = this.colIndex(cells[cells.length - 1].toString());
                totalCols = LastCol - firstCol + 1;
                totalCols = (totalCols > 0) ? totalCols : 1;
            } else if (arg.indexOf(';') > -1) {
                var arrayArg;
                arrayArg = this._splitArguments(arg.split(this.tic).join(this._string_empty), ';');
                for (var i = 1; i < arrayArg.length; i++) {
                    if (this.splitArgsPreservingQuotedCommas(arrayArg[i - 1]).length != this.splitArgsPreservingQuotedCommas(arrayArg[1]).length) {
                        if (this._rethrowLibraryComputationExceptions)
                            throw this.formulaErrorStrings[this._wrong_number_arguments];
                        return this.formulaErrorStrings[this._wrong_number_arguments];
                    }
                    totalCols = this.splitArgsPreservingQuotedCommas(arrayArg[0]).length;
                }
            } else {
                var arrayArg;
                arrayArg = this.splitArgsPreservingQuotedCommas(args[0].split(this.tic).join(this._string_empty));
                totalCols = arrayArg.length;
            }
            return totalCols.toString();
        };
        this.computeFormulaText = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            value = this.getValueFromArg(args[0]);
            if (this.getNamedRanges().containsValue(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (args[0].indexOf(this.sheetToken) == -1 && this.isSheetMember() && family.parentObjectToToken != null) {
                args[0] = family.parentObjectToToken.getItem(this.grid) + args[0];
            }
            if (this.getFormulaInfoTable().containsKey(args[0])) {
                var formula = this.getFormulaInfoTable().getItem(args[0]);
                value = formula.getFormulaText();
                this.getFormulaText(value);
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._bad_formula];
                }
                return (this.rowIndex(args[0]) <= 0) ? this.getErrorStrings()[5].toString() : this.getErrorStrings()[0].toString();
            }

            return value;
        };
        this.computeHyperlink = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 2) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstStr = this._stripTics0(this.getValueFromArg(args[0]));
            if (argCount == 2) {
                var displayText = this._stripTics0(this.getValueFromArg(args[1]));

                if (displayText == null) {
                    return "0";
                }
                if (displayText == this._string_empty) {
                    return this._string_empty;
                }
                return displayText;
            }
            return firstStr;
        };
        this.computeAbs = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            argValue = (argValue.split(this.tic).join("") == "TRUE") ? "1" : (argValue.split(this.tic).join("") == "FALSE") ? "0" : argValue;
            var dVal = this._parseDouble(argValue.split(this.tic).join(""));
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            return Math.abs(dVal).toString();
        };
        this.computeAcos = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal > 1 || dVal < -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return Math.acos(dVal).toString();
        };
        this.computeAcosh = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            z = Math.log(dVal + Math.sqrt(dVal * dVal - 1));
            if (dVal <= 0 || z == Infinity) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return z.toString();
        };
        this.computeAcot = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal < 1 && dVal > -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var d1 = this._parseDouble(argList), d2 = 0;

            if (!isNaN(d1)) {
                d2 = Math.PI / 2 - Math.atan(d1);
            }
            return d2.toString();
        };
        this.computeAcoth = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal < 1 && dVal > -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var acothVal = (Math.log((dVal + 1) / (dVal - 1)) / 2).toString();

            if (this.computeIsErr(acothVal) == this.trueValueStr)
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
            return this.getErrorStrings()[4].toString();

            return acothVal;
        };
        this.computeArabic = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            var dvalue = parseInt(text.split(this.tic).join("").toUpperCase());
            if (!isNaN(dvalue) || (!this._isCellReference(args[0]) && args[0][0] != this.tic)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (!isNaN(dvalue)) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            text = text.split(this.tic).join("").toUpperCase()
            var i = 0;
            var ch = null;
            var result = 0;
            var new_value = 0;
            var old_value = 0;
            old_value = 1000;
            var isNegative = false;
            for (i = 1; i <= text.length; i++) {
                // See what the next character is worth.
                ch = this._substring(text, i - 1, 1);
                switch (ch) {
                    case "-":
                        isNegative = true;
                        break;
                    case "I":
                        new_value = 1;
                        break;
                    case "V":
                        new_value = 5;
                        break;
                    case "X":
                        new_value = 10;
                        break;
                    case "L":
                        new_value = 50;
                        break;
                    case "C":
                        new_value = 100;
                        break;
                    case "D":
                        new_value = 500;
                        break;
                    case "M":
                        new_value = 1000;
                        break;
                    default: {
                        if (this._rethrowLibraryComputationExceptions)
                            throw this.formulaErrorStrings[this._bad_formula];
                        return this.getErrorStrings()[1].toString();
                    }
                }

                // See if this character is bigger
                // than the previous one.
                if (new_value > old_value) {
                    // The new value > the previous one.
                    // Add this value to the result
                    // and subtract the previous one twice.
                    result = result + new_value - 2 * old_value;
                } else {
                    // The new value <= the previous one.
                    // Add it to the result.
                    result = result + new_value;
                }
                old_value = new_value;
            }
            if (isNegative)
                result = -result;
            return result.toString();
        };
        this.computeAsin = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal > 1 || dVal < -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            var d1 = this._parseDouble(argValue), d2 = 0;

            if (!isNaN(d1)) {
                d2 = Math.asin(d1);
            }
            return d2.toString();
        };
        this.computeAtan = function (args) {
            return this._computeMath(args, Math.atan).toString();
        };
        this.computeAtan2 = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this.requires_2_args];
            }

            var x;
            var y = 0;
            var atan2 = 0;
            var param1, param2;
            var flag1, flag2;
            param1 = this.getValueFromArg(args[0]);
            param2 = this.getValueFromArg(args[1]);

            flag1 = (param1 == "true") ? true : false;
            flag2 = (param2 == "true") ? true : false;

            if (flag1)
                param1 = (flag1).toString();
            else if (param1 == (this._string_empty))
                param1 = "0";
            if (flag2)
                param2 = flag2.toString();
            else if (param2 == (this._string_empty))
                param2 = "0";
            if (this.getErrorStrings().indexOf(param1) > -1) {
                return param1;
            }
            if (this.getErrorStrings().indexOf(param2) > -1) {
                return param2;
            }
            x = this._parseDouble(param1);
            y = this._parseDouble(param2);
            if (isNaN(x) || isNaN(y)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                else
                    return this.getErrorStrings()[1].toString();
            } else if (y == 0 && x == 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[3].toString();
                else
                    return this.getErrorStrings()[3].toString();
            }
            atan2 = Math.atan2(y, x);
            return atan2.toString();
        };
        this.computeCeilingMath = function (args) {
            var range = this.splitArgsPreservingQuotedCommas(args);
            var argCount = range.length;
            if (this._isTextEmpty(args) || argCount > 3 || argCount < 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var range0 = this.getValueFromArg(range[0]);
            var range1 = (argCount > 1 && range[1].length != 0) ? this.getValueFromArg(range[1]) : "1";
            var range2 = (argCount == 3 && range[2].length != 0) ? this.getValueFromArg(range[2]) : "1";

            var d1, d2 = -1, d3 = -1, result;
            var numberresult = false, signresult = false, moderesult = false;
            numberresult = (range0.split(this.tic).join(this._string_empty) == "true") ? true : false;
            signresult = (range1.split(this.tic).join(this._string_empty) == "true") ? true : false;
            moderesult = (range2.split(this.tic).join(this._string_empty) == "true") ? true : false;
            range0 = numberresult ? (numberresult).toString() : range0;
            range1 = signresult ? (signresult).toString() : range1;
            range2 = moderesult ? (moderesult).toString() : range2;

            if (this.getErrorStrings().indexOf(range0) != -1)
                return range0;
            if (this.getErrorStrings().indexOf(range1) != -1)
                return range1;
            if (this.getErrorStrings().indexOf(range2) != -1)
                return range2;
            if (range1.indexOf(" ") > -1) {

                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }
            else if (range1.startsWith(this.tic))
            {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[4].toString();
            }


            d1 = this._parseDouble(range0); //.split(this.tic).join(this._string_empty);
            d2 = this._parseDouble(range1); //.split(this.tic).join(this._string_empty);
            d3 = this._parseDouble(range2); //.split(this.tic).join(this._string_empty);
            if (isNaN(d1) && !isNaN(d2) && !isNaN(d3)) {
                if ((d1 == 0 && !this._isTextEmpty(range0) && (this.computeIsText(range0) == this.falseValueStr)) || (d2 == 0 && !this._isTextEmpty(range1) && (this.computeIsText(range1) == this.falseValueStr) || (d3 == 0 && !this._isTextEmpty(range2) && (this.computeIsText(range2) == this.falseValueStr)))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[5].toString();
                }
                if ((this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) && range0[0] == (this.tic)) || (this._isTextEmpty(range1.split(this.tic).join(this._string_empty)) && range1[1] == (this.tic)) || (this._isTextEmpty(range2.split(this.tic).join(this._string_empty)) && range2[2] == (this.tic))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.getErrorStrings()[1].toString();
                }
                if (this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) || this._isTextEmpty(range1.split(this.tic).join(this._string_empty)) || this._isTextEmpty(range2.split(this.tic).join(this._string_empty))) {
                    return "0";
                }
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.getErrorStrings()[1].toString();
            }

            if (d1 < 0 && d3 < 0) {
                d2 = -1;
            }

            if (d1 == 0 || d2 == 0) {
                return "0";
            }
            var divideResult = 0;
            d1 = this._parseDouble(d1.toString());
            if (d1 > 0) {

                d1+=.4999999999;
            }
            if (d1 < 0 && -0.5 >= d1 - parseInt(d1)) {
                d1 -= .4999999999;
            }
            d1 =  Math.round(d1);
            divideResult = Math.floor(d1 / d2);

            result = divideResult * d2;

            if (d1 < 0 && (d1 % 2 != 0)) {
                if (d3 == 0) {
                    result += d2;
                }
            }

            return result.toString();
        };
        this.computeCeiling = function (args) {
            var range = this.splitArgsPreservingQuotedCommas(args);
            var argCount = range.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var range0 = (argCount > 1) ? this.getValueFromArg(range[0]) : "1";
            var range1 = (argCount > 1 && range[1].length != 0) ? this.getValueFromArg(range[1]) : "0";
            range1 = (range1 == "" || range1 == null) ? "0" : range1;
            var numberresult = false, signresult = false;

            numberresult = (range0.split(this.tic).join(this._string_empty) == "true") ? true : false;
            signresult = (range1.split(this.tic).join(this._string_empty) == "true") ? true : false;
            range0 = numberresult ? numberresult.toString() : range0;
            range1 = signresult ? signresult.toString() : range1;

            if (range1 == this.trueValueStr)
                return "1";
            else if (range1 == this.falseValueStr)
                return "0";
            if (range1.indexOf(" ") > -1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_index];
                return this.getErrorStrings()[1].toString();
            }

            if (this.getErrorStrings().indexOf(range0) != -1)
                return range0;
            if (this.getErrorStrings().indexOf(range1) != -1)
                return range1;
            var d1, d2 = -1, d3 = -1;
            d1 = this._parseDouble(range0);
            d2 = this._parseDouble(range1);
            if (isNaN(d1) && !isNaN(d2)) {
                if ((d1 == 0 && !this._isTextEmpty(range0) && (this.computeIsText(range0) == this.falseValueStr)) || (d2 == 0 && !this._isTextEmpty(range1) && (this.computeIsText(range1) == this.falseValueStr))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_index];
                    return this.getErrorStrings()[5].toString();
                }
                if ((this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) && range0[0] == (this.tic)) || (this._isTextEmpty(range1.split(this.tic).join(this._string_empty)) && range1[0] == (this.TIC))) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this.invalid_arguments];
                    return this.getErrorStrings()[1].toString();
                }
                if (this._isTextEmpty(range0.split(this.tic).join(this._string_empty)) || this._isTextEmpty(range1.split(this.tic).join(this._string_empty))) {
                    return "0";
                }
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.getErrorStrings()[1].toString();
            }
            d1 = this._parseDouble(range0);
            d2 = this._parseDouble(range1);
            if (!isNaN(d1) && !isNaN(d2)) {
                if (d1 > 0 && d2 < 0)
                {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.getErrorStrings()[4].toString();
                }
                if (d1 == 0 || d2 == 0)
                    return "0";
                d3 = Math.floor(d1 / d2) * d2;
                if (d2 > 0) {
                    while (d3 < d1) {
                        d3 += d2;
                    }
                } else {
                    while (d3 > d1) {
                        d3 += d2;
                    }
                }
            }
            return d3.toString();
        };
        this.computeDecimal2 = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 3) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var Base64 = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/";
            if (!this.getValueFromArg(args[0]).indexOf(this.tic) && !this._isCellReference(args[0])) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            var strValue;
            var radix = 0;
            try {
                strValue = this.getValueFromArg(args[0]).split(this.tic).join("");
                radix = parseInt(this.getValueFromArg(args[1]).split(this.tic).join(""));
            } catch (ex) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument] + ex;
                return this.getErrorStrings()[1].toString();
            }
            var result;
            if (radix < 2 || radix > Base64.length) {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Base requested outside range, it should be from 2 - 16";
                return this.getErrorStrings()[4].toString();
            }
            strValue = (this._isIE8) ? strValue.replace(/^\s+|\s+$/g, '') : strValue.trim();
            if (radix <= 36)
                strValue = strValue.toUpperCase();
            var pos = 0;
            result = 0;

            while (pos < strValue.length && !this._isWhiteSpace(strValue[pos])) {
                var digit = strValue.substring(pos, 1);
                var i = Base64.indexOf(digit);
                if (i >= 0 && i < radix) {
                    result *= radix;
                    result += i;
                    pos++;
                } else {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Base requested outside range, it should be from 2 - 16";
                    return this.getErrorStrings()[4].toString();
                }
            }
            return result.toString();
        };
        this.computeCombin = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._this.requires_2_args];
                else
                    return this.formulaErrorStrings[this.requires_2_args];
            }

            ////N things taken k at the time.
            var nd;
            var kd;
            var combin = 0;
            if (args[0].split(this.tic).join("") == (this._string_empty) || args[1].split(this.tic).join("") == (this._string_empty)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                else
                    return this.getErrorStrings()[1].toString();
            }
            nd = this._parseDouble(this.getValueFromArg(args[0]).split(this.tic).join(""));
            kd = this._parseDouble(this.getValueFromArg(args[1]).split(this.tic).join(""));
            if (!isNaN(nd) && !isNaN(kd)) {
                var k = parseInt((kd + 0.1).toString());
                var n = parseInt((nd + 0.1).toString());
                if (n < k || n < 1) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.getErrorStrings()[1].toString();
                    else
                        return this.getErrorStrings()[4].toString();
                }
                combin = this._comb(k, n);
            } else {
                if (!this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[1].toString();
                else
                    return this.formulaErrorStrings[this._invalid_arguments];
            }
            return combin.toString();
        };
        this.computeCombinA = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argList.indexOf("u") > -1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (argList.length > 15) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var number1 = parseInt(this.getValueFromArg(args[0]).split(this.tic).join(""));
            var choosedNumber = parseInt(this.getValueFromArg(args[1]).split(this.tic).join(""));
            var result;
            if (number1 == 0 && choosedNumber == 0) {
                result = "1";
            } else {
                var number2 = number1 + choosedNumber - 1;
                var pass = number2 + "," + (number1 - 1);
                result = this.computeCombin(pass);
            }
            return result;
        };
        this.computeCos = function (args) {
            return this._computeMath(args, Math.cos).toString();
        };
       
        this.computeCosh = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var y = Math.exp(value);
            return (y + 1 / y) / 2;
        };
        this.computeCot = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (argList[0] == (this.tic) && argList[length - 1] == (this.tic))
                argList = argList.split(this.tic).join("");
            if (argList == this._string_empty) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            var isNumber = this.computeIsNumber(argList);
            if (isNumber == this.trueValueStr) {
                var tempList = argList;
                tempList = tempList.indexOf("u") ? tempList.split("u").join(this._string_empty) : tempList;
                var number = this._parseDouble(tempList);
                if (number >= 134217728) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.getErrorStrings()[4].toString();
                    return this.getErrorStrings()[4].toString();
                }
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[5].toString();
            }
            if (argList == "0") {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[3].toString();
                return this.getErrorStrings()[3].toString();
            }
            var cotVal = this._computeMath(argList, Math.tan);
            if (cotVal != "#NUM!" || cotVal != "#VALUE!") {
                cotVal = (1 / parseFloat(cotVal)).toString();
            }
            return cotVal;
        };
        this.computeCsc = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal >= 134217728) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            
            var cscVal = (1 / parseFloat(Math.sin(dVal))).toString();
            return cscVal;
        };
        
        this.computeCsch = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var dVal = this._parseDouble(argValue);
            var isNumber = this.computeIsNumber(argList);
            if (isNaN(dVal)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (dVal == 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[3].toString();
            }
            if (dVal > 709) {
                return "0";
            }
            var cschVal = (2 / (Math.exp(dVal) - Math.exp(-dVal))).toString();

            return cschVal;
        };
        this.computeDecimal = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 3) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var Base64 = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz+/";
            if (this.getValueFromArg(args[0]).indexOf(this.tic) != -1 && !this._isCellReference(args[0])) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[1].toString();
            }
            var strValue;
            var radix = 0;
            try {
                strValue = this.getValueFromArg(args[0]).split(this.tic).join("");
                radix = parseInt(this.getValueFromArg(args[1]).split(this.tic).join(""));
            } catch (ex) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._invalid_Math_argument] + ex;
                return this.getErrorStrings()[1].toString();
            }
            var result;
            if (radix < 2 || radix > Base64.length) {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Base requested outside range, it should be from 2 - 16";
                return this.getErrorStrings()[4].toString();
            }

            strValue = (this._isIE8) ? strValue.replace(/^\s+|\s+$/g, '') : strValue.trim();
            if (radix <= 36)
                strValue = strValue.toUpperCase();
            var pos = 0;
            result = 0;

            while (pos < strValue.length ) {
                var digit = this._substring(strValue, pos, 1);
                var i = Base64.indexOf(digit);
                if (i >= 0 && i < radix) {
                    result *= radix;
                    result += i;
                    pos++;
                    
                } else {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Base requested outside range, it should be from 2 - 16";
                    return this.getErrorStrings()[4].toString();
                }
            }

            return result.toString();
        };
        this.computeDegrees = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argValue = this.getValueFromArg(args[0]);
            var radians = this._parseDouble(argValue);
            if (isNaN(radians)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argValue[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var degrees = 0;
            degrees = 180 * radians / Math.PI;
            return degrees.toString();
        };
        
        this.computeExp = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            if (argVal == "")
                argList = "0";
            var x = this._parseDouble(argVal);
            if (x > 709) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return this._computeMath(argList, Math.exp).toString();
        };

        this.computeEven = function (argList) {
            var even = 0;
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var number = this._parseDouble(argVal);
            if (!isNaN(number)) {
                var sgn = number > 0 ? 1 : (number === 0 || isNaN(number)) ? x : -1;
                number = Math.abs(number);
                number = Math.ceil(number);
                if ((number % 2) == 1) {
                    even = sgn * (number + 1);
                }
                else {
                    even = sgn * number;
                }
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }

            return even.toString();
        }

        this.computeFact = function (args) {
            var fact = 0;
            var numList = this.splitArgsPreservingQuotedCommas(args);
            if (numList.length > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            number = parseInt(this.getValueFromArg(args));
            if (isNaN(number)) {
                return this.getErrorStrings()[1].toString();
            } else if (number < 0) {
                return this.getErrorStrings()[4].toString();
            } else {
                var x = number;
                if (x > 12) {
                    fact = this._factorialTable[12];
                    for (var i = 13; i <= x; i++) {
                        fact *= i;
                    }
                } else {
                    fact = this._factorialTable[x];
                }
            }
            return fact.toString();
        };
        this.computeFactdouble = function (args) {
            var number;
            var fact = 1;
            var numList = this.splitArgsPreservingQuotedCommas(args);
            if (numList.length > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            number = parseInt(this.getValueFromArg(args));
            if (isNaN(number)) {
                return this.getErrorStrings()[1].toString();
            } else if (number < 0) {
                return this.getErrorStrings()[4].toString();
            } else {
                var x = number;
                var n = x;
                if (x > 3) {
                    while (n > 0) {
                        fact = fact * n;
                        n = n - 2;
                    }
                } else if (x == 0)
                    fact = 1;
                else
                    fact = x;
            }
            return fact.toString();
        };
        this.computeFloor = function (args) {
            var range = this.splitArgsPreservingQuotedCommas(args);
            var argCount = range.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var d1 = this._parseDouble(this.getValueFromArg(range[0]));
            var d2 = this._parseDouble(this.getValueFromArg(range[1])), d3 = 0;
            if (!isNaN(d1) && !isNaN(d2)) {
                if (d1 == 0)
                    return d3.toString();
                if (d1 * d2 <= 0 && !(d1 < 0)) {
                    return this.formulaErrorStrings[this._invalid_arguments];
                }

                d3 = Math.ceil(d1 / d2) * d2;
                if (d2 > 0) {
                    while (d3 > d1) {
                        d3 -= d2;
                    }
                } else {
                    while (d3 < d1) {
                        d3 -= d2;
                    }
                }
                if (d2 == 0.1) {
                    d3 = d1;
                }
            }
            return d3.toString();
        };
        this.computeInt = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var x;
           
            x = Math.floor(argVal).toString();
          
            if (!isNaN(x)) {
                return x;
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
        };
        this.computeLn = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var x = this._parseDouble(argVal);
            if (x <= 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            if (!isNaN(x)) {
                return (Math.log(x)).toString();
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
        };
        this.computeLog = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount > 2) {
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var argVal = this.getValueFromArg(args[0]);
            var x = this._parseDouble(argVal);
            var baseVal = (argCount == 2) ? this.getValueFromArg(args[1]) : "10";
            var b =  this._parseDouble(baseVal);
            if (x <= 0 || b <= 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            if (!isNaN(x) && !isNaN(b)) {
                return ((Math.log(x) / Math.LN10) / (Math.log(b) / Math.LN10)).toString();
            }
            else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || baseVal[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
        };
        this.computeLogTen = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (value <= 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return (this._computeMath(args, Math.log)/Math.LN10).toString();
        };
        
        this.computePI = function (args) {
            return Math.PI.toString();
        };
        this.computeProduct = function (range) {
            var prod = 1;
            var d;
            var s1;
            var nohits = true;
            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                var adjustRange = ranges[r];

                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s = 0; s < cellCollection.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]);
                        } catch (ex) {
                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                prod = prod * d;
                                nohits = false;
                            } else if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            }
                            else {
                                if (this._rethrowLibraryComputationExceptions)
                                    throw this.formulaErrorStrings[this._bad_formula];
                                return this.getErrorStrings()[1].toString();
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            prod = prod * d;
                            nohits = false;
                        } else if (this.getErrorStrings().indexOf(s1) > -1) {
                            return s1;
                        }
                        else {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.formulaErrorStrings[this._bad_formula];
                            return (s1 == this.tic || this._isCellReference(ranges[r])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
                        }
                    }
                }
            }
            return nohits ? "0" : prod.toString();
        };
        this.computeSecant = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            argList = this.getValueFromArg(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (argList[0] == (this.tic) && argList[length - 1] == (this.tic))
                argList = argList.split(this.tic).join("");
            if (argList == this._string_empty) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            var isNumber = this.computeIsNumber(argList);
            if (isNumber == this.trueValueStr) {
                var tempList = argList;
                tempList = tempList.indexOf("u") ? tempList.split("u").join("") : tempList;
                var number = parseFloat(tempList);
                if (number >= 134217728) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.getErrorStrings()[4].toString();
                    return this.getErrorStrings()[4].toString();
                }
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[5].toString();
            }
            var secVal = this._computeMath(argList, Math.cos);
            if (secVal != "#NUM!" || secVal != "#VALUE!")
                secVal = (1 / parseFloat(secVal)).toString();
            return secVal;
        };
        this.computeSeriessum = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 4) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var xValue = 0, nValue = 0, mValue = 0, coeff = 0, result = 0;
            var arrayArg;
            if (args[3].indexOf(';') > -1) {
                arrayArg = this._splitArguments(args[3].split(this.tic).join(""), ';');
            } else {
                arrayArg = this.getCellsFromArgs(args[3].split(this.tic).join(""));
            }
            if (args[0] == "" || args[1] == "" || args[2] == "") {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[0].toString();
            }
            xValue = this._parseDouble(this.getValueFromArg(args[0]));
            nValue = this._parseDouble(this.getValueFromArg(args[1]));
            mValue = this._parseDouble(this.getValueFromArg(args[2]));
            if (isNaN(xValue) || isNaN(nValue) || isNaN(mValue)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            arrayArg = this.getCellsFromArgs(args[3].split(this.tic).join(""));
            for (var i = 0; i < arrayArg.length; i++) {
                coeff = this._parseDouble(this.getValueFromArg(arrayArg[i]));
                if (!isNaN(coeff)) {
                    result = result + (coeff * Math.pow(xValue, (nValue + (i * mValue))));
                } else {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[1].toString();
                }
            }
            if (isNaN(result)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return result.toString();
        };
        this.computeSin = function (args) {
            return this._computeMath(args, Math.sin).toString();
        };
        this.computeSinh = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
           
            var y = Math.exp(value);
            return (y - 1 / y) / 2;
        };
        this.computeSqrt = function (args) {
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount > 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = parseInt(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            if (value < 0) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[4].toString();
            }
            return this._computeMath(args, Math.sqrt).toString();
        };
        this.computeSubTotal = function (args) {
            var ignoreSubtotal = false;
            var cellReference = this._string_empty;
            var val;
            var index = 0;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator().toString());
            var argCount = arg.length;
            if (argCount < 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (this._isCellReference(arg[0])) {
                cellReference = this.getValueFromArg(arg[0]);
            } else {
                cellReference = arg[0].toString();
            }
            val = this._parseDouble(cellReference);
            if (!isNaN(val)) {
                index = parseInt(val.toString());
                if ((index < 1 || index > 11) && (index < 101 || index > 111)) {
                    return this.getErrorStrings()[1].toString();
                }
            }
            else
            {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            }
            if (ignoreSubtotal)
                return "0";

            cellReference = args.substring((arg[0].toString()).length + 1);
            ignoreSubtotal = true;
            var value = this._string_empty;
            switch (index) {
                case 1:
                case 101:
                    value = this.computeAverage(cellReference);
                    break;
                case 2:
                case 102:
                    value = this.computeCount(cellReference);
                    break;
                case 3:
                case 103:
                    value = this.computeCounta(cellReference);
                    break;
                case 4:
                case 104:
                    value = this.computeMax(cellReference);
                    break;
                case 5:
                case 105:
                    value = this.computeMin(cellReference);
                    break;
                case 6:
                case 106:
                    value = this.computeProduct(cellReference);
                    break;
                case 7:
                case 107:
                    value = this.computeStdev(cellReference);
                    break;
                case 8:
                case 108:
                    value = this.computeStdevp(cellReference);
                    break;
                case 9:
                case 109:
                    value = this.computeSum(cellReference);
                    break;
                case 10:
                case 110:
                    value = this.computeVar(cellReference);
                    break;
                case 11:
                case 111:
                    value = this.ComputeVarp(cellReference);
                    break;
                default:
                    value = this.getErrorStrings()[1].toString();
                    break;
            }
            ignoreSubtotal = false;
            return value;
        };
        this.computeSumif = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var criteriaRange = args[0];
            var criteria = args[1];
            if (criteria.length < 1) {
                return "0";
            }

            var op = this.token_equal;
            if (criteria[0] != this.tic[0] && "=><".indexOf(criteria[0]) == -1) {
                ////cell reference
                criteria = this.getValueFromArg(criteria);
            }

            var offset = (criteria.length > 0 && criteria[0] == this.tic[0]) ? 1 : 0;
            var compare = this.minValue;
            if (criteria.substring(offset)[0] == (">=")) {
                criteria = this._substring(criteria, offset + 2, criteria.length - 2 - 2 * offset);
                op = this.token_greatereq;
            } else if (criteria.substring(offset)[0] == ("<=")) {
                criteria = this._substring(criteria, offset + 2, criteria.length - 2 - 2 * offset);
                op = this.token_lesseq;
            } else if (criteria.substring(offset)[0] == ("<")) {
                criteria = this._substring(criteria, offset + 1, criteria.length - 1 - 2 * offset);
                op = this.token_less;
            } else if (criteria.substring(offset)[0] == (">")) {
                criteria = this._substring(criteria, offset + 1, criteria.length - 1 - 2 * offset);
                op = this.token_greater;
            } else if (criteria.substring(offset)[0] == ("=")) {
                criteria = this._substring(criteria, offset + 1, criteria.length - 1 - 2 * offset);
            }

            criteria = criteria.split(this.tic).join("");
            var numer = this._parseDouble(criteria, compare);

            var sumRange = (argCount == 2) ? criteriaRange : args[2];

            var s1 = this.getCellsFromArgs(criteriaRange);
            var s2 = this.getCellsFromArgs(sumRange);

            var count = s1.length;

            if (count > s2.length) {
                var i = sumRange.indexOf(':');
                if (i > -1) {
                    var startRow = this.rowIndex(this._substring(sumRange, 0, i));
                    var startCol = this.colIndex(this._substring(sumRange, 0, i));
                    var row = this.rowIndex(this._substring(sumRange, i + 1));
                    var col = this.CclIndex(this._substring(sumRange, i + 1));
                    if (startRow != row) {
                        row += count - s2.length;
                    } else if (startCol != col) {
                        col += count - s2.length;
                    }

                    sumRange = this._substring(sumRange, 0, i + 1) + RangeInfo.getAlphaLabel(col) + row.toString();
                    s2 = this.getCellsFromArgs(sumRange);
                }
            }

            var sum = 0;

            var d;
            var s;
            switch (op) {
                case this.token_equal:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;

                            if (s.split(this.tic).join("") == criteria) {
                                s = s2[index];
                                s = this.getValueFromArg(s);
                                d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                                if (!isNaN(d)) {
                                    sum += d;
                                }
                            }
                        }
                    }

                    break;
                case this.token_greatereq:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d >= compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                case this.token_greater:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d > compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                case this.token_less:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d < compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                case this.token_lesseq:
                    {
                        for (var index = 0; index < count; ++index) {
                            s = this.getValueFromArg(s1[index]); //// +criteria;
                            d = this._parseDouble((s.split(this.tic).join("")).split("$").join(""));
                            if (!isNaN(d)) {
                                if (d <= compare) {
                                    s = s2[index];
                                    s = this.getValueFromArg(s);
                                    d = this._parseDouble(s);
                                    if (!isNaN(d)) {
                                        sum += d;
                                    }
                                }
                            }
                        }
                    }

                    break;
                default:
                    break;
            }
            return sum.toString();
        };
        this.computeTan = function (args) {
            return this._computeMath(args, Math.tan).toString();
        };
        this.computeTrunc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            var digits = 0;

            if (argCount == 2) {
                ////ignore return value...
                digits = this._parseDouble(this.getValueFromArg(args[1]));
            }
            var argVal = this.getValueFromArg(args[0]);
            var value = this._parseDouble(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(args[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var normalizer = Math.pow(10, digits);
            var d1 = (value < 0) ? -1 : 1;
            return (d1 * Math.floor(normalizer * Math.abs(value)) / normalizer).toString();
        };
        this.computeLognormOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var s;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.trueValueStr).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(u) && !isNaN(s)) {
                if (x <= 0 || s <= 0) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#Num! Passsed value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                if (args[3] == this.falseValueStr) {
                    var mean = 0;
                    var std = 0;

                    // mean = Math.Exp(u+Math.Pow(u,2))
                    dist = (Math.exp(-(Math.pow((Math.log(x) - u), 2) / (2 * Math.pow(s, 2))))); //this.normaldist(Math.Log(x), u, s);
                    dist = dist / (x * s * Math.sqrt(2 * Math.PI));
                } else {
                    dist = this._standardNormalCumulativeDistribution((Math.log(x) - u) / s);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#Value! Passsed value is non - numeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeLognormOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var u;
            var s;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            p = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(p) && !isNaN(u) && !isNaN(s)) {
                dist = Math.exp(this._normalinv(p, u, s));
            }
            return dist.toString();
        };
        this.computeNormOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var u;
            var s;
            var invdist = 0;

            for (var i = 0; i < argCount; ++i)
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);

            p = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(p) && !isNaN(u) && !isNaN(s)) {
                if (p >= 0 && p <= 0 || s <= 0) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw this.formulaErrorStrings[this._wrong_number_arguments] + this.getErrorStrings()[4].toString();
                    }
                    return this.getErrorStrings()[4].toString();
                }
                invdist = this._normalCumulativeDistributionFunctionInverse(p, u, s); //this.normalinv(p, u, s);
            }

            if (invdist <= 0) {
                return this.formulaErrorStrings[this._iterations_dont_converge];
            }
            return invdist.toString();
        };
        this.computeNormOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var s;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            s = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(u) && !isNaN(s)) {
                if (argCount != 3) {
                    cum = (args[3] == this.trueValueStr) ? 1 : 0;
                    var check = 0;
                    check = this._parseDouble(args[3]);
                    if (!isNaN(check))
                        cum = 1;
                }

                if (cum == 0) {
                    dist = this._normalProbabilityDensity(x, u, s);
                } else {
                    dist = this._normaldist(x, u, s);
                }
            }
            return dist.toString();
        };
        this.computeNormOsODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1 && argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            if (!isNaN(x)) {
                if (argCount == 2) {
                    cum = (args[1] == this.trueValueStr) ? 1 : 0;
                    var check = 0;
                    check = this._parseDouble(args[1]);
                    if (!isNaN(check))
                        cum = 1;
                }

                if (cum == 0) {
                    dist = this._standardNormalProbabilityDensityFunction(x);
                } else {
                    dist = this._standardNormalCumulativeDistributionFunction(x);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is non numeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeNormOsOInv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            if (!isNaN(x)) {
                if (x <= 0 || x >= 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value doesnot match with in range level";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                dist = this._standardNormalCumulativeDistributionFunctionInverse(x);
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is non numeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computePermut = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this.requires_2_args];
            }

            ////N things taken k at the time.
            var nd;
            var kd;
            var combin = 0;
            nd = this._parseDouble(this.getValueFromArg(args[0]));
            kd = this._parseDouble(this.getValueFromArg(args[1]));
            if (!isNaN(nd) && !isNaN(kd)) {
                var k = (kd + 0.1);
                var n = (nd + 0.1);

                var top = 1;
                for (var i = (n - k) + 1; i <= n; ++i) {
                    top = top * i;
                }

                combin = top;
            } else {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            return combin.toString();
        };
        this.computePermutationA = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this.requires_2_args];
            }

            var nd;
            var kd;
            var combin = 0;
            nd = this._parseDouble(args[0]);
            kd = this._parseDouble(args[1]);
            if (!isNaN(nd) && !isNaN(kd)) {
                var k = (kd + 0.1);
                var n = (nd + 0.1);
            } else {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            return Math.pow(nd, kd).toString();
        };
        this.computeStandardize = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var sd = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            sd = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(u) && !isNaN(sd)) {
                dist = (x - u) / sd;
            }
            return dist.toString();
        };
        this.computeBinomOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var successes;
            var trials;
            var p;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            successes = this._parseDouble(args[0]);
            trials = this._parseDouble(args[1]);
            p = this._parseDouble(args[2]);
            if (!isNaN(successes) && !isNaN(trials) && !isNaN(p)) {
                if (successes < 0 || successes > trials || p < 0 || p > 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NAME! Passed argument value is not equal to minimum par value";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                cum = (args[3] == this.trueValueStr) ? 1 : 0;
                if (cum == 0) {
                    dist = this._comb(successes, trials) * Math.pow(p, successes) * Math.pow(1 - p, trials - successes);
                } else {
                    dist = this._binomdist(trials, successes, p);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed value is nonnumeric";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeChisqOinvOrt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var v;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            p = this._parseDouble(args[0]);
            v = this._parseDouble(args[1]);
            if (!isNaN(p) && !isNaN(v)) {
                dist = this._chiinv(p, v);
            }
            return dist.toString();
        };
        this.computeChisqOdistOrt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var v;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            v = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(v)) {
                if (v == 1 && this._excelLikeComputations) {
                    //continue;
                } else if (v < 1) {
                    if (this._excelLikeComputations)
                        return "#NUM!";
                    return this.formulaErrorStrings[this._invalid_arguments];
                }

                dist = 1 - this._chidist(x, v);
                for (var i = 0; i < dist.toString().length; i++) {
                }
                var ss = Math.round(dist);
            }
            return dist.toString();
        };
        this.computeFOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var df2 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            df2 = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(df1) && !isNaN(df2)) {
                if (df1 < 1 || df2 < 1 || (x[0] == ("-"))) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }

                if (args[3] == this.trueValueStr) {
                    dist = this._fCumulativeDensity(x, df1, df2);
                } else if (args[3] == this.falseValueStr) {
                    dist = this._fProbabilityDensity(x, df1, df2);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is incorrect";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeGammaln = function (argList) {
            var x = 0;
            x = this._parseDouble(argList);
            if (!isNaN(x) && x > 0) {
                x = this._gammaln(x);
            } else {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            return x.toString();
        };
        this.computeConfidenceOnorm = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "Wrong number of arguments";
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var alpha;
            var s;
            var sz;
            var val = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }

            alpha = this._parseDouble(args[0]);
            s = this._parseDouble(args[1]);
            sz = this._parseDouble(args[2]);
            if (!isNaN(alpha) && !isNaN(s) && !isNaN(sz)) {
                if (alpha <= 0 || alpha >= 1 || s <= 0 || sz < 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "Passed argument value is different from minimum par";
                    }
                    return this.getErrorStrings()[4].toString();
                }

                //// 1 - alpha  would be from -inf to 1-alpha. adding alpha + 2
                //// takes away the left tail making the integral from alpha-1 to 1-alpha.
                if (this._excelLikeComputations)
                    val = this._normalinv(1 - alpha + alpha / 2, 0, 1);
                else
                    val = this._newnormalinv(1 - alpha + alpha / 2);
                val = val * s / Math.sqrt(sz);
            }

            return val.toString();
        };
        this.computeExponOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var lambda;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            lambda = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(lambda)) {
                if (x < 0 || lambda <= 0) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed argument value is below or equal to 0";
                    return this.getErrorStrings()[4].toString();
                }
                cum = (args[2] == this.trueValueStr) ? 1 : 0;
                var check = this._parseDouble(args[2]);
                if (!isNaN(check))
                    cum = 0;
                if (cum == 0) {
                    dist = lambda * Math.exp(-lambda * x);
                } else {
                    dist = 1 - Math.exp(-lambda * x);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed argument value is non-numerical";
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
        this.computeFisher = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var z = 0;

            var s = this.getValueFromArg(args[0]);

            x = this._parseDouble(s);
            if (!isNaN(x) && (x > -1 && x < 1)) {
                z = 0.5 * Math.log((1 + x) / (1 - x));
            }
            return z.toString();
        };
        this.computeFisherInv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var y;
            var x = 0;

            var s = this.getValueFromArg(args[0]);

            y = this._parseDouble(s);
            if (!isNaN(y)) {
                var d = Math.exp(2 * y);
                x = (d - 1) / (d + 1);
            }
            return x.toString();
        };
        this.computeGammalnOPrecise = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (args[0] == this.tic + this.tic) {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed Argument is empty";
                return this.getErrorStrings()[1].toString();
            }

            if (argCount != 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x = 0;
            x = this._parseDouble(args.toString());
            if (!isNaN(x) && x > 0)
                x = this._gammaln(x);
            else {
                if (x <= 0) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed Argument value is less than or equal to minimum value 0";
                    return this.getErrorStrings()[4].toString();
                }
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed Argument is non numerical";
                return this.getErrorStrings()[1].toString();
            }
            return x.toString();
        };
        
        this.computeLarge = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x0;
            var s1 = this.getValueFromArg(args[1]);
            x0 = this._parseDouble(s1);
            if (isNaN(x0)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var k = x0;
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;
            if (k < 1 || k > n) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a > b) return 1;
                    else return -1;
                }
                return a - b;
            });
            return dd[n - k].toString();
        };
        this.computeSmall = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var s1 = this.getValueFromArg(args[1]);
            var x0 = this._parseDouble(s1);
            if (isNaN(x0)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var k = x0;
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;
            if (k < 1 || k > dd.length) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
           
            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a < b) return 1;
                    else return -1;
                }
                return b - a;
            });
            return dd[n-k].toString();
        };
        this.computeCounta = function (range) {
            var count = 0;
            var s1;
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var argsRange;

            if (this._isTextEmpty(range)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_arguments];
                else
                    return this.formulaErrorStrings[this.invalid_arguments];
            }
            for (var r = 0; r < ranges.length; r++) {
                argsRange = ranges[r];

                ////is a cellrange
                if (argsRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(argsRange);
                    for (var s = 0; s < cellCollection.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join(this._string_empty);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            count++;
                        }
                    }
                } else if (argsRange == "") {
                    count++;
                } else if (this._isLetter(argsRange[0])) {
                    s1 = this.getValueFromArg(argsRange);
                    if (s1.length > 0) {
                        count++;
                    }
                } else {
                    if (argsRange.indexOf(this.getParseArgumentSeparator()) > -1) {
                        var array = this.splitArgsPreservingQuotedCommas(argsRange.split(this.tic).join(""));
                        for (var str = 0; str < array.length; str++) {
                            if (str[0] != this.tic && this._isCellReference(array[str])) {
                                if (this._rethrowLibraryComputationExceptions)
                                    throw this.formulaErrorStrings[this.invalid_arguments];
                                else
                                    return this.formulaErrorStrings[this.invalid_arguments];
                            }
                            if (str.length > 0 || str == "") {
                                count++;
                            }
                        }
                    }
                    else if (r.length > 0) {
                        count++;
                    }
                }
            }
            return count.toString();
        };
        this.computeAverage = function (range) {
            var sum = 0, count = 0, d, s1, adjustRange = [];
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            if (ranges.length < 1) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_arguments];
                else
                    return this.formulaErrorStrings[this.invalid_arguments];
            }
            for (var r = 0; r < ranges.length; r++) {
                adjustRange = ranges[r];
                if (adjustRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s = 0; s < cellCollection.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join("");
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[4].toString();
                            else
                                return this.getErrorStrings()[4].toString();
                        }

                        if (s1.length > 0) {
                            if(!isNaN(s1)){
                                d = this._parseDouble(s1);
                                if (!isNaN(d)) {
                                    sum = Number(sum) + Number(d);
                                    count++;
                                } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                    return s1;
                                } else {
                                    if (this._rethrowLibraryComputationExceptions)
                                        throw this.getErrorStrings()[5].toString();
                                    else
                                        return this.getErrorStrings()[5].toString();
                                }
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join(this._string_empty);
                        var s2 = this.getValueFromArg(adjustRange);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions)
                            throw this.getErrorStrings()[4].toString();
                        else
                            return this.getErrorStrings()[4].toString();
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            sum = Number(sum) + Number(d);
                            count++;
                        } 
                        else if (s2[0] == (this.tic)) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[1].toString();
                            else
                                return this.getErrorStrings()[1].toString();
                        } else {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[5].toString();
                            else
                                return this.getErrorStrings()[5].toString();
                        }
                    }
                }
            }

            if (count > 0) {
                sum = Number(sum) / Number(count);
            }
            return sum.toString();
        };
        this.computeAverageA = function (range) {
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var newargs = this._string_empty;
            var d = 0;
            if (ranges == this.trueValueStr || ranges == this.falseValueStr) {
                var boolvalue = (ranges == this.trueValueStr);
                if (boolvalue)
                    return +boolvalue;
                else
                    return +boolvalue;
            }

            for (var i = 0; i < ranges.length; i++) {
                if (ranges[i].indexOf(this.getParseArgumentSeparator().toString()) > -1) {
                    var temp = this.splitArgsPreservingQuotedCommas(ranges[i].split(this.tic).join(this._string_empty));
                    var j = 0;
                    for (var item = 0; item < temp.length; item++) {
                        var adjustRange = temp[item];
                        newargs = newargs + adjustRange + this.getParseArgumentSeparator().toString();
                        ranges[j] = adjustRange;
                    }
                } else if (this._isCellReference(ranges[i]))
                    newargs += ranges[i] + this.getParseArgumentSeparator().toString();
                else if (!this._parseDouble(ranges[i], d))
                    newargs = newargs + "0" + this.getParseArgumentSeparator().toString();
                else
                    newargs = newargs + ranges[i] + this.getParseArgumentSeparator().toString();
            }

            // if (newargs.endsWith(this.getParseArgumentSeparator().toString()))
            if (newargs[newargs.length - 1] == (","))
                newargs = newargs.substring(0, newargs.length - 1);

            var sum = this.computeAverage(newargs);
            return sum;
        };
        this.computeMax = function (range) {
            var max = this.minValue;
            var d;
            var s1;
            var argslist;
            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                argslist = ranges[r];

                ////cell range
                if (argslist.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(argslist);
                    for (var s = 0; s < cellCollection.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join(this._string_empty);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                max = Math.max(max, d);
                            } else if (this.getErrorStrings().indexOf(s1)) {
                                return max;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = (argslist == this._string_empty) ? "0" : this.getValueFromArg(argslist);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            max = Math.max(max, d);
                        } else if (this.getErrorStrings().indexOf(s1)) {
                            return s1;
                        }
                    }
                }
            }
            if (max != this.minValue) {
                return max.toString();
            }

            return this._string_empty;
        };
        this.computeMaxa = function (range) {
            var max = this.minValue;
            var d;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cell.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                                return ex;
                            }
                        }

                        ////ignore if empty
                        if (s1.length > 0) {
                            d = 0;
                            if (s1.toUpperCase() == (this.trueValueStr)) {
                                d = 1;
                            } else if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            } else {
                                d = this._parseDouble(s1);
                            }

                            max = Math.max(max, d);
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                            return ex;
                        }
                    }

                    if (s1.length > 0) {
                        d = 0;
                        if (s1.toUpperCase() == (this.trueValueStr)) {
                            d = 1;
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        } else {
                            d = this._parseDouble(s1);
                        }

                        max = Math.max(max, d);
                    }
                }
            }
            if (max != this.minValue) {
                return max.toString();
            }

            return this._string_empty;
        };
        this.computeMedian = function (range) {
            var dd = this._getDoubleArray(range);
            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a > b) return 1;
                    else return -1;
                }
                return a - b;
            });
            var n = parseInt((dd.length / 2).toString());
            var s1 = "";
            if (dd.length % 2 == 1) {
                s1 = dd[n].toString();
            } else {
                s1 = ((dd[n] + dd[n - 1]) / 2).toString();
            }
            return s1;
        };
        this.computeMin = function (range) {
            var min = this.maxValue;
            var d;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var rang = range.split(';');
            if (rang.length > 1) {
                for (var r = 0; r < rang.length; r++) {
                    var valRange = this.splitArgsPreservingQuotedCommas(rang[r]);
                    for (var s = 0; s < valRange.length; s++) {
                        var tempval = this.getValueFromArg(valRange[s]);
                        if (tempval.length > 0) {
                            d = this._parseDouble(tempval);
                            if (!isNaN(d)) {
                                min = Math.min(min, d);
                            } else if (this.getErrorStrings().indexOf(tempval) == -1) {
                                return tempval;
                            }
                        }
                    }
                }
                if (min != this.maxValue) {
                    return min.toString();
                }
            }
            for (var r = 0; r < ranges.length; r++) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cell.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                min = Math.min(min, d);
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return min;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = (ranges[r] == this._string_empty) ? "0" : this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            min = Math.min(min, d);
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        }
                    }
                }
            }
            if (min != this.maxValue) {
                return min.toString();
            }

            return this._string_empty;
        };
        this.computeMina = function (range) {
            var min = this.maxValue;
            var d;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var rang = range.split(';');
            if (rang.length > 1) {
                for (var r = 0; r < rang.length; r++) {
                    var valRange = this.splitArgsPreservingQuotedCommas(rang[r]);
                    for (var s = 0; s < valRange.length; s++) {
                        var tempval = this.getValueFromArg(valRange[s]);
                        if (tempval.length > 0) {
                            d = 0;
                            if (tempval.toUpperCase() == (this.trueValueStr)) {
                                d = 1;
                            } else if (this.getErrorStrings().indexOf(tempval) == -1) {
                                return tempval;
                            } else {
                                d = this._parseDouble(tempval);
                            }

                            min = Math.min(min, d);
                        }
                    }
                }
                if (min != this.maxValue) {
                    return min.toString();
                }
            }
            for (var r = 0; r < ranges.length; r++) {
                ////cell range
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cell.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = 0;
                            if (s1.toUpperCase() == (this.trueValueStr)) {
                                d = 1;
                            } else if (s1.toUpperCase() == (this.falseValueStr)) {
                                d = 0
                            } else if (this.getErrorStrings().indexOf(s1) != -1) {
                                return s1;
                            } else {
                                d = this._parseDouble(s1);
                            }

                            min = Math.min(min, d);
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = 0;
                        if (s1.toUpperCase() == (this.trueValueStr)) {
                            d = 1;
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        } else {
                            d = this._parseDouble(s1);
                        }

                        min = Math.min(min, d);
                    }
                }
            }
            if (min != this.maxValue) {
                return min.toString();
            }

            return this._string_empty;
        };
       
        this.computePercentrankInc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var signif = 3;
            var s1 = this.getValueFromArg(args[1]);
            x = this._parseDouble(s1);
            if (isNaN(x)) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._invalid_arguments];
                }
                return this.formulaErrorStrings[this._invalid_arguments];
            } else if (x == 0) {
            }

            if (argCount == 3) {
                s1 = this.getValueFromArg(args[2]);
                signif = this._parseDouble(s1);
                if (isNaN(signif) && signif < 1) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    }
                    return this.formulaErrorStrings[this._invalid_arguments];
                }
            }
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort();

            var d = 1;
            for (var i = 0; i < n; ++i) {
                if (dd[i] == x) {
                    var k = 0;
                    while (k + i < n && dd[k + i] == x) {
                        k++;
                    }

                    d = ((i - 1)) / (i + n - i - k - 1);

                    if (i > 0 && dd[i - 1] < x) {
                        var di = (i) / (n - 1);
                        d = di + (d - di) * (1 - (x - dd[i - 1]) / (dd[i] - dd[i - 1]));
                    }

                    break;
                }
            }
            var fmt = "0." + ('#', parseInt(signif.toString()));
            return d.toString();
        };
        
        this.computeRankOEq = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2 && argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var rank = 0;
            var x;
            var s1 = this.getValueFromArg(args[0]);
            if (args[0] == this._string_empty || args[1] == this._string_empty || !(this._isCellReference(args[1])))
                return this.getErrorStrings()[4].toString();
            x = this._parseDouble(s1);
            if (!isNaN(x)) {
                var order = 0;
                if (argCount == 3) {
                    s1 = this.getValueFromArg(args[2]);
                    if (s1 == this._string_empty)
                        s1 = "0";
                    else if (this._parseDouble(s1) > 1)
                        s1 = "1";
                    order = this._parseDouble(s1);
                }

                var r = args[1];

                var d = 0;
                var eq = false;
                if (r.indexOf(":") == -1)
                    return this.getErrorStrings()[4].toString();
                if (r.indexOf(':') > -1) {
                    var cells1 = this.getCellsFromArgs(r);
                    for (var s = 0; s < cells1.length; s++) {
                        try {
                            s1 = this._getDoubleArray(cells1[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }
                        d = this._parseDouble(s1.toString());
                        if (!isNaN(d)) {
                            if (order == 1) {
                                if (d < x) {
                                    rank += 1;
                                } else if (d == x) {
                                    eq = true;
                                }
                            } else {
                                if (d > x) {
                                    rank += 1;
                                } else if (d == x) {
                                    eq = true;
                                }
                            }
                        }
                    }

                    if (eq) {
                        rank += 1;
                    }
                }
            }
            return rank.toString();
        };
        this.computePercentile = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var k;
            var s1 = this.getValueFromArg(args[1]);
            k = this._parseDouble(s1);
            if (isNaN(k) && (k < 0 || k > 1)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a < b) return 1;
                    else return -1;
                }
                return a - b;
            });

            var h = 1 / (n - 1);
            var d = dd[n - 1];
            for (var i = 0; i < n - 1; ++i) {
                if ((i + 1) * h > k) {
                    k = (k - i * h) / h;
                    d = dd[i] + k * (dd[i + 1] - dd[i]);
                    break;
                }
            }
            return d.toString();
        };
        this.computePoissonODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var u;
            var cum = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            x = this._parseDouble(args[0]);
            u = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(u)) {
                cum = (args[2] == this.trueValueStr) ? 1 : 0;
                var n = x;
                if (cum == 0) {
                    var prod = 1;
                    for (var i = 2; i <= n; ++i) {
                        prod *= i;
                    }
                    dist = Math.exp(-u) * Math.pow(u, n) / prod;
                } else {
                    var prod = 1;
                    dist = 0;
                    var ui = 1;
                    for (var i = 0; i <= n; ++i) {
                        dist += ui / prod;
                        prod *= i + 1;
                        ui *= u;
                    }

                    dist = Math.exp(-u) * dist;
                }
            }
            return dist.toString();
        };
        this.computeWeiBullODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 4) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var alpha;
            var beta;
            var cumulative = 0;
            var cumBool = true;

            for (var i = 0; i < argCount; ++i)
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            x = this._parseDouble(args[0]);
            alpha = this._parseDouble(args[1]);
            beta = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(alpha) && !isNaN(beta)) {
                if (x < 0 || alpha <= 0 || beta <= 0) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed argument value is below 0";
                    return this.getErrorStrings()[4].toString();
                }
                if (alpha.toString().length >= 5 && beta.toString().length >= 3) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw "Passed argument length exceeded the minimum length";
                    return this.getErrorStrings()[4].toString();
                }
                if (argCount != 3) {
                    cumulative = (args[3] == this.trueValueStr) ? 1 : 0;
                    var check = 0;
                    check = this._parseDouble(args[3]);
                    if (!isNaN(check))
                        cumulative = 1;
                }

                cumBool = (cumulative == 1) ? true : false;
                cumulative = ((cumBool) ? 1 - Math.exp(-Math.pow(x / beta, alpha)) : Math.pow(x, alpha - 1) * Math.exp(-Math.pow(x / beta, alpha)) * alpha / Math.pow(beta, alpha));
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw "Passed argument value is non-numerical";
                return this.getErrorStrings()[1].toString();
            }
            return cumulative.toString();
        };
        this.computeFOinvOrt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var df1;
            var df2;
            var invdist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            p = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            df2 = this._parseDouble(args[2]);

            if (!isNaN(p) && (p > 0 && p < 1) && !isNaN(df1) && !isNaN(df2)) {
                invdist = this._finv(p, df1, df2);
            }

            if (invdist <= 0) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._iterations_dont_converge];
                }
                return this.formulaErrorStrings[this._iterations_dont_converge];
            }
            return invdist.toString();
        };
        this.computeTOdist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(df1)) {
                if (df1 < 1 || (x[0] == ("-"))) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }

                if (args[2] == this.trueValueStr) {
                    dist = this._tCumulativeDensity(x, df1);
                    //dist = StudentsTCumulativeDistributionFunctionInverse(x, (int)u);
                } else if (args[2] == this.falseValueStr) {
                    dist = this._tProbabilityDensity(x, df1);
                }
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is incorrect";
                }
                return this.getErrorStrings()[1].toString();
            }

            return dist.toString();
        };
        this.computeAvedev = function (range) {
            var sum = 0, s1, d, ranges;
            var x = [];
            ranges = this.splitArgsPreservingQuotedCommas(range);
            var adjustRange;
            for (var r = 0; r < ranges.length; r++) {
                adjustRange = ranges[r];

                ////is a cellrange
                if (adjustRange.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(adjustRange);
                    for (var s = 0; s < cellCollection.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]).split(this.tic).join(this._string_empty);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }
                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                sum = sum + d;
                                x.push(d);
                            } else if ((this.getErrorStrings().indexOf(s1)) > 0) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(adjustRange).split(this.tic).join(this._string_empty);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }
                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            sum = sum + d;
                            x.push(d);
                        } else if ((this.getErrorStrings().indexOf(s1) == -1)) {
                            return s1;
                        }
                    }
                }
            }

            if (x.length > 0) {
                var ave = sum / x.length;
                sum = 0;
                for (var i = 0; i < x.length; ++i) {
                    sum = sum + Math.abs((x[i]) - ave);
                }

                sum = sum / x.length;
            }
            return sum.toString();
        };
        this.computeTOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join(this._string_empty);
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            if (!isNaN(x) && !isNaN(df1)) {
                if (df1 < 1 || x < 0 || (x[0] == ("-"))) {
                    if (this._rethrowLibraryComputationExceptions) {
                        throw "#NUM! Passed argument value is incorrect";
                    }
                    return this.getErrorStrings()[4].toString();
                }
                dist = this._tCumulativeDistributionInverse(x, df1);
            } else {
                if (this._rethrowLibraryComputationExceptions) {
                    throw "#VALUE! Passed argument value is incorrect";
                }
                return this.getErrorStrings()[1].toString();
            }
            return dist.toString();
        };
       
        this.computeChisqOinv = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var p;
            var v;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }

            p = this._parseDouble(args[0]);
            v = this._parseDouble(args[1]);
            if (!isNaN(p) && !isNaN(v)) {
                dist = this._chiinv(p, v);
                return dist.toString();
            }
        };
        this.computeCount = function (range) {
            var count = 0;
            var s1;
            var d;
            var dt = new Date;

            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r].split(this.tic).join(""));
                    for (var s = 0; s < cell.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (Exception) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getLibraryComputationException();
                            return this.getErrorStrings()[4].toString();
                        }

                        if (s1.length > 0) {
                            if (s1 == (this.formulaErrorStrings[19])) {
                                if (this.geLibraryComputationException() != null)
                                    throw this.getLibraryComputationException();
                                return this.formulaErrorStrings[19];
                            }
                            d = this._parseDouble(s1);
                            dt = new Date(Date.parse(s1));
                            if ((!isNaN(d) || dt.toString() != "Invalid Date") && this.getErrorStrings().indexOf(s1) == -1) {
                                count++;
                            }
                        }
                    }
                } else {
                    try {
                        if (ranges[r] == this._string_empty && !(ranges[r][0] == (this.tic)))
                            count++;
                        s1 = this.getValueFromArg(r.split(this.tic).join(""));
                    } catch (Exception) {
                        if (this.getLibraryComputationException() != null)
                            throw this.getLibraryComputationException();
                        throw this.getErrorStrings()[4].toString();
                    }

                    if (s1.length > 0) {
                        if (s1[0] == (this.formulaErrorStrings[19])) {
                            if (this.getLibraryComputationException() != null)
                                throw this.getLibraryComputationException();
                            return this.formulaErrorStrings[19];
                        }
                        d = this._parseDouble(s1);
                        dt = new Date(Date.parse(s1));
                        if ((!isNaN(d) || dt.toString() != "Invalid Date") && this.getErrorStrings().indexOf(s1) == -1) {
                            count++;
                        }
                    }
                }
            }
            return count.toString();
        };
        this.computeFOdistORt = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x;
            var df1 = 0;
            var df2 = 0;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]).split(this.tic).join("");
            }
            x = this._parseDouble(args[0]);
            df1 = this._parseDouble(args[1]);
            df2 = this._parseDouble(args[2]);
            if (!isNaN(x) && !isNaN(df1) && !isNaN(df2)) {
                var mult = Math.exp(this._gammaln((df1 + df2) / 2) - this._gammaln(df1 / 2) - this._gammaln(df2 / 2) + (df1 / 2) * Math.log(df1 / df2));

                dist = 1 - mult * this._fdist(x, parseInt(df1.toString()), parseInt(df2.toString()));
            } else {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_Math_argument];
                return this._formulaErrorStrings[this._invalid_Math_argument];
            }
            return dist.toString();
        };
        this.computeGeomean = function (range) {
            var sum = 1;
            var s1;
            var d;
            var count = 0;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cell.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d)) {
                                count++;
                                sum = sum * d;
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d)) {
                            count++;
                            sum = sum * d;
                        } else if (this.getErrorStrings().indexOf(s1) == -1) {
                            return s1;
                        }
                    }
                }
            }

            if (count > 0) {
                sum = Math.pow(sum, 1 / count);
            }
            return sum.toString();
        };
        this.computeHarmean = function (range) {
            var sum = 0;
            var s1;
            var d;
            var count = 0;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1) {
                    var cell = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cell.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cell[s]);
                        } catch (ex) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return ex;
                        }

                        if (s1.length > 0) {
                            d = this._parseDouble(s1);
                            if (!isNaN(d) && d != 0) {
                                count++;
                                sum = sum + 1 / d;
                            } else if (this.getErrorStrings().indexOf(s1) == -1) {
                                return s1;
                            }
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        }

                        return ex;
                    }

                    if (s1.length > 0) {
                        d = this._parseDouble(s1);
                        if (!isNaN(d) && d != 0) {
                            count++;
                            sum = sum + 1 / d;
                        } else if (this.getErrorStrings().indexOf(s1)) {
                            return s1;
                        }
                    }
                }
            }

            if (count > 0) {
                sum = (count) / sum;
            }
            return sum.toString();
        };
        this.computeIntercept = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var y = this._getDoubleArray(args[0]);
            var x = this._getDoubleArray(args[1]);
            var n = x.length;

            if (n <= 0 || n != y.length) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx += x[i];
                sumy += y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumx2 = 0;
            var d;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumxy += d * (y[i] - sumy);
                sumx2 += d * d;
            }
            return (sumy - sumxy / sumx2 * sumx).toString();
        };
        this.computeCountblank = function (range) {
            var count = 0;
            var s1;

            this.adjustRangeArg(range);
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var arglist;
            if (this._isTextEmpty(range)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this.invalid_arguments];
                else
                    return this.formulaErrorStrings[this.invalid_arguments];
            }
            for (var r = 0; r < ranges.length; r++) {
                arglist = ranges[r];

                ////is a cellrange
                if (arglist.indexOf(':') > -1) {
                    var cellCollection = this.getCellsFromArgs(arglist);
                    for (var s = 0; s < cellCollection.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cellCollection[s]);
                        } catch (Exception) {
                            if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                                throw this.getLibraryComputationException();
                            }

                            return this.getErrorStrings()[4].toString();
                        }

                        if (s1 == this._string_empty) {
                            count++;
                        }
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges);
                    } catch (Exception) {
                        if (this._rethrowLibraryComputationExceptions && this.getLibraryComputationException() != null) {
                            throw this.getLibraryComputationException();
                        } else
                            return this.getErrorStrings()[4].toString();
                    }
                    if (!this._isCellReference(arglist)) {
                        var temp = this._parseDouble(s1);

                        if (!isNaN(temp) || this.getErrorStrings.indexOf(s1)) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.formulaErrorStrings[this.invalid_arguments].toString();
                            else
                                return this.formulaErrorStrings[this.invalid_arguments].toString();
                        }
                        if (!(s1.split(this.tic).join(this._string_empty) == (this._string_empty))) {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.getErrorStrings()[5].toString();
                            else
                                return this.getErrorStrings()[5].toString();
                        } else {
                            if (this._rethrowLibraryComputationExceptions)
                                throw this.formulaErrorStrings[this.invalid_arguments].toString();
                            else
                                return this.formulaErrorStrings[this.invalid_arguments].toString();
                        }
                    }
                    if (s1 == this._string_empty) {
                        count++;
                    }
                }
            }
            return count.toString();
        };
        this.computeDevsq = function (range) {
            var d;
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            var x = this._getDoubleArray(range);
            var n = x.length;
            if (ranges.length != 1 || range == "") {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (x.length <= 0)
            {
                if (this.getLibraryComputationException() != null)
                    throw this.formulaErrorStrings[this._bad_formula];
                throw this.getErrorStrings()[4].toString();
            }
            var sumx = 0;
            for (var i = 0; i < n; ++i) {
                sumx = sumx + x[i];
            }

            sumx = sumx / n;
            var sumx2 = 0;

            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumx2 = sumx2 + d * d;
            }
            return sumx2.toString();
        };
        this.computeForecast = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 3) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var x0, d;
            var s1 = this.getValueFromArg(args[0]);
            x0 = this._parseDouble(s1);
            if (isNaN(x0)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var y, x;
            y = this._getDoubleArray(args[1]);
            x = this._getDoubleArray(args[2]);
            var n = x.length;

            if (n <= 0 || n != y.length) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx = sumx + x[i];
                sumy = sumy + y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumx2 = 0;
            for (var i = 0; i < n; ++i) {
                d = x[i] - sumx;
                sumxy = sumxy + d * (y[i] - sumy);
                sumx2 = sumx2 + d * d;
            }

            var b = sumxy / sumx2;
            var a = sumy - b * sumx;
            return (a + b * x0).toString();
        };
        
        this.computeStdevOp = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;

            if (n < 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._invalid_arguments];
                }
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            return this._stdevdotP(dd).toString();
        };
        this.computeStdevOS = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;

            if (n < 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._invalid_arguments];
                }
                return this.formulaErrorStrings[this.invalid_arguments];
            }
            var xbar = 0;
            return this._sd(dd, xbar).toString();
        };
        this.computeStdeva = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;
            if (n < 2) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var xbar = 0;
            return this._sd(dd, xbar).toString();
        };
        this.computeStdevpa = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;
            if (n < 2) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var xbar = 0;
            var sdev = this._sd(dd, xbar);
            return (sdev * Math.sqrt(n - 1) / Math.sqrt(n)).toString();
        };
        this.computeVarp = function (range) {
            var dd = this._getDoubleArray(range);
            var n = dd.length;
            return ((n - 1) * this._var(dd) / n).toString();
        };
        this.computeVara = function (range) {
            var dd = this._getDoubleArrayA(range);
            return this._var(dd).toString();
        };
        this.computeVarpa = function (range) {
            var dd = this._getDoubleArrayA(range);
            var n = dd.length;
            return ((n - 1) * this._var(dd) / n).toString();
        };
        this.computeCorrel = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var y, x;
            y = this._getDoubleArray(args[0]);
            x = this._getDoubleArray(args[1]);
            var n = x.length;
            if (n <= 0 || n != y.length) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var sumx = 0;
            var sumy = 0;
            for (var i = 0; i < n; ++i) {
                sumx = sumx + x[i];
                sumy = sumy + y[i];
            }

            sumx = sumx / n;
            sumy = sumy / n;

            var sumxy = 0;
            var sumxb2 = 0;
            var sumyb2 = 0;
            var xb = 0;
            var yb = 0;

            for (var i = 0; i < n; ++i) {
                xb = x[i] - sumx;
                yb = y[i] - sumy;
                sumxy = sumxy + xb * yb;
                sumxb2 = sumxb2 + xb * xb;
                sumyb2 = sumyb2 + yb * yb;
            }

            //  var v = (sumxy / Math.Sqrt(sumxb2 * sumyb2)).toString();
            //        return v;
            return (sumxy / Math.sqrt(sumxb2 * sumyb2)).toString();
        };

        this.computeNegbinomODist = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 4) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var successes;
            var failures;
            var p;
            var dist = 0;

            for (var i = 0; i < argCount; ++i) {
                args[i] = this.getValueFromArg(args[i]);
            }
            failures = this._parseDouble(args[0]);
            successes = this._parseDouble(args[1]);
            p = this._parseDouble(args[2]);
            if (!isNaN(failures) && !isNaN(successes) && !isNaN(p)) {
                dist = this._negbinomdensity(parseInt(failures.toString()), parseInt(successes.toString()), p);
            }
            return dist.toString();
        };

        this.computePercentileExc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var k;
            var s1 = this.getValueFromArg(args[1]);
            k = this._parseDouble(s1);
            if (isNaN(k) && (k < 0 || k > 1)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var ss = this._parseDouble(s1);
            if (ss == 0 || ss <= 0 || ss >= 1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a < b) return 1;
                    else return -1;
                }
                return a - b;
            });

            var num = dd.length;
            if (k <= this._parseDouble((1 / (num + 1)).toString()) || k >= this._parseDouble((num / (num + 1)).toString())) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var l = k * (num + 1);
            return l.toString();
        };

        this.computePercentileOInc = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                }
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var k;
            var s1 = this.getValueFromArg(args[1]);
            k = this._parseDouble(s1);
            if (isNaN(k) && (k < 0 || k > 1)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            var ss = this._parseDouble(s1);
            if (ss == 0 || ss <= 0 || ss >= 1) {
                if (this._rethrowLibraryComputationExceptions) {
                    throw this.getErrorStrings()[4].toString();
                }
                return this.getErrorStrings()[4].toString();
            }
            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;

            dd.sort();

            var h = 1 / (n - 1);
            var d = dd[n - 1];
            for (var i = 0; i < n - 1; ++i) {
                if ((i + 1) * h > k) {
                    k = (k - i * h) / h;
                    d = dd[i] + k * (dd[i + 1] - dd[i]);
                    break;
                }
            }
            return d.toString();
        };

        this.computeTrimmean = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var percent;
            var s1 = this.getValueFromArg(args[1]);
            percent = this._parseDouble(s1);
            if (isNaN(percent)) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }

            var dd = this._getDoubleArray(args[0]);
            var n = dd.length;
            var k = parseInt((percent * n).toString());
            k = parseInt(k / 2);
            if (k < 1 || 2 * k >= n) {
                return this.formulaErrorStrings[this._invalid_arguments];
            }
            dd.sort(function (a, b) {
                if (isNaN(a) || isNaN(b)) {
                    if (a > b) return 1;
                    else return -1;
                }
                return a - b;
            });
            var sum = 0;
            n = n - k;
            for (var i = k; i < n; ++i) {
                sum += dd[i];
            }
            return (sum / (n - k)).toString();
        };

        this.computePearson = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length;
            if (argCount != 2) {
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            var y = this._getDoubleArray(args[0]);
            var x = this._getDoubleArray(args[1]);
            var n = y.length;
            if (n <= 0 || !isNaN(y)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[4].toString();
            }
            var result = this._pearson(x, y, n).toString();

            if ( this.computeIsErr(result) == this.trueValueStr) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[3].toString();
            }
            return result;
        };

        this.computeRsq = function (range) {

            var s = this.computePearson(range).toString();
                var d = 0;
                d = this._parseDouble(s);
                if (!isNaN(d)) {
                    d = d * d;
                } else if (this.getErrorStrings().indexOf(s) == -1) {
                    return s;
                }
                return d.toString();
            try {
            }
            catch (ex) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_Math_argument];
                return this.getErrorStrings()[5].toString();
            }
        };

        this.computeHLookUp = function (range) {
            var cachingEnabled = this._isHLookupCachingEnabled();
            var optimizedMatchesEnabled = this._isOptimizedMatchesEnabled();
            if (cachingEnabled && this._lookupTables == null)
                this.lookupTables = new HashTable();

            var s = this.splitArgsPreservingQuotedCommas(range);
            var lookUp = this.getValueFromArg(s[0]);
            lookUp = lookUp.split(this.tic).join("").toUpperCase();
            var r = s[1].split("\"").join("");
            var o1 = this.getValueFromArg(s[2]).split(this.tic).join("");
            var d = this._parseDouble(o1);
            if (isNaN(d)) {
                return "#N/A";
            }

            ////int row = int.Parse(o1);
            var row = parseInt(o1);

            var match = true;
            if (s.length == 4) {
                match = this.getValueFromArg(s[3]) == this.trueValueStr;
            }

            d = this._parseDouble(lookUp);
            var typeIsNumber = match ? !isNaN(d) : false;

            var i = r.indexOf(":");

            ////single cell
            if (i == -1) {
                r = r + ":" + r;
                i = r.indexOf(":");
            }

            var k = r.substring(0, i).lastIndexOf(this.sheetToken);
            var grd = this.grid;
            var family = CalcEngine.getSheetFamilyItem(this.grid);

            if (k > -1) {
                this.grid = family.tokenToParentObject.getItem(r.substring(0, k + 1));
            }

            var row1 = this.rowIndex(r.substring(0, i));
            var row2 = this.rowIndex(r.substring(i + 1));
            if (!(row1 != -1 || row2 == -1) == (row1 == -1 || row2 != -1)) {
                return this.getErrorStrings()[5].toString();
            }

            var col1 = this.colIndex(r.substring(0, i));
            var col2 = this.colIndex(r.substring(i + 1));

            var newTable = false;
            var key = "";
            var lookUpTable = null;
            var matchedTable = null;
            if (cachingEnabled) {
                key = row1.toString() + "_" + col1.toString() + "_" + col2.toString() + "_" + this.grid.GetHashCode();

                if (!this._lookupTables.containsKey(key)) {
                    if (optimizedMatchesEnabled) {
                        var _lookUp = new LookUps();
                        _lookUp.setMatchLookUpList(new HashTable());
                        this._lookupTables.add(key, _lookUp);
                    } else {
                        var _lookUp = new LookUps();
                        this._lookupTables.add(key, _lookUp);
                    }
                    newTable = true;
                }
                lookUpTable = this._lookupTables.getItem(key).getLinearLookUpList();
                if (optimizedMatchesEnabled) {
                    matchedTable = this._lookupTables.getItem(key).getMatchLookUpList();
                }
            }

            var val = "";
            var lastCol = col1;
            var s1 = "";
            var d1 = 0;
            var doLastColMark = true;
            var exactMatch = false;
            if (cachingEnabled && optimizedMatchesEnabled && matchedTable.containsKey(lookUp)) {
                lastCol = matchedTable.getItem(lookUp);
                s1 = lookUp;
            } else {
                for (var col = col1; col <= col2; ++col) {
                    if (!cachingEnabled || col - col1 >= lookUpTable.length || newTable) {
                        s1 = this.getValueFromParentObject(this.grid, row1, col).toString().toUpperCase().split("\"").join("");
                        if (cachingEnabled) {
                            lookUpTable.push(s1);
                            if (optimizedMatchesEnabled) {
                                matchedTable.add(s1, col);
                            }
                        }
                    } else {
                        s1 = lookUpTable[col - col1];
                    }
                    d1 = (typeIsNumber) ? this._parseDouble(s1) : d1;
                    if (s1 == lookUp || (match && (typeIsNumber ? (!isNaN(d1) && d1 > d) : (s1 > lookUp)))) {
                        if (s1 == lookUp) {
                            lastCol = col;
                            match = true;
                            exactMatch = true;
                        }

                        if (!newTable)
                            break;
                        else
                            doLastColMark = false;
                    }
                    if (doLastColMark)
                        lastCol = col;
                    match = newTable = true;
                }
            }

            if (match || s1 == lookUp) {
                if (!exactMatch && !typeIsNumber) {
                    this.grid = grd;
                    return "#N/A";
                }
                if ((row + row1 - 1) > row2)
                    return "#REF!";
                val = this.getValueFromParentObject(this.grid, row + row1 - 1, lastCol).toString();
                if (val.length > 0 && val[0] == this.getFormulaCharacter()) {
                    val = this.parseFormula(val);
                }

                d = this._parseDouble(val);
                if (val.length > 0 && val[0] != this.tic[0] && isNaN(d)) {
                    val =   val ;
                }
            } else {
                val = "#N/A";
            }

            ////val = "\"#N/A\"";
            this.grid = grd;

            return val;
        };
        this.computeIndex = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;

            if (argCount < 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var r = args[0];
            r = r.split(this.tic).join("");
            var i = r.indexOf(":");

            ////single cell
            if (i == -1) {
                if (this._isCellReference(r)) {
                    r = r + ":" + r;
                } else {
                    if (r.indexOf(";") > -1)
                    {
                        var rowNumber = parseInt(this.getValueFromArg(args[1]));
                        var arr = this._splitArguments(r, ';');
                        var colInd = (argCount == 3) ? parseInt(this.getValueFromArg(args[2])) : 1;
                        try
                        {
                            rowNumber = (rowNumber == 0) ? 1 : rowNumber;
                            colInd = (colInd == 0) ? 1 : colInd;
                            var colArray = this.splitArgsPreservingQuotedCommas(arr[rowNumber -1]);
                            return colArray[colInd -1];
                        }
                        catch(ex)
                        {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.formulaErrorStrings[this._invalid_Math_argument];
                            return this.getErrorStrings()[2].toString();
                        }
                    }
                    var indexArray = this.splitArgsPreservingQuotedCommas(r);
                    if (!isNaN(rowNumber) && rowNumber > 0) {
                        if (indexArray.length >= rowNumber) {
                            return (parseInt(rowNumber.toString()) == 0) ? indexArray[0] : indexArray[parseInt(rowNumber.toString()) - 1];
                        } else {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.formulaErrorStrings[this._invalid_arguments];
                            return "#REF";
                        }
                    } else {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._invalid_arguments];
                        return this.getErrorStrings()[1].toString();
                    }
                }
            }
            i = r.indexOf(":");
            if (arg.indexOf("#N/A") > -1 || arg.indexOf("#N~A") > -1) {
                return "#N/A";
            }
            if (arg.indexOf("#DIV/0!") > -1 || arg.indexOf("#DIV~0!") > -1) {
                return "#DIV/0!";
            }
            var sheet = this._getSheetTokenFromReference(r);
            args[1] = (argCount == 1 || args[1] == "") ? "1" : args[1];
            args[2] = (argCount <= 2 || args[2] == "") ? "1" : args[2];
            var d = parseInt(this.getValueFromArg(args[1]));
            var row = !isNaN(d) ? d: -1;
            d = parseInt(this.getValueFromArg(args[2]));
            var col = !isNaN(d) ? d : -1;
            if (row == -1 || col == -1) {
                return "#REF";
            }

            var top = this.rowIndex(r.substring(0, i));
            var bot = this.rowIndex(r.substring(i + 1));
            if (!(top != -1 || bot == -1) == (top == -1 || bot != -1)) {
                return this.getErrorStrings()[5].toString();
            }

            var left = this.colIndex(r.substring(0, i));
            var right = this.colIndex(r.substring(i + 1));

            if (row > bot - top + 1 || col > right - left + 1) {
                return "#REF";
            }

            row = this.rowIndex(r.substring(0, i)) + row - 1;

            col = this.colIndex(r.substring(0, i)) + col - 1;

            var argsVal = sheet + RangeInfo.getAlphaLabel(col) + row;
            var result = this.getValueFromArg(argsVal);
            if (this._isTextEmpty(result))
                return "0";
            return result;
        };
        this.computeIndirect = function (args) {
            var arg = this.splitArgsPreservingQuotedCommas(args);
            var argCount = arg.length;

            if (argCount > 2 || argCount == 0) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }

            arg[0] = arg[0].toUpperCase();
            arg[0] = this.setTokensForSheets(arg[0]);
            var sheetToken1 = this._sheetToken(arg[0].split(this.tic).join(""));

            ////Remove SheetToken
            if (sheetToken1 != "") {
                arg[0] = arg[0].split(sheetToken1).join("");
            }

            if (arg.length == 2 && arg[1] == this.falseValueStr) {
                var hasTIC = arg[0][0] == this.tic && arg[0][args[0].length - 1] == this.tic;
                var rcCell = (arg[0].toUpperCase()).split(this.tic).join("");
                var cells = rcCell.split(':');
                
                var rc = (cells[0].split("R").join("C")).split("C");
                var index = rc.indexOf("");
                if (index > -1)
                    rc.splice(index, 1);
                if (rc.length > 2) {
                    return this.getErrorStrings()[2].toString();
                }

                arg[0] = RangeInfo.getAlphaLabel(parseInt(rc[1])) + rc[0];

                if (cells.length == 2) {
                    var st = this._sheetToken(cells[1]);
                    if (st != null || st != "") {
                        cells[1] = cells[1].split(st).join("");
                    }

                    rc = (cells[0].split("R").join("C")).split("C");
                    arg[0] += ":" + RangeInfo.getAlphaLabel(parseInt(rc[1])) + rc[0];
                }

                if (hasTIC) {
                    arg[0] = this.tic + arg[0] + this.tic;
                }
            }

            var cellReference = "";

            ////String
            if (arg[0][0] == this.tic) {
                cellReference = sheetToken1 + arg[0].split(this.tic).join("");
            } else {
                if (this._isCellReference(arg[0])) {
                    var scopedRange = "";
                    cellReference = this.getValueFromArg(sheetToken1 + arg[0]);
                    if (cellReference == null || cellReference == "") {
                        return "0";
                    }
                    if (this._isInteriorFunction) {
                        this._isInteriorFunction = !this._isInteriorFunction;
                        return arg[0].split(this._string_fixedreference).join("");
                    }

                    scopedRange = this._checkIfScopedRange(cellReference.toUpperCase());
                    if (this.getNamedRanges().containsKey(cellReference.toUpperCase())) {
                        cellReference = this.getNamedRanges().getItem(cellReference.toUpperCase());
                        cellReference = cellReference.toUpperCase();
                        cellReference = cellReference.split(this._string_fixedreference).join("");
                        cellReference = this.setTokensForSheets(cellReference);
                        if (cellReference[0] != this.sheetToken) {
                            cellReference = sheetToken1 + cellReference;
                        }
                    } else if (scopedRange != "NaN") {
                        cellReference = scopedRange;
                        cellReference = cellReference.toUpperCase();
                        cellReference = cellReference.split(this._string_fixedreference).join("");
                        cellReference = this.setTokensForSheets(cellReference);
                        if (cellReference[0] != this.sheetToken) {
                            cellReference = sheetToken1 + cellReference;
                        }
                    } else if (this._isCellReference(cellReference)) {
                        return this.getValueFromArg(cellReference);
                    } else if (cellReference.indexOf("Invalid" == 0))
                        return this.getErrorStrings()[2].toString();
                    else {
                        return cellReference;
                    }
                } else {
                    cellReference = arg[0];
                }
            }

            if (!this._isCellReference(cellReference.split(this._string_fixedreference).join(""))) {
                var scopedRange = "";
                scopedRange = this._checkIfScopedRange(cellReference.toUpperCase());
                if (this.getNamedRanges().containsKey(cellReference.toUpperCase())) {
                    cellReference = this.getNamedRanges().getItem(cellReference.toUpperCase());
                    if (cellReference[0] != this.sheetToken) {
                        cellReference = sheetToken1 + cellReference;
                    }
                } else if (scopedRange != "NaN") {
                    cellReference = scopedRange;
                    if (cellReference[0] != this.sheetToken) {
                        cellReference = sheetToken1 + cellReference;
                    }
                } else {
                    return this.getErrorStrings()[2].toString();
                }
            }

            if (cellReference.indexOf(":") > -1 && this._isInteriorFunction) {
                this._isInteriorFunction = !this._isInteriorFunction;

                return cellReference.split(this._string_fixedreference).join("");
            }
            return this.getValueFromArg(cellReference);
        };
        this.computeLookUp = function (range) {
            var cachingEnabled = this._isHLookupCachingEnabled();
            var optimizedMatchesEnabled = this._isOptimizedMatchesEnabled();
            if (cachingEnabled && this._lookupTables == null)
                this.lookupTables = new HashTable();

            var s = this.splitArgsPreservingQuotedCommas(range);
            var lookUp = this.getValueFromArg(s[0]);
            lookUp = lookUp.split(this.tic).join("").toUpperCase();

            var finalResult = "";
            var r = s[1].split("\"").join("");
            s[2] = (s.length = 3) ? s[2].split(this.tic).join("") : s[2];
            var tempLookUp = this.getValueFromArg(s[0]);
            var cellValue =  [];
            var resultValue = [];

            var o1 = this.getValueFromArg(s[2]).split(this.tic).join("");
            //var d = this._parseDouble(o1);
            //if (isNaN(d)) {
            //    return "#N/A";
            //}
            if (s.length == 2) {
                var secondOne = s[1];
                var stringArg = this.splitArgsPreservingQuotedCommas(s[1]);
                if (s[1].indexOf(";") > -1) {
                    var sepArg = this._splitArguments(s[1], ';');
                    if (sepArg.length == 1) {
                        var string1 = this.splitArgsPreservingQuotedCommas(sepArg[0].toUpperCase());
                        for (var cell = 0; cell < string1.length; cell++)  {
                            cellValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                            resultValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                        }
                    } else if (sepArg.length == 2) {
                        var strin1 = this.splitArgsPreservingQuotedCommas(sepArg[0].toUpperCase());
                        var strin2 = this.splitArgsPreservingQuotedCommas(sepArg[1].toUpperCase());
                        for (var cell = 0; cell < strin1.length; cell++) {
                            cellValue.push(this.getValueFromArg(strin1[cell]).toUpperCase());
                        }
                        for (var cell = 0; cell < strin2.length; cell++)  {
                            resultValue.push(this.getValueFromArg(strin2[cell]).toUpperCase());
                        }
                    } else
                        for (var y = 0; y < sepArg.length; y++) {
                            var strin1 = this.splitArgsPreservingQuotedCommas(sepArg[y].toUpperCase());
                            cellValue.push(this.getValueFromArg(strin1[0]).toUpperCase());
                            resultValue.push(this.getValueFromArg(strin1[1]).toUpperCase());
                        }
                } else {
                    if (s[1].indexOf(':') > -1) {
                        var cells = this.getCellsFromArgs(s[1]);
                        for (var cell1 = 0; cell1 < cells.length; cell1++)  {
                            cellValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                            resultValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                        }
                    } else {
                        var string1 = this.splitArgsPreservingQuotedCommas(s[1].toUpperCase());
                        for (var cell = 0; cell < string1.length; cell++)  {
                            cellValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                            resultValue.push(this.getValueFromArg(string1[cell]).toUpperCase());
                        }
                    }
                }
            } else if (s.length == 3) {
                if (r.indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(r);

                    for (var cell1 = 0; cell1 < cells.length; cell1++)  {
                        try  {
                            cellValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                } else {
                    var s2 = this.splitArgsPreservingQuotedCommas(s[1]);
                    for (var cell1 = 0; cell1 < s2.length; cell1++) {
                        try  {
                            cellValue.push(this.getValueFromArg(s2[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                }
                var result = s[2];
                if (result.indexOf(':') > -1) {
                    var cells = this.getCellsFromArgs(result);

                    for (var cell1 = 0; cell1 < cells.length; cell1++)  {
                        try  {
                            resultValue.push(this.getValueFromArg(cells[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                } else {
                    var s2 = this.splitArgsPreservingQuotedCommas(s[1]);
                    for (var cell1 = 0; cell1 < s2.length; cell1++) {
                        try  {
                            resultValue.push(this.getValueFromArg(s2[cell1]).toUpperCase());
                        } catch (ex) {
                            return this.getErrorStrings()[0].toString();
                        }
                    }
                }
            }
            if (cellValue.indexOf(lookUp) == -1) {
                var temp = cellValue[0];
                var valCount = 0;
                for (var x = 0; x < cellValue.length; x++) {
                    if ((cellValue[x] > temp && lookUp > cellValue[x] && temp > lookUp) || lookUp > cellValue[x]) {
                        temp = (lookUp > cellValue[x]) ? cellValue[x] : cellValue[x+1];
                        valCount++;
                    } else if (valCount != 0)
                        break;
                }
                if (valCount == 0) {
                    return this.getErrorStrings()[0].toString();
                }
                lookUp = temp;
            }
            var no = 0;
            var index = 0;
            if (s.length == 3 && s[2].indexOf(':') > -1) {
                for (var y = 0; y < cellValue.length; y++) {
                    if (cellValue[y] == lookUp) {
                        index = cellValue.indexOf(lookUp);
                        no++;
                    } else if (no != 0)
                        break;
                }

                finalResult = resultValue[index + no - 1];
            } else {
                for (var y = 0; y < cellValue.length; y++) {
                    if (cellValue[y] == lookUp) {
                        index = cellValue.indexOf(lookUp);
                        no++;
                    } else if (no != 0)
                        break;
                }
                finalResult = resultValue[index + no - 1];
            }
            return finalResult;
        };
        this.computeOffSet = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;

            if (argCount < 3 || argCount > 5) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var r = args[0];
            var result;
            var d = this._parseDouble(this.getValueFromArg(args[1]));
            var rows = (!isNaN(d)) ? parseInt(d.toString()) : -1;
            d = this._parseDouble(this.getValueFromArg(args[2]));

            var cols = (!isNaN(d)) ? parseInt(d.toString()) : -1;
            var hightNum = (argCount >= 4) ? args[3] : "-1";
            d = parseInt(this.getValueFromArg(hightNum));
            var height = (!isNaN(d) ? (d) : 1);
            var widthNum = (argCount == 5) ? args[4] : "-1";
            d = parseInt(this.getValueFromArg(widthNum));
            var width = (!isNaN(d) ? (d) : 1);

            var i = r.indexOf(":");
            var singleCell = i == -1;

            ////single cell
            if (singleCell) {
                r = r + ":" + r;
                i = r.indexOf(":");
            }

            singleCell = (singleCell && width <= 1 && height <= 1); ////only treat as single cell if no hieght or width given...
            if (width == -1)
                singleCell = true;
            var sheet = this._getSheetTokenFromReference(r);

            var row1 = this.rowIndex(r.substring(0, i)) + rows;
            var row2 = this.rowIndex(r.substring(i + 1)) + rows;
            if (!(this.rowIndex(r.substring(0, i)) != -1 || this.rowIndex(r.substring(i + 1)) != -1) == ((this.rowIndex(r.substring(0, i)) == -1 || this.rowIndex(r.substring(i + 1)) != -1))) {
                return this.getErrorStrings()[5].toString();
            }

            var col1 = this.colIndex(r.substring(0, i)) + cols;
            var col2 = this.colIndex(r.substring(i + 1)) + cols;

            if (row1 <= 0 || col1 <= 0) {
                return this.getErrorStrings()[2].toString();
            }
            if ((height > 0 && width < 0) || (height < 1 && width > 1)) {
                row1 = this.rowIndex(this.cell);
            }
            if (col2 == (parseInt(this.computeColumn(r)) - 1))
                singleCell = true;
            var cellArg = sheet + RangeInfo.getAlphaLabel(col1) + row1;
            result = singleCell ? this.computedValue(cellArg) : cellArg + ":" + RangeInfo.getAlphaLabel(col1 + width - 1) + (row1 + height - 1);
            return result;
        };
        this.computeTranspose = function (arg) {
            if (!this._isCellReference(arg) && !this.getNamedRanges().containsKey(arg) && arg.indexOf(";") == -1) {
                var arrayArg = this._splitArguments(arg.split(this.tic).join(""), ';');
                if (arrayArg.length == 1) {
                    return this.getValueFromArg(this.splitArgsPreservingQuotedCommas(arrayArg[0])[0]);
                } else {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._invalid_arguments];
                    return this.formulaErrorStrings[this._invalid_arguments];
                }
            }
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;

            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstRow, LastRow, firstCol, LastCol;
            var result = "";
            if (arg.indexOf(';') > -1) {
                var arrayArg = this._splitArguments(arg.split(this.tic).join(""), ';');
                for (var i = 1; i < arrayArg.length; i++) {
                    if (this.splitArgsPreservingQuotedCommas(arrayArg[i - 1]).length != this.splitArgsPreservingQuotedCommas(arrayArg[1]).length) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.formulaErrorStrings[this._wrong_number_arguments];
                        return this.formulaErrorStrings[this._wrong_number_arguments];
                    }
                }
                result = this.splitArgsPreservingQuotedCommas(arrayArg[0])[0];
            } else {
                var cells = this.getCellsFromArgs(arg);
                firstRow = this.rowIndex(cells[0].toString());
                LastRow = this.rowIndex(cells[cells.length - 1].toString());
                firstCol = this.colIndex(cells[0].toString());
                LastCol = this.colIndex(cells[cells.length - 1].toString());
                var currentCell = this.cell;
                var currentColIndex = this.colIndex(currentCell);
                var currentRowIndex = this.rowIndex(currentCell);
                var arrayHeight = LastRow - firstRow + 1;
                var arrayWidth = LastCol - firstCol + 1;
                var startRow, startCol, endRow, endCol;

                this._getFormulaArrayBounds(this.cell, arrayHeight, arrayWidth);

                startRow = this._getFormulaArrayBoundsfirstRowIndex;
                startCol = this._getFormulaArrayBoundsfirstColIndex;
                endRow = this._getFormulaArrayBoundslastRowIndex;
                endCol = this._getFormulaArrayBoundslastColIndex;

                var x = currentRowIndex - startRow;
                var y = currentColIndex - startCol;
                var i1 = arg.indexOf(":");

                ////single cell
                if (i1 == -1) {
                    arg = arg + ":" + arg;
                    i1 = arg.indexOf(":");
                }

                var k = arg.substring(0, i1).lastIndexOf(this.sheetToken);
                var grd = this.grid;
                var family = CalcEngine.getSheetFamilyItem(this.grid);

                if (k > -1) {
                    this.grid = family.tokenToParentObject[arg.substring(0, k + 1)];
                }
                result = this._getValueComputeFormulaIfNecessary(firstRow + y, firstCol + x, this.grid);
                this.grid = grd;
            }
            if (result == null || result == "")
                result = "0";
            return result;
        };
        this.computeEncodeURL = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;

            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (!this._isCellReference(args[0]) && !this.getNamedRanges().containsKey(args[0]) && args[0].indexOf(";") == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._invalid_arguments];
                return this.getErrorStrings()[5];
            }
            var text = this._stripTics0(this.getValueFromArg(args[0]));
            return encodeURIComponent(text);
        };
        

        this.computeAnd = function (range) {
            var sum = true, s1, d;
            if (range == null || range == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                else
                    return this.formulaErrorStrings[this._bad_formula];
            }
            var ranges = this.splitArgsPreservingQuotedCommas(range);
            for (var r = 0; r < ranges.length; r++) {
                if (ranges[r] == (this.tic)) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.getErrorStrings()[1].toString();
                    else
                        return this.getErrorStrings()[1].toString();
                }
                if (ranges[r].indexOf(':') > -1) {
                    cells = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cells.length; s++) {
                        if (this.getErrorStrings().indexOf(s) > -1) {
                            return s;
                        } else if (s.startsWith(this.tic)) {
                            if (this.getRethrowLibraryComputationExceptions)
                                throw new this.getErrorStrings()[5].toString();
                            else
                                return this.getErrorStrings()[5].toString();
                        }
                        try {
                            s1 = this.getValueFromArg(s);
                            if (this.getErrorStrings().indexOf(s1) > -1) {
                                return s1;
                            }
                        } catch (ex) {
                            if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                                throw this.getErrorStrings()[4].toString();
                            }
                            return this.getErrorStrings()[4].toString();
                        }

                        sum = (sum & (s1 == "" ? true : ((s1.split(this.tic).join("").toUpperCase() == this.trueValueStr) || (d = this._parseDouble(s1)) && d != 0)));
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                        var tempdate = Date.parse(s1.split(this.tic).join(""));
                        d = this._parseDouble(s1.split(this.tic).join(""))
                        if (!isNaN(tempdate)) {
                            return this.trueValueStr;
                        } else if (isNaN(d) && !(s1 == this._string_empty) && !(s1.split(this.tic).join("").toUpperCase() == this.trueValueStr || s1.split(this.tic).join("").toUpperCase() == this.falseValueStr)) {
                            if (this.getRethrowLibraryComputationExceptions())
                                throw this.getErrorStrings()[1].toString();
                            else
                                return (this._isCellReference(ranges[r]) || s1[0] == this.tic) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
                        }
                    } catch (Exception) {
                        if (this.getRethrowLibraryComputationExceptions() && this.getLibraryComputationException() != null) {
                            throw this.getErrorStrings()[4].toString();
                        }

                        return this.getErrorStrings()[4].toString();
                    }
                    var dp = this._parseDouble(s1);
                    sum = (sum & ((s1.split(this.tic).join("").toUpperCase() == this.trueValueStr) || (!isNaN(dp) && d != 0)));
                    // sum &= (s1 == this.trueValueStr) || (d = this._parseDouble(s1)) && d != 0;
                }
            }
            return sum ? this.trueValueStr : this.falseValueStr;
        };
        this.computeFalse = function (empty) {
            return this.falseValueStr;
        };
        this.computeIf = function (args) {
            var s1 = this._string_empty;
            var array1 = [this.getParseArgumentSeparator(), ':'];

            ////parsed formula
            if (args.length > 0 && this._indexOfAny(args, array1) == -1) {
                return this.getFormulaErrorStrings();
            } else {
                var s = this.getCellsFromArgs(args);

                if (s.length >= 2) {
                    try {
                        s1 = this.getValueFromArg(s[0]);
                        var d;
                        if (s1.split(this.tic).join("") == this.trueValueStr || (d = this._parseDouble(s1)) && d != 0) {
                            s1 = this.getValueFromArg(s[1]);
                        } else if (s1 == this.falseValueStr || (s1 == "") || (d = this._parseDouble(s1)) && d == 0) {
                            s1 = s.length == 3 ? this.getValueFromArg(s[2]) : false;
                        } else {
                            //  if (this.tic().indexOf(s[0]) == -1)
                            if (s.indexOf(this.tic) > -1) {
                                var first = s[0].indexOf(this.tic) + 1;
                                var last = s[0].lastIndexOf(this.tic) - 1;
                                var st = s[0].substring(first, last - first + 1);
                                for (var err = 0; err < this.getFormulaErrorStrings().length; err++) {
                                    if (st === (err))
                                        return err;
                                }
                            }
                            s1 = "NaN";
                        }
                    } catch (ex) {
                        if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                            throw this.getLibraryComputationException;
                        }

                        return ex;
                    }
                } else {
                    return this.getFormulaErrorStrings();
                }
            }
            return s1.split(this.tic).join("");
        };
        this.computeIfError = function (args) {
            var argsArray = args.split(this.getParseArgumentSeparator());
            var range = argsArray[0];

            //    var arrgsArray = [this.getParseArgumentSeparator, ];
            if (argsArray.length != 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (range[0] == this.tic)
                return this.getValueFromArg(argsArray[0]);
            try {
                if (range[0] == this.bMARKER) {
                    range = range.replace(this.bMARKER, ' ');
                    range = (this._isIE8) ? range.replace(/^\s+|\s+$/g, '') : range.trim();
                    if (range[0] == ("NAN") || range[0] == ("-NAN") || range[0] == ("INFINITY") || range[0] == ("-INFINITY") || range[0] == ("#") || range[0] == ("n#"))
                        return this.getValueFromArg(argsArray[1]);
                }
                range = argsArray[0];

                range = this.getValueFromArg(range).toUpperCase().split(this.tic).join(this._string_empty);
            } catch (ex) {
                range = range.toUpperCase();
            }
            if (range[0] == ("NAN") || range[0] == ("-NAN") || range[0] == ("INFINITY") || range[0] == ("-INFINITY") || range[0] == ("#") || range[0] == ("n#"))
                return this.getValueFromArg(argsArray[1]);
            else
                return this.getValueFromArg(argsArray[0]);
        };
        this.computeIfNA = function (range) {
            var args = this.splitArgsPreservingQuotedCommas(range);
            var argCount = args.length, arg1 = this._string_empty, index = -1, result;
            if (argCount != 2) {
                return this.getFormulaErrorStrings();
            }

            if (this.computeIsNA(args[0]) == this.trueValueStr) {
                if ((this.tic.indexOf(args[1]) == -1) && !this._isCellReference(args[1]) && !(result = this._parseDouble(args[0])) && !this.namedRanges.containsKey(args[1])) {
                    return this.getErrorStrings()[5].toString();
                    ;
                }
                arg1 = this.getValueFromArg(args[1]);
            } else {
                if (this._isRange(args[0])) {
                    arg1 = this.getErrorStrings()[1].toString();
                } else {
                    if ((this.tic.indexOf(args[0]) == -1) && !this._isCellReference(args[0]) && !(result = this._parseDouble(args[0])) && !this.NamedRanges.ContainsKey(args[0])) {
                        return this.getErrorStrings()[5].toString();
                    }
                    arg1 = this.getValueFromArg(args[0]);
                }
            }
            if (arg1[0] == this.tic && arg1[length - 1] == this.tic) {
                arg1 = this.substring(arg1, 1, arg1.length - 2);
            }
            return arg1;
        };
        this.computeNot = function (args) {
            var s = args;
            var d1;
            var array1 = [this.getParseArgumentSeparator, ':'];

            //var array1 = [this.getParseArgumentSeparator()];
            ////parsed formula
            if ((args.length > 0) && this._indexOfAny(args, array1) > -1) {
                return this.getFormulaErrorStrings();
            } else {
                try {
                    s = this.getValueFromArg(s);
                    if (s == this.trueValueStr) {
                        s = this.falseValueStr;
                    } else if (s == this.falseValueStr) {
                        s = this.trueValueStr;
                    } else if (d1 = this._parseDouble(s)) {
                        ////Flip the value.
                        if (Math.abs(d1) > 1e-10) {
                            s = this.falseValueStr;
                        } else {
                            s = this.trueValueStr;
                        }
                    }
                } catch (ex) {
                    if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                        throw this.getLibraryComputationException;
                    }

                    return ex;
                }
            }

            return s;
        };
        this.computeOr = function (args) {
            var sum = this.falseValueStr;
            var s1;
            var d;

            // args = this.adjustRangeArg(args);
            var ranges = this.splitArgsPreservingQuotedCommas(args);
            for (var r = 0; r < ranges.length; r++) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1 && this._isCellReference(r)) {
                    cells = this.getCellsFromArgs(ranges[r]);
                    for (var s = 0; s < cells.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cells[s]);
                        } catch (ex) {
                            if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                                throw this.getLibraryComputationException;
                            }

                            return ex;
                        }
                        sum |= (s1 == this.trueValueStr) || (d == this._parseDouble(s1) && d != 0);
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                            throw this.getLibraryComputationException;
                        }

                        return ex;
                    }

                    sum |= (s1 == this.trueValueStr) || (d == this._parseDouble(s1) && d != 0);
                }
            }
            return sum ? this.trueValueStr : this.falseValueStr;
        };
        this.computeTrue = function (empty) {
            return this.trueValueStr;
        };
        this.computeXor = function (arg) {
            var sum = false, s1, d;

            // range = this.adjustRangeArg;
            var ranges = this.splitArgsPreservingQuotedCommas(arg);
            for (var r = 0; r < ranges.length; r++) {
                ////is a cellrange
                if (ranges[r].indexOf(':') > -1 && this._isCellReference(r)) {
                    var cells =this.getCellsFromArgs(ranges[r])
                    for (var s = 0; s < cells.length; s++) {
                        try {
                            s1 = this.getValueFromArg(cells[s]);
                        } catch (ex) {
                            if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                                throw this.getLibraryComputationException;
                            }

                            return ex;
                        }

                        sum = (s1 == this.trueValueStr) || (d == this._parseDouble(s1) && (d != 0));
                    }
                } else {
                    try {
                        s1 = this.getValueFromArg(ranges[r]);
                    } catch (ex) {
                        if (this.rethrowLibraryComputationExceptions && this.getLibraryComputationException != null) {
                            throw this.getLibraryComputationException;
                        }

                        return ex;
                    }

                    sum = (s1 == this.trueValueStr) || (d == this._parseDouble(s1)) && (d != 0);
                }
            }
            return sum ? this.trueValueStr : this.falseValueStr;
        };

        this.computeCell = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length > 2) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var reference = "";
            if (args.length == 2) {
                if (!this._isCellReference(args[1]) && !this.namedRanges.containsKey(args[1])) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[5].toString();
                }
                reference = args[1];
            } else {
                reference = this.cell;
            }
            var type = args[0].split(this.tic).join(this._string_empty).toLowerCase(), result = "";
            switch (type) {
                case "address":
                    result = this.rowIndex(reference).toString() + this.getParseArgumentSeparator() + this.colIndex(reference).toString();
                    result = this.computeAddress(result);
                    break;
                case "col":
                    result = this.computeColumn(reference);
                    break;
                case "color":
                    break;
                case "contents":
                    result = this.getValueFromArg(reference);
                    break;
                case "filename":
                    result = "Not Supported";
                    break;
                case "format":
                    break;
                case "parentheses":
                    break;
                case "prefix":
                    break;
                case "protect":
                    break;
                case "row":
                    result = this.computeRow(reference);
                    break;
                case "type":
                    if (this.computeIsBlank(reference) == this.trueValueStr)
                        result = "b";
                    else if (this.computeIsText(reference) == this.trueValueStr)
                        result = "l";
                    else
                        result = "v";
                    break;
                case "width":
                    break;
                default:
                    break;
            }
            return result.toString();
        };
        this.computeErrorType = function (args) {
            var cellReference = this._string_empty;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator());
            var argCount = arg.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (this._isCellReference(args)) {
                cellReference = this.getValueFromArg(args);
            } else {
                cellReference = args;
            }
            if (cellReference.length > 1 && cellReference[0] == this.tic[0] && cellReference[cellReference.length - 1] == this.tic[0]) {
                return "#N/A";
            }
            if (cellReference == "#NULL!")
                return "1";
            else if (cellReference == "#DIV/0!")
                return "2";
            else if (cellReference == "#VALUE!")
                return "3";
            else if (cellReference == "#REF!")
                return "4";
            else if (cellReference == "#NAME?")
                return "5";
            else if (cellReference == "#NUM!")
                return "6";
            else if (cellReference == "#N/A")
                return "7";
            else if (cellReference == "#GETTING_DATA")
                return "8";
            return "#N/A";
        };
        this.computeInfo = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            if (argCount != 1 || argList=="") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var firstStr = this._stripTics0(this.getValueFromArg(args[0])).toLowerCase();
            var result = this._string_empty;
            switch (firstStr) {
                case "directory":
                    result = window.location.toString();
                    break;
                case "numfile":
                    if (this.getSortedSheetNames() != null && CalcEngine._sheetFamiliesList.length == 0)
                        result = this.getSortedSheetNames().length.toString();
                    else
                        result = "1";
                    break;
                case "origin":
                    var cell = this.rowIndex(this.cell).toString() + this.getParseArgumentSeparator() + this.colIndex(this.cell).toString();
                    result = "$A: " + this.computeAddress(cell);
                    break;
                case "osversion":
                    result = navigator.platform.toString();
                    break;
                case "recalc":
                    if (this._alwaysComputeDuringRefresh)
                        result = "Automatic";
                    else
                        result = "Manual";
                    break;
                case "release":
                    result = this.System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.toString();
                    break;
                case "system":
                    switch (navigator.appVersion.indexOf()) {
                        case navigator.platform:
                            result = "unix";
                            break;
                        case navigator.platform:
                            result = "mac";
                            break;
                        default:
                            result = "pcdos";
                            break;
                    }
                    break;
                default:
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[1].toString();
            }
            return result;
        };
        this.computeIsBlank = function (args) {

            var arg = this.splitArgsPreservingQuotedCommas(args);
            var argCount = arg.length;
            if (argCount != 1 || args == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (this.getValueFromArg(args) == "") {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsErr = function (range) {
            var arg = this.splitArgsPreservingQuotedCommas(range);
            var argCount = arg.length;
            if (argCount != 1 || range == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            range = this.getValueFromArg(range).toUpperCase().split(this.tic).join(this._string_empty);
            if (range.count > 1) {

            }
            if ((range == ("NAN") || range == ("-NAN") || range == ("INFINITY") || range == ("-INFINITY") || range == ("#") || range == ("n#")) && !(range == ("#N/A"))) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };

        this.computeIsError = function (range) {

            var arg = this.splitArgsPreservingQuotedCommas(range);
            var argCount = arg.length;
            if (argCount != 1 || range == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            try {
                range = this.getValueFromArg(range).toUpperCase().split(this.tic).join(this._string_empty);
            }
            catch (ex) {
                return this.trueValueStr;
            }
            if (range.indexOf("NAN") == 0 || range.indexOf("-NAN") == 0 || range.indexOf("INFINITY") == 0 || range.indexOf("-INFINITY") == 1 || range.indexOf("#") == 0 || range.indexOf("n#") == 0) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };

        this.computeIsEven = function (args) {
            var cellReference = this._string_empty;
            var val = 0.0;
            var arg = this.splitArgsPreservingQuotedCommas(args);
            var argCount = arg.length;
            if (argCount != 1 || args == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var argVal = this.getValueFromArg(arg[0]);
            var value = parseInt(argVal.split(this.tic).join(""));
            if (isNaN(value)) {
                if (this._rethrowLibraryComputationExceptions)
                    throw this.formulaErrorStrings[this._bad_formula];
                return (argVal[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
            }
            var dt = new Date(Date.parse(argVal));
            if (isNaN(value) && dt.toString() != "invalid Date") {
                value = this._toOADate(dt);
            }
            var result = value;
            if (result % 2 == 0) {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsFormula = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            value = this.getValueFromArg(args[0]);
            if (this.namedRanges.containsValue(value)) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[1].toString();
            }
            var intVal = parseInt(value.split(this.tic).join(""));
            if (isNaN(intVal) || !this._isCellReference(args[0])) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            }
            var family = CalcEngine.getSheetFamilyItem(this.grid);
            if (this.isSheetMember() && family.parentObjectToToken != null) {
                args[0] = family.parentObjectToToken.getItem(this.grid) + args[0];
            }
            if (this.getFormulaInfoTable().containsKey(args[0])) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };
        this.computeIsLogical = function (args) {

            args = this.getValueFromArg(args);
            if (args == this.falseValueStr || args == this.trueValueStr) {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsNA = function (args) {
            if (this.getErrorStrings().indexOf(args.toUpperCase()) != -1) {
                if (args.toUpperCase() == ("#N/A")) {
                    return this.trueValueStr;
                } else {
                    return this.falseValueStr;
                }
            }
            try {
                args = this.getValueFromArg(args).toUpperCase();
            } catch (ex) {
                return this.falseValueStr;
            }
            if (args[0] == ("#N/A")) {
                return this.trueValueStr;
            }
            return this.falseValueStr;
        };
        this.computeIsNonText = function (args) {
            if (this.computeIsText(args) == this.trueValueStr) {
                return this.falseValueStr;
            } else {
                return this.trueValueStr;
            }
        };
        this.computeIsNumber = function (range) {
            var d;
            range = this.getValueFromArg(range);
            if (!isNaN(this._parseDouble(range))) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };
        this.computeIsRef = function (args) {
            var arg = this.splitArgsPreservingQuotedCommas(args);
            if (arg.length != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            return ((this.namedRanges.containsKey(args) || this._isCellReference(args)) && args[0] != (this.tic) && args[length - 1] != (this.tic)).toString().toUpperCase();
        };
        this.computeIsOdd = function (args) {
            var cellReference = this._string_empty;
            var val = 0.0;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator());
            var argCount = arg.length;
            var array1 = [this.getParseArgumentSeparator(), ':'];
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (args != this._string_empty) {
                if (this._indexOfAny(args, array1) != -1) {
                    return "#VALUE!";
                }
                cellReference = this.getValueFromArg(arg[0]);
                val = parseInt(cellReference.split(this.tic).join(this._string_empty))
                if (isNaN(val)) {
                    if (this._rethrowLibraryComputationExceptions)
                        throw this.formulaErrorStrings[this._bad_formula];
                    return (cellReference[0] == this.tic || this._isCellReference(arg[0])) ? this.getErrorStrings()[1].toString() : this.getErrorStrings()[5].toString();
                }
                if (val % 2 != 0) {
                    return this.trueValueStr;
                }
                return this.falseValueStr;
            }
            return this.trueValueStr;
        };
        this.computeIsText = function (args) {
            var isCell = this._isCellReference(args);
            var s = (isCell) ? this.getValueFromArg(args) : args;
            if ((isCell || s[0] == this.tic) && s.length > 0 && isNaN(this._parseDouble(s))) {
                return this.trueValueStr;
            } else {
                return this.falseValueStr;
            }
        };
        this.computeN = function (args) {
            var cellReference = this._string_empty;
            var val = 0.0;
            var date;
            var arg = this._splitArguments(args, this.getParseArgumentSeparator());
            var argCount = arg.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            
            cellReference = this.getValueFromArg(args);
            date = new Date(Date.parse(cellReference));
            if (this._parseDouble(cellReference))
            {
                val = this._parseDouble(cellReference);
                return val.toString();
            }
            else if (new Date(Date.parse(cellReference)))
           {
                val = this._getSerialDateTimeFromDate(date);
           }
            if (cellReference == this.trueValueStr) {
                val = 1;
            }
            else if (cellReference == this.falseValueStr) {
                val = 0;
            }
            else if (this.getErrorStrings().indexOf(cellReference) > -1 || this.formulaErrorStrings.indexOf(cellReference) > -1) {
                return cellReference;
            }
            else if (!isNaN(val)) {
                return val.toString();
            }
            else {
                return "0";
            }

            return val.toString();
        };
        this.computeNA = function () {
            return "#N/A";
        };
        this.computeSheet = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount > 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var cellReference = args[0].toUpperCase();
            cellReference = (cellReference == null || cellReference == this._string_empty) ? this.cell : cellReference;
            if (cellReference == null || cellReference == this._string_empty) {
                return "1";
            }
            if (this.getErrorStrings().indexOf(cellReference)>-1) {
                return cellReference;
            }
            if (!this._isCellReference(cellReference) && !this.namedRanges.containsKey(cellReference) && this.getSortedSheetNames().indexOf(cellReference) == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[0].toString();
            }
            var family = this.getSheetFamilyItem(this.grid);
            var sheet1 = this._getSheetTokenFromReference(cellReference);
            var sheetNumber = 0;
            if ((sheet1 == null || sheet1 == this._string_empty) && this.getSortedSheetNames().indexOf(cellReference) == -1 && !cellReference.indexOf(this.sheetToken.toString())) {
                sheetNumber = this.getSheetID(this.grid) + 1;
            } else if (sheet1.length > 0) {
                sheetNumber = parseInt(sheet1.Replace(this.sheetToken.split(this.tic).join(this._string_empty))) + 1;
            } else {
                try {

                    for (var s = 0; s < family.tokenToParentObject().length; s++){
                        if (family.sheetNameToParentObject[cellReference].Equals(family.tokenToParentObject()[s].Value)) {
                            var gg = griddata.Value;
                            sheetNumber = this.getSheetID(gg) + 1;
                            break;
                        }
                    }
                } catch (ex) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[2].toString();
                }
            }
            return sheetNumber.toString();
        };
        this.computeSheets = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var value = this._string_empty;
            if (argCount > 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var sheetNumber = 0;
            var cellReference = args[0].toUpperCase();
            if (this.getErrorStrings().indexOf(cellReference) > -1) {
                return cellReference;
            }
            if (cellReference == null || cellReference == this._string_empty) {
                return this.getSortedSheetNames().length.toString();
            }
            else if (cellReference.split(this.tic).join(this._string_empty) == "") {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            } else if (!this._isCellReference(cellReference) && !this.namedRanges.containsKey(cellReference) && this.getSortedSheetNames().indexOf(cellReference) == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._bad_formula];
                return this.getErrorStrings()[5].toString();
            }
            try {
                var sheetsCount = this._splitArguments(cellReference, '!');
                sheetNumber = (sheetsCount.length - 1) / 2;
                if (sheetNumber == 0) {
                    if (this.getRethrowLibraryComputationExceptions())
                        throw this.formulaErrorStrings[this._bad_formula];
                    return this.getErrorStrings()[5].toString();
                }
            } catch (ex) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[2].toString();
                return this.getErrorStrings()[2].toString();
            }
            return sheetNumber.toString();
        };
        this.computeType = function (argList) {
            var args = this.splitArgsPreservingQuotedCommas(argList);
            var argCount = args.length;
            var result = 0;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            var text = this.getValueFromArg(args[0]);
            if (text == null || text == this._string_empty) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[1].toString();
                return this.getErrorStrings()[1].toString();
            } else if (argList.indexOf(this.getParseArgumentSeparator()) > -1) {
                result = 64;
            } else if (this.computeIsNumber(text) == this.trueValueStr) {
                result = 1;
            } else if (this.computeIsLogical(text) == this.trueValueStr) {
                result = 4;
            } else if (this.getErrorStrings().indexOf(text) > -1) {
                result = 16;
            } else if (this.computeIsText(text) == this.trueValueStr) {
                result = 2;
            }
            return result.toString();
        };
        
        this.computeRow = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            var argCount = args.length;
            if (argCount != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (args[0] == this._string_empty) {
                return this.rowIndex(this.cell).toString();
            }
            if (!this._isCellReference(args[0]) && !this.namedRanges.containsKey(args[0])) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            if (arg.indexOf(':') > -1) {
                arg = this.getCellsFromArgs(arg)[0]; //considers first cell reference from the range of cells.
            }
            return this.rowIndex(arg).toString();
        };
        this.computeRows = function (arg) {
            var args = this.splitArgsPreservingQuotedCommas(arg);
            if (args.length != 1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.formulaErrorStrings[this._wrong_number_arguments];
                return this.formulaErrorStrings[this._wrong_number_arguments];
            }
            if (!this._isCellReference(args[0]) && arg.indexOf(';') == -1) {
                if (this.getRethrowLibraryComputationExceptions())
                    throw this.getErrorStrings()[4].toString();
                return this.getErrorStrings()[4].toString();
            }
            var firstRow, LastRow, totalRows = 1;
            if (args[0].indexOf(':') > -1) {
                var cells = this.getCellsFromArgs(arg);
                firstRow = this.rowIndex(cells[0].toString());
                LastRow = this.rowIndex(cells[cells.length - 1].toString());
                totalRows = LastRow - firstRow + 1;
                totalRows = (totalRows > 0) ? totalRows : 1;
            } else if (args[0].indexOf(';') > -1) {
                var arrayArg = this._splitArguments(arg.split(this.tic).join(this._string_empty), ';');
                for (var i = 1; i < arrayArg.length; i++) {
                    if (this.splitArgsPreservingQuotedCommas(arrayArg[i - 1]).length != this.splitArgsPreservingQuotedCommas(arrayArg[1]).length) {
                        if (this.getRethrowLibraryComputationExceptions())
                            throw this.getErrorStrings()[4].toString();
                        return this.getErrorStrings()[4].toString();
                    }
                    totalRows = arrayArg.length;
                }
            }
            return totalRows.toString();
        };


        //this.grid.wireParentObject();
        this._initLibraryFunctions();
        //var _rangeInfo = new RangeInfo();
    }



    CalcEngine.CalcEngine = function () {
    };

    CalcEngine.getFormulaCharacter= function () {
        if (this._formulaChar == '\0') {
            this._formulaChar = '=';
        }

        return this._formulaChar;
    };

    CalcEngine._formulaChar = '=';

    CalcEngine.sheetFamilyID = 0;
    CalcEngine._tokenCount = 0;
    CalcEngine.modelToSheetID = null;
    CalcEngine._sheetFamiliesList = null;

    CalcEngine._defaultFamilyItem = null;

    CalcEngine.createSheetFamilyID = function () {
        if (this.sheetFamilyID == Number.MAX_SAFE_INTEGER) {
            this.sheetFamilyID = Number.MIN_SAFE_INTEGER;
        }
        return this.sheetFamilyID++;
    };

    CalcEngine.getSheetFamilyItem = function (model) {
        if (this.sheetFamilyID == 0) {
            if (CalcEngine._defaultFamilyItem == null) {
                CalcEngine._defaultFamilyItem = new GridSheetFamilyItem();
            }

            return CalcEngine._defaultFamilyItem;
        }

        if (CalcEngine._sheetFamiliesList == null) {
            CalcEngine._sheetFamiliesList = new HashTable();
        }

        var i = CalcEngine.modelToSheetID.getItem(model);

        if (!CalcEngine._sheetFamiliesList.containsKey(i)) {
            CalcEngine._sheetFamiliesList.add(i, new GridSheetFamilyItem());
        }

        return CalcEngine._sheetFamiliesList.getItem(i);
    };

    CalcEngine.resetSheetFamilyID = function () {
        this.sheetFamilyID = 0;

        if (CalcEngine.modelToSheetID != null) {
            CalcEngine.modelToSheetID.clear();
            CalcEngine.modelToSheetID = null;
        }

        if (CalcEngine._sheetFamiliesList != null) {
            CalcEngine._sheetFamiliesList.clear();
            CalcEngine._sheetFamiliesList = null;

            //Fix for defect #12445.
            this.resetSheetIDs();
        }
    };

    CalcEngine.resetSheetIDs = function () {
        this._tokenCount = 0;
    };

    CalcEngine.unregisterGridAsSheet = function (refName, model) {
        var family = CalcEngine.getSheetFamilyItem(model);
        var refName1 = refName.toUpperCase();
        if (family.sheetNameToParentObject != null && family.sheetNameToParentObject.containsKey(refName1)) {
            family.sheetNameToParentObject.remove(refName1);
            var token = family.sheetNameToToken.getItem(refName1);
            family.sheetNameToToken.remove(refName1);
            family.tokenToParentObject.remove(token);
            family.parentObjectToToken.remove(model);
        }
    };
    return CalcEngine;
})(jQuery, Syncfusion);

var LookupCachingMode;
(function (LookupCachingMode) {
    LookupCachingMode[LookupCachingMode["None"] = 0] = "None";
    LookupCachingMode[LookupCachingMode["VLOOKUP"] = 1] = "VLOOKUP";
    LookupCachingMode[LookupCachingMode["HLOOKUP"] = 2] = "HLOOKUP";
    LookupCachingMode[LookupCachingMode["Both"] = 3] = "Both";
    LookupCachingMode[LookupCachingMode["OptimizeForMatches"] = 4] = "OptimizeForMatches";
})(LookupCachingMode || (LookupCachingMode = {}));
;

var FormulaInfoSetAction;
(function (FormulaInfoSetAction) {
    FormulaInfoSetAction[FormulaInfoSetAction["FormulaSet"] = 0] = "FormulaSet";
    FormulaInfoSetAction[FormulaInfoSetAction["NonFormulaSet"] = 1] = "NonFormulaSet";
    FormulaInfoSetAction[FormulaInfoSetAction["CalculatedValueSet"] = 2] = "CalculatedValueSet";
})(FormulaInfoSetAction || (FormulaInfoSetAction = {}));
;

var CalcQuickBase = (function () {
    /// <summary>
    /// Constructor that resets the CalcEngine object.
    /// </summary>
    /// <param name="resetStaticMembers">
    /// Indicates whether the static members of the CalcEngine class will be cleared.</param>
    function CalcQuickBase(resetStaticMembers) {
        this.resetStaticMembers = resetStaticMembers;
        this._calcQuickID = 0;
        this._controlModifiedFlags = null;
        this._dataStore = null;
        this._engine = new Object;
        this._keyToRowsMap = null;
        this._keyToVectors = null;
        this._nameToControlMap = null;
        this._rowsToKeyMap = null;
        this._isValueSetEventChanged = false;
        this._autoCalc = false;
        this._isIE8 = false;
        // Used internally to name the calcsheets.
        this._cellPrefix = "!0!A";
        this._checkKeys = true;
        this._disposeEngineResource = true;
        this._ignoreChanges = false;
        this._leftBrace = "{";
        this._tic = "\"";
        ////Used to change the Hashtable key references to
        ////row,col references needed by the CalcEngine.
        ////Every key is swapped for a reference like A1 or A2 or A14.
        this._leftBracket = "[";
        this._rightBracket = "]";
        this._validLeftChars = "+-*/><=^(&,";
        this._validRightChars = "+-*/><=^)&,";
        this.getKeyValue = function (key) {
            key = key.toUpperCase();

            if (this.getDataStore().containsKey(key)) {
                var fInfo = this.getDataStore().getItem(key);
                var s = fInfo.getFormulaText();
                if (s.length > 0 && s[0] == CalcEngine.getFormulaCharacter() && fInfo.calcID != this.getEngine().getCalcID()) {
                    this.getEngine().cell = this._cellPrefix + this.getKeyToRowsMap().getItem(key).toString();

                    s = s.substring(1); ////strip out formulaChar.

                    try {
                        fInfo.setParsedFormula(this.getEngine().parseFormula(this._markKeys(s)));
                        ;
                    } catch (ex) {
                        if (this.getCheckKeys()) {
                            fInfo.setFormulaValue(ex.Message);
                            fInfo.calcID = this.getEngine().getCalcID();

                            if (this._valueSetEvent != null) {
                                this._valueSetEvent.trigger(this, new ValueSetEventArgs(key, fInfo.getFormulaValue(), 2 /* CalculatedValueSet */));
                            }

                            return this.getDataStore().getItem(key).getFormulaValue();
                        }
                    }

                    try {
                        fInfo.setFormulaValue(this.getEngine().computeFormula(fInfo.getParsedFormula()));
                    } catch (ex) {
                        if (this.getThrowCircularException() && ex.toString().indexOf(this.getEngine().formulaErrorStrings[this.getEngine().circular_reference_]) == 0) {
                            throw ex;
                        }
                    }

                    fInfo.calcID = this.getEngine().getCalcID();

                    if (this._valueSetEvent != null) {
                        this._valueSetEvent.trigger(this, new ValueSetEventArgs(key, fInfo.getFormulaValue(), 2 /* CalculatedValueSet */));
                    }
                }
                if (this.getEngine().getThrowCircularException()) {
                    if (this.getEngine().getIterationMaxCount() > 0) {
                        fInfo.setFormulaValue(this.getEngine().handleIteration(this.getEngine().cell, fInfo));
                    }
                }
                return this.getDataStore().getItem(key).getFormulaValue();
            } else if (this.getKeyToVectors().containsKey(key)) {
                return this.getKeyToVectors().getItem(key).toString();
            } else {
                return "";
            }
        };
        this.setKeyValue = function (key, value) {
            key = key.toUpperCase();
            var s = (this._isIE8) ? value.toString().replace(/^\s+|\s+$/g, '') : value.toString().trim();

            if (!this.getDataStore().containsKey(key) || s.indexOf(this._leftBrace) == 0) {
                ////support for vectors
                if (s.indexOf(this._leftBrace) == 0) {
                    if (!this.getKeyToVectors().containsKey(key)) {
                        this.getKeyToVectors().add(key, "");
                    }

                    s = s.substring(1, s.length - 2);
                    var i = this.getKeyToRowsMap().length + 1;
                    var ss = s.split(this.getEngine().getParseArgumentSeparator());
                    var range = "A" + i + ":A" + (i + ss.length - 1);
                    this.getKeyToVectors()[key] = range;
                    for (var index = 0; index < ss.length; index++) {
                        var key1 = "Q_" + (this.getKeyToRowsMap().length + 1);
                        this.getDataStore().add(key1, new FormulaInfo());
                        this.getKeyToRowsMap().add(key1, this.getKeyToRowsMap().length + 1);
                        this.getRowsToKeyMap().add(this.getRowsToKeyMap().length + 1, key1);

                        var fInfo1 = this.getDataStore().getItem(key1);
                        fInfo1.setFormulaText("");
                        fInfo1.setParsedFormula("");

                        ////fInfo1.FormulaValue = s1;
                        fInfo1.setFormulaValue(this.parseAndCompute(ss[index]));
                    }

                    return;
                } else {
                    this.getDataStore().add(key, new FormulaInfo());
                    this.getKeyToRowsMap().add(key, this.getKeyToRowsMap().length + 1);
                    this.getRowsToKeyMap().add(this.getRowsToKeyMap().length + 1, key);
                }
            }

            if (this.getKeyToVectors().containsKey(key)) {
                this.getKeyToVectors().remove(key);
            }
            var fInfo = this.getDataStore().getItem(key);

            if (!this.ignoreChanges && fInfo.getFormulaText() != null && fInfo.getFormulaText().length > 0 && fInfo.getFormulaText() != s) {
                ////Reset its dependencies if there are any...
                var s1 = this._cellPrefix+ this.getKeyToRowsMap().getItem(key).toString();
                var ht = this.getEngine().getDependentFormulaCells().getItem(s1);
                if (ht != null) {
                    this.getEngine().clearFormulaDependentCells(s1);
                }
            }

            if (s.length > 0 && s[0] == CalcEngine.getFormulaCharacter()) {
                fInfo.setFormulaText(s);
                if (this._valueSetEvent != null) {
                    this._valueSetEvent.trigger(this, new ValueSetEventArgs(key, fInfo.getFormulaValue(), 2 /* CalculatedValueSet */));
                }
            } else if (fInfo.getFormulaValue() != s) {
                fInfo.setFormulaText("");
                fInfo.setParsedFormula("");
                fInfo.setFormulaValue(s);
                if (this._valueSetEvent != null) {
                    this._valueSetEvent.trigger(this, new ValueSetEventArgs(key, fInfo.getFormulaValue(), 2 /* CalculatedValueSet */));
                }
            }

            if (this.getAutoCalc()) {
                this.updateDependencies(key);
            }
        };
        /// <summary>
        /// A method to reset all the keys registered with CalcQuickBase object.
        /// </summary>
        this.resetKeys = function () {
            this.getDataStore().clear();
            this.getKeyToRowsMap().clear();
            this.getRowsToKeyMap().clear();
            this.getKeyToVectors().clear();
            this.getNameToControlMap().clear();
        };
        this._checkAdjacentPiece = function (s, validChars, first) {
            var b = true;
            s = (this._isIE8) ? s.replace(/^\s+|\s+$/g, '') : s.trim();
            if (s.length > 0) {
                b = validChars.indexOf(s[first ? 0 : s.length - 1]) > -1;
            }

            return b;
        };
        /// <summary>
        /// Creates the <see cref="CalcEngine"/> object used by this CalQuick object.
        /// </summary>
        /// <returns>Returns an instance of a CalcEngine object.</returns>
        /// <remarks>You can override this method and return a derived CalcEngine object use
        /// by the derived CalcQuick object.
        /// </remarks>
        this.createEngine = function () {
            return new CalcEngine(this);
        };
        this.dispose = function () {
            this._dataStore = null;
            this._rowsToKeyMap = null;
            this._keyToRowsMap = null;
            this._keyToVectors = null;

            this._controlModifiedFlags = null;
            this._nameToControlMap = null;

            //this.ValueChanged -= new ValueChangedEventHandler(this._engine.grid_ValueChanged);
            if (this.getDisposeEngineResource()) {
                ////CalcEngine.ResetSheetFamilyID();
                this._engine.getDependentFormulaCells().clear();
                this._engine.getDependentCells().clear();

                if (this._engine != null) {
                    this._engine.dispose();
                }

                this._engine = null;
            }
        };
        /// <summary>
        /// A method that parses and computes a well-formed algebraic expression passed in.
        /// </summary>
        /// <param name="formulaText">The text of the formula.</param>
        /// <returns>The computed value.</returns>
        /// <remarks>You would use this method if you have a formula string which
        /// contains only constants or library function references. Such formulas
        /// do not depend upon other values. If you have registered a variable through
        /// an indexer, then that variable can be used in a formula expression passed into this
        /// method. This method will return the Exception text if an exception is thrown
        /// during the computation.
        /// </remarks>
        this.tryParseAndCompute = function (formulaText) {
            var ret = "";
            try {
                ret = this.parseAndCompute(formulaText);
            } catch (ex) {
                ret = ex.message;
            }

            return ret;
        };
        /// <summary>
        /// A method that returns the formula string associated with the key passed in from a FormulaInfo object.
        /// </summary>
        /// <param name="key">The Hashtable key associated with the FormulaInfo object.</param>
        /// <returns>The formula string may be the empty string if no formula is stored with this key.</returns>
        this.getFormula = function (key) {
            key = key.toUpperCase();
            if (this.getDataStore().containsKey(key)) {
                return this.getDataStore().getItem(key).getFormulaText();
            }

            return "";
            ;
        };
        /// <summary>
        /// A method to get the value of the cell referred. For internal CalcQuick use only.
        /// </summary>
        /// <param name="row">Row index.</param>
        /// <param name="col">Column index.</param>
        /// <returns>(row, col) data.</returns>
        /// <remarks>
        /// CalcQuick does not expose a (row, col) data access model.
        /// But since CalcEngine requires such a model, CalcQuick uses
        /// a row, col access model internally, but only exposes the
        /// formula Key model to access values.
        /// </remarks>
        this.getValueRowCol = function (grid, row, col) {
            ////CalcQuick objects do not expose a row, col data access model.
            ////But the CalcEngine requires such a model. So, CalcQuick uses
            ////a row, col access model internally, but only exposes using
            ////formula Key values to access formulas.
            var key = this.getRowsToKeyMap().getItem(row).toString();
            var s = this.getKeyValue(key).toString();
            if (s != null && s[s.length - 1] == ("%") && s.length > 1) {
                var d = this.parseDouble(s.substring(0, s.length - 1));
                if (isNaN(d)) {
                    s = (Number(d) / 100).toString();
                }
            }

            return s;
        };
        /// <summary>
        /// Initializes any structures needed by this instance.
        /// </summary>
        /// <param name="resetStaticMembers">
        /// Indicates whether the static members of the CalcEngine class will be cleared.</param>
        this.initCalcQuick = function (resetStaticMembers) {
            this._dataStore = new HashTable();
            this._rowsToKeyMap = new HashTable();
            this._keyToRowsMap = new HashTable();
            this._keyToVectors = new HashTable();

            this._controlModifiedFlags = new HashTable();
            this._nameToControlMap = new HashTable();
            var i = CalcEngine.createSheetFamilyID();
            this._engine = this.createEngine();
            if (resetStaticMembers) {
                CalcEngine.resetSheetFamilyID();
                this._engine.getDependentFormulaCells().clear();
                this._engine.getDependentCells().clear();
            }

            
            this._cellPrefix= "!" + i + "!A";
            this._engine.registerGridAsSheet(RangeInfo.getAlphaLabel(this.getCalcQuickID()), this, i);

            ////By default, we will trigger the calculations locally,
            ////so turn off the engine calculating mechanism.
            this._engine.setCalculatingSuspended(true);
            this._engine.ignoreValueChanged = true;
            this._isIE8 = (ej.browserInfo().name == 'msie' && ej.browserInfo().version == '8.0') ? true : false;
        };
        /// <summary>
        /// Change the calcQuick keys into cell range. i.e mark the key A as A1
        /// </summary>
        /// <param name="formula">formula to compute value</param>
        /// <returns>marked fommula</returns>
        this._markKeys = function (formula) {
            var left = formula.indexOf(this._leftBracket);
            while (left > -1) {
                var len = formula.substring(left).indexOf(this._rightBracket) - 1;
                var key = "";

                if (len > 0) {
                    key = formula.substring(left + 1, len + left + 1).toUpperCase();
                    if (this.getKeyToVectors().containsKey(key)) {
                        var rightPiece = (left + len + 2 < formula.length) ? formula.substring(left + len + 2) : "";
                        if (this.getCheckKeys() && !this._checkAdjacentPiece(rightPiece, this._validRightChars, true)) {
                            throw "not followed properly" + key;
                        }

                        var leftPiece = (left > 0) ? formula.substring(0, left) : "";
                        if (this.getCheckKeys() && !this._checkAdjacentPiece(leftPiece, this._validLeftChars, false)) {
                            throw "not followed properly" + key;
                        }

                        formula = leftPiece + this.getKeyToVectors().getItem(key).toString() + rightPiece;
                        left = formula.indexOf(this._leftbraket);
                    } else if (this.getKeyToRowsMap().containsKey(key)) {
                        var rightPiece = (left + len + 2 < formula.length) ? formula.substring(left + len + 2) : "";
                        if (this.getCheckKeys() && !this._checkAdjacentPiece(rightPiece, this._validRightChars, true)) {
                            throw "not followed properly" + key;
                        }

                        var leftPiece = (left > 0) ? formula.substring(0, left) : "";
                        if (this.getCheckKeys() && !this._checkAdjacentPiece(leftPiece, this._validLeftChars, false)) {
                            throw "not followed properly" + key;
                        }

                        formula = leftPiece + "A" + this.getKeyToRowsMap().getItem(key).toString() + rightPiece;
                        left = formula.indexOf(this._leftBracket);
                    } else {
                        if (formula.toUpperCase().indexOf(this._tic + this._leftBracket + key + this._rightBracket + this.tic) > 0) {
                            break;
                        } else {
                            throw "Unknown key: " + key;
                        }
                    }
                } else {
                    left = -1;
                }
            }

            return formula;
        };
        /// <summary>
        /// A method that parses and computes a well-formed algebraic expression passed in.
        /// </summary>
        /// <param name="formulaText">The text of the formula.</param>
        /// <returns>The computed value.</returns>
        /// <remarks>You would use this method if you have a formula string which
        /// contains only constants or library function references. Such formulas
        /// do not depend upon other values. If you have registered a variable through
        /// an indexer, then that variable can be used in a formula expression passed into this
        /// method.
        /// </remarks>
        this.parseAndCompute = function (formulaText) {
            if (formulaText.length > 0 && formulaText[0] == this.getEngine().getFormulaCharacter()) {
                formulaText = formulaText.substring(1);
            }

            return this.getEngine().parseAndComputeFormula(this._markKeys(formulaText));
        };
        /// <summary>
        /// A method that recompute any formulas stored in the CalcQuick instance.
        /// </summary>
        /// <remarks>
        /// This method only has is used when AutoCalc is False. It loops through
        /// all FormulaInfo objects stored in the CalcQuick object and recomputes
        /// any formulas.
        /// </remarks>
        this.refreshAllCalculations = function () {
            if (!this.getAutoCalc()) {
                return;
            }

            this.setDirty();

            this.ignoreChanges = true;

            for (var keyIndex = 0; keyIndex < this.getDataStore().keys().length; keyIndex++) {
                var key = this.getDataStore().keys()[keyIndex];
                var fInfo = this.getDataStore().getItem(key);
                var s = fInfo.getFormulaText();
                if (s.length > 0 && s[0] == this.getEngine().getFormulaCharacter() && fInfo.calcID != this.getEngine().getCalcID()) {
                    s = s.substring(1); ////strip out formulaChar.
                    this.getEngine().cell = this._cellPrefix+ this.getKeyToRowsMap().getItem(key).toString();
                    fInfo.setParsedFormula(this.getEngine().parseFormula(this._markKeys(s)));
                    fInfo.setFormulaValue(this.getEngine().computeFormula(fInfo.getParsedFormula()));
                    fInfo.calcID = this.getEngine().getCalcID();
                    var rowNumber = parseInt(this.getKeyToRowsMap().getItem(key));
                    this.getEngine().valueChanged(this, new ValueChangedArgs(rowNumber, 1, fInfo.getFormulaValue()));
                }
                if (this._valueSetEvent != null) {
                    this._valueSetEvent.trigger(this, new ValueSetEventArgs(key, fInfo.getFormulaValue(), 2 /* CalculatedValueSet */));
                }
            }

            this.ignoreChanges = false;
        };
        /// <summary>
        /// A method to force all calculations to be performed the next time the CalcQuick object is
        /// accessed with an indexer requesting the value.
        /// </summary>
        /// <remarks>
        /// Each FormulaInfo object contained in the CalcQuick instance
        /// has a calculation index that is checked any time the computed value is needed. If this index
        /// is current, no calculation is done, and the last computed value is returned. If this index
        /// is not current, the calculation is redone before the value is returned. Calling this method
        /// guarantees that no FormulaInfo object's calculation indexes will be current.
        /// </remarks>
        this.setDirty = function () {
            this.getEngine().updateCalcID();
        };
        /// <summary>
        /// A method to set value to the specified cell. For internal CalcQuick use only.
        /// </summary>
        this.setValueRowCol = function (value, row, col) {
            ////No implementation code in this class.
            //// TODO:  Add CalcQuick.SetValueRowCol implementation
        };
        /// <summary>
        /// Loops through and updates all formula items that depend
        /// on the FormulaInfo object pointed to by the key.
        /// </summary>
        /// <param name="key">Identifies FormulaInfo object that triggered the update.</param>
        this.updateDependencies = function (key) {
            if (this.getAutoCalc()) {
                var s = this._cellPrefix+ this.getKeyToRowsMap().getItem(key).toString();
                var ht = this.getEngine().getDependentCells().getItem(s);
                this.setDirty();
                if (ht != null) {
                    for (var ind = 0; ind < ht.length; ind++) {
                        var i = ht[ind].indexOf('A');
                        if (i > -1) {
                            i = parseInt(ht[ind].substring(i + 1));
                            key = this.getRowsToKeyMap().getItem(i).toString();
                            this.ignoreChanges = true;
                            this.setKeyValue(key, this.getKeyValue(key)); ////triggers calculation in the getter
                            this.ignoreChanges = false;
                        }
                    }
                }
            }
        };
        this.initCalcQuick(resetStaticMembers);
    }
        /// <summary>
        /// A property that gets/sets the auto calculation mode of the CalcQuick.
        /// </summary>
        /// <remarks>
        /// By default, the CalcQuick will not update other values when you change
        /// a FormulaInfo object. By default, you explicitly call SetDirty()
        /// of the CalcQuick instance to force calculations to be done the next time
        /// they are required. Setting AutoCalc to True tells the CalcQuick to maintain
        /// the dependency information necessary to automatically update
        /// dependent formulas when values that affect these formulas change.
        /// </remarks>
    CalcQuickBase.prototype.getAutoCalc= function () {
        return this._autoCalc;
    };
    CalcQuickBase.prototype.setAutoCalc =  function (value) {
        this._autoCalc = value;
        this.getEngine().setCalculatingSuspended(!value);

        //this.getEngine().setIgnoreValueChanged(!value);
        this.getEngine().setUseDependencies(value);
        if (value) {
            this.setDirty();
        }
    };

        /// <summary>
        /// Get the calaQuick id of object.
        /// </summary>
    CalcQuickBase.prototype.getCalcQuickID= function () {
        this._calcQuickID++;
        if (this._calcQuickID == Number.MAX_VALUE) {
            this._calcQuickID = 1;
        }

        return this._calcQuickID;
    };

        /// <summary>
        /// Gets or sets whether formulas should be checked for syntax during key substitutions. Default is true.
        /// </summary>
    CalcQuickBase.prototype.getCheckKeys = function () {
        return this._checkKeys;
    };
    CalcQuickBase.prototype.setCheckKeys = function (value) {
        this._checkKeys = value;
    };

        /// <summary>
        /// Maintains a set of modified flags indicating whether
        /// any control has had a value changed.
        /// </summary>
    CalcQuickBase.prototype.getControlModifiedFlags = function () {
        return this._controlModifiedFlags;
    };

        /// <summary>
        /// Maintains a collection of FormulaInfo objects.
        /// </summary>
        /// <remarks>
        ///  This Hashtable serves as the data store for the
        ///  CalcQuick instance. The keys are the strings used
        ///  to identify formulas and the values are FormulaInfo
        ///  objects that hold the information on each formula or value.
        /// </remarks>
    CalcQuickBase.prototype.getDataStore = function () {
        return this._dataStore;
    };

        /// <summary>
        /// Determines whether the CalcEngine object of this CalcQuick should be disposed on disposing this object.
        /// <para/>Default value is true.
        /// </summary>
    CalcQuickBase.prototype.getDisposeEngineResource = function () {
        return this._disposeEngineResource;
    };
    CalcQuickBase.prototype.setDisposeEngineResource = function (value) {
        this._disposeEngineResource = value;
    };
    
        /// <summary>
        /// A read-only property that gets the reference to the CalcEngine object being used by this CalcQuick instance.
        /// </summary>
    CalcQuickBase.prototype.getEngine = function () {
        return this._engine;
    };

        /// <summary>
        /// A property that gets/sets character by which string starts with, are treated as
        /// formulas when indexing a CalcQuick object.
        /// </summary>
        /// <remarks>If you use the technique of indexing the CalcQuick object
        /// to set a varaible value, then you indicate that the value should be a
        /// formula by starting the string with this character. If you do not want
        /// to require your formulas to start with this character, then you will not
        /// be able to use the indexing technique. Instead, you will have to call
        /// ParseAndCompute directly to handle formulas not starting with this
        /// character.</remarks>
    CalcQuickBase.prototype.getFormulaCharacter = function () {
        return this.getEngine().getFormulaCharacter();
    };
    CalcQuickBase.prototype.setFormulaCharacter = function (value) {
        this.getEngine().setFormulaCharacter(value);
    };


        /// <summary>
        /// Maintains a mapping between the string key and the row
        /// used in a CalcSheet to identify a FormulaInfo object.
        /// </summary>
    CalcQuickBase.prototype.getKeyToRowsMap = function () {
        return this._keyToRowsMap;
    };

        /// <summary>
        /// Maintains a mapping between the string key and a
        /// vector of numbers entered using a brace expression.
        /// </summary>
    CalcQuickBase.prototype.getKeyToVectors = function () {
        return this._keyToVectors;
    };

        /// <summary>
        /// Maintains a mapping between the string key and the control
        /// which is being used to identify a FormulaInfo object.
        /// </summary>
    CalcQuickBase.prototype.getNameToControlMap = function () {
        return this._nameToControlMap;
    };

        /// <summary>
        /// Maintains a mapping between the row used in a CalcSheet
        /// and the string key used to identify a FormulaInfo object.
        /// </summary>
    CalcQuickBase.prototype.getRowsToKeyMap = function () {
        return this._rowsToKeyMap;
    };

        /// <summary>
        /// Gets / sets whether the CalcQuick should throw an exception when a circular calculation is encountered.
        /// </summary>
        /// <remarks>If this property is True, the CalcQuick will throw an exception
        /// when it detects a circular calculation. If ThrowCircularException is False, then
        /// no exception is thrown and the calculation will loop recursively until Engine.MaximumRecursiveCalls
        /// is exceeded.
        /// </remarks>
    CalcQuickBase.prototype.getThrowCircularException = function () {
        return this.getEngine().getThrowCircularException();
    };
    CalcQuickBase.prototype.setThrowCircularException = function (value) {
        this.getEngine().setThrowCircularException(value);
    };

    CalcQuickBase.prototype.getValueSetEventHandler = function () {
        if (this._valueSetEvent == null) {
            this._valueSetEvent = new ValueSetEvent();
        }
        return this._valueSetEvent.getValueSet();
    };
    CalcQuickBase.prototype.setValueSetEventHandler = function (value) {
        if (this._valueSetEvent == null) {
            this._valueSetEvent = new ValueSetEvent();
        }
        this._valueSetEvent.setValueSet(value);
    };

    return CalcQuickBase;
})();

var CalcQuick = (function (_super) {
    this.__calcQuickextends(CalcQuick, _super);
    function CalcQuick() {
        _super.apply(this, arguments);
        this.registerControlArray = function (controls) {
            for (var c = 0; c < controls.length; c++) {
                this.registerControl(controls[c]);
            }

            this.setAutoCalc(true);
        };
        /// <summary>
        /// Used to register a control as a calculation object in this CalcQuick instance.
        /// </summary>
        /// <param name="c">The control to register.</param>
        /// <remarks>
        /// To reference this calculation object from another calculation in this CalcQuick
        /// object, you use the Control.Name string. The value of this calculation object is
        /// bound to the Control.Text property.
        /// </remarks>
        this.registerControl = function (c) {
            ////Subscribe once.
            if (this.getNameToControlMap().length == 0) {
                this.setValueSetEventHandler(this.calcQuickValueSet);
            }

            if (this.getControlModifiedFlags().containsKey(c.id) || this.getNameToControlMap().containsKey(c.id)) {
                throw "error";
            }

            this.getControlModifiedFlags().add(c.id, false);
            this.getNameToControlMap().add(c.id.toUpperCase(), c.id);

            this.setKeyValue(c.id, c.value);
            if (c.type == "select-one") {
                var proxy = $.proxy(this.controlTextChanged, this, this);
                c.onchange = proxy;
            } else {
                var proxy = $.proxy(this.controlTextChanged, this, this);
                if (this._isIE8)
                    c.onpropertychange = proxy;
                else
                    c.oninput = proxy;
            }
            var proxy = $.proxy(this.controlLeave, this, this);
            c.onblur = proxy;
        };
        ////Used to autoset the value back to the control.
        this.calcQuickValueSet = function (eventObj, e) {
            if (eventObj.getNameToControlMap().containsKey(e.getKey())) {
                var c = eventObj.getNameToControlMap().getItem(e.getKey());
                document.getElementById(c).value = e.getValue();
            }
        };
        ////When using controls as keys, this event is
        ////used to mark a control as modified.
        this.controlTextChanged = function (e, sender) {
            var srcElement = (e._isIE8) ? event.srcElement : sender.target;
            if (srcElement != null && !e.ignoreChanges
                && e.getControlModifiedFlags().containsKey(srcElement.id)) {
                e.getControlModifiedFlags().items[srcElement.id] = true;
            }

        };
        ////When using controls as keys, this event is
        ////used to trigger an autochange if needed.
        this.controlLeave = function (e, sender) {
            var srcElement = (e._isIE8) ? event.srcElement : sender.target;
            if (srcElement != null) {
                if (this.getControlModifiedFlags().containsKey(srcElement.id)) {
                    if (e.getControlModifiedFlags().getItem(srcElement.id)) {
                        e.setKeyValue(srcElement.id, srcElement.value); ////triggers a possible recalculate
                        e.getControlModifiedFlags().items[srcElement.id] = false;
                    }
                    ////this.getControlModifiedFlags()[c] = false;
                }
            }
        };
    }
    return CalcQuick;
})(CalcQuickBase);


var ValueSetEventArgs = (function () {
    function ValueSetEventArgs(key, value, action) {
        this._id = key;
        this._val = value;
        this._action = action;
    }
        /// <summary>
        /// The reason the event was raised.
        /// </summary>
    ValueSetEventArgs.prototype.getAction = function () {
        return this._action;
    };
    ValueSetEventArgs.prototype.setAction =  function (value) {
        this._action = value;
    };


    /// <summary>
    /// A property that gets/sets the Hashtable lookup object for the FormulaInfo object being changed.
    /// </summary>
    ValueSetEventArgs.prototype.getKey = function () {
        return this._id;
    };
    ValueSetEventArgs.prototype.setKey = function (value) {
        this._id = value;
    };

        /// <summary>
        /// A property that gets/sets the new value being set.
        /// </summary>
    ValueSetEventArgs.prototype.getValue = function () {
        return this._val;
    };
    ValueSetEventArgs.prototype.setValue = function (value) {
        this._val = value;
    };

    return ValueSetEventArgs;
})();

var FormulaInfoSetAction;
(function (FormulaInfoSetAction) {
    /// <summary>
    /// A formula (string starting with FormulaCharacter) was assigned.
    /// </summary>
    FormulaInfoSetAction[FormulaInfoSetAction["FormulaSet"] = 0] = "FormulaSet";

    /// <summary>
    /// Something other than a formula was assigned.
    /// </summary>
    FormulaInfoSetAction[FormulaInfoSetAction["NonFormulaSet"] = 1] = "NonFormulaSet";

    /// <summary>
    /// A calculated value was assigned.
    /// </summary>
    FormulaInfoSetAction[FormulaInfoSetAction["CalculatedValueSet"] = 2] = "CalculatedValueSet";
})(FormulaInfoSetAction || (FormulaInfoSetAction = {}));
;


var FormulaInfoHashtable = (function () {

    function FormulaInfoHashtable() {
        this.get = function (key) {
            return new FormulaInfo();
        };
        this.set = function (key, value) {
            return new FormulaInfo();
        };
    }
    return FormulaInfoHashtable;
})();
var HashTable = (function () {
    function HashTable() {
        this.length = 0;
        this.items = [];
        this.add = function (key, value) {
            this.previous = undefined;
            if (this.containsKey(key)) {
                this.previous = this.items[key];
            } else {
                this.length++;
            }
            this.items[key] = value;
            return this.previous;
        };
        this.clear = function () {
            this.items = {};
            this.length = 0;
        };
        this.contains = function (key) {
            return this.items.hasOwnProperty(key);
        };
        this.containsKey = function (key) {
            return this.items.hasOwnProperty(key);
        };
        this.containsValue = function (key) {
            return (this.items.hasOwnProperty(key) && this.items[key] != undefined) ? true : false;
        };
        this.getItem = function (key) {
            if (this.containsKey(key))
            {

                return this.items[key]
            }
            else
            {
                return  undefined;
            }
        };
        this.keys = function () {
            var keys = [];
            for (var k in this.items) {
                if (this.containsKey(k)) {
                    keys.push(k);
                }
            }
            return keys;
        };
        this.remove = function (key) {
            if (this.containsKey(key)) {
                this.previous = this.items[key];
                this.length--;
                delete this.items[key];
                return this.previous;
            } else {
                return undefined;
            }
        };
        this.values = function () {
            var values = [];
            for (var k in this.items) {
                if (this.containsKey(k)) {
                    values.push(this.items[k]);
                }
            }
            return values;
        };
        this.each = function (fn) {
            for (var k in this.items) {
                if (this.containsKey(k)) {
                    fn(k, this.items[k]);
                }
            }
        };
        var previous = undefined;
    }
    return HashTable;
})();

var RangeInfo = (function () {
    function RangeInfo(top, left, bottom,right) {
        this.top = top;
        this.bottom = bottom;
        this.left = left;
        this.right = right;
        this.getBottom = function () {
            return this.bottom;
        };
        this.setBottom = function (value) {
            this.bottom = value;
        };
        this.getTop = function () {
            return this.top;
        };
        this.setTop = function (value) {
            this.top = value;
        };
        this.getLeft = function () {
            return this.left;
        };
        this.setLeft = function (value) {
            this.left = value;
        };
        this.getRight = function () {
            return this.right;
        };
        this.setRight = function (value) {
            this.right = value;
        };
    }
    RangeInfo.cells = function (top, left, bottom, right) {
        return new RangeInfo(top, left, bottom,right);
    };

    RangeInfo.getAlphaLabel = function (col) {
        var cols = [];
        var n = 0;
        var charText = "A";
        while (col > 0 && n < 9) {
            col--;
            var aCharNo = charText.charCodeAt(0);
            cols[n] = String.fromCharCode(col % 26 + aCharNo);
            col = parseInt((col / 26).toString());
            n++;
        }

        var chs = [];
        for (var i = 0; i < n; i++) {
            chs[n - i - 1] = cols[i];
        }

        return chs.join("");
    };
    return RangeInfo;
})();

var GridSheetFamilyItem = (function () {
    function GridSheetFamilyItem() {
        this.isSheetMember = false;
        this.parentObjectToToken = null;
        this.sheetDependentCells = null;
        this.sheetDependentFormulaCells = null;
        this.sheetFormulaInfoTable = null;
        /// <summary>
        /// Holds mapping from sheet name to parent object.
        /// </summary>
        this.sheetNameToParentObject = null;
        /// <summary>
        /// Holds mapping from parent object to sheet name.
        /// </summary>
        this.sheetNameToToken = null;
        /// <summary>
        /// Holds mapping from sheet token to parent object.
        /// </summary>
        this.tokenToParentObject = null;
    }
    return GridSheetFamilyItem;
})();

var FormulaInfo = (function () {
    function FormulaInfo() {
        this.calcID = Number.MIN_SAFE_INTEGER + 1;
        /// <summary>
        /// A property that gets/sets the text of the initial formula.
        /// </summary>
        this.getFormulaText = function () {
            return this._formulaText;
        };
        this.setFormulaText = function (value) {
            this._formulaText = value;
        };
        /// <summary>
        /// A property that gets/sets the last computed value of this formula.
        /// </summary>
        this.getFormulaValue = function () {
            return this._formulaValue;
        };
        this.setFormulaValue = function (value) {
            this._formulaValue = value;
        };
        /// <summary>
        /// A property that gets/sets the formula as a parsed string.
        /// </summary>
        this.getParsedFormula = function () {
            return this._parsedFormula;
        };
        this.setParsedFormula = function (value) {
            this._parsedFormula = value;
        };
        var _formulaText;
        var _formulaValue;
        var _parsedFormula;
        var _calcID = Number.MIN_SAFE_INTEGER + 1;
    }
    return FormulaInfo;
})();

var ValueChangedArgs = (function () {
    function ValueChangedArgs(row, col, value) {
        this.row = row;
        this.col = col;
        this.value = value;
        this.getRowIndex = function () {
            return this.row;
        };
        this.setRowIndex = function (value) {
            this.row = value;
        };
        this.getColIndex = function () {
            return this.col;
        };
        this.setColIndex = function (value) {
            this.col = value;
        };
        this.getValue = function () {
            return this.value;
        };
        this.setValue = function (value) {
            this.value = value;
        };
    }
    return ValueChangedArgs;
})();

var FormulaParsing = (function () {
    function FormulaParsing(_text) {
        this._text = _text;
        this.getText = function () {
            return this._text;
        };
        this.setText = function (value) {
            this._text = value;
        };
    }
    return FormulaParsing;
})();

var UnknownFunctionEventArgs = (function () {
    function UnknownFunctionEventArgs() {
        /// <summary>
        /// Gets the name of the unknown function.
        /// </summary>
        this.getMissingFunctionName = function () {
            return this.m_missingFunctionName;
        };
        this.setMissingFunctionName = function (value) {
            this.m_missingFunctionName = value;
        };
        this.getCellLocation = function () {
            return this.m_cellLocation;
        };
        this.setCellLocation = function (value) {
            this.m_cellLocation = value;
        };
    }
    return UnknownFunctionEventArgs;
})();

var LookUps = (function () {
    function LookUps() {
        this.getLinearLookUpList = function () {
            return this._linearLookUpList;
        };
        this.setLinearLookUpList = function (value) {
            this._linearLookUpList = value;
        };
        this.getMatchLookUpList = function () {
            return this._matchLookUpList;
        };
        this.setMatchLookUpList = function (value) {
            this._matchLookUpList = value;
        };
    }
    return LookUps;
})();

var ValueSetEvent = (function () {
    // Private member vars
    function ValueSetEvent() {
    }
    ValueSetEvent.prototype.getValueSet = function () {
        return this.eventFn;
    };
    ValueSetEvent.prototype.setValueSet = function (value) {
        this.eventFn = value;
    };

    ValueSetEvent.prototype.trigger = function () {
        var a = [];
        for (var _i = 0; _i < (arguments.length - 0) ; _i++) {
            a[_i] = arguments[_i + 0];
        }
        /// <summary>Invokes all of the listeners for this event.</summary>
        /// <param name="args">Optional set of arguments to pass to listners.</param>
        var context = {};
        this.getValueSet().apply(context, a || []);
    };
    return ValueSetEvent;
})();;
/**
* @fileOverview Plugin to style the Html CheckBox elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejCheckBox", "ej.CheckBox", {
        _rootCSS: "e-checkbox",

        element: null,

        model: null,
        validTags: ["input"],
        _setFirst: false,
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },


        defaults: {

            id: null,

            name: null,

            value: null,

            htmlAttributes: {},

            checked: false,

            enabled: true,

            enableTriState: false,

            showRoundedCorner: false,

            enablePersistence: false,

            cssClass: "",

            text: "",

            enableRTL: false,

            idPrefix: "ej",

            size: "small",

            checkState: null,

            validationRules: null,

            validationMessage: null,

            beforeChange: null,

            change: null,

            create: null,

            destroy: null

        },

        dataTypes: {
            id: "string",
            name: "string",
            enablePersistence: "boolean",
            enableTriState: "boolean",
            size: "enum",
            checkState: "enum",
            enabled: "boolean",
            idPrefix: "string",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data"
        },
        observables: ["checked"],
        checked: ej.util.valueFunction("checked"),

        _init: function () {
            var browserInfo = ej.browserInfo();
            this._isIE8 = (browserInfo.name == 'msie' && browserInfo.version == '8.0') ? true : false;
            this._setValue();
            this._renderControl();
            this.model.enableRTL && this._setRTL();
            if (this.model.enabled)
                this._wireEvents();
            this._setEnabled(this.model.enabled);
            if (this.model.validationRules != null) {
                this._initValidator();
                this._setValidation();
            }
            this._addAttr(this.model.htmlAttributes);            
        },
        _setRTL:function(){
            this.maindiv.addClass("e-rtl");
        },
        _initValidator: function () {
            var proxy = this;
            this.wrapper.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $container = element.closest(".e-widget").next().is("label") ? element.closest(".e-widget").next() : element.closest(".e-widget");
                    $(error).insertAfter($container);
                },
            });
        },
        _setValidation: function () {
            this.wrapper.find('input').rules("add", this.model.validationRules);
            var validator = this.wrapper.closest("form").validate();
            name = this.wrapper.find('input').attr("name");
            validator.settings.messages[name] = {};
            for (var ruleName in this.model.validationRules) {
                var message = null;
                if (!ej.isNullOrUndefined(this.model.validationRules[ruleName])) {
                    if (!ej.isNullOrUndefined(this.model.validationRules["messages"] && this.model.validationRules["messages"][ruleName]))
                        message = this.model.validationRules["messages"][ruleName];
                    else {
                        validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                        for (var msgName in this.model.validationMessage)
                            ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                    }
                    validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                }
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy._setEnabled(false);
                else if (key == "checked" && value == "checked") {
                    if (proxy.checked() instanceof Array)
                        proxy._updateCheckedItem();
                    else
                        proxy._checked(true);
                }
                else proxy.wrapper.attr(key, value);
            });
        },

        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "checked":
                        if (ej.util.getVal(options[prop]) instanceof Array) {
                            var checkedItemArray = ej.util.getVal(options[prop]);
                            var lastVal = checkedItemArray[checkedItemArray.length - 1];
                            if (this.model.value == lastVal) {
                                this._checked();
                                this._isChecked = true;
                            }
                        }
                        else {
                            this._isChecked = ej.util.getVal(options[prop]);
                            if (this._isChecked)
                                this._checked();
                            else
                                this._unChecked();
                            this.checked(ej.util.getVal(options[prop]));
                        }
                        break;
                    case "enableTriState":
                        if (options[prop]) {
                            this._indeterminateState = options[prop];
                            this._setIndeterminate(this._indeterminateState);
                        }
                        break;
                    case "checkState":
                        if (this.model.enableTriState) {
                            this.model.checkState = options[prop];
                            this._changeState(this.model.checkState);
                            if (this.checked() instanceof Array)
                                this._updateCheckedItem();
                            else if (options[prop] = "check") this._hiddenInput.removeAttr("name")
                            else this._hiddenInput.attr("name", this.model.name);
                        }
                        break;
                    case "cssClass": this._changeSkin(options[prop]); break;
                    case "enableRTL":
                        (options[prop]) ? this._setRTL() : this.maindiv.removeClass("e-rtl");
                        break;
                    case "text": this._setText(options[prop]); break;
                    case "validationRules":
                        if (this.model.validationRules != null) {
                            this.wrapper.find('input').rules('remove');
                            this.model.validationMessage = null;
                        }
                        this.model.validationRules = options[prop];
                        this._setValidation();
                        break;
                    case "validationMessage":
                        this.model.validationMessage = options[prop];
                        this._setValidation(); break;
                    case "id": this._setIdAttr(options[prop]); break;
                    case "name": this.element.attr('name', options[prop]);
                        if (!this._isChecked || this.spanImg.hasClass("e-chk-indeter")) this._hiddenInput.attr('name', options[prop]);
                        this.model.name = options[prop];
                        break;
                    case "value": this.element.attr('value', options[prop]); break;
                    case "size": this._setSize(options[prop]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[prop]); break;
                    case "enabled": this._setEnabled(options[prop]); break;
                    case "htmlAttributes": this._addAttr(options[prop]); break;
                }
            }
        },

        _destroy: function () {
            this.element.removeClass("e-checkbox e-input");
            this.element.insertBefore(this.wrapper);
            this.wrapper.remove();
        },

        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.wrapper.removeClass(this.model.cssClass).addClass(skin);
                $("#" + this.model.idPrefix + this.model.id + "_wrapper").removeClass(this.model.cssClass).addClass(skin);
            }
        },

        _setValue: function () {
            this._indeterminateState = false;
            this._isChecked = false;
            !ej.isNullOrUndefined(this.element.attr("id")) && (this.model.id = this.element.attr('id'));
            !ej.isNullOrUndefined(this.element.attr('name')) && (this.model.name = this.element.attr('name'));
            if (!ej.isNullOrUndefined(this.element.attr('value')) && this.element.attr('value') != "") this.model.value = this.element.attr('value');
            if (!this.checked() && !ej.isNullOrUndefined(this.element.attr('checked'))) this._isChecked = true;
            ej.isNullOrUndefined(this.model.name) && (this.model.name = this.model.id);
        },

        _setSize: function (val) {
            if (val == ej.CheckboxSize.Medium)
                this.innerdiv.removeClass('e-chkbox-small').addClass('e-chkbox-medium');
            else
                this.innerdiv.removeClass('e-chkbox-medium').addClass('e-chkbox-small');
        },

        _setRoundedCorner: function (val) {
            if (val)
                this.span.addClass("e-corner");
            else
                this.span.removeClass("e-corner");
        },

        _setEnabled: function (val) {
            if (val) {
                this.enable();
            } else {
                this.disable();
            }
        },
        _setCheckBoxState: function () {
            if (this.model.enableTriState) {
                if (this.model.checkState == "indeterminate")
                    this._indeterminateState = true
                else if (this.model.checkState == "check")
                    this._isChecked = true;
                else if (this.model.checkState == "uncheck")
                    this._isChecked = false;
            }
        },

        _renderControl: function () {
            this._setCheckBoxState();
            this.maindiv = ej.buildTag("span.e-chkbox-wrap e-widget " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id, "", {}, { "role": "checkbox", tabindex: 0 });
            this.innerdiv = $('<div></div>');
            this._setSize(this.model.size);
            this.span = $('<span></span>');
            this.spanImg = $('<span class="e-chk-image e-icon"></span>').attr("role", "presentation");
            this.element.addClass("e-input");
            this.model.name = ej.isNullOrUndefined(this.model.name) ? this.model.id : this.model.name;
            this.model.value = ej.isNullOrUndefined(this.model.value) ? true : this.model.value;
            this.element.attr({ "id": this.model.id, "name": this.model.name, "value": this.model.value });
            this._hiddenInput = ej.buildTag("input#" + this.model.name + "_hidden", "", {}, { type: "hidden", value: false });

            this._setRoundedCorner(this.model.showRoundedCorner);
            if (this.checked())
                this._setCheckedItem(this.checked());
            if (this._isChecked) {
                this.spanImg.addClass("e-chk-activeicon");
                this.span.addClass("e-chk-act");
                this.maindiv.attr({ "aria-checked": true });
            }
            else {
                this.span.addClass("e-chk-inact");
                this.maindiv.attr({ "aria-checked": false });
                this._hiddenInput.attr("name", this.model.name);
            }
            if (!(this.checked() instanceof Array))
                this.checked(this._isChecked);
            this.span.append(this.spanImg);
            this.innerdiv.append(this.span);
            this.maindiv.insertBefore(this.element);
            this.maindiv.append(this.element);
			this.maindiv.append(this._hiddenInput);
            this.maindiv.append(this.innerdiv);
			this.wrapper = this.maindiv;
            this._setTextWrapper(this.model.text);
            this.chkbx = this.element;
            if (this.model.enableTriState == true && this._indeterminateState == true)
                this._setIndeterminate(this._indeterminateState);
            if (this.checked() instanceof Array)
                this._updateCheckedItem();
        },
        _changeState: function (state) {
            if (state == "indeterminate") {
                this.spanImg.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-inact").addClass("e-chk-indeter");
                this.wrapper.attr({ "aria-checked": "mixed" });
                this.wrapper.find('input').prop('enableTriState', true);
                if (!(this.checked() instanceof Array))
                    this.checked(null);
            }
            else if (state == "check") {
                this.spanImg.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-inact").removeClass("e-chk-indeter").addClass("e-chk-act");
                this.wrapper.attr({ "aria-checked": true });
            }
            else if (state == "uncheck") {
                this.spanImg.removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-indeter").addClass("e-chk-inact");
                this.wrapper.attr({ "aria-checked": false });
            }
        },

        _setIndeterminate: function (indeter) {
            if (indeter) {
                this.spanImg.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this.span.removeClass("e-chk-act").removeClass("e-chk-inact").addClass("e-chk-indeter");
                this.wrapper.attr({ "aria-checked": "mixed" });
                this.wrapper.find('input').prop('enableTriState', true);
                this.model.checkState = "indeterminate";
                if (!(this.checked() instanceof Array))
                    this.checked(null);
                this._hiddenInput.attr("name", this.model.name);
            }
            else {
                this.wrapper.find(".e-chk-indeter").removeClass("e-chk-indeter");
                this.wrapper.find(".e-chk-image").removeClass("e-chk-indetericon");
                this.wrapper.find('input').removeAttr('enableTriState');
                this.wrapper.find('input').prop('enableTriState', false);
                if (this.checked())
                    this._checked();
                else
                    this._unChecked();
            }
        },

        _setTextWrapper: function (val) {
            if (val != "") {
                this.txtSpan = ej.buildTag("div.e-text", val);
                this.wrapper.append(this.txtSpan);
                this.model.enableRTL && this._setRTL();
            }
        },

        _setText: function (val) {
            if ((this.model.text == "") && (val != "")) {
                this._setTextWrapper(val);
            } else {
                this.txtSpan.html(val);
            }
        },

        _setIdAttr: function (val) {
            $("#" + this.model.idPrefix + this.model.id + "_wrapper").attr('id', this.model.idPrefix + val + "_wrapper");
            this.element.attr('id', val);
        },

        _wireEvents: function () {
            this._on(this.wrapper, "click", this._checkedHandler);
            if (this._isIE8) {
                this._on($("label[for=" + this.model.id + "]"), "click", function () { this.wrapper.click(); });
            }
            this._on(this.wrapper, "focus", this._focusIn);
            this._on(this.wrapper, "focusout", this._focusOut);
        },


        _unWireEvents: function () {
            this._off(this.wrapper, "click");
            if (this._isIE8) {
                this._off($("label[for=" + this.model.id + "]"), "click");
            }
            this._off(this.wrapper, "focus");
            this._off(this.wrapper, "focusout");
        },
        _focusIn: function (evt) {
            $(this.wrapper).addClass("e-focus");
            $(this.wrapper).bind("keydown", $.proxy(this._checkUnCheck, this));
        },
        _focusOut: function (evt) {
            $(this.wrapper).removeClass("e-focus");
            $(this.wrapper).unbind("keydown", $.proxy(this._checkUnCheck, this));
        },
        _checkUnCheck: function (evt) {
            //Space bar to check and uncheck
            if (evt.keyCode == 32) {
                evt.preventDefault();
                this._checkedHandler();
            }
        },
        _checkedHandler: function (evt) {
            var data = { isChecked: this._isChecked, event: evt };
            if (true == this._trigger("beforeChange", data)) {
                return false;
            }
            if (this.wrapper.find("span:first").hasClass("e-chk-inact")) {
                this._checked();
                if (!(this.checked() instanceof Array))
                    this.checked(true);
                if (this.model.enableTriState) {
                    this._indeterminateState = true;
                }
            }
            else if (this.wrapper.find("span:first").hasClass("e-chk-act")) {
                if ((this.model.enableTriState == true) && (this._indeterminateState == true)) {
                    this._setIndeterminate(true);
                    if (!(this.checked() instanceof Array))
                        this.checked(true);
                } else {
                    this._unChecked();
                    if (!(this.checked() instanceof Array))
                        this.checked(false);
                }
            }
            else if (this.wrapper.find("span:first").hasClass("e-chk-indeter")) {
                if (!(this.checked() instanceof Array))
                    this.checked(false);
                else
                    this._isChecked = false;
                this._setIndeterminate(false);
                this._indeterminateState = false;
            }
            if (this.checked() instanceof Array)
                this._updateCheckedItem();
            else
                this._isChecked = this.checked();
            var data = { isChecked: this._isChecked,checkState:this.model.checkState,event: evt };
            this._trigger("change", data);
            return true;
        },


        _checked: function () {
            this.wrapper.find("span:first").removeClass("e-chk-inact").addClass("e-chk-act");
            this.wrapper.find(".e-chk-image").removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
            this.wrapper.attr({ "aria-checked": true });
            this.wrapper.find('input[type=checkbox]').prop('checked', true);
            this.model.checkState = "check";
            this._hiddenInput.removeAttr("name");
        },


        _unChecked: function () {
            this.wrapper.find("span:first").removeClass("e-chk-act").addClass("e-chk-inact");
            this.wrapper.attr({ "aria-checked": false });
            this.wrapper.find(".e-chk-image").removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
            this.wrapper.find('input[type=checkbox]').removeAttr('checked');
            this.model.checkState = "uncheck";
            this._hiddenInput.attr("name", this.model.name);
        },

        _setCheckedItem: function (value) {
            if (!ej.isNullOrUndefined(value) && value != "" && !(value instanceof Array))
                this._isChecked = true;
            else if (value instanceof Array && !ej.isNullOrUndefined(this.model.value) && this.model.value != "") {
                for (var item = 0; item < value.length; item++) {
                    if (value[item] == this.model.value)
                        this._isChecked = true;
                }
            }
        },

        _updateCheckedItem: function () {
            if (!ej.isNullOrUndefined(this.model.value) && this.model.value != "" && !this.wrapper.find("span:first").hasClass("e-chk-indeter")) {
                if (($.inArray(this.model.value, this.checked()) < 0) && this.wrapper.find("span:first").hasClass("e-chk-act")) {
                    this.checked().push(this.model.value);
                    this._isChecked = true;
                    this._hiddenInput.removeAttr("name");
                }
                else if (($.inArray(this.model.value, this.checked()) > -1) && this.wrapper.find("span:first").hasClass("e-chk-inact")) {
                    this.checked().splice($.inArray(this.model.value, this.checked()), 1);
                    this._isChecked = false;
                    this._hiddenInput.attr("name", this.model.name);
                }
            }
        },


        disable: function () {
            if (!this.wrapper.hasClass("e-disable")) {
                this.wrapper.addClass("e-disable").attr("aria-disabled", true);
				this.element.attr("disabled","disabled");
                if (this._isIE8) this.span.addClass("e-disable");
                this._unWireEvents();
                this.model.enabled = false;
            }
        },

        enable: function () {
            if (this.wrapper.hasClass("e-disable")) {
                this.wrapper.removeClass("e-disable").attr("aria-disabled", false);
				this.element.removeAttr("disabled");
                if (this._isIE8) this.span.removeClass("e-disable");
                this._wireEvents();
                this.model.enabled = true;
            }
        },

        isChecked: function () {
            if ((this._isChecked != null) && (this._isChecked != undefined))
                return this._isChecked;
        }
    });

    ej.CheckboxSize = {
        /**  Creates checkbox with inbuilt small size height, width specified */
        Small: "small",
        /**  Creates checkbox with inbuilt medium size height, width specified */
        Medium: "medium"
    };

    ej.CheckState = {
        /**  Specifies the Check attribute of the Checkbox */
        Check: "check",
        /**  Specifies the Uncheck attribute of the Checkbox */
        Uncheck: "uncheck",
        /**  Specifies the Indeterminate state of the Checkbox */
        Indeterminate: "indeterminate"
    };
})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Html Radiobutton elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejRadioButton", "ej.RadioButton", {
        _rootCSS: "e-radiobtn",

        element: null,

        model: null,
        validTags: ["input"],
        _setFirst: false,
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },

        defaults: {

            id: null,

            name: null,

            value: null,

            checked: false,

            cssClass: "",

            text: "",

            enableRTL: false,

            htmlAttributes: {},

            enablePersistence: false,

            idPrefix: "ej",

            size: "small",

            enabled: true,

            validationRules: null,

            validationMessage: null,

            beforeChange: null,

            change: null,

            create: null,

            destroy: null
        },


        dataTypes: {
            id: "string",
            name: "string",
            checked: "boolean",
            enablePersistence: "boolean",
            size: "enum",
            enabled: "boolean",
            idPrefix: "string",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data"
        },


        _init: function () {
            var browserInfo = ej.browserInfo();
            this._isIE8 = (browserInfo.name == 'msie' && browserInfo.version == '8.0') ? true : false;
            this._setValue();
            this._renderControl();
            if (this.isChecked)
                this._checkedHandler();
            this._setEnabled(this.model.enabled);
            this._addAttr(this.model.htmlAttributes);
            if (this.model.validationRules != null) {
                this._initValidator();
                this._setValidation();
            }
            this._wireEvents();
            this.initialRender = false;
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.div.addClass(value);
                else if(key == "name") proxy.radbtn.attr(key, value);
                else if (key == "required") proxy.radbtn.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "checked" && value == "checked") proxy._checkedChange(true);
                else proxy.div.attr(key, value);
            });
        },

        _initValidator: function () {
            var proxy = this;
            this.element.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $container = element.closest(".e-widget").next().is("label") ? element.closest(".e-widget").next() : element.closest(".e-widget");
                    $(error).insertAfter($container);
                },
            });
        },
        _setValidation: function () {
            this.element.find("input").rules("add", this.model.validationRules);
            var validator = this.element.closest("form").validate();
            name = this.element.find('input').attr("name");
            validator.settings.messages[name] = {};
            for (var ruleName in this.model.validationRules) {
                var message = null;
                if (!ej.isNullOrUndefined(this.model.validationRules[ruleName])) {
                    if (!ej.isNullOrUndefined(this.model.validationRules["messages"] && this.model.validationRules["messages"][ruleName]))
                        message = this.model.validationRules["messages"][ruleName];
                    else {
                        validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                        for (var msgName in this.model.validationMessage)
                            ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                    }
                    validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                }
            }
        },


        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "cssClass": this._changeSkin(options[prop]); break;
                    case "enableRTL":
                        if (this.model.text)
                            (options[prop]) ? this.textWrapDiv.addClass("e-rtl") : this.textWrapDiv.removeClass("e-rtl");
                        else 
	                    	(options[prop]) ? this.element.closest('.e-radiobtn-wrap').addClass('e-rtl') : this.element.closest('.e-radiobtn-wrap').removeClass('e-rtl');
                        break;
                    case "text": this._setText(options[prop]); break;
                    case "size": this._setSize(options[prop]); break;
                    case "validationRules":
                        if (this.model.validationRules != null) {
                            this.element.find('input').rules('remove');
                            this.model.validationMessage = null;
                        }
                        this.model.validationRules = options[prop];
                        this._setValidation();
                        break;
                    case "validationMessage":
                        this.model.validationMessage = options[prop];
                        this._setValidation(); break;
                    case "checked": this.model.checked = options[prop]; this._checkedChange(this.model.checked); break;
                    case "enabled": this._setEnabled(options[prop]); break;
                    case "id": this._setIdAttr(options[prop]); break;
                    case "name": this.radbtn.attr('name', options[prop]); break;
                    case "value": this.radbtn.attr('value', options[prop]); break;
                    case "htmlAttributes": this._addAttr(options[prop]); break;
                }
            }
        },

        _destroy: function () {
            this.radbtn.removeClass("e-radiobtn e-input");
            this.radbtn.insertBefore(this.element);
            this.element.remove();
            this.element = this.radbtn;
        },

        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
                this.textWrapDiv.removeClass(this.model.cssClass).addClass(skin);
            }
        },

        _setValue: function () {
            if (!ej.isNullOrUndefined(this.element.attr("id")))
                this.model.id = this.element.attr("id");
            if (!ej.isNullOrUndefined(this.element.attr("name")))
                this.model.name = this.element.attr("name");
            if (!ej.isNullOrUndefined(this.element.attr("value")))
                this.model.value = this.element.attr("value");
            this.element.attr({ "id": this.model.id, "name": this.model.name, "value": this.model.value });
            if (!this.model.checked && !ej.isNullOrUndefined(this.element.attr('checked')))
                this.model.checked = this.isChecked = true;
            else
                this.isChecked = this.model.checked;
        },

        _setIdAttr: function (val) {
            $("#" + this.model.idPrefix + this.model.id + "_wrapper").attr('id', this.model.idPrefix + val + "_wrapper");
            $("#" + this.model.idPrefix + this.model.id).attr('id', this.model.idPrefix + val);
            this.radbtn.attr('id', val);
        },

        _setSize: function (val) {
            if (val == ej.RadioButtonSize.Medium)
                this.span.removeClass('e-radsmaller').addClass('e-radmedium');
            else
                this.span.removeClass('e-radmedium').addClass('e-radsmaller');
        },

        _setEnabled: function (val) {
            if (val) {
                this.enable();
            } else {
                this.disable();
            }
        },

        _renderControl: function () {
            this.initialRender = true;
            var browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie' && browserInfo.version == '8.0')
                this.div = $('<div class="e-radiobtn-wrap e-widget e-rad-outer" ></div>');
            else
                this.div = $('<div class="e-radiobtn-wrap e-widget" ></div>');
            this.div.attr({ 'id': this.model.idPrefix + this.model.id, "role": "radio", "tabindex": 0, "aria-checked": false });
            this.span = $('<span></span>');
            this.span.addClass("e-spanicon");
            this._setSize(this.model.size);
            this.spanImg = $('<span class="e-rad-icon e-icon e-rad-select"></span>', "", {}, { "role": "presentation" });
            this.element.addClass("e-input");
            this.model.name = ej.isNullOrUndefined(this.model.name) ? this.model.id : this.model.name;
            this.element.attr({ "id": this.model.id, "name": this.model.name, "value": this.model.value });
            this.div.addClass(this.model.cssClass);
            this.span.append(this.spanImg);
            this.div.insertBefore(this.element);
            this.div.append(this.element);
            this.div.append(this.span);
            this._setTextWrapper(this.model.text);
            this.radbtn = this.element;
            this.element = this.div;
            if (this.isChecked) {
                this.element.find(".e-input").attr('checked', true);
            }
        },

        _setTextWrapper: function (val) {
            if (val != "") {
                this.textWrapDiv = ej.buildTag("div.e-radiobtn-wrap " + this.model.cssClass + "#" + this.model.idPrefix + this.model.id + "_wrapper");
                this.div.wrapAll(this.textWrapDiv);
                this.txtSpan = ej.buildTag("div.e-text", this.model.text);
                this.textWrapDiv = $("#" + this.model.idPrefix + this.model.id + "_wrapper");
                this.textWrapDiv.append(this.txtSpan);
                if (this.model.enableRTL)
                    this.textWrapDiv.addClass("e-rtl");
            } else if (this.model.enableRTL)
                this.element.closest('.e-radiobtn-wrap').addClass('e-rtl');
        },

        _setText: function (val) {
            if ((this.model.text == "") && (val != "")) {
                this._setTextWrapper(val);
            } else {
                this.txtSpan.html(val);
            }
        },

        _wireEvents: function () {
            this._on(this.element, "click", this._checkedHandler);
            this._on(this.element, "focus", this._focusIn);
            this._on(this.element, "focusout", this._focusOut);
        },
        _focusIn: function (evt) {
            $(this.element).addClass("e-focus");
            $(this.element).bind("keydown", $.proxy(this._checkUnCheck, this));
        },
        _focusOut: function (evt) {
            $(this.element).removeClass("e-focus");
            $(this.element).unbind("keydown", $.proxy(this._checkUnCheck, this));
        },

        _checkUnCheck: function (evt) {
            //Space bar,and arrow keys to check and uncheck
            if (evt.keyCode == 32 || evt.keyCode == 37 || evt.keyCode == 38 || evt.keyCode == 39 || evt.keyCode == 40) {
                evt.preventDefault();
                this._checkedHandler();
            }
        },
        _checkedHandler: function (evt) {
            if (!this.element.hasClass('e-disable')) {
                this.isChecked = this.element.find('input.e-radiobtn:radio').attr('checked') == 'checked' ? true : false;
                if (!$(this.element).find(".e-rad-icon").hasClass("e-rad-active")) this._changeEvent();
            }
        },

        _checkedChange: function (val) {
            this.isChecked = val;
            if ((this.isChecked) && (this.model.enabled))
                this._changeEvent();
        },

        _changeEvent: function () {
            var data = { isChecked: this.isChecked };
            if (!this.initialRender) {
                if (true == this._trigger("beforeChange", data))
                    return false;
            }
            if (!$(this.element).find(".e-rad-icon").hasClass("e-rad-active")) {
                var curname = this.element.find(".e-input").attr('name'),
                input = $('input.e-radiobtn[name=' + curname + ']:radio'),
                proxy = this,
                currElement = this.element.find('.e-input'),
                currObj = $(currElement).data("ejRadioButton");
                if (data.isChecked) {
                    this.spanImg.addClass("e-rad-active").removeClass('e-rad-select');
                    this.div.attr({ "tabindex": 0, "aria-checked": true });
                }
                $.each(input, function (i, obj) {
                    $(obj).closest(".e-radiobtn-wrap").find(".e-rad-icon").removeClass("e-rad-active").addClass("e-rad-select");
                    $(obj).closest(".e-radiobtn-wrap").attr({ "tabindex": 0, "aria-checked": false });
                    var prevObj = $(obj).data("ejRadioButton");
                    if (prevObj != null) {
                        prevObj.model.checked = false;
                    }
                });
                if (currObj != null)
                    currObj.model.checked = true;
                this.element.find(".e-rad-icon").addClass("e-rad-active").removeClass("e-rad-select");
                this.div.attr({ "tabindex": 0, "aria-checked": true });
                this.element.find(".e-input").click();
                this.isChecked = true;
            }
            var data = { isChecked: this.isChecked };
            if (!this.initialRender)
                this._trigger("change", data);

        },

        disable: function () {
            if (!this.element.hasClass("e-disable")) {
                this.element.addClass("e-disable");
			this.radbtn.attr("disabled","disabled");
			}
            if (this._isIE8) this.span.addClass("e-disable");
            this.div.attr("aria-disabled", true);
            this.model.enabled = false;
        },

        enable: function () {
            if (this.element.hasClass("e-disable")) {
                this.element.removeClass("e-disable");
				this.radbtn.removeAttr("disabled");
			}
            if (this._isIE8) this.span.removeClass("e-disable");
            this.div.attr("aria-disabled", false);
            this.model.enabled = true;
        }
    });

    ej.RadioButtonSize = {
        /**  Creates radio button with inbuilt small size height, width specified */
        Small: "small",
        /**  Creates radio button with inbuilt medium size height, width specified */
        Medium: "medium"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejAutocomplete", "ej.Autocomplete", {

        element: null,

        model: null,
        validTags: ["input"],
        _ignoreOnPersist: ["fields", "dataSource", "query", "focusIn", "focusOut", "change", "select", "create", "destroy"],
        _setFirst: false,

        _rootCSS: "e-autocomplete",
        type: "editor",
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },


        defaults: {

            dataSource: null,

            query: null,

            fields: {

                text: null,

                key: null,
                //Deprecated
                category: null,

                groupBy:null,

                htmlAttributes: null
            },

            template: null,
            //Deprecated
            allowGrouping: false,

            enableDistinct: false,

            allowSorting: true,

            sortOrder: "ascending",

            htmlAttributes: {},

            multiSelectMode: "none",

            delimiterChar: ',',

            allowAddNew: false,

            addNewText: "Add New",

            showRoundedCorner: false,

            readOnly: false,

            cssClass: "",

            watermarkText: null,

            value: "",

            selectValueByKey: null,

            filterType: "startswith",

            caseSensitiveSearch: false,

            showLoadingIcon: true,

            itemsCount: 0,

            minCharacter: 1,

            delaySuggestionTimeout: 200,

            showPopupButton: false,

            highlightSearch: false,

            enableAutoFill: false,

            enableRTL: false,

            enabled: true,

            visible: true,

            height: "",

            width: "",

            emptyResultText: "No suggestions",

            animateType: "slide",

            showEmptyResultText: true,

            enablePersistence: false,

            popupHeight: "152px",

            popupWidth: "auto",

            autoFocus: false,

            validationRules: null,

            validationMessage: null,

            focusIn: null,

            focusOut: null,

            change: null,

            select: null,

            create: null,

            open: null,

            close: null,

            destroy: null,

            actionComplete: null,

            actionSuccess: null,

            actionFailure: null
        },

        dataTypes: {
            cssClass: "string",
            filterType: "enum",
            caseSensitiveSearch: "boolean",
            showLoadingIcon: "boolean",
            template: "string",
            //Deprecated
            allowGrouping: "boolean",
            enableDistinct: "boolean",
            allowSorting: "boolean",
            sortOrder: "enum",
            allowAddNew: "boolean",
            addNewText: "string",
            showRoundedCorner: "boolean",
            readOnly: "boolean",
            itemsCount: "number",
            animateType:"enum",
            minCharacter: "number",
            showPopupButton: "boolean",
            highlightSearch: "boolean",
            enableAutoFill: "boolean",
            enableRTL: "boolean",
            multiSelectMode: "enum",
            delimiterChar: "string",
            emptyResultText: "string",
            showEmptyResultText: "boolean",
            enabled: "boolean",
            visible: "boolean",
            enablePersistence: "boolean",
            dataSource: "data",
            query: "data",
            fields: "data",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data",
        },
        observables: ["value"],

        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this.target.disabled = false;
                this.element.removeClass("e-disable").attr({ "aria-disabled": false });
				this.element.removeAttr("disabled");
                if (this.model.showPopupButton) this.dropdownbutton.removeClass("e-disable").attr({ "aria-disabled": false });
                if (this.model.multiSelectMode == "visualmode") this._ulBox.removeClass("e-disable").attr({ "aria-disabled": false });
            }
        },

        disable: function () {
            if (this.model.enabled) {
                this._hideResult();
                this.model.enabled = false;
                this.target.disabled = true;
				this.element.attr("disabled","disabled");
                this.element.addClass("e-disable").attr({ "aria-disabled": true });
                if (this.model.showPopupButton) this.dropdownbutton.addClass("e-disable").attr({ "aria-disabled": true });
                if (this.model.multiSelectMode == "visualmode") this._ulBox.addClass("e-disable").attr({ "aria-disabled": true });
            }
        },

        clearText: function () {
            if (this.model.multiSelectMode == "visualmode")
                this._deleteBox(this._ulBox.children("li"));
            this.element.val("");
            this._valueChange();
            if (this._isFocused) this.element.blur();
            else this._targetBlur();
        },

        getValue: function () {
            if (this.model.multiSelectMode == "visualmode")
                return this._hiddenInput.val();
            return this.target.value;
        },

        getSelectedItems: function () {
            return this._selectedItems;
        },
        _setValue: function (value) {
            if (!this._isWatermark)
                this._hiddenSpan.css("display", "none");
            if (typeof value === "object" || (typeof value === "number" && isNaN(value)) || $.trim(value) == "") value = null;
            value = $.trim(value);
            if (this.model.multiSelectMode == "visualmode")
                this._hiddenInput.val("");
            this.element.val("");
            if (value) {
                if (this.model.multiSelectMode == "visualmode") {
                    this._selectedItems = [];
                    this._deleteBox(this._ulBox.children("li"));
                    this._hiddenInput.val(value);
                    var values = value.split(this.model.delimiterChar);

                    if (ej.DataManager && this.model.dataSource instanceof ej.DataManager) {
                        this._createBoxForObjectType(values);
                    }
                    else {
                        this.suggestionListItems = this.model.dataSource;
                        if (typeof this.suggestionListItems[0] != "object") {
                            for (var i = 0; i < values.length; i++) {
                                if (values[i]) {
                                    this._ulBox.append(this._createBox(values[i]));
                                    this._selectedItems.push(values[i]);
                                }
                            }
                        }
                        else this._createBoxForObjectType(values);
                    }
                }
                else {
                    this.element.val(value);
                    this._updateSelectedItemArray();
                }
            }
            this.model.value = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
            this._preVal = this.element.val();
            !this._isWatermark && this._setWatermarkTxt();
            return value;
        },
        _createBoxForObjectType: function (values) {
            var dataQuery, promise, mapper, strData, proxy = this;
            mapper = this.model.fields, mapFld = { _key: null, _text: null, _attr: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            for (var data = 0; data < values.length; data++) {
                var _val = $.trim(values[data]);
                if (ej.DataManager && this.model.dataSource instanceof ej.DataManager) {
                    dataQuery = this._getQuery().where(mapFld._text, "equal", _val, false);
                    promise = (this.model.dataSource).executeQuery(dataQuery);
                    promise.done(function (e) {
                        var res = e.result;
                        strData = res instanceof Array && res.length ? res[0] : _val;
                        proxy._selectedItems.push(strData);
                    }).fail(function (e) {
                        proxy._selectedItems.push(_val);
                    });
                }
                else {
                    dataQuery = ej.Query().where(mapFld._text, "equal", _val, false);
                    promise = ej.DataManager(this.suggestionListItems).executeLocal(dataQuery);
                    strData = promise instanceof Array && promise.length ? promise[0] : _val;
                    this._selectedItems.push(strData);
                }
                if (this.model.multiSelectMode == "visualmode")
                    this._ulBox.append(this._createBox(_val));
            }
        },


        selectValueByKey: function (key) {
            var proxy = this;
            key = key.toString();
            var result = key.split(",");
            if (Object.prototype.toString.call(result) === '[object Array]') {
                $.each(result, function (index, value) {
                    proxy._setOperation($.trim(value), "key");
                });
            }
        },

        selectValueByText: function (text) {
            this._setOperation(text, "text");
        },


        _setOperation: function (source, value) {
            if (!this.model.enabled) return false;
            var bindTo = "", promise, dataQuery, proxy = this, list = this.model.dataSource;
            if (ej.isNullOrUndefined(list)) return false;
            if (typeof list[0] == "object" || list instanceof ej.DataManager) {
                var mapper = this.model.fields;
                bindTo = (mapper && mapper[value]) ? mapper[value] : value;
            } else if (value == "key")
                return false;
            if (ej.DataManager && list instanceof ej.DataManager) {
                dataQuery = this._getQuery().where(bindTo, "equal", source, !this.model.caseSensitiveSearch);
                promise = (list).executeQuery(dataQuery);
                promise.done(function (e) {
                    proxy._setText(e.result[0]);
                });
            }
            else {
                if ((!list || !list.length || list.length < 1)) return false;
                dataQuery = ej.Query().where(bindTo, "equal", source, !this.model.caseSensitiveSearch);
                promise = ej.DataManager(list).executeLocal(dataQuery);
                this._setText(promise[0]);
            }
        },

        _setText: function (data) {
            if (!data) return false;
            var currentValue, mapper = this.model.fields;
            if ((typeof this.model.dataSource[0] == "object") || (typeof data == "object"))
                currentValue = (mapper && mapper["text"]) ? data[mapper["text"]] : data["text"];
            else currentValue = data;
            if (currentValue) {
                if (this.model.multiSelectMode == "visualmode" && this._removeDuplicates(data)) return false;
                this._valueToTextBox(currentValue, data, true);
                this.model.value = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
            }
        },

        _valueToTextBox: function (currentValue, data, flag) {
            var delimiterIndex;
            if (this.model.multiSelectMode == "visualmode") {
                if (this._addNewTemplate) currentValue = currentValue.substr(0, currentValue.length - this._addNewTemplate.length);
                delimiterIndex = this._hiddenInput.val() ? this.model.delimiterChar : "";
                this._hiddenInput.val(this._hiddenInput.val() + delimiterIndex + (data.key || currentValue));
                this.element.val("");
                this._ulBox.append(this._createBox(currentValue));
            }
            else if (this.model.multiSelectMode == "delimiter") {
                delimiterIndex = (this.target.value).lastIndexOf(this.model.delimiterChar)
                if (this._typed || this.element.val() == "" || flag || !this.model.showPopupButton) {
                    if (flag)
                        this.element.val(this.element.val() == "" ? (currentValue) : (this.element.val() + this.model.delimiterChar + currentValue));
                    else
                        this.element.val(this._queryString.substr(0, delimiterIndex + 1) + currentValue + this.model.delimiterChar + " ");
                }
                else {
                    if (this._checkDeli())
                        this.element.val(this._queryString.substr(0, delimiterIndex) + this.model.delimiterChar + currentValue + this.model.delimiterChar + " ");
                    else
                        this.element.val(this.element.val() + this.model.delimiterChar + currentValue + this.model.delimiterChar + " ");
                }
                this._typed = false;
            }
            else {
                this.element.val(currentValue);
                this.model.selectValueByKey = this._getField(data, this.model.fields.key).toString();
                this._selectedItems = [];
            }
            this._selectedItems.push(data);
            this._moveCaretToEnd(this.element[0]);
        },

        _removeDuplicates: function (currentValue) {
            if (this._selectedItems.length == 0) return false;
            if (this._selectedItems.indexOf(currentValue) != -1) return true;
        },

        search: function () {
            if (this.model.enabled && this._checkDelimiter()) {
                this._hideResult();
                this._autoFill = false;
                this._queryString = $.trim(this._queryString);
                if (this._queryString.length > 0) this._OnTextEnter();
            }
        },
        setVisible: function (value) {
            if (value)
                this.wrapper.show();
            else
                this.wrapper.hide();
        },

        hide: function () {
            this._hideResult();
        },

        open: function () {
            this._showFullList();
        },

        _changeWatermark: function (text) {
            if (this._isWatermark) this.element.attr("placeholder", text);
            else this._hiddenSpan.text(text);
        },
        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            this.suggestionList.removeClass(this.model.cssClass).addClass(skin);
        },
        _setDropdown: function (boolean) {
            this.model.showPopupButton = boolean;
            if (boolean) this._renderDropdown();
            else this._destroyDropdown();
        },
        _changeHeight: function (height) {
            this.wrapper.height(height);
        },
        _changeWidth: function (width) {
            this.wrapper.width(width);
            this._setListWidth();
        },


        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            //deprecated Property Added
            this.model.fields.groupBy = !this.model.fields.groupBy ? this.model.fields.category : this.model.fields.groupBy;
            this._initialize();
            this._render();
            this._wireEvents();
            this.initialRender = false;
			if(this.model.validationRules != null) {
			this._initValidator();
			this._setValidation();
         }			
        },

        _initValidator: function () {
            var proxy = this;
            this.element.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $(error).insertAfter(element.closest(".e-widget"));
                },
            });
        },
        _setValidation: function () {
            if (!this.model.validationRules) return;
            var element = this.model.multiSelectMode == "visualmode" ? this._hiddenInput : this.element;
            element.rules("add", this.model.validationRules);
            var validator = this.element.closest("form").validate();
            name = element.attr("name");
            validator.settings.messages[name] = {};
            for (var ruleName in this.model.validationRules) {
                var message = null;
                if (!ej.isNullOrUndefined(this.model.validationRules[ruleName])) {
                    if (!ej.isNullOrUndefined(this.model.validationRules["messages"] && this.model.validationRules["messages"][ruleName]))
                        message = this.model.validationRules["messages"][ruleName];
                    else {
                        validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                        for (var msgName in this.model.validationMessage)
                            ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                    }
                    validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                }
            }
        },


        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "dataSource": this.model.dataSource = null; break;
                    case "watermarkText": this._changeWatermark(options[option]); break;
                    case "delaySuggestionTimeout": this.model.delaySuggestionTimeout = parseInt(options[option]); break;
                    case "value":
                        options[option] = this._setValue(options[option]);
                        var currValue = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val() == "" ? null : this.element.val();
                        if (!this.initialRender) {
                            this._trigger("change", { value: currValue });
                            this._trigger("_change", { value: currValue });
                        }
                        break;
                    case "showPopupButton": this._setDropdown(options[option]); break;
                    case "enableRTL": this._RightToLeft(options[option]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[option]); break;
                    case "readOnly": this._checkReadOnly(options[option]); break;
                    case "delimiterChar": var delchar = this.model.delimiterChar; options[option] = this._validateDelimiter(options[option]);
                        this.model.delimiterChar = options[option]; this.element.val(this.model.value.replace(new RegExp(delchar, 'g'), this.model.delimiterChar));
                        this.model.value = this.element.val();
                        break;
                    case "validationRules":
                        if (this.model.validationRules != null) {
                            this.element.rules('remove');
                            this.model.validationMessage = null;
                        }
                        this.model.validationRules = options[option];
                        this._setValidation();
                        break;
                    case "validationMessage":
                        this.model.validationMessage = options[option];
                        this._setValidation(); break;
                    case "multiSelectMode":
                        this.model.multiSelectMode = options[option];
                        if (options[option] == "visualmode") {
                            if (this.model.validationRules != null)
                                this.element.rules('remove');
                            this._renderBoxModel();
                        }
                        else if (this.element.hasClass("e-visual-mode"))
                            this._destroyBoxModel();
                        this._setValue(this.model.value);
                        this._setValidation();
                        break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "visible":
                        this.setVisible(options[option]);
                        break;
                    case "height": this._changeHeight(options[option]); break;
                    case "width": this._changeWidth(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "itemsCount ": if (options[option] <= 0 || isNaN(options[option])) options[option] = 0; break;
                    case "popupHeight": this.suggestionList.css({ "max-height": options[option] }); break;
                    case "popupWidth": this.model.popupWidth = options[option]; this._setListWidth(); break;
                    case "selectValueByKey": this.selectValueByKey(options[option]); break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }
        },

        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            if (this.model.multiSelectMode == "visualmode")
                this.element.attr("name", this._hiddenInput.attr("name"));
            this.wrapper.remove();
            this.element.removeClass("e-input").val("");
            if (this._isWatermark) this.element.removeAttr("placeholder");
            this.suggestionList.remove();
        },

        _initialize: function () {
            this.model.value = this.model.value === "" ? this.element[0].value : this.model.value;
            this.element.attr("role", "combobox").attr("aria-label", "Autocomplete textbox").attr("aria-expanded", false).attr("tabindex", 0).attr("aria-autocomplete", "list").attr("value", this.model.value);
            this.target = this.element[0];
            this.dropdownbutton = null;
            this._isIE8 = (ej.browserInfo().name == "msie") && (ej.browserInfo().version == "8.0") ? true : false;
            this.showSuggestionBox = false;
            this.noresult = true;
            this._queryString = null;
            this.suggLen = 0;
            this._selectedItems = [];

            this._activeItem = 0;
            this.ctrlKeyPressed = false;
            this._isFocused = false;
            this._isOpened = false;
            this._typed = false;
            this._cancelEvent = false;
            this._isWatermark = this._checkWatermarkSupport();
        },

        _render: function () {
            this._renderWrapper();
            this._addAttr(this.model.htmlAttributes);
            this._setDimentions();
            this._renderDropdown();
            this._checkProperties();
            this._isWatermark ? this._setWatermark() : this._setWatermarkTxt();
            this._renderSuggestionList();
            this._RightToLeft(this.model.enableRTL);
            this._setRoundedCorner(this.model.showRoundedCorner);
            this.setVisible(this.model.visible);
            this._setListPosition();
        },
        _renderWrapper: function () {
            this.initialRender = true;
            this.element.addClass("e-input").attr("autocomplete", "off");
            this.wrapper = ej.buildTag("span.e-atc e-widget " + this.model.cssClass + "#" + this.target.id + "_wrapper").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
            if (!this._isWatermark) {
                this._hiddenSpan = ej.buildTag("span.e-input e-placeholder ").insertAfter(this.element);
                this._hiddenSpan.text(this.model.watermarkText);
                this._hiddenSpan.css("display", "none");
                var proxy = this;
                this._hiddenSpan.click(function (event) {
                    if (!this._isFocused) proxy.element.focus();
                });
            }
        },

        _renderDropdown: function () {
            if (this.model.showPopupButton) {
                var span = ej.buildTag("span.e-icon e-search").attr((this._isIE8) ? { 'unselectable': 'on' } : {});
                this.dropdownbutton = ej.buildTag("span.e-select#" + this.target.id + "_dropdown").attr((this._isIE8) ? { 'unselectable': 'on' } : {}).append(span);
                this.container.append(this.dropdownbutton).addClass("e-padding");
                this.dropdownbutton.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "required") proxy.element.attr(key, value);
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy._disabled(true);
                else if (key == "readOnly" && value == "readOnly") proxy._checkReadOnly(true);
                else proxy.wrapper.attr(key, value);
            });
        },
        _setDimentions: function () {
            if (this.model.height)
                this.wrapper.height(this.model.height);
            if (this.model.width)
                this.wrapper.width(this.model.width);
        },

        _renderBoxModel: function () {
            this._ulBox = ej.buildTag("ul.e-ul e-boxes");
            this._hiddenInput = ej.buildTag("input#" + this.target.id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
            this._hiddenInput.attr("name", this.element.attr("name"));
            this.element.val("").removeAttr("name").addClass("e-visual-mode");
            this.element.width(1);
            this.container.prepend(this._hiddenInput, this._ulBox);
            this.wrapper.height("auto");

            this._on(this.container, "mousedown", function (e) {
                if (!this.model.enabled) return false;
                var $target = $(e.target);
                if (!$target.is(this.element)) {
                    e.preventDefault();
                    if (!this._isFocused) this.element.focus();
                    if ($target.hasClass("e-options")) {
                        if (!e.ctrlKey && $target.siblings().hasClass("e-active")) this._removeActive();
                        if ($target.hasClass("e-active")) $target.removeClass("e-active");
                        else $target.addClass("e-active");
                    }
                    else this._moveCaretToEnd(this.element[0]);
                }
                if (!e.ctrlKey && ($target.hasClass("e-boxes") || $target.hasClass("e-input"))) this._removeActive();
            });
        },
        _destroyBoxModel: function () {
            this.container.prepend(this.element);
            this.element.attr({ name: this._hiddenInput.attr("name") }).removeAttr("style").removeClass("e-visual-mode");
            this.wrapper.height(this.model.height);
            this._hiddenInput.remove();
            this._ulBox.remove();
            this._off(this.container, "mousedown");
        },
        _deleteLastBox: function () {
            var items = this._ulBox.children();
            var item = items.last(), flag = item.hasClass("e-active");
            this._removeActive();
            flag ? this._deleteBox(item) : item.addClass("e-active");
        },
        _deleteBox: function (items) {
            for (var i = 0; i < items.length; i++) {
                var boxes = this._ulBox.children();
                var index = boxes.index(items[i]);
                this._selectedItems.splice(index, 1);

                var deli = this.model.delimiterChar;
                var values = this._hiddenInput.val().split(deli);
                values.splice(index, 1);
                this._hiddenInput.val(values.join(deli));
                $(items[i]).remove();
            }
            if (this.showSuggestionBox) this._refreshPopup();
            this._valueChange();
        },
        _removeActive: function () {
            this._ulBox.children("li.e-active").removeClass("e-active");
        },
        _adjustWidth: function () {
            var tempSpan = ej.buildTag("span", this.element.val()), wid, minWidth;
            this.container.append(tempSpan);
            minWidth = 30;  //  some additional width for textbox in visualmode
            wid = tempSpan.width() + minWidth;
            if (this.element.width() != wid)
                this.element.width(wid);
            tempSpan.remove();
        },

        _checkProperties: function () {
            this._checkReadOnly(this.model.readOnly);
            this.model.delimiterChar = this._validateDelimiter(this.model.delimiterChar);
            if (!this.model.enabled) {
                this.model.enabled = true;
                this._disabled(true);
            }
            else if (this.model.enabled && $(this.element).hasClass("e-disable")) {
                this.model.enabled = false;
                this._disabled(false);
            }
            this._checkNameAttr();
            if (this.model.multiSelectMode == "visualmode") this._renderBoxModel();
            this.model.value = this._setValue(this.model.value);
            if (this.model.selectValueByKey != null)
                this.selectValueByKey(this.model.selectValueByKey);
        },

        _checkNameAttr: function () {
            if (!this.element.attr("name"))
                this.element.attr("name", this.element[0].id);
        },

        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        _destroyDropdown: function () {
            this.dropdownbutton.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
            this.dropdownbutton.remove();
            this.dropdownbutton = null;
            this.container.removeClass("e-padding");
        },

        _validateDelimiter: function (deli) {
            if ($.trim(deli).length == 1) {
                var RegEx = /^[a-zA-Z0-9]+$/;
                if (!RegEx.test(deli)) return deli;
            }
            return ",";
        },

        _checkWatermarkSupport: function () {
            return 'placeholder' in document.createElement('input');
        },
        _setWatermark: function () {
            if (this._isWatermark) this.element.attr("placeholder", this.model.watermarkText);
        },
        _setWatermarkTxt: function () {
            if (this.model.watermarkText != null && $.trim(this.element.val()) == "" && (this.model.multiSelectMode != "visualmode" || $.trim(this._hiddenInput.val()) == "")) {
                this._hiddenSpan.css("display", "block").text(this.model.watermarkText);
            }
        },

        _renderSuggestionList: function () {
            var oldWrapper = $("#" + this.element.context.id + "_suggestion").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.suggestionList = ej.buildTag("div.e-atc-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.target.id + "_suggestion", "", { "display": "none" }).attr("role", "listbox");
            this.element.attr("aria-owns", this.target.id + "_suggestion");
            this.popup = this.suggestionList;
            var scrollerDiv = ej.buildTag("div");
            this.ul = ej.buildTag("ul.e-ul").attr("role", "listbox");
            scrollerDiv.append(this.ul);
            this.suggestionList.append(scrollerDiv);
            $('body').append(this.suggestionList);
            this._setListWidth();
            this._setListHeight();
            this.suggestionList.ejScroller({ height: this.suggestionList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.suggestionList.ejScroller("instance");
        },

        _checkEmptyList: function () {
            if (this.model.multiSelectMode == "visualmode") this._removeRepeated();
            if (this.suggestionListItems.length == 0) {
                this.suggestionListItems.push(this.model.emptyResultText);
                this.noresult = true;
            }
            else this.noresult = false;
        },
        _showSuggestionList: function () {
            this._checkEmptyList();
            this._addNewTemplate = null;
            if (this.noresult && this.model.multiSelectMode == "visualmode" && this.model.allowAddNew && this.element.val() != "") {
                this.noresult = false;
                this.suggestionListItems.pop();
                this._addNewTemplate = "   (" + this.model.addNewText + ")";
                this.suggestionListItems.push(this.element.val() + this._addNewTemplate);
                this._checkEmptyList();
            }

            if (!this.noresult || this.model.showEmptyResultText)
                this._generateSuggestionList();
        },

        _generateSuggestionList: function () {
            var list = this.suggestionListItems, i, suggList = [];
            this.ul.empty();

            if (typeof list[0] != "object") {

                if (this.model.enableDistinct) list = ej.dataUtil.distinct(list, "", true);
                for (i = 0; i < list.length; i++) {
                    var _txt = (this.model.highlightSearch && !this.noresult) ? this._highlightSuggestion(list[i]) : list[i];
                    this.ul.append(ej.buildTag("li", _txt).attr("role", "option").attr((this._isIE8) ? { 'unselectable': 'on' } : {}));
                }
                this._currList = list;
                this._mapper = { txt: null, key: null };
            }
            else {
                var mapper = this.model.fields, mapFld = { _key: null, _text: null, _attr: null };
                mapFld._key = (mapper && mapper.key) ? mapper["key"] : "key";
                mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
                mapFld._attr = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
                this._mapper = { txt: mapFld._text, key: mapFld._key }, this._currList = [];
                if (this.model.enableDistinct) list = ej.dataUtil.distinct(list, mapFld._text, true);
                if (this.model.fields.groupBy) {
                    var mapCateg = (mapper && mapper.groupBy) ? mapper["groupBy"] : "groupBy", groupedList, _query;
                    _query = ej.Query().group(mapCateg);
                    this._addSortingQuery(_query, "key");
                    groupedList = ej.DataManager(list).executeLocal(_query);
                    this._swapUnCategorized(groupedList);
                    for (i = 0; i < groupedList.length; i++) {
                        if (groupedList[i].key)
                            this.ul.append(ej.buildTag("li.e-category", groupedList[i].key).attr("role", "option").attr((this._isIE8) ? { 'unselectable': 'on' } : {}));
                        this._generateLi(groupedList[i].items, mapFld);
                    }
                }
                else this._generateLi(list, mapFld);
            }
            if (this._getLiTags().length > 0) this._showResult();
        },

        _swapUnCategorized: function (list) {
            $(list).each(function (i, obj) {
                if (!obj.key) {
                    for (var j = i; j > 0; j--) {
                        list[j] = list[j - 1];
                    }
                    list[j] = obj;
                    return false;
                }
            });
        },

        _generateLi: function (list, mapFld) {
            for (var j = 0; j < list.length; j++) {
                var _text = this._getField(list[j], mapFld._text);
                var _key = this._getField(list[j], mapFld._key);
                if (!ej.isNullOrUndefined(_text)) {
                    if (this.model.highlightSearch) _text = this._highlightSuggestion(_text);
                    if (this.model.template) _text = this._getTemplatedString(list[j], mapFld._text, _text);

                    var li = $(document.createElement("li")).append(_text);
                    if (_key)
                        li.attr("id", _key);
                    this._setAttributes(this._getField(list[j], mapFld._attr), li);
                    this.ul.append(li);
                    this._currList = $.merge(this._currList, [list[j]]);
                }
            }
        },

        _getLiTags: function () {
            return this.ul.children("li:not('.e-category')");
        },

        _getTemplatedString: function (list, searchLabl, searchTxt) {
            var str = this.model.template, start = str.indexOf("${"), end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                var replace = this._getField(list, field);
                // if highlightSearch is enabled, it replaces the highlighted search text
                if (searchLabl == field) replace = searchTxt;
                if (!replace) replace = "";
                str = str.split(content).join(replace);
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },

        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },

        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },

        _setListWidth: function () {
            var width = this.model.popupWidth;
            if (width && width != "auto") this.suggestionList.css({ "width": width });
            else this.suggestionList.css({ "width": this.wrapper.width() });
        },

        _setListHeight: function () {
            this.suggestionList.css({ "max-height": this.model.popupHeight });
        },

        _refreshPopup: function () {
            if (this.model.popupWidth == "auto" && this.wrapper.outerWidth() != this.suggestionList.outerWidth()) {
                this.suggestionList.css({ "width": this.wrapper.width() });
                this._refreshScroller();
            }
            this._setListPosition();
        },

        _showResult: function () {
            this._refreshScroller();
            this._refreshPopup();
            if (this._isOpened)
                $(document).bind("mousedown", $.proxy(this._OnDocumentClick, this));
            else {
                this.suggestionList.css("display", "none");
                var tis = this;
                clearTimeout(this._typing);
                this._typing = setTimeout(function () {
                    tis.suggestionList[(tis.model.animateType=="slide"?"slideDown":"fadeIn")]((tis.model.animateType=="none"?0:300), "easeOutQuad", function () {
                        $(document).bind("mousedown", $.proxy(tis._OnDocumentClick, tis));
                    });
                }, this.model.delaySuggestionTimeout);
                this._trigger("open");
                this.wrapper.addClass("e-active");
            }

            this._isOpened = true;
            this.showSuggestionBox = true;
            var _suggestionListItems = this._getLiTags();
            this._listSize = _suggestionListItems.size();

            _suggestionListItems.bind("mouseenter", $.proxy(this._OnMouseEnter, this));
            _suggestionListItems.bind("mouseleave", $.proxy(this._OnMouseLeave, this));
            _suggestionListItems.bind("click", $.proxy(this._OnMouseClick, this));
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            var scrObj = ej.getScrollableParents(this.wrapper);
            if (scrObj[0] != window)
                this._on(scrObj, "scroll", this._hideResult);
        },

        _hideResult: function () {
            if (this.showSuggestionBox) {
                this.showSuggestionBox = false;
                this._activeItem = 0;
                this.element.attr("aria-expanded", false);
                if (this._isOpened) {
                    this.suggestionList.css("display", "none");
                    this._trigger("close");
                }
                else this.suggestionList[(this.model.animateType=="slide"?"slideUp":"fadeOut")]((this.model.animateType=="none"?0:100), "easeOutQuad");
                $(document).unbind("mousedown", $.proxy(this._OnDocumentClick, this));
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
                this._off(ej.getScrollableParents(this.wrapper), "scroll", this._hideResult);
                this.wrapper.removeClass("e-active");
            }
        },

        _refreshScroller: function () {
            this.suggestionList.css("height", "auto");
            this.suggestionList.find(".e-content, .e-vscroll").removeAttr("style");
            this.suggestionList.find(".e-vscroll div").removeAttr("style");
            this.suggestionList.find(".e-content, .e-vhandle").removeAttr("style");
            this.suggestionList.find(".e-vhandle div").removeAttr("style");

            this.suggestionList.css("display", "block");

            if (this.suggestionList.height() > (parseInt(this.model.popupHeight.replace("px", "")) - 4)) {
                this.scrollerObj.model.height = this.suggestionList.height();
                this.scrollerObj.refresh();
                this.scrollerObj.option("scrollTop", 0);
                this.suggestionList.addClass("e-scroller")
            }
            else
                this.suggestionList.removeClass("e-scroller")
            this.suggestionList.css("height", "auto");
            if (this._isIE8)
                this.suggestionList.find('.e-vscroll div, .e-hscroll div').attr("unselectable", "on");
        },

        _setListPosition: function () {
            var elementObj = this.wrapper, pos = this._getOffset(elementObj), winWidth,
            winBottomHeight = $(document).scrollTop() + $(window).height() - (pos.top + $(elementObj).outerHeight()),
            winTopHeight = pos.top - $(document).scrollTop(),
            popupHeight = this.suggestionList.outerHeight(),
            popupWidth = this.suggestionList.outerWidth(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial(), popupmargin = 3,
            topPos = ((popupHeight < winBottomHeight || popupHeight > winTopHeight) ? pos.top + totalHeight + popupmargin : pos.top - popupHeight - popupmargin) - border;
            winWidth = $(document).scrollLeft() + $(window).width() - left;
            if (this.model.enableRTL || popupWidth > winWidth && (popupWidth < left + elementObj.outerWidth())) left -= this.suggestionList.outerWidth() - elementObj.outerWidth();
            this.suggestionList.css({
                "left": left + "px",
                "top": topPos + "px",
                "z-index": maxZ
            });
        },
        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },

        _getZindexPartial: function () {
            return ej.util.getZindexPartial(this.element, this.suggestionList);
        },

        _targetFocus: function () {
            var minWidth = 30;  //  minimum width for textbox in visualmode
            if (this.model.multiSelectMode == "visualmode") this.element.width(minWidth);
            if (!this._isWatermark)
                this._hiddenSpan.css("display", "none");
            this.wrapper.addClass("e-focus");
            this._isFocused = true;


            this._trigger("focusIn", { value: this.model.value });
        },

        _targetBlur: function () {
            this._isFocused = false;
            this.wrapper.removeClass("e-focus");
            !this._isWatermark && this._setWatermarkTxt();
            this._removeSelection();

            if (this.model.multiSelectMode == "visualmode") {
                this._removeActive();
                this.element.val("").width(1);
                this._preVal = "";
            }
            else {
                if (this.model.multiSelectMode == "delimiter") {
                    if (this._checkDeli())
                        this._valueChange();
                }
                this._updateSelectedItemArray();
            }
            this._trigger("focusOut", { value: this.model.value });
        },

        _checkDeli: function () {
            var val, deli, last;
            val = this.element.val();
            deli = this.model.delimiterChar;
            last = val.substr(val.lastIndexOf(deli), deli.length);
            if (last == deli) {
                this.element.val(val.substr(0, val.length - deli.length));
                return true;
            }
            else
                return false;
        },
        _removeSelection: function () {
            if (this.model.enableAutoFill) {
                this.element.attr("aria-autocomplete", "both");
                var selection = this._getCaretSelection();
                if (selection.end - selection.start != 0 && selection.end - selection.start != this.element.val().length)
                    this.target.value = this.target.value.substr(0, selection.start);
            }
        },

        _removeListHover: function () {
            this._getLiTags().removeClass("e-hover");
        },

        _addListHover: function () {
            var activeItem = $(this._getLiTags()[this._activeItem - 1]);
            activeItem.addClass("e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
            activeItem.focus();
        },

        _calcScrollTop: function () {
            var ulH = this.ul.outerHeight(), li = this.ul.find("li"), liH = 0, index, top, i;
            index = this.ul.find("li.e-hover").index();
            for (i = 0; i < index; i++) { liH += li.eq(i).outerHeight(); }
            top = liH - ((this.suggestionList.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top;
        },

        _getActiveText: function () {
            if (this._mapper.txt) return this._getField(this._currList[this._activeItem - 1], this._mapper.txt);
            else return this._currList[this._activeItem - 1];
        },

        _getUniqueKey: function () {
            var key = null;
            if (this._mapper.key) key = this._getField(this._currList[this._activeItem - 1], this._mapper.key);
            ej.isNullOrUndefined(key) && (key = null);
            return key;
        },

        _setTextBoxValue: function () {
            if (this._activeItem && !this.noresult) {
                var currentValue, selection, val, text;
                currentValue = this._getActiveText();
                selection = this._getCaretSelection();
                this.suggLen = selection.start;
                val = this.target.value.substr(0, this.suggLen);

                if (val && val.toLowerCase() == currentValue.substr(0, this.suggLen).toLowerCase()) {
                    if (this.model.multiSelectMode == "delimiter") {
                        var _deliIndex = (this.target.value).lastIndexOf(this.model.delimiterChar);
                        var query = $.trim(val.substr(_deliIndex + 1, this.suggLen));
                        text = val + currentValue.substr(query.length, currentValue.length);
                    }
                    else {
                        text = val + currentValue.substr(val.length, currentValue.length);
                    }
                    this.element.val(text);
                    this._autofilSelection();
                    if (this.model.multiSelectMode == "visualmode")
                        this._adjustWidth();
                    if (this.model.autoFocus && !this.noresult) this._addListHover();
                }
                else this._removeSelection();
            }
        },

        _enterTextBoxValue: function () {
            if (this._activeItem && !this.noresult && !this.model.readOnly) {
                var currentValue = this._getActiveText(), currItem = this._currList[this._activeItem - 1];
                this._valueToTextBox(currentValue, currItem, false);
                var _value = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val();
                this._trigger("select", { value: _value, text: currentValue, key: this._getUniqueKey(), item: currItem });
                this._valueChange();
            }
            this._trigger("close");
        },

        _createBox: function (value) {
            var span = ej.buildTag("span.e-icon e-close");
            var li = ej.buildTag("li.e-options", value).append(span);

            this._on(span, "click", function (e) {
                if (!this.model.enabled) return false;
                this._deleteBox($(e.target).parent());
            });
            return li;
        },

        _addLoadingClass: function () {
            if (this.model.showLoadingIcon)
                this.element.addClass("e-load");
        },

        _removeLoadingClass: function () {
            this.element.removeClass("e-load");
        },


        _highlightSuggestion: function (suggestion) {
            if ($.trim(this._queryString) != "") {
                var caseSensitive, tempQueryString, RegEx, mch, split, query, str;
                caseSensitive = this.model.caseSensitiveSearch ? "g" : "gi";
                query = $.trim(this._queryString);
                tempQueryString = this.model.filterType == "startswith" ? "^" + query : query;
                RegEx = new RegExp(tempQueryString, caseSensitive);

                if (RegEx.test(suggestion)) {
                    mch = suggestion.match(RegEx);
                    //for IE-8 the regEx doesnt returns the empty string.So manually we replace string to return empty string.
                    str = suggestion.replace(RegEx, "~^");
                    split = str.split("~^");
                    suggestion = "";
                    $(split).each(function (i, val) {
                        if (mch[i])
                            suggestion += val + "<span class='e-hilight-txt'>" + mch[i] + "</span>";
                        else
                            suggestion += val;
                    });
                }
            }
            return suggestion;
        },

        _RightToLeft: function (value) {
            if (value) {
                this.wrapper.addClass("e-rtl");
                this.suggestionList.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.suggestionList.removeClass("e-rtl");
            }
        },

        _setRoundedCorner: function (value) {
            if (value) {
                this.container.addClass("e-corner");
                this.suggestionList.addClass("e-corner");
            }
            else {
                this.container.removeClass("e-corner");
                this.suggestionList.removeClass("e-corner");
            }
        },

        _checkReadOnly: function (value) {
            this.model.readOnly = value;
            if (this.model.readOnly) {
                this.element.attr({ "readonly": "readonly", "aria-readonly": true });
                this._off(this.element, "keydown", this._OnKeyDown);
                this._off(this.element, "keyup", this._OnKeyUp);
            }
            else {
                this.element.removeAttr("readonly").removeAttr("aria-readonly");
                this._on(this.element, "keydown", this._OnKeyDown);
                this._on(this.element, "keyup", this._OnKeyUp);
            }
        },

        _OnKeyDown: function (e) {
            if (this.model.filterType != "startswith")
                this.model.enableAutoFill = false;

            switch (e.keyCode) {
                case 37:
                case 35:
                case 36:
                    this._removeSelection();
                case 39:
                    break;
                case 38:
                    if (this.showSuggestionBox && this.suggestionList) {
                        this._removeListHover();
                        if (this._activeItem > 1)
                            this._activeItem -= 1;
                        else
                            this._activeItem = this._listSize;
                        this._addListHover();
                        if (this.model.enableAutoFill) {
                            this.element.attr("aria-autocomplete", "both");
                            this._queryString = this.target.value;
                            this._setTextBoxValue();
                        }
                    }
                    this._PreventDefaultAction(e);
                    break;
                case 40:
                    if (this.showSuggestionBox && this.suggestionList) {
                        this._removeListHover();
                        if (this._activeItem < this._listSize)
                            this._activeItem += 1;
                        else
                            this._activeItem = 1;
                        this._addListHover();
                        if (this.model.enableAutoFill) {
                            this.element.attr("aria-autocomplete", "both");
                            this._queryString = this.target.value;
                            this._setTextBoxValue();
                        }
                    } else {
                        if (e.ctrlKey && this.element.val()=="") {
                            this._showFullList();
                        }
                        else if (e.ctrlKey) {
                            this._showSuggestionList();
                        }
                    }
                    this._PreventDefaultAction(e);
                    break;
                case 8:
                    if ($.trim(this.element.val()) == "") this._isOpened = false;
                    if (this.model.multiSelectMode == "visualmode" && this.element.val() == "") {
                        this._deleteLastBox();
                    }
                    break;
                case 17:
                    this.ctrlKeyPressed = true;
                    break;
                case 13:
                    e.preventDefault();
                    break;
                case 9:
                    if (this.showSuggestionBox) {
                        this._queryString = this.element.val();
                        this._enterTextBoxValue();
                        this._hideResult();
                        this._isOpened = false;
                        this._PreventDefaultAction(e);
                    }
                    break;
                case 27:
                    this._isOpened = false;
                    this._hideResult();
                    this._PreventDefaultAction(e);
                    break;
            }
            if (this.model.multiSelectMode == "visualmode") {
                if (e.keyCode != 17 && e.keyCode != 8 && e.keyCode != 46) this._removeActive();
                this._adjustWidth();
            }
        },


        _PreventDefaultAction: function (e) {

            e.preventDefault();
            if (!this._cancelEvent) {
                e.stopPropagation();
                e.cancelBubble = true;
                e.returnValue = true;
            }

        },
        _bubbleEvent: function (cancelevent) {

            if (typeof cancelevent == "boolean")
                this._cancelEvent = cancelevent;
            return this._cancelEvent;
        },


        _OnKeyUp: function (e) {
            this._keyDownComplete();
            if (this.ctrlKeyPressed) {
                if (e.keyCode == 17)
                    this.ctrlKeyPressed = false;
                return false;
            }

            if ($.trim(this.element.val()) == "" && e.keyCode == 38 && e.keyCode == 40) {
                this._hideResult();
                return false;
            }
            if (!this._checkDelimiter()) return false;
            this._queryString = $.trim(this._queryString);

            switch (e.keyCode) {
                case 38:
                case 40:
                case 37:
                case 39:
                case 20:
                case 16:
                case 17:
                case 18:
                case 35:
                case 36:
                case 144:
                case 9:
                case 27: break;

                case 13:
                    e.preventDefault();
                    this._queryString = this.element.val();
                    this._enterTextBoxValue();
                    if (this.model.enableAutoFill) {
                        this.element.attr("aria-autocomplete", "both");
                        this.suggLen = this.element.val().length;
                        this._autofilSelection();
                    }
                    this._isOpened = false;
                    this._hideResult();
                    break;

                case 46:
                    if (this.model.multiSelectMode == "visualmode" && this.element.val() == "") {
                        this._deleteBox(this._ulBox.children("li.e-active"));
                        break;
                    }
                case 8:
                    if (this._queryString.length >= this.model.minCharacter) {
                        this._autoFill = false;
                        this._OnTextEnter();
                    }
                    else {
                        this.noresult = true;
                        this._hideResult();
                        if ($.trim(this.element.val()) == "") {
                            this._isOpened = false;
                            this._trigger("close");
                        }
                    }
                    break;

                default:
                    if (this._queryString.length >= this.model.minCharacter) {
                        this._autoFill = true;
                        this._OnTextEnter();
                    }
                    else this.noresult = true;
                    this._typed = true;
                    break;
            }
        },

        _getFilteredList: function (list) {
            if (!list || !list.length || list.length < 1) this.suggestionListItems = [];
            else {
                var tempQuery = ej.Query();
                this._addQuery(tempQuery, typeof list[0] == "object");
                this.suggestionListItems = ej.DataManager(list).executeLocal(tempQuery);
            }
            this._doneRemaining();
        },

        _performSearch: function () {
            var source = this.model.dataSource;
            if (ej.DataManager && source instanceof ej.DataManager) {
                if (!source.dataSource.offline && !(source.dataSource.json && source.dataSource.json.length > 0)) {
                    window.clearTimeout(this.timer);
                    var proxy = this;

                    this.timer = window.setTimeout(function () {
                        proxy._fetchRemoteDat(source);
                    }, 700);
                }
                else this._getFilteredList(source.dataSource.json);
            }
            else this._getFilteredList(source);
        },

        _fetchRemoteDat: function (source) {
            var proxy = this, queryPromise, tempQuery = this._getQuery();
            this._addQuery(tempQuery, true);
            queryPromise = source.executeQuery(tempQuery);
            queryPromise.fail(function (e) {
                proxy.suggestionListItems = null;
                proxy._removeLoadingClass();
                proxy._trigger("actionFailure", e);
            }).done(function (e) {
                proxy.suggestionListItems = e.result;
                proxy._doneRemaining();
                proxy._trigger("actionSuccess", e);
            }).always(function (e) {
                proxy._trigger("actionComplete", e);
            });
        },

        _addSortingQuery: function (query, key) {
            if (this.model.allowSorting) {
                var order = (this.model.sortOrder == "descending") ? true : false;
                query.sortBy(key, order);
            }
        },

        _addQuery: function (_query, checkMapper) {
            var bindTo = "";
            if (checkMapper) {
                var mapper = this.model.fields;
                bindTo = (mapper && mapper.text) ? mapper["text"] : "text";
            }
            if (this._queryString) _query.where(bindTo, this.model.filterType, this._queryString, !this.model.caseSensitiveSearch);
            this._addSortingQuery(_query, bindTo);
            if (this.model.itemsCount > 0) _query.take(this.model.itemsCount);
        },

        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query.clone();
            return queryManager;
        },

        _OnTextEnter: function () {
            this._addLoadingClass();
            this.element.attr("aria-expanded", false);
            this._performSearch();
        },

        _doneRemaining: function () {
            this._showSuggestionList();
            this.element.attr("aria-expanded", true).attr("aria-haspopup", true);
            if (this.model.enableAutoFill && this._autoFill && !this.noresult || this.model.autoFocus) {
                this.element.attr("aria-autocomplete", "both");
                this._activeItem = 1;
                this._queryString = this.target.value;
            }
            if (this.model.enableAutoFill && this._autoFill && !this.noresult) this._setTextBoxValue();
            if (this.model.autoFocus && !this.noresult) this._addListHover();
            this._removeLoadingClass();
            if (this.noresult && !this.model.showEmptyResultText)
                this._hideResult();
        },

        _removeRepeated: function () {
            var results = this.suggestionListItems;
            if (!results || results.length == 0 || this._selectedItems.length == 0)
                return false;
            for (var i = 0; i < this._selectedItems.length; i++) {
                var index = results.indexOf(this._selectedItems[i]);
                if (index != -1) this.suggestionListItems.splice(index, 1);
            }
        },

        _checkDelimiter: function () {
            this._queryString = this.element.val();
            var _deliIndex = (this.model.multiSelectMode != "delimiter") ? -1 : this._queryString.lastIndexOf(this.model.delimiterChar);


            if (_deliIndex == -1) {
                return true;
            }
            else if (_deliIndex + 1 == this._queryString.length) {
                this._hideResult();
                return false;
            }
            else {
                this._queryString = this._queryString.substr(_deliIndex + 1, this._queryString.length);
                return true;
            }
        },

        _autofilSelection: function () {
            var element = this.element[0], totLen = this.element.val().length;

            if (element.setSelectionRange)
                element.setSelectionRange(this.suggLen, totLen);
            else if (element.createTextRange) {

                element = element.createTextRange();
                element.collapse(true);
                element.moveEnd('character', totLen);
                element.moveStart('character', this.suggLen);
                element.select();
            }
        },

        _OnDropdownClick: function (e) {
            if (this.model.enabled && !this.model.readOnly) {
                e.preventDefault();
                this._addLoadingClass();
                if (this.showSuggestionBox) {
                    this._isOpened = false;
                    this._hideResult();
                    this._removeLoadingClass();
                }
                else this._showFullList();
            }
        },

        _showFullList: function () {
            if (!this._isFocused)
                this.element.focus();
            this._queryString = null;
            this._autoFill = false;
            this._performSearch();
        },

        _OnMouseEnter: function (e) {
            var targetEle = e.target;
            if (e.target.tagName != "LI") targetEle = $(e.target).parents("li");
            this._getLiTags().removeClass("e-hover");
            $(targetEle).addClass("e-hover");

            this._queryString = this.element.val();
            this._activeItem = this._getLiTags().index($(targetEle)) + 1;
        },

        _OnMouseLeave: function (e) {
            this._getLiTags().removeClass("e-hover");
            if (this.model.highlightSearch)
                this._getLiTags().find(".e-hilight-txt").removeClass("e-hover");
        },

        _OnMouseClick: function (e) {
            if (!this.noresult) {
                this._enterTextBoxValue();
                this._isOpened = false;
                this._hideResult();
            }
        },

        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.suggestionList) && !$(e.target).parents(".e-atc-popup").is(this.suggestionList) &&
                !$(e.target).is(this.element) && !$(e.target).parents(".e-atc").is(this.wrapper)) {
                this._isOpened = false;
                this._hideResult();
                this._trigger("close");
            }
            else if ($(e.target).is(this.suggestionList) || $(e.target).parents(".e-atc-popup").is(this.suggestionList))
                e.preventDefault();
        },

        _OnWindowResize: function (e) {
            this._refreshPopup();
        },

        _valueChange: function () {
            var currValue = this.model.multiSelectMode == "visualmode" ? this._hiddenInput.val() : this.element.val() == "" ? null : this.element.val();
            if (this.model.value != currValue) {
                this.model.value = currValue;
                if (!this.initialRender) {
                    this._trigger("change", { value: currValue });
                    this._trigger("_change", { value: currValue });
                }
            }
        },

        _updateSelectedItemArray: function (e) {
            var value = this.getValue(), values = [];
            this._selectedItems = [];
            this.suggestionListItems = this.model.dataSource;
            if (this.model.multiSelectMode == "delimiter") {
                values = value.split(this.model.delimiterChar);
                if ((ej.DataManager && this.model.dataSource instanceof ej.DataManager) || ((!ej.isNullOrUndefined(this.suggestionListItems)) && (typeof this.suggestionListItems[0] != "object"))) {
                    for (var i = 0; i < values.length; i++) {
                        if (values[i])
                            this._selectedItems.push(values[i]);
                    }
                }
                else
                    this._createBoxForObjectType(values);
            }
            else if (this.model.multiSelectMode == "none" && value) {
                values.push(value);
                if ((ej.DataManager && this.model.dataSource instanceof ej.DataManager) || ((!ej.isNullOrUndefined(this.suggestionListItems)) && (typeof this.suggestionListItems[0] != "object")))
                    this._selectedItems.push(value);
                else
                    this._createBoxForObjectType(values);
            }
        },

        _keyDownComplete: function () {
            var currValue = this.element.val();
            if (this._preVal != currValue) {
                this._preVal = currValue;
                if (this.model.multiSelectMode == "visualmode") this._adjustWidth();
                this._valueChange();
            }
        },

        _moveCaretToEnd: function (el) {
            //Chrome Scroll content
            el.scrollLeft = el.scrollWidth;
            //IE Scroll the content
            if (el.createTextRange) {
                var rng = el.createTextRange();
                rng.moveEnd('textedit');
                rng.moveStart('textedit');
                rng.select();
            }
            if (typeof el.selectionStart == "number") {
                el.selectionStart = el.selectionEnd = el.value.length;
            } else if (typeof el.createTextRange != "undefined") {
                var range = el.createTextRange();
                range.collapse(false);
                range.select();
            }
        },

        _getCaretSelection: function () {
            var input = this.element[0], start = 0, end = 0;

            if (!isNaN(input.selectionStart)) {
                start = input.selectionStart;
                end = input.selectionEnd;
                return { start: Math.abs(start), end: Math.abs(end) };
            }

            var bookmark = document.selection.createRange().getBookmark();
            var selection = input.createTextRange();
            selection.moveToBookmark(bookmark);

            var before = input.createTextRange();
            before.collapse(true);
            before.setEndPoint("EndToStart", selection);
            var beforeLength = before.text.length, selLength = selection.text.length;
            return { start: beforeLength, end: beforeLength + selLength };
        },



        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
        }
    });

    ej.filterType = {
        /**  Supports to search text with startswith  */
        StartsWith: "startswith",
        /**  Supports to search text with contains */
        Contains: "contains",
        /**  Supports to search text with endswith */
        EndsWith: "endswith",
        /**  Supports only for number lessthan only */
        LessThan: "lessthan",
        /**  Supports only for number greaterthan only */
        GreaterThan: "greaterthan",
        /**  Supports only for number lessthanorequal only */
        LessThanOrEqual: "lessthanorequal",
        /**  Supports only for number greaterthanorequal only */
        GreaterThanOrEqual: "greaterthanorequal",
        /**  Supports only for number equal only */
        Equal: "equal",
        /** Supports only for number notequal only */
        NotEqual: "notequal"
    };

    ej.SortOrder = {
        /** Supports to sorts with ascending only */
        Ascending: "ascending",
        /** Supports to sorts with descending only */
        Descending: "descending"
    };

    ej.MultiSelectMode = {
        /** Supports to selection mode with none only */
        None: "none",
        /** Supports to selection mode with delimitter only */
        Delimiter: "delimiter",
        /** Supports to selection mode with visualmode only */
        VisualMode: "visualmode"
    };
    ej.Animation = {
        /** Supports to animation type with none only */
        None: "none",
        /** Supports to animation type with slide only */
        Slide: "slide",
        /** Supports to animation type with fade only */
        Fade: "fade"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin provides support to display calendar within your web page and allows to pick the date.
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejDatePicker", "ej.DatePicker", {

        element: null,
        _rootCss: "e-datepicker",

        model: null,
        validTags: ["input", "div", "span"],
        _setFirst: false,
        _cancelValue: false,
        type: "editor",
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },


        defaults: {

            dayHeaderFormat: "min",

            showPopupButton: true,

            enableAnimation: true,

            showFooter: true,

            displayInline: false,

            htmlAttributes: {},

            dateFormat: '',

            watermarkText: "Select date",

            value: null,

            minDate: new Date(1900, 01, 01),

            maxDate: new Date(2099, 12, 31),

            startLevel: "month",

            depthLevel: "",

            cssClass: "",

            startDay: -1,

            stepMonths: 1,

            locale: "en-US",

            showOtherMonths: true,

            enableStrictMode: false,

            enablePersistence: false,

            enabled: true,

            width: "",

            height: "",

            enableRTL: false,

            showRoundedCorner: false,

            headerFormat: 'MMMM yyyy',

            buttonText: 'Today',

            readOnly: false,

            specialDates: null,

            fields: {

                date: "date",

                tooltip: "tooltip",

                iconClass: "iconClass",
            },

            showTooltip: true,

            highlightSection: "none",

            highlightWeekend: false,

            validationRules: null,

            validationMessage: null,

            allowEdit: true,
            tooltipFormat: "ddd MMM dd yyyy",
            allowDrilDown: true,
            beforeDateCreate: null,

            open: null,

            close: null,

            select: null,

            change: null,

            focusIn: null,

            focusOut: null,

            beforeOpen: null,
            beforeClose:null,
            create: null,

            destroy: null

        },


        dataTypes: {
            startDay: "number",
            stepMonths: "number",
            showOtherMonths: "boolean",
            enableStrictMode: "boolean",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
            displayInline: "boolean",
            showPopupButton: "boolean",
            locale: "string",
            readOnly: "boolean",
            cssClass: "string",
            dateFormat: "string",
            watermarkText: "string",
            headerFormat: "string",
            buttonText: "string",
            specialDates: "data",
            showTooltip: "boolean",
            highlightSection: "enum",
            highlightWeekend: "boolean",
            enableAnimation: "boolean",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data",
            tooltipFormat: "string",
            allowEdit: "boolean",
            allowDrilDown: "boolean"
        },

        _setModel: function (jsondata) {
            var callRefresh = false, start = false, validate = false;
            for (var key in jsondata) {
                switch (key) {
                    case "dayHeaderFormat":
                        this.model.dayHeaderFormat = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "showPopupButton":
                        this._renderDateIcon(jsondata[key], true);
                        break;
                    case "displayInline":
                        this._setDisplayInline(jsondata[key]);
                        if (!this.model.allowEdit && !jsondata[key] && this._isInputBox())                            
                            this.element.bind("mousedown", $.proxy(this._showDatePopUp, this));
                        break;
                    case "value":
                        if (ej.isNullOrUndefined(jsondata["minDate"]) && ej.isNullOrUndefined(jsondata["maxDate"])) {
                            this._setDateValue(jsondata[key]);
                            jsondata[key] = this.model.value;
                        }
                        else
                            this._updateDateValue(jsondata[key]);
                        validate = true;
                        start = true;
                        callRefresh = true;
                        break;
                    case "specialDates":
                        this.model.specialDates = jsondata[key];
                        this._createSpecialDateObject();
                        callRefresh = true;
                        start = true;
                        break;
                    case "fields":
                        this.model.fields = jsondata[key];
                        this._mapField = this._getMapper();
                        callRefresh = true;
                        start = true;
                        break;
                    case "showTooltip":
                        this.model.showTooltip = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "highlightWeekend":
                        this.model.highlightWeekend = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "highlightSection":
                        this.model.highlightSection = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "dateFormat":
                        this.model.dateFormat = jsondata[key];
                        this._setDateValue(this.model.value);
                        break;
                    case "minDate":
                        this._setMinDate(jsondata[key]);
                        jsondata[key] = this.model.minDate;
                        validate = true;
                        callRefresh = true;
                        start = true;
                        break;
                    case "maxDate":
                        this._setMaxDate(jsondata[key]);
                        jsondata[key] = this.model.maxDate;
                        validate = true;
                        callRefresh = true;
                        start = true;
                        break;
                    case "locale":
                        this.model.locale = jsondata[key];
                        this.model.startDay = ((ej.isNullOrUndefined(this._options.startDay)) && (this.model.startDay === this.culture.calendar.firstDay))
                            ? -1 : this.model.startDay;
                        this.model.dateFormat = ((ej.isNullOrUndefined(this._options.dateFormat)) && (this.model.dateFormat === this.culture.calendar.patterns.d))
                            ? '' : this.model.dateFormat;
                        this._setCulture(jsondata[key]);
                        this._setDateValue(this.model.value);
                        jsondata[key] = this.model.locale;
                        callRefresh = true;
                        start = true;
                        break;
                    case "showOtherMonths":
                        this.model.showOtherMonths = jsondata[key];
                        this._otherMonthsVisibility();
                        break;
                    case "enableStrictMode":
                        this.model.enableStrictMode = jsondata[key];
                        validate = true;
                        callRefresh = true;
                        start = true;
                        break;
                    case "validationRules":
                        if (this.model.validationRules != null) {
                            this.element.rules('remove');
                            this.model.validationMessage = null;
                        }
                        this.model.validationRules = jsondata[key];
                        this._setValidation();
                        break;
                    case "validationMessage":
                        this.model.validationMessage = jsondata[key];
                        this._setValidation(); break;
                    case "readOnly":
                        this.model.readOnly = jsondata[key];
                        this._disbleMaualInput();
                        break;
                    case "width":
                        this._setWidth(jsondata[key]);
                        break;
                    case "height":
                        this._setHeight(jsondata[key]);
                        break;
                    case "cssClass":
                        this._setSkin(jsondata[key]);
                        break;
                    case "enableRTL":
                        this._setRTL(jsondata[key]);
                        break;
                    case "showRoundedCorner":
                        this._setRoundedCorner(jsondata[key]);
                        break;
                    case "enabled":
                        if (!jsondata[key]) this.disable();
                        else this.enable();
                        break;
                    case "buttonText":
                        this._setFooterText(jsondata[key]);
                        break;
                    case "showFooter":
                        this._enableFooter(jsondata[key]);
                        break;
                    case "watermarkText":
                        this.model.watermarkText = jsondata[key]; this._setWaterMark();
                        break;
                    case "startDay":
                        if (parseInt(jsondata[key]) < 0 || parseInt(jsondata[key]) > 6) jsondata[key] = 0;
                        this.model.startDay = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "startLevel":
                        this.model.startLevel = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "headerFormat":
                        this.model.headerFormat = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "depthLevel":
                        this.model.depthLevel = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "htmlAttributes": this._addAttr(jsondata[key]); break;
                    case "allowEdit": this._changeEditable(jsondata[key]); break;
                    case "tooltipFormat":
                        this.model.tooltipFormat = jsondata[key];
                        callRefresh = true;
                        start = true;
                        break;
                    case "allowDrilDown":
                        this._allowQuickPick(jsondata[key]);
                        callRefresh = true;
                        start = true;
                        break;
                }
            }
            if (validate) {
                this._validateMinMaxDate();
                jsondata["value"] = this.model.value;
                jsondata["maxDate"] = this.model.maxDate;
                jsondata["minDate"] = this.model.minDate;
            }
            this._setWaterMark();

            if (callRefresh && this.isValidState)
                this._refreshDatepicker();
            if(start)this._startLevel(this.model.startLevel);
            this._triggerChangeEvent();
            this._checkErrorClass();
        },
        observables: ["value"],

        _destroy: function () {
            if (this._isOpen)
                this.hide();
            this.sfCalendar.remove();
            if (this.wrapper) {
                this.element.insertAfter(this.wrapper);
                this.wrapper.remove();
            }
            this.element.removeClass('e-datepicker e-input');
        },

        _init: function (options) {
            this._options = options;
            this.animation = {
                open: { type: "easeOutQuad", duration: 200 },
                close: { type: "easeOutQuad", duration: 100 }
            };
            this._animating = false;
            this._isSupport = document.createElement("input").placeholder == undefined ? false : true;
            this._checkAttribute();
            this._setValues();
            this._createDatePicker();
            if (this.model.validationRules != null) {
                this._initValidator();
                this._setValidation();
            }
        },



        _initValidator: function () {
            var proxy = this;
            this.element.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $(error).insertAfter(element.closest(".e-widget"));
                },
            });
        },
        _setValidation: function () {
            this.element.rules("add", this.model.validationRules);
            var validator = this.element.closest("form").validate();
            name = this.element.attr("name");
            validator.settings.messages[name] = {};
            for (var ruleName in this.model.validationRules) {
                var message = null;
                if (!ej.isNullOrUndefined(this.model.validationRules[ruleName])) {
                    if (!ej.isNullOrUndefined(this.model.validationRules["messages"] && this.model.validationRules["messages"][ruleName]))
                        message = this.model.validationRules["messages"][ruleName];
                    else {
                        validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                        for (var msgName in this.model.validationMessage)
                            ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                    }
                    validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                }
            }
        },
        _checkAttribute: function () {
            var attr = ["min", "max", "readonly", "disabled"], propName = ["minDate", "maxDate", "readOnly", "enabled"], value, propValue;
            for (var i = 0; i < attr.length; i++) {
                value = this.element.attr(attr[i]); propValue = propName[i];
                if (!ej.isNullOrUndefined(value)) {
                    if (ej.isNullOrUndefined(this._options))
                        this.model[propValue] = ((propValue != "enabled") && (propValue != "readOnly")) ? new Date(value) : propValue == "readOnly" ? this.element.is("[readonly]") : !this.element.is("[disabled]");
                    else if (ej.isNullOrUndefined(this._options[propValue]))
                        this.model[propValue] = ((propValue != "enabled") && (propValue != "readOnly")) ? new Date(value) : propValue == "readOnly" ? this.element.is("[readonly]") : !this.element.is("[disabled]");
                }
            }
        },
        _updateDateValue: function (value) {
            var date = this._checkDateObject(value);
            if (date != null) {
                this.isValidState = true;
                if (date == "") {
                    this.element.val("");
                    this.model.value = "";
                } else {
                    this.model.value = date;
                    this._preTxtValue = this.element.val(this._formatter(this.model.value, this.model.dateFormat));
                }
            }
            else {
                (typeof date === "string" && this.model.enableStrictMode) ? this.element.val(value) : this.element.val("");
                this.model.value = "";
                this.isValidState = (this.element.val() == "") ? true : false;
            }
            this._removeWatermark();
        },
        _changeEditable: function (bool) {
            var action = bool ? "_on" : "_off";
            if (this.element.is(":input")) {
                if (bool) {
                    if (!this.model.readOnly) this.element.attr("readonly", false);
                    this.element.unbind("mousedown", $.proxy(this._showDatePopUp, this));
                }
                else {
                    if (!this.model.readOnly) this.element.attr("readonly", "readonly");
                    if (!this.model.displayInline) this.element.bind("mousedown", $.proxy(this._showDatePopUp, this));
                }
                this[action](this.element, "blur", this._onFocusOut);
                this[action](this.element, "focus", this._onFocusIn);
                this[action](this.element, "keyup", this._keyUp);
            }
        },
        _allowQuickPick: function (value) {           
            $('.e-datepicker-headertext', this.sfCalendar)[value ? "on" : "off"]("click", $.proxy(this._forwardNavHandler, this));
        },
        _setValues: function () {
            this.Date = new Date();
            this._id = this.element[0].id;
            this.isValidState = true;
            this._setCulture(this.model.locale);
            this._setMinDate(this.model.minDate);
            this._setMaxDate(this.model.maxDate);
            this._calendarDate = this._zeroTime(new Date());
            if (this.model.startDay < 0 || this.model.startDay > 6) this.model.startDay = 0;
            this.Date.firstDayOfWeek = this.model.startDay;
            this.Date.fullYearStart = '20';
            this._showHeader = true;
            if (ej.isNullOrUndefined(this.model.value) && this.element[0].value != "")
                this.model.value = this.element[0].value;
            this._validateMinMaxDate();
            this._dateValue = new Date(this._calendarDate.toString());
            this._isIE7 = this._checkIE7();
            this._isIE8 = (ej.browserInfo().name == "msie") && (ej.browserInfo().version == "8.0") ? true : false;
            this._isIE9 = (ej.browserInfo().name == "msie") && (ej.browserInfo().version == "9.0") ? true : false;
            // this variable is set to true in DateTimePicker control
            this._getInternalEvents = false;
            this._flag = true;
            this._ejHLWeekEnd = false;
            this._isOpen = false;
            this._prevDate = null;
            this._isFocused = false;
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "required") proxy.element.attr(key, value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "readOnly" && value == "readOnly") proxy.model.readOnly = true;
                else proxy.wrapper.attr(key, value);

            });
        },
        _createDatePicker: function () {
            this._createWrapper();
            this._wireEvents();
            if (this.model.displayInline)
                this.show();
            if (this.model.enableRTL) this._setRTL(true);
            if (this.model.showRoundedCorner) this._setRoundedCorner(true);
        },
        _checkNameAttr: function () {
            if (!this.element.attr("name") && this._isInputBox())
                this.element.attr("name", this.element[0].id);
            if (this.model.displayInline && !this._isInputBox())
                this._hiddenInput.attr("name", this.element[0].id);
        },
        _createWrapper: function () {
            this._mapField = this._getMapper();
            if (this.model.specialDates) {
                this._createSpecialDateObject();
            }
            if (this._isInputBox()) {
                this.element.addClass("e-input").attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', 'tabindex': '0' });
                this.wrapper = ej.buildTag("span.e-datewidget e-widget " + this.model.cssClass);
                this.innerWrapper = ej.buildTag("span.e-in-wrap e-box");
                this.wrapper.append(this.innerWrapper).insertBefore(this.element);
                this.innerWrapper.append(this.element);
                this.innerWrapper.addClass('e-padding');
                this.dateIcon = ej.buildTag("span.e-select#" + this._id + "-img", "", {}, (this._isIE8) ? { 'unselectable': 'on' } : {})
                    .append(ej.buildTag("span.e-icon e-date", "", {}, { 'aria-label': 'Select' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})).insertAfter(this.element);
            }
            if (!this._isSupport || (this.model.displayInline && !this._isInputBox())) {
                this._hiddenInput = ej.buildTag("input.e-input e-placeholder ", "", {}, { type: "text" }).insertAfter(this.element);
                if (this._isInputBox()) this._hiddenInput.val(this.model.watermarkText);
                this._hiddenInput.css("display", "block");
                var proxy = this;
                $(this._hiddenInput).focus(function () {
                    proxy.element.focus();
                });
            }
            this._checkNameAttr();
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
            if (this._id)
                $("#e-" + this._id).remove();
            this.sfCalendar = ej.buildTag('div.e-datepicker e-popup e-widget ' + this.model.cssClass + ' e-calendar ' + (this.model.specialDates ? (this.model.specialDates[0][this._mapField._icon] ? 'e-icons ' : '') : ''), "", {}, { id: (this._id ? 'e-' + this._id : "") }).attr({ 'aria-hidden': 'true' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})
                .insertBefore(this.element);
            if (this.model.displayInline && !this.element.is("input"))
                this.sfCalendar.addClass('e-inline');
            this.popup = this.sfCalendar;
            this._resizeCalender();
            this._setDateValue(this.model.value);
            this._preValue = this._parseDate(this.element.val(), this.model.dateFormat);
            this._setWaterMark();
            this._dateValue = new Date(this._calendarDate.toString());
            this._createCalender();
            this._setDisplayInline(this.model.displayInline);
            if (this.model.readOnly) this._disbleMaualInput();
            if (!this.model.enabled) this.disable();
            else if (this.model.enabled && $(this.element).hasClass("e-disable")) this.enable();
            this._layoutChanged();
            this._checkErrorClass();
            this._addAttr(this.model.htmlAttributes);
        },
        _isInputBox: function () {
            return (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type')));
        },

        _renderDateIcon: function (bool, reRender) {
            if (reRender && this.model.showPopupButton == bool) return;
            if (!bool && this.dateIcon) {
                this._bindInputEvent();
                this.dateIcon.css('display', 'none');
                this.innerWrapper.removeClass('e-padding');
            }
            else {
                if (this.innerWrapper) {
                    this.innerWrapper.addClass('e-padding');
                    this.dateIcon.css('display', 'block');
                }
                if (!this.model.displayInline)
                    this._bindDateButton();
            }
            this.model.showPopupButton = bool;
        },

        _resizeCalender: function () {
            if ((this.model.dayHeaderFormat == "short") || (this.model.dayHeaderFormat == "min") || (this.model.dayHeaderFormat == "none"))
                this.sfCalendar.removeClass("e-headerlong");
            else if (this.model.dayHeaderFormat == "long") {
                this.sfCalendar.addClass("e-headerlong");
            }
        },

        _setWidth: function (value) {
            if (value) {
                if (this.wrapper) this.wrapper.width(value);
                else this.element.width(value);
            }
            else
                this.model.width = this.wrapper ? this.wrapper.outerWidth() : this.element.width();
        },
        _setHeight: function (value) {
            if (value) {
                if (this.wrapper) this.wrapper.height(value);
                else this.element.height(value);
            }
            else
                this.model.height = this.wrapper ? this.wrapper.outerHeight() : this.element.height();
            if (this._isIE7) this.element.height(this.innerWrapper.height());
        },
        _setRTL: function (isRTL) {
            if (isRTL) {
                if (this.wrapper) {
                    this.wrapper.addClass("e-rtl");
                }
                this.sfCalendar.addClass("e-rtl");
            }
            else {
                if (this.wrapper) {
                    this.wrapper.removeClass("e-rtl");
                }
                this.sfCalendar.removeClass("e-rtl");
            }
        },
        _setRoundedCorner: function (bool) {
            if (bool) {
                if (this.innerWrapper)
                    this.innerWrapper.addClass("e-corner");
                this.sfCalendar.addClass("e-corner");
            }
            else {
                if (this.innerWrapper)
                    this.innerWrapper.removeClass("e-corner");
                this.sfCalendar.removeClass("e-corner");
            }
        },

        _refreshDatepicker: function () {            
            var _currentVal = this.element.val();
            //  For checking the year maximum range....
            var currentValue = this._parseDate(_currentVal, this.model.dateFormat);
            currentValue = this._validateYearValue(currentValue);
            this._setDateValue(currentValue);
            $(".e-datepicker-headertext", this.sfCalendar).text(this._formatter(this._calendarDate, this.model.headerFormat));
            this._resizeCalender();
            this._dateValue = new Date(this._calendarDate.toString());
            this._hoverDate = this._calendarDate.getDate() - 1;
            this._renderCalendar(this, this._dateValue);
            this._setFooterText(this.model.buttonText);
            this._enableFooter(this.model.showFooter);
            this._layoutChanged();
        },
        _validateYearValue: function (value) {
            if (value != null) {
                var twoDigitYearMax = Globalize.culture(this.model.locale).calendars.standard.twoDigitYearMax;
                twoDigitYearMax = typeof twoDigitYearMax === "string" ? new Date().getFullYear() % 100 + parseInt(twoDigitYearMax, 10) : twoDigitYearMax;
                if (this._calendarDate.getFullYear() - value.getFullYear() == 100) {
                    if (this._calendarDate.getFullYear() > twoDigitYearMax)
                        value.setFullYear(this._calendarDate.getFullYear())
                }
            }
            return value;
        },
        _setFooterText: function (footerText) {
            $('.e-footer-text', this.sfCalendar).html(footerText);
        },
        _setSkin: function (skin) {
            if (this.wrapper) {
                this.wrapper.removeClass(this.model.cssClass);
                this.wrapper.addClass(skin);
            }
            else {
                this.element.removeClass(this.model.cssClass);
                this.element.addClass(skin);
            }
            this.sfCalendar.removeClass(this.model.cssClass);
            this.sfCalendar.addClass(skin);
        },
        _setDisplayInline: function (isDisplayInline) {
            this.model.displayInline = isDisplayInline;
            if (isDisplayInline && this._isInputBox()) {
                this.sfCalendar.insertAfter(this.wrapper);
                this._setDatePickerPosition();
            }
            else if (isDisplayInline) {
                this.element.append(this.sfCalendar);
                if (!this._isSupport || !this._isInputBox()) this._hiddenInput.css("display", "none");
            }
            else {
                this.sfCalendar.css('display', 'none');
                $('body').append(this.sfCalendar);
                this._isOpen = false;
            }
            if (isDisplayInline) {
                this.show();
                this._off(this.dateIcon, "mousedown", this._showDatePopUp);
                this.element.unbind("mousedown", $.proxy(this._showDatePopUp, this));
            }
            else if (this._isInputBox()) this._renderDateIcon(this.model.showPopupButton, false);
        },

        _disbleMaualInput: function () {
            if (this.model.readOnly) {
                $(this.element).attr("readonly", "readonly");
                if (!this.model.displayInline) this.hide();
            }
            else if (this.model.allowEdit)
                $(this.element).removeAttr("readonly");
            
        },
        _checkDateObject: function (date) {
            if (!date || (typeof JSON === "object" && JSON.stringify(date) === "{}")) return date = null;
            else if (!(date instanceof Date)) {
                var val = this._parseDate(date);
                date = (val != null) ? val : new Date(date);
            }
            if (!isNaN(Date.parse(date))) return this._dateValue = this._calendarDate = this._zeroTime(date);
            return null;
        },
        _checkInstanceType: function (date) {
            date = this._stringToObject(date);
            if (!date) return null;
            else if (!(date instanceof Date)) {
                date = this._parseDate(date);
            }
            if (!isNaN(Date.parse(date))) return this._zeroTime(date);
            return null;
        },
        _stringToObject: function (value) {
            if (typeof value === "string") {
                var val = Globalize.parseDate(value, this.model.dateFormat);
                value = (val != null) ? val : new Date(value);
            }
            return value;
        },
        _validateMinMaxDate: function () {
            var dateChange = false, valueExceed = false;
            if (this.model.maxDate < this.model.minDate) this.model.minDate = this.model.maxDate;
            if (!this.model.enableStrictMode) {
                if (this.model.value) {
                    if (this.model.value < this.model.minDate) {
                        this._calendarDate = this.model.value = this.model.minDate;
                        dateChange = true;
                    }
                    else if (this.model.value > this.model.maxDate) {
                        this._calendarDate = this.model.value = this.model.maxDate;
                        dateChange = true;
                    }
                }
                else {
                    this.element.val("");
                    if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                    else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                }
                this.isValidState = true;
            }
            else {
                if (this.model.value) {
                    if (this.model.value < this.model.minDate) {
                        this._calendarDate = this.model.minDate;
                        this.isValidState = false;
                        valueExceed = true;
                    }
                    else if (this.model.value > this.model.maxDate) {
                        this._calendarDate = this.model.maxDate;
                        this.isValidState = false;
                        valueExceed = true;
                    }
                    else this.isValidState = true;
                }
                else {
                    if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                    else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                }
            }
            if (dateChange) this.element.val(this._formatter(this.model.value, this.model.dateFormat));
            if (valueExceed && this._getInternalEvents) this._trigger("outOfRange");
        },
        _setCulture: function (culture) {
            this.culture = Globalize.culture(culture);
            if (this.culture) {
                this.model.locale = this.culture.name == "en" ? "en-US" : this.culture.name;
                this.Date.dayNames = this.culture.calendar.days.names;
                this.Date.dayNamesMin = this.culture.calendar.days.namesShort;
                this.Date.abbrDayNames = this.culture.calendar.days.namesAbbr;
                this.Date.monthNames = this.culture.calendar.months.names;
                this.Date.abbrMonthNames = this.culture.calendar.months.namesAbbr;
                this.Date.format = this.culture.calendar.patterns.d;
                if (this.model.dateFormat == '') this.model.dateFormat = this.culture.calendar.patterns.d;
                if (this.model.startDay == -1) this.model.startDay = this.culture.calendar.firstDay;
            }
            this._separator = this._getSeparator();
        },

        _setWaterMark: function () {
            if (this.element != null && this.element.hasClass("e-input")) {
                if (this.model.watermarkText && this.element.val() == "") {
                    this.isValidState = true;
                    this._checkErrorClass();
                }
                if ((!this._isSupport) && this.element.val() == "")
                    this._hiddenInput.css("display", "block").val(this.model.watermarkText);
                else
                    $(this.element).attr("placeholder", this.model.watermarkText);
                return true;
            }
        },

        _setDatePickerPosition: function () {
            if (!this.model.displayInline || this._isInputBox()) {
                var elementObj = this.element.is('input') ? this.wrapper : this.element;
                var pos = this._getOffset(elementObj), winWidth,
                winBottomHeight = $(document).scrollTop() + $(window).height() - (pos.top + $(elementObj).outerHeight()),
                winTopHeight = pos.top - $(document).scrollTop(),
                popupHeight = this.sfCalendar.outerHeight(),
                popupWidth = this.sfCalendar.outerWidth(),
                left = pos.left,
                totalHeight = elementObj.outerHeight(),
                border = (totalHeight - elementObj.height()) / 2,
                maxZ = this._getZindexPartial(), popupmargin = 3,
                topPos = (popupHeight < winBottomHeight || popupHeight > winTopHeight) ? pos.top + totalHeight + popupmargin : pos.top - popupHeight - popupmargin; // popupmargin denotes space b/w the element and the popup.
                winWidth = $(document).scrollLeft() + $(window).width() - left;
                if (this.model.enableRTL || popupWidth > winWidth && (popupWidth < left + elementObj.outerWidth()) && !ej.isNullOrUndefined(this.wrapper)) left += this.wrapper.width() - this.sfCalendar.width();
                this.sfCalendar.css({
                    "left": left + "px",
                    "top": topPos + "px",
                    "z-index": maxZ
                });
            }
        },

        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },

        _getZindexPartial: function () {
            return ej.util.getZindexPartial(this.element, this.sfCalendar);
        },

        _setMinDate: function (d) {
            this.model.minDate = this._checkInstanceType(d);
            if (!this.model.minDate) {
                this.model.minDate = (new Date('11/31/1899'));
            }
        },

        _setMaxDate: function (d) {
            this.model.maxDate = this._checkInstanceType(d);
            if (!this.model.maxDate) {
                this.model.maxDate = (new Date('12/31/2099')); // using the JS Date.parse function which expects mm/dd/yyyy
            }
        },
        _setDateValue: function (date) {
            var newDate = this._checkDateObject(date);
            if (newDate != null) {
                this.isValidState = true;               
                this.model.value = new Date(newDate.toString());
                this._validateMinMaxDate();
                this._preTxtValue = this.element.val(this._formatter(this.model.value, this.model.dateFormat));                
            }
            else {
                (typeof date === "string" && this.model.enableStrictMode) ? this.element.val(date) : this.element.val(null);
                this.model.value = null;
                this.isValidState = (this.element.val() == "" || ej.isNullOrUndefined(this.element.val())) ? true : false;
            }
            this._removeWatermark();
        },
        _updateInputVal: function () {
            var val = this._validateValue();
            if (val != null && this.sfCalendar.find('.e-datepicker-days').is(':visible'))             
                    this._refreshDatepicker();            
        },
        _validateInputVal: function () {
            var val = this._validateValue();
            if (val != null) {
                if (!this.model.enableStrictMode) {
                    if (val <= this.model.maxDate && val >= this.model.minDate)
                        this.isValidState = true;
                    else {
                        this.model.value = "";
                        this.isValidState = true;
                    }
                }
            }
        },

        _validateValue: function () {
            var value = this._parseDate(this.element.val());
            return this._validateYearValue(value);
        },
        _getSeparator: function () {
            if (this.culture) {
                formats = this.culture.calendar.patterns.d;
            }
            else formats = this.model.dateFormat;
            var regex = new RegExp("^[a-zA-Z0-9]+$");
            for (var i = 0; i < formats.length; i++) {
                if (!regex.test(formats[i])) return formats[i];
            }
        },
        _checkIE7: function () {
            if (navigator.appName == 'Microsoft Internet Explorer') {
                var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})"), version = -1;
                if (re.exec(navigator.userAgent) != null)
                    version = parseFloat(RegExp.$1);
                if (version >= 7 && version < 8) return true;
            }
            return false;
        },
        _isValidDate: function (dateObj) {
            return dateObj && typeof dateObj.getTime === "function" && isFinite(dateObj.getTime());
        },

        //Date formatter - Convert date object to specific date format
        _formatter: function (date, format) {
            var newFormat = this._checkFormat(format);
            return Globalize.format(date, newFormat, this.model.locale);
        },
        _parseDate: function (date) {
            var newFormat = this._checkFormat(this.model.dateFormat);
            return Globalize.parseDate(date, newFormat, this.model.locale);
        },
        _checkFormat: function (format) {
            var proxy = this;
            var dateFormatRegExp = this._regExp();
            return format.replace(dateFormatRegExp, function (match) {
                match = match === "/" ? Globalize.culture(proxy.model.locale).calendars.standard['/'] !== "/" ? "'/'" : match : match;
                return match;
            });
        },
        _regExp: function () {
            return /\/dddd|ddd|dd|d|MMMM|MMM|MM|M|yyyy|yy|HH|H|hh|h|mm|m|fff|ff|f|tt|ss|s|zzz|zz|z|gg|g|"[^"]*"|'[^']*'|[/]/g;
        },

        isLeapYear: function (year) {
            return (year % 4 == 0 && year % 100 != 0) || year % 400 == 0;
        },
        //Sets the time component of this Date to zero for cleaner, easier comparison of dates where time is not relevant.
        _zeroTime: function (date) {
            var newDate = typeof date === "string" ? this._parseDate(date) : new Date(date);
            newDate.setMilliseconds(0);
            newDate.setSeconds(0);
            newDate.setMinutes(0);
            newDate.setHours(0);
            return newDate;
        },
        //Convert date object to specific format
        _formatDate: function (date) {
            return this._formatter(date, this.Date.format);
        },

        _getDaysInMonth: function (date) {
            return [31, (this.isLeapYear(date) ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31][date.getMonth()];
        },

        _addDays: function (d, number) {
            d.setDate(d.getDate() + number);
            return d;
        },

        _addYears: function (d, number) {
            d.setFullYear(d.getFullYear() + number);
            return d;
        },

        _addMonths: function (d, number) {
            var tempDatedateMonth = d.getDate();
            d.setMonth(d.getMonth() + number);
            if (tempDatedateMonth > d.getDate())
                this._addDays(d, -d.getDate());
            return d;
        },
        //Checks if the day is a weekend day (Sat or Sun).
        _isWeekend: function (date) {
            return date.getDay() == 0 || date.getDay() == 6;
        },

        _isSpecialDates: function (dates) {
            if (this.model.specialDates) {
                for (var i = 0; i < this.model.specialDates.length; i++) {
                    if (this.model.specialDates[i]) {
                        if (dates.getDate() == this.model.specialDates[i][this._mapField._date].getDate() && dates.getMonth() == this.model.specialDates[i][this._mapField._date].getMonth() && dates.getFullYear() == this.model.specialDates[i][this._mapField._date].getFullYear()) {
                            this._getIndex = i;
                            return true;
                        }
                    }
                }
            }
            return false;
        },
        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _date: null, _tooltip: null, _icon: null };
            mapFld._date = (mapper && mapper.date) ? mapper["date"] : "date";
            mapFld._tooltip = (mapper && mapper.tooltip) ? mapper["tooltip"] : "tooltip";
            mapFld._icon = (mapper && mapper.iconClass) ? mapper["iconClass"] : "iconClass";
            return mapFld;
        },
        _createSpecialDateObject: function () {
            for (var i = 0; i < this.model.specialDates.length; i++) {
                this.model.specialDates[i][this._mapField._date] = this._checkInstanceType(this.model.specialDates[i][this._mapField._date]);
            }
        },

        _getMonthName: function (abbreviated, date) {
            return abbreviated ? this.Date.abbrMonthNames[date.getMonth()] : this.Date.monthNames[date.getMonth()];
        },



        _displayNewMonth: function (m, y) {
            this._setDisplayedMonth(this.displayedMonth + m, this.displayedYear + y, true);
            return false;
        },

        _setDisplayedMonth: function (m, y, rerender) {
            if (this.model.minDate == undefined || this.model.maxDate == undefined) {
                return;
            }
            var s = new Date(this.model.minDate.getTime());
            s.setDate(1);
            var e = new Date(this.model.maxDate.getTime());
            e.setDate(1);

            var t;
            if ((!m && !y) || (isNaN(m) && isNaN(y))) {

                t = this._zeroTime(new Date());
                t.setDate(1);
            } else if (isNaN(m)) {

                t = new Date(y, this.displayedMonth, 1);
            } else if (isNaN(y)) {

                t = new Date(this.displayedYear, m, 1);
            } else {

                t = new Date(y, m, 1);
            }

            if (t.getTime() < s.getTime()) {
                t = s;
            } else if (t.getTime() > e.getTime()) {
                t = e;
            }
            var oldMonth = this.displayedMonth;
            var oldYear = this.displayedYear;
            this.displayedMonth = t.getMonth();
            this.displayedYear = t.getFullYear();
            var tempDate = t;
            if (rerender && (this.displayedMonth != oldMonth || this.displayedYear != oldYear)) {
                this._renderCalendar(this, tempDate);
                this._dateValue = tempDate;
                this._trigger("monthChanged", [this.displayedMonth, this.displayedYear]);
            }
        },
        _clearSelected: function () {
            this.numSelected = 0;
            if (this.model.highlightSection == "week") {
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false).parent().removeClass('e-selected-week');
            }
            else if (this.model.highlightSection == "month") {
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false).parent().parent().removeClass('e-selected-month');
            }
            else if (this.model.highlightSection == "workdays") {
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false).parent().removeClass('e-work-week');
            }
            else
                $('td.e-active', this.sfCalendar).removeClass('e-active').attr('aria-selected', false);

        },
        _addSelected: function () {
            if (this.model.highlightSection == "week") {
                $('td.e-active', this.sfCalendar).parent().addClass('e-selected-week');
            }
            else if (this.model.highlightSection == "month") {
                $('td.e-active, this.sfCalendar').parent().parent().addClass('e-selected-month');
            }
            else if (this.model.highlightSection == "workdays") {
                $('td.e-active', this.sfCalendar).parent().addClass('e-work-week');
            }
        },

        _hideOtherMonths: function (sfCalendar) {
            $('td.other-month', sfCalendar).css("visibility", "hidden");
        },
        _showOtherMonths: function (sfCalendar) {
            $('td.other-month', sfCalendar).css({ 'visibility': 'visible' });
        },
        _otherMonthsVisibility: function () {
            if (this.model.showOtherMonths)
                this._showOtherMonths(this.sfCalendar);
            else
                this._hideOtherMonths(this.sfCalendar);
        },

        _createCalender: function () {
            ej.buildTag("div.e-header").attr((this._isIE8) ? { 'unselectable': 'on' } : {})
                    .append(ej.buildTag("span.e-prev").append(ej.buildTag('a.e-icon e-left-arrow').attr({ 'role': 'button' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})))
                    .append(ej.buildTag("span.e-text").append(ej.buildTag("span.e-datepicker-headertext").text(this._formatter(this._calendarDate, this.model.headerFormat)).attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', 'role': 'heading' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})))
                    .append(ej.buildTag("span.e-next").append(ej.buildTag('a.e-icon e-right-arrow').attr({ 'role': 'button' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})))
                    .appendTo(this.sfCalendar);
            this._enableHeader(this._showHeader);
            var table = ej.buildTag("table.e-dp-viewdays", "", {}, { 'cellspacing': 2 }).data("e-table", "data").attr({ 'role': 'grid', 'aria-activedescendant': '' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            this.sfCalendar.append(table);
            this._renderCalendar(this);
            this._startLevel(this.model.startLevel);
            ej.buildTag("div.e-footer")
                .append(ej.buildTag("span.e-footer-icon"))
                .append(ej.buildTag("span.e-footer-text"))
                .appendTo(this.sfCalendar);
            $('.e-footer-text', this.sfCalendar).html(this.model.buttonText);
            this._enableFooter(this.model.showFooter);
        },
        _enableHeader: function (show) {
            if (show) $(".e-header", this.sfCalendar).show();
            else $(".e-header", this.sfCalendar).hide();
        },
        _enableFooter: function (show) {
            if (show) $('.e-footer', this.sfCalendar).show();
            else $('.e-footer', this.sfCalendar).hide();
        },


        _checkArrows: function (min, max) {
            this._preArrowCondition(min, this.model.minDate.getFullYear());
            this._nextArrowCondition(max, this.model.maxDate.getFullYear());
        },
        _checkDateArrows: function () {
            this._preArrowCondition(this._tempMinDate, this.model.minDate);
            this._nextArrowCondition(this._tempMaxDate, this.model.maxDate);
        },
        _preArrowCondition: function (val1, val2) {
            if (val1 <= val2) this.sfCalendar.find(".e-left-arrow").addClass("e-disable").attr({ "aria-disabled": true });
            else this.sfCalendar.find(".e-left-arrow").removeClass("e-disable").attr({ "aria-disabled": false });
        },
        _nextArrowCondition: function (val1, val2) {
            if (val1 >= val2) this.sfCalendar.find(".e-right-arrow").addClass("e-disable").attr({ "aria-disabled": true });
            else this.sfCalendar.find(".e-right-arrow").removeClass("e-disable").attr({ "aria-disabled": false });
        },

        _previousNextHandler: function (event) {
            if (this.model.readOnly || !this.model.enabled) return false;
            event.preventDefault();
            var element = ($(event.target).is('a')) ? $(event.target.parentNode) : $(event.target);
            var progress = element.hasClass('e-prev') ? true : false;
            this._processNextPrevDate(progress);
        },
        _processNextPrevDate: function (progress) {
            if (progress && this.sfCalendar.find(".e-left-arrow").hasClass("e-disable")) return false;
            else if (!progress && this.sfCalendar.find(".e-right-arrow").hasClass("e-disable")) return false;

            var currentTable = $("table", this.sfCalendar), temp;
            var tClassName = currentTable.get(0).className;
            switch (tClassName) {
                case 'e-dp-viewdays':
                    var step = this.model.stepMonths;
                    if (progress) {
                        if (this._dateValue <= this.model.minDate) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (this._dateValue >= this.model.maxDate) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._addMonths(this._dateValue, (progress ? -step : step));
                    if (this._clickedDate)
                        this._calendarDate = this._clickedDate;
                    this._dateValue = this._dateValue < this.model.minDate ? new Date(this.model.minDate.toString()) :this._dateValue; 
                    this._dateValue = this._dateValue > this.model.maxDate ? new Date(this.model.maxDate.toString()) : this._dateValue;
                    this._renderCalendar(this, this._dateValue);
                    $('.e-datepicker-headertext', this.sfCalendar).text(this._formatter(this._dateValue, this.model.headerFormat));
                    this._addFocus('day', this._hoverDate);
                    var dateRange = this._findFirstLastDay(new Date(this._dateValue.toString()));
                    this._preArrowCondition(dateRange.firstDay, this.model.minDate);
                    this._nextArrowCondition(dateRange.lastDay, this.model.maxDate);
                    break;
                case 'e-dp-viewmonths':
                    var dateValue = this._dateValue;
                    dateValue.setFullYear($('.e-datepicker-headertext', this.sfCalendar).text())
                    if (progress) {
                        if (dateValue.getFullYear() <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (dateValue.getFullYear() >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    this._addYears(dateValue, (progress ? -1 : 1));
                    this._renderCalendar(this, dateValue);
                    temp = dateValue.getFullYear();
                    $('.e-datepicker-headertext', this.sfCalendar).text(temp);
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-months').hide();
                    $($(currentTable).find('.e-datepicker-months')).show();
                    this._addFocus('month', this._hoverMonth);
                    this._checkArrows(temp, temp);
                    break;
                case 'e-dp-viewyears':
                    var yearValue;
                    yearValue = this._dateValue
                    yearValue.setFullYear($(currentTable).find(".e-state-hover").text());
                    if (progress) {
                        if (parseInt($('td.e-year-first:first').text()) <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    } else {
                        if (parseInt($('td.e-year-last:first').prev().text()) >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        }
                    }
                    this._flag = true;
                    if (($(currentTable).find(".e-state-hover").hasClass('e-year-first') && progress) || ($(currentTable).find(".e-state-hover").hasClass('e-year-last') && !progress))
                        this._dateValue.setFullYear(yearValue.getFullYear());
                    else if (($(currentTable).find(".e-state-hover").hasClass('e-year-first') && !progress))
                        this._dateValue.setFullYear(yearValue.getFullYear() + 11);
                    else if (($(currentTable).find(".e-state-hover").hasClass('e-year-last') && progress))
                        this._dateValue.setFullYear(yearValue.getFullYear() - 11);
                    else
                        this._dateValue.setFullYear(yearValue.getFullYear() + (progress ? -10 : 10));
                    this._renderCalendar(this, this._dateValue);
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 10) + 1);
                    $(".e-datepicker-headertext", this.sfCalendar).text((setYear + 1) + ' - ' + (setYear + 10));
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-years').hide();
                    $($(currentTable).find('.e-datepicker-years')).show();
                    this._addFocus('year', this._hoverYear);
                    this._checkArrows(setYear + 1, setYear + 10);
                    break;
                case 'e-dp-viewallyears':
                    var headYears;
                    if (progress) {
                        headYears = parseFloat($('td.e-allyear-first', currentTable.get(0)).text().split('-')[1]);
                        if (headYears <= this.model.minDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        } else {
                            this._flag = true;
                        }

                    } else {
                        headYears = parseFloat($('td.e-allyear-last', currentTable.get(0)).prev().text().split('-')[1]);
                        if (headYears >= this.model.maxDate.getFullYear()) {
                            this._flag = false;
                            return false;
                        } else
                            this._flag = true;
                    }
                    this._dateValue.setFullYear(headYears + (progress ? -10 : 10));
                    this._renderCalendar(this, this._dateValue);
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 100) + 1);
                    temp = parseFloat($('td.e-allyear-last', currentTable.get(0)).prev().text().split('-')[1]);
                    $('.e-datepicker-headertext', this.sfCalendar).text((setYear + 1) + ' - ' + temp);
                    $('tbody,tr.e-week-header', currentTable).not('.e-datepicker-allyears').hide();
                    $($(currentTable).find('.e-datepicker-allyears')).show();
                    this._addFocus('allyear', this._hoverAllYear);
                    this._checkArrows(setYear + 1, temp);
                    break;
            }
            this._layoutChanged();
        },
        _addFocus: function (selection, index) {
            var cls = 'e-current-' + selection;
            if (selection == 'day') cls = 'current-month';
            var items = this.sfCalendar.find('tbody tr td.' + cls);
            if (selection == "month") {
                $(items).each(function (i, ele) {
                    if (parseInt($(ele).attr("index")) == parseInt(index)) {
                        index = i;
                        return;
                    }
                });
            }
            var cell = items[index];
            if (!cell) cell = items.last();
            this.sfCalendar.find('table td').removeClass("e-state-hover");
            $(cell).addClass("e-state-hover");
            this._setActiveState(selection);           
            return index;
        },       
        _setActiveState: function (selection) {
            if (!(this.model.value instanceof Date)) return;
            var items = this.sfCalendar.find('tbody tr td.e-current-' + selection), cell, proxy = this;
            var indx = -1;
            switch (selection) {
                case "month":
                    if (this.model.value.getFullYear() === parseInt($('.e-text', this.sfCalendar).text())) {
                        $(items).each(function (i, ele) {
                            if (parseInt($(ele).attr("index")) == parseInt(proxy.model.value.getMonth())) {
                                indx = i;
                                return;
                            }
                        });
                    }
                    break;
                case "year":
                    var value = this.model.value.getFullYear();
                    $(items).each(function (i, ele) {
                        if (parseInt(ele.innerHTML) == parseInt(value)) {
                            indx = i;
                            return;
                        }
                    });
                    break;
                case "allyear":
                    var start = parseInt(this.model.value.getFullYear()) - ((parseInt(this.model.value.getFullYear()) % 10) + 1);
                    var active = (start + 1) + ' - ' + (start + 10);
                    $(items).each(function (i, ele) {
                        if (parseInt(ele.innerHTML) == parseInt(active)) {
                            indx = i;
                            return;
                        }
                    });
                    break;
            }
            cell = items[indx];
            if (cell) {
                this.sfCalendar.find('table td').removeClass("e-active");
                $(cell).addClass("e-active");
            }
        },
        _setFocusByName: function (name, value) {
            var allValues = this.sfCalendar.find('tbody tr td.e-current-' + name), index, cell;
            $(allValues).each(function (i, ele) {
                if (parseInt(ele.innerHTML) == parseInt(value)) {
                    index = i;
                    return;
                }
            });
            cell = allValues[index];
            if (!cell) cell = allValues.last();
            this.sfCalendar.find('table td').removeClass("e-state-hover");
            $(cell).addClass("e-state-hover");
            this._setActiveState(name);
            return index;
        },
        _getHeaderTxt: function () {
            return this.sfCalendar.find(".e-datepicker-headertext").text();
        },
        _findFirstLastDay: function (value) {
            y = value.getFullYear(), m = value.getMonth();
            var firstDay = new Date(y, m, 1);
            var lastDay = new Date(y, m + 1, 0);
            return { firstDay: firstDay, lastDay: lastDay }
        },
        _forwardNavHandler: function (event) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (event) event.preventDefault();

            var currentTable = $("table", this.sfCalendar);
            var tclassName = $("table", this.sfCalendar).get(0).className, proxy = this, headerTxt;
            switch (tclassName) {
                case 'e-dp-viewdays':
                    this._hoverMonth = this._getDateObj(currentTable.find(".e-state-hover")).getMonth() ||
                                this._getDateObj(currentTable.find(".e-active")).getMonth() || 0;
                    this._startLevel("year");
                    this._addFocus('month', this._hoverMonth);
                    break;
                case 'e-dp-viewmonths':
                    headerTxt = this._getHeaderTxt();
                    this._startLevel("decade");
                    this._hoverYear = this._setFocusByName('year', headerTxt);
                    break;
                case 'e-dp-viewyears':
                    headerTxt = this._getHeaderTxt();
                    this._startLevel("century");
                    this._hoverAllYear = this._setFocusByName('allyear', headerTxt);
                    break;
            }
            this._layoutChanged();
        },
        _cellSelection: function () {
            var currentTable = $("table", this.sfCalendar);
            var tclassName = $("table", this.sfCalendar).get(0).className;
            switch (tclassName) {
                case 'e-dp-viewmonths':
                    this._hoverMonth = this._addFocus('month', this._dateValue.getMonth());
                    break;
                case 'e-dp-viewyears':
                    var dateValue = new Date(this._dateValue.toString());
                    // Navigate to Prev/Next year Calendar while selecting the first/last year in the calendar view.
                    this._navigationToPrevNext('year');
                    // Reasssign the old value
                    this._dateValue = dateValue;
                    this._hoverYear = this._setFocusByName('year', this._dateValue.getFullYear());
                    break;
                case 'e-dp-viewallyears':
                    var dateValue = new Date(this._dateValue.toString());
                    // Navigate to Prev/Next year Calendar while selecting the first/last year in the calendar view.
                    this._navigationToPrevNext('allyear');
                    // Reasssign the old value
                    this._dateValue = dateValue;
                    var setYear = parseInt(this._dateValue.getFullYear()) - ((parseInt(this._dateValue.getFullYear()) % 10) + 1);
                    this._hoverAllYear = this._setFocusByName('allyear', setYear + 1 + ' - ' + setYear + 10);
                    break;
            }
            this._layoutChanged();
        },
        _navigationToPrevNext: function (name) {
            var allValues = this.sfCalendar.find('tbody tr td.e-current-' + name), index, cell;
            var value = this._dateValue.getFullYear();
            $(allValues).each(function (i, ele) {
                if (parseInt(ele.innerHTML) == parseInt(value)) {
                    index = i;
                    return;
                }
            });
            cell = allValues[index];
            if (cell) {
                if ($(cell).hasClass('e-'+name+'-last'))
                    this._processNextPrevDate(false)
                else if ($(cell).hasClass('e-'+name+'-first'))
                    this._processNextPrevDate(true);
            }           
        },
        _backwardNavHandler: function (event) {
            this._animating = true;
            if (this.model.readOnly || !this.model.enabled) return false;
            var element;
            if (event.type) {
                event.preventDefault();
                element = $(event.currentTarget);
            }
            else element = event;
            var cTable = $("table", this.sfCalendar), temp;
            var tclassName = $("table", this.sfCalendar).get(0).className, proxy = this;
            switch (tclassName) {
                case 'e-dp-viewmonths':
                    cTable.removeClass("e-dp-viewmonths").addClass("e-dp-viewdays");
                    this._dateValue = new Date(this._dateValue.getFullYear(), parseInt($(element).attr('index')), 1);
                    this._renderCalendar(this, this._dateValue);
                    $('tbody', cTable).not('.e-datepicker-days,.e-week-header').hide();
                    $($(cTable).find('.e-datepicker-days,.e-week-header')).fadeIn("fast", function () {
                        proxy._addFocus('day', proxy._hoverDate || 0);
                        proxy._animating = false;
                    });
                    $('.e-datepicker-headertext', this.sfCalendar).text(this._formatter(this._dateValue, this.model.headerFormat));
                    break;
                case 'e-dp-viewyears':
                    cTable.removeClass("e-dp-viewyears").addClass("e-dp-viewmonths");
                    this._dateValue.setFullYear(parseInt(element.text()));
                    this._renderCalendar(this, this._dateValue);
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-months').hide();
                    if (ej.isNullOrUndefined(this._hoverMonth) && !ej.isNullOrUndefined(this._dateValue)) this._hoverMonth = this._dateValue.getMonth();
                    $($(cTable).find('.e-datepicker-months')).fadeIn("fast", function () {
                        proxy._addFocus('month', proxy._hoverMonth || 0);
                        proxy._animating = false;
                    });
                    temp = element.text();
                    $('.e-datepicker-headertext', this.sfCalendar).text(temp);
                    this._checkArrows(temp, temp);
                    break;
                case 'e-dp-viewallyears':
                    var headYears = element.text().split('-');
                    cTable.removeClass("e-dp-viewallyears").addClass("e-dp-viewyears");
                    if (headYears[0] < this.model.minDate.getFullYear()) headYears[0] = this.model.minDate.getFullYear().toString();
                    else if (headYears[0] > this.model.maxDate.getFullYear()) headYears[0] = this.model.maxDate.getFullYear().toString();
                    this._renderCalendar(this, (new Date(headYears[0], 0, 1)));
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-years').hide();
                    $($(cTable).find('.e-datepicker-years')).fadeIn("fast", function () {
                        proxy._addFocus('year', proxy._hoverYear || 0);
                        proxy._animating = false;
                    });
                    $('.e-datepicker-headertext', this.sfCalendar).text(headYears[0] + ' - ' + headYears[1]);
                    this._checkArrows(headYears[0], headYears[1]);
                    break;
                default:
                    this._clearSelected();
                    this.sfCalendar.find('table td').removeClass("e-state-hover");
                    element.not('td.other-month').addClass('e-active').attr('aria-selected', true);
                    this._addSelected();

                    this._hoverDate = this._getDateObj(element).getDate() - 1;
                    this._dateValue = new Date(element.attr('date'));
                    this._clickedDate = new Date(element.attr('date'));
                    this._animating = false;
                    break;
            }
            this._layoutChanged();
        },

        _startLevel: function (start) {
            var cTable = $("table", this.sfCalendar);
            var headerText = $(".e-datepicker-headertext", this.sfCalendar), s, e;
            var dateValue = this._dateValue;
            switch (start) {
                case "decade":
                    cTable.removeClass("e-dp-viewallyears e-dp-viewmonths e-dp-viewdays").addClass("e-dp-viewyears");
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-years').hide();
                    $($(cTable).find('.e-datepicker-years')).show();
                    if (this.model.enableStrictMode && this._calendarDate < this._dateValue) dateValue = this._calendarDate;
                    else dateValue = dateValue;
                    var setYear = parseInt(dateValue.getFullYear()) - ((parseInt(dateValue.getFullYear()) % 10) + 1);
                    s = setYear + 1;
                    e = setYear + 10;
                    headerText.text(s + ' - ' + e);
                    this._checkArrows(s, e);
                    this._hoverYear = this._setFocusByName('year', dateValue.getFullYear());
                    break;
                case "century":
                    if (!(this._calendarDate < this._dateValue)) this._renderCalendar(this, dateValue);
                    cTable.removeClass("e-dp-viewyears e-dp-viewdays e-dp-viewmonths").addClass("e-dp-viewallyears");
                    $('tbody,tr.e-week-header', cTable).not('.e-datepicker-allyears').hide();
                    $($(cTable).find('.e-datepicker-allyears')).show();
                    s = parseFloat($('td.e-allyear-first', cTable.get(0)).text().split('-')[1]) + 1;
                    e = parseFloat($('td.e-allyear-last', cTable.get(0)).prev().text().split('-')[1]);
                    var headYears = s + ' - ' + e;
                    headerText.text(headYears);
                    this._checkArrows(s, e);
                    var setYear = parseInt(dateValue.getFullYear()) - ((parseInt(dateValue.getFullYear()) % 10) + 1);
                    this._hoverAllYear = this._setFocusByName('allyear', (setYear + 1) + ' - ' + (setYear + 10));
                    break;
                case "year":
                    cTable.removeClass("e-dp-viewyears e-dp-viewallyears e-dp-viewdays").addClass("e-dp-viewmonths");
                    $('tbody,tr.e-week-header', cTable).hide();
                    $($(cTable).find('.e-datepicker-months')).show();
                    if (this.model.enableStrictMode && this._calendarDate < this._dateValue) s = this._calendarDate.getFullYear();
                    else s = dateValue.getFullYear();
                    headerText.text(s);
                    this._checkArrows(s, s);
                    this._hoverMonth = dateValue.getMonth();
                    this._addFocus('month', this._hoverMonth);
                    break;
                case "month":
                    cTable.removeClass("e-dp-viewyears e-dp-viewallyears e-dp-viewmonths").addClass("e-dp-viewdays ");
                    break;
            }
        },
        _depthLevel: function (depth) {
            var calendarTable = this.sfCalendar;
            switch (depth) {
                case "year":
                    $(calendarTable.find('.e-current-year,.e-current-allyear')).bind("click", $.proxy(this._backwardNavHandler, this));
                    this._on($('.e-current-month', this.sfCalendar), "click", $.proxy(this._onDepthSelectHandler, this));
                    break;
                case "decade":
                    $(calendarTable.find('.e-current-allyear')).bind("click", $.proxy(this._backwardNavHandler, this));
                    $('.e-current-year', this.sfCalendar).bind("click", $.proxy(this._onDepthSelectHandler, this));
                    break;
                case "century":
                    $(calendarTable.find('.e-current-allyear')).bind("click", $.proxy(this._onDepthSelectHandler, this));
                    break;
                case "month":
                    this._on(calendarTable.find('.current-month,.other-month,.e-current-month,.e-current-year,.e-current-allyear'), "click", $.proxy(this._backwardNavHandler, this));
                    this._on(calendarTable.find('.current-month , .other-month'), "click", $.proxy(this._onSetCancelDateHandler, this));
            }
        },
        _onDepthSelectHandler: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if ($(e.target).hasClass("e-current-month"))
                this._dateValue = new Date(this._dateValue.setMonth(parseInt(e.target.attributes["index"].value)));
            else if ($(e.target).hasClass("e-current-year"))
                this._dateValue = new Date(this._dateValue.setFullYear(parseInt(e.target.innerHTML)));
            else if ($(e.target).hasClass("e-current-allyear"))
                this._dateValue = new Date(this._dateValue.setFullYear(parseInt(e.target.innerHTML)));
            this._onSetCancelDateHandler(e);
        },

        _datepickerMonths: function (tbody, calendarTable, currentDate) {
            var dc = function (a) {
                return document.createElement(a);
            };
            var month = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(dc('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(dc('td'))
                        .addClass('e-current-month e-state-default')
                        .attr({ 'index': month }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})
                        .html(this.Date.abbrMonthNames[month++]);
                    if (currentDate.getFullYear() < this.model.minDate.getFullYear() || currentDate.getFullYear() > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate');
                        td.removeClass('e-current-month');
                    }
                    else if ((currentDate.getFullYear() <= this.model.minDate.getFullYear() && month < this.model.minDate.getMonth() + 1) ||
                        (currentDate.getFullYear() >= this.model.maxDate.getFullYear() && month > this.model.maxDate.getMonth() + 1)) {
                        td.addClass('e-hidedate');
                        td.removeClass('e-current-month');
                    }
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
            var s = currentDate.getFullYear();
            this._checkArrows(s, s);
        },

        _datepickerYears: function (tbody, calendarTable, currentYear) {
            var dc = function (a) {
                return document.createElement(a);
            };
            var Year = parseInt(currentYear) - ((parseInt(currentYear) % 10) + 1);
            var years = [];
            for (var j = 0; j < 12; j++) {
                years.push(Year + j);
            }
            var year = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(dc('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(dc('td'));
                    td.attr((this._isIE8) ? { 'unselectable': 'on' } : {});
                    if (year == 0)                     
                        td.addClass('e-year-first e-current-year ');
                     else if (year == 11)                      
                        td.addClass('e-year-last e-current-year ');
                     else
                        td.addClass('e-current-year e-state-default');
                    if (years[year] < this.model.minDate.getFullYear() || years[year] > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate');
                        td.removeClass('e-current-year');
                    }
                    td.html(years[year++]);
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
            this._checkArrows(years[0], years[years.length]);
        },

        _datepickerAllYears: function (tbody, calendarTable, currentYear) {
            var Year = parseInt(currentYear) - ((parseInt(currentYear) % 100) + 10);
            var headYear = Year;
            var years = [], newline = this._isIE8 || this._isIE9 ? "" : "\n";

            for (var j = 0; j < 12; j++) {
                years.push(parseInt(Year) + " -" + newline + (parseInt(Year+9) < this.model.maxDate.getFullYear() ? parseInt(Year+9): this.model.maxDate.getFullYear()));
                Year = Year + 10;
            }
            var year = 0;
            for (var i = 0; i < 3; i++) {
                var row = $(document.createElement('tr'));
                for (var j = 0; j < 4; j++) {
                    var td = $(document.createElement('td'));
                    td.attr((this._isIE8) ? { 'unselectable': 'on' } : {});
                    if (year == 0)                       
                        td.addClass('e-allyear-first e-current-allyear ');
                     else if (year == 11)                      
                        td.addClass('e-allyear-last e-current-allyear ');
                     else
                        td.addClass('e-current-allyear e-state-default');
                    if (parseInt(years[year].split('-\n')[1]) < this.model.minDate.getFullYear() || parseInt(years[year].split('-\n')[0]) > this.model.maxDate.getFullYear()) {
                        td.addClass('e-hidedate');
                        td.removeClass('e-current-allyear');
                    }
                    td.html(years[year++]);
                    row.append(td);
                }
                tbody.append(row);
            }
            calendarTable.append(tbody);
        },
        _renderHeader: function (dpObject) {
            var thead = $(document.createElement('thead'));
            var cultureObj = Globalize.culture(this.model.locale).calendars.standard.days;
            if (dpObject.model.dayHeaderFormat != "none") {
                var headRow = ej.buildTag("tr.e-week-header").attr({ 'role': 'columnheader' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
                for (var i = this.Date.firstDayOfWeek; i < this.Date.firstDayOfWeek + 7; i++) {
                    var weekday = i % 7;
                    var day = cultureObj.names[weekday];
                    var headerday;
                    if (dpObject.model.dayHeaderFormat == "short")
                        headerday = cultureObj.namesAbbr[weekday];
                    else if (dpObject.model.dayHeaderFormat == "long") headerday = day;
                    else headerday = cultureObj.namesShort[weekday];
                    var th = ej.buildTag("th", "", {}, { 'scope': 'col', 'abbr': day, 'date': day, 'title': this._formatter(day, "dddd"), 'class': (weekday == 0 || weekday == 6 ? 'e-week-end' : 'e-week-day') }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})
                            .html(headerday);
                    headRow.append(th);
                }
            };
            return thead.append(headRow);
        },

        _renderCalendar: function (dpObject, date) {
            var proxy = this, today;
            dpObject = $.extend({}, ej.DatePicker.prototype.defaults, dpObject);
            this.Date.firstDayOfWeek = this.model.startDay;
            if (date) today = date;
            else if (this._calendarDate) today = this._calendarDate;
            else today = proxy._zeroTime(new Date());
            var calendarTable = $('table', this.sfCalendar);
            calendarTable.empty();

            calendarTable.append(this._renderHeader(dpObject));

            var tbody = ej.buildTag('tbody.e-datepicker-allyears', "", { 'display': 'none' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            this._datepickerAllYears(tbody, calendarTable, today.getFullYear());

            tbody = ej.buildTag("tbody.e-datepicker-years", "", { 'display': 'none' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            this._datepickerYears(tbody, calendarTable, today.getFullYear());
            var month = dpObject.model.month == undefined ? today.getMonth() : dpObject.model.month;
            var year = dpObject.model.year || today.getFullYear();
            var currentDate = (new Date(year, month, 1, 0, 0, 0));
            var firstDayOffset = this.Date.firstDayOfWeek - currentDate.getDay() + 1;
            if (firstDayOffset > 1) firstDayOffset -= 7;
            var weeksToDraw = Math.ceil(((-1 * firstDayOffset + 1) + this._getDaysInMonth(currentDate)) / 7);
            this._addDays(currentDate, (firstDayOffset - 1));
            var newdate = proxy._zeroTime(new Date());
            var selected = this._calendarDate;
            tbody = ej.buildTag('tbody.e-datepicker-months', "", { 'display': 'none' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {});

            this._datepickerMonths(tbody, calendarTable, today);

            tbody = ej.buildTag('tbody.e-datepicker-days', "", { 'display': 'none' }).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            var w = 0, _first = true, _last = true;
            while (w++ < weeksToDraw) {
                var r = jQuery(document.createElement('tr'));
                for (var i = 0; i < 7; i++) {
                    var thisMonth = currentDate.getMonth() == month;
                    var checkSpecialDate = this._isSpecialDates(currentDate);
                    var index = this._getIndex;
                    var d = $(document.createElement('td')).
                        html(checkSpecialDate ? '<span></span>' + currentDate.getDate() : currentDate.getDate() + '')
                        .attr({

                            'date': currentDate.toDateString(),
                            'title': (this.model.showTooltip ? (checkSpecialDate && this.model.specialDates[index][this._mapField._tooltip] ? this.model.specialDates[index][this._mapField._tooltip] : this._formatter(currentDate, this.model.tooltipFormat)) : ''),
                            'aria-selected': false,
                            'role': 'gridcell'
                        }).attr((this._isIE8) ? { 'unselectable': 'on' } : {})
                        .addClass((thisMonth ? 'current-month e-state-default ' : 'other-month e-state-default ') +
                            (this._isWeekend(currentDate) ? (this._ejHLWeekEnd ? 'e-dp-weekend e-week-end ' : (this.model.highlightWeekend ? 'e-week-end ' : '')) : 'e-week-day ') +                           
                            (thisMonth && currentDate.getTime() == newdate.getTime() ? 'today ' : ''));
                     
                    d.find('span:first-of-type').addClass((checkSpecialDate ? (this.model.specialDates[index][this._mapField._icon] ? 'e-special-date-icon ' + this.model.specialDates[index][this._mapField._icon] + ' ' : 'e-special-day') : ''));

                    if (selected.getTime() == currentDate.getTime() && thisMonth) {
                        d.addClass('e-active').attr({ 'aria-selected': true });
                        if (this.model.highlightSection == "week") {
                            r.addClass('e-selected-week');
                        }
                        if (this.model.highlightSection == "month") {
                            tbody.addClass('e-selected-month');
                        }
                        if (this.model.highlightSection == "workdays") {
                            r.addClass('e-work-week');
                        }

                        if (!this._hoverDate) {
                            d.addClass('e-state-hover');
                            this._hoverDate = currentDate.getDate() - 1;
                        }
                    }
                    var cond = true;
                    if (currentDate < this.model.minDate || currentDate > this.model.maxDate) {
                        d.addClass('e-hidedate');
                        d.removeClass('current-month');
                        if (this.model.showOtherMonths) d.removeClass('other-month');
                        cond = _last = false;
                    }
                    if (thisMonth) {
                        if (cond && _first) {
                            this._tempMinDate = currentDate;
                            _first = false; _last = true;
                        }
                        if (_last) this._tempMaxDate = currentDate;
                    }
                    this._trigger("beforeDateCreate", { date: currentDate, element: d });
                    r.append(d);
                    currentDate = new Date(currentDate.getFullYear(), currentDate.getMonth(), currentDate.getDate() + 1, 0, 0, 0);
                }
                tbody.append(r);
            }
            calendarTable.append(tbody);
            (this._isIE8 || this._isIE7) ? $(tbody).css("display", "block") : $(tbody).fadeIn("fast");
            if (this.model.startLevel === this.model.depthLevel)
                this._depthLevel(this.model.depthLevel);
            else if (this.model.depthLevel != "month" && this.model.depthLevel != "") {
                if (this.model.startLevel == "century")
                    this._depthLevel(this.model.depthLevel);
                else if (this.model.startLevel == "decade" && this.model.depthLevel != "century")
                    this._depthLevel(this.model.depthLevel);
                else if (this.model.startLevel == "year" && this.model.depthLevel != "decade" && this.model.depthLevel != "century")
                    this._depthLevel(this.model.depthLevel);
                else {
                    this._on(calendarTable.find('.current-month,.other-month,.e-current-month,.e-current-year,.e-current-allyear'), "click", $.proxy(this._backwardNavHandler, this));
                    this._on(calendarTable.find('.current-month , .other-month'), "click", $.proxy(this._onSetCancelDateHandler, this));
                }
            }
            else {
                this._on(calendarTable.find('.current-month,.other-month,.e-current-month,.e-current-year,.e-current-allyear'), "click", $.proxy(this._backwardNavHandler, this));
                this._on(calendarTable.find('.current-month , .other-month'), "click", $.proxy(this._onSetCancelDateHandler, this));
            }

            this._otherMonthsVisibility();
            this._checkDateArrows();
        },



        _keyboardNavigation: function (e) {
            if (this._animating) return false;
            if ((this._isOpen) && (e.keyCode == 37 || e.keyCode == 38 || e.keyCode == 39 || e.keyCode == 40 || e.keyCode == 13 || e.keyCode == 36 || e.keyCode == 35)) {
                e.preventDefault && e.preventDefault();
                var t = { row: null, col: null };

                t.col = this.sfCalendar.find('tbody tr td.e-state-hover').index();
                t.row = this.sfCalendar.find('tbody tr td.e-state-hover').parent().index();

                t.col = (t.col != -1) ? t.col + 1 : this.sfCalendar.find('tbody tr td.e-active').index() + 1;
                t.row = (t.row != -1) ? t.row + 1 : this.sfCalendar.find('tbody tr td.e-active').parent().index() + 1;

                var tableClass = this.sfCalendar.find('table')[0].className, next, rowLength = 3, colLength = 4;
                switch (tableClass) {
                    case "e-dp-viewallyears":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "yearall", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverAllYear = this.sfCalendar.find('tbody.e-datepicker-allyears tr td').index(next);
                        break;
                    case "e-dp-viewyears":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "year", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverYear = this.sfCalendar.find('tbody.e-datepicker-years tr td').index(next);
                        break;
                    case "e-dp-viewmonths":
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "month", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverMonth = this.sfCalendar.find('tbody.e-datepicker-months tr td').index(next);
                        break;
                    case "e-dp-viewdays":
                        rowLength = this.sfCalendar.find('tbody.e-datepicker-days tr').length, colLength = 7;
                        next = this._changeRowCol(t, e.keyCode, rowLength, colLength, "day", e.ctrlKey);
                        if (!e.ctrlKey) this._hoverDate = this._getDateObj(next).getDate() - 1;
                        break;
                }
                if (!e.ctrlKey) {
                    this.sfCalendar.find('table td').removeClass("e-state-hover");
                    next.addClass("e-state-hover");
                }
            }
            else if (!this.model.displayInline && (e.keyCode == 27 || e.keyCode == 9)) { this.hide(); e.stopPropagation(); }
            else if (e.altKey && e.keyCode == 40) { this.show(); return false; }
        },
        _changeRowCol: function (t, key, rows, cols, target, ctrlKey) {
            var eleClass, cls = { parent: null, child: null };
            switch (target) {
                case "day": eleClass = "tbody.e-datepicker-days tr td.current-month";
                    cls.parent = ".e-datepicker-days", cls.child = ".current-month";
                    break;
                case "month": eleClass = "tbody.e-datepicker-months tr td.e-current-month";
                    cls.parent = ".e-datepicker-months", cls.child = ".e-current-month";
                    break;
                case "year": eleClass = "tbody.e-datepicker-years tr td.e-current-year";
                    cls.parent = ".e-datepicker-years", cls.child = ".e-current-year";
                    break;
                case "yearall": eleClass = "tbody.e-datepicker-allyears tr td.e-current-allyear";
                    cls.parent = ".e-datepicker-allyears", cls.child = ".e-current-allyear";
                    break;
            }
            if (t.row <= 0 && t.col <= 0)
                return this.sfCalendar.find(eleClass + ':first');
            var cell, proxy = this;
            switch (key) {
                case 36:
                    return this.sfCalendar.find(eleClass + ':first');
                    break;
                case 35:
                    return this.sfCalendar.find(eleClass + ':last');
                    break;
                case 38:
                    if (ctrlKey && this.model.allowDrilDown) {
                        this._forwardNavHandler();
                    }
                    else if (t.row > 1) {
                        t.row -= 1;
                    }
                    else {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):last');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        cell = this._findVisible(t, cls, "up");
                        if (cell !== null) return cell;
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):last');
                    }
                    return cell;
                    break;
                case 37:
                    if (ctrlKey) {
                        this._processNextPrevDate(true);
                        return this.sfCalendar.find('tbody tr td.e-state-hover');
                    }
                    else if (t.col > 1)
                        t.col -= 1;
                    else if (t.row > 1) {
                        t = { row: t.row - 1, col: cols }
                    }
                    else {
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':last');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        cell = this._findVisible(t, cls, "left");
                        if (cell !== null) return cell;
                        this._processNextPrevDate(true);
                        cell = this.sfCalendar.find(eleClass + ':last');
                    }
                    return cell;
                    break;
                case 39:
                    if (ctrlKey) {
                        this._processNextPrevDate(false);
                        return this.sfCalendar.find('tbody tr td.e-state-hover');
                    }
                    else if (t.col < cols)
                        t.col += 1;
                    else if (t.row < rows) {
                        t = { row: t.row + 1, col: 1 }
                    }
                    else {
                        this._processNextPrevDate(false);
                        cell = this.sfCalendar.find(eleClass + ':first');
                        return cell;
                    }
                    cell = this._getCell(t, cls);
                    if (cell.length <= 0) {
                        cell = this._findVisible(t, cls, "right");
                        if (cell !== null) return cell;
                        this._processNextPrevDate(false);
                        cell = this.sfCalendar.find(eleClass + ':first');
                    }
                    return cell;
                    break;
                case 40:
                    if (!ctrlKey) {
                        if (t.row < rows) {
                            t.row += 1;
                        }
                        else {
                            this._processNextPrevDate(false);
                            cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):first');
                            return cell;
                        }
                        cell = this._getCell(t, cls);
                        if (cell.length <= 0) {
                            cell = this._findVisible(t, cls, "down");
                            if (cell !== null) return cell;
                            this._processNextPrevDate(false);
                            cell = this.sfCalendar.find(eleClass + ':nth-child(' + t.col + '):first');
                        }
                        return cell;
                        break;
                    }
                case 13:
                    var tclassName = $("table", this.sfCalendar).get(0).className, ele, element;
                    ele = this._getCell(t, cls); element = $(ele)[0];
                    if (tclassName == "e-dp-viewmonths" && this.model.startLevel == "year" && this.model.depthLevel == "year") {
                        this._dateValue = new Date(this._dateValue.setMonth(parseInt(element.attributes["index"].value)));
                        this._onSetCancelDateHandler({ type: null, target: ele });
                    }
                    else if ((tclassName == "e-dp-viewyears" && this.model.startLevel == "decade" && this.model.depthLevel == "decade") ||
                        (tclassName == "e-dp-viewallyears" && this.model.startLevel == "century" && this.model.depthLevel == "century")) {
                        this._dateValue = new Date(this._dateValue.setFullYear(parseInt(element.innerHTML)));
                        this._onSetCancelDateHandler({ type: null, target: ele });
                    }
                    else if (tclassName == "e-dp-viewdays") {
                        this._backwardNavHandler(ele);
                        this._onSetCancelDateHandler({ type: null, target: ele });
                    }
                    else
                        this._backwardNavHandler(ele);
                    break;
            }
            return this._getCell(t, cls);
        },
        _findVisible: function (t, cls, key) {
            var cols = t.col, rows = t.row, requiredClass = cls.child.slice(1, cls.child.length);
            for (i = 0; i >= 0; i++) {
                nextElement = this.sfCalendar.find('tbody' + cls.parent + ' tr:nth-child(' + rows + ') td:nth-child(' + cols + ')');
                if (nextElement.length <= 0) {
                    return null;
                }
                if (nextElement.hasClass('e-disable') || !nextElement.is(":visible")) {
                    key == "right" || key == "left" ? (key == "right" ? cols++ : cols--) : (key == "down" ? rows++ : rows--);
                    if ((rows <= 0) || (rows > this.sfCalendar.find('tbody' + cls.parent + ' tr').length)) {
                        // No more rows there in popup.
                        return null;
                    }
                    // Column exceeds the range. 
                    if (cols > this.sfCalendar.find('tbody' + cls.parent + ' tr:nth-child(' + rows + ') td').length) {
                        //move to next row and select first column
                        rows++;
                        cols = 1;
                    }
                    if (cols <= 0) {
                        //move to previous row and select last column
                        rows--;
                        cols = this.sfCalendar.find('tbody' + cls.parent + ' tr:nth-child(' + rows + ') td').length;
                    }
                    // Row exceeds the range.
                    if ((rows <= 0) || (rows > this.sfCalendar.find('tbody' + cls.parent + ' tr').length)) {
                        // No more rows there in popup.
                        return null;
                    }
                } else if (nextElement.hasClass('other-month')) {
                    return null;
                } else if (nextElement.hasClass(requiredClass)) {
                    t.col = cols; t.row = rows;
                    return nextElement;
                }
            }
        },
        _getCell: function (t, cls) {
            return this.sfCalendar.find('tbody' + cls.parent + ' tr:nth-child(' + t.row + ') td' + cls.child + ':nth-child(' + t.col + ')');
        },
        _getDateObj: function (element) {
            return new Date(element.attr("date"));
        },
        _touchCalendar: function (e) {
            var tableClass = this.sfCalendar.find('table')[0].className;
            switch (e.type) {
                case "pinchin":
                    if (tableClass != "e-dp-viewdays")
                        this._keyboardNavigation({ keyCode: 13 });
                    break;
                case "pinchout":
                    if (tableClass != "e-dp-viewallyears" && this.model.allowDrilDown)
                        this._forwardNavHandler();
                    break;
                case "swipeleft":
                    this._processNextPrevDate(false);
                    break;
                case "swiperight":
                    this._processNextPrevDate(true);
                    break;
            }
        },

        show: function (e) {
            if (this._isOpen) return false;
            var proxy = this;
            this._popupOpen = true;
            var previous = this._preValue != null ? new Date(this._preValue.toString()) : this._preValue;
            if (!this.model.enabled) return;
            if (!this.model.displayInline) this._setDatePickerPosition();
            if (this._trigger("beforeOpen", { element: this.sfCalendar, events: e })) return false;
            this.sfCalendar.attr({ 'aria-hidden': 'false' });
            proxy._isOpen = true;
            this.sfCalendar.slideDown(this.model.enableAnimation ? this.animation.open.duration : 0, this.animation.open.type, function () {
                if (!proxy.model.displayInline)
                    $(document).bind("mousedown", $.proxy(proxy._onDocumentClick, proxy));
            });
            if (this.element.val() !== "") this._updateInputVal();                   
            this._refreshLevel(previous);
            this._trigger("open", { prevDate: this._prevDate, value: this._formatter(this.model.value, this.model.dateFormat) });
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            if (!this.model.displayInline)
                this._on(ej.getScrollableParents(this.wrapper), "scroll", this.hide);
            this._isInputBox() && this.wrapper.addClass("e-active");
        },


        hide: function (e) {
            if (!this._isOpen || this._getInternalEvents) return false;
            if (this._trigger("beforeClose", { element: this.sfCalendar, events: e })) return false;
            var proxy = this;
            this._popupOpen = false;
            this.sfCalendar.attr({ 'aria-hidden': 'true' });
            this.sfCalendar.slideUp(this.model.enableAnimation ? this.animation.close.duration : 0, this.animation.close.type , function () {
                proxy._isOpen = false;
                $(document).unbind("mousedown", $.proxy(proxy._onDocumentClick, proxy));
                proxy._setWaterMark();
            });
            if (this.element.val() != "") this._validateInputVal();
            this._trigger("close", { prevDate: this._prevDate, value: this._formatter(this.model.value, this.model.dateFormat) });
            $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
            this._off(ej.getScrollableParents(this.wrapper), "scroll", this.hide);
            this._isInputBox() && this.wrapper.removeClass("e-active");
        },


        enable: function () {
            this.model.enabled = true;
            this.element.removeClass('e-disable').attr({ "aria-disabled": false });
			this.element.removeAttr("disabled");
            if (this.dateIcon) this.dateIcon.removeClass('e-disable').attr({ "aria-disabled": false });
            if (this._isIE8 && this.dateIcon) this.dateIcon.children().removeClass("e-disable");
            this.element.removeAttr("disabled");
            if (!this._isSupport)
                this._hiddenInput.removeAttr("disabled");
            this.sfCalendar.removeClass('e-disable').attr({ "aria-disabled": false });
        },


        disable: function () {
            this.model.enabled = false;
            this.element.addClass('e-disable').attr({ "aria-disabled": true });
			this.element.attr("disabled","disabled");
            if (this.dateIcon) this.dateIcon.addClass('e-disable').attr({ "aria-disabled": true });
            if (this._isIE8 && this.dateIcon) this.dateIcon.children().addClass("e-disable");
            this.element.attr("disabled", "disabled");
            if (!this._isSupport)
                this._hiddenInput.attr("disabled", "disabled");
            this.sfCalendar.addClass('e-disable').attr({ "aria-disabled": true });
            if (this._isOpen) {
                if (this.element.is(':input')) this.element.blur();
                if (!this.model.displayInline) this.hide();
            }
        },

        getValue: function () { return this._formatter(this.model.value, this.model.dateFormat); },

        _wireEvents: function () {
            this._allowQuickPick(this.model.allowDrilDown);
            this._on($('.e-next', this.sfCalendar), "click", $.proxy(this._previousNextHandler, this));
            this._on($('.e-prev', this.sfCalendar), "click", $.proxy(this._previousNextHandler, this));

            if (this.element.is(":input") && (this.model.allowEdit)) {
                this._on(this.element, "blur", this._onFocusOut);
                this._on(this.element, "focus", this._onFocusIn);
                this._on(this.element, "keyup", this._keyUp);
            }

            if (!this.model.allowEdit) {
                this.element.attr("readonly", "readonly");
                this.element.bind("mousedown", $.proxy(this._showDatePopUp, this));
            }

            if (this.model.showFooter)
                this._on($('.e-footer', this.sfCalendar), "click", this._setCurrDate);
            this._on(this.sfCalendar, "pinchin pinchout swipeleft swiperight", $.proxy(this._touchCalendar, this));
        },
        _bindDateButton: function () {
            this._on(this.dateIcon, "mousedown", this._showDatePopUp);
            if (this.model.allowEdit)
                this.element.unbind("mousedown", $.proxy(this._showDatePopUp, this));
        },
        _bindInputEvent: function () {
            this._off(this.dateIcon, "mousedown", this._showDatePopUp);
        },

        _onFocusOut: function (e) {
            this._isFocused = false;
            var previous = this._preValue != null ? new Date(this._preValue.toString()) : this._preValue;
            this._validateOnFocusOut(this._validateValue(), e);
            this.wrapper.removeClass("e-focus");
            if ((!this._isOpen || this.model.displayInline) && !this._setWaterMark() && !this._compareDate(this._preValue, this._parseDate(this.element.val(), this.model.dateFormat))) this._updateInputVal();
            if ((!this._isOpen || this.model.displayInline)) this._refreshLevel(previous);
            if (this.element.val() != "" && (!this._isOpen || this.model.displayInline)) { this._validateInputVal(); }
            this.element.unbind("keydown", $.proxy(this._keyboardNavigation, this));
            if (!this.model.showPopupButton) this._off(this.element, "click", this._elementClick);
            var _currentVal = this.element.val();
            var data = { prevDate: this._prevDate, value: _currentVal };
            this._trigger("focusOut", data);
            this._checkErrorClass();
        },
        _onFocusIn: function (e) {
            if (this._isSupport) {
                e.preventDefault();
                this._isFocused = true;
            }
            this.wrapper.removeClass('e-error').attr('aria-invalid', "false");
            this.isValidState = true;
            this.wrapper.addClass("e-focus");
            if (this.model.readOnly)
                return;
            if (!this._isSupport) this._hiddenInput.css("display", "none");
            this.element.bind("keydown", $.proxy(this._keyboardNavigation, this));
            if (!this.model.showPopupButton && !this.model.readOnly) this.show(e);
            if (!this.model.showPopupButton) this._on(this.element, "click", this._elementClick);
            this._trigger("focusIn", { value: this.model.value });
        },
        _elementClick: function (e) {
            if (!this._popupOpen) this.show(e);
        },
        _removeWatermark: function () {
            if (this.element.val() != "" && !this._isSupport)
                this._hiddenInput.css("display", "none");
        },
        _refreshPopup: function () {
            this._refreshDatepicker();
            this._startLevel(this.model.startLevel);
        },
        _refreshLevel: function (previous) {
            if ((this.model.startLevel == this.model.depthLevel) && this.model.startLevel != "month") {
                var val = this._stringToObject(this.element.val());
                val = this._validateYearValue(val);
                if (val)
                    if (!this._compareDate(previous, val))
                        this._refreshPopup();
            }
        },
        _validateOnFocusOut: function (val, e) {
            var calenderDate = this._formatter(this._calendarDate, this.model.dateFormat);
            var currDate = this._formatter(this._parseDate(this._formatter(new Date(), "MM/dd/yyyy")), this.model.dateFormat);
            var dateChange = false, valueExceed = false;
            if (val != null && !this.model.enableStrictMode) {
                if (ej.isNullOrUndefined(this.model.value))
                    this.model.value = this._parseDate(this.element.val());
                if (this.model.maxDate < this.model.minDate) this.model.minDate = this.model.maxDate;
                if (!this.model.enableStrictMode) {
                    if (val) {
                        if ((val < this.model.minDate) || (val > this.model.maxDate)) {
                            dateChange = true,
                            this._calendarDate = val = val < this.model.minDate ? this.model.minDate : this.model.maxDate
                        }
                    }
                    else {
                        this.element.val("");
                        if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                        else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                    }
                    this.isValidState = true;
                }
                if (dateChange) this.element.val(this._formatter(val, this.model.dateFormat));
                if (!this._compareDate(this._preValue, this._parseDate(this.element.val(), this.model.dateFormat))) this._triggerChangeEvent(e);
            }
            else if (val == null && !this.model.enableStrictMode) {
                if (this._preTxtValue == null || this.element.val() == "") {
                    this.element.val("");
                    if (!this._isSupport) this._hiddenInput.css("display", "block");
                } else
                    this.element.val(calenderDate);
                this._triggerChangeEvent(e);
            }
            else {
                if (val) {
                    if ((val < this.model.minDate) || (val > this.model.maxDate)) {
                        this.isValidState = false, valueExceed = true,
                        this._calendarDate = val < this.model.minDate ? this.model.minDate : this.model.maxDate
                    }
                    else
                        this.isValidState = true;
                    this._triggerChangeEvent(e);
                    if (valueExceed && this._getInternalEvents) this._trigger("outOfRange");
                }
                else {
                    this.isValidState = false;
                    if (this._calendarDate < this.model.minDate) this._calendarDate = this.model.minDate;
                    else if (this._calendarDate > this.model.maxDate) this._calendarDate = this.model.maxDate;
                }
            }
        },
        _keyUp: function (e) {
            if (e.keyCode === 13) {
                var previous = this._preValue != null ? new Date(this._preValue.toString()) : this._preValue;
                this._validateOnFocusOut(this._validateValue(), e);
                if ((!this._isOpen || this.model.displayInline) && !this._setWaterMark() && !this._compareDate(this._preValue, this._parseDate(this.element.val(), this.model.dateFormat))) this._updateInputVal();
                if ((!this._isOpen || this.model.displayInline)) this._refreshLevel(previous);
                if (this.element.val() != "" && (!this._isOpen || this.model.displayInline)) { this._validateInputVal(); }
                this._checkErrorClass();
            }
        },
        _showhidePopup: function (e) {
            if (!this.model.enabled) return false;
            if (this._isOpen) {
                if (!this._isFocused && this.element.is(':input')) this.element.focus();
                if (!this._cancelValue) this.hide(e);
            }
            else {
                if (!this._isFocused && this.element.is(':input')) this.element.focus();
                this.show(e);
            }
        },
        _compareDate: function (first, second) {
            var result = (+first === +second) ? true : false;
            return result;
        },
        _triggerChangeEvent: function (e) {
            var _currentVal = this.element.val() == "" ? null : this.element.val();
            this._prevDate = this._formatter(this._preValue, this.model.dateFormat);
            var data = { prevDate: this._prevDate, value: _currentVal };
            var currentValue = this._parseDate(_currentVal, this.model.dateFormat);
            currentValue = this._validateYearValue(currentValue);
            if (!this._compareDate(this._preValue, currentValue)) {
                this._preValue = this.model.value = currentValue;
                if (this.model.displayInline && !this._isInputBox()) this._hiddenInput.attr('value', _currentVal);
                this._trigger("change", data);
                this._trigger("_change", data);
                this._checkErrorClass();
            }
        },

        _triggerSelectEvent: function (e) {
            var val = this.element.val();
            if (this._parseDate(val)) {
                var data = { prevDate: this._prevDate, value: val, isSpecialDay: this._isSpecialDates(this.model.value) };
                if (this._prevDate != val) {
                    if (this._parseDate(data.value) && (this.model.value >= this.model.minDate && this.model.value <= this.model.maxDate)) {
                        this._cancelValue = this._trigger("select", data);
                    }
                }
            }
        },

        _onDocumentClick: function (e) {
            if (this.model) {
                if (!$(e.target).is(this.popup) && !$(e.target).parents(".e-popup").is(this.popup) &&
                    !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-datewidget").is(this.wrapper)) {                   
                        this.hide(e);
                }
                else if ($(e.target).is(this.popup) || $(e.target).parents(".e-popup").is(this.popup)) {
                    e.preventDefault();
                }
            }
        },

        _OnWindowResize: function (e) {
            this._setDatePickerPosition();
        },

        _showDatePopUp: function (e) {
            if (!this._isSupport && !this.model.showPopupButton) {
                e.preventDefault();
                this._onFocusIn();
            }
            if (this.model.readOnly) return;
            e.preventDefault();
            if (!this.model.enabled && this.model.displayInline) return false;
            this._showhidePopup(e);
        },
        _layoutChanged: function (e) {
            // this event internally used to observe the layout change in "DateTimePicker" control
            if (this._getInternalEvents) this._trigger("layoutChange");
        },
        _setCurrDate: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (e) e.preventDefault();
            var proxy = this;
            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            this._dateValue = this._zeroTime(new Date());
            this.model.value = this._calendarDate = new Date(this._dateValue.toString());
            this._setDateValue(this.model.value);
            this._triggerSelectEvent(e);
            this._triggerChangeEvent(e);
            this._refreshDatepicker();
            this._changeDayClass();
            this._startLevel(this.model.startLevel);
            this._onSetCancelDateHandler(e);
            this._layoutChanged();
        },
        _changeDayClass: function () {
            var className = this.popup.children("table")[0].className;
            if (className != "e-dp-viewdays") {
                this.popup.children("table").removeClass(className).addClass("e-dp-viewdays");
            }
        },

        _onSetCancelDateHandler: function (e) {
            if (this.model.readOnly || !this.model.enabled) return false;
            if (e && $(e.target).hasClass("e-disable")) return false;
            if (e && e.type) e.preventDefault();
            this.model.value = this._parseDate(this.element.val());

            this._prevDate = this._formatter(this.model.value, this.model.dateFormat);
            this._setDateValue(this._dateValue);
            this._triggerSelectEvent(e);
            this._triggerChangeEvent(e);
            this._dateValue = new Date(this.model.value.toString());
            if (this.element.is(':input') && !this.model.displayInline) {
                this._showhidePopup(e);
            }
            if (e && $(e.currentTarget).hasClass("other-month"))
                this._refreshDatepicker();
            this._cellSelection();
        },
        _closeCalendar: function (ele) {
            if (!ele || ele == this.element) {
                this.sfCalendar.empty().remove();
            }
        },
        //Error class for input value validation
        _checkErrorClass: function () {
            if (this.wrapper) {
                if (this.isValidState) this.wrapper.removeClass("e-error").attr('aria-invalid', "false");
                else this.wrapper.addClass("e-error").attr('aria-invalid', "true");
            }
        }
    });


    ej.DatePicker.Header = {
        /**  Removes the day header */
        None: "none",
        /**  Shows the day header format in short like Sun, Mon, Tue … */
        Short: "short",
        /**  Shows the day header format in min like Su, Mo, Tu … */
        Min: "min",
        /**  Shows the day header format in long like Sunday, Monday, Tuesday … */
        Long: "long"
    };

    ej.DatePicker.HighlightSection = {
        /**  Highlight the Current Month. */
        Month: "month",
        /**  Highlight the Current Week. */
        Week: "week",
        /**  Highlight the Current WorkDays. */
        WorkDays: "workdays",
        /** Don't Highlight Anything. */
        None: "none"
    };


    ej.DatePicker.Level = {
        /**  Starts from month level view. */
        Month: "month",
        /**  Starts from year level view. */
        Year: "year",
        /**  Starts from year decade level view. */
        Decade: "decade",
        /**  Starts from century level view.  */
        Century: "century"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to craete a Timepicker with the Html input element
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejTimePicker", "ej.TimePicker", {

        element: null,

        model: null,
        validTags: ["input"],
        _rootCSS: "e-timepicker",
        _setFirst: false,
        type: "editor",
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },

        defaults: {

            cssClass: "",

            timeFormat: "",

            value: null,

            enableAnimation: true,

            locale: "en-US",

            htmlAttributes: {},

            readOnly: false,

            showPopupButton: true,

            enableStrictMode: false,

            interval: 30,

            hourInterval: 1,

            minutesInterval: 1,

            secondsInterval: 1,

            height: "",

            width: "",

            minTime: "12:00:00 AM",

            maxTime: "11:59:59 PM",

            showRoundedCorner: false,

            enableRTL: false,

            popupHeight: "191px",

            popupWidth: "auto",

            enabled: true,

            enablePersistence: false,

            focusIn: null,

            focusOut: null,

            beforeChange: null,

            change: null,

            select: null,

            create: null,

            destroy: null,

            beforeOpen: null,

            open: null,

            close: null
        },


        dataTypes: {
            timeFormat: "string",
            minTime: "string",
            maxTime: "string",
            readOnly: "boolean",
            interval: "number",
            showPopupButton: "boolean",
            locale: "string",
            hourInterval: "number",
            minutesInterval: "number",
            secondsInterval: "number",
            enabled: "boolean",
            enablePersistence: "boolean",
            enableAnimation: "boolean",
            enableStrictMode: "boolean",
            htmlAttributes: "data"
        },

        observables: ["value"],

        enable: function () {
            if (!this.model.enabled) {
                this.element[0].disabled = false;
				this.element.removeAttr("disabled");
                this.model.enabled = true;
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                if (this.model.showPopupButton) {
                    this.timeIcon.removeClass("e-disable").attr("aria-disabled", false);
                    this.popupList.removeClass("e-disable").attr("aria-disabled", false);
                }
                if (this._isIE8) this.timeIcon.children().removeClass("e-disable");
            }
        },


        disable: function () {
            if (this.model.enabled) {
                this.element[0].disabled = true;
                this.model.enabled = false;
				this.element.attr("disabled","disabled");
                this.element.addClass("e-disable").attr("aria-disabled", true);
                if (this.model.showPopupButton) {
                    this.timeIcon.addClass("e-disable").attr("aria-disabled", true);
                    this.popupList.addClass("e-disable").attr("aria-disabled", true);
                }
                if (this._isIE8) this.timeIcon.children().addClass("e-disable");
                this._hideResult();
            }
        },


        getValue: function () {
            return this.element.val();
        },


        setCurrentTime: function () {
            if (!this.model.readOnly) this._setMask();
        },

        _timeFormat: function (format) {
            var validatedformat = this._validateTimeFormat(format);
            if (validatedformat) {
                this.model.timeFormat = validatedformat;
                this.model.value = this._localizeTime(this.getValue());
                this.element.val(this.model.value);
            }
            return validatedformat;
        },

        _getTimeFormat: function () {
            this.model.timeFormat = Globalize.culture(this.model.locale).calendar.patterns.t || "h:mm tt";
            this.seperator = this._getSeperator();
        },

        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            if (this.popupList) this.popupList.removeClass(this.model.cssClass).addClass(skin);
        },

        _localize: function (culture) {
            var currentTime = this._createObject(this.element.val());
            this.model.locale = culture;
            this._getTimeFormat()

            this.model.value = this._localizeTime(currentTime);
            this.element.val(this.model.value);
            this._getAmPm();
        },


        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._render();
            this._wireEvents();
        },

        _setModel: function (options) {
            var change = false, prev = this.model.value;
            if (!ej.isNullOrUndefined(options["minTime"]) || !ej.isNullOrUndefined(options["maxTime"])) {
                if (!ej.isNullOrUndefined(options["minTime"]) && $.trim(options["minTime"]) && this._isValid(options["minTime"])) {
                    this.model.minTime = options["minTime"];
                    this._validateTimes();
                }
                if (!ej.isNullOrUndefined(options["maxTime"]) && $.trim(options["maxTime"]) && this._isValid(options["maxTime"])) {
                    this.model.maxTime = options["maxTime"];
                    this._validateTimes();
                }

                this._validateMinMax();
                if (!ej.isNullOrUndefined(options["minTime"])) options["minTime"] = this.model.minTime;
                if (!ej.isNullOrUndefined(options["maxTime"])) options["maxTime"] = this.model.maxTime;

                if (this.model.showPopupButton) this._reRenderDropdown();
                if (options["value"] == undefined) this._enableMask();
                change = true;
            }

            var option;
            for (option in options) {
                switch (option) {
                    case "timeFormat":
                        this._preTimeformat = this.model.timeFormat;
                        this._setModelOption = true;
                        var newFormat = this._timeFormat(options[option]);
                        options[option] = this.model.timeFormat;
                        if (newFormat) {
                            this.seperator = this._getSeperator();
                            if (this.model.showPopupButton) this._reRenderDropdown();
                        }
                        break;
                    case "locale":
                        this._localize(options[option]);
                        if (this.model.showPopupButton) this._reRenderDropdown();
                        break;
                    case "interval":
                        this.model.interval = options[option];
                        if (this.model.showPopupButton) this._reRenderDropdown();
                        break;
                    case "showPopupButton": this._showButton(options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[option]); break;
                    case "enableRTL": this._setRtl(options[option]); break;
                    case "height":
                        this._setHeight(options[option]); break;
                    case "width":
                        this.wrapper.width(options[option]);
                        this._setListWidth();
                        break;
                    case "value":
                        if (!this.model.enableStrictMode) {
                            if (this._isValid(options[option])) {
                                this.model.value = options[option];
                                this._enableMask();
                            }
                        }
                        else {
                            this.model.value = options[option];
                            this._enableMask();
                        }
                        options[option] = this.model.value;
                        change = true;
                        break;
                    case "enableStrictMode":
                        this.model.enableStrictMode = options[option];
                        break;
                    case "popupHeight": this.model.popupHeight = options[option]; this._setListHeight(); break;
                    case "popupWidth": this.model.popupWidth = options[option]; this._setListWidth(); break;
                    case "enabled": if (options[option]) this.enable(); else this.disable(); break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }
            if (change) this._raiseChangeEvent(prev);
            this._checkErrorClass();
        },


        _destroy: function () {
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-input").removeAttr("ondragstart draggable").val("");
            if (this.popupList) this.popupList.remove();
        },

        _initialize: function () {
            this.target = this.element[0];
            this.timeIcon = null;
            this.popupList = null;
            this.focused = false;
            this.start = 0;
            this.end = 0;
            this.min = null;
            this.max = null;
            this.incomplete = false;
            this.downPosition = 0;
            this._getAmPm();
            this.showDropdown = false;
            this._activeItem = 0;
            this.isValidState = true;
            this._manualFocus = false;
            this._isIE7 = this._checkIE7();
            if (ej.isNullOrUndefined(this.model.value) && this.element[0].value != "")
                this.model.value = this.element[0].value;
            this._isIE8 = (ej.browserInfo().name == "msie") && (ej.browserInfo().version == "8.0") ? true : false;
            // _getInternalEvents is used when TimePicker used as a subcontrol of DateTimePicker 
            this._getInternalEvents = false;
            if (!this.model.timeFormat) this._getTimeFormat();
            else this.seperator = this._getSeperator();
        },

        _render: function () {
            this._renderWrapper();
            this._setDimentions();
            this._renderTimeIcon();
            this._validateTimes();
            this._renderDropdown();
            this._addAttr(this.model.htmlAttributes);
            this._checkProperties();
            this._enableMask();
            this._checkErrorClass();
            this.element.attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', "aria-readonly": this.model.readOnly, "value": this.model.value });
            if (this.model.showPopupButton || !ej.isNullOrUndefined(this.popupList))
                this.ul.find("li").attr({ 'tabindex': -1, 'aria-selected': false });
        },

        _renderWrapper: function () {
            this.element.addClass("e-input").attr("tabindex", "0");
            this.wrapper = ej.buildTag("span.e-timewidget e-widget " + this.model.cssClass + "#" + this.target.id + "_timewidget").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "readOnly" && value == "readOnly") proxy.model.readOnly = true;
                else proxy.wrapper.attr(key, value);
            });
        },
        _renderTimeIcon: function () {
            if (this.model.showPopupButton) {
                this.timeIcon = ej.buildTag("span.e-select").attr({ 'role': 'button', 'aria-label': 'select' });
                var icon = ej.buildTag("span.e-icon e-time").attr('role', 'presentation');
                if (this._isIE8) {
                    this.timeIcon.attr("unselectable", "on");
                    icon.attr("unselectable", "on");
                }
                this.timeIcon.append(icon);
                this.container.append(this.timeIcon).addClass("e-padding");
                this._on(this.timeIcon, "mousedown", this._timeIconClick);
            }
        },

        _renderDropdown: function () {
            var oldWrapper = $("#" + this.element.context.id + "_popup").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            if (!this.model.showPopupButton || this.popupList) return false;
            this.popupList = ej.buildTag("div.e-time-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.target.id + "_popup", "", {}, { 'tabindex': 0, 'aria-activedescendant': '' });
            this.popup = this.popupList;
            this.ul = ej.buildTag("ul.e-ul");
            if (this._isIE8)
                this.ul.attr("unselectable", "on");
            var scrollDiv = ej.buildTag("div").append(this.ul);
            $('body').append(this.popupList.append(scrollDiv));
            this._renderLiTags();
            this._setListHeight();
            this._setListWidth();
            this.popupList.ejScroller({ height: this.popupList.height(), width: 0, scrollerSize: 20 });
            this.scrollerObj = this.popupList.ejScroller("instance");
            this.popupList.css("display", "none");
            this._listSize = this.ul.find("li").size();
        },
        _renderLiTags: function () {
            if (this.model.interval < 1) return false;
            var start, end, timeVal, interval = this.model.interval * 60000
            start = (this.model.minTime) ? this._createObject(this.model.minTime) : this._createObject("12:00:00 AM");
            end = (this.model.maxTime) ? this._createObject(this.model.maxTime) : this._createObject("11:59:59 PM");

            while (this._compareTime(end, start, true)) {
                timeVal = this._localizeTime(start);
                var litag = ej.buildTag("li", timeVal);
                if (this._isIE8)
                    litag.attr("unselectable", "on");
                this.ul.append(litag);
                start = this._createObject(start).getTime() + interval;
            }

            var liTags = this.ul.find("li");
            this._on(liTags, "mouseenter", $.proxy(this._OnMouseEnter, this));
            this._on(liTags, "mouseleave", $.proxy(this._OnMouseLeave, this));
            this._on(liTags, "click", $.proxy(this._OnMouseClick, this));
        },
        _reRenderDropdown: function () {
            this.ul.empty();
            this._renderLiTags();
            this._refreshScroller();
            this._changeActiveEle();
        },
        _refreshScroller: function () {
            var flag = this.popupList.css("display") == "none" ? true : false;
            this.popupList.css("height", "auto");
            this.popupList.find(".e-content, .e-vscroll").removeAttr("style");
            this.popupList.find(".e-vscroll div").removeAttr("style");

            if (flag) this.popupList.css("display", "block");
            this.scrollerObj.model.height = this.popupList.height();
            this.scrollerObj.model.scrollTop = 0;
            this.scrollerObj.refresh();
            if (this._isIE8) {
                $("#" + this.scrollerObj._id).children('.e-vscroll').children().attr("unselectable", "on");
                $("#" + this.scrollerObj._id).find('.e-vhandle').attr("unselectable", "on");
            }
            if (flag) this.popupList.css("display", "none");
        },

        _setListWidth: function () {
            if (this.popupList) {
                var width = this.model.popupWidth;
                if (width && width != "auto") this.popupList.css({ "width": width });
                else this.popupList.css({ "width": this.wrapper.width() });
            }
            if (this.scrollerObj) {
                this._refreshScroller();
                this._updateScrollTop();
            }
        },
        _setListHeight: function () {
            if (this.popupList) this.popupList.css({ "max-height": this.model.popupHeight || "191px" });
            if (this.scrollerObj) {
                this._refreshScroller();
                this._updateScrollTop();
            }
        },
        _updateScrollTop: function () {
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
        },
        _refreshPopup: function () {
            if (this.model.popupWidth == "auto") this.popupList.css({ "width": this.wrapper.width() });
            this._setListPosition();
            this._refreshScroller();
        },

        _setListPosition: function () {
            var elementObj = this.wrapper, pos = this._getOffset(elementObj), winWidth,
            winBottomHeight = $(document).scrollTop() + $(window).height() - (pos.top + $(elementObj).outerHeight()),
            winTopHeight = pos.top - $(document).scrollTop(),
            popupHeight = this.popupList.outerHeight(),
            popupWidth = this.popupList.outerWidth(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial(), popupmargin = 3,
            topPos = ((popupHeight < winBottomHeight || popupHeight > winTopHeight) ? pos.top + totalHeight + popupmargin : pos.top - popupHeight - popupmargin) - border;
            winWidth = $(document).scrollLeft() + $(window).width() - left;
            if (this.model.enableRTL || popupWidth > winWidth && (popupWidth < left + elementObj.outerWidth())) left -= this.popupList.outerWidth() - elementObj.outerWidth();
            this.popupList.css({
                "left": left + "px",
                "top": topPos + "px",
                "z-index": maxZ
            });
        },

        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },

        _getZindexPartial: function () {
            return ej.util.getZindexPartial(this.element, this.popupList);
        },

        _enableMask: function () {
            if (this.model.value) {
                var flag = false;
                if ((this.model.minTime && this._compareTime(this.model.minTime, this.model.value)) ||
                    this.model.maxTime && this._compareTime(this.model.value, this.model.maxTime))
                    this.isValidState = false;
                else this.isValidState = true;

                this._setTime(this.model.value);
                if (this._getInternalEvents && !this.isValidState) this._trigger("outOfRange");
            }
            else this._setMask();
            this._changeActiveEle();
            this._preVal = this.element.val();
        },
        _setTime: function (time) {
            var modifiedTime = this._localizeTime(time);
            this.element.val(modifiedTime);
            this.model.value = modifiedTime;
        },
        _setMask: function () {
            this.model.value = new Date();
            this._enableMask();
        },

        _validateTimes: function () {
            var validatedformat = this._validateTimeFormat(this.model.timeFormat);
            if (validatedformat) this.model.timeFormat = validatedformat;
            else this.model.timeFormat = "h:mm tt";
            if (!this._isValid(this.model.minTime)) this.model.minTime = "12:00:00 AM";
            if (!this._isValid(this.model.maxTime)) this.model.maxTime = "11:59:59 PM";
            if (!this._isValid(this.model.value)) this.model.value = null;
            if (!this._checkMinMax(this.model.value) && !this.model.enableStrictMode)
                this.model.value = this.model.minTime;
            this._validateMinMax();
        },
        _validateMinMax: function () {
            if (this.model.minTime && this.model.maxTime && this._compareTime(this.model.minTime, this.model.maxTime)) {
                this.model.minTime = this.model.maxTime;
            }
        },
        _checkProperties: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this.disable();
            }
            else if (this.model.enabled && this.element.hasClass("e-disable")) {
                this.model.enabled = false;
                this.enable();
            }
            this._addProperty();
            this._checkAttributes();
        },
        _addProperty: function () {
            this._setRtl(this.model.enableRTL);
            this._setRoundedCorner(this.model.showRoundedCorner);
        },
        _setRtl: function (boolean) {
            if (boolean) {
                this.wrapper.addClass("e-rtl");
                if (this.popupList) this.popupList.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                if (this.popupList) this.popupList.removeClass("e-rtl");
            }
        },
        _setRoundedCorner: function (boolean) {
            if (boolean) {
                this.container.addClass("e-corner");
                if (this.popupList) this.popupList.addClass("e-corner");
            }
            else {
                this.container.removeClass("e-corner");
                if (this.popupList) this.popupList.removeClass("e-corner");
            }
        },
        _showButton: function (show) {
            this.model.showPopupButton = show;
            if (show) {
                this.container.addClass("e-padding");
                this._renderTimeIcon();
                this._renderDropdown();
                this._addProperty();
            }
            else {
                this.container.removeClass("e-padding");
                this.timeIcon.remove();
                this.popupList.remove();
                this.timeIcon = this.popupList = null;
            }
        },
        _checkAttributes: function () {
            if (!this.element.attr("name"))
                this.element.attr({ "name": this.element[0].id });
            if ('ondragstart' in document.createElement('input'))
                this.element.attr({ "ondragstart": "return false" });
            if ('draggable' in document.createElement('input'))
                this.element.attr({ "draggable": "false" });
        },

        _getAmPm: function () {
            this.ttAM = $.trim(this._localizeMeridian("00:00"));
            this.ttPM = $.trim(this._localizeMeridian("23:00"));
        },

        _setDimentions: function () {
            this._setHeight(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
        },
        _setHeight: function (height) {
            if (height) this.wrapper.height(height);
            if (this._isIE7) this.element.height(this.container.height());
        },

        _validateTimeFormat: function (timeFormat) {
            var parts = timeFormat.split(" "), format = "";
            if (parts.length == 1 || parts.length == 2) {
                $(parts).each(function (i, part) {
                    format += $.trim(part) + " ";
                });
                return $.trim(format);
            }
            else return null;
        },

        _getSeperator: function () {
            var p = this._getElePlace(), formats = this.model.timeFormat.split(" ")[p.time];
            var regex = new RegExp("^[a-zA-Z0-9]+$");

            for (var i = 0; i < formats.length; i++) {
                if (!regex.test(formats.charAt(i))) return formats.charAt(i);
            }
        },

        _checkInComplete: function () {
            var pos = this._getCaretSelection(), cursor = this._getStartEnd(pos);
            var replace = "00", selected = this._getSelectedValue(cursor), category = this._getCategory(cursor);
            if (pos.end - pos.start == this.element.val().length) this._checkAll();

            if (category && category != "tt") {
                this._findCategoryPosition(category);
                if (selected == "__") {
                    if (category == "h" || category == "hh") replace = "12";
                    this._changeToDefault(replace);
                }
                else if (category.length != 1 && selected.length == 1) {
                    selected = this._changeWhole(selected);
                    this.element.val(this._replaceAt(this.target.value, this.start, this.end, selected));
                }
            }
        },
        _checkAll: function () {
            var i, p = this._getElePlace(), categories = this.model.timeFormat.split(" ")[p.time].split(this.seperator);
            for (i = 0; i < categories.length; i++) {
                this._findCategoryPosition(categories[i]);
                var selected = this._getSelectedValue({ start: this.start, end: this.end });

                if (categories[i].length != 1 && selected.length == 1) {
                    selected = this._changeWhole(selected);
                    this.element.val(this._replaceAt(this.element.val(), this.start, this.end, selected));
                }
            }
        },

        _changeToDefault: function (replace) {
            this.incomplete = true;
            var preVal = this.element[0].value
            this.element[0].value = this._replaceAt(this.target.value, this.start, this.end, replace);
            var timeValue = this._checkExceedRange(this.target.value);
            if (!!timeValue) {
                this._setTime(this.model[timeValue]);
            }
            this._setSelection(this.start, this.end);
            this._raiseChangeEvent(preVal);
        },

        _setSelection: function (start, end) {
            var element = this.element[0];

            if (element.setSelectionRange)
                element.setSelectionRange(start, end);
            else if (element.createTextRange) {
                // For lower version browsers (IE8, IE7 ...)
                element = element.createTextRange();
                element.collapse(true);
                element.moveEnd('character', end);
                element.moveStart('character', start);
                element.select();
            }
        },

        _getSelectedValue: function (cursor) {
            return this.target.value.substring(cursor.start, cursor.end);
        },

        _getMinMax: function (currPart, keydown) {
            if (currPart == "hh" || currPart == "h") {
                this.min = 1; this.max = 11;
                if (keydown) this.max = 12;
            }
            else if (currPart == "HH" || currPart == "H") {
                this.min = 0; this.max = 23;
            }
            else if (currPart == "mm" || currPart == "m" || currPart == "ss" || currPart == "s") {
                this.min = 0; this.max = 59;
            }
        },

        _focusElement: function () {
            this._manualFocus = true;
            this.element.focus();
        },
        _targetFocus: function (e) {
            e.preventDefault();
            this.focused = true;
            this.element.bind('mousewheel DOMMouseScroll', $.proxy(this._mouseWheel, this));
            this.wrapper.addClass("e-focus").removeClass("e-error").attr('aria-invalid', "false");
            if (!this._manualFocus) {
                this._findCategoryPosition(this._getLeast(false));
                this._setSelection(this.start, this.end);
            }
            this._manualFocus = false;
            this._prevTimeVal = this.element.val();
            this._raiseEvent("focusIn");
        },
        _targetBlur: function () {
            this.focused = false;
            this.element.unbind('mousewheel DOMMouseScroll', $.proxy(this._mouseWheel, this));
            this.wrapper.removeClass("e-focus");
            this._checkInComplete();
            if (!this._checkMinMax(this.target.value)) {
                if (!this.model.enableStrictMode) {
                    this.element.val(this._prevTimeVal);
                    this.model.value = this._prevTimeVal;
                    this.isValidState = true;
                }
                else
                    this.isValidState = false;
            }
            else this.isValidState = true;
            this._checkErrorClass();
            this._raiseChangeEvent();
            this._raiseEvent("focusOut");
        },
        _checkErrorClass: function () {
            if (this.isValidState) this.wrapper.removeClass("e-error").attr('aria-invalid', "false");
            else this.wrapper.addClass("e-error").attr('aria-invalid', "true");
        },

        _getCaretSelection: function () {
            var input = this.element[0], start = 0, end = 0;
            if (!isNaN(input.selectionStart)) {
                start = input.selectionStart;
                end = input.selectionEnd;
                return { start: Math.abs(start), end: Math.abs(end) };
            }
            // For lower version browsers (IE8, IE7 ...)
            var bookmark = document.selection.createRange().getBookmark();
            var selection = input.createTextRange();
            selection.moveToBookmark(bookmark);

            var before = input.createTextRange();
            before.collapse(true);
            before.setEndPoint("EndToStart", selection);
            var beforeLength = before.text.length, selLength = selection.text.length;
            return { start: beforeLength, end: beforeLength + selLength };
        },

        _mouseDownOnInput: function (e) {
            if (!this.focused) this._focusElement();
            this.downPosition = this._getCaretSelection();
            this._checkInComplete();
            $(document).bind("mouseup", $.proxy(this._mouseUpOnInput, this));
        },

        _mouseUpOnInput: function (e) {
            e.preventDefault();
            $(document).unbind("mouseup", $.proxy(this._mouseUpOnInput, this));
            var pos = this._getCaretSelection();

            if (this.incomplete) {
                this.incomplete = false;
                pos = this.downPosition;
            }
            var cursor = this._getStartEnd(pos);
            this._setSelection(cursor.start, cursor.end);
        },

        _getCategoryPosition: function (category) {
            var s = 0, e = 0, parts = this.target.value.split(" "), p = this._getElePlace(), sep = this.seperator, valid = false;
            var fParts = this.model.timeFormat.split(" ")[p.time].split(sep);
            var tParts = parts[p.time].split(sep);
            if (fParts.length > tParts.length) return { start: s, end: e, isValid: valid };

            if (category == "tt") {
                if (parts[p.tt] == this.ttAM || parts[p.tt] == this.ttPM) {
                    if (p.tt == 0) s = 0;
                    else s = parts[p.time].length + 1;
                    e = s + parts[p.tt].length;
                    valid = true;
                }
            }
            else {
                if (p.time == 0) s = 0;
                else s = parts[p.tt].length + 1;

                var index = fParts.indexOf(category);
                if (index != -1) {
                    for (var i = 0; i < fParts.length; i++) {
                        e = tParts[i].length + 1;
                        if (i == index) break;
                        else s += e;
                    }
                    e += s - 1;
                    valid = true;
                }
            }
            return { start: s, end: e, isValid: valid };
        },
        _getCategory: function (cursor) {
            var parts = this.model.timeFormat.split(" "), sep = this.seperator;
            var p = this._getElePlace();
            if (cursor.isTT) return parts[p.tt];
            else return parts[p.time].split(sep)[cursor.index];
        },

        _getStartEnd: function (pos) {
            var sep = this.seperator;
            var value = this.element.val(), parts = value.split(" "), s = 0, e = 0, place = tt = null, i, j;

            for (j = 0; j < parts.length; j++) {
                if (parts[j] != this.ttAM && parts[j] != this.ttPM) {
                    var time = parts[j].split(sep), tempS = s, tempE = s + time[0].length;
                    for (i = 0; i < time.length; i++) {
                        e = time[i].length + s;
                        if (pos.start <= e) {
                            place = i;
                            tt = false;
                            j = parts.length;
                            break;
                        }
                        else s += time[i].length + 1;
                    }
                }
                else {
                    if (pos.start <= s + parts[j].length) {
                        e = parts[j].length + s;
                        place = 0;
                        tt = true;
                        j = parts.length;
                        break;
                    }
                    else s += parts[j].length + 1;
                }
            }
            if (place == null) s = tempS, e = tempE, place = 0, tt = false;

            return { start: s, end: e, index: place, isTT: tt };
        },

        _modifyValue: function (isIncrement) {
            this._checkInComplete();
            var pos = this._getCaretSelection(), cursor;
            if (pos.start == pos.end) {
                var cate = this._getLeast(true);
                var position = this._getCategoryPosition(cate);
                cursor = this._getStartEnd(position);
            }
            else cursor = this._getStartEnd(pos);
            this.start = cursor.start; this.end = cursor.end;

            this._changeValue(cursor, isIncrement);
        },

        _keyUpOnInput: function (e) {
            e.preventDefault();
            if (this._preVal != this.element.val()) {
                this._preVal = this.element.val();
            }
        },

        _getNextCategory: function (cate, direction) {
            var categories = [], sep = this.seperator;
            var fParts = this.model.timeFormat.split(" ");
            $(fParts).each(function (i, part) {
                if (part == "tt") categories.push(part);
                else {
                    var inner = part.split(sep);
                    categories = inner.concat(categories);
                }
            });
            var index = categories.indexOf(cate), ix;
            if (index != -1) {
                if (direction) {
                    if (index == 0) ix = categories.length - 1;
                    else ix = index - 1;
                }
                else {
                    if (index == categories.length - 1) ix = 0;
                    else ix = index + 1;
                }
                return categories[ix];
            }
            return cate;
        },
        _getElePlace: function () {
            var fParts = this.model.timeFormat.split(" "), time, tt;
            if (fParts[0] == "tt") time = 1, tt = 0;
            else time = 0, tt = 1;
            return { time: time, tt: tt };
        },
        _movePosition: function (pos, direction) {
            var cursor = this._getStartEnd(pos);
            var currCate = this._getCategory(cursor);
            if (!currCate) currCate = this._getLeast(direction);
            var next = this._getNextCategory(currCate, direction);
            var cursor = this._getCategoryPosition(next);

            if (cursor.isValid) {
                this._setSelection(cursor.start, cursor.end);
            }
        },

        _keyDownOnInput: function (e) {
            if (this.model.readOnly && !this._readOnlyKeys(e)) return false;
            var pos, cursor, category, key = e.keyCode;
            // _getInternalEvents is set to true when TimePicker used inside DateTimePicker control
            // in DateTimePicker control it allows Up, Down, Home, End, Tab keys only
            if (this._getInternalEvents && key != 38 && key != 40 && key != 36 && key != 35 && key != 9) return false;
            // Up, Down, Esc
            if (this.showDropdown && key != 38 && key != 40 && key != 27 && !this._readOnlyKeys(e)) return false;
            else if (this.showDropdown && (key == 37 || key == 39)) e.keyCode = (key == 37) ? 38 : 40;

            pos = this._getCaretSelection();
            cursor = this._getStartEnd(pos);
            category = this._getCategory(cursor);

            switch (e.keyCode) {
                case 38:
                    e.preventDefault();
                    if (!this.showDropdown)
                        this._modifyValue(true);
                    else {
                        if (this._activeItem > 1) this._activeItem -= 1;

                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 40:
                    e.preventDefault();
                    if (e.altKey && this.model.showPopupButton)
                        this._showhidePopup();
                    else if (!this.showDropdown)
                        this._modifyValue(false);
                    else {
                        if (this._activeItem < this._listSize) this._activeItem += 1;

                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 37:
                    e.preventDefault();
                    this._checkInComplete();
                    if (pos.start == pos.end) this._setSelection(pos.start - 1, pos.start - 1);
                    else this._movePosition(pos, true);
                    break;
                case 39:
                    e.preventDefault();
                    this._checkInComplete();
                    if (pos.start == pos.end) this._setSelection(pos.start + 1, pos.start + 1);
                    else this._movePosition(pos, false);
                    break;

                case 36:
                    e.preventDefault();
                    if (!this.showDropdown) {
                        var homecate = this._firstlastVal(true);
                        var hPos = this._getCategoryPosition(homecate);
                        if (hPos.isValid) this._setSelection(hPos.start, hPos.end);
                    }
                    else {
                        this._activeItem = 1;
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 35:
                    e.preventDefault();
                    if (!this.showDropdown) {
                        var endcate = this._firstlastVal(false);
                        var ePos = this._getCategoryPosition(endcate);
                        if (ePos.isValid) this._setSelection(ePos.start, ePos.end);
                    }
                    else {
                        this._activeItem = this._listSize;
                        this._addListHover();
                        this._selectTimeItem(this._getActiveItem());
                    }
                    break;
                case 9:
                    if (this._getInternalEvents) break;
                    this._hideResult();
                    var flag = null;
                    if (e.shiftKey && pos.start > 0) flag = true;
                    else if (!e.shiftKey && pos.end < this.element.val().length) flag = false;
                    if (flag != null) {
                        e.preventDefault();
                        this._checkInComplete();
                        this._movePosition(pos, flag);
                    }
                    break;
                case 13:
                    if (!this.showDropdown) {
                        this._checkInComplete();
                        this._raiseChangeEvent();
                        break;
                    }
                case 27:
                    e.preventDefault();
                    this._hideResult();
                    break;
                case 8:
                case 46:
                    e.preventDefault();
                    if (category && category != "tt") {
                        this._findCategoryPosition(category);
                        var _doBackspace = (key == 8 && pos.start != this.start), _doDelete = (key == 46 && pos.end != this.end), len;
                        len = this.end - this.start;

                        if ((pos.start != pos.end || len == 1) && (_doBackspace || _doDelete || pos.start != pos.end)) {
                            var s1 = this.start, s2 = this.end, te;
                            this.element[0].value = this._replaceAt(this.target.value, s1, s2, "__");
                            te = (s2 - s1 != 2) ? s2 + 1 : s2;
                            this._setSelection(s1, te);
                        }
                        else {
                            if (_doBackspace) {
                                this.element[0].value = this._replaceAt(this.target.value, pos.start - 1, pos.start, "");
                                this._setSelection(pos.start - 1, pos.start - 1);
                            }
                            else if (_doDelete) {
                                this.element[0].value = this._replaceAt(this.target.value, pos.end, pos.end + 1, "");
                                this._setSelection(pos.end, pos.end);
                            }
                        }
                    }
                    break;
            }

            var currSelection = this._getSelectedValue(cursor);
            var unicode = e.keyCode ? e.keyCode : e.charCode, actualkey;

            if (e.keyCode > 47 && e.keyCode < 58)
                actualkey = String.fromCharCode(unicode);
            else if (e.keyCode > 95 && e.keyCode < 106)
                actualkey = String.fromCharCode(unicode - 48);
            if (category == "tt" && ((!e.shiftKey && !e.ctrlKey && !e.altKey) && (e.keyCode > 64 && e.keyCode < 91) || (e.keyCode > 47 && e.keyCode < 58) || (e.keyCode > 95 && e.keyCode < 106))) {
                e.preventDefault();
                var ttPos = this._getCategoryPosition(category);
                this.start = ttPos.start;
                this.end = ttPos.end;
                this._changeAmPm(currSelection);
                this._raiseChangeEvent();
            }
            if ((!e.shiftKey && !e.ctrlKey && !e.altKey) && (e.keyCode > 47 && e.keyCode < 58) || (e.keyCode > 95 && e.keyCode < 106)) {
                if (category != "tt") {
                    this._getMinMax(category, true);
                    if (pos.start == pos.end) {
                        this._findCategoryPosition(category);
                        var newVal;
                        if (pos.start == this.start) newVal = actualkey + currSelection;
                        else newVal = currSelection + actualkey;
                        if (newVal.length > 2 || !(Number(newVal) >= this.min && this.max >= Number(newVal)))
                            e.preventDefault();
                    }
                    else if (!(Number(actualkey) >= this.min && this.max >= Number(actualkey)))
                        e.preventDefault();
                }
            }
            else if (!this._allowKeyCodes(e))
                !this.model.enableStrictMode ? e.preventDefault() : e.stopPropagation();
        },

        _allowKeyCodes: function (e) {
            if ((e.ctrlKey && (e.keyCode == 65 || e.keyCode == 67 || e.keyCode == 90 || e.keyCode == 89))
                || e.keyCode == 9 || e.keyCode == 116 || e.keyCode == 13)
                return true;
            return false;
        },
        _readOnlyKeys: function (e) {
            if (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || this._allowKeyCodes(e))
                return true;
            return false;
        },

        _firstlastVal: function (initial) {
            var parts = this.model.timeFormat.split(" "), sep = this.seperator;
            if (initial) {
                if (parts[0] != "tt") return parts[0].split(sep)[0];
                return "tt";
            }
            else {
                if (parts[0] != "tt") return "tt";
                else if (parts[1]) {
                    var lastItem = parts[1].split(sep);
                    return lastItem.length ? lastItem[lastItem.length - 1] : "tt";
                }
                return "tt";
            }
        },

        _mouseWheel: function (event) {
            event.preventDefault();
            if (this.model.readOnly) return false;
            var delta, rawEvent = event.originalEvent;
            if (rawEvent.wheelDelta) {
                // IE and Opera use wheelDelta, which is a multiple of 120 (possible values -120, 0, 120).
                delta = rawEvent.wheelDelta / 120;
                // In Opera, value is negated.
                //if (Sys.Browser.agent === Sys.Browser.Opera) delta = -delta;
            }
            else if (rawEvent.detail) {
                // Firefox uses detail property, which is a multiple of 3.
                delta = -rawEvent.detail / 3;
            }
            if (delta > 0)
                this._modifyValue(true);
            else if (delta < 0)
                this._modifyValue(false);
        },

        _addListHover: function () {
            this._addSelected();
            this._updateScrollTop();
        },
        _addSelected: function () {
            this.ul.find("li").removeClass("e-active");
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-active");
        },
        _getActiveItem: function () {
            return $(this.ul.find("li")[this._activeItem - 1]);
        },

        _timeIconClick: function (event) {
            event.preventDefault();
            if (!this.model.enabled || this.model.readOnly || this.ul.find("li").length < 1) return false;
            this._showhidePopup();
            var len = this.element.val().length;
            this._setSelection(len, len);
        },
        _showhidePopup: function () {
            if (this._getInternalEvents) return false;
            if (!this.showDropdown)
                this._showResult();
            else
                this._hideResult();
        },
        _showResult: function () {
            this._raiseEvent("beforeOpen");
            this._refreshPopup();
            if (!this.focused) this._focusElement();
            this._changeActiveEle();
            var proxy = this, sTop = this._vissibleAndCalculateTop();
            this.popupList.slideDown(this.model.enableAnimation ? 200 : 0, "easeOutQuad", function () {
                $(document).bind("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
            });
            this.scrollerObj.setModel({ "scrollTop": sTop });
            this.showDropdown = true;
            this._listSize = this.ul.find("li").size();
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            this._on(ej.getScrollableParents(this.wrapper), "scroll", this._hideResult);
            this._raiseEvent("open");
            this.wrapper.addClass("e-active");
        },
        _hideResult: function () {
            if (this.showDropdown && !this._getInternalEvents) {
                this.showDropdown = false;
                this.popupList.slideUp(this.model.enableAnimation ? 100 : 0, "easeOutQuad");
                $(document).unbind("mousedown", $.proxy(this._OnDocumentClick, this));
                $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
                this._off(ej.getScrollableParents(this.wrapper), "scroll", this._hideResult);
                this._raiseEvent("close");
                this.wrapper.removeClass("e-active");
            }
        },

        _vissibleAndCalculateTop: function () {
            this.popupList.css({ "display": "block" });
            var scrollTop = this._calcScrollTop();
            this.popupList.css({ "display": "none" });
            return scrollTop;
        },
        _calcScrollTop: function () {
            var ulH = this.ul.outerHeight(), liH = this.ul.find("li").outerHeight(), index, top;
            index = this.ul.find("li.e-active").index();
            top = (liH * index) - ((this.popupList.outerHeight() - liH) / 2);
            return top;
        },
        _changeActiveEle: function () {
            if (!this.model.showPopupButton || !this.popupList) return false;
            var elements = this.ul.find("li");
            var currTime = this.element.val(), firstTime = elements.first().html(), index;
            index = (this._parse(currTime) - this._parse(firstTime)) / (this.model.interval * 60000);
            index = Math.round(index);
            this._activeItem = (index == elements.length) ? index : index + 1;
            if (this._activeItem < 0 || this._activeItem > elements.length || isNaN(this._activeItem)) this._activeItem = 1;
            this._addListHover();
        },

        _OnDocumentClick: function (e) {
            if (!$(e.target).is(this.popupList) && !$(e.target).parents(".e-time-popup").is(this.popupList) &&
                !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-timewidget").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.popupList) || $(e.target).parents(".e-time-popup").is(this.popupList))
                e.preventDefault();
        },
        _OnWindowResize: function (e) {
            this._refreshPopup();
        },

        _OnMouseEnter: function (e) {
            var targetEle = e.target;
            this.ul.find("li").removeClass("e-hover");
            $(targetEle).addClass("e-hover");
        },
        _OnMouseLeave: function (e) {
            this.ul.find("li").removeClass("e-hover");
        },
        _OnMouseClick: function (e) {
            e.preventDefault();
            if (this.model.enabled && !this.model.readOnly) {
                this._activeItem = $(e.target).index() + 1;
                this.ul.find("li").attr({ 'tabindex': -1, 'aria-selected': false });
                $(e.target).attr({ 'aria-selected': true, 'tabindex': 0 });
                this._addSelected();
                this._selectTimeItem($(e.target));
            }
            this._showhidePopup();
        },
        _selectTimeItem: function (ele) {
            this._beforeChange(ele);
            var flag = this._raiseChangeEvent();
            if (flag)
                this._trigger("select", { value: this.model.value, prevTime: this._previousValue });
        },

        _findCategoryPosition: function (category) {
            if (category == "least") category = this._getLeast(true);
            var pos = this._getCategoryPosition(category);
            this.start = pos.start;
            this.end = pos.end;
        },

        _getLeast: function (lower) {
            var formats = this.model.timeFormat.split(" "), sep = this.seperator, res = null;
            $(formats).each(function (i, e) {
                if (e != "tt") {
                    var times = e.split(sep);
                    if (lower) res = times[times.length - 1];
                    else res = times[0];
                }
            });
            return res;
        },

        _changeValue: function (cursor, isIncrement) {
            var preVal = this.target.value, currValue, category = this._getCategory(cursor);
            if (!category) return false;
            this._setSelection(this.start, this.end);
            currValue = this.target.value.substring(this.start, this.end);
            if (this._checkMinMax(this.target.value)) {
                if (currValue != this.ttAM && currValue != this.ttPM) {
                    currValue = this._changeCurrentValue(currValue, category, isIncrement);
                    if (category.length != 1) currValue = this._changeWhole(currValue);
                    this._findCategoryPosition(category);
                    this.model.value = this._replaceAt(this.target.value, this.start, this.end, currValue);
                    this.element.val(this.model.value);
                    this.end = this.start + currValue.toString().length;
                    this._setSelection(this.start, this.end);
                }
                else this._changeAmPm(currValue);
            }
            else {
                var timeValue = this._checkExceedRange(this.target.value);
                this._setTime(this.model[timeValue]);
                this._findCategoryPosition(category);
                this._setSelection(this.start, this.end);
            }

            if (!this._checkMinMax(this.target.value)) {
                this.element.val(preVal);
                this._findCategoryPosition(category);
                this._setSelection(this.start, this.end);
            }
            else this._raiseChangeEvent(preVal);
        },

        _checkMinMax: function (value) {
            var res = this._checkExceedRange(value);
            return !res;
        },
        _checkExceedRange: function (value) {
            if (this.model.minTime && !this._compareTime(value, this.model.minTime, true)) return "minTime";
            if (this.model.maxTime && !this._compareTime(this.model.maxTime, value, true)) return "maxTime";
            return null;
        },

        _changeWhole: function (currValue) {
            return currValue > 9 ? "" + currValue : "0" + currValue;
        },
        _changeAmPm: function (ampm) {
            ampm = ampm == this.ttAM ? this.ttPM : this.ttAM;
            this.element.val(this._replaceAt(this.target.value, this.start, this.end, ampm));
            this._setSelection(this.start, this.end);
        },
        _changeMinute: function (isIncrement) {
            var formats = ["mm", "m"];
            var currFormat = this._getExactFormat(formats);
            if (currFormat) {
                this._findCategoryPosition(currFormat);
                var minute = Number(this.target.value.substring(this.start, this.end));
                this._getMinMax(currFormat);
                if (isIncrement) {
                    if (minute == this.max) {
                        minute = this.min;
                        this._changeHour();
                    }
                    else minute += 1;
                }
                else {
                    if (minute == this.min) {
                        minute = this.max;
                        this._changeHour();
                    }
                    else minute -= 1;
                }
                this._findCategoryPosition(currFormat);
                if (currFormat.length != 1) minute = this._changeWhole(minute);
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, minute));
            }
        },
        _changeHour: function (isIncrement) {
            var formats = ["hh", "h", "HH", "H"];
            var currFormat = this._getExactFormat(formats);
            if (currFormat) {
                this._findCategoryPosition(currFormat);
                var hour = Number(this.target.value.substring(this.start, this.end));
                this._getMinMax(currFormat);
                if (isIncrement) {
                    if (hour == this.max) {
                        hour += 1;
                        this._changeMeridian();
                    }
                    else if (hour > this.max) hour = this.min;
                    else hour += 1;
                }
                else {
                    if (hour == this.min) hour = this.max + 1;
                    else if (hour > this.max) {
                        hour = this.max;
                        this._changeMeridian();
                    }
                    else hour -= 1;
                }
                this._findCategoryPosition(currFormat);
                if (currFormat.length != 1) hour = this._changeWhole(hour);
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, hour));
            }
        },
        _getExactFormat: function (cate) {
            var tFormat = this.model.timeFormat;
            for (var i = 0; i < cate.length; i++) {
                if (tFormat.indexOf(cate[i]) != -1) return cate[i];
            }
            return null;
        },
        _changeMeridian: function () {
            var start = this.model.timeFormat.indexOf("tt");
            if (start != -1) {
                this._findCategoryPosition("tt");
                var meridian = this.target.value.substring(this.start, this.end);
                meridian = (meridian == this.ttAM) ? this.ttPM : this.ttAM;
                this.element.val(this._replaceAt(this.target.value, this.start, this.end, meridian));
            }
        },
        _changeCurrentValue: function (current, category, isIncrement) {
            current = Number(current);
            var c = category, step = 1, change = true;
            this._getMinMax(c);

            if (c == "hh" || c == "h" || c == "HH" || c == "H") step = this.model.hourInterval;
            else if (c == "mm" || c == "m") step = this.model.minutesInterval;
            else if (c == "ss" || c == "s") step = this.model.secondsInterval;
            if (step <= 0) return current;

            if (isIncrement) {
                if ((c == "hh" || c == "h") && current > this.max) current = this.min - 1 + step;
                else if (current < this.max) current += step;
                else {
                    change = false;
                    if (c != "hh" && c != "h") current = this.min - 1 + step;
                    else current += step;
                    this._changeAdjacent(c, isIncrement);
                }
                if ((c == "hh" || c == "h") && current == this.max + 1)
                    change && this._changeAdjacent(c, isIncrement);
                else if (current > this.max + 1) {
                    current = current - (this.max + 1);
                    change && this._changeAdjacent(c, isIncrement);
                }
                if ((c != "hh" && c != "h") && current == this.max + 1) {
                    current = this.min;
                    change && this._changeAdjacent(c, isIncrement);
                }
            }
            else {
                if ((c != "hh" && c != "h") && current > this.min) current -= step;
                else if ((c == "hh" || c == "h") && current > this.min && current <= this.max) current -= step;
                else if ((c == "hh" || c == "h") && current == this.min) current = this.max + 2 - step;
                else {
                    change = false;
                    current = this.max + 1 - step;
                    this._changeAdjacent(c, isIncrement);
                }
                if (current < this.min) {
                    current = current + (this.max + 1);
                    change && this._changeAdjacent(c, isIncrement);
                }
            }
            return current;
        },
        _changeAdjacent: function (c, isIncrement) {
            if (c == "ss" || c == "s") this._changeMinute(isIncrement);
            else if (c == "mm" || c == "m") this._changeHour(isIncrement);
            else if (c == "hh" || c == "h" || c == "HH" || c == "H") this._changeMeridian();
        },

        _valueChange: function (e) {
            this._raiseChangeEvent();
        },

        _beforeChange: function (ele) {
            if (!this._raiseEvent("beforeChange")) {
                this.element.val(ele.text());
            }
            return true;
        },

        _raiseChangeEvent: function (prev) {
            prev = (prev == undefined) ? this.model.value : prev;
            this._previousValue = prev;
            if (prev == this.target.value) return false;
            if (this._checkMinMax(this.target.value)) this.isValidState = true;
            else this.isValidState = false;
            this.model.value = this.target.value;
            this._raiseEvent("change", { prevTime: this._previousValue });
            this._raiseEvent("_change", { prevTime: this._previousValue });
            return true;
        },
        _raiseEvent: function (name) {
            return (this._trigger(name, { value: this.model.value, prevTime: this._previousValue }));
        },
        _checkIE7: function () {
            if (navigator.appName == 'Microsoft Internet Explorer') {
                var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})"), version = -1;
                if (re.exec(navigator.userAgent) != null)
                    version = parseFloat(RegExp.$1);
                if (version >= 7 && version < 8) return true;
            }
            return false;
        },
        _replaceAt: function (mainString, from, to, replace) {
            return mainString.substring(0, from) + replace + mainString.substring(to);
        },
        _localizeTime: function (value) {
            return $.trim(Globalize.format(this._createObject(value), this.model.timeFormat, this.model.locale));
        },
        _localizeMeridian: function (value) {
            return $.trim(Globalize.format(this._createObject(value), "tt", this.model.locale));
        },
        _compareTime: function (time1, time2, orEqual) {
            orEqual = (!orEqual) ? false : true;
            if (orEqual) return this._parse(time1) >= this._parse(time2);
            else return this._parse(time1) > this._parse(time2);
        },
        _isValid: function (time) {
            time = this._createObject(time);
            return time && typeof time.getTime === "function" && isFinite(time.getTime());
        },
        _parse: function (time) {
            return Date.parse(this._createObject(time));
        },
        _setEmptyDate: function (date) {
            var newDate = new Date(date);
            newDate.setDate(1);
            newDate.setMonth(0);
            newDate.setFullYear(2000);
            return newDate;
        },
        _createObject: function (value) {
            if (typeof value === "string" || value == null) {
                var format = this._setModelOption ? this._preTimeformat : this.model.timeFormat;
                var obj = Globalize.parseDate("1/1/2000 " + value, Globalize.culture(this.model.locale).calendar.patterns.d + " " + format);
                this._setModelOption = false;
                if (!obj) {
                    var isJSONString = new Date(value);
                    if (!isNaN(Date.parse(isJSONString)) && !ej.isNullOrUndefined(value))
                        return this._setEmptyDate(value);
                    else
                        obj = new Date("1/1/2000 " + value);
                }
                return obj;
            }
            else if (typeof value === "number")
                return new Date(value);
            else if (value instanceof Date)
                return this._setEmptyDate(value);
        },


        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.element, "mousedown", this._mouseDownOnInput);
            this._on(this.element, "keydown", this._keyDownOnInput);
            this._on(this.element, "keyup", this._keyUpOnInput);
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to select the date and time values.
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejDateTimePicker", "ej.DateTimePicker", {

        element: null,

        model: null,
        validTags: ["input"],
        _setFirst: false,
        _rootCSS: "e-datetimepicker",
        type: "editor",
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },


        defaults: {

            cssClass: "",

            locale: "en-US",

            readOnly: false,

            showRoundedCorner: false,

            enableRTL: false,

            htmlAttributes: {},

            enabled: true,

            value: "",

            minDateTime: new Date("1/1/1900 12:00:00 AM"),

            maxDateTime: new Date("12/31/2099 11:59:59 PM"),

            height: "",

            width: "",

            dateTimeFormat: "",

            showPopupButton: true,

            enableStrictMode: false,

            buttonText: {

                today: "Today",

                timeNow: "Time Now",

                done: "Done",

                timeTitle: "Time"
            },

            enablePersistence: false,

            interval: 30,

            timeDisplayFormat: "",

            timePopupWidth: 100,
			popupPosition:"bottom",

            dayHeaderFormat: "short",

            startLevel: "month",

            depthLevel: "",

            startDay: 0,

            stepMonths: 1,

            showOtherMonths: true,

            enableAnimation: true,

            headerFormat: 'MMMM yyyy',

            validationRules: null,

            validationMessage: null,

            open: null,

            close: null,

            change: null,

            create: null,

            destroy: null,

            focusIn: null,

            focusOut: null

        },


        dataTypes: {
            cssClass: "string",
            locale: "string",
            readOnly: "boolean",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
            enabled: "boolean",
            enableAnimation: "boolean",
            dateTimeFormat: "string",
            showPopupButton: "boolean",
            buttonText: "data",
            enablePersistence: "boolean",
            enableStrictMode: "boolean",
            interval: "number",
            timeDisplayFormat: "string",
            dayHeaderFormat: "string",
            startLevel: "string",
            depthLevel: "string",
            startDay: "number",
            stepMonths: "number",
            showOtherMonths: "boolean",
            headerFormat: "string",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data"
        },

        _setModel: function (options) {
            var option, validate = false;
            for (option in options) {
                switch (option) {
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "locale": this._localize(options[option]); break;
                    case "readOnly": this._readOnly(options[option]); break;
                    case "showRoundedCorner": this._setRoundedCorner(options[option]); break;
                    case "enableRTL": this._setRtl(options[option]); break;
                    case "enabled": this._enabled(options[option]); break;
                    case "validationRules":
                        if (this.model.validationRules != null) {
                            this.element.rules('remove');
                            this.model.validationMessage = null;
                        }
                        this.model.validationRules = options[option];
                        this._setValidation();
                        break;
                    case "validationMessage":
                        this.model.validationMessage = options[option];
                        this._setValidation(); break;
                    case "value":
                        options[option] = this._setValue(options[option]);
                        validate = true; break;
                    case "enableStrictMode":
                        this.model.enableStrictMode = options[option];
                        validate = true;
                        break;
                    case "minDateTime":
                        var temp = this._stringToObject(options[option]);
                        var mintime = this._getFormat(temp, "HH:mm:ss");
                        if (this._isValidDate(temp)) {
                            this.datePicker.option("minDate", temp);
                            this.timePicker.option("minTime", mintime);
                            options[option] = temp;
                        }
                        else options[option] = this.model[option];
                        validate = true; break;
                    case "maxDateTime":
                        var temp = this._stringToObject(options[option]);
                        var maxtime = this._getFormat(options[option], "HH:mm:ss");
                        if (this._isValidDate(temp)) {
                            this.datePicker.option("maxDate", temp);
                            this.timePicker.option("maxTime", maxtime);
                            options[option] = temp;
                        }
                        else options[option] = this.model[option];
                        validate = true; break;
                    case "height": this.wrapper.height(options[option]); break;
                    case "width": this.wrapper.width(options[option]); break;
                    case "dateTimeFormat":
                        this.model.dateTimeFormat = options[option];
                        if (this.isValidState) this._setValue(this.model.value);
                        break;
                    case "showPopupButton": this._showButton(options[option]); break;
                    case "buttonText": this._buttonText(options[option]); break;
                    case "interval":
                        this._updateTimeHeight();
                        this.timePicker.option("interval", options[option]); break;
                    case "timeDisplayFormat":
                        this._updateTimeHeight();
                        this.timePicker.option("timeFormat", options[option]); break;
                    case "timePopupWidth":
                        this._updateTimeHeight();
                        this.timePicker.option("popupWidth", options[option]);
                        break;
                    case "dayHeaderFormat": this.datePicker.option("dayHeaderFormat", options[option]); break;
                    case "startLevel": this.datePicker.option("startLevel", options[option]); break;
                    case "depthLevel": this.datePicker.option("depthLevel", options[option]); break;
                    case "startDay": this.datePicker.option("startDay", options[option]); break;
                    case "stepMonths": this.datePicker.option("stepMonths", options[option]); break;
                    case "showOtherMonths": this.datePicker.option("showOtherMonths", options[option]); break;
                    case "headerFormat": this.datePicker.option("headerFormat", options[option]); break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
					case "popupPosition":this.model.popupPosition = options[option]; this._setListPosition();
                }
            }
            if (validate) this._validateMinMax();
            this._valueChange();
            this._checkErrorClass();
        },
        observables: ["value"],

        _destroy: function () {
            if (this.isPopupOpen)
                this._hideResult();
            this.element.insertAfter(this.wrapper);
            this.element.removeClass("e-input").val("");
            this.wrapper.remove();
            this.popup.remove();
        },


        _init: function () {
            if (!this.element.is("input") || (this.element.attr('type') && this.element.attr('type') != "text")) return false;
            this._initialize();
            this._initial = true;
            this._render();
            this._wireEvents();
            this._addAttr(this.model.htmlAttributes);
            if (this.model.validationRules != null) {
                this._initValidator();
                this._setValidation();
            }
        },

        _initValidator: function () {
            var proxy = this;
            this.element.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $(error).insertAfter(element.closest(".e-widget"));
                },
            });
        },
        _setValidation: function () {
            this.element.rules("add", this.model.validationRules);
            var validator = this.element.closest("form").validate();
            name = this.element.attr("name");
            validator.settings.messages[name] = {};
            for (var ruleName in this.model.validationRules) {
                var message = null;
                if (!ej.isNullOrUndefined(this.model.validationRules[ruleName])) {
                    if (!ej.isNullOrUndefined(this.model.validationRules["messages"] && this.model.validationRules["messages"][ruleName]))
                        message = this.model.validationRules["messages"][ruleName];
                    else {
                        validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                        for (var msgName in this.model.validationMessage)
                            ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                    }
                    validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                }
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "required") proxy.element.attr(key, value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy._enabled(false);
                else if (key == "readOnly" && value == "readOnly") proxy._readOnly(true);
                else proxy.wrapper.attr(key, value);
            });
        },
        _initialize: function () {
            this.popup = null;
            this.isPopupOpen = false;
            this.isValidState = true;
            this.model.value = this.model.value === "" ? this.element[0].value : this.model.value;
            this._isIE8 = (ej.browserInfo().name == "msie") && (ej.browserInfo().version == "8.0") ? true : false;
            if (!this.model.dateTimeFormat || !this.model.timeDisplayFormat) this._getDateTimeFormat();
            if (typeof this.model.value == "string" && typeof this.model.value != "") {
                var val = this._stringToObject(this.model.value);
                if (val) this.model.value = val;
            }
            var min = this.model.minDateTime = this._stringToObject(this.model.minDateTime);
            if (!min || !this._isValidDate(min)) this.model.minDateTime = this.defaults.minDateTime;
            var max = this.model.maxDateTime = this._stringToObject(this.model.maxDateTime);
            if (!max || !this._isValidDate(max)) this.model.maxDateTime = this.defaults.maxDateTime;
        },

        _render: function () {
            this._renderWrapper();
            this._renderIcon();
            this._setDimentions();
            this._renderDropdown();
            this._checkProperties();
        },

        _renderWrapper: function () {
            this.element.addClass("e-input").attr({ 'aria-atomic': 'true', 'aria-live': 'assertive', "tabindex": "0", "value": this.model.value });
            this.wrapper = ej.buildTag("span.e-datetime-wrap e-widget " + this.model.cssClass + "#" + this.element[0].id + "_wrapper").insertAfter(this.element);
            this.container = ej.buildTag("span.e-in-wrap e-box").append(this.element);
            this.wrapper.append(this.container);
        },
        _renderIcon: function () {
            if (!this.model.showPopupButton) return false;
            this.datetimeIcon = ej.buildTag("span.e-select", "", {}).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            var icon = ej.buildTag("span.e-icon e-datetime", "", {}, { "aria-label": "select" }).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            this.datetimeIcon.append(icon);
            this.container.append(this.datetimeIcon).addClass("e-padding");
            this._on(this.datetimeIcon, "click", this._iconClick);
            this._on(this.datetimeIcon, "mousedown", function (e) { e.preventDefault(); });
        },
        _setDimentions: function () {
            if (this.model.height) this.wrapper.height(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
        },

        _renderDropdown: function () {
            var oldWrapper = $("#" + this.element.context.id + "_popup").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.popup = ej.buildTag("div.e-datetime-popup e-popup e-widget e-box " + this.model.cssClass + "#" + this.element[0].id + "_popup").css("visibility", "hidden");
            $('body').append(this.popup);
            this._renderControls();
            var _timeTitle, _dateContainer, _timeContainer, popupContainer, _today, _now, _done, buttonContainer;

            _timeTitle = ej.buildTag("div.e-header", this.model.buttonText.timeTitle).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            _dateContainer = ej.buildTag("div.e-datecontainer").append(this.datePicker.popup).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            _timeContainer = ej.buildTag("div.e-timecontainer").append(_timeTitle, this.timePicker.popup).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            popupContainer = ej.buildTag("div.e-popup-container").append(_dateContainer, _timeContainer).attr((this._isIE8) ? { 'unselectable': 'on' } : {});

            _today = ej.buildTag("div.e-dt-button e-dt-today", this.model.buttonText.today).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            _now = ej.buildTag("div.e-dt-button e-dt-now", this.model.buttonText.timeNow).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            _done = ej.buildTag("div.e-dt-button e-dt-done", this.model.buttonText.done).attr((this._isIE8) ? { 'unselectable': 'on' } : {});
            buttonContainer = ej.buildTag("div.e-button-container").append(_today, _now, _done).attr((this._isIE8) ? { 'unselectable': 'on' } : {});

            this.popup.append(popupContainer, buttonContainer);
            this._checkForResponsive();
            this._updateTimeHeight();

            this._bindOperations();
            this._updateValues();
            this.popup.css({ "visibility": "visible", "display": "none" });

            this._on(_today, "click", this._todayClick);
            this._on(_now, "click", this._nowClick);
            this._on(_done, "click", this._doneClick);
        },
        _renderControls: function () {
            this._renderDateControl();
            this._renderTimeControl();

            var tempContainer = ej.buildTag("span").append(this.datePicker.wrapper, this.timePicker.wrapper);
            tempContainer.find("span").css("display", "none");
            this.popup.append(tempContainer);
        },
        _renderDateControl: function () {
            var dateInput = ej.buildTag("input#" + this.element[0].id + "_date", "", {}, { "type": "text" });
            this.popup.append(dateInput);

            dateInput.ejDatePicker({

                height: "0px", width: "0px",
                displayInline: true,
                showDateIcon: false,
                showFooter: false,
                enableStrictMode: true,

                minDate: this._stringToObject(this.model.minDateTime),
                maxDate: this._stringToObject(this.model.maxDateTime),

                dayHeaderFormat: this.model.dayHeaderFormat,
                startLevel: this.model.startLevel,
                depthLevel: this.model.depthLevel,
                startDay: this.model.startDay,
                stepMonths: this.model.stepMonths,
                showOtherMonths: this.model.showOtherMonths,
                headerFormat: this.model.headerFormat,
                buttonText: this.model.buttonText.today,

                enabled: this.model.enabled,
                enableRTL: this.model.enableRTL,
                showRoundedCorner: this.model.showRoundedCorner,
                readOnly: this.model.readOnly,
                cssClass: this.model.cssClass,
                locale: this.model.locale
            });
            this.datePicker = dateInput.data("ejDatePicker");
            this.datePicker._getInternalEvents = true;
            this.datePicker.popup.css({ "position": "static", "display": "block" });
            if (this.model.value == "") {
                var date = new Date();
                if (this.model.minDateTime > date) date = this.model.minDateTime;
                else if (this.model.maxDateTime < date) date = this.model.maxDateTime;
                this.datePicker.option("value", date);
            }
        },
        _renderTimeControl: function () {
            var timeInput = ej.buildTag("input#" + this.element[0].id + "_time", "", {}, { "type": "text" });
            this.popup.append(timeInput);

            timeInput.ejTimePicker({
                height: "0px", width: "0px",
                interval: this.model.interval,
                timeFormat: this.model.timeDisplayFormat,
                popupWidth: this.model.timePopupWidth,
                enabled: this.model.enabled,
                enableRTL: this.model.enableRTL,
                showRoundedCorner: this.model.showRoundedCorner,
                readOnly: this.model.readOnly,
                cssClass: this.model.cssClass,
                locale: this.model.locale
            });
            this.timePicker = timeInput.data("ejTimePicker");
            this.timePicker._getInternalEvents = true;
            this.timePicker.showDropdown = true;
            this.timePicker.popup.css({ "position": "static", "display": "block" });
            var min = (this.model.minDateTime) ? this._stringToObject(this.model.minDateTime) : this.defaults.minDateTime;
            var max = (this.model.maxDateTime) ? this._stringToObject(this.model.maxDateTime) : this.defaults.maxDateTime;
            if (this.model.value == "") {
                if (+this.datePicker.model.value == +this.datePicker._zeroTime(this.model.minDateTime))
                    this.timePicker.option("minTime", this._getFormat(min, "HH:mm:ss"));
                if (+this.datePicker.model.value == +this.datePicker._zeroTime(this.model.maxDateTime))
                    this.timePicker.option("maxTime", this._getFormat(max, "HH:mm:ss"));
            }
        },
        _updateTimeHeight: function () {
            var height = this.datePicker.popup.outerHeight() - this.popup.find(".e-header").outerHeight() - 2;
            height = this.popup.hasClass("e-dt-responsive") ? "98px" : height;
            this.timePicker.option("popupHeight", height);
        },

        _bindOperations: function () {
            var proxy = this;
            this.datePicker.option("layoutChange", function () { proxy._updateTimeHeight(); });
            this.datePicker.option("outOfRange", function () { proxy.isValidState = false; });
            this.timePicker.option("outOfRange", function () { proxy.isValidState = false; });
            this.datePicker.option("change", function (a) { proxy._refreshTimes(a); });
            this.datePicker.option("select", function (e) { proxy._updateInput(e); });
            this.timePicker.option("select", function () { proxy._updateInput(); });
        },
        _updateInput: function (e) {
            var date = this._getDate() || new Date(), time = this._getTime() || new Date();
            this.model.value = new Date(date.getFullYear(), date.getMonth(), date.getDate(),
                time.getHours(), time.getMinutes(), time.getSeconds());
            this._preVal = this._objectToString(this.model.value);
            this._updateDateTime();
            this._raiseChangeEvent();
            if (e)
                e.cancel = true;
        },
        _updateDateTime: function () {
            this.isValidState = true;
            var datetime = this._objectToString(this.model.value);
            this.element.val(datetime);
        },
        _refreshTimes: function (args) {
            var value = this._getDateObj(args.value, this.datePicker.model.dateFormat);
            if (!value) return false;
            this.isValidState = true;

            if (this._compare(value, this._setEmptyTime(this.model.minDateTime))) {
                var mintime = this._getFormat(this.model.minDateTime, "HH:mm:ss");
                var preTime = this._getTime();
                this.timePicker.option("minTime", mintime);
                if (!this._compare(preTime, this._getTime())) {
                    this.timePicker.model.value = this._getFormat(preTime, this.timePicker.model.timeFormat);
                    this.isValidState = false;
                }
            }
            else this.timePicker.option("minTime", "12:00:00 AM");

            if (this._compare(value, this._setEmptyTime(this.model.maxDateTime))) {
                var maxtime = this._getFormat(this.model.maxDateTime, "HH:mm:ss");
                var preTime = this._getTime();
                this.timePicker.option("maxTime", maxtime);
                if (!this._compare(preTime, this._getTime())) {
                    this.timePicker.model.value = this._getFormat(preTime, this.timePicker.model.timeFormat);
                    this.isValidState = false;
                }
            }
            else this.timePicker.option("maxTime", "11:59:59 PM");
        },

        _updateValues: function () {
            this._setValue(this.model.value);
            if (this.model.value != null) {
                this.datePicker.option("value", this.model.value);
                this.timePicker.option("value", this.model.value);
            }
            this._validateMinMax();
            this._preVal = this.element.val();
            this._checkErrorClass();
        },
        _setValue: function (value) {
            if (value == null || (typeof JSON === "object" && JSON.stringify(value) === "{}") || $.trim(value) == "") {
                this.element.val("");
                this.model.value = null;
                this.isValidState = true;
            }
            else if (typeof value === "string") {
                this.element.val(value);
                this._updateModel();
                this._validateMinMax();
                this._checkStrictMode();
            }
            else if (value instanceof Date && this._isValidDate(value)) {
                this.model.value = value;
                this._updateDateTime();
                this._validateMinMax();
                this._checkStrictMode();
            }
            this._checkErrorClass();
            return this.model.value;
        },
        _validateValue: function (value) {
            var dateObj = this._stringToObject(value);
            if (!dateObj) {
                this.model.value = value;
                this.isValidState = false;
            }
            else {
                this.isValidState = true;
                this.model.value = dateObj;
            }
        },
        _validateMinMax: function () {
            var value, min, max;
            value = (this.model.value) ? this._stringToObject(this.model.value) : null;
            min = (this.model.minDateTime) ? this._stringToObject(this.model.minDateTime) : this.defaults.minDateTime;
            max = (this.model.maxDateTime) ? this._stringToObject(this.model.maxDateTime) : this.defaults.maxDateTime;
            if (!value || !min || !max) return false;
            if (min > max) this.model.minDateTime = this.model.maxDateTime;
            if (value < min || value > max) this.isValidState = false;
            else this.isValidState = true;
        },

        _checkProperties: function () {
            this.model.readOnly && this._readOnly(true);
            this.model.showRoundedCorner && this._setRoundedCorner(true);
            this.model.enableRTL && this._setRtl(true);
            this.model.enabled && this._enabled(true);
            if (!this.model.enabled) this._enabled(false);
            else if (this.model.enabled && this.element.hasClass("e-disable")) this._enabled(true);
            if (!this.element.attr("name")) this.element.attr({ "name": this.element[0].id });
            this._checkStrictMode();
            this._checkErrorClass();
        },

        _checkStrictMode: function () {
            if (!this.model.enableStrictMode) {
                if (!this.isValidState) {
                    if (this.model.value < this.model.minDateTime) {
                        this.element.val(this._objectToString(this.model.minDateTime));
                        this.model.value = this.model.minDateTime;
                        this.isValidState = true;
                    }
                    else if (this.model.value > this.model.maxDateTime) {
                        this.element.val(this._objectToString(this.model.maxDateTime));
                        this.model.value = this.model.maxDateTime;
                        this.isValidState = true;
                    }
                    else {
                        this.model.value = "";
                        this.element.val("");
                        this.isValidState = true;
                    }
                }
            }
        },

        _targetFocus: function (e) {
            e.preventDefault();
            this.isFocused = true;
            this.wrapper.addClass("e-focus");
            this.wrapper.removeClass("e-error");
            this._prevDateTimeVal = this.element.val();
            if (!this.model.showPopupButton && !this.model.readOnly) this._showResult();
            if (!this.model.showPopupButton) this._on(this.element, "click", this._elementClick);
            this._trigger("focusIn", { value: this.model.value });
        },
        _targetBlur: function () {
            this.isFocused = false;
            this.wrapper.removeClass("e-focus");
            if (!this.model.showPopupButton) this._hideResult();
            this._valueChange();
            if (!this.model.enableStrictMode) {
                if (!this.isValidState) {
                    this.element.val(this._prevDateTimeVal);
                    this._preVal = this._prevDateTimeVal;
                    this.model.value = this._stringToObject(this._prevDateTimeVal);
                    this.isValidState = true;
                }
                else
                    this._prevDateTimeVal = this.element.val();
            }
            this._checkErrorClass();
            if (!this.model.showPopupButton) this._off(this.element, "click", this._elementClick);
            this._trigger("focusOut", { value: this.model.value });
        },
        _elementClick: function () {
            if (!this.isPopupOpen) this._showResult();
        },
        _keyDownOnInput: function (e) {
            switch (e.keyCode) {
                case 40:
                    if (e.altKey) this._showhidePopup();
                    break;
                case 37:
                case 39:
                    if (e.altKey && this.isPopupOpen) {
                        e.preventDefault();
                        this._addPrevNextFocus(e.keyCode == 37);
                    }
                    break;
                case 27:
                    e.preventDefault();
                case 9:
                    this._hideResult();
                    break;
                case 13:    // Enter Key
    				this._valueChange();
    				this.isPopupOpen && this._hideResult();
    				break;
            }
        },
        _addFocus: function (target) {
            if (!target.hasClass("e-focus")) {
                this._removeFocus();
                target.addClass("e-focus");
                if (target.hasClass("e-datecontainer"))
                    $(document).bind("keydown", $.proxy(this.datePicker._keyboardNavigation, this.datePicker));
                else if (target.hasClass("e-timecontainer"))
                    $(document).bind("keydown", $.proxy(this.timePicker._keyDownOnInput, this.timePicker));
                else if (target.hasClass("e-dt-button"))
                    $(document).bind("keydown", $.proxy(this._buttonClick, this));
            }
        },
        _removeFocus: function () {
            var target = this._getFocusedElement();
            if (target.length > 0) {
                target.removeClass("e-focus");
                if (target.hasClass("e-datecontainer"))
                    $(document).unbind("keydown", $.proxy(this.datePicker._keyboardNavigation, this.datePicker));
                else if (target.hasClass("e-timecontainer"))
                    $(document).unbind("keydown", $.proxy(this.timePicker._keyDownOnInput, this.timePicker));
                else if (target.hasClass("e-dt-button"))
                    $(document).unbind("keydown", $.proxy(this._buttonClick, this));
            }
        },
        _addPrevNextFocus: function (flag) {
            // flag true means previous focus, false means next focus
            var target = this._getFocusedElement(), next;
            if (target.length > 0) {
                if (target.hasClass("e-datecontainer"))
                    next = flag ? this.popup.find(".e-dt-done") : this.popup.find(".e-timecontainer");
                else if (target.hasClass("e-timecontainer"))
                    next = flag ? this.popup.find(".e-datecontainer") : this.popup.find(".e-dt-today");
                else if (target.hasClass("e-dt-today"))
                    next = flag ? this.popup.find(".e-timecontainer") : this.popup.find(".e-dt-now");
                else if (target.hasClass("e-dt-now"))
                    next = flag ? this.popup.find(".e-dt-today") : this.popup.find(".e-dt-done");
                else if (target.hasClass("e-dt-done"))
                    next = flag ? this.popup.find(".e-dt-now") : this.popup.find(".e-datecontainer");
            }
            else next = flag ? this.popup.find(".e-dt-done") : this.popup.find(".e-datecontainer");
            this._addFocus(next);
        },
        _getFocusedElement: function () {
            return this.popup.children("div").children("div.e-focus");
        },

        _valueChange: function () {
            if (this._preVal != this.element.val()) {
                this._preVal = this.element.val();
                this._updateModel();
                this._validateMinMax();
                this._raiseChangeEvent();
            }
        },
        _updateModel: function () {
            var value = this.element.val();
            if (value == "") {
                this.model.value = null;
                this.isValidState = true;
            }
            else {
                var dateObj = this._stringToObject(value);
                if (dateObj) {
                    this.model.value = dateObj;
                    this.isValidState = true;
                    this._refreshPopup();
                }
                else {
                    this.model.value = value;
                    this.isValidState = false;
                }
            }
        },
        _refreshPopup: function () {
            if (this.isValidState && this.isPopupOpen) {
                var date = this._setEmptyTime(this.model.value), time = this._setEmptyDate(this.model.value);
                var getDate = this._getDate(), getTime = this._getTime();
                if (!getDate || !this._compare(getDate, date)) this.datePicker.option("value", date);
                if (!getTime || !this._compare(getTime, time)) this.timePicker.option("value", time);
            }
        },

        _buttonClick: function (e) {
            if (e.keyCode == 13) {
                e.preventDefault();
                var target = this._getFocusedElement();
                if (target.hasClass("e-dt-today"))
                    this._todayClick();
                else if (target.hasClass("e-dt-now"))
                    this._nowClick();
                else if (target.hasClass("e-dt-done"))
                    this._doneClick();
            }
        },
        _todayClick: function () {
            if (!this.model.enabled || this.model.readOnly) return false;
            if (!this.datePicker.popup.find(".today").hasClass("e-active") ||
                !this.datePicker.popup.children("table").hasClass("e-dp-viewdays") ||
                this.element.val() == "" || !this.isValidState) {
                this.datePicker._setCurrDate();
                this._updateInput();
            }
        },
        _nowClick: function () {
            if (!this.model.enabled || this.model.readOnly) return false;
            this.timePicker.setCurrentTime();
            var mintime = this.model.minDateTime, maxtime = this.model.maxDateTime, date = this.datePicker.model.value, time = new Date();
            var currTime = new Date(date.getFullYear(), date.getMonth(), date.getDate(), time.getHours(), time.getMinutes(), time.getSeconds());
            if (currTime < mintime)
                this.timePicker.option("value", this.timePicker._localizeTime(mintime));
            else if (currTime > maxtime)
                this.timePicker.option("value", this.timePicker._localizeTime(maxtime));
            this._updateInput();
        },
        _doneClick: function () {
            this._hideResult();
        },

        _iconClick: function (e) {
            e.preventDefault();
            if (!this.isFocused) this.element.focus();
            this._showhidePopup();
        },
        _showhidePopup: function () {
            if (this.model.readOnly) return false;
            if (!this.isPopupOpen)
                this._showResult();
            else
                this._hideResult();
        },
        _showResult: function () {
            if (this.isPopupOpen || !this.model.enabled) return false;
            if (this._raiseEvent("open")) return false;
            this.isPopupOpen = true;
            this._setListPosition();
            this._checkForResponsive();
            var proxy = this;
            this.popup.slideDown(this.model.enableAnimation ? 200 : 0, "easeOutQuad", function () {
                proxy._on($(document), "mousedown", proxy._OnDocumentClick);
            });
            this._updateModel();
            this._updateTimeHeight();
            this._validateMinMax();
            this._on($(window), "resize", this._OnWindowResize);
            this._on(ej.getScrollableParents(this.wrapper), "scroll", this._hideResult);
            if (this._initial) {
                this.timePicker._refreshScroller();
                this.timePicker._changeActiveEle();
                this._initial = false;
            }
            this.wrapper.addClass("e-active");
        },
        _hideResult: function () {
            if (!this.isPopupOpen) return false;
            if (this._raiseEvent("close")) return false;
            this.isPopupOpen = false;
            this._removeFocus();
            this.popup.slideUp(this.model.enableAnimation ? 100 : 0, "easeOutQuad");
            this._off($(document), "mousedown", this._OnDocumentClick);
            this._off($(window), "resize", this._OnWindowResize);
            this._off(ej.getScrollableParents(this.wrapper), "scroll", this._hideResult);
            this.wrapper.removeClass("e-active");
        },

        _setListPosition: function () {
            var elementObj = this.wrapper, pos = this._getOffset(elementObj), winWidth,
            winBottomHeight = $(document).scrollTop() + $(window).height() - (pos.top + $(elementObj).outerHeight()),
            winTopHeight = pos.top - $(document).scrollTop(),
            popupHeight = this.popup.outerHeight(),
            popupWidth = this.popup.outerWidth(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial(), popupmargin = 3,
			popupPosition = this.model.popupPosition;
			if (this.model.popupPosition == ej.PopupPosition.Bottom)
				var topPos = ((popupHeight < winBottomHeight || popupHeight > winTopHeight) ? pos.top + totalHeight + popupmargin : pos.top - popupHeight - popupmargin) - border;
            else
				var topPos = ((popupHeight > winTopHeight) ? pos.top + totalHeight + popupmargin : pos.top - popupHeight - popupmargin) - border;
			winWidth = $(document).scrollLeft() + $(window).width() - left;
            if (this.model.enableRTL || popupWidth > winWidth && (popupWidth < left + elementObj.outerWidth())) left -= this.popup.outerWidth() - elementObj.outerWidth();
            this.popup.css({
                "left": left + "px",
                "top": topPos + "px",
                "z-index": maxZ
            });
        },

        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },

        _OnDocumentClick: function (e) {
            if (this.model) {
                if (!$(e.target).is(this.popup) && !$(e.target).parents(".e-datetime-popup").is(this.popup) &&
                    !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-datetime-wrap").is(this.wrapper)) {
                    this._hideResult();
                }
                else if ($(e.target).is(this.popup) || $(e.target).parents(".e-datetime-popup").is(this.popup)) {
                    e.preventDefault();
                    if ($(e.target).parents(".e-datecontainer").length > 0) this._addFocus($(e.target).parents(".e-datecontainer"));
                    else if ($(e.target).parents(".e-timecontainer").length > 0) this._addFocus($(e.target).parents(".e-timecontainer"));
                    else if ($(e.target).hasClass("e-dt-button")) this._addFocus($(e.target));
                    else this._removeFocus();
                }
            }
        },
        _OnWindowResize: function (e) {
            this._setListPosition();
            this._checkForResponsive();
        },

        _raiseChangeEvent: function () {
            var previous = this._stringToObject(this._prevDateTimeVal);
            var current = this._stringToObject(this.element.val());
            if (this.element != null && !(+previous === +current)) {
                var value = $.trim(this.element.val()) == "" ? null : this.element.val()
                this._trigger("change", {
                    prevDateTime: this._prevDateTimeVal,
                    value: value,
                    isValidState: this.isValidState
                });
                this._trigger("_change", {
                    prevDateTime: this._prevDateTimeVal,
                    value: value,
                    isValidState: this.isValidState
                });
                this._prevDateTimeVal = this.element.val();
            }
        },
        _raiseEvent: function (name) {
            if (this.element != null && this.model[name]) return this._trigger(name, { prevDateTime: this._preVal, value: this.element.val() });
            return false;
        },
        _getDateTimeFormat: function () {
            var pattern = Globalize.culture(this.model.locale).calendar.patterns;

            if (!this.model.dateTimeFormat) this.model.dateTimeFormat = pattern.d + " " + pattern.t;
            if (!this.model.timeDisplayFormat) this.model.timeDisplayFormat = pattern.t;
        },
        _getZindexPartial: function () {
            return ej.util.getZindexPartial(this.element, this.popup);
        },
        _checkErrorClass: function () {
            if (this.isValidState) this.wrapper.removeClass("e-error");
            else this.wrapper.addClass("e-error");
        },
        _getDate: function () {
            return this.datePicker.model.value;
        },
        _getTime: function () {
            return this._getDateObj(this.timePicker.model.value, this.timePicker.model.timeFormat);
        },
        _setEmptyTime: function (date) {
            var newDate = new Date(date);
            newDate.setMilliseconds(0);
            newDate.setSeconds(0);
            newDate.setMinutes(0);
            newDate.setHours(0);
            return newDate;
        },
        _setEmptyDate: function (date) {
            var newDate = new Date(date);
            newDate.setDate(1);
            newDate.setMonth(0);
            newDate.setFullYear(2000);
            return newDate;
        },
        _objectToString: function (obj) {
            return this._getFormat(obj, this.model.dateTimeFormat);
        },
        _stringToObject: function (value) {
            return this._getDateObj(value, this.model.dateTimeFormat);
        },
        _getFormat: function (value, format) {
            if (value instanceof Date) {
                var newFormat = this._checkFormat(format);
                return Globalize.format(value, newFormat, this.model.locale);
            }
            else return value;
        },
        _checkFormat: function (format) {
            var proxy = this;
            var dateFormatRegExp = this._regExp();
            return format.replace(dateFormatRegExp, function (match) {
                match = match === "/" ? Globalize.culture(proxy.model.locale).calendars.standard['/'] !== "/" ? "'/'" : match : match;
                return match;
            });
        },
        _regExp: function () {
            return /\/dddd|ddd|dd|d|MMMM|MMM|MM|M|yyyy|yy|HH|H|hh|h|mm|m|fff|ff|f|tt|ss|s|zzz|zz|z|gg|g|"[^"]*"|'[^']*'|[/]/g;
        },
        _getDateObj: function (value, format) {
            if (typeof value === "string") {
                var newFormat = this._checkFormat(format);
                var temp = Globalize.parseDate(value, newFormat, this.model.locale);
                if (temp != null)
                    return temp;
                else {
                    if (value != "" && value != null) {
                        var dateregexp = /^\s*(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d).*Z\s*$/, month, datetimesplit = dateregexp.exec(value);
                        var datevariable = new Date();
                        if (datetimesplit) {
                            month = +datetimesplit[2];
                            datevariable.setUTCFullYear(datetimesplit[1], month - 1, datetimesplit[3]);
                            datevariable.setUTCHours(datetimesplit[4], datetimesplit[5], datetimesplit[6]);
                            if (month != datevariable.getUTCMonth() + 1)
                                datevariable.setTime();
                        }
                        if (this._isValidDate(datevariable))
                            return datevariable;
                    }
                    else return null;
                }
            }
            else return value;
        },
        _compare: function (obj1, obj2) {
            return obj1 && obj2 && obj1.getTime() == obj2.getTime();
        },
        _isValidDate: function (dateObj) {
            return dateObj && typeof dateObj.getTime === "function" && isFinite(dateObj.getTime());
        },


        _change: function (property, value) {
            this.datePicker.option(property, value);
            this.timePicker.option(property, value);
        },
        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            this.popup.removeClass(this.model.cssClass).addClass(skin);

            this._change("cssClass", skin);
        },
        _localize: function (culture) {
            this.model.locale = culture;
            this.model.dateTimeFormat = this.model.timeDisplayFormat = "";
            this._getDateTimeFormat();
            if (this.isValidState || (this.model.value instanceof Date && this._isValidDate(this.model.value)))
                this.element.val(this._objectToString(this.model.value));
            this._preVal = this.element.val();
            this._change("locale", culture);
        },
        _readOnly: function (boolean) {
            this.model.readOnly = boolean;
            if (boolean) this.element.attr("readonly", "readonly");
            else this.element.removeAttr("readonly");

            this._change("readOnly", boolean);
        },
        _setRoundedCorner: function (boolean) {
            if (boolean) {
                this.container.addClass("e-corner");
                this.popup.addClass("e-corner");
            }
            else {
                this.container.removeClass("e-corner");
                this.popup.removeClass("e-corner");
            }
            this.datePicker.option("showRoundedCorner", boolean);
        },
        _setRtl: function (boolean) {
            if (boolean) {
                this.wrapper.addClass("e-rtl");
                this.popup.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.popup.removeClass("e-rtl");
            }

            this._change("enableRTL", boolean);
        },
        _enabled: function (boolean) {
            if (boolean) {
                this.model.enabled = false;
                this.enable();
            }
            else {
                this.model.enabled = true;
                this.disable();
            }
        },
        _showButton: function (show) {
            this.model.showPopupButton = show;
            if (show) {
                this.container.addClass("e-padding");
                this._renderIcon();
            }
            else {
                this.container.removeClass("e-padding");
                this.datetimeIcon.remove();
                this.datetimeIcon = null;
            }
        },
        _buttonText: function (data) {
            $.extend(this.model.buttonText, data);
            this.popup.find(".e-dt-today").html(this.model.buttonText.today);
            this.popup.find(".e-dt-now").html(this.model.buttonText.timeNow);
            this.popup.find(".e-dt-done").html(this.model.buttonText.done);
            this.popup.find(".e-timecontainer").find(".e-header").html(this.model.buttonText.timeTitle);
        },
        _checkForResponsive: function () {
            if (($(window).outerWidth() > 200) && ($(window).outerWidth() <= 500)) {
                if (!this.popup.hasClass("e-dt-responsive")) {
                    this.popup.addClass("e-dt-responsive");
                    this.timePicker.option("popupWidth", this.datePicker.popup.outerWidth());
                    this.timePicker.option("popupHeight", 98);
                    this.timePicker._refreshScroller();
                    this.timePicker._changeActiveEle();
                }
            }
            else if (this.popup.hasClass("e-dt-responsive")) {
                this.popup.removeClass("e-dt-responsive");
                this.timePicker.option("popupWidth", this.model.timePopupWidth);
                var height = this.datePicker.popup.outerHeight() - this.popup.find(".e-header").outerHeight() - 2;
                this.timePicker.option("popupHeight", height);
                this.timePicker._refreshScroller();
                this.timePicker._changeActiveEle();
            }
        },

        enable: function () {
            if (!this.model.enabled) {
                this.element[0].disabled = false;
                this.model.enabled = true;
				this.element.removeAttr("disabled");
                this.element.removeClass("e-disable").attr("aria-disabled", false);
                if (this.datetimeIcon) this.datetimeIcon.removeClass("e-disable").attr("aria-disabled", false);
                if (this._isIE8 && this.datetimeIcon) this.datetimeIcon.children().removeClass("e-disable");
                this.popup.children("div").removeClass("e-disable").attr("aria-disabled", false);
                this._change("enabled", true);
            }
        },


        disable: function () {
            if (this.model.enabled) {
                this.element[0].disabled = true;
                this.model.enabled = false;
                this.element.addClass("e-disable").attr("aria-disabled", true);
				this.element.attr("disabled","disabled");
                if (this.datetimeIcon) this.datetimeIcon.addClass("e-disable").attr("aria-disabled", true);
                if (this._isIE8 && this.datetimeIcon) this.datetimeIcon.children().addClass("e-disable");
                this.popup.children("div").addClass("e-disable").attr("aria-disabled", true);
                this._hideResult();

                this._change("enabled", false);
                this.datePicker.popup.removeClass("e-disable").attr("aria-disabled", false);
                this.timePicker.popup.removeClass("e-disable").attr("aria-disabled", false);
            }
        },


        getValue: function () {
            return this._objectToString(this.model.value);
        },


        setCurrentDateTime: function () {
            if (!this.model.readOnly)
                this._setValue(new Date());
        },


        show: function () {
            this._showResult();
        },


        hide: function () {
            this._hideResult();
        },


        _wireEvents: function () {
            this._on(this.element, "focus", this._targetFocus);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.element, "keydown", this._keyDownOnInput);
        }
    });
	ej.PopupPosition = {
			Bottom: "bottom",
			Top: "top"
		};
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Dialog control
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejDialog", "ej.Dialog", {
        _rootCSS: "e-dialog",

        element: null,

        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        angular: {
            terminal: false
        },

        defaults: {

            showOnInit: true,

            closeOnEscape: true,

            closeIconTooltip: "close",

            enableAnimation: true,

            allowDraggable: true,

            height: "auto",

            minHeight: 120,

            minWidth: 200,

            maxHeight: null,

            maxWidth: null,

            enableModal: false,

            position: { X: "", Y: "" },

            enableResize: true,

            htmlAttributes: {},

            showHeader: true,

            contentType: null,

            contentUrl: null,

            ajaxSettings: {

                type: 'GET',

                cache: false,

                data: {},

                dataType: "html",

                contentType: "html",

                async: true
            },

            title: "",

            width: 400,

            zIndex: 1000,

            cssClass: "",

            enableRTL: false,

            allowKeyboardNavigation: true,

            showRoundedCorner: false,

            actionButtons: ["close"],

            animation:{
                show:{
                    effect:"fade",
                    duration:400
                },
                hide:{
                    effect:"fade",
                    duration:400
                }
            },            

            faviconCSS: null,

            content: null,

            target: null,

            enablePersistence: false,

            enabled: true,

            isResponsive: false,

            beforeClose: null,

            close: null,            

            expand: null,

            collapse: null,

            beforeOpen: null,

            open: null,

            drag: null,

            dragStart: null,

            dragStop: null,

            resize: null,

            resizeStart: null,

            resizeStop: null,

            contentLoad: null,

            ajaxSuccess: null,

            ajaxError: null,

            create: null,

            destroy: null,

            /*Deprecated*/            
            Close:null
        },

        dataTypes: {
            showOnInit: "boolean",
            closeOnEscape: "boolean",
            enableAnimation: "boolean",
            animation:"data",
            closeIconTooltip: "string",
            allowDraggable: "boolean",
            enableModal: "boolean",
            enableResize: "boolean",
            isResponsive: "boolean",
            showHeader: "boolean",
            title: "string",
            faviconCSS:"string",
            zIndex: "number",
            cssClass: "string",
            contentUrl: "string",
            contentType: "string",
            htmlAttributes: "data",
            ajaxSettings: "data",
            actionButtons: "array"
        },

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "closeIconTooltip": this._ejDialog.find("button.e-dialog-icon").attr("title", options[key]); break;
                    case "title":
                        this.model.title = options[key];
                        if (this._ejDialog.find("span.e-title").length <= 0) 
                            this._addTitleText();
                        else
                            this._ejDialog.find("span.e-title").html(options[key]);
                        this._updateCaptionWidth();
                        break;
                    case "width": this.model.width = options[key]; this._changeSize(); this._updateCaptionWidth(); break;
                    case "height": this.model.height = options[key]; this._changeSize(); break;
                    case "position": this.model.position = options[key]; this._dialogPosition(); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "showRoundedCorner": this._roundedCorner(options[key]); break;
                    case "contentType": { this._appendContent(options[key]); break; }
                    case "enabled": { this.model.enabled = options[key]; this._enabledAction(options[key]); break; }
                    case "contentUrl": { this.model.contentUrl = options[key]; this._appendContent(this.model.contentType); break; }
                    case "content":
                    case "target": 
                        this._ejDialog.appendTo($(options[key]));
                        this.model.target = this.model.content = options[key];
                        this._dialogPosition();
                        break;
                    case "minHeight": { this.model.minHeight = options[key]; this._ejDialog.css("minHeight", options[key]); this._resetScroller(),this._resizeDialog(); break; }
                    case "minWidth": { this.model.minWidth = options[key]; this._ejDialog.css("minWidth", options[key]); this._resetScroller(); this._resizeDialog(); break; }
                    case "maxHeight": { this.model.maxHeight = options[key]; this._ejDialog.css("maxHeight", options[key]); this._resetScroller(); this._resizeDialog(); break; }
                    case "maxWidth": { this.model.maxWidth = options[key]; this._ejDialog.css("maxWidth", options[key]); this._resetScroller(); this._resizeDialog(); break; }
                    case "zIndex": { this._ejDialog.css('z-index', options[key]); break; }
                    case "faviconCSS":
                        this.model.faviconCSS = options[key]; this._favIcon();this._updateCaptionWidth(); break;
                    case "isResponsive": {
                        this.model.isResponsive = options[key];
                        this.model.isResponsive ? this._ejDialog.addClass("e-dialog-resize") : this._ejDialog.removeClass("e-dialog-resize");
                        this._wireResizing();
                        break;
                    }
                    case "allowDraggable": {
                        this.model.allowDraggable = options[key];
                        if (options[key])
                            this._enableDrag();
                        else {
                            this._dialogTitlebar.removeClass("e-draggable");
                        }
                        break;
                    }
                    case "enableResize": {
                        this.model.enableResize = options[key];
                        if (options[key])
                            this._enableResize();
                        else {
                            this._ejDialog.removeClass("e-resizable");
                            this._ejDialog.find(".e-resize-handle").remove();
                        }
                        this._reRenderScroller();
                        break;
                    }
                    case "showHeader": {
                        this.model.showHeader = options[key];
                        if (options[key]) {
                            this._renderTitleBar();
                            this._iconsRender(this.model.actionButtons);
                            this._enableDrag();
                            this._updateScroller(this._ejDialog.outerHeight(true) - (this._dialogTitlebar.outerHeight(true)), this._ejDialog.width()-2);
                        }
                        else {
                            this._ejDialog.find(".e-titlebar").remove();
                            this._updateScroller(this._ejDialog.outerHeight(true), this._ejDialog.width()-2);
                        }
                        break;
                    }
                    case "enableRTL":
                        {
                            this.model.enableRTL = options[key];
                            if (this.model.faviconCSS) this._favIcon();
                            if (options[key]) {
                                this._ejDialog.addClass("e-rtl");
                                this.iframe && this.iframe.contents().find("body").css("direction", "rtl");
                                if (this.scroller) 
                                    this._resetScroller();
                            } else {
                                this._ejDialog.removeClass("e-rtl");
                                this.iframe && this.iframe.contents().find("body").css("direction", "ltr");
                                if (this.scroller) 
                                    this._resetScroller();
                            }
                            break;
                        }
                    case "actionButtons":
                        {
                            var diffArray = [];
                            proxy = this;
                            $.grep(this.model.actionButtons, function (el) {
                                if ($.inArray(el, options[key]) == -1) diffArray.push(el);
                            });
                            if (!ej.isNullOrUndefined(this._dialogTitlebar) && diffArray.length) {
                                this._removeAllIcons();
                                this._iconsRender(options[key]);
                            }
                            this.model.actionButtons = options[key];
                            this._updateCaptionWidth();
                            break;
                        }
                    case "enableModal": this._enableModal(options[key]); break;
                    case "htmlAttributes": this._addAttr(options[key]); break;
                }
            }
        },


        _destroy: function () {
            if (this._overLay) this._overLay.remove();
            this._cloneElement.appendTo(this._ejDialog.parent());
            this._ejDialog.remove();
            this._cloneElement.removeClass("e-dialog");
            this.element = this._cloneElement;
            this._isOpen = false;
        },

        keyConfigs: [37, 38, 39, 40],

        _init: function () {
            this._widthPercent = null;
            this._heightPercent = null;
            this._isOpen = this._maximize = this._minimize = false;            
            if (!this.model.close) this.model.close = this.model.Close;
            if (!this.model.target) this.model.target = this.model.content;
            this._renderControl();
            this._wireEvents();
        },

        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy._ejDialog.addClass(value);
                else if (key == "disabled" && value == "disabled") { proxy.model.enabled = false; proxy._enabledAction(false); }
                else proxy._ejDialog.attr(key, value);
            });
        },

        _renderControl: function () {
			this.initialRender = true;
            this._cloneElement = this.element.clone();
            this.element.attr("tabindex", 0).attr({ "role": "dialog", "aria-labelledby": this.element.prop("id") + "_title" });
            this._ejDialog = ej.buildTag("div.e-dialog e-widget e-box " + this.model.cssClass + " e-dialog-wrap e-shadow#" + this.element.prop("id") + "_wrapper", "", { display: "none", zIndex: this.model.zIndex }, { tabindex: 0 });
            if(this.model.isResponsive) this._ejDialog.addClass("e-dialog-resize");
            this.wrapper = this._ejDialog;
            this._addAttr(this.model.htmlAttributes);
            this._ejDialog.appendTo(!ej.isNullOrUndefined(this.model.target) ? $(this.model.target) : document.body);
            if (this.model.enableRTL) this._ejDialog.addClass("e-rtl");
            if (this.model.showHeader) {
                this._renderTitleBar();
                this._iconsRender(this.model.actionButtons);
                if (this.model.faviconCSS) this._favIcon();
            }
            this._appendContent(this.model.contentType);
            this._enableResize()._enableDrag()._setSize()._sizeInPercent();
            if (this.model.contentType != "ajax"){                                                                  
                if (this.model.showOnInit) this.open()._setContainerSize()._resetScroller();
                else this._setHiddenDialogSize();
            }
            this._roundedCorner(this.model.showRoundedCorner);
            this._enabledAction(this.model.enabled);
        },

        _setContainerSize: function () {
            if (this.model.height != "auto") {
                var cntHeight = this._ejDialog.outerHeight() - $(this._ejDialog.find("div.e-titlebar")).outerHeight();
                this.contentDiv.height(cntHeight);
                this.element.outerHeight(cntHeight);
            }
            return this;
        },

        _changeSize: function () {
            this._setSize()._sizeInPercent()._setContainerSize()._resetScroller();
        },

        _enableDrag: function () {
            if (this.model.allowDraggable && this.model.showHeader) {
                this._dialogTitlebar.addClass("e-draggable");
                this._dragDialog();
            }
            return this;
        },

        _enableResize: function () {
            if (this.model.enableResize) {
                this._ejDialog.addClass("e-resizable");
                var resizeDiv = ej.buildTag("div.e-icon e-resize-handle");
                resizeDiv.appendTo(this._ejDialog);
                this._resizeDialog();
            }
            return this;
        },


        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this._ejDialog.removeClass(this.model.cssClass).addClass(skin);
            }
        },
        _enableModal: function (value) {
            if (value) this._isOpen && this._createOverlay();
            else if (this._overLay) this._overLay.remove();
        },

        _enabledAction: function (flag) {
            if (flag) {
                this._ejDialog.removeClass("e-disable");
                if (!ej.isNullOrUndefined(this.scroller))
                    this.scroller.enable();
            }
            else {
                this._ejDialog.addClass("e-disable");
                if (!ej.isNullOrUndefined(this.scroller))
                    this.scroller.disable();
            }
        },

        _renderTitleBar: function () {
            this._elementTitle = this.element.attr("title");
            if (typeof this._elementTitle !== "string")
                this._elementTitle = "";
            this.model.title = this.model.title || this._elementTitle;
            this._dialogTitlebar = ej.buildTag("div#" + this.element.prop("id") + "_title.e-titlebar e-header e-dialog").prependTo(this._ejDialog);
            this._addTitleText();
        },
        _addTitleText: function () {
            if (this.model.title) 
                this._titleText = ej.buildTag("span.e-title", this.model.title).prependTo(this._dialogTitlebar);
            return this;
        },
        _updateCaptionWidth: function () {
            var addWidth=this.model.faviconCSS?this._dialogFavIcon.outerWidth():0;
			if(this._titleText)
			    this._titleText.css("max-width", (this._dialogTitlebar.width() - 20 - (this._dialogTitlebar.find(".e-dialog-icon").width() * this._dialogTitlebar.find(".e-dialog-icon").length) - addWidth));
			return this;
        },
        _iconsRender: function (iconArray) {
            for (var icon = 0; icon < iconArray.length; icon++) {
                switch (iconArray[icon]) {
                    case "close": {
                        this._closeIcon();
                        break;
                    }
                    case "collapse":
                    case "collapsible": {
                        this._collapsibleIcon();
                        break;
                    }
                    case "maximize": {
                        this._maximizeIcon();
                        break;
                    }
                    case "minimize": {
                        this._minimizeIcon();
                        break;
                    }
                    case "pin": {
                        this._pinIcon();
                        break;
                    }                    
                }
            }
        },

        _removeAllIcons: function () {
            this._dialogTitlebar.find("button.e-dialog-icon").remove();
        },

        _appendContent: function (contentType) {
            this.contentDiv = ej.isNullOrUndefined(this.contentDiv) ? ej.buildTag("div.e-dialog-scroller") : this.contentDiv;
            this.element.removeAttr("title").addClass("e-widget-content e-box");
            var proxy = this;
            if (!ej.isNullOrUndefined(this.model.contentUrl) && !ej.isNullOrUndefined(contentType)) {
                if (contentType == "ajax") {
                    this.model.ajaxSettings.url = this.model.contentUrl;
                    this._sendAjaxOptions(this.element, this.model.ajaxSettings.url);
                }
                else if (contentType == "iframe") {
                    if (this.element.children('.e-iframe').length > 0) {
                        this.iframe = this.element.find('iframe.e-iframe');
                        this.iframe.attr('src', this.model.contentUrl);
                    }
                    else {
                        this.iframe = ej.buildTag("iframe.e-iframe", "", { width: "100%", height:"100%" }, { scrolling: "no", frameborder: 0, src: this.model.contentUrl });
                        this.element.append(this.iframe).show().appendTo(this.contentDiv);
                    }
                    if (this.model.enableRTL) {
                        $(this.iframe).load(function () {
                            proxy.iframe.contents().find("body").css("direction", "rtl");
                        });
                    }
                    this._trigger("contentLoad", { contentType: contentType, url: this.model.contentUrl });
                }
                else if (contentType == "image") {
                    var img = ej.buildTag("img.e-images", "", "", { src: this.model.contentUrl });
                    this.element.append(img).show().appendTo(this.contentDiv);
                    $(img).load(function () {
                        proxy._dialogPosition();
                    });
                    this._trigger("contentLoad", { contentType: contentType, url: this.model.contentUrl });
                }
                else
                    this.element.show().appendTo(this.contentDiv);
            }
            else 
                this.element.show().appendTo(this.contentDiv);
            this._ejDialog.find("div.e-resize-handle").length > 0 ? this.contentDiv.insertBefore(this._ejDialog.find("div.e-resize-handle")) : this.contentDiv.appendTo(this._ejDialog);
        },

        _roundedCorner: function (value) {
            this._ejDialog[(value ? "addClass" : "removeClass")]('e-corner');
        },

        _reRenderScroller: function () {
            if (this.scroller != undefined) {
                this.scroller.refresh();
                if (!this.model.enableRTL) {
                    if ((this.scroller._vScrollbar && this.scroller._vScrollbar._scrollData) && this.model.enableResize) {
                        var height = this.scroller._vScrollbar.element.find('> div.e-vscroll').height(), padngSpace = 2, resizeHandleSize = this._ejDialog.find('div.e-resize-handle').outerHeight();
	                    if (this.contentDiv.outerHeight() === (this.scroller._vScrollbar.model.height + 1)) {
                            this.scroller._vScrollbar.model.height -= resizeHandleSize + padngSpace;
                            this.scroller._vScrollbar._scrollData.handle -= resizeHandleSize;
                            this.scroller._vScrollbar._scrollData.handleSpace -= resizeHandleSize + padngSpace;
                            this.scroller._vScrollbar._updateLayout(this.scroller._vScrollbar._scrollData);
                            this.scroller._vScrollbar.element.find('> div.e-vscroll').height(height - resizeHandleSize - padngSpace);
                        }
                    }
                    if (!(this.scroller._vScrollbar && this.scroller._vScrollbar._scrollData) && (this.scroller._hScrollbar && this.scroller._hScrollbar._scrollData) && this.model.enableResize) {
                        var width = this.scroller._hScrollbar.element.find('> div.e-hscroll').width(), padngSpace = 2, resizeHandleSize = this._ejDialog.find('div.e-resize-handle').outerWidth();
                        this.scroller._hScrollbar.model.width -= resizeHandleSize + padngSpace;
                        this.scroller._hScrollbar._scrollData.handle -= resizeHandleSize + padngSpace;
                        this.scroller._hScrollbar._scrollData.handleSpace -= resizeHandleSize + padngSpace;
                        this.scroller._hScrollbar._updateLayout(this.scroller._hScrollbar._scrollData);
                        this.scroller._hScrollbar.element.find('> div.e-hscroll').width(width - resizeHandleSize - padngSpace);
                    }
                }
            }
        },       

        _dialogMaxZindex: function () {
            var parents = this.element.parents(), bodyEle, contEle;
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });
            contEle = $(this.model.target).children(), cindex = contEle.index(this.popup);
            contEle.splice(cindex, 1);
            $(contEle).each(function (i, ele) { parents.push(ele); });
            var maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;
        },

        _setZindex: function () {
            var zindex = this._dialogMaxZindex();
            if (this.model.zIndex <= zindex)
                this._ejDialog.css({ zIndex: zindex + 1 });
        },

        _createOverlay: function () {
            var zindex = this._ejDialog.css('zIndex');
            this._overLay = ej.buildTag("div#" + this.element.attr("id") + "_overLay.e-overlay", "", { zIndex: zindex - 1});
            this._overLay.appendTo(this.model.target != null ? this.model.target : document.body);
        },

        _sendAjaxOptions: function (content, link) {
            //load waiting popup
            content.addClass("e-load");
            var proxy = this;
            var curTitle = this.model.title;
            var hrefLink = link;
            var ajaxOptions = {
                "success": function (data) {
                    try { proxy._ajaxSuccessHandler(data, content, link, curTitle); }
                    catch (e) { }
                },
                "error": function (e) {
                    try { proxy._ajaxErrorHandler({ "status": e.status, "responseText": e.responseText, "statusText": e.statusText }, content, link, curTitle); }
                    catch (e) { }
                },
                "complete": function () {
                    try {
                        proxy._setContainerSize();
                        proxy._resetScroller();
                        if (!proxy.model.showOnInit) proxy._setHiddenDialogSize();
                    } catch (e) {}
                }
            };
            $.extend(true, ajaxOptions, this.model.ajaxSettings);
            this._sendAjaxRequest(ajaxOptions);
        },

        _setHiddenDialogSize: function () {
            if (!this._isOpen) {
                this._ejDialog.css({ "display": "block", "visibility": "hidden" });
                this._setContainerSize()._resetScroller();
                this._ejDialog.css({ "display": "none", "visibility": "" });
            }
        },

        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },

        _ajaxSuccessHandler: function (data, content, link, curTitle) {
            content.removeClass("e-load");
            content.html(data).addClass("e-dialog-loaded").appendTo(this._ejDialog);
            content.appendTo(this.contentDiv);
            this._dialogPosition();
            if (this.model.showOnInit)
                this.open();
            this._trigger("ajaxSuccess", { data: data, url: link });
        },

        _ajaxErrorHandler: function (data, content, link, curTitle) {
            content.addClass("e-dialog-loaded").appendTo(this.contentDiv);
            this._dialogPosition().open();
            this._trigger("ajaxError", { data: data, url: link });
        },
        _createEJButton: function (element,cssClass) {
            element.ejButton({ size: "normal", contentType: "imageonly", prefixIcon: "e-icon " + cssClass });
        },
        _closeIcon: function () {
            this._dialogClose = ej.util.buildTag("button#" + this.element[0].id + "_closebutton.e-dialog-icon",null,null,{type:"button"});            
            this._createEJButton(this._dialogClose, "e-close");
            this._dialogClose.appendTo(this._dialogTitlebar).attr("title", this.model.closeIconTooltip);
            this._on(this._dialogClose, "touchstart click", this._closeClick);
        },
		
        _collapsibleIcon: function () {            
            this._dialogCollapsible = ej.util.buildTag("button#" + this.element[0].id + "_collapsbutton.e-dialog-icon",null,null,{type:"button"});            
            this._createEJButton(this._dialogCollapsible,"e-collapse-arrow");
            this._dialogCollapsible.appendTo(this._dialogTitlebar);
            this._on(this._dialogCollapsible, "touchstart click", this._collapsibleClick);
        },

        _maximizeIcon: function () {
            this._dialogMaximize = ej.util.buildTag("button#" + this.element[0].id + "_maximizebutton.e-dialog-icon",null,null,{type:"button"});
            this._createEJButton(this._dialogMaximize,"e-maximize");
            this._dialogMaximize.appendTo(this._dialogTitlebar);
            this._on(this._dialogMaximize, "touchstart click", this._maximizeClick);
        },

        _minimizeIcon: function () {
            this._dialogMinimize = ej.util.buildTag("button#" + this.element[0].id + "_minimizebutton.e-dialog-icon",null,null,{type:"button"});            
            this._createEJButton(this._dialogMinimize,"e-minimize");
            this._dialogMinimize.appendTo(this._dialogTitlebar);
            this._on(this._dialogMinimize, "touchstart click", this._minimizeClick);
        },

        _pinIcon: function () {
            this._dialogPin = ej.util.buildTag("button#" + this.element[0].id + "_pinbutton.e-dialog-icon",null,null,{type:"button"});
            this._createEJButton(this._dialogPin,"e-unpin");
            this._dialogPin.appendTo(this._dialogTitlebar);
            this._on(this._dialogPin, "touchstart click", this._pinClick);
        },

        _favIcon: function () {
            if (!this._dialogFavIcon) {
                this._dialogFavIcon = ej.util.buildTag("div.e-dialog-favicon", "", {}, { style: "float:"+ (this.model.enableRTL?"right":"left") });
                var span = ej.util.buildTag("span.e-dialog-custom", "", {}, { role: "presentation" });
                span.appendTo(this._dialogFavIcon);
                this._dialogFavIcon.appendTo(this._dialogTitlebar);
            }
            else 
                span = this._dialogFavIcon.find("span").removeClass().addClass("e-dialog-custom");
            if (!this.model.faviconCSS) this._dialogFavIcon.remove();
            else span.addClass(this.model.faviconCSS); this._dialogFavIcon.css("float", (this.model.enableRTL ? "right" : "left"));
        },

        _setSize: function () {
            var mdl = this.model;
            this._ejDialog.css({ width: mdl.width, minWidth: mdl.minWidth, maxWidth: mdl.maxWidth });
            this._ejDialog.css({ height: mdl.height, minHeight: mdl.minHeight, maxHeight: mdl.maxHeight });
            this._dialogPosition();
            return this;
        },

        _resetScroller: function () {		
            var scrHeight = this._ejDialog.outerHeight(true) - $(this._ejDialog.find("div.e-titlebar")).outerHeight(true) , eleHeight;
            var scrModel = { width: this._ejDialog.width()-2, enableRTL: this.model.enableRTL, height: scrHeight };
            this.element.css({ "height": "auto", "max-width": "", "max-height": "", "width": "" });
            if ((this.model.height == "auto" || this.model.height == "100%") && (this.element.height() < this.model.maxHeight || !this.model.maxHeight))
                scrModel.height = "100%";
            if (this.model.width == "auto" || this.model.width == "100%")
                scrModel.width = this.model.width;
            this.contentDiv.ejScroller(scrModel);
            this.scroller = this.contentDiv.data("ejScroller");
            this._reRenderScroller();
            this._padding = parseInt($(this.element).css("padding-top")) + parseInt($(this.element).css("padding-bottom"));
            if (!ej.isNullOrUndefined(this.contentDiv.height()) && this.contentDiv.height() > 0) {
                if (!ej.isNullOrUndefined(this.scroller))
                    if (!this.scroller._hScrollbar && this.scroller._vScrollbar)
                        eleHeight = this.contentDiv.outerHeight();
                    else if (this.scroller._hScrollbar)
                         eleHeight = this.contentDiv.outerHeight() - this.scroller.model.buttonSize;
                     else  eleHeight = this.contentDiv.outerHeight();
                 else
                    eleHeight = this.contentDiv.outerHeight() - this._padding;
                if ((this.model.height != "auto" && this.model.height != "100%"))
                    this.element.css({ "height": eleHeight });
                else
                    this.element.css("height", this.model.height);
                if ((this.model.height == "auto" || this.model.height == "100%") && !this.scroller._vScrollbar)
                    this.element.css({ "min-height": this.model.minHeight - $(this._ejDialog.find("div.e-titlebar")).outerHeight(true)});
                if (!this.scroller._vScrollbar && (this.model.width != "auto" && this.model.width != "100%"))
                    this.element.outerWidth(this._ejDialog.width() - 2);
                else if (!this.scroller._vScrollbar)
                    this.element.css("width", this.model.width);
                this.element.css({"max-width": this.model.maxWidth, "max-height": this.model.maxHeight });
            }
        },

        _updateScroller: function (height, width) {
            this.contentDiv.ejScroller({ width: width, height: height, enableRTL: this.model.enableRTL });
            this.scroller = this.contentDiv.data("ejScroller");
            this._reRenderScroller();
        },

        _dragDialog: function () {
            var proxy = this;
            var pos = this._ejDialog.parents(".e-dialog-scroller");
            this._dialogTitlebar.ejDraggable({
                handle: ".e-titlebar",
                cursorAt: { top: 0, left: 0 },
                dragStart: function (event) {
                    proxy._clickHandler();
                    if (proxy.dialogPin || !proxy.model.allowDraggable || !proxy.model.enabled) {
                        event.cancel = true;
                        return false;
                    }
                    if (proxy._trigger("dragStart", { event: event })) {
                        event.cancel = true;
                        return false;
                    }
                },
                drag: function (event) {
                    proxy._trigger("drag", { event: event });
                },
                dragStop: function (event) {
                    proxy._ejDialog.focus();
                    if (proxy.element.find("> .e-draggable.e-titlebar")) {
                        dragobject = $("#" + proxy.element.find("> .e-draggable.e-titlebar").attr("id")).data("ejDraggable");
                        if (dragobject)
                            dragobject.option("cursorAt", proxy.element.offset());
                    }
                    proxy._trigger("dragStop", { event: event });
                    proxy.model.position.X = this.position.left;
                    proxy.model.position.Y = this.position.top;
                },
                helper: function (event) {
                    return $(proxy._ejDialog).addClass("dragClone");
                }
            });
            return this;
        },

        _resizeDialog: function () {
            if (!this.model.enableResize)  return;
            var proxy = this, started = false, parentObj;
            var _minWidth = parseInt(this.model.minWidth), _minHeight = parseInt(this.model.minHeight);
            var _maxWidth = parseInt(this.model.maxWidth), _maxHeight = parseInt(this.model.maxHeight);
            parentObj = this._getParentObj();
            if (isNaN(this.model.minWidth) && (this.model.minWidth.indexOf("%") > 0))
                _minWidth = this._convertPercentageToPixel(parentObj.outerWidth(), _minWidth);
            if (isNaN(this.model.minHeight) && (this.model.minHeight.indexOf("%") > 0))
                _minHeight = this._convertPercentageToPixel(parentObj.outerHeight(), _minHeight);
            if (isNaN(this.model.maxWidth) && (this.model.maxWidth.indexOf("%") > 0))
                _maxWidth = this._convertPercentageToPixel(parentObj.innerWidth(), _maxWidth);
            if (isNaN(this.model.maxHeight) && (this.model.maxHeight.indexOf("%") > 0))
                _maxHeight = this._convertPercentageToPixel(parentObj.innerHeight(), _maxHeight);
            this._ejDialog.find("div.e-resize-handle").ejResizable(
                {
                    minHeight: _minHeight,
                    minWidth: _minWidth,
                    maxHeight: _maxHeight,
                    maxWidth: _maxWidth,
                    handle: "e-widget-content",
                    resizeStart: function (event) {
                        if (!proxy.model.enabled)
                            return false;
                        !started && proxy._trigger("resizeStart", { event: event });
                        started = true;
                        proxy.model.position = { X: proxy._ejDialog.css("left"), Y: proxy._ejDialog.css("top") };
                        proxy._dialogPosition();
                    },
                    resize: function (event) {
                        var reElement = $(event.element).parents("div.e-dialog-wrap");
                        proxy.model.height = $(reElement).outerHeight();
                        proxy.model.width = $(reElement).outerWidth();
                        proxy._setSize();
                        proxy._setContainerSize();
                        proxy._resetScroller();
                        proxy._updateCaptionWidth();
                        proxy._trigger("resize", { event: event });
                    },
                    resizeStop: function (event) {
                        proxy._ejDialog.focus();
                        proxy._sizeInPercent();
                        var reElement = $(event.element).parents("div.e-dialog-wrap");
                        proxy.model.height = $(reElement).outerHeight();
                        proxy.model.width = $(reElement).outerWidth();
                        proxy._setSize();
                        proxy._setContainerSize();
                        proxy._resetScroller();
                        started && proxy._trigger("resizeStop", { event: event });
                        started = false;
                    },
                    helper: function (event) {
                        return $(proxy._ejDialog);
                    }
                });
            return this;
        },

        _dialogPosition: function () {
            var location;
            if (this._ejDialog.parents("form").length > 0 && ej.isNullOrUndefined(this.model.target)) {
                this._ejDialog.appendTo(this._ejDialog.parents("form"));
            }
            if (this.model.position.X != "" || this.model.position.Y != "") {
                this._ejDialog.css("position", "absolute");
                this._ejDialog.css("left", this.model.position.X);
                this._ejDialog.css("top", this.model.position.Y);
            }
            else {
                var x = 0, y = 0;
                if (!ej.isNullOrUndefined(this.model.target)) {
                    var $content = $(this.model.target);
                    if ($content.css("position") == 'static') {
                        var parentOffset = $content.offsetParent().offset();
                        var contentOffset = $content.offset();
                        x = contentOffset.left - parentOffset.left;
                        y = contentOffset.top - parentOffset.top;
                    }
                    if ($content.outerWidth() > this._ejDialog.width()) x += ($content.outerWidth() - this._ejDialog.width()) / 2;
                    if ($content.outerHeight() > this._ejDialog.height()) y += ($content.outerHeight() - this._ejDialog.height()) / 2;
                }
                else {
                    var doc = document.documentElement;
                    x = (($(window).outerWidth() > this._ejDialog.width()) ? ($(window).outerWidth() - this._ejDialog.outerWidth()) / 2 : 0) + (window.pageXOffset || doc.scrollLeft);
                    y = (($(window).outerHeight() > this._ejDialog.height()) ? ($(window).outerHeight() - this._ejDialog.outerHeight()) / 2 : 0) + (window.pageYOffset || doc.scrollTop);
                }
                this._ejDialog.css({ top: y, left: x });
                this._ejDialog.css("position", "absolute");
            }
            return this;
        },

        _closeClick: function (event) {
            if (this.model.enabled) {
                event.stopPropagation();
                this.close(event);
            }
        },

        _collapsibleClick: function (e) {
            e.target = this._dialogCollapsible.find("span.e-icon");
            if (this.model.enabled) {
                if ($(e.target).hasClass("e-collapse-arrow")) {
                    this._actionCollapse();
                }
                else if ($(e.target).hasClass("e-expand-arrow")) {
                    this._actionExpand();
                }
            }
        },
        _actionCollapse: function () {
            this._dialogCollapsible && this._dialogCollapsible.find("span.e-collapse-arrow").removeClass("e-collapse-arrow").addClass("e-expand-arrow");
            this._ejDialog.find("div.e-resize-handle").hide();
            this._ejDialog.find(".e-widget-content").parent().slideUp("fast");
            this._ejDialog.removeClass("e-shadow");
            this._ejDialog.css("minHeight", "0");
            this._ejDialog.height("auto");
            this._trigger("collapse");
        },
        _actionExpand: function () {
            this._dialogCollapsible && this._dialogCollapsible.find("span.e-expand-arrow").removeClass("e-expand-arrow").addClass("e-collapse-arrow");
            this._ejDialog.addClass("e-shadow");
            this._ejDialog.find(".e-widget-content").parent().slideDown("fast");
            if (this._maximize) {
                this._ejDialog.css({ width: "100%", height: "100%" });
                this.element.css({ width: "100%", height: "100%" });
                this.contentDiv.css({ width: "100%", height: "100%" });
            }
            else this._ejDialog.height(this.model.height);
            this._ejDialog.find("div.e-resize-handle").show();
            this._trigger("expand");
        },

        _maximizeClick: function (e) {
            e.target = this._dialogMaximize.find("span.e-icon");
            if (this.model.enabled) {
                var _target = $(e.target);
                var hideIcon = this._dialogMaximize.find("span.e-icon");
                if (_target.hasClass("e-maximize")) {
                    this._actionMaximize();
                    if (this._dialogTitlebar)
                        this._dialogTitlebar.find('.e-restore').removeClass('e-restore').addClass('e-minimize');
                    _target.removeClass("e-maximize").addClass("e-restore");
                    this._hideIcon(true);
                }
                else if (_target.hasClass("e-restore")) {
                    this._actionRestore();
                    _target.removeClass("e-restore").addClass("e-maximize");
                    if (!ej.isNullOrUndefined(hideIcon) && $(hideIcon).hasClass('e-expand-arrow')) $(hideIcon).removeClass('e-expand-arrow').addClass('e-collapse-arrow');
                    this._hideIcon(true);
                }
            }
        },
        _actionMaximize: function () {
            this._ejDialog.css("top", "0px").css("left", "0px").css("position", (this.model.target ?"absolute":"fixed"));
            this._ejDialog.css({ width: "100%", height: "100%" });
            this.element.css({ width: "100%", height: "100%" });
            this.contentDiv.css({ width: "100%", height: "100%" });
            this._maximize = true;
            this._minimize = false;
            if (this._dialogCollapsible && !ej.isNullOrUndefined(this._dialogCollapsible.find("span.e-expand-arrow"))) {
                this._dialogCollapsible.find("span.e-expand-arrow").removeClass("e-expand-arrow").addClass("e-collapse-arrow");
                this._ejDialog.find(".e-widget-content").parent().slideDown("fast");
            }
            this._resetScroller();
        },
        _actionRestore: function () {
            this.element.height("").width("");
            this.contentDiv.height("").width("");
            this._restoreDialog();
            this._maximize = this._minimize = false;
        },

        _minimizeClick: function (e) {
            e.target = this._dialogMinimize.find("span.e-icon");
            if (this.model.enabled) {
                var _target = $(e.target);
                var hideIcon = this._dialogMinimize.find("span.e-icon");
                if (_target.hasClass("e-minimize")) {
                    if (this._maximize)
                        this._setSize();
                    this._actionMinimize();
                } else if (_target.hasClass("e-restore")) {
                    this._actionRestore();
                    _target.removeClass("e-restore").addClass("e-minimize");
                    if (!ej.isNullOrUndefined(hideIcon) && $(hideIcon).hasClass('e-expand-arrow')) $(hideIcon).removeClass('e-expand-arrow').addClass('e-collapse-arrow');
                    this._hideIcon(true);
                }
            }
        },

        _actionMinimize: function () {
            var top = ($(window).height() - this._ejDialog.height()) + this.element.height() + 14, _height;
            this._ejDialog.css("top", "").css("bottom", "0").css("left", "0").css("position", (this.model.target ? "absolute" : "fixed"));
            this._ejDialog.css("minHeight", "0");
            if (this._dialogTitlebar) {
                this._dialogTitlebar.find('.e-restore').removeClass('e-restore').addClass('e-maximize');
                this._dialogTitlebar.find(".e-minimize").removeClass("e-minimize").addClass("e-restore");
                if (this._isOpen)
                    _height = this._dialogTitlebar.outerHeight();
                else {
                    this._ejDialog.css({ "display": "block", "visibility": "hidden" });
                    _height = this._dialogTitlebar.outerHeight();
                    this._ejDialog.css({ "display": "none", "visibility": "" });
                }
                this._ejDialog.css("height", _height + 2); // 1px bordertop + 1px borderbottom of the dialog wrapper is added
                this._hideIcon(false);
            } else this._ejDialog.css("height", "");
            this.contentDiv.hide();
            this._maximize = false;
            this._minimize = true;
        },

        _hideIcon: function (value) {
            var hideIcon = this._dialogCollapsible ? this._dialogCollapsible.find("span.e-icon") : null;
            if (value) {
                if (!ej.isNullOrUndefined(hideIcon)) $(hideIcon).parent('.e-dialog-icon').show();
                this._ejDialog.find("div.e-resize-handle").show();
            } else {
                if (!ej.isNullOrUndefined(hideIcon)) $(hideIcon).parent('.e-dialog-icon').hide();
                this._ejDialog.find("div.e-resize-handle").hide();
            }
        },

        _pinClick: function (e) {
            e.target = this._dialogPin.find("span.e-icon");
            if (this.model.enabled) {
                var _target = $(e.target);
                if (_target.hasClass("e-unpin")) {
                    this.dialogPin = true;
                    _target.removeClass("e-unpin").addClass("e-pin");
                }
                else if (_target.hasClass("e-pin")) {
                    this.dialogPin = false;
                    _target.removeClass("e-pin").addClass("e-unpin");
                }
            }
        },

        _restoreDialog: function () {
            this.contentDiv.show();
            this._ejDialog.css({"position": "absolute", "bottom": ""}).addClass("e-shadow");
            this._setSize()._resetScroller();
            this._dialogTitlebar && this._dialogTitlebar.find(".e-minimize").parent().show();
        },

        _clickHandler: function (e) {
            var zindex = this._dialogMaxZindex();
            if (parseInt(this._ejDialog.css("zIndex")) < zindex) 
                this._ejDialog.css({ zIndex: zindex + 1 });            
        },

        _mouseClick: function (e) {
            if ($(e.target).hasClass("e-dialog") || $(e.target).hasClass("e-icon e-resize-handle")) {
                this._setZindex();
                $(e.target).closest(".e-dialog.e-widget").focus();
            }
        },

        _keyDown: function (e) {
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            if (this.model.allowKeyboardNavigation && this.model.enabled && $(e.target).hasClass("e-dialog"))
                if ($.inArray(code, this.keyConfigs) > -1 && this.model.allowDraggable && !this.dialogPin) {
                    this._keyPressed(code, e.ctrlKey);
                    e.preventDefault();
                }            
            if (this.model.closeOnEscape && code === 27 && this.model.enabled) 
                if (!this.element.find(".e-js.e-dialog").first().is(":visible")) {
                    this.close(e);
                    e.preventDefault();
                }         
            if (code == 9 && this.model.enableModal)
                this._focusOnTab(e);
        },

        _focusOnTab: function (e) {
            var focusEle = this._ejDialog.find("a, button, :input, select, [tabindex]:not('-1')");
            focusEle = $(focusEle).find("a, button, :input, select, [tabindex]:not('')");
            if (e.shiftKey) {
                if (!focusEle[focusEle.index(e.target) - 1]) {
                    e.preventDefault();
                    focusEle.last().focus();
                }
            }
            else if (!focusEle[focusEle.index(e.target) + 1]) {
                e.preventDefault();
                focusEle[0].focus();
            }
        },

        _keyPressed: function (code, ctrlKey) {
            switch (code) {
                case 40:
                    ctrlKey ? this.option("height", (this._ejDialog.outerHeight() + 3)) : this.option("position", { X: this._ejDialog.position().left, Y: (this._ejDialog.position().top + 3) });
                    break;
                case 39:
                    ctrlKey ? this.option("width", (this._ejDialog.outerWidth() + 3)) : this.option("position", { X: (this._ejDialog.position().left + 3), Y: this._ejDialog.position().top });
                    break;
                case 38:
                    ctrlKey ? this.option("height", (this._ejDialog.outerHeight() - 3)) : this.option("position", { X: this._ejDialog.position().left, Y: (this._ejDialog.position().top - 3) });
                    break;
                case 37:
                    ctrlKey ? this.option("width", (this._ejDialog.outerWidth() - 3)) : this.option("position", { X: (this._ejDialog.position().left - 3), Y: this._ejDialog.position().top });
                    break;
            }
        },

        _sizeInPercent: function () {
            if (!this._enableWindowResize()) return this;
            var parentObj = this._getParentObj();
            this._widthPercent = this._convertPixelToPercentage(parentObj.outerWidth(), this._ejDialog.outerWidth());
            this._heightPercent = this._convertPixelToPercentage(parentObj.outerHeight(), this._ejDialog.outerHeight());
            if (this._widthPercent > 100) {
                this._widthPercent = 100;
                this._ejDialog.outerWidth(parentObj.outerWidth());
                this.model.width = this._ejDialog.width();
            }
            if (this._heightPercent > 100) {
                this._heightPercent = 100;
                this._ejDialog.outerHeight(parentObj.outerHeight());
                this.model.height = this._ejDialog.height();
            }
            return this;
        },

        _getParentObj: function () {
            return (!ej.isNullOrUndefined(this.model.target)? $(this.model.target):$(window));
        },

        _convertPercentageToPixel: function (parent, child) {
            return Math.round((child * parent) / 100);
        },

        _convertPixelToPercentage: function (parent, child) {
            return Math.round((child / parent) * 100);
        },

        _reSizeHandler: function () {
            var parentObj;
            if (this._maximize) { this._resetScroller(); return;}
            if (!this._minimize) this._dialogPosition();
            parentObj = this._getParentObj();
            this._ejDialog.outerWidth(this._convertPercentageToPixel(parentObj.outerWidth(), this._widthPercent));            
            this._ejDialog.outerHeight(this._convertPercentageToPixel(parentObj.outerHeight(), this._heightPercent));
            this.contentDiv.width(this._ejDialog.width());
            this.element.outerWidth(this.contentDiv.width());
            this.contentDiv.height(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
            this.element.outerHeight(this._ejDialog.height() - $(this._ejDialog.find("div.e-titlebar")).outerHeight());
            this.scroller = this.contentDiv.ejScroller({ width: this._ejDialog.width() - 2, height: this.element.outerHeight(), rtl: this.model.rtl });
            this.scroller = this.contentDiv.data("ejScroller");
            this._reRenderScroller();
            this._padding = parseInt($(this.element).css("padding-top")) + parseInt($(this.element).css("padding-bottom"));
            if (!ej.isNullOrUndefined(this.contentDiv.height()) && this.contentDiv.height() > 0)
                this.element.css({ "height": this.contentDiv.height() - this._padding });
            if (!this._minimize) this._dialogPosition();
            else this._ejDialog.css("height", this._dialogTitlebar.outerHeight());
            this._updateCaptionWidth();
        },

        _enableWindowResize: function () {
            return (this.model.isResponsive || ((isNaN(this.model.width) && (this.model.width.indexOf("%") > 0)) && (isNaN(this.model.height) && (this.model.height.indexOf("%") > 0) && (this.model.height != "auto")))) ? true : false;
        },

        _wireResizing: function () {             
            $(window)[(this._enableWindowResize()?"bind":"unbind")]('resize', $.proxy(this._reSizeHandler, this));
        },

        _wireEvents: function () {
            this._on(this._ejDialog, "keydown", this._keyDown);
            this._on(this._ejDialog, "click", this._mouseClick);
            this._wireResizing();
        },

        refresh: function () {
            this._resetScroller();
        },

        open: function () {
            if (this._isOpen) return true;            
            if(!this.initialRender) if (true == this._trigger("beforeOpen")) return false;
            this.element.css("display", "block");
            this._setZindex();
            if (!this._minimize && !this._maximize) this._dialogPosition();
            var proxy = this, effect = {};
            this._ejDialog.show();
            if (this.model.enableAnimation) {
                this._ejDialog.css({ "opacity": 0});
                if (this.model.animation.show.effect == "slide") {
                    var eLeft = this._ejDialog.css("left");
                    this._ejDialog.css({ "left": -this._ejDialog.width() });
                    effect = { left: eLeft, opacity: 1 };
                }
                else {
                    effect = { opacity: 1 };
                }
            }
            this._ejDialog.animate(effect, (this.model.enableAnimation ? Number(this.model.animation.show.duration) : 0), function () {
                if (proxy.model) {
                    proxy._ejDialog.eq(0).focus();
                    proxy._ejDialog.css({ "opacity": "" });
                    proxy.contentDiv.find("a, button, :input, select").first().focus();
                    if(!proxy.initialRender) proxy._trigger("open");
                    proxy._updateCaptionWidth();
                    if (proxy.model.maxHeight || proxy.model.maxWidth)
                        proxy._updateScroller(proxy._ejDialog.outerHeight(true) - ((proxy._dialogTitlebar ? proxy._dialogTitlebar.outerHeight(true) : 0)), proxy._ejDialog.width() - 2);
					proxy.initialRender=false;
                }
            });
            if (this.model.enableModal == true) this._createOverlay();
            this._isOpen = true;
            if (this.element.find(".e-draggable.e-titlebar")) {
                dragobject = $("#" + this.element.find(".e-draggable.e-titlebar").attr("id")).data("ejDraggable");
                if (dragobject)
                    dragobject.option("cursorAt", this.element.offset());
            }
            return this;
        },

        close: function (event) {
            if (!this._isOpen || !this.model.enabled) return true;
            if (true == this._trigger("beforeClose", { event: event })) return false;
            this._isOpen = false;
            var proxy = this, effect = {};
            if (this.model.enableAnimation) 
            effect = this.model.animation.hide.effect == "slide" ? { left: -this._ejDialog.width(), opacity: 0 } : { opacity: 0 };
            this._ejDialog.animate(effect, (this.model.enableAnimation ? Number(this.model.animation.hide.duration) : 0), function () {
                proxy._trigger("close", { event: event });
                proxy._ejDialog.hide();
            });
            if (this.model.enableModal) 
                if (this._overLay) this._overLay.remove();
            return this;
        },

        isOpened: function () { return this.isOpen() },
        isOpen: function () { return this._isOpen; },

        setTitle: function (titleText) {
            this.model.title = titleText;
            this._titleText.text(titleText);
            this._updateCaptionWidth();
            return this;
        },

        setContent: function (htmlContent) {
            if (!this.model.enabled) return false;
            this.element.html(htmlContent);
            this._resetScroller();
            return this;
        },      

        focus: function () {
            this._setZindex();
            this.element.focus();
            return this;
        },

        minimize: function () {
            this._actionMinimize();
            $(this.wrapper.find(".e-minimize")[0]).parent().hide();
            return this;
        },

        maximize: function () {
            this._actionMaximize();
            $(this.wrapper.find(".e-maximize")[0]).removeClass("e-maximize").addClass("e-restore");
            this.contentDiv.show();
            this._dialogTitlebar && this._dialogTitlebar.find(".e-minimize").parent().show();
            return this;
        },

        restore: function () {
            this._actionRestore();
            $(this.wrapper.find(".e-restore")[0]).removeClass("e-restore").addClass("e-maximize");
            return this;
        },

        pin: function () {
            this.dialogPin = true;
            $(this.wrapper.find(".e-unpin")[0]).removeClass("e-unpin").addClass("e-pin");
            return this;
        },

        unpin: function () {
            this.dialogPin = false;
            $(this.wrapper.find(".e-pin")[0]).removeClass("e-pin").addClass("e-unpin");
            return this;
        },

        collapse: function () {
            this._actionCollapse();
            return this;
        },

        expand: function () {
            this._actionExpand();
            return this;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejDropDownList", "ej.DropDownList", {
        element: null,

        model: null,
        validTags: ["select", "input"],
        _ignoreOnPersist: ["fields", "dataSource", "query", "create", "popupHide", "popupShown", "beforePopupShown", "beforePopupHide", "change", "select", "checkChange", "destroy"],
        _setFirst: false,

        _rootCSS: "e-dropdownlist",
      


        defaults: {
            cssClass: "",

            value: "",

            htmlAttributes: {},

            text: "",

            itemValue: "",

            itemsCount: 0,

            dataSource: null,

            delimiterChar: ',',

            query: null,

            fields: {
                id: null,

                text: null,

                value: null,

                category: null,
                
                groupBy:null,

                imageUrl: null,

                imageAttributes: null,

                spriteCssClass: null,

                htmlAttributes: null,

                selected: null,

                tableName: null
            },

            watermarkText: null,

            height: "",

            width: "",

            popupHeight: "152px",

            popupWidth: "auto",

            maxPopupHeight: null,

            minPopupHeight: 0,

            maxPopupWidth: null,

            minPopupWidth: 0,

            targetID: null,

            template: null,

            selectedItemIndex: null,

            selectedIndex:null,

            disableItemsByIndex: null,

            enableItemsByIndex: null,

            selectedItems: [],

            selectedIndices:[],

            cascadeTo: null,

            enablePopupResize: false,

            allowVirtualScrolling: false,

            virtualScrollMode: "normal",

            showRoundedCorner: false,

            showPopupOnLoad: false,

            enableRTL: false,

            enabled: true,

            filterType: "contains",

            sortOrder: "ascending",

            caseSensitiveSearch: false,

            showCheckbox: false,

            checkAll: false,

            uncheckAll: false,

            enablePersistence: false,

            enableFilterSearch: false,

            enableIncrementalSearch:true,

            readOnly: false,

            enableAnimation: false,

            multiSelectMode: "none",

            allowGrouping: false,

            enableSorting: false,

            validationRules: null,

            validationMessage: null,

            actionComplete: null,

            actionFailure: null,

            actionSuccess: null,

            create: null,

            popupHide: null,

            popupShown: null,

            beforePopupShown: null,

            beforePopupHide: null,

            popupResizeStart: null,

            popupResize: null,

            popupResizeStop: null,

            change: null,

            select: null,

            dataBound: null,

            search: null,

            checkChange: null,

            cascade: null,

            destroy: null

        },

        dataTypes: {
            cssClass: "string",
            itemsCount: "number",
            watermarkText: "string",
            template: "string",
            disableItemsByIndex: "string",
            enableItemsByIndex: "string",
            enableIncrementalSearch:"boolean",
            cascadeTo: "string",
            delimiterChar: "string",
            showRoundedCorner: "boolean",
            showPopupOnLoad: "boolean",
            enableRTL: "boolean",
            enablePersistence: "boolean",
            allowVirtualScrolling: "boolean",
            virtualScrollMode: "enum",
            enabled: "boolean",
            readOnly: "boolean",
            multiSelectMode: "enum",
            dataSource: "data",
            query: "data",
            fields: "data",
            selectedItems: "array",
            enableAnimation: "boolean",
            allowGrouping: "boolean",
            enableSorting: "boolean",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data"
        },

        observables: ["value", "selectedItemIndex", "selectedIndex"],
        value: ej.util.valueFunction("value"),
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),
        selectedIndex: ej.util.valueFunction("selectedIndex"),
        

        enable: function () {
            if (this.element.hasClass("e-disable")) {
                this.target.disabled = false;
                this.model.enabled = true;
                this.element.removeClass('e-disable');
                this.dropdownbutton.removeClass('e-disable');
                this.popupListWrapper.removeClass('e-disable');
                if (this._isIE8) this.drpbtnspan.removeClass("e-disable");
                //Element not Maintain in Multiselection
                this.container.bind("mousedown", $.proxy(this._OnDropdownClick, this));
                if (this.model.multiSelectMode == "visualmode") this._ulBox.removeClass("e-disable");
            }
        },

        disable: function () {
            if (!this.element.hasClass("e-disable")) {
                this.target.disabled = true;
                this.model.enabled = false;
                this.element.addClass('e-disable');
                this.popupListWrapper.addClass('e-disable');
                this.dropdownbutton.addClass('e-disable');
                if (this._isIE8) this.drpbtnspan.addClass("e-disable");
                if (this.model.multiSelectMode == "visualmode") this._ulBox.addClass("e-disable");
                //Element not Maintain in Multiselection
                this.container.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
                if (this._isPopupShown()) this._hideResult();
            }
        },

        /* will deprecate with text property */
        getValue: function () {
            return this.target.value;
        },
        _setValue: function (value) {
            if (value) {
                this.unselectItemByText(this.model.text);
                this.selectItemByValue(value);
            }
        },
        _setText: function (text) {
            if (text) {
                this.unselectItemByText(this.model.text);
                this.selectItemByText(text);
            }
        },
        _setItemValue: function (itemValue) {
            this.model.itemValue = itemValue;
        },
        _changeWatermark: function (text) {
            if (!this.model.enabled) return false;
            if (this._isWatermark) this.element.attr("placeholder", text);
            else this._hiddenSpan.text(text);
        },

        hidePopup: function () {
            if (!this.model.enabled) return false;
            if (this.ultag.find('li').length > 0)
                this._hideResult();
        },

        showPopup: function () {
            if (!this.model.enabled) return false;
            if (this.ultag.find('li').length > 0)
                this._showResult();
        },

        clearText: function () {
            this._clearTextboxValue();
            if (!this._isWatermark)
                this._setWatermark();
        },

        addItem: function (itemTag) {
            if (!this.model.enabled || !itemTag) return false;
            this._mapFields();
            var list = $.isArray(itemTag) ? itemTag : [itemTag];
            if (list.length < 1) return false;
            var mapper = this.mapFld,
                mapFld = { _id: null, _imageUrl: null, _imageAttributes: null, _spriteCSS: null, _text: null, _value: null, _htmlAttributes: null, _selected: null, _category:null};
            mapFld._id = list[0][mapper._id] ? mapper._id : "id";
            mapFld._imageUrl = list[0][mapper._imageUrl] ? mapper._imageUrl : "imageUrl";
            mapFld._imageAttributes = list[0][mapper._imageAttributes] ? mapper._imageAttributes : "imageAttributes";
            mapFld._spriteCSS = list[0][mapper._spriteCSS] ? mapper._spriteCSS : "spriteCssClass";
            mapFld._text = list[0][mapper._text] ? mapper._text : "text";
            mapFld._value = list[0][mapper._value] ? mapper._value : "value";
            mapFld._htmlAttributes = list[0][mapper._htmlAttributes] ? mapper._htmlAttributes : "htmlAttributes";
            mapFld._selected = list[0][mapper._selected] ? mapper._selected : "selected";
            mapFld._category = list[0][mapper._category] ? mapper._category : "groupBy";
            this._generateLi(list, mapFld);
            this._virtualUl.append($(this.dummyUl).clone(true));
            this.ultag.append(this.dummyUl);
            var i, listItems = this.dummyUl;
            if (this.model.showCheckbox) {
                this._addCheckbox(listItems)
                for (i = 0; i < listItems.length; i++) {
                    if ($(listItems[i]).hasClass('chkselect')) {
                        $(listItems[i]).find(".listcheckbox").ejCheckBox({ "checked": true });
                        this._activeItem = i + this.popupListItems.length;
                        this.checkedStatus = true;
                        if (!this._initValue)
                            this._trigger('checkChange', { isChecked: this.checkedStatus, data:this.model });
                        this._enterTextBoxValue();
                        $(listItems[i]).removeClass('chkselect');
                    }
                }
            }
            for (var i = 0; i < list.length; i++)
                this._listItem(list[i], "add");
            this._renderBoxModel();
            if (this._isPopupShown()) {
                var scrollerPosition = this.scrollerObj ? this.scrollerObj.scrollTop() : 0;
                this._refreshScroller();
                if (this.scrollerObj) this.scrollerObj.option("scrollTop", scrollerPosition);
            }
        },

        _toArray: function (index, mode) {
            var items;
            if (typeof index == "function") index = ej.util.getVal(index);
            if ($.isArray(index)) items = index;
            else if (typeof index == "string") {
                if ((mode && (this.model.multiSelectMode == "visualmode" ||this.model.multiSelectMode == "delimiter" || this.model.showCheckbox))) {
                    items = index.split(this.model.delimiterChar);
                    if (items.length == 0) items = [index];                 
                }
                else if(!mode){
                    items = index.split(this.model.delimiterChar);
                    if (items.length == 0) items = [index];
                }
                else items = [index];
            } else items = [index];
            return items;
        },
        /*will deprecate with selectItemsByIndices */
        selectItemByIndex: function (index) {
            this._selectItemByIndex(index);
        },

        selectItemsByIndices: function (index) {
            this._selectItemByIndex(index);
        },

        _selectItemByIndex: function (val) {
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
            this._selectedIndices = $.map(this._selectedIndices, function (a) { return parseInt(a); });
            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
            var items = this._toArray(val,true), index;
            for (var k = 0; k < items.length; k++) {
                index = parseInt(items[k]);
                if (index != null && index >= 0) {
                    if ($.inArray(index, this._selectedIndices) == -1)
                        for (var i = 0; i < this.listitems.length; i++) {
                            if (!$(this.listitems[i]).hasClass('e-disable')) {
                                if (i == index) {
                                    this.selectedIndexValue = i;
                                    this._activeItem = index;
                                    this._enterTextBoxValue();
                                }
                            }
                        }
                } else if (!this.model.showCheckbox && this.model.multiSelectMode == "none") this._clearTextboxValue();
            }
        },

        unselectItemsByIndices: function (val) { this._unselectItemByIndex(val); },

        /*will deprecate with unselectItemsByIndices method */
        unselectItemByIndex: function (val) { this._unselectItemByIndex(val); },

        _unselectItemByIndex: function (val) {
            this._selectedIndices = $.map(this._selectedIndices, function (a) { return parseInt(a); });
            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
            var i, items = this._toArray(val,true), index;
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
            for (var k = 0; k < items.length; k++) {
                index = parseInt(items[k]);
                for (i = 0; i < this.listitems.length; i++) {
                    if (i == index) {
                        this.selectedIndexValue = i;
                        this._activeItem = index;
                        if (this._activeItem == this._aselectedItem) this._aselectedItem = null;
                        this._removeTextBoxValue();
                    }
                }
            }
        },

        /*Deprecated with selectedItemByValue */
        setSelectedValue: function (idvalue) { this.selectItemByValue(idvalue); },

        selectItemByValue: function (val) {
            var i, hidelement, items;
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
            if (ej.isNullOrUndefined(val) || val === "") this._clearTextboxValue();
            else {
                items = this._toArray(val,true);
                for (var k = 0; k < items.length; k++) {
                    for (i = 0; i < this.listitems.length; i++) {
                        if (!$(this.listitems[i]).hasClass('e-disable')) {
                            if (this._getAttributeValue(this.listitems[i]) || $(this.listitems[i]).text()) {
                                this._selectedValue = this._getAttributeValue(this.listitems[i]) || $(this.listitems[i]).text();
                                if (this._selectedValue == $.trim(items[k])) {
                                    this._activeItem = i;
                                    this._aselectedItem = this._activeItem;
                                    this._enterTextBoxValue();
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        },


        unselectItemByValue: function (val) {
            var i, hidelement, items = this._toArray(val,true);
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
            for (var k = 0; k < items.length; k++) {
                for (i = 0; i < this.listitems.length; i++) {
                    if (this._getAttributeValue(this.listitems[i])) {
                        this._selectedValue = this._getAttributeValue(this.listitems[i]);
                        if (this._selectedValue == $.trim(items[k])) {
                            this._activeItem = i;
                            if (this._activeItem == this._aselectedItem) this._aselectedItem = null;
                            this._removeTextBoxValue();
                            break;
                        }
                    }
                }
            }
        },

        /* Deprecated with selectItemByText*/
        setSelectedText: function (value) { this.selectItemByText(value); },

        selectItemByText: function (val) {
            var i, hidelement, items;
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
			this._mapFields();
            if (ej.isNullOrUndefined(val) || val === "") this._clearTextboxValue();
            else {
                items = this._toArray(val,true);
                for (var k = 0; k < items.length; k++) {
                    for (i = 0; i < this.listitems.length; i++) {
                        if(!$(this.listitems[i]).hasClass('e-disable')){
                            this.selectedTextValue =  this._isPlainType(this.popupListItems) ? this.popupListItems[i] : this._getField(this.popupListItems[i], this.mapFld._text);
                            if (this.selectedTextValue == $.trim(items[k])) {
                                this._activeItem = i;
                                this._aselectedItem = this._activeItem;
                                this._enterTextBoxValue();
                                break;
                            }
                        }
                    }
                }
            }
        },


        unselectItemByText: function (val) {
            var i, hidelement, items = this._toArray(val,true);
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
			this._mapFields();
            for (var k = 0; k < items.length; k++) {
                for (i = 0; i < this.listitems.length; i++) {
                    this.unselectedTextValue = this._isPlainType(this.popupListItems)? this.popupListItems[i] : this._getField(this.popupListItems[i], this.mapFld._text);
                    if (this.unselectedTextValue == $.trim(items[k])) {
                        this._activeItem = i;
                        if (this._activeItem == this._aselectedItem) this._aselectedItem = null;
                        this._removeTextBoxValue();
                        break;
                    }
                }
            }
        },

        getSelectedValue: function () {
            return this._hiddenInput.val();
        },

        getSelectedItem: function () {
            var k, selected = [];
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
            for (k = 0; k < this._selectedIndices.length; k++) {
                selected.push(this.listitems[this._selectedIndices[k]]);
            }
            return selected;
        },

        getListData: function() { return this._rawList; },

        /* will be deprecate in upcoming releases*/
        getSelectedItemsID: function () {
            return this._selectedItemsID;
        },

        disableItemsByIndices: function (value) { this._disableItemByIndex(value) },

        /*Deprecated with disableItemsByIndices method */
        disableItemByIndex: function (value) { this._disableItemByIndex(value) },

        _disableItemByIndex: function (value) {
            if (!this.model.enabled) return false;
            var listitems = this._toArray(value,false);
            for (var i = 0; i < listitems.length; i++) {
                if (listitems[i] != null) {
                    if (listitems.length > 0 && !($.inArray(parseInt(listitems[i]), this._disabledItems) > -1)) {
                        index = $.inArray(listitems[i], this._disabledItems);
                        var disable = $(this.ultag.children("li:not('.e-category')")[parseInt(listitems[i])]).addClass('e-disable');
                        disable.find(".listcheckbox").ejCheckBox("disable");
                        this._disabledItems.push(parseInt(listitems[i]));
                        this.model.disableItemsByIndex = String(this._disabledItems.join(","));
                    }
                }
            }
        },

        enableItemsByIndices:function(value){ this._enableItemByIndex(value)},

        /*Deprecated with enableItemsByIndices method */
        enableItemByIndex: function (value) { this._enableItemByIndex(value) },

        _enableItemByIndex: function (value) {
            var listitems = this._toArray(value,false), index;
            this.model.enableItemsByIndex = value;
            for (var i = 0; i < listitems.length; i++) {
                if (listitems.length > 0 && ($.inArray(parseInt(listitems[i]), this._disabledItems) > -1)) {
                    index = $.inArray(parseInt(listitems[i]), this._disabledItems);
                    var enable = $(this.ultag.children("li:not('.e-category')")[parseInt(listitems[i])]).removeClass('e-disable');
                    enable.find(".listcheckbox").ejCheckBox("enable");
                    this._disabledItems.splice(index, 1);
                }
            }
            this.model.enableItemsByIndex = null;
            this.model.disableItemsByIndex = this._disabledItems.join(this.model.delimiterChar);
        },
        _validateDelimiter: function (deli) {
            if ($.trim(deli).length == 1) {
                var RegEx = /^[a-zA-Z0-9]+$/;
                if (!RegEx.test(deli)) return deli;
            }
            return ",";
        },

        _removeText: function (currentValue) {
            var eleVal = this.element[0].value.split(this.model.delimiterChar), hidVal = this._hiddenInput[0].value.split(this.model.delimiterChar),
            index = $.inArray(currentValue, eleVal);
            if (index >= 0) {
                eleVal.splice(index, 1);
                hidVal.splice(index, 1);
            }
            this.element[0].value = eleVal.join(this.model.delimiterChar);
            this._hiddenInput[0].value = hidVal.join(this.model.delimiterChar);
        },
        _addText: function (currentValue) {
            if (this._checkContains(currentValue)) return false;
            var ele = ["element", "_hiddenInput"], val;
            for (var i = 0; i < ele.length; i++) {
                val = ele[i] == "element" ? currentValue : this._hiddenValue;
                if (this[ele[i]][0].value && this[ele[i]][0].value != "") {
                    var splitedText = this[ele[i]][0].value.split(this.model.delimiterChar);
                    splitedText.push(val);
                    this[ele[i]][0].value = splitedText.join(this.model.delimiterChar);
                } else this[ele[i]][0].value = val;
            }
        },
        _checkContains: function (chkValue) {
            var values = this.element[0].value.split(this.model.delimiterChar);
            this.contains = false;
            for (var i = 0; i < values.length; i++) {
                if (values[i] == chkValue)
                    this.contains = true;
            }
            return this.contains;
        },
        _init: function () {
            var browserInfo = ej.browserInfo();
            this._isIE8 = (browserInfo.name == 'msie' && browserInfo.version == '8.0') ? true : false;
            if ((this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) || this.element.is("select")) {
                this._isWatermark = 'placeholder' in document.createElement('input');
                this._id = this.element[0].id;
                this._initialize();
                this._render();
                this._addAttr(this.model.htmlAttributes);
                this._enabled(this.model.enabled);
                this._wireEvents();
                this._initValue = false;
                this._checkboxValue = false;
                if (this.model.validationRules != null) {
                    this._initValidator();
                    this._setValidation();
                }
            }
        },

        _initValidator: function () {
            var proxy = this;
            this.element.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $(error).insertAfter(element.closest(".e-widget"));
                },
            });
        },
        _setValidation: function () {
            if (this.element.closest("form").length != 0) {
                this._hiddenInput.rules("add", this.model.validationRules);
                var validator = this.element.closest("form").validate();
                var name = this._hiddenInput.attr("name");
                validator.settings.messages[name] = {};
                for (var ruleName in this.model.validationRules) {
                    var message = null;
                    if (!ej.isNullOrUndefined(this.model.validationRules[ruleName])) {
                        if (!ej.isNullOrUndefined(this.model.validationRules["messages"] && this.model.validationRules["messages"][ruleName]))
                            message = this.model.validationRules["messages"][ruleName];
                        else {
                            validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                            for (var msgName in this.model.validationMessage)
                                ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                        }
                        validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                    }
                }
            }
        },

        _setInitialPopup: function (value) {
            if(this.model.enabled && !this.model.readOnly)
                value == false ? this._hideResult() : this._showResult();
        },
        _changeSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin);
            this.popupListWrapper.removeClass(this.model.cssClass).addClass(skin);
        },

        _setRTL: function (val) {
            if (this.model.enableRTL != val) {
                this.model.enableRTL = val;
                this._RightToLeft();
                this._dropbtnRTL();
            }
        },

        _changeHeight: function (height) {
            this.wrapper.height(height);
            this._setListHeight();
        },
        _changeWidth: function (width) {
            this.wrapper.width(width);
            this._setListWidth();
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "dataSource": if (!this._isEqualDataSource(options[option])) this._checkModelDataBinding(options[option], this.model.query); break;
                    case "query": this._checkModelDataBinding(this.model.dataSource, options[option]); break;
                    case "fields": this.model.fields = options[option]; this._checkModelDataBinding(this.model.dataSource, this.model.query); break;
                    case "itemsCount": this.model.itemsCount = options[option]; this._checkModelDataBinding(this.model.dataSource, this.model.query);; break;
                    case "template": this.model.template = options[option]; this._checkModelDataBinding(this.model.dataSource, this.model.query); break;
                    case "value": if (ej.isNullOrUndefined(options[option]) || options[option] === "") this._clearTextboxValue();
                    else { this._setValue(ej.util.getVal(options[option])); options[option] = this.model.value; } break;
                    case "delimiterChar": var delchar = this.model.delimiterChar; options[option] = this._validateDelimiter(options[option]);
                        this.model.delimiterChar = options[option]; this.element.val(this.model.text.split(delchar).join(this.model.delimiterChar));
                        this.model.text = this.element.val();
                        break;
                    case "text": if (ej.isNullOrUndefined(options[option]) || options[option] === "") this._clearTextboxValue();
                    else { this._setText(options[option]); options[option] = this.model.text; } break;
                    case "itemValue": this._setItemValue(options[option]); break;
                    case "enableRTL": this._setRTL(options[option]); break;
                    case "enabled": this._enabled(options[option]); break;
                    case "height": this._changeHeight(options[option]); break;
                    case "width": this._changeWidth(options[option]); break;
                    case "popupHeight": this.model.popupHeight = options[option]; this._setListHeight(options[option]); break;
                    case "popupWidth": this.model.popupWidth = options[option]; this._setListWidth(); break;
                    case "minPopupHeight": this.model.minPopupHeight = options[option]; this._setListHeight(options[option]); break;
                    case "minPopupWidth": this.model.minPopupWidth = options[option]; this._setListWidth(); break;
                    case "maxPopupHeight": this.model.maxPopupHeight = options[option]; this._setListHeight(options[option]); break;
                    case "maxPopupWidth": this.model.maxPopupWidth = options[option]; this._setListWidth(); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showCheckbox": this.model.showCheckbox = options[option];
                        var _text = this.model.text;
                        var _textes = this.model.text.split(this.model.delimiterChar);
                        this._clearTextboxValue();
                        this._checkboxHideShow(options[option]);
                        if (this.model.showCheckbox || (this.model.multiSelectMode != "none")) {
                            if (this.model.multiSelectMode == "visualmode")
                                this._renderBoxModel();
                            this._setText(_text);
                        }
                        else
                            this._setText(_textes[0]);
                        if (this._isPopupShown())
                            this._setListPosition();
                        break;
                        /* will depreciate with checkAll  Method*/
                    case "checkAll": this._setCheckAll(options[option]); return false; break;
                        /* will depreciate with uncheckAll  Method*/
                    case "uncheckAll": this._setUncheckAll(options[option]); return false; break;
                    case "watermarkText": this._changeWatermark(options[option]); break;
                    case "validationRules":
                        if (this.element.closest("form").length != 0) {
                            if (this.model.validationRules != null) {
                                this._hiddenInput.rules('remove');
                                this.model.validationMessage = null;
                            }
                            this.model.validationRules = options[option];
                            this._setValidation();
                        }
                        break;
                    case "validationMessage":
                        if (this.element.closest("form").length != 0) {
                            this.model.validationMessage = options[option];
                            this._setValidation();
                        }
                        break;
                    case "showRoundedCorner": this._roundedCorner(options[option]); this.model.showRoundedCorner = options[option]; break;
                    case "showPopupOnLoad": this._setInitialPopup(options[option]); break;
                    case "targetID": this.model.targetID = options[option]; this._showFullList(); break;
                        /* will depreciate with selectedIndex  Method*/
                    case "selectedItemIndex":
                    case "selectedIndex":
                        this._selectItemByIndex(options[option]);
                        this.model.selectedItemIndex = this.model.selectedIndex = options[option];
                        break;
                        /* will depreciate with unselectItemByIndex API Method*/
                    case "unselectItemByIndex": this._unselectItemByIndex(options[option]); break;
                        /* will depreciate with disableItemsByIndex API Method*/
                    case "disableItemsByIndex": this._disableItemByIndex(options[option]); break;
                        /* will depreciate with enableItemsByIndex API Method*/
                    case "enableItemsByIndex": this._enableItemByIndex(options[option]); break;
                        /* will depreciate with selectedIndices  Method*/
                    case "selectedItems":
                    case "selectedIndices":
                        this._selectCheckedItem(options[option]);
                        this.model.selectedItems = this.model.selectedIndices = this._selectedIndices; break;
                    case "multiSelectMode":
                        if (this.model.multiSelectMode == "visualmode") {
                            this._swapUlandInput(false);
                            this._ulBox.remove();                         
                        }
                        this.model.multiSelectMode = options[option];
                        var _text = this.model.text;
                        var _textes = this.model.text.split(this.model.delimiterChar);
                        this._clearTextboxValue();
                        if (this.model.showCheckbox || (this.model.multiSelectMode != "none")) {
                            if (this.model.multiSelectMode == "visualmode")
                                this._renderBoxModel();
                            this._setText(_text);
                        }
                        else{ this._setText(_textes[0]);
                        this.wrapper.find("input[name=" + this._name + "]").remove();
                            }
                        if (this._isPopupShown())
                            this._setListPosition();
                        break;
                        /* will deprecate with fields.groupBy and e-category element in target element binding */
                    case "allowGrouping":
                        this._setGroupingAndSorting("allowGrouping", options[option]);
                        break;
                    case "enableSorting":
                        this._setGroupingAndSorting("enableSorting", options[option]);
                        break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                    case "enablePopupResize": this.model.enablePopupResize = options[option];
                        (options[option]) ? this._enablePopupResize() : this.popupListWrapper.find(".e-resize-handle").remove();
                        break;
                    case "enableFilterSearch":
                        if (!options[option]) this._removeSearch();
                        else {
                            this.model.enableFilterSearch = true;
                            this._enableSearch();
                            break;
                        }

                }
            }
        },

        _clearTextboxValue: function () {
            this.element.val("");
            this._hiddenInput.val("");
            this.value("");
            this.model.text = this.selectedTextValue = this._selectedValue = this._hiddenValue = "";
            this.selectedIndexValue = this._hiddenDelimiterIndex = this._activeItem = -1;
            this._selectedItemsID = []; this._selectedIndices = [];
            this.model.selectedItems = [];
            this.selectedIndex(null);
            this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
            if (this.model.showCheckbox) this.ultag.children("li").find(".listcheckbox").ejCheckBox({ "checked": false });
            if (this.wrapper.find('ul.e-ul.e-boxes').length != 0) {
                this._ulBox.children("li").remove();
                this.wrapper.find("input:hidden[id^='#'][name=" + this._id + "]").remove();
                $(this._hiddenInput).attr("name", this._name);
            }
        },

        _destroy: function () {
            this._destroyDropdown();
            this.element.removeClass("e-input " + this.model.cssClass);
            this.element.attr({ 'accesskey': this.wrapper.attr('accesskey'), 'name': this._hiddenInput.attr('name') });
            if (this._isWatermark) this.element.removeAttr("placeholder");
            this.element[0].value = "";
            if (!this.model.dataSource) {
                this.selectOptions ? this.docbdy && this.docbdy.append(this.selectOptions) && this.element.remove() && this.selectOptions && this.selectOptions.removeClass("e-dropdownlist e-js") : this.docbdy && this.docbdy.append(this.ultag.html());
            }
            this._hideResult();
            this.popupPanelWrapper.remove();
        },


        _finalize: function () {
            this.selectedIndex((this.selectedIndex()!=null) ? this.selectedIndex() : this.selectedItemIndex());
            if (this.selectedIndex() != null) {
                this._selectItemByIndex(this.selectedIndex());
            } else if (this._selectedIndices.length > 0) {
                this._selectCheckedItem(this._selectedIndices);
            } if (this.model.disableItemsByIndex != null)
                this._disableItemByIndex(this.model.disableItemsByIndex);
            if (this.model.enableItemsByIndex != null)
                this._enableItemByIndex(this.model.enableItemsByIndex);
        },


        _initialize: function () {
            this._selectedIndices = this.model.selectedIndices.length > 0 ? this.model.selectedIndices : this.model.selectedItems;
            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
            this.model.selectedIndex = this.model.selectedIndex !=null ? this.model.selectedIndex : this.model.selectedItemIndex;
            if (this.element.is("select")) {
                this.selectelement = true;
                this._renderSelectToDropdown();
            }
            this._selectedItemsID = [];
            this.target = this.element[0];
            this._disabledItems = new Array();
            this._queryString = null;
            this.suggLen = 0;
            this._itemId = null;
            this.checkedStatus = false;
            this._incqueryString = "";
            this._activeItem = null;
            this.ddWidth = 0;
            this._initValue = true;
            this._virtualCount = 0;
        },
        _renderSelectToDropdown: function () {
            var i, optionLength, optionText, item;
            this.inputElement = ej.buildTag("input.e-dropdownlist#" + this._id + "_input", "", {}, { "type": "text", "data-role": "none" });
            this.inputElement.insertAfter(this.element);
            this.optionDiv = ej.buildTag("div#" + this._id + "_list");
            this.optionDiv.insertAfter(this.inputElement);
            this.optionUl = ej.buildTag("ul");
            this.optionDiv.append(this.optionUl);
            if (this.element.attr("name")) {
                this.inputElement.attr("name", this.element.attr("name"));
                this.element.removeAttr("name");
            }
            this.selectOptions = this.element;
            this.selectOptions.attr('id', this._id);
            this.selectOptionItems = this.element.children("option");
            optionLength = this.selectOptionItems.length;
            this.optionDummyUl = $();

            for (i = 0; i < optionLength; i++) {
                item = this.selectOptionItems[i], optionText = $(item).attr('label') ? $(item).attr('label') : item.innerHTML;
                if (optionText != null) {
                    this.optionLi = ej.buildTag("li", optionText, {}, { 'data-value': item.value, "unselectable": "on" });
                    this.optionDummyUl.push(this.optionLi[0]);
                    if ($(item).attr("selected")) {
                        if (this.model.showCheckbox)
                            if($.inArray(i, this._selectedIndices) == -1){
                                this._selectedIndices.push(i);
                                this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                            }
                        else if (ej.isNullOrUndefined(this.selectedIndex())) this.selectedIndex(i);
                    }
                }
            }
            this.optionUl.append(this.optionDummyUl);

            this.element.css('display', 'none');
            this.element = this.inputElement;
        },


        _render: function () {
            this._renderDropdown();
            this._setWatermark();
            this._renderPopupPanelWrapper();
            this._showFullList();
            if (this.model.text)
                this.selectItemByText(this.model.text);
            if (this.value() == "" && this.element[0].value !== "")
                this.value(this.element[0].value);
            this._setValue(this.value());
            this._roundedCorner(this.model.showRoundedCorner);
            if (!(ej.DataManager && this.model.dataSource instanceof ej.DataManager))
                this._finalize();
            this._setCheckAll(this.model.checkAll);
        },

        _isEqualDataSource: function (source) {
            if (!this.model.dataSource || !source || !(this.model.dataSource.length === source.length)) return false;
            var equal = true;
            for (var i = 0, len = this.model.dataSource.length; i < len; i++) {
                if (this.model.dataSource[i] !== source[i]) {
                    equal = false;
                    break;
                }
            }
            return equal;
        },

        _checkModelDataBinding: function (source, query) {
            this.element.val("");
            this._hiddenInput.val("");
            this.value("");
            this.model.text = this.selectedTextValue = this._selectedValue = this._hiddenValue = "";
            this.selectedIndexValue = this._hiddenDelimiterIndex = this._activeItem = -1;
            this._selectedItemsID = [];
            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices= [];
            this.model.selectedIndex = this.model.selectedItemIndex = null;
            if (this.model.multiSelectMode == "visualmode") this._destroyBoxModel();
            this.model.dataSource = source;
            this.model.query = query;
            this.ultag.empty();
            this._showFullList();
        },
        _initDataSource: function (source) {
            var proxy = this;
            if (ej.DataManager && source instanceof ej.DataManager) {
                proxy._addLoadingClass();
                var queryPromise = source.executeQuery(this._getQuery());
                queryPromise.done(function (e) {
                    proxy._trigger("actionSuccess", e);
                    proxy._totalCount = e.count;
                    proxy._listItem(e.result);
                    proxy._removeLoadingClass();
                    proxy._renderPopupList();
                    proxy._finalize();

                }).fail(function (e) {
                    proxy.model.dataSource = null;
                    proxy._addLoadingClass();
                    proxy._trigger("actionFailure", e);
                }).always(function (e) {
                    proxy._trigger("actionComplete", e);
                });
            }
        },
        _listItem: function (list, type) {
            if (type == "add") {
                this.popupListItems.push(list);
                this._rawList.push(list);
            } else if ($.isArray(list)) {
                this.popupListItems = list.slice(0);
                this._rawList = list.slice(0);
            }
        },
        _getQuery: function (isLocal) {
            var remoteUrl, mapper = this.model.fields, queryManager = ej.Query();
            if (ej.isNullOrUndefined(this.model.query) && !this.model.template && !isLocal) {
                var column = [];
                for (var col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
            }
            else if (this.model.query) queryManager = this.model.query.clone();

            if (this.model.allowVirtualScrolling) if (this.model.virtualScrollMode == "normal") queryManager.requiresCount();
            if (this.model.itemsCount > 0) queryManager.take(this.model.itemsCount);

            remoteUrl = this.model.dataSource.dataSource;
            if ((remoteUrl && remoteUrl.url && !remoteUrl.url.match(mapper.tableName + "$") || (remoteUrl && !remoteUrl.url) || !remoteUrl))
                !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);

            return queryManager;
        },

        _addLoadingClass: function () {
            if (this._isPopupShown()) {
                this.popupListWrapper.addClass("e-load");
            } else {
                this.dropdownbutton.addClass("e-load");
                this.drpbtnspan.removeClass("e-icon e-down-arrow");
            }
            this.model.readOnly = true;
        },
        _removeLoadingClass: function () {
            this.dropdownbutton.removeClass("e-load");
            this.drpbtnspan.addClass("e-icon e-down-arrow");
            this.model.readOnly = false;
            this.popupListWrapper.removeClass("e-load");
        },

        _renderDropdown: function () {
            this.element.addClass("e-input " + this.model.cssClass);
            this.wrapper = ej.buildTag("span.e-ddl e-widget " + this.model.cssClass + "#" + this._id + "_wrapper", "", {}, { "tabindex": "0", "accesskey": this.element.attr("accesskey") });
            this.container = ej.buildTag("span.e-in-wrap e-box " + this.model.cssClass + "#" + this._id + "_container");
            this.element.removeAttr('accesskey');
            this.element.attr({ "readonly": true, "tabindex": -1, "role": "combobox", "aria-expanded": false, "aria-autocomplete": "list", "aria-haspopup": true, "aria-owns": this._id + "_popup" });
            this.drpbtnspan = ej.buildTag("span.e-icon e-down-arrow", "", {}, { "aria-label": "select", "unselectable": "on" });
            this.dropdownbutton = ej.buildTag("span.e-select#" + this._id + "_dropdown", "", {}, { "role": "button", "unselectable": "on" }).append(this.drpbtnspan);
            this.container.insertAfter(this.element);
            this.container.append(this.element);
            this.container.append(this.dropdownbutton);
            this.wrapper.insertBefore(this.container);
            this.wrapper.append(this.container);
            if (this.selectelement) {
                this.selectOptions.insertBefore(this.element);
            }
            this._hiddenInput = ej.buildTag("input#" + this._id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
            if (!this._isWatermark) {
                this._hiddenSpan = ej.buildTag("span.e-input e-placeholder ").insertAfter(this.element);
                this._hiddenSpan.text(this.model.watermarkText);
                this._hiddenSpan.css("display", "none");
                this._hiddenSpan.bind("mousedown", $.proxy(this._OnDropdownClick, this));
            }
            this._checkNameAttr();
            this._setDimentions();
            this._RightToLeft();
            this.ddWidth = (this.dropdownbutton.outerWidth() > 0) ? this.dropdownbutton.outerWidth() : 24;
            //Element not Maintain in Multiselection
            this.container.bind("mousedown", $.proxy(this._OnDropdownClick, this));
        },

        _checkNameAttr: function () {
            this._name = ej.isNullOrUndefined(this.element.attr("name")) ? this._id : this.element.attr("name");
            this._hiddenInput.attr("name", this._name);
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "name") proxy._hiddenInput.attr(key, value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "readOnly" && value == "readOnly") proxy.model.readOnly = true;
                else proxy.wrapper.attr(key, value)
            });
        },
        _renderBoxModel: function () {
            if (this.model.multiSelectMode != "visualmode") return false;
            this._ulBox = ej.buildTag("ul.e-ul e-boxes");
            this.container.prepend(this._ulBox);
            this._ulBox.css('min-height', '30px');
            this._ulBox.css('display', 'none');
            this._on(this.container, "click", function (e) {
                if (!this.model.enabled) return false;
                var $target = $(e.target);
                if ($target.hasClass("e-options")) {
                    if (!e.ctrlKey && $target.siblings().hasClass("e-active")) this._removeActive();
                    if ($target.hasClass("e-active")) $target.removeClass("e-active");
                    else $target.addClass("e-active");
                }
                if (!e.ctrlKey && ($target.hasClass("e-boxes"))) this._removeActive();
            });
        },

        _renderPopupPanelWrapper: function () {
            var oldWrapper = $("#" + this.element.context.id + "_popup_wrapper").get(0);
            if (oldWrapper)
                $(oldWrapper).remove();
            this.popupPanelWrapper = ej.buildTag("div#" + this._id + "_popup_wrapper");
            $('body').append(this.popupPanelWrapper);
            this.popupListWrapper = ej.buildTag("div.e-ddl-popup e-box e-widget  e-popup#" + this._id + "_popup_list_wrapper", "", { display: "none", overflow: "hidden" });
            this.popupList = ej.buildTag("div#" + this._id + "_popup", { "tabIndex": 0 });
            this.popupList.addClass(this.model.cssClass);
            this.popup = this.popupList;
            this.popupScroller = ej.buildTag("div"); this.ultag = ej.buildTag("ul.e-ul", "", {}, { "role": "listbox" });
            this.popupScroller.append(this.ultag);
            this.popupList.append(this.popupScroller);
            this.popupListWrapper.append(this.popupList);
            this.popupPanelWrapper.append(this.popupListWrapper);
            this.ultag.on({ mouseenter: $.proxy(this._OnMouseEnter, this), mouseleave: $.proxy(this._OnMouseLeave, this), click: $.proxy(this._OnMouseClick, this) }, "li:not('.e-category')");
            $(window).on("resize", $.proxy(this._OnWindowResize, this));

        },

        _setGroupingAndSorting: function (prop, value) {
            this.model[prop] = value;
            var oldValue = this.model.text;
            this.value("");
            this._selectedIndices = [];
            this.ultag.empty();
            this._showFullList();
            if (this.model.showCheckbox) {
                var values = oldValue.split(this.model.delimiterChar);
                for (var i = 0; i < values.length; i++)
                    this.selectItemByText(values[i]);
            }
            else
                this.selectItemByText(oldValue);
        },
        _setSortingList: function () {
            var sortedlist = document.createElement("ul"), i, sortitems;
            $(sortedlist).append(this.itemsContainer.children());
            if (this.model.allowGrouping || $(sortedlist).find(">.e-category").length >0) {
                this.popupListWrapper.addClass("e-atc-popup");
                for (i = 0; i < $(sortedlist).find(">.e-category").length; i++) {
                    sortitems = $(sortedlist).find(">.e-category").eq(0).first().nextUntil(".e-category").get();
                    this._setSortList(sortedlist, sortitems);
                }
            }
            else {
                $(sortedlist).children('>.e-category').remove();
                sortitems = $(sortedlist).children('li').get();
                this._setSortList(sortedlist, sortitems);
            }
            this.itemsContainer = $(sortedlist);
        },
        _setSortList: function (sortedlist, sortitems) {
            sortitems.sort(function (objA, objB) {
                var sortA = $(objA).text().toUpperCase();
                var sortB = $(objB).text().toUpperCase();
                return (sortA < sortB) ? -1 : (sortA > sortB) ? 1 : 0;
            });
            if (this.model.sortOrder == "descending") sortitems.reverse();
            if (this.model.allowGrouping || $(sortedlist).find(">.e-category").length >0) {
                $(sortedlist).append($("<li>").text($(sortedlist).find(">.e-category").eq(0).text()).addClass("e-category"));
                $(sortedlist).find(">.e-category").eq(0).remove();
            }
            $.each(sortitems, function (index, item) {
                $(sortedlist).append(item);
            });
        },

        _renderPopupList: function () {
            this._doDataBind();
            this._renderRemaining();
        },
        _renderRemaining: function () {
            var proxy = this;
            this._dropbtnRTL();
            if (this.model.enableFilterSearch) this._enableSearch();
            if (this.model.enablePopupResize) this._enablePopupResize();
            if (this.model.allowVirtualScrolling && this.model.virtualScrollMode == "normal") {
                if (this._totalCount && this._totalCount > 0) {
                    this._totalHeight = this._totalCount * 29;
                    this._totalPages = this._totalCount / (this.model.itemsCount * 29);
                    this._loadedItemHeight = this.ultag.find("li:not('.e-category')").length * 29;
                    this.ultag.find("li:not('.e-category')").attr("page", 0);
                    this._virtualPages = [0];
                    this.ultag.append($("<span>").addClass("e-virtual").css({ "height": this._totalHeight - this._loadedItemHeight, "display": "block" }));
                }
            }
            this._virtualUl = this.ultag.clone(true);
            this._setListWidth();
            this._setListHeight();
            this._setListPosition();
            this.popupScroller.css({ "height": "", "width": "" });
            this.popupList.ejScroller({
                height: this._getPopupHeight(), width: 0, scrollerSize: 20, scroll: function (e) {
                    if (proxy.model.allowVirtualScrolling) proxy._onScroll(e);
                }
            });
            this.scrollerObj = this.popupList.ejScroller("instance");
            this.popupList.find("div.e-scrollbar div").attr("unselectable", "on");
            this.popupListWrapper.css({ 'display': 'none', 'visibility': 'visible' });
            if (this.model.showCheckbox) this._checkboxHideShow(this.model.showCheckbox);
            this._renderBoxModel();
            this._setUncheckAll(this.model.uncheckAll);
            this._changeSkin(this.model.cssClass);
            this.model.showPopupOnLoad && this._showResult();
           
        },

        _enableSearch: function () {
            if (this.model.enableFilterSearch)
                if (!this.inputSearch) {
                    this.inputSearch = ej.buildTag("input#" + this._id + "_inputSearch.e-input", "", {}, { "type": "text", "data-role": "none" });
                    this.popupListWrapper.prepend($("<span>").addClass("e-atc e-search").append($("<span>").addClass("e-in-wrap ").append(this.inputSearch).append($("<span>").addClass(" e-icon e-isearch"))));
                    this._on(this.inputSearch, "keyup", this._OnSearchEnter);
                    this._on(this.popupListWrapper.find(".e-isearch"), "mousedown", this._OnSearchEnter);
                }
        },
        _removeSearch: function () {
            this.model.enableFilterSearch = false;
            this.popupListWrapper.find(".e-atc.e-search").remove();
            if (this._isPopupShown()) this.hidePopup();
			this.inputSearch = null;
        },

        _OnSearchEnter: function (e) {
            var proxy = this, keycode = ej.browserInfo().name == "mozilla" ? e.charCode : e.keyCode;
            if ($.inArray(keycode, [38, 40]) != -1) {
                this._OnKeyDown(e);
            }
            else if ($.inArray(keycode, [13]) != -1 && this.popupList.find("li").length > 0) {
                this._OnKeyUp(e);
            }
            else {
                this._queryString = this.inputSearch.val();
                if (this._queryString == "" && this._virtualUl ) {
                    this._resetList();
                    this._updateSelectedIndexByValue(this.model.value);
                    this._refreshScroller();
                    this._setListPosition();
                } else {
                    this._mapFields();
                    var searchQuery = this._addSearchQuery(ej.Query(), !this._isPlainType(this._rawList));
                    var args = { searchString: this._queryString, items: this._rawList };
                    this._trigger("search", args) ?
                        window.setTimeout(function() {
                            proxy._filterSearch(searchQuery, args);
                        }, 1000) : proxy._filterSearch(searchQuery, args);
                }
            }
        },
        _filterSearch: function (searchQuery, args) {
            var flag =false, resultList = args.result ? args.result : ej.DataManager(this._rawList).executeLocal(searchQuery);
            if (resultList.length == 0) {
                flag = true;
                resultList.push("No suggestions");
            }
            this.popupListItems = resultList;
            this.ultag.empty();
            this._isPlainType(this.popupListItems) ? this._plainArrayTypeBinding(resultList) :
            this._objectArrayTypeBinding(resultList, "search");
            if (flag && this.ultag.find("li").length==1 ) {
                this.ultag.find("li").eq(0).addClass("e-nosuggestion");
            }
            if (this.model.showCheckbox && !flag) {
                this._addCheckbox(this.ultag.children("li:not('.e-category')"));
            }
            this._onSearch = true;
            this._setValue(this.model.value);
            this._onSearch = false;
            this._updateSelectedIndexByValue(this.model.value);
            this._refreshScroller();
            this._setListPosition();
        },
        _updateSelectedIndexByValue: function (value) {
            if (!value || !this.model.enableFilterSearch) return;
            this._selectedIndices = this.model.selectedItems = this.model.selectedIndices = [];
            this._virtualList = this._virtualUl.children("li:not('.e-category')");
            var item = this._toArray(value);
            for (var k = 0; k < item.length; k++) {
                for (var m = 0; m < this._virtualList.length; m++) {
                    if ($.trim(item[k]) == ($.trim(this._getIndexedValue(this._virtualList[m])))) {
                        this._selectedIndices.push(m);
                        break;
                    }
                }
            }
            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
        },
        _getIndexedValue: function(item) {
            return this._getAttributeValue(item) ? this._getAttributeValue(item) : item.textContent;
        },

        _resetSearch: function() {
            if (!this.inputSearch || !this.model.enableFilterSearch) return;
            if (this.inputSearch.val() != "" && this._virtualUl) {
                this.inputSearch.val("");
                this._resetList();
            }
        },
        _resetList: function () {
            this._listItem(this._rawList);
            if (this.model.allowVirtualScrolling && this.model.virtualScrollMode == "normal") {
                this._removeCheckbox($("<span>").append(this._virtualUl));
                this._virtualUl.find('.e-chkbox-wrap').remove();
            }
            this.ultag.empty().append(this._virtualUl.children().clone(true));
            if (this.model.showCheckbox) {
                this._addCheckbox(this.ultag.children("li:not('.e-category')"));
            }
            // this_onSearch variable will restrict the change and select event on searching
            this._onSearch = true;
            this._setValue(this.model.value);
            this._onSearch = false;
        },

       
        _addSearchQuery: function (query, checkMapper) {
            var bindTo = "";
            if (checkMapper) {
                var mapper = this.model.fields;
                bindTo = (mapper && mapper.text) ? mapper["text"] : "text";
            }
            if (this._queryString) query.where(bindTo, this.model.filterType, this._queryString, !this.model.caseSensitiveSearch);
            if (this.model.itemsCount > 0) query.take(this.model.itemsCount);
            return query;
        },

        _targetElementBinding: function () {
            var predecessor = this.element.parents().last();
            this.docbdy = this.model.targetID ? predecessor.find("#" + this.model.targetID) : this.optionDiv ? this.optionDiv : null;
            if (!this.docbdy) return false;
            this.itemsContainer = this.docbdy[0].nodeName == "UL" ? this.docbdy : this.docbdy.children("ol,ul");
            if ((this.model.allowGrouping || this.itemsContainer.find(">.e-category").length >0 ) && !this.model.enableSorting) {
                this.popupListWrapper.addClass("e-atc-popup");
                for (var k = 0; k < this.itemsContainer.find(">.e-category").length; k++) {
                    var ele = this.itemsContainer.find(">.e-category").eq(k);
                    ele.replaceWith($("<li>").text(ele.text()).addClass("e-category"));
                }
            }
            else if (this.model.enableSorting) this._setSortingList();
            this.itemsContainer.children("ol,ul").remove();
            this.items = this.itemsContainer.children('li');
            this.items.children("img,div").addClass("e-align");
            this._listItem([]);
            for (var i = 0; i < this.items.length; i++)
                this._listItem({ text: $(this.items[i]).text() }, "add");
            //This will append the list with the popup wrapper
            this.ultag.empty().append(this.itemsContainer.children());
            this.ultag.children('li').attr({ "role": "option", "unselectable": "on" });
            this.docbdy.css({ 'display': 'none' }).children("ol,ul").remove();
        },
        _plainArrayTypeBinding: function (list) {
            this.dummyUl = $();
            if (this.model.enableSorting) {
                list.sort();
                if (this.model.sortOrder == "descending") list.reverse();
            }
            if (list.length > 0) {
                for (var i = 0; i < list.length; i++) {
                    if (!ej.isNullOrUndefined(list[i])) {
                        var litag = ej.buildTag("li", list[i], {}, { 'data-value': list[i], "unselectable": "on" });
                        this.dummyUl.push(litag[0]);
                    }
                }
                this.ultag.append(this.dummyUl);
                this._trigger('dataBound', { data: list });
            }

        },
        _mapFields: function () {
            this.model.fields.groupBy = this.model.fields.groupBy ? this.model.fields.groupBy : this.model.fields.category;
            var mapper = this.model.fields;
            this.mapFld = { _id: null, _imageUrl: null, _imageAttributes: null, _spriteCSS: null, _text: null, _value: null, _htmlAttributes: null, _selected: null };
            this.mapFld._id = (mapper && mapper.id) ? mapper["id"] : "id";
            this.mapFld._imageUrl = (mapper && mapper.imageUrl) ? mapper["imageUrl"] : "imageUrl";
            this.mapFld._imageAttributes = (mapper && mapper.imageAttributes) ? mapper["imageAttributes"] : "imageAttributes";
            this.mapFld._spriteCSS = (mapper && mapper.spriteCssClass) ? mapper["spriteCssClass"] : "spriteCssClass";
            this.mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            this.mapFld._value = (mapper && mapper.value) ? mapper["value"] : "value";
            this.mapFld._htmlAttributes = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
            this.mapFld._selected = (mapper && mapper.selected) ? mapper["selected"] : "selected";
            this.mapFld._category = (mapper && mapper.groupBy) ? mapper["groupBy"] : "groupBy";
        },
        _doDataBind: function () {
            var source = this.model.dataSource, list = this.popupListItems;
            !source || !list || !list.length || list.length < 1 ? this._targetElementBinding()
            : this._isPlainType(list) ? this._plainArrayTypeBinding(list)
            : this._objectArrayTypeBinding(list);
        },
        _isPlainType: function (list) {
            return typeof list[0] != "object";
        },
        _objectArrayTypeBinding: function (list, from) {
            this.dummyUl = $();
            this._mapFields();
            if (this.model.enableSorting) {
                var sortQuery = ej.Query().sortBy(this.mapFld._text, this.model.sortOrder, true);
                list = ej.DataManager(list).executeLocal(sortQuery);
            }
            if (this.model.allowGrouping || this.model.fields.groupBy) {
                this.popupListWrapper.addClass("e-atc-popup");
                var mapCateg = this.mapFld._category, groupedList, groupQuery;
                groupQuery = ej.Query().group(mapCateg);
                if (!this.model.enableSorting) groupQuery.queries.splice(0, 1);
                groupedList = ej.DataManager(list).executeLocal(groupQuery);
                this._swapUnCategorized(groupedList);
                (from == "search") ? this.popupListItems = [] : this._listItem([]);
                for (var i = 0; i < groupedList.length; i++) {
                    if (groupedList[i].key)
                        this.ultag.append(ej.buildTag("li.e-category", groupedList[i].key).attr("role", "option")[0]);
                    this._generateLi(groupedList[i].items, this.mapFld);
                    this.ultag.append(this.dummyUl);
                    for (var j = 0; j < groupedList[i].items.length; j++) {
                        (from == "search") ? this.popupListItems.push(groupedList[i].items[j]) : this._listItem(groupedList[i].items[j], "add");
                    }
                }
            }
            else {
                this._generateLi(list, this.mapFld);
                this.ultag.append(this.dummyUl);//ko binding
            }
            this._trigger('dataBound', { data: list });
        },
        _onScroll: function (e) {
            if (!e.scrollTop) return;
            var scrollerPositon = e.scrollTop, proxy = this;
            if (proxy.model.allowVirtualScrolling && proxy.model.virtualScrollMode == "continuous") {
                var list, queryPromise, skipQuery = ej.Query().skip(proxy._rawList.length).take(proxy.model.itemsCount).clone();
                if (scrollerPositon >= $(proxy.popupList).find("ul,ol").height() - $(proxy.popupList).height()) {
                    proxy._addLoadingClass();
                    if (ej.DataManager && proxy.model.dataSource instanceof ej.DataManager) {
                        skipQuery = proxy._getQuery().skip(proxy._rawList.length).take(proxy.model.itemsCount).clone();
                        queryPromise = proxy.model.dataSource.executeQuery(skipQuery);
                        queryPromise.done(function (e) {
                            proxy.addItem(e.result);
                            proxy._removeLoadingClass();
                            proxy._trigger("actionSuccess", e);
                        }).fail(function () {
                            proxy.model.dataSource = null;
                            proxy._removeLoadingClass();
                            proxy._trigger("actionFailure", e);
                        }).always(function (e) {
                            proxy._trigger("actionComplete", e);
                        });
                    } else {
                        list = ej.DataManager(proxy.model.dataSource).executeLocal(skipQuery);
                        proxy.addItem(list);
                        proxy._removeLoadingClass();
                    }
                }
            } else if (proxy.model.allowVirtualScrolling && proxy.model.virtualScrollMode == "normal") {

                window.setTimeout(function () {
                    if (proxy._virtualCount == 0) {
                        proxy._loadList();
                    }
                }, 300);

            }

        },
        _loadList: function () {
            this._virtualCount++;
            var top = this.scrollerObj.scrollTop(), proxy = this, prevIndex = 0, prevPageLoad, nextIndex = null;
            this._currentPage = Math.round(top / (29 * this.model.itemsCount));
            if (($.inArray(this._currentPage, this._virtualPages.sort(function (a, b) { return a - b; }))) != -1) {
                if (this._currentPage == 0) {
                    if (($.inArray(this._currentPage + 1, this._virtualPages)) != -1) {
                        this._virtualCount--;
                        return false;
                    } else {
                        this._currentPage = this._currentPage + 1;
                    }
                }
                else if (($.inArray(this._currentPage - 1, this._virtualPages)) != -1) {
                    if (($.inArray(this._currentPage + 1, this._virtualPages)) != -1) {
                        this._virtualCount--;
                        return false;
                    } else {
                        this._currentPage = this._currentPage + 1;
                    }
                }
                else {
                    this._currentPage = this._currentPage - 1;
                }
            }
            prevPageLoad = !($.inArray(this._currentPage - 1, this._virtualPages) != -1);
            this._addLoadingClass();
            for (var i = this._virtualPages.length - 1; i >= 0; i--) {
                if (this._virtualPages[i] < this._currentPage) {
                    prevIndex = this._virtualPages[i];
                    if (!(i + 1 == this._virtualPages.length))
                        nextIndex = this._virtualPages[i + 1];
                    break;
                }
            }

            var firstArg = prevPageLoad ? (this._currentPage - 1) * this.model.itemsCount : this._currentPage * this.model.itemsCount;
            var skipQuery = ej.Query().range(firstArg, this._currentPage * this.model.itemsCount + this.model.itemsCount), queryPromise, list;
            if (ej.DataManager && proxy.model.dataSource instanceof ej.DataManager) {
                var skipParam = prevPageLoad ? (this._currentPage - 1) * this.model.itemsCount : this._currentPage * this.model.itemsCount;
                skipQuery = this._getQuery().skip(skipParam);
                if (prevPageLoad) {
                    for (i = 0; i < skipQuery.queries.length; i++) {
                        if (skipQuery.queries[i].fn == "onTake") {
                            skipQuery.queries.splice(i, 1);
                            break;
                        }
                    }
                    skipQuery.take(2 * this.model.itemsCount);
                }
                queryPromise = proxy.model.dataSource.executeQuery(skipQuery);
                queryPromise.done(function(e) {
                    proxy._appendVirtualList(e.result, prevIndex, proxy._currentPage, nextIndex, prevPageLoad);
                    proxy._removeLoadingClass();
                    proxy._trigger("actionSuccess", e);
                }).fail(function() {
                    proxy._virtualCount--;
                    proxy._removeLoadingClass();
                    proxy._trigger("actionFailure", e);
                }).always(function(e) {
                    proxy._trigger("actionComplete", e);
                });
            } else {

                list = ej.DataManager(proxy.model.dataSource).executeLocal(skipQuery);
                this._appendVirtualList(list, prevIndex, this._currentPage, nextIndex, prevPageLoad);
                proxy._removeLoadingClass();
            }
        },
        _appendVirtualList: function (list, prevIndex, currentIndex, nextIndex, prevPageLoad) {
            this._virtualCount--;
            if (($.inArray(currentIndex, this._virtualPages.sort(function (a, b) { return a - b; }))) != -1) return false;
            if (prevPageLoad && ($.inArray(currentIndex - 1, this._virtualPages.sort()) != -1)) {
                list.splice(0, this.model.itemsCount);
                prevPageLoad = false;
            }
            var items = this.model.itemsCount, tempUl = $("<ul>"), firstVirtualHeight, secondVirtualHeight;
            firstVirtualHeight = prevPageLoad ? ((currentIndex - 1) * items * 29) - (prevIndex * items + items) * 29 : (currentIndex * items * 29) - (prevIndex * items + items) * 29;
            if (firstVirtualHeight != 0) tempUl.append($("<span>").addClass("e-virtual").css({ display: "block", height: firstVirtualHeight }));
            this._mapFields();
            this._generateLi(list, this.mapFld);
            $(this.dummyUl).attr("page", currentIndex);
            if (prevPageLoad) {
                $(this.dummyUl).slice(0, items).attr("page", currentIndex - 1);
            }
            if (this.model.showCheckbox) {
                this._addCheckbox(this.dummyUl);
            }
            tempUl.append(this.dummyUl);
            secondVirtualHeight = (currentIndex * items + items) * 29;
            if (nextIndex != null) secondVirtualHeight = (nextIndex * items * 29) - secondVirtualHeight;
            else secondVirtualHeight = this.ultag.height() - secondVirtualHeight;
            if (secondVirtualHeight != 0) tempUl.append($("<span>").addClass("e-virtual").css({ display: "block", height: secondVirtualHeight }));
            var selector = this.ultag.find("li[page=" + prevIndex + "]").last();
            selector.next().remove();
            tempUl.children().insertAfter(selector);
            this._virtualPages.push(currentIndex);
            if (prevPageLoad) this._virtualPages.push(currentIndex - 1);
            for (var i = 0; i < list.length; i++) {
                this._listItem(list[i], "add");
            }
            this._virtualUl = this.ultag.clone(true);
            this._renderBoxModel();
        },

        _generateLi: function (list, mapFld) {
            this.mapFld = mapFld;
            this.dummyUl = [];
            if (!list || !list.length || list.length < 1) return false;
            for (var i = 0; i < list.length; i++) {
                var _did = this._getField(list[i], this.mapFld._id),
                    _dimageUrl = this._getField(list[i], this.mapFld._imageUrl),
                    _dimageAttributes = this._getField(list[i], this.mapFld._imageAttributes),
                    _dspriteCss = this._getField(list[i], this.mapFld._spriteCSS),
                    _dtext = this._getField(list[i], this.mapFld._text),
                    _dvalue = this._getField(list[i], this.mapFld._value),
                    _dhtmlAttributes = this._getField(list[i], this.mapFld._htmlAttributes),
                    _dselected = this._getField(list[i], this.mapFld._selected),
                    litag = document.createElement("li");

                if (!ej.isNullOrUndefined(_dvalue) && _dvalue.toString().length > 0) {
                    litag.setAttribute('data-value', typeof _dvalue == "object" ? JSON.stringify(_dvalue) : _dvalue);
                }
                if (_did)
                    litag.setAttribute('id', _did);
                if ((_dhtmlAttributes) && (_dhtmlAttributes != "")) {
                    this._setAttr(litag, _dhtmlAttributes);
                }

                if (this.model.template) {
                    $(litag).append(this._getTemplatedString(list[i]));
                } else {
                    if ((_dimageUrl) && (_dimageUrl != "")) {
                        var imgtag = document.createElement("img");
                        this._setClass(imgtag, "e-align")._setAttr(imgtag, { 'src': _dimageUrl, 'alt': _dtext });
                        if ((_dimageAttributes) && (_dimageAttributes != "")) {
                            this._setAttr(imgtag, _dimageAttributes);
                        }
                        litag.appendChild(imgtag);
                    }
                    if ((_dspriteCss) && (_dspriteCss != "")) {
                        var divtag = document.createElement("div");
                        this._setClass(divtag, 'div.e-align ' + _dspriteCss + ' sprite-image');
                        litag.appendChild(divtag);
                    }
                    if (_dselected) {
                        this._setClass(litag, "chkselect");
                    }
                    if ((_dtext) && (_dtext != "")) {
                        var flag = litag.innerHTML;
                        litag.textContent = _dtext;
                        litag.innerHTML = flag + litag.innerHTML;
                    }
                }
                this._setAttr(litag, { "role": "option", "unselectable": "on" });
                this.dummyUl.push(litag);
             
            }
        },

        _setAttr: function (element, attrs) {
            if (typeof attrs == "string") {
                var sAttr = attrs.replace(/['"]/g, "").split("=");
                if (sAttr.length == 2) element.setAttribute(sAttr[0], sAttr[1]);
            }
            else {
                for (var idx in attrs) {
                    if ((idx == 'styles' || idx == 'style') && typeof attrs[idx] == 'object') {
                        for (var prop in attrs[idx]) {
                            element.style[prop] = attrs[idx][prop];
                        }
                    }
                    else
                        element.setAttribute(idx, attrs[idx]);
                }
            }
            return this;
        },
        _setClass: function (element, newClass) {
            element.className += " " + newClass;
            return this;
        },
        _swapUnCategorized: function (list) {
            $(list).each(function (i, obj) {
                if (!obj.key) {
                    for (var j = i; j > 0; j--) {
                        list[j] = list[j - 1];
                    }
                    list[j] = obj;
                    return false;
                }
            });
        },

        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },

        _getTemplatedString: function (list) {

            var str = this.model.template, start = str.indexOf("${"), end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                str = str.replace(content, this._getField(list, field));
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },

        _setWatermark: function () {
            if ((this.model.watermarkText != null) && (this.element.val() == "") && $.trim(this._hiddenInput.val()) == "") {
                if (this._isWatermark)
                    this.element.attr("placeholder", this.model.watermarkText);
                else
                    this._hiddenSpan.css("display", "block").text(this.model.watermarkText);
                //In visual mode, to show watermark text when no items selected.
                if (this.model.multiSelectMode == "visualmode" && this._ulBox && this._ulBox.find('li').length == 0) this._swapUlandInput(false);
            }
        },

        _checkboxHideShow: function (value) {
            if (value) {
                this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
                var chklist = this.listitems.find('input[type=checkbox]');
                if (chklist.length == 0) {
                    this._createCheckbox(this.listitems);
                }
            }

            else
                this._removeCheckbox(this.popupList);
            this.model.showCheckbox = value;
        },
        _createCheckbox: function (listItems) {
            this._addCheckbox(listItems);
            this.element.val("");
            this._setWatermark();
            for (i = 0; i < listItems.length; i++) {
                if ($(listItems[i]).hasClass('chkselect')) {
                    $(listItems[i]).find(".listcheckbox").ejCheckBox({ "checked": true });
                    this._activeItem = i;
                    this.checkedStatus = true;
                    if (!this._initValue)
                        this._trigger('checkChange', { isChecked: this.checkedStatus,data:this.model });
                    this._enterTextBoxValue();
                    $(listItems[i]).removeClass('chkselect');
                }
            }
        },
        _addCheckbox: function (listItems) {
            for (var i = 0; i < listItems.length; i++) {
                var status =true, $checkbox = document.createElement("input");
                this._setAttr($checkbox, { type: "checkbox", name: "list" + i, "data-role": "none", id: this._id + "_" + "check" + i })._setClass($checkbox, "listcheckbox e-align");
                listItems[i].insertBefore($checkbox, listItems[i].childNodes[0]);
                if ($.inArray(i, this._disabledItems) != -1) status = false;
                $($checkbox).ejCheckBox({ cssClass: this.model.cssClass, enabled:status, change: $.proxy(this._OnClickCheckList, this) });
            }
            this.popupList.find("span.e-chkbox-wrap,div.e-chkbox-small,span.e-chk-inact,span.e-chk-image").attr("unselectable", "on");

        },

        _removeCheckbox: function (list) {
            var i, checkbox;
            this.listitems = list.find("ol,ul").children("li");
            checkbox = this.listitems.find('.listcheckbox');
            if (checkbox.length > 0) {
                this.listitems.find('.listcheckbox').ejCheckBox('destroy');
                this.listitems.find('input[type=checkbox]').remove();
            }
        },
        _setCheckAll: function (value) {
            if ((this.model.showCheckbox || (!this.model.showCheckbox && this.model.multiSelectMode != "none")) && (value))
                this.checkAll();
            else this.model.checkAll = false;
        },
        _setUncheckAll: function (value) {
            if ((this.model.showCheckbox || (!this.model.showCheckbox && this.model.multiSelectMode != "none")) && (value))
                this.uncheckAll();
            else this.model.uncheckAll = false;
        },

        checkAll: function () {
            var _nodes = this._selectedIndices;
            this._mapFields();
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
            if (!this._isWatermark)
                this._hiddenSpan.css("display", "none");
            for (i = 0; i < this.listitems.length; i++) {
                if (!$(this.listitems[i]).hasClass('e-disable'))
                    if (($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked') == false) || (!this.model.showCheckbox && this.model.multiSelectMode != "none" && !$(this.listitems[i]).hasClass("e-active"))) {
                        $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                        this.currentValue =  this._isPlainType(this.popupListItems) ? this.popupListItems[i] : this._getField(this.popupListItems[i], this.mapFld._text);
                        this._itemID = $(this.listitems[i]).attr("id");
                        if (!ej.isNullOrUndefined(this._itemID) && this._itemID != "")
                            this._selectedItemsID.push(this._itemID);
                        this._hiddenValue = this._getAttributeValue(this.listitems[i]) || this.currentValue;
                        this.checkedStatus = true;
                        this._createListHidden(this._hiddenValue);
                        if (this.model.multiSelectMode == "visualmode") {
                            this._ulBox.append(this._createBox(this.currentValue));
                            if (this._isPopupShown())
                                this._setListPosition();
                        }
                        if (!this.model.showCheckbox && this.model.multiSelectMode != "none") {
                            $(this.listitems[i]).addClass("e-active");
                        }
                        this._addText(this.currentValue);
                        if ($.inArray(i, _nodes) == -1) {
                            this._selectedIndices.push(i);
                            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                        }
                        this._selectedValue = this._getAttributeValue(this.listitems[i]) || "";
                        if (!this._initValue)
                            this._trigger('checkChange', { isChecked: this.checkedStatus ,data:this.model});
                        args = { text: this.element[0].value, selectedText: this.currentValue, itemId: i, selectedValue: this._selectedValue, value: this._selectedValue, isChecked: this.checkedStatus };
                        if (!this._initValue)
                            this._trigger("change", args);
                        this._activeItem = i;
                        this.activeItem = this._getActiveItem();
                        this._cascadeAction();
                    }
            }
            this.value(this._hiddenInput.val());
            this.model.text = this.element.val();
            this.model.itemValue = this._selectedValue;
            this.model.uncheckAll = false;
            this.model.checkAll = true;
            this._setWatermark();
        },
        _createListHidden: function (value) {
            var arrayHidden = document.createElement("input");
            this._setAttr(arrayHidden, { type: "hidden", name: this._name, value: value, id: "#" + value  });
            $("#" + this._id + "_container").append(arrayHidden);
            $(this._hiddenInput).attr("name", "hiddenEle");
        },
        _removeListHidden: function (value) {
            var arrayEle = document.getElementById("#" + value );
            $(arrayEle).remove();
        },
        _getAttributeValue: function (val) {
            return val ? val.getAttribute("data-value") || val.getAttribute("value") : null;
        },
        _selectSingleItem: function (sngitems) {
            this.currentValue = $.trim($(this.listitems[sngitems]).text());
            this._hiddenValue = this._getAttributeValue(this.listitems[sngitems]) || this.currentValue;
            this._addText(this.currentValue);
            this._selectedValue = this._getAttributeValue(this.listitems[sngitems]) || "";
            this.value(this._hiddenInput.val());
            this.model.itemValue = this._selectedValue;
            this.model.text = this.element.val();
            if (!this.model.showCheckbox) $(this.listitems[sngitems]).addClass("e-active");
        },
        _selectCheckedItem: function (chkitems) {
            if (chkitems.length > 0) {
                if (!this._isWatermark)
                    this._hiddenSpan.css("display", "none");
                this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
                if (!this.listitems.length > 0) return false;
                for (var i = 0; i < chkitems.length; i++) {
                    if (this.listitems.length > chkitems[i])
                        if ($(this.listitems[chkitems[i]].firstChild).find('.listcheckbox').ejCheckBox('isChecked') == false || (!this.model.showCheckbox && (this.model.multiSelectMode != "none") && !$(this.listitems[chkitems[i]]).hasClass("e-active"))) {
                            $(this.listitems[chkitems[i]].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                            this.checkedStatus = true;                           
                            if (this.model.multiSelectMode == "visualmode")
                                this._ulBox.append(this._createBox($.trim($(this.listitems[chkitems[i]]).text())));
                            this._selectSingleItem(chkitems[i]);
                            this._createListHidden(this._hiddenValue);
                            if ($.inArray(chkitems[i], this._selectedIndices) == -1) {
                                this._selectedIndices.push(chkitems[i]);
                                this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                            }
                            if (!this._initValue)
                                this._trigger('checkChange', { isChecked: this.checkedStatus,data:this.model });
                            var args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, selectedValue: this._selectedValue, value: this._selectedValue, isChecked: this.checkedStatus };
                            if (!this._initValue)
                                this._trigger("change", args);
                            this._cascadeAction();
                        }
                }
                if (!this.model.showCheckbox && (this.model.multiSelectMode == "none") && !$(this.listitems[chkitems[i]]).hasClass("e-active")) {
                    this._selectSingleItem(chkitems[0]);
                    this._selectedIndices[0] = chkitems[0];
                    args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, selectedValue: this._selectedValue };
                    if (!this._initValue)
                        this._trigger("change", args);
                    this._cascadeAction();
                }


                this._setWatermark();
            }
        },

        /* will deprecate with uncheckAll method */
        unCheckAll: function () { this.uncheckAll(); },

        uncheckAll: function () {
            this.listitems = this.popupList.find("ol,ul").children("li:not('.e-category')");
            this._mapFields();
            for (i = 0; i < this.listitems.length; i++) {
                if (($(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) || (!this.model.showCheckbox && (this.model.multiSelectMode != "none") && $(this.listitems[i]).hasClass("e-active"))) {
                    // $(this.listitems[i]).removeClass('e-active');
                    $(this.listitems[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                    this.currentValue =  this._isPlainType(this.popupListItems) ? this.popupListItems[i] : this._getField(this.popupListItems[i], this.mapFld._text);
                    this._hiddenValue = this._getAttributeValue(this.listitems[i]) || this.currentValue;
                    if (this.model.multiSelectMode == "visualmode") {
                        this._deleteBoxCheck(this.currentValue);
                        if (this._isPopupShown())
                            this._setListPosition();
                    }
                    if (!this.model.showCheckbox && this.model.multiSelectMode != "none") $(this.listitems[i]).removeClass("e-active");
                    this._activeItem = i;
                    this.activeItem = this._getActiveItem();
                    this._removeText(this.currentValue);
                    this._removeListHidden(this._hiddenValue);
                }
                this.checkedStatus = false;
                var _nodes = this._selectedIndices;
                if ($.inArray(i, _nodes) > -1) {
                    this._selectedIndices.splice($.inArray(i, _nodes), 1);
                    this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                }
                this._selectedValue = this._getAttributeValue(this.listitems[i]) || "";
                this._itemID = $(this.listitems[i]).attr("id");
                if (!ej.isNullOrUndefined(this._itemID) && this._itemID != "")
                    this._removeSelectedItemsID();
                if (!this._initValue)
                    this._trigger('checkChange', { isChecked: this.checkedStatus ,data:this.model});
                args = { text: this.element[0].value, selectedText: this.currentValue, itemId: i, selectedValue: this._selectedValue, value: this._selectedValue, isChecked: this.checkedStatus };
                if (!this._initValue)
                    this._trigger("change", args);
                this._cascadeAction();
            }
            this.value(this._hiddenInput.val());
            this.model.text = this.element.val();
            this.model.itemValue = this._selectedValue;
            this.model.checkAll = false;
            this.model.uncheckAll = true;
            this._setWatermark();
        },
        _removeSelectedItemsID: function () {
            var itemToRemove;
            itemToRemove = this._selectedItemsID.indexOf(this._itemID);
            this._selectedItemsID.splice(itemToRemove, 1);
            this._itemID = "";
        },


        _refreshScroller: function () {
            this.popupList.find(".e-content, .e-vhandle").removeAttr("style");
            this.popupList.find(".e-vhandle div").removeAttr("style");
            this.popupList.children(".e-content").removeClass("e-content");
            var flag = this._isPopupShown();
            this.popupListWrapper.css("display", "block");
            this.popupList.css({ "display": "block" });  // For get the height of the popup
            this.scrollerObj.model.height = this._getPopupHeight();
            this.scrollerObj.refresh();
            if (!this.model.enablePopupResize) { // to set popup height as per the no.of list items
                this.popupList.css("height", "auto");
                this.popupListWrapper.css("height", "auto");
            }
            this.scrollerObj.option("scrollTop", 0);
            if (!flag) this.popupListWrapper.css("display", "none");
        },
        _enablePopupResize: function () {
            if (this.model.enablePopupResize) {
                this.popupListWrapper.addClass("e-resizable").append(ej.buildTag("div.e-icon e-resize-handle"))
                .find(".e-resize-handle").addClass((this.model.enableRTL) ? "e-rtl-resize" : "");
                this._resizePopup();
            }
        },


        _resizePopup: function () {
            var proxy = this, started = false;
            this.popupListWrapper.find("div.e-resize-handle").ejResizable(
                {
                    minHeight: proxy.model.minPopupHeight,
                    minWidth: proxy.model.minPopupWidth,
                    maxHeight: proxy.model.maxPopupHeight,
                    maxWidth: proxy.model.maxPopupWidth,
                    handle: "e-ddl-popup",
                    resizeStart: function (event) {
                        if (!proxy.model.enabled)
                            return false;
                        !started && proxy._trigger("popupResizeStart", { event: event });
                        started = true;
                    },
                    resize: function (event) {
                        var reElement = $(event.element).parents("div.e-ddl-popup");
                        proxy._refreshPopupOnResize($(reElement).outerHeight(), $(reElement).outerWidth());
                        proxy._trigger("popupResize", { event: event });
                    },
                    resizeStop: function (event) {
                        if (started) {
                            proxy._refreshPopupOnResize(proxy.model.popupHeight, proxy.model.popupWidth);
                            started && proxy._trigger("popupResizeStop", { event: event });
                            started = false;
                        }
                    },
                    helper: function (event) {
                        var reElement = $(event.element).parents("div.e-ddl-popup");
                        proxy._refreshPopupOnResize($(reElement).outerHeight(), $(reElement).outerWidth());
                        return $(proxy.popupListWrapper);
                    }
                });
        },

        _refreshPopupOnResize: function (currHeight, currWidth) {
            if (currHeight) this.model.popupHeight = currHeight;
            if (currWidth) this.model.popupWidth = currWidth;
            this.popupListWrapper.css({ "height": this.model.popupHeight, "min-height": this.model.minPopupHeight, "max-height": this.model.maxPopupHeight });
            this._setListWidth();
            this._refreshScroller();
        },

        _setListWidth: function () {
            var width = this.model.popupWidth;
            if (width != "auto") this.popupListWrapper.css({ "width": width });
            else this.popupListWrapper.css({ "min-width": this.model.minPopupWidth });
            this.popupListWrapper.css({ "max-width": this.model.maxPopupWidth });
        },

        _setListHeight: function () {
            this.model.enablePopupResize ? this.popupListWrapper.css({ "min-height": this.model.minPopupHeight, "max-height": this.model.maxPopupHeight, "height": this.model.popupHeight }) :
            this.popupListWrapper.css({ "max-height": this.model.popupHeight, "min-height": this.model.minPopupHeight });
        },

        _getPopupHeight: function () {
            var wrap = this.popupListWrapper.height();
            if (this.model.enablePopupResize) wrap -= this.popupListWrapper.find(">div.e-resize-handle").height();
            if (this.model.enableFilterSearch && this.inputSearch) {
                var ele = this.inputSearch.parent(".e-in-wrap");
                wrap -= (parseInt(ele.css("height")) + parseInt(ele.css('margin-top')) + parseInt(ele.css('margin-bottom')));
            }
            return wrap;
        },

        _refreshPopup: function () {
            if (this.model.popupWidth == "auto") this.popupListWrapper.css({ "min-width": this.wrapper.width() });
            this._refreshScroller();
            this._setListPosition();
        },

        _setListPosition: function () {
            var elementObj = this.wrapper, pos = this._getOffset(elementObj), winWidth,
            winBottomHeight = $(document).scrollTop() + $(window).height() - (pos.top + $(elementObj).outerHeight()),
            winTopHeight = pos.top - $(document).scrollTop(),
            popupHeight = this.popupListWrapper.outerHeight(),
            popupWidth = this.popupListWrapper.outerWidth(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial(), popupmargin = 3,
            topPos = ((popupHeight < winBottomHeight || popupHeight > winTopHeight) ? pos.top + totalHeight + popupmargin : pos.top - popupHeight - popupmargin) - border;
            winWidth = $(document).scrollLeft() + $(window).width() - left;
            if (this.model.enableRTL || popupWidth > winWidth && (popupWidth < left + elementObj.outerWidth())) left -= this.popupListWrapper.outerWidth() - elementObj.outerWidth();
            this.popupListWrapper.css({
                "left": left + "px",
                "top": topPos + "px",
                "z-index": maxZ
            });

        },

        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },

        _getZindexPartial: function () {
            return ej.util.getZindexPartial(this.element, this.popupList);
        },


        _showResult: function () {
            var proxy = this;
            if (this._trigger("beforePopupShown", { text: this.element[0].value, value: this._selectedValue })) return;
            this._refreshPopup();
            $(this.popupListWrapper).slideDown(this.model.enableAnimation ? 200 : 1, "easeOutQuad", function () {
                $(document).on("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
                proxy._on(ej.getScrollableParents(proxy.wrapper), "scroll", proxy._hideResult);
            });
            this.element[0].setAttribute("aria-expanded", true);
            this._listSize = this.ultag.children("li:not('.e-category')").size();
            this._trigger("popupShown", { text: this.element[0].value, value: this._selectedValue });
            this.scrollerObj.setModel({ scrollTop: this._calcScrollTop('active') });
        },

        _OnWindowResize: function (e) {
            if (!ej.isNullOrUndefined(this.model) && this._isPopupShown()) {
                this._refreshPopup();
            }
        },
        _hideResult: function () {
            if (this.model && this._isPopupShown()) {
                var proxy = this;
                if (this._trigger("beforePopupHide", { text: this.element[0].value, value: this._selectedValue })) return;
                $(this.popupListWrapper).slideUp(this.model.enableAnimation ? 100 : 0, "easeOutQuad", function () {
                    $(document).off("mousedown", $.proxy(proxy._OnDocumentClick, proxy));
                });
                if (this.element != null)
                    this.element.attr({ "aria-expanded": false });
                this._off(ej.getScrollableParents(this.wrapper), "scroll", this._hideResult);
                if (this.element != null)
                    this._trigger("popupHide", { text: this.element[0].value, value: this._selectedValue });
                setTimeout(function() {proxy._resetSearch();},100);
            }
        },

        _isPopupShown: function () {
            return (this.popupListWrapper.css("display") == "block");
        },

        _enterTextBoxValue: function () {
            var args;
            this.removeID = false;
            if (!this._isWatermark)
                this._hiddenSpan.css("display", "none");
            this._chooseSelectionType();
            args = { text: this.currentValue, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if (!this._initValue && !this._onSearch) this._trigger("select", args);
            if (!this.selectedIndexValue != this._activeItem) {
                if ((this.model.multiSelectMode != "none") || this.model.showCheckbox) {
                    if (!this._checkContains(this.currentValue)) {
                        if (this.model.showCheckbox) {
                            if (this.activeItem.find('.listcheckbox').ejCheckBox('isChecked') == false) {
                                this.activeItem.find('.listcheckbox').ejCheckBox('option', 'checked', true)
                                this.checkedStatus = true;
                            }
                        }
                        if (this.model.multiSelectMode == "visualmode") {
                            this._ulBox.append(this._createBox(this.currentValue));
                            if (this._isPopupShown())
                                this._setListPosition();
                        }
                        if (!this.model.showCheckbox) this.activeItem.addClass('e-active');
                        this._maintainHiddenValue();
                        this._addText(this.currentValue);
                        this._createListHidden(this._hiddenValue);
                        var _nodes = this._selectedIndices;
                        if ($.inArray(this.selectedIndexValue, _nodes) == -1) {
                            this._selectedIndices.push(this.selectedIndexValue);
                            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                        }
                    }
                    else {
                        this._removeTextBoxValue();
                        this._removeListHidden(this._hiddenValue);
                        if (this._selectedIndices != null) {
                            var itemIndex = this._selectedIndices.indexOf(this.selectedIndexValue);
                            this._selectedIndices.splice(itemIndex, 1);
                            this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                        }
                    }
                } else {
                    this.ultag.children("li").removeClass('e-hover').removeClass('e-active');
                    this.activeItem.addClass('e-active');
                    this.element.val(this.currentValue);
                    this._maintainHiddenValue();
                    this._hiddenInput.val(this._hiddenValue);
                    this.selectedItemIndex(this.selectedIndexValue);
                    this.selectedIndex(this.selectedIndexValue);
                    this._selectedIndices[0] = this.selectedIndexValue;
                }
                this._updateSelectedIndexByValue(this._hiddenInput.val());
                args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, selectedValue: this._selectedValue, value: this._selectedValue, isChecked: this.checkedStatus };

                if (!this._initValue && !this._onSearch)
                    this._trigger("change", args);
                this._cascadeAction();
                this.value(this._hiddenInput.val());
                this.model.itemValue = this._selectedValue;
                this.model.text = this.element.val();
                if (this.selectelement) {
                    if ($("#" + this._id).children().length > this.selectedIndexValue) $("#" + this._id).children()[this.selectedIndexValue].selected = true;
                }
            }
            this.model.uncheckAll = false;
            this._setWatermark();
        },
        _chooseSelectionType: function () {
            this.activeItem = this._getActiveItem();
            this.selectedIndexValue = this._activeItem;
            this._mapFields();
            if (this.model.dataSource != null && typeof this.model.dataSource[0] == "object") {
                this.currentValue = $.trim(this._getField(this.popupListItems[this._activeItem], this.mapFld._text));
                this.currentValue = (this.currentValue == "") ? $.trim(this.activeItem.text()) : this.currentValue;
                this.hidelement = this._getField(this.popupListItems[this._activeItem], this.mapFld._value);
                this._selectedValue = this.hidelement;
                this._itemID = this._getField(this.popupListItems[this._activeItem], this.mapFld._id);
            } else {
                this.currentValue = $.trim(this.activeItem.text());
                if (this._getAttributeValue(this.activeItem[0]))
                    this._selectedValue = this._getAttributeValue(this.activeItem[0]);
                else {
                    if (this.currentValue != null) {
                        this.activeItem[0].setAttribute("value", this.currentValue);
                        this._selectedValue = this.currentValue;
                    }
                    else
                        this._selectedValue = "";
                }
                this._itemID = $(this.activeItem).attr("id");
            }
            if (!ej.isNullOrUndefined(this._itemID) && this._itemID != "") {
                if (!this.model.showCheckbox) {
                    this._selectedItemsID = [];
                    !this.removeID && this._selectedItemsID.push(this._itemID);
                }
                else
                    !this.removeID ? this._selectedItemsID.push(this._itemID) : this._removeSelectedItemsID();
            }
            this.selectedTextValue = this.currentValue;
        },
        _maintainHiddenValue: function () {
            this._hiddenValue = this._getAttributeValue(this.activeItem[0]) || this.currentValue;
        },
        _removeTextBoxValue: function () {
            this.removeID = true;
            this._chooseSelectionType();
            this._maintainHiddenValue();
            this._removeText(this.currentValue);
            this._removeListHidden(this._hiddenValue);
            this.activeItem.removeClass('e-active');
            if (this.model.showCheckbox) {
                if (this.activeItem.find('.listcheckbox').ejCheckBox('isChecked') == true) {
                    this.activeItem.find('.listcheckbox').ejCheckBox('option', 'checked', false);
                    this.checkedStatus = false;
                }
            }
            if ((this.model.multiSelectMode != "none") || this.model.showCheckbox) {
                var _nodes = this._selectedIndices;
                if ($.inArray(this.selectedIndexValue, _nodes) > -1) {
                    this._selectedIndices.splice($.inArray(this.selectedIndexValue, _nodes), 1);
                    this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                }
                if (this.model.multiSelectMode == "visualmode") {
                    this._deleteBoxCheck(this.currentValue);
                    if (this._isPopupShown())
                        this._setListPosition();
                }
            }
            if ((this.model.cascadeTo != null) && this.model.showCheckbox) this._cascadeAction();
            this._updateSelectedIndexByValue(this._hiddenInput.val());
            var args = { text: this.currentValue, selectedText: this.currentValue, itemId: this.selectedIndexValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if (!this._initValue && !this._onSearch)
                this._trigger("select", args);
            args = { text: this.element[0].value, selectedText: this.currentValue, itemId: this.selectedIndexValue, selectedValue: this._selectedValue, value: this._selectedValue, isChecked: this.checkedStatus };
            if (!this._initValue && !this._onSearch)
                this._trigger("change", args);
            this.value(this._hiddenInput.val());
            this.model.itemValue = this._selectedValue;
            this.model.text = this.element.val();
            this.model.checkAll = false;
            this._setWatermark();
        },


        _createBox: function (value) {
            if (!this._checkContains(value)) {

                if (this._ulBox.css('display') == "none" && this.element.css('display') != "none") this._swapUlandInput(true);
                var span = ej.buildTag("span.e-icon e-close", "", {}, { "unselectable": "on" });
                var li = ej.buildTag("li.e-options").text(value).append(span);
                this._on(span, "click", function (e) {
                    if (!this.model.enabled) return false;
                    this._deleteBox($(e.target).parent());
                    if (!this._isFocused && !this._isPopupShown())
                        this._setWatermark();
                    if (!this.model.showCheckbox) {
                        for (var i = 0; i < this.ultag.children("li").length; i++) {
                            if (this.ultag.children("li")[i].innerText == $(e.target).parent()[0].textContent)
                                $(this.ultag.children("li")[i]).removeClass("e-active").removeClass("e-hover");
                        }
                        this._removeText($(e.target).parent()[0].textContent);
                    }
                    if (this.model.multiSelectMode != "none") {
                        var itemIndex = this._selectedIndices.indexOf(this.selectedIndexValue);
                        this._selectedIndices.splice(itemIndex, 1);
                        this.model.selectedItems = this.model.selectedIndices = this._selectedIndices;
                    }
                });
                return li;
            }
        },
        _deleteBoxCheck: function (val) {
            var items = this._ulBox.children('li');
            for (var i = 0; i < items.length; i++) {
                if ($(items[i]).text() == val) {
                    $(items[i]).remove();
                }
            }
        },
        _deleteLastBox: function () {
            var items = this._ulBox.children("li:not(.e-search-box)");
            var item = items.last();
            if (item.hasClass("e-active")) this._deleteBox(item);
            else {
                this._removeActive();
                item.addClass("e-active");
            }
        },
        _deleteBox: function (items) {
            for (var i = 0; i < items.length; i++) {
                var cobj = $(items[i]);
                var deltext = cobj.text();
                var listItems = this.ultag.children("li:not('.e-category')");
                for (var j = 0; j < listItems.length; j++) {
                    if ($(listItems[j]).text() == deltext) {
                        this._activeItem = j;
                        this._uncheckboxModel(this._activeItem);
                        var hidval = this._getAttributeValue(listItems[j]) || deltext;
                        this._removeListHidden(hidval);
                    }
                }
                cobj.remove();
            }
        },

        _swapUlandInput: function (inputHide) {
            if (inputHide) {
                this.element.css('display', 'none');
                this._ulBox.css('display', 'block');
                this.wrapper.css({ 'height': 'auto' });
            }
            else {
                this.element.css('display', 'block');
                this._ulBox.css('display', 'none');
                this.wrapper.css({ 'height': this.model.height });
            }
        },
        _removeActive: function () {
            this._ulBox.children("li").removeClass("e-active");
        },
        _adjustWidth: function () {
            var tempSpan = ej.buildTag("span", this.element.val());
            this.container.append(tempSpan);
            this.element.width(tempSpan.width() + 30);
            tempSpan.remove();
        },
        _uncheckboxModel: function (val) {
            var listItems = this.ultag.children("li:not('.e-category')");
            if ($(listItems[val].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                $(listItems[val].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                this._removeTextBoxValue();
                if (!this._initValue)
                    this._trigger('checkChange', { isChecked: this.checkedStatus,data:this.model });
            }
        },
        _destroyBoxModel: function () {
            this.container.prepend(this.element);
            this.element.css('display', 'block');
            this.wrapper.height(this.model.height);
            this._ulBox.remove();
            this._off(this.container, "click");
        },

        _removeListHover: function () {
            this.ultag.children("li").removeClass("e-hover");
        },

        _addListHover: function () {
            var activeItem = this._getActiveItem();
            activeItem.addClass("e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop('hover') });
            activeItem.focus();
        },
        _calcScrollTop: function (val) {
            var ulH = this.ultag.outerHeight(), li = this.ultag.find("li"), liH = 0, index, top, i;
            index = this._selectedIndices && this._selectedIndices.length > 0 && val == "active" ?
               this._selectedIndices[this._selectedIndices.length - 1] :
                this.ultag.find("li.e-" + val).index();
            for (i = 0; i < index; i++) { liH += li.eq(i).outerHeight(); }
            top = liH - ((this.popupList.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top < 0 ? 0 : top;
        },
        _getActiveItem: function () {
            return $(this.ultag.children("li:not('.e-category')")[this._activeItem]);
        },
        _setDimentions: function () {
            if (this.model.height)
                this.wrapper.height(this.model.height);
            if (this.model.width)
                this.wrapper.width(this.model.width);
        },


        _roundedCorner: function (val) {
            if (val) {
                this.container.addClass("e-corner");
                this.popupListWrapper.addClass("e-corner");
                if (this.inputSearch) this.inputSearch.parent('.e-in-wrap').addClass("e-corner");
            }
            else {
                this.container.removeClass("e-corner");
                this.popupListWrapper.removeClass("e-corner");
                if (this.inputSearch) this.inputSearch.parent('.e-in-wrap').removeClass("e-corner");
            }

        },

        _enabled: function (boolean) {
            if (boolean) this.enable();
            else this.disable();
        },

        _destroyDropdown: function () {
            this.element.insertAfter(this.wrapper);
            this.element.width(this.element.width() + this.dropdownbutton.outerWidth());
            this.wrapper.remove();
            this.container.remove();

            this.container.unbind("mousedown", $.proxy(this._OnDropdownClick, this));
        },


        _RightToLeft: function () {
            if (this.model.enableRTL) {
                this.wrapper.addClass("e-rtl");
            }
            else {
                this.wrapper.removeClass("e-rtl");
            }

        },
        _dropbtnRTL: function () {
            if (this.model.enableRTL) {
                this.popupListWrapper.addClass("e-rtl");
                this.popupList.addClass("e-rtl");
            }
            else {
                this.popupListWrapper.removeClass("e-rtl");
                this.popupList.removeClass("e-rtl");
            }
        },
        _OnDropdownClick: function (e) {
            e.preventDefault();
            if (($(e.target).is("li") && $(e.target).parent().hasClass("e-boxes")) || ($(e.target).parents("ul").hasClass("e-boxes") && $(e.target).hasClass("e-icon e-close")))
                return false;
            if (this.model.readOnly) return false;
            if (this.ultag.find('li').length > 0 || (this.model.enableFilterSearch && this._rawList.length > 0)) {
                this._OnPopupHideShow();
            }
        },
        _OnPopupHideShow: function () {
            if (this._isPopupShown()) {
                this._hideResult();
            }
            else {
                this._showResult();
                this.wrapper.focus();
            }
        },

        _showFullList: function () {
            var source = this.model.dataSource;
            if (ej.DataManager && source instanceof ej.DataManager) {
                if (!source.dataSource.offline && !(source.dataSource.json && source.dataSource.json.length > 0)) {
                    var proxy = this;
                    proxy._initDataSource(source);
                }
                else this._getFilteredList(source.dataSource.json);
            }
            else this._getFilteredList(source);
        },
        _getFilteredList: function (list) {
            if (!list || !list.length || list.length < 1) {
                this._targetElementBinding();
                this._renderRemaining();
            }
            else {
                var listItem = ej.DataManager(list).executeLocal(this._isPlainType(list) ? ej.Query() : this._getQuery(true));
                this._totalCount = listItem.count;
                this._listItem(listItem.result ? listItem.result : listItem);
                this._renderPopupList();
            }
        },

        _cascadeAction: function () {
            if (this.model.cascadeTo) {
                var citem = this.model.cascadeTo.split(","), i;
                for (i = 0; i < citem.length; i++) {
                    if ($('#' + citem[i]).hasClass("e-dropdownlist")) {
                        this._doCascadeAction(citem[i], this, this.checkedStatus);
                    }
                    else {
                        $('#' + citem[i]).on("ejDropDownListcreate", { Obj: this, status:this.checkedStatus }, function (e) {
                            if (!ej.isNullOrUndefined(e.data.Obj.getValue()) && e.data.Obj.getValue() != "") {
                                e.data.Obj._doCascadeAction(this.id, e.data.Obj, e.data.status);
                            }
                        });
                    }
                }
            }
        },
        _doCascadeAction: function (id, parentObj,status) {
            parentObj._currentValue = parentObj._getField(parentObj.popupListItems[parentObj._activeItem], parentObj.mapFld._value);
            parentObj.selectDropObj = $('#' + id).ejDropDownList('instance');
            var args = { cascadeModel: parentObj.selectDropObj.model, cascadeValue: parentObj._currentValue, requiresDefaultFilter: true };
            this._trigger("cascade", args);
            if (ej.isNullOrUndefined(parentObj[id])) {
                parentObj[id] = parentObj.selectDropObj.model.dataSource;
            }
            (ej.DataManager && parentObj[id] instanceof ej.DataManager) ?
                parentObj._cascadeOdataInit(parentObj[id], args.requiresDefaultFilter,status) :
            parentObj._cascadeJsonInit(parentObj.selectDropObj, parentObj[id], parentObj.mapFld._value, args.requiresDefaultFilter,status);
        },
        _cascadeOdataInit: function (_dSource, requiresFilter,status) {
            var proxy = this, queryPromise, tempQuery;
            proxy._dQuery = this.selectDropObj._getQuery().clone();
            tempQuery = proxy._dQuery.clone();
            if (requiresFilter) tempQuery.where(proxy.mapFld._value, "equal", proxy._currentValue);
            proxy.selectDropObj._addLoadingClass();
            queryPromise = _dSource.executeQuery(tempQuery);
            queryPromise.fail(function (e) {
                proxy._changedSource = null;
                proxy.selectDropObj.setModel({ dataSource: proxy._changedSource, enabled: false });
                proxy._trigger("actionFailure", e);
            }).done(function (e) {
                proxy._trigger("actionSuccess", e);
                proxy._cascadeDataBind(proxy.selectDropObj, e.result,status);
                proxy.selectDropObj._removeLoadingClass();
                //if (!ej.isNullOrUndefined(proxy.selectDropObj.model.selectedIndex) && proxy._setSelectedItem)
                //    proxy.selectDropObj._selectItemByIndex(proxy.selectDropObj.model.selectedIndex);
                //proxy._setSelectedItem = false;
            }).always(function (e) {
                proxy._trigger("actionComplete", e);
            });
        },

        _cascadeJsonInit: function (cascadeDropDownObj, _dSource, mapFld, requiresFilter,status) {
            var tempQuery = ej.Query();
            if (requiresFilter) tempQuery.where(mapFld, "==", this._currentValue);
            var changedSource = ej.DataManager(_dSource).executeLocal(tempQuery);
            this._cascadeDataBind(cascadeDropDownObj, changedSource,status);
        },

        _cascadeDataBind: function (cascadeDropDownObj, changedSource,status) {
            if ((this.model.showCheckbox && status) || (this.model.multiSelectMode != "none" && this.activeItem.hasClass("e-active"))) {
                this._changedSource = (!ej.isNullOrUndefined(this._changedSource)) ? this._changedSource.concat(changedSource) : changedSource;}
            else if (!this.model.showCheckbox && this.model.multiSelectMode == "none") this._changedSource = changedSource;
            else {
                for (var i = 0; i < changedSource.length; i++) {
                    if (this._isPlainType(changedSource) && this._isPlainType(this._changedSource)) this._changedSource.splice(this._changedSource.indexOf(changedSource[i]), 1);
                    else {
                        for (var j = 0; j < this._changedSource.length; j++) {
                            if (JSON.stringify(this._changedSource[j]) == JSON.stringify(changedSource[i]))
                                this._changedSource.splice(j, 1);
                        }

                    }
                }
                cascadeDropDownObj.setModel({ dataSource: null });
            }
            var cascadeModel = JSON.parse(JSON.stringify(cascadeDropDownObj.model));
            cascadeDropDownObj.setModel({ dataSource: this._changedSource, enabled: true });
            if (cascadeDropDownObj.model.showCheckbox || cascadeDropDownObj.model.multiSelectMode != "none") {
                $("input:hidden[id^='#'][name=" + cascadeDropDownObj._id + "]").remove();
                cascadeDropDownObj._hiddenInput.attr('name', cascadeDropDownObj._name);
            }
            var delProp = ["dataSource", "query", "fields", "itemsCount", "template"];
            for (var k = 0; k < delProp.length; k++) delete cascadeModel[delProp[k]];
            cascadeModel.enabled = true;
            cascadeDropDownObj.setModel(cascadeModel);
        },

        _OnMouseEnter: function (e) {
            if (!this.model.enabled || this.model.readOnly) return false;
            var targetEle;
            this.ultag.children("li").removeClass("e-hover");
            if ($(e.target).is("li:not('.e-category')")) { $(e.target).addClass("e-hover"); }
            if ($(e.target).hasClass("e-disable"))
                $(e.target).removeClass('e-hover');
            else if (e.target.tagName != "li") {
                targetEle = $(e.target).parents("li:not('.e-category')");
                $(targetEle).addClass("e-hover");
            }
            var activeItem;
            this.ultag.children("li:not('.e-category')").each(function (index) {
                if ($(this).hasClass("e-hover")) {
                    activeItem = index;
                    return false;
                }
            });
            this._activeItem = activeItem;
        },
        _OnMouseLeave: function (e) {
            if (!this.model.enabled || this.model.readOnly) return false;
            this.ultag.children("li").removeClass("e-hover");
        },
        _OnMouseClick: function (e) {
            if (!this.model.enabled || this.model.readOnly) return false;
            if(this.model.enableFilterSearch && $(e.target).is("li") && $(e.target).hasClass('e-nosuggestion')) {
                    return false;
                }
            else if (($(e.target).is("li") && !$(e.target).hasClass('e-disable')) || (!$(e.target).is("li") && !$(e.target).closest("li").hasClass('e-disable'))) {
                if (!this.model.showCheckbox) {
                    this._enterTextBoxValue();
                    if (this.model.multiSelectMode == "none")
                        this._hideResult();
                } else {
                    if (($(e.currentTarget).is("li")) && ($(e.target).is("li") || $(e.target.parentElement).is("li.e-hover")) && $(e.currentTarget.firstChild).find('.listcheckbox')) {
                        if ($(e.target.parentElement).is("li.e-hover"))
                            e.target = e.target.parentElement;
                        var element = this.popupPanelWrapper.find(".e-popup").find("li:not('.e-category')");
                        this._activeItem = $.inArray(e.target, element);
                        if ($(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                            $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                            this.checkedStatus = false;
                            this._checkboxValue = true;
                            this._removeTextBoxValue();
                        } else {
                            $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                            this.checkedStatus = true;
                            this._checkboxValue = true;
                            this._enterTextBoxValue();
                        }
                    }
                    if (!this._initValue && this._checkboxValue)
                        this._trigger('checkChange', { isChecked: this.checkedStatus, text: this.element.val(), itemId: this.selectedIndexValue, selectedText: this.currentValue, selectedValue: this._selectedValue, value: this._selectedValue,data:this.model });
                }
            }
        },
        _OnClickCheckList: function (e) {
            var element = this.popupPanelWrapper.find(".e-popup").find("li:not('.e-category')");
            this._activeItem = $.inArray($(e.event.target).parents("li")[0], element);
            if (e.isChecked) {
                this.checkedStatus = true;
                this._checkboxValue = true;
                this._enterTextBoxValue();
            } else {
                this.checkedStatus = false;
                this._checkboxValue = true;
                this._removeTextBoxValue();
            }

        },
        _OnListSelect: function (e) {
            e.preventDefault();
            var activeItem = this._getActiveItem();
            if ($(activeItem).find('.listcheckbox').ejCheckBox('isChecked')) {
                $(activeItem).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                this.checkedStatus = false;
                this._removeTextBoxValue();
            } else {
                $(activeItem).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                this.checkedStatus = true;
                this._enterTextBoxValue();
            }

            if (!this._initValue)
                this._trigger('checkChange', { isChecked: this.checkedStatus,data:this.model });
        },
        _OnDocumentClick: function (e) {
            if (this.model && (!this.model.enabled || this.model.readOnly)) return false;
            if (!$(e.target).is(this.popupList) && !$(e.target).parents(".e-ddl-popup").is(this.popupListWrapper) &&
                !$(e.target).is(this.element) && !$(e.target).parents(".e-ddl").is(this.wrapper)) {
                this._hideResult();
            }
            else if ($(e.target).is(this.inputSearch)) {
                this.inputSearch.focus();
            }
            else if ($(e.target).is(this.popupList) || $(e.target).parents(".e-ddl-popup").is(this.popupListWrapper))
                e.preventDefault();
        },
        _disableItemSelectUp: function () {
            var disableList = this.model.disableItemsByIndex.split(",").sort().reverse();
            if (this._activeItem == 0) this._activeItem = this._listSize;
            for (var lists in disableList)
                if (this._activeItem == parseInt(disableList[lists])) this._activeItem--;
        },
        _disableItemSelectDown: function () {
            var disableList = this.model.disableItemsByIndex.split(",").sort();
            if ((this._listSize) == this._activeItem) this._activeItem = 0;
            for (var lists in disableList)
                if (this._activeItem == parseInt(disableList[lists])) this._activeItem++;
        },
        _OnKeyPress: function (e) {
            if (this.model.enableIncrementalSearch) {
                this._OnTextEnter((ej.browserInfo().name == "mozilla") ? e.charCode : e.keyCode);
            }
        },
        _OnTextEnter: function (from) {
            var proxy = this;
            this._incqueryString += String.fromCharCode(from);
            if (this._incqueryString.length > 0) {
                setTimeout(function () { proxy._incqueryString = ""; }, 1000);
            }
            var list = this.popupList.find("ol,ul").children("li:not('.e-category')"), i, strlen,
            caseSence = this.model.caseSensitiveSearch, mapper = this.model.fields,
            tempSuggestion = [],
            str, queryStr = this._incqueryString,
            querylength = this._incqueryString.length, searchflag = false;

            if (!caseSence) queryStr = queryStr.toLowerCase();

            for (i = 0; i < list.length; i++) {
                str = $.trim($(list[i]).text());
                str = caseSence ? str : str.toLowerCase();
                if (str.substr(0, querylength) == queryStr) {
                    if (this._isPopupShown()) {
                        this._removeListHover();
                        this._activeItem = i;
                        if (!this.model.showCheckbox && this.model.multiSelectMode == "none")
                            this._enterTextBoxValue();
                        this._addListHover();
                    } else {
                        this._activeItem = i;
                        if (!this.model.showCheckbox && this.model.multiSelectMode =="none" )
                            this._enterTextBoxValue();
                    }
                    searchflag = true;
                } else if ((i == list.length - 1) && (searchflag == false)) {
                    if (this.model.itemValue == "") {
                        this._activeItem = null;
                        if (!this.model.showCheckbox && this.model.multiSelectMode == "none")
                            this.element.val("");
                    }
                    searchflag = true;
                }
                if (searchflag) break;
            }

        },

        _OnKeyDown: function (e) {
            if ((!this.model.enabled) || (this.model.readOnly)) return false;
            this._itemId = null;
            var _popupListItems = this.ultag.children("li:not('.e-category')");
            this._listSize = this.ultag.children("li:not('.e-category')").size();
            var liH, popupH, activeitem, flag;
            popupH = this.popupList.height();
            liH = this.ultag.children("li").outerHeight();
            activeitem = Math.round(popupH / liH);

            switch (e.keyCode) {
                case 38:
                    if (e.altKey) {
                        if (this.ultag.find('li').length > 0)
                            this._hideResult();
                        break;
                    }
                case 33:
                    flag = ((e.keyCode == 38) ? 1 : activeitem);
                    if (this._isPopupShown()) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= flag;
                        if (this.model.disableItemsByIndex != null) this._disableItemSelectUp();
                        this._addListHover();
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= flag;
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 40:
                    if (e.altKey) {
                        if (this.ultag.find('li').length > 0)
                            this._showResult();
                        break;
                    }
                case 34:
                    flag = ((e.keyCode == 40) ? 1 : activeitem);
                    if (this._isPopupShown()) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += flag;
                        if (this.model.disableItemsByIndex != null) this._disableItemSelectDown();
                        this._addListHover();
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += flag;
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 37:
                    if (this._isPopupShown()) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= 1;
                        if (this.model.disableItemsByIndex != null) this._disableItemSelectUp();
                        this._addListHover();
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem <= 0) || (this._activeItem == null) || (this._activeItem > this._listSize - 1))
                            this._activeItem = this._listSize - 1;
                        else
                            this._activeItem -= 1;
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 39:
                    if (this._isPopupShown()) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += 1;
                        if (this.model.disableItemsByIndex != null) this._disableItemSelectDown();
                        this._addListHover();
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    } else {
                        this.ultag.children("li").removeClass('e-active');
                        if ((this._activeItem >= this._listSize - 1) || (this._activeItem == null) || (this._activeItem < 0))
                            this._activeItem = 0;
                        else
                            this._activeItem += 1;
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 8:
                    if (this.model.multiSelectMode == "visualmode" || this.model.showCheckbox) {
                        if (this._ulBox) this._deleteLastBox();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 9:
                    this._hideResult();
                    break;
                case 35:
                    if (this._isPopupShown()) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = this._listSize - 1;
                        this._addListHover();
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    else {
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = this._listSize - 1;
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 36:
                    if (this._isPopupShown()) {
                        this._removeListHover();
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = 0;
                        this._addListHover();
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    else {
                        this.ultag.children("li").removeClass('e-active');
                        this._activeItem = 0;
                        if ((!this.model.showCheckbox))
                            this._enterTextBoxValue();
                    }
                    e.preventDefault();
                    return false;
                    break;
                case 17:
                    break;
                case 18:
                    break;
                case 27:
                    if (this._isPopupShown()) e.stopPropagation();
                    this._hideResult();
                    break;
                case 32:
                    e.preventDefault();
                    return false;
                    break;
            }
        },

        _OnKeyUp: function (e) {
            if ((!this.model.enabled) || (this.model.readOnly)) return false;
            e.preventDefault();
            var target = e.target;
            if ($.trim(this.element.val()) == "" && e.keyCode == 38 && e.keyCode == 40) {
                this._hideResult();
                return false;
            }

            switch (e.keyCode) {
                case 38: break;
                case 40: break;
                case 37: break;
                case 39: break;

                case 20: break;
                case 16: break;
                case 17: break;
                case 18: break;
                case 35: break;
                case 36: break;
                case 144: break;
                case 27: break;
                case 9: break;

                case 13:
                    e.preventDefault();
                    if (this.model.showCheckbox) {
                        this._OnListSelect(e);
                    }
                    else {
                        this._enterTextBoxValue();
                        this._hideResult();
                    }
                    return false;
                    break;
                case 32:
                    e.preventDefault();
                    if (this.model.showCheckbox) {
                        this._OnListSelect(e);
                    }
                    else {
                        this._enterTextBoxValue();
                        this._hideResult();
                    }
                    return false;
                    break;
                case 8:

                    this._hideResult();
                    break;
                case 46:
                    if (this.model.multiSelectMode == "visualmode" || this.model.showCheckbox) {
                        this._deleteBox(this._ulBox.children("li.e-active"));
                        break;
                    }
            }
        },

        _targetFocus: function () {
            if (this.model.enabled && !this._isFocused) {
                if (!this._isWatermark)
                    this._hiddenSpan.css("display", "none");
                this.wrapper.addClass("e-focus");
                this._isFocused = true;
                this._trigger("focusIn");
            }
        },

        _targetBlur: function () {
            if (this.model.enabled) {
                this._isFocused = false;
                this.wrapper.removeClass("e-focus");
                this._setWatermark();
                this._trigger("focusOut");
            }
        },


        _wireEvents: function () {
            this._on(this.wrapper, "focus", this._targetFocus);
            this._on(this.wrapper, "blur", this._targetBlur);
            this._on(this.wrapper, "keydown", this._OnKeyDown);
            this._on(this.popupList, "keydown", this._OnKeyDown);
            this._on(this.popupList, "keyup", this._OnKeyUp);
            this._on(this.wrapper, "keyup", this._OnKeyUp);
            this._on(this.popupList, "keypress", this._OnKeyPress);
            this._on(this.wrapper, "keypress", this._OnKeyPress);
        }
    });

    ej.MultiSelectMode = {
        /** Supports to selection mode with none only */
        None: "none",
        /** Supports to selection mode with delimitter only */
        Delimiter: "delimiter",
        /** Supports to selection mode with visualmode only */
        VisualMode: "visualmode"
    };
    ej.FilterType = {
        Contains: "contains",
        StartsWith: "startsWith"
    }
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ListView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejListView", "ej.ListView", {
        _rootCSS: "e-lv",

        _prefixClass: "e-",
        defaults: {

            fieldSettings: {
                "navigateUrl": "navigateUrl",
                "href": "href",
                "enableAjax": "enableAjax",
                "preventSelection": "preventSelection",
                "persistSelection": "persistSelection",
                "text": "text",
                "enableCheckMark": "enableCheckMark",
                "checked": "checked",
                "primaryKey": "primaryKey",
                "parentPrimaryKey": "parentPrimaryKey",
                "imageClass": "imageClass",
                "imageUrl": "imageUrl",
                "childHeaderTitle": "childHeaderTitle",
                "childId": "childId",
                "childHeaderBackButtonText": "childHeaderBackButtonText",
                "renderTemplate": "renderTemplate",
                "templateId": "templateId",
                "attributes": "attributes",
                "mouseUp": "mouseUp",
                "mouseDown": "mouseDown",
                "groupID": "groupID",
                "id": "id"
            },

            mouseDown: null,

            mouseUp: null,

            items: []
        },



        dataTypes: {
            dataSource: "data",
            fieldSettings: "data",
            renderMode: "enum",
            theme: "enum",
            enablePersistence: "boolean"
        },
        observables: ["selectedItemIndex", "dataSource"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),
        dataSource: ej.util.valueFunction("dataSource"),

        _tags: [{
            tag: "items",
            attr:
                ["navigateUrl",
                "href",
                "text",
                "checked",
                "primaryKey",
                "parentPrimaryKey",
                "imageClass",
                "imageUrl",
                "childHeaderTitle",
                "childId",
                "childHeaderBackButtonText",
                "mouseUP",
                "mouseDown",
                "attributes",
                "renderTemplate",
                "templateId",
                "enableAjax",
                "preventSelection",
                "persistSelection",
                "enableCheckMark",
                "attributes",
                ],
            content: "template"
        }
        ],


        _init: function () {
            this._preventDefaultException = { tagName: /^(INPUT|TEXTAREA|BUTTON|SELECT)$/ };
            this._storedContent = [];
            this._storedTemplate = [];
            this._tempContent = $("body");
            this._touchStart = this.model.mouseDown;
            this._touchEnd = this.model.mouseUp;
            this._oldEle = this.element.clone();
            if (this._oldEle.find("ul").length)
                this.model.items = [];
            this._indexVal = 0;
            this._updateModelItems();
            this._load();
            var navObj = this.element.closest(".e-nb.e-js");
            this._isInsideNavigation = navObj.length;
            if (this._isInsideNavigation)
                this._nearestND = navObj.ejNavigationDrawer("instance");
        },

        _itemsObjectCollection: function (element, primaryKey, parentPrimaryKey, groupid) {
            if (ej.getAttrVal(element, "data-ej-rendertemplate")) {
                var tempid = this._tempContent.find('#' + ej.getAttrVal(element, "data-ej-templateid"));
                if (tempid.length) {
                    var ele = tempid.remove();
                    this._storedTemplate[this._indexVal] = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                }
                else if (!this._storedTemplate[this._indexVal])
                    if ($(element)[0].innerHTML)
                        this._storedTemplate[this._indexVal] = $(element)[0].innerHTML;
                    else if (this._storedTemplate[this._indexVal] && this._tempContent.find('#' + template).length) {
                        var ele = this._tempContent.find('#' + template).remove();
                        ele = ele[0].nodeName && ele[0].nodeName.toLowerCase() == "script" ? ej.getClearString(ele[0].innerHTML) : ele[0].outerHTML;
                        this._storedTemplate = ej._pushValue(this._storedTemplate, ele, this._indexVal);
                    }
                this._indexVal++;
            }
            var item = {};
            item.groupID = groupid ? groupid : "",
            item.text = ej.getAttrVal(element, "data-ej-text") ? ej.getAttrVal(element, "data-ej-text") : this._getText(element);
            item.preventSelection = ej.getAttrVal(element, "data-ej-preventselection");
            item.persistSelection = ej.getAttrVal(element, "data-ej-persistselection");
            item.navigateUrl = ej.getAttrVal(element, "data-ej-navigateurl");
            item.href = ej.getAttrVal(element, "data-ej-href");
            item.checked = ej.getAttrVal(element, "data-ej-checked");
            item.primaryKey = ej.getAttrVal(element, "data-ej-primarykey", primaryKey);
            item.parentPrimaryKey = ej.getAttrVal(element, "data-ej-parentprimarykey", parentPrimaryKey);
            item.imageClass = ej.getAttrVal(element, "data-ej-imageclass");
            item.imageUrl = ej.getAttrVal(element, "data-ej-imageurl");
            item.childHeaderTitle = ej.getAttrVal(element, "data-ej-childheadertitle");
            item.childId = item.href ? ej.getAttrVal(element, "data-ej-childid") ? ej.getAttrVal(element, "data-ej-childid") : ("page_" + parseInt(Math.random().toFixed(3) * 1000)) : "";
            item.childHeaderBackButtonText = ej.getAttrVal(element, "data-ej-childheaderbackbuttontext");
            item.mouseUp = ej.getAttrVal(element, "data-ej-mouseUp");
            item.mouseDown = ej.getAttrVal(element, "data-ej-mouseDown");
            item.attributes = element.attrNotStartsWith(/^data-ej-/);
            item.renderTemplate = ej.getAttrVal(element, "data-ej-rendertemplate");
            item.templateId = this._storedTemplate[this._indexVal - 1],
            item.enableAjax = ej.getAttrVal(element, "data-ej-enableajax");
            item.enableCheckMark = ej.getAttrVal(element, "data-ej-enablecheckmark");
            return item;
        },



        _renderLists: function () {
            var list = ej.buildTag("li", "", {}, {
                "class": this._prefixClass + "user-select " + this._prefixClass + "list " + this._prefixClass + "state-default{{if " + this.model.fieldSettings['primaryKey'] + " || " + this.model.fieldSettings['enableAjax'] + " || " + this.model.enableAjax + "}} " + this._prefixClass + "arrow{{/if}}{{if " + this.model.fieldSettings['imageClass'] + "}} " + this._prefixClass + "margin{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}} " + this._prefixClass + "margin{{/if}}{{/if}}",
                "data-childid": "{{if " + this.model.fieldSettings['primaryKey'] + "}}child{{>" + this.model.fieldSettings['primaryKey'] + "}}{{else " + this.model.fieldSettings['childId'] + "}}{{>" + this.model.fieldSettings['childId'] + "}}{{else}}{{/if}}",
                "data-childheadertitle": "{{>" + this.model.fieldSettings['childHeaderTitle'] + "}}",
                "data-childheaderbackbuttontext": "{{>" + this.model.fieldSettings['childHeaderBackButtonText'] + "}}",
                "data-preventSelection": "{{>" + this.model.fieldSettings['preventSelection'] + "}}",
                "data-persistSelection": "{{>" + this.model.fieldSettings['persistSelection'] + "}}",
                "data-navigateUrl": "{{>" + this.model.fieldSettings['navigateUrl'] + "}}",
                "data-loadajax": "{{>" + this.model.fieldSettings['enableAjax'] + "}}",
                "data-href": '{{>' + this.model.fieldSettings['href'] + '}}{{:~_checkAjaxUrls()}}',
                "data-checked": '{{>' + this.model.fieldSettings['checked'] + '}}',
                "data-templateid": "{{>" + this.model.fieldSettings['renderTemplate'] + "}}",
                "data-mouseup": "{{>" + this.model.fieldSettings['mouseUp'] + "}}",
                "data-mousedown": "{{>" + this.model.fieldSettings['mouseDown'] + "}}",
                "data-id": "{{>" + this.model.fieldSettings['id'] + "}}"
            });
            if (this.model.renderTemplate) {
                if (this._hasDataSource() && this._template)
                    list[0].innerHTML = this._template;
            }
            else {
                var linkTag = ej.buildTag("a", "", {}, { "class": this._prefixClass + "list-anchor " + this._prefixClass + "remove-shadow{{if " + this.model.fieldSettings['imageClass'] + "}} " + this._prefixClass + "margin{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}} " + this._prefixClass + "margin{{/if}}{{/if}}{{if " + this.model.fieldSettings['primaryKey'] + " || " + this.model.fieldSettings['enableAjax'] + " || " + this.model.enableAjax + "}} " + this._prefixClass + "fontimage{{/if}}", "href": (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 ? "" : "{{if " + this.model.fieldSettings['navigateUrl'] + " == '' || " + this.model.fieldSettings['navigateUrl'] + " == undefined }}{{else}}{{:" + this.model.fieldSettings['navigateUrl'] + "}}{{/if}}") });
                var span = ej.buildTag("span", "{{>" + this.model.fieldSettings['text'] + "}}", {}, { "class": this._prefixClass + "list-text " + this._prefixClass + "rel " + this._prefixClass + "user-select{{if " + this.model.fieldSettings['imageClass'] + "}} " + this._prefixClass + "text{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}} " + this._prefixClass + "text{{/if}}{{/if}}" });
                var imgClass = ej.buildTag("div", "", {}, { 'class': this._prefixClass + "list-img " + this._prefixClass + "rel " + this._prefixClass + "user-select {{>" + this.model.fieldSettings['imageClass'] + "}}" });
                var check = this._createCheckBox();
                if (this.model.renderMode == "windows" && ej.isMobile())
                    linkTag[0].innerHTML = "{{if !" + this.model.fieldSettings['primaryKey'] + "}}{{if " + this.model.fieldSettings['enableAjax'] + " == undefined || " + this.model.fieldSettings['enableAjax'] + ".toString() == 'false'}}{{if !" + this.model.fieldSettings['navigateUrl'] + "}}{{if " + this.model.fieldSettings['enableCheckMark'] + " !== undefined}}{{if " + this.model.fieldSettings['enableCheckMark'] + ".toString() == 'false' ? " + this.model.fieldSettings['enableCheckMark'] + " : " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{else}}{{if " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{/if}}{{/if}}{{/if}}{{/if}}" + "{{if " + this.model.fieldSettings['imageClass'] + "}}" + imgClass[0].outerHTML + "{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}}{{:~_checkImgUrls()}}{{/if}}{{/if}}" + span[0].outerHTML;
                else
                    linkTag[0].innerHTML = "{{if " + this.model.fieldSettings['imageClass'] + "}}" + imgClass[0].outerHTML + "{{else}}{{if " + this.model.fieldSettings['imageUrl'] + "}}{{:~_checkImgUrls()}}{{/if}}{{/if}}" + span[0].outerHTML + "{{if !" + this.model.fieldSettings['primaryKey'] + "}}{{if " + this.model.fieldSettings['enableAjax'] + " == undefined || " + this.model.fieldSettings['enableAjax'] + ".toString() == 'false'}}{{if !" + this.model.fieldSettings['navigateUrl'] + "}}{{if " + this.model.fieldSettings['enableCheckMark'] + " !== undefined}}{{if " + this.model.fieldSettings['enableCheckMark'] + ".toString() == 'false' ? " + this.model.fieldSettings['enableCheckMark'] + " : " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{else}}{{if " + this.model.enableCheckMark + "}}" + check[0].outerHTML + "{{/if}}{{/if}}{{/if}}{{/if}}{{/if}}";

                list[0].innerHTML = "{{if " + this.model.fieldSettings['renderTemplate'] + " == undefined || " + this.model.fieldSettings['renderTemplate'] + ".toString() == 'false'}}" + linkTag[0].outerHTML + "{{else}}{{:templateId}}{{/if}}";
            }
            if (this.model.renderMode == "ios7") {
                var innerDiv = ej.buildTag("div." + this._prefixClass + "list-div");
                innerDiv[0].innerHTML = list[0].innerHTML;
                list.empty().append(innerDiv);
            }
            return list;
        },


        _updateContent: function (currentItem, backButton) {
            var proxy = this;
            var $curIte = $(currentItem), childid = $curIte.attr('data-childid');
            id = this._isInsideNavigation && currentItem.attr('data-href') ? $('body').find($("#" + childid)) : this.element.find($("#" + childid));
            if (id.length) {
                var hdr = this.element.find('#' + childid + '_header');
                if (this.model.enableFiltering)
                    this._initializeFiltering($(id));
                var title = $curIte.attr('data-childheadertitle') == undefined ? currentItem.text() : $curIte.attr('data-childheadertitle');
                if (hdr.hasClass('e-header')) {
                    this._setHeaderVal(childid, title, backButton);
                }
                id.attr("data-hdr-title", title).attr("data-hdr-bckbtn", backButton);
                this._initEJCheckBox(id);
                if (this._isInsideNavigation && currentItem.attr('data-href') && this._nearestND.model.contentId) {
                    if ($('body').find('.e-lv.subpage.e-childitem')) {
                        $("#" + this._nearestND.model.contentId).empty().append(id.show());
                    }
                }
                else {
                    $curIte.closest('.subpage').hide();
                    id.show();
                    $(this.element.children()[0]).removeClass("e-slideright");
                    id.addClass("e-slideleft");
                }
            }
        },



        _renderHeader: function (id, showbutton, title, backtext) {
            var header = ej.buildTag("div", "", {}, { "id": id + "_header", "class": "e-header e-box" });
            if (showbutton) {
                header.append("<span class='e-hicon e-icon e-rarrowleft-2x'></span>");
                header.append("<div class='e-btn-text'>" + (backtext ? backtext : "Back") + "</div>");
            }
            else
                header.append("<div class='e-htitle'>" + title + "</div>");
            return header;
        },
        _setHeaderVal: function (id, title, backtext) {
            this._onBackButtonDelegate = $.proxy(this._onBackButtonClick, this);
            ej.listenTouchEvent($("#" + id + "_header"), ej.endEvent(), this._onBackButtonDelegate, false, this);
            $("#" + id + "_header").find(".e-btn-text").text(backtext ? backtext : "Back");
            $("#" + id + "_header").find(".e-htitle").text(title);
        },
        _onBackButtonClick: function (e) {
            this.element.find(".e-slideleft").removeClass("e-slideleft");
            this.element.children(":visible").hide();
            $(this.element.children()[0]).show();
            $(this.element.children()[0]).addClass("e-slideright");
        },


        _returnData: function () {
            var checkedItem = this._currentItem.closest('ul.e-list-hdr').find('.e-chkbox-wrap[aria-checked="true"]').closest("li");
            var elementId = ej.getAttrVal(this._currentItem, "data-id", this._currentItem.text());
            var items = this.dataSource().length ? typeof this.dataSource() == "string" ? eval(this.dataSource()) : this.dataSource() : this.model.items;
            return {
                hasChild: this._currentItem.attr("data-childid") && this._currentItem.attr('data-childid').length > 0 ? (this.element.find('#' + this._currentItem.attr('data-childid')).length ? true : false) : false,
                item: this._currentItem,
                text: this._currentItem.text(),
                index: this._currentItem.index(),
                isChecked: this._currentItem.find("input.e-lv-check").closest('.e-chkbox-wrap').attr('aria-checked') == "true" ? true : false,
                checkedItems: checkedItem.length ? checkedItem : null,
                checkedItemsText: $(checkedItem).map(function () { return $(this).text(); }).get(),
                itemData: this._generateData(items, elementId)
            };
        },
        _initEJCheckBox: function (ul) {
            var liItems = ul.find("li");
            var count = liItems.length;
            for (i = 0; i < count; i++) {
                var checked = ej.getAttrVal($(liItems[i]), "data-checked");
                var checked = checked == "true" ? true : false
                $($(liItems[i]).find(".e-lv-check")).ejCheckBox({ checked: checked });
            }
            ul.find('.e-lv-checkdiv').removeClass("e-lv-checkdiv")
            ul.find('.e-lv-check').parent().addClass("e-lv-checkdiv")
        },
        _triggerStartEvent: function (data) {
            this.model.mouseDown = ej.getAttrVal(this._currentItem, 'data-mousedown', this._touchStart);
            if (this.model.mouseDown)
                this._trigger("mouseDown", data);
        },
        _triggerEndEvent: function (data) {
            this.model.mouseUp = ej.getAttrVal(this._currentItem, 'data-mouseup', this._touchEnd);
            this.selectedItemIndex(this._currentItem.index());
            if (this.model.mouseUp)
                this._trigger("mouseUp", data);
        },
        _createFilterDiv: function () {
            return ej.buildTag('div.e-lv-filter', ej.buildTag('input.e-lv-input', "", {}, { 'type': 'text', 'placeholder': 'search' }));
        },
        _emptyFilterTextValue: function (element) {
            element.find('.e-lv-input').val("");
        },
        _createListDiv: function (childId) {
            this._div = ej.buildTag("div#" + childId + "." + this._rootCSS + " subpage e-childitem e-ajaxchild", (this.model.showHeader && this._isInsideNavigation && !this._nearestND.model.contentId) ? this._renderHeader(childId, true, this._currentItem.text()) : "");
        },
        _createCheckBox: function () {
            return ej.buildTag("input.e-lv-check", "", {}, { "type": "checkbox", });
        },
        _toggleCheckboxValue: function (lbCheck) {
            lbCheck.ejCheckBox({ checked: $(lbCheck.closest('.e-chkbox-wrap')).attr('aria-checked') == "true" ? false : true });
        },
        _setCheckboxValue: function (element, val) {
            element.ejCheckBox({ checked: val });
        },
        _convertToRelativeUrl: function (url) {
            return url;
        }
    });
    $.extend(true, ej.ListView.prototype, ej.ListViewBase.prototype);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Navigation Drawer elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejNavigationDrawer", "ej.NavigationDrawer", {

        _setFirst: true,
        validTags: ["div"],
        _rootCSS: "e-nb",
        _prefixClass: "e-",

        defaults: {
            isPaneOpen: false
        },

        dataTypes: {
            isPaneOpen: "boolean"
        },

        _init: function () {
            this._load();
            this._renderControl();
            this._renderEJMControls();
            this._createDelegate();
            this._wireEvents();
        },

        _load: function () {
            this._browser = this._getBrowserAgent();
            if (this.model.position == "normal") {
                if (!this.model.isPaneOpen) {
                    this._elementWrapper = ej.buildTag("div#" + this._id + "_WrapContainer", {}, {}, { "class": this._rootCSS + " e-nb-container e-nb-" + this.model.direction.toLowerCase() });
                    this.element.wrapAll(this._elementWrapper);
                    this._nbHome = $("#" + this._id + "_WrapContainer").parent();
                }
                else {
                    this.element.siblings().wrapAll("<div id='" + this._id + "_PageContainer' class='e-nb-container e-nb-pageContainer'></div>")
                    this._pageContainer = this._nbHome = this.element.siblings();
                }
            }
            else {
                this._nbHome = $("body");
                this.element.appendTo(this._nbHome);
            }
            this._nbHomeCss = this._nbHome.clone(true)[0].style;
        },

        _renderEJMControls: function () {
            if (this.model.enableListView) {
                this._lb = ej.buildTag("div#" + this._id + "_listview", {}, {}, { "class": "e-nb-listview" });
                if (this.model.items.length || this.model.listViewSettings.dataSource.length)
                    this._lb.appendTo(this.element);
                else
                    this.element.find(">ul").wrapAll(this._lb);
                this._lb = $("#" + this._id + "_listview");
                if (!this.model.listViewSettings["width"])
                    this.model.listViewSettings["width"] = 240;
                this.model.listViewSettings.items = this.model.items;
                this._lb.ejListView($.extend({}, this.model.listViewSettings, { loadComplete: $.proxy(this._setLayout, this) }));
                this._lb.ejListView("selectItem", this.model.listViewSettings.selectedItemIndex);
            }
        },

        _setModel: function (options) {
            var refresh = false;
            for (var prop in options) {
                switch (prop) {
                    case "width":
                        this._setWidth();
                        this._setLayout();
                        break;
                    case "direction":
                        this._setLayout();
                        break;
                    case "type":
                        this._transform(0, 0, true);
                        this._transform(0, 0, false);
                        break;
                    case "isPaneOpen":
                        this.model[prop] = options[prop];
                        if (this._pageContainer)
                            this._pageContainer.children().unwrap();
                        else
                            this._nbHome[0].style.cssText = this._nbHomeCss.cssText;
                    default:
                        refresh = true;
                }
                if (refresh) { this._refresh(); }
            }
        },

        _destroyEJMPlugin: function () {
            if (this._lb) {
                this._lb.ejListView("destroy");
                this._lb.children().unwrap();
            }
            if (this.model.contentId) $("#" + this.model.contentId).empty();
        },

        _getBrowserAgent: function () {
            return (/webkit/i).test(navigator.appVersion) ? 'webkit' : (/firefox/i).test(navigator.userAgent) ? 'Moz' : (/trident/i).test(navigator.userAgent) ? 'ms' : 'opera' in window ? 'O' : '';
        }

    });
    $.extend(true, ej.NavigationDrawer.prototype, ej.NavigationDrawerBase.prototype);
    $.extend(true, ej.NavigationDrawer.prototype.defaults.listViewSettings, ej.ListView.prototype.defaults);
    ej.NavigationDrawer.prototype._tags = ej.ListView.prototype._tags;

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html UL elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejListBox", "ej.ListBox", {

        element: null,

        model: null,
        validTags: ["ul"],
        _setFirst: false,
        _rootCSS: "e-listbox",
        defaults: {
            itemsCount: null,
            dataSource: null,
            query: ej.Query(),
            itemRequestCount: 5,
            fields: {
                id: null,
                text: null,
                imageUrl: null,
                imageAttributes: null,
                spriteCssClass: null,
                htmlAttributes: null,
                tooltipText: null,
                selectBy: null,
                checkBy: null,
                groupBy: null,
                tableName: null,

                //deprecated field properties
                selected: null,
                category: null,
                toolTipText: null
            },
            height: "220",
            width: "200",
            template: null,
            text: "",
            selectedIndex: null,
            checkedIndices: [],
            selectedIndices: [],
            cascadeTo: null,
            value: "",
            cssClass: "",
            targetID: null,
            htmlAttributes: {},
            showRoundedCorner: false,
            enableRTL: false,
            enabled: true,
            showCheckbox: false,
            allowVirtualScrolling: false,
            virtualScrollMode: "normal",
            enablePersistence: false,
            allowMultiSelection: false,
            allowDrag: false,
            allowDrop: false,
            enableIncrementalSearch: false,
            caseSensitiveSearch: false,
            loadDataOnInit: true,
            create: null,
            change: null,
            select: null,
            unselect: null,
            itemDragStart: null,
            itemDrag: null,
            itemDragStop: null,
            itemDrop: null,
            checkChange: null,
            destroy: null,
            actionComplete: null,
            actionSuccess: null,
            actionFailure: null,

            //Deprecated Members
            enableVirtualScrolling: false,
            checkAll: false,
            uncheckAll: false,
            enableLoadOnDemand: false,
            itemRequest: null,
            allowDragAndDrop: undefined,
            selectedItemIndex: null,
            enableItemsByIndex: null,
            checkItemsByIndex: null,
            disableItemsByIndex: null,
            uncheckItemsByIndex: null,
            itemDropped: null,
            selected: null,
            selectIndexChanged: null,
            selectedItems: [],
            checkedItems: [],
            checkedItemlist: [],
            selectedItemlist: [],
        },
        dataTypes: {
            cssClass: "string",
            itemsCount: "number",
            itemRequestCount: "number",
            allowDrag: "boolean",
            allowDrop: "boolean",
            enableIncrementalSearch: "boolean",
            caseSensitiveSearch: "boolean",
            height: "string",
            width: "string",
            template: "string",
            targetID: "string",
            selectedIndex: "number",
            cascadeTo: "string",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
            enablePersistence: "boolean",
            enabled: "boolean",
            allowMultiSelection: "boolean",
            dataSource: "data",
            checkedIndices: "data",
            selectedIndices: "data",
            htmlAttributes: "data",
            loadDataOnInit: "boolean",
            showCheckbox: "boolean"
        },
        observables: ["value"],
        value: ej.util.valueFunction("value"),
        enable: function () {
            if (this.listContainer.hasClass("e-disable")) {
                this.target.disabled = false;
                this.model.enabled = this.model.enabled = true;
                this.element.removeAttr("disabled");
                this.listContainer.removeClass('e-disable');
                if (this.model.allowMultiSelection) this.listContainer.removeClass("e-disable");
                var scroller = this.listContainer.find(".e-vscrollbar,.e-hscrollbar");
                if (this.model.showCheckbox) this.element.find(".listcheckbox").ejCheckBox("enable");
                if (scroller.length > 0)
                    this.scrollerObj.enable();
            }
        },
        disable: function () {
            if (!this.listContainer.hasClass("e-disable")) {
                this.target.disabled = true;
                this.model.enabled = this.model.enabled = false;
                this.element.attr("disabled", "disabled");
                this.listContainer.addClass('e-disable');
                if (this.model.allowMultiSelection) this.listContainer.addClass("e-disable");
                var scroller = this.listContainer.find(".e-vscrollbar,.e-hscrollbar");
                if (this.model.showCheckbox) this.element.find(".listcheckbox").ejCheckBox("disable");
                if (scroller.length > 0)
                    this.scrollerObj.disable();
            }
        },
        selectItemByIndex: function (index) {
            if (!this.model.enabled) return this;
            index = parseInt(index);
            if (index != null && !this.model.showCheckbox) {
                this._activeItem = index;
                if (this._activeItem <= 0) {
                    this._activeItem = 0;
                } else if (this._activeItem == this._listSize - 1) {
                    this._activeItem = this._listSize - 1;
                }
                var activeitem = $(this.element.children("li")[this._activeItem]);
                if (!activeitem.hasClass("e-select")) {
                    this.element.children("li").removeClass("e-select");
                    this._selectedItems = [];
                    this.model.selectedIndices = [];
                    activeitem.addClass("e-select");
                    this._selectedItems.push(activeitem);
                    this.model.selectedIndices.push(index);
                }
            }
            if (this.model.cascadeTo) {
                this._activeItem = index;
                this._cascadeAction();
            }
            this._setSelectionValues();
        },
        checkItemByIndex: function (index) {
            if (typeof (index) == "number")
                this.checkItemsByIndices(index.toString());
        },
        uncheckItemByIndex: function (index) {
            if (typeof (index) == "number")
                this.uncheckItemsByIndices(index.toString());
        },
        checkItemsByIndices: function (index) {
            if ((!this.model.enabled) || (ej.isNullOrUndefined(index))) return false;
            var checkitems = index.toString().split(',');
            if (checkitems.length > 0) {
                for (var i = 0; i < checkitems.length; i++) {
                    if (checkitems[i] != null) {
                        this._activeItem = parseInt(checkitems[i]);
                        if (this._activeItem < 0) this._activeItem = 0;
                        var activeitem = $(this.element.children("li")[this._activeItem]);
                        if (this.model.showCheckbox) {
                            if (!($(activeitem).find('.listcheckbox').ejCheckBox('isChecked')) && (!($(activeitem).hasClass('e-disable')))) {
                                $(activeitem).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                                this.checkedStatus = true;
                                if (!($.inArray(this._activeItem, this._checkedItems) > -1)) this._checkedItems.push(this._activeItem);
                                if (!($.inArray(activeitem[0], this.model.checkedIndices) > -1)) this.model.checkedIndices.push(activeitem[0]);
                            }
                        }
                    }
                }
            }
            this._setSelectionValues();
        },
        uncheckItemsByIndices: function (value) {
            if ((!this.model.enabled) || (ej.isNullOrUndefined(value))) return false;
            var checkitems = value.toString().split(',');
            if (checkitems.length > 0) {
                for (var i = 0; i < checkitems.length; i++) {
                    if (checkitems[i] != null) {
                        var index = parseInt(checkitems[i]);
                        var unselectitem = $(this.element.children("li")[parseInt(index)]);
                        if (this.model.showCheckbox) {
                            if (($(unselectitem).find('.listcheckbox').ejCheckBox('isChecked')) && (!($(unselectitem).hasClass('e-disable')))) {
                                $(unselectitem).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                                this.checkedStatus = false;
                                var itemIndex = this.model.checkedIndices.indexOf(unselectitem[0]);
                                if ($.inArray(index, this._checkedItems) > -1) this._checkedItems.splice(itemIndex, 1);
                                if ($.inArray(unselectitem[0], this.model.checkedIndices) > -1) this.model.checkedIndices.splice(itemIndex, 1);
                            }
                        }
                    }
                }
            }
            this._setSelectionValues();
        },
        selectAll: function () {
            if (!this.model.showCheckbox && this.model.allowMultiSelection) {
                var activeItem = this.element.children("li");
                for (var i = 0; i < activeItem.length; i++) {
                    if (!$(activeItem[i]).hasClass("e-select") && !$(activeItem[i]).hasClass("e-disable")) {
                        $(activeItem[i]).addClass("e-select");
                        this._selectedItems.push($(activeItem[i]));
                        this.model.selectedIndices.push(i);
                    }
                }
            }
            this._setSelectionValues();
        },
        //Deprecated Method
        unSelectAll: function () { this.unselectAll(); },
        unselectAll: function () {
            if (!this.model.showCheckbox)
                this._removeListHover();
            this._setSelectionValues();
            return this;
        },
        //deprecated function
        selectItemsByIndex: function (value) {
            this.selectItemsByIndices(value);
        },
        selectItemsByIndices: function (value) {
            var selectitems = value.toString().split(',');
            if (!this.model.showCheckbox && this.model.allowMultiSelection) {
                for (var i = 0; i < selectitems.length; i++) {
                    if (selectitems[i] != null) {
                        var index = parseInt(selectitems[i]);
                        this._activeItem = index;
                        if (this._activeItem < 0) {
                            this._activeItem = 0;
                        } else if (this._activeItem == this._listSize - 1) {
                            this._activeItem = this._listSize - 1;
                        }
                        var activeitem = $(this.element.children("li")[this._activeItem]);
                        if (!activeitem.hasClass("e-select") && !activeitem.hasClass("e-disable")) {
                            activeitem.addClass("e-select");
                            this._selectedItems.push(activeitem);
                            this.model.selectedIndices.push(index);
                        }
                    }
                }
            }
            this._setSelectionValues();
        },
        //deprecated property
        unselectItemsByIndex: function (value) {
            this.unselectItemsByIndices(value);
        },
        unselectItemsByIndices: function (value) {
            if (this.model.showCheckbox) return false;
            var selectitems = value.toString().split(',');
            for (var i = 0; i < selectitems.length; i++) {
                if (selectitems[i] != null) {
                    var index = parseInt(selectitems[i]);
                    var activeitem = $(this.element.children("li")[index]);
                    this._activeItem = index;
                    activeitem.removeClass('e-active e-select');
                    var itemIndex = this._selectedItems.indexOf(activeitem[0]);
                    this._selectedItems.splice(this.model.selectedIndices.indexOf(itemIndex), 1);
                    this.model.selectedIndices.splice(this.model.selectedIndices.indexOf(itemIndex), 1);
                }
            }
            this._setSelectionValues();
        },
        unselectItemByIndex: function (index) {
            if (this.model.showCheckbox) return false;
            index = parseInt(index);
            var unselectitem = $(this.element.children("li")[index]);
            if (unselectitem.hasClass('e-select')) {
                unselectitem.removeClass('e-active e-select');
                var itemIndex = this._selectedItems.indexOf(unselectitem[0]);
                this._selectedItems.splice(this.model.selectedIndices.indexOf(itemIndex), 1);
                this.model.selectedIndices.splice(this.model.selectedIndices.indexOf(itemIndex), 1);
            }
            this._setSelectionValues();
        },
        selectItemByText: function (text) {
            this[(this.model.allowMultiSelection ? "selectItemsByIndices" : "selectItemByIndex")](this.getIndexByText(text));
        },
        selectItemByValue: function (value) {
            this[(this.model.allowMultiSelection ? "selectItemsByIndices" : "selectItemByIndex")](this.getIndexByValue(value));
        },
        unselectItemByText: function (text) {
            this[(this.model.allowMultiSelection ? "unselectItemsByIndices" : "unselectItemByIndex")](this.getIndexByText(text));
        },
        unselectItemByValue: function (value) {
            this[(this.model.allowMultiSelection ? "unselectItemsByIndices" : "unselectItemByIndex")](this.getIndexByValue(value));
        },
        getSelectedItems: function () {
            var items = [], proxy = this;
            $(proxy.model.selectedIndices).each(function (index, elementIndex) {
                items.push(proxy.getItemByIndex(elementIndex));
            });
            return items;
        },
        getCheckedItems: function () {
            var items = [], proxy = this;
            $(proxy.model.checkedIndices).each(function (index, elementIndex) {
                items.push(proxy.getItemByIndex(elementIndex));
            });
            return items;
        },
        removeItem: function () {
            return this.removeSelectedItems();
        },
        removeItemByText: function (text) {
            if (!this.model.enabled) return false;
            return this.removeItemByIndex(this.getItemByText(text).index);
        },
        removeItemByIndex: function (index) {
            if (!this.model.enabled) return false;
            if (ej.isNullOrUndefined(this.model.dataSource)) {
                var text = $(this.element.find("li")[index]).remove().text();
                this._refreshItems();
                return text;
            }
            else if (!(this.model.dataSource instanceof ej.DataManager))
                return this._getRemovedItems([parseInt(index)]);
        },
        removeSelectedItems: function () {
            if (!this.model.enabled) return false;
            if (ej.isNullOrUndefined(this.model.dataSource)) {
                var text = this.value();
                $(this.getSelectedItems()).each(function (i, e) {
                    e.item.remove()
                });
                this._refreshItems();
                return text;
            }
            else if (!(this.model.dataSource instanceof ej.DataManager))
                return this._getRemovedItems(this.model.selectedIndices);
        },
        _getRemovedItems: function (index) {
            var removedItems = [];
            this.model.dataSource = this.model.dataSource.filter(function (e, i) {
                if (index.indexOf(i) != -1)
                    removedItems.push(e);
                else
                    return true;
            });
            this.refresh(true);
            this.unselectAll();
            return removedItems;
        },
        getIndexByValue: function (value) {
            var index;
            this.element.find("li").each(function () {
                if ($(this).attr("value") == value) {
                    index = $(this).index();
                    return false;
                }
            });
            return index;
        },
        getIndexByText: function (text) {
            var index;
            this.element.find("li").each(function () {
                if ($(this).text() == text) {
                    index = $(this).index();
                    return false;
                }
            });
            return index;
        },
        getTextByIndex: function (index) {
            return $(this.element.find("li")[index]).text();
        },
        getItemByText: function (text) {
            var proxy = this, obj;
            this.element.find("li").each(function () {
                if ($(this).text() == text) {
                    obj = proxy._getItemObject($(this));
                    return false;
                }
            });
            return obj;
        },
        getItemByIndex: function (index) {
            return this._getItemObject($(this.element.find("li")[index]));
        },
        getListData: function () {
            if (ej.DataManager && this.model.dataSource instanceof ej.DataManager)
                return this.listitems;
            else if (this.model.dataSource)
                return this.model.dataSource;
            else
                return;
        },
        enableItem: function (text) {
            this.element.find("li").each(function () {
                if ($(this).text() == text) {
                    $(this).removeClass("e-disable");
                    return false;
                }
            });
        },
        disableItem: function (text) {
            this.element.find("li").each(function () {
                if ($(this).text() == text) {
                    $(this).addClass("e-disable").removeClass("e-select");
                    return false;
                }
            });
        },
        moveUp: function () {
            var PreviousItem = this._getItem(this._selectedIndices()).prev();
            var selectedItem = this._getItem(this._selectedIndices());
            this._addListHover();
            this._getItem(this._selectedItem).removeClass("e-hover");
            PreviousItem.insertAfter(selectedItem);
        },
        moveDown: function () {
            var nextItem = this._getItem(this._selectedIndices()).next();
            var selectedItem = this._getItem(this._selectedIndices());
            this._addListHover();
            this._getItem(this._selectedItem).removeClass("e-hover");
            nextItem.insertBefore(selectedItem);
        },
        checkAll: function () {
            if (!this.model.enabled || !this.model.showCheckbox) return false;
            var items = this.element.find("li");
            for (i = 0; i < items.length; i++) {
                if (!($(items[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked')) && !($(items[i].firstChild).hasClass("e-disable"))) {
                    $(items[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                    this._checkedItems.push(items[i]);
                    this.model.checkedIndices.push(i);
                }
            }
            this.model.uncheckAll = false;
        },
        //Deprecated Method
        unCheckAll: function () { this.uncheckAll(); },
        uncheckAll: function () {
            if (!this.model.enabled || !this.model.showCheckbox) return false;
            var items = this.element.find("li");
            for (i = 0; i < items.length; i++)
                if ($(items[i].firstChild).find('.listcheckbox').ejCheckBox('isChecked'))
                    $(items[i].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
            this._checkedItems = [];
            this.model.checkedIndices = [];
            this.model.checkAll = false;
        },
        addItem: function (val, index) {
            var index = (!ej.isNullOrUndefined(index) && index <= this.element.find('li').length) ? index : this.element.find('li').length;
            var proxy = this;
            if (ej.isNullOrUndefined(this.model.dataSource)) {
                if (!(val instanceof Array)) {
                    this.listitem = (this.element.find('li').length ?
                                        ((index - 1 < 0) ? $(this.element.find('li')[0]).before('<li role="option">' + val + '</li>') : $(this.element.find('li')[index - 1]).after('<li role="option">' + val + '</li>'))
                                         : $(this.element).html('<li role="option">' + val + '</li>'));
                    this.listitems = this.element.find('li');
                    this._addItemIndex = index;
                    if (this.model.showCheckbox) {
                        $checkbox = ej.buildTag("input.listcheckbox e-align#popuplist" + (this.listitems.length - 1) + "_" + this._id, "", {}, {
                            type: "checkbox",
                            name: "list" + (this.listitems.length - 1)
                        });
                        $(this.listitems[index]).prepend($checkbox);
                        $($(this.listitems[index]).find(".listcheckbox")).ejCheckBox({
                            change: $.proxy(this._onClickCheckList, this)
                        });
                    }
                    if (this.model.allowDrag || this.model.allowDrop) this._enableDragDrop();
                    this._addItemIndex = null;
                    this._refreshItems();
                }
                else {
                    $(val).each(function (i, e) {
                        proxy.addItem(e, index);
                        index = index + 1;
                    })
                }
            }
            else if (!(this.model.dataSource instanceof ej.DataManager)) {
                if (!(val instanceof Array)) val = [val];
                $(val).each(function (i, e) {
                    proxy.model.dataSource.splice(index, 0, e);
                    index = index + 1;
                })
                this.refresh(true);
            }
        },
        enableItemByIndex: function (index) {
            if (typeof (index) == "number")
                this.enableItemsByIndices(index.toString());
        },
        disableItemByIndex: function (index) {
            if (typeof (index) == "number")
                this.disableItemsByIndices(index.toString());
        },
        disableItemsByIndices: function (value) {
            if (!this.model.enabled) return false;
            var selectitems = value.toString().split(',');
            for (var i = 0; i < selectitems.length; i++) {
                if (selectitems.length > 0 && !($.inArray(selectitems[i], this._disabledItems) > -1)) {
                    if ($(this.element.children("li")[parseInt(selectitems[i])]).hasClass('e-select')) $(this.element.children("li")[parseInt(selectitems[i])]).removeClass('e-select');
                    var disable = $(this.element.children("li")[parseInt(selectitems[i])]).addClass('e-disable');
                    disable.find(".listcheckbox").ejCheckBox("disable");
                    this._disabledItems.push(selectitems[i]);
                }
            }
        },
        enableItemsByIndices: function (value) {
            if (!this.model.enabled) return false;
            var selectitems = value.toString().split(','), index;
            for (var i = 0; i < selectitems.length; i++) {
                if (selectitems.length > 0 && ($.inArray(selectitems[i], this._disabledItems) > -1)) {
                    index = $.inArray(selectitems[i], this._disabledItems);
                    var enable = $(this.element.children("li")[parseInt(selectitems[i])]).removeClass('e-disable');
                    enable.find(".listcheckbox").ejCheckBox("enable");
                    this._disabledItems.splice(index, 1);
                }
            }
        },
        _init: function () {
            this._id = this.element[0].id;
            this._isMozilla = ej.browserInfo().name == "mozilla" ? true : false;
            this._cloneElement = this.element.clone();
            this._deprecatedValue()._initialize()._render()._enabled(this.model.enabled)._wireEvents();
            this._initValue = false;
            this._typeInterval = null;
            this._typingThreshold = 2000;
            //deprecatedFunction
            if (this.model.checkAll)
                this.checkAll();
            if (this.model.uncheckAll)
                this.uncheckAll();
            if (this.model.disableItemsByIndex)
                this.disableItemsByIndices(this.model.disableItemsByIndex.toString());
            if (this.model.enableItemsByIndex)
                this.enableItemsByIndices(this.model.enableItemsByIndex.toString());
            if (this.model.uncheckItemsByIndex)
                this.uncheckItemsByIndices(this.model.uncheckItemsByIndex.toString());

        },
        _deprecatedValue: function () {
            this.model.itemDrop = (this.model.itemDrop || this.model.itemDropped);
            this.model.change = (this.model.change || this.model.selectIndexChanged);
            this.model.fields.checkBy = this.model.fields.selected || this.model.fields.checkBy;
            this.model.fields.tooltipText = this.model.fields.toolTipText || this.model.fields.tooltipText;
            this.model.fields.groupBy = this.model.fields.category || this.model.fields.groupBy;
            this.model.select = (this.model.select || this.model.selected);
            if (this.model.allowDragAndDrop != undefined)
                this.model.allowDrag = this.model.allowDrop = true;
            this.model.selectedIndex = this.model.selectedIndex != null ? this.model.selectedIndex : this.model.selectedItemIndex;
            this.model.checkedIndices = ((this.model.checkedIndices.length ? this.model.checkedIndices : null) || (this.model.checkItemsByIndex ? this.model.checkItemsByIndex.toString().split(",") : null) || (this.model.checkedItems.length ? this.model.checkedItems : null) || (this.model.checkedItemlist.length ? this.model.checkedItemlist : []));
            this.model.selectedIndices = ((this.model.selectedIndices.length ? this.model.selectedIndices : null) || (this.model.selectedItems.length ? this.model.selectedItems : null) || (this.model.selectedItemlist.length ? this.model.selectedItemlist : []));
            return this;
        },
        _setModel: function (options) {
            if (!(this.model.enabled) && ej.isNullOrUndefined(options["enabled"])) return false;
            var option, refresh = false;
            for (option in options) {
                switch (option) {
                    case "value":
                        this._setText(ej.util.getVal(options[option]));
                        break;
                    case "dataSource":
                        this._checkModelDataBinding(options[option]);
                        break;
                    case "query":
                        this._queryCheck(options[option]);
                        break;
                    case "fields":
                        this.model.fields = options[option];
                        this._checkModelDataBinding(this.model.dataSource);
                        break;
                    case "template":
                        this.model.template = options[option];
                        this.refresh(true);
                        break;
                    case "loadDataOnInit":
                        this._loadContent = options[option];
                        this._checkModelDataBinding(this.model.dataSource);
                        break;
                    case "enableRTL":
                        this.model.enableRTL = options[option];
                        (this.model.enableRTL) ? this.listContainer.addClass("e-rtl") : this.listContainer.removeClass("e-rtl");
                        break;
                    case "enabled":
                        this.model.enabled = options[option];
                        this._enabled(options[option]);
                        break;
                    case "height":
                    case "width":
                        this.model[option] = options[option];
                        this._setDimensions();
                        break;
                    case "cssClass":
                        this.model.cssClass = options[option];
                        this.listContainer.addClass(this.model.cssClass);
                        break;
                    case "showCheckbox":
                        this._removeListHover()._checkboxHideShow(options[option]).uncheckAll();
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "selectedItemIndex":
                    case "selectedIndex":
                        if(!$(this.listitem[this.model.selectedIndex]).hasClass('e-disable')){
                            this.selectItemByIndex(options[option]);
                            this.model.selectedIndex = this.model.selectedItemIndex = options[option];
                        }
                        break;
                    case "checkItemsByIndex":
                    case "checkedItemlist":
                    case "checkedItems":
                    case "checkedIndices":
                        this.uncheckAll();
                        this.checkItemsByIndices(options[option].toString());
                        this.model.checkedIndices = this.model[option] = options[option];
                        break;
                    case "uncheckItemsByIndex":
                        this.uncheckItemsByIndices(options[option].toString());
                        this.model[option] = options[option];
                        break;
                    case "selectedItemlist":
                    case "selectedItems":
                    case "selectedIndices":
                        this.unselectAll();
                        this.selectItemsByIndices(options[option].toString());
                        this.model.selectedIndices = options[option];
                        break;
                    case "enableItemsByIndex":
                        this.model[option] = options[option];
                        this.enableItemsByIndices(options[option].toString());
                        break;
                    case "disableItemsByIndex":
                        this.model[option] = options[option];
                        this.disableItemsByIndices(options[option].toString());
                        break;
                    case "enableVirtualScrolling":
                        this.model.allowVirtualScrolling = options[option]; refresh = true;
                        break;
                    case "allowDrag":
                    case "allowDrop":
                    case "allowDragAndDrop":
                    case "allowVirtualScrolling":
                    case "virtualScrollMode":
                        this.model[option] = options[option]; refresh = true;
                        break;
                    case "checkAll":
                        this.model[option] = options[option]; if (options[option]) this.checkAll(); else this.uncheckAll();
                        break;
                    case "uncheckAll":
                        this.model[option] = options[option]; if (options[option]) this.uncheckAll(); else this.checkAll();
                        break;
                    case "htmlAttributes":
                        this._addAttr(options[option]);
                        break;
                    case "itemsCount":
                        this.model.itemsCount = options[option];
                        this._setItemsCount()._setDimensions();
                        break;
                    case "allowMultiSelection":
                        this.model.allowMultiSelection = options[option];
                        this._removeListHover();
                        break;
                }
            }
            if (refresh) this._refresh();
        },
        _destroy: function () {
            this.element.insertAfter(this.listContainer);
            this.listContainer.remove();
            if (!this._isList) this.element.empty();
            $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
            return this;
        },
        _refresh: function () {
            this._destroy()._init();
        },
        _finalize: function () {
            if (this.model.selectedIndex != null)
                this.selectItemByIndex(this.model.selectedIndex);
            else if ((this.model.showCheckbox == true) && (this._selectedItems.length > 0))
                this._selectCheckedItem(this._selectedItems);
            if (this.model.checkedIndices != null) this.checkItemsByIndices(this.model.checkedIndices.toString());
            return this;
        },
        _initialize: function () {
            this._isList = this.element.children().length ? true : false;
            this.target = this.element[0];
            this._queryString = null;
            this._disabledItems = [];
            this._itemId = null;
            this._up = this._down = this._ctrlClick = false;
            this.checkedStatus = this._isScrollComplete = false;
            this._incqueryString = "";
            this._activeItem = null;
            this._initValue = true;
            this.model.allowVirtualScrolling = (this.model.allowVirtualScrolling) ? this.model.allowVirtualScrolling : this.model.enableLoadOnDemand;
            this.model.virtualScrollMode = (this.model.enableVirtualScrolling) ? "continuous" : this.model.virtualScrollMode;
            this._selectedItems = [];
            this._checkedItems = [];
            this._loadContent = this.model.loadDataOnInit;
            if (this.model.enableVirtualScrolling) this.model.allowVirtualScrolling = true;
            this._setItemsCount();
            return this;
        },
        _render: function () {
            this._renderContainer()._addAttr(this.model.htmlAttributes);
            if (ej.DataManager && this.model.dataSource instanceof ej.DataManager)
                this._initDataSource(this.model.dataSource);
            else
                this._showFullList();
            if (!this.model.dataSource) this._finalize();
            if (this.model.showRoundedCorner)
                this._roundedCorner(true);
            return this;
        },
        _queryCheck: function (value) {
            this.model.query = value;
            this.element.empty();
            this._checkModelDataBinding(this.model.dataSource);
        },
        _checkModelDataBinding: function (source) {
            this.model.dataSource = source;
            if (source != null) {
                if (source.length != 0) {
                    if (ej.DataManager && source instanceof ej.DataManager) this._initDataSource(source);
                    else this._showFullList();
                } else this.element.empty();
            } else this.element.empty();
        },
        _initDataSource: function (source) {
            var proxy = this;
            proxy.listitems = proxy.model.dataSource;
            proxy._updateLoadingClass(true);
            var queryPromise = source.executeQuery(this._getQuery());
            queryPromise.done(function (e) {
                proxy.listitems = e.result;
                proxy._updateLoadingClass()._showFullList()._trigger("actionSuccess", e);
                proxy._finalize();
            }).fail(function (e) {
                proxy.model.dataSource = null;
                proxy._updateLoadingClass(true)._trigger("actionFailure", e);
            }).always(function (e) {
                proxy._trigger("actionComplete", e);
            });
        },
        _getQuery: function () {
            var queryManager;
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [],
                    mapper = this.model.fields;
                queryManager = ej.Query();
                for (var col in mapper)
                    if (col !== "tableName") column.push(mapper[col]);
                if (column.length > 0) queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$")) !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            } else queryManager = this.model.query;
            return queryManager;
        },
        _addDragableClass: function () {
            if (this.model.allowDrag || this.model.allowDrop) {
                this.element.css("cursor", "pointer");
                if (this.model.allowDrop) {
                    this.listContainer.addClass("e-droppable");
                    this.listBoxScroller.addClass("e-droppable");
                }
                var proxy = this;
                this.element.children("li").each(function (index) {
                    if (proxy.model.allowDrag) ($(this).addClass("e-draggable"));
                    if (proxy.model.allowDrop) ($(this).addClass("e-droppable"));
                });
            }
            return this;
        },
        _enableDragDrop: function () {
            if (this.model.allowDrag || this.model.allowDrop) this._drag();
        },
        _updateLoadingClass: function (value) {
            this.listContainer[(value ? "addClass" : "removeClass")]("e-load"); return this;
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.listContainer.addClass(value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy._enabled(false);
                else proxy.listContainer.attr(key, value);
            });
        },
        _renderContainer: function () {
            this.listContainer = ej.buildTag("div.e-ddl-popup e-box e-popup e-widget " + this.model.cssClass + "#" + this._id + "_container", "", {
                "visibility": "hidden"
            }, {
                "tabIndex": 0
            });
            this.listBoxScroller = ej.buildTag("div");
            this.ultag = ej.buildTag("ul.e-ul", "", {}, {
                "role": "listbox"
            });
            this.element = this.element.addClass("e-ul");
            this.listContainer.append(this.listBoxScroller).insertAfter(this.element);
            this.listBoxScroller.append(this.element);
            this.element.attr('unselectable', 'on').css('user-select', 'none').on('selectstart', false);
            this._hiddenInput = ej.buildTag("input#" + this._id + "_hidden", "", {}, {
                type: "hidden"
            }).insertBefore(this.element);
            this._hiddenInput.attr('name', this._id);
            return this;
        },
        _renderlistContainer: function () {
            this.hold = this.touchhold = false;
            this.item = "";
            this.startime = 0;
            var list = this.listitems,
                i, ulempty, ulno, litag, _id, _txt, mapper = this.model.fields,
                predecessor;
            this.lastScrollTop = -1;
            this.dummyUl = $();
            if (this.model.enableRTL) this.listContainer.addClass("e-rtl");
            if (this.model.dataSource == null || this.model.dataSource.length < 1) {
                predecessor = this.element.parents().last();
                if (this.model.targetID) this.docbdy = predecessor.find("#" + this.model.targetID);
                else this.docbdy = predecessor.find("#" + this._id);
                this.itemsContainer = this.docbdy;
                this.itemsContainer.children("ol,ul").remove();
                this.items = this.itemsContainer.children('li');
                this.items.children("img").addClass("e-align");
                this.items.children("div").addClass("e-align");
                this.itemsContainer.children("span").addClass("e-ghead");
                this.element.append(this.itemsContainer.children());
            }
            else if (this.model.dataSource != null && typeof list[0] != "object") { }
            else {
                this.mapFld = {
                    _id: null,
                    _imageUrl: null,
                    _imageAttributes: null,
                    _tooltipText: null,
                    _spriteCSS: null,
                    _text: null,
                    _value: null,
                    _htmlAttributes: null,
                    _selectBy: null,
                    _checkBy: null
                };
                this.mapFld._id = (mapper && mapper.id) ? mapper["id"] : "id";
                this.mapFld._imageUrl = (mapper && mapper.imageUrl) ? mapper["imageUrl"] : "imageUrl";
                this.mapFld._tooltipText = (mapper && mapper.tooltipText) ? mapper["tooltipText"] : "tooltipText";
                this.mapFld._imageAttributes = (mapper && mapper.imageAttributes) ? mapper["imageAttributes"] : "imageAttributes";
                this.mapFld._spriteCSS = (mapper && mapper.spriteCssClass) ? mapper["spriteCssClass"] : "spriteCssClass";
                this.mapFld._text = (mapper && mapper.text) ? mapper["text"] : this.listitems[0].text ? "text" : Object.keys(this.listitems[0])[0];
                this.mapFld._value = (mapper && mapper.value) ? mapper["value"] : "value";
                this.mapFld._htmlAttributes = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
                this.mapFld._checkBy = (mapper && mapper.checkBy) ? mapper["checkBy"] : "checkBy";
                this.mapFld._selectBy = (mapper && mapper.selectBy) ? mapper["selectBy"] : "selectBy";
                var mapCateg = (mapper && mapper.groupBy) ? mapper["groupBy"] : "",
                    groupedList, _query;
                _query = this.model.query;
                this.listContainer.height(this.model.height);
                this.listitemheight = 24;
                if (this.model.allowVirtualScrolling) {
                    if (this.model.virtualScrollMode == "normal") {
                        this.realUllength = 0;
                        if (this.model.dataSource.length < 0) {
                            query = this.model.query.take(parseInt(this.listContainer.height() / this.listitemheight));
                            var proxy = this;
                            if (ej.DataManager && this.model.dataSource instanceof ej.DataManager) {
                                proxy.listitems = proxy.model.dataSource;
                                var queryPromise = this.model.dataSource.executeQuery(query);
                                queryPromise.done(function (e) {
                                    proxy.listitems = e.result;
                                    proxy._trigger("actionSuccess", e);
                                }).fail(function (e) { proxy._trigger("actionFailure", e); })
                                  .always(function (e) { proxy._trigger("actionComplete", e); });
                            }
                        }
                        for (i = 0; i < this.listitems.length; i++)
                            this.dummyUl.push(ej.buildTag('li', null, null, { style: "height:20px;" })[0]);
                        this.element.append(this.dummyUl);
                        this._loadInitialData(_query, list);
                    } else if (this.model.virtualScrollMode == "continuous")
                        this._loadInitialData(_query, list);
                } else {
                    if (mapCateg && mapCateg != "") {
                        _query = ej.Query().group(mapCateg);
                        groupedList = ej.DataManager(list).executeLocal(_query);
                        this.model.dataSource = [];
                        for (i = 0; i < groupedList.length; i++) {
                            this.dummyUl.push(ej.buildTag('span.e-ghead', groupedList[i].key)[0]);
                            this._loadlist(groupedList[i].items);
                            this.model.dataSource = this.model.dataSource.concat(groupedList[i].items);
                        }
                    }
                    else {
                        groupedList = ej.DataManager(list).executeLocal(_query);
                        if (groupedList.length > 0) {
                            if (this.model.template != null) {
                                for (i = 0; i < list.length; i++) {
                                    var _dhtmlAttributes = this._getField(list[i], this.mapFld._htmlAttributes);
                                    var _did = this._getField(list[i], this.mapFld._id);
                                    litag = ej.buildTag('li');
                                    if ((_dhtmlAttributes) && (_dhtmlAttributes != "")) litag.attr(_dhtmlAttributes);
                                    if (_did) litag.attr('id', _did);
                                    if (this.model.template) litag.append(this._getTemplatedString(list[i]));
                                    this.dummyUl.push(litag[0]);
                                }
                                this.element.append(this.dummyUl);
                            }
                            else {
                                this.realUllength = 0;
                                this._loadlist(groupedList);
                            }
                        }
                    }
                }
            }
            var proxy = this;
            this._setDimensions();
            this.listContainer.css({ "position": "relative", "height": "" });
            this.listBoxScroller.css({ "height": "", "width": "" });
            this.listContainer.ejScroller({
                height: this.listContainer.height(),
                width: 0,
                scrollerSize: 20,
                scroll: function (e) {
                    if ((e.source == "button" || e.source == "key" || e.source == "wheel" || e.source != "thumb") && proxy.model != null) proxy._onScroll(e);
                },
                thumbEnd: function (e) {
                    if (proxy.model != null && e.originalEvent.target.tagName.toUpperCase() != "LI")
                        proxy._onScroll(e);
                }
            });
            this.scrollerObj = this.listContainer.ejScroller("instance");
            this._setDimensions();
            this.listContainer.css({ 'display': 'none', 'visibility': 'visible' });
            this._checkboxHideShow(this.model.showCheckbox)._checkitems()._showResult();
            if (this.model.virtualScrollMode == "continuous" && !ej.isNullOrUndefined(this.scrollerObj._vScrollbar))
                this.checkToLoadData = this.scrollerObj._vScrollbar._scrollData.scrollable + (this.element.find('li').height() * 3);
        },
        _loadInitialData: function (query, list) {
            var _query = query.clone();
            this.realUllength = 0;
            _query = _query.range(0, this.model.itemsCount ? this.model.itemsCount : parseInt(this.listContainer.height() / this.listitemheight));
            groupedList = ej.DataManager(list).executeLocal(_query);
            this._loadlist(groupedList);
        },
        _loadlist: function (sublist) {
            if (this.element != null) {
                var selectionArray = [];
                for (var j = 0; j < sublist.length; j++) {
                    var _did = this._getField(sublist[j], this.mapFld._id);
                    var _dimageUrl = this._getField(sublist[j], this.mapFld._imageUrl);
                    var _dimageAttributes = this._getField(sublist[j], this.mapFld._imageAttributes);
                    var _dspriteCss = this._getField(sublist[j], this.mapFld._spriteCSS);
                    var _dtext = this._getField(sublist[j], this.mapFld._text);
                    var _dvalue = this._getField(sublist[j], this.mapFld._value);
                    var _dhtmlAttributes = this._getField(sublist[j], this.mapFld._htmlAttributes);
                    var _dselectBy = this._getField(sublist[j], this.mapFld._selectBy);
                    var _dcheckBy = this._getField(sublist[j], this.mapFld._checkBy);
                    var _dtooltipText = this._getField(sublist[j], this.mapFld._tooltipText);
                    var k = this.realUllength;
                    if ((_dvalue) && (_dvalue != "")) litag = ej.buildTag('li', "", "", {
                        value: _dvalue, style: "height:20px;"
                    });
                    else litag = ej.buildTag('li', null, null, { style: "height:20px;" }); if (_did) litag.attr('id', _did);
                    if ((_dimageUrl) && (_dimageUrl != "")) {
                        imgtag = ej.buildTag('img.e-align', '', {}, {
                            'src': _dimageUrl,
                            'alt': _dtext
                        });
                        if ((_dimageAttributes) && (_dimageAttributes != "")) imgtag.attr(_dimageAttributes);
                        litag.append(imgtag);
                    }
                    if ((_dspriteCss) && (_dspriteCss != "")) {
                        divtag = ej.buildTag('div.e-align ' + _dspriteCss + ' sprite-image');
                        litag.append(divtag);
                    }
                    if ((_dtext) && (_dtext != "")) litag.append(_dtext);
                    if ((_dhtmlAttributes) && (_dhtmlAttributes != "")) litag.attr(_dhtmlAttributes);
                    if ((_dtooltipText) && (_dtooltipText != "")) litag.attr('title', _dtooltipText);
                    if (_dcheckBy) litag.addClass("checkItem");
                    if (_dselectBy) litag.addClass("selectItem");
                    if (this.element.children()[k] != null && this.model.allowVirtualScrolling && $(this.element.children()[k]).text() == "") {
                        $(this.element.children()[k]).replaceWith(litag[0]);
                    } else this.dummyUl.push(litag[0]);
                    this.realUllength += 1;
                }
                if (!this.model.allowVirtualScrolling) this.element.children().remove();
                if (this.element.children()[k] == null && (!this.model.allowVirtualScrolling || this.model.virtualScrollMode == ej.VirtualScrollMode.Continuous) && this._loadContent)
                    this.element.append(this.dummyUl);
                this.element.find('.selectItem').each(function (i, e) {
                    selectionArray.push($(e).index());
                });
                var proxy = this;
                this.element.find('.checkItem').each(function (i, e) {
                    proxy.model.checkedIndices.push($(e).index());
                });
                this.model.allowMultiSelection ? this.model.selectedIndices = selectionArray : this.model.selectedIndex = selectionArray[0];
                this._loadContent = true;
            }
            return this;
        },
        _applySelection: function () {
            if (!(this.model.fields.checkBy || this.model.fields.selectBy)) return false;
            if (this.model.showCheckbox) {
                this.uncheckAll();
                this.checkItemsByIndices(this.model.checkedIndices);
            }
            else {
                if (this.model.allowMultiSelection)
                    this.selectItemsByIndices(this.model.selectedIndices);
                else {
                    this.unselectAll();
                    this.selectItemByIndex(this.model.selectedIndex);
                }
            }
        },
        _getField: function (obj, fieldName) {
            return ej.pvt.getObject(fieldName, obj);
        },
        _getTemplatedString: function (list) {
            var str = this.model.template,
                start = str.indexOf("${"),
                end = str.indexOf("}");
            while (start != -1 && end != -1) {
                var content = str.substring(start, end + 1);
                var field = content.replace("${", "").replace("}", "");
                str = str.replace(content, this._getField(list, field));
                start = str.indexOf("${"), end = str.indexOf("}");
            }
            return str;
        },
        _checkboxHideShow: function (value) {
            (value) ? this._createCheckbox() : this._removeCheckbox();
            this.model.showCheckbox = value;
            return this;
        },
        _createCheckbox: function () {
            var i, _extchk, chklist, me = this;
            this._listitems = this.listContainer.find("ol,ul").children("li");
            chklist = this._listitems.find('input[type=checkbox]');
            for (i = 0; i < this._listitems.length; i++) {
                $checkbox = ej.buildTag("input.listcheckbox e-align#popuplist" + i + "_" + this._id, "", {}, {
                    type: "checkbox",
                    name: "list" + i
                });
                if (!$(this._listitems[i]).find('input[type=checkbox]').length)
                    $(this._listitems[i]).prepend($checkbox);
            }
            this.listContainer.find(".listcheckbox").ejCheckBox({
                cssClass: this.model.cssClass,
                change: $.proxy(this._onClickCheckList, this)
            });
            for (i = 0; i < this._listitems.length; i++) {
                var checkbox = $(this._listitems[i]).find(".listcheckbox");
                if ($(this._listitems[i]).hasClass('e-disable'))
                    checkbox.ejCheckBox('disable');
                else if ($(this._listitems[i]).hasClass('checkitem')) {
                    checkbox.ejCheckBox({
                        "checked": true
                    });
                    this._activeItem = i;
                    this.checkedStatus = true;
                    if (!this._initValue) this._trigger('checkChange', this._getItemObject($(this._listitems[i])));
                    $(this._listitems[i]).removeClass('checkItem');
                }
            }
        },
        _removeCheckbox: function () {
            var i, checkbox;
            this.listitem = this.listContainer.find("ol,ul").children("li");
            checkbox = this.listitem.find('.listcheckbox');
            if (checkbox.length > 0) {
                this.listitem.find('.listcheckbox').ejCheckBox('destroy');
                this.listitem.find('input[type=checkbox]').remove();
                this._checkedItems = [];
            }
        },
        _selectCheckedItem: function (chkitems) {
            if (chkitems.length > 0)
                for (i = 0; i < chkitems.length; i++)
                    this._selectedItems.push(chkitems[i]);
        },
        _refreshScroller: function () {
            if (this.model.virtualScrollMode == "continuous") {
                this.listContainer.find(".e-content, .e-vhandle,.e-vhandle div").removeAttr("style");
                this.listContainer.css({ "display": "block" });
                if (this.scrollerObj) {
                    this.scrollerObj.model.height = this.listContainer.height();
                    this.scrollerObj.refresh();
                    this.scrollerObj.option("scrollTop", 0);
                }
            } else {
                this.listContainer.find(".e-vhandle div").removeAttr("style");
                var listboxcontent = this.listBoxScroller.height();
                this.listContainer.css({ "display": "block" });
                if (this.scrollerObj) {
                    this.scrollerObj.model.height = this.listContainer.height();
                    this.scrollerObj.refresh();
                }
                this.listBoxScroller.css("height", "100%");
            }
            if (!this.model.enabled) {
                if (this.scrollerObj) this.scrollerObj.disable();
                if (this.model.showCheckbox) this.element.find(".listcheckbox").ejCheckBox("disable");
            }
        },
        _setDimensions: function () {
            this.listContainer.css({ "width": this.model.width, "height": this.model.height });
            this._refreshScroller();
            return this;
        },
        _setItemsCount: function () {
            if (this.model.itemsCount && this.model.itemsCount != 0)
                this.model.height = this.model.itemsCount * 30;
            return this;
        },
        _refreshContainer: function () {
            this.listContainer.css({ "position": "relative" });
            this._setDimensions()._roundedCorner()._refreshScroller();
        },
        _drag: function () {
            var proxy = this,
                pre = false,
                _clonedElement = null,
                dragContainment = null;
            this._listitem = this.element.parent();
            lilist = this._addItemIndex ? $($(this._listitem).find("li")[this._addItemIndex]) : $(this._listitem).find("li");
            lilist.not(".e-js").ejDraggable({
                dragArea: dragContainment,
                clone: true,
                dragStart: function (args) {
                    if (!$(args.element.closest('.e-ddl-popup.e-js')).hasClass('e-disable') && !args.element.hasClass('e-disable')) {
                        var draggedobj = $("#" + this.element.parent()[0].id).data("ejListBox");
                        draggedobj._refreshItems();
                        var dragEle = proxy.getSelectedItems();
                        if (proxy._onDragStarts(dragEle.length > 1 ? dragEle : proxy._getItemObject(args.element))) {
                            args.cancel = true;
                            _clonedElement && _clonedElement.remove();
                            return false;
                        }
                    } else {
                        _clonedElement && _clonedElement.remove();
                        return false;
                    }
                },
                drag: function (args) {
                    var target = args.target;
                    var dragEle = proxy.getSelectedItems();
                    if (proxy._onDrag(dragEle.length > 1 ? dragEle : proxy._getItemObject(args.element))) return false;
                    if ($(target).hasClass('e-droppable') || $(target).parent().hasClass('e-droppable'))
                        $(target).addClass("allowDrop");
                },
                dragStop: function (args) {
                    if (!args.element.dropped)
                        _clonedElement && _clonedElement.remove();
                    var target = args.target, targetObj = proxy;
                    var position = pre ? "Before" : "After";
                    var dragEle = proxy.getSelectedItems();
                    if (proxy._onDragStop(dragEle.length > 1 ? dragEle : proxy._getItemObject(args.element))) return false;
                    $(args.element).removeClass("e-active");
                    if (target.nodeName == 'UL') target = $(target).children()[0];
                    if ($(target).closest('li').length) target = $(args.target).closest('li')[0];
                    else if (target.nodeName != 'LI') target = $(target).closest('.e-ddl-popup.e-droppable')[0];
                    if (target && target.nodeName == 'LI' && $(target).hasClass('e-droppable') && $(target).closest('.e-ddl-popup.e-droppable').length) proxy._dropItem(target, args.element, pre, args.event);
                    else if ($(target).hasClass('e-droppable') && $(target).closest('.e-ddl-popup.e-droppable').length) proxy._dropItemContainer(target, args.element, args.event);
                    $(".allowDrop").removeClass("allowDrop");
                    if (args.target != proxy.element[0] && (args.element.parent().length && $(args.element.parent()[0]).data().ejWidgets[0] == "ejListBox")) {
                        proxy = $("#" + args.element.parent()[0].id).data($(args.element.parent()[0]).data().ejWidgets[0]);
                        if (proxy._onDropped(dragEle.length > 1 ? dragEle : proxy._getItemObject(args.element))) return false;
                    }
                },
                helper: function (event, ui) {
                    if (!ej.isNullOrUndefined(event.element) && !$(event.element.closest('.e-ddl-popup.e-js')).hasClass('e-disable') && $(event.element).hasClass('e-draggable')) {
                        proxy = $(event.element).closest('.e-listbox.e-js').data('ejListBox');
                        proxy._tempTarget = $(event.element).text();
                        if (proxy) {
                            _clonedElement = $(event.sender.target).clone().addClass("dragClone dragClonelist");
                            return _clonedElement.appendTo($("body"));
                        }
                    }
                }
            });
        },
        _dropItem: function (target, element, pre, event) {
            var targetid = $(target).closest('.e-ddl-popup.e-droppable')[0].id.replace('_container', '');
            var dataIndex = [], dataObj = [];
            var droppedobj = $("#" + targetid).data("ejListBox");
            var preventDrop = (droppedobj.model.showCheckbox ? !this.model.showCheckbox : this.model.showCheckbox) || (ej.DataManager && this.model.dataSource instanceof ej.DataManager);
            if (preventDrop) return;
            var data = this._getDropObject(target, element, event);
            dataIndex = data.dataIndex;
            dataObj = data.dataObj;
            pre ? $(li).insertBefore(target) : $(li).insertAfter(target);
            this._refreshItems();
            if (dataObj)
                this._dropDataSource(droppedobj, dataIndex, dataObj, li.index());
            droppedobj._refreshItems();
        },
        _dropItemContainer: function (target, element, event) {
            var targetid = $(target)[0].id.replace('_container', '');
            var droppedobj = $("#" + targetid).data("ejListBox");
            var preventDrop = (droppedobj.model.showCheckbox ? !this.model.showCheckbox : this.model.showCheckbox) || (ej.DataManager && this.model.dataSource instanceof ej.DataManager);
            if (preventDrop) return;
            var dataIndex = [], dataObj = [];
            var data = this._getDropObject(target, element, event);
            dataIndex = data.dataIndex;
            dataObj = data.dataObj;
            li.insertAfter($($(target).find('li')).last());
            $(target).find('ul').append(li);
            this._refreshItems();
            if (dataObj)
                this._dropDataSource(droppedobj, dataIndex, dataObj, droppedobj.model.dataSource ? droppedobj.model.dataSource.length : 0);
            if (!droppedobj.model.allowDrag)
                $(li).ejDraggable("instance")._destroy();
            droppedobj._refreshItems();
        },
        _dropDataSource: function (droppedobj, dataIndex, dataObj, droppedIndex) {
            if (dataIndex instanceof Array) {
                var proxy = this;
                $.each(dataObj, function (index) {
                    indx = proxy.model.dataSource.indexOf(dataObj[index]);
                    proxy.model.dataSource.splice(indx, 1);
                });
            }
            else
                this.model.dataSource.splice(dataIndex, 1);
            if (droppedobj.model.dataSource instanceof Array) {
                droppedobj.model.dataSource.splice.apply(droppedobj.model.dataSource, [droppedIndex, 0].concat(dataObj));
            }
            else {
                droppedobj.model.dataSource = dataObj;
            }
        },
        _getDropObject: function (target, element, event) {
            var dataIndex = [], dataObj = [];
            if (this.model.allowMultiSelection) {
                li = $(element).parent().find(".e-select").removeClass("e-select e-hover");
                if (!li.length)
                    li = element.removeClass("e-select e-hover");
            }
            else
                li = element.removeClass("e-select e-hover");

            if (li.length) {
                proxy = this;
                $.each(li, function (ele) {
                    dataIndex.push($(this).index());
                    dataObj.push((proxy.model.dataSource) ? proxy.model.dataSource[$(this).index()] : null);
                });
            }
            else {
                dataIndex = li.index();
                dataObj = (this.model.dataSource) ? this.model.dataSource[dataIndex] : null;
            }
            return { "dataIndex": dataIndex, "dataObj": dataObj };
        },
        _showResult: function () {
            var proxy = this;
            this._refreshContainer();
            this.element.attr({
                "aria-expanded": true
            });
            var _ListItemsContainer = this.element.children("li");
            this._listSize = _ListItemsContainer.size();
            _ListItemsContainer.unbind("contextmenu", $.proxy(this._OnMouseContext, this));
            _ListItemsContainer.unbind("click", $.proxy(this._OnMouseClick, this));
            _ListItemsContainer.unbind("touchstart mouseenter", $.proxy(this._OnMouseEnter, this));
            _ListItemsContainer.unbind("touchend mouseleave", $.proxy(this._OnMouseLeave, this));
            _ListItemsContainer.bind("touchstart mouseenter", $.proxy(this._OnMouseEnter, this));
            _ListItemsContainer.bind("touchend mouseleave", $.proxy(this._OnMouseLeave, this));
            _ListItemsContainer.bind("click", $.proxy(this._OnMouseClick, this));
            _ListItemsContainer.bind("contextmenu", $.proxy(this._OnMouseContext, this));
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            return this;
        },
        _OnWindowResize: function (e) {
            this._refreshContainer();
            this.listContainer.css("display", "block");
        },
        refresh: function (value) {
            if (value || ej.isNullOrUndefined(value)) {
                if (this.model.template)
                    this.element.empty();
                this._checkModelDataBinding(this.model.dataSource);
            }
            else {
                this.listContainer.css({ "height": this.model.height, "width": this.model.width });
                this._refreshScroller();
            }
        },
        _removeListHover: function () {
            this._selectedItems = [];
            this.model.selectedIndices = [];
            this.element.children("li").removeClass("e-hover e-select");
            return this;
        },
        _addListHover: function () {
            this._activeItem = this._selectedItem;
            var activeItem = this._getItem(this._selectedItem);
            activeItem.addClass("e-select e-hover");
            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop() });
            activeItem.focus();
            this._OnListSelect(this.prevselectedItem, this._selectedItem);
        },
        _calcScrollTop: function (value) {
            var ulH = this.element.outerHeight(),
                li = this.element.find("li"),
                liH = 0,
                index, top, i;
            index = value ? value : this.element.find("li.e-select").index();
            for (i = 0; i < index; i++)
                liH += li.eq(i).outerHeight();
            top = liH - ((this.listContainer.outerHeight() - li.eq(index).outerHeight()) / 2);
            return top;
        },
        _refreshItems: function () {
            this.listBoxScroller.append(this.element);
            this.listContainer.append(this.listBoxScroller);
            this._refreshContainer();
            this._showResult();
            this._setSelectionValues();
        },
        _selectedIndices: function () {
            var selectItem;
            this.element.children("li").each(function (index) {
                if ($(this).hasClass("e-select")) {
                    selectItem = index;
                    return false
                }
            });
            this._selectedItem = selectItem;
            return selectItem;
        },
        _addSelectedItem: function (e) {
            if (this.model.disableItemsByIndex != null) {
                if (e.keyCode == 40 || e.keyCode == 39) this._disableItemSelectDown();
                else this._disableItemSelectUp();
                this._selectedItem = this._activeItem
            }
            var activeItem = this._getItem(this._selectedItem);
            this._selectedItems.push(activeItem)
        },
        _getItem: function (val) {
            return $(this.element.children("li")[val])
        },
        _getItemObject: function (item) {
            return {
                item: item,
                index: item.index(),
                text: item.text(),
                value: item.attr("value") ? item.attr("value") : item.text(),
                isEnabled: !item.hasClass("e-disable"),
                isSelected: item.hasClass("e-select"),
                isChecked: item.find(".e-chkbox-wrap").attr("aria-checked"),
                data: this.model.dataSource ? this.getListData()[item.index()] : null
            };
        },
        _roundedCorner: function (val) {
            this.listContainer[(val ? "addClass" : "removeClass")]("e-corner-all");
            return this;
        },
        _enabled: function (boolean) {
            boolean ? this.enable() : this.disable();
            return this;
        },
        _showFullList: function () {
            if (this.model.dataSource != null) {
                if (!(ej.DataManager && this.model.dataSource instanceof ej.DataManager))
                    this.listitems = this.model.dataSource;
            }
            this._renderlistContainer();
            if (!(this.model.dataSource instanceof ej.DataManager)) this._trigger("actionComplete");
            this._addDragableClass()._enableDragDrop();
            return this;
        },
        _cascadeAction: function () {
            if (this.model.cascadeTo) {
                this._currentValue = this._getField(this.listitems[this._activeItem], this.mapFld._value);
                this.selectDropObj = $('#' + this.model.cascadeTo).ejListBox('instance');
                if (ej.isNullOrUndefined(this._dSource))
                    this._dSource = this.selectDropObj.model.dataSource;
                this._performJsonDataInit();
            }
        },
        _performJsonDataInit: function () {
            this._changedSource = ej.DataManager(this._dSource).executeLocal(ej.Query().where(this.mapFld._value, "==", this._currentValue));
            this.selectDropObj.setModel({
                dataSource: this._changedSource,
                enable: true,
                value: ""
            })
        },
        _OnMouseContext: function (e) {
            e.preventDefault();
            return false
        },
        _OnMouseEnter: function (e) {
            this.startime = 0;
            this.item = "";
            if (e.type == "touchstart") {
                this.item = $(e.target).text();
                this.startime = new Date().getTime()
            }
            if (this.model.enabled) {
                var targetEle;
                this.element.children("li").removeClass("e-hover");
                if ($(e.target).is("li")) $(e.target).addClass("e-hover");
                if ($(e.target).hasClass("e-disable")) $(e.target).removeClass('e-hover e-select');
                else if (e.target.tagName != "li") {
                    targetEle = $(e.target).parents("li");
                    $(targetEle).addClass("e-hover")
                }
                var activeItem, selectItem = 0;
                this.element.children("li").each(function (index) {
                    if ($(this).hasClass("e-hover")) {
                        activeItem = index;
                        return false
                    }
                });
                this._activeItem = activeItem
            }
        },
        _OnMouseLeave: function (e) {
            this.element.children("li").removeClass("e-hover");
            this.endtime = new Date().getTime();
            if ((((this.endtime - this.startime) / 200) > 2))
                if ((this.item == $(e.target).text())) this.hold = (((this.endtime - this.startime) / 200) > 2) ? !this.hold : false;
        },
        _OnMouseClick: function (e) {
            if (e.which == 3)
                this.hold = true;
            this.endtime = new Date().getTime();
            if ((((this.endtime - this.startime) / 200) > 2))
                if ((!this.model.template && this.item == $(e.target).text()) && (!this.hold))
                    this.hold = (((this.endtime - this.startime) / 200) > 2);
            if (this.model.enabled && this._activeItem != undefined) {
                if (!this.model.showCheckbox) {
                    this.prevselectedItem = this._lastEleSelect ? this._lastEleSelect : this._selectedIndices();
                    var activeitem = $(this.element.children("li")[this._activeItem]);
                    if (!this.model.allowMultiSelection || (!(e.ctrlKey || this.touchhold || this.hold) && !e.shiftKey))
                        this._removeListHover();
                    this.element.children("li").removeClass('e-hover');
                    if (!activeitem.hasClass('e-select')) {
                        activeitem.addClass('e-select');
                        this._selectedItems.push(activeitem);
                        this.model.selectedIndices.push(this._activeItem);
                        if (e.shiftKey && (this.model.allowMultiSelection)) {
                            if (!e.ctrlKey) this._removeListHover();
                            var initial, last;
                            if (this.prevselectedItem < this._activeItem)
                                initial = this.prevselectedItem, last = this._activeItem;
                            else
                                initial = this._activeItem, last = this.prevselectedItem;
                            for (var i = initial; i <= last; i++) {
                                activeitem = $(this.element.children("li")[i]);
                                if (!activeitem.hasClass('e-disable')) {
                                    if (!activeitem.hasClass('e-select'))
                                        activeitem.addClass('e-select');
                                    this._selectedItems.push(activeitem);
                                    this.model.selectedIndices.push(this._activeItem);
                                }
                            }
                        }
                    } else {
                        activeitem.removeClass('e-select');
                        this._selectedItems.splice(this.model.selectedIndices.indexOf(this._activeItem), 1);
                    }
                    this._selectedItem = this._selectedIndices();
                    this._cascadeAction();
                    var selecteditem = $(this.element.children("li")[this._selectedItem]);
                    if ($(selecteditem).text() != "") {
                        this.element.val($(selecteditem).text());
                        this.element.attr({
                            "value": this.element.val()
                        });
                    }
                    this.model.selectedText = activeitem.text();
                    this._selectedData = this._getItemObject($(selecteditem));
                    if (this._prevSelectedData && (this._selectedData.text != this._prevSelectedData.text))
                        this._trigger("unselect", this._prevSelectedData)
                    this._trigger("select", this._selectedData);
                    this._prevSelectedData = this._selectedData;
                    this._lastEleSelect = this._activeItem;
                    if (this._selectedItems && this._selectedItems.length != 1)
                        this._ctrlClick = true;
                } else {
                    if (($(e.currentTarget).is("li")) && ($(e.target).is("li"))) {
                        if ($(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('isChecked')) {
                            $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', false);
                            var index = this.model.checkedIndices.indexOf(e.currentTarget);
                            this._checkedItems.splice(index, 1);
                            this.model.checkedIndices.splice(index, 1);
                            this.checkedStatus = false;
                        } else {
                            $(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                            this._checkedItems.push(this._activeItem);
                            this.model.checkedIndices.push(e.currentTarget);
                            this.checkedStatus = true;
                        }
                    } else if ($(e.currentTarget).is("li")) {
                        if (!($(e.currentTarget.firstChild).find('.listcheckbox').ejCheckBox('isChecked'))) {
                            if (!($(e.target).is("[class^='e-chk-']")))
                                $(e.currentTarget.firstChild).find('.listcheckbox').prop("checked", true);
                            this._checkedItems.push(this._activeItem);
                            this.model.checkedIndices.push(e.currentTarget);
                            this.checkedStatus = true;
                        } else {
                            if (!($(e.target).is("[class^='e-chk-']")))
                                $(e.currentTarget.firstChild).find('.listcheckbox').prop("checked", false);
                            var index = this.model.checkedIndices.indexOf(e.currentTarget);
                            this._checkedItems.splice(index, 1);
                            this.model.checkedIndices.splice(index, 1);
                            this.checkedStatus = false;
                        }
                    } else
                        return false;
                    this.selectedTextValue = $(e.currentTarget).text();
                    if (!this.element.hasClass("e-disable") && $(e.target).is("li")) {
                        var args = {
                            status: this.model.enabled,
                            isChecked: this.checkedStatus,
                            selectedTextValue: this.selectedTextValue
                        };
                        this._trigger("checkChange", this._getItemObject($(e.target)));
                    }
                    this._lastEleSelect = $(e.currentTarget).index();
                }
                this._setSelectionValues()._OnListSelect(this.prevselectedItem, this._activeItem);
            }
            if (e.target.nodeName != "INPUT")
                this.listContainer.focus();
        },
        _setSelectionValues: function () {
            var selectionArray = []
            this.model.selectedIndices = [];
            this.model.checkedIndices = [];
            var proxy = this;
            if (!this.model.showCheckbox) {
                var liItem = this.element.children("li");
                this.element.children("li.e-select").each(function (index, ele) {
                    selectionArray.push($(ele).attr("value") ? $(ele).attr("value") : $(ele).text());
                    proxy.model.selectedIndices.push(liItem.index(ele));
                    proxy.model.selectedIndex = liItem.index(ele);
                });
            }
            else {
                this.element.find("li .listcheckbox:checked").closest('li').each(function (index, ele) {
                    selectionArray.push($(ele).attr("value") ? $(ele).attr("value") : $(ele).text());
                    proxy.model.checkedIndices.push($(ele).index());
                });
            }
            this.model.selectedItemIndex = this.model.selectedIndex;
            this.model.selectedItems = this.model.selectedItemlist = this.model.selectedIndices;
            this.model.checkedItems = this.model.checkedItemlist = this.model.checkItemsByIndex = this.model.checkedIndices;
            this.value(selectionArray.toString());
            this._hiddenInput.val(this.value());
            return this;
        },
        _onClickCheckList: function (e) {
            this.checkedStatus = e.isChecked ? true : false;
            if (!this._initValue) this._trigger('checkChange', this._getItemObject($(e.event.target).closest('li')));
        },
        _disableItemSelectUp: function () {
            var disableList = this.model.disableItemsByIndex.split(",").sort().reverse();
            if (this._activeItem == 0) this._activeItem = this.listitems.length - 1;
            else this._activeItem--;
            for (var lists = 0;
                ($.inArray(this._activeItem.toString(), disableList)) > -1; lists++) {
                this._activeItem--;
                if (this._activeItem == 0) this._activeItem = this.listitems.length - 1
            }
            $(this.element.children("li")[this._activeItem]).addClass('e-select')
        },
        _disableItemSelectDown: function () {
            var disableList = this.model.disableItemsByIndex.split(",").sort();
            if ((this.listitems.length - 1) == this._activeItem) this._activeItem = 0;
            else this._activeItem++;
            for (var lists = 0;
                ($.inArray(this._activeItem.toString(), disableList)) > -1; lists++) {
                this._activeItem++;
                if ((this.listitems.length) == this._activeItem) this._activeItem = 0
            }
            $(this.element.children("li")[this._activeItem]).addClass('e-select')
        },
        _checkitems: function () {
            if (!this.model.enabled) return this;
            if (this.model.showCheckbox) {
                for (i = 0; i < this.model.checkedIndices.length; i++) {
                    if ($.inArray(this.model.checkedIndices[i], this.listitems) > -1) {
                        var item = $.inArray(this.model.checkedIndices[i], this.listitems);
                        if (!($(this.listitems[item].firstChild).find('.listcheckbox').ejCheckBox('isChecked'))) {
                            $(this.listitems[item].firstChild).find('.listcheckbox').ejCheckBox('option', 'checked', true);
                            this._checkedItems.push(this.listitems[item])
                        }
                    }
                }
            } else {
                if (this.model.allowMultiSelection) {
                    for (i = 0; i < this.model.selectedIndices.length; i++) {
                        var item = this.model.selectedIndices[i];
                        if (!($(this.listitem[item]).hasClass("e-select"))) {
                            $(this.listitem[item]).addClass("e-select");
                            this._selectedItems.push($(this.listitem[item]));
                        } else {
                            $(this.listitem[item]).removeClass("e-select");
                            this._selectedItems.pop($(this.listitem[item]));
                        }
                    }
                } else {
                    this.unselectAll();
                    if (!($(this.listitem[this.model.selectedIndex]).hasClass("e-select")))
                        $(this.listitem[this.model.selectedIndex]).addClass("e-select");
                    this._selectedItems.push($(this.listitem[this.model.selectedIndex]))
                }
            }
            this._setSelectionValues();
            return this;
        },
        _OnListSelect: function (previtem, selecteditem) {
            if (!ej.isNullOrUndefined(previtem) && previtem != selecteditem && !this.model.showCheckbox)
                this._trigger('change', this._getItemObject($(this.element.find("li")[selecteditem])));
        },
        _OnKeyDown: function (e) {
            if (this.model.enabled) {
                if (this._selectedItems && this._selectedItems.length == 1 && !this.model.showCheckbox)
                    this._lastEleSelect = $(this.element.children("li.e-select")).index();
                this._itemId = null;
                var _ListItemsContainer = this.element.children("li"), proxy = this;
                this._listSize = this.element.children("li").size();
                this.prevselectedItem = this._selectedIndices
                if (!e.shiftKey) this._up = this._down;
                switch (e.keyCode) {
                    case 37:
                    case 38:
                        var liItems = this.element.find("li");
                        var selectedIndex = (this.model.showCheckbox) ? (this._lastEleSelect || 0) : liItems.index(this.element.find("li.e-select"));
                        if (e.shiftKey && this.model.allowMultiSelection && !this.model.showCheckbox) {
                            if (this._lastEleSelect == 0) return false;
                            this._lastEleSelect = (this._ctrlClick) ? this._lastEleSelect - 1 : this._lastEleSelect;
                            selectedIndex = this._lastEleSelect;
                            this._selectedItem = (selectedIndex || selectedIndex == 0) ? (selectedIndex == 0 ? this._listSize - 1 : (this._down ? selectedIndex : selectedIndex - 1)) : 0;
                            for (var i = this._selectedItem; $(_ListItemsContainer[i]).hasClass("e-disable") ; i--)
                                this._selectedItem -= 1;
                            var activeItem = $(_ListItemsContainer[this._selectedItem]);
                            if (activeItem.hasClass("e-select")) {
                                if (this._selectedItem == 0) return;
                                activeItem.removeClass("e-select");
                                this._selectedItems.pop();
                            }
                            else {
                                activeItem.addClass("e-select");
                                this._selectedItems.push(activeItem);
                            }
                            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop(this._selectedItem) });
                            this._up = true;
                            this._down = false;
                            this._ctrlClick = false;
                        }
                        else {
                            this._selectedItem = (selectedIndex || selectedIndex == 0) ? (selectedIndex == 0 ? this._listSize - 1 : selectedIndex - 1) : 0;
                            for (var i = this._selectedItem; $(_ListItemsContainer[i]).hasClass("e-disable") ; i--)
                                this._selectedItem -= 1;
                            this._addSelectedItem(e);
                            $(_ListItemsContainer).removeClass("e-hover e-select");
                            var addClass = (this.model.showCheckbox) ? "e-hover" : "e-select";
                            $(_ListItemsContainer[this._selectedItem]).addClass(addClass);
                            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop(this._selectedItem) });
                        }
                        this._OnListSelect(this._selectedItem + 1, this._selectedItem);
                        this._lastEleSelect = this._selectedItem;
                        this._keyCascade(_ListItemsContainer[this._selectedItem]);
                        this._setSelectionValues();
                        var focusEle = $(_ListItemsContainer[this._selectedItem]).find("input").length ? $(_ListItemsContainer[this._selectedItem]).find("input") : this.listContainer;
                        focusEle.focus();
                        e.preventDefault();
                        return false;
                        break;
                    case 39:
                    case 40:
                        var liItems = this.element.find("li");
                        var selectedIndex = (this.model.showCheckbox) ? (this._lastEleSelect || 0) : liItems.index(this.element.find("li.e-select"));
                        if (e.shiftKey && this.model.allowMultiSelection && !this.model.showCheckbox) {
                            if (this._lastEleSelect == this._listSize - 1) return false;
                            this._lastEleSelect = (this._ctrlClick) ? this._lastEleSelect + 1 : this._lastEleSelect;
                            selectedIndex = this._lastEleSelect;
                            this._selectedItem = (selectedIndex || selectedIndex == 0) ? (selectedIndex == this._listSize - 1 ? 0 : ((this._up || this._ctrlClick) ? selectedIndex : selectedIndex + 1)) : 0;
                            for (var i = this._selectedItem; $(_ListItemsContainer[i]).hasClass("e-disable") ; i++)
                                this._selectedItem += 1;
                            var activeItem = $(_ListItemsContainer[this._selectedItem]);
                            if (activeItem.hasClass("e-select")) {
                                activeItem.removeClass("e-select");
                                this._selectedItems.pop();
                            }
                            else {
                                activeItem.addClass("e-select");
                                this._selectedItems.push(activeItem);
                            }
                            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop(this._selectedItem) });
                            this._up = false;
                            this._down = true;
                            this._ctrlClick = false;
                        }
                        else {
                            this._selectedItem = (selectedIndex || selectedIndex == 0) ? (selectedIndex == this._listSize - 1 ? 0 : selectedIndex + 1) : 0;
                            for (var i = this._selectedItem; $(_ListItemsContainer[i]).hasClass("e-disable") ; i++)
                                this._selectedItem += 1;
                            this._addSelectedItem(e);
                            $(_ListItemsContainer).removeClass("e-hover e-select");
                            var addClass = (this.model.showCheckbox) ? "e-hover" : "e-select";
                            $(_ListItemsContainer[this._selectedItem]).addClass(addClass);
                            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop(this._selectedItem) });
                        }
                        this._OnListSelect(this._selectedItem - 1, this._selectedItem);
                        this._lastEleSelect = this._selectedItem;
                        this._keyCascade(_ListItemsContainer[this._selectedItem]);
                        this._setSelectionValues();
                        var focusEle = $(_ListItemsContainer[this._selectedItem]).find("input").length ? $(_ListItemsContainer[this._selectedItem]).find("input") : this.listContainer;
                        focusEle.focus();
                        return false;
                        break;
                    case 8:
                    case 9:
                    case 13:
                        if (this.model.showCheckbox) {
                            if (this.model.checkedIndices.indexOf(this._selectedItem) < 0)
                                this.checkItemByIndex(this._selectedItem);
                            else
                                this.uncheckItemByIndex(this._selectedItem);
                        }
                        break;
                    case 18:
                    case 33:
                    case 34:
                        break;
                    case 35:
                        if ($(':focus').length && $(':focus')[0].nodeName != "INPUT") {
                            this._OnListSelect(this._selectedItem, this._listSize - 1);
                            this.selectItemByIndex(this._listSize - 1);
                            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop(this._listSize - 1) });
                            if (this.model.showCheckbox) {
                                this._removeListHover();
                                $(_ListItemsContainer[this._listSize - 1]).addClass("e-hover");
                                this._lastEleSelect = this._selectedItem = this._listSize - 1;
                            }
                            this._keyCascade(_ListItemsContainer[this._listSize - 1]);
                            e.preventDefault();
                            return false;
                        }
                        break;
                    case 36:
                        if ($(':focus').length && $(':focus')[0].nodeName != "INPUT") {
                            this._OnListSelect(this._selectedItem, 0);
                            this.selectItemByIndex(0);
                            this.scrollerObj.setModel({ "scrollTop": this._calcScrollTop(0) });
                            if (this.model.showCheckbox) {
                                this._removeListHover();
                                $(_ListItemsContainer[0]).addClass("e-hover");
                                this._lastEleSelect = this._selectedItem = 0;
                            }
                            this._keyCascade(_ListItemsContainer[0]);
                            e.preventDefault();
                            return false;
                        }
                        break;
                }
            }
        },
        _keyCascade: function (obj) {
            this._trigger("select", this._getItemObject($(obj)));
            if (this.model.cascadeTo) {
                this._activeItem = this._selectedItem;
                this._cascadeAction();
            }
        },
        _onScroll: function (e) {
            var proxy = this, liEle = this.element.find('li')[0];
            if (this.model.allowVirtualScrolling && this.model.virtualScrollMode == "normal") {
                this.itemIndex = parseInt(this.scrollerObj.scrollTop() / $(liEle).outerHeight());
                this.realUllength = this.itemIndex;
                var start = this.itemIndex, end;
                if (this.itemIndex > this.listContainer.height() / $(liEle).height() && e.source != "wheel") {
                    end = this.itemIndex + this._getLiCount();
                } else if (this.itemIndex >= 0) {
                    end = parseInt((this.scrollerObj.scrollTop() / $(liEle).outerHeight()) + this._getLiCount());
                }
                if (this.model.dataSource.length == undefined) {
                    if (ej.DataManager && this.model.dataSource instanceof ej.DataManager)
                        this._queryPromise(start, proxy, start + Math.round(this.model.height / this.listitemheight), e);
                }
                else if (this.model.dataSource.length > 0)
                    this._loadQueryData(start, end, proxy);
            } else if (this.model.allowVirtualScrolling && this.model.virtualScrollMode == "continuous") {
                if (this.checkToLoadData >= this.scrollerObj._vScrollbar._scrollData.scrollable - this.scrollerObj.scrollTop()) {
                    this.lastScrollPosition = this.scrollerObj.scrollTop();
                    var start = this.element.children().length;
                    if (this.element.find("li").length < this.listitems.length) {
                        if (this.model.dataSource.length == undefined) {
                            if (ej.DataManager && this.model.dataSource instanceof ej.DataManager)
                                this._queryPromise(start, proxy, start + this.model.itemRequestCount, e);
                        }
                        else if (this.model.dataSource.length > 0) {
                            if (this._isScrollComplete) return;
                            this._loadQueryData(proxy.realUllength, proxy.realUllength + this.model.itemRequestCount, proxy);
                        }
                    }
                }
            }
        },
        _queryPromise: function (start, proxy, end, e) {
            this._trigger('itemRequest', e);
            var mQuery = this.model.query.clone();
            var queryPromise = this.model.dataSource.executeQuery(mQuery.range(start, end));
            this._updateLoadingClass(true);
            queryPromise.done(function (d) {
                proxy.realUllength = start;
                proxy._loadlist(d.result)._checkboxHideShow(proxy.model.showCheckbox)._showResult()._updateLoadingClass();
                proxy._applySelection();
                if (proxy.model.virtualScrollMode == "continuous") {
                    proxy.scrollerObj.refresh();
                    proxy.scrollerObj.option("scrollTop", proxy.lastScrollPosition);
                }
                proxy._trigger("actionSuccess", d);
            }).fail(function (e) {
                proxy._trigger("actionFailure", e);
            }).always(function (e) {
                proxy._trigger("actionComplete", e);
            });
        },
        _loadQueryData: function (start, end, proxy) {
            this._isScrollComplete = (end >= this.listitems.length) ? true : false;
            this._updateLoadingClass(true);
            var tempSrc = this.model.dataSource;
            this._loadlist(tempSrc.slice(start, end))._showResult()._updateLoadingClass();
            proxy._applySelection();
            if (proxy.model.virtualScrollMode == "continuous") {
                proxy.scrollerObj.option("scrollTop", proxy.lastScrollPosition);
                proxy.scrollerObj.refresh();
            }
        },
        _setText: function (text) {
            for (i = 0; i < this.listitems.length; i++)
                if ($(this.element.children("li")[i]).text() == text) this.unselectAll().selectItemByIndex(i);
        },
        _getLiCount: function () {
            return parseInt(this.listContainer.height() / this.element.find("li").height());
        },
        _onDragStarts: function (data) {
            return this._trigger("itemDragStart", { items: data });
        },
        _onDrag: function (data) {
            return this._trigger("itemDrag", { items: data });
        },
        _onDragStop: function (data) {
            return this._trigger("itemDragStop", { items: data });
        },
        _onDropped: function (data) {
            return this._trigger("itemDrop", { items: data });
        },
        _OnKeyPress: function (e) {
            if (this.model.enableIncrementalSearch && this.model.enabled) {
                this._incrementalSearch(this._isMozilla ? e.charCode : e.keyCode)
            }
        },
        _incrementalSearch: function (from) {
            _proxy = this;
            var typedCharacter = String.fromCharCode(from);
            if (this._incqueryString != typedCharacter) this._incqueryString += typedCharacter;
            else this._incqueryString = typedCharacter; if ((this._incqueryString.length > 0) && (this._typeInterval == null)) {
                this._typeInterval = setTimeout(function () {
                    _proxy._incqueryString = "";
                    _proxy._typeInterval = null
                }, _proxy._typingThreshold)
            }
            var list = this.listContainer.find("ol,ul").children("li:not('.e-category')"),
                i, strlen;
            var caseSence = this.model.caseSensitiveSearch,
                str, queryStr = this._incqueryString;
            var querylength = this._incqueryString.length,
                searchflag = false;
            if (!caseSence) queryStr = queryStr.toLowerCase();
            var initialSelection = this._activeItem;
            --initialSelection;
            var startIndex = this._activeItem != list.length - 1 ? (this._activeItem + 1) : 0;
            if (this._incqueryString.length > 1) startIndex = this._activeItem;
            for (var i = startIndex;
                (i < list.length && initialSelection != i) ; i++) {
                str = $.trim($(list[i]).text());
                str = caseSence ? str : str.toLowerCase();
                if (str.substr(0, querylength) === queryStr) {
                    this._removeListHover();
                    this.element.children("li").removeClass('e-active');
                    this._selectedItem = i;
                    this._addListHover();
                    searchflag = true;
                } else if ((i == list.length - 1) && (searchflag == false)) {
                    if (startIndex != 0) {
                        i = -1;
                        ++initialSelection;
                    } else searchflag = true;
                }
                if (searchflag) break;
            }
        },
        _wireEvents: function () {
            this._on(this.listContainer, "keydown", this._OnKeyDown);
            this._on(this.listContainer, "keypress", this._OnKeyPress);
            return this;
        }
    });
    ej.VirtualScrollMode = {
        /** Supports to Virtual Scrolling mode with normal only */
        Normal: "normal",
        /** Supports to Virtual Scrolling mode with continuous only */
        Continuous: "continuous"
    }
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget(
    {
        "ejNumericTextbox": ["ej.NumericTextbox", "e-numerictextbox"],
        "ejPercentageTextbox": ["ej.PercentageTextbox", "e-percentagetextbox"],
        "ejCurrencyTextbox": ["ej.CurrencyTextbox", "e-currencytextbox"]
    },
    {
        element: null,

        model: null,
        validTags: ["input"],
        _setFirst: false,
        type: "editor",
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },

        defaults: {

            width: "",

            height: "",

            value: null,

            name: null,

            htmlAttributes: {},

            minValue: -(Number.MAX_VALUE),

            maxValue: Number.MAX_VALUE,

            incrementStep: 1,

            decimalPlaces: 0,

            validateOnType: false,

            cssClass: "",

            enablePersistence: false,

            showSpinButton: true,

            locale: "en-US",

            enableStrictMode: false,

            showRoundedCorner: false,

            readOnly: false,

            enabled: true,

            enableRTL: false,

            watermarkText: "",

            validationRules: null,

            validationMessage: null,

            groupSeparator: null,

            change: null,

            focusIn: null,

            focusOut: null,

            create: null,

            destroy: null
        },

        dataTypes: {
            minValue: "number",
            maxValue: "number",
            incrementStep: "number",
            decimalPlaces: "number",
            showSpinButton: "boolean",
            enableStrictMode: "boolean",
            showRoundedCorner: "boolean",
            enableRTL: "boolean",
            locale: "string",
            watermarkText: "string",
            cssClass: "string",
            readOnly: "boolean",
            enabled: "boolean",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data",
            validateOnType: "boolean",
            groupSeparator: "string"
        },
        observables: ["value"],

        _setModel: function (jsondata) {
            var validate = false;
            for (var key in jsondata) {
                switch (key) {
                    case "value":
                        if (ej.isNullOrUndefined(jsondata["minValue"]) && ej.isNullOrUndefined(jsondata["maxValue"])) {
                            this._setValue(jsondata[key]);
                            jsondata[key] = this.model.value;
                        }
                        else {
                            this.model.value = this._checkNumValue(jsondata[key]);
                            this._localizedFormat();
                            this._raiseChangeEvent();
                            validate = true;
                        }
                        break;
                    case "enableRTL": this._enableRTL(jsondata[key]); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "validationRules":
                        if (this.model.validationRules != null) {
                            this.element.rules('remove');
                            this.model.validationMessage = null;
                        }
                        this.model.validationRules = jsondata[key];
                        this._setValidation();
                        break;
                    case "validationMessage":
                        this.model.validationMessage = jsondata[key];
                        this._setValidation(); break;
                    case "minValue":
                        if (isNaN(jsondata[key])) return;
                        this.model.minValue = parseFloat(jsondata[key].toFixed(this.model.decimalPlaces));
                        validate = true;
                        break;
                    case "maxValue":
                        if (isNaN(jsondata[key])) return;
                        this.model.maxValue = parseFloat(jsondata[key].toFixed(this.model.decimalPlaces));
                        validate = true;
                        break;
                    case "incrementStep":
                        if (isNaN(jsondata[key])) return;
                        this.model.incrementStep = parseFloat(jsondata[key].toFixed(this.model.decimalPlaces));
                        break;
                    case "enableStrictMode": this.model.enableStrictMode = jsondata[key]; break;
                    case "showSpinButton": this._showSpin(jsondata[key]); break;
                    case "showRoundedCorner": this._roundedCorner(jsondata[key]); break;
                    case "locale": this._setLocalize(jsondata[key]);
                        jsondata[key] = this.model.locale; break;
                    case "decimalPlaces": this._setDecimal(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "readOnly": this._setReadOnly(jsondata[key]); break;
                    case "enabled": if (jsondata[key]) this.enable(); else this.disable(); break;
                    case "watermarkText": this.model.watermarkText = jsondata[key]; this._changeWatermark(jsondata[key]); break;
                    case "htmlAttributes": this._addAttr(jsondata[key]); break;
                    case "groupSeparator":
                        this._checkSeparator(jsondata[key]);
                        jsondata[key] = this.model.groupSeparator;
                        break;
                }
            }
            if (validate) {
                this._validateMinMaxValue(true);
                jsondata["value"] = this.model.value;
                jsondata["maxValue"] = this.model.maxValue;
                jsondata["minValue"] = this.model.minValue;
                if (this.model.minValue != -(Number.MAX_VALUE)) this._startValue = this.model.minValue;
                this.element.attr({ 'aria-valuemin': this.model.minValue, 'aria-valuemax': this.model.maxValue, 'aria-valuenow': this.model.value });
            }
            this._checkSeparator(this.model.groupSeparator);
            this.element.val(this._removeSeparator(this.model.value));
            this._updateSeparator();
            this._checkErrorClass();
        },


        _destroy: function () {
            if (this.wrapper) {
                this.element.insertBefore(this.wrapper);
                this.wrapper.remove();
            }
            if (this._isWatermark) this.element.removeAttr("placeholder");
            this.element.val("").removeClass('e-input').empty();
            this.element.css("display", "block");
        },



        _init: function (options) {
            this._options = options;
            if (this.element.is("input") && (this.element.is("input[type=text]") || !this.element.attr('type'))) {
                this._isWatermark = 'placeholder' in document.createElement('input');
                this.model.locale = Globalize.culture(this.model.locale).name == "en" ? "en-US" : Globalize.culture(this.model.locale).name;
                this._prevSeparator = null;
                this._checkSeparator(this.model.groupSeparator);
                this._checkAttribute();
                this._renderControl();
                this._setValues();
                this._wireEvents();
                this._initObjects();
                this._addAttr(this.model.htmlAttributes);
                if (this.model.validationRules != null) {
                    this._initValidator();
                    this._setValidation();
                }
                this._updateSeparator();
            }
            else {
                this._destroy();
                return false;
            }
        },
        _checkAttribute: function () {
            var attr = ["min", "max", "step", "readonly", "disabled", "placeholder"], propName = ["minValue", "maxValue", "incrementStep", "readOnly", "enabled", "watermarkText"], value, propValue;
            for (var i = 0; i < attr.length; i++) {
                value = this.element.attr(attr[i]); propValue = propName[i];
                if ((!ej.isNullOrUndefined(value)) && (ej.isNullOrUndefined(this._options[propValue]))) {
                    if (propValue == "watermarkText") this.model[propValue] = value;
                    else this.model[propValue] = ((propValue != "disabled") && (propValue != "readOnly")) ? Globalize.parseFloat(value, this.model.locale) : propValue == "readOnly" ? this.element.is("[readonly]") : !this.element.is("[disabled]");
                }
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "accesskey") proxy._hiddenInput.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "readOnly" && value == "readOnly") proxy._setReadOnly(true);
                else proxy.wrapper.attr(key, value);
            });
        },

        _setValues: function () {
            this._id = this.element[0].id;
            this._textBox = this._hiddenInput[0];
            this._error = false;
            this._timeout = null;
            this.isValidState = true;
            this._allowkeyboard = true;
            this._validateOnType = false;
            this._focused = false;
            this._startValue = 0;
            if (this.sfType === "ej.CurrencyTextbox" && this.model.minValue == -(Number.MAX_VALUE))
                this.model.minValue = 0;
            this.model.minValue = parseFloat(this.model.minValue.toFixed(this.model.decimalPlaces));
            this.model.maxValue = parseFloat(this.model.maxValue.toFixed(this.model.decimalPlaces));
            if (this.model.minValue != -(Number.MAX_VALUE)) this._startValue = this.model.minValue;
            this._localizedFormat();
            this._validateMinMaxValue(true);
            this._updateSymbol(this.model.locale);
            if(ej.isNullOrUndefined(this.model.value)) value = this.model.value;
            else value = (this.model.value.toString().indexOf('e') == -1) ? this._removeSeparator(this.model.value) : this._convertToExponetial(this.model.value).unformattedValue;
            this.element.val(value);
        },

        _initValidator: function () {
            var proxy = this;
            this.element.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $(error).insertAfter(element.closest(".e-widget"));
                },
            });
        },
        _setValidation: function () {
            var rules = this.model.validationRules;
            this.element.rules("add", rules);
            var validator = this.element.closest("form").validate();
            name = this.element.attr("name");
            validator.settings.messages[name] = {};
            for (var ruleName in rules) {
                var message = null;
                if (!ej.isNullOrUndefined(rules[ruleName])) {
                    if (!ej.isNullOrUndefined(rules["messages"] && rules["messages"][ruleName]))
                        message = rules["messages"][ruleName];
                    else {
                        validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                        for (var msgName in this.model.validationMessage)
                            ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                    }
                    validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                }
            }
        },


        _renderControl: function () {
            this.wrapper = ej.buildTag("span.e-widget ");
            this.innerWrap = ej.buildTag("span.e-in-wrap e-box ");
            this.wrapper.attr("style", this.element.attr("style"));
            if (this.sfType === "ej.NumericTextbox")
                this.wrapper.addClass('e-numeric');
            else if (this.sfType === "ej.PercentageTextbox")
                this.wrapper.addClass('e-percent');
            else if (this.sfType === "ej.CurrencyTextbox")
                this.wrapper.addClass('e-currency');
            this.wrapper.append(this.innerWrap).insertAfter(this.element);
            this.innerWrap.append(this.element);
            this._hiddenInput = ej.buildTag("input", "", {}, { type: "text" }).insertBefore(this.element);
            this._hiddenInput.attr('data-role', 'none');

            this._hiddenInput[0].tabIndex = this.element[0].tabIndex;
            this._hiddenInput[0].style.cssText = this.element[0].style.cssText;
            this._hiddenInput.attr("accesskey", this.element[0].accessKey);
            this.element[0].accessKey = "";

            this._hiddenInput.css("display", "block");
            this.element.css("display", "none");
            if (!this._isWatermark) {
                this._hiddenSpan = ej.buildTag("span.e-input e-placeholder ").insertAfter(this.element);
                this._hiddenSpan.text(this.model.watermarkText);
                this._hiddenSpan.css("display", "none");
                this._hiddenSpan.bind("mousedown", $.proxy(this._focusIn, this));
                this._hiddenSpan.bind('mousewheel', $.proxy(this._mouseWheel, this));
                this._hiddenSpan.bind('DOMMouseScroll', $.proxy(this._mouseWheel, this));
            }
            this.model.name = this.element.attr("name") != null ? this.element.attr("name") : (this.model.name != null ? this.model.name : this.element[0].id);
            this.element.attr("name") == null ? this.element.attr("name", this.model.name) : "";
            this.element.addClass('e-input');
            if (ej.isNullOrUndefined(this.model.value) && this.element[0].value != "")
                this.model.value = this._checkNumValue(this.element[0].value);
            else
                this.model.value = this._checkNumValue(this.model.value);
            this._hiddenInput.attr({ 'value': this.model.value }).addClass('e-input');
            this.element.attr({ 'role': 'spinbutton', 'aria-valuemin': this.model.minValue, 'aria-valuemax': this.model.maxValue, 'aria-valuenow': this.model.value, 'tabindex': '0', 'aria-live': 'assertive', "value": this.model.value });
            var spinbutton = $('<span class="e-select"><span class="e-spin e-spin-up " role="button" "aria-label"="Increase Value" unselectable="on" /><span class="e-spin e-spin-down" role="button" "aria-label"="Decrease Value" unselectable="on" /></span>');
            var spinimg = ej.buildTag('span.e-icon e-arrow').attr({ 'role': 'presentation', 'unselectable': 'on' });
            this.innerWrap.append(spinbutton);
            spinbutton.find('.e-spin').append(spinimg);
            this.spin = this.wrapper.find('.e-select');
            this.spinUp = this.wrapper.find('.e-spin-up');
            this.spinDown = this.wrapper.find('.e-spin-down');
            this._setWidth(this.model.width);
            this._setHeight(this.model.height);
            if (this.model.cssClass != "") this._setSkin(this.model.cssClass);
            this._showSpin(this.model.showSpinButton);
            if (this.model.showRoundedCorner) this._roundedCorner(this.model.showRoundedCorner);
            if (this.model.enableRTL) this._enableRTL(this.model.enableRTL);
            if (this.model.readOnly) this._setReadOnly(this.model.readOnly);
            if (!this.model.enabled) this.disable();
            else if (this.model.enabled && this.element.hasClass("e-disable")) this.enable();
        },


        _initObjects: function () {
            this._preVal = this._numberValue();
            if (this.sfType === "ej.PercentageTextbox" && this._textBox.value != "")
                this._appendPercentSymbol(this._textBox.value);
            else if (this.sfType === "ej.CurrencyTextbox" && this._textBox.value != "")
                this._appendCurrencySymbol(this._textBox.value);
            if (this.model.value === null) {
                this.isValidState = true;
                this._hiddenInput.val(null);
            }
            else if ((this.model.value < this.model.minValue) || (this.model.value > this.model.maxValue))
                this.isValidState = false;
            this._checkErrorClass();
            this._setWaterMark();
        },


        _showSpin: function (value) {
            if (!value) {
                if (this.spin) {
                    this.spin.hide();
                    this.innerWrap.removeClass('e-padding');
                }
                this._spinEvents("_off");
            }
            else {
                if (this.spin) {
                    this.spin.show();
                    this.innerWrap.addClass('e-padding');
                }
                this._spinEvents("_on");
            }
        },


        _roundedCorner: function (value) {
            if (value && !this.innerWrap.hasClass('e-corner')) {
                this.innerWrap.addClass('e-corner');
            } else if (this.innerWrap.hasClass('e-corner')) {
                this.innerWrap.removeClass('e-corner');
            }
        },


        _enableRTL: function (enableRTL) {
            if (enableRTL) {
                if (this.spin) {
                    this.wrapper.addClass("e-rtl");
                }
                else this.element.addClass("e-rtl");
            }
            else {
                if (this.spin) {
                    this.wrapper.removeClass("e-rtl");
                }
                else this.element.removeClass("e-rtl");
            }
        },


        _setWidth: function (value) {
            value != "" ? this.wrapper.width(value) : this.model.width = this.wrapper.outerWidth();
        },


        _setHeight: function (value) {
            value != "" ? this.wrapper.height(value) : this.model.height = this.wrapper.outerHeight();
        },


        _setSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass);
            this.wrapper.addClass(skin);
        },


        _setValue: function (value) {
            if (!this._isWatermark)
                this._hiddenSpan.css("display", "none");
            this.model.value = this._checkNumValue(value);
            this._validateMinMaxValue(false);
            if ((this.model.minValue <= this.model.value) && (this.model.value <= this.model.maxValue)) {
                this.isValidState = true;
            } else
                this.isValidState = false;
            this._checkErrorClass();
            this._localizedFormat();
            this._raiseChangeEvent();
            this._setWaterMark();
        },

        _setLocalize: function (val) {
            var prevSeparator = Globalize.culture(this.model.locale).numberFormat[',']
            this.model.locale = Globalize.culture(val).name == "en" ? "en-US" : Globalize.culture(val).name;
            this.model.groupSeparator = ((ej.isNullOrUndefined(this._options.groupSeparator) && (prevSeparator === this.model.groupSeparator))) ?
                Globalize.culture(this.model.locale).numberFormat[','] : this.model.groupSeparator;
            this._updateSymbol(this.model.locale);
            this._localizedFormat();
            value = (this.model.value.toString().indexOf('e') == -1) ? this._formatValue(this.model.value, "n") : this._convertToExponetial(this.model.value).unformattedValue;
            this.element.val(value);
        },
        _updateSymbol: function (locale) {
            this._percentSymbol = Globalize.culture(locale).numberFormat.percent.symbol;
            this._currencySymbol = Globalize.culture(locale).numberFormat.currency.symbol;
        },

        _setDecimal: function (val) {
            this.model.decimalPlaces = val;
            this._localizedFormat();
        },


        _validateMinMaxValue: function (fromMinMax) {
            var valChange = false, value;
            if (this.model.minValue > this.model.maxValue) this.model.minValue = this.model.maxValue;
            if (this.model.value !== "" && this.model.minValue > this.model.value) {
                if (this.model.enableStrictMode != true) {
                    this.isValidState = true;
                    this._startValue = this.model.value = this.model.minValue;
                }
                else this.isValidState = false;
                valChange = true;
            }
            else if (this.model.value !== "" && this.model.maxValue < this.model.value) {
                if (this.model.enableStrictMode != true) {
                    this.isValidState = true;
                    this.model.value = this.model.maxValue;
                }
                else this.isValidState = false;
                valChange = true;
            }
            else this.isValidState = true;
            if (this.model.minValue == this.model.maxValue) this._startValue = this.model.minValue;
            if ((valChange && fromMinMax)) {
                value = this._formatValue(this.model.value, "n");
                this._hiddenInput.val(value);
                value = (this.model.value.toString().indexOf('e') == -1) ? this._formatValue(this.model.value, "n") : this._convertToExponetial(this.model.value).unformattedValue;
                this.element.val(value);
                this._localizedFormat();
                this._raiseChangeEvent();
            }
        },

        _convertToExponetial: function (value) {
            var number = "", pattern, format;
            var negative = value < 0 ? true : false;
            value = value.toString();
            format = Globalize.culture(this.model.locale).numberFormat;
            value = value.replace(".", format["."]);
            unformattedValue = value;
            var length = null;
            if (this.sfType === "ej.NumericTextbox") {
                pattern = negative ? format.pattern[0] : "n";
                value = negative ? value.replace("-", "") : value;
                symbol = "";
            }
            else if (this.sfType === "ej.PercentageTextbox") {
                pattern = negative ? format.percent.pattern[0] : format.percent.pattern[1];
                value = negative ? value.replace("-", "") : value;
                symbol = format.percent.symbol;
            }
            else if (this.sfType === "ej.CurrencyTextbox") {
                pattern = negative ? format.currency.pattern[0] : format.currency.pattern[1];
                value = negative ? value.replace("-", "") : value;
                symbol = format.currency.symbol;
            }
            for (idx = 0, length = pattern.length; idx < length; idx++) {
                ch = pattern.charAt(idx);
                (ch === "n") ? number += value : (ch === "$" || ch === "%") ? number += symbol : number += ch;
            }
            return { formattedValue: number, unformattedValue: unformattedValue }
        },

        _localizedFormat: function () {
            if (ej.isNullOrUndefined(this.model.value)) {
                this._textBox.value = "";
                return;
            }
            if (this.model.value.toString().indexOf('e') == -1) {
                this._textBox.value = this._removeSeparator(this.model.value)
                if (!this._focused && this._textBox.value != "") {
                    this._textBox.value = Globalize.format((this.model.value), "n" + this.model.decimalPlaces, this.model.locale);
                    if (this.sfType === "ej.PercentageTextbox") this._appendPercentSymbol(this._textBox.value);
                    else if (this.sfType === "ej.CurrencyTextbox") this._appendCurrencySymbol(this._textBox.value);
                }
            }
            else {
                var value = this._convertToExponetial(this.model.value);
                this._textBox.value = value.unformattedValue;
                if (!this._focused && this._textBox.value != "")
                    this._textBox.value = value.formattedValue;
            }
        },


        _checkNumValue: function (value) {
            if (typeof value == "string")
                value = !this._changeSeparator ? value : this._replaceSeparator(value, this.model.groupSeparator, Globalize.culture(this.model.locale).numberFormat[',']);
            if (typeof value == "string" && !isNaN(Globalize.parseFloat(value, this.model.locale))) {
                value = Globalize.parseFloat(value, this.model.locale);
                return parseFloat(value);
            }
            else if ((typeof value == "number") && !isNaN(value))
                return value;
            else return null;
        },


        _setReadOnly: function (bool) {
            this.model.readOnly = bool;
            if (bool) {
                this.element.attr("readonly", true);
                this._hiddenInput.attr("readonly", true);
            }
            else {
                this.element.removeAttr("readonly");
                this._hiddenInput.removeAttr("readonly");
            }
        },


        _setWaterMark: function () {
            if ((this.model.watermarkText != null) && (this._textBox.value === "") && $.trim(this._hiddenInput.val()) === "") {
                if (this._isWatermark) {
                    this._hiddenInput.attr("placeholder", this.model.watermarkText);
                    this.element.attr("placeholder", this.model.watermarkText);
                }
                else
                    this._hiddenSpan.css("display", "block").text(this.model.watermarkText);
            }
        },


        _changeWatermark: function (text) {
            if (!this.model.enabled) return false;
            if (this._isWatermark) this._hiddenInput.attr("placeholder", text);
            else this._hiddenSpan.text(text);
        },


        _setSelectionRange: function (selectionStart, selectionEnd) {
            var input = this._textBox;
            try {
                if (input.setSelectionRange) {
                    input.setSelectionRange(selectionStart, selectionEnd);
                }
                else if (input.createTextRange) {
                    var range = input.createTextRange();
                    _setselction(range);
                }
            }
            catch (e) {
                var control = this;
                window.setTimeout(function () {
                    document.body.focus();
                    control._textBox.select();
                    var range = document.selection.createRange();
                    _setselction(range);
                }, 1);
            }
            function _setselction(range) {
                range.collapse(true);
                range.moveEnd('character', selectionEnd);
                range.moveStart('character', selectionStart);
                range.select();
            }
        },


        _getSelection: function (value) {
            var oSel = null;
            if (document.selection) {
                oSel = document.selection.createRange();
                return (oSel.text === "" ? oSel.text : this._removeFormats(oSel.text));
            }
            else {
                if (value == null)
                    return this.model.value;
                else {
                    oSel = this._removeFormats(value.substring(this._textBox.selectionStart, this._textBox.selectionEnd));
                    return oSel;
                }
            }
        },


        _caretPosition: function () {
            var oField = this._textBox;
            var iCaretPos = 0;
            // IE Support
            if (document.selection) {
                oField.focus();
                // To get cursor position, get empty selection range
                var oSel = document.selection.createRange();
                // Move selection start to 0 position
                oSel.moveStart('character', -oField.value.length);
                // The caret position is selection length
                iCaretPos = oSel.text.length;
            }
            // Firefox support
            else if (oField.selectionStart || oField.selectionStart == '0')
                iCaretPos = oField.selectionEnd;
            return (iCaretPos);
        },


        _appendPercentSymbol: function (value) {
            if (this._percentSymbol) value = value.replace(this._percentSymbol, "");
            if (value.indexOf(Globalize.culture(this.model.locale).numberFormat.percent.symbol) < 0)
                this._textBox.value = this._formatValue(Globalize.parseFloat(value, this.model.locale) / 100, "p");
            this._percentSymbol = Globalize.culture(this.model.locale).numberFormat.percent.symbol;

        },

        _appendCurrencySymbol: function (value) {
            if (this._currencySymbol) value = value.replace(this._currencySymbol, "");
            if (value.indexOf(Globalize.culture(this.model.locale).numberFormat.currency.symbol) < 0)
                this._textBox.value = this._formatValue(Globalize.parseFloat(value, this.model.locale), "c");
            this._currencySymbol = Globalize.culture(this.model.locale).numberFormat.currency.symbol;

        },

        _removeFormats: function (val) {
            var grpSep = Globalize.culture(this.model.locale).numberFormat[","];
            if (val != null) {
                if (grpSep == ".")
                    return (val.toString().replace(/["."]/g, ""));
                else if (val.toString().match(new RegExp(grpSep, "g")))
                    return (val.toString().replace(new RegExp(grpSep, "g"), ""));
                else
                    return val;
            }
            else
                return null;
        },


        _checkErrorClass: function (obj) {
            if (this.isValidState) this.wrapper.removeClass("e-error");
            else this.wrapper.addClass("e-error");
        },


        enable: function () {
            this.model.enabled = true;
            this.element[0].disabled = false;
            this.element.removeAttr("disabled");
            this._hiddenInput.removeAttr("disabled");
            this.element.removeClass('e-disable').attr({ "aria-disabled": false });
            this._hiddenInput.removeClass('e-disable').attr({ "aria-disabled": false });
            this.wrapper.find(".e-select").removeClass('e-disable').attr({ "aria-disabled": false });
            this.wrapper.find(".e-select span.e-icon.e-arrow").removeClass('e-disable');
        },


        disable: function () {
            this.model.enabled = false;
            this.element[0].disabled = true;
            this.element.attr("disabled", "disabled");
            this._hiddenInput.attr("disabled", "disabled");
            this.element.addClass('e-disable').attr({ "aria-disabled": true });
            this._hiddenInput.addClass('e-disable').attr({ "aria-disabled": true });
            this.wrapper.find(".e-select").addClass('e-disable').attr({ "aria-disabled": true });
            this.wrapper.find(".e-select span.e-icon.e-arrow").addClass('e-disable');
        },


        getValue: function () {
            return this.model.value == null ? "" : this.model.value;
        },

        _wireEvents: function () {
            this._on(this._hiddenInput, 'focus', this._focusIn);
            this._on(this.element, 'blur', this._focusOut);
            this._on(this.element, 'keydown', this._keyDown);
            this._on(this.element, 'keypress', this._keyPress);
            this._on(this.element, 'mousewheel', this._mouseWheel);
            this._on(this.element, 'DOMMouseScroll', this._mouseWheel);
        },

        _spinEvents: function (action) {
            this[action](this.spinUp, "mousedown", this._spinUpClick);
            this[action](this.spinDown, "mousedown", this._spinDownClick);
            this[action](this.spinUp, "mouseup", this._spinUpClick);
            this[action](this.spinDown, "mouseup", this._spinDownClick);
        },

        _spinUpClick: function (event) {
            var self = this;
            event.preventDefault();
            clearTimeout(this._timeout);
            if (!this.model.enabled || this.model.readOnly) return;
            this._on(this.spinUp, 'mouseleave', this._mouseUpClick);
            this.spinUp.addClass("e-active");
            var self = this;
            if (event.type == "mouseup") {
                this._updateInputField("increment");
                this.spinUp.removeClass("e-active");
                this._off($(document), 'mouseup', this._mouseUpClick);
            }
            else if (event.type == "mousedown") {
                if (!this._focused) this._hiddenInput[0].focus();
                this._timeout = setInterval(function () {
                    self._updateInputField("increment");
                }, 150);
                this._on($(document), 'mouseup', this._mouseUpClick);
            }
        },

        _spinDownClick: function (event) {
            var self = this;
            event.preventDefault();
            clearTimeout(this._timeout);
            if (!this.model.enabled || this.model.readOnly) return;
            this._on(this.spinDown, 'mouseleave', this._mouseUpClick);
            this.spinDown.addClass("e-active");
            if (event.type == "mouseup") {
                this._updateInputField("decrement");
                this.spinDown.removeClass("e-active");
                this._off($(document), 'mouseup', this._mouseUpClick);
            }
            else if (event.type == "mousedown") {
                if (!this._focused) this._hiddenInput[0].focus()
                this._timeout = setInterval(function () {
                    self._updateInputField("decrement");
                }, 150);
                this._on($(document), 'mouseup', this._mouseUpClick);
            }
        },

        _mouseUpClick: function (event) {
            event.stopPropagation();
            clearTimeout(this._timeout);
            this._off(this.spinUp, 'mouseleave', this._mouseUpClick);
            this._off(this.spinDown, 'mouseleave', this._mouseUpClick);
            this.spinDown.removeClass("e-active");
            this.spinUp.removeClass("e-active");
        },

        _mouseWheel: function (event) {
            event.preventDefault();
            if (!this._focused) this.element[0].focus();
            if (!this.model.enabled || this.model.readOnly) return;
            var delta;
            var rawEvent = event.originalEvent;
            if (rawEvent.wheelDelta) {
                // IE and Opera use wheelDelta, which is a multiple of 120 (possible values -120, 0, 120).
                delta = rawEvent.wheelDelta / 120;
                // In Opera, value is negated.
                //if (Sys.Browser.agent === Sys.Browser.Opera) delta = -delta;
            }
            else if (rawEvent.detail) {
                // Firefox uses detail property, which is a multiple of 3.
                delta = -rawEvent.detail / 3;
            }
            if (delta > 0)
                this._updateInputField("increment");
            else if (delta < 0)
                this._updateInputField("decrement");
            this._cancelEvent(event);
        },


        _numberValue: function () {
            var value = this._textBox.value;
            if (this.sfType === "ej.PercentageTextbox")
                value = this._formatValue(Globalize.parseFloat(value.split(this._percentSymbol)[0], this.model.locale), "n");
            else if (this.sfType === "ej.CurrencyTextbox")
                value = this._formatValue(Globalize.parseFloat(value, this.model.locale), "n");
            value = (typeof value == "string" && !isNaN(Globalize.parseFloat(value, this.model.locale))) ? value : "";
            return value;
        },
        _formatValue: function (value, format) {
            return Globalize.format(value, format + this.model.decimalPlaces, this.model.locale);
        },
        _toggleTextbox: function (toggle) {
            var proxy = this;
            proxy._hiddenInput.toggle(toggle);
            proxy.element.toggle(!toggle);
        },

        _focusIn: function (event) {
            if (this.model.readOnly)
                return;
            this._focused = true;
            if(!ej.isNullOrUndefined(this.model.value)) this.element.val(this._removeSeparator(this.model.value));
            this._toggleTextbox(false);
            this._textBox = this.element[0];
            this.element[0].focus();
            this._preVal = this._numberValue();
            if (!this._isWatermark)
                this._hiddenSpan.css("display", "none");
            this.wrapper.addClass("e-focus");
            this.wrapper.removeClass('e-error');
            if (!this._error) {
                if (this._textBox.value != "") {
                    var value = this._formatValue(this._textBox.value, "n");
                    this._textBox.value = value;
                    this._hiddenInput.val(value);
                }
                this._setSelectionRange(0, this._textBox.value.length);
            }
            this._trigger("focusIn", { value: this.model.value });
        },

        _focusOut: function (event) {
            this._focused = false;
            this.wrapper.removeClass("e-focus");
            if (!this._error) {
                if (this._textBox.value != "") {
                    var value = Globalize.parseFloat(this._textBox.value, this.model.locale);
                    if (value < this.model.minValue) {
                        if (!this.model.enableStrictMode)
                            this._textBox.value = this._formatValue(this.model.minValue, "n")
                        else
                            this.isValidState = false
                    }
                    else if (value > this.model.maxValue) {
                        if (!this.model.enableStrictMode)
                            this._textBox.value = this._formatValue(this.model.maxValue, "n");
                        else
                            this.isValidState = false;
                    }
                    this.model.value = Globalize.parseFloat(this._textBox.value, this.model.locale);
                    if (isNaN(this.model.value) && !this.model.value) {
                        this.model.value = null;
                        this.isValidState = false;
                    }
                    this._toggleTextbox(true);
                    this._textBox = this._hiddenInput[0];
                    this._localizedFormat();
                    this._checkErrorClass();
                }
                else {
                    var value = this._textBox.value == "" ? null : this._textBox.value;
                    this.model.value = value;
                    this._hiddenInput.val(value);
                    this._toggleTextbox(true);
                    this._textBox = this._hiddenInput[0];
                }
                if (this.model.value === null || this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) this.isValidState = true;
                else if (this.model.enableStrictMode) this.isValidState = false;
                this._raiseChangeEvent();
                this._setWaterMark();
                if (ej.isNullOrUndefined(this.model.value)) value = this.model.value;
                else value = (this.model.value.toString().indexOf('e') == -1) ? this._removeSeparator(this.model.value) : this._convertToExponetial(this.model.value).unformattedValue;
                this.element.val(value);
                this._updateSeparator();
                this._trigger("focusOut", { value: this.model.value });
            }
            this._checkErrorClass();
        },

        _cancelEvent: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            e.stopPropagation();
            e.preventDefault();
            return false;
        },

        _updateInputField: function (operation) {
            this.isValidState = true;
            var step = this.model.incrementStep;
            var itemValue = this.model.value;
            if (this._textBox.value === "") {
                this._textBox.value = this._formatValue(this._startValue, "n");
                step = 0;
            }
            else if (this._textBox.value.indexOf(" ") >= 0)
                this._textBox.value = this._textBox.value.replace(" ", "");
            if (this.sfType === "ej.PercentageTextbox" || this.sfType === "ej.CurrencyTextbox") {
                var editorvalue = this._textBox.value, value = editorvalue;
                if (editorvalue.indexOf(Globalize.culture(this.model.locale).numberFormat.percent.symbol) > -1)
                    value = editorvalue.substring(0, editorvalue.length - 1);
                this.model.value = Globalize.parseFloat(value, this.model.locale);
            }
            else if (this.sfType == "ej.NumericTextbox")
                this.model.value = Globalize.parseFloat(this._textBox.value, this.model.locale);

            if (isNaN(this.model.value) && !this.model.value)
                this.model.value = this._startValue;

            if (this.model.value >= this.model.minValue && this.model.value > this.model.maxValue) {
                this.model.value = this.model.maxValue;
                this._setValue(this.model.value);
            }
            else if (this.model.value < this.model.minValue && this.model.value <= this.model.maxValue) {
                this.model.value = this.model.minValue;
                this._setValue(this.model.value);
            }
            else if (this.model.value >= this.model.minValue && this.model.value <= this.model.maxValue) {
                value = operation == "increment" ? this.model.value + step : this.model.value - step;
                if (value >= this.model.minValue && value <= this.model.maxValue)
                    this._setValue(value);
            }
            this._checkErrorClass();
        },

        _validateDecimal: function (e) {
            var char = String.fromCharCode(e.keyCode);
            char = e.keyCode == 188 ? "," : (e.keyCode == 190) ? "." : (e.keyCode == 110) ? Globalize.culture(this.model.locale).numberFormat["."] : char;
            if (Globalize.culture(this.model.locale).numberFormat["."].charCodeAt(0) == char.charCodeAt(0))
                return true;
            return false;
        },

        _allowKeyCodes: function (e) {
            var keys = new Array(38, 40, 35, 36, 109, 189, 46, 8, 127, 37, 39, 190, 9, 13, 16, 17, 18, 20, 110, 173, 86, 88, 67);
            for (var i = 0; i < keys.length; i++) {
                if (e.keyCode == keys[i] || (this._validateDecimal(e) && this.model.decimalPlaces != 0))
                    return true;
            }
            return false;
        },

        _raiseChangeEvent: function () {
            var currVal = this._numberValue(), value;
            if (!ej.isNullOrUndefined(this._preVal) && (this._checkNumValue(this._preVal) !== this._checkNumValue(currVal))) {
                this._preVal = currVal;
                this.model.value = this._checkNumValue(this._preVal);
                this._updateHiddenField();
                if (ej.isNullOrUndefined(this.model.value)) value = this.model.value;
                else value = (this.model.value.toString().indexOf('e') == -1) ? this._removeSeparator(this.model.value) : this._convertToExponetial(this.model.value).unformattedValue;
                this.element.val(value);
                this.element.attr('aria-valuenow', value);
                this._updateSeparator();
                // Trigger the Jquery change event for the input element.
                this.element.trigger("change");
                this._trigger("change", { value: this.model.value });
                this._trigger("_change", { value: this.model.value });
            }
        },
        _updateHiddenField: function () {
            var prevActive = this._textBox;
            this._textBox = this._hiddenInput[0];
            this._localizedFormat();
            this._textBox = prevActive;
        },
        _removeSeparator: function (number) {
            if (ej.isNullOrUndefined(number)) return;
            var value, format;
            number = this._checkNumValue(Globalize.format((number), "n" + this.model.decimalPlaces, this.model.locale))
            value = number.toString();
            format = Globalize.culture(this.model.locale).numberFormat;
            value = value.replace(".", format["."]);
            return value;
        },
        _updateSeparator: function () {
            if (this._changeSeparator && this.model.value) {
                var formatValue;
                if (this.sfType === "ej.NumericTextbox")
                    formatValue = this._formatValue(this.model.value, "n");
                else if (this.sfType === "ej.PercentageTextbox")
                    formatValue = this._formatValue((this.model.value) / 100, "p");
                else if (this.sfType === "ej.CurrencyTextbox")
                    formatValue = this._formatValue(this.model.value, "c");
                this._hiddenInput.val(this._replaceSeparator(formatValue, Globalize.culture(this.model.locale).numberFormat[","], this.model.groupSeparator));
            }
        },

        _replaceSeparator: function (value, find, replaceWith) {
            var reg = find === "" ? new RegExp("\\s", "g") : new RegExp("\\" + find, "g");
            return value.replace(reg, replaceWith);
        },

        _checkSeparator: function (value) {
            this.model.groupSeparator = value != null ? this._validateSeparator(value) : Globalize.culture(this.model.locale).numberFormat[','];
            this._changeSeparator = Globalize.culture(this.model.locale).numberFormat[","] != this.model.groupSeparator ? true : false;
            this._prevSeparator = this.model.groupSeparator;
        },
        _validateSeparator: function (value) {
            var separator = value.toString();
            var reg = new RegExp("[a-zA-Z0-9]");
            separator = (separator.length > 1) ? separator[0] : separator;
            separator = (!reg.test(separator)) ? separator : this._prevSeparator != null ? this._prevSeparator : Globalize.culture(this.model.locale).numberFormat[','];
            return separator;
        },
        _keyPress: function (e) {
            if (e.which === 0 || e.metaKey || e.ctrlKey || e.keyCode === 8 || e.keyCode === 13)
                return;
            var proxy = this, regExp = null, point = ".";
            cursor = this._caretPosition();
            beforeCursor = this._textBox.value.toString().substring(0, cursor);
            afterCursor = this._textBox.value.toString().substring(cursor);
            numberFormat = Globalize.culture(this.model.locale).numberFormat;

            var character = String.fromCharCode(e.which);
            value = beforeCursor + character + afterCursor;
            var separator = numberFormat[point];
            separator = (separator === point) ? "\\" + separator : separator;
            regExp = proxy.model.decimalPlaces === 0 ? /^(-)?(\d*)$/ : new RegExp("^(-)?(((\\d+(" + separator + "\\d*)?)|(" + separator + "\\d*)))?$")
            // Validate the textbox value 
            if (!regExp.test(value))
                return false;
        },
        _validateDecimalOnType: function (e) {
            var cursor, cancelEvent, beforeCursor, afterCursor, sel;
            cursor = this._caretPosition();
            beforeCursor = this._textBox.value.toString().substring(0, cursor);
            afterCursor = this._textBox.value.toString().substring(cursor);
            sel = this._getSelection(this._textBox.value);
            var decSep = Globalize.culture(this.model.locale).numberFormat["."];
            var temp = this.model.value.toString();
            if (this.model.decimalPlaces != 0) {
                var afterdeci = (this._textBox.value).split(decSep)[1];
                if (temp.indexOf(decSep) > 0) {
                    if ((temp.substring(temp.indexOf(decSep) + 1, temp.length)).length > this.model.decimalPlaces)
                        cancelEvent = true;
                    else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimalPlaces) cancelEvent = true;
                    else cancelEvent = false;
                }
                else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimalPlaces) cancelEvent = true;
                else
                    cancelEvent = false;
            }
            else
                cancelEvent = false;
            if (cancelEvent) {
                this._keypressFlag = false;
                this._cancelEvent(e);
                return false;
            }
        },

        _keyDown: function (e) {
            if (this.model.readOnly) return;
            this._CurrentCultureInfo = JSON.parse(JSON.stringify(Globalize.culture()));
            var cursor, cancelEvent, beforeCursor, afterCursor, sel;
            if (e.keyCode >= 48 && e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105 || this._allowKeyCodes(e)) {
                if (e.shiftKey && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || e.keyCode == 46 || e.keyCode == 127) || (e.ctrlKey && (e.keyCode == 86 || e.keyCode == 118 || e.keyCode == 67 || e.keyCode == 88)))
                    return true;
                if ((((e.ctrlKey == true) && (e.keyCode != 9 && e.keyCode != 17 && e.keyCode != 86 && e.keyCode != 67))) || (e.keyCode == 67 || e.keyCode == 86 || e.keyCode == 88)) {//Prevent Shift + Tab event
                    this._keypressFlag = false;
                    this._cancelEvent(e);
                    return false;
                }

                if (e.keyCode >= 48 && e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105 || e.keyCode == 110) {//Numbers from 0 to 9 in keyboard and numpad (110 for decimal)
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.toString().substring(0, cursor);
                    afterCursor = this._textBox.value.toString().substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                    // For Numpad issue
                    if (e.keyCode >= 96 && e.keyCode <= 105)
                        e.keyCode -= 48;
                    if (this.model.validateOnType) this._validateDecimalOnType(e);
                    if (this._validateOnType) {
                        var decSep = Globalize.culture(this.model.locale).numberFormat["."];
                        var temp = this.model.value.toString();
                        var tempIndex = temp.indexOf(decSep);
                        var value = this.model.value;
                        var replaceminVal = this.model.minValue.toFixed(this.model.decimalPlaces);
                        var replacemaxVal = this.model.maxValue.toFixed(this.model.decimalPlaces);
                        var exp = false;
                        if (replaceminVal.toString().indexOf("e") > 0 || replacemaxVal.toString().indexOf("e") > 0)
                            exp = true;
                        if (!exp) {
                            if (decSep != ".") {
                                temp = this._textBox.value.toString();
                                if (this.model.minValue.toString().match(new RegExp(".", "g")))
                                    replaceminVal = this.model.minValue.toString().replace(/["."]/g, decSep);
                                if (this.model.maxValue.toString().match(new RegExp(".", "g")))
                                    replacemaxVal = this.model.maxValue.toString().replace(/["."]/g, decSep);
                            }
                            if (replaceminVal.toString().indexOf(decSep) > 0)
                                minVal = Number(replaceminVal.toString().substring(0, replaceminVal.toString().indexOf(decSep)));
                            else
                                minVal = this.model.minValue;
                            if (replacemaxVal.toString().indexOf(decSep) > 0)
                                maxVal = Number(replacemaxVal.toString().toString().substring(0, replacemaxVal.toString().indexOf(decSep)));
                            else
                                maxVal = this.model.maxValue;
                            if (tempIndex > 0) {
                                value = Number(temp.substring(temp.indexOf(decSep) + 1, temp.toString().length));
                                if (replaceminVal.toString().indexOf(decSep) > 0)
                                    minVal = Number(replaceminVal.toString().substring(replaceminVal.toString().indexOf(decSep) + 1, replaceminVal.toString().length));
                                else
                                    minVal = 0;
                                if (replacemaxVal.toString().indexOf(decSep) > 0)
                                    maxVal = Number(replacemaxVal.toString().substring(replacemaxVal.toString().indexOf(decSep) + 1, replacemaxVal.toString().length));
                                else
                                    maxVal = 0;
                                var valb4Dec = Number(temp.substring(0, temp.indexOf(decSep)));
                                var minValb4Dec = Number(replaceminVal.toString().substring(0, replaceminVal.toString().indexOf(decSep)));
                                var maxValb4Dec = Number(replacemaxVal.toString().substring(0, replacemaxVal.toString().indexOf(decSep)));
                                if (!this._validateValue(value, minVal, maxVal, true, valb4Dec, minValb4Dec, maxValb4Dec))
                                    cancelEvent = true;
                                else
                                    cancelEvent = false;
                            }
                            else {
                                if (!this._validateValue(value, minVal, maxVal, false))
                                    cancelEvent = true;
                                else
                                    cancelEvent = false;
                            }
                        }
                        else if (this.model.decimalPlaces != 0) {
                            var afterdeci = (this._textBox.value).split(".")[1];
                            if (temp.indexOf(decSep) > 0 || (Number(temp) < Number(replaceminVal) || Number(temp) > Number(replacemaxVal))) {
                                if ((temp.substring(temp.indexOf(decSep) + 1, temp.length)).length > this.model.decimalPlaces)
                                    cancelEvent = true;
                                else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimalPlaces) cancelEvent = true;
                                else cancelEvent = false;
                            }
                            else if (sel.length == 0 && cursor > (this._textBox.value.indexOf(decSep)) && afterdeci && afterdeci.length >= this.model.decimalPlaces) cancelEvent = true;
                            else
                                cancelEvent = false;
                        }
                        else
                            cancelEvent = false;
                        if (cancelEvent) {
                            this._keypressFlag = false;
                            this._cancelEvent(e);
                            return false;
                        }
                    }

                }
                if (e.keyCode == 38 && this._allowkeyboard) {
                    this._updateInputField("increment");
                    this._cancelEvent(e);
                }

                if (e.keyCode == 40 && this._allowkeyboard) {
                    this._updateInputField("decrement");
                    this._cancelEvent(e);

                }
                if (e.keyCode == 8) {
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.substring(0, cursor);
                    afterCursor = this._textBox.value.substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                }

                if (e.keyCode == 46 || e.keyCode == 127) {
                    cursor = this._caretPosition();
                    beforeCursor = this._textBox.value.substring(0, cursor);
                    afterCursor = this._textBox.value.substring(cursor);
                    sel = this._getSelection(this._textBox.value);
                }
                if (this._validateDecimal(e) && this.model.decimalPlaces != 0 && e.keyCode != 46) {
                    var decChar = Globalize.culture(this.model.locale).numberFormat["."];
                    var minVal, maxVal;
                    var dotSplit = this._textBox.value.split(decChar);
                    if (dotSplit[1] == undefined) {
                        var curPosition = this._caretPosition();
                        var strBeforeCursor = dotSplit[0].substring(0, curPosition);
                        var strAfterCursor = dotSplit[0].substring(curPosition);
                        if (this.model.minValue.toString().indexOf("e") > 0 || this.model.maxValue.toString().indexOf("e") > 0)
                            exp = true;
                        if (strAfterCursor.length > this.model.decimalPlaces && this.model.validateOnType == true) {
                            this._keypressFlag = false;
                            this._cancelEvent(e);
                            return false;
                        }
                        if (this._validateOnType && !exp) {
                            if (this.model.minValue.toString().match(new RegExp(".", "g")))
                                minVal = this.model.minValue.toString().replace(/["."]/g, decChar);
                            if (this.model.maxValue.toString().match(new RegExp(".", "g")))
                                maxVal = this.model.maxValue.toString().replace(/["."]/g, decChar);
                            if (minVal.indexOf(decChar) > 0)
                                minVal = Number(minVal.substring(0, minVal.indexOf(decChar)));
                            else
                                minVal = Number(minVal);
                            if (maxVal.indexOf(decChar) > 0)
                                maxVal = Number(maxVal.substring(0, maxVal.indexOf(decChar)));
                            else
                                maxVal = Number(maxVal);
                            if (this._validateValue(this.model.value, minVal, maxVal, "DecimalKeyPressed"))
                                this._textBox.value = strBeforeCursor + decChar + strAfterCursor;
                            else {
                                this._keypressFlag = false;
                                this._cancelEvent(e);
                                return false;
                            }
                        }
                        else
                            this._textBox.value = strBeforeCursor + decChar + strAfterCursor;
                        this._setSelectionRange(curPosition + 1, curPosition + 1);
                    }
                    this._cancelEvent(e);
                } else if (e.keyCode == 190 || e.keyCode == 110) {
                    this._cancelEvent(e);
                }
                if (e.keyCode == 109 || e.keyCode == 189 || e.keyCode == 173) { //'-' char key press.
                    if ((this._caretPosition() != 0 && this._getSelection(this._removeFormats(this._textBox.value)) != this.model.value) || (this.model.minValue >= 0 && !this.model.enableStrictMode) || (this._textBox.value.toString().match(new RegExp("-", "g")) && this._getSelection(this._textBox.value) === ""))
                        this._cancelEvent(e);

                    else if (this._getSelection() == this.model.value)
                        this.model.value = null;
                }

                if (e.keyCode == 13)
                    if (!ej.isNullOrUndefined(this._preVal) && (this._checkNumValue(this._preVal) !== this._checkNumValue(this._textBox.value)))
                        this._setValue(this._textBox.value);
            }

            else if (e.keyCode != 27 && !e.ctrlKey || (e.ctrlKey && e.keyCode == 90 && $.trim(this._textBox.value) === "")) {
                this._keypressFlag = false;
                this._cancelEvent(e);
            }
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejMaskEdit", "ej.MaskEdit", {

        element: null,

        model: null,
        validTags: ["input"],
        _setFirst: false,
        type: "editor",
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },

        defaults: {

            maskFormat: "",

            value: null,

            watermarkText: "",

            height: "",

            width: "",

            showError: false,

            htmlAttributes: {},

            cssClass: "",

            customCharacter: null,

            inputMode: "text",

            readOnly: false,

            textAlign: ej.TextAlign.Left,

            hidePromptOnLeave: false,

            showRoundedCorner: false,

            enablePersistence: false,

            enabled: true,

            validationRules: null,

            validationMessage: null,

            keydown: null,

            keyup: null,

            keyPress: null,

            change: null,

            mouseover: null,

            mouseout: null,

            focusIn: null,

            focusOut: null,

            create: null,

            destroy: null

        },

        dataTypes: {
            maskFormat: "string",
            showError: "boolean",
            enabled: "boolean",
            customCharacter: "string",
            cssClass: "string",
            watermarkText: "string",
            showRoundedCorner: "boolean",
            inputMode: "enum",
            textAlign: "enum",
            hidePromptOnLeave: "boolean",
            readOnly: "boolean",
            validationRules: "data",
            validationMessage: "data",
            htmlAttributes: "data"
        },

        _setModel: function (jsondata) {
            for (var key in jsondata) {
                switch (key) {
                    case "value": this._setValue(jsondata[key]); jsondata[key] = this.get_UnstrippedValue(); this._raiseEvents("change"); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "watermarkText": this.model.watermarkText = jsondata[key]; this._changeWatermark(jsondata[key]); break;
                    case "showRoundedCorner": this._roundedCorner(jsondata[key]); break;
                    case "validationRules":
                        if (this.model.validationRules != null) {
                            this._hiddenInput.rules('remove');
                            this.model.validationMessage = null;
                        }
                        this.model.validationRules = jsondata[key];
                        this._setValidation();
                        break;
                    case "validationMessage":
                        this.model.validationMessage = jsondata[key];
                        this._setValidation(); break;
                    case "textAlign": this._setTextAlign(jsondata[key]); break;
                    case "inputMode": this._setInputMode(jsondata[key]); break;
                    case "maskFormat": this._setMask(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "readOnly": this._setReadOnly(jsondata[key]); break;
                    case "enabled": this.model.enabled = jsondata[key]; this._controlStatus(jsondata[key]); break;
                    case "htmlAttributes": this._addAttr(jsondata[key]); break;
                }
            }
        },
        observables: ["value"],


        _destroy: function () {
            this.element.attr("name", this._hiddenInput.attr("name"));
            if (this._isWatermark) this.element.removeAttr("placeholder");
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            if (this.model.textAlign) this.element.css("text-align", "");
            this.element.val("").removeClass(' e-mask e-input e-disable').empty();
        },


        _init: function (options) {
            this._options = options;
            if (this.element.is("input") && (this.element.is("input[type=text]") || this.element.is("input[type=password]") || !this.element.attr('type'))) {
                this._isWatermark = 'placeholder' in document.createElement('input');
                this._setValues();
                this._renderControl();
                this._initObjects();
                this._wireEvents();
				this._addAttr(this.model.htmlAttributes);
                if (this.model.validationRules != null) {
                    this._initValidator();
                    this._setValidation();
                }
            } else {
                this._destroy();
                return false;
            }
        },

        _initValidator: function () {
            var proxy = this;
            this.element.closest("form").validate({
                ignore: ".e-hidden",
                errorClass: 'e-validation-error',
                errorElement: 'span',
                errorPlacement: function (error, element) {
                    $(error).insertAfter(element.closest(".e-widget"));
                },
            });
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "readOnly" && value == "readOnly") proxy._setReadOnly(true);
                else proxy.wrapper.attr(key, value);
            });
        },
        _setValidation: function () {
            this._hiddenInput.rules("add", this.model.validationRules);
            var validator = this.element.closest("form").validate();
            name = this._hiddenInput.attr("name");
            validator.settings.messages[name] = {};
            for (var ruleName in this.model.validationRules) {
                var message = null;
                if (!ej.isNullOrUndefined(this.model.validationRules[ruleName])) {
                    if (!ej.isNullOrUndefined(this.model.validationRules["messages"] && this.model.validationRules["messages"][ruleName]))
                        message = this.model.validationRules["messages"][ruleName];
                    else {
                        validator.settings.messages[name][ruleName] = $.validator.messages[ruleName];
                        for (var msgName in this.model.validationMessage)
                            ruleName == msgName ? (message = this.model.validationMessage[ruleName]) : "";
                    }
                    validator.settings.messages[name][ruleName] = message != null ? message : $.validator.messages[ruleName];
                }
            }
        },

        _controlStatus: function (value) {
            value != true ? this.disable() : this.enable();
        },

        _setValues: function () {
            this._tempMask = null;
            this._unStrippedMask = null;
            //Predefined character definitions
            this._charMap = {
                '9': "[0-9 ]",
                'a': "[A-Za-z0-9 ]",
                'A': "[A-Za-z0-9]",
                'N': "[0-9]",
                '#': "[0-9]",
                '&': '[^\x7f]+',
                '<': "",
                '>': "",
                'C': "[A-Za-z ]",
                '?': "[A-Za-z]",
            };
        },

        _renderControl: function () {
            this.model.name = this.element.attr("name") != null ? this.element.attr("name") : (this.model.name != null ? this.model.name : this.element[0].id);
            if (this.element.attr("name") != null)
                this.element.removeAttr('name');
            this.wrapper = ej.buildTag("span.e-mask e-widget " + this.model.cssClass);
            this.innerWrapper = ej.buildTag("span.e-in-wrap e-box");
            this.wrapper.append(this.innerWrapper).insertBefore(this.element);
            this.innerWrapper.append(this.element);
            this._hiddenInput = ej.buildTag("input#" + this._id + "_hidden", "", {}, { type: "hidden" }).insertBefore(this.element);
            if (!this._isWatermark && this.model.inputMode != "password") {
                this._hiddenSpan = ej.buildTag("span.e-input e-placeholder ").insertAfter(this.element);
                this._hiddenSpan.text(this.model.watermarkText);
                this._hiddenSpan.css("display", "none");
                this._hiddenSpan.bind("mousedown", $.proxy(this._OnFocusHandler, this));
            }
            if(ej.isNullOrUndefined(this.model.value) && this.element[0].value != "") this.model.value =  this.element[0].value;
            this._hiddenInput.attr({ "name": this.model.name, "value": this.model.value });
            this.element.attr({ 'aria-invalid': false, 'value': this.model.value }).addClass('e-input');
            if (ej.isNullOrUndefined(this._options.inputMode) && !ej.isNullOrUndefined(this.element.attr("type")))
                this.model.inputMode = this.element.attr("type");
            else
                this._setInputMode(this.model.inputMode);
            this._setWidth(this.model.width);
            this._setHeight(this.model.height);
            this._roundedCorner(this.model.showRoundedCorner);
            this._setTextAlign(this.model.textAlign);
            this._setReadOnly(this.model.readOnly);
            this._controlStatus(this.model.enabled);
            this.previousValue = this.model.value;
        },

        _initObjects: function () {
            this._textbox = this.element[0];
            this._keypressFlag = 0;
            this._selectedTextKeyDown = 0;
            // KeyDown trace [ 0 - None, 1 - BackSpace, 2 - Delete]
            this._keydownFlag = 0;
            // Length of the Mask
            if (!(this.model.maskFormat.indexOf("\\") >= 0))
                this._maskLength = this.model.maskFormat.length;
            else {
                var _position = 0;
                for (var temp = 0; temp < this.model.maskFormat.length; temp++) {
                    if ((this.model.maskFormat[temp] == "\\"))
                        _position += 1;
                }
                this._maskLength = this.model.maskFormat.length - _position;
            }
            //Replacing all [9?$CANa*] to '_'           
            if (!(this.model.maskFormat.indexOf("\\") >= 0))
                this._maskModel = this.model.maskFormat.replace(/[9?CANa#&]/g, '_');
            else
                this._maskModel = this.model.maskFormat;
            this._validatedValue = this._maskModel;
            if (this.model.inputMode != "password") {
                this._setValue(this.model.value);
                this._setWaterMark();
            }
            if (this.model.showError) {
                this.element.addClass("e-error").attr('aria-invalid', "true");
            }
        },


        _setWidth: function (value) {
            this.wrapper.width(value);
        },

        _setHeight: function (value) {
            this.wrapper.height(value);
        },

        _roundedCorner: function (value) {
            if (value && !this.innerWrapper.hasClass('e-corner')) {
                this.innerWrapper.addClass('e-corner');
            }
            else if (this.innerWrapper.hasClass('e-corner')) {
                this.innerWrapper.removeClass('e-corner');
            }
        },

        _setTextAlign: function (align) {
            if (align == "right") {
                this.wrapper.addClass('e-rtl');
            }
            else {
                this.wrapper.removeClass('e-rtl');
            }
        },

        _setInputMode: function (type) {
            this.element.attr('type', type);
        },

        _setReadOnly: function (bool) {
            this.model.readOnly = bool;
            if (bool) this.element.attr("readonly", true);
            else this.element.removeAttr("readonly");
        },

        _setSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass);
            this.wrapper.addClass(skin);
        },

         _setWaterMark: function () {
            if (!(this.model.maskFormat.indexOf("\\") >= 0))
                this._maskModel = $.trim(this.model.maskFormat.replace(/[9?CANa#&]/g, '_')) === "" ? null : this.model.maskFormat.replace(/[9?CANa#&]/g, '_');
            else
                this._maskModel = this.model.maskFormat === "" ? null : this.model.maskFormat;
            if (this.model.watermarkText && ((this._maskModel != '' && this._textbox.value == this._maskModel) || (this._maskModel == '' && this._textbox.value == "")) && this.model.inputMode != "password") {
                this._textbox.value = "";
                if (this._isWatermark)
                    this.element.attr("placeholder", this.model.watermarkText);
                else
                    this._hiddenSpan.css("display", "block").text(this.model.watermarkText);
            }
        },

        _changeWatermark: function (text) {
            if (!this.model.enabled) return false;
            if (this.model.watermarkText &&  (this._textbox.value == this._maskModel || this._textbox.value=='')  && this.model.inputMode != "password") {
 
				this._textbox.value = "";
                (this._isWatermark ? this.element.attr("placeholder", text) : this._hiddenSpan.text(text))     
            }
        },

        _showAlert: function () {
            var ptr = this;
            this.element.addClass("e-error").attr('aria-invalid', "true").animate({ Opacity: 1 }, 700, null, function () {
                $(ptr._textbox).removeClass("e-error").attr('aria-invalid', "false");
            });
        },

        _validateValue: function () {
            if (ej.isNullOrUndefined(this.model.value)) return;
            var tempValue = this.model.value;
            var tempModel = this._maskModel;
            var valueIndex = -1;
            if (!this._isWatermark && this.model.inputMode != "password")
                this._hiddenSpan.css("display", "none");
            // Check if Mask is Empty display the text
            if (this._maskLength == 0) {
                this._textbox.value = tempValue;
                this._setWaterMark();
                return true;
            }
            else {
                var replacestring = this.model.value.toString();
                if (!((this.model.maskFormat.indexOf("\\") >= 0)))
                    tempValue = this.model.value = replacestring.replace(/[\(\)-]/g, "");
                else
                    tempValue = this.model.value;
            }

            for (var maskIndex = 0; maskIndex < this._maskLength; maskIndex++) {
                if (this._charMap[this.model.maskFormat.charAt(maskIndex)] != null) {
                    valueIndex++;
                    var charCode = tempValue.charCodeAt(valueIndex);
                    if (this._validateChars(charCode, maskIndex)) {
                        var strBefore = tempModel.substring(0, maskIndex);
                        var strAfter = tempModel.substring(maskIndex);
                        tempModel = strBefore + tempValue.charAt(valueIndex) + strAfter.substr(1, strAfter.length);
                    }
                }
            }
            if (this.model.value) {
                if (!((this.model.maskFormat.indexOf("\\") >= 0) || (this.model.maskFormat.indexOf("&") >= 0)))
                    this._textbox.value = tempModel;
                else {
                    this._textbox.value = "";
                    var _tPos = 0;
                    for (_pos = 0; _pos < this.model.maskFormat.length; _pos++) {
                        if (this.model.maskFormat[_pos] == "\\")
                            _tPos += 1;
                    }
                    var _position = 0;
                    if (this.model.value.length == tempModel.length - _tPos) {
                        for (var pos = 0; pos < this.model.maskFormat.length; ++pos) {
                            if (this.model.maskFormat[pos] == "\\")
                                _position = _position + 1;
                            else if (this.model.maskFormat[pos - 1] == "\\")
                                this._textbox.value += tempValue[pos - _position];
                            else if (!(this.model.maskFormat[pos] == "\\")) {
                                if (tempValue.length <= pos - _position)
                                    this._textbox.value += tempModel[pos].replace(/[9?CANa#&]/g, '_');
                                else
                                    this._textbox.value += tempValue[pos - _position].replace(/[9?CANa#&]/g, '_');
                            }

                        }
                    }
                    else {
                        this._textbox.value = "";
                        for (var pos = 0; pos < this.model.maskFormat.length; ++pos) {
                            if (this.model.maskFormat[pos - 1] == "\\")
                                this._textbox.value += tempModel[pos];
                            else if (!(this.model.maskFormat[pos] == "\\"))
                                this._textbox.value += tempModel[pos].replace(/[9?CANa#&]/g, '_');
                        }
                    }

                }
            }
            else {
                if (!(this.model.maskFormat.indexOf("\\") >= 0))
                    this._textbox.value = this.model.maskFormat.replace(/[9?CANa#&]/g, '_');
                else {
                    for (var pos = 0; pos < this.model.maskFormat.length; ++pos) {
                        var val;
                        if (this.model.maskFormat[pos - 1] == "\\")
                            this._textbox.value += this.model.maskFormat[pos];

                        else if (!(this.model.maskFormat[pos] == "\\"))
                            this._textbox.value += this.model.maskFormat[pos].replace(/[9?CANa#&]/g, '_');

                    }
                }
            }
            if (this._textbox.value != undefined && this.model.hidePromptOnLeave && this.model.watermarkText == "") {
                this._unStrippedMask = this._textbox.value;
                this._textbox.value = this.get_UnstrippedValue();
            }
            if (this._textbox.value.indexOf(">") >= 0 || this._textbox.value.indexOf("<") >= 0) {
                this._textbox.value = this._textbox.value.replace(/[<>]/g, '');
                this._tempMask = this.model.maskFormat;
                this.model.maskFormat = this.model.maskFormat.replace(/[<>]/g, '');
            }
            this._setWaterMark();
        },


        _selectionText: function (begin, end) {
            var replaceValue = this._maskModel.substring(begin, end);
            this._textbox.value = this._textbox.value.substring(0, begin) + replaceValue + this._textbox.value.substring(end);
            if (this._keydownFlag == 1) {
                this._setCaretPosition(begin);
                return begin;
            }
            else if (this._keydownFlag == 2) {
                this._setCaretPosition(end);
                return end;
            }
            return begin;
        },


        _caretPosition: function (textbox) {
            var caretPos = 0;
            // Microsoft supports only IE for using document.selection
            if (document.selection) {
                // Create a Range of selected position
                var selectedRange = document.selection.createRange();
                var selectionLength = selectedRange.text.length;
                // Move selection start to 0 position
                selectedRange.moveStart('character', -textbox.value.length);
                // Get caret position by selection length, becoz now range is from Zero to current focus
                caretPos = selectedRange.text.length - selectionLength;
                if (selectionLength != 0) {
                    caretPos = this._selectionText(caretPos, selectedRange.text.length);
                    this._selectedTextKeyDown = 1;
                }
            }
            // For Firefox
            else if (textbox.selectionStart || textbox.selectionStart == '0') {
                caretPos = textbox.selectionStart;
                if (textbox.selectionStart != textbox.selectionEnd) {
                    if (this._keydownFlag)
                        caretPos = this._selectionText(textbox.selectionStart, textbox.selectionEnd);
                    this._selectedTextKeyDown = 1;
                }
            }
            return (caretPos);
        },


        _setCaretPosition: function (caretPos) {
            var element = this._textbox;
            //For IE
            if (window.navigator.appName == "Microsoft Internet Explorer") {
                if (element.createTextRange) {
                    var range = element.createTextRange();
                    range.move('character', caretPos);
                    range.select();
                }
            }
            else
            /* For FireFox and Chrome */
                if (element.selectionStart || element.selectionStart == '0') {
                    element.focus();
                    element.setSelectionRange(caretPos, caretPos);
                }
                else
                    element.focus();
        },


        _validateChars: function (keyChar, caretPos) {
            var charmap = this._charMap, match = false;
            if (this.model.maskFormat.indexOf("\\") >= 0) {
                _position = this._getCunrrentPos(caretPos);
                var maskChar = this.model.maskFormat.substr(caretPos + _position, 1);
            }
            else
                var maskChar = this.model.maskFormat.substr(caretPos, 1);
            var customChar = this.model.customCharacter;
            var actualkey = String.fromCharCode(keyChar);
            $.each(charmap, function (key, value) {
                if (maskChar == key) {
                    if (customChar != null) {
                        if (key == "C")
                            value = "[" + customChar + "]";
                        else if (key == "A" || key == "N" || key == "#")
                            value = value.replace(("]"), "") + customChar + "]";
                    }
                    if (actualkey.match(new RegExp(value))) match = true;
                    else match = false;
                }
            });
            return match;
        },


        _seekNext: function () {
            var currentPos = this._caretPosition(this._textbox);
            var tempPos = currentPos;
            var seekFlag = true;
            //seeking the focus to next valid position on right of MaskEdit TextBox
            while (seekFlag) {
                if (currentPos >= 0 && currentPos < this._maskLength) {
                    var _position = 0;
                    for (var temp = 0; temp <= currentPos; temp++) {
                        if ((this.model.maskFormat[temp] == "\\"))
                            _position += 1;
                    }
                    if (this.model.maskFormat.charAt(currentPos) != "C") {
                        if (this.model.maskFormat.indexOf("\\") >= 0) {
                            if (this.model.maskFormat.charAt(currentPos + _position) != "\\" && this.model.maskFormat.charAt(currentPos + _position - 1) != "\\") {
                                if (!this._charMap[this.model.maskFormat.charAt(currentPos + _position)]) {
                                    this._setCaretPosition(currentPos);
                                    currentPos++;
                                }
                            }
                        }
                        else {
                            if (!this._charMap[this.model.maskFormat.charAt(currentPos)]) {
                                this._setCaretPosition(currentPos);
                                currentPos++;
                            }
                        }
                        if (currentPos != tempPos) {
                            tempPos = currentPos;
                            continue;
                        }
                    }
                }
                seekFlag = false;
            }
            return currentPos;
        },


        _seekBefore: function (caretPos) {
            var currentPos = this._caretPosition(this._textbox);
            var tempPos = --currentPos;
            var seekFlag = true;
            // Stop Seek when backspace of select more than a char
            if (this._selectedTextKeyDown == 1)
                seekFlag = false;
            //seeking the focus to next valid position on left of MaskEdit TextBox
            while (seekFlag) {
                if (currentPos >= 0 && currentPos < this._maskLength) {
                    if (this.model.maskFormat.charAt(currentPos) != "C") {
                        if (!this._charMap[this.model.maskFormat.charAt(currentPos)]) {
                            this._setCaretPosition(--currentPos);
                        }
                    }
                    if (currentPos != tempPos) {
                        tempPos = currentPos;
                        continue;
                    }
                }
                seekFlag = false;
            }
            return currentPos;
        },


        _writeBuffer: function (key, cursorPos) {
            if (cursorPos <= this._maskLength) {
                var input = this._textbox.value;
                if (this._tempMask != null)
                    var tempkey = this._updateCasing(key, cursorPos);
                key = (tempkey == undefined) ? key : tempkey;
                var strBeforeCursor = input.substring(0, cursorPos);
                var strAfterCursor = input.substring(cursorPos);
                if (this.model.maskFormat.indexOf("\\") >= 0) {
                    var _position = 0;
                    for (var temp = 0; temp <= cursorPos; temp++) {
                        if ((this.model.maskFormat[temp].indexOf("\\") >= 0) && (cursorPos != 0))
                            _position += 1;
                    }
                    if (this.model.maskFormat[cursorPos + _position].indexOf("\\") >= 0)
                        this._textbox.value = strBeforeCursor + input[cursorPos] + strAfterCursor.substr(1, strAfterCursor.length);
                    else
                        this._textbox.value = strBeforeCursor + key + strAfterCursor.substr(1, strAfterCursor.length);
                }
                else
                    this._textbox.value = strBeforeCursor + key + strAfterCursor.substr(1, strAfterCursor.length);
                this._setCaretPosition(cursorPos + 1);
            }
        },

        _updateCasing: function (key, pos) {
            for (var i = 0; i < pos + 1; i++) {
                if (this._tempMask.substr(i, 1) == '<' || this._tempMask.substr(i, 1) == '>')
                    pos++;
            }

            for (var j = pos; j > -1; j--) {
                if (this._tempMask.substr(j, 1) == '<') {
                    return key.toLowerCase();
                }
                else if (this._tempMask.substr(j, 1) == '>') {
                    return key.toUpperCase();
                }
            }
        },


        get_StrippedValue: function () {
            var i, value, mask = this.model.maskFormat, stripVal = null;
            value = (this._textbox.value == "" && this.model.inputMode != "password") ? this._maskModel : this._textbox.value;
            if (mask.length == 0) return value;
            var _position = 0;
            for (var i = 0; i < mask.length; i++) {
                stripVal = $.trim(stripVal);
                if ((this.model.maskFormat.indexOf("\\") >= 0)) {
                    if ((this.model.maskFormat[i] == "\\"))
                        _position += 1;
                    else if (this.model.maskFormat[i - 1] == "\\")
                        stripVal += this._textbox.value[i - _position];
                    else {
                        var char = mask[i], exp = null;

                        if ("9?$a*".indexOf(char) != -1)
                            exp = this._charMap[char];
                        else if (char == "A" || char == "N" || char == "#")
                            exp = this._charMap[char].replace(("]"), "") + this.model.customCharacter + "]";
                        else if (char == "C")
                            exp = "[" + this.model.customCharacter + "]";
                        else if (char == "&")
                            stripVal += value[i - _position];
                        if (exp && value[i - _position] && value[i - _position].match(new RegExp(exp)))
                            stripVal += value[i - _position];
                    }

                }
                else {
                    var char = mask[i], exp = null;

                    if ("9?$a*".indexOf(char) != -1)
                        exp = this._charMap[char];
                    else if (char == "A" || char == "N" || char == "#")
                        exp = this._charMap[char].replace(("]"), "") + this.model.customCharacter + "]";
                    else if (char == "C")
                        exp = "[" + this.model.customCharacter + "]";
                    else if (char == "&")
                        stripVal += value[i - _position];
                    if (exp && value[i] && value[i].match(new RegExp(exp)))
                        stripVal += value[i];
                }
            }
            return stripVal;
        },


        get_UnstrippedValue: function () {
            var value = (this._textbox.value == "" && this.model.inputMode != "password") ? this._maskModel : this._textbox.value, unstripVal = null;
            if (this.model.maskFormat.length == 0) return value;

            if (value != undefined)
                unstripVal = $.trim(value.replace(/[_]/g, " ")) == "" ? null : value.replace(/[_]/g, " ");
            return unstripVal;
        },

        _setValue: function (value) {
            if (ej.isNullOrUndefined(value) || $.trim(value) == "") value = null;
            this.model.value = value;
            if (!this._isWatermark && this.model.inputMode != "password")
                this._hiddenSpan.css("display", "none");
            if (!ej.isNullOrUndefined(this.model.maskFormat) && this.model.maskFormat != "")
                this._validateValue();
            else
                this._textbox.value = value;
            this._setWaterMark();
        },

        _setMask: function (maskValue) {
            this._maskLength = maskValue.length;
            this.model.maskFormat = maskValue;
            this._maskModel = maskValue.replace(/[9?CANa]/g, '_');
            this._validateValue();
        },

        enable: function () {
            this.element.disabled = false;
            this.element.removeAttr("disabled").removeClass('e-disable').attr({ "aria-disabled": false });
            this.model.enabled = true;
        },

        disable: function () {
            this.element.disabled = true;
            this.element.attr("disabled", "disabled").addClass('e-disable').attr({ "aria-disabled": true });
            this.model.enabled = false;
        },

        clear: function () {
            this._textbox.value = this.model.maskFormat.replace(/[9?aCAN]/g, '_');
        },

        _wireEvents: function () {
            this._on(this.element, 'focus', this._OnFocusHandler);
            this._on(this.element, 'blur', this._OnBlurHandler);
            this._on(this.element, 'keydown', this._OnKeyDownHandler);
            this._on(this.element, 'keypress', this._OnKeyPressHandler);
            this._on(this.element, 'keyup', this._OnKeyUpHandler);
            this._on(this.element, 'mouseover', this._OnMouseOverHandler);
            this._on(this.element, 'mouseout', this._OnMouseOutHandler);
            this._on(this.element, 'paste', this._OnPasteHandler);
            this._on(this.element, 'cut', this._OnCutHandler);
        },


        _OnCutHandler: function (e) {
            var selectedValue = this._maskModel.substring(this._textbox.selectionStart, this._textbox.selectionEnd);
            var beforeSelection = this._textbox.value.substring(0, this._textbox.selectionStart);
            var afterSelection = this._textbox.value.substring(this._textbox.selectionEnd);
            var cursorPosition = this._textbox.selectionStart;
            var context = this;
            setTimeout(function () {
                context._textbox.value = beforeSelection + selectedValue + afterSelection;
                context._setCaretPosition(cursorPosition);
                context._raiseEvents("change");
            }, 0);
        },


        _OnPasteHandler: function (e) {
            var context = this;
            setTimeout(function () {
                var text = $(context._textbox).val();
                context._setValue(text);
                context._raiseEvents("change");
            }, 0);
            return true;
        },


        _OnFocusHandler: function (e) {
            this.wrapper.addClass("e-focus");
            if (this.model.readOnly)
                return;
            if (!this._isWatermark && this.model.inputMode != "password")
                this._hiddenSpan.css("display", "none");
            if (this._textbox.value == "" && this._maskModel != "" && this.model.inputMode != "password")
                if (this.model.maskFormat.indexOf("\\") >= 0) {
                    this._textbox.value = this.model.watermarkText;
                }
                else
                    this._textbox.value = this._maskModel;
            if (this._textbox.value.indexOf("<") >= 0 || this._textbox.value.indexOf(">") >= 0) {
                this._textbox.value = this._textbox.value.replace(/[<>]/g, '');
                this._maskModel = this._textbox.value;
                this._tempMask = this.model.maskFormat;
                this.model.maskFormat = this.model.maskFormat.replace(/[<>]/g, '');
            }

            if (this._textbox.value != this._maskModel && this._unStrippedMask != null && this.model.hidePromptOnLeave)
                this._textbox.value = this._unStrippedMask;
            $.fn.selectRange = function (start, end) {
                return this.each(function () {
                    if (this.setSelectionRange) {
                        this.focus();
                        this.setSelectionRange(start, end);
                    } else if (this.createTextRange) {
                        var range = this.createTextRange();
                        range.collapse(true);
                        range.moveEnd('character', end);
                        range.moveStart('character', start);
                        range.select();
                    }
                });
            };
            if (this.model.maskFormat) {
                $(this.element).selectRange(0, 0);
            }

            this._raiseEvents("focusIn");
        },

        _OnBlurHandler: function (e) {
            this.wrapper.removeClass("e-focus");            
            this.model.value = this.get_StrippedValue();
            var dup = new Array();
            var option = new Array();
            if ((this._textbox.value.indexOf('_') != -1 || this._textbox.value.indexOf(' ') != -1)&& !ej.isNullOrUndefined(this.model.value)) {
                for (var i = 0; i < this.model.maskFormat.length; i++) {
                    if ((this.model.maskFormat[i] == "A") || (this.model.maskFormat[i] == "?")) dup.push(i);
                    else if ((this.model.maskFormat[i] == "9") || (this.model.maskFormat[i] == "a") || (this.model.maskFormat[i] == "C")) {
                        option.push(i);
                    }
                }
                for (var i = 0; i < dup.length; i++) {
                    if (this._textbox.value[dup[i]] === "_") {
                        this._showAlert();
                        this.model.value = this._textbox.value;
                    }
                }
                for (var i = 0; i < option.length; i++) {
                    if ((this.model.value[option[i]] == "_") || (this.model.value[option[i]] == " ")) {
                        this.model.value = this.model.value.substr(0, option[i]) + this.model.value.substr(option[i] + 1, this.model.value.length)
                        for (var j = i; j < option.length; j++) {
                            option[j] = option[j] - 1;
                        }
                    }
                }
            }
            this.model.value = (this.model.value != null) ? this.model.value.replace(/\s+/g, "") : this.model.value;
            if (this._textbox.value != undefined && this.model.hidePromptOnLeave && (this.model.watermarkText == "" || (this.model.watermarkText != "" && this.model.value != ""))) {
                this._unStrippedMask = this._textbox.value;
                this._textbox.value = this.get_UnstrippedValue();
            }
            if (this.model.inputMode != "password")
                this._setWaterMark();
            this._raiseEvents("change");
            this._raiseEvents("focusOut");
        },


        _OnKeyDownHandler: function (e) {
            if (this.model.readOnly) return;
            this._keypressFlag = 0;
            this._raiseEvents("onKeyDown");

            this.element.removeClass("error");

            if (this._maskLength == 0)
                return true;

            var unicode = e.keyCode ? e.keyCode : e.charCode;
            var actualkey = String.fromCharCode(unicode);
            var currentPos = this._seekNext();
            var val = ej.browserInfo().name;
            if ((val == "msie") && (this._textbox.value == "")) {
                if (!(this.model.maskFormat.indexOf("\\") >= 0))
                    this._textbox.value = this.model.maskFormat.replace(/[9?CANa#&]/g, '_');
                else {
                    for (var pos = 0; pos < this.model.maskFormat.length; ++pos) {
                        var val;
                        if (this.model.maskFormat[pos - 1] == "\\")
                            this._textbox.value += this.model.maskFormat[pos];
                        else

                            if (!(this.model.maskFormat[pos] == "\\"))
                                this._textbox.value += this.model.maskFormat[pos].replace(/[9?CANa#&]/g, '_');
                    }
                }
                this._setCaretPosition(currentPos);
            }

            if (unicode >= 35 && unicode <= 41) {
                if (window.navigator.appCodeName == "Mozilla" || window.navigator.appCodeName == "opera") {
                    this._keypressFlag = 1;
                }
            }
            if (e.shiftKey && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39 || e.keyCode == 46 || e.keyCode == 127) || (e.ctrlKey && (e.keyCode == 86 || e.keyCode == 65 || e.keyCode == 67 || e.keyCode == 88))) {
                this._keypressFlag = 1;
                return;
            }
            else if (unicode == 8) {

                if (this.model.inputMode != "password") {
                    this._keydownFlag = 1;
                    // Seek to next valid position
                    var currentPos = this._seekBefore();
                    if (!this._selectedTextKeyDown) {
                        if (currentPos >= 0 && currentPos < this._maskLength)
                            this._writeBuffer("_", currentPos);
                        this._setCaretPosition(currentPos);
                    }
                    this._keydownFlag = 0;
                    this._selectedTextKeyDown = 0;
                    e.preventDefault();
                    return false;
                }
                this._keypressFlag = 1;
                return true;
            }
            else if (e.keyCode == 46 || e.keyCode == 127) {
                if (this.model.inputMode != "password") {
                    this._keydownFlag = 2;
                    // Seek to next valid position
                    currentPos = this._seekNext();
                    if (!this._selectedTextKeyDown) {
                    if (currentPos >= 0 && currentPos < this._maskLength) {                        
                        _position=this. _getCunrrentPos(currentPos);
                            if (!((this.model.maskFormat[currentPos + _position] == "&")))
                                this._writeBuffer("_", currentPos);
                        }
                    }
                    this._keydownFlag = 0;
                    this._selectedTextKeyDown = 0;
                    e.preventDefault();
                    return false;
                }
                this._keypressFlag = 1;
                return true;
            }
        },


        _ErrorHandler: function (currentPos) {
            if (this._textbox.value == "") this._textbox.value = this.model.maskFormat.replace(/[9?$CANa*]/g, '_');
            this._setCaretPosition(currentPos);
            this._showAlert();
        },
        _getCunrrentPos: function (currentPos) {
            var _position = 0;
            for (var temp = 0; temp <= currentPos + _position; temp++) {
                if ((this.model.maskFormat[temp] == "\\"))
                    _position += 1;
            }
            return _position;
        },


        _OnKeyPressHandler: function (e) {
            if (this.model.readOnly) return;
            this._raiseEvents("keyPress");
            if (this._maskLength == 0)
                return true;

            var unicode = e.keyCode ? e.keyCode : e.charCode;
            var actualkey = String.fromCharCode(unicode);
            var currentPos = this._seekNext();
            var val = ej.browserInfo().name;
            if (this._validateChars(unicode, currentPos)) {
                if (((val == "mozilla") || (val == "opera")) && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39)) {
                }
                else if (((val == "mozilla") || (val == "opera")) && (e.ctrlKey && (unicode == 97 || unicode == 99 || unicode == 118 || unicode == 120))) {
                }
                else {
                    if (this._textbox.value == "") this._textbox.value = this.model.maskFormat.replace(/[9?$CANa*]/g, '_');
                    if (this.model.maskFormat.indexOf("\\") >= 0) {
                        if (currentPos < this._maskLength)
                            this._writeBuffer(actualkey, currentPos);
                    }
                    else
                        this._writeBuffer(actualkey, currentPos);
                }
            }
            else
                if (((val == "mozilla") || (val == "opera")) && (e.keyCode == 35 || e.keyCode == 36 || e.keyCode == 37 || e.keyCode == 39)) {
                }
                else if (((val == "mozilla") || (val == "opera")) && (e.ctrlKey && (unicode == 97 || unicode == 99 || unicode == 118 || unicode == 120))) {
                }
                else if (this.model.maskFormat.indexOf("&") >= 0 || this.model.maskFormat.indexOf("\\") >= 0) {
                    _position = this._getCunrrentPos(currentPos);
                    if (!((this.model.maskFormat[currentPos + _position] == "&") && (unicode == 127))) {
                        if (this.model.maskFormat.indexOf("\\") >= 0) {
                            if (currentPos < this._maskLength) {
                                if (this._validateChars(unicode, currentPos + _position) || (this.model.maskFormat[currentPos + _position - 1] == "\\"))
                                    this._writeBuffer(actualkey, currentPos);
                                else
                                    this._ErrorHandler(currentPos);
                            }
                        }
                        else {
                            if (this._validateChars(unicode, currentPos + _position))
                                this._writeBuffer(actualkey, currentPos);
                            else
                                this._ErrorHandler(currentPos);
                        }
                    }
                }
                else
                    this._ErrorHandler(currentPos);


            if (!this._keypressFlag && unicode != 9) {
                this._keypressFlag = 0;
                e.preventDefault();
                return false;
            }
            this._keypressFlag = 0;

        },

        _OnKeyUpHandler: function (e) {
            if (this._maskLength == 0) this._raiseEvents("change");
            this._raiseEvents("keyUp");
        },


        _OnMouseOverHandler: function (e) {
            this._raiseEvents("mouseOver");
        },


        _OnMouseOutHandler: function (e) {
            this._raiseEvents("mouseOut");
        },


        _raiseEvents: function (eventName) {
            var eventArgs, strippedVal = this.get_StrippedValue(), unstrippedVal = this.get_UnstrippedValue();
            this.model.value = unstrippedVal;
            if (eventName == "change") {
                if (this.previousValue != this.model.value) {
                    this.previousValue = this.model.value;
                }
                else
                    return false;
            }
            eventArgs = { value: unstrippedVal, unmaskedValue: strippedVal };
            this._trigger(eventName, eventArgs);
            if (eventName == "change") this._trigger("_change", eventArgs);
            this._hiddenInput.val(strippedVal);
        },

        _OnValueChange: function () {
            if (this._textbox.value == "" && this._maskModel != "")
                this._textbox.value = this._maskModel;
            this._setValue(this._textbox.value);
        }

    });

    ej.InputMode = {
		/**  support for user enter character in password format. */
		Password: "password", 
		/**  support for user enter character in normal format. */
        Text: "text"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Menu control.
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejMenu", "ej.Menu", {

        element: null,

        model: null,
        validTags: ["ul"],
        _ignoreOnPersist: ["fields", "create", "beforeOpen", "open", "close", "mouseover", "mouseout", "click", "keydown", "destroy"],
        _setFirst: false,
        _rootCss: "e-menu",
        angular: {
            terminal: false
        },


        defaults: {

            height: "",

            width: "",

            animationType: "default",

            orientation: ej.Orientation.Horizontal,

            menuType: "normalmenu",

            contextMenuTarget: null,

            htmlAttributes: {},

            cssClass: "",

            openOnClick: false,

            subMenuDirection: "none",

            enableCenterAlign: false,

            showRootLevelArrows: true,

            showSubLevelArrows: true,

            enableAnimation: true,

            enableSeparator: true,

            enabled: true,

            fields: {

                child: null,

                dataSource: null,

                query: null,

                tableName: null,

                id: "id",

                parentId: "parentId",

                text: "text",

                spriteCssClass: "spriteCssClass",

                url: "url",

                imageAttribute: "imageAttribute",

                htmlAttribute: "htmlAttribute",

                linkAttribute: "linkAttribute",

                imageUrl: "imageUrl",
            },

            enableRTL: false,

            titleText: "Menu",

            excludeTarget: null,

            beforeOpen: null,

            open: null,

            close: null,

            mouseover: null,

            mouseout: null,

            click: null,

            keydown: null,

            create: null,

            destroy: null
        },

        dataTypes: {
            animationType: "enum",
            cssClass: "string",
            titleText: "string",
            openOnClick: "boolean",
            enabled: "boolean",
            enableCenterAlign: "boolean",
            showArrow: "boolean",
            showRootLevelArrows: "boolean",
            showSubLevelArrows: "boolean",
            enableSeparator: "boolean",
            enableRTL: "boolean",
            enableAnimation: "boolean",
            fields: {
                dataSource: "data",
                query: "data",
                child: "data"
            },
            excludeTarget: "string",
            htmlAttributes: "data"
        },


        _setModel: function (jsondata) {
            for (var key in jsondata) {
                switch (key) {
                    case "menuType":
                        jsondata[key] = this.model.menuType;
                        break;
                    case "fields":
                        this._wireEvents("_off");
                        this.element.empty().insertBefore(this.wrapper);
                        this.wrapper.remove();
                        $.extend(this.model.fields, jsondata[key]);
                        this._intializeData();
                        if (!this.model.enabled)
                            this._wireEvents("_off");
                        break;
                    case "orientation": this._setOrientation(jsondata[key]); break;
                    case "showRootLevelArrows": this._addArrow(jsondata[key], this.model.showSubLevelArrows); break;
                    case "showSubLevelArrows": this._addArrow(this.model.showRootLevelArrows, jsondata[key]); break;
                    case "enableSeparator": this._setSeparator(jsondata[key]); break;
                    case "height": this._setHeight(jsondata[key]); break;
                    case "width": this._setWidth(jsondata[key]); break;
                    case "cssClass": this._setSkin(jsondata[key]); break;
                    case "htmlAttributes": this._addAttr(jsondata[key]); break;
                    case "enableRTL": this._setRTL(jsondata[key]); break;
                    case "enableCenterAlign": this._centerAlign(jsondata[key]); break;
                    case "excludeTarget": this.model.excludeTarget = jsondata[key];
                        break;
                    case "enabled": this.model.enabled = jsondata[key]; this._controlStatus(jsondata[key]); break;
                    case "animationType":
                        this._setAnimation(jsondata[key]);
                        break;
                    case "enableAnimation": this.model.enableAnimation = jsondata[key]; break;
                    case "openOnClick":
                        if (jsondata[key] == true) {
                            this._hoverOpen = false;
                            this._hoverClose = false;
                        }
                        break;
                    case "subMenuDirection": this._setSubMenuDirection(this.model.subMenuDirection); break;
                    case "titleText":
                        if ((this.model.menuType != ej.MenuType.ContextMenu) && (this.model.orientation != "vertical"))
                            $(this.label).text(jsondata[key]);
                        break;
                }
            }
        },

        _destroy: function () {
            this._cloneElement.insertBefore(this.wrapper);
            this._cloneElement.removeClass('e-menu e-js');
            this.wrapper.remove();
        },


        _init: function () {
            this._cloneElement = this.element.clone();
            this.element.css("visibility", "hidden");
            this._setValues();
            this._intializeData();
            this.element.css("visibility", "visible");
        },

        _setValues: function () {
            this._mouseOver = true;
            this._hoverOpen = true;
            this._hoverClose = true;
            this._isMenuOpen = false;
            this._hideSpeed = 100;
            this._showSpeed = 100;
            this._isSubMenuOpen = false;
            this._isContextMenuOpen = false;
            this._disabledMenuItems = new Array();
            this._delayMenuHover = 0;
            this._delaySubMenuHover = 0;
            this._delaySubMenuShow = 0;
            this._preventContextOpen = true;
            this._setAnimation(this.model.animationType);
            this._isFocused = true;
        },
        _intializeData: function () {
            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                this._generateTemplate(this.model.fields["dataSource"]);
                this._renderMenu();
            }
            else {
                this._renderMenu();
                this._wireEvents("_on");
            }
        },
        _renderMenu: function () {
            this._renderControl();
            this._addArrow(this.model.showRootLevelArrows, this.model.showSubLevelArrows);
            this._intializeMenu();
        },

        _renderControl: function () {
            var label, checkBox, checkObj, list, spanlist, i;
            if (this.model.menuType == "normalmenu") {
                this.wrapper = ej.buildTag("div");
                this.wrapper.addClass(this.model.cssClass + " e-menu-wrap");
            } else
                this.wrapper = ej.buildTag("div.e-menu-wrap");
            if ((this.model.menuType != ej.MenuType.ContextMenu) && (this.model.orientation != "vertical")) {
                this.resWrap = ej.buildTag('span.e-menu-res-wrap');
                this.inResWrap = ej.buildTag('span.e-in-wrap e-box e-menu-res-in-wrap');
                this.label = ej.buildTag('span.e-res-title', this.model.titleText);
                this.check = ej.buildTag('span.e-check-wrap e-icon');
                this.checkBox = ej.buildTag('input#responsive.e-res-icon', '', { "opacity": 0 }, { type: 'checkbox' });
                this.wrapper.append(this.resWrap)
                this.resWrap.append(this.inResWrap);
                this.inResWrap.append(this.label).append(this.check);
                this.check.append(this.checkBox);
            }
            if (this.model.menuType != ej.MenuType.ContextMenu) {
                this.wrapper.insertBefore(this.element);
                this.wrapper.append(this.element);
                if (this.checkBox != null) {
                    this.checkBox.ejCheckBox({ size: "medium", change: this._oncheck });

                }
            }
            this.element.addClass("e-menu e-widget e-box").attr({ "role": "menu", "tabindex": 0 });
            this._addAttr(this.model.htmlAttributes);
            if (this.model.enableRTL) this._setRTL(this.model.enableRTL);
            this._setSubMenuDirection(this.model.subMenuDirection);
            if (this.model.menuType == "normalmenu") {
                this.model.orientation == "horizontal" ? this.element.addClass("e-horizontal") : this.element.addClass("e-vertical");
            }
            //For ContextMenu Mode
            else this._contextMenu_Template();
            //Adding arrows to items with sub items
            this.element.find('li:has("> ul")').find('> a,> span').addClass('aschild');
            this.element.find('>li').addClass('e-list').attr({ "role": "menuitem" });
            list = this.element.find('.e-list a.aschild');
            spanlist = this.element.find('.e-list span.aschild');
            for (i = 0; i < list.length; i++) {
                $(list[i]).siblings().attr({ "aria-hidden": true });
                $(list[i]).parent().attr({ "aria-haspopup": true, "role": "menu" });
                $(list[i]).parent().addClass("e-haschild");
                $(list[i]).siblings('ul').children('li').addClass('e-list').attr("role", "menuitem");
            }
            for (i = 0; i < spanlist.length; i++) {
                $(spanlist[i]).siblings().attr({ "aria-hidden": true });
                $(spanlist[i]).parent().attr({ "aria-haspopup": true, "role": "menu" });
                $(spanlist[i]).parent().addClass("e-haschild");
                $(spanlist[i]).siblings('ul').children('li').addClass('e-list').attr("role", "menuitem");
            }
            if (this.model.enableCenterAlign) this._centerAlign(this.model.enableCenterAlign);
            if (this.model.enableSeparator) this._setSeparator(true);
            this._controlStatus(this.model.enabled);
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else proxy.element.attr(key, value)
            });
        },

        _oncheck: function (e) {
            var obj = this.element.parents('.e-menu-wrap').children('.e-menu');
            e.isChecked ? obj.removeClass('e-res-hide').addClass('e-res-show') : obj.removeClass('e-res-show').addClass('e-res-hide');
        },

        _generateTemplate: function (data) {
            var proxy = this, queryPromise;
            if (data instanceof ej.DataManager) {
                queryPromise = data.executeQuery(this._columnToSelect(this.model.fields));
                queryPromise.done(function (e) {
                    proxy._odataFlag = true;
                    proxy._generateItemTemplate(e.result);
                    proxy._wireEvents("_on");
                });
            } else {
                proxy._odataFlag = false;
                this._generateItemTemplate(proxy.model.fields['dataSource']);
                this._wireEvents("_on");
            }
        },

        _generateItemTemplate: function (items) {
            for (var i = 0; i < items.length; i++) {
                if ((items[i][this.model.fields.parentId] == null) || (items[i][this.model.fields.parentId] == 0)) {
                    var subItem = this._menuTemplate(items[i], items, this.model.fields);
                    this.element.append(subItem);
                }
            }
        },

        _menuTemplate: function (item, tableData, mapper) {
            var liTag, aTag, imgTag, spanTag;
            liTag = $(document.createElement('li'));
            liTag.attr("class", 'e-list');
            if (item[mapper.htmlAttribute]) this._setAttributes(item[mapper.htmlAttribute], liTag);

            if (item[mapper.id]) {
                aTag = $(document.createElement('a'));
                if (item[mapper.imageUrl] && item[mapper.imageUrl] != "") {
                    imgTag = $(document.createElement('img'));
                    imgTag.attr('src', item[mapper.imageUrl]);
                    if (item[mapper.imageAttribute]) this._setAttributes(item[mapper.imageAttribute], imgTag);
                    aTag.append(imgTag);
                }
                else if (item[mapper.spriteCssClass] && item[mapper.spriteCssClass] != "") {
                    spanTag = $(document.createElement('span'));
                    spanTag.addClass(item[mapper.spriteCssClass]);
                    aTag.append(spanTag);
                }
                aTag.append(item[mapper.text]);
                if (item[mapper.linkAttribute]) this._setAttributes(item[mapper.linkAttribute], aTag);
                if (item[mapper.url])
                    aTag.attr('href', item[mapper.url]);
                liTag.append(aTag);
            }
            if (item[mapper.id]) {
                liTag.prop("id", item[mapper.id]);
            }
            if (!ej.isNullOrUndefined(mapper["child"])) {
                this._odataFlag = true;
                if (mapper["child"]["dataSource"] instanceof ej.DataManager) {
                    var proxy = this, queryManager = ej.Query();
                    queryManager = this._columnToSelect(mapper["child"]);
                    queryManager.where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]);
                    var queryPromise = mapper["child"]["dataSource"].executeQuery(queryManager);
                    queryPromise.done(function (e) {
                        var childItems = e.result;
                        if (childItems && childItems.length > 0) {
                            var ul = $(document.createElement('ul'));
                            for (var i = 0; i < childItems.length; i++) {
                                var liItem = proxy._menuTemplate(childItems[i], mapper["child"]["dataSource"], mapper["child"]);
                                ul.append(liItem);
                            }
                            liTag.append(ul);
                            $(liTag).children('a').addClass('aschild');
                            if ($(liTag).parent().hasClass('e-menu') && (proxy.model.showRootLevelArrows))
                                $(liTag).children('a.aschild').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
                            else if (proxy.model.showSubLevelArrows)
                                $(liTag).children('a.aschild').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
                        }
                    });
                }
                else {
                    var childItems = ej.DataManager(mapper["child"]["dataSource"]).executeLocal(ej.Query().where(mapper["child"]["parentId"], ej.FilterOperators.equal, item[mapper.id]));
                    if (childItems && childItems.length > 0) {
                        var ul = $(document.createElement('ul'));
                        for (var i = 0; i < childItems.length; i++) {
                            var liItem = this._menuTemplate(childItems[i], mapper["child"]["dataSource"], mapper["child"]);
                            ul.append(liItem);
                        }
                        liTag.append(ul);
                    }
                }
            }
            else if (!this._odataFlag) {
                var childItems = ej.DataManager(mapper["dataSource"]).executeLocal(ej.Query().where(mapper["parentId"], ej.FilterOperators.equal, item[mapper.id]));
                if (childItems && childItems.length > 0) {
                    var ul = ej.buildTag('ul');
                    for (var i = 0; i < childItems.length; i++) {
                        var liItem = this._menuTemplate(childItems[i], mapper["dataSource"], mapper);
                        ul.append(liItem);
                    }
                    liTag.append(ul);
                }
            }
            return liTag;
        },

        _setAttributes: function (data, element) {
            for (var key in data) {
                if (key == "class")
                    element.addClass(data[key]);
                else
                    element.attr(key, data[key]);
            }
        },

        _addArrow: function (topArrows, bottomArrows) {
            if (topArrows)
                this.element.find('>li.e-list:has("> ul")').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else {
                this.element.find('>li.e-list:has("> ul")').children('a,span').removeClass("e-arrow-space").children('span.e-icon').remove();
            }

            if (bottomArrows)
                this.element.find('>li.e-list > ul li.e-list:has(>ul)').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else {
                this.element.find('>li.e-list > ul li.e-list:has(>ul)').children('a,span').removeClass("e-arrow-space").children('span.e-icon').remove();
            }

        },

        _intializeMenu: function () {
            if (this.model.height != 0) this._setHeight(this.model.height);
            if (this.model.width != 0) this._setWidth(this.model.width);
            if (this.model.menuType == "contextmenu")
                this.model.openOnClick = false;
            if (this.model.openOnClick) {
                this._hoverOpen = false;
                this._hoverClose = false;
            }
        },

        _setOrientation: function (val) {
            if (val == "horizontal") {
                this.element.removeClass("e-vertical e-horizontal").addClass("e-horizontal");
            } else {
                this.element.removeClass("e-horizontal e-vertical").addClass("e-vertical");
            }
        },

        _setHeight: function (value) {
            if (this.model.orientation == "horizontal") {
                value = typeof value === "number" ? value + "px" : value;
                this.element.find('> li').find('>a:first').css("line-height", value);
                if (this.model.showRootLevelArrows)
                    this.element.find('> li').find('>a:first').find('> span:first').css({ "line-height": value, "top": "0px" })
            }
            else
                this.element.height(value);
        },

        _setWidth: function (value) {
            this.element.css("width", value);
            if (this.model.orientation === "horizontal")
                this.resWrap.css("width", value);
        },

        _setRTL: function (isRTL) {
            if (isRTL) {
                this.element.removeClass("e-rtl").addClass("e-rtl");
            } else {
                this.element.removeClass("e-rtl");
            }
            if (isRTL && this.model.orientation === "horizontal")
                this.wrapper.removeClass("e-menu-rtl").addClass("e-menu-rtl");
            else
                this.wrapper.removeClass("e-menu-rtl");
        },

        _setSubMenuDirection: function (direction) {
            if (direction != "left" && direction != "right")
                this.model.subMenuDirection = this.model.enableRTL ? "left" : "right";
        },

        _setAnimation: function (value) {
            value === "none" ? (this._showAnim = "none", this._hideAnim = "none") : (this._showAnim = "slideDown", this._hideAnim = "slideUp");
        },

        _controlStatus: function (value) {
            value != true ? this.disable() : this.enable();
        },

        _centerAlign: function (enableCenterAlign) {
            if (this.model.orientation == "horizontal" && enableCenterAlign)
                this.element.css('text-align', 'center');
            else
                this.element.css('text-align', 'inherit');
        },
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "child" && col !== "dataSource" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },


        _max_zindex: function () {
            var parents, bodyEle, maxZ;
            if (this.model.menuType == "contextmenu") {
                parents = $(this._targetElement).parents();
                parents.push(this._targetElement);
            }
            else
                parents = $(this.element).parents();
            bodyEle = $('body').children(), index = bodyEle.index(this.popup);
            bodyEle.splice(index, 1);
            $(bodyEle).each(function (i, ele) { parents.push(ele); });
            maxZ = Math.max.apply(maxZ, $.map(parents, function (e, n) {
                if ($(e).css('position') != 'static') return parseInt($(e).css('z-index')) || 1;
            }));
            if (!maxZ || maxZ < 10000) maxZ = 10000;
            else maxZ += 1;
            return maxZ;

        },

        _recursiveFunction: function (items, menuText) {
            var context = this;
            var isFound = false;
            $.each(items, function (key, value) {
                if (value.Text == menuText) {
                    context.selectedItem = value;
                    isFound = true;
                    return false;
                }
                else if (value.ChildItems != null) {
                    context._recursiveFunction(value.ChildItems, menuText);
                }
                if (isFound)
                    return false;
            });
        },

        _contextMenu_Template: function () {
            var oldWrapper = $(".e-menu-wrap #" + this.element.context.id).get(0);
            if (oldWrapper)
                $(oldWrapper.parentElement).remove();
            this.model.orientation = "vertical";
            this.element.addClass(this.model.cssClass + " e-context");
			 this.element.css("display", "none");
            $("body").append(this.element);
            this.wrapper.insertBefore(this.element);
            this.wrapper.append(this.element);
        },

        _closeMenu: function () {
            this._hideAnimation(this.element.find('li.e-list:has("> ul")').find('> ul:visible'), this._hideAnim);
        },

        _onMenuIntent: function (element, obj, canOpen) {
            obj._delayMenuHover = window.setTimeout(function () {
                if (obj._mouseOver == true && canOpen) {
                    var showanim = obj._showAnim;
                    var hideanim = obj._hideAnim;
                    var showSpeed = obj._showSpeed;
                    var hideSpeed = obj._hideSpeed;
                    obj._show(element, showanim, hideanim);
                }
            }, this._showSpeed);
        },

        _onHide: function (element, obj, canHide) {
            obj._delaySubMenuHover = window.setTimeout(function () {
                if (obj._mouseOver == false && canHide) {
                    var id = obj._id;
                    var hideanim = obj._hideAnim;
                    var hideSpeed = obj._hideSpeed;
                    obj._closeAll();
                }

            }, obj._hideSpeed);
        },

        _subMenuPos: function (element, direction) {
            var parentPos = $(this.wrapper).parent().offset();
            var pos = $(element).offset();
            var posLeft = pos.left - parentPos.left;
            var subMenu = $('ul:first', element);
            var menuWidth = $(element).outerWidth();
            if (pos == null || pos == undefined)
                return false;
            var submenuWidth = subMenu.outerWidth() + 1; // +1 for the space between menu and submenu
            var left = document.documentElement.clientWidth + $(document).scrollLeft();
            if (this.model.menuType == "normalmenu") {
                if ($(element.parentNode).is(this.element)) {
                    if (this.model.orientation == "horizontal")
                        subMenu.css("top", $(element).outerHeight() + "px");
                    else if ((direction == "left" && posLeft > submenuWidth) || (direction == "right" && left <= pos.left + menuWidth + submenuWidth))
                        subMenu.css("left", -(submenuWidth + 4) + "px");
                    else {
                        subMenu.css("left", ($(element).outerWidth() + 4) + "px");
                    }
                } else if ((direction == "left" && posLeft > submenuWidth) || (direction == "right" && left <= pos.left + menuWidth + submenuWidth)) {
                    subMenu.css("left", -(submenuWidth + 4) + "px");
                }
                else
                    subMenu.css("left", ($(element).outerWidth() + 4) + "px");
            }
            else {
                left -= (pos.left + (2 * submenuWidth) + 4);
                if (left < 0) {
                    var menuLeftPos = (submenuWidth == null) ? "-206.5px" : "-" + (submenuWidth + 5) + "px";
                    subMenu.css("left", menuLeftPos);
                }
                else {
                    if (subMenu.parent('li.e-list').parent('ul').width() && direction == "right") {
                        subMenu.css("left", (subMenu.parent('li.e-list').parent('ul').width() + 4) + "px");
                    }
                    else if (pos.left > submenuWidth)
                        subMenu.css("left", -(submenuWidth + 4) + "px");
                }
                var submenuHeight = subMenu.outerHeight();
                if ((pos.top + submenuHeight > $(window).height())) {
                    var top = -(submenuHeight) + $(element).outerHeight();
                    if (submenuHeight > (pos.top + ($(element).outerHeight() / 2))) {
                        subMenu.css("top", -(submenuHeight / 2) + "px");
                    }
                    else
                        subMenu.css("top", top + "px");
                }
                else
                    subMenu.css("top","0px");
            }
        },


        _setSkin: function (skin) {
            this.wrapper.removeClass(this.model.cssClass).addClass(skin + " e-menu-wrap");
        },

        _setSeparator: function (separator) {
            if (separator)
                this.element.addClass("e-separator");
            else this.element.removeClass("e-separator");
        },

        _contextMenuEvents: function (action) {
            this[action]($(this.model.contextMenuTarget), "mouseup taphold", this._ContextMenuHandler);
            this[action](this.element, "contextmenu", this._onDefaultPreventer);
            this[action]($(this.model.contextMenuTarget), "contextmenu", this._onDefaultPreventer);
            this[action]($(document), "mousedown", this._onContextClose);
        },

        _show: function (element, showanim, hideanim) {
            var siblingElement;
            var sibling = $('> ul', element);
            sibling.attr({ "aria-hidden": false });
            this._hideAnimation($(element).siblings().find(' > ul:visible'), hideanim);
            if (!($.inArray(this._disabledMenuItems, element) > -1)) {
                if (sibling.css('display') != "none") {
                    siblingElement = this.model.openOnClick ? $(sibling) : sibling.children().find('> ul');
                    this._hideAnimation(siblingElement, hideanim);
                }
                else $('> ul', element).children().find('> ul').hide();
                this._subMenuPos(element, this.model.subMenuDirection);
                sibling.css({ "z-index": this._max_zindex() + 1 });
                if ($('> ul', element).css('display') != 'block' && !$(element).hasClass("e-disable-item"))
                    this._showAnimation(sibling, showanim);
                if ($(element).siblings("li.e-active").length > 0)
                    $(element).siblings("li.e-active").removeClass("e-active");
                sibling.closest('li').addClass('e-active');
            }
        },

        _closeAll: function () {
            this._hideAnimation(this.element.find('li.e-list:has("> ul")').find('> ul:visible'), this._hideAnim);
            this._hideAnimation(this.element.find('> ul:visible'), this._hideAnim);
        },

        _showAnimation: function (element, anim) {
            switch (anim) {
                case "slideDown":
                    element.slideDown(this.model.enableAnimation ? 200 : 0, "easeOutQuad"); break;
                case "none":
                    element.css("display", "block"); break;
            }
        },

        _hideAnimation: function (element, anim) {
            switch (anim) {
                case "slideUp":
                    $(element).attr({ "aria-hidden": true });
                    element.slideUp(this.model.enableAnimation ? 100 : 0, "easeOutQuad"); break;
                case "none":
                    element.css("display", "none"); break;
            }
            element.closest('li').removeClass('e-active');
        },

        _removeValue: function (text, disableList) {
            var $browInfo = ej.browserInfo(), elementText;
            $browInfo.version === "8.0" && $browInfo.name === "msie" ? elementText = text[0].outerText : elementText = text[0].textContent;
            var count = $(disableList).length, i = 0;
            var childEle = $(disableList).children('a').length == 0 ? $(disableList).children('span') : $(disableList).children('a');
            while (i <= count) {
                if ($(childEle[i]).text() === elementText)
                    return i;
                i++;
            }
        },

        _createSubLevelItem: function (target, element) {
            var ulTag;
            ulTag = $(document.createElement('ul'));
            ulTag.append(element);
            target.append(ulTag);
            target.attr({ 'role': 'menu', 'aria-haspopup': 'true' });
            target.addClass("e-haschild");
            this.element.find('li:has("> ul")').find('> a,>span').addClass('aschild e-arrow-space');
            this._insertArrows(ulTag);
        },

        _insertArrows: function (ulTag) {
            if (this.model.showRootLevelArrows)
                ulTag.find('>a,>span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else
                ulTag.find('>a,>span').removeClass("e-arrow-space").find('>span.e-icon').remove();

            if (this.model.showSubLevelArrows)
                ulTag.parent('li.e-list:has(>ul)').children('a,span').append($('<span>').addClass("e-icon e-arrows")).addClass("e-arrow-space");
            else
                ulTag.parent('li.e-list:has(>ul)').children('a,span').removeClass("e-arrow-space").find('>span.e-icon').remove();
        },

        _createMenuItem: function (item) {
            var liTag, aTag, imgTag, spanTag;
            liTag = $(document.createElement('li'));
            liTag.attr({ "class": 'e-list', "role": "menuitem" });
            if (item["htmlAttribute"]) this._setAttributes(item["htmlAttribute"], liTag);
            if (item["text"] && item["text"] != "") {
                aTag = $(document.createElement('a'));
                if (item["imageUrl"] && item["imageUrl"] != "") {
                    imgTag = $(document.createElement('img'));
                    imgTag.attr('src', item["imageUrl"]);
                    if (item["imageAttribute"]) this._setAttributes(item["imageAttribute"], imgTag);
                    aTag.append(imgTag);
                }
                else if (item["spriteCssClass"] && item["spriteCssClass"] != "") {
                    spanTag = $(document.createElement('span'));
                    spanTag.addClass(item["spriteCssClass"]);
                    aTag.append(spanTag);
                }
                aTag.append(item["text"]);
                if (item["linkAttribute"]) this._setAttributes(item["linkAttribute"], aTag);
                if (item["url"])
                    aTag.attr('href', item["url"]);
                liTag.append(aTag);
            }
            if (item["id"]) {
                liTag.prop("id", item["id"]);
            }
            if (!this.model.enabled)
                liTag.addClass("e-disable-item");
            return liTag;
        },

        _insertNode: function (itemCollection, targetNode, operation) {
            var item = 0, targetList = 0, target = 0, targetCollection = [];
            if ($(targetNode).is(this.element))
                targetCollection.push(this.element);
            else
                typeof (targetNode) === "string" ? targetCollection.push(this.element.find(targetNode)) : typeof (targetNode) === "undefined" ? targetCollection.push(this.element) : targetCollection.push(targetNode);
            for (targetList = 0; targetList < targetCollection.length; targetList++) {
                for (target = 0; target < targetCollection[targetList].length; target++)
                    for (item = 0; item < itemCollection.length && !ej.isNullOrUndefined(itemCollection[item]) ; item++)
                        this._addItem(itemCollection[item], targetCollection[targetList][target], operation);
            }
        },

        _addItem: function (item, target, operation) {
            var element, targetElement;
            this._wireEvents("_off");
            element = this._createMenuItem(item);
            target = target === "default" ? $("#" + item["parentId"]) : $(target);
            switch (operation) {
                case "insert":
                    $(target).is(this.element) ? targetElement = target : targetElement = target.children('ul');
                    targetElement.length != 0 ? targetElement.append(element) : this._createSubLevelItem(target, element);
                    break;
                case "insertBefore":
                    if (!$(target).is(this.element))
                        element.insertBefore(target);
                    else
                        target.prepend(element);
                    break;
                case "insertAfter":
                    if (!$(target).is(this.element))
                        element.insertAfter(target);
                    else
                        target.append(element);
                    break;
            }
            this._wireEvents("_on");
        },

        _removeItem: function (item) {
            if (item.siblings('li').length == 0) {
                item.closest("ul").siblings('a.aschild').removeClass("aschild e-arrow-space").children('span.e-icon').remove();
                !item.closest("ul").hasClass("e-menu") ? item.closest("ul").remove() : item.remove();
            }
            else
                item.remove();
        },

        insert: function (item, target) {
            this._insertNode(item, target, "insert");
        },

        insertBefore: function (item, target) {
            this._insertNode(item, target, "insertBefore");
        },

        insertAfter: function (item, target) {
            this._insertNode(item, target, "insertAfter");
        },

        remove: function (targetCollection) {
            var target = 0, innerTarget = 0;
            for (target = 0; target < targetCollection.length; target++) {
                targetCollection[target] = typeof (targetCollection[target]) === "string" ? (this.element.find(targetCollection[target])) : targetCollection[target];
                for (innerTarget = 0; innerTarget < targetCollection[target].length; innerTarget++)
                    (targetCollection[target][innerTarget].tagName === "LI" || targetCollection[target][innerTarget].tagName === "UL") ? this._removeItem($(targetCollection[target][innerTarget])) : targetCollection[target][innerTarget].remove();
            }
        },

        showContextMenu: function (locationX, locationY, targetElement, e, update) {
            this._closeMenu();
            this._eventArgs = e;
            if (!ej.isNullOrUndefined(e) && this._checkForExclusion(e.target)) return;
            if (this._trigger("beforeOpen", { target: targetElement, events: e })) return false;
            if (this._preventContextOpen) {
                if (!ej.isNullOrUndefined(targetElement))
                    this._targetElement = targetElement;
                else if (!ej.isNullOrUndefined(target))
                    this._targetElement = target;
                else
                    this._targetElement = this.element;
                if (update) {
                    var position = this._calculateContextMenuPosition(e);
                    locationX = position.X;
                    locationY = position.Y;
                }
                this.element.css({ "left": locationX, "top": locationY });
                this.element.css({ "z-index": this._max_zindex() + 1 });
                this._showAnimation(this.element, this._showAnim);
                this._isContextMenuOpen = true;
                this.element.focus();

                this._trigger("open", { target: targetElement });
                this._on(ej.getScrollableParents($(this.model.contextMenuTarget)), "scroll", this.hideContextMenu);
            }
            return false;
        },

        _checkForExclusion: function (e) {
            if (!ej.isNullOrUndefined(this.model.excludeTarget)) {
                var excludeTargets = this.model.excludeTarget.split(",");
                for (var target = 0; target < excludeTargets.length; target++) {
                    if ($(e).closest(this.model.excludeTarget).is($.trim(excludeTargets[target])))
                        return true;
                }
            }
        },


        hideContextMenu: function (e) {
            this._closeMenu();
            this.element.find(".e-mhover").removeClass("e-mhover");
            this.element.find(".e-mfocused").removeClass("e-mfocused");
            this._hideAnimation(this.element, this._hideAnim);
            this._isContextMenuOpen = false;

            this._trigger("close", $.extend({ events: e }, e));
            this._off(ej.getScrollableParents($(this.model.contextMenuTarget)), "scroll", this.hideContextMenu);
        },


        disableItem: function (itemToDisable) {
            var isMenuItem = $(this.element.find('li.e-list >a ,li.e-list >span')).filter(function () { return $.trim($(this).text()) === itemToDisable; });
            if (isMenuItem.length > 0 && !($.inArray(isMenuItem.parent()[0], this._disabledMenuItems) > -1)) {
                isMenuItem.parent().addClass("e-disable-item").attr({ "aria-disabled": true });
                isMenuItem.parent().find('>a.aschild span.e-icon.e-arrows').addClass("e-disable");
                this._disabledMenuItems.push(isMenuItem.parent()[0]);
            }
        },


        disableItemByID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToDisable = this.element.find("#" + itemId) ? this.element.find("#" + itemId)[0] : undefined;
                if (itemToDisable && !($.inArray(itemToDisable, this._disabledMenuItems) > -1)) {
                    $(itemToDisable).addClass("e-disable-item").attr({ "aria-disabled": true });
                    $(itemToDisable).find('>a.aschild span.e-icon.e-arrows').addClass("e-disable");
                    this._disabledMenuItems.push(itemToDisable);
                }
            }
        },

        enableItem: function (itemToEnable) {
            var isMenuItem = $(this.element.find('li.e-list >a ,li.e-list >span')).filter(function () { return $.trim($(this).text()) === itemToEnable; });
            if (isMenuItem.length > 0 && ($.inArray(isMenuItem.parent()[0], this._disabledMenuItems) > -1)) {
                isMenuItem.parent().removeClass("e-disable-item").attr({ "aria-disabled": false });
                isMenuItem.parent().find('>a.aschild span.e-icon.e-arrows').removeClass("e-disable");
                var index = this._removeValue(isMenuItem, this._disabledMenuItems);
                this._disabledMenuItems.splice(index, 1);
            }
        },


        enableItemByID: function (itemId) {
            if (itemId && itemId != "") {
                var itemToEnable = this.element.find("#" + itemId)[0];
                if (itemToEnable && ($.inArray(itemToEnable, this._disabledMenuItems) > -1)) {
                    $(itemToEnable).removeClass("e-disable-item").attr({ "aria-disabled": false });
                    $(itemToEnable).find('>a.aschild span.e-icon.e-arrows').removeClass("e-disable");
                    for (var i = this._disabledMenuItems.length - 1; i >= 0; i--) {
                        if (this._disabledMenuItems[i].id == itemId) {
                            this._disabledMenuItems.splice(i, 1);
                        }
                    }
                }
            }
        },


        disable: function () {
            this.model.enabled = false;
            var menuItemCollection = this.element.find('>li[class!=e-separator]');
            var proxy = this;
            $.each(menuItemCollection, function (key, value) {
                $(value).addClass("e-disable-item").attr({ "aria-disabled": true });
                $(value).find('>a.aschild span.e-icon.e-arrows').addClass("e-disable");
                proxy._disabledMenuItems.push(value);
            });
        },


        enable: function () {
            var proxy = this;
            this.model.enabled = true;
            var menuItemCollection = this.element.find('li.e-disable-item');
            $.each(menuItemCollection, function (key, value) {
                $(value).removeClass("e-disable-item").attr({ "aria-disabled": false });
                $(value).find('>a.aschild span.e-icon.e-arrows').removeClass("e-disable");
                proxy._disabledMenuItems.pop(value);
            });
        },

        show: function (locationX, locationY, targetElement, e) {
            if (!this.model.enabled) return false;
            if (this.model.menuType == "contextmenu")
                this.showContextMenu(locationX, locationY, targetElement, e, false);
            else
                this.element.css("display", "block");
        },

        hide: function (e) {
            if (!this.model.enabled) return false;
            if (this.model.menuType == "contextmenu")
                this.hideContextMenu(e);
            else {
                this._closeMenu();
                this.element.css("display", "none");
            }
        },

        _wireEvents: function (action) {
            this[action](this.element.find("li.e-list"), "mouseout", this._mouseOutHandler);
            this[action](this.element.find("li.e-list"), "mouseover", this._mouseOverHandler);
            this[action](this.element, "click", this._onClickHandler);
            this[action](this.element, "keydown", this._onKeyDownHandler);
            this[action](this.element, "focus", this._OnFocusHandler);
            this[action](this.element, "blur", this._OnFocusOutHandler);
            if (this.model.menuType == "contextmenu" && $(this.model.contextMenuTarget)[0] != null) {
                this._contextMenuEvents(action);
            }
            if (this.model.menuType != "contextmenu") {
                this[action]($(document), "click", this._onDocumentClick);
                this[action](this.element, "mousedown", this._onMouseDownHandler);
            }

        },

        _mouseOverHandler: function (event) {
            var element, itemId = "";
            this.element.find(".e-mhover").removeClass("e-mhover e-mfocused");
            event.currentTarget = $(event.target).closest("li")[0];
            if (!$(event.currentTarget).hasClass('e-disable-item'))
                $(event.currentTarget).addClass("e-mhover e-mfocused");
            else this._isFocused = false;
            if (event.stopPropagation)
                event.stopPropagation();
            if (typeof (this._delaySubMenuHover) !== 'undefined') {
                clearTimeout(this._delaySubMenuHover);
            }
            if (typeof (this._delaySubMenuHover) !== 'undefined') {
                clearTimeout(this._delayMenuHover);
            }
            this._mouseOver = true;
            this._isMenuOpen = true;
            if ($(event.currentTarget.parentNode.parentNode).is(this.element)) {
                this._isSubMenuOpen = false;
            }
            else {
                this._isSubMenuOpen = true;
            }
            if (event.currentTarget.nodeName == "LI" && !$(event.currentTarget).hasClass('e-separator'))
                element = event.currentTarget;
            else if (event.currentTarget.parentNode) {
                if (event.currentTarget.parentNode.nodeName == "LI" && !$(event.currentTarget.parentNode).hasClass('e-separator'))
                    element = event.currentTarget.parentNode;
                else
                    return false;
            }
            else {
                event.preventDefault();
                return false;
            }
            if (!$(event.currentTarget).hasClass('e-disable-item'))
                this._onMenuIntent(element, this, this._hoverOpen);
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                var menuText = $(element).children('a,span').text();
                itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id : "";
                var eventArgs = { "text": menuText, "element": element, "event": event, "ID": itemId };

                this._trigger("mouseover", $.extend({ events: eventArgs }, eventArgs));
            }
        },

        _onMouseDownHandler: function (e) {
            this._isFocused = false;
        },


        _mouseOutHandler: function (event) {
            var element, itemId = "";
            $(event.currentTarget).removeClass("e-mhover e-mfocused");
            if (event.stopPropagation)
                event.stopPropagation();
            if (typeof (this._delaySubMenuHover) !== 'undefined') {
                clearTimeout(this._delaySubMenuHover);
            }
            if (typeof (this._delaySubMenuHover) !== 'undefined') {
                clearTimeout(this._delayMenuHover);
            }
            this._mouseOver = false;
            this._isMenuOpen = false;

            if (event.currentTarget.nodeName == "LI" && !$(event.currentTarget).hasClass('e-separator'))
                element = event.currentTarget;
            else if (event.currentTarget.parentNode) {
                if (event.currentTarget.parentNode.nodeName == "LI" && !$(event.currentTarget.parentNode).hasClass('e-separator'))
                    element = event.currentTarget.parentNode;
                else
                    return false;
            }
            else {
                event.preventDefault();
                return false;
            }
            this._onHide(element, this, this._hoverClose);
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                var menuText = $(element).children('a,span').text();
                itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id : "";
                var eventArgs = { "text": menuText, "element": element, "event": event, "ID": itemId };

                this._trigger("mouseout", $.extend({ events: eventArgs }, eventArgs));
            }
        },

        _onClickHandler: function (event) {
            var element, itemId = "";
            this._isFocused = true;
            var openOnClickStart = false;
            if (!$(event.target).closest("li.e-list").hasClass('e-separator') && !$(event.target).closest("li.e-list").hasClass('e-disable-item') && $(event.target).closest("li.e-list").length > 0) {
                element = $(event.target).closest("li.e-list")[0];
                if ($(element).is(this.element.find(">li.e-list")))
                    this._activeElement = element;
            }
            else {
                if ($(event.target).is(this.element))
                    this._activeElement = this.element.find(">li:first");
                return;
            }
            if ($(event.target).is("a") && $(element).find(">a,>span").hasClass('aschild') && this.model.openOnClick) {
                this._isFocused = false;
            }
            if (!this._hoverOpen && $(element).find(">a,>span").hasClass('aschild')) {
                this._show(element, this._showAnim, this._hideAnim);
                this._hoverOpen = false;
                openOnClickStart = true;
            }
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                //Check if Context Menu, then hide the context menu firing the events
                if (this.model.menuType == "contextmenu") {
                    if (this._isContextMenuOpen) {
                        this._hideAnimation(this.element, this._hideAnim);
                        this._isContextMenuOpen = false;

                        this._trigger("close", $.extend({ events: event }, event));
                    }
                }
                if (!openOnClickStart) {



                    if (!$(element).find(">a,>span").hasClass("aschild")) {
                        this._closeMenu();
                        if (this.model.openOnClick)
                            this._hoverOpen = false;
                    }
                }
                var menuText = $(element).children('a,span').text();
                var parent = $(element).closest("ul").parent("li");
                if (parent.length != 0) {
                    parentId = parent.attr("id");
                    parentText = parent.children('a,span').text();
                }
                else {
                    parentId = $(element).attr("id");
                    parentText = menuText;
                }
                itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id : "";
                var eventArgs = { "text": menuText, "element": element, "event": event, "selectedItem": this.selectedItem, "ID": itemId, "parentId": parentId, "parentText": parentText };
                this._trigger("click", $.extend({ events: eventArgs }, eventArgs));
                this.selectedItem = null;
                if (this.model.openOnClick && this.model.menuType != "contextmenu")
                    this.element.focus();
            }
        },


        _onKeyDownHandler: function (e) {
            if (this.model.menuType == "contextmenu" && !this._isContextMenuOpen)
                return;
            var element, itemId = "", hoverElement = this.element.find(".e-mhover"), focusedElement = this.element.find(".e-mfocused");
            if (!$(hoverElement).length > 0 && $(this._activeElement).length > 0)
                hoverElement = focusedElement = $(this._activeElement);

            if (e.keyCode == 9) {
                this._isFocused = false;
                this._OnFocusOutHandler();
            }
            else if (e.keyCode == 37 || e.keyCode == 38 || e.keyCode == 39 || e.keyCode == 40)
                e.preventDefault();

            if (e.keyCode == 40) {
                if (this.model.orientation != "horizontal" && $(hoverElement).length > 0) {
                    focusedElement.removeClass("e-mfocused");
                    if (hoverElement.next().hasClass("e-separator"))
                        hoverElement.next().next().addClass("e-mfocused");
                    else if (hoverElement.is(':last-child')) {
                        hoverElement.siblings().first().addClass("e-mfocused");
                    }
                    else
                        hoverElement.next().addClass("e-mfocused");
                }
                else {
                    if (this.model.menuType == "contextmenu")
                        this.element.find("li").first().addClass("e-mfocused e-mhover");
                }
                if (this.model.orientation == "horizontal" && (this.element.find(">li.e-mhover").children("ul").length > 0 || $(this._activeElement).length > 0)) {
                    if ($(hoverElement).children("ul").css('display') === "none")
                        this._show(hoverElement[0], this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                    if ($(this._activeElement).length > 0)
                        $(this._activeElement).addClass("e-mfocused");
                }
                else {
                    if (hoverElement.is(':last-child')) {
                        hoverElement.removeClass("e-mhover").siblings().first().addClass("e-mhover");
                    }
                    else if (hoverElement.next().hasClass("e-separator"))
                        hoverElement.removeClass("e-mhover").next().next().addClass("e-mhover");
                    else {
                        hoverElement.removeClass("e-mhover").next().addClass("e-mhover");
                    }
                }
            }
            if (e.keyCode == 39) {
                if (this.model.orientation == "horizontal" && (this.element.find(">li.e-list").hasClass("e-mhover") || $(this._activeElement).length > 0)) {
                    this._closeAll();
                    if (hoverElement.is(":last-child")) {

                        hoverElement.removeClass("e-mhover e-mfocused");
                        if (this.element.find("li.e-list").first().hasClass("e-separator")) this.element.find("li.e-list").first().next().addClass("e-mhover e-mfocused");
                        else this.element.find("li.e-list").first().addClass("e-mhover e-mfocused");
                    }
                    else if (hoverElement.next().hasClass("e-separator")) {

                        hoverElement.removeClass("e-mhover e-mfocused").next().next().addClass("e-mhover e-mfocused");
                    }
                    else {

                        hoverElement.removeClass("e-mhover e-mfocused").next().addClass("e-mhover e-mfocused");
                    }
                }
                else if ($(hoverElement).children("ul").length > 0) {
                    this._show(hoverElement[0], this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                }
                else if (hoverElement.children("ul").length <= 0) {
                    this._closeMenu();
                    hoverElement.removeClass("e-mhover");
                    if (focusedElement.next().hasClass("e-separator")) {
                        focusedElement.removeClass("e-mfocused").next().next().addClass("e-mhover e-mfocused");

                    } else if (focusedElement.next().length <= 0) {
                        focusedElement.removeClass("e-mfocused").siblings().first().addClass("e-mhover e-mfocused");
                    } else focusedElement.removeClass("e-mfocused").next().addClass("e-mhover e-mfocused");
                }
            }

            if (e.keyCode == 38) {
                if (this.model.orientation != "horizontal" && $(hoverElement).length > 0) {
                    hoverElement.children("ul").hide();
                    focusedElement.removeClass("e-mfocused");
                    if (hoverElement.prev().hasClass("e-separator"))
                        hoverElement.prev().prev().addClass("e-mfocused");
                    else if (hoverElement.is(':first-child')) {
                        hoverElement.siblings().last().addClass("e-mfocused");
                    }
                    else
                        hoverElement.prev().addClass("e-mfocused");
                }
                else {
                    if (this.model.menuType == "contextmenu")
                        this.element.find("li").last().addClass("e-mfocused e-mhover");
                }
                if (hoverElement.is(':first-child')) {
                    hoverElement.removeClass("e-mhover e-mfocused").siblings().last().addClass("e-mhover");
                }
                else if (hoverElement.prev().hasClass("e-separator"))
                    hoverElement.removeClass("e-mhover").prev().prev().addClass("e-mhover");
                else
                    hoverElement.removeClass("e-mhover").prev().addClass("e-mhover");

            }

            if (e.keyCode == 37) {
                if (this.model.orientation == "horizontal" && (this.element.find(">li.e-list").hasClass("e-mhover") || $(this._activeElement).length > 0)) {
                    this._closeAll();
                    if (hoverElement.is(":first-child")) {
                        hoverElement.removeClass("e-mhover e-mfocused");
                        if (this.element.find(">li.e-list").last().hasClass("e-separator"))
                            this.element.find(">li.e-list").last().prev().addClass("e-mhover e-mfocused");
                        else
                            this.element.find(">li.e-list").last().addClass("e-mhover e-mfocused");
                    }
                    else if (hoverElement.prev().hasClass("e-separator")) {
                        hoverElement.removeClass("e-mhover e-mfocused").prev().prev().addClass("e-mhover e-mfocused");
                    }
                    else {
                        hoverElement.removeClass("e-mhover e-mfocused").prev().addClass("e-mhover e-mfocused");
                    }
                }
                else if ($(hoverElement).length > 0) {
                    if (!hoverElement.parent().hasClass("e-context")) {
                        this._hideAnimation(hoverElement.parent(), this._hideAnim);
                        hoverElement.parent().closest("li.e-list").addClass("e-mhover");
                        hoverElement.removeClass("e-mhover");
                    }
                }
                else {
                    this._hideAnimation(hoverElement, this._showAnim, this._hideAnim);
                    hoverElement.removeClass("e-mhover").parent().addClass("e-mhover");
                }
            }
            if (e.keyCode == 13) {
                var menuText = $(hoverElement).children('a,span').text();
                itemId = !ej.isNullOrUndefined($(hoverElement)[0]) ? $(hoverElement)[0].id : "";
                var eventArgs = { "menuId": this.element[0].id, "text": menuText, "selectedItem": focusedElement, "ID": itemId };
                if (this.model.menuType == "contextmenu") {
                    if (this._isContextMenuOpen && hoverElement.length > 0 && !focusedElement.hasClass("e-disable-item")) {
                        if (this.model.click)
                            this._trigger("click", $.extend({ events: eventArgs }, eventArgs));
                        this.selectedItem = null;
                        this.hideContextMenu(e);
                    }
                } else {
                    if (hoverElement.length > 0 && !hoverElement.hasClass("e-disable-item")) {
                        if ($(hoverElement).find(">a,>span").hasClass('aschild') && $(hoverElement).children("ul").css('display') === "none") {
                            this._show(hoverElement[0], this._showAnim, this._hideAnim);
                            hoverElement.removeClass("e-mhover").children("ul:first").find("li:first").addClass("e-mhover");
                        }
                        else {
                            this.element.find(".e-mhover >a,.e-mhover >span ").focus();
                            this.element.find("li.e-list").removeClass("e-mhover e-mfocused");
                            this._closeAll();
                        }
                        if (ej.isNullOrUndefined($(hoverElement).find(">a").attr("href")))
                            this._trigger("click", $.extend({ events: eventArgs }, eventArgs));
                    }
                }
            }
            if (e.keyCode == 27) {
                if (this.model.menuType == "contextmenu")
                    this.hideContextMenu(e);
                else
                    this._closeAll();
            }
            if (e.target.nodeName == "DIV" && $(e.target).children().is(this.element))
                element = $(e.target).children()[0];
            else if (e.target.nodeName == "LI" && !$(e.target).hasClass('e-separator'))
                element = e.target;
            else if (e.target.parentNode) {
                if (e.target.parentNode.nodeName == "LI" && !$(e.target.parentNode).hasClass('e-separator'))
                    element = e.target.parentNode;
            }
            else
                return false;
            if (!($.inArray(element, this._disabledMenuItems) > -1)) {
                var menuText = $(element).children('a,span').text();

                itemId = !ej.isNullOrUndefined(element) ? $(element)[0].id : "";
                var eventArgs = { "text": menuText, "element": element, "event": e, "ID": itemId };

                this._trigger("keydown", $.extend({ events: eventArgs }, eventArgs));
            }
            this._activeElement = null; focusedElement = this.element.find(".e-mfocused");
        },

        _OnFocusHandler: function (event) {
            if (this.model.menuType != "contextmenu" && !this.element.find(">li:first").hasClass("e-disable-item") && this._isFocused && this.element.find(".e-mhover").length == 0) {
                this.element.find(">li:first").addClass("e-mhover e-mfocused");
            }
            else this._isFocused = true;
            if (this.model.menuType != "contextmenu")
                this._activeElement = this.element.find(">li:first");
        },

        _OnFocusOutHandler: function () {
            if (!this._isFocused) {
                this.element.find("li.e-list").removeClass("e-mhover e-mfocused");
                this._closeAll();
            }
            this._isFocused = true;
        },

        _onDocumentClick: function (event) {
            if (this.model.openOnClick)
                this._hoverOpen = false;
            if (!$(event.target).parents("ul.e-menu").is(this.element)) {
                this._isFocused = false;
                this._OnFocusOutHandler();
            }
        },


        _ContextMenuHandler: function (e) {
            var isRightClick = false;
            if (e.type == "taphold")
                isRightClick = true;
            else if (e.button)
                isRightClick = (e.button == 2);
            else if (e.which)
                isRightClick = (e.which == 3); //for Opera
            var targetElement = e.target;
            if (isRightClick) {
                var evt = e;
                if (e.type == "taphold") {
                    if (e.options.type == "touchstart") evt = e.options.touches[0];
                    else evt = e.options;
                }
                var showSpeed = this._showSpeed;
                this.showContextMenu(null, null, targetElement, evt, true);
            }
            else {
                if (this._isContextMenuOpen) {
                    var hideanim = this._hideAnim;
                    var hideSpeed = this._hideSpeed;
                    this.hideContextMenu(e, hideanim, hideSpeed);
                }
            }
        },

        _calculateContextMenuPosition: function (e) {
            locationX = (e.clientX + this.element.width() < $(window).width()) ? e.pageX : e.pageX - this.element.width();
            locationY = (e.clientY + this.element.height() < $(window).height() || e.clientY < this.element.height()) ? e.pageY : e.pageY - this.element.height();
            var bodyPos = $("body").css("position") != "static" ? $("body").offset() : { left: 0, top: 0 };
            locationX -= bodyPos.left, locationY -= bodyPos.top;
            return {
                X: locationX,
                Y: locationY
            };
        },


        _onDefaultPreventer: function (e) {
            e.preventDefault();
            e.stopPropagation();
            return false;
        },


        _onContextClose: function (e) {
            var proxy = this;
            if (this._isContextMenuOpen) {
                var isRightClick = false;
                if (e.button)
                    isRightClick = (e.button == 2);
                else if (e.which)
                    isRightClick = (e.which == 3); //for Opera
                if (($(e.target).is(this.model.contextMenuTarget) || $(e.target).parents().is(this.model.contextMenuTarget) ||
                    $(e.target).is(this.element) || $(e.target).parents(".e-context").is(this.element)) &&
                    !this._checkForExclusion(e.target))
                    isRightClick = true;
                else
                    isRightClick = false;
                if (!isRightClick) {
                    var hideanim = this._hideAnim;
                    var hideSpeed = this._hideSpeed;
                    this.hideContextMenu(e, hideanim, hideSpeed);
                    var parentElements = $(e.target).parents();
                    $.each(parentElements, function (index, value) {
                        if (value.id == proxy._ContextTargetId) {
                            return;
                        }
                    });

                }
            }
        }

    });


    ej.MenuType = {
        /**  support for list of items appears as normal menu in horizontal or vertical direction. */
        NormalMenu: "normalmenu",
        /**  support for list of items appears as menu when right clicked on target area, thereby preventing browser’s default right click.. */
        ContextMenu: "contextmenu"
    };

    ej.Direction = {
        /**  support for Render sub menu popup in left direction. */
        Left: "left",
        /**  support for Render sub menu popup in Right direction. */
        Right: "right",
        /** Default opening direction of menu sub items */
        None: "none",
    };

    ej.AnimationType = {
        /**  support for disable the AnimationType while hover or click an menu items. */
        None: "none",
        /**  support for enable the AnimationType while hover or click an menu items. */
        Default: "default"
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {

    ej.widget("ejPager", "ej.Pager", {
        _rootCSS: "e-pager",
        validTags: ["div"],
        // default model
        defaults: {
            pageSize: 12,
            pageCount: 10,
            currentPage: 1,
            enableExternalMessage: false,
            externalMessage:"",
            enableQueryString:false,
            locale: "en-US",
            masterObject: null,
            click: null,
            enableRTL: false,
            totalRecordsCount: null,
            totalPages: null,
            customText:""
        },

        // constructor function
        _init: function () {
            this._initPrivateProperties();
            this.model.enableQueryString && this._queryStringValue();
            this.renderPager();
            this._wireEvents();
        },
        _initPrivateProperties: function () {
            this._links = [];
            this._$prev = null;
            this._$first = null;
            this._$PP = null;
            this._$NP = null;
            this._lastNP = false;
            this._lastpageCount = null;
            this._$last = null;
            this._$next = null;
            this._prevPageNo = null;
        },
        _wireEvents: function () {
            if (this.model.click != undefined)
                this._on(this.element, "click", this._pagerClickHandler);

        },
        renderPager: function () {
            var $pagerContainer = ej.buildTag('div.e-pagercontainer', "", {}, { unselectable: "on" });
            var $parentMsgBar = ej.buildTag('div.e-parentmsgbar');
            this._renderPagerContainer($pagerContainer);
            this._renderMsgBar($parentMsgBar);
            this.element.append($pagerContainer).append($parentMsgBar).addClass(this.model.enableRTL ? "e-pager e-rtl" : "e-pager");
            this.model.enableExternalMessage && this._renderPagerMessage();
        },
        _queryStringValue: function () {
            var results = new RegExp('[\\?&]page=([^&#]*)').exec(window.location.href);
            !results ? this.model.currentPage = 1 : this.model.currentPage = parseInt(results[1] || 1);
        },
        _renderPagerMessage: function () {
            var $messageDiv = ej.buildTag('div.e-pagermessage');
            if (this.model.externalMessage.toString().length)
                $messageDiv.html(this.model.externalMessage);
            else
                $messageDiv.hide();
            this.element.append($messageDiv);
        },
        _renderPagerContainer: function ($pagerContainer) {
            //Update pager styles here for next versions
            this._renderBackwardButton($pagerContainer);
            this._renderpreviousPager($pagerContainer);
            this._renderNumericItem($pagerContainer);
            this._renderForwardPager($pagerContainer);
            this._renderForwardButton($pagerContainer);
        },
        _renderMsgBar: function ($ParentMsgBar) {
            var $msgBar = ej.buildTag('span', String.format(this._getLocalizedLabels("pagerInfo"), this.model.currentPage, this.model.totalPages || 0, this.model.totalRecordsCount || 0));
            $ParentMsgBar.append($msgBar);
            $ParentMsgBar.css("text-align", ej.TextAlign.Right);
        },
        _renderpreviousPager: function ($pagerContainer) {
            this._$PP = ej.buildTag('a.e-link e-nextprevitemdisabled e-disable e-spacing e-PP', "...", {}, { title: this._getLocalizedLabels("previousPagerTooltip") , role:"link" });
            $pagerContainer.append(this._$PP);
        },
        _renderForwardPager: function ($pagerContainer) {
            this._$NP = ej.buildTag('a.e-link e-NP e-numericitem e-spacing e-default', "...", {}, { title: this._getLocalizedLabels("nextPagerTooltip"), role: "link" });
            $pagerContainer.append(this._$NP);
        },
        _renderBackwardButton: function ($pagerContainer) {
            this._$first = ej.buildTag('div.e-first e-icon e-mediaback  e-firstpagedisabled e-disable', "", {}, { unselectable: "on", title: this._getLocalizedLabels("firstPageTooltip") });
            this._$prev = ej.buildTag('div.e-prev e-icon e-arrowheadleft-2x  e-prevpagedisabled e-disable', "", {}, { unselectable: "on", title: this._getLocalizedLabels("previousPageTooltip") });
            $pagerContainer.append(this._$first);
            $pagerContainer.append(this._$prev);
        },
        _renderNumericItem: function ($pagerContainer) {
            var $numericContainer = ej.buildTag('div.e-numericcontainer e-default', "", {}, { unselectable: "on", id: "NumericContainer" });
            this._renderNumericLinks($numericContainer,this.model.pageCount);
            $pagerContainer.append($numericContainer);
        },
        _renderNumericLinks: function ($numericContainer) {
            $numericContainer.empty();
            for (var page = 1; page <= this.model.pageCount; page++) {
                var $link = ej.buildTag('a.e-link', this.model.customText + page, {}, { role: "link" }).addClass("e-numericitem e-spacing e-default").data("index",page);
                if (page == this.model.currentPage)
                    $link.removeClass("e-default").addClass("e-currentitem e-active");
                $numericContainer.append($link);
            }
            this._links = $numericContainer.children();
        },
        _renderForwardButton: function ($pagerContainer) {
            this._$next = ej.buildTag('div.e-nextpage e-icon e-arrowheadright-2x  e-default', "", {}, { unselectable: "on", title: this._getLocalizedLabels("nextPageTooltip") });
            this._$last = ej.buildTag('div.e-lastpage e-icon e-mediaforward  e-default', "", {}, { unselectable: "on", title: this._getLocalizedLabels("lastPageTooltip") });
            $pagerContainer.append(this._$next);
            $pagerContainer.append(this._$last);

        },
        _applyCss: function () {
            if (this.model.currentPage > 1) {
                this._$prev.removeClass("e-prevpagedisabled").removeClass("e-disable").addClass("e-prevpage e-default");
                this._$first.removeClass("e-firstpagedisabled").removeClass("e-disable").addClass("e-firstpage e-default");
            } else {
                this._$prev.addClass("e-prevpagedisabled e-disable").removeClass("e-prevpage").removeClass("e-default");
                this._$first.addClass("e-firstpagedisabled e-disable").removeClass("e-firstpage").removeClass("e-default");
            }
            (this.model.currentPage > this.model.pageCount) ?
                 (this._$PP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default")) : (this._$PP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default"));

            if (parseInt(this._links[0].innerHTML.replace(this.model.customText,""), 10) + this.model.pageCount > this.model.totalPages)
                this._lastNP = true;
            else
                this._lastNP = false;

            if (this._lastNP == false)
                this._$NP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default");
            else
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");

            this._lastpageCount = this.model.totalPages % this.model.pageCount;
            if (this._lastpageCount == 0)
                this._lastpageCount = this.model.pageCount;
            if (this.model.currentPage > (this.model.totalPages - this._LastpageCount)) {
                this._$PP.removeClass("e-nextprevitemdisabled").removeClass("e-disable").addClass("e-numericitem e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            }
            if (this.model.currentPage == this.model.totalPages || this.model.totalRecordsCount == 0) {
                this._$last.addClass("e-lastpagedisabled e-disable").removeClass("e-lastpage").removeClass("e-default");
                this._$next.addClass("e-nextpagedisabled e-disable").removeClass("e-nextpage").removeClass("e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            } else {
                this._$last.addClass("e-lastpage e-default").removeClass("e-lastpagedisabled").removeClass("e-disable");
                this._$next.addClass("e-nextpage e-default").removeClass("e-nextpagedisabled").removeClass("e-disable");
            }
            if (this._links.length) {
                this._links.removeClass("e-currentitem").removeClass("e-active").addClass("e-default");
                $(this._links[(this.model.currentPage - 1) % this.model.pageCount]).removeClass("e-default").addClass("e-currentitem e-active");
                $(this._links[(this._prevPageNo - 1) % this.model.pageCount]).removeClass("e-default").addClass("e-numericitem");
            }
            if (this.model.pageSize >= (this.model.totalRecordsCount / this.model.pageCount) && this._$PP != null && this._$PP.length != 0) {
                this._$PP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
                this._$NP.addClass("e-nextprevitemdisabled e-disable").removeClass("e-numericitem").removeClass("e-default");
            }
        },
        _SetTotalPages: function () {
            this.model.totalPages = (this.model.totalRecordsCount % this.model.pageSize == 0) ? (this.model.totalRecordsCount / this.model.pageSize) : (parseInt(this.model.totalRecordsCount / this.model.pageSize, 10) + 1);
        },
        _refreshNumericItem: function () {
            if (this._links.length != 0 && this._links != null) {
                this.model.currentPage = this.model.totalPages == 1 ? 1 : this.model.currentPage;
                if (this.model.currentPage > this.model.totalPages && this.model.totalPages != 0)
                    this.model.currentPage = this.model.totalPages;
                var _pagerTarget = parseInt(this.model.currentPage / this.model.pageCount, 10);
                if (this.model.currentPage % this.model.pageCount == 0)
                    if (_pagerTarget > 0)
                        _pagerTarget = _pagerTarget - 1;
                this._links.css("display", "none");
                for (var i = 0; i < this.model.pageCount; i++) {
                    var start = (_pagerTarget * this.model.pageCount) + 1 + i;
                    if (start <= this.model.totalPages) {
                        this._links[i].style.display = '';
						$(this._links[i]).data('index', start);
                        $(this._links[i]).html(this.model.customText + start);
                    }
                }
            }
        },
        _refreshPagerInfo: function () {
            if (this.model.totalRecordsCount == 0)
                this.model.currentPage = 0;
			if(!this.model.enableTemplates)
            this.element.find("span").text(String.format(this._getLocalizedLabels("pagerInfo"), this.model.currentPage, this.model.totalPages || 0, this.model.totalRecordsCount || 0));
        },
        _refreshExternalMessage: function () {
            if (this.model.externalMessage.toString().length)
                this.element.find(".e-pagermessage").empty().html(this.model.externalMessage).show();
            else
                this.element.find(".e-pagermessage").hide();
        },
        refreshPager: function () {
            this._SetTotalPages();
            this._refreshNumericItem();
            this._refreshPagerInfo();
            this._applyCss();
            this.model.enableExternalMessage && this._refreshExternalMessage();
            if (this.model.enableRTL)
                this.element.addClass("e-rtl");
            else
                this.element.removeClass("e-rtl");
        },

        _kDownHandler: function (e) {
            var code;
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            e.target = null;
            if (this.model.masterObject.checkKey("firstPage", code, e))
                e.target = this._$first;
            else if (this.model.masterObject.checkKey("previousPager", code, e))
                e.target = this._$PP;
            else if (this.model.masterObject.checkKey("previousPage", code, e))
                e.target = this._$prev;
            else if (this.model.masterObject.checkKey("lastPage", code, e))
                e.target = this._$last;
            else if (this.model.masterObject.checkKey("nextPager", code, e))
                e.target = this._$NP;
            else if (this.model.masterObject.checkKey("nextPage", code, e))
                e.target = this._$next;
            else
                return false;
            this._pagerClickHandler(e);
        },

        _pagerClickHandler: function (e) {
            this._prevPageNo = this.model.currentPage;
            var $target = $(e.target);
            if ($.inArray(e.target, this._links) != -1) {
				this.model.currentPage = parseInt($(e.target).data("index"), 10);
            }
            else if ($target.hasClass("e-nextpage") && $target.hasClass("e-nextpagedisabled") != true) {
                if (this.model.currentPage % this.model.pageCount == 0) {
                    this.model.currentPage++;
                    if (this._links != undefined && this._links.length != 0)
                        this.model.currentPage = parseInt($(this._links[0]).data("index"), 10) + this.model.pageCount;
                    if (parseInt(this._links[this.model.pageCount - 1].innerHTML, 10) + this.model.pageCount >= this.model.totalPages)
                        this._lastNP = true;
                }
                else
                    this.model.currentPage++;
            }
            else if ($target.hasClass("e-prevpage") && $target.hasClass("e-prevpagedisabled") != true) {
                if (this.model.currentPage % this.model.pageCount == 1)
                    this._lastNP = false;
                this.model.currentPage--;
                if (this.model.currentPage < 0) { this.model.currentPage = 0; }
            }
            else if ($target.hasClass("e-lastpage") && $target.hasClass("e-lastpagedisabled") != true) {
                this._LastpageCount = this.model.totalPages % this.model.pageCount;
                (this._LastpageCount == 0) ? (this._LastpageCount = this.model.pageCount) : null;
                this.model.currentPage = this.model.totalPages;
                this._lastNP = true;
            }
            else if ($target.hasClass("e-firstpage") && $target.hasClass("e-firstpagedisabled") != true) {
                this.model.currentPage = 1;
                this._lastNP = false;
            }
            else if ($target.hasClass("e-NP") && $target.hasClass("e-nextprevitemdisabled") != true) {
                if (this._links != undefined)
                    this.model.currentPage = parseInt($(this._links[0]).data("index"), 10) + this.model.pageCount;
                if (parseInt(this._links[this.model.pageCount - 1].innerHTML.replace(this.model.customText,""), 10) + this.model.pageCount >= this.model.totalPages) {
                    this._lastNP = true;
                    if ((this.model.totalRecordsCount - this.model.pageSize) < this.model.pageSize)
                        this._LastpageCount = this.model.totalRecordsCount - this.model.pageSize;
                    else
                        this._LastpageCount = ((this.model.totalRecordsCount / this.model.pageSize) % this.model.pageCount);
                    (this._LastpageCount == 0) ? (this._LastpageCount = this.model.pageCount) : null;
                    if (this._links != undefined)
                        this.model.currentPage = parseInt($(this._links[this.model.pageCount - 1]).data("index"), 10) + 1;
                }
            }
            else if ($target.hasClass("e-PP") && $target.hasClass("e-nextprevitemdisabled") != true) {
                if (this._links != undefined)
                    this.model.currentPage = parseInt($(this._links[0]).data("index"), 10) - this.model.pageCount;
                this._lastNP = false;
            }
            this.goToPage(this.model.currentPage,e);
            return false;
        },
        goToPage: function (pageIndex,event) {
            if (pageIndex != this.model.currentPage)
                this._prevPageNo = this.model.currentPage;
            if (this._prevPageNo !== pageIndex && (pageIndex >= 1 && pageIndex <= this.model.totalPages)) {
                this.model.currentPage = pageIndex;
                this.model.enableQueryString && this._updateQueryString(this.model.currentPage);
                this._trigger("click", { "currentPage": pageIndex, "event": event });
            }
            else
                this._trigger("click", { "currentPage": pageIndex, "event": event });

        },
        _updateQueryString: function (value) {
            var _newUrl = this._getUpdatedURL(window.location.href, "page", value);
            if (history.pushState) {
                window.history.pushState({ path: _newUrl }, '', _newUrl);
            }
            else
                window.location.href = _newUrl;
        },

        _getUpdatedURL: function (uri, key, value) {
            var re = new RegExp("([?|&])" + key + "=.*?(&|#|$)", "i");
            if (uri.match(re)) {
                return uri.replace(re, '$1' + key + "=" + value + '$2');
            } else {
                var hash = '';
                var separator = uri.indexOf('?') !== -1 ? "&" : "?";
                if (uri.indexOf('#') !== -1) {
                    hash = uri.replace(/.*#/, '#');
                    uri = uri.replace(/#.*/, '');
                }
                return uri + separator + key + "=" + value + hash;
            }
        },
        _getLocalizedLabels: function (property) {
            return (ej.Pager.locale[this.model.locale] === undefined || ej.Pager.locale[this.model.locale][property] === undefined) ? ej.Pager.locale["en-US"][property] : ej.Pager.locale[this.model.locale][property];
        },
        _setFirst: true,
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "pageCount":
                        this._renderNumericLinks(this.element.find("#NumericContainer"));
                        break;
                }
            }
            this.refreshPager();
        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {
            this.element.empty().removeClass("e-pager");
        }
    });
    ej.Pager.locale = {};
    ej.Pager.locale["en-US"] = {
        pagerInfo: "{0} of {1} pages ({2} items)",
        firstPageTooltip: "Go to first page",
        lastPageTooltip: "Go to last page",
        nextPageTooltip: "Go to next page",
        previousPageTooltip: "Go to previous page",
        nextPagerTooltip: "Go to next Pager",
        previousPagerTooltip: "Go to previous Pager"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html div elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejSlider", "ej.Slider", {

        element: null,

        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-slider",
        _setFirst: false,

        defaults: {

            orientation: "horizontal",

            enableAnimation: true,

            animationSpeed: 500,

            showTooltip: true,

            cssClass: "",

            showRoundedCorner: false,

            readOnly: false,

            enableRTL: false,

            htmlAttributes: {},

            minValue: 0,

            maxValue: 100,

            sliderType: "default",

            value: null,

            values: null,

            incrementStep: 1,

            height: null,

            width: null,

            enabled: true,

            showScale: false,

            largeStep: 10,

            smallStep: 1,

            showSmallTicks: true,

            enablePersistence: false,

            start: null,

            stop: null,

            slide: null,

            change: null,

            create: null,

            destroy: null
        },

        dataTypes: {
            orientation: "enum",
            enableAnimation: "boolean",
            animationSpeed: "number",
            cssClass: "string",
            showRoundedCorner: "boolean",
            readOnly: "boolean",
            enableRTL: "boolean",
            minValue: "number",
            maxValue: "number",
            sliderType: "enum",
            incrementStep: "number",
            enabled: "boolean",
            showScale: "boolean",
            largeStep: "number",
            smallStep: "number",
            showSmallTicks: "boolean",
            enablePersistence: "boolean",
            htmlAttributes: "data"
        },

        observables: ["value", "values"],
        value: ej.util.valueFunction("value"),
        values: ej.util.valueFunction("values"),

        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                if (this.wrapper) this.wrapper.removeClass("e-disable");
                this.element.removeClass("e-disable");
                this._wireEvents();
            }
        },

        disable: function () {
            if (this.model.enabled) {
                this.model.enabled = false;
                if (this.wrapper) this.wrapper.addClass("e-disable");
                this.element.addClass("e-disable");
                this._unWireEvents();
            }
        },

        _validateValue: function (value) {
            if (value == null || value === "") value = this.model.minValue;
            else if (typeof value === "string") value = parseFloat(value);

            if (this._isNumber(value))
                this.value(value);
            else if (!this._isNumber(this.value()))
                this.value(this.model.minValue);

            if (this.model.sliderType != "range") this._setValue();
        },

        _validateRangeValue: function (value) {
            if (value == null) value = new Array(this.model.minValue, this.model.maxValue);
            else if (typeof value === "string") {
                var vals = value.split(",");
                if (vals.length > 1) value = new Array(parseFloat(vals[0]), parseFloat(vals[1]));
            }

            if (typeof value === "object" && this._isNumber(value[0]) && this._isNumber(value[1]))
                this.values(new Array(value[0], value[1]));
            else if (!(typeof this.values() === "object" && this._isNumber(this.values()[0]) && this._isNumber(this.values()[1])))
                this.values(new Array(this.model.minValue, this.model.maxValue));

            if (this.model.sliderType == "range") this._setRangeValue();
        },

        _validateStartEnd: function () {
            if (isNaN(this.model.minValue)) this.model.minValue = 0;
            if (isNaN(this.model.maxValue)) this.model.maxValue = 100;
        },

        _isNumber: function (number) {
            return typeof number === "number" && !isNaN(number);
        },

        _outerCorner: function (boolean) {
            if (boolean) this._roundedCorner();
            else this._sharpedCorner();
        },

        _changeSkin: function (skin) {
            this.element.removeClass(this.model.cssClass).addClass(skin);
            if (this.model.showScale)
                this.ul.removeClass(this.model.cssClass).addClass(skin);
        },

        getValue: function () {

            return this._getHandleValue();
        },


        _init: function () {
            this._initialize();
            this._render();
        },

        _setModel: function (options) {

            if (!ej.isNullOrUndefined(options["minValue"]) || !ej.isNullOrUndefined(options["maxValue"])) {
                if (this._isNumber(options["minValue"])) this.model.minValue = options["minValue"];
                else options["minValue"] = this.model.minValue;

                if (this._isNumber(options["maxValue"])) this.model.maxValue = options["maxValue"];
                else options["maxValue"] = this.model.maxValue;

                if (this.model.sliderType == "range" && options["values"] == undefined) this._setRangeValue();
                else if (this.model.sliderType != "range" && options["value"] == undefined) this._setValue();
            }

            var option;
            for (option in options) {
                switch (option) {
                    case "value":
                        this._validateValue(ej.util.getVal(options[option]));
                        break;
                    case "values":
                        this._validateRangeValue(ej.util.getVal(options[option]));
                        break;
                    case "height": this.model.height = options[option]; this._setDimension();
                        if (this.model.showScale) this._scaleAlignment();
                        break;
                    case "width": this.model.width = options[option]; this._setDimension();
                        if (this.model.showScale) this._scaleAlignment();
                        break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "showRoundedCorner": this._outerCorner(options[option]); break;
                    case "enableRTL": this.model.enableRTL = options[option]; this._checkRTL();
                        options[option] = this.model.enableRTL;
                        break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showScale": this._renderScale(options[option]); break;
                    case "orientation":
                        var t = this.model.height;
                        this.model.height = this.model.width;
                        this.model.width = t;
                    case "sliderType":
                        this._sliderOptions(option, options[option]); break;
                    case "smallStep":
                    case "largeStep":
                    case "showSmallTicks":
                    case "minValue":
                    case "maxValue":
                        this._scaleOptions(option, options[option]); break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }
        },

        _destroy: function () {
            if (this.model.showScale) this._destroyScale();
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            this.element.removeClass("e-widget e-box " + this.model.cssClass).empty();
        },

        _initialize: function () {
            this.target = this.element[0];
            this.horDir = "left";
            this.verDir = "bottom";
            this._isFocused = false;
        },

        _render: function () {
            this.initialRender = true;
            this.wrapper = ej.buildTag("div.e-slider-wrap " + this.model.cssClass + "#" + this.target.id + "_wrapper", { tabindex: "0", role: "slider" })
                .insertAfter(this.element);
            this.wrapper.append(this.element);

            this.element.addClass("e-widget e-box " + this.model.cssClass);
            if (this.model.sliderType != "default") {
                this.header = ej.buildTag("div.e-range");
                this.element.append(this.header);
                if (this.model.sliderType == "range") {
                    this.secondHandle = this._createHandle();
                }
            }
            this.firstHandle = this._createHandle();
            this._setOrientation();
            this._setDimension();
            this._insertHiddenField();
            this._checkProperties();
            this._addAttr(this.model.htmlAttributes);
            this._setSliderValue();
        },

        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy._disabled(false);
                else proxy.element.attr(key, value)
            });
        },

        _renderScale: function (showScale) {
            if (showScale) {
                var width = "width", orien = "h";
                if (this.model.orientation == "vertical") {
                    width = "height";
                    orien = "v";
                }

                var _smallStep = this.model.smallStep;
                if (!this.model.showSmallTicks) {
                    if (this.model.largeStep > 0)
                        _smallStep = this.model.largeStep;
                    else
                        _smallStep = this.model.maxValue - this.model.minValue;
                }
                else if (_smallStep <= 0)
                    _smallStep = this.model.incrementStep;
                var count = Math.abs(this.model.maxValue - this.model.minValue) / _smallStep;

                this.ul = ej.buildTag("ul.e-scale e-" + orien + "-scale " + this.model.cssClass);
                this.wrapper.append(this.ul);

                var li, start = this.model.minValue, left = 0, tickWidth = 100 / count;
                if (orien == "v") start = this.model.maxValue;
                for (var i = 0; i <= count; i++) {
                    li = ej.buildTag("li.e-tick", "", {}, { "title": start });
                    li.css(width, tickWidth + "%");

                    if (start % this.model.largeStep == 0) {
                        var span = ej.buildTag("span.e-tick-value", "" + start);
                        li.addClass("e-large").append(span);
                    }
                    this.ul.append(li);

                    if (orien == "h") start += _smallStep;
                    else start -= _smallStep;
                    left += _smallStep;
                }

                this.ul.children().first().addClass("e-first-tick").css(width, (tickWidth / 2) + "%");
                this.ul.children().last().addClass("e-last-tick").css(width, (tickWidth / 2) + "%");

                this._scaleAlignment();
            }
            else this._destroyScale();
        },
        _destroyScale: function () {
            this.ul.remove();
            this.ul = null;
        },

        _tickValuePosition: function () {
            var width = (this.model.orientation == "vertical") ? "height" : "width";
            var left = (this.model.orientation == "vertical") ? "top" : "left";
            var firstTick = this.ul.find(".e-tick.e-first-tick");
            var first = firstTick.find(".e-tick-value");
            var other = this.ul.find(".e-tick.e-large:not(.e-first-tick)").find(".e-tick-value");
            tickWidth = firstTick[width]() * 2;
            first.css(left, -first[width]() / 2);
            other.css(left, (tickWidth - other[width]()) / 2);
        },

        _scaleAlignment: function () {
            this._tickValuePosition();
            var smallTick = 12, largeTick = 20, half = largeTick / 2;
            var height = "height", top = "top", orien = "h";
            if (this.model.orientation == "vertical") {
                height = "width";
                top = "right";
                orien = "v";
            }
            // scale
            this.ul.css(top, -(this.wrapper[height]() + half));
            if (orien == "v") this.ul.css("top", -this.wrapper.height()).css(top, half);
            this.ul[height](this.wrapper[height]() + largeTick);
            // small-ticks
            var topSize = -(largeTick - smallTick) / 2;
            if (this.model.largeStep == null && orien != "v") topSize = -topSize;
            this.ul.find(".e-tick:not(.e-large)").css(height, this.wrapper[height]() + smallTick).css(top, topSize);
            // tick-values   // 4 - distance between tick value and tick
            if (orien == "v") this.ul.children(".e-large").find(".e-tick-value").css("right", this.wrapper.width() + largeTick + 4);
        },

        _createHandle: function () {
            handle = ej.buildTag("a.e-handle e-select", "", {}, { "aria-label": "drag", "tabindex": 0, role: "slider", "aria-valuemin": this.model.minValue, "aria-valuemax": this.model.maxValue });
            this.element.append(handle);
            return handle;
        },

        _setDimension: function () {
            if (this.model.height) this.wrapper.height(this.model.height);
            if (this.model.width) this.wrapper.width(this.model.width);
            this._setHandleSize();
            this._handleAlignment(this.model.enableRTL);
        },

        _insertHiddenField: function () {
            this._hidden = ej.buildTag("input", "", {},
                { "type": "hidden", "name": this.element[0].id }).val(this._getHandleValue());
            this.element.append(this._hidden);
        },

        _checkProperties: function () {
            if (!this.model.enabled) {
                if (this.wrapper) this.wrapper.addClass("e-disable");
                else this.element.addClass("e-disable");
            }
            else this._wireEvents();
            if (this.model.showScale) this._renderScale(true);
            if (this.model.enableRTL) this._checkRTL();
            if (this.model.showRoundedCorner) this._roundedCorner();
        },

        _roundedCorner: function () {
            this.element.addClass("e-corner");
        },

        _sharpedCorner: function () {
            this.element.removeClass("e-corner");
        },

        _handleAlignment: function (rtl) {
            var mar = -(this.firstHandle.outerWidth() / 2) + "px", margin;
            if (this.model.orientation != "vertical") {
                if (!rtl) margin = "0 0 0 " + mar;
                else margin = "0 " + mar + " 0 0";
            }
            else {
                if (!rtl) margin = "0 0 " + mar + " 0";
                else margin = mar + " 0 0 0";
            }
            this.element.children('.e-handle').css("margin", margin);
        },

        _checkRTL: function () {
            if (this.model.showScale && this.model.orientation == "vertical" && this.model.enableRTL) this.model.enableRTL = false;
            var rtl = this.model.enableRTL, preDir = (this.model.orientation != "vertical") ? this.horDir : this.verDir;
            if (rtl) {
                this.wrapper.addClass("e-rtl");
                this.horDir = "right";
                this.verDir = "top";
            }
            else {
                this.wrapper.removeClass("e-rtl");
                this.horDir = "left";
                this.verDir = "bottom";
            }
            var currDir = (this.model.orientation != "vertical") ? this.horDir : this.verDir;

            if (preDir != currDir) {
                this.firstHandle.css(currDir, this.firstHandle.css(preDir)).css(preDir, "auto");;
                if (this.model.sliderType != "default") {
                    this.header.css(currDir, this.header.css(preDir)).css(preDir, "auto");
                    if (this.model.sliderType == "range")
                        this.secondHandle.css(currDir, this.secondHandle.css(preDir)).css(preDir, "auto");
                }
            }
            this._handleAlignment(rtl);
        },

        _setOrientation: function () {
            if (this.model.orientation != "vertical") {
                this.wrapper.addClass("e-horizontal");
            }
            else {
                this.wrapper.addClass("e-vertical");
                this.firstHandle.css(this.verDir, "0");
            }
        },

        _setHandleSize: function () {
            var size;
            if (this.model.orientation != "vertical")
                size = this.wrapper.height() + 2;
            else
                size = this.wrapper.width() + 2;
            this.element.find(".e-handle").height(size).width(size);
        },

        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        _sliderOptions: function (prop, value) {
            this._unWireEvents();
            this._destroy();
            this.model[prop] = value;
            this._init();
        },

        _scaleOptions: function (prop, value) {
            if (this.model.showScale) {
                this._destroyScale();
                this.model[prop] = value;
                this._renderScale(true);
            }
        },

        _showTooltip: function () {
            if (this.model.showTooltip) {
                $('body .e-tooltip').remove();
                this.tooltip = ej.buildTag("div.e-tooltip " + this.model.cssClass + " e-corner", { role: "tooltip" });
                $("body").append(this.tooltip);
                this._setTooltipPosition();
            }
        },

        _hideTooltip: function () {
            if (this.model.showTooltip)
                this.tooltip.fadeOut(800);
        },

        _showhideTooltip: function (showTooltip) {
            if (this.model.showTooltip && showTooltip) {
                this._showTooltip();
                this._hideTooltip();
            }
        },

        _setTooltipPosition: function () {
            if (this.model.showTooltip) {
                this._updateTooltipValue();
                var top, left, remainLeft, remainTop, handle, pos, gap = 5, broder; // gap -> distance between tooltip and slider
                handle = this._getHandle(), pos = this._getOffset(handle), tooltipPos = this._getOffset(this.tooltip);
                border = $(handle).outerHeight() - $(handle).height();
                if (this.model.orientation == "vertical") {
                    remainTop = (this.tooltip.outerHeight() - handle.outerHeight()) / 2;
                    remainLeft = handle.outerWidth() + gap;
                    top = pos.top - remainTop;
                    left = pos.left + remainLeft;
                    if (top < 0) top = 0;
                    else if ($(window).height() < top + this.tooltip.outerHeight()) top = $(window).height() - this.tooltip.outerHeight() - border;
                    if ($(window).width() < left + this.tooltip.outerWidth()) left = pos.left - this.tooltip.outerWidth() - border;
                }
                else {
                    if (tooltipPos.left + this.tooltip.outerWidth() > $(window).width()) this.tooltip.css({ "left": '0px' });
                    remainLeft = (this.tooltip.outerWidth() - handle.outerWidth()) / 2;
                    remainTop = this.tooltip.outerHeight() + gap;
                    top = pos.top - remainTop;
                    left = pos.left - remainLeft;
                    if (left < 0) left = 0;
                    else if ($(window).width() < left + this.tooltip.outerWidth()) left = $(window).width() - this.tooltip.outerWidth() - border;
                    if (top < 0 || pos.top < remainTop) {
                        if (pos.top + handle.outerHeight() + border + this.tooltip.outerHeight() > $(window).height()) {
                            top = pos.top;
                            if (pos.left > this.tooltip.outerWidth() + gap + border) left = pos.left - (this.tooltip.outerWidth() + border);
                            else left = pos.left + (handle.outerWidth() + gap + border);
                        } else top = pos.top + handle.outerHeight() + border;
                    }
                }
                var zindex = this._maxZindex();
                this.tooltip.css({ "top": top, "left": left, "zIndex": zindex + 1 });
            }
        },
        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },


        _maxZindex: function () {
            return ej.util.getZindexPartial(this.element, this.popup);
        },

        _updateTooltipValue: function () {
            var one = 0, two = 1;
            if (this.model.enableRTL) { one = 1, two = 0; }
            var val = this._getHandleValue();
            if (this.model.sliderType != "range")
                this.tooltip[0].innerHTML = val;
            else
                this.tooltip[0].innerHTML = val[one] + " - " + val[two];
        },

        _increaseHeaderWidth: function (animation) {
            if (this.model.sliderType != "default") {
                var size = "width", direction = this.horDir, properties = {};
                if (this.model.orientation == "vertical") { size = "height", direction = this.verDir; }

                if (this.model.sliderType == "range") {
                    properties[size] = this.handlePos - this.handlePos2 + "%";
                    properties[direction] = this.handlePos2 + "%";
                }
                else {
                    properties[size] = this.handlePos + "%";
                    properties[direction] = 0;
                }
                if (!animation) this.header.css(properties);
                else this.header.animate(properties, this.model.animationSpeed);
            }
        },

        _setSliderValue: function () {
            this._validateStartEnd();

            if (this.model.sliderType == "range")
                this._validateRangeValue(this.values());
            else
                this._validateValue(this.value());
        },


        _hoverOnHandle: function (evt) {
            $(evt.target).addClass("e-hover");
        },

        _leaveFromHandle: function (evt) {
            $(evt.target).removeClass("e-hover");
        },

        _firstHandleClick: function (evt) {
            evt.preventDefault();
            this.firstHandle.focus();
            if (this._raiseEvent("start")) return false;

            this.mouseDownPos = this.handlePos;
            if (!this.model.readOnly)
                $(document).bind(ej.eventType.mouseMove, $.proxy(this._firstHandleMove, this));
            $(document).bind(ej.eventType.mouseUp, $.proxy(this._firstHandleUp, this));
            $(document).bind("mouseleave", $.proxy(this._firstHandleUp, this));
            this._showTooltip();
        },

        _firstHandleMove: function (evt) {
            evt.preventDefault();
            evt = evt.type == "touchmove" ? evt.originalEvent.changedTouches[0] : evt;
            var position = { x: evt.pageX, y: evt.pageY };
            this.handlePos = this._xyToPosition(position);

            if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                this.handlePos = this.handlePos2;
            }
            if (this.handlePos != this.preHandlePos) {
                this.preHandlePos = this.handlePos;
                this.handleVal = this._positionToValue(this.handlePos);
                this._increaseHeaderWidth(false);
                this._setHandlePosition(false, false, false);
                this._setTooltipPosition();

                this._updateModelValue();
                this._raiseEvent("slide");
            }
        },

        _firstHandleUp: function (evt) {
            evt.preventDefault();
            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._firstHandleMove, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._firstHandleUp, this));
            $(document).unbind("mouseleave", $.proxy(this._firstHandleUp, this));
            this._hideTooltip();

            if (this.mouseDownPos != this.handlePos) this._raiseChangeEvent();
        },

        _secondHandleClick: function (evt) {
            evt.preventDefault();
            this.secondHandle.focus();
            if (this._raiseEvent("start")) return false;

            this.mouseDownPos2 = this.handlePos2;
            if (!this.model.readOnly)
                $(document).bind(ej.eventType.mouseMove, $.proxy(this._secondHandleMove, this));
            $(document).bind(ej.eventType.mouseUp, $.proxy(this._secondHandleUp, this));
            $(document).bind("mouseleave", $.proxy(this._secondHandleUp, this));
            this._showTooltip();
        },

        _secondHandleMove: function (evt) {
            evt.preventDefault();
            evt = evt.type == "touchmove" ? evt.originalEvent.changedTouches[0] : evt;
            var position2 = { x: evt.pageX, y: evt.pageY };
            this.handlePos2 = this._xyToPosition(position2);

            if (this.handlePos2 > this.handlePos) {
                this.handlePos2 = this.handlePos;
            }
            if (this.handlePos2 != this.preHandlePos2) {
                this.preHandlePos2 = this.handlePos2;
                this.handleVal2 = this._positionToValue(this.handlePos2);
                this._increaseHeaderWidth(false);
                this._setHandlePosition(false, false, false);
                this._setTooltipPosition();

                this._updateModelValue();
                this._raiseEvent("slide");
            }
        },

        _secondHandleUp: function (evt) {
            evt.preventDefault();
            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._secondHandleMove, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._secondHandleUp, this));
            $(document).unbind("mouseleave", $.proxy(this._secondHandleUp, this));
            this._hideTooltip();

            if (this.mouseDownPos2 != this.handlePos2) this._raiseChangeEvent();
        },

        _focusInHandle: function (evt) {
            if (!this._isFocused) {
                this._isFocused = true;
                $(evt.target).addClass("e-focus");
                if (!this.model.readOnly)
                    $(document).bind("keydown", $.proxy(this._moveHandle, this));
                this.activeHandle = $(evt.target).is(this.firstHandle) ? 1 : 2;
                this._setZindex();
            }
        },

        _focusOutHandle: function (evt) {
            this._isFocused = false;
            $(evt.target).removeClass("e-focus");
            $(document).unbind("keydown", $.proxy(this._moveHandle, this));
        },

        _moveHandle: function (e) {
            var oper, val, handleNo;
            handleNo = this._getHandleIndex(this.activeHandle) - 1;

            switch (e.keyCode) {
                case 37:
                case 40:
                    e.preventDefault();
                    oper = "sub";
                    break;
                case 38:
                case 39:
                    e.preventDefault();
                    oper = "add";
                    break;
                case 36:
                    e.preventDefault();
                    if (this._raiseEvent("start")) return false;
                    if (this.model.sliderType != "range" && this.value() != this.model.minValue) {
                        this._changeHandleValue(this.model.minValue, this.model.enableAnimation);
                    }
                    else if (this.model.sliderType == "range") {
                        val = (this.activeHandle == 2) ? this.model.minValue : this.handleVal2;
                        if (this.values()[handleNo] != val)
                            this._changeHandleValue(val, this.model.enableAnimation);
                    }
                    break;
                case 35:
                    e.preventDefault();
                    if (this._raiseEvent("start")) return false;
                    if (this.model.sliderType != "range" && this.value() != this.model.maxValue) {
                        this._changeHandleValue(this.model.maxValue, this.model.enableAnimation);
                    }
                    else if (this.model.sliderType == "range") {
                        val = (this.activeHandle == 1) ? this.model.maxValue : this.handleVal;
                        if (this.values()[handleNo] != val)
                            this._changeHandleValue(val, this.model.enableAnimation);
                    }
                    break;
                case 27:
                    e.preventDefault();
                    this._getHandle().focusout();
                    break;
            }

            if (oper == "add" || oper == "sub") {
                if (this._raiseEvent("start")) return false;
                var hVal = (this.activeHandle == 1) ? this.handleVal : this.handleVal2;
                var value = (oper == "add") ? this._add(hVal, this.model.incrementStep, true) : this._add(hVal, this.model.incrementStep, false);
                this._changeHandleValue(value, false);
            }
        },

        _changeHandleValue: function (value, animate) {
            var position = null;
            if (this.activeHandle == 1) {
                this.handleVal = this._checkHandleValue(value);
                this.handlePos = this._checkHandlePosition(this.handleVal);

                if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                    this.handlePos = this.handlePos2;
                    this.handleVal = this.handleVal2;
                }
                if (this.handlePos != this.preHandlePos)
                    position = this.preHandlePos = this.handlePos;
            }
            else {
                this.handleVal2 = this._checkHandleValue(value);
                this.handlePos2 = this._checkHandlePosition(this.handleVal2);

                if (this.model.sliderType == "range" && this.handlePos < this.handlePos2) {
                    this.handlePos2 = this.handlePos;
                    this.handleVal2 = this.handleVal;
                }
                if (this.handlePos2 != this.preHandlePos2)
                    position = this.preHandlePos2 = this.handlePos2;
            }

            if (position != null) {
                this._increaseHeaderWidth(animate);
                this._setHandlePosition(animate, true, true);
            }
        },

        _sliderBarClick: function (evt) {
            if (this.model.readOnly) return false;
            if (evt.target == this.target || (this.model.sliderType != "default" && evt.target == this.header[0])) {
                evt.preventDefault();
                if (this._raiseEvent("start")) return false;
                var pos = { x: evt.pageX, y: evt.pageY },
                handlepos = this._xyToPosition(pos),
                handleVal = this._positionToValue(handlepos);

                if (this.model.sliderType == "range" && (this.handlePos - handlepos) > (handlepos - this.handlePos2)) {
                    this.handlePos2 = this.preHandlePos2 = handlepos;
                    this.handleVal2 = handleVal;
                    this.activeHandle = 2;
                }
                else {
                    this.handlePos = this.preHandlePos = handlepos;
                    this.handleVal = handleVal;
                    this.activeHandle = 1;
                }

                this._getHandle().focus();
                if (this.model.sliderType != "default") this._increaseHeaderWidth(this.model.enableAnimation);
                this._setHandlePosition(this.model.enableAnimation, true, true);
            }
        },

        _setHandlePosition: function (animation, showTooltip, changeEvt) {
            var Handle = this._getHandle(), proxy = this, properties = {}, pos, val, direction;
            pos = (this.activeHandle == 1) ? this.handlePos : this.handlePos2;
            val = (this.activeHandle == 1) ? this.handleVal : this.handleVal2;
            Handle.attr("aria-label", val);
            direction = (this.model.orientation == "vertical") ? this.verDir : this.horDir;
            properties[direction] = pos + "%";

            if (!animation) {
                Handle.css(properties);
                this._showhideTooltip(showTooltip);
                if (changeEvt) this._raiseChangeEvent();
            }
            else Handle.animate(properties, this.model.animationSpeed, function () {
                proxy._showhideTooltip(showTooltip);
                if (changeEvt) proxy._raiseChangeEvent();
            });
        },

        _xyToPosition: function (position) {
            if (this.model.minValue == this.model.maxValue)
                return 100;
            if (this.model.orientation != "vertical") {
                var left = position.x - this.element.offset().left,
                num = this.element.width() / 100,
                val = (left / num);
            }
            else {
                var top = position.y - this.element.offset().top,
                num = this.element.height() / 100,
                val = 100 - (top / num);
            }
            val = this._stepValueCalculation(val);
            if (val < 0) val = 0;
            else if (val > 100) val = 100;
            if (this.model.enableRTL) return 100 - val;
            return val;
        },

        _updateValue: function () {
            this.handleVal = this._checkHandleValue(this.value());
            this.handlePos = this._checkHandlePosition(this.handleVal);
            this.preHandlePos = this.handlePos;
            this.activeHandle = 1;
        },

        _setValue: function () {
            this._updateValue();
            this._increaseHeaderWidth(this.model.enableAnimation);
            this._setHandlePosition(this.model.enableAnimation, false, true);
        },

        _updateRangeValue: function () {
            var values = this.values();
            this.handleVal = this._checkHandleValue(values[1]);
            this.handleVal2 = this._checkHandleValue(values[0]);
            this.handlePos = this._checkHandlePosition(this.handleVal);
            this.handlePos2 = this._checkHandlePosition(this.handleVal2);

            if (this.handlePos < this.handlePos2) {
                this.handlePos = this.handlePos2;
                this.handleVal = this.handleVal2;
            }
            this.preHandlePos = this.handlePos;
            this.preHandlePos2 = this.handlePos2;
        },

        _setRangeValue: function () {
            this._updateRangeValue();
            this._increaseHeaderWidth(this.model.enableAnimation);
            this.activeHandle = 1;
            this._setHandlePosition(this.model.enableAnimation, false, true);
            this.activeHandle = 2;
            this._setHandlePosition(this.model.enableAnimation, false, true);
        },

        _checkHandlePosition: function (value) {
            if (this.model.minValue == this.model.maxValue)
                return 100;
            var handle = this._tempStartEnd();
            if (value >= handle.start && value <= handle.end)
                value = (100 * (value - this.model.minValue)) / (this.model.maxValue - this.model.minValue);
            else if (value < handle.start) value = 0;
            else value = 100;
            return value;
        },

        _checkHandleValue: function (value) {
            if (this.model.minValue == this.model.maxValue)
                return this.model.minValue;
            var handle = this._tempStartEnd();
            if (value < handle.start) value = handle.start;
            else if (value > handle.end) value = handle.end;
            return value;
        },

        _tempStartEnd: function () {
            if (this.model.minValue > this.model.maxValue)
                return {
                    start: this.model.maxValue,
                    end: this.model.minValue
                };
            else
                return {
                    start: this.model.minValue,
                    end: this.model.maxValue
                };
        },

        _positionToValue: function (pos) {
            var diff = this.model.maxValue - this.model.minValue,
            val = this._round(diff * pos / 100),
            total = this._add(val, this.model.minValue, true);
            return total;
        },

        _getHandle: function () {
            return (this.activeHandle == 1) ? this.firstHandle : this.secondHandle;
        },

        _getHandleIndex: function (no) {
            if (this.model.sliderType == "range" && no == 1)
                return 2;
            return 1;
        },

        _getHandleValue: function () {
            if (this.model.sliderType == "range") return [this.handleVal2, this.handleVal];
            else return this.handleVal;
        },

        _updateModelValue: function () {
            var value = this._getHandleValue();
            if (this.model.sliderType == "range") this.values(value);
            else this.value(value);
            this._hidden.val(value);
        },

        _add: function (a, b, addition, precision) {
            var x = Math.pow(10, precision || 3), val;
            if (addition) val = (Math.round(a * x) + Math.round(b * x)) / x;
            else val = (Math.round(a * x) - Math.round(b * x)) / x;
            return val;
        },

        _round: function (a) {
            var _f = this.model.incrementStep.toString().split(".");
            return _f[1] ? parseFloat(a.toFixed(_f[1].length)) : Math.round(a);
        },

        _raiseChangeEvent: function () {
            this._updateModelValue();
            if (this.initialRender)
                this.initialRender = false;
            else {
                this._raiseEvent("change");
                this._raiseEvent("stop");
            }            
        },

        _raiseEvent: function (name) {
            return this._trigger(name, {
                id: this.target.id,
                value: this._getHandleValue(),
                sliderIndex: this._getHandleIndex(this.activeHandle)
            });
        },

        _setZindex: function () {
            if (this.model.sliderType == "range") {
                if (this.activeHandle == 1) {
                    this.firstHandle.css("z-index", 2);
                    this.secondHandle.css("z-index", 1);
                }
                else {
                    this.firstHandle.css("z-index", 1);
                    this.secondHandle.css("z-index", 2);
                }
            }
        },

        _stepValueCalculation: function (value) {
            if (this.model.incrementStep == 0) this.model.incrementStep = 1;
            var percentStep = this.model.incrementStep / ((this.model.maxValue - this.model.minValue) / 100);
            var remain = value % Math.abs(percentStep);
            if (remain != 0) {
                if ((percentStep / 2) > remain) value -= remain;
                else value += Math.abs(percentStep) - remain;
            }
            return value;
        },

        _wireEvents: function () {
            this._on(this.element, "mousedown", this._sliderBarClick);
            this._on(this.firstHandle, ej.eventType.mouseDown, this._firstHandleClick);
            this._on(this.firstHandle, "mouseenter", this._hoverOnHandle);
            this._on(this.firstHandle, "mouseleave", this._leaveFromHandle);
            this._on(this.firstHandle, "focus", this._focusInHandle);
            this._on(this.firstHandle, "focusout", this._focusOutHandle);

            if (this.model.sliderType == "range") {
                this._on(this.secondHandle, ej.eventType.mouseDown, this._secondHandleClick);
                this._on(this.secondHandle, "mouseenter", this._hoverOnHandle);
                this._on(this.secondHandle, "mouseleave", this._leaveFromHandle);
                this._on(this.secondHandle, "focus", this._focusInHandle);
                this._on(this.secondHandle, "focusout", this._focusOutHandle);
            }
        },


        _unWireEvents: function () {
            this._off(this.element, "mousedown");
            this._off(this.firstHandle, ej.eventType.mouseDown);
            this._off(this.firstHandle, "mouseenter");
            this._off(this.firstHandle, "mouseleave");
            this._off(this.firstHandle, "focus");
            this._off(this.firstHandle, "focusout");

            if (this.model.sliderType == "range") {
                this._off(this.secondHandle, ej.eventType.mouseDown);
                this._off(this.secondHandle, "mouseenter");
                this._off(this.secondHandle, "mouseleave");
                this._off(this.secondHandle, "focus");
                this._off(this.secondHandle, "focusout");
            }
        }
    });

    ej.SliderType = {
        /**  support for slider control to select a single value. */
        Default: "default",
        /**  support for slider control to select a single value considered from start value to current handle. */
        MinRange: "minrange",
        /**  support for slider control to select a range of value between the two handles. */
        Range: "range"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejSplitButton", "ej.SplitButton", {

        element: null,

        model: null,
        validTags: ["button"],
        _setFirst: false,

        _rootCSS: "e-splitbutton",

        defaults: {

            size: "normal",

            width: "",

            height: "",

            enabled: true,

            htmlAttributes: {},

            text: null,

            contentType: "textonly",

            imagePosition: "imageleft",

            buttonMode: "split",

            arrowPosition: "right",

            targetID: null,

            showRoundedCorner: false,

            prefixIcon: null,

            suffixIcon: null,

            cssClass: "",

            enableRTL: false,

            create: null,

            beforeOpen: null,

            click: null,

            itemMouseOver: null,

            itemMouseOut: null,

            itemSelected: null,
			
			open: null,
			
		    close: null,

            destroy: null
        },

        dataTypes: {
            size: "string",
            enabled: "boolean",
            showRoundedCorner: "boolean",
            text: "string",
            contentType: "enum",
            imagePosition: "enum",
            buttonMode: "enum",
            arrowPosition: "enum",
            targetID: "string",
            prefixIcon: "string",
            suffixIcon: "string",
            cssClass: "string",
            enableRTL: "boolean",
            htmlAttributes: "data"
        },


        disable: function () {
            this.element.addClass("e-disable");
            if (this.contstatus) {
                this._hidePopup();
            }
            if (this.model.buttonMode == ej.ButtonMode.Split)
                this.dropbutton.addClass("e-disable").attr("aria-disabled", true);
            if (this.model.buttonMode == ej.ButtonMode.Dropdown)
                this.btnimgwrap.addClass("e-disable").attr("aria-disabled", true);
            this.model.enabled = false;
        },

        enable: function () {
            this.element.removeClass("e-disable");
            if (this.model.buttonMode == ej.ButtonMode.Split)
                this.dropbutton.removeClass("e-disable").attr("aria-disabled", false);
            if (this.model.buttonMode == ej.ButtonMode.Dropdown)
                this.btnimgwrap.removeClass("e-disable").attr("aria-disabled", false);
            this.model.enabled = true;
        },

        hide: function () {
            if (this.contstatus) {
                this._hidePopup();
            }
        },

        show: function () {
            if (!this.contstatus) {
                if (this.model.buttonMode == ej.ButtonMode.Dropdown)
                    this.element.click();
                else if (this.model.buttonMode == ej.ButtonMode.Split)
                    this.dropbutton.click();
            }
        },


        _init: function () {
            this._cloneElement = this.element.clone();
            this._initialize();
            this._controlStatus(this.model.enabled);
            this._wireEvents();
        },


        _destroy: function () {
            this.splitwrap.removeClass("e-drop");
            this.innerWrap.removeClass("e-splitarrowright e-splitarrowleft e-splitarrowbottom e-splitarrowtop");
            this.element.removeClass(this.model.cssClass + " e-select e-corner e-btn e-disable e-split-btn e-droparrowright e-droparrowleft e-droparrowbottom e-droparrowtop e-left-btn e-txt").empty();
            this.element.append(this._cloneElement.text());
            this.element.insertAfter(this.wrapper);
            this.wrapper.remove();
            $("#" + this.model.targetID).ejMenu('destroy');
            //this has to be worked out in Menu
            $("#" + this.model.targetID).insertAfter(this.element);
            this._off(this.element, "click", this._btnMouseClick);
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "width":
                        this._splitbtnWidth(options[option]);
                        break;
                    case "height":
                        this._splitbtnHeight(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "buttonMode":
                        this._setButtonMode(options[option]);
                        break;
                    case "arrowPosition":
                        this._setArrowPosition(options[option]);
                        break;
                    case "text":
                        this._setText(options[option]);
                        break;
                    case "prefixIcon":
                        this._setMajorIcon(options[option]);
                        break;
                    case "suffixIcon":
                        this._setMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "targetID":
                        this._setTargetId(options[option]);
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        $("#" + this.model.targetID).ejMenu({ enableRTL: options[option] });
                        break;
                    case "htmlAttributes":
                        this._addAttr(options[option]);
                        break;
                }
            }
        },

        _setText: function (val) {
            if (this.model.contentType == ej.ContentType.TextOnly) {
                if (this.model.buttonMode == ej.ButtonMode.Split)
                    this.element.html(val);
                else {
                    this.element.empty();
                    this.imgtxtwrap = val;
                    if (this.model.arrowPosition == ej.ArrowPosition.Left || this.model.arrowPosition == ej.ArrowPosition.Top)
                        this.element.append(this.btnimgwrap, this.imgtxtwrap);
                    else
                        this.element.append(this.imgtxtwrap, this.btnimgwrap);
                }
            } else {
                this.textspan.html(val);
            }
        },

        _setMajorIcon: function (val) {
            this.majorimgtag.removeClass(this.model.prefixIcon);
            this.majorimgtag.addClass(val);
        },

        _setMinorIcon: function (val) {
            this.minorimgtag.removeClass(this.model.suffixIcon);
            this.minorimgtag.addClass(val);
        },
        _setTargetId: function (val) {
            this.model.targetID = val;
            this._renderContxtMenu();
        },

        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.element.empty();
                this.model.contentType = val;
                this._renderButtonContent();
            }
        },

        _setImagePosition: function (val) {
            if (val == ej.ImagePosition.ImageRight || val == ej.ImagePosition.ImageLeft || val == ej.ImagePosition.ImageBottom || val == ej.ImagePosition.ImageTop) {
                if ((this.model.contentType == ej.ContentType.TextAndImage) && (val != this.model.imagePosition)) {
                    this.element.empty();
                    this.model.imagePosition = val;
                    this._renderButtonContent();
                }
            }
        },

        _setButtonMode: function (val) {
            if (val == ej.ButtonMode.Split || val == ej.ButtonMode.Dropdown) {
                if (val != this.model.buttonMode) {
                    this._destroy();
                    this.model.buttonMode = val;
                    this._init();
                }
            }
        },

        _setArrowPosition: function (val) {
            if (val == ej.ArrowPosition.Right || val == ej.ArrowPosition.Left || val == ej.ArrowPosition.Bottom || val == ej.ArrowPosition.Top) {
                if ((this.model.buttonMode == ej.ButtonMode.Dropdown) && (val != this.model.arrowPosition)) {
                    this.model.arrowPosition = val;
                    this.element.empty();
                    this._setSize(this.model.size);
                    this.element.removeClass("e-droparrowright e-droparrowleft e-droparrowbottom e-droparrowtop");
                    this._renderButtonContent();
                }
                else if ((this.model.buttonMode == ej.ButtonMode.Split) && (val != this.model.arrowPosition)) {
                    this.model.arrowPosition = val;
                    this._setSize(this.model.size);
                    this.innerWrap.removeClass("e-splitarrowright e-splitarrowleft e-splitarrowbottom e-splitarrowtop");
                    this._setRTL(this.model.enableRTL);
                }
            }
        },

        _setRTL: function (val) {
            if (this.model.buttonMode == ej.ButtonMode.Split) {
                this.dropdownimg.removeClass("e-up-arrow").addClass("e-down-arrow");
                switch (this.model.arrowPosition) {
                    case ej.ArrowPosition.Right:
                        this.innerWrap.addClass("e-splitarrowright");
                        break;
                    case ej.ArrowPosition.Left:
                        this.innerWrap.addClass("e-splitarrowleft");
                        break;
                    case ej.ArrowPosition.Bottom:
                        this.innerWrap.addClass("e-splitarrowbottom");
                        break;
                    case ej.ArrowPosition.Top:
                        this.innerWrap.addClass("e-splitarrowtop");
                        this.dropdownimg.addClass("e-up-arrow").removeClass("e-down-arrow");
                        break;
                }
                val == true ? this.splitwrap.addClass("e-rtl e-btnrtl") : this.splitwrap.removeClass("e-rtl e-btnrtl");
            }
            else
                val == true ? this.splitwrap.addClass('e-rtl') : this.splitwrap.removeClass('e-rtl');
            this.model.enableRTL = val;
            this._roundedCorner(this.model.showRoundedCorner);
        },

        _roundedCorner: function (value) {
            value == true ? this.element.addClass('e-corner') : this.element.removeClass('e-corner');
        },

        _controlStatus: function (value) {
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },

        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass);
            if (this.model.buttonMode == ej.ButtonMode.Split) {
                this.dropbutton.removeClass(this.model.cssClass);
                this.dropbutton.addClass(skin);
            }
            this.element.addClass(skin);
            $('#' + this.model.targetID).ejMenu('option', 'cssClass', skin);
        },

        _initialize: function () {
            if (this.element.is("button")) {
                this._render();
            } else {
                this.element.removeClass("e-splitbutton");//need to change in src level
            }
            this._timeout = null;
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy._controlStatus(false);
                else proxy.wrapper.attr(key, value)
            });
        },


        _render: function () {
            this.element.addClass(this.model.cssClass + " e-btn e-select e-split-btn").attr("role", "button");
            if ((this.model.text == null) || (this.model.text == "")) {
                this.model.text = this.element.text();
            }
            else
                this.element.attr("aria-describedby", this.model.text);
            if (this.model.buttonMode != ej.ButtonMode.Split && this.model.buttonMode != ej.ButtonMode.Dropdown)
                this.model.buttonMode = ej.ButtonMode.Split;
            if (this.model.arrowPosition != ej.ArrowPosition.Right && this.model.arrowPosition != ej.ArrowPosition.Left && this.model.arrowPosition != ej.ArrowPosition.Bottom && this.model.arrowPosition != ej.ArrowPosition.Top)
                this.model.arrowPosition = ej.ArrowPosition.Right;
            this.element.empty();

            this.splitwrap = (this.model.buttonMode == ej.ButtonMode.Split ? ej.buildTag('span.e-split e-widget') : ej.buildTag('span.e-split e-drop e-widget'));
            this.splitwrap.insertBefore(this.element);
            this.innerWrap = ej.buildTag('span.e-in-wrap e-box e-padding');
            this.splitwrap.append(this.innerWrap);
            this.wrapper = this.splitwrap;
            /*DropDown Image*/
            this.dropdownimg = ej.buildTag('span.e-icon e-down-arrow');
            this.btnimgwrap = ej.buildTag('div.e-split-btn-div');
            this.btnimgwrap.append(this.dropdownimg);
            /*Split Button*/
            if (this.model.buttonMode == ej.ButtonMode.Split) {
                this.dropbutton = ej.buildTag('button.e-split-btn e-btn e-select ' + this.model.cssClass + ' e-drp-btn#' + this.element[0].id + 'drpbtn', "", {}, { type: "button", "data-role": "none" });
                this.dropbutton.append(this.btnimgwrap);
                this.dropbutton.insertAfter(this.element);
                if (this.model.contentType == ej.ContentType.TextOnly)
                    this.dropbutton.addClass("e-btn-txt");
                else
                    this.dropbutton.addClass("e-rht-btn");
            }
            this._setSize(this.model.size);
            this.element.addClass("e-left-btn");
            this._renderButtonContent();
            if (this.model.buttonMode == ej.ButtonMode.Dropdown)
                this.innerWrap.append(this.element);
            else
                this.innerWrap.append(this.element, this.dropbutton);
            /*Rendering Context menu*/
            this._renderContxtMenu();
            this._roundedCorner(this.model.showRoundedCorner);
            this._setRTL(this.model.enableRTL);
            this._addAttr(this.model.htmlAttributes);
        },

        _renderButtonContent: function () {
            /*Image and Text*/
            this.textspan = ej.buildTag('span.e-btntxt', this.model.text);
            this.majorimgtag = ej.buildTag('span.e-icon ' + this.model.prefixIcon);
            this.minorimgtag = ej.buildTag('span.e-icon ' + this.model.suffixIcon);
            this.imgtxtwrap = ej.buildTag('div');
            /*Rendering Option*/
            if (this.model.contentType == ej.ContentType.TextAndImage) {
                switch (this.model.imagePosition) {
                    case ej.ImagePosition.ImageRight:
                        this.imgtxtwrap.append(this.textspan, this.majorimgtag);
                        break;
                    case ej.ImagePosition.ImageLeft:
                        this.imgtxtwrap.append(this.majorimgtag, this.textspan);
                        break;
                    case ej.ImagePosition.ImageBottom:
                        this.majorimgtag.css("display", "inline-table");
                        this.imgtxtwrap.append(this.textspan, this.majorimgtag);
                        break;
                    case ej.ImagePosition.ImageTop:
                        this.majorimgtag.css("display", "inline-table");
                        this.imgtxtwrap.append(this.majorimgtag, this.textspan);
                        break;
                }
            } else if (this.model.contentType == ej.ContentType.ImageTextImage) {
                this.imgtxtwrap.append(this.majorimgtag, this.textspan, this.minorimgtag);
            } else if (this.model.contentType == ej.ContentType.ImageBoth) {
                this.imgtxtwrap.append(this.majorimgtag, this.minorimgtag);
            } else if (this.model.contentType == ej.ContentType.ImageOnly) {
                this.imgtxtwrap.append(this.majorimgtag);
            } else {
                this.element.addClass("e-txt");
                this.imgtxtwrap = this.model.text;
            }
            if (this.model.buttonMode == ej.ButtonMode.Dropdown)
                this._renderDropdownArrow();
            else
                this.element.append(this.imgtxtwrap);
        },

        _renderDropdownArrow: function () {
            this.btnimgwrap.css("position", "absolute");
            this.dropdownimg.removeClass("e-up-arrow").addClass("e-down-arrow");
            switch (this.model.arrowPosition) {
                case ej.ArrowPosition.Right:
                    this.element.addClass("e-droparrowright");
                    this.element.append(this.imgtxtwrap, this.btnimgwrap);
                    break;
                case ej.ArrowPosition.Left:
                    this.element.addClass("e-droparrowleft");
                    this.element.append(this.btnimgwrap, this.imgtxtwrap);
                    break;
                case ej.ArrowPosition.Bottom:
                    this.element.addClass("e-droparrowbottom");
                    this.element.append(this.imgtxtwrap, this.btnimgwrap);
                    break;
                case ej.ArrowPosition.Top:
                    this.element.addClass("e-droparrowtop");
                    this.dropdownimg.addClass("e-up-arrow").removeClass("e-down-arrow");
                    this.element.append(this.btnimgwrap, this.imgtxtwrap);
                    break;
            }
        },


        _setSize: function (val) {
            var ht = this.model.height, wd = this.model.width;
            switch (val) {
                case "mini":
                    if (ht == "") ht = "28px";
                    if (wd == "") wd = "63px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '12px');
                    break;
                case "small":
                    if (ht == "") ht = "32px";
                    if (wd == "") wd = "89px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '13px');
                    break;
                case "medium":
                    if (ht == "") ht = "36px";
                    if (wd == "") wd = "113px";
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    this.element.css('font-size', '14px');
                    break;
                case "large":
                    if (ht == "") ht = "40px";
                    if (wd == "") wd = "130px";
                    this.element.css('font-size', '15px');
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    break;
                default:
                    this._splitbtnHeight(ht);
                    this._splitbtnWidth(wd);
                    break;
            }
            if (this.model.arrowPosition == ej.ArrowPosition.Bottom && this.model.height == "")
                this._splitbtnHeight(this.splitwrap.height() + (this.model.buttonMode == ej.ButtonMode.Dropdown ? 15 : 24));  // 15px added the height of the wrapper due to Arrow positioned in bottom
            else if (this.model.arrowPosition == ej.ArrowPosition.Top && this.model.height == "")
                this._splitbtnHeight(this.splitwrap.height() + (this.model.buttonMode == ej.ButtonMode.Dropdown ? 10 : 24));  // 10px added the height of the wrapper due to Arrow positioned in top
        },

        _splitbtnHeight: function (val) {
            if ((val == "") || (val == null)) val = '30px';
            this.splitwrap.css("height", val);
        },


        _splitbtnWidth: function (val) {
            this.splitwrap.css("width", val);
        },


        _renderContxtMenu: function () {
            $("#" + this.model.targetID).ejMenu({
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                contextMenuTarget: "",
                fields: this.model.fields,
                showArrow: true,
                cssClass: "e-split " + this.model.cssClass,
                enableRTL: this.model.enableRTL
            }).on("ejMenuclose", $.proxy(this._onKeyDown, this));
        },
        _onKeyDown: function (e) {
            e.keyCode==27 && this._hide();
        },
        _itemClick: function (args) {
            args = { status: this.model.enabled, ID: args.ID, text: args.text };
            this._hide();
            this._trigger("itemSelected", args);			
        },

        _itemMouseOver: function (args) {
            this._trigger("itemMouseOver", args);
        },

        _itemMouseOut: function (args) {
            this._trigger("itemMouseOut", args);
        },


        _wireEvents: function () {
            this._on(this.element, "click", this._btnMouseClick);
            this._on(this.element, "mousedown", this._btnMouseDown);
            /*DrpBTN*/
            if (this.model.buttonMode == ej.ButtonMode.Split)
                this._on(this.dropbutton, "click", this._btnMouseClick);
            /*DocClk*/
            this._on($(document), "mousedown", this._docrhtclk);
        },


        _btnMouseClick: function (e) {
            var args;
            if (!$(e.currentTarget).hasClass("e-disable")) {
                if (e.currentTarget.id != this.element[0].id + "drpbtn" && this.model.buttonMode == ej.ButtonMode.Split) {
                    args = { status: this.model.enabled };
                    this._trigger("click", args);
                } else {
                    if (this._trigger("beforeOpen")) return;
                    this.wrapper.addClass('e-active');
                    if (this.contstatus) {
                        this._hidecontext(e);
                    } else {
                        this._contextPosition(e);
						this._trigger("open");
                        this._on($(window),"resize",this._OnWindowResize);
                        this.contstatus = true;
                        this.element.bind("click", $.proxy(this._hidecontext, this));
                        this._on($(document), "mousedown", this._documentClick);
                        this._on(ej.getScrollableParents(this.wrapper), "scroll", this._hidePopup);
                    }
                }
            }
        },

        _OnWindowResize:function(e)
        {
            this._contextPosition(e);
        },

        _contextPosition:function(e)
        {
            var position = this._getXYpos(e),posleft,targetElement;
            targetElement = (this.model.buttonMode == ej.ButtonMode.Split ? this.dropbutton : this.element);
            var contextObj = $("#" + this.model.targetID).ejMenu("instance");
            posleft = position.x - ($("#" + this.model.targetID).outerWidth() - (this.model.buttonMode == ej.ButtonMode.Split ? this.dropbutton.outerWidth() : this.element.outerWidth()));
            if (this.model.enableRTL)
                position.x = (posleft < $("#" + this.model.targetID).outerWidth()) ? position.x : posleft;
            else
                position.x = (position.x + $("#" + this.model.targetID).outerWidth() < $(window).width()) ? position.x : posleft;

            contextObj.option({
                click: $.proxy(this._itemClick, this),
                mouseover: $.proxy(this._itemMouseOver, this),
                mouseout: $.proxy(this._itemMouseOut, this)
            });
            contextObj.show(position.x, position.y, targetElement, e);
        },

        _getXYpos: function (e) {
            var btnpos, btnposx, btnposy, poscur = 1, postop;
            btnpos = this._getOffset(this.model.buttonMode == ej.ButtonMode.Split ? this.dropbutton : this.element);
            btnposx = btnpos.left;
            postop = this.model.arrowPosition == ej.ArrowPosition.Top ? (btnpos.top - $("#" + this.model.targetID).outerHeight() + 1) : (this.model.buttonMode == ej.ButtonMode.Split ? (btnpos.top + this.dropbutton.outerHeight()) : (btnpos.top + this.element.outerHeight())) - poscur;//1px added to top due to element border-top as none
            btnposy = (postop < 0) ? (btnpos.top + (this.model.buttonMode == ej.ButtonMode.Split ? this.dropbutton.outerHeight() + this.element.outerHeight() : this.element.outerHeight()) - poscur) : postop;
            return { x: btnposx, y: btnposy };
        },
        _getOffset: function (ele) {
            var pos = ele.offset();
            if ($("body").css("position") != "static") {
                var bodyPos = $("body").offset();
                pos.left -= bodyPos.left;
                pos.top -= bodyPos.top;
            }
            return pos;
        },


        _btnMouseDown: function (e) {
            if (!$(e.currentTarget).hasClass("e-disable")) {
                this._docrhtclk(e);
            }
        },
        _hidePopup: function (e) {
            $("#" + this.model.targetID).ejMenu('hide', e);
            this._hide();
            this._off(ej.getScrollableParents(this.wrapper), "scroll", this._hidePopup);            
        },
        _hide: function () {
            this.contstatus = false;
            this.wrapper.removeClass('e-active');
            this.element.unbind("click", $.proxy(this._hidecontext, this));
            this._off($(document), "mousedown", this._documentClick);
            this._off($(window),"resize", this._OnWindowResize);
            this._off(ej.getScrollableParents(this.wrapper), "scroll", this._hide);
            this._closeEvent();
        },
        _closeEvent: function () {
            this._trigger("close");
        },

        _hidecontext: function (e) {
            if (($(e.target).is(this.element) || $(e.target).is(this.dropbutton) || $(e.target).is(this.textspan) || $(e.target).is(this.dropdownimg) || $(e.target).is(this.btnimgwrap) || !$(e.target).is(this.majorimgtag) || !$(e.target).is(this.minorimgtag)) && !$(e.target).is(this.splitwrap) && !$(e.target).parents().is($("#" + this.model.targetID)) || (this.element.hasClass("e-txt") || $(e.target).is(this.imgtxtwrap))) {
                this._hidePopup(e);
            }
        },

        _documentClick: function (e) {
            if (!$(e.target).is(this.element) && !$(e.target).is(this.dropbutton) && !$(e.target).is(this.textspan) && !$(e.target).is(this.dropdownimg) && !$(e.target).is(this.btnimgwrap) && !$(e.target).is(this.majorimgtag) && !$(e.target).is(this.minorimgtag) && !$(e.target).is(this.splitwrap) && !$(e.target).parents().is($("#" + this.model.targetID)) && (this.element.hasClass("e-txt") || !$(e.target).is(this.imgtxtwrap))) {
                this._hidePopup(e);
            }
        },

        _docrhtclk: function (e) {
            var isRightClick, targetElement;
            isRightClick = false;
            if (e.button) {
                isRightClick = (e.button == 2);
            } else if (e.which) {
                isRightClick = (e.which == 3); //for Opera
            }
            targetElement = e.target;
            if (isRightClick) {
                e.preventDefault();
            }
        },

    });

    ej.ContentType = { TextOnly: "textonly", ImageOnly: "imageonly", ImageBoth: "imageboth", TextAndImage: "textandimage", ImageTextImage: "imagetextimage" };

    ej.ImagePosition = { ImageRight: "imageright", ImageLeft: "imageleft", ImageTop: "imagetop", ImageBottom: "imagebottom" };

    ej.ButtonSize = { Mini: "mini", Small: "small", Medium: "medium", Large: "large" };

    ej.ButtonMode = { Split: "split", Dropdown: "dropdown" };

    ej.ArrowPosition = { Right: "right", Left: "left", Top: "top", Bottom: "bottom" };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejToggleButton", "ej.ToggleButton", {

        element: null,

        model: null,
        validTags: ["input"],
        _setFirst: false,

        _rootCSS: "e-togglebutton",

        defaults: {

            size: "normal",

            type: "button",

            width: "",

            height: "",

            enabled: true,

            toggleState: false,

            defaultText: null,

            preventToggle: false,

            activeText: null,

            contentType: "textonly",

            htmlAttributes: {},

            imagePosition: "imageleft",

            showRoundedCorner: false,

            enablePersistence: false,

            cssClass: "",

            defaultPrefixIcon: null,

            defaultSuffixIcon: null,

            activePrefixIcon: null,

            activeSuffixIcon: null,

            enableRTL: false,

            create: null,

            click: null,

            change: null,

            destroy: null
        },


        dataTypes: {
            size: "string",
            type: "enum",
            enabled: "boolean",
            showRoundedCorner: "boolean",
            preventToggle: "boolean",
            defaultText: "string",
            activeText: "string",
            contentType: "enum",
            imagePosition: "enum",
            defaultPrefixIcon: "string",
            defaultSuffixIcon: "string",
            activePrefixIcon: "string",
            activeSuffixIcon: "string",
            cssClass: "string",
            enableRTL: "boolean",
            enablePersistence: "boolean",
            htmlAttributes: "data"
        },
        observables: ["toggleState"],
        toggleState: ej.util.valueFunction("toggleState"),


        disable: function () {
            this.buttontag.addClass("e-disable").attr({ "aria-disabled": true });
            this.model.enabled = false;
        },

        enable: function () {
            if (this.buttontag.hasClass("e-disable")) {
                this.buttontag.removeClass("e-disable").attr({ "aria-disabled": false });
                this.model.enabled = true;
            }
        },


        _init: function () {
            this._initialize();
            this._controlStatus(this.model.enabled);
            this._wireEvents();
            this.initialRender = false;
        },

        _destroy: function () {
            this.element.unwrap();
            this.element.removeClass('e-chkbx-hidden');
            this.labelFinder.empty();
            this.labelFinder.text(this.model.defaultText);
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "size":
                        this._setSize(options[option]);
                        break;
                    case "type":
                        this._settype(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "contentType":
                        this._setContentType(options[option]);
                        break;
                    case "imagePosition":
                        this._setImagePosition(options[option]);
                        break;
                    case "defaultText":
                        this._setDefaultText(options[option]);
                        break;
                    case "activeText":
                        this._setActiveText(options[option]);
                        break;
                    case "defaultPrefixIcon":
                        this._setDefaultMajorIcon(options[option]);
                        break;
                    case "defaultSuffixIcon":
                        this._setDefaultMinorIcon(options[option]);
                        break;
                    case "activePrefixIcon":
                        this._setActiveMajorIcon(options[option]);
                        break;
                    case "activeSuffixIcon":
                        this._setActiveMinorIcon(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "toggleState":
                        this._tglevaluestatus(ej.util.getVal(options[option]));
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }
        },

        _setSize: function (val) {
            this.buttontag.removeClass('e-btn-mini e-btn-medium e-btn-small e-btn-large e-btn-normal');
            this.buttontag.addClass("e-btn-" + val);
        },

        _settype: function (val) {
            this.model.type = val;
        },


        _setHeight: function (val) {
            this.buttontag.css('height', val);
        },

        _setWidth: function (val) {
            this.buttontag.css('width', val);
        },

        _setDefaultText: function (val) {
            if (!this.toggleState()) {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.buttontag.html(val);
                } else {
                    this.defaulttxtspan.html(val);
                }
            }	
        },

        _setActiveText: function (val) {
            if (this.toggleState()) {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.buttontag.html(val);
                } else {
                    this.defaulttxtspan.html(val);
                }
            }
        },

        _setDefaultMajorIcon: function (val) {
            this.defMainIcon.removeClass(this.model.defaultPrefixIcon);
            this.defMainIcon.addClass(val);
        },

        _setDefaultMinorIcon: function (val) {
            this.defMiniIcon.removeClass(this.model.defaultSuffixIcon);
            this.defMiniIcon.addClass(val);
        },

        _setActiveMajorIcon: function (val) {
            if (this.toggleState()) {
                this.defMainIcon.removeClass(this.model.activePrefixIcon);
                this.defMainIcon.addClass(val);
            }
        },

        _setActiveMinorIcon: function (val) {
            if (this.toggleState()) {
                this.defMiniIcon.removeClass(this.model.activeSuffixIcon);
                this.defMiniIcon.addClass(val);
            }
        },

        _setContentType: function (val) {
            if (val != this.model.contentType) {
                this.buttontag.empty();
                this.model.contentType = val;
                this._renderButtonContent();
            }
        },

        _setImagePosition: function (val) {
            if ((this.model.contentType == ej.ContentType.TextAndImage) && (val != this.model.imagePosition)) {
                this.buttontag.empty();
                this.model.imagePosition = val;
                this._renderButtonContent();
            }
        },

        _setRTL: function (val) {
            if (val) {
                this.buttontag.addClass("e-rtl");
            } else {
                this.buttontag.removeClass("e-rtl");
            }
        },

        _controlStatus: function (value) {
            if (!value) {
                this.disable();
            } else {
                this.enable();
            }
        },

        _setSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.buttontag.removeClass(this.model.cssClass);
                this.buttontag.addClass(skin);
            }
        },

        _initialize: function () {
            if (this.element.is('[type = "checkbox"]')) {
                this._render();
            } else {
                this.element.removeClass("e-togglebutton");
            }
        },


        _render: function () {
			this.initialRender = true;
            var predecessor, labelSelector;
            /*Determine Button Type*/
            this.element.addClass('e-chkbx-hidden e-tbtn');
            if (!this.element.attr("name"))
                this.element.attr("name", this.element[0].id);
            /*Finding the label element in the dom*/
            predecessor = this.element.parents().last();
            labelSelector = "label[for='" + this.element[0].id + "']";
            this.labelFinder = predecessor.find(labelSelector);
            if (!this.labelFinder.length) {
                this.labelFinder = ej.buildTag('label', "Button", {}, { "for": this.element[0].id });
                this.labelFinder.insertAfter(this.element);
            }
            if ((this.model.defaultText == null) || (this.model.defaultText == "")) {
                this.model.defaultText = this.labelFinder.text();
            }
            this.labelFinder.empty();
            this.wrapper = $('<span id="' + this.element[0].id + '-wrapper" class="e-tbtn-wrap e-widget"></span>');
            $(this.element).wrap(this.wrapper);
            this.labelFinder.insertAfter(this.element);
            this.buttontag = ej.buildTag('button.e-togglebutton e-btn e-tbtn ' + this.model.cssClass + ' e-select', "", {}, { "role": "button", "tabindex": 0, "type": this.model.type, "data-role": "none" });
            this.labelFinder.append(this.buttontag);
            if (ej.util.isNullOrUndefined(this.model.activeText)) {
                this.model.activeText = this.model.defaultText;
            }
            if (ej.util.isNullOrUndefined(this.model.activePrefixIcon)) {
                this.model.activePrefixIcon = this.model.defaultPrefixIcon;
            }
            if (ej.util.isNullOrUndefined(this.model.activeSuffixIcon)) {
                this.model.activeSuffixIcon = this.model.defaultSuffixIcon;
            }
            this._setSize(this.model.size);
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
            this._setRTL(this.model.enableRTL);
            this._renderButtonContent();
            if (!this.toggleState() && !ej.isNullOrUndefined(this.element.attr('checked')))
                this.toggleState(true);
            this._tglevaluestatus(this.toggleState());
            this._roundedCorner(this.model.showRoundedCorner);
            this._addAttr(this.model.htmlAttributes);
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "checked" && value == "checked") proxy._tglevaluestatus(this.toggleState());
                else proxy.wrapper.attr(key, value);
            });
        },

        _renderButtonContent: function () {
            this.imgtxtwrap = ej.buildTag('div');
            this.defaulttxtspan = ej.buildTag('span.e-btntxt' + '#' + this.element[0].id + 'textstatic',(this.toggleState() ? this.model.activeText : this.model.defaultText));
            if (this.model.contentType.indexOf("image") > -1) {
                this.defMainIcon = ej.buildTag('span.e-icon ' + (this.toggleState() ? this.model.activePrefixIcon : this.model.defaultPrefixIcon) + '#' + this.element[0].id + 'mainiconstatic');
                this.defMiniIcon = ej.buildTag('span.e-icon ' + (this.toggleState() ? this.model.activeSuffixIcon : this.model.defaultSuffixIcon) + '#' + this.element[0].id + 'miniconstatic');
            }
            if (this.model.contentType == ej.ContentType.TextAndImage) {
                switch (this.model.imagePosition) {
                    case ej.ImagePosition.ImageRight:
                        this.imgtxtwrap.append(this.defaulttxtspan, this.defMainIcon);
                        break;
                    case ej.ImagePosition.ImageLeft:
                        this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan);
                        break;
                    case ej.ImagePosition.ImageBottom:
                        this.defMainIcon.attr("style", "display:inherit");
                        this.imgtxtwrap.append(this.defaulttxtspan, this.defMainIcon);
                        break;
                    case ej.ImagePosition.ImageTop:
                        this.defMainIcon.attr("style", "display:inherit");
                        this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan);
                        break;
                }
                this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageTextImage) {
                this.imgtxtwrap.append(this.defMainIcon, this.defaulttxtspan, this.defMiniIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageOnly) {
                this.imgtxtwrap.append(this.defMainIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else if (this.model.contentType == ej.ContentType.ImageBoth) {
                this.imgtxtwrap.append(this.defMainIcon, this.defMiniIcon);
                this.buttontag.append(this.imgtxtwrap);
            } else {
                this.buttontag.addClass("e-txt");
                this.buttontag.text(this.model.defaultText);
            }
        },

        _tglevaluestatus: function (value) {
            if (value) {
                this._toggleButtonStatus(value);
                this.element.attr("checked", "checked");
            } else {
                this._toggleButtonStatus(value);
                this.element.removeAttr("checked");
            }
        },

        _roundedCorner: function (value) {
            value == true ? this.buttontag.addClass('e-corner') : this.buttontag.removeClass('e-corner');
        },


        _wireEvents: function () {
            this._on(this.buttontag, "click", this._tglebtnclicked);
        },

        _tglebtnclicked: function (e) {
            if (!this.model.preventToggle) {
                if (!this.buttontag.hasClass("e-disable")) {
                    this.toggleState(this.toggleState() ? false : true);
                    if (ej.browserInfo().name == "msie" && (ej.browserInfo().version == "8.0"))
                        this._tglevaluestatus(this.toggleState());
                    else
                        this._toggleButtonStatus(this.toggleState());
                    $(this.element).prop("checked", this.toggleState());
                    e.preventDefault();
                    this._trigger("click", { "isChecked": this.toggleState(), status: this.model.enabled });
                }
            }
        },

        _toggleButtonStatus: function (buttonstatus) {
            if (buttonstatus) {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.buttontag.html(this.model.activeText);
                } else {
                    this.defaulttxtspan.html(this.model.activeText);
                    this.defMainIcon.removeClass(this.model.defaultPrefixIcon).addClass(this.model.activePrefixIcon);
                    this.defMiniIcon.removeClass(this.model.defaultSuffixIcon).addClass(this.model.activeSuffixIcon);
                }
                this.buttontag.addClass("e-active").attr("aria-pressed", true);
            } else {
                if (this.model.contentType == ej.ContentType.TextOnly) {
                    this.buttontag.html(this.model.defaultText);
                } else {
                    this.defaulttxtspan.html(this.model.defaultText);
                    this.defMainIcon.removeClass(this.model.activePrefixIcon).addClass(this.model.defaultPrefixIcon);
                    this.defMiniIcon.removeClass(this.model.activeSuffixIcon).addClass(this.model.defaultSuffixIcon);
                }
                this.buttontag.removeClass("e-active").attr("aria-pressed", false);
            }
            this.toggleState(buttonstatus);
            if(!this.initialRender)
            this._trigger('change', { "isChecked": this.toggleState() });
        },

    });

    ej.ContentType = {
		/**  Supports only for text content only */
		TextOnly: "textonly", 
		/** Supports only for image content only */
		ImageOnly: "imageonly", 
		/** Supports image for both ends of the button */
		ImageBoth: "imageboth", 
		/** Supports image with the text content */
		TextAndImage: "textandimage", 
		/** Supports image with both ends of the text */
        ImageTextImage: "imagetextimage"
    };


    ej.ImagePosition = {
		/**  support for aligning text in left and image in right. */
		ImageRight: "imageright", 
		/**  support for aligning text in right and image in left. */
		ImageLeft: "imageleft", 
		/**  support for aligning text in bottom and image in top. */
		ImageTop: "imagetop", 
		/**  support for aligning text in top and image in bottom. */
        ImageBottom: "imagebottom"
    };

    ej.ButtonSize = {
		/**  Creates button with inbuilt default size height, width specified */
		Normal : "normal",
		/**  Creates button with inbuilt mini size height, width specified */
		Mini: "mini", 
		/**  Creates button with inbuilt small size height, width specified */
		Small: "small",
		/**  Creates button with inbuilt medium size height, width specified */
		Medium:"medium", 
		/**  Creates button with inbuilt large size height, width specified */
		Large: "large" 
    };
        
    ej.ButtonType = {
        /**  Creates button with button type as button */
        Button: "button",
        /**  Creates button with button type as reset */
        Reset: "reset",
        /**  Creates button with button type as submit */
        Submit: "submit"

    };
})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Toolbar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejToolbar", "ej.Toolbar", {

        element: null,

        model: null,
        validTags: ["div", "span"],
        _ignoreOnPersist: ["fields", "dataSource", "query", "create", "click", "itemHover", "itemLeave", "destroy"],
        _setFirst: false,

        _rootCSS: "e-toolbar",
        angular: {
            terminal: false
        },


        defaults: {

            height: "",

            width: "",

            enabled: true,

            hide: false,

            enableSeparator: false,

            orientation: "horizontal",

            enableRTL: false,

            isResponsive: false,

            showRoundedCorner: false,

            htmlAttributes: {},

            dataSource: null,

            query: null,

            fields: {

                id: "id",

                tooltipText: "tooltipText",

                imageUrl: "imageUrl",

                text: "text",

                imageAttributes: "imageAttributes",

                spriteCssClass: "spriteCssClass",

                htmlAttributes: "htmlAttributes",

                group: "group",
            },

            cssClass: "",

            create: null,

            click: null,

            itemHover: null,

            itemLeave: null,

            destroy: null

        },


        dataTypes: {
            enabled: "boolean",
            hide: "boolean",
            enableSeparator: "boolean",
            orientation: "enum",
            enableRTL: "boolean",
            showRoundedCorner: "boolean",
            isResponsive: "boolean",
            dataSource: "data",
            query: "data",
            fields: "data",
            cssClass: "string",
            htmlAttributes: "data"
        },



        hide: function () {
            if (!this.model.enabled && (this.element.css("display") == "none")) return false;
            this.element.css("display", "none");
        },

        show: function () {
            if (!this.model.enabled && (this.element.css("display") != "none")) return false;
            this.element.css("display", "block");
        },


        _init: function () {
            this._cloneElement = $(this.element).clone();
            if (this.model.dataSource != null) {
                this._generateTemplate();
            }
            (!(this.model.dataSource instanceof ej.DataManager)) && this._initialize();
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "width":
                        this.model.width = options[option];
                        this._setWidth();
                        this._bindUnbindWidth();
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "isResponsive":
                        this.model.isResponsive = options[option];
                        (this.model.isResponsive) ? this._responsiveLayout() : this._removeResponsive();
                        break;
                    case "hide":
                        this._controlVisibleOptions(options[option]);
                        break;
                    case "orientation":
                        this._setOrientation(options[option]);
                        break;
                    case "enableRTL":
                        this._enableRTL(options[option]);
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "htmlAttributes":
                        this._addAttr(options[option]);
                        break;
                    case "enableSeparator":
                        this.model.enableSeparator = options[option];
                        this._renderToolbarSeparator();
                        break;
                    case "fields":
                        $.extend(this.model.fields, options[option]);
                        break;
                    case "query":
                        this.model.query = options[option];
                        break;
                    case "dataSource":
                        for (optio in options) {
                            if (optio == "fields") $.extend(this.model.fields, options[optio]);
                            if (optio == "query") this.model.query = options[optio];
                        }
                        this._refreshTagItems(option, options[option]);
                        break;
                }
            }
        },
        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.element.empty();
            this._generateTemplate();
            this._initialize();
        },

        _setHeight: function (val) {
            this.element.css('height', val);
        },

        _setWidth: function () {
            if (this.model.width == "auto") {
                var itemswidth = parseFloat(this.element.css('border-left-width')) + parseFloat(this.element.css('border-right-width')),
                listElement = this.element.find('ul').children('li'), listUl = this.element.find('ul'),
                resEle = this.element.find('.e-toolbar-res-arrow');
				for(var i=0, len = listUl.length; i < len; i++) {           // calculate separator width also
                    if (listUl[i] != null && $(listUl[i]).hasClass('e-separator'))
                        itemswidth += parseFloat(listUl.eq(i).css('border-right-width')) + parseFloat(listUl.eq(i).css('border-left-width'));
                }
                if (resEle[0] != null)
                    itemswidth += Math.round(this.element.find('.e-toolbar-res-arrow').outerWidth()) + 8; // 8px for position
                for (var i = 0, len = listElement.length; i < len; i++) {
                    if (listElement[i] != null)
                        itemswidth += Math.round(listElement.eq(i).outerWidth());
                }
                this.element.css("width", Math.ceil(itemswidth));
            } else
                this.element.css('width', this.model.width);
        },
        _bindUnbindWidth: function () {
            (this.model.width != "auto") ? this._off($(window), "resize", this._widthHandler) : this._on($(window), "resize", this._widthHandler);
        },
        _widthHandler: function (e) {
            this._setWidth(this.model.width);
        },

        _setOrientation: function (value) {
            if (value != ej.Orientation.Vertical) {
                this.element.removeClass(this.model.cssClass).addClass("e-toolbarspan " + this.model.cssClass);
                this.itemsContainer.removeClass("e-ul e-vertical").addClass("e-ul e-horizontal");
            }
            else {
                this.element.removeClass("e-toolbarspan " + this.model.cssClass).addClass(this.model.cssClass);
                this.itemsContainer.removeClass("e-ul e-horizontal").addClass("e-ul e-vertical");
            }
            this.items.addClass("e-tooltxt");
        },

        _reSizeHandler: function () {
            this._elementWidth = (typeof window.getComputedStyle == "function") ? parseFloat(window.getComputedStyle(this.element[0]).width) : this.element.width()-1; // -1 for IE8 get exact width of an element no border and margin
            if (this._isResized && this._liTemplte.children().length > 0) {
                $(this.element).append(this._liTemplte.children());
                this._renderToolbarSeparator();
                this._liTemplte.addClass("e-display-none");
                this._contstatus = false;
            }
            this._liWidth = this._spanWidth;
            this._spantag.removeClass("e-display-block").addClass("e-display-none");
            for (var i = 0; i < this.itemsContainer.length; i++) {
                this._liWidth += $(this.itemsContainer[i]).outerWidth(true);
                if (this._liWidth > this._elementWidth) {
                    if (!this._spantag.hasClass("e-display-block"))
                        this._spantag.removeClass("e-display-none").addClass("e-display-block");
                    this._spantag.addClass('e-res-pos');
                    this._liTemplte.append(this.itemsContainer[i]);
                }
            }
            this._isResized = true;
        },

        _getZindexPartial: function () {
            return ej.util.getZindexPartial(this.element, this.popup);
        },
        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },

        _btnMouseClick: function (e) {
            var args, btnpos, btnposx, btnposy, poscur = 1, eleWidth = this.element.outerWidth(), eleHeight = this.element.outerHeight(), resWidth = $(this._liTemplte).outerWidth(), winWidth = $(window).width(), winLeft = $(window).scrollLeft();
            if (!$(e.currentTarget).hasClass("e-disable")) {
                /*Context Menu Functionality*///need to write enableRTL
                maxZ = this._getZindexPartial();
                btnpos = this._getOffset(this.element);
                btnposx = this.model.enableRTL ? btnpos.left : (btnpos.left + eleWidth - resWidth);
                btnposy = btnpos.top + eleHeight - poscur + 2;
                btnposx = btnposx < winLeft ? winLeft + poscur : ((btnposx + resWidth) > (winWidth + winLeft) ? winWidth - resWidth + winLeft - poscur : btnposx);
                if (this._contstatus) {
                    this._liTemplte.addClass("e-display-none");
                    this._contstatus = false;
                } else {
                    this._liTemplte.removeClass("e-display-none")
                                   .css({ "top": btnposy + "px", "left": btnposx + "px", "z-index": maxZ });
                    this._contstatus = true;
                    this.element.bind("click", $.proxy(this._hidecontext, this));
                }
                if (this._contstatus)
                    this._on($(document), "mouseup", this._documentClick);
            }
        },

        _hidecontext: function (e) {
            if (!$(e.target).is($("#" + this.element[0].id + "_target"))) {
                this._liTemplte.addClass("e-display-none");
                this._contstatus = false;
                this.element.unbind("click", $.proxy(this._hidecontext, this));
                this._off($(document), "mouseup", this._documentClick);
            }
        },

        _documentClick: function (e) {
            if (!$(e.target).is($("#" + this.element[0].id + "_target")) && !($(e.target).closest("div#" + this.element[0].id + "_hiddenlist").length != 0 || $(e.target).parents().hasClass("e-ddl-popup"))) {
                this._liTemplte.addClass("e-display-none");
                this._contstatus = false;
                this._off($(document), "mouseup", this._documentClick);
            }
        },

        _setSkin: function (skin) {
            this.element.removeClass(this.model.cssClass).addClass(skin);
        },

        _destroy: function () {
            this.element.html("");
            this._cloneElement.removeClass('e-toolbar e-js');
            this.element.replaceWith(this._cloneElement);
            this._liTemplte && this._liTemplte.remove();
        },

        _initialize: function () {
            this.element.attr({ "role": "toolbar", "tabindex": "0" });
            var liList = this.element.children().find('li');
            this._focusEnable = true;
            this._renderControl();
            this._responsiveLayout();
            this._wireEvents();
        },
        _responsiveLayout: function () {
            this._roundedCorner(this.model.showRoundedCorner);
            if (this.model.isResponsive && (this.model.orientation == "horizontal")) {
                this._spantag = $("<span id='" + this.element[0].id + "_target' class='e-icon e-toolbar-res-arrow e-rel-position e-display-block'></span>");
                this._spantag.appendTo(this.element);
                this._spanWidth = this._spantag.outerWidth(true) + 8; // 8px from right of span 
                this._liTemplte = $("<div id='" + this.element[0].id + "_hiddenlist' class='e-responsive-toolbar e-display-none e-abs-position " + $(this.element)[0].className + "'></div>");
                if (this.model.enableRTL) this._liTemplte.addClass('e-rtl');
                this._isResized = false;
                this._reSizeHandler();
                $('body').append(this._liTemplte);
            }
            this._on($("#" + this.element[0].id + "_target"), "mousedown", this._btnMouseClick);
            this._wireResizing();
            this._controlVisibleOptions(this.model.hide);
            this._enableRTL(this.model.enableRTL);
            this._controlStatus(this.model.enabled);
            this._setWidth();
        },
        _removeResponsive: function () {
            $(this._spantag).remove();
            var list = $(this._liTemplte).children('ul');
            $(this._liTemplte).detach();
            $(this.element).append(list);
            this.itemsContainer = this.element.children("ol,ul");
            this.items = this.itemsContainer.children('li');
            this._off(this.items, "mouseup");
            this._off(this.items, "mousedown");
            this._off(this.element);
            this._wireEvents();
            this._wireResizing();
            this._setWidth();
        },

        _controlVisibleOptions: function (value) {
            value != false ? this.hide() : this.show();
        },

        _controlStatus: function (value) {
            value != true ? this.disable() : this.enable();
        },

        _roundedCorner: function (value) {
            if (value) {
                this.element.addClass('e-corner');
                this._liTemplte && this._liTemplte.addClass('e-corner');
            } else {
                this.element.removeClass('e-corner');
                this._liTemplte && this._liTemplte.removeClass('e-corner');
            }
        },

        _generateTemplate: function () {
            var proxy = this, queryPromise;
            this.element.css("visibility", "hidden");
            if (this.model.dataSource instanceof ej.DataManager) {
                queryPromise = this.model.dataSource.executeQuery(this.model.query);
                queryPromise.done(function (e) {
                    proxy._generateGroup(e.result);
                    proxy._initialize();
                    proxy.element.css("visibility", "visible");
                });
            } else {
                proxy._generateGroup(proxy.model.dataSource);
                proxy.element.css("visibility", "visible");
            }
        },
        _generateGroup: function (value) {
            var proxy = this;
            var y = -1;
            var groupArray = [];
            for (var i = 0; i < value.length; i++) {
                if (this._isNewGroup(value[i].group, groupArray)) {
                    groupArray[++y] = value[i].group;
                    var index = -1;
                    proxy.itemsSource = [];
                    for (var x = i; x < value.length; x++) {
                        if (value[i].group == value[x].group) {
                            proxy.itemsSource[++index] = value[x];
                        }
                    }
                    proxy.element.append(proxy._generateTagitems());
                }
            }

        },
        _isNewGroup: function (value, group) {
            if (!group)
                return true;
            for (var i = 0; i < group.length; i++) {
                if (value == group[i])
                    return false;
            }
            return true;
        },


        _renderControl: function () {
            this.element.addClass("e-widget e-box");
            this._renderToolbarItems();
            this._addAttr(this.model.htmlAttributes);
            this._setOrientation(this.model.orientation);
            this._renderToolbarSeparator();
            this._setHeight(this.model.height);
            this._bindUnbindWidth();
        },

        _renderToolbarItems: function () {
            this.target = this.element[0];
            this.itemsContainer = this.element.children("ol,ul");
            this.itemsContainer.children("ol,ul").remove();
            this.items = this.itemsContainer.children('li');
            for (var i = 0; i < this.items.length; i++) {
                if (ej.isNullOrUndefined($(this.items[i]).attr("title")))
                    $(this.items[i]).attr("aria-label", this.items[i].id);
            }
            this._liCount = this.items.length;
        },

        _generateTagitems: function () {
            var list, i;
            list = this.itemsSource;
            this.ultag = ej.buildTag('ul');
            for (i = 0; i < list.length; i++) {
                this.ultag.append(this._generateLi(list[i]));
            }
            return this.ultag;
        },

        _generateLi: function (toolbarItem) {
            var litag, imgtag, divtag, i;
            litag = ej.buildTag('li');
            if (toolbarItem[this.model.fields.id])
                litag.attr('id', toolbarItem[this.model.fields.id]);
            else 
                (toolbarItem[this.model.fields.text]) && litag.attr('id', toolbarItem[this.model.fields.text]);
            if (toolbarItem[this.model.fields.tooltipText])
                litag.attr('title', toolbarItem[this.model.fields.tooltipText]);
            if ((toolbarItem[this.model.fields.imageUrl]) && (toolbarItem[this.model.fields.imageUrl] != "")) {
                //Creating the image tag
                imgtag = ej.buildTag('img.e-align', '', {}, { 'src': toolbarItem[this.model.fields.imageUrl], 'alt': toolbarItem[this.model.fields.text] });
                if (toolbarItem[this.model.fields.imageAttributes]) this._setAttributes(toolbarItem[this.model.fields.imageAttributes], imgtag);
                litag.append(imgtag);
            }
            if ((toolbarItem[this.model.fields.spriteCssClass]) && (toolbarItem[this.model.fields.spriteCssClass] != "")) {
                //Creating the Sprite image tag
                divtag = ej.buildTag('div.e-align ' + toolbarItem[this.model.fields.spriteCssClass] + ' e-spriteimg');
                litag.append(divtag);
            }
            if ((toolbarItem[this.model.fields.text]) && (toolbarItem[this.model.fields.text] != "")) {
                //Creating text Content inside the  tag
                litag.append(toolbarItem[this.model.fields.text]);
            }
            if (toolbarItem[this.model.fields.htmlAttributes]) this._setAttributes(toolbarItem[this.model.fields.htmlAttributes], litag);

            return litag;
        },


        _setAttributes: function (data, element) {
            for (var key in data) {
                $(element).attr(key, data[key]);
            }
        },

        _renderToolbarSeparator: function () {
            var i, len;
            if (this.model.enableSeparator) {
                for (i = 0, len = this.itemsContainer.length - 1; i < len; i++) {
                    $(this.itemsContainer[i]).addClass("e-separator");
                }
                if (this.model.enableRTL) {
                    if (this.itemsContainer.length == 1) {
                        for (i = 0, len = this.itemsContainer[0].children.length; i < len; i++) {
                            if (i == 0) $(this.itemsContainer[0].children[i]).removeClass("e-separator");
                            else if (!this.model.fields.group || !this.model.dataSource || !this.model.dataSource[i].group)
                                $(this.itemsContainer[0].children[i]).addClass("e-separator");
                        }
                    }
                }
                else {
                    if (this.itemsContainer.length == 1) {
                        for (i = 0, len = this.itemsContainer[0].children.length; i < len; i++) {
                            if (i == len - 1) $(this.itemsContainer[0].children[i]).removeClass("e-separator");
                            else if (!this.model.fields.group || !this.model.dataSource || !this.model.dataSource[i].group)
                                $(this.itemsContainer[0].children[i]).addClass("e-separator");
                        }
                    }
                }
            } else {
                this.itemsContainer.removeClass('e-separator').find('.e-separator').removeClass('e-separator');
            }
            this._setWidth();
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else proxy.element.attr(key, value)
            });
        },


        _enableRTL: function (value) {
            if (value) {
                this.element.addClass('e-rtl');
                this.items.addClass('e-comnrtl');
                if (this.model.orientation == ej.Orientation.Horizontal) {
                    this.itemsContainer.addClass('e-comnrtl');
                }
                if (this._liTemplte) this._liTemplte.addClass('e-rtl');
            } else {
                this.element.removeClass('e-rtl');
                this.items.removeClass('e-comnrtl');
                if (this.model.orientation == ej.Orientation.Horizontal) {
                    this.itemsContainer.removeClass('e-comnrtl');
                }
                if (this._liTemplte) this._liTemplte.removeClass('e-rtl');
            }
            this.model.enableRTL = value;
            this._renderToolbarSeparator();
        },

        _addOverlay: function (element) {
            for (var i = 0; i < element.length; i++) {
                if (!$(element[i]).hasClass('e-disable')) {
                    $(element[i]).addClass('e-disable');
                    var ele = ej.buildTag("div.e-item-overlay");
                    $(element[i]).append(ele);
                }
            }
        },
        _removeOverlay: function (element) {
            for (var i = 0; i < element.length; i++) {
                $(element[i]).children(".e-item-overlay").remove();
            }
        },



        disableItem: function (lielement) {
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) return;
            if (!current.hasClass("e-disable")) {
                current.attr("aria-disabled", true).removeAttr("aria-label");
                this._addOverlay(current);
            }
        },


        enableItem: function (lielement) {
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) return;
            current.removeClass("e-disable").attr("aria-disabled", false).removeClass('e-disable');
            this._removeOverlay(current);
        },


        disableItemByID: function (liid) {
            var lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) return;
            this.disableItem(lielement);
        },

        enableItemByID: function (liid) {
            var lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) return;
            this.enableItem(lielement);
        },

        disable: function () {
            if (this.element.attr("aria-disabled") == "true") return false;
            this.element.attr("aria-disabled", true).removeAttr("aria-label");
            this._addOverlay(this.items);
            if (this.model.isResponsive && (this.model.orientation == "horizontal"))
                this._spantag.addClass("e-disable");
            this.model.enabled = false;
        },


        enable: function () {
            if (this.element.attr("aria-disabled") == "false") return false;
            this.items.removeClass("e-disable");
            this._removeOverlay(this.items);
            this.element.attr("aria-disabled", false);
            if (this.model.isResponsive && (this.model.orientation == "horizontal"))
                this._spantag.removeClass("e-disable");
            this.model.enabled = true;
        },


        selectItem: function (lielement) {
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) return;
            current.addClass("e-active").attr("aria-selected", true);
            this._activeItem = $(this.items).index(current);
        },


        deselectItem: function (lielement) {
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) return;
            current.removeClass("e-active").attr("aria-selected", false);
        },

        selectItemByID: function (liid) {
            var lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) return;
            this.selectItem(lielement);
        },


        deselectItemByID: function (liid) {
            var lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) return;
            this.deselectItem(lielement);
        },


        removeItem: function (lielement) {
            var current = $(lielement);
            if ((current == null) || (current.length <= 0)) return;
            current.remove();
            this.items = this.itemsContainer.children('li');
        },


        removeItemByID: function (liid) {
            var lielement = this.itemsContainer.find("li#" + liid);
            if ((lielement == null) || (lielement.length <= 0)) return;
            this.removeItem(lielement);
        },
        _wireResizing: function () {
            (this.model.isResponsive && (this.model.orientation == "horizontal")) ? $(window).bind('resize', $.proxy(this._reSizeHandler, this)) : $(window).unbind('resize', $.proxy(this._reSizeHandler, this))
        },


        _wireEvents: function () {
            this._on(this.element, "mousedown", this._clickEventHandler);
            this._on(this.element, "focus", this._focusElement);
            this._on(this.element, "blur", this._targetBlur);
            this._on(this.items, "mouseenter", this._onItemHover);
            this._on(this.items, "mouseleave", this._onItemLeave);
            this._on(this.items, "mousedown", this._onItemClick);
            this._on(this.items, "mouseup", this._onItemClick);
        },

        _onItemHover: function (e) {
            var currentItem = $(e.currentTarget), targetItem = e.target;
            if (!currentItem.hasClass("e-disable")) {
                this.items.removeClass("e-hover");
                currentItem.addClass("e-hover");
                args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                this._trigger("itemHover", args);
            }
        },

        _onItemClick: function (e) {
            if ($(e.currentTarget).hasClass('e-disable')) return false;
            var currentItem = e.currentTarget, targetItem = e.target;
            if (e.type == "mousedown") {
                this._focusEnable = false;
                $(currentItem).addClass('e-active');
                this._currentItem = $(currentItem);
            }
            else if (e.type == "mouseup") {
                if (!$(currentItem).hasClass("e-disable")) {
                    args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled, event: e };
                    if (this.model.isResponsive && this._contstatus && $(currentItem).closest('.e-responsive-toolbar').length > 0)
                        this._activeItem = this._liTemplte.find('.e-tooltxt').index(currentItem);
                    else
                        this._activeItem = $(this.items).index(currentItem);
                    this._trigger("click", args);
                }
            }
        },

        _onItemLeave: function (e) {
            var currentItem = $(e.currentTarget), targetItem = e.target;
            if (!currentItem.hasClass("e-disable")) {
                this._removeSelection();
                currentItem.removeClass("e-hover");
                args = { currentTarget: currentItem, target: targetItem, status: this.model.enabled };
                this._trigger("itemLeave", args);
            }
        },

        _OnKeyDown: function (e) {
            this._focusEnable = true;
            this.e = e;
            var itemsLength, action = false;
            if (this.model.isResponsive && this._liTemplte.find('.e-tooltxt').length > 0) {
                this._keyPressed = true;
                this._items = (this._contstatus) ? this._liTemplte.find('.e-tooltxt') : this.element.find('.e-tooltxt');
                itemsLength = this._items.length - 1;
            } else {
                itemsLength = this.items.length - 1;
                this._keyPressed = false;
            }
            switch (e.keyCode) {
                case 33:
                case 36:
                case 38:
                case 39:
                    e.preventDefault();
                    if (e.keyCode == 33 || e.keyCode == 36)
                        this._activeItem = 0;
                    else {
                        if (this._activeItem > itemsLength - 1)
                            this._activeItem = 0;
                        else
                            this._activeItem += 1;
                    }
                    var _currentItem = (this._activeItem == 0) ? itemsLength : (this._activeItem - 1);
                    for (var i = this._activeItem; i != _currentItem; i++) {
                        if (this._getActiveItem().hasClass('e-disable') || this._getActiveItem().is(':hidden')) {
                            if (this._activeItem == itemsLength)
                                this._activeItem = 0;
                            else
                                this._activeItem += 1;
                        }
                        else
                            break;
                    }
                    action = true;
                    break;
                case 34:
                case 35:
                case 37:
                case 40:
                    e.preventDefault();
                    if (e.keyCode == 34 || e.keyCode == 35)
                        this._activeItem = itemsLength;
                    else {
                        if (this._activeItem <= 0)
                            this._activeItem = itemsLength;
                        else
                            this._activeItem -= 1;
                    }
                    var _currentItem = (this._activeItem == itemsLength) ? 0 : (this._activeItem + 1);
                    for (var i = this._activeItem; i != _currentItem; i--) {
                        if (this._getActiveItem().hasClass('e-disable') || this._getActiveItem().is(':hidden')) {
                            if (this._activeItem == 0)
                                this._activeItem = itemsLength;
                            else
                                this._activeItem -= 1;
                        }
                        else
                            break;
                    }
                    action = true;
                    break;
                case 13:
                case 32:
                    break;
            }
            if (action) {
                this._removeListHover();
                this._addListHover();
            }
        },

        _OnKeyUp: function (e) {
            this.e = e;
            var itemsLength = this.items.length;
            switch (e.keyCode) {
                case 33:
                case 35:
                case 34:
                case 36:
                case 37:
                case 38:
                case 39:
                case 40:
                    break;
                case 13:
                case 32:
                    if ($(e.target).is("div"))
                        var currentItem = this._getActiveItem(), targetItem = currentItem;
                    else
                        var currentItem = $(e.target).closest("li"), targetItem = e.target;
                    if (!$(currentItem).hasClass("e-disable")) {
                        args = { currentTarget: currentItem[0], target: $(targetItem)[0], status: this.model.enabled, event: this.e };
                        this._trigger("click", args);
                        this._removeListHover();
                    }
                    break;
                case 27:
                    this.element.focusout();
                    this._removeListHover();
                    break;
                case 9:
                    this._activeItem = 0;
                    if (this._getActiveItem().hasClass('e-disable') || this._getActiveItem().is(':hidden')) {
                        this._activeItem = $(this.items).filter('li:not([class*="e-disable"])').first().index();
                    }
                    this._addListHover();
                    break;
            }
        },

        _removeListHover: function () {
            $(this.items).removeClass("e-hover");
        },

        _addListHover: function () {
            var activeItem = this._getActiveItem();
            if (!activeItem.hasClass('e-disable'))
                activeItem.addClass("e-hover").focus();
        },

        _getActiveItem: function () {
            if (this.model.isResponsive && this._keyPressed)
                return $(this._items[this._activeItem]);
            else
                return $(this.items[this._activeItem]);
        },

        _targetBlur: function (e) {
            e.preventDefault();
            this.element.focusout().removeClass("e-focus");
            this._removeListHover();
            this._off(this.element, "keyup", this._OnKeyUp);
            this._off(this.element, "keydown", this._OnKeyDown);
            this._trigger("focusOut");
            (this._contstatus) && this._focusElement(); // rebind the keyevents when responsive
        },

        _clickEventHandler: function (e) {
            this._clicked = true;
        },

        _removeSelection: function() {
            (this._currentItem && this._currentItem.attr("aria-selected") != "true") && this._currentItem.removeClass('e-active');
        },

        _focusElement: function (e) {
            this.element.addClass("e-focus");
            this._on(this.element, "keyup", this._OnKeyUp);
            this._on(this.element, "keydown", this._OnKeyDown);
            if (!this._focusEnable) {
                this._focusEnable = true;
                return;
            }
            this._removeListHover();
            this._activeItem = (this._clicked) ? -1 : 0;
            if (this._getActiveItem().hasClass('e-disable') || this._getActiveItem().is(':hidden'))
                this._activeItem = $(this.items).filter('li:not([class*="e-disable"])').first().index();
            this._addListHover();
        },
    });

})(jQuery, Syncfusion);


;
/**
* @fileOverview Plugin to style the Html Accordion elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejAccordion", "ej.Accordion", {
        _rootCSS: "e-acrdn",

        element: null,

        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        angular: {
            terminal: false
        },

        defaults: {

            collapsible: false,

            ajaxSettings: {

                type: 'GET',

                cache: false,

                data: {},

                dataType: "html",

                contentType: "html",

                async: true
            },

            events: "click",

            customIcon: {

                header: "e-collaps",

                selectedHeader: "e-expand"
            },

            heightAdjustMode: "content",

            height: null,

            width: null,

            headerSize: null,

            enableAnimation: true,

            selectedItemIndex: 0,

            cssClass: "",

            enableRTL: false,

            showRoundedCorner: false,

            allowKeyboardNavigation: true,

            enableMultipleOpen: false,

            expandSpeed: 300,

            collapseSpeed: 300,

            selectedItems: [],

            disabledItems: [],

            enabledItems: [],

            enablePersistence: false,

            enabled: true,

            htmlAttributes: {},

            ajaxLoad: null,

            ajaxBeforeLoad: null,

            activate: null,

            beforeActivate: null,

            inActivate: null,

            beforeInactivate: null,

            ajaxSuccess: null,

            ajaxError: null,

            create: null,

            destroy: null

        },

        dataTypes: {
            cssClass: "string",
            collapsible: "boolean",
            enabled: "boolean",
            events: "string",
            heightAdjustMode: "enum",
            ajaxSettings: "data",
            customIcon: "data",
            selectedItems: "data",
            disabledItems: "data",
            enableAnimation: "boolean",
            htmlAttributes: "data",
        },


        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "events": {
                        this._off(this._headers, this.model.events);
                        this._on(this._headers, options[key], this._headerEventHandler);
                        break;
                    }
                    case "disabledItems": this.disableItems(options[key]); break;
                    case "enabledItems": this.enableItems(options[key]); break;
                    case "enabled": this._enabledAction(options[key]); break;
                    case "selectedItemIndex": (!this.model.enableMultipleOpen) && this._activeTab(options[key]); break;
                    case "heightAdjustMode": this._setHeightStyle(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "showRoundedCorner": this._roundedCorner(options[key]); break;
                    case "customIcon":
                        {
                            var icons = this.model.customIcon;
                            var newIcons = options[key];
                            this._headers.find("span." + icons.header).removeClass(icons.header).addClass(newIcons.header);
                            this._headers.find("span." + icons.selectedHeader).removeClass(icons.selectedHeader).addClass(newIcons.selectedHeader);
                            break;
                        }
                    case "height": {
                        this.element.height(options[key]);
                        this.model.height = options[key];
                        if (this.model.heightAdjustMode != "fill") {
                            if (this._getHeight() > this.element.height())
                                this._scrollerObj ? this._scrollerObj.option({ height: this.element.height() }) : this._setScoller();
                        }
                        else this._setHeight();
                        break;
                    }
                    case "width": {
                        this.element.width(options[key]);
                        if (this._scrollerObj) this._scrollerObj.option("width", options[key]);
                        if (options[key].toString().indexOf("%"))
                            $(window).bind('resize', $.proxy(this._scrollerRefresh, this));
                        else $(window).unbind('resize', $.proxy(this._scrollerRefresh, this));
                        if (this._scrollerObj) this._scrollerObj.refresh();
                        break;
                    }
                    case "headerSize": {
                        this.model.headerSize = options[key];
                        this._setHeaderSize();
                        break;
                    }
                    case "allowKeyboardNavigation": {
                        if (options[key]) {
                            this._off(this.element, "keydown");
                            this._on(this.element, 'keydown', this._keyPress);
                        }
                        else
                            this._off(this.element, "keydown");
                        break;
                    }
                    case "enableRTL":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            if (this._scrollerObj) this._scrollerObj.option("enableRTL", options[key]);
                            break;
                        }
                    case "selectedItems": this._showHideSelectedItems(options[key]); break;
                    case "expandSpeed": {
                        this.model.expandSpeed = parseInt(this.model.expandSpeed);
                    }
                    case "collapseSpeed": {
                        this.model.collapseSpeed = parseInt(this.model.collapseSpeed);
                    }
                    case "htmlAttributes": this._addAttr(options[key]); break;
                }
            }
        },

        _destroy: function () {

            this._removeBaseClass();
            this.element.children('.e-disable').removeClass("e-disable");
            this.element.removeClass("e-acrdn e-corner");
            this._headers.removeClass("e-active  e-state-disabled");
            this._contentPanels.removeClass("e-content-active e-disable");
            $(this._headers.find(".e-icon")).remove();
            if (this.model.height) this._contentPanels.height("auto");
            if (this.model.width) this._contentPanels.width("auto");
            if (this._scrollerObj) this._scrollerObj.destroy();
            this._unWireEvents();
        },

        _init: function () {
            this._renderControl();
            this._prevSize = this._getDimension($(this.element).parent(), "height");
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else proxy.element.attr(key, value);
            });
        },

        _renderControl: function () {
            this._headers = this.element.find("> :even");
            this.element.attr("tabindex", 0).attr("role", "tablist");
            this._contentPanels = this._headers.next();
            //Index check negative values
            this._nagativeIndexCheck(this.model.selectedItemIndex);

            this._addBaseClass();
            this._addAttr(this.model.htmlAttributes);
			// Add header icons
            ej.buildTag("span.e-icon " + this.model.customIcon.header).prependTo(this._headers);
            this._setHeightStyle(this.model.heightAdjustMode);
            if (this.model.height) this.element.css("height", this.model.height);
            this.model.width && this.element.css("width", this.model.width);
            (!this.model.enableMultipleOpen) && this._renderHeaderIcon();
            if (!ej.isNullOrUndefined(this.model.headerSize)) this._setHeaderSize();
            this._wireEvents();
            this._roundedCorner(this.model.showRoundedCorner);
            this._enabledAction(this.model.enabled);
            //Activate the header & content               
            this._selectedItemsAction(this.model.enableMultipleOpen ? this.model.selectedItems : this.model.selectedItemIndex);
            this._disableItems(this.model.disabledItems);
            this._setEnabledItems();
        },
        _setHeight: function () {
            var proxy = this, headersHeight = 0, activeHeaderHeight = 0;
            this.element.css("height", this.model.height);
            maxHeight = this.element.height();
            this._contentPanels.css("height", "auto");
            if (this._scrollerObj) this._scrollerObj.option("height", this.element.height());
            this._headers.each(function (i) {
                maxHeight -= $(this).outerHeight();
                headersHeight += $(this).outerHeight();
                if ($(proxy._contentPanels[i]).hasClass("e-acrdn-content-active"))
                    activeHeaderHeight += $(proxy._contentPanels[i]).outerHeight();
            });
            maxPadding = 0;
            var padding = this.element.height() - (activeHeaderHeight + headersHeight);
            if (padding > 0)
                this._contentPanels.each(function () {
                    $(this).height($(this).height() + padding / proxy.element.find(".e-select.e-active").length);
                });
            else this._setScoller();
            if (this._scrollerObj) this._scrollerObj.refresh();
        },
        _setScoller: function () {
            if (this.element.parent(".e-acrdn-scroller.e-scroller").length < 1) {
                this.scroller = ej.buildTag("div.e-acrdn-scroller");
                this.element.wrap(this.scroller);
                var width = this.element.css("width");
                this.element.parent(".e-acrdn-scroller").ejScroller({ "height": this.element.height(), "width": this.model.width });
                this.element.css("width", width);
                this._scrollerObj = this.element.parent(".e-js").data("ejScroller");
                if (this.model.showRoundedCorner) this.element.parent(".e-acrdn-scroller").addClass("e-corner");
                if (this.model.enableRTL) this._scrollerObj.option("enableRTL", true);
            }
        },
        _setHeaderSize: function () {
            var proxy = this;
            this._headers.each(function () {
                $(this).css({ "padding-top": 0, "padding-bottom": 0 });
                $(this).css("height", proxy.model.headerSize);
                $($(this).find("a[href]")).css("line-height", proxy.model.headerSize);
                if ($(this).find(".e-icon").length > 0) {
                    $(this).find(".e-icon").css("margin-top", ($(this).height() / 2 - 5));
                }
            });
            if (this.model.height) {
                if (this.model.heightAdjustMode != "fill") {
                    if (this._getHeight() > this.element.height())
                        this._scrollerObj ? this._scrollerObj.refresh() : this._setScoller();
                }
                else this._setHeight();
                if (this._scrollerObj) this._scrollerObj.refresh();
            }
        },
        _getHeight: function () {
            var headersHeight = 0, activeHeaderHeight = 0;
            var proxy = this;
            this._headers.each(function (i) {
                headersHeight += $(this).outerHeight();
                if ($(proxy._contentPanels[i]).hasClass("e-acrdn-content-active"))
                    activeHeaderHeight += $(proxy._contentPanels[i]).outerHeight();
            });
            return headersHeight + activeHeaderHeight;
        },
        _setEnabledItems: function () {
            for (var index = 0; index < this.getItemsCount() ; index++) {
                if (($.inArray(index, this.model.disabledItems) < 0) && ($.inArray(index, this.model.enabledItems) < 0))
                    this.model.enabledItems.push(index);
            }
        },


        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },


        _showHideSelectedItems: function (activeList) {
            if (this.model.enableMultipleOpen && activeList instanceof Array) {
                for (var index = 0; index < activeList.length; index++) {
                    if (activeList[index] >= 0 && this._headers.eq(activeList[index]).attr('aria-selected') != 'true')
                        this._activeTab(activeList[index], true);
                }
                var len = this.model.selectedItems.length, temp = JSON.parse(JSON.stringify(this.model.selectedItems));
                for (var index = 0; index < len; index++) {
                    if (activeList.indexOf(temp[index]) == -1)
                        this._hideTab(temp[index]);
                }
            }
        },

        _selectedItemsAction: function (activeList) {
            if (this.model.enableMultipleOpen && activeList instanceof Array && activeList.length > 0) {
                for (var seletedindex = 0; seletedindex < activeList.length; seletedindex++) {
                    this._activeTab(activeList[seletedindex], true);
                }
            }
            else
                if (!(this.collapseALL && this.model.collapsible))
                    this._activeTab(this.model.selectedItemIndex, true);
        },

        _enabledAction: function (flag) {
            if (!flag)
                this.disable();
            else
                this.enable();
        },


        _addBaseClass: function () {
            this.element.addClass(this.model.cssClass + " e-widget");
            if (this.model.enableRTL) this.element.addClass("e-rtl");
            this._headers.addClass(" e-select").attr("role", "tab").attr("tabindex", 0);
            this._contentPanels.addClass("e-content").attr("role", "tabpanel").attr("aria-hidden", true).hide(); // hide all the content panels while loding
            if (!this.model.enableMultipleOpen)
                $(this._contentPanels[this.model.selectedItemIndex]).show();
        },


        _removeBaseClass: function () {
            this._headers.removeClass(" e-select");
            this._contentPanels.removeClass("e-content").show(); // hide all the content panels while loding
        },

        _roundedCorner: function (value) {
            if (value) {
                this.element.addClass('e-corner');
            }
            else if (this.element.hasClass('e-corner')) {
                this.element.removeClass('e-corner');
            }
        },

        _nagativeIndexCheck: function (index) {

            var panels;
            if (index < 0 && this.model.collapsible) {
                this.collapseALL = true;
                panels = this.getItemsCount();
                for (var pane = 0; pane < panels; pane++) {
                    this._hideTab(pane);
                }
            }
            else if (index < 0 || index >= this._headers.length)
                this.model.selectedItemIndex = 0;
        },

        _activeTab: function (index, raiseEvent) {

            if ($.inArray(index, this.model.disabledItems) < 0) {
                var data = {
                    activeIndex: index
                };

                if (true === (!raiseEvent && this._trigger("beforeActivate", data))) {
                    return false;
                }
                if (!this.model.enableMultipleOpen)
                    this._hideTab(this.model.selectedItemIndex, raiseEvent);
                this.model.selectedItemIndex = index;
                var icons = this.model.customIcon;
                this._headers.attr("tabindex", 0).removeAttr("aria-expanded");
                this._activeHeader = this._headers.eq(index).addClass("e-active ").attr("aria-selected", true).attr("tabindex", 0).attr("aria-expanded", true);
                this._activeHeader.find(".e-icon")
                        .removeClass(icons.header)
                        .addClass(icons.selectedHeader);
                this._addSelectedItems(index);
                this._ajaxLoad();
                this._roundedCorner(this.model.showRoundedCorner);
                data = {
                    activeHeader: this._activeHeader,
                    activeIndex: this.model.selectedItemIndex
                };
                var proxy = this;
                this._activeHeader.next().addClass("e-acrdn-content-active").removeAttr("aria-hidden", false);
                this._paneAdjustSize();
                this._activeContent = this._activeHeader.next().slideDown(this.model.enableAnimation ? this.model.expandSpeed : 0, "easeOutQuad", function () {

                    if (!raiseEvent) proxy._trigger("activate", data);
                    if (proxy.model.height) {
                        if (proxy.model.heightAdjustMode != "fill") {
                            if (proxy._getHeight() > proxy.element.height())
                                proxy._scrollerObj ? proxy._scrollerObj.refresh() : proxy._setScoller();
                        }
                        else proxy._setHeight();
                        if (proxy._scrollerObj) proxy._scrollerObj.refresh();
                    }
                });
            }
        },
        _paneAdjustSize: function () {
            var proxy = this;
            if (this.model.heightAdjustMode === "fill") {
                if (this.model.enableMultipleOpen) {
                    maxHeight = this._getDimension($(this.element).parent(), "height");
                    $(this.element).parent().css({ "overflow": "auto" });
                    this._headers.each(function () {
                        maxHeight -= proxy._getDimension($(this), "outerHeight");
                    });
                    var maxPadding = 0;
                    maxPadding = Math.max(maxPadding, proxy._getDimension($(this._activeHeader.next()), "outerHeight") - proxy._getDimension($(this._activeHeader.next()), "height"));
                    maxHeight = (maxHeight / this.model.selectedItems.length) - maxPadding;
                    this._headers.next().animate({ "height": maxHeight }, 300);
                }
            }
        },


        _hideTab: function (index, raiseEvent) {
            var icons = this.model.customIcon, data;
            if (index >= 0) {
                data = {
                    inActiveIndex: index
                };

                if (true === (!raiseEvent && this._trigger("beforeInactivate", data))) {
                    return false;
                }
                this._activeHeader = this._headers.eq(index).removeClass("e-active ").removeAttr("aria-selected").attr("tabindex", 0).attr("aria-expanded", false);
                this._activeHeader.find(".e-icon")
                        .removeClass(icons.selectedHeader)
                        .addClass(icons.header);
                data = {
                    inActiveHeader: this._activeHeader,
                    inActiveIndex: index
                };
                var proxy = this; proxy.raiseEvent=raiseEvent;
                this._activeHeader.next().removeClass("e-acrdn-content-active").attr("aria-hidden", true).slideUp(this.model.enableAnimation ? this.model.collapseSpeed : 0, "easeOutQuad", function () {
                    if(!proxy.raiseEvent) proxy._trigger("inActivate", data);
                    if (proxy.model.height) {
                        if (proxy.model.heightAdjustMode != "fill") {
                            if (proxy._getHeight() > proxy.element.height())
                                proxy._scrollerObj ? proxy._scrollerObj.refresh() : proxy._setScoller();
                        }
                        else proxy._setHeight();
                    }
                    if (proxy._scrollerObj) proxy._scrollerObj.refresh();
                    proxy._paneAdjustSize();
                });
                this._deleteSelectedItems(index);
            }
        },


        _renderHeaderIcon: function () {
            if (this._headers.find(".e-icon").length <= 0) {
                var icons = this.model.customIcon;
                if (icons) {
                    if (!(this.collapseALL && this.model.collapsible)) {
                        this._headers.eq(this.model.selectedItemIndex).find(".e-icon")
                        .removeClass(icons.header)
                        .addClass(icons.selectedHeader);
                    }
                }
            }
        },


        _ajaxLoad: function () {
            var anchor = this._activeHeader.find("a[href]");
            if (anchor.length > 0) {
                var content = $(this._contentPanels[this.model.selectedItemIndex]);
                var href = $(anchor).attr("href");
                if (href && href !== "#")
                    this._sendAjaxOptions(content, anchor[0]);
            }
        },


        _sendAjaxOptions: function (content, link) {


            if (true === (this._trigger("ajaxBeforeLoad", { url: link })))
                return false;
            content.addClass("e-load");
            var proxy = this;
            var curTabTitle = $(link).html();
            var _hrefLink = link.href.replace("#", "");
            var _ajaxOptions = {
                type: this.model.ajaxSettings.type, cache: this.model.ajaxSettings.cache, url: _hrefLink, data: this.model.ajaxSettings.data,
                dataType: this.model.ajaxSettings.dataType, contentType: this.model.ajaxSettings.contentType, async: this.model.ajaxSettings.async,
                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTabTitle);
                    } catch (e) {

                    }
                }, "error": function () {
                    try {
                        proxy._ajaxErrorHandler(data, link, proxy.model.selectedItemIndex, curTabTitle);
                    } catch (e) {

                    }
                }
            };
            this._sendAjaxRequest(_ajaxOptions);
        },


        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },


        _ajaxSuccessHandler: function (data, content, link, curTabTitle) {
            if (curTabTitle != null)
                $(link).html(curTabTitle);
            content.removeClass("e-load");
            content.html(data).addClass("e-acrdn-loaded"); //to indicate the content is already loaded
            var eventData = { data: data, url: link, content: content };
            this._trigger("ajaxSuccess", eventData);

            this._trigger("ajaxLoad", { url: link });
        },


        _ajaxErrorHandler: function (data, link, index, title) {
            this._trigger("ajaxError", { data: data, url: link });

            this._trigger("ajaxLoad", { url: link });
        },


        _setHeightStyle: function (heightFormat) {
            var proxy = this, maxHeight;
            if (ej.Accordion.HeightAdjustMode.Fill == heightFormat) {
                if (this.model.height != null) this._setHeight();
                else {
                    maxHeight = this._getDimension($(this.element).parent(), "height");
                    $(this.element).parent().css({ "overflow": "auto" });
                    proxy._headers.each(function () {
                        maxHeight -= proxy._getDimension($(this), "outerHeight");
                    });
                    var maxPadding = 0;
                    proxy._headers.next().each(function () {
                        maxPadding = Math.max(maxPadding, proxy._getDimension($(this), "outerHeight") - proxy._getDimension($(this), "height"));
                    }).height(maxHeight - maxPadding).css({ "overflow": "auto" });
                }
            } else if (ej.Accordion.HeightAdjustMode.Auto == heightFormat) {
                maxHeight = 0;
                proxy._headers.next().each(function () {
                    maxHeight = Math.max(maxHeight, proxy._getDimension($(this), "outerHeight"));
                }).height(maxHeight);
                this.maxAutoHeight = maxHeight;
            }
            if (ej.Accordion.HeightAdjustMode.Fill != heightFormat)
                if (this.model.height && this._getHeight() > this.element.height()) this._setScoller();
        },

        _getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            var prop = { visibility: 'hidden', display: 'block' };
            var tmp = [];
            $hidden.each(function () {
                var temp = {}, name;
                for (name in prop) {
                    temp[name] = this.style[name];
                    this.style[name] = prop[name];
                }
                tmp.push(temp);
            });
            value = /(outer)/g.test(method) ?
            $(element)[method](true) :
           $(element)[method]();

            $hidden.each(function (i) {
                var temp = tmp[i], name;
                for (name in prop) {
                    this.style[name] = temp[name];
                }
            });

            return value;
        },


        _headerEventHandler: function (event) {
            if (this.model.enabled) {
                event.preventDefault();
                var clicked = $(event.currentTarget);
                var index = this._headers.index(clicked);
                if (this.model.enableMultipleOpen && this._headers.eq(index).hasClass("e-active")) {
                    this._hideTab(index);
                }
                else if (this.model.selectedItemIndex == index && this.model.collapsible) {
                    this._hideTab(this.model.selectedItemIndex);
                    this.model.selectedItemIndex = -1
                }
                else if (!this._headers.eq(index).hasClass("e-active")) {
                    this._activeTab(index);
                }
            }
        },

        _addSelectedItems: function (index) {
            if ($.inArray(index, this.model.selectedItems) < 0)
                this.model.selectedItems.push(index);
        },

        _deleteSelectedItems: function (index) {
            if ($.inArray(index, this.model.selectedItems) > -1) {
                position = $.inArray(index, this.model.selectedItems);
                this.model.selectedItems.splice(position, 1);
            }
        },

        _deleteDisabledItems: function (index) {
            if ($.inArray(index, this.model.disabledItems) > -1) {
                position = $.inArray(index, this.model.disabledItems);
                this.model.disabledItems.splice(position, 1);
            }
        },
        _addEnabledItems: function (index) {
            if ($.inArray(index, this.model.enabledItems) < 0)
                this.model.enabledItems.push(index);
        },
        _deleteEnabledItems: function (index) {
            if ($.inArray(index, this.model.enabledItems) > -1) {
                position = $.inArray(index, this.model.enabledItems);
                this.model.enabledItems.splice(position, 1);
            }
        },

        _keyPress: function (e) {
            if (this.model.enabled) {
                var index, currentEle, targetEle = $(e.target);
                if ((targetEle.hasClass("e-select")) || (targetEle.hasClass("e-acrdn"))) {
                    index = this.model.selectedItemIndex;
                }

                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
                if ((targetEle.hasClass("e-select")) || (targetEle.hasClass("e-acrdn"))) {
                    switch (code) {

                        case 39:
                        case 40:
                            e.preventDefault();
                            index = index == (this.getItemsCount() - 1) ? 0 : index + 1;
                            this._activeTab(index);
                            currentEle = $(this._headers[index]);
                            break;

                        case 38:
                        case 37:
                            e.preventDefault();
                            index = index == 0 ? (this.getItemsCount() - 1) : index - 1;
                            this._activeTab(index);
                            currentEle = $(this._headers[index]);
                            break;

                        case 35:
                            e.preventDefault();
                            currentEle = $(this._headers[this.getItemsCount() - 1]);
                            this._activeTab((this.getItemsCount() - 1));
                            break;

                        case 36:
                            e.preventDefault();
                            currentEle = $(this._headers[0]);
                            this._activeTab(0);
                            break;
                        case 32:
                        case 13:
                            e.preventDefault();
                            $(this._headers[index]).hasClass("e-active") && (this.model.enableMultipleOpen || this.model.collapsible) ? this._hideTab(index) : this._activeTab(index);
                            break;
                    }
                }
                else if (e.ctrlKey && !targetEle.hasClass("e-acrdn")) {
                    switch (code) {
                        case 38:
                            e.preventDefault();
                            index = this._contentPanels.index(targetEle.parent(".e-content"));
                            currentEle = $(this._headers[index]);
                            this._activeTab(index);
                            break;
                        case 33:
                            e.preventDefault();
                            currentEle = $(this._headers[0]);
                            this._activeTab(0);
                            break;
                        case 34:
                            e.preventDefault();
                            currentEle = $(this._headers[this.getItemsCount() - 1]);
                            this._activeTab((this.getItemsCount() - 1));
                            break;
                    }
                }
                if (!ej.isNullOrUndefined(currentEle)) {
                    currentEle.addClass("e-focus");
                    currentEle.focus();
                }
            }
        },


        _wireEvents: function () {

            this._on(this._headers, this.model.events, this._headerEventHandler);
            if (this.model.allowKeyboardNavigation) this._on(this.element, 'keydown', this._keyPress);
            $(window).bind('resize', $.proxy(this._contentPaneSize, this));
            if (this.model.height && this.model.width && this.model.width.toString().indexOf("%") != -1)
                $(window).bind('resize', $.proxy(this._scrollerRefresh, this));
        },
        _scrollerRefresh: function (e) {
            if (this.model.height) {
                if (this.model.heightAdjustMode != "fill") {
                    if (this._getHeight() > this.model.height)
                        this._scrollerObj ? this._scrollerObj.refresh() : this._setScoller();
                }
                else this._setHeight();
            }
        },


        _unWireEvents: function () {
            this._off(this._headers, this.model.events);
            $(window).unbind('resize', $.proxy(this._contentPaneSize, this));
            if (this.model.allowKeyboardNavigation) {
                this._off(this.element, 'keydown');
            }
        },

        _contentPaneSize: function () {
            var maxHeight = this._getDimension($(this.element).parent(), "height");
            if (this._prevSize == maxHeight) return;
            if (ej.Accordion.HeightAdjustMode.Fill == this.model.heightAdjustMode) {
                if (this.model.enableMultipleOpen) this._paneAdjustSize()
                else this._setHeightStyle(this.model.heightAdjustMode);
            }
            this._prevSize = maxHeight;
        },
        _disableItems: function (itemIndexes) {
            if ($.isArray(itemIndexes)) {
                for (var i = 0; i < itemIndexes.length; i++) {
                    $(this._headers[itemIndexes[i]]).addClass("e-disable");
                    $(this._contentPanels[itemIndexes[i]]).addClass("e-disable");
                    $(this._headers[itemIndexes[i]]).unbind(this.model.events);
                    if (this.model.disabledItems.indexOf(itemIndexes[i]) == -1) this.model.disabledItems.push(itemIndexes[i]);
                    this._hideTab(itemIndexes[i], true);
                    this._deleteEnabledItems(itemIndexes[i]);

                }
            }
            else {
                $(this._headers[itemIndexes]).addClass("e-disable");
                $(this._contentPanels[itemIndexes]).addClass("e-disable");
                $(this._headers[itemIndexes]).unbind(this.model.events);
                if (this.model.disabledItems.indexOf(itemIndexes[i]) == -1) this.model.disabledItems.push(itemIndexes);
                this._hideTab(itemIndexes, true);
                this._deleteEnabledItems(itemIndexes[i]);
            }
            if (!this.model.enableMultipleOpen) {
                var index, previous = this.model.selectedItemIndex;
                for (i = 0; i < this.model.disabledItems.length; i++) {
                    index = this.model.selectedItemIndex;
                    if ($.inArray(index, this.model.disabledItems) >= 0) {
                        index++;
                        this.model.selectedItemIndex = index;
                        //this._selectedItemsAction(index);
                    }
                }
                if (previous !== this.model.selectedItemIndex)
                    this._selectedItemsAction(index);
            }
        },
        enableItems: function (itemIndexes) {
            if (!this.model.enabled) return false;
            if ($.isArray(itemIndexes)) {
                for (var i = 0; i < itemIndexes.length; i++) {
                    $(this._headers[itemIndexes[i]]).removeClass("e-disable");
                    $(this._contentPanels[itemIndexes[i]]).removeClass("e-disable");
                    this._on($(this._headers[itemIndexes[i]]), this.model.events, this._headerEventHandler);
                    if (this.model.enabledItems.indexOf(itemIndexes[i]) == -1) this.model.enabledItems.push(itemIndexes[i]);
                    this._deleteDisabledItems(itemIndexes[i]);
                }
            }
            else {
                $(this._headers[itemIndexes]).removeClass("e-disable");
                $(this._contentPanels[itemIndexes]).removeClass("e-disable");
                this._on($(this._headers[itemIndexes]), this.model.events, this._headerEventHandler);
                if (this.model.enabledItems.indexOf(itemIndexes[i]) == -1) this.model.enabledItems.push(itemIndexes[i]);
                this._deleteDisabledItems(itemIndexes);
            }
        },
        disableItems: function (itemIndexes) {
            if (!this.model.enabled) return false;
            this._disableItems(itemIndexes);
        },

        addItem: function (header_name, content, index, isAjaxReq) {
            var contentTag, headerTag, anchorTag;
            if (ej.isNullOrUndefined(isAjaxReq)) isAjaxReq = false;
            if (ej.isNullOrUndefined(index)) this._addItemIndex = this._headers.length;
            else if (index >= 0 && index <= this._headers.length) this._addItemIndex = index;
            else return false;
            for (var disable_index = 0; disable_index < this.model.disabledItems.length; disable_index++) {
                if (this.model.disabledItems[disable_index] >= this._addItemIndex)
                    this.model.disabledItems[disable_index]++;
            }
            var headerTag = ej.buildTag('h3').addClass("e-select").attr("role", "tab").attr("tabindex", 0);
            headerTag.insertAfter($(this._headers[this._addItemIndex - 1]).next());
            var anchorTag = ej.buildTag("a").attr("href", isAjaxReq ? content : "#").text(header_name);
            anchorTag.appendTo(headerTag);
            if (isAjaxReq) content = "";
            if ($(content).length > 0) contentTag = $(content);
            else contentTag = ej.buildTag('div').text(content);
            contentTag.addClass("e-content").attr("role", "tabpanel").attr("aria-hidden", true).hide();
            contentTag.insertAfter(headerTag);
            this._headers = this.element.find("> :even");
            this._contentPanels = this._headers.next();
            ej.buildTag("span.e-icon " + this.model.customIcon.header).prependTo(headerTag);
            this._selectedItemsAction(this.model.selectedItems);
            this.model.width = this.model.width || this.element.css("width");
            if (this.model.height) this.element.css("height", this.model.height);
            this._setHeightStyle(this.model.heightAdjustMode);
            if (this.model.enableRTL) {
                headerTag.addClass("e-rtl");
                contentTag.addClass("e-rtl");
            }
            (!this.model.enableMultipleOpen) && this._renderHeaderIcon();
            if (!ej.isNullOrUndefined(this.model.headerSize)) this._setHeaderSize();
            this._on(headerTag, this.model.events, this._headerEventHandler);
            this._roundedCorner(this.model.showRoundedCorner);

            this._enabledAction(this.model.enabled);
            this.disableItems(this.model.disabledItems)
            this._setEnabledItems();
        },

        removeItem: function (index) {
            if (!this.model.enabled) return false;
            if (index != null && index > -1 && index < this._headers.length) {
                var removedTab = $(this._headers[index]).remove();
                for (var disable_index = 0; disable_index < this.model.disabledItems.length; disable_index++) {
                    if (this.model.disabledItems[disable_index] > index)
                        this.model.disabledItems[disable_index]--;
                    else if (this.model.disabledItems[disable_index] == index) {
                        this.model.disabledItems.splice(disable_index, 1);
                    }
                }
                $(this.element.find(">div.e-content")[index]).remove();
                this._contentPanels.splice(index, 1);
                this._headers.splice(index, 1);
                this._unWireEvents();
                this._wireEvents();
                this._enabledAction(this.model.enabled);
                this.disableItems(this.model.disabledItems);
                this._selectedItemsAction(this.model.selectedItems);
                this._setEnabledItems();
            }
        },

        collapseAll: function () {
            if (this.model.collapsible) {
                this.model.selectedItemIndex = null;
                for (var index = 0; index < this.getItemsCount() ; index++)
                    this._hideTab(index);
            }
        },

        expandAll: function (val) {
            if (this.model.enableMultipleOpen) {
                for (var index = 0; index < this.getItemsCount() ; index++) {
                    this._activeTab(index);
                    this.model.selectedItemIndex = index;
                }
            }
        },


        disable: function () {
            for (var i = 0; i < this._headers.length; i++) {
                $(this._headers[i]).addClass("e-disable");
                $(this._contentPanels[i]).addClass("e-disable");
                this.model.disabledItems.push(i);
            }
            
            $(this.element.parent(".e-scroller")).addClass(".e-disable");
            if (this._scrollerObj) {
                var scroller = $(this.element.parent(".e-scroller")).find(".e-vscrollbar");
                if (scroller.length > 0) this._scrollerObj.disable();
            }
            this._unWireEvents();
        },

        enable: function () {
            for (var i = 0; i < this._headers.length; i++) {
                $(this._headers[i]).removeClass("e-disable");
                $(this._contentPanels[i]).removeClass("e-disable");
            }            
            this._setEnabledItems();
            if (this._scrollerObj) this._scrollerObj.enable();
            this._unWireEvents();
            this._wireEvents();
        },

        refresh: function () {
            this._contentPaneSize();
        },
        _selected: function (index) {
            if (!this.model.enabled) return false;
            if (index != null && this.model.selectedItemIndex != index) {
                this._activeTab(index);
            }
        },

        show: function () {
            if (!this.model.enabled) return false;
            this.element.css("visibility", "visible");
        },

        hide: function () {
            if (!this.model.enabled) return false;
            this.element.css("visibility", "hidden");
        },


        getItemsCount: function () {
            if (this._headers) {
                return this._headers.length;
            }
        }
    });


    ej.Accordion.HeightAdjustMode = {
		/**  Height fit to the content in the panel */
        Content: "content",
		/**  Height set to the largest content in the panel */
        Auto: "auto",
		/**  Height filled to the content of the panel */
        Fill: "fill"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Progressbar elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejProgressBar", "ej.ProgressBar", {

        element: null,

        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        _rootCSS: "e-progressbar",

        defaults: {

            text: "",

            cssClass: "",

            minValue: 0,

            maxValue: 100,

            value: 0,

            percentage: 0,

            height: null,

            htmlAttributes: {},

            width: null,

            enabled: true,

            enableRTL: false,

            showRoundedCorner: false,

            enablePersistence: false,

            start: null,

            complete: null,

            change: null,

            create: null,

            destroy: null

        },


        dataTypes: {
            cssClass: "string",
            minValue: "number",
            maxValue: "number",
            enabled: "boolean",
            enableRTL: "boolean",
            showRoundedCorner: "boolean",
            htmlAttributes: "data"
        },


        _setValue: function (value) {
            if (value == null) value = this.model.minValue;
            else if (typeof value === "string") value = parseFloat(value);
            if (this._isNumber(value)) this.model.value = value;
            else if (!this._isNumber(this.model.value)) this.model.value = this.model.minValue;
            this.model.value = this._validateRange(this.model.value, this.model.minValue, this.model.maxValue);
            this._setProgressValue();
        },

        _setPercent: function (percent) {
            this.initial = this.model.percentage;
            if (this.initial == 100) this.initial = 0;
            if (percent == null) percent = 0;
            else if (typeof percent === "string") percent = parseFloat(percent);
            if (this._isNumber(percent)) this.model.percentage = percent;
            else if (!this._isNumber(this.model.percentage)) this.model.percentage = 0;
            this.model.percentage = this._validateRange(this.model.percentage, 0, 100);
            this.model.value = this._percentToValue(this.model.percentage);
            this._increaseProgressWidth();
        },

        _validateMinMax: function () {
            if (isNaN(this.model.minValue)) this.model.minValue = 0;
            if (isNaN(this.model.maxValue)) this.model.maxValue = 100;
        },

        _setText: function (text) {
            if (text) {
                if (this.text) this.text.html(text);
                else {
                    this.text = ej.buildTag("div.e-progress-txt", text);
                    this.element.append(this.text);
                    this._setTop();
                }
            }
            else if (this.text) {
                this.text.remove();
                this.text = null;
            }
        },

        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },


        enable: function () {
            this.element.removeClass("e-disable");
            this.model.enabled = true;
        },

        disable: function () {
            this.element.addClass("e-disable");
            this.model.enabled = false;
        },

        getValue: function () {
            return this.model.value;
        },

        getPercentage: function () {
            return this.model.percentage;
        },


        _init: function () {
            this._initialize();
            this._render();
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "value":
                        this._setValue(options[option]);
                        options[option] = this.model.value; break;
                    case "percentage":
                        this._setPercent(options[option]);
                        options[option] = this.model.percentage; break;
                    case "minValue":
                        if (!isNaN(options[option])) this._minValidation(options[option]);
                        options[option] = this.model.minValue;
                    case "maxValue":
                        if (!isNaN(options[option])) this._maxValidation(options[option]);
                        options[option] = this.model.maxValue;
                    case "text": this._setText(options[option]); break;
                    case "height": this._setHeight(options[option]); if (this.text) this._setTop(); break;
                    case "width": this._setWidth(options[option]); break;
                    case "enabled": this._disabled(!options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "enableRTL": this._rtl(options[option]); break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }
        },

        _destroy: function () {
            this.element.empty();
            this.element.removeClass("e-widget e-box e-corner " + this.model.cssClass);
        },


        _initialize: function () {
            this.text = null;
            this.header = null;
            this._preVal = null;
        },


        _render: function () {
            this.initialRender = true;
            this.element.addClass("e-widget e-box " + this.model.cssClass).attr("role", "ProgressBar");
            this._setDimention();
            this.header = ej.buildTag("div.e-progress");
            this.element.append(this.header);
            this._setText(this.model.text);
            this._setInitialValue();
            this._addAttr(this.model.htmlAttributes);
            this._checkProperties();
            this._roundedCorner(this.model.showRoundedCorner);
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy._disabled(false);
                else proxy.element.attr(key, value)
            });
        },


        _setDimention: function () {
            this._setHeight(this.model.height);
            this._setWidth(this.model.width);
        },

        _setHeight: function (height) {
            if (height) this.element.height(height);
        },

        _setWidth: function (width) {
            if (width) this.element.css("width", width);
        },


        _setInitialValue: function () {
            this._validateMinMax();
            if (this.model.percentage) this._setPercent(this.model.percentage);
            else this._setValue(this.model.value);
        },

        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        _checkProperties: function () {
            if (this.model.enableRTL) this._rtl(this.model.enableRTL);
            this._minValidation(this.model.minValue);
            this._maxValidation(this.model.maxValue);
            if (!this.model.enabled) this._disabled(true);
        },

        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },

        _roundedCorner: function (value) {
            if (value && !this.element.hasClass('e-corner'))
                this.element.addClass('e-corner');
            else if (this.element.hasClass('e-corner'))
                this.element.removeClass('e-corner');
        },

        _minValidation: function (minvalue) {
            if (this.model.maxValue && this.model.maxValue < minvalue) this.model.maxValue = minvalue;
            if (this.model.value < minvalue)
                this.model.value = minvalue;
            this.model.minValue = minvalue;
            this._setProgressValue();
        },

        _maxValidation: function (maxValue) {
            if (this.model.minValue && this.model.minValue > maxValue) this.model.minValue = maxValue;
            if (this.model.value > maxValue)
                this.model.value = maxValue;
            this.model.maxValue = maxValue;
            this._setProgressValue();
        },


        _setTop: function () {
            var top = (this.element.height() - this.text.height()) / 2;
            this.text.css("top", Math.floor(top));
        },


        _increaseProgressWidth: function () {
            this.header.css("width", this.model.percentage + "%");
            if (this.initial == 0 && this.model.percentage != this.initial)
                this._raiseEvent("start");
            if (this._preVal != this.model.value) {
                this._preVal = this.model.value;
                $(this.header).attr("aria-label", this.model.percentage);
                if (this.header.hasClass("e-complete"))
                    this.header.removeClass("e-complete");
				this.initialRender ? this.initialRender = false : this._raiseEvent("change");
                if (this.model.percentage == 100) {
                    this.header.addClass("e-complete");
                    this._raiseEvent("complete");
                }
            }
        },

        _raiseEvent: function (event) {
            this._trigger(event, { value: this.model.value, percentage: this.model.percentage });
        },

        _setProgressValue: function () {
            this.initial = this.model.percentage;
            this.model.percentage = this._valueToPercent(this.model.value);
            this._increaseProgressWidth();
        },


        _isNumber: function (number) {
            return typeof number === "number" && !isNaN(number);
        },


        _validateRange: function (value, minvalue, maxValue) {
            if (value < minvalue) return minvalue;
            else if (value > maxValue) return maxValue;
            return value;
        },


        _valueToPercent: function (value) {
            if (this.model.maxValue <= this.model.minValue) return 100;
            value = this._validateRange(value, this.model.minValue, this.model.maxValue);
            value = (100 * (value - this.model.minValue)) / (this.model.maxValue - this.model.minValue);
            return value;
        },


        _percentToValue: function (percent) {
            if (this.model.maxValue <= this.model.minValue) { return this.model.minValue; }
            if (percent >= 0 && percent <= 100) {
                var diff = this.model.maxValue - this.model.minValue;
                var val = diff * percent / 100;
                percent = val + this.model.minValue;
            }
            else if (percent < 0) percent = this.model.minValue;
            else if (percent > 100) percent = this.model.maxValue;
            return percent;
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html input elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejRating", "ej.Rating", {
        _rootCSS: "e-rating",

        element: null,

        model: null,
        validTags: ["input"],
        _setFirst: false,
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },

        defaults: {

            maxValue: 5,

            minValue: 0,

            value: 1,

            allowReset: true,

            shapeWidth: 23,

            shapeHeight: 23,

            orientation: "horizontal",

            incrementStep: 1,

            readOnly: false,

            htmlAttributes: {},

            enabled: true,

            showTooltip: true,

            precision: "full",

            cssClass: "",

            width: null,

            height: null,

            enablePersistence: false,

            create: null,

            click: null,

            mouseover: null,

            mouseout: null,

            change: null,

            destroy: null
        },

        dataTypes: {
            maxValue: "number",
            minValue: "number",
            allowReset: "boolean",
            shapeWidth: "number",
            shapeHeight: "number",
            orientation: "enum",
            incrementStep: "number",
            readOnly: "boolean",
            precision: "enum",
            enabled: "boolean",
            htmlAttributes: "data"
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "allowReset": {
                        if (options[key]) {
                            this._showResetButton();
                        }
                        else {
                            this._hideResetButton();
                        }
                        break;
                    }
                    case "value": {
                        this.setValue(ej.util.getVal(options[key]));
                        options[key] = this.model.value;
                        break;
                    }
                    case "enabled": this._enabledAction(options[key]); break;
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "height": this._mainDiv.height(options[key]); break;
                    case "width": this._mainDiv.width(options[key]); break;
                    case "readOnly": {
                        this.model.readOnly = options[key];
                        if (options[key])
                            this._unWireEvents();
                        else
                            this.refresh();
                        break;
                    }
                    case "orientation":
                        {
                            this.model.orientation = options[key];
                            this.refresh();
                            break;
                        }
                    case "maxValue":
                        {
                            this.model.maxValue = options[key];
                            this.refresh();
                            break;
                        }
                    case "minValue":
                        {
                            this.model.minValue = options[key];
                            this.refresh();
                            break;
                        }
                    case "incrementStep":
                        {
                            this.model.incrementStep = options[key];
                            this.refresh();
                            break;
                        }
                    case "shapeWidth":
                        {
                            this.model.shapeWidth = options[key];
                            this.refresh();
                            break;
                        }
                    case "shapeHeight":
                        {
                            this.model.shapeHeight = options[key];
                            this.refresh();
                            break;
                        }
                    case "htmlAttributes": this._addAttr(options[key]); break;
                }
            }
        },


        _destroy: function () {
            this.element.show();
            this._unWireEvents();
            this._mainDiv.remove();
        },

        _init: function () {
            this._initialize();
        },

        _initialize: function () {
            this.element.hide();
            this._mainDiv = ej.buildTag("div.e-rating e-widget " + this.model.cssClass, "", {}, { tabindex: "0", role: "group", "aria-label": "Rating" });
            if (!ej.isNullOrUndefined(this.model.width))
                this._mainDiv.width(this.model.width);
            if (!ej.isNullOrUndefined(this.model.height))
                this._mainDiv.height(this.model.height);
            this._mainDiv.insertBefore(this.element);
            if (this.model.orientation == ej.Orientation.Horizontal) {
                this._mainDiv.addClass("e-horizontal");
            }
            else
                this._mainDiv.addClass("e-vertical");

            if (this.model.allowReset && !this.model.readOnly) {
                this._createReset();
            }
            this._validation();
            if (this.model.value == 1 && this.element[0].value != "")
                this.model.value = this.element[0].value;
            this._renderShape();
            this._shapes = this._mainDiv.find("div.e-shape");
            this._wireEvents();
            this._CurrentIndex = 0;
            this._initCurrentValue();
            this.element.val(this.value());
            this._enabledAction(this.model.enabled);
            this._checkNameAttr();
            this._addAttr(this.model.htmlAttributes);
        },
        _checkNameAttr: function () {
            if (!this.element.attr("name"))
                this.element.attr({ "name": this.element[0].id });
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy._mainDiv.addClass(value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "readonly") proxy.model.readOnly = value;
                else if (key == "disabled" && value == "disabled") proxy._enabledAction(false);
                else proxy._mainDiv.attr(key, value)
            });
        },

        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this._mainDiv.removeClass(this.model.cssClass).addClass(skin);
            }
        },

        _enabledAction: function (flag) {
            if (flag) {
                this._mainDiv.removeClass("e-disable");
				this.element.removeAttr("disabled");
            }
            else {
                this._mainDiv.addClass("e-disable");
				this.element.attr("disabled","disabled");
            }
        },

        _validation: function () {
            if (this.model.incrementStep < 1) {
                this.model.incrementStep = 1;
            }
            else if (this.model.incrementStep > this.model.maxValue) {
                this.model.incrementStep = this.model.maxValue;
            }
        },

        _createReset: function () {
            if (this._mainDiv.find("div.e-reset").length <= 0)
                ej.buildTag("div.e-reset", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight, title: "Reset" }, { role: "button", "aria-label": "reset", "aria-hidden": false }).appendTo(this._mainDiv);
            else
                this._mainDiv.find("div.e-reset").show();
        },

        _renderShape: function () {
            var _shapeCount = Math.round(this.model.maxValue / this.model.incrementStep);
            var _startCount = Math.round(this.model.minValue / this.model.incrementStep);
            var containerWith, containerHeight;
            if (this.model.orientation == ej.Rating.Orientation.Horizontal) {
                containerWith = (_shapeCount + 1) * this.model.shapeWidth;
                containerHeight = this.model.shapeHeight;
            } else if (this.model.orientation == ej.Rating.Orientation.Vertical) {
                containerWith = this.model.shapeWidth;
                containerHeight = this.model.shapeHeight * _shapeCount;
            }
            if (_shapeCount > 0) {
                var _ulTag = ej.buildTag("ul.e-ul", "", { width: containerWith + "px", height: containerHeight }, {});
                for (var i = _startCount ; i < _shapeCount; i++) {
                    var _liTag = ej.buildTag("li.e-shape-list", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight }, { tabindex: -1, "aria-describedby": this.element.prop("id") + "_tooltip" });
                    ej.buildTag("div.e-shape inactive", "", { width: this.model.shapeWidth + "px", height: this.model.shapeHeight }).appendTo(_liTag);
                    _liTag.appendTo(_ulTag);
                }
                _ulTag.appendTo(this._mainDiv);
            }
        },

        _initCurrentValue: function () {
            if (ej.isNullOrUndefined(this.value()) || this.value() == "" || this.value() == 0)
                this._CurrentIndex = 0;
            else if (this.value() > this.model.minValue) {
                var count = (this.value() / this.model.incrementStep) - (this.model.minValue / this.model.incrementStep);
                this._CurrentIndex = count;
                this._valueRefresh(this._CurrentIndex);
                for (var i = 0; i < count; i++) {
                    if (this._shapes[i]) {
                        $(this._shapes[i]).removeClass('inactive').removeClass('active').addClass('selected');
                    }
                    else {
                        this._CurrentIndex = i;
                        break;
                    }
                }
            }
            this.value(this._CurrentIndex);
        },

        _valueRefresh: function (index) {
            index = Math.ceil(index);
            var value = (this.value() / this.model.incrementStep).toFixed(1);
            if (this.model.orientation == ej.Rating.Orientation.Horizontal) {
                var len;
                if (this.model.precision == "exact")
                    len = (value % 1).toFixed(1);
                else if (this.model.precision == "half")
                    len = (value % 1 > 0.5) ? 1 : 0.5;
                else
                    len = 1;
                value = ((value % 1).toFixed(1) != 0 ? len : (value / this._CurrentIndex)) * this.model.shapeWidth;
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ width: this.model.shapeWidth + "px" });
                }
                if (this._shapes != null)
                    $(this._shapes[index - 1]).css({ width: value + "px" });
            }
            else {
                value = ((value % 1).toFixed(1) != 0 ? (value % 1).toFixed(1) : (value / this._CurrentIndex)) * this.model.shapeHeight;
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ height: this.model.shapeHeight + "px" });
                }
                if (this._shapes != null) $(this._shapes[index - 1]).css({ height: value + "px" });
            }
        },


        _reset: function () {
            if (!this.model.enabled) return false;
            $(this._shapes).removeClass('active').removeClass('selected').addClass('inactive');
            this._prevValue = this.value();
            this.value(0);
            this._CurrentIndex = 0;
            //Update Hidden field
            this._currValue = this.value();
            this.element.value = this._currValue;
            this._valueChanged();
        },

        _fillExactPrecision: function (element, position) {
            var index = this._shapes.index(element) + (this.model.minValue / this.model.incrementStep) + 1;
            var option = this.model;
            if (option.orientation == ej.Rating.Orientation.Horizontal) {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ width: option.shapeWidth + "px" });
                    else
                        $(this._shapes[i]).css({ width: "0px" });
                }
                //Set width and title
                $(element).css({ width: position + "px" });
                this.toolTipValue = ((index - 1) * option.incrementStep + (position / option.shapeWidth) * option.incrementStep).toFixed(1);
            }
            else {
                for (var i = 0; i < index; i++) {
                    $(this._shapes[i]).css({ height: option.shapeHeight + "px" });
                }
                for (var i = index; i < this._shapes.length; i++) {
                    $(this._shapes[i]).css({ height: "0px" });
                }
                //Set width and title
                $(element).css({ height: position + "px" });
                this.toolTipValue = ((index - 1) * option.incrementStep + (position / option.shapeHeight) * option.incrementStep).toFixed(1);
            }
        },


        _fillHalfPrecision: function (element, position) {
            var index = this._shapes.index(element) + (this.model.minValue / this.model.incrementStep) + 1;
            var options = this.model;
            if (options.orientation == ej.Rating.Orientation.Horizontal) {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ width: options.shapeWidth + "px" });
                    else
                        $(this._shapes[i]).css({ width: "0px" });
                }
                if ((position + 1) % 2 == 0) {
                    position = options.shapeWidth / 2;
                    $(element).css({ width: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeWidth) * options.incrementStep).toFixed(1);
                }
                else {
                    position = options.shapeWidth;
                    $(element).css({ width: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeWidth) * options.incrementStep).toFixed(1);
                }
            }
            else {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ height: this._starHeight + "px" });
                    else
                        $(this._shapes[i]).css({ height: "0px" });
                }
                //Set width and title
                if (position <= options.shapeHeight / 2) {
                    position = options.shapeHeight / 2;
                    $(element).css({ height: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeHeight) * options.incrementStep).toFixed(1);
                }
                else {
                    position = options.shapeHeight;
                    $(element).css({ height: position + "px" });
                    this.toolTipValue = ((index - 1) * options.incrementStep + (position / options.shapeHeight) * options.incrementStep).toFixed(1);
                }
            }
        },

        _fillFullPrecision: function (element) {
            var index = this._shapes.index(element) + (this.model.minValue / this.model.incrementStep) + 1;
            var option = this.model;
            if (option.orientation == ej.Rating.Orientation.Horizontal) {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ width: option.shapeWidth + "px" });
                    else
                        $(this._shapes[i]).css({ width: "0px" });
                }
                $(element).css({ width: option.shapeWidth + "px" });
                this.toolTipValue = index;
            }
            else {
                for (var i = 0; i < this._shapes.length; i++) {
                    if (i < index)
                        $(this._shapes[i]).css({ height: option.shapeHeight + "px" });
                    else
                        $(this._shapes[i]).css({ height: "0px" });
                }
                $(element).css({ height: option.shapeHeight + "px" });
                this.toolTipValue = index;
            }
        },

        _ClickHandler: function (e) {
            //Check if stars and rate it
            if (this.model.enabled) {
                var element;
                if (e.target.tagName == "LI")
                    element = e.target.firstChild;
                else if (e.target.parentNode.tagName == "LI")

                    element = e.target;
                if (element) {
                    var index = this._shapes.index(element) + 1;
                    for (var i = 0; i < index; i++) {
                        if (this._shapes[i]) {
                            $(this._shapes[i]).removeClass('inactive').removeClass('active').addClass('selected');
                        }
                    }
                    this._prevValue = this.value();
                    this._CurrentIndex = index;
                    index = index + (this.model.minValue / this.model.incrementStep);
                    var _IncrementStep = this.model.incrementStep;

                    if (this.model.orientation == ej.Rating.Orientation.Horizontal)
                        this._currValue = ((index - 1) * _IncrementStep + (element.clientWidth / this.model.shapeWidth) * _IncrementStep).toFixed(1);
                    else
                        this._currValue = ((index - 1) * _IncrementStep + (element.clientHeight / this.model.shapeHeight) * _IncrementStep).toFixed(1);
                    this.element.value = this._currValue;
                    this.value(this._currValue);
                    this._trigger("click", { event: e, value: this.value(), prevValue: this._prevValue });
                    this._valueChanged();
                }
                //Check if reset button and reset
                if ($(e.target).hasClass('e-reset')) {
                    this._reset();
                }
            }
        },

        _MouseOverHandler: function (e) {
            if (this.model.enabled) {
                var element, target = e.target;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    target = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                if (target.tagName == "LI")
                    element = target.firstChild;
                else if (target.parentNode.tagName == "LI")
                    element = target;
                else
                    return false;
                var index = this._shapes.index(element) + 1;
                for (var i = 0; i <= this._shapes.length; i++) {
                    if (this._shapes[i]) {
                        if (i < index)
                            $(this._shapes[i]).removeClass('inactive').removeClass('selected').addClass('active');
                        else
                            $(this._shapes[i]).removeClass('active').removeClass('selected').addClass('inactive');
                    }
                }
                this._trigger("mouseover", { event: e, index: index, value: this.value() });
            }
        },

        _MouseOutHandler: function (e) {
            if (!this.model.readOnly && this.model.enabled) {
                var index = this._CurrentIndex;
                if (index != 0) {
                    for (var i = 0; i < this._shapes.length; i++) {
                        if (this._shapes[i]) {
                            if (i < index)
                                $(this._shapes[i]).removeClass('active').removeClass('inactive').addClass('selected');
                            else
                                $(this._shapes[i]).removeClass('active').addClass('inactive');
                        }
                    }
                } else
                    $(this._shapes).removeClass('active').addClass('inactive');
                //On mouse out, keep selected the stars w.r.t current Value
                this._valueRefresh(index);
                this._trigger("mouseout", { event: e, value: this.value() });
            }
            this._hideTooltip(1000);
        },

        _MouseMoveHandler: function (e) {
            if (!this.model.readOnly && this.model.enabled) {
                var element, position, target = e.target;
                if (e.type == 'touchmove') {
                    var coor = e.originalEvent.changedTouches[0];
                    target = document.elementFromPoint(coor.pageX, coor.pageY);
                }
                if (target.tagName == "LI")
                    element = target.firstChild;
                else if (target.parentNode.tagName == "LI")
                    element = target;
                else
                    return false;
                if (this.model.orientation == ej.Rating.Orientation.Horizontal) {
                    if (ej.isNullOrUndefined(e.offsetX))
                        position = e.pageX - $(target).offset().left;
                    else
                        position = e.offsetX + 1;
                } else {
                    if (ej.isNullOrUndefined(e.offsetY))
                        position = e.pageY - $(target).offset().top;
                    else
                        position = e.offsetY + 1;
                }
                this.toolTipValue = (this.model.minValue + (this._shapes.index(element) + 1)) * this.model.incrementStep;
                if (this.model.precision == ej.Rating.Precision.Exact)
                    this._fillExactPrecision(element, position);
                else if (this.model.precision == ej.Rating.Precision.Half)
                    this._fillHalfPrecision(element, Math.round(position));
                else this._fillFullPrecision(element);
            } else {
                this.toolTipValue = (this.value() == null || this.value() == "" ? 0 : this.value()) + ' / ' + this.model.maxValue;
            }
            $(e.target).attr("aria-label", this.toolTipValue);
            if (parseFloat(this.model.maxValue) >= parseFloat(this.toolTipValue))
                this._showhideTooltip(e.target);
        },

        _showTooltip: function (element) {
            if (this.model.showTooltip) {
                $('.e-tooltip').remove();
                this.tooltip = ej.buildTag("div#" + this.element.prop("id") + "_tooltip.e-tooltip " + this.model.cssClass + " e-corner-all", "", {}, { role: "tooltip" });
                $("body").append(this.tooltip);
                this._isTooltip = true;
                this._setTooltipPosition(element);
            }
        },

        _hideTooltip: function (animationSpeed) {
            if (this.model.showTooltip) {
                $(this.tooltip).fadeOut(animationSpeed, function () {
                    $('.e-tooltip').remove();
                });
                this._isTooltip = false;
            }
        },

        _showhideTooltip: function (element) {
            if (this.model.showTooltip) {
                this._showTooltip(element);
            }
        },

        _setTooltipPosition: function (element) {
            if (this.model.showTooltip) {
                this._updateTooltipValue();
                var top, left, remainLeft, remainTop, handle, pos, gap = 5;
                handle = $(element), pos = this._getOffset(handle);
                if (this.model.orientation == "vertical") {
                    remainTop = (this.tooltip.outerHeight() - handle.outerHeight()) / 2;
                    remainLeft = handle.outerWidth() + gap;
                    top = pos.top - remainTop;
                    left = pos.left + remainLeft;
                }
                else {
                    remainLeft = (this.tooltip.outerWidth() - handle.outerWidth()) / 2;
                    remainTop = this.tooltip.outerHeight() + gap;
                    top = pos.top - remainTop;
                    left = pos.left - remainLeft;
                }
                this.tooltip.css({ "top": top, "left": left });
            }
        },
        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },


        _updateTooltipValue: function () {
            this.tooltip[0].innerHTML = this.toolTipValue;
        },

        _wireEvents: function () {
            if (!this.model.readOnly) {
                this._on(this._mainDiv, "mousedown", this._ClickHandler);
                this._on(this._mainDiv.find("li"), "mouseenter touchmove", this._MouseOverHandler);
            }
            this._on(this._mainDiv.find("li"), "mouseleave touchend", this._MouseOutHandler);
            this._on(this._mainDiv.find("li"), ej.eventType.mouseMove, this._MouseMoveHandler);

        },

        _unWireEvents: function () {
            this._mainDiv.find("li").unbind("mouseenter touchmove");
            this._mainDiv.find("li").unbind("mouseleave touchend");
            this._mainDiv.unbind("mousedown");
            if (this.model.precision !== ej.Rating.Precision.Full) {
                this._mainDiv.find("li").unbind(ej.eventType.mouseMove);
            }
        },

        _valueChanged: function () {
            this.element.val(this.value());
            if (this._prevValue != this._currValue)
                this._trigger("change", { value: this.value(), prevValue: this._prevValue });
        },

        reset: function () {
            this._reset();
        },

        show: function () {
            if (!this.model.enabled) return false;
            this._mainDiv.css("visibility", "visible");
        },

        hide: function () {
            if (!this.model.enabled) return false;
            this._mainDiv.css("visibility", "hidden");
        },

        _showResetButton: function () {
            if (!this.model.enabled) return false;
            this._createReset();
        },

        _hideResetButton: function () {
            if (!this.model.enabled) return false;
            this._mainDiv.find("div.e-reset").hide()
        },

        getValue: function () {
            return this.value() == null || this.value() === "" ? "" : this.value();
        },

        setValue: function (value) {
            if (value != null) {
                if (this.model.maxValue < value)
                    value = this.model.maxValue;
                else if (this.model.minValue > value)
                    value = this.model.minValue;
                this.value(value);
                $(this._shapes).removeClass('selected').addClass('inactive');
                this._initCurrentValue();
                this.element.val(this.value());
            }
        },

        refresh: function () {
            this._destroy();
            this._unWireEvents();
            this._initialize();
        }
    });

    ej.Rating.Precision = {
        /**  Allows user select full shape of Rating. */
        Full: "full",
        /**  Allows user select half shape of Rating. */
        Half: "half",
        /**  Allows user select exact shape of Rating. */
        Exact: "exact"
    };

    ej.Rating.Orientation = {
        /**  Renders Rating control in horizontal direction. */
        Horizontal: "horizontal",
        /**  Renders Rating control in vertical direction. */
        Vertical: "vertical"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Rotator control.
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejRotator", "ej.Rotator", {
        element: null,
        model: null,
        _ignoreOnPersist: ["fields", "dataSource", "query", "create", "change", "start", "stop", "thumbItemClick", "pagerClick", "destroy"],
        _setFirst: false,

        defaults: {

            cssClass: "",

            dataSource: null,

            query: null,

            fields: {

                text: "text",

                url: "url",

                linkAttribute: "linkAttribute",

                targetAttribute: "targetAttribute",

                thumbnailText: "text",

                thumbnailUrl: "url"
            },

            enabled: true,

            displayItemsCount: "1",

            navigateSteps: "1",

            animationSpeed: 600,

            startIndex: "0",

            showPlayButton: false,

            enableAutoPlay: false,

            showNavigateButton: true,

            slideWidth: "",

            slideHeight: "",

            frameSpace: "",

            isResponsive: false,

            orientation: "horizontal",

            pagerPosition: "outside",

            showThumbnail: false,

            showPager: true,

            stopOnHover: false,

            htmlAttributes: {},

            thumbnailSourceID: null,

            showCaption: false,

            allowKeyboardNavigation: true,

            enableRTL: true,

            animationType: "slide",

            delay: 500,

            circularMode: true,

            create: null,

            change: null,

            start: null,

            stop: null,

            thumbItemClick: null,

            pagerClick: null,

            destroy: null

        },

        dataTypes: {
            cssClass: "string",
            dataSource: "data",
            query: "data",
            fields: "data",
            enabled: "boolean",
            displayItemsCount: "",
            navigateSteps: "",
            animationSpeed: "",
            transitionDelay: "",
            startIndex: "",
            showPlayButton: "boolean",
            enableAutoPlay: "boolean",
            slideWidth: "",
            slideHeight: "",
            frameSpace: "",
            isResponsive: "boolean",
            orientation: "enum",
            pagerPosition: "enum",
            showThumbnail: "boolean",
            thumbnailSourceID: "",
            showPager: "boolean",
            showCaption: "boolean",
            enableRTL: "boolean",
            allowKeyboardNavigation: "boolean",
            circularMode: "boolean",
            animationType: "string",
            delay: "",
            htmlAttributes: "data"
        },


        _init: function () {
            this._initialize();
            if (this.model.dataSource == null)
                this._setValues();
            this._render();
            this._wireEvent();
            if (this.model.enableAutoPlay) this.play();
            if ((!this.model.enabled) && (ej.browserInfo().name == "msie") && (ej.browserInfo().version == "8.0")) {
                this._outerWrapper.addClass("e-disable").attr({ "aria-disabled": true });
                this._ieEnable("addClass");
            }
        },

        _initialize: function () {
            this.element.attr("tabindex", 0);
            this._liCount = null;
            this.transitionDelay = null;
            this._animating = false;
            this._isPlaying = false;
            this._interval = null;
            this._thumbCount = null;
            this._liSize = null;
            this._containerSize = null;
            this.containerCss = this.model.orientation == ej.Orientation.Horizontal ? "outerWidth" : "outerHeight";
            this.displayCss = this.model.orientation == ej.Orientation.Horizontal ? "left" : "top";
            $.extend(jQuery.easing, {
                slowSlide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    return b + c * (ts * ts);
                },
                fastSlide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    var tc = ts * t;
                    return b + c * (tc * ts + -5 * ts * ts + 10 * tc + -10 * ts + 5 * t);
                },
                slide: function (x, t, b, c, d) {
                    var ts = (t /= d) * t;
                    var tc = ts * t;
                    return b + c * (6 * tc * ts + -15 * ts * ts + 10 * tc);
                }
            })
        },

        _setValues: function (val) {
            this._setLicount();
            this._setSpeed(this.model.animationSpeed);
            this._setVisibleItemCount(this.model.displayItemsCount);
            this._setItemMove(this.model.navigateSteps);
            this._setFrameSpace(this.model.frameSpace);
            this._setSlideWidth(this.model.slideWidth);
            this._setSlideHeight(this.model.slideHeight);
            this._changeProperty();
            this._setInitial();
        },
        _refresh: function () {
            this._setValues();
            this._setDimension();
            this._changeSkin();
        },
        _reRenderControl: function () {
            this._setLicount();
            this._setSpeed(this.model.animationSpeed);
            this._setVisibleItemCount(this.model.displayItemsCount);
            this._setItemMove(this.model.navigateSteps);
            this._setFrameSpace(this.model.frameSpace);
            this._setSlideWidth(this.model.slideWidth);
            this._setSlideHeight(this.model.slideHeight);
            this._changeProperty();
            this._cloneItem();
            this._createCaption();
            this._changeSkin();
        },
        _refreshControl: function () {
            if (this.model.showPager) {
                if (!ej.isNullOrUndefined(this._bulletWrapper))
                    this._bulletWrapper.remove();
                this._createBulletControl();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-pager-wrap"))
                this._bulletWrapper.remove();

            if (this.model.showCaption) {
                if (this.element.siblings().hasClass("e-caption"))
                    this._caption.remove();
                this._createCaption();
            } else if (this.element.siblings().hasClass("e-caption"))
                this._caption.remove();

            if (this.model.showThumbnail) {
                if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb")) {
                    this._thumb.remove();
                    if (!ej.isNullOrUndefined(this._thumbControl)) this._thumbControl.remove();
                }
                this._createThumb();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb")) {
                this._thumb.remove();
                if (!ej.isNullOrUndefined(this._thumbControl)) this._thumbControl.remove();
            }
            this._changeSkin();
        },

        _checkDataBinding: function () {
            var source = this.model.dataSource;
            if (source != null) {
                this.element.addClass("onloading");
                if (ej.DataManager && source instanceof ej.DataManager)
                    this._initDataSource(source);
                else
                    this._renderItems(source);
            }
            this._setValues();
        },

        _initDataSource: function (source) {
            var proxy = this, queryPromise;
            queryPromise = source.executeQuery(this._getQuery());
            queryPromise.done(function (e) {
                proxy._renderItems(e.result);
            }).fail(function (e) {
                proxy.element.removeClass("onloading");
            });
        },
        _getQuery: function () {
            var column;
            if (ej.isNullOrUndefined(this.model.query)) {
                column = [], queryManager = ej.Query(), mapper = this.model.fields, col;
                for (col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },

        _renderItems: function (list) {
            this.list = list;
            this._generateTagItems(this.list);
            this.element.removeClass("onloading");
        },

        _generateTagItems: function (list) {
            var i; this.mapField = this._getMapper();
            for (i = 0; i < list.length; i++) {
                this.element.append(this._generateLi(list[i], this.mapField));
            }
        },

        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _text: null, _url: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            mapFld._url = (mapper && mapper.url) ? mapper["url"] : "url";
            mapFld._attr = (mapper && mapper.htmlAttr) ? mapper["htmlAttr"] : "htmlAttr";
            mapFld._linkAttribute = (mapper && mapper.linkAttribute) ? mapper["linkAttribute"] : "linkAttribute";
            mapFld._targetAttribute = (mapper && mapper.targetAttribute) ? mapper["targetAttribute"] : "targetAttribute";
            mapFld._thumbnailText = (mapper && mapper.thumbnailText) ? mapper["thumbnailText"] : "text";
            mapFld._thumbnailUrl = (mapper && mapper.thumbnailUrl) ? mapper["thumbnailUrl"] : "url";
            return mapFld;
        },

        _generateLi: function (list, map) {
            var li = ej.buildTag("li"), imgTag, anchorTag;
            imgTag = ej.buildTag("img");
            if (!ej.isNullOrUndefined(list[map._url])) imgTag.attr("src", list[map._url]);
            if (!ej.isNullOrUndefined(list[map._text])) imgTag.attr("title", list[map._text]);
            if (!ej.isNullOrUndefined(list[map._linkAttribute])) {
                anchorTag = ej.buildTag("a").attr({ 'href': list[map._linkAttribute] }).append(imgTag);
                if (!ej.isNullOrUndefined(list[map._targetAttribute])) anchorTag.attr("target", "_" + list[map._targetAttribute]);
                li.append(anchorTag);
            } else
                li.append(imgTag);
            this._setAttributes((list[map._attr]), imgTag);
            return li;
        },

        _generateThumbnailLi: function (list, map) {
            var li = ej.buildTag("li"), imgTag;
            imgTag = ej.buildTag("img");
            if (!ej.isNullOrUndefined(list[map._thumbnailUrl])) imgTag.attr("src", list[map._thumbnailUrl]);
            if (!ej.isNullOrUndefined(list[map._thumbnailText])) imgTag.attr("title", list[map._thumbnailText]);
            li.append(imgTag);
            return li;
        },

        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    element.attr(key, data[key]);
            }
        },
        _createWrapper: function () {
            this._wrapper = ej.buildTag("div.e-inner e-box e-" + this.model.orientation).insertBefore(this.element);
            if (this.model.dataSource != null)
                this._checkDataBinding();
            this._wrapper.append(this.element);
            this._innerWrapper = ej.buildTag("div.e-in-wrap").insertBefore(this._wrapper);
            this._innerWrapper.append(this._wrapper);
            this._outerWrapper = ej.buildTag("div.e-rotator-wrap e-widget " + this.model.cssClass).insertBefore(this._innerWrapper);
            this._outerWrapper.append(this._innerWrapper);
            if (!this.model.enabled) this._outerWrapper.addClass("e-disable").attr({ "aria-disabled": true });
        },
        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.element.children('li').remove();
            this._checkDataBinding();
            this._cloneItem();
            this._refreshControl();
        },

        _createButtonControl: function () {
            this._buttonWrapper = ej.buildTag("div.e-nav").insertAfter(this._wrapper);
            if (this.model.showNavigateButton) {
                this._prevButton = ej.buildTag("span.e-nav-btn e-icon e-previous").attr({ 'role': 'button' }).appendTo(this._buttonWrapper);
                this._nextButton = ej.buildTag("span.e-nav-btn e-icon e-next").attr({ 'role': 'button' }).appendTo(this._buttonWrapper);
            }
            this._buttonWrapper.appendTo(this._wrapper);
            this._createAutoPlay();
            if (this.model.enabled && this.model.showNavigateButton) this._wireBtnEvents();
        },

        _createAutoPlay: function () {
            var icon;
            if (!this.model.showPlayButton) return false;
            icon = this._interval != null ? 'pause' : 'play';
            this._autoButton = ej.buildTag("span.e-nav-btn e-icon " + icon, "", {}, { role: "button" }).appendTo(this._buttonWrapper);
            if (this.model.enabled) this._wireAutoPlayEvents();
        },

        _createBulletControl: function () {
            var end, liElement;
            this._bulletWrapper = ej.buildTag("div.e-pager-wrap", "", {}, { role: "group", "aria-label": "showPager", tabindex: "0" }).insertAfter(this._innerWrapper);
            this._bullet = ej.buildTag("ul.e-bullet e-default");
            if (this.model.circularMode) {
                end = (this._liCount - (this.model.displayItemsCount * 2)) / this.model.navigateSteps;
                if ((this._liCount - (this.model.displayItemsCount * 2)) % this.model.navigateSteps != 0)
                    end++;
            }
            else {
                end = ((this._liCount - this.model.displayItemsCount) / this.model.navigateSteps) + 1;
                if ((this._liCount - this.model.displayItemsCount) % this.model.navigateSteps != 0)
                    end++;
            }
            for (i = 1; i <= end ; i++)
                ej.buildTag("li.e-icon", "", {}, { tabindex: "-1", "aria-selected": "false" }).appendTo(this._bullet);
            this._bullet.appendTo(this._bulletWrapper);
            this._setActiveBullet();
            this._setPagerposition(this.model.pagerPosition);
            if (this.model.enabled) this._wireBulletEvents();
        },

        _createCaption: function () {
            var index;
            if (!this.model.showCaption) return false;
            this._caption = ej.buildTag("div.e-caption").append($('<span>')).insertAfter(this._buttonWrapper);
            if (this.model.circularMode) {
                index = Math.round((-Math.round(this.element.position()[this.displayCss])) / this._liSize);
                index--;
            }
            else
                index = Math.round((-Math.round(this.element.position()[this.displayCss])) / (this._liSize));

            this._setCaptionText(index);
        },

        _createThumb: function () {
            var liElements, i;
            if (this.model.dataSource != null || this.model.thumbnailSourceID != null) {
                this._thumb = this.model.showPager ? ej.buildTag("div.e-thumb", "", {}, { tabindex: "0", role: "group" }).insertAfter(this._bulletWrapper) : ej.buildTag("div.e-thumb", "", {}, { tabindex: "0", role: "group" }).insertAfter(this._innerWrapper);
                this._thumbItems = ej.buildTag("ul.e-thumb-items e-ul");
                if (this.model.dataSource != null) {
                    this._thumbItems.addClass("onloading");
                    this._generateThumbnailTagItems(this.list);
                } else {
                    for (i = 0; i < $('#' + this.model.thumbnailSourceID).children('li').length ; i++) {
                        liElements = $('#' + this.model.thumbnailSourceID).children('li').eq(i).clone();
                        liElements.addClass("e-thumb-ele").attr('tabindex', -1).removeAttr("style").appendTo(this._thumbItems);
                    }
                    $('#' + this.model.thumbnailSourceID).css("display", "none");
                }
                this._thumbItems.appendTo(this._thumb);
                this._setThumbProperty();
                this._setActiveThumb();
            }
        },

        _createThumbControl: function () {
            this._thumbControl = ej.buildTag("div.e-thumb-nav").insertAfter(this._thumb);
            this._previous = ej.buildTag("span.e-thumb-btn e-icon e-previous e-disable", "", {}, { "aria-disabled": true }).appendTo(this._thumbControl);
            this._next = ej.buildTag("span.e-thumb-btn e-icon e-next e-enable", "", {}, { "aria-disabled": false }).appendTo(this._thumbControl);
            this._thumbControl.appendTo(this._outerWrapper);
            if (this.model.enabled) this._wireThumbEvents();
        },

        _generateThumbnailTagItems: function (list) {
            var i;
            for (i = 0; i < list.length; i++) {
                this._thumbItems.append(this._generateThumbnailLi(list[i], this.mapField).addClass("e-thumb-ele").attr('tabindex', -1));
            }
            this._thumbItems.removeClass("onloading");
        },

        _setSkin: function (skin) {
            this._outerWrapper.removeClass(this.model.cssClass).addClass(skin);
        },

        _cloneItem: function () {
            if (this.model.circularMode) {
                var i = 0, CloneElements = ej.buildTag("ul");
                for (i = 0; i < this.model.displayItemsCount ; i++)
                    this.element.children('li').eq(i).clone().addClass("clone").appendTo(this.element);
                for (i = (this._liCount - this.model.displayItemsCount) ; i < this._liCount; i++)
                    this.element.children('li').eq(i).clone().addClass("clone").prependTo(CloneElements);
                for (i = 0; i < CloneElements.children('li').length; i++)
                    CloneElements.children('li').eq(i).clone().prependTo(this.element);
            }
            this._setDimension();
        },

        _undoClone: function () {
            for (i = 0; i < this._liCount; i++) {
                if (this.element.children('li').eq(i).hasClass("clone")) {
                    this.element.children('li').eq(i).remove();
                    this._setLicount();
                    i--;
                }
            }
        },

        _setModel: function (options) {
            var option, valueChange = false, controlChange = false;
            for (option in options) {
                switch (option) {
                    case "enableAutoPlay":
                        (this.model.enableAutoPlay = options[option]) ? this.play() : this._setPause();
                        break;
                    case "fields":
                    case "query":
                    case "dataSource":
                        this._refreshTagItems(option, options[option]);
                        break;
                    case "enabled":
                        this._changeState(options[option]);
                        break;
                    case "slideWidth":
                        this._setSlideWidth(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "slideHeight":
                        this._setSlideHeight(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "displayItemsCount":
                        this._undoClone();
                        this._setVisibleItemCount(options[option]);
                        this._cloneItem();
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "navigateSteps":
                        options[option] = this._validateItemMove(options[option]);
                        this.model.navigateSteps = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "pagerPosition":
                        this._setPagerposition(options[option]);
                        break;
                    case "showPager":
                        this.model.showPager = options[option];
                        controlChange = true;
                        break;
                    case "showThumbnail":
                        this.model.showThumbnail = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "thumbnailSourceID":
                        this.model.thumbnailSourceID = options[option];
                        if (this.model.dataSource == null) {
                            valueChange = true;
                            controlChange = true;
                        }
                        break;
                    case "showCaption":
                        this.model.showCaption = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "isResponsive":
                        this.model.isResponsive = options[option];
                        this._wireResizeEvents();
                        break;
                    case "orientation":
                        this.model.orientation = options[option];
                        this._setOrientation(options[option]);
                        break;
                    case "animationSpeed":
                        this._setSpeed(options[option]);
                        break;
                    case "frameSpace":
                        this._setFrameSpace(options[option]);
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "showPlayButton":
                        this.model.showPlayButton = options[option];
                        this._setAutoplay(options[option]);
                        this._changeSkin();
                        break;
                    case "startIndex":
                        this.model.startIndex = options[option];
                        valueChange = true;
                        controlChange = true;
                        break;
                    case "allowKeyboardNavigation":
                        this.model.allowKeyboardNavigation = options[option];
                        this._wireKeyboardEvents();
                        break;
                    case "showNavigateButton":
                        this.model.showNavigateButton = options[option];
                        this._setSlideButton(options[option]);
                        this._changeSkin();
                        break;
                    case "animationType":
                        this.model.animationType = options[option];
                        this._setSpeed(this.model.animationSpeed);
                        break;
                    case "cssClass": this._setSkin(options[option]); break;
                    case "enableRTL":
                        this.model.enableRTL = options[option];
                        break;
                    case "circularMode":
                        this.model.circularMode = options[option];
                        this._reRenderControl();
                        this._refresh();
                        break;
                    case "delay":
                        this.model.delay = options[option]
                        this._setSpeed(this.model.animationSpeed);
                        break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }

            if (valueChange) this._refresh();
            if (controlChange) this._refreshControl();
        },

        _setDimension: function () {
            this._setLicount();
            this._setUlInitial();
            this._setWidth();
            this._setHeight();
            this._setContainerPercent();
            this._changeProperty();
        },

        _render: function () {
            this._createWrapper();
            this._addAttr(this.model.htmlAttributes);
            this._cloneItem();
            this._createButtonControl();
            this._createCaption();
            if (this.model.showPager) this._createBulletControl();
            if (this.model.showThumbnail) this._createThumb();
            this._outerWrapper.find(this.element).addClass("e-ul");
            this._changeSkin();
        },

        _setWidth: function () {
            this._containerWidth = this.model.orientation == "horizontal" ?
            ((parseInt(this.model.slideWidth) * this.model.displayItemsCount) + (parseInt(this.model.frameSpace) * (parseInt(this.model.displayItemsCount) - 1))) : this.model.slideWidth;
            this.element.parent().css('width', this._containerWidth);
            this.element.parents('.e-in-wrap').css('width', this._containerWidth);
            this.element.parents('.e-rotator-wrap').css('width', this._containerWidth);
            this._changeProperty();
        },
        _setHeight: function () {
            this._containerHeight = this.model.orientation == "vertical" ?
            ((parseInt(this.model.slideHeight) * this.model.displayItemsCount) + (parseInt(this.model.frameSpace) * (parseInt(this.model.displayItemsCount) - 1))) : this.model.slideHeight;
            this.element.parent().css('height', this._containerHeight);
            this._changeProperty();
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy._outerWrapper.addClass(value);
                else proxy._outerWrapper.attr(key, value);
                if (key == "disabled" && value == "disabled") proxy.disable();
            });
        },

        _setUlInitial: function () {
            if (this.model.orientation == ej.Orientation.Horizontal) {
                this.element.css({
                    "width": this._liCount * this._liWidth,
                    "height": this._liHeight
                });
            }
            else {
                this.element.css({
                    "height": this._liCount * this._liHeight,
                    "width": this._liWidth
                });
            }
        },

        _setInitial: function () {
            this.model.circularMode ? this.element.css(this.displayCss, -(this._liSize * this.model.displayItemsCount) - (this.model.startIndex * this.model.navigateSteps * this._liSize)) : this.element.css(this.displayCss, "0");
        },
        _setBegin: function () {
            this.model.circularMode ? this.element.css(this.displayCss, -(this._liSize * this.model.displayItemsCount)) : this.element.css(this.displayCss, "0");
        },

        _setSlideWidth: function (val) {
            if (isNaN(parseInt(val)))
                this.model.slideWidth = this._getMaxSize("width");
            else {
                val = val.toString();
                this.model.slideWidth = this._checkValue(val, "width");
            }
            this.element.children('li').css("width", this.model.slideWidth);
            this._liWidth = this.element.children('li').outerWidth(true);
        },

        _setSlideHeight: function (val) {
            if (isNaN(parseInt(val)))
                this.model.slideHeight = this._getMaxSize("height");
            else {
                val = val.toString();
                this.model.slideHeight = this._checkValue(val, "height");
            }
            this.element.children('li').css("height", this.model.slideHeight);
            this._liHeight = this.element.children('li').outerHeight(true);
        },

        _setPagerposition: function (val) {
            if (this.model.showPager) {
                this._bullet.removeClass('e-default e-thumb-pos e-topleft e-topright e-bottomleft e-bottomright e-topCenter e-outside');
                this._bullet.addClass("e-" + val);
                if ((val == "bottomleft" || val == "bottomright") && this.model.showThumbnail && (this.model.dataSource != null || this.model.thumbnailSourceID != null))
                    this._bullet.addClass('e-thumb-pos');
            }
        },

        _setCaptionText: function (index) {
            if (this.model.showCaption) {
                var indx = (this.model.circularMode) ? ++index : index;
                this._caption.children().html(this.element.children('li').eq(indx).children('img').attr("title"));
            }
        },

        _setAutoplay: function (val) {
            if (val)
                this._createAutoPlay();
            else if (this._autoButton != null) {
                this._setPause();
                this._autoButton.remove();
            }
        },

        _setSlideButton: function (val) {
            if (val)
                this._createButtonControl()
            else {
                this._prevButton.remove();
                this._nextButton.remove();
            }
        },

        _setOrientation: function (val) {
            this._wrapper.removeClass("e-" + this.model.orientation).addClass("e-" + val);
            this._setFrameSpace(this.model.frameSpace);
            this._refresh();
            if (this.model.orientation == ej.Orientation.Horizontal)
                this.element.css({ "left": -(Math.round(-(this.element.position().top) / this._liHeight) * this._liSize), "top": 0 });
            else
                this.element.css({ "top": -(Math.round(-(this.element.position().left) / this._liWidth) * this._liSize), "left": 0 });
        },

        _setActiveBullet: function () {
            var index;
            if (this.model.circularMode) {
                index = Math.round(-Math.round(this.element.position()[this.displayCss]) / (this._liSize * this.model.displayItemsCount));
                if (this.model.navigateSteps < this.model.displayItemsCount)
                    index = -(Math.round(this.element.position()[this.displayCss]) + (this._liSize * this.model.displayItemsCount)) / (this._liSize * this.model.navigateSteps);
                index = this.model.navigateSteps == this.model.displayItemsCount ? index - 1 : index;
            }
            else
                index = Math.round(-(Math.round(this.element.position()[this.displayCss])) / (this._liSize * this.model.navigateSteps));

            index = index < 0 ? -index : index;
            this._bullet.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
        },

        _setActiveThumb: function () {
            if (this.model.circularMode) {
                var index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize);
                index--;
            }
            else {
                var index = Math.round(Math.round(this.element.position()[this.displayCss]) / (this._liSize));
                index = (index == 0) ? index : -(index);
            }
            this._thumbItems.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
            this._transferThumb(index);
        },
        _checkValue: function (val, orient) {
            if ((val.indexOf("%")) > 0) {
                if (Math.round(parseInt(val)) == "100") {
                    this.model.displayItemsCount = 1;
                    this._setVisibleItemCount(this.model.displayItemsCount);
                }
                val = this._convertPercentageToPixel($(this.element[0]).parent()[orient](), parseInt(val));
            }
            return (val);
        },

        _convertPercentageToPixel: function (parent, child) {
            return Math.round((child * parent) / 100);
        },

        _convertPixelToPercentage: function (parent, child) {
            return Math.round((child / parent) * 100);
        },
        _changeProperty: function () {
            this.containerCss = this.model.orientation == ej.Orientation.Horizontal ? "outerWidth" : "outerHeight";
            this.displayCss = this.model.orientation == ej.Orientation.Horizontal ? "left" : "top";
            if (this.model.orientation == ej.Orientation.Horizontal) {
                this._liSize = this._liWidth;
                this._containerSize = this._containerWidth;
            }
            else {
                this._liSize = this._liHeight;
                this._containerSize = this._containerHeight;
            }
        },

        _setFrameSpace: function (val) {
            if (isNaN(parseInt(val))) {
                this.model.frameSpace = "0px";
            } else {
                this.model.frameSpace = val;
                this.model.orientation == ej.Orientation.Horizontal ? this.element.children('li').css({ "margin-right": val, "margin-bottom": 0 }) : this.element.children('li').css({ "margin-bottom": val, "margin-right": 0 });
            }
        },

        _setLicount: function () {
            this._liCount = this.element.children('li').length;
        },

        _setVisibleItemCount: function (val) {
            this.model.displayItemsCount = val;
            this.model.navigateSteps = (this.model.navigateSteps <= this.model.displayItemsCount) ? this.model.navigateSteps : this.model.displayItemsCount;
            this._checkAndSet();
        },
        _checkAndSet: function () {
            if ((this.model.displayItemsCount > 1) || (this.model.navigateSteps > 1)) {
                if (this.model.showThumbnail) this.model.showThumbnail = false;
                if (this.model.showCaption) this.model.showCaption = false;
            }
        },

        _changeSkin: function () {
            var value;
            if ((this.model.displayItemsCount > 1) || (this.model.navigateSteps > 1)) {
                this._innerWrapper.addClass("e-multiple");
                this._wrapper.removeClass("e-single");
                this._outerWrapper.css('width', this._innerWrapper.outerWidth(true));
            } else {
                this._innerWrapper.removeClass('e-multiple');
                this._wrapper.addClass("e-single");
            }
            if (this.model.showPlayButton)
                this._autoButton.css({
                    "left": (((this._innerWrapper.outerWidth(true) / 2) - (this._autoButton.width() / 2)) / this._innerWrapper.outerWidth(true)) * (100) + "%",
                    "top": (((this._innerWrapper.outerHeight(true) / 2) - (this._autoButton.height() / 2)) / this._innerWrapper.outerHeight(true)) * (100) + "%"
                });

            if (this.model.showNavigateButton) {
                value = (((this._innerWrapper.outerHeight(true) / 2) - (this._prevButton.height() / 2)) / this._innerWrapper.outerHeight(true)) * (100) + "%";
                this._prevButton.css("top", value);
                this._nextButton.css("top", value);
            }
            if ((this.model.showThumbnail) && (this.model.dataSource != null || this.model.thumbnailSourceID != null) && (this._previous != null) && (this._next != null)) {
                value = this._outerWrapper.outerHeight(true) + (this._thumbItems.children('li').outerHeight(true) / 2) - (this._thumbItems.children('li').outerHeight(true)) - (this._previous.height() / 2);
                value = (value / this._outerWrapper.outerHeight(true)) * 100 + "%";
                this._previous.css("top", value);
                this._next.css("top", value);
            }
        },

        _setItemMove: function (val) {
            this.model.navigateSteps = this._validateItemMove(val);
            this._checkAndSet();
        },
        _validateItemMove: function (val) {
            return ((val <= this.model.displayItemsCount) ? val : this.model.displayItemsCount);
        },

        _setPlay: function () {
            if ((this.model.showPlayButton) && (this._autoButton.hasClass("play")))
                this._autoButton.removeClass("play").addClass("pause");
            this._animate = true;
            this._raiseEvent("start");
        },

        _setPause: function () {
            if (this._interval != null) {
                if ((this.model.showPlayButton) && (this._autoButton.hasClass("pause")))
                    this._autoButton.removeClass("pause").addClass("play");
                clearInterval(this._interval);
                this._animate = this.model.stopOnHover ? true : false;
                this._interval = null;
                this._raiseEvent("stop");
            }
        },
        _setContainerPercent: function () {
            this._containerPercent = this._convertPixelToPercentage(this.element.parents('.e-rotator-wrap').parent().width(), this.element.parent().width());
        },

        _setSpeed: function (val) {
            this.transitionDelay = this.model.animationType != "slideshow" ? (val + this.model.delay) : ((val * 2) + this.model.delay);
        },

        _setThumbProperty: function () {
            var thumbwidth, itemsWidth = 0, margin;
            var listElement = this._thumbItems.children('li');
            for (var i = 0; i < listElement.length; i++) {
                if (listElement[i] != null)
                    itemsWidth = itemsWidth + (parseFloat(listElement.eq(i).width()) + parseFloat(listElement.eq(i).css("padding-left")) + Math.ceil(parseFloat(listElement.eq(i).css("border-left-width"))) + parseFloat(listElement.eq(i).css("margin-left")) + parseFloat(listElement.eq(i).css("padding-right")) + Math.ceil(parseFloat(listElement.eq(i).css("border-right-width"))) + parseFloat(listElement.eq(i).css("margin-right")));
            }
            thumbwidth = Math.round((this.element.parent().outerWidth(true)) - (((this.element.parent().outerWidth(true)) * 12) / 100));
            this._thumb.css('width', thumbwidth);
            this._thumbCount = Math.floor(this._thumb.outerWidth() / (itemsWidth / (listElement.length)));
            margin = Math.round(((this.element.parent().outerWidth(true) * 6) / 100) + (thumbwidth - (this._thumbCount * (itemsWidth / (listElement.length)))) / 2);
            this._thumb.css({ 'margin-left': margin, 'margin-right': margin, 'width': Math.ceil(this._thumbCount * Math.ceil(itemsWidth / (listElement.length))) });
            this._thumbItems.css({ 'width': Math.ceil(itemsWidth), 'height': this._thumbItems.children('li').outerHeight(true) });
            this._checkState();
            this._previous.css({ 'left': (margin - this._previous.width()) / 2 });
            this._next.css({ 'right': (margin - this._next.width()) / 2 });
        },
        _checkState: function () {
            if (this._thumbCount <= (this._liCount - (this.model.displayItemsCount * 2))) {
                if (!this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb-nav"))
                    this._createThumbControl();
            }
            else if (this.element.parents('.e-in-wrap').siblings().hasClass("e-thumb-nav"))
                this._thumbControl.remove();
        },

        _removeActiveItem: function () {
            if (this.model.showPager)
                this._bullet.children().removeClass('e-active').attr({ 'tabindex': -1, 'aria-selected': "false" });
            if (this.model.showThumbnail)
                this._thumbItems.children().removeClass('e-active').attr({ 'tabindex': -1, 'aria-selected': "false" });
        },

        _setActiveItem: function (index) {
            if (this.model.showPager)
                this._bullet.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
            if (this.model.showThumbnail) {
                this._thumbItems.children().eq(index).addClass('e-active').attr({ 'tabindex': 0, 'aria-selected': "true" });
                this._transferThumb(index);
                for (i = 0; i < this._thumbItems.children('li').length; i++)
                    if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover'))
                        this._thumbItems.children('li').eq(i).removeClass('e-thumbhover');
            }
        },

        _transferThumb: function (index) {
            var move, right;
            if (this.model.showThumbnail) {
                move = (index * this._thumbItems.children('li').outerWidth(true)) - (-this._thumbItems.position().left);
                right = -(this._thumbItems.position().left) + this._thumb.width();
                if ((this._thumbItems.width() - right) > move)
                    this._thumbMove(move);
                else
                    this._thumbMove((this._thumbItems.width() - right));
            }
        },

        _changeThumbControl: function (index) {
            var licount;
            if ((this.model.showThumbnail) && (this.model.dataSource != null || this.model.thumbnailSourceID != null)) {
                licount = this._thumbItems.children('li').length;
                if ((index * this._thumbItems.children('li').outerWidth(true)) > 0) {
                    if (this._previous != null && this._previous.hasClass("e-disable"))
                        this._previous.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
                }
                else if (this._previous != null && this._previous.hasClass("e-enable"))
                    this._previous.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });

                if ((index * this._thumbItems.children('li').outerWidth(true)) < ((licount - this._thumbCount) * (this._thumbItems.children('li').outerWidth(true)))) {
                    if (this._next != null && this._next.hasClass("e-disable"))
                        this._next.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
                }
                else if (this._next != null && this._next.hasClass("e-enable"))
                    this._next.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
            }
        },

        _getMaxSize: function (display) {
            var max = 0, current = 0, i = 0, proxy = this;
            for (i = 0; i < this._liCount; i++) {
                current = this.element.children('li').eq(i)[display]();
                max = current > max ? current : max;
            }
            return max;
        },
        _calculateValue: function () {
            return (this._convertPercentageToPixel((this._containerWidth - ((this.model.displayItemsCount - 1) * parseInt(this.model.frameSpace))), (100 / this.model.displayItemsCount)));
        },

        _changeState: function (val) {
            val ? this.enable() : this.disable();
        },

        _findActive: function () {
            var index, i;
            if (this.model.showPager) {
                for (i = 0; i < this._bullet.children('li').length; i++) {
                    if (this._bullet.children('li').eq(i).hasClass('e-active'))
                        return i;
                }
            }
            else if (this.model.showThumbnail) {
                for (i = 0; i < this._thumbItems.children('li').length; i++) {
                    if (this._thumbItems.children('li').eq(i).hasClass('e-active'))
                        return i;
                }
            }
            else if (!(this.model.showPager) && !(this.model.showThumbnail)) {
                index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize);
                index = this.model.circularMode ? index - 1 : index;
            }
            return index;
        },

        _wireEvent: function () {
            if (!this.model.enabled) return false;
            this._wrapper.bind("mouseenter", $.proxy(this._showControl, this));
            this._wrapper.bind("mouseleave", $.proxy(this._hideControl, this));
            this._wireKeyboardEvents();
            this._wireResizeEvents();
            this._on(this._wrapper, "swiperight swipeleft swipedown swipeup", { control: true }, this._touchOperations);
            if (this.model.showThumbnail && (this.model.dataSource != null || this.model.thumbnailSourceID != null)) this._on(this._thumb, "swiperight swipeleft", { control: false }, this._touchOperations);
        },

        _wireKeyboardEvents: function () {
            if (this.model.allowKeyboardNavigation) {
                this._outerWrapper.bind("focusin", $.proxy(this._onFocusIn, this));
                this._outerWrapper.bind("focusout", $.proxy(this._onFocusOut, this));
            }
            else {
                this._outerWrapper.unbind("focusin", $.proxy(this._onFocusIn, this));
                this._outerWrapper.unbind("focusout", $.proxy(this._onFocusOut, this));
            }
        },

        _wireBtnEvents: function () {
            if (this.model.showNavigateButton) {
                this._prevButton.bind("click", $.proxy(this._prevAction, this));
                this._nextButton.bind("click", $.proxy(this._nextAction, this));
            }
            else {
                this._prevButton.unbind("click", $.proxy(this._prevAction, this));
                this._nextButton.unbind("click", $.proxy(this._nextAction, this));
            }
        },

        _wireAutoPlayEvents: function () {
            if (this.model.showPlayButton)
                this._on(this._autoButton, "click", this._autoPlay);
            else
                this._off(this._autoButton, "click", this._autoPlay);
        },

        _wireResizeEvents: function () {
            if (this.model.isResponsive)
                $(window).bind("resize", $.proxy(this._resizeEvent, this));
            else
                $(window).unbind("resize", $.proxy(this._resizeEvent, this));
        },

        _wireThumbEvents: function () {
            if (this.model.showThumbnail) {
                this._on(this._thumbItems.children(), "click", this._thumbClick);
                this._on(this._next, "click", this._thumbNext);
                this._on(this._previous, "click", this._thumbPrev);
            }
            else {
                this._off(this._thumbItems.children(), "click", this._thumbClick);
                this._off(this._next, "click", this._thumbNext);
                this._off(this._previous, "click", this._thumbPrev);
            }
        },

        _wireBulletEvents: function () {
            if (this.model.showPager)
                this._on(this._bullet.children(), "click", this._bulletClick);
            else
                this._off(this._bullet.children(), "click", this._bulletClick);
        },

        _thumbNext: function () {
            var proxy = this, rightValue;
            if (!this.model.enabled) return false;
            if (this._previous.hasClass("e-disable"))
                this._previous.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
            rightValue = (-Math.round(this._thumbItems.position().left)) + this._thumb.width();
            if (this._thumbItems.width() - rightValue >= this._thumbItems.children('li').outerWidth(true))
                this._thumbMove(this._thumbItems.children('li').outerWidth(true));
            if ((this._thumbItems.width() - rightValue) <= this._thumbItems.children('li').outerWidth(true))
                this._next.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
        },

        _thumbMove: function (move) {
            var proxy = this;
            proxy._thumbItems.animate({
                "left": "-=" + move + "px",
            }, 600, function () { proxy._thumbItems.stop(true, true); });
        },

        _thumbPrev: function () {
            var proxy = this, leftValue, itemWidth;
            if (!this.model.enabled) return false;
            if (this._next.hasClass("e-disable"))
                this._next.removeClass("e-disable").addClass("e-enable").attr({ "aria-disabled": false });
            leftValue = Math.round(this._thumbItems.position().left);
            itemWidth = this._thumbItems.children('li').outerWidth(true);
            if ((-leftValue) >= itemWidth) this._thumbMove(-itemWidth);
            if ((-leftValue) <= itemWidth) this._previous.removeClass("e-enable").addClass("e-disable").attr({ "aria-disabled": true });
        },
        _findLength: function () {
            var proxy = this, len;
            if (this.model.showThumbnail) len = proxy._thumbItems.children('li').length - 1;
            if (this.model.showPager) len = proxy._bullet.children('li').length - 1;
            if (!(this.model.showPager) && !(this.model.showThumbnail))
                len = (this.model.circularMode) ? ((this.element[this.containerCss]()) / this._liSize) - ((this.model.displayItemsCount) * 2) : ((this.element[this.containerCss]()) / this._liSize) - 1;
            return len;
        },

        _changeActiveState: function (action) {
            var proxy = this, index, len;
            index = proxy._findActive();
            len = proxy._findLength();
            proxy._removeActiveItem();
            if (action == "prev") {
                if ((index > 0)) {
                    proxy._setActiveItem((index - 1));
                    if (this.model.animationType != "slideshow") this._setCaptionText((index - 1));
                }
                else {
                    proxy._setActiveItem(len);
                    if (this.model.animationType != "slideshow") this._setCaptionText((len));
                }
            }
            else if (index < len) {
                proxy._setActiveItem((index + 1));
                if ((this.model.animationType != "slideshow"))
                    this._setCaptionText((index + 1));
            }
            else {
                proxy._setActiveItem(0);
                if (this.model.animationType != "slideshow") this._setCaptionText(0);
            }
        },

        _moveSlide: function (move, action) {
            var proxy = this, animationType, animationSpeed, index, len, properties = {};
            animationType = this.model.animationType != "slideshow" ? this.model.animationType : "slideshow";
            animationSpeed = this.model.animationType != "slideshow" ? proxy.model.animationSpeed : 0;
            index = proxy._findActive();
            len = proxy._findLength();
            properties[this.displayCss] = proxy.element.position()[proxy.displayCss];
            properties[this.displayCss] -= move;
            index = action == "next" ? (index < len ? index + 1 : 0) : action == "prev" ? (index > 0 ? index - 1 : len) : 0;
            if (this.model.animationType == "slideshow") { proxy.element.fadeOut(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeOut(proxy.model.animationSpeed / 2); }
            proxy.element.animate(properties, animationSpeed, animationType, function () { if (action == "end") proxy._setBegin(); proxy._animating = false; proxy._setCaptionText(index); proxy._raiseEvent("change"); });
            if (this.model.animationType == "slideshow") { proxy.element.fadeIn(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeIn(proxy.model.animationSpeed / 2); }
        },

        _moveNext: function () {
            var proxy = this, ltValue, endValue, rbValue, move;
            proxy._animating = true;

            if (this.model.circularMode) {
                ltValue = Math.round(proxy.element.position()[proxy.displayCss]);
                endValue = (proxy.model.displayItemsCount * proxy._liSize) - (proxy._liCount * proxy._liSize);
                rbValue = Math.round(proxy.element.position()[proxy.displayCss]) + proxy.element[proxy.containerCss]();
                proxy._calculateItemMove();
                if ((ltValue > endValue) && (rbValue - proxy._containerSize > (proxy._move + parseInt(proxy.model.frameSpace)))) {
                    return { move: proxy._move, action: "next" }
                }
                else {
                    if (rbValue - proxy._containerSize <= (proxy._move + parseInt(proxy.model.frameSpace))) {
                        move = rbValue - proxy._containerSize - parseInt(proxy.model.frameSpace);
                        return { move: move, action: "end" }
                    }
                }

            }
            else {
                ltValue = Math.round(proxy.element.position()[proxy.displayCss]);
                endValue = -(proxy._liCount * proxy._liSize);
                rbValue = Math.round(proxy.element.position()[proxy.displayCss]) + proxy.element[proxy.containerCss]();
                proxy._calculateItemMove();
                if ((ltValue - proxy._liSize) != endValue) {
                    if (rbValue - proxy._containerSize <= (proxy._move + parseInt(proxy.model.frameSpace))) {
                        proxy._move = rbValue - proxy._containerSize - parseInt(proxy.model.frameSpace);
                        proxy._move = proxy._move == 0 ? ltValue : proxy._move;
                    }
                    return { move: proxy._move, action: "next" }
                }
                else {
                    if (rbValue - proxy._containerSize <= (proxy._move + parseInt(proxy.model.frameSpace)))
                        return { move: ltValue, action: "end" }
                }
            }
        },

        _movePrev: function () {
            var proxy = this, ltValue, rbValue, endValue, setValue;
            ltValue = Math.round(proxy.element.position()[proxy.displayCss]);
            rbValue = proxy.element.position()[proxy.displayCss]; +proxy.element[proxy.containerCss]();
            endValue = proxy.element[proxy.containerCss]();
            proxy._calculateItemMove();
            if (this.model.circularMode) {
                if ((rbValue < endValue) && (-ltValue > (proxy.model.displayItemsCount * proxy._liSize)))
                    return { move: -proxy._move, action: "prev" }
                else {
                    setValue = ((proxy.model.displayItemsCount * proxy._liSize) * 2) + ltValue - (proxy._liCount * proxy._liSize);
                    proxy._move = ((proxy._liCount - (proxy.model.displayItemsCount * 2)) % proxy.model.navigateSteps) * proxy._liSize;
                    if (proxy._move == 0) proxy._calculateItemMove();
                    if (proxy.model.orientation == ej.Orientation.Horizontal)
                        proxy.element.css("left", setValue);
                    else
                        proxy.element.css("top", setValue);
                    return { move: -proxy._move, action: "prev" }
                }
            }
            else {
                if ((rbValue < endValue) && (-ltValue >= (proxy._liSize * proxy.model.navigateSteps)))
                    return { move: -proxy._move, action: "prev" }
                else {
                    setValue = ((proxy.model.displayItemsCount * proxy._liSize)) + ltValue - (proxy._liCount * proxy._liSize);
                    proxy._move = ((-ltValue < proxy._move) && (ltValue != 0)) ? -ltValue : setValue;
                    return { move: -proxy._move, action: "prev" }
                }
            }
        },

        _autoPlay: function (e) {
            if (!this.model.enabled) return false;
            var proxy = this, value, move;
            if (this._interval == null) {
                this._setPlay();
                move = function () {
                    proxy._animating = true;
                    value = proxy.model.enableRTL ? proxy._moveNext() : proxy._movePrev();
                    proxy._moveSlide(value.move, value.action);
                    proxy._changeActiveState(value.action);
                    proxy._changeThumbControl(proxy._findActive());
                }
                function SetInterval(move, animationSpeed) {
                    return (setInterval(move, animationSpeed))
                }
                if (!this._animating)
                    this._interval = SetInterval(move, this.transitionDelay);
            }
            else
                this._setPause();
        },

        _nextAction: function () {
            if (!this.model.enabled) return false;
            var proxy = this, interval, next, value;
            next = function () {
                value = proxy._moveNext();
                proxy._moveSlide(value.move, value.action);
                clearInterval(interval);
                proxy._changeActiveState("next");
                proxy._changeThumbControl(proxy._findActive());
            }
            if (this._interval != null)
                this._setPause();

            if (!this._animating) {
                this._animating = true;
                interval = setInterval(next, "");
            }
        },

        _prevAction: function () {
            if (!this.model.enabled) return false;
            var proxy = this, interval, previous, value;
            previous = function () {
                value = proxy._movePrev();
                proxy._moveSlide(value.move, value.action);
                clearInterval(interval);
                proxy._changeActiveState("prev");
                proxy._changeThumbControl(proxy._findActive());
            }
            if (this._interval != null)
                this._setPause();

            if (!this._animating) {
                this._animating = true;
                interval = setInterval(previous, "")
            }
        },

        _touchOperations: function (e) {
            var isThumb = !e.data.control, direction, orient = this.model.orientation;
            if (isThumb)
                direction = e.type == "swipeleft" ? "left" : "right";
            else {
                if ((orient == "horizontal" && e.type == "swipeleft") ||
                    (orient == "vertical" && e.type == "swipeup"))
                    direction = "left";
                else if ((orient == "horizontal" && e.type == "swiperight") ||
                    (orient == "vertical" && e.type == "swipedown"))
                    direction = "right";
            }
            if (direction == "left")
                isThumb ? this._thumbNext() : this.slideNext();
            else if (direction == "right")
                isThumb ? this._thumbPrev() : this.slidePrevious();
        },

        _resizeEvent: function () {
            var index, parent;
            if (!this.model.enabled) return false;
            index = Math.round(-Math.round(this.element.position().left) / this._liSize);
            parent = Math.round(this.element.parents('.e-rotator-wrap').parent().width());
            this._containerWidth = this._convertPercentageToPixel(parent, this._containerPercent);
            this._setSlideWidth(this._calculateValue().toString());
            this._setWidth();
            this.element.css("left", -(index * this._liSize));
            this._setUlInitial();
            if ((this.model.showThumbnail) && (this.model.dataSource != null || this.model.thumbnailSourceID != null))
                this._setThumbProperty();
        },

        _calculateItemMove: function () {
            if (!isNaN(this.model.navigateSteps))
                this._move = this._liSize * this.model.navigateSteps;
            else {
                this.model.navigateSteps = 1;
                this._move = this._liSize * this.model.navigateSteps;
            }
        },

        _thumbClick: function (event) {
            var index;
            var btnStatus = this._autoButton != null ? (this._autoButton.hasClass("play") ? false : true) : (this.model.enableAutoPlay ? true : false);
            if (!this.model.enabled) return false;

            if (this._interval != null)
                this._setPause();

            this._removeActiveItem();
            index = ((event.target.className == 'e-thumb-ele e-thumbhover') || (event.target.className == 'e-thumb-ele')) ? this._thumbItems.children('li').index($(event.target)) :
            this._thumbItems.children('li').index($(event.target).parents('.e-thumb-ele'));
            this._changeThumbControl(index);
            this._setActiveItem(index);
            this._raiseEvent("thumbItemClick");
            if (this.model.circularMode)
                this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.displayItemsCount * this._liSize) + (index * this.model.navigateSteps * this._liSize)), index, btnStatus);
            else
                this._moveSlideContent(this.element.position()[this.displayCss], -((index * this.model.navigateSteps * this._liSize)), index, btnStatus);
        },

        _bulletClick: function (event) {
            var index, target;
            var btnStatus = this._autoButton != null ? (this._autoButton.hasClass("play") ? false : true) : (this.model.enableAutoPlay ? true : false);
            if (!this.model.enabled) return false;
            if (this._interval != null)
                this._setPause();
            this._removeActiveItem();
            index = this._bullet.children('').index(event.target);
            this._changeThumbControl(index);
            this._setActiveItem(index);
            if (this.model.circularMode)
                this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.displayItemsCount * this._liSize) + (index * this.model.navigateSteps * this._liSize)), index, btnStatus);
            else {
                target = (index * this.model.navigateSteps * this._liSize);
                if ((this.element[this.containerCss]()) - (this.model.displayItemsCount * this._liSize) < target)
                    target = (this.element[this.containerCss]()) - (this.model.displayItemsCount * this._liSize);
                this._moveSlideContent(this.element.position()[this.displayCss], -target, index, btnStatus);
            }
            this._raiseEvent("pagerClick");

        },

        _moveSlideContent: function (Current, Target, index, btnStatus) {
            var proxy = this, animationSpeed, animationType, properties = {};
            animationType = this.model.animationType != "slideshow" ? this.model.animationType : "slideshow";
            animationSpeed = this.model.animationType != "slideshow" ? proxy.model.animationSpeed : 0;
            properties[this.displayCss] = proxy.element.position()[proxy.displayCss];
            properties[this.displayCss] += (Target - Current);
            if (this.model.animationType == "slideshow") { proxy.element.fadeOut(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeOut(proxy.model.animationSpeed / 2); }
            proxy.element.animate(properties, animationSpeed, animationType, function () {
                proxy._setCaptionText(index); if (btnStatus) {
                    setTimeout(function () {
                        proxy.play();
                    }, proxy.model.animationSpeed);
                }
            });
            if (this.model.animationType == "slideshow") { proxy.element.fadeIn(proxy.model.animationSpeed / 2); if (proxy.model.showCaption) proxy._caption.fadeIn(proxy.model.animationSpeed / 2); }
            if (this._prevIndex != index)
                proxy._raiseEvent("change");
            this._prevIndex = index;
        },

        _destroy: function () {
            this._interval != null ? this.pause() : "";
            this._undoClone();
            this.element.removeAttr("style").insertAfter(this._outerWrapper);
            this.element.children().removeAttr("style");
            this._outerWrapper.remove();
        },

        _raiseEvent: function (event) {
            return this._trigger(event, {
                activeItemIndex: this.getIndex(),
                itemID: this.element.attr('id'),
            });

        },

        _showControl: function (event) {
            if (!this.model.enabled) return false;
            if (this.model.showNavigateButton) {
                this._prevButton.stop(true, true).fadeIn('slow');
                this._nextButton.stop(true, true).fadeIn('slow');
            }
            if (this.model.showPlayButton)
                this._autoButton.stop(true, true).fadeIn('slow');
            if ((this.model.showCaption) && (this._caption != null))
                this._caption.stop(true, true).fadeIn('slow');
            if (this.model.stopOnHover && this._animate)
                this.pause();
            event.stopPropagation();
        },

        _hideControl: function (event) {
            if (!this.model.enabled) return false;
            if (this.model.showNavigateButton) {
                this._prevButton.stop(true, true).fadeOut('slow');
                this._nextButton.stop(true, true).fadeOut('slow');
            }
            if (this.model.showPlayButton)
                this._autoButton.stop(true, true).fadeOut('slow');
            if ((this.model.showCaption) && (this._caption != null))
                this._caption.stop(true, true).fadeOut('slow');
            if (this.model.stopOnHover && this._animate)
                this.play();
            event.stopPropagation();
        },

        _onFocusIn: function () {
            if (!this.model.enabled) return false;
            this._outerWrapper.addClass("e-Focused");
            $(this._outerWrapper).bind("keydown", $.proxy(this._focuseHandle, this));
        },

        _onFocusOut: function () {
            this._outerWrapper.removeClass("e-Focused");
            $(this._outerWrapper).unbind("keydown", $.proxy(this._focuseHandle, this));
        },

        _focuseHandle: function (e) {
            if (!this.model.enabled) return false;
            if (this._outerWrapper.hasClass("e-Focused")) {
                e.keyCode = ((e.altKey) && ((e.keyCode == 37) || (e.keyCode == 40))) ? 1000 : e.keyCode;
                e.keyCode = ((e.altKey) && ((e.keyCode == 38) || (e.keyCode == 39))) ? 1001 : e.keyCode;
                switch (e.keyCode) {
                    case 37:
                    case 40:
                        e.preventDefault();
                        this._prevAction();
                        break;
                    case 38:
                    case 39:
                        e.preventDefault();
                        this._nextAction();
                        break;
                    case 32:
                        e.preventDefault();
                        this._autoPlay();
                        break;
                    case 1000:
                        e.preventDefault();
                        this._thumbPrev();
                        this._thumbHover("prev");
                        break;
                    case 1001:
                        e.preventDefault();
                        this._thumbNext();
                        this._thumbHover("next");
                        break;
                    case 13:
                        e.preventDefault();
                        for (i = 0; i < this._thumbItems.children('li').length; i++)
                            if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover'))
                                this.gotoIndex(i);

                }
            }
        },

        _thumbHover: function (action) {
            var proxy = this, index = null, len;
            for (i = 0; i < this._thumbItems.children('li').length; i++) {
                if (this._thumbItems.children('li').eq(i).hasClass('e-thumbhover')) {
                    index = i;
                    this._thumbItems.children('li').eq(i).removeClass('e-thumbhover');
                    break;
                }
            }
            if (index == null)
                index = this._findActive();
            len = proxy._findLength();
            if (action == "prev") {
                if (index > 0)
                    this._thumbItems.children().eq(--index).addClass('e-thumbhover');
                else
                    this._thumbItems.children().eq(0).addClass('e-thumbhover');
            }
            else if (index < len)
                this._thumbItems.children().eq(++index).addClass('e-thumbhover');
            else
                this._thumbItems.children().eq(len).addClass('e-thumbhover');
        },

        _ieEnable: function (action) {
            this.element[action]("e-disable");
            this.model.showPager && this._bulletWrapper.children('ul.e-bullet')[action]('e-disable')
            this.model.showThumbnail && this._thumbItems.children('li.e-thumb-ele')[action]("e-disable");
            if (this.model.showThumbnail) {
                this._next[action]('e-disable');
                this._previous[action]('e-disable');
            }
        },


        enable: function () {
            if (!this.model.enabled) {
                this.model.enabled = true;
                this._outerWrapper.removeClass("e-disable").attr({ "aria-disabled": false });
                if ((ej.browserInfo().name == "msie") && (ej.browserInfo().version == "8.0"))
                    this._ieEnable("removeClass");
                if (this._isPlaying) this.play();
            }
        },

        disable: function () {
            if (this.model.enabled) {
                this.model.enabled = false;
                this._outerWrapper.addClass("e-disable").attr({ "aria-disabled": true });
                if ((ej.browserInfo().name == "msie") && (ej.browserInfo().version == "8.0"))
                    this._ieEnable("addClass");
                this._isPlaying = this._interval != null ? true : false;
                if (this._interval != null)
                    this._setPause();
            }
        },

        slideNext: function () {
            this._nextAction();
        },

        slidePrevious: function () {
            this._prevAction();
        },

        play: function () {
            if (this._interval == null)
                this._autoPlay();
        },

        pause: function () {
            if (this._interval != null)
                this._setPause();
        },

        getIndex: function () {
            var index, i, slideIndex = [];
            if (this.model.displayItemsCount > 1) {
                if (this.model.circularMode) {
                    index = Math.round(-Math.round(this.element.position()[this.displayCss]) / this._liSize);
                    index = index - this.model.displayItemsCount;
                    for (i = 0; i < this.model.displayItemsCount ; i++) {
                        if (index < (this._liCount - (this.model.displayItemsCount * 2)))
                            slideIndex.push(index++);
                        else {
                            index = 0; i--;
                        }
                    }
                }
                else {
                    index = Math.round(Math.round(this.element.position()[this.displayCss]) / this._liSize);
                    index = index < 0 ? (index * -1) : index;
                    for (i = 0; i < this.model.displayItemsCount ; i++)
                        slideIndex.push(index++);
                }
                return slideIndex;
            }
            else
                return this._findActive();
        },

        gotoIndex: function (index) {
            var btnStatus = this._autoButton != null ? (this._autoButton.hasClass("play") ? false : true) : (this.model.enableAutoPlay ? true : false);
            if (this._interval != null)
                this._setPause();
            this._removeActiveItem();
            if (this.model.showThumbnail) this._changeThumbControl(index);
            this._setActiveItem(index);
            if (this.model.circularMode)
                this._moveSlideContent(this.element.position()[this.displayCss], -((this.model.displayItemsCount * this._liSize) + (index * this.model.navigateSteps * this._liSize)), index, btnStatus);
            else {
                target = ((index * this.model.navigateSteps * this._liSize));
                if ((this.element[this.containerCss]()) - (this.model.displayItemsCount * this._liSize) < target)
                    target = (this.element[this.containerCss]()) - (this.model.displayItemsCount * this._liSize);
                this._moveSlideContent(this.element.position()[this.displayCss], -target, index, btnStatus);
            }
        },
    })

    ej.Rotator.PagerPosition = {
        /**  Pager is positioned at the top-left corner of an Item. */
        TopLeft: "topleft",
        /**  Pager is positioned at the top-left corner of an Item. */
        TopRight: "topright",
        /**  Pager is positioned at the top-right corner of an Item. */
        BottomLeft: "bottomleft",
        /**  Pager is positioned at the bottom-left corner of an Item. */
        BottomRight: "bottomright",
        /**  Pager is positioned at the bottom-right corner of an Item. */
        TopCenter: "topCenter",
        /**  Pager is positioned outside an Item. */
        Outside: "outside"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Div elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejSplitter", "ej.Splitter", {

        element: null,

        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-splitter",
        _setFirst: false,
        angular: {
            terminal: false
        },

        defaults: {

            cssClass: "",

            orientation: "horizontal",

            enableAnimation: true,

            properties: [],

            height: null,

            width: null,

            enableAutoResize: false,

            enableRTL: false,

            htmlAttributes: {},

            animationSpeed: 300,

            beforeExpandCollapse: null,

            expandCollapse: null,

            resize: null,

            create: null,

            destroy: null
        },

        dataTypes: {
            cssClass: "string",
            orientation: "enum",
            properties: "data",
            enableAutoResize: "boolean",
            enableRTL: "boolean",
            animationSpeed: "number",
            enableAnimation: "boolean",
            htmlAttributes: "data",
        },


        _init: function () {
            this._initialize();
            this._render();
            this._wireEvents(this.model.enableAutoResize);
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "enableAutoResize": this._windowResizing(options[option]); break;
                    case "enableRTL": this._rtl(options[option]); break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                    case "orientation": this._setOrientation(options[option]); break;
                }
            }
        },

        refresh: function () {
            this._setPanesSize();
            this._getPanesPercent();
        },

        collapse: function (paneIndex) {
            this._clickArrow(paneIndex, true, true);
        },

        expand: function (paneIndex) {
            this._clickArrow(paneIndex, false, true);
        },

        _clickArrow: function (index, bool, canClick) {
            if (this._inMovement || index < 0 || index > this.panes.length || this.panes.length <= 1) return false;
            var arrow, cls = bool ? "e-collapse" : "e-expand", splitbars = this.element.children(".e-splitbar:not(.e-shadowbar)");
            if (index == splitbars.length) arrow = this._clickArrow(index - 1, !bool, false);
            else arrow = $(splitbars[index]).children("." + cls);
            if (canClick) arrow.css("display") != "none" && arrow.mousedown();
            else return arrow;
        },


        addItem: function (content, property, index) {
            var paneCount = this.panes.length;
            index = this._getNumber(index);
            if (ej.isNullOrUndefined(index)) index = paneCount;
            if (index < 0 || index > paneCount) return "";
            var property = this._getPaneProperty(property), paneDiv, paneDivSize, requiredSize;
            property = this._checkMinMaxSize(property);
            paneDiv = ej.buildTag("div.e-pane e-" + this.model.orientation.substr(0, 1) + "-pane");
            this.element.append(paneDiv[this.containerCss](property.paneSize));
            paneDivSize = property.paneSize = paneDiv[this.containerCss]();
            paneDiv.remove();
            var start = index, end = paneCount, i, j, insert, before, direction = 2, getters = {}, taken = 0, canInsert = false;
            requiredSize = paneDivSize + this._bar;

            for (j = 0; j < direction; j++) {
                for (i = start; i < end; i++) {
                    var _paneSize = $(this.panes[i])[this.containerCss]();
                    var availableSpace = _paneSize - this.model.properties[i].minSize;
                    if (availableSpace >= requiredSize - taken) {
                        getters[i] = _paneSize - (requiredSize - taken);
                        canInsert = true;
                        break;
                    }
                    else if (availableSpace > 0) {
                        getters[i] = this.model.properties[i].minSize;
                        taken += availableSpace;
                    }
                }
                if (canInsert) break;
                else end = start, start = 0;
            }
            if (!canInsert) return "";
            for (var pos in getters)
                $(this.panes[pos])[this.containerCss](getters[pos]);
            if (index == paneCount) {
                insert = "insertBefore", before = 1;
                paneDiv.insertAfter($(this.panes[index - 1]));
            }
            else {
                insert = "insertAfter", before = 0;
                paneDiv.insertBefore($(this.panes[index]));
            }
            this.model.properties.splice(index, 0, property);
            this.panes.splice(index, 0, paneDiv);
            var splitBar = this._createSplitBar(index - before);
            splitBar[insert](paneDiv);
            paneDiv.append(content);
            this._updateModel();
            return paneDiv;
        },


        removeItem: function (index) {
            var paneCount = this.panes.length - 1;
            index = this._getNumber(index);
            if (ej.isNullOrUndefined(index)) index = paneCount;
            if (index < 0 || index > paneCount || paneCount < 1) return null;
            var targetPane, nextPane, splitbars, removedSize;
            targetPane = $(this.panes[index]);
            removedSize = targetPane[this.containerCss]() + this._bar;
            targetPane.remove();
            splitbars = this.element.children(".e-splitbar:not(.e-shadowbar)");
            if (index == paneCount) {
                nextPane = $(this.panes[index - 1]);
                $(splitbars[index - 1]).remove();
            }
            else {
                nextPane = $(this.panes[index + 1]);
                $(splitbars[index]).remove();
            }
            nextPane[this.containerCss](nextPane[this.containerCss]() + removedSize);
            this._removeArrays(index);
            this._updateModel();
        },
        _checkMinMaxSize: function (property) {
            if (property.paneSize < property.minSize)
                property.paneSize = property.minSize;
            if (property.paneSize > property.maxSize)
                property.paneSize = property.maxSize;
            return property;
        },

        _removeArrays: function (index) {
            this.model.properties.splice(index, 1);
            this.panes.splice(index, 1);
            this.oldPaneSize.splice(index, 1);
            this.oldPanePercent.splice(index, 1);
            this._sizePercent.splice(index, 1);
        },

        _getNumber: function (value) {
            value = parseFloat(value);
            return isNaN(value) ? null : value;
        },

        _updateModel: function () {
            for (var i = 0; i < this.panes.length; i++)
                this.model.properties[i].paneSize = $(this.panes[i])[this.containerCss]();
            this._getPanesPercent();
        },

        _getPaneProperty: function (property) {
            var _default = { paneSize: 10, minSize: 10, maxSize: null, collapsible: true, resizable: true };
            return $.extend(_default, property);
        },

        _changeSkin: function (skin) {
            this.element.removeClass(this.model.cssClass).addClass(skin);
        },

        _windowResizing: function (boolean) {
            if (boolean) this._wireEvents(boolean);
            else this._unWireEvents();
        },

        _setOrientation: function (boolean) {
            this._unWireEvents();
            this._destroy();
            this.model.orientation = boolean;
            this._init();
        },


        _destroy: function () {
            this.element.removeClass("e-widget e-box " + this.model.cssClass + " e-" + this.model.orientation);
            this.element.children(".e-splitbar").remove();
            this.element.children(".e-pane").removeClass("e-pane e-" + this.model.orientation.substr(0, 1) + "-pane").height("").width("");
        },

        _initialize: function () {
            this.panes = [];
            this.oldPaneSize = [];
            this.oldPanePercent = [];
            this.shadowBar = null;
            this._inMovement = false;
            this.containerCss = this.model.orientation == "horizontal" ? "width" : "height";
            this.displayCss = this.model.orientation == "horizontal" ? "left" : "top";
            this.borderCss = this.model.orientation == "horizontal" ? "right" : "bottom";
            this._bar = 9;      // For the center splitbar size
        },

        _render: function () {
            this.element.addClass("e-widget e-box " + this.model.cssClass + " e-" + this.model.orientation).attr("data-role", "splitter");
            var i, j, target = this.element[0];
            for (i = 0, j = 0; i < target.children.length; i++) {
                $(target.children[i]).addClass("e-pane");
                this.panes.push(target.children[i]);
            }
            this._setPanesProperty();
            this._insertSplitBar();
            this._setDimentions();
            this._setPanesSize();
            this._getPanesPercent();
            this._addAttr(this.model.htmlAttributes);
            this._checkProperties();
            this.element.find(".e-pane").addClass("e-" + this.model.orientation.substr(0, 1) + "-pane");
        },

        _setPanesProperty: function () {
            for (var p = 0; p < this.panes.length; p++) {
                if (this.model.properties[p] != undefined) {
                    this.model.properties[p].paneSize = this.model.properties[p].paneSize == undefined ? "0px" : this.model.properties[p].paneSize;
                    this.model.properties[p].minSize = isNaN(parseFloat(this.model.properties[p].minSize)) ? 10 : parseFloat(this.model.properties[p].minSize);
                    this.model.properties[p].maxSize = isNaN(parseFloat(this.model.properties[p].maxSize)) ? null : parseFloat(this.model.properties[p].maxSize);
                    this.model.properties[p].collapsible = this.model.properties[p].collapsible != false ? true : false;
                    this.model.properties[p].resizable = this.model.properties[p].resizable != false ? true : false;
                }
                else this.model.properties.push({ paneSize: "0px", minSize: 10, maxSize: null, collapsible: true, resizable: true });
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else proxy.element.attr(key, value)
            });
        },


        _insertSplitBar: function () {
            if (this.panes.length > 1) {
                var i, splitBar;
                for (i = 0; i < this.panes.length - 1; i++) {
                    splitBar = this._createSplitBar(i);
                    splitBar.insertAfter(this.panes[i]);
                }
            }
        },

        _createSplitBar: function (i) {
            var orient = this.model.orientation.substr(0, 1), arrow1, arrow2, splitBar, accessible = false;
            splitBar = ej.buildTag("span.e-box e-splitbar e-split-divider e-" + orient + "-bar").attr("aria-expanded", true);
            if (this.model.properties[i].collapsible && this.model.properties[i + 1].collapsible) {
                arrow1 = ej.buildTag("span.e-icon e-collapse e-" + orient + "-arrow " + orient + "-backward");
                splitBar.append(arrow1);
                arrow2 = ej.buildTag("span.e-icon e-expand e-" + orient + "-arrow " + orient + "-forward");
                splitBar.append(arrow2);
                accessible = true;
                this._on(arrow1, "mousedown", this._collapseArrowClick);
                this._on(arrow2, "mousedown", this._expandArrowClick);
            }
            if (this.model.properties[i].resizable && this.model.properties[i + 1].resizable) {
                splitBar.addClass("e-resize");
                accessible = true;
                this._on(splitBar, ej.eventType.mouseDown, this._mouseDownOnDivider);
            }
            if (accessible) {
                splitBar.attr({ "role": "separator", "tabindex": "0" });
                this._on(splitBar, "focus focusout", this._focusOnDivider);
            }
            else splitBar.attr({ "role": "presentation" });
            return splitBar;
        },

        _getPanesPercent: function () {
            this._sizePercent = [];
            var size = this.element[this.containerCss](), outerSize = size - ((this.panes.length - 1) * this._bar), i;
            for (i = 0; i < this.panes.length; i++) {
                if (!$(this.panes[i]).hasClass("collapsed"))
                    this.oldPaneSize[i] = $(this.panes[i])[this.containerCss]();
                this.oldPanePercent[i] = this._convertToPercent(outerSize, this.oldPaneSize[i]);
                this._sizePercent.push(this._convertToPercent(outerSize, $(this.panes[i])[this.containerCss]()));
            }
        },

        _setDimentions: function () {
            if (this.model.height)
                this.element.css("height", this.model.height);
            if (this.model.width)
                this.element.css("width", this.model.width);
        },

        _checkProperties: function () {
            if (this.model.enableRTL) this._rtl(this.model.enableRTL);
            this._prevSize = this.element[this.containerCss]();
        },
        _getExactInnerWidth: function () {
            var browser = ej.browserInfo(), exactInnerWidth;
            if (browser.name == "msie") {
                if (browser.version == 8)
                    exactInnerWidth = $(this.element)[this.containerCss]();
                else
                    exactInnerWidth = parseFloat(window.getComputedStyle(this.element[0])[this.containerCss]);
            }
            else
                exactInnerWidth = parseFloat(window.getComputedStyle(this.element[0])[this.containerCss])
                                 - (parseFloat(this.element.css("border-" + this.displayCss + "-width")) +
                                     parseFloat(this.element.css("border-" + this.borderCss + "-width")) +
                                     parseFloat(this.element.css("padding-" + this.displayCss)) +
                                     parseFloat(this.element.css("padding-" + this.borderCss)));
            return exactInnerWidth;
        },

        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },

        _setPanesSize: function () {
            var attr = this.containerCss,
            zeroPanes = 0,
            totalPaneSize = 0,
            totalSize = this.element[attr](),
            remainZero = false,
            bar = this._bar,
            zerothPanes = [],
            panLength, j, paneCount = this.panes.length;

            if (paneCount > 1) {
                for (j = 0; j < paneCount ; j++) {
                    $(this.panes[j]).css(attr, this.model.properties[j].paneSize);
                    bar = (j == paneCount - 1) ? 0 : bar;
                    panLength = Math.round($(this.panes[j])[attr]());
                    $(this.panes[j]).css(attr, panLength);
                    if (panLength <= 0) {
                        zeroPanes++;
                        zerothPanes.push(j);
                        totalPaneSize += bar;
                    }
                    else {
                        if (remainZero) {
                            $(this.panes[j]).css(attr, 0);
                            totalPaneSize += panLength + bar;
                            this.model.properties[j].paneSize = 0;
                        }
                        else {
                            totalPaneSize += panLength + bar;
                            if (totalPaneSize > totalSize) {
                                totalPaneSize -= panLength - bar;
                                var currPane = totalSize - totalPaneSize,
                                remainDivider = paneCount - j - 1;
                                currPane -= remainDivider * bar;
                                $(this.panes[j]).css(attr, currPane);
                                remainZero = true;
                                totalPaneSize += currPane + bar;
                            }
                            this.model.properties[j].paneSize = panLength;
                        }
                    }
                }
            }
            else if (paneCount == 1) {
                $(this.panes[0]).css(attr, "100%");
                totalPaneSize = totalSize;
            }

            if (paneCount > 1 && totalPaneSize != totalSize) {
                var remainingSize, lastPane = $(this.panes[paneCount - 1]);
                if (totalPaneSize > totalSize) {
                    remainingSize = totalPaneSize - totalSize;
                    lastPane.css(attr, remainingSize);
                }
                else if (totalPaneSize < totalSize) {
                    remainingSize = totalSize - totalPaneSize;
                    if (zeroPanes > 0) {
                        var z, avgWid = Math.round(remainingSize / zeroPanes);
                        for (z = 0; z < zeroPanes ; z++) {
                            $(this.panes[zerothPanes[z]]).css(attr, avgWid);
                            this.model.properties[zerothPanes[z]].paneSize = avgWid;
                        }
                    }
                    else {
                        lastPane.css(attr, Math.round(lastPane[attr]() + remainingSize));
                        this.model.properties[paneCount - 1].paneSize = lastPane[attr]();
                    }
                }
            }
            if (paneCount > 1) this._checkPaneSize();
        },

        _getUnit: function (str) {
            if (str == "px") return "px";
            else if (str == "pt") return "pt";
            else if (str.substr(1) == "%") return "%";
            else return "px";
        },

        _getNormalValue: function (position) {
            var currentLOB, currentLOBPercent, totalValue, currentValue;
            if (this.model.orientation == "vertical") {
                currentLOB = position.y - this.element.offset().top;
                currentLOBPercent = currentLOB / this.element.outerHeight();
                totalValue = this.element.height();
            }
            else {
                currentLOB = position.x - this.element.offset().left;
                currentLOBPercent = currentLOB / this.element.outerWidth();
                totalValue = this.element.width();
            }
            if (currentLOBPercent > 1) {
                currentLOBPercent = 1;
            }
            if (currentLOBPercent < 0) {
                currentLOBPercent = 0;
            }
            currentValue = currentLOBPercent * totalValue;
            return this._trimValue(currentValue);
        },

        _trimValue: function (value) {
            var step, stepModValue, correctedValue;
            step = 1;
            stepModValue = (value) % step;
            correctedValue = value - stepModValue;
            if (Math.abs(stepModValue) * 2 >= step)
                correctedValue += (stepModValue > 0) ? step : (-step);
            return parseFloat(correctedValue.toFixed(5));
        },

        _getSplitbarIndex: function () {
            return this.element.children(".e-splitbar:not(.e-shadowbar)").index(this.currentSplitBar);
        },

        _paneResize: function () {
            if (this.shadowBar == null) return false;
            this.currentSplitBar = this.shadowBar.next();
            var newPosition, prevPane, nextPane, prevPaneIndex, nextPaneIndex, index = this._getSplitbarIndex();
            prevPane = this.shadowBar.prev(), nextPane = this.currentSplitBar.next();
            prevPaneIndex = index, nextPaneIndex = index + 1;
            newPosition = this.shadowBar.offset()[this.displayCss];
            newPosition = newPosition - this.currentSplitBar.offset()[this.displayCss];
            $(prevPane).css(this.containerCss, newPosition + $(prevPane)[this.containerCss]() + "px");
            $(nextPane).css(this.containerCss, $(nextPane)[this.containerCss]() - newPosition + "px");
            this.oldPaneSize[prevPaneIndex] = $(prevPane)[this.containerCss]();
            this.oldPaneSize[nextPaneIndex] = $(nextPane)[this.containerCss]();
            this.shadowBar.remove();
            this._checkPaneSize();
            var prevObj = { item: prevPane, index: prevPaneIndex, size: this.oldPaneSize[prevPaneIndex] };
            var nextObj = { item: nextPane, index: nextPaneIndex, size: this.oldPaneSize[nextPaneIndex] };
            this._updateModelValue(prevObj, nextObj);
            this._trigger("resize", {
                prevPane: prevObj,
                nextPane: nextObj,
                splitbarIndex: index
            });
        },

        _checkPaneSize: function () {
            var total = 0, len, i, splitterLen, paneCount = this.panes.length;
            for (i = 0; i < paneCount; i++) {
                len = $(this.panes[i])[this.containerCss]();
                total += len + this._bar;
            }
            total -= this._bar;
            splitterLen = this._getExactInnerWidth();
            if (total != splitterLen) {
                var remain = splitterLen - total;
                var last = $(this.panes[paneCount - 1])[this.containerCss]();
                if (last == 0) {
                    for (i = paneCount - 1; i >= 0; i--) {
                        if ($(this.panes[i]).hasClass("expanded") && !$(this.panes[i]).hasClass("collapsed")) {
                            last = $(this.panes[i])[this.containerCss]();
                            $(this.panes[i]).css(this.containerCss, Math.floor(last + remain));
                            break;
                        }
                    }
                }
                else
                    $(this.panes[paneCount - 1]).css(this.containerCss, Math.floor(last + remain));
            }
        },

        _maxminDraggableRange: function (shadowbarPos) {
            var prevPane, nextPane, prevPaneSize, nextPaneSize, splitbarPosition, prevPaneRange, nextPaneRange,
                prevPaneIndex, nextPaneIndex, PaneMax1, PaneMax2, PaneMin1, PaneMin2, index;
            prevPane = this.shadowBar.prev();
            this.currentSplitBar = this.shadowBar.next();
            nextPane = this.currentSplitBar.next();
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();
            splitbarPosition = this.displayCss == "left" ? this.currentSplitBar[0].offsetLeft : this.currentSplitBar[0].offsetTop;
            prevPaneRange = splitbarPosition - prevPaneSize;
            nextPaneRange = nextPaneSize + splitbarPosition;
            index = this._getSplitbarIndex();
            prevPaneIndex = index;
            nextPaneIndex = index + 1;
            PaneMax1 = this.model.properties[prevPaneIndex].maxSize;
            PaneMax2 = this.model.properties[nextPaneIndex].maxSize;
            PaneMax1 = PaneMax1 != null ? parseInt(PaneMax1, 10) : null;
            PaneMax2 = PaneMax2 != null ? parseInt(PaneMax2, 10) : null;
            this.model.properties[prevPaneIndex].minSize = parseInt(this.model.properties[prevPaneIndex].minSize, 10);
            this.model.properties[nextPaneIndex].minSize = parseInt(this.model.properties[nextPaneIndex].minSize, 10);
            PaneMin1 = this.model.properties[prevPaneIndex].minSize;
            PaneMin2 = this.model.properties[nextPaneIndex].minSize;
            this.shadowBar.removeClass("e-end-indicaton");
            if (shadowbarPos > nextPaneRange - PaneMin2) {
                this.resizedPosition = nextPaneRange - PaneMin2;
                this.shadowBar.addClass("e-end-indicaton");
            }
            else if (shadowbarPos < prevPaneRange + PaneMin1) {
                this.resizedPosition = prevPaneRange + PaneMin1;
                this.shadowBar.addClass("e-end-indicaton");
            }
            if (PaneMax1 != null) {
                if (shadowbarPos > prevPaneRange + PaneMax1) {
                    this.resizedPosition = prevPaneRange + PaneMax1;
                    this.shadowBar.addClass("e-end-indicaton");
                }
            }
            if (PaneMax2 != null) {
                if (shadowbarPos < nextPaneRange - PaneMax2) {
                    this.resizedPosition = nextPaneRange - PaneMax2;
                    this.shadowBar.addClass("e-end-indicaton");
                }
            }
        },

        _collapseArrowClick: function (event) {
            if (this.shadowBar != null) return;
            var $target = $(event.target);
            this._inMovement = true;
            this.currentSplitBar = $target.parent();
            var currBarNo, prevPane, nextPane, prevPaneIndex, nextPaneIndex, prevPaneSize, nextPaneSize, properties = {};
            var paneCount = this.panes.length;
            currBarNo = this._getSplitbarIndex();
            prevPane = this.currentSplitBar.prev();
            nextPane = this.currentSplitBar.next();
            prevPaneIndex = currBarNo;
            nextPaneIndex = currBarNo + 1;
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();
            var proxy = this, collapsed, expanded;
            collapsed = { item: prevPane, index: prevPaneIndex, size: prevPaneSize };
            expanded = { item: nextPane, index: nextPaneIndex, size: nextPaneSize };
            if (this._raiseEvent("beforeExpandCollapse", collapsed, expanded, currBarNo))
                return false;
            if (!nextPane.hasClass("collapsed")) {
                this.oldPaneSize[prevPaneIndex] = prevPaneSize;
                prevPane.addClass("collapsed");
                nextPane.addClass("expanded");
                this.currentSplitBar.attr("aria-expanded", false);
                $target.parent().removeClass("e-resize");
                $target.css("display", "none");
                if (prevPaneIndex != 0) {
                    var preBar = prevPane.prev();
                    preBar.find(".e-expand").css("display", "none");
                    preBar.removeClass("e-resize");
                }
                properties[this.containerCss] = 0;
                prevPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0);
                properties[this.containerCss] = prevPaneSize + nextPaneSize;
                nextPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0, function () {
                    proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                });
            }
            else {
                if (prevPaneSize < this.oldPaneSize[nextPaneIndex]) {
                    $target.addClass("e-end-indicaton");
                    this._inMovement = false;
                    $(document).bind("mouseup", $.proxy(this._mouseUpOnArrow, this));
                    return false;
                }
                else {
                    prevPane.removeClass("expanded");
                    nextPane.removeClass("collapsed");
                    $target.parent().addClass("e-resize");
                    $target.siblings().css("display", "block");

                    if (nextPaneIndex != paneCount - 1) {
                        var nextBar = nextPane.next();
                        nextBar.find(".e-collapse").css("display", "block");
                        if (!nextBar.next().hasClass("collapsed")) {
                            nextBar.addClass("e-resize");
                            nextBar.attr("aria-expanded", true);
                        }
                    }

                    properties[this.containerCss] = this.oldPaneSize[nextPaneIndex];
                    nextPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0);
                    properties[this.containerCss] = prevPaneSize - this.oldPaneSize[nextPaneIndex];
                    prevPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0, function () {
                        proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                    });
                }
            }
        },

        _expandArrowClick: function (event) {
            if (this.shadowBar != null) return;
            var $target = $(event.target);
            this._inMovement = true;
            this.currentSplitBar = $target.parent();
            var currBarNo, prevPane, nextPane, prevPaneIndex, nextPaneIndex, prevPaneSize, nextPaneSize, properties = {};
            var paneCount = this.panes.length;
            currBarNo = this._getSplitbarIndex();
            prevPane = this.currentSplitBar.prev();
            nextPane = this.currentSplitBar.next();
            prevPaneIndex = currBarNo;
            nextPaneIndex = currBarNo + 1;
            prevPaneSize = prevPane[this.containerCss]();
            nextPaneSize = nextPane[this.containerCss]();
            var proxy = this, collapsed, expanded;
            collapsed = { item: nextPane, index: nextPaneIndex, size: nextPaneSize };
            expanded = { item: prevPane, index: prevPaneIndex, size: prevPaneSize };

            if (this._raiseEvent("beforeExpandCollapse", collapsed, expanded, currBarNo))
                return false;
            if (!prevPane.hasClass("collapsed")) {
                this.oldPaneSize[nextPaneIndex] = nextPaneSize;
                prevPane.addClass("expanded");
                nextPane.addClass("collapsed");
                $target.parent().removeClass("e-resize");
                $target.css("display", "none");
                if (nextPaneIndex != paneCount - 1) {
                    var nextBar = nextPane.next();
                    nextBar.find(".e-collapse").css("display", "none");
                    nextBar.removeClass("e-resize");
                    nextBar.attr("aria-expanded", false);
                }
                properties[this.containerCss] = prevPaneSize + nextPaneSize;
                prevPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0);
                properties[this.containerCss] = 0;
                nextPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0, function () {
                    proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                });
            }
            else {
                if (nextPaneSize < this.oldPaneSize[prevPaneIndex]) {
                    $target.addClass("e-end-indicaton");
                    this._inMovement = false;
                    $(document).bind("mouseup", $.proxy(this._mouseUpOnArrow, this));
                    return false;
                }
                else {
                    prevPane.removeClass("collapsed");
                    nextPane.removeClass("expanded");
                    this.currentSplitBar.attr("aria-expanded", true);
                    $target.parent().addClass("e-resize");
                    $target.siblings().css("display", "block");

                    if (prevPaneIndex != 0) {
                        var preBar = prevPane.prev();
                        preBar.find(".e-expand").css("display", "block");
                        if (!preBar.prev().hasClass("collapsed")) preBar.addClass("e-resize");
                    }

                    properties[this.containerCss] = this.oldPaneSize[prevPaneIndex];
                    prevPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0);
                    properties[this.containerCss] = nextPaneSize - this.oldPaneSize[prevPaneIndex];
                    nextPane.animate(properties, this.model.enableAnimation ? this.model.animationSpeed : 0, function () {
                        proxy._raiseEvent("expandCollapse", collapsed, expanded, currBarNo);
                    });
                }
            }
        },

        _raiseEvent: function (evtName, collapsed, expanded, index) {
            if (evtName == "expandCollapse") {
                this._inMovement = false;
                this._updateModelValue(collapsed, expanded);
            }
            return this._trigger(evtName, {
                collapsed: collapsed,
                expanded: expanded,
                splitbarIndex: index
            });

        },

        _updateModelValue: function (collapsed, expanded) {
            this.model.properties[collapsed.index].paneSize = collapsed.item[this.containerCss]();
            this.model.properties[expanded.index].paneSize = expanded.item[this.containerCss]();
            this._getPanesPercent();
        },

        _mouseUpOnArrow: function (event) {
            this.element.find(".e-end-indicaton").removeClass("e-end-indicaton");
            $(document).unbind("mouseup", $.proxy(this._mouseUpOnArrow, this));
        },

        _keydownOnDivider: function (e) {
            var key = e.keyCode;
            if (key == 37 || key == 38 || key == 39 || key == 40) {
                e.preventDefault();
                var oriTarget = $(e.data.target);
                if (e.ctrlKey) {
                    if (this.shadowBar == null) {
                        this.currentSplitBar = oriTarget;
                        var index = this._getSplitbarIndex();
                        if (this.model.orientation == "vertical") {
                            if (e.keyCode == 38) this.collapse(index);
                            else if (e.keyCode == 40) this.expand(index);
                        }
                        else {
                            if (e.keyCode == 37) this.collapse(index);
                            else if (e.keyCode == 39) this.expand(index);
                        }
                    }
                }
                else if (oriTarget.hasClass("e-resize")) {
                    var target = (this.shadowBar != null) ? this.shadowBar : oriTarget;
                    var offset = target.offset(), location = { pageX: offset.left, pageY: offset.top };
                    $.extend(true, e, location);
                    if ((this.model.orientation == "vertical" && (e.keyCode == 38 || e.keyCode == 40)) ||
                        (this.model.orientation == "horizontal" && (e.keyCode == 37 || e.keyCode == 39))) {
                        if (e.keyCode == 38) e.pageY -= 5;
                        else if (e.keyCode == 40) e.pageY += 5;
                        else if (e.keyCode == 37) e.pageX -= 5;
                        else if (e.keyCode == 39) e.pageX += 5;
                        this._mouseMoveOnDivider(e);
                    }
                }
            }
            else if (key == 13) {
                e.preventDefault();
                this._mouseUpOnDivider();
            }
            else if (key == 27) {
                e.preventDefault();
                if (this.shadowBar != null) this.shadowBar.remove();
                this.shadowBar = null;
                this._mouseUpOnDivider();
                this.element.children(".e-splitbar.e-hover").focusout();
            }
        },

        _focusOnDivider: function (e) {
            if (e.type == "focus") {
                if (!$(e.target).hasClass("e-hover")) {
                    $(e.target).addClass("e-hover");
                    $(document).bind("keydown", { target: e.target }, $.proxy(this._keydownOnDivider, this));
                }
            }
            else {
                this.element.children(".e-splitbar.e-hover").removeClass("e-hover");
                this._mouseUpOnDivider();
                $(document).unbind("keydown", $.proxy(this._keydownOnDivider, this));
            }
        },

        _mouseDownOnDivider: function (event) {
            event.preventDefault();
            var $target = $(event.target);
            if ($target.hasClass("e-splitbar") && $target.hasClass("e-resize")) {
                this._overlayElement = ej.buildTag('div.e-pane-overlay');
                if (!$target.hasClass("e-hover")) $target.focus();
                this.element.find(".e-pane").not(".e-splitter").append(this._overlayElement);
                $(document).bind(ej.eventType.mouseMove, { target: event.target }, $.proxy(this._mouseMoveOnDivider, this));
                $(document).bind(ej.eventType.mouseUp, $.proxy(this._mouseUpOnDivider, this));
                $(document).bind("mouseleave", $.proxy(this._mouseUpOnDivider, this));
            }
            else if ($target.hasClass("e-expand") || $target.hasClass("e-collapse")) {
                $target.parent().focus();
            }
        },

        _mouseMoveOnDivider: function (event) {
            var _data = event.data;
            event = event.type == "touchmove" ? event.originalEvent.changedTouches[0] : event;
            var position = { x: event.pageX, y: event.pageY };
            this.resizedPosition = this._getNormalValue(position);
            if (this.shadowBar == null) {
                var target = $(_data.target);
                this.shadowBar = target.clone().addClass("e-shadowbar").removeClass("e-hover").removeClass("e-split-divider").insertBefore(target);
                this.shadowBar.children().remove();
            }
            this._maxminDraggableRange(this.resizedPosition);
            this.shadowBar.css(this.displayCss, this.resizedPosition);
        },

        _mouseUpOnDivider: function (event) {
            this._paneResize();
            this.element.find(".e-pane").not(".e-splitter").find(".e-pane-overlay").remove();
            $(document).unbind(ej.eventType.mouseMove, $.proxy(this._mouseMoveOnDivider, this));
            $(document).unbind(ej.eventType.mouseUp, $.proxy(this._mouseUpOnDivider, this));
            $(document).unbind("mouseleave", $.proxy(this._mouseUpOnDivider, this));
            // sets shadowBar null after removing shadowBar element
            this.shadowBar = null;
        },

        _windowResized: function (event) {
            var size = this._getExactInnerWidth();
            if (this._prevSize == size) return false;
            var paneCount = this.panes.length, outerSize = size - ((paneCount - 1) * this._bar), i, val;
            this._prevSize = size;
            for (i = 0; i < paneCount; i++) {
                val = this._convertToPixel(outerSize, this._sizePercent[i]);
                $(this.panes[i]).css(this.containerCss, val + "px");
            }
            for (i = 0; i < this.oldPaneSize.length; i++)
                this.oldPaneSize[i] = this._convertToPixel(outerSize, this.oldPanePercent[i]);
            this._checkPaneSize();
        },

        _convertToPercent: function (outer, pane) {
            return (pane * 100) / outer;
        },
        _convertToPixel: function (tot, percent) {
            return Math.floor((tot * percent) / 100);
        },

        _wireEvents: function (boolean) {
            if (boolean) $(window).bind('resize', $.proxy(this._windowResized, this));
        },

        _unWireEvents: function () {
            $(window).unbind('resize', $.proxy(this._windowResized, this));
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the tab control.
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejTab", "ej.Tab", {
        _rootCSS: "e-tab",

        element: null,

        model: null,
        validTags: ["div", "span"],
        _setFirst: false,
        angular: {
            terminal: false
        },

        defaults: {

            collapsible: false,

            enableAnimation: true,

            ajaxSettings: {

                type: 'GET',

                cache: false,

                data: {},

                dataType: "html",

                contentType: "html",

                async: true
            },

            disabledItemIndex: [],

            enabledItemIndex: [],

            events: "click",

            idPrefix: "ej-tab-",

            heightAdjustMode: "content",

            selectedItemIndex: 0,

            cssClass: "",

            showCloseButton: false,

            htmlAttributes: {},

            enableTabScroll: false,

            showReloadIcon: false,

            headerPosition: "top",

            width: null,

            height: null,

            headerSize: null,

            enableRTL: false,

            allowKeyboardNavigation: true,

            showRoundedCorner: false,

            enablePersistence: false,

            enabled: true,

            ajaxLoad: null,

            ajaxBeforeLoad: null,

            ajaxSuccess: null,

            ajaxError: null,

            itemActive: null,

            beforeActive: null,

            itemAdd: null,

            itemRemove: null,

            beforeItemRemove: null,

            create: null,

            destroy: null

        },
        dataTypes: {
            cssClass: "string",
            collapsible: "boolean",
            events: "string",
            heightAdjustMode: "enum",
            enabled: "boolean",
            ajaxSettings: "data",
            disabledItemIndex: "data",
            enabledItemIndex: "data",
            enableAnimation: "boolean",
            htmlAttributes: "data"
        },
        observables: ["selectedItemIndex"],
        selectedItemIndex: ej.util.valueFunction("selectedItemIndex"),


        _destroy: function () {
            this._unWireEvents();
            this._removeBaseClass();
        },

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "events": {
                        this._off(this.anchors, this.model.events);
                        this._on(this.anchors, options[key], this._tabItemClick);
                        break;
                    }
                    case "disabledItemIndex": {
                        this._disableItems(options[key]);
                        options[key] = this.model.disabledItemIndex;
                        break;
                    }
                    case "enabledItemIndex": this._enableItems(options[key]); break;
                    case "enabled": this._enabledAction(options[key]); break;
                    case "selectedItemIndex": this.showItem(ej.util.getVal(options[key]));
                        options[key] = this.model.selectedItemIndex;
                        break;
                    case "heightAdjustMode": {
                        this.model.heightAdjustMode = options[key];
                        this._setTabsHeightStyle(options[key]);
                        this._resizeEvents(options[key]);
                        break;
                    }
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "showRoundedCorner": this._roundedCorner(options[key]); break;
                    case "height": {
                        this.model.height = options[key];
                        this._setTabsHeightStyle(this.model.heightAdjustMode);
                        break;
                    }
                    case "width": this.element.width(options[key]); break;
                    case "headerSize": this._setHeaderSize(options[key]); break;
                    case "allowKeyboardNavigation": {
                        if (options[key])
                            this._on(this.element, 'keydown', this._keyPress);
                        else
                            this._off(this.element, "keydown");
                        break;
                    }
                    case "headerPosition":
                        {
                            this.model.headerPosition = options[key];
                            if (this.model.headerPosition == ej.Tab.Position.Top) {
                                this._removeVerticalClass();
                                this.itemsContainer.remove();
                                this.itemsContainer.insertBefore(this.element.find(">div").first());
                                this.element.find("div.e-active-content").removeClass("e-activebottom");
                            }
                            else if (this.model.headerPosition == ej.Tab.Position.Bottom) {
                                this._removeVerticalClass();
                                this.element.find("div.e-active-content").removeClass("e-activetop");
                            }
                            else if (this.model.headerPosition == ej.Tab.Position.Left || this.model.headerPosition == ej.Tab.Position.Right) {
                                this._removeHeaderClass();
                            }
                            this._refresh();
                            break;
                        }
                    case "showCloseButton":
                        {
                            if (options[key]) {
                                this._addDeleteIcon();
                                this._on(this.element.find("div.e-delete"), "click", this._tabDeleteClick);
                            } else
                                this.element.find("div.e-delete").remove();
                            break;
                        }
                    case "enableTabScroll":
                        {
                            if (options[key]) {
                                this._addScrollIcon();
                                this._on(this.element.find("div .e-scroll"), "click", this._tabScrollClick);
                                this.element.addClass("e-scrolltab");
                            } else
                                this.element.find("div .e-scroll").remove();
                            this.element.removeClass("e-scrolltab");
                            break;
                        }
                    case "showReloadIcon":
                        {
                            if (options[key]) {
                                this._addReloadIcon();
                            } else
                                this.element.find("div.e-reload").remove();
                            break;
                        }
                    case "enableRTL":
                        {
                            if (options[key])
                                this.element.addClass("e-rtl");
                            else
                                this.element.removeClass("e-rtl");
                            break;
                        }
                    case "htmlAttributes": this._addAttr(options[key]); break;
                }
            }
        },


        _init: function () {
            this._addItemIndex = null;
            this.tabId = 0;
            this._initialize();
            this._prevSize = this._getDimension($(this.element).parent(), "height");
        },


        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                this.element.removeClass(this.model.cssClass).addClass(skin);
            }
        },


        _initialize: function () {
			this.initialRender = true;
            this.element.attr("tabindex", 0).attr("role", "tablist");
            this._itemsRefreshing();
            $(this.anchors).addClass("e-link");
            this._preTabIndex = -1;
            if (!ej.isNullOrUndefined(this.model.width))
                this.element.width(this.model.width);
            this._setTabPosition(this.model.headerPosition);
            if (this.model.showCloseButton)
                this._addDeleteIcon();
            if (this.model.showReloadIcon)
                this._addReloadIcon();
            if (this.model.showRoundedCorner)
                this._roundedCorner(this.model.showRoundedCorner);
            this._enabledAction(this.model.enabled);
            this.contentPanels = [];
            this._addAttr(this.model.htmlAttributes);
            this._reinitialize();
            this._addBaseClass();
            if (!ej.isNullOrUndefined(this.model.headerSize))
                this._setHeaderSize(this.model.headerSize);
            this._disableTabs();
            this._roundedCorner(this.model.showRoundedCorner);
            if (this.model.enableTabScroll) {
                if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                    if (this.itemsContainer.width() > (this.element.width()))
                        this._addScrollIcon();
                }
                else {
                    if (this.element.height() < (this.items.height() * this.items.length))
                        this._addScrollIcon();
                }
            }
            this._wireEvents(this.model.events);
            // Added for rendering the Windows tab...
            this.showItem(this.selectedItemIndex());
            this._setTabsHeightStyle(this.model.heightAdjustMode);
            this._enabledAction(this.model.enabled);
            this._resizeEvents(this.model.heightAdjustMode);
        },

        _reinitialize: function () {
            var hid, href, hrefbase;
            for (var i = (this._addItemIndex != null) ? this._addItemIndex : 0; i < this.anchors.length; i++) {
                hid = this.anchors[i];
                if (this.divId == undefined)
                    href = $(hid).attr("href");
                else
                    href = this.divId;
                this.divId = undefined;
                hrefBase = href.split("#")[0];
                if (hrefBase && (hrefBase === location.toString().split("#")[0])) {
                    href = a.hash;
                    hid.href = href;
                }
                if (href && href !== "#") {
                    this._addContentTag(href, i);
                }
                else if (!this.model.enablePersistence) {
                    this.model.disabledItemIndex.push(i);
                }
                if (this._addItemIndex != null) { this._wireEvents(this.model.events); break; }
            }
            if (this.items.length == 1) {
                this.showItem(this.selectedItemIndex());
            }

        },
        _itemsRefreshing: function () {
            this.itemsContainer = this.element.find("ol,ul").eq(0);
            this.items = this.itemsContainer.find(" > li:has(a[href])");
            this.anchors = this.items.find('a[href]');
        },
        _setHeaderSize: function (size) {
            this.element.find(">ul li.e-item").css("height", "auto");
            this.element.find(">ul li.e-item").children("a.e-link").css("margin-top", "0px");
            if (this.model.headerPosition == "left") {
                this.element.find(">ul.e-left").css({ "width": size, "text-align": "center" });
            }
            else if (this.model.headerPosition == "right") {
                this.element.find(">ul.e-right").css({ "width": size, "text-align": "center" });
            }
            else {
                this.element.find(">ul.e-header li.e-item").css("height", size);
                this.element.find(">ul.e-header li.e-item a.e-link").css("margin-top", ((this.element.find(">ul.e-header").outerHeight() / 2) - this.element.find(">.e-header li.e-item a.e-link").outerHeight()).toString() + "px")
            }
        },
        _enabledAction: function (flag) {
            if (flag) {
                this.element.removeClass("e-disable");
            }
            else {
                this.element.addClass("e-disable");
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else if (key == "disabled" && value == "disabled") { proxy.model.enabled = false; proxy._enabledAction(false); }
                else proxy.element.attr(key, value)
            });
        },


        _setTabPosition: function (position) {
            if (position == ej.Tab.Position.Bottom) {
                this.itemsContainer.appendTo(this.element);
                this.items.removeClass("e-bottom-line");
                this.items.addClass("e-top-line");
            }
            else if (position == ej.Tab.Position.Top) {
                this.items.removeClass("e-top-line");
                if (this.model.enableRTL)
                    this.items.addClass("e-rtl-top-line e-top-hover");
                else
                    this.items.addClass("e-bottom-line");
            }
            else if (position == ej.Tab.Position.Left || position == ej.Tab.Position.Right) {
                if (this.items.length >= 0) {
                    if (this.model.height)
                        this.itemsContainer.css("height", this.model.height);
                    else {
                        if (this.model.heightAdjustMode === "fill")
                            $(this.itemsContainer).css("height", "100vh");
                        else $(this.itemsContainer).css("height", "");
                    }
                    this.element.addClass("e-vertical");
                }
            }
        },


        _addDeleteIcon: function () {
            if (this.element.find("div.e-delete").length <= 0 && this.items.length > 0) {
                var deleteIcon = ej.buildTag('div.e-icon e-delete', "", {}, { role: "presentation" }).css("visibility", "hidden");
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    var delIconPosition = this.items.find("a");
                    deleteIcon.insertBefore(delIconPosition);
                }
                else
                    this.items.append(deleteIcon);
            }
        },
        _addScrollIcon: function () {
            if (this.element.find("div.e-scroll").length <= 0 && this.items.length > 0) {
                var scrollIcon = ej.buildTag('div.e-icon e-scroll', "", {}, { role: "presentation" }).css("visibility", "hidden");
                this.itemsContainer.append(scrollIcon);
                this.scrollstep = 30;
                scrollIcon.css("position", "relative");
                if (!this.model.enableRTL) {
                    if (this.model.headerPosition == "left")
                        scrollIcon.css("margin-right", this.itemsContainer.width() / 2 + "px");
                    else if (this.model.headerPosition == "right")
                        scrollIcon.css("margin-left", this.itemsContainer.width() / 2 - this.scrollstep + "px");
                    else {
                        if (this.model.enablePersistence == true && this._beforeWidth != 0 && (this._beforeWidth > this.scrollPanelWidth))
                            scrollIcon.css("margin-right", this.itemsContainer.width() - this.scrollPanelWidth + 20 - ((this.items[this.selectedItemIndex()].offsetLeft - this.scrollPanelWidth) * 2) + "px");
                        else
                            scrollIcon.css("margin-right", this.itemsContainer.width() - this.scrollPanelWidth + 20 + "px");
                    }
                }
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    scrollIcon.css("transform", "rotate(270deg)");
                    if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11) {
                        if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom")
                            scrollIcon.css("margin-top", "-" + ((this.itemsContainer.height() + this.scrollPanelWidth) - 3) + "px");
                        else {
                            if (!this.model.enableRTL)
                                scrollIcon.css("margin-top", "-" + ((this.items.height() * this.items.length) - 20) + "px");
                            else
                                scrollIcon.css("margin-top", "-" + ((this.items.height() * this.items.length)) + "px");
                        }
                    }
                    else {
                        if (this.model.enablePersistence == true && this._beforeWidth != 0 && (this._beforeWidth > this.scrollPanelHeight))
                            scrollIcon.css("margin-top", "-" + ((this.items[this.selectedItemIndex()].offsetTop * 3 - this.scrollPanelHeight * 2)) + "px");
                        else
                            if (!this.model.enableRTL)
                                scrollIcon.css("margin-top", "-" + ((this.items.height() * this.items.length) - 20) + "px");
                            else
                                scrollIcon.css("margin-top", "-" + ((this.items.height() * this.items.length)) + "px");
                    }
                }

                else {
                    if (this.model.enableRTL) {
                        scrollIcon.css("margin-top", "-" + (this.itemsContainer.height()) + "px");
                        scrollIcon.css("margin-left", (this.itemsContainer.width() - 14) + "px");
                    }
                    else
                        scrollIcon.css("margin-top", "-" + ((this.itemsContainer.height() / 2) + 10) + "px");
                }
                if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11) {
                    if (this.model.headerPosition == "top") {
                        if (parseInt(scrollIcon.prev().offset().top) - parseInt(scrollIcon.offset().top) == (-parseInt(scrollIcon.css("margin-top"))))
                            scrollIcon.css("top", ((this.itemsContainer.height() / 2) + 27) + "px");
                    }
                    else if (this.model.headerPosition == "bottom") {
                        if (!this.model.enableRTL)
                            scrollIcon.css("top", ((this.itemsContainer.height() / 2) + 27) + "px");
                        else
                            scrollIcon.css("top", ((this.itemsContainer.height() / 2) + 20) + "px");
                    }
                }
                if (this.model.headerPosition == "left") {
                    scrollIcon.css("margin-right", this.itemsContainer.width() / 2 + "px");
                }
                else if (this.model.headerPosition == "right")
                    scrollIcon.css("margin-left", this.itemsContainer.width() / 2 - this.scrollstep + "px");
                this.element.attr('unselectable', 'on')
                this.element.css('user-select', 'none')
                this.element.on('selectstart', false);
                this._on(this.element.find("div .e-scroll"), "click", this._tabScrollClick);
            }
        },
        _addScrollBackIcon: function () {
            if (this.element.find("div.e-right-scroll").length <= 0) {
                var scrollIcon = ej.buildTag('div.e-icon e-right-scroll', "", {}, { role: "presentation" }).css("visibility", "hidden");
                this.itemsContainer.prepend(scrollIcon);
                this.rightscrollstep = 30;
                this.element.attr('unselectable', 'on')
                this.element.css('user-select', 'none')
                this.element.on('selectstart', false);
                scrollIcon.css("position", "absolute");
                if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                    if (!this.model.enableRTL) {
                        scrollIcon.css("margin-left", "20px");
                    }
                    else
                        scrollIcon.css("margin-right", (this.itemsContainer.width() / 2) + "px");
                }
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    if (!this.model.enableRTL) {
                        if (this.model.headerPosition == "right")
                            scrollIcon.css("margin-left", ((this.itemsContainer.width() / 2) - 30) + "px");
                        else if (this.model.headerPosition == "left")
                            scrollIcon.css("margin-left", ((this.itemsContainer.width() / 2) - 15) + "px");
                        scrollIcon.css("margin-top", +this.scrollPanelHeight + 20 + "px");
                    }
                    else {
                        scrollIcon.css("margin-top", +(this.scrollPanelHeight - 4) + "px");
                        if (this.model.headerPosition == "right")
                            scrollIcon.css("margin-right", (this.itemsContainer.width() / 2 + 14) + "px");
                        else
                            scrollIcon.css("margin-right", (this.itemsContainer.width() / 2) + "px");
                    }
                    scrollIcon.css("transform", "rotate(270deg)");
                }
                else
                    if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11)
                        scrollIcon.css("margin-top", "0px");
                    else {
                        if (!this.model.enableRTL)
                            scrollIcon.css("margin-top", "-5px");
                        else
                            scrollIcon.css("margin-top", "0px");
                    }
                this._on(this.element.find("div.e-right-scroll"), "click", this._tabScrollBackClick);
            }
        },


        _addReloadIcon: function () {
            if (this.element.find("div.e-reload").length <= 0 && this.items.length > 0) {
                var reloadIcon = ej.buildTag('div.e-icon e-reload', "", {}, { role: "presentation" }).css("visibility", "hidden");
                if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                    var iconPosition = this.items.find("a");
                    reloadIcon.insertBefore(iconPosition);
                }
                else
                    this.items.append(reloadIcon);
            }
        },


        _addBaseClass: function () {
            this.element.addClass("e-widget " + this.model.cssClass);
            this.itemsContainer.addClass("e-box")
            if (this.model.enableRTL)
                this.element.addClass("e-rtl");
            // To avoid border blinking 
            if (this.model.headerPosition == "top") {
                $(this.contentPanels).addClass("e-hidebottom e-addborderbottom");
                $(this.itemsContainer).addClass("e-addborderbottom");
                $(this.contentPanels).removeClass("e-hidetop e-addbordertop e-hideright e-addborderright e-hideleft e-addborderleft");
                $(this.itemsContainer).removeClass("e-addbordertop e-addborderright e-addborderleft");
                this.items.length > 0 && this.itemsContainer.addClass("e-header");
                if (this.model.enableRTL) {
                    this.items.addClass("e-rtl-top-line");
                    this.items.removeClass("e-rtl-bottom-line");
                }
            }
            if (this.model.headerPosition == "bottom") {
                $(this.contentPanels).removeClass("e-hidebottom e-addborderbottom e-hideright e-addborderright e-hideleft e-addborderleft");
                $(this.itemsContainer).removeClass("e-addborderbottom e-addborderright e-addborderleft");
                $(this.contentPanels).addClass("e-hidetop e-addbordertop");
                $(this.itemsContainer).addClass("e-addbordertop");
                this.items.length > 0 && this.itemsContainer.addClass("e-header");
                if (this.model.enableRTL) {
                    this.items.addClass("e-rtl-bottom-line")
                    this.items.removeClass("e-rtl-top-line e-top-line");
                }
            }
            if (this.model.headerPosition == "left") {
                this.items.length > 0 && this.itemsContainer.addClass("e-left");
                $(this.contentPanels).removeClass("e-hidetop e-addbordertop e-hidebottom e-addborderbottom e-hideright e-addborderright");
                $(this.itemsContainer).removeClass("e-addbordertop e-addborderbottom e-addborderright");
                $(this.contentPanels).addClass("e-hideleft e-addborderleft");
                $(this.itemsContainer).addClass("e-addborderleft");
                $(this.items).removeClass("e-rtl-bottom-line e-rtl-top-line");
            }
            if (this.model.headerPosition == "right") {
                this.items.length > 0 && this.itemsContainer.addClass("e-right");
                $(this.contentPanels).removeClass("e-hidetop e-addbordertop e-hidebottom e-addborderbottom e-hideleft e-addborderleft");
                $(this.itemsContainer).removeClass("e-addbordertop e-addborderbottom e-addborderleft");
                $(this.contentPanels).addClass("e-hideright e-addborderright");
                $(this.itemsContainer).addClass("e-addborderright");
                $(this.items).removeClass("e-rtl-bottom-line e-rtl-top-line");
            }
            this.items.addClass("e-select e-item").attr("role", "tab").attr("tabindex", -1).attr("aria-expanded", true).attr("aria-selected", false);
            $(this.contentPanels).addClass("e-content  e-content-item e-box").attr("role", "tabpanel").attr("aria-hidden", true);
            this.scrollPanelWidth = this.itemsContainer.width();
            this.scrollPanelHeight = this.itemsContainer.height();
            if (this.model.enableTabScroll == true && this.itemsContainer.height() > this.items.height()) {
                this.scrollstep = 0;
                this.model.enableTabScroll = true;
                if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                    this.itemsContainer.css({ "width": ((parseInt(this.itemsContainer.css("width")) + (parseInt(this.items.css("width")) * this.items.length))) + "px", "position": "absolute" });
                }
                this._beforeWidth = 0;
                if (this.model.enablePersistence == true) {
                    if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                        for (var val = 0; val < this.selectedItemIndex() ; val++) {
                            this._beforeWidth += this.items[val].offsetWidth;
                        }
                    }
                    else {
                        for (var val = 0; val < this.selectedItemIndex() ; val++) {
                            this._beforeWidth += this.items[val].offsetHeight;
                        }
                    }
                }
                var widthValue = parseInt(this.itemsContainer.css("width"));
                if (this.model.headerPosition == "top") {
                    if (!this.model.enableRTL) {
                        if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11)
                            $(this.contentPanels).css("margin-top", parseInt(this.itemsContainer.css("height")) + 4 + "px");
                        else {
                            if (this.itemsContainer.height() >= this.itemsContainer.css("height"))
                                $(this.contentPanels).css("margin-top", (this.itemsContainer.outerHeight()) + 4 + "px");
                            else
                                $(this.contentPanels).css("margin-top", (this.itemsContainer.outerHeight()) + 4 + "px");
                        }

                    }
                    else
                        $(this.contentPanels).css("margin-top", (this.itemsContainer.outerHeight() + 4) + "px");
                    $(this.contentPanels).css({ "position": "absolute", "border-top": "none", "width": parseInt(this.element.width()) - 1 + "px" });
                    this.itemsContainer.css("border-bottom", "1px solid #bbbcbb");
                }
                if (this.model.headerPosition == "bottom") {
                    $(this.contentPanels).css({ "position": "relative", "width": parseInt(this.element.width()) - 1 + "px", "border-bottom": "none" });
                    this.itemsContainer.css("border-top", "1px solid #bbbcbb");
                    $(this.contentPanels).css({ "border-top": "" });
                }

                var rigtVaule = parseInt(widthValue - (this.scrollPanelWidth + this.scrollstep - 1));
                if (this.model.enableRTL && (this.model.headerPosition == "top" || this.model.headerPosition == "bottom")) {
                    this.itemsContainer.css("clip", "rect(0px," + widthValue + "px,100px," + rigtVaule + "px)");
                }
                else if (this.model.headerPosition == "left") {
                    if (this.itemsContainer.height() < (this.items.height() * this.items.length)) {
                        this.itemsContainer.css({ "height": ((parseInt(this.itemsContainer.css("height")) + (parseInt(this.items.css("height")) * this.items.length)) + 30) + "px", "position": "absolute", "border-right": "1px solid #bbbcbb", "background": "white" });
                        $(this.contentPanels).css("padding-left", this.itemsContainer.width() + 5 + "px");
                        if (this.model.enableRTL)
                            this.itemsContainer.css("margin-right", (this.element.width() - this.itemsContainer.width()));
                        if (this._beforeWidth == 0 || this._beforeWidth < this.scrollPanelHeight)
                            this.itemsContainer.css("clip", "rect(0px," + (this.itemsContainer.width() + 2) + "px," + this.scrollPanelHeight + "px," + (this.scrollstep) + "px)");
                        else {
                            if (this._beforeWidth > this.scrollPanelHeight) {
                                this.itemsContainer.css({
                                    "clip": "rect(" + ((this.items[this.selectedItemIndex()].offsetTop - this.scrollPanelHeight) * 2) + "px," + (this.itemsContainer.width() + 2) + "px," + (this.items[this.selectedItemIndex()].offsetTop * 2 - this.scrollPanelHeight) + "px,0px)",
                                    "margin-top": "-" + ((this.items[this.selectedItemIndex()].offsetTop - this.scrollPanelHeight) * 2) + "px"
                                });
                            }
                        }

                        this.element.removeClass("e-scrolltab");
                    }
                }
                else if (this.model.headerPosition == "right") {
                    if (this.itemsContainer.height() < (this.items.height() * this.items.length)) {
                        this.itemsContainer.css({ "height": ((parseInt(this.itemsContainer.css("height")) + (parseInt(this.items.css("height")) * this.items.length)) + 30) + "px", "position": "absolute", "margin-left": "-1px" });
                        $(this.contentPanels).css({ "position": "absolute", "width": parseInt(this.element.width() - 1) + "px", "height": parseInt(this.element.height()) + "px", "margin-left": "-" + parseInt(this.element.width()) + "px", "border-right": "none" });
                        $(this.element).css("margin-left", parseInt(this.element.width()) + "px");
                        this.itemsContainer.css("border-left", "1px solid #bbbcbb");
                        if (this.model.enableRTL)
                            this.itemsContainer.css("margin-right", "-" + (this.itemsContainer.width() + 1) + "px");
                        if (this._beforeWidth == 0 || this._beforeWidth < this.scrollPanelHeight)
                            this.itemsContainer.css("clip", "rect(0px," + this.itemsContainer.width() + 2 + "px," + this.scrollPanelHeight + "px," + (this.scrollstep) + "px)");
                        else {
                            if (this._beforeWidth > this.scrollPanelHeight) {
                                this.itemsContainer.css({
                                    "clip": "rect(" + ((this.items[this.selectedItemIndex()].offsetTop - this.scrollPanelHeight) * 2) + "px," + (this.itemsContainer.width() + 2) + "px," + (this.items[this.selectedItemIndex()].offsetTop * 2 - this.scrollPanelHeight) + "px,0px)",
                                    "margin-top": "-" + ((this.items[this.selectedItemIndex()].offsetTop - this.scrollPanelHeight) * 2) + "px"
                                });
                            }
                        }
                        this.element.removeClass("e-scrolltab");
                    }
                }
                else {
                    if (this._beforeWidth == 0 || this._beforeWidth < this.scrollPanelWidth) {
                        this.itemsContainer.css({ "clip": "rect(0px," + (this.scrollPanelWidth + this.scrollstep) + "px,100px," + (this.scrollstep) + "px)", "margin-left": "-" + this.scrollstep + "px" });
                    }
                    else {
                        if (this._beforeWidth > this.scrollPanelWidth) {
                            this.itemsContainer.css({ "clip": "rect(0px," + (this.items[this.selectedItemIndex()].offsetLeft * 2 - this.scrollPanelWidth) + "px,100px," + ((this.items[this.selectedItemIndex()].offsetLeft - this.scrollPanelWidth) * 2) + "px)", "margin-left": "-" + ((this.items[this.selectedItemIndex()].offsetLeft - this.scrollPanelWidth) * 2) + "px" });
                        }
                    }
                }
            }
        },
        _executeForwardScrolling: function (args) {
            $("div#" + this._id + " .e-icon.e-scroll").css("position", "relative");
            if ($("div#" + this._id + " .e-icon.e-scroll")[0] == args.target) {
                var step = this.scrollstep - this.rightscrollstep; var calcscroll;
                this.offsetwidth = this.itemsContainer.width() - this.scrollPanelWidth;
                this.offsetheight = this.itemsContainer.height() - this.scrollPanelHeight;
                if (this.rightscrollstep > 30)
                    this.rightscrollstep = this.rightscrollstep - 30;
                if (this.checkwidth == null || this.rightscrollstep == 30) {
                    var widthValue = parseInt(this.itemsContainer.css("width"));
                    var rigtVaule = parseInt(widthValue - (this.scrollPanelWidth + this.scrollstep - 1));
                    if (this.model.enableRTL) {
                        if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {

                            this.itemsContainer.css({ "clip": "rect(0px," + (widthValue - this.scrollstep) + "px,100px," + (rigtVaule) + "px)", "margin-right": "-" + this.scrollstep + "px" });

                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-right", (this.scrollPanelWidth + this.scrollstep - 40) + "px");
                            if (this.scrollstep < (this.items.width() * this.items.length - 30)) {
                                if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-left", (widthValue - this.scrollstep - 14) + "px");
                            }

                            else
                                if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("display", "none");
                        }
                    }
                    else if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                        this.itemsContainer.css({ "clip": "rect(0px," + (this.scrollPanelWidth + this.scrollstep) + "px,100px," + (this.scrollstep) + "px)", "margin-left": "-" + this.scrollstep + "px" });
                        if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-right", (this.offsetwidth - this.scrollstep + 20) + "px");
                        $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", (this.scrollstep - 10) + "px");
                    }
                    if (this.model.headerPosition == "left" || this.model.headerPosition == "right") {
                        this.scrollDown = this.items.height();
                        this.itemsContainer.css({ "clip": "rect(" + this.scrollstep + "px," + (this.itemsContainer.width() + 3) + "px," + (this.scrollPanelHeight + this.scrollstep) + "px," + "0px)", "margin-top": "-" + this.scrollstep + "px", "margin-left": "-1px" });
                        if (!this.model.enableRTL) {
                            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + (this.offsetheight - this.scrollstep - 50) + "px");
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-top", (this.scrollPanelHeight + this.scrollstep - 10) + "px");
                        }
                        else {
                            if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11) {
                                if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + (this.offsetheight - this.scrollstep - 25) + "px");
                            }
                            else
                                if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + (this.offsetheight - this.scrollstep - 28) + "px");
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-top", (this.scrollPanelHeight + this.scrollstep - 34) + "px");
                        }
                        if (this.model.headerPosition != "left")
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", ((this.itemsContainer.width() / 2) - 30) + "px");
                        else
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", ((this.itemsContainer.width() / 2) - 15) + "px");
                    }
                    calcscroll = this.scrollstep + 30;
                }
                else {
                    if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                        if (this.model.enableRTL) {
                            this.itemsContainer.css({ "clip": "rect(0px," + (this.itemsContainer.width() - this.scrollstep + 1) + "px,100px," + (this.itemsContainer.width() - (this.scrollPanelWidth + this.scrollstep - 1)) + "px)", "margin-right": "-" + this.scrollstep + "px" });
                            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-left", (this.itemsContainer.width() - this.scrollstep - 14) + "px");
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-right", (this.scrollPanelWidth + this.scrollstep - 40) + "px");
                        }
                        else {
                            this.itemsContainer.css({ "clip": "rect(0px," + (this.updatewidth + 30) + "px,100px," + (this.updatestep + 30) + "px)", "margin-left": "-" + (this.updatestep + 30) + "px" });
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", (this.updatestep + 20) + "px");
                            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-right", (parseInt($("div#" + this._id + " .e-icon.e-scroll").css("margin-right")) - 30) + "px");
                        }
                    }
                    else {
                        this.itemsContainer.css({ "clip": "rect(" + this.scrollstep + "px," + (this.itemsContainer.width() + 3) + "px," + (this.scrollPanelHeight + this.scrollstep) + "px," + "0px)", "margin-top": "-" + this.scrollstep + "px", "margin-left": "-1px" });
                        if (!this.model.enableRTL) {
                            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + (this.offsetheight - this.scrollstep - 50) + "px");
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-top", (this.scrollPanelHeight + this.scrollstep - 14) + "px");
                        }
                        else {
                            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + (this.offsetheight - this.scrollstep - 34) + "px");
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-top", (this.scrollPanelHeight + this.scrollstep - 31) + "px");
                        }
                        if (this.model.headerPosition != "left")
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", ((this.itemsContainer.width() / 2) - 30) + "px");
                        else
                            $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", ((this.itemsContainer.width() / 2) - 15) + "px");
                    }
                    this.updatestep += 30; this.updatewidth += 30;
                }
                if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11) {
                    var tabAvailwidth = 0;
                    for (i = 0; i < this.items.length; i++) {
                        tabAvailwidth = (parseInt($(this.items[i]).css("width"))) + tabAvailwidth;
                    }
                    if (this.updatewidth >= tabAvailwidth + 53 || calcscroll + this.scrollPanelWidth >= tabAvailwidth + 53)
                        $("div#" + this._id + " .e-icon.e-scroll").css("top", 0);
                }
                this.scrolledwidth = this.scrollPanelWidth + this.scrollstep;
                this.scrolledheight = this.scrollPanelHeight + this.scrollstep;
                if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                    this.checkwidth = this.scrolledwidth;
                }
                else {
                    this.checkwidth = this.scrolledheight;
                }
            }
        },
        _executeBackwardScrolling: function (args) {
            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("display", "block");
            var step = this.scrollstep - this.rightscrollstep;
            var scrollwidth = this.scrolledwidth - this.rightscrollstep;
            if (this.scrolledwidth != this.checkwidth) {
                if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                    if (!this.model.enableRTL) {
                        this.itemsContainer.css({ "clip": "rect(0px," + (this.checkwidth - this.rightscrollstep + 30) + "px,100px," + (step - 30) + "px)", "margin-left": "-" + (step - 30) + "px" });
                        if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-right", (this.offsetwidth - step + 50) + "px");
                    }
                    else {
                        this.itemsContainer.css({ "clip": "rect(0px," + (this.itemsContainer.width() - this.scrollstep + this.rightscrollstep + 30 + 1) + "px,100px," + (this.itemsContainer.width() - (this.scrollPanelWidth + step - 30) + "px)"), "margin-right": "-" + (step - 30) + "px" });
                        if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-left", (this.itemsContainer.width() - step + 16) + "px");
                    }
                }
                else {
                    if (this.rightscrollstep > 30)
                        this.itemsContainer.css("clip", "rect(" + (step - 30) + "px," + (this.itemsContainer.width() + 3) + "px," + (this.checkwidth - this.rightscrollstep + 30) + "px," + "0px)");
                    else
                        this.itemsContainer.css("clip", "rect(" + (step - 30) + "px," + (this.itemsContainer.width() + 3) + "px," + (this.checkwidth - this.rightscrollstep) + "px," + "0px)");
                    this.itemsContainer.css("margin-top", "-" + (step - 30) + "px");
                    if (!this.model.enableRTL) {
                        if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + ((this.offsetheight - step) - 20) + "px");
                    }
                    else {
                        if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11) {
                            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + ((this.offsetheight - step) + 5) + "px");
                        }
                        else {
                            if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-top", "-" + ((this.offsetheight - step) + 2) + "px");
                        }

                    }
                }
            }
            else {
                if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                    if (!this.model.enableRTL) {
                        this.itemsContainer.css({ "clip": "rect(0px," + (this.checkwidth - this.rightscrollstep) + "px,100px," + (step - 30) + "px)", "margin-left": "-" + (step - 30) + "px" });
                        if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css({ "margin-right": (this.offsetwidth - step + 50) + "px", "top": ((this.itemsContainer.height() / 2) + 27) + "px" });
                    }
                    else {
                        this.itemsContainer.css({ "clip": "rect(0px," + (this.itemsContainer.width() - this.scrollstep + 60) + "px,100px," + (this.itemsContainer.width() - (this.scrollPanelWidth + step - 30) + "px)"), "margin-right": "-" + (step - 30) + "px" });
                        if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-scroll").css("margin-left", (this.itemsContainer.width() - step + 16) + "px");
                    }
                }
                else {
                    this.itemsContainer.css({ "clip": "rect(" + (step - 30) + "px," + (this.itemsContainer.width()) + "px," + (this.checkwidth - this.rightscrollstep) + "px," + "0px)", "margin-top": "-" + (step - 30) + "px" });
                    if ($("div#" + this._id + " .e-icon.e-scroll") != "") $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-top", (this.scrollPanelHeight + this.scrollstep - 10) + "px");
                }
            }
            this.updatewidth = this.checkwidth - this.rightscrollstep + 30;
            this.updatestep = step - 30;
            if (this.model.headerPosition != "right" && this.model.headerPosition != "left")
                this.contentPanels.css("margin-right", "-1px");
            if (step <= 30) {
                $("div#" + this._id + " .e-icon.e-right-scroll").css("display", "none");
                this._addScrollIcon();
                this.scrollstep = 0;
                this.rightscrollstep = 0;
            }
            else {
                if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom") {
                    if (!this.model.enableRTL)
                        $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", (step - 40) + "px");
                    else
                        $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-right", (this.scrollPanelWidth + this.scrollstep - 70 - this.rightscrollstep) + "px");
                }
                else {
                    if (this.model.headerPosition != "left")
                        $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", ((this.itemsContainer.width() / 2) - 30) + "px");
                    else
                        $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-left", ((this.itemsContainer.width() / 2) - 15) + "px");
                    if (!this.model.enableRTL)
                        $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-top", (this.scrollPanelHeight + step - 40) + "px");
                    else
                        $("div#" + this._id + " .e-icon.e-right-scroll").css("margin-top", (this.scrollPanelHeight + step - 64) + "px");
                }
            }
            if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version) < 11) {
                if (this.model.headerPosition == "top") {
                    if ($("div#" + this._id + " .e-icon.e-scroll").prev().offset().top - $("div#" + this._id + " .e-icon.e-scroll").offset().top == (-parseInt($("div#" + this._id + " .e-icon.e-scroll").css("margin-top")) + 1))
                        $("div#" + this._id + " .e-icon.e-scroll").css("top", ((this.itemsContainer.height() / 2) + 27) + "px");
                }
                else if (this.model.headerPosition == "bottom") {
                    if (!this.model.enableRTL)
                        $("div#" + this._id + " .e-icon.e-scroll").css("top", ((this.itemsContainer.height() / 2) + 27) + "px");
                    else
                        $("div#" + this._id + " .e-icon.e-scroll").css("top", ((this.itemsContainer.height() / 2) + 20) + "px");
                }
            }
            else {
                $("div#" + this._id + " .e-icon.e-scroll").css("top", "0px");
            }
            this.rightscrollstep = this.rightscrollstep + 30;
            if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom")
                this.checkwidth = this.scrolledwidth - 30;
            else
                this.checkwidth = this.scrolledheight - 30;

        },

        _removeHeaderClass: function () {
            this.itemsContainer.remove();
            this.itemsContainer.insertBefore(this.element.find(">div").first());
            this.items.removeClass("e-bottom-line e-top-line");
            $(this.contentPanels).removeClass("e-content-bottom e-activetop e-activebottom");
            this.itemsContainer.removeClass("e-header e-left e-right");
        },

        _removeVerticalClass: function () {
            this.element.removeClass("e-vertical");
            this.itemsContainer.removeClass("e-left e-right").removeAttr("style");
        },


        _removeBaseClass: function () {
            this.element.removeClass("e-tab e-widget e-corner");
            if ((this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                this._removeVerticalClass();
            this.itemsContainer.removeClass("e-header e-box e-clearall e-select");
            this.anchors.removeClass("e-link");
            this.items.removeClass("e-select e-item e-active e-bottom-line e-top-line");
            $(this.contentPanels).removeClass("e-content  e-content-item e-box");
            $(this.contentPanels).removeClass("e-content  e-content-item e-box e-content-bottom e-activetop e-activebottom e-active-content");
            $(this.contentPanels).css("display", "");
        },


        _addContentTag: function (href, index) {
            var id = this._getTabId(href);
            var panel = this.element.find("#" + id);
            if (!panel.length) {
                panel = ej.buildTag("div.e-content  e-content-item e-box e-content-bottom #" + id)
                    .insertAfter(this.contentPanels[index - 1] || this.itemsContainer);
            }
            this.contentPanels = $(this.contentPanels).add(panel);
        },

        _roundedCorner: function (value) {
            if (value) {
                this.element.addClass('e-corner');
            }
            else if (this.element.hasClass('e-corner')) {
                this.element.removeClass('e-corner');
            }
        },


        _setTabsHeightStyle: function (heightFormat) {
            if (ej.Tab.HeightAdjustMode.Content != heightFormat) $(this.contentPanels).height("");
            if (ej.Tab.HeightAdjustMode.Fill == heightFormat) {
                if (ej.Tab.Position.Left === this.model.headerPosition || ej.Tab.Position.Right === this.model.headerPosition) {
                    $(this.contentPanels).css("height", "100vh");
                }
                else this._contentPaneSize();
            }
            else if (ej.Tab.HeightAdjustMode.Auto == heightFormat) {
                var maxHeight = 0;
                $(this.contentPanels).css({ "display": "none" }).addClass('e-active-content');
                for (var i = 0; i < this.contentPanels.length; i++) {
                    maxHeight = Math.max(maxHeight, this._getDimension($(this.contentPanels[i]), "height"));
                }
                $(this.contentPanels).removeClass('e-active-content');
                $(this.contentPanels).height(maxHeight);
                this.maxAutoHeight = maxHeight;
                this.showItem(this.selectedItemIndex());
            }
            else if (ej.Tab.HeightAdjustMode.None == heightFormat) {
                if (this.model.height != null) {
                    this._contentPaneSize();
                }
            }
            if (ej.Tab.HeightAdjustMode.Fill !== heightFormat) $(this.itemsContainer).height("");
        },
        _getDimension: function (element, method) {
            var value;
            var $hidden = $(element).parents().andSelf().filter(':hidden');
            var prop = { visibility: 'hidden', display: 'block' };
            var hiddenCollection = [];
            $hidden.each(function () {
                var hidden = {}, name;
                for (name in prop) {
                    hidden[name] = this.style[name];
                    this.style[name] = prop[name];
                }
                hiddenCollection.push(hidden);
            });
            value = /(outer)/g.test(method) ?
            $(element)[method](true) :
           $(element)[method]();

            $hidden.each(function (i) {
                var hidden = hiddenCollection[i], name;
                for (name in prop) {
                    this.style[name] = hidden[name];
                }
            });
            return value;
        },
        // Tab to active given index value
        showItem: function (index) {
            if ($.inArray(index, this.model.disabledItemIndex) < 0) {
                var proxy = this;
                this._preTabIndex = this.selectedItemIndex();
                this.selectedItemIndex(index);
                if (this.selectedItemIndex() >= this.contentPanels.length) {
                    this.selectedItemIndex(0);
                    index = this.selectedItemIndex();
                }
                if (index >= 0 && !this.initialRender && true === this._onBeforeActive(index)) return false;
                $(this.items[this.selectedItemIndex()]).attr("aria-expanded", true).attr("aria-selected", true).attr("tabindex", 0);
                if (this.selectedItemIndex() != null && this.selectedItemIndex() < this.contentPanels.length) {
                    this._ajaxLoad();
                    this.hideItem(this._preTabIndex);
                    $(this.contentPanels[this.selectedItemIndex()]).fadeIn(this.model.enableAnimation ? 200 : 0, "easeOutQuad", function () {
                        if (!proxy.initialRender && proxy._onActive())
                            return true;
						proxy.initialRender=false;
                    });
                    if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right")) {
                        var activeClass = this.model.headerPosition == ej.Tab.Position.Top ? "e-activetop" : "e-activebottom";
                        $(this.contentPanels[this.selectedItemIndex()]).addClass(activeClass)
                    }
                    else
                        $(this.contentPanels[this.selectedItemIndex()]).addClass("e-active-content ");
                    $(this.items[this.selectedItemIndex()]).addClass("e-active").removeClass("e-select");
                    $(this.items[this.selectedItemIndex()]).removeClass("e-margine-top e-margine-bottom");

                    for (var i = 0; i <= $(this.items).length; i++) {
                        if ($(this.items[i]).hasClass("e-select")) {
                            if (this.model.headerPosition == "right") {
                                $(this.items[i]).removeClass("e-margine-top e-margine-bottom");
                            }
                            if (this.model.headerPosition == "left") {
                                $(this.items[i]).removeClass("e-margine-top e-margine-bottom");
                            }
                            if (this.model.headerPosition == "top") {
                                if (!this.element.hasClass("e-tab-collapsed"))
                                    $(this.items[i]).addClass("e-margine-top");
                                else
                                    $(this.items[i]).removeClass("e-margine-top");
                                $(this.items[i]).removeClass("e-margine-bottom");
                            }
                            if (this.model.headerPosition == "bottom") {
                                $(this.items[i]).removeClass("e-margine-top");
                                $(this.items[i]).addClass("e-margine-bottom");
                            }
                        }
                    }
                    $(this.contentPanels[this.selectedItemIndex()]).addClass("e-active-content").removeAttr("aria-hidden", false);


                }
            }
        },

        hideItem: function (index) {
            $(this.contentPanels[index]).fadeOut(0);
            if (!(this.model.headerPosition == "left" || this.model.headerPosition == "right"))
                var activeClass = this.model.headerPosition == ej.Tab.Position.Top ? "e-activetop" : "e-activebottom";
            $(this.items[index]).removeClass("e-active").addClass("e-select");
            $(this.contentPanels[index]).removeClass("e-active-content " + activeClass).attr("aria-hidden", true);
        },


        _ajaxLoad: function () {
            var content = $(this.contentPanels[this.selectedItemIndex()]);
            var link = this.anchors[this.selectedItemIndex()];
            var href = $(link).attr("href");
            if (content.is(':empty') && href.indexOf("#") !== 0)
                this._sendAjaxOptions(content, link);
        },


        _getTabId: function (href) {
            return !href.indexOf("#") ? href.replace("#", "") : this.model.idPrefix + this._getNextTabId();
        },

        _getNextTabId: function () {
            return ++this.tabId;
        },


        _disableTabs: function () {
            for (var i = 0, li; (li = this.items[i]) ; i++) {
                if ($.inArray(i, this.model.disabledItemIndex) > -1) {
                    $(li).find("a").unbind(this.model.events);
                    this._off($(li).find("div.e-delete"), "click");
                }
                $(li)[$.inArray(i, this.model.disabledItemIndex) != -1 &&
                    !$(li).hasClass("e-tab-selected") ? "addClass" : "removeClass"]("e-disable");
                $(this.contentPanels[i])[$.inArray(i, this.model.disabledItemIndex) != -1 &&
                    !$(this.contentPanels[i]).hasClass("e-tab-selected") ? "addClass" : "removeClass"]("e-disable");
            }
        },


        _tabItemClick: function (args) {
            if (this.model.enabled) {
                args.preventDefault(); // Prevent the ancher tag url action
                var index;
                if (this.selectedItemIndex() == $(this.items).index($(args.currentTarget)) && this.model.collapsible) {
                    index = -1; $(this.element).addClass("e-tab-collapsed");
                }
                else {
                    index = $(this.items).index($(args.currentTarget));
                }
                if (index != this.selectedItemIndex())
                    this.showItem(index);
                $(this.element).removeClass("e-tab-collapsed");
            }
        },

        _tabDeleteClick: function (args) {
            if (this.model.enabled) {
                var currentTab = $(args.target);
                if (currentTab.hasClass("e-delete"))
                    var index = $(this.items).index($(args.target).parent());
                if (index == this.selectedItemIndex() && this.items.length > index)
                    this.selectedItemIndex(this.selectedItemIndex() + 1)
                this.removeItem(index);
            }
        },
        _tabScrollClick: function (args) {
            if (this.model.enabled) {
                if (($("div#" + this._id + " .e-icon.e-scroll")[0] == args.target)) {
                    this._executeForwardScrolling(args);
                    this.scrollstep = this.scrollstep + 30;
                    this._addScrollBackIcon();
                    $("div#" + this._id + " .e-icon.e-right-scroll").css("display", "block");
                }
            }
        },
        _tabScrollBackClick: function (args) {
            if (this.model.enabled) {
                if (($("div.e-icon.e-right-scroll")[0] == args.target)) {
                    this._executeBackwardScrolling(args);
                }
            }
        },

        _tabReloadClick: function (args) {
            if (this.model.enabled) {
                var currentTab = $(args.target);
                if (currentTab.hasClass("e-reload")) {
                    var link = this.anchors[this.selectedItemIndex()];
                    var href = $(link).attr("href");
                    var content = $(this.contentPanels[this.selectedItemIndex()]);
                    if (href.indexOf("#") !== 0)
                        this._sendAjaxOptions(content, link);
                    else
                        this.showItem(this.selectedItemIndex());
                }
            }
        },


        _sendAjaxOptions: function (content, link) {
            //load waiting popup
            if (this._onBeforeLoad(link))
                return true;
            content.addClass("e-load");
            var proxy = this;
            var curTabTitle = $(link).html();
            var hrefLink = link.href.replace("#", "");
            var ajaxOptions = {
                type: this.model.ajaxSettings.type, cache: this.model.ajaxSettings.cache, url: hrefLink, data: this.model.ajaxSettings.data,
                dataType: this.model.ajaxSettings.dataType, contentType: this.model.ajaxSettings.contentType, async: this.model.ajaxSettings.async,
                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, content, link, curTabTitle);
                    } catch (e) {

                    }
                }, "error": function () {
                    try {
                        proxy._ajaxErrorHandler(link, proxy.selectedItemIndex(), curTabTitle);
                    } catch (e) {

                    }
                }
            };
            this._sendAjaxRequest(ajaxOptions);
        },

        _sendAjaxRequest: function (ajaxOptions) {
            $.ajax({
                type: ajaxOptions.type,
                cache: ajaxOptions.cache,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                contentType: ajaxOptions.contentType,
                async: ajaxOptions.async,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
                beforeSend: ajaxOptions.beforeSend,
                complete: ajaxOptions.complete
            });
        },


        _ajaxSuccessHandler: function (data, content, link, curTabTitle) {
            if (curTabTitle != null)
                $(link).html(curTabTitle);
            content.removeClass("e-load");
            content.html(data).addClass("e-tab-loaded"); //to indicate the content is already loaded
            var eventData = { data: data, url: link, content: content };
            this._trigger("ajaxSuccess", eventData);
            if (this._onLoad(link))
                return true;
        },


        _ajaxErrorHandler: function (data, link, index, title) {
            this._trigger("ajaxError", { data: data, url: link });
            this._onLoad(link);
        },

        _createContentPanel: function (id) {
            return $('<div></div>')
				.attr("id", id)
				.addClass("e-content  e-content-item e-content-bottom e-box");
        },


        _refresh: function () {
            this._unWireEvents();
            this._initialize();
        },

        _keyPress: function (e) {
            if (this.model.enabled) {
                var index, currentEle, targetEle = $(e.target);
                if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
                else if (e.which) code = e.which; // ns4 and opera
                else code = e.charCode;
                if (targetEle.hasClass("e-link") || targetEle.hasClass("e-item")) {
                    switch (code) {
                        case 39:
                        case 40:
                            {
                                e.preventDefault();
                                this.showItem(this.selectedItemIndex() + 1);
                                break;
                            }
                        case 37:
                        case 38:
                            {
                                e.preventDefault();
                                if (this.selectedItemIndex() != 0)
                                    this.showItem(this.selectedItemIndex() - 1);
                                else if (this.selectedItemIndex() == 0)
                                    this.showItem(this.getItemsCount() - 1);
                                break;
                            }
                        case 35:
                            {
                                e.preventDefault();
                                this.showItem(this.getItemsCount() - 1);
                                break;
                            }
                        case 36:
                            {
                                e.preventDefault();
                                this.showItem(0);
                                break;
                            }
						case 13:
							{
								e.preventDefault();
								this.showItem(this.selectedItemIndex());
								break;
                    }
                }
                }
                else if (e.ctrlKey && !targetEle.hasClass("e-tab")) {
                    switch (code) {
                        case 38:
                            e.preventDefault();
                            index = $(this.contentPanels).index(targetEle.parent(".e-content"));
                            currentEle = $(this.items[index]);
                            break;
                        case 33:
                            e.preventDefault();
                            currentEle = $(this.items[0]);
                            this.showItem(0);
                            break;
                        case 34:
                            e.preventDefault();
                            currentEle = $(this._headers[this.getItemsCount() - 1]);
                            this.showItem(this.getItemsCount() - 1);
                            break;
                    }
                }
                if (!ej.isNullOrUndefined(currentEle)) {
                    currentEle.addClass("e-focus");
                    currentEle.focus();
                }
            }
        },

        _hoverHandler: function (args) {
            args.preventDefault();
            if (this.model.enabled) {
                var index = $(this.items).index($(args.target).parent());
                if (index == -1)
                    index = $(this.items).index($(args.target));
                args.type === "mouseout" ? $(this.element.find("div.e-delete")[index]).css("visibility", "hidden") : $(this.element.find("div.e-delete")[index]).css("visibility", "visible");
                args.type === "mouseout" ? $(this.element.find("div.e-scroll")).css("visibility", "hidden") : $(this.element.find("div.e-scroll")).css("visibility", "visible");
                args.type === "mouseout" ? $(this.element.find("div.e-right-scroll")).css("visibility", "hidden") : $(this.element.find("div.e-right-scroll")).css("visibility", "visible");
                args.type === "mouseout" ? $(this.element.find("div.e-reload")[index]).css("visibility", "hidden") : $(this.element.find("div.e-reload")[index]).css("visibility", "visible");
            }
        },


        _wireEvents: function (event) {
            this._on(this.items, event, this._tabItemClick);
            this._on(this.itemsContainer, "mouseover", this._hoverHandler);
            this._on(this.itemsContainer, "mouseout", this._hoverHandler);
            this._on(this.element.find("div.e-delete"), "click", this._tabDeleteClick);
            this._on(this.element.find("div.e-scroll"), "click", this._tabScrollClick);
            this._on(this.element.find("div.e-right-scroll"), "click", this._tabScrollBackClick);
            this._on(this.itemsContainer, "focusin", this._focusIn);
            this._on(this.itemsContainer, "focusout", this._focusOut);
            this._on(this.element.find("div.e-reload"), "click", this._tabReloadClick);
        },


        _unWireEvents: function () {
            this._off(this.anchors, this.model.events);
            this._off(this.element.find("div.e-delete"), "click");
            this._off(this.element.find("div.e-scroll"), "click");
            this._off(this.element.find("div.e-right-scroll"), "click");
            this._off(this.itemsContainer, "mouseover", this._hoverHandler);
            this._off(this.itemsContainer, "mouseout", this._hoverHandler);
            this._off(this.itemsContainer, "focusin", this._focusIn);
            this._off(this.itemsContainer, "focusout", this._focusOut);
            this._off(this.element.find("div.e-reload"), "click");
        },
        _resizeEvents: function (value) {
            if (value === "fill") $(window).bind('resize', $.proxy(this._windowResized, this));
            else $(window).unbind('resize', $.proxy(this._windowResized, this));
        },


        _windowResized: function (e) {
            var maxHeight = this._getDimension($(this.element).parent(), "height");
            if (this._prevSize == maxHeight) return;
            else this._contentPaneSize();
            this._prevSize = maxHeight;
        },
        _contentPaneSize: function () {
            if (this.model.height != null && this.model.heightAdjustMode == "none") {
                $(this.element).height(this.model.height);
                var maxHeight = this._getDimension($(this.element), "height");
            }
            else var maxHeight = this._getDimension($(this.element).parent(), "height");
            $(this.contentPanels).height("");
            $(this.element).parent().css({ "overflow": "auto" });
            if (this.model.headerPosition === "top" || this.model.headerPosition === "bottom")
                maxHeight -= this._getDimension($(this.itemsContainer), "outerHeight");
            var maxPadding = 0;
            for (var i = 0; i < this.contentPanels.length; i++) {
                maxPadding = Math.max(maxPadding, this._getDimension($(this.contentPanels[i]), "outerHeight") - this._getDimension($(this.contentPanels[i]), "height"));
                if (this.model.height != null && this.model.heightAdjustMode == "none") $(this.contentPanels[i]).outerHeight(maxHeight).css({ "overflow": "auto" })
                else $(this.contentPanels).outerHeight(maxHeight - maxPadding).css({ "overflow": "auto" });
            }
        },
        _disableItems: function (indexes) {
            if (!this.model.enabled) return false;
            if (indexes != null) {
                for (var i = 0; i < indexes.length; i++) {
                    if ($.inArray(indexes[i], this.model.disabledItemIndex) == -1)
                        this.model.disabledItemIndex.push(indexes[i]);
                }
                this.model.disabledItemIndex.sort();
                this._disableTabs();
            }
        },

        _enableItems: function (indexes) {
            if (!this.model.enabled) return false;
            for (var i = 0; i < indexes.length; i++) {
                var index = indexes[i];
                !ej.isNullOrUndefined(this.contentPanels[index]) && $(this.contentPanels[index]).removeClass("e-disable");
                if ($.inArray(index, this.model.disabledItemIndex) == -1) {
                    return false;
                }
                this.model.disabledItemIndex = $.grep(this.model.disabledItemIndex, function (n, i) {
                    return n != index;
                });
            }
            this._refresh();
            this._disableTabs();
        },


        disable: function () {
            var indexes = [];
            for (var index = 0; index < this.getItemsCount() ; index++) {
                indexes.push(index);
            }
            this._disableItems(indexes);
            this.model.enabledItemIndex = [];
            this._unWireEvents();
        },

        enable: function () {
            var indexes = [];
            for (var index = 0; index < this.getItemsCount() ; index++) {
                if ($.inArray(index, this.model.enabledItemIndex) < 0) {
                    this.model.enabledItemIndex.push(index);
                    indexes.push(index);
                }
                this._enableItems(index);
            }
            this.model.disabledItemIndex = [];
        },

        getItemsCount: function () {
            if (this.items) {
                return this.items.length;
            }
        },

        addItem: function (url, displayLabel, index, cssClass, id) {
            (index >= 0 && index < this.items.length) ? this._addItemIndex = index : this._addItemIndex = this.items.length;
            for (var disable_index = 0; disable_index < this.model.disabledItemIndex.length; disable_index++) {
                if (this.model.disabledItemIndex[disable_index] >= index)
                    this.model.disabledItemIndex[disable_index]++;
            }
            if (this.model.headerPosition == "left") {
                this.items.length >= 0 && this.itemsContainer.addClass("e-left");
            }
            else if (this.model.headerPosition == "right") {
                this.items.length >= 0 && this.itemsContainer.addClass("e-right");
            }
            else
                this.items.length == 0 && this.itemsContainer.addClass("e-header");
            var liTag = ej.buildTag("li.e-select e-item");
            if ((this.model.headerPosition == "top"))
                if (this.model.enableRTL)
                    $(liTag).addClass("e-rtl-top-line e-top-hover");
                else
                    $(liTag).addClass("e-bottom-line");
            if (!ej.isNullOrUndefined(cssClass)) {
                var span = ej.buildTag('span').addClass(cssClass);
                liTag.append(span);
            }
            if (index === undefined && displayLabel === undefined && url != null)
                displayLabel = "Item";
            if (index === undefined && displayLabel === undefined && id === undefined) {
                url = "#Item" + this.items.length;
                displayLabel = "Item"
            }
            if (id != undefined) {
                if (id.indexOf("#") != 0)
                    id = "#" + id;
                this.divId = id;
            }
            else if (url != undefined)
                id = url;
            var aTag = ej.buildTag("a", displayLabel, {}, { href: url });
            if (this.model.headerPosition == "top" || this.model.headerPosition == "bottom")
                aTag.appendTo(liTag);
            if (this.model.showCloseButton) {
                var deleteIcon = ej.buildTag('div.e-icon e-delete', "", {}, {}).css("visibility", "hidden");
                liTag.append(deleteIcon);
                this._on(deleteIcon, "click", this._tabDeleteClick);
            }
            if (this.model.headerPosition == "left" || this.model.headerPosition == "right")
                aTag.appendTo(liTag);
            if (index === undefined) {
                index = this.anchors.length;
            }
            var insertIndex = index >= this.items.length;
            if (insertIndex) {
                liTag.appendTo(this.itemsContainer);
            } else {
                liTag.insertBefore(this.items[index]);
            }
            if (!ej.isNullOrUndefined(this.model.headerSize))
                this._setHeaderSize(this.model.headerSize);
            if (this.selectedItemIndex() == index) {
                this.hideItem[index];
                this.selectedItemIndex(this.selectedItemIndex() + 1);
            } else {
                this.hideItem[index];
                if (index < this.selectedItemIndex())
                    this.selectedItemIndex(this.selectedItemIndex() + 1);
            }
            this._itemsRefreshing();
            this._reinitialize();
            if (this.model.headerPosition == "top")
                $(this.contentPanels[index]).addClass("e-hidebottom");
            if (this.model.headerPosition == "bottom") {
                $(this.contentPanels[index]).addClass("e-hidetop");
                liTag.addClass("e-top-line e-item e-select e-margine-bottom")
            }
            if (this.model.headerPosition == "left")
                $(this.contentPanels[index]).addClass("e-hideleft");
            if (this.model.headerPosition == "right")
                $(this.contentPanels[index]).addClass("e-hideright");
            var data = {
                tabHeader: this.anchors[index],
                tabContent: this.contentPanels[index]
            };
            this._addItemIndex = null;
            this._onAdd(data);
        },

        removeItem: function (index) {
            if (!this.model.enabled) return false;
            if (index != null && index > -1 && index < this.items.length) {
                if (this._onBrforeRemove({ index: index }) === true)
                    return false;
                var removedTab = $(this.items[index]).remove();
                for (var disable_index = 0; disable_index < this.model.disabledItemIndex.length; disable_index++) {
                    if (this.model.disabledItemIndex[disable_index] > index)
                        this.model.disabledItemIndex[disable_index]--;
                    else if (this.model.disabledItemIndex[disable_index] == index) {
                        this.model.disabledItemIndex.splice(disable_index, 1);
                    }
                }
                if (removedTab.hasClass("e-active")) {
                    index == 0 ? this.selectedItemIndex(index + 1) : this.selectedItemIndex(index - 1);
                    this.showItem(this.selectedItemIndex());
                }
                $(this.element.find(">div.e-content")[index]).remove();
                this.contentPanels.splice(index, 1);
                index < this.selectedItemIndex() ? this.selectedItemIndex(this.selectedItemIndex() - 1) : this.selectedItemIndex();

                if (index < 0 || index >= this.anchors.length) {
                    this.selectedItemIndex(0);
                }
                if ((this.model.headerPosition == "left" || this.model.headerPosition == "right") && this.items.length == 1)
                    this._removeVerticalClass();
                else
                    this.items.length == 1 && this.itemsContainer.removeClass("e-header");
                this._unWireEvents();
                this._itemsRefreshing();
                this._wireEvents();
                this._disableTabs();
                var data = {
                    removedTab: removedTab
                };
                this._onRemove(data);
            }
        },

        show: function () {
            if (!this.model.enabled) return false;
            this.element.css("visibility", "visible");
        },

        hide: function () {
            if (!this.model.enabled) return false;
            this.element.css("visibility", "hidden");
        },

        _onBeforeLoad: function (link) {
            var data;
            if (this.selectedItemIndex() == -1 && this.model.collapsible) {
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: null,
                    activeIndex: null,
                    url: link
                };
            }
            else
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: this.items[this.selectedItemIndex()],
                    activeIndex: this.selectedItemIndex(),
                    url: link
                };
            return this._trigger("ajaxBeforeLoad", data);
        },

        _focusIn: function () {
            if (!this.model.readOnly && this.model.allowKeyboardNavigation)
                $(this.element).bind("keydown", $.proxy(this._keyPress, this));
        },

        _focusOut: function (e) {
            $(this.element).unbind("keydown", $.proxy(this._keyPress, this));
        },

        _onLoad: function (link) {
            var data;
            if (this.selectedItemIndex() == -1 && this.model.collapsible) {
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: null,
                    activeIndex: null,
                    url: link
                };
            }
            else
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: this.items[this.selectedItemIndex()],
                    activeIndex: this.selectedItemIndex(),
                    url: link
                };
            return this._trigger("ajaxLoad", data);
        },

        _onActive: function () {
            var data;
            if (this.selectedItemIndex() == -1 && this.model.collapsible) {
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: null,
                    activeIndex: null
                };
            }
            else
                data = {
                    prevActiveHeader: this.items[this._preTabIndex],
                    prevActiveIndex: this._preTabIndex,
                    activeHeader: this.items[this.selectedItemIndex()],
                    activeIndex: this.selectedItemIndex()
                };
            return this._trigger("itemActive", data);
        },

        _onBeforeActive: function (index) {
            if (this.model.beforeActive != null) {
                var data;
                if (this.selectedItemIndex() == -1 && this.model.collapsible) {
                    data = {
                        prevActiveHeader: this.items[this._preTabIndex],
                        prevActiveIndex: this._preTabIndex,
                        activeHeader: null,
                        activeIndex: null
                    };
                }
                else
                    data = {
                        prevActiveHeader: this.items[this._preTabIndex],
                        prevActiveIndex: this._preTabIndex,
                        activeHeader: this.items[index],
                        activeIndex: index
                    };
                return this._trigger("beforeActive", data);
            }
        },

        _onAdd: function (data) {
            return this._trigger("itemAdd", data);
        },

        _onRemove: function (data) {
            return this._trigger("itemRemove", data);
        },

        _onBrforeRemove: function (data) {
            return this._trigger("beforeItemRemove", data);
        }
    });

    ej.Tab.HeightAdjustMode = {
        /**  Panel height adjusts based on the content */
        Content: "content",
        /**  All panel height will be set the tallest panel height. */
        Auto: "auto",
        /**  Content panel take based on the parent height. */
        Fill: "fill",
		 /**  Content panel take based on the height property. */
        None: "none"
    };

    ej.Tab.Position = {
        /**  Tab headers display to top position. */
        Top: "top",
        /**  Tab headers display to bottom position. */
        Bottom: "bottom",
        /**  Tab headers display to left position. */
        Left: "left",
        /** Tab headers display to right position. */
        Right: "right"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the tagCloud control.
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    
    ej.widget("ejTagCloud", "ej.TagCloud", {
        
        element: null,
        
        model: null,
        validTags: ["div", "span"],
        _ignoreOnPersist: ["fields", "dataSource", "query", "create", "destroy", "click", "mouseout", "mouseover"],
        _rootCSS: "e-tagcloud",
        _setFirst: false,
        
        defaults: {
            
            cssClass : "",
			
            htmlAttributes : {},
            
            dataSource: null,
            
            query: null,
            
            fields:  { 
				
			    text: "text", 
				
			    url: "url", 
				
			    frequency: "frequency",
                
			    htmlAttributes: "htmlAttributes"
			},
            
            showTitle: true,
            
            titleText: "Title",
            
            titleImage: null,
            
            format: "cloud",
            
            enableRTL: false,
            
            minFontSize: "10px",
            
            maxFontSize: "40px",
                        
            mouseover: null,
            
            mouseout: null,
            
            click: null,
            
            create: null,
            
            destroy: null
        },
        
        dataTypes: {
            cssClass : "string",
            showTitle: "boolean",
            titleText: "string",
            titleImage: "string",
            format: "enum",
            enableRTL: "boolean",
            dataSource: "data",
            query: "data",
            fields: "data",
            htmlAttributes: "data"
        },

        
        
        _init: function () {
            this._initialize();
            this._render();
        },
        
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "fields":
                    case "query":
                    case "dataSource":
                    case "minFontSize":
                    case "maxFontSize":
                        this._refreshTagItems(option, options[option]); break;
                    case "showTitle": this._showTitle(options[option]); break;
                    case "titleText": this._title(options[option]); break;
                    case "titleImage": this._titleImage(options[option]); break;
                    case "cssClass ": this._changeSkin(options[option]); break;
                    case "format": this._format(options[option]); break;
                    case "enableRTL": this._rtl(options[option]); break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                }
            }
        },

        _refreshTagItems: function (key, value) {
            this.model[key] = value;
            this.ul.empty();
            this._checkDataBinding();
        },
        
        _showTitle: function (boolean) {
            if (boolean) this._generateTitle();
            else {
                this.titleText.remove();
                this.titleText = null;
            }
        },
        
        _title: function (text) {
            if (this.titleText) {
                if (text) {
                    if (this.text) this.text.html(text);
                    else this._generateTextTag(text);
                }
                else if (this.text) {
                    this.text.remove();
                    this.text = null;
                }
            }
        },
        
        _titleImage: function (imagePath) {
            if (this.titleText) {
                if (imagePath) {
                    if (this.image) this.image.attr("src", imagePath);
                    else this._generateImageTag(imagePath);
                }
                else if (this.image) {
                    this.image.remove();
                    this.image = null;
                }
            }
        },
        
        _changeSkin: function (skin) {
            if (this.model.cssClass  != skin) {
                this.element.removeClass(this.model.cssClass ).addClass(skin);
            }
        },
        
        insert: function (tag) {
            if ($.trim(tag.text)) {
                this.ul.append(this._generateLi(tag, this._getMapper()));
            }
        },
        
        insertAt: function (tag, position) {
            if ($.trim(tag.text)) {
                $(this.ul.children()[position - 1]).before(this._generateLi(tag, this._getMapper()));
            }
        },
        
        remove: function (txt) {
            var li = this.ul.children(), liTag, i;
            for (i = 0; i < li.length; i++) {
                liTag = $(li[i]);
                if (liTag.children()[0].innerHTML == txt)
                    liTag.remove();
            }
        },
        
        removeAt: function (position) {
            var li = this.ul.children();
            $(li[position - 1]).remove();
        },
        
        _format: function (format) {
            if (format == "cloud") {
                this.ul.removeClass("e-list");
                this.ul.addClass("e-cloud");
            }
            else if (format == "list") {
                this.ul.removeClass("e-cloud");
                this.ul.addClass("e-list");
            }
        },
        
        _destroy: function () {
            this.element.removeClass("e-widget " + this.model.cssClass );
            this.element.empty();
        },
        
        _initialize: function () {
            this.minFreq = 0;
            this.maxFreq = 30;
            this.ul = null;
            this.titleText = null;
            this.image = null;
            this.text = null;
        },
        
        _render: function () {
            this.element.addClass("e-widget " + this.model.cssClass );
            if (this.model.showTitle) this._generateTitle();
            this._renderWrapper();
            this._checkDataBinding();
			this._addAttr(this.model.htmlAttributes);
            this._checkProperties();
        },
        
        _generateTitle: function () {
            this.titleText = ej.buildTag("div.e-header e-title");
            if (this.model.titleImage)
                this._generateImageTag(this.model.titleImage);
            if (this.model.titleText)
                this._generateTextTag(this.model.titleText);
            if (this.ul) this.titleText.insertBefore(this.ul);
            else this.element.append(this.titleText);
        },
        
        _generateImageTag: function (titleImage) {	
			if(!this.image)
				this.image = $('<img class="e-title-img" src="' + titleImage + '" />');
			if(this.text && !this.model.titleImage) this.image.insertBefore(this.text);			
			else this.titleText.append(this.image);
        },		
        
        _generateTextTag: function (titleText) { 
			if(!this.text) this.text = ej.buildTag("span", titleText);					
			this.titleText.append(this.text);
        },		
        
        _renderWrapper: function () {
            var format;
            format = (this.model.format == "list") ? "list" : "cloud";
            this.ul = ej.buildTag("ul.e-ul e-box e-" + format);
            this.element.append(this.ul);
        },
        
        _renderItems: function (list) {
            this._generateTagItems(list);
            this.ul.removeClass("e-load");
        },
		_addAttr: function (htmlAttr) {
			var proxy = this;
			$.map(htmlAttr, function(value, key) {
				if (key == "class") proxy.element.addClass(value);
				else proxy.element.attr(key,value)
			});
		},

        _checkProperties: function () {
            if (this.model.enableRTL) this._rtl(this.model.enableRTL);
        },
        
        _rtl: function (boolean) {
            if (boolean) this.element.addClass("e-rtl");
            else this.element.removeClass("e-rtl");
        },
        
        _checkDataBinding: function () {
            var source = this.model.dataSource;
            if (source != null) {
                this.ul.addClass("e-load");
                if (ej.DataManager && source instanceof ej.DataManager)
                    this._initDataSource(source);
                else
                    this._renderItems(source);
            }
        },
        
        _initDataSource: function (source) {
            var proxy = this;
            var queryPromise = source.executeQuery(this._getQuery());
            queryPromise.done(function (e) {
                proxy._renderItems(e.result);
            }).fail(function (e) {
                proxy.ul.removeClass("e-load");
            });
        },
        
        _setAttributes: function (data, element) {
            if (data) {
                for (var key in data)
                    if (key == "class")
                        element.addClass(data[key]);
                    else
                        element.attr(key, (key == "style" ? element.attr("style") + ";" : "") + data[key]);
            }
        },

        _getQuery: function () {
            if (ej.isNullOrUndefined(this.model.query)) {
                var column = [], queryManager = ej.Query(), mapper = this.model.fields;
                for (var col in mapper) {
                    if (col !== "tableName" && mapper[col])
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.dataSource.dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = this.model.query;
            return queryManager;
        },

        _generateTagItems: function (list) {
            var i, mapField = this._getMapper(), callback = function (o) { return o[mapField._freq]; };
            var mappedArray = list.map ? list.map(callback) : $.map(list, callback);
            this.minFreq = Math.min.apply(Math, mappedArray);
            this.maxFreq = Math.max.apply(Math, mappedArray);

            for (i = 0; i < list.length; i++) {
                this.ul.append(this._generateLi(list[i], mapField));
            }
        },
        
        _getMapper: function () {
            var mapper = this.model.fields, mapFld = { _text: null, _freq: null, _url: null };
            mapFld._text = (mapper && mapper.text) ? mapper["text"] : "text";
            mapFld._freq = (mapper && mapper.frequency) ? mapper["frequency"] : "frequency";
            mapFld._url = (mapper && mapper.url) ? mapper["url"] : "url";
            mapFld._attr = (mapper && mapper.htmlAttributes) ? mapper["htmlAttributes"] : "htmlAttributes";
            return mapFld;
        },
        
        _generateLi: function (list, map) {
            var li = ej.buildTag("li.e-tagitems"), aTag;
            aTag = ej.buildTag("a.e-txt", list[map._text] || list[map._url],
                    { "font-size": this._calculateFontSize(list[map._freq]) },
                    { role: "link" });
            if (list[map._url]) aTag.attr({ "href": list[map._url], "target": "blank" });
            this._setAttributes((list[map._attr]), aTag);
            li.append(aTag);
            this._on(aTag, "mouseenter", this._mouseEnter);
            this._on(aTag, "mouseleave", this._mouseLeave);
            this._on(aTag, "click", this._mouseClick);
            return li;
        },

        
        _mouseEnter: function (e) {
            $(e.target).addClass("hover");
            this._raiseEvent(e, "mouseover");
        },
        
        _mouseLeave: function (e) {
            $(e.target).removeClass("hover");
            this._raiseEvent(e, "mouseout");
        },
        
        _mouseClick: function (e) {
            $(e.target).removeClass("hover");
            this._raiseEvent(e, "click");
        },

        _raiseEvent: function (e, _type) {
            this._trigger(_type, { value: $(e.target).html(), url: $(e.target).attr('href'), eventType: _type, target: $(e.target) });
        },
        
        _calculateFontSize: function (frequency) {
            if (frequency) {
                var C = 2, k, fontRange, fontSize,
                minSize = parseInt(this.model.minFontSize, 10), maxSize = parseInt(this.model.maxFontSize, 10);
                k = (frequency - this.minFreq) / (this.maxFreq - this.minFreq);
                fontRange = maxSize - minSize;
                fontSize = minSize + (C * Math.floor(k * (fontRange / C)));
                return fontSize;
            }
            else return this.model.minFontSize;
        }
    });
    
    ej.Format = {
        /**  Render the tagCloud items in cloud format */
        Cloud: "cloud",
        /**  Render the tagCloud items in list format */
        List: "list"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html TileView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejTileBase", "ej.TileBase", {
        defaults: {

            badge: {

                enabled: false,

                value: 1,

                maxValue: 100,

                minValue: 1,

                text: null
            },

            captionTemplateId: null,

            enablePersistence: false,

            imageClass: null,

            imagePosition: "center",

            imageTemplateId: null,

            imageUrl: null,

            liveTile: {

                enabled: false,

                imageClass: null,

                imageTemplateId: null,

                imageUrl: null,

                type: "flip",

                updateInterval: 2000
            },

            showText: true,

            cssClass: "",

            text: "Text",

            textAlignment: "normal",

            tileSize: "small",

            width: null,

            height: null
        },
        dataTypes: {
            badge: { enabled: "boolean", value: "number", maxValue: "number", minValue: "number" },
            imagePosition: "enum",
            liveTile: {
                type: "enum",
                enabled: "boolean",
                updateInterval: "number",
                imageUrl: "array",
                imageClass: "array",
                imageTemplateId: "array"
            },
            showText: "boolean",
            textAlignment: "enum",
            tileSize: "enum",
            width: "number",
            height: "number"
        },

        _tileRender: function () {
            this.element.addClass(this._prefix + "tile " + this._prefix + "tile-wrapper " + this._prefix + "tile-wrapper-" + this.model.tileSize + " " + this.model.cssClass);
            this._mainTile = ej.buildTag("div." + this._prefix + "child-tile " + this._prefix + "tile-image" + this.model.imagePosition);
            this.model.liveTile.enabled && this.model.liveTile.type == "flip" ? this.element.addClass(this._prefix + 'tile-' + this.model.liveTile.type) : this.element.addClass(this._prefix + 'tile-common');
            this.element.append(this._mainTile);
            this._image = ej.buildTag("div." + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
            this._layer = ej.buildTag("div." + this._prefix + "layer");
            this._text = ej.buildTag("div").addClass(this._prefix + "tile-text " + this._prefix + "user-select " + this._prefix + "textalign-" + this.model.textAlignment);
            if (this.model.showText)
                this.element.addClass(this._prefix + "enabled-text");
            else
                this.element.removeClass(this._prefix + "enabled-text").addClass(this._prefix + "disabled-text");
            //Live Tile implementation
            if ((!this._isAppView() || this.model.renderMode == "windows") && this.model.liveTile.enabled) {
                this.element.addClass(this._prefix + "live-type-" + this.model.liveTile.type);
                this._mainTile.addClass(this._prefix + "live-tile-enable");
                if (!this._liveImageWrapper) {
                    this._liveImageWrapper = ej.buildTag("div." + this._prefix + "live-image-wrapper");
                    if (this.model.liveTile.imageTemplateId) {
                        this._liveImageWrapper.addClass(this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
                        for (var i = 0; i < this.model.liveTile.imageTemplateId.length; i++) {
                            var template = this._isAppView() ? ej.getCurrentPage().find('#' + this.model.liveTile.imageTemplateId[i]).addClass(this._prefix + "tile-template " + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type) :
                                $('#' + this.model.liveTile.imageTemplateId[i]).addClass(this._prefix + "tile-template " + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type);
                            this._liveImageWrapper.append(template);
                        }
                        this._mainTile.addClass(this._prefix + "tile-template");
                    }
                    else {
                        if (this.model.liveTile.imageClass) {
                            for (var i = 0; i < this.model.liveTile.imageClass.length; i++)
                                this._liveImageWrapper.append(ej.buildTag("div").addClass(this.model.liveTile.imageClass[i] + " " + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type));
                        }
                        else {
                            for (var i = 0; i < this.model.liveTile.imageUrl.length; i++) {
                                var url = this._isAppView() && this.model.imagePath ? this._getAbsolutePath(this.model.imagePath) + "/windows/" + this.model.liveTile.imageUrl[i] : this._getAbsolutePath(this.model.liveTile.imageUrl[i]);
                                this._liveImageWrapper.append(ej.buildTag("div." + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type).css({ "background-image": "url('" + url + "')" }));
                            }
                        }
                    }
                    $(this._liveImageWrapper.children()[0]).addClass(this._prefix + "tile-" + this.model.liveTile.type + 'back').removeClass(this._prefix + "tile-" + this.model.liveTile.type);
                }
                this._mainTile.append(this._liveImageWrapper);
                if (this.model.imageTemplateId && this._prefix == "e-m-" && (App.angularAppName || ej.angular.defaultAppName))
                    ej.angular.compile(this._liveImageWrapper);
            }
                //Normal Tile implementation
            else {
                if (this.model.imageTemplateId) {
                    var imageTemp = this._isAppView() ? ej.getCurrentPage().find('#' + this.model.imageTemplateId).addClass(this._prefix + "tile-template") : $('#' + this.model.imageTemplateId).addClass(this._prefix + "tile-template");
                    this._image.append(imageTemp);
                }
                else {
                    this._image = ej.buildTag("div").addClass(this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition);
                    if (this.model.imageClass)
                        this._image.addClass(this.model.imageClass);
                    else if (this.model.imageUrl) {
                        var url = this._isAppView() && this.model.imagePath ? (this._getAbsolutePath(this.model.imagePath) + "/" + this.model.renderMode.toLowerCase() + "/" + this.model.imageUrl) : this._getAbsolutePath(this.model.imageUrl);
                        this._image.css({ "background-image": "url('" + url + "')" });
                        if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && this.model.imagePosition == "fill")
                            this._image.css({ "filter": "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + url + "',sizingMethod='scale')", "-ms-filter": "progid:DXImageTransform.Microsoft.AlphaImageLoader(src='" + url + "',sizingMethod='scale')" });
                    }
                }
                this._mainTile.append(this._image).addClass(this._prefix + "tile-image" + this.model.imagePosition);
                if (this.model.imageTemplateId && this._prefix == "e-m-" && (App.angularAppName || ej.angular.defaultAppName))
                    ej.angular.compile(this._image);
            }

            if (this.model.badge.enabled)
                this._setBadgeEnabled();

            if (this._text) {
                if (this.model.captionTemplateId) {
                    var templateContent = this._isAppView() ? ej.getCurrentPage().find('#' + this.model.captionTemplateId) : $('#' + this.model.captionTemplateId);
                    this._template = templateContent[0] && templateContent[0].nodeName == "SCRIPT" ? ej.getClearString(templateContent[0].innerHTML) : templateContent;
                    this._text.append(this._template).addClass(this._prefix + "template-caption");
                    this._mainTile.addClass(this._prefix + "template-caption")
                }
                else
                    this._text.html(this.model.text);
                this._mainTile.append(this._text);
                if (this._prefix == "e-m-" && (App.angularAppName || ej.angular.defaultAppName))
                    ej.angular.compile(this._text);
            }

            if (this._isCustomizeSize()) {
                this.element.css({ "width": this.model.width ? this.model.width : "", "height": this.model.height ? this.model.height : "" });
                if (this.model.imagePosition != "fill") {
                    if (this.model.renderMode == "windows" || this.model.renderMode == "flat" || !this._isAppView()) {
                        this.model.showText ? this.element.find("." + this._prefix + "tile-image").css({ "width": (this.model.width) - 20, "height": (this.model.height) - 45 }) : this.element.find("." + this._prefix + "tile-image").css({ "width": (this.model.width) - 20, "height": (this.model.height) - 20 });
                        this.model.textAlignment == "left" || ((this.model.renderMode == "windows" || this.model.renderMode == "flat" || !this._isAppView()) && this.model.textAlignment == "normal") ? this.element.find("." + this._prefix + "tile-text").css({ "width": this.model.width - 14 }) : "";
                    }
                    else
                        this.model.showText ? this.element.find("." + this._prefix + "tile-image").css({ "width": (this.model.width) - 14, "height": (this.model.height) - 39 }) : this.element.find("." + this._prefix + "tile-image").css({ "width": (this.model.width) - 14, "height": (this.model.height) - 14 });
                }
                else {
                    this.element.find("." + this._prefix + "tile-image").css({ "width": this.model.width, "height": this.model.height });
                    this.model.textAlignment == "left" || ((this.model.renderMode == "windows" || this.model.renderMode == "flat" || !this._isAppView()) && this.model.textAlignment == "normal") ? this.element.find("." + this._prefix + "tile-text").css({ "width": this.model.width - 14 }) : "";
                }
                if (this._isBelowSmallSize()) {
                    this.element.find("." + this._prefix + "tile-text").css("display", "none");
                    this.element.find("." + this._prefix + "tile-image").css({ "margin": 0, "width": this.model.width, "height": this.model.height });
                }
                else
                    this.element.find("." + this._prefix + "tile-text").css("display", "block");
                this.model.tileSize == "small" ? this.element.removeClass(this._prefix + "tile-wrapper-small").addClass(this._prefix + "tile-wrapper-medium") :
                this._isBelowSmallSize() ? (this.model.renderMode == "windows" || this.model.renderMode == "flat" || !this._isAppView()) ? this.element.removeClass(this._prefix + "tile-wrapper-" + this.model.tileSize).addClass(this._prefix + "tile-wrapper-small") : this.element.removeClass(this._prefix + "tile-wrapper-" + this.model.tileSize).addClass(this._prefix + "tile-wrapper-medium") : "";
            }
        },
        _isBelowSmallSize: function () {
            return ((this.model.width <= 70 && this.model.width != null) || (this.model.height <= 70 && this.model.height != null));
        },
        _isCustomizeSize: function () {
            return this.model.height || this.model.width;
        },

        _createDelegates: function () {
            this._onStartDelegate = $.proxy(this._isAppView() ? this._onTouchStartHandler : this._onMouseDownHandler, this);
            this._onEndDelegate = $.proxy(this._isAppView() ? this._onTouchEndHandler : this._onMouseUpHandler, this);
            this._onMoveDelegate = $.proxy(this._isAppView() ? this._onTouchMoveHandler : this._onMouseMoveHandler, this);
            this._onDocClickDelegate = $.proxy(this._onDocClickHandler, this);
            this._startAnimationDelegate = $.proxy(this["_" + this.model.liveTile.type + "Tile"], this);
        },

        _wireEvents: function (remove) {
            this._createDelegates();
            ej.listenEvents([this.element, $(document)], [ej.startEvent(), ej.endEvent()], [this._onStartDelegate, this._onDocClickDelegate], remove, this);
            if (this.model.liveTile.enabled && (!this._isAppView() || this.model.renderMode == "windows"))
                remove ? this._stopTileAnimation() : this._startTileAnimation();
        },

        _onDocClickHandler: function (evt) {
            this._removeActiveClass();
        },

        _removeActiveClass: function () {
            (!this._isAppView() || this.model.renderMode == "windows") ? ej._removeSkewClass(this.element) : $(this.element).removeClass(this._prefix + "state-active");
        },

        _startTileAnimation: function () {
            this._intervalCounter = setInterval(this._startAnimationDelegate, this.model.liveTile.updateInterval);
        },

        _stopTileAnimation: function () {
            clearInterval(this._intervalCounter);
        },

        _flipTile: function () {
            var front = this.element.find("." + this._prefix + "tile-flipback");
            var back = this.element.find("." + this._prefix + "tile-flip").first();
            front.addClass(this._prefix + "tile-flip").removeClass(this._prefix + "tile-flipback");
            back.addClass(this._prefix + "tile-flipback").removeClass(this._prefix + "tile-flip");
        },

        _slideTile: function () {
            this._animateEffect();
        },

        _carouselTile: function () {
            this._animateEffect();
        },

        _animateEffect: function () {
            var type = this.model.liveTile.type.toLowerCase();
            var items = this.element.find("." + this._prefix + "live-tile");
            var currentitem = this.element.find("." + this._prefix + "tile-" + type + "back");
            var nextitem = currentitem.next()[0] == null ? items.first() : currentitem.next();
            var previtem = currentitem.prev()[0] == null ? items.last() : currentitem.prev();
            previtem.addClass(this._prefix + "tile-" + type).removeClass(this._prefix + "tile-" + type + "up");
            currentitem.addClass(this._prefix + "tile-" + type + "up").removeClass(this._prefix + "tile-" + type + "back");
            nextitem.removeClass(this._prefix + "tile-" + type).addClass(this._prefix + "tile-" + type + "back");
        },

        _getAbsolutePath: function (path) {
            return this._isAppView() ? App.route.makeUrlAbsolute(path, true).toString() : path;
        },
        _isAppView: function () {
            return this._prefix == "e-m-" ? true : false;
        },

        updateTemplate: function (id, index) {
            if (this.model.liveTile.enabled) {
                $(this._liveImageWrapper).children()[index].remove();
                var tempLive = this._isAppView() ? ej.getCurrentPage().find('#' + id).addClass(this._prefix + "tile-template " + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type) :
                    $('#' + id).addClass(this._prefix + "tile-template " + this._prefix + "live-tile " + this._prefix + "fontimage " + this._prefix + "tile-image " + this._prefix + "tile-image" + this.model.imagePosition + " " + this._prefix + "tile-" + this.model.liveTile.type);
                if ($(this._liveImageWrapper).children()[index - 1])
                    $($(this._liveImageWrapper).children()[index - 1]).after(tempLive);
                else
                    $($(this._liveImageWrapper).children()[index + 1]).before(tempLive);
            }
            else {
                var template = this._isAppView() ? ej.getCurrentPage().find('#' + id).addClass(this._prefix + "tile-template") : $('#' + id).addClass(this._prefix + "tile-template");
                this._image.empty().append(template);

            }
        },

        _setModel: function (options) {
            var refresh = false;
            for (var property in options) {
                var setModel = "_set" + property.charAt(0).toUpperCase() + property.slice(1);
                if (this[setModel])
                    this[setModel](options[property]);
                else
                    refresh = true
            }
            if (refresh)
                this._refresh();
        },

        _setImagePosition: function (value) {
            if (this.model.liveTile.enabled && $($(this._mainTile.children()[0]).children()).hasClass(this._prefix + "tile-image")) {
                $($(this._mainTile.children()[0]).children()).removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this._mainTile.removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this.model.imagePosition = value;
            }
            else {
                $(this._mainTile.children()[0]).removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this._mainTile.removeClass(this._prefix + "tile-imagecenter " + this._prefix + "tile-imagetop " + this._prefix + "tile-imagebottom " + this._prefix + "tile-imageright " + this._prefix + "tile-imageleft " + this._prefix + "tile-imagetopleft " + this._prefix + "tile-imagetopright " + this._prefix + "tile-imagebottomright " + this._prefix + "tile-imagebottomleft " + this._prefix + "tile-imagefill").addClass(this._prefix + "tile-image" + value);
                this.model.imagePosition = value;
            }
        },

        _setShowText: function (showtext) {
            this.model.showText = showtext;
            if (showtext) {
                this.element.removeClass(this._prefix + "disabled-text").addClass(this._prefix + "enabled-text");
                this._setText(this.model.text);
                this._isAppView() && this.model.renderMode != "windows" && this.model.renderMode != "flat" && this.model[this.model.renderMode].textPosition == "outer" ? this.element.removeClass(this._prefix + "text-inner").addClass(this._prefix + "text-outer") : "";
            }
            else {
                this.element.removeClass(this._prefix + "enabled-text").addClass(this._prefix + "disabled-text");
                this._isAppView() && this.model.renderMode != "windows" && this.model.renderMode != "flat" && this.model[this.model.renderMode].textPosition == "outer" ? this.element.removeClass(this._prefix + "text-outer").addClass(this._prefix + "text-inner") : "";
            }
        },

        _setText: function (text) {
            if (this.model.showText) {
                if (this.element.find('.' + this._prefix + "tile-text").length)
                    this.element.find('.' + this._prefix + "tile-text").html(text);
                else
                    this.element.prepend(ej.buildTag("div." + this._prefix + "tile-text", text));
            }
        },

        _setTextPosition: function (position) {
            if (position == "outer")
                this.element.removeClass(this._prefix + "text-inner").addClass(this._prefix + "text-outer")
            else
                this.element.removeClass(this._prefix + "text-outer").addClass(this._prefix + "text-inner").find("." + this._prefix + "child-tile");
        },

        _setTextAlignment: function (value) {
            $(this.element.find("." + this._prefix + "tile-text")).removeClass(this._prefix + "textalign-left " + this._prefix + "textalign-right " + this._prefix + "textalign-normal " + this._prefix + "textalign-center").addClass(this._prefix + "textalign-" + value);
            this.model.textAlignment = value;
        },

        _setBadge: function (options) {
            if (this.model.badge.enabled) {
                this._setBadgeEnabled(true);
                for (var property in options) {
                    var setModel = "_setBadge" + property.charAt(0).toUpperCase() + property.slice(1);
                    if (this[setModel])
                        this[setModel](options[property]);
                }
            }
            else
                this._setBadgeEnabled(false);
        },

        _setBadgeValue: function () {
            var badgeValue = this.model.badge.text ? this.model.badge.text : ((this.model.badge.value <= this.model.badge.maxValue && this.model.badge.value >= this.model.badge.minValue) ? this.model.badge.value.toString() : (this.model.badge.value > this.model.badge.maxValue ? this.model.badge.maxValue.toString() + "+" : this.model.badge.minValue.toString()));
            this.element.find('.' + this._prefix + "tile-badge").html(badgeValue);
        },

        _setBadgeEnabled: function (enabled) {
            if (this.element.find('.' + this._prefix + "tile-badge").length)
                this.element.find("." + this._prefix + "tile-badge")[enabled ? "show" : "hide"]();
            else {
                this.element.prepend(ej.buildTag("div." + this._prefix + "tile-badge"));
                this._setBadgeValue();
            }
        },

        _setBadgeMaxValue: function (value) {
            this.model.badge.maxValue = value;
            this._setBadgeValue();
        },

        _setBadgeMinValue: function (value) {
            this.model.badge.minValue = value;
            this._setBadgeValue();
        },

        _setBadgeText: function (value) {
            this.model.badge.text = value;
            this._setBadgeValue();
        },

        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },

        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        }
    });
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html TileView elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejTile", "ej.Tile", {
        _setFirst: true,
        validTags: ["div"],
        _rootCSS: "e-tile",
        defaults: {

            mouseDown: null,

            mouseUp: null
        },

        _init: function () {
            this._orgEle = this.element.clone();
            this._renderEJControl();
        },

        _renderEJControl: function () {
            if (this.model.captionTemplateId) {
                this._captionTemplateParent = $("#" + this.model.captionTemplateId).parent();
                this._cloneCaptionTemplateElement = $("#" + this.model.captionTemplateId).clone();
            }
            if (this.model.imageTemplateId) {
                this._imageTemplateParent = $("#" + this.model.imageTemplateId).parent();
                this._cloneImageTemplateElement = $("#" + this.model.imageTemplateId).clone();
            }
            if (this.model.liveTile.enabled && this.model.liveTile.imageTemplateId) {
                this._liveTileimageTemplateParent = $("#" + this.model.liveTile.imageTemplateId).parent();
                this._cloneLiveTileImageTemplateElement = $("#" + this.model.liveTile.imageTemplateId).clone();
            }
            this.element.addClass("e-tile-web");
            this._prefix = "e-";
            this._tileRender();
            this._mainTile.append(this._layer);
            this._wireEvents();
        },

        _onMouseDownHandler: function (evt) {
            var item = this.element;
            item.addClass(ej._getSkewClass(item, evt.pageX, evt.pageY).replace('e-m-', 'e-'));
            this._trigger("mouseDown", { text: item.find(".e-tile-text").html(), index: $(evt.target).index() });
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, false, this);
            ej.listenTouchEvent(this.element, ej.endEvent(), this._onEndDelegate, false, this);
            this._isMoved = false;
        },

        _onMouseMoveHandler: function (evt) {
            this._isMoved = true;
            this._removeActiveClass();
        },

        _onMouseUpHandler: function (evt) {
            if (!this._isMoved && this.model.mouseUp)
                this._trigger("mouseUp", { text: this.element.find(".e-tile-text").html(), index: $(evt.target).index() });
            this._removeActiveClass();
            this._isMoved = false;
            ej.listenTouchEvent(this.element, ej.moveEvent(), this._onMoveDelegate, true, this);
            ej.listenTouchEvent(this.element, ej.endEvent(), this._onEndDelegate, true, this);
        },

        _refresh: function () {
            this._destroy();
            this.element.addClass("e-tile e-js");
            if (this.model.captionTemplateId)
                $(this._captionTemplateParent).append(this._cloneCaptionTemplateElement);
            if (this.model.imageTemplateId)
                $(this._imageTemplateParent).append(this._cloneImageTemplateElement);
            if (this.model.liveTile.enabled && this.model.liveTile.imageTemplateId)
                $(this._liveTileimageTemplateParent).append(this._cloneLiveTileImageTemplateElement);
            this._renderEJControl();
        }
    });

    ej.Tile.ImagePosition = {
        /**
        Enum for center position of tile image*/
        Center: "center",
        /**
        Enum for top position of tile image*/
        Top: "top",
        /**
        Enum for bottom position of tile image*/
        Bottom: "bottom",
        /**
        Enum for right position of tile image*/
        Right: "right",
        /**
        Enum for left position of tile image*/
        Left: "left",
        /**
        Enum for topleft position of tile image*/
        TopLeft: "topleft",
        /**
        Enum for topright position of tile image*/
        TopRight: "topright",
        /**
        Enum for bottomright position of tile image*/
        BottomRight: "bottomright",
        /**
        Enum for bottomleft position of tile image*/
        BottomLeft: "bottomleft",
        /**
        Enum for fill position of tile image*/
        Fill: "fill"
    },

    ej.Tile.TextAlignment = {
        /**
        Enum for normal alignment of tile text*/
        Normal: "normal",
        /**
        Enum for left alignment of tile text*/
        Left: "left",
        /**
        Enum for right alignment of tile text*/
        Right: "right",
        /**
        Enum for center alignment of tile text*/
        Center: "center"
    },
    ej.Tile.TextPosition = {
        /**
        Enum for inner position of tile text*/
        Inner: "inner",
        /**
        Enum for outer position of tile text*/
        Outer: "outer"
    },

    ej.Tile.TileSize = {
        /**
        Enum for medium size of tile*/
        Medium: "medium",
        /**
        Enum for small size of tile*/
        Small: "small",
        /**
        Enum for large size of tile*/
        Large: "large",
        /**
        Enum for wide size of tile*/
        Wide: "wide"
    },

    ej.Tile.LiveTileType = {
        /**
        Enum for flip type of livetile*/
        Flip: "flip",
        /**
        Enum for slide type of livetile*/
        Slide: "slide",
        /**
        Enum for carousel type of livetile*/
        Carousel: "carousel"
    }
    $.extend(true, ej.Tile.prototype, ej.TileBase.prototype);
})(jQuery, Syncfusion);;
// JavaScript source code
/**
* @fileOverview Plugin to style the Html RadialSlider elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejRadialSliderBase", "ej.RadialSliderBase", {
        defaults: {
            radius: 200,
            endAngle: 360,
            startAngle: 0,
            ticks: [0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100],
            enableRoundOff: true,
            value: 10,
            strokeWidth: 2,
            autoOpen: true,
            enableAnimation: true,
            cssClass: null,
            innerCircleImageClass: null,
            innerCircleImageUrl: null,
            showInnerCircle: true,
            inline: false
        },
        dataTypes: {
            renderMode: "enum",
            theme: "enum",
            ticks: "array",
            radius: "number",
            endAngle: "number",
            enableRoundOff: "boolean",
            strokeWidth: "number",
            enableAnimation: "boolean",
            cssClass: "string",
            innerCircleImageClass: "string",
            innerCircleImageUrl: "string",
            showInnerCircle: "boolean"
        },
        observables: ["value"],
        observableArray: ["ticks"],
        value: ej.util.valueFunction("value"),
        ticks: ej.util.valueFunction("ticks"),
        _initialization: function () {
            this.element.addClass(this.model.cssClass);
            this._svgLink = "http://www.w3.org/2000/svg";
            this._startXY = this.model.radius - (this.model.inline ? 50 : 0);
            this._diameter = 2 * this.model.radius;
            this.model.radius = this._startXY;
            this._tickCount = this.ticks().length;
            this._labelSpacing = this.model.inline ? 0 : 200;
            this.element.css({ "width": this._diameter + this._labelSpacing, "height": this._diameter + this._labelSpacing });
            this._radialSliderWrapper = ej.buildTag("div", {}, {}, { "class": this._prefix + "radail-slider-wrapper" }).css({ "width": this._diameter + this._labelSpacing, "height": this._diameter + this._labelSpacing });
            if (this.model.showInnerCircle) {
                this._innerCircle = ej.buildTag("div", {}, {}, { "class": this._prefix + "inner-circle" }).css({ "left": (((this._diameter + this._labelSpacing) / 2) - (20 + this.model.strokeWidth + 1)), "top": (((this._diameter + this._labelSpacing) / 2) - (20 + this.model.strokeWidth + 1)), "border-width": this.model.strokeWidth + 1 });
                if (this.model.innerCircleImageClass)
                    this._innerCircle.addClass(this.model.innerCircleImageClass);
                else
                    this._innerCircle.css({ "background-image": "url('" + this.model.innerCircleImageUrl + "')" });
                this._radialSliderWrapper.append(this._innerCircle);
            }
            this.element.append(this._radialSliderWrapper);
        },
        _renderControl: function () {
            //svg element creaton
            this._radialSVG = $(this._createSVGElements("svg", { "id": this._prefix + "radial-slider-svg", "width": this._diameter + this._labelSpacing, "height": this._diameter + this._labelSpacing }));
            //Outer line circle group
            var pathTranslate = this.model.inline ? 50 : 100, textTranslateX = this.model.inline ? 45 : 95, textTranslateY = this.model.inline ? 55 : 105;
            this._pathLineGroupElement = $(this._createSVGElements("g", { "id": "outerLineCircle", "transform": "translate(" + pathTranslate + "," + pathTranslate + ")" }));
            this._lineDirection = this._outerLineCalculation(50, 40, this._tickCount, true, true, "path");
            //Outer test  circle group
            this._textGroupElement = $(this._createSVGElements("g", { "id": "outerTextCircle", "transform": "translate(" + textTranslateX + "," + textTranslateY + ")" }));
            //Circle group
            this._circleGroupElement = $(this._createSVGElements("g", { "id": "circlegroup", "transform": "translate(" + pathTranslate + "," + pathTranslate + ")" }));
            this._circlePath = this._createSVGElements("path", { "id": "circlepath", "d": this._circleArcDirection(this.model.endAngle), "class": this._prefix + "radialarcdefault", "fill": "none", "stroke-width": this.model.strokeWidth });
            this._circleGroupElement.append(this._circlePath);
            this._circleGroupElement.append(this._radialCircle);
            this._outerTextDirection = this._outerTextCalculation(this._tickCount);
            // Text value set in corresponding point
            for (var i = 0; i < this._tickCount + 1; i++) {
                var k = (i == 0) && (this.model.startAngle == 0) && (this.model.endAngle == 360) ? (this._tickCount - 1) : i;
                this._outerTextElement;
                if (i == this._tickCount) {
                    if ((this.ticks()[0] == this.value()))
                        continue;
                    else
                        this._outerTextElement = this._createSVGElements("text", { "stroke-width": 0.5, "x": this._outerTextDirection[i].X2, "y": this._outerTextDirection[i].Y2, "class": this._prefix + "dynamic-text", "textContent": this.value(), "text-anchor": this._outerTextDirection[i].textAlignment });
                }
                else
                    this._outerTextElement = this._createSVGElements("text", { "stroke-width": 0.5, "x": this._outerTextDirection[i].X2, "y": this._outerTextDirection[i].Y2, "class": this._prefix + "ticks-text", "textContent": this.ticks()[k], "text-anchor": this._outerTextDirection[i].textAlignment });
                this._textGroupElement.append(this._outerTextElement);
            }
            this._pathLineElement = this._createSVGElements("path", { "class": this._prefix + "radial-needle", "d": this._lineDirection, "fill": "none", "stroke-width": this.model.strokeWidth });
            this._pathLineGroupElement.append(this._pathLineElement);
            this._radialSliderWrapper.append(this._radialSVG.append(this._pathLineGroupElement).append(this._textGroupElement).append(this._circleGroupElement));
            if (this.model.autoOpen)
                this.show();
            else
                this.hide();
        },
        _outerTextCalculation: function (length) {
            var radius = this.model.radius;
            var startAngle = this.model.startAngle,
                 endAngle = startAngle + ((this.model.endAngle - this.model.startAngle) / (length - 1));
            this._point = ((this.model.endAngle - this.model.startAngle) / (length - 1));
            var endValueCal = endAngle,
                startValueCal = endValueCal;
            var textPoints = [];
            for (var j = 0; j < length; j++) {
                var line = {};
                var angleCondition = startAngle;
                startAngle = startAngle * Math.PI / 180.0;
                endAngle = endAngle * Math.PI / 180.0;
                line.X2 = (this._startXY) + (radius + 30) * Math.cos(startAngle);
                line.Y2 = (this._startXY) + (radius + 30) * Math.sin(startAngle);
                if (angleCondition <= 270 && 90 <= angleCondition)
                    line.textAlignment = "middle";
                else
                    line.textAlignment = "start";
                startAngle = endValueCal;
                endAngle = endValueCal + this._point;
                endValueCal += this._point;
                textPoints.push(line);
            }
            var line = {};
            var angleCondition = this._startValueAngle;
            this._startValueAngle = this._startValueAngle * Math.PI / 180.0;
            line.X2 = (this._startXY) + (radius + 30) * Math.cos(this._startValueAngle);
            line.Y2 = (this._startXY) + (radius + 30) * Math.sin(this._startValueAngle);
            if (angleCondition <= 270 && 90 <= angleCondition)
                line.textAlignment = "middle";
            else
                line.textAlignment = "start";
            textPoints.push(line);
            return textPoints;
        },
        _outerLineCalculation: function (dis1, dis2, length, isOuterLine, isinLine, element) {
            var radius = this.model.radius;
            var startAngle = this.model.startAngle,
                 endAngle = startAngle + ((this.model.endAngle - this.model.startAngle) / (length - 1));
            this._point = ((this.model.endAngle - this.model.startAngle) / (length - 1));
            var endValueCal = endAngle,
                startValueCal = endValueCal;
            var textPoints = [];
            this._degPoint = [];
            this._degPoint.push(startAngle);
            if (isOuterLine) {
                for (var j = 0; j < length; j++) {
                    var line = {};
                    startAngle = startAngle * Math.PI / 180.0;
                    endAngle = endAngle * Math.PI / 180.0;
                    this._degPoint.push(endValueCal);
                    if (element != "text") {
                        line.X1 = this._startXY + (radius) * Math.cos(startAngle);
                        line.Y1 = this._startXY + (radius) * Math.sin(startAngle);
                    }
                    line.X2 = this._startXY + (radius + 10) * Math.cos(startAngle);
                    line.Y2 = this._startXY + (radius + 10) * Math.sin(startAngle);
                    startAngle = endValueCal;
                    endAngle = endValueCal + this._point;
                    endValueCal += this._point;
                    element == "path" ? this._directionLine.append([" M", line.X1, line.Y1, "L", line.X2, line.Y2].join(" ")) : textPoints.push(line);
                }
            }
            if (isinLine) {
                for (var i = 0; i < this.ticks().length; i++) {
                    var k = i + 1;
                    var lastValue = i == (this.ticks().length - 1) ? this.ticks()[this.ticks().length - 1] + 0.5 : this.ticks()[k];
                    var firstValue = i == 0 ? this.ticks()[i] - 0.5 : this.ticks()[i];
                    var ticksSecond = firstValue <= 0 && firstValue >= this.value() ? true : false;
                    var ticksFirst = lastValue <= 0 && lastValue <= this.value() ? true : false;
                    var condition = ticksSecond && ticksFirst ? true : (firstValue <= this.value() && this.value() < lastValue);
                    if (condition) {
                        if (this.ticks()[i] != this.value()) {
                            var difference = this._point / (this.ticks()[k] - this.ticks()[i]);
                            var angleDifference = this.ticks()[k] - this.value();
                            this._startValueAngle = this._degPoint[k] - (difference * angleDifference);
                            this._dynamicLineDirection(this._startValueAngle, this._tickCount - 2, false);
                            this._inLineCalculation(this._startValueAngle, false);
                        }
                        else {
                            this._startValueAngle = this._degPoint[i];
                            this._inLineCalculation(this._startValueAngle, false);
                        }
                    }
                }
            }
            this._path = element == "path" ? this._directionLine.toString() : textPoints;
            return this._path;
        },
        _dynamicOuterLine: function (cX, cY, radius, angle, isDynamic) {
            var angleRadians = (angle) * Math.PI / 180.0;
            return {
                x: cX + (radius * Math.cos(angleRadians)),
                y: cY + (radius * (isDynamic ? (-Math.sin(angleRadians)) : (Math.sin(angleRadians))))
            }
        },
        _dynamicLineDirection: function (angle, lastValue, isDynamic) {
            var outerLine = {};
            var firstPoint = this._dynamicOuterLine(this._startXY, this._startXY, (this.model.radius), angle, isDynamic);
            var secondPoint = this._dynamicOuterLine(this._startXY, this._startXY, (this.model.radius + 10), angle, isDynamic);
            outerLine.X1 = firstPoint.x;
            outerLine.Y1 = firstPoint.y;
            outerLine.X2 = secondPoint.x;
            outerLine.Y2 = secondPoint.y;
            this._isTapSelected = true;
            this._directionLine.insert(lastValue + 2, [" M", outerLine.X1, outerLine.Y1, "L", outerLine.X2, outerLine.Y2].join(" "));
            return this._directionLine.toString();
        },
        _inLineCalculation: function (angle, isDynamic) {
            var line = {};
            var secondPoint = this._polarToCartesian(this._startXY, this._startXY, (this.model.radius - 5), angle, isDynamic);
            line.X1 = this._startXY, line.Y1 = this._startXY, line.X2 = secondPoint.x, line.Y2 = secondPoint.y;
            this._directionLine.append([" M", line.X1, line.Y1, "L", line.X2, line.Y2].join(" "));
            return this._directionLine.toString();
        },

        _dynamicLineCalculation: function (angle, isAppend, isRemove, isInsert, lastValue, insertValue, isOuter) {
            this.line = {};
            var radius = this.model.radius;
            this.line.X1 = this._startXY;
            this.line.Y1 = this._startXY;
            var cosAngle = Math.cos(angle * Math.PI / 180.0);
            var sinAngle = -(Math.sin(angle * Math.PI / 180.0));
            this.line.X2 = this._startXY + (radius - 5) * cosAngle;
            this.line.Y2 = this._startXY + (radius - 5) * sinAngle;
            if (isOuter)
                this._dynamicLineDirection(angle, lastValue, true);
            if (isAppend)
                this._directionLine.append([" M", this.line.X1, this.line.Y1, "L", this.line.X2, this.line.Y2].join(" "));
            if (isInsert)
                this._directionLine.insert(lastValue + 3, [" M", this.line.X1, this.line.Y1, "L", this.line.X2, this.line.Y2].join(" "));
            return this._directionLine.toString();
        },

        _getStringBuilder: function () {
            var data = [];
            var counter = 0;
            return {
                append: function (s) {
                    data[counter++] = s;
                    return this;
                },
                remove: function (i, j, k) {
                    counter = counter - (k || 1);
                    data.splice(i, j || 1);
                    return this;
                },
                insert: function (i, s) {
                    data.splice(i, 0, s);
                    counter++;
                    return this;
                },
                toString: function (s) { return data.join(s || ""); }
            }
        },
        _polarToCartesian: function (cX, cY, radius, angle, isDynamic) {
            var angleRadians = (angle) * Math.PI / 180.0;
            return {
                x: cX + (radius * Math.cos(angleRadians)),
                y: cY + (radius * Math.sin(isDynamic ? -angleRadians : angleRadians))
            };
        },
        //Circel direction calculation
        _circleArcDirection: function (endAngle) {
            var startAngle = this.model.startAngle, radius = (0.5 * Math.min(this._diameter - (this.model.inline ? 100 : 0), this._diameter - (this.model.inline ? 100 : 0)));
            var startPoint = this._polarToCartesian(this._startXY, this._startXY, radius, startAngle),
            endPoint = this._polarToCartesian(this._startXY, this._startXY, radius, endAngle);
            ArcSweep = endAngle - startAngle <= 180 ? "0" : "1";
            var direction = [
            "M", startPoint.x, startPoint.y,
            "A", radius, radius, 0, ArcSweep, 1, endPoint.x, endPoint.y - 1
            ].join(" ");
            return direction;
        },

        _overNeedleMoveHandler: function (e) {
            e.preventDefault();
            var baseRect = document.getElementById(this._prefix + "radial-slider-svg").getBoundingClientRect();
            var y2 = e.clientY, x2 = e.clientX, y1 = baseRect.top + (baseRect.width / 2), x1 = baseRect.left + (baseRect.width / 2);
            this._dynamicAngleCalculation(x1, x2, y1, y2);
            if ((this._angle > (360 - this.model.endAngle) && this._angle < (360 - this.model.startAngle))) {
                //Over element
                this.line = {};
                var radius = this.model.radius;
                this.line.X1 = this._startXY;
                this.line.Y1 = this._startXY;
                var cosAngle = Math.cos(this._angle * Math.PI / 180.0);
                var sinAngle = -(Math.sin(this._angle * Math.PI / 180.0));
                this.line.X2 = this._startXY + (radius - 5) * cosAngle;
                this.line.Y2 = this._startXY + (radius - 5) * sinAngle;
                if (this._overLine != undefined)
                    $(this._overLine).attr({ "d": [" M", this.line.X1, this.line.Y1, "L", this.line.X2, this.line.Y2].join(" "), "stroke-width": (this.model.strokeWidth == 1 ? this.model.strokeWidth - 0.5 : this.model.strokeWidth - 1) });
                else
                    this._overLine = this._createSVGElements("path", { "class": this._prefix + "needle-over", "d": [" M", this.line.X1, this.line.Y1, "L", this.line.X2, this.line.Y2].join(" "), "stroke-width": this.model.strokeWidth == 1 ? this.model.strokeWidth - 0.5 : this.model.strokeWidth - 1 });
                this._pathLineGroupElement.append(this._overLine);
                this._isNeedleOver = true;
                var selectValue = this._ticksCalculation();
                if (this.model.mouseover) {
                    this._trigger("mouseover", { value: selectValue, selectedValue: this.value() });
                }
            }
        },
        _needleMoveHandler: function (e) {
            var baseRect = document.getElementById(this._prefix + "radial-slider-svg").getBoundingClientRect();
            var y2 = e.clientY, x2 = e.clientX, y1 = baseRect.top + (baseRect.width / 2), x1 = baseRect.left + (baseRect.width / 2);
            this._dynamicAngleCalculation(x1, x2, y1, y2);
            if ((this._angle > (360 - this.model.endAngle) && this._angle < (360 - this.model.startAngle))) {
                $(this._overLine).remove();
                this._lineAngleCalculation();
                var index = this._isTapSelected ? this._tickCount + 1 : this._tickCount;
                this._directionLine.remove(index, 1, 1);
                this._dynamicLineCalculation(this._angle, false, true, true, this._tickCount, this._tickCount + 1, false);
                $(this._pathLineElement).attr("d", this._directionLine.toString());
                var moveEvent = this._isAppView() ? this.model.touchMove : this.model.slide;
                if (moveEvent) {
                    var selectValue = this._ticksCalculation();
                    this._trigger("slide", { value: selectValue, selectedValue: this.value() });
                }
                this._needleMove = true;
            }
        },
        _tapHandlerEvent: function (e) {
            var baseRect = document.getElementById(this._prefix + "radial-slider-svg").getBoundingClientRect();
            var y2 = e.clientY, x2 = e.clientX, y1 = baseRect.top + (baseRect.width / 2), x1 = baseRect.left + (baseRect.width / 2);
            this._dynamicAngleCalculation(x1, x2, y1, y2);
            if ((this._angle > (360 - this.model.endAngle) && this._angle < (360 - this.model.startAngle))) {
                $(this._overLine).remove();
                this._lineAngleCalculation(true);
                this._previousAngle = this._angle;
                this._pathBeforeAddlength = this._tickCount + 1;
                this._pathAfterAddLength = this._directionLine.toString().replace(/[^M]/g, "").length;
                if (this._pathBeforeAddlength < this._pathAfterAddLength) {
                    var deleteCount = this._isTapSelected ? 2 : 1;
                    this._directionLine.remove(this._tickCount, deleteCount, deleteCount);
                }
                else
                    this._directionLine.remove(this._tickCount, 1);
                this._dynamicLineCalculation(this._angle, false, true, true, this._tickCount, this._tickCount + 1, true);
                if ($(this._textGroupElement).find('[class=' + this._prefix + "dynamic-text" + ']').length > 0)
                    this._textGroupElement.find('[class=' + this._prefix + "dynamic-text" + ']').remove();
                var select = (((this._dynamicAngle - this.model.startAngle) - (360 - this.model.endAngle) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1))) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1)));
                var selectValue;
                if (this._isTicksControl()) {
                    selectValue = this._ticksCalculation();
                    this._outerTextElement = this._createSVGElements("text", { "stroke-width": 0.5, "x": this.line.X2, "y": this.line.Y2, "class": this._prefix + "dynamic-text", "textContent": selectValue, "text-anchor": this.line.textAlignment });
                    this._textGroupElement.append(this._outerTextElement);
                }
                else
                    selectValue = this.value(this.ticks()[select.toFixed()]);
                var previousValue = selectValue != undefined ? selectValue : this.value();
                if (this.model.change)
                    this._trigger("change", { value: selectValue, oldValue: this.value() });
                if (this._needleStop && this.model.stop)
                    this._trigger("stop", { value: selectValue });
                this.value(selectValue);
                $(this._pathLineElement).attr("d", this._directionLine.toString());
                this._needleStop = true;
                this._needleMove = false;
            }
        },
        _lineAngleCalculation: function (isMove) {
            var select = (((this._dynamicAngle - this.model.startAngle) - (360 - this.model.endAngle) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1))) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1)));
            var firstValue = this.ticks()[parseInt(select)];
            var secondValue = this.ticks()[parseInt(select) + 1];
            var difference = (secondValue - firstValue);
            this._degPoint.splice(this.ticks().length, this.ticks().length + 1);
            this._degPoint.push(this._degPoint[this._degPoint.length - 1] + this._point);
            var innerValue = (((this._degPoint[parseInt(select) + 1] - this._degPoint[parseInt(select)]) - (this._degPoint[parseInt(select) + 1] - (this._dynamicAngle))) / (this._point / difference));
            var startDegree = parseInt(select) != 0 ? this._degPoint[parseInt(select)] : this._degPoint[parseInt(select)];
            var range;
            if (this.model.enableRoundOff && isMove) {
                var smallValue = parseFloat(innerValue.toFixed(2));
                innerValue = parseInt(innerValue.toFixed());
                range = (difference == 0.5 && smallValue >= 0.25) ? this._point * 1 : (this._point / difference) * innerValue;
            }
            else
                range = (this._point / difference) * parseFloat(innerValue.toFixed(2));
            this._angle = (360 - this._degPoint[parseInt(select)]) - range;
        },

        _isTicksControl: function () {
            var select = (((this._dynamicAngle - this.model.startAngle) - (360 - this.model.endAngle) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1))) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1)));
            var firstValue = this.ticks()[parseInt(select)];
            var secondValue = this.ticks()[parseInt(select) + 1];
            var difference = (secondValue - firstValue);
            this._degPoint.splice(this.ticks().length, this.ticks().length + 1);
            this._degPoint.push(this._degPoint[this._degPoint.length - 1] + this._point);
            var innerValue = (((this._degPoint[parseInt(select) + 1] - this._degPoint[parseInt(select)]) - (this._degPoint[parseInt(select) + 1] - (this._dynamicAngle))) / (this._point / difference));
            var isSecSame = this.model.enableRoundOff ? parseInt(innerValue.toFixed()) == 0 || parseInt(innerValue.toFixed()) == (difference) ? false : true : $.inArray(this._angle, this._degPoint) > -1 ? false : true;
            return isSecSame;
        },

        _ticksCalculation: function () {
            var select = (((this._dynamicAngle - this.model.startAngle) - (360 - this.model.endAngle) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1))) / ((this.model.endAngle - this.model.startAngle) / (this.ticks().length - 1)));
            var firstValue = this.ticks()[parseInt(select)];
            var secondValue = this.ticks()[parseInt(select) + 1];
            var difference = (secondValue - firstValue);
            var angleCondition = this._angle;
            var angle = this._angle * (Math.PI / 180.0)
            this.line.X2 = (this._startXY) + (this.model.radius + 30) * Math.cos(angle);
            this.line.Y2 = (this._startXY) + (this.model.radius + 30) * Math.sin(-angle);
            if (angleCondition <= 270 && 90 <= angleCondition)
                this.line.textAlignment = "middle";
            else
                this.line.textAlignment = "start";
            var startDegree = parseInt(select) != 0 ? this._degPoint[parseInt(select)] : this._degPoint[parseInt(select)];
            var range = (this._dynamicAngle - startDegree) / (this._point / difference);
            var setValue;
            if (this.model.enableRoundOff) {
                var controlValue = parseFloat(range.toFixed(1).substr(1, 3));
                range = parseInt(range.toFixed());
                setValue = firstValue + range;
            }
            else {
                setValue = firstValue + parseFloat(range.toFixed(2));
                setValue = parseFloat(setValue.toFixed(2));
            }
            return setValue;
        },
        _dynamicAngleCalculation: function (x1, x2, y1, y2) {
            var theta = Math.atan2((y2 - y1), (x2 - x1));
            this._angle = (360 - ((theta * 180) / Math.PI)) % 360;
            this._dynamicAngle = (360 + ((theta * 180) / Math.PI)) % 360;
        },
        _createSVGElements: function (element, attr) {
            var svgObj = document.createElementNS(this._svgLink, element);
            $.each(attr, function (attr, value) {
                if (attr == 'xlink:href')
                    svgObj.setAttributeNS('http://www.w3.org/1999/xlink', 'xlink:href', value);
                if (attr != "textContent")
                    svgObj.setAttribute(attr, value);
                else
                    svgObj.textContent = value;
            })
            return svgObj;
        },
        show: function () {
            if (this.model.enableAnimation) {
                this.element.removeClass(this._prefix + "slider-hide").addClass(this._prefix + "slider-show");
                this._radialSVG.attr("class", "").attr("class", (this._prefix + "radialslider-svg-show"));
            }
            this.element.css("display", "block");
        },
        hide: function () {
            var proxy = this;
            if (this.model.enableAnimation) {
                this.element.removeClass(this._prefix + "slider-show").addClass(this._prefix + "slider-hide");
                this._radialSVG.attr("class", "").attr("class", this._prefix + "radialslider-svg-hide");
                setTimeout(function () {
                    proxy.element.css("display", "none");
                }, 400)
            }
            else
                proxy.element.css("display", "none");
        },
        _setModel: function (options) {
            this._refresh();
        },
        _clearElement: function () {
            this.element.removeAttr("class");
            this.element.html(this._orgEle.html());
        },
        _destroy: function () {
            this._wireEvents(true);
            this._clearElement();
        },
        _isAppView: function () {
            return (this._prefix == "e-m-" ? true : false);
        },
    });
})(jQuery, Syncfusion);;
// JavaScript source code
/**
* @fileOverview Plugin to style the Html Radial Slider elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejRadialSlider", "ej.RadialSlider", {
        _setFirst: true,
        validTags: ["div"],
        _rootCSS: "e-radialslider",
        defaults: {
            stop: null,
            slide: null,
            start: null,
            change: null,
            mouseover: null,
            create: null,
            destroy: null
        },
        _init: function () {
            this._orgEle = this.element.clone();
            this._renderEJControl();
        },
        _renderEJControl: function () {
            this._prefix = "e-";
            this._directionLine = this._getStringBuilder();
            this._initialization();
            this._renderControl();
            this._wireEvents(false);
        },
        _refresh: function () {
            this._destroy();
            this.element.addClass("e-radialslider e-js");
            this._renderEJControl();
        },
        //events part
        _createDelegates: function () {
            this._touchStartDelegate = $.proxy(this._touchStartHandler, this);
            this._needleMoveDelegate = $.proxy(this._needleMoveHandler, this);
            this._touchEndDelegate = $.proxy(this._touchEndHandler, this);
            this._overNeedleMoveDelegate = $.proxy(this._overNeedleMoveHandler, this);
            this._mouseOutDelegate = $.proxy(this._mouseOutHandler, this);
            this._enterMouseDelegates = $.proxy(this._entermouse, this);
        },
        _wireEvents: function (remove) {
            var eventType = remove ? "unbind" : "bind";
            this._createDelegates();
            ej.listenEvents([this._radialSVG, this._radialSVG, this._radialSVG, this._radialSVG], ["mouseup", "mousedown", "mouseenter", "mouseleave"], [this._touchEndDelegate, this._touchStartDelegate, this._enterMouseDelegates, this._mouseOutDelegate], false);
        },
        _entermouse: function (e) {
            ej.listenEvents([this._radialSVG], ["mousemove"], [this._overNeedleMoveDelegate], false);
        },
        _mouseOutHandler: function (e) {
            e.preventDefault();
            if (this._radialSVG.has(e.target).length == 0 && this._needleMove) {
                ej.listenEvents([this._radialSVG], ["mousemove"], [this._needleMoveDelegate], true);
                var lastAngle = this._previousAngle != undefined ? this._previousAngle : this._startValueAngle;
                this._pathAfterAddLength = this._directionLine.toString().replace(/[^M]/g, "").length;
                this._directionLine.remove(this._pathAfterAddLength - 1, 1);
                this._inLineCalculation(lastAngle, true);
                $(this._pathLineElement).attr("d", this._directionLine.toString());
                this._needleMove = false;
            }
            $(this._overLine).remove();
        },
        _touchEndHandler: function (e) {
            ej.listenEvents([this._radialSVG], ["mousemove"], [this._needleMoveDelegate], true);
            this._tapHandlerEvent(e);
        },
        _touchStartHandler: function (e) {
            this._needleStop = false;
            e.preventDefault();
            if (this.model.start)
                this._trigger("start", { value: this.value() });
            ej.listenEvents([this._radialSVG], ["mousemove"], [this._needleMoveDelegate], false);
        }

    })
    $.extend(true, ej.RadialSlider.prototype, ej.RadialSliderBase.prototype);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html div elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    ej.widget("ejTreeView", "ej.TreeView", {
        _rootCSS: "e-treeview",

        element: null,

        model: null,

        validTags: ["ul", "div"],

        _ignoreOnPersist: [
            "fields", "nodeClick", "beforeExpand", "nodeExpand", "beforeCollapse", "nodeCollapse", "nodeSelect", "nodeCheck", "nodeUncheck", "inlineEditValidation", "beforeEdit", "keyPress", "nodeDragStart", "nodeDrag", "nodeDragStop", "nodeDropped", "create", "destroy"
        ],

        _setFirst: false,

        defaults: {

            showCheckbox: false,

            enableAnimation: true,

            allowDragAndDrop: false,

            htmlAttributes: {},

            allowDropChild: true,

            allowDropSibling: true,

            allowDragAndDropAcrossControl: true,

            allowEditing: false,

            allowKeyboardNavigation: true,

            items: null,

            fields: {

                dataSource: null,

                query: null,

                tableName: null,

                child: null,

                id: "id",

                parentId: "parentId",

                text: "text",

                spriteCssClass: "spriteCssClass",

                expanded: "expanded",

                hasChild: "hasChild",

                selected: "selected",

                linkAttribute: "linkAttribute",

                imageAttribute: "imageAttribute",

                htmlAttribute: "htmlAttribute",

                imageUrl: "imageUrl",

                isChecked: "isChecked"

            },

            autoCheckParentNode: false,

            loadOnDemand: false,

            cssClass: "",

            template: null,

            enableRTL: false,

            expandOn: "dblclick",

            enablePersistence: false,

            enabled: true,

            expandedNodes: [],

            checkedNodes: [],

            selectedNode: -1,

            width: null,

            height: null,

            autoCheck: true,

            enableMultipleExpand: true,

            nodeClick: null,

            beforeExpand: null,

            nodeExpand: null,

            beforeCollapse: null,

            nodeCollapse: null,

            beforeSelect: null,

            nodeSelect: null,

            nodeCheck: null,

            nodeUncheck: null,

            inlineEditValidation: null,

            beforeEdit: null,

            nodeEdit: null,

            keyPress: null,

            nodeDragStart: null,

            nodeDrag: null,

            nodeDragStop: null,

            nodeDropped: null,

            beforeAdd: null,

            nodeAdd: null,

            beforeDelete: null,

            nodeDelete: null,

            beforeCut: null,

            nodeCut: null,

            beforePaste: null,

            nodePaste: null,

            beforeLoad: null,

            loadSuccess: null,

            loadError: null,

            create: null,

            destroy: null

        },

        dataTypes: {

            cssClass: "string",

            showCheckbox: "boolean",

            enableAnimation: "boolean",

            allowDragAndDrop: "boolean",

            allowDropChild: "boolean",

            allowDragAndDropAcrossControl: "boolean",

            allowEditing: "boolean",

            allowKeyboardNavigation: "boolean",

            autoCheckParentNode: "boolean",

            loadOnDemand: "boolean",

            enableRTL: "boolean",

            expandOn: "string",

            enablePersistence: "boolean",

            enableMultipleExpand: "boolean",

            items: "data",

            fields: {

                dataSource: "data",

                query: "data",

                child: "data"

            },

            expandedNodes: "array",

            checkedNodes: "array",

            selectedNode: "number",

            htmlAttributes: "data"
        },

        observables: ["fields.dataSource"],

        dataSource: ej.util.valueFunction("fields.dataSource"),

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "cssClass": this._changeSkin(options[key]); break;
                    case "fields":
                        this._unWireEvents();
                        var tempUl, f = this.element.hasClass("e-js") ? false : true, element = f ? this.element.children("ul") : this.element;
                        element.empty();
                        $.extend(this.model.fields, options[key]);
                        this.model.expandedNodes = [];
                        this.model.checkedNodes = [];
                        this.model.selectedNode = -1;
                        this._checkDataBinding();
                        if (f) {
                            tempUl = this.element.children(".e-treeview-ul");
                            element.append(tempUl.children());
                            tempUl.remove();
                        }
                        break;
                    case "allowDragAndDropAcrossControl":
                        this.model.allowDragAndDropAcrossControl = options[key];
                        this._enableDragDrop();
                        break;
                    case "enabled": this._enabledAction(options[key]); break;
                    case "showCheckbox":
                        if (options[key]) {
                            this.model.showCheckbox = options[key];
                            this._showCheckBox();
                            (!this.model.enabled) && this.element.find('.nodecheckbox').ejCheckBox("disable");
                        } else
                            this.element.find('.e-item > div > .e-chkbox-wrap').remove();
                        break;
                    case "autoCheck": this.model.autoCheck = options[key]; break;
                    case "autoCheckParentNode": this.model.autoCheckParentNode = options[key]; break;
                    case "expandedNodes":
                        var len = options[key].length, temp = JSON.parse(JSON.stringify(this.model.expandedNodes));
                        if (len > 0) {
                            this._expandNodes(options[key]);
                            for (var i = 0, len = temp.length; i < len; i++) {
                                if (options[key].indexOf(temp[i]) == -1) {
                                    this._animating = false;
                                    this._collpaseNode($(this._liList[temp[i]]));
                                }
                            }
                        } else
                             (len == 0) && this._collapseAll();
                        break;
                    case "checkedNodes":
                        if (this.model.showCheckbox) {
                            var len = options[key].length, temp = JSON.parse(JSON.stringify(this.model.checkedNodes));
                            if (len > 0) {
                                this._checkedNodes(options[key]);
                                for (var i = 0, len = temp.length; i < len; i++)
                                    (options[key].indexOf(temp[i]) == -1) && this._nodeUncheck($(this._liList[temp[i]]).find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0]);
                            } else
                                 (len == 0) && this._uncheckAll();
                        }
                        break;
                    case "expandOn": this._on(this.element, options[key], this._expandEventHandler); break;
                    case "allowEditing":
                        this._preventEditable();
                        (options[key]) && this._allowEditable();
                        break;
                    case "allowKeyboardNavigation":
                        var action = (options[key]) ? "_on" : "_off";
                        this[action](this.element, 'keydown', this._KeyPress);
                        break;
                    case "allowDragAndDrop":
                        this.model.allowDragAndDrop = options[key];
                        (options[key]) ? this._addDragableClass() : this._preventDraggable();
                        break;
                    case "allowDropChild":
                        this.model.allowDropChild = options[key];
                        (options[key]) ? this._addDragableClass() : this._preventDropChild();
                        break;
                    case "allowDropSibling":
                        this.model.allowDropSibling = options[key];
                        (options[key]) ? this._addDragableClass() : this._preventDropSibling();
                        break;
                    case "enableRTL":
                        this.model.enableRTL = options[key];
                        var action, ele = (this.element.is('UL')) ? this.element.parent('.e-treeview-wrap') : this.element;
                        action = (this.model.enableRTL) ? "addClass" : "removeClass";
                        ele[action]("e-rtl");
                        break;
                    case "height": this.element.height(options[key]); break;
                    case "width": this.element.width(options[key]); break;
                    case "selectedNode":
                        this.model.selectedNode = options[key];
                        (this.model.selectedNode >= 0) ? this._nodeSelectionAction($(this._liList[this.model.selectedNode])) : this.unselectNode(this.element.find('.e-item > div > .e-active').closest('.e-item'));
                        break;
                    case "htmlAttributes": this._addAttr(options[key]); break;
                    case "enableMultipleExpand": (!options[key]) && this.collapseAll(); break;
                }
            }
        },

        _destroy: function () {
            this.element.html("");
            this._cloneElement.removeClass('e-treeview e-js e-treeview-wrap');
            var ele = (this.element.is('UL')) ? this.element.parent('.e-treeview-wrap') : this.element;
            ele.replaceWith(this._cloneElement);
            $("#" + this._id + "_WaitingPopup").remove();
            if (window.localStorage) {
                window.localStorage.removeItem(this._id + "_childNodes");
                window.localStorage.removeItem(this._id + "_persistedValues");
            }
            else if (window.cookie) {
                document.cookie = this._id + "_childNodes=; expires=Thu, 01 Jan 1970 00:00:00 UTC";
                document.cookie = this._id + "_persistedValues=; expires=Thu, 01 Jan 1970 00:00:00 UTC";
            }
        },

        _init: function () {
            this._cloneElement = this.element.clone(), this._dataSource = [], this._fragment = [], this._templateType = "", this._indexID = 0, this._animating = false, this._newDataSource = this.dataSource(), this._id = this.element.prop("id");
            if (this.model.enablePersistence) {
                var cookieData = this._getCookies("_persistedValues");
                if (!cookieData) {
                    var obj = { selectedNode: "", expandedNodes: [], checkedNodes: [] };
                    this._setCookies("_persistedValues", JSON.stringify(obj));
                }
            }
            (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null) ? this._checkDataBinding() : this._initialize();
        },

        _initialize: function () {
            this._cutNode = null, this._beforeEditText = null, this._CurrenctSelectedNodes = [];
            if (this.element.is("ul")) {
                this._createWrapper();
            }
            else {
                this.element.addClass("e-treeview-wrap e-widget").attr("tabindex", 0)
                            .children('ul:first').addClass("e-ul");
                (this.model.width != null) && this.element.width(this.model.width);
                (this.model.height != null) && this.element.height(this.model.height);
                (this.model.enableRTL) && this.element.addClass("e-rtl");
                (this.model.cssClass != null) && this.element.addClass(this.model.cssClass);
                this._addAttr(this.model.htmlAttributes);
            }
            this._elementSettings();
            this._beforeBaseClass();
            if (this.dataSource() == null) {
                this._addBaseClass();
                this._controlRender();
            }
            this._addDragableClass();
            this._finalize();
            this._enabledAction(this.model.enabled);
        },

        _elementSettings: function () {
            var ele = (this.element.is('UL')) ? this.element.parent('.e-treeview-wrap') : this.element;
            (this.element.is("ul")) ? ele.attr("role", "tree") : ele.attr("aria-activedescendant", this._id + "_active").children('ul:first').attr("role", "tree");
        },

        _beforeBaseClass: function () {
            var ele = (this.element.is('UL')) ? this.element.parent('.e-treeview-wrap')[0] : this.element[0];
            this._ulList = $(ele.querySelectorAll('ul'));
            this._liList = $(ele.querySelectorAll('li'));
        },

        _createWrapper: function () {
            var mainWidget = ej.buildTag("div.e-treeview-wrap e-widget " + this.model.cssClass, "", "", { tabindex: 0, "aria-activedescendant": this._id + "_active" });
            (this.model.width != null) && mainWidget.width(this.model.width);
            (this.model.height != null) && mainWidget.height(this.model.height);
            (this.model.enableRTL) && mainWidget.addClass("e-rtl");
            mainWidget.insertAfter(this.element);
            mainWidget.append(this.element.addClass("e-ul").attr("tabindex", 0));
            this._addAttr(this.model.htmlAttributes);
        },

        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy._enabledAction(false);
                else proxy.element.attr(key, value)
            });
        },

        _changeSkin: function (skin) {
            if (this.model.cssClass != skin) {
                var ele = (this.element.is('UL')) ? this.element.parent('.e-treeview-wrap') : this.element;
                ele.removeClass(this.model.cssClass).addClass(skin);
                ele.find('.e-item > div > .e-chkbox-wrap').removeClass(this.model.cssClass).addClass(skin);
            }
        },

        _enabledAction: function (flag) {
            this.model.enabled = flag;
            if (flag) {
                this.element.removeClass("e-disable");
                this._wireEvents();
            }
            else {
                this.element.addClass("e-disable");
                this._unWireEvents();
            }
        },

        _checkDataBinding: function () {
            if (this.dataSource() instanceof ej.DataManager) {
                this._initDataSource();
            } else {
                if (ej.isNullOrUndefined(this.dataSource().length) || this.dataSource() == "") this.model.fields.dataSource = [];
                this._ensureDataSource(this.dataSource());
                this._initialize();
            }
        },

        _initDataSource: function () {
            this.element.ejWaitingPopup();
            var waitingPopup = this.element.ejWaitingPopup("instance"), query, proxy = this, queryPromise;
            this.element.ejWaitingPopup("refresh");
            this.element.ejWaitingPopup("show");
            query = this._columnToSelect(this.model.fields);
            queryPromise = this.dataSource().executeQuery(query);
            queryPromise.done(function (e) {
                proxy.element.ejWaitingPopup("hide");
                proxy.retriveData = e.result;
                proxy._ensureDataSource(e.result);
                proxy._initialize();
            });
        },

        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query) && !ej.isNullOrUndefined(mapper.tableName)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "child" && col !== "dataSource" && mapper[col])
                        column.push(mapper[col]);
                }
                (column.length > 0) && queryManager.select(column);
                if (!this.dataSource().dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else queryManager = (mapper.query) ? mapper.query : queryManager;
            return queryManager;
        },

        _ensureDataSource: function (dataSource) {
            this.currentSelectedData = dataSource;
            this._renderTemplate(dataSource);
            (this.element.is("ul")) ? this.element.html(this._fragment.firstChild.children) : this.element.html(this._fragment);
        },

        _getTemplateType: function (ds, mapper) {
            for (var i = 0, len = ds.length; i < len; i++) {
                if (ds[i].hasOwnProperty(mapper.parentId) || ds[i].hasOwnProperty(mapper.hasChild)) return 1;
                if (ds[i].hasOwnProperty('child')) return 2;
            }
            return 1;
        },

        _groupingObjects: function (array, f) {
            var childs = {}, keys = [];
            for (var i = 0, len = array.length; i < len; i++) {
                var child = JSON.stringify(f(array[i]));
                childs[child] = childs[child] || [];
                childs[child].push(array[i]);
            }
            for (var i in childs)
                keys.push(childs[i]);
            return keys;
        },

        _renderTemplate: function (item) {
            this._onlineData = false, this._loadOnDemandNodes = false;
            var proxy = this;
            this._templateType = this._getTemplateType(item, this.model.fields);
            this._dataSource = (this._templateType == 1) ? this._groupingObjects(item, function (item) { return [!ej.isNullOrUndefined(item) && [item[proxy.model.fields.parentId]].toString()]; }) : item;
            this._templateNodeCreation(item, this.model.fields);
        },

        _templateNodeCreation: function (data, mapper) {
            this._fragment = document.createDocumentFragment();
            var ulTag = document.createElement('ul'), fragment;
            this._fragment.appendChild(ulTag);
            fragment = this._fragment.firstChild;
            if (this._onlineData || this._loadOnDemandNodes) {
                fragment.className = "e-treeview-ul";
                fragment.setAttribute('role', "group");
                $(fragment).attr('style', 'display:none');
            }
            if (this.dataSource() != null) {
                for (var i = 0, objlen = data.length; i < objlen; i++) {
                    if (data[i]) {
                        if (ej.isNullOrUndefined(data[i][this.model.fields.parentId]) || data[i][this.model.fields.parentId] == 0 || this._loadOnDemandNodes)
                            fragment.appendChild(this._genTemplate(data[i], mapper));
                    }
                }
                $(fragment).find('.e-item:first-child:not(:last-child)').addClass('first');
                $(fragment).find('.e-item:last-child').addClass('last');
            }
        },

        _onlineDataSource: function (childItems, id, mapper) {
            this._loadOnDemandNodes = true;
            this._templateNodeCreation(childItems, mapper);
            var element = this.element.find('.e-item#' + id);
            if (element[0] != null) {
                element.append(this._fragment);
                element.children().find('> div:first').addClass("e-icon e-plus");
                this._finalizeLoadOnDemand(element);
            }
            this._onlineData = false;
        },

        _genTemplate: function (item, mapper) {
            var liFrag = document.createDocumentFragment(), liEle, liTag, spanTag, imgTag, aTag, childItems;
            liTag = document.createElement('li');
            liFrag.appendChild(liTag);
            liEle = liFrag.firstChild;
            liEle.id = (item[mapper.id]) ? item[mapper.id] : (!this.model.template && !this.model.showCheckbox) ? item[mapper.text] : "";
            liEle.setAttribute('role', "treeitem");
            (item[mapper.htmlAttribute]) && this._setAttributes(item[mapper.htmlAttribute], liEle);
            liEle.className += ' e-item';
            if (this.model.template || item[mapper.text]) {
                aTag = document.createElement('a');
                if (item[mapper.imageUrl]) {
                    imgTag = document.createElement('img');
                    imgTag.className = "e-align";
                    imgTag.src = item[mapper.imageUrl];
                    (item[mapper.imageAttribute]) && this._setAttributes(item[mapper.imageAttribute], imgTag);
                    aTag.appendChild(imgTag);
                }
                else if (item[mapper.spriteCssClass]) {
                    spanTag = document.createElement('span');
                    spanTag.className = item[mapper.spriteCssClass];
                    aTag.appendChild(spanTag);
                }
                if (this.model.template) {
                    if (typeof $.fn.render != "function") throw "Error : JsRender dependecy script missing";
                    aTag.innerHTML += this._renderEjTemplate(this.model.template, item);
                }
                else aTag.innerHTML += item[mapper.text];
                (item[mapper.linkAttribute]) && (typeof item[mapper.linkAttribute] == "object" ? this._setAttributes(item[mapper.linkAttribute], aTag) : aTag.href = item[mapper.linkAttribute]);
                liEle.appendChild(aTag);
            } else return;
            (item[mapper.expanded]) && (liEle.className += ' expanded');
            (item[mapper.selected]) && (liEle.className += ' selected');
            if (!ej.isNullOrUndefined(item[mapper.isChecked]))
                liEle.className += (item[mapper.isChecked]) ? ' checked' : ' unchecked';
            liEle.setAttribute('aria-selected', false);
            liEle.setAttribute('aria-expanded', false);
            if (this.dataSource() instanceof ej.DataManager) {
                this._updateElement(liEle, true);
                this.model.showCheckbox && this._checkboxOnTemplate(liEle.children[0]);
                if (!this.model.loadOnDemand) {
                    if (!ej.isNullOrUndefined(mapper["child"]) && mapper["child"]["dataSource"] instanceof ej.DataManager) {
                        var proxy = this, queryManager = ej.Query(), pid, id;
                        queryManager = proxy._columnToSelect(mapper["child"]);
                        pid = (mapper["child"]["parentId"]) ? mapper["child"]["parentId"] : proxy.model.fields.parentId, id;
                        id = (mapper.id) ? mapper.id : this.model.fields.id;
                        if (!ej.isNullOrUndefined(pid))
                            queryManager.where(pid, ej.FilterOperators.equal, item[id]);
                        queryPromise = mapper["child"]["dataSource"].executeQuery(queryManager);
                        queryPromise.done(function (e) {
                            childItems = e.result;
                            if (childItems && childItems.length > 0) {
                                proxy._onlineData = true;
                                proxy._checkMapperFields(mapper["child"]);
                                proxy._onlineDataSource(childItems, childItems[0][mapper["child"]["parentId"]], mapper.child);
                            }
                        });
                    } else if (ej.isNullOrUndefined(this.model.fields["child"])) {
                        this._childNodeCreation(item, liEle, mapper);
                    }
                }
                else {
                    if (item[mapper.hasChild] || item.hasOwnProperty('child'))
                        liEle.children[0].firstChild.className = "e-icon e-plus";
                }
            }
            else if (this.model.loadOnDemand) {
                this._updateElement(liEle, true);
                if (item[mapper.hasChild] || item.hasOwnProperty('child'))
                    liEle.children[0].firstChild.className = "e-icon e-plus";
                this.model.showCheckbox && this._checkboxOnTemplate(liEle.children[0]);
                (item[mapper.expanded]) && this._childNodeCreation(item, liEle, mapper);
                if (this.model.enablePersistence) {
                    var value, childObj;
                    value = this._getCookies("_childNodes");
                    if (value) {
                        childObj = JSON.parse(value);
                        for (var i = 0, objlen = childObj.length; i < objlen; i++) {
                            if (childObj[i].toString() == item[mapper.id].toString()) {
                                this._childNodeCreation(item, liEle, mapper);
                                break;
                            }
                        }
                    }
                }
            }
            else if (!this._onlineData) {
                this._updateElement(liEle, true);
                this.model.showCheckbox && this._checkboxOnTemplate(liEle.children[0]);
                this._childNodeCreation(item, liEle, mapper);
            }
            return liEle;
        },

        _childNodeCreation: function (item, liEle, mapper) {
            var childItems, tempFrag, ul, tFrag;
            childItems = (this._templateType == 2) ? (!ej.isNullOrUndefined(item.child) && item.child.length > 0) && item.child : this._getChildNodes(this._dataSource, { id: item[mapper.id] });
            if (!ej.isNullOrUndefined(childItems) && childItems.length > 0) {
                liEle.children[0].firstChild.className = "e-icon e-plus";
                tempFrag = document.createDocumentFragment();
                ul = document.createElement('ul');
                tempFrag.appendChild(ul);
                tFrag = tempFrag.firstChild;
                tFrag.className = "e-treeview-ul";
                tFrag.setAttribute('role', "group");
                $(tFrag).attr('style', 'display:none');
                for (var i = 0, objlen = childItems.length; i < objlen; i++)
                    (childItems[i]) && tFrag.appendChild(this._genTemplate(childItems[i], mapper));
                liEle.appendChild(ul);
            }
        },

        _checkboxOnTemplate: function (element) {
            if (element.parentElement.id == "") {
                element.parentElement.id = this._id + "_" + this._indexID;
                this._indexID++;
            }
            var checkbox = document.createElement('input');
            checkbox.setAttribute('type', 'checkbox');
            checkbox.setAttribute('class', 'nodecheckbox');
            checkbox.setAttribute('name', this._id + "_Checkbox_" + element.parentElement.id);
            checkbox.setAttribute('value', element.parentElement.id);
            element.insertBefore(checkbox, element.children[1]);   // Checkbox to be inserted after the expand/collapse div            
        },

        _updateElement: function (liElement, subItem) {
            var outerdiv, exCollpasediv, linkElement, spanTag, node = liElement.firstChild, firstchild, nodeText, nText;
            if (node) {
                firstchild = node.nodeName;
                if (firstchild == 'SPAN' || firstchild == 'IMG') {
                    spanTag = node;
                    linkElement = liElement.lastChild;
                }
                else
                    linkElement = liElement.lastChild;
                if (linkElement) {
                    nText = (linkElement.lastChild != null) ? linkElement.lastChild.nodeValue : "";
                    nodeText = linkElement.innerHTML;
                    textElement = $(linkElement).clone()[0];
                    $(linkElement).remove();
                    textElement.className += " e-text CanSelect";
                    textElement.innerHTML = nodeText;
                    liElement.innerHTML = '';
                    this._on($(textElement), "mouseenter", this._mouseEnterEvent)
                        ._on($(textElement), "mouseleave", this._mouseLeaveEvent);
                }

                outerdiv = document.createElement('div');
                outerdiv.setAttribute('role', "presentation");
                if (subItem) {
                    exCollpasediv = document.createElement('div');
                    exCollpasediv.setAttribute('role', "presentation");
                    outerdiv.appendChild(exCollpasediv);
                }
                (spanTag) && outerdiv.appendChild(spanTag);
                outerdiv.appendChild(textElement);
                ($(liElement).prop('name') == undefined) && $(liElement).prop('name', nText);
                liElement.appendChild(outerdiv);
            }
        },

        _setAttributes: function (data, element) {
            for (var key in data) {
                $(element).attr(key, data[key]);
            }
        },

        _addDragableClass: function () {
            if (this.model.allowDragAndDrop) {
                this._anchors = this._liList.map(function () {
                    return $("a", this)[0];
                });
                this._anchors.addClass("e-draggable e-droppable");
                this._enableDragDrop();
                this._on(this._anchors, "mouseup", this._focusElement);
            }
        },

        _addBaseClass: function () {
            this._ulList.addClass("e-treeview-ul").attr("role", "group");
            this._liList.addClass("e-item").attr("role", "treeitem");
            if (!this.element.is("ul"))
                this.element.find("ul:first").removeClass("e-treeview-ul").addClass('e-ul');
        },

        _controlRender: function () {
            var element = this.element, licoll;
            if (element != null) {
                licoll = element.find('.e-item');
                for (var i = 0; i < licoll.length; i++) {
                    var listEl = $(licoll[i]), textElement, customElement, nodeText, subItems, linkElement, span, exCollpasediv, outerdiv;
                    subItems = listEl.children('ul')[0];
                    if (subItems)
                        $(listEl.children('ul')[0]).remove();
                    linkElement = listEl.children('a')[0];
                    if (linkElement) {
                        nodeText = $(linkElement).text();
                        textElement = $(linkElement).clone();
                        $(linkElement).remove();
                        $(textElement).addClass('e-text CanSelect');
                        listEl.html('');
                    }
                    else {
                        nodeText = this._getText(listEl);
                        customElement = listEl.clone();
                        listEl.html('');
                        textElement = ej.buildTag("a.e-text CanSelect", "", "", { alt: "" });
                        textElement.append(customElement.children());
                        textElement[0].innerHTML += nodeText;
                    }
                    exCollpasediv = ej.buildTag('div', "", {}, { role: "presentation" });
                    outerdiv = ej.buildTag('div', "", {}, { role: "presentation" });
                    $(outerdiv).append(exCollpasediv).append(textElement);
                    listEl.prepend(outerdiv);
                    if (subItems)
                        listEl.append(subItems);
                    (listEl.prop('name') == undefined) && listEl.prop('name', nodeText);
                    (listEl.attr('id') == undefined) && listEl.attr('id', nodeText);
                    (this.model.showCheckbox) && this._checkboxOnTemplate(licoll[i].children[0]);
                }
                if (this.model.showCheckbox)
                    element.find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
            }
            var liCollection = element.find('.e-item'), acollection;
            liCollection.find('>ul').hide();
            acollection = liCollection.find('.e-text');
            acollection.focus(function () {
                $(this).blur();
            });
            liCollection.filter('.e-item:last-child').addClass('last');
            $(liCollection[0]).addClass('first');
            if ($(liCollection[0]).hasClass('first') && $(liCollection[0]).hasClass('last'))
                $(liCollection[0]).removeClass('first');
            $(liCollection.filter(':has(ul)')).each(function () {
                $(this).attr("aria-expanded", false).attr("aria-selected", false)
                var liHasul = $(this).children('ul:first');
                if ($(liHasul).is(':hidden')) {
                    $(this).find('> div > div:first').removeClass('e-icon e-minus').addClass('e-icon e-plus');
                }
                else {
                    $(this.childNodes[1]).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                }
            });
        },

        _getText: function (element) {
            return $(element)
                      .clone()
                      .children()
                      .remove()
                      .end()
                      .text();
        },

        _expandNodes: function (indexColl) {
            var len = indexColl.length, element = [], temp;
            for (var i = 0; i < len; i++) {
                element = (!this.model.enableMultipleExpand) ? $(this._liList[indexColl[len - 1]]) : $(this._liList[indexColl[i]]);
                if (this.isExpanded(element)) continue;
                if (this.model.loadOnDemand && !(this.dataSource() instanceof ej.DataManager))
                    this._createChildNodesWhenExpand(element);
                else
                    this._expandNode(element);
                if (!this.model.enableMultipleExpand) break;
            }
        },

        _checkedNodes: function (indexColl) {
            var node;
            for (var j = 0, len = indexColl.length; j < len; j++) {
                node = $(this._liList[indexColl[j]]);
                (node[0] != null) && this._nodeCheck(node.find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0]);
            }
        },

        _finalize: function () {
            this._isRender = false;
            var thisElement = this.element, cookieData = this._getCookies("_persistedValues"), idColl = [], id = 0, element = [], expandList, parsedData, temp;
            if (!(this.model.expandedNodes instanceof Array && this.model.expandedNodes.length > 0)) {
                expandList = thisElement.find("li.expanded"), len = expandList.length;
                if (!this.model.enableMultipleExpand && len > 0) {
                    this._expandNode($(expandList[len - 1]));
                } else {
                    for (var i = 0; i < len; i++)
                        this._expandNode($(expandList[i]));
                }
            }
            if (cookieData)
                parsedData = JSON.parse(cookieData), idColl = parsedData.expandedNodes, element = [];
            if (idColl.length > 0 && !ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null) {
                for (var i = 0, len = idColl.length; i < len; i++) {
                    if (!this.model.enableMultipleExpand && len > 0) {
                        element = this.element.find('.e-item#' + idColl[len - 1]);
                        (element[0] != null) && this._expandNode(element);
                        break;
                    } else {
                        element = this.element.find('.e-item#' + idColl[i]);
                        if (element[0] != null) {
                            if (this.model.loadOnDemand && !(this.dataSource() instanceof ej.DataManager))
                                this._createChildNodesWhenExpand(element);
                            else
                                this._expandNode(element);
                        }
                    }
                }
            } else {
                temp = JSON.parse(JSON.stringify(this.model.expandedNodes));
                this._expandNodes(temp);
            }
            if (this.model.showCheckbox) {
                thisElement.find('.e-item > div .nodecheckbox').ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                if (cookieData)
                    parsedData = JSON.parse(cookieData), idColl = parsedData.checkedNodes, element = [];
                if (idColl.length > 0 && !ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null) {
                    for (var i = 0, len = idColl.length; i < len; i++) {
                        element = this.element.find('.e-item#' + idColl[i]);
                        if (element[0] != null && !(this.dataSource() instanceof ej.DataManager))
                            this._nodeCheck(element.find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0]);
                    }
                } else {
                    temp = JSON.parse(JSON.stringify(this.model.checkedNodes));
                    this._checkedNodes(temp);
                }
            }
            if (!(this.model.checkedNodes instanceof Array && this.model.checkedNodes.length > 0))
                this.model.showCheckbox && this._isCheckedAction();
            element = [];
            (cookieData) && (id = parsedData.selectedNode);
            if (id && !ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null)
                element = this.element.find('.e-item#' + id);
            (element[0] != null) ? this._nodeSelectionAction(element) : (this.model.selectedNode >= 0) ? this._nodeSelectionAction($(this._liList[this.model.selectedNode])) : (this.model.selectedNode != null) && this._isSelectedAction();
            this._isRender = true;
            this.model.allowEditing && this._allowEditable();
            thisElement.find('.e-item.checked, .e-item.expanded, .e-item.selected').removeClass("checked expanded selected");
        },

        _updateSelectedNode: function () {
            var node = $(this._liList).find('.e-text.e-active').closest('.e-item'), element = [];
            this.model.selectedNode = (node[0] != null && node.length > 0) ? $(this._liList).index(node) : null;
            if (this.model.selectedNode >= 0) {
                element = $(this._liList[this.model.selectedNode]);
                if (element[0] != null) {
                    this._isRender = false;
                    this.selectNode(element);
                    this._isRender = true;
                }
            }
            this._persistValues([this.model.selectedNode], "selectedNode");
        },

        _setCookies: function (name, value) {
            if (window.localStorage)
                window.localStorage.setItem(this._id + name, value);
            else if (document.cookie)
                ej.cookie.set(this._id + name, value);
        },

        _getCookies: function (name) {
            if (window.localStorage)
                return window.localStorage.getItem(this._id + name);
            else if (window.cookie)
                return ej.cookie.get(this._id + name);
        },

        _updateCheckedNodes: function () {
            var checkedList, chkdNodes = [];
            checkedList = this.element.find('.e-item > div > .e-chkbox-wrap > .nodecheckbox.checked').closest('.e-item');
            for (var chk = 0; chk < checkedList.length; chk++)
                chkdNodes.push($(this._liList).index(checkedList[chk]));
            (chkdNodes.length > 0 || checkedList.length == 0) && (this.model.checkedNodes = chkdNodes);
            (checkedList.length == 0) && this.model.checkedNodes.push(-1);
            this._persistValues(this.model.checkedNodes, "checkedNodes");
        },

        _updateExpandedNodes: function () {
            var expandedEle, expndedNodes = [];
            expandedEle = this.element.find('.e-item > div > .e-minus').closest('.e-item').addClass('e-collapse');
            for (var i = 0, objlen = expandedEle.length; i < objlen; i++)
                expndedNodes.push($(this._liList).index(expandedEle[i]));
            (expndedNodes.length > 0 || expandedEle.length == 0) && (this.model.expandedNodes = expndedNodes);
            (expandedEle.length == 0) && this.model.expandedNodes.push(-1);
            this._persistValues(this.model.expandedNodes, "expandedNodes");
        },

        _isCheckedAction: function () {
            var chk, checkedList = this.element.find('.e-item.checked');
            for (var chk = 0; chk < checkedList.length; chk++) {
                if ($(checkedList[chk]).children('ul').find('.e-item').length == 0 || $(checkedList[chk]).find('.e-item.unchecked').length == 0)
                    this._nodeCheck($(checkedList[chk]).find("> div > .e-chkbox-wrap > input.nodecheckbox:first")[0]);
            }
            this.element.find('.e-item.checked, .e-item.unchecked').removeClass("checked unchecked");
        },

        _isSelectedAction: function () {
            var snode = this.element.find('.e-item.selected:last');
            if (snode[0] != null)
                this._nodeSelectionAction(snode);
            this.element.find('.e-item.selected').removeClass("selected");
        },

        _checkedChange: function (args) {
            var treeview = this.element.closest(".e-treeview.e-js").data("ejTreeView");
            action = (args.isChecked) ? "_nodeCheck" : "_nodeUncheck";
            treeview[action]($(this.element)[0], args);
        },

        _ClickEventHandler: function (event) {
            var proxy = this, parentLi, element = $(event.target);
            if (!this._isclicked) {
                parentLi = element.closest('.e-item');
                if (!parentLi.hasClass('e-node-disable')) {
                    this._isclicked = true;
                    this._triggerEvent('nodeClick', { event: event, currentElement: element[0] });
                    if (!ej.isNullOrUndefined(element) && element.is('DIV')) {
                        if (element.hasClass('e-plus') || element.hasClass('e-minus')) {
                            var expandUl = null, args, nodeDetails;
                            if (element.is('SPAN'))
                                element = element.parent('div').find('div:first');
                            expandUl = parentLi.children('ul:first');
                            this._isExpanded = expandUl.find('li').length > 0 ? true : false;
                            if (!this._isExpanded && this.model.loadOnDemand) {
                                element.removeClass('e-plus').addClass("e-load");
                                this._checkboxChecked = element.parent().find('.nodecheckbox').hasClass('checked');
                                nodeDetails = this._getNodeDetails(parentLi);
                                args = { currentElement: parentLi, targetElement: element[0], id: nodeDetails.id, value: nodeDetails.text, parentId: nodeDetails.parentId, isChildLoaded: this._isExpanded, hasParent: true, async: true };
                                this._triggerEvent('beforeExpand', args);
                                this._loadOnDemand(args, element[0]);
                            }
                            else
                                (element.hasClass('e-plus')) ? this._expandNode(parentLi) : this._collpaseNode(parentLi);
                        }
                    }
                    (!element.is('A') && !element.hasClass('input-text')) && (element = element.parent());
                    if (!ej.isNullOrUndefined(element) && element.is('A'))
                        element.hasClass('CanSelect') && !element.hasClass('e-active') && this._nodeSelectionAction(element.closest('.e-item'));
                }
            }
            setTimeout(function () { proxy._isclicked = false; }, 300);
        },

        _getChildTables: function (mapper, parentLen, currentLen) {
            if (!ej.isNullOrUndefined(mapper.child))
                return (parentLen == currentLen) ? mapper.child : this._getChildTables(mapper.child, parentLen, currentLen + 1);
        },

        _checkMapperFields: function (mapper) {
            for (var i in this.model.fields)
                if (!mapper[i] && i != 'dataSource' && i != 'child' && i != 'tableName' && i != 'query') mapper[i] = this.model.fields[i];
        },

        _loadOnDemand: function (args) {
            var childItems, parent, mapper, proxy = this, queryManager;
            if (this.dataSource() instanceof ej.DataManager) {
                parent = args.currentElement.parents('ul').length;
                mapper = this._getChildTables(proxy.model.fields, parent, 1);
                if (ej.isNullOrUndefined(mapper) && ej.isNullOrUndefined(this.model.fields["child"]))
                    mapper = this.model.fields;
                if (!mapper) return;
                this._checkMapperFields(mapper);
                queryManager = ej.Query();
                queryManager = proxy._columnToSelect(mapper);
                queryManager.where(mapper["parentId"], ej.FilterOperators.equal, args.currentElement[0].id);
                queryPromise = mapper["dataSource"].executeQuery(queryManager);
                queryPromise.done(function (e) {
                    childItems = e.result;
                    if (childItems.length > 0) {
                        proxy._onlineData = true;
                        proxy._loadChildNodeWhenOnDemand(childItems, args, mapper);
                    }
                });
            }
            else {
                childItems = this._getChildNodes(this._dataSource, args);
                !ej.isNullOrUndefined(childItems) && childItems.length > 0 && setTimeout(function () {
                    proxy._createSubNodesWhenLoadOnDemand(childItems, args.targetElement, proxy.model.fields);
                }, 400);
            }
        },

        _loadChildNodeWhenOnDemand: function (childItems, args, mapper) {
            var proxy = this;
            !ej.isNullOrUndefined(childItems) && childItems.length > 0 && setTimeout(function () {
                proxy._createSubNodesWhenLoadOnDemand(childItems, args.targetElement, mapper);
            }, 100);
        },

        _createSubNodesWhenLoadOnDemand: function (childItems, element, mapper) {
            this._loadOnDemandNodes = true;
            this._templateNodeCreation(childItems, mapper);
            $(this._fragment.firstChild).attr('style', 'display:none');
            this._fragment.firstChild.className = "e-treeview-ul";
            this._fragment.firstChild.setAttribute('role', "group");
            element.parentNode.parentNode.appendChild(this._fragment);
            var liElement = $(element).closest('.e-item');
            liElement.attr({ 'aria-expanded': false, 'aria-expanded': true });
            $(element).removeClass("e-load").addClass('e-plus');
            this._expandNode(liElement);
            this._finalizeLoadOnDemand(liElement);
        },

        _finalizeLoadOnDemand: function (element) {
            this._beforeBaseClass();
            this._isRender = false;
            if (this.model.showCheckbox) {
                element.children('ul').find('li > div > input.nodecheckbox').ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                if (element.children('ul').find('.e-item.unchecked').length == 0 && this._checkboxChecked)
                    element.children('ul').find('.e-item').addClass('checked');
                else if (element.children('ul').find('.e-item.unchecked').length == element.children('ul').find('.e-item').length && this.isNodeChecked(element))
                    this._nodeUncheck(element.find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0]);
                this._isCheckedAction();
                this._updateCheckedNodes();
            }
            this._isRender = true;
            this._addDragableClass();
            this._preventEditable();
            this.model.allowEditing && this._allowEditable();
            this._isSelectedAction();
            this._updateSelectedNode();
            this._updateExpandedNodes();
            if (this.model.loadOnDemand && this.model.enablePersistence && !(this.dataSource() instanceof ej.DataManager)) {
                var obj, value = this._getCookies("_childNodes");
                if (value) obj = JSON.parse(value);
                (!ej.isNullOrUndefined(obj) && obj.indexOf(element[0].id) == -1) ? obj.push(element[0].id) : obj = [element[0].id];
                this._setCookies("_childNodes", JSON.stringify(obj));
            }
        },

        _getChildNodes: function (obj, rootNodes) {
            var childNodes = [];
            if (this._templateType == 1) {
                for (var i = 0, objlen = obj.length; i < objlen; i++) {
                    if (!ej.isNullOrUndefined(obj[i][0]) && !ej.isNullOrUndefined(obj[i][0][this.model.fields.parentId]) && obj[i][0][this.model.fields.parentId] == rootNodes.id)
                        return obj[i];
                }
            }
            else {
                for (var i = 0, objlen = obj.length; i < objlen; i++) {
                    if (obj[i][this.model.fields.id] == rootNodes.id)
                        return obj[i].child;
                    if (obj[i].hasOwnProperty('child')) {
                        childNodes = this._getChildNodes(obj[i].child, rootNodes);
                        if (!ej.isNullOrUndefined(childNodes) && childNodes.length > 0)
                            break;
                    }
                }
                return childNodes;
            }
        },

        _getPath: function (element) {
            var path = element.prop('name'), liEle;
            var liEle = element.parents('.e-item:first');
            while (liEle[0] != null && liEle[0].parentNode.id != this._id) {
                path = liEle.prop('name') + '/' + path;
                liEle = liEle.parents('.e-item:first');
            }
            path = this._id + "/" + path;
            return path;
        },

        _nodeSelectionAction: function (liElement) {
            if (liElement[0] == null && liElement.length == 0) return;
            var nodeDetails, data;
            if (this._triggerEvent('beforeSelect', { target: liElement, nodeDetails: this._getNodeDetails(liElement) })) return;
            this.element.find('li > div > a').removeClass('e-node-focus e-active').closest('li')
                        .find('[aria-selected=true]').attr("aria-selected", false);
            liElement.attr("aria-selected", true)
                     .find('> div > .e-text').addClass('e-active').attr("id", this._id + "_active");
            this.model.selectedNode = this._liList.index(liElement[0]);
            nodeDetails = this._getNodeDetails(liElement);
            data = { currentElement: liElement, value: nodeDetails.text, id: nodeDetails.id, parentId: nodeDetails.parentId };
            if (this.model.enablePersistence)
                this._persistValues([this.model.selectedNode], "selectedNode");
            this._triggerEvent('nodeSelect', data);
        },

        _nodeUnSelectionAction: function (liElement) {
            liElement.attr("aria-selected", false).find('> div > .e-text').removeClass('e-active').attr("id", "");
            this.model.selectedNode = null;
        },

        _nodeEnableAction: function (liElement) {
            var parent = this.getParent(liElement);
            if (parent[0] != null && parent.hasClass('e-node-disable')) return;
            if (this.model.showCheckbox)
                liElement.find('div > .e-chkbox-wrap > .nodecheckbox').ejCheckBox('enable').prop('disabled', false);
            liElement.find('.e-text').removeClass('e-node-disable')
                     .closest('.e-item')
                     .removeClass('e-node-disable')
                     .prop('disabled', true);
        },

        _nodeDisableAction: function (liElement) {
            this._animating = false;
            this._collpaseNode(liElement);
            if (this.model.showCheckbox)
                liElement.find('div > .e-chkbox-wrap > .nodecheckbox').ejCheckBox('disable').prop('disabled', true);
            liElement.find('.e-text').addClass('e-node-disable')
                     .removeClass('e-active')
                     .attr("id", "")
                     .closest('.e-item')
                     .addClass('e-node-disable')
                     .prop('disabled', true);
            var snode = this._liList[this.model.selectedNode];
            if ($(this._liList).index(liElement) == this.model.selectedNode || liElement.find(snode)[0] != null)
                this.model.selectedNode = null;
        },

        _getNodeDetails: function (liElement) {
            if (liElement[0] != null && liElement.is('LI') && liElement.hasClass('e-item')) {
                var id, text, pid, level, childs, expanded, checked, selected, index;
                id = liElement.attr('id');
                text = liElement.children('div').find('.e-text:first').text();
                pid = liElement.closest('ul').closest('.e-item').attr('id');
                level = liElement.parents('ul').length;
                childs = liElement.find('.e-item').length;
                expanded = this.isExpanded(liElement);
                checked = this.isNodeChecked(liElement);
                selected = this.isSelected(liElement);
                index = this.getNodeIndex(liElement);
                return { id: id, text: text, parentId: (pid) ? pid : "", level: level, count: childs, expanded: expanded, checked: checked, selected: selected, index: index };
            } else {
                return { id: "", text: "", parentId: "", level: "", count: "", expanded: "", checked: "", selected: "", index: "" };
            }
        },

        _denyMultipleExpand: function (liElement) {
            var enodes = this.getExpandedNodes(),
            parents = liElement.parents('.e-item'),
            distinctEle = enodes.not(parents);
            distinctEle = distinctEle.not(liElement);
            for (var i = 0, len = distinctEle.length; i < len; i++) {
                this._animating = false;
                this._collpaseNode($(distinctEle[i]));
            }
        },

        _expandCollapseAction: function (element) {
            if (element && !this._animating) {
                var expandUl, proxy = this, parentLi, nodeDetails, data;
                parentLi = element.closest('.e-item');
                if (element.is('SPAN'))
                    element = $(element).parent('div').find('div:first');
                expandUl = parentLi.children('ul:first');
                if (expandUl.find('> .e-item').length > 0) {
                    this._animating = (this.model.enableAnimation) ? true : false;
                    nodeDetails = this._getNodeDetails(parentLi);
                    data = { currentElement: parentLi, value: nodeDetails.text, isChildLoaded: this._isExpanded, id: nodeDetails.id, parentId: nodeDetails.parentId, async: false };
                    if (!this._isNodeExpanded(parentLi)) {
                        parentLi.attr("aria-expanded", true);
                        this.model.loadOnDemand ? (this._isExpanded ? (this._triggerEvent('beforeExpand', data) ? true : false) : false) : (this._triggerEvent('beforeExpand', data) ? true : false);
                        expandUl.animate({ height: 'toggle' }, this.model.enableAnimation ? 350 : 0, 'linear', function () {
                            proxy._animating = false;
                            proxy._triggerEvent('nodeExpand', data);
                            proxy._addExpandedNodes(proxy._liList.index(parentLi));
                        });
                        $(element).removeClass('e-icon e-plus').addClass('e-icon e-minus');
                        parentLi.addClass('e-collapse');
                    }
                    else {
                        parentLi.attr("aria-expanded", false);
                        if (this._triggerEvent('beforeCollapse', data) === true)
                            return false;
                        expandUl.animate({ height: 'toggle' }, this.model.enableAnimation ? 200 : 0, 'linear', function () {
                            proxy._animating = false;
                            proxy._triggerEvent('nodeCollapse', data);
                            proxy._removeExpandedNodes(proxy._liList.index(data.currentElement));
                        });
                        $(element).removeClass('e-icon e-minus').addClass('e-icon e-plus');
                        parentLi.removeClass('e-collapse');
                    }
                }
            }
        },

        _isChecked: function (liElement) {
            return (liElement.find("> div > .e-chkbox-wrap:first").attr("aria-checked") === 'true') ? true : false;
        },

        _doRecursiveCheck: function (parentLi, checkedArray) {
            var chkWrapper, chkEle, liElement, textvalue, parentLi, childUl;
            childUl = parentLi.children[1];
            chkWrapper = (childUl) ? childUl.querySelectorAll('.e-chkbox-wrap[aria-checked="true"]').length : 0;
            allChkEle = (childUl) ? childUl.querySelectorAll('.e-item > div > .e-chkbox-wrap > .nodecheckbox') : [];
            chkEle = parentLi.firstChild.querySelector('.nodecheckbox');
            if (chkEle && chkEle.nodeName.toUpperCase() == 'INPUT') {
                var chkboxObj = $(chkEle).data('ejCheckBox');
                if (chkWrapper == allChkEle.length || this.model.autoCheckParentNode) {
                    chkboxObj.setModel({ enableTriState: false, checked: true });
                    chkEle.className += " checked";
                    textvalue = parentLi.firstChild.getElementsByTagName('a')[0].innerHTML;
                    checkedArray.push({ id: parentLi.id, text: textvalue });
                    this._addHiddenInputElement(chkEle, textvalue);
                    this._addCheckNodes(this._liList.index(parentLi));
                } else
                    chkboxObj.setModel({ enableTriState: true, checkState: "indeterminate" });
                parentLi = parentLi.parentNode.parentNode;
                if (parentLi.nodeName.toUpperCase() == 'LI')
                    this._doRecursiveCheck(parentLi, checkedArray);
            }
        },

        _nodeCheck: function (chkEle, args) {
            if ($(chkEle)[0] == null) return;
            this._CurrenctSelectedNodes = [];
            var checkedArray = [], chkObj, currentLi, textvalue, chkCollection, chklen, parentLi;
            chkEle.className += " checked";
            chkObj = $(chkEle).data('ejCheckBox');
            if (chkObj) {
                chkObj.setModel({ enableTriState: false, checked: true });
                currentLi = chkEle.parentNode.parentNode.parentNode;
                textvalue = currentLi.firstChild.getElementsByTagName('a')[0].lastChild.nodeValue;
                checkedArray.push({ id: currentLi.id, text: textvalue });
                this._addHiddenInputElement(chkEle, textvalue);
                this._addCheckNodes(this._liList.index(currentLi));
                if (this.model.autoCheck) {
                    var allChkEle, liElement, childUl;
                    childUl = currentLi.children[1];
                    allChkEle = (childUl) ? childUl.querySelectorAll('.e-item > div > .e-chkbox-wrap > .nodecheckbox') : [];
                    for (var i = 0, chklen = allChkEle.length; i < chklen; i++) {
                        $(allChkEle[i]).ejCheckBox({ checked: true, enableTriState: false })[0].className += " checked";
                        liElement = allChkEle[i].parentNode.parentNode.parentNode;
                        textvalue = liElement.firstChild.getElementsByTagName('a')[0].lastChild.nodeValue;
                        checkedArray.push({ id: liElement.id, text: textvalue });
                        this._addHiddenInputElement(allChkEle[i], textvalue);
                        this._addCheckNodes(this._liList.index(liElement));
                    }
                    parentLi = currentLi.parentNode.parentNode;
                    if (parentLi.nodeName.toUpperCase() == 'LI')
                        this._doRecursiveCheck(parentLi, checkedArray);
                }
                (args) && (args.checknode = checkedArray);
                this._onChecked($(chkEle), args);
            }
            this._persistValues(this.model.checkedNodes, "checkedNodes");
        },

        _addHiddenInputElement: function (checkedElement, textVal) {
            if (checkedElement.firstChild == null) {
                var hiddenElement = document.createElement('input');
                hiddenElement.setAttribute("type", "hidden");
                hiddenElement.setAttribute("name", this._id + "_Checkbox_" + checkedElement.value + ".Text");
                hiddenElement.setAttribute("value", textVal);
                try {
                    checkedElement.appendChild(hiddenElement);
                }
                catch (err) { }
            }
        },

        _doRecursiveUncheck: function (parentLi, uncheckedArray) {
            var chkWrapper, chkEle, liElement, textvalue, parentLi, childUl;
            childUl = parentLi.children[1];
            chkWrapper = (childUl) ? childUl.querySelectorAll('.e-chkbox-wrap[aria-checked="true"]').length : 0;
            allChkEle = (childUl) ? childUl.querySelectorAll('.e-item > div > .e-chkbox-wrap > .nodecheckbox') : [];
            chkEle = parentLi.firstChild.querySelector('.nodecheckbox');
            if (chkEle && chkEle.nodeName.toUpperCase() == 'INPUT') {
                var chkboxObj = $(chkEle).data('ejCheckBox');
                if (chkWrapper <= 0) {
                    chkboxObj.setModel({ enableTriState: false, checked: false });
                    $(chkEle).removeClass("checked").removeClass('checked').children().remove();
                    textvalue = parentLi.firstChild.getElementsByTagName('a')[0].innerHTML;
                    this._removeCheckNodes(this._liList.index(parentLi));
                    uncheckedArray.push({ id: parentLi.id, text: textvalue });
                }
                if (chkWrapper > 1 && this.model.autoCheckParentNode) {
                    chkboxObj.setModel({ checked: true });
                    chkEle.className += " checked";
                    textvalue = parentLi.firstChild.getElementsByTagName('a')[0].innerHTML;
                    this._addHiddenInputElement(parentLi, textvalue);
                }
                else if (chkWrapper > 0 && !this.model.autoCheckParentNode) {
                    if (allChkEle.length > 1)
                        chkboxObj.setModel({ enableTriState: true, checkState: "indeterminate" });
                    else
                        chkboxObj.setModel({ checked: false });
                    $(chkEle).removeClass("checked").removeClass('checked').children().remove();
                    this._removeCheckNodes(this._liList.index(parentLi));
                }
                parentLi = parentLi.parentNode.parentNode;
                if (parentLi.nodeName.toUpperCase() == 'LI')
                    this._doRecursiveUncheck(parentLi, uncheckedArray);
            }
        },

        _nodeUncheck: function (chkEle, args) {
            if ($(chkEle)[0] == null) return;
            var proxy = this, uncheckedArray = [];
            currentLi = chkEle.parentNode.parentNode.parentNode;
            $(currentLi).find('div > .e-chkbox-wrap > .nodecheckbox').removeClass('checked').children().remove();
            chkObj = $(chkEle).data('ejCheckBox');
            this._removeCheckNodes(this._liList.index(currentLi));
            if (chkObj) {
                chkObj.setModel({ enableTriState: false, checked: false });
                textvalue = currentLi.firstChild.getElementsByTagName('a')[0].lastChild.nodeValue;
                uncheckedArray.push({ id: currentLi.id, text: textvalue });
                if (this.model.autoCheck) {
                    childUl = currentLi.children[1];
                    allChkEle = (childUl) ? childUl.querySelectorAll('.e-item > div > .e-chkbox-wrap > .nodecheckbox') : [];
                    for (var i = 0, chklen = allChkEle.length; i < chklen; i++) {
                        $(allChkEle[i]).ejCheckBox({ enableTriState: false, checked: false });
                        liElement = allChkEle[i].parentNode.parentNode.parentNode;
                        textvalue = liElement.firstChild.getElementsByTagName('a')[0].lastChild.nodeValue;
                        uncheckedArray.push({ id: liElement.id, text: textvalue });
                        this._removeCheckNodes(this._liList.index(liElement));
                    }
                    parentLi = currentLi.parentNode.parentNode;
                    if (parentLi.nodeName.toUpperCase() == 'LI')
                        this._doRecursiveUncheck(parentLi, uncheckedArray);
                }
                (args) && (args.unchecknode = uncheckedArray);
                this._onUnChecked($(chkEle), args);
            }
            this._persistValues(this.model.checkedNodes, "checkedNodes");
        },

        _createChildNodesWhenExpand: function (parentLi) {
            var nodeid, nodeText, args, element;
            if (parentLi.length > 0 && parentLi.find('ul .e-item').length == 0) {
                nodeid = parentLi.attr('id');
                nodeText = parentLi.children('div').find('.e-text:first').text();
                element = parentLi.children('div').find('div.e-plus:first');
                args = { currentElement: parentLi, targetElement: element[0], id: nodeid, value: nodeText, isChildLoaded: false, hasParent: true, async: false };
                var childItems, parent, mapper, proxy = this, queryManager;
                this._triggerEvent('beforeExpand', args);
                if (this.dataSource() instanceof ej.DataManager) {
                    parent = args.currentElement.parents('ul').length;
                    mapper = this._getChildTables(this.model.fields, parent, 1);
                    if (ej.isNullOrUndefined(mapper) && ej.isNullOrUndefined(this.model.fields["child"]))
                        mapper = this.model.fields;
                    if (!mapper) return;
                    this._checkMapperFields(mapper);
                    queryManager = ej.Query();
                    queryManager = proxy._columnToSelect(mapper);
                    queryManager.where(mapper["parentId"], ej.FilterOperators.equal, parseInt(args.currentElement[0].id));
                    queryPromise = mapper["dataSource"].executeQuery(queryManager);
                    queryPromise.done(function (e) {
                        childItems = e.result;
                        if (!ej.isNullOrUndefined(childItems) && childItems.length > 0) {
                            proxy._checkboxChecked = parentLi.find('> div > .e-chkbox-wrap > .nodecheckbox').hasClass('checked');
                            proxy._loadOnDemandNodes = true;
                            if (parentLi.find('ul .e-item').length == 0) {
                                proxy._templateNodeCreation(childItems, mapper);
                                parentLi.append(proxy._fragment);
                            }
                            proxy._finalizeLoadOnDemand(parentLi);
                            proxy._expandNode(parentLi);
                        }
                    });
                }
                else {
                    var childItems = this._getChildNodes(this._dataSource, { id: parentLi[0].id });
                    if (!ej.isNullOrUndefined(childItems) && childItems.length > 0) {
                        this._checkboxChecked = parentLi.find('> div > .e-chkbox-wrap > .nodecheckbox').hasClass('checked');
                        this._loadOnDemandNodes = true;
                        this._templateNodeCreation(childItems, this.model.fields);
                        parentLi.append(this._fragment);
                        this._finalizeLoadOnDemand(parentLi);
                        this._expandNode(parentLi);
                    }
                }
            }
            else
                this._expandNode(parentLi);
        },

        _expandNode: function (liElement) {
            if (liElement[0] != null && liElement.length > 0) {
                var expandIcon = liElement.find('> div > div:first');
                if (this.model.loadOnDemand && !this.model.enablePersistence) {
                    if (liElement.find('> ul > .e-item').length > 0 && expandIcon.hasClass('e-plus')) {
                        if (!this.model.enableMultipleExpand) this._denyMultipleExpand(liElement);
                        this._animating = false;
                        this._expandCollapseAction(liElement.find('> div > div.e-plus:first'));
                    } else if (expandIcon.hasClass('e-icon') && !expandIcon.hasClass('e-minus')) {
                        (this._allowOnDemand) && this._createChildNodesWhenExpand(liElement);
                    }
                }
                else {
                    if (expandIcon.hasClass('e-plus')) {
                        var isExpanded = liElement.find('> ul > .e-item').length > 0 ? true : false;
                        if (isExpanded) {
                            if (!this.model.enableMultipleExpand) this._denyMultipleExpand(liElement);
                            this._animating = false;
                            this._expandCollapseAction(expandIcon);
                        }
                    }
                }
            }
            return true;
        },

        _collpaseNode: function (liElement) {
            if (liElement[0] != null && liElement.length > 0) {
                if (liElement.find('> ul > .e-item').length > 0) {
                    var collapseIcon = liElement.find('> div > div:first');
                    if (collapseIcon.hasClass('e-minus'))
                        this._expandCollapseAction(collapseIcon);
                }
            }
        },

        _expandAll: function () {
            var element = this.element, i, liCollection;
            if (this.model.loadOnDemand) {
                liCollection = $(element).find(".e-item");
                for (i = 0, len = liCollection.length; i < len; i++)
                    this._createChildNodesWhenExpand($(liCollection[i]));
            }
            else {
                cnodes = element.find('.e-item > div > .e-plus').closest('.e-item');
                if (element.find('ul:hidden').length == cnodes.length) {
                    for (i = 0, len = cnodes.length; i < len; i++)
                        this._expandNode($(cnodes[i]));
                }
            }
        },

        _collapseAll: function () {
            var element = this.element, i, liCollection;
            liCollection = element.find('.e-item > ul:not(:hidden)').closest('.e-item');
            enodes = liCollection.find('> div > .e-minus').closest('.e-item');
            if (enodes.length > 0) {
                for (i = 0, len = enodes.length; i < len; i++) {
                    this._animating = false;
                    this._collpaseNode($(enodes[i]));
                }
            }
        },

        _checkAll: function () {
            this._CurrenctSelectedNodes = [];
            var element = this.element, chkColl;
            chkColl = element.find('.e-item > div > .e-chkbox-wrap > .nodecheckbox');
            chkColl.addClass('checked');
            for (i = 0, len = chkColl.length; i < len; i++) {
                $(chkColl[i]).ejCheckBox("setModel", { checked: true });
                this._addHiddenInputElement(chkColl[i], $(chkColl[i]).parent().siblings('.e-text')[0].lastChild.nodeValue);
            }
            this.model.checkedNodes = this.getCheckedNodesIndex();
        },

        _uncheckAll: function () {
            var element = this.element, chkColl;
            chkColl = element.find('.e-item > div > .e-chkbox-wrap > .nodecheckbox');
            chkColl.removeClass("checked").children().remove();
            for (i = 0, len = chkColl.length; i < len; i++)
                $(chkColl[i]).ejCheckBox("setModel", { checked: false });
            this.model.checkedNodes = [];
            this.model.checkedNodes.push(-1);
        },

        _isNodeExpanded: function (liElement) {
            if (liElement[0] != null)
                return liElement.children('ul:first').length > 0 && liElement.find('> div > div.e-minus').length > 0;
        },

        _showCheckBox: function () {
            var element = this.element, licoll, i, subItems;
            licoll = element.find('li');
            for (var i = 0; i < licoll.length; i++)
                this._checkboxOnTemplate(licoll[i].children[0]);
            element.find(".e-item > div > .nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
            element.find(".e-item.e-node-disable > div > .e-chkbox-wrap > .nodecheckbox").ejCheckBox("disable");
        },

        _drag: function () {
            var proxy, pre = false, browserInfo = ej.browserInfo(), _clonedElement = null, _draggedElement = null, dragContainment = null, _isIE8;
            _isIE8 = (browserInfo.name == 'msie' && browserInfo.version == '8.0') ? true : false;
            if (this.element.is("ul")) {
                this._treeView = this.element.parent();
                if (!this.model.allowDragAndDropAcrossControl)
                    dragContainment = this.element.parent();
            }
            else {
                this._treeView = this.element;
                if (!this.model.allowDragAndDropAcrossControl)
                    dragContainment = this.element;
            }
            $(this._treeView).find("ul li div a").not(".e-js").ejDraggable({
                dragArea: dragContainment,
                clone: true,
                dragStart: function (args) {
                    if (proxy && !ej.isNullOrUndefined(args.target) && !$(args.target).hasClass('e-node-disable') && proxy.element.find('.e-item > div > .e-text.e-editable').length == 0) {
                        if ($(args.target).is('A') && !$(args.target).hasClass('e-draggable')) return false;
                        if (_isIE8) document.ondragstart = function () { return false; }
                        var data = { target: $(args.target).closest(".e-item"), targetElementData: proxy._getNodeDetails($(args.target).closest(".e-item")), dragTarget: args.target, parentElement: $(args.target).closest("ul").closest('.e-item'), parentElementData: proxy._getNodeDetails($(args.target).closest("ul").closest('.e-item')), event: args.event };
                        if (proxy._triggerEvent('nodeDragStart', data)) {
                            args.cancel = true;
                            _clonedElement && _clonedElement.remove();
                            return false;
                        }
                    } else return false;
                },
                drag: function (args) {
                    pre = false;
                    $('.e-sibling').remove();
                    var target = args.target, trgtEle = proxy._findTarget($(target)), data;
                    $(_clonedElement).css({ "margin-left": "20px", "margin-top": args.event.clientY - 20 < 0 ? "0px" : "-20px" });
                    data = { draggedElement: $(args.element).closest(".e-item"), draggedElementData: proxy._getNodeDetails($(args.element).closest(".e-item")), dragTarget: target, target: trgtEle, targetElementData: proxy._getNodeDetails(trgtEle), event: args.event };
                    if (($(target).hasClass('e-droppable') || $(target).parent().hasClass('e-droppable')) && $(target).hasClass('e-dropchild') && !$(target).hasClass('e-node-disable')) {
                        document.body.style.cursor = '';
                        $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-plus");
                        $(target).addClass("allowDrop");
                    } else if (($(target).hasClass('e-droppable') && !$(target).hasClass('e-item') && !$(target).hasClass('e-text')) || ($(target).is("UL") && $(target).hasClass('e-ul') && $(target).find('.e-item').length == 0 && $(target).parent('.e-treeview-wrap').length > 0)) {
                        document.body.style.cursor = '';
                        $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-plus");
                    }
                    else if ((!$(target).hasClass('e-sibling') && !$(target).find('a').hasClass('e-text')) || (!$(target).hasClass('e-sibling') && !$(target).parent().parent().hasClass('e-item') && !$(target).parent().hasClass('e-item') && !$(target).hasClass('e-item') && !$(target).hasClass('e-text'))) {
                        document.body.style.cursor = 'not-allowed';
                        $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-minus");
                        $(target).removeClass('showline-hover');
                        $(target).removeClass('noline-hover');
                    }
                    if (target.nodeName != 'A') {
                        if (target.nodeName == 'UL' && $(target).children()[0] != null)
                        { target = $(target).children()[0]; pre = true; }
                        if (target.nodeName != 'LI')
                            target = $(target).closest('.e-droppable')[0] || $(target).parent();
                        if (target.nodeName == 'LI' && $(target).hasClass("e-droppable") && $(target).hasClass("e-dropsibling")) {
                            var div = ej.buildTag('div.e-sibling'), targetY;
                            targetY = $(target).offset().top + $(target).find('a').height() || -1;
                            pre = (args.event.pageY > targetY) ? false : true;
                            pre ? div.insertBefore($(target).find('> div > a').parent()) : div.insertAfter($(target).find('> div > a').parent());
                            if ($(target).parents().hasClass("e-rtl")) {
                                document.body.style.cursor = '';
                                $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-insertInbetween-rtl");
                            }
                            else {
                                document.body.style.cursor = '';
                                $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-insertInbetween");
                            }
                        }
                        else if (target.nodeName == 'A' && $(target) && $(target).hasClass('e-droppable') && $(target).hasClass('e-dropchild')) {
                            document.body.style.cursor = '';
                            $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-plus");
                            $(target).addClass("allowDrop");
                        } else if ($(target).hasClass('e-droppable') && !$(target).hasClass('e-item') && !$(target).hasClass('e-text')) {
                            document.body.style.cursor = '';
                            $(_clonedElement).find('span.e-dropedStatus').removeClass().addClass("e-dropedStatus e-icon e-plus");
                        }
                    }
                    else $('.e-sibling').remove();
                    if (proxy._triggerEvent('nodeDrag', data))
                        return false;
                },
                dragStop: function (args) {
                    if (_isIE8) document.ondragstart = function () { return true; }
                    if (!args.element.dropped) {
                        _clonedElement && _clonedElement.remove();
                        document.body.style.cursor = '';
                    }
                    var target = args.target, position, data;
                    if (target.className == "e-sibling")
                        target = $(target).closest(".e-item")[0];
                    $('.e-sibling').remove();
                    if ($(target).hasClass('e-node-disable')) return false;
                    position = pre ? "Before" : "After", trgtEle = proxy._findTarget($(target));
                    position = target.nodeName == 'A' ? "Over" : position;
                    data = { draggedElementData: proxy._getNodeDetails($(args.element).closest('.e-item')), draggedElement: $(args.element).closest('.e-item'), dropTarget: $(target), target: trgtEle, targetElementData: proxy._getNodeDetails(trgtEle), position: position, event: args.event };
                    if (proxy._triggerEvent('nodeDragStop', data))
                        return false;
                    if (target.nodeName == 'A' && $(target).hasClass('e-dropchild') && $(target).hasClass('e-droppable') || (target.nodeName == 'UL' && $(target).children().length == 0)) {
                        position = "Over";
                        if ($(target).is("UL") && $(target).hasClass('e-ul') && $(target).find('.e-item').length == 0 && $(target).parent('.e-treeview-wrap').length > 0)
                            proxy._dropAsChildNode($(target), $(args.element), args.event);
                        else if (($(args.element).parent().parent().has($(target)).length == 0) && ($(target).parent().parent().has($(args.element)).length == 0 || proxy._isDescendant($(target).parents("li:last").find('>ul>li'), $(args.element).parents("li:first")[0])))
                            proxy._dropAsChildNode($(target).closest('.e-item'), $(args.element), args.event);
                    }
                    else {
                        if (target.nodeName == 'UL')
                            target = $(target).children()[0];
                        if (target.nodeName != 'LI')
                            target = $(target).closest('.e-droppable')[0] || $(target).parent();
                        if (target.nodeName == 'LI' && $(target).hasClass('e-dropsibling') && $(target).hasClass('e-droppable')) {
                            if ($(args.element).parent().parent().has($(target)).length < 1)
                                proxy._dropAsSublingNode($(target), $(args.element), pre, args.event);
                        }
                        else if (target.nodeName == 'A' && $(target).hasClass('e-dropchild') && $(target).hasClass('e-droppable')) {
                            position = "Over";
                            if (($(args.element).parent().parent().has($(target)).length == 0) && ($(target).parent().parent().has($(args.element)).length == 0 || proxy._isDescendant($(target).parents("li:last").find('>ul>li'), $(args.element).parents("li:first")[0])))
                                proxy._dropAsChildNode($(target).closest('.e-item'), $(args.element), args.event);
                        }
                    }
                    $(".allowDrop").removeClass("allowDrop");
                    if (!$(target).hasClass('e-dropchild')) {
                        _clonedElement && _clonedElement.remove();
                    }
                    trgtEle = proxy._findTarget($(target));
                    data = { droppedElementData: proxy._getNodeDetails($(args.element).closest('.e-item')), droppedElement: $(args.element).closest('.e-item'), dropTarget: $(target), target: trgtEle, targetElementData: proxy._getNodeDetails(trgtEle), position: position, event: args.event };
                    if (proxy._triggerEvent('nodeDropped', data))
                        return false;
                    document.body.style.cursor = '';
                },
                helper: function (event, ui) {
                    if (!ej.isNullOrUndefined(event.element) && !$(event.element).hasClass('e-node-disable') && $(event.element).hasClass('e-draggable')) {
                        proxy = $(event.element).closest('.e-treeview.e-js').data('ejTreeView');
                        if (proxy) {
                            _clonedElement = ej.buildTag('div.e-dragedNode');
                            _clonedElement.addClass(proxy.model.cssClass);
                            _draggedElement = $(event.element).clone().addClass("dragClone");
                            this.spanEle = ej.buildTag('span.e-icon e-plus e-dropedStatus');
                            _clonedElement.append(this.spanEle);
                            _clonedElement.append(_draggedElement);
                            return _clonedElement.appendTo($("body"));
                        }
                    }
                }
            });
        },

        _findTarget: function (trgt) {
            return trgt.hasClass("e-text") ? trgt.closest(".e-item") : (trgt.closest("ul").closest('.e-item').length > 0) ? trgt.closest("ul").closest('.e-item') : this.element.find(trgt).length > 0 ? trgt.parents("ul").first() : trgt;
        },

        _isDescendant: function (src, target) {
            var match = true;
            $(src).each(function (i, item) {
                if (item == target) {
                    match = false;
                    return false;
                }
                else
                    match = true;
            });
            return match;
        },

        _childDrop: function () {
            $(this._treeView).find("ul .e-item div .e-text").ejDroppable({
                accept: $(this._treeView).find("ul .e-item div .e-text").addClass('e-dropchild'),
                drop: function (event, ui) {
                    $(ui.helper).hide();
                }
            });
        },

        _siblingDrop: function () {
            $(this._treeView).find("ul .e-item").addClass('e-dropsibling').ejDroppable({
                drop: function (event, ui) {
                    $(ui.helper).hide();
                }
            });
        },

        _dropAsSublingNode: function (target, element, pre, event) {
            var li = element.parent().parent(), li = $(li), parentNode;
            parentNode = $(element.parents('.e-item')[1]);
            if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null && !(this.dataSource() instanceof ej.DataManager))
                this._updateDataSource(element.parents('.e-item:first'), $(target), pre);
            pre ? li.insertBefore(target) : li.insertAfter(target);
            (!this.model.template) && this._autoGenerateNodes(element.parents('.e-item:first'));
            this._modifyCss(parentNode);
            this._finalizeEditing(li);
            this._updateCheckState(li);
            this._updateCheckState(parentNode);
            (li.find('> div > .e-minus').length > 0 && !this.model.enableMultipleExpand) && this._denyMultipleExpand(li);
            this._updateChanges($(target));
        },

        _dropAsChildNode: function (target, element, event) {
            var li = element.parent().parent(), li = $(li), parentNode;
            parentNode = $(element.parents('.e-item')[1]);
            if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null && !(this.dataSource() instanceof ej.DataManager))
                this._updateDataSource(element.parents('.e-item:first'), $(target), "");
            ($(target).is('UL')) ? $(target).append(li) : this._appendNode(target, li);
            (!this.model.template) && this._autoGenerateNodes(element.parents('.e-item:first'));
            this._modifyCss(parentNode);
            this._finalizeEditing(li);
            this._updateCheckState(li);
            this._updateCheckState(parentNode);
            if (target.find('> div > .e-icon.e-plus:first').length > 0) this._expandNode(target);
            this._updateChanges($(target));
        },

        _updateChanges: function (dropEle) {
            if ($(dropEle)[0] != null) {
                var desTree = this, tree = 0;
                do {
                    desTree._updateNodes();
                    tree++;
                    desTree = dropEle.closest('.e-treeview.e-js').data('ejTreeView')
                } while (desTree && this._id != desTree._id && tree == 1);
            }
        },

        _orderDataSource: function () {
            this._beforeBaseClass();
            var orderedData = [], datasource = this._newDataSource;
            for (var j = 0, objlen = this._liList.length; j < objlen; j++) {
                for (var i = 0, datalen = datasource.length; i < datalen; i++) {
                    if (!ej.isNullOrUndefined(datasource[i]) && !ej.isNullOrUndefined(datasource[i][this.model.fields.id]) && datasource[i][this.model.fields.id].toString() == $(this._liList[j]).attr('id')) {
                        if ($(this._liList[j]).find('> ul .e-item').length == 0)
                            delete datasource[i][this.model.fields.hasChild];
                        else if (this._templateType == 1)
                            datasource[i][this.model.fields.hasChild] = true;
                        orderedData.push(datasource[i]);
                        break;
                    }
                }
            }
            return orderedData;
        },

        _updateDataSource: function (dragEle, dropEle, before) {
            if ($(dragEle)[0] != null && $(dropEle)[0] != null) {
                var sourceTree = dragEle.closest('.e-treeview.e-js').data('ejTreeView'), desTree, pid, indexPos;
                desTree = dropEle.closest('.e-treeview.e-js').data('ejTreeView');
                if (sourceTree && desTree && !ej.isNullOrUndefined(desTree.model.fields) && desTree.dataSource() != null && !(desTree.dataSource() instanceof ej.DataManager)) {
                    if (dropEle.is("A")) {
                        pid = dropEle.closest('.e-item').attr('id');
                        var child = dropEle.closest('.e-item').find('> ul > .e-item');
                        indexPos = (child.length > 0) ? child.length : dropEle.closest('.e-item').index();
                    }
                    else {
                        pid = dropEle.parents('.e-item:first').attr('id');
                        var nextLi = dropEle.next('.e-item'), prevLi = dropEle.prev('.e-item');
                        if (nextLi.length > 0)
                            indexPos = (prevLi.length > 0) ? indexPos = nextLi.index() - 1 : (before) ? indexPos = dropEle.index() : indexPos = nextLi.index();
                        else
                            indexPos = (before) ? dropEle.index() : dropEle.index() + 1;
                    }
                    (desTree) && sourceTree._findAndUpdate(dragEle.attr('id'), pid, sourceTree._newDataSource, indexPos, desTree);
                }
            }
        },

        _removeObject: function (data, searchId, key) {
            for (var i = 0, len = data.length; i < len; i++) {
                if (!ej.isNullOrUndefined(data[i]) && !ej.isNullOrUndefined(data[i][key]) && data[i][key].toString() == searchId) {
                    data.splice(i, 1);
                    break;
                }
            }
        },

        _convertDataSourceTypes: function (data, resultobj) {
            for (var j = 0; j < data.child.length; j++) {
                data.child[j][this.model.fields.parentId] = data[this.model.fields.id];
                resultobj.push(data.child[j]);
                if (data.child[j].hasOwnProperty('child') && data.child[j].child.length > 0) {
                    this._convertDataSourceTypes(data.child[j], resultobj);
                }
            }
            if (data['child']) {
                delete data['child'];
                data[this.model.fields.hasChild] = true;
            }
            return resultobj;
        },

        _childObjectCollection: function (childNodes, groupedObj, resultobj) {
            for (var j = 0, len = childNodes.length; j < len; j++) {
                var temp = this._getChildNodes(groupedObj, { id: childNodes[j][this.model.fields.id] });
                if (temp) {
                    resultobj = resultobj.concat(temp);
                    resultobj = this._childObjectCollection(temp, groupedObj, resultobj);
                }
            }
            return resultobj;
        },

        _findAndUpdate: function (searchId, parentId, obj, index, tree2) {
            if (this._templateType == 1) {
                for (var i = 0, objlen = obj.length; i < objlen; i++) {
                    if (!ej.isNullOrUndefined(obj[i]) && !ej.isNullOrUndefined(obj[i][this.model.fields.id]) && obj[i][this.model.fields.id].toString() == searchId) {
                        if (this._id == tree2._id) {
                            (!ej.isNullOrUndefined(parentId)) ? (obj[i][this.model.fields.parentId] = parentId) : (delete obj[i][this.model.fields.parentId]);
                        } else {
                            var remObj, childNodes, proxy = this, groupedObj, result = [];
                            groupedObj = this._groupingObjects(obj, function (obj) { return [!ej.isNullOrUndefined(obj) && [obj[proxy.model.fields.parentId]].toString()]; });
                            remObj = obj.splice(i, 1);
                            childNodes = this._getChildNodes(groupedObj, { id: searchId });
                            if (childNodes && childNodes.length > 0)
                                childNodes = this._childObjectCollection(childNodes, groupedObj, childNodes);
                            if (tree2._templateType == 2) {
                                if (childNodes && childNodes.length > 0) {
                                    for (var j = 0, len = childNodes.length; j < len; j++) {
                                        this._removeObject(obj, searchId, this.model.fields.parentId);
                                        delete childNodes[j][tree2.model.fields.parentId];
                                        delete childNodes[j][tree2.model.fields.hasChild];
                                    }
                                    if (!remObj[0].hasOwnProperty('child'))
                                        remObj[0].child = [];
                                    remObj[0].child = childNodes;
                                }
                                delete remObj[0][tree2.model.fields.parentId];
                                delete remObj[0][tree2.model.fields.hasChild];
                                (parentId) ? this._changeObjectPos(parentId, remObj, tree2._newDataSource, index) : tree2._newDataSource.splice(index, 0, remObj[0]);
                            } else {
                                if (childNodes && childNodes.length > 0) {
                                    for (var j = 0, len = childNodes.length; j < len; j++)
                                        this._removeObject(obj, searchId, this.model.fields.parentId);
                                }
                                (parentId) ? (remObj[0][tree2.model.fields.parentId] = parentId) : (delete remObj[0][tree2.model.fields.parentId]);
                                tree2._newDataSource.push(remObj[0]);
                                if (childNodes && childNodes.length > 0)
                                    tree2._newDataSource = tree2._newDataSource.concat(childNodes);
                            }
                        }
                        break;
                    }
                }
            }
            else {
                for (var i = 0, objlen = obj.length; i < objlen; i++) {
                    if (obj[i][this.model.fields.id].toString() == searchId) {
                        var remObj = obj.splice(i, 1);
                        if (this._id == tree2._id) {
                            (parentId) ? this._changeObjectPos(parentId, remObj, this._newDataSource, index) : this._newDataSource.splice(index, 0, remObj[0]);
                        } else {
                            if (tree2._templateType == 1) {
                                // convert nested child datasource into id & parentId case datasource
                                var result = [];
                                if (remObj[0].hasOwnProperty('child')) {
                                    tree2._convertDataSourceTypes(remObj[0], result);
                                    remObj[0] = remObj.concat(result);
                                } else
                                    remObj[0] = remObj.concat();
                                (parentId) ? (remObj[0][0][tree2.model.fields.parentId] = parentId) : (delete remObj[0][0][tree2.model.fields.parentId]);
                                tree2._newDataSource = tree2._newDataSource.concat(remObj[0]);
                            } else {
                                (parentId) ? this._changeObjectPos(parentId, remObj, tree2._newDataSource, index) : tree2._newDataSource.splice(index, 0, remObj[0]);
                            }
                        }
                        return true;
                    }
                    else if (obj[i].hasOwnProperty('child')) {
                        if (this._findAndUpdate(searchId, parentId, obj[i].child, index, tree2))
                            break;
                    }
                }
            }
        },

        _changeObjectPos: function (searchId, remObj, data, index) {
            for (var i = 0, objlen = data.length; i < objlen; i++) {
                if (data[i][this.model.fields.id].toString() == searchId) {
                    if (data[i].hasOwnProperty('child')) {
                        data[i].child.splice(index, 0, remObj[0]);
                    } else {
                        data[i].child = [];
                        data[i].child.push(remObj[0]);
                    }
                    return true;
                }
                else if (data[i].hasOwnProperty('child')) {
                    if (this._changeObjectPos(searchId, remObj, data[i].child, index))
                        break;
                }
            }
        },

        _finalizeEditing: function (element) {
            if ($(element)[0] != null) {
                var thisObj = element.closest('.e-treeview.e-js').data('ejTreeView');
                if (thisObj) {
                    thisObj._preventEditable();
                    thisObj.model.allowEditing && thisObj._allowEditable();
                }
            }
        },

        _updateCheckState: function (element) {
            if (this._isTreeElement(element)) {
                var thisObj = element.closest('.e-treeview.e-js').data('ejTreeView'), chdNodes;
                if (thisObj && thisObj.model.showCheckbox) {
                    chdNodes = element.children('ul').find('.e-item > div > .e-chkbox-wrap > .checked').closest('.e-item').addClass('checked');
                    if (chdNodes.length > 0)
                        thisObj._isCheckedAction();
                    else {
                        var action = (thisObj.isNodeChecked(element)) ? "_nodeCheck" : "_nodeUncheck";
                        thisObj[action](element.find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0]);
                    }
                }
            }
        },

        _autoGenerateNodes: function (element) {
            if ($(element)[0] != null) {
                var thisObj = element.closest('.e-treeview.e-js').data('ejTreeView');
                if (thisObj && thisObj.model.showCheckbox) {
                    if (element.find(".e-chkbox-wrap").length == 0) {
                        this._checkboxOnTemplate(element[0].children[0]);
                        var subItems = element.find('.e-item');
                        for (var i = 0, len = subItems.length; i < len ; i++)
                            this._checkboxOnTemplate(subItems[i].children[0]);
                        element.find(".nodecheckbox").ejCheckBox({ cssClass: thisObj.model.cssClass, change: thisObj._checkedChange });
                    }
                } else
                    element.find('.e-chkbox-wrap').remove();
            }
        },

        _appendNode: function (element, outerLi) {
            if (this._isTreeElement(element)) {
                var thisObj, outerUl, divTag, nodeid, nodeText, data, parentId, expandList;
                if (element.find('ul')[0] == null) {
                    thisObj = element.closest('.e-treeview.e-js').data('ejTreeView');
                    if (thisObj) {
                        thisObj._isExpanded = this._isNodeExpanded(element);
                        if (!thisObj._isExpanded && thisObj.model.loadOnDemand && !(this.dataSource() instanceof ej.DataManager) && element.find('> div > div').first().hasClass('e-plus'))
                            thisObj._createChildNodesWhenExpand(element);
                    }
                    outerUl = ej.buildTag("ul.e-treeview-ul", "", {}, { role: "group", style: "display:none" });
                    $(outerUl).append($(outerLi));
                    if ($(element.find('div')[1]).length == 0) {
                        divTag = document.createElement('div');
                        divTag.setAttribute('role', 'presentation');
                        divTag.className = 'e-icon e-minus';
                        $(element.find('div')).append($(divTag));
                    } else if (!$(element.find('div')[1]).hasClass('e-minus'))
                        $(element.find('div')[1]).addClass('e-icon e-plus');
                    if (thisObj && thisObj.model.loadOnDemand && !(this.dataSource() instanceof ej.DataManager) && element.find('ul').length > 0)
                        element.children('ul').append($(outerLi));
                    else
                        element.append($(outerUl));
                }
                else
                    $(element.find('ul')[0]).append(outerLi);
                (!this.isDisabled(element)) && this._expandNode(element);
            }
            else {
                if (this.element.is('UL')) {
                    this.element.append(outerLi);
                } else if (this.element.children('ul:first').length > 0) {
                    this.element.children('ul:first').append(outerLi);
                } else {
                    outerUl = ej.buildTag("ul.e-treeview-ul", "", {}, { role: "group" });
                    outerUl.append(outerLi);
                    this.element.append(outerUl);
                }
            }
            expandList = this.element.find(".e-item.expanded");
            for (var i = 0; i < expandList.length; i++) {
                (this.model.loadOnDemand && !(this.dataSource() instanceof ej.DataManager)) ? this._createChildNodesWhenExpand($(expandList[i])) : this._expandNode($(expandList[i]));
            }
            expandList.removeClass("expanded");
            this.model.showCheckbox && this._isCheckedAction();
        },

        _modifyCss: function (liElement) {
            liElement = $(liElement);
            if (liElement[0] != null && liElement.find('.e-item').length == 0)
                liElement.removeClass('e-collapse').attr("aria-expanded", false).find('> div > .e-icon').removeClass('e-icon e-minus').closest('.e-item').find('ul').remove();
        },

        _applyFirstLastChildClass: function () {
            $(this._liList).removeClass('first last').filter(':first-child:not(:last-child)').addClass('first');
            $(this._liList).filter(':last-child').addClass('last');
        },

        _expandEventHandler: function (event) {
            var target = $(event.target), selectedNode, liElement;
            if (target.hasClass("e-icon")) return;
            if (!(event.type === "dblclick" && this.model.allowEditing)) {
                event.preventDefault();
                selectedNode = (target.is('A')) ? $(target.siblings('div')) : $(target.parent().siblings('div'));
                liElement = target.closest('.e-item');
                if (selectedNode.hasClass('e-minus'))
                    this._collpaseNode(liElement);
                else {
                    if (this.model.loadOnDemand && selectedNode.find(".e-item").length == 0) {
                        this._isclicked = false;
                        this._ClickEventHandler({ target: selectedNode[0] });
                    }
                    else this._expandNode(liElement);
                }
            }
        },

        _inlineEdit: function (event) {
            event.preventDefault();
            var target = $(event.target);
            if (!target.hasClass('input-text') && !target.hasClass("e-node-disable") && target.closest('.e-item').hasClass('AllowEdit')) {
                if (!target.is('A'))
                    target = target.closest('.e-text');
                if (target.is('A') && !target.hasClass("e-node-disable") && target.closest('.e-item').hasClass('AllowEdit')) {
                    this.unselectNode(target.closest('.e-item'));
                    this._inlineEditAction(target);
                }
            }
            return false;
        },

        _inlineEditAction: function (element) {
            var editTextBox = this.element.find('.e-item > div > .e-text > #Edit_Input');
            (editTextBox[0] == null) && this._createEditTextBox(element);
        },

        _createEditTextBox: function (values) {
            var argsData = { currentElement: values };
            if (this._triggerEvent('beforeEdit', argsData))
                return false;
            var editTextBox = this.element.find('.e-item > div > .e-text > #Edit_Input');
            if (editTextBox[0] == null) {
                var size, textBox = ej.buildTag('Input.input-text#Edit_Input', "", "", { type: 'text', value: $.trim(values.text()).replace(/\n\s+/g, " "), name: 'inplace_value' });
                textBox.width(values.outerWidth() + 5);
                textBox.height(values.outerHeight() - 2);
                this._beforeEditText = values.text();
                values[0].lastChild.nodeValue = "";
                values.addClass('e-editable').append(textBox);
                editTextBox = textBox;
                size = (editTextBox.val().length == '') ? 3 : values.outerWidth() + 20;
                this._mousePositionAtEnd(editTextBox);
                this._currentEditableNode = values;
                this._on(editTextBox, 'keypress', this._editTextBoxKeyPress)
                    ._on(editTextBox, 'keydown', this, this._pressEscKey)
                    ._on(editTextBox, 'mousedown', this, this._preventPropagation)
                    ._on(editTextBox, 'blur', this._focusout);
            }
            return editTextBox;
        },

        _preventPropagation: function (e) {
            e.stopImmediatePropagation();
            this._off(this._anchors, "mouseup", this._focusElement);
        },

        _editTextBoxKeyPress: function (event) {
            event.target.size = event.target.value.length + 1;
        },

        _mousePositionAtEnd: function (ctl) {
            (ctl.focus) && ctl.focus();
            (ctl.select) && ctl.select();
            return true;
        },

        _focusElement: function () {
            this.element.focus();
        },

        _focusout: function (e) {
            var editTextBox = $(e.currentTarget), data, element;
            if (this.getSelectedNode().length == 0) {
                element = editTextBox.closest('.e-item');
                this._isRender = false;
                this._nodeSelectionAction(element);
                this._isRender = true;
            }
            data = { id: editTextBox.closest('.e-item').attr('id'), oldText: this._beforeEditText, newText: editTextBox.val() };
            (this._triggerEvent('inlineEditValidation', data)) ? this._cancelAction(editTextBox) : this._saveAction(editTextBox, e);
        },

        _pressEscKey: function (event) {
            event.cancelBubble = true;
            event.returnValue = false;
            var editTextBox = $(event.currentTarget);
            if (editTextBox[0] != null) {
                if (event.keyCode == 13) {
                    event.stopPropagation();
                    this._focusout(event);
                }
                if (event.keyCode == 27)
                    this._cancelAction(editTextBox);
                if (event.keyCode == 13 || event.keyCode == 27) {
                    var browserInfo = ej.browserInfo(), _isIE8;
                    _isIE8 = (browserInfo.name == 'msie' && browserInfo.version == '8.0') ? true : false;
                    (_isIE8) && this.element.focus();
                }
            }
        },

        _onFocusHandler: function (event) {
            event.preventDefault();
            if (this.model.allowKeyboardNavigation && this._isFocus) {
                if (this.getSelectedNode().length == 0) {
                    this._isRender = false;
                    this.selectNode(this.element.find('.e-item:first'));
                    this._isRender = true;
                    this._updateSelectedNode();
                }
                this._isFocus = false;
            }
        },

        _onKeyDown: function (currentEle, focusEle) {
            currentEle.find('> div > .e-text:first').removeClass('e-node-focus');
            focusEle.find('> div > .e-text:first').addClass('e-node-focus');
        },

        _KeyPress: function (e) {
            var code, node, element;
            var proxy = this;
            this._isFocus = true;
            element = (this.element.is("ul")) ? this.element : this.element.find("> ul");
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            if (proxy.model.allowKeyboardNavigation && (proxy.element.find('#Edit_Input').length < 1)) {
                var nextElement, selectedItem, liVisible, activeNode;
                selectedItem = element.find(".e-item > div > .e-text.e-active").closest('.e-item');
                liVisible = element.find('.e-item:visible:not(.hidden, .e-node-disable)');
                if (proxy._focusedNode) {
                    activeNode = proxy._focusedNode;
                    proxy._focusedNode = null;
                } else
                    activeNode = $(liVisible).find('.e-text.e-node-focus').closest('.e-item');
                if (code == 113) {
                    e.preventDefault();
                    this._isFocus = false;
                    var element = (activeNode.length > 0) ? activeNode : selectedItem;
                    if (element.length > 0 && proxy.model.allowEditing) {
                        e.target = element.find('> div > .e-text:first');
                        proxy._inlineEdit(e);
                    }
                }
                if (code == 40 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    e.preventDefault();
                    var nextEle = (activeNode.length > 0) ? $(liVisible[liVisible.index(activeNode) + 1]) : $(liVisible[liVisible.index(selectedItem) + 1]);
                    if (proxy._KeyPressEventHandler((nextEle.length > 0) ? nextEle : null, proxy, code, e)) return;
                    if (nextEle.length > 0)
                        proxy._onKeyDown(activeNode, nextEle);
                }
                else if (code == 38 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    e.preventDefault();
                    var prevEle = (activeNode.length > 0) ? $(liVisible[liVisible.index(activeNode) - 1]) : $(liVisible[liVisible.index(selectedItem) - 1]);
                    if (proxy._KeyPressEventHandler((prevEle.length > 0) ? prevEle : null, proxy, code, e)) return;
                    if (prevEle.length > 0)
                        proxy._onKeyDown(activeNode, prevEle);
                }
                else if ((code == 39 && !this.model.enableRTL && !e.ctrlKey && !e.altKey && !e.shiftKey) || (code == 37 && this.model.enableRTL)) {
                    e.preventDefault();
                    var nextEle, expandIcon;
                    expandIcon = (activeNode.length > 0) ? activeNode.find('> div > div').first() : selectedItem.find('> div > div').first();
                    nextEle = expandIcon.closest('.e-item');
                    if (expandIcon.hasClass('e-plus')) {
                        if (this.model.loadOnDemand && nextEle.find("> ul .e-item").length == 0) {
                            this._ClickEventHandler({ target: expandIcon[0] });
                        }
                        else this._expandNode($(liVisible[liVisible.index(nextEle)]));
                    } else {
                        nextEle = $(liVisible[liVisible.index(nextEle) + 1]);
                        if (proxy._KeyPressEventHandler((nextEle.length > 0) ? nextEle : "", proxy, code, e)) return;
                        if (nextEle.length > 0 && nextEle[0] == expandIcon.closest('.e-item').find('ul > .e-item:first')[0] || nextEle.find('ul > .e-item').first().hasClass('e-node-disable'))
                            proxy._onKeyDown(activeNode, nextEle);
                    }
                }
                else if ((code == 37 && !this.model.enableRTL && !e.ctrlKey && !e.altKey && !e.shiftKey) || (code == 39 && this.model.enableRTL)) {
                    e.preventDefault();
                    var prevEle, collapseIcon;
                    collapseIcon = (activeNode.length > 0) ? activeNode.find('> div > div').first() : selectedItem.find('> div > div').first();
                    prevEle = collapseIcon.closest('.e-item');
                    if (collapseIcon.hasClass('e-minus')) {
                        this._collpaseNode($(liVisible[liVisible.index(collapseIcon.closest('.e-item'))]));
                    } else {
                        prevEle = $(liVisible[liVisible.index(prevEle)]).closest('ul').closest('.e-item');
                        if (proxy._KeyPressEventHandler((prevEle.length > 0) ? prevEle : "", proxy, code, e)) return;
                        if (prevEle.length > 0) {
                            proxy._onKeyDown(activeNode, prevEle);
                        }
                    }
                }
                else if (code == 36 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    e.preventDefault();
                    var firstEle, ele = (activeNode.length > 0) ? activeNode : selectedItem;
                    firstEle = $(liVisible).first();
                    if ((selectedItem.length > 0 && activeNode.length == 0 && selectedItem[0] != firstEle[0]) || (activeNode.length > 0 && activeNode[0] != firstEle[0])) {
                        if (proxy._KeyPressEventHandler(firstEle, proxy, code, e)) return;
                        proxy._onKeyDown(ele, firstEle);
                    }
                }
                else if (code == 35 && !e.ctrlKey && !e.altKey && !e.shiftKey) {
                    e.preventDefault();
                    var lastEle, ele = (activeNode.length > 0) ? activeNode : selectedItem;
                    lastEle = $(liVisible).last();
                    if ((selectedItem.length > 0 && activeNode.length == 0 && selectedItem[0] != lastEle[0]) || (activeNode.length > 0 && activeNode[0] != lastEle[0])) {
                        if (proxy._KeyPressEventHandler(lastEle, proxy, code, e)) return;
                        proxy._onKeyDown(ele, lastEle);
                    }
                }
                else if (code == 13) {
                    e.preventDefault();
                    var currentEle = (activeNode.length > 0) ? activeNode : selectedItem;
                    currentEle.find('> div > .e-text').removeClass('e-node-focus');
                    this._nodeSelectionAction(currentEle);
                }
                else if (code == 32) {
                    e.preventDefault();
                    var currentEle = (activeNode.length > 0) ? activeNode : selectedItem;
                    if (currentEle.length > 0 && this.model.showCheckbox) {
                        var chkBoxEle = currentEle.find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0];
                        if (chkBoxEle.checked)
                            this._nodeUncheck(chkBoxEle);
                        else
                            this._nodeCheck(chkBoxEle);
                    }
                }
                else if (code == 46) {
                    e.preventDefault();
                    var currentEle = (activeNode.length > 0) ? activeNode : selectedItem;
                    (currentEle.length > 0) && this._removeNode(currentEle, e);
                }
                else if (e && e.ctrlKey == true) {
                    if (code == 88 && this.model.allowDragAndDrop && this.model.allowDropChild) {
                        e.preventDefault();
                        var currentEle = (activeNode.length > 0) ? activeNode : selectedItem;
                        if (currentEle.length > 0) {
                            var parent = currentEle.parents('.e-item:first');
                            this._cutNode = $(liVisible[liVisible.index(currentEle)]).detach();
                            if ($(this._cutNode)[0] != null) {
                                if (this._triggerEvent('beforeCut', { target: currentEle, nodeDetails: this._getNodeDetails(currentEle), keyCode: code, event: e })) return;
                                this.unselectNode(currentEle);
                                if (parent.find('> ul > .e-item').length == 0) {
                                    this.collapseNode(parent);
                                    this._modifyCss(parent);
                                }
                                this._triggerEvent('nodeCut', { parentElement: parent, parentDetails: this._getNodeDetails(parent), keyCode: code, event: e });
                            }
                        }
                    }
                    else if (code == 86 && this._cutNode != null && this.model.allowDragAndDrop && this.model.allowDropChild) {
                        e.preventDefault();
                        var currentEle = (activeNode.length > 0) ? activeNode : selectedItem;
                        currentEle = $(liVisible[liVisible.index(currentEle)]);
                        currentEle.length === 0 && (currentEle = this.element);
                        var element = this._cutNode.find(" > div > .e-text").first();
                        if ($(liVisible).length > 0) {
                            if (this._triggerEvent('beforePaste', { target: currentEle, nodeDetails: this._getNodeDetails(currentEle), keyCode: code, event: e })) return;
                            this._dropAsChildNode(currentEle, element, e);
                            this._cutNode = null;
                            var currentNode = element.closest('.e-item');
                            this._triggerEvent('nodePaste', { target: currentNode, nodeDetails: this._getNodeDetails(currentNode), keyCode: code, event: e });
                        }
                    }
                }
            }
        },

        _removeChildNodes: function (obj, groupedObj, id) {
            var len = 0, currentid;
            for (var i = 0; i < groupedObj.length; i++) {
                if (!ej.isNullOrUndefined(groupedObj[i][0][this.model.fields.parentId]) && (groupedObj[i][0][this.model.fields.parentId]).toString() == id) {
                    len = groupedObj[i].length; break;
                }
            }
            for (var i = 0; i < len; i++) {
                for (var j = 0; j < obj.length; j++) {
                    if (!ej.isNullOrUndefined(obj[j][this.model.fields.parentId]) && (obj[j][this.model.fields.parentId]).toString() == id) {
                        if (!ej.isNullOrUndefined(obj[j])) currentid = obj[j][this.model.fields.id].toString();
                        obj.splice(j, 1);
                        this._removeChildNodes(obj, groupedObj, currentid);
                        break;
                    }
                }
            }
        },

        _removeNode: function (node, event) {
            if (node[0] == null) return;
            if (node[0] != null && node.length > 0) {
                var parentNode, currentNode, _dataObj, id, liVisible, childNodes, proxy = this, groupedObj;
                parentNode = node.closest('ul').closest('.e-item');
                currentNode = node;
                if (this._triggerEvent('beforeDelete', { target: currentNode, nodeDetails: this._getNodeDetails(currentNode), parentElement: (parentNode[0] != null) ? parentNode : null, parentDetails: this._getNodeDetails(parentNode), event: event })) return;
                if (!ej.isNullOrUndefined(this.dataSource()) && this.dataSource().length > 0) {
                    _dataObj = this.dataSource();
                    for (var i = 0; i < _dataObj.length; i++) {
                        id = currentNode.attr("id");
                        if ((_dataObj[i][this.model.fields.id]).toString() == id) {
                            groupedObj = this._groupingObjects(_dataObj, function (_dataObj) { return [!ej.isNullOrUndefined(_dataObj) && [_dataObj[proxy.model.fields.parentId]].toString()]; });
                            _dataObj.splice(i, 1);
                            this._removeChildNodes(_dataObj, groupedObj, id);
                            break;
                        }
                    }
                }
                if ($(this._liList).index(currentNode) == this.model.selectedNode)
                    this.model.selectedNode = null;
                currentNode.remove();
                this._modifyCss(parentNode);
                this._updateNodes();
                this._updateCheckState(parentNode);
                if (this._triggerEvent('nodeDelete', { parentElement: (parentNode[0] != null) ? parentNode : null, parentDetails: this._getNodeDetails(parentNode), event: event })) return;
                var proxy = this, _dataObj = this.dataSource();
                setTimeout(function () {
                    if (proxy.dataSource() != null && !(proxy.dataSource() instanceof ej.DataManager))
                        proxy._dataSource = proxy._groupingObjects(_dataObj, function (_dataObj) { return [!ej.isNullOrUndefined(_dataObj) && [_dataObj[proxy.model.fields.parentId]].toString()]; });
                }, 300);
            }
        },

        _updateNodes: function () {
            this._beforeBaseClass();
            this._applyFirstLastChildClass();
            this._updateExpandedNodes();
            this._updateCheckedNodes();
            this._updateSelectedNode();
        },

        _updateText: function (obj, searchId, newText) {
            if (this._templateType == 1) {
                for (var i = 0, objlen = obj.length; i < objlen; i++) {
                    if (!ej.isNullOrUndefined(obj[i]) && !ej.isNullOrUndefined(obj[i][this.model.fields.id]) && obj[i][this.model.fields.id].toString() == searchId) {
                        obj[i][this.model.fields.text] = newText;
                        var newobj = obj[i];
                        obj.splice(i, 1, newobj);
                        break;
                    }
                }
            }
            else {
                for (var i = 0, objlen = obj.length; i < objlen; i++) {
                    if (obj[i][this.model.fields.id].toString() == searchId) {
                        obj[i][this.model.fields.text] = newText;
                        var newobj = obj[i];
                        obj.splice(i, 1, newobj);
                        break;
                    }
                    if (obj[i].hasOwnProperty('child'))
                        this._updateText(obj[i].child, searchId, newText);
                }
            }
        },

        _KeyPressEventHandler: function (nextElement, proxy, code, event) {
            var nodeDetails, data, path, isExpanded;
            if ($(nextElement)[0] != null) {
                isExpanded = this._isNodeExpanded(nextElement);
                nodeDetails = this._getNodeDetails(nextElement);
                path = proxy._getPath(nextElement);
                data = { keyCode: code, currentElement: nextElement, value: nodeDetails.text, isExpanded: isExpanded, path: path, event: event, id: nodeDetails.id, parentId: nodeDetails.parentId };
            } else {
                data = { keyCode: code, currentElement: nextElement, value: "", isExpanded: "", path: "", event: event, id: "", parentId: "" };
            }
            return this._triggerEvent('keyPress', data);
        },

        _documentClick: function (event) {
            if (event.target.id != 'Edit_Input')
                var editTextBox = $('#Edit_Input')[0], aTag, newText;
            if (editTextBox != null) {
                var aTag = $(editTextBox).closest('.e-text')[0], newText = editTextBox.value, parent = $(editTextBox).closest('.e-item');
                if (this.getSelectedNode().length == 0) {
                    this._isRender = false;
                    this._nodeSelectionAction(parent);
                    this._isRender = true;
                }
                $(editTextBox).remove();
                aTag.lastChild.nodeValue = newText;
                $(aTag).removeClass('e-editable').removeAttr('style');
                if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null && !(this.dataSource() instanceof ej.DataManager))
                    this._updateText(this._newDataSource, parent.attr('id'), newText);
                this._triggerEvent('nodeEdit', { id: parent.attr('id'), oldText: this._beforeEditText, newText: newText, target: parent, nodeDetails: this._getNodeDetails(parent), event: event });
            }
            if (this.element.find(event.target).length == 0) {
                var ele = this.element.find('.e-item > div > .e-text.e-node-focus');
                this._focusedNode = ele.closest('.e-item');
                ele.removeClass('e-node-focus');
            }
        },

        _saveAction: function (values, event) {
            var newText = values.val(), parent = values.closest('.e-item'), aTag = values.closest('.e-text')[0];
            values.remove();
            if (aTag != null) {
                aTag.lastChild.nodeValue = newText;
                $(aTag).removeClass('e-editable');
                if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null && !(this.dataSource() instanceof ej.DataManager))
                    this._updateText(this._newDataSource, parent.attr('id'), newText);
                this._triggerEvent('nodeEdit', { id: parent.attr('id'), oldText: this._beforeEditText, newText: newText, target: parent, nodeDetails: this._getNodeDetails(parent), event: event });
                this.element.focus();
            }
        },

        _cancelAction: function (values) {
            var aTag = values.closest('.e-text')[0];
            values.remove();
            aTag.lastChild.nodeValue = this._beforeEditText;
            $(aTag).removeClass('e-node-hover e-editable');
            if (this.getSelectedNode().length == 0) {
                this._isRender = false;
                this.selectNode($(aTag).closest('.e-item'));
                this._isRender = true;
            }
            this.element.focus();
        },

        _mouseEnterEvent: function (event) {
            this.element.find('.e-node-hover').removeClass("e-node-hover");
            if ($(event.currentTarget).hasClass("e-text") && !$(event.currentTarget).hasClass("e-node-disable"))
                $(event.currentTarget).addClass("e-node-hover");
        },

        _mouseLeaveEvent: function (event) {
            $(event.currentTarget).removeClass("e-node-hover");
        },

        _createObjectByText: function (text, targetNode) {
            if (typeof text != "string") return;
            var obj = {};
            obj[this.model.fields.id] = text;
            obj[this.model.fields.text] = text;
            ($(targetNode)[0] != null) && (obj[this.model.fields.parentId] = targetNode.attr('id'));
            return obj;
        },

        _addNodesWhenObject: function (obj, selectedNode) {
            var tempObj = JSON.stringify(this._dataSource), _dataObj = this.dataSource(), parentId, tempType;
            this._dataSource = [];
            if (this.dataSource() != null && this.dataSource() instanceof ej.DataManager)
                _dataObj = [];
            if (selectedNode[0] != null && selectedNode.length > 0)
                parentId = selectedNode.attr('id');
            if (ej.isNullOrUndefined(obj.length)) {
                this._dataSource.push(obj);
                if (!ej.isNullOrUndefined(this.dataSource())) {
                    (!ej.isNullOrUndefined(parentId)) && (obj[this.model.fields.parentId] = parentId);
                    _dataObj.push(obj);
                }
            }
            else {
                this._dataSource = obj;
                if (!ej.isNullOrUndefined(this.dataSource())) {
                    for (var j = 0; j < obj.length; j++) {
                        if (parentId)
                            obj[j][this.model.fields.parentId] = parentId;
                        else
                            delete obj[j][this.model.fields.parentId];
                        _dataObj.push(obj[j]);
                    }
                }
            }
            tempType = this._templateType;
            this._templateType = 2;
            if (!this._liList)
                this._liList = $("li", this.element);
            var fragmentObj = document.createDocumentFragment();
            for (var i = 0, len = this._dataSource.length ; i < len; i++) {
                fragmentObj.appendChild(this._genTemplate(this._dataSource[i], this.model.fields));
                this._liList.push($(fragmentObj).children()[0]);
            }
            this._templateType = tempType;
            if (this.dataSource() != null && !(this.dataSource() instanceof ej.DataManager))
                this._dataSource = JSON.parse(tempObj);
            return fragmentObj;
        },

        _addExpandedNodes: function (index) {
            var _nodes = this.model.expandedNodes;
            this._removeNullInArray(_nodes);
            if ($.inArray(index, _nodes) == -1)
                this.model.expandedNodes.push(index);
            this._persistValues(this.model.expandedNodes, "expandedNodes");
        },

        _removeExpandedNodes: function (index) {
            var _nodes = this.model.expandedNodes;
            if ($.inArray(index, _nodes) > -1) {
                this.model.expandedNodes.splice($.inArray(index, _nodes), 1);
                _nodes.length == 0 && (_nodes.push(-1));
            }
            this._persistValues(this.model.expandedNodes, "expandedNodes");
        },

        _persistValues: function (indexColl, valueType) {
            if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null) {
                var _nodes = indexColl, idColl = [], ele, id;
                for (var i = 0, len = _nodes.length; i < len; i++) {
                    if (!ej.isNullOrUndefined(_nodes[i]) && _nodes[i] >= 0) {
                        ele = this._liList[_nodes[i]], id = $(ele).attr('id');
                        (id) && idColl.push(id);
                    }
                }
                var cookieData = this._getCookies("_persistedValues");
                if (cookieData) {
                    var parsedData = JSON.parse(cookieData);
                    (valueType == "selectedNode") ? parsedData[valueType] = (idColl[0]) ? idColl[0] : "" : parsedData[valueType] = idColl;
                    this._setCookies("_persistedValues", JSON.stringify(parsedData));
                }
            }
        },

        _onChecked: function (element, args) {
            var liElement = element.closest('.e-item'), isChecked, evt, checknode, data;
            nodeDetails = this._getNodeDetails(liElement);
            this._CurrenctSelectedNodes.push(nodeDetails.text);
            isChecked = this._isChecked(liElement);
            evt = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.event) ? args.event : "" : "";
            checknode = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.checknode) ? args.checknode : "" : "";
            data = { currentElement: liElement, id: nodeDetails.id, parentId: nodeDetails.parentId, value: nodeDetails.text, currentNode: this._CurrenctSelectedNodes, currentCheckedNodes: checknode, isChecked: isChecked, event: evt };
            if (this._isRender) this._triggerEvent('nodeCheck', data);
        },

        _onUnChecked: function (element, args) {
            var liElement = element.closest('.e-item'), isUnChecked, evt, unchecknode, data;
            nodeDetails = this._getNodeDetails(liElement);
            this._CurrenctSelectedNodes.push(nodeDetails.text);
            isUnChecked = this._isChecked(liElement);
            evt = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.event) ? args.event : "" : "";
            unchecknode = !ej.isNullOrUndefined(args) ? !ej.isNullOrUndefined(args.unchecknode) ? args.unchecknode : "" : "";
            data = { currentElement: liElement, id: nodeDetails.id, parentId: nodeDetails.parentId, value: nodeDetails.text, currentNode: nodeDetails.text, currentUncheckedNodes: unchecknode, isChecked: isUnChecked, event: evt };
            this._triggerEvent('nodeUncheck', data);
        },

        _addCheckNodes: function (item) {
            var checkedArray = this.model.checkedNodes;
            this._removeNullInArray(checkedArray);
            !checkedArray instanceof Array && (checkedArray = []);
            if (checkedArray.indexOf(item) == -1) checkedArray.push(item);
        },

        _removeCheckNodes: function (item) {
            var checkedArray = this.model.checkedNodes;
            !checkedArray instanceof Array && (checkedArray = []);
            var i = checkedArray.indexOf(item);
            if (i != -1) {
                checkedArray.splice(i, 1);
                checkedArray.length == 0 && (checkedArray.push(-1));
            }
        },

        _removeNullInArray: function (array) {
            var i = array.indexOf(-1);
            if (i != -1) array.splice(i, 1);
        },

        _afterInsertingNode: function (outerLi) {
            this._addDragableClass();
            this._finalizeEditing(outerLi);
            var parentLi = outerLi.closest('ul').closest('.e-item');
            this._modifyCss(parentLi);
            this._updateCheckState(parentLi);
            this._isSelectedAction();
            if (parentLi.length > 0 && parentLi.hasClass('e-node-disable'))
                this._nodeDisableAction(parentLi);
            this._updateSelectedNode();
        },

        _insertBeforeOrAfter: function (txtobj, afterEle, before) {
            afterEle = this._getNodeByID(afterEle);
            if (afterEle[0] != null && afterEle.is('LI') && afterEle.hasClass('e-item')) {
                if (this._triggerEvent('beforeAdd', { data: txtobj, targetParent: (afterEle[0] != null) ? afterEle : null, parentDetails: this._getNodeDetails(afterEle) })) return;
                if (typeof txtobj != 'object')
                    txtobj = this._createObjectByText(txtobj);
                if (typeof txtobj != "object") return;
                (txtobj[this.model.parentId]) && delete txtobj[this.model.parentId];
                outerLi = this._addNodesWhenObject(txtobj, afterEle);
                temp = document.createElement('ul');
                $(temp).append(outerLi); outerLi = $(temp.children);
                this.model.showCheckbox && outerLi.children().find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
                (afterEle.parents('.e-item:first').length > 0) ? afterEle.parents('.e-item:first').append(outerLi) : this.element.append(outerLi);
                this.model.showCheckbox && this._isCheckedAction();
                this._dropAsSublingNode(afterEle, outerLi.find("> div > .e-text"), before, "");
                this._afterInsertingNode(outerLi);
                this._triggerEvent('nodeAdd', { data: txtobj, nodes: outerLi, parentElement: (afterEle[0] != null) ? afterEle : null, parentDetails: this._getNodeDetails(afterEle) });
                var proxy = this, _dataObj = this.dataSource();
                setTimeout(function () {
                    if (proxy.dataSource() != null && !(proxy.dataSource() instanceof ej.DataManager))
                        proxy._dataSource = proxy._groupingObjects(_dataObj, function (_dataObj) { return [!ej.isNullOrUndefined(_dataObj) && [_dataObj[proxy.model.fields.parentId]].toString()]; });
                }, 300);
            }
        },

        _getNodeByID: function (node) {
            (typeof node != "object" && node != "") && (node = this.element.find(".e-item#" + node));
            node = $(node), node = $(node[0]);
            (node.is("A") && node.hasClass('e-text')) && (node = node.closest('.e-item'));
            return node;
        },

        _isTreeElement: function (node) {
            return $(node)[0] != null && node.is('LI') && node.hasClass('e-item');
        },

        _isUrl: function (url) {
            var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
            return regexp.test(url);
        },

        _sendAjaxOptions: function (url, trgt) {
            var proxy = this, dataobj, ajaxOptions;
            dataobj = this._getNodeDetails(trgt);
            dataobj['mapper'] = this.model.fields;
            dataobj['model'] = this.model;
            ajaxOptions = {
                url: url, data: dataobj, type: 'POST', async: true, crossDomain: true,
                dataType: 'JSON', contentType: "application/json; charset=utf-8",
                "success": function (data) {
                    try {
                        proxy._ajaxSuccessHandler(data, trgt);
                    } catch (e) { }
                }, "error": function (data) {
                    try {
                        proxy._ajaxErrorHandler(data);
                    } catch (e) { }
                }
            };
            this._sendAjaxRequest(ajaxOptions);
        },

        _sendAjaxRequest: function (ajaxOptions) {
            var temp = JSON.parse(JSON.stringify(ajaxOptions));
            delete temp.success;     // To prevent user to edit ajax success and edit event handler
            delete temp.error;
            if (this._triggerEvent('beforeLoad', { ajaxOptions: temp })) return;
            delete temp.success;
            delete temp.error;
            $.extend(ajaxOptions, temp);  // The modified data will be restored in ajaxOptions
            $.ajax({
                type: ajaxOptions.type,
                url: ajaxOptions.url,
                dataType: ajaxOptions.dataType,
                data: ajaxOptions.data,
                async: ajaxOptions.async,
                contentType: ajaxOptions.contentType,
                crossDomain: ajaxOptions.crossDomain,
                success: ajaxOptions.success,
                error: ajaxOptions.error,
            });
        },

        _ajaxSuccessHandler: function (data, targetNode) {
            this._isRender = false;        // To prevent event triggering
            (typeof data == "object") && this.addNode(data, targetNode);
            this._isRender = true;
            this._triggerEvent('loadSuccess', { data: data, targetParent: targetNode, parentDetails: this._getNodeDetails(targetNode) });
        },

        _ajaxErrorHandler: function (args) {
            this._triggerEvent('loadError', { error: args });
        },

        _wireEvents: function () {
            this._on(this.element, "click", this._ClickEventHandler)
                ._on(this.element.find(".e-text"), "mouseenter", this._mouseEnterEvent)
                ._on(this.element.find(".e-text"), "mouseleave", this._mouseLeaveEvent)
                ._on(this.element, this.model.expandOn, this._expandEventHandler)
                ._on(this.element, "focus", this._onFocusHandler);
            this.model.allowKeyboardNavigation && this._on(this.element, "keydown", this._KeyPress);
        },

        _unWireEvents: function () {
            this._off(this.element, "click")
                ._off(this.element.find(".e-text"), "mouseenter")
                ._off(this.element.find(".e-text"), "mouseleave")
                ._off(this.element, this.model.expandOn)
                ._off(this.element, "focus");
            this._preventEditable()
            this._preventDraggable();
            this.model.allowKeyboardNavigation && this._off(this.element, 'keydown');
            this.model.showCheckbox && this.element.find('.nodecheckbox').ejCheckBox("disable");
        },

        _enableDragDrop: function () {
            if (this.model.allowDragAndDrop) {
                this._drag();
                this.model.allowDropChild && this._childDrop();
                this.model.allowDropSibling && this._siblingDrop();
            }
        },

        _allowEditable: function () {
            if (!this.model.template) {
                this.element.find('.e-item').addClass('AllowEdit');
                this._on($(document), 'click', this._documentClick)
                    ._on(this.element.find('.e-item > div > .e-text'), 'dblclick', this._inlineEdit);
            }
        },

        _preventEditable: function () {
            this.element.find('.e-item').removeClass('AllowEdit');
            this._off($(document), 'click')
                ._off(this.element.find('.e-item > div > .e-text'), 'dblclick');
        },

        _preventDraggable: function () {
            this.element.find('.e-draggable, .e-droppable').removeClass("e-draggable e-droppable");
            this._preventDropSibling();
            this._preventDropChild();
            this._off(this._anchors, "mouseup", this._focusElement);
        },

        _preventDropSibling: function () {
            this.element.find('.e-dropsibling').removeClass("e-dropsibling");
        },

        _preventDropChild: function () {
            this.element.find('.e-item > div > .e-dropchild').removeClass("e-dropchild");
        },

        refresh: function () {
            this._unWireEvents();
            this.element.html("");
            this._init();
        },

        expandAll: function () {
            this.model.enableMultipleExpand && this._expandAll();
        },

        collapseAll: function () {
            this.model.enableMultipleExpand && this._collapseAll();
        },

        checkAll: function () {
            this.model.showCheckbox && this._checkAll();
        },

        unCheckAll: function () {
            this.model.showCheckbox && this._uncheckAll();
        },

        selectNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && this._nodeSelectionAction(node);
        },

        unselectNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && this._nodeUnSelectionAction(node);
        },

        enableNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && this._nodeEnableAction(node);
        },

        disableNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && this._nodeDisableAction(node);
        },

        addNodes: function (collection, targetNode) {
            if (collection && collection.length > 0)
                this.addNode(collection, targetNode);
        },

        addNode: function (newNodeText, targetNode) {
            if (ej.isNullOrUndefined(newNodeText)) return;
            var outerLi = null, innerUl = null, temp, activeNode, element = this.element, id, selectedNode, template;
            selectedNode = targetNode ? this._getNodeByID(targetNode) : this.getSelectedNode();
            if (!element.is('UL'))
                element = this.element.children('ul:first');
            selectedNode = (this._isTreeElement(selectedNode)) ? selectedNode : [];
            if (typeof newNodeText == 'object') {
                if (ej.isNullOrUndefined(newNodeText.length) && !ej.isNullOrUndefined(newNodeText[this.model.fields.parentId]))
                    id = newNodeText[this.model.fields.parentId];
                else if (!ej.isNullOrUndefined(newNodeText.length) && newNodeText.length == 1)
                    id = newNodeText[0][this.model.fields.parentId];
                if (id) selectedNode = this._getNodeByID(id);
            }
            if (this._triggerEvent('beforeAdd', { data: newNodeText, targetParent: (selectedNode[0] != null) ? selectedNode : null, parentDetails: this._getNodeDetails(selectedNode) })) return;
            (selectedNode.length != 0 && !this.isDisabled(selectedNode)) && this._expandNode(selectedNode);
            if (typeof newNodeText != 'object')
                newNodeText = this._createObjectByText(newNodeText, selectedNode);
            if (typeof newNodeText != "object" || (ej.isNullOrUndefined(newNodeText.length) && newNodeText.length == 0)) return;
            outerLi = this._addNodesWhenObject(newNodeText, selectedNode);
            temp = document.createElement('ul');
            $(temp).append(outerLi); outerLi = $(temp.children);
            this.model.showCheckbox && outerLi.children().find(".nodecheckbox").ejCheckBox({ cssClass: this.model.cssClass, change: this._checkedChange });
            this._appendNode(selectedNode, outerLi);
            (selectedNode[0] != null && selectedNode.find('> div > .e-minus').length > 0 && !this.model.enableMultipleExpand) && this._denyMultipleExpand(selectedNode);
            this._updateNodes();
            this._afterInsertingNode(outerLi);
            this._triggerEvent('nodeAdd', { data: newNodeText, nodes: outerLi, parentElement: (selectedNode[0] != null) ? selectedNode : null, parentDetails: this._getNodeDetails(selectedNode) });
            var proxy = this, _dataObj = this.dataSource();
            setTimeout(function () {
                if (proxy.dataSource() != null && !(proxy.dataSource() instanceof ej.DataManager))
                    proxy._dataSource = proxy._groupingObjects(_dataObj, function (_dataObj) { return [!ej.isNullOrUndefined(_dataObj) && [_dataObj[proxy.model.fields.parentId]].toString()]; });
            }, 300);
        },

        removeNode: function (node) {
            node = node ? this._getNodeByID(node) : this.getSelectedNode();
            this._isTreeElement(node) && this._removeNode(node);
        },

        removeAll: function () {
            this._liList.remove();
            this._updateNodes();
        },

        checkNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && this._nodeCheck(node.find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0]);
        },

        uncheckNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && this._nodeUncheck(node.find("> div > .e-chkbox-wrap > .nodecheckbox:first")[0]);
        },

        expandNode: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node)) {
                this._allowOnDemand = true;
                this._expandNode(node);
                this._allowOnDemand = false;
            }
        },

        collapseNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && this._collpaseNode(node);
        },

        showNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && node.css("visibility", "").removeClass('hidden');
        },

        hideNode: function (node) {
            node = this._getNodeByID(node);
            this._isTreeElement(node) && node.css("visibility", "hidden").addClass('hidden');
        },

        show: function () {
            this.element.css("visibility", "").find('.e-item').removeClass('hidden');
        },

        hide: function () {
            this.element.css("visibility", "hidden").find('.e-item').addClass('hidden');
        },

        hasChildNode: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node))
                return node.find('> ul > .e-item').length > 0 ? true : false;
        },

        isChildLoaded: function (node) {
            node = this._getNodeByID(node);
            return (this._isTreeElement(node) && node.find('ul > .e-item').length > 0) ? true : false;
        },

        isNodeChecked: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node))
                return this._isChecked(node);
        },

        isExpanded: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node))
                return this._isNodeExpanded(node);
        },

        isVisible: function (node) {
            node = this._getNodeByID(node);
            return (this._isTreeElement(node) && node.css("visibility") != "hidden") ? true : false;
        },

        isExist: function (node) {
            node = this._getNodeByID(node);
            return (this._isTreeElement(node) && this._liList.index(node) != -1) ? true : false;
        },

        isSelected: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node)) return node.find('> div > .e-text').hasClass('e-active');
        },

        isDisabled: function (node) {
            node = this._getNodeByID(node);
            return (this._isTreeElement(node) && node.hasClass('e-node-disable')) ? true : false;
        },

        getTreeData: function () {
            if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null && !(this.dataSource() instanceof ej.DataManager))
                return this._orderDataSource();
        },

        getText: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node))
                return $.trim(this.element.find(node).find('> div > .e-text').text());
        },

        getSelectedNode: function () {
            return this.element.find('.e-item > div > .e-active').closest('.e-item');
        },

        getCheckedNodes: function () {
            if (this.model.showCheckbox)
                return this._liList.find('> div > .e-chkbox-wrap[aria-checked="true"]').closest('.e-item');
        },

        getExpandedNodes: function () {
            return this.element.find('.e-item > div > .e-minus').closest('.e-item');
        },

        getExpandedNodesIndex: function () {
            var enodes = this.getExpandedNodes(), eindex = [];
            for (var i = 0, len = enodes.length; i < len; i++)
                eindex.push(this._liList.index(enodes[i]));
            return eindex;
        },

        getCheckedNodesIndex: function () {
            var cnodes = this.getCheckedNodes(), cindex = [];
            for (var i = 0, len = cnodes.length; i < len; i++)
                cindex.push(this._liList.index(cnodes[i]));
            return cindex;
        },

        getSelectedNodeIndex: function () {
            return this._liList.index(this.getSelectedNode());
        },

        getVisibleNodes: function () {
            return this.element.find('.e-item:visible:not(.hidden, .e-node-disable)');
        },

        getNodeCount: function () {
            return this.element.find('.e-item').length;
        },

        getNode: function (node) {
            node = this._getNodeByID(node);
            return (this._isTreeElement(node)) ? this._getNodeDetails(node) : null;
        },

        getNodeIndex: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node))
                return this._liList.index(node);
        },

        getNodeByIndex: function (index) {
            if (typeof index == "number")
                return $(this._liList[index]);
        },

        getParent: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node))
                return node.parents('.e-item:first');
        },

        updateText: function (node, newText) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node) && !ej.isNullOrUndefined(newText)) {
                var aTag = this.element.find(node).children('div').find('> .e-text')[0], oldText;
                if (aTag != null)
                    oldText = aTag.lastChild.nodeValue;
                aTag.lastChild.nodeValue = newText;
                if (!ej.isNullOrUndefined(this.model.fields) && this.dataSource() != null && !(this.dataSource() instanceof ej.DataManager))
                    this._updateText(this._newDataSource, node.attr('id'), newText);
                this._triggerEvent('nodeEdit', { id: node.attr('id'), oldText: oldText, newText: newText, target: node, nodeDetails: this._getNodeDetails(node), event: null });
            }
        },

        insertAfter: function (nodeObj, afterEle) {
            this._insertBeforeOrAfter(nodeObj, afterEle, false);
        },

        insertBefore: function (nodeObj, afterEle) {
            this._insertBeforeOrAfter(nodeObj, afterEle, true);
        },

        moveNode: function (srcNode, desNode, index) {
            srcNode = this._getNodeByID(srcNode);
            desNode = this._getNodeByID(desNode);
            if (this._isTreeElement(srcNode)) {
                if ((srcNode.parents('.e-item')[0] == desNode[0] && ej.isNullOrUndefined(index)) || desNode.find(srcNode).index() == index || (desNode[0] == null && this._liList.index(srcNode) == index)) return;
                var target = (desNode[0] != null && index >= 0) ? desNode.find("> ul > .e-item").eq(index) : this.getNodeByIndex(index);
                if (this._isTreeElement(target)) {
                    if (target.parents('.e-item:first')[0] == srcNode.parents('.e-item:first')[0] && target.next('.e-item')[0] == null)
                        this._dropAsSublingNode(target, srcNode.find(" > div > .e-text"), false, "");
                    else
                        this._dropAsSublingNode(target, srcNode.find(" > div > .e-text"), true, "");
                } else {
                    this._dropAsChildNode(desNode, srcNode.find(" > div > .e-text"), "");
                }
                if (desNode.length > 0 && desNode.hasClass('e-node-disable'))
                    this._nodeDisableAction(desNode);
            }
        },

        loadData: function (url, targetNode) {
            targetNode = this._getNodeByID(targetNode);
            if (this._isUrl(url) && (targetNode[0] == null || (this._isTreeElement(targetNode))))
                this._sendAjaxOptions(url, targetNode);
        },

        ensureVisible: function (node) {
            node = this._getNodeByID(node);
            if (this._isTreeElement(node) && !this.isDisabled(node) && this.isVisible(node)) {
                var parents = node.parents('.e-item'), offset;
                for (var j = 0, len = parents.length; j < len; j++)
                    this._expandNode($(parents[j]));
                node.find('> div > .e-text').addClass('e-node-hover');
                offset = node.offset();
                node.animate({ scrollTop: offset.top }, this.model.enableAnimation ? 350 : 0, 'linear', function () {
                    node[0].scrollIntoView(false);
                });
                return true;
            } else
                return false;
        },

        _triggerEvent: function (e, data) {
            if (this._isRender) return this._trigger(e, data);
        },

    });
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html div elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejUploadbox", "ej.Uploadbox", {

        element: null,

        model: null,
        validTags: ["div", "span"],
        _setFirst: false,

        _rootCSS: "e-uploadbox",

        defaults: {

            buttonText: {

                browse: "Browse",

                upload: "Upload",

                cancel: "Cancel",

                close: "Close",
            },

            width: "100px",

            height: "35px",

            htmlAttributes: {},

            dialogPosition: { X: "", Y: "" },

            dialogText: {

                title: "Upload Box",

                name: "Name",

                size: "Size",

                status: "Status"
            },

            customFileDetails: {

                title: true,

                name: true,

                size: true,

                status: true,

                action: true
            },

            dialogAction: {

                modal: false,

                closeOnComplete: false,

                drag: true,

                content: null

            },

            locale: "en-US",

            asyncUpload: true,

            pushFile: null,

            enabled: true,

            multipleFilesSelection: true,

            autoUpload: false,

            showFileDetails: true,

            fileSize: 31457280,

            extensionsAllow: "",

            extensionsDeny: "",

            saveUrl: "",

            removeUrl: "",

            uploadName: "",

            cssClass: "",

            dropAreaText: "Drop files or click to upload ",

            allowDragAndDrop: false,

            showBrowseButton: true,

            showRoundedCorner: true,

            dropAreaHeight: "100%",

            dropAreaWidth: "100%",

            enableRTL: false,

            create: null,

            fileSelect: null,

            begin: null,

            cancel: null,

            inProgress: null,
            
            success: null,

            complete: null,

            remove: null,

            error: null,

            destroy: null
        },


        dataTypes: {
            buttonText: "data",
            dialogText: "data",
            disbled: "boolean",
            customFileDetails: "data",
            pushFile: "data",
            dialogAction: "data",
            multipleFilesSelection: "boolean",
            autoUpload: "boolean",
            showFileDetails: "boolean",
            allowDragAndDrop: "boolean",
            showBrowseButton: "boolean",
            showRoundedCorner: "boolean",
            fileSize: "number",
            extensionsAllow: "string",
            extensionsDeny: "string",
            saveUrl: "string",
            removeUrl: "string",
            cssClass: "string",
            enableRTL: "boolean",
            htmlAttributes: "data"
        },



        disable: function () {
            this._changeState(false);
        },

        enable: function () {
            this._changeState(true);
        },
		
        _changeState: function (state) {
            state ? $(this.control).removeClass("e-disable") : $(this.control).addClass("e-disable");
            this.model.enabled = state;
            $(this._currentElement).find(".e-uploadinput")[0].disabled = !state;
            if (this.diaObj) {
                this.diaObj.option("beforeClose", this._onBeforeClose);
                this.diaObj.setModel({ enabled: state, allowDraggable: state });
            }
        },

        refresh: function () {
            if (this.diaObj) {
                if (this.diaObj.isOpened())
                    this.diaObj.close();
                else
                    this._uploadFileListDelete();
            }
        },


        _init: function () {
            this.s = ej.browserInfo();
            this._initialize();
            this._wireEvents();
            /*Sync Uploads*/
            if (!this.model.asyncUpload) {
                this._initObjectsSyncUpload();
            }
            this._controlStatus(this.model.enabled);
            this.model.locale == "en-US" ? this._buttonText(this.model.buttonText) : this._setLocale();
            this._roundedCorner(this.model.showRoundedCorner);
        },

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "buttonText":
                        this._buttonText(options[option]);
                        break;
                    case "htmlAttributes":
                        this._addAttr(options[option]);
                        break;
                    case "dialogText":
                        this._dialogText(options[option]);
                        break;
                    case "cssClass":
                        this._setSkin(options[option]);
                        break;
                    case "enableRTL":
                        this._setRTL(options[option]);
                        break;
                    case "enabled":
                        this._controlStatus(options[option]);
                        break;
                    case "locale":
                        this.model.locale = options[option];
                        this._setLocale();
                        break;
                    case "height":
                        this.model.height = options[option];
                        this._setHeight(this.model.height);
                        break;
                    case "width":
                        this.model.width = options[option];
                        this._setWidth(this.model.width);
                        break;
                    case "dialogPosition":
                        this.model.dialogPosition = options[option];
                        this._dialogPosition();
                        break;
                    case "allowDragAndDrop":
                        this.model.allowDragAndDrop = options[option];
                        this._dragAndDrop();
                        break;
                    case "dropAreaText":
                        this.model.dropAreaText = options[option];
						this._dropAreaText(this.model.dropAreaText);
                        break;
                    case "showBrowseButton":
                        this.model.showBrowseButton = options[option];
                        this._hideBrowseButton();
						this._refreshUploadDialogParent();
                        break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "dropAreaHeight":
                        this.model.dropAreaHeight = options[option];
                        this._setSize();
                        break;
                    case "dropAreaWidth":
                        this.model.dropAreaWidth = options[option];
                        this._setSize();
                        break;
                    case "pushFile":
                        this.model.pushFile = options[option];
                        if (this.model.pushFile != null && this.UploadType == "Xhr" && this.model.asyncUpload) this._files = this.model.pushFile; this._onXhrSelect();
                        break;
                    case "multipleFilesSelection":
                        this.model.multipleFilesSelection = options[option];
                        if (this.model.multipleFilesSelection)
                            this._currentElement.find(".e-uploadinput").attr('multiple', 'multiple');
                        else
                            this._currentElement.find(".e-uploadinput").removeAttr('multiple');
                        break;
                    case "uploadName":
                        this.model.uploadName = options[option];
                        this.inputupload.attr('name', this.model.uploadName);
                        this.refresh();
                        break;
                    case "autoUpload":
                        this.model.autoUpload = options[option];
                        break;
                    case "showFileDetails":
                        this.model.showFileDetails = options[option];
                        this.refresh();
                        break;
                    case "fileSize":
                        this.model.fileSize = options[option];
                        break;
                    case "extensionsAllow":
                        this.model.extensionsAllow = options[option];
						this._currentElement.find(".e-uploadinput").attr('accept', options[option]);
                        break;
                    case "extensionsDeny":
                        this.model.extensionsDeny = options[option];
                        break;
                    case "saveUrl":
                        this.model.saveUrl = options[option];
                        break;
                    case "removeUrl":
                        this.model.removeUrl = options[option];
                        break;
                }
                this._currentElement = this.model.allowDragAndDrop && !this.model.showBrowseButton ? this.dragWrapper : this.element;
            }
        },

        _controlStatus: function (value) {
            value != true ? this.disable() : this.enable();
        },

        _setRTL: function (val) {
            val ? this._currentElement.addClass("e-rtl") : this._currentElement.removeClass("e-rtl");
            if (this.updialog) this.updialog.ejDialog({ enableRTL: val });
        },
        _getLocalizedLabels: function (property) {
            var textType;
            if (property == "browse" || property == "upload" || property == "cancel" || property == 'close')
                textType = "buttonText";
            else
                textType = "dialogText";
            return (ej.Uploadbox.Locale[this.model.locale] === undefined || ej.Uploadbox.Locale[this.model.locale][property] === undefined) ?
            ej.Uploadbox.Locale[this.model.locale][textType][property] :
            ej.Uploadbox.Locale[this.model.locale][property];
        },

        _setLocale: function () {
            this._buttonText(ej.Uploadbox.Locale[this.model.locale].buttonText);
            this._dialogText(ej.Uploadbox.Locale[this.model.locale].dialogText);
			this._dropAreaText(ej.Uploadbox.Locale[this.model.locale].dropAreaText);
        },

        _buttonText: function (data) {
            $.extend(this.model.buttonText, data);
            this.buttondiv.val(this.model.buttonText.browse);
            if (this.updialog) {
                this.updialog.find(".e-action-container .e-uploadbtn").html(this.model.buttonText.upload);
                this.updialog.find(".e-action-container .e-uploadclosebtn").html(this.model.buttonText.cancel);
            }
        },

        _dialogText: function (data) {
            $.extend(this.model.dialogText, data);
            if (!(this.diaObj == undefined))
                this.diaObj.option('title', this.model.dialogText.title);
            if (this.updialog) {
                this.updialog.find('.e-head-name').html(this.model.dialogText.name);
                this.updialog.find('.e-head-size').html(this.model.dialogText.size);
                this.updialog.find('.e-head-status').html(this.model.dialogText.status);
            }
        },


        _destroy: function () {
            if (this._currentElement.hasClass("e-uploadbox")) {
                this._currentElement.removeClass("e-uploadbox e-widget");
                this._currentElement.empty();
                $(this.dragWrapper).after(this._currentElement);
                this._bindResizeHandler(false);
                this.dragWrapper.remove();
            }
        },

        _setSkin: function (skin) {
            this._currentElement.removeClass(this.model.cssClass);
            this._currentElement.addClass(skin);
			if (this.diaObj)
                this.diaObj.setModel({ cssClass:this.model.cssClass });
        },


        _initialize: function () {
            this.control = this.element[0];
            this.element.addClass("e-widget " + this.model.cssClass);
            this.innerdiv = ej.buildTag('div.e-selectpart e-select e-box');
            this.element.append(this.innerdiv);
            this.buttondiv = ej.buildTag('input.e-inputbtn e-btn#' + this.control.id + '_SelectButton', '', {}, { type: 'button', "data-role": "none", value: this._getLocalizedLabels("browse") });
            this.inputupload = ej.buildTag('input.e-uploadinput', "", {}, { type: 'file', "data-role": "none", name: this.model.uploadName != "" ? this.model.uploadName : this.control.id }).attr("accept",this.model.extensionsAllow);
            if (this.model.multipleFilesSelection) {
                this.inputupload.attr('multiple', 'multiple');
            }
            this.innerdiv.append(this.buttondiv);
            this.innerdiv.append(this.inputupload);
            if (this.model.allowDragAndDrop) this._dragAndDrop(true);
            this._Selector = this.buttondiv[0];
            this._currentElement = this.model.allowDragAndDrop && !this.model.showBrowseButton ? this.dragWrapper : this.element;
            this._setRTL(this.model.enableRTL);
            this._addAttr(this.model.htmlAttributes);
            this.Uploadframes = []; //For IFrame
            this.UploadForms = [];
            this._successFiles = [];
            this.UploadType = this._isXhrSupported() ? "Xhr" : "IFrame";
            if (this.model.width != "") this._setWidth(this.model.width);
            if (this.model.height != "") this._setHeight(this.model.height);
            this._hideBrowseButton();
        },		
        _refreshUploadDialogParent: function () {
            if (this.diaObj)
                this.diaObj.setModel({ target: (this.model.dialogAction.content != null && this.model.dialogAction.content != "") ? this.model.dialogAction.content : (!this.model.showBrowseButton) && (this.model.allowDragAndDrop) ? "#" + this.control.id + '_dragWrapper' : "#" + this.control.id });
        },
        _hideBrowseButton: function () {
            if ((!this.model.showBrowseButton) && (this.model.allowDragAndDrop)) {
                this.element.addClass("e-browse-hide");
                this._dropAreaSize();
            }
            else {
                if (this.element.hasClass("e-browse-hide")) {
                    this.element.removeClass("e-browse-hide");
                    this.element.width(this.model.width);
                    this.element.height(this.model.height);
                }
            }
        },
        _setSize: function () {
            if (this.model.allowDragAndDrop) {
                if (this.model.dropAreaWidth != "") this.dragWrapper.width(this.model.dropAreaWidth);
                if (this.model.dropAreaHeight != "") this.dragWrapper.height(this.model.dropAreaHeight);
                if (!this.model.showBrowseButton) this._dropAreaSize();
            }
        },

        _dropAreaSize: function () {
            if (this.model.dropAreaWidth != "") this.element.width(this.model.dropAreaWidth);
            if (this.model.dropAreaHeight != "") this.element.height(this.model.dropAreaHeight);
        },

        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.element.addClass(value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else proxy.element.attr(key, value)
            });
        },

        _dragAndDrop: function () {
            if (this.model.allowDragAndDrop) {
                this.dragWrapper = ej.buildTag("div.e-drag-wrapper e-widget-wrapper e-upload-box " + this.model.cssClass + "#" + this.control.id + '_dragWrapper')
                this.innerWrapper = ej.buildTag("div.e-drag-container");
                this._spanTag = ej.buildTag("span.e-drag-text").html(this.model.locale == "en-US" ? this.model.dropAreaText : ej.Uploadbox.Locale[this.model.locale].dropAreaText);
                this.innerWrapper.append(this._spanTag);
                this.innerWrapper.insertBefore(this.element);
                this.dragWrapper.insertBefore(this.element);
                this.innerWrapper.append(this.element);
                this.dragWrapper.append(this.innerWrapper);
                this._bindDragEvents();
                this._refreshUploadDialogParent();
                this._setSize();
            }
            else {
                this._refreshUploadDialogParent();
                this.element.insertBefore(this.dragWrapper);
                this.dragWrapper.remove();
                this._unBindDragEvents();
            }
            this._hideBrowseButton();
        },
        _resizeHandler: function () {
            if (this.diaObj) {
                if ($(window).width() > 750) {
                    $(this.updialog).closest('.e-dialog.e-uploadbox').removeClass("e-mobile-upload");
                    this.diaObj.setModel({ width: this._getDialogContentWidth(), height: "auto" });
                }
                else {
                    $(this.updialog).closest('.e-dialog.e-uploadbox').addClass("e-mobile-upload");
                    this.diaObj.setModel({ width: 250, height: "auto"});
                }
                if (this.diaObj.scroller) this.diaObj.scroller.setModel({ height: "auto" });
            }
        },
        _getDialogContentWidth: function () {
            var dialogWidth = 0;
            if (this.model.customFileDetails.name)   dialogWidth = dialogWidth + 200;
            if (this.model.customFileDetails.size)  dialogWidth = dialogWidth + 100;
            if (this.model.customFileDetails.status) dialogWidth = dialogWidth + 200; 
            else if (this.model.customFileDetails.action) dialogWidth = dialogWidth + 45;
            return dialogWidth+5;
        },
		_dropAreaText: function (areaText) {
            if (this.model.allowDragAndDrop)
                this._spanTag.html(areaText);
        },
        _bindDragEvents: function () {
            this._on(this.dragWrapper, "dragover", this._onDragOverHandler);
            this._on(this.dragWrapper, "drop", this._onDropHandler);
        },
        _unBindDragEvents: function () {
            this._off(this.dragWrapper, "dragover", this._onDragOverHandler);
            this._off(this.dragWrapper, "drop", this._onDropHandler);
        },
        _setWidth: function (value) {
            this.element.css("width", value);
        },
        _setHeight: function (value) {
            this.element.height(value);
        },
        _dialogPosition: function () {
            if (this.diaObj) {
                if ((this.diaObj.model.content == "#" + this.control.id || this.diaObj.model.content == "#" + this.control.id + "_dragWrapper") && this.model.dialogPosition.X == "" && this.model.dialogPosition.Y == "") {
                    var left = parseInt(this.diaObj.wrapper.css("left")) - ((this.diaObj.wrapper.outerWidth() - $(this.diaObj.model.content).outerWidth()) / 2);
                    this.model.dialogPosition.X = parseInt(left) > 0 ? left : parseInt(this.diaObj.wrapper.css("left"));
                    this.model.dialogPosition.Y = parseInt(this.diaObj.wrapper.css("top")) + $(this.diaObj.model.content).outerHeight();
                }
                if (this.model.dialogPosition.X != "" && this.model.dialogPosition.Y != "") this.diaObj.setModel({ position: this.model.dialogPosition });
            }
        },

        _roundedCorner: function (value) {
            if (value) {
                this.innerdiv.addClass('e-corner');
            }
            else if (this.innerdiv.hasClass('e-corner')) {
                this.innerdiv.removeClass('e-corner');
            }
        },


        _wireEvents: function () {

            this._on(this.element, "click", this._disableclickselect);
            this._on(this.element, "keydown", this._keydownselect);
            this._bindInputChangeEvent();
            this._bindResizeHandler(true);
        },
        _bindResizeHandler: function (responsive) {
            responsive ? $(window).bind('resize', $.proxy(this._resizeHandler, this)) : $(window).unbind('resize', $.proxy(this._resizeHandler, this));
        },

        _keydownselect: function (e) {
             if (!this.element.hasClass("e-disable") && e.keyCode == 13 && (!this.diaObj || !this.diaObj.isOpened()))
               this.element.find('.e-uploadinput').click();
        },

        _onDropHandler: function (e) {
            if (ej.browserInfo().name === "msie" && ej.browserInfo().version === "8.0" || ej.browserInfo().version === "9.0") return false;
            if (this._currentElement.hasClass("e-disable")) return false;
            e.stopPropagation();
            e.preventDefault();
            this._files = this._getAllFileInfo(e.originalEvent.dataTransfer.files);
            this._fileUpload(e);
        },


        _onDragOverHandler: function (e) {
            if (this._currentElement.hasClass("e-disable")) return false;
            e.stopPropagation();
            e.preventDefault();
        },


        _disableclickselect: function (e) {
            if (this._currentElement.hasClass("e-disable")) {
                e.preventDefault();
            }
        },


        _bindInputChangeEvent: function () {
            this._on(this.inputupload, "change", this._inputValueChange);
        },


        _inputValueChange: function (e) {
            this._files = this._getInputFileInfo($(e.target));
            if (this._trigger("fileSelect", {files:this._files})) return false;
            this._fileUpload(e);
        },
        _fileUpload: function (e) {
            var filteredFiles = this._isAllowed(this._files);
            this._files = filteredFiles.files;
            if (!this.model.asyncUpload) {
                if (filteredFiles.status)
                    this._onSelectSyncUpload(e); //Sync Upload
                } else {
                    if (filteredFiles.files != null && filteredFiles.files.length > 0) {
                        if (this.UploadType == "Xhr") 
                            this._onXhrSelect(e); //removed uploadcore // For XHR Upload
                        else 
                           this._onSelectIFrame(e); //For Iframe
                        this._resetFileInput(this._currentElement.find(".e-uploadinput"));
                    }
                }
        },


        __uploadButtonClick: function (e) {
            if (!this._currentElement.hasClass("e-disable")) {
                var uploadbtn = this._currentElement.find(".e-uploadbtn");
                if (this._currentElement.find(".e-uploadbtn").hasClass("e-disable")) e.preventDefault();
                else {
                    if (this.UploadType == "Xhr") {
                        this._xhrOnUploadButtonClick(e); // Xhr Upload
                    } else {
                        this._onUploadButtonClickIFrame(e); //Iframe Upload
                    }
                }
                $(e.target).attr('disabled', 'disabled').addClass('e-disable');
            }
        },

        _actionClick: function (e) {
            if (!this._currentElement.hasClass("e-disable")) {
                var targetAction, fileItem, file;
                targetAction = $(e.target);
                fileItem = targetAction.closest(".e-upload-file");
                this._file = $(fileItem).data("file");
                if (targetAction.hasClass("e-file-delete")) {

                    if (this.UploadType == "Xhr") {
                        this._xhrOnRemove(e, fileItem); //XHr Remove
                    } else {
                        this._onRemoveIFrame(e, fileItem); //IFrame File Upload
                    }
                } else if (targetAction.hasClass("e-file-cancel")) {
                    this._trigger("cancel", { fileStatus: this._file });
                    var fileclose = this.updialog.find(".e-file-cancel");
                    if (fileclose.hasClass("e-disable")) e.preventDefault();
                    else {
                        if (!this.model.asyncUpload) {
                            this._onCancelSyncUpload(e, fileItem); //sync
                        } else {
                            if (this.UploadType == "Xhr") {
                                this._xhrOnCancel(e, fileItem); //Xhr Cancel Item
                            } else {
                                this._onCancelIFrame(e, fileItem); //IFrame Cancel Item
                            }
                        }
                    }
                } else if (targetAction.hasClass("e-file-retry")) {//removed uploadcore
                    if (this.UploadType == "Xhr") {
                        this._xhrOnRetry(e, fileItem); //xhr remove
                    } else {
                        this._onRetryIFrame(e, fileItem); //IFrame Remove
                    }
                }
            }
        },

        _removeFileEntry: function (file) {
            file.remove();
        },


        _isFileUpload: function (fileEntry) {
            if (this.model.customFileDetails.action) {
                var actiondiv = $(fileEntry).find("div.e-icon");
                return actiondiv.is(".e-file-cancel");
            }
            else return true;
        },


        _isXhrSupported: function () {
            return (((this.s.name == "msie" && parseInt(this.s.version) < 9) || ((this.s.name == "safari" && this.s.name == "chrome") && this.s.version == "536")) ? false : (typeof (FormData) != "undefined") && (typeof (File) != "undefined"));
        },


        _getFileName: function (input) {
            return $.map(this._getAllFileInfo(input), function (file) {
                return file.name;
            }).join(", ");
        },

        _getFileSize: function (input) {
            var tempProxy = this;
            return $.map(this._getAllFileInfo(input), function (file) {
                return tempProxy._formatSize(file.size);
            }).join(", ");
        },

        _pushFile: function (files, datapart) {
            var fileListDetails, addedFile, actionlist, i, action, diaObj, addedheading, addedfilesize, addname, addsize, addstatus;
            addedheading = $("<div class='e-head-content'></div>");
            addname = $("<div class='e-file-head e-head-name'>" + this._getLocalizedLabels("name") + "</div>");
            addsize = $("<div class='e-file-head e-head-size'>" + this._getLocalizedLabels("size") + "</div>");
            addstatus = $("<div class='e-file-head e-head-status'>" + this._getLocalizedLabels("status") + "</div>");
            if (this.model.customFileDetails.name) $(addname).appendTo(addedheading);
            if (this.model.customFileDetails.size) $(addsize).appendTo(addedheading);
            if (this.model.customFileDetails.status) $(addstatus).appendTo(addedheading);
            filedialog = this.updialog;
            if (filedialog && filedialog.length != 0) {
                if (this.model.showFileDetails) {
                    if (this.updialog) this.updialog.find(".e-uploadbtn").removeAttr('disabled').removeClass('e-disable');
                    this.diaObj.open();
                }
            }
            else {
                this.updialog = ej.buildTag('div.uploaddialog#' + this.element[0].id + '_dialog', "", {}, { 'title': this._getLocalizedLabels("title") });
                if (this.model.allowDragAndDrop && !this.model.showBrowseButton) {
                    this.dragWrapper.append(this.updialog);
                    DialogContentContainer = "#" + this.control.id + '_dragWrapper';
                }
                else {
                    this.element.append(this.updialog);
                    DialogContentContainer = "#" + this.control.id;
                }
            }
            fileListDetails = this.updialog.find(".e-ul");
            if ($(this.updialog.find(".e-head-content")))
                $(this.updialog.find(".e-head-content")).replaceWith(addedheading);
            if (fileListDetails.length == 0) {
                addedheading.appendTo(this.updialog);
                fileListDetails = ej.buildTag('ul.e-ul').appendTo(this.updialog);
			}
			if($(this.updialog).find('.e-file-upload').length>0)
				$(this.updialog).find('.e-file-upload').remove();
			fileListActions = ej.buildTag('div.e-file-upload').appendTo(this.updialog).append((this.model.autoUpload || !this.model.asyncUpload) ? $(ej.buildTag('div.e-action-container')).addClass("sync") : ej.buildTag('div.e-action-container'));
            dialogActions = ej.buildTag('button.e-uploadclosebtn e-btn e-select', this._getLocalizedLabels("cancel"), {}, { type: 'button', "data-role": "none" }).appendTo($(this.updialog).find(".e-action-container"));
            if (this.model.showRoundedCorner)
                dialogActions.addClass('e-corner');
            this._on(dialogActions, "click", this._dialogclose);
            this._on(dialogActions, 'keydown', this._keydownDialogClose);            
            var dialogContainer = (this.model.dialogAction.content != null && this.model.dialogAction.content != "") ? this.model.dialogAction.content : DialogContentContainer
            this.updialog.ejDialog({ showOnInit: false, closeIconTooltip: this._getLocalizedLabels("closeToolTip"), minWidth: 240, width: ($(window).width() < 750) ? 250 : this._getDialogContentWidth(), height: "auto", cssClass: "e-uploadbox " + this.model.cssClass, close: $.proxy(this._uploadFileListDelete, this), enableRTL: this.model.enableRTL, target: dialogContainer, enableResize: false, allowDraggable: this.model.dialogAction.drag, enableModal: this.model.asyncUpload ? this.model.dialogAction.modal : false, showHeader: this.model.customFileDetails.title });
            $(window).width() < 750 ? $(this.updialog).closest('.e-dialog.e-uploadbox').addClass("e-mobile-upload") : $(this.updialog).closest('.e-dialog.e-uploadbox').removeClass("e-mobile-upload");
			this.diaObj = this.updialog.data('ejDialog');
			if(this.model.cssClass!="")
				this.diaObj.setModel({ cssClass:this.model.cssClass });
            this._dialogPosition();
            if (!this.model.multipleFilesSelection) {
                this.updialog.find(".e-ul>.e-upload-file").remove();
            }
            for (i = 0; i < files.length; i++) {//localization can be given for not started
                addedFile = $("<li class='e-upload-file'></li>").appendTo(fileListDetails).data(datapart);
                addedlist = $("<div class='e-file-list'></div>");
                addedfilename = $("<div class='e-file-progress e-file-view'><div class='e-file-name e-file-view'><span class='e-file-name-txt'>" + files[i].name + "</span></div>");
                addedfilesize = "<div class='e-file-size e-file-view'><span class='e-file-name-txt'>" + this._formatSize(files[i].size) + "</span></div>";
                addedfilestatus = $("<div class='e-file-percentage e-file-view'><div class='e-file-progress-bar'><div class='e-file-progress-status'></div></div></div>");
                addedfileaction = $("<div class='e-action-perform'><div class='e-icon e-file-view'></div></div>")
                if (this.model.customFileDetails.name) $(addedfilename).appendTo(addedlist);
                if (this.model.customFileDetails.size) {
                    if (this._getFileSize(files[i]).toString().toLowerCase().match("0.0kb").length == 0 || files[i].size != null) {
                        $(addedfilesize).appendTo(addedlist);
                    }
                    else $(this.updialog).find(".e-head-size").remove();
                }
                if (this.model.customFileDetails.status) $(addedfilestatus).appendTo(addedlist);
                if (this.model.customFileDetails.action) {
                    $(addedfileaction).appendTo(addedlist);
                    action = "cancel";
                    addedlist.find(".e-icon").remove().addClass(action.toString());
                    if (action == "cancel") {
                        actionlist = ej.buildTag('div.e-icon e-file-cancel', '', {}, { title: this._getLocalizedLabels("cancelToolTip") });
                    } else if (action == "remove") {
                        actionlist = ej.buildTag('div.e-icon e-file-delete', '', {}, { title: this._getLocalizedLabels("removeToolTip") });
                    } else if (action == "retry") {
                        actionlist = ej.buildTag('div.e-icon e-file-retry', '', {}, { title: this._getLocalizedLabels("retryToolTip") });
                    }

                    addedlist.find(".e-action-perform").append(actionlist);

                    this._on(actionlist, "click", this._actionClick);
                }
                $(addedlist).appendTo(addedFile);
            }
            if (this.model.showFileDetails) {
                this.diaObj.open();
            }
            this._buttonText(this.model.buttonText);
            this._dialogText(this.model.dialogText);
            return addedFile;
        },

        _keydownDialogClose: function (e) {
            if (!this._currentElement.hasClass("e-disable")) {
                if (e.keyCode == 13) this._dialogclose();
            }
        },

        _pushFileDetails: function (files) {
            var fileListDetails, addedFile, actionlist, i, action, me, diaObj, addedlist, addedheading, addname, addsize, addstatus, addaction, addedfilename, addedfilestatus, addedfilesize, addedfileaction;
            addedheading = $("<div class='e-head-content'></div>");
            addname = $("<div class='e-file-head e-head-name'>" + this._getLocalizedLabels("name") + "</div>");
            addsize = $("<div class='e-file-head e-head-size'>" + this._getLocalizedLabels("size") + "</div>");
            addstatus = $("</div><div class='e-file-head e-head-status'>" + this._getLocalizedLabels("status") + "</div>");
            if (this.model.customFileDetails.name) $(addname).appendTo(addedheading);
            if (this.model.customFileDetails.size) $(addsize).appendTo(addedheading);
            if (this.model.customFileDetails.status) $(addstatus).appendTo(addedheading);
            me = this;
            filedialog = this.updialog;
            if (filedialog && filedialog.length != 0) {
                if (this.model.showFileDetails) {
                    if (this.updialog) this.updialog.find(".e-uploadbtn").removeAttr('disabled').removeClass('e-disable');
                    me.diaObj.open();
                }
            } else {
                this.updialog = ej.buildTag('div.uploaddialog#' + this.element[0].id + '_dialog', "", {}, { 'title': this._getLocalizedLabels("title") });
                if (this.model.allowDragAndDrop && !this.model.showBrowseButton) {
                    this.dragWrapper.append(this.updialog);
                    DialogContentContainer = "#" + this.control.id + '_dragWrapper';
                }
                else {
                    this.element.append(this.updialog);
                    DialogContentContainer = "#" + this.control.id;
                }
            }
            var fileListDetails, addedFile;
            fileListDetails = this.updialog.find(".e-ul");
            if ($(this.updialog.find(".e-head-content")))
                $(this.updialog.find(".e-head-content")).replaceWith(addedheading);
            if (fileListDetails.length == 0) {
                addedheading.appendTo(this.updialog);
                fileListDetails = ej.buildTag('ul.e-ul').appendTo(this.updialog);
			}
			if($(this.updialog).find('.e-file-upload').length>0)
				$(this.updialog).find('.e-file-upload').remove();
			fileListActions = ej.buildTag('div.e-file-upload').appendTo(this.updialog).append((this.model.autoUpload || !this.model.asyncUpload) ? $(ej.buildTag('div.e-action-container')).addClass("sync") : ej.buildTag('div.e-action-container'));
            dialogActions = ej.buildTag('button.e-uploadclosebtn e-btn e-select', this._getLocalizedLabels("cancel"), {}, { type: 'button', "data-role": "none" }).appendTo($(this.updialog).find(".e-action-container"));
            if (this.model.showRoundedCorner)
            dialogActions.addClass('e-corner');
            this._on(dialogActions, "click", this._dialogclose);            
            if (!this.model.multipleFilesSelection) {
                this.updialog.find(".e-ul>.e-upload-file").remove();
            }
            addedFile = $("<li class='e-upload-file'></li>").appendTo(fileListDetails).data("file", files);
            addedlist = $("<div class='e-file-list'></div>");
            addedfilename = $("<div class='e-file-progress e-file-view'><div class='e-file-name e-file-view'><span class='e-file-name-txt'>" + files.name + "</span></div>");
            addedfilesize = $("<div class='e-file-size e-file-view'><span class='e-file-name-txt'>" + this._formatSize(0) + "\\" + this._formatSize(files.size) + "</span></div>");
            addedfilestatus = $("<div class='e-file-percentage e-file-view'><div class='e-file-progress-bar'><div class='e-file-progress-status'></div></div></div>");
            addedfileaction = $("<div class='e-action-perform'><div class='e-icon e-file-view'></div></div></div>")
            if (this.model.customFileDetails.name) $(addedfilename).appendTo(addedlist);
            if (this.model.customFileDetails.size) $(addedfilesize).appendTo(addedlist);
            if (this.model.customFileDetails.status) $(addedfilestatus).appendTo(addedlist);
            if (this.model.customFileDetails.action) $(addedfileaction).appendTo(addedlist);
            $(addedlist).appendTo(addedFile);
            var dialogContainer = (this.model.dialogAction.content != null && this.model.dialogAction.content != "") ? this.model.dialogAction.content : DialogContentContainer;
            this.updialog.ejDialog({ showOnInit: false, closeIconTooltip: this._getLocalizedLabels("closeToolTip"), minWidth: 240, width: ($(window).width() < 750) ? 250 : this._getDialogContentWidth(), height: "auto", cssClass: "e-uploadbox " + this.model.cssClass, close: $.proxy(this._uploadFileListDelete, this), enableRTL: this.model.enableRTL, target: dialogContainer, enableResize: false, allowDraggable: this.model.dialogAction.drag, enableModal: this.model.asyncUpload ? this.model.dialogAction.modal : false, showHeader: this.model.customFileDetails.title })
			$(window).width() < 750 ? $(this.updialog).closest('.e-dialog.e-uploadbox').addClass("e-mobile-upload") : $(this.updialog).closest('.e-dialog.e-uploadbox').removeClass("e-mobile-upload");
			me.diaObj = this.updialog.data("ejDialog");
			if (me.model.cssClass!="")
                me.diaObj.setModel({ cssClass:this.model.cssClass });
            me._dialogPosition();
            if (this.model.showFileDetails) {
                me.diaObj.open();
            }
            this._buttonText(this.model.buttonText);
            this._dialogText(this.model.dialogText);
            return addedFile;
        },

        _setProgress: function (filelist, percentage, e) {
            var progressbar, progress, filesize, loaded, total;
            if (this.model.customFileDetails.status) {
                progressbar = $(filelist).find(".e-file-progress-status");
                progressbar.width(percentage + "%");
            }
            if (this.model.customFileDetails.size) {
                filesize = $(filelist).find(".e-file-size .e-file-name-txt");
                loaded = this._formatSize(e.loaded);
                total = this._formatSize(e.total);
                filesize.html(loaded + "\\" + total);
            }
        },

        _setAction: function (element, action) {
            if (this.model.customFileDetails.action) {
                var actionlist;
                element.find(".e-icon").remove().addClass(action.toString());
                if (action == "cancel") {
                    actionlist = ej.buildTag('div.e-icon e-file-cancel', '', {}, { title: this._getLocalizedLabels("cancelToolTip") });
                } else if (action == "remove") {
                    actionlist = ej.buildTag('div.e-icon e-file-delete', '', {}, { title: this._getLocalizedLabels("removeToolTip") });
                } else if (action == "retry") {
                    actionlist = ej.buildTag('div.e-icon e-file-retry', '', {}, { title: this._getLocalizedLabels("retryToolTip") });
                }
                element.find(".e-action-perform").append(actionlist);
                this._on(actionlist, "click", this._actionClick);
            }
        },

        _setStatus: function (element, status) {
            if (this.model.customFileDetails.status) {
                var progress, upstatus = ej.buildTag('div');
                if (status == "success") {
                    element.find(".file-status").addClass("e-file-status-success").html("Completed");
                    element.find(".e-file-percentage").html("").attr("title", this._getLocalizedLabels("completedToolTip"));
                    upstatus.addClass("e-icon e-file-percentage-success");
                    element.find(".e-file-percentage").append(upstatus);
                }
                if (status == "failed") {
                    element.find(".file-status").addClass("e-file-status-failed").html("Failed");
                    element.find(".e-file-percentage").html("").attr("title", this._getLocalizedLabels("failedToolTip"));
                    upstatus.addClass("e-icon e-file-percentage-failed");
                    element.find(".e-file-percentage").append(upstatus);
                }
                if (status == "progress") {
                    element.find(".file-status").addClass("file-status-inprogress").html("in progress");
                }
                if (status == "uploading") {
                    element.find(".file-status").addClass("file-status-inprogress").html("uploading");
                    progress = element.find(".e-file-percentage");
                    progress.html("");
                }
            }
        },

        _createInputandBind: function () {
            var tempInput = ej.buildTag('input', '', {}, { type: 'file', "data-role": "none" });
            tempInput.attr("name", this.model.uploadName != "" ? this.model.uploadName : this.control.id).attr("autocomplete", "off").attr("class", "e-uploadinput").attr("accept",this.model.extensionsAllow);
            if (this.model.multipleFilesSelection) {
                tempInput.attr("multiple", "multiple");
            }
            tempInput.appendTo(this.element.find(".e-selectpart"));
            this.inputupload = tempInput;
            this._bindInputChangeEvent();
        },

        _showUploadButton: function () {
            var uploadbutton = this.updialog.find(".e-uploadbtn");
            if (uploadbutton.length == 0) {
                uploadbutton = ej.buildTag('button.e-uploadbtn e-btn e-select', this._getLocalizedLabels("upload"), {}, { type: "button", "data-role": "none" });
                if (this.model.showRoundedCorner)
                    uploadbutton.addClass('e-corner');
                this.updialog.find(".e-action-container").append(uploadbutton);
                this._on(uploadbutton, "click", this.__uploadButtonClick);
                this._on(uploadbutton, 'keydown', this._keydownUpload);
                $(uploadbutton).focus();
            }
            this._buttonText(this.model.buttonText);
        },

        _keydownUpload: function (e) {
            if (e.keyCode == 13) this.__uploadButtonClick(e);
        },

        _resetFileInput: function ($element) {
            var clone = $element.clone(false, false);
            this._on(clone, "change", this._inputValueChange);
            $element.replaceWith(clone);
        },

        _isAllowed: function (files) {
            var inputfield, uploadControl, args, allowExtension, allowSize, denyExtension, proxy = this, allowedFiles = [], filteredFiles = [],status=true;
            inputfield = this.element.find(".e-uploadinput");
            uploadControl = this;
            if (this.model.extensionsAllow != "") {
                allowExtension = this.model.extensionsAllow.replace(/\s/g, "").toLowerCase().split(",");
                $(files).each(function () {
                    if ($.inArray((this.extension).toLowerCase(), allowExtension) == -1) {

                        args = { action: "Files Processing", error:proxy._getLocalizedLabels("allowError").replace("#Extension", allowExtension), files: this };
                        uploadControl._trigger('error', args);
						proxy._resetFileInput(inputfield);
                        status = false;
                    }
                    else {
                        if (this.extension != "")
                            allowedFiles.push(this);
                    }
                });
            }

            else if (this.model.extensionsDeny != "") {
                denyExtension = this.model.extensionsDeny.replace(/\s/g, "").toLowerCase().split(",");
                $(files).each(function () {
                    if ($.inArray((this.extension).toLowerCase(), denyExtension) != -1) {

                        args = { action: "Files Processing", error: proxy._getLocalizedLabels("denyError").replace("#Extension", denyExtension), files: this };
                        uploadControl._trigger('error', args);
						proxy._resetFileInput(inputfield);
                        status = false;
                    }
                    else {
                        if (this.extension != "")
                            allowedFiles.push(this);
                    }
                });
            }
            else {
                $(files).each(function () {
                    if (this.extension != "")
                        allowedFiles.push(this);
                });
            }
            if (this.model.fileSize != "") {
                allowSize = this.model.fileSize;
                $(allowedFiles).each(function () {
                    if ((this.size > allowSize)) {

                        args = { action: "Files Processing", error: proxy._getLocalizedLabels("filedetail"), files: this };
                        uploadControl._trigger('error', args);
						proxy._resetFileInput(inputfield);
                        status = false;
                    }
                    else {
                        filteredFiles.push(this);
                    }
                });
            }
            else
                filteredFiles = allowedFiles;
            return { files: filteredFiles, status: status };
        },

        _fileListRemove: function () {
            fileList = this.updialog.find(".e-upload-file .e-file-delete");
            if (fileList.length == 0) {
                this.updialog.find(".e-uploadbtn").attr('disabled', 'disabled').addClass('e-disable');
                this.updialog.ejDialog('close');
            }
        },

        _uploadHide: function () {
            fileList = this.updialog.find(".e-upload-file .e-file-cancel");
            if (fileList.length == 0) {
                this.updialog.find(".e-ul").empty();
                this.updialog.find(".e-uploadbtn").attr('disabled', 'disabled').addClass('e-disable');
                this.updialog.ejDialog('close');
            }
        },

        _onBeforeClose: function (args) {
            if (!ej.isNullOrUndefined(args.event)) if ($(args.event.currentTarget).hasClass("e-disable")) args.cancel = true;
            else args.cancel = false;
        },

        _dialogclose: function (e) {
            if (!this._currentElement.hasClass("e-disable")) {
                var closebtn = this.updialog.find(".e-uploadclosebtn");
                var file, fileItem;
                fileItem = this.updialog.find(".e-ul").children().first();
                file = $(fileItem).data("file");
                if (closebtn.hasClass("e-disable")) e.preventDefault();
                else {
                    this._uploadFileListDelete();
                    this.updialog.ejDialog('close');
                }
                this._trigger("cancel", { fileStatus: file });
            }
        },

        _uploadFileListDelete: function () {
            this.updialog.find(".e-ul").empty();
            this._resetFileInput(this._currentElement.find(".e-uploadinput"));
            if (!this.model.asyncUpload)
                this._formResetSyncUpload();
        },

        _onXhrSelect: function (e) {
            var files, xhrUpload, addedFile;
            this._xhrBeforeUpload(this._files);
            xhrUpload = this;
            $.each(xhrUpload._files, function (i, fileItem) {
                addedFile = $(fileItem).data("filelist");
                xhrUpload._setAction(addedFile, "cancel"); // XhrUpload._Uploader._setAction(addedFile, "cancel");
                if (xhrUpload.model.autoUpload) {
                    xhrUpload._xhrPerformUpload(fileItem);
                } else {
                    xhrUpload._showUploadButton();
                }
            });
        },

        _xhrBeforeUpload: function (files) {
            var fileEntry, xhrUpload, formdata, addedFile;
            fileEntry = files;
            xhrUpload = this;
            $.each(fileEntry, function (i, fileItem) {
                formdata = xhrUpload._createFormObjectXhr(fileItem);
                $(fileItem).data("formobject", formdata);
                addedFile = xhrUpload._pushFileDetails(fileItem);
                $(fileItem).data("filelist", addedFile);
            });

            return fileEntry;
        },

        _xhrPerformUpload: function (fileItem) {
            var isPrevented, url, xhrUpload, formdata, xhr, args;
            xhr = new XMLHttpRequest();
            xhrUpload = this;
            args = { files: fileItem };
            if (this._trigger('begin', args)) {
                $(fileItem).data("xhr", xhr);
                xhrUpload._onRequestError(xhrUpload, undefined, fileItem);
                return;
            }
            url = this.model.saveUrl;
            formdata = $(fileItem).data("formobject");

            formdata.append(this.control.id + "_data", JSON.stringify(args.data));
            $(fileItem).data("xhr", xhr);
            xhr.addEventListener("load", function (e) {
                xhrUpload._onRequestSuccess(xhrUpload, e, fileItem);
            }, false);
            xhr.addEventListener("error", function (e) {
                xhrUpload._onRequestError(xhrUpload, e, fileItem);
            }, false);
            xhr.upload.addEventListener("progress", function (e) {
                xhrUpload._onRequestProgress(xhrUpload, e, fileItem);
            }, false);
            xhr.open("POST", url);
            xhr.send(formdata);
        },

        _xhrOnUploadButtonClick: function (e) {
            var xhrUpload, fileEntry, started, fileCount;
            xhrUpload = this.model.showBrowseButton ? this.diaObj.wrapper : this.dragWrapper;
            fileCount = xhrUpload.find(".e-ul li.e-upload-file");
            for (i = 0; i < fileCount.length; i++) {
                fileEntry = $(this);
                started = this._isFileUpload(fileCount[i]);
                if (started) {
                    this._xhrPerformUpload($(fileCount[i]).data("file"));
                }
            }
        },

        _xhrOnRemove: function (e, fileItem) {
            var filename = $(fileItem).find(".e-file-name").text().toString().split(","), proxy = this;
            $.ajax({
                url: this.model.removeUrl,
                type: "POST",
                data: "fileNames=" + filename,
                success: function () {
                    $(fileItem).remove();
                    proxy._fileListRemove();
                    proxy._trigger("remove", { fileStatus: proxy._file });

                }
            });
        },

        _xhrOnCancel: function (e, fileItem) {
            var file, xhr;
            file = $(fileItem).data("file");
            xhr = $(file).data("xhr");
            if (xhr) {
                $(file).data("xhr").abort();
            }
            $(file).data("xhr", null);
            $(fileItem).data("file", null);
            $(fileItem).remove();
            this._uploadHide();
        },

        _xhrOnRetry: function (e, fileItem) {
            if (!this._currentElement.hasClass("e-disable")) {
                var file = $(fileItem).data("file");
                this._xhrPerformUpload(file);
            }
        },

        _onRequestSuccess: function (xhrUpload, e, fileEntry) {
            var xhr = $(fileEntry).data("xhr");
            if (xhr.status >= 200 && xhr.status <= 299) {
                xhrUpload._onXhrUploadSuccess(xhrUpload, e, fileEntry);
            }
            else {
                xhrUpload._onRequestError(xhrUpload, e, fileEntry);
            }
        },

        _onXhrUploadSuccess: function (xhrUpload, e, fileEntry) {
            var addedFile, xhr, progressbar, size, fSize, filesize, args;
            addedFile = $(fileEntry).data("filelist");
            xhr = $(fileEntry).data("xhr");
            if (xhrUpload.model.removeUrl) {
                xhrUpload._setAction(addedFile, "remove");
                xhrUpload._setStatus(addedFile, "success");
            } else {
                addedFile.find(".e-icon").remove();
                xhrUpload._setStatus(addedFile, "success");
            }
            if ($(fileEntry).length > 0) {
                progressbar = $(addedFile).find(".e-file-progress-status");
                progressbar.width("100%");
                size = $(fileEntry)[0].size;
                fSize = this._formatSize(size);
                filesize = $(addedFile).find(".e-file-size .e-file-name-txt");
                filesize.html(fSize + "\\" + fSize);
            }
            args = { files: fileEntry,responseText: xhr.responseText, xhr: xhr, e: e };
            this.updialog.find(".e-file-upload .e-uploadclosebtn").html(this.model.buttonText.close).focus();
            xhrUpload._trigger('success', args);
            this._successFiles.push(fileEntry);
            if (xhrUpload._files.length == this._successFiles.length) {
                args = { files: fileEntry, responseText: xhr.responseText,success: this._successFiles, xhr: xhr, e: e };
                xhrUpload._trigger('complete', args);
                this._successFiles.length = 0;
				//Will close the dialog box once upload has complete.
                if (this.model.dialogAction.closeOnComplete) this._dialogclose();
            }
        },

        _onRequestError: function (xhrUpload, e, fileEntry) {
            var addedFile, xhr, args;
            addedFile = $(fileEntry).data("filelist");
            xhr = $(fileEntry).data("xhr");
            xhrUpload._setAction(addedFile, "retry");
            xhrUpload._setStatus(addedFile, "failed");
            args = { action: "File Processing", error: xhr.status + xhr.statusText, files: fileEntry, xhr: xhr, e: e };
            xhrUpload._trigger('error', args);
        },

        _onRequestProgress: function (xhrUpload, e, fileEntry) {
            var percentage, addedFile,args;
            percentage = Math.round(e.loaded * 100 / e.total);
            addedFile = $(fileEntry).data("filelist");
            xhrUpload._setProgress(addedFile, percentage, e);
            xhrUpload._setStatus(addedFile, "progress");
            args = { file: fileEntry, percentage: percentage, e: e };
            this._trigger('inProgress', args);
        },

        _createFormObjectXhr: function (file) {
            var formData = new FormData();
            formData.append(this.model.uploadName != "" ? this.model.uploadName : this.control.id, file.rawFile);
            return formData;
        },

        _getInputFileInfo: function ($input) {
            var input = $input[0];
            if (input.files) {
                return this._getAllFileInfo(input.files);
            } else {
                return [{
                    name: this._GetName(input.value),
                    extension: this._getFileExtensionType(input.value),
                    size: this._getFileSizeinIE(input.value)
                }];
            }
        },

        _getFileSizeinIE: function (fileName) {
            var actievXObj, fileSize;
            actievXObj = null;
            fileSize = null;
            try {
                actievXObj = new ActiveXObject("Scripting.FileSystemObject");
            } catch (e) {
                fileSize = null;
            }
            if (actievXObj) {
                fileSize = actievXObj.getFile(fileName).size;
            }
            return fileSize;
        },

        _getFileExtensionType: function (fileName) {
            return fileName.match ? (fileName.match(/\.([^\.]+)$/) ? fileName.match(/\.([^\.]+)$/)[0] : "" || "") : "";
        },

        _getAllFileInfo: function (rawFiles) {
            var tempProxy = this;
            return $.map(rawFiles, function (file) {
                return tempProxy._getFileInfo(file || rawFiles);
            });
        },

        _GetName: function (fullname) {
            var nameIndex = fullname.lastIndexOf("\\");
            return (nameIndex != -1) ? fullname.substr(nameIndex + 1) : fullname;
        },

        _getFileInfo: function (rawFile) {
            // Older Firefox versions (before 3.6) use fileName and fileSize
            var fileName = rawFile.name || rawFile.fileName || rawFile;
            return {
                name: fileName,
                extension: this._getFileExtensionType(fileName),
                size: rawFile.size || rawFile.fileSize,
                rawFile: rawFile
            };
        },

        _formatSize: function (bytes) {
            var i = -1;
            if (!bytes)
                return "0.0KB";
            do {
                bytes = bytes / 1024;
                i++;
            } while (bytes > 99);
            return Math.max(bytes, 0).toFixed(1) + ['KB', 'MB', 'GB', 'TB', 'PB', 'EB'][i];
        },

        _onSelectIFrame: function (e) {
            var input, files, addedFile, uploadframe;
            input = $(e.target);
            files = this._getInputFileInfo(input);
            addedFile = this._beforeUploadIFrame(files);
            uploadframe = addedFile.data("iframe");
            if (this.model.autoUpload) {
                this._performUploadIFrame(addedFile);
            } else {
                this._showUploadButton();
            }
            this._off(this.inputupload, "change");
            this._bindInputChangeEvent();
        },

        _onRemoveIFrame: function (e, fileItem) {
            var iframe, fileNames, proxy, fileDetails;
            iframe = fileItem.data("iframe");
            fileDetails = $(fileItem).data("file");
            fileNames = fileDetails[0].name;
            proxy = this;
            if (iframe) {
                this._removeFileEntry(fileItem);
                if (this.model.removeUrl) {
                    $.ajax({
                        url: this.model.removeUrl,
                        type: "POST",
                        data: "fileNames=" + fileNames,
                        success: function () {
                            proxy._fileListRemove();
                            proxy._trigger("remove", { fileStatus: proxy._file });
                        }
                    });
                }
            } else {
                this._removeFileEntry(fileItem);
                this._trigger("remove", { fileStatus: this._file });
            }
        },

        _onCancelIFrame: function (e, fileItem) {
            var iframe;
            this._trigger('cancel', { Status: fileItem });
            iframe = fileItem.data("iframe");
            if (iframe) {
                this._removeFileEntry(fileItem);
                if (typeof (iframe.stop) != "undefined") {
                    iframe.stop();
                } else if (iframe.document) {
                    iframe.document.execCommand("Stop");
                    iframe.contentWindow.location.href = iframe.contentWindow.location.href;
                }
                this._processServerResponse(iframe, "");
            }
            this._uploadHide();
        },

        _onRetryIFrame: function (e, fileItem) {
            this._performUploadIFrame(fileItem);
        },

        _beforeUploadIFrame: function (files) {
            var uploadframe, uploadform, addedfile;
            //creating iframe and adding it to the upload div block.
            uploadframe = this._createFrame(this.control.id + "_Iframe" + this.Uploadframes.length);
            this.Uploadframes.push(uploadframe);
            uploadform = this._createForm(this.model.saveUrl, uploadframe[0].id);
            this._currentElement.find("input.e-uploadinput").removeClass("e-uploadinput").css("display", "none").appendTo(uploadform);
            this._createInputandBind();
            addedfile = this._pushFile(files, { "iframe": uploadframe, "form": uploadform, "file": files });
            uploadframe.data({ "filelist": addedfile });
            this._setAction(addedfile, "cancel");
            return addedfile;
        },

        _performUploadIFrame: function (addedFile) {
            var isPrevented, files, uploadframe, uploadform, args;
            args = { files: addedFile };
            if (this._trigger('begin', args)) {
                this._failureIframeUpload(addedFile, "File upload has been denied");
                return;
            }
            files = addedFile.data("file");
            this._setStatus(addedFile, "uploading");
            uploadframe = addedFile.data("iframe");
            uploadform = addedFile.data("form");
            var hiddendata = $("<input>").attr("name", this.control.id + "_data").attr('type', 'hidden').val(JSON.stringify(args.data));
            uploadform.append(hiddendata);
            uploadframe.appendTo(document.body);
            uploadform.appendTo(document.body);
            //error here calls the upload even if the url is wrong
            this._on(uploadframe, "load", this._removeFramesIFrame);
            uploadform.submit();
        },

        _onUploadButtonClickIFrame: function (e) {
            var iframeUpload, fileEntry, started;
            iframeUpload = this;
            $(".e-ul li.e-upload-file", iframeUpload.updialog).each(function () {
                fileEntry = $(this);
                started = iframeUpload._isFileUpload(fileEntry);
                if (started) {
                    iframeUpload._performUploadIFrame(fileEntry);
                }
            });
        },

        _removeFramesIFrame: function (e) {
            var uploadframe, response, filelist, fileEntry, args;
            uploadframe = $(e.target);
            filelist = uploadframe.data("filelist");
            try {
                response = $.trim(uploadframe.context.contentDocument.body.innerText.replace(/\n|\r/g, ' '));
            } catch (e) {
                response = "Server Error trying to get server response: " + e;
            }
            if (response.substring(0, 12) != "Server Error" && response.indexOf("HTTP Error") != 0) {
                this._processServerResponse(uploadframe, response);
                this._setIframeProgress(filelist, 100, e);
                this._setStatus(filelist, "progress");
                this._successIframeUpload(filelist, response);
            }
            else { this._failureIframeUpload(filelist, response); }
        },


        _setIframeProgress: function (filelist, percentage, e) {
            var progressbar, progress, filesize, loaded, total;
            progressbar = $(filelist).find(".e-file-progress-status");
            progressbar.width(percentage + "%");
        },

        _successIframeUpload: function (filelist, response) {
            fileEntry = filelist.data("file");
            if (this.model.removeUrl) {
                this._setAction(filelist, "remove");
                this._setStatus(filelist, "success");
            } else {
                filelist.find(".file-action").remove();
                this._setStatus(filelist, "success");
            }
            args = { files: fileEntry, responseText: response };
            this.updialog.find(".e-file-upload .e-uploadclosebtn").html(this.model.buttonText.close).focus();
            this._trigger('success', args);
            this._successFiles.push(fileEntry);
            if (filelist.length == this._successFiles.length) {
                args = { files: fileEntry, responseText: response, success: this._successFiles };
                this._trigger('complete', args);
                this._successFiles.length = 0;
				//Will close the dialog box once upload has complete.
                if (this.model.dialogAction.closeOnComplete) this._dialogclose();
            }
        },

        _failureIframeUpload: function (filelist, response) {
            fileEntry = filelist.data("file");
            if (this.model.saveUrl && response != "File upload has been denied") {
                this._setAction(filelist, "retry");
                this._setStatus(filelist, "failed");
            } else {
                filelist.find(".file-action").remove();
                this._setStatus(filelist, "failed");
            }
            if (response.indexOf("HTTP Error") == 0)
                args = { files: fileEntry, responseText: response, status: response.match(/\d+/).toString() };
            else
                args = { files: fileEntry, responseText: response };
            this._trigger('error', args);
        },

        _processServerResponse: function (uploadframe) {
            var uploadform;
            uploadform = $(document.body).find("form[target='" + $(uploadframe).attr("id") + "']");
            setTimeout(function () {
                uploadform.remove();
                uploadframe.remove();
            }, 0);
        },

        _createDivBlock: function (className) {
            return ej.buildTag('div.' + className);
        },

        _createForm: function (action, target) {
            return ej.buildTag('form', '', {}, { enctype: 'multipart/form-data', method: 'POST', action: action, target: target });
        },

        _createFrame: function (id) {
            return ej.buildTag('iframe#' + id, '', { display: 'none' }, { name: id });
        },

        _createInput: function (id) {
            return ej.buildTag('input', '', {}, { type: 'file', name: id, "data-role": "none" });
        },



        _initObjectsSyncUpload: function () {
            this._currentElement.closest("form")
                    .attr("enctype", "multipart/form-data")
                    .attr("encoding", "multipart/form-data");
            this._wireEventsSyncUpload();
        },

        _wireEventsSyncUpload: function () {
            var closestform = this._currentElement.closest("form")[0];
            this._on($(closestform), "submit", this._formSubmitSyncUpload);
            this._on($(closestform), "reset", this._formResetSyncUpload);
        },

        _onSelectSyncUpload: function (e) {
            var input, files, selection, addedfile;
            input = $(e.target);
            selection = $(".e-selectpart", this.control);
            this._currentElement.find("input.e-uploadinput").removeClass("e-uploadinput").css("display", "none").appendTo(selection);
            this._createInputandBind();
            addedfile = this._pushFile(this._files, { "file": this._files, "Input": input });
        },

        _onCancelSyncUpload: function (e, fileItem) {
            var inputfield = fileItem.data("Input");
            fileItem.data("file", null);
            fileItem.data("Input", null);
            fileItem.remove();
            inputfield.remove();
            this._uploadHide();
        },

        _formSubmitSyncUpload: function (e) {
            var input, uploader;
            input = $(".e-uploadinput", this.control);
            input.attr("name", "");
            uploader = this.model.uploadName != "" ? this.model.uploadName : this.control.id;
            setTimeout(function () {
                input.attr("name", uploader);
            }, 0);
        },

        _formResetSyncUpload: function (e) {
            $(".e-selectpart", this.control).children('input[type="file"]').each(function () {
                if (!$(this).hasClass('e-uploadinput')) {
                    $(this).remove();
                }
            });
        }
        /*Sync End*/
    });
    ej.Uploadbox.Locale = {};

    ej.Uploadbox.Locale["en-US"] = {
        buttonText: {
            upload: "Upload",
            browse: "Browse",
            cancel: "Cancel",
            close: "Close"
        },
        dialogText: {
            title: "Upload Box",
            name: "Name",
            size: "Size",
            status: "Status"
        },
		dropAreaText: "Drop files or click to upload",
        filedetail: "The selected file size is too large. Please select a file within the valid size.",
        denyError: "Files with #Extension extensions are not allowed.",
        allowError: "Only files with #Extension extensions are allowed.",
        cancelToolTip: "Cancel",
        removeToolTip: "Remove",
        retryToolTip: "Retry",
        completedToolTip: "Completed",
        failedToolTip: "Failed",
        closeToolTip: "Close"
    };
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html TextArea
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    var xhr = null;
    ej.widget("ejRTE", "ej.RTE", {

        element: null,

        model: null,
        _rootCSS: "e-rte",
        validTags: ["textarea"],
        _setFirst: false,
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },

        defaults: {

            allowEditing: true,

            isResponsive: false,

            allowKeyboardNavigation: true,

            cssClass: "",

            height: "370",

            width: "786",

            enabled: true,

            htmlAttributes: {},

            enableXHTML: false,

            maxLength: 7000,

            minWidth: 400,

            maxWidth: null,

            minHeight: 280,

            maxHeight: null,

            showToolbar: true,

            showFooter: false,

            showHtmlSource: false,

            showWordCount: true,
            showCharCount: true,

            showHtmlTagInfo: true,

            showClearAll: false,

            iframeAttributes: {},

            showClearFormat: true,

            showFontOption: true,
            autoFocus: false,
            enableHtmlEncode: false,
            showDimensions: false,

            showRoundedCorner: false,

            locale: "en-US",

            name: "",

            tools: {

                formatStyle: ["format"],

                style: ["bold", "italic", "underline", "strikethrough"],

                alignment: ["justifyLeft", "justifyCenter", "justifyRight", "justifyFull"],

                lists: ["unorderedList", "orderedList"],

                indenting: ["outdent", "indent"],

                doAction: ["undo", "redo"],

                links: ["createLink","removeLink"],

                    images: ["image"],
                    media: ["video"],

                tables: ["createTable", "addRowAbove", "addRowBelow", "addColumnLeft", "addColumnRight", "deleteRow", "deleteColumn", "deleteTable"],
				view:["fullScreen"]

            },

            toolsList: ["formatStyle", "font", "style", "effects", "alignment", "lists", "indenting", "clipboard", "doAction", "clear", "links", "images", "media", "tables", "casing", "customTools", "view"],

            colorCode: [
			            "000000", "FFFFFF", "C4C4C4", "ADADAD", "595959", "262626", "4f81bd", "dbe5f1", "b8cce4", "95b3d7", "366092", "244061", "c0504d", "f2dcdb", "e5b9b7", "d99694", "953734",
			            "632423", "9bbb59", "ebf1dd", "d7e3bc", "c3d69b", "76923c", "4f6128", "8064a2", "e5e0ec", "ccc1d9", "b2a2c7", "5f497a", "3f3151", "f79646", "fdeada", "fbd5b5", "fac08f",
			            "e36c09", "974806"
            ],

            format: [
                { text: "Paragraph", value: "<p>", spriteCssClass: "e-paragraph" },
                { text: "Quotation", value: "<blockquote>", spriteCssClass: "e-quotation" },
                { text: "Heading 1", value: "<h1>", spriteCssClass: "e-h1" },
                { text: "Heading 2", value: "<h2>", spriteCssClass: "e-h2" },
                { text: "Heading 3", value: "<h3>", spriteCssClass: "e-h3" },
                { text: "Heading 4", value: "<h4>", spriteCssClass: "e-h4" },
                { text: "Heading 5", value: "<h5>", spriteCssClass: "e-h5" },
                { text: "Heading 6", value: "<h6>", spriteCssClass: "e-h6" }
            ],

            fontName: [
                       {
                           text: "Segoe UI",
                           value: "Segoe UI"
                       },
                       {
                           text: "Arial",
                           value: "Arial,Helvetica,sans-serif"
                       },
                       {
                           text: "Courier New",
                           value: "Courier New,Courier,monospace"
                       },
                       {
                           text: "Georgia",
                           value: "Georgia,serif"
                       },
                       {
                           text: "Impact",
                           value: "Impact,Charcoal,sans-serif"
                       },
                       {
                           text: "Lucida Console",
                           value: "Lucida Console,Monaco,monospace"
                       },
                       {
                           text: "Tahoma",
                           value: "Tahoma,Geneva,sans-serif"
                       },
                       {
                           text: "Times New Roman",
                           value: "Times New Roman,Times,serif"
                       },
                       {
                           text: "Trebuchet MS",
                           value: "Trebuchet MS,Helvetica,sans-serif"
                       },
                       {
                           text: "Verdana",
                           value: "Verdana,Geneva,sans-serif"
                       }
            ],

            fontSize: [
                {
                    text: "1",
                    value: "1"
                },
                {
                    text: "2 (10pt)",
                    value: "2"
                },
                {
                    text: "3 (12pt)",
                    value: "3"
                },
                {
                    text: "4 (14pt)",
                    value: "4"
                },
                {
                    text: "5 (18pt)",
                    value: "5"
                },
                {
                    text: "6 (24pt)",
                    value: "6"
                },
                {
                    text: "7 (36pt)",
                    value: "7"
                }
            ],

            tableRows: 8,

            tableColumns: 10,

            colorPaletteRows: 6,

            colorPaletteColumns: 6,

            showCustomTable: true,

            value: null,            

            undoStackLimit: 50,

            enableResize: true,

            enablePersistence: false,

            enableRTL: false,
			
            autoHeight: false,

            imageBrowser: {

                filePath: "",

                extensionAllow: "*.png,*.gif,*.jpg,*.jpeg",

                ajaxAction: "",

                ajaxSettings: ""
            },

            fileBrowser: {

                filePath: "",

                extensionAllow: "*.doc,*.docx,*.pdf,*.txt",

                ajaxAction: ""                
            },


            change: null,

            execute: null,

            keydown: null,

            keyup: null,

            create: null,

            destroy: null,

            preRender: null
        },

        dataTypes: {
            enabled: "boolean",
            autoFocus: "boolean",
            enableHtmlEncode: "boolean",
            autoHeight: "boolean",
            enableXHTML: "boolean",
            isResponsive: "boolean",
            enablePersistence: "boolean",
            showRoundedCorner: "boolean",
            toolsList: "array",
            colorCode: "array",
            format: "data",
            fontName: "data",
            fontSize: "data",
            htmlAttributes: "data",
            iframeAttributes:"data",
            tools: {
                formatStyle: "array",
                font: "array",
                style: "array",
                effects: "array",
                alignment: "array",
                lists: "array",
                indenting: "array",
                clipboard: "array",
                doAction: "array",
                clear: "array",
                links: "array",
                media: "array",
                images: "array",
                tables: "array",
                casing: "array",
                customTools: "array"
            }
        },

        observables: ["value"],
        value: ej.util.valueFunction("value"),

        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "allowEditing": this._enableEdit(options[option]); break;
                    case "autoFocus ": { this.model.autoFocus = options[option]; if (options[option]) this._focus(); break; }
                    case "showToolbar": {
                        options[option] ? ej.isNullOrUndefined(this._rteToolbar) ? this._renderToolBar() : this._rteToolbar.show() : this._rteToolbar.hide();
                        break;
                    }
                    case "showFooter": {
                        this.model.showFooter = options[option];
                        options[option] ? ej.isNullOrUndefined(this._rteFooter) ? this._renderFooter() : this._rteFooter.show() : this._rteFooter.hide();
                        break;
                    }
                    case "isResponsive": {
                        this.model.isResponsive = options[option];
                        this._renderToolBarList();
                        var iframeHeight = $(this._rteIframe).outerHeight();
                        var toolListHeight = $(this._rteToolbar).outerHeight();
                        $(this._rteWapper).height(iframeHeight + toolListHeight);
                        if (!ej.isNullOrUndefined($(this._rteFooter))) {
                            var footerHeight = $(this._rteFooter).outerHeight();
                            $(this._rteWapper).height(iframeHeight + footerHeight + toolListHeight);
                        }
                        break;
                    }
                    case "enabled": this._disabled(options[option]); break;
                    case "height": this._rteWapper.height(options[option]); break;
                    case "width": this._rteWapper.css("width", options[option]); break;
                    case "cssClass": this._changeSkin(options[option]); break;
                    case "showHtmlSource": this._footerElement("div.e-rte-source", options[option]); break;
                    case "showHtmlTagInfo": this._footerElement("div.e-rte-htmltaginfo", options[option]); break;
                    case "showWordCount": {
                        this.model.showWordCount = options[option];
                        this._createCountElement(options[option],this._wordCount);                        
                        break;
                    }
                    case "showCharCount": {
                        this.model.showCharCount = options[option];
                        this._createCountElement(options[option], this._charCount);                        
                        break;
                    }
                    case "showClearAll": this._footerElement("div.clearAll", options[option]); break;
                    case "showClearFormat": this._footerElement("div.clearFormat", options[option]); break;
                    case "enableResize": this._footerElement("div.e-rte-resize", options[option]); break;
                    case "fontName": this._fontStyleDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "fontSize": this._fontSizeDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "format": this._formatDDL.ejDropDownList({ dataSource: options[option] }); break;
                    case "value": this._getDocument().body.innerHTML = ej.util.getVal(options[option]);
                        var args = { text: this._getText(), htmlText: this.getHtml() };
                        this._onChange();
                        break;
                    case "showDimensions":
                        if (options[option]) {
                            this._imgDialog.find("div.e-rte-imgdimensions").show();
                            this._imgDialog.find("div.e-rte-videoDimensions").show();
                        }
                        else {
                            this._imgDialog.find("div.e-rte-imgdimensions").hide();
                            this._imgDialog.find("div.e-rte-videoDimensions").hide();
                        }
                        break;
                    case "showFontOption":
                        if (options[option])
                            this._rteWapper.find("ul.e-rte-fontgroup").show();
                        else
                            this._rteWapper.find("ul.e-rte-fontgroup").hide();
                        break;
                    case "locale":
                        this.model.locale = options[option];
                        var model = this.model;
                        this.element.ejRTE("destroy").ejRTE(model);
                        break;
                    case "tableRows": {
                        this.model.tableRows = options[option];
                        var tblDiv = this._createTable.find("div.e-rte-table").html("");
                        this._drawTable(tblDiv);
                        break;
                    }
                    case "iframeAttributes": {
                        this._updateIframeSkin(options[option]);
                        break;
                    }
                    case "tableColumns": {
                        this.model.tableColumns = options[option];
                        var tblDiv = this._createTable.find("div.e-rte-table").html("");
                        this._drawTable(tblDiv);
                        break;
                    }
                    case "showCustomTable": {
                        var custLink = this._createTable.find("div.customtable-group");
                        options[option] ? ej.isNullOrUndefined(this._customTableDialog) ? this._createCustomTable() : custLink.show() : custLink.hide();
                        break;
                    }
                    case "colorPaletteRows": {
                        this.model.colorPaletteRows = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "colorPaletteColumns": {
                        this.model.colorPaletteColumns = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "colorCode": {
                        this.model.colorCode = options[option];
                        this._updateColorPalette();
                        break;
                    }
                    case "name": {
                        this.model.name = options[option];
                        this._checkNameAttr();
                        break;
                    }
                    case "enableRTL": this._enableRTL(options[option]); break;
                    case "htmlAttributes": this._addAttr(options[option]); break;
                    case "showRoundedCorner":
                        this._roundedCorner(options[option]);
                        break;
                    case "autoHeight": this.model.autoHeight = options[option];
                        this._setAutoHeight();
                }
                if (option != "locale")
                    this._setIframeHeight();
            }
            this._updateCount();
        },


        _init: function () {
            if (!ej.isNullOrUndefined(this.element) && this.element[0].type == "textarea") {
                this.element.hide();
                this._initialize()
                    ._render();
                this.model.allowEditing && this._wireEvents();
                this._updateIframeSkin(this.model.iframeAttributes);
                if (this.model.enabled == false) this._disabled(this.model.enabled);
                this._addAttr(this.model.htmlAttributes);
                if (this.model.autoFocus)
                    this._focus();
                if (this.model.enableHtmlEncode)
                    this.model.value = this._encode(this.value());
                this.disableToolbarItem("removeLink");
                try{
                this._rteIframe.contents().find("body").attr("style", "color:#5C5C5C");
                }
                catch (error) {
                return "";
                }
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy._rteWapper.addClass(value);
                else if (key == "name") proxy.element.attr(key, value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy.disable();
                else if (key == "readOnly" && value == "readOnly") proxy._enableEdit(false);
                else proxy._rteWapper.attr(key, value);
            });
        },

        _disabled: function (value) {
            value != true ? this.disable() : this.enable();
        },

        _destroy: function () {
            this.element.insertBefore(this._rteWapper);
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList("destroy");
            !ej.isNullOrUndefined(this._fontColorSplit) && this._fontColorSplit.ejSplitButton("destroy");
            !ej.isNullOrUndefined(this._bgColorSplit) && this._bgColorSplit.ejSplitButton("destroy");
            if (!ej.isNullOrUndefined(this._customTableDialog)) {
                !ej.isNullOrUndefined(this._customTableDialog.find(".e-rte-ddlAlignment")) && this._customTableDialog.find(".e-rte-ddlAlignment").ejDropDownList("destroy");
                !ej.isNullOrUndefined(this._customTableDialog.find(".e-rte-txtBorder")) && this._customTableDialog.find(".e-rte-txtBorder").ejDropDownList("destroy");
            }
            !ej.isNullOrUndefined(this._linkDialog) && $(this._linkDialog).closest("div.e-rte.e-dialog-wrap").remove();
            if (!ej.isNullOrUndefined(this._imgDialog)) {
                this._imgDialog.find(".e-rte-imgBorderStyle").ejDropDownList("destroy");
                $(this._imgDialog).closest("div.e-rte.e-dialog-wrap").remove();
            }
            !ej.isNullOrUndefined(this._videoDialog) && $(this._videoDialog).closest("div.e-rte.e-dialog-wrap").remove();
            !ej.isNullOrUndefined(this._createTable) && $(this._createTable).closest("div.e-rte.e-dialog-wrap").remove();
            !ej.isNullOrUndefined(this._sourceDialog) && $(this._sourceDialog).closest("div.e-dialog-wrap").remove();
            !ej.isNullOrUndefined(this._customTableDialog) && $(this._customTableDialog).closest("div.e-rte.e-dialog-wrap").remove();
            !ej.isNullOrUndefined(this._alertWindow) && $(this._alertWindow).closest("div.e-rte.e-dialog-wrap").remove();
            if (!ej.isNullOrUndefined(this._eTblDialog)) {
                this._eTblDialog.find(".e-rte-dropdown").ejDropDownList("destroy");
                $(this._eTblDialog).closest("div.e-rte.e-dialog-wrap").remove();
            }
            !ej.isNullOrUndefined(this._explorerObj) && this._explorerObj.destroy();
            !ej.isNullOrUndefined(this._imgPicker) && this._imgPicker.destroy();
            !ej.isNullOrUndefined(this._fileDialog) && $(this._fileDialog).closest("div.e-rte.e-dialog-wrap").remove();
            this._rteWapper.remove();
            this.element.show();
            this._createTable = null;
        },

        _initialize: function () {
            this._toolsList = this.model.toolsList;
            this._rteId = this.element[0].id;
            this._rteId = this._rteId ? this._rteId : "";
            this._backupArray = new Array();
            this._undoRedoPosition = 0;
            this._fontColor = "#000000";
            this._bgColor = "#ffffff";
            this._imgWidth = 0;
            this._imgHeight = 0;
            this._videoWidth = 0;
            this._videoHeight = 0;
            this._keypressFlag = true;
            this._styleItems = ej.isNullOrUndefined(this.model.tools["style"]) ? [] : this.model.tools["style"];
            this._alignItems = ej.isNullOrUndefined(this.model.tools["alignment"]) ? [] : this.model.tools["alignment"];
            this._listItems = ej.isNullOrUndefined(this.model.tools["lists"]) ? [] : this.model.tools["lists"];
                this._scriptsItems = ej.isNullOrUndefined(this.model.tools["effects"]) ? [] : this.model.tools["effects"];
            return this;
        },

        _render: function () {
            this._renderWrapper()
            ._checkNameAttr();
            if (this.model.showToolbar) {
                this._renderToolBar();
            }
            this._trigger("preRender", this.element);
            this._renderEditArea();
            this.model.showFooter && this._renderFooter();
            this._setIframeHeight();
            this._renderAlertDialog();
            this._roundedCorner(this.model.showRoundedCorner);
        },

        _checkNameAttr: function () {
            if (!this.model.name)
                this.element.attr({ "name": this.element[0].id });
            else
                this.element.attr({ "name": this.model.name });
        },
        _enableEdit: function (args) {
            this.model.allowEditing = args;
            if (args) {
                this._wireEvents();
                this.model.showToolbar && this._toolBarItems.ejToolbar("option", "click", this._toolBarClick);
                this._rteIframe.contents().find("body").attr("contenteditable", true);

            }
            else {
                this._unwireEvents();
                this.model.showToolbar && this._toolBarItems.ejToolbar("option", "click", null);
                this._rteIframe.contents().find("body").attr("contenteditable", false);

            }
        },
        _changeSkin: function (value) {
            this._rteWapper.removeClass(this.model.cssClass).addClass(value);
            this._subControlsSetModel("cssClass", value);
        },

        _updateIframeSkin: function (htmlAttr) {
            try {
                if (proxy.model.enableRTL) this._enableRTL(true);
                var target = this._rteIframe.contents().find("body");
                proxy = this;                
                $.map(htmlAttr, function (value, key) {
                    if (key == "class") target.addClass(value);
                    else if (key == "required")target.attr(key, value);
                    else if (key == "disabled" && value == "disabled") proxy.disable();
                    else if (key == "readOnly" && value == "readOnly") proxy._enableEdit(false);
                    else target.attr(key, value);               
                });
            }
            catch (err) {
                return false;
            }
        },


        _renderWrapper: function () {
            if (this.model.minHeight.toString().indexOf("%") > 0)
                this.model.minHeight = (parseInt(this.model.height) * parseInt(this.model.minHeight)) / 100;
            if (!ej.isNullOrUndefined(this.model.maxHeight) && (this.model.maxHeight.toString().indexOf("%") > 0))
                this.model.maxHeight = parseInt(this.model.height) + (parseInt(this.model.height) * parseInt(this.model.maxHeight)) / 100;
            if (this.model.minWidth.toString().indexOf("%") > 0)
                this.model.minWidth = (parseInt(this.model.width) * parseInt(this.model.minWidth)) / 100;
            if (!ej.isNullOrUndefined(this.model.maxWidth) && (this.model.maxWidth.toString().indexOf("%") > 0))
                this.model.maxWidth = parseInt(this.model.width) + (parseInt(this.model.width) * parseInt(this.model.maxWidth)) / 100;
            this._rteWapper = ej.buildTag("div.e-rte e-rte-wrapper e-widget e-box " + this.model.cssClass + "#" + this._rteId + "_wrapper", "", {}, { role: "presentation" }).insertBefore(this.element);
            this._rteWapper.css("height", this.model.height).css("width", this.model.width).css("min-height", this.model.minHeight).css("min-width", this.model.minWidth).css("max-height", this.model.maxHeight).css("max-width", this.model.maxWidth);
            this._rteWapper.append(this.element.hide());
            return this;
        },

        _renderToolBar: function () {
            this._rteToolbar = ej.buildTag("div.e-rte-toolbar #" + this._rteId + "_tools").insertAfter(this.element);
            this._crateToolbarTemplate()
            ._initToolBarItems()
            ._renderToolBarList();
            this._renderToolBarItems();
            if (!ej.isNullOrUndefined(this._explorerObj))
                this._rteToolbar.find("li.e-rteItem-image").length > 0 && this._explorerObj.setModel({ showFooter: false, showTreeview: false });
            if (!ej.isNullOrUndefined(this._fileBrowserObj))
                this._rteToolbar.find("li.e-rteItem-fileBrowser").length > 0 && this._fileBrowserObj.setModel({ showFooter: false, showTreeview: false });
                if (ej.browserInfo().name === "msie" && ej.browserInfo().version === "8.0") this._toolBarObj.disableItemByID(this._rteId + "_" + "video");
        },
        _renderToolBarList: function () {
            var model = {};
            model.click = this.model.allowEditing ? this._toolBarClick : null;
            model.cssClass = this.model.cssClass;
            model.enableRTL = this.model.enableRTL;
            model.enableSeparator = this.model.isResponsive ? true : false;
            model.isResponsive = this.model.isResponsive;
            model.height = model.isResponsive ? "" : "auto";
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._toolBarItems.ejToolbar(model);
            this._toolBarObj = this._toolBarItems.ejToolbar("instance");
        },
        _crateToolbarTemplate: function () {
            this._toolBarItems = ej.buildTag("div#" + this._rteId + "_toolbar").appendTo(this._rteToolbar).height(30);
            for (var items in this._toolsList) {
                items = this._toolsList[items];
                if (!ej.isNullOrUndefined(this.model.tools[items])) {
                    if (items == "customTools")
                        !ej.isNullOrUndefined(this.model.tools[items]) && this._customTools(this.model.tools[items]);
                    else
                        this.model.tools[items].length > 0 && this._createToolsItems(this.model.tools[items], items);
                }
            }
            if (this.model.fileBrowser.filePath != "") {
                var ulTag = ej.buildTag("ul#" + this._rteId + "-file-exporer"), liTag;
                liTag = $("<li id='" + this._rteId + "_fileBrowser" + "' class='e-rte-explorer e-rteItem-fileBrowser' title='" + this._getLocalizedLabels("fileBrowser") + "' ><div class='e-rte-toolbar-icon fileBrowser e-fileBrowser' unselectable = 'on'></div></li>");
                liTag.appendTo(ulTag);
                ulTag.appendTo(this._toolBarItems);
            }
            return this;
        },

        _createToolsItems: function (items, itemName) {
                var ulTag = ej.buildTag("ul#" + (this._rteId + "_" + itemName)), liTag;
            (itemName == "font") && ulTag.addClass("e-rte-fontgroup");
            (itemName == "formatStyle") && ulTag.addClass("e-rte-format");
            for (var i = 0; i < items.length; i++) {
                    liTag = $("<li id='" + (this._rteId + "_" + items[i].replace(/ /g, '')) + "' class='e-rteItem-" + items[i] + "' ><div class='e-rte-toolbar-icon " + items[i] + "' unselectable = 'on'></div></li>");
                if (items[i] === "video" && ej.browserInfo().name === "msie" && ej.browserInfo().version === "8.0") liTag.attr('title', this._getLocalizedLabels("html5Support"));
                else liTag.attr('title', this._getLocalizedLabels(items[i].replace(/ /g, '')));
                if (items[i] === "fullScreen")  liTag.attr('title', this._getLocalizedLabels("maximize")).children(".e-rte-toolbar-icon").addClass("maximize");              
			   liTag.appendTo(ulTag);
            }
            ulTag.appendTo(this._toolBarItems);
            if (itemName == "font" && !this.model.showFontOption)
                ulTag.hide();
        },

        _customTools: function (toolbarItems) {
            for (var item = 0; item < toolbarItems.length; item++) {
                var ulTag = ej.buildTag("ul"), liTag;
                liTag = $("<li id='" + toolbarItems[item].name + "' title='" + toolbarItems[item].tooltip + "' ><div class='" + (ej.isNullOrUndefined(toolbarItems[item].css) ? "" : toolbarItems[item].css) + "'>" + (ej.isNullOrUndefined(toolbarItems[item].text) ? "" : toolbarItems[item].text) + "</div></li>");
                var fn = toolbarItems[item].action;
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }
                !ej.isNullOrUndefined(toolbarItems[item].action) && this._on(liTag, "click", fn);
                $(toolbarItems[item].template).appendTo(liTag.find("div"));
                liTag.appendTo(ulTag);
                ulTag.appendTo(this._toolBarItems);
            }
        },

        _initToolBarItems: function () {
            this._rteToolbar.find("li.e-rteItem-format").length > 0 && this._renderFormat();
            this._rteToolbar.find("li.e-rteItem-createLink").length > 0 && this._renderLinkDialog();
            this._rteToolbar.find("li.e-rteItem-video").length > 0 && this._renderVideoDialog();
            this._rteToolbar.find("li.e-rteItem-image").length > 0 && this._renderImageDialog();
            this._rteToolbar.find("li.e-rteItem-createTable").length > 0 && this._renderTableDialog();
            this._rteToolbar.find("li.e-rteItem-fontName").length > 0 && this._renderFontStyle();
            this._rteToolbar.find("li.e-rteItem-fontSize").length > 0 && this._renderFontSize();
            this._rteToolbar.find("li.e-rteItem-fontColor").length > 0 && this._renderFontColor();
            this._rteToolbar.find("li.e-rteItem-backgroundColor").length > 0 && this._renderBGColor();
            this._rteToolbar.find("li.e-rteItem-createTable").length > 0 && this._renderEditTableDialog();
            this._rteToolbar.find("li.e-rteItem-fileBrowser").length > 0 && this._renderFileBrowserDialog();
            !this.model.showClearAll && this._rteToolbar.find("li.e-rteItem-clearAll").hide();
            return this;
        },
        _renderToolBarItems: function () {
                this._toolBarObj.disableItemByID(this._rteId + "_" + "undo");
                this._toolBarObj.disableItemByID(this._rteId + "_" + "redo");
            // Default action when render the RTE
            if (!this.model.enableRTL)
                    this._toolBarObj.selectItemByID(this._rteId + "_" + "justifyLeft");
            else
                    this._toolBarObj.selectItemByID(this._rteId + "_" + "justifyRight");
            //Hide the editable table icons
            var items = this._rteToolbar.find("li.e-rteItem-addColumnLeft,li.e-rteItem-addColumnRight,li.e-rteItem-addRowAbove,li.e-rteItem-addRowBelow,li.e-rteItem-deleteRow,li.e-rteItem-deleteColumn,li.e-rteItem-deleteTable");
            if (this.model.isResponsive && items.length == 0) this._toolBarObj._liTemplte.find("li.e-rteItem-addColumnLeft,li.e-rteItem-addColumnRight,li.e-rteItem-addRowAbove,li.e-rteItem-addRowBelow,li.e-rteItem-deleteRow,li.e-rteItem-deleteColumn,li.e-rteItem-deleteTable").hide();
            else this._rteToolbar.find("li.e-rteItem-addColumnLeft,li.e-rteItem-addColumnRight,li.e-rteItem-addRowAbove,li.e-rteItem-addRowBelow,li.e-rteItem-deleteRow,li.e-rteItem-deleteColumn,li.e-rteItem-deleteTable").hide();
            return this;
        },

        _renderEditArea: function () {
            this._rteEditor = ej.buildTag("div.editarea #" + this._rteId + "_editor").appendTo(this._rteWapper);
            this._rteIframe = ej.buildTag("iframe.content-iframe #" + this._rteId + "_Iframe", "", {}, { frameborder: "0" }).appendTo(this._rteEditor);
            this._setIFrames();
            if (navigator.userAgent.match(/iPad/i) != null)
                this._rteEditor.addClass('e-ipadscroll');
        },

        _xhtmlTagValidation: function () {
            if (this.model.value != null) {
                this.model.value = this.model.value.replace(/\r/gi, " ").replace(/\n/gi, "<br/> ").replace(/\r\n/gi, " ").replace(/( )+/gi, " ").replace(/&nbsp;/gi, " ").replace(/&bull;/gi, "*").replace(/&ltquo;/gi, "<").replace(/&rtquo;/gi, ">").replace(/&trze;/gi, "(tm)").replace(/&copy;/gi, "(c)").replace(/&reg;/gi, "(r)");
                if (this.model.value.indexOf("<body>") != -1) {
                    var indexBody = this.model.value.indexOf("<body>");
                    var delBody = this.model.value.slice(0, indexBody + 6);
                    this.model.value = this.model.value.replace(delBody, "").replace("</body></html>", "");
                }
                var htmlText = this.model.value;
                htmlText = htmlText.replace(/<br>/g, "<br/>").replace(/<hr>/g, "<hr/>");
                htmlText = this._imageXhtml(htmlText);
                htmlText = this._styleXhtml(htmlText);
                htmlText = this._startXhtml(htmlText);
                if (htmlText.indexOf("alt=") != -1) {
                    var imageTag = /alt=\"(.*?)\"/ig;
                    var altTemp = new Array();
                    var tempAlt;
                    var j = 0;
                    while ((tempAlt = imageTag.exec(htmlText)) != null) {
                        altTemp[j] = tempAlt[0];
                        j++;
                    }
                    for (var i = 0; i < altTemp.length; i++) {
                        htmlText = htmlText.replace(altTemp[i], "alt=\"\"");
                    }
                }
                if (!ej.isNullOrUndefined(this._fontXhtml(htmlText))) {
                htmlText = this._fontXhtml(htmlText);
                if (htmlText.indexOf("ol") != -1) {
                    var olStyle = /<ol(.*)<\/ol>/ig;
                    var tempOl;
                    var k = 0;
                    var olTemp = new Array();
                    while ((tempOl = olStyle.exec(htmlText)) !== null) {
                        olTemp[k] = tempOl[0].toString();
                        k++;
                    }
                    var olEx = new Array();
                    for (var f = 0; f < olTemp.length; f++) {
                        olEx[f] = olTemp[f];
                    }
                    for (var i = 0; i < olTemp.length; i++) {
                        olTemp[i] = olTemp[i].replace(/<br\/>/gi, "");
                    }
                    for (var i = 0; i < olTemp.length; i++) {
                        htmlText = htmlText.replace(olEx[i], olTemp[i]);
                    }
                }
                htmlText = htmlText.replace(/<o:p>/gi, "").replace(/<\/o:p>/gi, "");
                }
                this.model.value = htmlText;
            }
        },

        _setIFrames: function () {
            var rteContent = null;
            if ((this.value() != null && this.value() != "")) rteContent = this.value().replace(/'/g, "\"");
            if (rteContent == null) this.value($.trim(this.element[0].value));
            var isIE8 = ej.browserInfo().name == 'msie' && parseInt(ej.browserInfo().version) == 8;
            var _htmlText = ("<!DOCTYPE html><html><head><meta charset='utf-8' /><style>html,body{padding:0;margin:-1px 5px 0;height:100%;min-height:100%;padding-top:1px;font-size: 13px;font-family:Segoe UI;word-wrap: break-word;-webkit-nbsp-mode: space;-webkit-line-break: after-white-space;}"
			    + (isIE8 ? 'html{padding-top:0px}' : '')
                + "h1{font-size:2em;margin:.67em 0}h2{font-size:1.5em}h3{font-size:1.16em}h4{font-size:1em}h5{font-size:.83em}h6{font-size:.7em}"
                + (this.model.enableRTL ? 'ul,ol{padding:0 2.5em 0 0}' : 'ul,ol{padding:0 0 0 2.5em}') + "a{color:#00a} blockquote{margin-right: 0}"
                + ".e-rte-table{margin: 0.2em 0 0.5em;border: 1px solid;}.e-rte-table caption{border: 1px solid;border-bottom:none}"
                + ".e-rte-table,.e-rte-table td{outline:0; min-height:1px;vertical-align: top;text-indent: 5px;} .e-rte-table p{margin:0;padding:0;}"
                + "</style></head><body autocorrect='off' contenteditable=" + (this.model.allowEditing ? 'true' : 'false') + ">" + $.trim(this.value()) + "</body></html>");
            this._rteIframe.css({ "width": "100%" });
            if (this.element[0].value == "<div></div>" || this.element[0].value == "")
                this.element[0].innerHTML = this.value();

            this._setContent(_htmlText);
            if (this.model.enableXHTML) this._updateValue();
            this._setAutoHeight();
			if (!ej.isNullOrUndefined(this.model.undoStackLimit) || this.model.undoStackLimit !== "" || this.model.undoStackLimit > 0)
                this._backupArray[0] = this.getHtml();
        },


        _setIframeHeight: function () {
            var _height = (!ej.isNullOrUndefined(this._rteToolbar) && this._rteToolbar.is(':visible') ? this._rteToolbar.outerHeight() : 6) + (!ej.isNullOrUndefined(this._rteFooter) && this._rteFooter.is(':visible') ? this._rteFooter.outerHeight() : 0);
            this._rteIframe.height(_height == 0 ? this.model.height : (this._rteWapper.outerHeight() - _height) - (parseInt(this.model.showToolbar ? $(".e-rte .e-toolbarspan").css("border-bottom-width") : 0) + parseInt($(".e-rte-wrapper").css("border-bottom-width")) + parseInt($(".e-rte-wrapper").css("border-top-width")) + parseInt(this.model.showFooter ? $(".e-rte-footer").css("border-top-width") : 0)));          
        },

        // Footer Div render 
        _renderFooter: function () {
            var options = this.model;
            this._rteFooter = ej.buildTag("div.e-rte-footer #" + this._rteId + "_footer").appendTo(this._rteWapper);
            var _leftPan = ej.buildTag("div.e-rte-footer-left");
            this._htmlSource = ej.buildTag("div.e-rte-icons e-rte-footeritems e-rte-footericon e-rte-source", "", {}, { title: this._getLocalizedLabels("viewHtml") }).appendTo(_leftPan);
            this._htmlInfo = ej.buildTag("div.e-rte-htmltaginfo e-rte-footeritems").appendTo(_leftPan);
            _leftPan.appendTo(this._rteFooter);
            var _rightPan = ej.buildTag("div.e-rte-footer-right");
            this._wordCount = ej.buildTag("div.e-rte-wordcount e-rte-footeritems").appendTo(_rightPan);
            this._charCount = ej.buildTag("div.e-rte-charcount e-rte-footeritems").appendTo(_rightPan);
            this._clearFormat = ej.buildTag("div.e-rte-icons clearFormat e-rte-footericon e-rte-footeritems", "", {}, { title: this._getLocalizedLabels("clearFormat") }).appendTo(_rightPan);
            this._clearAll = ej.buildTag("div.e-rte-icons clearAll e-rte-footericon e-rte-footeritems", "", {}, { title: this._getLocalizedLabels("clearAll") }).appendTo(_rightPan);
            this._resize = ej.buildTag("div.e-icons e-rte-resize e-rte-footeritems").appendTo(_rightPan);
            _rightPan.appendTo(this._rteFooter);
            this._footerEvents("_on");
            !this.model.showHtmlSource && this._htmlSource.hide().removeClass("e-rte-footeritems");
            !this.model.showHtmlTagInfo && this._htmlInfo.hide().removeClass("e-rte-footeritems");
            !this.model.showWordCount && this._wordCount.hide().removeClass("e-rte-footeritems");
            !this.model.showCharCount && this._charCount.hide().removeClass("e-rte-footeritems");
            !this.model.showClearAll && this._clearAll.hide().removeClass("e-rte-footeritems");
            !this.model.showClearFormat && this._clearFormat.hide().removeClass("e-rte-footeritems");
            if (options.showFooter && this.model.enableRTL) this._rteFooter.addClass("e-rtl");
            if (options.enableResize)
                this._enableResize();
            else
                this._resize.hide().removeClass("e-rte-footeritems");
                this._updateCount();            
        },

        _footerElement: function (selector, visible) {
            if (this.model.showFooter) {
                var ele = this._rteFooter.find(selector);
                visible ? ele.show().addClass("e-rte-footeritems") : ele.hide().addClass("e-rte-footeritems");
            }
        },

        _updateColorPalette: function () {
            var _fontColor = $("ul#" + this._rteId + "_colorTable");
            var _bgColor = $("ul#" + this._rteId + "_colorBGTable");
            _fontColor.find("div.e-rte-table").remove();
            _fontColor.find("li").append(this._colorTable());
            //Bg Color table Update
            _bgColor.find("div.e-rte-table").remove();
            _bgColor.find("li").append(this._colorTable());
        },
        _enableRTL: function (value) {
            if (value)
                this._rteIframe.contents().find("body").css("direction", "rtl");
            else
                this._rteIframe.contents().find("body").css("direction", "ltr");
            this._subControlsSetModel("enableRTL", value);
        },
        _roundedCorner: function (value) {
            this._rteWapper.toggleClass('e-corner-all');
            this._subControlsSetModel("showRoundedCorner", value);
        },
        _subControlsSetModel: function (prop, value) {
            !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.ejDropDownList("option", prop, value);
            !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.ejDropDownList("option", prop, value);
            !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.ejDropDownList("option", prop, value);
            !ej.isNullOrUndefined(this._fontColorSplit) && this._fontColorSplit.ejSplitButton("option", prop, value);
            !ej.isNullOrUndefined(this._bgColorSplit) && this._bgColorSplit.ejSplitButton("option", prop, value);
            if (!ej.isNullOrUndefined(this._toolBarItems)) {
                if ((prop == "showRoundedCorner") && this._toolBarItems.hasClass("e-corner-top"))
                    this._toolBarItems.removeClass("e-corner-top").addClass("e-corner-all");
                this._toolBarItems.ejToolbar("option", prop, value);
                if ((prop == "showRoundedCorner") && this._toolBarItems.hasClass("e-corner-all"))
                    this._toolBarItems.removeClass("e-corner-all").addClass("e-corner-top");
            }
            if (!ej.isNullOrUndefined(this._linkDialog)) {
                this._linkDialog.ejDialog("option", prop, value);
                this._linkDialog.find(".e-rte-btn").ejButton("option", prop, value);
                this._chkTarget.ejCheckBox("option", prop, value);
                if (prop == "showRoundedCorner") (value == true) ? this._linkDialog.find(".e-inputtext").addClass('e-corner-all') : this._linkDialog.find(".e-inputtext").removeClass('e-corner-all');
            }
            if (!ej.isNullOrUndefined(this._imgDialog)) {
                this._imgDialog.ejDialog("option", prop, value);
                this._imgDialog.find("div.e-rte-imageTab").ejTab("option", prop, value);
                this._imgDialog.find(".e-rte-imgBorderPx").ejNumericTextbox("option", prop, value);
                this._imgDialog.find(".e-rte-imgBorderStyle").ejDropDownList("option", prop, value);
                this._chkImgDimensions.ejCheckBox("option", prop, value);
                this._dialogchkTarget.ejCheckBox("option", prop, value);
                this._imgDialog.find(".e-rte-btn").ejButton("option", prop, value);
                if (prop == "showRoundedCorner") (value == true) ? this._imgDialog.find(".e-inputtext").addClass('e-corner-all') : this._imgDialog.find(".e-inputtext").removeClass('e-corner-all');
            }
            if (!ej.isNullOrUndefined(this._videoDialog)) {
                this._videoDialog.ejDialog("option", prop, value);
                this._videoDialog.find(".e-rte-btn").ejButton("option", prop, value);
                this._chkvideoDimensions.ejCheckBox("option", prop, value);
                if (prop == "showRoundedCorner") (value == true) ? this._videoDialog.find(".e-inputtext").addClass('e-corner-all') : this._videoDialog.find(".e-inputtext").removeClass('e-corner-all');
            }
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog("option", prop, value);
            if (!ej.isNullOrUndefined(this._sourceDialog)) {
                this._sourceDialog.ejDialog("option", prop, value);
                this._sourceDialog.find(".e-rte-btn").ejButton("option", prop, value);
                if (prop == "showRoundedCorner") (value == true) ? this._sourceDialog.find(".e-inputtext").addClass('e-corner-all') : this._sourceDialog.find(".e-inputtext").removeClass('e-corner-all');
            }
            if (!ej.isNullOrUndefined(this._alertWindow)) {
                this._alertWindow.ejDialog("option", prop, value);
                this._alertWindow.find(".e-rte-btn").ejButton("option", prop, value);
            }
            if (!ej.isNullOrUndefined(this._customTableDialog)) {
                this._customTableDialog.ejDialog("option", prop, value);
                this._customTableDialog.find(".e-rte-btn").ejButton("option", prop, value);
                this._customTableDialog.find(".numerictextbox").ejNumericTextbox("option", prop, value);
                this._customTableDialog.find(".e-rte-txtColumns").ejNumericTextbox("option", prop, value);
                this._customTableDialog.find(".e-rte-ddlAlignment").ejDropDownList("option", prop, value);
                this._customTableDialog.find(".e-rte-txtBorder").ejDropDownList("option", prop, value);
                this._chkTblCaption.ejCheckBox("option", prop, value);
                if (prop == "showRoundedCorner") (value == true) ? this._customTableDialog.find(".e-inputtext").addClass('e-corner-all') : this._customTableDialog.find(".e-inputtext").removeClass('e-corner-all');
            }
            if (!ej.isNullOrUndefined(this._eTblDialog)) {
                this._eTblDialog.ejDialog("option", prop, value);
                this._eTblDialog.find(".e-numerictextbox").ejNumericTextbox("option", prop, value);
                this._eTblDialog.find(".e-rte-dropdown").ejDropDownList("option", prop, value);
                this._getTableObj("_Tab").ejTab("option", prop, value);
                this._eTblDialog.find(".e-rte-btn").ejButton("option", prop, value);
                this._eTblCaption.ejCheckBox("option", prop, value);
                if (prop == "showRoundedCorner") (value == true) ? this._eTblDialog.find(".e-inputtext").addClass('e-corner-all') : this._eTblDialog.find(".e-inputtext").removeClass('e-corner-all');
            }
            if (!ej.isNullOrUndefined(this._fileDialog)) {
                this._fileDialog.ejDialog("option", prop, value);
                this._fileDialog.find(".e-rte-btn").ejButton("option", prop, value);
                this._chkFileTarget.ejCheckBox("option", prop, value);
                if (prop == "showRoundedCorner") (value == true) ? this._fileDialog.find(".e-inputtext").addClass('e-corner-all') : this._fileDialog.find(".e-inputtext").removeClass('e-corner-all');
            }
        },
        _createCountElement: function (option, element) {
            option ? element.show() : element.hide();
            element.toggleClass("e-rte-footeritems");
        },

        _enableResize: function () {
            if (this.model.enableResize && this.model.enabled) {
                this._rteWapper.addClass("e-resizable");
                this._resizeRTE();
            }
        },

        _resizeRTE: function () {
            var proxy = this;
            this._overlayElement = ej.buildTag('div.e-rte-overlay');
            this._rteEditor.css("position", "relative");
            this._rteWapper.find("div.e-rte-resize").ejResizable(
                {
                    minHeight: parseInt(proxy.model.minHeight),
                    minWidth: parseInt(proxy.model.minWidth),
                    maxHeight: parseInt(proxy.model.maxHeight),
                    maxWidth: parseInt(proxy.model.maxWidth),
                    resizeStart: function (event) {
                        proxy._rteEditor.append(proxy._overlayElement);
                        proxy._trigger("resizeStart", { event: event });
                    },
                    resize: function (event) {
                        var reElement = $(event.element).parents("div.e-rte-wrapper");
                        proxy._rteWapper.height($(reElement).height());
                        proxy._rteWapper.width($(reElement).width());
                        proxy._setIframeHeight();
                        proxy._trigger("resize", { event: event });
                        if (proxy.model.isResponsive) proxy._toolBarObj._reSizeHandler();
                    },
                    resizeStop: function (event) {
                        proxy._overlayElement.remove();
                        proxy._trigger("resizeStop", { event: event });
                    },
                    helper: function (event) {
                        var reElement = $(event.element).parents("div.e-rte-wrapper");
                        proxy._rteWapper.height($(reElement).height());
                        proxy._rteWapper.width($(reElement).width());
                        proxy._setIframeHeight();
                        return $(proxy._rteWapper);
                    }
                });
        },

        _getDialogModel: function () {
            var rtecontent = this._rteWapper;
            return dialogModel = {
                enableResize: false,
                showOnInit: false,
                enableModal: true,
                cssClass: this.model.cssClass,
                target: rtecontent,
                enableRTL: this.model.enableRTL
            };
        },
        _labelFor: function (val) {
            return this._rteId ? " for=" + this._rteId + val + " " : "";
        },
        _TagId: function (val) {
            return this._rteId ? " id=" + this._rteId + val + " " : "";
        },
        _renderLinkDialog: function () {
            var btnDiv = (this.model.enableRTL) ? "e-rtlrte" : "";
            var createLink = this._linkDialog = ej.buildTag("div#" + this._rteId + "_link");
            var content = $("<div class='e-rte-label " + btnDiv + "'><label " + this._labelFor("_link_url") + ">" + this._getLocalizedLabels("linkWebUrl") + "</label></div><div class='e-rte-field " + btnDiv + "'><input type='text'  data-role ='none' class='e-inputtext e-rte-linkUrl' " + this._TagId("_link_url") + "/></div><div class='e-rte-label " + btnDiv + "'>" +
                           "<label " + this._labelFor("_link_text") + ">" + this._getLocalizedLabels("linkText") + "</label></div><div class='e-rte-field " + btnDiv + "'><input type='text' data-role ='none' class='e-inputtext e-rte-linkText' " + this._TagId("_link_text") + "></div><div class='e-rte-label " + btnDiv + "'>" +
                           "<label " + this._labelFor("_link_title") + ">" + this._getLocalizedLabels("linkToolTip") + " </label></div><div class='e-rte-field " + btnDiv + "'><input type='text' data-role ='none' class='e-inputtext e-rte-linkTitle' " + this._TagId("_link_title") + "></div><div class='e-rte-label " + btnDiv + "'></div>" +
                           "<div class='e-rte-field " + btnDiv + "'><input type='checkbox' class='e-rte-linkTarget'  data-role ='none' " + this._TagId("_link_target") + "><label " + this._labelFor("_link_target") + " style='padding: 5px;'>" + this._getLocalizedLabels("linkOpenInNewWindow") + "</label></div>" +
                           "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn " + btnDiv + "' type='button' data-role ='none' id='link_insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn " + btnDiv + "'type='button' data-role ='none' id='link_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(createLink);
            createLink.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.title = this._getLocalizedLabels("createLink");
            model.target = null;
            model.closeIconTooltip = this._getLocalizedLabels("closeIcon");
            model.showRoundedCorner = this.model.showRoundedCorner;
            createLink.ejDialog(model);
            $($(createLink.parents()).find(".e-dialog-wrap")).addClass("e-rte");
            this.model.showRoundedCorner && createLink.find(".e-inputtext").addClass('e-corner-all');
            this._chkTarget = createLink.find(".e-rte-linkTarget");
            this._chkTarget.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            createLink.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(createLink.find(".e-rte-btn"), "click", this._linkBtnClick)
            ._on(this._linkDialog.find(".e-rte-linkUrl"), "keypress", this._urlValidation);
        },


        _renderImageDialog: function () {
            var proxy = this, pathData = "";
            this._imgDialog = ej.buildTag("div#" + this._rteId + "_Image");
            var btnDiv = (this.model.enableRTL) ? "e-rtlrte" : "";
            var content = "<div " + this._TagId("_img_Tab") + " class='e-rte-imageTab'><ul><li class='e-first'><a href='#" + this._rteId + "_general'>" + this._getLocalizedLabels("general") + " </a></li><li class='e-nofirst'><a href='#" + this._rteId + "_advanced'>" + this._getLocalizedLabels("advanced") + " </a></li></ul>" +
                "<div id='" + this._rteId + "_general'>" +
                        "<div class='e-img-field-group'><div class='e-rte-label " + btnDiv + "'><label " + this._labelFor("_img_url") + ">" + this._getLocalizedLabels("imageWebUrl") + " </label></div><div class='e-rte-field " + btnDiv + "'><input type='text'  data-role ='none' class='e-inputtext e-rte-imgUrl' " + this._TagId("_img_url") + "></div></div><div class='e-img-field-group'><div class='e-rte-label " + btnDiv + "'>" +
                         "<label " + this._labelFor("_img_text") + ">" + this._getLocalizedLabels("imageAltText") + " </label></div><div class='e-rte-field " + btnDiv + "'><input type='text'  data-role ='none'  class='e-inputtext e-rte-imgText' " + this._TagId("_img_text") + "></div></div>" +
                        "<div class='e-rte-imgdimensions' " + this._TagId("_imgdimensions") + "><div class='e-rte-label " + btnDiv + "'><label>" + this._getLocalizedLabels("dimensions") + " </label></div><div class='e-rte-field " + btnDiv + "'><input type='text' data-role ='none'  title='" + this._getLocalizedLabels("dimensions") + "'  data-role ='none' class='e-inputtext e-dimensions e-rte-imgX' maxlength='3' " + this._TagId("_imgX") + "><label style='padding: 5px;'>X</label><input type='text'  data-role ='none'  title='" + this._getLocalizedLabels("dimensions") + "' style='margin-right: 5px;' class='e-inputtext e-dimensions e-rte-imgY' maxlength='3' " + this._TagId("_imgY") + "></div>" +
                 "<input type='checkbox' class='e-rte-imgConsrn' data-role ='none'" + this._TagId("_img_consrn") + "<label " + this._labelFor("_img_consrn") + "  style='padding: 5px;'>" + this._getLocalizedLabels("constrainProportions") + "</label></div>" +
                "</div>" +
                "<div id='" + this._rteId + "_advanced'>" +
                        "<div class='e-img-field-group'><div class='e-rte-label " + btnDiv + "'><label " + this._labelFor("_img_link") + ">" + this._getLocalizedLabels("imageLink") + " </label></div><div class='e-rte-field " + btnDiv + "'><input type='text'  data-role ='none' class='e-inputtext e-rte-imgLink' " + this._TagId("_img_link") + "></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-label " + btnDiv + "'><label " + this._labelFor("_img_border_px") + ">" + this._getLocalizedLabels("imageBorder") + "</label></div><div class='e-rte-field " + btnDiv + "'><div class='e-img-border-field'><input type='text' data-role ='none' title='" + this._getLocalizedLabels("imageBorder") + "' class='e-rte-imgBorderPx' maxlength='3' " + this._TagId("_img_border_px") + "></div><div class='e-img-border-field'><select class='e-rte-imgBorderStyle'  data-role ='none' " + this._TagId("_img_border_style") + "><option value='solid'>" + this._getLocalizedLabels("solid") + "</option><option value='dotted'>" + this._getLocalizedLabels("dotted") + "</option><option value='dashed'>" + this._getLocalizedLabels("dashed") + "</option><option value='double'>" + this._getLocalizedLabels("doubled") + "</option></select></div><div class='e-img-border-field'><input  data-role ='none' type='text' title='" + this._getLocalizedLabels("imageBorder") + "' class='e-inputtext e-rte-imgBorderColor' " + this._TagId("_img_border_color") + "></div></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-label " + btnDiv + "'><label " + this._labelFor("_img_style") + ">" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-field " + btnDiv + "'><input type='text' data-role ='none'  class='e-inputtext e-rte-imgStyle' " + this._TagId("_img_style") + "></div></div>" +
                        "<div class='e-img-field-group'><div class='e-rte-label " + btnDiv + "'></div><div class='e-rte-field " + btnDiv + "'><input type='checkbox' class='e-rte-imgLinkTarget' data-role ='none' " + this._TagId("_img_link_target") + "><label " + this._labelFor("_img_link_target") + " style='padding: 5px;'>" + this._getLocalizedLabels("linkOpenInNewWindow") + "</label></div></div>" +
                "</div>" +

                "</div>" +
                "<div class='e-rte-button'><button class='e-rte-btn " + btnDiv + "' data-role ='none'  type='button' id='" + this._rteId + "_img_insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn " + btnDiv + "' type='button' data-role ='none' id='" + this._rteId + "_img_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>";
            if (this.model.imageBrowser.filePath != "")
                pathData = "<div " + this._TagId("_img_explorer") + " class='e-rte-explorer e-rte-imgBrowser'></div>";
            content = $(pathData + content);
            content.appendTo(this._imgDialog);
            this._imgDialog.appendTo(this._rteWapper);
            !this.model.showDimensions && this._rteWapper.find("div.e-rte-imgdimensions").hide();
            var model = this._getDialogModel();
            model.width = model.width = this.model.imageBrowser.filePath !== "" ? "720px" : "470px";
            model.title = this._getLocalizedLabels("image");
            model.target = null;
            model.closeIconTooltip = this._getLocalizedLabels("closeIcon");
            model.showRoundedCorner = this.model.showRoundedCorner;
            model.open = function () {
                if (!ej.isNullOrUndefined(proxy._imgSplitObj))
                    proxy._imgSplitObj.option("enableAutoResize", true);
            };
            model.close = function () {
                if (!ej.isNullOrUndefined(proxy._imgSplitObj))
                    proxy._imgSplitObj.option("enableAutoResize", false);
            };
            this._imgDialog.ejDialog(model);
            this._imgDialog.parents("#" + this._rteId + "_Image_wrapper").css({ "visibility": "hidden", "display": "block" });
            $($(this._imgDialog.parents()).find(".e-dialog-wrap")).addClass("e-rte");
            if (this.model.imageBrowser.filePath != "") {
                var ajaxSettings = { upload: { url: this.model.imageBrowser.uploadAction } };
                if (this.model.imageBrowser.ajaxSettings)
                    $.extend(true, ajaxSettings, this.model.imageBrowser.ajaxSettings);

                this._explorerObj = this._imgDialog.find("div.e-rte-imgBrowser").ejFileExplorer({
                    allowMultiSelection: false, enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "690px", height: "300px", path: this.model.imageBrowser.filePath, fileTypes: this.model.imageBrowser.extensionAllow, layout: "list", select: function (e) { proxy._UpdateImgDetails(e); },
                    tools: {
                        creation: ["NewFolder", "Open"],
                        navigation: ["Back", "Forward"],
                        addressBar: ["Addressbar"],
                        editing: ["Refresh", "Upload", "Delete"],
                        searchBar: ["Searchbar"]
                    }, toolsList: ["creation", "navigation", "addressBar", "editing", "searchBar"], enableResize: false, showContextMenu: false,
                    ajaxAction: this.model.imageBrowser.ajaxAction, locale: this.model.locale,
                    ajaxSettings: ajaxSettings
                }).data('ejFileExplorer');
                this._imgSplitObj = this._imgDialog.find(".e-splitter").data("ejSplitter");
                this._imgSplitObj.option("enableAutoResize", false);
            }
            this.model.showRoundedCorner && this._imgDialog.find(".e-inputtext").addClass('e-corner-all');
            this._imgDialog.find("div.e-rte-imageTab").ejTab({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled: true, showRoundedCorner: this.model.showRoundedCorner });
            this._imgDialog.find(".e-rte-imgBorderPx").ejNumericTextbox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", minValue: 0, value: 0, showRoundedCorner: this.model.showRoundedCorner });
            this._imgDialog.find(".e-rte-imgBorderStyle").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled: true, showRoundedCorner: this.model.showRoundedCorner });
            this._chkImgDimensions = this._imgDialog.find(".e-rte-imgConsrn");
            this._chkImgDimensions.ejCheckBox({ check: true, enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._imgDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._imgPicker = this._imgDialog.find(".e-rte-imgBorderColor").ejColorPicker({ cssClass: this.model.cssClass, modelType: "palette", buttonText: { apply: this._getLocalizedLabels("buttonApply"), cancel: this._getLocalizedLabels("buttonCancel"), swatches: this._getLocalizedLabels("swatches") } }).data('ejColorPicker');
            this._imgDialog.parents("#" + this._rteId + "_Image_wrapper").css({ "visibility": "visible", "display": "none" });
            this._dialogchkTarget = this._imgDialog.find(".e-rte-imgLinkTarget");
            this._dialogchkTarget.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(this._imgDialog.find(".e-rte-btn"), "click", this._imageBtnClick)
            this._on(this._imgDialog.find(".e-rte-imgUrl"), "keypress", this._urlValidation);
            this._on(this._imgDialog.find("input.e-dimensions"), "change", this._recalcImgSize);
        },
        _renderFileBrowserDialog: function () {
            var proxy = this, pathData = "";
            var btnDiv = (this.model.enableRTL) ? "e-rtlrte" : "";
            this._fileDialog = ej.buildTag("div#" + this._rteId + "_fileDialog");
            var content = "<div class='e-rte-elements'><div class='e-rte-label " + btnDiv + "'><label " + this._labelFor("_file_url") + ">" + this._getLocalizedLabels("linkWebUrl") + "</label></div><div class='e-rte-field " + btnDiv + "'><input type='text'  data-role ='none' class='e-inputtext e-rte-fileUrl' " + this._TagId("_file_url") + "/></div><div class='e-rte-label " + btnDiv + "'>" +
                           "<label " + this._labelFor("_file_text") + ">" + this._getLocalizedLabels("linkText") + "</label></div><div class='e-rte-field " + btnDiv + "'><input type='text' data-role ='none' class='e-inputtext e-rte-fileText' " + this._TagId("_file_text") + "></div> </div>" +
                           "<div class='e-rte-field'><input type='checkbox' class='e-rte-fileTarget'  data-role ='none' " + this._TagId("_file_target") + "><label " + this._labelFor("_link_target") + " style='padding: 5px;'>" + this._getLocalizedLabels("linkOpenInNewWindow") + "</label></div>" +
                           "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn " + btnDiv + "' type='button' data-role ='none' id='" + this._rteId + "_file_insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn " + btnDiv + "'type='button' data-role ='none' id='" + this._rteId + "_file_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>";
            pathData = "<div " + this._TagId("_file_explorer") + " class='e-rte-explorer e-rte-fileBrowser'></div>";
            content = $(pathData + content);
            content.appendTo(this._fileDialog);
            this._fileDialog.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.width = "720px";
            model.title = this._getLocalizedLabels("fileBrowser");
            model.target = null;
            model.showRoundedCorner = this.model.showRoundedCorner;
            model.open = function () {
                if (!ej.isNullOrUndefined(proxy._fileSplitObj))
                    proxy._fileSplitObj.option("enableAutoResize", true);
            };
            model.close = function () {
                if (!ej.isNullOrUndefined(proxy._fileSplitObj))
                    proxy._fileSplitObj.option("enableAutoResize", false);
            };
            this._fileDialog.ejDialog(model);
            this._fileDialog.parents("#" + this._rteId + "_fileDialog_wrapper").css({ "visibility": "hidden", "display": "block" });
            $($(this._fileDialog.parents()).find(".e-dialog-wrap")).addClass("e-rte");
            var ajaxSettings = { upload: { url: this.model.fileBrowser.uploadAction } };
            if (this.model.fileBrowser.ajaxSettings)
                $.extend(true, ajaxSettings, this.model.fileBrowser.ajaxSettings);

            this._fileBrowserObj = this._fileDialog.find("div.e-rte-fileBrowser").ejFileExplorer({
                allowMultiSelection: false, enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "690px", height: "300px", path: this.model.fileBrowser.filePath, fileTypes: this.model.fileBrowser.extensionAllow, layout: "list", select: function (e) { proxy._UpdateFileDetails(e); },
                tools: {
                    creation: ["NewFolder", "Open"],
                    navigation: ["Back", "Forward"],
                    addressBar: ["Addressbar"],
                    editing: ["Refresh", "Upload", "Delete"],
                    searchBar: ["Searchbar"]
                }, toolsList: ["creation", "navigation", "addressBar", "editing", "searchBar"], enableResize: false, showContextMenu: false,
                ajaxAction: this.model.fileBrowser.ajaxAction, locale: this.model.locale,
                ajaxSettings: ajaxSettings
            }).data('ejFileExplorer');
            this._fileSplitObj = this._fileDialog.find(".e-splitter").data("ejSplitter");
            this._fileSplitObj.option("enableAutoResize", false);
            this.model.showRoundedCorner && this._fileDialog.find(".e-inputtext").addClass('e-corner-all');
            this._fileDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._fileDialog.parents("#" + this._rteId + "_fileDialog_wrapper").css({ "visibility": "visible", "display": "none" });
            this._chkFileTarget = content.find(".e-rte-fileTarget");
            this._chkFileTarget.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(this._fileDialog.find(".e-rte-btn"), "click", this._fileBtnClick);
        },
        _UpdateImgDetails: function (e) {
            if ($(this._imgURL).hasClass('e-error')) this._imgURL.removeClass("e-error");
            if (e.path === "") this._imgURL.val("http://");
            else this._imgDialog.find("div.e-rte-imageTab").find('.e-rte-imgUrl').val(e.path + (e.name[0] ? e.name[0] : ""));
        },
        _UpdateFileDetails: function (e) {
            if ($(this._fileURL).hasClass('e-error')) this._fileURL.removeClass("e-error");
            if (e.path === "") this._fileURL.val("http://");
            else this._fileDialog.find('.e-rte-fileUrl').val(e.path + (e.name[0] ? e.name[0] : ""));
        },
        _fileBtnClick: function (sender) {
            if (sender.target.id === this._rteId + "_file_cancel") {
                this._clearFileFields();
                this._fileDialog.ejDialog("close");
            }
            else if (sender.target.id === this._rteId + "_file_insert") {
                this._restoreSelection(this._selectionRange);
                if (this._fileDialog.find('e-rte-fileUrl').val() != "")
                    this._onInsertFileLink();
                else this._linkDialog.ejDialog("close");
                this._trigger("execute", { commandName: "hyperlink" });
                this._onChange();
                this._setBackupData();
            }
        },
        _clearFileFields: function () {
            this._fileURL = this._fileDialog.find(".e-rte-fileUrl");
            this._fileURL.val("http://");
            this._fileLinkText = this._fileDialog.find(".e-rte-fileText");
            this._fileLinkText.val("");
            this._chkFileTarget.ejCheckBox({ check: false });
        },
        _onGetInsertFileContent: function (Url) {
            if (this._fileLinkText.val() === "") this._selectedHTML = Url;
            else this._selectedHTML = this._fileLinkText.val();
        },
        _onInsertFileLink: function () {
            var oEl, sHtml;
            var fileVal = this._fileURL.val().match(/(ftp:|http:|https:)\/\//g);
            var fsrcUrl = (this._fileURL.val().startsWith("~")) ? this._fileURL.val().replace(this._fileURL.val().substr('0', '1'), "..") : this._fileURL.val();
            if ((ej.isNullOrUndefined(fileVal) && fsrcUrl !== "") || (!ej.isNullOrUndefined(fileVal) && fileVal[0].length < fsrcUrl.length)) {
                var selectedNode = (this._isIE()) ? this._currentSelNode : this._getSelectedNode();
                if (!ej.isNullOrUndefined(selectedNode) && selectedNode.tagName.toUpperCase() == 'A') {
                    selectedNode.href = fsrcUrl;
                    selectedNode.innerHTML = this._fileLinkText.val();
                    (this._chkFileTarget.ejCheckBox("isChecked")) ? selectedNode.target = "_blank" : selectedNode.target = "_self";
                }
                else if (!ej.isNullOrUndefined(selectedNode) && selectedNode.tagName.toUpperCase() == 'IMG') {
                    var parentNode = $(selectedNode).parent('a')[0];
                    if (ej.isNullOrUndefined(parentNode)) {
                        oEl = ej.buildTag("a", "", {}, { href: fsrcUrl });
                        $(selectedNode.outerHTML).appendTo($(oEl));
                        $(selectedNode).replaceWith($(oEl));
                        (this._chkFileTarget.ejCheckBox("isChecked")) ? oEl[0].target = "_blank" : oEl[0].target = "_self";
                    } else {
                        parentNode.href = fsrcUrl;
                        (this._chkFileTarget.ejCheckBox("isChecked")) ? parentNode.target = "_blank" : parentNode.target = "_self";
                    }
                }
                else {
                    oEl = ej.buildTag("a", "", {}, { href: fsrcUrl });
                    (this._chkFileTarget.ejCheckBox("isChecked")) ? oEl[0].target = "_blank" : oEl[0].target = "_self";
                    var oSelection;
                    var oSelRange;
                    this._focus();
                    if (this._isIE()) {
                        if (!ej.isNullOrUndefined(this._selectedHTML)) {
                            if (this._selectedHTML.length == 0)
                                this._onGetInsertFileContent(fsrcUrl);
                        }
                        else
                            this._onGetInsertFileContent(fsrcUrl);
                        oEl[0].innerHTML = this._selectedHTML;
                        sHtml = oEl[0].outerHTML;
                    }
                    else {
                        if (this._fileLinkText.val() === "")
                            oEl.html(fsrcUrl);
                        else oEl.html(this._fileLinkText.val());
                        sHtml = $('<div>').append($(oEl).clone()).html();
                    }
                    if (sHtml && (this.model.maxLength != null && (this.model.maxLength > $.trim(this._getText()).length))) {
                        this._isIE() ? this._pasteHtml(sHtml) : this._getDocument().execCommand('inserthtml', false, sHtml);
                        this._focus();
                    }
                }
                this._fileDialog.ejDialog("close");
            } else {
                this._fileURL.addClass("e-error");
                return false;
            }
            this.enableToolbarItem("removeLink");
            this._updateCount();
        },
        //Render the Table selection rows and columans popup
        _renderVideoDialog: function () {
            this._videoDialog = ej.buildTag("div#" + this._rteId + "_video");
            var btnDiv = (this.model.enableRTL) ? "e-rtlrte" : "";
            var content = $("<div><label>" + this._getLocalizedLabels("embedVideo") + "</label></div><textarea class='e-rte-video e-inputtext'  aria-label=" + this._getLocalizedLabels("embedVideo") + "></textarea>" +
               "<div class='e-rte-videoDimensions'" + this._TagId("_videodimensions") + " style='margin-top:7px'><div class='e-rte-label'><label>" + this._getLocalizedLabels("dimensions") + " </label></div><div><input type='text'  data-role ='none' title='" + this._getLocalizedLabels("dimensions") + "' class='e-inputtext e-dimensions e-rte-videoX' maxlength='3' " + this._TagId("_videoX") + "><label style='padding: 5px;'>X</label><input type='text'  data-role ='none'  title='" + this._getLocalizedLabels("dimensions") + "' style='margin-right: 5px;' class='e-inputtext e-dimensions e-rte-videoY' maxlength='3'" + this._TagId("_videoY") + ">" +
               "<input type='checkbox' class='e-rte-videoConsrn' data-role ='none'" + this._TagId("_video_consrn") + "<label " + this._labelFor("_video_consrn") + "style='padding: 5px;'>" + this._getLocalizedLabels("constrainProportions") + "</label></div></div>" +
                "<div class='e-rte-button e-fieldseparate'><button id='video_insert'type='button' data-role ='none' class='e-rte-btn " + btnDiv + "'>" + this._getLocalizedLabels("dialogInsert") + "</button><button id='video_cancel' type='button' data-role ='none' class='e-rte-btn " + btnDiv + "'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._videoDialog);
            this._videoDialog.appendTo(this._rteWapper);
            !this.model.showDimensions && this._rteWapper.find(".e-rte-videoDimensions").hide();
            var model = this._getDialogModel();
            model.width = "460px";
            model.closeIconTooltip = this._getLocalizedLabels("closeIcon");
            model.title = this._getLocalizedLabels("video");
            model.target = null;
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._videoDialog.ejDialog(model);
            $($(this._videoDialog.parents()).find(".e-dialog-wrap")).addClass("e-rte");
            this.model.showRoundedCorner && this._videoDialog.find(".e-inputtext").addClass('e-corner-all');
            this._chkvideoDimensions = this._videoDialog.find(".e-rte-videoConsrn");
            this._chkvideoDimensions.ejCheckBox({ checked: true, enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._videoDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(this._videoDialog.find(".e-rte-btn"), "click", this._insertVideo);
            this._on(this._videoDialog.find("input.e-dimensions"), "change", this._recalcVideoSize);
        },


        _renderTableDialog: function () {
            this._createTable = ej.buildTag("div.e-rte-table-picker#" + this._rteId + "_table");
            var tableCell, rowDiv, colDiv, tableDiv;
            this._tblheaderDiv = ej.buildTag("div.e-rte-tableheader");
            this._tblheaderDiv.html(this._getLocalizedLabels("createTable"));
            this._tblheaderDiv.appendTo(this._createTable);
            var tableDiv = ej.buildTag("div.e-rte-table");
            this._drawTable(tableDiv);
            tableDiv.appendTo(this._createTable);
            this._createCustomTable();
            this._createTable.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.enableModal = false;
            model.showHeader = false;
            model.width = "270px";
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._createTable.ejDialog(model);
        },

        _createCustomTable: function () {
            var customTable = $("<div class='customtable-group e-rte-createCustomTableLink' " + this._TagId("_createCustomTableLink") + "><span class='e-rte-toolbar-icon customtable-image'></span><a class='customtable-link' role='button' title='" + this._getLocalizedLabels("customTable") + "'>" + this._getLocalizedLabels("customTable") + "</a></div>" +
               "<div class='customtable-group e-rte-eTblProperties' " + this._TagId("_eTblProperties") + "><span class='e-rte-toolbar-icon customtable-image'></span><a class='customtable-link' role='button' title='" + this._getLocalizedLabels("editTable") + "'>" + this._getLocalizedLabels("editTable") + "</a></div>");
            this._on(customTable, "click", this._openCustomTable);
            customTable.appendTo(this._createTable);
            if (!this.model.showCustomTable)
                this._createTable.find("#" + this._rteId + "_createCustomTableLink").hide();

            this._customTableDialog = ej.buildTag("div.e-rte-customtable#" + this._rteId + "_customTable");
            var content = $("<div class='e-fieldgroup'><div class='e-rte-tablefields'><label " + this._labelFor("_txtColumns") + ">" + this._getLocalizedLabels("tableColumns") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='numerictextbox e-rte-txtColumns' " + this._TagId("_txtColumns") + "></div><div class='e-rte-tablefields'>" +
                 "<label " + this._labelFor("_txtRows") + ">" + this._getLocalizedLabels("tableRows") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='numerictextbox e-rte-txtRows' " + this._TagId("_txtRows") + "></div><div class='e-rte-tablefields'>" +
                 "<label " + this._labelFor("_txtWidth") + ">" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext e-rte-txtWidth'" + this._TagId("_txtWidth") + "></div>" +
                 "<div class='e-rte-tablefields'><label " + this._labelFor("_txtHeight") + ">" + this._getLocalizedLabels("tableHeight") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext e-rte-txtHeight'" + this._TagId("_txtHeight") + "></div></div>" +
                 "<div class='e-fieldseparate'><div class='e-rte-tablefields'><label " + this._labelFor("_txtSpacing") + ">" + this._getLocalizedLabels("tableCellSpacing") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext e-rte-txtSpacing' " + this._TagId("_txtSpacing") + "></div>" +
                 "<div class='e-rte-tablefields'><label " + this._labelFor("_txtPadding") + ">" + this._getLocalizedLabels("tableCellPadding") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext e-rte-txtPadding'" + this._TagId("_txtPadding") + "></div>" +
                 "<div class='e-rte-tablefields'><label " + this._labelFor("_txtBorder") + ">" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-tablefields'><select class='e-rte-txtBorder' data-role ='none' " + this._TagId("_txtBorder") + "><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='double'>Double</option></select></div>" +
                 "<div class='e-rte-tablefields'><label " + this._labelFor("_chkCaption") + ">" + this._getLocalizedLabels("tableCaption") + "</label></div><div class='e-rte-tablefields'><input type='checkbox' class='e-rte-chkCaption'  data-role ='none'" + this._TagId("_chkCaption") + "></div>" +
                 "<div class='e-rte-tablefields'><label " + this._labelFor("_ddlAlignment") + ">" + this._getLocalizedLabels("tableAlignment") + "</label></div><div class='e-rte-tablefields'><select class='e-rte-ddlAlignment' data-role ='none' " + this._TagId("_ddlAlignment") + "><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div></div>" +
                 "<div class='e-rte-button e-fieldseparate'><button class='e-rte-btn' data-role ='none' id='insert'>" + this._getLocalizedLabels("dialogInsert") + "</button><button class='e-rte-btn' data-role ='none' id='cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._customTableDialog);
            var model = this._getDialogModel();
            model.width = "480px";
            model.title = this._getLocalizedLabels("customTable");
            model.target = null;
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._customTableDialog.ejDialog(model);
            $($(this._customTableDialog.parents()).find(".e-dialog-wrap")).addClass("e-rte");
            this.model.showRoundedCorner && this._customTableDialog.find(".e-inputtext").addClass('e-corner-all');
            this._createTable.find("div.e-rte-eTblProperties").addClass('e-disable');
            this._customTableDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._customTableDialog.find(".numerictextbox").ejNumericTextbox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", minValue: 1, value: 3, showRoundedCorner: this.model.showRoundedCorner });
            this._customTableDialog.find(".e-rte-txtColumns").ejNumericTextbox({ maxValue: 63, showRoundedCorner: this.model.showRoundedCorner });
            this._customTableDialog.find(".e-rte-ddlAlignment").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled: false, showRoundedCorner: this.model.showRoundedCorner });
            this._customTableDialog.find(".e-rte-txtBorder").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled: true, showRoundedCorner: this.model.showRoundedCorner });
            this._chkTblCaption = this._customTableDialog.find(".e-rte-chkCaption");
            this._chkTblCaption.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(this._customTableDialog.find(".e-rte-btn"), "click", this._insertCustomTable);
        },
        _renderEditTableDialog: function () {
            this._eTblDialog = ej.buildTag("div#" + this._rteId + "_eTbl");
            var rteID = this._rteId;
            var content = $("<div id='" + this._rteId + "_eTbl_Tab'><ul><li class='e-first'><a href='#" + this._rteId + "_eTblTable'>" + this._getLocalizedLabels("table") + " </a></li><li class='e-nofirst'><a href='#" + this._rteId + "_eTblRow'>" + this._getLocalizedLabels("row") + " </a></li><li class='e-nofirst'><a href='#" + this._rteId + "_eTblCell'>" + this._getLocalizedLabels("cell") + " </a></li></ul>" +
            "<div id='" + this._rteId + "_eTblTable'>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblWidth") + ">" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext' " + this._TagId("_eTblWidth") + "></div>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblHeight") + ">" + this._getLocalizedLabels("tableHeight") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext' " + this._TagId("_eTblHeight") + "></div>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblCellSpace") + ">" + this._getLocalizedLabels("tableCellSpacing") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext e-numerictextbox' " + this._TagId("_eTblCellSpace") + "></div>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblCellPad") + ">" + this._getLocalizedLabels("tableCellPadding") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext e-numerictextbox' " + this._TagId("_eTblCellPad") + "></div>" +
                   "<div class='e-rte-tablefields'><label  " + this._labelFor("_eTblAlign") + ">" + this._getLocalizedLabels("tableAlignment") + "</label></div><div class='e-rte-tablefields'><select id='" + rteID + "_eTblAlign' data-role ='none'  class='e-rte-dropdown'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblCaption") + ">" + this._getLocalizedLabels("tableCaption") + "</label></div><div class='e-rte-tablefields'><input type='checkbox' data-role ='none' " + this._TagId("_eTblCaption") + "></div>" +
                    "<div class='e-img-field-group'><div class='e-rte-tablefields'><label " + this._labelFor("_eTblBrdrPx") + ">" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-stylefield'><div class='e-img-border-field'><input  data-role ='none' type='text' title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext e-numerictextbox' maxlength='3'  " + this._TagId("_eTblBrdrPx") + "></div><div class='e-img-border-field'><select " + this._TagId("_eTbl_border_style") + " class='e-rte-dropdown' data-role ='none' ><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='solid'>Double</option></select></div><div class='e-img-border-field'><input type='text' data-role ='none'  title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext' maxlength='7' " + this._TagId("_eTbl_border_color") + "></div></div></div>" +
                    "<div class='e-img-field-group'><div class='e-rte-tablefields'><label " + this._labelFor("_eTblStyle") + ">" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-stylefield'><input  data-role ='none' type='text' class='e-inputtext' " + this._TagId("_eTblStyle") + "></div></div>" +
            "</div>" +
            "<div id='" + rteID + "_eTblRow'>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblRHeight") + ">" + this._getLocalizedLabels("tableHeight") + "</label></div><div class='e-rte-tablefields'><input  data-role ='none' type='text' class='e-inputtext' " + this._TagId("_eTblRHeight") + "></div>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblRAlign") + ">" + this._getLocalizedLabels("textAlign") + "</label></div><div class='e-rte-tablefields'><select " + this._TagId("_eTblRAlign") + " data-role ='none'  class='e-rte-dropdown'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div>" +
                    "<div class='e-img-field-group'><div class='e-rte-tablefields'><label " + this._labelFor("_eTblRBrdrPx") + ">" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-stylefield'><div class='e-img-border-field'><input type='text' data-role ='none'  title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext e-numerictextbox' maxlength='3'  " + this._TagId("_eTblRBrdrPx") + "></div><div class='e-img-border-field'><select " + this._TagId("_eTblRBrdrStyle") + " data-role ='none' class='e-rte-dropdown'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='solid'>Double</option></select></div><div class='e-img-border-field'><input type='text' data-role ='none'  title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext' maxlength='7' " + this._TagId("_eTblRBrdrColor") + "></div></div></div>" +
                    "<div class='e-img-field-group'><div class='e-rte-tablefields'><label " + this._labelFor("_eTblRStyle") + ">" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-stylefield'><input type='text' data-role ='none'  class='e-inputtext' " + this._TagId("_eTblRStyle") + "></div></div>" +
            "</div>" +
            "<div id='" + rteID + "_eTblCell'>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblClWidth") + ">" + this._getLocalizedLabels("tableWidth") + "</label></div><div class='e-rte-tablefields'><input type='text' data-role ='none'  class='e-inputtext' " + this._TagId("_eTblClWidth") + "></div>" +
                    "<div class='e-rte-tablefields'><label " + this._labelFor("_eTblClAlign") + ">" + this._getLocalizedLabels("textAlign") + "</label></div><div class='e-rte-tablefields'><select " + this._TagId("_eTblClAlign") + " data-role ='none'  class='e-rte-dropdown'><option value='left'>Left</option><option value='right'>Right</option><option value='center'>Center</option></select></div>" +
                    "<div class='e-img-field-group'><div class='e-rte-tablefields'><label " + this._labelFor("_eTblClBrdrPx") + ">" + this._getLocalizedLabels("tableBorder") + "</label></div><div class='e-rte-stylefield'><div class='e-img-border-field'><input type='text' data-role ='none'  title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext e-numerictextbox' maxlength='3' " + this._TagId("_eTblClBrdrPx") + " ></div><div class='e-img-border-field'><select " + this._TagId("_eTblClBrdrStyle") + " data-role ='none' class='e-rte-dropdown'><option value='solid'>Solid</option><option value='dotted'>Dotted</option><option value='dashed'>Dashed</option><option value='solid'>Double</option></select></div><div class='e-img-border-field'><input type='text' data-role ='none'  title='" + this._getLocalizedLabels("tableBorder") + "' class='e-inputtext' maxlength='7' " + this._TagId("_eTblClBrdrColor") + "></div></div></div>" +
                    "<div class='e-img-field-group'><div class='e-rte-tablefields'><label " + this._labelFor("_eTblClStyle") + ">" + this._getLocalizedLabels("imageStyle") + " </label></div><div class='e-rte-stylefield'><input type='text' data-role ='none'  class='e-inputtext' " + this._TagId("_eTblClStyle") + "></div></div>" +
            "</div>" +
            "</div>" +
            "<div class='e-rte-button'><button class='e-rte-btn' data-role ='none' id='" + rteID + "_eTbl_apply'>" + this._getLocalizedLabels("dialogApply") + "</button><button class='e-rte-btn' data-role ='none' id='" + rteID + "_eTbl_cancel'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>"
            );
            content.appendTo(this._eTblDialog);
            this._eTblDialog.appendTo(this._rteWapper);
            var model = this._getDialogModel();
            model.width = "470px";
            model.title = this._getLocalizedLabels("editTable");
            model.target = null;
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._eTblDialog.ejDialog(model);
            $($(this._eTblDialog.parents()).find(".e-dialog-wrap")).addClass("e-rte");
            this.model.showRoundedCorner && this._eTblDialog.find(".e-inputtext").addClass('e-corner-all');
            this._eTblDialog.find(".e-numerictextbox").ejNumericTextbox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", minValue: 0, value: 0, showRoundedCorner: this.model.showRoundedCorner });
            this._eTblDialog.find(".e-rte-dropdown").ejDropDownList({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", showRoundedCorner: this.model.showRoundedCorner });
            this._eTblAlign = this._getTableObj("Align").data('ejDropDownList');
            this._eTblAlign.option('enabled', false);
            this._getTableObj("_Tab").ejTab({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, width: "100%", enabled: true, showRoundedCorner: this.model.showRoundedCorner });
            this._eTblCaption = this._getTableObj("Caption");
            this._eTblCaption.ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._eTblDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(this._eTblDialog.find(".e-rte-btn"), "click", this._eTblBtnClick);
        },

        _drawTable: function (tableDiv) {
            for (var row = 0; row < this.model.tableRows ; row++) {
                rowDiv = ej.buildTag("div.e-rtetablerow");
                for (var col = 0; col < this.model.tableColumns; col++) {
                    tableCell = ej.buildTag("div.e-rte-tablecell e-default");
                    tableCell.appendTo(rowDiv);
                }
                rowDiv.appendTo(tableDiv);
            }
        },


        _renderSourceDialog: function () {
            this._sourceDialog = ej.buildTag("div#" + this._rteId + "_Source");
            var content = $("<textarea class='e-rte-srctextarea e-inputtext'></textarea><div class='e-rte-button e-fieldseparate'><button id='src_update' data-role ='none' class='e-rte-btn'>" + this._getLocalizedLabels("dialogUpdate") + "</button><button id='src_cancel' class='e-rte-btn'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._sourceDialog);
            var model = this._getDialogModel();
            model.title = this._getLocalizedLabels("viewHtml");
            model.width = "auto";
            model.target = null;
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._sourceDialog.ejDialog(model);
            this.model.showRoundedCorner && this._sourceDialog.find(".e-inputtext").addClass('e-corner-all');
            this._sourceDialog.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(this._sourceDialog.find(".e-rte-btn"), "click", this._srcBtnClick)
        },


        _renderAlertDialog: function () {
            this._alertWindow = ej.buildTag("div#" + this._rteId + "_Alert");
            var content = $("<p class='e-alerttext'></p><div class='e-rte-button e-fieldseparate'><button data-role ='none' class='e-rte-btn e-alert-ok' type='button'>" + this._getLocalizedLabels("dialogOk") + "</button><button class='e-rte-btn e-alert-cancel' data-role='none'>" + this._getLocalizedLabels("dialogCancel") + "</button></div>");
            content.appendTo(this._alertWindow);
            var model = this._getDialogModel();
            model.showHeader = false;
            model.width = "auto";
            model.target = null;
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._alertWindow.ejDialog(model);
            $($(this._alertWindow.parents()).find(".e-dialog-wrap")).addClass("e-rte");
            this._alertWindow.find(".e-rte-btn").ejButton({ enableRTL: this.model.enableRTL, type: "button", cssClass: this.model.cssClass, showRoundedCorner: this.model.showRoundedCorner });
            this._on(this._alertWindow.find(".e-rte-btn"), "click", this._alertBtnClick)
            this._alertWindow.find(".e-alert-cancel").hide();
        },

        _alertBtnClick: function (e) {
            if ($.trim(this._alertWindow.find(".e-alerttext").html()) == this._getLocalizedLabels("deleteAlert") && $(e.target).hasClass("e-alert-ok")) {
                this._getDocument().body.innerHTML = "";
                if (this.model.showToolbar)
                        this._toolBarObj.selectItemByID(this._rteId + "_" + "justifyLeft");
                this._setBackupData();
                this._updateCount();                
                var selectNode = (this._isIE()) ? this._currentSelNode : this._getSelectedNode();
                this._updateTagInfo(selectNode);
                this._onChange();
            }
            this._alertWindow.ejDialog("close");
            this._alertWindow.find(".e-alert-cancel").hide();
        },

        _openAlert: function (alertText) {
            this._alertWindow.find(".e-alerttext").html(alertText);
			$("#" + this._rteId + "_Alert").css({"z-index":this._onGetMaxZindex()});
            this._alertWindow.ejDialog("open");
        },


        _renderFormat: function () {
            this._formatDDL = ej.buildTag("input#" + this._rteId + "_format", "", "", { type: "text", title: this._getLocalizedLabels("format"), "data-role": "none" });
            var model = {};
            model.dataSource = this.model.format;
            model.width = "105px";
            model.enableRTL = this.model.enableRTL;
            model.popupWidth = "175px";
            model.popupHeight = "auto";
            model.selectedItemIndex = 0;
            model.popupHide = this._formatChange;
            model.cssClass = "e-rte-format-ddl";
            model.showRoundedCorner = this.model.showRoundedCorner;
            model.fields = { text: "text", value: "value", spriteCssClass: "spriteCssClass" };
            this._formatDDL.appendTo(this._rteToolbar.find("li.e-rteItem-format").html(""));
            this._formatDDL.ejDropDownList(model);
            this._formatPopupStyle();
        },

        _formatPopupStyle: function () {
            var _liList = this._formatDDL.data("ejDropDownList").popupPanelWrapper.find("li");
            for (var i = 0; i < _liList.length; i++) {
                $(_liList[i]).addClass(this.model.format[i].spriteCssClass);
            }
        },


        _renderFontStyle: function () {
            this._fontStyleDDL = ej.buildTag("input#" + this._rteId + "_fontName", "", "", { type: "text", title: this._getLocalizedLabels("fontName"), "data-role": "none" });
            var model = {};
            model.dataSource = this.model.fontName;
            model.enableRTL = this.model.enableRTL;
            model.selectedItemIndex = 0;
            model.popupHide = this._fontStyleChange;
            model.showRoundedCorner = this.model.showRoundedCorner;
            model.fields = { text: "text", value: "value" };
            this._fontStyleDDL.appendTo(this._rteToolbar.find("li.e-rteItem-fontName").html(""));
            this._fontStyleDDL.ejDropDownList(model);
        },


        _renderFontSize: function () {
            this._fontSizeDDL = ej.buildTag("input#" + this._rteId + "_fontSize", "", "", { type: "text", title: this._getLocalizedLabels("fontSize"), "data-role": "none" });
            var model = {};
            model.dataSource = this.model.fontSize;
            model.width = "100px";
            model.enableRTL = this.model.enableRTL
            model.selectedItemIndex = 1;
            model.popupHide = this._fontSizeChange;
            model.showRoundedCorner = this.model.showRoundedCorner;
            model.fields = { text: "text", value: "value" };
            this._fontSizeDDL.appendTo(this._rteToolbar.find("li.e-rteItem-fontSize").html(""));
            this._fontSizeDDL.ejDropDownList(model);
        },


        _renderFontColor: function () {
            var proxy = this;
            this._fontColorSplit = ej.buildTag("button#" + this._rteId + "_fontColor", "", {}, { type: "button", "data-role": "none" });
            var _liTemplte = $("<ul id='" + this._rteId + "_colorTable' class='e-rte-colorpalette'><li></li></ul>");
            _liTemplte.find("li").append(this._colorTable());
            var model = {};
            model.width = "50px";
            model.contentType = "textandimage";
            model.targetID = this._rteId + "_colorTable";
            model.enableRTL = this.model.enableRTL;
            model.prefixIcon = "e-fontcolor-icon";
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._fontColorSplit.appendTo(this._rteToolbar.find("li.e-rteItem-fontColor").html(""));
            var oldWrapper = $("#" + this._rteId + "_colorTable").get(0);
            if (oldWrapper) {
                if ($(oldWrapper).parent().hasClass("e-menu-wrap"))
                    $(oldWrapper).parent().remove();
                else
                    $(oldWrapper).remove();
            }
            _liTemplte.appendTo(this._rteToolbar.find("li.e-rteItem-fontColor"));
            this._fontColorSplit.ejSplitButton(model);
            this._splitObj = this._fontColorSplit.ejSplitButton("instance");
            this._splitObj.option("beforeOpen", function () { proxy._bindClickOperation("FontColor"); });
            this._on(this._fontColorSplit, "click", this._fontColorClick);
            this._fontColorSplit.find("span.e-fontcolor-icon").removeClass("e-icon");
            this._fontColorSplit.find("span.e-btntxt").css("background-color", this._fontColor);
        },

        _renderBGColor: function () {
            var proxy = this;
            this._bgColorSplit = ej.buildTag("button#" + this._rteId + "_backgroundColor", "", {}, { type: "button", "data-role": "none" });
            var _liTemplte = $("<ul id='" + this._rteId + "_colorBGTable' class='e-rte-colorpalette'><li></li></ul>");
            _liTemplte.find("li").append(this._colorTable());
            var model = {};
            model.width = "50px";
            model.contentType = "textandimage";
            model.targetID = this._rteId + "_colorBGTable";
            model.enableRTL = this.model.enableRTL;
            model.prefixIcon = "e-bgcolor-icon";
            model.showRoundedCorner = this.model.showRoundedCorner;
            this._bgColorSplit.appendTo(this._rteToolbar.find("li.e-rteItem-backgroundColor").html(""));
            var oldWrapper = $("#" + this._rteId + "_colorBGTable").get(0);
            if (oldWrapper) {
                if ($(oldWrapper).parent().hasClass("e-menu-wrap"))
                    $(oldWrapper).parent().remove();
                else
                    $(oldWrapper).remove();
            }
            _liTemplte.appendTo(this._rteToolbar.find("li.e-rteItem-backgroundColor"));
            this._bgColorSplit.ejSplitButton(model);
            this._bgSplitObj = this._bgColorSplit.ejSplitButton("instance");
            this._bgSplitObj.option("beforeOpen", function () { proxy._bindClickOperation("BGColor"); });
            this._on(this._bgColorSplit, "click", this._bgColorClick);
            this._bgColorSplit.find("span.e-bgcolor-icon").removeClass("e-icon");
            this._bgColorSplit.find("span.e-btntxt").css("background-color", this._bgColor);
        },
        _bindClickOperation: function (toolName) {
            var proxy = this;
            var splitObj = toolName == "BGColor" ? proxy._bgSplitObj : proxy._splitObj;
            var element = toolName == "BGColor" ? "_colorBGTable" : "_colorTable";
            SplitMenu = $("#" + proxy._rteId + element).ejMenu("instance");
            toolName == "BGColor" ? proxy._bgSplitMenu = SplitMenu : proxy._splitMenu = SplitMenu;
            SplitMenu.model.close = function () {
                splitObj.contstatus = false;
                proxy._off($("#" + proxy._rteId + element).find("div.e-rte-palettetable"), "click", toolName == "BGColor" ? proxy._bgColorPaletteClick : proxy._colorPaletteClick);
            }
            proxy._on($("#" + proxy._rteId + element).find("div.e-rte-palettetable"), "click", toolName == "BGColor" ? proxy._bgColorPaletteClick : proxy._colorPaletteClick);
        },
        _colorTable: function () {
            var tableDiv = ej.buildTag("div.e-rte-table");
            var color = 0, hexCode, rowDiv;
            for (var row = 0; row < this.model.colorPaletteRows ; row++) {
                rowDiv = ej.buildTag("div.e-rtetablerow");
                for (var col = 0; col < this.model.colorPaletteColumns ; col++) {
                    if (color < this.model.colorCode.length) {
                        hexCode = "#" + this.model.colorCode[color];
                        tableCell = ej.buildTag("div.e-rte-palettetable", "", { "background-color": hexCode }, { title: hexCode, "color-code": hexCode });
                        tableCell.appendTo(rowDiv);
                        color++;
                    }
                }
                rowDiv.appendTo(tableDiv);
            }
            return tableDiv;
        },

        _recalcVideoSize: function (e) {
            var txtWidth, txtheight, newWidth, newHeight;
            txtWidth = this._videoDialog.find(".e-rte-videoX");
            txtheight = this._videoDialog.find(".e-rte-videoY");

            newWidth = txtWidth.val();
            newHeight = txtheight.val();

            if (this._chkvideoDimensions.ejCheckBox("isChecked") && this._videoWidth && this._videoHeight && newWidth && newHeight) {
                if (e.target.id == txtWidth[0].id) {
                    newHeight = Math.round((newWidth / this._videoWidth) * newHeight);
                    txtheight.val(newHeight);
                } else {
                    newWidth = Math.round((newHeight / this._videoHeight) * newWidth);
                    txtWidth.val(newWidth);
                }
            }
            this._videoWidth = newWidth;
            this._videoHeight = newHeight;
        },


        _recalcImgSize: function (e) {
            var txtWidth, txtheight, newWidth, newHeight;
            txtWidth = this._imgDialog.find(".e-rte-imgX");
            txtheight = this._imgDialog.find(".e-rte-imgY");

            newWidth = txtWidth.val();
            newHeight = txtheight.val();

            if (this._chkImgDimensions.ejCheckBox("isChecked") && this._imgWidth && this._imgHeight && newWidth && newHeight) {
                if (e.target.id == txtWidth[0].id) {
                    newHeight = Math.round((newWidth / this._imgWidth) * newHeight);
                    txtheight.val(newHeight);
                } else {
                    newWidth = Math.round((newHeight / this._imgHeight) * newWidth);
                    txtWidth.val(newWidth);
                }
            }
            this._imgWidth = newWidth;
            this._imgHeight = newHeight;
        },

        _urlValidation: function (e) {
            var _target = $(e.target);
            _target.hasClass("e-error") && _target.removeClass("e-error");
        },

        _linkBtnClick: function (sender) {
            if (sender.target.id === "link_cancel") {
                this._clearLinkFields();
                this._linkDialog.ejDialog("close");
            }
            else if (sender.target.id === "link_insert") {
                this._restoreSelection(this._selectionRange);
                if (this._txtURL.val() != "")
                    this._onInsertLink();
                else {
                    this._onUnlink();
                    this._linkDialog.ejDialog("close");
                }
                this._trigger("execute", { commandName: "hyperlink" });
                this._onChange();
                this._setBackupData();
            }
        },

        _imageBtnClick: function (sender) {
            if (sender.target.id === this._rteId + "_img_cancel") {
                this._imgDialog.ejDialog("close");
            }
            else if (sender.target.id === this._rteId + "_img_insert") {
                var imageVal = this._imgURL.val().match(/(ftp:|http:|https:)\/\//g);
                if ((ej.isNullOrUndefined(imageVal) && this._imgURL.val() !== "") || (!ej.isNullOrUndefined(imageVal) && imageVal[0].length < this._imgURL.val().length)) {
                    (this.model.maxLength != null && this.model.maxLength > $.trim(this._getText()).length) ? this._onInsertImage() : this._imgDialog.ejDialog("close");
                    this._onChange();
                    this._setBackupData();
                } else {
                    this._imgURL.addClass("e-error");
                    return false;
                }
            }
        },


        _srcBtnClick: function (sender) {
            if (sender.target.id === "src_update") {
                var _editHTML = $.trim(this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value);
                if (this.model.enableHtmlEncode)
                    _editHTML = this._decode(_editHTML);
                this._getDocument().body.innerHTML = _editHTML;
                this._onChange();
                this._updateCount();               
            }
            this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value = "";
            this._sourceDialog.ejDialog("close");
        },

        _eTblBtnClick: function (sender) {
            if (sender.target.id === this._rteId + "_eTbl_cancel") {
                this._eTblDialog.ejDialog("close");
            }
            else if (sender.target.id === this._rteId + "_eTbl_apply") {
                this._oneditTable();
                this._onChange();
                this._setBackupData();
            }
        },

        _insertCustomTable: function (sender) {
            if (sender.target.id === "insert") {
                var cols = this._customTableDialog.find(".e-rte-txtColumns").ejNumericTextbox("getValue");
                var row = this._customTableDialog.find(".e-rte-txtRows").ejNumericTextbox("getValue");
                var height = this._customTableDialog.find(".e-rte-txtHeight").val();
                var width = this._customTableDialog.find(".e-rte-txtWidth").val();
                var align = this._customTableDialog.find(".e-rte-ddlAlignment").ejDropDownList("getSelectedValue");
                var spacing = this._customTableDialog.find(".e-rte-txtSpacing").val();
                var padding = this._customTableDialog.find(".e-rte-txtPadding").val();
                var border = this._customTableDialog.find(".e-rte-txtBorder").ejDropDownList("getSelectedValue");
                var caption = this._chkTblCaption.ejCheckBox("isChecked");
                if (!ej.isNullOrUndefined(this._tableInsertAt) && !ej.isNullOrUndefined(this._tableInsertAt.startContainer))
                    this._restoreSelection(this._tableInsertAt);
                var tableEle = this._tableGenerator(row, cols, false, width, height, spacing, padding, align, border, caption);
                this.executeCommand('inserthtml', tableEle);
                var _tableEle = $(this._getDocument()).find("table.e-rte-table");
                if(!caption) _tableEle.attr("contentEditable", false);
                _tableEle.find("td").attr("contentEditable", true);
                var drpDwnObj = this._customTableDialog.find(".e-rte-ddlAlignment").data("ejDropDownList");
                if (drpDwnObj.option('value') == "Center")
                    _tableEle.attr('style', "margin: 0 auto");

                this._onChange();
                drpDwnObj.option({ value: "", enabled: false });
            }
            this._customTableDialog.ejDialog("close");
            this._clearTableFields();
        },

        _clearTableFields: function () {
            this._customTableDialog.find(".e-rte-txtColumns").ejNumericTextbox("option", "value", 3);
            this._customTableDialog.find(".e-rte-txtRows").ejNumericTextbox("option", "value", 3);
            this._customTableDialog.find(".e-rte-txtHeight").val("");
            this._customTableDialog.find(".e-rte-txtWidth").val("");
            this._customTableDialog.find(".e-rte-ddlAlignment").ejDropDownList("clearText");
            this._customTableDialog.find(".e-rte-txtSpacing").val("");
            this._customTableDialog.find(".e-rte-txtPadding").val("");
            var border = this._customTableDialog.find(".e-rte-txtBorder").ejDropDownList("clearText");
            var caption = this._chkTblCaption.ejCheckBox({ check: false });
        },

        _clearEditTableFields: function () {
            this._eTblHeight = this._eTblDialog.find("input").val("");
            this._eTblDialog.find(".e-numerictextbox").data("ejNumericTextbox").option("value", "");
            this._eTblDialog.find(".e-rte-dropdown").data("ejDropDownList").option("value", "");
            this._eTblAlign.disable();
            this._eTblCaption.ejCheckBox({ check: false });
        },

        _clearLinkFields: function () {
            this._txtURL = this._linkDialog.find(".e-rte-linkUrl");
            this._txtURL.removeClass("e-error");
            this._txtURL.val("http://");
            this._txtLinkText = this._linkDialog.find(".e-rte-linkText");
            this._txtLinkText.val("");
            this._txtLinkTitle = this._linkDialog.find(".e-rte-linkTitle");
            this._txtLinkTitle.val("");
            this._chkTarget.ejCheckBox({ check: false });
            this._txtURL.focus();
        },
        _clearImgFields: function () {
            this._imgURL = this._imgDialog.find(".e-rte-imgUrl").val("http://").removeClass("e-error");
            this._imgDialog.find(".e-rte-imgText").val("");
            this._chkImgDimensions.ejCheckBox({ checked: true });
            this._imageLink = this._imgDialog.find(".e-rte-imgLink");
            this._imageLink.removeClass("e-error");
            this._imageLink.val("");
            this._imageBorderPx = this._imgDialog.find(".e-rte-imgBorderPx").data('ejNumericTextbox');
            this._imageBorderPx.option('value', 0);
            this._imageBorderStyle = this._imgDialog.find(".e-rte-imgBorderStyle").data('ejDropDownList');
            this._imageBorderStyle.option('value', "");
            this._imgPicker.option('value', null);
            this._imgDialog.find('.e-selected-color').removeAttr('style');
            this._imageStyle = this._imgDialog.find(".e-rte-imgStyle");
            this._imageStyle.val("");
            this._dialogchkTarget.ejCheckBox({ check: false });
        },

        _videoManager: function () {
            var selectNode = this._getSelectedNode();
            this._editVideo = null;
            this._pasteRangeVal = this._getRange();
            this._pasteFlag = true;
            if (selectNode && !ej.isNullOrUndefined(selectNode.tagName) && selectNode.tagName.toLowerCase() === "iframe") {
                this._editVideo = selectNode;
                this._videoWidth = parseInt(selectNode.width);
                this._videoHeight = parseInt(selectNode.height);
                this._videoDialog.find("textarea.e-rte-video").val(selectNode.outerHTML);
                this._videoDialog.find(".e-rte-videoX").val(this._videoWidth);
                this._videoDialog.find(".e-rte-videoY").val(this._videoHeight);
            }
			$("#" + this._rteId + "_video_wrapper").css({"z-index":this._onGetMaxZindex()});
            this._videoDialog.ejDialog("open");
            this._videoDialog.find("textarea.e-rte-video").focus();
        },

        _sourceCodeManager: function () {
            ej.isNullOrUndefined(this._sourceDialog) && this._renderSourceDialog();
            this._updateTagInfo("body");
            this._sourceDialog.find("textarea.e-rte-srctextarea")[0].value = this.getHtml();
			$("#" + this._rteId + "_Source").css({"z-index":this._onGetMaxZindex()});
            this._sourceDialog.ejDialog("open");
            this._sourceDialog.find("textarea.e-rte-srctextarea").focus();
        },
        _hyperLinkManager: function (filebrowser) {
            var selectNode = (this._isIE()) ? this._currentSelNode : this._getSelectedNode();
            this._pasteRangeVal = this._getRange();
            this._pasteFlag = true;
            this._selectedHTML = (ej.browserInfo().name === "msie" && ej.browserInfo().version === "11.0") ? this._pasteRangeVal.toString() : this._getSelText();
            if (filebrowser) {
                this._clearFileFields();
                (!ej.isNullOrUndefined(selectNode) && selectNode.tagName.toUpperCase() == 'A') ? this._fileLinkText.val($(selectNode).html()) : this._fileLinkText.val(this._selectedHTML);
                $("#" + this._rteId + "_fileDialog").css({"z-index":this._onGetMaxZindex()});
                this._fileDialog.ejDialog("open");
            }
            else {
            this._clearLinkFields();
            (!ej.isNullOrUndefined(selectNode) && selectNode.tagName.toUpperCase() == 'A') ? this._txtLinkText.val($(selectNode).html()) : this._txtLinkText.val(this._selectedHTML);
                $("#" + this._rteId + "_link").css({"z-index":this._onGetMaxZindex()});
                this._linkDialog.ejDialog("open");
            }

            if (selectNode && !(/^(A)$/).test(selectNode.nodeName)) {
                if (selectNode.tagName.toUpperCase() == "IMG")
                    selectNode = $(selectNode).parent('a')[0];
            }
            if (filebrowser) {
                if (selectNode && (/^(A)$/).test(selectNode.nodeName)) this._fileURL.val(selectNode.href);
            }
            else if (selectNode && (/^(A)$/).test(selectNode.nodeName)) {
                this._txtURL.val(selectNode.href);
                this._txtLinkTitle.val(selectNode.title);
            }
        },
        _tableManager: function () {
            var tableLi = this._rteToolbar.find("li.e-rteItem-createTable"), Xpos, Ypos;
            this._pasteRangeVal = this._getRange();
            this._pasteFlag = true;
            if (this.model.isResponsive && tableLi.length == 0) {
                var border = $(this._toolBarObj._liTemplte).outerHeight() - $(this._toolBarObj._liTemplte).height();
                tableLi = this._toolBarObj._liTemplte.find("li.e-rteItem-createTable");
                $("#" + this._rteId + "_table_wrapper").css({ "visibility": "hidden", "display": "block" });
                Ypos = ($(this._toolBarObj._liTemplte).offset().top + $(this._toolBarObj._liTemplte).outerHeight() + border - $("#" + this._rteId + "_table_wrapper").offsetParent().offset().top);
                Xpos = ($(tableLi).offset().left - $("#" + this._rteId + "_table_wrapper").offsetParent().offset().left);
                $("#" + this._rteId + "_table_wrapper").css({ "visibility": "visible" });
            }
            else
                Xpos = $(tableLi).position().left, Ypos = $(tableLi).position().top + tableLi.height() + 7;
			if((parseInt(Xpos) + parseInt(this._createTable.ejDialog("model").width)) > document.body.clientWidth) Xpos = (parseInt(Xpos) -(parseInt(this._createTable.ejDialog("model").width) - $(tableLi).width())) + "px";
            this._createTable.ejDialog("option", "position", { X: Xpos, Y: Ypos });
            this._createTable.ejDialog("open");
        },
        _eTblManager: function () {
            this._clearEditTableFields();
            var rteID = this._rteId;
            var selectNode = this._currentSelNode;
            if (!ej.isNullOrUndefined(selectNode) && selectNode.tagName.toLowerCase() === "td") {
                var parentTable = $(selectNode).closest('tbody').closest('table');
                this._getTableObj("Height").val($(parentTable).height());
                this._getTableObj("Width").val($(parentTable).width());
                if(!ej.isNullOrUndefined(parentTable[0])){
                this._getTableObj("CellSpace").data('ejNumericTextbox').option('value', parentTable[0].cellSpacing);
                this._getTableObj("CellPad").data('ejNumericTextbox').option('value', parentTable[0].cellPadding);
                var align = parentTable[0].align;
                }                
                if (!ej.isNullOrUndefined(align) && align != "") {
                    this._eTblAlign.enable();
                    this._eTblAlign.option('value', align);
                }
                var captionEle = $($(selectNode).closest('tbody')).prev('caption');
                if (captionEle.length > 0)
                    this._eTblCaption.ejCheckBox({ checked: true });
                if (!ej.isNullOrUndefined(parentTable[0]) && parentTable[0].style != "") {
                    this._getTableObj("BrdrPx").data('ejNumericTextbox').option('value', parentTable[0].style.borderWidth.split('px')[0]);
                    var obj = this._getTableObj("_border_style").data('ejDropDownList');
                    (parentTable[0].style.borderStyle != "") ? obj.option('value', parentTable[0].style.borderStyle) : obj.option({ selectedItemIndex: 0 });
                    obj.inputElement.val(parentTable[0].style.borderStyle);
                    this._getTableObj("_border_color").val(parentTable[0].style.borderColor);
                }
                this._getTableObj("Style").val(parentTable.attr('style'));


                var parentRow = $(selectNode).closest('tr');
                this._getTableObj("RHeight").val($(parentRow).height());
                this._getTableObj("RAlign").data('ejDropDownList').option('value', parentRow[0].style.textAlign);
                if (parentRow[0].style != "") {
                    this._getTableObj("RBrdrPx").data('ejNumericTextbox').option('value', parentRow[0].style.borderWidth.split('px')[0]);
                    var obj = this._getTableObj("RBrdrStyle").data('ejDropDownList');
                    (parentRow[0].style.borderStyle != "") ? obj.option('value', parentRow[0].style.borderStyle) : obj.option({ selectedItemIndex: 0 });
                    obj.inputElement.val(parentRow[0].style.borderStyle);
                    this._getTableObj("RBrdrColor").val(parentRow[0].style.borderColor);
                }
                this._getTableObj("RStyle").val(parentRow.attr('style'));
                this._getTableObj("ClWidth").val($(selectNode).width());
                this._getTableObj("ClAlign").data('ejDropDownList').option('value', selectNode.style.textAlign);
                if (selectNode.style != "") {
                    this._getTableObj("ClBrdrPx").data('ejNumericTextbox').option('value', selectNode.style.borderLeftWidth.split('px')[0]);
                    var obj = this._getTableObj("ClBrdrStyle").data('ejDropDownList');
                    (selectNode.style.borderStyle != "") ? obj.option('value', selectNode.style.borderStyle) : obj.option({ selectedItemIndex: 0 });
                    obj.inputElement.val(selectNode.style.borderStyle);
                    this._getTableObj("ClBrdrColor").val(selectNode.style.borderColor);
                }
                this._getTableObj("ClStyle").val($(selectNode).attr('style'));
            }
        },

        _getTableObj: function (value) {
            return this._eTblDialog.find('#' + this._rteId + "_eTbl" + value);
        },

        _imageManager: function () {
            this._clearImgFields();
            var selectNode = (this._isIE()) ? this._currentSelNode : this._getSelectedNode();
            this._pasteRangeVal = this._getRange();
            this._pasteFlag = true;
            if (!ej.isNullOrUndefined(selectNode) && selectNode.tagName.toLowerCase() === "img") {
                this._imgWidth = parseInt(selectNode.width);
                this._imgHeight = parseInt(selectNode.height);
                this._imgDialog.find(".e-rte-imgUrl").val(selectNode.src);
                this._imgDialog.find(".e-rte-imgText").val(selectNode.alt);
                this._imgDialog.find(".e-rte-imgX").val(this._imgWidth);
                this._imgDialog.find(".e-rte-imgY").val(this._imgHeight);
                var linkNode = $(selectNode).parent('a');
                if (linkNode.length > 0)
                    this._imageLink.val(linkNode.attr('href'));
                if (selectNode.style != "") {
                    this._imageBorderPx.option('value', selectNode.style.borderWidth.split('px')[0]);
                    this._imageBorderStyle.option('value', selectNode.style.borderStyle);
                    if (selectNode.style.borderColor.length > 0) {
                        if (ej.browserInfo().name === "msie" && ej.browserInfo().version === "8.0") this._imgPicker.option('value', selectNode.style.borderColor);
                        else {
                            var colorCode = selectNode.style.borderColor.split("(")[1].split(")")[0];
                            colorCode = colorCode.split(",");
                            colorCode = { r: $.trim(colorCode[0]), g: $.trim(colorCode[1]), b: $.trim(colorCode[2]) };
                            this._imgPicker.option('value', this._imgPicker.RGBToHEX(colorCode));
                        }
                    }
                }
                this._imageStyle.val($(selectNode).attr('style'));
            }
			$("#" + this._rteId + "_Image_wrapper").css({"z-index":this._onGetMaxZindex()});
            this._imgDialog.ejDialog("open");
            if (ej.isNullOrUndefined(this._explorerObj)) this._imgDialog.find(".e-rte-imgUrl").focus();
        },

        _toolBarClick: function (sender) {
            var $rteEle = $(this.itemsContainer).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            var _itemId = ej.isNullOrUndefined(rteId) ? sender.currentTarget.id.replace("_", "") : sender.currentTarget.id.replace(rteId + "_", "");
            if (($(sender.currentTarget).hasClass("e-rteItem-copy") || $(sender.currentTarget).hasClass("e-rteItem-cut") || $(sender.currentTarget).hasClass("e-rteItem-paste")) && !rteInstcne._isIE())
                rteInstcne._openAlert(rteInstcne._getLocalizedLabels("copyPasteAlert"));
            else {
                if (ej.browserInfo().name != "chrome") {
                    var index = rteInstcne._scriptsItems.indexOf(_itemId);
                    if (rteInstcne._scriptsItems.indexOf(_itemId) >= 0 && $("#" + rteId+ "_" + rteInstcne._scriptsItems[index == 0 ? 1 : 0]).hasClass("e-active"))
                        rteInstcne._selectCommand(rteId+ "_"+ rteInstcne._scriptsItems[index == 0 ? 1 : 0]);
                }
                    var commandName = $.grep(sender.currentTarget.children[0].className.split(/\s+/), function (a) {
                        return !/^e-(rteItem|rte-toolbar-icon)$/i.test(a)
                    });
                if (commandName.indexOf("e-split") >= 0 || commandName.indexOf("e-ddl") >= 0)
                    commandName[0] = _itemId;
                    rteInstcne._selectCommand(commandName[0]);
            }

            if (rteInstcne._styleItems.indexOf(_itemId) >= 0 || rteInstcne._scriptsItems.indexOf(_itemId) >= 0) {
                if (!$(sender.currentTarget).hasClass("e-isactive"))
                    this.selectItemByID(sender.currentTarget.id);
                else
                    this.deselectItemByID(sender.currentTarget.id);
                $(sender.currentTarget).hasClass("e-isactive") ? $(sender.currentTarget).removeClass("e-isactive") : $(sender.currentTarget).addClass("e-isactive");
                if (rteInstcne._scriptsItems.indexOf(_itemId) >= 0) {
                    var index = rteInstcne._scriptsItems.indexOf(_itemId);
                    $("#" + rteId+ "_" + rteInstcne._scriptsItems[index == 0 ? 1 : 0]).removeClass("e-isactive");
                    this.deselectItemByID(rteId+ "_"+ rteInstcne._scriptsItems[index == 0 ? 1 : 0]);
                }
            }


            else if (rteInstcne._alignItems.indexOf(_itemId) >= 0) {
                this.selectItemByID(sender.currentTarget.id);
                var index = rteInstcne._alignItems.indexOf(_itemId)
                for (var i = 0; i < rteInstcne._alignItems.length; i++) {
                    if (i != index)
                        this.deselectItemByID(rteId+ "_"+ rteInstcne._alignItems[i]);
                }
            }

            else if (rteInstcne._listItems.indexOf(_itemId) >= 0) {
                if (!$(sender.currentTarget).hasClass("e-isactive"))
                    this.selectItemByID(sender.currentTarget.id);
                else
                    this.deselectItemByID(sender.currentTarget.id);
                $(sender.currentTarget).hasClass("e-isactive") ? $(sender.currentTarget).removeClass("e-isactive") : $(sender.currentTarget).addClass("e-isactive");
                var index = rteInstcne._listItems.indexOf(_itemId);
                $("#" + rteId+"_" + rteInstcne._listItems[index == 0 ? 1 : 0]).removeClass("e-isactive");
                if (index == 0)
                    this.deselectItemByID(rteId+"_" + rteInstcne._listItems[1]);
                else
                    this.deselectItemByID(rteId+"_" + rteInstcne._listItems[0]);
            }
        },

        _toggleEditTable: function () {
            var selTag = (this._isIE()) ? this._currentSelNode : this._getSelectedNode(), _eTblItems;
            if (ej.isNullOrUndefined(selTag)) return false;
            if (!ej.isNullOrUndefined(this._toolBarItems)) {
                var items = this._rteToolbar.find("li.e-rteItem-addColumnLeft,li.e-rteItem-addColumnRight,li.e-rteItem-addRowAbove,li.e-rteItem-addRowBelow,li.e-rteItem-deleteRow,li.e-rteItem-deleteColumn,li.e-rteItem-deleteTable");
                if (this.model.isResponsive && items.length == 0)
                    _eTblItems = this._toolBarObj._liTemplte.find("li.e-rteItem-addColumnLeft,li.e-rteItem-addColumnRight,li.e-rteItem-addRowAbove,li.e-rteItem-addRowBelow,li.e-rteItem-deleteRow,li.e-rteItem-deleteColumn,li.e-rteItem-deleteTable");
                else _eTblItems = this._toolBarItems.find("li.e-rteItem-addColumnLeft,li.e-rteItem-addColumnRight,li.e-rteItem-addRowAbove,li.e-rteItem-addRowBelow,li.e-rteItem-deleteRow,li.e-rteItem-deleteColumn,li.e-rteItem-deleteTable");
                if (!ej.isNullOrUndefined(selTag.tagName)) {
                    if (selTag && selTag.tagName && selTag.tagName.toLowerCase() == "td" || selTag.tagName.toLowerCase() == "table" || selTag.tagName.toLowerCase() == "tr") {
                        _eTblItems.show();
                        this._createTable.find("div.e-rte-eTblProperties").removeClass('e-disable');
                    }
                    else if (!ej.isNullOrUndefined(this._createTable)) {
                        _eTblItems.hide();
                        this._createTable.find("div.e-rte-eTblProperties").addClass('e-disable');
                    }
                }
            }
            if (!this.model.autoHeight || $(this._toolBarObj.itemsContainer.find(".resize")).length)
                this._setIframeHeight();
        },


        _formatChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFormatBlock(this.getSelectedValue());
            rteInstcne._focus();
            rteInstcne._onChange();
        },


        _fontStyleChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFontName(this.getSelectedValue());
            rteInstcne._onChange();
        },


        _fontSizeChange: function (args) {
            var $rteEle = $(this.element).closest("div.e-rte").find("textarea.e-rte"), rteInstcne = $rteEle.ejRTE("instance"), rteId = $rteEle.attr('id');
            this.getSelectedValue() !== "" && rteInstcne._onFontSize(this.getSelectedValue());
            rteInstcne._onChange();
        },


        _colorPaletteClick: function (e) {
            var value;
            if ((value = e.target.getAttribute('color-code'))) {
                this._focus();
                this._onFontColor(value);
                this._splitMenu.hide(e);
                this._fontColor = value;
                this._fontColorSplit.find("span.e-btntxt").css("background-color", value);
            }
        },


        _bgColorPaletteClick: function (e) {
            var value;
            if ((value = e.target.getAttribute('color-code'))) {
                this._focus();
                this._onBGColor(value);
                this._bgSplitMenu.hide(e);
                this._bgColor = value;
                this._bgColorSplit.find("span.e-btntxt").css("background-color", value);
            }
        },

        _fontColorClick: function (e) {
            this._onFontColor(this._fontColor);
            if (ej.isNullOrUndefined(this._splitMenu)) return false;
            this._splitMenu.hide(e);
        },

        _bgColorClick: function (e) {
            this._onBGColor(this._bgColor);
            if (ej.isNullOrUndefined(this._bgSplitMenu)) return false;
            this._bgSplitMenu.hide(e);
        },

        _selectCommand: function (selectedId) {
                if (!(selectedId == "format" || selectedId == "fontName" || selectedId == "fontSize" || selectedId == "fontColor" || selectedId == "backgroundColor")) {
                this._focus();
            }
            switch (selectedId) {
			        case "fullScreen":
					this._onFullScreen();
					break;
                    case "bold":
                    this._onBold();
                    break;
                    case "italic":
                    this._onItalics();
                    break;
                    case "underline":
                    this._onUnderLine();
                    break;
                    case "strikethrough":
                    this._onStrikeThrough();
                    break;
                    case "justifyLeft":
                    this._onJustifyLeft();
                    break;
                    case "justifyRight":
                    this._onJustifyRight();
                    break;
                    case "justifyCenter":
                    this._onJustifyCenter();
                    break;
                    case "justifyFull":
                    this._onJustifyFull();
                    break;
                    case "cut":
                    this._onCut();
                    break;
                    case "copy":
                    this._onCopy();
                    break;
                    case "paste":
                    this._onPaste();
                    break;
                    case "clearFormat":
                    this._onClearFormat();
                    break;
                    case "clearAll":
                    this._clearAllManager();
                    break;
                    case "orderedList":
                    this._onOrderList();
                    break;
                    case "unorderedList":
                    this._onUnorderList();
                    break;
                    case "undo":
                    this._undo();
                    break;
                    case "redo":
                    this._redo();
                    break;
                    case "indent":
                    this._onIndent();
                    this._indentdepth++;
                    break;
                    case "outdent":
                    this._onOutdent();
                    if (this._indentdepth > 0)
                        this._indentdepth--;
                    break;
                    case "createLink":
                    this._hyperLinkManager(false);
                    break;
                case "removeLink":
                    this._onUnlink();
                    break;
                    case "image":
                    this._imageManager();
                    break;
                    case "createTable":
                    this._tableManager();
                    break;
                    case "addRowAbove":
                    this.insertRow(true);
                    break;
                    case "addRowBelow":
                    this.insertRow(false);
                    break;
                    case "addColumnLeft":
                    this.insertColumn(true);
                    break;
                    case "addColumnRight":
                    this.insertColumn(false);
                    break;
                    case "deleteRow":
                    this.removeRow();
                    break;
                    case "deleteColumn":
                    this.removeColumn();
                    break;
                    case "deleteTable":
                    this.removeTable();
                    break;
                    case "superscript":
                    this._onSuperScript();
                    break;
                    case "subscript":
                    this._onSubScript();
                    break;
                    case "upperCase":
                    this._onUpperCase();
                    break;
                    case "lowerCase":
                    this._onLowerCase();
                    break;
                    case "video":
                    this._videoManager();
                    break;
                    case "fileBrowser":
                    this._hyperLinkManager(true);
                    break;
                default:
                    this._onChange();
                    break;
            }
                if (selectedId != "createLink" && selectedId != "image" && selectedId != "video" && selectedId != "createTable") {
                this._onChange();
            }
                if ((selectedId != "undo") && (selectedId != "redo"))
                this._setBackupData();
            var selectEle = (this._isIE()) ? this._currentSelNode : this._getSelectedNode();
            this._updateTagInfo(selectEle);
        },
        _focus: function () {
            browserInfo = ej.browserInfo();
            if (browserInfo.name == "webkit")
                this._getWindow().focus();
            else
                this._getWindow().document.body.focus();
            this._restoreSelection(this._selectionRange);
        },
        _getWindow: function () {
            var iframe = $($(this._rteWapper.find(".content-iframe"))[0].contentWindow)[0];
            return iframe;
        },
        _getDocument: function () {
            var iframe = this._rteWapper.find(".content-iframe");
            try {
                return iframe.contents()[0];
            }
            catch (err) {
                return "";
            }
        },
        _onFullScreen: function(){
		    if(!$(this._toolBarObj.itemsContainer.find(".resize")).length){
		        this._rteOffset = $(this._rteWapper).position();
			    this._rteWidth = this._rteWapper.width();
			    this._rteHeight = $(this._rteWapper).height();
                this._rtePosition = $(this._rteWapper).css("position");				
		        $(this._rteWapper).css({"top":"0px","left":"0px","width":"100%","height":"100%","position":"fixed","z-index":this._onGetMaxZindex()});
			    $("body").css("overflow", "hidden");
			    $(this._toolBarObj.itemsContainer.find(".maximize")).removeClass("maximize").addClass("resize").parent("li").attr("title",this._getLocalizedLabels("resize"));
			    this._setAutoHeight();
		    }
			else{
		        $(this._rteWapper).css({ "top": this._rteOffset.top + "px", "left": this._rteOffset.left + "px", "width": this._rteWidth + "px", "height": this._rteHeight + "px", "position": this._rtePosition, "z-index": "" });
			    $("body").css("overflow", "");
			    $(this._toolBarObj.itemsContainer.find(".resize")).addClass("maximize").removeClass("resize").parent("li").attr("title",this._getLocalizedLabels("maximize"));
			    this._setAutoHeight();
		    }
			if (this.model.isResponsive) this._toolBarObj._reSizeHandler();
			this._setIframeHeight();
		},
		_onGetMaxZindex: function(){
		    return Math.max.apply(null, 
                  $.map($('body *'), function(e,n) {
                  return parseInt($(e).css('z-index')) + 1 || 1;
            }));
        },
        _onBold: function () {
            var args;
            this.executeCommand('bold', args);
        },

        _onItalics: function () {
            var args;
            this.executeCommand('italic', args);
        },
        _onStrikeThrough: function () {
            var args;
            this.executeCommand('strikethrough', args);
        },

        _onUnderLine: function () {
            var args;
            this.executeCommand('underline', args);
        },

        _onJustifyLeft: function () {
            var args;
            this.executeCommand('JustifyLeft', args);
        },

        _onJustifyRight: function () {
            var args;
            this.executeCommand('JustifyRight', args);
        },

        _onJustifyCenter: function () {
            var args;
            this.executeCommand('JustifyCenter', args);
        },

        _onJustifyFull: function () {
            this._alignFlag = true;
            this.executeCommand('justifyfull');
        },

        _onCut: function () {
            var args;
            this.executeCommand('cut', args);
        },
        _onCopy: function () {
            var args;
            this.executeCommand('copy', args);
        },
        _onPaste: function () {
            var args;
            this.executeCommand('paste', args);

        },
        _onOrderList: function () {
            var args;
            this.executeCommand('insertorderedlist', args);
        },
        _onUnorderList: function () {
            var args;
            this.executeCommand('insertunorderedlist', args);
        },
        _onUndo: function () {
            var args;
            this.executeCommand('undo', null);
        },
        _onDelete: function () {
            var args;
            this.executeCommand('delete', null);
        },

        _onRedo: function () {
            var args;
            this.executeCommand('redo', null);
        },

        _onClearFormat: function () {
            var args;
            this.executeCommand('removeformat', args);

            this._setBackupData();
            this._onChange();
            this._updateIframeSkin(this.model.iframeAttributes);
        },

        _onFormatBlock: function (formatBlock) {
            this._focus();
            var selection, range, node;
            node = document.getSelection ? this._getSelectedNode() : this._getWindow().document.selection.createRange().parentElement();
            this._onClearFormat();
            this.executeCommand('formatBlock', formatBlock);
            if (!ej.isNullOrUndefined(this._tempEle) && !ej.isNullOrUndefined(this._tempEle.get(0))) {
                if ($(this._tempEle).get(0).nodeName == "OL") {
                    $($(this._tempEle).get(0)).find("li").each(function () {
                        $(this).contents().unwrap();
                    });
                }
                else if ($(this._tempEle).get(0).nodeName == "LI") this._tempEle.contents().unwrap();
            }
        },

        _onFontName: function (fontsname) {
            this.executeCommand("fontName", fontsname);
        },

        _onFontSize: function (Size) {
            this.executeCommand('fontSize', Size);
        },

        _onFontColor: function (color) {
            this.executeCommand('foreColor', color);
            this._onChange();
            this._updateIframeSkin(this.model.iframeAttributes);
        },

        _onBGColor: function (color) {
            this.executeCommand('backColor', color);
            this._onChange();
        },

        _onIndent: function () {
            this._getWindow().document.body.style.wordWrap = "break-word";
            var node = document.getSelection ? this._getWindow().document.getSelection().focusNode : this._getWindow().document.selection.createRange().parentElement();
            if (node && (/^(OL|UL|LI)$/).test(node.nodeName))
                this._listIndent(node);
            else if (node && (/^(#text)$/).test(node.nodeName) && (/^(OL|UL|LI)$/).test($(node).parent()[0].nodeName)) {
                this._listIndent($(node).parents("li")[0], node);
            }
            else
                this.executeCommand('indent', false);
            node && $(node).focus();
            this._updateIframeSkin(this.model.iframeAttributes);
        },

        _onOutdent: function () {
            var node = document.getSelection ? this._getWindow().document.getSelection().focusNode : this._getWindow().document.selection.createRange().parentElement();
            if (node && (/^(OL|UL|LI)$/).test(node.nodeName))
                this._listOutdent(node);
            else if (node && (/^(#text)$/).test(node.nodeName) && (/^(OL|UL|LI)$/).test($(node).parent()[0].nodeName)) {
                this._listOutdent($(node).parents("li")[0], node);
            }
            else
                this.executeCommand('outdent', false);
        },

        _onSelectAll: function () {
            this.executeCommand('selectall');
        },
        selectAll: function () {
            this._onSelectAll();
        },
        _onHyperLink: function (val) {
            this.executeCommand('createlink', val);
        },

        _onInsertLink: function () {
            var oEl, sHtml;
            if (!this._isUrl($.trim(this._txtURL.val()))) {
                this._txtURL.addClass("e-error");
                return false;
            }
            var selectedNode = (ej.browserInfo().name == "msie" || ej.browserInfo().name == "chrome") ? this._currentSelNode : this._getSelectedNode();
            if (!ej.isNullOrUndefined(selectedNode) && selectedNode.tagName.toUpperCase() == 'A') {
                selectedNode.href = this._txtURL.val();
                selectedNode.title = this._txtLinkTitle.val();
                selectedNode.innerHTML = this._txtLinkText.val();
                (this._chkTarget.ejCheckBox("isChecked")) ? selectedNode.target  = "_blank":selectedNode.target  = "";
            }
            else if (!ej.isNullOrUndefined(selectedNode) && selectedNode.tagName.toUpperCase() == 'IMG') {
                var parentNode = $(selectedNode).parent('a')[0];
                if (ej.isNullOrUndefined(parentNode)) {
                    oEl = ej.buildTag("a", "", {}, { href: this._txtURL.val(), title: this._txtLinkTitle.val() });
                    (this._chkTarget.ejCheckBox("isChecked")) ? oEl[0].target = "_blank" : oEl[0].target = "";
                    $(selectedNode.outerHTML).appendTo($(oEl));
                    $(selectedNode).replaceWith($(oEl));
                } else {
                    parentNode.href = this._txtURL.val();
                    parentNode.title = this._txtLinkTitle.val();
                    (this._chkTarget.ejCheckBox("isChecked"))? parentNode.target = "_blank" : parentNode.target = "";
                }
            }
            else {
                oEl = ej.buildTag("a", "", {}, { href: this._txtURL.val(), title: this._txtLinkTitle.val() });
                (this._chkTarget.ejCheckBox("isChecked"))?  oEl[0].target = "_blank":oEl[0].target = "";

                var oSelection;
                var oSelRange;
                this._focus();
                if (this._isIE()) {
                    if (this._selectedHTML.length == 0) {
                        if (this._txtLinkText.val() === "") this._selectedHTML = this._txtURL.val();
                        else this._selectedHTML = this._txtLinkText.val();
                    }
                    oEl[0].innerHTML = this._selectedHTML;
                    sHtml = oEl[0].outerHTML;
                }
                else {
                    if (this._txtLinkText.val() === "") oEl.html(this._txtURL.val());
                    else oEl.html(this._txtLinkText.val());
                    sHtml = $('<div>').append($(oEl).clone()).html();
                }
                if (sHtml && (this.model.maxLength != null && (this.model.maxLength > $.trim(this._getText()).length))) {
                    this._isIE() ? this._pasteHtml(sHtml) : this._getDocument().execCommand('inserthtml', false, sHtml);
                    this._focus();
                }
            }
            this._linkDialog.ejDialog("close");
            this.enableToolbarItem("removeLink");
            this._updateCount();           
        },

        _setImageProperties: function (imgEle, imgX, imgY, imageStyle, borderPx, borderStyle, borderColor) {
            if (this.model.showDimensions) {
                (imgX != "") && (imgEle[0].width = imgX);
                (imgY != "") && (imgEle[0].height = imgY);
            }

            imgEle = $(imgEle).attr('style', imageStyle);

            border = borderPx + "px ";
            (!ej.isNullOrUndefined(borderStyle) && borderStyle != "") ? border += borderStyle + " " : border += "solid ";
            (!ej.isNullOrUndefined(borderColor) && borderColor != "") ? border += borderColor : border += "#000000";
            imgEle = imgEle.css('border', border);
        },
        _onInsertImage: function () {
            var url = this._imgDialog.find(".e-rte-imgUrl");
            var altText = this._imgDialog.find(".e-rte-imgText");
            var imgX = this._imgDialog.find(".e-rte-imgX").val();
            var imgY = this._imgDialog.find(".e-rte-imgY").val();
            var proxy = this;
            this._restoreSelection(this._selectionRange);
            var element, border, nodeName = "", imageLink = proxy._imageLink.val(), borderPx = proxy._imageBorderPx.getValue(), borderStyle = proxy._imageBorderStyle.getSelectedValue(), borderColor = proxy._imgPicker.getValue(), imageStyle = proxy._imageStyle.val();
            var selectedNode = (proxy._isIE()) ? proxy._currentSelNode : proxy._getSelectedNode();
            (!ej.isNullOrUndefined(selectedNode)) && (nodeName = selectedNode.nodeName.toUpperCase());
            borderColor = !ej.isNullOrUndefined(borderColor) && borderColor.length > 0 ? borderColor : "";
            var srcUrl = (url.val().startsWith("~")) ? url.val().replace(url.val().substr('0', '1'), "..") : url.val();
            if (nodeName != 'IMG') {
                //Url Validation
                var _img = $("<img>", {
                    src: srcUrl,
                    error: function () { url.addClass("e-error"); },
                    success: function () {
                        var imgEle = ej.buildTag("img", "", {}, { alt: altText.val(), src: srcUrl });
                        proxy._setImageProperties(imgEle, imgX, imgY, imageStyle, borderPx, borderStyle, borderColor);
                        if (imageLink != "") {
                            if (proxy._isUrl($.trim(imageLink))) {
                                var link = ej.buildTag("a", "", {}, { href: imageLink });
                                (proxy._dialogchkTarget.ejCheckBox("isChecked")) ? link[0].target = "_blank" : link[0].target = "_self";
                                $(imgEle[0].outerHTML).appendTo(link);
                                imgEle = link;
                            }
                            else {
                                proxy._imageLink.addClass("e-error");
                                return false;
                            }
                        }
                        var element = imgEle[0].outerHTML;
                        proxy.executeCommand('inserthtml', element);
                    }
                });
            }
            else {
                selectedNode.src = srcUrl;
                selectedNode.alt = altText.val();
                proxy._setImageProperties($(selectedNode), imgX, imgY, imageStyle, borderPx, borderStyle, borderColor);
                var linkNode = $(selectedNode).parent('a');
                if (imageLink != "") {
                    if (proxy._isUrl($.trim(imageLink))) {
                        if (linkNode.length > 0)
                            linkNode.attr('href', imageLink);
                        else {
                            var linkNode = ej.buildTag("a", "", {}, { href: imageLink });
                            $(selectedNode.outerHTML).appendTo(linkNode);
                            $(selectedNode).replaceWith(linkNode);
                        }
                    }
                    else {
                        proxy._imageLink.addClass("e-error");
                        return false;
                    }
                }
                else
                    (linkNode.length > 0) && (linkNode.attr('href', ""));
                if (!ej.isNullOrUndefined(linkNode) && linkNode.length > 0)
                    (this._dialogchkTarget.ejCheckBox("isChecked")) ? linkNode[0].target = "_blank" : linkNode[0].target = "_self";
            }
            proxy._setAutoHeight();
            proxy._imgDialog.ejDialog("close");
        },

        _insertVideo: function (sender) {
            var videoLink = $.trim(this._videoDialog.find("textarea.e-rte-video")[0].value);
            var videoX = this._videoDialog.find(".e-rte-videoX").val();
            var videoY = this._videoDialog.find(".e-rte-videoY").val();
            this._restoreSelection(this._selectionRange);
            if (sender.target.id === "video_insert" && videoLink !== "") {
                var ele = $(videoLink)[0];
                if (this.model.showDimensions && !ej.isNullOrUndefined(ele)) {
                    (videoX != "") && (ele.width = videoX);
                    (videoY != "") && (ele.height = videoY);
                }
                this._editVideo && $(this._editVideo).remove();
                    if (!ej.isNullOrUndefined(ele) && (this.model.maxLength != null && (this.model.maxLength > $.trim(this._getText()).length)))
                    this.executeCommand('inserthtml', ele.outerHTML);

                this._onChange();
                this._setBackupData();
                this._videoDialog.ejDialog("close");
            }
            else if (sender.target.id === "video_cancel")
                this._videoDialog.ejDialog("close");
            else
                this._openAlert(this._getLocalizedLabels("videoError"));
            this._videoDialog.find("textarea.e-rte-video")[0].value = "";
            this._videoDialog.find(".e-rte-videoX").val("");
            this._videoDialog.find(".e-rte-videoY").val("");
            this._chkvideoDimensions.ejCheckBox({ check: true });
            this._setAutoHeight();
        },

        _oneditTable: function () {
            var selectedNode = this._currentSelNode;

            var parentTable = $(selectedNode).closest('tbody').closest('table');
            var captionEle = $($(selectedNode).closest('tbody')).prev('caption');
            (this._eTblCaption.ejCheckBox("isChecked")) ? (captionEle.length == 0) && ej.buildTag('caption').append($('<br type="_moz">')).insertBefore($($(selectedNode).closest('tbody'))) : (captionEle.length > 0) && captionEle.remove();
            var tblAlign = this._eTblAlign.getSelectedValue();
            var style = this._getTableObj("Style").val();
            if (tblAlign == 'center') style += ";margin: 0 auto;";
            (style) && parentTable.attr('style', style);
            parentTable.attr({
                width: this._getTableObj("Width").val(),
                height: this._getTableObj("Height").val(),
                cellPadding: this._getTableObj("CellPad").data('ejNumericTextbox').getValue(),
                cellSpacing: this._getTableObj("CellSpace").data('ejNumericTextbox').getValue(),
                align: tblAlign,
            }).css({
                'border-width': this._getTableObj("BrdrPx").data('ejNumericTextbox').getValue() + "px",
                'border-style': this._getTableObj("_border_style").data('ejDropDownList').getSelectedValue(),
                'border-color': this._getTableObj("_border_color").val()
            });
            var parentRow = $(selectedNode).closest('table').find('tr');
            parentRow.attr({
                style: this._getTableObj("RStyle").val()
            }).css({
                'height': this._getTableObj("RHeight").val(),
                'text-align': this._getTableObj("RAlign").data('ejDropDownList').getSelectedValue(),
                'border-width': this._getTableObj("RBrdrPx").data('ejNumericTextbox').getValue() + "px",
                'border-style': this._getTableObj("RBrdrStyle").data('ejDropDownList').getSelectedValue(),
                'border-color': this._getTableObj("RBrdrColor").val()
            });
            $(selectedNode).attr({
                style: this._getTableObj("ClStyle").val()
            }).css({
                    'width': this._getTableObj("ClWidth").val(),
                'text-align': this._getTableObj("ClAlign").data('ejDropDownList').getSelectedValue(),
                'border-width': this._getTableObj("ClBrdrPx").data('ejNumericTextbox').getValue() + "px",
                'border-style': this._getTableObj("ClBrdrStyle").data('ejDropDownList').getSelectedValue(),
                'border-color': this._getTableObj("ClBrdrColor").val()
            });
            this._eTblDialog.ejDialog("close");
        },


        _clearAllManager: function () {
            this._alertWindow.find(".e-alert-cancel").show();
            this._openAlert(this._getLocalizedLabels("deleteAlert"));
        },

        _onUnlink: function () {
            var args;
            this.executeCommand('unlink', args);
        },
        _onSuperScript: function () {
            var args;
            this.executeCommand('SuperScript', args);
        },
        _onSubScript: function () {
            var args;
            this.executeCommand('SubScript', args);
        },
        _onInsertTable: function (rows, cols, designTime) {
            if (this.model.maxLength != null && (this.model.maxLength > $.trim(this._getText()).length)) {
            var tableEle = this._tableGenerator(rows, cols, designTime);
            this.executeCommand('inserthtml', tableEle);
            this._setAutoHeight();
            }
        },

        _onUpperCase: function () {
            if (!document.getSelection) {
                if (this._getWindow().document.selection.type.toLowerCase() == "control") {
                    return;
                }
                this._seleText = ej.isNullOrUndefined(this._seleText) ? "" : this._seleText;
                this._getDocument().selection.createRange().pasteHTML(this._seleText.toUpperCase());
            }
            else {
                var selection = this._getWindow().getSelection();
                var range = this._saveSelection();
                var node = range.commonAncestorContainer;
                var children = range.commonAncestorContainer.children;
                if (children != null) {
                    var contents = range.cloneContents()
                    var result = this._changeCase(contents, "Upper");
                    var resultCopy = $(result).clone();
                    var tempDiv = document.createElement('div');
                    $(tempDiv).append($(resultCopy));
                    if (tempDiv.innerHTML != null && tempDiv.innerHTML != "")
                        this._getWindow().document.execCommand('inserthtml', false, tempDiv.innerHTML);
                }
                else {
                    var replaceString = node.nodeValue.substring(range.startOffset, range.endOffset).toUpperCase();
                    var startString = node.nodeValue.substr(0, range.startOffset);
                    var endString = node.nodeValue.substring(range.endOffset);
                    node.nodeValue = startString + replaceString + endString;
                }
            }
            this._trigger("execute", { commandName: "uppercase" });
        },

        _changeCase: function (parentNode, toChangeCase) {
            var s = parentNode.childNodes;
            if (s.length > 0) {
                for (var t = 0; t < s.length; t++) {
                    if (s[t].nodeType == 3) {
                        s[t].nodeValue = toChangeCase == "Upper" ? s[t].nodeValue.toUpperCase() : s[t].nodeValue.toLowerCase();
                    }
                    else {
                        if (s[t].nodeType == 1 && s[t].tagName.toUpperCase() != "FONT") {
                            this._changeCase(s[t], toChangeCase);
                        }
                    }
                }
            }
            return s;
        },


        _onLowerCase: function () {
            if (!document.getSelection) {
                if (this._getWindow().document.selection.type.toUpperCase() == "control") {
                    return;
                }
                this._seleText = ej.isNullOrUndefined(this._seleText) ? "" : this._seleText;
                this._getDocument().selection.createRange().pasteHTML(this._seleText.toLowerCase());
            }
            else {
                var selection = this._getWindow().getSelection();
                var range = this._saveSelection();
                var node = range.commonAncestorContainer;
                var children = range.commonAncestorContainer.children;
                if (children != null) {
                    var contents = range.cloneContents();
                    var result = this._changeCase(contents, "Lower");
                    var resultCopy = $(result).clone();
                    var tempDiv = document.createElement('div');
                    $(tempDiv).append($(resultCopy));
                    if (tempDiv.innerHTML != null && tempDiv.innerHTML != "")
                        this._getWindow().document.execCommand('inserthtml', false, tempDiv.innerHTML);
                }
                else {
                    var replaceString = node.nodeValue.substring(range.startOffset, range.endOffset).toLowerCase();
                    var startString = node.nodeValue.substr(0, range.startOffset);
                    var endString = node.nodeValue.substring(range.endOffset);
                    node.nodeValue = startString + replaceString + endString;
                }
            }
            this._trigger("execute", { commandName: "lowercase" });
        },

        _listIndent: function (node, selection) {
            var sibling, newList;
            sibling = node.previousSibling;
            if (sibling && this._isListNode(sibling)) {
                sibling.appendChild(node);
                return true;
            }
            if (sibling && sibling.nodeName == 'LI' && this._isListNode(sibling.lastChild)) {
                sibling.lastChild.appendChild(node);
                this._combineLists(node.lastChild, sibling.lastChild, node);
                return true;
            }
            sibling = node.nextSibling;
            if (sibling && this._isListNode(sibling)) {
                sibling.insertBefore(node, sibling.firstChild);
                return true;
            }
            if (sibling && sibling.nodeName == 'LI' && this._isListNode(node.lastChild)) {
                return false;
            }
            sibling = node.previousSibling;
            if (sibling && sibling.nodeName == 'LI') {
                newList = ej.buildTag(node.parentNode.nodeName)[0];
                sibling.appendChild(newList);
                newList.appendChild(node);
                this._combineLists(node.lastChild, newList, node);
                return true;
            }
            return true;
        },

        _combineLists: function (from, to, li) {
            var node;
            if (this._isListNode(from)) {
                while ((node = li.lastChild.firstChild)) {
                    to.appendChild(node);
                }
                $(from).remove();
            }
        },
        _isListNode: function (node) {
            return node && (/^(OL|UL)$/).test(node.nodeName);
        },
        _isFirstChild: function (node) {
            return node.parentNode.firstChild == node;
        },

        _isLastChild: function (node) {
            return node.parentNode.lastChild == node;
        },
        _removeEmptyList: function (node) {
            if (node && $(node).children().length === 0)
                $(node).remove();
        },
        _listOutdent: function (node) {
            var ul = $(node.parentNode), ulParent = ul[0].parentNode, newBlock;
            node = $(node);
            if (this._isFirstChild(node[0]) && this._isLastChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                    this._removeEmptyList(ulParent);
                    ul.remove();
                } else if (this._isListNode(ulParent)) {
                    ul.remove();
                } else {
                    this.executeCommand('outdent', false);
                }

                return true;
            } else if (this._isFirstChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                    node[0].appendChild(ul[0]);
                    this._removeEmptyList(ulParent);
                } else if (this._isListNode(ulParent)) {
                    ulParent.insertBefore(node[0], ul[0]);
                } else {
                    this.executeCommand('outdent', false);
                }

                return true;
            } else if (this._isLastChild(node[0])) {
                if (ulParent.nodeName == "LI") {
                    node.insertAfter(ulParent);
                } else if (this._isListNode(ulParent)) {
                    node.insertAfter(ul[0]);
                } else {
                    this.executeCommand('outdent', false);
                }
                return true;
            } else {
                this.executeCommand('outdent', false);
            }
            return false;
        },
        _updateIndent: function (node) {
            if (!ej.isNullOrUndefined(this._toolBarObj)) {
                    if (!$("#" + this._rteId + "_" + "indent").hasClass('e-disable')) this._toolBarObj.enableItemByID(this._rteId + "_" + "indent");
                if (node && (/^(LI)$/).test(node.nodeName)) {
                    if ($(node).parents(node.parentNode.nodeName).last().find("li").index(node) === 0)
                            this._toolBarObj.disableItemByID(this._rteId + "_" + "indent");
                }
            }
        },
        _updateCount: function () {
            if (!ej.isNullOrUndefined(this.model)) {
            if (this.model.showFooter && this.model.showWordCount)                
                    this._wordCount.html(this._getLocalizedLabels("words") + " : " + $.trim(this._getText()).split(/\W+/).length);
            if (this.model.showFooter && this.model.showCharCount)
                    this._charCount.html(this._getLocalizedLabels("characters") + " : " + $.trim(this._getText()).length);
            }
        },
        _updateTagInfo: function (element) {
            var text = "", pTag, count = 1;
                if (!ej.isNullOrUndefined(element))
            if (element.localName == "body" || element.localName == "html")
                element = this._getSelectedNode();
            if (ej.isNullOrUndefined(element)) this._curNode = "";
            else this._curNode = element.tagName ? element.tagName.toLowerCase() : "html";
            if (this.model.showFooter && this.model.showHtmlTagInfo) {
                if (this._curNode != "body" && this._curNode != "html" && this._curNode !== "") {
                    var parentNodes = $(element).parents();
                    for (var i = parentNodes.length - 1; i >= 0; i--) {
                        pTag = parentNodes[i].tagName.toLowerCase();
                        if (pTag != "body" && pTag != "html")
                            text += " » " + pTag;
                        else count++;
                    }
                    text += " » " + this._curNode;
                    if ($(element).parents().length > 0) {
                        if (!ej.isNullOrUndefined(parentNodes[parentNodes.length - count]))
                            this._pTag = parentNodes[parentNodes.length - count].tagName.toLowerCase();
                    }
                }
                this._tags = text;
                this._responsiveTagList();
            }
        },
        _responsiveTagList: function () {
            $(this._htmlInfo).html(this._tags);
            if ($(this._rteWapper).outerWidth() < $(this._rteWapper).find('.e-rte-footer-left').outerWidth() + $(this._rteWapper).find('.e-rte-footer-right').outerWidth())
                $(this._htmlInfo).html(" » " + this._pTag + " » ... ...  » " + this._curNode);
            else $(this._htmlInfo).html(this._tags);
        },
        _updateValue: function () {
            var updatedval = this._updateXhtml();
            if (this.model.enableHtmlEncode)
                updatedval = this._encode(updatedval);
            this.value(updatedval);
            this.element.val(updatedval);
        },
        _imageXhtml: function (imgVal) {
            var imgRegexp = /<img(.*?)>/ig;
            var imgTemp;
            var j = 0;
            var imgDup = new Array();
            while ((imgTemp = imgRegexp.exec(imgVal)) !== null) {
                imgDup.push(imgTemp[0].toString());
            }
            var imgEx = new Array();
            imgEx = imgDup.slice(0);
            for (var i = 0; i < imgDup.length; i++) {
                if (imgDup[i].indexOf("/") == -1) {
                    imgDup[i] = imgDup[i].substr(0, imgDup[i].length) + "/" + imgDup[i].substr(imgDup[i].length - 1, imgDup[i].length);
                }
                else if (imgDup[i].lastIndexOf("/") != imgDup[i].length - 2) {
                    imgDup[i] = imgDup[i].substr(0, imgDup[i].length - 1) + "/" + imgDup[i].substr(imgDup[i].length - 1, imgDup[i].length);
                }
            }
            var altReg = /(alt)=("[^"]*")/i;
            var srcReg = /(src)=("[^"]*")/g;
            var altRegS = /(alt)=('[^"]*')/i;
            var srcRegS = /(src)=('[^"]*')/g;
            for (var i = 0; i < imgDup.length; i++) {
                var temp = ej.isNullOrUndefined(altReg.exec(imgDup[i])) ? altRegS.exec(imgDup[i]) : altReg.exec(imgDup[i]);
                var dup = ej.isNullOrUndefined(srcReg.exec(imgDup[i])) ? srcRegS.exec(imgDup[i]) : srcReg.exec(imgDup[i]);
                var srcVar = ej.isNullOrUndefined(imgDup[i].match(srcReg)) ? imgDup[i].match(srcRegS) : imgDup[i].match(srcReg);
                var altVar = ej.isNullOrUndefined(imgDup[i].match(/(alt)=("[^"]*")/g)) ? imgDup[i].match(/(alt)=('[^"]*')/g) : imgDup[i].match(/(alt)=("[^"]*")/g);
                if (temp == null && altVar == null)
                    imgDup[i] = imgDup[i].substr(0, imgDup[i].length - 2) + " alt=\"\"" + imgDup[i].substr(imgDup[i].length - 2, imgDup[i].length);
                else {
                    for (var j = altVar.length - 1; j > 0; j--) {
                        imgDup[i] = imgDup[i].replace(altVar[j], "");
                    }
                }
                if (dup == null && srcVar == null)
                    imgDup[i] = imgDup[i].substr(0, imgDup[i].length - 2) + " src=\"\"" + imgDup[i].substr(imgDup[i].length - 2, imgDup[i].length);
                else {
                    for (var j = srcVar.length - 1; j > 0; j--) {
                        imgDup[i] = imgDup[i].replace(srcVar[j], "");
                    }
                }
            }
            for (var g = 0; g < imgDup.length; g++) {
                imgVal = imgVal.replace(imgEx[g], imgDup[g]);
            }
            return imgVal;
        },
        _tagCssReplace: function (styleTag, styleDup, styleRegval) {
            var tagStyle = /(style)=("[^"]*")/i;
            var tagCss = tagStyle.exec(styleDup);
            if (tagCss != null) {
                var cssTag = tagCss[0];
                if (styleTag != "")
                    cssTag = cssTag.substr(0, cssTag.length - 1) + styleTag + ";\"";
                if (!ej.isNullOrUndefined(styleTag))
                    styleDup = styleDup.replace(styleRegval[0], "").replace(tagCss[0], cssTag);
                return styleDup;
            }
        },
        _replaceHeightWidth: function (htmlTagreg, styleDup) {
            var styleRegval = htmlTagreg.exec(styleDup);
            if (!ej.isNullOrUndefined(styleRegval)) {
                var styleTag = styleRegval[0];
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":");
            }
            return (this._tagCssReplace(styleTag, styleDup, styleRegval));
        },
        _typeXhtmlValidation: function (styleDup) {
            var styleTag = styleRegval[0];
            if (styleTag == "type=\"a\"")
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("type", "list-style-type").replace("a", "lower-alpha");
            else if (styleTag == "type=\"A\"")
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("type", "list-style-type").replace("A", "upper-alpha");
            else if (styleTag == "type=\"1\"")
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("type", "list-style-type").replace("1", "decimal");
            else if (styleTag == "type=\"i\"")
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("type", "list-style-type").replace("i", "lower-roman");
            else if (styleTag == "type=\"I\"")
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("type", "list-style-type").replace("I", "upper-roman");
            else
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("type", "list-style-type");
        },
        _getCssXhtml: function (tagReg, styleDup) {
            var styleRegval = tagReg.exec(styleDup);
            if (!ej.isNullOrUndefined(styleRegval)) {
                var styleTag = styleRegval[0];
                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("valign", "text-align").replace("align", "text-align");
                styleDup = styleDup.replace(styleRegval[0], "style=\"" + styleTag + ";\"");
            }
            return styleDup;
        },
        _styleXhtml: function (styleVal) {
            var styleRegex = /<((?:(?!\/)).)*?>/ig;
            var styleTemp;
            var k = 0;
            var styleDup = new Array();
            var arr = new Array();
            while ((styleTemp = styleRegex.exec(styleVal)) !== null) {
                styleDup.push(styleTemp[0].toString());
            }
            var styleEx = new Array();
            styleEx = styleDup.slice(0);
            for (var i = 0; i < styleDup.length; i++) {
                if (styleDup[i].indexOf("style=") != -1) {
                    if (styleDup[i].indexOf("height=") != -1) {
                        var htmlTagreg = /( height)=("[^"]*")/i;
                        styleDup[i] = this._replaceHeightWidth(htmlTagreg, styleDup[i]);
                        }
                    if (styleDup[i].indexOf("width=") != -1) {
                        var widthTag = /( width)=("[^"]*")/i;
                        styleDup[i] = this._replaceHeightWidth(widthTag, styleDup[i]);
                    }
                    if (styleDup[i].indexOf("type=") != -1) {
                        var typeTag = /( type)=("[^"]*")/i;
                        var styleRegval = typeTag.exec(styleDup[i]);
                        if (!ej.isNullOrUndefined(styleRegval))
                            var styleTag = this._typeXhtmlValidation(styleDup[i]);
                        styleDup[i] = this._tagCssReplace(styleTag, styleDup[i], styleRegval);
                    }
                    if (styleDup[i].indexOf("valign=") != -1) {
                        var valignTag = /( valign)=("[^"]*")/i;
                        var styleRegval = valignTag.exec(styleDup[i]);
                        var styleTag = styleRegval[0];
                        if (styleDup[i].indexOf("vertical-align:") != -1)
                            styleTag = "";
                        else
                            styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("valign", "vertical-align");
                        styleDup[i] = this._tagCssReplace(styleTag, styleDup[i], styleRegval);
                    }
                    if (styleDup[i].indexOf("align=") != -1) {
                        var alignTag = /( align)=("[^"]*")/i;
                        var styleRegval = alignTag.exec(styleDup[i]);
                        if (!ej.isNullOrUndefined(styleRegval)) {
                            var styleTag = styleRegval[0];
                            if (styleDup[i].indexOf("text-align:") != -1)
                                styleTag = "";
                            else
                                styleTag = styleTag.replace(/\"/gi, "").replace("=", ":").replace("align", "text-align");
                        }
                        styleDup[i] = this._tagCssReplace(styleTag, styleDup[i], styleRegval);
                    }
                }
                if (styleDup[i].indexOf("style=") == -1) {
                    if (styleDup[i].indexOf("height=") != -1) {
                        var htmlTagreg = /(height)=("[^"]*")/i;
                        styleDup[i] = this._getCssXhtml(htmlTagreg, styleDup[i]);
                    }
                    if (styleDup[i].indexOf("width=") != -1) {
                        var widthTag = /(width)=("[^"]*")/i;
                        styleDup[i] = this._getCssXhtml(widthTag, styleDup[i]);
                    }
                    if (styleDup[i].indexOf("type=") != -1) {
                        var typeTag = /( type)=("[^"]*")/i;
                        styleDup[i] = this._getCssXhtml(typeTag, styleDup[i]);
                    }
                    if (styleDup[i].indexOf("valign=") != -1) {
                        var valignTag = /(valign)=("[^"]*")/i;
                        styleDup[i] = this._getCssXhtml(valignTag, styleDup[i]);
                    }
                    if (styleDup[i].indexOf("align=") != -1) {
                        var alignTag = /(align)=("[^"]*")/i;
                        styleDup[i] = this._getCssXhtml(alignTag, styleDup[i]);
                    }
                }
            }
            for (var i = 0; i < styleDup.length; i++) {
                styleVal = styleVal.replace(styleEx[i], styleDup[i]);
            }
            return styleVal;
        },
        _getFormatXhtml: function (startRegex, startVal) {
                var startDup = new Array();
                var j = 0;
                var startTemp;
                while ((startTemp = startRegex.exec(startVal)) != null) {
                startDup.push(startTemp[0]);
                }
                for (var i = 0; i < startDup.length; i++) {
                    startVal = startVal.replace(startDup[i], "");
                }
        },
        _startXhtml: function (startVal) {
            if (startVal.indexOf("start=") != -1) {
                var startRegex = /start=\"(.*?)\"/ig;
                startVal = this._getFormatXhtml(startRegex, startVal);
            }
            return startVal;
        },
        _fontXhtml: function (fontVal) {
            if (fontVal.indexOf("<font") != -1) {
                var fontRegex = /<font(.*?)>/ig;
                fontVal = this._getFormatXhtml(fontRegex, fontVal);
            }
            return fontVal;
        },
        _filerXhtmlNode: function (tableRegex, xhtmlVal) {
            var dupArray = new Array();
            var tableTemp;
            while ((tableTemp = tableRegex.exec(xhtmlVal)) != null) {
                dupArray.push(tableTemp[0]);
            }
            return dupArray;
        },
        _replaceXhtmlNode: function (tableDup, xhtmlVal, tableEx) {
            for (var i = 0; i < tableDup.length; i++) {
                xhtmlVal = xhtmlVal.replace(tableEx[i], tableDup[i]);
            }
            return xhtmlVal;
        },
        _tdXhtml: function (xhtmlVal) {
            var tableRegex = /<td((?:(?!<\/td>).))*?<td/ig;
            var tableDup = new Array();
            tableDup = this._filerXhtmlNode(tableRegex, xhtmlVal);
            var tableEx = new Array();
            tableEx = tableDup.slice(0);
            for (var i = 0; i < tableDup.length; i++) {
                var z = tableDup[i].lastIndexOf("<td");
                tableDup[i] = tableDup[i].substr(0, z) + "</td>" + tableDup[i].substr(z, tableDup[i].length);
            }
            xhtmlVal = this._replaceXhtmlNode(tableDup, xhtmlVal, tableEx);
            return xhtmlVal;
        },
        _ptagXhtml: function (htmlText) {
            var ptagRegex1 = /<\/p>((?:(?!<p).))*<\/p>/ig;
            var pTemp1;
            var pEx = new Array();
            if ((pTemp1 = ptagRegex1.exec(htmlText)) != null) {
                while ((pTemp1 = ptagRegex1.exec(htmlText)) != null) {
                    pEx.push(pTemp1[0]);
                }
            }
            var ptagEx = new Array();
            ptagEx = pEx.slice(0);
            for (var i = 0; i < pEx.length; i++) {
                if (pEx[i].indexOf("<p") == -1)
                    pEx[i] = pEx[i].substr(0, pEx[i].length - 4);
            }
            for (var i = 0; i < pEx.length; i++) {
                htmlText = htmlText.replace(ptagEx[i], pEx[i]);
            }
            return htmlText;
        },
        _getUpdaeNodeXhtml: function (tableRegex, xhtmlVal, tableDup) {
            var tableTemp;
            while ((tableTemp = tableRegex.exec(xhtmlVal)) != null) {
                tableDup.push(tableTemp[0]);
            }
            return tableDup;
        },
        _replaceUpdateXhtml: function (tableRegex, xhtmlVal, value) {
            var tableDup = new Array();
            tableDup = this._getUpdaeNodeXhtml(tableRegex, xhtmlVal, tableDup);
            for (var i = 0; i < tableDup.length; i++) {
                xhtmlVal = xhtmlVal.replace(tableDup[i], value);
            }
            return xhtmlVal;
        },
        _replaceXhtmlVal: function (tableDup, xhtmlVal, tableEx) {
            for (var i = 0; i < tableDup.length; i++) {
                xhtmlVal = xhtmlVal.replace(tableEx[i], tableDup[i]);
            }
            return xhtmlVal;
        },
        _replaceTagUpdateXhtml: function (tagRegex, xhtmlVal, value1, value2) {
            var tempArray = new Array();
            tempArray = this._getUpdaeNodeXhtml(tagRegex, xhtmlVal, tempArray);
            var tempEx = new Array();
            tempEx = tempArray.slice(0);
            tempArray[0] = value1 + tempArray[0] + value2;
            return xhtmlVal.replace(tempEx[0], tempArray[0]);
        },
        _replaceTagXHTML: function (styleRegex, xhtmlVal) {
            var styleDup = new Array();
            styleDup = this._getUpdaeNodeXhtml(styleRegex, xhtmlVal, styleDup);
            var styleEx = new Array();
            styleEx = styleDup.slice(0);
            for (var i = 0; i < styleDup.length; i++) {
                styleDup[i] = styleDup[i].replace(/<br\/>/gi, "");
            }
            return this._replaceXhtmlVal(styleDup, xhtmlVal, styleEx);
        },
        _replaceInputParams: function (inputRegex, xhtmlVal) {
            var tableDup = new Array();
            tableDup = this._getUpdaeNodeXhtml(inputRegex, xhtmlVal, tableDup);
            var tableEx = new Array();
            tableEx = tableDup.slice(0);
            for (var i = 0; i < tableDup.length; i++) {
                var divInndex = tableDup[i].indexOf(">");
                tableDup[i] = tableDup[i].substr(0, divInndex) + "/" + tableDup[i].substr(divInndex, tableDup[i].length - 1);
            }
            return this._replaceXhtmlVal(tableDup, xhtmlVal, tableEx);
        },
        _updateXhtml: function () {
            var updateVal = this._getDocument().body.innerHTML;
            if (this.model.enableXHTML) {
                updateVal = updateVal.replace(/hhh/g, "").replace(/<br>/g, "<br/>").replace(/contenteditable="false"/g, "").replace(/contenteditable="true"/g, "").replace(/&nbsp;/g, " ").replace(/<hr>/g, "<hr/>").replace(/cellspacing =[0-9]/g, "").replace(/cellpadding  =[0-9]/g, "");
                this.model.value = updateVal;
                this._xhtmlTagValidation();
                updateVal = this.model.value;
                var xhtmlVal = updateVal.replace(/\s+/g, ' ').replace(/> </gi, "><");
                if (xhtmlVal.indexOf("<?xml") != -1) {
                    var xmlRegexp = /<?xml:(.*?)>/ig;
                    var xmlDup = new Array();
                    var k = 0;
                    while ((xmlTemp = xmlRegexp.exec(xhtmlVal)) != null) {
                        var j = xhtmlVal.indexOf("<?xml");
                        xmlDup[k] = xmlTemp[0];
                        xmlDup[k] = xhtmlVal.substr(j, xmlDup[k].length + 2);
                        xhtmlVal = xhtmlVal.replace(xmlDup[k], "");
                        k++;
                    }
                }
                if (xhtmlVal.indexOf("<table") != -1) {
                    //tabletag
                    var tableRegex = /<table(.*?)<\/table>/ig;
                    var tableDup = new Array();
                    tableDup = this._getUpdaeNodeXhtml(tableRegex, xhtmlVal, tableDup);
                    var tableEx = new Array();
                    tableEx = tableDup.slice(0);
                    for (var i = 0; i < tableDup.length; i++) {
                        tableDup[i] = tableDup[i].replace(/<p>/gi, "").replace(/<\/p>/gi, "");
                        if (tableDup[i].indexOf("<p") != -1) {
                            for (var k = 0; k < tableDup[i].length; k++) {
                                var divInndex = tableDup[i].indexOf("<p");
                                var t = 0;
                                if (divInndex != -1) {
                                    for (var j = divInndex + 4; j < tableDup[i].length - 7; j++) {
                                        if (tableDup[i].charAt(j) == '>') {
                                            if (t == 0) {
                                                tableDup[i] = tableDup[i].substr(0, divInndex) + tableDup[i].substr(j + 1, tableDup[i].length - 1);
                                                t++;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    xhtmlVal = this._replaceXhtmlVal(tableDup, xhtmlVal, tableEx);
                }
                //td tag
                if (xhtmlVal.indexOf("<td") != -1) {
                    for (var i = 0; i < 2; i++) {
                        xhtmlVal = this._tdXhtml(xhtmlVal);
                    }
                }
                //tr tag
                if (xhtmlVal.indexOf("<tr") != -1) {
                    var trtdRegex = /<tr(.*?)>(?!<\/td>)/ig;
                    var tabletrDup = new Array();
                    tabletrDup = this._getUpdaeNodeXhtml(trtdRegex, xhtmlVal, tabletrDup);
                    var tabletrEx = new Array();
                    tabletrEx = tabletrDup.slice(0);
                    for (var i = 0; i < tabletrDup.length; i++) {
                        if (tabletrDup[i].indexOf("</td>") != -1)
                            tabletrDup[i] = tabletrDup[i].replace("</td>", "");
                    }
                    xhtmlVal = this._replaceXhtmlVal(tabletrDup, xhtmlVal, tabletrEx);
                    var tableRegex = /<tr(.*?)<\/tr>/ig;
                    var tableDup = new Array();
                    tableDup = this._getUpdaeNodeXhtml(tableRegex, xhtmlVal, tableDup);
                    var tableEx = new Array();
                    tableEx = tableDup.slice(0);
                    for (var i = 0; i < tableDup.length; i++) {
                        var z = tableDup[i].lastIndexOf("</tr>");
                        var w = tableDup[i].charAt(z - 5) + tableDup[i].charAt(z - 4) + tableDup[i].charAt(z - 3) + tableDup[i].charAt(z - 2) + tableDup[i].charAt(z - 1);
                        if (w != "</td>") {
                            tableDup[i] = tableDup[i].substr(0, z) + "</td>" + tableDup[i].substr(z, tableDup[i].length);
                        }
                    }
                    xhtmlVal = this._replaceXhtmlVal(tableDup, xhtmlVal, tableEx);
                }
                //input tag
                if (xhtmlVal.indexOf("<input") != -1) {
                    var inputRegex = /<input((?:(?!\/).))*?>/ig;
                    xhtmlVal = this._replaceInputParams(inputRegex, xhtmlVal);
                    xhtmlVal = xhtmlVal.replace(/<\/input>/gi, "");
                }
                if (xhtmlVal.indexOf("<param") != -1) {
                    var inputRegex = /<param((?:(?!\/).))*?>/ig;
                    xhtmlVal = this._replaceInputParams(inputRegex, xhtmlVal);
                    xhtmlVal = xhtmlVal.replace(/<\/param>/gi, "");
                }
                if (xhtmlVal.indexOf("<p ") != -1 || xhtmlVal.indexOf("<p>") != -1) {
                    var ptagRegex = /<p(.*)<\/p>/ig;
                    var pDup = new Array();
                    pDup = this._getUpdaeNodeXhtml(ptagRegex, xhtmlVal, pDup);
                    var pEx = new Array();
                    pEx.slice(0);
                    var r = pDup[0].length - 1;
                    var htmlText = pDup[0];
                    if (htmlText.indexOf("<p") != -1) {
                        for (var m = htmlText.indexOf("<p") ; m <= r; m++) {
                            var z = "";
                            var i = 0;
                            if (m < r - 3) {
                                z = htmlText.charAt(m) + htmlText.charAt(m + 1);
                            }
                            if (z == "<p") {
                                for (var s = m + 1; s <= r; s++) {
                                    var w;
                                    if ((w = htmlText.charAt(s) + htmlText.charAt(s + 1) + htmlText.charAt(s + 2) + htmlText.charAt(s + 3)) == "</p>") {
                                        i++;
                                    }
                                    if ((w = htmlText.charAt(s) + htmlText.charAt(s + 1)) == "<p") {
                                        if (i == 0) {
                                            if (htmlText.indexOf("<div") != -1) {
                                                var divSplit = htmlText.substr(m, s - m);
                                                if (divSplit.indexOf("<div") != -1) {
                                                    var divIn = divSplit.indexOf("<div");
                                                    var q = 0;
                                                    var u;
                                                    for (var l = m + divIn + 1; l < htmlText.length - 1; l++) {
                                                        var z = htmlText.charAt(l) + htmlText.charAt(l + 1) + htmlText.charAt(l + 2) + htmlText.charAt(l + 3) + htmlText.charAt(l + 4) + htmlText.charAt(l + 5);
                                                        if (z == "</div>")
                                                            if (q == 0) {
                                                                u = l;
                                                            }
                                                    }
                                                    htmlText = htmlText.substr(0, m + divIn) + "</span></p>" + htmlText.substr(m + divIn, ((-(m + divIn) + u) + 6)) + "<p><span>" + htmlText.substr(u + 6, htmlText.length - 1);
                                                    i++;
                                                }
                                            }
                                        }
                                        if (i == 0) {
                                            if (htmlText.indexOf("<table") != -1) {
                                                var divSplit = htmlText.substr(m, s - m);
                                                if (divSplit.indexOf("<table") != -1) {
                                                    var divIn = divSplit.indexOf("<table");
                                                    var q = 0;
                                                    var u;
                                                    for (var l = m + divIn + 1; l < htmlText.length - 1; l++) {
                                                        var z = htmlText.charAt(l) + htmlText.charAt(l + 1) + htmlText.charAt(l + 2) + htmlText.charAt(l + 3) + htmlText.charAt(l + 4) + htmlText.charAt(l + 5) + htmlText.charAt(l + 6) + htmlText.charAt(l + 7);
                                                        if (z == "</table>")
                                                            if (q == 0) {
                                                                u = l;
                                                            }
                                                    }
                                                    htmlText = htmlText.substr(0, m + divIn) + "</span></p>" + htmlText.substr(m + divIn, ((-(m + divIn) + u) + 8)) + "<p><span>" + htmlText.substr(u + 8, htmlText.length - 1);
                                                    i++;
                                                }
                                            }
                                            if (i == 0) {
                                                htmlText = htmlText.substr(0, s - 1) + "</p>" + htmlText.substr(s, htmlText.length - 1);
                                                r++;
                                                i++;
                                            }
                                        }
                                    }

                                }
                            }
                        }
                    }
                    htmlText = htmlText.replace(/<\/p><\/p>/gi, "</p>");
                    var pDupex = new Array();
                    pDupex = pDup.slice(0);
                    for (var i = 0; i < 2; i++) {
                        htmlText = this._ptagXhtml(htmlText);
                    }
                    var htmlRegex = /<p(.*?)<\/p>/ig;
                    var htmlDup = new Array();
                    htmlDup = this._getUpdaeNodeXhtml(htmlRegex, htmlText, htmlDup);
                    var htmlEx = new Array();
                    htmlEx.slice(0);
                    for (var j = 0; j < htmlDup.length; j++) {
                        if (htmlDup[j].indexOf("<table") != -1) {
                            var tagRegex = /<table(.*)<\/table>/ig;
                            htmlDup[j] = this._replaceTagUpdateXhtml(tagRegex, htmlDup[j], "</span></p>", "<p><span>");
                        }
                        if (htmlDup[j].indexOf("<div") != -1) {
                            var tagRegex = /<div(.*)<\/div>/ig;
                            htmlDup[j] = this._replaceTagUpdateXhtml(tagRegex, htmlDup[j], "</span></p>", "<p><span>");
                        }
                    }
                    htmlText = this._replaceXhtmlVal(htmlDup, htmlText, htmlEx);
                    xhtmlVal = xhtmlVal.replace(pEx[0], htmlText);
                }
                if (xhtmlVal.indexOf("<span") != -1) {
                    var spanRegex = /<span(.*?)<\/span>/ig;
                    //var spanTemp;                    
                    var spanDup = new Array();
                    spanDup = this._getUpdaeNodeXhtml(spanRegex, xhtmlVal, spanDup);
                    var spanEx = new Array();
                    sapnEX = spanDup.slice(0);
                    for (var j = 0; j < spanDup.length; j++) {
                        if (spanDup[j].indexOf("<table") != -1) {
                            var tagRegex = /<table(.*)<\/table>/ig;
                            spanDup[j] = this._replaceTagUpdateXhtml(tagRegex, spanDup[j], "</span>", "<span>");
                        }
                        if (spanDup[j].indexOf("<div") != -1) {
                            var tagRegex = /<div(.*)<\/div>/ig;
                            spanDup[j] = this._replaceTagUpdateXhtml(tagRegex, spanDup[j], "</span>", "<span>");
                        }
                        if (spanDup[j].indexOf("<p") != -1) {
                            var tagRegex = /<p(.*)<\/p>/ig;
                            spanDup[j] = this._replaceTagUpdateXhtml(tagRegex, spanDup[j], "</span>", "<span>");
                        }
                    }
                    xhtmlVal = this._replaceXhtmlVal(spanDup, xhtmlVal, spanEx);
                }
                if (xhtmlVal.indexOf("<ol") != -1) {
                    var styleRegex = /<ol(.*?)<\/ol>/ig;
                    xhtmlVal = this._replaceTagXHTML(styleRegex, xhtmlVal);
                }
                if (xhtmlVal.indexOf("<table") != -1) {
                    var styleRegex = /<table(.*?)><\/table>/ig;
                    xhtmlVal = this._replaceTagXHTML(styleRegex, xhtmlVal);
                }
                if (xhtmlVal.indexOf("style") != -1) {
                    var styleRegex = /style="(.*?)"/ig;
                    xhtmlVal = this._replaceTagXHTML(styleRegex, xhtmlVal);
                }
                //style tag 
                if (!ej.isNullOrUndefined(this._fontXhtml(xhtmlVal)))
                    xhtmlVal = this._fontXhtml(xhtmlVal);
                if (xhtmlVal.indexOf("<br ") != -1 || xhtmlVal.indexOf("<br>") != -1) {
                    var htmlTagreg = /<br(.*?)>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "<br/>");
                }
                if (xhtmlVal.indexOf("<strong ") != -1 || xhtmlVal.indexOf("<strong>") != -1) {
                    var htmlTagreg = /<strong((?:(?!r).))*?>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "<b>").replace(/<\/strong>/gi, "</b>");
                }
                if (xhtmlVal.indexOf("<strong ") != -1) {
                    var htmlTagreg = /<strong (.*?)>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "<b>").replace(/<\/strong>/gi, "</b>");
                }
                if (xhtmlVal.indexOf("<u") != -1) {
                    var htmlTagreg = /<u((?:(?!l).))*?>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "<span style=\"text-decoration: underline;\">").replace(/<\/u>/gi, "</span>");
                }
                if (xhtmlVal.indexOf("<hr") != -1) {
                    var htmlTagreg = /<hr(.*?)>/ig;
                    var htmlTagtemp = new Array();
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "<hr/>");
                }
                if (!ej.isNullOrUndefined(this._startXhtml(xhtmlVal)))
                    xhtmlVal = this._startXhtml(xhtmlVal);
                if (xhtmlVal.indexOf("v:shapes=") != -1) {
                    var htmlTagreg = /v:shapes=\"(.*?)\"/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("v:") != -1) {
                    var htmlTagreg = /<v:(.*?)>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("/v:") != -1) {
                    var htmlTagreg = /<\/v:(.*?)>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("o:") != -1) {
                    var htmlTagreg = /<o:(.*?)>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("/o:") != -1) {
                    var htmlTagreg = /<\/o:(.*?)>/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("role=") != -1) {
                    var htmlTagreg = /role=\"(.*?)\"/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("target=") != -1) {
                    var htmlTagreg = /target=\"(.*?)\"/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("language=") != -1) {
                    var htmlTagreg = /language=\"(.*?)\"/ig;
                    xhtmlVal = this._replaceUpdateXhtml(htmlTagreg, xhtmlVal, "");
                }
                if (xhtmlVal.indexOf("lang=") != -1) {
                    var htmlTagreg = /lang=\"(.*?)\"/ig;
                    htmlTagtemp = this._getUpdaeNodeXhtml(htmlTagreg, xhtmlVal, htmlTagtemp);
                    var height3 = new Array();
                    height3 = htmlTagtemp.slice(0);
                    for (var i = 0; i < htmlTagtemp.length; i++) {
                        xhtmlVal = xhtmlVal.replace(height3[i], htmlTagtemp[i].toLowerCase());
                    }
                }
                xhtmlVal = xhtmlVal.replace(/<\/td><\/td>/gi, "</td>").replace(/<\/th><\/td>/gi, "</th>");
                xhtmlVal = "<div>" + xhtmlVal + "</div>";
                updateVal = xhtmlVal.replace(/&quot;/g, "'").replace(/cellspacing=("[0-9]")/g, '').replace(/cellSpacing=("[0-9]")/g, '').replace(/cellpadding=("[0-9]")/g, '').replace(/cellPadding=("[0-9]")/g, '').replace(/border=("[^"]")/g, '').replace(/<!--(.*?)-->/gm, '').replace(/(vAlign)=("[^"]*")/g, "").replace(/(valign)=("[^"]*")/g, "").replace(/(colspan)=("[^"]*")/g, "").replace(/(colSpan)=("[^"]*")/g, "");
                this.model.value = updateVal;
            }
            return updateVal;
        },
        _setClearFormat: function () {
            var selItemslength = this._rteToolbar.find(".e-active").length;
        },
        //Updates the status of the toolbar items
        _updateToolbarStatus: function () {
            var _isAligned = false;
            if (this.model.showToolbar) {
                //Style Tools
                for (var i = 0; i < this._styleItems.length; i++) {
                    if (this._getCommandStatus($.trim(this._styleItems[i].toLowerCase()))) {
                            this._toolBarObj.selectItemByID(this._rteId + "_" + this._styleItems[i]);
                            $("#" + this._rteId + "_" + this._styleItems[i]).addClass("e-isactive");
                    } else {
                            this._toolBarObj.deselectItemByID(this._rteId + "_" + this._styleItems[i]);
                            $("#" + this._rteId + "_" + this._styleItems[i]).removeClass("e-isactive");
                    }
                }
                for (var i = 0; i < this._alignItems.length; i++) {
                    if (this._getCommandStatus($.trim(this._alignItems[i].toLowerCase()))) {
                            this._toolBarObj.selectItemByID(this._rteId + "_" + this._alignItems[i]);
                        _isAligned = true;
                    } else
                            this._toolBarObj.deselectItemByID(this._rteId + "_" + this._alignItems[i]);
                }
                    if (!_isAligned) this._toolBarObj.selectItemByID(this._rteId + "_" + "justifyLeft");
                for (var i = 0; i < this._scriptsItems.length; i++) {
                    if (this._getCommandStatus($.trim(this._scriptsItems[i].toLowerCase()))) {
                            this._toolBarObj.selectItemByID(this._rteId + "_" + this._scriptsItems[i]);
                            $("#" + this._rteId + "_" + this._scriptsItems[i]).addClass("e-isactive");
                    } else {
                            this._toolBarObj.deselectItemByID(this._rteId + "_" + this._scriptsItems[i]);
                            $("#" + this._rteId + "_" + this._scriptsItems[i]).removeClass("e-isactive");
                    }
                }
                if (this._getCommandStatus('InsertOrderedList')) {
                        this._toolBarObj.selectItemByID(this._rteId + "_" + "orderedList");
                        $("#" + this._rteId + "_" + "orderedList").addClass("e-isactive");
                } else {
                        this._toolBarObj.deselectItemByID(this._rteId + "_" + "orderedList");
                        $("#" + this._rteId + "_" + "orderedList").removeClass("e-isactive");
                } if (this._getCommandStatus('InsertUnorderedList')) {
                        this._toolBarObj.selectItemByID(this._rteId + "_" + "unorderedList");
                        $("#" + this._rteId + "_" + "unorderedList").addClass("e-isactive");
                } else {
                        this._toolBarObj.deselectItemByID(this._rteId + "_" + "unorderedList");
                        $("#" + this._rteId + "_" + "unorderedList").removeClass("e-isactive");
                }
            }
        },

        _updateFormat: function () {
            if (this._formatDDL && this.model.showToolbar && !ej.isNullOrUndefined(this.model.tools["formatStyle"])) {
                var formatName = $.trim(this._getCommandValue('formatblock').replace(/'/g, ""));
                if (!this._isIE())
                    formatName = "<" + formatName + ">";
                if (this._formatDDL.hasClass('e-dropdownlist')) {
                    if (ej.isNullOrUndefined(formatName) || formatName == "")
                        this._formatDDL.ejDropDownList({ "value": this._getLocalizedLabels("format") });
                    else if (!$(this._formatDDL).parents("ul.e-rte-format").length == 0)
                        this._formatDDL.ejDropDownList("setSelectedValue", formatName);
                }
            }
        },

        _updateFontOptionStatus: function () {
            if (this.model.showFontOption && this.model.showToolbar && !ej.isNullOrUndefined(this.model.tools["font"])) {
                var fontName = $.trim(this._getCommandValue('fontname').replace(/"/g, "").replace(/'/g, "").replace(/, /g, ","));
                (ej.isNullOrUndefined(fontName) || fontName == "") && (fontName = "Segoe UI");
                if (!ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.hasClass('e-dropdownlist') && !$(this._fontStyleDDL).parents("ul.e-rte-fontgroup").length == 0)
                    this._fontStyleDDL.ejDropDownList("setSelectedValue", fontName);

                var fontSize = this._getCommandValue('fontsize');
                (ej.isNullOrUndefined(fontSize) || fontSize == "") && (fontSize = 2);
                if (!ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.hasClass('e-dropdownlist') && !$(this._fontSizeDDL).parents("ul.e-rte-fontgroup").length == 0)
                    this._fontSizeDDL.ejDropDownList("setSelectedValue", fontSize);
            }
        },

        _getSelText: function () {
            var selectedtext = '';
            var selection;
            var textnode;
            if (!this._isIE()) {
                if (window.getSelection) {
                    selection = this._getWindow().getSelection();
                    var range = this._saveSelection();
                    textnode = document.createTextNode(range.toString());
                    selectedtext = textnode.nodeValue;
                }
                else if (document.getSelection) {
                    selectedtext = this._getWindow().document.getSelection();
                }
            }
            else
                selectedtext = this._seleText;
            return selectedtext;
        },

        _isIE: function () {
            var _ie = false, browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie') {
                _ie = true;
            }
            return _ie;
        },

        _isIE8: function () {
            var _ie8 = false, browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie' && browserInfo.version == "8.0") {
                _ie8 = true;
            }
            return _ie8;
        },
        //Vaidate the input URL string
        _isUrl: function (url) {
            var regexp = /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/;
            return regexp.test(url);
        },
        _pasteHtml: function (html) {
            var sel, range;
            this._focus();
            if (window.getSelection) {
                // IE9 and non-IE
                sel = this._getWindow().getSelection();
                if (sel.getRangeAt && sel.rangeCount) {
                    range = this._pasteFlag ? this._pasteRangeVal : sel.getRangeAt(0);
                    var cnode = range.commonAncestorContainer, parentElement, i, container;
                    parentElement = cnode.nodeType == 3 ? $(cnode).parentsUntil("body") : $(cnode).children();
                    for (i = parentElement.length - 1; i >= 0; i--) {
                        if (parentElement[i].textContent == range.toString()) {
                            range.selectNode(parentElement[i]);
                            break;
                        }
                    }
                    range.deleteContents();
                    // some browsers (IE9, for one)
                    var el = document.createElement("div");
                    el.innerHTML = html;
                    var frag = this._getDocument().createDocumentFragment(), node, lastNode;
                    var parent = el.firstChild;                    
                    if (!(this._pasteFlag || this._pasteAPIFlag)) {
                        while ((node = el.firstChild)) {
                            lastNode = frag.appendChild(node);
                        }
                    }
                    else {
                        while ((node = el.firstChild)) {
                            if (parent != node)
                                lastNode = parent.appendChild(node);
                            else                              
                                el.removeChild(el.firstChild)
                            this._pasteAPIFlag = this._pasteFlag = false;
                        }
                        lastNode = frag.appendChild(parent);
                    }
                    range.insertNode(frag);
                    // Preserve the selection
                    if (lastNode) {
                        range = range.cloneRange();
                        range.setStartAfter(lastNode);
                        range.collapse(true);
                        sel.removeAllRanges();
                        sel.addRange(range);
                    }
                }
            } else if (this._getDocument().selection && this._getDocument().selection.type != "Control") {
                // IE < 9
                if (this._ieSelectionRange && !(this._ieSelectionRange.offsetLeft == 0 && this._ieSelectionRange.offsetTop == 0 && this._ieSelectionRange.boundingHeight == 0)) {
                    if (this._pasteFlag)
                        this.selectRange(this._pasteRangeVal);
                    var ieParentElement = this._ieParents(this._getRange()), i, isParent = false;
                    if (this._pasteAPIFlag)
                        this._getRange().pasteHTML(html);
                    else {
                        var text = this._getRange().text;
                    for (i = ieParentElement.length - 1; i >= 0; i--) {
                            if (ieParentElement[i].innerText == text) {
                            ieParentElement[i].nodeName == "BODY" ? ieParentElement[i].innerHTML = html : ieParentElement[i].outerHTML = html;
                            isParent = true;
                            break;
                        }
                    }
                    if (!isParent)
                            this._getRange().pasteHTML(html);
                }
                }
                else
                    this._getDocument().body.insertAdjacentHTML("afterBegin", html);
            }
            this._pasteAPIFlag = this._pasteFlag = false;
        },
        _ieParents: function (range) {
            var parentElement = [];
            rangeval = this._isIE8() ? range.duplicate() : range.cloneRange();
            range.parentElement = rangeval.parentElement();
            while (range.parentElement.nodeName != "HTML") {
                if (range.parentElement.nodeName == "TD" || range.parentElement.nodeName == "TH")
                    break;
                parentElement.push(range.parentElement);
                range = range.parentElement;
            }
            return parentElement;
        },

        _getLocalizedLabels: function (property) {
            return ej.RTE.Locale[this.model.locale][property] === undefined ? ej.RTE.Locale["en-US"][property] : ej.RTE.Locale[this.model.locale][property];
        },


        _getSelectedNode: function () {
            var temp = this._getRange();
            var element, rng = (this._ieSelectionRange && temp.offsetLeft == 0 && temp.offsetTop == 0 && temp.boundingHeight == 0) ? this._ieSelectionRange : temp;
            var startContainer = rng.startContainer, endContainer = rng.endContainer;
            var startOffset = rng.startOffset, endOffset = rng.endOffset;
            if (!ej.isNullOrUndefined(rng.commonAncestorContainer))
                if (rng.commonAncestorContainer.nodeName == "BODY")
                    this._tempEle = $(rng.commonAncestorContainer);
                else if ($(rng.cloneContents()).text() == $($($(rng.startContainer).parents("li")).contents()[0]).text() && (startOffset == 0 && endOffset == $(rng.startContainer).parents("li").text().length) || startOffset == endOffset || startOffset == 0 && endOffset == 1)
                    this._tempEle = $($(rng.startContainer).parents("li"));
                else if ((rng.commonAncestorContainer.nodeName == "OL") && startOffset != endOffset && endOffset != 0 && endOffset != 1)
                    this._tempEle = $(rng.commonAncestorContainer);
                else
                    this._tempEle = $(rng.startContainer);
            if (!rng) {
                return this._getDocument().selection;
            }
            if (rng.setStart) {
                element = rng.commonAncestorContainer;
                //Image tage and link selection
                if (!rng.collapsed) {
                    if (startContainer == endContainer) {
                        if (endOffset - startOffset < 2) {
                            if (startContainer.hasChildNodes()) {
                                element = startContainer.childNodes[startOffset];
                            }
                        }
                    }
                    if (startContainer.nodeType === 3 && endContainer.nodeType === 3) {
                        if (startContainer.length === startOffset) {
                            startContainer = this._skipEmptyNode(startContainer.nextSibling, true);
                        } else {
                            startContainer = startContainer.parentNode;
                        }

                        if (endOffset === 0) {
                            endContainer = this._skipEmptyNode(endContainer.previousSibling, false);
                        } else {
                            endContainer = endContainer.parentNode;
                        }

                        if (startContainer && startContainer === endContainer) {
                            return startContainer;
                        }
                    }
                }

                if (element && element.nodeType == 3) {
                    return element.parentNode;
                }
                return element;
            }
            element = rng.item ? rng.item(0) : rng.parentElement();
            return element;
        },

        _skipEmptyNode: function (node, forwards) {
            var orig = node;
            while (node && node.nodeType === 3 && node.length === 0) {
                node = forwards ? node.nextSibling : node.previousSibling;
            }
            return node || orig;
        },

        _getSelection: function () {
            var selCurrent;
            if (this._isIE()) {
                selCurrent = this._getDocument().selection.createRange();
                selCurrent.type = this._getDocument().selection.type;
            }
            else {
                selCurrent = window.getSelection();
                if (selCurrent.rangeCount) {
                    sText = selCurrent.getRangeAt(0);
                }
                selCurrent.type = this._getWindow().getSelection().type;
            }
            return selCurrent;
        },

        _saveSelection: function () {
            var win = this._getWindow();
            var doc = win.document;
            var sel = win.getSelection ? win.getSelection() : doc.selection;
            var range;

            if (sel) {
                if (sel.createRange) {
                    range = sel.createRange();
                } else if (sel.rangeCount > 0 && sel.getRangeAt(0)) {
                    range = sel.getRangeAt(0);
                } else if (sel.anchorNode && sel.focusNode && doc.createRange) {
                    // Older WebKit browsers
                    range = doc.createRange();
                    range.setStart(sel.anchorNode, sel.anchorOffset);
                    range.setEnd(sel.focusNode, sel.focusOffset);
                    if (range.collapsed !== sel.isCollapsed) {
                        range.setStart(sel.focusNode, sel.focusOffset);
                        range.setEnd(sel.anchorNode, sel.anchorOffset);
                    }
                } else {
                    var iframe = document.getElementsByTagName("iframe")[0];
                    var win = iframe.contentWindow;
                    range = win.document.createRange();
                    range.setStart(win.document.body, 0);
                    range.setEnd(win.document.body, 0);
                }
            }
            return range;
        },

        _restoreSelection: function (range) {
            var win = this._getWindow();
            var doc = win.document;
            var sel = win.getSelection ? win.getSelection() : doc.selection;
            if (sel && range) {
                if (range.select) {
                    range.select();
                } else if (sel.removeAllRanges && sel.addRange) {
                    sel.removeAllRanges();
                    sel.addRange(range);
                }
            }
        },

        _getRange: function () {
            if (this._isIE8()) {
                var temp = this._saveSelection();
                var rng = (this._ieSelectionRange && temp.offsetLeft == 0 && temp.offsetTop == 0 && temp.boundingHeight == 0) ? this._ieSelectionRange : temp;
                return rng;
            }
            else if (this._isIE())
			//Table Aciton Not working in IE condition (this._selectionRange.length>0)
                return this._ieSelection = (window.getSelection && (ej.isNullOrUndefined(this._selectionRange) ? false : (this._selectionRange.length > 0))) ? this._selectionRange : this._saveSelection();
            else if (this._getWindow().getSelection().rangeCount > 0)
                return this._getWindow().getSelection().getRangeAt(0);
            else {
                var iframe = document.getElementsByTagName("iframe")[0];
                var win = iframe.contentWindow;
                var range = win.document.createRange();
                range.setStart(win.document.body, 0);
                range.setEnd(win.document.body, 0);
                return range;
            }
        },

        _setBackupData: function () {
            var htmlTxt = this.getHtml();
            var bkuplen = this._backupArray.length;
            if (bkuplen == this.model.undoStackLimit + 1)
                this._backupArray.splice(0, 1);
            if (htmlTxt != this._backupArray[bkuplen - 1] && this.model.undoStackLimit > 0)
                this._backupArray.push(htmlTxt);
            this._undoRedoPosition = this._backupArray.length - 1;
                if (this.model.showToolbar && this._rteToolbar.find("li.e-rteItem-undo") && this._backupArray.length > 1 && !$("#" + this._rteId + "_" + "undo").hasClass("e-rteTooldisable"))
                    this._toolBarObj.enableItemByID(this._rteId + "_" + "undo");
        },
        _getSelectedHtmlString: function () {
            if (typeof document.selection != "undefined" && this._isIE8()) {
                var ieParentElement = this._ieParents(this._getRange()), i, isParent = false, html;
                for (i = ieParentElement.length - 1; i >= 0; i--) {
                    if (ieParentElement[i].innerText == this._getRange().text) {
                        html = ieParentElement[i].nodeName == "BODY" ? ieParentElement[i].innerHTML : ieParentElement[i].outerHTML;
                        isParent = true;
                        break;
                    }
                }
                if (!isParent)
                    html = this._getRange().htmlText.substring(0, 1) == "<" ? this._getRange().text : this._getRange().htmlText;
                return html;
            }
            else {
                var range = this._getRange(), cnode = range.commonAncestorContainer, parentElement, i, container;
                parentElement = cnode.nodeType == 3 ? $(cnode).parentsUntil("body") : $(cnode).children();
                for (i = parentElement.length - 1; i >= 0; i--) {
                    if (parentElement[i].textContent == range.toString()) {
                        range.selectNode(parentElement[i]);
                        break;
                    }
                }
                container = document.createElement("div");
                container.appendChild(range.cloneContents());
                return container.innerHTML;
            }
        },

        _getText: function () {
            var _body = this._getDocument().body;
            if (_body != null) {
                if (!this._isIE())
                    return _body.textContent;
                else
                    return _body.innerText;
            }
        },
        _validateMaxLength: function (e) {
            if (this.model.maxLength != null) {
                var len = this.model.maxLength;
                var usertext = this._getText();
                if (len <= usertext.length) {
                    if (!(e.keyCode < 47 || e.ctrlKey && e.keyCode == 65 || e.ctrlKey && e.keyCode == 67 || e.ctrlKey && e.keyCode == 86 || e.ctrlKey && e.keyCode == 88)) {
                        this._keypressFlag = false;
                        this._cancelEvent(e);
                    }
                }
            }
        },
        _cancelEvent: function (e) {
            e.returnValue = false;
            e.stopPropagation();
            e.preventDefault();
            return false;
        },

        _alignToolUpdate: function (name) {
            for (var i = 0; i < this._alignItems.length; i++) {
                if (this._alignItems[i] === name)
                        this._toolBarObj.selectItemByID(this._rteId + "_" + name);
                else
                        this._toolBarObj.deselectItemByID(this._rteId + "_" + this._alignItems[i]);
            }
        },


        _getCommandStatus: function (commandName) {
            var state = this._getDocument().queryCommandState(commandName);
            return state;
        },


        _getCommandValue: function (commandName) {
            var value = this._getDocument().queryCommandValue(commandName);
            if (this._isIE()) {
                if (commandName == "fontname") {
                    for (var i = 0; i < this.model.fontName.length; i++) {
                        if (value == this.model.fontName[i].text) {
                            value = this.model.fontName[i].value;
                            break;
                        }
                    }
                    if (ej.isNullOrUndefined(value) || value == "")
                        value = this.model.fontName[0].value;
                }
                if (commandName == "formatblock") {
                    for (var i = 2; i < this.model.format.length; i++) {
                        if (value == this.model.format[i].text) {
                            value = this.model.format[i].value;
                            break;
                        }
                    }
                    if (value == "Normal")
                        value = this.model.format[0].value;
                }
            }
            return value;
        },


        _undo: function () {
            if (this._undoRedoPosition > 0) {
                if (this.model.enableHtmlEncode)
                    this.setHtml(this._decode(this._backupArray[this._undoRedoPosition - 1]));
                else
                this.setHtml(this._backupArray[this._undoRedoPosition - 1]);
                this._undoRedoPosition--;
                this._updateToolbarStatus();
                this._updateFontOptionStatus();
                this._updateFormat();
                this._updateCount();
                this._toolBarObj.enableItemByID(this._rteId + "_" + "redo");
            }
            if (this._undoRedoPosition == 0) {
                    this._toolBarObj.disableItemByID(this._rteId + "_" + "undo");
                    this._toolBarObj.enableItemByID(this._rteId + "_" + "redo");
            }
            this._focus();
            this._trigger("execute", { commandName: "undo" });
        },
        _getRedoContent: function(){
            if (this.model.enableHtmlEncode)
                return (this._decode(this._backupArray[this._undoRedoPosition + 1]));
            else
                return (this._backupArray[this._undoRedoPosition + 1]);
        },

        _redo: function () {
            if (this._undoRedoPosition != this._backupArray.length) {
                if (this._backupArray[this._undoRedoPosition + 1] != null) {
                    var _redoContent = this._getRedoContent();
                    this.setHtml(_redoContent)
                    this._undoRedoPosition++;
                }
                else {                    
                    var redoString = this._getRedoContent();
                    if (!ej.isNullOrUndefined(redoString)) this.setHtml(redoString);
                    this._undoRedoPosition = this._backupArray.length - 2;
                }
                this._updateToolbarStatus();
                this._updateFontOptionStatus();
                this._updateFormat();
                this._updateCount();                   
                if (!ej.isNullOrUndefined(this._toolBarObj)) {
                    if (this._backupArray.length - 1 > 0)
                            this._toolBarObj.enableItemByID(this._rteId + "_" + "undo");
                    if (this._undoRedoPosition == this._backupArray.length - 1)
                            this._toolBarObj.disableItemByID(this._rteId + "_" + "redo");
                }
            }
            this._focus();
            this._trigger("execute", { commandName: "redo" });
        },


        _setContent: function (htmltxt) {
            var editdoc = this._getDocument();
            if (editdoc != "" && !ej.isNullOrUndefined(editdoc)) {
                editdoc.open();
                editdoc.write(htmltxt);
                editdoc.close();
                editdoc.EditMode = true;
            }
        },


        _tableCellStart: function (e) {
            this._tableInsertAt = this._saveSelection();
        },


        _tableCellSelect: function (e) {
            var cellEle, target = $(e.target);
            var row = target.parent().index();
            var col = target.index();
            this._createTable.find("div.e-rte-tablecell").each(function (index) {
                cellEle = $(this);
                if (cellEle.index() <= col && cellEle.parent().index() <= row) {
                    cellEle.addClass("e-active");
                }
                else {
                    cellEle.removeClass("e-active");
                }
            });
            this._tblheaderDiv.html((col + 1) + "x" + (row + 1) + " Table");
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            if (!ej.isNullOrUndefined(this._tableInsertAt))
                this._restoreSelection(this._tableInsertAt);
        },
        _tableCellLeave: function (e) {
            this._createTable.find("div.e-rte-tablecell").removeClass("e-active");
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            this._tblheaderDiv.html(this._getLocalizedLabels("createTable"));
        },
        _tableCellDown: function (e) {
            var target = $(e.target);
            var row = target.parent().index() + 1;
            var col = target.index() + 1;
            this._focus();
            $(this._getDocument()).find("table.e-rte-tableremove").remove();
            if (!ej.isNullOrUndefined(this._tableInsertAt)) {
                this._restoreSelection(this._tableInsertAt);
                this._onInsertTable(row, col, false);
            }
            var _tableEle = $(this._getDocument()).find("table.e-rte-table");
                //Content editable propert for table and td removed and key navigation support added in keydown           
            this._createTable.ejDialog("close");
            this._setBackupData();
            this._onChange();
            if (this.model.isResponsive && this._toolBarObj._liTemplte) {
                this._toolBarObj._liTemplte.addClass("e-display-none");
                this._toolBarObj.contstatus = false;
            }
        },

        _tableGenerator: function (rows, cols, designTime, width, height, spacing, padding, align, border, caption) {
            var tblClass = "e-rte-table", colTemplate = "<td><br _moz_dirty=''/></td>";
            if (designTime)
                tblClass += " e-rte-tableremove";
            width = ej.isNullOrUndefined(width) || width == "" ? "width='100%'" : "width=" + width + "'";
            height = ej.isNullOrUndefined(height) || height == "" ? "" : "height='" + height + "'";
            spacing = ej.isNullOrUndefined(spacing) || spacing == "" ? "" : "cellspacing='" + spacing + "'";
            padding = ej.isNullOrUndefined(padding) || padding == "" ? "" : "cellpadding='" + padding + "'";
            align = ej.isNullOrUndefined(align) ? "" : " align='" + align + "'";
            switch (border) {

                case "dotted": borderstyle = "dotted";
                    break;
                case "double":
                    this._border = "style='border:3px double #5C5C5C'";
                    borderstyle = "double";
                    return "<table border='1' class='" + tblClass + "' " + width + " " + height + " " + this._border + " " + spacing + " " + padding + " " + align + ">" + (caption ? "<caption></caption>" : "") + Array(rows + 1).join("<tr>" + Array(cols + 1).join("<td><br _moz_dirty=''/></td>") + "</tr>") + "</table>";
                    break;
                case "dashed": borderstyle = "dashed";
                    break;
                case "solid": borderstyle = "solid";
                    break;
                default: borderstyle = "solid";
                    break;
            }
            if (borderstyle != "double") {
                this._border = "style='border:1px " + borderstyle + "#5C5C5C'";
                return "<table border='1' class='" + tblClass + "' " + width + " " + height + " " + this._border + " " + spacing + " " + padding + " " + align + ">" + (caption ? "<caption><br type=\"_moz\"></caption>" : "") + Array(rows + 1).join("<tr>" + Array(cols + 1).join("<td><br _moz_dirty=''/></td>") + "</tr>") + "</table>";
            }

            return "<table border='1' class='" + tblClass + "' " + width + " " + height + " " + this._border + " " + spacing + " " + padding + " " + align + ">" + (caption ? "<caption><br type=\"_moz\"></caption>" : "") + Array(rows + 1).join("<tr>" + Array(cols + 1).join(colTemplate) + "</tr>") + "</table>";
        },

        _openCustomTable: function (e) {
            if (e.currentTarget.id == this._rteId + "_createCustomTableLink") {
			$("#" + this._rteId + "_customTable").css({"z-index":this._onGetMaxZindex()});
                this._customTableDialog.ejDialog("open");
			}
            else {
                if (!this._createTable.find("div.e-rte-eTblProperties").hasClass('e-disable')) {
                    this._eTblManager();
					$("#" + this._rteId + "_eTbl").css({"z-index":this._onGetMaxZindex()});
                    this._eTblDialog.ejDialog("open");
                    this._on(this._getTableObj("Width"), "focusout", this._widthFocusOut);
                }
            }
        },
        _documentClick: function (e) {
            if ((!ej.isNullOrUndefined(this._createTable)) && ($(e.target).parents("#" + this._rteId + "_" + "tables").length <= 0)) {
                if (!ej.isNullOrUndefined(this._createTable.ejDialog("instance")))
                this._createTable.ejDialog("isOpened") && this._createTable.ejDialog("close");
                this._createTable.find("div.e-rte-eTblProperties").addClass('e-disable');
            }
            else
                $(this._createTable.find("div.e-rte-eTblProperties").hasClass('e-disable')).addClass('e-disable');
            if (this._isIE() && !window.getSelection) {
                var checkdropdown = e.target.id;
                if (checkdropdown == "")
                    checkdropdown = e.target.parentNode.id;
                if (checkdropdown == (this._rteId + "_fontSize") || checkdropdown == (this._rteId + "_fontName") || checkdropdown == (this._rteId + "_format") || checkdropdown == (this._rteId + "_fontSize_dropdown") || checkdropdown == (this._rteId + "_fontName_dropdown") || checkdropdown == (this._rteId + "_format_dropdown"))
                    this._restoreSelection(this._selectionRange);
            }
        },


        _iframeFocus: function (e) {
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog("isOpened") && this._createTable.ejDialog("close");
            !$(this._fontSizeDDL).parents("ul.e-rte-fontgroup").length == 0 && !ej.isNullOrUndefined(this._fontSizeDDL) && this._fontSizeDDL.hasClass("e-dropdownlist") && this._fontSizeDDL.ejDropDownList("hidePopup");
            !$(this._fontStyleDDL).parents("ul.e-rte-fontgroup").length == 0 && !ej.isNullOrUndefined(this._fontStyleDDL) && this._fontStyleDDL.hasClass("e-dropdownlist") && this._fontStyleDDL.ejDropDownList("hidePopup");
            !$(this._formatDDL).parents("ul.e-rte-format").length == 0 && !ej.isNullOrUndefined(this._formatDDL) && this._formatDDL.hasClass("e-dropdownlist") && this._formatDDL.ejDropDownList("hidePopup");
            !ej.isNullOrUndefined(this._createTable) && this._createTable.ejDialog("close");
            !ej.isNullOrUndefined(this._splitMenu) && this._splitMenu.hide(e);
            !ej.isNullOrUndefined(this._bgSplitMenu) && this._bgSplitMenu.hide(e);
            this._contentBeforfocus = this.value();
        },
        _updateKeyDownSelectStatus: function (_toolEle, commandName) {
            if (!_toolEle.hasClass("e-active"))
                this._toolBarObj.selectItemByID(this._rteId + "_" + commandName);
            else
                this._toolBarObj.deselectItemByID(this._rteId + "_" + commandName);
        },
        _iframeKeyDown: function (e) {
            var _toolEle;
            if (this.model.showToolbar && this.model.allowKeyboardNavigation) {
                if (e && e.ctrlKey == true) {
                    e.keyCode == 65 ? this._contolAKeyfalg = true : ej.isNullOrUndefined(this._contolAKeyfalg) ? this._contolAKeyfalg : this._contolAKeyfalg = false;
                    if (e.keyCode == 66) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-bold");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onBold();
                            this._updateKeyDownSelectStatus(_toolEle, "bold");
                        }
                        return false;
                    }
                    else if (e.keyCode == 73) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-italic");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onItalics();
                            this._updateKeyDownSelectStatus(_toolEle, "italic");
                        }
                        return false;
                    }
                    else if (e.keyCode == 85) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-underline");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onUnderLine();
                            this._updateKeyDownSelectStatus(_toolEle, "underline");
                        }
                        return false;
                    }
                    else if (e.keyCode == 76 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-justifyLeft");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyLeft();
                            this._alignToolUpdate("justifyLeft");
                        }
                        return false;
                    }
                    else if (e.keyCode == 82) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-justifyRight");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyRight();
                            this._alignToolUpdate("justifyRight");
                        }
                        return false;
                    }
                    else if (e.keyCode == 69 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-justifyCenter");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyCenter();
                            this._alignToolUpdate("justifyCenter");
                        }
                        return false;
                    }
                    else if (e.keyCode == 74 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-justifyFull");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onJustifyFull();
                            this._alignToolUpdate("justifyFull");
                        }
                        return false;
                    }
                    else if (e.keyCode == 90 && !e.altKey) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-undo");
                        _toolEleRedo = this._toolBarItems.find("li.e-rteItem-redo");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onUndo();
                            _toolEleRedo.removeClass("e-disable");
                            this._toolBarObj.deselectItemByID(this._rteId + "_redo")
                        }
                        return false;
                    }
                    else if (e.keyCode == 89) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-redo");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._onRedo();
                            this._toolBarItems.find("li.e-rteItem-undo").addClass("e-disable");
                            this._toolBarObj.deselectItemByID(this._rteId + "_undo");
                        }
                        return false;
                    }
                    else if (e.keyCode == 67) {
                        _toolEle = this._toolBarItems.find("li.e-rteItem-copy");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }
                    }
                    else if (e.keyCode == 88) {
                        _toolEle = this._toolBarItems.find("li.e-rteItem-cut");
                        if (_toolEle.hasClass("e-disable")) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }

                    }
                    else if (e.keyCode == 86) {
                        _toolEle = this._toolBarItems.find("li.e-rteItem-paste");
                        if (_toolEle.hasClass("e-disable") || ((this.model.maxLength <= $.trim(this._getText()).length) && (ej.isNullOrUndefined(this._contolAKeyfalg) ? true : this._contolAKeyfalg))) {
                            this._keypressFlag = false;
                            e.preventDefault();
                        }
                    }
                    else if (e.keyCode == 77) {
                        if (e.shiftKey == true) {
                            e.preventDefault();
                            _toolEle = this._toolBarItems.find("li.e-rteItem-outdent");
                            if (!_toolEle.hasClass("e-disable")) {
                                this._onOutdent();
                                if (this._indentdepth > 0)
                                    this._indentdepth--;
                            }
                            return false;
                        }
                        else {
                            e.preventDefault();
                            _toolEle = this._toolBarItems.find("li.e-rteItem-indent");
                            if (!_toolEle.hasClass("e-disable")) {
                                this._onIndent();
                                this._indentdepth++;
                            }
                            return false;
                        }
                    }
                    else if (e.keyCode == 75) {
                        e.preventDefault();
                        _toolEle = this._toolBarItems.find("li.e-rteItem-createLink");
                        if (!_toolEle.hasClass("e-disable")) {
                            this._hyperLinkManager(false);
                        }
                        return false;
                    }
                }
                else if (e.keyCode == 127) {
                    if ((this.getHtml() == this.getSelHtmlString()) || (this._currentIFrame.get_Text() == this._currentIFrame.getSelText())) {
                        var isClearAll = this.ClearAllManager();
                        if (!isClearAll) {
                            e.preventDefault();
                            return isClearAll;
                        }
                    }
                }
                else if (e.keyCode == 8) {
                    ele = this._getSelectedNode();
                    if ($(ele).parents("BLOCKQUOTE").length > 0) {
                        this._onOutdent();
                        e.preventDefault();
                    }
                }
            }
            else if (e.ctrlKey == true && (e.keyCode == 66 || e.keyCode == 73 || e.keyCode == 85 || e.keyCode == 76 || e.keyCode == 82 || e.keyCode == 69 || e.keyCode == 74 || e.keyCode == 90 || e.keyCode == 89 || e.keyCode == 67 || e.keyCode == 88 || e.keyCode == 86 || e.keyCode == 8 || e.keyCode == 46))
                e.preventDefault();
            if (this.model.maxLength != null && (ej.isNullOrUndefined(this._contolAKeyfalg) ? true : !this._contolAKeyfalg))
                this._validateMaxLength(e);
            if (this._isIE8() ? this._currentSelNode.nodeName.toLowerCase() == "td" : this._getSelectedNode().nodeName.toLowerCase() == "td") {
                ele = this._isIE8() ? this._currentSelNode : this._getSelectedNode();
                _newElementIndex = $(ele).parent().children().index($(ele));
                if (e.keyCode == 9)
                    _newElement = (!ej.isNullOrUndefined(ele.nextSibling)) ? ele.nextSibling : (!ej.isNullOrUndefined($(ele).parents("tr")[0].nextSibling) ? $(ele).parents("tr")[0].nextSibling.childNodes[0] : (!ej.isNullOrUndefined($(ele).parents("table")[0].nextSibling)) ? ($(ele).parents("table")[0].nextSibling.nodeName.toLowerCase() == "td") ? $(ele).parents("table")[0].nextSibling : ele : ele);
                else if (e.keyCode == 40)
                    _newElement = (!ej.isNullOrUndefined($(ele).parents("tr")[0].nextSibling)) ? $(ele).parents("tr")[0].nextSibling.children[_newElementIndex] : ((!ej.isNullOrUndefined($(ele).parents("table")[0].nextSibling) && !this._isIE8()) ? $(ele).parents("table")[0].nextSibling : ele);
                else if (e.keyCode == 38)
                    _newElement = (!ej.isNullOrUndefined($(ele).parents("tr")[0].previousSibling)) ? $(ele).parents("tr")[0].previousSibling.children[_newElementIndex] : ((!ej.isNullOrUndefined($(ele).parents("table")[0].previousSibling)) && !this._isIE8() ? $(ele).parents("table")[0].previousSibling : ele);
                if (e.keyCode == 9 && ele == _newElement) {
                    this.insertRow("addRowBelow");
                    this._setRange(this._isIE8()?$(this._getRange().parentElement()).parents("tr").children().first()[0]:$(this._getRange().startContainer).parents("tr").children().first()[0], this._getRange(), true);
                    e.preventDefault();
                }
                else if (e.keyCode == 9 || e.keyCode == 40 || e.keyCode == 38) {
                    this._setRange(_newElement, this._getRange(), true);
                    e.preventDefault();
                }
                if(this._isIE8())
                this._currentSelNode=this._getRange().parentElement();
            }
            this._setAutoHeight();
            this._trigger("keydown", { keyCode: e.keyCode });
        },
        _iframeKeypress: function (e) {
            if (!this._keypressFlag) {
                e.preventDefault();
                this._keypressFlag = true;
                return false;
            }
        },
        _setAutoHeightValue: function (selectedtext) {
            if (selectedtext.nodeName == "#text") {
                if (selectedtext.parentNode.nodeName == "BODY")
                    $(selectedtext).wrap("<div></div>");
                range = $(selectedtext.parentNode).position().top + parseInt($(selectedtext.parentNode).css("height"));
            }
            if (selectedtext.nodeName == "DIV")
                range = $(selectedtext).position().top + parseInt($(selectedtext).css("height"));
            return range;
        },
        _setAutoHeight: function () {
            if (this.model.autoHeight && !$(this._toolBarObj.itemsContainer.find(".resize")).length)
			    this._autoHeight();
			else if (!ej.isNullOrUndefined(this._getDocument())) $(this._getDocument().body).css("overflow", "");
		},
	    _autoHeight: function(){
            if (!ej.isNullOrUndefined(this._getDocument().body)) {
		        $(this._getDocument().body).css("overflow","hidden");
                var iframeTop = (!ej.isNullOrUndefined(this._getDocument().body.lastChild)) ? this._getDocument().body.lastChild.clientHeight + this._getDocument().body.lastChild.offsetTop : 0;
                if ((parseInt($(this._rteWapper[0]).css("min-height")) - this._toolBarItems.outerHeight()) <= iframeTop)
                    $(this._rteIframe[0]).css("height", (iframeTop + 30) + "px");
			    else
			        $(this._rteIframe[0]).css("height", (parseInt($(this._rteWapper[0]).css("min-height")) - this._toolBarItems.outerHeight()) + "px");
			    range = 0;
			    range = (window.getSelection && !ej.isNullOrUndefined(this._getRange().startContainer)) ? this._setAutoHeightValue(this._getRange().startContainer) : (((this._getDocument().selection) && !ej.isNullOrUndefined(this._ieParents(this._getRange())[0])) ? this._setAutoHeightValue(this._ieParents(this._getRange())[0]) : 0);
			    if(range >= (this._rteIframe.height() - 30)) {
			        if(this._rteIframe[0].contentDocument)
                    $(this._rteIframe[0]).css("height", (this._rteIframe[0].contentDocument.documentElement.scrollHeight + 20) + "px");
                else
                    $(this._rteIframe[0]).css("height", (this._rteIframe[0].contentWindow.document.body.scrollHeight + 20) + "px");
			    }
			    this._rteWapper.outerHeight(this._rteEditor.outerHeight() + this._toolBarItems.outerHeight() + parseInt(this.model.showFooter ? 19 : 0));
            }
		},
        _ieSelRange: function () {
            if (this._isIE()) {
                this._selectionRange = "";
                this._ieSelectionRange = this._getRange();
                this._seleText = ej.isNullOrUndefined(this._getWindow().document.selection) ? this._selectionRange.toString() : this._getWindow().document.selection.createRange().text;
            }
            else
                this._ieSelectionRange = this._ieSelection;
        },
        _iframeKeyUp: function (e) {
            this._toggleEditTable();
            this._setBackupData();
            this._updateToolbarStatus();
            this._updateFontOptionStatus();
            this._updateFormat();
            this._updateCount();            
            var selectEle = this._isIE8() ? this._currentSelNode : this._getSelectedNode();
            this._updateTagInfo(selectEle);
            this._updateValue();
            this._setAutoHeight();
            this._ieSelRange();
            this._trigger("keyup", { keyCode: e.keyCode });
        },
        _iframeMouseUp: function (e) {
		   this._resizeBtnInit();
			if(!ej.isNullOrUndefined(this._imgDup)){
			 $(this._imgOrg).css({"width" :$(this._imgDup).css("width"),"height" :$(this._imgDup).css("height")});
			  $(this._imgOrg).position({left:$(this._imgDup).position().left,top : $(this._imgDup).position().top});
			  $(this._imgDup).css("display","none");
			  this._resizeImgPos();
			  this._setAutoHeight();
			  }
			 if(e.target.nodeName != "IMG" && !$(e.target).hasClass("e-rte-imageboxmark"))
			 {
			   $(this._imgOrg).css("outline","");
			   $(this._imgDupDiv).remove();
			   $(this._getDocument()).unbind("mousemove",$.proxy(this._imgBoxMouseMove,this));
			   }
            this._currentSelNode = e.target; //For IE
            this._toggleEditTable();
            this._updateToolbarStatus();
            this._updateFontOptionStatus();
            this._updateFormat();
            this._ieSelRange();
        },
        _iframeMouseDown: function (e) {
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version == "8.0")
                this._focus();
            this._contolAKeyfalg = false;
            this._updateTagInfo(e.target);
            this._updateIndent(e.target);
			if(e.target.nodeName =="IMG" && (ej.browserInfo().name == "chrome" || ej.browserInfo().name == "webkit"))
			   this._imageClick(e.target);
			if($(e.target).hasClass("e-rte-imageboxmark")) {
			       this._resizeBtnInit();
			       this._commonBoxDown = true;
		           if($(e.target).hasClass("e-rte-rigmed")) this._rigMidBox = true;
		           if($(e.target).hasClass("e-rte-toplef")) this._topLefBox = true;
		           if($(e.target).hasClass("e-rte-topmed")) this._topMidBox = true;
		           if($(e.target).hasClass("e-rte-toprig")) this._topRigBox = true;
		           if($(e.target).hasClass("e-rte-lefmed")) this._lefMidBox = true;
		           if($(e.target).hasClass("e-rte-botlef")) this._botLefBox = true;
		           if($(e.target).hasClass("e-rte-botmid")) this._botMidBox = true;
		           if($(e.target).hasClass("e-rte-botrig")) this._botRigBox = true;
                   return false;				   
			}			
            if ((ej.browserInfo().name == "msie" || ej.browserInfo().name == "chrome") && (e.ctrlKey == true && e.which == 1)) {
                if (e.target.tagName.toUpperCase() == 'A' && !ej.isNullOrUndefined($(e.target).attr('href')))
                    window.open($(e.target).attr('href'), "_blank");
            }
            if (this.model.isResponsive && this._toolBarObj._liTemplte.children('ul').length > 0) {
                this._toolBarObj.contstatus = true;
                this._toolBarObj._liTemplte.addClass("e-display-none");
                this._toolBarObj.contstatus = false;
            }
            this._iframeFocus();
        },
        _iframeFocusOut: function (e) {
            _proxy._onChange();
            _proxy._selectionRange = _proxy._isIE8() ? _proxy._getRange() : _proxy._saveSelection();
            _proxy._seleText = ej.isNullOrUndefined(_proxy._getWindow().document.selection) ? _proxy._selectionRange.toString() : !_proxy._isIE8() ? _proxy._getWindow().getSelection().toString() : _proxy._getWindow().document.selection.createRange().text;

        },
        _widthFocusOut: function (e) {
            var dialog, drpdwnId, widthId;
            if (e.currentTarget.id === this._rteId + "_txtWidth") {
                dialog = this._customTableDialog;
                drpdwnId = "#" + this._rteId + "_ddlAlignment";
                widthId = "#" + this._rteId + "_txtWidth";
            }
            else {
                dialog = this._eTblDialog;
                drpdwnId = "#" + this._rteId + "_eTblAlign";
                widthId = "#" + this._rteId + "_eTblWidth";
            }
            var val = dialog.find(widthId).val();
            var obj = $(drpdwnId).data('ejDropDownList');
            var option = (val && !isNaN(val) || (val.slice(-2) == 'px') || (val.slice(-2) == 'em') || (val.slice(-2) == 'pt') || (val.slice(-1) == '%')) ? true : false;
            obj.option('enabled', option);
        },
        _windowResize: function (e) {
            this._setAutoHeight();
            this._setIframeHeight();
        },
		_resizeBtnInit: function(){
		    this._commonBoxDown = false;
		    this._rigMidBox = false;
		    this._topLefBox = false;
		    this._topMidBox = false;
		    this._topRigBox = false;
		    this._lefMidBox = false;
		    this._botLefBox = false;
		    this._botMidBox = false;
		    this._botRigBox = false;
		 },
		_imgSpanBox: function(e){
		    this._resizeBtnInit();
		    this._imgOrg = e;	  
            this._imgDupDiv = ej.buildTag("span#RTE_imgDupDiv");	
            this._imgDup = ej.buildTag("span#RTE_imgdivspan","",{"display":"none","position":"absolute","background-size":"100% 100%","outline":"1px dashed #000","opacity":".64"},{}).appendTo(this._imgDupDiv);		
		    this._topLef = ej.buildTag("span.e-rte-imageboxmark e-rte-toplef#RTE_toplef","",{"cursor":"nwse-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);	
            this._topMid = ej.buildTag("span.e-rte-imageboxmark e-rte-topmed#RTE_topmed","",{"cursor":"ns-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);	
            this._topRig = ej.buildTag("span.e-rte-imageboxmark e-rte-toprig#RTE_toprig","",{"cursor":"nesw-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);	
            this._leftMid = ej.buildTag("span.e-rte-imageboxmark e-rte-lefmed#RTE_lefmed","",{"cursor":"ew-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);	
            this._rigMid = ej.buildTag("span.e-rte-imageboxmark e-rte-rigmed#RTE_rigmed","",{"cursor":"ew-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);	
            this._botLef = ej.buildTag("span.e-rte-imageboxmark e-rte-botlef#RTE_botlef","",{"cursor":"nesw-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);	
            this._botMid = ej.buildTag("span.e-rte-imageboxmark e-rte-botmid#RTE_botmid","",{"cursor":"ns-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);	
            this._botRig = ej.buildTag("span.e-rte-imageboxmark e-rte-botrig#RTE_botrig","",{"cursor":"nwse-resize","width":"5px","height":"5px","position":"absolute","display":"block","background":"#fff","border":"1px solid #000"},{}).appendTo(this._imgDupDiv);
            this._resizeImgPos(); 
            this._resizeImgDupPos();		 
	        $(e).after(this._imgDupDiv).css({"outline":"1px dashed #000"});		
		    $(this._getDocument()).bind("mousemove",$.proxy(this._imgBoxMouseMove,this));
		
		},	
        _resizeImgPos: function(){
		    this._ltPos = $(this._imgOrg).position();
		    this._imgWid = $(this._imgOrg).css("width");
		    this._imgHgt = $(this._imgOrg).css("height");
		    this._imgDup.css({"background-image":"url("+$(this._imgOrg).attr("src")+")","width":this._imgWid,"height":this._imgHgt,"left":this._ltPos.left+"px","top":this._ltPos.top+"px"})
		    var _wid = (parseInt($(this._imgOrg).css("width")) - 2)+this._ltPos.left;
		    var _midWid = ((parseInt($(this._imgOrg).css("width")) - 2)/2)+this._ltPos.left;
		    var _hgt = (parseInt($(this._imgOrg).css("height")) - 2)+this._ltPos.top;
		    var _midHgt = ((parseInt($(this._imgOrg).css("height")) - 2)/2)+this._ltPos.top; 
		    $(this._rigMid).css({"left":(_wid)+"px","top":(_midHgt)+"px"});
			$(this._botLef).css({"left":(this._ltPos.left-3)+"px","top":(_hgt)+"px"});
			$(this._botRig).css({"left":(_wid)+"px","top":(_hgt)+"px"});
			$(this._botMid).css({"left":(_midWid)+"px","top":(_hgt)+"px"});			  
			$(this._topRig).css({"left":(_wid)+"px","top":(this._ltPos.top-3)+"px"});
			$(this._topLef).css({"left":(this._ltPos.left-3)+"px","top":(this._ltPos.top-3)+"px"});
			$(this._topMid).css({"left":(_midWid)+"px","top":(this._ltPos.top-3)+"px"});
            $(this._leftMid).css({"left":(this._ltPos.left-3)+"px","top":(_midHgt)+"px"});			  
		
		},		
		_resizeImgDupPos: function(){
		    this._ltDupPos = $(this._imgOrg).position();
		    this._imgMidWid = $(this._imgOrg).css("width");
		    this._imgDupHgt = $(this._imgOrg).css("height");
		},
		_imgBoxMouseMove: function(e) {
		   if(this._commonBoxDown) 
		   {		
		        if(this._rigMidBox)		  
		            this._imgDupMouseMove(parseInt(e.pageX)-this._ltDupPos.left +"px",this._imgDupHgt);
		        else if(this._lefMidBox)
		        {	 	
		            this._imgDupMouseMove(((parseInt(this._imgMidWid) +this._ltDupPos.left) - e.pageX)+"px",this._imgDupHgt);
                    $(this._imgDup).offset({left:e.pageX});			  
		        }
		        else if(this._topMidBox)
		        {	  
		            this._imgDupMouseMove(this._img_wid,((parseInt(this._imgDupHgt)+this._ltDupPos.top)-e.pageY) +"px");
                    $(this._imgDup).offset({top:e.pageY});		  
		        }
		        else if(this._botMidBox)
		            this._imgDupMouseMove(this._img_wid,(e.pageY - this._ltDupPos.top) +"px");		  
		        else if(this._botRigBox)
		            this._imgDupMouseMove((e.pageX - this._ltDupPos.left)+"px",(e.pageY - this._ltDupPos.top)+"px");
		        else if(this._botLefBox)
		        {	
		            this._imgDupMouseMove(((parseInt(this._imgMidWid) +this._ltDupPos.left) - e.pageX)+"px",(e.pageY - this._ltDupPos.top) +"px");
                    $(this._imgDup).offset({left:e.pageX});	   
		        }
		        else if(this._topRigBox)
		        {	 
		            this._imgDupMouseMove((e.pageX - this._ltDupPos.left)+"px",((parseInt(this._imgDupHgt) + this._ltDupPos.top) - e.pageY)+"px");
                    $(this._imgDup).offset({top:e.pageY});           
		        }
		        else if(this._topLefBox)
		        {
		            this._imgDupMouseMove(((parseInt(this._imgMidWid) +this._ltDupPos.left) - e.pageX)+"px",((parseInt(this._imgDupHgt)+this._ltDupPos.top)-e.pageY)+"px");
                    $(this._imgDup).offset({top:e.pageY,left:e.pageX});				
		        }
		    }
		},
		_imgDupMouseMove: function (width,height) {
		  	$(this._imgDup).css({"display":"block","width":width,"height":height});
		    this._resizeImgDupPos();
		},
		_imageClick: function (e) {	
         	$(this._getDocument().body).find("img").css("outline","");
		   if(this._imgDupDiv)
		       this._imgDupDiv.remove();
		   this._imgSpanBox(e);		
		},
		_iframePaste: function (e) {
		    var _proxy = this;
		    setTimeout(function () {
		        _proxy._updateValue();
		        _proxy._setAutoHeight();
		    }, 0);
		},
        _wireEvents: function () {
            var _iframe = $(this._getDocument());
            if (this._getDocument() != "") {
                if (!ej.isNullOrUndefined(this._createTable)) {
                    this._on(this._createTable, "mouseenter", "div.e-rte-tablecell", this._tableCellStart)
                    ._on(this._createTable, "mousemove", "div.e-rte-tablecell", this._tableCellSelect)
                    ._on(this._createTable, "mouseleave", "div.e-rte-table", this._tableCellLeave)
                    ._on(this._createTable, "mousedown", "div.e-rte-tablecell", this._tableCellDown)
                    ._on($(document), "click", this._documentClick);
                }
                this._on(_iframe, "focus", this._iframeFocus)
                ._on(_iframe, "keydown", this._iframeKeyDown)
                ._on(_iframe, "keypress", this._iframeKeypress)
                ._on(_iframe, "keyup", this._iframeKeyUp)
                ._on(_iframe, "mouseup", this._iframeMouseUp)
                ._on(_iframe, "mousedown", this._iframeMouseDown)

                ._on(_iframe, "paste", this._iframePaste)
                ._on($(window), "resize", this._windowResize);
                _proxy = this;
                if (!ej.isNullOrUndefined((document.getElementById(this._id + "_Iframe"))))
                $(document.getElementById(this._id + "_Iframe").contentWindow).on('blur', this._iframeFocusOut);
                if (!ej.isNullOrUndefined(this._customTableDialog))
                    this._on(this._customTableDialog.find(".e-rte-txtWidth"), "focusout", this._widthFocusOut);
            }
        },

        _unwireEvents: function () {
            var _iframe = $(this._getDocument());
            if (!ej.isNullOrUndefined(this._createTable)) {
                this._off(this._createTable.find("div.e-rte-tablecell"), "mousemove")
                ._off(this._createTable.find("div.e-rte-table"), "mouseleave")
                ._off(this._createTable.find("div.e-rte-tablecell"), "mousedown")
                ._off($(document), "click", this._documentClick);
            }
            this._off(_iframe, "focus")
            ._off(_iframe, "keydown")
            ._off(_iframe, "keyup")
            ._off(_iframe, "mouseup")
            ._off(_iframe, "mousedown")
            ._off($(window), "resize");
            $(document.getElementById(this._id + "_Iframe").contentWindow).off('blur', this._iframeFocusOut);
            if (!ej.isNullOrUndefined(this._customTableDialog))
                this._off(this._customTableDialog.find(".e-rte-txtWidth"), "focusout");
        },

        _footerEvents: function (val) {
            this[val](this._htmlSource, "click", this._sourceCodeManager);
            this[val](this._clearFormat, "click", this._onClearFormat);
            this[val](this._clearAll, "click", this._clearAllManager);
        },


        _onChange: function () {
            _prevhtml = $.trim(this.value());
            if (this.model.enableXHTML)
                this._updateXhtml();
            if (_prevhtml !== this.getHtml() || (ej.isNullOrUndefined((this._contentBeforfocus)) ? false : ($.trim(this._contentBeforfocus) !== this.getHtml()))) {
                this._updateValue();
                var args = { text: this._getText(), htmlText: this.getHtml() };
                this._contentBeforfocus = this.value();
                this._trigger("change", args);
            }
        },
        _disableIframeContent: function () {
            try {
                this._rteIframe.contents().find("body").attr("contenteditable", false);
            }
            catch (err) { }
        },

        disable: function () {
            this.model.enabled = false;
			this.element.attr("disabled","disabled");
            this._unwireEvents();
            this.model.showToolbar && this._toolBarObj.disable();
            this._disableIframeContent();
            this._rteEditor.addClass("e-disable");
            if (this.model.showFooter) {
                this._rteFooter.addClass("e-disable");
                this._resize.removeClass("e-resizable");
                this._footerEvents("_off");
            }
        },

        enable: function () {
            this.model.enabled = true;
			this.element.removeAttr("disabled");
            this._wireEvents();
            this.model.showToolbar && this._toolBarObj.enable();
            this._rteIframe.contents().find("body").attr("contenteditable", true);
            this._rteEditor.removeClass("e-disable");
            if (this.model.showFooter) {
                this._rteFooter.removeClass("e-disable");
                this._resize.addClass("e-resizable");
                this._footerEvents("_on");
            }
        },

        disableToolbarItem: function (id) {
                !ej.isNullOrUndefined(id) && this._toolBarObj.disableItemByID(this._rteId + "_" + id);
                if (id == "undo") $("#" + this._rteId + "_" + id).addClass("e-rteTooldisable");
        },


        enableToolbarItem: function (id) {
                !ej.isNullOrUndefined(id) && this._toolBarObj.enableItemByID(this._rteId + "_" + id);
                if (id == "undo" && $("#" + this._rteId + "_" + id).hasClass("e-rteTooldisable"));
            $("#" + id).removeClass("e-rteTooldisable");
        },


        removeToolbarItem: function (id) {
                !ej.isNullOrUndefined(id) && this._toolBarObj.removeItemByID(this._rteId + "_" + id);
            this._setIframeHeight();
        },

        refresh: function () {
            this._unwireEvents();
            this._setIFrames();
            this._setIframeHeight();
            if (this.model.enabled) this._wireEvents();
            else this._disableIframeContent();
            if (this.model.enableRTL) this._rteIframe.contents().find("body").css("direction", "rtl");
        },


        show: function () {
            this._rteWapper.show();
        },


        hide: function () {
            this._rteWapper.hide();
        },

        setColorPickerType: function (value) {
            if (value == "palette" || value === "picker")
                this._imgPicker.option('modelType', value);
            else
                this._imgPicker.option('modelType', "default");
        },
        pasteContent: function (html) {
            this._pasteAPIFlag = true;
            this._pasteHtml(html);
            this._onChange();
        },
        getDocument: function () {
            return this._getDocument();
        },
        getHtml: function (encode) {
            if (this.model.enableXHTML) {
                if (this.model.enableHtmlEncode) {
                    if (encode == false)
                        return $.trim(this._decode(this.model.value));
                    else
                        return $.trim(this.model.value);
                }
				else
                        return $.trim(this.model.value);
            }
            else {
                try {
                    if (this.model.enableHtmlEncode) {
                        if (encode == false)
                            return $.trim(this._getDocument().body.innerHTML);
                        else
                            return this._encode($.trim(this._getDocument().body.innerHTML));
                    }
                    else
                        return $.trim(this._getDocument().body.innerHTML);
                }
                catch (err) {
                    return "";
                }
            }
        },
        setHtml: function (value) {
            this._getDocument().body.innerHTML = "";
            this._getDocument().body.innerHTML = $.trim(value);
            this._onChange();
        },


        getText: function () {
            return this._getText();
        },


        executeCommand: function (cmdName, args) {
            if (this._isIE8() && ej.isNullOrUndefined(this._ieSelectionRange)) this._ieSelectionRange = this._getRange();
            if (this._trigger("execute", { commandName: cmdName })) return false;
            var anchor = "";
            if (ej.browserInfo().name == "chrome" && this._getWindow().getSelection().rangeCount == 0)
                this._restoreSelection(this._saveSelection());
            if (cmdName == "underline") {
                if (window.getSelection) {
                    anchor = $(this._getWindow().getSelection().anchorNode).parents("a");
                    if (anchor.length > 0) {
                        $(anchor).css("text-decoration", ($(anchor).css("text-decoration") == "none") ? "underline" : "none");
                        return;
                    }
                }
            }
            else if (cmdName.toLowerCase() == "inserthtml") {
                if (this._isIE()) {
                    this._pasteHtml(args);
                }
                else
                    this._getDocument().execCommand(cmdName, false, args);
                return;
            }
            else if (cmdName.toLowerCase() == "formatblock") {
                if (this._isIE()) {
                    if (args.charAt(0) != "<" && args.charAt(args.length - 1) != ">")
                        args = "<" + args + ">";
                }
            }
            if (ej.browserInfo().name == "msie")
                this._focus();
            if (this._isIE() && cmdName == "fontName") args = args.substring(0, args.indexOf(","));
            if (!ej.isNullOrUndefined(this._tempEle) && this._tempEle.length > 0)
                if ((this._tempEle.get(0).nodeName == "#text") && ($(this._tempEle.parents("li")).length > 0))
                    if ($(this._tempEle.parents("li")).prop("style")["font-size"] != "")
                        var lisize = $(this._tempEle.parents("li")).css("font-size");
            this._getDocument().execCommand(cmdName, false, args);
            this._onChange();
            if (cmdName != "fontName" && cmdName != "fontSize" && cmdName != "indent" && cmdName != "outdent")
                if (!this._toolBarObj.itemsContainer.find("li#" + this._rteId + "_" + cmdName).hasClass("e-active"))
                        this._toolBarObj.selectItemByID(this._rteId + "_" + cmdName);
                else
                        this._toolBarObj.deselectItemByID(this._rteId + "_" + cmdName);
            if (!ej.isNullOrUndefined(this._tempEle)) this._bulletsResizing(lisize, cmdName);
            this._setBackupData();
        },
        _bulletsResizing: function (lisize, cmd) {
            if (!ej.isNullOrUndefined(this._tempEle.get(0)))
                var li = this._tempEle.get(0).tagName == "BODY" ? this._tempEle.find("li") : this._tempEle.get(0).nodeName != "#text" ? $(this._tempEle.parents("body")).find("li") : "";
            if (li != "")
                $(li).each(function (index) {
				//refreshig the li font size after font size of li content executed
                    fonttag = $(this).find("font")[0] || "";
                    if (fonttag != "") {
                        $(this).css("font-size", $($(fonttag)[0]).css("font-size"));
                        $(li).find("font").removeAttr("style");
                    }
                });


            if (cmd == "fontSize" && (this._getCommandStatus('InsertOrderedList') || this._getCommandStatus('InsertOrderedList')))
                if (this._tempEle.get(0).tagName == "LI" || this._tempEle.get(0).tagName == "OL") {

                    this._tempEle.css("font-Size", this._tempEle.find("font").css("font-Size"));
                    $(this._tempEle.find("font")).removeProp("style")["font-size"];
                }
            if (lisize != null) {

                $(this._tempEle.parents("li")).css("font-size", lisize);
                $(this._tempEle.parents("li")).find("font").removeAttr("style");
            }

        },


        focus: function () {
            this._focus();
        },


        getCommandStatus: function (value) {
            if (ej.isNullOrUndefined(value) || value != "")
                return this._getCommandStatus(value);
        },


        getSelectedHtml: function () {
            return this._getSelectedHtmlString();
        },


        insertRow: function (before, cell) {
            if (this.model.maxLength != null && (this.model.maxLength > $.trim(this._getText()).length)) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var colTemplate = "<td " + this._border + "contentEditable='true'><br _moz_dirty=''/></td>", curRow = $(seletedCell).closest("tr");
            var cols = curRow.find("td").length;
            var newRow = $("<tr>" + Array(cols + 1).join(colTemplate) + "</tr>");
            before ? newRow.insertBefore(curRow) : newRow.insertAfter(curRow);
            curRow.find("td:first").focus();
            this._setAutoHeight();
			}
        },


        insertColumn: function (before, cell) {
            if (this.model.maxLength != null && (this.model.maxLength > $.trim(this._getText()).length)) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var colTemplate = "<td " + this._border + "contentEditable='true'><br _moz_dirty=''/></td>", curRow = $(seletedCell).closest("tr"), curCell;
            var allRows = curRow.closest("table").find("tr");
            var colIndex = curRow.find("td").index(seletedCell);
            for (var i = 0; i < allRows.length; i++) {
                curCell = $(allRows[i]).find("td")[colIndex];
                before ? $(colTemplate).insertBefore(curCell) : $(colTemplate).insertAfter(curCell);
            }
            }
        },


        removeRow: function (cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var range = this._getRange();
            var select; var node = null;
            var parentTable = $(seletedCell).closest("table");

            if (parentTable.find("tr").length == 1) {
                node = $(seletedCell).parent("tr").closest("table").next()[0] || $(seletedCell).parent("tr").closest("table").prev()[0];
                $(seletedCell).parent("tr").closest("table").remove();
            }
            else {
                node = ($(seletedCell).parent("tr").prev("tr").length != 0) ? $(seletedCell).parent("tr").prev("tr").find("td:last")[0] : $(seletedCell).parent("tr").next("tr").find("td:first")[0];
                curRow = $(seletedCell).closest("tr").remove();
            }
            if (!ej.isNullOrUndefined(node)) {
                this._setRange(node, range);
            }
            this._setAutoHeight();
        },
        selectRange: function (range, node) {
            this._focus();
            if (!this._isIE8()) {
                select = this._getDocument().getSelection();
                select.removeAllRanges();
                select.addRange(range);
            }
            else {
                range.select();
            }
        },
        _encode: function (value) {
            return $('<div/>').text(value).html();
        },
        _decode: function (value) {
            return value.replace(/&amp;/g, "&").replace(/&amp;lt;/g, "<").replace(/&lt;/g, "<").replace(/&amp;gt;/g, ">").replace(/&gt;/g, ">").replace(/&nbsp;/g, " ").replace(/&amp;nbsp;/g, " ").replace(/&quot;/g);
        },
        createRange: function () {
            if (!this._isIE8())
                return this._getRange();
            else
                return this.getDocument().body.createTextRange();
        },
        _setRange: function (node, range, collapse) {
            if (!this._isIE8()) {
                range.setStart(node, 0);
                range.collapse(true);
                if (ej.browserInfo().name != "msie" || ej.browserInfo().version == "11.0") {
                    select = this._getDocument().getSelection();
                    select.removeAllRanges();
                    select.addRange(range);
                }
                else if (ej.isNullOrUndefined(collapse)) {
                    selection = this._getDocument().body.createTextRange();
                    selection.moveToElementText(node);
                }
                else {
                    selection = this._getDocument().getSelection();
                    selection.removeAllRanges();
                    selection.addRange(range);
                }
            }
            else if (!ej.isNullOrUndefined(node)) {
                range.collapse(true);
                range.moveToElementText(node);
                range.moveStart("character", 1);
                range.select();
            }
        },

        removeColumn: function (cell) {
            var seletedCell = ej.isNullOrUndefined(cell) ? this._getSelectedNode() : cell;
            var range = this._getRange();
            var curRow = $(seletedCell).closest("tr");
            var allRows = curRow.closest("table").find("tr");
            var colIndex = curRow.find("td").index(seletedCell);
            var node = !ej.isNullOrUndefined(seletedCell.nextElementSibling) ? $(seletedCell.nextElementSibling)[0] : $(seletedCell.previousSibling)[0];
            if (ej.isNullOrUndefined(node))
                $(seletedCell).parent("tr").parents("table").remove();
            else {
                for (var i = 0; i < allRows.length; i++) {
                    $($(allRows[i]).find("td")[colIndex]).remove();
                }
            }
            if (!ej.isNullOrUndefined(node)) {
                this._setRange(node, range);
            }
        },


        removeTable: function (table) {
            var seletedTable = ej.isNullOrUndefined(table) ? $(this._getSelectedNode()).closest("table") : table, tableAction = $(seletedTable).parent().closest('table').length < 1;
            this._currentSelNode = $(seletedTable).parent()[0];
            $(seletedTable).remove();
            if (tableAction)      
                this._toggleEditTable();
            this._setAutoHeight();
        }
    });

    ej.RTE.Locale = {};

    ej.RTE.Locale["en-US"] = {
        bold: "Bold",
        italic: "Italic",
        underline: "Underline",
        strikethrough: "Strikethrough",
        superscript: "Superscript",
        subscript: "Subscript",
        justifyCenter: "Align text center",
        justifyLeft: "Align text left",
        justifyRight: "Align text right",
        justifyFull: "Justify",
        unorderedList: "Insert unordered list",
        orderedList: "Insert ordered list",
        indent: "Increase Indent",
        fileBrowser: "File Browser",
        outdent: "Decrease Indent",
        cut: "Cut",
        copy: "Copy",
        paste: "Paste",
        paragraph: "Paragraph",
        undo: "Undo",
        redo: "Redo",
        upperCase: "Upper Case",
        lowerCase: "Lower Case",
        clearAll: "Clear All",
        clearFormat: "Clear Format",
        createLink: "Insert/Edit hyperlink",
        removeLink:"Remove hyperlink",
        image: "Insert image",
        video: "Insert video",
        editTable: "Edit Table Properties",
        embedVideo: "Paste your embed code below",
        viewHtml: "View HTML",
        fontName: "Select font family",
        fontSize: "Select font size",
        fontColor: "Select color",
        format: "Format",
        backgroundColor: "Background color",
        style: "Styles",
        deleteAlert: "Are you sure you want to clear all the contents?",
        copyPasteAlert: "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.",
        videoError: "The text area can not be empty",
        imageWebUrl: "Web Address",
        imageAltText: "Image description",
        dimensions: "Dimensions",
        constrainProportions: "Constrain Proportions",
        linkWebUrl: "Web Address",
        imageLink: "Image as Link",
        imageBorder: "Image Border",
        imageStyle: "Style",
        linkText: "Text",
        linkToolTip: "Tooltip",
        html5Support: "This tool icon only enabled in HTML5 supported browsers",
        linkOpenInNewWindow: "Open link in new window",
        tableColumns: "No.of Columns",
        tableRows: "No.of Rows",
        tableWidth: "Width",
        tableHeight: "Height",
        tableCellSpacing: "Cell spacing",
        tableCellPadding: "Cell padding",
        tableBorder: "Border",
        tableCaption: "Caption",
        tableAlignment: "Alignment",
        textAlign: "Text align",
        dialogUpdate: "Update",
        dialogInsert: "Insert",
        dialogCancel: "Cancel",
        dialogApply: "Apply",
        dialogOk: "Ok",
        createTable: "Insert table",
        addColumnLeft: "Add column on the left",
        addColumnRight: "Add column on the right",
        addRowAbove: "Add row above",
        addRowBelow: "Add row below",
        deleteRow: "Delete row",
        deleteColumn: "Delete column",
        deleteTable: "Delete the table",
        customTable: "Create custom table...",
        characters: "Characters",
        words: "Words",
        general: "General",
        advanced: "Advanced",
        table: "Table",
        row: "Row",
        column: "Column",
        cell: "Cell",
        solid: "Solid",
        dotted: "Dotted",
        dashed: "Dashed",
        doubled: "Doubled",
		maximize:"Maximize",
		resize: "Minimize",
		swatches: "Swatches"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin provides support to display color picker within your web page and allows to pick the color.
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejColorPicker", "ej.ColorPicker", {
        _rootCSS: "e-colorpicker",

        element: null,

        model: null,
        validTags: ["input", "div"],
        _setFirst: false,
        angular: {
            require: ['?ngModel', '^?form', '^?ngModelOptions']
        },


        defaults: {

            enableOpacity: true,

            opacityValue: 100,

            columns: 10,

            palette: "basicpalette",

            htmlAttributes: {},

            buttonMode: "split",

            custom: [],

            presetType: "basic",

            modelType: "picker",

            showPreview: true,

            showTooltip: false,

            showClearButton: false,

            showSwitcher: true,

            value: null,

            displayInline: false,

            buttonText: {
                apply: "Apply",
                cancel: "Cancel",
                swatches: "Swatches"
            },

            tooltipText: {
                switcher: "Switcher",
                addbutton: "Add Color",
                basic: "Basic",
                monochrome: "Mono Chrome",
                flatcolors: "Flat Color",
                seawolf: "Sea Wolf",
                webcolors: "Web Colors",
                sandy: "Sandy",
                pinkshades: "Pink Shades",
                misty: "Misty",
                citrus: "Citrus",
                vintage: "Vintage",
                moonlight: "Moon Light",
                candycrush: "Candy Crush",
                currentcolor: "Current Color",
                selectedcolor: "Selected Color",
            },

            showApplyCancel: true,

            showRecentColors: false,

            toolIcon: null,

            cssClass: "",

            enabled: true,

            change: null,

            select: null,

            open: null,

            close: null,

            create: null,

            destroy: null,
        },
        datetypes: {
            columns: "number",
            modelType: "enum",
            palette: "enum",
            presetType: "enum",
            cssClass: "string",
            opacityValue: "number",
            displayInline: "boolean",
            showSwitcher: "boolean",
            showRecentColors: "boolean",
            enabled: "boolean",
            showPreview: "boolean",
            enableOpacity: "boolean",
            buttonText: "data",
            custom: "data",
            htmlAttributs: "data"
        },

        observables: ["value", "opacityValue"],
        value: ej.util.valueFunction("value"),
        opacityValue: ej.util.valueFunction("opacityValue"),

        _setModel: function (jsondata) {
            for (var key in jsondata) {
                switch (key) {
                    case "enableOpacity":
                        this.model.enableOpacity = jsondata[key];
                        this._previewSlider(this.model.enableOpacity);
                        this._valueOperation();
                        break;
                    case "opacityValue":
                        if (this.model.enableOpacity) {
                            this._tempOpacity = ej.util.getVal(jsondata[key]);
                            this._changeOpacity();
                            this._opacity.option('value', this._tempOpacity);
                            this._updateValue();
                            this.opacityValue(this._tempOpacity);
                            break;
                        } else return false;
                    case "custom":
                        this.model.custom = jsondata[key];
                        this._reInitialize();
                        break;
                    case "palette":
                        this.model.palette = jsondata[key];
                        this._reInitialize();
                        break;
                    case "columns":
                        this.model.columns = jsondata[key];
                        this._reInitialize();
                        break;
                    case "presetType":
                        this.model.presetType = jsondata[key];
                        if (ej.isNullOrUndefined(Colors[this.model.presetType])) return false;
                        else
                            this._reInitialize();
                        break;
                    case "buttonMode":
                        this._unBindIconClick();
                        this._buttonElement = ej.ColorPicker.ButtonMode.Split == jsondata[key] ? this.dropdownbutton : this.wrapper;
                        ej.ColorPicker.ButtonMode.Split == jsondata[key] ? this.wrapper.addClass("e-split") : this.wrapper.removeClass("e-split");
                        this._bindIconClick();
                        break;
                    case "showTooltip":
                        this._colorSlider.option('showTooltip', jsondata[key]);
                        this._opacity.option('showTooltip', jsondata[key]);
                        break;
                    case "value":
                        this.setValue(ej.util.getVal(jsondata[key]));
                        if (typeof jsondata[key] == "function")
                            jsondata[key](this.value());
                        else
                            jsondata[key] = this.value();
                        break;
                    case "modelType":
                        this.model.modelType = jsondata[key];
                        this._reInitialize();
                        break;
                    case "showSwitcher":
                        this.model.showSwitcher = jsondata[key];
                        this._showSwitcher();
                        break;
                    case "tooltipText":
                        this._toolTipText(jsondata[key]);
                        break;
                    case "showPreview":
                        this.model.showPreview = jsondata[key];
                        this._previewPane(this.model.showPreview);
                        break;
                    case "buttonText":
                        this._buttonText(jsondata[key]);
                        break;
                    case "displayInline":
                        this._setDisplayInline(jsondata[key]);
                        break;
                    case "cssClass":
                        this._setSkin(jsondata[key]);
                        this.model.cssClass = jsondata[key];
                        break;
                    case "enabled":
                        this._enabled(jsondata[key]);
                        break;
                    case "showRecentColors":
                        this.model.showRecentColors = jsondata[key];
                        this._previewColor(this.model.showRecentColors);
                        break;
                    case "htmlAttributes": this._addAttr(jsondata[key]); break;
                    case "showClearButton": this._showClearIcon(jsondata[key]); break;
                    case "showApplyCancel": this.model.showApplyCancel = jsondata[key]; this._buttonContainer(); break;
                }
            }
        },


        _setSkin: function (className) {
            if (this.wrapper)
                this.wrapper.removeClass(this.model.cssClass).addClass(className);
            else
               this.element.removeClass(this.model.cssClass).addClass(className);
        },
        _showSwitcher: function () {
            if (this.model.showSwitcher) {
                this._changeTag.removeClass('e-hide');
                this.model.modelType == "picker" ? this._switcher.addClass('e-paletteModel').removeClass('e-pickerModel') : this._switcher.addClass('e-pickerModel').removeClass('e-paletteModel');
            }
            else
                this._changeTag.addClass('e-hide');            
        },
        _pickerType: function () {
            this._modelType = "picker";
            this.PaletteWrapper.removeAttr('style');
            this.PaletteWrapper.addClass('e-hide');
            this._gradient.removeClass('e-hide');
            this._gradient.fadeIn(200);
            this._presetTag.parents('.e-split.e-widget').addClass('e-hide');
            this._switcher.removeAttr('class');
            this._switcher.addClass('e-color-image e-paletteModel');
            this._switch = true;
            this._rgbValue();
            this._hueGradient();
            this._updateUI();
            this._alphaGradient(this.RGBToHEX(this.rgb));
            this._hsva.ejButton("enable");
            this._switchEvents();
            this._unSwitchEvents();
            this._hideUnBindEvents();
            this.isPopupOpen && this._showBindEvents();
            this.model.modelType == "default" ? this._changeTag.removeClass('e-hide') : this._changeTag.addClass('e-hide');
            this.popupList.prepend(this._gradient);
            this._showSwitcher();
        },
        _paletteType: function () {
            this._gradient.removeAttr('style');
            this._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
            this.PaletteWrapper.removeClass('e-hide');
            this.PaletteWrapper.fadeIn(200);
            this._switch = false;
            this._disableHSVButton();
            this._cellSelect();
            this._switchEvents();
            this._unSwitchEvents();
            this._splitObj.option('prefixIcon', 'e-color-image e-' + this.model.presetType);
            this.popupList.prepend(this.PaletteWrapper);
            this._showSwitcher();
        },
        _reInitialize: function () {
            this._destroyPalette(false);
        },
        _destroyPalette: function (presets) {
            this.PaletteWrapper.remove();
            if (presets || this._columns != this.model.columns && this.model.palette !== "custompalette") this.PaletteWrapper = this._presetType(this._presetsId);
            if (this._temp !== this.model.presetType) this.PaletteWrapper = this._layoutType(this.model.palette);
            if (this.model.modelType == "palette") {
                this._modelType = "palette";
                this.PaletteWrapper = this._layoutType(this.model.palette);
                this._hideUnBindEvents();
                this.isPopupOpen && this._showBindEvents();
                this._gradient.addClass('e-hide');
                this._paletteType();
                presets || this.model.palette == "custompalette" ? "" : this._splitObj.option('prefixIcon', 'e-color-image e-' + this.model.presetType);
            }
            if (this.model.modelType == "picker") {
                this._pickerType();
                if (this.model.displayInline && !this.element.is('input'))
                    this._footer.addClass('e-hide');
            }
            this._temp = this.model.presetType; this._columns = this.model.columns;
            if (this._modelType == "picker")
                this._presetTag.parents('.e-split.e-widget').addClass('e-hide');
            else
                this._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
            this.refresh();
        },
        _previewColor: function (color) {
            if (color) {
                this._swatchesArea.fadeIn(200);
                this._bindRecentEvent();
            }
            else {
                this._swatchesArea.fadeOut(200);
                this._unBindRecentEvent();
            }
        },
        _buttonText: function (data) {
            $.extend(this.model.buttonText, data);
            this._buttonTag.html(this.model.buttonText.apply);
            this._cancelTag.html(this.model.buttonText.cancel);
            this._spnTag.html(this.model.buttonText.swatches);
        },
        _toolTipText: function (data) {
            $.extend(this.model.tooltipText, data);
            this._addTitleText();
        },
        _previewPane: function (showPreview) {
            showPreview ? this._previewTag.removeClass("e-hide") : this._previewTag.addClass("e-hide");
        },
        _previewSlider: function (slider) {
            slider ? this._opacity.enable() : this._opacity.disable();
        },
        _destroy: function () {
            if (this.isPopupOpen) this.hide();
            this.popupContainer.remove();
            if (this.wrapper) {
                this.element.insertAfter(this.wrapper);
                this.wrapper.remove();
                this._presetContainer.parent('.e-menu-wrap').remove();
            }
            this.element.removeClass('e-colorpicker e-input');
        },
        _init: function () {
            this._browser = ej.browserInfo();
            this._isFocused = false;
            this.isPopupOpen = false;
            this._dataBind = false;
            this._modelType = "picker";
            if (this._id)
                $("#" + this._id + "_popup").remove();
            if ("#" + this._id + "_Presets")
                $('#' + this._id + "_Presets").parent('.e-menu-wrap').remove();
            this.model.palette === "basicpalette" ? this._presetsId = "e-presets30" : "";
            if (ej.isNullOrUndefined(this.value()) && this.element[0].value !== "") this._tempValue = this.element[0].value;
            else this._tempValue = this.value();
            this._previousValue = this._previousColor = this._tempValue;
            this._renderControl();
            this._tempOpacity = this.opacityValue();
            this.model.palette === "custompalette" ? this._presetTag.parents('.e-split.e-widget').addClass('e-hide') : "";
            this.popupContainer.find('button.e-presets').ejSplitButton({ targetID: this._presetContainer.attr('id') });
            if (this._tempValue) {
                this._setValue(this._tempValue);
                if (this._switch) this._rgbValue();
            }
            this._hsvValue();
            this._hueGradient();
            this._addTitleText();
            this._showClearIcon(this.model.showClearButton);
            this._columns = this.model.columns;
            this._temp = this.model.presetType;
            if (!this._tempValue) {
                this._colorSlider.option('value', parseInt(this._hsv.h));
                this._opacity.option('value', this._tempOpacity);
                this._alphaGradient("#fff");
                this._previousValue = "";
            }
            !this.model.enabled && this._enabled(this.model.enabled);
        },
        _renderControl: function () {
            this._createWrapper();
            this._renderPopupPanelWrapper();
            this._selectedButton = this._groupTag.find('.e-click');
            this._buttonContainer();
            this._renderPopupElement();
            this._buttonElement = ej.ColorPicker.ButtonMode.Split == this.model.buttonMode ? this.dropdownbutton : this.wrapper;
            if (this.model.buttonMode == ej.ColorPicker.ButtonMode.Split && this.element.is('input')) this.wrapper.addClass("e-split");
            this._addAttr(this.model.htmlAttributes);
            this._setDisplayInline(this.model.displayInline);
            this._previewPane(this.model.showPreview);
            this._previewColor(this.model.showRecentColors);
            if (this._switch) this._previewSlider(this.model.enableOpacity);
            this._wireEvents();
            this._switchEvents();
        },

        _createWrapper: function () {
            if (this.element.is("input")) {
                this.element.addClass('e-input e-widget');
                this.spanElement = ej.buildTag("span.e-selected-color");
                this.wrapper = ej.buildTag("span.e-colorwidget e-picker e-widget " + this.model.cssClass).attr({ 'tabindex': '0', "aria-expanded": false, "aria-haspopup": true, "aria-owns": "popup" });
                if (this._id) this.wrapper.attr('id', this._id + "Wrapper");
                this.container = ej.buildTag("span.e-in-wrap e-box");
                this.drpbtnspan = ej.buildTag("span.e-icon e-down-arrow", "", {}, { "aria-label": "select" });
                this.dropdownbutton = ej.buildTag("span.e-select", "", {}, { "role": "button" }).append(this.drpbtnspan);
                this.iconWrapper = ej.buildTag("span.e-tool-icon " + this.model.toolIcon);
                this.colorContainer = ej.buildTag("span.e-color-container");
                this.colorContainer.append(this.spanElement);
                this.container.insertAfter(this.element);
                if (!ej.isNullOrUndefined(this.model.toolIcon)) {
                    this.colorContainer.prepend(this.iconWrapper);
                    this.container.addClass('e-tool');
                    this.container.append(this.colorContainer);
                }
                else
                    this.container.append(this.colorContainer);
                this.container.append(this.element, this.dropdownbutton);
                this.wrapper.insertBefore(this.container);
                this.wrapper.append(this.container);
                this.element.css("display", "none").val(this.value());
            }
            this._checkNameAttr();
        },
        _addTitleText: function () {
            this._switcher.attr('title', this.model.tooltipText.switcher);
            this._spanTag.attr('title', this.model.tooltipText.addbutton);
            this._presetLi.find("#e-presets00").attr('title', this.model.tooltipText.webcolors);
            this._presetLi.find("#e-presets01").attr('title', this.model.tooltipText.vintage);
            this._presetLi.find("#e-presets02").attr('title', this.model.tooltipText.seawolf);
            this._presetLi.find("#e-presets10").attr('title', this.model.tooltipText.sandy);
            this._presetLi.find("#e-presets11").attr('title', this.model.tooltipText.pinkshades);
            this._presetLi.find("#e-presets12").attr('title', this.model.tooltipText.moonlight);
            this._presetLi.find("#e-presets20").attr('title', this.model.tooltipText.monochrome);
            this._presetLi.find("#e-presets21").attr('title', this.model.tooltipText.misty);
            this._presetLi.find("#e-presets22").attr('title', this.model.tooltipText.flatcolors);
            this._presetLi.find("#e-presets30").attr('title', this.model.tooltipText.basic);
            this._presetLi.find("#e-presets31").attr('title', this.model.tooltipText.candycrush);
            this._presetLi.find("#e-presets32").attr('title', this.model.tooltipText.citrus);
            this._currentTag.attr('title', this.model.tooltipText.currentcolor);
            this._previousTag.attr('title', this.model.tooltipText.selectedcolor);
        },
        _renderPopupPanelWrapper: function () {
            this.popupContainer = ej.buildTag("div.e-colorpicker e-box e-popup e-widget " + this.model.cssClass, "", {}, { "role": "grid", "aria-readonly": "true", "tabindex": '0', "style": "visibility:hidden" });
            if (this._id) this.popupContainer.attr('id', this._id + "_popup");
            $('body').append(this.popupContainer);

            this.popupList = ej.buildTag("div.e-popupWrapper");

            this._gradient = ej.buildTag("div.e-container");

            this._colorArea = ej.buildTag("div.e-hsv-color");
            this._gradientArea = ej.buildTag("div.e-hsv-gradient")
            this._handleArea = ej.buildTag("div.e-draghandle e-color-image");
            this._colorArea.append(this._gradientArea, this._handleArea);

            this._picker = ej.buildTag("div.e-gradient");
            this._hueSlider = ej.buildTag("div.e-widget e-hue e-state-default");
            this._alphaSlider = ej.buildTag("div.e-widget e-opacity e-state-default");
            this._picker.append(this._hueSlider, this._alphaSlider);

            this._gradient.append(this._colorArea, this._picker);

            this.popupList.append(this._gradient);


            this._footerBlock = ej.buildTag("div.e-footerContainer");

            this._templateWrapper = ej.buildTag("div.e-buttons");


            this._groupTag = ej.buildTag("div.e-groupButton");

            this._formEle = ej.buildTag("div.e-form");
            this._rgb = ej.buildTag("button.e-rgbButton e-click", "", {}, { type: "button" });
            this._hexCode = ej.buildTag("button.e-hexButton", "", {}, { type: "button" });
            this._hsva = ej.buildTag("button.e-hsvButton", "", {}, { type: "button" });
            this._groupTag.append(this._rgb, this._hexCode, this._hsva);


            this._codeEditor = ej.buildTag("div.e-codeeditor");
            this._inputTag = ej.buildTag("input.e-color-code", "", {}, { "type": "text", 'tabindex': '0', "maxLength": "22" });
            this._codeEditor.append(this._inputTag);

            this._formEle.append(this._groupTag, this._codeEditor);

            this._previewTag = ej.buildTag("div.e-preview").attr({ 'tabindex': '0' });
            this._currentTag = ej.buildTag("div.e-current");
            this._previousTag = ej.buildTag("div.e-previous");
            this._previewTag.append(this._currentTag, this._previousTag);

            this._templateWrapper.append(this._formEle, this._previewTag);

            this._swatchesArea = ej.buildTag("div.e-color-labels");
            var blockCount = 11;
            this._divTag = ej.buildTag("div.e-recent-color");
            this._addTag = ej.buildTag('div.e-colorblock e-block');
            this._spanTag = ej.buildTag('div.e-color e-color-image e-add');
            this._addTag.append(this._spanTag)
            this._divTag.append(this._addTag);
            for (var count = 0; count < blockCount; count++) {
                this._liTag = ej.buildTag('div.e-colorblock e-block');
                var spanTag = ej.buildTag('div.e-color e-color-image e-empty');
                this._liTag.append(spanTag);
                this._divTag.prepend(this._liTag);
            }
            this._swatchesArea.append(this._divTag);

            this._footer = ej.buildTag('div.e-footer');
            this._swatches = ej.buildTag('div.e-element');
            this._changeTag = ej.buildTag('div.e-switcher');
            this._switcher = ej.buildTag('div.e-color-image');
            this._presetTag = ej.buildTag('button.e-presets e-colorSplit');
            this._presets = ej.buildTag('div');            
            this._changeTag.append(this._switcher);
            this._presetTag.append(this._presets);
            this._swatches.append(this._changeTag, this._presetTag);
            this._footer.append(this._swatches);
            this._footerBlock.append(this._templateWrapper, this._swatchesArea, this._footer);

            this.PaletteWrapper = this._layoutType(this.model.palette);
            this.popupList.append(this.PaletteWrapper, this._footerBlock);
            var oldWrapper = $("#" + this._id + "_Presets").get(0);
            if (oldWrapper) {
                if ($(oldWrapper).parent().hasClass("e-menu-wrap"))
                    $(oldWrapper).parent().remove();
                else
                    $(oldWrapper).remove();
            }
            this._presetContainer = $("<ul id='" + this._id + "_Presets' class='e-presetWrapper' style: top:87px ></ul>");
            this._presetLi = ej.buildTag('li.e-item');
            this._presetLi.append(this._renderPresets());
            this._presetContainer.append(this._presetLi);
            this.popupList.append(this._presetContainer);
            this.popupContainer.append(this.popupList);

            //IE Support
            if (this._browser.name = "msie" && (this._browser.version == "9.0" || this._browser.version == "8.0"))
                this._hueSlider.addClass('e-color-image e-filter');
            else
                this._hueSlider.addClass('e-common');
        },
        _buttonContainer: function () {
            if (this.model.displayInline) return;
            if (this.model.showApplyCancel) {
                this._buttonTag = ej.buildTag("button.e-applyButton", "", {}, { type: "button" });
                this._cancelTag = ej.buildTag("button.e-cancelButton", "", {}, { type: "button" });
                this._footer.append(this._buttonTag, this._cancelTag);
                this._applyObj = this.popupContainer.find('button.e-applyButton').ejButton({ text: this.model.buttonText.apply, type: "button" }).data('ejButton');
                this._cancelObj = this.popupContainer.find('button.e-cancelButton').ejButton({ text: this.model.buttonText.cancel, type: "button" }).data('ejButton');
                this._on(this._cancelTag, "click", this._hidePopup);
                this._on(this._buttonTag, "click", this._buttonClick);
            } else {
                this._buttonTag.remove();
                this._cancelTag.remove();
                this._off(this._cancelTag, "click", this._hidePopup);
                this._off(this._buttonTag, "click", this._buttonClick);
            }
        },
        _addAttr: function (htmlAttr) {
            var proxy = this;
            $.map(htmlAttr, function (value, key) {
                if (key == "class") proxy.wrapper.addClass(value);
                else if (key == "required") proxy.element.attr(key, value);
                else if (key == "disabled" && value == "disabled") proxy._enabled(false);
                else proxy.wrapper.attr(key, value)
            });
        },
        _showClearIcon: function (bool) {
            if (bool) {
                this._clearIcon = ej.buildTag("div.e-clearicon").hide();
                this._codeEditor.append(this._clearIcon);
                this._on(this._clearIcon, "mousedown", this._clearColor);
                this._on(this._clearIcon, "click", this._clearColor);
            } else {
                this._clearIcon && this._clearIcon.remove();
                this._off(this._clearIcon, "mousedown", this._clearColor);
                this._off(this._clearIcon, "click", this._clearColor);
            }
        },
        _colorPresetsClick: function (e) {
            this._presetsId = e.currentTarget.id;
            if (this._presetsId === "e-presets00") this.model.presetType = "webcolors";
            else if (this._presetsId === "e-presets01") this.model.presetType = "vintage";
            else if (this._presetsId === "e-presets02") this.model.presetType = "seawolf";
            else if (this._presetsId === "e-presets10") this.model.presetType = "sandy";
            else if (this._presetsId === "e-presets11") this.model.presetType = "pinkshades";
            else if (this._presetsId === "e-presets12") this.model.presetType = "moonlight";
            else if (this._presetsId === "e-presets20") this.model.presetType = "monochrome";
            else if (this._presetsId === "e-presets21") this.model.presetType = "misty";
            else if (this._presetsId === "e-presets22") this.model.presetType = "flatcolors";
            else if (this._presetsId === "e-presets30") this.model.presetType = "basic";
            else if (this._presetsId === "e-presets31") this.model.presetType = "candycrush";
            else if (this._presetsId === "e-presets32") this.model.presetType = "citrus";
            this._splitObj.option('prefixIcon', 'e-color-image e-' + this.model.presetType);
            this.PaletteWrapper.remove();
            if (this._modelType == "palette") {
                this.PaletteWrapper = this._layoutType(this.model.palette);
                this._gradient.addClass('e-hide');
                this._paletteType();
                this._switcher.addClass('e-pickerModel').removeClass('e-paletteModel');
            } 
        },

        _renderPresets: function () {
            var tableDiv = ej.buildTag("div.e-presets-table");
            this._spnTag = ej.buildTag("span.e-presetHeader");
            this._spnTag.html(this.model.buttonText.swatches);
            tableDiv.append(this._spnTag);
            var color = 0, hexCode, rowDiv;
            for (var row = 0; row < 4 ; row++) {
                rowDiv = ej.buildTag("ul.e-tablerow");
                for (var col = 0; col < 3 ; col++) {
                    tableCell = ej.buildTag("li.e-color-image e-preset-row" + "#" + "e-presets" + [row] + [col]);
                    tableCell.appendTo(rowDiv);
                }
                rowDiv.appendTo(tableDiv);
            }
            return tableDiv;
        },
        _renderPopupElement: function () {
            var proxy = this;
            this._hsv =
            {
                h: 360, s: 0, v: 100
            };
            this._rgb.ejButton({ text: "RGBA", type: "button" });
            this._hexCode.ejButton({ text: "HEX", type: "button" });
            this._hsva.ejButton({ text: "HSVA", type: "button" });            
            this._splitObj = this._presetTag.ejSplitButton({ size: "normal", showRoundedCorner: true, contentType: "imageonly" }).data('ejSplitButton');
            this._splitObj.option("beforeOpen", function (e) { proxy._bindClickOperation(e); });
            this._presetTag.parents('.e-split.e-widget').css({ "height": "27px" });
            this.model.custom.length == 0 ? this._splitObj.option('prefixIcon', "e-color-image e-" + this.model.presetType) : "";
            this._splitObj._getXYpos = function (e) {
                $("#" + this.model.targetID).ejMenu({ animationType: "none" });
                btnpos = this.dropbutton.offset();
                btnposx = btnpos.left - this.dropbutton.prev().outerWidth() - 1;
                btnposy = (btnpos.top - $("#" + this.model.targetID).outerHeight()) - 1;
                return { x: btnposx, y: btnposy }
            }
            this._colorSlider = this._hueSlider.ejSlider({ orientation: "Vertical", showTooltip: this.model.showTooltip, minValue: 0, maxValue: 360, change: function (e) { proxy._changeHue(e); }, slide: function (e) { proxy._changeHue(e); } }).data('ejSlider');
            this._opacity = this._alphaSlider.ejSlider({ value: this.opacityValue(), showTooltip: this.model.showTooltip, orientation: "Vertical", incrementStep: 5, value: 100, change: function (e) { proxy._changeAlpha(e); }, slide: function (e) { proxy._changeAlpha(e); } }).data('ejSlider');
            this._colorSlider.firstHandle.css({ "height": "13px", "margin": "0px 0px -12px -2px", "width": "13px" });
            this._opacity.firstHandle.css({ "height": "13px", "margin": "0px 0px -12px -2px", "width": "13px" });
            this.popupContainer.css({ "visibility": "visible", "display": "none" });
            if (this.model.modelType == "picker") {
                this._modelType = "picker";
                this._gradient.removeClass('e-hide');
                this.PaletteWrapper.addClass('e-hide');
                this._presetTag.parents('.e-split.e-widget').addClass('e-hide');
                this._showSwitcher();
                this._switch = true;
            } else if (this.model.modelType == "palette") {
                this._modelType = "palette";
                this._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
                this.PaletteWrapper.removeClass('e-hide');
                this._gradient.addClass('e-hide');
                this._hsva.ejButton("disable");
                this._showSwitcher();
                this._switch = false;
            } 
        },
        _layoutType: function (type) {
            if (typeof type === "string" && type == "basicpalette")
                this._collection = this._paletteGenerate(Colors[this.model.presetType], this.model.columns);
            else if (typeof type === "string" && type == "custompalette" && this.model.modelType == "palette")
                this._collection = this._paletteGenerate(this.model.custom, this.model.columns);
            type == "custompalette" ? this._collection.addClass('e-custom') : "";
            return this._collection;
        },
        _presetType: function (type) {
            if (type === "e-presets00") this._collection = this._paletteGenerate(Colors.webcolors, this.model.columns);
            else if (type === "e-presets01") this._collection = this._paletteGenerate(Colors.vintage, this.model.columns);
            else if (type === "e-presets02") this._collection = this._paletteGenerate(Colors.seawolf, this.model.columns);
            else if (type === "e-presets10") this._collection = this._paletteGenerate(Colors.sandy, this.model.columns);
            else if (type === "e-presets11") this._collection = this._paletteGenerate(Colors.pinkshades, this.model.columns);
            else if (type === "e-presets12") this._collection = this._paletteGenerate(Colors.moonlight, this.model.columns);
            else if (type === "e-presets20") this._collection = this._paletteGenerate(Colors.monochrome, this.model.columns);
            else if (type === "e-presets21") this._collection = this._paletteGenerate(Colors.misty, this.model.columns);
            else if (type === "e-presets22") this._collection = this._paletteGenerate(Colors.flatcolors, this.model.columns);
            else if (type === "e-presets30") this._collection = this._paletteGenerate(Colors.basic, this.model.columns);
            else if (type === "e-presets31") this._collection = this._paletteGenerate(Colors.candycrush, this.model.columns);
            else if (type === "e-presets32") this._collection = this._paletteGenerate(Colors.citrus, this.model.columns);
            return this._collection;
        },
        _paletteGenerate: function (colors, columns) {
            var color;
            this._PresetTable = ej.buildTag("div.e-palette-color").attr({ "role": "presentation" });
            this._tag = ej.buildTag("div.e-row");
            for (color = 0; color < colors.length; color++) {
                if (color && color % columns == 0)
                    this._tag = ej.buildTag("div.e-row");
                this._td = ej.buildTag("div.e-item").attr({ 'role': 'gridcell', "aria-label": "#" + $.trim(colors[color]), "value": "#" + $.trim(colors[color]), "style": "background-color:" + "#" + $.trim(colors[color]) });
                this._tag.append(this._td);
                this._PresetTable.append(this._tag);
            }
            return this._PresetTable;
        },
        _checkNameAttr: function () {
            if (!this.element.attr("name")) this.element.attr({ "name": this.element[0].id });
        },
        _enabled: function (bool) {
            if (bool) this.enable();
            else {
                this.model.enabled = true;
                this.disable();
            }
        },
        _setDisplayInline: function (isDisplayInline) {
            this.model.displayInline = isDisplayInline;
            if (isDisplayInline && this.element.is("input")) {
                this.popupContainer.insertAfter(this.wrapper);
                this._setPopupPosition();
            }
            else if (isDisplayInline) {
                this.element.append(this.popupContainer);
                this.popupContainer.find('button.e-applyButton').css({ "display": "none" });
                this.popupContainer.find('button.e-cancelButton').css({ "display": "none" });
                this._footer.css({ "display": "none" });
            }
            else {
                this.popupContainer.css('display', 'none');
                $('body').append(this.popupContainer);
                this._isOpen = false;
                if (this.element.is("input")) {
                    this._bindIconClick();
                    this.wrapper.removeClass("e-focus");
                }
                this._isFocused = this.isPopupOpen = false;
            }
            if (isDisplayInline) {
                this.show();
                if (this.element.is("input")) this._off($(this._buttonElement), "mousedown", this._iconClick);
            }
        },
        _bindIconClick: function () {
            var count = $._data($(this._buttonElement)[0], "events");
            if (ej.isNullOrUndefined(count) || ej.isNullOrUndefined(count.mousedown)) this._on(this._buttonElement, "mousedown", this._iconClick);
            else if (count.mousedown.length == 0) this._on(this._buttonElement, "mousedown", this._iconClick);
        },
        _unBindIconClick: function () {
            this._off(this._buttonElement, "mousedown", this._iconClick);
        },
        _setPopupPosition: function () {
            var elementObj = this.element.is('input') ? this.wrapper : this.element;
            var pos = this._getOffset(elementObj), winWidth,
            winBottomHeight = $(document).scrollTop() + $(window).height() - (pos.top + $(elementObj).outerHeight()),
            winTopHeight = pos.top - $(document).scrollTop(),
            popupHeight = this.popupContainer.outerHeight(),
            popupWidth = this.popupContainer.outerWidth(),
            left = pos.left,
            totalHeight = elementObj.outerHeight(),
            border = (totalHeight - elementObj.height()) / 2,
            maxZ = this._getZindexPartial(), popupmargin = 3,
            topPos = ((popupHeight < winBottomHeight || popupHeight > winTopHeight) ? pos.top + totalHeight + popupmargin : pos.top - popupHeight - popupmargin) - border;
            winWidth = $(document).scrollLeft() + $(window).width() - left;
            if (this.model.enableRTL || popupWidth > winWidth && (popupWidth < left + elementObj.outerWidth())) left -= this.popupContainer.outerWidth() - elementObj.outerWidth();
            this.popupContainer.css({
                "left": left + "px",
                'position': 'absolute',
                "top": topPos + "px",
                "z-index": maxZ
            });
        },

        _getOffset: function (ele) {
            return ej.util.getOffset(ele);
        },

        _getZindexPartial: function () {
            return ej.util.getZindexPartial(this.element, this.popupContainer);
        },

        _setValue: function (value) {
            var reg = "^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$";
            if (typeof value == "object" || (typeof value == "number") || ej.isNullOrUndefined(value.match(reg))) value = null;
            this.value(value);
            this._tempValue = value;
            ej.isNullOrUndefined(value) ? this._setEmptyValue() : this._renderModelValue(value);
            this._changeEvent(false);
            this._selectEvent();
            this.element.is("input") && this._updateValue();
        },
        _renderModelValue: function (value) {
            var color;
            if (value && typeof value === "string") {
                color = this._HexToHSV(value);
                this._oldValue = this.rgb;
                if (this._switch) {
                    this._valueOperation();
                    this._colorSlider.option('value', parseInt(this._hsv.h));
                }
                this._inputTagValue(this._selectedButton);
                if (!this._switch) {
                    this._updateUI();
                    this.element.val(this.value());
                }
                this._hueGradient();
            }
        },
        setValue: function (code) {
            this._setValue(code);
        },

        enable: function () {
            if (this.model.enabled) return false;
            if (this.wrapper && this.wrapper.hasClass("e-disable")) {
                this.wrapper.removeClass("e-disable");
				this.element.removeAttr("disabled");
                if (this.container.hasClass("e-disable")) this.container.removeClass('e-disable');
                this.popupList.removeClass('e-disable');
                this.dropdownbutton.removeClass('e-disable');
            } else if (this.model.displayInline) this.element.removeClass('e-disable');
            var temp = this._switch;
            this._switch = true;
            this._cancelObj.enable();
            this._colorSlider.enable();
            this._opacity.enable();
            this._splitObj.enable();
            this._applyObj.enable();
            this._wireEvents();
            this._switchEvents();
            this._switch = temp;
            $(this._inputTag).removeAttr('readonly');
            (this._buttonElement) && this._on(this._buttonElement, "mousedown", this._iconClick);
            this.model.enabled = true;
        },

        disable: function () {
            if (!this.model.enabled) return false;
            if (this.wrapper && !this.wrapper.hasClass("e-disable")) {
                this.wrapper.addClass("e-disable");
				this.element.attr("disabled","disabled");
                if (!this.container.hasClass("e-disable")) this.container.addClass('e-disable');
                this.popupList.addClass('e-disable');
                this.dropdownbutton.addClass('e-disable');
            } else if (this.model.displayInline) this.element.addClass('e-disable');
            var temp = this._switch;
            this._switch = false;
            this._cancelObj.disable();
            this._colorSlider.disable();
            this._opacity.disable();
            this._splitObj.disable();
            this._applyObj.disable();
            this._unWireEvents();
            this._unSwitchEvents();
            this._switch = temp;
            this._unBindIconClick();
            $(this._inputTag).attr('readonly', 'readonly');
            if (this.isPopupOpen && !this.model.displayInline) this.hide();
            this.model.enabled = false;
        },

        getColor: function () {
            return this.rgb;
        },

        getValue: function () {
            return this.value();
        },
        _alphaGradient: function (value) {
            var browser = ej.browserInfo();
            var value = ej.isNullOrUndefined(value) ? "#000000" : value;
            if (browser.name == "mozilla")
                this._alphaSlider.attr({ "style": "background:-moz-linear-gradient(center top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            else if ((browser.name == "msie") && (browser.version == "9.0"))
                this._alphaSlider.attr({ "style": "filter: progid:DXImageTransform.Microsoft.gradient( startColorstr=" + value + ", endColorstr='#ffffff',GradientType=0 );" });
            else if ((browser.name == "msie") && (browser.version == "8.0"))
                if (this.model.enableOpacity)
                    this._alphaSlider.attr({ "style": "filter: progid:DXImageTransform.Microsoft.gradient( startColorstr=" + value + ", endColorstr='#ffffff',GradientType=0 );" });
                else
                    this._alphaSlider.attr({ "style": "filter: progid:DXImageTransform.Microsoft.gradient( startColorstr=" + value + ", endColorstr='#ffffff',GradientType=0 ) alpha(opacity=35);" });
            else if (browser.name == "msie")
                this._alphaSlider.attr({ "style": "background:-ms-linear-gradient(top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            else if (browser.name == "chrome" || browser.name == "safari")
                this._alphaSlider.attr({ "style": "background:-webkit-linear-gradient(top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            else if (browser.name == "opera")
                this._alphaSlider.attr({ "style": "background:-o-linear-gradient(top," + value + ",#fff) repeat scroll 0 0 rgba(0, 0, 0, 0);" });
            if (browser.name == "msie" && browser.version == "8.0")
                this._alphaSlider.children(".e-handle").css({ "background": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
        },
        _hueGradient: function () {
            var temp = this._hsv;
            var value = { h: this._hsv.h, s: 100, v: 100 };
            this._hueSlider.children(".e-handle").css({ "background": this._formRGB(this.HSVToRGB(value)) });
            this._hsv = temp;
        },
        _updateColor: function ()
        {
            if (this.model.displayInline || !this.model.showApplyCancel) {
                this.value(this._tempValue);
                if (this.element.is("input")) this._updateValue();
                this._trigger("select", { value: this.value() });
                this._previousColor = this._tempValue;
            }
        },
        _changeEvent: function (element) {
            if (this._change && this._previousValue !== this._tempValue) {
                this._alphaGradient(this._tempValue);
                this._previousValue = this._tempValue;                
                this._trigger("change", { value: this._tempValue, changeFrom: element ? "slider" : "picker" });
                this._updateColor(element);
            }
        },
        _selectEvent: function () {
            if (this._previousColor !== this._tempValue) {
                this.value(this._tempValue);
                if (this.element.is("input")) this._updateValue();
                this.element.val(this.value());
                this._trigger("select", { value: this.value() });
                this._previousColor = this._tempValue;
            }
        },
        _changeHue: function (e) {
            this._handleArea.css("visibility", "visible");
            if (parseInt(this._hsv.h) !== parseInt(e.value) && this._switch) {
                this._hsv.h = Math.round(e.value);
                this._hueGradient();
                this._tempValue = this.RGBToHEX(this.rgb);
                this._hsvValue();
                this._changeEvent(true);
            }
        },
        _changeAlpha: function (e) {
            this._handleArea.css("visibility", "visible");
            if (this._switch) {
                this.rgb.a = e.value / 100;
                this._tempOpacity = parseInt(this.rgb.a * 100);
                this._changeOpacity();
            }
        },
        _changeOpacity: function () {
            this.rgb.a = this._tempOpacity / 100;
            if (this._browser.name = "msie" && this._browser.version == "8.0") {
                this._currentTag.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
                this._alphaSlider.children(".e-handle").css({ "background": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
            }
            else {
                this._currentTag.css("background-color", this._formRGBA(this.rgb));
                this._alphaSlider.children(".e-handle").css({ "background": this._formRGBA(this.rgb) });
            }
            this._inputTagValue(this._selectedButton);
        },
        _updateValue: function () {
            if (this.value()) {
                if (this._browser.name = "msie" && this._browser.version == "8.0") this.spanElement.css({ "background-color": this._formRGB(this._HexToRGB(this.value())), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
                else this.spanElement.css({ "background-color": this._formRGBA(this._HexToRGB(this.value())) });
            }
            else this.spanElement.removeAttr('style');
        },
        _bindClickOperation: function (e) {
            var proxy = this, splitMenu;
            proxy._on($("#" + this._id + "_Presets").find("li.e-preset-row"), "mousedown", proxy._colorPresetsClick);
            SplitMenu = $("#" + proxy._presetContainer.attr('id')).ejMenu("instance");
            SplitMenu.model.close = function () {
                proxy._splitObj.contstatus = false;
                proxy._off($("#" + this._id + "_Presets").find("li.e-preset-row"), "mousedown", proxy._colorPresetsClick);
            }
        },
        _wireEvents: function () {
            if (this.element.is('input')) {
                this._on(this.wrapper, "blur", this._targetBlur);
                this._on(this.wrapper, "focus", this._targetFocus);
                this._on(this.wrapper, "keydown", this._popupShown);
                this._on(this.colorContainer, "click", this._containerClick);
            }
            this._on(this._changeTag, "click", this._switchModel);
            this._on(this._groupTag, "click", this._groupButton);
            this._on(this._addTag, "click", this._addColor);
            this._on(this._codeEditor, "mouseenter", this._inputEvent);
            this._on(this._codeEditor, "mouseleave", this._inputEvent);
            this._on(this._inputTag, "blur", this._inputEvent);
            this._on(this._inputTag, "focus", this._inputEvent);
            this._on(this._inputTag, "keyup", this._inputEvent);
            this._on(this.popupContainer, "focus", this._targetFocus);
            this._on(this.popupContainer, "blur", this._targetBlur);
        },
        _unWireEvents: function () {
            if (!this.model.displayInline && this.element.is('input')) {
                this._off(this.wrapper, "blur", this._targetBlur);
                this._off(this.wrapper, "focus", this._targetFocus);
                this._off(this.wrapper, "keydown", this._popupShown);
                this._off(this.colorContainer, "click", this._containerClick);
            }
            this._off(this._changeTag, "click", this._switchModel);
            this._off(this._groupTag, "click", this._groupButton);
            this._off(this._addTag, "click", this._addColor);
            this._off(this._codeEditor, "mouseenter", this._inputEvent);
            this._off(this._codeEditor, "mouseleave", this._inputEvent);
            this._off(this._inputTag, "blur", this._inputEvent);
            this._off(this._inputTag, "focus", this._inputEvent);
            this._off(this._inputTag, "keyup", this._inputEvent);
            this._off(this.popupContainer, "focus", this._targetFocus);
            this._off(this.popupContainer, "blur", this._targetBlur);
        },
        _inputEvent: function (e) {
            if (!this._clearIcon) return;
            if (e.type === "focus") this._off(this._codeEditor, "mouseleave", this._inputEvent);
            if (e.type === "blur") this._on(this._codeEditor, "mouseleave", this._inputEvent);
            if (e.type === "keyup") this._inputTag.val() !== "" ? (this._clearIcon.show(), this._handleArea.css("visibility", "visible")) : (this._clearIcon.hide(), this._handleArea.css("visibility", "hidden"));
            if (e.type === "mouseleave" || e.type === "blur" || this._inputTag.val() == "") this._clearIcon.hide();
            else this._clearIcon.show();
        },
        _clearColor: function (e) {
            this._tempValue = "";
            this._inputTag.val("");
            this._setEmptyValue();
            if (e.type == "click") {
                this._clearIcon.hide();
                this._inputTag.focus();
            }
        },
        _containerClick: function () {
            if (this.model.buttonMode == "split") this._trigger("select", { value: this.value() });
        },
        _popupShown: function (e) {
            if (e.keyCode == 13) {
                this._showHidePopup();
                if (!this.isPopupOpen)
                    this._buttonClick(e);
                return false;
            }
        },
        _recentColor: function (e) {
            this._divTag.find('.e-select').removeClass('e-select').addClass('e-block');
            var hexCode = e.target.attributes.value, value;
            var rgbCode = e.target.attributes.style;
            if (ej.isNullOrUndefined(hexCode) || ej.isNullOrUndefined(rgbCode)) { this._change = false; return false; }
            $(e.target.parentNode).addClass('e-select').removeClass('e-block');
            var alpha = rgbCode.value.replace(/^(background-color:rgb|background-color:rgba)\(/, '').replace(/\)$/, '').replace(/\s/g, '').split(',');
            if (!ej.isNullOrUndefined(alpha[3])) {
                this._opacity.option('value', parseInt((parseFloat(alpha[3]) * 100).toFixed(2)));
                this.rgb.a = parseFloat(parseFloat(alpha[3]).toFixed(2));
            }
            else if (this._browser.name = "msie" && this._browser.version == "8.0") {
                value = parseInt(rgbCode.nodeValue.replace(/^(FILTER: alpha)\(/, '').split('=')[1].split(')')[0]);
                this._opacity.option('value', value);
                this.rgb.a = value / 100;
            }
            else {
                this._opacity.option('value', 100);
                this.rgb.a = 1;
            }
            this._HexToHSV(hexCode.value);
            this._inputTagValue(this._selectedButton);
            this._tempValue = this.RGBToHEX(this.rgb);
            this._valueOperation();
            this._colorSlider.option('value', parseInt(this._hsv.h));
            this._hueGradient();
            this._changeEvent(false);
            if (!this.model.displayInline || this.element.is("input"))
                this.wrapper.focus();
        },

        _handleClick: function (e) {
            e.preventDefault();
            if (this.model.displayInline)
                this._isFocused = true;
            this._handleArea.css("visibility", "visible");
            this.mouseDownPos = this._handlePos;
            this._gradientArea.bind("mousemove", $.proxy(this._handleMovement, this));
            $(document).bind("mouseup", $.proxy(this._handleUp, this));
        },
        _handleMove: function (e) {
            this._handleArea.css("visibility", "visible");
            this._handleMovement(e);
            this._focusWrapper(e);
        },
        _handleMovement: function (e) {
            if (!this.model.enabled) return false;
            if (this.element.is("input")) this.wrapper.focus();
            this._hsv.v = parseInt(100 * (this._gradientArea.height() - Math.max(0, Math.min(this._gradientArea.height(), (e.pageY - this._gradientArea.offset().top)))) / this._gradientArea.height(), 10);
            this._hsv.s = parseInt(100 * (Math.max(0, Math.min(this._gradientArea.width(), (e.pageX - this._gradientArea.offset().left)))) / this._gradientArea.width(), 10);
            this._hsvValue();
            this._tempValue = this.RGBToHEX(this.rgb);
            this._change = true;
            this._changeEvent(false);
        },
        _handleUp: function (e) {
            $(document).unbind('mouseup', this._handleUp);
            this._gradientArea.unbind('mousemove', this._handleMovement);
            this._focusWrapper(e);
            return false;
        },
        _handlePosition: function () {
            this._handlePos = this._handleArea ? {
                left: parseInt(this._gradientArea.width() * this._hsv.s / 100, 10) + "px",
                top: parseInt(this._gradientArea.height() * (100 - this._hsv.v) / 100, 10) + "px"
            } : "";
            this._handleArea.css({ "left": this._handlePos.left, "top": this._handlePos.top });
        },
        _addColor: function () {
            var collection = this._divTag.find('> div');
            value = this._selectedButton.html() != "HSVA" ? this._inputTag.val() : this._formRGBA(this.HSVToRGB(this._hsv));
            if (value !== "" && this._change) {
                if (this.model.showRecentColors && collection.length <= 12) {
                    $($(collection)[collection.length - 2]).remove();
                    this._generateLi();
                }
            }
            if (!this.model.displayInline || this.element.is("input")) this.wrapper.focus();
        },
        _buttonClick: function (e) {
            this._change = true;
            var value = this._inputTag.val(), collection = this._divTag.find('div');
            this._opacity.option('value', this._tempOpacity);
            this._tempValue = this.RGBToHEX(this.rgb);
            this._updatePreviewColor();
            if (this._inputTag.val() === "") {
                this._tempValue = "";
                if (this.model.showClearButton) this._setValue("");
                else {
                    this._inputTag.addClass('e-error');
                    return false;
                }
            }
            this._selectEvent();
            if (this.element.is("input")) {
                this._updateValue();
                this.wrapper.focus();
            }
            !this.model.displayInline && this.hide();
            this.opacityValue(this._tempOpacity);
        },
        _generateLi: function () {
            this._liTag = ej.buildTag('div.e-colorblock e-block').attr({ "value": this.RGBToHEX(this.rgb), "tabindex": "0" });
            var spanTag = ej.buildTag('div.e-color e-set').attr({ "value": this.RGBToHEX(this.rgb), "title": this.RGBToHEX(this.rgb) });
            if (this._browser.name = "msie" && this._browser.version == "8.0") spanTag.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
            else spanTag.css({ "background-color": this._formRGBA(this.rgb) });
            this._liTag.append(spanTag);
            this._divTag.prepend(this._liTag);
        },
        _colorCodeValue: function (e) {
            var newValue = "", codeValue = this._inputTag.val(), value, code, count;
            value = $.trim(codeValue);
            value.length == 5 ? this._inputTag.removeClass('e-error') : "";
            if ((e.shiftKey && e.keyCode >= 35 && e.keyCode <= 40) || (e.ctrlKey && (e.keyCode == 88 || e.keyCode == 86)) || e.keyCode == 190)
                this._keyPressFlag = 1;
            else if ((!e.crtlKey && !e.shiftKey) && ((e.keyCode >= 65 && e.keyCode < 71) || (e.keyCode >= 35 && e.keyCode <= 40) || (e.keyCode >= 48 && e.keyCode <= 57) || e.keyCode == 13 || e.keyCode == 8 || e.keyCode == 46 || e.type === "blur"))
                this._keyPressFlag = 1;
            else this._keyPressFlag = 0;
            if (this.model.enableOpacity && (e.keyCode == 188 || e.keyCode == 71 || e.keyCode == 82) || (e.shiftKey && (e.keyCode == 57 || e.keyCode == 48)))
                this._keyPressFlag = 1;
            if (this._keyPressFlag == 1) {
                this._inputTag.removeClass('e-error');
                if (e.keyCode === 13 || e.type === "blur") {
                    if (value === "") {
                        if (this.model.showClearButton) this._setEmptyValue();
                        else this._inputTag.addClass('e-error');
                    }
                    var regex = /^\#([a-fA-F0-9]{6}|[a-fA-F0-9]{3})$/;
                    code = value.match(regex);
                    if (!ej.isNullOrUndefined(code)) {
                        if (code[1].length === 3) {
                            for (count = 0; count < code[1].length; count++) {
                                newValue += code[1][count] + code[1][count];
                            }
                        }
                        else if (code[1].length === 6)
                            newValue = code[1];
                        value = this.hexCodeToRGB("#" + newValue);
                        this._inputTag.val("#" + newValue);
                        this.rgb = this.HSVToRGB(this.RGBToHSV(value));
                        this._tempValue = this.RGBToHEX(this.rgb);
                        this._change = true;
                    }
                    else {
                        var rgbRegex = /^rgba?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/;
                        code = value.match(rgbRegex);
                        if (!ej.isNullOrUndefined(code)) this._rgbaColor(code);
                        else {
                            var hsvRegex = /^hsva?\((\d+),\s*(\d+),\s*(\d+)(?:,\s*(\d+(?:\.\d+)?))?\)$/;
                            code = value.match(hsvRegex);
                            if (!ej.isNullOrUndefined(code)) {
                                this._hsvColor(code);
                            }
                            else {
                                value != "" && this._inputTag.addClass('e-error');
                                this._change = false;
                                return false;
                            }
                        }
                    }
                    if (this._change) {
                        this._valueOperation();
                        this._colorSlider.option('value', parseInt(this._hsv.h));
                        this._hueGradient();
                        this._changeEvent(false);
                        !this.element.is('input') && this._selectEvent();
                        if (this._inputTag.val() !== "") this._inputTag.removeClass("e-error");
                    }
                }
            }
            else {
                if (e.keyCode != 9) e.preventDefault();
                if (!e.shiftKey && !e.ctrlKey && e.keyCode !== 27) this._inputTag.addClass('e-error');
            }
        },
        _setEmptyValue: function () {
            this._handleArea.css("visibility", "hidden");
            this._currentTag.css({ "background-color": "" });
            this._removeClass();
            this._tempValue = null;
            this._inputTag.val(null);
            if (this._previousValue !== this._tempValue) {
                this._trigger("change", { value: null });
                this._previousValue = this._tempValue
            }
        },
        _rgbaColor: function (code) {
            var rgb = {}, color;
            if (code[0].split('(')[0] == "rgba" && !ej.isNullOrUndefined(code[4])) {
                rgb.r = code[1]; rgb.g = code[2]; rgb.b = code[3]; rgb.a = code[4];
            }
            else if (code[0].split('(')[0] == "rgb" && ej.isNullOrUndefined(code[4])) {
                rgb.r = code[1]; rgb.g = code[2]; rgb.b = code[3]; rgb.a = this.rgb.a;
                this._inputTag.val("rgba(" + rgb.r + "," + rgb.g + "," + rgb.b + "," + rgb.a + ")");
            }
            else {
                this._inputTag.addClass('e-error');
                this._change = false;
                return false;
            }
            this.rgb.a = parseFloat(rgb.a);
            this._tempOpacity = this.rgb.a * 100;
            this._opacity.option('value', this._tempOpacity);
            this.opacityValue(this._tempOpacity);
            this._tempValue = this.RGBToHEX(rgb);
            this._HexToHSV(this._tempValue);
            this._inputTag.removeClass("e-error");
            this._change = true;
        },
        _hsvColor: function (code) {
            var hsv = {};
            if (!ej.isNullOrUndefined(code[4])) {
                hsv.h = code[1]; hsv.s = code[2]; hsv.v = code[3]; hsv.a = code[4];
            }
            else {
                this._inputTag.addClass('e-error');
                this._change = false;
                return false;
            }
            this.rgb.a = parseFloat(hsv.a);
            this.rgb = this.HSVToRGB(hsv);
            this._tempValue = this.RGBToHEX(this.rgb);
            this._inputTag.removeClass("e-error");
            this._change = true;
        },
        _iconClick: function (e) {
            e.preventDefault();
            this._showHidePopup();
            this.wrapper.focus();
        },
        _showHidePopup: function () {
            if (this.model.displayInline) return false;
            if (!this.isPopupOpen) this.show();
            else {
                this.hide();
                this.wrapper.focus();
            }
        },

        hide: function () {
            var proxy = this;
            if (!this.isPopupOpen) return false;
            this.isPopupOpen = this._dataBind = false;
            if (this.element.is('input')) {
                this.wrapper.focus();
                this.wrapper.removeClass("e-active");
            }
            this.popupContainer.slideUp(200, "easeOutQuad", function () {
                proxy._tempOpacity = proxy.opacityValue();
                proxy.rgb.a = proxy._tempOpacity / 100;
                proxy._tempValue = proxy.value();
                proxy._renderModelValue(proxy.value());
                proxy._opacity.option('value', parseInt(proxy.opacityValue()));
                if (!proxy.model.displayInline)
                    proxy._off($(document), "mousedown", proxy._onDocumentClick);
                proxy._trigger("close");
            });
            if (!this.model.displayInline)
                this._off($(document), "mousedown", this._onDocumentClick);
            this._off(this._inputTag, "keydown", this._colorCodeValue);
            this._off(this._inputTag, "blur", this._colorCodeValue);
            this._modelType == "palette" ? this._off($(document), "keydown", this._keyDown) : this._off($(document), "keydown", this._onKeyDown);
            this._off(ej.getScrollableParents(this.wrapper), "scroll", this.hide);
            $(window).unbind("resize", $.proxy(this._OnWindowResize, this));
        },
        _hidePopup: function () {
            !this.model.displayInline && this.hide();
        },

        show: function () {
            if (this.element.is("input")) {
                this.wrapper.focus();
                this.wrapper.addClass("e-active");
            }
            if (this.isPopupOpen || !this.model.enabled) return false;
            this.isPopupOpen = true;
            if (this.model.modelType == "palette") this._cellSelect();
            if (!this.model.displayInline && (this.value() === "" || ej.isNullOrUndefined(this.value())))
                this._setEmptyValue();
            else
                this._handleArea.css("visibility", "visible");
            this._previousColor = this._previousValue = this.value();
            this.popupContainer.children().find('.e-focus').removeClass('e-focus');
            if (!this.model.displayInline) this._setPopupPosition();
            var proxy = this;
            this.popupContainer.slideDown(200, "easeOutQuad", function () {
                proxy.isFocused = true;
                if (!proxy.model.displayInline)
                    proxy._on($(document), "mousedown", proxy._onDocumentClick);
                proxy._trigger("open");
            });
            if (!this._dataBind)
                this._modelType == "palette" ? this._on($(document), "keydown", this._keyDown) : this._on($(document), "keydown", this._onKeyDown);
            this._on(this._inputTag, "keydown", this._colorCodeValue);
            this._on(this._inputTag, "blur", this._colorCodeValue);
            this._dataBind = true;
            $(window).bind("resize", $.proxy(this._OnWindowResize, this));
            if (!this.model.displayInline) this._on(ej.getScrollableParents(this.wrapper), "scroll", this.hide);
            if (this._prevSize !== $(window).width()) this.refresh();
        },
        _showBindEvents: function () {
            this._modelType == "palette" ? this._on($(document), "keydown", this._keyDown) : this._on($(document), "keydown", this._onKeyDown);
        },
        _hideUnBindEvents: function () {
            this._off($(document), "keydown");
            this._modelType == "palette" ? this._off($(document), "keydown", this._onKeyDown) : this._off($(document), "keydown", this._keyDown);
        },
        _switchEvents: function () {
            if (this._switch) {
                this._on(this._gradientArea, "mousedown", this._handleMove);
                this._on(this._handleArea, "mousedown", this._handleClick);
                this._on(this._gradientArea, "mousedown", this._handleClick);
            }
            else this._on(this._collection, "mousedown", this._onSelect);
        },
        _unSwitchEvents: function () {
            if (!this._switch) {
                this._off(this._gradientArea, "mousedown", this._handleMove);
                this._off(this._handleArea, "mousedown", this._handleClick);
                this._off(this._gradientArea, "mousedown", this._handleClick);
            }
            else this._off(this._collection, "mousedown", this._onSelect);
        },
        _groupButton: function (e) {
            if ($(e.target).hasClass('e-disable')) return false;
            if ($(e.target).hasClass("e-button")) {
                var element = this._groupTag.find('.e-btn.e-select');
                if (this._inputTag.val() !== "") this._inputTagValue($(e.target));
                else this._selectedButton = $(e.target);
                this._selectedButton.html() !== "HEX" ? this._inputTag.attr('maxLength', '22') : this._inputTag.attr('maxLength', '7');
                this._groupTag.find('.e-click').removeClass('e-click');
                $(e.target).addClass('e-click');
            }
        },
        _inputTagValue: function (type) {
            if (type.html() == "RGBA") this._inputTag.val(this._formRGBA(this.rgb));
            else if (type.html() == "HEX") this._inputTag.val(this.RGBToHEX(this.rgb));
            else if (type.html() == "HSVA") {
                if (this.model.modelType != "palette")
                    this._inputTag.val("hsva(" + Math.round(this._hsv.h) + "," + Math.round(this._hsv.s) + "," + Math.round(this._hsv.v) + "," + this.rgb.a + ")");
            }
            this._selectedButton = type;
        },
        _bindRecentEvent: function () {
            this._on(this._divTag, "click", this._recentColor);
        },
        _unBindRecentEvent: function () {
            this._off(this._divTag, "click", this._recentColor);
        },
        _handlePlacement: function (prop, value, bool) {
            this._handleArea.css("visibility", "visible");
            var hsv = this._hsv;
            hsv[prop] += value * (bool ? 1 : 3);
            if (hsv[prop] < 0) { hsv[prop] = 0; }
            prop === "s" ? this._hsv.s = hsv[prop] : this._hsv.v = hsv[prop];
            this._hsvValue();
            this._tempValue = this.RGBToHEX(this.rgb);
            this._changeEvent(false);
        },
        _onKeyDown: function (e) {
            var key = e.keyCode;
            if (!this.model.enabled) return;
            if (e.shiftKey && key == 9) if ($(this._hueSlider).find('.e-handle').hasClass('e-focus')) this._focusWrapper(e);
            if (!this._isFocused) if (key == 9 || key !== 27) return;
            this._change = true;
            if ((!e.altKey && !e.shiftKey)) {
                switch (key) {
                    case 39:
                        if (this.element.is('input') && $(e.target).is(this.wrapper)) {
                            e.preventDefault();
                            this._handlePlacement("s", 1, e.ctrlKey);
                        }
                        break;
                    case 38:
                        if (this.element.is('input') && $(e.target).is(this.wrapper)) {
                            e.preventDefault();
                            this._handlePlacement("v", 1, e.ctrlKey);
                        }
                        break;
                    case 37:
                        if (this.element.is('input') && $(e.target).is(this.wrapper)) {
                            e.preventDefault();
                            this._handlePlacement("s", -1, e.ctrlKey);
                        }
                        break;
                    case 40:
                        if (this.element.is('input') && $(e.target).is(this.wrapper)) {
                            e.preventDefault();
                            this._handlePlacement("v", -1, e.ctrlKey);
                        }
                        break;
                    case 13:
                        e.preventDefault();
                        this._buttonClick(e);
                        break;
                    case 27:
                        e.preventDefault();
                        !this.model.displayInline && this.hide();
                        this._tempValue = this.value();
                        break;
                    case 9:
                        active = document.activeElement;
                        if ($(active).is(this.wrapper)) this._focusPopup(e);
                        break;
                }
            }
        },
        _focusPopup: function (e) {
            $(this._hueSlider).find('.e-handle').focus();
            e.preventDefault();
        },
        _focusWrapper: function (e) {
            this.element.is('input') && $(this.wrapper).focus();
            e.preventDefault();
        },
        _onDocumentClick: function (e) {
            if (!$(e.target).is(this.popupContainer) && !$(e.target).parents(".e-colorpicker").is(this.popupContainer) &&
              !$(e.target).is(this.wrapper) && !$(e.target).parents(".e-colorpickerwidget").is(this.wrapper) && !$(e.target).parents('.e-presetWrapper').is("#" + this._id + "_Presets")) {
                this.hide();
                this._isFocused = false;
                if (this.element.is('input')) this.wrapper.removeClass('e-focus');
            }
        },

        _OnWindowResize: function (e) {
            if (this.element.is('input')) this._setPopupPosition();
            this.refresh();
        },
        refresh: function () {
            var element = $(this._PresetTable.find('.e-item')[1]);
            var count = 10, paddingSize = 36; //paddingSize is fixed for  palette model cells
            if (!this.isPopupOpen) {
                this.popupContainer.css({ "display": "block", "visibility": "hidden" });
                this._modelType == "palette" && this.PaletteWrapper.css({ "display": "block", "visibility": "hidden" });
            } else if (this._modelType == "picker") {
                this.PaletteWrapper.css({ "display": "block", "visibility": "hidden" });
            }
            if (parseFloat(this._tag.outerHeight()) > element.outerHeight(true) || (($(this._tag).outerWidth() - ($(element).outerWidth(true) * count)) > element.outerWidth()) || $(this._tag).outerWidth() == 0) {
                var rowSize = parseFloat($(this._tag).outerWidth()) - paddingSize;
                var cellWidth = (rowSize / count) - (element.outerWidth() - element.width());
                this._PresetTable.find('.e-item').css('width', cellWidth);
                if (element.outerWidth(true) * count > this._tag.outerWidth()) this._PresetTable.find('.e-item').css('width', cellWidth - 1);
            }
            if (!this.isPopupOpen) {
                this.popupContainer.css({ "display": "none", "visibility": "visible" });
                this._modelType == "palette" && this.PaletteWrapper.css({ "visibility": "visible" });
            } else if (this._modelType == "picker") {
                this.PaletteWrapper.css({ "display": "none", "visibility": "visible" });
            }
            this._prevSize = $(window).width();
        },

        _range: function (range, value) {
            if (value === "") return value = 0;
            else if (value > range) return range;
            else return value;
        },
        _hsvValue: function () {
            var colorCode, hsv;
            this._change = true;
            this._hsv.v = this._hsv.v >= 100 ? 100 : this._hsv.v;
            this._hsv.s = this._hsv.s >= 100 ? 100 : this._hsv.s;
            this.hsv = this._hsv;
            this.rgb = this.HSVToRGB(this.hsv);
            this._valueOperation();
            this._inputTagValue(this._selectedButton);
        },
        _formRGB: function (value) {
            if (!ej.isNullOrUndefined(value)) return "rgb(" + value.r + "," + value.g + "," + value.b + ")";
        },
        _formRGBA: function (value) {
            if (!ej.isNullOrUndefined(value)) return "rgba(" + value.r + "," + value.g + "," + value.b + "," + value.a + ")";
        },
        _rgbValue: function (e) {
            var rgbColor, colorCode;
            value = this._HexToRGB(this._tempValue);
            if (!ej.isNullOrUndefined(value)) {
                this.rgb = value;
                this._change = true;
                this.HSVToRGB(this.RGBToHSV(this.rgb));
                this._colorSlider.option('value', parseInt(this._hsv.h));
                this._opacity.option('value', this.rgb.a * 100);
            }
            this._valueOperation();
            this._inputTagValue(this._selectedButton);
        },
        _valueOperation: function () {
            this._handlePosition();
            this._alphaGradient(this._tempValue);
            this._inputTag.removeClass("e-error");
            this._updateUI();
        },
        _HexToHSV: function (hex) {
            return this.HSVToRGB(this.RGBToHSV(this._HexToRGB(hex)));
        },
        _HexToRGB: function (hex) {
            if (!ej.isNullOrUndefined(hex)) {
                var reg = "^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$", hex, validate = hex.match(reg);
                if (ej.isNullOrUndefined(validate)) { this._change = false; return false; }
                if (validate[1].length == 3)
                    hex = "#" + validate[1][0] + validate[1][0] + validate[1][1] + validate[1][1] + validate[1][2] + validate[1][2];
                this._change = true;
                hex = parseInt(((hex.indexOf('#') > -1) ? hex.substring(1) : hex), 16);
                value = ej.isNullOrUndefined(this.rgb) ? parseFloat(this._tempOpacity) / 100 : this.rgb.a;
                return this.rgb = { r: hex >> 16, g: (hex & 0x00FF00) >> 8, b: (hex & 0x0000FF), a: value };
            }
        },

        RGBToHSV: function (rgb) {
            var hsv = { h: 0, s: 0, v: 0 };
            var min = Math.min(rgb.r, rgb.g, rgb.b);
            var max = Math.max(rgb.r, rgb.g, rgb.b);
            var differ = max - min;
            hsv.v = max;
            hsv.v *= 100 / 255;
            if (differ === 0) {
                this._hsv = hsv;
                return hsv;
            }
            hsv.s = max != 0 ? 255 * differ / max : 0;
            if (hsv.s != 0) {
                if (rgb.r == max) hsv.h = (rgb.g - rgb.b) / differ;
                else if (rgb.g == max) hsv.h = 2 + (rgb.b - rgb.r) / differ;
                else hsv.h = 4 + (rgb.r - rgb.g) / differ;
            } else hsv.h = -1;
            hsv.h *= 60;
            if (hsv.h < 0) hsv.h += 360;
            hsv.s *= 100 / 255;
            this._hsv = hsv;
            return hsv;
        },

        HSVToRGB: function (hsv) {
            var rgb = {};
            var h = Math.round(hsv.h);
            var s = Math.round(hsv.s * 255 / 100);
            var v = Math.round(hsv.v * 255 / 100);
            if (s == 0) {
                rgb.r = rgb.g = rgb.b = v;
            } else {
                var t1 = v;
                var t2 = (255 - s) * v / 255;
                var t3 = (t1 - t2) * (h % 60) / 60;
                if (h == 360) h = 0;
                if (h < 60) { rgb.r = t1; rgb.b = t2; rgb.g = t2 + t3 }
                else if (h < 120) { rgb.g = t1; rgb.b = t2; rgb.r = t1 - t3 }
                else if (h < 180) { rgb.g = t1; rgb.r = t2; rgb.b = t2 + t3 }
                else if (h < 240) { rgb.b = t1; rgb.r = t2; rgb.g = t1 - t3 }
                else if (h < 300) { rgb.b = t1; rgb.g = t2; rgb.r = t2 + t3 }
                else if (h < 360) { rgb.r = t1; rgb.g = t2; rgb.b = t1 - t3 }
                else { rgb.r = 0; rgb.g = 0; rgb.b = 0 }
            }
            this._hsv = hsv;
            var value = ej.isNullOrUndefined(this.rgb) ? parseFloat(this._tempOpacity) / 100 : this.rgb.a;
            return { r: Math.round(rgb.r), g: Math.round(rgb.g), b: Math.round(rgb.b), a: value }
        },
        _HSVToHex: function (hsv) {
            return this.RGBToHEX(this.HSVToRGB(hsv));
        },
        _toHEX: function (rgb) {
            if (rgb.indexOf("#") !== -1) return rgb;
            rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
            return "#" + this._hex(rgb[1]) + this._hex(rgb[2]) + this._hex(rgb[3]);
        },



        RGBToHEX: function (rgb) {
            if (!ej.isNullOrUndefined(rgb))
                return "#" + this._hex(rgb.r) + this._hex(rgb.g) + this._hex(rgb.b);
        },
        _hex: function (x) {
            return ("0" + parseInt(x).toString(16)).slice(-2);
        },
        _colorValue: function (value) {
            this._color = value.indexOf("#") != -1 ? this.hexCodeToRGB(value) : "";
            return "rgb(" + this._color.r + ", " + this._color.g + ", " + this._color.b + ")";
        },


        hexCodeToRGB: function (colorCode) {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(colorCode);
            return result ? {
                r: parseInt(result[1], 16),
                g: parseInt(result[2], 16),
                b: parseInt(result[3], 16),
                a: this.rgb.a
            } : null;
        },

        _updatePreviewColor: function () {
            if (this._browser.name = "msie" && this._browser.version == "8.0")
                this._previousTag.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
            else
                this._previousTag.css({ "background-color": this._formRGBA(this.rgb) });
            this._oldValue = this.rgb;
        },
        _updateUI: function () {
            var value = this._hsv;
            if (this._switch) {
                var hsv = { h: this._hsv.h, s: 100, v: 100 };
                this._gradientArea.css({ "background-color": this._colorValue(this._HSVToHex(hsv)) });
            } else this._cellSelect();
            if (this._browser.name = "msie" && this._browser.version == "8.0") {
                this._currentTag.css({ "background-color": this._formRGB(this.rgb), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
                this._previousTag.css({ "background-color": this._formRGB(this._oldValue), "filter": "alpha(opacity=" + this.rgb.a * 100 + ")" });
            }
            else {
                this._currentTag.css({ "background-color": this._formRGBA(this.rgb) });
                this._previousTag.css({ "background-color": this._formRGBA(this._oldValue) });
                this._alphaSlider.children(".e-handle").css({ "background": this._formRGBA(this.rgb) });
            }
            this._hsv = value;
        },

        _targetFocus: function (e) {
            e.preventDefault();
            if (!this._isFocused) {
                this._isFocused = true;
                if (this.element.is("input")) this.wrapper.addClass("e-focus");
            }
        },
        _targetBlur: function (e) {
            this._isFocused = false;
            if (!this.isPopupOpen && this.element.is("input")) this.wrapper.removeClass("e-focus");
        },
        _switchModel: function () {
            this._tempValue = this.RGBToHEX(this.rgb);
            this.refresh();
            var proxy = this;
            this._off(this._changeTag, "click", this._switchModel);
            if (this._modelType == "palette") {
                this._modelType = "picker";
                this._switcher.removeClass("e-pickerModel").addClass("e-paletteModel");
                this.PaletteWrapper.fadeOut(300, function () {
                    proxy._presetTag.parents('.e-split.e-widget').addClass('e-hide');
                    proxy._gradient.fadeIn(300);
                    proxy._on(proxy._changeTag, "click", proxy._switchModel);
                });
                this._switch = true;
                this._rgbValue();
                this._hueGradient();
                this._hsva.ejButton("enable");
            } else {
                this.PaletteWrapper.remove();
                this._modelType = "palette";
                this.PaletteWrapper = this._layoutType(this.model.palette);
                this._gradient.addClass('e-hide');
                this._paletteType();
                this._switcher.removeClass("e-paletteModel").addClass("e-pickerModel");
                this._gradient.fadeOut(300, function () {
                    proxy._presetTag.parents('.e-split.e-widget').removeClass('e-hide');
                    if (ej.isNullOrUndefined(proxy.PaletteWrapper)) {
                        proxy.PaletteWrapper = proxy._layoutType(proxy.model.palette);
                        proxy._splitObj.option('prefixIcon', 'e-color-image e-' + proxy.model.presetType);
                        proxy.popupList.prepend(proxy.PaletteWrapper);
                    }
                    proxy.PaletteWrapper.fadeIn(300);
                    proxy._on(proxy._changeTag, "click", proxy._switchModel);
                });
                if (this.value() !== "")
                    this._cellSelect();
                this._disableHSVButton();
            }
            this.model.palette === "custompalette" && this._presetTag.parents('.e-split.e-widget').addClass('e-hide'); 
            if (!this.model.displayInline || this.element.is(":input")) this.wrapper.focus();
            this._switchEvents();
            this._unSwitchEvents();
            if (this.isPopupOpen) {
                this._hideUnBindEvents();
                this._showBindEvents();
            }
        },
        _disableHSVButton: function () {
            if ($(this._groupTag.find('.e-click')).hasClass('e-hsvButton')) {
                this._inputTagValue(this._rgb);
                this._rgb.addClass('e-click');
                this._hsva.removeClass('e-click');
            }
            this._hsva.ejButton("disable");
        },

        _cellSelect: function () {
            var code, element, proxy = this;
            this._removeClass();
            this._collection.find('.e-item').each(function () {
                code = $(this).css("background-color");
                if (code && code.replace(/ /g, '') === proxy._formRGB(proxy.rgb)) {
                    element = this;
                }
            });
            $(element).addClass("e-state-selected").attr("aria-selected", true);
        },
        _removeClass: function () {
            this._collection.find('.e-item').removeClass('e-state-selected').removeAttr("aria-selected");
        },
        _position: function (items, element, columns) {
            items = Array.prototype.slice.call(items);
            var n = items.length,
            index = items.indexOf(element);
            if (index < 0) return columns < 0 ? items[n - 1] : items[0];
            index += columns;
            return items[index < 0 ? index += n : index %= n];
        },
        _onSelect: function (e) {
            if (!this.model.enabled) return false;
            this._isFocused = true;
            this._handleArea.css("visibility", "visible");
            if (e.target.style.backgroundColor != "") {
                this._collection.find('.e-item').removeClass("e-state-selected").removeAttr('aria-selected');
                this._HexToRGB(this._toHEX(e.target.style.backgroundColor));
                this._updateUI();
                this._inputTagValue(this._selectedButton);
                if (!this.model.displayInline || this.element.is("input")) this.wrapper.focus();
                this._tempValue = this.RGBToHEX(this.rgb);
                this._changeEvent(false);
            }
            e.preventDefault();
        },
        _keyDown: function (e) {
            if (this._isFocused) {
                this._change = true;
                if (!this.model.enabled) return false;
                var selected = "",
                    key = e.keyCode,
                    items = this._collection.find('.e-item'),
                    element = items.filter(".e-state-selected").get(0),
                    columnSize = this.model.columns;
                if (!e.altKey && (key == 37 || key == 38 || key == 39 || key == 40) && (e.target.className !== "e-color-code"))
                    this._removeClass();
                switch (!e.altKey && key) {
                    case 40:
                        if (e.target.className !== "e-color-code") {
                            e.preventDefault();
                            selected = this._position(items, element, columnSize);
                        }
                        break;
                    case 37:
                        if (e.target.className !== "e-color-code") {
                            e.preventDefault();
                            selected = this._position(items, element, -1);
                        }
                        break;
                    case 38:
                        if (e.target.className !== "e-color-code") {
                            e.preventDefault();
                            selected = this._position(items, element, -columnSize);
                        }
                        break;
                    case 39:
                        if (e.target.className !== "e-color-code") {
                            e.preventDefault();
                            selected = this._position(items, element, 1);
                        }
                        break;
                    case 13:
                        this._collection.find('.e-item').removeClass('e-state-selected').removeAttr("aria-selected");
                        this._buttonClick(e);
                        this._updateUI();
                        !this.model.displayInline && this.hide();
                        if (this.element.is("input")) this.wrapper.focus();
                        break;
                    case 27:
                        !this.model.displayInline && this.hide();
                        $(this._presetContainer).hide();
                        if (this.element.is("input")) this.wrapper.focus();
                        break;
                    case 9:
                        active = document.activeElement;
                        if ($(active).is(this.wrapper)) this._focusPalettePopup(e, true);
                        break;
                }
                if (selected) {
                    $(selected).addClass('e-state-selected').attr("aria-selected", true);
                    this._currentTag.css({ "background-color": this._formRGB(this._HexToRGB(selected.attributes.value.value)) });
                    this._inputTagValue(this._selectedButton);
                    this._tempValue = this.RGBToHEX(this.rgb);
                    this._changeEvent(false);
                }
            } else {
                if (e.keyCode == 27) {
                    this.hide();
                    $(this._presetContainer).hide();
                }
            }
        },
        _focusPalettePopup: function (e, type) {
            $(this.popupContainer).focus();
            e.preventDefault();
        },
    })

    ej.ColorPicker.Palette = {
        /**  Represents the basic Palette. This is default Type*/
        BasicPalette: "basicpalette",
        /**  Represents the custom Palette. User will customize the palette*/
        CustomPalette: "custompalette",
    }

    ej.ColorPicker.ModelType = {
        /**  Represents Palette Type*/
        Palette: "palette",
        /** Represents Picker Type */
        Picker: "picker"
    }

    ej.ColorPicker.ButtonMode = {
        /**  Represents the default bahavior*/
      Default: "dropdown",
        /**  Represents the Split bahavior, to perform the separete operation for each button*/
        Split: "split",
    }

    ej.ColorPicker.PresetType = {
        Basic: "basic",
        MonoChrome: "monochrome",
        FlatColors: "flatcolors",
        SeaWolf: "seawolf",
        WebColors: "webcolors",
        Sandy: "sandy",
        PinkShades: "pinkshades",
        Misty: "misty",
        Citrus: "citrus",
        Vintage: "vintage",
        MoonLight: "moonlight",
        CandyCrush: "candycrush",
    }
    var Colors = {
        basic: ["ffffff", "facbcb", "fccb98", "faf39a", "fbf8cd", "a6d38b", "aadee8", "d1ecf2", "cdcae5", "eecde1", "cccbcb", "f16667", "f69668", "f8ee6b", "f7ec37", "89c987", "75cddd", "8bd3e1", "9494c8", "9494c8", "b3b2b3", "ec2024", "f7971d", "ffcb67", "f5ea14", "74bf44", "69c8c9", "46c7f4", "6666ad", "b76cab", "676767", "971b1e", "ca6828", "ca9732", "979937", "0d9948", "339898", "4857a6", "62449a", "973794", "000000", "2f1110", "973620", "663433", "343416", "183319", "023334", "22205f", "3b2f8d", "310e31"],
        monochrome: ["ffffff", "e3e3e3", "c6c6c6", "aaaaaa", "8e8e8e", "717171", "555555", "393939", "1c1c1c", "000000", "f9e6e7", "f4d0d2", "efbabc", "e9a4a7", "e48e92", "df787c", "da6267", "d44c52", "cf363c", "ca2027", "fff4ca", "ffeb9e", "fff0b4", "ffeb9e", "ffe788", "ffe272", "ffd947", "ffd531", "ffd01b", "ffcc05", "e4f4eb", "ccead9", "b4e0c7", "9cd6b5", "84cca3", "6dc190", "55b77e", "3dad6c", "25a35a", "0d9948", "e8f4f4", "d6e3eb", "c4d1e3", "b3c0da", "a1aed1", "8f9dc9", "7d8bc0", "6c7ab7", "5a68af", "4857a6"],
        flatcolors: ["7477b8", "488bca", "18b1d4", "1db369", "78c162", "acc063", "ffe84e", "f6b757", "f79853", "ed6346", "E87F3D", "E4C45D", "B7A575", "999999", "67809F", "002228", "00A578", "F9A41F", "F3770B", "D7401B", "FFCB36", "82CC2C", "36B595", "6370AD", "D4264E", "004D8E", "22A04B", "F3A414", "C77405", "F3420B", "1ABC9C", "3498DB", "9B59B6", "E67E22", "E74C3C", "3A738A", "EBD9A7", "89AD78", "FF766D", "C76160", "BF3542", "CDC5BA", "EBE3D6", "3C3C3C", "2E2E2E", "77A7FB", "E57368", "FBCB43", "34B67A", "FFFFFF"],
        seawolf: ["0EEAFF", "15A9FA", "1B76FF", "1C3FFD", "2C1DFF", "0B3C4C", "0E5066", "13647F", "127899", "1A8BB2", "74B8E8", "659EBB", "3C9FFF", "26466F", "2472FF", "0069A4", "009BF2", "004165", "49A0B4", "274C5F", "000000", "7A5848", "E0A088", "F9DEC6", "3A2A22", "DC3522", "D9CB9E", "374140", "2A2C2B", "1E1E20", "CB3937", "FE6B2C", "654E44", "6DD16F", "70FE2C", "275673", "4681A6", "FDDEC9", "F22816", "400101", "071C2F", "388494", "E6A934", "F3DB5F", "534329", "206956", "47683B", "E1BFA6", "BF7950", "903932"],
        webcolors: ["0066aa", "00bbdd", "338800", "77bb00", "ffcc99", "990c0c", "0303c9", "336699", "669933", "cccccc", "EEEEEE", "E7C36F", "F7B230", "E35B20", "000033", "7D7A74", "BD524A", "FCB200", "8CFCC2", "2ACD6B", "666666", "666553", "FFFEEC", "B2B2A4", "AAA4B2", "9CA5E3", "5A668C", "BBA469", "CFC295", "FFFFFF", "DBBF56", "2E94B3", "808080", "E96656", "14A168", "DE185B", "D8806F", "DBE186", "D8CC63", "DCC527", "4E6C89", "E2BDAD", "EC6053", "81BBAD", "DFCDA5", "453394", "66398A", "313E7D", "336694", "788E91"],
        sandy: ["c0a986", "ecd9c3", "dfc79b", "f6d58e", "ecdaad", "fff3e0", "7f6b4a", "ffd694", "7f7a70", "ccac76", "E6E2AF", "A79A71", "EFECCA", "806F4C", "2F2F2E", "997F1A", "CCB65F", "FFD291", "6B674A", "635F3A", "7F693A", "FFEBC1", "FFD275", "7F7561", "CCA85E", "D29854", "4A4034", "C9AD8D", "4A351D", "968169", "E6E39F", "9A9757", "FFFDC9", "94909A", "E2E0E6", "960010", "EB1517", "CD7C29", "9A571C", "1F7A94", "7F6826", "7F724C", "FFE499", "FFD04C", "CCA63D", "FFA669", "92FFB6", "FFF352", "E8C269", "D7E8CB"],
        pinkshades: ["F6B1C3", "F0788C", "DE264C", "BC0D35", "A20D1E", "E12256", "BB1C48", "7B132F", "3B0917", "FA2660", "FFB7B5", "9A423F", "FF6D68", "BB5855", "CC5753", "E88161", "D66C60", "C2646E", "996072", "705262", "FFA1BD", "FF8FB7", "FF82AE", "E9719B", "CC6882", "F250C7", "BF1774", "BF2696", "AC60AA", "BB90C5", "BF1553", "F20775", "F2F2F2", "A68F63", "261414", "FFFFFF", "7F3443", "FF6887", "7F3443", "CC536C", "D06AA9", "E65F41", "650017", "BC1620", "FA427E", "3B1132", "84476E", "B83D65", "E6E0E8", "FF6EE8"],
        misty: ["5C7A84", "3D5372", "7C9196", "50748A", "ADBFBF", "010735", "052159", "194073", "376B8C", "FFFFFF", "985999", "C811CC", "892EFF", "FF6852", "DBA211", "0A0D0C", "85A67C", "46593E", "BBD9AD", "202616", "BF8E63", "734327", "A66C4B", "593A2F", "BFBFBF", "8DB0B6", "1B778A", "F46C1B", "881801", "192129", "81808C", "ABAABF", "0C0E09", "6A7366", "37402F", "5D6663", "84867B", "A4A66A", "BABBB1", "20211C", "6B9695", "646E8C", "6B8196", "61787F", "648C80", "8E9FBA", "89A8C8", "799ED1", "7FAEE7", "849EBD"],
        citrus: ["FAEA41", "E7F03E", "E3C647", "FAC541", "F0AB3E", "CCCA1F", "FFF300", "FFCB0D", "FF9500", "804A00", "6A692A", "FFFCA0", "FFFFFF", "CF664E", "EFAC66", "EFF299", "F2DC6D", "F29727", "F2600C", "592202", "214001", "4F7302", "1A2601", "BCD97E", "C0D904", "AAFEFF", "359D6D", "E5FF45", "65FCCF", "ABDC4B", "42B200", "C6FF00", "F2E304", "FFB200", "FF8600", "52EC04", "04E206", "94D507", "ECE404", "E2C904", "DA321C", "FF7913", "FBD028", "C0D725", "9FC131", "547312", "ADBF26", "DEB329", "F1DB47", "E08214"],
        vintage: ["684132", "fdbe30", "eaac21", "87783c", "3e4028", "ffc706", "cd5648", "5bafa9", "828282", "363636", "424862", "fb9a63", "bfc4d5", "f6fbf4", "febc98", "657050", "FCF082", "D8D98B", "A2AB80", "4D584A", "5ADED4", "4DAAAB", "26596A", "163342", "6C98A1", "010A26", "28403B", "557359", "AEBF8A", "C7D9AD", "AFFCCB", "CB4243", "D2997E", "36857E", "4AC6BB", "28394B", "191313", "AF0A18", "DC373D", "122438", "43734A", "A6A26D", "D9B448", "BF8C2C", "734002", "26010F", "866F53", "ACBD91", "7BAB87", "546859"],
        moonlight: ["241D37", "2A233D", "322B45", "362F49", "D4BA73", "261225", "592040", "8C3063", "A64985", "73345D", "A3C8FF", "85B6FF", "000040", "213190", "050859", "FFFFFF", "6AAED9", "4184BF", "224573", "A3C8FF", "CFC496", "B3B391", "889486", "61797B", "366577", "123340", "436E73", "7B8C61", "D7D996", "F0EBB4", "CFC496", "B3B391", "889486", "61797B", "366577", "123340", "436E73", "7B8C61", "D7D996", "F0EBB4", "341F36", "D9B5E0", "9889AB", "4D4E66", "1B2129", "5CBBE3", "FCF1BC", "5C8182", "383A47", "B4F257"],
        candycrush: ["0779f4", "30da00", "fb8006", "f9d802", "a71df7", "f70200", "fd49ae", "682e07", "9b2424", "5e7693", "F9AB3B", "EF5627", "FF0000", "00A398", "803C2C", "DE5711", "FFF026", "FF0048", "14A0CC", "00B229", "FFFFBE", "F7CD99", "FF77A1", "9886E8", "97CACB", "EAEDE5", "FFD127", "FF870C", "EC4610", "9A1900", "993460", "CC1464", "C300FF", "FFFFBC", "CCB914", "FFFEE2", "B24C5F", "FF274F", "0A94CC", "679DB2", "C2FFE6", "16B271", "5FFFBC", "B2442F", "FFA190", "E89359", "FFFB75", "F36EFF", "5999E8", "73EB86"],
    }

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html div elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejFileExplorer", "ej.FileExplorer", {

        element: null,

        model: null,
        validTags: ["div"],

        _rootCSS: "e-fileexplorer",

        defaults: {

            fileTypes: "*.*",

            filterSettings: {

                caseSensitiveSearch: false,

                filterType: "contains"
            },

            showToolbar: true,

            showNavigationPane: true,

            showContextMenu: true,

            showFooter: true,

            layout: "grid",

            locale: "en-US",

            selectedFolder: "",

            selectedItems: "",

            gridSettings: {
                allowSorting: true,
                columns: [
                    { field: "name", headerText: "Name", width: "25%" },
                    { field: "type", headerText: "Type", width: "20%" },
                    { field: "dateModified", headerText: "Date Modified", width: "35%" },
                    { field: "size", headerText: "Size", width: "15%", textAlign: "right", headerTextAlign: "left" }]
            },

            tools: {
                creation: ["NewFolder"],
                navigation: ["Back", "Forward", "Upward"],
                addressBar: ["Addressbar"],
                editing: ["Refresh", "Upload", "Delete", "Rename", "Download"],
                copyPaste: ["Cut", "Copy", "Paste"],
                getProperties: ["Details"],
                searchBar: ["Searchbar"]
            },

            toolsList: ["creation", "navigation", "addressBar", "editing", "copyPaste", "getProperties", "searchBar"],

            path: "",

            height: 400,

            width: 850,

            minWidth: "400px",

            maxWidth: null,

            minHeight: "250px",

            maxHeight: null,

            isResponsive: false,

            uploadSettings: {

                allowMultipleFile: true,

                maxFileSize: 31457280,

                autoUpload: false

            },

            enableResize: false,

            cssClass: "",

            enableRTL: false,

            ajaxAction: "",

            ajaxDataType: "json",

            ajaxSettings: {
                read: {},
                createFolder: {},
                remove: {},
                rename: {},
                paste: {},
                getDetails: {},
                download: {},
                upload: {},
                getImage: {}
            },

            allowMultiSelection: true,

            layoutChange: null,

            select: null,

            createFolder: null,

            remove: null,

            cut: null,

            copy: null,

            paste: null,

            open: null,

            beforeOpen: null,

            beforeUpload: null,

            beforeDownload: null,

            beforeAjaxRequest: null,

            resizeStart: null,

            resize: null,

            resizeStop: null

        },
        dataTypes: {
            filterSettings: "data",
            showToolbar: "boolean",
            showNavigationPane: "boolean",
            showContextMenu: "boolean",
            showFooter: "boolean",
            layout: "enum",
            gridSettings: "data",
            tools: {
                creation: "array",
                navigation: "array",
                addressBar: "array",
                editing: "array",
                copyPaste: "array",
                getProperties: "array",
                searchBar: "array"
            },
            toolsList: "array",
            uploadSettings: "data",
            ajaxSettings: "data",
        },
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "showToolbar":
                        options[prop] ? ej.isNullOrUndefined(this._toolBarItems) ? this._updateToolbar() : this._toolBarItems.show() : this._toolBarItems.hide();
                        this.adjustSize();
                        break;
                    case "showNavigationPane":
                    case "showTreeview":
                        this.model.showNavigationPane = this.model.showTreeview = options[prop];
                        this._showHideSplitBar(false);
                        break;
                    case "showContextMenu":
                        this._showHideContextMenu();
                        break;
                    case "height":
                        this.element.css("height", this._getProperValue(this.model.height));
                        this.adjustSize();
                        break;
                    case "width":
                        this.element.css("width", this._getProperValue(this.model.width));
                        this.adjustSize();
                        break;
                    case "layout":
                        this._switchLayoutView();
                        break;
                    case "path":
                        options[prop] ? this._setPath(options[prop]) : this._getPath();
                        break;
                    case "enableRTL": this._enableRTL(options[prop]); break;
                    case "showFooter":
                        if (options[prop]) {
                            if (this._statusbar.hasClass("e-statusbar")) {
                                var setHeight = this._splittag.find(".e-cont2").outerHeight() - this._statusbar.outerHeight();
                                this._tileView.height(setHeight);
                                this._gridtag.height(setHeight);
                                this._statusbar.show();
                            }
                            else {
                                this._createStatusBar();
                                this._updateData();
                                if (this.model.enableResize)
                                    this._resizeFileExplorer();
                                this._on($('#' + this._ExplorerId + '_switchGridView'), "click", this._switchView);
                                this._on($('#' + this._ExplorerId + '_swithListView'), "click", this._switchView);
                            }
                            var height = this._splittag.height() - this._gridtag.find(".e-gridheader").outerHeight();
                            this._tileContent.parent(".e-tile-wrapper").ejScroller({ height: this._splittag.outerHeight() - this._statusbar.outerHeight(), scrollerSize: this._scrollerSize });
                        } else {
                            this._tileView.height("auto");
                            this._gridtag.height("auto");
                            this._statusbar.hide();
                            var height = this._splittag.height() - this._gridtag.find(".e-gridheader").outerHeight();
                            this._tileContent.parent(".e-tile-wrapper").ejScroller({ height: this._splittag.outerHeight(), scrollerSize: this._scrollerSize });
                        }
                        break;
                    case "gridSettings":
                        this._gridtag.ejGrid(options[prop]);
                        break;
                    case "locale":
                        this._restrictRTL = true;
                        this.model.locale = options[prop];                        
                        this.element.html("");
                        this._init();
                        break;
                    case "cssClass": this._changeSkin(options[prop]); break;
                    case "fileTypes":
                        var proxy = this;
                        $.each(proxy._fileExplorer, function (itemPath, value) {
                            proxy._fileExplorer[itemPath] = "";
                        });
                        this._removeOldSelectionDetails();
                        this._refreshItems(this._selectedNode, this._currentPath);
                        this._uploadtag.ejUploadbox("option", { extensionsAllow: this.model.fileTypes == "*.*" ? "" : this.model.fileTypes.replace(/\*/g, "") });
                        this._updateItemStatus(this._fileExplorer[this._currentPath]);
                        break;
                    case "selectedFolder":
                        this._selectedFolder(this.model.selectedFolder);
                        break;
                    case "selectedItems":
                        this._setSelectedItems(options[prop]);
                        break;
                    case "allowMultiSelection":
                        if (this._gridtag.find(".e-gridheader").length)
                            this._gridtag.ejGrid("option", { selectionType: (options[prop] ? "multiple" : "single") });
                        break;
                    case "isResponsive": {
                        this.model.isResponsive = options[prop];
                        if (this._toolBarObj) {
                            this._toolBarObj.option("isResponsive", this.model.isResponsive);
                            this.model.showToolbar ? this._toolBarItems.show() : this._toolBarItems.hide();
                            this.adjustSize();
                        }
                        this._wireResizing();
                        break;
                    }
                    case "tools":
                    case "toolsList":
                        if (prop == "tools")
                            $.extend(this.model.tools, options[prop]);
                        else
                            this.model.toolsList = options[prop];
                        if (this._toolBarObj) {
                            this._toolBarObj.destroy();
                            this.element.find("#" + this._ExplorerId + "_toolbar").remove();
                            this._updateToolbar();
                            this.model.showToolbar ? this._toolBarItems.show() : this._toolBarItems.hide();
                            this.adjustSize();
                        }
                        break;
                    case "enableResize":
                        if (!options[prop])
                            this._resizeItem && this._resizeItem.remove();
                        else if (options[prop] && this.model.showFooter) {
                            this._resizeItem = ej.buildTag("div.e-icon e-fe-resize e-resize-handle");
                            this._resizeItem.insertBefore(this.element.find(".e-switchView"));
                            this._resizeFileExplorer();
                        }
                        break;
                    case "minHeight":
                        this.element.css("min-height", this._getProperValue(this.model.minHeight));
                        this._refreshResizeHandler();
                        break;
                    case "maxHeight":
                        this.element.css("max-height", this._getProperValue(this.model.maxHeight));
                        this._refreshResizeHandler();
                        break;
                    case "minWidth":
                        this.element.css("min-width", this._getProperValue(this.model.minWidth));
                        this._refreshResizeHandler();
                        break;
                    case "maxWidth":
                        this.element.css("max-width", this._getProperValue(this.model.maxWidth));
                        this._refreshResizeHandler();
                        break;
                }
            }
        },

        _init: function () {
            this._cloneElement = this.element.clone();
            (!ej.isNullOrUndefined(this.model.uploadBoxSettings)) && (this.model.uploadSettings = this.model.uploadBoxSettings);
            (!ej.isNullOrUndefined(this.model.showTreeview)) && (this.model.showNavigationPane = this.model.showTreeview);
            (!ej.isNullOrUndefined(this.model.move)) && (this.model.cut = this.model.move);
            this._initialize();
            this._render();
        },
        _postInit: function () {
            this._enablePostInit = false;
            !this._restrictRTL && this._enableRTL(this.model.enableRTL);
            this._wireEvents();
            this._wireResizing();
            this._setMinMaxSizeInInteger();
            if (this.model.enableResize && this.model.showFooter)
                this._resizeFileExplorer();
        },

        _initialize: function () {
            this._ExplorerId = this.element[0].id;
            this._fileExplorer = {};
            this._updateImages = {};
            this._selectedStates = [];
            this._updateOnNodeSelection = false;
            this._isClicked = true;
            this._tileView = null;
            this._originalPath = null;            
            this._initPath = "";
            this._initUpdate = false;
            this._scrollerSize = 8;
            this._editingToolsState = true;
            this.element.css({ "height": this._getProperValue(this.model.height), "width": this._getProperValue(this.model.width), "min-width": this._getProperValue(this.model.minWidth), "max-width": this._getProperValue(this.model.maxWidth), "min-height": this._getProperValue(this.model.minHeight), "max-height": this._getProperValue(this.model.maxHeight) });
            this._customCssClass = this.model.cssClass;
            this.element.addClass(this.model.cssClass);            
            this._isTreeNode = false;
            this._selectedItems = [];
            this._selectedTileItems = [];
            this._downloadDialog = null;
            this._newFolderDialog = null;
            this._renameDialog = null;
            this._openDialog = null;
            this._detailsDialog = null;
            this._alertDialog = null;
            this._enablePostInit = true;
            this._initialTime = new Date().getTime();
            this._currentPath = this.model.path.replace(/\\/g, "/");
            this._rootPath = this._currentPath = this._currentPath.endsWith("/") ? this._currentPath : this._currentPath + "/";
            this._gridObj = null;
            this._setUploadLocalization();
            this._restrictedToolbarOptions = [];
            this._restrictedMenuOption = [];
            this._contextMenu = {
                // navbarfolder menu
                navbar: ["NewFolder", "Upload", "|", "Delete", "Rename", "|", "Cut", "Copy", "Paste", "|", "Getinfo"],
                // current directory menu
                cwd: ["Refresh", "Paste", "|", "NewFolder", "Upload", "|", "Getinfo"],
                // current directory file menu
                files: ["Open", "Download", "|", "Upload", "|", "Delete", "Rename", "|", "Cut", "Copy", "Paste", "|", "Getinfo"]
            };   
        },
        _initContextMenuOptions: function (menu) {
            var menuOptions = [];
            var parameter = { "class": "e-fe-separator" };
            for (var i = 0; i < this._contextMenu[menu].length; i++) {
                if (this._contextMenu[menu][i] != "|")                    
                {
                    this["_menu" + this._contextMenu[menu][i]] = this._getLocalizedLabels("ContextMenu" + this._contextMenu[menu][i]);
                    menuOptions.push({ text: this["_menu" + this._contextMenu[menu][i]], sprite: "e-fileexplorer-toolbar-icon " + this._contextMenu[menu][i], htmlAttr: (this._contextMenu[menu][i + 1] == "|" ? parameter : "") });
                }                
           }
           return menuOptions;  
        },
        _render: function () {
            var selectedTreeFolder = this.model.selectedFolder;
            var selectedNodes = this.model.selectedItems;
            this.element.addClass('e-widget e-box').attr({ role: "fileexplorer", tabindex: 0 });
            this.model.showToolbar && this._renderToolBar();
            this._createSplitPane();
            this._read();
            if (selectedTreeFolder && this._currentPath != selectedTreeFolder)
                this._selectedFolder(selectedTreeFolder);
            if (selectedNodes.length) {
                this._isClicked = false;
                this._setSelectedItems(selectedNodes);
                this._isClicked = true;
            }
        },
        _read: function () {
            var proxy = this;
            var _ajaxOptions = {
                data: { ActionType: "Read", Path: this._currentPath, ExtensionsAllow: this.model.fileTypes, SelectedItems: this._getSelectedItemDetails(this._getFolderPath(), this._selectedContent) },
                url: this.model.ajaxAction,
                type: "POST",
                async: false,
                success: function (result) {
                    if (result.hasOwnProperty("d"))
                        result = result.d;
                    proxy._readSuccess(result.files);
                    proxy._enablePostInit && proxy._postInit();
                },
                successAfter: this.model.ajaxSettings.read.success
            };
            this.model.ajaxSettings.read.success = undefined;
            $.extend(true, _ajaxOptions, this.model.ajaxSettings.read);
            this._sendAjaxRequest(_ajaxOptions);
        },

        _sendAjaxRequest: function (ajaxOptions, hideWaitingState) {
            if (!ajaxOptions.dataType && this.model.ajaxDataType)
                ajaxOptions.dataType = this.model.ajaxDataType;
            if (this.model.ajaxAction == "" || this._currentPath == "")
                return;
            if (ajaxOptions.data.ActionType != "Read")
                this._selectedItemDetails = ajaxOptions.data.SelectedItems;
            var defaultData = { Name: "", ActionType: "", Path: "", ExtensionsAllow: "", LocationFrom: "", LocationTo: "", Action: "", NewName: "", Names:[], FileUpload: null, CommonFiles: null };
            if (!hideWaitingState) {
                this._waitingPopup.show();
            }
            $.extend(true, defaultData, ajaxOptions.data);
            var args = { data: defaultData };
            var proxy = this;
            if (this._trigger("beforeAjaxRequest", args))
                return;
            $.ajax({
                data: ((ajaxOptions.dataType && ajaxOptions.dataType.toLowerCase() == "jsonp") ? { json: JSON.stringify(args.data) } : JSON.stringify(args.data)),
                url: ajaxOptions.url,
                type: ajaxOptions.type,
                async: ajaxOptions.async,
                success: function(args) {
                    ajaxOptions.success.call(this, args);
                    if(typeof ajaxOptions.successAfter == "function")
                        ajaxOptions.successAfter.apply(this, arguments);
                },
                contentType: (ajaxOptions.contentType ? ajaxOptions.contentType : "application/json"),
                dataType: ajaxOptions.dataType,
                jsonpCallback: ajaxOptions.jsonpCallback ? ajaxOptions.jsonpCallback : ((ajaxOptions.dataType && ajaxOptions.dataType.toLowerCase() == "jsonp") ? "MyCallbackFunction" : ""),
                error: ajaxOptions.error ? ajaxOptions.error : function (xhr, textStatus, errorThrown) {
                    var text = xhr.responseJSON ? xhr.responseJSON.ExceptionType + ", " + xhr.responseJSON.ExceptionMessage : xhr.statusText;
                    proxy._alertDialog = proxy._createDialog(ej.buildTag('div.e-fe-dialog-label', text), { width: 400, height: "auto", title: proxy._getLocalizedLabels("Error") });
                    proxy._alertDialogObj = proxy._alertDialog.data("ejDialog");
                },
                beforeSend: ajaxOptions.beforeSend,
                complete: function () {
                    proxy._waitingPopup.hide();
                    ajaxOptions.complete;
                }
            });

        },

        _onBeforeOpen: function () {
            var fileUrl, selectedNodes = [], args;
            fileUrl = this._nodeType == "File" ? this._currentPath.replace("~", "..") + this._selectedFile : this._currentPath;
            if (this._selectedFile)
                selectedNodes = this._getSelectedItemDetails(this._currentPath, this._selectedFile);
            else if (this._selectedContent)
                selectedNodes = this._getSelectedItemDetails(this._getFolderPath(), this._selectedContent);
            args = { path: fileUrl, itemType: this._nodeType, selectedItems: selectedNodes };
            return this._trigger("beforeOpen", args);
        },

        _readSuccess: function (result) {
            this._update = false;
            var size = 0;
            for (var i = 0; i < result.length; i++) {
                size += result[i].size;
                result[i].sizeInByte = result[i].size;
                result[i].size = result[i].isFile ? this._bytesToSize(result[i].size) : "";
            }
            size = this._bytesToSize(size);
            this._fileExplorer[this._currentPath] = result;
            this._itemStatus && this._itemStatus.html(result.length + " " + (result.length == 1 ? this._getLocalizedLabels("Item") : this._getLocalizedLabels("Items")) + ", " + this._getLocalizedLabels("Size") + ": " + size);
            if (!this._treetag.hasClass("e-treeview")) {
                this._renderTreeView(result);
                this._updateOnNodeSelection = true;
            }
            this.model.layout == "grid" ? this._renderGridView(result) : this._renderTileView(result);
        },
        _bytesToSize: function (bytes) {
            var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
            if (bytes == 0) return '0 Byte';
            var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
            var value = (bytes / Math.pow(1024, i));
            return value.toFixed(2) + ' ' + sizes[i];
        },

        _setPath: function (path) {
            this.model.path = path;
            this._treetag.remove();
            this._treetag = ej.buildTag('div#' + this._ExplorerId + '_treeView');
            this._splittag.find(".e-cont1 > .e-tree-wrapper").append(this._treetag);
            this._fileExplorer = this._updateImages= {};             
            this._selectedStates = this._selectedItems= this._selectedTileItems= [];             
            this._initUpdate = false;
            this._initPath = "";
            this._currentPath = this.model.path.replace(/\\/g, "/");
            this._originalPath = this._rootPath = this._currentPath = this._currentPath.endsWith("/") ? this._currentPath : this._currentPath + "/";
            this._read();            
        },

        _getPath: function () {
            return this.model.path;
        },
        _changeSkin: function (value) {
            this.element.removeClass(this._customCssClass).addClass(value);
            this._customCssClass = value;
            this._waitingPopup.option("cssClass", value);
            this._subControlsSetModel("cssClass", value);
        },
        _enableRTL: function (value) {
            this._restrictRTL = false;
            this.model.enableRTL = value;
            if (value) {                
                var gridColumnSettings = this.model.gridSettings.columns;
                this._oldFormat = JSON.parse(JSON.stringify(gridColumnSettings));
                for (var i = 0; i < gridColumnSettings.length; i++) {
                    gridColumnSettings[i].textAlign = gridColumnSettings[i].headerTextAlign = "right";
                }
                this._gridObj && this._gridObj.columns(gridColumnSettings);                                   
                this.element.addClass("e-rtl");
                this.element.find(".e-scroller").addClass("e-rtl");
            }
            else {
                if (this._gridObj && this._oldFormat) 
                    this._gridObj.columns(this._oldFormat);                                    
                if (this._oldFormat)
                    this.model.gridSettings.columns = this._oldFormat;
                this.element.removeClass("e-rtl");
                this.element.find(".e-scroller").removeClass("e-rtl");
                (this._gridtag) && this._gridtag.removeClass('e-rtl');
            }
            this._subControlsSetModel("enableRTL", value);
        },
        _destroy: function () {
            this.element.html("");
            $(this._cloneElement).removeClass('e-fileexplorer e-js');
            this.element.replaceWith(this._cloneElement);
            this._treeContextMenutag && this._treeContextMenutag.parent().remove();
            this._tileContextMenutag && this._tileContextMenutag.parent().remove();
            this._downloadDialog && this._downloadDialog.parents(".e-dialog").remove();
            this._newFolderDialog && this._newFolderDialog.parents(".e-dialog").remove();
            this._renameDialog && this._renameDialog.parents(".e-dialog").remove();
            this._openDialog && this._openDialog.parents(".e-dialog").remove();
            this._detailsDialog && this._detailsDialog.parents(".e-dialog").remove();
            this._waitingPopup && this._waitingPopup.destroy();
        },
        _createFolder: function (_name) {
            var proxy = this;
            var _ajaxOptions = {
                data: { ActionType: "CreateFolder", Name: _name, Path: this._originalPath, SelectedItems: this._getSelectedItemDetails(this._originalPath, this._treeObj.getText(this._selectedNode)) },
                url: this.model.ajaxAction,
                type: "POST",
                success: function (result) {
                    if (result.hasOwnProperty("d"))
                        result = result.d;
                    var selectedNode = proxy._selectedNode;
                    var nodeObj = [{ id: result.files[0].name, name: result.files[0].name, spriteCssClass: "e-tree-foldericon", hasChild: false }];
                    if (!proxy._treetag.ejTreeView("isExpanded", selectedNode))
                        selectedNode.find(".e-icon").click();
                    proxy._nodeExpanded = true;
                    proxy._treeObj.addNode(nodeObj, selectedNode);
                    proxy._nodeExpanded = false;
                    proxy._update = true;
                    proxy._treeObj.selectNode(selectedNode);
                    proxy._setSelectedItems([result.files[0].name]);
                    proxy._refreshTreeScroller();
                    proxy._treetag.find("li").removeAttr("tabindex");
                    var args = { data: result, selectedItems: proxy._selectedItemDetails };
                    proxy._trigger("createFolder", args);
                },
                successAfter: this.model.ajaxSettings.createFolder.success
            };
            this.model.ajaxSettings.createFolder.success = undefined;
            $.extend(true, _ajaxOptions, this.model.ajaxSettings.createFolder);
            this._sendAjaxRequest(_ajaxOptions);
        },
        _cut_copy: function () {
            if (this._sourcePath == this._currentPath && this._option == "move") {
                this.element.find(".e-blur").removeClass("e-blur");
                this._setSelectedItems(this._fileName);
                return;
            }
            var data = this._fileExplorer[this._currentPath], _selectedFiles = (typeof this._fileName == "string") ? [this._fileName] : this._fileName;
            this._pastedFiles = _selectedFiles.slice();
            for (var i = 0; i < _selectedFiles.length; i++) {
                if (!this._isNameExist(this._fileExplorer[this._sourcePath], _selectedFiles[i]))
                    return;
            }
            if (ej.isNullOrUndefined(data)) {
                this._getFileDetails(this._currentPath);
                data = this._fileExplorer[this._currentPath];
            }
            this._existingItems = [];
            if (this._sourcePath != this._currentPath && data.length) {
                this._getDuplicateItems(this._sourcePath, this._currentPath, (typeof this._fileName == "string" ? [this._fileName] : this._fileName));
                if (this._existingItems.length) {
                    this._createReplaceConformationDiaolg("_pasteOperation", "ReplaceAlert");
                }
                else
                    this._pasteOperation();
            }
            else {
                for (var i = 0; i < _selectedFiles.length; i++) {
                    this._pastedFiles[i] = this._getDuplicateName(this._fileExplorer[this._sourcePath], _selectedFiles[i]);
                }
                this._pasteOperation();
            }

        },
        _createReplaceConformationDiaolg: function (executableFunction, alert) {
            var proxy = this;
            var i = 0;
            var viewerData = String.format(this._getLocalizedLabels(alert), this._existingItems[i].Path);
            var dialogContent = ej.buildTag('div.e-get-name');
            var labeltag = ej.buildTag('div.e-fe-dialog-label', viewerData);
            var divtag = ej.buildTag('div.e-fe-dialog-btn e-replace');
            var yesButton = ej.buildTag('button.e-fe-btn-yes ', this._getLocalizedLabels("YesButton"));
            var yesToAllButton = ej.buildTag('button.e-fe-btn-yes e-all ', this._getLocalizedLabels("YesToAllButton"));
            var noButton = ej.buildTag('button.e-fe-btn-no ', this._getLocalizedLabels("NoButton"));
            var noToAllButton = ej.buildTag('button.e-fe-btn-no e-all ', this._getLocalizedLabels("NoToAllButton"));
            yesButton.ejButton({
                type: "button",
                click: function () {
                    if ((/\.(gif|jpg|jpeg|tiff|png|bmp)$/i).test(proxy._existingItems[i].Name))
                        proxy._updateImages[proxy._existingItems[i].Path] = new Date().getTime();
                    proxy._existingItems[i].IsReplace = true;
                    i++;
                    if (i < proxy._existingItems.length)
                        proxy._alertDialog.find(".e-fe-dialog-label").text(String.format(proxy._getLocalizedLabels(alert), proxy._existingItems[i].Path));
                    else
                        proxy._destroyReplaceConformationDiaolg(executableFunction);
                }
            });
            noButton.ejButton({
                type: "button",
                click: function () {
                    proxy._existingItems[i].IsReplace = false;
                    i++;
                    if (i < proxy._existingItems.length)
                        proxy._alertDialog.find(".e-fe-dialog-label").text(String.format(proxy._getLocalizedLabels(alert), proxy._existingItems[i].Path));
                    else
                        proxy._destroyReplaceConformationDiaolg(executableFunction);
                }
            });
            yesToAllButton.ejButton({
                type: "button",
                click: function () {
                    for (var j = i; j < proxy._existingItems.length; j++) {
                        if ((/\.(gif|jpg|jpeg|tiff|png|bmp)$/i).test(proxy._existingItems[j].Name))
                            proxy._updateImages[proxy._existingItems[j].Path] = new Date().getTime();
                        proxy._existingItems[j].IsReplace = true;
                    }
                    proxy._destroyReplaceConformationDiaolg(executableFunction);
                }
            });
            noToAllButton.ejButton({
                type: "button",
                click: function () {
                    for (var j = i; j < proxy._existingItems.length; j++) {
                        proxy._existingItems[j].IsReplace = false;
                    }
                    proxy._destroyReplaceConformationDiaolg(executableFunction);
                }
            });
            divtag.append(yesButton, yesToAllButton, noButton, noToAllButton);
            $(dialogContent).append(labeltag, divtag);
            var open = function () {
                yesButton.focus();
            };
            this._alertDialog = this._createDialog(dialogContent, { width: 500, height: "auto", title: this._getLocalizedLabels("Error"), open: open });
            this._alertDialogObj = this._alertDialog.data("ejDialog");
            this._alertDialogObj.initialRender = false;
            this._alertDialogObj.open();
        },
        _destroyReplaceConformationDiaolg: function (executableFunction) {
            this._removeDialog(this._alertDialogObj);
            this[executableFunction]();
        },
        _pasteOperation: function () {
            this._removeBlurEffect();
            var proxy = this;
            var nodes = this._currentPath.split('/'), tempPath = this._sourcePath + this._fileName + "/";
            if (this._sourceType == "Directory" && this._currentPath.indexOf(tempPath) != -1) {
                var viewerData = this._getLocalizedLabels("CancelPasteAction");
                var dialogContent = ej.buildTag('div');
                var labeltag = ej.buildTag('div.e-fe-dialog-label', viewerData);
                var divtag = ej.buildTag('div.e-fe-dialog-centerbtn');
                var cancelButton = ej.buildTag('button.e-fe-btn-cancel ', this._getLocalizedLabels("CancelButton"));
                cancelButton.ejButton({
                    type: "button",
                    click: function () {
                        proxy._removeDialog(proxy._alertDialogObj);
                    }
                });
                divtag.append(cancelButton);
                $(dialogContent).append(labeltag, divtag);
                var open = function () {
                    cancelButton.focus();
                };
                this._alertDialog = this._createDialog(dialogContent, { width: 400, height: "auto", title: this._getLocalizedLabels("Error"), open: open });
                this._alertDialogObj = this._alertDialog.data("ejDialog");
                this._alertDialogObj.initialRender = false;
                this._alertDialogObj.open();
            }
            else {
                var _ajaxOptions = {
                    data: { ActionType: "Paste", LocationFrom: this._sourcePath, LocationTo: this._currentPath, Names: (typeof this._fileName == "string") ? [this._fileName] : this._fileName, Action: this._option, CommonFiles: this._existingItems, SelectedItems: this._getSelectedItemDetails(this._sourcePath, this._fileName), TargetFolder: this._getSelectedItemDetails(this._getFolderPath(), nodes[nodes.length - 2]) },
                    url: this.model.ajaxAction,
                    type: "POST",
                    success: function (result) {
                        if (typeof proxy._fileName == "string")
                            proxy._fileName = [proxy._fileName];
                        proxy._existingItems && proxy._existingItems.filter(function (item) {
                            if (item.IsReplace == false)
                            {
                                var index = $.inArray(item.Name, proxy._pastedFiles);
                                if (index > -1)
                                    proxy._pastedFiles.splice(index, 1);
                            }
                        })
                        proxy._existingItems = [];
                        var oldPath = proxy._originalPath;
                        var oldNode = proxy._selectedNode;
                        if (proxy._option == "move") {
                            for (var i = 0; i < proxy._fileName.length; i++) {
                                proxy._modifySelectedStates(proxy._sourcePath + proxy._fileName[i], "");
                            }
                            if (proxy._fileName.length == 1)
                                proxy._sourceType == "Directory" && proxy._treeObj.removeNode(proxy._refreshNode);
                            else {
                                nodes = proxy._refreshNode.siblings();
                                proxy._sourceType == "Directory" && proxy._treeObj.removeNode(proxy._refreshNode);
                                for (var i = 0; i < proxy._fileName.length; i++) {
                                    for (var j = 0; j < nodes.length; j++) {
                                        if (proxy._fileName[i] == $(nodes[j]).text())
                                            proxy._treeObj.removeNode(nodes[j]);
                                    }
                                }
                            }
                            proxy._fileExplorer[proxy._sourcePath] = "";
                        }
                        $.each(proxy._fileExplorer, function (path, value) {
                            //display the key and value pair
                            if (path.startsWith(oldPath))
                                proxy._fileExplorer[path] = "";
                        });
                        proxy._currentPath = proxy._originalPath;
                        proxy._highlightedNodes = proxy._pastedFiles;
                        proxy._refreshItems(oldNode, oldPath);                        
                        proxy._updateItemStatus(proxy._fileExplorer[oldPath]);
                        var folderPath = proxy._getFolderPath();
                        if (proxy._option == "move") {
                            proxy._fileName = "";
                            proxy._option = null;
                            proxy._toolBarItems && proxy._toolBarItems.ejToolbar("disableItemByID", proxy._ExplorerId + "Paste");
                            if (proxy.model.showContextMenu) {
                                proxy._viewMenuObj.disableItem(proxy._menuPaste);
                                proxy._treeMenuObj.disableItem(proxy._menuPaste);
                            }
                        }
                        var args = { name: proxy._fileName, targetPath: proxy.model.path, selectedItems: proxy._selectedItemDetails, targetFolder: proxy._getSelectedItemDetails(folderPath, proxy._selectedContent) };
                        proxy._trigger("paste", args);
                    },
                    successAfter: this.model.ajaxSettings.paste.success
                };
                this.model.ajaxSettings.paste.success = undefined;
                $.extend(true, _ajaxOptions, this.model.ajaxSettings.paste);
                this._sendAjaxRequest(_ajaxOptions);
            }
        },
        _deletion: function (names, path) {
            var proxy = this;
            var selectedNode = this._treeObj.getSelectedNode();
            if (!this._treeObj.isExpanded(selectedNode))
                selectedNode.find(".e-icon").click();
            var _ajaxOptions = {
                data: { ActionType: "Remove", Names: (typeof names=="string")? [names]: names , Path: path, SelectedItems: this._getSelectedItemDetails(path, names) },
                traditional: true,
                url: this.model.ajaxAction,
                type: "POST",
                success: function (result) {
                    proxy._selectedItemsTag && proxy._selectedItemsTag.html("");
                    proxy._fileExplorer[proxy._ajax_folderPath] = "";
                    $.each(proxy._fileExplorer, function (path, value) {
                        //display the key and value pair
                        if (path.startsWith(proxy._ajax_folderPath))
                            proxy._fileExplorer[path] = "";
                    });
                    for (var i = 0; i < _ajaxOptions.data.SelectedItems.length; i++) {
                        _ajaxOptions.data.SelectedItems[i].isFile == false && proxy._modifySelectedStates(path + _ajaxOptions.data.SelectedItems[i].name, "");
                    }                    
                    proxy._currentPath = proxy._ajax_folderPath;
                    var node = (proxy._selectedContent == proxy._selectedTreeText) ? proxy._parentNode : proxy._selectedNode;
                    proxy._refreshItems(node, proxy._ajax_folderPath);
                    proxy._updateItemStatus(proxy._fileExplorer[proxy._currentPath]);
                    if (proxy._treeObj.getSelectedNode().length == 0)
                        proxy._treeObj.selectNode(proxy._parentNode);
                    else {
                        if (proxy.model.layout == "tile")
                            proxy._tileViewWrapper.click();
                        else if (proxy.model.layout == "grid")
                            proxy._gridtag.find(".e-gridcontent").click();
                    }
                    var args = { data: result, path: proxy._ajax_folderPath, name: names, selectedItems: proxy._selectedItemDetails };
                    proxy._trigger("remove", args);
                },
                successAfter: this.model.ajaxSettings.remove.success
            };
            this.model.ajaxSettings.remove.success = undefined;
            $.extend(true, _ajaxOptions, this.model.ajaxSettings.remove);
            this._sendAjaxRequest(_ajaxOptions);
        },
        _rename: function () {
            var proxy = this;
            var _ajaxOptions = {
                data: { ActionType: "Rename", Path: proxy._currentPath, Name: proxy._selectedContent, NewName: proxy._ajax_person, CommonFiles: proxy._existingItems, SelectedItems: this._getSelectedItemDetails(proxy._currentPath, this._selectedContent) },
                url: this.model.ajaxAction,
                type: "POST",
                success: function (result) {
                    proxy._existingItems = [];
                    proxy._fileExplorer[proxy._currentPath] = "";
                    var node = (proxy._selectedContent == proxy._selectedTreeText) ? proxy._parentNode : proxy._selectedNode;
                    proxy._nodeType != "File" && proxy._modifySelectedStates(proxy._currentPath + proxy._selectedContent, proxy._currentPath + proxy._ajax_person);
                    if (proxy._splittag.find(".e-cont2 .e-active").length <= 0) {
                        $.each(proxy._fileExplorer, function (itemPath, value) {
                            if (itemPath.startsWith(proxy._currentPath + proxy._selectedContent + "/"))
                                proxy._fileExplorer[itemPath] = "";
                        });
                        node.find("> div > .e-text")[0].lastChild.nodeValue = proxy._ajax_person;
                        proxy._originalPath = proxy._currentPath += proxy._ajax_person + "/";
                        proxy._updateAddressBar();
                    }
                    else {
                        if (proxy.model.selectedItems.length > 0) {
                            proxy.model.selectedItems[proxy.model.selectedItems.length - 1] = proxy._ajax_person;
                            proxy._highlightedNodes = proxy.model.selectedItems;
                        }
                        proxy._refreshItems(node, proxy._currentPath);
                        if (proxy._treeObj.getSelectedNode().length == 0) {
                            $(proxy._parentNode).find('li a').each(function () {
                                if ($(this).text() == proxy._ajax_person) {
                                    proxy._treeObj.selectNode($(this).closest('li.e-item'));
                                    return false;
                                }
                            });
                        }
                    }
                },
                successAfter: this.model.ajaxSettings.rename.success
            };
            this.model.ajaxSettings.rename.success = undefined;
            $.extend(true, _ajaxOptions, this.model.ajaxSettings.rename);
            this._sendAjaxRequest(_ajaxOptions);
        },
        _downloadFile: function () {
            var selectedItems = this._getSelectedItemDetails(this._currentPath, this._selectedItems);
            var dataObj = {
                Path: this._currentPath,
                ActionType: "Download",
                SelectedItems: JSON.stringify(selectedItems),
                Names: this._selectedItems
            };            
            var url = this.model.ajaxAction + "?" + ($.param(dataObj, true));            
            if (this.model.ajaxSettings.download.url) {
                if (this.model.ajaxSettings.download.url.indexOf("{") >= 0)
                    url = String.format(this.model.ajaxSettings.download.url, "?" + ($.param(dataObj, true)));
                else
                    url = this.model.ajaxSettings.download.url;
            }
            var args = { path: this._currentPath, files: this._selectedItems, selectedItems: selectedItems };
            if (this._trigger("beforeDownload", args))
                return false;
            window.location = url;
        },
        _removeBlurEffect: function () {
            if (this._currntNode) {
                this._currntNode.hasClass("e-active") && this._currntNode.removeClass("e-blur");
                this._currntNode.find(".e-active").length && this._currntNode.find(".e-active").removeClass("e-blur");
            }
        },
        _renderTreeView: function (result) {
            var rootDIRID = 1, _hasChild = result.length > 0 ? true : false;
            this._collapse = false;
            var proxy = this;
            var nodes = this._currentPath.split("/");
            for (var i = 0; i < nodes.length - 2; i++) {
                this._initPath += nodes[i] + "/";
            }
            var startNode = nodes[nodes.length - 2];
            this._localData = [{ id: rootDIRID, name: startNode, spriteCssClass: "e-tree-foldericon", hasChild: _hasChild }];
            this._treetag._collapse = false;
            this._treetag.ejTreeView(
               {
                   loadOnDemand: true,
                   cssClass: proxy.model.cssClass,
                   enableRTL: proxy.model.enableRTL,
                   fields: { dataSource: proxy._localData, id: "id", parentId: "pid", text: "name", hasChild: "hasChild", spriteCssClass: "spriteCssClass" },
                   nodeCollapse: function (e) { proxy._refreshTreeScroller(e); },
                   nodeExpand: function (e) { proxy._refreshTreeScroller(e); },
                   nodeClick: function (e) { proxy._treenodeClicked(e); },
                   beforeExpand: function (e) { proxy._treeNodeBeforeExpand(e); },
                   nodeSelect: function (e) { proxy._updateTreePath(e); },
                   beforeCollapse: function (e) { proxy._onBeforeCollapse(e); },
                   beforeDelete: function (e) { proxy._treeBeforeDelete(e); },
               }
           );
            this._treeObj = this._treetag.data("ejTreeView");
            this._treeViewEvents("_off");
            this._treeViewEvents("_on");
            this._treetag.ejTreeView("selectNode", $(this._treetag).find("li")[0]);
            this._treeScroll = this._treetag.parent(".e-tree-wrapper").ejScroller({ height: this._splittag.height(), width: this._splittag.find(".e-cont1").width(), buttonSize: 0, scrollerSize: this._scrollerSize+10 }).data("ejScroller");
            this._addChild(result);
        },
        _treeViewEvents: function (action) {
            this[action](this._treetag, "focus", this._focusTreeView);
            this[action](this._treetag, "blur", this._blurTreeView);
        },
        _focusTreeView: function (e) {
            if (!this._treetag.hasClass("e-focus")) {
                this._treetag.addClass("e-focus");
                this._itemList = [];
                this._on(this._treetag, "keydown", this._OnKeyUp);
            }
        },
        _blurTreeView: function (e) {
            this._treetag.removeClass("e-focus");
            this._off(this._treetag, "keydown", this._OnKeyUp);
        },
        _treeBeforeDelete: function (args) {
            if (ej.isNullOrUndefined(args.event)) return;
            var code = this._getKeyCode(args.event);
            if (code == 46) // Prevent the treeview Delete action
                args.cancel = true;
        },
        _treeNodeBeforeExpand: function (args) {
            if (!this._nodeExpanded) {
                var path = this._updatePath(args.currentElement, args.value);
                !this._fileExplorer[path] && this._getFileDetails(path, args.currentElement);
                if (!this._treetag.ejTreeView("hasChildNode", args.currentElement)) {
                    this._fileExplorer[path] && this._addChild(this._fileExplorer[path], args.currentElement);
                }
            }
        },
        _treenodeClicked: function (args) {
            if ($(args.currentElement).hasClass("e-text") && args.currentElement != this._selectedNode.find('> div > .e-text')[0]) {
                this._selectedContent = args.currentElement.text;
                this._selectedNode = $(args.currentElement).closest('li.e-item');
                var node = $(this._selectedNode.parents('li.e-item')[0]);
                this._parentNode = node.length != 0 ? node : this._selectedNode;
                this._nodeType = "Directory";
                this._parentNode ? this._enableEditingTools() : this._disableEditingTools();
                this._isTreeNode = true;
            }
        },
        _showHideContextMenu: function () {
            if (this.model.showContextMenu) {
                var events = { beforeOpen: "", click: "" }, menuOptions, menuDetails = { id: "", targetId: "" };                
                menuOptions = this._initContextMenuOptions("navbar");
                menuDetails.id = this._ExplorerId + "_treeView";
                menuDetails.targetId = "#" + this._ExplorerId + "_treeView";
                events.beforeOpen = $.proxy(this._beforeOpenContextMenu, this);
                events.close = $.proxy(this._onHideContextMenu, this);
                events.click = $.proxy(this._contextMenuClick, this);
                this._treeContextMenutag = this._createContextMenuTag(menuOptions, menuDetails, events);
                this._treeMenuObj = this._treeContextMenutag.ejMenu('instance');

                this._cwdMenuOptions = this._initContextMenuOptions("cwd");
                menuDetails.id = this._ExplorerId + "_tileView";
                menuDetails.targetId = ".e-tile-wrapper,#" + this._ExplorerId + "_grid";
                events.beforeOpen = $.proxy(this._beforeOpenTileContextMenu, this);
                events.click = $.proxy(this._fileContextMenuClick, this);
                this._tileContextMenutag = this._createContextMenuTag(this._cwdMenuOptions, menuDetails, events);
                this._viewMenuObj = this._tileContextMenutag.ejMenu('instance');
                if (ej.isNullOrUndefined(this._fileName) || this._fileName == "") {
                    this._treeMenuObj.disableItem(this._menuPaste);                    
                }
                this._fileMenuOptions = this._initContextMenuOptions("files");                
            }
            else {
                if (this._treeContextMenutag && this._tileContextMenutag) {
                    this._treeMenuObj.destroy();
                    this._viewMenuObj.destroy();
                }
            }
        },
        enableMenuItem: function (operation) {            
            operation = this._findCommand(operation, this._contextMenu);
            for(var i=0; i < this._restrictedMenuOption.length; i++){
                if (this._restrictedMenuOption[i] == operation) {
                    this._restrictedMenuOption.splice(i, 1);
                    break;
                }                    
            }                        
            this._treeMenuObj.enableItem(operation);
            this._viewMenuObj.enableItem(operation);
        },
        disableMenuItem: function (operation) {
            operation = this._findCommand(operation, this._contextMenu);
            this._restrictedMenuOption.push(operation);
            this._treeMenuObj.disableItem(operation);
            this._viewMenuObj.disableItem(operation);
        },
        _renderGridView: function (result) {
            var proxy = this;
            var lastcolumn = this.model.gridSettings.columns[this.model.gridSettings.columns.length - 1];
            if (lastcolumn) {
                if (lastcolumn["customAttributes"]) {
                    if (lastcolumn["customAttributes"]["class"]) {
                        if (lastcolumn["customAttributes"]["class"].search("e-rowcell e-last-rowcell") == -1)
                            lastcolumn["customAttributes"]["class"] = lastcolumn["customAttributes"]["class"] + " e-rowcell e-last-rowcell";
                    }
                    else
                        lastcolumn["customAttributes"]["class"] = " e-rowcell e-last-rowcell";
                }
                else {
                    lastcolumn["customAttributes"] = { 'class': "e-rowcell e-last-rowcell" };
                }
            }
            if(this._gridObj) {
                this._gridObj.option('dataSource', result);
                this._gridObj.option('enableRTL', this.model.enableRTL);
            } else {
                this._gridtag.ejGrid({
                       cssClass: proxy.model.cssClass,
                       enableRTL: proxy.model.enableRTL,
                       dataSource: result,
                       selectionType: (proxy.model.allowMultiSelection ? "multiple" : "single"),
                       allowSorting: proxy.model.gridSettings.allowSorting,
                       columns: proxy.model.gridSettings.columns,
                       isResponsive: true,
                       scrollSettings: { width: 186, height: 200 },
                       allowScrolling: true,
                       enableResponsiveRow: false,
                       rowSelected: function (e) { proxy._updatePathFromGrid(e); },
                       recordDoubleClick: function (e) { proxy._openAction(e); }
               }
               );
            }
            this._gridObj = this._gridtag.ejGrid("instance");
            this.gridItems = this._gridObj.getRows();
            this._reSizeHandler();
            this._gridtag.attr("tabindex", -1);
            this._gridViewEvents("_off");
            this._gridViewEvents("_on");
        },
        _gridViewEvents: function (action) {
            this[action](this._gridtag.find(".e-gridcontent"), "focusin", this._focusGridView);
            this[action](this._gridtag.find(".e-gridcontent"), "focusout", this._blurGridView);
        },
        _focusGridView: function (e) {
            if (!this._gridtag.find(".e-gridcontent").hasClass("e-focus")) {
                this._gridtag.find(".e-gridcontent").addClass("e-focus");
                this._itemList = [];
                this._on(this._gridtag.find(".e-gridcontent"), "keydown", this._OnKeyUp);
            }
        },
        _blurGridView: function (e) {
            this._gridtag.find(".e-gridcontent").removeClass("e-focus");
            this._off(this._gridtag.find(".e-gridcontent"), "keydown", this._OnKeyUp);
        },
        _setThumbImageHeight: function () {
            var perRow = 1;
            if (this.items) {
                for (var i = 0; i < this.items.length - 1; i++) {
                    if (this.items[i].getBoundingClientRect().top == this.items[i + 1].getBoundingClientRect().top)
                        perRow++;
                    else
                        break;
                }
                this.items.find(".e-thumb-image.e-image").css('height', 'auto');
                if (perRow == null || perRow < 2) return true;
                for (var i = 0, j = this.items.length; i < j; i += perRow) {
                    var maxHeight = 0,
                    row = this.items.slice(i, i + perRow);
                    row.each(function () {
                        var itemHeight = parseInt($(this).find(".e-thumb-image").outerHeight());
                        if (itemHeight > maxHeight) maxHeight = itemHeight;
                    });
                    row.find(".e-thumb-image.e-image").css('height', maxHeight);
                }
                this._tileContent.parent(".e-tile-wrapper").ejScroller("refresh");
            }            
            this._perRow = perRow;
        },
        _renderTileView: function (result) {
            this._tileView.children() && this._tileView.children().remove() && this._tileView.removeClass("e-tileview");
            this._tileView.appendTo(this._tileContent);
            this._tileView.addClass("e-tileview").attr("role", "tileview");
            result.length == 0 ? this._tileView.html("No items to display") : this._tileView.html("");
            var proxy = this;
            var spantag = null;
            $.each(result, function (index, value) {
                var liTag = $("<li class='e-tilenode'></li>");
                if (value.isFile) {
                    if ((/\.(bmp|dib|jpg|jpeg|jpe|jfif|gif|tif|tiff|png|ico)$/i).test(value.name)) {
                        var path = proxy._currentPath.replace("~", "..") + value.name;
                        var imageWrapper = $("<div class='e-thumb-image e-image ' unselectable='on'></div>")
                        imageWrapper.appendTo(liTag);
                        var url = proxy._getImage(path, value.name);
                        var imagetag = ej.buildTag('img.e-thumbImage', "", "", { src: (url ? url : (path + "?" + (proxy._updateImages[path] ? proxy._updateImages[path] : proxy._initialTime))), "unselectable": "on" });
                        imagetag.appendTo(imageWrapper);
                    }
                    else {
                        spantag = $("<span class='e-thumb-image e-nonImage-icon' unselectable='on'></span>");
                        spantag.appendTo(liTag);
                    }
                }
                else {
                    spantag = $("<span class='e-thumb-image e-folder-icon' unselectable='on'></span>");
                    spantag.appendTo(liTag);
                }
                var divtag = $("<div class='e-name' unselectable='on'>" + value.name + "</div>");
                liTag.attr({
                    "aria-selected": false,
                    "title": (value.isFile) ? value.dateModified + " (" + value.size + ")" : value.dateModified,
                    "role": "tileitem"
                });
                divtag.appendTo(liTag);
                liTag.appendTo(proxy._tileView);
            });
            this._activeItem = 0;
            this.items = this._tileView.find("li.e-tilenode");
            this._tileViewEvents("_off");
            this._tileViewEvents("_on");
            this._tileContent.addClass("e-content");
            !this._tileContent.parent(".e-tile-wrapper").hasClass("e-scroller") && this._tileContent.parent(".e-tile-wrapper").ejScroller({ height: this.model.showFooter ? this._splittag.outerHeight() - this._statusbar.outerHeight() : this._splittag.outerHeight(), width: "100%", buttonSize: 0, scrollerSize: this._scrollerSize });
            if (!proxy._tileView.find(".e-image"))
                proxy._setThumbImageHeight();
            else {
                var images = proxy._tileView.find(".e-image > img");
                var increament = 0;
                for (var i = 0; i < images.length; i++) {
                    var img = new Image();
                    img.onload = img.onabort = img.onerror = function () {
                        ++increament == images.length && proxy._setThumbImageHeight();
                    };
                    img.src = $(images[i]).attr('src');
                }
            }
        },
        _getImage: function (path, name) {
            if (this._currentPath.indexOf(":") == 1 || this.model.ajaxSettings.getImage.url) {
                var selectedItems = this._getSelectedItemDetails(this._currentPath, name);
                var url = this.model.ajaxAction + "?Path=" + path + "&ActionType=GetImage&SelectedItems=" + JSON.stringify(selectedItems);
                if (this.model.ajaxSettings.getImage.url) {
                    if (this.model.ajaxSettings.getImage.url.indexOf("{") >= 0)
                        url = String.format(this.model.ajaxSettings.getImage.url, "?Path=" + path + "&SelectedItems=" + JSON.stringify(selectedItems));
                    else
                        url = this.model.ajaxSettings.getImage.url;
                }
            }
            return url ? url : "";
        },
        _gridtagClick: function (event) {
            event.stopPropagation();
            if ($(event.target).hasClass("e-gridcontent") || $(event.target).hasClass("e-content") || $(event.target).hasClass("e-table")) {
                this._addFocus(this._gridtag.find(".e-gridcontent"));
                this._gridObj.clearSelection();
                this._removeOldSelectionDetails();
            }
        },

        _updateGridSelection: function (args) {
            if (args.events && !args.events.ctrlKey && $(args.target)[0] != null) {
                this._gridObj.selectRows($(args.target).parent().index(), null, $(args.target));
            }
        },

        _updateTileSelection: function (args) {
            if (args.events && !args.events.ctrlKey && $(args.target)[0] != null) {
                $(args.target).click();
            }
        },

        _tileViewEvents: function (action) {
            this[action](this.items, "click", this._upDatePathFromTileView);
            this[action](this.items, "dblclick", this._openAction);
            this[action](this.items, "mouseenter", this._onItemHover);
            this[action](this.items, "mouseleave", this._onItemLeave);
            this[action](this._tileViewWrapper, "focusin", this._focusTileView);
            this[action](this._tileViewWrapper, "focusout", this._blurTileView);
        },
        _tileViewWrapperClick: function (event) {
            if ($(event.target).hasClass("e-tile-wrapper") || $(event.target).hasClass("e-tile-content") || $(event.target).hasClass("e-tileview")) {
                this._lastItemIndex = this._lastItemIndex ? this._lastItemIndex : this._itemList.filter(".e-active").index();
                this._addFocus(this._tileViewWrapper);
                if (this.items.hasClass("e-active"))
                    this.items.removeClass("e-active").attr("aria-selected", false);
                this._removeOldSelectionDetails();
            }
        },
        _onItemHover: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (!$(currentItem).hasClass("e-disable")) {
                this.items.removeClass("e-hover");
                $(currentItem).addClass("e-hover");
            }
        },
        _onItemLeave: function (e) {
            var currentItem = e.currentTarget, targetItem = e.target;
            if (!$(currentItem).hasClass("e-disable")) {
                $(currentItem).removeClass("e-hover");
            }
        },
        _focusTileView: function (e) {
            if (!this._tileViewWrapper.hasClass("e-focus")) {
                this._tileViewWrapper.addClass("e-focus");
                this._itemList = this.items;
                this._on(this._tileViewWrapper, "keydown", this._OnKeyUp);
                this._on(this._tileViewWrapper, "keydown", this._OnKeyDown);
            }
        },
        _blurTileView: function (e) {
            this._tileViewWrapper.removeClass("e-focus");
            this._off(this._tileViewWrapper, "keydown", this._OnKeyUp);
            this._off(this._tileViewWrapper, "keydown", this._OnKeyDown);
        },
        _OnKeyDown: function (e) {
            var itemsLength = this._itemList.length - 1, activeItem;
            this._activeItem = this._lastItemIndex ? this._lastItemIndex : this._itemList.filter(".e-active").index();
            var code = this._getKeyCode(e);
            switch (code) {
                case 38:
                    e.preventDefault();
                    if (this._activeItem < this._perRow-1)
                        return;
                    this._activeItem -= this._perRow;
                    this._beforeListHover(e.shiftKey);
                    break;
                case 37:
                    e.preventDefault();
                    if (this._activeItem == 0)
                        return;
                    if ((this._activeItem < 0) || (this._activeItem == null) || (this._activeItem > itemsLength))
                        this._activeItem = 0;
                    else if (this._activeItem == 0)
                        this._activeItem = itemsLength;
                    else
                        this._activeItem -= 1;
                    this._beforeListHover(e.shiftKey);                    
                    break;
                case 40:
                    e.preventDefault();
                    if (this._activeItem + this._perRow >= this._itemList.length)
                        return;
                    this._activeItem += this._perRow;
                    this._beforeListHover(e.shiftKey);
                    break;
                case 39:
                    e.preventDefault();
                    if (this._activeItem == itemsLength)
                        return;                    
                    if ((this._activeItem > itemsLength) || (this._activeItem == null) || (this._activeItem < 0))
                        this._activeItem = itemsLength;
                    else if (this._activeItem == itemsLength)
                        this._activeItem = 0;
                    else
                        this._activeItem += 1;
                    this._beforeListHover(e.shiftKey);
                    break;
                case 33:
                case 36:
                    e.preventDefault();
                    this._activeItem = 0;
                    this._beforeListHover(e.shiftKey);
                    break;
                case 34:
                case 35:
                    e.preventDefault();
                    this._activeItem = itemsLength;
                    this._beforeListHover(e.shiftKey);
                    break;
            }
            this._lastItemIndex= e.shiftKey ? this._activeItem : null;
        },
        _OnKeyUp: function (e) {
            var activeItem;
            var code = this._getKeyCode(e);
            switch (code) {
                case 13:
                    e.preventDefault();
                    if (e.altKey) {
                        this._lastFocusedElement = e.currentTarget;
                        this._getDetails();
                    }
                    else {
                        e.preventDefault();
                        this._lastFocusedElement = e.currentTarget;
                        if (!$(e.currentTarget).hasClass("e-treeview")) {
                            var isPresent = false;
                            if ($(e.currentTarget).hasClass("e-tile-wrapper"))
                                isPresent = this.items.hasClass("e-active");
                            else if ($(e.currentTarget).hasClass("e-gridcontent"))
                                isPresent = $(this.gridItems).find("td").hasClass("e-active");
                            if (isPresent && this._selectedContent)
                                this._openAction();
                        }
                    }
                    break;
                case 86:
                    if (e.ctrlKey) {
                        e.preventDefault();
                        if (!ej.isNullOrUndefined(this._option) && this._selectedContent) {
                            if (this._currentPath != this._originalPath) {
                                this._currentPath = this._originalPath;
                            }
                            this._lastFocusedElement = e.currentTarget;
                            this._cut_copy();
                        }
                    }
                    break;
                case 46:
                case 67:
                case 68:
                case 88:
                case 113:
                    e.preventDefault();
                    this._lastFocusedElement = e.currentTarget;
                    var isPresent = false;
                    if ($(e.currentTarget).hasClass("e-treeview")) {
                        if (this._rootPath != this._currentPath)
                            isPresent = (this._treetag.find(".e-active").length > 0) ? true : false;
                    }
                    else {
                        if ($(e.currentTarget).hasClass("e-tile-wrapper"))
                            isPresent = this.items.hasClass("e-active");
                        else if ($(e.currentTarget).hasClass("e-gridcontent"))
                            isPresent = $(this.gridItems).find("td").hasClass("e-active");
                    }
                    if (isPresent && this._selectedContent) {
                        switch (code) {
                            case 46:
                                this._deleteFolder();
                                break;
                            case 67:
                                if (e.ctrlKey)
                                    this._copyMoveNode("copy");
                                break;
                            case 68:
                                if (e.ctrlKey)
                                    this._deleteFolder();
                                break;
                            case 88:
                                if (e.ctrlKey)
                                    this._copyMoveNode("move");
                                break;
                            case 113:
                                this._renameFolder();
                                break;
                        }
                    }
                    break;
                case 93:
                case 121:
                    if ((e.shiftKey || code == 93) && this.model.showContextMenu) {
                        e.preventDefault();
                        this._lastFocusedElement = e.currentTarget;
                        var pos = null, _target, _element;
                        if ($(e.currentTarget).hasClass("e-treeview")) {
                            _element = this._treeObj.getSelectedNode().find(".e-active");
                            pos = this._getMenuPosition(_element);
                            this._treeMenuObj.show(pos.left, pos.top, _element, e, true);
                        }
                        else {
                            if ($(e.currentTarget).hasClass("e-tile-wrapper")) {
                                for (var i = 0; i < this.items.length; i++) {
                                    _element = $(this.items[i]);
                                    if (_element.text() == this._selectedContent) {
                                        pos = this._getMenuPosition(_element);
                                        _target = _element;
                                        break;
                                    }
                                }
                            } else if ($(e.currentTarget).hasClass("e-gridcontent")) {
                                for (var i = 0; i < this.gridItems.length; i++) {
                                    _element = $(this.gridItems[i]).find("td:first");
                                    if (_element.text() == this._selectedContent) {
                                        pos = this._getMenuPosition(_element);
                                        _target = _element;
                                        break;
                                    }
                                }
                            }
                            if (pos == null) {
                                pos = $(e.currentTarget).offset();
                                _target = $(e.currentTarget);
                            }
                            this._viewMenuObj.show(pos.left, pos.top, _target, e, true);
                        }
                    }
                    break;
            }
        },
        _getMenuPosition: function (element) {
            var height = element.outerHeight() / 2;
            var width = element.outerWidth() / 2;
            var pos = element.offset();
            pos = { top: pos.top + height, left: pos.left + width };
            return pos;
        },

        _beforeListHover: function (enabledShift) {            
            activeItem = this._getActiveItem();
            if (!activeItem.hasClass("e-disable")) {
                var e = jQuery.Event("click", { keyCode: 91, shiftKey: enabledShift, innerEvent: true });
                activeItem.trigger(e);
            }            
        },
        _getActiveItem: function () {
            return $($(this._itemList)[this._activeItem]);
        },
        _getURL: function () {
            var url = this.model.ajaxAction + "?Path=" + this._currentPath + "&ActionType=Upload" + (this._selectedContent ? "&SelectedItems=" + JSON.stringify(this._getSelectedItemDetails(this._getFolderPath(), [this._selectedContent])) : "");
            if (this.model.ajaxSettings.upload.url) {
                if (this.model.ajaxSettings.upload.url.indexOf("{") >= 0)
                    url = String.format(this.model.ajaxSettings.upload.url, "?Path=" + this._currentPath + (this._selectedContent ? "&SelectedItems=" + JSON.stringify(this._getSelectedItemDetails(this._getFolderPath(), [this._selectedContent])) : ""));
                else
                    url = this.model.ajaxSettings.upload.url;
            }
            return url;
        },

        _renderUploadBox: function () {
            var proxy = this;
            var url = this._getURL();
            this._uploadtag.ejUploadbox({
                cssClass: this.model.cssClass,
                enableRTL: this.model.enableRTL,
                height: "0px",
                width: "0px",
                uploadName: "FileUpload",
                /* For Automatic Upload */
                autoUpload: this.model.uploadSettings.autoUpload,
                showFileDetails: !this.model.uploadSettings.autoUpload,

                dialogText: { title: proxy._getLocalizedLabels("Upload") },
                showRoundedCorner: false,
                extensionsAllow: this.model.fileTypes == "*.*" ? "" : this.model.fileTypes.replace(/\*/g, ""),
                multipleFilesSelection: this.model.uploadSettings.allowMultipleFile,
                fileSize: this.model.uploadSettings.maxFileSize,
                buttonText: { browse: "Upload file" },
                saveUrl: url,
                removeUrl: this.model.ajaxAction + "?Path=" + this._currentPath + "&ActionType=Remove",
                locale: ej.Uploadbox.Locale[this.model.locale] ? this.model.locale : "en-US",
                complete: function (e) { proxy._uploadSuccess(e); },
                remove: function (e) { proxy._uploadSuccess(e); },
                fileSelect: function (e) {
                    !proxy._fileExplorer[proxy._currentPath] && proxy._getFileDetails(proxy._currentPath);
                    var targetFiles = proxy._fileExplorer[proxy._currentPath];
                    proxy._existingItems = [];
                    var files = proxy._files = e.files;
                    for (var j = 0; j < files.length; j++) {
                        for (var i = 0; i < targetFiles.length; i++) {
                            if (files[j].name == targetFiles[i].name) {
                                proxy._existingItems.push({ Name: files[j].name, Path: proxy._currentPath + files[j].name, IsReplace: true });
                                break;
                            }
                        }
                    }
                    if (proxy._existingItems.length) {
                        proxy._createReplaceConformationDiaolg("_customUpload", "ReplaceAlert");
                        e.cancel = true;
                    }

                },
                error: function (e) {
                    if (!proxy._alertDialog || !proxy._alertDialog.is(":visible")) {
                        proxy._alertDialog = proxy._createDialog(ej.buildTag('div.e-fe-dialog-label', (e.error ? e.error : e.responseText)), { width: 400, height: "auto", title: proxy._getLocalizedLabels("Error") });
                        proxy._alertDialogObj = proxy._alertDialog.data("ejDialog");
                        proxy._alertDialogObj.initialRender = false;
                        proxy._alertDialogObj.open();
                    }
                },
                begin: function (e) {
                    var nodes = proxy._currentPath.split('/');
                    var args = { path: proxy._currentPath, selectedItems: proxy._getSelectedItemDetails(proxy._getFolderPath(), nodes[nodes.length - 2]) };
                    if (proxy._trigger("beforeUpload", args))
                        e.cancel = true;
                    proxy._uploadtag.ejUploadbox({ saveUrl: proxy._getURL() });
                }
            });
        },
        _customUpload: function () {
            for (var i = 0; i < this._existingItems.length; i++) {
                if (!this._existingItems[i].IsReplace) {
                    for (var k = 0; k < this._files.length; k++) {
                        if (this._files[k].name == this._existingItems[i].Name) {
                            this._files.splice(k, 1);
                            break;
                        }
                    }
                }
            }
            this._uploadtag.ejUploadbox({ pushFile: this._files });
        },
        _uploadSuccess: function (args) {
            this._uploadtag.find(".e-dialog.e-js .e-action-perform").remove();
            this._fileExplorer[this._currentPath] = "";
            this._treeObj.selectNode(this._selectedNode);

        },
        _createAddressBar: function () {
            this._addresstag = ej.buildTag('input.e-addressBar e-tool-input', "", {}, { id: this._ExplorerId + '_addressbar', type: "text" });
            this._addresstag.appendTo(this._toolBarItems.find("#" + this._ExplorerId + "Addressbar").html(""));
            var spanTag = $("<span class='e-addressbar-foldericon'></span>");
            spanTag.insertBefore(this._addresstag);
            this._on($('#' + this._ExplorerId + '_addressbar'), "keydown", this._searchPath);
            this._on($('#' + this._ExplorerId + '_addressbar'), "blur", this._addressbarFocusout);
        },
        _updateAddressBar: function () {
            if (this._addresstag) {
                var temp = this._currentPath;
                this._addresstag.val(temp.replace(this._initPath, ""));
            }
        },
        _onEpand: function (args) {
            proxy = $("#" + this.element.attr("id")).parents(".e-fileexplorer").ejFileExplorer("instance");
            proxy._reSizeHandler(args);
        },
        _createSplitPane: function () {
            var proxy = this;
            var pane1 = ej.buildTag('div');
            var pane2 = ej.buildTag('div');
            this._splittag = ej.buildTag('div#' + this._ExplorerId + '_splitter');
            pane1.addClass('e-cont1');
            pane2.addClass('e-cont2');
            this._splittag.append(pane1, pane2);
            this.element.append(this._splittag);
            this._splitObj = this._splittag.ejSplitter({
                enableAutoResize: true,
                width: this.element.width(),
                cssClass: this.model.cssClass,
                enableRTL: this.model.enableRTL,
                height: Math.ceil(this.element.height() - (this._toolBarItems ? this._toolBarItems.outerHeight() : 0)),
                properties: [{ paneSize: "25%" }, {}],
                resize: function (e) { proxy._reSizeHandler(e); }
            }).data('ejSplitter');
            this._splitObj._bar = 1;
            this._splitObj.refresh();
            this._splittag.ejSplitter("option", "expandCollapse", this._onEpand);
            var treeWrapper = ej.buildTag('div.e-tree-wrapper');
            pane1.append(treeWrapper);
            this._treetag = ej.buildTag('div#' + this._ExplorerId + '_treeView');
            treeWrapper.append(this._treetag);
            this._gridtag = ej.buildTag('div#' + this._ExplorerId + '_grid');
            pane2.append(this._gridtag);
            this._tileViewWrapper = ej.buildTag('div.e-tile-wrapper', "", "", { tabindex: 0 });
            pane2.append(this._tileViewWrapper);
            this._tileViewWrapper.append("<div class='e-tile-content' unselectable='on'> </div>");
            this._tileContent = this._tileViewWrapper.find(".e-tile-content");
            this._tileView = ej.buildTag('ul#' + this._ExplorerId + '_tileView', "", "", { unselectable: "on" });
            this._tileContent.append(this._tileView);

            this._statusbar = ej.buildTag('div', "", "", { tabindex: 0 });
            this._statusbar.insertAfter(this._tileView);
            pane2.append(this._statusbar);
            this.model.showFooter && this._createStatusBar();
            this._waitingPopup = this._splittag.find(".e-cont2").ejWaitingPopup({ showOnInit: false, cssClass: this.model.cssClass }).data("ejWaitingPopup");
            this._showHideSplitBar(true);
            this._showHideContextMenu();
            if (this.model.layout == "grid") {
                this._tileContent.parent().hide();
            }
            else {
                this._gridtag.hide();
            }
            this._createUploadBox();

        },
        _selectedFolder: function (targetNavigationPath) {
            var navigationPath = targetNavigationPath.replace(this._initPath, "");
            var selectedNode = this._treeObj.getSelectedNode();          
            var folders = navigationPath.split("/");
            navigationPath && this._treeObj.selectNode(this._treeObj.element.find("li:first"));
            for (var j = 0; j < folders.length; j++) {
                if (folders[j] != "") {
                    selectedNode = this._treeObj.getSelectedNode();
                    if (!this._treeObj.isExpanded(selectedNode))
                        selectedNode.find(".e-icon").click();
                    var childItems = selectedNode.find('ul:first>li').find('div:first .e-text');
                    for (var i = 0; i < childItems.length; i++) {
                        if (childItems[i].text == folders[j]) {
                            this._treeObj.selectNode(childItems[i].parentNode.parentNode);
                        }
                    }
                }
            }
        },
        _setSelectedItems: function (selectedItems) {
            if (typeof selectedItems == "string")
                selectedItems = [selectedItems];
            this._removeOldSelectionDetails();
            if (this.model.layout == "grid") {
                this._gridObj.clearSelection();
                var _ctrlKey = this._gridObj.multiSelectCtrlRequest;
                this._gridObj.multiSelectCtrlRequest = true;
                var nodes = this._gridtag.find(".e-gridcontent tr");
                for (var j = 0; j < selectedItems.length; j++) {
                    for (var i = 0; i < nodes.length; i++) {
                        if (selectedItems[j] == $(nodes[i]).find("td:first").text()) {
                            this._gridObj.selectRows(i, null, $(nodes[i]).find("td:first"));
                        }
                    }
                }
                this._gridObj.multiSelectCtrlRequest = _ctrlKey;
            }
            else {
                this.items.removeClass("e-active").attr("aria-selected", false);
                var nodes = this._tileView.find(".e-tilenode");
                for (var j = 0; j < selectedItems.length; j++) {
                    for (var i = 0; i < nodes.length; i++) {
                        if (selectedItems[j] == $(nodes[i]).text()) {
                            var e = jQuery.Event("click", { keyCode: 91, ctrlKey: true, innerEvent: true });
                            $(nodes[i]).trigger(e);
                        }
                    }
                }
            }
        },
        _refreshTreeScroller: function (args) {
            !this._treetag.hasClass("e-content") || this._treetag.addClass("e-content");
            this._treeScroll && this._treeScroll.refresh();
        },
        _createStatusBar: function () {
            this._statusbar.addClass("e-statusbar");
            this._itemStatus = ej.buildTag('div.e-itemStaus');
            this._selectedItemsTag = ej.buildTag('div.e-itemStaus e-selected-items');
            this._switchBtn = ej.buildTag('div.e-switchView');
            this._statusbar.append(this._switchBtn, this._itemStatus, this._selectedItemsTag);
            if (this.model.enableResize) {
                this._resizeItem = ej.buildTag("div.e-icon e-fe-resize e-resize-handle");
                this._resizeItem.insertBefore(this._switchBtn);
            }
            var setHeight = this._splittag.find(".e-cont2").outerHeight() - this._statusbar.outerHeight();
            this._gridtag.height(setHeight);
            var button = ej.buildTag('button.e-switchGridView#' + this._ExplorerId + '_switchGridView', "", {}, { title: this._getLocalizedLabels("Grid"), tabindex: 0 });
            this._switchBtn.append(button);
            button.ejButton({ type: "button", size: "normal", contentType: "imageonly", prefixIcon: "e-switchGridView-icon" });
            button = ej.buildTag('button.e-swithListView#' + this._ExplorerId + '_swithListView', "", {}, { title: this._getLocalizedLabels("Tile"), tabindex: 0 });
            this._switchBtn.append(button);
            button.ejButton({ type: "button", size: "normal", contentType: "imageonly", prefixIcon: "e-swithListView-icon" });
            this.model.layout == "grid" ? this._statusbar.find(".e-switchGridView").addClass("e-active") : this._statusbar.find(".e-swithListView").addClass("e-active");
            this._statusBarEvents("_off");
            this._statusBarEvents("_on");
        },
        _statusBarEvents: function (action) {
            this[action](this._statusbar, "focus", this._focusStatusBar);
            this[action](this._statusbar, "blur", this._blurStatusBar);
        },
        _focusStatusBar: function (e) {
            if (!this._statusbar.hasClass("e-focus")) {
                this._statusbar.addClass("e-focus");
                this._itemList = this._switchBtn.find("button");
                this._on(this._statusbar, "keydown", this._OnKeyDown);
            }
        },
        _blurStatusBar: function (e) {
            this._statusbar.removeClass("e-focus");
            this._off(this._statusbar, "keydown", this._OnKeyDown);
        },
        _refreshResizeHandler: function () {
            this._setMinMaxSizeInInteger();
            this.adjustSize();
            if (this.model.showFooter && this.model.enableResize)
                this._resizeFileExplorer();
        },
        _refreshResizeEventHandler: function (event) {
            var reElement = $(event.element).parents("div.e-fileexplorer");
            this.model.height = $(reElement).outerHeight();
            this.model.width = $(reElement).outerWidth();
            this.element.css({ "height": this.model.height, "width": this.model.width });
            this.adjustSize();
        },
        _convertPercentageToPixel: function (parent, child) {
            return Math.round((child * parent) / 100);
        },
        _getProperValue: function (value) {
            if (value == null) return null;
            else return !isNaN(value) ? value : value;
        },
        _setMinMaxSizeInInteger: function () {
            var parentObj;
            this._minWidth = parseInt(this.model.minWidth);
            this._minHeight = parseInt(this.model.minHeight);
            this._maxWidth = parseInt(this.model.maxWidth);
            this._maxHeight = parseInt(this.model.maxHeight);
            parentObj = this.element.parent()[0].nodeName == "BODY" ? $(window) : $(this.element.parent()[0]);
            if (isNaN(this.model.minWidth) && (this.model.minWidth.indexOf("%") > 0))
                this._minWidth = this._convertPercentageToPixel(parentObj.outerWidth(), this._minWidth);
            if (isNaN(this.model.minHeight) && (this.model.minHeight.indexOf("%") > 0))
                this._minHeight = this._convertPercentageToPixel(parentObj.outerHeight(), this._minHeight);
            if (isNaN(this.model.maxWidth) && (this.model.maxWidth.indexOf("%") > 0))
                this._maxWidth = this._convertPercentageToPixel(parentObj.innerWidth(), this._maxWidth);
            if (isNaN(this.model.maxHeight) && (this.model.maxHeight.indexOf("%") > 0))
                this._maxHeight = this._convertPercentageToPixel(parentObj.innerHeight(), this._maxHeight);
        },
        _resizeFileExplorer: function () {
            var proxy = this;
            this.element.find("div.e-fe-resize").ejResizable(
                {
                    minHeight: proxy._minHeight,
                    minWidth: proxy._minWidth,
                    maxHeight: proxy._maxHeight,
                    maxWidth: proxy._maxWidth,
                    resizeStart: function (event) {
                        proxy._trigger("resizeStart", { event: event });
                    },
                    resize: function (event) {
                        proxy._refreshResizeEventHandler(event);
                        proxy._trigger("resize", { event: event });                       
                    },
                    resizeStop: function (event) {
                        proxy._refreshResizeEventHandler(event);
                        proxy._trigger("resizeStop", { event: event });
                    },
                    helper: function (event) {
                        return $(proxy.element);
                    }
                });
        },
        _showHideSplitBar: function (option) {
            this._splittag.show();
            if (this._splitObj.model.properties[1].paneSize == 0) {
                this.element.find(".e-splitbar").show();
                this._splitObj.expand(1);
                if (!this._gridtag.hasClass("e-grid") && option) {
                    this._updateData();
                }
            }
            else if (this._splitObj.model.properties[0].paneSize == 0) {
                this.element.find(".e-splitbar").show();
                var olderWidth = proxy._splittag.outerWidth();
                //here 1 represents border width
                proxy.element.find(".e-cont2").width(olderWidth - this.element.find(".e-splitbar").outerWidth() - 1);
                this._splitObj.expand(0);
            }
            if (!this.model.showNavigationPane) {
                this._splitObj.collapse(0);
                this.element.find(".e-splitbar").hide();
                if (!this._gridtag.hasClass("e-grid") && option) {
                    this._updateData();
                }
            }
        },
        _updateTreePath: function (args) {
            this._removeBlurEffect();
            this._searchbar && this._searchbar.val("");
            this._selectedTreeText = this._selectedContent = args.value;
            this._selectedNode = args.currentElement;
            var node = $(this._selectedNode.parents('li.e-item')[0]);
            this._parentNode = node.length != 0 ? node : this._selectedNode;
            this._nodeType = "Directory";
            if (this._initUpdate) {
                this._currentPath = this._updatePath(args.currentElement, args.value);
            } else
                this._initUpdate = true;            
            this._updateOnNodeSelection && this._updateData();
            this._originalPath = this._currentPath;
            if (!this._isStateNavigation) {
                if (!ej.isNullOrUndefined(this._currentState)) {
                    for (var i = this._selectedStates.length - 1; i > this._currentState; i--)
                        this._selectedStates.pop();
                    this._toolBarItems && this._disableToolbarItem("Forward");
                }
                if (this._selectedStates[this._selectedStates.length - 1] != this._originalPath) {
                    this._currentState = this._selectedStates.length;
                    this._selectedStates.push(this._originalPath);
                    this._selectedStates.length == 2 && this._toolBarItems && this._enableToolbarItem("Back");
                }
            }
           
            this.model.selectedFolder = this._currentPath;
            this._updateAddressBar();
            this._currntNode = this._selectedNode.find("> div > .e-text");
            this.model.selectedItems = [];
            this._selectedItems = [];
            this._selectedTileItems = [];
            var args = { name: "", path: this._currentPath, selectedItems: this._getSelectedItemDetails(this._getFolderPath(), this._selectedContent) };
            this._trigger("select", args);
            this._toolBarItems && this._disableToolbarItem("Download");
        },
        _modifySelectedStates: function (startsWith, replace) {
            var proxy = this;            
            $.each(proxy._selectedStates, function (index, path) {
                if (path && path.startsWith(startsWith))
                    proxy._selectedStates[index]= (replace ? proxy._selectedStates[index].replace(startsWith, replace): replace);                                                            
            });            
        },
        _onBeforeCollapse: function (args) {           
           this._collapse = true;
        },
        _updatePath: function (node, val) {
            var parentPath = "";
            var unnecesaryULtags = 1;
            for (var i = 0; i < node.parents("ul").length - unnecesaryULtags; i++) {
                if ($(node.parents("ul")[i]).siblings("div").find("a").text())
                    parentPath = $(node.parents("ul")[i]).siblings("div").find("a").text() + "/" + parentPath;
            }
            return this._initPath + parentPath + val + "/";
        },
        _updatePathFromGrid: function (args) {
            var proxy = this, target = $(args.target);
            this._addFocus(this._gridtag.find(".e-gridcontent"));
            var isFolder, _isUpdate = true;
            this._isTreeNode = false;
            var _childItems;
            this._selectedItems = [];
            var sizeInByte = 0;
            var selectedRecords = this._gridtag.ejGrid("getSelectedRecords");
            for (var record = 0; record < selectedRecords.length; record++) {
                this._selectedItems.push(selectedRecords[record].name)
                sizeInByte += selectedRecords[record].sizeInByte;
                if (!selectedRecords[record].isFile) _isUpdate = false;
            }
            if (this._currentPath != this._originalPath) {
                this._currentPath = this._originalPath;
            }
            $.each(proxy._fileExplorer[proxy._originalPath], function (index, value) {
                if (value.name == args.data.name && !value.isFile)
                    isFolder = true;
            });
            if (isFolder) {
                this._nodeType = "Directory";
                if (target.hasClass('e-active'))
                    this._updateNode(args.data.name);
            }
            else {
                this._nodeType = "File";
            }
            this._currntNode = target[0] != null && target.parent("tr");
            this._selectedContent = args.data.name;
            if (!target.hasClass('e-active') && selectedRecords.length > 0) {
                this._selectedContent = selectedRecords[selectedRecords.length - 1].name;
                this._nodeType = selectedRecords[selectedRecords.length - 1].isFile ? "File" : "Directory";
                if (this._nodeType == "Directory")
                    this._updateNode(this._selectedContent);
            }
            this._selectedItemsTag && this._selectedItemsTag.html((this._selectedItems.length > 1 ? (this._selectedItems.length + " items Selected") : (selectedRecords[0] ? selectedRecords[0].name : "")) + (sizeInByte ? (_isUpdate ? ",  " + this._bytesToSize(sizeInByte) : "") : ""));
            this.model.selectedItems = this._selectedItems;
            var args = { name: this._selectedItems, path: this.model.selectedFolder, selectedItems: this._getSelectedItemDetails(this.model.selectedFolder, this._selectedItems) };
            this._urlTag && this._urlTag.find("input").val(args.path);
            this._nameTag && this._nameTag.find("input").val(args.name);
            if (this._selectedItems.length == 0)
                this._removeOldSelectionDetails();
            else {
                if (this._toolBarItems)
                    _isUpdate ? this._enableToolbarItem("Download") : this._disableToolbarItem("Download");
                if (this._viewMenuObj)
                    _isUpdate ? (this._restrictedMenuOption.indexOf(this._menuDownload) < 0 && this._viewMenuObj.enableItem(this._menuDownload)) : this._viewMenuObj.disableItem(this._menuDownload);
                this._enableEditingTools();
            }
            if (this._isClicked)
                this._trigger("select", args);
            this._isupdate = _isUpdate;
        },
        _enableEditingTools: function () {
            if (!this._editingToolsState && this._toolBarItems) {
                var items = ["Rename", "Delete", "Cut", "Copy"];
                for (var i = 0; i < items.length; i++) {
                    this._enableToolbarItem(items[i]);
                }
                this._editingToolsState = true;
            }
        },
        _enableToolbarItem: function (suffixID) {
            if (this._restrictedToolbarOptions.indexOf(suffixID) < 0)
            {
                var id = this._ExplorerId + suffixID.replace(/ /g, '');
                if (this._toolBarObj && this._toolbar.find("li#" + id).hasClass("e-disable"))
                    this._toolBarObj.enableItemByID(id);
            }            
        },
        _disableToolbarItem: function (suffixID) {
            var id = this._ExplorerId + suffixID.replace(/ /g, '');
            if (this._toolBarObj && !this._toolbar.find("li#" + id).hasClass("e-disable"))
                this._toolBarObj.disableItemByID(id);
        },

        _disableEditingTools: function () {
            if (this._toolBarItems) {
                var items = ["Rename", "Delete", "Cut", "Copy"];
                for (var i = 0; i < items.length; i++) {
                    this._disableToolbarItem(items[i]);
                }
                this._editingToolsState = false;
            }
        },
        _upDatePathFromTileView: function (event) {
            if (!event.innerEvent)
                this._lastItemIndex = $(event.currentTarget).index();
            this._addFocus(this._tileViewWrapper);
            this._isTreeNode = false;
            if (!event.shiftKey) {
                if (!event.ctrlKey || !this.model.allowMultiSelection) {
                    this._sizeInByte = 0;
                    this._selectedItems = [];
                    this._selectedTileItems = [];
                    $(event.currentTarget).siblings().removeClass("e-active").attr("aria-selected", false);
                    $(event.currentTarget).removeClass("e-active").attr("aria-selected", false);
                }
                if ($(event.currentTarget).hasClass("e-active"))
                    $(event.currentTarget).removeClass("e-active").attr("aria-selected", false);
                else
                    $(event.currentTarget).addClass("e-active").attr("aria-selected", true);
                var nodeType = "File", _childItems, _isUpdate = true;
                var name = $(event.currentTarget).find(".e-name").text();
                if (this._currentPath != this._originalPath) {
                    this._currentPath = this._originalPath;
                }
                var items = this._fileExplorer[this._currentPath];
                if (items) {
                    for (var i = 0; i < items.length; i++) {
                        if (items[i].name == name) {
                            if ($(event.currentTarget).hasClass("e-active")) {
                                this._sizeInByte += items[i].sizeInByte;
                                this._selectedTileItems.push(items[i]);
                            } else if (event.ctrlKey) {
                                this._sizeInByte -= items[i].sizeInByte;
                                var index = $.inArray(items[i], this._selectedTileItems);
                                if (index > -1)
                                    this._selectedTileItems.splice(index, 1);
                            }
                            break;
                        }
                    }
                }
                $.each(this._selectedTileItems, function (index, value) {
                    if (!value.isFile) {
                        this._update = _isUpdate = false;                        
                        return false;
                    }
                });
                if ($(event.currentTarget).find(".e-folder-icon").length) {
                    nodeType = "Directory";
                    if ($(event.currentTarget).hasClass("e-active"))
                        this._updateNode(name);
                }
                this._currntNode = $(event.currentTarget);
                this._selectedContent = name;
                if ($(event.currentTarget).hasClass("e-active"))
                    this._selectedItems.push(this._selectedContent);
                else {
                    var index = $.inArray(this._selectedContent, this._selectedItems);
                    if (index > -1)
                        this._selectedItems.splice(index, 1);
                    if (this._selectedTileItems.length > 0) {
                        this._selectedContent = this._selectedTileItems[this._selectedTileItems.length - 1].name;
                        nodeType = this._selectedTileItems[this._selectedTileItems.length - 1].type;
                        if (nodeType == "Directory")
                            this._updateNode(this._selectedContent);
                    }
                }                
                if ((!this._startNode) || this._selectItems.length == 0 || this._selectItems.length == this._selectedItems.length) {
                    if (!this._selectItems || this._selectItems.length <= 0)
                        this._startNode = null;
                    this._selectedItemsTag && this._selectedItemsTag.html(this._selectedItems.length > 0 ? ((this._selectedItems.length > 1 ? this._selectedItems.length + " items Selected " : this._tileView.find(".e-active").text()) + (this._sizeInByte ? ",  " + this._bytesToSize(this._sizeInByte) : "")) : "");
                    this._nodeType = nodeType;
                    this.model.selectedItems = this._selectedItems;
                    var args = { name: this._selectedItems, path: this.model.selectedFolder, selectedItems: this._getSelectedItemDetails(this.model.selectedFolder, this._selectedItems) };
                    this._urlTag && this._urlTag.find("input").val(args.url);
                    this._nameTag && this._nameTag.find("input").val(args.name);
                    if (this._selectedItems.length == 0)
                        this._removeOldSelectionDetails();
                    else {
                        if (this._toolBarItems)
                            _isUpdate ? this._enableToolbarItem("Download") : this._disableToolbarItem("Download");
                        if (this._viewMenuObj)
                            _isUpdate ? (this._restrictedMenuOption.indexOf(this._menuDownload) < 0 && this._viewMenuObj.enableItem(this._menuDownload)) : this._viewMenuObj.disableItem(this._menuDownload);
                        this._enableEditingTools();
                    }
                    if (this._isClicked)                        
                        this._trigger("select", args);
                    this._selectItems = [];
                }                
            } else {
                this._selectItems = [];
                if (!this._startNode)
                    this._startNode = this._currntNode;
                var startIndex = $(event.currentTarget).index();
                var endIndex = this._startNode.index();
                if (startIndex > endIndex) {
                    var temp = startIndex;
                    startIndex = endIndex;
                    endIndex = temp;
                }
                var items = this._tileContent.find(".e-tilenode");
                items.removeClass("e-active").attr("aria-selected", false);
                for (var i = startIndex ; i <= endIndex; i++) {
                    this._selectItems.push($(items.get(i)).find(".e-name").text());
                }
                this._setSelectedItems(this._selectItems);
            }
            if (_isUpdate != undefined)
                this._isupdate = _isUpdate;
        },
        _updateNode: function (name) {
            this._currentPath += name + "/";
        },
        _getFileURL: function () {
            if (this._nodeType == "File") {
                if ((/\.(gif|jpg|jpeg|tiff|png|bmp)$/i).test(this._selectedContent)) {
                    this._widthTag && this._widthTag.show().removeClass("e-hide");
                    this._heightTag && this._heightTag.show().removeClass("e-hide");
                } else {
                    this._widthTag && this._widthTag.hide().addClass("e-hide");
                    this._heightTag && this._heightTag.hide().addClass("e-hide");
                }
                return (this._currentPath.replace("~", "..") + this._selectedContent);
            }
            else
                return "";
        },
        _updateData: function () {
            var details = this._fileExplorer[this._currentPath];
            var size = 0;
            if (details)
                for (var i = 0; i < details.length; i++) {
                    size += details[i].sizeInByte;
                }
            this._selectedItemsTag && this._selectedItemsTag.html("");
            size = this._bytesToSize(size);
            if (details && !this._update) {
                this._itemStatus && this._itemStatus.html(details.length + " " + (details.length == 1 ? this._getLocalizedLabels("Item") : this._getLocalizedLabels("Items")) + (size ? ", " + this._getLocalizedLabels("Size") + ": " + size : ""));
                this.model.layout == "grid" ? this._renderGridView(details) : this._renderTileView(details);
            }
            else {
                this._read();
            }
        },

        _addChild: function (result, targetNode) {
            var directories = [];
            if (result) {
                for (var i = 0; i < result.length; i++) {
                    if (!result[i].isFile) {
                        result[i].id = result[i].name;
                        result[i].spriteCssClass = "e-tree-foldericon";
                        directories.push(result[i]);
                    }
                }
            }
            this._nodeExpanded = true;
            var selectedNode = targetNode ? $(targetNode).closest('li.e-item') : this._treeObj.getSelectedNode();
            directories.length && this._treeObj.addNode(directories, selectedNode);
            selectedNode.find(".e-load") && selectedNode.find(".e-load").removeClass("e-load");
            this._nodeExpanded = false;
            this._treetag.find("li").removeAttr("tabindex");
        },

        _createContextMenuTag: function (menuOptions, menuDetails, events) {
            var contextMenu = $("body").find('#' + menuDetails.id + 'ContextMenu');
            var ContextMenutag = contextMenu.length ? contextMenu : ej.buildTag('ul.fe-context-menu #' + menuDetails.id + 'ContextMenu');
            ContextMenutag.ejMenu({
                menuType: ej.MenuType.ContextMenu,
                enableSeparator: true,
                enableRTL: this.model.enableRTL,
                cssClass: this.model.cssClass,
                contextMenuTarget: menuDetails.targetId,
                beforeOpen: events.beforeOpen,
                close: events.close,
                click: events.click,
                fields: { dataSource: menuOptions, id: "text", text: "text", htmlAttribute: "htmlAttr", spriteCssClass: "sprite" },
            });
            ContextMenutag.hide();
            return ContextMenutag;
        },

        _beforeOpenContextMenu: function (args) {
            if (!$(args.target).hasClass("e-text"))
                args.cancel = true;
            else {
                this._menuNode = $(args.target).closest('li.e-item');
                (this._treeObj) && this._treeObj.element.find('.e-node-focus').removeClass('e-node-focus');
                this._menuNode.find('> div > .e-text:first').addClass('e-node-focus');
                if ($(args.target).parents("li.e-item:first").attr("id") == 1) {
                    this._treeMenuObj.disableItem(this._menuCut);
                    this._treeMenuObj.disableItem(this._menuCopy);
                    this._treeMenuObj.disableItem(this._menuDelete);
                    this._treeMenuObj.disableItem(this._menuRename);
                }
                else {
                    this._restrictedMenuOption.indexOf(this._menuCut) < 0 && this._treeMenuObj.enableItem(this._menuCut);
                    this._restrictedMenuOption.indexOf(this._menuCopy) < 0  && this._treeMenuObj.enableItem(this._menuCopy);
                    this._restrictedMenuOption.indexOf(this._menuDelete) < 0 && this._treeMenuObj.enableItem(this._menuDelete);
                    this._restrictedMenuOption.indexOf(this._menuRename) < 0 && this._treeMenuObj.enableItem(this._menuRename);
                }
            }
        },

        _beforeOpenTileContextMenu: function (args) {
            if ($(args.target).hasClass("e-scrollbar") || $(args.target).parents().hasClass("e-scrollbar")) {
                args.cancel = true;
                return;
            }
            if ($(args.target).hasClass('e-rowcell') && !$(args.target).hasClass('e-active') && args.events && (args.events.button == 2 || args.events.which == 3))
                this._updateGridSelection(args);
            else if (($(args.target).is(".e-tilenode") || $(args.target).closest("li").hasClass("e-tilenode")) && !($(args.target).is(".e-tilenode.e-active") || $(args.target).closest(".e-tilenode").hasClass("e-active")))
                this._updateTileSelection(args);
            if ($(args.target).is(".e-tilenode.e-active") || $(args.target).closest(".e-tilenode").hasClass("e-active") || $(args.target).is(".e-rowcell.e-active") || (args.events.ctrlKey && ($(this.items).hasClass('e-active') || $(this.gridItems).find('td').hasClass('e-active'))))  {
                JSON.stringify(this._viewMenuObj.model.fields.dataSource) == JSON.stringify(this._cwdMenuOptions) && this._viewMenuObj.option("fields", { dataSource: this._fileMenuOptions, id: "text", text: "text", htmlAttribute: "htmlAttr", spriteCssClass: "sprite" });
            }
            else {
                JSON.stringify(this._viewMenuObj.model.fields.dataSource) == JSON.stringify(this._fileMenuOptions) && this._viewMenuObj.option("fields", { dataSource: this._cwdMenuOptions, id: "text", text: "text", htmlAttribute: "htmlAttr", spriteCssClass: "sprite" });
                if ($(args.events.currentTarget).hasClass("e-grid") && ($(args.target).hasClass("e-gridcontent") || $(args.target).hasClass("e-content") || $(args.target).hasClass("e-table") || !$(args.target).is(".e-rowcell.e-active"))) {
                    this._gridObj.clearSelection();
                }
                if ($(args.events.currentTarget).hasClass("e-tile-wrapper") && ($(args.target).hasClass("e-tile-wrapper") || $(args.target).hasClass("e-tile-content") || $(args.target).hasClass("e-tileview") || !($(args.target).is(".e-tilenode.e-active") || $(args.target).parent(".e-tilenode").hasClass("e-active")))) {
                    if (this.items.hasClass("e-active"))
                        this.items.removeClass("e-active").attr("aria-selected", false);
                }
                this._removeOldSelectionDetails();                
            }
            for (var i = 0; i < this._restrictedMenuOption.length; i++) {
                this._viewMenuObj && this._viewMenuObj.disableItem(this._restrictedMenuOption[i]);
            }
            if (!(/\.(bmp|dib|jpg|jpeg|jpe|jfif|gif|tif|tiff|png|ico)$/i).test(this.model.selectedItems) && this._nodeType == "File")
                this._viewMenuObj && this._viewMenuObj.disableItem("Open");            
            else
                this._viewMenuObj && this._viewMenuObj.enableItem("Open");
            if (!this._option)
                this._viewMenuObj.disableItem(this._menuPaste);
            this._isupdate ? (this._restrictedMenuOption.indexOf(this._menuDownload) < 0 && this._viewMenuObj.enableItem(this._menuDownload)) : this._viewMenuObj.disableItem(this._menuDownload);
            
        },
        _removeOldSelectionDetails: function () {
            if (this._currentPath != this._originalPath) {
                this._currentPath = this._originalPath;
            }
            this._sizeInByte = 0;
            this._selectedItems = [];
            this._selectedTileItems = [];
            this.model.selectedItems = [];
            this._selectedContent = this._selectedTreeText;
            this._nodeType = "Directory";
            this._selectedItemsTag && this._selectedItemsTag.html("");
            this._toolBarItems && this._disableToolbarItem("Download");            
            this._viewMenuObj && this._viewMenuObj.disableItem(this._menuDownload);
            this._disableEditingTools();
        },
        _contextMenuClick: function (args) {
            this._treeObj.selectNode(this._menuNode);
            this._fileContextMenuClick(args);
        },
        _fileContextMenuClick: function (args) {
            if (this.model.ajaxAction == "" || this._currentPath == "")
                return;
            switch (args.text) {
                case this._menuOpen:
                    this._openAction();
                    break;
                case this._menuNewFolder:
                    this._createNewFolder();
                    break;
                case this._menuDelete:
                    this._deleteFolder();
                    break;
                case this._menuRefresh:
                    this.refresh();
                    break;
                case this._menuRename:
                    this._renameFolder();
                    break;
                case this._menuUpload:
                    this.element.find(".e-uploadinput").click();
                    break;
                case this._menuDownload:
                    this._downloadFile();
                    break;
                case this._menuCut:
                    this._copyMoveNode("move");
                    break;
                case this._menuCopy:
                    this._copyMoveNode("copy");
                    break;
                case this._menuPaste:
                    this._cut_copy();
                    break;
                case this._menuGetinfo:
                    this._getDetails();
                    break;
            }
        },


        _createNewFolder: function () {
            var proxy = this;
            var name = "";
            var viewerData = this._getLocalizedLabels("NewFolderAlert");
            var dialogContent = ej.buildTag('div.e-get-name');
            var labeltag = ej.buildTag('div.e-fe-dialog-label', viewerData);
            var inputtag = ej.buildTag('input.e-fe-dialog-text e-ejinputtext', "", "", { type: "text" });
            inputtag.val("New folder");
            var divtag = ej.buildTag('div.e-fe-dialog-btn');
            var okButton = ej.buildTag('button.e-fe-btn-ok ', this._getLocalizedLabels("OkButton"));
            var cancelButton = ej.buildTag('button.e-fe-btn-cancel ', this._getLocalizedLabels("CancelButton"));
            okButton.ejButton({
                type: "button",
                click: function () {
                    proxy._removeDialog(proxy._newFolderDialogObj);
                    name = inputtag.val();
                    if (!$.trim(name)) {
                        name = "New folder";
                    }
                    !proxy._fileExplorer[proxy._currentPath] && proxy._getFileDetails(proxy._currentPath);
                    var data = proxy._fileExplorer[proxy._currentPath];
                    if (!data.length)
                        name && proxy._createFolder(name);
                    else {
                        for (var i = 0; i < data.length; i++) {
                            if (!data[i].isFile && data[i].name == name) {
                                var dialogContent = ej.buildTag('div.e-get-name');
                                var labeltag = ej.buildTag('div.e-fe-dialog-label', String.format(proxy._getLocalizedLabels("ErrorOnFolderCreation"), name));
                                var divtag = ej.buildTag('div.e-fe-dialog-btn');
                                var okButton = ej.buildTag('button.e-fe-btn-ok ', proxy._getLocalizedLabels("YesButton"));
                                var cancelButton = ej.buildTag('button.e-fe-btn-cancel ', proxy._getLocalizedLabels("NoButton"));
                                okButton.ejButton({
                                    type: "button",
                                    click: function () {
                                        proxy._removeDialog(proxy._alertDialogObj);
                                    }
                                });
                                cancelButton.ejButton({
                                    type: "button",
                                    click: function () {
                                        proxy._removeDialog(proxy._alertDialogObj);
                                        proxy._createFolder(proxy._getDuplicateName(data, "New folder"));
                                    }
                                });
                                divtag.append(okButton, cancelButton);
                                $(dialogContent).append(labeltag, divtag);
                                var open = function () {
                                    okButton.focus();
                                };
                                proxy._alertDialog = proxy._createDialog(dialogContent, { width: 400, height: "auto", title: proxy._getLocalizedLabels("Error"), open: open });
                                proxy._alertDialogObj = proxy._alertDialog.data("ejDialog");
                                proxy._alertDialogObj.initialRender = false;
                                proxy._alertDialogObj.open();
                                break;
                            }
                            else if (i == data.length - 1) {
                                name && proxy._createFolder(name);
                            }
                        }
                    }
                }
            });
            cancelButton.ejButton({
                type: "button",
                click: function () {
                    proxy._removeDialog(proxy._newFolderDialogObj);
                }
            });
            divtag.append(okButton, cancelButton);
            $(dialogContent).append(labeltag, inputtag, divtag);
            var open = function () {
                inputtag.focus();
                inputtag.select();
                inputtag.keyup(function (e) {
                    var code = proxy._getKeyCode(e);
                    if (code == 13)
                        okButton.click();
                });
            };
            this._newFolderDialog = this._createDialog(dialogContent, { width: 350, height: "auto", open: open, title: this._getLocalizedLabels("NewFolder") });
            this._newFolderDialogObj = this._newFolderDialog.data("ejDialog");
            this._newFolderDialogObj.initialRender = false;
            this._newFolderDialogObj.open();
        },

        _deleteFolder: function () {
            this._alertWindow = ej.buildTag("div#" + "e-fe_deleteAlert");
            var viewerData;
            if (this._selectedItems.length > 1)
                viewerData = String.format(this._getLocalizedLabels("DeleteMultipleFolder"), this._selectedItems.length);
            else
                viewerData = this._getLocalizedLabels("DeleteFolder") + this._selectedContent + "?";
            var labeltag = ej.buildTag('div.e-fe-dialog-label', viewerData);
            var divTag = ej.buildTag('div.e-fe-dialog-btn');
            var okButton = ej.buildTag('button.e-fe-btn-ok', this._getLocalizedLabels("OkButton"));
            var cancelButton = ej.buildTag('button.e-fe-btn-cancel', this._getLocalizedLabels("CancelButton"));
            divTag.append(okButton, cancelButton);
            this._alertWindow.append(labeltag, divTag);
            proxy = this;
            okButton.ejButton({
                type: "button",
                click: function () {
                    proxy._removeDialog(proxy._alertWindowObj);
                    proxy._ajax_folderPath = proxy._nodeType == "Directory" ? proxy._getFolderPath() : proxy._currentPath;
                    proxy._deletion(proxy._selectedItems.length > 1 ? proxy._selectedItems : proxy._selectedContent, proxy._ajax_folderPath);
                    proxy._disableEditingTools();
                }
            });
            cancelButton.ejButton({
                type: "button",
                click: function () {
                    proxy._removeDialog(proxy._alertWindowObj);
                }
            });
            var open = function () {
                okButton.focus();
            };
            this._alertWindow = this._createDialog(this._alertWindow, { width: 350, height: "auto", title: this._getLocalizedLabels("Delete"), open: open });
            this._alertWindowObj = this._alertWindow.data("ejDialog");
            this._alertWindowObj.initialRender = false;
            this._alertWindowObj.open();
        },
        _getFileDetails: function (path, updateTreeNode, updateLayoutContent) {
            var proxy = this;
            var _ajaxOptions = {
                data: { ActionType: "Read", Path: path, ExtensionsAllow: this.model.fileTypes, SelectedItems: this._getSelectedItemDetails(this._getFolderPath(path), (this.model.selectedItems.length ? this.model.selectedItems : this._selectedContent)) },
                url: this.model.ajaxAction,
                type: "POST",
                async: false,
                success: function (result) {
                    result = (result.hasOwnProperty("d")) ? result.d : result;
                    for (var i = 0; i < result.files.length; i++) {
                        result.files[i].sizeInByte = result.files[i].size;
                        result.files[i].size = result.files[i].size ? proxy._bytesToSize(result.files[i].size) : "";
                    }
                    proxy._fileExplorer[path] = result.files;
                    updateTreeNode && proxy._addChild(proxy._fileExplorer[path], $(updateTreeNode));
                    updateLayoutContent && (proxy.model.layout == "grid" ? proxy._renderGridView(result.files) : proxy._renderTileView(result.files));
                    if (proxy._highlightedNodes) {
                        proxy._setSelectedItems(proxy._highlightedNodes);
                        proxy._highlightedNodes = "";
                    }
                },
                successAfter: this.model.ajaxSettings.read.success
            };
            this.model.ajaxSettings.read.success = undefined;
            $.extend(true, _ajaxOptions, this.model.ajaxSettings.read);
            this._sendAjaxRequest(_ajaxOptions);
        },
        _getDuplicateName: function (fileDetails, name) {
            var directoryCount = 0;
            var initialName = name;
            while (this._isNameExist(fileDetails, name)) {
                directoryCount++;
                name = initialName.split(".")[0] + (directoryCount > 0 ? "(" + directoryCount + ")" : "") + (name.split(".")[1] ? "." + name.split(".")[1] : "");
            }
            return name;
        },
        _isNameExist: function (fileDetails, name) {
            for (var i = 0; i < fileDetails.length; i++) {
                if (name == fileDetails[i].name)
                    return true;
            }
            return false;
        },

        _renameFolder: function () {
            var proxy = this;
            var viewerData = this._getLocalizedLabels("RenameAlert");
            var dialogContent = ej.buildTag('div.e-rename');
            var labeltag = ej.buildTag('div.e-fe-dialog-label', viewerData);
            var inputtag = ej.buildTag('input.e-fe-dialog-text e-ejinputtext', "", "", { type: "text", value: (this._nodeType == "File" ? this._selectedContent.split(".")[0] : this._selectedContent) });
            var divtag = ej.buildTag('div.e-fe-dialog-btn');
            var okButton = ej.buildTag('button.e-fe-btn-ok', this._getLocalizedLabels("OkButton"));
            var cancelButton = ej.buildTag('button.e-fe-btn-cancel', this._getLocalizedLabels("CancelButton"));
            okButton.ejButton({
                type: "button",
                click: function () {
                    var person = inputtag.val(), _oldName = (proxy._nodeType == "File" ? proxy._selectedContent.split(".")[0] : proxy._selectedContent);
                    proxy._removeDialog(proxy._renameDialogObj);
                    if ($.trim(person) && person != _oldName) {
                        var extention = (proxy._nodeType == "File" ? proxy._selectedContent.split(".")[1] : "");
                        person = extention ? (person + "." + extention) : person;
                        proxy._currentPath = proxy._nodeType == "Directory" ? proxy._getFolderPath() : proxy._currentPath;
                        !proxy._fileExplorer[proxy._currentPath] && proxy._getFileDetails(proxy._currentPath);
                        var data = proxy._fileExplorer[proxy._currentPath];
                        proxy._ajax_person = person;
                        for (var i = 0; i < data.length; i++) {
                            if (data[i].name == person) {
                                if (proxy._nodeType == "File") {
                                    person = proxy._getDuplicateName(proxy._fileExplorer[proxy._currentPath], person);
                                    proxy._ajax_person = person;
                                }
                                var dialogContent = ej.buildTag('div.e-get-name');
                                var labeltag = ej.buildTag('div.e-fe-dialog-label', String.format(proxy._getLocalizedLabels(proxy._nodeType == "File" ? "DuplicateFileCreation" : "ErrorOnFolderCreation"), person));
                                var divtag = ej.buildTag('div.e-fe-dialog-btn');
                                var okButton = ej.buildTag('button.e-fe-btn-ok ', proxy._getLocalizedLabels("OkButton"));
                                var cancelButton = ej.buildTag('button.e-fe-btn-cancel ', proxy._getLocalizedLabels("CancelButton"));
                                okButton.ejButton({
                                    type: "button",
                                    click: function () {
                                        proxy._removeDialog(proxy._alertDialogObj);
                                        if (proxy._nodeType == "File")
                                            proxy._rename();
                                        else {
                                            proxy._existingItems = [];
                                            proxy._getDuplicateItems(proxy._currentPath + proxy._selectedContent + "/", proxy._currentPath + person + "/", true);
                                            if (proxy._existingItems.length) {
                                                proxy._createReplaceConformationDiaolg("_rename", "DuplicateAlert");
                                            }
                                            else
                                                proxy._rename();
                                        }
                                    }
                                });
                                cancelButton.ejButton({
                                    type: "button",
                                    click: function () {
                                        proxy._removeDialog(proxy._alertDialogObj);
                                    }
                                });
                                divtag.append(okButton, cancelButton);
                                $(dialogContent).append(labeltag, divtag);
                                var open = function () {
                                    okButton.focus();
                                };
                                proxy._alertDialog = proxy._createDialog(dialogContent, { width: 400, height: "auto", title: proxy._getLocalizedLabels("Error"), open: open });
                                proxy._alertDialogObj = proxy._alertDialog.data("ejDialog");
                                proxy._alertDialogObj.initialRender = false;
                                proxy._alertDialogObj.open();
                                break;
                            }
                            else if (i == data.length - 1) {
                                proxy._rename();
                            }
                        }
                    }
                }
            });
            cancelButton.ejButton({
                type: "button",
                click: function () {
                    proxy._removeDialog(proxy._renameDialogObj);
                }
            });

            divtag.append(okButton, cancelButton);
            $(dialogContent).append(labeltag, inputtag, divtag);
            var open = function () {
                inputtag.focus();
                inputtag.select();
                inputtag.keyup(function (e) {
                    var code = proxy._getKeyCode(e);
                    if (code == 13)
                        okButton.click();
                });
            };
            this._renameDialog = this._createDialog(dialogContent, { width: 350, height: "auto", open: open, title: this._getLocalizedLabels("Rename") });
            this._renameDialogObj = this._renameDialog.data("ejDialog");
            this._renameDialogObj.initialRender = false;
            this._renameDialogObj.open();
        },

        _isSelectedFile: function (files, name) {
            for (var i = 0; i < files.length; i++) {
                if (name == files[i])
                    return true;
            }
            return false;
        },
        _refreshItems: function (node, path) {
            node = $(node);
            this._fileExplorer[path] = "";
            var proxy = this;
            $.each(proxy._fileExplorer, function (itemPath, value) {
                //display the key and value pair
                if (itemPath.startsWith(path))
                    proxy._fileExplorer[itemPath] = "";
            });
            var childNodes = node.find('ul:first > li');
            for (var i = 0; i < childNodes.length; i++) {
                node.find(childNodes[i]).length && this._treeObj.removeNode($(childNodes[i]));
            }
            this._getFileDetails(path, node.find('> div > .e-text'), true);
        },
        _getDuplicateItems: function (source, target, files) {
            !this._fileExplorer[target] && this._getFileDetails(target);
            var targetFiles = this._fileExplorer[target];
            !this._fileExplorer[source] && this._getFileDetails(source);
            var sourceFiles = this._fileExplorer[source];
            for (var s = 0; s < sourceFiles.length; s++) {
                for (var t = 0; t < targetFiles.length; t++) {
                    if (sourceFiles[s].name == targetFiles[t].name) {
                        if (files == true || this._isSelectedFile(files, targetFiles[t].name)) {
                            this._existingItems[this._existingItems.length] = { Name: targetFiles[t].name, Path: target + targetFiles[t].name + (!targetFiles[t].isFile ? "/" : ""), IsReplace: true };
                            if (!targetFiles[t].isFile)
                                this._getDuplicateItems(source + sourceFiles[s].name + "/", target + targetFiles[t].name + "/", true);
                        }
                    }
                }
            }
        },
        _backward: function () {
            if (this._currentState > 0) {
                //minimum number of states to show forward icon
                var minState = 2;
                var update = true;
                var disableBackwardPosition = 0;
                this._isStateNavigation = true;
                var oldIndex = this._currentState;
                --this._currentState;
                while (this._selectedStates[this._currentState] == "" || this._selectedStates[this._currentState] == this._selectedStates[oldIndex]) {
                    if (this._currentState)
                        --this._currentState;
                    else
                        update = false;                        
                }
                update && this._selectedFolder(this._selectedStates[this._currentState]);
                this._isStateNavigation = false;                
                (this._currentState == disableBackwardPosition || this._selectedStates.length - minState == this._currentState) && this._toolBarItems && this._enableToolbarItem("Forward");
                this._currentState == disableBackwardPosition && this._toolBarItems && this._disableToolbarItem("Back");
            }
        },
        _forward: function () {
            if (this._currentState + 1 < this._selectedStates.length) {
                //used to get index from array length
                var update = true;
                var reduceIndex = 1;
                var disableState = 0;
                var enableState = 1;
                this._isStateNavigation = true;
                var oldIndex = this._currentState;
                ++this._currentState;
                while (this._selectedStates[this._currentState] == "" || this._selectedStates[this._currentState] == this._selectedStates[oldIndex]) {
                    if (this._currentState < this._selectedStates.length - 1)
                        ++this._currentState;
                    else
                        update = false;                        
                }
                update && this._selectedFolder(this._selectedStates[this._currentState]);
                this._isStateNavigation = false;
                this._selectedStates.length - reduceIndex == this._currentState && this._toolBarItems && this._disableToolbarItem("Forward");
                this._currentState == 0 && this._toolBarItems && this._disableToolbarItem("Back");
                this._currentState == 1 && this._toolBarItems && this._enableToolbarItem("Back");
            }
        },
        _copyMoveNode: function (action) {
            if (action == "move") {
                this.element.find(".e-blur").removeClass("e-blur");
                var activeElement = this.element.find(".e-splitter .e-active");
                activeElement.length && activeElement.length == 1 ? activeElement.addClass("e-blur") : this.element.find(".e-cont2 .e-active").addClass("e-blur");
            }
            this._option = action;
            this._sourcePath = this._nodeType == "Directory" ? this._getFolderPath() : this._currentPath;
            this._sourceType = this._nodeType;
            this._fileName = this._selectedItems.length > 1 ? this._selectedItems : this._selectedContent;
            this._refreshNode = this._originalPath != this._currentPath ? this._findMatchingElement(this._selectedNode.find("ul:first"), this._selectedContent) : this._selectedNode;
            this._toolBarItems && this._enableToolbarItem("Paste");
            if (this.model.showContextMenu) {
                this._restrictedMenuOption.indexOf(this._menuPaste) < 0 && this._viewMenuObj.enableItem(this._menuPaste);
                this._restrictedMenuOption.indexOf(this._menuPaste) < 0 && this._treeMenuObj.enableItem(this._menuPaste);
            }
            var args = { name: this._fileName, sourcePath: this._sourcePath, selectedItems: this._getSelectedItemDetails(this._sourcePath, this._fileName) };
            if (action == "move")
                this._trigger("cut", args);
            else
                this._trigger("copy", args);
        },
        _openAction: function () {
            var proxy = this;
            var selectedNodes;
            if (this._onBeforeOpen()) return;
            if (this._nodeType == "Directory") {
                var selectedNode = this._treeObj.getSelectedNode(), _selectedContent = this._selectedContent;
                if (!this._treeObj.isExpanded(selectedNode))
                    selectedNode.find(".e-icon").click();
                var _childItems = selectedNode.find('ul:first>li').find('div:first .e-text');
                if (!this._treeObj.hasChildNode(selectedNode)) {
                    var path = (this._isTreeNode == true ? this._currentPath : this._currentPath.replace(this._selectedContent + "/", ""));
                    this._isTreeNode = false;
                    this._addChild(this._fileExplorer[path]);
                }
                for (var i = 0; i < _childItems.length; i++) {
                    if ($(_childItems[i]).text() == _selectedContent) {
                        this._treeObj.selectNode(_childItems[i].parentNode.parentNode);
                        break;
                    }
                }
                selectedNodes = this._getSelectedItemDetails(this._getFolderPath(), _selectedContent);
                this._selectedItems = [];
                this._selectedTileItems = [];
            }
            else if (this._nodeType == "File") {
                this._selectedFile = this._selectedContent;
                if ((/\.(bmp|dib|jpg|jpeg|jpe|jfif|gif|tif|tiff|png|ico)$/i).test(this._selectedFile)) {
                    proxy._openDialog = ej.buildTag('div.e-imageViewer', "", "", { id: proxy._ExplorerId + '_basicDialog', title: proxy._selectedFile });
                    var path = proxy._currentPath.replace("~", "..") + this._selectedFile;
                    var url = this._getImage(path, this._selectedFile);
                    var imagetag = ej.buildTag('img', "", "", { src: (url ? url : path) });
                    $(proxy._openDialog).append(imagetag);
                    $(proxy.element).append(proxy._openDialog);
                    proxy._openDialog.ejDialog({
                        width: 450,
                        height: 350,
                        minHeight: 200,
                        minWidth: 300,
                        closeIconTooltip: proxy._getLocalizedLabels("DialogCloseToolTip"),
                        enableRTL: proxy.model.enableRTL,
                        cssClass: proxy.model.cssClass,
                        close: function (e) { proxy._onDialogClose(e); }
                    }).parents(".e-dialog-wrap").addClass("e-imageViewer-wrap");
                    proxy._openDialog.css("height", "90%");
                    proxy._openDialogObj = proxy._openDialog.data("ejDialog");
                }
            }
            var fileUrl;
            fileUrl = this._nodeType == "File" ? path : this._currentPath;
            var args = { path: fileUrl, itemType: this._nodeType, selectedItems: selectedNodes };
            this._trigger("open", args);
        },
        _getSelectedItemDetails: function (path, names) {
            if (typeof names == "string")
                names = [names];
            var itemDetails = [];
            var data = this._fileExplorer[path];
            if (data) {
                for (var j = 0; j < names.length; j++) {
                    for (var i = 0; i < data.length; i++) {
                        if (data[i].name == names[j])
                            itemDetails.push(data[i]);
                    }
                }
            }
            return itemDetails;
        },
        _getDetails: function () {
            var _path = this._nodeType == "Directory" ? this._getFolderPath() : this._currentPath;
            var proxy = this;
            var _ajaxOptions = {

                data: { ActionType: "GetDetails", Path: _path, Names: (typeof this._selectedContent == "string") ? [this._selectedContent] : this._selectedContent, SelectedItems: this._getSelectedItemDetails(_path, (this.model.selectedItems.length ? this.model.selectedItems : this._selectedContent)) },
                url: this.model.ajaxAction,
                type: "POST",
                async: false,
                success: function (result) {
                    if (result.hasOwnProperty("d"))
                        result = result.d;
                    var dialogContent = ej.buildTag('div.e-fe-table');
                    var tabletag = ej.buildTag('table'), trtag, tdtag1, tdtag2, wrapDiv, inputtag, kbsize, tdata;
                    $.each(result.details[0], function (name, value) {
                        trtag = ej.buildTag('tr');
                        tdtag1 = ej.buildTag('td', ej.isNullOrUndefined(proxy._getLocalizedLabels(name)) ? name : proxy._getLocalizedLabels(name));
                        tdtag2 = ej.buildTag('td');
                        if (name == "Name" || name == "Location") {
                            wrapDiv = ej.buildTag('div');
                            inputtag = ej.buildTag('input.e-readonly', "", "", { style: "border:none;", type: 'text', value: value, title: value, readonly: true });
                            inputtag.focus(function () {
                                $(this).blur();
                            });
                            wrapDiv.append(inputtag);
                        } else if (name == "Size") {
                            kbsize = proxy._bytesToSize(value);
                            wrapDiv = ej.buildTag('span', kbsize + " (" + value + " Bytes)");
                        } else {
                            wrapDiv = ej.buildTag('span', value);
                        }
                        tdtag2.append(wrapDiv);
                        $(tabletag).append(trtag);
                        $(trtag).append(tdtag1, tdtag2);
                        (name == "Name" || name == "Size") && trtag.addClass('e-border');
                    });
                    $(dialogContent).append(tabletag);
                    proxy._detailsDialog = proxy._createDialog(dialogContent, { width: 500, height: "auto", title: proxy._getLocalizedLabels("Details") });
                    proxy._detailsDialogObj = proxy._detailsDialog.data("ejDialog");
                    proxy._detailsDialogObj.initialRender = false;
                    proxy._detailsDialogObj.open();
                },
                successAfter: this.model.ajaxSettings.getDetails.success
            };
            this.model.ajaxSettings.getDetails.success = undefined;
            $.extend(true, _ajaxOptions, this.model.ajaxSettings.getDetails);
            this._sendAjaxRequest(_ajaxOptions, true);
        },
        _uploadFile: function () {
            proxy = this.element.find(".e-uploadbox");
            proxy = proxy.ejUploadbox("instance");
        },

        _getFolderPath: function (path) {
            var str_array = path? path.split('/'):this._currentPath.split('/');            
            var editedPath = "";
            for (var i = 0; i < str_array.length - 2; i++) {
                editedPath += str_array[i] + "/";
            }
            return editedPath;
        },
        _renderToolBar: function () {
            this._crateToolbarTemplate();
            this._initToolbarItems();
            this._toolbar = this._toolBarItems;
            var model = {};
            model.click = $.proxy(this._toolBarClick, this);
            model.cssClass = this.model.cssClass;
            model.enableRTL = this.model.enableRTL;
            model.enableSeparator = true;
            model.isResponsive = this.model.isResponsive;
            model.height = model.isResponsive ? "" : "auto";
            model.cssClass = this.model.cssClass + " e-fe-toolbar";
            model.enableRTL = this.model.enableRTL;
            this._toolBarItems.ejToolbar(model);
            this._toolBarObj = this._toolBarItems.ejToolbar("instance");
            this._disableToolbarItem("Paste");
            this._disableToolbarItem("Download");
            this._disableToolbarItem("Back");
            this._disableToolbarItem("Forward");
            this._disableEditingTools();
        },
        _initToolbarItems: function () {
            this._toolBarItems.find("#" + this._ExplorerId + "Addressbar").length > 0 && this._createAddressBar();
            this._toolBarItems.find("#" + this._ExplorerId + "Searchbar").length > 0 && this._searchDetails();
        },
        _createUploadBox: function () {
            this._uploadtag = ej.buildTag('div#' + this._ExplorerId + 'FileUpload', "", { padding: "0px", height: "0px", width: "0px" });
            this.element.prepend(this._uploadtag);
            this._renderUploadBox();
            this._uploadtag.find(".e-inputbtn").hide();
            this._uploadtag.find(".e-uploadinput").attr("tabindex", -1);
        },
        _searchDetails: function () {
            this._searchbar = ej.buildTag('input.e-searchBar e-tool-input', "", {}, { id: this._ExplorerId + '_searchbar', type: "text", placeholder: this._getLocalizedLabels("Search") });
            this._searchbar.appendTo(this._toolBarItems.find("#" + this._ExplorerId + "Searchbar").html(""));
            this._on($('#' + this._ExplorerId + '_searchbar'), "keyup", this._validateKeyCode);
        },
        _setUploadLocalization: function () {
            $.each(ej.FileExplorer.Locale, function (locale, value) {
                if (value.UploadSettings) {
                    ej.Uploadbox.Locale[locale] = value.UploadSettings;
                }                
            });            
        },
        _getLocalizedLabels: function (property) {
            return ej.FileExplorer.Locale[this.model.locale][property] === undefined ? (ej.FileExplorer.Locale["en-US"][property] ? ej.FileExplorer.Locale["en-US"][property] : property) : ej.FileExplorer.Locale[this.model.locale][property];
        },
        _crateToolbarTemplate: function () {
            this._toolBarItems = ej.buildTag("div#" + this._ExplorerId + "_toolbar").prependTo(this.element);
            for (var item = 0; item < this.model.toolsList.length; item++) {
                items = this.model.toolsList[item];
                if (!ej.isNullOrUndefined(this.model.tools[items])) {
                    if (items == "customTool")
                        !ej.isNullOrUndefined(this.model.tools[items]) && this._customTools(this.model.tools[items]);
                    else
                        this.model.tools[items].length > 0 && this._createToolsItems(this.model.tools[items], items);
                }
            }
        },

        _createToolsItems: function (items, itemName) {
            var ulTag = ej.buildTag("ul#" + (this._ExplorerId + itemName)), liTag;
            ulTag.addClass("e-ul-" + itemName);
            for (var i = 0; i < items.length; i++) {
                liTag = $("<li id='" + (this._ExplorerId + items[i].replace(/ /g, '')) + "' class='e-feItem-" + items[i] + "' title='" + this._getLocalizedLabels(items[i].replace(/ /g, '')) + "' ><div class='e-fileexplorer-toolbar-icon " + items[i] + "'></div></li>");
                liTag.appendTo(ulTag);
            }
            ulTag.appendTo(this._toolBarItems);
        },
        _customTools: function (toolbarItems) {
            for (var item = 0; item < toolbarItems.length; item++) {
                var ulTag = ej.buildTag("ul"), liTag;
                liTag = $("<li id='" + (this._ExplorerId + toolbarItems[item].name.replace(/ /g, '')) + "' title='" + toolbarItems[item].tooltip + "' ><div class='" + (ej.isNullOrUndefined(toolbarItems[item].css) ? "" : toolbarItems[item].css) + "'></div></li>");
                var fn = toolbarItems[item].action;
                if (typeof fn === "string") {
                    fn = ej.util.getObject(fn, window);
                }
                !ej.isNullOrUndefined(toolbarItems[item].action) && this._on(liTag, "click", fn);
                $(toolbarItems[item].template).appendTo(liTag.find("div"));
                liTag.appendTo(ulTag);
                ulTag.appendTo(this._toolBarItems);
            }
        },
        _toolBarClick: function (args) {
            if ((args.event.which && (args.event.which == 3 || args.event.which == 2)) || (args.event.button && args.event.button == 2))
                return false;
            if (this.model.ajaxAction == "" || this._currentPath == "")
                return;
            this._lastFocusedElement = $(args.currentTarget);
            var currrentElement = $(args.currentTarget);
            var selected = currrentElement.attr("id");
            switch (selected) {
                case this._ExplorerId + "Addressbar":
                case this._ExplorerId + "Searchbar":
                    currrentElement.find("input").focus();
                    currrentElement.find("input").select();
                    break;
                case this._ExplorerId + "Download":
                    this._downloadFile();
                    break;
                case this._ExplorerId + "Upward":
                    this._upward();
                    break;
                case this._ExplorerId + "NewFolder":
                    this._createNewFolder();
                    break;
                case this._ExplorerId + "Delete":
                    this._deleteFolder();
                    break;
                case this._ExplorerId + "Rename":
                    this._renameFolder();
                    break;
                case this._ExplorerId + "Refresh":
                    this._currentPath = this._originalPath;
                    this._highlightedNodes = this.model.selectedItems;
                    this._refreshItems(this._treeObj.getSelectedNode(), this._originalPath);
                    break;
                case this._ExplorerId + "Back":
                    this._backward();
                    break;
                case this._ExplorerId + "Forward":
                    this._forward();
                    break;
                case this._ExplorerId + "Cut":
                    this._copyMoveNode("move");
                    break;
                case this._ExplorerId + "Copy":
                    this._copyMoveNode("copy");
                    break;
                case this._ExplorerId + "Paste":
                    this._currentPath = this._originalPath;
                    this._cut_copy();
                    break;
                case this._ExplorerId + "Open":
                    this._openAction();
                    break;
                case this._ExplorerId + "Details":
                    this._getDetails();
                    break;
                case this._ExplorerId + "Upload":
                    this.element.find(".e-uploadinput").click();
                    break;
            }
        },
        _upward: function () {            
            this._treeObj && this._treeObj.selectNode(this._treeObj.getSelectedNode().parent().closest('li.e-item'));            
        },
        _getFilteredList: function (list) {
            if (!list || !list.length || list.length < 1) this.suggestionListItems = [];
            else {
                var tempQuery = ej.Query();
                this._addQuery(tempQuery, typeof list[0] == "object");
                this.suggestionListItems = ej.DataManager(list).executeLocal(tempQuery);
            }
            this.model.layout == "grid" ? this._renderGridView(this.suggestionListItems) : this._renderTileView(this.suggestionListItems);
            this._updateItemStatus(this.suggestionListItems);
        },
        _addQuery: function (_query, checkMapper) {
            var bindTo = "";
            if (checkMapper) {
                bindTo = "name";
            }
            if (this._queryString)
                _query.where(bindTo, this.model.filterSettings.filterType, this._queryString, !this.model.filterSettings.caseSensitiveSearch);
        },
        _updateItemStatus: function (items) {
            if (items) {
                var size = 0;
                for (var i = 0; i < items.length; i++) {
                    size += items[i].sizeInByte;
                }
                size = this._bytesToSize(size);
                this._itemStatus && this._itemStatus.html(items.length + " " + (items.length == 1 ? this._getLocalizedLabels("Item") : this._getLocalizedLabels("Items")) + ", " + this._getLocalizedLabels("Size") + ": " + size);
            }            
        },
        _validateKeyCode: function (event) {
            switch (event.which) {
                case 38:
                case 40:
                case 37:
                case 39:
                case 20:
                case 16:
                case 17:
                case 18:
                case 35:
                case 36:
                case 144: break;
                case 27:
                    this._searchbar.val("");
                    this._queryString = "";
                    var address = this._addresstag.val();
                    this._removeOldSelectionDetails();
                    this._currentPath = this._currentPath.split(address)[0] + address;
                    this._getFilteredList(this._fileExplorer[this._currentPath]);
                    this.model.layout == 'grid' ? this._addFocus(this._gridtag.find(".e-gridcontent")) : this._addFocus(this._tileViewWrapper);
                    break;
                case 9:
                case 46:
                case 8:
                case 13:
                default:
                    this._queryString = event.currentTarget.value;
                    var address = this._addresstag.val();
                    this._removeOldSelectionDetails();
                    this._currentPath = this._currentPath.split(address)[0] + address;
                    this._getFilteredList(this._fileExplorer[this._currentPath]);
                    break;
            }
        },
        _onDialogClose: function (args) {
            $("body").find("#" + this._ExplorerId + "_basicDialog_wrapper").remove();
            this._lastFocusedElement && this._lastFocusedElement.focus();
        },
        _switchView: function (event) {
            var changed = false;
            if (event.currentTarget.getAttribute("id") == this._ExplorerId + '_swithListView') {
                if (this.model.layout != "tile") {
                    this.model.layout = "tile";
                    changed = true;
                }
            } else {
                if (this.model.layout != "grid") {
                    this.model.layout = "grid";
                    changed = true;
                }
            }
            if (changed) {
                this._switchLayoutView();
            }
        },
        _switchLayoutView: function () {
            var changed = false;
            this._currentPath = this._originalPath;
            if (this.model.layout == "tile") {
                this._gridtag.hide();
                this._tileContent.parent().show();
                this._renderTileView(this._fileExplorer[this._currentPath]);
                if (this._statusbar) {
                    this._statusbar.find(".e-swithListView").addClass("e-active");
                    this._statusbar.find(".e-switchGridView").removeClass("e-active");
                }
                changed = true;
            } else if (this.model.layout == "grid") {
                this._tileView && this._tileContent.parent().hide();
                this._gridtag.show();
                this._renderGridView(this._fileExplorer[this._currentPath]);
                if (this._statusbar) {
                    this._statusbar.find(".e-swithListView").removeClass("e-active");
                    this._statusbar.find(".e-switchGridView").addClass("e-active");
                }
                changed = true;
            }
            if (changed) {
                this._updateItemStatus(this._fileExplorer[this._currentPath]);
                this._setSelectedItems(this.model.selectedItems);
                this._searchbar && this._searchbar.val("");
                var args = { layoutType: this.model.layout };
                this._trigger("layoutChange", args);
            }
        },
        _wireEvents: function () {
            this._on($('#' + this._ExplorerId + '_newFolder'), "click", this._createNewFolder);
            this._on($('#' + this._ExplorerId + '_switchGridView'), "click", this._switchView);
            this._on($('#' + this._ExplorerId + '_swithListView'), "click", this._switchView);
            this._on(this.element, "keydown", this._keyDownOnInput);
            this._on(this._gridtag, "click", this._gridtagClick);
            this._on(this._tileViewWrapper, "click", this._tileViewWrapperClick);
        },
        _onHideContextMenu: function() {
            (this._treeObj) && this._treeObj.element.find('.e-node-focus').removeClass('e-node-focus');
        },

        _adjustSize: function(args){
            this._isWindowResized = args ? true : false;
            this.adjustSize();
        },
        _closeDialog: function (event) {
            this._unwireDialogEvent(event);
            $(event.target).closest("div.e-dialog").find(".e-dialog").ejDialog("close");
        },

        _searchPath: function (event) {
            var code = this._getKeyCode(event);
            switch (code) {
                case 13:
                    this._searchbar.val("");
                    var text = this._addresstag.val(), Items = text.split("/"), element = this._treetag;
                    for (var i = 0; i < Items.length; i++) {
                        if (Items[i]) {
                            element = this._findMatchingElement($(element).children("ul"), Items[i]);
                            if(element.length)
                                this._treetag.ejTreeView("selectNode", element) 
                            else {
                                this._alertDialog = this._createDialog(ej.buildTag('div.e-fe-dialog-label', String.format(this._getLocalizedLabels("ErrorPath"), text)), { width: 400, height: "auto", title: this._getLocalizedLabels("Error") });
                                this._alertDialogObj = this._alertDialog.data("ejDialog");
                                this._alertDialogObj.initialRender = false;
                                this._alertDialogObj.open();
                            }
                        }
                    }
                    break;
                case 27:
                    event.preventDefault();
                    this._updateAddressBar();
                    this._toolBarItems.focus();
                    break;
            }
        },

        _addressbarFocusout: function (event) {
            this._updateAddressBar();
        },

        _createDialog: function (contentTag, model) {
            var proxy = this;
            var dialog = ej.buildTag('div#' + this._ExplorerId + '_basicDialog.e-fe-dialog');
            $(contentTag).css("overflow", "hidden");
            $(dialog).append(contentTag);
            dialog.ejDialog({
                title: (model.title) ? model.title : "",
                width: model.width,
                closeIconTooltip: proxy._getLocalizedLabels("DialogCloseToolTip"),
                height: model.height,
                enableModal: true,
                showHeader: true,
                showOnInit:false,
                enableResize: false,
                enableAnimation: false,
                allowKeyboardNavigation: false,
                enableRTL: this.model.enableRTL,
                cssClass: this.model.cssClass + " e-fe-dialog",
                open: model.open,
                close: function (e) {  proxy._onDialogClose(e); }
            })
            return dialog;
        },
        _findMatchingElement: function (element, text) {
            return $(element).children("li").filter(function (index) {
                if ($(this).find(".e-text:first").text() == text)
                    return $(this)
            });
        },
        _getKeyCode: function (e) {
            var code;
            if (e.keyCode) code = e.keyCode; // ie and mozilla/gecko
            else if (e.which) code = e.which; // ns4 and opera
            else code = e.charCode;
            return code
        },


        _keyDownOnInput: function (e) {
            if ($(e.target).hasClass("e-tool-input"))
                return;
            var code = this._getKeyCode(e);
            switch (code) {
                case 49: // Ctrl + Shift + 1 for toolbar focus
                    if (e.shiftKey && e.ctrlKey && this.model.showToolbar) {
                        e.preventDefault();
                        this._addFocus(this._toolBarItems);
                    }
                    break;
                case 50: // Ctrl + Shift + 2 for treeview focus
                    if (e.shiftKey && e.ctrlKey && this.model.showNavigationPane) {
                        e.preventDefault();
                        this._addFocus(this._treetag);
                    }
                    break;
                case 51: // Ctrl + Shift + 3 for splitter focus
                    if (e.shiftKey && e.ctrlKey) {
                        e.preventDefault();
                        this._addFocus($(this._splittag.find(".e-splitbar")[0]));
                    }
                    break;
                case 52: // Ctrl + Shift + 4 for gridview focus
                    if (e.shiftKey && e.ctrlKey) {
                        e.preventDefault();
                        this._addFocus(this._gridtag.find(".e-gridcontent"));
                    }
                    break;
                case 53: // Ctrl + Shift + 5 for tileview focus
                    if (e.shiftKey && e.ctrlKey) {
                        e.preventDefault();
                        this._addFocus(this._tileViewWrapper);
                    }
                    break;
                case 54: // Ctrl + Shift + 6 for Statusbar focus
                    if (e.shiftKey && e.ctrlKey && this.model.showFooter) {
                        e.preventDefault();
                        this._addFocus(this._statusbar);
                    }
                    break;
                case 65: // Ctrl + A for select all file
                    if (e.ctrlKey) {
                        e.preventDefault();
                        var _items;
                        if (this.model.layout == "grid") {
                            _items = this.gridItems;
                            this._gridObj.clearSelection();
                            this._removeOldSelectionDetails();
                            this._addFocus(this._gridtag.find(".e-gridcontent"));
                            this._gridObj.selectRows(0, _items.length - 1, $(_items[_items.length - 1]).find("td:first"));
                        } else {
                            _items = this.items;
                            if (this.items.hasClass("e-active"))
                                this.items.removeClass("e-active").attr("aria-selected", false);
                            this._removeOldSelectionDetails();
                            this._addFocus(this._tileViewWrapper);
                            for (var i = 0; i < _items.length; i++) {
                                var e = jQuery.Event("click", { keyCode: 91, ctrlKey: true });
                                $(_items[i]).trigger(e);
                            }
                        }
                    }
                    break;
                case 78: // Alt + N for Create newfolder
                    if (e.altKey) {
                        e.preventDefault();
                        this._createNewFolder();
                    }
                    break;
                case 85: // Ctrl + U for upload
                    if (e.ctrlKey) {
                        e.preventDefault();
                        this.element.find(".e-uploadinput").click();
                    }
                    break;
                case 116: // F5 for refresh
                    e.preventDefault();
                    this._currentPath = this._originalPath;
                    this._highlightedNodes = this.model.selectedItems;
                    this._refreshItems(this._treeObj.getSelectedNode(), this._originalPath);
                    break;
            }
        },
        _getFocusedElement: function () {
            var focusedElement = this.element.find(".e-focus");
            return focusedElement = focusedElement ? focusedElement : $(':focus');
        },

        _addFocus: function (target) {
            if (!target.hasClass("e-focus")) {
                this._removeFocus();
                target.focus();
            }
        },
        _removeFocus: function () {
            var target = this._getFocusedElement();
            if (target.length > 0) {
                target.blur();
            }
        },

        _subControlsSetModel: function (prop, value) {
            this._treeObj && this._treeObj.option(prop, value);
            if (this._downloadDialogObj && this._downloadDialogObj.isOpen()) this._downloadDialogObj.option(prop, value);
            if (this._newFolderDialogObj && this._newFolderDialogObj.isOpen()) this._newFolderDialogObj.option(prop, value);
            if (this._renameDialogObj && this._renameDialogObj.isOpen()) this._renameDialogObj.option(prop, value);
            if (this._openDialogObj && this._openDialogObj.isOpen()) this._openDialogObj.option(prop, value);
            if (this._detailsDialogObj && this._detailsDialogObj.isOpen()) this._detailsDialogObj.option(prop, value);
            if (this._alertDialogObj && this._alertDialogObj.isOpen()) this._alertDialogObj.option(prop, value);
            if (this._alertWindowObj && this._alertWindowObj.isOpen()) this._alertWindowObj.option(prop, value);
            this._treeContextMenutag && this._treeMenuObj.option(prop, value);
            this._tileContextMenutag && this._viewMenuObj.option(prop, value);
            this._toolBarObj && this._toolBarObj.option(prop, value);
            this._uploadtag && this._uploadtag.data("ejUploadbox").option(prop, value);
            this._splitObj && this._splitObj.option(prop, value);
        },
        _removeDialog: function (obj) {
            obj.close();
            var isOverLay = obj._overLay;
            isOverLay && isOverLay.remove();
            obj._ejDialog.remove();
        },
        _reSizeHandler: function (args) {
            if (this.model.ajaxAction == "" || this._currentPath == "")
                return;
            if (this.model.layout == "grid") {
                var height = this._splittag.outerHeight() - this._gridtag.find(".e-gridheader").outerHeight();
                this._gridObj && this._gridtag.ejGrid({
                    scrollSettings: { height: this.model.showFooter ? height - this._statusbar.outerHeight() : height, width: this._splittag.find(".e-cont2").width() }
                });
                this._gridtag.find(".e-gridcontent").hasClass("e-scroller") && this._gridtag.find(".e-gridcontent").ejScroller({ buttonSize: 0, scrollerSize: this._scrollerSize });
                this._gridObj && this._gridObj.getContentTable().css("width", this._gridObj.getContent().find('.e-content').width());
            } else {
                this._setThumbImageHeight();
            }
            this._treetag.parent(".e-tree-wrapper").ejScroller({ width: this._splittag.find(".e-cont1").width() });
            this._waitingPopup && this._waitingPopup.refresh();
        },
        _findCommand: function(command, obj) {
            $.each(obj, function(key, val) {
                for(var j = 0, leng = val.length; j < leng; j++) {
                    if(val[j].toLowerCase() == command.toLowerCase()) {
                        command = val[j];
                        return false;
                    }
                }
            });
            return command;
        },
        _getElement: function (node) {
            typeof node == "string" && (node = this._findCommand(node, this.model.tools));
            (typeof node != "object") && (node = (typeof node == "string") ? (this._toolBarObj.itemsContainer.find("li#" + node).length > 0 ? this._toolBarObj.itemsContainer.find("li#" + node) : this._toolBarObj.itemsContainer.find("li#" + this._ExplorerId + node.replace(/ /g, ''))) : this._toolbar.find('li').eq(node));
            node = $(node);
            return $(node[0]);
        },
        _updateToolbar: function () {
            this._renderToolBar();
            this._selectedStates.length >= 2 && this._enableToolbarItem("Back");
            this._updateAddressBar();
            if (this._selectedItems.length > 0) {
                this._isupdate && this._enableToolbarItem("Download");
                this._enableEditingTools();
            }
            if (!ej.isNullOrUndefined(this._fileName) && this._fileName != "")
                this._enableToolbarItem("Paste");
        },
        _unwireEvents: function () {
            this._off($('#' + this._ExplorerId + '_newFolder'), "click", this._createNewFolder);
            this._off($('#' + this._ExplorerId + '_switchGridView'), "click", this._switchView);
            this._off($('#' + this._ExplorerId + '_swithListView'), "click", this._switchView);
            this._off($('#' + this._ExplorerId + '_addressbar'), "keydown", this._searchPath);
            this._off($('#' + this._ExplorerId + '_addressbar'), "blur", this._addressbarFocusout);
            this._off($('#' + this._ExplorerId + '_searchbar'), "keyup", this._validateKeyCode);
            this._off(this.element, "keydown", this._keyDownOnInput);
            this._off(this._gridtag, "click");
            this._off(this._tileViewWrapper, "click");
        },

        _wireResizing: function () {
            this.model.isResponsive ? $(window).bind('resize', $.proxy(this._adjustSize, this)) : $(window).unbind('resize', $.proxy(this._adjustSize, this));
        },
        adjustSize: function () {
            (!this._isWindowResized) && this.model.showToolbar && this.model.isResponsive && this._toolBarObj._reSizeHandler();
            this._isWindowResized = false;
            this._splittag.css('height', (this.element.height() - ((this.model.showToolbar && this._toolBarItems) ? this._toolBarItems.outerHeight() : 0)));
            this._splittag.css('width', (this.element.width()));
            var layoutHeight = this._splittag.outerHeight() - (this.model.showFooter ? this._statusbar.outerHeight() : 0);
            this._tileContent && this._tileContent.parent(".e-tile-wrapper").height(layoutHeight).ejScroller({ height: layoutHeight });
            this._gridtag && this._gridtag.height(layoutHeight);
            this._splitObj && this._splitObj._windowResized();
            this._treeScroll && this._treetag.parent(".e-tree-wrapper").ejScroller("option", { height: this._splittag.height(), width: this._splittag.find(".e-cont1").width() });
            this._reSizeHandler();
        },
        refresh: function () {
            this._refreshItems(this._selectedNode, this._currentPath);
        },
        enableToolbarItem: function (liElement) {
            this._removeResrictedToolItem(liElement);
            liElement = this._getElement(liElement);
            (liElement[0] != null && this._toolBarObj) && this._toolBarObj.enableItem(liElement);
        },
        disableToolbarItem: function (liElement) {
            liElement = this._getElement(liElement);
            if(liElement[0] != null && this._toolBarObj) {
                var operation = liElement.attr("id").replace(this._ExplorerId, "");
                this._restrictedToolbarOptions.push(operation);
                this._toolBarObj.disableItem(liElement);
            }
        },
        removeToolbarItem: function (liElement) {
            this._removeResrictedToolItem(liElement);
            liElement = this._getElement(liElement);
            (liElement[0] != null && this._toolBarObj) && this._toolBarObj.removeItem(liElement);
        },
        _removeResrictedToolItem: function (item) {
            var operation = typeof item == "string" ? item : item.attr("id").replace(this._ExplorerId, "");
            for (var i = 0; i < this._restrictedToolbarOptions.length; i++) {
                if (this._restrictedToolbarOptions[i] == operation) {
                    this._restrictedToolbarOptions.splice(i, 1);
                    break;
                }
            }
        }
    });
    ej.FileExplorer.Locale = {};

    ej.FileExplorer.Locale["en-US"] = {
        Back: "Backward",
        Forward: "Forward",
        Upward:"Upward",
        Refresh: "Refresh",
        Addressbar: "Address bar",
        Upload: "Upload",
        Rename: "Rename",
        Delete: "Delete",
        Download: "Download",
        Error: "Error",
        Cut: "Cut",
        Copy: "Copy",
        Paste: "Paste",
        Details: "Details",
        Searchbar: "Search bar",
        Open: "Open",
        Search: "Search",
        NewFolder: "New Folder",
        Size: "Size",
        RenameAlert: "Please enter new name",
        NewFolderAlert: "Please enter new folder name",        
        ContextMenuOpen: "Open",
        ContextMenuNewFolder: "New Folder",
        ContextMenuDelete: "Delete",
        ContextMenuRename: "Rename",
        ContextMenuUpload: "Upload",
        ContextMenuDownload: "Download",
        ContextMenuCut: "Cut",
        ContextMenuCopy: "Copy",
        ContextMenuPaste: "Paste",
        ContextMenuGetinfo: "Get info",
        ContextMenuRefresh: "Refresh",
        Item: "item",
        Items: "items",
        ErrorOnFolderCreation: "This destination already contains a folder named '{0}'. Do you want to merge this folder content with already existing folder '{0}'?",
        GeneralError: "Please see browser console window for more information",
        ErrorPath: "FileExplorer can't find '{0}'. Check the spelling and try again.",
        ReplaceAlert: "File named '{0}' already exists. Replace old file with new one?",
        DuplicateAlert: "There is already a file with the same name '{0}'. Do you want to create file with duplicate name",
        DuplicateFileCreation: "There is already a file with the same name in this location. Do you want to rename '{0}' to '{1}'?",
        DeleteFolder: " Are you sure you want to delete ",
        DeleteMultipleFolder: "Are you sure you want to delete these {0} items?",
        CancelPasteAction: "The destination folder is a subfolder of source folder.",
        OkButton: "Ok",
        CancelButton: "Cancel",
        YesToAllButton: "Yes to all",
        NoToAllButton: "No to all",
        YesButton: "Yes",
        NoButton: "No",
        Grid: "Grid View",
        Tile: "Tile View",
        Name: "Name",
        Location: "Location",
        Type: "Item type",
        Created: "Created",
        Accessed: "Accessed",
        Modified: "Modified",
        DialogCloseToolTip: "close",
        UploadSettings: {
            buttonText: {
                upload: "Upload",
                browse: "Browse",
                cancel: "Cancel",
                close: "Close"
            },
            dialogText: {
                title: "Upload Box",
                name: "Name",
                size: "Size",
                status: "Status"
            },
            dropAreaText: "Drop files or click to upload",
            filedetail: "The selected file size is too large. Please select a file within the valid size.",
            denyError: "Files with #Extension extensions are not allowed.",
            allowError: "Only files with #Extension extensions are allowed.",
            cancelToolTip: "Cancel",
            removeToolTip: "Remove",
            retryToolTip: "Retry",
            completedToolTip: "Completed",
            failedToolTip: "Failed",
            closeToolTip: "Close"
        }
    };


    ej.FileExplorer.filterType = {
        /**  Supports to search text with startswith  */
        StartsWith: "startswith",
        /**  Supports to search text with contains */
        Contains: "contains",
        /**  Supports to search text with endswith */
        EndsWith: "endswith"        
    };
    ej.FileExplorer.layoutType = {
        Tile: "tile",
        Grid: "grid"
    };
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html Captcha elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejCaptcha", "ej.Captcha", {

        element: null,

        model: null,
        validTags: ["div"],
        _setFirst: false,

        _rootCSS: "e-captcha",

        defaults: {

            enablePattern: true,

            targetInput: "",

            targetButton: "",

            height: 50,

            width: 150,

            characterSet: "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789",

            maximumLength: 8,

            minimumLength: 4,

            enableCaseSensitivity: true,

            enableAutoValidation: false,

            encryptedCode: "",

            customErrorMessage: "InValid Captcha",

            showAudioButton: false,

            showRefreshButton: false,

            enableRTL: false,

            requestMapper: "",

            refreshBegin: "",

            refreshSuccess: "",

            refreshFailure: "",

            refreshComplete: "",

            mapper: "",

            hatchStyle: "BackwardDiagonal"
        },


        dataTypes: {
            targetInput: "string",
            targetButton: "string",
            height: "number",
            width: "number",
            characterSet: "string",
            maximumLength: "number",
            minimumLength: "number",
            enableCaseSensitivity: "boolean",
            enableAutoValidation: "boolean",
            encryptedCode: "string",
            customErrorMessage: "string",
            requestMapper: "string",
            showAudioButton: "boolean",
            showRefreshButton: "boolean",
            enableRTL: "boolean",
            mapper: "string",
            hatchStyle: "enum",
            enablePattern: "boolean"
        },

        _init: function () {
            this._initialize();
            this._wireEvents();
        },

        _initialize: function () {
            $("#" + this._id + "_RefreshButton").ejButton({ size: "normal", showRoundedCorner: true, contentType: "imageonly", prefixIcon: "e-captcha refreshicon", type: "button" });
            $("#" + this._id + "_PlayAudio").ejButton({ size: "normal", showRoundedCorner: true, contentType: "imageonly", prefixIcon: "e-captcha audioicon", type: "button" });
            this._control = $("#" + this._id).get(0);
            if (!this.model.targetInput)
                this.model.targetInput = this._id + "_ValidText";
            this._target = $("#" + this.model.targetInput).get(0);
            this._captchaImage = $("#" + this._id + "_CaptchaImage").get(0);
            this._message = $("#" + this._id + "_CaptchaMessage").get(0);
            this._refreshButton = $("#" + this._id + "_RefreshButton").get(0);
            this._form = $(this._target).parents('form')[0];
            this._audioObject = $("#" + this._id + "_AudioObject").get(0);
            this._audioButton = $("#" + this._id + "_PlayAudio").get(0);
            this._audioPluginObject = null;
            this._audioType = "audio/wav";
            this._submitButton = $("#" + this.model.targetButton).get(0);
            if (!this._submitButton)
                this._submitButton = $("[id$='" + this.model.targetButton + "']").get(0);
            this._hiddenField = $("#" + this._id + "_Hidden").get(0);

            if (this._audioObject) {
                if (!this._isSupportAudio(this._audioType)) {
                    this._appendAudioPlugin();
                }
            }
        },


        _wireEvents: function (val) {
            this._FormSubmitDelegate = $.proxy(this.formSubmit, this);
            this._CaptchaRefreshDelegate = $.proxy(this.refresh, this);
            this._PlayAudioDelegate = $.proxy(this.playAudio, this);
            if (this._audioButton)
                $(this._audioButton).on('click', this._PlayAudioDelegate);
            if (this._submitButton)
                $(this._submitButton).on('click', this._FormSubmitDelegate);
            if (this._refreshButton)
                $(this._refreshButton).on('click', this._CaptchaRefreshDelegate);
        },


        _unwireEvents: function () {
            if (this._audioButton)
                $(this._audioButton).unbind('click', this._PlayAudioDelegate);
            if (this._submitButton)
                $(this._submitButton).unbind('click', this._FormSubmitDelegate);
            if (this._refreshButton)
                $(this._refreshButton).unbind('click', this._CaptchaRefreshDelegate);
        },


        playAudio: function () {
            try {
                if (this._audioPluginObject) {
                    this._audioPluginObject.Play();
                } else {
                    if (this._audioObject && this._audioObject.play) {
                        this._audioObject.play();
                    }
                }
            } catch (e) { }
        },


        _isSupportAudio: function (type) {
            if (!document.createElement("audio").canPlayType) {
                return false;
            } else {
                return document.createElement("audio").canPlayType(type).match(/maybe|probably/i) ? true : false;
            }
        },

        _appendAudioPlugin: function () {
            $(this._audioPluginObject).remove();
            var obj = this._createAudioPluginObject();
            this._audioObject.parentNode.appendChild(obj);
            this._audioPluginObject = $("#" + this._id + "_AudioPlugin").get(0);
        },

        _createAudioPluginObject: function () {
            var a = document.createElement("embed");
            a.setAttribute("id", this._id + "_AudioPlugin");
            a.setAttribute("src", this.model.audioUrl);
            a.setAttribute("name", "AudioPlugin");
            a.setAttribute("enablejavascript", "true");
            a.setAttribute("type", "audio/wav");
            a.setAttribute("autostart", "false");
            a.setAttribute("pluginspage", "http://www.apple.com/quicktime/download/");
            a.style.top = 0;
            a.style.left = 0;
            a.style.width = "0px";
            a.style.height = "0px";
            a.style.position = "absolute";
            return a;
        },


        _onSuccess: function (captchaResult) {
            var jsonCaptcha = captchaResult.d ? JSON.parse(captchaResult.d) : captchaResult;
            try {
                if (this._audioPluginObject) {
                    this._audioPluginObject.Pause();
                } else if (this._audioObject && this._audioObject.pause) {
                    this._audioObject.pause();
                }
            } catch (e) { }

            this._captchaImage.src = jsonCaptcha.NewChallenge;
            if (this._audioObject)
                this._audioObject.src = jsonCaptcha.AudioLink;

            this.model.audioURL = jsonCaptcha.AudioLink;

            if (this._audioPluginObject) {
                this._appendAudioPlugin();
            }

            if (jsonCaptcha.Validation == "False")
                this.model.isValid = false;
            else
                this.model.isValid = true;
            if (this.model.enableAutoValidation)
                this._displayMessages();
            if ($("#" + this._id + "_ValidText"))
                $("#" + this._id + "_ValidText").val("");
            this.model.encryptedCode = jsonCaptcha.Script;
            $(this._hiddenField).val(this.model.encryptedCode);
        },


        _displayMessages: function () {
            if (this.model.isValid) {
                $("#" + this._id + "_CaptchaMessage").html("");
            }
            else {
                $("#" + this._id + "_CaptchaMessage").html(this.model.customErrorMessage);
                $("#" + this._id + "_ValidText").addClass("error");
            }

        },
            //On captcha refresh ajax Success
        _onRefreshSuccess: function (captchaRefresh) {
            var jsonCaptcha = captchaRefresh.d ? JSON.parse(captchaRefresh.d) : captchaRefresh;
            try {
                if (this._audioPluginObject) {
                    this._audioPluginObject.Pause();
                } else if (this._audioObject && this._audioObject.pause) {
                    this._audioObject.pause();
                }
            } catch (e) { }

            this._captchaImage.src = jsonCaptcha.NewChallenge;
            if (this._audioObject) {
                this._audioObject.src = jsonCaptcha.AudioLink;
            }
            this.model.audioUrl = jsonCaptcha.AudioLink;

            if (this._audioPluginObject) {
                this._appendAudioPlugin();
            }
            this.model.encryptedCode = jsonCaptcha.EncryptedText;
            $(this._hiddenField).val(jsonCaptcha.EncryptedText);
        },


            //Hanldes the form submit event
        formSubmit: function (event) {
            var proxy = this;
            var data = {
                Height: this.model.height,
                Width: this.model.width,
                CharacterSet: this.model.characterSet,
                MaximumLength: this.model.maximumLength,
                MinimumLength: this.model.minimumLength,
                CaseSensitive: this.model.enableCaseSensitivity,
                ShowRefreshButton: this.model.showRefreshButton,
                ShowAudioButton: this.model.showAudioButton,
                EncryptedText: this.model.encryptedCode,
                ResponseText: $(this._target).val(),
                Id: this._id,
                HatchStyle: this.model.hatchStyle,
                EnablePattern: this.model.enablePattern,
                RequestType: "CaptchaRequest",
                ActionType: "Validation"
            };
            $.ajax({
                type: 'POST',
                url: this.model.requestMapper ? this.model.requestMapper : this.model.pathName + "/" + this.model.mapper,
                data: JSON.stringify({ captchaModel: data }),
                dataType: "json",
                contentType: "application/json; charset=utf-8",
                async: false,
                success: function (CaptchaResult) {
                    if (CaptchaResult)
                        proxy._onSuccess(CaptchaResult);
                    return;
                },
                error: function (e) {
                    proxy._trigger("refreshFailure");
                    return;
                }
            });
                //Check if invalid and stop form submit
            if (!this.model.isValid && this.model.enableAutoValidation) {
                event.preventDefault();
                event.stopPropagation();
                return false;
            }
        },
            //Hanlder for captcha refresh event
        refresh: function (event) {
            var proxy = this;
            var data = {
                Height: this.model.height,
                Width: this.model.width,
                CharacterSet: this.model.characterSet,
                MaximumLength: this.model.maximumLength,
                MinimumLength: this.model.minimumLength,
                CaseSensitive: this.model.enableCaseSensitivity,
                EncryptedText: this.model.encryptedCode,
                ShowRefreshButton: this.model.showRefreshButton,
                ShowAudioButton: this.model.showAudioButton,
                Id: this._id,
                HatchStyle: this.model.hatchStyle,
                EnablePattern: this.model.enablePattern,
                RequestType: "CaptchaRequest",
                ActionType: "Refresh"
            };
            var handler = this.model.refreshBegin;
            var eventArgs = this.model;
            if (handler) {
                handler(proxy, eventArgs);
                if (eventArgs.getCancel()) {
                    return false;
                }
            }
            $.ajax({
                type: 'POST',
                url: this.model.requestMapper ? this.model.requestMapper : this.model.pathName + "/" + this.model.mapper,
                data: JSON.stringify({ captchaModel: data }),
                dataType: "json",
                contentType: "application/json; charset=utf-8",
                async: false,
                success: function (CaptchaRefresh) {
                    if (CaptchaRefresh) {
                        proxy._onRefreshSuccess(CaptchaRefresh);
                        proxy._trigger("refreshSuccess");
                    }
                    return;
                },
                error: function (errData) {
                    proxy._trigger("refreshFailure")
                    return;
                },
                complete: function (jqXHR, textStatus) {
                    proxy._trigger("refreshComplete")
                    return;
                }
            });
        }
    });
    ej.HatchStyle = {
         None:"none",        
         BackwardDiagonal:"backwardDiagonal",        
         Cross:"cross",
         DarkDownwardDiagonal:"darkDownwardDiagonal",
         DarkHorizontal:"darkHorizontal",
         DarkUpwardDiagonal:"darkUpwardDiagonal",
         DarkVertical:"darkVertical",
         DashedDownwardDiagonal:"dashedDownwardDiagonal",
         DashedHorizontal:"dashedHorizontal",
         DashedUpwardDiagonal:"dashedUpwardDiagonal",
         DashedVertical:"dashedVertical",
         DiagonalBrick:"diagonalBrick",
         DiagonalCross:"diagonalCross",
         Divot:"divot",
         DottedDiamond:"dottedDiamond",       
         DottedGrid:"dottedGrid",
         ForwardDiagonal:"forwardDiagonal",
         Horizontal:"horizontal",
         HorizontalBrick:"horizontalBrick",
         LargeCheckerBoard:"largeCheckerBoard",
         LargeConfetti:"largeConfetti",
         LargeGrid:"largeGrid",
         LightDownwardDiagonal:"lightDownwardDiagonal",
         LightHorizontal:"lightHorizontal",
         LightUpwardDiagonal:"lightUpwardDiagonal",
         LightVertical:"lightVertical",
         Max:"max",
         Min:"min",
         NarrowHorizontal:"narrowHorizontal",
         NarrowVertical:"narrowVertical",
         OutlinedDiamond:"outlinedDiamond",
         Percent90:"percent90",
         Wave:"wave",
         Weave:"weave",
         WideDownwardDiagonal:"wideDownwardDiagonal",
         WideUpwardDiagonal:"wideUpwardDiagonal",
         ZigZag:"zigZag"    
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.common = {
        
        refreshContent: function (refreshTemplate) {
            refreshTemplate && this.refreshTemplate();
            var args = {};
            this._initialRenderings();
            args.requestType = ej.Grid.Actions.Refresh;
            this._processBindings(args);
        },

        
        rowHeightRefresh: function () {
            if (this.model.scrollSettings.frozenColumns > 0 && !ej.isNullOrUndefined(this.model.currentViewData) && this.model.currentViewData.length) {
                var frozenRows = this.getContentTable().get(0).rows;
                var movableRows = this.getContentTable().get(1).rows, height = 0;
                for (var i = 0; i < frozenRows.length; i++) {
                    if ($(frozenRows[i]).css("display") == "none")
                        continue;
                    height = ej.max([frozenRows[i].getClientRects()[0].height, movableRows[i].getClientRects()[0].height]);
                    if (i == this.model.scrollSettings.frozenRows - 1 || i == frozenRows.length - 1)
                        height = height + 1;

                    $(frozenRows[i]).height(height); $(movableRows[i]).height(height);
                    if (this.model.isEdit && $(frozenRows[i]).find("#" + this._id + "EditForm").length)
                        $(frozenRows[i]).find("#" + this._id + "EditForm td").css("height", height); $(movableRows[i]).find("#" + this._id + "EditForm td").css("height", height);
                }
                this._getRowHeights()
                var scroller = this.getScrollObject()._vScrollbar;
                if (scroller.value() != scroller.model.maximum)
                    this.getContent().ejScroller("refresh");

            }
        },
                
        dataSource: function (dataSource, templateRefresh) {
            if (templateRefresh)
                this._templateRefresh = true;
            this._dataSource(dataSource);
            if (dataSource.length > 0)
                this._currentPage(1);
            else
                this._currentPage(0);
            this._refreshDataSource(dataSource);
            var model = this._refreshVirtualPagerInfo();
            this._showPagerInformation(model)
            if (this.model.scrollSettings.allowVirtualScrolling) {
                this._refreshVirtualContent();
                this.getContent().first().ejScroller("refresh");
                if (this.getContent().ejScroller("isHScroll"))
                    this.getContent().ejScroller("scrollX", 0, true);
                if (this.getContent().ejScroller("isVScroll")) {
                    this.getContent().ejScroller("scrollY", 0, true);
                    this.element.find(".e-gridheader").addClass("e-scrollcss");
                }
                else
                    this.element.find(".e-gridheader").removeClass("e-scrollcss");
            }
            this._addLastRow();
        },
        _refreshDataSource: function (dataSource) {
            this._isLocalData = (!(this._dataSource() instanceof ej.DataManager) || (this._dataManager.dataSource.offline || this._isRemoteSaveAdaptor));
            if (dataSource instanceof ej.DataManager)
                this._dataManager = dataSource;
            else
                this._dataManager = ej.DataManager(dataSource);
            this.refreshContent(true);
            if (!ej.isNullOrUndefined(this.getPager())) {
                this.getPager().ejPager("model.currentPage", 1);
                this._refreshGridPager();
            }
        },
        
        hideColumns: function (c) {
            var i, count = 0, args = {}, index, colIndex, newHideCol = []; var htext, field;
            htext = typeof (c) == "string" ? this.getColumnByHeaderText(c) : this.getColumnByHeaderText(c[0]);
            field = typeof (c) == "string" ? this.getColumnByField(c) : this.getColumnByField(c[0]);
            if (this.model.showColumnChooser && !this._columnChooserClick) {
                ej.merge(this._hiddenColumns, c);
                this._hiddenColumns = $.unique(this._hiddenColumns);
            }
            this._columnChooserClick = false;
            var duparr = this._isDuplicate($.merge($.merge([], this._visibleColumns), this._hiddenColumns));
            var hidden = !duparr ? "_hiddenColumns" : "_hiddenColumnsField";
            var visible = !duparr ? "_visibleColumns" : "_visibleColumnsField";
            if (!duparr && field != null)
                c = field.headerText;
            if ($.isArray(c)) {
                for (i = 0; i < c.length; i++) {
                    index = $.inArray(c[i], this[visible]);
                    colIndex = $.inArray(c[i], this._visibleColumns);
                    if (colIndex != -1)
                        this._visibleColumns.splice(colIndex, 1);
                    if (index != -1) {
                        this[hidden].push(c[i]);
                        this[visible].splice(index, 1);
                    }
					else if(index==-1 && visible=="_visibleColumnsField" && $.inArray(c[i],this[hidden])==-1 && ej.isNullOrUndefined(this.getColumnByField(c[i]))){
						this[hidden].push(this.getColumnByHeaderText(c[i]).field) && this["_hiddenColumns"].push(this.getColumnByHeaderText(c[i]).field)
						this[visible].splice($.inArray(this.getColumnByHeaderText(c[i]).field, this[visible]),1) && this["_visibleColumns"].splice($.inArray(c[i], this["_visibleColumns"]),1)
					}
                }
            } else {
                index = $.inArray(c, this[visible]);
                if (index != -1) {
                    this[hidden].push(c);
                    this[visible].splice(index, 1);
                }
				else if(index==-1 && visible=="_visibleColumnsField" && $.inArray(c,this[hidden])==-1 && ej.isNullOrUndefined(this.getColumnByField(c))){
						this[hidden].push(this.getColumnByHeaderText(c).field) && this["_hiddenColumns"].push(this.getColumnByHeaderText(c).field)
						this[visible].splice($.inArray(this.getColumnByHeaderText(c).field, this[visible]),1) && this["_visibleColumns"].splice($.inArray(c, this["_visibleColumns"]),1)
				}
            }
            for (i = 0; i < this.model.columns.length; i++) {
                var com = !duparr ? "headerText" : "field";
                if ($.inArray(ej.isNullOrUndefined(this.model.columns[i][com]) || this.model.columns[i][com] == "" ? this.model.columns[i]["headerText"] : this.model.columns[i][com], this[hidden]) != -1) {
                    this.model.columns[i].visible && newHideCol.push(this.model.columns[i]);
                    this.model.columns[i].visible = false;
                    count++;
                }
                if (this[hidden].length == count)
                    break;
            }
            args.requestType = "refresh";
            this._hideHeaderColumn(this[hidden], duparr);
            if (this.model.allowScrolling && this.model.scrollSettings.frozenColumns > 0) {
                var frozenHide = false;
                for (var i = 0; i < newHideCol.length; i++) {
                    var index = this.model.columns.indexOf(newHideCol[i]);
                    if (index < this.model.scrollSettings.frozenColumns) {
                        this.model.scrollSettings.frozenColumns++;
                        frozenHide = true;
                    }
                }
                if (frozenHide) {
                    var $table = this._renderGridHeader();
                    this.element.find('.e-gridheader').replaceWith($table[0])
                    if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar") this._renderFiltering();
                }
            }
            this.refreshTemplate();
            if (this.model.scrollSettings.allowVirtualScrolling)
                this._virtualShowHide(args);
            this.sendDataRenderingRequest(args);
            this.rowHeightRefresh();
            if (this.model.allowScrolling) {
                if (this.model.scrollSettings.frozenColumns == 0) {
                    var tableWidth = this._calculateWidth();
                    this.getHeaderTable().width(tableWidth);
                    this.getContentTable().width(tableWidth);
                }
                this.getScrollObject().refresh();
            }
            if (this.model.showColumnChooser)
                this._refreshColumnChooserList();
        },
        
        showColumns: function (c) {
            var i, count = 0, args = {}, index, colIndex, column, newVisColumns = []; var htext, field;
            htext = typeof (c) == "string" ? this.getColumnByHeaderText(c) : null;
            field = typeof (c) == "string" ? this.getColumnByField(c) : null;
            if (this.model.showColumnChooser && !this._columnChooserClick) {
                ej.merge(this._visibleColumns, c);
                this._visibleColumns = $.unique(this._visibleColumns);
            }
            this._columnChooserClick = false;
            var duparr = this._isDuplicate($.merge($.merge([], this._visibleColumns), this._hiddenColumns));//updated for
            var hidden = !duparr ? "_hiddenColumns" : "_hiddenColumnsField";
            var visible = !duparr ? "_visibleColumns" : "_visibleColumnsField";
            if (!duparr && field != null)
                c = field.headerText;
            if ($.isArray(c)) {
                for (i = 0; i < c.length; i++) {
                    index = $.inArray(c[i], this[hidden]);
                    colIndex = $.inArray(c[i], this._hiddenColumns);
                    if (colIndex != -1)
                        this._hiddenColumns.splice(colIndex, 1);
                    if (index != -1) {
                        this[hidden].splice(index, 1);
                        this[visible].push(c[i]);
                    }
					else if(index==-1 && hidden=="_hiddenColumnsField" && $.inArray(c[i],this[visible])==-1 && ej.isNullOrUndefined(this.getColumnByField(c[i]))){
						this[visible].push(this.getColumnByHeaderText(c[i]).field) && this["_visibleColumns"].push(c[i])
						this[hidden].splice($.inArray(this.getColumnByHeaderText(c[i]).field, this[hidden]),1) && this["_hiddenColumns"].splice($.inArray(c[i], this["_hiddenColumns"]),1)
					}
                }
            } else {
                index = $.inArray(c, this[hidden]);
                if (index != -1) {
                    this[hidden].splice(index, 1);
                    this[visible].push(c);
                }
				else if(index==-1 && hidden=="_hiddenColumnsField" && $.inArray(c,this[visible])==-1 && ej.isNullOrUndefined(this.getColumnByField(c))){
					this[visible].push(this.getColumnByHeaderText(c).field) && this["_visibleColumns"].push(c)
					this[hidden].splice($.inArray(this.getColumnByHeaderText(c).field, this[hidden]),1) && this["_hiddenColumns"].splice($.inArray(c, this["_hiddenColumns"]),1)
				}
            }
            for (i = 0; i < this.model.columns.length; i++) {
                var com = !duparr ? "headerText" : "field";
                if ($.inArray(ej.isNullOrUndefined(this.model.columns[i][com]) || this.model.columns[i][com] == "" ? this.model.columns[i]["headerText"] : this.model.columns[i][com], this[visible]) != -1) {
                    !this.model.columns[i].visible && newVisColumns.push(this.model.columns[i])
                    this.model.columns[i].visible = true;
                    count++;
                }
                if (this[visible].length == count)
                    break;
            }


            if (this.model.allowScrolling && this.model.scrollSettings.frozenColumns > 0) {
                var frozenHide = false;
                for (var i = 0; i < newVisColumns.length; i++) {
                    var index = this.model.columns.indexOf(newVisColumns[i]);
                    if (index < this.model.scrollSettings.frozenColumns) {
                        this.model.scrollSettings.frozenColumns--;
                        frozenHide = true;
                    }
                }
                if (frozenHide) {
                    var $table = this._renderGridHeader();
                    this.element.find('.e-gridheader').replaceWith($table[0])
                    if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar") this._renderFiltering();
                }
            }

            args.requestType = "refresh";
            this._showHeaderColumn(this[visible], duparr);

            this.refreshTemplate();
            if (this.model.scrollSettings.allowVirtualScrolling)
                this._virtualShowHide(args);
            this.sendDataRenderingRequest(args);
            this.rowHeightRefresh();
            if (this.model.allowScrolling) {
                this.getScrollObject().refresh();
            }
            if (this.model.showColumnChooser)
                this._refreshColumnChooserList();
        },
        _virtualShowHide: function (args) {
            this._currentPage(1);
            this.model.query = this.model.enablePersistence ? new ej.Query() : this.commonQuery.clone();
            this._ensureDataSource(args);
            this._loadedJsonData = [];
            this._prevPage = this._currentPage();
        },
        
        resizeColumns: function (column, width) {
            if (column instanceof Array) {
                for (var i = 0; i < column.length; i++) {
                    var colWidth = width instanceof Array ? width[i] : width;
                    this._setWidthColumnCollection(column[i], colWidth);
                }
            }
            else
                this._setWidthColumnCollection(column, width);
            this.setWidthToColumns();
        },
        _setWidthColumnCollection: function (column, width) {
            var col = this.getColumnByHeaderText(column) || this.getColumnByField(column);
            col.width = width;
            if ($.inArray(col.field, this._disabledResizingColumns) == -1)
                this._columnsWidthCollection[$.inArray(col, this.model.columns)] = width;
        },
        
        refreshTemplate: function () {
            this.addInitTemplate();
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                if (this.model.editSettings.editMode == "normal")
                    this.addEditingTemplate();
                else if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "externalform" ||
                    this.model.editSettings.editMode == "inlineform")
                    this.addDialogEditingTemplate();
                else if (this.model.editSettings.editMode == "dialogtemplate" || this.model.editSettings.editMode == "externalformtemplate" ||
                    this.model.editSettings.editMode == "inlineformtemplate")
                    this.addExternalDialogEditingTemplate();
            }
            if (this.model.allowGrouping) this.addGroupingTemplate();
        },
        set_dropColumn: function (from, to) {
            if (this.model.allowReordering && from != to) {
                this.model.columns.splice(to, 0, this.model.columns.splice(from, 1)[0]);
                var columns = this.model.columns;
                var $header = this.getHeaderTable().find(".e-headercell").not(".e-stackedHeaderCell");
                var $headerCell = $header.find(".e-headercelldiv");
                if (!this.model.scrollSettings.frozenColumns)
                    this.getHeaderTable().find("colgroup").replaceWith(this._getMetaColGroup());
                (this.model.detailsTemplate != null || this.model.childGrid != null) && this.getHeaderTable().find("colgroup").prepend(this._getIndentCol());

                //Remove and Returns name/value pair of element attributes
                var removeAttr = function (ele) {
                    var names = []; if (ele == undefined) return;
                    if (["", undefined].indexOf(ele.value) == -1) {
                        names.push({ name: "value", value: ele.value }); ele.value = "";
                    }
                    for (var e = 0, eAttr = ele.attributes, eLen = eAttr.length; e < eLen; e++) {
                        var regex = /^jQuery[0-9]+$/;
                        !eAttr[e].name.match(regex) && names.push({ name: eAttr[e].name, value: eAttr[e].value });
                    }
                    for (var a = 0, aLen = names.length; a < aLen; a++) {
                        $(ele).removeAttr(names[a].name);
                    }
                    return names;
                };
                //Add attributes to the ele
                var addAttr = function (coll, ele) {
                    if (ele == undefined) return;
                    for (var e = 0, eLen = coll.length; e < eLen; e++) {
                        if (coll[e].name == "value") ele.value = coll[e].value;
                        $(ele).attr(coll[e].name, coll[e].value);
                    }
                };

                if (this.getHeaderTable().find(".e-filterdiv").length > 0)
                    var $filterCell = this.getHeaderTable().find(".e-filterdiv input");
                var $fState = ej.isNullOrUndefined($filterCell);
                var $attributeCollection = { "cellattributes": [], "headerattributes": [], "filtercellattributes": [], "filterThattributes": [] };
                var fromIndex = from < to ? from : to;
                var toIndex = from < to ? to : from;
                
                for (var i = fromIndex, j = 0; i <= toIndex; i++) {
                    var hIndx = (this.model.detailsTemplate != null || this.model.childGrid != null) ? i + 1 : i;
                    $attributeCollection.headerattributes[j] = removeAttr($header[hIndx]);
                    $attributeCollection.cellattributes[j] = removeAttr($headerCell[i]);
                    $attributeCollection.filtercellattributes[j] = !$fState ? removeAttr($filterCell[i]) : [];
                    $attributeCollection.filterThattributes[j] = !$fState ? removeAttr($($filterCell[i]).closest("th")[0]) : [];
                    j++;
                }

                var spliceFrom = from < to ? $attributeCollection.cellattributes.length - 1 : 0;
                var spliceTo = from < to ? 0 : $attributeCollection.cellattributes.length - 1;

                for (var prop in $attributeCollection)
                    $attributeCollection[prop].splice(spliceFrom, 0, $attributeCollection[prop].splice(spliceTo, 1)[0]);

                
                for (var i = fromIndex, j = 0; i <= toIndex; i++) {
                    var indx = (this.model.detailsTemplate != null || this.model.childGrid != null) ? i + 1 : i;
                    addAttr($attributeCollection.headerattributes[j], $header[indx]);
                    addAttr($attributeCollection.cellattributes[j], $headerCell[i]);
                    !$fState && addAttr($attributeCollection.filtercellattributes[j], $filterCell[i]);
                    !$fState && addAttr($attributeCollection.filterThattributes[j], $($filterCell[i]).closest("th")[0]);
                    j++;
                }
                if (this.model.allowFiltering && ["menu", "excel"].indexOf(this.model.filterSettings.filterType) != -1) {
                    var col = this.model.columns;
                    $header.find(".e-filtericon").remove();
                    for (var i = 0; i < col.length; i++) {
                        if (col[i]["allowFiltering"] || ej.isNullOrUndefined(col[i]["allowFiltering"])) {
                            var filterHeader = $header.find(".e-headercelldiv[ej-mappingname=" + col[i].field + "]").closest(".e-headercell")
                            filterHeader.append(ej.buildTag('div.e-filtericon e-icon e-filterset'));
                        }
                    }
                    this._refreshFilterIcon();
                }

                this._columnsWidthCollection.splice(to, 0, this._columnsWidthCollection.splice(from, 1)[0]);
                var headerCell;
                this._fieldColumnNames = this._headerColumnNames = [];
                for (var count = 0; count < columns.length; count++) {
                    this._fieldColumnNames[columns[count].headerText] = columns[count].field;
                    this._headerColumnNames[columns[count].field] = columns[count].headerText;
                    headerCell = $($headerCell[count]);
                    if (!ej.isNullOrUndefined(columns[count].headerTemplateID))
                        headerCell.html($(columns[count]["headerTemplateID"]).html());
                    else
                    headerCell.text(columns[count].headerText);
                    if (this.model.groupSettings.showToggleButton && (ej.isNullOrUndefined(columns[count].allowGrouping) || columns[count].allowGrouping)) {
                        if ($.inArray(columns[count].field, this.model.groupSettings.groupedColumns) != -1)
                            headerCell.append(this._getToggleButton().addClass("e-toggleungroup"));
                        else
                            headerCell.append(this._getToggleButton().addClass("e-togglegroup"));
                    }
                }
                if (this.model.allowGrouping && this.model.allowSorting != true) {
                    for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                        this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
                }
                if (this.model.allowSorting) {
                    for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                        var opacity = 1;
                        var $sCol = this.model.sortSettings.sortedColumns[i];
                        var sortcolumn = this.getsortColumnByField($sCol.field);
                        var index = this.getColumnIndexByField($sCol.field);
                        var sortindex = $.inArray(sortcolumn, this.model.sortSettings.sortedColumns);
                        imageDirection = $sCol.direction != "descending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
                        if (this.model.allowMultiSorting) {
                            for (var j = 1; j <= sortindex; j++) {
                                opacity = opacity + 1;
                            }
                            if ($headerCell.eq(index).css("text-align") == "right") {
                                if(this.model.sortSettings.sortedColumns.length > 1)
                                    $headerCell.eq(index).prepend(this._createSortNumber(opacity, $headerCell.eq(index)).addClass("e-sortnumber"));
                                $headerCell.eq(index).append(this._createSortElement().addClass("e-" + ($sCol.direction || "ascending") + " " + imageDirection));
                            }
                            else {
                                $headerCell.eq(index).append(this._createSortElement().addClass("e-" + ($sCol.direction || "ascending") + " " + imageDirection));
                                if (this.model.sortSettings.sortedColumns.length > 1)
                                    $headerCell.eq(index).append(this._createSortNumber(opacity, $headerCell.eq(index)).addClass("e-sortnumber"));
                            }
                        }
                        else {
                            imageDirection = $sCol.direction != "descending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
                            $headerCell.eq(index).append(this._createSortElement().addClass("e-" + ($sCol.direction || "ascending") + " " + imageDirection));
                        }
                    }
                }
                var args = {};
                args.requestType = ej.Grid.Actions.Reorder;
                this.refreshTemplate();
                this.sendDataRenderingRequest(args);
            }
        },
        
        getPager: function () {
            return this._gridPager;
        },
        
        getFooterTable: function () {
            return this._gridFooterTable;
        },

        setGridFooterTable: function (value) {
            this._gridFooterTable = value;
        },
        
        getFooterContent: function () {
            return this._gridFooterContent;
        },

        setGridFooterContent: function (value) {
            this._gridFooterContent = value;
        },
        
        getScrollObject: function () {
            if (this._scrollObject == null || ej.isNullOrUndefined(this._scrollObject.model))
                this._scrollObject = this.getContent().ejScroller("instance");
            return this._scrollObject;
        },
        setGridPager: function (value) {
            this._gridPager = value;
        },
        
        getRowHeight: function () {
            var rowHeight = -1;
            var trColl = this.getContentTable().find('tr:not(.e-virtualrow)'), index = trColl.length > 2 ? 1 : 0;
            var $trBound = trColl[index].getBoundingClientRect();
            if (trColl.length > 1) {
                if ($trBound && $trBound.height) {
                    rowHeight = $trBound.height;
                } else
                    rowHeight = trColl[index].offsetHeight;
            }
            return rowHeight == -1 ? 32 : rowHeight;
        },
        
        getCurrentIndex: function () {
            return ((this._currentPage() - 1) * (this.model.pageSettings.pageSize));
        },
        
        getColumnByIndex: function (index) {
            if (index < this.model.columns.length)
                return this.model.columns[index];
            return null;
        },
        set_currentPageIndex: function (val) {
            var pageSetting = this.model.pageSettings;
            var recordCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            if (pageSetting.totalPages == null)
                pageSetting.totalPages = Math.ceil(recordCount / pageSetting.pageSize);
            if (val > pageSetting.totalPages || val < 1 || val == this._currentPage())
                return false;
            if (ej.isNullOrUndefined(this._prevPageNo))
                this._prevPageNo = this._currentPage();
            this._currentPage(val);
            if (this._currentPage() != this._prevPageNo) {
                var args = {};
                args.requestType = "paging";
                this.gotoPage(this._currentPage(), args);
                return true;
            }
            else
                return false;
        },
        
        expandCollapse: function ($target) {
            if ($target.prop("tagName") == "DIV" && ($target.parent().hasClass("e-recordplusexpand") || $target.parent().hasClass("e-recordpluscollapse") || $target.parent().hasClass("e-detailrowcollapse") || $target.parent().hasClass("e-detailrowexpand")))
                $target = $target.parent();
            var index = -1, fieldName, fieldvalue, parentGroup, collapsed;
            if (this.model.allowGrouping && (ej.isOnWebForms || this.initialRender)) {
                fieldName = $target.attr("ej-mappingname");
                fieldValue = $target.attr("ej-mappingvalue");
                if ($target.parents(".e-tabletd").length)
                    parentGroup = $target.parents(".e-tabletd").parent("tr").prev("tr").find(".e-recordplusexpand").attr("ej-mappingvalue");
                collapsed = this.model._groupingCollapsed;
                for (var i = 0; i < collapsed.length; i++) {
                    if (collapsed[i].key == fieldName && collapsed[i].value == fieldValue && (collapsed[i].parent == undefined || collapsed[i].parent == parentGroup)) {
                        index = i;
                        break;
                    }
                }
            }
            if (!($target.hasClass("e-recordplusexpand") || $target.hasClass("e-recordpluscollapse") || $target.hasClass("e-detailrowcollapse") || $target.hasClass("e-detailrowexpand")))
                return;
            if ($target.hasClass("e-recordplusexpand") && this.model.groupSettings.groupedColumns.length) {
                var cellIndex = $target.index();
                var $rows = $target.closest('tr').next();
                $rows.hide();
                $target.removeClass("e-recordplusexpand").addClass("e-recordpluscollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
                if (index == -1)
                    this.model._groupingCollapsed.push({ key: fieldName, value: fieldValue, parent: parentGroup })
            } else if ($target.hasClass("e-recordpluscollapse") && this.model.groupSettings.groupedColumns.length) {
                var cellIndex = $target.index();
                var $rows = $target.closest('tr').next();
                var toExpandRows = [];
                var $row = $rows;
                if ($($row[0].cells[cellIndex]).hasClass("e-indentcell")) {
                    if ($row.children(".e-indentcell").length == ($target.parent().children('.e-indentcell').length) + 1) {
                        $row.show();
                        var $expand = $row.children(".e-recordplusexpand");
                        if ($expand != null && $expand.length > 0) {
                            toExpandRows.push($expand);
                        }
                    }
                }
                $target.removeClass("e-recordpluscollapse").addClass("e-recordplusexpand").find("div").removeClass("e-gnextforward").addClass("e-gdiagonalnext");
                for (var i = 0; i < toExpandRows.length; i++) {
                    toExpandRows[i].removeClass("e-recordplusexpand").addClass("e-recordpluscollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
                    this.expandCollapse(toExpandRows[i]);
                }
                if (index != -1)
                    this.model._groupingCollapsed.splice(index, 1);
            } else if ($target.hasClass("e-detailrowexpand")) {
                var cellIndex = $target.index(), proxy = this;
                var rowIndexValue;
                if (this.model.groupSettings.groupedColumns.length > 0)
                    rowIndexValue = this.getIndexByRow($target.closest('tr')) - $target.closest('tr').parents('tr').prevAll('tr').find('td.e-summaryrow').parent().length;
                else
                    rowIndexValue = this.getIndexByRow($target.closest('tr'));
                var $rows = $target.closest('tr').next();
                $rows.hide(0, function () {
                    var args = { masterRow: $target.closest('tr'), detailsRow: $rows, masterData: proxy._currentJsonData[rowIndexValue] };
                    var foreignKeyData = proxy._getForeignKeyData(args.masterData);
                    if (!ej.isNullOrUndefined(foreignKeyData))
                        args.foreignKeyData = foreignKeyData;
                    proxy._trigger("detailsCollapse", args);
                    proxy.model.childGrid != null && proxy.model.allowScrolling && proxy._refreshScroller({ requestType: "refresh" });
                });
                $target.removeClass("e-detailrowexpand").addClass("e-detailrowcollapse").find("div").addClass("e-gnextforward").removeClass("e-gdiagonalnext");
            } else if ($target.hasClass("e-detailrowcollapse")) {
                var cellIndex = $target.index(), proxy = this;
                var rowIndexValue;
                if (this.model.groupSettings.groupedColumns.length > 0)
                    rowIndexValue = this.getIndexByRow($target.closest('tr')) - $target.closest('tr').parents('tr').prevAll('tr').find('td.e-summaryrow').parent().length;
                else
                    rowIndexValue = this.getIndexByRow($target.closest('tr'));
                var detailrow = $target.closest('tr').next();
                if (detailrow.hasClass("e-detailrow"))
                    $rows = detailrow;
                else {
                    var detailtr = ej.buildTag("tr.e-detailrow", "", { 'display': 'none' }, {});
                    var indenttd = ej.buildTag("td.e-detailindentcell");
                    var hideGroupColumnCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                    var detailstd = ej.buildTag("td.e-detailcell", "", {}, { colspan: this._visibleColumns.length - hideGroupColumnCount });
                    var detaildiv = ej.buildTag("div");
                    var count = $($target.closest('tr')).parents('.e-grid').length;
                    detaildiv.attr("id", "child" + count + "_grid" + rowIndexValue);
                    $(detailtr).append(indenttd);
                    $(detailtr).append(detailstd);
                    var rowData = this._currentJsonData[rowIndexValue];
                    if (this.model.detailsTemplate)
                        $(detailtr).append(detailstd.append(this._renderEjTemplate(this.model.detailsTemplate, rowData)));

                    $($target.closest('tr')).after(detailtr);
                    this._trigger("detailsDataBound", { detailsElement: detailtr, data: rowData }); //    $(tbody).append(trchild);
                    if (this.model.childGrid) {
                        var temp = this.model.childGrid.queryString, ftemp = this.model.childGrid.foreignKeyField;
                        this.model.childGrid.query = ej.Query().where(ej.isNullOrUndefined(ftemp) ? temp : ftemp, "equal", rowData[temp], true);
                        this.model.childGrid["parentDetails"] = {
                            parentID: this._id,
                            parentPrimaryKeys: this.getPrimaryKeyFieldNames(),
                            parentKeyField: temp,
                            parentKeyFieldValue: rowData[temp],
                            parentRowData: rowData
                        }
                        $(detailtr).append(detailstd.append(detaildiv));
                    }
                    $rows = detailtr;
                }
                this._showGridLines();
                var toExpandRows = [];
                var $row = $rows;
                if ($($row[0].cells[cellIndex]).hasClass("e-detailindentcell")) {
                    $row.show(0, function () {
                        var args = { masterRow: $target.closest('tr'), detailsRow: $rows, masterData: proxy._currentJsonData[rowIndexValue] };
                        var foreignKeyData = proxy._getForeignKeyData(args.masterData);
                        if (!ej.isNullOrUndefined(foreignKeyData))
                            args.foreignKeyData = foreignKeyData;
                        proxy._trigger("detailsExpand", args);
                        proxy.model.childGrid != null && proxy.model.allowScrolling && proxy._refreshScroller({ requestType: "refresh" });
                    });
                    this.model.childGrid && !ej.isNullOrUndefined(detaildiv) && detaildiv.ejGrid(this.model.childGrid);
                    var $expand = $row.children(".e-detailrowexpand");
                    if ($expand != null && $expand.length > 0) {
                        toExpandRows.push($expand);
                    }
                }
                $target.removeClass("e-detailrowcollapse").addClass("e-detailrowexpand").find("div").addClass("e-gdiagonalnext").removeClass("e-gnextforward");
                for (var i = 0; i < toExpandRows.length; i++) {
                    toExpandRows[i].removeClass("e-detailrowexpand").addClass("e-detailrowcollapse").find("div").removeClass("e-gdiagonalnext").addClass("e-gnextforward");
                    this.expandCollapse(toExpandRows[i]);
                }
            }
            if (this.model.allowScrolling && !ej.isNullOrUndefined(this._scrollObject && this._scrollObject.model) && !$target.closest(".e-hscroll").length)
                this.getScrollObject().refresh();
        },
        _refreshGridPager: function () {
            if (this.getPager() != null) {
                var pagerModel = this.getPager().ejPager("model"), model = {};
                model.currentPage = this._currentPage();
                if (this._filteredRecordsCount == 0 && this.model.currentViewData.length == 0 && (ej.isNullOrUndefined(this._prevPageNo) || this._prevPageNo)) {
                    model.currentPage = 0;
                    this._prevPageNo = pagerModel.currentPage;
                    this.model.pageSettings.currentPage = 0;
                } else if (pagerModel.currentPage == 0 && (ej.isNullOrUndefined(this._prevPageNo) || this._prevPageNo))
                    model.currentPage = this._prevPageNo;
                model.totalRecordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
                if (ej.util.isNullOrUndefined(model.currentPage))
                    model.currentPage = this._currentPage();
                this.getPager().ejPager("option", model).ejPager("refreshPager");
                this.model.pageSettings.totalPages = pagerModel.totalPages;
            }
        },
        _showHeaderColumn: function (showColumns, field) {
            var $head = this.getHeaderTable().find("thead");
            var $headerCell = $head.find("tr").not(".e-stackedHeaderRow").find(".e-headercell");
            var $filterBarCell = $head.find(".e-filterbar").find(".e-filterbarcell");
            var $col = this.getHeaderTable().find("colgroup").find("col"), column;
            for (var i = 0; i < showColumns.length; i++) {
                if (field)
                    column = ej.isNullOrUndefined(this.getColumnByField(showColumns[i])) ? this.getColumnByHeaderText(showColumns[i], ej.isNullOrUndefined(this.getColumnByField(showColumns[i]))) : this.getColumnByField(showColumns[i]);
                else
                    column = this.getColumnByHeaderText(showColumns[i]);
                var index = $.inArray(column, this.model.columns);
                index = (this.model.detailsTemplate != null || this.model.childGrid) ? index + 1 : index;
                var frznCol = this.model.scrollSettings.frozenColumns;
                if (frznCol != 0 && index >= frznCol)
                    var thIndex = $headerCell.eq(index).removeClass("e-hide").index() + frznCol;
                else
                    var thIndex = $headerCell.eq(index).removeClass("e-hide").index();
                $filterBarCell.eq(thIndex).removeClass("e-hide");
				if ($col.length > this.model.columns.length)
                    $col = $col.slice($col.length - this.model.columns.length);
                $col.eq(index).css("display", "");
            }
            if (this.model.showStackedHeader)
                this._refreshStackedHeader();
        },
        _hideHeaderColumn: function (hiddenColumns, field) {
            var $head = this.getHeaderTable().find("thead");
            var $headerCell = $head.find("tr").not(".e-stackedHeaderRow").find(".e-headercell");
            var $filterBarCell = $head.find(".e-filterbar").find(".e-filterbarcell");
            var $col = this.getHeaderTable().find("colgroup").find("col"), column;
            for (var i = 0; i < hiddenColumns.length; i++) {
                if (field)
                    column = ej.isNullOrUndefined(this.getColumnByField(hiddenColumns[i])) ? this.getColumnByHeaderText(hiddenColumns[i], ej.isNullOrUndefined(this.getColumnByField(hiddenColumns[i]))) : this.getColumnByField(hiddenColumns[i]);
                else
                    column = this.getColumnByHeaderText(hiddenColumns[i]);
                var index = $.inArray(column, this.model.columns);
                var dindex = (this.model.detailsTemplate != null || this.model.childGrid) ? index + 1 : index;
                var frznCol = this.model.scrollSettings.frozenColumns;
                if (frznCol != 0 && index >= frznCol)
                    var thIndex = $headerCell.eq(dindex).addClass("e-hide").index() + frznCol;
                else
                    var thIndex = $headerCell.eq(dindex).addClass("e-hide").index();
                $filterBarCell.eq(thIndex).addClass("e-hide");
                if ($col.length > this.model.columns.length)
                    $col = $col.slice($col.length - this.model.columns.length);
                $col.eq(index).css("display", "none");
            }
            if (this.model.showStackedHeader) {
                this._refreshStackedHeader();
                this._colgroupRefresh();
            }
        },
        _refreshStackedHeader: function () {
            var stackedRows = this.model.stackedHeaderRows;
            for (var i = 0; i < stackedRows.length; i++) {
                if (this.model.scrollSettings.frozenColumns != 0) {
                    var frznHeader = $(this.getHeaderContent().find(".e-frozenheaderdiv"));
                    var movHeader = $(this.getHeaderContent().find(".e-movableheader"));
                    var newFrzn = this._createStackedRow(stackedRows[i], true);
                    var newMov = this._createStackedRow(stackedRows[i], false);
                    $(frznHeader.find("tr.e-stackedHeaderRow")[i]).replaceWith(newFrzn);
                    $(movHeader.find("tr.e-stackedHeaderRow")[i]).replaceWith(newMov);
                }
                else {
                    var stackedTR = this._createStackedRow(stackedRows[i], false);
                    if (this.getHeaderTable().find("tr.e-stackedHeaderRow")[i])
                        $(this.getHeaderTable().find("tr.e-stackedHeaderRow")[i]).replaceWith(stackedTR);
                    else
                        stackedTR.insertBefore(this.getHeaderTable().find("tr.e-columnheader:last"));
                }
            }
            var args = {};
            args.requestType = "refresh";
            if (this.model.allowGrouping && this.model.groupSettings.groupedColumns.length > 0) {
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                    this.getHeaderTable().find(".e-stackedHeaderRow").prepend(this._getGroupTopLeftCell());
            }
            this.model.allowScrolling && this._refreshScroller(args);
        },
        _checkSkipAction: function (args) {
            switch (args.requestType) {
                case ej.Grid.Actions.Save:
                case ej.Grid.Actions.Delete:
                    return true;
            }
            return false;
        },
        _unboundTemplateRendering: function (unboundTemplateId) {
            return $("#" + unboundTemplateId).html();
        },
        _processBindings: function (args) {
            this.model.query = this.model.enablePersistence ? new ej.Query() : this.commonQuery.clone();
            if (!this._checkSkipAction(args) && this._trigger("actionBegin", args))
                return true;
            if (this.model.editSettings.editMode == "batch" && args.requestType != "batchsave" && args.requestType != "cancel" && !this._confirmedValue && this._bulkChangesAcquired() && this.model.editSettings.showConfirmDialog) {
                this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("BatchSaveLostChanges"));
                this._confirmDialog.ejDialog("open");
                this._requestArgs = args;
                return false;
            }
            if (!ej.isNullOrUndefined(this.model.dataSource) && args.requestType == "refresh" && this.model.scrollSettings.allowVirtualScrolling) {
                this._currentPage(1);
                this._scrollValue = 0;
                this._loadedJsonData = [];
                this._prevPage = this._currentPage();
            }
            this._ensureDataSource(args);
            if (this.model.scrollSettings.allowVirtualScrolling) {
                if (args.requestType == "virtualscroll") {
                    this._loadedJsonData.push({ pageIndex: this._prevPage, data: this._currentJsonData });
                    this._prevPage = this._currentPage();
                }
                else
                    this._virtualLoadRecords[this._currentPage()] = this.model.currentViewData;
                if (args.requestType == "filtering") {
                    this._loadedJsonData = [];
                    this._prevPage = this._currentPage();
                }
            }
            if (this.model.scrollSettings.allowVirtualScrolling && args.requestType == "filtering") {
                this.refreshContent();
            }
            if (this.model.enableRTL) {
                !this.element.hasClass("e-rtl") && this.element.addClass("e-rtl");
            } else {
                this.element.hasClass("e-rtl") && this.element.removeClass("e-rtl")
            }
            if (args.requestType == ej.Grid.Actions.Delete && this.model.groupSettings.groupedColumns.length == 0) {
                if (this.model.editSettings.showAddNewRow)
                    this.getContentTable().find(".e-addedrow").remove();
                args.tr.remove();
            }
            if ((args.requestType == "save" || args.requestType == "cancel") && this._editForm !== null) {
                $(this._editForm).find("select.e-dropdownlist").ejDropDownList("destroy");
                $(this._editForm).find(".e-datepicker").ejDatePicker("destroy");
                $(this._editForm).find(".e-datetimepicker").ejDateTimePicker("destroy");
                $(this._editForm).find(".e-numerictextbox").ejNumericTextbox("destroy");
            }
            if (this._dataSource() instanceof ej.DataManager && !this._isRemoteSaveAdaptor && args.requestType != ej.Grid.Actions.BeginEdit && args.requestType != ej.Grid.Actions.Cancel && args.requestType != ej.Grid.Actions.Add) {
                if (this.model.scrollSettings.allowVirtualScrolling && this.model.pageSettings.totalPages == this.model.pageSettings.currentPage) {
                    var pageQuery = ej.pvt.filterQueries(this.model.query.queries, "onPage");
                    this.model.query.queries.splice($.inArray(pageQuery[0], this.model.query.queries), 1);
                    this.model.query.page(this._currentPage() - 1, this.model.pageSettings.pageSize);
                    var lastQueryPromise = this._dataSource().executeQuery(this.model.query);
                    this.model.query.queries.splice($.inArray(pageQuery[0], this.model.query.queries), 1);
                    this.model.query.page(this._currentPage(), this.model.pageSettings.pageSize);
                }
                var queryPromise = this._queryPromise = this._dataSource().executeQuery(this.model.query);
                var waitingPopup = this.element.ejWaitingPopup("instance");
                var proxy = this;
                this.element.ejWaitingPopup("show");
                if (proxy._dataSource().ready) {
                    proxy._dataSource().ready.done(function () {
                        proxy._processDataRequest(proxy, args, queryPromise, lastQueryPromise)
                    });
                }
                else {
                    proxy._processDataRequest(proxy, args, queryPromise, lastQueryPromise)
                }
            } else
                this.sendDataRenderingRequest(args);
        },
        _processDataRequest: function (proxy, args, queryPromise, lastQueryPromise) {
            queryPromise.done(ej.proxy(function (e) {
                proxy.element.ejWaitingPopup("hide");
                if (lastQueryPromise && !proxy._previousPageRendered) {
                    proxy._processLastPageData(proxy, args, e.result, lastQueryPromise);
                    if (proxy.model.previousViewData && proxy.model.previousViewData.length != 0) {
                        proxy.model.previousViewData.splice(0, e.result.length);
                        proxy._previousPageLength = e.result.length;
                        proxy._currentPageData = e.result;
                        ej.merge(proxy.model.previousViewData, e.result);
                        proxy.model.currentViewData = proxy.model.previousViewData;
                        proxy._remoteLastPageRendered = true;
                    }
                }
                else if (proxy._remoteLastPageRendered && proxy.model.pageSettings.currentPage == proxy.model.pageSettings.totalPages - 1) {
                    var count = proxy.model.pageSettings.pageSize - proxy._previousPageLength;
                    for (var dupRow = 0; dupRow < count; dupRow++) {
                        var removeEle = proxy.getRows()[proxy.getRows().length - (proxy.model.pageSettings.pageSize - dupRow)];
                        removeEle.remove();
                    }
                    proxy._tempPageRendered = true;
                    proxy.model.currentViewData = e.result;
                }
                else {
                    if (proxy._identityKeys.length && args.action == "add" && this.adaptor instanceof ej.ODataAdaptor)
                        proxy._processIdentityField(e.result, args);
                    if (proxy.model.pageSettings.currentPage == proxy.model.pageSettings.totalPages - 1 && !proxy._remoteLastPageRendered)
                        proxy._previousPageRendered = true;
                    proxy.model.currentViewData = e.result == null ? [] : e.result;
                }
                if (!ej.isNullOrUndefined(e.aggregates))
                    proxy._remoteSummaryData = e.aggregates;
                proxy._processData(e, args);
            }));
            queryPromise.fail(ej.proxy(function (e) {
                proxy.element.ejWaitingPopup("hide");
                args.error = e.error;
                e = [];
                proxy.model.currentViewData = [];
                proxy._processData(e, args);
                proxy._trigger("actionFailure", args);
            }));
        },
        _processIdentityField: function (result, args) {
            var _pKey = this._primaryKeys[0];
            var resultPK = ej.distinct(result, _pKey);
            var curPK = ej.distinct(this.model.currentViewData, _pKey);
            var addPK = $.grep(resultPK, function (value) {
                if ($.inArray(value, curPK) == -1)
                    return true;
                return false;
            });
            args.data = ej.DataManager(result).executeLocal(new ej.Query().where(_pKey, "equal", addPK))[0];
        },
        _processLastPageData: function (proxy, args, currentData, lastQueryPromise) {
            lastQueryPromise.done(ej.proxy(function (e) {
                proxy.model.previousViewData = e.result;
            }));
            lastQueryPromise.fail(ej.proxy(function (e) {
                proxy.element.ejWaitingPopup("hide");
                args.error = e.error;
                e = [];
                proxy.model.previousViewData = [];
                proxy._processData(e, args);
                proxy._trigger("actionFailure", args);
            }));
        },
        _createUnboundElement: function (column) {
            var divElement = document.createElement("div");
            column.headerText = ej.isNullOrUndefined(column.headerText) ? column.field : column.headerText;
            divElement.id = this._id + column.headerText.replace(/\s+/g, "") + "_UnboundTemplate";
            var $div = ej.buildTag("div.e-unboundcelldiv"), commands = column["commands"];
            for (var unbounType = 0; unbounType < commands.length; unbounType++) {
                var $button = ej.buildTag("button.e-" + commands[unbounType].type.replace(/\s+/g, "") + "button", "", {}, { type: "button" });
                $button.val(commands[unbounType].type);
                $div.append($button);
            }
            $("body").append($(divElement).html($div).hide());
            return divElement;
        },
        _refreshUnboundTemplate: function ($target) {
            if (this._isUnboundColumn) {
                var index = 0;
                for (var column = 0; column < this.model.columns.length; column++) {
                    if (this.model.columns[column]["isUnbound"]) {
                        var $unboundDivs = $target.find(".e-unboundcell.e-" + this.model.columns[column]["headerText"].replace(/\s+/g, "")).find(".e-unboundcelldiv");
                        var commands = $.extend(true, [], this.model.columns[column].commands);
                        for (var j = 0; j < commands.length; j++) {
                            var width = ej.isNullOrUndefined(commands[j].buttonOptions.width) ? "52" : commands[j].buttonOptions.width;
                            var height = ej.isNullOrUndefined(commands[j].buttonOptions.height) ? "30" : commands[j].buttonOptions.height;
                            commands[j].buttonOptions.width = ej.isNullOrUndefined(commands[j].buttonOptions.width) ? "52" : commands[j].buttonOptions.width;
                            commands[j].buttonOptions.height = ej.isNullOrUndefined(commands[j].buttonOptions.height) ? "28" : commands[j].buttonOptions.height;
                            commands[j].buttonOptions.cssClass = ej.isNullOrUndefined(commands[j].buttonOptions.cssClass) ? this.model.cssClass : commands[j].buttonOptions.cssClass;
                            commands[j].buttonOptions.enableRTL = this.model.enableRTL;
                            var $buttons = $unboundDivs.find(".e-" + commands[j].type.replace(/\s+/g, "") + "button");
                            if ($target.closest(".e-editcell").length) {
                                if (commands[j].type == "save" || commands[j].type == "cancel")
                                    $buttons.show();
                                else {
                                    $buttons.hasClass("e-deletebutton") && $buttons.hide();
                                    $buttons.hasClass("e-editbutton") && $buttons.hide();
                                }
                            } else {
                                if (commands[j].type == "save" || commands[j].type == "cancel")
                                    $buttons.hide();
                                else {
                                    $buttons.hasClass("e-deletebutton") && $buttons.show();
                                    $buttons.hasClass("e-editbutton") && $buttons.show();
                                }
                            }
                            if ($($buttons[0]).data("ejButton"))
                                $buttons.filter(".e-button").ejButton("destroy");
                            $buttons.ejButton(commands[j].buttonOptions);
                        }
                    } else
                        continue;
                }
            }
        },
        _gridTemplate: function (self, templateId, index) {
            var $column = self.model.columns[index];
            return self._renderEjTemplate("#" + templateId, this.data, this.index, $column);
        },
        _createTemplateElement: function (column) {
            var scriptElement, t;
            if ($(column["templateID"]).length > 0)
                scriptElement = $(column["templateID"])[0];
            else {
                scriptElement = document.createElement("script"), t;
                scriptElement.id = (this._id + column.headerText + $.inArray(column, this.model.columns) + "_Template").split(" ").join("");
                scriptElement.type = "text/x-template";
                if (column.template.slice(0, 1) !== "#")
                    scriptElement.text = column.template;
                else
                    t = $(column["template"]);
                if (t) {
                    scriptElement.text = t.html();
                    scriptElement.type = t.attr("type") || scriptElement.type;
                }
                $("body").append(scriptElement);
            }
            return scriptElement;
        },
        _renderGridPager: function () {
            var $div = $(document.createElement('div'));
            var pagerModel = {};
            this.model.pageSettings.click = this._gPagerClickHandler;
            this.model.pageSettings.totalRecordsCount = this._gridRecordsCount;
            this.model.pageSettings.enableRTL = this.model.enableRTL;
            this.model.pageSettings.locale = this.model.locale;
            this.model.pageSettings.enableQueryString = this.model.pageSettings.enableQueryString;
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar")
                pagerModel.enableExternalMessage = this.model.filterSettings.showFilterBarStatus;
            $.extend(pagerModel, this.model.pageSettings);
            pagerModel.currentPage = this._currentPage();
            pagerModel.masterObject = this;
            this.setGridPager($div);
            $div.ejPager(pagerModel);
            $div.ejPager("refreshPager");
            pagerModel = $div.ejPager("model");
            this.model.pageSettings.totalPages = pagerModel.totalPages;
            if (this._currentPage() !== pagerModel.currentPage)
                this._currentPage(pagerModel.currentPage);
            if (this.model.pageSettings.enableTemplates) {
                if (!this.model.pageSettings.showDefaults) {
                    $div.children().remove();
                }
                var $tempHtml;
                if ($(this.model.pageSettings.template)[0].type == 'text/x-jsrender')
                    $tmpHtml = $(this.model.pageSettings.template).hide().html()
                else
                    $tempHtml = $(this.model.pageSettings.template)[0];
                var $divPagerContainer = $div.find('.e-pagercontainer');
                var $customDiv = ej.buildTag('div.e-pagercontainer ');
                $customDiv.append($tmpHtml);
                $div.append($customDiv)
            }
            return $div;
        },

        _renderContext: function (e) {
            var menuitems = this.model.contextMenuSettings.contextMenuItems;
            var item, item2, i = 0;
            var ul = ej.buildTag('ul', "", {}, { id: this._id + '_Context' });
            for (i; i < menuitems.length; i++) {
                item = menuitems[i];
                item2 = this._items(item, ul);
                ul.append(item2);
            }
            var customitems = this.model.contextMenuSettings.customContextMenuItems;
            var custom, custom2, j = 0;
            for (j; j < customitems.length; j++) {
                custom = customitems[j];
                custom2 = this._items(custom, ul);
                ul.append(custom2);
            }
            this.element.append(ul);
            var parentControl = this.element.parents("div.e-grid");
            var native = ej.Menu.prototype._showContextMenu;
            ej.Menu.prototype._showContextMenu = function (locationX, locationY, target, evt) {
                $(this.model.contextMenuTarget).hasClass("e-grid") && (++locationX, ++locationY)
                native.apply(this, [locationX, locationY, target, evt]);
            };
            $(ul).ejMenu({
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                contextMenuTarget: "#" + this._id,
                click: $.proxy(this._clickevent, this),
                width: "auto",
                beforeOpen: $.proxy(this._menu, this)
            });
            if (parentControl.length) {
                var parentElement = $("#" + parentControl[0].id + '_Context');
                var target = parentElement.ejMenu("model.excludeTarget");
                parentElement.ejMenu({ excludeTarget: !ej.isNullOrUndefined(target) ? target.concat(",#" + this._id) : "#" + this._id });
            }
            this._conmenu = ul.data("ejMenu");
        },
        _clickevent: function (sender) {
            var args = sender.events.text;
            var tr = $(this._contexttarget.parentNode);
            var c = $(this._contexttarget);
            if (c.hasClass("e-filterset"))
                c = c.siblings();
            else if (c.hasClass("e-icon") || c.hasClass("e-number"))
                c = c.parent();
            var columnName = c.attr("ej-mappingname");
            if (this._trigger("contextClick", sender))
                return;
            switch (args) {
                case this._getLocalizedLabels("AddRecord"): this._startAdd();
                    break;
                case this._getLocalizedLabels("EditRecord"):
                    if (this.model.editSettings.editMode == "batch") {
                        var colindex = $(this._contexttarget.parentNode).find(".e-rowcell").index(c), index = this._excludeDetailRows().index(tr), fieldName = this.model.columns[colindex]["field"];
                        this.editCell(index, fieldName);
                    }
                    else
                        this.startEdit(tr);
                    break;
                case this._getLocalizedLabels("DeleteRecord"):
                    if (this.model.editSettings.showDeleteConfirmDialog)
                        this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("ConfirmDelete")).end().ejDialog("open");
                    else
                    this._deleteRow(tr);
                    break;
                case this._getLocalizedLabels("Save"): this.endEdit();
                    break;
                case this._getLocalizedLabels("Cancel"): this.cancelEdit();
                    break;
                case this._getLocalizedLabels("SortInDescendingOrder"): var columnSortDirection = ej.sortOrder.Descending;
                    this.sortColumn(columnName, columnSortDirection);
                    break;
                case this._getLocalizedLabels("SortInAscendingOrder"): var columnSortDirection = ej.sortOrder.Ascending;
                    this.sortColumn(columnName, columnSortDirection);
                    break;
                case this._getLocalizedLabels("Grouping"): this.groupColumn(columnName);
                    break;
                case this._getLocalizedLabels("Ungrouping"): this.ungroupColumn(columnName);
                    break;
                case this._getLocalizedLabels("NextPage"):
                case this._getLocalizedLabels("PreviousPage"):
                case this._getLocalizedLabels("LastPage"):
                case this._getLocalizedLabels("FirstPage"): this._page(sender);
                    break;
            }

        },

        _menu: function (sender) {
            var context = this._conmenu.element;
            this._contexttarget = sender.target;
            var targetelement = $(sender.target), element, value;
            var td = $(this._contexttarget);
            if (td.hasClass("e-rowcell") && this.model.allowSelection)
                if (!this.model.isEdit)
                    this.selectRows(this.getIndexByRow(td.parent()), null, td);
            if ((targetelement.hasClass("e-ascending") || targetelement.hasClass("e-descending")) && !targetelement.parent().hasClass("e-headercelldiv"))
                return false;
            if (targetelement.hasClass("e-filtericon") || targetelement.hasClass("e-headercelldiv"))
                element = sender.target.parentNode.getAttribute("aria-sort");
            else
                element = targetelement.parent().parent().attr("aria-sort");
            var target = sender.target.className;
            var sorting = $(context);
            if (targetelement.hasClass("e-filterset"))
                value = targetelement.siblings().attr("ej-mappingname");
            else if (targetelement.hasClass("e-icon") || targetelement.hasClass("e-number"))
                value = targetelement.parent().attr("ej-mappingname");
            else
                value = targetelement.attr("ej-mappingname");
            switch (target) {
                case "e-pager e-js e-rtl":
                case "e-pager e-js": var a = $(context.find(".e-page"));
                    context.find(".e-head").css("display", "none");
                    context.find(".e-content").css("display", "none");
                    context.find(".e-savcan").css("display", "none");
                    a.css("display", "block");
                    break;
                case "e-filtericon e-icon e-filterset":
                case "e-icon e-ascending e-rarrowup-2x":
                case "e-icon e-descending e-rarrowdown-2x":
                case "e-number e-sortnumber":
                case "e-headercelldiv":
                case "e-headercelldiv e-droppable e-js":
                case "e-headercell e-default e-draggable e-js":
                    var a = $(context.find(".e-head"));
                    context.find(".e-page").css("display", "none");
                    context.find(".e-content").css("display", "none");
                    context.find(".e-savcan").css("display", "none");
                    a.css("display", "block");
                    break;
                case "e-rowcell e-selectionbackground e-active":
                case "e-rowcell":
                case "e-rowcell e-lastrowcell":
                case "e-rowcell e-lastrowcell e-selectionbackground e-active":
                case "e-rowcell e-boolrowcell e-selectionbackground e-active":
                case "e-rowcell e-templatecell e-selectionbackground e-active":
                    var a = $(context.find(".e-content"));
                    context.find(".e-head").css("display", "none");
                    context.find(".e-page").css("display", "none");
                    context.find(".e-savcan").css("display", "none");
                    a.css("display", "block");
                    break;
                default: var a = $(context);
                    return false;
            }
            switch (element) {
                case "ascending": sorting.find(".ascending").parent().css("display", "none");
                    break;
                case "descending": sorting.find(".descending").parent().css("display", "none");
                    break;
            }
            if (this.model.isEdit) {
                var a = $(context.find(".e-savcan"));
                context.find(".e-head").css("display", "none");
                context.find(".e-content").css("display", "none");
                context.find(".e-page").css("display", "none");
                a.css("display", "block");
            }
            if (this.model.pageSettings.currentPage == 1) {
                sorting.find(".previous").parent().css("display", "none");
                sorting.find(".first").parent().css("display", "none");
            }
            else if (this.model.pageSettings.currentPage == this.model.pageSettings.totalPages) {
                sorting.find(".last").parent().css("display", "none");
                sorting.find(".nextpage").parent().css("display", "none");
            }
            if (this.model.groupSettings.groupedColumns.indexOf(value) != -1)
                a.find(".group").parent().css("display", "none");
            else if (this.model.groupSettings.groupedColumns.indexOf(value) == -1)
                a.find(".ungroup").parent().css("display", "none");
            if (this.model.contextOpen)
                this._trigger("contextOpen", sender);
        },

        _items: function (item) {
            if (item == "")
                return false;
            if (item.indexOf("Record") != -1) {
                var li = ej.buildTag('li', "", {}, { "class": "e-content" });
                li.css("display", "none");
            }
            else if (item.indexOf("Page") != -1) {
                var li = ej.buildTag('li', "", {}, { "class": "e-page" });
                if (item.indexOf("Next") != -1)
                    var div = ej.buildTag('div', "", {}, { "class": "nextpage" });
                else if (item.indexOf("Previous") != -1)
                    var div = ej.buildTag('div', "", {}, { "class": "previous" });
                else if (item.indexOf("Last") != -1)
                    var div = ej.buildTag('div', "", {}, { "class": "last" });
                else if (item.indexOf("First") != -1)
                    var div = ej.buildTag('div', "", {}, { "class": "first" });
                li.css("display", "none");
            }
            else if (item == "Save" || item == "Cancel") {
                var li = ej.buildTag('li', "", {}, { "class": "e-savcan" });
                li.css("display", "none");
            }
            else if (item.indexOf("Order") != -1 || item == "Grouping" || item == "Ungrouping") {
                var li = ej.buildTag('li', "", {}, { "class": "e-head" });
                if (item.indexOf("Ascending") != -1)
                    var div = ej.buildTag('div', "", {}, { "class": "ascending" });
                else if (item.indexOf("Descending") != -1)
                    var div = ej.buildTag('div', "", {}, { "class": "descending" });
                else if (item == "Grouping")
                    var div = ej.buildTag('div', "", {}, { "class": "group" });
                else if (item == "Ungrouping")
                    var div = ej.buildTag('div', "", {}, { "class": "ungroup" });
                li.css("display", "none");
            }
            else {
                var li = ej.buildTag('li', "", {}, { "class": "e-customitem" });
                li.css("display", "block");
            }
            li.append(div);
            var a = document.createElement("a");
            a.innerHTML = !ej.isNullOrUndefined(this._getLocalizedLabels(item.replace(/\s+/g, ''))) ? this._getLocalizedLabels(item.replace(/\s+/g, '')) : item;
            li.append(a);
            return li;
        },

        _page: function (send) {
            if (send.events.text == "Next Page") {
                var b = this.model.pageSettings.currentPage;
                ++b;
                this.gotoPage(b);
            }
            else if (send.events.text == "Previous Page") {
                var b = this.model.pageSettings.currentPage;
                if (b > 1) {
                    --b;
                    this.gotoPage(b);
                }
                else
                    this.gotoPage(b);
            }
            else if (send.events.text == "Last Page") {
                var b = this.model.pageSettings.totalPages
                this.gotoPage(b);
            }
            else
                this.gotoPage(1);


        },

        
        gotoPage: function (pageIndex) {
            if (!this.model.allowPaging && (!this.model.allowScrolling && !this.model.scrollSettings.allowVirtualScrolling))
                return;
            var args = {}, returnValue;
            args.previousPage = this._currentPage();
            this._currentPage(pageIndex);
            args.endIndex = ((this._currentPage() * this.model.pageSettings.pageSize) > this._gridRecordsCount) ? (this._gridRecordsCount) : (this._currentPage() * this.model.pageSettings.pageSize);
            args.startIndex = (this._currentPage() * this.model.pageSettings.pageSize) - this.model.pageSettings.pageSize;
            args.currentPage = pageIndex;
            if (this.model.allowPaging) {
                //this.model.pageSettings.currentPage = pageIndex;
                //this. getPager().ejPager("refreshPager");
                args.requestType = ej.Grid.Actions.Paging;
            }
            if (this.model.scrollSettings.allowVirtualScrolling && this.model.allowScrolling) {
                this._isVirtualRecordsLoaded = false;
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
                args.requestType = ej.Grid.Actions.VirtualScroll;
            }
            returnValue = this._processBindings(args);
            if (returnValue)
                this._currentPage(args.previousPage);
            this._primaryKeyValues = [];
        },
        _gPagerClickHandler: function (sender) {
            if (this._prevPageNo == sender.currentPage)
                return;
            this.model.masterObject.gotoPage(sender.currentPage);
            return false;
        },
        _processData: function (e, args) {
            if (e.count == 0 && this.model.currentViewData.length)
                this._gridRecordsCount = e.result.length;
            else
                this._gridRecordsCount = e.count;
            if (this.getPager() != null)
                this.model.pageSettings.totalRecordsCount = this._gridRecordsCount;
            if ((args.requestType == ej.Grid.Actions.Filtering || ej.Grid.Actions.Save || (this.model.filterSettings.filteredColumns.length > 0 && args.requestType == ej.Grid.Actions.Refresh)))
                this._filteredRecordsCount = e.count;
            this.sendDataRenderingRequest(args);
        },

        _frozenCell: function (rowIndex, cellIndex) {
            var currentIndex = cellIndex, frozenDiv = 0, row = this.getRowByIndex(rowIndex), cell;
            if (cellIndex >= this.model.scrollSettings.frozenColumns) {
                frozenDiv = 1;
                currentIndex = currentIndex - this.model.scrollSettings.frozenColumns;
            }
            cell = $(row.eq(frozenDiv).find(".e-rowcell:eq(" + currentIndex + ")"));
            return cell;
        },
        _frozenColumnSelection: function (gridRows, columnIndex, endIndex) {
            var currentIndex = columnIndex, frozenDiv = 0;
            if (endIndex) {
                for (var i = columnIndex; i < endIndex; i++) {
                    currentIndex = i;
                    if (i >= this.model.scrollSettings.frozenColumns) {
                        frozenDiv = 1;
                        currentIndex = i - this.model.scrollSettings.frozenColumns;
                    }
                    for (var j = 0; j < gridRows[frozenDiv].length; j++) {
                        $(gridRows[frozenDiv][j].cells[currentIndex]).addClass("e-columnselection");
                    }
                    $(this.getHeaderTable().find("th.e-headercell")[i]).addClass("e-columnselection");
                    this.selectedColumnIndexes.push(i);
                }
            }
            else {
                if (columnIndex >= this.model.scrollSettings.frozenColumns) {
                    frozenDiv = 1;
                    currentIndex = columnIndex - this.model.scrollSettings.frozenColumns;
                }
                for (var i = 0; i < gridRows[frozenDiv].length; i++) {
                    $(gridRows[frozenDiv][i].cells[currentIndex]).addClass("e-columnselection");
                }
            }

        },
        _renderGridFooter: function () {
            if (this.model.summaryRows.length > 0) {
                var _$gridFooter = ej.buildTag("div.e-gridfooter");
                var $table = ej.buildTag("table.e-gridsummary", "", {}, { cellspacing: "0.25px" });
                this.setGridFooterContent(_$gridFooter);
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var $frozenFooterDiv = ej.buildTag("div.e-frozenfooterdiv"), $movableFooter = ej.buildTag("div.e-movablefooter")
                        , $tableClone = $table.clone(), $movableFooterDiv = ej.buildTag("div.e-movablefooterdiv");
                    $movableFooter.append($movableFooterDiv);
                    $table.append(this.getHeaderTable().first().find('colgroup').clone());
                    $tableClone.append(this.getHeaderTable().last().find('colgroup').clone());
                    $frozenFooterDiv.append($table);
                    $movableFooterDiv.append($tableClone);
                    this.setGridFooterTable($table.add($tableClone));
                    this._createSummaryRows(this.getFooterTable());
                    _$gridFooter.append($frozenFooterDiv.add($movableFooter));
                    _$gridFooter.find(".e-frozenfooterdiv").outerWidth(this.getHeaderContent().find(".e-frozenheaderdiv").width())
                          .end().find(".e-movablefooterdiv").outerWidth(this.getContent().find(".e-movablecontentdiv").width());
                }
                else {
                    $table.append(this.getHeaderTable().find('colgroup').clone());
                    this.setGridFooterTable($table);
                    this._createSummaryRows(this.getFooterTable());
                    _$gridFooter.append($table);
                }
                return _$gridFooter;
            } else
                throw "summary row collection is missing";
        },
        _setSummaryAggregate: function (queryManager) {
            var rows = this.model.summaryRows, scolumns, sCol = [];
            for (var row = 0, rlen = rows.length; row < rlen; row++) {
                scolumns = rows[row].summaryColumns;
                for (var col = 0, clen = scolumns.length; col < clen; col++) {
                    queryManager.aggregate(scolumns[col].summaryType, scolumns[col].dataMember);
                }
            }
        },
        _createSummaryRows: function (table, summaryData, aggregates, item, showGroup) {
            var col = table.find("col");
            if (table.find("tbody").length > 0)
                table.find("tbody").remove();
            var $tBody = ej.buildTag('tbody'), proxy = this, $tBodyClone = $tBody.clone();
            var summaryCol = this.model.summaryRows;
            if (!ej.isNullOrUndefined(summaryData) && this._isCaptionSummary)
                summaryCol = this._captionSummary(showGroup);
            $.each(summaryCol, function (indx, row) {
                if (row.showTotalSummary === false && ej.isNullOrUndefined(summaryData)) return true;
                var $tr = ej.buildTag('tr.e-gridSummaryRows');
                if (ej.isNullOrUndefined(item && item.level)) {
                    for (var i = 0; i < proxy.model.groupSettings.groupedColumns.length; i++) {
                        $tr.prepend(ej.buildTag('td').addClass("e-indentcell"));
                    }
                }
                var gc = showGroup ? " e-gcsummary" : "";
                if (proxy.model.detailsTemplate != null || proxy.model.childGrid != null) {
                    if (proxy.model.groupSettings.groupedColumns.length != 0)
                        $tr.children("td.e-indentcell").last().after("<td class='e-summaryrow" + gc + "'></td>");
                    else
                        $tr.prepend("<td class='e-summaryrow'></td>");
                }
                var $cells = proxy.getHeaderTable().find('td').clone().addClass("e-summaryrow" + gc + ""), count = 0;
                var index = 0;
                if (!ej.isNullOrUndefined(row.titleColumn)) {
                    var index = proxy.getColumnIndexByField(row.titleColumn);
                    if (index >= proxy.model.columns.length)
                        index = proxy.getColumnIndexByHeaderText(row.titleColumn);
                }
                $cells = proxy._assignTitleColumn(index, row.title, $cells, count);
                proxy._hideSummaryColumn($cells, col);
                if (proxy.model.scrollSettings.frozenColumns > 0) {
                    var $trClone = $tr.clone();
                    $tBody.append($tr.append($cells.slice(0, proxy.model.scrollSettings.frozenColumns)));
                    $tBodyClone.append($trClone.append($cells.slice(proxy.model.scrollSettings.frozenColumns)));
                }
                else {
                    if ((!proxy._isCaptionSummary || showGroup) && !ej.isNullOrUndefined(item && item.level)) {
                        var level = proxy.model.groupSettings.groupedColumns.length - item.level + 1, tableClone = table.clone().addClass("e-groupsummary"), captionData = {};
                        captionData["data"] = { items: item };
                        $tr.prepend("<td class='e-summaryrow" + gc + "' colspan=" + proxy._colSpanAdjust(null, null, captionData) + " style = 'padding:0;' ></td>");
                        $($tr[0].cells).filter(".e-summaryrow").html(tableClone.append(ej.buildTag("tr", $cells)));
                        var len = tableClone.find("col").length - (proxy.model.columns.length + level);
                        for (var i = 0; i < len; i++) {
                            tableClone.find("col").first().remove();
                        }
                        for (var i = 0; i < level; i++) {
                            $(tableClone[0].rows).prepend("<td class='e-indentcell'></td>");
                            $(tableClone.find("col")[i]).addClass("e-summary");
                        }
                        $tBody.append($tr);
                    }
                    else
                        $tBody.append($tr.append($cells));
                }
                $.each(row.summaryColumns, function (cindx, col) {
                    var value;
                    if (col.summaryType != "custom")
                        value = aggregates ? aggregates[col.dataMember + " - " + col.summaryType] : proxy._remoteSummaryData[col.dataMember + " - " + col.summaryType];
                    else
                        value = proxy.getSummaryValues(col, summaryData);
                    prefix = col.prefix ? col.prefix : "";
                    var index = proxy.getColumnIndexByField(col.displayColumn), suffix = col.suffix ? col.suffix : "";
                    if ($($cells[index]).html() != "" && prefix == "")
                        prefix = $($cells[index]).html();
                    $($cells[index]).html(prefix + (col.format ? proxy.formatting(col.format, value, proxy.model.locale) : value) + suffix).css("text-align", proxy.model.columns[index].textAlign);
                });
            });
            if (this.model.scrollSettings.frozenColumns > 0) {
                table.first().append($tBody);
                table.last().append($tBodyClone);
            }
            else
                table.append($tBody);
        },
        _assignTitleColumn: function (index, title, $cells, count) {
            for (var i = index; i < this.model.columns.length; i++) {
                var colindex = this.model.columns[i];
                if (count == 0 && colindex.visible != false) {
                    $cells.eq(i).html(title);
                    break;
                }
            }
            return $cells;
        },
        getSummaryValues: function (summaryCol, summaryData) {
            var $value, jsonData;
            if (!ej.isNullOrUndefined(summaryData))
                jsonData = summaryData;
            else if (this.model.filterSettings.filteredColumns.length > 0)
                jsonData = this._filteredRecords;
            else
                jsonData = this._dataSource();

            var dbMgr;
            if (jsonData instanceof ej.DataManager) {
                dbMgr = jsonData;
                jsonData = jsonData.dataSource.json;
            } else
                dbMgr = ej.DataManager(jsonData);

            switch (summaryCol.summaryType) {
                case ej.Grid.SummaryType.Maximum:
                    var obj = ej.max(jsonData, summaryCol.dataMember);
                    $value = obj[summaryCol.dataMember];
                    break;
                case ej.Grid.SummaryType.Minimum:
                    var obj = ej.min(jsonData, summaryCol.dataMember);
                    $value = obj[summaryCol.dataMember];
                    break;
                case ej.Grid.SummaryType.Average:
                    $value = ej.avg(jsonData, summaryCol.dataMember);
                    break;
                case ej.Grid.SummaryType.Sum:
                    $value = ej.sum(jsonData, summaryCol.dataMember);
                    break;
                case ej.Grid.SummaryType.Count:
                    $value = jsonData.length;
                    break;
                case ej.Grid.SummaryType.TrueCount:
                    var predicate = ej.Predicate(summaryCol.dataMember, "equal", true);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.Grid.SummaryType.FalseCount:
                    var predicate = ej.Predicate(summaryCol.dataMember, "equal", false);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.Grid.SummaryType.Custom:
                    var fn = summaryCol.customSummaryValue;
                    if (fn) {
                        if (typeof fn === "string")
                            fn = ej.util.getObject(fn, window);
                        if ($.isFunction(fn))
                            $value = fn.call(this, summaryCol, jsonData);
                    }
                    break;
            }
            return $value;
        },
        _hideCaptionSummaryColumn: function () {
            var headerColumn = this.getHeaderTable().find('.e-headercelldiv[ej-mappingname]').first();
            var captionTd = this.getContentTable().find('.e-groupcaption').clone();
            var groupCaptionParent = this.getContentTable().find('.e-groupcaption').parent();
            var colLength = this.model.columns.length - 1;
            if (this._isCaptionSummary) {
                this.getContentTable().find('.e-summaryrow:not(.e-gcsummary)').remove();
                this.getFooterTable().find("tbody td").slice(-colLength).removeClass("e-groupcaptionsummary").addClass("e-summaryrow");
                if (this.getFooterTable() != null) {
                    this.getContentTable().find('.e-recordplusexpand').parent().children('.e-indentcell').remove();
                }
                if (!this.model.groupSettings.showGroupedColumn && this.getContentTable().find(".e-groupcaptionsummary").not(".e-hide").length) {
                    var sumColumn = +this.getContentTable().find(".e-recordtable:first").parents("tbody:first").find(".e-groupcaption").attr("colspan");
                    if (this._hiddenColumnsField.length == this.model.columns.length - 1 && headerColumn.parent().hasClass("e-hide") || !sumColumn) {
                        for (i = 0; i < captionTd.length; i++) {
                            groupCaptionParent.eq(i).children().not('.e-hide,.e-recordplusexpand').filter('td.e-groupcaptionsummary:first').addClass("e-hide");
                            var caption = groupCaptionParent.eq(i).find(".e-groupcaption");
                            var colspan = parseInt(caption.attr("colspan"));
                            caption.attr("colspan", ++colspan)
                        }
                    }
                }
            }
            this.getContentTable().find('.e-recordtable').find('.e-indentcell').remove();
        },
        _hideSummaryColumn: function (td, col) {
            if (col.length > this.model.columns.length)
                col = col.slice(col.length - this.model.columns.length);
            if (!this.model.groupSettings.showGroupedColumn && this.model.showSummary) {
                for (i = 0; i < this.model.columns.length; i++) {
                    for (j = 0; j < this.model.groupSettings.groupedColumns.length || j < this._hiddenColumnsField.length; j++) {
                        var headerColumn = this.getHeaderTable().find('.e-headercelldiv:not(.e-emptyCell)');
                        if (!headerColumn.eq(i).is(':visible')) {
                            col.eq(i).css("display", "none");
                            $(td[i]).addClass("e-hide");
                            break;
                        }
                        else {
                            if (col.eq(i).css("display") == "none")
                                col.eq(i).css("display", "");
                        }
                    }
                }
            }
            else {
                for (i = 0; i < this.model.columns.length; i++) {
                    if (!this.model.columns[i]["visible"]) {
                        col.eq(i).css("display", "none");
                        $(td[i]).addClass("e-hide");
                    }
                    else {
                        if (col.eq(i).css("display") == "none")
                            col.eq(i).css("display", "");
                    }
                }
            }
        },

        _initScrolling: function () {
            var frozen = [], unfrozen = [], hideColumns = 0;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if (this.model.columns[columnCount].visible === false && columnCount < this.model.scrollSettings.frozenColumns)
                    hideColumns++;
                if (this.model.columns[columnCount]["isFrozen"] === true)
                    frozen.push(this.model.columns[columnCount]);
                else
                    unfrozen.push(this.model.columns[columnCount]);
            }
            this.model.scrollSettings.frozenColumns += hideColumns;
            if (frozen.length > 0) {
                var freeze = this.model.scrollSettings.frozenColumns;
                this.model.columns = $.merge($.merge([], frozen), unfrozen);
                this.model.scrollSettings.frozenColumns = frozen.length;
                if (frozen.length != freeze && freeze != 0)
                    this.model.scrollSettings.frozenColumns = freeze;
            }
            if ((this.model.scrollSettings.frozenColumns > 0 || this.model.scrollSettings.frozenRows > 0) && (this.model.allowGrouping || this.model.rowTemplate != null || this.model.detailsTemplate != null || this.model.childGrid != null || this.model.scrollSettings.allowVirtualScrolling || this.model.editSettings.editMode == "batch")) {
                this._renderAlertDialog();
                this._alertDialog.find(".e-content").text(this._getLocalizedLabels("FrozenNotSupportedException"));
                this._alertDialog.ejDialog("open");
                return;
            }
            if (this.model.scrollSettings.allowVirtualScrolling && this.model.allowScrolling) {
                this.model.pageSettings.pageSize = this.model.pageSettings.pageSize == 12 ? Math.round(this.model.scrollSettings.height / 32) + 1 : this.model.pageSettings.pageSize;
                this.model.pageSettings.totalPages = Math.ceil(this._gridRecordsCount / this.model.pageSettings.pageSize);
            }
            if (this.model.width || this.model.height) {
                this.model.allowScrolling = true;
                if (this.model.width) this.model.scrollSettings.width = this.model.width;
                if (this.model.height) this.model.scrollSettings.height = this.model.height;
            }
            this._originalScrollWidth = ej.isNullOrUndefined(this.model.scrollSettings.previousStateWidth) ? this.model.scrollSettings.width : this.model.scrollSettings.previousStateWidth;
        },
        _checkScrollActions: function (requestType) {
            if ((!this.model.scrollSettings.allowVirtualScrolling && requestType == ej.Grid.Actions.Sorting) || requestType == ej.Grid.Actions.Grouping || requestType == ej.Grid.Actions.Ungrouping || requestType == ej.Grid.Actions.Add || requestType == ej.Grid.Actions.Cancel
                || requestType == ej.Grid.Actions.Save || requestType == ej.Grid.Actions.Delete || requestType == ej.Grid.Actions.Filtering || requestType == ej.Grid.Actions.Paging || requestType == ej.Grid.Actions.Refresh || requestType == ej.Grid.Actions.Reorder)
                return true;
            return false;
        },
        _refreshScroller: function (args) {
            var gridContent = this.getContent().first(), direction, browserDetails = this.getBrowserDetails(), temp;
            if (ej.isNullOrUndefined(gridContent.data("ejScroller")))
                return;
            if (this.model.scrollSettings.frozenColumns > 0) {
                direction = this.model.enableRTL ? "margin-right" : "margin-left";
                gridContent.find(".e-movablecontent").css(direction, browserDetails.browser === "safari" ? "auto" : gridContent.find(".e-frozencontentdiv").width() + "px");
                this.getHeaderContent().find(".e-movableheader").removeAttr("style").css(direction, browserDetails.browser === "safari" ? "auto" : this.getHeaderContent().find(".e-frozenheaderdiv").width() + "px");
                this.refreshScrollerEvent();
                gridContent.find(".e-movablecontent").scrollLeft(this.getHeaderContent().find(".e-movableheader").scrollLeft());
                if (!ej.isNullOrUndefined(this.getScrollObject()._vScrollbar) && this.getScrollObject()._vScrollbar.value() > this.getScrollObject()._vScrollbar.model.maximum)
                    temp = this.getScrollObject()._vScrollbar.model.maximum;
            }
            if (this.model.scrollSettings.frozenRows > 0) {
                this._initFrozenRows();
                var temp = this.getScrollObject().model.scrollTop;
                if (!ej.isNullOrUndefined(this.getScrollObject()._vScrollbar) && temp > this.getScrollObject()._vScrollbar.model.maximum)
                    temp = this.getScrollObject()._vScrollbar.model.maximum;
                if (args.requestType == "cancel" || args.requestType == "save")
                    temp = temp - this._editFormHeight;
                if (args.requestType == ej.Grid.Actions.Add)
                    this.getScrollObject().scrollY(0, true);
                if (!ej.isNullOrUndefined(this.getScrollObject()._vScrollbar) && !ej.isNullOrUndefined(this.getScrollObject()._vScrollbar._scrollData))
                    this.getScrollObject()._vScrollbar._scrollData.skipChange = true;
            }
            if (args.requestType == "beginedit") {
                var temp = this.getScrollObject().model.scrollTop;
                this.getScrollObject().scrollY(0, true);
            }
            if (!ej.isNullOrUndefined(this.model.dataSource) && args.requestType == "refresh" && this.model.scrollSettings.allowVirtualScrolling) {
                this._refreshVirtualContent(1);
                this.getScrollObject().scrollY(0);
            }
            gridContent.ejScroller("refresh");
            gridContent.ejScroller("model.enableRTL", this.model.enableRTL);
            if (gridContent.ejScroller("isVScroll") && !this.getScrollObject().model.autoHide) {
                this.getHeaderContent().addClass("e-scrollcss");
                !this.getHeaderContent().find(".e-headercontent").hasClass("e-hscrollcss") && this.getHeaderContent().find(".e-headercontent").addClass("e-hscrollcss");
            }
            else
                this._showHideScroller();
            this._getRowHeights();
            if (temp && args.requestType != ej.Grid.Actions.Add) {
                this._currentTopFrozenRow = 0;
                if (temp > this.getScrollObject()._vScrollbar.model.maximum)
                    temp = this.getScrollObject()._vScrollbar.model.maximum;
                this.getScrollObject()._vScrollbar.scroll(temp);
            }
            if (args.requestType == "virtualscroll") {
                var top = this.getScrollObject().model.scrollTop + this.getScrollObject().model.height - (this.getScrollObject().model.height * .3);
                this.getScrollObject().scrollY(top, true);
            }
        },
        _renderScroller: function () {
            if (!this.model.scrollSettings)
                this.model.scrollSettings = {};
            if (this.model.enablePersistence && ej.isNullOrUndefined(this.model.scrollSettings.previousStateWidth) && this.model.isResponsive)
                this.model.scrollSettings.previousStateWidth = this.model.scrollSettings.width;
            if (typeof (this._originalScrollWidth) == "string") {
                this.element.css("width", "auto");
                var width = this.element.width();
                if (this.model.scrollSettings.width == "auto" || this._originalScrollWidth == "auto")
                    this._originalScrollWidth = "100%";
                this.model.scrollSettings.width = width * (parseFloat(this._originalScrollWidth) / 100)
            }

            if (typeof (this.model.scrollSettings.height) == "string") {
                var height = this.element.height();
                if (this.model.scrollSettings.height == "auto")
                    this.model.scrollSettings.height = "100%";
                this.model.scrollSettings.height = height * (parseFloat(this.model.scrollSettings.height) / 100)
            }

            if ((this.model.scrollSettings.width || this.model.width) && !this._mediaQuery)
                this.element.width(this.model.scrollSettings.width || this.model.width);

            var $content = this.getContent().attr("tabindex", "0"), staticWidth, direction, gridRows = this.getRows();

            if (this.model.scrollSettings.frozenColumns > 0) {
                scrollWidth = this.getContent().find(".e-frozencontentdiv").width() + 20;
                if (scrollWidth > this.model.scrollSettings.width) {
                    this.getContent().remove();
                    this.getHeaderTable().eq(1).remove();
                    this._alertDialog.find(".e-content").text(this._getLocalizedLabels("FrozenColumnsViewAlert"));
                    this._alertDialog.ejDialog("open");
                    return;
                }
                staticWidth = this.getContent().find(".e-frozencontentdiv").width();
                direction = this.model.enableRTL ? "margin-right" : "margin-left";
                this.getContent().find(".e-movablecontent").css(direction, staticWidth + "px");
                this.getHeaderContent().find(".e-movableheader").css(direction, staticWidth + "px");
                this.model.scrollSettings["targetPane"] = ".e-movablecontent";
            }
            this._initFrozenRows();
            if (this.model.scrollSettings.autoHide)
                this.model.scrollSettings["show"] = $.proxy(this._showHideScroller, this);
            var proxy = this;
            if (!this.model.scrollSettings.frozenRows)
                this.model.scrollSettings["scroll"] = function (e) {
				if (!ej.isNullOrUndefined(e.scrollData) && e.scrollData.handler == "e-hhandle"){
					if (proxy.model.allowFiltering && (proxy.model.filterSettings.filterType == "menu" || proxy._isExcelFilter)) 				
						!proxy._isExcelFilter ? proxy._closeFilterDlg() : proxy._excelFilter.closeXFDialog();	
					proxy._checkScroller(e, this);
				}
			};
            $content.ejScroller(this.model.scrollSettings);
            if (this.model.rowTemplate != null && (this.getBrowserDetails().browser == "msie" || this.getBrowserDetails().browser == "safari"))
                this.getScrollObject().refresh();            
            if (this.model.scrollSettings.frozenColumns > 0 && this.model.scrollSettings.frozenRows == 0 && this.getScrollObject()._vScrollbar && this.getScrollObject()._hScrollbar)
                this.getScrollObject()._vScrollbar._scrollData.skipChange = this.getScrollObject()._hScrollbar._scrollData.skipChange = true;
            if (!this.model.scrollSettings.autoHide)
                this._showHideScroller();
            if (this.getBrowserDetails().browser == "safari" && this.model.scrollSettings.frozenColumns > 0)
                this.getHeaderContent().find(".e-movableheader").add(this.getContent().find(".e-movablecontent")).css(direction, "auto");
            this.refreshScrollerEvent();            
            if (proxy.model.scrollSettings.allowVirtualScrolling) {
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
                $content.ejScroller({
                    scroll: function (e) {
                        if (!ej.isNullOrUndefined(e.scrollData) && e.scrollData.handler == "e-hhandle" && proxy.model.allowFiltering && (proxy.model.filterSettings.filterType == "menu" || proxy._isExcelFilter))
                            !proxy._isExcelFilter ? proxy._closeFilterDlg() : proxy._excelFilter.closeXFDialog();
                        e["reachedEnd"] = this.content()[0].scrollHeight - e.scrollTop == this.content()[0].clientHeight;
                        if ((e.source == "button" || e.source == "key" || e.source == "wheel") && proxy.model != null)
                            proxy._virtualScroll(e);
                        if (e.source == "wheel" && e.scrollTop != proxy._scrollValue)
                            e.scrollTop = proxy._scrollValue;
						proxy._checkScroller(e, this);
                    },
                    thumbEnd: function (e) {
                        if (!$(e.originalEvent.target).hasClass("e-rowcell"))
                            e["reachedEnd"] = this.content()[0].scrollHeight - e.scrollData.sTop == this.content()[0].clientHeight;
                        if (e.scrollData.handler == "e-hhandle")
                            return;
                        if (proxy.model != null)
                            proxy._virtualScroll(e);
                    }
                });
            }
        },
		_checkScroller: function(e, scrollObj){
			var scrollLeft = e.scrollLeft > 0 ? e.scrollLeft : Math.abs(e.scrollLeft);
			if(e.source == "thumb" && (scrollObj.content()[0].scrollWidth - scrollLeft == scrollObj.content()[0].clientWidth || scrollLeft == 0)){
				if(this.model.enableRTL){
					var hLeft = scrollLeft == 0 ? e.scrollData.scrollable: 0;
					e.scrollData.sTop = e.model.scrollLeft = hLeft;
					scrollObj.content().scrollLeft(hLeft);	
				}
				scrollObj.refresh();
			}
		},
        _showHideScroller: function () {
            if (this.getContent().ejScroller("isVScroll")) {
                this.getHeaderContent().find("div").first().addClass("e-headercontent");
                !this.model.scrollSettings.autoHide && this.getHeaderContent().addClass("e-scrollcss")
            } else
                this.element.find(".e-gridheader").removeClass("e-scrollcss");
            if (this.getBrowserDetails().browser != "msie" && this.model.scrollSettings.frozenColumns == 0 && !this._mediaQuery) {
                if (!this.element.find(".e-gridheader").hasClass("e-scrollcss") && this.model.filterSettings.filteredColumns) {
                    this.getHeaderTable().removeAttr('style');
                    this.getContentTable().removeAttr('style');
                }
                else {
                    this.getHeaderContent().find("div table").first().width(this.getContentTable().width());
                    this.getContent().find("div table").first().width(this.getContentTable().width());
                    this.getHeaderTable().width(this.getContentTable().width());
                }
            }
            if (this.getBrowserDetails().browser == "msie" && this.model.scrollSettings.frozenColumns == 0)
                !this.getContent().ejScroller("isVScroll") ? this.getContent().width(this.getHeaderContent().width()) : this.getContent().width(this.getHeaderContent().width() + 18);
            this._isHscrollcss();
        },
        _isHscrollcss: function () {
            var scroller = this.getContent().data("ejScroller"), css = scroller && (scroller.isHScroll() || scroller.isVScroll()) ? "addClass" : "removeClass";
            this.getHeaderContent().find(".e-headercontent")[css]("e-hscrollcss")
        },
        _initFrozenRows: function () {
            var gridRows = this.getRows();
            if (!this.model.currentViewData || this.model.currentViewData.length == 0)
                return;
            if (this.model.scrollSettings.frozenRows > 0 && gridRows != null) {
                this.model.scrollSettings["scroll"] = $.proxy(this._scroll, this);
                this.getContent().find(".e-frozeny").removeClass("e-frozeny")
                    .end().find(".e-frozenrow").removeClass("e-frozenrow");
                if (!ej.isNullOrUndefined(gridRows[0][this.model.scrollSettings.frozenRows - 1]) && !ej.isNullOrUndefined(gridRows[1][this.model.scrollSettings.frozenRows - 1]) && this.model.scrollSettings.frozenColumns > 0)
                    $(gridRows[0][this.model.scrollSettings.frozenRows - 1].cells).add(gridRows[1][this.model.scrollSettings.frozenRows - 1].cells).addClass("e-frozeny").parent().addClass("e-frozenrow");
                else
                    $(gridRows[this.model.scrollSettings.frozenRows - 1].cells).addClass("e-frozeny").parent().addClass("e-frozenrow");
                this.model.scrollSettings.height = this._rowHeightCollection[Math.floor(this.model.scrollSettings.height / this._rowHeightCollection[1])] + 18;
            }
            else
                delete this.model.scrollSettings["scroll"];
        },
        refreshScrollerEvent: function () {
            var proxy = this;
            this.getContent().find(".e-content,.e-movablecontent").scroll(ej.proxy(function (e) {
                if (this.model.scrollSettings.targetPane)
                    this.getHeaderContent().find(".e-movableheader").scrollLeft($(e.currentTarget).scrollLeft());
                else
                    this.getHeaderContent().find("div").first().scrollLeft($(e.currentTarget).scrollLeft());
                if (this.model.scrollSettings.frozenRows > 0 && this.model.editSettings.editMode.indexOf("inlineform") != -1 && this.model.isEdit) {
                    var scrollTop = e.target.scrollTop;
                    this.getContent().find(".e-content").scrollTop(0);
                    this.getScrollObject().scrollY(this.getScrollObject().model.scrollTop + scrollTop, true);
                }
            }, this));
            this.element.find(".e-gridheader").find(".e-headercontent,.e-movableheader").scroll(ej.proxy(function (e) {
                var $currentTarget = $(e.currentTarget);
                if (this.model.scrollSettings.targetPane) {
                    this.getContent().find(".e-movablecontent").scrollLeft($currentTarget.scrollLeft());
                    this.model.showSummary && this.getFooterContent().find(".e-movablefooter").scrollLeft($currentTarget.scrollLeft());;
                }
                else {
                    this.model.showSummary && this.getFooterContent().scrollLeft($currentTarget.scrollLeft());
                    this.getContent().find(".e-content").first().scrollLeft($currentTarget.scrollLeft());
                }
            }, this));
        },
		clearFiltering: function(field){
			if(field)
				this._clearFilter(field);					
			else{
				var fltrCols = this.model.filterSettings.filteredColumns, i=0;				
				while(i < fltrCols.length){
					this._clearFilter(fltrCols[i].field);					
				}
				if(this.model.filterSettings.filterType == "menu" || this.model.filterSettings.filterType == "excel")
					this.getHeaderTable().find(".e-filtericon").removeClass("e-filteredicon e-filternone");
			}
		},
		_clearFilter: function(field){			
			switch(this.model.filterSettings.filterType){
				case ej.Grid.FilterType.FilterBar:
					if($.inArray(this.getColumnByField(field), this.filterColumnCollection) != -1){
						this.getHeaderTable().find("#" + field + "_filterBarcell").val("");
						this._currentFilterbarValue = "";
						var index = $.inArray(field, this.filterColumnCollection);	
						this._currentFilterColumn = this.getColumnByField(field);
						this._showFilterMsg();
					}
					break;
				case ej.Grid.FilterType.Menu:
					var id = "#" + this._id + "_" + this._$colType + "Dlg";
					if (this._$colType == "boolean")
						$(id).find('.e-value input[type!="hidden"]').ejCheckBox("model.checked", false);
					else
						if (this._$colType == "number")
							$(id).find('.e-numerictextbox').ejNumericTextbox("model.value", "");
						else
							$(id).find(".e-value input").val("");
					if (this._$colType == "date" || this._$colType == "datetime") {
						for (var j = 0; j < this.model.filterSettings.filteredColumns.length; j++) {
							if (this.model.filterSettings.filteredColumns[j].isComplex) {
								var preobject = this.model.filterSettings.filteredColumns[j].predicates;
								if (this.model.filterSettings.filteredColumns.length == 1) {
									this.model.filterSettings.filteredColumns = preobject;
								}
								else {
									this.model.filterSettings.filteredColumns[j] = preobject[0];
									this.model.filterSettings.filteredColumns.push(preobject[1]);
								}
							}
						}
					}
					this._$curFieldName = field;						
					break;
				case ej.Grid.FilterType.Excel:
					delete this._excelFilter._predicates[0][field];						
					this._excelFilter.closeXFDialog();
					this._$curFieldName = field;
					break;
			}
			this.filterColumn(field, "", "", "or");							
		},
		clearSearching: function(){
			this.element.find(".e-gridtoolbar #" + this._id + "_search").val("");
			this.search("");
			$.extend(this.model.searchSettings, this.defaults.searchSettings);
		},
        _renderByFrozenDesign: function () {
            var $div = $(document.createElement('div')), col = this._getMetaColGroup().find("col"), colgroups = {};
            colgroups["colgroup1"] = $div.append(ej.buildTag("colgroup").append(col.splice(0, this.model.scrollSettings.frozenColumns))).html();
            colgroups["colgroup2"] = $div.html(ej.buildTag("colgroup").append(col)).html();
            this.getContent().find("div").first().get(0).innerHTML = $.render[this._id + "_FrozenTemplate"]({ datas: this.model.currentViewData }, colgroups);
            this.setGridContentTable(this.getContent().find(".e-table").attr("role", "grid"));
        },
        addFrozenTemplate: function () {
            var template = "<div class='e-frozencontentdiv'>"
            + "<table cellspacing='0.25px' class='e-table'>{{:~colgroup1}}<tbody>"
            + "{{for datas tmpl='" + this._id + "_JSONFrozenTemplate'/}}"
            + "</tbody></table></div>"
            + "<div class='e-movablecontent'><div class='e-movablecontentdiv'><table cellspacing='0.25px' class='e-table'>{{:~colgroup2}}<tbody>"
            + "{{for datas tmpl='" + this._id + "_JSONTemplate'/}}"
            + "</tbody></table></div></div>", templates = {};
            templates[this._id + "_FrozenTemplate"] = template;
            $.templates(templates);
        },
        _getTopRow: function (offsetTop) {
            var currentTopRow = this.model.scrollSettings.frozenRows, i = 0;
            if (offsetTop > 10) {
                for (var i = 0; i < this._rowHeightCollection.length; i++) {
                    if (this._rowHeightCollection[i] > offsetTop) {
                        currentTopRow = this.model.scrollSettings.frozenRows + i - 1;
                        break;
                    }
                }
            }
            return { imaginaryIndex: currentTopRow, actualIndex: i };
        },
        _showHideRow: function (from, to, action, scrollPosition) {
            var rows = this.getRows();
            if (this.model.scrollSettings.frozenColumns > 0)
                $(rows[0]).slice(from, to).add($(rows[1]).slice(from, to).toArray())[action]();
            else
                $(rows).slice(from, to)[action]();
            this._currentTopFrozenRow = action == "show" ? from : to;
            this.getScrollObject()._changevHandlerPosition(scrollPosition);
        },
        _scroll: function (args) {
            if (args.scrollData != null && args.scrollData.dimension != "width") {
                args.cancel = true;
                var rows = this.getRows(), indexes = this._getTopRow(args.scrollTop), currentTopRow = indexes.imaginaryIndex, frozenRows;
                if (currentTopRow > this._currentTopFrozenRow)
                    this._showHideRow(this.model.scrollSettings.frozenRows, currentTopRow, "hide", args.scrollTop);
                else if (currentTopRow < this._currentTopFrozenRow)
                    this._showHideRow(currentTopRow, this._currentTopFrozenRow + 1, "show", args.scrollTop);
                var movableContent = this.getContentTable().last().find("tr");
                var border = (parseInt(movableContent.last().find("td:first").css("border-top-width")) * 2) + 1;
                if (args.scrollTop == this.getScrollObject()._vScrollbar.model.maximum && ((movableContent.last()[0].offsetTop + movableContent.last().height() - border) > this.element.find(".e-content").height())) {
                    var totalHeight = movableContent.last().prev()[0].offsetTop + movableContent.last().prev().height();
                    var count = 1;
                    for (var i = (movableContent.length - 2) ; totalHeight - border > this.element.find(".e-content").height() ; i++) {
                        totalHeight = movableContent[i].offsetTop + movableContent.eq(i).height();
                        count++;
                        break;
                    }
                    this._showHideRow(this.model.scrollSettings.frozenRows, currentTopRow + count, "hide", args.scrollTop);
                }
                args.model.scrollTop = args.scrollTop;
            }
            else {
                if (!ej.isNullOrUndefined(this.getScrollObject()._vScrollbar) && !ej.isNullOrUndefined(this.getScrollObject()._vScrollbar._scrollData))
                    this.getScrollObject()._vScrollbar._scrollData.skipChange = true;
            }
        },
        _renderAlertDialog: function () {
            var $contentDiv = ej.buildTag('div.e-content', this._getLocalizedLabels("frozenColumnsMessage"))
                , $buttons = ej.buildTag('span.e-buttons', "<input type='button' id=" + this._id + 'ConfirmDialogOK' + " value='" + this._getLocalizedLabels("OkButton") + "'/>");
            this._alertDialog = ej.buildTag('div#' + this._id + 'AlertDialog');
            this._alertDialog.append($contentDiv).append($buttons);
            this.element.append(this._alertDialog);
            $buttons.find("input").ejButton({
                cssClass: this.model.cssClass,
                showRoundedCorner: true,
                size: "mini",
                click: $.proxy(function (args) {
                    this._alertDialog.ejDialog("close");
                }, this)
            });
            this._renderFDialog(this._id + 'AlertDialog');
            this._alertDialog.ejDialog({ width: "auto", enableModal: true });
        },
        _renderFDialog: function (id) {
            $("#" + id).ejDialog({ showOnInit: false, "enableRTL": this.model.enableRTL, "cssClass": this.model.cssClass, "showHeader": false, width: 260, enableResize: false, allowKeyboardNavigation: false, content: "#" + this._id });
        },
        _virtualScroll: function (e) {
            if (e != null) {
                var flag = 0;
                var recordCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
                var pageInfo = this.model.pageSettings;
                var tbody = this.getContentTable()[0].tBodies[0];
                var virtualRows = $(tbody).find('tr.e-virtualrow');
                pageInfo.totalPages = Math.ceil(recordCount / pageInfo.pageSize);
                if (e.scrollTop !== undefined)
                    e.model.scrollTop = e.scrollTop;
                if (e.reachedEnd != undefined) e.model.reachedEnd = e.reachedEnd;
                var currentPageNo = this._calculateCurrenPage(virtualRows, this.getContentTable(), e.model);
                if (currentPageNo > pageInfo.totalPages)
                    currentPageNo = pageInfo.totalPages;
                if (pageInfo.currentPage != currentPageNo && $.inArray((currentPageNo - 1) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                    this._isVirtualRecordsLoaded = false;
                }
                if (!this._isVirtualRecordsLoaded) {
                    if ($.inArray((currentPageNo - 1) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                        if (this.model.scrollSettings.virtualScrollMode == "continuous" && !e.reachedEnd)
                            return
                        if (currentPageNo == pageInfo.totalPages && $.inArray((currentPageNo - 2) * pageInfo.pageSize, this.virtualLoadedPages) == -1) {
                            flag++;
                            this.set_currentPageIndex(currentPageNo);
                        }
                        if (flag == 1) this._lastRow = true;
                        this.set_currentPageIndex(currentPageNo);
                    }
                    pageInfo.currentPage = currentPageNo;
                }
                else
                    pageInfo.currentPage = currentPageNo;
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
            }
        },

        _createPagerStatusBar: function () {
            var $statusBar = this.element.find(".e-pagerstatusbar");
            if ($statusBar.length)
                $statusBar.remove();
            var $pagermsgDiv = ej.buildTag('div.e-pagermsgdiv');
            this.$pagerStatusBarDiv = ej.buildTag('div.e-pagerstatusbar').append($pagermsgDiv);
            this.$pagerStatusBarDiv.appendTo(this.element);
            this.$pagerStatusBarDiv.css("display", "none");
        },
        _refreshVirtualContent: function (currentPage) {
            var rowHeight = this.getRowHeight();
            var recordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            if (currentPage != null) {
                this._currentPage(currentPage);
                var model = this._refreshVirtualPagerInfo();
                this._showPagerInformation(model);
            }
            var currentIndex = this.getCurrentIndex();
            var tbody = this.getContentTable()[0].tBodies[0];
            if (currentIndex > 0) {
                var virtualTRTop = document.createElement("tr");
                $(virtualTRTop).addClass("e-virtualrow").css("height", rowHeight * currentIndex).prependTo(tbody);
            } if (currentIndex + this.model.pageSettings.pageSize <= recordsCount) {
                var virtualTRBottom = document.createElement("tr");
                var virtualHeight = this.model.scrollSettings.virtualScrollMode == "normal" ? rowHeight * (recordsCount - (currentIndex + this.model.pageSettings.pageSize)) : 1;
                $(virtualTRBottom).addClass("e-virtualrow").css("height", virtualHeight).appendTo($(tbody));
            }
            this.virtualLoadedPages = new Array();
            this.orderedVirtualLoadedPage = [];
            this.virtualLoadedPages.push(currentIndex >= 0 ? currentIndex : 0);
            this.orderedVirtualLoadedPage.push(currentIndex >= 0 ? currentIndex : 0);
            var focusTR = $(tbody).find('tr:not(.e-virtualrow)').attr('name', currentIndex >= 0 ? currentIndex : 0)[0];
            if (focusTR && focusTR.previousSibling && ($(focusTR.previousSibling).hasClass("e-virtualrow") || focusTR.previousSibling.offsetTop > (currentIndex * this.getContent().height()))) {
                this.getContent().children("div").first().scrollTop(this.getContent().find(".content").scrollTop() - (this.getContent().find(".content").scrollTop() - focusTR.offsetTop));
                this._isVirtualRecordsLoaded = true;
            }
        },

        _calculateCurrenPage: function (virtualRows, target, args) {
            var pageSize = this.model.pageSettings.pageSize;
            var currentPage, tempCPage, diff, proxy = this, trEle, isLast, viewTr = [], cur, oTop, len;
            var rowHeight = this.getRowHeight();

            currentPage = (args.scrollTop + this.model.scrollSettings.height) / rowHeight / pageSize;            
            if (args.scrollTop < this._scrollValue && this.virtualLoadedPages.indexOf(Math.ceil(currentPage - 1) * pageSize) !== -1)
                currentPage = Math.floor(currentPage);
            else
                currentPage = Math.ceil(currentPage);

            if (args.scrollTop >= this._scrollValue && args.virtualScrollMode == "continuous" && args.reachedEnd) {
                currentPage = this.virtualLoadedPages[this.virtualLoadedPages.length - 1] / pageSize + 2;
            }

            if ($.inArray((currentPage - 1) * pageSize, this.virtualLoadedPages) !== -1) {
                var viewTrs = this.getContentTable().children("tbody").children("tr"); len = viewTrs.length;
                isLast = args.scrollTop >= this._scrollValue;
                for (var i = 0; i < len; i++) {
                    cur = viewTrs[i];
                    oTop = cur.offsetHeight + cur.offsetTop;
                    if (oTop > args.scrollTop + proxy.model.scrollSettings.height) {
                        if (viewTr.length === 0 && i !== 0)
                            viewTr = [viewTrs[cur.offsetTop <= args.scrollTop + proxy.model.scrollSettings.height ? i : i - 1]];
                        break;
                    }
                    if (oTop >= args.scrollTop && oTop <= args.scrollTop + proxy.model.scrollSettings.height) {
                        viewTr.push(cur);
                        if (isLast === false && viewTr.length > 1)
                            break;
                    }
                }
                trEle = $(args.scrollTop >= this._scrollValue ? viewTr[viewTr.length - 1] : viewTr[0]);
                if (trEle.hasClass('e-virtualrow')) {
                    if (viewTr.length === 1) {
                        currentPage++;
                    }
                }
                else
                    currentPage = parseInt(trEle.attr("name"), 10) / pageSize + 1;
            }
            this._scrollValue = args.scrollTop;
            for (var index = 0; index < virtualRows.length; index++) {
                var val = virtualRows[index];
                if (val.offsetTop + val.offsetHeight >= args.scrollTop) {
                    var prevVirtualPage = this._calculatePrevPage(virtualRows, target, args);
                    this._prevPageNo = prevVirtualPage;
                    if (currentPage == 0)
                        currentPage = 1;
                    currentPage = currentPage > this.model.pageSettings.totalPages ? this.model.pageSettings.totalPages : currentPage;
                    return currentPage;
                }
            }
            return currentPage;
        },
        _calculatePrevPage: function (virtualRows, target, args) {
            for (var i = 0; i < virtualRows.length; i++) {
                var val = virtualRows[i];
                if (val.offsetTop + val.offsetHeight >= args.scrollTop) {
                    var trElement = $(val).prevAll('tr[name]')[0];
                    if (trElement != null) {
                        return Math.ceil(parseInt($(trElement).attr('name'), 10) / this.model.pageSettings.pageSize) + 1;
                    }
                }
            }
            return -1;
        },
        _refreshVirtualPagerInfo: function () {
            var model = {};
            model.pageSize = this.model.pageSettings.pageSize;
            model.currentPage = this._currentPage();
            model.totalRecordsCount = this.model.filterSettings.filteredColumns.length == 0 ? this._searchCount == null ? this._gridRecordsCount : this._searchCount : this._filteredRecordsCount;
            model.totalPages = Math.ceil(model.totalRecordsCount / model.pageSize);

            return model;
        },
        _showPagerInformation: function (model) {
            var from = (model.currentPage - 1) * model.pageSize;
            $(this.$pagerStatusBarDiv).find("div:first").html(String.format(this._getLocalizedLabels("PagerInfo"), model.currentPage, model.totalPages, model.totalRecordsCount), from, from + model.pageSize);
            $(this.$pagerStatusBarDiv).css('display', 'block');
        },
        _cellMerging: function (args) {
            args.colMerge = function (range) {
                if (this.cell.className.indexOf("e-colmerge") == -1) {
                    this.cell.className += " e-colmerge";
                    if (this.model.columns.length - this.cell.cellIndex < range)
                        range = this.model.columns.length - this.cell.cellIndex;
                    this.cell.colSpan = range;
                    for (var i = 1; i < range; i++) {
                        if (!ej.isNullOrUndefined(this.cell.parentElement.children[this.cell.cellIndex + i]))
                            this.cell.parentElement.children[this.cell.cellIndex + i].className += " e-hide";
                    }
                }
            };
            args.rowMerge = function (range) {
                if (this.cell.className.indexOf("e-rowmerge") == -1) {
                    this.cell.className += " e-rowmerge";
                    var ele = this.cell.parentNode.parentNode;
                    if (ele.rows.length - this.cell.parentElement.rowIndex < range)
                        range = ele.rows.length - this.cell.parentElement.rowIndex;
                    this.cell.rowSpan = range;
                    for (var i = 0; i < range - 1; i++) {
                        if (!ej.isNullOrUndefined(ele.children[this.cell.parentElement.rowIndex + i].nextSibling)) {
                            if (!($(".e-grid").children().is('.e-dialog')) || ($(".e-grid").find('.e-dialog').attr("style").indexOf("display: none")) != -1 || this.model.allowFiltering)
                                ele.children[this.cell.parentElement.rowIndex + i].nextSibling.children[this.cell.cellIndex].className += " e-merged e-hide";
                        }
                        else
                            break;
                    }
                }
            };
            args.merge = function (col, row) {
                if (col > 1 && row > 1) {
                    if (this.cell.className.indexOf("e-colmerge") == -1) {
                        this.cell.className += " e-colmerge";
                        var ele = this.cell.parentNode.parentNode;
                        if (ele.rows.length - this.cell.parentElement.rowIndex < row)
                            row = ele.rows.length - this.cell.parentElement.rowIndex;
                        if (!($(".e-grid").children().is('.e-dialog')) || ($(".e-grid").find('.e-dialog').attr("style").indexOf("display: none")) != -1 || this.model.allowFiltering) {
                            for (var i = 0; i < row ; i++) {
                                if (!ej.isNullOrUndefined(ele.children[this.cell.parentElement.rowIndex + i])) {
                                    var selectCell = ele.children[this.cell.parentElement.rowIndex + i].children[this.cell.cellIndex];
                                    if (this.model.columns.length - selectCell.cellIndex < col)
                                        col = this.model.columns.length - selectCell.cellIndex;
                                    selectCell.colSpan = col;
                                    for (var j = 1; j < col; j++) {
                                        if (!ej.isNullOrUndefined(selectCell.parentElement.children[this.cell.cellIndex + j]))
                                            selectCell.parentElement.children[this.cell.cellIndex + j].className += " e-hide";
                                    }
                                }
                                else
                                    break;
                            }
                        }
                        else {
                            this.cell.colSpan = col;
                            for (var j = 1; j < col; j++) {
                                if (!ej.isNullOrUndefined(this.cell.nextSibling))
                                    this.cell.parentElement.children[this.cell.cellIndex + j].className += " e-hide";
                            }
                        }
                        args.rowMerge(row);
                    }
                }
                else {
                    if (col > 1)
                        args.colMerge(col);
                    if (row > 1)
                        args.rowMerge(row);
                }
            };
        },
        _replacingContent: function () {
            var temp = document.createElement('div');
            var currentIndex = this.getCurrentIndex();
            var contentTable = this.getContentTable()[0];
            var colGroup = $(contentTable).find("colgroup").first();
            var rowHeight = this.getRowHeight();
            colGroup.replaceWith(this._getMetaColGroup());
            this.model.detailsTemplate != null && colGroup.prepend(this._getIndentCol());
            var tbody = contentTable.tBodies[0];
            var cloneEle = $(tbody).clone();
            var currentData = this.model.currentViewData;
            if (!ej.isNullOrUndefined(this._currentPageData)) {
                this._virtualLoadRecords[this._currentPage()] = this._currentPageData;
                this._currentPageData = null;
            }
            else
                this._virtualLoadRecords[this._currentPage()] = currentData;
            var elementTbody = $("<tbody></tbody>").append($.render[this._id + "_JSONTemplate"](currentData));
            var proxy = this;
            var $elementTbody = elementTbody.children("tr");
            this.virtualLoadedPages.push(currentIndex >= 0 ? currentIndex : 0);
            var orderedVirtualPages = ej.dataUtil.mergeSort(ej.distinct(this.virtualLoadedPages));
            var tempTbody = $("<tbody></tbody>");
            $($elementTbody).attr('name', currentIndex);
            var minValue = ej.dataUtil.min(orderedVirtualPages);
            var maxValue = ej.dataUtil.max(orderedVirtualPages);
            $(cloneEle).children(".e-virtualrow").remove();
            $(tempTbody).append($(cloneEle).children("tr"));
            for (var i = 0; i < orderedVirtualPages.length; i++) {
                var val = orderedVirtualPages[i];
                var pVal = orderedVirtualPages[i - 1];
                if (val != this.orderedVirtualLoadedPage[i] || this.orderedVirtualLoadedPage[i] == undefined) {
                    if (pVal != undefined)
                        $elementTbody.insertAfter($(tempTbody).children('[name=' + pVal + ']:last'));
                    else
                        $elementTbody.insertBefore($(tempTbody).children('[name=' + this.orderedVirtualLoadedPage[i] + ']:first'));
                    this.orderedVirtualLoadedPage = orderedVirtualPages;
                }
                if (val != 0) {
                    var prevValue = val == minValue ? minValue : pVal;
                    var middleRows = val - prevValue - proxy.model.pageSettings.pageSize;
                    if (middleRows > 0) {
                        var virtualTRMiddle = document.createElement("tr");
                        $(virtualTRMiddle).addClass("e-virtualrow").css("height", rowHeight * middleRows).insertBefore($(tempTbody).children('[name=' + val + ']:first'));
                    }
                }
                if (val == maxValue) {
                    var bottomRows = proxy._gridRecordsCount - maxValue - proxy.model.pageSettings.pageSize;
                    if (bottomRows > 0) {
                        var virtualTRBottom = document.createElement("tr");
                        $(virtualTRBottom).addClass("e-virtualrow").css("height", rowHeight * bottomRows).appendTo(tempTbody);
                    }
                }
            }
            if (minValue > 0) {
                var virtualTRTop = document.createElement("tr");
                $(virtualTRTop).addClass("e-virtualrow").css("height", rowHeight * minValue).prependTo(tempTbody);
            }
            var $content = this.getContent();
            var focusTR = $(tbody).html($(tempTbody).html()).children("tr[name=" + currentIndex + "]")[0];
            var focusPrev = focusTR.previousSibling;
            var con = $content.height();
            var focus = focusTR.offsetTop
            if ((focusTR && focusPrev && ((this._virIndex || $(focusPrev).hasClass("e-virtualrow")) || focusPrev.offsetTop > (currentIndex * con))
            && (this._gridRecordsCount - currentIndex >= this.model.pageSettings.pageSize || focusTR.offsetParent.offsetHeight - focus < con)) || this._lastRow) {
                if (this._lastRow) this._lastRow = false;
                if (this._virIndex) this._virIndex = false;
                this._isVirtualRecordsLoaded = true;
                //this.getContent().children("div").first().scrollTop(this.getContent().find(".content").scrollTop() - (this.getContent().find(".content").scrollTop() - focusTR.offsetTop));
                $content.find(".e-content").scrollTop(focus);
                this._scrollValue = this.getContent()[0].firstChild.scrollTop;
            }
            $(temp).append($("<table></table>").append(tempTbody));
            var $contentTableTr = $(contentTable).get(0);
            var tFirst = temp.firstChild;
            $contentTableTr.replaceChild(this.model.rowTemplate != null ? tFirst.lastChild : tFirst.firstChild, $contentTableTr.lastChild);
            this._currentJsonData = currentData;
            this._gridRows = $(contentTable).get(0).rows;
            var lastVirtualRow = $(contentTable).find(".e-virtualrow").last();
            var lastVirtualRowHeight = this.model.scrollSettings.virtualScrollMode == "normal" ? (lastVirtualRow.height() - ($(contentTable).height() - (this._gridRecordsCount * rowHeight))) : 1;
            lastVirtualRow.css("height", lastVirtualRowHeight);
            this._eventBindings();
        },
        _refreshPagerTotalRecordsCount: function () {
            if (this.model.filterSettings.filteredColumns.length)
                this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
            else
                this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
        },
        _maxZindex: function () {
            var maxZ = 1;
            maxZ = Math.max.apply(null, $.map($('body *'), function (e, n) {
                if ($(e).css('position') == 'absolute')
                    return parseInt($(e).css('z-index')) || 1;
            }));
            if (maxZ == undefined || maxZ == null)
                maxZ = 1;
            return maxZ;
        },
        _keyPressed: function (action, target, e) {
            var $target = $(target);
            if ($target.hasClass('e-tooltxt') && e.code == 13) {
                var args = { currentTarget: target, target: target.firstChild }, $toolbar = $(target).closest(".e-gridtoolbar");
                $toolbar.ejToolbar("instance")._trigger("click", args);
                return false;
            }

            if (!this.model.allowKeyboardNavigation || (target.tagName == 'INPUT' && e.code != 40 && e.code != 38 && e.code != 13 && e.code != 27 && e.code != 9) || String.fromCharCode(e.code).toLowerCase() == this.element[0].accessKey.toLowerCase())
                return true;
            if (this.model.allowFiltering && ($target.hasClass('e-filtertext') && e.code == 13) || ($target.hasClass('e-fltrbtn') && e.code == 13))
                return true;
            if ((this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") && $(target).closest("#" + this._id + "EditForm").length)
                return true;
            else if ($(target).parent().siblings("#" + this._id + "EditForm").length)
                return true;
            if (e.code == 13 && $target.parent().hasClass("e-unboundcelldiv"))
                return true;
            if (e.code == 13 && target.tagName == 'INPUT' && $target.parent().is("#" + this._id + "_search"))
                action = "searchRequest";
            if (this.getPager() != null)
                var pager = this.getPager().ejPager("model"), pageIndex = pager.currentPage;
            var returnValue = false, curEl, $target = $(target);

            switch (action) {
                case "insertRecord":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_add");
                    break;
                case "searchRequest":
                    this.search($target.val());
                    break;
                case "saveRequest":
                    if (ej.gridFeatures.edit) {
                        $target.blur();
                        if (this.model.editSettings.editMode == "batch")
                            this._moveCurrentCell("down");
                        else
                            this._toolbarOperation(this._id + "_update");
                    }
                    break;
                case "cancelRequest":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_cancel");
                    break;
                case "deleteRecord":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_delete");
                    break;
                case "editRecord":
                    if (ej.gridFeatures.edit)
                        this._toolbarOperation(this._id + "_edit");
                    break;
                case "totalGroupCollapse":
                    if (ej.gridFeatures.group) {
                        this.collapseAll();
                        this.element.focus();
                    }
                    break;
                case "totalGroupExpand":
                    if (ej.gridFeatures.group) {
                        this.expandAll();
                        this.element.focus();
                    }
                    break;
                case "selectedGroupExpand":
                    if (ej.gridFeatures.group) {
                        this._$currentTr = $(this.getRows()).eq(this._selectedRow());
                        curEl = this._$currentTr.parents("tr").first().prev().find(".e-recordpluscollapse");
                        this.expandCollapse(curEl);
                    }
                    break;
                case "selectedGroupCollapse":
                    if (ej.gridFeatures.group) {
                        this._$currentTr = $(this.getRows()).eq(this._selectedRow());
                        curEl = this._$currentTr.parents("tr").first().prev().find(".e-recordplusexpand");
                        this.expandCollapse(curEl);
                    }
                    break;
                case "firstRowSelection":
                    if (ej.gridFeatures.selection)
                        this.selectRows(0);
                    break;
                case "lastRowSelection":
                    if (ej.gridFeatures.selection)
                        this.selectRows($(this.getRows()).length - 1);
                    break;
                case "upArrow":
                    if (ej.gridFeatures.selection && (this._selectedRow() != -1 || this._previousRowCellIndex != undefined)) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        if (this._selectedRow() > 0) {
                            this.selectRows(this._selectedRow() - 1);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("up");
                        }
                        if (this._previousRowCellIndex && this._previousRowCellIndex.length != 0 && this._previousRowCellIndex[0][0] != 0 && this._allowcellSelection) {
                            this.selectCells([[this._previousRowCellIndex[0][0] - 1, this._previousRowCellIndex[0][1]]]);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("down");
                        }
                    }
                    break;
                case "downArrow":
                    if (ej.gridFeatures.selection && (this._selectedRow() != -1 || this._previousRowCellIndex != undefined)) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = this._excludeDetailRows().length - 1;
                        if (this.model.scrollSettings.frozenColumns > 0 && !ej.isNullOrUndefined(this.getRows()[0]))
                            lastRow = this.getRows()[0].length - 1;
                        if (this._selectedRow() != lastRow && this._selectedRow() != -1) {
                            this.selectRows(this._selectedRow() + 1);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("down");
                        }
                        if (this._previousRowCellIndex && this._previousRowCellIndex.length != 0 && this._previousRowCellIndex[0][0] != lastRow && this._allowcellSelection) {
                            this.selectCells([[this._previousRowCellIndex[0][0] + 1, this._previousRowCellIndex[0][1]]]);
                            if (this.model.editSettings.editMode == "batch")
                                this._moveCurrentCell("down");
                        }
                    }
                    break;
                case "rightArrow":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = $(this.getRows()).length - 1;
                        if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] == this.model.columns.length - 1 && this._previousRowCellIndex[0][0] != lastRow) {
                            this.selectCells([[this._previousRowCellIndex[0][0] + 1, [0]]]);
                        } else if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] != this.model.columns.length - 1)
                            this.selectCells([[this._previousRowCellIndex[0][0], [parseInt(this._previousRowCellIndex[0][1]) + 1]]]);

                    }
                    break;
                case "leftArrow":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] == 0 && this._previousRowCellIndex[0][0] != 0)
                            this.selectCells([[this._previousRowCellIndex[0][0] - 1, [this.model.columns.length - 1]]]);
                        else if (this._previousRowCellIndex && this._previousRowCellIndex[0][1] != 0)
                            this.selectCells([[this._previousRowCellIndex[0][0], [parseInt(this._previousRowCellIndex[0][1]) - 1]]]);

                    }
                    break;
                case "firstCellSelection":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = $(this.getRows()).length - 1;
                        lastRow > -1 && this.selectCells([[0, [0]]]);
                    }
                    break;
                case "lastCellSelection":
                    if (ej.gridFeatures.selection && this._allowcellSelection) {
                        if ((target["type"] == "text" || target["type"] == "checkbox") && this.model.isEdit && this.model.editSettings.editMode != "batch")
                            return true;
                        var lastRow = $(this.getRows()).length - 1;
                        lastRow > -1 && this.selectCells([[lastRow, [this.model.columns.length - 1]]]);
                    }
                    break;
                case "nextPage":
                    if (this.getPager() != null)
                        pageIndex = pageIndex + 1;
                    break;
                case "previousPage":
                    if (this.getPager() != null)
                        pageIndex = pageIndex - 1;
                    break;
                case "lastPage":
                    if (this.getPager() != null)
                        pageIndex = pager.totalPages;
                    break;
                case "firstPage":
                    if (this.getPager() != null)
                        pageIndex = 1;
                    break;
                case "nextPager":
                    if (this.getPager() != null)
                        pageIndex = Math.ceil(pager.currentPage / pager.pageCount) * pager.pageCount + 1;
                    break;
                case "previousPager":
                    if (this.getPager() != null)
                        pageIndex = (Math.floor(pager.currentPage / pager.pageCount) - 1) * pager.pageCount + 1;
                    break;
                case "moveCellLeft":
                    if (this.model.editSettings.editMode == "batch")
                        returnValue = this._moveCurrentCell("left");
                    else
                        returnValue = true;
                    break;
                case "moveCellRight":
                    if (this.model.editSettings.editMode == "batch" && $target)
                        returnValue = this._moveCurrentCell("right");
                    else
                        returnValue = true;
                    break;
                default:
                    returnValue = true;
            }
            if (this.getPager() != null && pageIndex <= pager.totalPages && pager.currentPage !== pageIndex && action != "searchRequest" && action !== "deleteRecord")
                this.getPager().ejPager("goToPage", pageIndex);
            return returnValue;
        },
        _findColumnsWidth: function () {
            var j = this.getHeaderTable().find(".e-headercell").not(".e-stackedHeaderCell"), index = 0;
            if (j.eq(0).hasClass("e-detailheadercell"))
                index = 1;
            for (var i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i]["visible"])
                    this._columnsWidthCollection[i] = j.eq(i + index).outerWidth();
            }
        },
        _calculateWidth: function () {
            var j = this.getHeaderTable().find(".e-columnheader").last().find("th:visible"), width = 0;
            for (var i = 0; i < j.length; i++)
                width += j.eq(i).outerWidth();
            return width;

        },
        _initIndicators: function () {
            var indicatorId = this._id + "_ColumnDropIndicator";
            if ($("#" + indicatorId).length)
                $("#" + indicatorId).remove();
            this._Indicator = document.createElement("DIV");
            $(this._Indicator).attr('id', indicatorId).addClass("e-columndropindicator").addClass("e-dropAcceptor").appendTo(document.body);
            $(this._Indicator).css({ "display": "none" });

        },
        _refreshGroupSummary: function () {
            var headerCols = this.getHeaderContent().find("colgroup col").clone();
            headerCols.splice(0, this.model.groupSettings.groupedColumns.length);
            var $gsColgroup = this.getContentTable().find(".e-groupsummary colgroup");
            for (var i = 0; i < $gsColgroup.length; i++) {
                if (!$($gsColgroup[i]).find(".e-summary").is("visible"))
                    $($gsColgroup[i]).find(".e-summary").show();
                $($gsColgroup[i]).find("col:not('.e-summary')").remove();
                $($gsColgroup[i]).append(headerCols.clone());
            }
        },
        
        reorderColumns: function (fromfname, tofname) {
            var fromindex = this.model.columns.indexOf(this.getColumnByField(fromfname));
            var toindex = this.model.columns.indexOf(this.getColumnByField(tofname));
            this.set_dropColumn(fromindex, toindex);
            if (this.model.showStackedHeader)
                this._refreshStackedHeader();
            if (this.model.scrollSettings.allowVirtualScrolling)
                this._refreshVirtualContent();
        },
        
        columns: function (details, action) {
            if (ej.isNullOrUndefined(details)) return;
            var isString = false;
            if (typeof details === "string") {
                details = [details];
                isString = true;
            }
            else if (details instanceof Array && details.length && typeof details[0] === "string")
                isString = true;
            for (i = 0; i < details.length; i++) {
                var index = $.inArray(this.getColumnByField(isString ? details[i] : details[i]["field"]), this.model.columns);
                if (action == "add" || ej.isNullOrUndefined(action)) {
                    if (index == -1)
                        this.model.columns.push(isString ? { field: details[i] } : details[i]);
                    else
                        this.model.columns[index] = isString ? { field: details[i] } : details[i];
                }
                else {
                    if (index != -1)
                        this.model.columns.splice(index, 1);
                }
            }
            var $header = this.element.find(".e-gridheader");
            this._columnsWidthCollection = [];
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++)
                this._columnsWidthCollection.push(this.model.columns[columnCount]["width"]);
            this.element[0].replaceChild(this._renderGridHeader()[0], $header[0]);
            this._headerCellgDragDrop();
            this.model.allowReordering && this._headerCellreorderDragDrop();
            this.model.showColumnChooser && this._renderColumnChooserData(true);
            this.refreshContent(true);
        },
        _enableRowHover: function () {
            if (this.model.enableRowHover)
                this._on(this.element, "mouseenter mouseleave", ".e-gridcontent tr td", this._rowHover);
            else
                this._off(this.element, "mouseenter mouseleave", ".e-gridcontent tr td");
        },
        _rowHover: function (e) {
            if (!this.model.enableRowHover)
                return;
            var $target = $(e.target);
            var $gridRows = $(this.getRows());
            if (($target.hasClass("e-rowcell") && $target.closest("#" + this._id + "EditForm").length) || !$target.hasClass("e-rowcell"))
                return;
            if (e.type == "mouseenter" && !this._dragActive) {
                if (this.model.scrollSettings.frozenColumns > 0 && !ej.isNullOrUndefined($gridRows[0]) && !ej.isNullOrUndefined($gridRows[1]))
                    $gridRows = $($gridRows[0]).add($gridRows[1]);
                $gridRows.removeClass("e-hover");
                var index = this.getIndexByRow($target.parent());
                index != -1 && this.getRowByIndex(index).addClass("e-hover");
            } else {
                if (this.model.scrollSettings.frozenColumns > 0 && !ej.isNullOrUndefined($gridRows[0]) && !ej.isNullOrUndefined($gridRows[1]))
                    $gridRows = $($gridRows[0]).add($gridRows[1]);
                $gridRows.removeClass("e-hover");
            }
            return false;
        },
        _rightClickHandler: function (e) {
            e.preventDefault(); var browser = ej.browserInfo();
            if (e.which == 3 || (browser.name == "msie" && browser.version == "8.0")) {
                var args = {};
                $target = $(e.target);
                $gridRow = $(this.getRows());
                if (this.getContentTable().has($target).length) {
                    var index = $gridRow.index($target.parent());
                    if (index == -1)
                        return;
                    args = { rowIndex: index, row: this.getRowByIndex(index), data: this._currentJsonData[index], cellIndex: $target.index(), cellValue: $target.html(), cell: $target };
                }
                else if (this.getHeaderTable().has($target).length) {
                    var index = 0
                    $th = this.getHeaderTable().find('th').not('.e-detailheadercell,.e-grouptopleftcell,.e-filterbarcell');
                    if ($target.is('.e-headercelldiv'))
                        index = $th.index($target.closest('.e-headercell'));
                    else
                        index = $th.index($target);
                    if (index == -1)
                        return;
                    args = { headerIndex: index, headerText: this.getColumnFieldNames()[index], headerCell: $th.eq(index), column: this.getColumnByIndex(index) }
                }
                else if ($target.is('.e-pager') || (this.getPager() != null && this.getPager().has($target).length)) {
                    args = { pager: this.model.pageSettings }
                }
                this._trigger("rightClick", args);
            }
        },
        _touchGrid: function (e) {
            var curPage = this._currentPage();
            switch (e.type) {
                case "swipeleft":
                    if (this.model.allowPaging && curPage != this.model.pageSettings.totalPages && !this.model.isEdit)
                        this.element.ejGrid("gotoPage", curPage + 1);
                    break;
                case "swiperight":
                    if (this.model.allowPaging && curPage > 1 && !this.model.isEdit)
                        this.element.ejGrid("gotoPage", curPage - 1);
                    break;
            }
        },
        _recorddblClickHandler: function (e) {
            var args = {}, $target = $(e.target).is(".e-rowcell") ? $(e.target) : $(e.target).closest("td");
            if ($target.closest(".e-grid").attr("id") !== this._id) return;
            if ((!$target.is('.e-rowcell') && !$target.closest("td").is(".e-rowcell")) || ($target.closest('.e-editcell,.e-editedbatchcell').length > 0))
                return;
            args = { rowIndex: this._selectedRow(), row: this.getRowByIndex(this._selectedRow()), data: this._currentJsonData[this._selectedRow()], cell: $target };
            this._trigger("recordDoubleClick", args);
        },
        _recordClick: function (e) {
            var args = {};
            $target = $(e.target);
            if ($target.closest(".e-grid").attr("id") !== this._id) return;
            if ((!$target.is('.e-rowcell') && !$target.closest("td").is(".e-rowcell")) || ($target.closest('.e-editcell,.e-editedbatchcell')).length > 0)
                return;
            var index = this.getIndexByRow($target.closest('tr'));
            args = { rowIndex: index, row: this.getRowByIndex(index), data: this._currentJsonData[index], cell: $target.closest("td") };
            this._trigger("recordClick", args);
        },
        _headerMouseDown: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseDown(e);
            if (($(e.target).hasClass("e-headercelldiv") && !$(e.target).parent().hasClass("e-grouptopleftcell")) || $(e.target).hasClass("e-headercell")) {
                var $headercell = $(e.target).hasClass("e-headercelldiv") ? $(e.target).parent() : $(e.target);
                this.model.enableHeaderHover && $headercell.removeClass("e-hover e-headercell-hover").addClass("e-headercellactive e-active");
            }
        },
        _headerHover: function (e) {
            var $target = $(e.target);
            if (e.type == "mouseover" || e.type == "mousemove" || e.type == "touchmove" || e.type == "MSPointerMove") {
                if (this.model.allowResizing || this.model.allowResizeToFit)
                    this._resizer._mouseHover(e);

                if (this.model.enableHeaderHover && !this._dragActive && (($target.hasClass("e-headercelldiv") && !$target.parent().hasClass("e-grouptopleftcell")) || $target.hasClass("e-headercell"))) {
                    if ($target.hasClass("e-headercelldiv"))
                        $target = $target.parent();
                    this.getHeaderTable().find(".e-columnheader").find(".e-headercell-hover").removeClass("e-headercell-hover").removeClass("e-hover");
                    $target.addClass("e-headercell-hover e-hover");
                }
                if (e.type == "mouseover")
                    this._addCursor();
            } else
                this.model.enableHeaderHover && this.getHeaderTable().find(".e-columnheader").find(".e-headercell-hover").removeClass("e-headercell-hover").removeClass("e-hover");
        },
        _addCursor: function () {
            var flag = (this.model.allowResizing || this.model.allowResizeToFit || this.model.allowGrouping || this.model.allowFiltering || this.model.allowSorting || this.model.allowReordering || this.model.contextMenuSettings.enableContextMenu || this.model.selectionSettings.selectionMode == "column");
            if (!flag) {
                this.getHeaderTable().find(".e-columnheader").addClass("e-defaultcursor");
                this.getHeaderTable().find(".e-headercell").removeClass("e-defaultcursor");
            }
            else {
                this.getHeaderTable().find(".e-columnheader").removeClass("e-defaultcursor");
                if (!(this.model.contextMenuSettings.enableContextMenu || this.model.selectionSettings.selectionMode == "column")) {
                    var propArray = [{ val: this.model.allowSorting, str: "sort" }, { val: this.model.allowGrouping, str: "group" }, { val: this.model.allowReordering, str: "reorder" }, { val: this.model.allowFiltering, str: "filter" }];
                    var colpropcount = 0;
                    for (var i = 0; i < propArray.length; i++) {
                        if (propArray[i].val == false) {
                            propArray.splice(i, 1);
                            i--;
                        }
                    }

                    for (i = 0; i < this.model.columns.length && propArray.length > 0 ; i++) {
                        for (j = 0; j < propArray.length; j++) {
                            switch (propArray[j].str) {
                                case "sort":
                                    if (!ej.isNullOrUndefined(this.model.columns[i].allowSorting) && !this.model.columns[i].allowSorting)
                                        colpropcount++;
                                    break;
                                case "group":
                                    if (!ej.isNullOrUndefined(this.model.columns[i].allowGrouping) && !this.model.columns[i].allowGrouping)
                                        colpropcount++
                                    break;
                                case "reorder":
                                    if (!ej.isNullOrUndefined(this.model.columns[i].allowReordering) && !this.model.columns[i].allowReordering)
                                        colpropcount++
                                    break;
                                case "filter":
                                    if (!ej.isNullOrUndefined(this.model.columns[i].allowFiltering) && !this.model.columns[i].allowFiltering)
                                        colpropcount++;
                                    break;
                            }
                        }
                        if (colpropcount == propArray.length)
                            this.getHeaderTable().find(".e-headercell").eq(i).addClass("e-defaultcursor");
                        colpropcount = 0;
                    }
                }
            }
        },
        _colgroupRefresh: function () {
            if (this.model.allowResizing && this.model.scrollSettings.frozenColumns > 0) {
                var gridheaderCol = $(this.getHeaderTable()).find('colgroup');
                var gridcontentCol = $(this.getContentTable()).find('colgroup');
            }
            else {
                var gridheaderCol = $(this.getHeaderTable()).find('colgroup')[0];
                var gridcontentCol = $(this.getContentTable()).find('colgroup')[0];
            }
            var headerColClone = $(gridheaderCol).clone();
            var contentColClone = $(gridcontentCol).clone();
            $(gridcontentCol).remove();
            $(gridheaderCol).remove();
            if (this.model.allowResizing && this.model.scrollSettings.frozenColumns > 0) {
                $(headerColClone[0]).prependTo(this.getHeaderTable()[0]);
                $(headerColClone[1]).prependTo(this.getHeaderTable()[1]);
                $(contentColClone[0]).prependTo(this.getContentTable()[0]);
                $(contentColClone[1]).prependTo(this.getContentTable()[1]);
            }
            else {
                $(headerColClone).prependTo(this.getHeaderTable());
                $(contentColClone).prependTo(this.getContentTable());
            }
        },
        _detailColsRefresh: function () {
            this._$headerCols = this.getHeaderTable().children("colgroup").find("col");
            this._$contentCols = this.getContentTable().children("colgroup").find("col");
            var colCount = this.model.columns.length;
            if (this._$headerCols.length > colCount) this._$headerCols.splice(0, (this._$headerCols.length - colCount));
            if (this._$contentCols.length > colCount) this._$contentCols.splice(0, (this._$contentCols.length - colCount));
        },
        _headerdblClickHandler: function (e) {
            if (this.model.allowResizeToFit)
                this._resizer._columnResizeToFit(e);
            if (this.model.allowScrolling)
                this.getScrollObject().refresh();
        },

        _mouseUp: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseUp(e);
        },

        _mouseMove: function (e) {
            if (this.model.allowResizing)
                this._resizer._mouseMove(e);
        },
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "enableResponsiveRow":
                        if (options[prop]) {
                            this.element.addClass("e-responsive");
                            if (this.model.minWidth) {
                                this._removeMedia();
                                this._scrollerAddedOnMedia = false;
                            }
                            if (this.model.allowFiltering) {
                                this.element.find('.e-filterDialoge').remove();
                                this._renderFilterDialogs();
                                this._renderResponsiveFilter();
                                this.element.find('.e-gridtoolbar').remove();
                            }
                        } else {
                            this.element.removeClass("e-responsive");
                            if (this.model.minWidth)
                                this._addMedia();
                            if (this.model.allowFiltering) {
                                $('body').find('.e-filterDialoge').remove();
                                this._renderFilterDialogs();
                            }
                        }
                        this._tdsOffsetWidth = [];
                        if (this.model.allowFiltering || this.model.allowSorting) {
                            var index = this.model.toolbarSettings.toolbarItems.indexOf('responsiveFilter');
                            index != -1 && this.model.toolbarSettings.toolbarItems.splice(index, 1);
                            var sortIndex = this.model.toolbarSettings.toolbarItems.indexOf('responsiveSorting');
                            sortIndex != -1 && this.model.toolbarSettings.toolbarItems.splice(sortIndex, 1);
                            this.element.find('.e-gridtoolbar').remove();
                            this._renderToolBar().insertBefore(this.element.find(".e-gridheader").first());
                        }
                        break;
                    case "showColumnChooser":
                        if (options[prop]) {
                            this._renderGridHeaderInternalDesign(this.model.columns);
                            this._renderColumnChooser();
                        }
                        else {
                            this.element.find(".e-ccButton").remove()
                            $("#" + this._id + 'ccDiv_wrapper').remove();
                        }
                        break;
                    case "gridLines":
                        this.getContent().removeClass("e-horizontallines e-verticallines e-hidelines");
						this.getHeaderContent().removeClass("e-horizontallines e-verticallines e-hidelines");
						this._showHeaderGridLines();
                        this._showGridLines();
                        break;
                    case "showDeleteConfirmDialog":
                        this.model.editSettings.showDeleteConfirmDialog = options[prop];
                        if (options[prop])
                            this._renderConfirmDialog();
                        else
                            this.element.find("#" + this._id + 'ConfirmDialog_wrapper').remove()
                        break;
                    case "showConfirmDialog":
                        this.model.editSettings.showConfirmDialog = options[prop];
                        if (options[prop])
                            this._renderConfirmDialog();
                        else
                            this.element.find("#" + this._id + 'ConfirmDialog_wrapper').remove()
                        break;
                    case "pageSettings":
                        var pageModel = this.getPager().ejPager("model");
                        if (ej.isNullOrUndefined(options[prop]["currentPage"]) || pageModel.currentPage != this._currentPage()) {
                            for (var pageProp in options[prop]) {
                                if (pageProp != "currentPage" && options[prop][pageProp] === pageModel[pageProp])
                                    delete options[prop][pageProp];
                            }
                            if ($.isEmptyObject(options[prop]))
                                break;
                            options[prop]["currentPage"] = this._currentPage();
                            this.getPager().ejPager("option", options[prop]);
                            this._currentPage(this._currentPage() > pageModel.totalPages ? pageModel.totalPages : this._currentPage());
                            this.refreshContent();
                        }
                        break;
                    case "allowPaging":
                        this.model.allowPaging = options[prop];
                        if (options[prop] && this.element.children(".e-pager").length == 0) {
                            this.element.append(this._renderGridPager());
                            this.refreshContent();
                            this.getPager().ejPager("refreshPager");
                        } else {
                            this.getPager().remove();
                            this.setGridPager(null);
                            this.refreshContent();
                            if (this.model.filterSettings.filterType == "filterbar" && this.model.allowFiltering)
                                this._createPagerStatusBar();
                        }
                        break;
                    case "allowSearching":
                        this.model.allowSearching = options[prop];
                        break;
                    case "searchSettings":
                        $.extend(this.model.searchSetings, options[prop]);
                        this.refreshContent();
                        break;
                    case "allowGrouping":
                        if (options[prop] && this.element.children(".e-groupdroparea").length == 0) {
                            this.model.allowGrouping = options[prop];
                            this.addGroupingTemplate();
                            this.element.prepend(this._renderGroupDropArea());
                            this._enableGroupingEvents();
                            this._headerCellgDragDrop();
                            this._off(this.element, "mouseenter mouseleave", ".e-groupdroparea,.e-groupheadercell", this._dropAreaHover);
                            this._on(this.element, "mouseenter mouseleave", ".e-groupdroparea,.e-groupheadercell", this._dropAreaHover);
                        } else
                            this.element.children(".e-groupdroparea").remove();
                        if (this.model.allowGrouping) {
                            !ej.isNullOrUndefined(options["groupSettings"]) && $.extend(this.model.groupSettings, options["groupSettings"]);
                            this._enableGrouping();
                        }
                        break;
                    case "groupSettings":
                        $.extend(this.model.groupSettings, options[prop]);
                        if(this.model.allowGrouping && ej.isNullOrUndefined(options["allowGrouping"]))
							this._enableGrouping();
                        break;
                    case "cssClass":
                        this.element.removeClass(this.model.cssClass).addClass(options[prop]);
                        break;
                    case "allowFiltering":
                    case "filterSettings":
                        if (prop == "filterSettings")
                            $.extend(this.model.filterSettings, options[prop]);
                        else
                            this.model.allowFiltering = options[prop];
                        this._$fDlgIsOpen && (this._isExcelFilter ? this._excelFilter.closeXFDialog() : this._closeFilterDlg())
                        if (!this.model.allowFiltering) {
                            if (this.model.filterSettings.filterType == ej.Grid.FilterType.FilterBar)
                                this.getHeaderTable().find(".e-filterbar").remove();
                            else if (this.model.filterSettings.filterType == ej.Grid.FilterType.Menu || this.model.filterSettings.filterType == ej.Grid.FilterType.Excel)
                                this.getHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                            this.model.filterSettings.filteredColumns = [];
                            this.refreshContent();
                        } else {
                            if (this.model.filterSettings.filterType == ej.Grid.FilterType.FilterBar) {
                                this.getHeaderTable().find(".e-filterbar").remove();
                                this.getHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                                this._renderFiltering();
                                if (this.model.filterSettings.showFilterBarStatus && !this.model.allowPaging)
                                    this._createPagerStatusBar();
                                else if (this.model.allowPaging)
                                    this.getPager().ejPager({ enableExternalMessage: this.model.filterSettings.showFilterBarStatus });
                                var $filterbar = this.getHeaderTable().find(".e-filterbar");
                                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                                    $filterbar.prepend(this._getEmptyFilterBarCell());
                            } else if (!_filter && (this.model.filterSettings.filterType == ej.Grid.FilterType.Menu || this.model.filterSettings.filterType == ej.Grid.FilterType.Excel)) {
                                var _filter = 1;
                                this.getHeaderTable().find(".e-columnheader").find(".e-filtericon").remove()
                                    .end().find(".e-headercellfilter").removeClass("e-headercellfilter");
                                this.getHeaderTable().find(".e-filterbar").remove();
                                var columnHeader = this.getHeaderTable().find(".e-columnheader").last().find(".e-headercell").not(".e-detailheadercell");
                                for (var i = 0; i < columnHeader.length; i++) {
                                    var columnName = columnHeader.eq(i).find(".e-headercelldiv").attr("ej-mappingname");
                                    var column = this.getColumnByField(columnName);
                                    if (ej.isNullOrUndefined(column.allowFiltering) || column.allowFiltering)
                                        columnHeader.eq(i).addClass("e-headercellfilter").append(ej.buildTag('div.e-filtericon e-icon e-filterset'));
                                }
                                if (this.model.filterSettings.filterType == ej.Grid.FilterType.Excel) {
                                    this._isExcelFilter = true;
                                    this._renderExcelFilter();
                                } else if (this._isExcelFilter) {
                                    this._isExcelFilter = false;
                                    this._excelFilter.resetExcelFilter();
                                    this._excelFilter = null;
                                }
                                this._renderFilterDialogs();
                                this.model.filterSettings.filteredColumns = [];
                                this.refreshContent();
                            }
                            this._enableFilterEvents();
                        }
                        break;
                    case "enableRowHover":
                        this.model.enableRowHover = options[prop];
                        this._enableRowHover();
                        break;
                    case "allowScrolling":
                    case "scrollSettings":
                        var $content = this.getContent();
                        if (prop != "allowScrolling") {
                            if (!ej.util.isNullOrUndefined(options["scrollSettings"])) {
                                if ($.isEmptyObject(options["scrollSettings"])) break;
                                $.extend(this.model.scrollSettings, options["scrollSettings"]);
                            }
                            if (options["scrollSettings"]["frozenColumns"] !== undefined || options["scrollSettings"]["frozenRows"] !== undefined ||
                                options["scrollSettings"]["allowVirtualScrolling"] !== undefined || options["scrollSettings"]["virtualScrollMode"] !== undefined) {
                                var model = this.model;
                                model.query = this.commonQuery.clone();
                                if (this._selectedRow() != -1)
                                    this.clearSelection(this._selectedRow());
                                if (options["scrollSettings"]["virtualScrollMode"] != undefined)
                                    model.pageSettings.currentPage = 1;
                                this.element.ejGrid("destroy").ejGrid(model);
                            }
                            else {
                                if (!ej.util.isNullOrUndefined(options["allowScrolling"]))
                                    this.model.allowScrolling = options["allowScrolling"];
                                !ej.util.isNullOrUndefined($content.data("ejScroller")) && $content.ejScroller("destroy");
                                if (this.model.allowScrolling) {
                                    this.getHeaderContent().find("div").first().addClass("e-headercontent");
									 this._originalScrollWidth = this.model.scrollSettings.width;
                                    this._renderScroller();
                                } else {
                                    this.element.children(".e-gridheader").removeClass("e-scrollcss");
                                    this.element.get(0).style.width.length == 0 && this.element.css("width", "auto");
                                }
                            }
                        }
                        break;
                    case "locale":
                        this.model.locale = options[prop];
                        var model = this.model;
                        model.query = this.commonQuery.clone();
                        this.element.ejGrid("destroy").ejGrid(model);
                        break;
                    case "dataSource":
                        var $content = this.element.find(".e-gridcontent").first();
                        if (this.model.showSummary && this.model.summaryRows.length > 0) {
                            this.element.children(".e-gridfooter").remove();
                            this._renderGridFooter().insertAfter($content);
                        }
                        if (!$.isFunction(options["dataSource"]))
                            this.resetModelCollections();
                        this._refreshDataSource(this._dataSource());
                        if (this._gridRecordsCount && this.model.allowFiltering) {
                            this._initColumns(this.model.currentViewData[0] != undefined ? this.model.currentViewData[0] : this.model.currentViewData.value);
                            this._renderFilterDialogs();
                        }
                        this._addLastRow();
                        break;
                    case "selectedRowIndex":
                        if (this._selectedRow() != -1 && $.inArray(this._selectedRow(), this.selectedRowsIndexes) == -1)
                            this.selectRows(this._selectedRow());
                        else if (this._selectedRow() == -1) {
                            this.clearSelection();
                            this.selectedRowsIndexes = [];
                        }
                        break;
                    case "editType":
                        if (this._selectedRow() != -1 && $.inArray(this._selectedRow(), this.selectedRowsIndexes) == -1)
                            this.selectRows(this._selectedRow());
                        break;
                    case "editSettings":
                        $.extend(this.model.editSettings, options[prop]);
                        this._processEditing();
                        this.refreshBatchEditMode();
                        this.refreshToolbar();
                        this._tdsOffsetWidth = [];
                        $("#" + this._id + "_dialogEdit").data("ejDialog") && $("#" + this._id + "_dialogEdit").ejDialog("destroy");
                        $("#" + this._id + "_dialogEdit_wrapper,#" + this._id + "_dialogEdit").remove();
                        $("#" + this._id + "_externalEdit").remove();
                        if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                            if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate")
                                this.element.append(this._renderDialog());
                            else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                                this.element.append(this._renderExternalForm());
                            else if (this.model.editSettings.showAddNewRow && this.model.editSettings.editMode == "normal")
                                this._startAdd();
                            else if (!this.model.editSettings.showAddNewRow)
                                this.getContentTable().find(".e-addedrow").remove();
                        }
                        if (this.model.editSettings.allowDeleting && this.model.editSettings.showDeleteConfirmDialog)
                            ej.isNullOrUndefined(this._confirmDialog) && this._renderConfirmDialog();
                        this._enableEditingEvents();
                        break;
                    case "allowResizing":
                        this.model.allowResizing = options[prop];
                        if (this.model.allowResizing) {
                            this._on(this.element, ej.eventType.mouseMove, this._mouseMove);
                            this._on(this.element, "mouseup", this._mouseUp);
                            this._resizer = new ej.gridFeatures.gridResize(this);
                        }
                        else {
                            this._off(this.element, ej.eventType.mouseMove, this._mouseMove);
                            this._off(this.element, "mouseup", this._mouseUp);
                            this._resizer = null;
                        }
                        break;
                    case "allowReordering":
                        this.model.allowReordering = options[prop];
                        if (this.model.allowReordering)
                            this._headerCellreorderDragDrop();
                        break;
                    case "showSummary":
                    case "summaryRows":
                        if (prop == "showSummary" && !options[prop])
                            this.element.children(".e-gridfooter").remove();
                        else if (prop == "summaryRows" && this.model.showSummary || prop == "showSummary") {
                            this.element.children(".e-gridfooter").remove();
                            this.element.children(".e-gridfooter").remove();
                            var $content = this.element.find(".e-gridcontent").first();
                            var query = this.model.query.queries;
                            var pageQuery = [];
                            for (var i = 0; i < query.length; i++) {
                                if (query[i].fn === "onPage") {
                                    pageQuery = query.splice(i, 1);
                                }
                            }
                            var queryManager = this.model.query;
                            this._setSummaryAggregate(queryManager);
                            queryManager.queries.push(pageQuery[0]);
                            if (this._isLocalData) {
                                this._remoteSummaryData = this._dataManager.executeLocal(queryManager).aggregates;
                                this._renderGridFooter().insertAfter($content);
                            }
                            else {
                                var proxy = this;
                                var promise = this._dataManager.executeQuery(queryManager);
                                promise.done(function (e) {
                                    proxy.element.children(".e-gridfooter").remove();
                                    proxy._remoteSummaryData = e.aggregates
                                    proxy._renderGridFooter().insertAfter($content);
                                });
                            }
                            if (this.model.allowGrouping) {
                                this._rowCol = this._captionSummary();
                                this._isCaptionSummary = (this._rowCol != null && this._rowCol.length) > 0 ? true : false;
                                this.refreshContent(true);
                            }
                        }
                        break;
                    case "enableAltRow":
                        this.model.enableAltRow = options[prop];
                        this.addInitTemplate();
                        this.refreshContent();
                        break;
                    case "toolbarSettings":
                        $.extend(this.model.toolbarSettings, options[prop]);
                        this.element.children(".e-gridtoolbar").remove();
                        if (this.model.toolbarSettings.showToolbar)
                            this._renderToolBar().insertBefore(this.element.find(".e-gridheader").first());
                        break;
                    case "selectionSettings":
                        $.extend(this.model.selectionSettings, options[prop]);
                        this._allowrowSelection = this._allowcellSelection = this._allowcolumnSelection = false;
                        this._initSelection();
                        break;
                    case "sortSettings":
                        $.extend(this.model.sortSettings, options[prop]);
                        this.refreshContent();
                        break;
                    case "contextMenuSettings":
                        $.extend(this.model.contextMenuSettings, options[prop]);
                        !ej.isNullOrUndefined($("#" + this._id + "_Context").data("ejMenu")) && $("#" + this._id + "_Context").ejMenu("destroy");
                        if (this.model.contextMenuSettings.enableContextMenu)
                            this._renderContext()
                        break;
                    case "enableRTL":
                        this.model.enableRTL = options[prop];
                        var model = this.model;
                        model.query = this.commonQuery.clone();
                        this.element.ejGrid("destroy");
                        model.enableRTL ? $("#" + this._id).addClass("e-rtl") : $("#" + this._id).removeClass("e-rtl");
                        $("#" + this._id).ejGrid(model);
                        break;
                    case "enableTouch":
                        this.model.enableTouch = options[prop];
                        if (!this.model.enableTouch) {
                            this.element.addClass("e-touch");
                            this._off(this.element, "swipeleft swiperight", ".e-gridcontent .e-table");
                        }
                        else {
                            this._on(this.element, "swipeleft swiperight", ".e-gridcontent .e-table", $.proxy(this._touchGrid, this));
                            this.element.removeClass("e-touch");
                        }
                        break;
                    case "allowSelection":
                        if (options[prop]) {
                            this._off(this.element, ($.isFunction($.fn.tap) && this.model.enableTouch) ? "tap" : "click", this._clickHandler);
                            this._on(this.element, "click", this._clickHandler);
                        }
                        break;
                    case "query":
                        this.commonQuery = $.extend(true, {}, options[prop]);
                        break;
                    case "showStackedHeader":
                    case "stackedHeaderRows":
                        this.model.stackedHeaderRows = options[prop];
                        this._refreshStackedHeader();
                        break;
                    case "allowTextWrap":
                        this.element.addClass("e-wrap");
                        break;
                    case "rowTemplate":
                        this.refreshContent(true);
                        break;
                    case "detailsTemplate":
                        var $header = this.element.children(".e-gridheader");
                        $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
                        this.setGridHeaderContent($header);
                        this.refreshContent(true);
                        if (this.model.allowGrouping)
                            this._headerCellgDragDrop();
                        break;
                }
            }

        },
        
        resetModelCollections: function () {
            this.model.groupSettings.groupedColumns = [];
            this.model.filterSettings.filteredColumns = [];
            this.model.sortSettings.sortedColumns = [];
            this.model.pageSettings.currentPage = this.defaults.pageSettings.currentPage;
            this.model.summaryRows = [];
        },
        _enableGrouping: function(){
			this.refreshTemplate();
			if (this.model.groupSettings.showToggleButton) {
				var $table = this._renderGridHeader().find("table");
				this.element.children(".e-gridheader").find("table").replaceWith($table);
				this.setGridHeaderTable($table);
				this._headerCellgDragDrop();
				this.setWidthToColumns();
			}
			if (!ej.isNullOrUndefined(this.model.groupSettings.groupedColumns.length)) {
				var args = {};
				args.columnName = this.model.groupSettings.groupedColumns[this.model.groupSettings.groupedColumns.length - 1];
				args.requestType = ej.Grid.Actions.Grouping;
				$(".e-grid").find(".e-groupdroparea").empty();
				for (var i = 0; i < this.model.groupSettings.groupedColumns.length - 1; i++)
					this._addColumnToGroupDrop(this.model.groupSettings.groupedColumns[i]);
				this._processBindings(args);
			}
			if (!this.model.groupSettings.enableDropAreaAnimation)
				this.expandGroupDropArea();
			else
				this.collapseGroupDropArea();
        },
        
        addIgnoreOnExport: function (args) {
            if (typeof (args) == 'string')
                this.ignoreOnExport.push(args);
            else
                this.ignoreOnExport = this.ignoreOnExport.concat(args);
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.edit = {
        _processEditing: function () {
            var query = this._columnToSelect();
            this.model.query._fromTable != "" && query.from(this.model.query._fromTable);
            if (this._dataSource() instanceof ej.DataManager && query.queries.length && !this._dataManager.dataSource.offline) {
                var queryPromise = this._dataSource().executeQuery(query);
                queryPromise.done(ej.proxy(function (e) {
                    this._dropDownManager = ej.DataManager(e.result);
                    if (this.model.editSettings.editMode == "normal") this.addEditingTemplate();
                    else if (this.model.editSettings.editMode == "batch") this.addBatchEditTemplate();
                    else if (this.model.editSettings.editMode == "dialog" ||
                        this.model.editSettings.editMode == "externalform" ||
                        this.model.editSettings.editMode == "inlineform")
                        this.addDialogEditingTemplate();
                    else this.addExternalDialogEditingTemplate();
                }, this));
            } else {
                if (this.model.editSettings.editMode == "normal") this.addEditingTemplate();
                else if (this.model.editSettings.editMode == "batch") this.addBatchEditTemplate();
                else if (this.model.editSettings.editMode == "dialog" ||
                    this.model.editSettings.editMode == "externalform" ||
                    this.model.editSettings.editMode == "inlineform")
                    this.addDialogEditingTemplate();
                else this.addExternalDialogEditingTemplate();
            }
        },
        addEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $tbody = ej.buildTag('tbody');
            var $tr = ej.buildTag('tr');
            var $td = ej.buildTag('td', "", {}, { colSpan: this.model.scrollSettings.frozenColumns > 0 ? this.model.scrollSettings.frozenColumns : this.model.columns.length });
            var $form = ej.buildTag('form', "", {}, { id: this._id + "EditForm", "class": "gridform" });
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: 0.25 });
            var $innerTbody = ej.buildTag('tbody');
            var $innerTr = ej.buildTag('tr');
            $tbody.append($tr);
            $tr.append($td);
            $td.append($form);
            var $colGroup = $(document.createElement('colgroup'));
            $form.append($table);
            $table.append($colGroup);
            $innerTbody.append($innerTr).appendTo($table);
            if (this.model.scrollSettings.frozenColumns > 0) {
                var $tbodyClone = $tbody.clone();
                $tbodyClone.find("td").first().prop("colSpan", this.model.columns.length - this.model.scrollSettings.frozenColumns);
            }
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                var $innerTd = ej.buildTag('td.e-rowcell');
                $innerTr.append($innerTd.get(0));
                if (ej.isNullOrUndefined(this.model.columns[columnCount]["isUnbound"]) && ej.isNullOrUndefined(this.model.columns[columnCount]["template"])) {
                    this._initCellEditType(columnCount, $innerTd);
                } else if (this.model.columns[columnCount]["template"]) {
                    var helpers = {};
                    helpers["_" + this._id + "ColumnTemplating"] = ej.proxy(this._gridTemplate, null, this);
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + columnCount + "_Template").remove();
                    var scriptElement = this._createTemplateElement(this.model.columns[columnCount]);
                    $innerTd.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "')}}");
                } else if (this.model.columns[columnCount]["isUnbound"]) {
                    var helpers = {};
                    helpers["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + "_UnboundTemplate").remove();
                    divElement = this._createUnboundElement(this.model.columns[columnCount]);
                    $innerTd.addClass("e-unboundcell").addClass("e-" + this.model.columns[columnCount]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                    this.model.scrollSettings.frozenColumns > 0 && $innerTd.addClass("e-frozenunbound");
                    this._isUnboundColumn = true;
                }
                if (this.model.columns[columnCount]["textAlign"] != undefined)
                    $innerTd.css("text-align", this.model.columns[columnCount]["textAlign"]);
                this.model.columns[columnCount]["allowEditing"] == false && $innerTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                if (this.model.columns[columnCount]["isPrimaryKey"] === true) {
                    $innerTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
                if (this.model.columns[columnCount]["isIdentity"] === true) {
                    $innerTd.find(".e-field").addClass("e-identity");
                    this._identityKeys.push($.trim(this.model.columns[columnCount].field));
                    this._identityKeys = $.unique(this._identityKeys);
                }
                var $col = $(document.createElement('col'));
                if (this.model.columns[columnCount]["priority"]) {
                    $innerTd.addClass("e-table-priority-" + this.model.columns[columnCount]["priority"]);
                    $col.addClass("e-table-priority-" + this.model.columns[columnCount]["priority"]);
                }
                if (this.model.columns[columnCount]["visible"] === false) {
                    $col.css("display", "none");
                    $innerTd.addClass("e-hide");
                }
                if (!ej.isNullOrUndefined(this.model.columns[columnCount]["cssClass"])) {
                    $innerTd.addClass(this.model.columns[columnCount]["cssClass"]);
                }
                !this.model.groupSettings.showGroupedColumn && $innerTd.addClass("{{for ~groupedColumns}}" +
                    " {{if #data == '" + this.model.columns[columnCount]["field"] + "'}}e-hide{{/if}}" +
                    "{{/for}}") && $col.css("display", "none");
                $colGroup.append($col);
                if (columnCount == this.model.scrollSettings.frozenColumns - 1) {
                    $innerTr = $tbodyClone.find("tr").last();
                    $colGroup = $tbodyClone.find("colgroup");
                    $.templates(this._id + "_JSONFrozenEditingTemplate", $tbody.html());
                    $tbody = $tbodyClone;
                }
            }
            $.templates(this._id + "_JSONEditingTemplate", $tbody.html());
        },

        addDialogEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $tbody = ej.buildTag('div');
            var $form = ej.buildTag('form.gridform', "", {}, { id: this._id + "EditForm" });
            var $table = ej.buildTag('table', "", {}, { cellspacing: "14px" });
            var $innerTr, $labelTd, $valueTd, trElement, tdElement;
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
                if (this.model.editSettings.editMode == "dialog") {
                    trElement = 'tr';
                    tdElement = 'td';
                }
                else trElement = tdElement = 'div';
                $innerTr = ej.buildTag(trElement);
                $labelTd = ej.buildTag(tdElement, "", { "text-align": "right" }).addClass("e-label");
                $valueTd = ej.buildTag(tdElement, "", { "text-align": "left" }).addClass("e-rowcell");
                 if (this.model.columns[columnCount]["priority"] && this.model.editSettings.editMode == "inlineform") 
                    $innerTr.addClass("e-table-priority-" + this.model.columns[columnCount]["priority"]);
                $innerTr.append($labelTd.get(0)).append($valueTd.get(0));
                if (this.model.columns[columnCount].headerText == undefined)
                    this.model.columns[columnCount].headerText = this.model.columns[columnCount].field;
                $labelTd.append("<label for='" + this.model.columns[columnCount].field + "'>" + this.model.columns[columnCount].headerText + "</label>");
                if (ej.isNullOrUndefined(this.model.columns[columnCount]["isUnbound"]) && ej.isNullOrUndefined(this.model.columns[columnCount]["template"]))
                    this._initCellEditType(columnCount, $valueTd);
                else if (this.model.columns[columnCount]["template"]) {
                    var helpers = {};
                    helpers["_" + this._id + "ColumnTemplating"] = ej.proxy(this._gridTemplate, null, this);
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + columnCount + "_Template").remove();
                    var scriptElement = this._createTemplateElement(this.model.columns[columnCount]);
                    $valueTd.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "')}}");
                } else if (this.model.columns[columnCount]["isUnbound"]) {
                    var helpers = {};
                    helpers["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                    $.views.helpers(helpers);
                    $("#" + this._id + this.model.columns[columnCount].headerText + "_UnboundTemplate").remove();
                    divElement = this._createUnboundElement(this.model.columns[columnCount]);
                    $valueTd.addClass("e-unboundcell").addClass("e-" + this.model.columns[columnCount]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                    this.model.scrollSettings.frozenColumns > 0 && $valueTd.addClass("e-frozenunbound");
                    this._isUnboundColumn = true;
                    $innerTr.addClass("e-hide");
                }
                this.model.columns[columnCount]["allowEditing"] == false && $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                if (this.model.columns[columnCount]["isIdentity"] === true) {
                    $valueTd.find(".e-field").addClass("e-identity");
                    this._identityKeys.push($.trim(this.model.columns[columnCount].field));
                    this._identityKeys = $.unique(this._identityKeys);
                }
                if (this.model.columns[columnCount]["visible"] === false)
                    $innerTr.addClass("e-hide");
                 if (!ej.isNullOrUndefined(this.model.columns[columnCount]["cssClass"])) {
                     $valueTd.addClass(this.model.columns[columnCount]["cssClass"]);
                }
                if (this.model.editSettings.editMode == "dialog") {
                    $form.append($table);
                    $table.append($innerTr);
                } else
                    $form.append($innerTr);
                $form.appendTo($tbody);
                if (this.model.columns[columnCount]["isPrimaryKey"] === true) {
                    $valueTd.find(".e-field").attr("disabled", true).addClass("e-disable");
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
            }
            if (this.model.editSettings.editMode == "dialog") $form.append($table);
            $tbody = this.renderDiaglogButton($form, $tbody);
            $.templates(this._id + "_JSONDialogEditingTemplate", $tbody.html());
        },
        _editEventTrigger: function (args) {
            if (args.requestType == "save" || args.requestType == "delete") {
                var params = {
                    data: args.data,
                    action: args.action !== undefined ? args.action : args.requestType,
                };
                if (!ej.isNullOrUndefined(args.foreignKeyData))
                    params.foreignKeyData = args.foreignKeyData;
				this._trigger("end" + params.action.charAt(0).toUpperCase() + params.action.slice(1), params);
            }
        },
        _compiledDropDownTemplate: function (valueField, textField, colType, format) {
            var helpers = { _gridFormatting: this.formatting };
            $.views.helpers(helpers);
            var $select = ej.buildTag('select');
            var $option = ej.buildTag("option", ej.isNullOrUndefined(textField) ? "{{:text}}" : colType == "date" && format != null ? "{{:~_gridFormatting('" + format + "'," + textField + ",'"+ this.model.locale +"')}}" : "{{:" + textField + "}}", {}, { value: ej.isNullOrUndefined(valueField) ? "{{:value}}" : "{{:" + valueField + "}}" });
            $select.append($option);
            return $.templates($select.html());
        },
        _initCellEditType: function (columnCount, element) {
            var fName = this.model.columns[columnCount].field;
            if (this.model.columns[columnCount]["foreignKeyValue"])
                this.model.columns[columnCount]["editType"] = "dropdownedit";
            if (this.model.columns[columnCount]["editTemplate"])
                this.model.columns[columnCount]["editType"] = "edittemplate";
            if (ej.isNullOrUndefined(this.model.columns[columnCount]["editType"]))
                if (this.model.columns[columnCount]["type"] == "date" || this.model.columns[columnCount]["type"] == "datetime")
                    this.model.columns[columnCount]["editType"] = this.model.columns[columnCount]["type"] + "picker";
                else
                    this.model.columns[columnCount]["editType"] = "stringedit";
            if (this.model.isResponsive)
                element.attr("data-cell", this.model.columns[columnCount]["headerText"]);
            var splits = (fName || "").split("."), sLen = splits.length - 1, braces = "";
            if (!ej.isNullOrUndefined(fName))
                fName = fName.replace(/[^a-z0-9\s_]/gi, '');
            while (sLen) {
                braces += "(";
                sLen--;
            }
            switch (this.model.columns[columnCount]["editType"]) {
                case "edittemplate":
                    var temp = this.model.columns[columnCount].editTemplate.create;
                    if (typeof temp == "string") {
                        var temp1 = ej.util.getObject(temp, window);
                        if (!$.isFunction(temp1)) {
                            if ($(temp).length == 1 && $(temp).get(0).tagName == "SCRIPT")
                                var $edittemplate = $($(temp).html()).attr({ id: this._id + fName, name: this.model.columns[columnCount].field });
                            else
                                var $edittemplate = $(temp).attr({ id: this._id + fName, name: this.model.columns[columnCount].field });
                        }
                        else
                            var $edittemplate = $(temp1()).attr({ id: this._id + fName, name: this.model.columns[columnCount].field });
                    }
                    else
                        var $edittemplate = $(temp()).attr({ id: this._id + fName, name: this.model.columns[columnCount].field });
                    element.append($edittemplate);
                    break;
                case "stringedit":
                    element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{html:" + braces + "#data['" + splits.join("'] || {})['") + "']}}", id: this._id + fName, name: this.model.columns[columnCount].field }));
                    break;
                case "booleanedit":
                    element.html('{{if #data["' + splits.join('"]["') + '"]}} <input class="e-field e-checkbox" type ="checkbox" id=' + this._id + fName + ' name=' + this.model.columns[columnCount].field + ' checked="checkbox"></input>{{else}} <input class="e-field e-checkbox" type ="checkbox" id=' + this._id + fName + ' name=' + this.model.columns[columnCount].field + ' > {{/if}}');
                    if (this.model.editSettings.editMode == "normal")
                        element.addClass("e-boolcell");
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, { type: "text", value: "{{:" + braces + "#data['" + splits.join("'] || {})['") + "']}}", id: this._id + fName, name: this.model.columns[columnCount].field });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + this.model.columns[columnCount]["editType"] + ' e-field', "", {}, { type: "text", value: "{{:" + braces + "#data['" + splits.join("'] || {})['") + "']}}", id: this._id + fName, name: this.model.columns[columnCount].field });
                    element.append($datePicker);
                    break;
                case "dropdownedit":
                    var currColumn = this.model.columns[columnCount];
                    if (ej.isNullOrUndefined(currColumn.dataSource)) {
                        var arrayOfDatas;                            
                            var query = new ej.Query().select(currColumn.field);
                        if (ej.isNullOrUndefined(this._dropDownManager) || (!this._dataManager.dataSource.offline && this._dataManager.dataSource.json))
                            arrayOfDatas = this._dataManager.executeLocal(query);
                        else{
                            if( this._dataManager.adaptor instanceof ej.JsonAdaptor && currColumn.field.indexOf('.') != -1 ){
                                var field = currColumn.field.replace(/\./g, ej.pvt.consts.complexPropertyMerge);
                                query = new ej.Query().select(field);
                            }
                            arrayOfDatas = this._dropDownManager.executeLocal(query);
                        }
                        var selectedItems = [];
                        var uniqueData = uniqueData = ej.dataUtil.mergeSort(ej.distinct(arrayOfDatas));
                        for (var index = 0; index < uniqueData.length; index++)
                            selectedItems.push({ text: uniqueData[index], value: uniqueData[index] });
                    }
                    else
                        selectedItems = currColumn.dataSource;
                    var dropDownTemplate;
                    var fieldName = ej.isNullOrUndefined(currColumn.foreignKeyField) ? currColumn.field : currColumn.foreignKeyField;
                    if (currColumn.foreignKeyValue)
                        dropDownTemplate = this._compiledDropDownTemplate(fieldName, currColumn.foreignKeyValue, currColumn.type, currColumn.format);
                    else
                        dropDownTemplate = this._compiledDropDownTemplate();
                    element.get(0).innerHTML = ["<select>", dropDownTemplate.render(selectedItems), "</select>"].join("");
                    element.find("select").prop({ id: this._id + fName, name: currColumn.field }).addClass("e-field e-dropdownlist");
                    break;
            }
        },
        addBatchEditTemplate: function () {
            if (this.model.columns.length == 0)
                return;
            var $outerDiv = ej.buildTag('div', "", { display: "none" }, { id: this._id + "_BulkEditTemplate" }), i, columnCount, $innerDiv;
            for (i = 0, columnCount = this.model.columns.length; i < columnCount; i++) {
                if (!this.model.columns[i]["template"] || !this.model.columns[i]["isUnbound"]) {
                    $innerDiv = ej.buildTag('div', "", {}, { id: this.model.columns[i].field + "_BulkEdit" });
                    this._initCellEditType(i, $innerDiv);
                    $outerDiv.append($innerDiv);
                }
                if (this.model.columns[i]["isPrimaryKey"] === true) {
                    this._primaryKeys.push($.trim(this.model.columns[i].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
                if (this.model.columns[i]["isIdentity"] === true) {
                    $innerDiv.find(".e-field").addClass("e-identity");
                    this._identityKeys.push($.trim(this.model.columns[i].field));
                    this._identityKeys = $.unique(this._identityKeys);
                }
            }
            if ($outerDiv.children().length)
                this._bulkEditTemplate = $outerDiv;

        },
        addExternalDialogEditingTemplate: function () {
            if (this.model.columns.length == 0)
                return;
			  var  $valueTd;
			   $valueTd = ej.buildTag('td', "", { "text-align": "left" }).addClass("e-rowcell");
            for (var columnCount = 0; columnCount < this.model.columns.length; columnCount++) {
				if (ej.isNullOrUndefined(this.model.columns[columnCount]["isUnbound"]) && ej.isNullOrUndefined(this.model.columns[columnCount]["template"]) && !ej.isNullOrUndefined(this.model.columns[columnCount].editTemplate) )
                    this._initCellEditType(columnCount, $valueTd);
                if (this.model.columns[columnCount]["isPrimaryKey"] === true) {
                    this._primaryKeys.push($.trim(this.model.columns[columnCount].field));
                    this._primaryKeys = $.unique(this._primaryKeys);
                }
            }
            var $tbody = ej.buildTag('div', "", { 'display': 'none' });
            var $form = ej.buildTag('form.gridform', "", {}, { id: this._id + "EditForm" });
            var cloneElement;
            if (this.model.editSettings.editMode == "dialogtemplate" && this.model.editSettings.dialogEditorTemplateID != null)
                cloneElement = this.model.editSettings.dialogEditorTemplateID;
            else if (this.model.editSettings.editMode == "externalformtemplate" && this.model.editSettings.externalFormTemplateID != null) {
                cloneElement = this.model.editSettings.externalFormTemplateID;
                $form.addClass("e-display");
            }
            else {
                cloneElement = this.model.editSettings.inlineFormTemplateID;
                $form.addClass("e-display");
            }

            $form.html($(cloneElement).html());
            $tbody = this.renderDiaglogButton($form, $tbody);
            $.templates(this._id + "_JSONdialogTemplateMode", $tbody.html());
        },
        _editdblClickHandler: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-rowcell")) {
                if (!this.model.isEdit || (this.model.editSettings.showAddNewRow && this.model.editSettings.editMode == "normal")) {
                    this._$currentTr = (this.model.scrollSettings.frozenColumns > 0 || this.model.scrollSettings.frozenRows > 0)
                        ? this.getRowByIndex($target.closest('tr').index())
                        : $target.closest('tr');
                    this.startEdit(this._$currentTr);
                }
            }
            return false;
        },
        _columnToSelect: function () {
            var column = [];
            for (var i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i]["editType"] === ej.Grid.EditingType.Dropdown && ej.isNullOrUndefined(this.model.columns[i]["dataSource"]))
                    column.push(this.model.columns[i].field);
            }
            if (column.length)
                return ej.Query().select(column);
            return ej.Query();
        },
        _renderExternalForm: function () {
            var $externalform = ej.buildTag("div", "", { display: "none" }, { id: this._id + "_externalEdit", 'class': "e-form-container" });
            var $eformHeader = ej.buildTag("div", "", "", { id: this._id + "_eFormHeader", 'class': "e-form-titlebar" });
            var $eformTitle = ej.buildTag("span", "", "", { 'class': "e-form-title" });
            var $eformToggleBtn = ej.buildTag("div", "", "", { id: this._id + "_eFormToggleBtn", 'class': "e-form-togglebtn" });
            var $eformToggleIcon = ej.buildTag("span", "", "", { 'class': "e-form-toggle-icon e-icon" });
            $eformToggleBtn.append($eformToggleIcon);
            $eformHeader.append($eformTitle).append($eformToggleBtn);

            var $eformContent = ej.buildTag("div", "", "", { id: this._id + "_eFormContent", 'class': "e-form-content" });
            var $eform = ej.buildTag("div", "", "", { id: this._id + "_externalForm", 'class': "e-externalform" });
            var $contentOuterDiv = ej.buildTag("div", "", "", { 'class': "e-externalformedit" });
            $eform.append($contentOuterDiv);
            $eformContent.append($eform);
            return $externalform.append($eformHeader).append($eformContent);;
        },
        _buttonClick: function (e) {
            if (e.type == "close") {
                if (!this.model.isEdit)
                    return;
                this.model.isEdit = false;
                this.element.ejGrid("cancelEdit");
                this.refreshToolbar();
                return;
            }
            if (e.keyCode !== undefined && e.keyCode != 13 || this.model == null)
                return true;
            if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                if (e.target.id == "EditDialog_" + this._id + "_Save") {
                    if (this.element.ejGrid("endEdit").length !== undefined && $("#" + this._id + "_dialogEdit").data("ejDialog"))
                        $("#" + this._id + "_dialogEdit").ejDialog("close");
                } else if (e.target.id == "EditDialog_" + this._id + "_Cancel") {
                    this.element.ejGrid("cancelEdit");
                    $("#" + this._id + "_dialogEdit").ejDialog("close");
                }
            }
            else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate") {
                if ($(e.target).hasClass("e-form-toggle-icon")) {
                    this.element.ejGrid("cancelEdit");
                    $("#" + this._id + "_externalEdit").css("display", "none");
                }
                else {
                    if (e.target.id == "EditExternalForm_" + this._id + "_Save") {
                        if (this.element.ejGrid("endEdit").length !== undefined)
                            $("#" + this._id + "_externalEdit").css("display", "none");
                    } else if (e.target.id == "EditExternalForm_" + this._id + "_Cancel") {
                        this.element.ejGrid("cancelEdit");
                        $("#" + this._id + "_externalEdit").css("display", "none");
                    }
                }
            }
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                if (e.target.id == "InlineEditForm_" + this._id + "_Save")
                    this.element.ejGrid("endEdit");
                else if (e.target.id == "InlineEditForm_" + this._id + "_Cancel")
                    this.element.ejGrid("cancelEdit");
            }
            else
                this.element.ejGrid("cancelEdit");
        },
        _enableEditingEvents: function () {
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                if (this.model.editSettings.allowEditing && this.model.editSettings.editMode != "batch" && this.model.editSettings.allowEditOnDblClick)
                    this._on(this.element, "dblclick", ".e-gridcontent", this._editdblClickHandler);
                else
                    this._off(this.element, "dblclick", ".e-gridcontent");
                this._off($("#" + this._id + "_dialogEdit"), "click keypress", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel");
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                    this._on($("#" + this._id + "_dialogEdit"), "click keypress", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel", this._buttonClick);
                }
                else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate") {
                    this._on($("#" + this._id + "_externalEdit"), "click keypress", "#EditExternalForm_" + this._id + "_Save ,#EditExternalForm_" + this._id + "_Cancel", this._buttonClick);
                    $(this.element).on("click", ".e-form-toggle-icon", $.proxy(this._buttonClick, this));
                }
                else if (this.model.editSettings.editMode == "batch")
                    this._on($(document), "mousedown", this._saveCellHandler);

                else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate")
                    $(this.element).on("click keypress", "#InlineEditForm_" + this._id + "_Save ,#InlineEditForm_" + this._id + "_Cancel", $.proxy(this._buttonClick, this));

            } else {
                this._off($("#" + this._id + "_dialogEdit"), "click", "#EditDialog_" + this._id + "_Save ,#EditDialog_" + this._id + "_Cancel");
                $(this.element).off("click", ".e-icon");
                this._off($("#" + this._id + "_externalEdit"), "click", "#EditExternalForm_" + this._id + "_Save ,#EditExternalForm_" + this._id + "_Cancel");
                $(this.element).off("click", ".e-form-toggle-icon");
                $(this.element).off("click", "#InlineEditForm_" + this._id + "_Save ,#InlineEditForm_" + this._id + "_Cancel");
            }
        },

        _deleteRow: function ($tr) {
            if (!this.model.editSettings.allowDeleting || (this.model.isEdit && this.model.editSettings.editMode != "batch"))
                return;
            if (this.model.editSettings.editMode == "batch")
                this._bulkDelete(this.getIndexByRow($tr));
            else {
                if (this._primaryKeys.length == 0 && !this.model.editSettings.allowEditing && !this.model.editSettings.allowAdding) {
                    for (i = 0; i < this.model.columns.length; i++) {
                        if (this.model.columns[i]["isPrimaryKey"] === true) {
                            this._primaryKeys.push($.trim(this.model.columns[i].field));
                            this._primaryKeys = $.unique(this._primaryKeys);
                        }
                    }
                }
                if (this._selectedRow() == -1 && ej.isNullOrUndefined($tr)) {
                    alert(this._getLocalizedLabels("DeleteOperationAlert"));
                    return;
                }
                if (ej.isNullOrUndefined($tr))
                    $tr = this.getRowByIndex(this._selectedRow());
                this._primaryKeyValues = [];
                for (var index = 0; index < this._primaryKeys.length; index++) {
                    var column = this.getColumnByField(this._primaryKeys[index]);
                    this._primaryKeyValues.push(this._currentJsonData[this.getIndexByRow($tr)][column.field]);
                }
                var deleteManager = ej.DataManager(this._currentJsonData);
                var query = new ej.Query();
                for (var i = 0; i < this._primaryKeys.length; i++)
                    query = query.where(this._primaryKeys[i], ej.FilterOperators.equal, this._primaryKeyValues[i]);
                currentData = deleteManager.executeLocal(query);
                var args = {};
                args.tr = $tr;
                args.data = currentData[0];
                var foreignKeyData = this._getForeignKeyData(args.data);
                if (!ej.isNullOrUndefined(foreignKeyData))
                    args.foreignKeyData = foreignKeyData;
                args.requestType = ej.Grid.Actions.Delete;
                if (this._trigger("actionBegin", args))
                    return true;
                this._cDeleteData = currentData;
                var promise;
                if (this._dataSource() instanceof ej.DataManager && (!this._dataManager.dataSource.offline && this._dataManager.dataSource.json !== undefined) || (this._dataSource().adaptor instanceof ej.remoteSaveAdaptor)) {
                    promise = this._dataManager.remove(this._primaryKeys[0], currentData[0][this._primaryKeys[0]]);
                    var proxy = this;
                    if ($.isFunction(promise.promise)) {
                        promise.done(function(e) {
                            proxy._processBindings(args);
                            proxy._primaryKeyValues = [];
                            proxy._cDeleteData = null;
                        });
                        promise.fail(function(e) {
                            args.error = e
                            proxy._trigger("actionFailure", args)
                        });
                    } else
                        this._processBindings(args);
                } else
                    this._processBindings(args);
                if (promise == undefined || !$.isFunction(promise.promise)) {
                    this._primaryKeyValues = [];
                    this._cDeleteData = null;
                }
            }
        },
        
        _htmlEncode: function (html) {
            var str =isNaN(html) ? html.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&apos;/g, '\'') : html;
            return str;
        },
        startEdit: function ($tr) {
		    if (!this.model.editSettings.allowEditing)
                return;
            if (this._selectedRow() == -1 && ej.isNullOrUndefined($tr)) {
                alert(this._getLocalizedLabels("EditOperationAlert"));
                return;
            }
            if (ej.isNullOrUndefined($tr)) {
                this._currentTrIndex = this._selectedRow();
                this._$currentTr = this.getRowByIndex(this._currentTrIndex);
            } else {
                this._currentTrIndex = $tr.index();
                this._$currentTr = $tr;
            }
            if (!$(this._$currentTr).is(":visible"))
                return false;
            this._primaryKeyValues = [];
            for (var index = 0; index < this._primaryKeys.length; index++) {
                var column = this.getColumnByField(this._primaryKeys[index]);
                this._primaryKeyValues.push(this._htmlEncode(this._currentJsonData[this.getIndexByRow(this._$currentTr)][column.field]));
            }
            var args = { row: this._$currentTr, rowIndex: this._currentTrIndex, primaryKey: this._primaryKeys, primaryKeyValue: this._primaryKeyValues };
            var cancel = this._trigger("beginEdit", args);
            if (cancel) {
                this._primaryKeyValues = [];
                return;
            }
            args.requestType = ej.Grid.Actions.BeginEdit;
            this._processBindings(args);

        },
         _startAdd: function() {
            if (!this.model.editSettings.allowAdding)
                return;
			this._isAddNew = true;
            if (this.model.editSettings.editMode == "batch")
                this._bulkAddRow();
            else {
                var cloneData = {};
                for (var i = 0; i < this.model.columns.length; i++) {
                    if (!ej.isNullOrUndefined(this.model.parentDetails) && this.model.columns[i].field == this.model.parentDetails.parentKeyField)
                        this.model.columns[i].defaultValue = this.model.parentDetails.parentKeyFieldValue;
                    cloneData[this.model.columns[i].field] = !ej.isNullOrUndefined(this.model.columns[i].defaultValue) ? this.model.columns[i].defaultValue : "";
                }
                var args = {}, complexObject = {};
                args.data = cloneData;
                if (this.model.editSettings.editMode.indexOf('template') != -1) {
                    for (var i = 0; i < this.model.columns.length; i++) {
                        if (!ej.isNullOrUndefined(this.model.columns[i].field) && this.model.columns[i].field.indexOf(".") != -1) {
                            var splits = this.model.columns[i].field.split('.');
                            ej.createObject(this.model.columns[i].field, args.data[this.model.columns[i].field], complexObject);
                            args.data[splits[0]] = complexObject;
                            delete args.data[this.model.columns[i].field];
                        }
                    }
                }
                var foreignKeyData = this._getForeignKeyData(args.data);
                if (!ej.isNullOrUndefined(foreignKeyData))
                    args.foreignKeyData = foreignKeyData;
                args.requestType = "add";
                this.clearSelection();
                var returnValue = this._processBindings(args);
                if (!returnValue)
                    this.model.editSettings.showAddNewRow ? this._selectedRow(-1) : this._selectedRow(0);
                var groupedColumns = this.model.groupSettings.groupedColumns.length;
                if (groupedColumns > 1) {
                    var $editCol = this.getContentTable().find(".e-addedrow").find("table").find("colgroup").children();
                    $($editCol.slice(0, groupedColumns - 1)).css('width', this.getHeaderTable().find('colgroup').children()[0].style.width);
                }
            }
        },
        
        endEdit: function () {
            if (this.model.isEdit) {
                var formElement, $formElement, editedTr;
                if (!this.editFormValidate())
                    return true;
                var obj = {};
                var editedRowWrap;
                if (this.model.editSettings.editMode == "batch")
                    this.saveCell();
                else {
					if(this.model.editSettings.showAddNewRow)
						editedTr = this.getContentTable().find(".e-editedrow");
                    formElement = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : !ej.isNullOrUndefined(editedTr) && editedTr.length == 1 ? editedTr[0].lastChild.lastChild: document.getElementById(this._id + "EditForm");
                    this._editForm = this.element.find(".gridform");
                    $formElement = $(formElement);
                    if (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate")
                        editedRowWrap = $formElement.closest('tr');
                    else
                        editedRowWrap = $formElement.closest('div');
                    formElement = this.model.scrollSettings.frozenColumns > 0 ? formElement[0] : formElement;
                    for (var index = 0; index < formElement.length; index++) {
                        if (editedRowWrap.hasClass("e-addedrow") && $(formElement[index]).hasClass("e-identity"))
                            continue;
                        var columnName = formElement[index].name, $element = $(formElement[index]);
						if($element.hasClass("e-dropdownlist e-input") && $element.attr("id").indexOf("_input") != -1)
							continue;
                        if (columnName != undefined) {
                            if (columnName == "") {
                                if (formElement[index].id.indexOf("Save") != -1 || formElement[index].id.indexOf("Cancel")!= -1)
                                    columnName = "";
                                else
                                    columnName = formElement[index].id.replace(this._id, "");
                            }
                            if (columnName != "" && obj[columnName] == null) {
                                var column = this.getColumnByField(columnName), value = formElement[index].value;
                                if (!ej.isNullOrUndefined(column) && (column.editType == "edittemplate")) {
                                    if (!ej.isNullOrUndefined(formElement[index + 1]) && formElement[index].type == 'hidden') index++;
                                    if (formElement[index].type != 'hidden') {
                                        var temp1 = column.editTemplate.read; $element = $(formElement[index]);
                                        if (typeof temp1 == "string")
                                            temp1 = ej.util.getObject(temp1, window);
                                        value = ej.isNullOrUndefined(column.foreignKeyField) ? temp1($element) : $element.ejDropDownList("getSelectedValue");
                                    }
                                }
                                else if ($(formElement[index]).hasClass("e-datepicker")) {
                                    value = $element.ejDatePicker("model.value");
                                }
                                else if ($(formElement[index]).hasClass("e-datetimepicker")) {
                                    value = $element.ejDateTimePicker("model.value");
                                }
                                else if ($element.is(".e-numerictextbox"))
                                    value = $element.ejNumericTextbox("getValue");
                                else if ($element.data("ejDropDownList")) {
                                    value = $element.ejDropDownList("getSelectedValue");
                                }
                                if (column == null)
                                    value = !isNaN(parseFloat(value)) && isFinite(value) ? parseFloat(value) : value;
                                else if (column.type == "number" && value.length)
                                    value = parseFloat(value.replace(/[\,\%]/g, ""));
                                var originalvalue;
                            if (formElement[index].type != "checkbox")
                                originalvalue = value;
                            else
                                originalvalue = $(formElement[index]).is(':checked');
                            if (columnName.indexOf(".") != -1)
                                ej.createObject(columnName, originalvalue, obj);
                            else
                                obj[columnName] = originalvalue;
                            }
                        }
                        if (index == formElement.length - 1 && $formElement.length > 1 && $formElement.index(formElement) == 0) {
                            formElement = $formElement[1];
                            index = -1;
                        }

                    }
                    var args = { data: obj };
                    var foreignKeyData = this._getForeignKeyData(args.data);
                    if (!ej.isNullOrUndefined(foreignKeyData))
                        args.foreignKeyData = foreignKeyData;
                    args.requestType = ej.Grid.Actions.Save;
                    args.selectedRow = this._selectedRow();
                    var currentData;
                    if (this._trigger("actionBegin", args))
                        return true;
                    if (editedRowWrap.hasClass("e-editedrow")) {
                        this._cModifiedData = obj;
                        args.action = "edit";
                    } else if (editedRowWrap.hasClass("e-addedrow")) {
                        this._cAddedRecord = obj;
                        args.action = "add";
                    }
                    this._updateAction(args);
                }
            }
        },
        _updateAction: function (args) {
            var promise;
            if (this._dataSource() instanceof ej.DataManager && (!this._dataManager.dataSource.offline && this._dataManager.dataSource.json !== undefined) || (this._dataSource().adaptor instanceof ej.remoteSaveAdaptor)) {
                if (!ej.isNullOrUndefined(this._cModifiedData))
                    promise = this._dataManager.update(this._primaryKeys[0], args.data);
                else
                    promise = this._dataManager.insert(args.data);
				this.model.isEdit = false;
                var proxy = this;
                if ($.isFunction(promise.promise)) {
                    promise.done(function (e) {
                        if (!ej.isNullOrUndefined(e) && $.isPlainObject(e.record)) {
                            args.data = e.record;
                            if (args.action == "add")
                                proxy._cAddedRecord = e.record;
                            if (args.action == "edit")
                                proxy._cModifiedData = e.record;
                        }
                        proxy._processBindings(args);
                        proxy._cModifiedData = null;
                        proxy._cAddedRecord = null;
                        proxy._primaryKeyValues = [];
                    });
                    promise.fail(function (e) {
                        args.error = (e && e.error) ? e.error : e;
                        proxy._trigger("actionFailure", args)
                    });
                } else
                    proxy._processBindings(args);
            } else
                this._processBindings(args);
            if (promise == undefined || !$.isFunction(promise.promise)) {
                this._cModifiedData = null;
                this._cAddedRecord = null;
                this._primaryKeyValues = [];
            }
        },
        
        cancelEdit: function () {
            var args = {};
            this._editForm = this.element.find(".gridform");
            args.requestType = ej.Grid.Actions.Cancel;
            this._processBindings(args);
            this._primaryKeyValues = [];
            this._currentData = null;
        },
        
        refreshToolbar: function () {
            var $toolbar = $("#" + this._id + "_toolbarItems");
            var lis = $toolbar.find("li");
            $toolbar.ejToolbar("enableItem", lis);
			var editedTr = this.getContentTable().find(".e-editedrow");
			if(this.model.editSettings.showAddNewRow && this.model.editSettings.editMode == "normal" && editedTr.length==0)
				this._disabledToolItems.push(lis.filter('[id='+this._id+'_add]'));
			else
			{
				for (var i = 0; i < lis.length; i++) {
					switch (lis[i].id) {
						case this._id + "_add":
						case this._id + "_edit":
						case this._id + "_delete":
					    case this._id + "_responsiveFilter":
					    case this._id + "_responsiveSorting":
					    case this._id + "_search":
					        if (this.model.isEdit) {
					            if (!(this.model.editSettings.showAddNewRow && this.model.editSettings.editMode == "normal") || editedTr.length != 0)
					                this._disableToolBarItems(lis[i]);
					        }
					        else if (!this.model.editSettings.allowAdding && lis[i].id == this._id + "_add")
					            this._disableToolBarItems(lis[i]);
					        else if (!this.model.editSettings.allowEditing && lis[i].id == this._id + "_edit")
					            this._disableToolBarItems(lis[i]);
					        else if (!this.model.editSettings.allowDeleting && lis[i].id == this._id + "_delete")
					            this._disableToolBarItems(lis[i]);
					        break;
					    case this._id + "_update":
						case this._id + "_cancel":
						    if (!this.model.isEdit) {
						        $(lis[i]).hasClass("e-hover") && $(lis[i]).removeClass("e-hover");
						        this._disabledToolItems.push(lis[i]);
						    }
							break;
					}
				}
			}
            $toolbar.ejToolbar("disableItem", this._disabledToolItems);
            $toolbar.ejToolbar("model.enableRTL", this.model.enableRTL);
            this._disabledToolItems = $();

        },
        _disableToolBarItems: function (element) {
            $(element).hasClass("e-hover") && $(element).removeClass("e-hover");
            this._disabledToolItems.push(element);
        },
        _getHiddenCount: function (elements) {
            var count = 0;
            for (var i = 0; i < elements.length; i++) {
                if (elements.eq(i).hasClass("e-hide"))
                    count++;
            }
            return count;
        },
        _refreshTemplateCell: function (temp, data) {
            var tempcell = temp.find('.e-templatecell');
			for(var i =0; i< tempcell.length;i++){						
				var args = { cell: tempcell[i], data: data, column: this.model.columns[$(tempcell[i]).index()], rowIndex: temp.index()};
				this._trigger("templateRefresh", args);	
			}			
        },
        _edit: function (args) {
            var editingManager = ej.DataManager(this._currentJsonData);
            var queryManager = new ej.Query();
            if (this.model.allowFiltering)
                this._previousFilterCount = this._filteredRecordsCount;
            for (var index = 0; index < this._primaryKeys.length; index++)
                queryManager = queryManager.where(this._primaryKeys[index], ej.FilterOperators.equal, this._primaryKeyValues[index]);
            this._currentData = editingManager.executeLocal(queryManager);
            var temp = document.createElement('div');
            var $temp = $(temp), $tempSecondTR;
            if (this.model.editSettings.editMode == "normal") {
                temp.innerHTML = ['<table>', $.render[this._id + "_JSONEditingTemplate"](this._currentData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                var $tr = $temp.find("tr").first(), detailCount = 0, $tempFirstTR, firstHidden = this.model.columns.length
                , $currentTrFr = args.row.first();
                if (this.model.scrollSettings.frozenColumns > 0) {
                    $temp.prepend(['<table>', $.render[this._id + "_JSONFrozenEditingTemplate"](this._currentData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join(""));
                    $tr.splice(0, 0, $temp.find("table").first().find("tr").first().get(0));
                    $currentTrLa = args.row.last();
                    $tempLastTR = $tr.last();
                }
                $tempFirstTR = $temp.find("tr").first();
                $temp.find('td').not(".e-rowcell").addClass("e-editcell e-normaledit");
                this._setEditDropdownValue($temp);
                if (this.model.groupSettings.groupedColumns.length >= 2) {
                    var $indentCell = args.row.find("td.e-indentcell");
                    $temp.find("tr").first().prepend($indentCell);
                }
                if (this.model.detailsTemplate != null || this.model.childGrid != null) {
                    detailCount++;                    
                    $temp.find(".e-editcell").find("tr").prepend(args.row.find("[class^=e-detailrow]").removeClass("e-selectionbackground e-active"));
                    if (this.model.gridLines != "both")
                        $temp.find(".e-editcell .e-rowcell:first").addClass("e-detailrowvisible");
                }
                if (this.model.scrollSettings.frozenColumns > 0) {
                    $temp.find(".e-editcell").get(1).colSpan = this.model.columns.length - this.model.scrollSettings.frozenColumns - args.row.last().find(".e-hide").length + detailCount;
                    firstHidden = this.model.scrollSettings.frozenColumns;
                    $currentTrLa.hasClass("e-alt_row") && $tempLastTR.addClass("e-alt_row")
                }
                $temp.find(".e-editcell").get(0).colSpan = firstHidden - $currentTrFr.find("td").not(":visible").length + detailCount;
                $currentTrFr.hasClass("e-alt_row") && $tempFirstTR.addClass("e-alt_row");                
                $currentTrFr.empty().replaceWith($tempFirstTR.addClass("e-editedrow"));
                if (this.model.scrollSettings.frozenColumns > 0)
                    $currentTrLa.empty().replaceWith($tempLastTR.addClass("e-editedrow"));
                this._refreshUnboundTemplate($tr.find(".gridform"));
                this._gridRows = $(this.getContentTable().get(0).rows).toArray();
                if (this.model.scrollSettings.frozenColumns > 0)
                    this.getScrollObject().scrollY(this.getScrollObject().model.scrollTop, true);
                if (this.model.scrollSettings.frozenColumns > 0) 
                    this._gridRows = [this._gridRows, $(this.getContentTable().get(1).rows).toArray()];
            }
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                var detailCount = 0;
                temp.innerHTML = this.model.editSettings.editMode == "inlineform" ? $.render[this._id + "_JSONDialogEditingTemplate"](this._currentData) : $.render[this._id + "_JSONdialogTemplateMode"](this._currentData);
				this._setEditDropdownValue($temp);
                var tr = ej.buildTag('tr');
                var td = ej.buildTag('td');
                tr.addClass("e-editedrow");
                td.addClass("e-inlineformedit");
                td.html($(temp).children());
                tr.append(td);
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var $trClone = tr.clone();
                    $trClone.find("td").empty().prop("colspan", this.model.scrollSettings.frozenColumns);
                    args.row.eq(1).after(tr).end().eq(0).after($trClone);
                    this._gridRows = [this._gridRows, this.getContentTable().last().find("tr").toArray()];
                }
                else
                    args.row.after(tr);
                this._gridRows = this.getContentTable().first().find(".e-rowcell,.e-inlineformedit").closest("tr").toArray();
				if (this.model.detailsTemplate != null || this.model.childGrid != null)
                        detailCount++;
                if (this.model.scrollSettings.frozenColumns > 0)
                    td.prop("colspan", this.model.columns.length - this.model.scrollSettings.frozenColumns - tr.find("form").children().not(":visible").length - hideCount + detailCount);
                else
                    td.prop("colspan", this.model.columns.length - this._hiddenColumns.length - hideCount + detailCount);
                if (this.model.scrollSettings.frozenColumns > 0)
                    this._gridRows = [this._gridRows, this.getContentTable().last().find("tr").toArray()];
                $("#" + this._id + "_inlineFormTitle").text(this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]]);
                args.row.find("input").attr('disabled', 'disabled').addClass("e-disable");
            }
            else {
                $temp.addClass("e-editedrow");
                temp.innerHTML = this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "externalform" ? $.render[this._id + "_JSONDialogEditingTemplate"](this._currentData) : $.render[this._id + "_JSONdialogTemplateMode"](this._currentData);
                this._setEditDropdownValue($temp);
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                    $("#" + this._id + "_dialogEdit").html($(temp));
                    var model = {};
                    model.cssClass = this.model.cssClass;
                    model.enableRTL = this.model.enableRTL;
                    model.width = "auto";
                    model.enableResize = this.phoneMode;
                    model.close = $.proxy(this._buttonClick, this);
                    model.content = "#" + this._id;
                    model.enableModal = true;
                    model.allowKeyboardNavigation = false;
                    model.title = this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]];
                    $("#" + this._id + "_dialogEdit").ejDialog(model);
                    $("#" + this._id + "_dialogEdit").ejDialog("open");
                }
                else {
                    $("#" + this._id + "_externalEdit").css("display", "block").css('z-index', this._maxZindex() + 1);
                    $(".e-externalformedit").html($(temp));
                    $("#" + this._id + "_eFormHeader").find(".e-form-title").text(this._getLocalizedLabels("EditFormTitle") + this._currentData[0][this._primaryKeys[0]]);
                    this._externalFormPosition();
                    args.row.find("input").attr('disabled', 'disabled').addClass("e-disable");
                }
            }
			if(!ej.isNullOrUndefined(this.model.templateRefresh) && $tempFirstTR.find(".e-templatecell").length != 0) 
				this._refreshTemplateCell($tempFirstTR, this.model.currentViewData[$tempFirstTR.index()]);
        },
        _setEditDropdownValue: function ($temp) {
            var $select = $temp.find("select.e-field"),x;
            for (var i = 0; i < $select.length; i++) {
                var ddlTemplate = {}, opPara = "";
                if ($select[i].name.indexOf('.') != -1) {
                    for (var j = 1; j < $select[i].name.split(".").length; j++)
                        opPara = opPara.concat("(");
                    ddlTemplate[this._id + "ddlTemp"] = "{{:" + opPara + "#data['" + $select[i].name.split('.').join("'] || {})['") + "']}}";
                }
                else
                    ddlTemplate[this._id + "ddlTemp"] = "{{:" + $select[i].name.replace(/[^a-z0-9\s]/gi, '') + "}}"
                $.templates(ddlTemplate);
			    x = $.render[this._id + "ddlTemp"](this._currentData);
				$temp.find("select option[value='"+x+"']").attr("selected","selected");
                $select.eq(i).val(x);
            }
        },
        _add: function (args) {
            var temp = document.createElement('div');
            if (this._isLocalData && this.model.groupSettings.groupedColumns.length == 0 && this.model.scrollSettings.frozenColumns == 0 && this.model.scrollSettings.frozenRows == 0)
                !(this._dataSource() instanceof ej.DataManager) ? this._dataSource().splice(0, 1) : this._dataSource().dataSource.json.splice(0, 1);
            this._previousFilterCount = this._filteredRecordsCount;
            if (this.model.editSettings.editMode == "normal") {
                var $tempFirstTR, $temp = $(temp), frozenColSpan = this.model.columns.length;
                temp.innerHTML = ['<table>', $.render[this._id + "_JSONEditingTemplate"](args.data, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                var $select = $(temp).find('select.e-field');
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(args.data[$select[i].name]);
                if (this.model.scrollSettings.frozenColumns > 0) {
                    $tempLastTR = $temp.find("table").first().find("tr").first();
                    $temp.prepend(['<table>', $.render[this._id + "_JSONFrozenEditingTemplate"](args.data, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join(""));
					$($tempLastTR).find("td").first().addClass("e-editcell");
                }
                $tempFirstTR = $temp.find("tr").first();
                var td = $(temp).find(".e-editcell").get(0);
                $(temp).find('td').first().addClass("e-editcell");
                if (this.model.allowPaging && this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0 && !this.model.editSettings.showAddNewRow)
                    this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                if ((this.model.detailsTemplate != null || this.model.childGrid != null) && $(this.getContentTable().get(0).lastChild.lastChild).children('.e-detailrowexpand').length)
                    this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                if (this.model.currentViewData.length == 0 || this.getContentTable().find('td.e-rowcell').length == 0)
                    this.getContentTable().find('tr').first().replaceWith($(temp).find("tr").first().addClass("e-addedrow e-normaledit"));
                else {
					var  $contentTbody = this.getContentTable().first().find('tbody').first();
                    if (this.model.editSettings.rowPosition == "top")
                        $contentTbody.prepend($tempFirstTR.addClass("e-addedrow e-normaledit"));
                    else if (this.model.editSettings.rowPosition == "bottom")
                        $contentTbody.append($tempFirstTR.addClass("e-addedrow e-normaledit"));
                    if (this.model.scrollSettings.frozenColumns > 0)
                        this.getContentTable().last().find('tbody').first().prepend($tempLastTR.addClass("e-addedrow e-normaledit"));
                }
                var $editTr = this.getContentTable().find("tr.e-addedrow");
                if (this.model.detailsTemplate != null || this.model.childGrid != null)
                    $editTr.find('tr').first().prepend(ej.buildTag('td.e-detailrowcollapse'));
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                if (this.model.groupSettings.groupedColumns.length) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                        if (i == 0)
                            $editTr.prepend(ej.buildTag("td.e-indentcell"));
                        else {
                            $editTr.find("tr").prepend(ej.buildTag("td.e-indentcell"));
                        }
                    }
                    if (this.model.groupSettings.groupedColumns.length >= 2)
                        $editTr.find("td.e-editcell").prop("colspan", (this.model.columns.length + (this.model.groupSettings.groupedColumns.length - 1) - this._hiddenColumns.length - hideCount));
                    else
                        $editTr.find("td.e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount));
                } else if (this.model.detailsTemplate != null || this.model.childGrid != null)
                    $editTr.find(".e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount + 1));
                else {
                    if (this.model.scrollSettings.frozenColumns > 0) {
                        $editTr.find(".e-editcell").last().prop("colspan", (this.model.columns.length - this.model.scrollSettings.frozenColumns - $tempLastTR.find("td").not(":visible").length - hideCount));
                        frozenColSpan = this.model.scrollSettings.frozenColumns;
                    }
                    $editTr.find(".e-editcell").first().prop("colspan", (frozenColSpan - $tempFirstTR.find("td").not(":visible").length - hideCount));

                }
                if (this.getBrowserDetails().browser == "msie" && this.model.editSettings.rowPosition == "bottom")
					this._colgroupRefresh();
                this._refreshUnboundTemplate($editTr.find(".gridform"));
                this._gridRows = this.getContentTable().first().find(".e-rowcell").closest("tr").toArray();
                if (this.model.scrollSettings.frozenColumns > 0)
                    this._gridRows = [this._gridRows, this.getContentTable().last().find(".e-rowcell").closest("tr").toArray()];
            }
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                temp.innerHTML = this.model.editSettings.editMode == "inlineform" ? $.render[this._id + "_JSONDialogEditingTemplate"](args.data) : $.render[this._id + "_JSONdialogTemplateMode"](args.data);
                var $select = $(temp).find('select.e-field');
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(args.data[$select[i].name]);
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                var detailCount = 0;
                var tr = ej.buildTag('tr');
                var td = ej.buildTag('td');
                tr.addClass("e-addedrow");
                td.addClass("e-inlineformedit e-editcell");
                td.html($(temp).children());
                tr.append(td);
                var hideCount = !this.model.groupSettings.showGroupedColumn ? this.model.groupSettings.groupedColumns.length : 0;
                if (this.model.groupSettings.groupedColumns.length) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                        tr.prepend(ej.buildTag("td.e-indentcell"));
                    }
                } else
                    tr.find("td.e-editcell").prop("colspan", (this.model.columns.length - this._hiddenColumns.length - hideCount));
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var $trClone = tr.clone(), $divs = td.find(".gridform").children();
                    $trClone.find("td").empty().prop("colSpan", this.model.scrollSettings.frozenColumns - this._getHiddenCount($divs.slice(0, this.model.scrollSettings.frozenColumns)));
                    td.prop("colSpan", this.model.columns.length - this.model.scrollSettings.frozenColumns - this._getHiddenCount($divs.slice(this.model.scrollSettings.frozenColumns)));
                    this.getContentTable().first().find('tbody').first().prepend($trClone);
                    this.getContentTable().last().find('tbody').first().prepend(tr);
                }
                else {
					if (this.model.detailsTemplate != null || this.model.childGrid != null)
                        detailCount++;
                    td.prop("colspan", this.model.columns.length - this._hiddenColumns.length - hideCount + detailCount);
                    if (this.model.currentViewData.length == 0 || this.getContentTable().find('td.e-rowcell').length == 0)
                        this.getContentTable().find('tr').first().replaceWith($(tr));
                    else
                        if (this.model.allowPaging && this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0)
                            this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                    if (this.model.editSettings.rowPosition == "top")
                        this.getContentTable().first().find('tbody').first().prepend(tr);
                    else if (this.model.editSettings.rowPosition == "bottom")
                        this.getContentTable().first().find('tbody').first().append(tr);
                }
                
                if ((this.model.detailsTemplate != null || this.model.childGrid != null) && $(this.getContentTable().get(0).lastChild.lastChild).children('.e-detailrowexpand').length)
                    this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                $("#" + this._id + "_inlineFormTitle").text(this._getLocalizedLabels("AddFormTitle"));
                this._refreshUnboundTemplate($("#" + this._id + "EditForm"));
                this._gridRows = this.getContentTable().first().find(".e-rowcell,.e-inlineformedit").closest("tr").toArray();
                if (this.model.scrollSettings.frozenColumns > 0)
                    this._gridRows = [this._gridRows, this.getContentTable().last().find(".e-rowcell").closest("tr").toArray()];
            }
            else {
                $(temp).addClass("e-addedrow");
                temp.innerHTML = this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "externalform" ? $.render[this._id + "_JSONDialogEditingTemplate"](args.data) : $.render[this._id + "_JSONdialogTemplateMode"](args.data);
                var $select = $(temp).find('select.e-field');
                for (var i = 0; i < $select.length; i++)
                    $select.eq(i).val(args.data[$select[i].name]);
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate") {
                    $("#" + this._id + "_dialogEdit").html($(temp));
                    var model = {};
                    model.cssClass = this.model.cssClass;
                    model.width = "auto";
                    model.enableResize = this.phoneMode;
                    model.content = "#" + this._id;
                    model.enableModal = true;
                    model.close = $.proxy(this._buttonClick, this);
                    model.enableRTL = this.model.enableRTL;
                    model.allowKeyboardNavigation = false;
                    model.title = this._getLocalizedLabels("AddFormTitle");
                    $("#" + this._id + "_dialogEdit").ejDialog(model);
                    $("#" + this._id + "_dialogEdit").ejDialog("open");
                }
                else {
                    $("#" + this._id + "_externalEdit").css("display", "block").css('z-index', this._maxZindex() + 1);
                    $(".e-externalformedit").html($(temp));
                    $("#" + this._id + "_eFormHeader").find(".e-form-title").text(this._getLocalizedLabels("AddFormTitle"));
                    this._externalFormPosition();
                }
            }
			if(!ej.isNullOrUndefined(this.model.templateRefresh) && $editTr.find(".e-templatecell").length != 0) 
				this._refreshTemplateCell($editTr, args.data);
            if (this.model.allowPaging) {
                if (this.model.filterSettings.filteredColumns.length)
                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
                else
                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
                this._refreshGridPager();
            }
        },
        editFormValidate: function () {
            if ($.isFunction($.validator)) {
                if (this.model.scrollSettings.frozenColumns > 0) {
                    var forms = this.element.find(".gridform");
                    return forms.length > 1 ? forms.eq(0).validate().form() && forms.eq(1).validate().form() : forms.validate().form();
                }
                return $("#" + this._id + "EditForm").validate().form();
            }
            return true;
        },
        _refreshAltRow: function () {
            var $gridRows = this._excludeDetailRows();
            for (var r = 0; r < $gridRows.length; r++) {
                var $row = $($gridRows[r]);
                $row.hasClass("e-alt_row") && $row.removeClass("e-alt_row");
                if ( r % 2 != 0)
                    $row.addClass("e-alt_row");
            }
        },
        _editCompleteAction: function (args) {
            var $form = this.element.find(".gridform");
            this.model.isEdit = true;
            var $cols1 = this.getContentTable().children("colgroup").find("col");
            var width = this.element.width()
            this.setWidthToColumns();
            if (ej.Grid.Actions.Add == args.requestType) {
                var disabledElements = $form.find(".e-field:disabled");
                for (var j = 0; j < disabledElements.length; j++) {
                    var fieldName = $(disabledElements[j]).attr("name");
                    if (!$(disabledElements[j]).hasClass("e-identity"))
                        if ($.inArray(fieldName, this._disabledEditableColumns) == -1 || $.inArray(fieldName, this._primaryKeys) !== -1)
                            $(disabledElements[j]).removeAttr("disabled").removeClass("e-disable");
                }
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length - 1; i++)
                    $form.find("colgroup").prepend(this._getIndentCol());
            }
            if (this._tdsOffsetWidth.length == 0 || this.model.groupSettings.groupedColumns.length)
                this._setoffsetWidth();
            this._refreshEditForm(args);
            if (this.model.scrollSettings.frozenColumns > 0 && (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate")) {
                if (args.requestType == "beginedit")
                    args.row.eq(0).next().find("td").height(args.row.eq(1).next().find("td").height());
                else
                    this.getContentTable().first().find("tr").first().find("td").height(this.getContentTable().last().find("tr").first().find("td").height());
            }
            if (this.model.scrollSettings.frozenRows > 0 && args.requestType == "beginedit")
                this._initFrozenRows();
            if ($.isFunction($.validator))
                this.initValidator();
        },
        _refreshEditForm: function (args) {
			var editedTr; 
			if(this.model.editSettings.showAddNewRow)
				editedTr = this.getContentTable().find(".e-editedrow");
            var form = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : !ej.isNullOrUndefined(editedTr) && editedTr.length == 1 ? editedTr[0].lastChild.lastChild : document.getElementById(this._id + "EditForm");
            var elementFocused = false, columnIndex, matchMedia;
            if (this.model.enableResponsiveRow && $.isFunction(window.matchMedia))
                matchMedia = window.matchMedia("(max-width: 320px)");
            var $formElement = $(form).find("input,select,div.e-field"), percent = 86;
            if (this._isUnboundColumn || this.getContentTable().find(".e-templatecell") != null)
                $formElement = $formElement.filter(function () { return (!$(this).closest(".e-rowcell").hasClass("e-unboundcell") && !$(this).closest(".e-rowcell").hasClass("e-templatecell")) })
            for (var i = 0; i < $formElement.length; i++) {
                var $element = $formElement.eq(i);
                var inputWidth, column = this.getColumnByField(!ej.isNullOrUndefined($element.prop("name")) ? $element.prop("name") : $element.attr("name"));
                if (column != null)
                    columnIndex = $.inArray(column, this.model.columns);
                if (this.model.editSettings.editMode == "batch") {
                    percent = 95;
                }
                else if (this.model.editSettings.editMode == "normal")
                    percent = 96;
                if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                    $element.parent().css("width", ej.dataUtil.max(this._tdsOffsetWidth) + "px");
                    inputWidth = ej.max(this._tdsOffsetWidth) * (percent / 100);
                }
                else
                    inputWidth = this._tdsOffsetWidth[i] * (percent / 100);
                if (column !=null && columnIndex !== undefined && (columnIndex < this.model.columns.length && this.model.columns[columnIndex].editTemplate)) {
                    var temp = { rowdata: this.model.editSettings.editMode == "batch" ? this._batchEditRowData : this._currentJsonData[this._bulkEditCellDetails.rowIndex], column: this.model.columns, element: $element };
                    var temp1 = this.model.columns[columnIndex].editTemplate.write;
                    if (!ej.isNullOrUndefined(args) && args.requestType == "add") temp.rowdata = {};
                    if (typeof temp1 == "string")
                        temp1 = ej.util.getObject(temp1, window);
                    temp1(temp);
                }
                else if ($element.hasClass("e-numerictextbox")) {
                    var params = { width: inputWidth }, value = $element.val(), customParams = this.getColumnByField($element.prop("name"));
                    if ((!ej.isNullOrUndefined(matchMedia) && matchMedia.matches) || customParams["width"] && typeof customParams["width"] == "string" && customParams["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    params.enableRTL = this.model.enableRTL;
                    params.showSpinButton = true;
                    params.cssClass = this.model.cssClass;
                    if (value.length)
                        params.value = parseFloat(value);
                    if ($element.hasClass("e-disable"))
                        params.enabled = false;
                    if (!ej.isNullOrUndefined(customParams["editParams"]))
                        $.extend(params, customParams["editParams"]);
                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(this._id, ""));
                } else if ($element.hasClass("e-datepicker")) {
                    var params = { width: inputWidth }, column = this.getColumnByField($element.prop("name"));
                    if ((!ej.isNullOrUndefined(matchMedia) && matchMedia.matches) || column["width"] && typeof column["width"] == "string" && column["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    params.enableRTL = this.model.enableRTL;
                    params.cssClass = this.model.cssClass;
                    params.displayDefaultDate = true;
                    params.showPopupButton = false;
                    params.watermarkText = this._getLocalizedLabels("DatePickerWaterMark");
                    if ($element.val().length)
                        params.value = new Date($element.val());
                    if ($element.hasClass("e-disable"))
                        params.enabled = false;
                    if (column["format"] !== undefined && column.format.length > 0) {
                        var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        var formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDatePicker(params);
					if(this.model.editSettings.editMode == "batch")
                        $element.ejDatePicker("show");
				}
                else if ($element.hasClass("e-datetimepicker")) {
                    var column = this.getColumnByField($element.prop("name")),
                        params = {
                            width: inputWidth,
                            enableRTL: this.model.enableRTL,
                            cssClass: this.model.cssClass,
                            showPopupButton: false
                        };
                    if ((!ej.isNullOrUndefined(matchMedia) && matchMedia.matches) || column["width"] && typeof column["width"] == "string" && column["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    if ($element.val().length)
                        params.value = new Date($element.val());
                    if ($element.hasClass("e-disable"))
                        params.enabled = false;
                    if (column["format"] !== undefined && column.format.length > 0) {
                        var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        var formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDateTimePicker(params);
					if(this.model.editSettings.editMode == "batch")
                        $element.ejDateTimePicker("show");
                }
                else if ($element.hasClass("e-dropdownlist")) {
                    var column = this.getColumnByField($element.prop("name")),
                        params = {
                            width: inputWidth,
                            enableRTL: this.model.enableRTL,
                            enableIncrementalSearch: true
                        };
                    if ((!ej.isNullOrUndefined(matchMedia) && matchMedia.matches) || column["width"] && typeof column["width"] == "string" && column["width"].indexOf("%") != -1 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "batch"))
                        params.width = "100%";
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);
                    $element.ejDropDownList(params);
                    $element.ejDropDownList("setSelectedValue", args.requestType == "add" ? $element.val("") : $element.val());
                    if ($element.hasClass("e-disable"))
                        $element.ejDropDownList("disable");
                }
				 else if ($element.hasClass("e-save e-button") || $element.hasClass("e-cancel e-button") )
					 $element.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, width: "100","text-align":"centre",height:"35px"});
                    //else if ($element.hasClass("checkbox"))
                    //{
                    //    var value = $element.prop("checked");
                    //    $element.ejCheckBox({
                    //        id: $element[0].id,
                    //        checked: value
                    //    });
                    //}
                else {
                    switch ($element.prop('tagName')) {
                        case "INPUT":
                            if (!ej.isNullOrUndefined(column) && column.format && $element.val() != "") {
                                switch (column.type) {
                                    case ("date" || "datetime"):
                                        var value = $element.val();
                                        var date = new Date(value);
                                        var format = column.format.replace("{0:", "").replace("}", "");
                                        var newformat = Globalize.format(date, format);
                                        $element.val(newformat);
                                        break;
                                    case "number":
                                        var value = $element.val();
                                        var format = new RegExp("\\{0(:([^\\}]+))?\\}", "gm").exec(column.format);
                                        format[2].toLowerCase().split("")[0] == "c" ? format[2] = format[2].toLowerCase().replace("c", "n") : format[2];
                                        $element.val(Globalize.format(parseFloat(value), format[2]));
                                        break;
                                }
                            }
                            if ($element.attr("type") != "checkbox") {
                                $element.css("text-align", $element.attr("name") != null && this.getColumnByField($element.attr("name")) != null ?
                                this.getColumnByField($element.attr("name")).textAlign : "center");
                                if (this.model.editSettings.editMode == "batch"){
                                    $element.css('width', '100%').css("height", "28px");
									if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version, 10) == 8)
									   $element.css("line-height", "22px");
							    }
                                else if (this.model.editSettings.editMode == "normal"){
                                    $element.css('width', '100%').css("height", "30px");
									if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version, 10) == 8)
									   $element.css("line-height", "24px");
								}	
                                else{
                                    $element.outerWidth(inputWidth).height(28);
									if (ej.browserInfo().name == "msie" && parseInt(ej.browserInfo().version, 10) == 8)
									   $element.css("line-height", "26px");
								}

                            }
                            else
                                $element.width(inputWidth > 0 ? ($element.width() > inputWidth ? inputWidth : $element.width()) : 1);
								if(this.model.editSettings.editMode == "batch")
									$element.is(':checked') ? $element.prop("checked",false) : $element.prop("checked",true);
                            break;
                        case "SELECT":
                            $element.width(inputWidth).height(28);
                            break;
                    }
                }
                if (column != null && !column.visible && column.validationRules && !(this.model.editSettings.editMode == "dialogtemplate" || this.model.editSettings.editMode == "externalformtemplate" || this.model.editSettings.editMode == "inlineformtemplate")){
					if(column.editType == ej.Grid.EditingType.Dropdown)
						$element.closest(".e-rowcell").find("input").addClass("e-hide");					
					$element.addClass("e-hide");
				}
                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof ($element.data("ejDropDownList") || $element.data("ejNumericTextbox")) == "object")) {
                    this._focusElements($element.closest('td'));
                    elementFocused = true;
                }
            }

        },
        _focusElements: function ($currentCell) {
            if ($currentCell.length) {
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length) {
                    if ($childElem.is(".e-ddl"))
                        $($childElem[0]).focus();
                    else
                        $childElem.find(".e-ddl").focus();
                }
                else
                    $childElem.find('input:visible,select').first().select().focus();
            }
        },
        _renderToolBar: function () {
            var $div = ej.buildTag('div.e-gridtoolbar', "", {}, { id: this._id + "_toolbarItems" });
            var $ul = ej.buildTag("ul");
            (!ej.isNullOrUndefined(this.model.toolbarSettings.toolbarItems) && this.model.toolbarSettings.toolbarItems.length) && this._renderLi($ul);
            $div.append($ul);
            var $customUl = ej.buildTag("ul");
            $div.append($customUl);
            (!ej.isNullOrUndefined(this.model.toolbarSettings.customToolbarItems) && this.model.toolbarSettings.customToolbarItems.length) && this._renderCustomLi($customUl);
            var model = {};
            model.click = this._toolBarClick;
            model.cssClass = this.model.cssClass;
            model.enableRTL = this.model.enableRTL;
            model.enableSeprator = false;
            $div.ejToolbar(model);
            $div.ejToolbar("disableItem", this._disabledToolItems);
            this._disabledToolItems = $();
            return $div;
        },
        _renderCustomLi: function ($ul) {
            var $li; var customToolbar;
            for (var i = 0; i < this.model.toolbarSettings.customToolbarItems.length; i++) {
                customToolbar = this.model.toolbarSettings.customToolbarItems[i]["templateID"] ? this.model.toolbarSettings.customToolbarItems[i]["templateID"].replace("#", "") : this.model.toolbarSettings.customToolbarItems[i];
                $li = ej.buildTag("li", "", {}, { id: this._id + "_" + customToolbar, title: customToolbar });
                switch (typeof this.model.toolbarSettings.customToolbarItems[i]) {
                    case "string":
                        var $item = ej.buildTag("a.e-toolbaricons e-icon", "", {}).addClass(this.model.toolbarSettings.customToolbarItems[i]);
                        break;
                    case "object":
                        $li.attr("title", this.model.toolbarSettings.customToolbarItems[i]["templateID"].replace("#", ""));
                        var $item = $(this.model.toolbarSettings.customToolbarItems[i]["templateID"]).hide().html();
                        break;
                }
                $li.html($item);
                $ul.append($li);
            }
        },
        _renderLi: function ($ul) {
            if ($.isFunction(window.matchMedia)) {
                if (this.model.enableResponsiveRow && window.matchMedia("(max-width: 320px)").matches) {
                    var searchIndex = this.model.toolbarSettings.toolbarItems.indexOf('search');
                    searchIndex != -1 && this.model.toolbarSettings.toolbarItems.splice(searchIndex, 1);
                    if (this.model.allowFiltering)
                        this.model.toolbarSettings.toolbarItems.push('responsiveFilter');
                    if (this.model.allowSorting) {
                        this.model.toolbarSettings.toolbarItems.push('responsiveSorting');
                    }
                    searchIndex != -1 && this.model.toolbarSettings.toolbarItems.push('search');
                }
            }
            for (var i = 0; i < this.model.toolbarSettings.toolbarItems.length; i++) {
                var $li = ej.buildTag("li", "", {}, { id: this._id + "_" + this.model.toolbarSettings.toolbarItems[i], title: this._getLocalizedLabels(this.model.toolbarSettings.toolbarItems[i].slice(0, 1).toUpperCase() + this.model.toolbarSettings.toolbarItems[i].slice(1)), "tabindex": "0" });
                this._renderLiContent($li, this.model.toolbarSettings.toolbarItems[i]);
                $ul.append($li);
            }
        },
        _renderLiContent: function ($li, item) {
            var $a, $input;
            switch (item) {
                case "add":
                    $a = ej.buildTag("a.e-addnewitem e-toolbaricons e-icon e-addnew", "", {});
                    break;
                case "edit":
                    $a = ej.buildTag("a.e-edititem e-toolbaricons e-icon e-edit", "", {});
                    break;
                case "delete":
                    $a = ej.buildTag("a.e-deleteitem e-toolbaricons e-icon e-delete", "", {});
                    break;
                case "update":
                    $a = ej.buildTag("a.e-saveitem e-toolbaricons e-disabletool e-icon e-save", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;
                case "cancel":
                    $a = ej.buildTag("a.e-cancel e-toolbaricons e-disabletool e-icon e-gcancel", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;
				
                case "search":
                    $a = ej.buildTag("a.e-searchitem e-toolbaricons e-disabletool e-icon e-searchfind", "", {});
                    if (!this.model.enableResponsiveRow) {
                        $input = ej.buildTag("input.e-ejinputtext", "", {}, { type: "text" });
                        $li.append($input);
                        if (!ej.isNullOrUndefined(this.model.searchSettings.key))
                            $input.val(this.model.searchSettings.key);
                    }
                    this.model.allowSearching = true;
                    break;
				case "printGrid":
					 $a = ej.buildTag("a.e-print e-toolbaricons e-icon", "", {});
					 break;
                case "excelExport":
                    $a = ej.buildTag("a.e-toolbaricons e-excelIcon e-icon", "", {});
                    break;
                case "wordExport":
                    $a = ej.buildTag("a.e-toolbaricons e-wordIcon e-icon", "", {});
                    break;
                case "pdfExport":
                    $a = ej.buildTag("a.e-toolbaricons e-pdfIcon e-icon", "", {});
                    break;
                case "responsiveFilter":
                    $a = ej.buildTag("a.e-toolbaricons e-filterset e-icon", "", {});
                    break;
                case "responsiveSorting":
                    $a = ej.buildTag("a.e-toolbaricons e-respponsiveSorting e-icon", "", {});
                    break;
            }
            $li.append($a);

        },
        _toolBarClick: function (Sender) {
            var $gridEle = $(this.itemsContainer).closest(".e-grid"), gridInstance = $gridEle.ejGrid("instance"), gridId = $gridEle.attr('id');
            if (Sender.event == undefined && Sender.target.tagName == "INPUT" && Sender.currentTarget.id == gridId + "_search")
                return;
            $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            var currentTarget = Sender.currentTarget; var target = Sender.target;
            var args = { itemName: currentTarget.title, itemId: currentTarget.id, currentTarget: currentTarget, target: target, itemIndex: $(currentTarget).index(), itemCurrentTarget: currentTarget.outerHTML, gridModel:gridInstance.model, itemTarget: target.outerHTML, toolbarData: Sender };
            if ($gridEle.ejGrid("instance")._trigger("toolbarClick", args))
                return;
            switch (args.itemId) {
                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add");
                    break;
                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit");
                    break;
                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete");
                    break;
                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update");
                    break;
                case gridId + "_cancel":
                    if (gridInstance.model.editSettings.editMode == "batch")
                        gridInstance._confirmDialog.find(".e-content").html(gridInstance._getLocalizedLabels("CancelEdit")).end().ejDialog("open");
                    else
                        gridInstance._toolbarOperation(gridId + "_cancel");
                    break;
                case gridId + "_search":
                    if (gridInstance.model.enableResponsiveRow) {
                        if (ej.isNullOrUndefined(gridInstance.element.find('.e-responsesearch')[0])) {
                            var $div = ej.buildTag('div.e-gridtoolbar', "", {}, { id: this._id + "_toolbarItems" });
                            var $ul = ej.buildTag('div.e-responsesearch', '', { 'width': '95%', 'height': '38px', 'margin-top': '7px', 'margin-left': '6px' });
                            var $span = ej.buildTag('span.e-ttoltxt', '', { width: '98%', 'margin-left': '2%' }, { id: gridInstance._id + "_search" });
							$a = ej.buildTag("span.e-searchitem e-toolbaricons e-disabletool e-icon e-searchfind", "", { 'position': 'absolute', 'right': '2%', 'margin-top': '1%' });
                            if (ej.browserInfo().name === "webkit")
                                $a.css("margin-top", "-2px");
                            var $input = ej.buildTag("input.e-ejinputtext", "", { width: '97%', 'height': '30px' }, { type: "search", id: gridInstance._id + "_searchInput" });
                            if (!ej.isNullOrUndefined(gridInstance.model.searchSettings.key))
                                $input.val(gridInstance.model.searchSettings.key);
                            $span.append($input);
                            $span.append($a);
                            $ul.append($span);
                            $div.append($ul);
                            $div.ejToolbar({
                                click: function (sender) {
                                    gridInstance._toolbarOperation(gridId + "_search", $(sender.currentTarget).find("input").val());
                                }
                            });
                            $input.bind('keyup', function (e) {
                                if ($input.val() != '') {
                                    $a.removeClass('e-searchfind');
                                    $a.addClass('e-cancel')
                                }
                                else {
                                    $a.removeClass('e-cancel');
                                    $a.addClass('e-searchfind');
                                }
                            });
                            $a.click(function () {
                                if ($a.hasClass('e-cancel')) {
                                    $input.val('');
                                    $a.removeClass('e-cancel');
                                    $a.addClass('e-searchfind');
                                    gridInstance._toolbarOperation(gridId + "_search", $("#" + gridId + "_searchInput").val());
                                }
                            })
                            $div.insertBefore(gridInstance.getHeaderContent());
                        }
                        else {
                            if (gridInstance.element.find('.e-responsesearch').css('display') == 'block')
                                gridInstance.element.find('.e-responsesearch').css('display', 'none');
                            else
                                gridInstance.element.find('.e-responsesearch').css('display', 'block');
                        }
                    }
                    else
                        gridInstance._toolbarOperation(gridId + "_search", $(Sender.currentTarget).find("input").val());
                    break;
				case gridId + "_printGrid":
                    gridInstance._toolbarOperation(gridId + "_printGrid");
                    break;
                case gridId + "_excelExport":
                    gridInstance._toolbarOperation(gridId + "_excelExport");
                    break;
                case gridId + "_wordExport":
                    gridInstance._toolbarOperation(gridId + "_wordExport");
                    break;
                case gridId + "_pdfExport":
                    gridInstance._toolbarOperation(gridId + "_pdfExport");
                    break;
                case gridId + "_responsiveFilter":
                    gridInstance._toolbarOperation(gridId + "_responsiveFilter");
                    break;
                case gridId + "_responsiveSorting":
                    gridInstance._toolbarOperation(gridId + "_responsiveSorting");
                    break;
            }
            return false;
        },

        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejGrid("instance"), batchEnable = gridObject.model.editSettings.editMode == "batch", gridId = $gridEle.attr('id'), fieldName;
            gridObject._exportTo = gridObject["export"];
            switch (operation) {
                case gridId + "_add":
                    if (batchEnable)
                        gridObject._bulkAddRow();
                    else
                        gridObject._startAdd();
                    break;
                case gridId + "_edit":
                    if (batchEnable && gridObject.model.editSettings.allowEditing) {
					    if (gridObject._bulkEditCellDetails.columnIndex == -1) {
                            alert(this._getLocalizedLabels("EditOperationAlert"));
                            return;
                        }
                        fieldName = gridObject.model.columns[gridObject._bulkEditCellDetails.columnIndex].field;
                        fieldName && gridObject.editCell(gridObject._bulkEditCellDetails.rowIndex, fieldName);
                    }
                    else
                        gridObject.startEdit();
                    break;
                case gridId + "_delete":
                    if (this._selectedRow() == -1) {
                        alert(this._getLocalizedLabels("DeleteOperationAlert"));
                        return;
                    }
                    if (this.model.editSettings.showDeleteConfirmDialog)
                        this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("ConfirmDelete")).end().ejDialog("open");
                    else
                        batchEnable ? this._bulkDelete() : this._deleteRow();
                    break;
                case gridId + "_update":
                    if (batchEnable && $("#" + this._id + "EditForm").children().find(".e-field-validation-error").length == 0)
                        this.model.editSettings.showConfirmDialog ? this._confirmDialog.find(".e-content").html(this._getLocalizedLabels("BatchSaveConfirm")).end().ejDialog("open") : this.batchSave();
                    else
                        gridObject.endEdit();
                    break;
                case gridId + "_cancel":
                    if (batchEnable) {
                        if ($("#" + gridId + "ConfirmDialog").ejDialog("isOpened") === true)
                            this._triggerConfirm();
                        else
                            gridObject.cancelEditCell();
                    }
                    else
                        gridObject.cancelEdit();
                    break;
                case gridId + "_search":
                    $gridEle.ejGrid("search", searchEle);
                    break;
				case gridId + "_printGrid":
                    this.print();
                    break;
                case gridId + "_excelExport":
                    gridObject._exportTo(gridObject.model.exportToExcelAction, 'excelExporting', gridObject.model.allowMultipleExporting);
                    break;
                case gridId + "_wordExport":
                    gridObject._exportTo(gridObject.model.exportToWordAction, 'wordExporting', gridObject.model.allowMultipleExporting);
                    break;
                case gridId + "_pdfExport":
                    gridObject._exportTo(gridObject.model.exportToPdfAction, 'pdfExporting', gridObject.model.allowMultipleExporting);
                    break;
                case gridId + "_responsiveFilter":
                    $("#responsiveFilter").css('display', 'block');
                    setTimeout(function () { gridObject.element.css('display', 'none'), 0 });
                    break;
                case gridId + "_responsiveSorting":
                    this._sortColumns = []; this._removeSortCol = []
                    for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                        this._sortColumns.push({ field: this.model.sortSettings.sortedColumns[i].field, direction: this.model.sortSettings.sortedColumns[i].direction });
                    }
                    if (ej.isNullOrUndefined($("#responsiveSort")[0])) {
                        var $dlg = $("#responsiveFilter").clone().css('display', 'block');
                        $dlg.insertAfter(this.element);
                        $dlg.attr('id', 'responsiveSort');
                        gridObject._setSortingButton();
                        var $btnDiv = ej.buildTag('div.btnContainer', '', { width: '100%', 'bottom': '0px', 'position': 'absolute' });
                        var $inputOk = ej.buildTag('input.e-resposnsiveFilterBtnLeft', 'OK', { 'width': '45.6%' });
                        var $inputCancel = ej.buildTag('input.e-resposnsiveFilterBtnRight', 'Cancel', { 'width': '46%' });
                        var $closeIcon = ej.buildTag('div.e-resFIlterRigthIcon');
                        var $cspanIcon = ej.buildTag('span.e-icon e-responsiveClose e-resIcon');
                        $dlg.find('.e-resFilterleftIcon').remove();
                        $dlg.find('.labelRes').text('Sorting');
                        $dlg.find('.e-resFilterDialogHeaderDiv').append($closeIcon.append($cspanIcon))
                        $dlg.find('.e-resFilterDialogHeaderDiv').find('.e-resFIlterRigthIcon').click(function (e) {
                            $("#responsiveSort").css('display', 'none');
                            gridObject.element.css('display', 'block');
                        })
                        var $divIcon = ej.buildTag('div.e-resFilterleftIcon', '', { 'margin-top': '3%' });
                        var $spanIcon = ej.buildTag('span.e-icon e-resIcon e-responsiveSortClear');
                        $divIcon.click(function () {
                            $dlg.find('.e-responsivefilterColDiv').find('.e-button').remove();
                            gridObject._setSortingButton(true);
                            for (var i = 0; i < gridObject._sortColumns.length; i++) {
                                if (gridObject._removeSortCol.indexOf(gridObject._sortColumns[i].field) == -1)
                                    gridObject._removeSortCol.push(gridObject._sortColumns[i].field);
                            }
                            gridObject._sortColumns = [];
							gridObject._removeSortCol = [];
                        });
                        $dlg.find('.e-resFilterDialogHeaderDiv').prepend($divIcon.append($spanIcon));
                        $dlg.append($btnDiv);
                        $btnDiv.append($inputOk).append($inputCancel);
                        $inputOk.ejButton({
                            text: 'OK', type: 'button',
                            click: $.proxy(this._resSortOperation, this)
                        })
                        $inputCancel.ejButton({
                            text: 'Cancel', type: 'button',
                            click: function () {
                                $("#responsiveSort").css('display', 'none');
                                gridObject.element.css('display', 'block');
                                gridObject._sortColumns = [];
                                $dlg.find('.e-responsivefilterColDiv').find('.e-button').remove();
                                gridObject._setSortingButton();
                            }
                        })
                    }
                    else {
                        $("#responsiveSort").find('.e-responsivefilterColDiv').find('.e-button').remove();
                        this._setSortingButton();
                    }
                    $("#responsiveSort").find('.e-responsivefilterColDiv').find('.e-filternone').remove();
                    $("#responsiveSort").css('display', 'block');
                    setTimeout(function () { gridObject.element.css('display', 'none'), 0 });
                    break;
            }
            return false;
        },
        _resSortOperation: function (sender) {
            var rCol=[];
			for(var i=0; i< this.model.sortSettings.sortedColumns.length;i++)
				rCol.push(this.model.sortSettings.sortedColumns[i].field);
            for (var i = 0; i < rCol.length; i++)
                this.removeSortedColumns(rCol[i]);
            for (var i = 0 ; i < this._sortColumns.length; i++) {
                if (this.model.allowMultiSorting)
                    this.multiSortRequest = true;
                this.sortColumn(this._sortColumns[i].field, this._sortColumns[i].direction);
            }
            $("#responsiveSort").css('display', 'none');
            this.element.css('display', 'block');
        },
        _setSortingButton: function (clear) {
            var $sortDiv = $("#responsiveSort");
            var gridObj = this;
            this._sortCols = [];
            if (ej.isNullOrUndefined($sortDiv.find('.e-responsivefilterColDiv').find('.e-button')[0])) {
                $sortDiv.find('.e-responsivefilterColDiv').each(function (index, object) {
                    var $btnDiv = ej.buildTag('div', '', { 'float': 'right', 'margin-right': '2%', 'margin-top': '-1%' })
                    var fieldName = $(object).attr('ej-MappingName');
                    var $but = ej.buildTag('button#' + fieldName + ".e-sortingBtn", '');
                    $(object).append($btnDiv.append($but));
                    var btnText = 'None', icon = '';
                    if (!clear) {
                        for (var sortC = 0; sortC < gridObj.model.sortSettings.sortedColumns.length; sortC++) {
                            if (gridObj.model.sortSettings.sortedColumns[sortC].field == fieldName) {
                                btnText = gridObj.model.sortSettings.sortedColumns[sortC].direction == 'ascending' ? 'Ascending' : 'Descending';
                                icon = btnText == 'Ascending' ? 'e-resIcon e-respponsiveSortingAsc' : 'e-resIcon e-respponsiveSortingDesc';
                            }
                        }
                        if (icon != '') {
                            $but.ejButton({
                                text: btnText, type: 'button',
                                height: 28,
                                width: 120,
                                cssClass: 'e-resSortIconBtn',
                                id: fieldName,
                                prefixIcon: icon,
                                imagePosition: "imageright",
                                contentType: "textandimage",
                                showRoundedCorner: true,
                                click: $.proxy(gridObj._resSortButClick, gridObj)
                            })
                        }
                        else {
                            $but.ejButton({
                                text: btnText, type: 'button',
                                height: 28,
                                cssClass: 'e-resSortIconBtn',
                                width: 120,
                                id: fieldName,
                                showRoundedCorner: true,
                                click: $.proxy(gridObj._resSortButClick, gridObj)
                            })
                        }
                    }
                    else {
                        $but.ejButton({
                            text: btnText, type: 'button',
                            height: 28,
                            width: 120,
                            id: fieldName,
                            showRoundedCorner: true,
                            click: $.proxy(gridObj._resSortButClick, gridObj)
                        })
                    }
                });
            }
        },
        _sortOperation: function (field, direction) {
            if (this._removeSortCol.indexOf(field) != -1) {
                this._sortColumns.splice(this._removeSortCol.indexOf(field), 0);
				this._removeSortCol.splice(this._removeSortCol.indexOf(field), 0);
            }
            for (var column = 0; column < this._sortColumns.length; column++) {
                if (this._sortColumns[column]["field"] == field)
                    break;
            }
            if (this.model.allowMultiSorting) {
                this.multiSortRequest = true;
                if (column != -1) {
                    this._sortColumns.splice(column, 1);
					if (this._removeSortCol.indexOf(field) == -1)
						this._removeSortCol.push(field);
                }
            }
            else {
                $("#responsiveFilter").find('.e-responsivefilterColDiv').find('.e-button').removeClass('e-disable');
                var $divColg = $("#responsiveFilter").find('.e-responsivefilterColDiv').not(".e-responsivefilterColDiv[ej-mappingname='" + field + "']");
                var $btn = $divColg.find('.e-button').addClass('e-disable');
                $btn.text('None');
                if (this._sortColumns.length > 0) {
                    this._removeSortCol.push(this._sortColumns[0].field);
                    this._sortColumns = [];
                }
            }
            this._sortColumns.push({ field: field, direction: direction });
        },
        _resSortButClick: function (sender) {
            var text = '', prefixIcon = '', fieldName = sender.model.id;
            var obj = $("#" + sender.model.id).ejButton('instance');
            if (sender.model.text == 'None') {
                text = 'Ascending';
                prefixIcon = 'e-resIcon e-respponsiveSortingAsc';
                this._sortOperation(fieldName, 'ascending');
            }
            else if (sender.model.text == 'Ascending') {
                text = 'Descending';
                prefixIcon = 'e-resIcon e-respponsiveSortingDesc';
                this._sortOperation(fieldName, 'descending');
            }
            else {
                obj.model.text = 'None';
                obj.model.prefixIcon = '';
                obj.type = 'button';
                obj.model.contentType = "text";
                obj._render();
                for (var column = 0; column < this._sortColumns.length; column++) {
                    if (this._sortColumns[column]["field"] == fieldName)
                        break;
                }
                this._removeSortCol.push(fieldName);
                this._sortColumns.splice(column, 1);
                $("#responsiveFilter").find('.e-responsivefilterColDiv').find('.e-button').removeClass('e-disable');
                return;
            }
            obj.model.text = text; obj.model.prefixIcon = 'e-resIcon ' + prefixIcon; obj.model.imagePosition = "imageright";
            obj.model.contentType = "textandimage"; obj._render();
        },
        renderDiaglogButton: function (form, tbody) {
            var btnId;
            if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                btnId = "EditExternalForm_";
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                btnId = "InlineEditForm_";
                var inlineTitleBar = ej.buildTag("div", "", "", { id: this._id + "_inlineFormTitle", 'class': "e-inlineform-titlebar" });
                inlineTitleBar.appendTo(tbody);
                    }
					
            else
                btnId = "EditDialog_";
            var savebtn = ej.buildTag('input.e-save', "", { 'margin-left': '30px' }, { type: "button", id: btnId + this._id + "_Save" });
            savebtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: this._getLocalizedLabels("SaveButton"), width: "100" });
            var cancelbtn = ej.buildTag('input.e-cancel', "", { 'margin-left': '19px', 'margin-right': '13px' }, { type: "button", id: btnId + this._id + "_Cancel" });
            cancelbtn.ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, text: this._getLocalizedLabels("CancelButton"), width: "100" });
            var btnDiv = (this.model.editSettings.editMode != "dialog" && this.model.editSettings.editMode != "dialogtemplate") ? ej.buildTag('div', "", "", { 'class': "e-editform-btn" }) : ej.buildTag('div');
            btnDiv.append(savebtn);
            btnDiv.append(cancelbtn);
            form.appendTo(tbody);
            if (this.model.editSettings.editMode != "dialog" && this.model.editSettings.editMode != "dialogtemplate")
                btnDiv.appendTo(tbody);
            else
                form.append(btnDiv);
            return tbody;
        },
        _externalFormPosition: function () {
            var pos = $(this.element).offset();
            var width = $(this.element).width();
            var height = $(this.element).height();
            var DivElement = $("#" + this._id + "_externalEdit");
            switch (this.model.editSettings.formPosition) {
                case "topright":
                    $(DivElement).find('.e-form-toggle-icon').removeClass('e-bottomleft').addClass('e-topright');
                    $(DivElement).css({ "left": (pos.left + width + 1) + "px", "top": pos.top + "px", "position": "absolute", "width": "290px" });
                    $("#" + this._id + "_eFormContent").height("auto");
                    break;
                case "bottomleft":
                    $(DivElement).find('.e-form-toggle-icon').removeClass('e-topright').addClass('e-bottomleft');
                    $(DivElement).css({ "left": (pos.left) + "px", "top": (pos.top + height + 1) + "px" });
                    $("#" + this._id + "_eFormContent").width("100%");
                    break;
            }
        },
        _setoffsetWidth: function () {
            var tds, $form = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : $("#" + this._id + "EditForm");
            if (this._gridRecordsCount == 0 && this.model.editSettings.editMode != "batch")
                return;
            if (this.model.editSettings.editMode == "batch")
                tds = $form.closest("td");
            else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate" || this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                tds = $form.find(".e-rowcell").not(".e-unboundcell,.e-templatecell");
            else
                tds = $form.find("tr").find(".e-rowcell").not(".e-unboundcell,.e-templatecell");
            for (var i = 0; i < tds.length; i++)
                this._tdsOffsetWidth[i] = tds.get(i).offsetWidth;
        },
        _bulkChangesAcquired: function () {
            if (this.batchChanges.added.length > 0 || this.batchChanges.changed.length || this.batchChanges.deleted.length)
                return true;
            return false;
        },
        _renderDialog: function () {
            var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogEdit" });
            return $dialog;
        },
        
        getCurrentEditCellData: function () {
            if (this.model.isEdit && $("#" + this._id + "EditForm").length) {
                var $element = $("#" + this._id + this._bulkEditCellDetails.fieldName), cellValue;
                switch (this._bulkEditCellDetails.cellEditType) {
                    case ej.Grid.EditingType.String:
					case ej.Grid.EditingType.Numeric:
                        cellValue = $element.val();
                        break;                                          
                    case ej.Grid.EditingType.Dropdown:
                        cellValue = this._bulkEditCellDetails.isForeignKey ? { "value": $element.ejDropDownList("getSelectedValue"), "text": $element.ejDropDownList("getValue") } : $element.ejDropDownList("getSelectedValue");
                        break;
                    case ej.Grid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;
                    case ej.Grid.EditingType.DatePicker:
                        cellValue = $element.ejDatePicker("model.value");
                        break;
                    case ej.Grid.EditingType.DateTimePicker:
                        cellValue = $element.ejDateTimePicker("model.value");
                        break;
                }
                if (typeof cellValue == "string" && cellValue.length && this.model.columns[this._bulkEditCellDetails.columnIndex].type == "number")
                    cellValue = parseFloat(cellValue);
                return cellValue;
            }
            return null;
        },
        cancelEditCell: function () {
            if (this.model.isEdit) {
                var tr = this.getRows()[this._bulkEditCellDetails.rowIndex], cellData = {}, cell;
                cellData[this._bulkEditCellDetails.fieldName] = this._bulkEditCellDetails.cellValue;
                if ($(tr).hasClass("e-insertedrow"))
                    cell = tr.cells[this._bulkEditCellDetails.columnIndex + this.model.groupSettings.groupedColumns.length];
                else
                    cell = tr.cells[this._bulkEditCellDetails.columnIndex];
                $(cell).removeClass("e-editedbatchcell").empty().html($($.templates[this._id + "_JSONTemplate"].render(cellData))[0].cells[this._bulkEditCellDetails.columnIndex].innerHTML);
                this.model.isEdit = false;
                this.element.focus();
            }
        },
        saveCell: function (preventSaveEvent) {
            if (this.model.isEdit) {
                if (!this.editFormValidate())
                    return true;
                var $form = $("#" + this._id + "EditForm"), $targetTR = $form.closest("tr"), $targetTD = $form.closest("td"), $toolBar, tempVal, formattedValue
                , args = {}, column = this.model.columns[this._bulkEditCellDetails.columnIndex], $element = $("#" + this._id + this._bulkEditCellDetails.fieldName);
                args = {
                    columnName: column.field,
                    value: this.getCurrentEditCellData(),
                    rowData: this._bulkEditCellDetails.rowData,
                    previousValue: this._bulkEditCellDetails.cellValue,
                    columnObject: column,
                    cell: $targetTD,
                    isForeignKey: this._bulkEditCellDetails.isForeignKey
                };
                if (!preventSaveEvent && this._trigger("cellSave", args)) {
                    this._focusElements(args.cell);
                    this._bulkEditCellDetails.cancelSave = true;
                    return;
                }
                if (this._bulkEditCellDetails.cellEditType == "datetimepicker" || this._bulkEditCellDetails.cellEditType == "dropdownedit" || this._bulkEditCellDetails.cellEditType == "datepicker")
                    $element[$element.data("ejWidgets")[0]]("destroy");
                if (!ej.isNullOrUndefined(column.format)) {
                    formattedValue = this.formatting(column.format, args.isForeignKey ?
                                              (!isNaN(parseFloat(args.value.text)) && isFinite(args.value.text)
                                              ? parseFloat(args.value.text)
                                              : args.value.text) : args.value);
                    args.cell.empty().html(formattedValue);
                }
                if (this._bulkEditCellDetails.cellEditType == "edittemplate") {
                    var temp1 = column.editTemplate.read;
                    if (typeof temp1 == "string")
                        temp1 = ej.util.getObject(temp1, window);
                    args.value = args.isForeignKey ? { "value": $element.ejDropDownList("getSelectedValue"), "text": $element.ejDropDownList("getValue") } : temp1($element);
                    args.rowData[this._bulkEditCellDetails["fieldName"]] = args.value;
                    args.cell.empty().html(args.isForeignKey ? args.value.text : args.value);
                }
                else {
                    if (args.columnObject.type == "boolean" || args.columnObject.editType == "booleanedit") {
                        var cellData = {};
                        cellData[args.columnObject.field] = args.value;
                        args.cell.empty().html($($.templates[this._id + "_JSONTemplate"].render(cellData))[0].cells[this._bulkEditCellDetails.columnIndex].innerHTML);
						args.cell.removeClass("e-boolrowcell");
					}
					else if(args.columnObject.editType == "datepicker" || args.columnObject.editType == "datetimepicker")
						args.cell.empty().html(formattedValue);
                    else {
                        if (ej.isNullOrUndefined(column.format))
                            args.cell.empty().html(args.isForeignKey ? args.value.text : args.value)
                    }
                }
                tempVal = args.isForeignKey ? args.value.value : args.value;
                var isValueModified = ((this._bulkEditCellDetails.cellEditType == "datepicker" || this._bulkEditCellDetails.cellEditType == "datetimepicker")
										 && tempVal instanceof Date && args.previousValue instanceof Date) ? (tempVal.getTime() !== args.previousValue.getTime()) : (tempVal != args.previousValue);
                this.model.isEdit = false;
                if (isValueModified) {
                    this._enableSaveCancel();
                    args.cell.addClass("e-updatedtd e-icon e-gupdatenotify");
                    if (typeof args.rowData[this._bulkEditCellDetails["fieldName"]] == "string" && args.rowData[this._bulkEditCellDetails["fieldName"]].length)
                        args.rowData[this._bulkEditCellDetails["fieldName"]] = args.isForeignKey ? args.value.value.toString() : args.value.toString();
                    else
                        args.rowData[this._bulkEditCellDetails["fieldName"]] = args.isForeignKey ? (!isNaN(parseInt(args.value.value)) ? parseInt(args.value.value) : args.value.value) : args.value;
                    if ($.inArray(args.rowData, this.batchChanges.changed) == -1 && $.inArray(args.rowData, this.batchChanges.added) == -1)
                        this.batchChanges.changed.push(args.rowData);
                }
                $targetTR.removeClass("e-editedrow").removeClass("e-batchrow");
                args.cell.removeClass("e-editedbatchcell");
            }
        },
        _enableSaveCancel: function () {
            if (this.model.toolbarSettings.showToolbar) {
                $toolBar = this.element.find("#" + this._id + "_toolbarItems");
                $toolBar.ejToolbar("enableItemByID", this._id + "_update");
                $toolBar.ejToolbar("enableItemByID", this._id + "_cancel");
            }
        },

        setCellValue: function (index, fieldName, cellValue) {
            var data = this.getDataByIndex(index), tr = this.getRows()[index], dataIndex
                , columnIndex = this.getColumnIndexByField(fieldName);
            data[fieldName] = cellValue;
            if ($(tr).hasClass("e-insertedrow"))
                tr.cells[columnIndex + this.model.groupSettings.groupedColumns.length].innerHTML = cellValue;
            else {
                $.inArray(data, this.batchChanges.changed) == -1 && this.batchChanges.changed.push(data);
                tr.cells[columnIndex].innerHTML = cellValue;
            }
            this._enableSaveCancel();
        },
        setDefaultData: function (defaultData) {
            if (ej.isNullOrUndefined(defaultData)) {
                var fieldNames = [];
				for (var column = 0; column < this.model.columns.length; column++) 
					fieldNames.push(this.model.columns[column]["field"]);
                defaultData = $.extend(true, {}, this._bulkEditCellDetails._data[0]);
                for (var field in defaultData) {
                    var index = $.inArray(field, fieldNames);
                    index == -1 && delete defaultData[field];
                    if (defaultData.hasOwnProperty(field)) {
                        var isChild = !ej.isNullOrUndefined(this.model.parentDetails) ? this.model.parentDetails.parentKeyField : "";
                        if (field == isChild)
                            this.model.columns[index].defaultValue = this.model.parentDetails.parentKeyFieldValue;
                        if (!ej.isNullOrUndefined(this.model.columns[index]["defaultValue"])) {
                            defaultData[field] = this.model.columns[index]["defaultValue"];
                        }
                        else {
                            switch (typeof defaultData[field]) {
                                case "number":
                                    defaultData[field] = 0;
                                    break;
                                case "string":
                                    defaultData[field] = "";
                                    break;
                                case "boolean":
                                    defaultData[field] = false;
                                    break;
                                case "object":
                                    if ($.isArray(defaultData[field]))
                                        defaultData[field] = new Array();
                                    else
                                        defaultData[field] = "";
                            }
                        }
                    }
                }
            }
            this._bulkEditCellDetails.defaultData = defaultData;
        },
        _bulkDelete: function (index) {
            if (this.model.editSettings.allowDeleting) {
                if (ej.isNullOrUndefined(index))
                    index = this._selectedRow();
                if (index == -1) {
                    alert(this._getLocalizedLabels("DeleteOperationAlert"));
                    return;
                }
                var tr = this.getRows()[index], $tr = $(tr), data = this.getDataByIndex(index), args = {};
                args = {
                    primaryKey: this._primaryKeys,
                    rowIndex: index,
                    rowData: data,
                    row: $tr
                };
                if (this._trigger("beforeBatchDelete", args))
                    return;
                if ($tr.hasClass("e-insertedrow")) {
                    $tr.remove();
                    index = $.inArray(tr, this._bulkEditCellDetails.insertedTrCollection);
                    if (index != -1) {
                        this._bulkEditCellDetails.insertedTrCollection.splice(index, 1);
                        this.batchChanges.added.splice(index, 1);
                    }
                }
                else {
                    $tr.hide();
                    this.batchChanges.deleted.push(data);
                }
                this._enableSaveCancel();
                this._selectedRow(-1);
                args = {
                    primaryKey: this._primaryKeys,
                    rowIndex: index,
                    rowData: data
                };
                this._trigger("batchDelete", args);
            }
        },
        _bulkAddRow: function (defaultData) {
            if (this.model.editSettings.allowAdding) {
                var args = {}, $tr, editCellIndex, rows = this.getRows();
                if (!ej.isNullOrUndefined(defaultData))
                    this._bulkEditCellDetails.defaultData = defaultData;
                ej.isNullOrUndefined(this._bulkEditCellDetails.defaultData) && this.setDefaultData();
                args = {
                    defaultData: $.extend(true, {}, this._bulkEditCellDetails.defaultData),
                    primaryKey: this._primaryKeys,
                };
                if (this._trigger("beforeBatchAdd", args))
                    return;
                $tr = $($.render[this._id + "_JSONTemplate"](args.defaultData)).addClass("e-insertedrow");
                rows != null && $(rows[0]).hasClass("e-alt_row") && $tr.removeClass("e-alt_row");
                if (this.model.editSettings.rowPosition == "top")
                    this.getContentTable().first().find('tbody').first().prepend($tr);
                else if (this.model.editSettings.rowPosition == "bottom")
                    this.getContentTable().first().find('tbody').first().append($tr);
                if (this._gridRecordsCount === 0)
                    this.getContentTable().find("tbody .emptyrecord").first().remove();
                this._gridRows = this.getContentTable().find("td.e-rowcell").closest("tr").toArray();
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                    $tr.prepend(ej.buildTag("td.e-indentcell"));
                this._bulkEditCellDetails.insertedTrCollection.push($tr.get(0));
                this.batchChanges.added.push(args.defaultData);
                this._enableSaveCancel();
                var rowindex = this._gridRows.length - 1;
                if (this.model.editSettings.rowPosition == "bottom") {
                    editCellIndex = this._findNextEditableCell(0);
                    this.selectRows(rowindex);
                    this.editCell(rowindex, this.model.columns[editCellIndex].field);
                }
                else {
                    editCellIndex = this._findNextEditableCell(0);
                    this.selectRows(0);
                    this.editCell(0, this.model.columns[editCellIndex].field);
                }
                $tr.find(".e-rowcell").addClass("e-updatedtd e-icon e-gupdatenotify");
                args = { defaultData: args.defaultData };
                $.extend(args, {
                    columnObject: this.model.columns[editCellIndex],
                    columnIndex: editCellIndex,
                    row: $tr,
                    primaryKey: this._primaryKeys,
                    cell: $($tr[0].cells[editCellIndex])
                });
                this._trigger("batchAdd", args);
            }
        },
        getDataByIndex: function (rowIndex) {
            var $tr = $(this.getRows()[rowIndex]), insertedRowIndex, currentRowData, index;
            if ($tr.hasClass("e-insertedrow")) {
                insertedRowIndex = $.inArray($tr[0], this._bulkEditCellDetails.insertedTrCollection);
                return this.batchChanges.added[insertedRowIndex];
            }
            else
                return this._bulkEditCellDetails._data[rowIndex - this._bulkEditCellDetails.insertedTrCollection.length];

        },
        
        refreshBatchEditChanges: function () {
            this._bulkEditCellDetails = {
                cellValue: null,
                rowIndex: -1,
                _data: null,
                columnIndex: -1,
                fieldName: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                rowData: null,
                isForeignKey: false
            };
            this.batchChanges = {
                added: [],
                deleted: [],
                changed: []
            };
        },
        refreshBatchEditMode: function () {
            if (this.model.editSettings.editMode == "batch") {
                this.refreshBatchEditChanges();
                this._bulkEditCellDetails._data = $.extend(true, [], this.getCurrentViewData());
            }
        },
        
        batchCancel: function () {
            this.cancelEdit();
        },
        
        batchSave: function () {
            var args = {}, deferedObject, gridObject = this;
            this.saveCell();
            args["batchChanges"] = this.getBatchChanges();
            if (this._trigger("beforeBatchSave", args))
                return;
            args = {};
            args.requestType = "batchsave";
            var deferedObject = this._dataManager.saveChanges(this.getBatchChanges(), this._primaryKeys[0], this.model.query._fromTable);
            if (this._dataManager instanceof ej.DataManager && !this._dataManager.dataSource.offline) {
                deferedObject.done(function (e) {
                    gridObject._processBindings(args);
                });
				deferedObject.fail(function (e) {
                    var args = { error: e.error };
                    gridObject._trigger("actionFailure", args);
                });
            }
            else
                this._processBindings(args);

        },
        
        getBatchChanges: function () {
            return this.batchChanges;
        },
        
        editCell: function (index, fieldName) {
            if (this.model.editSettings.allowEditing && $.inArray(fieldName, this._disabledEditableColumns) == -1) {
                var $form = $("#" + this._id + "EditForm");
                this.model.isEdit && this.saveCell();
                if ($.isFunction($.validator) && $form.length && $form.validate().errorList.length)
                    return;
                var $targetTR = $(this.getRows()[index]), columnIndex = this.getColumnIndexByField(fieldName), $targetTd = $targetTR.find(".e-rowcell").eq(columnIndex), column = this.model.columns[columnIndex], rowData = this.getDataByIndex(index);
                var args = {
                    validationRules: ej.isNullOrUndefined(column.validationRules) ? {} : $.extend(true, {}, column.validationRules),
                    columnName: column.field,
                    value: rowData[fieldName],
                    rowData: rowData,
                    primaryKey: this._primaryKeys,
                    columnObject: column,
                    cell: $targetTd,
                    isForeignKey: !ej.isNullOrUndefined(column.foreignKeyValue) && this.model.editSettings.editMode == "batch" ? true : false,
                }, isEditable = true;
                this._batchEditRowData = rowData;
                if (this._trigger("cellEdit", args))
                    return;
                if ($targetTR.hasClass("e-insertedrow")) args.requestType = "add";
                if ($.inArray(fieldName, this._primaryKeys) != -1 || args.columnObject.allowEditing === false || args.columnObject.template || args.columnObject.isUnbound) {
                    $.extend(this._bulkEditCellDetails, {
                        cellValue: args.value,
                        rowIndex: index,
                        fieldName: fieldName,
                        rowData: args.rowData,
                        columnIndex: columnIndex,
                        isForeignKey: ej.isNullOrUndefined(args.columnObject.foreignKeyValue) ? false : true
                    });
                    isEditable = false;
                }
                if ($targetTR.hasClass("e-insertedrow") && (!args.columnObject.template && !args.columnObject.isUnbound))
                    isEditable = true;
                if (isEditable) {
                    $.extend(this._bulkEditCellDetails, {
                        rowIndex: index,
                        cellValue: args.value,
                        columnIndex: columnIndex,
                        fieldName: fieldName,
                        cellEditType: args.columnObject.editType,
                        rowData: rowData,
                        isForeignKey: ej.isNullOrUndefined(args.columnObject.foreignKeyValue) ? false : true
                    });
                    this._renderBulkEditObject(args, $targetTd);
                    $targetTR.addClass("e-editedrow").addClass("e-batchrow");
                    args.cell.addClass("e-editedbatchcell");
                    if (args.columnObject.editType == "booleanedit")
                        args.cell.addClass("e-boolrowcell");
                }
            }
        },
        _findNextEditableCell: function (columnIndex) {
            var endIndex = this.model.columns.length;
            for (var i = columnIndex; i < endIndex; i++) {
                if (!this.model.columns[i].template && !this.model.columns[i].isUnbound)
                    return i;
            }
            return -1;
        },
        _findNextCell: function (columnIndex, direction) {
            var splittedColumn, visibleColumns = [], predicate, rows = this.getRows();
            predicate = ej.Predicate("visible", "equal", true).and("template", "notequal", true);
            if (!$(rows[this._bulkEditCellDetails.rowIndex]).hasClass("e-insertedrow"))
                predicate = predicate.and("allowEditing", "notequal", false);
            splittedColumn = direction == "right" ? this.model.columns.slice(columnIndex) : this.model.columns.slice(0, columnIndex + 1).reverse();
            visibleColumns = ej.DataManager(splittedColumn).executeLocal(ej.Query().where(predicate));
            if (visibleColumns.length == 0 && (!(direction == "left" && this._bulkEditCellDetails.rowIndex == 0) && !(direction == "right" && this._bulkEditCellDetails.rowIndex + 1 == this.getRows().length))) {
                splittedColumn = direction == "right" ? this.model.columns.slice(0, columnIndex) : this.model.columns.slice(columnIndex).reverse();
                visibleColumns = ej.DataManager(splittedColumn).executeLocal(ej.Query().where(predicate));
                this._bulkEditCellDetails.rowIndex = visibleColumns.length && direction == "right" ? this._bulkEditCellDetails.rowIndex + 1 : this._bulkEditCellDetails.rowIndex - 1;
            }
            return visibleColumns.length ? $.inArray(visibleColumns[0], this.model.columns) : -1;
        },
        _moveCurrentCell: function (direction) {
            var editCellIndex, rowIndex = this._bulkEditCellDetails.rowIndex, currentRow, $form = $("#" + this._id + "EditForm");
            if (this._bulkEditCellDetails.rowIndex == -1 && this._bulkEditCellDetails.columnIndex == -1)
                return true;
            switch (direction) {
                case "right":
                    if ((this._bulkEditCellDetails.rowIndex == this.getRows().length - 1 && this._bulkEditCellDetails.columnIndex == this.model.columns.length - 1) || (!this.element.is(document.activeElement) && $form.length == 0))
                        return true;
                    if (this._bulkEditCellDetails.columnIndex == this.model.columns.length - 1) {
                        editCellIndex = 0;
                        this._bulkEditCellDetails.rowIndex = this._bulkEditCellDetails.rowIndex + 1;
                    }
                    else
                        editCellIndex = this._bulkEditCellDetails.columnIndex + 1;
                    if (this.model.columns[editCellIndex].template === true || this.model.columns[editCellIndex].visible === false || this.model.columns[editCellIndex].allowEditing === false)
                        editCellIndex = this._findNextCell(editCellIndex, direction);
                    this._bulkEditCellDetails.rowIndex != rowIndex && this.selectRows(this._bulkEditCellDetails.rowIndex);
                    editCellIndex != -1 && this.editCell(this._bulkEditCellDetails.rowIndex, this.model.columns[editCellIndex].field);
                    break;
                case "left":
                    if ((this._bulkEditCellDetails.rowIndex == 0 && this._bulkEditCellDetails.columnIndex == 0) || (!this.element.is(document.activeElement) && $form.length == 0))
                        return true;
                    if (this._bulkEditCellDetails.columnIndex == 0) {
                        editCellIndex = this.model.columns.length - 1;
                        this._bulkEditCellDetails.rowIndex = this._bulkEditCellDetails.rowIndex - 1;
                        this.selectRows(this._bulkEditCellDetails.rowIndex);
                    }
                    else
                        editCellIndex = this._bulkEditCellDetails.columnIndex - 1;
                    if (this.model.columns[editCellIndex].template === true || this.model.columns[editCellIndex].visible === false || this.model.columns[editCellIndex].allowEditing === false)
                        editCellIndex = this._findNextCell(editCellIndex, direction);
                    this._bulkEditCellDetails.rowIndex != rowIndex && this.selectRows(this._bulkEditCellDetails.rowIndex);
                    editCellIndex != -1 && this.editCell(this._bulkEditCellDetails.rowIndex, this.model.columns[editCellIndex].field);
                    break;
                case "up":
                    if (this._bulkEditCellDetails.rowIndex == 0)
                        return;
                    editCellIndex = this._bulkEditCellDetails.columnIndex;
                    this.selectRows(this._bulkEditCellDetails.rowIndex - 1);
                    this.editCell(this._bulkEditCellDetails.rowIndex - 1, this.model.columns[this._bulkEditCellDetails.columnIndex].field);
                    break;
                case "down":
                    if (this._bulkEditCellDetails.rowIndex == this.getRows().length - 1)
                        return;
                    editCellIndex = this._bulkEditCellDetails.columnIndex;
                    this.selectRows(this._bulkEditCellDetails.rowIndex + 1);
                    this.editCell(this._bulkEditCellDetails.rowIndex + 1, this.model.columns[this._bulkEditCellDetails.columnIndex].field);
                    break;

            }
            addedRow = !$(this.getRows()[this._bulkEditCellDetails.rowIndex]).hasClass("e-insertedrow");
            if (editCellIndex != -1 && (this.model.columns[editCellIndex].isUnbound || (this.model.columns[editCellIndex].isPrimaryKey && addedRow) || this.model.columns[editCellIndex].template))
                this.element.focus();
            return false;
        },
        _renderBulkEditObject: function (cellEditArgs, $td) {
            var $form = ej.buildTag("form", "", {}, { id: this._id + "EditForm" }), $bulkEditTemplate = this._bulkEditTemplate, mappingName = this._id + cellEditArgs.columnObject.field, $element, htmlString, cellData = {};
            cellData[cellEditArgs.columnObject.field] = cellEditArgs.value; var args={requestType:cellEditArgs.requestType};
            $td.empty();
            htmlString = $bulkEditTemplate.find("#" + cellEditArgs.columnObject.field + "_BulkEdit").html();
            $element = $($.templates(htmlString).render(cellData));
            if ($element.get(0).tagName == "SELECT") {
                $element.val(cellData[cellEditArgs.columnObject.field]);
                $element.val() == null && $element.val($element.find("option").first().val());
            }
            $form.append($element);
            $td.append($form);
            this._setoffsetWidth();
            this._refreshEditForm(args);
            if ($.isFunction($.validator) && !$.isEmptyObject(cellEditArgs.validationRules)) {
                this.initValidator();
                this.setValidationToField(cellEditArgs.columnObject.field, cellEditArgs.validationRules);
            }
            this.model.isEdit = true;
        },
        _triggerConfirm: function (args) {
            if (args !== undefined && args.model.text == this._getLocalizedLabels("OkButton")) {
                if (this._confirmDialog.find(".e-content").text() == this._getLocalizedLabels("BatchSaveConfirm"))
                    this.batchSave();
                else if (this._confirmDialog.find(".e-content").text() == this._getLocalizedLabels("ConfirmDelete"))
                    this.model.editSettings.editMode == "batch" ? this._bulkDelete() : this._deleteRow();
                else if (this._confirmDialog.find(".e-content").text() == this._getLocalizedLabels("CancelEdit"))
                    this.cancelEdit();
                else {
                    this._confirmedValue = true;
                    this._processBindings(this._requestArgs);
                }
            }
            else {
                if (this._confirmDialog.find(".e-content").text() != this._getLocalizedLabels("BatchSaveConfirm") && this._confirmDialog.find(".e-content").text() != this._getLocalizedLabels("ConfirmDelete")) {
                    if (this._confirmDialog.find(".e-content").text() != this._getLocalizedLabels("CancelEdit")) {
                        switch (this._requestArgs.requestType) {
                            case "grouping":
                                this.model.groupSettings.groupedColumns.pop();
                                break;
                            case "ungrouping":
                                this.model.groupSettings.groupedColumns.push(this._requestArgs.columnName);
                                break;
                            case "sorting":
                                this._cSortedDirection = this._cSortedColumn = null;
                                break
                            case "filtering":
                                this.model.filterSettings.filteredColumns.reverse().splice(0, this._requestArgs.currentFilterObject);
                                this.model.filterSettings.filteredColumns.reverse();
                                break;
                            case "paging":
                                this._currentPage(this._requestArgs.previousPage);
                                this.getPager().ejPager("model.currentPage", this._requestArgs.previousPage);
                                break

                        }
                    }
                }
                this._confirmedValue = false;
            }
            this._requestArgs = null;
            this._confirmDialog.ejDialog("close");
        },
        _saveCellHandler: function (e) {
            var $target = $(e.target);
            e.stopPropagation();
            if ($target.closest(".e-popup").length == 0 && $target.closest(".e-rowcell").find("#" + this._id + "EditForm").length == 0)
                this.saveCell();
        },
        initValidator: function () {
            var gridObject = this, elements = this.model.scrollSettings.frozenColumns > 0 ? this.element.find(".gridform") : $("#" + this._id + "EditForm");
            for (var i = 0; i < elements.length ; i++) {
                elements.eq(i).validate({
                    ignore: ".e-hide",
                    errorClass: 'e-field-validation-error',
                    errorElement: 'div',
                    wrapper: "div",
                    errorPlacement: function (error, element) {
                        if (element.is(":hidden"))
                            element = element.siblings("input:visible");
                        var $td = element.closest("td"), $container = $(error).addClass("e-error")
                        , $tail = ej.buildTag("div.e-errortail e-toparrow"), $container;
                        $td.find(".e-error").remove();
                        if (element.parent().hasClass("e-in-wrap"))
                            $container.insertAfter(element.closest(".e-widget"));
                        else
                            $container.insertAfter(element);
                        $container.prepend($tail)
						if (gridObject.model.enableRTL)
                            gridObject.model.editSettings.editMode != "dialog" && $container.offset({ top: element.offset().top + element.height() });
                        else
							gridObject.model.editSettings.editMode != "dialog" && $container.offset({ left: element.offset().left, top: element.offset().top + element.height() });
                        $container.fadeIn("slow");
                    },

                });
            }
        },
        setValidation: function () {
            for (var i = 0; i < this.model.columns.length; i++) {
                if (!ej.isNullOrUndefined(this.model.columns[i]["validationRules"])) {
                    this.setValidationToField(this.model.columns[i].field, this.model.columns[i].validationRules);
                }
            }
        },
        
        setValidationToField: function (name, rules) {
            var fName = name, ele;
            if (!ej.isNullOrUndefined(name))
                fName = fName.replace(/[^a-z0-9\s_]/gi, '');
            var form = this.element.find(".gridform");
            ele = form.find("#" + this._id + fName).length > 0 ? form.find("#" + this._id + fName) : form.find("#" + fName);
            if (rules["regex"]) {
                rules[name + "regex"] = rules["regex"]; delete rules["regex"];
                $.validator.addMethod(fName + "regex", function (value, element, options) {
                    var ptn = options instanceof RegExp ? options : new RegExp(options);
                    return ptn.test(value);
                }, ej.getObject("messages.regex", rules) || this.getColumnByField(name).headerText + " should match the given pattern");
            }
            !ele.attr("name") && ele.attr("name", name);
            ele.rules("add", rules);
            var validator = $("#" + this._id + "EditForm").validate();
            validator.settings.messages[name] = validator.settings.messages[name] || {};
            if (!ej.isNullOrUndefined(rules["required"])) {
                if (!ej.isNullOrUndefined(rules["messages"] && rules["messages"]["required"]))
                    var message = rules["messages"]["required"];
                else
                    var message = $.validator.messages.required;
                if (message.indexOf("This field") == 0)
                    message = message.replace("This field", this.getColumnByField(name).headerText);               
                validator.settings.messages[name]["required"] = message;
                if (ele.hasClass("e-datepicker"))
                    ele.ejDatePicker({watermarkText: ""});
            }
        },
        _renderConfirmDialog: function () {
            var $contentDiv = ej.buildTag('div.e-content', this._getLocalizedLabels("BatchSaveConfirm"))
            , $buttons = ej.buildTag('span.e-buttons', "<input type='button' id=" + this._id + 'ConfirmDialogOK' + " value='" + this._getLocalizedLabels("OkButton") + "' /> "
                + "<input type='button' id=" + this._id + 'ConfirmDialogCancel' + " value='" + this._getLocalizedLabels("CancelButton") + "' />");

            this._confirmDialog = ej.buildTag('div#' + this._id + 'ConfirmDialog');
            this._confirmDialog.append($contentDiv).append($buttons);
            this.element.append(this._confirmDialog);
            $buttons.find("input").ejButton({
                cssClass: this.model.cssClass,
                showRoundedCorner: true,
                size: "mini",
                click: $.proxy(this._triggerConfirm, this)
            });
            this._renderFDialog(this._id + 'ConfirmDialog');
            this._confirmDialog.ejDialog({ width: "auto",minWidth:0,minHeight:0, enableModal: true });
        },
        _unboundClickHandler: function (e) {
            var $target = $(e.target).closest("button");
            if ($target.hasClass("e-button") && ($target.hasClass("e-disable") || $target.prop("disabled"))) return;
            var $editTrLen = 0;
            if ($(e.target).hasClass("e-unboundcelldiv"))
                return;
            if ($target.hasClass("e-cancelbutton"))
                this.model.isEdit = false;
            if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlinetemplate")
                $editTrLen = $("#" + this._id).find(".e-editedrow").length;
            this.model.allowSelection && !this.model.isEdit && this.selectRows(this.getIndexByRow($target.closest("tr")) - $editTrLen);
            $.isFunction($.fn.ejDatePicker) && $("#" + this._id + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            if ($target.hasClass("e-editbutton")) {
                var index = this.getIndexByRow($target.closest("tr"));
                if (this.model.isEdit)
                    this.cancelEdit();
                var $tr = this.getRowByIndex(index);
                this.startEdit($tr);
            } else if ($target.hasClass("e-deletebutton")) {
                var index = this.getIndexByRow($target.closest("tr"));
                var $tr = this.getRowByIndex(index);
                if (this.model.editSettings.showDeleteConfirmDialog) {
                    this._toolbarOperation(this._id + "_delete");
                    return;
                }
                this._deleteRow($tr);
            }
            else if ($target.hasClass("e-savebutton"))
                this.endEdit();
            else if ($target.hasClass("e-cancelbutton"))
                this.cancelEdit();
        },
          
        addRecord: function (data, serverChange) {
            if (this.model.editSettings.allowAdding) {
            if (data) {
                if (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                    var $addRow = ej.buildTag('tr.e-addedrow');
                    this.getContentTable().find('tbody').first().prepend($addRow);
                }
                var args = { data: data };
                args.action = "add";
                args.selectedRow = this._selectedRow();
                this._cAddedRecord = data;
                args.requestType = ej.Grid.Actions.Save;
                this._updateAction(args);
                args.selectedRow  != -1 && this.selectRows( args.selectedRow + 1)
                if (this._isUnboundColumn)
                    this._refreshUnboundTemplate(this.getContentTable());
                if (!serverChange) {
                    if ((this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") && this.model.allowPaging && this.model.pageSettings.pageSize < this.model.currentViewData.length && this.model.groupSettings.groupedColumns.length == 0 && !this.model.editSettings.showAddNewRow)
                        this.getContentTable().get(0).lastChild.removeChild(this.getContentTable().get(0).lastChild.lastChild);
                }
            } else
                    this._startAdd();
            }
        },
        
        updateRecord: function (keyField, data, action) {
            this._updateDeleteRecord(keyField, data, "update");
            if (this.model.sortSettings.sortedColumns.length || this.model.groupSettings.groupedColumns.length || !ej.isNullOrUndefined(this._searchCount) || this.filterColumnCollection.length)
                this.refreshContent();
        },
        _updateDeleteRecord: function (keyField, data, action) {
            var dataMgr = ej.DataManager(this._currentJsonData), dataFilter, index, $row, $newrow;
            dataFilter = dataMgr.executeLocal(ej.Query().where(keyField, ej.FilterOperators.equal, data[keyField]));
            if (dataFilter.length) {
                index = $.inArray(dataFilter[0], this._currentJsonData);
                if (index != -1) {
                    $row = this.getRowByIndex(index);
                    if (action == "update") {
                        $newrow = $($.render[this._id + "_JSONTemplate"](data));
                        $row.replaceWith($newrow);
                        !$row.hasClass("e-alt_row") && $newrow.removeClass("e-alt_row");
                        if (this._isUnboundColumn)
                            this._refreshUnboundTemplate(this.getContentTable());
                            this._dataManager[action](keyField, data);
                    }
                    else {
                        if ($.inArray(index, this.selectedRowsIndexes)==-1) 
                            this.selectedRowsIndexes.push(index);
                        this._deleteRow($row);
                    }
                }
            }
           
        },
        
        deleteRecord: function (keyField, data) {
            this._updateDeleteRecord(keyField, data, "remove");
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.sort = {
        _addSortElementToColumn: function (field, direction) {
            var duplicateColumn = ej.DataManager(this.model.columns).executeLocal(ej.Query().where("field", "equal", field)), imageDirection;
            if (duplicateColumn.length > 1)
                var column = this.getColumnByHeaderText(this._$curSElementTarget.text());
            else
                var column = this.getColumnByField(field);
            if (ej.isNullOrUndefined(column))
                return;
            var index = $.inArray(column, this.model.columns);
            var sortcolumn = this.getsortColumnByField(field);
            var sortindex = $.inArray(sortcolumn, this.model.sortSettings.sortedColumns);
            var $headerCellDiv = this.getHeaderTable().find("thead tr:not('.e-stackedHeaderRow')").find(".e-headercell").not(".e-detailheadercell").eq(index).find(".e-headercelldiv");
            direction = ej.isNullOrUndefined(direction) ? "ascending" : direction.toLowerCase();
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            if (this.model.allowGrouping && this.model.groupSettings.groupedColumns.length != 0)
                this.element.find(".e-groupdroparea").find("div[ej-mappingname='" + field + "']").find(".e-ascending,.e-descending").not(".e-ungroupbutton").remove();
            imageDirection = direction != "descending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
            var opacity = 1;
            if (this.model.allowSorting && this.model.allowMultiSorting && this.model.sortSettings.sortedColumns.length > 1) {
                for (var i = 1; i <= sortindex; i++) {
                    opacity = opacity + 1;
                }
                if ($headerCellDiv.css("text-align") == "right") {
                    $headerCellDiv.prepend(this._createSortNumber(opacity, $headerCellDiv).addClass("e-sortnumber"));
                    $headerCellDiv.append(this._createSortElement().addClass("e-" + (direction || "ascending") + " " + imageDirection));
                }
                else {
                    $headerCellDiv.prepend(this._createSortNumber(opacity, $headerCellDiv).addClass("e-sortnumber"));
                    $headerCellDiv.append(this._createSortElement().addClass("e-" + (direction || "ascending") + " " + imageDirection));
                }
            }
            else
                $headerCellDiv.append(this._createSortElement().addClass("e-" + (direction || "ascending") + " " + imageDirection));
            if (this.model.allowGrouping && this.model.groupSettings.groupedColumns.length != 0)
                this.element.find(".e-groupdroparea").find("div[ej-mappingname='" + field + "']").append(this._createSortElement().addClass("e-" + (direction || "ascending") + " " + imageDirection));
            $headerCellDiv.parent().attr("aria-sort", direction);
        },
        _removeSortElementFromColumn: function (field) {
            var column = this.getColumnByField(field);
            var index = $.inArray(column, this.model.columns);
            var $headerCellDiv = this.getHeaderTable().find("thead").find(".e-headercell").not(".e-detailheadercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            $headerCellDiv.parent().removeAttr("aria-sort");
        },
        _sortCompleteAction: function (args) {
            var imageDirection;
            this.getHeaderTable().find(".e-columnheader").find(".e-headercelldiv")
                    .find(".e-ascending,.e-descending,.e-number").remove();
            if (this.model.allowGrouping && this.model.groupSettings.groupedColumns.length != 0)
                this.element.find(".e-groupdroparea").find("div[ej-mappingname='" + args.columnName + "']").find(".e-ascending,.e-descending,.e-number").not(".e-ungroupbutton").remove();
            this.getHeaderTable().find("[aria-sort]").removeAttr("aria-sort");
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
            if (this.model.groupSettings.groupedColumns.length && this._$curSElementTarget != null) {
                var $element = this._checkEinGroupDrop($.trim(this._$curSElementTarget.attr("ej-mappingname")));
                if (!ej.isNullOrUndefined($element)) {
                    imageDirection = (ej.isNullOrUndefined(args.columnSortDirection) || args.columnSortDirection == "" ? this.getsortColumnByField(this._$curSElementTarget.attr("ej-mappingname")).direction.toLowerCase() : args.columnSortDirection) == "ascending" ? "e-rarrowup-2x" : "e-rarrowdown-2x"
                    $element.find(".e-ascending,.e-descending").removeClass().addClass("e-icon e-" + (ej.isNullOrUndefined(args.columnSortDirection) || args.columnSortDirection == "" ? this.getsortColumnByField(this._$curSElementTarget.attr("ej-mappingname")).direction.toLowerCase() : args.columnSortDirection) + " " + imageDirection);
                }
            }
            this.multiSortRequest = false;
            this.setWidthToColumns();
        },
        
        removeSortedColumns: function (fieldName) {
            if ($.isArray(fieldName)) {
                for (var i = 0; i < fieldName.length; i++) {
                    this._removeSortedColumnFromCollection(fieldName[i]);
                }
            }
            else
                this._removeSortedColumnFromCollection(fieldName);
            this.multiSortRequest = true;
            this.sortColumn(null, null);
        },
        _removeSortedColumnFromCollection: function (fieldName) {
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == fieldName) {
                    this.model.sortSettings.sortedColumns.splice(i, 1);
                    break;
                }
            }
        },
        
        clearSorting: function () {
            var proxy = this;
            this.model.sortSettings.sortedColumns = $.grep(this.model.sortSettings.sortedColumns, function (value, index) {
                if ($.inArray(value.field, proxy.model.groupSettings.groupedColumns) != -1)
                    return true;
                return false;
            });
            this._$prevSElementTarget = null;
            this._$curSElementTarget = null;
            this.refreshContent();
        },
        
        sortColumn: function (columnName, columnSortDirection) {
            if (!this.model.allowSorting || $.inArray(columnName, this._disabledSortableColumns) != -1 || (columnName != null && columnName.length == 0))
                return;
            var args = {};
            if (!this.multiSortRequest) {
                var proxy = this;
                this.model.sortSettings.sortedColumns = $.grep(this.model.sortSettings.sortedColumns, function (value, index) {
                    if ($.inArray(value.field, proxy.model.groupSettings.groupedColumns) != -1)
                        return true;
                    return false;
                });
            }
            args.requestType = ej.Grid.Actions.Sorting;
            this._cSortedColumn = args.columnName = columnName;
            this._cSortedDirection = args.columnSortDirection = ej.isNullOrUndefined(columnSortDirection) ? ej.sortOrder.Ascending : columnSortDirection.toLowerCase();
            if (this._cSortedColumn !== null) {
                this._removeSortedColumnFromCollection(columnName);
                this.model.sortSettings.sortedColumns.push({ field: this._cSortedColumn, direction: this._cSortedDirection });
            }
            var returnValue = this._processBindings(args);
            if (returnValue)
                this._cSortedDirection = this._cSortedColumn = null;
            this._primaryKeyValues = [];
        },
        _createSortElement: function () {
            return ej.buildTag('span.e-icon', "&nbsp;");
        },
        _renderMultiTouchDialog: function () {
            this._customPop = ej.buildTag("div.e-gridpopup", "", { display: "none" });
            var $content = ej.buildTag("div.e-content"), $downTail = ej.buildTag("div.e-downtail e-tail");
            if (this.model.allowMultiSorting) {
                var $selElement = ej.buildTag("span.e-sortdirect e-icon");
                $content.append($selElement);
            }
            if (this.model.selectionType == "multiple") {
                var $selElement = ej.buildTag("span.e-rowselect e-icon");
                $content.append($selElement);
            }
            this._customPop.append($content);
            this._customPop.append($downTail);
            this.element.append(this._customPop);
        },

    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.filter = {
        
        getFilterBar: function () {
            return this._gridFilterBar;
        },

        setGridFilterBar: function (value) {
            this._gridFilterBar = value;
        },
        
        filterColumn: function (fieldName, filterOperator, filterValue, predicate, matchcase, actualFilterValue) {
            if (!this.model.allowFiltering)
                return;
            var args = {};
            args.requestType = ej.Grid.Actions.Filtering;
            args.currentFilterObject = [];
			this._$curFieldName =  fieldName;
            if (!$.isArray(filterOperator))
                filterOperator = $.makeArray(filterOperator);
            if (!$.isArray(filterValue))
                filterValue = $.makeArray(filterValue);
            var firstLoop = false;
            var filterCol = this._filterCollection;
            if (ej.util.isNullOrUndefined(this._currentFilterColumn))
                this._currentFilterColumn = this.getColumnByField(fieldName);
            for (var index = 0; index < filterOperator.length; index++) {
                var filterObject = {
                    field: fieldName,
                    operator: filterOperator[index],
                    value: filterValue[index],
                    matchcase: matchcase,
                    predicate: predicate,
                    actualFilterValue: actualFilterValue
                };
                if (this.model.filterSettings.filterType == "filterbar")
                    this._$colType = this._currentFilterColumn.type;
                if (this.model.filterSettings.filteredColumns.length == 0 && filterObject.value !== "") {
                    if ((this._$colType == "date" || this._$colType == "datetime") && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                        this._setDateFilters(filterObject);
                    else
                        this.model.filterSettings.filteredColumns.push(filterObject);
                } else {
                    var proxy = this;
                    if (!firstLoop) {
                        var dataManger = ej.DataManager(this.model.filterSettings.filteredColumns);
                        var query = new ej.Query().where("field", ej.FilterOperators.equal, filterObject.field);
                        var object = dataManger.executeLocal(query);
                        for (var i = 0; i < object.length; i++) {
                            var objectIndex = $.inArray(object[i], this.model.filterSettings.filteredColumns)
                            if (objectIndex != -1)
                                this.model.filterSettings.filteredColumns.splice(objectIndex, 1);
                        }
                    }
                    if (filterObject.value !== "") {
                        if ((this._$colType == "date" || this._$colType == "datetime") && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                            this._setDateFilters(filterObject);
                        else
                            this.model.filterSettings.filteredColumns.push(filterObject);
                    }
                }
                firstLoop = true;
                args.currentFilterObject.push(filterObject);
            }
            args.filterCollection = this.model.filterSettings.filteredColumns;
            args.currentFilteringColumn = fieldName;
            var returnValue = this._processBindings(args);
            if (returnValue) {
                this.model.filterSettings.filteredColumns.reverse().splice(0, filterOperator.length);
                this.model.filterSettings.filteredColumns.reverse();
            }

        },
        
        search: function (searchString) {
            var args = {};
            if ($("#" + this._id + "_search").find("input").val() != searchString);
                $("#" + this._id + "_search").find("input").val(searchString);
            args.requestType = ej.Grid.Actions.Search;
            args.keyValue = searchString;
            if (this.model.locale != "en-US")
                this.model.searchSettings.key = searchString.toLowerCase() == this._getLocalizedLabels("True").toLowerCase() ? "true" : searchString.toLowerCase() == this._getLocalizedLabels("False").toLowerCase() ? "false" : searchString;
            else
                this.model.searchSettings.key = searchString;
            this._processBindings(args);
            this._primaryKeyValues = [];
        },
        _filterBarHandler: function (e) {
            var keycode = e.keyCode;
            if ((this.model.filterSettings["filterBarMode"] == "immediate" || keycode == 13) && keycode != 9) {
                var $target = $(e.target);
                this.filterStatusMsg = "";
                var fieldName = $target.prop("id").replace("_filterBarcell", "");
                var column = this.getColumnByField(fieldName);
                if (column == null)
                    return;
                this._currentFilterColumn = column;
                if (this._currentFilterColumn != this._oldFilterColumn)
                    this.filterValueOldLength = 0;
				if (this.model.locale != "en-US")
                    this._currentFilterbarValue = $target.val().toLowerCase() == this._getLocalizedLabels("True").toLowerCase() ? "true" : $target.val().toLowerCase() == this._getLocalizedLabels("False").toLowerCase() ? "false":$target.val();
                else
                    this._currentFilterbarValue = $target.val();
                this.filterValueCurrentLength = this._currentFilterbarValue.length;
                if (((this.filterValueCurrentLength == 0 && this.filterValueOldLength == 0) || this._currentFilterbarValue == this.OldfilterValue) && this._currentFilterColumn == this._oldFilterColumn) {
                    this._showFilterMsg();
                    return;
                }
                this._skipFilterProcess = this._checkForSkipInput();
                if (!this._skipFilterProcess) {
                    this._processFilter();
                } else {
                    if (this._currentFilterColumn.type == "string") {
                        this.filterStatusMsg = "Invalid Filter Data";
                        this._showFilterMsg();
                    } else {
                        this._skipFilterProcess = false;
                        this._showFilterMsg();
                        return;
                    }
                }
            }
        },
        _renderResponsiveFilter: function () {
            var $outerDiv = ej.buildTag('div#responsiveFilter.resFilterDiv', '', { 'width': '100%', 'padding': '0px' });
            var height = $(window).height() + 1;
            var headerHieght = height * (8 / 100);
            var width = this.element.height() > height ? $(window).width() + 16.5 : $(window).width();
            var $columnDiv = ej.buildTag('div.columnDiv', '', { width: '100%' });
            for (var i = 0; i < this.model.columns.length; i++) {
                var $cDiv = ej.buildTag('div.e-responsivefilterColDiv', '', { width: '100%' }, { 'ej-MappingName': this.model.columns[i].field });
                var $span = ej.buildTag('span', this.model.columns[i].headerText, { 'margin-left': '4%' });
                $cDiv.append($span);
                $columnDiv.append($cDiv);
            }
            $outerDiv.append($columnDiv);
            this.element.append($outerDiv);
            var gridObj = this;
            var widt = this.element.outerWidth();
            var $headerDiv = ej.buildTag('div.e-resFilterDialogHeaderDiv', '', { 'height': headerHieght });
            var $span = ej.buildTag('div.labelRes', '<span>Filter</span>');
            var $resIcon = ej.buildTag('div.e-resFilterleftIcon', '', { 'margin-top': '3%' });
            var $resspan = ej.buildTag('span.e-icon e-responsiveFilterClear e-resIcon', '', { 'font-size': '23px' });
            var $divIcon = ej.buildTag('div.e-resFIlterRigthIcon', '', { 'float': 'right', 'margin-top': '3%' }, { closeDialogue: 'responsiveFilter', gridEle: true });
            var $spanIcon = ej.buildTag('span.e-icon e-responisveClose e-resIcon', '', { 'font-size': '23px' }, { closeDialogue: 'responsiveFilter', gridEle: true });
            $divIcon.click(function (e) {
                $("#responsiveFilter").css('display', 'none');
                gridObj.element.css('display', 'block');
            });
            $resIcon.click(function (e) {
                $("#responsiveFilter").find('.e-responsivefilterColDiv').find('.e-filternone').click();
            });
            $headerDiv.append($resIcon.append($resspan));
            $headerDiv.append($span).append($divIcon.append($spanIcon));
            $outerDiv.prepend($headerDiv);
            $outerDiv.insertAfter(this.element);
            $(".resFilterDiv").bind('keydown', $.proxy(this._responsiveDialogueKeyUp, this))
            $outerDiv.css('display', 'none');
            $(".e-responsivefilterColDiv").bind('click', $.proxy(this._mouseClickHandler, this));
        },
        _closeDivIcon: function (sender) {
            var $div = $(sender.target);
            if (!ej.isNullOrUndefined($div.attr('closeDialogue'))) {
                var $dialog = $("#" + $div.attr('closeDialogue'));
                if (!ej.isNullOrUndefined($dialog.data('ejDialog')))
                    $dialog.ejDialog('close');
                else
                    $dialog.css('display', 'none');
            }
            if (!ej.isNullOrUndefined($div.attr('gridEle'))) {
                this.element.css('display', 'block');
            }
            if (!ej.isNullOrUndefined($div.attr('openDialogue'))) {
                if (this.model.enableResponsiveRow || $div.attr('closeDialogue').indexOf('Custom') != -1) {
                    var $dialog = $("#" + $div.attr('openDialogue'));
                    if (!ej.isNullOrUndefined($dialog.data('ejDialog')))
                        $dialog.ejDialog('open');
                    else
                        $dialog.css('display', 'block');
                }
                else
                    this.element.css('display', 'block');
            }
        },
        _setResponsiveFilterIcon: function () {
            var $div = $("#responsiveFilter").find('.columnDiv'), $proxy = this;
            $div.find('.e-filtericon').remove();
            for (var i = 0; i < this.model.filterSettings.filteredColumns.length; i++) {
                var column = this.model.filterSettings.filteredColumns[i];
                var $selcDiv = $div.find('.e-responsivefilterColDiv[ej-MappingName=' + column.field + ']');
                var $divIcon = ej.buildTag('div.e-filtericon e-icon e-resIcon e-filterset e-filternone e-filterreset', '', { float: 'right', height: '22px', width: '21px', 'font-size': '20px', 'margin-right': '3%', 'margin-top': '2%' });
                var $iconSapn = ej.buildTag('span.e-filtericon e-icon e-resIcon e-filterset e-filternone', '', {}, { 'colType': column.type });
                $selcDiv.find('.e-filternone').remove();
                $selcDiv.append($divIcon);
                $iconSapn.click(function (e) {
                    var $target = e.target;
                    $proxy._$colType = $target.attr('colType');
                    $proxy._fltrClrHandler();
                    $target.remove();
                })
            }
        },
        _renderExcelFilter: function () {
            var filterCol = this.model.filterSettings.filteredColumns.length != 0 ? this.model.filterSettings.filteredColumns[0].field : null;
            var model = {
                instance: this,
                showSortOptions: this.model.allowSorting,
                allowFormatFiltering: this.model.filterSettings.allowFormatFiltering,
                allowCaseSensitive: this.model.filterSettings.enableCaseSensitivity,
                maxFilterLimit: this.model.filterSettings.maxFilterChoices,
                filterHandler: ej.proxy(this._filterHandler, this),
                initFilterCol: filterCol,
                actionBegin: "actionBegin",
                actionComplete: "actionComplete"
            };
            this._excelFilter = new ej.excelFilter(model);
        },
        _filterHandler: function (args) {
            var arg = {}, fQMgr;
            arg.requestType = args.action == "sorting" ? args.action : "filtering";
            var temp = this.model.filterSettings.filteredColumns;
            if (args.action == "filtering") {
                fQMgr = ej.DataManager(this.model.filterSettings.filteredColumns);
                var query = new ej.Query().where("field", ej.FilterOperators.equal, args.fieldName);
                var object = fQMgr.executeLocal(query);
                for (var i = 0; i < object.length; i++) {
                    var objectIndex = $.inArray(object[i], this.model.filterSettings.filteredColumns)
                    if (objectIndex != -1)
                        this.model.filterSettings.filteredColumns.splice(objectIndex, 1);
                }
                ej.merge(this.model.filterSettings.filteredColumns, args.filterCollection);
                args.currentFilterCollection = args.filterCollection;
            }
            else if (args.action == "clearfiltering") {
                var filterObj = args.filterDetails;
                delete this._excelFilter._predicates[0][args.fieldName];
                this.filterColumn(filterObj.field, filterObj.operator, filterObj.value, filterObj.predicate);
                return;
            }
            else if (args.action == "sorting") {
                var sortObj = args.sortDetails;
                if (ej.gridFeatures.sort)
                    this.sortColumn(sortObj.field, sortObj.direction);
                this._excelFilter.closeXFDialog();
                return;
            }

            arg.currentFilteringColumn = args.fieldName;
            arg.predicated = args.ejpredicate;
            var returnValue = this._processBindings(arg);
            if (returnValue)
                this.model.filterSettings.filteredColumns = temp;
        },
        _renderFiltering: function () {
            var $headerTable = this.getHeaderTable();
            var $tr = ej.buildTag('tr.e-filterbar'), $trClone;
            if (this.model.detailsTemplate || this.model.childGrid) $tr.append(ej.buildTag('th.e-filterbarcell e-mastercell'));
            for (var column = 0; column < this.model.columns.length; column++) {
                var $input = ej.buildTag('input.e-ejinputtext e-filtertext', "", {}, { title: this.model.columns[column]["headerText"] + this._getLocalizedLabels("FilterbarTitle"), type: "search", id: this.model.columns[column]["field"] + "_filterBarcell" });
                var $th = ej.buildTag('th.e-filterbarcell'), $div = ej.buildTag('div.e-filterdiv'), $span = ej.buildTag('span.e-cancel e-icon e-hide');
                if (this.model.enablePersistence && this.model.filterSettings.filteredColumns.length > 0 && this.model.filterSettings.filterType == "filterbar") {
                    for (var fColumn = 0; fColumn < this.model.filterSettings.filteredColumns.length; fColumn++) {
                        if (this.getColumnIndexByField(this.model.filterSettings.filteredColumns[fColumn].field) == column) {
                            if (this.model.filterSettings.filteredColumns[fColumn].operator == "greaterthan")
                                $input.val(">" + this.model.filterSettings.filteredColumns[fColumn].value);
                            else if (this.model.filterSettings.filteredColumns[fColumn].operator == "lessthan")
                                $input.val("<" + this.model.filterSettings.filteredColumns[fColumn].value);
                            else if (this.model.filterSettings.filteredColumns[fColumn].operator == "notequal")
                                $input.val("!=" + this.model.filterSettings.filteredColumns[fColumn].value);
                            else
                                $input.val(this.model.filterSettings.filteredColumns[fColumn].value);
                        }
                    }
                }
                if (this.model.columns[column]["allowFiltering"] === false || this.model.columns[column]["field"] == "" || ej.isNullOrUndefined(this.model.columns[column]["field"])) {
                    $input.attr("disabled", true).addClass("e-disable");
                    this._disabledFilterableColumns.push(this.model.columns[column]["headerText"]);
                }
                this.model.columns[column]["visible"] === false && $th.addClass("e-hide");
                $div.append($input).append($span);
                $tr.append($th.append($div));
                if (column == this.model.scrollSettings.frozenColumns - 1) {
                    $trClone = $tr.clone();
                    $headerTable.find("thead").first().append($trClone);
                    $tr.empty();
                }
            }
            $headerTable.find("thead").last().append($tr);
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 10) {
                var filterBarCell = $headerTable.find("thead").find(".e-filtertext");
                for (var cell = 0; cell < filterBarCell.length; cell++)
                    ej.ieClearRemover(filterBarCell[cell]);
            }
            this.setGridFilterBar($tr);
        },
        _closeFilterDlg: function () {
            if (!ej.isNullOrUndefined($("#" + this._id + "_" + this._$colType + "Dlg").data('ejDialog')))
                $("#" + this._id + "_" + this._$colType + "Dlg").ejDialog('close');
            else
                $("#" + this._id + "_" + this._$colType + "Dlg").css('display', 'none');
            this._$fDlgIsOpen = false;
        },
        _filterBarClose: function (e) {
            var $target = $(e.target);
            if (e.type == "click" && $target.hasClass("e-cancel")) {
                var $targetText = $target.prev();
                $targetText.focus().val("");
                $targetText.trigger("keyup");
                e.stopPropagation();
            }
            if (e.type == "focusin" && $target.hasClass("e-filtertext")) {
                $target = $(e.target).next();
                this.getFilterBar().find(".e-cancel").addClass("e-hide");
                $target.removeClass("e-hide");
            }
        },
        _processFilter: function () {
            if (!this._alreadyFilterProcessed) {
                this._alreadyFilterProcessed = true;
                this._startTimer();
            } else {
                this._stopTimer();
                this._startTimer();
            }
        },
        _startTimer: function () {
            var proxy = this;
            this._timer = window.setTimeout(
                function () {
                    proxy._onTimerTick();
                },
                1500);
        },
        _stopTimer: function () {
            this._oldFilterColumn = this._currentFilterColumn;
            this.filterValueOldLength = this.filterValueCurrentLength;
            if (this._timer != null)
                window.clearTimeout(this._timer);
        },

        _onTimerTick: function () {
            this.OldfilterValue = this._currentFilterbarValue;
            this._findPredicate();
            var result = null;
            var matchcase = this._currentFilterColumn.type == "string" ? false : true;
            if (this._currentFilterColumn.foreignKeyValue && this._currentFilterColumn.dataSource && this._currentFilterbarValue != "")
                result = this._fltrForeignKeyValue(this._operator, this._currentFilterbarValue, matchcase,
                         this._currentFilterColumn.dataSource, this._currentFilterColumn.foreignKeyField,
                         this._currentFilterColumn.foreignKeyValue, this._currentFilterColumn.type);
            var collection = $.extend([], this.model.filterSettings.filteredColumns);
            for (var i = 0; i < collection.length; i++) {
                if (this.getHeaderContent().find(".e-filterbar #" + collection[i].field + "_filterBarcell").val() == "") {
                    this.model.filterSettings.filteredColumns.splice(i, 1);
                    this.filterColumnCollection.splice(i, 1);
                }
            }
			if (this._currentFilterColumn.type == "date" || this._currentFilterColumn.type == "datetime") {
                for (var j = 0; j < this.model.filterSettings.filteredColumns.length; j++) {
                    if (this.model.filterSettings.filteredColumns[j].isComplex) {
                        var preobject = this.model.filterSettings.filteredColumns[j].predicates;
                        if (this.model.filterSettings.filteredColumns.length == 1) {
                           this.model.filterSettings.filteredColumns = preobject;
                        }
                        else {
                            this.model.filterSettings.filteredColumns[j] = preobject[0];
                            this.model.filterSettings.filteredColumns.push(preobject[1]);
                        }
                    }
                }
            }
            if (!this._skipFilterProcess) {
                if (ej.isNullOrUndefined(result))
                    this.filterColumn(this._currentFilterColumn.field, this._operator, this._currentFilterbarValue, this._predicate, matchcase);
                else
                    this.filterColumn(this._currentFilterColumn.field, result.operator, result.value, result.value.length > 1 ? "or" : this._predicate, matchcase, result.filterValue);
            }
            else
                this.filterStatusMsg = "Invalid Filter Data";
            this._showFilterMsg();
            this._stopTimer();
        },

        _findPredicate: function () {
            var _value = this._currentFilterbarValue.replace(/ && /i, " and ").replace(" || ", " or ");
            var _predicateFinder = _value.split(' ');
            this._predicate = "and";
            if (_predicateFinder.length != 0) {
                if ($.isFunction(ej.Predicate[_predicateFinder[1]])) {
                    this._skipFilterProcess = false;
                    this._predicate = _predicateFinder[1];
                    var valuesArray = _value.split(" " + _predicateFinder[1] + " ");
                    var tempOperator = [];
                    var filterValues = [];
                    for (var i = 0; i < valuesArray.length; i++) {
                        this._validateFilterValue(valuesArray[i]);
                        tempOperator.push(this._operator);
                        if (this._currentFilterColumn.type == "number")
                            filterValues.push(this._currentFilterbarValue);
                        else if (this._currentFilterColumn.type == "string")
                            filterValues.push(valuesArray[i]);
                    }
                    this._currentFilterbarValue = filterValues;
                    this._operator = tempOperator;
                } else
                    this._validateFilterValue($.trim(this._currentFilterbarValue));
            } else
                this._validateFilterValue($.trim(this._currentFilterbarValue));
        },

        _validateFilterValue: function (_value) {
            switch (this._currentFilterColumn.type) {
                case "number":
                    this._operator = ej.FilterOperators.equal;
                    var stringSkipInput = new Array(">", "<", "=", "!");
                    for (var i = 0; i < _value.length; i++) {
                        if (jQuery.inArray(_value[i], stringSkipInput) != -1) {
                            break;
                        }
                    }
                    if (i != _value.length) {
                        this._getOperator(_value.substring(i));
                        if (i != 0)
                            this._currentFilterbarValue = _value.substring(0, i);
                    }
                    if (this._currentFilterbarValue != "" && _value.length >= 1)
                        this._currentFilterbarValue = parseFloat(this._currentFilterbarValue);
                    else
                        this._currentFilterbarValue = _value.length > 1 ? parseFloat(_value) : _value;
                    break;
                case "date":
                case "datetime":
                    this._operator = ej.FilterOperators.equal;
                    this._getOperator(_value);
                    var _format;
                    if (ej.isNullOrUndefined(this._currentFilterColumn.format)) {
                        if (this._currentFilterColumn.type == "date")
                            _format = Globalize.culture().calendar.patterns.d; //System Date format
                        else
                            _format = Globalize.culture().calendar.patterns.f; //System DateTime format
                    }
                    else
                        _format = this._currentFilterColumn.format.replace("{0:","").replace('}', "");
                    if (this._currentFilterbarValue != "") {
                        var filterbarValue = Globalize.parseDate(this._currentFilterbarValue, _format);
                        if (!ej.isNullOrUndefined(filterbarValue))
                            this._currentFilterbarValue = Globalize.parseDate(this._currentFilterbarValue, _format);
                        else
                            this.filterStatusMsg = "Invalid Filter Data";
                    }
                    break;
                case "string":
                    this._operator = ej.FilterOperators.startsWith;
                    break;
                case "boolean":
                    if (this._currentFilterbarValue.toLowerCase() == "true" || this._currentFilterbarValue == "1")
                        this._currentFilterbarValue = true;
                    else if (this._currentFilterbarValue.toLowerCase() == "false" || this._currentFilterbarValue == "0")
                        this._currentFilterbarValue = false;
                    this._operator = ej.FilterOperators.equal;
                    break;
                default:
                    this._operator = ej.FilterOperators.equal;
            }
        },
        _getOperator: function (_value) {
            if (_value.charAt(0) == "=") {
                this._operator = ej.FilterOperators.equal;
                this._currentFilterbarValue = _value.substring(1);
            }
            if (ej.data.operatorSymbols[_value.charAt(0)] !== undefined || ej.data.operatorSymbols[_value.slice(0, 2)] !== undefined) {
                this._operator = ej.data.operatorSymbols[_value.charAt(0)];
                this._currentFilterbarValue = _value.substring(1);
                if (this._operator === undefined) {
                    this._operator = ej.data.operatorSymbols[_value.slice(0, 2)];
                    this._currentFilterbarValue = _value.substring(2);
                }
            }
            if (this._operator == ej.FilterOperators.lessThan || this._operator == ej.FilterOperators.greaterThan) {
                if (this._currentFilterbarValue.charAt(0) == "=") {
                    this._operator = this._operator + "orequal";
                    this._currentFilterbarValue = this._currentFilterbarValue.substring(1);
                }
            }

        },

        _checkForSkipInput: function () {
            var isSkip = false;
            var skipInput = new Array("=", " ", "!");
            var context = this;
            if (this._currentFilterColumn.type == "number") {
                if (ej.data.operatorSymbols[this._currentFilterbarValue] !== undefined || $.inArray(this._currentFilterbarValue, skipInput) != -1)
                    isSkip = true;
            }
            if (this._currentFilterColumn.type == "string") {
                var stringSkipInput = new Array(">", "<", "=", "!");
                for (var i = 0; i < this._currentFilterbarValue.length; i++) {
                    if ($.inArray(this._currentFilterbarValue[i], stringSkipInput) != -1)
                        isSkip = true;
                }
            }
            return isSkip;
        },
        _showFilterMsg: function () {
            var index = $.inArray(this._currentFilterColumn, this.filterColumnCollection);
            if (this._currentFilterbarValue !== "" && index == -1)
                this.filterColumnCollection.push(this._currentFilterColumn);
            if (this._currentFilterbarValue == "" && index != -1) {
                this.filterColumnCollection.splice(index, 1);
            }
            if ((!this._skipFilterProcess || this.filterColumnCollection.length > 0) && this.filterStatusMsg != "Invalid Filter Data") {
                for (var index = 0; index < this.filterColumnCollection.length; index++) {
                    if (index > 0)
                        this.filterStatusMsg += " && ";
                    this.filterStatusMsg += this.filterColumnCollection[index].headerText + ": " + $("#" + this.filterColumnCollection[index].field + "_filterBarcell").val();
                }
            }

            if (this.model.allowPaging)
                this.getPager().ejPager("model.externalMessage", this.filterStatusMsg);
            else {
                this.$pagerStatusBarDiv.find("div").html(this.filterStatusMsg);
                if (this.filterStatusMsg.length)
                    this.$pagerStatusBarDiv.css("display", "block");
                else
                    this.$pagerStatusBarDiv.hide();
            }
            if (this.filterStatusMsg == "Invalid Filter Data") {
                index = $.inArray(this._currentFilterColumn, this.filterColumnCollection);
                this.filterColumnCollection.splice(index, 1);
            }
            this.filterStatusMsg = "";
        },
        _renderFilterDialogs: function () {
            var $strDlg, $numDlg, $boolDlg, $dateDlg, $datetimeDlg,$guidDlg;

            $.each(this.model.columns, ej.proxy(function (indx, col) {
                if (this._isExcelFilter && col.type =='datetime')
                    col.type = "date";
                if (col.type == "string" && !$strDlg) {
                    $strDlg = true;
                    !this._isExcelFilter ? this._renderStringDialog() : this._excelFilter.renderDialog("string");
                }else if (col.type == "guid" && !$guidDlg) {
                    $guidDlg = true;
                    !this._isExcelFilter ? this._renderGuidDialog() : this._excelFilter.renderDialog("guid");
				}else if (col.type == "number" && !$numDlg) {
                    $numDlg = true;
                    !this._isExcelFilter ? this._renderNumDialog() : this._excelFilter.renderDialog("number");
                } else if (col.type == "date" && !$dateDlg) {
                    $dateDlg = true;
                    !this._isExcelFilter ? this._renderDateDialog(col.field) : this._excelFilter.renderDialog("date");
                } else if (col.type == "datetime" && !$datetimeDlg) {
                    $datetimeDlg = true;
                    this._renderDateTimeDialog();
                } else if (col.type == "boolean" && !$boolDlg) {
                    $boolDlg = true;
                    !this._isExcelFilter ? this._renderBoolDialog() : this._excelFilter.renderDialog("boolean");
                }
            }, this));
        },
        _renderStringDialog: function () {
            var $id = this._id + "_stringDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer e-filterDialoge");
            $content.appendTo("body");
            if (!this.model.isResponsive || !this._mediaStatus)
                this._renderFDialog($id);
            this._renderDlgContent($content, "string");

        },
        _renderBoolDialog: function () {
            var $id = this._id + "_booleanDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer e-filterDialoge");
            $content.appendTo("body");
            if (!this.model.isResponsive || !this._mediaStatus)
                this._renderFDialog($id);
            if (!this.model.filterSettings.showPredicate)
                $("#" + $id).ejDialog({ minHeight: 90, width:"100%" });
            else
                $("#" + $id).ejDialog({ minHeight: 136, width:"100%" });
            this._renderDlgContent($content, "boolean");
        },
		 _renderGuidDialog: function () {
            var $id = this._id + "_guidDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer e-filterDialoge");
            $content.appendTo("body");
            if (!this.model.isResponsive || !this._mediaStatus)
                this._renderFDialog($id);
            this._renderDlgContent($content, "guid");

        },
        _renderNumDialog: function () {
            var $id = this._id + "_numberDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer e-filterDialoge");
            $content.appendTo("body");
            if (!this.model.isResponsive || !this._mediaStatus)
                this._renderFDialog($id);
            this._renderDlgContent($content, "number");
        },
        _renderDateDialog: function (col) {
            var $id = this._id + "_dateDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer e-filterDialoge");
            $content.appendTo("body");
            if (!this.model.isResponsive || !this._mediaStatus)
                this._renderFDialog($id);
            this._renderDlgContent($content, "date", col);

        },
        _renderDateTimeDialog: function () {
            var $id = this._id + "_datetimeDlg";
            if ($("#" + $id).length > 0) return;
            var $content = ej.buildTag("div#" + $id + ".e-dlgcontainer e-filterDialoge");
            $content.appendTo("body");
            if (!this.model.isResponsive || !this._mediaStatus)
                this._renderFDialog($id);
            this._renderDlgContent($content, "datetime");
        },
        _renderFDialog: function (id) {
            $("#" + id).ejDialog({ showOnInit: false, "enableRTL": this.model.enableRTL, "cssClass": this.model.cssClass, "showHeader": false, width: 260, enableResize: false, allowKeyboardNavigation: false, content: "#" + this._id });
        },
        _renderDlgContent: function (content, type, col) {
            content.addClass("e-grid");
            var $predicate = ej.buildTag("div.e-predicate"), $operator = ej.buildTag("div.e-operator"), $value = ej.buildTag("div.e-value");
            var $strOp = this._getLocalizedLabels("StringMenuOptions");
            var $numOp = this._getLocalizedLabels("NumberMenuOptions");
            var $drdown = ej.buildTag("input#" + this._id + type + "_ddinput", {}, {}, { "type": "text" });
            var $drdownDiv = ej.buildTag("div#" + this._id + type + "_dropdown");
            var $drdownUl = ej.buildTag("ul");
            var $radio = ej.buildTag("input", {}, {}, { "type": "radio", "name": this._id + "_predicate" + type, "value": "or" });
            var $andRadio = ej.buildTag("input", {}, {}, { "type": "radio", "name": this._id + "_predicate" + type, "value": "and", "checked": "checked" });
            var $cbox;
            $predicate.append($andRadio)
                .append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("PredicateAnd")))
                .append($radio)
                .append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("PredicateOr")));
            !this.model.filterSettings.showPredicate && $predicate.hide();
            if (type == "string") {
                $cbox = ej.buildTag("input", {}, {}, { "type": "checkbox" });
                $predicate.append($cbox)
                    .append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("MatchCase")));
                $.each($strOp, function (indx, operator) {
                    $drdownUl.append(ej.buildTag("li", {}, {}, { "value": operator.value }).html(operator.text));
                });
            }
            if (type == "number" || type == "date" || type == "datetime"|| type == "guid" ) {
                if(type=="guid")
                $numOp = $numOp.slice(4,6);
                $.each($numOp, function (indx, operator) {
                $drdownUl.append(ej.buildTag("li", {}, {}, { "value": operator.value }).html(operator.text));
                });
            }
            if (type != "boolean") {
                $drdownDiv.append($drdownUl);
                $operator.append($drdown);
                $operator.append($drdownDiv);
            }
            var $tBox = ej.buildTag("input", {}, {}, { "type": "text" });
            var $tchkBox = ej.buildTag("input", {}, {}, { "type": "checkbox" });
            var filterVal = this.model.enableResponsiveRow ? 'Ok' : 'Filter';
            var clearVal = this.model.enableResponsiveRow ? 'Cancel' : 'Clear';
            var $filter = ej.buildTag("input.e-filter", {}, {}, { "type": "button", "value":this._getLocalizedLabels( filterVal) });
            var $clear = ej.buildTag("input.e-clear", {}, {}, { "type": "button", "value": this._getLocalizedLabels(clearVal) });
            $value.append(ej.buildTag("span.e-caption").html(this._getLocalizedLabels("FilterMenuCaption") + " : "));
            content.append($predicate);
            if (type == "boolean") {
                $value.find("span.e-caption").css("top","1px");
                $value.append($tchkBox);
            }
            else {
                $value.append(ej.buildTag("br")).append($tBox);
                content.append($operator);
            }
            content.append($value);
            content.append(ej.buildTag("div.e-dlgBtns").append($filter)
                .append($clear));
            if (type != "boolean")
                $drdown.ejDropDownList({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "targetID": this._id + type + "_dropdown", width: "100%", height: "26px", selectedItemIndex: 0 });
            $radio.ejRadioButton({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL });
            $andRadio.ejRadioButton({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, checked: true });
            if ($cbox)
                $cbox.ejCheckBox({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL });
            content.css("display", "none");
            this._createButton("filter", $filter);
            this._createButton("clear", $clear);
            if (type == "number")
                $tBox.ejNumericTextbox({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "value": 0, showSpinButton: false, height: "26px", decimalPlaces: 2, width: "100%" });
            else if (type == "guid" )
                $tBox.css({  "height": "26px", "width": "100%" });
			else if (type == "date") {
                $tBox.attr("id", this._id + "_dpDate")
                $tBox.ejDatePicker({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, width: "100%", watermarkText: this._getLocalizedLabels("DatePickerWaterMark"), locale: this.model.locale });
            }
            else if (type == "datetime") {
                $tBox.attr("id", this._id + "_dpDateTime")
                $tBox.ejDateTimePicker({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, width: "100%", locale: this.model.locale });
            }
            else if (type == "boolean")
                $tchkBox.ejCheckBox({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL });
            else if (type == "string") {
                $tBox.attr("id", this._id + "_acString");
                $tBox.ejAutocomplete({
                    "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "dataSource": this._dataSource(), width: "100%", height: 26, enableDistinct: true, focusIn: function (args) {
                        var $dropdown = this.element.closest(".e-dialog").find(".e-dropdownlist");
                        var $matchcase = this.element.closest(".e-dialog").find(".e-checkbox").prop("checked");
                        this.model.filterType = $dropdown.ejDropDownList("model.value");
                        this.model.caseSensitiveSearch = $matchcase;
                    },
                    open: function (args) {
                        var doped = !this.element.closest(".e-dialog").ejDialog("isOpened")
                        this.suggestionList.css({ visibility: (doped ? "hidden" : "visible") });
                    }
                    
                });
            }
        },
        _createButton: function (name, element) {
            var $func = name == "filter" ? ej.proxy(this._fltrBtnHandler, this) : ej.proxy(this._fltrClrHandler, this);
            element.ejButton({ "cssClass": this.model.cssClass, "enableRTL": this.model.enableRTL, "click": $func });
        },
        _getIdField: function () {
            var $key;
            $.each(this.model.columns, function (indx, col) {
                if (col.key) {
                    $key = col.field;
                    return false;
                }
            });
            return $key;
        },
        _filterCompleteAction: function () {
            if (this.model.allowPaging)
                this._refreshGridPager();
            if (this.model.filterSettings.filterType == "menu" || this._isExcelFilter) {
                if (!this._isExcelFilter)
                    this._closeFilterDlg();
                else if (this._excelFilter._openedFltr) {
                    this._excelFilter.closeXFDialog();
                }
                var column = this.getColumnByField(this._$curFieldName);
                var index = $.inArray(column, this.model.columns), proxy = this, _addicon = false;
                $.each(this.model.filterSettings.filteredColumns, function (indx, col) {
                    if (col.field == proxy._$curFieldName) {
                        _addicon = true;
                        return false;
                    }
                    else if (col.isComplex) {
                        if (col.predicates[0].field == proxy._$curFieldName) {
                            _addicon = true;
                            return false;
                        }
                    }
                });
                var $fIcon = this.getHeaderTable().find("thead").find(".e-headercell").not(".e-detailheadercell,.e-stackedHeaderCell").eq(index).find(".e-filtericon");
                if (_addicon)
                    $fIcon.addClass("e-filteredicon e-filternone");
                else
                    $fIcon.removeClass("e-filteredicon e-filternone");
            }
        },
        _refreshFilterIcon: function () {
            if (!this.model.filterSettings.filteredColumns.length)
                return;
            var filteredCols = ej.distinct(this.model.filterSettings.filteredColumns, "field", true), _$headerCells = this.getHeaderTable().find("thead").find(".e-headercell").not(".e-detailheadercell"), index, col;
            if (this.model.showStackedHeader)
                _$headerCells = _$headerCells.not(".e-stackedHeaderCell");
            if (this.model.allowReordering)
                _$headerCells.find(".e-filtericon").removeClass("e-filteredicon e-filternone");
            for (var i = 0, flen = filteredCols.length; i < flen; i++) {
                col = filteredCols[i]
                index = this.getColumnIndexByField(col.isComplex ? col.predicates[0].field : col.field);
                _$headerCells.eq(index).find(".e-filtericon").addClass("e-filteredicon e-filternone");
            }
        },
        _setFilterFieldValues: function (id) {
            var $fVal = "", proxy = this;
            var flchk = -1;
            $.each(this.model.filterSettings.filteredColumns, function (indx, value) {
                if (value.field == proxy._$curFieldName)
                    flchk = indx;
            });
            if (flchk == -1 && this._$colType != "boolean") {
                if (this._$colType == "string") {
                    $(".e-predicate input[type='checkbox']").ejCheckBox({
                        checked: false
                    });
                }
                $(".e-predicate input[name =" + this._id + "_predicate" + this._$colType + "]:first").ejRadioButton({ checked: true });
                $("#" + this._id + this._$colType + "_ddinput").ejDropDownList({
                    selectedItemIndex: 0, change: function (args) {
                        this.element.closest(".e-dialog").find(".e-autocomplete").val($fVal);
                    }
                });
            }
            if (this._$curFieldName != this._$prevFieldName) {
                $.each(this.model.filterSettings.filteredColumns, function (indx, col) {
                    if (col.field == proxy._$curFieldName) {
                        var index;
                        var option = proxy._$colType == "number" || proxy._$colType == "date" || proxy._$colType == "datetime" ? "Number" : "String";
                        var $dlist = proxy._getLocalizedLabels(option + "MenuOptions");
                        for (index = 0; index < $dlist.length; index++) {
                            if ($dlist[index].value.toLowerCase() == col.operator)
                                break;
                        }
                        if (proxy._$colType == "string")
                            $(".e-predicate input[type='checkbox']").ejCheckBox({ checked: col.matchcase });
                        $("input[value=" + col.predicate + "]").ejRadioButton({ checked: true });
                        $("#" + proxy._id + proxy._$colType + "_ddinput").ejDropDownList({ selectedItemIndex: index });
                        $fVal = col.actualFilterValue != null ? col.actualFilterValue : col.value;
                        return false;
                    }
                });
                if (this._$colType == "boolean") {
                    if ($fVal && $fVal != "")
                        $(id).find(".e-value input").attr("checked", true);
                    else
                        $(id).find(".e-value input").attr("checked", false);
                } else if (this._$colType == "date" || this._$colType == "datetime") {
                    $(id).find(".e-value .e-datepicker")[this._$colType == "date" ? "ejDatePicker" : "ejDateTimePicker"]("model.value", $fVal);
                } else if(this._$colType == "number"){
					$(id).find(".e-value .e-numerictextbox").ejNumericTextbox("model.value", $fVal);
				} else
                    $(id).find(".e-value input").val($fVal);
            }
        },
        _fltrBtnHandler: function (e) {
            var id = this._id + "_" + this._$colType + "Dlg", str;
            var $par = $("#" + id);
            var $input = $par.find(".e-value input"), $operator, result;
            var value = $input.val(), matchcase = undefined, filterValue;
            if (this._$colType == "number") {
                $input = $input.filter(".e-numerictextbox");
                value = parseFloat($input.ejNumericTextbox("getValue"));
                matchcase = true;
            }
            if (this._$colType == "string")
                matchcase = $par.find(".e-predicate input[type='checkbox']").is(":checked");
            if (this._$colType == "date" || this._$colType == "datetime") {
                str = this._$colType == "date" ? "_dpDate" : "_dpDateTime";
                value = $("#" + this._id + str)[this._$colType == "date" ? "ejDatePicker" : "ejDateTimePicker"]("model.value");                
                matchcase = true;
            }
            if (this._$colType == "boolean") {
                value = $input.ejCheckBox("model.checked") != null ? $input.ejCheckBox("model.checked") : false;
                $operator = "equal";
            } else
                $operator = $("#" + this._id + this._$colType + "_ddinput").ejDropDownList("getSelectedValue").toLowerCase();
            if (this._$colForeignKeyValue && this._$colDropdownData)
                result = this._fltrForeignKeyValue($operator, value, matchcase, this._$colDropdownData, this._$colForeignKeyField, this._$colForeignKeyValue);
            if (ej.isNullOrUndefined(result))
                this.filterColumn(this._$curFieldName, $operator, value, $par.find(".e-predicate input[type='radio']:checked").attr("value"), matchcase);
            else {
                if ($par.find(".e-predicate input[type='radio']:checked").css("display") == "none" && result.value.length > 1)
                    this.filterColumn(this._$curFieldName, result.operator, result.value, "or", matchcase, result.filterValue);
                else
                    this.filterColumn(this._$curFieldName, result.operator, result.value, $par.find(".e-predicate input[type='radio']:checked").attr("value"), matchcase, result.filterValue);
            }
            if (this.model.isResponsive) {
                $par.css('display', 'none');
                this._setResponsiveFilterIcon();
                this.element.css('display', 'block');
                if (this.model.allowScrolling && (!this.model.enableResponsiveRow || !this.model.minWidth)) {
                    var args = {};
                    args.requestType = 'refresh';
                    this._refreshScroller(args);
                }
            }
        },
        _fltrClrHandler: function (e) {                        
			this.clearFiltering(this._$curFieldName);
        },

        _fltrForeignKeyValue: function (operator, value, matchcase, dataSource, fieldName, mapFieldName, colType) {
            if (ej.isNullOrUndefined(matchcase))
                matchcase = true;
            var operatorCol = [], query, filterValue;
            var data = ej.DataManager(dataSource);
            if (colType == "date") {
                var $prevDate = new Date(value.setDate(value.getDate() - 1));
                var $nextDate = new Date(value.setDate(value.getDate() + 2));
                if (operator == "equal" || operator == "notequal") {
                    if (operator == "equal")
                        query = new ej.Query().where(ej.Predicate(mapFieldName, ">", $prevDate, !matchcase)
                            .and(mapFieldName, "<", $nextDate, !matchcase)).select(fieldName);
                    else
                        query = new ej.Query().where(ej.Predicate(mapFieldName, "<=", $prevDate, !matchcase)
                           .or(mapFieldName, ">=", $nextDate, !matchcase)).select(fieldName);
                }
                else
                    query = new ej.Query().where(mapFieldName, operator, value, !matchcase).select(fieldName);
            }
            else
                query = new ej.Query().where(mapFieldName, operator, value, !matchcase).select(fieldName);
            filterValue = value;
            value = data.executeLocal(query);
            operatorCol.push("equal");
            for (i = 1; i < value.length ; i++) {
                operatorCol.push("equal");
            }
            return { operator: operatorCol, value: value, filterValue: filterValue };
        },
        _setDateFilters: function (filterObject) {
            var $prevDate = new Date(filterObject.value.setSeconds(filterObject.value.getSeconds() - 1));
            var $nextDate = new Date(filterObject.value.setSeconds(filterObject.value.getSeconds() + 2));
            var $prevObj = $.extend({}, filterObject);
            var $nextObj = $.extend({}, filterObject);
            $prevObj.value = $prevDate;
            $nextObj.value = $nextDate;
            if (filterObject.operator == "equal") {
                $prevObj.operator = "greaterthan";
                $prevObj.predicate = "and";
                $nextObj.operator = "lessthan";
                $nextObj.predicate = "and";
            } else {
                $prevObj.operator = "lessthanorequal";
                $prevObj.predicate = "or";
                $nextObj.operator = "greaterthanorequal";
                $nextObj.predicate = "or";
            }
            pred = ej.Predicate($prevObj.field, $prevObj.operator, $prevObj.value, false);
            predicate = pred[$nextObj.predicate](ej.Predicate($nextObj.field, $nextObj.operator, $nextObj.value, false));
            this.model.filterSettings.filteredColumns.push(predicate);
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.group = {
        _renderGroupDropArea: function () {
            if (!this.model.groupSettings.enableDropAreaAutoSizing)
                this.model.groupSettings.enableDropAreaAnimation = this.model.groupSettings.enableDropAreaAutoSizing;
            var dragLabel = this.model.groupSettings.enableDropAreaAnimation ? "" : this._getLocalizedLabels("GroupDropArea");
            if (this.model.groupSettings.showDropArea) {
                var $div = ej.buildTag("div.e-groupdroparea", dragLabel);
                this.model.groupSettings.enableDropAreaAutoSizing && $div.append(ej.buildTag("div.e-animatebutton e-icon").addClass(this.model.groupSettings.enableDropAreaAnimation ? "e-animatebuttondown e-gdownarrow" : "e-animatebuttonup e-guparrow"));
                return $div;
            }
        },
        _getColGroup: function (gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            if (gridObject.model.groupSettings.groupedColumns.length == 1)
                var level = this.data.items.level === undefined ? 0 : this.data.items.level - 1;
            else
                var level = this.data.items.level === undefined ? gridObject.model.groupSettings.groupedColumns.length : this.data.items.level - 1;
            var $div = $(document.createElement("div"));
            var $colGroup;
            if (gridObject._isCaptionSummary)
                $colGroup = gridObject._getCaptionColGroup(level);
            else {
                $colGroup = gridObject._getMetaColGroup();
                if (level != gridObject.model.groupSettings.groupedColumns.length && gridObject.model.groupSettings.groupedColumns.length > 1)
                    $colGroup.prepend(gridObject._getIndentCol());
            }
            $div.html($colGroup);
            return $div.html();
        },
        _colSpanAdjust: function (gridObjectId, type, captionDetails) {
            var gridObject, groupData;
            if (ej.isNullOrUndefined(gridObjectId)) {
                gridObject = this;
                groupData = captionDetails;
            }
            else {
                gridObject = this.getRsc("helpers", gridObjectId);
                groupData = this;
            }
            if (gridObject.model.groupSettings.groupedColumns.length == 1) {
                var level = groupData.data.items.level === undefined ? 1 : groupData.data.items.level - 1;
                if (type == "groupcaption")
                    gridObject._currentJsonData = gridObject._currentJsonData.concat(groupData.data.items);
            } else {
                var level = groupData.data.items.level === undefined ? gridObject.model.groupSettings.groupedColumns.length : groupData.data.items.level - 1;
                if (type == "groupcaption" && groupData.data.items.level === undefined)
                    gridObject._currentJsonData = gridObject._currentJsonData.concat(groupData.data.items);
            }
            var hideGroupColumnCount = !gridObject.model.groupSettings.showGroupedColumn ? gridObject.model.groupSettings.groupedColumns.length : 0;
            var count = 0;
            $.each(gridObject._hiddenColumnsField, function (indx, col) {
                var tempIndex = $.inArray(col, gridObject.model.groupSettings.groupedColumns);
                if (tempIndex != -1) {
                    count = count + 1;
                    if (gridObject.model.groupSettings.showGroupedColumn)
                        hideGroupColumnCount = hideGroupColumnCount + 1;
                }
            })
            var colspan = gridObject.model.columns.length + gridObject.model.groupSettings.groupedColumns.length - level - gridObject._hiddenColumnsField.length - hideGroupColumnCount + count;
            colspan = (gridObject.model.detailsTemplate != null || gridObject.model.childGrid != null) ? colspan + 1 : colspan;

            if (gridObject._isCaptionSummary && type == "groupcaption") {
                var index = [], cIndex = 0;
                var row = gridObject._captionSummary();
                var hiddenIndexCount = 0, summaryColIndexes = [];
                $.each(row[0].summaryColumns, function (cindx, col) {
                    if ($.inArray(col.displayColumn, gridObject._hiddenColumnsField) != -1)
                        cIndex++;
                    summaryColIndexes.push(gridObject.getColumnIndexByField(col.displayColumn));
                    index.push(gridObject.getColumnIndexByField(col.displayColumn) + gridObject.model.groupSettings.groupedColumns.length - level);
                });
                var sumColIndex = ej.min(summaryColIndexes)
                for (var i = 0; i < gridObject._hiddenColumnsField.length; i++) {
                    var colIndex = ej.isNullOrUndefined(gridObject.getColumnByField(gridObject._hiddenColumnsField[i])) ? gridObject.getColumnIndexByHeaderText(gridObject._hiddenColumnsField[i], ej.isNullOrUndefined(gridObject.getColumnByField(gridObject._hiddenColumnsField[i]))) : gridObject.getColumnIndexByField(gridObject._hiddenColumnsField[i]);
                    if (sumColIndex > colIndex)
                        hiddenIndexCount++;
                }
                if (index.length > 0)
                    colspan = ej.min(index);
                colspan = colspan - hiddenIndexCount;
                colspan = gridObject.model.detailsTemplate != null ? colspan + 1 : colspan;
            }
            return colspan;
        },
        _captionEncode: function (gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            var column = gridObject.getColumnByField(this.data.field);
            return column.disableHtmlEncode;
        },
        _captionFormat: function (gridObjectId) {
            var gridObject = this.getRsc("helpers", gridObjectId);
            var keyValue, captionData = this.data;
            var column = gridObject.getColumnByField(captionData.field);
            if (column.foreignKeyValue && column.dataSource)
                keyValue = gridObject._foreignKeyBinding(column, captionData.key, gridObject._id);
            else
                keyValue = captionData.key;
            captionData.key = column.format ? gridObject.formatting(column.format, keyValue, gridObject.model.locale) : keyValue;
            captionData.headerText = column.headerText;
            return $.render[gridObject._id + "_CaptionTemplate"](captionData);
        },
        _getCaptionColGroup: function (level) {
            var cloneColGroup = this.getHeaderTable().find("colgroup").clone();
            var colColl = cloneColGroup.find("col");
            var indentCol = colColl.length - this.model.columns.length;
            if (this.model.detailsTemplate != null)
                indentCol = indentCol - 1;
            cloneColGroup.find("col:lt(" + indentCol + ")").remove();
            if (level > 0 && level != this.model.groupSettings.groupedColumns.length) {
                if (this.model.groupSettings.groupedColumns.length > 2 && level != this.model.groupSettings.groupedColumns.length - 1) {
                    for (var i = 0; i < this.model.groupSettings.groupedColumns.length - level; i++) {
                        cloneColGroup.prepend(this._getIndentCol());
                    }
                }
                else
                    cloneColGroup.prepend(this._getIndentCol());
            }
            return cloneColGroup;
        },
        _groupSummaryRow: function (item, aggregates, gridObjectId, showGroup) {
            var gridObject = this.getRsc("helpers", gridObjectId), showGroup = !ej.isNullOrUndefined(showGroup);
            if (gridObject.model.showSummary) {
                if (gridObject.getFooterTable() == null)
                    gridObject._renderGridFooter();
                gridObject._createSummaryRows(gridObject.getFooterTable(), item.records == null ? item : item.records, aggregates, item, showGroup);
                if (gridObject._isCaptionSummary && !showGroup) {
                    var index = [];
                    var row = gridObject._captionSummary();
                    $.each(row[0].summaryColumns, function (cindx, col) {
                        index.push(gridObject.getColumnIndexByField(col.displayColumn));
                    });
                    if (index.length > 0)
                        colIndex = ej.min(index);
                    var colLength = gridObject.model.columns.length;
                    gridObject.getFooterTable().find("tbody td").slice(-(colLength - colIndex)).removeClass("e-summaryrow").addClass("e-groupcaptionsummary");
                }
                if (!gridObject.model.groupSettings.showGroupedColumn) {
                    var groupedcolumns = gridObject.model.groupSettings.groupedColumns;
                    var count = 0;
                    var gridfooterrow = gridObject.getFooterTable().children('tbody').find('tr');
                    for (var j = 0; j < gridObject.model.summaryRows.length; j++) {
                        for (var k = 0; k < gridObject.model.summaryRows[j].summaryColumns.length; k++) {
                            for (var i = 0; i < groupedcolumns.length; i++) {
                                if (groupedcolumns[i] == gridObject.model.summaryRows[j].summaryColumns[k].displayColumn) {
                                    count++;
                                    if (gridObject.model.summaryRows[j].summaryColumns.length == count) {
                                        $(gridfooterrow[j]).addClass("e-hide")
                                    }
                                }
                            }
                        }
                        count = 0;
                    }
                }
                return !showGroup ? gridObject.getFooterTable().find("tbody").find("tr").html() : gridObject.getFooterTable().find("tbody").html();
            }
        },
        addGroupingTemplate: function () {
            var tbody = document.createElement('tbody');
            var expandTd = "<td class='e-recordplusexpand' ej-mappingname='{{:field}}' ej-mappingvalue='{{:key}}'><div class='e-icon e-gdiagonalnext'></div></td>";
            var proxy = this;
            var helpers = {};
            helpers["_" + proxy._id + "ColSpanAdjust"] = this._colSpanAdjust;
            helpers["_" + proxy._id + "Colgroup"] = this._getColGroup;
            if (ej.isNullOrUndefined(this.model.groupSettings.captionFormat))
                $.templates(proxy._id + "_CaptionTemplate", this._getLocalizedLabels("GroupCaptionFormat"));
            else
                $.templates(proxy._id + "_CaptionTemplate", this.model.groupSettings.captionFormat);
            helpers["_" + proxy._id + "CaptionFormat"] = this._captionFormat;
            helpers["_" + proxy._id + "GroupSummaryRow"] = this._groupSummaryRow;
            helpers["_" + proxy._id + "CaptionEncode"] = this._captionEncode;
            helpers[proxy._id + "Object"] = this;
            $.views.helpers(helpers);
            var caption = " ~_" + proxy._id + "CaptionFormat('" + proxy._id + "Object')";
            var cpationTd = expandTd + "<td class='e-groupcaption' colspan='{{:~_" + proxy._id + "ColSpanAdjust('" + proxy._id + "Object" + "','groupcaption') }}'>{{if ~_" + proxy._id + "CaptionEncode('" + proxy._id + "Object')}}{{html:" + caption + "}}{{else}}{{:" + caption + "}}{{/if}}</td>";
            if (this._isCaptionSummary && this.model.showSummary)
                cpationTd = cpationTd + "{{:~_" + proxy._id + "GroupSummaryRow(items, aggregates,'" + proxy._id + "Object')}}";
            var captionTr = "<tr>" + cpationTd + "</tr>";
            var $tbody = ej.buildTag("tbody");
            $tbody.html("{{if items.GROUPGUID}}" +
                "{{for items tmpl='" + proxy._id + "_GroupingTemplate'/}}" +
                "{{else}}" +
                "{{for items tmpl='" + proxy._id + "_JSONTemplate'/}}" +
                "{{/if}}");
            var indentTd = "<td class='e-indentcell'></td>";
            var table = "<table class='e-table {{if items.GROUPGUID}}{{else}}e-recordtable{{/if}}' cellspacing='0.25px' >" +
                "{{:~_" + proxy._id + "Colgroup('" + proxy._id + "Object')}}" +
                $tbody.html() + "{{:~_" + proxy._id + "GroupSummaryRow(items, aggregates,'" + proxy._id + "Object', '" + proxy._id + "showGroupCaption')}}" +
            "</table>";
            var tableTd = "<td class='e-tabletd' colspan='{{:~_" + proxy._id + "ColSpanAdjust('" + proxy._id + "Object" + "','table')}}'>" + table + "</td>";
            var tr = "<tr>" + indentTd + tableTd + "</tr>";
            $.templates(proxy._id + "_GroupingTemplate", captionTr + tr);
        },
        _getGroupTopLeftCell: function () {
            var $th = ej.buildTag("th.e-grouptopleftcell");
            $th.append(ej.buildTag("div.e-headercelldiv e-emptyCell", "&nbsp;"));
            return $th;
        },
        _getEmptyFilterBarCell: function () {
            return ej.buildTag("th.e-filterbarcell");
        },
        _groupingAction: function (refWidth) {
            var $groupTopCell = this.getHeaderTable().find("thead").find(".e-columnheader:not(.e-stackedHeaderRow)").find(".e-grouptopleftcell"), $col = this.getHeaderTable().find("colgroup").find("col");
            var groupColumn = $groupTopCell.length;
            if (groupColumn) {
                this.getHeaderTable().find("colgroup").replaceWith(this._getMetaColGroup());
                (this.model.detailsTemplate != null || this.model.childGrid != null) && this.getHeaderTable().find("colgroup").prepend(this._getIndentCol());
                $groupTopCell.remove();
                this.getHeaderTable().find("thead").find(".e-filterbar").find(".e-filterbarcell:lt(" + groupColumn + ")").remove();
            }
            if (!this.model.allowResizeToFit || refWidth)
                this.setWidthToColumns();
            for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                this.getHeaderTable().find("colgroup").prepend(this._getIndentCol());
                this.getHeaderTable().find("thead").find(".e-columnheader").prepend(this._getGroupTopLeftCell());
                this.getHeaderTable().find("thead").find(".e-filterbar").prepend(this._getEmptyFilterBarCell());
            }
            this.getHeaderTable().find("thead").find("th.e-grouptopleftcell").last().addClass("e-lastgrouptopleftcell");
        },
        
        groupColumn: function (columnName) {
            if (!this.model.allowGrouping || $.inArray(columnName, this._disabledGroupableColumns) != -1)
                return;
            if (ej.isNullOrUndefined(this.getColumnByField(columnName)) || $.inArray(columnName, this.model.groupSettings.groupedColumns) != -1)
                return;
            this.model.groupSettings.groupedColumns.push(columnName);
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == columnName) {
                    break;
                }
            }
            this.model.sortSettings.sortedColumns.length == i && this.model.sortSettings.sortedColumns.push({ field: columnName, direction: ej.sortOrder.Ascending });
            var args = {};
            args.columnName = columnName;
            args.requestType = ej.Grid.Actions.Grouping;
            var returnValue = this._processBindings(args);
            if (returnValue) {
                if (!($.inArray(columnName, this._scolumns) != -1 || this._gridSort == columnName))
                    this.model.sortSettings.sortedColumns.pop();
                this.model.groupSettings.groupedColumns.pop();
            }
            this._primaryKeyValues = [];
        },
        
        ungroupColumn: function (columnName) {
            if (!this.model.allowGrouping)
                return;
            if ($.inArray(columnName, this.model.groupSettings.groupedColumns) != -1)
                this.model.groupSettings.groupedColumns.splice($.inArray(columnName, this.model.groupSettings.groupedColumns), 1);
            else
                return null;
            var column = this.getColumnByField(columnName)
            if (!this.model.groupSettings.showGroupedColumn && !column["visible"]) {
                var index = this._hiddenColumnsField.indexOf(columnName);
                this._hiddenColumnsField.splice(index, 1);
                column["visible"] = true;
            }
            var args = {};
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                if (this.model.sortSettings.sortedColumns[i].field == columnName) {
                    if (this._scolumns.indexOf(columnName) != -1 && this.model.allowSorting && this.model.allowMultiSorting || this._gridSort == columnName)
                        if (this.model.allowSorting && this.model.allowMultiSorting) {
                            var no = $.inArray(columnName, this._scolumns);
                            this._scolumns.splice(no, 1);
                            break;
                        }
                        else {
                            this._gridSort = null;
                            break;
                        }
                    else
                        this.model.sortSettings.sortedColumns.splice(i, 1);
                    break;
                }
            }
            args.columnName = columnName;
            args.requestType = ej.Grid.Actions.Ungrouping;
            var returnValue = this._processBindings(args);
            if (returnValue)
                this.model.groupSettings.groupedColumns.push(columnName);
            this._primaryKeyValues = [];
        },
        
        collapseGroupDropArea: function () {
            var $groupDropArea = this.element.find(".e-groupdroparea").first(), proxy = this;
            this.model.groupSettings.groupedColumns.length == 0 && this.model.groupSettings.enableDropAreaAnimation && $groupDropArea.animate({ height: "10px" }, 200, function () {
                if (proxy.model != null && proxy.model.groupSettings.groupedColumns.length == 0) {
                    $(this).html("").append(ej.buildTag("div.e-animatebutton e-animatebuttondown e-icon e-gdownarrow"));
                    $(this).dequeue().css("height", "auto");
                }
            });
        },
        
        expandGroupDropArea: function () {
            var $groupDropArea = this.element.find(".e-groupdroparea").first(), proxy = this;
            this.model.groupSettings.groupedColumns.length == 0 && proxy.model.groupSettings.enableDropAreaAnimation && $groupDropArea.animate({ height: "30px" }, 150, function () {
                proxy.model.groupSettings.groupedColumns.length == 0 && $groupDropArea.html(proxy._getLocalizedLabels("GroupDropArea")).append(ej.buildTag("div.e-animatebutton e-animatebuttonup e-icon e-guparrow"));
                $groupDropArea.dequeue().css("height", "30px");
            });
        },
        _enableGroupingEvents: function () {
            if (this.model.allowGrouping) {
                this._on(this.element, "click", ".e-groupdroparea,.e-groupheadercell", this._groupHeaderCellClick);
                
                this._on(this.element, "mousedown", ".e-groupheadercell", function (e) {
                    return false;
                });
            }

        },
        _recalculateIndentWidth: function () {
            var proxy = this;
            var browserDetails = this.getBrowserDetails();
            var indentWidth = this.getHeaderTable().find(".e-lastgrouptopleftcell").width(), newWidth = indentWidth, perPixel = indentWidth / 30, $col;
            if (perPixel >= 1)
                newWidth = (30 / perPixel);
            this.getHeaderTable().find("colgroup").find("col").slice(0, this.model.groupSettings.groupedColumns.length).css("width", newWidth + "px");
            var $conCol = this.getContentTable().find("table").filter(":not(.e-recordtable)");
            indentWidth = this.getHeaderTable().find(".e-lastgrouptopleftcell").width();
            if (indentWidth > 30 || (this._isCaptionSummary && (indentWidth >= 30 || (indentWidth > newWidth)))) {
                if (this._isCaptionSummary) {
                    var colgroup = this.model.isEdit ? $conCol.parent(":not(.gridform)").children("colgroup") : $conCol.children("colgroup");
                    $.each(colgroup, function (index, item) {
                        var indentCol = $(item).find("col").length - proxy.model.columns.length;
                        if (proxy.model.detailsTemplate != null) {
                            if (indentCol > 0)
                                indentCol = indentCol - 1;
                        }
                        $(item).find("col").slice(0, indentCol).css("width", newWidth + "px");
                    });
                }
                else
                    this.model.isEdit ? $conCol.parent(":not(.gridform)").children("colgroup").find("col:first-child").css("width", indentWidth + "px") : $conCol.children("colgroup").find("col:first-child").css("width", indentWidth + "px");
                $col = this.getContentTable().find("colgroup").first().find("col").slice(0, this.model.groupSettings.groupedColumns.length);
                if (browserDetails.browser != "msie")
                    $col.css("width", newWidth + "px");
                else{
                    if (this._isCaptionSummary) 
						$col.css("width", newWidth + "px");
					else
						$col.first().css("width", ((indentWidth / this.element.width()) * 100) + "%");
				}
            } else {
                this.getContentTable().find("colgroup").first().find("col").slice(0, this.model.groupSettings.groupedColumns.length).css("width", newWidth + "px");
                this.getContentTable().find("table").filter(":not(.e-recordtable)").children("colgroup").find("col:first-child").css("width", indentWidth + "px");
            }
            if (this.model.showSummary) {
                var sumCols = this.getContentTable().find("table").filter(".e-groupsummary").find(".e-summary");
                sumCols.css("width", newWidth + "px");
            }
        },
        
        getFieldNameByHeaderText: function (headerText) {
            if (ej.isNullOrUndefined(this._fieldColumnNames[headerText]))
                return null;
            return this._fieldColumnNames[headerText];
        },
        
        getHeaderTextByFieldName: function (field) {
            if (ej.isNullOrUndefined(this._headerColumnNames[field]))
                return null;
            return this._headerColumnNames[field];
        },
        
        expandAll: function () {
            var recordPlus = this.element.find(".e-recordpluscollapse");
            var detailRow = this._excludeDetailRows().find(".e-detailrowcollapse");
            if (recordPlus.length != 0) {
                for (var i = 0; i < recordPlus.length; i++)
                    this.expandCollapse($(recordPlus[i]));
            }
            if (detailRow.length != 0) {
                for (var i = 0; i < detailRow.length; i++)
                    this.expandCollapse($(detailRow[i]));
            }
        },
        
        collapseAll: function () {
            var recordPlus = this.element.find(".e-recordplusexpand");
            var detailRow = this.element.find(".e-detailrowexpand");
            if (recordPlus.length != 0) {
                for (var i = 0; i < recordPlus.length; i++)
                    this.expandCollapse($(recordPlus[i]));
            }
            if (detailRow.length != 0) {
                for (var i = 0; i < detailRow.length; i++)
                    this.expandCollapse($(detailRow[i]));
            }
        },
        _group: function (args) {
            if (this.model.groupSettings.groupedColumns.length && this.model.currentViewData) {
                this._currentJsonData = [];
                var temp = document.createElement('div');
                if (!this.model.groupSettings.showGroupedColumn) {
                    if (!this.initialRender && !ej.isNullOrUndefined(args.columnName) && args.requestType == "grouping") {
                        var col = this.getColumnByField(args.columnName);
                        if ($.inArray(args.columnName, this._hiddenColumnsField) == -1) {
                            this._hiddenColumnsField.push(args.columnName)
                            col.visible = false;
                        }
                    }
                    else {
                        for (i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                            if ($.inArray(this.model.groupSettings.groupedColumns[i], this._hiddenColumnsField) == -1) {
                                this._hiddenColumnsField.push(this.model.groupSettings.groupedColumns[i])
                                this.getColumnByField(this.model.groupSettings.groupedColumns[i]).visible = false;
                            }
                        }
                    }
                    this._hideHeaderColumn(this.model.groupSettings.groupedColumns, true);
                    this.getContentTable().children("colgroup").replaceWith(this.getHeaderTable().find('colgroup').clone());
                }
                if (args.requestType == "reorder")
                    this._isReorder = true;
                else
                    this._isReorder = false;
                var $col = this.getContentTable().children("colgroup").find('col');
                var length = $col.length - this.model.columns.length;
                if (this.model.detailsTemplate != null || this.model.childGrid != null)
                    length--;
                if ($col.length > this.model.columns.length)
                    this.getContentTable().children("colgroup").find('col:lt(' + length + ')').remove();
                this.getContentTable().find("colgroup").first().replaceWith(this._getMetaColGroup());
                var dlen;
                if (this.model.detailsTemplate != null || this.model.childGrid != null) {
                    dlen = this.model.groupSettings.groupedColumns.length + 1;
                }
                else
                    dlen = this.model.groupSettings.groupedColumns.length;
                for (var i = 0; i < dlen; i++)
                    this.getContentTable().children("colgroup").prepend(this._getIndentCol());
                if (this.model.currentViewData.length) {
                    var $tbody = this.getContentTable().children('tbody');
                    $tbody.empty();
                    temp.innerHTML = ['<table>', $.render[this._id + "_GroupingTemplate"](this.model.currentViewData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
                    this.getContentTable().get(0).replaceChild(temp.firstChild.firstChild, this.getContentTable().get(0).lastChild);
                    this._hideCaptionSummaryColumn();
                }
                this._groupingAction();
                this._gridRows = this.getContentTable().find(".e-recordtable").find("tbody").find("tr").not(".e-gridSummaryRows");
                this._eventBindings();
            }
        },
        _ungroup: function () {
            if (!ej.isNullOrUndefined(this.model.detailsTemplate))
                this._detailsOuterWidth = null;
            var $header = this.element.children(".e-gridheader");
            var $filterInput = $header.find(".e-filterbar").find("th").find("input");
            $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
            this.setGridHeaderContent($header);
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar") {
                this._renderFiltering();
                var filterThNew = this.element.children(".e-gridheader").find(".e-filterbar").find("th").find("input");
                for (var i = 0; i < $filterInput.length; i++)
                    filterThNew.eq(i).val($filterInput.eq(i).val());
            }
            if (!this.model.groupSettings.showGroupedColumn)
                this._hideHeaderColumn(this.model.groupSettings.groupedColumns, true);
            this.refreshTemplate();
            this.element.find(".e-gridcontent").children("div").first().empty().append(this._renderGridContent().find("table").first());
            this.setGridContent(this.element.find(".e-gridcontent"));
            if (this.model.groupSettings.groupedColumns.length != 0)
                this._gridRows = this.getContentTable().find(".e-recordtable").find("tbody").find("tr").toArray();
            else
                this._gridRows = this.getContentTable().get(0).rows;
            this._eventBindings();
        },
        _groupHeaderCellClick: function (e) {
            var $target = $(e.target);
            if ($target.hasClass('e-groupdroparea'))
                return;
            if ($target.hasClass("e-ungroupbutton")) {
                var field = $target.parent().attr("ej-mappingname");
                this.ungroupColumn(field);
            } else if ($target.hasClass("e-togglegroupbutton")) {
                var field = $target.parent().attr("ej-mappingname");
                $target.hasClass("e-toggleungroup") && this.ungroupColumn(field);
            } else if ($target.hasClass("e-animatebutton")) {
                if (!this.model.groupSettings.enableDropAreaAnimation) {
                    this.model.groupSettings.enableDropAreaAnimation = true;
                    this.collapseGroupDropArea();
                } else {
                    this.expandGroupDropArea();
                    this.model.groupSettings.enableDropAreaAnimation = false;
                }
            }
            else {
                $target.addClass("e-headercelldiv");
                this._mouseClickHandler(e);
                $target.removeClass("e-headercelldiv");
            }

            return false;
        },
        _captionSummary: function (nocaption) {
            var summary = null, cols = this.model.summaryRows, k, len = cols.length;
            for (k = 0; k < len; k++) {
                if (cols[k].showCaptionSummary == true) {
                    summary = $(cols[k]);
                    break;
                }
            }
            
            if (nocaption) {
                left = cols.slice(0, k), right = cols.slice(k + 1, len);
                ej.merge(summary = left, right);
            }

            return summary;
        },
        _dropAreaHover: function (e) {
            var $target = $(e.target), proxy = this;
            if (e.type == "mouseenter") {
                if (this._dragActive) {
                    if ($target.hasClass("e-groupdroparea"))
                        $target.addClass("e-hover");
                } else
                    $target.removeClass("e-hover");
                $target.hasClass("e-groupheadercell") && this.model.groupSettings.showUngroupButton && $target.find(".e-ungroupbutton").show(150);
            } else if (e.type == "mouseleave") {
                if ($target.hasClass("e-groupdroparea")) {
                    $target.find(".e-ungroupbutton").hide(150);
                    $target.removeClass("e-hover");
                }
                $target.hasClass("e-groupheadercell") && this.model.groupSettings.showUngroupButton && $target.find(".e-ungroupbutton").hide(150);
            }
            return false;
        },
        _groupingCompleteAction: function (args) {
            var $groupDrop = this.element.children(".e-groupdroparea");
            if (this.model.groupSettings.groupedColumns.length && $groupDrop.find(".e-grid-icon").length == 0 || ej.Grid.Actions.Refresh == args.requestType)
                $groupDrop.empty();
            if (this.initialRender || ej.Grid.Actions.Refresh == args.requestType) {
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++)
                    this._addColumnToGroupDrop(this.model.groupSettings.groupedColumns[i]);
                this._refreshGridPager();
            } else
                this._addColumnToGroupDrop(args.columnName);
            this.getHeaderTable().find(".e-columnheader").find(".e-headercelldiv").find(".e-ascending,.e-descending,.e-number").remove();
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
            if (!this.initialRender && ej.gridFeatures.dragAndDrop)
                this._groupHeaderCelldrag();
            this.model.allowScrolling && this.getContentTable().parent().scrollLeft(this.getHeaderTable().parent().scrollLeft() - 1);
            this.element.children(".e-cloneproperties").remove();
            if (ej.gridFeatures.filter && ["menu", "excel"].indexOf(this.model.filterSettings.filterType))
                this._refreshFilterIcon();
        },
        _ungroupingCompleteAction: function (args) {
            var $groupDrop = this.element.children(".e-groupdroparea");
            if (args.requestType != ej.Grid.Actions.Refresh)
               this._removeColumnFromDrop(args.columnName);
            this.getHeaderTable().find(".e-columnheader").find(".e-headercelldiv").find(".e-ascending,.e-descending,.e-number").remove();
            for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++)
                this._addSortElementToColumn(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);
            if (this.model.groupSettings.groupedColumns.length == 0) {
                $groupDrop.html(this.model.groupSettings.enableDropAreaAnimation ? "" : this._getLocalizedLabels("GroupDropArea"));
                this.model.groupSettings.enableDropAreaAutoSizing && $groupDrop.append(ej.buildTag("div.e-animatebutton e-icon").addClass(this.model.groupSettings.enableDropAreaAnimation ? "e-animatebuttondown e-gdownarrow" : "e-animatebuttonup e-guparrow"));
                $groupDrop.css("height", "auto");
            }
            if (ej.gridFeatures.dragAndDrop)
                this._headerCellgDragDrop();
            if (this.model.allowReordering && ej.gridFeatures.dragAndDrop)
                this._headerCellreorderDragDrop();
            this.model.allowScrolling && this.getContentTable().parent().scrollLeft(this.getHeaderTable().parent().scrollLeft() + 10);
            this.element.children(".e-cloneproperties").remove();
            if (ej.gridFeatures.filter && ["menu", "excel"].indexOf(this.model.filterSettings.filterType))
                this._refreshFilterIcon();
        },
        _getToggleButton: function () {
            return ej.buildTag("span.e-togglegroupbutton e-icon e-groupbutton", "&nbsp;");
        },
        _checkEinHeader: function (field) {
            var $headerCell = this.element.children(".e-gridheader").find("thead").find(".e-columnheader").find(".e-headercell");
            for (var i = 0; i < $headerCell.length; i++) {
                if ($.trim($headerCell.eq(i).find("div").attr("ej-mappingname")) == field)
                    return $headerCell.eq(i);
                else if (this.model.allowSorting && this.model.allowMultiSorting) {
                    var header = $($headerCell.eq(i)).clone();
                    header.find(".e-number").remove();
                    if ($.trim(header.find("div").attr("ej-mappingname")) == field)
                        return header;
                }
            }
            return null;

        },

        _checkEinGroupDrop: function (field) {
            var $groupHeaderCell = this.element.children(".e-groupdroparea").find(".e-grid-icon");
            for (var i = 0; i < $groupHeaderCell.length; i++) {
                if ($.trim($groupHeaderCell.eq(i).find("div").attr("ej-mappingname")) == field)
                    return $groupHeaderCell.eq(i);
            }
            return null;
        },

        _addColumnToGroupDrop: function (field) {
            var $groupedColumn = ej.buildTag("div.e-grid-icon e-groupheadercell"), $groupDropArea = this.element.find(".e-groupdroparea").first();
            var $childDiv = ej.buildTag("div", {}, {}, { "ej-mappingname": field }), imageDirection = "e-rarrowup-2x";
            var column = this.getColumnByField(field)
            if (column.disableHtmlEncode)
                $groupedColumn.append($childDiv.text(column.headerText));
            else if (column.headerTemplateID)
                $groupedColumn.append($childDiv.html($(column.headerTemplateID).html()))
            else
                $groupedColumn.append($childDiv.html(column.headerText));
            var $headerCell = this._checkEinHeader(field);
            if (this.model.groupSettings.showToggleButton) {
                $childDiv.append(this._getToggleButton().addClass("e-toggleungroup"));
                $headerCell.find(".e-headercelldiv").find(".e-togglegroupbutton").remove().end().append(this._getToggleButton().addClass("e-toggleungroup"));
            }
            var direction = "ascending";
            if ($headerCell.find(".e-ascending,.e-descending").length) {
                direction = $headerCell.find(".e-ascending,.e-descending").hasClass("e-ascending") ? "ascending" : "descending";
                imageDirection = direction == "ascending" ? "e-rarrowup-2x" : "e-rarrowdown-2x";
            }
            $childDiv.append(this._createSortElement().addClass("e-" + direction + " " + imageDirection));
            this.model.groupSettings.showUngroupButton && $childDiv.append(ej.buildTag("span.e-ungroupbutton e-icon e-cancel", " ", {}, { title: this._getLocalizedLabels("UnGroup") }));
            $groupDropArea.append($groupedColumn).css("height", "auto");
            var left = $groupedColumn.offset().left, $cloned = $groupedColumn.clone().css("position", "absolute"), proxy = this;
            $groupedColumn.css("visibility", "hidden")
            $groupDropArea.append($cloned).dequeue();
            $cloned.css({ "left": left + 150 }).animate({ left: left }, 150, function (e) {
                $groupedColumn.css("visibility", "visible");
                $cloned.remove();
            });
        },
        _removeColumnFromDrop: function (field) {
            var headerText = this.getHeaderTextByFieldName(field), proxy = this, $groupDropArea = this.element.children(".e-groupdroparea");
            var $groupHeaderCell = $groupDropArea.css("height", "30px").find(".e-grid-icon");
            for (var i = 0; i < $groupHeaderCell.length; i++) {
                if ($.trim($groupHeaderCell.eq(i).find("div").attr("ej-mappingname")) == field) {
                    if (this.model.groupSettings.groupedColumns.length == 0) {
                        this.collapseGroupDropArea();
                    } else
                        $groupHeaderCell.eq(i).remove();
                }
            }
        },
        _setAggreatedCollection: function (clonedQuery) {
            if (this._dataSource() instanceof ej.DataManager && this._dataManager.dataSource.url != undefined && !this._isRemoteSaveAdaptor)
                return;
            var data;
            data = this._dataManager.executeLocal(clonedQuery).result;
            this._aggregatedCollection = data;
        },
        _setAggregates: function (data, collection) {
            var indx, pred, query = new ej.Query();
            data = data || this.model.currentViewData, collection = collection || this._aggregatedCollection;
            var dLen = data.length, cLen;
            if (dLen != 0)
                pred = ej.Predicate("field", "equal", data[0].field)["and"]("key", "equal", data[0].key);
            for (indx = 1; indx < dLen; indx++) {
                pred = pred["or"](ej.Predicate("field", "equal", data[indx].field)["and"]("key", "equal", data[indx].key));
            }
            collection = pred ? new ej.DataManager(collection).executeLocal(query.where(pred)) : collection;
            cLen = collection.length;
            if (data.length > 0) {
                for (indx = 0; indx < cLen; indx++) {
                    if (indx > 0 && indx < cLen - 1) continue;
                    data[indx].count = collection[indx].count; 
                    if (data[indx].items.GROUPGUID)
                        this._setAggregates(data[indx].items, collection[indx].items);
                    if (this.model.showSummary) { 
                        var agg = data[indx]["aggregates"] = [];
                        var rows = this.model.summaryRows, scolumns, summaryData;
                        for (var row = 0, rlen = rows.length; row < rlen; row++) {
                            scolumns = rows[row].summaryColumns;
                            for (var col = 0, clen = scolumns.length; col < clen; col++) {
                                summaryData = collection[indx].items.level ? collection[indx].items.records : collection[indx].items;
                                agg[scolumns[col].dataMember + " - " + scolumns[col].summaryType] = this.getSummaryValues(scolumns[col], summaryData);
                            }
                        }
                    }
                }
            }
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.dragAndDrop = {
        _headerCellgDragDrop: function () {
            var proxy = this;
            this.dragHeaderElement();
            var $droppableElements = this.element.children("div.e-groupdroparea");
            $droppableElements.ejDroppable({
                accept: $droppableElements,
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible"))
                        return;
                    var column = proxy.getColumnByField($.trim($(ui.draggable.context).find("div").attr("ej-mappingname")));
                    ui.helper.remove();
                    if ($.inArray(column["field"], proxy._disabledGroupableColumns) != -1)
                        return;
                    if (!(ej.isNullOrUndefined(column.field) || column.field == ""))
                        proxy.groupColumn(column.field);
                }
            });
        },
        _headerCellreorderDragDrop: function () {
            var proxy = this;
            this.dragHeaderElement();
            var $droppableElements = this.element.find(".e-headercell:not(.e-stackedHeaderCell)");
            $droppableElements.ejDroppable({
                accept: $droppableElements,
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible"))
                        return;
                    if (ui.draggable.attr("aria-sort") == "ascending" || ui.draggable.attr("aria-sort") == "descending") {
                        var scolumn = proxy.getColumnByField($.trim($(ui.draggable.context).find("div").attr("ej-mappingname")));
                        if (proxy.model.allowSorting && proxy.model.allowMultiSorting)
                            proxy._scolumns.push(scolumn.field);
                        else
                            proxy._gridSort = scolumn.field;
                    }
                    var column = proxy.getColumnByField($.trim($(ui.draggable.context).find("div").attr("ej-mappingname")));
                    ui.helper.remove();
                    //if ($.inArray(column["headerText"], proxy._disabledGroupableColumns) != -1)
                    //  return;
                    var header = $(event.dropTarget).clone();
                    header.find(".e-number").remove();
                    if (!ej.isNullOrUndefined(column.field)) {
                        if ($(event.dropTarget).hasClass("e-droppable")) {
                            header = header.children(".e-headercelldiv");
                            var eDropTarget = $(event.dropTarget).children(".e-headercelldiv");
                        }
                        else {
                            header = $(event.dropTarget).parent();
                            var eDropTarget = $(event.dropTarget);
                            if ($(eDropTarget).hasClass("e-filtericon"))
                                eDropTarget = header = $(eDropTarget).siblings(".e-headercelldiv");
                        }
                        if (proxy.model.allowSorting && proxy.model.allowMultiSorting)
                            if (event.dropTarget.hasClass("e-number") || event.dropTarget.hasClass("e-icon")) {
                                var toColumn = proxy.getColumnByField($.trim(header.attr("ej-mappingname")));
                            }
                            else {
                                var toColumn = proxy.getColumnByField($.trim(eDropTarget.attr("ej-mappingname")));
                            }
                        else {
                            if (event.dropTarget.hasClass("e-icon") && !event.dropTarget.hasClass("e-filtericon"))
                                var toColumn = proxy.getColumnByField($.trim(header.attr("ej-mappingname")));
                            else
                                var toColumn = proxy.getColumnByField($.trim(eDropTarget.attr("ej-mappingname")));
                        }
                        proxy.reorderColumns(column.field, toColumn.field);
                    }
                }
            });
        },

        dragHeaderElement: function () {
            var proxy = this;
            var $dragableElements = this.element.children("div.e-gridheader").find("th.e-headercell").not(".e-detailheadercell,.e-stackedHeaderCell");
            var $visualElement = ej.buildTag('div.e-cloneproperties e-grid', "", { 'height': '20px', 'z-index': 2 }), column;
            //header element columnDrag
            $dragableElements.ejDraggable({
                cursorAt: { top: -50, left: -2 },
                helper: function (event, ui) {
                    if (proxy.element.find(".e-dragclone").length > 0) proxy.element.find(".e-dragclone").remove();
                    var $th;
                    if ($(event.sender.target).hasClass("e-headercell"))
                        $th = $(event.sender.target);
                    else
                        $th = $(event.sender.target).closest("th");
                    column = proxy.getColumnByField($th.find(".e-headercelldiv").attr("ej-mappingname"));
                    if (proxy.model.allowSorting && proxy.model.allowMultiSorting) {
                        var header = $($th).clone();
                        header.find(".e-number").remove();
                        return $visualElement.text(header.text()).clone().width($th.outerWidth() + 2).height($th.height() + 2).css({ "font-size": parseInt(($th.height() + 3) / 2) }).addClass("e-dragclone").appendTo(proxy.element);
                    }
                    else
                        return $visualElement.text($th.text()).clone().width($th.outerWidth() + 2).height($th.height() + 2).css({ "font-size": parseInt(($th.height() + 3) / 2) }).addClass("e-dragclone").appendTo(proxy.element);
                },
                dragStart: function (args) {
                    var target = args.target;
                    var data = { target: target, draggableType: "headercell", column: column };
                    if (proxy._resizer != null && proxy._resizer._expand) {
                        $(".e-dragclone").remove();
                        return false;
                    }
                    proxy._dragActive = true;
                    var $target = proxy.element.find(".e-groupdroparea");
                    if (proxy.model.allowGrouping)
                        proxy.expandGroupDropArea();
                    if (proxy._trigger("columnDragStart", data))
                        return false;
                },
                drag: function (args) {
                    var $target = $(args.target);
                    var data = { target: $target, draggableType: "headercell", column: column };
                    if (proxy._trigger("columnDrag", data))
                        return false;

                    if (($target.hasClass('e-headercelldiv') || $target.hasClass('e-headercell')) && proxy.model.allowReordering) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                        proxy.getHeaderTable().find(".e-reorderindicate").removeClass("e-reorderindicate");
                        if($target.hasClass('e-headercell') && !$target.hasClass('e-stackedHeaderCell')) $target.addClass("e-reorderindicate");
                        else $target.parent().addClass("e-reorderindicate");
                    }
                    if ($target.hasClass('e-groupdroparea') || $target.closest('.e-groupdroparea').length) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                    } else
                        document.body.style.cursor = 'not-allowed';
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        var $target = $(args.target);
                        var data = { target: $target, draggableType: "headercell", column: column };
                        proxy._trigger("columnDrop", data);
                        proxy.element.find(".e-groupdroparea").removeClass("e-hover");
                        proxy.getHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive").removeClass("e-active");
                        $(".e-dragclone").remove();
                        proxy._dragActive = false;
                        proxy.getHeaderTable().find(".e-reorderindicate").removeClass("e-reorderindicate");
                        if (proxy.model.allowGrouping)
                            proxy.collapseGroupDropArea();
                        document.body.style.cursor = '';
                        $(proxy._Indicator).css('display', 'none');
                    }
                }
            });
        },
        _groupHeaderCelldrag: function () {
            //grouped header cell drag.
            var $visualElement = ej.buildTag('div.e-cloneproperties e-grid', "", { 'height': '20px', 'z-index': 2 });
            var proxy = this;
            var $groupedHeaderCells = this.element.children(".e-groupdroparea").find(".e-groupheadercell");
            $groupedHeaderCells.ejDraggable({
                cursorAt: { top: -35, left: -2 },
                helper: function (event, ui) {
                    var $div = $(event.sender.target).closest(".e-grid-icon");
                    return $visualElement.text($(event.sender.target).closest(".e-groupheadercell").text()).clone().width($div.width() + 2).height($div.height() + 2).addClass("e-dragclone").appendTo(proxy.element);
                },
                dragStart: function (args) {
                    var target = args.target;
                    args.model.cursorAt = { top: -35, left: -2 };
                    var data = { target: target, draggableType: "groupheadercell" };
                    if (proxy._trigger("columnDragStart", data))
                        return false;
                },
                drag: function (args) {
                    $(".Sibling").remove();
                    var $target = $(args.target);
                    var data = { target: $target, draggableType: "groupheadercell" };
                    if (proxy._trigger("columnDrag", data))
                        return false;
                    if ($target.closest('div.e-gridcontent').length) {
                        document.body.style.cursor = '';
                        $target.addClass("e-allowDrop");
                    } else
                        document.body.style.cursor = 'not-allowed';
                },
                dragStop: function (args) {
                    if (!args.element.dropped) {
                        var $target = $(args.target);
                        var data = { target: $target, draggableType: "groupheadercell" };
                        proxy._trigger("columnDrop", data);
                        $(".e-dragclone").remove();
                        document.body.style.cursor = '';
                    }
                }
            });

            //grid content drop
            var $contentDroppableElements = this.element.children(".e-gridcontent, .e-gridheader");
            $contentDroppableElements.ejDroppable({
                accept: proxy.element.children("div.e-groupdroparea").find(".e-groupheadercell"),
                drop: function (event, ui) {
                    if (ej.isNullOrUndefined(ui.helper) || !ui.helper.is(":visible") || !ui.draggable.hasClass("e-groupheadercell"))
                        return;
                    var field = $(ui.draggable.context).find("div").attr("ej-mappingname");
                    ui.helper.remove();
                    if (!ej.isNullOrUndefined(field))
                        proxy.ungroupColumn(field);


                }
            });
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.selection = {
        
        selectRows: function (rowIndex, toIndex, target) {
            if (!this.model.allowSelection)
                return false;
            if ($.inArray("row", this.model.selectionSettings.selectionMode) == -1)
                return false;
            var rowIndexCollection = [];
            if ($.isArray(rowIndex)) {
                rowIndexCollection = rowIndex;
                rowIndex = rowIndexCollection[0];
            }
            var $gridRows = $(this.getRows());
            var args = {}, ascend,res;  
			var Data = this._currentJsonData[ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex];
			if(this.model.scrollSettings.allowVirtualScrolling){			
				var pageSize = this.model.pageSettings.pageSize;
				var nameIndx = this.getRowByIndex(rowIndex).attr("name");                
				var pageIndex = !ej.isNullOrUndefined(nameIndx) ? (parseInt(nameIndx)/pageSize) + 1 : rowIndex;
				var trIndex = (rowIndex)%(pageSize);
                var pageto = parseInt(rowIndex / pageSize);
				 var nameattr = pageto * pageSize;
				 if ((!ej.isNullOrUndefined(pageto) && pageto > 0 && $.inArray(nameattr, this.virtualLoadedPages) == -1) && ej.isNullOrUndefined(target)) {
				    trIndex--; this._virIndex = true;
                    this.gotoPage(pageto + 1);
					var proxy=this;
					if(this._dataSource() instanceof ej.DataManager){
					this._queryPromise.done(function(){
						proxy._virtualdata = proxy._currentJsonData;
						proxy._pageTo=pageto;
						proxy.selectRows((rowIndex % pageSize));
						return false;
					});
					}
					else{
					  proxy._virtualdata = proxy._currentJsonData;
					  proxy._pageTo=pageto;
					}
				 }
				 for (var i = 0; i < this._loadedJsonData.length; i++) {
				     if (this._loadedJsonData[i].pageIndex == pageIndex)
				         res = this._loadedJsonData[i].data;
				     if (this.model.pageSettings.totalPages == pageIndex && this._loadedJsonData[i].pageIndex == pageIndex && this._tempPageRendered) {
				         var dupCount = this.model.pageSettings.pageSize - this._previousPageLength;
				         trIndex = trIndex + dupCount;
				     }
				 }
				 Data = ej.isNullOrUndefined(res) ? this._currentJsonData[trIndex] : res[trIndex];
                var nameattr = this._pageTo * pageSize;
                if (!ej.isNullOrUndefined(this._virtualdata) && this._virtualdata.length > 0) {
                    rowIndex = $(document.getElementsByName(nameattr)[trIndex]).index();
                    Data = this._virtualdata[trIndex];
                    this._virtualdata = [];
                }
                else if ($(document.getElementsByName(pageto * pageSize)).length > 0 && !ej.isNullOrUndefined(this._pageTo)) 
                    rowIndex = $(document.getElementsByName(pageto * pageSize)[rowIndex % pageSize]).index();

			}
		    args = { rowIndex: rowIndex, row: $gridRows.eq(rowIndex), data: Data, target: target, prevRow: this.getRowByIndex(this._previousIndex), prevRowIndex : this._previousIndex };
            if (this._trigger("rowSelecting", args))
                return;
			var $gridRows = $(this.getRows());
            if ((this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) && this.model.isEdit && this.model.enableAutoSaveOnSelectionChange) {
                if(!(this.model.editSettings.showAddNewRow && this.model.editSettings.editMode == "normal") || this.getContentTable().find(".e-editedrow").length!=0){ 
                    if (this.endEdit())
                        return;
                    else if(this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                           $("#" + this._id + "_externalEdit").css("display", "none");
                    }
            }
            if (rowIndexCollection.length > 0) {
                for (var i = 0; i < rowIndexCollection.length; i++) {
                    this.selectedRowsIndexes.push(rowIndexCollection[i]);
                }
                rows =this.getRowByIndex(rowIndexCollection);
                if (this.model.scrollSettings.frozenColumns)
                    rows = $(rows[0]).add(rows[1]);
                $(rows).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                Array.prototype.push.apply(this.model.selectedRecords, this.getSelectedRecords());
            }
            else if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(rowIndex)) {
                rowIndex = ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex;
                switch (this.model.selectionType) {
                    case ej.Grid.SelectionType.Multiple:
                        if (this.multiSelectCtrlRequest) {
							this.model.selectedRecords = [];
                            var selectedRowIndex = $.inArray(rowIndex, this.selectedRowsIndexes);
                            selectedRowIndex != -1 && this.clearSelection(rowIndex) && this.selectedRowsIndexes.splice(selectedRowIndex, 0);
                            if (selectedRowIndex == -1) {
                                this.selectedRowsIndexes.push(rowIndex);
                                this.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                                this._virtualSelectAction(pageIndex, rowIndex, pageSize);
                            }
                            Array.prototype.push.apply(this.model.selectedRecords, this.getSelectedRecords());
                            break;
                        }
                    case ej.Grid.SelectionType.Single:
                        this.clearSelection();
                        this.clearColumnSelection();
                        this.selectedRowsIndexes = [];
                        this.model.selectedRecords = [];
                        this._virtualSelectedRecords = {};
                        this.selectedRowsIndexes.push(rowIndex);
                        this.getRowByIndex(rowIndex).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                        this._virtualSelectAction(pageIndex, rowIndex, pageSize);
                        Array.prototype.push.apply(this.model.selectedRecords, this.getSelectedRecords());
                        break;
                }
            } else {
                if (this.model.selectionType == ej.Grid.SelectionType.Multiple) {
                    this.clearSelection();
                    this.clearColumnSelection();
                    this.selectedRowsIndexes = [];
                    this.model.selectedRecords = [];
                    ascend = rowIndex - toIndex < 0;
                    rows = ascend ? this.getRowByIndex(rowIndex, toIndex + 1) : this.getRowByIndex(toIndex, rowIndex + 1);
                    if (this.model.scrollSettings.frozenColumns)
                        rows = $(rows[0]).add(rows[1]);
                     for (var i = ascend ? rowIndex : toIndex, to = ascend ? toIndex : rowIndex; i <= to; i++) {
                        var nameIndx = this.getRowByIndex(i).attr("name");
                        var pageIndex = !ej.isNullOrUndefined(nameIndx) ? (parseInt(nameIndx) / pageSize) + 1 : rowIndex;
                        this._virtualSelectAction(pageIndex, i, pageSize);
                        this.selectedRowsIndexes.push(i);
                    }
                    $(rows).attr("aria-selected", "true").find('.e-rowcell, .e-detailrowcollapse, .e-detailrowexpand').addClass("e-selectionbackground e-active");
                    Array.prototype.push.apply(this.model.selectedRecords, this.getSelectedRecords());
                }
            }
            if (this._selectedRow() !== rowIndex)
                this._selectedRow(rowIndex);
            var args = { rowIndex: this._selectedRow(), row: this.getRowByIndex(this._selectedRow()), data: Data, target: target, prevRow: this.getRowByIndex(this._previousIndex), prevRowIndex : this._previousIndex };
            this._previousIndex = this.selectedRowsIndexes.length ? rowIndex : undefined;
            if ($(this.getRowByIndex(rowIndex)).is('[role="row"]'))
                this._trigger("rowSelected", args);
            // this.multiSelectCtrlRequest = false;
        },

        _virtualSelectAction: function (pageIndex, rowIndex, pageSize) {
            if (this.model.scrollSettings.allowVirtualScrolling && !ej.isNullOrUndefined(rowIndex)) {
                if (!ej.isNullOrUndefined(this._virtualLoadRecords[pageIndex]))
                    this._virtualSelectedRecords[rowIndex] = this._virtualLoadRecords[pageIndex][rowIndex % pageSize];
                else
                    this._virtualSelectedRecords[rowIndex] = this._currentJsonData[rowIndex % pageSize];
            }
        },
        selectCells: function (rowCellIndexes) {
            if (!this.model.allowSelection && !this._allowcellSelection)
                return false;
            var $cell = null;
            var gridRows = this._excludeDetailRows();
            if (this.model.scrollSettings.frozenColumns)
                $cell = this._frozenCell(rowCellIndexes[0][0], rowCellIndexes[0][1][0]);
            else
                $cell = gridRows.eq(rowCellIndexes[0][0]).find(".e-rowcell:eq(" + rowCellIndexes[0][1] + ")");
            if(!ej.isNullOrUndefined(this._previousRowCellIndex) && this._previousRowCellIndex.length != 0 )
                var previousRowCell = $(this.getRowByIndex(this._previousRowCellIndex[0][0]).find(".e-rowcell:eq("+this._previousRowCellIndex[0][1]+")"))	
            var args = { currentCell: $cell, cellIndex: rowCellIndexes[0][1], data: this._currentJsonData[rowCellIndexes[0][0]], previousRowCellIndex: this._previousRowCellIndex, previousRowCell: previousRowCell };
            if (this.model.selectionType == "multiple") {
                args["isCtrlPressed"] = this.multiSelectCtrlRequest;
                args["isShiftPressed"] = this.multiSelectShiftRequest;
            }
            if (this._trigger("cellSelecting", args))
                return;
            switch (this.model.selectionType) {
                case ej.Grid.SelectionType.Multiple:
                    if (this.multiSelectCtrlRequest) {
                        var selectedCellIndex = $.inArray(rowCellIndexes[0][0], this._rowIndexesColl);
                        if (selectedCellIndex != -1)
                            this.selectedRowCellIndexes[selectedCellIndex].cellIndex.push(parseInt(rowCellIndexes[0][1].toString()));
                        else {
                            if ($.inArray(rowCellIndexes[0][0], this._rowIndexesColl) == -1)
                                this._rowIndexesColl.push(rowCellIndexes[0][0]);
                            this.selectedRowCellIndexes.push({ rowIndex: rowCellIndexes[0][0], cellIndex: rowCellIndexes[0][1] });
                        }
                        $cell.addClass("e-cellselectionbackground e-activecell");
                        break;
                    }
                    else if (this.multiSelectShiftRequest && this._previousRowCellIndex != undefined) {
                        this.clearCellSelection();
                        this.clearColumnSelection();
                        this.selectedRowCellIndexes = [];
                        var previousRowIndex = this._previousRowCellIndex[0][0];
                        var previousCellIndex = parseInt(this._previousRowCellIndex[0][1]);
                        var currentRowIndex = rowCellIndexes[0][0];
                        var currentCellIndex = parseInt(rowCellIndexes[0][1]);
                        var newCellColl = [];
                        var min, max;
                        if(this.model.selectionSettings.cellSelectionMode == "box"){
							var $rowCount =  this.model.scrollSettings.frozenColumns ? this.getRows()[0].length - 1 : this.getRows().length - 1;
							var pCellIndex = previousCellIndex < currentCellIndex ? previousCellIndex : currentCellIndex;
							var cCellIndex = pCellIndex == currentCellIndex ? previousCellIndex : currentCellIndex;	
							var newRowColl = [], newCellColl = [];							
							for (var i = pCellIndex; i <= cCellIndex; i++) {								
								min = previousRowIndex;									
								max = currentRowIndex;
								if( min < max ){
									for (var j = min; j <= max; j++) {
										$.inArray(j, newRowColl) == -1 && newRowColl.push(j);											
										this._selectMultipleCells(j, i, currentCellIndex, previousCellIndex);										
									}
								}
								else{
									for (var j = max; j <= min; j++) {
										$.inArray(j, newRowColl) == -1 && newRowColl.push(j);											
										this._selectMultipleCells(j, i, currentCellIndex, previousCellIndex);										
									}
								}
								newCellColl.push(i);								
							}
							for(var i = 0; i < newRowColl.length; i++){
								this.selectedRowCellIndexes.push({ rowIndex: newRowColl[i], cellIndex: newCellColl });										
							}
							this._rowIndexesColl = ej.distinct(newRowColl);
						}
                        else if (currentRowIndex > previousRowIndex) {
                            for (var i = previousRowIndex; i <= currentRowIndex; i++) {
                                newCellColl = [];
                                min = i == previousRowIndex ? previousCellIndex : 0;
                                max = i == currentRowIndex ? currentCellIndex : this.model.columns.length - 1;
                                for (var j = min; j <= max; j++) {
                                    newCellColl.push(j);
                                    this._selectMultipleCells(i, j, currentCellIndex, previousCellIndex);
                                }
                                this.selectedRowCellIndexes.push({ rowIndex: i, cellIndex: newCellColl });
                                this._rowIndexesColl.push(i);
                            }
                        } 						 						
						else {
                            for (var i = previousRowIndex; i >= currentRowIndex; i--) {
                                newCellColl = [];
                                min = i == previousRowIndex ? previousCellIndex : this.model.columns.length - 1;
                                max = i == currentRowIndex ? currentCellIndex : 0;
								if( min > max ){
									for (var j = min; j >= max; j--) {
										newCellColl.push(j);
										this._selectMultipleCells(i, j, currentCellIndex, previousCellIndex);
									}
								}
								else{
									for (var j = max; j >= min; j--) {
										newCellColl.push(j);
										this._selectMultipleCells(i, j, currentCellIndex, previousCellIndex);
									}
								}
                                this.selectedRowCellIndexes.push({ rowIndex: i, cellIndex: newCellColl });
                                this._rowIndexesColl.push(i);
                            }
                        }
                        break;
                    }
                    else {
                        this.clearCellSelection();
                        for (var i = 0; i < rowCellIndexes.length; i++) {
                            if (rowCellIndexes[i][1].length > 1) {
                                var td = gridRows.eq(rowCellIndexes[i][0]).find(".e-rowcell");
                                if (this.model.scrollSettings.frozenColumns)
                                    td = $(gridRows[0]).eq(rowCellIndexes[i][0]).find(".e-rowcell").add($(gridRows[1]).eq(rowCellIndexes[i][0]).find(".e-rowcell"));
                                for (var j = 0; j < td.length; j++) {
                                    var index = (this.model.detailsTemplate != null || this.model.childGrid != null) ? td[j].cellIndex - 1 : j;
                                    if ($.inArray(index, rowCellIndexes[i][1]) != -1) {
                                        $(td[j]).addClass("e-cellselectionbackground e-activecell");
                                        var selectedCellIndex = $.inArray(rowCellIndexes[i][0], this._rowIndexesColl);
                                        if (selectedCellIndex != -1)
                                            this.selectedRowCellIndexes[selectedCellIndex].cellIndex.push(td[j].cellIndex);
                                        else {
                                            this.selectedRowCellIndexes.push({ rowIndex: rowCellIndexes[i][0], cellIndex: [td[j].cellIndex] });
                                            this._rowIndexesColl.push(rowCellIndexes[i][0]);
                                        }
                                    }
                                }
                            }
                            else {
                                if (this.model.scrollSettings.frozenColumns)
                                    this._frozenCell(rowCellIndexes[i][0], rowCellIndexes[i][1][0]).addClass("e-cellselectionbackground e-activecell");
                                else
                                    $(this.getRowByIndex(rowCellIndexes[i][0]).find(".e-rowcell:eq(" + rowCellIndexes[i][1] + ")")).addClass("e-cellselectionbackground e-activecell");
                                this.selectedRowCellIndexes.push({ rowIndex: rowCellIndexes[i][0], cellIndex: rowCellIndexes[i][1] });
                                this._rowIndexesColl.push(rowCellIndexes[i][0]);
                            }
                        }

                        break;

                    }
                case ej.Grid.SelectionType.Single:
                    this.clearCellSelection();
                    this.clearColumnSelection();
                    this.selectedRowCellIndexes = [];
                    if ($.inArray(rowCellIndexes[0][0], this._rowIndexesColl) == -1)
                       this._rowIndexesColl.push(rowCellIndexes[0][0]);
                    this.selectedRowCellIndexes.push({ rowIndex: rowCellIndexes[0][0], cellIndex: rowCellIndexes[0][1] });
                    if (this.model.scrollSettings.frozenColumns)
                        this._frozenCell(rowCellIndexes[0][0], rowCellIndexes[0][1][0]).addClass("e-cellselectionbackground e-activecell");
                    else
                    $(this.getRowByIndex(rowCellIndexes[0][0]).find(".e-rowcell:eq(" + rowCellIndexes[0][1] + ")")).addClass("e-cellselectionbackground e-activecell");
                    break;
            }
            var args = { currentCell: $cell, cellIndex: rowCellIndexes[0][1], data: this._currentJsonData[rowCellIndexes[0][0]], selectedRowCellIndex: this.selectedRowCellIndexes, previousRowCellIndex: this._previousRowCellIndex, previousRowCell: previousRowCell};
            if (!this.multiSelectShiftRequest || ej.isNullOrUndefined(this._previousRowCellIndex)) this._previousRowCellIndex = rowCellIndexes;
            if (this._trigger("cellSelected", args))
                return;
        },
		_selectMultipleCells: function(i, j, currentCellIndex, previousCellIndex){			
			if (this.model.scrollSettings.frozenColumns)
				this._frozenCell(i, j).addClass("e-cellselectionbackground e-activecell");
			else
				$(this.getRowByIndex(i).find(".e-rowcell:eq(" + j + ")")).addClass("e-cellselectionbackground e-activecell");
		},

        
        selectColumns: function (columnIndex, toIndex) {
            if (!this.model.allowcolumnSelection && !this.model.allowSelection)
                return false;
            var gridRows = this.getRows();
            var prevColumnHeaderCell = this.getHeaderTable().find("th.e-headercell")[this._previousColumnIndex];
            var args = { columnIndex: columnIndex == undefined ? toIndex : columnIndex, headerCell: $(this.getHeaderTable().find("th.e-headercell")[columnIndex]), column: this.model.columns[columnIndex], previousColumnIndex: this._previousColumnIndex, prevColumnHeaderCell: prevColumnHeaderCell };
            if (this.model.selectionType == "multiple") {
                args["isCtrlPressed"] = this.multiSelectCtrlRequest;
                args["isShiftPressed"] = this.multiSelectShiftRequest;
            }
            if (this._trigger("columnSelecting", args))
                return;
            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(columnIndex)) {
                columnIndex = ej.isNullOrUndefined(columnIndex) ? toIndex : columnIndex;
                switch (this.model.selectionType) {
                    case ej.Grid.SelectionType.Multiple:
                        if (this.multiSelectCtrlRequest) {
                            var selectedColumnIndex = $.inArray(columnIndex, this.selectedColumnIndexes);
                            selectedColumnIndex != -1 && this.clearColumnSelection(columnIndex) && this.selectedColumnIndexes.splice(selectedColumnIndex, 0);
                            if (selectedColumnIndex == -1) {
                                this.selectedColumnIndexes.push(columnIndex);
                                this._previousColumnIndex = this.selectedColumnIndexes.length ? columnIndex : undefined;
                                (this.model.detailsTemplate != null || this.model.childGrid != null) && ++columnIndex;
                                if (this.model.scrollSettings.frozenColumns)
                                    this._frozenColumnSelection(gridRows, columnIndex);
                                else
                                    for (var i = 0; i < gridRows.length; i++) {
                                        $(gridRows[i].cells[columnIndex]).addClass("e-columnselection");
                                    }
                                $(this.getHeaderTable().find("th.e-headercell")[columnIndex]).addClass("e-columnselection");
                            }
                            break;
                        }
                    case ej.Grid.SelectionType.Single:
                        this.clearSelection();
                        this.clearCellSelection();
                        this.clearColumnSelection();
                        this.selectedColumnIndexes = [];
                        this.selectedColumnIndexes.push(columnIndex);
                        this._previousColumnIndex = this.selectedColumnIndexes.length ? columnIndex : undefined;
                        (this.model.detailsTemplate != null || this.model.childGrid != null) && ++columnIndex;
                        if (this.model.scrollSettings.frozenColumns)
                            this._frozenColumnSelection(gridRows, columnIndex);
                        else
                        for (var i = 0; i < gridRows.length; i++) {
                            $(gridRows[i].cells[columnIndex]).addClass("e-columnselection");
                        }
                        $(this.getHeaderTable().find("th.e-headercell")[columnIndex]).addClass("e-columnselection");
                        break;
                }
            } else {
                if (this.model.selectionType == ej.Grid.SelectionType.Multiple) {
                    this.clearColumnSelection();
                    this.selectedColumnIndexes = [];
                    var indent = 0;
                    if (this.model.detailsTemplate != null || this.model.childGrid != null) {
                        ++columnIndex; ++toIndex;
                        indent = 1;
                    }
                    var startIndex = columnIndex > toIndex ? toIndex : columnIndex;
                    var endIndex = columnIndex > toIndex ? columnIndex + 1 : toIndex + 1;
                    if (this.model.scrollSettings.frozenColumns)
                        this._frozenColumnSelection(gridRows, startIndex, endIndex);
                    else
                    for (var i = startIndex; i < endIndex; i++) {
                        for (var j = 0; j < gridRows.length; j++) {
                            $(gridRows[j].cells[i]).addClass("e-columnselection");
                        }
                        $(this.getHeaderTable().find("th.e-headercell")[i]).addClass("e-columnselection");
                        this.selectedColumnIndexes.push(i - indent);
                    }
                }
            }
            var args = { columnIndex: columnIndex == undefined ? toIndex : columnIndex, headerCell: $(this.getHeaderTable().find("th.e-headercell")[columnIndex]), column: this.model.columns[columnIndex], selectedColumnIndex: this.selectedColumnIndexes, previousColumnIndex: this._previousColumnIndex, prevColumnHeaderCell: prevColumnHeaderCell};            
            if (this._trigger("columnSelected", args))
                return;
        },
        
        clearSelection: function (index) {
            if (this._selectedRow() >= -1) {
                var $gridRows = this._excludeDetailRows(), index;
                
                if (!ej.isNullOrUndefined(index)) {
                    this.getRowByIndex(index).removeAttr("aria-selected").find(".e-selectionbackground").removeClass("e-selectionbackground").removeClass("e-active");
                    index = $.inArray(index, this.selectedRowsIndexes);
                    if (index != -1)
                        this.selectedRowsIndexes.splice(index, 1);
                } else {
                    if (this.model.scrollSettings.frozenColumns > 0)
                        $gridRows = $($gridRows[0]).add($gridRows[1]);
                    $gridRows.removeAttr("aria-selected").find(".e-rowcell, .e-detailrowcollapse, .e-detailrowexpand").removeClass("e-selectionbackground").removeClass("e-active");
                    this.selectedRowsIndexes = [];
                    this.model.selectedRecords = [];
                }
                if (!this.selectedRowsIndexes.length)
                    this._selectedRow(-1);
            }
            return true;
        },

        _excludeDetailRows:function()
		{
			var $gridRows;
			if (!ej.isNullOrUndefined(this.model.detailsTemplate || this.model.childGrid || this.model.showSummary))
                    $gridRows = $(this.getRows()).not(".e-detailrow,.e-gridSummaryRows");
                else
                    $gridRows = $(this.getRows());
			return $gridRows;
		},
        
        clearCellSelection: function (rowIndex, columnIndex) {
            if (this._allowcellSelection) {
                var $gridRows = this._excludeDetailRows(), cellIndex;
                if (ej.isNullOrUndefined(rowIndex)) {
                    if (this.model.scrollSettings.frozenColumns)
                        $gridRows = $($gridRows[0]).add($gridRows[1]);
                    $gridRows.find(".e-rowcell, .e-detailrowcollapse, .e-detailrowexpand").removeClass("e-cellselectionbackground").removeClass("e-activecell");
                    this.selectedRowCellIndexes = [];
                    this._rowIndexesColl = [];
                }
                else {
                    for (var i = 0; i < this.selectedRowCellIndexes.length ; i++) {
                        if (this.selectedRowCellIndexes[i].rowIndex == rowIndex) {
                            cellIndex = $.inArray(columnIndex, this.selectedRowCellIndexes[i].cellIndex);
                            if (this.model.scrollSettings.frozenColumns)
                                this._frozenCell(rowIndex, columnIndex).removeClass("e-cellselectionbackground").removeClass("e-activecell");
                            else
                            $gridRows.eq(rowIndex).find(".e-rowcell").eq(columnIndex).removeClass("e-cellselectionbackground").removeClass("e-activecell");
                            break;
                        }
                    }
                    if (i != this.selectedRowCellIndexes.length) {
                        this.selectedRowCellIndexes[i].cellIndex.splice(cellIndex, 1);
                        if (this.selectedRowCellIndexes[i].cellIndex.length == 0) {
                            this.selectedRowCellIndexes.splice(i, 1);
                            this._rowIndexesColl.splice($.inArray(rowIndex, this._rowIndexesColl), 1);
                        }
                    }
                }
            }
            return true;
        },

        
        clearColumnSelection: function (index) {
            if (this._allowcolumnSelection) {
                var $gridRows = $(this.getRows());
                if (!ej.isNullOrUndefined(index)) {
                    var indent = 0;
                    if (this.model.detailsTemplate != null || this.model.childGrid != null) {
                        ++index; indent = 1;
                    }
                    if (this.model.scrollSettings.frozenColumns) {
                        var frozenDiv = 0; currentIndex = index;
                        if (index >= this.model.scrollSettings.frozenColumns) {
                            frozenDiv = 1;
                            currentIndex = index - this.model.scrollSettings.frozenColumns;
                        }
                        for (var j = 0; j < $gridRows[frozenDiv].length; j++) {
                            $($gridRows[frozenDiv][j].cells[currentIndex]).removeClass("e-columnselection");
                        }
                    }
                    else
                        for (var i = 0; i < $gridRows.length; i++) {
                            $($gridRows[i].cells[index]).removeClass("e-columnselection");
                        }
                    $(this.getHeaderTable().find("th.e-headercell")[index]).removeClass("e-columnselection");
                    this.selectedColumnIndexes.splice(0, index - indent);

                } else {
                    if (this.model.scrollSettings.frozenColumns)
                        $gridRows = $($gridRows[0]).add($gridRows[1]);
                    $gridRows.find(".e-rowcell").removeClass("e-columnselection");
                    $(this.getHeaderTable().find("th.e-headercell")).removeClass("e-columnselection");
                    this.selectedColumnIndexes = [];
                }
            }
            return true;
        },
        
        getSelectedRecords: function () {
            var records = [];
            for (var i = 0; i < this.selectedRowsIndexes.length; i++) {
                if (this.selectedRowsIndexes[i] != -1) {
                    if (this.model.scrollSettings.allowVirtualScrolling)
                        records.push(this._virtualSelectedRecords[this.selectedRowsIndexes[i]]);
                    else
                        records.push(this._currentJsonData[this.selectedRowsIndexes[i]]);
                }
            }
            return records;
        },
        _setCurrentRow: function (requestType) {
            if (requestType == ej.Grid.Actions.Refresh || requestType == ej.Grid.Actions.Ungrouping || requestType == ej.Grid.Actions.Grouping || requestType == ej.Grid.Actions.Filtering || requestType == ej.Grid.Actions.Sorting || requestType == ej.Grid.Actions.Delete || requestType == ej.Grid.Actions.Save || requestType == ej.Grid.Actions.Cancel || requestType == ej.Grid.Actions.Paging) {
                this._selectedRow(-1);
                this.selectedRowsIndexes = [];
            }
        },
        _renderMultiTouchDialog: function () {
            this._customPop = ej.buildTag("div.e-gridpopup", "", { display: "none" });
            var $content = ej.buildTag("div.e-content"), $downTail = ej.buildTag("div.e-downtail e-tail");
            if (this.model.allowMultiSorting) {
                var $selElement = ej.buildTag("span.e-sortdirect e-icon");
                $content.append($selElement);
            }
            if (this.model.selectionType == ej.Grid.SelectionType.Multiple) {
                var $selElement = ej.buildTag("span.e-rowselect e-icon");
                $content.append($selElement);
            }
            this._customPop.append($content);
            this._customPop.append($downTail);
            this.element.append(this._customPop);
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    
    ej.widget("ejGrid", "ej.Grid",  {
        
        _rootCSS: "e-grid",
        // widget element will be automatically set in this
        element: null,
        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,
        keyConfigs: /** @lends ejGrid# */{
            focus: "e",
            insertRecord: "45", //Insert
            deleteRecord: "46", // delete
            editRecord: "113", //F2
            saveRequest: "13", // enter
            cancelRequest: "27", //Esc
            nextPage: "34", // PgDn
            previousPage: "33", // PgUp
            lastPage: "ctrl+alt+34", //"CtrlAltPgDn",
            firstPage: "ctrl+alt+33", //"CtrlPlusAltPlusPgUp",
            nextPager: "alt+34", //"AltPlusPgDown",
            previousPager: "alt+33", //"AltPlusPgUp",
            firstCellSelection: "36", //"Home",
            lastCellSelection: "35", //"End",
            firstRowSelection: "ctrl+36", //"CtrlPlusHome",
            lastRowSelection: "ctrl+35", //"CtrlPlusEnd",
            upArrow: "38", //Up arrow
            downArrow: "40", //Down arrow
            rightArrow: "39", //Right arrow
            leftArrow: "37", //Left arrow
            moveCellRight: "9", //tab
            moveCellLeft: "shift+9", //shifttab
            selectedGroupExpand: "alt+40", //"AltPlusDownArrow",
            totalGroupExpand: "ctrl+40", //"CtrlPlusDownArrow",
            selectedGroupCollapse: "alt+38", //"AltPlusUpArrow",
            totalGroupCollapse: "ctrl+38" //"CtrlPlusUpArrow",
        },
        _ignoreOnPersist: [
            "query", "isEdit", "toolbarClick", "queryCellInfo", "mergeCellInfo", "currentViewData", "enableAltRow", "enableRTL", "contextClick", "contextOpen",
            "rowDataBound", "rowTemplate", "detailsDataBound", "detailsTemplate", "childGrid", "summaryRows", "toolbarSettings",
            "editSettings", "allowMultiSorting", "enableAutoSaveOnSelectionChange", "locale", "allowScrolling", "allowCellMerging",
            "allowTextWrap", "cssClass", "dataSource", "groupSettings.enableDropAreaAnimation", "enableRowHover", "showSummary", "allowGrouping",
            "enableHeaderHover", "allowKeyboardNavigation", "scrollSettings.frozenRows", "scrollSettings.frozenColumns", "enableTouch", "editSettings.rowPosition", "editSettings.showAddNewRow", "contextMenuSettings.enableContextMenu",
        ],
        ignoreOnExport: [
            "isEdit", "toolbarClick", "query", "queryCellInfo", "selectionType", "currentViewData", "enableRTL", "rowDataBound", "rowTemplate",
            "detailsDataBound", "detailsTemplate", "editSettings", "pageSettings", "enableAutoSaveOnSelectionChange", "localization", "allowScrolling",
            "cssClass", "dataSource", "groupSettings.enableDropAreaAnimation", "enableRowHover", "allowSummary",
            "enableHeaderHover", "allowKeyboardNavigation"
        ],
        observables: ["dataSource", "selectedRowIndex", "pageSettings.currentPage"],
        _tags: [{
            tag: "columns",
            attr: ["allowEditing", "allowFiltering", "allowGrouping", "allowSorting", "cssClass", "customAttributes", "dataSource", "defaultValue",
			"disableHtmlEncode", "editTemplate", "editType", "foreignKeyField", "foreignKeyValue", "headerTemplateID", "headerText", "isFrozen",
			"isIdentity", "isPrimaryKey", "isUnbound", "textAlign", "templateID", "textAlign", "headerTextAlign",
            "validationRules.minlength", "validationRules.maxlength", "validationRules.range", "validationRules.number", "validationRules.required",
            "editParams.decimalPlaces", [{ tag: "commands", attr: ["type", "buttonOptions"] }]
            ],
            content: "template"
        }, {
            tag: "summaryRows",
            attr: ["showCaptionSummary", "showTotalSummary", [{
                tag: "summaryColumns", attr: ["customSummaryValue", "dataMember", "displayColumn", "summaryType"]
            }]]
        }, {
            tag: "stackedHeaderRows",
            attr: [
            [{
                tag: "stackedHeaderColumns", attr: ["headerText", "column"]
            }]]
        }, {
            tag: "filterSettings.filteredColumns", attr: []
        }, {
            tag: "sortSettings.sortedColumns", attr: []
        }],
        _dataSource: ej.util.valueFunction("dataSource"),
        _selectedRow: ej.util.valueFunction("selectedRowIndex"),
        _currentPage: ej.util.valueFunction("pageSettings.currentPage"),
        // default model
        defaults: /** @lends ejGrid# */ {

            
            allowPaging: false,
            
            showColumnChooser: false,
            
            gridLines: "both",
            
            allowSorting: false,
            
            showStackedHeader: false,
            
            
            
            

            

            

            
            selectedRecords: [],
            stackedHeaderRows: [],
            
            allowFiltering: false,
            allowMultipleExporting: false,
            
            allowSelection: true,
            
            allowGrouping: false,
            
            showSummary: false,
            
            allowResizing: false,
            
            allowResizeToFit: false,
            
            allowTextWrap: false,
            
            allowCellMerging: false,
            
            enableRowHover: true,
            
            enablePersistence: false,
            
            selectedRowIndex: -1,
            
            allowSearching: false,
            
            enableHeaderHover: false,
            
            allowReordering: false,
            
            allowKeyboardNavigation: true,
            
            enableTouch: true,
            
            selectionType: "single",
            
            dataSource: null,
            
            cssClass: "",
            
            allowScrolling: false,
            
            locale: "en-US",
            
            enableAutoSaveOnSelectionChange: true,
            
            allowMultiSorting: false,
            exportToExcelAction: "",
            exportToWordAction: "",
            exportToPdfAction: "",
            
            editSettings:  {
                
                allowEditing: false,
                
                showAddNewRow: false,
                
                allowAdding: false,
                
                showAddNewRow: false,
                
                allowDeleting: false,
                
                editMode: "normal",
                
                rowPosition: "top",
                
                dialogEditorTemplateID: null,
                
                allowEditOnDblClick: true,
                
                externalFormTemplateID: null,
                
                inlineFormTemplateID: null,
                
                formPosition: "bottomleft",
                
                showConfirmDialog: true,
                
                showDeleteConfirmDialog: false,

                _groupingCollapsed: [],
            },
            
            selectionSettings:  {

                
                selectionMode: ["row"],

                
                enableToggle: false,

                
                cellSelectionMode: "flow"
            },

            
            pageSettings:  {
                
                pageSize: 12,
                
                pageCount: 8,
                
                currentPage: 1,
                
                totalPages: null,
                
                enableTemplates: false,
                
                showDefaults: false,
                
                template: null,
                
                totalRecordsCount: null,
                
                enableQueryString: false,
                printMode: "allpages"
            },
            
            groupSettings:  {
                
                showDropArea: true,
                
                showToggleButton: false,
                
                enableDropAreaAnimation: true,
                
                showGroupedColumn: true,
                
                showUngroupButton: true,
                
                enableDropAreaAutoSizing: true,
                
                captionFormat: null,
                
                groupedColumns: []
            },

            

            contextMenuSettings: {
                
                enableContextMenu: false,
                
                contextMenuItems: ["Add Record", "Edit Record", "Delete Record", "Sort In Ascending Order", "Sort In Descending Order", "Next Page", "Last Page", "Previous Page", "First Page", "Save", "Cancel", "Grouping", "Ungrouping"],
                
                customContextMenuItems: []
            },
            
            filterSettings:  {
                
                filterType: "filterbar",
                
                filterBarMode: "immediate",
                
                showFilterBarStatus: true,
                statusBarWidth: 450,
                
                showPredicate: false,
                
                filteredColumns: [],
                
                maxFilterChoices: 1000,
                
                enableCaseSensitivity: false
            },
            searchSettings:  {
                
                fields: [],
                
                key: "",
                
                operator: "contains",
                
                ignoreCase: true

            },
            
            sortSettings:  {

                

                

                
                sortedColumns: []
            },
            
            toolbarSettings:  {
                
                showToolbar: false,
                
                toolbarItems: [],
                
                customToolbarItems: []
            },
            
            minWidth: 0,
            
            scrollSettings: 
            {
                
                width: "auto",
                
                height: 0,
                
                enableTouchScroll: true,
                
                allowVirtualScrolling: false,
                
                virtualScrollMode: "normal",
                
                frozenRows: 0,
                
                frozenColumns: 0
            },
            

            

            

            

            

            

            

            

            

            

            

            

            
            summaryRows: [],
            
            enableRTL: false,
            
            enableAltRow: true,
            currentViewData: null,
            
            detailsTemplate: null,
            
            childGrid: null,
            
            keySettings: null,
            
            rowTemplate: null,
            
            detailsDataBound: null,
            
            rowDataBound: null,
            
            queryCellInfo: null,
            
            
            mergeCellInfo: null,
            
            create: null,
            
            actionBegin: null,
            
            actionComplete: null,
            
            actionFailure: null,
            
            beginEdit: null,
            
            endEdit: null,
            
            endAdd: null,
            
            endDelete: null,
            
            beforeBatchAdd: null,
            
            beforeBatchSave: null,
            
            beforeBatchDelete: null,
            
            batchAdd: null,
            
            batchDelete: null,
            
            cellSave: null,
            
            cellEdit: null,
            
            resizeStart: null,
            
            resizeEnd: null,
            
            resized: null,
            
            load: null,
            
            destroy: null,
            
            rowSelecting: null,
            
            rowSelected: null,
            
            cellSelecting: null,
            
            cellSelected: null,
            
            columnSelecting: null,
            
            columnSelected: null,
            
            columnDragStart: null,
            
            columnDrag: null,
            
            columnDrop: null,
            
            dataBound: null,
            
            recordClick: null,
            
            recordDoubleClick: null,
            
            templateRefresh: null,
            
            rightClick: null,
            
            detailsCollapse: null,
            
            detailsExpand: null,
            
            toolbarClick: null,
            

            contextOpen: null,
            
            contextClick: null,

            

            

            

            

            

            

            

            

            

            

            

            

            

            

            

            

            

            

            

            

            
			 

            

            

            

            

            

            

            

            

            

            

            

            

            

            columns: [],
            
            query: null,
            isEdit: false,
            
            isResponsive: false,
            
            enableResponsiveRow: false,
        },
        dataTypes: {
            dataSource: "data",
            query: "data",
            columns: "array",
            childGrid: "parent",
            gridLines: "enum",
            summaryRows: "array",
            stackedHeaderRows: "array",
            toolbarSettings: {
                toolbarItems: "array",
                customToolbarItems: "array"
            },
            contextMenuSettings: {
                contextMenuItems: "array",
                customContextMenuItems: "array"
            },
            selectionSettings: {
                selectionMode: "array",
                selectedRecords: "array"
            },
            sortSettings: {
                sortedColumns: "array"
            },
            filterSettings: {
                filteredColumns: "array",
                filterType: "enum",
                filterBarMode: "enum",
            },
            groupSettings: {
                groupedColumns: "array"
            },
            editSettings: {
                editMode: "enum",
                formPosition: "enum",
                rowPosition: "enum",
            }
        },

        _columns: function (index, property, value, old) {
            var $header = this.element.find(".e-gridheader");
            $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
            this._headerCellgDragDrop();
            this.refreshContent(true);
            this._trigger("refresh");
        },
        _summaryRows: function (index, property, value, old) {
            if (property == "showTotalSummary" || property == "showCaptionSummary") {
                var indx = index.summaryRows;
                var val = value.toLowerCase() == "true" || value.toLowerCase() == "false" ? ej.parseJSON(value) : false;
                this.option("summaryRows")[indx][property] = val;
            }
            this.element.find(".e-gridfooter").remove();
            this._renderGridFooter().insertAfter(this.getContent());
            if (property == "showCaptionSummary" || property == "title") {
                this._isCaptionSummary = this.option("summaryRows")[indx]["showCaptionSummary"];
                this.model.showSummary = this._isCaptionSummary;
                if (this.model.groupSettings.groupedColumns.length != 0)
                    this._refreshCaptionSummary();
            }
        },
        _summaryRows_summaryColumns: function (index, property, value, old) {
            if (property == "displayColumn" || property == "dataMember") {
                if (ej.isNullOrUndefined(this.getColumnByField(value)))
                    return;
            }
            this._createSummaryRows(this.getFooterTable());
            if (this.element.find(".e-groupcaptionsummary").length != 0)
                this._refreshCaptionSummary();
        },
        _stackedHeaderRows_stackedHeaderColumns: function (index, property, value, old) {
            this._refreshStackedHeader();
        },
        _sortSettings_sortedColumns: function (index, property, value, old) {
            var colName, direction;
            var sortObj = this.model.sortSettings.sortedColumns[index["sortSettings.sortedColumns"]];
            if (property == "field") {
                colName = this.getColumnByField(value) != null ? value : null;
                direction = (sortObj.direction == "ascending" || sortObj.direction == "descending") ? sortObj.direction : null;
            }
            else if (property == "direction") {
                colName = this.getColumnByField(sortObj.field) != null ? sortObj.field : null;
                direction = (value == "ascending" || value == "descending") ? value : null;
            }
            if (colName != null && direction != null)
                this.sortColumn(colName, direction);
        },
        _filterSettings_filteredColumns: function (index, property, value, old) {
            var field, operator, matchcase, predicate, filtervalue;
            var filterObj = this.model.filterSettings.filteredColumns[index["filterSettings.filteredColumns"]];
            switch (property) {
                case "field":
                    field = this.getColumnByField(value) != null ? value : null;
                    operator = this._map(ej.FilterOperators, filterObj.operator);
                    filtervalue = filterObj.value;
                    predicate = (filterObj.predicate == "and" || filterObj.predicate == "or") ? filterObj.predicate : null;
                    matchcase = filterObj.matchcase;
                    break;
                case "matchcase":
                    field = this.getColumnByField(filterObj.field) != null ? filterObj.field : null;
                    operator = this._map(ej.FilterOperators, filterObj.operator);
                    filtervalue = filterObj.value;
                    predicate = (filterObj.predicate == "and" || filterObj.predicate == "or") ? filterObj.predicate : null;
                    matchcase = value.toLowerCase() == "true" || value.toLowerCase() == "false" ? ej.parseJSON(value) : false;
                    break;
                case "operator":
                    field = this.getColumnByField(filterObj.field) != null ? filterObj.field : null;
                    operator = this._map(ej.FilterOperators, value);
                    filtervalue = filterObj.value;
                    predicate = (filterObj.predicate == "and" || filterObj.predicate == "or") ? filterObj.predicate : null;
                    matchcase = filterObj.matchcase;
                    break;
                case "predicate":
                    field = this.getColumnByField(filterObj.field) != null ? filterObj.field : null;
                    operator = this._map(ej.FilterOperators, filterObj.operator);
                    filtervalue = filterObj.value;
                    predicate = (value == "and" || value == "or") ? value : null;
                    matchcase = filterObj.matchcase;
                    break;
                case "value":
                    field = this.getColumnByField(filterObj.field) != null ? filterObj.field : null;
                    operator = this._map(ej.FilterOperators, filterObj.operator);
                    filtervalue = value;
                    predicate = (filterObj.predicate == "and" || filterObj.predicate == "or") ? filterObj.predicate : null;
                    matchcase = filterObj.matchcase;
                    break;
            }
            if (field != null && operator != null && filtervalue != null && predicate != null && matchcase != null)
                this.filterColumn(field, operator, filtervalue, predicate, matchcase);
        },
        _map: function (object, value) {
            var data = $.map(object, function (obj) {
                if (obj === value)
                    return obj;
            });
            return data.length != 0 ? data[0] : null;
        },
        _refreshCaptionSummary: function () {
            var temp = document.createElement('div');
            temp.innerHTML = ['<table>', $.render[this._id + "_GroupingTemplate"](this.model.currentViewData, { groupedColumns: this.model.groupSettings.groupedColumns }), '</table>'].join("");
            this.getContentTable().get(0).replaceChild(temp.firstChild.firstChild, this.getContentTable().get(0).lastChild);
            this.refreshContent();
        },
        
        getContentTable: function () {
            return this._gridContentTable;
        },

        setGridContentTable: function (value) {
            this._gridContentTable = value;
        },
        
        getContent: function () {
            return this._gridContent;
        },

        setGridContent: function (value) {
            this._gridContent = value;
        },
        
        getHeaderContent: function () {
            return this._gridHeaderContent;
        },

        setGridHeaderContent: function (value) {
            this._gridHeaderContent = value;
        },
        
        getHeaderTable: function () {
            return this._gridHeaderTable;
        },

        setGridHeaderTable: function (value) {
            this._gridHeaderTable = value;
        },
        
        getRows: function () {
            return this._gridRows;
        },
        
        getFilteredRecords: function () {
            return this._filteredRecords;
        },
        
        getRowByIndex: function (from, to) {
            try {
                var gridRows = this.getRows(), $gridRows = this._excludeDetailRows(), $row = $();
                if ($.isArray(from)) {
                    for (var i = 0; i < from.length; i++) {
                        if (this.model.scrollSettings.frozenColumns > 0) {
                            $row.push(gridRows[0][from[i]]);
                            $row.push(gridRows[1][from[i]]);
                        }
                        else
                            $row.push(gridRows[from[i]]);
                    }
                    return $row;
                }
                else if (ej.isNullOrUndefined(to)) {
                    if (this.model.scrollSettings.frozenColumns > 0) {
                        $row.push(gridRows[0][from]);
                        $row.push(gridRows[1][from]);
                        return $row;
                    }
                    return $(($gridRows).not(".e-virtualrow")[from]);
                } else {
                    if (this.model.scrollSettings.frozenColumns > 0) {
                        $row.push($(gridRows[0]).slice(from, to));
                        $row.push($(gridRows[1]).slice(from, to));
                        return $row;
                    }
                    return $($gridRows.not(".e-virtualrow").slice(from, to));
                }
            } catch (e) {
                return $();
            }
        },
        
        getColumnIndexByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column;
        },
        
        getColumnIndexByHeaderText: function (headerText, field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["headerText"] == headerText) {
                    if (field) {
                        if (ej.isNullOrUndefined(this.model.columns[column]["field"]) || this.model.columns[column]["field"] == "")
                            break;
                    }
                    else
                        break;
                }
            }
            return column;
        },
        
        getIndexByRow: function ($tr) {
            var gridRows = this.getRows(), $gridRows = this._excludeDetailRows(), rowIndex;
            if (this.model.scrollSettings.frozenColumns > 0) {
                rowIndex = $(gridRows[0]).index($tr);
                if (rowIndex == -1)
                    rowIndex = $(gridRows[1]).index($tr);
                return rowIndex;
            } else
                return $gridRows.not(".e-virtualrow").index($tr);
        },
        
        getPrimaryKeyFieldNames: function () {
            if (this._primaryKeys.length != 0)
                return this._primaryKeys;
            for (var key = 0, col = this.model.columns, cLen = col.length; key < cLen; key++) {
                if (col[key]["isPrimaryKey"])
                    this._primaryKeys.push(col[key]["field"]);
            }
            return this._primaryKeys;
        },
        
        getVisibleColumnNames: function (headerText) {
            return this._visibleColumns;
        },
        
        getHiddenColumnNames: function (headerText) {
            return this._hiddenColumns;
        },
        
        getColumnByField: function (field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"] == field)
                    break;
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },
        
        getsortColumnByField: function (field) {
            for (var column = 0; column < this.model.sortSettings.sortedColumns.length; column++) {
                if (this.model.sortSettings.sortedColumns[column]["field"] == field)
                    break;
            }
            return column == this.model.sortSettings.sortedColumns.length ? null : this.model.sortSettings.sortedColumns[column];
        },
        
        getColumnByHeaderText: function (headerText, field) {
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["headerText"] == headerText) {
                    if (field) {
                        if (ej.isNullOrUndefined(this.model.columns[column]["field"]) || this.model.columns[column]["field"] == "")
                            break;
                    }
                    else
                        break;
                }
            }
            return column == this.model.columns.length ? null : this.model.columns[column];
        },
        
        getCurrentViewData: function () {
            return this._currentJsonData;
        },
        
        getColumnFieldNames: function () {
            var columnNames = [];
            for (var column = 0; column < this.model.columns.length; column++) {
                if (this.model.columns[column]["field"])
                    columnNames.push(this.model.columns[column]["field"]);
            }
            return columnNames;
        },
        
        getBrowserDetails: function () {
            var b = navigator.userAgent.match(/(firefox|chrome|opera|msie|safari)\s?\/?(\d+(.\d+)*)/i);
            if (!!navigator.userAgent.match(/Trident\/7\./))
                return { browser: "msie", version: $.uaMatch(navigator.userAgent).version };
            return { browser: b[1].toLowerCase(), version: b[2] };
        },
        _initPrivateProperties: function () {
            this._click = 0;
            this._gridHeaderTable = null;
            this._gridWidth = this.element.width();
            this._id = this.element.attr("id");
            this._gridRows = null;
            this._gridContentTable = null;
            this._gridContent = null;
            this._remoteSummaryData = null;
            this._gridSort = null;
            this._gridHeaderContent = null;
            this._gridFooterContent = null;
            this._gridFooterTable = null;
            this._gridRecordsCount = this._dataSource() !== null ? (this.model.pageSettings.totalRecordsCount == null ? this._dataSource().length : this.model.pageSettings.totalRecordsCount) : 0;
            this._links = null;
            this._gridPager = null;
            this._cSortedColumn = null;
            this._cSortedDirection = null;
            this._$curSElementTarget = null;
            this._gridFilterBar = null;
            this._$curFieldName = null;
            this._$prevFieldName = null;
            this._mediaStatus = false;
            this._$fDlgIsOpen = false;
            this._$colType = null;
            this._$colFormat = null;
            this._$prevColType = null;
            this._$prevSElementTarget = null;
            this._currentFilterColumn = null;
            this._filteredRecordsCount = null;
            this._filteredRecords = [];
            this.filterColumnCollection = [];
            this._previousFilterCount = null;
            this._excelFilter = null;
            this._isExcelFilter = this.model.filterSettings.filterType == "excel";
            this._primaryKeys = [];
            this._identityKeys = [];
            this._primaryKeyValues = [];
            this._modifiedRecords = [];
            this._addedRecords = [];
            this._tdsOffsetWidth = [];
            this._deletedRecords = [];
            this._disabledToolItems = $();
            this._validationRules = {};
            this._groupedColumns = [];
            this._scolumns = [];
            this._currentJsonData = [];
            this._groupingColumnIndex = 0;
            this._dataManager = this._dataSource() instanceof ej.DataManager ? this._dataSource() : this._dataSource() != null ? ej.DataManager(this._dataSource()) : null;
            if (this._dataManager != null && this.model.allowScrolling && this.model.scrollSettings.allowVirtualScrolling && this.model.pageSettings.totalRecordsCount != null && this._dataManager.dataSource.json != null)
                this._dataManager.dataSource.json.splice(this.model.pageSettings.totalRecordsCount);
            this._isRemoteSaveAdaptor = (this._dataSource() instanceof ej.DataManager && this._dataSource().adaptor instanceof ej.remoteSaveAdaptor);
            this._isLocalData = true;
            this._disabledResizingColumns = [];
            this._disabledSortableColumns = [];
            this._disabledGroupableColumns = [];
            this._disabledFilterableColumns = [];
            this._disabledEditableColumns = [];
            this._hiddenColumns = [];
            this._visibleColumns = [];
            this._visibleColumnsField = [];
            this._hiddenColumnsField = [];
            this._ccVisibleColumns = [];
            this._ccHiddenColumns = [];
            this._sortedColumns = [];
            this.multiSortRequest = false;
            this.multiSelectCtrlRequest = false;
            this.multiSelectShiftRequest = false;
            this._enableSelectMultiTouch = false;
            this._enableSortMultiTouch = false;
            this._templateRefresh = false;
            this.initialRender = false;
            this._isAddNew = false;
            this._fieldColumnNames = {};
            this._headerColumnNames = {};
            this._virtualLoadRecords = {};
            this._virtualSelectedRecords = {};
            this.selectedRowsIndexes = [];
            this._isReorder = false;
            this._searchString = "";
            this._searchCount = null;
            this._columnsWidthCollection = [];
            this._Indicator = null;
            this._resizer = null;
            this._bulkEditCellDetails = {
                cellValue: null,
                rowIndex: -1,
                columnIndex: -1,
                fieldName: null,
                _data: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                rowData: null
            };
            this.batchChanges = {
                added: [],
                deleted: [],
                changed: []
            };
            this._bulkEditTemplate = $();
            this._confirmDialog = null;
            this._confirmedValue = false;
            this._lastRow = false;
            this._isVirtualRecordsLoaded = false;
            this._scrollValue = 0;
            this._currentTopFrozenRow = this.model.scrollSettings.frozenRows;
            this._rowHeightCollection = [];
            this._scrollObject = null;
            this._customPop = null;
            this.selectedRowCellIndexes = [];
            this._rowIndexesColl = [];
            this.selectedColumnIndexes = [];
            this.commonQuery = $.extend(true, {}, this.model.query);
            if (ej.gridFeatures.group) {
                this._rowCol = this._captionSummary();
                this._isCaptionSummary = (this._rowCol != null && this._rowCol.length) > 0 ? true : false;
            }
            this.phoneMode = this.model.isResponsive && document.documentElement.clientWidth < 360 ? true : false;
            if (this.model.selectionSettings.selectionMode.length > 0)
                this._initSelection();
            this._mediaQuery = false;
            this._columnChooserList = null;
            this._$headerCols = null;
            this._$contentCols = null;
            this._detailsOuterWidth = null;
            this._editForm = null;
            this._cloneQuery = null;
        },
        _init: function () {
            this._trigger("load");
            if (ej.isNullOrUndefined(this.model.query) || !(this.model.query instanceof ej.Query))
                this.model.query = ej.Query();
            if (ej.gridFeatures.common)
                this._initScrolling();
            this._initPrivateProperties();
            if (this.model.enableResponsiveRow)
                this.element.addClass("e-responsive");
            if (ej.gridFeatures.edit) {
                if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding)
                    this._processEditing();
            }
            this._checkDataBinding();
        },
        _initColumns: function (object) {
            while (object.items != undefined)
                object = object.items[0];
            if (this.model.columns.length == 0 && object) {
                for (var field in object) {
                    if (object.hasOwnProperty(field) && (typeof (object[field]) != "object" || object[field] instanceof Date)) {
                        var value = object[field];
                        this.model.columns.push({
                            field: field,
                            type: value != null ? (value.getDay ? (value.getHours() > 0 || value.getMinutes() > 0 || value.getSeconds() > 0 || value.getMilliseconds() > 0 ? "datetime" : "date") : typeof (value)) : null
                        });
                    }
                }
                this.model.columns.length && this._renderAfterColumnInitialize();
            } else {
                for (var index = 0; index < this.model.columns.length; index++) {
                    this.model.columns[index].field = ej.isNullOrUndefined(this.model.columns[index].field) ? "" : this.model.columns[index].field;
                    if (ej.isNullOrUndefined(this.model.columns[index].type)) {
                        var $field;
                        if (this.model.columns[index].dataSource && this.model.columns[index].foreignKeyValue)
                            $field = this.model.columns[index].dataSource[0][this.model.columns[index].foreignKeyValue];
                        else
                            $field = !ej.isNullOrUndefined(this.model.columns[index].field) ? ej.getObject(this.model.columns[index].field, object) : null;
                        this.model.columns[index].type = $field != null ? ($field.getDay ? ($field.getHours() > 0 || $field.getMinutes() > 0 || $field.getSeconds() > 0 || $field.getMilliseconds() > 0 ? "datetime" : "date") : typeof ($field)) : null;
                    }
                    else if (this.model.columns[index]["type"] == "date" && this.model.columns[index].format == undefined)
                        if (ej.isNullOrUndefined(Globalize))
                            $.extend(this.model.columns[index], { format: "{0:" + Globalize.culture().calendars.standard.patterns.d + "}" });
                        else
                            $.extend(this.model.columns[index], { format: "{0:M/d/yyyy}" });
                    else if (this.model.columns[index]["type"] == "datetime" && this.model.columns[index].format == undefined)
                        if (ej.isNullOrUndefined(Globalize))
                            $.extend(this.model.columns[index], { format: "{0:" + Globalize.culture().calendars.standard.patterns.d + " " + Globalize.culture().calendars.standard.patterns.t + "}" });
                        else
                            $.extend(this.model.columns[index], { format: "{0:M/d/yyyy h:mm tt}" });
                }
            }
        },
        _initSelection: function () {
            var mode = this.model.selectionSettings.selectionMode;
            for (i = 0; i < mode.length; i++) {
                this["_allow" + mode[i] + "Selection"] = true;
            }
        },
        _checkDataBinding: function () {
            if (!this.model.columns.length && (((this._dataSource() == null || !this._dataSource().length) && !(this._dataSource() instanceof ej.DataManager)) || ((this._dataSource() instanceof ej.DataManager) && this._dataManager.dataSource.url == undefined && !this._dataSource().dataSource.json.length))) {
                this._renderAlertDialog();
                this._alertDialog.find(".e-content").text(this._getLocalizedLabels("EmptyDataSource"));
                this._alertDialog.ejDialog("open");
                return;
            }
            if (this.model.enablePersistence)
                this.model.query = new ej.Query();
            this._initialRenderings();
            this.initialRender = true;
            this.model.enableRTL && this.element.addClass("e-rtl");
            if (this.model.allowFiltering && this._isExcelFilter)
                this._renderExcelFilter();
            if (this.model.cssClass != null)
                this.element.addClass(this.model.cssClass);
            if (this.model.allowGrouping)
                this.element.append(this._renderGroupDropArea());
            if (this.model.toolbarSettings.showToolbar)
                this.element.append(this._renderToolBar());
            var columns = this.model.columns;
            if (columns && columns.length) {
                var expands = [];
                if (typeof columns[0] === "string")
                    for (var i = 0; i < columns.length; i++)
                        columns[i] = { field: columns[i] };
                for (var i = 0; i < columns.length; i++) {
                    if (!columns[i].field || columns[i].field.indexOf('.') === -1) continue;
                    this._getExpands(columns[i].field, expands);
                }
                this.model.query.expand(expands);
                this.commonQuery.expand(expands);
                this._renderAfterColumnInitialize();
            }
            if (this.model.allowPaging)
                this.element.append(this._renderGridPager());
            if (this.model.contextMenuSettings.enableContextMenu)
                this.element.append(this._renderContext());
            if ($.isFunction($.fn.ejWaitingPopup)) {
                this.element.ejWaitingPopup({ showOnInit: false });
                $("#" + this._id + "_WaitingPopup").addClass("e-gridwaitingpopup");
            }
            if (this.model.scrollSettings.allowVirtualScrolling) {
                this._loadedJsonData = [];
                this._prevPage = 1;
            }
            if (this._dataSource() instanceof ej.DataManager) {
                this.element.ejWaitingPopup("show");
                if (this._dataSource().ready != undefined) {
                    var proxy = this;
                    this._dataSource().ready.done(function (args) {
                        proxy._initDataSource();
                        proxy.model.dataSource = ej.DataManager(args.result);
                    });
                } else {
                    this.element.ejWaitingPopup("show");
                    this._initDataSource();
                }
            } else {
                this._ensureDataSource();
                this._trigger("actionBegin");
                this._initGridRender();
            }
            if (this.model.showColumnChooser)
                this._renderColumnChooser();
        },
        _renderColumnChooser: function () {
            var $columnButton = ej.buildTag("button .e-ccButton", this._getLocalizedLabels("Columns"), { 'float': 'right' }).attr("type", "button");
            this.element.prepend($columnButton);
            $columnButton.ejButton({
                prefixIcon: "e-down-arrow",
                imagePosition: "imageright",
                contentType: "textandimage",
                type: 'button',
                click: $.proxy(this._ccClickHandler, this),
                width: 90
            });
            var buttHeight = $columnButton.outerHeight();
            $columnButton.css('margin-top', 0 - (buttHeight));
            var elementTop = parseInt(this.element.css('margin-top'),10);
            this.element.css('margin-top', elementTop + buttHeight);
            var $mainDiv = ej.buildTag("div");
            var $outerDiv = ej.buildTag("div .e-grid e-columnChooser", '', {}, { id: this._id + "ccDiv" });
            var $searchBox = ej.buildTag("div.e-searchbox e-fields").append(ej.buildTag("input#" + this._id + "_ccSearchBox.e-ejinputtext e-filtertext", {}, {}, { "type": "text" }))
            var $sapnDiv = ej.buildTag('span .e-searchfind e-icon')
            $searchBox.append($sapnDiv);
            var $listOuterDiv = ej.buildTag('div', '', { 'height': '228px' }, { id: this._id + "liScrollerDiv" })
            this._renderColumnChooserData(false);
            $listOuterDiv.append(this._columnChooserList);
            $outerDiv.append($searchBox);
            $outerDiv.append($listOuterDiv);

            var $splitterDiv = ej.buildTag('div .e-columnChooserSplitter', '', { 'border-bottom': '0px' });
            $outerDiv.append($splitterDiv);
            if (this.model.enableRTL) {
                $buttonDiv = ej.buildTag('div', '', { 'float': 'left', 'margin-top': '7px', 'margin-right': '-13px' });
                var $cancelButton = ej.buildTag("button", this._getLocalizedLabels("Cancel"), { 'margin-right': '7px', 'margin-left': '9px' });
                $($sapnDiv).addClass("e-rtl");
            }
            else {
                $buttonDiv = ej.buildTag('div', '', { 'float': 'right', 'margin-top': '7px', 'margin-right': '-13px' });
                var $cancelButton = ej.buildTag("button", this._getLocalizedLabels("Cancel"), { 'margin-right': '20px', 'margin-left': '6px' });
            }
            var $addButton = ej.buildTag("button", this._getLocalizedLabels("Done"));
            $buttonDiv.append($addButton);
            $buttonDiv.append($cancelButton);
            $outerDiv.append($buttonDiv);
            $addButton.ejButton({
                click: $.proxy(this._addButtonCC, this),
                showRoundedCorner: true,
                width: 66
            });
            $cancelButton.ejButton({
                click: $.proxy(this._cancelButtonHandler, this),
                showRoundedCorner: true,
                width: 66
            });
            $outerDiv.insertBefore(this.element)
            $outerDiv.ejDialog({ width: 'auto', beforeClose: $.proxy(this._columnChooserBeforeClose, this), showOnInit: false, allowKeyboardNavigation: false, enableResize: false, "enableRTL": this.model.enableRTL, "cssClass": this.model.cssClass, showHeader: false, width: 260 });
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 10) {
                var searchBox = $(".e-columnChooser").find("input#" + this._id + "_ccSearchBox")[0];
                ej.ieClearRemover(searchBox);
            }
        },
        _renderColumnChooserData: function (refresh) {
            var selectAllCheck = this.model.columns.length == this.getVisibleColumnNames().length;
            this._ccCheckedLength = this.getVisibleColumnNames().length;
            this._ccCheckBoxList = [];
            var $listBox = ej.buildTag("div", '', { 'margin-left': '0px', 'width': '250px' }), count = 0;
            for (var index = -1; index < this.model.columns.length; index++) {
                var isSelectAll = index == -1;
                if (isSelectAll || this.model.columns[index].showInColumnChooser) {
                    var column = this.model.columns[index];
                    var colValue = isSelectAll ? '(Select All)' : ej.isNullOrUndefined(column.headerText) || column.headerText == "" ? column.field == "" ? null : column.field : column.headerText;
                    if (!ej.isNullOrUndefined(colValue) || isSelectAll) {
                        var $innerDiv = ej.buildTag('div', '', {}, { 'class': 'e-columnChooserListDiv' });
                        var styleAttr = {};
                        var id = isSelectAll ? this._id + 'selectAll' : this._id + colValue.replace(/\s|\.|[^a-zA-Z0-9]|&nbsp/g, "_");
                        var inDom = $listBox.find("#" + id).length; inDom && count++;
                        var $input = ej.buildTag('input', '', styleAttr, { 'id': (!inDom ? id : id + count + ""), 'value': colValue, 'type': 'checkbox', "ej-field": isSelectAll ? '' : column.field, "ej-headertext": isSelectAll ? '' : column.headerText, 'class': isSelectAll ? 'e-selectall' : '' });
                        var label = ej.buildTag('label', colValue, { 'font-size': '13px' }, { 'for': (!inDom ? id : id + count + "") });
                        $innerDiv.append($input);
                        $innerDiv.append(label);
                        $listBox.append($innerDiv);
                        var checked = !isSelectAll && !ej.isNullOrUndefined(column.visible) ? column.visible : true;
                        $input.ejCheckBox({
                            checked: isSelectAll ? selectAllCheck : checked,
                            change: $.proxy(this._columnChooserCheckChange, this)
                        });
                        if (!isSelectAll && !ej.isNullOrUndefined(column.visible))
                            $input[column.visible ? "attr" : "removeAttr"]("checked", true);
                    }
                }
            }
            if (!refresh)
                this._columnChooserList = $listBox;
            else {
                this._columnChooserList.empty().append($listBox.children());
                $("#" + this._id + "liScrollerDiv").is(":visible") && $("#" + this._id + "liScrollerDiv").ejScroller('refresh');
            }
            this._ccCheckBoxList = this._columnChooserList.find("input:checkbox").not(".e-selectall");
        },
        _columnChooserCheckChange: function (args) {
            var checked = args.isChecked;
            if (args.model.id == this._id + 'selectAll') {
                if (!checked)
                    this._ccCheckBoxList.filter(function () { if ($(this).hasClass("e-checkbox") && $(this).prop("checked")) return this; }).ejCheckBox({ checked: checked });
                else
                    this._ccCheckBoxList.filter(":not(:checked)").ejCheckBox({ checked: checked });
            }
            else {
                if (!checked)
                    this._ccCheckedLength--;
                else
                    this._ccCheckedLength++;
                this._columnChooserList.find('input.e-selectall').ejCheckBox('model.checked', this._ccCheckedLength == this.model.columns.length);
                checked = $("#" + this._id + "ccDiv").find(".e-columnChooserListDiv [aria-checked=true]").length != 0;
                this.element[checked ? "attr" : "removeAttr"]("checked", true);
            }
            var operation = !checked ? "addClass" : "removeClass";
            $("#" + this._id + "ccDiv").find("button[aria-describedby='Done']")[operation]("e-disable");
        },
        _columnChooserBeforeClose: function () {
            $(".e-columnChoosertail").remove();
            $(".e-columnChoosertailAlt").remove();
            $("#" + this._id + "ccDiv").find("button[aria-describedby='Done']").removeClass("e-disable");
            $("#" + this._id + "_ccSearchBox").val('');
            var args = {};
            args.target = {}; args.target.value = '';
            this._columnChooserSearch(args);
        },
        _columnChooserSearch: function (e) {
            if (e.type == 'click') {
                e.target.value = '';
                $("#" + this._id + "_ccSearchBox").val('');
            }
            var val = e.target.value;
            var span = $("#" + this._id + "_ccSearchBox").next('span');
            if (val != '') {
                span.removeClass("e-searchfind");
                span.addClass("e-cancel");
            }
            else {
                span.removeClass("e-cancel");
                span.addClass("e-searchfind");
            }
            $(".e-cancel").bind('click', $.proxy(this._columnChooserSearch, this));
            var currentCheckedItemsData = this.model.columns;
            var columnCollection = [], gridColumns = [];
            if (val != '')
                currentCheckedItemsData = ej.DataManager(this.model.columns).executeLocal(ej.Query().where("headerText", ej.FilterOperators.startsWith, val, true));
            currentCheckedItemsData.forEach(function (obj) {
                if (obj.showInColumnChooser)
                    columnCollection.push(obj.headerText);
            });
            if (!ej.isNullOrUndefined($("#nomatches")[0]))
                $("#nomatches").remove();
            var div = $("#" + this._id + "ccDiv").find("#" + this._id + "liScrollerDiv")

            var divs = this._columnChooserList.find(".e-columnChooserListDiv");

            for (i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i].showInColumnChooser)
                    gridColumns.push(this.model.columns[i].headerText)
            }

            for (var index = 0; index < gridColumns.length; index++) {
                var colValue = gridColumns[index];
                var indx = columnCollection.indexOf(colValue)
                if (!ej.isNullOrUndefined(colValue))
                    divs.eq(index + 1)[indx == -1 ? "addClass" : "removeClass"]("e-hide");
            }

            if (columnCollection.length == 0) {
                var $labeldiv = ej.buildTag('div#nomatches', '');
                var $label = ej.buildTag('span', 'No Matches Found');
                $labeldiv.append($label);
                $(div).append($labeldiv);
            }
            divs.eq(0)[columnCollection.length == 0 ? "addClass" : "removeClass"]("e-hide");
            $("#" + this._id + "liScrollerDiv").ejScroller('refresh')
        },
        _addButtonCC: function () {
            this._visibleColumns = [];
            this._hiddenColumns = [];
            this._visibleColumnsField = [];
            this._hiddenColumnsField = [];
            this._columnChooserClick = true;
            var args = {}; args.requestType = "columnchooser";
            var chbxs = this._columnChooserList.find("input:checkbox").not('.e-selectall');
            for (var i = 0, len = chbxs.length; i < len; i++) {
                var ele = $(chbxs[i]), hTxt = ele.attr("ej-headertext"), field = ele.attr("ej-field");
                this[chbxs[i].checked ? "_visibleColumns" : "_hiddenColumns"].push(hTxt);
                this[chbxs[i].checked ? "_visibleColumnsField" : "_hiddenColumnsField"].push(field != "" ? field : hTxt);
            }
            var array1 = this._visibleColumns;
            var array2 = this._hiddenColumns;
            var arr = [], obj, duparr;
            obj = $.merge($.merge([], array1), array2);
            duparr = this._isDuplicate(obj);
            this._trigger("actionBegin", args);
            if (duparr) {
                this.showColumns(this._visibleColumnsField);
                this.hideColumns(this._hiddenColumnsField);
            }
            else {
                this.showColumns(this._visibleColumns);
                this.hideColumns(this._hiddenColumns);
            }
            $("#" + this._id + "ccDiv").ejDialog('close');
            $(".e-columnChoosertail").remove();
            this.refreshScrollerEvent();
            args = { requestType: "columnchooser", removedcolumns: [], addedcolumns: [], visiblecolumns: this.getVisibleColumnNames(), hiddencolumns: this.getHiddenColumnNames() };
            this._ccColumnUpdate(args.addedcolumns, this.getVisibleColumnNames(), this._ccVisibleColumns);
            this._ccColumnUpdate(args.removedcolumns, this.getHiddenColumnNames(), this._ccHiddenColumns);
            this._trigger("actionComplete", args);
        },
        _ccColumnUpdate: function (args, getColumns, ccColumns) {
            for (i = 0; i < getColumns.length; i++) {
                if ($.inArray(getColumns[i], ccColumns) == -1)
                    args.push(getColumns[i]);
            }
        },
        _isDuplicate: function (arr) {
            var temp, count = [], duplicate = [];
            for (var i = 0; i < arr.length; i++) {
                temp = arr[i];
                if (count[temp] >= 1)
                    count[temp] = count[temp] + 1;
                else
                    count[temp] = 1;
            }
            for (temp in count) {
                if (count[temp] > 1)
                    return true;
            }
            return false;
        },
        _cancelButtonHandler: function () {
            $("#" + this._id + "ccDiv").ejDialog('close');
            $(".e-columnChoosertailAlt").remove();
            $(".e-columnChoosertail").remove();
        },
        _ccClickHandler: function (e) {
            var dlgWidth = 230, xPos;
            var chooserButton = this.element.find(".e-ccButton");
            xPos = chooserButton.offset().left + chooserButton.width();
            var dialogObj = $("#" + this._id + "ccDiv").data('ejDialog')
            if (dialogObj && dialogObj.isOpened()) {
                dialogObj.close();
                $(".e-columnChoosertail").remove();
                $(".e-columnChoosertailAlt").remove();
            }
            else {
                $("#" + this._id + "ccDiv").ejDialog({ width: '230px', height: '309px', position: { X: xPos - dlgWidth, Y: chooserButton.offset().top + 35 } })
                   .ejDialog("open");
                var maxZindex = parseInt($("#" + this._id + "ccDiv_wrapper").css('z-index'));
                var $tailDiv = ej.buildTag("div #" + this._id + "_ccTail .e-columnChoosertail", '', { 'display': 'block', 'position': 'absolute', 'left': xPos - 29, 'top': chooserButton.offset().top + 15 });
                var $tailDiv2 = ej.buildTag("div #" + this._id + "_ccTailAlt .e-columnChoosertailAlt", '', { 'display': 'block', 'z-index': maxZindex + 2, 'position': 'absolute', 'left': xPos - 29, 'top': chooserButton.offset().top + 16 });
                $tailDiv.insertBefore($("#" + this._id + "ccDiv_wrapper"));
                $tailDiv2.insertBefore($("#" + this._id + "ccDiv_wrapper"));
            }
            this._refreshColumnChooserList();
            this._ccVisibleColumns = this.getVisibleColumnNames();
            this._ccHiddenColumns = this.getHiddenColumnNames();
            $("#" + this._id + "liScrollerDiv").ejScroller({ height: '228', width: '230', buttonSize: 0 });
            $("#" + this._id + "liScrollerDiv").ejScroller('refresh');
            if (this.getBrowserDetails().browser == 'chrome')
                $('.e-columnChooser .e-hscrollbar').attr('style', 'height: 10px !important;');
            $(".e-ejinputtext").bind('keyup', $.proxy(this._columnChooserSearch, this))
        },
        _refreshColumnChooserList: function (collection) {
            this._ccCheckedLength = this.getVisibleColumnNames().length;
            var chbxs = this._columnChooserList.find("input:checkbox").not('.e-selectall');
			var duparr = this._isDuplicate($.merge($.merge([], this._visibleColumns), this._hiddenColumns));
            for (var i = 0, len = chbxs.length; i < len; i++) {
                var ele = $(chbxs[i]), hTxt = ele.attr("ej-headertext"), field = ele.attr("ej-field"), flag = undefined, isDup = chbxs.filter("[ej-headertext='" + hTxt + "']").length;
                if (this.model.allowGrouping && !this.model.groupSettings.showGroupedColumn && $.inArray($(chbxs[i]).attr("ej-field"), this.model.groupSettings.groupedColumns) != -1) {
                    $(chbxs[i]).parents(".e-columnChooserListDiv").css("display", "none")
                    chbxs[i].checked = false;
                }
                else
                    $(chbxs[i]).parents(".e-columnChooserListDiv").css("display", "block")
                var colValue = duparr ? (field == "" ? hTxt : field) : hTxt;
				flag = this[duparr ? "_hiddenColumnsField" : "_hiddenColumns"].indexOf(colValue) != -1;
                ele[!flag ? "attr" : "removeAttr"]("checked", true);
                ele.ejCheckBox("model.checked", !flag);
            }
            this._columnChooserList.find("input:checkbox.e-selectall").ejCheckBox({checked:this.model.columns.length==this.getVisibleColumnNames().length})
        },
        _initDataSource: function () {
            this._isLocalData = (!(this._dataSource() instanceof ej.DataManager) || (this._dataSource().dataSource.offline || this._isRemoteSaveAdaptor));
            this._ensureDataSource();
            this._trigger("actionBegin");
            var queryPromise = this._dataSource().executeQuery(this.model.query);
            if (this._dataManager.dataSource.table != null)
                this._dataManager.dataSource.table.css("display", "none");
            if (!this.element.is(":visible"))
                this.element.ejWaitingPopup("hide");
            queryPromise.done(ej.proxy(function (e) {
                this.element.ejWaitingPopup("hide");
                if (!this.model.columns.length && !e.count) {
                    proxy._renderAlertDialog();
                    proxy._alertDialog.find(".e-content").text(proxy._getLocalizedLabels("EmptyDataSource"));
                    proxy._alertDialog.ejDialog("open");
                    proxy.element.ejWaitingPopup("hide");
                    return;
                }
                if (!ej.isNullOrUndefined(e.aggregates))
                    this._remoteSummaryData = e.aggregates;
                this.model.currentViewData = e.result;
                if (this.model.pageSettings.totalRecordsCount != null && this.model.filterSettings.filteredColumns.length == 0)
                    this._gridRecordsCount = this.model.pageSettings.totalRecordsCount;
                else if (e.count == 0 && e.result.length)
                    this._gridRecordsCount = e.result.length;
                else
                    this._gridRecordsCount = e.count;
                if (this.model.filterSettings.filteredColumns.length > 0)
                    this._filteredRecordsCount = e.count;
                if (this.getPager() != null)
                    this.model.pageSettings.totalRecordsCount = this._gridRecordsCount;
                this._initGridRender();
            }, this));
            var proxy = this;
            queryPromise.fail(function (e) {
                var args = { error: e.error };
                proxy._trigger("actionFailure", args)
            })
        },
        _initialRenderings: function () {
            if (this.model.groupSettings.groupedColumns.length) {
                var sortedColumns = new Array();
                for (var i = 0; i < this.model.sortSettings.sortedColumns.length; i++) {
                    if (ej.isNullOrUndefined(this.model.sortSettings.sortedColumns[i].direction))
                        this.model.sortSettings.sortedColumns[i].direction = ej.sortOrder.Ascending;
                    sortedColumns.push(this.model.sortSettings.sortedColumns[i].field);
                }
                if(this.model.allowGrouping){
					for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
						if ($.inArray(this.model.groupSettings.groupedColumns[i], sortedColumns) == -1)
							this.model.sortSettings.sortedColumns.push({ field: this.model.groupSettings.groupedColumns[i], direction: ej.sortOrder.Ascending });
					}
				}
            }
        },
        _getExpands: function (field, arr) {
            var splits = field.split('.'), tmp = "";
            splits.splice(splits.length - 1, 1);
            for (var i = 0; i < splits.length; i++, tmp = "") {
                for (var j = 0; j < i; j++)
                    tmp += splits[j] + "/";
                tmp = tmp + splits[i];
                if (arr.indexOf(tmp) === -1)
                    arr.push(tmp);
            }
        },
        _renderAfterColumnInitialize: function () {
            this.element.append(this._renderGridHeader());
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar")
                this._renderFiltering();
			if(this.model.gridLines != "both")
				this._showHeaderGridLines();
            if (this.model.allowPaging)
                this.element.append(this.element.find(".e-pager").first());
        },
        _ensureDataSource: function (args) {
            if (this._dataSource() == null && !(this._dataSource() instanceof ej.DataManager)) {
                if (!ej.isNullOrUndefined(args) && args.requestType == "add")
                    this.dataSource([]);
                else
                    return;
            }
            this.model.query.requiresCount();
            var queryManagar = this.model.query;
            var cloneQuery = queryManagar.clone();
            if (!(this._dataSource() instanceof ej.DataManager))
                this.model.currentViewData = this._dataSource();
            if (this._isLocalData && (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) && (!ej.isNullOrUndefined(this._cModifiedData) || !ej.isNullOrUndefined(this._cAddedRecord))) {
                if (ej.isNullOrUndefined(this._cAddedRecord)) {
                    for (var index = 0; index < this._primaryKeys.length; index++)
                        queryManagar = queryManagar.where(this._primaryKeys[index], ej.FilterOperators.equal, this._primaryKeyValues[index]);
                    var currentData = this._dataManager.executeLocal(queryManagar);
                    if (!(this._dataSource() instanceof ej.DataManager))
                        $.extend(this._dataSource()[$.inArray(currentData.result[0], this._dataSource())], this._cModifiedData);
                    else
                        $.extend(this._dataSource().dataSource.json[$.inArray(currentData.result[0], this._dataSource().dataSource.json)], this._cModifiedData);
                    this._cModifiedData = null;
                } else {
                    var tmpRcrd = this._cAddedRecord;
                    this._cAddedRecord = null;
                    (this._dataSource() instanceof ej.DataManager) ? this._dataSource().dataSource.json.unshift(tmpRcrd) : this._dataSource(undefined, true).splice(0, 0, tmpRcrd);
                }
                queryManagar.queries = cloneQuery.queries;
                if (!this.model.editSettings.showAddNewRow)
                    this.model.isEdit = false;
            }
            if (args && this.model.editSettings.allowDeleting && args.requestType == "delete" && this._excludeDetailRows().length == 1 && this.model.pageSettings.currentPage != 1)
                this._currentPage(this.model.pageSettings.totalPages - 1)
            if (args && this.model.editSettings.allowDeleting && args.requestType == "delete" && !ej.isNullOrUndefined(this._cDeleteData) && this._isLocalData) {
                if (!(this._dataSource() instanceof ej.DataManager)) {
                    var index = $.inArray(this._cDeleteData[0], this._dataSource());
                    this._dataSource(undefined, true).splice(index, 1);
                }
                else {
                    var index = $.inArray(this._cDeleteData[0], this._dataSource().dataSource.json);
                    this._dataSource().dataSource.json.splice(index, 1);
                }
            }
            if (this.model.sortSettings.sortedColumns.length) {
                for (var i = this.model.sortSettings.sortedColumns.length - 1; i >= 0; i--)
                    queryManagar.sortBy(this.model.sortSettings.sortedColumns[i].field, this.model.sortSettings.sortedColumns[i].direction);

            }

            if (this.model.allowSearching && this.model.searchSettings.key.length) {
                var searchDetails = this.model.searchSettings;
                searchDetails.fields = searchDetails.fields.length != 0 ? searchDetails.fields : this.getColumnFieldNames();
                queryManagar.search(searchDetails.key ,searchDetails.fields, searchDetails.operator || "contains", searchDetails.ignoreCase || true);
                if (!this.initialRender && args.requestType == "searching")
                    this._currentPage(1);
                if (!(this._dataSource() instanceof ej.DataManager))
                    this._filteredRecords = this._dataManager.executeLocal(queryManagar).result;
            }

            if (this.model.allowFiltering && this.model.filterSettings.filteredColumns.length) {
                var predicate, firstFilterCondition = this.model.filterSettings.filteredColumns[0];
                if (this._isExcelFilter) {
                    this._excelFilter.getPredicate(this.model.filterSettings.filteredColumns, null, true);
                    var predicates = this._excelFilter._predicates[0];
                    for (var prop in predicates) {
                        var obj = predicates[prop], isTake = obj["from"] != undefined;
                        if (isTake)
                            queryManagar.skip(obj["from"] == "top" ? 0 : this._gridRecordsCount - obj["take"]).take(obj["take"]);
                        else
                            predicate = predicate != undefined ? predicate["and"](obj) : obj;
                    }
                }
                else {
                    if (!firstFilterCondition.isComplex)
                        predicate = ej.Predicate(firstFilterCondition.field, firstFilterCondition.operator, firstFilterCondition.value, !firstFilterCondition.matchcase);
                    else
                        predicate = firstFilterCondition;
                    for (var i = 1; i < this.model.filterSettings.filteredColumns.length; i++) {
                        if (!this.model.filterSettings.filteredColumns[i].isComplex)
                            predicate = predicate[this.model.filterSettings.filteredColumns[i].predicate](this.model.filterSettings.filteredColumns[i].field, this.model.filterSettings.filteredColumns[i].operator, this.model.filterSettings.filteredColumns[i].value, !this.model.filterSettings.filteredColumns[i].matchcase);
                        else
                            predicate = predicate["and"](this.model.filterSettings.filteredColumns[i]);
                    }
                }
                predicate && queryManagar.where(predicate);
                if (this._isLocalData) {
                    var fresults = this._dataManager.executeLocal(queryManagar);
                    this._filteredRecordsCount = isTake ? fresults.result.length : fresults.count;
                    var lastPage = (this._filteredRecordsCount % this.model.pageSettings.pageSize == 0) ? (this._filteredRecordsCount / this.model.pageSettings.pageSize) : (parseInt(this._filteredRecordsCount / this.model.pageSettings.pageSize, 10) + 1);
                    if (this._currentPage() > lastPage)
                        this._currentPage(lastPage);
                    this._filteredRecords = this._dataManager.executeLocal(queryManagar).result;
                } else if (!ej.isNullOrUndefined(args) && args.requestType == ej.Grid.Actions.Filtering)
                    this._currentPage(1);
            }

            if (this.model.summaryRows) {
                this._setSummaryAggregate(queryManagar);
            }

            if (this.model.allowPaging || (this.model.scrollSettings.allowVirtualScrolling && this.model.allowScrolling)) {
                if (this._currentPage() == 0) {
                    if (this._prevPageNo == 0 || this._prevPageNo == null)
                        this._currentPage(1);
                    else
                        this._currentPage(this._prevPageNo);
                }
                queryManagar.page(this._currentPage(), this.model.pageSettings.pageSize);
            }

            if (this.model.allowGrouping) {
                var cloned = queryManagar.clone();
                if (this.model.allowPaging && this.model.groupSettings.groupedColumns.length) {
                    cloned.queries = cloned.queries.slice(0, cloned.queries.length - 1);
                }
                for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                    queryManagar.group(this.model.groupSettings.groupedColumns[i]);
                    cloned.group(this.model.groupSettings.groupedColumns[i]);
                }
                if (this.model.groupSettings.groupedColumns.length)
                    this._setAggreatedCollection(cloned);
            }
            if (args != undefined && args.requestType == "add" && this._isLocalData && this.model.groupSettings.groupedColumns.length == 0 && this.model.scrollSettings.frozenColumns == 0 && this.model.scrollSettings.frozenRows == 0)
                !(this._dataSource() instanceof ej.DataManager) ? this._dataSource().unshift(args.data) : this._dataSource().dataSource.json.unshift(args.data);
            if ((!ej.isNullOrUndefined(args) && args.action == "add") && !ej.isNullOrUndefined(this.model.parentDetails)) {
                var column = this.getColumnByField(this.model.parentDetails.parentKeyField);
                var ix = $.inArray(column, this.model.columns)
                if (ix == -1) {
                    var newdata = {};
                    newdata[this.model.parentDetails.parentKeyField] = this.model.parentDetails.parentKeyFieldValue;
                    $.extend(true, this.model.currentViewData[0], newdata);
                }
            }
            this._cloneQuery = queryManagar.clone();
                if (this._isLocalData) {
                var dataMgrJson = this._dataManager.dataSource.json;
                var dataSource = this._dataSource().dataSource;
                if (!ej.isNullOrUndefined(dataSource) && this._dataSource() instanceof ej.DataManager)
                    this._dataManager.dataSource.json = dataMgrJson != dataSource.json ? dataSource.json : dataMgrJson;
                var result = this._dataManager.executeLocal(queryManagar);
                if (this.model.scrollSettings.allowVirtualScrolling && this.model.pageSettings.currentPage == this.model.pageSettings.totalPages - 1)
                    this._prevPageRendered = true;
                if (this.model.scrollSettings.allowVirtualScrolling && !this._prevPageRendered && result.result.length != this.model.pageSettings.pageSize && this.model.pageSettings.totalPages == this.model.pageSettings.currentPage) {
                    var pageQuery = ej.pvt.filterQueries(queryManagar.queries, "onPage");
                    queryManagar.queries.splice($.inArray(pageQuery[0], queryManagar.queries), 1);
                    queryManagar.page(this._currentPage() - 1, this.model.pageSettings.pageSize);
                    var lastPageResult = this._dataManager.executeLocal(queryManagar);
                    lastPageResult.result.splice(0, result.result.length);
                    this._previousPageRecords = $.extend(true, [], lastPageResult.result);
                    this._previousPageLength = result.result.length;
                    this._currentPageData = result.result;
                    ej.merge(lastPageResult.result, result.result);
                    this.model.currentViewData = lastPageResult.result;
                    this._lastPageRendered = true;
                }
                else if (this._lastPageRendered && this.model.pageSettings.currentPage == this.model.pageSettings.totalPages - 1) {
                    var count = this.model.pageSettings.pageSize - this._previousPageLength;
                    for (var dupRow = 0; dupRow < count; dupRow++) {
                        var removeEle = this.getRows()[this.getRows().length - (this.model.pageSettings.pageSize - dupRow)];
                        removeEle.remove();
                    }
                    this._tempPageRendered = true;
                    this.model.currentViewData = result.result;
                }
                else
                    this.model.currentViewData = result.result;
                this._gridRecordsCount = result.count;
                this._remoteSummaryData = result.aggregates;
                this._searchCount = this._searchString.length ? result.count : null;
                this.model.groupSettings.groupedColumns.length && this._setAggregates();
            }

        },
        _initGridRender: function () {
            this.addInitTemplate();
            if (this.model.scrollSettings.frozenColumns > 0)
                this.addFrozenTemplate();
            this.model.allowGrouping && this.addGroupingTemplate();
            if (this.model.allowResizing || this.model.allowResizeToFit)
                this._resizer = new ej.gridFeatures.gridResize(this);
            if (this.model.keySettings)
                $.extend(this.model.keyConfigs, this.model.keySettings);
            if (this.model.allowTextWrap == true)
                this.element.addClass("e-wrap");
            this.render();
            this._trigger("dataBound", {});
            if (this.model.allowGrouping && ej.gridFeatures.dragAndDrop)
                this._headerCellgDragDrop();
            if (this.model.allowReordering && ej.gridFeatures.dragAndDrop) {
                this._headerCellreorderDragDrop();
                this._initIndicators();
            }
            this._wireEvents();
            if (this.model.allowGrouping && !ej.isNullOrUndefined(this.model.serverProperties)) {
                this.model._groupingCollapsed = this.model.serverProperties._groupingCollapsed;
                for (var i = 0; i < this.model._groupingCollapsed.length; i++) {
                    var content = this.getContent().find(".e-recordplusexpand");
                    var tr = content.filter("td[ej-mappingname='" + this.model._groupingCollapsed[i].key + "'    ][ej-mappingvalue='" + this.model._groupingCollapsed[i].value + "']");
                    if (tr.length > 1 && !ej.isNullOrUndefined(this.model._groupingCollapsed[i].parent)) {
                        var parent = this.model._groupingCollapsed[i].parent;
                        tr = tr.filter(function (e) { return $(this).parents(".e-tabletd").parent("tr").prev("tr").find(".e-recordplusexpand[ej-mappingvalue=" + parent + "]").length })
                    }

                    this.expandCollapse(tr);
                }
            }
            this.initialRender = false;
            if (this.model.width && !this.model.allowScrolling)
                this.element.width(this.model.width);
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding)
                this._processEditing();
            if (this.model.parentDetails) {  //refreshes parent scroller on child expand
                var id = this.model.parentDetails.parentID, parentObj = $("#" + id).data("ejGrid");
                parentObj.model.allowScrolling && parentObj._refreshScroller({ requestType: "refresh" });
            }
            if (this.element.closest('tr').hasClass('e-detailrow') && !this.model.parentDetails) {
                var parentObj = this.element.closest('tr.e-detailrow').closest('.e-grid').data("ejGrid");
                parentObj.model.allowScrolling && parentObj.getScrollObject().refresh();
            }
            if (this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this.model.filterSettings.filterType == "excel")) {
                this._renderFilterDialogs();
                if (this.model.enableResponsiveRow)
                    this._renderResponsiveFilter();
            }
            if (this.model.allowGrouping && this.model.showSummary)
                this._refreshGroupSummary();
        },
        _getMetaColGroup: function () {
            var $colgroup = ej.buildTag("colgroup");
            for (var i = 0; i < this.model.columns.length; i++) {
                var $col = $(document.createElement("col"));
                this.model.columns[i]["visible"] === false && $col.css("display", "none");
                !ej.isNullOrUndefined(this.model.columns[i]["cssClass"]) && $col.addClass(this.model.columns[i]["cssClass"]);
                if ( this.model.allowGrouping && !this.model.groupSettings.showGroupedColumn && $.inArray(this.model.columns[i]["field"], this.model.groupSettings.groupedColumns) != -1)
                    $col.css("display", "none");
                $colgroup.append($col);
            }
            return $colgroup;
        },
        _alternateRow: function () {
            return this.getIndex() % 2 == 0 ? "e-row" : "e-alt_row";
        },
        addInitTemplate: function () {
            var headerCellDiv = this.element.find(".e-headercelldiv:not(.e-emptyCell)"), templates = {}, firstVisible = true;
            var tbody = document.createElement('tbody'), $tbody = $(tbody);
            if (this.model.rowTemplate == null) {
                var tr = document.createElement('tr'),
                    $tr = $(tr),
                    columns = this.model.columns,
                    i;
                if (this._gridRecordsCount)
                    this._initColumns(this.model.currentViewData[0] != undefined ? this.model.currentViewData[0] : this.model.currentViewData.value);
                else if (this._isLocalData && (this.model.dataSource != null && this.model.dataSource.length || this._dataManager.dataSource.json.length))
                    this._initColumns(this.model.dataSource[0] != undefined ? this.model.dataSource[0] : this._dataManager.dataSource.json[0]);
                var helpers = { _gridFormatting: this.formatting };
                $.views.helpers(helpers);

                var viewHelper = {};
                viewHelper["_foreignKey"] = this._foreignKeyBinding; // move to common methods
                $.views.helpers(viewHelper);

                if (this.model.detailsTemplate || this.model.childGrid || this.model.detailsDataBound) {
                    var $tdDetailCell = ej.buildTag("td.e-detailrowcollapse", "<div class='e-icon e-gnextforward'></div>");
                    $tr.append($tdDetailCell);
                }
                for (i = 0; i < this.model.columns.length; i++) {
                    var $tdCell = ej.buildTag("td.e-rowcell");
                    if (this.model.isResponsive)
                        $tdCell.attr("data-cell", this.model.columns[i]["headerText"]);
                    if (columns[i]["visible"] == false)
                        $tdCell.addClass("e-hide");
                    else {
                        if (firstVisible && (this.model.detailsTemplate != null || this.model.childGrid != null || this.model.detailsDataBound != null))
                            $tdCell.addClass('e-detailrowvisible');
                        firstVisible = false;
                    }
                    !this.model.groupSettings.showGroupedColumn && $tdCell.addClass("{{for ~groupedColumns}}" +
                        " {{if #data == '" + this.model.columns[i]["field"] + "'}}e-hide{{/if}}" +
                        "{{/for}}");
                    if (!ej.isNullOrUndefined(columns[i]["templateID"] || columns[i]["template"])) {
                        var viewHelper = {}, index, htxt = columns[i].headerText;
                        viewHelper["_" + this._id + "ColumnTemplating"] = ej.proxy(this._gridTemplate, null, this, index);
                        $.views.helpers(viewHelper);
                        if(!ej.isNullOrUndefined(htxt.match(/[^0-9\s\w]/g)))
                            htxt = htxt.replace(/[^0-9\s\w]/g,"_");
                        $("#" + this._id + htxt + i + "_Template").remove();
                        var scriptElement = this._createTemplateElement(columns[i]);
                        if ((columns[i].field == "") || ej.isNullOrUndefined(columns[i].field))
                            this.model.columns[i]["allowGrouping"] = this.model.columns[i]["allowFiltering"] = this.model.columns[i]["allowSorting"] = false;
                        $tdCell.addClass("e-templatecell").html("{{:~_" + this._id + "ColumnTemplating('" + scriptElement.id + "','" + i + "')}}");
                    } else {
                        var splits = (columns[i].field || "").split("."), sLen = splits.length - 1, braces = "";
                        while (sLen) {
                            braces += "(";
                            sLen--;
                        }
                        var columnType = columns[i].type || columns[i].editType
                        switch (columnType) {
                            case "boolean":
                            case "booleanedit":
                                if (this.model.locale != "en-US" && this._getLocalizedLabels("True") != ej.Grid.locale["en-US"]["True"] && columns[i]["displayAsCheckbox"] != true)
                                    $tdCell.html('{{if ' + columns[i].field + '}}' + this._getLocalizedLabels("True") + '{{else}}' + this._getLocalizedLabels("False") + '{{/if}}');
                                else
                                    $tdCell.addClass("e-boolrowcell").html("{{if #data['" + splits.join("']['") + "']}} <input type ='checkbox' disabled='disabled' checked='checked'></input>{{else}} <input type ='checkbox' disabled='disabled'></input> {{/if}}");
                                break;
                            default:
                                if (columns[i].disableHtmlEncode)
                                    $tdCell.html("{{html:" + "#data['" + splits.join("'] || {})['") + "']}}");
                                else
                                    $tdCell.html("{{:" + braces + "#data['" + splits.join("'] || {})['") + "']}}");
                        }
                        if (columns[i]["format"] != undefined && (!columns[i]["foreignKeyValue"] && !columns[i]["dataSource"]))
                            $tdCell.html("{{:~_gridFormatting('" + columns[i]["format"] + "'," + braces + "#data['" + splits.join("'] || {})['") + "'],'" + this.model.locale + "')}}");
                        if (columns[i]["foreignKeyValue"] && columns[i]["dataSource"]) {
                            var foreignKeyColumn = columns[i]["field"] != columns[i]["foreignKeyField"] ? columns[i]["field"] : columns[i]["foreignKeyField"];
                            var foreignColumnClone = $.extend(true, {}, columns[i]);
                            delete foreignColumnClone["dataSource"];
                            delete foreignColumnClone["validationRules"];
                            $tdCell.html("{{:~_foreignKey('" + JSON.stringify(foreignColumnClone) + "',#data['" + foreignKeyColumn + "'],'" + this._id + "')}}");
                        }
                        if (columns[i]["isUnbound"]) {
                            var viewHelper = {};
                            viewHelper["_" + this._id + "UnboundTemplate"] = this._unboundTemplateRendering;
                            $.views.helpers(viewHelper);
                            if ((ej.isNullOrUndefined(columns[i]["field"])) || (columns[i].field == ""))
                                this.model.columns[i]["allowGrouping"] = this.model.columns[i]["allowFiltering"] = this.model.columns[i]["allowSorting"] = false;
                            $("#" + this._id + columns[i].headerText + "_UnboundTemplate").remove();
                            divElement = this._createUnboundElement(columns[i]);
                            $tdCell.addClass("e-unboundcell").addClass("e-" + columns[i]["headerText"].replace(/\s+/g, "")).html("{{:~_" + this._id + "UnboundTemplate('" + divElement.id + "')}}");
                            this.model.scrollSettings.frozenColumns > 0 && $tdCell.addClass("e-frozenunbound");
                            this._isUnboundColumn = true;
                        }

                    }
                    if (columns[i]["textAlign"] == undefined)
                        columns[i]["textAlign"] = "left";
                    if (!this.phoneMode && columns[i]["textAlign"] != undefined) {
                        $tdCell.css("text-align", columns[i]["textAlign"]);
                        $(headerCellDiv[i]).css("text-align", columns[i]["textAlign"]);
                    }
                    if (!this.phoneMode && !ej.isNullOrUndefined(columns[i]["headerTextAlign"])) {
                        $(headerCellDiv[i]).css("text-align", columns[i]["headerTextAlign"]);
                    }
                    if (!ej.isNullOrUndefined(columns[i]["cssClass"])) {
                        $tdCell.addClass(columns[i]["cssClass"]);
                    }
                    if (!ej.isNullOrUndefined(columns[i]["priority"]))
                        $tdCell.addClass("e-table-priority-" + columns[i]["priority"]);
                    if (!ej.isNullOrUndefined(columns[i]["customAttributes"]))
                        $tdCell.attr(columns[i]["customAttributes"]);
                    $tdCell.attr("role", "gridcell");
                    $tr.append($tdCell);
                    if (this.model.enableAltRow) {
                        helpers["_" + this._id + "AlternateRow"] = this._alternateRow;
                        $.views.helpers(helpers);
                        $tr.addClass("{{:~_" + this._id + "AlternateRow()}}");
                    }
                    $tr.attr("role", "row");
                    if (this.model.scrollSettings.frozenColumns > 0 && this.model.scrollSettings.frozenColumns == i + 1) {
                        tbody.appendChild(tr);
                        templates[this._id + "_JSONFrozenTemplate"] = $tbody.html();
                        $tr.empty();
                        $tbody.empty();
                    }
                }
                tbody.appendChild(tr);
            }
            templates[this._id + "_JSONTemplate"] = this.model.rowTemplate != null ? $(this.model.rowTemplate).html() : $tbody.html();
            $.templates(templates);
        },
        
        
        render: function () {
            this.model.showSummary = this.model.summaryRows.length > 0 || this.model.showSummary;
            this._renderGridContent().insertAfter(this.element.children(".e-gridheader"));
            this.model.allowResizeToFit && this.setWidthToColumns();
            if (this.model.allowGrouping && ej.gridFeatures.dragAndDrop)
                this._groupHeaderCelldrag();
            if (this.model.showSummary && this._currentJsonData.length) {
                this._renderGridFooter().insertAfter(this.getContent());
                this._hideCaptionSummaryColumn();
            }
            this._initialEndRendering();

        },
        _createStackedRow: function (stackedHeaderRow, frozenHeader) {
            var $tr = ej.buildTag('tr.e-columnheader e-stackedHeaderRow');
            var sHeader = [], sCss = []; tAl = [];
            for (var c = 0; c < this.model.columns.length; c++) {
                var column = this.model.columns[c];
                if (column.visible != false) {
                    if (this.model.allowGrouping && !this.model.groupSettings.showGroupedColumn && this.model.groupSettings.groupedColumns.length > 0) {
                        if ($.inArray(column.field, this.model.groupSettings.groupedColumns) != -1)
                            continue;
                    }
                    var headerText = '', cssClass = '', txtAlign = '';
                    var sColumn = stackedHeaderRow.stackedHeaderColumns;
                    for (var col = 0; col < sColumn.length; col++) {
                        if (sColumn[col].column.indexOf(column.field) != -1) {
                            headerText = sColumn[col].headerText;
                            cssClass = sColumn[col]["cssClass"];
                            txtAlign = sColumn[col].textAlign;
                        }
                    }
                    sHeader.push(headerText);
                    sCss.push(cssClass);
                    tAl.push(txtAlign);
                }
            }
            var colsPanList = []
            for (var i = 0; i < sHeader.length; i++) {
                var colSpan = 1;
                for (var j = i + 1; j < sHeader.length; j++) {
                    if (sHeader[i] == sHeader[j]) {
                        colSpan++;
                    }
                    else
                        break;
                }
                colsPanList.push({ sapnCount: colSpan, headerText: sHeader[i], cssClass: sCss[i], txtAlign: tAl[i] })
                i += colSpan - 1;
            }
            var $tr = ej.buildTag('tr.e-columnheader e-stackedHeaderRow');
            var frzCol = this.model.scrollSettings.frozenColumns;
            if (this.model.allowScrolling && frzCol > 0) {
                var frozenColspanList = [];
                var forzenColumn = 0, index = 0, frzHideCol = 0;
                for (var i = 0; i < this.model.columns.length; i++) {
                    var col = this.model.columns[i];
                    if (i < frzCol && col.visible == false)
                        frzHideCol++;
                }
                forzenColumn = frzCol - frzHideCol;
                while (forzenColumn > 0) {
                    var spanC = colsPanList[index].sapnCount;
                    if (colsPanList[index].sapnCount < forzenColumn) {
                        frozenColspanList.push(colsPanList[index])
                        if (!frozenHeader)
                            colsPanList.splice(index, 1);
                        else
                            index++;
                    }
                    else if (colsPanList[index].sapnCount > forzenColumn) {
                        colsPanList[index].sapnCount = colsPanList[index].sapnCount - forzenColumn
                        if (frozenHeader)
                            frozenColspanList.push({ sapnCount: forzenColumn, headerText: colsPanList[index].headerText });
                    }
                    else {
                        frozenColspanList.push(colsPanList[index])
                        if (!frozenHeader)
                            colsPanList.splice(index, 1);
                    }
                    forzenColumn -= spanC;
                }
                if (frozenHeader)
                    colsPanList = frozenColspanList
            }
            if (this.model.detailsTemplate || this.model.childGrid || this.model.detailsDataBound)
                $tr.append(ej.buildTag('th.e-headercell e-detailheadercell', '<div></div>'));
            for (var c = 0; c < colsPanList.length; c++) {
                var $th = ej.buildTag('th.e-headercell e-stackedHeaderCell e-default', colsPanList[c].headerText, {}, { 'colspan': colsPanList[c].sapnCount });
                $th.css("textAlign", colsPanList[c].txtAlign);
                $tr.append($th);
                if (colsPanList[c]["cssClass"] != undefined)
                    $th.addClass(colsPanList[c]["cssClass"]);
            }
            return $tr;
        },
        _renderGridHeaderInternalDesign: function (columns, frozenHeader) {
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0.25px", role: "grid" });
            var $thead = ej.buildTag('thead');
            var $tbody = ej.buildTag('tbody.e-hide');
            var $columnHeader = ej.buildTag('tr.e-columnheader');
            var $colGroup = $(document.createElement('colgroup'));
            var $rowBody = $(document.createElement('tr'));
            if (this.model.detailsTemplate || this.model.childGrid || this.model.detailsDataBound) {
                $columnHeader.append(ej.buildTag('th.e-headercell e-detailheadercell', '<div></div>'));
                $colGroup.append(this._getIndentCol());
            }
            if (this.model.showStackedHeader) {
                for (var index = 0; index < this.model.stackedHeaderRows.length; index++) {
                    var $tr = this._createStackedRow(this.model.stackedHeaderRows[index], frozenHeader);
                    $thead.append($tr);
                }
            }
            for (var columnCount = 0; columnCount < columns.length; columnCount++) {
                var $headerCell = ej.buildTag('th.e-headercell e-default', "", {}, { role: "columnheader" });
                var bodyCell = document.createElement('td');
                var $headerCellDiv = ej.buildTag('div.e-headercelldiv', columns[columnCount]["headerText"] === undefined ? columns[columnCount]["headerText"] = columns[columnCount]["field"] : columns[columnCount]["headerText"], {}, { "ej-mappingname": columns[columnCount]["field"] });
                if (columns[columnCount].disableHtmlEncode)
                    $headerCellDiv.text(columns[columnCount]["headerText"]);
                $headerCell.append($headerCellDiv);
                if (this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this.model.filterSettings.filterType == "excel") &&
                (columns[columnCount]["allowFiltering"] == undefined ||
                    columns[columnCount]["allowFiltering"] === true)) {
                    if (!(!ej.isNullOrUndefined(columns[columnCount]["templateID"] || columns[columnCount]["template"] || columns[columnCount]["isUnbound"]) && (columns[columnCount].field == "" || ej.isNullOrUndefined(columns[columnCount].field))))
                        var filtericon = 'e-filterset';
                    if (!this.initialRender && this.model.filterSettings.filteredColumns) {
                        for (var i = 0; i < this.model.filterSettings.filteredColumns.length; i++) {
                            if (this.model.filterSettings.filteredColumns[i].field == columns[columnCount].field) {
                                filtericon = 'e-filterset e-filteredicon e-filternone';
                            }
                        }
                    }
                    $headerCell.append(ej.buildTag('div.e-filtericon e-icon ' + filtericon));
                    if (ej.browserInfo().name == "msie" && ej.browserInfo().version == "8.0" && this.model.enableRTL)
                        $($headerCellDiv).css("padding", "0 0 0 2em");
                }
                var col = document.createElement('col');
                if (columns[columnCount]["priority"])
                    $(bodyCell).addClass("e-table-priority-" + columns[columnCount]["priority"]);
                $rowBody.append(bodyCell);
                $columnHeader.append($headerCell);
                $colGroup.append(col);
                if (columns[columnCount]["visible"] === false) {
                    $headerCell.addClass("e-hide") && $(col).css("display", "none")
                    if ($.inArray(columns[columnCount].headerText, this._hiddenColumns) == -1 && $.inArray(columns[columnCount].field, this._hiddenColumnsField) == -1)
                        this._hiddenColumns.push(columns[columnCount].headerText) && columns[columnCount].field != "" ? this._hiddenColumnsField.push(columns[columnCount].field) : this._hiddenColumnsField.push(columns[columnCount].headerText);
                    if ($.inArray(columns[columnCount].field, this._visibleColumnsField) != -1)
                        this._visibleColumnsField.splice($.inArray(columns[columnCount].field, this._visibleColumnsField), 1) && this._visibleColumns.splice($.inArray(columns[columnCount].headerText, this._visibleColumns), 1)
                }
                else {
                    this._visibleColumns.push(columns[columnCount].headerText) && columns[columnCount].field != "" ? this._visibleColumnsField.push(columns[columnCount].field) : this._visibleColumnsField.push(columns[columnCount].headerText);
                    columns[columnCount]["visible"] = true;
                    if ($.inArray(columns[columnCount].field == "" ? columns[columnCount].headerText : columns[columnCount].field, this._hiddenColumnsField) != -1)
                        this._hiddenColumnsField.splice($.inArray(columns[columnCount].field == "" ? columns[columnCount].headerText : columns[columnCount].field, this._hiddenColumnsField), 1) && this._hiddenColumns.splice($.inArray(columns[columnCount].headerText, this._hiddenColumns), 1)
                }
                if (this.model.showColumnChooser && columns[columnCount]["showInColumnChooser"] !== false)
                    columns[columnCount]["showInColumnChooser"] = true;
                if (this.model.allowResizing && columns[columnCount]["allowResizing"] !== false)
                    columns[columnCount]["allowResizing"] = true;
                if (!ej.isNullOrUndefined(columns[columnCount]["headerTextAlign"]))
                    $headerCellDiv.css("text-align", columns[columnCount]["headerTextAlign"]);
                else if (columns[columnCount]["textAlign"] != undefined)
                    $headerCellDiv.css("text-align", columns[columnCount]["textAlign"]);
                columns[columnCount]["allowResizing"] === false && this._disabledResizingColumns.push(columns[columnCount].field);
                columns[columnCount]["allowSorting"] === false && this._disabledSortableColumns.push(columns[columnCount].field);
                columns[columnCount]["allowGrouping"] === false && this._disabledGroupableColumns.push(columns[columnCount].field);
                columns[columnCount]["allowEditing"] === false && this._disabledEditableColumns.push(columns[columnCount].field);
                if (!ej.isNullOrUndefined(columns[columnCount]["cssClass"])) {
                    $headerCell.addClass(columns[columnCount]["cssClass"]);
                    $(col).addClass(columns[columnCount]["cssClass"]);
                }
                if (!ej.isNullOrUndefined(columns[columnCount]["headerTemplateID"])) {
                    $headerCellDiv.html($(columns[columnCount]["headerTemplateID"]).hide().html()).parent().addClass("e-headertemplate");
                    var index = $.inArray(columns[columnCount].field, this._disabledGroupableColumns);
                    index == -1 && ej.isNullOrUndefined(columns[columnCount].field) && this._disabledGroupableColumns.push(columns[columnCount].field);
                }
                if (this.model.allowGrouping && this.model.groupSettings.showToggleButton && $.inArray(columns[columnCount].field, this._disabledGroupableColumns) == -1 && !ej.isNullOrUndefined(columns[columnCount].field) && columns[columnCount].field != "") {
                    if ($.inArray(columns[columnCount].field, this.model.groupSettings.groupedColumns) != -1)
                        $headerCellDiv.append(this._getToggleButton().addClass("e-toggleungroup"));
                    else
                        $headerCellDiv.append(this._getToggleButton().addClass("e-togglegroup"));
                }
                if (this.model.isResponsive)
                    $headerCell.attr("title", columns[columnCount].headerText);
                if (columns[columnCount]["priority"]) {
                    $headerCell.attr("data-priority", columns[columnCount]["priority"]).addClass("e-table-priority-" + columns[columnCount]["priority"]);
                    $(col).addClass("e-table-priority-" + columns[columnCount]["priority"]);
                }
                if (this.initialRender) {
                    if (typeof (columns[columnCount].width) == "string" && columns[columnCount].width.indexOf("%") != -1)
                        this._columnsWidthCollection.push(parseInt(columns[columnCount]["width"]) / 100 * this.element.width());
                    else
                        this._columnsWidthCollection.push(columns[columnCount]["width"]);
                }
                if (columns[columnCount]["width"] == undefined && this.model.commonWidth !== undefined)
                    this._columnsWidthCollection[columnCount] = this.model.commonWidth;
                this._fieldColumnNames[columns[columnCount].headerText] = columns[columnCount].field;
                this._headerColumnNames[columns[columnCount].field] = columns[columnCount].headerText;
                this.model.allowFiltering && ej.getObject("allowFiltering",columns[columnCount]) !== false  && ("menu" == this.model.filterSettings.filterType || this._isExcelFilter) && $headerCell.addClass("e-headercellfilter");
            }
            $thead.append($columnHeader);
            $tbody.append($rowBody);
            $table.append($colGroup).append($thead).append($tbody);
            return $table;
        },
        _renderGridHeader: function () {
            var $div = ej.buildTag('div.e-gridheader'), temp, $frozenDiv, $movableDiv;
            var $innerDiv = ej.buildTag('div');
            if (this.model.allowScrolling)
                $innerDiv.addClass("e-headercontent");
            this.setGridHeaderContent($div);
            if (this.initialRender) {
                this._columnsWidthCollection = [];
                this._hiddenColumns = [];
                this._hiddenColumnsField = [];
            }
            this._visibleColumns = [];
            this._visibleColumnsField = [];
            this._disabledGroupableColumns = [];
            this._fieldColumnNames = {};
            this._headerColumnNames = {};
            if (this.model.scrollSettings.frozenColumns > 0) {
                $frozenDiv = ej.buildTag("div.e-frozenheaderdiv", this._renderGridHeaderInternalDesign(this.model.columns.slice(0, this.model.scrollSettings.frozenColumns), true));
                $movableDiv = ej.buildTag("div.e-movableheader", ej.buildTag("div.e-movableheaderdiv", this._renderGridHeaderInternalDesign(this.model.columns.slice(this.model.scrollSettings.frozenColumns), false)));
                $innerDiv.append($frozenDiv).append($movableDiv);
            } else
                $innerDiv.append(this._renderGridHeaderInternalDesign(this.model.columns));
            $div.html($innerDiv);
            if (this.model.isResponsive)
                $div.addClass("e-textover");
            this.setGridHeaderTable(this.getHeaderContent().find(".e-table"));
            return $div;
        },
        _renderGridContent: function () {
            var $div = ej.buildTag('div.e-gridcontent');
            var $innderDiv = ej.buildTag('div');
            var $table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0.25px" });
            var $tbody = $(document.createElement('tbody'));
            $table.append(this.getHeaderTable().find('colgroup').clone()).append($tbody);
            $innderDiv.html($table);
            $div.html($innderDiv);
            this.setGridContentTable($table);
            this.setGridContent($div);
            $table.attr("role", "grid");
            var args = {};
            if (this.model.allowGrouping && this.model.groupSettings.groupedColumns.length) {
                if (this.initialRender) {
                    args.columnName = this.model.groupSettings.groupedColumns[this.model.groupSettings.groupedColumns.length - 1];
                    if (!this.model.groupSettings.showGroupedColumn) {
                        for (var i = 0; i < this.model.groupSettings.groupedColumns.length; i++) {
                            var col = this.model.groupSettings.groupedColumns[i];
                            if ($.inArray(col, this._hiddenColumnsField) == -1) {//updated for
                                this._hiddenColumnsField.push(col);//updated for
                                this.getColumnByField(col).visible = false;
                            }
                        }
                    }
                }
                args.requestType = ej.Grid.Actions.Grouping;
            } else
                args.requestType = ej.Grid.Actions.Refresh;
            if (this._dataSource() == null || this._dataSource().length == 0) {
                var $emptyTd = ej.buildTag('td.emptyrecord', this._getLocalizedLabels("EmptyRecord"), {}, { colSpan: this.model.columns.length });
                $tbody.append($(document.createElement("tr")).append($emptyTd));
                this.setWidthToColumns();
                if (this.model.groupSettings.groupedColumns.length && this.initialRender)
                    this.sendDataRenderingRequest(args)
            } else
                this.sendDataRenderingRequest(args);
            if (this._isCaptionSummary && args.requestType == "grouping" && this.model.groupSettings.groupedColumns.length > 1) {
                var colgroup = this.getContentTable().find(".e-table").not(".e-recordtable").children("colgroup");
                var $cols1 = $(this.getContentTable().find(".e-recordtable")[0]).children("colgroup").find("col");
                for (i = 0; i < colgroup.length; i++) {
                    var colCount = $(colgroup[i]).find("col").length;
                    $(colgroup[i]).find("col:gt(" + (colCount - $cols1.length - 1) + ")").remove();
                    $(colgroup[i]).append($cols1.clone());
                }
            }
            return $div;
        },
        
        print: function () {
            var args = {}; args.requestType = "print";
            this._printselectrows = this.getContentTable().find('tr[aria-selected="true"]');
            
            this._trigger("actionBegin", args);
            var allowPaging = this.model.allowPaging;
            var elementClone = this.element.clone();
            if (this.model.allowPaging && this.model.pageSettings.printMode == "currentpage") {
                this.getPager().css('display', 'none');
                elementClone = this.element.clone();
                if (!this.model.allowScrolling) {
                    this.refreshContent();
                    this.getPager().css('display', 'block');
                }
            }
            else if (allowPaging) {
                this.model.allowPaging = false;
                this.refreshContent();
                this.getPager().css('display', 'none');
                elementClone = this.element.clone();
                this.model.allowPaging = true;
                if (!this.model.allowScrolling) {
                    this.refreshContent();
                    this.getPager().css('display', 'block');
                }
            }
            if (this.model.allowScrolling) {
                var scrollWidth = this.model.scrollSettings.width, scrollHeight = this.model.scrollSettings.height;
				if (this.getScrollObject().isVScroll() || this.getScrollObject().isHScroll()) {
					var scrollContent = this.getContent().find('.e-content')[0];
					elementClone.find('.e-gridcontent').height(scrollContent.scrollHeight);
					elementClone.find('.e-gridcontent').ejScroller({ width: scrollContent.scrollWidth, height: scrollContent.scrollHeight });
					elementClone.width(scrollContent.scrollWidth);
				}
                if (this.model.allowPaging) {
                    this.refreshContent();
                    this.getPager().css('display', 'block');
                }
            }
            if (this.model.toolbarSettings.showToolbar)
                elementClone.find(".e-gridtoolbar").remove();
            args = { requestType: "print", element: elementClone, selectedRows: this._printselectrows };
            this._trigger("beforePrint", args);
            if (!ej.isNullOrUndefined(args.element))
                elementClone = args.element;
            ej.print(elementClone);
            args.requestType = "print";
            this._trigger("actionComplete", args);
        },
        
        "export": function (action, serverEvent, multipleExport) {
            var tempObj = {}, proxy = this;
            $.extend(tempObj, ej.Grid.locale["en-US"], ej.Grid.locale[this.model.locale]);
            var temp = tempObj.GroupCaptionFormat;
            var split1 = temp.indexOf("{{if");
            var split2 = temp.indexOf("{{else}}");
            var grpText = temp.slice(split1, split2).replace("{{if count == 1}}", "");
            var localeProp = { EmptyRecord: tempObj.EmptyRecord, GroupCaptionFormat: temp.slice(0, split1), GroupText: grpText };
            this._locale = this.model.locale;
            this.model.locale = this.model.locale.concat(JSON.stringify(localeProp));
            var modelClone = $.extend(true, {}, this.model);
            for (var i = 0; i < modelClone.columns.length; i++) {
                if (modelClone.columns[i].editType != undefined) {
                    switch (modelClone.columns[i].editType) {
                        case "stringedit":
                        case "edittemplate":
                            modelClone.columns[i].editType = "string";
                            break;
                        case "numericedit":
                            modelClone.columns[i].editType = "numeric";
                            break;
                        case "dropdownedit":
                            modelClone.columns[i].editType = "dropdown";
                            break;
                        case "booleanedit":
                            modelClone.columns[i].editType = "boolean";
                            break;
                        default:
                            break;
                    }
                }
            }
            for (var i = 0; i < modelClone.filterSettings.filteredColumns.length; i++) {
                if (modelClone.filterSettings.filteredColumns[i].operator == "equal")
                    modelClone.filterSettings.filteredColumns[i].operator = "equals";
                else if (modelClone.filterSettings.filteredColumns[i].operator == "notequal")
                    modelClone.filterSettings.filteredColumns[i].operator = "notequals";
            }
            if (modelClone.showStackedHeader) {
                modelClone.stackedHeaderRow = modelClone.stackedHeaderRows;
                for (var i = 0; i < modelClone.stackedHeaderRow.length; i++) {
                    modelClone.stackedHeaderRow[i].stackedHeaderColumn = modelClone.stackedHeaderRow[i].stackedHeaderColumns;
                }
            }
            var attr = { action: action, method: 'post', "data-ajax": "false" };
            var form = ej.buildTag('form', "", null, attr);
            var gridob = this;
            if (this.ignoreOnExport) {
                for (var i = 0; i < this.ignoreOnExport.length; i++) {
                    delete modelClone[this.ignoreOnExport[i]];
                }
            }
            if (ej.raiseWebFormsServerEvents) {
                var args = { model: modelClone, originalEventType: serverEvent };
                var clientArgs = { model: JSON.stringify(modelClone) };
                ej.raiseWebFormsServerEvents(serverEvent, args, clientArgs);
                setTimeout(function () {
                    ej.isOnWebForms = true;
                }, 1000);
            }
            else {
                var gridobjectArray = {};
                if (multipleExport) {
                    $('body').find('.e-grid').each(function (index, object) {
                        var gridObject = $(object).data('ejGrid');
                        if (!ej.isNullOrUndefined(gridObject)) {
                            var modelClone = JSON.parse(JSON.stringify(gridObject.model));
                            if (gridob.ignoreOnExport) {
                                for (var i = 0; i < gridob.ignoreOnExport.length; i++) {
                                    delete modelClone[gridob.ignoreOnExport[i]];
                                }
                                gridobjectArray[index] = JSON.stringify(modelClone);
                                var inputAttr = { name: 'GridModel', type: 'hidden', value: JSON.stringify(modelClone) }
                                var input = ej.buildTag('input', "", null, inputAttr);
                                form.append(input);
                            }
                        }
                    });
                }
                else {
                    var inputAttr = { name: 'GridModel', type: 'hidden', value: JSON.stringify(modelClone) }
                    var input = ej.buildTag('input', "", null, inputAttr);
                    form.append(input);
                    form.append(this);
                }
                $('body').append(form);
                form.submit();
            }
            setTimeout(function () {
                proxy.model.locale = proxy._locale;
            }, 0);
            return true;
        },
        sendDataRenderingRequest: function (args) {
            if (this._templateRefresh) {
                this.refreshTemplate();
                this._templateRefresh = false;
            }
            this._previousColumnIndex = null;
            this._previousRowCellIndex = null;
            this._previousIndex = null;
            if (args.requestType == "add" || args.requestType == "grouping" || (this.model.currentViewData != null && this.model.currentViewData.length)) {
                switch (args.requestType) {
                    case ej.Grid.Actions.Refresh:
                    case ej.Grid.Actions.Paging:
                    case ej.Grid.Actions.Sorting:
                    case ej.Grid.Actions.Filtering:
                    case ej.Grid.Actions.Save:
                    case ej.Grid.Actions.Cancel:
                    case ej.Grid.Actions.Delete:
                    case ej.Grid.Actions.Search:
                    case ej.Grid.Actions.Reorder:
                    case ej.Grid.Actions.BatchSave:
                        var cloneGroupedColumns = this.model.groupSettings.groupedColumns
                        if(this.model.allowGrouping && args.requestType == ej.Grid.Actions.Refresh && cloneGroupedColumns.length == 0 && this.element.find(".e-grouptopleftcell").length > 0) {
                            var $header = this.element.children(".e-gridheader");
                            $header.find("div").first().empty().append(this._renderGridHeader().find("table"));
                        }
                        if(!this.model.allowGrouping)
                            cloneGroupedColumns = [];
                        if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                            $("#" + this._id + "_externalEdit").css("display", "none");
                        if (cloneGroupedColumns.length == 0) {
                            var temp = document.createElement('div'), temp1, insertIndex = -1, isRemoteAdaptor = false;
                            if (!this.phoneMode)
                                this.getContentTable().find("colgroup").first().replaceWith(this._getMetaColGroup());
                            (this.model.detailsTemplate != null || this.model.childGrid != null || this.model.detailsDataBound != null) && this.getContentTable().find("colgroup").first().prepend(this._getIndentCol());
                            var currentPage = this._currentPage();
                            if ((this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate" || this.model.editSettings.editMode == "normal") && (args.requestType == "cancel" || args.requestType == "save"))
                                this._editFormHeight = this.element.find(".gridform").closest("tr").height();
     
                            if (this.model.scrollSettings.frozenColumns > 0)
                                temp.innerHTML = this._renderByFrozenDesign();
                            else {
                                temp.innerHTML = ['<table>', $.render[this._id + "_JSONTemplate"](this.model.currentViewData), '</table>'].join("");
                                if (args.data) {
                                    temp1 = document.createElement('div');
                                    temp1.innerHTML = ['<table>', $.render[this._id + "_JSONTemplate"](args.data), '</table>'].join("");
                                    if (this._dataSource() instanceof ej.DataManager && args.requestType == ej.Grid.Actions.Save) {
                                        insertIndex = this._getDataIndex(this.model.currentViewData, args.data);
                                        isRemoteAdaptor = this._dataSource().adaptor instanceof ej.remoteSaveAdaptor;
                                    }
                                }
                                var tableEle = this.getContentTable().get(0);
                                var tbodyEle = tableEle.lastChild;
                                var rindex = this.getContentTable().first().find('tbody').first();
                                if ((args.requestType == "save" || args.requestType == "cancel") && this.model.editSettings.editMode != "batch") {
                                    if (this.model.editSettings.editMode.indexOf("inlineform") != -1)
                                        rowIndex = !ej.isNullOrUndefined(args.selectedRow) ? args.selectedRow : this._selectedRow();
                                    else
                                        rowIndex = this.getContentTable().find('.e-' + args.action + 'edrow').index();
                                    var a = this._currentTrIndex;
                                    if (rowIndex == -1)
                                        rowIndex = a;
                                    if (this.model.detailsTemplate != null || this.model.childGrid != null) {
                                        if (this.model.editSettings.editMode == "inlineform")
                                            var rowTr = $($(tbodyEle.childNodes).not('.e-detailrow')[rowIndex]);
                                        else
                                            var rowTr = $(tbodyEle.childNodes[rowIndex]);
                                        var rowEle = $(tbodyEle.childNodes).not('.e-detailrow');
                                        for (var i = 0; i < rowEle.length; i++) {
                                            if (rowTr.is(rowEle[i]))
                                                rowIndex = i;
                                        }
                                    }

                                    if (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                                        if (args.action == "add" && !this.getContentTable().find(".e-addedrow").length) break;
                                        $oldChild = this.getContentTable().find('.e-addedrow').get(0);
                                        $editedTr = this.getContentTable().find('.e-editedrow');
                                        $newChild = ($editedTr.length || args.requestType == "cancel") ? temp.firstChild.firstChild.firstChild : temp1.firstChild.firstChild.lastChild;
                                        if (this.model.sortSettings.sortedColumns.length && this._currentJsonData.length > 0 || !ej.isNullOrUndefined(this._searchCount))
                                            this.refreshContent();
                                        else if ($editedTr.length) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                                                $oldChild = $editedTr.prev('tr').get(0);
                                                $editedTr.remove();
                                            } else
                                                $oldChild = $editedTr.get(0);
                                            var $newChildObj = $($newChild), $oldChildObj = $($oldChild);
                                            if ((this.model.detailsTemplate != null || this.model.childGrid != null) && $oldChildObj.next('tr.e-detailrow:visible').length) {
                                                var $target = $newChildObj.find('.e-detailrowcollapse');
                                                $target.removeClass("e-detailrowcollapse").addClass("e-detailrowexpand").find("div").removeClass("e-gnextforward").addClass("e-gdiagonalnext");
                                            }
                                            if (args.requestType == "cancel") {
                                                if (this.model.editSettings.showAddNewRow)
                                                    this.getContentTable().find('.e-addedrow').addClass("e-showaddrow");
                                                $oldChildObj.replaceWith($newChildObj);
                                            } else if (!ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount < this._previousFilterCount) {
                                                if (this.model.detailsTemplate != null && $oldChildObj.next('tr.e-detailrow').length)
                                                    tbodyEle.removeChild($oldChildObj.next('tr.e-detailrow').get(0));
                                                $oldChildObj.remove();
                                                if (this.model.allowPaging && this.model.pageSettings.pageSize <= this.model.currentViewData.length && cloneGroupedColumns.length == 0)
                                                    tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                            } else {
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                            }
                                        } else {
                                            var $newChildObj = $($newChild), $oldChildObj = $($oldChild);
                                            if (args.action == "add" && args.requestType == "save" && this.model.editSettings.showAddNewRow)
                                                this.model.editSettings.rowPosition == "bottom" ? tbodyEle.lastChild.previousSibling.remove() : tbodyEle.lastChild.remove();
                                            if (args.requestType == "cancel" || this._dataSource() instanceof ej.DataManager || this._currentPage() != 1 || (args.requestType == "save" && !ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount == this._previousFilterCount)) {
                                                if (!ej.isNullOrUndefined($oldChild)) {
                                                    $oldChildObj.remove();
                                                    if (this._dataSource() instanceof ej.DataManager && insertIndex != -1) {
                                                        if (insertIndex == 0)
                                                            tbodyEle.insertBefore($newChild, tbodyEle.children[insertIndex]);
                                                        else
                                                            $newChildObj.insertAfter(tbodyEle.children[insertIndex - 1]);
                                                    }
                                                    else if ((!(this._dataSource() instanceof ej.DataManager) || isRemoteAdaptor) && this._currentPage() != 1 && args.requestType == "save")
                                                        $(tbodyEle).prepend($(temp.firstChild.firstChild.firstChild));
                                                    if (this.model.allowPaging && this.model.pageSettings.pageSize <= this.model.currentViewData.length && cloneGroupedColumns.length == 0 &&
                                                        ((this._dataSource() instanceof ej.DataManager && insertIndex == -1 && (!isRemoteAdaptor && args.requestType != "save")) ||
                                                        (args.requestType != "save" && !(this._dataSource() instanceof ej.DataManager))) && (args.requestType == "cancel" && !this.model.editSettings.showAddNewRow))
                                                        tableEle.lastChild.appendChild(temp.firstChild.firstChild.lastChild);
                                                }
                                                if (args.requestType == "cancel" && this._selectedRow() != -1)
                                                    this.clearSelection();

                                            } else if (this.model.currentViewData.length == 1) {
                                                $(tbodyEle).empty();
                                                tbodyEle.appendChild($newChild);
                                            } else if (this.model.editSettings.rowPosition == "bottom") {
                                                rindex.prepend($oldChild);
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                            } else
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                        }
                                    } else if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate" || this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate") {
                                        if (args.action == "add" && !this.element.find(".e-addedrow").length) break;
                                        $editedTr = this.element.find('.e-editedrow');
                                        if (args.requestType == "cancel" || (!$editedTr.length && !ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount == this._previousFilterCount)) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            $oldChild = tbodyEle.childNodes[rowIndex];
                                            var $newChildObj = $($newChild), $oldChildObj = $($oldChild);
                                            $oldChildObj.replaceWith($newChildObj);
                                            this.clearSelection();
                                            this.model.allowPaging && this._refreshGridPager();

                                        } else if ($editedTr.length) {
                                            $newChild = temp.firstChild.firstChild.childNodes[rowIndex];
                                            $oldChild = this._excludeDetailRows(tbodyEle.childNodes)[rowIndex];
                                            if (this.model.allowCellMerging != null) {
                                                var $oldChildObj = $($oldChild);
                                                if ($($oldChild.childNodes).hasClass("e-merged")) {
                                                    var index = $oldChildObj.children('.e-merged').index();
                                                    var count = $oldChild.children[index].colSpan;
                                                    for (var i = 0 ; i < count; i++) {
                                                        $newChild.childNodes[index + i].className += " e-merged e-hide";
                                                        $newChild.childNodes[index].colSpan = i + 1;
                                                    }
                                                }
                                            }
                                            if (this.model.detailsTemplate != null)
                                                $oldChild = $(tbodyEle.childNodes).not('.e-detailrow').eq(rowIndex).get(0);
                                            if ((this.model.detailsTemplate != null || this.model.childGrid != null) && $oldChildObj.next('tr.e-detailrow:visible').length) {
                                                var $target = $($newChild).find(".e-detailrowcollapse");
                                                $target.removeClass("e-detailrowcollapse").addClass("e-detailrowexpand").find("div").removeClass("e-gnextforward").addClass("e-gdiagonalnext");
                                            }
                                            if (!ej.isNullOrUndefined(this._filteredRecordsCount) && this._filteredRecordsCount < this._previousFilterCount) {
                                                var $oldChildObj = $($oldChild);
                                                if (this.model.detailsTemplate != null && $oldChildObj.next('tr.e-detailrow').length)
                                                    tbodyEle.removeChild($oldChildObj.next('tr.e-detailrow').get(0));
                                                $oldChildObj.remove();
                                                if (this.model.allowPaging && this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                                    tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                            } else
                                                tbodyEle.replaceChild($newChild, $oldChild);
                                        } else if (this.model.currentViewData.length == 1 && this.getContentTable().find('td.e-rowcell').length == 0) {
                                            $newChild = temp.firstChild.firstChild.firstChild;
                                            $(tbodyEle).empty();
                                            tbodyEle.appendChild($newChild);
                                        } else {
                                            newChild = ($editedTr.length || args.requestType == "cancel") ? temp.firstChild.firstChild.firstChild : temp1.firstChild.firstChild.lastChild;
                                            if (!(this._dataSource() instanceof ej.DataManager)) {
                                                if (this._currentPage() == 1)
                                                    this.getContentTable().find('tbody').first().prepend($(newChild));
                                                else
                                                    this.getContentTable().find('tbody').first().prepend($(temp.firstChild.firstChild.firstChild));
                                                if (this.model.allowPaging && this.model.pageSettings.pageSize <= this.model.currentViewData.length)
                                                    tbodyEle.removeChild(tbodyEle.lastChild);
                                            }
                                            else if (insertIndex != -1) {
                                                if (insertIndex == 0)
                                                    tbodyEle.insertBefore(newChild, tbodyEle.children[insertIndex]);
                                                else
                                                    $(newChild).insertAfter(tbodyEle.children[insertIndex - 1]);
                                                if (this.model.allowPaging && (this.model.pageSettings.pageSize <= this.model.currentViewData.length || insertIndex == this.model.pageSettings.pageSize - 1))
                                                    tbodyEle.removeChild(tbodyEle.lastChild);
                                            }
                                            if (this.model.detailsTemplate != null && $(tableEle.lastChild.lastChild).children('.e-detailrowexpand').length)
                                                tbodyEle.removeChild(tbodyEle.lastChild);
                                        }
                                    }
                                    this._gridRows = tableEle.rows;
                                    if (this.model.enableAltRow)
                                        this._refreshAltRow();
                                } else if (args.requestType == "delete") {
                                    if (this._isUnboundColumn) {
                                        $editedrow = this.element.find('.e-editedrow');
                                        $oldChild = this.getContentTable().find('.e-editedrow').get(0);
                                        $newChild = ($editedrow.length) ? temp.firstChild.firstChild.firstChild : temp1.firstChild.firstChild.lastChild;

                                        if ($editedrow.length != 0 && (this.model.editSettings.editMode == "normal" || this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")) {
                                            $($oldChild).replaceWith($($newChild));
                                        }
                                        else if (this.model.editSettings.editMode == "inlineform" || this.model.editSettings.editMode == "inlineformtemplate") {
                                            $oldChild = $editedrow.prev('tr').get(0);
                                            $editedrow.remove();
                                        }
                                        else
                                            $oldChild = $editedrow.get(0);
                                    }
                                    if (this.model.allowPaging && this.model.pageSettings.pageSize <= this.model.currentViewData.length && this.getContentTable()[0].rows.length != this.model.currentViewData.length) {
                                        if (this._excludeDetailRows().length)
                                            tbodyEle.appendChild(temp.firstChild.firstChild.lastChild);
                                        else
                                            $(tbodyEle).prepend(temp.firstChild.firstChild.rows);
                                    }
                                    if (this.model.detailsTemplate != null || this.model.childGrid != null) {
                                        var visibleRow = this.getContentTable().find('.e-detailrow:visible');
                                        $.each(visibleRow, function (indx, item) {
                                            if (visibleRow.eq(indx).closest('tr').prev().children('.e-detailrowexpand').length == 0)
                                                visibleRow.eq(indx).remove();
                                        });
                                    }
                                    if (this.model.enableAltRow)
                                        this._refreshAltRow();
                                } else
                                    this.getContentTable().get(0).replaceChild(this.model.rowTemplate != null ? temp.firstChild.lastChild : temp.firstChild.firstChild, this.getContentTable().get(0).lastChild);

                            }
                            this._currentJsonData = this.model.currentViewData;
                            this._gridRows = this.getContentTable().get(0).rows;
                            if (this.model.scrollSettings.frozenColumns > 0)
                                this._gridRows = [this._gridRows, this.getContentTable().get(1).rows];

                            var model = {};
                            if ((args.requestType == "sorting" || args.requestType == "filtering") && this.model.scrollSettings.allowVirtualScrolling) {
                                if (args.requestType == "filtering") {
                                    this.getContent().first().ejScroller("refresh").ejScroller("isVScroll") ? this.element.find(".gridheader").addClass("e-scrollcss") : this.element.find(".gridheader").removeClass("e-scrollcss");
                                    var model = this._refreshVirtualPagerInfo();
                                }
                                this._refreshVirtualContent(currentPage, model);
                                args.requestType == "filtering" && this.getContent().first().ejScroller("refresh");
                            }
                            if (this.model.allowPaging) {
                                if (this.model.filterSettings.filteredColumns.length)
                                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._filteredRecordsCount : this._searchCount, currentPage: this._currentPage() });
                                else
                                    this.getPager().ejPager({ totalRecordsCount: this._searchCount == null ? this._gridRecordsCount : this._searchCount, currentPage: this._currentPage() });
                                this._refreshGridPager();
                            }
                            this._eventBindings();
                            break;
                        }
                    case ej.Grid.Actions.Grouping:
                        this._group(args);
                        this._refreshStackedHeader();
                        break;
                    case ej.Grid.Actions.BeginEdit:
                        this._edit(args);
                        break;
                    case ej.Grid.Actions.Add:
                        this._add(args);
                        break;
                    case ej.Grid.Actions.Ungrouping:
                        this._ungroup(args);
                        break;
                    case ej.Grid.Actions.VirtualScroll:
                        !this._isVirtualRecordsLoaded && this._replacingContent();
                        break;
                }
            } else
                this._newungroup(args);
            this._showGridLines();
            this._completeAction(args);
        },
        _showGridLines: function () {
            var $lines = this.model.gridLines;
            if ($lines != "both") {
				this.getContent().addClass($lines != "none" ? "e-" + $lines + "lines" : "e-hidelines");
            }
        },
		_showHeaderGridLines: function(){
			var $lines = this.model.gridLines;
			if ($lines != "both") 
				this.getHeaderContent().addClass($lines != "none" ? "e-"+$lines+"lines" : "e-hidelines");
		},
        _newungroup: function (args) {
            if (args.requestType == "ungrouping")
                this._ungroup(args);
            else
                this.getContentTable().find('tbody').empty().first().append(this._getEmptyTbody());
        },
        _completeAction: function (args) {
            if (!this.model.editSettings.showAddNewRow)
                this.model.isEdit = false;
            this._confirmedValue = false;
			if(args.requestType == "save" || args.requestType =="cancel")
				this._isAddNew=false;
            if (ej.Grid.Actions.Grouping == args.requestType && ej.isNullOrUndefined(args.columnName))
                return;
            if ((args.columnSortDirection == "ascending" || args.columnSortDirection == "descending") && !ej.isNullOrUndefined(args.columnName)) {
                var scolumn = this.getColumnByField(args.columnName);
                if (this.model.allowSorting && this.model.allowMultiSorting)
                    this._scolumns.push(scolumn.field);
                else
                    this._gridSort = scolumn.field;
            }
            if (args.requestType!='beginedit')
               this.setWidthToColumns();
            if (!this.initialRender && (ej.Grid.Actions.UnGrouping == args.requestType || this.model.groupSettings.groupedColumns.length > 0) && !$("#" + this._id + "EditForm").length)
                this._recalculateIndentWidth();
            if (ej.Grid.Actions.Paging == args.requestType || ej.Grid.Actions.BatchSave == args.requestType)
                this._refreshGridPager();
            else if ((ej.Grid.Actions.Sorting == args.requestType && this.model.allowSorting) || ej.Grid.Actions.Refresh == args.requestType) {
                if (ej.gridFeatures.sort && this.getHeaderTable() !== null)
                    this._sortCompleteAction(args);
                if (this.model.allowPaging) {
                    var pageModel = this.getPager().ejPager("option");
                    this._currentPage(pageModel.currentPage);
                    delete pageModel.currentPage;
                    $.extend(this.model.pageSettings, pageModel);
                    delete this.model.pageSettings.masterObject;
                    this._refreshGridPager();
                }
                if (!this.initialRender && (this.model.scrollSettings.frozenRows > 0 || this.model.scrollSettings.frozenColumns > 0))
                    this._refreshScroller(args);
            } else if (ej.Grid.Actions.Delete == args.requestType || ej.Grid.Actions.Save == args.requestType || ej.Grid.Actions.Search == args.requestType) {
                this._editEventTrigger(args);
                if (this.model.allowPaging)
                    this._refreshPagerTotalRecordsCount();
            } else if (ej.Grid.Actions.Filtering == args.requestType)
                this._filterCompleteAction();
            else if (ej.Grid.Actions.BeginEdit == args.requestType || ej.Grid.Actions.Add == args.requestType)
                this._editCompleteAction(args);
            else if (ej.Grid.Actions.Grouping == args.requestType || ej.Grid.Actions.Ungrouping == args.requestType)
                this["_" + args.requestType + "CompleteAction"](args);
            if (this.model.toolbarSettings.showToolbar)
                this.refreshToolbar();
            if (!this.initialRender && this.model.showSummary && this.model.summaryRows.length > 0 && this._currentJsonData.length) {
                if ((args.requestType == ej.Grid.Actions.Save || args.requestType == ej.Grid.Actions.Refresh ) && !this.element.children(".e-gridfooter").length)
                    this._renderGridFooter().insertAfter(this.getContent());
                this.getFooterTable().find('colgroup').remove();
                this.getFooterTable().append(this.getHeaderTable().find("colgroup").clone());
                this._createSummaryRows(this.getFooterTable());
            }
            if (!this.initialRender && ej.gridFeatures.selection) {
                if (!this.getContent().find("td.e-selectionbackground").length)
                    this._setCurrentRow(args.requestType);
                this.clearColumnSelection();
            }
            this.model.editSettings.editMode == "batch" && this.refreshBatchEditMode();
            if (!this.initialRender && this.model.allowScrolling && (this.model.scrollSettings.frozenColumns > 0))
                this.rowHeightRefresh();
            if (!this.initialRender && this.model.allowScrolling && (this._checkScrollActions(args.requestType) || (this.model.editSettings.editMode.indexOf("inline") != -1 && args.requestType == "beginedit")) ||
                (this.model.scrollSettings.virtualScrollMode == "continuous" && args.requestType == "virtualscroll"))
                this._refreshScroller(args);
            if (this.model.scrollSettings.virtualScrollMode == "normal" && args.requestType == "virtualscroll")
                this.getContent().find("div:first").scrollLeft(this.getScrollObject().scrollLeft());
            if (this._customPop != null && args.requestType != "sorting") {
                this._customPop.hide();
            }
            if (this.model.allowScrolling && !this.initialRender)
                this.getContentTable().find("tr:last").find("td").addClass("e-lastrowcell");

            if (this.model.allowGrouping && this.model.showSummary)
                this._refreshGroupSummary();
            if (ej.Grid.Actions.Refresh == args.requestType && !this.initialRender && this.model.groupSettings.groupedColumns.length > 0)
                this._groupingCompleteAction(args);
            if (ej.Grid.Actions.Refresh == args.requestType && !this.initialRender && this.model.groupSettings.groupedColumns.length < 1)
                this._ungroupingCompleteAction(args);
            this._trigger("actionComplete", args);
            this._trigger("refresh");
            if ((args.requestType == "delete" || (args.action == "add" && args.requestType == "save") || (args.requestType == "cancel" && !this.getContent().find("tr").hasClass("e-showaddrow")) || args.requestType == "paging") && (this.model.editSettings.showAddNewRow && this.model.editSettings.editMode == "normal"))
                this._startAdd();
            if ((ej.Grid.Actions.BeginEdit == args.requestType || ej.Grid.Actions.Add == args.requestType) && $.isFunction($.validator))
                this.setValidation();
            if (!this.initialRender)
                this.model._groupingCollapsed = [];
        },
        _getForeignKeyData: function (data) {
            var proxy = this;
            var column;
            for (i = 0; i < this.model.columns.length; i++) {
                if (this.model.columns[i].foreignKeyValue && this.model.columns[i].dataSource) {
                    var fieldName = ej.isNullOrUndefined(proxy.model.columns[i]["foreignKeyField"]) ? proxy.model.columns[i]["field"] : proxy.model.columns[i]["foreignKeyField"];
                    this.model.columns[i].dataSource.filter(function (col) {
                        var value = data[proxy.model.columns[i]["field"]];
                        var fValue = !isNaN(parseFloat(value)) && isFinite(value) ? parseFloat(value) : value;
                        if (col[fieldName] == fValue)
                            return column = col;
                    });
                }
            }
            return column;
        },
        _foreignKeyBinding: function (curColumn, cellValue, gridId) {
            if (typeof (curColumn) == "string")
                curColumn = JSON.parse(curColumn);
            var cellData;
            var gridObj = $("#" + gridId).ejGrid('instance');
            for (var column = 0; column < gridObj.model.columns.length; column++) {
                if (gridObj.model.columns[column]["field"] == curColumn.field && gridObj.model.columns[column]["foreignKeyValue"] == curColumn.foreignKeyValue) {
                    var foreignKeyColumn = gridObj.model.columns[column];
                    break;
                }
            }
            foreignKeyColumn.dataSource.filter(function (col) {
                if (col[foreignKeyColumn.foreignKeyField] == cellValue)
                    return cellData = foreignKeyColumn.type == "date" ? new Date(col[foreignKeyColumn.foreignKeyValue]) : col[foreignKeyColumn.foreignKeyValue];
            });
            if (curColumn.format) {
                var gridObj = ej.isNullOrUndefined(gridId) ? this : this.getRsc("helpers", gridId);
                cellData = gridObj.formatting(curColumn.format, cellData, gridObj.model.locale);
            }
            return cellData;
        },
        _getDataIndex: function (data, item) {
            var flag = 0, _plen;
            for (var d = 0, len = data.length; d < len; d++) {
                for (var key = 0, _plen = this._primaryKeys.length; key < _plen; key++) {
                    if (this._checkPrimaryValue(data[d][this._primaryKeys[key]], item[this._primaryKeys[key]], this._primaryKeys[key]))
                        continue;
                    else if (key == _plen - 1)
                        flag = 1;
                }
                if (flag) return d;
            }
            return -1;
        },
        _checkPrimaryValue: function (keyData, keyItem, field) {
            if (this.getColumnByField(field).type == "string")
                keyData = keyData.trim();
            if (keyData != keyItem)
                return true;
            else
                return false;
        },
        _eventBindings: function () {
            var rowLength = this.model.scrollSettings.frozenColumns > 0 ? this._gridRows[0].length : this._gridRows.length;
            var trIndex = 0;
            var prev;
            var pageSize = this.model.pageSettings.pageSize;
            if (ej.gridFeatures.common)
                this._refreshUnboundTemplate(this.getContentTable());
            if (this._gridRecordsCount != 0) {
                if (this.model.queryCellInfo != null || this.model.rowDataBound != null || this.model.mergeCellInfo != null || this.model.templateRefresh != null) {
                    for (var row = 0; row < rowLength; row++) {
                        trIndex = row;
                        if (this.model.scrollSettings.allowVirtualScrolling && row < pageSize) {
                            for (var i = 0; i < this._cloneQuery.queries.length; i++)
                                prev = this._cloneQuery.queries[i].fn == "onPage" &&  this._cloneQuery.queries[i].e.pageIndex - 1;
                            var value = pageSize * prev;
                            if (value != 0)
                                trIndex = this.getContentTable().find("tr[name=" + value + "]").eq(row).index();
                        }
                        var rowIndex = this.getRowByIndex(trIndex);
                        if (rowIndex.hasClass("e-virtualrow") || ej.isNullOrUndefined(this._currentJsonData[row]))
                            break;
                        this._rowEventTrigger(rowIndex, this._currentJsonData[row]);
                    }
                }
            }
        },
        _rowEventTrigger: function (row, data) {
            var args = { row: row, data: data };
            this._trigger("rowDataBound", args);
            var tdCells = row.cells;
            var $tdRowcells = $(row).find(".e-rowcell");
            for (var cellIndex = 0; cellIndex < $tdRowcells.length; cellIndex++) {
                var args = { cell: $tdRowcells[cellIndex], data: data, text: $tdRowcells[cellIndex].innerHTML };
                var foreignKeyData = this._getForeignKeyData(args.data);
                if ($($tdRowcells[cellIndex]).hasClass("e-rowcell"))
                    args.column = this.model.columns[cellIndex];
                if (!ej.isNullOrUndefined(foreignKeyData))
                    args.foreignKeyData = foreignKeyData;
                if (this.model.allowCellMerging == true) {
                    this._cellMerging(args);
                    this._trigger("mergeCellInfo", args);
                }
                this._trigger("queryCellInfo", args);
                if ($($tdRowcells[cellIndex]).hasClass("e-templatecell")) {
                    var args = { cell: $tdRowcells[cellIndex], column: this.model.columns[cellIndex], data: data, rowIndex: $(row).index() };
                    this._trigger("templateRefresh", args);
                }
            }

        },

        setWidthToColumns: function () {
            var $cols1 = this.getContentTable().children("colgroup").find("col");
            var $cols2 = this.getHeaderTable().children("colgroup").find("col");
            var width = this.element.width(), frozenWidth = 0, columnsTotalWidth = 0, finalWidth=0;
            if (this.model.groupSettings.groupedColumns.length && !this.model.allowScrolling && this.model.groupSettings.showGroupedColumn) {
                var browserDetails = this.getBrowserDetails();
                if (browserDetails.browser == "msie" && parseInt(browserDetails.version, 10) > 8)
                    $cols1.first().css("width", ((30 / width) * 100) + "%");
            }
            if (!ej.isNullOrUndefined(this.model.detailsTemplate)) {
                var headerIndx = this.model.groupSettings.groupedColumns.length;
                var contentIndx = this.model.groupSettings.groupedColumns.length != 0 ? 1 : 0;
                $cols1.eq(contentIndx).css("width", this._detailsOuterWidth);
                $cols2.eq(headerIndx).css("width", this._detailsOuterWidth);
            }
            this._detailColsRefresh();
            $cols1 = this._$headerCols;
            $cols2 = this._$contentCols;

            for (var i = 0; i < $cols2.length; i++) {
                if (this.model.allowResizeToFit && this.model.columns[i]["width"] === undefined) {
                    hCellIndex = this.model.groupSettings.groupedColumns.length ? (i + this.model.groupSettings.groupedColumns.length) : i;
                    var contentWidth = this._resizer._getContentWidth(i);
                    var cellDiv = this.getHeaderTable().find('.e-headercelldiv').eq(hCellIndex);
                    var headerWidth = this._resizer._getHeaderContentWidth(cellDiv);
                    if (this.model.editSettings.editMode == "normal" && (this.model.isEdit || this._isAddNew))
                        finalWidth = $cols2.eq(i).width();
                    else {
                         finalWidth = contentWidth > headerWidth ? contentWidth : headerWidth;
					     finalWidth += parseInt(cellDiv.css("padding-left"), 10)
					}
                        this._columnsWidthCollection[i] = finalWidth;
                        columnsTotalWidth += finalWidth;
                    
                } else
                    columnsTotalWidth += parseInt(this.model.columns[i]["width"], 10);
                if (!ej.isNullOrUndefined(this._columnsWidthCollection[i])) {
                    $cols1.eq(i).width(this._columnsWidthCollection[i]);
                    $cols2.eq(i).width(this._columnsWidthCollection[i]);
                    if (this.model.columns[i]["priority"])
                        $cols2.eq(i).addClass("e-table-priority-" + this.model.columns[i]["priority"]);
                } else if (this.model.allowScrolling) {
                    var colWidth = (width / this.model.columns.length).toFixed(2), bSize = (width / (this.model.scrollSettings.buttonSize || 18) / 100).toFixed(2), cWidth = colWidth - bSize;
                    $cols1.eq(i).css("width", cWidth + "px");
                    $cols2.eq(i).css("width", cWidth + "px");
                    this.model.columns[i].width = cWidth;
                    this._columnsWidthCollection[i] = parseFloat(cWidth);
                }
            }
            if (!this.model.allowScrolling && this.model.allowResizeToFit && columnsTotalWidth > width) {
                this.model.allowScrolling = true;
                this.model.scrollSettings.width = width;
                this.getHeaderTable().parent().addClass("e-headercontent");
                if (!this.model.scrollSettings.frozenColumns > 0)
                    this.getHeaderTable().width(width);
            }
            if (this.model.isEdit) {
                var clonedCol = $cols1.clone();
                var editedTr;
                if (this.model.editSettings.showAddNewRow)
                    editedTr = this.getContentTable().find(".e-editedrow");
                $colGroup = this.model.scrollSettings.frozenColumns > 0 ? this.getContent().find(".gridform").find("colgroup") : !ej.isNullOrUndefined(editedTr) && editedTr.length == 1 ? editedTr.find("colgroup") : $("#" + this._id + "EditForm").find("colgroup");
                this.model.scrollSettings.frozenColumns > 0 && $colGroup.first().empty().append(clonedCol.splice(0, this.model.scrollSettings.frozenColumns));
                $colGroup.last().empty().append(clonedCol);
                if (this.model.detailsTemplate != null || this.model.childGrid != null)
                    $colGroup.prepend(this._getIndentCol());
            }
            if (this.model.groupSettings.groupedColumns.length) {
                var $grouedColGroup = this.getContentTable().find(".e-recordtable").children("colgroup");
                for (var i = 0; i < $grouedColGroup.length; i++) {
                    var clonedCol = $cols1.clone();
                    var detailsWidth = this._detailsOuterWidth != null ? this._detailsOuterWidth : "30px";
                    if (this.model.detailsTemplate != null || this.model.childGrid != null) clonedCol.splice(0, 0, $(this._getIndentCol()).width(detailsWidth)[0]);
                    $grouedColGroup.eq(i).empty().append(clonedCol);
                }
            }
            if (this.model.scrollSettings.frozenColumns > 0) {
                var totalWidth = 0, frozenWidth;
                for (var i = 0; i < this._columnsWidthCollection.length; i++) {
                    totalWidth += this._columnsWidthCollection[i];
                    if (this.model.scrollSettings.frozenColumns - 1 == i)
                        frozenWidth = Math.ceil(totalWidth);
                }
                this.getContent().find(".e-frozencontentdiv").outerWidth(frozenWidth)
                    .end().find(".e-movablecontentdiv").outerWidth(totalWidth - frozenWidth);
                this.getHeaderContent().find(".e-frozenheaderdiv").outerWidth(frozenWidth)
                    .end().find(".e-movableheaderdiv").outerWidth(totalWidth - frozenWidth);
            }
        },
        _initialEndRendering: function () {
            // use this method to add behaviour after grid render.
            if (this.model.editSettings.allowEditing || this.model.editSettings.allowAdding) {
                if (this.model.editSettings.editMode == "dialog" || this.model.editSettings.editMode == "dialogtemplate")
                    this.element.append(this._renderDialog());
                else if (this.model.editSettings.editMode == "externalform" || this.model.editSettings.editMode == "externalformtemplate")
                    this.element.append(this._renderExternalForm());
            }
            if (this._selectedRow() != -1)
                this.selectRows(this._selectedRow());
            (this.model.editSettings.editMode == "batch" || this.model.editSettings.showDeleteConfirmDialog) && this._renderConfirmDialog();
            (this.model.scrollSettings.frozenColumns > 0 || this.model.scrollSettings.frozenRows > 0) && $("#" + this._id + 'AlertDialog') == null && this._renderAlertDialog();
            if (this.model.allowMultiSorting || this.model.selectionType == "multiple")
                this._renderMultiTouchDialog();
            if (this.model.scrollSettings.frozenColumns > 0 && !this.model.allowScrolling) {
                this.getContent().remove();
                this.getHeaderTable().eq(1).remove();
                this._alertDialog.find(".e-content").text(this._getLocalizedLabels("FrozenColumnsScrollAlert"));
                this._alertDialog.ejDialog("open");
                return;
            }
            this.model.scrollSettings.allowVirtualScrolling && this._createPagerStatusBar();
            this._getRowHeights();
            this.model.allowScrolling && this._renderScroller();
            this._addMedia();
            if (this.model.scrollSettings.allowVirtualScrolling) {
                if (this._currentPage() == 1)
                    this._virtualLoadRecords[this._currentPage()] = this._currentJsonData;
                this._refreshVirtualContent();
                this.getContent().first().ejScroller("refresh");
                if (this.getContent().ejScroller("isVScroll")) {
                    this.element.find(".e-gridheader").addClass("e-scrollcss");
                    this.getHeaderTable().first().width(this.getContentTable().width());
                }
                else
                    this.element.find(".e-gridheader").removeClass("e-scrollcss");
            }
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "filterbar" && !this.model.allowPaging && !this.model.scrollSettings.allowVirtualScrolling)
                this._createPagerStatusBar();
            if (ej.gridFeatures.common)
                this.rowHeightRefresh()
            if (ej.gridFeatures.filter && ["menu", "excel"].indexOf(this.model.filterSettings.filterType) != -1)
                this._refreshFilterIcon();
            if (this.model.allowGrouping && this.model.groupSettings.groupedColumns.length != 0)
                this._recalculateIndentWidth();
            if (this.initialRender)
                this._addLastRow();
        },

        _addLastRow: function () {
            var lastRowtd = this.getContentTable().find("tr:last").find("td"), rowHeight = 0;

            if (this.model.allowScrolling && !this.model.scrollSettings.allowVirtualScrolling && !ej.isNullOrUndefined(this.model.dataSource) && !ej.isNullOrUndefined(this.getRows())) {
                for (var i = 0; i < this.getRows().length; i++)
                    rowHeight += $(this.getRows()[i]).height();

                if (rowHeight < this.getContent().height() - 1)
                    lastRowtd.addClass("e-lastrowcell");
            }
        },
        _addMedia: function () {
            if (this.model.allowScrolling)
                this._scrollWidth = this.model.scrollSettings.width;
            if (this.model.minWidth && !this.model.enableResponsiveRow) {
                this._$onresize = $.proxy(this.windowonresize, this);
                $(window).bind("resize", this._$onresize);
                if ($.isFunction(window.matchMedia)) {
                    var mediaFilterObj = window.matchMedia("(max-width: 768px)");
                    this._mediaStatus = mediaFilterObj.matches;
                }
                this.windowonresize();
            }
        },
        _getNoncontentHeight: function () {
            var height = this.getHeaderContent().outerHeight();
            if (this.model.toolbarSettings.showToolbar)
                height += this.element.find('.e-gridtoolbar').outerHeight();
            if (this.model.allowPaging)
                height += this.element.find('.e-pager').outerHeight();
            if (this.model.allowGrouping && this.model.groupSettings.showDropArea)
                height += this.element.find('.e-groupdroparea').outerHeight();
            return height;
        },
        
        setDimension: function (height, width) {
            var originalHeight = height - this._getNoncontentHeight();
            this.model.scrollSettings.height = originalHeight;
            this.model.scrollSettings.width = width;
            this._renderScroller();
        },
        _mediaQueryUpdate: function (verticalScroll,horizontalScroll) {
            if (horizontalScroll || verticalScroll) {
                var top = this.element.height();
                var cTop = this.getContent().offset()['top'];
                var height = $(window).height() - this._getNoncontentHeight();
                var s = this.getContent().data('ejScroller');
                if (!verticalScroll) {
                    height = this.model.scrollSettings.height;
                }
                this.model.scrollSettings.height = height;
                if (this.model.minWidth && !this.element.hasClass("e-responsive") && !this._scrollerAddedOnMedia) {
                    this.getContentTable().css("min-width", this.model.minWidth);
                    this.getHeaderTable().css("min-width", this.model.minWidth);
                    this.model.allowScrolling = true;
                    this._mediaQuery = true;
                    this.model.scrollSettings.width = this.getContent().width();
                    this.model.scrollSettings.height = height;
                    this.element.find(".e-gridheader").first().find("div").first().addClass("e-headercontent");
                    this._renderScroller();
                    this._mediaQuery = false;
                    this._scrollerAddedOnMedia = true;
                }
                if (this._scrollerAddedOnMedia)
                    this.getContent().ejScroller({ width: Math.floor(this.getHeaderContent()[0].getBoundingClientRect().width), height: height });
                if (verticalScroll && !this.getHeaderContent().hasClass("e-scrollcss")) {
                    this.getHeaderContent().addClass("e-scrollcss")
                    this.getHeaderContent().find('div.e-headercontent').addClass('e-hscrollcss');
                }
                if (!verticalScroll) {
                    this.getHeaderContent().removeClass("e-scrollcss")
                    this.getHeaderContent().find('div.e-headercontent').removeClass('e-hscrollcss');
                }
            } else if (this._scrollerAddedOnMedia) {
                if (this.getHeaderContent().hasClass("e-scrollcss")) {
                    this.getHeaderContent().removeClass("e-scrollcss");
                    this.getHeaderContent().find(".e-headercontent").first().removeClass("e-headercontent");
                }
                if (this.getContent().data("ejScroller"))
                    this.getContent().ejScroller("destroy");
                this.getContentTable().css("min-width", "");
                this.getHeaderTable().css("min-width", "");
                this._scrollerAddedOnMedia = false;
            }
        },
        windowonresize: function () {
            this.model.scrollSettings.width = this._originalScrollWidth;
            if (typeof (this._originalScrollWidth) == "number" && this.element.parent().width() < this._originalScrollWidth)
                this.model.scrollSettings.width = this.element.parent().width();
            var $width = typeof (this._originalScrollWidth) == "string" || this.element.parent().width() < this.element.width() ? "auto" : this.model.scrollSettings.width;
            this.getHeaderContent().find("div table").first().width("100%");
            this.getContent().find("div table").first().width("100%");
            this.element.css("width", $width);
            this.getContent().css("width", $width);
            if (this.model.allowGrouping)
                this._recalculateIndentWidth();
            var scrollHorizontal = this.getContent().width() < this.getContent().find('table').width();
            var contentTableHeight = scrollHorizontal ? this.getContentTable().height() + 18 : this.getContentTable().height();
            var scrollVertical = ($(window).height() - this._getNoncontentHeight()) < contentTableHeight;
            this._mediaQueryUpdate(scrollVertical, scrollHorizontal);
        },
        _removeMedia: function () {
            $(window).unbind("resize", this._$onresize);
            this.getContentTable().css("min-width", "");
            this.getHeaderTable().css("min-width", "");
            this.getContentTable().css("width", "");
            this.model.scrollSettings.width = "auto";
            if (this.getContent().data("ejScroller"))
                this.getContent().ejScroller("destroy");
        },
        _getRowHeights: function () {
            var trs = this.getRows();
            if (trs !== null) {
                this._rowHeightCollection = [];
                if (trs[1] !== undefined && trs[1].length && ((this.model.scrollSettings.frozenColumns > 0 && trs[0] !== undefined) || (trs[0] !== undefined && typeof trs[0].item !== "undefined" && typeof trs[0].length == "number" && typeof trs[1].item !== "undefined" && typeof trs[1].length == "number"))) {
                    frotrs = trs[0];
                    movtrs = trs[1];
                    for (var i = 0 ; i < frotrs.length ; i++) {
                        this._rowHeightCollection[i] = frotrs[i].offsetTop >= movtrs[i].offsetTop ? frotrs[i].offsetTop : movtrs[i].offsetTop;
                    }
                }
                else {
                    for (var i = 0 ; i < trs.length ; i++) {
                        this._rowHeightCollection[i] = trs[i].offsetTop;
                    }
                }
            }
            return this._rowHeightCollection;
        },
        _getEmptyTbody: function () {
            var $emptyTd = ej.buildTag('td.emptyrecord', this._getLocalizedLabels("EmptyRecord"), {}, { colSpan: this.model.columns.length });
            return $(document.createElement("tr")).append($emptyTd);
        },
        _getIndentCol: function () {
            return ej.buildTag("col", "", { width: "30px" });
        },
        _createSortElement: function () {
            return ej.buildTag('span.e-icon', "&nbsp;");
        },
        _createSortNumber: function (number, header) {
            if (header.css("text-align") == "right")
                return ej.buildTag('span.e-number', number, { "color": "white", "font-size": "9px", "float": "left" });
            else
                return ej.buildTag('span.e-number', number, { "color": "white", "font-size": "9px", "float": "right" });
        },
        _wireEvents: function () {
            this._on(this.element, ($.isFunction($.fn.tap) && this.model.enableTouch) ? "tap" : "click", this._clickHandler);
            this._on(this.element, ($.isFunction($.fn.tap) && this.model.enableTouch) ? "tap" : "click", ".e-gridheader", this._mouseClickHandler);
            if (ej.gridFeatures.common) {
                this._on(this.element, "dblclick", ".e-gridcontent > div:first", this._recorddblClickHandler);
                if (this.model.rightClick)
                    this._on(this.element, "contextmenu", this._rightClickHandler);
                this._on(this.element, "click", ".e-gridcontent", this._recordClick);
                this._enableRowHover();
                if (this.model.enableTouch)
                    this._on(this.element, "swipeleft swiperight", ".e-gridcontent div > .e-table", $.proxy(this._touchGrid, this));
                else
                    this.element.addClass("e-touch");
                this._on(this.element, "mousedown", ".e-gridheader", this._headerMouseDown);
                this._on(this.element, "mouseover mouseleave", ".e-gridheader", this._headerHover);
                this._on(this.element, ej.eventType.mouseMove, ".e-gridheader", this._headerHover);
                this.model.allowResizeToFit && this._on(this.element, "dblclick", ".e-gridheader", this._headerdblClickHandler);
                if (this.model.allowResizing) {
                    this._on(this.element, ej.eventType.mouseMove, this._mouseMove);
                    this._on(this.element, "mouseup", this._mouseUp);
                }
                if (this.model.allowKeyboardNavigation) {
                    this.element[0].tabIndex = this.element[0].tabIndex == -1 ? 0 : this.element[0].tabIndex;
                    this.element[0].accessKey = (!ej.isNullOrUndefined(this.element[0].accessKey) && this.element[0].accessKey != "") ? this.element[0].accessKey : "e";
                    this._on(this.element, "keyup", this._keyDownHandler);
                }
            }
            if (ej.gridFeatures.edit) {
                this._enableEditingEvents();
                this._on(this.element, "click", ".e-gridcontent .e-unboundcelldiv", this._unboundClickHandler);
            }
            if (this.model.allowGrouping) {
                this._enableGroupingEvents();
                this._on(this.element, "mouseenter mouseleave", ".e-groupdroparea,.e-groupheadercell", this._dropAreaHover);

            }
            this._enableFilterEvents();
        },
        _enableFilterEvents: function () {
            if (this.model.allowMultiSorting || this.model.selectionType == "multiple" || this.model.allowFiltering)
                this._on($(document), "mousedown", this._docClickHandler);
            if (this.model.allowFiltering) {
                var proxy = this, $target;
                this._off(this.element, "keyup", ".e-filterbar input")._on(this.element, "keyup", ".e-filterbar input", this._filterBarHandler);
                this._on(this.element, "focus click", ".e-filterbar", this._filterBarClose);
            }
        },
        _docClickHandler: function (e) {
            var details = this.getBrowserDetails(), $target = $(e.target);
            if (this._customPop != null && this.element.find(e.target).length == 0)
                this._customPop.hide();
            if (this.model.allowFiltering) {
                if (this.model.filterSettings.filterType == "menu" || this.model.filterSettings.filterType == "excel") {
                    if (this._$colType && ($(e.target).find(".e-grid.e-dlgcontainer").length > 1 || $(e.target).find(".e-excelfilter").length > 1))
                        if (details.browser == "msie")
                            e.target.tagName != "BODY" && (!this.isExcelFilter ? this._closeFilterDlg() : this._excelFilter.closeXFDialog(e));
                        else
                            !this._isExcelFilter ? this._closeFilterDlg() : this._excelFilter.closeXFDialog(e);
                } else if (!$target.hasClass("e-filtertext") && !$target.hasClass("e-cancel"))
                    this.getFilterBar().find(".e-cancel").addClass("e-hide");
            }

        },
        _mouseClickHandler: function (e) {
            var $temp = $(e.target), $target;
            if (!(this.model.isResponsive || this.model.enableResponsiveRow) && $temp.closest(".e-grid").length != 0 && $temp.closest(".e-grid").attr("id") !== this._id) return;
            if (this.getHeaderTable().find('.e-columnheader').not('.e-stackedHeaderRow').css('cursor') == "col-resize")
                return;
            if ($(e.target).is(".e-ascending, .e-descending"))
                $target = $(e.target.parentNode);
            else if ($temp.hasClass('e-groupheadercell'))
                $target = $temp.children("div");
            else
                $target = $(e.target);
            if (this._$fDlgIsOpen && this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this._isExcelFilter)) {
                $.fx.off = true;
                if (this._isExcelFilter)
                    this._excelFilter.closeXFDialog();
                else if ($("#" + this._id + "_" + this._$prevColType + "Dlg").data("ejDialog") != undefined)
                    $("#" + this._id + "_" + this._$prevColType + "Dlg").ejDialog("close");
                $.fx.off = false;
            }
            this.getHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive").removeClass("e-active");
            if ($target.hasClass("e-headercelldiv") || (!$target.hasClass("e-togglegroupbutton") && $target.closest(".e-headercelldiv").length && $.inArray($target[0].tagName, ["SELECT", "INPUT", "TEXTAREA"]) == -1)
                || ($target.closest(".e-groupheadercell").length && $(e.target).is(".e-ascending, .e-descending"))) {
                if (!this.model.allowSorting || ej.gridFeatures.sort === undefined)
                    return;
                $target = ($target.hasClass("e-headercelldiv") || $target.closest(".e-groupheadercell").length) ? $target : $target.closest(".e-headercelldiv");
                var columnName = $target.attr("ej-mappingname");
                var columnSortDirection = ej.sortOrder.Ascending;
                this._$prevSElementTarget = this._$curSElementTarget;
                this._$curSElementTarget = $target;
                if ($target.find('span').hasClass("e-ascending"))
                    var columnSortDirection = ej.sortOrder.Descending;
                else
                    var columnSortDirection = ej.sortOrder.Ascending;
                if (e["pointerType"] == "touch" && this._customPop != null && !this._customPop.is(":visible") && this._customPop.find(".e-sortdirect").hasClass("e-spanclicked"))
                    this._customPop.show();
                if (e["pointerType"] == "touch" && this._customPop != null && (this._customPop.find(".e-rowselect").is(":visible") || !this._customPop.find(".e-sortdirect").hasClass("e-spanclicked")) && this.model.allowMultiSorting) {
                    var $offset = $target.offset();
                    this._customPop.removeAttr("style");
                    this._customPop.offset({ left: $offset.left, top: $offset.top - this.getHeaderTable().find(".e-columnheader").height() }).find(".e-sortdirect").show().end()
                        .find(".e-rowselect").hide().end().show();
                }
                if (this.model.allowMultiSorting && (e.ctrlKey || this._enableSortMultiTouch))
                    this.multiSortRequest = true;
                if (e.shiftKey && $.inArray(columnName, this.model.groupSettings.groupedColumns) == -1) {
                    this._removeSortedColumnFromCollection(columnName);
                    this.multiSortRequest = true;
                    columnName = null;
                    this.sortColumn(columnName, columnSortDirection);
                }
                if (!ej.isNullOrUndefined(columnName))
                    this.sortColumn(columnName, columnSortDirection);
            } else if ($target.hasClass("e-togglegroupbutton") && this.model.allowGrouping) {
                var field = $target.parent().attr("ej-mappingname");
                $target.hasClass("e-togglegroup") && this.groupColumn(field);
                $target.hasClass("e-toggleungroup") && this.ungroupColumn(field);
            } else if ($target.hasClass("e-filtericon") || $target.hasClass("e-filteredicon") || $target.hasClass('e-responsivefilterColDiv') || $target.parent().hasClass('e-responsivefilterColDiv')) {
                var columnName = $target.parent().find(".e-headercelldiv").attr("ej-mappingname") || $target.attr("ej-mappingname") || $($target.parent()).attr("ej-mappingname");
                this._$prevFieldName = this._$curFieldName;
                if (this.model.allowFiltering) {
                    var proxy = this;
                    $.each(this.model.columns, function (indx, col) {
                        if (col.field == columnName) {
                            proxy._$colType = col.type;
                            proxy._$curFieldName = col.field;
                            proxy._$colFormat = col.format;
                            proxy._$colForeignKeyField = col.foreignKeyField ? col.foreignKeyField : col.field;
                            proxy._$colForeignKeyValue = col.foreignKeyValue;
                            proxy._$colDropdownData = col.dataSource;
                        }
                    });
                    if (!this._isExcelFilter) {
                        var $id = "#" + this._id + "_" + this._$colType + "Dlg";
                        if (this._$colType == "string") {
                            if (this._$colForeignKeyValue && this._$colDropdownData)
                                $("#" + this._id + "_acString").ejAutocomplete({ fields: { text: proxy._$colForeignKeyValue, key: proxy._$colForeignKeyField }, dataSource: proxy._$colDropdownData });
                            else
                                $("#" + this._id + "_acString").ejAutocomplete({ fields: { text: proxy._$curFieldName, key: this._getIdField() }, dataSource: this._dataSource() });
                        } else if (this._$colType == "date") {
                            if (this._$colFormat != undefined) {
                                this._$colFormat = this._$colFormat.replace("{0:", "").replace("}", "");
                                $($id).find(".e-datewidget .e-datepicker").ejDatePicker({ dateFormat: this._$colFormat.replace("{0:", "").replace("}", "") });
                            }
                            else
                                $($id).find(".e-datewidget .e-datepicker").ejDatePicker({ dateFormat: "MM/dd/yyyy" });
                        }
                        else if (this._$colType == "datetime") {
                            if (this._$colFormat != undefined) {
                                this._$colFormat = this._$colFormat.replace("{0:", "").replace("}", "");
                                $($id).find(".e-datetimewidget input").ejDateTimePicker({ dateFormat: this._$colFormat.replace("{0:", "").replace("}", "") });
                            }
                            this._setFilterFieldValues($id);
                        }
                    }
                    this._mediaStatus = document.documentElement.clientWidth < 768;
                    if (this.model.isResponsive && this._mediaStatus) {
                        var gridObj = this;
                        var $headerDiv = ej.buildTag('div.e-resFilterDialogHeaderDiv');
                        var $titleSapn = ej.buildTag('div.labelRes', '<span>Filter</span>');
                        if ($(".e-filterMenuBtn").length > 0)
                            $(".e-filterMenuBtn").remove();
                        var $dlgBtn = ej.buildTag('div.e-filterMenuBtn');
                        var $inputOk = ej.buildTag('input.e-resposnsiveFilterBtnLeft');
                        var $inputCancel = ej.buildTag('input.e-resposnsiveFilterBtnRight');

                        $headerDiv.append($titleSapn);
                        $headerDiv.css('width', '100%');
                        var $dlgClone = $($id).css('padding-left', '0px');
                        if ($target.parent().hasClass('e-responsivefilterColDiv') && $target.hasClass('e-filternone')) {
                            proxy._fltrClrHandler();
                            $target.remove();
                            $("#responsiveFilter").css('display', 'block');
                            // this.element.css('display', 'block');
                        }
                        else {
                            this.element.css('display', 'none');
                            setTimeout(function () { $("#responsiveFilter").css('display', 'none'), 0 });
                            if (!this._isExcelFilter) {
                                var btnText = this.model.enableResponsiveRow ? 'OK' : 'Filter', clearText = this.model.enableResponsiveRow ? 'Cancel' : 'Clear';
                                $inputOk.ejButton({ text: btnText, type: 'button', click: $.proxy(this._fltrBtnHandler, this) });
                                $inputCancel.ejButton({
                                    text: clearText, type: 'button', click: function () {
                                        if (clearText == 'Clear') {
                                            proxy.element.css('display', 'block');
                                            proxy._fltrClrHandler();
                                        }
                                        $($id).css('display', 'none');
                                        if ($inputCancel.hasClass("e-resposnsiveFilterBtnRight"))
                                            proxy.element.css('display', 'block');
                                    }
                                });
                                if ($target.parent().hasClass('e-responsivefilterColDiv') && $target.hasClass('e-filternone')) {
                                    proxy._fltrClrHandler();
                                    $target.remove();
                                }
                                else {
                                    $dlgClone.addClass('e-resMenuFltr');
                                    $dlgClone.css('height', $(window).height() - 1).css('width', $(window).width() - 2);
                                    $dlgClone.find('.e-operator').addClass('e-resFilterOperator');
                                    $dlgClone.find('.e-value').addClass('e-resFilterOperator');
                                    var $btnContainer = $dlgClone.find('.e-dlgBtns').remove().addClass('e-filterMenuBtn');
                                    $dlgClone.append($dlgBtn.append($inputOk).append($inputCancel));
                                    if (ej.isNullOrUndefined($dlgClone.find('.e-resFilterDialogHeaderDiv')[0])) {
                                        $dlgClone.insertAfter(this.element);
                                        var $backIcon = ej.buildTag('div.e-resFilterleftIcon', '', {}, { closeDialogue: $id.slice(1), openDialogue: 'responsiveFilter' });
                                        var $spanIcon = ej.buildTag('span.e-icon e-resIcon e-responisveBack', '', {}, { closeDialogue: $id.slice(1), openDialogue: 'responsiveFilter' })
                                        $backIcon.click(function (e) {
                                            $dlgClone.css('display', 'none');
                                            if (gridObj.model.enableResponsiveRow)
                                                $("#responsiveFilter").css('display', 'block');
                                            else
                                                gridObj.element.css('display', 'block');
                                        })
                                        $headerDiv.append($backIcon.append($spanIcon));
                                        var $closeIcon = ej.buildTag('div.e-resFIlterRigthIcon', '', {}, { closeDialogue: $id.slice(1), gridEle: true });
                                        var $closeSpan = ej.buildTag('span.e-icon e-resIcon e-responisveClose', '', {}, { closeDialogue: $id.slice(1), gridEle: true })
                                        $closeIcon.click(function (e) {
                                            $dlgClone.css('display', 'none');
                                            gridObj.element.css('display', 'block');
                                        });
                                        var $ejWid = $($dlgClone.find('.e-value').find('input:last'));
                                        if (proxy._$colType == 'string') {
                                            var model = $($dlgClone.find('.e-value').find('input:last')).ejAutocomplete('model');
                                            $ejWid.ejAutocomplete('destroy').ejAutocomplete({
                                                enableDistinct: true, dataSource: model.dataSource, fields: model.fields, width: model.width, focusIn: function (args) {
                                                    var $dropdown = this.element.closest(".e-filterDialoge").find(".e-dropdownlist");
                                                    this.model.filterType = $dropdown.val();
                                                }
                                            });
                                        }
                                        $headerDiv.append($closeIcon.append($closeSpan));
                                        $dlgClone.prepend($headerDiv);
                                    }
                                    $dlgClone.find('.e-responsiveLabelDiv').remove();
                                    var $label = ej.buildTag('div.e-responsiveLabelDiv', '', { 'margin-left': '5%', 'font-size': '17px', 'margin-top': '5%' }).append(ej.buildTag('span', this.getHeaderTextByFieldName(columnName), { 'font-weight': 'bold' }));
                                    $label.insertAfter($dlgClone.find('.e-resFilterDialogHeaderDiv'));
                                    $dlgClone.fadeIn(100, "easeOutQuad", function () {
                                    });
                                }
                            }
                            else {
                                this._excelDlg = $id = "#" + this._id + this._$colType + "_excelDlg";
                                if (ej.isNullOrUndefined($dlgClone.find('.e-resFilterDialogHeaderDiv')[0])) {
                                    $inputOk.ejButton({
                                        text: 'OK', type: 'button', click: function (sender) {
                                            gridObj._excelFilter._openedFltr = $(gridObj._excelDlg);
                                            gridObj._excelFilter._fltrBtnHandler();
                                            gridObj._setResponsiveFilterIcon();
                                            gridObj.element.css('display', 'block');
                                        }
                                    });
                                    $inputCancel.ejButton({ text: 'Cancel', type: 'button', click: function () { $($id).css('display', 'none'); proxy.element.css('display', 'block') } });
                                    $($id).children().not('.e-searchcontainer').remove();
                                    var excelObj = this._excelFilter;
                                    var $backIcon = ej.buildTag('div.e-resFilterleftIcon', '', {}, { closeDialogue: $id.slice(1), openDialogue: 'responsiveFilter' });
                                    var $spanIcon = ej.buildTag('span.e-icon e-resIcon e-responisveBack', '', {}, { closeDialogue: $id.slice(1), openDialogue: 'responsiveFilter' })
                                    $backIcon.click(function (e) {
                                        $.proxy(gridObj._closeDivIcon(e), this);
                                    })
                                    $headerDiv.append($backIcon.append($spanIcon));
                                    var $closeIcon = ej.buildTag('div.e-resFIlterRigthIcon', '', {}, { closeDialogue: $id.slice(1), gridEle: true });
                                    var $closeSpan = ej.buildTag('span.e-icon e-resIcon e-responisveCustomFilter', '', {}, { closeDialogue: $id.slice(1), gridEle: true })
                                    $closeIcon.click(function (e) {
                                        $.proxy(gridObj._closeDivIcon(e), this);
                                    })
                                    if (proxy._$colType != 'boolean')
                                        $headerDiv.append($closeIcon.append($closeSpan));
                                    $($id).css('padding', '0px');
                                    var $searchContainer = $($id).css('height', $(window).height() - 2);
                                    $searchContainer.css('width', $(window).width() - 2);
                                    var $searchBox = $searchContainer.find('.e-searchcontainer .e-searchbox').css('margin-top', '10px');
                                    $searchBox.children().css('margin-top', '10px');
                                    var $checkBoxDiv = $searchContainer.find('.e-checkboxlist');
                                    var scrolWidth = $(window).width() * (97 / 100), scrollHeight = $(window).height() * (65 / 100);
                                    $($searchContainer.find('.e-searchcontainer')).addClass('e-resSearch');
                                    var $btn = $searchContainer.find('.e-resSearch .e-btncontainer').remove();
                                    $btn.find('input:first').css('width', '45.6%');
                                    $btn.find('input:first').addClass('e-resposnsiveFilterBtnLeft');
                                    $btn.find('input:last').addClass('e-resposnsiveFilterBtnRight');
                                    $searchContainer.find('.e-excelLabel').remove();
                                    var $labelDiv = ej.buildTag('div.e-excelLabel', 'Order Id', { 'font-weight': 'bold', 'margin-top': '10px' });
                                    var $searchBox = $searchContainer.find('.e-searchcontainer');
                                    $labelDiv.insertAfter($searchContainer.find('.e-searchcontainer .e-searchbox'));
                                    $searchContainer.prepend($headerDiv);
                                    $($id).append($dlgBtn.append($inputOk).append($inputCancel))
                                    $checkBoxDiv.ejScroller({ height: scrollHeight, width: scrolWidth }).ejScroller('refresh');
                                }
                                this._excelFilter.openXFDialog({ field: columnName, enableResponsiveRow: true, displayName: this.getHeaderTextByFieldName(columnName), dataSource: this._dataSource(), position: { X: xPos, Y: yPos }, dimension: { height: $(window).height(), width: $(window).width() }, cssClass: "resFilter", type: this._$colType, format: this._$colFormat });
                                $($id).insertAfter(this.element);
                                !ej.isNullOrUndefined($($id).parents('.e-grid')[0]) && $($id).remove();
                                $closeIcon.click(function (e) {
                                    $(gridObj._excelDlg).css('display', 'none');
                                    var height = $(window).height() - 5, width = $(window).width();
                                    excelObj._openCustomFilter('equal');
                                    var $dlgClone = $id = $("#" + gridObj._id + gridObj._$colType + "_CustomFDlg").addClass('e-responsviesExcelFilter');
                                    if (ej.isNullOrUndefined($dlgClone.find('.e-resFilterDialogHeaderDiv')[0])) {
                                        var $headerDivCustom = ej.buildTag('div.e-resFilterDialogHeaderDiv');
                                        var $titleSapn = ej.buildTag('div.labelRes', '<span>Custom Filter</span>');
                                        $headerDivCustom.append($titleSapn);
                                        var $backIcon = ej.buildTag('div.e-resFilterleftIcon', '', {}, { closeDialogue: gridObj._id + gridObj._$colType + "_CustomFDlg", openDialogue: gridObj._excelDlg.slice(1) });
                                        var $spanIcon = ej.buildTag('span.e-icon e-resIcon e-responisveBack', '', {}, { closeDialogue: gridObj._id + gridObj._$colType + "_CustomFDlg", openDialogue: gridObj._excelDlg.slice(1) })
                                        $backIcon.click(function (e) {
                                            $dlgClone.css('display', 'none');
                                            if (gridObj.model.enableResponsiveRow)
                                                $("#responsiveFilter").css('display', 'block');
                                            else
                                                gridObj.element.css('display', 'block');
                                        });
                                        $headerDivCustom.append($backIcon.append($spanIcon));
                                        var $closeIconCust = ej.buildTag('div.e-resFIlterRigthIcon', '', {}, { closeDialogue: gridObj._id + gridObj._$colType + "_CustomFDlg", gridEle: true });
                                        var $closeSpan = ej.buildTag('span.e-icon e-resIcon e-responisveClose', '', {}, { closeDialogue: gridObj._id + gridObj._$colType + "_CustomFDlg", gridEle: true })
                                        $headerDivCustom.append($closeIconCust.append($closeSpan));
                                        $closeIconCust.click(function (e) {
                                            $dlgClone.css('display', 'none');
                                            gridObj.element.css('display', 'block');
                                        })
                                        $dlgClone.prepend($headerDivCustom);
                                        $dlgClone.insertAfter(gridObj.element);
                                        $dlgClone.find('.e-dlgfields').css('width', '100%');
                                        var $firstDiv = $dlgClone.find('.e-dlgfields:first').css('width', '92%').css('margin-left', '6%');
                                        $firstDiv.css('margin-top', '4%');
                                        var colName = $dlgClone.find('.e-dlgfields').find('.e-fieldset legend').text();
                                        var $labelDiv = ej.buildTag('div.e-responsiveLabelDiv', colName, { 'margin-left': '6%' });
                                        $labelDiv.insertAfter($firstDiv);
                                        var $fieldSet = $dlgClone.find('.e-dlgfields').find('.e-fieldset').find('table').css('width', '61%');
                                        $dlgClone.find('.e-dlgfields').find('.e-fieldset').replaceWith($fieldSet);
                                        var $fieldDiv = $fieldSet.parent('div').addClass('e-responsiveExcelFilterFieldDiv');
                                        var $ddl = $fieldSet.find('.e-dropdownlist')
                                        $fieldSet.find('.e-dropdownlist').each(function (index, object) {
                                            var ds = $(object).ejDropDownList('model.dataSource');
                                            var wid = $(window).width() * (40 / 100);
                                            $(object).ejDropDownList('destroy').ejDropDownList({ width: wid, popupWidth: wid + "px", dataSource: ds });
                                        });
                                        $fieldSet.find('.e-autocomplete').each(function (index, object) {
                                            var model = $(object).ejAutocomplete('model.dataSource');
                                            var wid = $(window).width() * (40 / 100);
                                            $(object).ejAutocomplete('destroy').ejAutocomplete({ width: wid, dataSource: model.dataSource, fields: model.fields });
                                        });
                                        $fieldSet.find('.e-datepicker').each(function (index, object) {
                                            var ds = $(object).ejDatePicker('model.dataSource');
                                            var wid = $(window).width() * (40 / 100);
                                            $(object).ejDatePicker('destroy').ejDatePicker({ width: wid });
                                        });
                                        var $okClone = $inputOk.clone(), $cancelClone = $inputCancel.clone();
                                        $okClone.ejButton({
                                            text: 'OK', type: 'button', click: function (sender) {
                                                gridObj._excelFilter._openedFltr = $dlgClone;
                                                gridObj._excelFilter._fltrBtnHandler();
                                                if ($dlgClone.hasClass('e-dlgcustom'))
                                                    $dlgClone.ejDialog('close');
                                                gridObj._setResponsiveFilterIcon();
                                                gridObj.element.css('display', 'block');
                                            }
                                        });
                                        $cancelClone.ejButton({ text: 'Cancel', type: 'button', click: function () { $dlgClone.ejDialog('close'); proxy.element.css('display', 'block') } });
                                        $dlgClone.append($dlgBtn.clone().append($okClone).append($cancelClone))
                                        var $btnContainer = $dlgClone.find('.e-dlgfields .e-btncontainer').remove();
                                        $btnContainer.find('input:first').addClass('e-resposnsiveFilterBtnLeft');
                                        $btnContainer.find('input:first').css('width', '45.6%')
                                        $btnContainer.find('input:last').addClass('e-resposnsiveFilterBtnRight');
                                    }
                                    gridObj.element.css('display', 'none');
                                    $dlgClone.ejDialog({ enableModal: false, height: height, width: width, position: { X: 0, Y: 0 }, enableResize: false, showHeader: false }).ejDialog('open');
                                })
                                var $searchdiv = ej.buildTag('div');
                            }
                        }
                    }
                    else {
                        $($id).ejDialog({ position: { X: "", Y: "" } });
						var docWidth = $(document).width(), dlgWidth = document.documentElement.clientWidth < 800 ? 200 : 250, xPos = $target.position().left + 18, yPos = $target.position().top + 2;
                        var filterDlgLargeCss = "e-filterdialoglarge";
                        dlgWidth = this._isExcelFilter ? this._excelFilter._dialogContainer.width() : dlgWidth;
                        if ($target.offset().left + 18 + dlgWidth > docWidth)
                            xPos = xPos - dlgWidth;
                        if (dlgWidth == 200)
                            filterDlgLargeCss = "";
                        if (!this._isExcelFilter)
                            $($id).ejDialog({ position: { X: xPos, Y: yPos }, width: dlgWidth, cssClass: filterDlgLargeCss })
                            .ejDialog("open");
                        else
                            this._excelFilter.openXFDialog({ field: columnName, displayName: this.getHeaderTextByFieldName(columnName), dataSource: this._dataSource(), position: { X: xPos, Y: yPos }, type: this._$colType, format: this._$colFormat, foreignKeyValue: this._$colForeignKeyValue, foreignDataSource: this._$colDropdownData });
                    }
                    this._setFilterFieldValues($id);
                    if (this._$colType == "number" && this.getColumnByField(columnName)["serverType"] != undefined )
                        $($id).find(".e-numerictextbox").ejNumericTextbox({ width: "100%",decimalPlaces: 0 });
                    else if(this._$colType == "number")
                        $($id).find(".e-numerictextbox").ejNumericTextbox({ width: "100%",decimalPlaces: 2 });
                    this._$prevColType = this._$colType;
                    this._$fDlgIsOpen = true;
                }
            }
        },
        _responsiveFilterClose: function () {
            this.element.css('display', 'block');
        },
        _clickHandler: function (e) {
            var $target = $(e.target), fieldName, $form = $("#" + this._id + "EditForm"), index, columnIndex, rowIndex;
            if ($target.hasClass("e-button") && ($target.hasClass("e-disable") || $target.prop("disabled"))) return;
            if ($target.closest(".e-grid").attr("id") !== this._id) return;
            if ($target.closest("#" + this._id + "EditForm").length)
                return;
            if ($target.hasClass("e-rowcell") || $target.closest("td").is(".e-rowcell") || ($target.hasClass("e-headercell") && ((e.clientY - $target.offset().top) < ($target.height() / 4)))) {
                if (this._bulkEditCellDetails.cancelSave) {
                    this._bulkEditCellDetails.cancelSave = false;
                    return;
                }
                if (this.model.editSettings.editMode == "batch" && ($.isFunction($.validator) && $form.length && $form.validate().errorList.length > 0))
                    return;
                this.model.editSettings.editMode == "batch" && this.element.focus();
                index = $target.closest("tr").hasClass("e-insertedrow") ? this.model.groupSettings.groupedColumns.length : 0;
                var tempIndex = $target.closest(".e-rowcell").index() != -1 ? $target.closest(".e-rowcell").index() : $target.closest(".e-headercell").index() - this.model.groupSettings.groupedColumns.length;
                columnIndex = $target.hasClass("e-rowcell") ? $target.index() - index : tempIndex - index;
                columnIndex = (this.model.detailsTemplate != null || this.model.childGrid != null) ? columnIndex - 1 : columnIndex;
                if (this.model.scrollSettings.frozenColumns && ($target.closest(".e-movableheaderdiv").length || $target.closest(".e-movablecontentdiv").length))
                    columnIndex = columnIndex + this.model.scrollSettings.frozenColumns;
                rowIndex = this.getIndexByRow($target.closest("tr"));
                this._bulkEditCellDetails.columnIndex = columnIndex;
                this._bulkEditCellDetails.rowIndex = rowIndex;
                if (this.model.allowSelection && ej.gridFeatures.selection) {
                    if (this.model.selectionType == "multiple") {
                        if (e.ctrlKey || this._enableSelectMultiTouch) {
                            this.multiSelectCtrlRequest = true;
                        }
                        if (e.shiftKey) {
                            this.multiSelectShiftRequest = true;
                            if (this._allowcellSelection && rowIndex > -1)
                                this.selectCells([[rowIndex, [columnIndex]]]);
                            if (this._allowrowSelection && rowIndex > -1)
                                this.selectRows(this._previousIndex, this.getIndexByRow($target.closest('tr')), $target);
                            if (this._allowcolumnSelection && $target.hasClass("e-headercell") && ((e.clientY - $target.offset().top) < ($target.height() / 4)))
                                this.selectColumns(this._previousColumnIndex, columnIndex);
                        }
                        if (e["pointerType"] == "touch" && this._customPop != null && !this._customPop.is(":visible") && this._customPop.find(".e-rowselect").hasClass("e-spanclicked") && this.model.selectionSettings.selectionMode == "row")
                            this._customPop.show();
                        if (e["pointerType"] == "touch" && this._customPop != null && (this._customPop.find(".e-sortdirect").is(":visible") || !this._customPop.find(".e-rowselect").hasClass("e-spanclicked")) && this.model.selectionType == "multiple") {
                            this._customPop.removeAttr("style");
                            var offset = $target.offset();
                            this._customPop.offset({ top: 0, left: 0 }).offset({ left: offset.left, top: offset.top - this.getRowHeight() }).find(".e-sortdirect").hide().end()
                                .find(".e-rowselect").show().end().show();
                        }
                    }
                    if (!this.multiSelectShiftRequest) {
                        if (this._allowcellSelection && rowIndex > -1) {
                            var cellProto = this._checkCellSelectionByRow(rowIndex, columnIndex);
                            if ((this.model.selectionSettings.enableToggle || (e.ctrlKey && this.model.selectionType == 'multiple')) && (cellProto != -1 && this.selectedRowCellIndexes.length > 0 && this.selectedRowCellIndexes[0].cellIndex.length > 0))
                                this.clearCellSelection(cellProto.rowIndex, columnIndex);
                            else
                                this.selectCells([[rowIndex, [columnIndex]]]);
                        }
                        if (this._allowrowSelection && rowIndex > -1) {
                            if (this.model.selectionSettings.enableToggle && this.getSelectedRecords().length == 1 && $.inArray(this.getIndexByRow($target.closest('tr')), this.selectedRowsIndexes) != -1)
                                this.clearSelection(this.getIndexByRow($target.closest('tr')));
                            else
                                this.selectRows(this.getIndexByRow($target.closest('tr')), null, $target);
                        }
                        if (this._allowcolumnSelection && $target.hasClass("e-headercell") && ((e.clientY - $target.offset().top) < ($target.height() / 4))) {
                            if (this.model.selectionSettings.enableToggle && this.selectedColumnIndexes.length == 1 && $.inArray(columnIndex, this.selectedColumnIndexes) != -1)
                                this.clearColumnSelection(columnIndex);
                            else
                                this.selectColumns(columnIndex);
                        }
                        this.multiSelectCtrlRequest = false;
                    }
                    this.multiSelectShiftRequest = false;
                }

                fieldName = this.model.columns[this._bulkEditCellDetails.columnIndex]["field"];
                if ($target.closest(".e-rowcell").length && fieldName)
                    this.model.editSettings.allowEditing && this.model.editSettings.editMode == ej.Grid.EditMode.Batch && this.editCell($.inArray($target.closest("tr").get(0), this.getRows()), fieldName);
            }
            if ($target.hasClass("e-rowselect") || $target.hasClass("e-sortdirect")) {
                if (!$target.hasClass("e-spanclicked")) {
                    $target.addClass("e-spanclicked");
                    if ($target.hasClass("e-rowselect"))
                        this._enableSelectMultiTouch = true;
                    if ($target.hasClass("e-sortdirect"))
                        this._enableSortMultiTouch = true;
                } else {
                    $target.removeClass("e-spanclicked");
                    if ($target.hasClass("e-rowselect"))
                        this._enableSelectMultiTouch = false;
                    if ($target.hasClass("e-sortdirect"))
                        this._enableSortMultiTouch = false;
                    this._customPop.hide();
                }
            }
            if (ej.gridFeatures.common) {
                this.expandCollapse($target);
            }
            if ($target.is(".e-filtericon") && $target.closest(".e-detailrow").length != 0)
                e.stopPropagation();
            if (this.model.allowFiltering && (this.model.filterSettings.filterType == "menu" || this._isExcelFilter) && !$target.is(".e-filtericon") && $target.closest(".e-dlgcontainer").length != 1)
                !this._isExcelFilter ? this._closeFilterDlg() : this._excelFilter.closeXFDialog(e);
        },
        _checkCellSelectionByRow: function (rowIndex, columnIndex) {
            for (var i = 0; i < this.selectedRowCellIndexes.length; i++) {
                if (this.selectedRowCellIndexes[i].rowIndex == rowIndex)
                    break;
            }
            if (i != this.selectedRowCellIndexes.length && $.inArray(columnIndex, this.selectedRowCellIndexes[i].cellIndex) != -1)
                return this.selectedRowCellIndexes[i];
            return -1;
        },
        
        _destroy: function () {
            /// <summary>This function is  used to destroy the Grid Object</summary>
            this.element.off();
            this.element.find(".e-gridheader").find(".e-headercontent,.e-movableheader")
                .add(this.getContent().find(".e-content,.e-movablecontent")).unbind('scroll');
            var editForm = $("#" + this._id + "EditForm");
            if (editForm.length) {
                var $formEle = editForm.find('.e-field'), $element;
                for (var i = 0; i < $formEle.length; i++) {
                    $element = $($formEle[i]);
                    if ($element.hasClass('e-datetimepicker'))
                        $element.ejDateTimePicker("destroy");
                    else if ($element.hasClass('e-datepicker'))
                        $element.ejDatePicker("destroy");
                    else if ($element.hasClass('e-dropdownlist'))
                        $element.ejDropDownList("destroy");
                }
                editForm.remove();
            }
            if (this._confirmDialog)
                this._confirmDialog.ejDialog("destroy");
            this.element.find('.e-dropdownlist').ejDropDownList('model.dataSource', []);
            if (this.model.showColumnChooser) {
                $("#" + this._id + "ccDiv").ejDialog("destroy");
                $("#" + this._id + "ccDiv").remove();
                $("#" + this._id + "_ccTail").remove();
                $("#" + this._id + "_ccTailAlt").remove();
            }
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "excel")
                this._excelFilter.resetExcelFilter()
            if (this.model.allowReordering)
                $(".e-columndropindicator").remove();
            if (this.model.allowFiltering && this.model.filterSettings.filterType == "menu") {
                var proxy = this, $colType;
                $.each(this.model.columns, function (indx, col) {
                    $colType = col.type;
                    $("#" + proxy._id + $colType + "_ddinput_popup_wrapper").remove();
                    if ($colType == "string")
                        $("#" + proxy._id + "_stringDlg").find('.e-autocomplete').ejAutocomplete("destroy");
                    else if ($colType == "date")
                        $("#" + proxy._id + "_dateDlg").find('.e-datepicker').ejDatePicker("destroy");
                    else if ($colType == "datetime")
                        $("#" + proxy._id + "_datetimeDlg").find('.e-datetimepicker').ejDateTimePicker("destroy");
                    else if ($colType == "number")
                        $("#" + proxy._id + "_numberDlg").find('.e-numerictextbox').ejNumericTextbox("destroy");
                });
            }
            if (this._$onresize)
                $(window).unbind("resize", this._$onresize);
            this.element.empty().removeClass("e-grid " + this.model.cssClass);
            this.element.ejWaitingPopup("destroy");
            if (this.model.contextMenuSettings.enableContextMenu)
                $("#" + this._id + "_Context").ejMenu('destroy');
        },
        _getLocalizedLabels: function (property) {
            return (ej.Grid.locale[this.model.locale] === undefined || ej.Grid.locale[this.model.locale][property] === undefined) ? ej.Grid.locale["en-US"][property] : ej.Grid.locale[this.model.locale][property];
        },
    });
    if (ej.gridFeatures.common)
        $.extend(ej.Grid.prototype, ej.gridFeatures.common);
    if (ej.gridFeatures.edit)
        $.extend(ej.Grid.prototype, ej.gridFeatures.edit);
    if (ej.gridFeatures.filter)
        $.extend(ej.Grid.prototype, ej.gridFeatures.filter);
    if (ej.gridFeatures.group)
        $.extend(ej.Grid.prototype, ej.gridFeatures.group);
    if (ej.gridFeatures.selection)
        $.extend(ej.Grid.prototype, ej.gridFeatures.selection);
    if (ej.gridFeatures.sort)
        $.extend(ej.Grid.prototype, ej.gridFeatures.sort);
    if (ej.gridFeatures.dragAndDrop)
        $.extend(ej.Grid.prototype, ej.gridFeatures.dragAndDrop);

    ej.Grid.locale = {};

    ej.Grid.locale["en-US"] = {
        EmptyRecord: "No records to display",
        GroupDropArea: "Drag a column header here to group its column",
        DeleteOperationAlert: "No records selected for delete operation",
        EditOperationAlert: "No records selected for edit operation",
        SaveButton: "Save",
        OkButton: "OK",
        CancelButton: "Cancel",
        EditFormTitle: "Details of ",
        AddFormTitle: "Add New Record",
        Notactionkeyalert: "This Key-Combination is not available",
        Keyconfigalerttext: "This Key-Combination has already been assigned to ",
        GroupCaptionFormat: "{{:headerText}}: {{:key}} - {{:count}} {{if count == 1 }} item {{else}} items {{/if}} ",
        BatchSaveConfirm: "Are you sure you want to save changes?",
        BatchSaveLostChanges: "Unsaved changes will be lost. Are you sure you want to continue?",
        ConfirmDelete: "Are you sure you want to Delete Record?",
        CancelEdit: "Are you sure you want to Cancel the changes?",
        PagerInfo: "{0} of {1} pages ({2} items)",
        FrozenColumnsViewAlert: "Frozen columns should be in grid view area",
        FrozenColumnsScrollAlert: "Enable allowScrolling while using frozen Columns",
        FrozenNotSupportedException: "Frozen Columns and Rows are not supported for Grouping, Row Template, Detail Template, Hierarchy Grid and Batch Editing",
        Add: "Add",
        Edit: "Edit",
        Delete: "Delete",
        Update: "Update",
        Cancel: "Cancel",
        Done: "Done",
        Columns: "Columns",
        PrintGrid: "Print",
        ExcelExport: "Excel Export",
        WordExport: "Word Export",
        PdfExport: "PDF Export",
        StringMenuOptions: [{ text: "StartsWith", value: "StartsWith" }, { text: "EndsWith", value: "EndsWith" }, { text: "Contains", value: "Contains" }, { text: "Equal", value: "Equal" }, { text: "NotEqual", value: "NotEqual" }],
        NumberMenuOptions: [{ text: "LessThan", value: "LessThan" }, { text: "GreaterThan", value: "GreaterThan" }, { text: "LessThanOrEqual", value: "LessThanOrEqual" }, { text: "GreaterThanOrEqual", value: "GreaterThanOrEqual" }, { text: "Equal", value: "Equal" }, { text: "NotEqual", value: "NotEqual" }],
        PredicateAnd: "AND",
        PredicateOr: "OR",
        Filter: "Filter",
        FilterMenuCaption: "Filter Value",
        FilterbarTitle: "'s filterbar cell",
        MatchCase: "Match Case",
        Clear: "Clear",
        ResponsiveFilter: "Filter",
        ResponsiveSorting: "Sort",
        Search: "Search",
        DatePickerWaterMark: "Select date",
        EmptyDataSource: "DataSource must not be empty at initial load since columns are generated from dataSource in AutoGenerate Column Grid",
        True: "true",
        False: "false",
        UnGroup: "Click here to ungroup",
        AddRecord: "Add Record",
        EditRecord: "Edit Record",
        DeleteRecord: "Delete Record",
        Save: "Save",
        Grouping: "Grouping",
        Ungrouping: "Ungrouping",
        SortInAscendingOrder: "Sort In Ascending Order",
        SortInDescendingOrder: "Sort In Descending Order",
        NextPage: "Next Page",
        PreviousPage: "Previous Page",
        FirstPage: "First Page",
        LastPage: "Last Page"
    };
    ej.Grid.Actions = {
        /** Used to specify paging action in grid   */
        Paging: "paging",
        /** Used to specify sorting action in grid   */
        Sorting: "sorting",
        /** Used to specify filtering action in grid   */
        Filtering: "filtering",
        /** Used to specify begin edit action in grid   */
        BeginEdit: "beginedit",
        /** Used to specify saving action in grid   */
        Save: "save",
        /** Used to specify adding action in grid   */
        Add: "add",
        /** Used to specify deleting action in grid   */
        Delete: "delete",
        /** Used to specify cancelling action in grid   */
        Cancel: "cancel",
        /** Used to specify grouping action in grid   */
        Grouping: "grouping",
        /** Used to specify un-grouping action in grid   */
        Ungrouping: "ungrouping",
        /** Used to specify refresh action in grid   */
        Refresh: "refresh",
        /** Used to specify reordering action in grid   */
        Reorder: "reorder",
        /** Used to specify searching action in grid   */
        Search: "searching",
        /** Used to specify batch save action in grid   */
        BatchSave: "batchsave",
        /** Used to specify virtual scroll action in grid   */
        VirtualScroll: "virtualscroll"
    };

    ej.Grid.SummaryType = {
        /**  Creates grid with summary type as Average */
        Average: "average",
        /**  Creates grid with summary type as Minimum */
        Minimum: "minimum",
        /**  Creates grid with summary type as Maximum */
        Maximum: "maximum",
        /**  Creates grid with summary type as Count */
        Count: "count",
        /**  Creates grid with summary type as Sum */
        Sum: "sum",
        /**  Creates grid with summary type as TrueCount */
        TrueCount: "truecount",
        /**  Creates grid with summary type as FalseCount */
        FalseCount: "falsecount",
        /**  Creates grid with summary type as Custom */
        Custom: "custom"
    };

    ej.Grid.EditMode = {
        /**  Creates grid with editMode as Normal */
        Normal: "normal",
        /**  Creates grid with editMode as Dialog */
        Dialog: "dialog",
        /**  Creates grid with editMode as DialogTemplate */
        DialogTemplate: "dialogtemplate",
        /**  Creates grid with editMode as Batch */
        Batch: "batch",
        /**  Creates grid with editMode as ExternalForm */
        ExternalForm: "externalform",
        /**  Creates grid with editMode as ExternalFormTemplate */
        ExternalFormTemplate: "externalformtemplate",
        /**  Creates grid with editMode as InlineForm */
        InlineForm: "inlineform",
        /**  Creates grid with editMode as InlineTemplateForm */
        InlineTemplateForm: "inlineformtemplate"
    };

    ej.Grid.PrintMode = {
        /**  print all pages in grid */
        AllPages: "allpages",
        /**   print current pages in grid */
        CurrentPage: "currentpage",
    };

    ej.Grid.Rowposition = {
        /** Add new row in the top of the grid */
        Top: "top",
        /** Add new row in the bottom of the grid */
        Bottom: "bottom",
    };

    ej.Grid.FormPosition = {
        /**  Creates grid with formPosition as BottomLeft */
        BottomLeft: "bottomleft",
        /**  Creates grid with formPosition as TopRight */
        TopRight: "topright"
    };

    ej.Grid.EditingType = {
        /**  Allows to set edit type as string edit type */
        String: "stringedit",
        /**  Allows to set edit type as boolean edit type */
        Boolean: "booleanedit",
        /**  Allows to set edit type as numeric edit type */
        Numeric: "numericedit",
        /**  Allows to set edit type as drop down edit type */
        Dropdown: "dropdownedit",
        /**  Allows to set edit type as date picker edit type */
        DatePicker: "datepicker",
        /**  Allows to set edit type as date time picker edit type */
        DateTimePicker: "datetimepicker",
    };

    ej.Grid.UnboundType = {
        /** Used to specify unbound type as Edit   */
        Edit: "edit",
        /** Used to specify unbound type as Save   */
        Save: "save",
        /** Used to specify unbound type as Delete   */
        Delete: "delete",
        /** Used to specify unbound type as Cancel   */
        Cancel: "cancel"
    };

    ej.Grid.ToolBarItems = {
        /** Used to add toolbar item for adding records    */
        Add: "add",
        /** Used to add toolbar item for editing records    */
        Edit: "edit",
        /** Used to add toolbar item for deleting records    */
        Delete: "delete",
        /** Used to add toolbar item for updating records    */
        Update: "update",
        /** Used to add toolbar item for cancelling records    */
        Cancel: "cancel",
        /** Used to add toolbar item for searching records    */
        Search: "search",
        /** Used to add toolbar item for printing grid    */
        PrintGrid: "printGrid",
        /** Used to add toolbar item for exproting grid to excel    */
        ExcelExport: "excelExport",
        /** Used to add toolbar item for exporting grid to word    */
        WordExport: "wordExport",
        /** Used to add toolbar item for exporting grid to pdf    */
        PdfExport: "pdfExport"
    };

    ej.Grid.FilterType = {
        /**  Creates grid with filtering type as Menu */
        Menu: "menu",
        /**  Creates grid with filtering type as FilterBar */
        FilterBar: "filterbar",
        /** Creates grid with filtering type as Excel */
        Excel: "excel"
    };

    ej.Grid.FilterBarMode = {
        /** Used to set filter bar mode as Immediate mode */
        Immediate: "immediate",
        /** Used to set filter bar mode as OnEnter mode */
        OnEnter: "onenter"
    };

    ej.Grid.SelectionType = {
        /**  Support for Single selection only in grid */
        Single: "single",
        /**  Support for multiple selections in grid */
        Multiple: "multiple"
    };

    ej.Grid.GridLines = {
        /**  Support for Show both the vertical and horizontal line in grid  */
        Both: "both",
        /**  Support for Hide both the vertical and horizontal line in grid  */
        None: "none",
        /**  Support for Shows the horizontal line only in grid */
        Horizontal: "horizontal",
        /**  Support for Shows the vertical line only in grid  */
        Vertical: "vertical",
    };

    ej.Grid.VirtualScrollMode = {
        /** Used to set the Normal mode virtual paging*/
        Normal: "normal",
        /** Used to set the Continuous mode virtual paging*/
        Continuous: "continuous"
    };

    ej.Grid.SelectionMode = {
        /**  Support for Row selection in grid */
        Row: "row",
        /**  Support for Cell selection in grid */
        Cell: "cell",
        /**  Support for Column selection in grid */
        Column: "column"
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.gridFeatures = ej.gridFeatures || {};
    ej.gridFeatures.gridResize = function (instance) {
        this.$headerTable = instance.getHeaderTable();
        this.gridInstance = instance;
        this._colMinWidth = 15;
        this._$visualElement = $();
        this._currentCell = -1;
        this._allowStart = false;
        this._oldWidth = null;
        this._orgX = null;
        this._orgY = null;
        this._extra = null;
        this._expand = false;
        this._target = null;
    }

    ej.gridFeatures.gridResize.prototype = {
        _mouseHover: function (e) {
            if (this._$visualElement.is(":visible"))
                return;
            this._allowStart = false;
            if ($(e.target).is(".e-headercelldiv"))
                e.target = e.target.parentNode;
            var $target = $(e.target);
			if ($(e.target).hasClass("e-filtericon") && ($(e.target).css("cursor") == "col-resize" || $(e.target).closest("tr").css("cursor") == "col-resize")) {
                $(e.target).css("cursor", "pointer");
                $(e.target).closest("tr").css("cursor", "pointer");
            }
            if ($target.hasClass("e-headercell")) {
                var _resizableCell = e.target;
                var location = _resizableCell.getBoundingClientRect(), _x = 0, _y = 0;
                if (e.type = "mousemove") {
                    _x = e.clientX;
                    _y = e.clientY;
                }
                else if (e.type = "touchmove") {
                    _x = evt.originalEvent.changedTouches[0].clientX;
                    _y = evt.originalEvent.changedTouches[0].clientY;
                }
                else if (e.type = "MSPointerMove") {
                    _x = e.originalEvent.clientX;
                    _y = e.originalEvent.clientY;
                }
                if (this.gridInstance.model.scrollSettings && this.gridInstance.model.scrollSettings.frozenColumns)
                    var _nlx = this.gridInstance.getHeaderContent().width() + this.gridInstance.element.children(".e-gridheader").find(".e-columnheader").offset().left;
                else
                    var _nlx = this.gridInstance.getHeaderTable().width() + this.gridInstance.element.children(".e-gridheader").find(".e-columnheader").offset().left;
                if (((_x >= (location.left + document.documentElement.scrollLeft + _resizableCell.offsetWidth - 5)) || ((_x <= (location.left + 3)))) && (_x < _nlx) && (_x >= location.left) && (_y <= location.top + document.documentElement.scrollTop + e.target.offsetHeight)) {
                    if (_x > location.left + 3)
                        var tempTarget = $(e.target).find(".e-headercelldiv");
                    else
                        var tempTarget = $(e.target).prevAll("th:visible:first").find(".e-headercelldiv");
                    this.gridInstance.model.showStackedHeader && $.inArray($(tempTarget).attr("ej-mappingname"), this.gridInstance._disabledResizingColumns) == -1 && $($target.parents('thead')).find('tr').css("cursor", "col-resize");
                    !this.gridInstance.model.showStackedHeader && $.inArray($(tempTarget).attr("ej-mappingname"), this.gridInstance._disabledResizingColumns) == -1 && $target.parent().css("cursor", "col-resize");
                    if ($(e.target).hasClass('e-stackedHeaderCell'))
                        this._currentCell = this.gridInstance.getHeaderContent().find(".e-headercell:visible").index(_resizableCell);
                    else
                        this._currentCell = this.gridInstance.getHeaderContent().find(".e-headercell:visible").not(".e-stackedHeaderCell").index(_resizableCell);
                    this._allowStart = true;
                }
                else {
                    $target.parent().css("cursor", "pointer");
                    this._currentCell = -1;
                }
            }
        },
        _start: function (_x, _y) {
            var _myrow = this.gridInstance.getHeaderTable().find(".e-columnheader"), _top;
            var _cells, _mycel;
            if ($(this._target).hasClass('e-stackedHeaderCell'))
                _cells = _myrow.find(".e-headercell").not(".e-hide");
            else
                _cells = _myrow.find(".e-headercell").not(".e-stackedHeaderCell,.e-hide");
            if (this._currentCell != -1 && this._currentCell < _cells.length)
                _mycel = _cells[this._currentCell];
            if (typeof (_mycel) == 'undefined')
                return;
            var _j = _mycel.getBoundingClientRect();
            _top = this._tableY = _j.top + parseInt(navigator.userAgent.indexOf("WebKit") != -1 ? document.body.scrollTop : document.documentElement.scrollTop);
            if (this._allowStart) {
                var vElement = this._$visualElement = $(document.createElement('div'));
                _height = this.gridInstance.element.find(".e-gridcontent").first().height() + this.gridInstance.element.find(".e-gridheader").height();
                if (this.gridInstance.model.showStackedHeader && this.gridInstance.model.stackedHeaderRows.length > 0) {
                    var headerRow = this.gridInstance.getHeaderTable().find('tr.e-columnheader')
                    var lenght = headerRow.length;
                    var currentIndex = $(this._target).parent('tr')[0].rowIndex;
                    for (var i = 0; i < currentIndex; i++) {
                        _height = _height - $(headerRow[i]).height();
                    }
                    // _height = _height - $(".e-stackedHeaderRow").height();
                }
                vElement.addClass("e-reSizeColbg").appendTo(this.gridInstance.element).attr("unselectable", "on").css("visibility", "hidden");
                this.gridInstance._resizeTimeOut = setTimeout(function() {
                    vElement.css({ visibility: "visible", height: _height + 'px', cursor: 'col-resize', left: _x, top: _top, position: 'fixed' });
                }, 100);
                this._oldWidth = _mycel.offsetWidth;
                this._orgX = _x;
                this._orgY = _y;
                this._extra = _x - this._orgX;
                this._expand = true;
            }
            else {
                this._currentCell = -1;
            }
        },
        _mouseMove: function (e) {
            if (this._expand) {
                var _x = 0, _y = 0;
                if (e.type = "mousemove") {
                    _x = e.clientX;
                    _y = e.clientY;
                }
                else if (e.type = "touchmove") {
                    _x = evt.originalEvent.changedTouches[0].clientX;
                    _y = evt.originalEvent.changedTouches[0].clientY;
                }
                else if (e.type = "MSPointerMove") {
                    _x = e.originalEvent.clientX;
                    _y = e.originalEvent.clientY;
                }
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                _x += document.documentElement.scrollLeft;
                e.preventDefault();
                this._moveVisual(_x);
            }
            else
                this._mouseHover(e);
        },
        _getCellIndex: function (e) {
            var $target = $(e._target);
            var targetCell = e._target;
            var hCellIndex = targetCell.cellIndex;
            var cellIndex = hCellIndex;
            if (e.gridInstance.model.groupSettings.groupedColumns.length) {
                cellIndex = hCellIndex - e.gridInstance.model.groupSettings.groupedColumns.length;
            }
            return cellIndex;
        },
        _reSize: function (_x, _y) {
            // Function used for Resizing the column
            var proxy = this;
            var resized = false, $content;
            if (this.gridInstance.model.scrollSettings && this.gridInstance.model.scrollSettings.frozenColumns)
                this._initialTableWidth = this.gridInstance.getHeaderTable().first().parent().width() + this.gridInstance.getHeaderTable().last().parent().width();
            else
                this._initialTableWidth = this.gridInstance.getHeaderTable().parent().width();
            !this.gridInstance.model.enableRTL && this._getResizableCell();
            if (this.gridInstance.model.scrollSettings && this.gridInstance.model.scrollSettings.frozenColumns > 0)
                var _rowobj = this.gridInstance.getHeaderTable().find('thead');
            else
                var _rowobj = $(this._target).parents('thead');
            if (this._currentCell != -1 && this._expand) {
                this._expand = false;
                var _childTH = _rowobj.find(".e-headercell:not(.e-detailheadercell)").filter(":visible");
                var _outerCell = _childTH[this._currentCell];
                var _oldWidth = _outerCell.offsetWidth;
                var _extra = _x - this._orgX;
                //Check whether the column minimum width reached
                if (parseInt(_extra) + parseInt(_oldWidth) > this._colMinWidth) {
                    if (_extra != 0)
                        _rowobj.css("cursor", 'default');
                    this._resizeColumnUsingDiff(_oldWidth, _extra);
                    $content = this.gridInstance.element.find(".e-gridcontent").first();
                    var scrollContent = $content.find("div").hasClass("e-content");
                    this.gridInstance.model.allowScrolling && $content.ejScroller("refresh");                    
                    var browser = this.gridInstance.getBrowserDetails();
                    if (browser.browser == "msie" && this.gridInstance.model.allowScrolling) {
                        var oldWidth = this.gridInstance.getContentTable().width(), newwidth = this.gridInstance._calculateWidth();
                        if (this.gridInstance.model.scrollSettings && this.gridInstance.model.scrollSettings.frozenColumns > 0) {
                            this.gridInstance.getHeaderTable().last().width(newwidth - this.gridInstance.getHeaderContent().find(".e-frozenheaderdiv").width());
                            this.gridInstance.getContentTable().last().width(newwidth - this.gridInstance.getContent().find(".e-frozencontentdiv").width());
                            this.gridInstance.model.showSummary && this.gridInstance.getFooterTable().last().width(newwidth - this.gridInstance.getFooterContent().find(".e-frozenfootertdiv").width());
                        }
                        else {
                            if (newwidth > oldWidth) {
                                this.gridInstance.getHeaderTable().width(newwidth);
                                this.gridInstance.getContentTable().width(newwidth);
                                this.gridInstance.model.showSummary && this.gridInstance.getFooterTable().width(newwidth);
                            }
                        }
                        if (parseInt(browser.version, 10) > 8 && this.gridInstance.model.groupSettings && this.gridInstance.model.groupSettings.groupedColumns.length) {
                            if (newwidth > oldWidth) {
                                this.gridInstance.getContentTable().width(newwidth);
                                this.gridInstance.getContentTable().children("colgroup").find("col").first().css("width", (20 / $content.find("table").first().width()) * 100 + "%");
                            }
                            else {
                                this.gridInstance.getContentTable().css("width", "100%");
                                this.gridInstance._groupingAction(true);
                                this.gridInstance.getContentTable().children("colgroup").find("col").first().css("width", ((this.gridInstance.getHeaderTable().find("colgroup").find("col").first().width() / $content.find("table").first().width()) * 100).toFixed(2) + "%");
                            }
                        }
                        this.gridInstance.getHeaderTable().parent().scrollLeft($content.find(".e-content").scrollLeft() - 1);
                    }
                    this.gridInstance._colgroupRefresh();
                    if (this.gridInstance.model.groupSettings.groupedColumns.length)
                        this.gridInstance._recalculateIndentWidth();
                    if (!(browser.browser == "msie") && this.gridInstance.model.allowScrolling && this.gridInstance.model.scrollSettings.frozenColumns == 0) {
                        this.gridInstance.getHeaderTable().width("100%");
                        this.gridInstance.getContentTable().width("100%");
                        var tableWidth = this.gridInstance._calculateWidth();
                        if (tableWidth <= this.gridInstance.getContentTable().width() || this.gridInstance.getHeaderTable().width() > this.gridInstance.getContentTable().width()) {
                            this.gridInstance.getHeaderTable().width(tableWidth);
                            this.gridInstance.getContentTable().width(tableWidth);
                        }
                    }
                    if (this.gridInstance.model.allowResizing && this.gridInstance.getHeaderTable().find(".e-columnheader").css('cursor') == 'default') {
                        var cellIndex = this._getCellIndex(this);
                        var newWidth = _oldWidth + _extra;
                        var args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], oldWidth: _oldWidth, newWidth: newWidth };
                        this.gridInstance._trigger("resized", args);
                    }
                    if (this.gridInstance.model.allowScrolling) {
                        $content.ejScroller("refresh");
                        if (!scrollContent && $content.find("div").hasClass("e-content"))
                            this.gridInstance.refreshScrollerEvent();
                        this.gridInstance._isHscrollcss();
                    }
                }

            }

            this._target = null;
            this._$visualElement.remove();
            this._expand = false;
            this._currentCell = -1;
            this._allowStart = false;

        },
        _getFrozenResizeWidth: function () {
            var $frozenColumnsCol = this.gridInstance.getHeaderTable().find('colgroup').find("col").slice(0, this.gridInstance.model.scrollSettings ? this.gridInstance.model.scrollSettings.frozenColumn : 0), width = 0;
            for (var i = 0; i < $frozenColumnsCol.length; i++) {
                if ($frozenColumnsCol.eq(i).css("display") != "none")
                    width += parseInt($frozenColumnsCol[i].style.width.replace("px", ""));
            }
            return width;
        },
        _diaplayFinder: function () {
            return $(this).css('display') != 'none';
        },
        _resizeColumnUsingDiff: function (_oldWidth, _extra) {
            var proxy = this, _extraVal;
            this._currntCe = this._currentCell;
            if (this.gridInstance.model.showStackedHeader && !$(this._target).parent('tr').hasClass('e-stackedHeaderRow'))
                this._currentCell = this._currentCell - this.gridInstance.getHeaderTable().find(".e-stackedHeaderCell").length;
            var $headerCols = this.gridInstance.getHeaderTable().find('colgroup').find("col");
            var $ContentCols = this.gridInstance.getContentTable().find('colgroup').find("col");
            if (!ej.isNullOrUndefined(this.gridInstance.model.detailsTemplate)) {
                this.gridInstance._detailColsRefresh();
                $headerCols = this.gridInstance._$headerCols;
                $ContentCols = this.gridInstance._$contentCols;
            }
            var $headerCol = $headerCols.filter(this._diaplayFinder).eq(!this.gridInstance.model.allowGrouping || !ej.isNullOrUndefined(this.gridInstance.model.detailsTemplate) ? this._currentCell : this._currentCell + this.gridInstance.model.groupSettings.groupedColumns.length)
                    , $ContentCol, $footerCol, $frozenCols = $headerCols.slice(0, this.gridInstance.model.scrollSettings ? this.gridInstance.model.scrollSettings.frozenColumns : 0);
            var colWidth = $headerCol[0].style.width, isPercent = colWidth.indexOf("%") != -1;
            var _inlineWidth = (!colWidth || isPercent)? $(this._target).outerWidth() : colWidth;
            var indent = !isPercent ? _oldWidth / parseInt(_inlineWidth) : 1;
            _extraVal = _extra = _extra / indent
            var _newWidth = parseInt(_extra) + parseInt(_inlineWidth);
            if (_newWidth > 0 && _extra != 0) {
                if (_newWidth < this._colMinWidth)
                    _newWidth = this._colMinWidth;
                var _extra = _newWidth - _oldWidth;
                if (this.gridInstance.model.scrollSettings && this.gridInstance.model.scrollSettings.frozenColumns) {
                    if (this._currentCell >= 0 && this._currentCell < this.gridInstance.model.scrollSettings.frozenColumns && this._getFrozenResizeWidth() + _extra > this.gridInstance.element.find(".e-headercontent").first().width())
                        return;
                    $ContentCol = $ContentCols.filter(this._diaplayFinder).eq(this._currentCell);
                }
                else
                    $ContentCol = $ContentCols.filter(this._diaplayFinder).eq(!this.gridInstance.model.allowGrouping || !ej.isNullOrUndefined(this.gridInstance.model.detailsTemplate) ? this._currentCell : this._currentCell + this.gridInstance.model.groupSettings.groupedColumns.length);
                if (this.gridInstance.model.showSummary) {
                    $footerCol = this.gridInstance.getFooterTable().find('colgroup').find("col").filter(this._diaplayFinder).eq(this._currentCell);
                    $footerCol.outerWidth(_newWidth);
                }
                if ($(this._target).parent('tr').hasClass('e-stackedHeaderRow')) {
                    this._resizeStackedHeaderColumn($(this._target).parent('tr'), _extraVal, this._currntCe);
                }
                else
                    $headerCol.outerWidth(_newWidth);
                if ($(this._target).parent('tr').hasClass('e-stackedHeaderRow')) {
                    if (this.gridInstance.model.groupSettings.groupedColumns.length) {
                        var $tables = this.gridInstance.getContentTable().find(".e-recordtable");
                        var $colGroup = $tables.find("colgroup");
                        for (var i = 0; i < this._changedcell.length; i++) {
                            var cellIndex = this._changedcell[i];
                            for (var j = 0 ; j < $colGroup.length; j++) {
                                var visibleCols = $($colGroup[j]).children().filter(this._diaplayFinder);
                                var width = parseInt((_extraVal)) + parseInt(visibleCols[cellIndex].style.width);
                                if (width < this._colMinWidth)
                                    width = this._colMinWidth
                                $(visibleCols[cellIndex]).width(width);
                            }
                        }
                    }
                    var length = this.gridInstance.getContentTable().find('colgroup').find("col").filter(this._diaplayFinder).length;
                    for (var i = 0; i < this._changedcell.length; i++) {
                        var $conCol = this.gridInstance.getContentTable().find('colgroup').find("col").filter(this._diaplayFinder)[this._changedcell[i]]
                        var width = parseInt((_extraVal)) + parseInt($conCol.style.width);
                        if (width < this._colMinWidth)
                            width = this._colMinWidth
                        $($conCol).outerWidth(width);
                        if (this.gridInstance.model.isEdit && (this.gridInstance.model.allowGrouping && this.gridInstance.model.groupSettings.groupedColumns.length == 0)) {
                            $sEditCol = this.gridInstance.getContentTable().find(".gridform").find("colgroup col").filter(this._diaplayFinder)[this._changedcell[i]];
                            $($sEditCol).outerWidth(width);
                        }
                    }
                }
                else {
                    if (this.gridInstance.model.groupSettings && this.gridInstance.model.groupSettings.groupedColumns.length) {
                        var $tables = this.gridInstance.getContentTable().find(".e-recordtable");
                        var $colGroup = $tables.find("colgroup");
                        var cellIndex = this._currentCell;
                        var colCount = this.gridInstance.model.columns.length;
                        for (var i = 0 ; i < $colGroup.length; i++) {
                            var cols = $($colGroup[i]).find("col").filter(this._diaplayFinder);
                            if (cols.length > colCount) cols.splice(0, (cols.length - colCount));
                            $(cols[cellIndex]).width(_newWidth);
                        }
                    }
                    $ContentCol.outerWidth(_newWidth);
                    if (this.gridInstance.model.isEdit) {
                        var $editableRow = this.gridInstance.getContentTable().find(".e-editedrow,.e-addedrow");
                        var $editCols = $editableRow.find("table").find("colgroup col");
                        var addCol;
                        if ($editableRow.hasClass("e-addedrow") && this.gridInstance.model.groupSettings.groupedColumns.length)
                            addCol = this._currentCell + this.gridInstance.model.groupSettings.groupedColumns.length - 1;
                        else
                            addCol = this._currentCell;
                        var $editCol = $editCols.filter(this._diaplayFinder).eq(addCol);
                        $editCol.outerWidth(_newWidth);
                    }
                }
                this.gridInstance._findColumnsWidth();
                if (this.gridInstance.model.scrollSettings && this.gridInstance.model.scrollSettings.frozenColumns) {
                    var frozenColumns = this.gridInstance.getContentTable().find('colgroup').find("col").slice(0, this.gridInstance.model.scrollSettings.frozenColumns)
                        , width = 0, direction;
                    for (i = 0; i < frozenColumns.length; i++)
                        width += parseInt(frozenColumns[i].style.width.replace("px", ""));
                    this.gridInstance.getHeaderContent().find(".e-frozenheaderdiv").width(width);
                    direction = this.gridInstance.model.enableRTL ? "margin-right" : "margin-left";
                    this.gridInstance.getContent().find(".e-frozencontentdiv").width(width).next().css(direction, width + "px");
                    this.gridInstance.getHeaderContent().find(".e-frozenheaderdiv").width(width).next().css(direction, width + "px");
                    this.gridInstance.model.showSummary && this.gridInstance.getFooterContent().find(".e-frozenfooterdiv").width(width);
                }
                this.gridInstance.getHeaderTable().find(".e-columnheader").css("cursor", "default");
            }
        },
        _resizeStackedHeaderColumn: function (currentTr, extra, currentCell) {
            // var currentIndex = this._currntCe;
            this._changedcell = [];
            var headerCells = this.gridInstance.getHeaderContent().find(".e-headercell");
            var preCol = 0, limit = 0, currentTh = headerCells[currentCell], currentSpan = $(currentTh).attr('colspan'), commonExtra = extra / currentSpan, tr = $(currentTh).parent('tr');
            var nextTr = tr.next();
            var currentIndex = currentTh.cellIndex;
            if (this.gridInstance.model.groupSettings.showGroupedColumn) {
                limit = this.gridInstance.model.groupSettings.groupedColumns.length;
                preCol += limit
            }
            while (currentIndex > limit) {
                currentIndex--;
                var th = $(tr).children('th')[currentIndex];
                preCol += parseInt($(th).attr('colspan'));
            }
            this._currentCell = preCol;
            var length = preCol + parseInt(currentSpan);
            for (var i = preCol; i < length; i++) {
                var $colG = this.gridInstance.getHeaderTable().find('col').filter(this._diaplayFinder)[i];
                this._changedcell.push(i - limit)
                var width = parseInt(extra) + parseInt($colG.style.width);
                if (width < this._colMinWidth)
                    width = this._colMinWidth;
                $($colG).outerWidth(width);
             }
        },
        _triggerResizeEvents: function (event, _x) {
            var _rowobj = this.gridInstance.getHeaderTable().find(".e-columnheader");
            var _childTH = _rowobj.find(".e-headercell").filter(":visible");
            var _outerCell = _childTH[this._currentCell];
            var _oldWidth = _outerCell.offsetWidth;
            var cellIndex = this._getCellIndex(this);
            if (event == "resizeStart") {
                var args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], target: $(_outerCell), oldWidth: _oldWidth };
                return this.gridInstance._trigger("resizeStart", args);
            }
            else {
                var _childth = _rowobj.find(".e-headercell").not(".e-detailheadercell").filter(":visible");
                !this.gridInstance.model.enableRTL && this._getResizableCell();
                cellIndex = this._currentCell;
                if (!$(_outerCell).hasClass("e-detailheadercell") && this._currentCell != -1)
                    var _oldWidth = _childth[this._currentCell].offsetWidth;
                var _extra = _x - this._orgX;
                var newWidth = _oldWidth + _extra;
                this.gridInstance._colgroupRefresh();
                var args = { columnIndex: cellIndex, column: this.gridInstance.model.columns[cellIndex], target: $(_outerCell), oldWidth: _oldWidth, newWidth: newWidth, extra: _extra };
                return this.gridInstance._trigger("resizeEnd", args);
            }
        },
        _mouseUp: function (e) {
            if (this.gridInstance._resizeTimeOut){
                clearTimeout(this.gridInstance._resizeTimeOut);
                this.gridInstance._resizeTimeOut = 0;
            }
            if (this._expand) {
                var _x = e.clientX, _y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY;
                }
                if (this.gridInstance.model.allowResizing && this.gridInstance.getHeaderTable().find(".e-columnheader").css('cursor') == 'col-resize') {
                    if (this._triggerResizeEvents("resizeEnd", _x)) {
                        this.gridInstance.element.find(".e-reSizeColbg").remove();
                        return;
                    }
                }
                _x += document.documentElement.scrollLeft;
                this._reSize(_x, _y);
                var index = this.gridInstance.model.showStackedHeader ? this._currntCe - this.gridInstance.getHeaderTable().find(".e-stackedHeaderCell").length : this._currntCe;
                if (index >= 0)
                if (!ej.isNullOrUndefined(index))
                    this.gridInstance.model.columns[index].width = this.gridInstance._columnsWidthCollection[index];
            }
            setTimeout(function(){
                $(document).off("selectstart.ejgrid");
            }, 10);
        },
        _getResizableCell: function () {
            var row = this.gridInstance.getHeaderTable().find(".e-columnheader");
            var cell = row.find(".e-headercell").not(".e-hide,.e-detailheadercell");
            var scrollLeft = navigator.userAgent.indexOf("WebKit") != -1 ? document.body.scrollLeft : document.documentElement.scrollLeft;
            for (var i = 0; i < cell.length; i++) {
                point = cell[i].getBoundingClientRect();
                var xlimit = point.left + scrollLeft + 5;
                if (xlimit > this._orgX && $(cell[i]).height() + point.top >= this._orgY) {
                    this._currentCell = i - 1;
                    return;
                }
                if (i == cell.length - 1 || (this.gridInstance.model.showStackedHeader && $(this._target).get(0) === cell[i])) {
                    this._currentCell = i;
                    return;
                }
            }
        },
        _moveVisual: function (_x) {
            /// Used to move the visual element in mouse move
            var _bounds = this.gridInstance.getHeaderContent().find("div").first()[0].getBoundingClientRect();
            if ((_bounds.left + document.documentElement.scrollLeft + _bounds.width < _x) || (_x < _bounds.left + document.documentElement.scrollLeft))
                this._$visualElement.remove();
            else if (this._currentCell != -1)
                this._$visualElement.css({ left: _x, top: this._tableY });
        },
        _mouseDown: function (e) {
            if (this._allowStart && ($(e.target).closest("tr").css("cursor") == 'col-resize')) {
                this._target = e.target;
                var _x = e.clientX, _y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") != -1) {
                    _x = e.pageX;
                    _y = e.pageY - document.body.scrollTop;
                }
                if (this.gridInstance.model.allowResizing && this.gridInstance.getHeaderTable().find(".e-columnheader").css('cursor') == 'col-resize') {
                    if ($(e.target).is(".e-headercelldiv"))
                        e.target = e.target.parentNode;
                    this._target = e.target;
                    if (this._triggerResizeEvents("resizeStart", _x))
                        return;
                }
                var gridobj = this;
                _x += document.documentElement.scrollLeft;
                if (e.button != 2)
                    this._start(_x, _y);
                $(document).on("selectstart.ejgrid", function(){return false});
            }
            return false;
        },
        _columnResizeToFit: function (e) {
            var resize = this.gridInstance.getHeaderTable().find(".e-columnheader").filter(function (e) {
                return $(this).css("cursor") == "col-resize";
            });
            if (this.gridInstance.model.allowResizeToFit && resize.length) {
                if ($(e.target).is(".e-headercelldiv"))
                    e.target = e.target.parentNode;
                var $target = $(e.target);
                var headerCells = this.gridInstance.getHeaderContent().find(".e-headercell"), preCol = 0, indent = 0;
                this._target = $target;
                if ($target.hasClass("e-headercell")) {
                    var targetCell = e.target;
                    var hCellIndex = $.inArray(targetCell, headerCells);
                    var cellIndex = hCellIndex;
                    this._orgX = e.pageX;
                    if(!this.gridInstance.model.enableRTL) 
						this._getResizableCell();
					else
						this._currentCell = hCellIndex;
                    if (hCellIndex != this._currentCell) {
                        hCellIndex = cellIndex = this._currentCell;
                        targetCell = e.target.previousSibling;
                    }
                    var currentTh = headerCells.filter(":visible")[cellIndex], changesCellIndex = [], changesFinalWdith = [], changesOldWidth = [];
                    if (this.gridInstance.model.groupSettings.showGroupedColumn)
                        indent = this.gridInstance.model.groupSettings.groupedColumns.length;
                    if ($(targetCell).parent("tr").hasClass('e-stackedHeaderRow')) {
                        currentSpan = $(currentTh).attr('colspan'), tr = $(currentTh).parent('tr'), tHeadIndex = currentTh.cellIndex;
                        var nextTr = tr.next();
                        while (tHeadIndex > indent) {
                            tHeadIndex--
                            var th = $(tr).children('th')[tHeadIndex];
                            preCol += parseInt($(th).attr('colspan'))
                        };
                        var length = preCol + parseInt(currentSpan);
                    }
                    else {
                        if (this.gridInstance.model.showStackedHeader)
                            cellIndex = hCellIndex = cellIndex - this.gridInstance.getHeaderTable().find(".e-stackedHeaderCell").length;
                        preCol = cellIndex; length = cellIndex + 1;
                    }
                    var finalWidth = 0, headerWidth = 0, contentWidth = 0, argCols = [], argExtra = [];;
                    if (preCol != -1) {
                        var args = { columnIndex: preCol, column: this.gridInstance.model.columns[preCol], target: $target, oldWidth: oldWidth };
                        this.gridInstance._trigger("resizeStart", args);
                        for (var i = preCol; i < length; i++) {
                            var hiddenLen = headerCells.slice(0, i + 1).filter(".e-hide").length;
                            contentWidth = this._getContentWidth(i + hiddenLen);
                            $cellDiv = this.gridInstance.getHeaderTable().find('.e-headercell:not(.e-hide, .e-stackedHeaderCell)').children(".e-headercelldiv").eq(i);
                            headerWidth = this._getHeaderContentWidth($cellDiv);
                            finalWidth = headerWidth > contentWidth ? headerWidth : contentWidth;
                            finalWidth += parseInt($cellDiv.css("padding-left"), 10);
                            var oldWidth = this.gridInstance.getHeaderTable().find('col').filter(this._diaplayFinder).eq(i + indent).width();
                            finalWidth = oldWidth > finalWidth ? finalWidth : (this._colMinWidth < finalWidth ? finalWidth : this._colMinWidth);

                            this.gridInstance.getHeaderTable().find('col').filter(this._diaplayFinder).eq(i + indent).width(finalWidth);
                            if (this.gridInstance.model.groupSettings.groupedColumns.length) {
                                var $colGroups = this.gridInstance.getContentTable().find('.e-recordtable').find('colgroup');
                                var proxy = this;
                                $.each($colGroups, function (indx, colgroup) {
                                    $(colgroup).find('col').filter(proxy._diaplayFinder).eq(i).width(finalWidth);
                                });
                            }
                            this.gridInstance.getContentTable().find('col').filter(this._diaplayFinder).eq(i + indent).width(finalWidth);
                            argCols.push(this.gridInstance.model.columns[i]);
                            argExtra.push(Math.abs(finalWidth - oldWidth))
                            changesCellIndex.push(i); changesFinalWdith.push(finalWidth); changesOldWidth.push(oldWidth);
                            if (this.gridInstance.model.scrollSettings.frozenColumns>0)
                                this.gridInstance.setWidthToColumns();
                        }

                    }
                    this.gridInstance._colgroupRefresh();
                    args = { columnIndex: changesCellIndex, column: argCols, target: currentTh, oldWidth: changesOldWidth, newWidth: changesFinalWdith, extra: argExtra };
                    this.gridInstance._trigger("resizeEnd", args);
                    for (var i = 0; i < changesCellIndex.length; i++) {
                        this.gridInstance._columnsWidthCollection[changesCellIndex[i]] = changesFinalWdith[i];
                        this.gridInstance.model.columns[changesCellIndex[i]]["width"] = changesFinalWdith[i];
                    }
                    args = { columnIndex: changesCellIndex, column: argCols, target: currentTh, oldWidth: changesOldWidth, newWidth: changesFinalWdith, extra: argExtra };
                    this.gridInstance._trigger("resized", args);
                }
            }
        },
        _getContentWidth: function (cellindx) {
            var contentWidth = 0;
            var $span = ej.buildTag('span', {}, {}), proxy = this.gridInstance, tdWidth;
            if (!ej.isNullOrUndefined(proxy._gridRows)) {
                var rows = proxy._gridRows;
                if (this.gridInstance.model.scrollSettings.frozenColumns && cellindx >= this.gridInstance.model.scrollSettings.frozenColumns) {
                    rows = rows[1];
                    cellindx = cellindx - this.gridInstance.model.scrollSettings.frozenColumns;
                }
                $.each(rows, function (indx, row) {
					var td = $(row).find('td.e-rowcell').eq(cellindx);
					var content = $(td).html();
					if (proxy.model.columns[cellindx]["isUnbound"])
						$span.html($(content).children());
					else
						$span.html(content);
					$(td).html($span);
					tdWidth = td.find('span:first').width();
					if (tdWidth > contentWidth)
						contentWidth = tdWidth;
					$(td).html(content);
				});
			}
            proxy._refreshUnboundTemplate(this.gridInstance.getContentTable());
            return contentWidth;
        },
        _getHeaderContentWidth: function ($cellDiv) {
            var headerWidth = 0, $span = ej.buildTag('span', {}, {});
            var content = $cellDiv.html();
            $span.html(content);
            $cellDiv.html($span);
            headerWidth = $cellDiv.find('span:first').width();
            if (this.gridInstance.model.allowFiltering && this.gridInstance.model.filterSettings.filterType == "menu")
                headerWidth = headerWidth + $cellDiv.parent().find(".e-filtericon").width() + 10;
            $cellDiv.html(content);
            return headerWidth;
        },
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.scheduleFeatures = ej.scheduleFeatures || {};

    var horizontalHeaderTemplate = function () {
        this._horiDateHeader = ("{{if multiRes == true}}<td class = 'e-horizontalheader'><div class='e-horizontalresheader'>{{:label}}</div></td>{{else}}{{/if}}" +
                                "<td class='e-horizontaldaterender'><div class='e-horizontaltimecells' style='height:{{:height}}px;'>" +
                                "<div class='e-headerdays' style='position:absolute;'>" +
                                "<table class ='e-table' cellpadding='0px' cellspacing='0px' style='width:100%;border-collapse:separate;'><tbody>{{if view !== 'month'}}" +
                                "<tr style='width:100%;'> <td>{{if view === 'day'}}<table cellspacing='0px' style='width:100%;'>{{else}}<table class='e-table' cellspacing='0px'>{{/if}}<tbody><tr> {{for cols ~headercellWidth = headercellWidth}}<td class='e-headercells {{:cellToday}} e-horizontalheadertddate'><div class='{{:currentDateClass}} e-horizontalheaderdiv'>{{:currentDay}}</div></td>{{/for}} </tr></tbody></table></td> </tr>" +
                                "<tr style='width:100%;position:absolute;'> <td><table class =' e-table e-horizontaltabletimecell' cellspacing='0px' style='border-collapse:collapse;'><tbody><tr> {{for timeTdCount }}{{for ~name=time ~dayend=timecellclass}}<td class='e-horizontaltimetd {{:~dayend}}'><div style='width:{{:timeCellWidth}}'><span class ='e-horizontaltimespan'>{{:~name.slice(0,2)}}</span><div class ='e-horizontaltimespandiv'>{{:~name.slice(3,5)}}</div></div></td><td class='e-horizontalalternatetd'><div style='width:{{:timeCellWidth}}'></div></td>{{/for}}{{/for}}  </tr></tbody></table></td> </tr>" +
                                "{{/if}}" +
                                "{{if view == 'month'}}<tr>{{for cols}}<td class='e-headercells e-horizontmonthheaderdatetd {{:cellToday}}'><div class='e-dateheadercell e-horizontmonthdatealign'>{{:currentDate}}</div></td>{{/for}}</tr>" +
                                "<tr>{{for cols}}<td class ='e-horizontmonthheaderdaytd'><div class='e-dateheadercell e-horizontmonthdatealign' style='width:{{:cellWidth}}'>{{:currentDay}}</div></td>{{/for}}</tr>" +
                                "{{/if}}</tbody></table></div></div></td>");
        return this._horiDateHeader;
    };

    var horizontalResourceHeaderTemplate = function () {
        this._horizontalResHeader = ("<div class ='e-horizontresdiv'><div class='e-horires' style='height:100%'><table class='e-table e-resourceheadertable' height='100%' width='100%' cellpadding='0px' cellspacing='0px' style='table-layout:fixed'>" +
            "<tbody>{{for trs}}<tr>{{for ~resname=name ~marginleft=marginleft }}{{if classname == 'e-parentnode'}}<td class='{{:classname}}' style='width:25px;height:{{:cellHeight}}'></td>{{else}}<td style='width:25px;height:{{:cellHeight}}'></td>{{/if}}<td class='{{:classname}}' style='vertical-align:middle;height:{{:cellHeight}}'>" +
            "<div class='e-resourceheadertext' style='text-align:left;margin-left:{{:~marginleft}};' title='{{if userResTemplId == true}}{{:name}}{{else}}{{:~resname}}{{/if}}'>{{if userResTemplId == true}}{{:userResHeader}}{{else}}{{:~resname}}{{/if}}</div>" +
            "</td>{{if classname == 'e-parentnode'}}<td id={{:idnum}} class='{{:classname}} {{:classname}}category e-resourceicon e-resourcecollapse' style='width:25px;vertical-align:middle;height:{{:cellHeight}}'></td>{{else}}<td class='{{:classname}} {{:classname}}category' style='width:25px;height:{{:cellHeight}}'></td>{{/if}}{{/for}}</tr>{{/for}}</tbody></table></div></div><div class='e-resfootertr' style='height:18px;'></div>");
        return this._horizontalResHeader;
    };

    var horizontalCellsTemplate = function () {
        this._cellsTemplate = ("<table class='e-table e-workcellstab' height='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for cellrows ~cols=cols ~hourdiff=hourdiff ~view=view ~classname=classname ~tdWidth=tdWidth}}" +
        "{{if classname == 'e-parentnode'}} <tr class='e-resourceheadertr' style='width:100%'>{{for ~cols ~tdWidth=tdWidth}}{{if #index % ~hourdiff == 0 && #index != 0 && ~view !== 'month' }}<td class='e-resourceheadercells e-workcells e-parentworkcell e-dayend' style='width:{{:cellWidth}};height:{{:cellHeight}}'>{{else}}<td class='e-resourceheadercells e-workcells e-parentworkcell' style='width:{{:cellWidth}};height:{{:cellHeight}}'>{{/if}}<div class='e-appointwrapper'></div></td>{{if ~view !== 'month'}}<td class='e-resourceheadercells e-workcells e-parentworkcell e-alternateworkcell' style='width:{{:cellWidth}};height:{{:cellHeight}}'><div class='e-appointwrapper'></div></td>{{/if}}{{/for}}</tr>" +
        "{{else}}<tr style='width:100%'>{{for ~cols ~tdWidth=tdWidth}}{{if #index % ~hourdiff == 0 && #index != 0 && ~view !== 'month' }}<td class='e-workcells e-childworkcell e-dayend' style='width:{{:cellWidth}};height:{{:cellHeight}}'>{{else}}<td class='e-workcells e-childworkcell' style='width:{{:cellWidth}};height:{{:cellHeight}}'>{{/if}}<div class='e-appointwrapper'></div></td>{{if ~view !== 'month'}}<td class='e-workcells e-childworkcell e-alternateworkcell' style='width:{{:cellWidth}};height:{{:cellHeight}}'><div class='e-appointwrapper'></div></td>{{/if}}{{/for}}" +
        "</tr>{{/if}}" +
        "{{/for}}</tbody></table>");
        return this._cellsTemplate;
    };

    ej.scheduleFeatures.horizontal = {

        _renderHorizontalTemplates: function () {
            this.horizontalHeader = $.templates(this._createTemplate(horizontalHeaderTemplate(), "_HorizontalHeader"));
            this.cellTemplate = $.templates(this._createTemplate(horizontalCellsTemplate(), "_HorizontalCells"));
            this.horizontalResHeader = $.templates(this._createTemplate(horizontalResourceHeaderTemplate(), "_HorizontalResources"));
        },

        _horizontalViewNavigation: function (e) {
            var currentDateIndex, navidate;
            if (e.type == "click") {
                if (this.currentView() === "week" || this.currentView() === "workweek" || (this.currentView() === "customview" && this._renderDays <= 7)) {
                    this.element.find(".e-headercells").children().removeClass("e-activeview");
                    currentDateIndex = e.currentTarget.cellIndex;
                    navidate = new Date(this._getRenderDates()[currentDateIndex]);
                    if (this.model.minDate <= navidate && this.model.maxDate >= navidate) {
                        (this.model.showTimeScale) && $(e.currentTarget).children().addClass("e-activeview");
                        this.currentDate(navidate);
                    }
                    else { return false; }
                    if (this.model.highlightBusinessHours == true && !ej.isNullOrUndefined(this.model.businessStartHour))
                        this._businessHourScroller();
                    else {
                        var cellOffsetLeft = this.element.find(".e-workcells")[currentDateIndex * ((this.model.endHour - this.model.startHour) * 2)].offsetLeft;
                        if (!this.model.enableRTL) {
                            $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", (-cellOffsetLeft) + "px");
                            $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: cellOffsetLeft });
                        } else {
                            var workcellLeft = this.element.find(".e-workcells")[0].offsetLeft;
                            var highScrol = workcellLeft - cellOffsetLeft;
                            $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: highScrol });
                            var maxLeft = workcellLeft + this.element.find(".e-workcells").width() - this.element.find('.e-draggableworkarea').width();
                            highScrol = highScrol >= maxLeft ? maxLeft : highScrol;
                            $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", highScrol + "px");
                        }
                    }
                }
                else if (this.currentView() == "month" || (this._isCustomView())) {
                    currentDateIndex = e.currentTarget.cellIndex;
                    if (this.model.views.indexOf("day") == -1) return false;
                    navidate = new Date(this.monthDays[currentDateIndex]);
                    if (this.model.minDate <= navidate && this.model.maxDate >= navidate) {
                        this.currentDate(navidate);
                    }
                    else { return false; }
                    newView = "day";
                    this._navView = true;
                    this._viewChange(e, this.currentView(), newView);
                }
            }
            else if (e.type == "dblclick") {
                if (this.currentView() === "week" || this.currentView() === "workweek" || (this.currentView() === "customview" && this._renderDays <= 7)) {
                    var target = $(e.target);
                    var dayTarget = target.hasClass("e-headercells") ? target : target.parent().hasClass("e-headercells") ? target.parent() : target.parent().parent();
                    if (this.model.views.indexOf("day") == -1 || !dayTarget.hasClass("e-headercells")) return false;

                    navidate = new Date(this._getRenderDates()[dayTarget.index()]);
                    if (this.model.minDate <= navidate && this.model.maxDate >= navidate) {
                        this.currentDate(navidate);
                    }
                    else { return false; }
                    var newView = "day";
                    this._navView = true;
                    this._viewChange(e, this.currentView(), newView);
                }
            }

        },

        _renderHorizontalHeaderDays: function (localizeDays) {
            this.element.find(".e-headercells").addClass("e-dateheaderselect");
            if (this.currentView() !== "month" && !(this._isCustomView())) {
                var curday, temp;
                for (var i = 0; i < this._dateRender.length; i++) {
                    curday = (this.currentView() === "day") ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? localizeDays.calendar.days.names[new Date(this._dateRender[i]).getDay()] : localizeDays.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3) : localizeDays.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3);
                    if (this.model.showTimeScale)
                        temp = this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMMM yyyy") : curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMM yy") : curday + " " + new Date(this._dateRender[i]).getDate();
                    else
                        temp = this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? Globalize.format(new Date(this._dateRender[i]), "dd MMMM yyyy") : Globalize.format(new Date(this._dateRender[i]), "dd MMM yy") : ("0" + new Date(this._dateRender[i]).getDate()).slice(-2);
                    $(this.element.find(".e-headercells")[i]).find("div")[0].innerHTML = temp;
                }
                var currentDate = new Date(this.currentDate());
                if (this.model.orientation == "horizontal" && this.currentView() != "month" && this.model.showTimeScale) {
                    var active = this.element.find(".e-headercells").children(".e-activeview").parent().index();
                    this.element.find(".e-headercells").children().removeClass("e-activeview");
                    var index = this.currentView() === "day" ? 0 : this.currentView() === "workweek" ? currentDate.getDay() - 1 : active;
                    this.currentView() != "day" && $(this.element.find(".e-headercells")[index]).children().addClass("e-activeview");
                    new Date(new Date(this.currentDate()).setHours(0, 0, 0, 0)).getTime() === new Date(new Date().setHours(0, 0, 0, 0)).getTime() ? $(this.element.find("td.e-horizontalheadertddate")[index]).addClass("e-headerToday") : "";
                }
            }
        },

        _horizontalScroll: function (e, proxy) {
            if (proxy.currentView() == "week" || proxy.currentView() == "workweek" || (this.currentView() === "customview" && this._renderDays <= 7)) {
                var currentDateIndex = $(proxy.element.find(".e-headercells").find(".e-activeview").parent()).index();
                var index = Math.floor((Math.abs(e.scrollLeft) / (proxy.element.find(".e-workcells").width() + 1)) / ((this.model.endHour - this.model.startHour) * 2));
                if (currentDateIndex !== index)
                    proxy.element.find(".e-headercells").children().removeClass("e-activeview");
                $(proxy.element.find(".e-headercells")[index]).children().addClass("e-activeview");
                (!this.model.showTimeScale) && proxy.element.find(".e-headercells").children().removeClass("e-activeview");
            }
        },

        _getHorizontalheaderCellCount: function (weekcols) {
            var cols = [];
            for (var i = 0; i < this._dateRender.length; i++) {
                var curday = (this.currentView() === "month" || this.currentView() === "day" || (this._isCustomView())) ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()] : weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3) : weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3);
                var headerCellClass = (new Date(new Date(this._dateRender[i]).setHours(0, 0, 0, 0)).getTime() === new Date(new Date().setHours(0, 0, 0, 0)).getTime()) ? "e-headerToday" : "";
                var temp = this.currentView() === "month" || (this._isCustomView()) ? { currentDay: curday, currentDateClass: "", cellToday: headerCellClass, cellWidth: this.model.cellWidth } : this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? { currentDay: curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMMM yyyy"), currentDateClass: "", cellToday: headerCellClass, cellWidth: this.model.cellWidth } : { currentDay: curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMM yy"), currentDateClass: "", cellToday: headerCellClass, cellWidth: this.model.cellWidth } : { currentDay: curday + " " + new Date(this._dateRender[i]).getDate(), currentDateClass: new Date(this._dateRender[i]).getDate() === new Date(this.currentDate()).getDate() ? "e-activeview" : "", cellToday: headerCellClass, cellWidth: this.model.cellWidth };
                cols.push(temp);
            }
            return cols;
        },

        _getHoriStartEndTime: function (localizeAmPm) {
            var startendTimes = [];
            var daysCount = this.currentView() == "week" ? 7 : this.currentView() == "day" ? 1 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : this.model.workWeek.length;
            for (var day = 0; day < daysCount; day++) {
                for (var i = this.model.startHour; i < this.model.endHour; i++) {
                    if (this._timeMode == "12") {
                        var timelist = ((i === this.model.startHour || i === 12) && (localizeAmPm.AM && localizeAmPm.PM)) ? Globalize.format(new Date(new Date().setHours(i)), "hh tt") : Globalize.format(new Date(new Date().setHours(i)), "hh 00");
                        startendTimes.push({ time: timelist, id: i, timecellclass: (i == this.model.startHour && day != 0) ? "e-dayend" : "", cellWidth: (!this.model.cellWidth) ? "30px" : this.model.cellWidth, cellHeight: this.model.cellHeight, timeCellWidth: (!this.model.cellWidth) ? (this._browserInfo.name == "webkit") ? "29px" : "30px" : (this._browserInfo.name == "webkit") ? (parseInt(this.model.cellWidth) - 1 + "px") : this.model.cellWidth });
                    }
                    else
                        startendTimes.push({ time: (i <= 9 ? "0" + i : i) + " 00", timecellclass: (i == this.model.startHour && day != 0) ? "e-dayend" : "", cellWidth: (!this.model.cellWidth) ? "30px" : this.model.cellWidth, cellHeight: this.model.cellHeight, timeCellWidth: (!this.model.cellWidth) ? (this._browserInfo.name == "webkit") ? "29px" : "30px" : (this._browserInfo.name == "webkit") ? (parseInt(this.model.cellWidth) - 1 + "px") : this.model.cellWidth });
                }
            }
            return startendTimes;
        },

        _getHorizontalDayAppointments: function (appointmentsList, dateCount, day, res) {
            for (var app = 0; app < appointmentsList.length; app++) {
                if (new Date(new Date(dateCount[day]).setHours(0, 0, 0, 0)).getTime() == new Date(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() &&
                      new Date(new Date(dateCount[day]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime() && (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? appointmentsList[app][this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]] == this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]] : true &&
                           new Date(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) {
                    this._renderApp(appointmentsList[app], day, res);
                    this._filtered.push(appointmentsList[app]);
                }
            }
        },

        _renderHorizontalApp: function (rStartTime, rEndTime, record, day, cellHeight, timeScaleRowCount, startHour, endHour) {
            var headerCell = this.element.find(".e-workcellstab tr");
            var newResCollection = this._getResourceCollection();
            var appHeight, appWidth, topPosition, leftPosition;
            if (rStartTime <= rEndTime) {
                var startTime = ((new Date(rStartTime).getHours()) < startHour) ? (new Date(rStartTime).getHours() + (-new Date(rStartTime).getHours()) + (-new Date(rStartTime).getMinutes())) : (new Date(rStartTime).getHours() + (-startHour));
                var endTime = ((new Date(rEndTime).getHours()) > endHour) ? (new Date(rEndTime).getHours() + (-new Date(rEndTime).getHours() + (endHour - startHour)) + (-new Date(rEndTime).getMinutes())) : (new Date(rEndTime).getHours() + (-startHour));
                endTime = (new Date(rEndTime).getHours() == endHour) ? (new Date(endTime).getMinutes() + (-new Date(rEndTime).getMinutes())) : endTime;
                appHeight = 20;
                topPosition = 0;
                var level = 0;
                if (this.model.startHour <= new Date(rStartTime).getHours()) {
                    var groupIndex = (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) ? this._findResourceIndex(newResCollection[0].dataSource, newResCollection[0].id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]) + 1 : 0;
                    var resValue = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this._findResourceIndex(this.level_Resources, this.model.resources[this.model.resources.length - 1].resourceSettings.id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) + 1 : 1;
                    var overlapList = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? this._overlapApp(record["AppTaskId"], record[this.model.appointmentSettings["startTime"]], record[this.model.appointmentSettings["endTime"]], record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : this._overlapApp(record["AppTaskId"], record[this.model.appointmentSettings["startTime"]], record[this.model.appointmentSettings["endTime"]], null);
                    var proxy = this;
                    level = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? this._getOverlapCount(record, day, this._dateRender, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : this._getOverlapCount(record, day, this._dateRender, null);
                    topPosition = (topPosition + (level * appHeight));
                    var diffTime, diffDays, diffMinutes, cellIndex;
                    if (record[this.model.appointmentSettings["allDay"]]) {
                        diffTime = new Date(new Date(new Date(rEndTime).setHours(0, 0, 0, 0)).getTime()) - new Date(new Date(new Date(rStartTime).setHours(0, 0, 0, 0)).getTime());
                        diffDays = Math.round(diffTime / 86400000) + 1;
                        if (diffDays > this._dateRender.length)
                            diffDays = new Date(this._dateRender[this._dateRender.length - 1]).getDay() + 1 - new Date(rStartTime).getDay();
                        diffMinutes = (diffDays * (endHour - startHour) * 60);
                        cellIndex = ((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day;
                        var totHours = Math.round((record[this.model.appointmentSettings["endTime"]] - record[this.model.appointmentSettings["startTime"]]) / 3600000);
                        appWidth = ((this.element.find(".e-workcells").outerWidth() / (30)) * diffMinutes) + (totHours * timeScaleRowCount);
                        leftPosition = "0";
                    }
                    else if (new Date(new Date(new Date(record[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime()) < new Date(new Date(new Date(record[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime())) {
                        diffTime = new Date(new Date(new Date(rEndTime).setHours(0, 0, 0, 0)).getTime()) - new Date(new Date(new Date(rStartTime).setHours(0, 0, 0, 0)).getTime());
                        var endDiffer = (new Date(rEndTime) < new Date(new Date(rEndTime).setHours(this.model.endHour, 0, 0)) && new Date(rEndTime) < new Date(new Date(rEndTime).setHours(this.model.startHour, 0, 0))) ? (((this.model.endHour) / 2) * 60) : ((new Date(rEndTime).getHours() * 60) + new Date(rEndTime).getMinutes());
                        var startDiffer = new Date(rStartTime) > new Date(new Date(rEndTime).setHours(this.model.startHour, 0, 0)) ? (this.model.startHour * 60) : ((new Date(rStartTime).getHours() * 60) + new Date(rStartTime).getMinutes());
                        diffDays = Math.round(diffTime / 86400000);
                        diffMinutes = (((diffDays) * ((endHour - startHour)) * 60) - startDiffer) + endDiffer;
                        cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
                        var totHours = Math.round((record[this.model.appointmentSettings["endTime"]] - record[this.model.appointmentSettings["startTime"]]) / 3600000);
                        appWidth = ((this.element.find(".e-workcells").outerWidth() / (30)) * diffMinutes) + (totHours * timeScaleRowCount);
                        if (diffDays > this._dateRender.length) {
                            diffDays = new Date(this._dateRender[this._dateRender.length - 1]).getDay() + 1 - new Date(rStartTime).getDay();
                            diffMinutes = ((diffDays) * (endHour - startHour) * 60);
                            appWidth = ((this.element.find(".e-workcells").outerWidth() / (30)) * diffMinutes) - (cellIndex * 25) + 10;
                        }
                        leftPosition = (parseFloat((new Date(rStartTime).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);

                    }
                    else {
                        leftPosition = (parseFloat((new Date(rStartTime).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);
                        if ((this.model.endHour - this.model.startHour) == 24)
                            var diffInMinutes = ((new Date(rEndTime).getHours() * 60) + new Date(rEndTime).getMinutes()) - ((new Date(rStartTime).getHours() * 60) + new Date(rStartTime).getMinutes());
                        else {
                            var endDiffer = ((new Date(rEndTime).getHours() * 60) + new Date(rEndTime).getMinutes()) > (this.model.endHour * 60) ? (this.model.endHour * 60) : ((new Date(rEndTime).getHours() * 60) + new Date(rEndTime).getMinutes());
                            var startDiffer = ((new Date(rStartTime).getHours() * 60) + new Date(rStartTime).getMinutes()) < (this.model.startHour * 60) ? (this.model.startHour * 60) : ((new Date(rStartTime).getHours() * 60) + new Date(rStartTime).getMinutes());
                            var diffInMinutes = endDiffer - startDiffer;
                        }
                        appWidth = ((this.element.find(".e-workcells").width() / 30) * diffInMinutes) + ((new Date(rEndTime).getHours() - new Date(rStartTime).getHours()) * timeScaleRowCount) - 2;
                        cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
                    }
                    var colorAppointment = this._appointmentColor(record);
                    var appointColor, categoryClass;
                    if ((topPosition + appHeight) < cellHeight) {
                        var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                        var userTempHtml = this._getUserAppointmentTemplate(record);
                        var startTimeDisp = this._timeMode == "12" ? Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), this._pattern.t) : Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "HH:mm");
                        var endTimeDisp = this._timeMode == "12" ? Globalize.format(new Date(record[this.model.appointmentSettings["endTime"]]), this._pattern.t) : Globalize.format(new Date(record[this.model.appointmentSettings["endTime"]]), "HH:mm");
                        var timeCellhtml = this.appTemplate.render({ id: record["AppTaskId"], appResize: this.model.enableAppointmentResize, currentview: this.currentView(), appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp, left: leftPosition + "px", top: topPosition, appWidth: appWidth + "px", userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-appointment", appointData: colorAppointment.appointColor.appointData, appointCustomcss: colorAppointment.appointColor.appointCustomcss, appointtextcolor: colorAppointment.appointColor.appointtextcolor, value: colorAppointment.value, multiDiv: colorAppointment.multiDiv, orientation: this.model.orientation, priority: this.model.prioritySettings.enable ? record[this.model.appointmentSettings["priority"]] : '', enablePriority: this.model.prioritySettings["template"], priorityTemplate: this.model.prioritySettings["template"] ? this._priorityTemplate(record) : record[this.model.appointmentSettings['priority']], uid: record.Guid });
                        $($(headerCell[resValue + groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append(timeCellhtml);
                        if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) {
                            appointColor = this._getResourceColor(record);
                            categoryClass = (appointColor.appointData == "") ? "e-categorycolor" : "";
                            if (this.model.enableRTL)
                                $($(this.element.find(".e-resourceheadertr")[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar " + categoryClass + "' id='categorybar_" + record.Guid + "' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;right:" + leftPosition + "px;'></div>");
							else
                                $($(this.element.find(".e-resourceheadertr")[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar " + categoryClass + "' id='categorybar_" + record.Guid + "' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + leftPosition + "px;'></div>");
                        }
                    }
                    else {
                        if ($("#" + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + cellIndex).length == 0) {
                            $($(headerCell[resValue + groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append('<div id="' + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + cellIndex + '" class="e-icon e-appsoverflow"></div>');
                            if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) {
                                appointColor = this._getResourceColor(record);
                                categoryClass = (appointColor.appointData == "") ? "e-categorycolor" : "";
                                if (this.model.enableRTL)
                                    $($(this.element.find(".e-resourceheadertr")[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar " + categoryClass + "' id='categorybar_" + record.Guid + "' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;right:" + leftPosition + "px;'></div>");
								else
                                    $($(this.element.find(".e-resourceheadertr")[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar " + categoryClass + "' id='categorybar_" + record.Guid + "' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + leftPosition + "px;'></div>");
                            }
                            this.element.find("#" + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + cellIndex).css(this.model.enableRTL ? "right" : "left", (this.element.find(".e-workcells").width() + leftPosition - 25) + "px").css("top", topPosition + "px");
                        }
                    }

                }
            }
        },

        _renderCategory: function (headerCell, groupIndex, record, cellHeight, minDate, maxDate, appointColor, timeScaleRowCount, day, startTime, endTime) {
            var diffTime, diffDays, diffMinutes, cellIndex, appWidth, leftPosition;
            if (record[this.model.appointmentSettings["allDay"]]) {
                diffTime = new Date(new Date(new Date(maxDate).setHours(0, 0, 0, 0)).getTime()) - new Date(new Date(new Date(minDate).setHours(0, 0, 0, 0)).getTime());
                diffDays = Math.round(diffTime / 86400000) + 1;
                if (diffDays > this.dateRender.length)
                    diffDays = new Date(this.dateRender[this.dateRender.length - 1]).getDay() + 1 - new Date(minDate).getDay();
                diffMinutes = (diffDays * 24 * 60);
                cellIndex = ((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day;
                appWidth = ((this.element.find(".e-workcells").width() / (30 - 1.1)) * diffMinutes) + 3;
                leftPosition = "0";
            }
            else if (new Date(new Date(new Date(record[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime()) < new Date(new Date(new Date(record[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime())) {
                diffTime = new Date(new Date(new Date(maxDate).setHours(0, 0, 0, 0)).getTime()) - new Date(new Date(new Date(minDate).setHours(0, 0, 0, 0)).getTime());
                diffDays = Math.round(diffTime / 86400000) + 1;
                diffMinutes = ((diffDays - 1) * 24 * 60);
                cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
                appWidth = ((this.element.find(".e-workcells").width() / (30 - 1.1)) * diffMinutes) + (new Date(maxDate).getMinutes() * ((this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount) / 60) + ((new Date(maxDate).getHours() - this.model.startHour) * timeScaleRowCount);
                if (diffDays > this.dateRender.length) {
                    diffDays = new Date(this.dateRender[this.dateRender.length - 1]).getDay() + 1 - new Date(minDate).getDay();
                    diffMinutes = ((diffDays) * 24 * 60);
                    appWidth = ((this.element.find(".e-workcells").width() / (30 - 1.1)) * diffMinutes) - (cellIndex * 25) + 10;
                }
                leftPosition = (parseFloat((new Date(minDate).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);

            }
            else {
                leftPosition = (parseFloat((new Date(minDate).getMinutes())) * (this.element.find(".e-workcells")[0].offsetWidth) * timeScaleRowCount / 60);
                var diffInMinutes = ((new Date(maxDate).getHours() * 60) + new Date(maxDate).getMinutes()) - ((new Date(minDate).getHours() * 60) + new Date(minDate).getMinutes());
                appWidth = ((this.element.find(".e-workcells").width() / 30) * diffInMinutes) + ((new Date(maxDate).getHours() - new Date(minDate).getHours()) * timeScaleRowCount) - 2;
                cellIndex = (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * day) + (startTime * timeScaleRowCount);
            }
            if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1)
                $($($(headerCell.find("td.e-resourceheadercells")).parent()[groupIndex - 1]).find("div.e-appointwrapper")[cellIndex]).append("<div class='e-categorybar' style='position:absolute;height:" + (cellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + leftPosition + "px;'></div>");

        },

        _getHorizontalOverlapCount: function (appointments, dateRender, day, record, resValue) {
            var filterData = [];
            for (var i = 0; i < appointments.length; i++) {
                if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) {
                    if ((new Date(new Date(appointments[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                    (new Date(new Date(appointments[i][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime() >= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                    (((new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]])))
                        && ((resValue != null) ? (resValue == appointments[i][this.model.resources[this.model.resources.length - 1]["field"]]) : resValue == null))) {
                        filterData.push(appointments[i]);
                    }
                }
                else {
                    if ((new Date(new Date(appointments[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                    (new Date(new Date(appointments[i][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime() >= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                    (((new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["startTime"]]) >= new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["startTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]])) ||
                        (new Date(record[this.model.appointmentSettings["endTime"]]) > new Date(appointments[i][this.model.appointmentSettings["startTime"]]) && new Date(record[this.model.appointmentSettings["endTime"]]) < new Date(appointments[i][this.model.appointmentSettings["endTime"]]))))) {
                        filterData.push(appointments[i]);
                    }
                }
            }
            return filterData;
        },

        _moveScroller: function () {
            if (this.currentView() == "week" || this.currentView() == "workweek" || (this.currentView() === "customview" && this._renderDays <= 7)) {
                if (this.element.find(".e-headercells").children().hasClass("e-activeview")) {
                    var currentDateIndex = $(this.element.find(".e-headercells").find(".e-activeview").parent()).index();
                    var cellOffsetLeft = this.element.find(".e-workcells")[currentDateIndex * ((this.model.endHour - this.model.startHour) * 2)].offsetLeft;
                    if (this.model.enableRTL) {
                        var workcellLeft = this.element.find(".e-workcells")[0].offsetLeft;
                        cellOffsetLeft = workcellLeft - cellOffsetLeft;
                        $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: cellOffsetLeft });
                        var maxLeft = workcellLeft + this.element.find(".e-workcells").width() - this.element.find('.e-draggableworkarea').width();
                        cellOffsetLeft = cellOffsetLeft >= maxLeft ? maxLeft : cellOffsetLeft;
                        $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", cellOffsetLeft + "px");
                    }
                    else {
                        $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", (-cellOffsetLeft) + "px");
                        $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: cellOffsetLeft });
                    }
                }
            }
        },


        _renderHorizontalHeaderBar: function () {
            var view = this.model.showTimeScale ? (this._isCustomView()) ? "month" : this.currentView() : "month"; var weekcols = Globalize.culture(this.model.locale);
            var viewColCount = view == "day" ? 1 : view == "week" ? 7 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : this.model.workWeek.length;

            this.datesColumn = this._getHeaderAllDayCellsCount();
            if (view === 'month' || (this._isCustomView())) {
                this.datesColumn = [], this.monthDays = [], this.totdays = [];
                for (var i = 0; i < this._dateRender.length; i++) {
                    if (this.currentView() === "month") {
                        if (new Date(this.currentDate()).getMonth() === new Date(this._dateRender[i]).getMonth())
                            this.totdays.push(this._dateRender[i]);
                    }
                    else
                        this.totdays.push(new Date(this._dateRender[i]));
                }
                var date = this.totdays.length;
                for (var i = 0; i < date; i++) {
                    var curday = weekcols.calendar.days.names[new Date(this.totdays[i]).getDay()];
                    if (this.model.showTimeScale)
                        var temp = { currentDate: Globalize.format(new Date(this.totdays[i]), "dd"), currentDateClass: "", currentDay: weekcols.calendar.days.names[new Date(this.totdays[i]).getDay()].toString().slice(0, 1), cellWidth: (!this.model.cellWidth && this.currentView() != "month" && !(this._isCustomView())) ? "30px" : this.model.cellWidth, cellHeight: this.model.cellHeight };
                    else {
                        if (this.currentView() == "day" || this.currentView() == "week" || this.currentView() == "workweek")
                            var temp = { currentDate: Globalize.format(new Date(this.totdays[i]), "dd"), currentDateClass: "", currentDay: this.currentView() == "day" ? weekcols.calendar.days.names[new Date(this.totdays[i]).getDay()] : weekcols.calendar.days.names[new Date(this.totdays[i]).getDay()].toString().slice(0, 3), cellWidth: !this.model.cellWidth ? "100%" : this.model.cellWidth, cellHeight: this.model.cellHeight };
                        else
                            var temp = { currentDate: Globalize.format(new Date(this.totdays[i]), "dd"), currentDateClass: "", currentDay: weekcols.calendar.days.names[new Date(this.totdays[i]).getDay()].toString().slice(0, 1), cellWidth: this.model.cellWidth, cellHeight: this.model.cellHeight };
                    }
                    this.datesColumn.push(temp);
                    this.monthDays.push(this.totdays[i]);
                }
            }
            var multiResources = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? true : false;
            var html = this.horizontalHeader.render({ cols: this.datesColumn, label: this._getLocalizedLabels("Resources"), multiRes: multiResources, view: view, timeTdCount: this._getStartEndTime(), colspan: view == "month" ? 1 : (this._getStartEndTime().length / viewColCount) * 2, height: view == 'month' ? 62 : 59, width: this.model.width, headercellWidth: view == "day" ? "100%" : "55px" });
            this.aTR.append(html);
        },

        _renderHorizontalContent: function ($contentAreaDiv, $contentTable, $contentBody, $contentTR, $WorkCellTD, $scrollerbarDiv) {
            var trCount, resHtml;
            if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) {
                resHtml = this._renderHorizontalResources();
                trCount = this._horiResources;
            }
            else {
                var tdWidth = this.currentView() == "month" || (this._isCustomView()) ? "24px" : "25px";
                resHtml = "";
                trCount = [{ classname: "e-childnode", tdWidth: tdWidth }];
            }
            var workCellhtml = this.cellTemplate.render({ cellrows: trCount, hourdiff: (this.model.endHour - this.model.startHour), view: this.model.showTimeScale ? (this._isCustomView()) ? "month" : this.currentView() : "month", cols: this.model.showTimeScale ? this.currentView() == 'month' || (this._isCustomView()) ? this.datesColumn : this._getStartEndTime() : this.datesColumn });
            if (resHtml == "")
                this.wTR.append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv.append(workCellhtml)))))))));
            else
                this.wTR.append(ej.buildTag('td.e-horizontalrestd', {}, {}, { width: "200px" }).append(resHtml)).append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv.append(workCellhtml)))))))));
        },

        _getResourceHeadTemplate: function (record) {
            var userTemp = $.templates($(this.model.resourceHeaderTemplateId).html());
            var userTempHtml = userTemp.render(record);
            return userTempHtml;
        },

        _renderHorizontalResources: function () {
            this._resCollect = []; this._horizontalResRender = []; this._horiResources = [];
            var view = this.currentView();
            var viewColCount = view == "day" ? 1 : view == "week" || view == "month" || (this._isCustomView()) ? 7 : this.model.workWeek.length;
            var tdWidth = this.currentView() == "month" || (this._isCustomView()) ? "24px" : "25px";
            var userResTemplId = (this.model.resourceHeaderTemplateId) ? true : false;
            var resTempHtml = "";
            if (!ej.isNullOrUndefined(this.model.group)) {
                if (this.model.group.resources.length === 1) {
                    var index;
                    for (var i = 0; i < this.model.group.resources.length; i++) {
                        index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
                    }
                    for (var i = 0; i < this._resourceInfo[index].dataSource.length; i++) {
                        this._horiResources.push({ cellHeight: this.model.cellHeight, name: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].text], idnum: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].id], classname: "e-childnode", marginleft: "20px", width: "10px", backgroundcolor: "", border: "1px solid #bbbcbb", bordertop: "", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate($.extend(this._resourceInfo[index].dataSource[i], { classname: "e-childnode" })), userResTemplId: userResTemplId });
                        this._resCollect.push(this._resourceInfo[index].dataSource[i]);
                    }
                    this._horizontalResRender.push(this._resCollect);
                }
                else {
                    var newResCollection = this._getResourceCollection();
                    var res = [], query, resApp, resourceHeaderHtml;

                    var d = 0;
                    var k = 0; var group = [];

                    for (var j = 0; j < newResCollection[d].dataSource.length; j++) {
                        this._horiResources.push({ cellHeight: this.model.cellHeight, name: newResCollection[d].dataSource[j][newResCollection[d].text], idnum: newResCollection[d].dataSource[j][newResCollection[d].id], classname: "e-parentnode", marginleft: "20px", width: "10px", backgroundcolor: "#bbbcbb", border: "1px solid #bbbcbb", bordertop: "1px solid #bbbcbb", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate($.extend(newResCollection[d].dataSource[j], { classname: "e-parentnode" })), userResTemplId: userResTemplId });
                        this._resCollect.push(newResCollection[d].dataSource[j]);
                        k++;

                        for (i = 0; i < newResCollection[k].dataSource.length; i++) {
                            var p = k;
                            if (newResCollection[k - 1].dataSource[j][newResCollection[k - 1].id] == newResCollection[k].dataSource[i][newResCollection[k].groupId]) {
                                group.push(newResCollection[k].dataSource[i]);
                                this._horiResources.push({ cellHeight: this.model.cellHeight, name: newResCollection[k].dataSource[i][newResCollection[k].text], idnum: newResCollection[k].dataSource[i][newResCollection[k].id] + "_" + newResCollection[k].dataSource[i][newResCollection[k].groupId], classname: "e-childnode", marginleft: "45px", width: "20px", backgroundcolor: "", border: "1px dotted #bbbcbb", bordertop: "", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate($.extend(newResCollection[k].dataSource[i], { classname: "e-childnode" })), userResTemplId: userResTemplId });
                                this._resCollect.push(newResCollection[k].dataSource[i]);
                                while (k < newResCollection.length - 1) {
                                    for (var m = 0; m < newResCollection[k + 1].dataSource.length; m++) {
                                        if (newResCollection[k].dataSource[i][newResCollection[k].id] == newResCollection[k + 1].dataSource[m][newResCollection[k + 1].groupId]) {
                                            group.push(newResCollection[k + 1].dataSource[m]);
                                            this._horiResources.push({ cellHeight: this.model.cellHeight, name: newResCollection[k + 1].dataSource[m][newResCollection[k + 1].text], classname: "e-childnode", marginleft: "55px", width: "20px", backgroundcolor: "", border: "1px dotted #bbbcbb", bordertop: "", tdWidth: tdWidth, userResHeader: this._getResourceHeadTemplate($.extend(newResCollection[k + 1].dataSource[m], { classname: "e-childnode" })), userResTemplId: userResTemplId });
                                            this._resCollect.push(newResCollection[k + 1].dataSource[m]);
                                        }
                                    }
                                    k++;
                                }

                            }
                            k = p;
                        }
                        k = 0;
                    }
                    this._horizontalResRender.push(this._resCollect);
                }
            }
            else {
                for (var k = 0; k < this.model.resources.length; k++) {
                    this._horizontalResRender.push(this.model.resources[k].resourceSettings.dataSource);
                    this._resCollect.push(this.model.resources[k].resourceSettings.dataSource);
                }
            }
            var html = this.horizontalResHeader.render({ trs: this._horiResources });
            return html;
        },

        _renderHorizontalCurrentTime: function () {
            if (this.currentView() !== "month" && !(this._isCustomView())) {
                if (this.element.find("td.e-headerToday").length > 0) {
                    this.element.find('.e-horizontaltabletimecell').parent().append('<div id="' + this._id + '_HighlightCurrentTimeline" class="e-highlightcurrenttimeline" style="height: 92%; width: 2px; position: absolute; background-color: red; "></div>');
                    $('#' + this._id + '_HighlightCurrentTimeline').css("top", this.element.find('.e-headerdays tr')[1].offsetTop);
                }
            }
            else {
                var monthDays = [];
                for (var k = 0; k < this._dateRender.length; k++) {
                    if (new Date(this.currentDate()).getMonth() === new Date(this._dateRender[k]).getMonth())
                        monthDays.push(new Date(this._dateRender[k]));
                }
                var target = this.element.find(".e-headerdays").find("table").find("td");
                for (var i = 0; i < monthDays.length; i++) {
                    if (new Date(new Date(monthDays[i]).setHours(0, 0, 0, 0)).getTime() === new Date(new Date().setHours(0, 0, 0, 0)).getTime())
                        $(target[i]).addClass("e-monthcurrenttime");
                }
            }
        },

        _horizontalTimePosition: function (temp, now) {
            if (temp.element.find("td.e-headerToday").length > 0) {
                var headertodaypos = temp.element.find("td.e-headerToday")[0].cellIndex;
                var startTime = (now.getHours() - temp.model.startHour);
                var cellIndex = (((temp.model.endHour - temp.model.startHour) * 2) * headertodaypos) + (startTime * 2);
                var leftPosition = (parseFloat((now.getMinutes())) * (temp.element.find(".e-workcells")[1].offsetWidth) * 2 / 60);
                $('#' + temp._id + '_HighlightCurrentTimeline').css("left", (cellIndex * temp.element.find(".e-workcells")[1].offsetWidth) + leftPosition);
            }
        },

        _horizontalBusinessHighlight: function (renderDates, bStartHour, bEndHour, timeRowsCount, target) {
            var tdLength = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group)) ? this._horiResources.length : 1;
            if (this.currentView() !== "month" && !(this._isCustomView()) && this.model.showTimeScale) {
                for (var res = 0; res < tdLength; res++) {
                    for (var k = 0; k < renderDates.length; k++) {
                        for (var i = (bStartHour * timeRowsCount) ; i < (bEndHour * timeRowsCount) ; i++) {
							for(var j = 0; j < this.model.workWeek.length; j++) {
                           if(this._dayNamesArray.indexOf(this.model.workWeek[j]) == new Date(renderDates[k]).getDay())
                                $($(this.element.find(".e-workcellstab tr")[res]).find(".e-workcells")[i + (((this.model.endHour - this.model.startHour) * timeRowsCount) * k)]).addClass("e-businesshighlightworkcells");
							}
                        }
                    }
                }
                this._businessHourScroller();
            }
            else {
                var monthDays = this.monthDays;
                target = this.element.find(".e-workcells"); target.removeClass("e-monthcellhighlight");
                for (var res = 0; res < tdLength; res++) {
                    for (var i = 0; i < monthDays.length; i++) {
                        for(var j = 0; j < this.model.workWeek.length; j++) {
                        if(this._dayNamesArray.indexOf(this.model.workWeek[j]) == new Date(monthDays[i]).getDay())
                            $($(this.element.find(".e-workcellstab tr")[res]).find(".e-workcells")[i]).addClass("e-monthcellhighlight");
						}
                    }
                }
            }
        },

        _onResourceClick: function (e) {
            var target = $(e.currentTarget);
            if (target.hasClass("e-resourceicon e-resourcecollapse")) {
                target.removeClass("e-resourceicon e-resourcecollapse");
                target.addClass("e-resourceicon e-resourceexpand");
            }
            else
                target.removeClass("e-resourceicon e-resourceexpand").addClass("e-resourceicon e-resourcecollapse");
            var index = $(e.currentTarget).parent().index(); var newResCollection = this._getResourceCollection();
            var query = new ej.Query().where(newResCollection[newResCollection.length - 1]["groupId"], ej.FilterOperators.equal, parseInt(e.currentTarget.id));
            var resApp = new ej.DataManager(newResCollection[newResCollection.length - 1].dataSource).executeLocal(query);
            for (var i = index + 1; i <= (resApp.length + index) ; i++) {
                $(this.element.find(".e-resourceheadertable").find("tr")[i]).toggle();
                $(this.element.find(".e-workcellstab").find("tr")[i]).toggle();
            }
            this._renderScroller();
            this.element.find("div.e-prevapp,div.e-nextapp").remove();
            this.element.find(".e-horires").css("top", "0px");
            this.element.find(".e-categorybar").height(this.element.find(".e-parentworkcell").height() - 3);
            this._businessHourScroller();
            this._renderAppointmentAll();
        },

    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {

    ej.ExcelFilter = ej.ExcelFilter || {};

    ej.excelFilter = function (options) {
        this._ctrlInstance = options["instance"];
        this.id = this._ctrlInstance._id;
        this._dialogContainer = null;
        this._showSort = options["showSortOptions"] || false;
        this._maxCount = options["maxFilterLimit"] || 1000;
        this._formatFiltering = true;
        this._locale = this._ctrlInstance.model.locale || "en-US";
        this._filterHandler = options["filterHandler"] || null;
		this._cancelHandler = options["cancelHandler"] || null;
		this._customFilterHandler = options["customFilterHandler"] || null;
        this._cssClass = options["cssClass"] || null;
        this._matchCase = options["allowCaseSensitive"] || false;
        this._title = options["title"] || this._getLocalizedLabel("title");
        this.fName = options["initFilterCol"] || null;
        this._spliter = options["valueDelimiter"] || ej.ExcelFilter.valueDelimiter;
        this._initialFName = this.fName;
        this._displayName = null;
        this._dataSource = null;
        this._isUrlAdaptor = false;
		this._$tableID = null;
        this._$blankVal = null;
        this._$selectedColors = [];
        this._$enableColor = false;
        this._$filteredByColor = "";
        this._$colType = null;
        this._$key = 0;
        this.filteredColumn = null;
        this.sortedColumns = null;        
        this._chkList = null;
        this._listsWrap = null;
        this._menuWrap = null;
        this._localJSON = null;
        this._actualCount = 0;
        this._totalRcrd = 0;
        this._enableResponsiveRow=false;
        this._searchCount = 0;
        this._currentData = null;
        this._openedFltr = null;
        this._predicates = [];
        this.cFilteredCols = this.fName!=null ? [this.fName] : [];
        this._columnsFiltered = [];
        this._noDlg = [];
        this._sepAftr = ["sortDesc","notequal", "between", "top10","endswith","contains"];
        this._posType = ["number", "date", "string", "boolean","guid"];
        this._empties = ["null", "undefined", ""];
        this._reqInProgess = false;
        this._isFiltered = false;
        this._onActionBegin = options["actionBegin"] || null;
        this._onActionComplete = options["actionComplete"] || null;
        this.enableNormalize = true; /*To clean up redundant values after formatting */
        this._selectAll = "<div class='e-ftrchk'><input type='checkbox' class='e-selectall' value='selectall' class='e-ftrchk' /><label class='e-ftrchk'>(Select All)</label></div>";
        this._blanks = "<div class='e-ftrchk'><input type='checkbox' id='blanks' class='e-ftrchk' value='null,undefined,' @/><label class='e-ftrchk' for='blanks' value=''>(Blanks)</label></div>";
        this._blank = undefined;
        this._addAtLast = false;
		 this.guidMenuOpt = [
              { id: 1, text: this._getLocalizedLabel("SortNoSmaller"), sprite: "e-sortasc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortAsc" } },
              { id: 2, text: this._getLocalizedLabel("SortNoLarger"), sprite: "e-sortdesc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortDesc" } },
              { id: 3, text: this._getLocalizedLabel("SortByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("SortColorOptions") },
              { id: 4, text: this._getLocalizedLabel("ClearFilter"), sprite: "e-filternone e-icon", htmlAttribute: { "ejfnrole": "clearfilter" } },
              { id: 5, text: this._getLocalizedLabel("FilterByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("FilterColorOptions") },
              { id: 6, text: this._getLocalizedLabel("GuidFilter"), htmlAttribute: { "ejfnrole": "filterpopup" }, child: this._getLocalizedLabel("GuidMenuOptions") }];
        this.numberMenuOpt = [
              { id: 1, text: this._getLocalizedLabel("SortNoSmaller"), sprite: "e-sortasc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortAsc" } },
              { id: 2, text: this._getLocalizedLabel("SortNoLarger"), sprite: "e-sortdesc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortDesc" } },
              { id: 3, text: this._getLocalizedLabel("SortByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("SortColorOptions") },
              { id: 4, text: this._getLocalizedLabel("ClearFilter"), sprite: "e-filternone e-icon", htmlAttribute: { "ejfnrole": "clearfilter" } },
              { id: 5, text: this._getLocalizedLabel("FilterByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("FilterColorOptions") },
              { id: 6, text: this._getLocalizedLabel("NumberFilter"), htmlAttribute: { "ejfnrole": "filterpopup" }, child: this._getLocalizedLabel("NumberMenuOptions") }];
        this.stringMenuOpt  = this.booleanMenuOpt = [
              { id: 1, text: this._getLocalizedLabel("SortTextAscending"), sprite: "e-sortasc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortAsc" } },
              { id: 2, text: this._getLocalizedLabel("SortTextDescending"), sprite: "e-sortdesc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortDesc" } },
              { id: 3, text: this._getLocalizedLabel("SortByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("SortColorOptions") },
              { id: 4, text: this._getLocalizedLabel("ClearFilter"), sprite: "e-filternone e-icon", htmlAttribute: { "ejfnrole": "clearfilter" } },
              { id: 5, text: this._getLocalizedLabel("FilterByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("FilterColorOptions") },
              { id: 6, text: this._getLocalizedLabel("TextFilter"), htmlAttribute: { "ejfnrole": "filterpopup" }, child: this._getLocalizedLabel("StringMenuOptions") }];
        this.dateMenuOpt = [
              { id: 1, text: this._getLocalizedLabel("SortDateOldest"), sprite: "e-sortasc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortAsc" } },
              { id: 2, text: this._getLocalizedLabel("SortDateNewest"), sprite: "e-sortdesc e-icon e-fnsort", htmlAttribute: { "ejfnrole": "sortDesc" } },
              { id: 3, text: this._getLocalizedLabel("SortByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("SortColorOptions") },
              { id: 4, text: this._getLocalizedLabel("ClearFilter"), sprite: "e-filternone e-icon", htmlAttribute: { "ejfnrole": "clearfilter" } },
              { id: 5, text: this._getLocalizedLabel("FilterByColor"),htmlAttribute: { "ejfnrole": "popup" }, child: this._getLocalizedLabel("FilterColorOptions") },
              { id: 6, text: this._getLocalizedLabel("DateFilter"), htmlAttribute: { "ejfnrole": "filterpopup" }, child: this._getLocalizedLabel("DateMenuOptions") }
        ];
        
        return this;
    };
    ej.excelFilter.prototype = {
        //Helpers to handle (Blanks) value.
        isNotBlank: function (key, global) {
            var e = ej.isNullOrUndefined(key) || key === "";

            if (!global && key === "")/* Handle special case - since "" string sorted before all */
                this._addAtLast = true;

            return !e;
        },   
        _checkBlank: function (key) {         

            if (this.isNotBlank(key))
                return true;
            /*Ensure Blank value added only once */
            var ret = this._blank == undefined && !this._addAtLast;

            if (ret) this._blank = true;            

            return ret;
        },
        _getValueData: function (key, data) {
            var arr = this._empties;

            if (this.isNotBlank(key, true))
                arr = ej.distinct(data, this._$foreignField || this.fName, false);

            return arr.join(this._spliter); /*Return value will be set to input element value attr*/
        },
        //Collection to predicate processing
        getPredicate: function (cols, field, updateModel) {
            var c, dis = field != undefined ? [field] : ej.distinct(cols, "field", false), collection, pred = {};
            for (var f = 0, flen = dis.length; f < flen; f++) {
                collection = new ej.DataManager(cols).executeLocal(new ej.Query().where("field", "equal", dis[f]));
                pred[dis[f]] = this.generatePredicate(collection);
                if (updateModel) {
                    this._predicates[this._$key] = this._predicates[this._$key] || {};
                    this._predicates[this._$key][dis[f]] = pred[dis[f]];
                }

            }
            return pred;
        },

        generatePredicate: function (cols) {
            var len = cols ? cols.length : 0, predicate, first;
            if (!len) return;
            first = this._updateDateFilter(cols[0]); 
            predicate = first.ejpredicate ? first.ejpredicate : ej.Predicate(first.field, first.operator, first.value, first.ignoreCase || !first.matchcase);
            for (var p = 1; p < len; p++) {
                cols[p] = this._updateDateFilter(cols[p]);
                if (this._isUrlAdaptor && len > 2 && p > 1 && cols[p].predicate == "or")
                    predicate.predicates.push(ej.Predicate(cols[p].field, cols[p].operator, cols[p].value, cols[p].ignoreCase || !cols[p].matchcase));
                else
                    predicate = cols[p].ejpredicate ? predicate[cols[p].predicate](cols[p].ejpredicate) : predicate[cols[p].predicate](cols[p].field, cols[p].operator, cols[p].value, cols[p].ignoreCase || !cols[p].matchcase);
            }
            return predicate || null;
        },
        /*Main Dialog*/
        renderDialog: function (type) {
            this._$colType = type;
            var dlgId = this.id + type + "_excelDlg";           
            //if ($("#" + dlgId).length != 0)
            //    return;
            var $dlg = ej.buildTag("div#" + dlgId + ".e-excelfilter e-dlgcontainer e-shadow");
            var $ul = ej.buildTag("ul#" + this.id + type + "_MenuItem");
            var menuData = this._getMenuData(type);
            $ul = this._createLiTag($ul, menuData, false);
            var $searchBox = ej.buildTag("span.e-searchbox e-fields").append(ej.buildTag("input#" + this.id + "_SearchBox.e-ejinputtext e-searchinput", {}, {}, { "type": "text", "placeholder":"Search" })).append(ej.buildTag("span.e-search e-icon"))
            var $lbox = ej.buildTag("div#" + this.id + type + "_CheckBoxList.e-checkboxlist e-fields").append(ej.buildTag("div"));
            var $btns = this._createBtn();
            var $sBox = ej.buildTag("div.e-searchcontainer");            
            var $status = ej.buildTag("div.e-status e-fields e-hide", this._getLocalizedLabel("CheckBoxStatusMsg"));
            $dlg.append($ul);            
            $sBox.append($searchBox);
            $sBox.append($status);
            $sBox.append($lbox);
            $sBox.append($btns);
            $dlg.append($sBox);
            $dlg.appendTo(this._ctrlInstance.element);
            $dlg.css("display", "none");
            this._renderCustomFDlg(type);
            this._dialogContainer = $dlg;
            if(this._cssClass!=null)
                $dlg.addClass(this._cssClass);
            if (!this._showSort) {
                $ul.find(".e-fnsort").closest("li").css("display", "none");
                $ul.find("li.e-separator:first").css("display", "none");
            }
            this._lsitBoxTemplate();
            this._renderSubCtrls(type);
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 10)
                ej.ieClearRemover($searchBox.find("input")[0]);
            this._wireEvents();
        },
        _renderSubCtrls: function (type) {
            $("#" + this.id + type + "_MenuItem").ejMenu({ orientation: "vertical", width: 266, click: ej.proxy(this._menuHandler, this), enableRTL: this._ctrlInstance.model.enableRTL, enableSeparator: false });
            $("#" + this.id + type + "_OkBtn").ejButton({ text: this._getLocalizedLabel("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._fltrBtnHandler, this), enabled: true });
            $("#" + this.id + type + "_CancelBtn").ejButton({ text: this._getLocalizedLabel("Cancel"), showRoundedCorner: true, width: 60, click: ej.proxy(this.closeXFDialog, this) });
            $("#" + this.id + type + "_CheckBoxList").ejScroller({ height: 130, width: 234, scroll: ej.proxy(this._virtualize,this) });
            $("#" + this.id + type + "_CheckBoxList").ejWaitingPopup({ showOnInit: false });
        },
        openXFDialog: function (options) {           
            this.fName = options["field"];
            this._dataSource = options["dataSource"];
            this._$colType = options["type"] || "string";
            this._$format = options["format"] || "";
            this._enableResponsiveRow=options["enableResponsiveRow"];
            this._isUrlAdaptor = this._dataSource instanceof ej.DataManager && (this._dataSource.adaptor instanceof ej.UrlAdaptor || this._dataSource.adaptor instanceof ej.WebMethodAdaptor);
            this.filteredColumn = options["filteredColumns"] || this._ctrlInstance.model.filterSettings.filteredColumns;
            this.sortedColumns = options["sortedColumns"] || this._ctrlInstance.model.sortSettings.sortedColumns;
            this._displayName = options["displayName"];
            this._$key = options["key"] || 0;
			this._$tableID = options["tableID"];
			this._$blankVal = ej.isNullOrUndefined(this._$tableID) ? this._$blankVal : options["blank"];
            this._$selectedColors = options["selectedColors"] || [];
            this._$enableColor = options["enableColor"] || false;
            this._$filteredByColor = options["filteredByColor"] || this._$filteredByColor;
			this._$foreignField = options["foreignKeyValue"];
			this._$foreignData = options["foreignDataSource"];
            var args = { requestType : "filterbeforeopen", filterModel : this, columnName:this.fName, columnType: this._$colType };
            if (this._ctrlInstance._trigger(this._onActionBegin, args))
                return;
            if (this._openedFltr == null || !this._openedFltr.is($("#" + this.id + this._$colType + "_excelDlg"))) {
                this.closeXFDialog();
                this._openedFltr = $("#" + this.id + this._$colType + "_excelDlg");
            }
            this._listsWrap = $("#" + this.id + this._$colType + "_CheckBoxList");
            this._menuWrap = $("#" + this.id + this._$colType + "_MenuItem");
            this._searchBox = this._openedFltr.find(".e-searchbox input");
            this._setPosition(this._openedFltr, options["position"]);
            this._openedFltr.addClass(options["cssClass"]);
            this._openedFltr.fadeIn(300, "easeOutQuad", function () {
            });
			var $sortul = $("#" + this.id + this._$colType + "_MenuItem").find(".e-shadow").eq(0);
            var $filterul = $("#" + this.id + this._$colType + "_MenuItem").find(".e-shadow").eq(1);
			if (this._$enableColor) {
                this._createDivTag($sortul, this._$selectedColors, false, "sort");
                (this._$filteredByColor == -1 || (this._$filteredByColor.length < 1 || this._$filteredByColor === this.fName)) ? this._createDivTag($filterul,this._$selectedColors, false, "filter") : $filterul.parent().addClass("e-disable-item");
            }    
            else {
                $sortul.parent().hide();
                $filterul.parent().hide();
            }
            this._isFiltered = this._predicates[this._$key] != undefined && this._predicates[this._$key][this.fName] != undefined;
            this._isFiltered = options["isFiltered"] || this._isFiltered;
			this._$format = (this._$colType == "date" && this._$format === "") ? "{0:MM/dd/yyyy hh:mm:ss}" : this._$format;
            this._processListData();            
            this._actualCount > this._maxCount ? this._openedFltr.find("div.e-status").removeClass("e-hide") : this._openedFltr.find("div.e-status").addClass("e-hide");            
            this._setDisable();
            var args = { requestType: "filterafteropen", filterModel: this, columnName: this.fName, columnType: this._$colType };
            if (this._ctrlInstance._trigger(this._onActionComplete, args))
                return;
        },
        closeXFDialog: function (e) {
            if (e != null) {
                var $target = $(e.target);
                if ($target.closest("#"+ this.id + this._$colType + "_CustomFDlg").length)
                    return;
            }            
            if (this._openedFltr) {
                if (!this._openedFltr.hasClass("e-dlgcustom")) {
                    this._openedFltr.fadeOut(300, "easeOutQuad", function () {
                    });
                    this._listsWrap.ejWaitingPopup("hide");
                }
                else {
                    this._openedFltr.ejDialog("close");
                }
                !ej.isNullOrUndefined(this._cancelHandler) && this._cancelHandler();
                this.resetFilterModel();/*Reset private properties after filter closed*/
            }
        },
        _setPosition: function (ele, pos) {
            ele.css("position", "absolute");
            ele.css("left", pos["X"]).css("top", pos["Y"]);                      
        },
        _setDisable: function () {
            var $clr = this._menuWrap.find("li[ejfnrole='clearfilter']"), $fltr = this._menuWrap.find("li[ejfnrole='filterpopup']");
            var $sort = this._menuWrap.find("li[ejfnrole *= 'sort']"), $checked = this._listsWrap.find("input").filter(":checked:not(.e-selectall)").length;
            !this._isFiltered ? $clr.addClass("e-disable-item") : $clr.removeClass("e-disable-item");
            if (this._showSort) {
                var sQM = ej.DataManager(this.sortedColumns).executeLocal(new ej.Query().where("field", "equal", this.fName));
                ( sQM.length && sQM[0]["direction"] == "ascending") ? $sort.filter("[ejfnrole='sortAsc']").addClass("e-disable-item") : $sort.filter("[ejfnrole='sortAsc']").removeClass("e-disable-item");
                (sQM.length && sQM[0]["direction"] == "descending") ? $sort.filter("[ejfnrole='sortDesc']").addClass("e-disable-item") : $sort.filter("[ejfnrole ='sortDesc']").removeClass("e-disable-item");
            }
            var $flteredList = $fltr.find(".aschild");
            if (this.cFilteredCols.length != 0 && $.inArray(this.fName,this.cFilteredCols) != -1) {
                for (var f = 0; f < this.filteredColumn.length; f++) {
                    if (this.filteredColumn[f].field == this.fName) {
                        if ($flteredList.find("#ejFiltercheck").length == 0) {
                            $flteredList.append("<input type='checkbox' id='ejFiltercheck' />");
                            $flteredList.find("#ejFiltercheck").ejCheckBox({ checked: true });
                            $flteredList.find("#ejFiltercheck").ejCheckBox("disable");
                        }
                        if ($fltr.find(".e-shadow .e-exceltick").length > 0)
                            $fltr.find(".e-shadow .e-exceltick").remove();
                        if (this.filteredColumn[f].field == this.fName && !ej.isNullOrUndefined(this.filteredColumn[f + 1]) && this.filteredColumn[f + 1].field == this.fName) {
                            if (this.filteredColumn[f].operator == "greaterthanorequal" && this.filteredColumn[f + 1].operator == "lessthanorequal")
                                $fltr.find(".e-shadow .e-list[ejvalue=between]").find("a").append("<span class='e-exceltick e-icon' />");
                            else
                                $fltr.find(".e-shadow .e-list[ejvalue=customfilter]").find("a").append("<span class='e-exceltick e-icon' />");
                        }
                        else
                            $fltr.find(".e-shadow .e-list[ejvalue=" + this.filteredColumn[f].operator + "]").find("a").append("<span class='e-exceltick e-icon' />");
                        break;
                    }
                }
            }
            else {
                for (var f = 0; f < this.filteredColumn.length; f++) {
                    if (this.filteredColumn[f].field == this.fName) {
                        this._removeTick($fltr,$flteredList);
                        break;
                    }
                }
            }
            if (!this._isFiltered && $flteredList.find("#ejFiltercheck").length != 0) {
                 this._removeTick($fltr,$flteredList);
            }
            if (this._$colType == "boolean")
                this._menuWrap.find("li[aria-haspopup=true]").addClass("e-hide");            
            this._searchBox.val(""); this._searchBox.siblings().addClass("e-search").removeClass("e-cancel");
        },
        _removeTick: function($fltr,$flteredList){
            $flteredList.find("#ejFiltercheck").ejCheckBox("destroy");
            $fltr.find(".aschild #ejFiltercheck").remove();
            $fltr.find(".e-shadow .e-exceltick.e-icon").remove();
        },
        _createBtn: function (isCDlg) {
            var id = isCDlg ? this.id + this._$colType + "Custom" : this.id + this._$colType;
            var $divCon = ej.buildTag("div.e-btncontainer e-fields");
            var $div = ej.buildTag("div");
           $div.append(ej.buildTag("input#" + id + "_OkBtn.e-fltrbtn",{},{},{ type : "button" })).append(ej.buildTag("input#" + id + "_CancelBtn.e-fltrbtn",{},{},{ type : "button" }));
            return $divCon.append($div);
        },
        _menuHandler: function (args) {
            var $ele = $(args.element);
            var role = $ele.attr("ejfnrole");
            var arg = {};
            if (role === "filterbgcolor" || role === "filterfgcolor") {
                var fDetails = { field: this.fName, operation: role, color: $(args.element).css('background-color')};
			    arg = { originalEvent: args.event, action: "filterbycolor", filterDetails: fDetails, tableID: this._$tableID }; 
                this._isFiltered = false;
            }
            else if (role == "clearfilter") {
                var ftrDetails = { field: this.fName, operator: "", value: "", predicate: "or" };
                arg = { originalEvent: args.event, fieldName: this.fName, action: "clearfiltering", filterDetails: ftrDetails, tableID: this._$tableID };
                var indx = $.inArray(this.fName, this.cFilteredCols);
                indx!=-1 && this.cFilteredCols.splice(indx, 1);
				var $cIndx = $.inArray(this.fName, this._columnsFiltered);
                $cIndx!=-1 && this._columnsFiltered.splice($cIndx, 1);
                if (this._initialFName == this.fName)
                    this._initialFName = null;
            }
            else if(role == "popup" || role == "filterpopup") {
                return;
            }
            else if (role == "operator") {
                this._openCustomFilter($ele.attr("ejvalue"));
            }
			else{
                var sOrder, sDetails;
				if(role == "sortAsc" || role == "sortDesc"){
					sOrder = role == "sortAsc" ? "ascending" : "descending";
					sDetails = { field: this.fName, direction: sOrder };
					arg = { originalEvent: args.event, action: "sorting", sortDetails: sDetails, tableID: this._$tableID };
				}
				else{
					sDetails = { field: this.fName, direction: sOrder, operation: role, color: $(args.element).css('background-color')};
					arg = { originalEvent: args.event, action: "sortbycolor", sortDetails: sDetails, tableID: this._$tableID }; 
				}
            }
            if (role != "operator") {
                this._filterHandler(arg);
                this.closeXFDialog();
            }
        },
        _searchBoxFocus: function (e) {
            var $target = $(e.target);
            if ($target.hasClass("e-cancel")) {
                $target.prev().val("");
                $target.next().addClass("e-cancel")
                $target.addClass("e-search");
                $target.prev().trigger("keyup");
            }
            if (e.type == "focusin") {
                $target.next().addClass("e-cancel");
                $target.next().removeClass("e-search");
            }
        },
        _search: function (e) {
            var enterText = e.target.value, args = {}, parsed, operator, $target = $(e.target);
            parsed = parseFloat(enterText) ? parseFloat(enterText) : enterText;
            operator ="contains" ;
            parsed = (parsed == "" || parsed == undefined) ? undefined : parsed;
            if (this._$colType == "boolean" && this._locale != "en-US") {
                if (parsed != undefined && this._getLocalizedLabel("True").toLocaleLowerCase().indexOf(parsed.toLowerCase()) != -1)
                    parsed = "true";
                else if (parsed != undefined && this._getLocalizedLabel("False").toLocaleLowerCase().indexOf(parsed.toLowerCase()) != -1)
                    parsed = "false";
            }
            if (this._$colType == "date" || this._$colType == "dateTime") {
                parsed = Globalize.parseDate(enterText, this.replacer(this._$format, /{0:|}/g, ""));
                operator = "equal";
                if (this._previousValue == null && parsed == null)
                    return;
            }
            this._previousValue = parsed;
            this._processListData({ type: "filterchoicesearch", value: parsed, operator: operator });
            if ($target.val() == "") {
                $target.next().addClass("e-search");
                $target.next().removeClass("e-cancel");
            } else {
                $target.next().addClass("e-cancel");
                $target.next().removeClass("e-search");
            }
        },
        _getLocalizedLabel: function (property) {
            return (ej.ExcelFilter.locale[this._locale] !== undefined && ej.ExcelFilter.locale[this._locale][property] !== undefined) ? ej.ExcelFilter.locale[this._locale][property] : ej.ExcelFilter.locale["en-US"][property];
        },
        _getMenuData: function (type) {
            return type != undefined ? this[type + "MenuOpt"] : [];
        },
        /*CheckBox list*/
        _lsitBoxTemplate: function () {
            var helper = {}, temp = {};            
            helper[this.id + "isNotBlank"] = ej.proxy(this.isNotBlank, this);
            helper[this.id + "checkBlank"] = ej.proxy(this._checkBlank, this);
            helper[this.id + "_getValueData"] = ej.proxy(this._getValueData, this);            
            helper[this.id + "_checkBoxState"] = ej.proxy(this._setCheckState, this);     
            helper[this.id + "_genId"] = this._genCheckID;
            $.views.helpers(helper); 			
            var genID = this.id + this._$colType + "{{:~" + this.id + "_genId()}}";
            temp[this.id + this._$colType + "_listBox_Template"] = "{{if ~" + this.id + "checkBlank(key)}}<div class='e-ftrchk'><input type='checkbox' id='" + genID + "' value='{{html:~" + this.id + "_getValueData(key, items)}}' class='e-ftrchk' {{:~" + this.id + "_checkBoxState(~" + this.id + "_getValueData(key, items))}}/><label class='e-ftrchk' for='" + genID + "'>{{if ~" + this.id + "isNotBlank(key,'true')}} {{>key}} {{else}} (Blanks) {{/if}}</label></div>{{/if}}";
            $.templates(temp);
        },
      
        _processListData: function (params) {
            var result, promise, args = {}, query = new ej.Query(), evtArgs = {}; this._searchCount = 0;
            var predicates = this._predicates[this._$key], pred;
			var data = null, columnName = null, localJSON = null, result = null;
			if(ej.isNullOrUndefined(this._$foreignField && this._$foreignData)){
				data = this._dataSource; 
				columnName =  this.fName;
				localJSON = this._localJSON;
			}
			else{
				data = localJSON = this._$foreignData; 
				if(this._dataSource instanceof ej.DataManager)
                    data = ej.DataManager(data);
				columnName = this._$foreignField;
			}
            for (var prop in predicates) {
                if (prop == this.fName)
                    continue;
                var obj = predicates[prop], from = obj["from"];
                if (from)
                    query.skip(from == "top" ? 0 : data.length - (data.length - obj["take"])).take(obj["take"]);
                else
                    pred = pred != undefined ? pred["and"](obj) : obj;
            }
            args.columnName = columnName;           
            query.requiresCount();						
				pred && query.where(pred);
            evtArgs.requestType = params ? params.type : "filterchoicerequest",evtArgs.filterModel = this, evtArgs.query = query, evtArgs.dataSource = data;
            if (this._ctrlInstance._trigger(this._onActionBegin, evtArgs))
                return;
            if (params && params.type == "filterchoicesearch") {
                params.value && query.where(columnName, params.operator, params.value, !this._matchCase);
                promise = ej.DataManager(localJSON).executeLocal(query);               
                args.data = this._currentData = promise.result;
                args.type = params.type;
                this._totalRcrd = this._searchCount =  promise.count;
                this._setCheckBoxList(args);
            }
            else if (!(this._dataSource instanceof ej.DataManager)) {
				var result = [];
                promise = ej.DataManager(this._dataSource).executeLocal(query);
                args.type = "filterchoicerequest";
				if(this._$foreignField)
					result = this._filterForeignData(promise.result);				
				else				
					result = promise.result;
                args.data = this._localJSON = this._currentData = result;
                this._totalRcrd = promise.count;
                this._setCheckBoxList(args);
            }
            else {
                proxy = this; args.type = "filterchoicerequest";
                this._listsWrap.ejWaitingPopup("show");
                if (!this._reqInProgess) {
                    this._reqInProgess = true;
					if(this._$foreignField && pred)
                        data = this._dataSource;
                    promise = data.executeQuery(query);
                    promise.done(function (e) {
						if(proxy._$foreignField  && pred)                            
                            result = proxy._filterForeignData(e.result);
                        else
                            result = e.result;
                        args.data = proxy._localJSON = proxy._currentData = result;
                        proxy._totalRcrd = e.count;
                        proxy._listsWrap.ejWaitingPopup("hide");
                        proxy._setCheckBoxList(args);
                        proxy._reqInProgess = false;
                    });
                }
            }           
        },		
        _filterForeignData: function($results){
            var result =[], proxy = this;
            var values = ej.distinct(ej.DataManager($results).executeLocal(ej.Query().select(this.fName)));
			values.forEach(function(val){
				result.push(ej.DataManager(proxy._$foreignData).executeLocal(ej.Query().where(proxy.fName, "equal", val))[0]);
			});
            return result;
        },
        _setCheckBoxList: function (args) {
            var evtArgs = {}, _blank ="", sortedData, flag, blank = this._blanks, $checked;
            evtArgs.requestType = args.type, evtArgs.dataSource = this._dataSource, evtArgs.filterModel = this;
            if (this._currentData.length != 0) {                
                sortedData = this.getDistinct(args.data, args.columnName, true);
                flag = this._isFiltered;                
                this._actualCount = sortedData.length; sortedData.length = this._maxCount > this._actualCount ? this._actualCount : this._maxCount;               
                this._filterdCol = ej.DataManager(this.filteredColumn).executeLocal(ej.Query().where("field", "equal", this.fName));                                    
                this._listsWrap.find("div:first").html([this._selectAll, $.render[this.id + this._$colType + "_listBox_Template"](sortedData), this._addAtLast ? this.replacer(blank, /@/g, this._setCheckState, this._empties.join(this._spliter)) : ""].join(""));
                this._chkList = this._listsWrap.find("input:checkbox").not(".e-selectall"), $inView = this._chkList.slice(0, 20);
                $inView.ejCheckBox({ change: ej.proxy(this._checkHandler, this) });
                $inView.siblings().height(14).width(14);
                this._listsWrap.find(".e-selectall").ejCheckBox({  change: ej.proxy(this._selectAllHandler, this) });
				this._listsWrap.find(".e-selectall").attr("id",this.id + this._$colType + "SelectAll");
                this._listsWrap.find(".e-selectall").siblings().height(14).width(14);
				this._listsWrap.find(".e-selectall").closest("span").siblings("label").attr("for", this.id + this._$colType + "SelectAll");
            }
            else
                this._listsWrap.find("div:first").html(ej.buildTag("div.e-ftrchk", this._getLocalizedLabel("NoResult"), {}, {}));
            if (!ej.isNullOrUndefined(this._chkList))
                $checked = this._chkList.filter(":checked").length;
            if (!this._isFiltered || this._actualCount == $checked)
                this._listsWrap.find(".e-selectall").ejCheckBox({ checked: true });
            $("#" + this.id + this._$colType + "_OkBtn").ejButton({ enabled: $checked != 0 });
            this._listsWrap.ejScroller({ scrollTop: 0 }).ejScroller("refresh");
            if (this._ctrlInstance._trigger(this._onActionComplete, evtArgs))
                return;
        },
     
        _createLiTag: function ($ul, menuData, isChild) {
            proxy = this;
            $.each(menuData, function (index, obj) {
                var $li = ej.buildTag("li", {}, {}, obj["htmlAttribute"] || (isChild && { "ejfnrole": "operator", "ejvalue": obj.value }) || {}), $child;
                var apd = (isChild && $.inArray(obj.value,proxy._noDlg)==-1) ? "..." : "";
                var $a = ej.buildTag("a", obj.text + apd, {}, {});
                obj["sprite"] != undefined && $a.append(ej.buildTag("span", {}, {}, { "class": obj["sprite"] }))
                if (obj["child"] != undefined){
                    if(obj.id != 3 && obj.id != 5)
                        $child = proxy._createLiTag(ej.buildTag("ul.e-shadow"), obj["child"], true);
                    else if(obj.id === 3 )
                        $child = proxy._createDivTag(ej.buildTag("ul.e-shadow"), obj["child"], true, "sort");
                    else
                        $child = proxy._createDivTag(ej.buildTag("ul.e-shadow"), obj["child"], true, "filter");
				}
                obj["child"] == undefined ? $ul.append($li.append($a)) : $ul.append($li.append($a).append($child))
                if ($.inArray(obj.value || (obj.htmlAttribute && obj.htmlAttribute.ejfnrole), proxy._sepAftr) != -1)
                    $ul.append(ej.buildTag("li.e-separator"));
            });
            return $ul;
        },
		
		_createDivTag: function ($ul, menuData, isChild, reqType) {
            var $li, $a, $div, bgColor = [], fgColor = [], avble = false, cellcolor = "", fontcolor = "", selCellHdr = "", selCellClr = "", selFontHdr = "", selFontClr = "";
            if (reqType == "sort") {
                cellcolor = this._getLocalizedLabel("SortByCellColor");
                fontcolor = this._getLocalizedLabel("SortByFontColor");
            }
            else{
                cellcolor = this._getLocalizedLabel("FilterByCellColor");
                fontcolor = this._getLocalizedLabel("FilterByFontColor");
            }
			selCellHdr = reqType + "colorhdr";
            selFontHdr = reqType + "fonthdr";
            selCellClr = reqType + "bgcolor";
            selFontClr = reqType + "fgcolor";
            ($ul.length > 0) && $ul.children().remove();
            if(menuData.length > 0){
			    for(var i = 0; i < menuData.length; i++){
				    for(var j = 0; j < bgColor.length; j++){
					    if(bgColor[j] == menuData[i].background){
						    avble = true;
						    break;
					    }
				    }
				    if(!avble && (!ej.isNullOrUndefined(menuData[i].background) && !menuData[i].background.startsWith('#6n'))) 
                        bgColor.push(menuData[i].background);
				    avble = false;
				    for(var j = 0; j < fgColor.length; j++){
					    if(fgColor[j] == menuData[i].foreground){
						    avble = true;
						    break;
					    }
				    }
				    if(!avble && (!ej.isNullOrUndefined(menuData[i].foreground) && !menuData[i].foreground.startsWith('#6n'))) 
                        fgColor.push(menuData[i].foreground);
				    avble = false;
			    }
                if(bgColor.length > 0){
                    $li = ej.buildTag("li.e-list e-bghdrcolor", "", "" , {"ejfnrole": selCellHdr });
                    $a = ej.buildTag("a", cellcolor, {});
                    $li.append($a);
                    $ul.append($li);
			        for(var i = 0; i < bgColor.length; i++){
                        $li = ej.buildTag("li.e-list e-valcolor", {}, {'background-color': bgColor[i]}, {"ejfnrole": selCellClr });
                        $ul.append($li);
			        }
                }
                if(fgColor.length > 0){
			        $li = ej.buildTag("li.e-list e-fghdrcolor", "", "" , {"ejfnrole": selFontHdr });
                    $a = ej.buildTag("a", fontcolor, {});
                    $li.append($a);
                    $ul.append($li);
                    for(var i = 0; i < fgColor.length; i++) {
                        $li = ej.buildTag("li.e-list e-valcolor", {}, {'background-color': fgColor[i]}, {"ejfnrole": selFontClr });
                        $ul.append($li);
                    }
                }
                (bgColor.length < 1 && fgColor.length < 1) ? $ul.parent().addClass("e-disable-item"): $ul.parent().removeClass("e-disable-item");
            }
            else {
                $ul.parent().addClass("e-disable-item");
            }
            return $ul;
        },
		
        _setCheckState: function (value) {           
            var val = value, fobj, splts = value.split(this._spliter), splen = splts.length, flag = false;           
            if (!this._isFiltered || this._searchCount)
                return "checked";
            else {
                while (splen--) { /*looped for split values*/
                    val = this.processValue(splts[splen]);

                    if (this._$foreignField)
                        val = this._getForeignFilterData(val);
                    //Date and boolean will be checked as iteration due to 
                    if (this._$colType == "date" || this._$colType == "datetime" || this._$colType == "boolean") {
                        for (var i = 0, len = this._filterdCol.length; i < len; i++) {
                            if (this._$colType == "boolean" && val !== this._filterdCol[i].value)
                                continue;
                            if((this._$colType == "date" || this._$colType == "datetime") && !this._dateComparer(val, this._filterdCol[i].value))
                                continue;
                            return this._filterdCol[i]["isCustom"] === true ? "" : "checked";
                        }
                    }
                    else {
                        var fQ = ej.DataManager(this._filterdCol).executeLocal(ej.Query().where("value", "equal", val).where("operator", "equal", "equal"));
                        if (fQ.length != 0)
                            return fQ[0]["isCustom"] === true ? "" : "checked";
                    }
                }
            }
        },
        _genCheckID: function () {
            return "CheckBox" + this.getIndex();
        },
        _formatting: function (format, value) {

            if (this._$colType == "date" && format == "")
                format = "{0:MM/dd/yyyy hh:mm:ss}";

            if (this._$colType == "boolean")                 
                return value === "" ? "" : this._getLocalizedLabel(value == true ? "True" : value == false ? "False" : "");            
           
            if (this._formatFiltering || this._$colType == "date") {
                var formatter = this._ctrlInstance.formatting;                
                return format != "" ? formatter(format, value) : value;
            } 
        },
        //Method to perform equality comparison between Date objects
        _dateComparer: function (dest, source) {
            return +dest === +source;
        },
        _updateDateFilter: function (filter) {
            if (filter.type != "date" && !(filter.value instanceof Date))
                return filter;            
            filter.value = filter.value instanceof Date ? filter.value : ej.parseJSON({ val: filter.value }).val;
            return ["equal", "notequal"].indexOf(filter.operator) == -1 ? filter : this._setDateObject(filter);
        },        
        _checkHandler: function (args) {
            $("#" + args.model.id).prop("checked", args.isChecked);
            var $checkedlist = this._listsWrap.find("input").filter(":checked:not(.e-selectall)"), clen = $checkedlist.length;
            var $selectall = this._listsWrap.find(".e-selectall"), canCheck;
            if (clen == this._chkList.length) {
                $selectall.prop("checked", true);
                canCheck = true;
            }
            else {
                $selectall.removeProp("checked");
                canCheck = false;
            }
            $selectall.ejCheckBox({ checked: canCheck });
            $("#" + this.id + this._$colType + "_OkBtn").ejButton({ enabled: clen != 0 });
        },
        _selectAllHandler: function (args) {
            
            if (args.isChecked) {
                this._chkList.filter(":not(:checked)").ejCheckBox({ checked: args.isChecked });
                this._chkList.prop("checked", true);
            }
            else {
                this._chkList.filter(function () { if ($(this).hasClass("e-checkbox") && $(this).prop("checked")) return this; }).ejCheckBox({ checked: args.isChecked });
                this._chkList.removeProp("checked"); this._chkList.removeAttr("checked");
            }
            $("#" + this.id + this._$colType + "_OkBtn").ejButton({ enabled: args.isChecked });
        },
        /*Custom Dialog*/
        _renderCustomFDlg: function (type) {
            var dlgId = this.id + type + "_CustomFDlg";
            if ($("#" + dlgId).length)
                return;
            var $dlg = ej.buildTag("div#" + dlgId + ".e-excelfilter e-dlgcustom");
            $dlg.addClass(this._cssClass);
            this._renderCDlgContent($dlg, type);
            $dlg.ejDialog({ showOnInit: false, enableResize: false, enableModal:true, allowKeyboardNavigation: false, title: this._title, width: 370,content:"#" + this.id, enableRTL:this._ctrlInstance.model.enableRTL , closeIconTooltip:"Close" });
			$dlg.ejDialog("refresh");
        },

        _renderCDlgContent: function (content, type) {
            var $div1 = ej.buildTag("div.e-dlgfields", "Show rows where", {}, {});
            var $id = this.id + this._$colType;
            var $fset = ej.buildTag("fieldset.e-fieldset");
            var $op1 = ej.buildTag("tr.e-fields"), $op2 = ej.buildTag("tr.e-fields"), $op3 = ej.buildTag("tr.e-fields e-top");
            var $dp1 = ej.buildTag("input#" + $id + "_CustomDrop1"), $dp2 = ej.buildTag("input#" + $id + "_CustomDrop2"), $dp3 = ej.buildTag("input#" + $id + "_CustomDrop3");
            var $in1 = ej.buildTag("input#" + $id + "_CustomValue1.e-ejinputtext e-excustmfltr", {}, {}, { "type": "text" }), $in2 = ej.buildTag("input#" + $id + "_CustomValue2.e-ejinputtext e-excustmfltr", {}, {}, { "type": "text" }), $in3 = ej.buildTag("input#" + $id + "_CustomValue3.e-ejinputtext e-excustmfltr", {}, {}, { "type": "text" });
            var $pred = ej.buildTag("tr.e-predicate");
            var radioAnd = ej.buildTag("input#" + $id + "_CustomAndPredicate", {}, {}, { "type": "radio", "name": "predicate" , "value":"and" }), radioOr = ej.buildTag("input#" + $id + "_CustomOrPredicate", {}, {}, { "type": "radio", "name": "predicate", "value":"or" });
            var matchCase = ej.buildTag("input#" + $id + "_CustomMatchPredicate", {}, {}, { "type": "checkbox" });
            content.append($div1);
            var table = ej.buildTag("table.e-optable");

            $op1.append(ej.buildTag("td.e-operator").append($dp1))
                .append(ej.buildTag("td.e-value").append($in1));
            $pred.append(ej.buildTag("td", {}, {}, {}).append(radioAnd).append(ej.buildTag("label.e-caption", this._getLocalizedLabel("PredicateAnd"), {}, { "for": $id + "_CustomAndPredicate" }))
                .append(radioOr).append(ej.buildTag("label.e-caption", this._getLocalizedLabel("PredicateOr"), {}, { "for": $id + "_CustomOrPredicate" })));
            $op2.append(ej.buildTag("td.e-operator").append($dp2))
                .append(ej.buildTag("td.e-value").append($in2));
            table.append($op1)
            .append($pred)
            .append($op2);
			if(type == "string")
			    $pred.append(ej.buildTag("td", {}, {}, {}).append(matchCase).append(ej.buildTag("label.e-caption", this._getLocalizedLabel("MatchCase"), {}, { "for": $id + "_CustomMatchPredicate" })));
            if (type == "number" || type == "guid") {
                $op3.append(ej.buildTag("td.e-operator").append($dp3))
                .append(ej.buildTag("td.e-value").append($in3));
                table.append($op3)
            }
            $fset.append(ej.buildTag("legend"))
                .append(table)
            content.append(ej.buildTag("div.e-dlgfields").append($fset));
            content.append(ej.buildTag("div.e-dlgfields").append(this._createBtn(true)));
            content.appendTo("body");
            var uppertype = type.replace(type.charAt(0), type.charAt(0).toUpperCase());
            $([$dp1, $dp2]).ejDropDownList({ fields: { text: "text", value: "value" }, height: 27, width: 120, enableRTL: this._ctrlInstance.model.enableRTL });
            if (type == "number") {
				$([$dp1, $dp2]).ejDropDownList({ popupWidth: "170px" });
                $([$in1, $in2]).ejNumericTextbox({ showSpinButton: false, height: "27px", width: "177px", enableRTL: this._ctrlInstance.model.enableRTL, focusOut: function(args){ if(this.model.decimalPlaces == 0) this.element.prev(".e-input").val(this.model.value); } });
            }
			else if (type == "guid") {
                $([$dp1, $dp2]).ejDropDownList({ popupWidth: "170px" });
                $($in1).css({"height":"22px","width":"175px" });
                $($in2).css({"height":"22px","width":"177px" });
            }
            else if (type == "date") {
				$([$dp1, $dp2]).ejDropDownList({ popupWidth: "170px" });
                $([$in1, $in2]).ejDatePicker({ "cssClass": this._ctrlInstance.model.cssClass, height: "27px", width: "177px", "enableRTL": this._ctrlInstance.model.enableRTL, watermarkText: this._getLocalizedLabel("DatePickerWaterMark") });
            }
            else if (type == "string") {
                $([$in1, $in2]).ejAutocomplete({ "cssClass": this._ctrlInstance.model.cssClass, "enableRTL": this._ctrlInstance.model.enableRTL, enableDistinct: true, width:"177px",height:"27px" });
            }
            $([radioAnd, radioOr]).ejRadioButton({ "cssClass": this._ctrlInstance.model.cssClass, "enableRTL": this._ctrlInstance.model.enableRTL });
            matchCase.ejCheckBox({ });
            $("#" + $id + "Custom_OkBtn").ejButton({ text: this._getLocalizedLabel("Ok"), showRoundedCorner: true, width:"23.6%", click: ej.proxy(this._fltrBtnHandler, this), enabled: true });
            $("#" + $id + "Custom_CancelBtn").ejButton({ text: this._getLocalizedLabel("Cancel"), showRoundedCorner: true, width: "23.6%", click: ej.proxy(this.closeXFDialog, this) })
        },
                     
        _openCustomFilter: function (operator) {
            var oper = operator != "top10" ? this._$colType: operator;
            var type = oper.replace(oper.charAt(0), oper.charAt(0).toUpperCase());
            var id = this.id + this._$colType;
            this.closeXFDialog();
            this._openedFltr = $("#" + id + "_CustomFDlg");
            var args = { requestType: "filterbeforeopen", filterModel: this, columnName: this.fName, columnType: this._$colType, isCustomFilter: true };
            if (this._ctrlInstance._trigger(this._onActionBegin, args))
                return;
            this._openedFltr.ejDialog("open");
            this._openedFltr.ejDialog({open: function(args){
				$("#" + id + "_CustomValue1").hasClass("e-autocomplete") ? $("#" + id + "_CustomValue1").focus() : $("#" + id + "_CustomValue1").prev().focus();
			}});
            this._openedFltr.find("legend").html(this._displayName);            
            var sl = (["Number","Date"].indexOf(type) != -1) ? 6 : 5;
            $("#" + id + "_CustomDrop1").ejDropDownList({ dataSource: this._getLocalizedLabel(type + "MenuOptions").slice(0,sl) });
            $("#" + id + "_CustomDrop2").ejDropDownList({ dataSource: this._getLocalizedLabel(type + "MenuOptions").slice(0, sl) });
			if (this._$colType == "number") {
                var decimalPlace = 0;
                if(operator == "top10"){
                    this._openedFltr.find(".e-optable tr").not(".e-top").addClass("e-hide");   
                    this._openedFltr.find(".e-optable tr.e-top").removeClass("e-hide");
                    $("#" + id + "_CustomDrop3").ejDropDownList({ dataSource: this._getLocalizedLabel(type + "MenuOptions") });
                }
                else{                   
                    decimalPlace = this._$format.length != 0 ? parseInt(this._$format.replace(/\D/g,'')) : 0;                    
                    this._openedFltr.find(".e-optable tr.e-top").addClass("e-hide");
                    this._openedFltr.find(".e-optable tr").not(".e-top").removeClass("e-hide"); 
                    $("#" + id + "_CustomValue1").ejNumericTextbox("model.decimalPlaces", decimalPlace);
                    $("#" + id + "_CustomValue2").ejNumericTextbox("model.decimalPlaces", decimalPlace);
                }
            }
            else {
                this._openedFltr.find(".e-optable tr.e-top").addClass("e-hide");
                this._openedFltr.find(".e-optable tr").not(".e-top").removeClass("e-hide");
            }
            if(this._$colType == "string"){
				var fName = this._$foreignField ? this._$foreignField : this.fName;
				var data = this._$foreignData ? this._$foreignData : this._dataSource;
                this._openedFltr.find(".e-autocomplete").ejAutocomplete({
                    fields: { text: fName }, dataSource: data, focusIn: function (args) {
                        var type = this.element.closest("td").siblings().find(".e-dropdownlist").ejDropDownList("getSelectedValue");
                        var $matchCase = this.element.closest(".e-dialog-scroller").find(".e-checkbox").prop("checked");
						 this.model.caseSensitiveSearch = $matchCase;
						 this.model.filterType = type == "" ? this.model.filterType : type;
                    }
                });
            }
            if (["date", "datetime"].indexOf(this._$colType) > -1 && this._$format != "") {
                this._openedFltr.find(".e-datepicker").ejDatePicker({ dateFormat: this._$format.replace(/{0:|}/g, function () { return "" }) });
            }
            this._setFilteredData(id, operator);
			if(!ej.isNullOrUndefined(this._customFilterHandler)) 
				this._customFilterHandler();
            var args = { requestType: "filterafteropen", filterModel: this, columnName: this.fName, columnType: this._$colType, isCustomFilter: true };
            
			if (this._ctrlInstance._trigger(this._onActionComplete, args))
                return;
        },

        _setFilteredData: function ($id, op) {
            var indx = $.inArray(this.fName, this.cFilteredCols);
            var fQM = [], optrs = [], fLen;
            var between = ["greaterthanorequal", "lessthanorequal"];
            if (op == "top10")
                return;
            if (indx != -1)
                fQM = ej.DataManager(this.filteredColumn).executeLocal(ej.Query().where("field", "equal", this.fName));
            if (this._initialFName != null && this._initialFName == this.fName)
                fQM[0]["isCustom"] = true;
            fLen = fQM.length;
            var drops = this._openedFltr.find(".e-dropdownlist"), inputs = this._openedFltr.find(".e-value input.e-ejinputtext.e-input"), $pred = this._openedFltr.find(".e-predicate");            
            if (indx != -1 && fLen && fQM[0]["from"]!=undefined)
                optrs = [op, ""];
			else if(op == "between" && indx != -1 && fLen )
				optrs = [fQM[0]["operator"], !ej.isNullOrUndefined(fQM[1]) ? fQM[1]["operator"] : ""];
            else if (indx != -1 && fLen && fQM[0]["isCustom"])
                optrs = [(op != "customfilter" && indx != -1) ? op : fQM[0]["operator"], !ej.isNullOrUndefined(fQM[1]) ? fQM[1]["operator"] : ""];
			else if(indx == -1 && op == "customfilter") 
                optrs = ["equal", ""];
            else if (op == "between" || op == "customfilter")
                optrs = op != "customfilter" ? between : ["", ""];           
            else
                optrs = [op, ""];
           
            for (var i = 0; i < (indx != -1 ? fLen : 2) ; i++) {
                var opt = /\D*/.exec(optrs[i])[0];
                $(drops[i]).prop("value", opt);
                $(drops[i]).ejDropDownList("setSelectedValue", opt);
				 $(drops[i]).ejDropDownList({ change: function(arg){ 
					this.element.closest(".e-fields").find(".e-autocomplete").val("");
				 }});
				var value = (indx != -1 && fLen && fQM[i]["isCustom"] && ($.inArray(op,optrs) !=-1 || op == "customfilter" || op == "between")) ? fQM[i]["value"] : "";
                if ($(inputs[i]).hasClass("e-datepicker"))   
                    $("#" + inputs[i].id ).ejDatePicker("model.value", (indx != -1 && fLen && fQM[i]["isCustom"]) ? fQM[i]["value"] : null );                                
				else if($(inputs[i]).hasClass("e-numerictextbox"))
					$("#" + inputs[i].id ).ejNumericTextbox("model.value", value);					
                else
                    $(inputs[i]).val(value);
                var $pre = (indx != -1 && fLen && fQM[i]["isCustom"] && fQM[i]["predicate"] != undefined) ? fQM[i]["predicate"] : "and";
				$pred.find("input[value=" + $pre + "]").ejRadioButton({checked:  true});
				if(this._$colType == "string"){
				    var $match = (indx != -1 && fLen && fQM[i]["isCustom"]) ? fQM[i]["matchcase"] : this._matchCase;
					$pred.find("input[type='checkbox']").ejCheckBox({checked: $match});
				}
				this._openedFltr.find(".e-value input:visible:eq(0)").select();
            }            
        },
        _setDateObject: function (filterObject) {
            if (filterObject.value != null) {                
                var $prevObj = $.extend(true, {}, filterObject);
                var $nextObj = $.extend(true, {}, filterObject);
                var $prevDate = new Date($prevObj.value.setSeconds($prevObj.value.getSeconds() - 1));
                var $nextDate = new Date($nextObj.value.setSeconds($nextObj.value.getSeconds() + 2));
                $prevObj.value = $prevDate;
                $nextObj.value = $nextDate;
                filterObject.value = new Date(filterObject.value.setSeconds($nextObj.value.getSeconds() - 1));
                if (filterObject.operator == "equal") {
                    $prevObj.operator = "greaterthan";
                    $prevObj.predicate = "and";
                    $nextObj.operator = "lessthan";
                    $nextObj.predicate = "and";
                } else if (filterObject.operator == "notequal") {
                    $prevObj.operator = "lessthanorequal";
                    $prevObj.predicate = "or";
                    $nextObj.operator = "greaterthanorequal";
                    $nextObj.predicate = "or";
                }
                var predicate = ej.Predicate($prevObj.field, $prevObj.operator, $prevObj.value, !$prevObj.matchcase);
                predicate = predicate[$nextObj.predicate]($nextObj.field, $nextObj.operator, $nextObj.value, !$nextObj.matchcase);
                filterObject.ejpredicate = predicate; filterObject.type = "date";
                return filterObject;
            }
            else
                return filterObject;
        },
        _getCDlgFields: function () {
            var dropDowns = this._openedFltr.find(".e-dropdownlist:visible"), defaults;
            var $match = this._openedFltr.find(".e-checkbox"), matchcase = true,valueColl = [];
            var $inputs = this._openedFltr.find(".e-value input.e-ejinputtext.e-input");
			 var $ginputs = this._openedFltr.find(".e-value input");
            for (var i = 0,len = dropDowns.length; i < len; i++) {
                var dvalue = $(dropDowns[i]).ejDropDownList("getSelectedValue"), value;
                if (this._$colType == "number")
                    value = parseFloat($inputs.eq(i).ejNumericTextbox("model.value"));
				if(this._$colType == "guid")
                    value = $ginputs.eq(i).val();
                if (this._$colType == "string") {
                    value = $inputs.eq(i).val();
                    matchcase = $match.is(":checked");
                }
                if (this._$colType == "date") 
                    value = $inputs.eq(i).ejDatePicker("model.value");
				if(this._$foreignField)
				    value = this._getForeignFilterData(value);
				defaults = { field: this.fName,predicate: this._openedFltr.find(".e-predicate input[type='radio']:checked").val(),  matchcase: matchcase, isCustom: true };
                if (dvalue == "top" || dvalue == "bottom") {
                    valueColl.push($.extend(true, { value: "", operator: "notequal", take: value, from: dvalue }, defaults));
                }
                else if (dvalue != "") {
                    var filterObj = {}; $.extend(true, filterObj,{ value: value, operator: dvalue.toLowerCase(), isCustom: true }, defaults); 
                    if (this._$colType == "date")
                        filterObj.type = "date";
                    valueColl.push((this._$colType == "date" && ["equal", "notequal"].indexOf(dvalue.toLowerCase()) != -1) ? this._setDateObject(filterObj) : filterObj);
                }
                else
                    return valueColl;
            }
            return valueColl;
        },
      
        _fltrBtnHandler: function (args) {
            var matchcase, valColl = [], arg = {}, predicate, fObj = {};
            if (!this._openedFltr.hasClass("e-dlgcustom")) {
                if (!this._isFiltered && this._listsWrap.find(".e-selectall").ejCheckBox("model.checked") && this._searchCount == 0) 
                    return this.closeXFDialog();                                  
                var checked = this._listsWrap.find("input").filter(":checked:not(.e-selectall)");
                var len = checked.length, cVal;
                var optr = this._colType == "string" ? "startswith" : "equal";
                var _isCase = this._colType == "string" ? this._matchCase : true;
                var defaults = { field: this.fName, predicate: "or", operator: optr, matchcase: _isCase };
                for (var i = 0; i < len; i++) {
                    cVal = checked[i].value;
                    if (this.enableNormalize && cVal.indexOf(this._spliter) != -1) {
                        ej.merge(valColl, this.iterateAndGetCollection(cVal, defaults));
                        continue;
                    }
                    cVal = this.processValue(cVal, this._$colType);

                    if (this._$foreignField)
                        cVal = this._getForeignFilterData(cVal);

					$.extend(true, fObj, { value: cVal }, defaults);
                    
                    valColl.push(this._$colType == "date" ? this._setDateObject(fObj) : fObj);
                     
                    fObj = {};
                }
                var cIndex = $.inArray(this.fName, this.cFilteredCols);
                if(cIndex != -1)
					this.cFilteredCols.splice(cIndex,1);
            }
            else {
                valColl = this._getCDlgFields();
                $.inArray(this.fName, this.cFilteredCols) == -1 && this.cFilteredCols.push(this.fName);
            }
            var firstVal = valColl[0], isTake = firstVal["from"];
            if (!ej.isNullOrUndefined(firstVal)) {
                predicate = firstVal["ejpredicate"] ? firstVal["ejpredicate"] : ej.Predicate(firstVal.field, firstVal.operator, firstVal.value, !firstVal.matchcase);
                for (var j = 1, jlen = valColl.length; j < jlen; j++) {
                    predicate = valColl[j].ejpredicate != undefined ? predicate[valColl[j].predicate](valColl[j].ejpredicate) : predicate[valColl[j].predicate](valColl[j].field, valColl[j].operator, valColl[j].value, !valColl[j].matchcase);
                }
                arg = { action: "filtering", filterCollection: valColl, fieldName: this.fName, ejpredicate: predicate, tableID: this._$tableID};
                if (this._predicates[this._$key] == undefined)
                    this._predicates[this._$key] = {};
                this._predicates[this._$key][this.fName] = !isTake ? predicate : { from: firstVal["from"], take: firstVal["take"] };
                if (!this._openedFltr.hasClass("e-dlgcustom") && this._listsWrap.find(".e-selectall").ejCheckBox("model.checked") && !this._searchCount && $.inArray(this.fName, this._columnsFiltered) != -1)
                    arg = { action: "clearfiltering", filterDetails: { field: this.fName, operator: "", predicate: "or", value: "" }, fieldName: this.fName, tableID: this._$tableID }
                if ($.inArray(this.fName, this._columnsFiltered) == -1)
                    this._columnsFiltered.push(this.fName);
                this.closeXFDialog();
                this._filterHandler(arg);
            }
        },
		_getForeignFilterData: function(data){
			return ej.DataManager(this._$foreignData).executeLocal(ej.Query().where(this._$foreignField, "equal", data).select(this.fName))[0];
		},
		/*Util method to perform formatted group on the inputted json*/
		getDistinct: function (json, field, fullRecord) {
		    var lookup = {}, len = json.length, result = [], current, value, fd;

		    while (len--) {
		        current = json[len], value = ej.getObject(field, current), fd = value;

		        if (this.enableNormalize)
		            fd = this._formatting(this._$format, value);

		        current["ejvalue"] = fd;

		        if (!(value in lookup))
		            result.push(fullRecord ? current : value);

		        lookup[value] = true;
		    }

		    result = ej.group(ej.mergeSort(result, field), "ejvalue"); /* sort with field and group with ejvalue */

		    return result;
		},
        /*Util method to split string and build filterobject collection */
		iterateAndGetCollection: function (valuestring, defaults) {
		    var splts = valuestring.split(this._spliter), len = splts.length, result = [], value;

		    while (len--) {
		        value = this.processValue(splts[len]);

		        if (this._$foreignField)
		            value = this._getForeignFilterData(value);

		        result.push($.extend(true, { value: value }, defaults));
		    }

		    return result;
		},
        /*Util method to perform type conversion*/
		processValue: function (val, type/* optional */) {
		    type = type || this._$colType; type = this._empties.indexOf(val) != -1 ? "empty" : type;
		    switch (type) {
		        case "empty":/*Handle blanks*/
		            val = val == "null" ? null : val == "undefined" ? undefined : "";
		            break;
		        case "date":
		        case "datetime":
		            val = new Date(val);
		            break;
		        case "number":
		            val = +val;
		            break;
		        case "boolean":		           
		            val = val === "true" ? true : false;
		            break;
		    }		    		      
		    return val;
		},
		replacer: function (input, pattern, processor, params) {
		    if (typeof processor == "function")
		        processor = processor.call(this, params)
		   return input.replace(pattern, processor);
		},
        _virtualize: function (e) {
            var height = $("#" + this.id + this._$colType + "_CheckBoxList").height();
            var chks = this._chkList.not(".e-checkbox").filter(function () { if (this.offsetTop > e.scrollTop - this.offsetHeight && e.scrollTop + height+70 > this.offsetTop + this.offsetHeight) return this; });
            if (chks.length == 0) return;
            chks.filter(":checked").ejCheckBox({ checked: true }); chks.filter(":not(:checked)").ejCheckBox({ checked: false });
            chks.ejCheckBox({ change: ej.proxy(this._checkHandler, this) });
        },
        /*Method to reset private properties*/
        resetFilterModel: function (destroy) {
            this._blank = undefined;
            this._addAtLast = false;
            this._isFiltered = false;
            this._searchCount = 0;
        },
        resetExcelFilter: function (template) {
            var _i, _type, _id = this.id,_len, _$id;
            this._predicates = [];          
            this.cFilteredCols = [];
            this.resetFilterModel();
            for (_i = 0, _len = this._posType.length; _i < _len; _i++) {
                _type = this._posType[_i], _$id = _id + _type;
                var cDlg = $("#" + _$id + "_CustomFDlg");               
                $("#" + _$id + "_CheckBoxList").ejWaitingPopup("destroy"); $("#" + _$id + "_excelDlg").remove();
                cDlg.find(".e-dropdownlist").ejDropDownList("destroy");
                cDlg.find(".e-button").ejButton("destroy");
                if (_type == "string" || _type == "boolean") cDlg.find(".e-autocomplete").ejAutocomplete("destroy");
                if (_type == "number") cDlg.find(".e-numerictextbox").ejNumericTextbox("destroy");
                if (_type == "date") cDlg.find(".e-datepicker").ejDatePicker("destroy");
                cDlg.ejDialog("destroy");
                $("#" + _$id + "_CustomFDlg").remove();
            }
        },
        _wireEvents: function () {
            this._ctrlInstance._on(this._dialogContainer, "focus click", ".e-searchbox", ej.proxy(this._searchBoxFocus, this));
            this._ctrlInstance._on(this._dialogContainer, "keyup", ".e-searchbox input", ej.proxy(this._search, this));
        }
    };

    ej.ExcelFilter.valueDelimiter = "@|@";
    ej.ExcelFilter.locale = ej.ExcelFilter.locale || {};
    ej.ExcelFilter.locale["en-US"] = {
        SortNoSmaller: "Sort Smallest to Largest",
        SortNoLarger: "Sort Largest to Smallest",
        SortTextAscending: "Sort A to Z",
        SortTextDescending: "Sort Z to A",
        SortDateOldest: "Sort by Oldest",
        SortDateNewest:"Sort by Newest",
		SortByColor: "Sort By Color",
        SortColorOptions: [{ id: 1, background: "#FFFFFF" }, {id: 2, background: "#5EABDA" }],
        SortByCellColor: "Sort by Cell Color",
        SortByFontColor: "Sort by Font Color",
        FilterByColor: "Filter By Color",
		CustomSort: "Custom Sort",
        FilterColorOptions: [{ id: 1, background: "#FFFFFF" }, {id: 2, background: "#5EABDA" }],
        FilterByCellColor: "Filter by Cell Color",
        FilterByFontColor: "Filter by Font Color",
        ClearFilter: "Clear Filter",
        NumberFilter: "Number Filters",
		GuidFilter: "Guid Filters",
        TextFilter: "Text Filters",
        DateFilter: "Date Filters",
        StringMenuOptions: [{ text: "Equal", value: "equal" }, { text: "Not Equal", value: "notequal" }, { text: "Starts With", value: "startswith" }, { text: "Ends With", value: "endswith" }, { text: "Contains", value: "contains" }, { text: "Custom Filter", value: "customfilter" }],
        NumberMenuOptions: [{ text: "Equal", value: "equal" }, { text: "Not Equal", value: "notequal" }, { text: "Less Than", value: "lessthan" }, { text: "Less Than Or Equal", value: "lessthanorequal" }, { text: "Greater Than", value: "greaterthan" }, { text: "Greater Than Or Equal", value: "greaterthanorequal" }, { text: "Between", value: "between" }, { text: "Custom Filter", value: "customfilter" }],
        GuidMenuOptions: [{ text: "Equal", value: "equal" }, { text: "Not Equal", value: "notequal" }, { text: "Custom Filter", value: "customfilter" }],
		DateMenuOptions: [{ text: "Equal", value: "equal" }, { text: "Not Equal", value: "notequal" }, { text: "Less Than", value: "lessthan" }, { text: "Less Than Or Equal", value: "lessthanorequal" }, { text: "Greater Than", value: "greaterthan" }, { text: "Greater Than Or Equal", value: "greaterthanorequal" }, { text: "Between", value: "between" }, { text: "Custom Filter", value: "customfilter" }],
        Top10MenuOptions: [{ text: "Top", value: "top" }, { text: "Bottom", value: "bottom" }],
        title:"Custom Filter",
        PredicateAnd: "AND",
        PredicateOr: "OR",
        Ok: "OK",
        MatchCase: "Match Case",
        Cancel: "Cancel",
        NoResult: "No Matches Found",
        CheckBoxStatusMsg: "Not all items showing",
        DatePickerWaterMark: "Select date",
		True: "true",
        False: "false"
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.scheduleFeatures = ej.scheduleFeatures || {};

    var resourceHeadTemplate = function () {
        this._resHeadTemplate = (
        "{{for resourceName}}{{for ~cols=colspan}}<td class='e-resourcecells' colspan='{{:~cols}}'> {{for ~resource=resourceNam}}<div class='e-resourceheadertext' align='center' title='{{if userResTemplId == true}}{{:resourceNam}}{{else}}{{:~resource}}{{/if}}'>{{if userResTemplId == true}}{{:userResHeader}}{{else}}{{:~resource}}{{/if}}</div>{{/for}}  </td>{{/for}}{{/for}}");
        return this._resHeadTemplate;
    };

    ej.scheduleFeatures.resources = {
        __getMultipleResourceCellsCount: function (count) {
            if (this.model.group.resources.length === 1) {
                var index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[0]);
                var resCount = this._resourceInfo[index].dataSource.length;
            }
            else {
                var newResCollection = this._getResourceCollection(), resources = [];
                var groupData, prevGroupDataId, query, resApp;
                for (var j = 0; j < newResCollection.length; j++) {
                    resources = [];
                    if (j == 0) {
                        for (var i = 0; i < newResCollection[j].dataSource.length; i++) {
                            resources.push({ resourceNam: newResCollection[j].dataSource[i][newResCollection[j].text] });
                        }
                    }
                    else {
                        if (j > 1) {
                            newResCollection[j - 1].dataSource = [];
                            newResCollection[j - 1].dataSource = this.res;
                        }
                        this.res = [];
                        groupData = ej.group(newResCollection[j].dataSource, newResCollection[j].groupId);
                        prevGroupDataId = ej.group(newResCollection[j - 1].dataSource, newResCollection[j - 1].id);
                        this._resourceManager1 = groupData instanceof ej.DataManager ? groupData : ej.DataManager(groupData);
                        for (var l = 0; l < prevGroupDataId.length; l++) {
                            for (var k = 0; k < prevGroupDataId[l].items.length; k++) {
                                query = new ej.Query().where("key", ej.FilterOperators.equal, prevGroupDataId[l].items[k][newResCollection[j - 1].id]);
                                resApp = this._resourceManager1.executeLocal(query);
                                if (resApp.length > 0) {
                                    for (var a = 0; a < resApp[0].items.length; a++) {
                                        resources.push({ resourceNam: resApp[0].items[a][newResCollection[j].text] });
                                        this.res.push(resApp[0].items[a]);
                                    }
                                }
                            }
                        }
                    }
                }
                resCount = resources.length;
                this.res1 = resources.length;
            }
            count = count * resCount;
            return { resCount: resCount, count: count };
        },

        _initResourceData: function (fields, i) {
            var query = this._columnToSelect(fields);
            var proxy = this;
            var queryPromise = fields.dataSource.executeQuery(query);
            queryPromise.done(function (e) {
                fields.dataSource = e.result;
                proxy._resourceInfo.push(fields);
                if (i == proxy.model.resources.length - 1)
                    proxy._renderInitSchedule();
            });
        },

        _showMultipleResourceDetails: function (temp) {
            var prevdata, query, resApp1;
            if (this.model.resources.length > 0) {
                var newResCollection = this._getResourceCollection();
                for (var i = 0; i < newResCollection.length; i++) {
                    this._appointmentAddWindow.find('.owner_' + i).ejAutocomplete("clearText");
                }
                for (var j = 0; j < this.render_Resources.length - 1; j++) {
                    prevdata = this.render_Resources[j + 1] instanceof ej.DataManager ? this.render_Resources[j + 1] : ej.DataManager(this.render_Resources[j + 1]);
                    query = new ej.Query().where(newResCollection[j + 1].groupId, ej.FilterOperators.equal, temp[this.model.appointmentSettings.resourceFields.toString().split(',')[j]]);
                    resApp1 = prevdata.executeLocal(query);
                    this._appointmentAddWindow.find('.owner_' + (j + 1)).ejAutocomplete("clearText");
                    this._appointmentAddWindow.find('.owner_' + (j + 1)).ejAutocomplete("option", "dataSource", resApp1);
                }
                for (var i = 0; i < newResCollection.length; i++) {
                    this._appointmentAddWindow.find('.owner_' + i).data("ejAutocomplete").selectValueByKey(temp[this.model.appointmentSettings.resourceFields.toString().split(',')[i]]);
                }
            }
        },

        _renderAppWindowResources: function (control) {
            var newResCollection = this.render_Resources;
			var Text = this.model.enableRTL ? 'margin-right:30px;margin-top:-18px' : 'float:left';
            var Color = this.model.enableRTL ? 'float:none': 'float:left';
            this._resCollection = this._getResourceCollection();
            for (var i = 0; i < newResCollection.length; i++) {
                if (!this.model.resources[i].allowMultiple)
                    control.find('.owner_' + i).ejAutocomplete({
                        enableRTL: this.model.enableRTL,
                        showPopupButton: true,
                        width: "100%", cssClass: this.model.cssClass,
                        multiSelectMode: "none",
                        dataSource: newResCollection[i], fields: { text: this._resCollection[i]["text"], key: this._resCollection[i]["id"] }, change: this._onResourceChange,
                        template: "<div class='e-resourcediv' style='height:15px; margin-right:5px; margin-top:3px; float:left; width:15px;"+ Color +"; background-color: ${" + this._resCollection[i].color + "}'></div><div style= "+Text+">${" + this._resCollection[i]["text"] + "}</div>"
                    });
                else
                    control.find('.owner_' + i).ejAutocomplete({
                        enableRTL: this.model.enableRTL,
                        showPopupButton: true,
                        width: "100%", cssClass: this.model.cssClass,
                        multiSelectMode: "visualmode",
                        dataSource: newResCollection[i], fields: { text: this._resCollection[i]["text"], key: this._resCollection[i]["id"] }, change: this._onResourceChange,
                        template: "<div class='e-resourcediv' style='height:15px; margin-right:5px; margin-top:3px; float:left; width:15px;"+ Color +"; background-color: ${" + this._resCollection[i].color + "}'></div> <div style= "+Text+">${" + this._resCollection[i]["text"] + "}</div>"
                    });
            }
        },

        _onResourceChange: function (e) {
            if (e.value !== "") {
                var scheduleDiv = $("#" + this._id.toString().split("_")[0]), proxy = scheduleDiv.ejSchedule("instance");
                var controlId = parseInt(this._id.toString().split("_")[2]);
                var controlMode = proxy._appointmentAddWindow.find('.owner_' + controlId).ejAutocomplete("option", "multiSelectMode");
                var selectedItems = this.getSelectedItems();
                var resourceCollection = proxy._getResourceCollection();
                if (controlId < proxy.render_Resources.length - 1 && selectedItems.length > 0) {
                    var j = controlId + 1;
                    var data = proxy.render_Resources[j - 1] instanceof ej.DataManager ? proxy.render_Resources[j - 1] : ej.DataManager(proxy.render_Resources[j - 1]);
                    var prevdata = proxy.render_Resources[j] instanceof ej.DataManager ? proxy.render_Resources[j] : ej.DataManager(proxy.render_Resources[j]);
                    var query, resApp, resApp1, fieldCollection = [];
                    if (controlMode == "none") {
                        if (!ej.isNullOrUndefined(e.value)) {
                            var textCheck = new ej.Query().where(resourceCollection[j - 1].text, ej.FilterOperators.equal, e.value);
                            var result = data.executeLocal(textCheck);
                        }
                        var currentVal = ((!e.value) || ej.isNullOrUndefined(e.value) || result.length <= 0) ? selectedItems[0][resourceCollection[j - 1].id] : result[0][resourceCollection[j - 1].id];
                        if (!ej.isNullOrUndefined(currentVal)) {
                            query = new ej.Query().where(resourceCollection[j - 1].id, ej.FilterOperators.equal, currentVal);
                            resApp = data.executeLocal(query);
                            query = new ej.Query().where(resourceCollection[j].groupId, ej.FilterOperators.equal, resApp[0][resourceCollection[j - 1].id]);
                            resApp1 = prevdata.executeLocal(query);
                            fieldCollection = [];
                            for (var k = 0; k < resApp1.length; k++) {
                                fieldCollection.push(resApp1[k]);
                            }
                            proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("clearText");
                            proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("option", "dataSource", fieldCollection);
                            proxy._appointmentAddWindow.find('.owner_' + j).data("ejAutocomplete").selectValueByKey(resApp1[0][resourceCollection[j].id]);
                            selectedItems = resApp1[0][resourceCollection[j].id];
                        }
                    }
                    else {
                        var predicate;
                        predicate = ej.Predicate(resourceCollection[j - 1].id, ej.FilterOperators.equal, selectedItems[0][resourceCollection[j - 1].id]);
                        selectedItems = e.value.split(",");
                        for (var i = 1; i < selectedItems.length; i++) {
                            predicate = predicate["or"](resourceCollection[j - 1].text, ej.FilterOperators.equal, selectedItems[i]);
                        }
                        query = new ej.Query().where(predicate);
                        resApp = data.executeLocal(query);
                        if (resApp.length > 0) {
                            var prevPredicate;
                            prevPredicate = ej.Predicate(resourceCollection[j].groupId, ej.FilterOperators.equal, resApp[0][resourceCollection[j - 1].id]);
                            for (var i = 1; i < resApp.length; i++) {
                                prevPredicate = prevPredicate["or"](resourceCollection[j].groupId, ej.FilterOperators.equal, resApp[i][resourceCollection[j - 1].id]);
                            }
                            query = new ej.Query().where(prevPredicate);
                            resApp1 = prevdata.executeLocal(query);
                            fieldCollection = [];
                            for (var k = 0; k < resApp1.length; k++) {
                                fieldCollection.push(resApp1[k]);
                            }
                            proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("clearText");
                            proxy._appointmentAddWindow.find('.owner_' + j).ejAutocomplete("option", "dataSource", fieldCollection);
                            proxy._appointmentAddWindow.find('.owner_' + j).data("ejAutocomplete").selectValueByKey(resApp1[0][resourceCollection[j].id]);
                            selectedItems.push(resApp1[0][resourceCollection[j].id]);
                        }
                    }
                }
            }
        },

        _getResourceCollection: function () {
            var resIndexCollection = [], index;
            if (!ej.isNullOrUndefined(this.model.resources)) {
                if (!ej.isNullOrUndefined(this.model.group)) {
                    for (var i = 0; i < this.model.group.resources.length; i++) {
                        index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
                        resIndexCollection.push({ index: index, name: this.model.resources[i].name });
                    }
                    resIndexCollection = resIndexCollection.sort(function (a, b) {
                        var x = a.index; var y = b.index;
                        return ((x < y) ? -1 : ((x > y) ? 1 : 0));
                    });
                }
                else {
                    for (var i = 0; i < this.model.resources.length; i++) {
                        resIndexCollection.push({ index: i });
                    }
                }
                var newResCollection = [];
                for (var k = 0; k < resIndexCollection.length; k++) {
                    newResCollection.push(this._resourceInfo[resIndexCollection[k].index]);
                }
                this._resCollection = newResCollection;
                return newResCollection;
            }
        },

        _renderMultipleResourceHeaderTemplate: function (tbody, cols) {
            this.res1 = []; this.render_Resources = []; this.level_Resources = [];
            var view = this.currentView();
            var viewColCount = view == "day" ? 1 : view == "week" || view == "month" ? 7 : this.model.workWeek.length;
            viewColCount = (view == "customview" && this._dateRender.length >= 7) ? 7 : (view == "customview" && this._dateRender.length < 7) ? this._dateRender.length : viewColCount;
            if (!ej.isNullOrUndefined(this.model.group)) {
                var userResTemplId = (this.model.resourceHeaderTemplateId) ? true : false;
                var resources = [];
                if (this.model.group.resources.length === 1) {
                    var index;
                    for (var i = 0; i < this.model.group.resources.length; i++) {
                        index = this._findResourceIndex(this.model.resources, "name", this.model.group.resources[i]);
                    }
                    for (var i = 0; i < this._resourceInfo[index].dataSource.length; i++) {
                        (this.currentView() != "agenda") && resources.push({ resourceNam: this._resourceInfo[index].dataSource[i][this._resourceInfo[index].text], colspan: viewColCount, userResHeader: this._getResourceHeadTemplate($.extend(this._resourceInfo[index].dataSource[i], { classname: "e-childnode" })), userResTemplId: userResTemplId });
                        this.res1.push(this._resourceInfo[index].dataSource[i]);
                        this.level_Resources.push(this.res1[i]);
                    }
                    this._valueCollection = this.res1;
                    this.render_Resources.push(this.res1);
                    if (this.model.orientation == "vertical" && this.currentView() != "agenda" && tbody !== null && cols != null) {
                        var leftCellwidth = view == "month" ? 50 : 56;
                        var resourceHeaderHtml = this.resourceHeadTemplate.render({ cols: cols, resourceName: resources, leftWidth: leftCellwidth });
                        tbody.append(ej.buildTag('tr.e-headerbar').css("height", "28px").append(resourceHeaderHtml));
                    }
                }
                else {
                    var newResCollection = this._getResourceCollection(), groupDatasource = [], firstValue = [], collection = [], resAppValueCount = 0, valueCount = [];
                    var res = []; var level = []; var variable = []; this.resourceValue; var resourceLength;
                    groupDatasource = $.merge(groupDatasource, newResCollection);
                    if (this.model.resources.length) {
                        var k = 0;
                        for (j = 0; j < newResCollection[newResCollection.length - 2].dataSource.length; j++) {
                            var query = new ej.Query().where(newResCollection[newResCollection.length - 1].groupId, ej.FilterOperators.equal, newResCollection[newResCollection.length - 2].dataSource[j][newResCollection[newResCollection.length - 2].id]);
                            var resApp = new ej.DataManager(newResCollection[newResCollection.length - 1].dataSource).executeLocal(query);
                            var m = 0;
                            while (m < resApp.length) {
                                firstValue.push({ resourceNam: resApp[m][newResCollection[newResCollection.length - 1].text], colspan: 1 * viewColCount, userResHeader: this._getResourceHeadTemplate($.extend(resApp[m], { classname: "e-childnode" })), userResTemplId: userResTemplId });
                                collection[k] = resApp[m];
                                this.res1.push(resApp[m]);
                                k++;
                                m++;
                            }
                        }
                        this._valueCollection = collection;
                        newResCollection[newResCollection.length - 1].dataSource = collection;
                    }
                    for (var i = this.model.resources.length - 1; i >= 0 ; i--) {
                        level.push(i);
                        for (var j = 0; j < newResCollection[i].dataSource.length; j++) {
                            var valuefirst = i != 0 ? this._findResourceIndex(newResCollection[i - 1].dataSource, newResCollection[i - 1].id, newResCollection[i].dataSource[j][newResCollection[i].groupId]) : 0;
                            var valuelast = i != this.model.resources.length - 1 ? this._findResourceIndex(newResCollection[i + 1].dataSource, newResCollection[i + 1].groupId, newResCollection[i].dataSource[j][newResCollection[i].id]) : this.model.resources.length - 1;
                            (ej.isNullOrUndefined(valuefirst) || ej.isNullOrUndefined(valuelast)) ? newResCollection[i].dataSource[j] = 0 : "";
                            if (!ej.isNullOrUndefined(valuefirst) && !ej.isNullOrUndefined(valuelast) && newResCollection[i].dataSource[j] !== 0) {
                                res.push(newResCollection[i].dataSource[j]);
                                this.resourceValue = res.length;
                                if (i <= this.model.resources.length - 2) {
                                    query = new ej.Query().where(newResCollection[i + 1].groupId, ej.FilterOperators.equal, newResCollection[i].dataSource[j][newResCollection[i].id]);
                                    var resApp = new ej.DataManager(newResCollection[i + 1].dataSource).executeLocal(query);
                                    if (i <= this.model.resources.length - 3) {
                                        for (k = 0; k < resApp.length; k++)
                                            resAppValueCount = resAppValueCount + resApp[k].count;

                                        resourceLength = resAppValueCount;
                                        newResCollection[i].dataSource[j].count = resourceLength;
                                        resAppValueCount = 0;
                                    }
                                    if (i == this.model.resources.length - 2) {
                                        newResCollection[i].dataSource[j].count = resApp.length;
                                        resourceLength = resApp.length;
                                    }
                                    this.resourceValue = resourceLength;
                                    resApp.length = 0;
                                }
                                (this.currentView() != "agenda") && resources.push({ resourceNam: newResCollection[i].dataSource[j][newResCollection[i].text], colspan: this.resourceValue * viewColCount, userResHeader: this._getResourceHeadTemplate($.extend(newResCollection[i].dataSource[j], { classname: "e-childnode" })), userResTemplId: userResTemplId });
                                this.level_Resources.push(newResCollection[i].dataSource[j]);
                                res.length = 0;

                            }
                            if (this.model.orientation == "vertical" && this.currentView() != "agenda" && tbody !== null && cols != null) {
                                leftCellwidth = view == "month" ? 50 : 56;
                                resourceHeaderHtml = this.resourceHeadTemplate.render({ cols: cols, resourceName: resources, leftWidth: leftCellwidth });
                                variable[i] = ej.buildTag('tr.e-headerbar').css("height", "28px").append(resourceHeaderHtml);
                            }
                        }
                        valueCount[i] = newResCollection[i].dataSource;
                        resources.length = 0;
                    }
                    this.render_Resources = valueCount;
                    this.model.orientation == "vertical" && this.currentView() != "agenda" ? tbody.append(variable) : "";
                }
            }
            else {
                for (var k = 0; k < this.model.resources.length; k++) {
                    this.render_Resources.push(this.model.resources[k].resourceSettings.dataSource);
                }
            }
            return tbody;
        },

        _getMultipleResourceAlldayApp: function (renderDate, day, res, i) {
			this.allDayCount = [];
            if ((new Date(new Date(renderDate[day]).setHours(0, 0, 0, 0)).getTime() == new Date(new Date(this._processed[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime()) && (this._processed[i][this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]] == this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings.id]) &&
				(this._processed[i][this.model.appointmentSettings["allDay"]] === true || (((new Date(this._processed[i][this.model.appointmentSettings["endTime"]]) - new Date(this._processed[i][this.model.appointmentSettings["startTime"]])) / 3600000) >= 24))) {
                this._setAllDayPos(this._processed[i], day, res);
                this._filtered.push(this._processed[i]);
				this.allDayCount.push(i);
            }
        },

        _getResourceColor: function (record) {
            var newResCollection = this._resCollection;
            var resourceDataIndex, appointData = "", appointCustomcss = "";
            if (!ej.isNullOrUndefined(this.model.group)) {
                resourceDataIndex = this._findResourceIndex(this.render_Resources[0], newResCollection[0].id, record[this.model.appointmentSettings.resourceFields.split(',')[0]]);
                if (!ej.isNullOrUndefined(resourceDataIndex)) {
                    appointData = !ej.isNullOrUndefined(this.render_Resources[0][resourceDataIndex][newResCollection[0].color]) ? this.render_Resources[0][resourceDataIndex][newResCollection[0].color] : appointData;
                    appointCustomcss = !ej.isNullOrUndefined(this.render_Resources[0][resourceDataIndex][newResCollection[0].appointmentClass]) ? this.render_Resources[0][resourceDataIndex][newResCollection[0].appointmentClass] : appointCustomcss;
                }
            }
            else {
                resourceDataIndex = this._findResourceIndex(this.render_Resources[this.render_Resources.length - 1], newResCollection[newResCollection.length - 1].id, record[this.model.appointmentSettings.resourceFields.split(',')[this.model.appointmentSettings.resourceFields.split(',').length - 1]]);
                if (!ej.isNullOrUndefined(resourceDataIndex)) {
                    appointData = !ej.isNullOrUndefined(this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].color]) ? this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].color] : appointData;
                    appointCustomcss = !ej.isNullOrUndefined(this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].appointmentClass]) ? this.render_Resources[this.render_Resources.length - 1][resourceDataIndex][newResCollection[newResCollection.length - 1].appointmentClass] : appointCustomcss;
                }
            }
            return { appointData: appointData, appointCustomcss: appointCustomcss };
        },

        _getResourceValue: function (target, index) {
            var resDetails;
            if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources)) {
                var resource = this._getResourceDetails(target);
                resDetails = ej.DataManager(this._getResourceCollection()[resource.length - 1].dataSource).executeLocal(new ej.Query().where(this.model.resources[resource.length - 1].resourceSettings["id"], "equal", resource[0].id))[0];
            }
            return resDetails;
        },

        _getResourceDetails: function (target, index) {
            if (!ej.isNullOrUndefined(this.model.resources)) {
                if (this.model.resources.length > 0) {
                    var noOfDays = this.currentView() == "day" ? 1 : this.currentView() == "week" || this.currentView() == "month" ? 7 : this.model.workWeek.length;
                    noOfDays = (this.currentView() == "customview" && this._dateRender.length >= 7) ? 7 : (this.currentView() == "customview" && this._dateRender.length < 7) ? this._dateRender.length : noOfDays;
                    index = this.currentView() === "month" || this.currentView() == "customview" ? target.index() : !ej.isNullOrUndefined(this._cellIndex) ? this._cellIndex : target.index();
                    !target.hasClass("e-detailedapp") ? this.index = index : "";
                    index = target.hasClass("e-detailedapp") ? this.index : index;
                    index = this.model.orientation == "vertical" ? parseInt(index / noOfDays) : !ej.isNullOrUndefined(this._workCellIndex) ? this._workCellIndex : target.parent().index();
                    var res1 = this.model.orientation == "vertical" ? this._valueCollection : this._resCollect;
                    var group, resourceData, query, result; this._resourceCollection = [];
                    var newResCollection = this._getResourceCollection();
                    for (var i = newResCollection.length - 1; i >= 0; i--) {
                        if (i == newResCollection.length - 1) {
                            if (!ej.isNullOrUndefined(this.model.group)) {
                                this._resourceCollection.push({ id: res1[index][newResCollection[i].id] });
                                group = res1[index][newResCollection[i].groupId];
                            }
                            else {
                                this._resourceCollection.push({ id: newResCollection[newResCollection.length - 1].dataSource[index][newResCollection[newResCollection.length - 1].id] });
                                group = newResCollection[newResCollection.length - 1].dataSource[index][newResCollection[newResCollection.length - 1].groupId];
                            }
                        }
                        else {
                            resourceData = newResCollection[i].dataSource instanceof ej.DataManager ? newResCollection[i].dataSource : ej.DataManager(newResCollection[i].dataSource);
                            query = new ej.Query().where(newResCollection[i].id, ej.FilterOperators.equal, group);
                            result = resourceData.executeLocal(query);
                            this._resourceCollection.push({ id: result[0][newResCollection[i].id] });
                            group = result[0][newResCollection[i].groupId];
                        }
                    }
                    return this._resourceCollection;
                }
            }
        },

        _getResourceFields: function (obj) {
            if (!ej.isNullOrUndefined(this.model.group)) {
                for (var j = 0, i = this._resourceCollection.length - 1; j < this._resourceCollection.length; j++, i--)
                    obj[this.model.appointmentSettings.resourceFields.split(',')[j]] = this._resourceCollection[i].id;
            }
            else
                obj[this.model.appointmentSettings.resourceFields.split(',')[0]] = this._resourceCollection[0].id;
            return obj;
        },
        _getDragDropResourceData: function (proxy, cellIndex, newAddList) {
            if (!ej.isNullOrUndefined(cellIndex)) {
                var noOfCols = proxy.model.currentView == "day" ? 1 : proxy.model.currentView == "week" || proxy.model.currentView == "month" ? 7 : this.model.workWeek.length;
                noOfCols = (proxy.model.currentView == "customview" && this._dateRender.length >= 7) ? 7 : (proxy.model.currentView == "customview" && this._dateRender.length < 7) ? this._dateRender.length : noOfCols;
                var index = proxy.model.enableRTL ? proxy.res1.length - Math.floor(cellIndex / noOfCols) - 1 : Math.floor(cellIndex / noOfCols);
                var newRes = proxy.model.orientation == "vertical" ? proxy.res1[index] : proxy._resCollect[cellIndex];
                this._ownerId = newRes[proxy.model.resources[proxy.model.resources.length - 1].resourceSettings.id];
                this._groupId = newRes[proxy.model.resources[proxy.model.resources.length - 1].resourceSettings.groupId];
                newAddList[proxy.model.appointmentSettings.resourceFields.split(',')[proxy.model.appointmentSettings.resourceFields.split(',').length - 1]] = newRes[proxy.model.resources[proxy.model.resources.length - 1].resourceSettings.id];
                if(this.model.appointmentSettings.resourceFields.split(',').length>1)
				newAddList[proxy.model.appointmentSettings.resourceFields.split(',')[proxy.model.appointmentSettings.resourceFields.split(',').length - 2]] = newRes[proxy.model.resources[proxy.model.resources.length - 1].resourceSettings.groupId];
            }
            return newAddList;
        },

        _renderResourceData: function (targetElement, index) {
            this._getResourceDetails(targetElement);
            var fieldCollection = [], collection = [];
            if (this.model.resources.length) {
                for (var k = this._resourceCollection.length - 1; k >= 0; k--) {
                    var resourceDataIndex = this._findResourceIndex(this._resCollection[k].dataSource, this._resCollection[k].id, this._resourceCollection[(this._resourceCollection.length - 1) - k].id);
                    fieldCollection.push(this._resCollection[k].dataSource[resourceDataIndex]);
                    if (k != 0) {
                        var query = new ej.Query().where(ej.Predicate(this._resCollection[k].groupId, ej.FilterOperators.equal, this._resCollection[k].dataSource[resourceDataIndex][this._resCollection[k].groupId]));
                        var resApp = new ej.DataManager(this._resCollection[k].dataSource).executeLocal(query);
                        index = this._findResourceIndex(resApp, this._resCollection[k].id, this._resourceCollection[(this._resourceCollection.length - 1) - k].id);
                        collection = resApp;
                        this._appointmentAddWindow.find('.owner_' + (k)).ejAutocomplete("clearText");
                        this._appointmentAddWindow.find('.owner_' + (k)).ejAutocomplete("option", "dataSource", collection);
                        this._appointmentAddWindow.find('.owner_' + (k)).data("ejAutocomplete").selectValueByKey(collection[index][this._resCollection[k].id]);
                    }
                    else {
                        this._appointmentAddWindow.find('.owner_' + (k)).ejAutocomplete("clearText");
                        this._appointmentAddWindow.find('.owner_' + (k)).data("ejAutocomplete").selectValueByKey(fieldCollection[(this._resourceCollection.length - 1) - k][this._resCollection[k].id]);
                    }
                }
            }
        },

        _bindResourcesData: function () {
            if (!ej.isNullOrUndefined(this.model.resources)) {
                this._resourceInfo = []; var resFlag = false;
                for (var i = 0; i < this.model.resources.length; i++) {
                    if (!ej.isNullOrUndefined(this.model.resources[i].resourceSettings)) {
                        if (ej.isNullOrUndefined(this.model.resources[i].resourceSettings.query) || !(this.model.resources[i].resourceSettings.query instanceof ej.Query))
                            this.model.resources[i].resourceSettings.query = ej.Query();
                        if (this.model.resources[i].resourceSettings.dataSource instanceof ej.DataManager)
                            this._initResourceData(this.model.resources[i].resourceSettings, i);
                        else {
                            this._resourceInfo.push(this.model.resources[i].resourceSettings);
                            resFlag = true;
                        }
                    }
                }
                if (resFlag)
                    this._renderInitSchedule();
            }
        },

        _renderResourceElements: function ($appWindow) {
            for (var i = 0; i < this.model.resources.length; i++) {
                $appWindow += "<tr><td class='e-leftfields e-textlabel'>" + this.model.resources[i].title + ":</td><td colspan='3' class='e-rightfields'>" +
						  "<input id='" + this._id + "_ownerfield_" + i + "'  class='owner_" + i + "' type='text' value=''/><div id='ownerlist'></div></td></tr>";
            }
            return $appWindow;
        },

        _renderResourcesTemplate: function () {
            if (this.model.orientation == "vertical")
                this.resourceHeadTemplate = $.templates(this._createTemplate(resourceHeadTemplate(), "_ResourceHeader"));
        }
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Schedule elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.2
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    ej.scheduleFeatures = ej.scheduleFeatures || {};

    var monthCellsTemplate = function () {
        this._monthCellsTemplate = ("<table class='e-table e-workcellstab'  height='100%'width='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for cellrows ~cols1=cols1}}" +
        "<tr>{{for ~cwidth=cwidth ~cheight=cheight ~cols1}}" +
        "<td class='e-monthcells' width='{{:~cwidth}}' height='{{:~cheight}}'>" +
        "<div class='e-monthheader'></div><div class='e-appointwrapper'>" +
        "</td>{{/for}}" +
        "</tr>{{/for}}" +
        "</tbody></table>");
        return this._monthCellsTemplate;
    };

    var appointmentTemplate = function () {
        this._appTemplate = ("<div class='{{:appClass}} {{:appointCustomcss}}' id='Appointment_{{:id}}' guid= '{{:uid}}' role='presentation' style='{{if rtl == true}}right{{else}}left{{/if}}: {{:left}}; width: {{:appWidth}}; background: {{:appointData}};  color:{{:appointtextcolor}}; height: {{:appHeight}}px; top: {{:top}}px;' >"
       + "{{if appResize == true}}{{if appClass == 'e-appointment'}}" +
       "{{if orientation == 'vertical'}}<div class='e-tophandle'></div><div class='e-bottomhandle'></div>{{else}}<div class='e-lefthandle'></div><div class='e-righthandle'></div>{{/if}}" +
       "{{else}}{{if currentview != 'day'}}<div class='e-lefthandle'></div><div class='e-righthandle'></div>{{/if}}" +
       "{{/if}}{{/if}}<span class='e-icon e-schedulemouseclose'/></span>{{if userAppTemplId == false}}" +
       "<div class='e-appointinnertext'><div class='e-apptext {{if (priority!=undefined && priority!='')}}e-priorwidth{{/if}}'>{{:subject}} </div><div class='e-apptime {{if (priority!=undefined && priority!='')}}e-priorwidth{{/if}}'> {{:startTime}} - {{:endTime}}</div>"
       + "{{if (priority!=undefined && priority!='') && ((priorityTemplate!=undefined && priorityTemplate!=''))}}" +
      "{{if enablePriority==null}} <div class='e-priority e-{{:priorityTemplate}}'></div>{{else}}<div class='e-priority'>{{:priorityTemplate}}</div>{{/if}}" +
      "</div>{{/if}}{{if value == true}}<div class='e-multipleCategory'>{{:multiDiv}}</div>{{/if}}</div>" +
       "{{else}}{{:userTemplate}}{{/if}}</div>");
        return this._appTemplate;
    };

    var categorizeMultipleTemplate = function () {
        this._categorizeMultipleTemplate = (
            "{{for renderData }}<div style='background:{{:value}};height:11px;width:6px;float:left'>" +
            "</div>{{/for}}"
            );
        return this._categorizeMultipleTemplate;
    };

    var headerTemplate = function () {
        this._headerTemplate = (
        "{{for cols ~view=view ~colspan=colspan}}" +
        "<td class='e-headercells' colspan='{{:~colspan}}'>" +
        "<div class='e-dateheadercell'> <span class='e-headerdaydisplay'>{{:currentDay}}</span></div>" +
        "</td>" +
        "{{/for}}");
        return this._headerTemplate;
    };
    var leftHeaderTemplate = function () {
        this._leftIndent = ("<table class='e-table e-leftindenttable' width='{{:cswidth}}' cellpadding='0px' cellspacing='0px'>" +
        "<tbody> {{for trs ~tdClass=tdClass ~divClass=divClass}} <tr>{{for ~tdClass=tdClass ~divClass=divClass}} <td class='{{:~tdClass}}'style='height:27px;'> <div class='{{:~divClass}}'></div></td>{{/for}}</tr>" +
        "{{/for}}</tbody></table>");
        return this._leftIndent;
    };
    var alldayTemplate = function () {
        this._alldayTemplate = (
     "{{for cols ~colspan=colspan}} <td class='e-alldaycells' style='height:27px;' colspan='{{:~colspan}}'> </td>" +
        "{{/for}}");
        return this._alldayTemplate;
    };

    var workCellsTemplate = function () {
        this._workCellsTemplate = ("<table class='e-table e-workcellstab' width='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody >{{for cellrows ~cols1=cols1}}" +
        "<tr>{{for ~index=#index ~cwidth=cwidth ~cheight=cheight ~cols1 }}" +
        "{{if (~index+1) % 2 == 0 }}<td class='e-workcells' width='{{:~cwidth}}' height='{{:~cheight}}'>{{else}}<td class='e-workcells e-alternatecells'width='{{:~cwidth}}' height='{{:~cheight}}'>{{/if}}<div class='e-appointwrapper'></div></td>" +
        "{{/for}}" +
        "</tr>" +
        "{{/for}}</tbody></table>");
        return this._workCellsTemplate;
    };

    var timeCellsTemplate = function () {
        this._timeCellsTemplate = ("<div class='e-scrolltimecells'><table class='e-table e-timecelltable' cellpadding='0px' cellspacing='0px'>" +
        "<tbody>{{for timerows ~cols1=cols1 ~view1=viewclass}}" +
        "<tr>{{for ~name=time ~id=id ~view=~view1 }} " +
        "{{if ~view == 0}}<td class='e-timecells'><div class='e-timecelldivs'>{{:~name.slice(0,2)}}</div>" +
        "</td> <td class='e-timecells' rowspan='1'>{{for ~cols1}}{{if (#index+1)%2 ==0}}<div class='e-ampmdisplay'></div>{{else}}<div id='{{:~id}}' class='e-ampmdisplay e-ampmstyle'>{{:~name.slice(3,5)}}</div> {{/if}}{{/for}} </td>{{else}}<td class='e-timecells' style='width:51px;'><div class='e-timecelldivs'></div></td>{{/if}}{{/for}}</tr>{{/for}}</tbody></table></div>");
        return this._timeCellsTemplate;
    };

    var agendaHeaderTemplate = function () {
        this._agendaHeaderTemplate = (
        "<tr>{{for cols}}" +
        "<td class='e-headercells {{:className}} {{if (#index==0)}}e-firstcolumn{{/if}}'>" +
        "<div class='e-textwrapper'><b>{{:headertext}}</b></div>" +
        "</td>" +
        "{{/for}}</tr>");
        return this._agendaHeaderTemplate;
    };

    var agendaCellsTemplate = function () {
        this._agendaCellsTemplate = ("<table class='e-table e-agendacellstab' width='100%' cellpadding='0px' cellspacing='0px'>" +
        "<tbody >{{for cols}}" +
        "<tr>" +
        "{{if resGroup}}{{for resGroup}}<td class='e-agendacells {{:resId}} e-resourcecolumn {{if (#index==0)}}e-firstcolumn{{/if}}'><div class='e-textwrapper'>{{:text}}</div></td>{{/for}}{{/if}}" +
        "{{if resChild}}<td rowspan={{:resRowSpan}} class='e-agendacells e-resourcecolumn {{if (resGroup.length==0)}}e-firstcolumn{{/if}}'><div class='e-textwrapper'>{{if userResourceId == false}}{{:resName}}{{else}}{{:userResourceHtml}}{{/if}}</div></td>{{/if}}" +
        "{{if dayChild}}<td rowspan={{:dayRowSpan}} class='e-agendacells e-datecolumn {{if (resGroup.length != 0) }}{{else (resName =='')}}e-firstcolumn{{/if}}'><div class='e-textwrapper'>{{if userDateId == false}}{{:date}}{{else}}{{:userDateHtml}}{{/if}}</div></td>{{/if}}" +
        "<td class='e-agendacells e-timecolumn {{if dayBorderBottom==false}}e-alternatecells{{/if}}'><div class='e-textwrapper'>{{if userTimeId == false}}{{:time}}{{else}}{{:userTimeHtml}}{{/if}}</div></td>" +
        "<td class='e-agendacells e-eventcolumn {{if dayBorderBottom==false}}e-alternatecells{{/if}}'><div class='e-textwrapper'><div id='Appointment_{{:id}}'  guid= '{{:uid}}' class='e-agendaappointment'>{{if userAppTemplId == false}}<span class='e-agendacolordiv' style='background:{{:background}};'/>{{:event}}{{else}}{{:userTemplate}}{{/if}}{{if readOnly == false}}<div class='e-agendahover'><span class='e-icon e-editapp' title={{:edittext}}/><span class='e-icon e-deleteapp' title={{:deletetext}}/></div>{{/if}}</div></div></td>" +
        "</tr>" +
        "{{/for}}</tbody></table>");
        return this._agendaCellsTemplate;
    };

    ej.widget("ejSchedule", "ej.Schedule", {
        _rootCSS: "e-schedule",
        element: null,
        validTags: ["div"],
        model: null,
        defaults: {
            timeZoneCollection: {
                dataSource: [
                { text: "UTC -12:00", id: "1", value: "UTC -12:00" },
                { text: "UTC -11:00", id: "2", value: "UTC -11:00" },
                { text: "UTC -10:00", id: "3", value: "UTC -10:00" },
                { text: "UTC -09:00", id: "4", value: "UTC -09:00" },
                { text: "UTC -08:00", id: "5", value: "UTC -08:00" },
                { text: "UTC -07:00", id: "6", value: "UTC -07:00" },
                { text: "UTC -06:00", id: "7", value: "UTC -06:00" },
                { text: "UTC -05:00", id: "8", value: "UTC -05:00" },
                { text: "UTC -04:30", id: "9", value: "UTC -04:30" },
                { text: "UTC -04:00", id: "10", value: "UTC -04:00" },
                { text: "UTC -03:30", id: "11", value: "UTC -03:30" },
                { text: "UTC -03:00", id: "12", value: "UTC -03:00" },
                { text: "UTC -02:00", id: "13", value: "UTC -02:00" },
                { text: "UTC -01:00", id: "14", value: "UTC -01:00" },
                { text: "UTC +00:00", id: "15", value: "UTC +00:00" },
                { text: "UTC +01:00", id: "16", value: "UTC +01:00" },
                { text: "UTC +02:00", id: "17", value: "UTC +02:00" },
                { text: "UTC +03:00", id: "18", value: "UTC +03:00" },
                { text: "UTC +03:30", id: "19", value: "UTC +03:30" },
                { text: "UTC +04:00", id: "20", value: "UTC +04:00" },
                { text: "UTC +04:30", id: "21", value: "UTC +04:30" },
                { text: "UTC +05:00", id: "22", value: "UTC +05:00" },
                { text: "UTC +05:30", id: "23", value: "UTC +05:30" },
                { text: "UTC +05:45", id: "24", value: "UTC +05:45" },
                { text: "UTC +06:00", id: "25", value: "UTC +06:00" },
                { text: "UTC +06:30", id: "26", value: "UTC +06:30" },
                { text: "UTC +07:00", id: "27", value: "UTC +07:00" },
                { text: "UTC +08:00", id: "28", value: "UTC +08:00" },
                { text: "UTC +09:00", id: "29", value: "UTC +09:00" },
                { text: "UTC +09:30", id: "30", value: "UTC +09:30" },
                { text: "UTC +10:00", id: "31", value: "UTC +10:00" },
                { text: "UTC +11:00", id: "32", value: "UTC +11:00" },
                { text: "UTC +12:00", id: "33", value: "UTC +12:00" },
                { text: "UTC +13:00", id: "34", value: "UTC +13:00" }
                ],
                text: "text",
                id: "id",
                value: "value",
            },
            views: ["Day", "Week", "WorkWeek", "Month", "Agenda"],
            agendaViewSettings: {
                daysInAgenda: 7,
                dateColumnTemplateId: null,
                timeColumnTemplateId: null,
            },
            currentView: "week",
            timeMode: null,
            firstDayOfWeek: null,
            workWeek: ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday"],
            dateFormat: "",
            isDST: false,
            timeZone: null,
            startHour: 0,
            endHour: 24,
            highlightBusinessHours: true,
            enableLoadOnDemand: false,
            enablePersistence: false,
            showQuickWindow: true,
            showLocationField: false,
            businessStartHour: 9,
            businessEndHour: 18,
            workHours: {
                highlight: null,
                start: null,
                end: null,
            },
            width: "100%",
            height: "1120px",
            cellHeight: "20px",
            cellWidth: "",
            currentDate: new Date(),
            minDate: new Date(1900, 01, 01),
            maxDate: new Date(2099, 12, 31),
            cssClass: "",
            locale: "en-US",
            enableResize: false,
            readOnly: false,
            enableRTL: false,
            enableAppointmentNavigation: true,
            appointmentTemplateId: null,
            resourceHeaderTemplateId: null,
            allowDragDrop: true,
            enableAppointmentResize: true,
            showCurrentTimeIndicator: true,
            showAllDayRow: true,
            showHeaderBar: true,
            isResponsive: true,
            enableRecurrenceValidation: true,
            reminderSettings: {
                enable: false,
                alertBefore: 5
            },
            contextMenuSettings: {
                enable: false,
                menuItems: {
                    appointment: [
                        { id: "open", text: "Open Appointment" },
                        { id: "delete", text: "Delete Appointment" }
                    ],
                    cells: [
                        { id: "new", text: "New Appointment" },
                        { id: "recurrence", text: "New Recurring Appointment" },
                        { id: "today", text: "Today" },
                        { id: "gotodate", text: "Go to date" },
                        { id: "settings", text: "Settings" },
                        { id: "view", text: "View", parentId: "settings" },
                        { id: "timemode", text: "TimeMode", parentId: "settings" },
                        { id: "view_Day", text: "Day", parentId: "view" },
                        { id: "view_Week", text: "Week", parentId: "view" },
                        { id: "view_Workweek", text: "Workweek", parentId: "view" },
                        { id: "view_Month", text: "Month", parentId: "view" },
                        { id: "view_Agenda", text: "Agenda", parentId: "view" },
                        { id: "timemode_Hour12", text: "12 Hours", parentId: "timemode" },
                        { id: "timemode_Hour24", text: "24 Hours", parentId: "timemode" },
                        { id: "businesshours", text: "Business Hours", parentId: "settings" }
                    ]
                }
            },
            group: null,
            resources: null,
            allowKeyboardNavigation: true,
            appointmentSettings: {
                dataSource: [],
                query: null,
                tableName: null,
                id: "Id",
                subject: "Subject",
                description: "Description",
                startTime: "StartTime",
                endTime: "EndTime",
                recurrence: "Recurrence",
                recurrenceRule: "RecurrenceRule",
                allDay: "AllDay",
                resourceFields: null,
                categorize: null,
                recurrenceId: "RecurrenceId",
                recurrenceExDate: "recurrenceExDate",
                location: null,
                priority: null,
                startTimeZone: "StartTimeZone",
                endTimeZone: "EndTimeZone"
            },
            renderDates: null,
            orientation: "vertical",
            categorizeSettings: {
                enable: false,
                allowMultiple: false,
                dataSource:
                   [
                   { text: "Blue Category", id: 1, color: "#7499e1", fontColor: "green" },
                   { text: "Green Category", id: 2, color: "#7cce6e", fontColor: "green" },
                   { text: "Orange Category", id: 3, color: "#f19d5a", fontColor: "green" },
                   { text: "Purple Category", id: 4, color: "#937bd1", fontColor: "green" },
                   { text: "Red Category", id: 5, color: "#d98889", fontColor: "green" },
                   { text: "Yellow Category", id: 6, color: "#f8f264", fontColor: "green" }
                   ],
                text: "text",
                id: "id",
                color: "color",
                fontColor: "fontColor"
            },
            prioritySettings: {
                enable: false,
                template: null,
                dataSource:
                [
                { text: "None", value: "none" },
                { text: "High", value: "high" },
                { text: "Medium", value: "medium" },
                { text: "Low", value: "low" }
                ],
                text: "text",
                value: "value"
            },
            cellClick: null,
            cellHover: null,
            appointmentClick: null,
            appointmentHover: null,
            cellDoubleClick: null,
            appointmentWindowOpen: null,
            appointmentSaved: null,
            appointmentEdited: null,
            appointmentDeleted: null,
            navigation: null,
            dragStart: null,
            drag: null,
            dragStop: null,
            resizeStart: null,
            resize: null,
            resizeStop: null,
            menuItemClick: null,
            beforeContextMenuOpen: null,
            reminder: null,
            actionBegin: null,
            actionComplete: null,
            showTimeScale: true,
            tooltipSettings: {
                enable: false,
                template: null,
            },
        },
        dataTypes: {
            views: "array",
            workWeek:"array",
            timeZoneCollection: {
                dataSource: "data",
                query: "data"
            },
            renderDates: "data",
            enablePersistence: "boolean",
            showTimeScale: "boolean",
            resources: "array",
            group: {
                resources: "array"
            },
            appointmentSettings: {
                dataSource: "data",
                query: "data"
            },
            categorizeSettings: {
                dataSource: "data",
                query: "data"
            },
            prioritySettings: {
                dataSource: "data",
                query: "data"
            },
            contextMenuSettings: {
                menuItems: {
                    appointment: "array",
                    cells: "array",
                }
            }
        },
        _ignoreOnPersist: [
            "appointmentSettings.query", "appointmentSettings.dataSource", "timeZoneCollection.dataSource",
            "allowKeyboardNavigation", "showQuickWindow", "cssClass", "locale", "enableResize", "showHeaderBar",
            "readOnly", "enableRTL", "enableAppointmentNavigation", "appointmentTemplateId", "resourceHeaderTemplateId", "allowDragDrop", "enableAppointmentResize",
            "showCurrentTimeIndicator", "reminderSettings.enable", "contextMenuSettings.enable", "group.resources", "categorizeSettings.enable", "categorizeSettings.dataSource",
            "cellClick", "appointmentClick", "cellDoubleClick", "appointmentWindowOpen", "appointmentSaved", "appointmentEdited", "appointmentDeleted", "navigation", "dragStart", "dragStop",
            "resizeStart", "resize", "resizeStop", "menuItemClick", "beforeContextMenuOpen", "reminder", "prioritySettings.dataSource", "prioritySettings.enable", "agendaViewSettings.daysInAgenda", "agendaViewSettings.dateColumnTemplateId", "agendaViewSettings.timeColumnTemplateId"
        ],
        _ignoreFromAppointmentOnExport: ["dataSource", "query", "tableName", "resourceFields"],
        _resources: function (index, property, value, old) {
            this.element.empty();
            this._bindResourcesData();
            if (!ej.isNullOrUndefined(this.dataSource()))
                this._renderAppointmentAll();
        },
        observables: ["appointmentSettings.dataSource", "currentView", "currentDate"],
        _tags: [
            { tag: "timeZoneCollection", attr: [], "singular": "timeZoneCollection" },
            { tag: "appointmentSettings.dataSource", attr: [], "singular": "dataSource" },
            { tag: "resources", attr: ["allowMultiple", "field", "title", "name", "resourceSettings"] },
            { tag: "contextMenuSettings.menuItems.appointment", attr: ["id", "text", "parentId"] },
            { tag: "contextMenuSettings.menuItems.cells", attr: ["id", "text", "parentId"] },
            { tag: "categorizeSettings.dataSource", attr: ["text", "id", "color", "fontColor"] },
            { tag: "timeZoneCollection.dataSource", attr: ["text", "id", "value"] },
			{ tag: "prioritySettings.dataSource", attr: ["text", "value"] }
        ],
        currentView: ej.util.valueFunction("currentView"),
        currentDate: ej.util.valueFunction("currentDate"),
        dataSource: ej.util.valueFunction("appointmentSettings.dataSource"),

        _init: function () {
            // Added while deprcating the Business hours API
            if (ej.isNullOrUndefined(this.model.workHours.start)) this.model.workHours.start = this.model.businessStartHour;
            if (ej.isNullOrUndefined(this.model.workHours.end)) this.model.workHours.end = this.model.businessEndHour;
            if (ej.isNullOrUndefined(this.model.workHours.highlight)) this.model.workHours.highlight = this.model.highlightBusinessHours;
            if (ej.isNullOrUndefined(this.model.timeZoneCollection)) this.model.timeZoneCollection = this.model.timezoneCollection;
            this._currentDateFormat(this.model.dateFormat);
            this._trigger("load");
            this.element.addClass("e-schedule");
            if (ej.isNullOrUndefined(this.model.appointmentSettings.query) || !(this.model.appointmentSettings.query instanceof ej.Query))
                this.model.appointmentSettings.query = ej.Query();
            (this.model.cssClass != null) && this.element.addClass(this.model.cssClass);
            this._initPrivateProperties();
            if (!ej.isNullOrUndefined(this.model.categorizeSettings) && (this.model.categorizeSettings.dataSource instanceof ej.DataManager))
                this._bindCategorizeData();
            if (!ej.isNullOrUndefined(this.model.prioritySettings) && (this.model.prioritySettings.dataSource instanceof ej.DataManager))
                this._bindPriorityData();
            if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                this._bindResourcesData();
            else
                this._renderInitSchedule();
            this._wireEvents();
            !ej.isNullOrUndefined(this.model.appointmentSettings) && (this._dataManager = this.dataSource() instanceof ej.DataManager ? this.dataSource() : this.dataSource() != null ? ej.DataManager(this.dataSource()) : null);
            this._bindAppointmentsData("Load");
            this.model.enableRTL && this.element.addClass("e-rtl");
            if (this.model.reminderSettings.enable) {
                this._checkReminders();
                this._reminderInterval = window.setInterval($.proxy(this._checkReminders, this), 60000);
            }
            this._businessHourScroller();
        },

        _setModel: function (options) {
            var flag = false, resFlag = false;
            for (var prop in options) {
                switch (prop) {
                    case "views":
                        this.model.views = options[prop];
                        this.hTD.find("table.e-viewstable").remove();
                        this.hTD.find("div.e-viewsdiv").append(this._renderViewNavigation(options[prop].toString().toLowerCase().split(",")));
                        this.element.find(".e-commonviewbutton.e-" + this.currentView()).addClass("e-activeview").attr("tabIndex", 0).focus();
                        if (!ej.isNullOrUndefined(this._scheduleDrawer)) {
                            this._scheduleDrawer.ejNavigationDrawer('destroy');
                            $(this._scheduleDrawer).remove(); this._scheduleDrawer = null;
                            ((this._mediaQuery || this.element.width() < 600) && !$("#" + this._id + "navDrawer").hasClass("e-scheduledrawer")) && this._renderNavigationDrawer();
                        }
                        if (this.model.views.toString().toLowerCase().split(',').indexOf(this.currentView()) == -1) {
                            this._navView = true;
                            this._viewChange(null, null, this.model.views[0].toLowerCase());
                        }
                        break;
                    case "currentView":
                        this._navView = true;
                        (this.model.views.toString().toLowerCase().split(',').indexOf(ej.util.getVal(options[prop]).toLowerCase()) != -1) ? this._viewChange(null, null, ej.util.getVal(options[prop]).toLowerCase()) : this.currentView((this.element.find(".e-activeview")[0].className.split("e-")[2]).split(" ")[0]);
                        break;
                    case "currentDate":
                        this._setCurrentDate = true;
                        if ($.type(options[prop]) == "string")
                            this._currentDateFormat(this.model.dateFormat);
                        ((this.currentDate() < this.model.minDate) && this.currentDate(this.model.minDate));
                        ((this.currentDate() > this.model.maxDate) && this.currentDate(this.model.maxDate));
                        this._navigateClick(null, "DateNavigation");
                        break;
                    case "minDate":
                    case "maxDate":
                        this._currentDateFormat(this.model.dateFormat);
                        flag = true;
                        break;
                    case "timeMode":
                        this.model.timeMode = this._timeMode = options[prop];
                        this._appointmentAddWindow.find('.startendtime').ejTimePicker({ timeFormat: (this._timeMode == "12") ? this._pattern.t : "HH:mm" });
                        if (this.model.orientation == "horizontal" && this.currentView() !== "month" && !(this._isCustomView()) && this.currentView() !== "agenda") {
                            var view = this.model.showTimeScale ? (this._isCustomView()) ? "month" : this.currentView() : "month"; var weekcols = Globalize.culture(this.model.locale);
                            var viewColCount = view == "day" ? 1 : view == "week" ? 7 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : 5;
                            this.datesColumn = this._getHeaderAllDayCellsCount();
                            var multiResources = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? true : false;
                            var html = this.horizontalHeader.render({ cols: this.datesColumn, label: this._getLocalizedLabels("Resources"), multiRes: multiResources, view: view, timeTdCount: this._getStartEndTime(), colspan: view == "month" ? 1 : (this._getStartEndTime().length / viewColCount) * 2, height: view == 'month' ? 62 : 59, width: this.model.width, headercellWidth: view == "day" ? "100%" : "55px" });
                            this.aTR.children().remove();
                            this.aTR.append(html);
                            this._borderAddRemove();
                            this._moveScroller();
							(this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                        	this._businessHourScroller();
                        }
                        else {
                            if (this.currentView() !== "month" && !(this._isCustomView()) && this.currentView() !== "agenda") {
                                var viewclass = this.currentView() == "month" ? 1 : 0;
                                var timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });
                                this.$TimeCellTD.children().remove();
                                this.$TimeCellTD.append(timeCellhtml); if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                                this.element.find(".e-scrolltimecells").css("top", -(this.element.find(".e-draggableworkarea").scrollTop()) + "px");
                                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                            }
                        }
                        if (!ej.isNullOrUndefined(this.dataSource()))
                            this._renderAppointmentAll();
                        break;
                    case "timeZone":
                        this._setmodelvalue = true;
                        this.model.timeZone = options[prop];
                        this.element.find("div.e-categorybar").remove();
                        this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,.e-appsoverflow").remove();
                        this._bindAppointmentsData("Load");
                        break;
                    case "timezoneCollection":
                    case "timeZoneCollection":
                        $.extend(this.model.timeZoneCollection, options[prop]);
                        this._appointmentAddWindow.find('.startutc').ejDropDownList({ dataSource: this.model.timeZoneCollection.dataSource });
                        this._appointmentAddWindow.find('.endutc').ejDropDownList({ dataSource: this.model.timeZoneCollection.dataSource });
                        break;
                    case "startHour":
                        this.model.startHour = options[prop];
                        (this.currentView() !== "agenda") && this._setNewStartEndHours();
                        break;
                    case "endHour":
                        this.model.endHour = options[prop];
                        (this.currentView() !== "agenda") && this._setNewStartEndHours();
                        break;
                    case "highlightBusinessHours":
                        this.model.workHours.highlight = options[prop];
                        if (this.currentView() !== "agenda") {
                            (this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") && this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                            (this.model.workHours.highlight == false) ? this._businessHourScroller() : "";
                        }
                        break;
                    case "workHours":
                        $.extend(this.model.workHours, options[prop]);
                        if (this.currentView() !== "agenda") {
                            (this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") && this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                            (this.model.workHours.highlight == false) ? this._businessHourScroller() : "";
                        }
                        break;
                    case "showAllDayRow":
                        this.model.showAllDayRow = options[prop];
                        (!this.model.showTimeScale) && (this.model.showAllDayRow = false);
                        if (this.model.showTimeScale && this.currentView() != "agenda") {
                            if (this.currentView() != "month") {
                                this.aTR.empty();
                                this._renderHeaderAllDayTemplate();
                                this._borderAddRemove();
                                this._headerbarWidth();
                                this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
                                (this.model.showAllDayRow) && this._reRenderAllDayAppointments();
                                if (this.model.orientation == "vertical" && this.model.cellWidth) {
                                    this.element.find(".e-workcellstab tr td:first-child").width(this.element.find(".e-headercells").width());
                                }
                                this.refreshScroller();
                                this.element.find(".e-emptytd").length < 1 && this.model.orientation == "vertical" ? this.aTR.find(".e-headeralldaydiv").parent().next().append(this._emptyCell()) : "";
                                this.element.find(".e-emptyalldaytd").css('height', this.element.find(".e-alldaycells").height() + "px");
                                this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());                               
                            }
                            else
                                this.refreshScroller();
                        }
                        break;
                    case "isResponsive":
                        this.model.isResponsive = options[prop];
                        this._onScheduleResize();
                        break;
                    case "isDST":
                        this.model.isDST = options[prop];
                        if (ej.isNullOrUndefined(this.model.timeZone)) {
                            var today = new Date();
                            this._timeValue = this._true == false ? -(today.getTimezoneOffset() - this._stdTimezoneOffset()) : today.getTimezoneOffset() - this._stdTimezoneOffset();
                            this._true = (this.model.isDST == true) ? false : true;
                            this._dayLight = true;
                            this.element.find("div.e-categorybar").remove();
                            this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,div.e-appsoverflow").remove();
                            !ej.isNullOrUndefined(this.model.appointmentSettings) && (this._dataManager = this.dataSource() instanceof ej.DataManager ? this.dataSource() : this.dataSource() != null ? ej.DataManager(this.dataSource()) : null);
                            this._bindAppointmentsData("Load");
                        }
                        break;
                    case "enableRecurrenceValidation":
                        this.model.enableRecurrenceValidation = options[prop];
                        (this.model.enableRecurrenceValidation && ej.isNullOrUndefined(this._alertWindow)) && this._renderAlertWindow();
                        this._dataProcessing(this._currentAppointmentData);
                        this._renderAppointmentAll();
                        break;
                    case "businessStartHour":
                        this.model.workHours.start = options[prop];
                        if (this.currentView() !== "agenda") {
                            this.currentView() !== "month" && !(this._isCustomView()) ? this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") : this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                            this.model.workHours.highlight && this._highlightBusinessHours();
                        }
                        break;
                    case "businessEndHour":
                        this.model.workHours.end = options[prop];
                        if (this.currentView() !== "agenda") {
                            this.currentView() !== "month" && !(this._isCustomView()) ? this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") : this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                            this.model.workHours.highlight && this._highlightBusinessHours();
                        }
                        break;
                    case "height":
                        this.model.height = options[prop];
                        if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                        flag = true;
                        break;
                    case "width":
                        this.model.width = options[prop];
                        this._clientWidth = this.model.width;
                        if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                        flag = true;
                        break;
                    case "cssClass":
                        this.element.removeClass(this.model.cssClass).addClass(options[prop]);
                        break;
                    case "locale":
                        this.model.locale = options[prop];
                        var culture = Globalize.culture(this.model.locale);
                        this._dayShortNames = culture.calendar.days.namesAbbr;
                        this._dayFullNames = culture.calendar.days.names;
                        this._monthNames = culture.calendar.months.names;
                        this._pattern = culture.calendar.patterns;
						if(this.model.contextMenuSettings.enable) this._gotoDate=true;
                        flag = true;
                        break;
                    case "dateFormat":
                        this._appointmentAddWindow.find('.startenddate').ejDatePicker({ dateFormat: options[prop] });
                        this._appointmentAddWindow.find('.recurstartdate').ejDatePicker({ dateFormat: options[prop] });
                        this._appointmentAddWindow.find('.e-until').ejDatePicker({ dateFormat: options[prop] });
                        this._currentDateFormat(options[prop]);
                        break;
                    case "enableResize":
                        this.model.enableResize = options[prop];
                        if (this.model.enableResize) {
                            !ej.isTouchDevice() ? this._on($(window), "resize", this._scheduleResize) : this._on($(window), "orientationchange", this._scheduleResize);
                            this._wireResizeEvents();
                        }
                        break;
                    case "readOnly":
                        this.model.readOnly = options[prop];
                        this._renderAppointmentAll();
                        break;
                    case "showCurrentTimeIndicator":
                        this.model.showCurrentTimeIndicator = options[prop];
                        if (this.currentView() !== "agenda") {
                            this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                            if (!this.model.showCurrentTimeIndicator) this.currentView() !== "month" && !(this._isCustomView()) ? this.element.find(".e-highlightcurrenttimeline").remove() : $(this.element.find(".e-monthcells")).removeClass("e-monthcurrenttime");
                        }
                        break;
                    case "enableRTL":
                        options[prop] ? this.element.addClass("e-rtl") : this.element.removeClass("e-rtl");
                        this.model.enableRTL = options[prop];
                        flag = true;
                        this._quickAppointWindow = null;
                        break;
                    case "appointmentTemplateId":
                        this.model.appointmentTemplateId = options[prop];
                        this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,.e-appsoverflow").remove();
                        this.element.find("div.e-categorybar").remove();
                        if (!ej.isNullOrUndefined(this.dataSource()))
                            this._renderAppointmentAll();
                        break;
                    case "allowKeyboardNavigation":
                        this.model.allowKeyboardNavigation = options[prop];
                        if (this.model.allowKeyboardNavigation) {
                            this._on(this.element, "keydown", $.proxy(this._onKeyDown, this));
                            this._on($(document), "keydown", $.proxy(this._documentOnKeyDown, this));
                        }
                        else {
                            this._off(this.element, "keydown");
                            this._off($(document), "keydown");
                        }
                        break;
                    case "renderDates":
                        $.extend(this.model.renderDates, options[prop]);
                        this._navView = true; this._navigate = false;
                        this._viewChange(null, null, "customview");
                        break;
                    case "showTimeScale":
                        this.model.showTimeScale = options[prop];
                        if (this.model.showTimeScale == true) {
                            this.model.showAllDayRow = true;
                            this.model.showCurrentTimeIndicator = true;
                        }
                        else {
                            this.model.showAllDayRow = false;
                            this.model.showCurrentTimeIndicator = false;
                        }
                        (this.currentView() !== "agenda") && (flag = true);
                        break;
                    case "appointmentSettings":
                        if (!$.isEmptyObject(options[prop])) {
                            $.extend(this.model.appointmentSettings, options[prop]);
                            this.element.find("div.e-categorybar").remove();
                            this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,div.e-appsoverflow").remove();
                            !ej.isNullOrUndefined(this.model.appointmentSettings) && (this._dataManager = this.dataSource() instanceof ej.DataManager ? this.dataSource() : this.dataSource() != null ? ej.DataManager(this.dataSource()) : null);
                            this._bindAppointmentsData("Load");
                        }
                        break;
                    case "contextMenuSettings":
                        $.extend(this.model.contextMenuSettings, options[prop]);
                        if (this.model.contextMenuSettings.enable && ej.isNullOrUndefined(this._contextMenu))
                            this._showContextMenu(this.model.contextMenuSettings.enable);
                        break;
                    case "reminderSettings":
                        if (options[prop].enable)
                            this._checkReminders() && (this._reminderInterval = window.setInterval($.proxy(this._checkReminders, this), 60000));
                        else
                            window.clearInterval(this._reminderInterval);
                        break;
                    case "enableAppointmentNavigation":
                        this.model.enableAppointmentNavigation = options[prop];
                        if (!this.model.enableAppointmentNavigation)
                            this.element.find(".e-prevapp,.e-nextapp").remove();
                        else {
                            this._nextPrevAppointment(this.model.enableAppointmentNavigation);
                            if (!ej.isNullOrUndefined(this.dataSource()))
                                this._renderAppointmentAll();
                        }
                        break;
                    case "enableLoadOnDemand":
                        this.model.enableLoadOnDemand = options[prop];
                        this.element.find("div.e-categorybar").remove();
                        this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,.e-appsoverflow").remove();
                        this._bindAppointmentsData("Load");
                        break;
                    case "resources":
                        if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources)) {
                            ej.copyObject(this.model.resources, options[prop]);
                            flag = true; resFlag = true;
                        }
                        break;
                    case "categorizeSettings":
                        var control;
                        $.extend(this.model.categorizeSettings, options[prop]);
                        if (!ej.isNullOrUndefined(this.model.categorizeSettings)) {
                            if (this.model.categorizeSettings.enable && this._appointmentAddWindow.find('.e-categorizevalue').length == 0) {
                                var $appWindow = "<tr id='" + this._id + "categorize'><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("Categorize") + ":</td><td colspan='3' class='e-rightfields'>" + "<input id= '" + this._id + "_categorize' class='e-categorizevalue' type='text' value=''/><div id='ownerlist'></div></td></tr>";
                                this._appointmentAddWindow.find('.' + this._id + "summary").after($appWindow);
                                this._renderTemplates();
                                control = this._appointmentAddWindow;
                                this._renderCategoryItems(control);
                            }
                            else
                                this._appointmentAddWindow.find('#' + this._id + "categorize").remove();
                            this._showContextMenu(this.model.contextMenuSettings.enable);
                            this._appointmentAddWindow.find(".e-categorizevalue").ejAutocomplete("clearText");
                            control = this._appointmentAddWindow;
                            this._renderCategoryItems(control);
                            if (!ej.isNullOrUndefined(this.dataSource()))
                                this._renderAppointmentAll();
                        }
                        break;

                    case "group":
                        if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources)) {
                            $.extend(this.model.group, options[prop]);
                            flag = true; resFlag = true;
                        }
                        break;
                    case "cellWidth":
                        this.model.cellWidth = options[prop];
                        if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                        (this.currentView() !== "agenda") && (flag = true);
                        break;
                    case "cellHeight":
                        this.model.cellHeight = options[prop];
                        if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                        (this.currentView() !== "agenda") && (flag = true);
                        break;
                    case "orientation":
                        this.model.orientation = options[prop];
                        if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && this.model.orientation == "horizontal")
                            this._on(this.element, "click", "td.e-parentnodecategory", $.proxy(this._onResourceClick, this));
                        flag = true;
                        break;
                    case "resourceHeaderTemplateId":
                        this.model.resourceHeaderTemplateId = options[prop];
                        if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) {
                            if (this.currentView() != "agenda") {
                                if (this.model.orientation == "horizontal") {
                                    var resDiv = $(this.wTR.find("td")[0]);
                                    resDiv.find("div.e-horizontresdiv").remove();
                                    var resHtml = this._renderHorizontalResources();
                                    resDiv.html(resHtml);
                                }
                                else {
                                    this.aTR.empty();
                                    this._renderHeaderAllDayTemplate();
                                    this._borderAddRemove();
                                    this._headerbarWidth();
                                    this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
                                    (this.model.showAllDayRow) && this._reRenderAllDayAppointments();
                                }
                            }
                            else
                                this._renderAgendaContent();
                        }
                        break;
                    case "showLocationField":
                        this.model.showLocationField = options[prop];
                        if (this.model.showLocationField) {
                            $appWindow = "<tr id='" + this._id + "showLocation' class='" + this._id + "showLocation'><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("Location") + ":</td><td colspan='3' class='e-rightfields'>" +
                                 "<input id='" + this._id + "location' tabIndex='0' class='location e-inputtext' name='" + this.model.appointmentSettings["location"] + "' type='text' /></td></tr>";
                            this._appointmentAddWindow.find("." + this._id + "subjecttr").after($appWindow);
                        }
                        else {
                            this._appointmentAddWindow.find("." + this._id + "showLocation").remove();
                        }
                        break;
                    case "prioritySettings":
                        $.extend(this.model.prioritySettings, options[prop]);
                        if (this.model.prioritySettings.enable) {
                            if (this._appointmentAddWindow.find("." + this._id + "priorityshow").length == 0) {
                                $appWindow = "<td colspan='1' class='e-textlabel " + this._id + "priorityshow'>" + this._getLocalizedLabels("Priority") + ":</td><td colspan='1' class='" + this._id + "prioritylistshow'><input id='" + this._id + "_priority' class='priority' type='text' name='" + this.model.appointmentSettings["priority"] + "' value='' /><div id='prioritylist'></div></td>";
                                this._appointmentAddWindow.find("." + this._id + "parrow").prepend($appWindow);
                            }
                            var control = this._appointmentAddWindow;
                            this._renderPriorityItems(control);
                            this._renderTemplates();
                        }
                        else {
                            this._appointmentAddWindow.find("." + this._id + "priorityshow").remove();
                            this._appointmentAddWindow.find("." + this._id + "prioritylistshow").remove();
                        }
                        if (!ej.isNullOrUndefined(this.dataSource())) {
                            this._renderAppointmentAll();
                        }
                        break;
                    case "showQuickWindow":
                        this.model.showQuickWindow = options[prop];
                        (this.model.showQuickWindow && ej.isNullOrUndefined(this._quickAppointWindow)) && (this._renderQuickAppWindow() || this._renderQuickAppDetailsView());
                        (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
                        break;
                    case "allowDragDrop":
                        this.model.allowDragDrop = options[prop];
                        if (this.currentView() == "agenda") return false;
                        this.model.allowDragDrop ? this._dragdropAppointments() : this._unWireDragDrop();
                        break;
                    case "enableAppointmentResize":
                        this.model.enableAppointmentResize = options[prop];
                        if (this.currentView() == "agenda") return false;
                        this._renderAppointmentAll();
                        break;
                    case "showHeaderBar":
                        this.model.showHeaderBar = options[prop];
                        if (this.model.showHeaderBar) {
                            this.element.find(".e-scheduleheader").show();
                            this._reRenderHeaderBar();
                        }
                        else {
                            this.element.find(".e-scheduleheader").empty();
                            this.element.find(".e-scheduleheader").hide();
                        }
                        this.refreshScroller();
                        break;
                    case "tooltipSettings":
                        $.extend(this.model.tooltipSettings, options[prop]);
                        if (!ej.isNullOrUndefined(this.model.tooltipSettings) && this.model.tooltipSettings.enable) {
                            this._on(this.element, "mouseover", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._showToolTip, this));
                            this._on(this.element, "mouseout", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._hideToolTip, this));
                        }
                        else {
                            this._off(this.element, "mouseover", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._showToolTip, this));
                            this._off(this.element, "mouseout", ".e-appointment,.e-alldayappointment,.e-monthappointment", $.proxy(this._hideToolTip, this));
                        }
                        break;
                    case "firstDayOfWeek":
                        this.model.firstDayOfWeek = this._firstdayofweek = options[prop];
                        if (this._firstdayofweek)
                            flag = true;
                        break;
                    case "workWeek":
                        this.model.workWeek = options[prop];
                        if (this.model.workWeek == null || this.model.workWeek.length >= 0)
                            flag = true;
                        break;
                    case "agendaViewSettings":
                        $.extend(this.model.agendaViewSettings, options[prop]);
                        if (this.currentView() == "agenda") {
                            (this.model.showHeaderBar) && (this.element.find("div.e-datedisplay")[0].innerHTML = this._getCurrentDateHeaderValue());
                            this._renderAgendaContent();
                        }
                        break;
                }
            }
            if (flag) {
                this.element.empty();
                $("#" + this._id + "AppointmentAddEditWindow_wrapper,#" + this._id + "AppointmentQuickWindow_wrapper,#" + this._id + "AppDetailsWindow_wrapper,#" + this._id + "RecurrenceEdit_wrapper").remove();
                (!resFlag) ? this._renderInitSchedule() : this._bindResourcesData();
                (this.model.workHours.highlight == true) ? this._businessHourScroller() : "";
                if (!ej.isNullOrUndefined(this.dataSource()))
                    this._renderAppointmentAll();
            }
        },

        _touchSchedule: function (e) {
            if ($(e.target).is(".e-appointment,.e-alldayappointment,.e-monthappointment") || $(e.target).parents().is(".e-appointment,.e-alldayappointment,.e-monthappointment"))
                return false;
            var event, value = parseInt(this.element.find(".e-draggableworkarea").css("margin-left"));
            switch (e.type) {
                case "swipeleft":
                    if (this._mediaQuery && this.currentView() != "agenda") {
                        if (-value < (this.element.find(".e-draggableworkarea").width() - (this.element.find(".e-draggableworkarea").width() / this.res1.length))) {
                            this.element.find(".e-headeralldaydiv,.e-draggableworkarea").animate({ "margin-left": value + (-this.element.find(".e-draggableworkarea").width() / this.res1.length) }, 500);
                            return false;
                        }
                        this.element.find(".e-headeralldaydiv,.e-draggableworkarea").css("margin-left", "0px");
                    }
                    event = { currentTarget: this.element.find('.e-navigatenext') };
                    this._navigateClick(event, "Next");
                    break;
                case "swiperight":
                    if (this._mediaQuery && this.currentView() != "agenda") {
                        if (-value > 0) {
                            this.element.find(".e-headeralldaydiv,.e-draggableworkarea").animate({ "margin-left": value + this.element.find(".e-draggableworkarea").width() / this.res1.length }, 500);
                            return false;
                        }
                        this.element.find(".e-headeralldaydiv,.e-draggableworkarea").css("margin-left", -(this.element.find(".e-draggableworkarea").width() - (this.element.find(".e-draggableworkarea").width() / this.res1.length)));
                    }
                    event = { currentTarget: this.element.find('.e-navigateprevious') };
                    this._navigateClick(event, "Previous");
                    break;
            }
        },

        _initPrivateProperties: function () {
            this._navSelected = this._navDrawer = this._alldayLevel = this._reminderClose = this._navView = this._editKey = this._cellClick = this._recurChecked = this._setCurrentDate = this._leftResizing = this._rightResizing = this._topResizing = this._bottomResizing = this._navigate = this._navigate = false;
            this._dragDrop = this._gotoDate = this._bindapp = true;
            this._tempStart = this._tempEnd = this._currentAction = "";
            this._currentRowIndex = this._currentColIndex = 0;
            this._appId = this._parentId = this._splitId = 1;
            this.overlapList = this._currentAppointmentData = this._processed = this._dateRender = [];
            this.res1 = [0];
            this._appMainId = null;
            this._rRule = {};
            this._dayNamesArray = ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"];
            this._dayNames = ["SU", "MO", "TU", "WE", "TH", "FR", "SA"];
            var culture = Globalize.culture(this.model.locale);
            this._dayShortNames = culture.calendar.days.namesAbbr;
            this._dayFullNames = culture.calendar.days.names;
            this._monthNames = culture.calendar.months.names;
            this._pattern = culture.calendar.patterns;
            this._browserInfo = ej.browserInfo();
			this._focused = null;
        },

        _destroy: function () {
            this._destorySubControls($("#" + this._id + "_AddEditForm"));
            this._destorySubControls($("#" + this._id + "_RecurForm"));
            this.element.find("#" + this._id + "_navigator").ejDatePicker("destroy");
            window.clearInterval(this._intervalID);
            $("#" + this._id + "AppointmentAddEditWindow_wrapper,#" + this._id + "AppointmentQuickWindow_wrapper,#" + this._id + "AppDetailsWindow_wrapper,#" + this._id + "RecurrenceEdit_wrapper").remove();
            $("#" + this._id + "_WorkCells_Template,#" + this._id + "_MonthCells_Template,#" + this._id + "_TimeCells_Template,#" + this._id + "_HeaderDate_Template,#" + this._id + "_Appointment_Template,#" + this._id + "_AlldayCells_Template,#" + this._id + "_LeftIndentCells_Template").remove();
            this.element.empty().removeClass("e-schedule");
        },

        _destorySubControls: function (formelement) {
            var $formEle = formelement.find('.e-timepicker,.e-datepicker,.e-dropdownlist,.e-autocomplete');
            for (var index = 0; index < $formEle.length; index++) {
                var $element = $($formEle[index]);
                if ($element.hasClass('e-timepicker'))
                    $element.ejTimePicker("destroy");
                else if ($element.hasClass('e-datepicker'))
                    $element.ejDatePicker("destroy");
                else if ($element.hasClass('e-dropdownlist'))
                    $element.ejDropDownList("destroy");
                else if ($element.hasClass('e-autocomplete'))
                    $element.ejAutocomplete("destroy");
            }
        },

        _wireEvents: function () {
            this._on((this.element, $(window)), "mousedown", $.proxy(this._onScheduleMouseDown, this));
            this._on(this.element, "click", "div.e-navigateprevious,div.e-navigatenext", $.proxy(this._navigateClick, this));
            this._on($(window), "resize", $.proxy(this._onScheduleResize, this));
            this._on(this.element, "click", ".e-commonviewbutton", $.proxy(this._viewChange, this));
            this._on(this.element, "click", ".e-datedisplay", $.proxy(this._viewCalendar, this));
            this._on(this.element, "click", "div.e-appsoverflow", $.proxy(this._appOverflowClick, this));
            this._on(this.element, "click", ".e-prevapp.enable,.e-nextapp.enable", $.proxy(this._nextPrev, this));
            this._on(this.element, "dblclick", ".e-headercells", $.proxy(this._navigateView, this));
            this._on(this.element, "click", ".e-headercells,.e-timecells,td.e-monthcells div.e-monthheader", $.proxy(this._navigateView, this));
            this._on(this.element, "mouseover", "td.e-alldaycells,td.e-workcells:not('.e-resourceheadercells'),td.e-monthcells,td.e-agendacells", $.proxy(this._onCellMouseOver, this));
            this._on(this.element, "mouseout", "td.e-alldaycells,td.e-workcells:not('.e-resourceheadercells'),td.e-monthcells,td.e-agendacells", $.proxy(this._onCellMouseOver, this));
            this._on(this.element, "click", ".e-workcells,.e-alldaycells,.e-alldayappointment,.e-monthcells,.e-agendacells", $.proxy(this._cellSelection, this));
            if (!this._mediaQuery) {
                this._on(this.element, ej.eventType.mouseDown, "td.e-workcells,td.e-monthcells,td.e-alldaycells", $.proxy(this._mouseButtonClick, this));
                this._on(this.element, "keydown click", "td.e-workcells,td.e-monthcells,td.e-alldaycells", $.proxy(this._shiftSelection, this));
            }
            this._on(this.element, (ej.isMobile()) ? "click" : "dblclick", ".e-workcells:not('.e-resourceheadercells'),.e-alldaycells,.e-alldayappointment,.e-monthcells", $.proxy(this._appointmentWindow, this));
            (this._mediaQuery || ej.isMobile()) && this._on(this.element, "tap", ".e-appointment,.e-alldayappointment,div.e-monthappointment,.e-agendaappointment", $.proxy(this._appointmentWindow, this));
            this._on(this.element, "mouseover", ".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment", $.proxy(this._onAppointMouseOver, this));
            this._on(this.element, "mouseout", ".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment", $.proxy(this._onAppointMouseOver, this));
            this._on(this.element, "mouseover", ".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment", $.proxy(this._showToolTip, this));
            this._on(this.element, "mouseout", ".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment", $.proxy(this._hideToolTip, this));
            (this._mediaQuery || this.element.width() < 600) && this._on(this.element, "click", "#" + this._id + "navDrawerIcon", $.proxy(this._drawerOpen, this));
            if (this.model.allowKeyboardNavigation) {
                this._on(this.element, "keydown", $.proxy(this._onKeyDown, this));
                this._on($(document), "keydown", $.proxy(this._documentOnKeyDown, this));
            }
            if (this.model.orientation == "horizontal")
                this._on(this.element, "click", "td.e-parentnodecategory", $.proxy(this._onResourceClick, this));
            this._on(this.element, "swipeleft swiperight", ".e-contentarearow .e-workcellstab,.e-agendacellstab", $.proxy(this._touchSchedule, this));
            if (this.model.enableResize)
                !ej.isTouchDevice() ? this._on($(window), "resize", this._scheduleResize) : this._on($(window), "orientationchange", this._scheduleResize);
            this._on(this.element, "click", ".e-schedulemouseclose,.e-deleteapp", $.proxy(this._deleteAppoint, this));
            this._on(this.element, "click", ".e-editapp", $.proxy(this._appointmentWindow, this));
        },

        _drawerOpen: function () {
            this._scheduleDrawer.ejNavigationDrawer("open");            
        },

        _mouseButtonClick: function (e) {
            if (!(e.altKey) && !(e.shiftKey) && !(e.ctrlKey) && (e.which != 3)) {
                if ((e.which == 1 || e.type == "touchstart") && !$(e.target).hasClass("e-resourceheadercells")) {
                    var oEl = e.target || document.activeElement || e.srcElement;
                    var oRowCell = this.element.find('div.e-draggableworkarea tr'), proxy = this;
                    ($(oEl).hasClass("e-alldaycells")) && (oRowCell = this.element.find(".e-alldaycells").parent());
                    if ($(oEl).hasClass("e-selectedAppointment")) {
                        this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                        $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    this._currentRowIndex = $(oEl).parent()[0].rowIndex;
                    this._currentColIndex = $(oEl)[0].cellIndex;
                    var renderDate = (proxy.model.orientation == "horizontal" && proxy.currentView() == "month") ? proxy.monthDays : proxy.model.orientation == "vertical" ? proxy.dateRender : proxy._dateRender;
                    var cellindexs = $(e.target).hasClass("e-workcells") || $(e.target).hasClass("e-alldaycells") ? $(e.target).index() : 7 - ((parseInt($(e.target).index() / 7) + 1) * 7 - $(e.target).index()) + ($(e.target).parent().index() * 7);
                    ((this.model.orientation == "horizontal") && (cellindexs = proxy.currentView() !== "month" ? Math.floor(cellindexs / ((this.model.endHour - this.model.startHour) * 2)) : cellindexs));
                    var curDates = new Date(renderDate[cellindexs]);
                    if ((curDates >= this.model.minDate) && (curDates <= this.model.maxDate)) {
                        if ($(oEl).hasClass('e-workcells') || $(oEl).hasClass('e-monthcells') || $(oEl).hasClass('e-alldaycells')) {
                            oRowCell.bind(ej.eventType.mouseMove, function (e) {
                                proxy.model.showQuickWindow && (proxy._quickAppointWindow.ejDialog("close") && proxy._quickAppDetailsWindow.ejDialog("close"));
                                e.preventDefault();
                                var currTarget = e.target;
                                if (e.type == 'touchmove') {
                                    var coor = e.originalEvent.changedTouches[0];
                                    currTarget = document.elementFromPoint(coor.pageX, coor.pageY);
                                }
                                $(currTarget).hasClass('e-selectedCell') && proxy.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
                                proxy._mouseSelection(e, proxy, oEl, currTarget);
                                proxy._moveScrollContent(oEl, proxy);
                            });
                            oRowCell.bind(ej.eventType.mouseUp, function (e) {
                                oRowCell.unbind(ej.eventType.mouseMove);
                            });
                            this.element.find("e.workcells,e.alldaycells,.e-monthcells").removeClass("e-mousehover");
                            this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
                            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                        }
                    }
                }
            }
        },
        _mouseSelection: function (e, proxy, oE1, currTarget) {
            var oRowCell = proxy.element.find('div.e-draggableworkarea tr');
            var sourceRowIndex = $(oE1).parent()[0].rowIndex;
            var sourceColumnIndex = $(oE1)[0].cellIndex;
            var rowcount = oRowCell.length - 1;
            var i, j, k;
            var currentRowIndex = $(currTarget).parent()[0].rowIndex;
            var currentColumnIndex = $(currTarget)[0].cellIndex;
            var ad = proxy.element.find(".e-alldaycells").parent();
            var alldaytr = proxy.model.showAllDayRow;
            var viewCount = proxy.model.orientation == "horizontal" ? oRowCell[0].cells.length : ((proxy.currentView() == "month" || proxy._isCustomView()) ? (oRowCell[0].cells.length / proxy.res1.length) : proxy._dateRender.length);
            var resIndex = Math.floor($(currTarget).index() / viewCount);
            var resmaxcolindex = (viewCount * (resIndex + 1));
            var resmincolindex = (viewCount * resIndex);
            if ($(currTarget).hasClass('e-workcells')) {
                if (proxy.model.orientation == "vertical") {
                    if (sourceColumnIndex >= parseInt(resmincolindex) && sourceColumnIndex <= parseInt(resmaxcolindex) - 1 && currentColumnIndex != "undefined") {
                        if (sourceColumnIndex == currentColumnIndex) {
                            if (currentRowIndex > sourceRowIndex) {
                                for (k = parseInt(sourceRowIndex) ; k <= currentRowIndex; k++)
                                    $(oRowCell[k].cells[sourceColumnIndex]).addClass("e-selectedCell");
                            }
                            else if (currentRowIndex < sourceRowIndex) {
                                for (k = parseInt(sourceRowIndex) ; k >= currentRowIndex; k--)
                                    $(oRowCell[k].cells[sourceColumnIndex]).addClass("e-selectedCell");
                            }
                            else
                                $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell");
                        }
                        else if (currentColumnIndex > sourceColumnIndex) {
                            var difference = currentColumnIndex - sourceColumnIndex;
                            for (i = parseInt(sourceRowIndex) ; i <= rowcount; i++)
                                $(oRowCell[i].cells[sourceColumnIndex]).addClass("e-selectedCell");
                            alldaytr && $(ad[0].cells[[sourceColumnIndex]]).addClass("e-selectedCell");
                            alldaytr && $(ad[0].cells[[currentColumnIndex]]).addClass("e-selectedCell");
                            if (difference > 1) {
                                for (k = 1; k <= difference - 1; k++) {
                                    for (var m = 0; m <= rowcount; m++)
                                        $(oRowCell[m].cells[currentColumnIndex - k]).addClass("e-selectedCell");
                                    alldaytr && $(ad[0].cells[[currentColumnIndex - k]]).addClass("e-selectedCell");
                                }
                            }
                            for (j = 0; j <= currentRowIndex; j++)
                                $(oRowCell[j].cells[currentColumnIndex]).addClass("e-selectedCell");
                        }
                        else if (currentColumnIndex < sourceColumnIndex) {
                            var difference = sourceColumnIndex - currentColumnIndex;
                            for (i = parseInt(sourceRowIndex) ; i >= 0 ; i--)
                                $(oRowCell[i].cells[sourceColumnIndex]).addClass("e-selectedCell");
                            alldaytr && $(ad[0].cells[[sourceColumnIndex]]).addClass("e-selectedCell");
                            alldaytr && $(ad[0].cells[[currentColumnIndex]]).addClass("e-selectedCell");
                            if (difference > 1) {
                                for (k = 1; k <= difference - 1; k++) {
                                    for (var m = 0; m <= rowcount; m++)
                                        $(oRowCell[m].cells[[currentColumnIndex + k]]).addClass("e-selectedCell");
                                    alldaytr && $(ad[0].cells[[currentColumnIndex + k]]).addClass("e-selectedCell");
                                }
                            }
                            for (j = parseInt(currentRowIndex) ; j <= rowcount; j++)
                                $(oRowCell[j].cells[currentColumnIndex]).addClass("e-selectedCell");
                        }
                        proxy._currentRowIndex = currentRowIndex;
                        proxy._currentColIndex = currentColumnIndex;
                        $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else { return false; }
                }
                else {
                    if (sourceRowIndex == currentRowIndex && sourceColumnIndex >= parseInt(resmincolindex) && sourceColumnIndex <= parseInt(resmaxcolindex) - 1 && currentColumnIndex != "undefined") {
                        if (currentColumnIndex > sourceColumnIndex) {
                            for (j = sourceColumnIndex ; j <= currentColumnIndex; j++)
                                $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                        else if (currentColumnIndex <= sourceColumnIndex) {
                            for (j = sourceColumnIndex ; j >= currentColumnIndex; j--)
                                $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                    else { return false; }
                }
            }
            else if ($(currTarget).hasClass('e-alldaycells')) {
                if (sourceColumnIndex >= parseInt(resmincolindex) && sourceColumnIndex <= parseInt(resmaxcolindex) - 1 && currentColumnIndex != "undefined") {
                    if (currentColumnIndex < sourceColumnIndex) {
                        for (k = sourceColumnIndex; k >= currentColumnIndex; k--)
                            $(ad[0].cells[k]).addClass("e-selectedCell");
                    }
                    else if (currentColumnIndex > sourceColumnIndex) {
                        for (k = sourceColumnIndex; k <= currentColumnIndex; k++)
                            $(ad[0].cells[k]).addClass("e-selectedCell");
                    }
                    proxy._currentRowIndex = 0;
                    proxy._currentColIndex = currentColumnIndex;
                    $(ad[0].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else { return false; }
            }
            else if ($(currTarget).hasClass('e-monthcells')) {
                if (sourceColumnIndex >= parseInt(resmincolindex) && sourceColumnIndex <= parseInt(resmaxcolindex) - 1 && currentColumnIndex != "undefined") {
                    if (currentRowIndex == sourceRowIndex) {
                        if (currentColumnIndex < sourceColumnIndex) {
                            for (k = sourceColumnIndex; k >= currentColumnIndex; k--)
                                $(oRowCell[currentRowIndex].cells[k]).addClass("e-selectedCell");
                        }
                        else if (currentColumnIndex > sourceColumnIndex) {
                            for (k = sourceColumnIndex; k <= currentColumnIndex; k++)
                                $(oRowCell[currentRowIndex].cells[k]).addClass("e-selectedCell");
                        }
                        else if (currentColumnIndex == sourceColumnIndex)
                            $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell");
                    }
                    else if (currentRowIndex > sourceRowIndex) {
                        var difference = (currentRowIndex - sourceRowIndex);
                        for (i = sourceColumnIndex; i < resmaxcolindex; i++)
                            $(oRowCell[sourceRowIndex].cells[i]).addClass("e-selectedCell");
                        if (difference > 1) {
                            for (k = 1; k <= difference - 1; k++) {
                                for (i = resmincolindex ; i < resmaxcolindex; i++)
                                    $(oRowCell[sourceRowIndex + k].cells[i]).addClass("e-selectedCell");
                            }
                        }
                        for (j = resmincolindex; j <= currentColumnIndex; j++)
                            $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell");
                    }
                    else if (currentRowIndex < sourceRowIndex) {
                        var difference = (sourceRowIndex - currentRowIndex);
                        for (i = sourceColumnIndex; i >= resmincolindex; i--)
                            $(oRowCell[sourceRowIndex].cells[i]).addClass("e-selectedCell");
                        if (difference > 1) {
                            for (k = 1; k <= difference - 1; k++) {
                                for (i = resmincolindex; i < resmaxcolindex; i++)
                                    $(oRowCell[sourceRowIndex - k].cells[i]).addClass("e-selectedCell");
                            }
                        }
                        for (j = resmaxcolindex - 1; j >= currentColumnIndex; j--)
                            $(oRowCell[currentRowIndex].cells[j]).addClass("e-selectedCell");
                    }
                    proxy._currentRowIndex = currentRowIndex;
                    proxy._currentColIndex = currentColumnIndex;
                    $(oRowCell[currentRowIndex].cells[currentColumnIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else { return false; }
            }
        },
        _shiftSelection: function (e) {
            if (e.shiftKey == true && e.which == 1 && !$(e.target).hasClass("e-resourceheadercells")) {
                var oE1 = this._focused, proxy = this, currTarget = e.target;
                this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
                this.element.find("e.workcells,e.alldaycells,.e-monthcells").removeClass("e-mousehover");
                this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                var renderDate = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : this.model.orientation == "vertical" ? this.dateRender : this._dateRender;
                var cellindex = ($(e.target).hasClass("e-workcells") || $(e.target).hasClass("e-alldaycells")) ? $(e.target).index() : 7 - ((parseInt($(e.target).index() / 7) + 1) * 7 - $(e.target).index()) + ($(e.target).parent().index() * 7);
                ((this.model.orientation == "horizontal") && (cellindex = this.currentView() !== "month" ? Math.floor(cellindex / ((this.model.endHour - this.model.startHour) * 2)) : cellindex));
                var curDate = new Date(renderDate[cellindex]);
                if ((curDate >= this.model.minDate) && (curDate <= this.model.maxDate))
                    this._mouseSelection(e, proxy, oE1, currTarget);
            }
        },
        _moveScrollContent: function (oEl, proxy) {
            if (proxy.model.orientation === "horizontal") {
                var leftValue = proxy.element.find(".e-draggableworkarea").scrollLeft();
                if (proxy.model.enableRTL) {
                    if (ej.browserInfo().name == "mozilla")
                        leftValue = leftValue > 0 ? leftValue : (leftValue * -1);
                    else if (ej.browserInfo().name == "chrome" || ej.browserInfo().name == "webkit")
                        leftValue = proxy.element.find(".e-workcells")[0].offsetLeft + proxy.element.find(".e-workcells").width() - proxy.element.find(".e-draggableworkarea").scrollLeft() - proxy.element.find(".e-draggableworkarea").width();
                    else
                        leftValue = Math.abs(leftValue);
                }
                else
                    leftValue = (-leftValue);
                proxy.currentView() != "month" ? $(proxy.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", leftValue + "px") : proxy.element.find(".e-headerdays").css("left", leftValue + "px");
                if (proxy.currentView() !== "day" && proxy.currentView() !== "month" && !(proxy.currentView() == "customview" && proxy._dateRender.length > 7) && proxy.model.showTimeScale) {
                    var currentDateIndex = $(proxy.element.find(".e-headercells").find(".e-activeview").parent()).index();
                    var index = Math.floor((Math.abs(leftValue) / (proxy.element.find(".e-workcells").width() + 1)) / ((proxy.model.endHour - proxy.model.startHour) * 2));
                    if (currentDateIndex !== index)
                        proxy.element.find(".e-headercells").children().removeClass("e-activeview");
                    $(proxy.element.find(".e-headercells")[index]).children().addClass("e-activeview");
                }
            }
            else
                if (!$(oEl).hasClass("e-monthcells"))
                    proxy.element.find(".e-scrolltimecells").css("top", (-proxy.element.find('.e-draggableworkarea').scrollTop()) + "px");
        },

        _onScheduleMouseDown: function (e) {
            var _targ = $(e.target);
            var appointdiv = _targ.hasClass("e-appointment,e-alldayappointment,e-monthappointment") ? _targ : _targ.parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
            if (!_targ.hasClass("e-workcells") && !appointdiv.hasClass("e-appointment") && !appointdiv.hasClass("e-alldayappointment") && !appointdiv.hasClass("e-monthappointment") && !_targ.hasClass("e-alldaycells") && !_targ.hasClass("e-monthcells") && !(_targ).parents(".e-scheduledialog").hasClass("e-scheduledialog")) {
                this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
                this.element.find(".e-workcells").removeClass("e-mousehover");
            }
        },

        _onCellMouseOver: function (e) {
            if (this.currentView() == "agenda") {
                if (this._mediaQuery == true) return false;
                var element = $(e.currentTarget);
                if (e.type === "mouseover") {
                    this.element.find(".e-mousehover").removeClass("e-mousehover");
                    if (element.hasClass("e-agendacells")) {
                        element.parent().find(".e-timecolumn,.e-eventcolumn").addClass("e-mousehover");
                        element.parent().find(".e-editapp").parent().show();
                    }
                }
                else if (e.type === "mouseout") {
                    if (element.hasClass("e-agendacells")) {
                        element.parent().find(".e-timecolumn,.e-eventcolumn").removeClass("e-mousehover");
                        element.parent().find(".e-editapp").parent().hide();
                    }
                }
            }
            else {
                var element = $(e.currentTarget);
                var index = $(e.target).hasClass("e-workcells") ? $(e.target).index() : $(e.target).hasClass("e-alldaycells") ? $(e.target).index() : 7 - ((parseInt($(e.target).index() / 7) + 1) * 7 - $(e.target).index()) + ($(e.target).parent().index() * 7);
                if (this.model.orientation == "horizontal") {
                    index = this.currentView() !== "month" ? Math.floor(index / ((this.model.endHour - this.model.startHour) * 2)) : index;
                }
                var renderDate = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : this.model.orientation == "vertical" ? this.dateRender : this._dateRender;
                var curDate = new Date(renderDate[index]);
                var event = { currentDate: curDate, cellIndex: index, target: e, resources: (!ej.isNullOrUndefined(this.model.cellHover) && !ej.isNullOrUndefined(this.model.resources)) ? this._getResourceValue(element, index) : null };
                if (this._trigger("cellHover", event)) return false;
                if ((curDate >= this.model.minDate) && (curDate <= this.model.maxDate)) {
                    if (e.type === "mouseover") {
                        this.element.find(".e-mousehover").removeClass("e-mousehover");
                        if ((element.hasClass("e-workcells") || element.hasClass("e-monthcells")) && !element.hasClass("e-resourceheadercells"))
                            element.addClass("e-mousehover");
                    }
                    else if (e.type === "mouseout") {
                        if (element.hasClass("e-workcells") || element.hasClass("e-monthcells"))
                            element.removeClass("e-mousehover");
                    }
                } else
                    return false;
            }
        },

        _getAppDetails: function (e) {
            var _target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
            var appoint = _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") || _target.hasClass("e-agendaappointment") ? _target : _target.parents(".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment");
            appId = appoint.attr('guid');
            var pId = this._getAppointmentByParentId(appId);
            var query = new ej.Query().where("Guid", ej.FilterOperators.equal, appId);
            var appointDetails = ej.DataManager(this._processed).executeLocal(query);
            if (appointDetails[0][this.model.appointmentSettings["recurrence"]] == 1) {
                var parentQuery = new ej.Query().where("Guid", ej.FilterOperators.equal, appId);
                var templist = new ej.DataManager(this._processed).executeLocal(parentQuery);
            } else {
                var parentQuery = new ej.Query().where("ParentId", ej.FilterOperators.equal, pId.ParentId);
                var templist = new ej.DataManager(this._processed).executeLocal(parentQuery);
            }
            return templist[0];
        },

        _onAppointMouseOver: function (e) {
            if (!this.model.readOnly) {
                if (!ej.isNullOrUndefined(this.model.appointmentHover)) {
                    var data = { appointment: this._getAppDetails(e) };
                    if (this._trigger("appointmentHover", data))
                        return false;
                }
                if (this.currentView() == "agenda") {
                    if (this._mediaQuery == true) return false;
                    var element = $(e.currentTarget);
                    if (e.type === "mouseover") {
                        this.element.find(".e-mousehover").removeClass("e-mousehover");
                        if (element.hasClass("e-agendaappointment")) {
                            element.addClass("e-mousehover");
                            element.find(".e-editapp").parent().show();
                        }
                    }
                    else if (e.type === "mouseout") {
                        if (element.hasClass("e-agendaappointment")) {
                            element.removeClass("e-mousehover");
                            element.find(".e-editapp").parent().hide();
                        }
                    }
                }
                else {
                    var appointdiv = $(e.target).hasClass("e-appointment,e-alldayappointment,e-monthappointment") ? $(e.target) : $(e.target).parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                    if (appointdiv.hasClass("e-appointment") && this.model.orientation == "vertical") {
                        if (e.type === "mouseover") {
                            if (this.model.enableAppointmentResize) {
                                appointdiv.find(".e-bottomhandle,.e-tophandle").addClass("e-icon e-nsexpand");
                                appointdiv.find(".e-bottomhandle").addClass("e-appointresizer-bottom");
                                appointdiv.find(".e-tophandle").addClass("e-appointresizer-top");
                            }
                            appointdiv.find(".e-schedulemouseclose").css("display", "block");
                        }
                        else if (e.type === "mouseout") {
                            if (this.model.enableAppointmentResize) {
                                appointdiv.find(".e-bottomhandle,.e-tophandle").removeClass("e-icon e-nsexpand");
                            }
                            appointdiv.find(".e-schedulemouseclose").css("display", "none");
                        }

                    }
                    else if (((appointdiv.hasClass("e-alldayappointment") || appointdiv.hasClass("e-monthappointment")) && this.model.orientation == "vertical") || (this.model.orientation == "horizontal")) {
                        if (e.type === "mouseover") {
                            if (this.model.enableAppointmentResize) {
                                appointdiv.find(".e-lefthandle,.e-righthandle").addClass("e-icon e-ewexpand");
                                appointdiv.find(".e-lefthandle").addClass("e-appointresizer-left");
                                appointdiv.find(".e-righthandle").addClass("e-appointresizer-right");
                            }
                            appointdiv.find(".e-schedulemouseclose").css("display", "block");
                        }
                        else if (e.type === "mouseout") {
                            if (this.model.enableAppointmentResize) {
                                appointdiv.find(".e-lefthandle,.e-righthandle").removeClass("e-icon e-ewexpand");
                            }
                            appointdiv.find(".e-schedulemouseclose").css("display", "none");
                        }
                    }
                }
            }
        },

        _cellSelection: function (e) {
            if (e.shiftKey)
                return false;
            this.element.find("e.workcells,e.alldaycells,.e-monthcells,.e-agendacells").removeClass("e-mousehover");
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
            var localizeDays = Globalize.culture(this.model.locale);
            var _target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
            _target.hasClass("e-appointwrapper") || _target.hasClass("e-textwrapper") && (_target = $(_target).parent());
            if (_target.hasClass("e-lefthandle") || _target.hasClass("e-righthandle") || _target.hasClass("e-tophandle") || _target.hasClass("e-bottomhandle"))
                return false;
            if (!ej.isNullOrUndefined(e.type) && e.type == "beforeOpen") {
                var selectedCells = this.element.find("td.e-selectedCell");
                if ((selectedCells.length > 0) && !(_target.hasClass("e-selectedCell")))
                    this.element.find(".e-workcells,.e-alldaycells,.e-monthcells,.e-agendacells").removeClass("e-selectedCell");
            }
            else
                this.element.find(".e-workcells,.e-alldaycells,.e-monthcells,.e-agendacells").removeClass("e-selectedCell");
            if (this.currentView() != "agenda") {
                if ((_target.hasClass("e-workcells") || _target.hasClass("e-alldaycells") || _target.hasClass("e-monthcells")) && !_target.hasClass("e-resourceheadercells")) {
                    this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
                    if (this.model.showTimeScale)
                        this._cellIndex = _target.hasClass("e-workcells") ? _target.index() : _target.hasClass("e-alldaycells") ? _target.index() : this._isCustomView() && this._oneWeek ? _target.index() : 7 - ((parseInt(_target.index() / 7) + 1) * 7 - _target.index()) + (_target.parent().index() * 7);
                    else
                        this._cellIndex = this.model.orientation == "vertical" && this.currentView() == "month" || (this.currentView() == "customview" && this._dateRender.length >= 7) ? 7 - ((parseInt(_target.index() / 7) + 1) * 7 - _target.index()) + (_target.parent().index() * 7) : _target.index();
                    if (this.model.orientation == "horizontal") {
                        this._cellIndex = this.model.showTimeScale ? this.currentView() !== "month" && !(this._isCustomView()) ? Math.floor(this._cellIndex / ((this.model.endHour - this.model.startHour) * 2)) : this._cellIndex : _target.index();
                        this._workCellIndex = _target.parent().index();
                    }
                    var renderDate = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : (this.model.orientation == "vertical" || this.currentView() == "customview" && this._dateRender.length <= 7) ? this.dateRender : this._dateRender;
                    renderDate = this.model.orientation == "horizontal" && this.currentView() == "customview" && this._dateRender.length <= 7 ? this._dateRender : renderDate;
                    var curDate = new Date(renderDate[this._cellIndex]);
                    if ((this.model.minDate > curDate || this.model.maxDate < curDate)) { return false; }
                    if (_target.hasClass("e-workcells") && (this.model.showTimeScale) && this.currentView() !== "month" && !(this._isCustomView())) {
                        var time = this.model.orientation == "vertical" ? this.model.startHour + (_target.parent().index() / 2) : this.model.startHour + ((_target.index() - (((this.model.endHour - this.model.startHour) * 2) * this._cellIndex)) / 2);
                        var timemin = time.toString().split(".");
                        this.cur_StartTime = new Date(curDate).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                        var min = (parseInt(new Date(this.cur_StartTime).getHours()) == 23 && parseInt(new Date(this.cur_StartTime).getMinutes()) == 30) ? new Date(this.cur_StartTime).getMinutes() + 29 : new Date(this.cur_StartTime).getMinutes() + 30;
                        this.cur_EndTime = new Date(new Date(this.cur_StartTime).setMinutes(min));
                        this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: false });
                    }
                    else if (_target.hasClass("e-workcells") && this.model.orientation == "horizontal" && this.currentView() == "month") {
                        this.cur_StartTime = new Date(new Date(curDate).setHours(0, 0));
                        this.cur_EndTime = new Date(new Date(curDate).setHours(23, 59));
                        this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
                    }
                    else {
                        this.cur_StartTime = new Date(new Date(curDate).setHours(0, 0));
                        this.cur_EndTime = new Date(new Date(curDate).setHours(23, 59));
                        this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
                    }
                    var event = { startTime: new Date(this.cur_StartTime), endTime: this.cur_EndTime, target: e, resources: (this.model.cellClick != null && !ej.isNullOrUndefined(this.model.resources)) ? this._getResourceValue(_target) : null };
                    if (this._trigger("cellClick", event)) return false;
                    if (this.model.showQuickWindow) {
                        this._quickAppointWindow.find("#" + this._id + "_quickAppwindow").css("display", "block");
                        this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(curDate).getDay()] + ", " + Globalize.format(new Date(curDate), "MMMM") + " " + new Date(curDate).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), this._timeMode == "12" ? this._pattern.t : "HH:mm") + " - " + Globalize.format(new Date(this.cur_EndTime), this._timeMode == "12" ? this._pattern.t : "HH:mm"));
                        var topElmt = $(e).hasClass("e-workcells") || $(e).hasClass("e-alldaycells") || $(e).hasClass("e-monthcells") ? $(e) : $(e.target);
                        (e.type != "beforeOpen" && !this.model.readOnly) ? this._quickAppointWindow.ejDialog("open") : "";
                        this._quickAppointWindow.find('.subject').val("");
                        var x = topElmt.offset().left - ((this._quickAppointWindow.width() - topElmt.width()) / 2) + 10;
                        var y = topElmt.offset().top - this._quickAppointWindow.height() - 10;
                        this._quickAppointWindow.ejDialog({ position: { X: x, Y: y } });
                        if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                            this._getResourceDetails(_target, this._cellIndex);
                    }
                    _target.addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    this._focused = $(":focus");
                    this.model.showQuickWindow && this._quickAppointWindow.find('.subject').focus();
                    this._cellClick = true;
                    this._appointmentAddWindow.ejDialog({ title: this._getLocalizedLabels("CreateAppointmentTitle") });
                }
                else {
                    if (!_target.hasClass("e-resourceheadercells") && !_target.hasClass("e-appsoverflow") && !_target.hasClass("e-monthheader") && !_target.hasClass("e-categorybar")) {
                        this._cellClick = false;
                        this._quickAppDetailsWindow && this._quickAppDetailsWindow.find("" + "#" + this._id + "_quickAppDetailswindow").css("display", "block");
                        this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close");
                        var appoint = _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") ? _target : _target.parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                        this._appUid = appoint.attr('guid');
                        if (_target.hasClass("e-schedulemouseclose")) return false;
                        this._processed = this._sortAppById(this._processed);
                        var splitObj = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                        for (i = 0; i < splitObj.length; i++) {
                            var templist = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, splitObj[i]["Guid"]));
                            break;
                        }
                        for (i = 0; i < templist.length; i++) {
                            var eleFind = this.element.find('[guid=' + templist[i]["Guid"] + ']');
                            eleFind.addClass("e-selectedAppointment");
                        }
                        if (this._mediaQuery)
                            this._mAppId = this._appUid;
                        var templist1 = this._sortAppById(templist);
                        var temp = templist1[0][this.model.appointmentSettings["recurrence"]] ? templist1[0] : this._getAppointmentByParentId(this._appUid);
                        var diffTime = new Date().getTime() - (temp[this.model.appointmentSettings["startTime"]].getTime());
                        var diffMinute = 60 * 1000;
                        var diffHour = 60 * diffMinute;
                        var diffDays = 24 * diffHour;
                        var result = Math.abs(new Date() - temp[this.model.appointmentSettings["startTime"]]);
                        var h = Math.floor(result / 1000 / 60 / 60); h = (h <= 9 ? '0' + h : h);
                        var m = new Date(result).getUTCMinutes(); m = (m <= 9 ? '0' + m : m);
                        var dueIn = Math.ceil(diffTime / diffMinute - 1) < 0 && h < 24 ? this._getLocalizedLabels("EventBeginsin") + " " + h + " " + this._getLocalizedLabels("Hours") + " " + m + " " + this._getLocalizedLabels("Minutes") : Math.ceil(diffTime / diffMinute - 1) < 0 && h >= 24 ? this._getLocalizedLabels("EventBeginsin") + " " + Math.abs(Math.ceil(diffTime / diffDays)) + " " + this._getLocalizedLabels("Days") : this._getLocalizedLabels("Overdue");
                        var data = { appointment: templist[0] };
                        if (this._trigger("appointmentClick", data)) {
                            $(".e-selectedAppointment").css({ 'border': "none" });
                            return false;
                        }
                        if (this.model.showQuickWindow) {
							this._quickAppDetailsWindow && this._quickAppDetailsWindow.find('.e-quicksubject').html(temp[this.model.appointmentSettings["subject"]]);
                            var startTimeDisp = this._timeMode == "12" ? Globalize.format(temp[this.model.appointmentSettings["startTime"]], this._pattern.t) : Globalize.format(temp[this.model.appointmentSettings["startTime"]], "HH:mm");
							var endTimeDisp = this._timeMode == "12" ? Globalize.format(temp[this.model.appointmentSettings["endTime"]], this._pattern.t) : Globalize.format(temp[this.model.appointmentSettings["endTime"]], "HH:mm");                       
							this._quickAppDetailsWindow.find('.e-rightfields').html(dueIn);
                            if ((templist.length > 1) || (templist[0][this.model.appointmentSettings["startTime"]].getDay() != templist[templist.length - 1][this.model.appointmentSettings["endTime"]].getDay()))
                                var dateShow = localizeDays.calendar.days.names[templist[0][this.model.appointmentSettings["startTime"]].getDay()] + ", " + Globalize.format(templist[0][this.model.appointmentSettings["startTime"]], "MMMM") + " " + templist[0][this.model.appointmentSettings["startTime"]].getDate() + " - " + startTimeDisp + " " + "to" + " " + localizeDays.calendar.days.names[templist[templist.length - 1][this.model.appointmentSettings["endTime"]].getDay()] + ", " + Globalize.format(templist[templist.length - 1][this.model.appointmentSettings["endTime"]], "MMMM") + " " + templist[templist.length - 1][this.model.appointmentSettings["endTime"]].getDate() + " - " + endTimeDisp;
							else
                                var dateShow = localizeDays.calendar.days.names[templist[0][this.model.appointmentSettings["startTime"]].getDay()] + ", " + Globalize.format(templist[0][this.model.appointmentSettings["startTime"]], "MMMM") + " " + templist[0][this.model.appointmentSettings["startTime"]].getDate() + ", " + startTimeDisp + " " + "-" + " " + endTimeDisp;
                            this._quickAppDetailsWindow.find('.e-quickstartendtime').html(dateShow);
                            if (temp[this.model.appointmentSettings["recurrence"]] == 1)
                                this._quickAppDetailsWindow.find('.e-editseries').removeClass("e-disable");
                            else
                                this._quickAppDetailsWindow.find('.e-editseries').addClass("e-disable");
                            (e.type != "beforeOpen") ? this._quickAppDetailsWindow.ejDialog("open") : "";
                            (this.model.readOnly) ? this._quickAppDetailsWindow.find(".e-scheduledelete").addClass("e-disable") : this._quickAppDetailsWindow.find(".e-scheduledelete").removeClass("e-disable");
                            if ($(appoint).width() > this.element.width())
                                var left = ($(appoint).offset().left > this.element.offset().left) ? $(appoint).offset().left : (this.element.find('.e-draggableworkarea').scrollLeft() > $(appoint).width()) ? this.element.offset().left : this.element.width() / 2 - 4;
                            else
                                var left = $(appoint).offset().left - ((this._quickAppDetailsWindow.width() - $(appoint).width()) / 2 - 4);
                            var top = $(appoint).offset().top - this._quickAppDetailsWindow.height() - 10;
                            this._quickAppDetailsWindow.ejDialog({ position: { X: left, Y: top } });
                            this._quickAppDetailsWindow.focus();
                        }
                        this._appointmentAddWindow.ejDialog({ title: this._getLocalizedLabels("Editevent") });
                    }
                }
                if (!$(_target).hasClass("e-monthcells"))
                    this.element.find(".e-scrolltimecells").css("top", (-this.element.find('.e-draggableworkarea').scrollTop()) + "px");
            } else {
                this._cellIndex = 0;
                this.cur_StartTime = new Date(new Date(this.currentDate()).setHours(0, 0));
                this.cur_EndTime = new Date(new Date(this.currentDate()).setHours(23, 59));
                $(_target).parentsUntil('.e-agendacellstab tbody').find(".e-eventcolumn,.e-timecolumn").addClass("e-selectedCell").attr("tabIndex", 0).focus();
                var appoint = _target.hasClass("e-agendaappointment") ? _target : this.element.find(".e-selectedCell").parent().find(".e-agendaappointment");
                this._appUid = appoint.attr('guid');
                if (this._mediaQuery)
                    this._mAppId = this._appUid;
                if (_target.hasClass("e-editapp") || _target.hasClass("e-deleteapp")) return false;
                var appoint = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                if (this._trigger("appointmentClick", { appointment: appoint[0] })) {
                    return false;
                }
                this._cellClick = false;
            }
        },

        _checkReminders: function (args) {
            this._todayAppointments = [];
            for (var i = 0; i < this._processed.length; i++) {
                if ((new Date(new Date(this._processed[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date().setHours(0, 0, 0, 0)).getTime())) {
                    this._todayAppointments.push(this._processed[i]);
                }
            }
            var proxy = this;
            jQuery.each(this._todayAppointments, function (val, i) {
                var reminderValue = proxy.model.reminderSettings.alertBefore;
                var startTime = i[proxy.model.appointmentSettings["startTime"]];
                if (reminderValue != -1 && reminderValue != null) {
                    var currTime = new Date().getTime();
                    var difference = (currTime - new Date(startTime).getTime()) / (1000 * 60);
                    if (Math.round(difference) == (reminderValue * -1))
                        proxy._trigger("reminder", { reminderAppointment: i });
                }
            });
        },

        _showContextMenu: function (flag) {
            var gotoFlag = false, categorizeFlag = false, cellCollection = this.model.contextMenuSettings.menuItems.cells;
            if (flag) {
                this._contextMenu = ej.buildTag("ul#" + this._id + "contextMenu");
                this.element.append(this._contextMenu);
                this._contextMenu.ejMenu({
                    fields: { id: "id", text: "text", parentId: "parentId" },
                    menuType: ej.MenuType.ContextMenu,
                    openOnClick: false,
                    contextMenuTarget: "#" + this._id,
                    width: "auto",
                    click: $.proxy(this._onMenuSelect, this),
                    beforeOpen: $.proxy(this._beforeopen, this)
                });
                if (this.model.categorizeSettings.enable && this.model.contextMenuSettings.menuItems.appointment) {
                    var menuItems = this.model.contextMenuSettings.menuItems.appointment;
                    var dataLength = this.model.categorizeSettings.dataSource.length;
                    for (var i = 0; i < menuItems.length; i++) {
                        if (menuItems[i].id == 'categorize') {
                            var menuId = i;
                            categorizeFlag = true;
                            break;
                        }
                    }
                    if (categorizeFlag)
                        for (var i = 0; i < dataLength; i++) {
                            var dataSet = this.model.categorizeSettings;
                            this.model.contextMenuSettings.menuItems.appointment.push({ text: dataSet.dataSource[i][dataSet.text], id: "categorize_" + dataSet.dataSource[i][dataSet.id], fontColor: dataSet.dataSource[i][dataSet.fontColor], color: dataSet.dataSource[i][dataSet.color], parentId: this.model.contextMenuSettings.menuItems.appointment[menuId].id.toString() });
                        }
                }
                if (this._gotoDate) {
                    if (this.model.contextMenuSettings.enable) {
                        for (var id in cellCollection) {
                            if (cellCollection[id].id == "gotodate") {
                                gotoFlag = true;
                                break;
                            }
                        }
                        if (gotoFlag) {
                            this._renderGotoWindow();
                            this._gotoDate = false;
                        }
                    }
                }
            }
        },

        _beforeopen: function (args) {
            if (this.model.contextMenuSettings.enable) {
                $(document).unbind(ej.eventType.mouseMove);
                var targetClass = $(args.target), appFlag, appointment = null, event;
                var tempargs = ($(targetClass).parents(".e-draggableworkarea tr").length > 0 && $(targetClass).parentsUntil(".e-draggableworkarea tr").is(".e-agendaappointment,.e-eventcolumn,.e-timecolumn")) || $(targetClass).is(".e-agendaappointment,.e-eventcolumn,.e-timecolumn");
                if (!targetClass.hasClass("e-appointment") && !targetClass.hasClass("e-workcells") && !targetClass.hasClass("e-alldaycells") && !targetClass.hasClass("e-monthcells") && !ej.isNullOrUndefined(this.model.appointmentTemplateId)) {
                    targetClass = $(args.target).offsetParent();
                }
                args.cancel = (targetClass.hasClass("e-alldayappointment") || targetClass.hasClass("e-monthappointment") || targetClass.hasClass("e-appointment") || targetClass.hasClass("e-workcells") || targetClass.hasClass("e-alldaycells") || targetClass.hasClass("e-appointinnertext") || targetClass.hasClass("e-monthcells") || targetClass.hasClass("e-apptime") || targetClass.hasClass("e-apptext") || tempargs) ? false : true;
                if (args.cancel == true) {
                    var obj = this._contextMenu.data("ejMenu");
                    obj.hide();
                    return false;
                }
                if (this.currentView() == "agenda" && tempargs) {
                    args = $(targetClass).hasClass("e-agendaappointment") ? args : $(targetClass).parents(".e-draggableworkarea tr").find(".e-agendaappointment");
                }
                this._cellSelection(args);
                var renderDate = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : (this.model.orientation == "vertical" || this.currentView() == "customview" && this._dateRender.length <= 7) ? this.dateRender : this._dateRender;
                renderDate = this.model.orientation == "horizontal" && this.currentView() == "customview" && this._dateRender.length <= 7 ? this._dateRender : renderDate;
                var curDate = new Date(renderDate[this._cellIndex]);
                if (targetClass.hasClass("e-workcells") || targetClass.hasClass("e-alldaycells") || targetClass.hasClass("e-monthcells")) {
                    event = { currentDate: curDate, cellIndex: this._cellIndex, events: args, resources: (!ej.isNullOrUndefined(this.model.beforeContextMenuOpen) && !ej.isNullOrUndefined(this.model.resources)) ? this._getResourceValue(targetClass, this._cellIndex) : null };
                }
                if (targetClass.hasClass("e-alldayappointment") || targetClass.hasClass("e-monthappointment") || targetClass.hasClass("e-appointinnertext") || targetClass.hasClass("e-apptime") || targetClass.hasClass("e-apptext") || targetClass.hasClass("e-appointment")) {
                    appointment = this._getAppDetails(args);
                    event = { appointment: appointment, events: args };
                }
                if (this.currentView() == "agenda") {
                    appointment = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid))[0];
                    event = { appointment: appointment, events: args };
                }
                this._baseTarget = targetClass;
                if (this._trigger("beforeContextMenuOpen", event)) return false;
                var menuObj = this._contextMenu.data("ejMenu");
                appFlag = (targetClass.hasClass("e-alldayappointment") || targetClass.hasClass("e-monthappointment") || targetClass.hasClass("e-appointment") || targetClass.hasClass("e-appointinnertext") || targetClass.hasClass("e-apptime") || targetClass.hasClass("e-apptext") || tempargs) ? false : true;
                (appFlag) ? menuObj.option("fields.dataSource", this.model.contextMenuSettings.menuItems.cells) : menuObj.option("fields.dataSource", this.model.contextMenuSettings.menuItems.appointment);
                if (this.model.readOnly) {
                    menuObj.disableItemByID("new");
                    menuObj.disableItemByID("recurrence");
                    menuObj.disableItemByID("delete");
                    menuObj.disableItemByID("categorize");
                }
                else {
                    menuObj.enableItemByID("new");
                    menuObj.enableItemByID("recurrence");
                    menuObj.enableItemByID("delete");
                    menuObj.enableItemByID("categorize");
                }
                var tds = this.element.find('td.e-selectedCell');
                if (tds.length > 1 && this.currentView() != "agenda")
                    this._multipleCellAppointCreation(args);
                this._target = args;
                if (this.model.showQuickWindow) {
                    this._quickAppointWindow.ejDialog("close");
                    this._quickAppDetailsWindow.ejDialog("close");
                }
                if (this.model.categorizeSettings.enable && appFlag == false) {
                    var findDiv = this._contextMenu.find('li#categorize').find('li').children();
                    if (findDiv.find('div').length == 0) {
                        for (var i = 0; i < this.model.categorizeSettings.dataSource.length; i++) {
                            this._contextMenu.find('li#categorize').find('li#categorize_' + this.model.categorizeSettings.dataSource[i][this.model.categorizeSettings["id"]]).children().append('<div><div style="border: 1px solid black;height: 12px;margin-left: 2px;margin-top: 2px;width: 12px;background-color:' + this.model.categorizeSettings.dataSource[i][this.model.categorizeSettings["color"]] + '"></div></div>');
                        }
                    }
                    for (var i = 1; i <= this.model.categorizeSettings.dataSource.length; i++) {
                        if (this._contextMenu.find('li#categorize').find('li#categorize_' + i).children().find('div:first-child.e-categorizeMenuSelected'))
                            this._contextMenu.find('li#categorize').find('li#categorize_' + i).children().find('div:first-child').removeClass('e-categorizeMenuSelected');
                    }
                    var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                    var addItem = ej.DataManager(this._processed).executeLocal(query);
                    if (addItem[0][this.model.appointmentSettings["categorize"]]) {
                        var categorizeItems = addItem[0][this.model.appointmentSettings["categorize"]].split(',');
                        if (this.model.categorizeSettings.allowMultiple) {
                            if (categorizeItems[0].length > 0) {
                                for (var i = 0; i < categorizeItems.length; i++) {
                                    this._contextMenu.find('li#categorize').find('li#categorize_' + categorizeItems[i]).children().find('div:first-child').addClass('e-categorizeMenuSelected');
                                }
                            }
                        }
                        else {
                            if (categorizeItems[0].length > 0) {
                                var lastValue = categorizeItems.length - 1;
                                this._contextMenu.find('li#categorize').find('li#categorize_' + categorizeItems[lastValue]).children().find('div:first-child').addClass('e-categorizeMenuSelected');
                            }
                        }
                    }
                }
                else {
                    if (this._contextMenu.find('li#categorize'))
                        this._contextMenu.find('li#categorize').css("display", "none");
                }

            }
            else {
                var obj = this._contextMenu.data("ejMenu");
                obj.hide();
                return false;
            }
        },

        _onMenuSelect: function (args) {
            var id = args.events.ID.split("_"), parentData = "";
            var target = this._baseTarget;
            if ($(target).is(".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment") || target.parents(".e-appointment").hasClass("e-appointment") || target.parents(".e-monthappointment").hasClass("e-monthappointment") || target.parents(".e-alldayappointment").hasClass("e-alldayappointment") || this.currentView() == "agenda" && this.element.find(".e-selectedCell").parent().find(".e-agendaappointment")) {
                var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                var appointDetails = ej.DataManager(this._processed).executeLocal(query);
                parentData = (appointDetails[0][this.model.appointmentSettings["recurrence"]] == 1 && appointDetails[0][this.model.appointmentSettings["recurrenceRule"]].indexOf("RECUREDITID") != -1) ? appointDetails[0] : this._getAppointmentByParentId(this._appUid);
            }
            else {
                parentData = { startTime: new Date(this.cur_StartTime), endTime: this.cur_EndTime, resourceInfo: this._getResourceDetails(target) };
            }
            if (this._trigger("menuItemClick", { events: args, targetInfo: parentData })) return false;
            switch (id[0]) {
                case "new":
                    this._appointmentWindow(this._target);
                    break;
                case "recurrence":
                    this._appointmentWindow(this._target);
                    this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: true });
                    this._repeat(args = { isChecked: true });
                    break;
                case "open":
                    this._appointmentWindow(this._target);
                    break;
                case "today":
                    this._navigateClick(this.currentDate(new Date()), "DateNavigation");
                    break;
                case "view":
                    if (!ej.isNullOrUndefined(id[1])) {
                        var newView = id[1].toLowerCase();
                        this._navView =  true;
                        (this.currentView() != newView) && this._viewChange(null, this.currentView(), newView);
                    }
                    break;
                case "timemode":
                    if (!ej.isNullOrUndefined(id[1])) {
                        this.model.timeMode = this._timeMode = (id[1] == "Hour12") ? "12" : "24";
                        this._appointmentAddWindow.find('.startendtime').ejTimePicker({ timeFormat: (this._timeMode == "12") ? this._pattern.t : "HH:mm" });
                        if (this.model.orientation == "horizontal" && this.currentView() !== "month" && !(this._isCustomView()) && this.currentView() !== "agenda") {
                            var view = this.model.showTimeScale ? (this._isCustomView()) ? "month" : this.currentView() : "month"; var weekcols = Globalize.culture(this.model.locale);
                            var viewColCount = view == "day" ? 1 : view == "week" ? 7 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : 5;
                            this.datesColumn = this._getHeaderAllDayCellsCount();
                            var multiResources = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group)) ? true : false;
                            var html = this.horizontalHeader.render({ cols: this.datesColumn, label: this._getLocalizedLabels("Resources"), multiRes: multiResources, view: view, timeTdCount: this._getStartEndTime(), colspan: view == "month" ? 1 : (this._getStartEndTime().length / viewColCount) * 2, height: view == 'month' ? 62 : 59, width: this.model.width, headercellWidth: view == "day" ? "100%" : "55px" });
                            this.aTR.children().remove();
                            this.aTR.append(html);
                            (this.model.showTimeScale) && this._borderAddRemove();
                            this._moveScroller();
                            (this.model.highlightBusinessHours) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                            this._businessHourScroller();
                        }
                        else {
                            if (this.currentView() !== "month" && !(this._isCustomView()) && this.currentView() !== "agenda") {
                                var viewclass = this.currentView() == "month" ? 1 : 0;
                                var timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });
                                this.$TimeCellTD.children().remove();
                                this.$TimeCellTD.append(timeCellhtml); if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                                this.element.find(".e-scrolltimecells").css("top", -(this.element.find(".e-draggableworkarea").scrollTop()) + "px");
                                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                            }
                        }
                        (!ej.isNullOrUndefined(this.dataSource())) && this._renderAppointmentAll();
                    }
                    break;
                case "businesshours":
                    this.model.workHours.highlight = (this.model.workHours.highlight) ? false : true;
                    (this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells") && this.element.find(".e-monthcells").removeClass("e-monthcellhighlight");
                    break;
                case "gotodate":
                    this._gotoWindow.ejDialog("open");
                    break;
                case "delete":
                    this._deleteAppoint();
                    break;
                case "print":
                    this._printAppointment();
                    break;
                case "categorize":
                    if (ej.isNullOrUndefined(id[1]))
                        this.element.find(".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment").removeClass("e-selectedAppointment");
                    else {
                        if (this.model.categorizeSettings.enable) {
                            var getId = id[1]; var idValue = "";
                            if (parentData[this.model.appointmentSettings["categorize"]]) {
                                var categorizeItems = parentData[this.model.appointmentSettings["categorize"]].split(',');
                                if (!this.model.categorizeSettings.allowMultiple) {
                                    if (categorizeItems[categorizeItems.length - 1] == getId)
                                        idValue = "";
                                    else
                                        idValue = getId.toString();
                                }
                                else {
                                    if (categorizeItems[0].length > 0) {
                                        for (var i = 0; i < categorizeItems.length; i++) {
                                            if (categorizeItems[i] == getId) {
                                                var findId = true;
                                                break;
                                            }
                                        }
                                        if (findId == true) {
                                            for (var i = 0; i < categorizeItems.length; i++) {
                                                if (categorizeItems[categorizeItems.length - 1] != getId) {
                                                    if (categorizeItems[i] != getId) {
                                                        if (categorizeItems.length - 1 != i)
                                                            idValue = idValue + categorizeItems[i] + ",";
                                                        else
                                                            idValue = idValue + categorizeItems[i];
                                                    }
                                                }
                                                else {
                                                    if (categorizeItems[i] != getId) {
                                                        if (categorizeItems.length - 2 != i)
                                                            idValue = idValue + categorizeItems[i] + ",";
                                                        else
                                                            idValue = idValue + categorizeItems[i];
                                                    }
                                                }
                                            }
                                        }
                                        else
                                            idValue = parentData[this.model.appointmentSettings["categorize"]] + "," + getId;
                                    }
                                }
                            }
                            else {
                                idValue = getId.toString();
                            }
                            parentData[this.model.appointmentSettings["categorize"]] = idValue;
                            this.element.find('[guid=' + this._appUid + ']').remove();
                            new ej.DataManager(this._currentAppointmentData).update("AppTaskId", parentData);
                            new ej.DataManager(this._processed).update("AppTaskId", parentData);
                            this._renderSingleApp(parentData, parentData);
                            break;

                        }
                    }
            }
        },

        _gotoOk: function (args) {
            if ($(args.currentTarget).hasClass("e-gotoOk")) {
                var date = this._gotoWindow.find("#" + this._id + "gotodate").ejDatePicker("option", "value");
                var view = this._gotoWindow.find("#" + this._id + "show").ejDropDownList("option", "itemValue");
                data = { currentView: this.currentView(), currentDate: new Date(this.currentDate()), target: args, requestType: "dateNavigate" };
                if (this._trigger("actionBegin", { data: data, requestType: "dateNavigate" }))
                    return false;
                var oldDate = this.currentDate();
                var oldview = this.currentView();
                this.currentDate(new Date(date));
                this.currentView(view.toLowerCase());
                data = { previousDate: oldDate, currentDate: this.currentDate(), previousView: oldview, currentView: this.currentView(), target: args, requestType: "dateNavigate" };
                if (this._trigger("navigation", data)) {
                    this.currentDate(new Date(oldDate));
                    this.currentView(oldview);
                    this._navView = false;
                    return false;
                }
                this._renderCurrentView("ViewChange");
                this._trigger("actionComplete", { data: data, requestType: "dateNavigate" });
            }
            this._gotoCancel();
        },

        _gotoCancel: function () {
            this._gotoWindow.find("#" + this._id + "gotodate").ejDatePicker({ value: this.currentDate() });
            this._gotoWindow.find("#" + this._id + "show").ejDropDownList({ value: this.currentView() });
            this._gotoWindow.ejDialog("close");
        },

        _renderGoto: function () {
            this._gotoWindow.find("#" + this._id + "gotodate").ejDatePicker({ buttonText: this._getLocalizedLabels("Today"), enableRTL: this.model.enableRTL, width: '100%', value: this.currentDate(), locale: this.model.locale, cssClass: this.model.cssClass, minDate: this.model.minDate, maxDate: this.model.maxDate });
            this._gotoWindow.find("#" + this._id + "show").ejDropDownList({ enableRTL: this.model.enableRTL, dataSource: [{ text: this._getLocalizedLabels("Day"), value: "day" }, { text: this._getLocalizedLabels("Week"), value: "week" }, { text: this._getLocalizedLabels("WorkWeek"), value: "workweek" }, { text: this._getLocalizedLabels("Month"), value: "month" }, { text: this._getLocalizedLabels("Agenda"), value: "agenda" }], width: "100%", selectedItemIndex: 0, cssClass: this.model.cssClass });
            this._gotoWindow.find(".e-gotobutton").ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "60px", cssClass: this.model.cssClass });
        },

        _renderGotoWindow: function () {
            this._gotoWindow = ej.buildTag('div.e-gotodialog#' + this._id + "gotoWindow");
            var $goWindow = "<div><table width='100%' cellpadding='5'><tbody><tr><td>" + this._getLocalizedLabels("Date") + ":</td><td><input id='" + this._id + "gotodate' type='text' value='' /></td></tr>" +
                    "<tr><td>" + this._getLocalizedLabels("Showin") + ":</td><td><input type='text' id='" + this._id + "show' /></td></tr></tbody></table>";
            $goWindow += "<div class='e-floatright'><table><tr><td><div><button id='" + this._id + "gotook' type='button' class='e-gotobutton e-gotoOk'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                                  "<td><div><button id='" + this._id + "gotocancel' type='button' class='e-gotobutton e-gotoCancel'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></table></div></div>";
            this._on(this._gotoWindow, "click", ".e-gotobutton", $.proxy(this._gotoOk, this));
            this._gotoWindow.append($goWindow);
            this.element.append(this._gotoWindow);
            this._renderGoto();
            this._gotoWindow.ejDialog({
                width: 300,
                cssClass: this.model.cssClass,
                showOnInit: false,
                enableRTL: this.model.enableRTL, enableModal: true,
                title: this._getLocalizedLabels("Gotodate"),
                target: "#" + this._id,
                enableResize: false,
                allowKeyboardNavigation: false,
                close: $.proxy(this._gotoCancel, this)
            });
            $("#" + this._id + "gotoWindow_wrapper").appendTo("body").addClass("e-scheduledialog").find(".e-titlebar").addClass("e-dialogheader");
        },


        _highlightCurrentTime: function (showIndicator) {
            Globalize.culture(this.model.locale);
            if (showIndicator) {
                var flag = false, index = 0;
                for (var i = 0; i < this._dateRender.length; i++) {
                    if (new Date(new Date(this._dateRender[i]).setHours(0, 0, 0, 0)).getTime() === new Date(new Date().setHours(0, 0, 0, 0)).getTime()) {
                        flag = true;
                        index=i;
						break;
                    }
                }
				var tdelement=this.element.find(".e-workcellstab tr").first().find("td");
                if (this.currentView() !== "month" && !(this._isCustomView())) {
                    if (flag) {
                        var temp = this;
                        var view = this.currentView() === "week" ? 7 : this.currentView() === "workweek" ? this.model.workWeek.length : this.currentView() == "customview" ? this._dateRender.length : 1;
                        var timeWidth = (this.model.cellWidth) ? parseInt(this.model.cellWidth) : this.element.find('.e-draggableworkarea')[0].offsetWidth / (view * this.res1.length);
                        if (new Date().getHours() >= temp.model.startHour && new Date().getHours() <= temp.model.endHour) {
                            if (temp.model.orientation == "vertical") {
                                for (var i = 0; i < this.res1.length; i++) {
                                    this.element.find('.e-draggableworkarea').append('<div id="' + this._id + i + '_HighlightCurrentTimeline" class="e-highlightcurrenttimeline"></div>');
                                    $('#' + this._id + i + '_HighlightCurrentTimeline').append('<div class="e-currenttime"></div>').append(ej.buildTag("div.e-icon e-timeidicatorcircle e-lefttimeindicator")).append(ej.buildTag("div.e-icon e-timeidicatorcircle e-righttimeindicator")).css("width", timeWidth + "px").css("left", tdelement[index + (view * i)].offsetLeft + "px").find('.e-righttimeindicator').css("margin-left", (timeWidth - 3) + "px");
                                }
                            }
                            else {
                                this._renderHorizontalCurrentTime();
                            }
                        }
                        function ChangeCurrentTimePosition() {
                            var now = new Date();
                            now.setMinutes(now.getMinutes());
                            var position;
                            if (now.getHours() >= temp.model.startHour && now.getHours() <= temp.model.endHour) {
                                if (temp.model.orientation == "vertical") {
                                    var topVal = ((((now.getHours() - temp.model.startHour) * 60) + now.getMinutes()) * (temp.element.find(".e-workcells").height() + 1) * 2) / 60;
                                    for (var i = 0; i < temp.res1.length; i++)
                                        $('#' + temp._id + i + '_HighlightCurrentTimeline').css("top", topVal - 1);
                                    var currentTime = temp._timeMode == "12" ? Globalize.format(new Date(), "hh:mm tt") : Globalize.format(new Date(), "HH:mm");
                                    for (var i = 0; i < temp.res1.length; i++) {
                                        $('#' + temp._id + i + '_HighlightCurrentTimeline').find(".e-currenttime").html(currentTime);
                                        if (temp._timeMode == "12") $('#' + temp._id + i + '_HighlightCurrentTimeline').find(".e-currenttime").addClass("e-timewidth");
                                    }
                                }
                                else {
                                    temp._horizontalTimePosition(temp, now);
                                }
                            }
                        }
                        ChangeCurrentTimePosition();
                        this._intervalID = window.setInterval(ChangeCurrentTimePosition, 60000);
                    }
                    else {
                        clearInterval(this._intervalID);
                        this.element.find(".e-highlightcurrenttimeline").remove();
                    }
                }
                else {
                    if (flag) {
                        if (this.model.orientation == "vertical") {
                            var rowPosition = Math.ceil((index + 1) / 7); var cellId;
                            if (!this.model.group) this.res1 = ["0"];
                            for (var i = 0; i < this.res1.length; i++) {
                                cellId = (i * (7) + (new Date(this._dateRender[index]).getDay() + 1)) + ((rowPosition - 1) * ((this.res1.length) * 7));
                                cellId = this._firstdayofweek < 4 ? cellId - this._firstdayofweek : this._firstdayofweek < 7 ? cellId + 7 - this._firstdayofweek : cellId; 
                                $(this.element.find(".e-monthcells")[Math.floor(cellId) - 1]).addClass("e-monthcurrenttime");
                            }
                        }
                        else
                            this._renderHorizontalCurrentTime();
                    }
                    else
                        this.model.orientation == "vertical" ? this.element.find(".e-monthcells").removeClass("e-monthcurrenttime") : this.element.find(".e-workcells").removeClass("e-monthcurrenttime");
                }
            }
        },
        _filter: function () {
            var filter = []; var filtAppoint = [];
            this._filt = true;
            this._dataProcessing(this._currentAppointmentData);
            filtAppoint = $.extend(true, [], this._processed);
            this._processed.length = 0;
            this._filt = false;
            for (var a = 0; a < filtAppoint.length; a++) {
                if ((new Date(filtAppoint[a][this.model.appointmentSettings["endTime"]]) >= this.model.minDate) && (new Date(filtAppoint[a][this.model.appointmentSettings["endTime"]]) <= this.model.maxDate))
                    filter.push(filtAppoint[a]);
            }
            return filter;
        },
        _getPreviousNextAppointment: function () {
            var dateCount = this._dateRender;
			if (this.model.orientation == "horizontal" && this.currentView() == "month") {
				var dateFilter = [];
				for (var i = 0; i < this._dateRender.length; i++) {
					if (new Date(this.currentDate()).getMonth() === new Date(this._dateRender[i]).getMonth())
                        dateFilter.push(this._dateRender[i]);
                }
				dateCount = $.extend(true,[],dateFilter);
			}
            var appointmentsList = this._filter();
            this._previousList = [];
            this._nextList = [];
            this.appointment;
			if (this.currentView() != "workweek") {
				for (var app = 0; app < appointmentsList.length; app++) {
					if (new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]) < new Date(dateCount[0]) && new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]) < new Date(dateCount[0]))
						this._previousList.push(appointmentsList[app]);
					else if (new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]) > new Date(dateCount[(dateCount.length - 1)]) && new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]) > new Date(dateCount[(dateCount.length - 1)]))
						this._nextList.push(appointmentsList[app]);
				}
			}
			else {
				for (var app = 0; app < appointmentsList.length; app++) {
					for (var i = 0; i < dateCount.length; i++) {
						if (new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]).getDay() == new Date(dateCount[i]).getDay()) {
							if (new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]) < new Date(dateCount[0]) && new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]) < new Date(dateCount[0]))
								this._previousList.push(appointmentsList[app]);
							else if (new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]) > new Date(dateCount[(dateCount.length - 1)]) && new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]) > new Date(dateCount[(dateCount.length - 1)]))
								this._nextList.push(appointmentsList[app]);
						}
					}
				}
			}
            this.appointment = { previous: this._previousList, next: this._nextList };
        },

        _nextPrevAppointment: function (flag) {
            if (flag && !(this._browserInfo.name === "msie" && this._browserInfo.version < 9)) {
                var base = this.element.find("#" + this._id + "_scroller");
                this.height = base.height();
                this.width = base.width();
                var scroll = this.element.find(".e-vscroll").width();
                var timecell = this.element.find(".e-scrolltimecells").width();
                timecell = ej.isNullOrUndefined(timecell) ? this.model.orientation == "vertical" ? 56 : 0 : timecell;
                var top = (this.height / 2) - (150 / 2), left;
                var prev = ej.buildTag("div.e-prevapp enable", this._getLocalizedLabels("PreviousAppointment"), {}, {});
                var next = ej.buildTag("div.e-nextapp enable", this._getLocalizedLabels("NextAppointment"), {}, {});
                left = (this.model.showTimeScale && this.currentView() != "agenda") ? (this.model.orientation == "vertical") ? ((this.currentView() != "month") && !(this._isCustomView()) ? -4 : -10) : -61 : -61;
                this.width = (this.model.showTimeScale && this.currentView() != "agenda") ? this.width : this.width - timecell;
                if (!this.model.enableRTL) {
                    prev.css("top", (top + 40) + "px").css("transform", "rotate(90deg)").css("left", left + "px");
                    next.css("top", (top + 40) + "px").css("transform", "rotate(-90deg)").css("left", (this.width - scroll - (this.model.orientation == "vertical" && (this.currentView() == "month" || this._isCustomView()) ? 41 : this.model.orientation == "vertical" ? 34 : 90)) + "px");
                }
                else {
                    prev.css("top", (top + 40) + "px").css("transform", "rotate(-90deg)").css("right", left + "px");
                    next.css("top", (top + 40) + "px").css("transform", "rotate(90deg)").css("right", (this.width - scroll - (this.model.orientation == "vertical" && (this.currentView() == "month" || this._isCustomView()) ? 41 : this.model.orientation == "vertical" ? 34 : 90)) + "px");
                }
                base.append(prev).append(next);
            }
        },

        _nextPrev: function (e) {
            var proxy = this;
            if ($(e.currentTarget).hasClass("e-prevapp")) {
                this.appointment.previous.sort(function (a, b) {
                    return new Date(b[proxy.model.appointmentSettings["startTime"]]) - new Date(a[proxy.model.appointmentSettings["startTime"]]);
                });
				if (this.currentView() == "customview") this._daterend = this.appointment.previous[0][this.model.appointmentSettings["startTime"]];
                this._navigateClick(this.currentDate(new Date(this.appointment.previous[0][this.model.appointmentSettings["startTime"]])), "Previous");
            }
            else if ($(e.currentTarget).hasClass("e-nextapp")) {
                this.appointment.next.sort(function (a, b) {
                    return new Date(a[proxy.model.appointmentSettings["startTime"]]) - new Date(b[proxy.model.appointmentSettings["startTime"]]);
                });
				if (this.currentView() == "customview") this._daterend = this.appointment.next[0][this.model.appointmentSettings["startTime"]];
                this._navigateClick(this.currentDate(new Date(this.appointment.next[0][this.model.appointmentSettings["startTime"]])), "Next");
            }
        },

        _nextPrevButton: function (app) {
            if (app.length != 0)
                this.element.find(".e-prevapp,.e-nextapp").css("display", "none");
            else {
                this._getPreviousNextAppointment();
                this.element.find(".e-prevapp,.e-nextapp").css("display", "block");
                if (this.appointment.previous.length == 0)
                    this.element.find(".e-prevapp").removeClass("enable").css("opacity", "0.4");
                else
                    this.element.find(".e-prevapp").addClass("enable").css("opacity", "1");
                if (this.appointment.next.length == 0)
                    this.element.find(".e-nextapp").removeClass("enable").css("opacity", "0.4");
                else
                    this.element.find(".e-nextapp").addClass("enable").css("opacity", "1");
            }

        },

        _navigateView: function (e) {
            var newView, navidate, index, cellIndex, oldview = this.currentView();
            var target = $(e.target);
            if (this.currentView() == "agenda") {
                return false;
            }
            else {
                if (this.model.orientation == "vertical") {
                    if (this.currentView() === "month" || (this._isCustomView())) {
                        if (target.hasClass("e-monthheader")) {
                            if (this.model.views.toString().toLowerCase().split(",").indexOf("day") == -1) return false;
                            cellIndex = target.parent().index();
                            index = (cellIndex > 6) ? (cellIndex % 7) : cellIndex;
                            navidate = new Date(this._getRenderDates()[((target.parent().parent().index() * 7) + index)]);
                            if (this.model.minDate <= navidate && this.model.maxDate >= navidate)
                                this.currentDate(navidate);
                            else return false;
                            newView = "day";
                        }
                        else if (target.hasClass("e-timecells") || target.hasClass("e-timecelldivs")) {
                            target = target.hasClass("e-timecells") ? target : target.parent();
                            if (this.model.views.toString().toLowerCase().split(",").indexOf("week") == -1) return false;
                            navidate = new Date(this._getRenderDates()[(target.parent().index() * 7)]);
                            if (this.model.minDate <= navidate && this.model.maxDate >= navidate)
                                this.currentDate(navidate);
                            else return false;
                            newView = "week";
                        }
                    }
                    else if (this.currentView() === "week" || this.currentView() === "workweek" || (this.currentView() === "customview" && this._renderDays <= 7)) {
                        var dayTarget = target.hasClass("e-headercells") ? target : target.parent().hasClass("e-headercells") ? target.parent() : target.parent().parent();
                        if (this.model.views.toString().toLowerCase().split(",").indexOf("day") == -1 || !dayTarget.hasClass("e-headercells")) return false;
                        cellIndex = dayTarget.index();
                        var cols = this.currentView() === "workweek" ? this.model.workWeek.length : this.currentView() === "customview" ? this._dateRender.length : 7;
                        index = (cellIndex > cols - 1) ? (cellIndex % cols) : cellIndex;
                        navidate = new Date(this._getRenderDates()[index]);
                        if (this.model.minDate <= navidate && this.model.maxDate >= navidate)
                            this.currentDate(new Date(navidate));
                        else return false;
                        newView = "day";
                    }
                    this._navView = true;
                    if (oldview != "day" && !ej.isNullOrUndefined(newView)) {
                        this._viewChange(e, oldview, newView);
                        this._businessHourScroller();
                    }
                }
                else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                    this._horizontalViewNavigation(e);
            }
        },

        _appOverflowClick: function (e) {
            if (this.model.views.toString().toLowerCase().split(",").indexOf("day") !== -1) {
                var overflowId = $(e.target)[0].id.toString().split("_");
                this.currentDate(new Date(overflowId[3], overflowId[1] - 1, overflowId[2]));
                this._navView = true;
                this._viewChange(e, this.currentView(), "day");
            }
            else return false;
        },

        _viewCalendar: function (e) {
            var obj = this.element.find("#" + this._id + "_navigator").data("ejDatePicker");
            if (obj) {
                obj.show();
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                var controlmargin = this.element.offset();
                if (this.model.enableRTL)
                    $("#e-" + this._id + "_navigator").css("top", (controlmargin.top + 47) + "px").css("left", (controlmargin.left + this.element.width() - 280) + "px");
                else
                    $("#e-" + this._id + "_navigator").css("top", (controlmargin.top + 47) + "px").css("left", (controlmargin.left + 55) + "px");
            }
            this.element.find(".e-workcells").removeClass("e-selectedCell");
            this.element.find(".e-workcells").removeClass("e-mousehover");
            this.element.find(".e-appointment,.e-alldayappointment,.e-monthappointment").removeClass("e-selectedAppointment");
        },

        _viewChange: function (e, oldview, newView) {
            this._viewchange = true;
            var flag = true, data;
            data = { currentView: this.currentView(), currentDate: new Date(this.currentDate()), target: e, requestType: "viewNavigate" };
            if (this._trigger("actionBegin", { data: data, requestType: "viewNavigate" }))
                return false;
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
            if (oldview !== newView || (!oldview && !newView)) {
                oldview = this.currentView();
                if (!this._navView) {
                    flag = (this.currentView() == (e.currentTarget.className.split("e-")[2]).split(" ")[0]) ? false : true;
                    this.currentView($(e.currentTarget).hasClass("e-activeview") ? this.currentView() : (e.currentTarget.className.split("e-")[2]).split(" ")[0]);
                }
                else
                    this.currentView(newView);
                data = { previousView: oldview, currentView: this.currentView(), currentDate: new Date(this.currentDate()), target: e, requestType: "viewNavigate" };
                if (this._trigger("navigation", data)) {
                    this.currentView(oldview);
                    this._navView = false;
                    return false;
                }
                if (flag) this._renderCurrentView("ViewChange");
            }
            data = { previousView: oldview, currentView: this.currentView(), currentDate: new Date(this.currentDate()), target: e, requestType: "viewNavigate" };
            (this._mediaQuery && (oldview != this.currentView())) && this.element.find(".e-workareadiv,.e-navigate").addClass("navigate");
            this._trigger("actionComplete", { data: data, requestType: "viewNavigate" });
        },

        _renderCurrentView: function (currentAction) {
            this.element.find(".e-commonviewbutton").blur();
            this.element.find(".e-commonviewbutton").removeClass("e-activeview");
            if (this.model.orientation == "vertical")
                this.element.find(".e-headerdaydisplay,.e-datedisplay,.e-headrealldaytable").children().remove();
            this.wTR.children().remove();
            this.aTR.children().remove();
            this._calculateCustomDays()._getRenderDates();
            this.element.find(".e-commonviewbutton.e-" + this.currentView()).addClass("e-activeview").attr("tabIndex", 0).focus();
            if((!ej.isNullOrUndefined(this._scheduleDrawer)) && (!this._navSelected))
            this._scheduleDrawer.ejNavigationDrawer({ listViewSettings: { selectedItemIndex: this.model.views.toString().toLowerCase().split(',').indexOf(this.currentView()) + 1 } });
            this._renderHeaderAllDayTemplate();
            this._renderContentAreaTemplate();
            if (this.currentView() != "agenda") {
                (this.element.find('.e-workcells').width() < this.element.find('.e-dateheadercell').width() && this.model.orientation == "horizontal" && this.model.cellWidth == "") ? this.element.find('.e-workcells').width(this.currentView() == "month" || (this._isCustomView()) ? this.element.find('.e-horizontmonthheaderdaytd').width() : this.element.find('.e-horizontaltimetd ').width()) : "";
                if (this.model.orientation == "vertical") {
                    this._headerbarWidth();
                    this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
                    this.element.find(".e-ampmdisplay").height(this.element.find(".e-workcells").height());
                    if (!this.model.showTimeScale) {
                        this.element.find(".e-headercells").first("td").css("border-left", "none");
                        this.element.find(".e-workcellstab").find("tr").find("td:first-child").css("border-left", "none");
                    }
                }
                (this.model.showTimeScale) && this._borderAddRemove();
                this.model.enableRTL && this.element.removeClass("e-rtl");
                this._renderScroller();
                this.model.enableRTL && this.element.addClass("e-rtl");
                this.model.orientation == "vertical" ? this.aTR.find(".e-headeralldaydiv").parent().next().append(this.element.find(".e-vscroll").length > 0 ? this._emptyCell() : "") : "";
                if (this._mediaQuery && this.model.cellWidth && this.currentView() == "day" && (parseInt(this.element.find(".e-headertr td").siblings("td:last-child").css("width")) > this.element.find(".e-vscroll").width())) {
                    this.element.find(".e-headertr td").siblings("td:last-child").css("width", this.element.find(".e-vscroll").width() + "px");
                }
                if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
                    this._moveScroller();
                    if (!this.model.showTimeScale) {
                        // this.element.find(".e-horizontaldaterender").css("border-left", "none");
                        this.element.find(".e-headercells").removeClass("e-dateheaderselect");
                    }
                }
                this._renderHeaderDays();
            }
            else {
                if (this.model.showHeaderBar) {
                    var dateVal = this._getCurrentDateHeaderValue();
                    this.element.find("div.e-datedisplay")[0].innerHTML = dateVal;
                }
            }
            if (!ej.isNullOrUndefined(this.dataSource())) {
                if (this.model.enableLoadOnDemand)
                    this._bindAppointmentsData(currentAction);
                else {
                    this._dataProcessing(this._currentAppointmentData);
                    this._renderAppointmentAll();
                }
                if (this._app.length < 1 && this.model.enableAppointmentNavigation)
                    this._nextPrevButton(this._app);
            }
            if (this.currentView() != "agenda") {
                (this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
            }
            this._validateMinMaxDate();
            this._navView = false;
            (this._dateRender.length > 60 || ($.isNumeric(this.element.find(".e-monthcells").height()) && (this.element.find(".e-monthcells").height() != $(this.element.find(".e-timecells")[3]).height()) && this.model.orientation == "vertical")) ? this.element.find('td.e-timecells .e-timecelldivs').addClass('e-timedivcells') : "";
            (this.model.orientation == "horizontal" && this._browserInfo.name == "webkit") && this.element.find(".e-workcells.e-dayend").width(this.element.find(".e-workcells.e-dayend").outerWidth() + 1);
            if (this.currentView() != "agenda" && this.model.orientation == "horizontal" && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources))) {
                var border = this.model.enableRTL ? 'border-right' : 'border-left';
                this.element.find(".e-horizontaldaterender").css(border, '0px');
                this.element.find(".e-workareadiv").parent().css(border, '0px');
            }
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && this.model.orientation == "vertical") {
            this.element.find(".e-workleftindent").find("tr").last().find("td").addClass("e-lastcell");
            this.element.find(".e-workcellstab").find("tr").last().find("td").addClass("e-lastcell");
            }
        },

        _renderHeaderDays: function () {
            this.dateRender = [];
            var dateList = this._getRenderDates();
            this.res1 = (!ej.isNullOrUndefined(this.model.group)) ? this.res1 : [0];
            for (var i = 0; i < this.res1.length; i++) {
                for (var j = 0; j < dateList.length; j++) {
                    this.dateRender.push(new Date(dateList[j]));
                }
            }
            var localizeDays = Globalize.culture(this.model.locale);
            if (this.model.showHeaderBar) {
                var dateVal = this._getCurrentDateHeaderValue();
                this.element.find("div.e-datedisplay")[0].innerHTML = dateVal;
            }
            (this.currentDate() < this.model.minDate) && this.currentDate(this.model.minDate);
            (this.currentDate() > this.model.maxDate) && this.currentDate(this.model.maxDate);
            if (this.model.orientation == "vertical") {
                if (this.currentView() !== "month" && !(this._isCustomView())) {
                    var curday, temp;
                    for (var i = 0; i < this.dateRender.length; i++) {
                        curday = (this.currentView() === "day") ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? localizeDays.calendar.days.names[new Date(this.dateRender[i]).getDay()] : localizeDays.calendar.days.names[new Date(this.dateRender[i]).getDay()].slice(0, 3) : localizeDays.calendar.days.names[new Date(this.dateRender[i]).getDay()].slice(0, 3);
                        temp = this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? curday + " " + Globalize.format(new Date(this.dateRender[i]), "dd MMMM yyyy") : curday + " " + Globalize.format(new Date(this.dateRender[i]), "dd MMM yy") : curday + " " + new Date(this.dateRender[i]).getDate();
                        this.element.find(".e-headerdaydisplay")[i].innerHTML = temp;
                    }
                    if (!this._mediaQuery)
                        this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").css('height', "27px");
                    else {
                        this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").attr('style', "height:40px !important");
                        if (!this.model.enableRTL)
                            this.element.find(".e-alldaycells").first().css("border-left", "none");
                        else
                            this.element.find(".e-alldaycells").first().css("border-right", "none");
                    }
                    this.element.find(".e-headercells").addClass("e-dateheaderselect");
                }
                else
                    this._renderMonthTimeCells();
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                this._renderHorizontalHeaderDays(localizeDays);
        },

        _renderMonthTimeCells: function () {
            var timeCellIndex = 0; var dateRender = this._dateRender, k = 0, resCount = !ej.isNullOrUndefined(this.model.group) ? this.res1.length * 7 : 7;
            var findResource = (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) ? true : false;
            if (findResource)
                resCount = this._isCustomView() && this._oneWeek ? this.res1.length * this._dateRender.length : this.res1.length * 7;
            else
                resCount = !ej.isNullOrUndefined(this.model.group) ? this._isCustomView() && this._oneWeek ? this.res1.length * this._dateRender.length : this.res1.length * 7 : 7;
            var res = findResource && this._isCustomView() && this._oneWeek ? true : false;
            for (var i = 0; i < dateRender.length; i++) {
                for (var j = k; j < resCount; j = j = res ? j + this._dateRender.length : j + 7) {
                    $(this.element.find("div.e-monthheader"))[j].innerHTML = (Globalize.format(new Date(dateRender[i]), "dd")).toString();
                    if (this.currentView() !== "customview") {
                        if (new Date(dateRender[i]).getMonth() !== new Date(this.currentDate()).getMonth())
                            $(this.element.find("div.e-monthheader")[j]).addClass("e-monthdateshighlight");
                        else
                            $(this.element.find("div.e-monthheader")[j]).removeClass("e-monthdateshighlight");
                    }
                }
                k++;
                if ((i + 1) % 7 == 0) {
                    k = resCount;
                    resCount = resCount + ((!ej.isNullOrUndefined(this.model.group) ? this.res1.length : 1) * 7);
                    start = Globalize.format(new Date(dateRender[i - 6]), "MMM dd");
                    end = Globalize.format(new Date(dateRender[i]), "MMM dd");
                    if (this.model.showTimeScale) $(this.element.find(".e-timecelldivs"))[timeCellIndex].innerHTML = start + "- " + end;
                    timeCellIndex++;
                }
            }
            this.element.find(".e-timecells").addClass("e-dateheaderselect").height(this.element.find(".e-monthcells").height());
            if (j != 6) this.element.find(".e-timecells").addClass("e-timecellpadding");
        },

        _navigateClick: function (e, dateNavigate) {
            this._viewchange = true;
            var data, currentAction = dateNavigate;
            var oldDate = new Date(this.currentDate());
            data = { currentView: this.currentView(), currentDate: new Date(this.currentDate()), target: e, requestType: "dateNavigate" };
            if (this._trigger("actionBegin", { data: data, requestType: "dateNavigate" }))
                return false;
            if (!ej.isNullOrUndefined(e) && !ej.isNullOrUndefined(e.value)) {
                var selectdate = Globalize.parseDate(e.value, this._pattern.d);
                this.currentDate(selectdate);
            }
            if (e != null && !this._setCurrentDate) {
                if ($(e.currentTarget).hasClass("e-disable")) { return false; }
                var dat = new Date(this.currentDate());
                var day = this.currentView() === "day" || this.currentView() === "agenda" ? 1 : this.currentView() === "customview" ? this._dateRender.length : 7;
                var todayDate;
                if ($(e.currentTarget).hasClass("e-navigateprevious")) {
                    this._daterend = this._dateRender;
                    if (this._isCustomView()) {
                        todayDate = new Date(this._dateRender[0]), weekDate = new Date(this._dateRender[0]);
                        weekDate.setTime(todayDate.getTime() - (this._dateRender.length * 24 * 3600000));
                        this.currentDate(new Date(weekDate));
                    }
                    else
                        this.currentDate((this.currentView() === "month" ? new Date(dat.setMonth(new Date(this.currentDate()).getMonth() - 1)) : new Date(dat.setDate(new Date(this.currentDate()).getDate() - day))));
                    this.element.find(".e-navigateprevious").attr("tabIndex", 0).focus();
                    currentAction = "Previous";
                }
                else if ($(e.currentTarget).hasClass("e-navigatenext")) {
                    if (this.currentView() == "customview") {
                        this._daterend = this._dateRender;
                        todayDate = new Date(this._dateRender[0]), weekDate = new Date(this._dateRender[0]);
                        weekDate.setTime(todayDate.getTime() + (this._dateRender.length * 24 * 3600000));
                        this.currentDate(new Date(weekDate));
                    }
                    else
                        this.currentDate((this.currentView() === "month" ? new Date(dat.setMonth(new Date(this.currentDate()).getMonth() + 1)) : new Date(dat.setDate(new Date(this.currentDate()).getDate() + day))));
                    this.element.find(".e-navigatenext").attr("tabIndex", 0).focus();
                    currentAction = "Next";
                }

            }
            if (e != null)
                if ($(e.currentTarget).hasClass("e-navigateprevious") || $(e.currentTarget).hasClass("e-navigatenext") || (e.type == "select"))
                    data = { previousDate: oldDate, currentDate: dat, currentView: this.currentView(), target: e, requestType: "dateNavigate" };
            if (this._trigger("navigation", data)) {
                this.currentDate(new Date(oldDate));
                return false;
            }
            this.element.find(".e-workcells,.e-alldaycells,.e-monthcells").removeClass("e-selectedCell");
            this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment"); this.element.find(".e-appsoverflow").remove();
            this.element.find(".e-monthcells").removeClass("e-monthcurrenttime");
            if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
            if (this.currentView() === "month" || this.currentView() === "agenda" || (this._isCustomView())) {
                this._navigate = true;
                this._renderCurrentView(currentAction);
            }
            else {
                this.model.showTimeScale ? this._renderHeaderDays() : this._renderCurrentView(currentAction);
                (!this._mediaQuery) && this.model.orientation == "vertical" && (this.model.showTimeScale) && this._reRenderScroller();
                if (!ej.isNullOrUndefined(this.dataSource())) {
                    if (this.model.enableLoadOnDemand)
                        this._bindAppointmentsData(currentAction);
                    else {
                        this._dataProcessing(this._currentAppointmentData);
                        this._renderAppointmentAll();
                    }
                }
                (this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
            }
            this._validateMinMaxDate();
            this._setCurrentDate = false;
            if (this._mediaQuery) {
                var value = ($(e.currentTarget).hasClass("e-navigateprevious")) ? this.element.width() : ($(e.currentTarget).hasClass("e-navigatenext") || (e.type == "select")) ? -this.element.width() : 0;
                this.element.find(".e-headeralldaydiv,.e-draggableworkarea,.e-vscroll").toggle();
                this.element.find(".e-headeralldaydiv,.e-draggableworkarea,.e-vscroll").css("margin-left", -value).css("margin-right", value).toggle().animate({ "margin-left": 0, "margin-right": 0 }, 500, (this.model.orientation == "vertical" && this.element.find(".e-vscroll").length == 1 && (this._browserInfo.name == "webkit" || this._browserInfo.name == "msie")) ? $.proxy(this._animateScroll, this) : "");
            }
            this._trigger("actionComplete", { data: data, requestType: "dateNavigate" });
        },
        _animateScroll: function () {
            highScrol = this.element.find('.e-businesshighlightworkcells').length != 0 ? this.element.find('.e-businesshighlightworkcells').first().parent()[0].rowIndex * $('.e-workcells').height() : 0;
            $("#" + this._id + "_scroller").ejScroller({ scrollTop: highScrol + 1 });
            this.element.find('.e-scrolltimecells').css('top', -(highScrol + 1) + 'px');
        },
        _validateMinMaxDate: function () {
            this.element.find(".e-navigateprevious").removeClass("e-disable");
            this.element.find(".e-navigatenext").removeClass("e-disable");
            var currDate = new Date(this.currentDate());
            var renderDate = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : this._dateRender;
            var firstWeekDate = new Date(renderDate[0]);
            var lastWeekDate = new Date(renderDate[renderDate.length - 1]);
            if (this.currentView() == "workweek") {
                firstWeekDate = new Date(firstWeekDate.setDate(firstWeekDate.getDate() - 1));
                lastWeekDate = new Date(lastWeekDate.setDate(lastWeekDate.getDate() + 1));
            }
            currDate = firstWeekDate;
            if (currDate <= this.model.minDate)
                this.element.find(".e-navigateprevious").addClass("e-disable");
            else
                this.element.find(".e-navigateprevious").removeClass("e-disable");
            currDate = lastWeekDate;
            if (currDate >= this.model.maxDate)
                this.element.find(".e-navigatenext").addClass("e-disable");
            else
                this.element.find(".e-navigatenext").removeClass("e-disable");
        },

        _onScheduleResize: function () {
            !ej.isNullOrUndefined(this._scheduleDrawer) && this._scheduleDrawer.ejNavigationDrawer("close");
            var temp = this._mediaQuery;
            var hasScrollbar = (typeof window.innerWidth === 'number') && window.innerWidth > document.documentElement.clientWidth;
            this._mediaQuery = (this._browserInfo.name == "msie" && parseInt(this._browserInfo.version, 10) <= 8) ? false : ((document.documentElement.clientWidth + (hasScrollbar ? 17 : 0) < 361 || ej.isMobile()) && this.model.isResponsive);
            (this._mediaQuery) ? this.element.addClass("e-scheduleresponsive") : this.element.removeClass("e-scheduleresponsive");
            var scrollerObj = this.element.find(".e-scroller").data("ejScroller");
            if (this.currentView() != "agenda") {
                var scroll = this.element.find(".e-vscroll").width();
                this.element.find(".e-nextapp").css("left", (this.element.width() - scroll - 90) + "px");
                this.model.enableRTL && this.element.find(".e-nextapp").css("left", -45 + "px");
                this.model.enableRTL && this.element.find(".e-prevapp").css("left", (this.element.width() - this.element.find(".e-scrolltimecells").width() - 89) + "px");
                var cols = this.currentView() == "workweek" ? this.model.workWeek.length : this.currentView() == "day" ? 1 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : 7;
                var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
                var overallWidth = parseInt(this.model.cellWidth) * col;
                var scrollWidth = (this.model.orientation === "vertical") ? ((hasScrollbar && this._mediaQuery) ? this.element.outerWidth() + 17 : this.element.outerWidth()) - (this._mediaQuery && this.currentView() == "month" ? "" : this.element.find('.e-scrolltimecells').parent().outerWidth()) : (this.model.orientation == "horizontal") && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.element.width() - this.element.find(".e-resourceheadertable").width() : this.element.width();
                scrollerObj.model.width = scrollWidth - 2;
                scrollerObj.model.height = this.scHeight - 1 - (this.element.find('.e-draggableworkarea').offset().top - this.element.offset().top);
                scrollerObj.refresh();
                if (this.model.orientation == "vertical") {
                    if (this.model.cellWidth && !this._mediaQuery) {
                        this.element.find(".e-draggableworkarea").width((this.element.width() > ((parseInt(this.model.cellWidth) + 1) * col) + this.element.find(".e-scrolltimecells").parent().width() + (scrollerObj.isVScroll() ? 18 : 0)) ? ((parseInt(this.model.cellWidth) + 1) * col) - 1 : scrollWidth - 2 - (scrollerObj.isVScroll() ? 18 : 0));
                    }
					if (!this.model.showTimeScale) {
						this.element.find('.e-draggableworkarea').css("height", (this.model.cellHeight != "20px" ? parseInt(this.model.cellHeight) : (scrollerObj.model.height - 1 + "px")));
						this.element.find('.e-draggableworkarea').height(!ej.isNullOrUndefined(this.element.find(".e-hscroll")) ? this.element.find(".e-draggableworkarea").height() - this.element.find(".e-hscroll").height() : this.element.find(".e-draggableworkarea").height());
					}
                    else if (this._mediaQuery && this.res1.length > 1 && this.model.group.resources.length == 1)
                        this.element.find(".e-draggableworkarea").width((this.element.find(".e-draggableworkarea").width() * this.res1.length) - 1);
                    this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());
                    this.element.find(".e-datecommondiv").parent().width(this.element.width() - 2);
                    this.element.find(".e-headrealldaytable").css("left", -(this.element.find(".e-draggableworkarea").scrollLeft()) + "px");
                }
                else
                    $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", -(scrollerObj.model.scrollLeft) + "px");
                if (this.currentView() != "month" && !(this._isCustomView()) && this.model.orientation == "vertical") {
                    if (this.model.enableRTL && (this.model.cellWidth || this.model.cellHeight != "20px"))
                        this._renderCurrentView("ScheduleResize");
                    else {
                        (this.model.showAllDayRow) && this.element.find("div.e-alldayappointment").remove() && this._renderAllDayAppointments();
                        if (!this.model.showTimeScale) {
                            this.element.find("div.e-monthappointment").remove();
                            this._renderMonthAppointment();
                        }
                        this._unWireResizeEvents();
                        this._wireResizeEvents();
                        this._dragdropAppointments();
                    }
                }
                else {
                    this._renderCurrentView("ScheduleResize");
                }
                this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());
                if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                this.element.find(".e-scrolltimecells").css("top", -(this.element.find(".e-draggableworkarea").scrollTop()) + "px");
                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
            }else {
                scrollerObj.model.width = this.element.width() - 2;
                scrollerObj.model.height = this.scHeight - 1 - (this.element.find('.e-draggableworkarea').offset().top - this.element.offset().top);
                scrollerObj.refresh();
                var _scrollDiv = $('div#' + this._id + "_scroller");
                this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());
                this.element.find(".e-datecommondiv").parent().width(this.element.width() - 2);
                 _scrollDiv.ejScroller("isVScroll") ? (this.element.find(".e-emptytd").show() && this.element.find('.e-draggableworkarea').height(this.element.find('.e-draggableworkarea').height() + 1)) : this.element.find(".e-emptytd").hide();             
            }
            ((this._mediaQuery || this.element.width() < 600) && !$("#" + this._id + "navDrawer").hasClass("e-scheduledrawer")) && this._renderNavigationDrawer();
            var flag = (this._mediaQuery == temp) ? false : true;
            if (this._mediaQuery)
                this.element.find(".e-vscroll").css("left", (this.model.enableRTL) ? 0 : this.element.outerWidth() - 18);
            if (!flag && (this.element.width() < 600 && this.hTD.find(".e-mobileNavigation").length < 1 || this.element.width() > 600 && this.hTD.find("table.e-viewstable").length < 1)) {
                this._reRenderHeaderBar();
                this._on(this.element, "click", "#" + this._id + "navDrawerIcon", $.proxy(this._drawerOpen, this));
            }         
            if (flag) {
                this.model.enableAppointmentNavigation = flag && !this._mediaQuery;
                if (!this.model.enableAppointmentNavigation)
                    this.element.find(".e-prevapp,.e-nextapp").remove();
                else {
                    this._nextPrevAppointment(this.model.enableAppointmentNavigation);
                }
                this.model.showHeaderBar && this._reRenderHeaderBar();
                this.element.find(".e-leftindenttable").width(this.element.find(".e-workleftindent").width());
                this.element.find(".e-ampmdisplay").height(this.element.find(".e-workcells").height());
                this._businessHourScroller();
                (this._mediaQuery) && (this.model.showQuickWindow = false);
                this.model.showQuickWindow && (this._renderQuickAppWindow() || this._renderQuickAppDetailsView());
                this._destorySubControls($("#" + this._id + "_AddEditForm"));
                this._destorySubControls($("#" + this._id + "_RecurForm"));
                $("#" + this._id + "AppointmentAddEditWindow_wrapper").remove();
                this._alertWindow.ejDialog("close");
                this._recurEditWindow.ejDialog("close");
                this._renderAppointmentWindow();
                if (this._mediaQuery)
                    this._phoneChanges();
                this._renderAppointmentAll();
                if (this._mediaQuery) {
                    this._off(this.element, ej.eventType.mouseDown, "td.e-workcells,td.e-monthcells,td.e-alldaycells");
                    this._off(this.element, "keydown click", "td.e-workcells,td.e-monthcells,td.e-alldaycells");
                    this._on(this.element, "click", "#" + this._id + "navDrawerIcon", $.proxy(this._drawerOpen, this));
                }
                else {
                    this._on(this.element, ej.eventType.mouseDown, "td.e-workcells,td.e-monthcells,td.e-alldaycells", $.proxy(this._mouseButtonClick, this));
                    this._on(this.element, "keydown click", "td.e-workcells,td.e-monthcells,td.e-alldaycells", $.proxy(this._shiftSelection, this));
                    (this.element.width() > 600) && this._off(this.element, "click", "#" + this._id + "navDrawerIcon");
                }
                this._on(this.element, (ej.isMobile()) ? "click" : "dblclick", ".e-workcells,.e-alldaycells,.e-alldayappointment,.e-monthcells", $.proxy(this._appointmentWindow, this));
                if (this._mediaQuery || ej.isMobile())
                    this._on(this.element, "tap", ".e-appointment,.e-alldayappointment,div.e-monthappointment,.e-agendaappointment", $.proxy(this._appointmentWindow, this));
                else
                    this._off(this.element, "tap", ".e-appointment,.e-alldayappointment,div.e-monthappointment,.e-agendaappointment");
                var scheduleResponsive = $("#" + this._id + "AppointmentAddEditWindow_wrapper,#" + this._id + "RecurrenceEdit_wrapper,#" + this._id + "navDrawer,#" + this._id + "alertWindow_wrapper");
                (this._mediaQuery) ? scheduleResponsive.addClass("e-scheduleresponsive") : scheduleResponsive.removeClass("e-scheduleresponsive");
            }
            (this._mediaQuery || this.element.width() < 600) && this.element.find(".e-mobileNavigation").css("left", (this.model.enableRTL) ? 18 : this.element.outerWidth() - 36);
        },

        _reRenderHeaderBar: function () {
            var header = this.element.find(".e-scheduleheader");
            header.empty();
            this._renderHeaderBar(header);
            this.element.find("#" + this._id + "_navigator").ejDatePicker({ enableRTL: this.model.enableRTL, buttonText: this._getLocalizedLabels("Today"), select: $.proxy(this._onselect, this), locale: this.model.locale, cssClass: this.model.cssClass, minDate: this.model.minDate, maxDate: this.model.maxDate });
            this.element.find(".e-datecommondiv").find(".e-datewidget").css("display", "none");
            this.element.find(".e-commonviewbutton.e-" + this.currentView()).addClass("e-activeview");
        },

        _renderTemplates: function () {
            if (this.model.orientation == "vertical") {
                this.workcellTemp = $.templates(this._createTemplate(workCellsTemplate(), "_WorkCells"));
                this.monthcellTemp = $.templates(this._createTemplate(monthCellsTemplate(), "_MonthCells"));
                this.timeCellTemplate = $.templates(this._createTemplate(timeCellsTemplate(), "_TimeCells"));
                this.headTemplate = $.templates(this._createTemplate(headerTemplate(), "_HeaderDate"));
                this.alldayTemp = $.templates(this._createTemplate(alldayTemplate(), "_AlldayCells"));
                this.leftIndent = $.templates(this._createTemplate(leftHeaderTemplate(), "_LeftIndentCells"));
            }
            if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                this._renderResourcesTemplate();
            this.agendaheadTemplate = $.templates(this._createTemplate(agendaHeaderTemplate(), "_HeaderDate"));
            this.agendacellTemp = $.templates(this._createTemplate(agendaCellsTemplate(), "_AgendaCells"));
            this.appTemplate = $.templates(this._createTemplate(appointmentTemplate(), "_Appointment"));
            if (ej.scheduleFeatures.horizontal && this.model.orientation === "horizontal")
                this._renderHorizontalTemplates();
            if (!ej.isNullOrUndefined(this.model.categorizeSettings) && this.model.categorizeSettings.enable) {
                this._renderCategorizeTemplate();
                this._categorizeMultipleTemplate = $.templates(this._createTemplate(categorizeMultipleTemplate(), "_CategorizeMultiple"));
            }
        },

        _setNewStartEndHours: function () {
            if (this.currentView() !== "month" && !(this._isCustomView())) {
                this.wTR.children().remove();
                this._renderContentAreaTemplate();
                this._borderAddRemove();
                this._renderScroller();
                this.element.find(".e-ampmdisplay").height(this.element.find(".e-workcells").height());
                (this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").addClass("e-businesshighlightworkcells");
                if (this.model.showCurrentTimeIndicator) this.element.find(".e-highlightcurrenttimeline").remove();
                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                this._renderAppointmentAll();
            }
        },

        _renderInitSchedule: function () {
            if (!this.model.showTimeScale) { this.model.showAllDayRow = false; this.model.showCurrentTimeIndicator = false; }
            var hasScrollbar = (typeof window.innerWidth === 'number') && window.innerWidth > document.documentElement.clientWidth;
            this._mediaQuery = (this._browserInfo.name == "msie" && parseInt(this._browserInfo.version, 10) <= 8) ? false : ((document.documentElement.clientWidth + (hasScrollbar ? 17 : 0) < 361 || ej.isMobile()) && this.model.isResponsive);
            (this._mediaQuery) ? this.element.addClass("e-scheduleresponsive") : this.element.removeClass("e-scheduleresponsive");
            (this._mediaQuery) && (this.model.showQuickWindow = false);
            var culture = Globalize.culture(this.model.locale);
            var localizeAmPm = culture.calendar;
            (ej.isNullOrUndefined(this._clientWidth)) && (this._clientWidth = this.model.width);
            this.currentView(this.currentView().toLowerCase());
            this.model.views = this.model.views.toString().toLowerCase().split(",");
            this.model.workWeek = (this.model.workWeek == "" || this.model.workWeek == null) ? this._dayNamesArray : this.model.workWeek;
            this._timeMode = (this.model.timeMode == null) ? (localizeAmPm.AM == null && localizeAmPm.PM == null) ? "24" : "12" : this.model.timeMode;
            if (this.model.timeMode == null) { this._pattern = culture.calendar.patterns; }
            else this._pattern.t = (this._timeMode == "12") ? "h:mm tt" : "HH:mm";
            this._firstdayofweek = (this.model.firstDayOfWeek != null) ? (typeof this.model.firstDayOfWeek == "string") ? this._dayNamesArray.indexOf(this.model.firstDayOfWeek.toString().toLowerCase()) : culture.calendar.firstDay : this.model.firstDayOfWeek;
            if(this.model.firstDayOfWeek == null)this._firstdayofweek = culture.calendar.firstDay;
            if(this.model.firstDayOfWeek == null)this._firstdayofweek = culture.calendar.firstDay;
            else this._firstdayofweek = this._firstdayofweek;
            this.model.workWeek = this.model.workWeek.toString().toLowerCase().split(",");
            this.model.width = this.model.width ? this.element.width(this._clientWidth).width() : this.element.width();
            this.model.height = this.model.height ? this.element.height(this.model.height).height() : this.element.height();
            this.scWidth = this._scWidth ? parseInt(this._scWidth) : parseInt(this.model.width);
            this.scHeight = this._scHeight ? parseInt(this._scHeight) : parseInt(this.model.height);
            (this.currentDate() < this.model.minDate) && this.currentDate(this.model.minDate);
            (this.currentDate() > this.model.maxDate) && this.currentDate(this.model.maxDate);
            this.element.addClass("e-scheduleouter").height(this.model.height).width(this._clientWidth).attr("role", "presentation");
            $(this.element).attr("tabIndex", 1);
            var $scheduleDiv = ej.buildTag('div.e-scheduleinner', "", {}, { height: this.scHeight, width: this.scWidth }).css("height", this.scHeight - 2 + "px").css("position","relative");
            var $scheduleTable = ej.buildTag('table.e-table', "", {}, { cellpadding: "0px", cellspacing: "0px" }).addClass("e-scheduleoutertable");
            var $tBody = ej.buildTag('tbody');
            var hTR = ej.buildTag("tr.e-scheduleheader");
            this.aTR = ej.buildTag("tr.e-headerarearow");
            this.model.views.toString().toLowerCase().split(',').indexOf(this.currentView()) == -1 && this.currentView(this.model.views[0].toLowerCase());
            this._calculateCustomDays()._getRenderDates();
            this.wTR = ej.buildTag("tr.e-contentarearow");
            this._renderTemplates();
            this.model.showHeaderBar && this._renderHeaderBar(hTR);
            this._renderHeaderAllDayTemplate();
            this._renderContentAreaTemplate();
            $scheduleDiv.append($scheduleTable.append($tBody.append(hTR).append(this.aTR).append(this.wTR)));
            this.element.append($scheduleDiv);
            !this.model.showHeaderBar && this.element.find(".e-scheduleheader").hide();
            if (this.model.showHeaderBar) {
                this.element.find(".e-commonviewbutton.e-" + this.currentView()).addClass("e-activeview");
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ startDay: this._firstdayofweek, enableRTL: this.model.enableRTL, buttonText: this._getLocalizedLabels("Today"), select: $.proxy(this._onselect, this), locale: this.model.locale, cssClass: this.model.cssClass, minDate: this.model.minDate, maxDate: this.model.maxDate });
                this.element.find("span.e-datewidget").css("display", "none");
            }
            if (this.currentView() != "agenda") {
                if (this.model.orientation == "vertical") {
                    this._headerbarWidth(); (this.currentView() === "month" || (this._isCustomView())) && this._renderMonthTimeCells();
                    if (!this.model.showTimeScale) {
                        this.element.find(".e-headercells").first("td").css("border-left", "none");
                        this.element.find(".e-workcellstab").find("tr").find("td:first-child").css("border-left", "none");
                    }
                    if (!this.model.showAllDayRow && this.model.cellWidth) {
                        this.element.find(".e-workcellstab tr td:first-child").width(this.element.find(".e-headercells").width());
                    }
                }
                this.model.enableRTL && this.element.addClass("e-rtl");
                (this.model.showTimeScale) && this._borderAddRemove();
                this.model.enableRTL && this.element.removeClass("e-rtl");
                this.element.find(".e-headercells").addClass("e-dateheaderselect");
                this.element.find(".e-schedulesettings").html(this._getLocalizedLabels("AllDay"));
                (this.element.find('.e-workcells').width() < this.element.find('.e-dateheadercell').width() && this.model.orientation == "horizontal" && this.model.cellWidth == "") ? this.element.find('.e-workcells').width(this.currentView() == "month" || (this._isCustomView()) ? this.element.find('.e-horizontmonthheaderdaytd').width() : this.element.find('.e-horizontaltimetd ').width()) : "";
                this._renderScroller();
                this.model.orientation == "vertical" ? this.aTR.find(".e-headeralldaydiv").parent().next().append(this.element.find(".e-vscroll").length > 0 ? this._emptyCell() : "") : "";
                if (this._mediaQuery && this.model.cellWidth && this.currentView() == "day" && (parseInt(this.element.find(".e-headertr td").siblings("td:last-child").css("width")) > this.element.find(".e-vscroll").width())) {
                    this.element.find(".e-headertr td").siblings("td:last-child").css("width", this.element.find(".e-vscroll").width() + "px");
                }
                this.currentView() == "month" || (this._isCustomView()) ? this.element.find(".e-timecells").height($(this.element.find(".e-monthcells")).height()) : this.element.find(".e-ampmdisplay").height($(this.element.find(".e-workcells")).height());
                this._highlightCurrentTime(this.model.showCurrentTimeIndicator);
                this.model.enableRTL && this.element.addClass("e-rtl");
                if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
                    this._moveScroller();
                    if (!this.model.showTimeScale) {
                        //this.element.find(".e-horizontaldaterender").css("border-left", "none");
                        this.element.find(".e-headercells").removeClass("e-dateheaderselect");
                    }
                }
                (this.model.workHours.highlight) ? this._highlightBusinessHours() : this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                (this._dateRender.length > 60 || ($.isNumeric(this.element.find(".e-monthcells").height()) && (this.element.find(".e-monthcells").height() != $(this.element.find(".e-timecells")[3]).height()) && this.model.orientation == "vertical")) ? this.element.find('td.e-timecells .e-timecelldivs').addClass('e-timedivcells') : "";
            }
            this._validateMinMaxDate();
            this._renderAppointmentWindow();
            (this._mediaQuery || this.element.width() < 600) && this._renderNavigationDrawer();
            this._renderRecurEditWindow();
            this.model.showQuickWindow && (this._renderQuickAppWindow() || this._renderQuickAppDetailsView());
            this._showContextMenu(this.model.contextMenuSettings.enable);
            (this.model.enableRecurrenceValidation) && this._renderAlertWindow();
            var scheduleResponsive = $("#" + this._id + "AppointmentAddEditWindow_wrapper,#" + this._id + "RecurrenceEdit_wrapper,#" + this._id + "navDrawer,#" + this._id + "alertWindow_wrapper");
            (this._mediaQuery) ? scheduleResponsive.addClass("e-scheduleresponsive") : scheduleResponsive.removeClass("e-scheduleresponsive");
            (this.model.orientation == "horizontal" && this._browserInfo.name == "webkit") && this.element.find(".e-workcells.e-dayend").width(this.element.find(".e-workcells.e-dayend").outerWidth() + 1);
            if (this.currentView() != "agenda" && this.model.orientation == "horizontal" && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources))) {
                var border = this.model.enableRTL ? 'border-right' : 'border-left';
                this.element.find(".e-horizontaldaterender").css(border, '0px');
                this.element.find(".e-workareadiv").parent().css(border, '0px');
            }
            if (this._mediaQuery)
                this._phoneChanges();
            if (this.model.enableRTL && ej.browserInfo().name == "webkit")
                this._appointmentAddWindow.find(".e-chk-image").css("margin-right", "-13px");
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9 && this.model.orientation == "vertical") {
            this.element.find(".e-workleftindent").find("tr").last().find("td").addClass("e-lastcell");
            this.element.find(".e-workcellstab").find("tr").last().find("td").addClass("e-lastcell");
            }
        },

        _borderAddRemove: function () {
            if (!this.model.enableRTL) {
                this.element.find(".e-headrealldaytable tr td:first-child,.e-workcellstab tr td:first-child,.e-headerdays table tr td:first-child").css("border-left", "none");
                this.element.find(".e-horizontalheader,.e-horizontalrestd,.e-monthleftindent,.e-workleftindent,.e-empty").addClass("e-borderright");
            }
            else {
                this.element.find(".e-headrealldaytable tr td:first-child,.e-workcellstab tr td:first-child,.e-headerdays table tr td:first-child").css("border-right", "none");
                this.element.find(".e-horizontalheader,.e-horizontalrestd,.e-monthleftindent,.e-workleftindent,.e-empty").addClass("e-borderleft");
            }
            if (this.model.orientation == "horizontal") {
                if (this.currentView() == "month")
                    if (this._browserInfo.name != "webkit" || this.model.cellWidth == "")
                        this.element.find(".e-headerdays table tr td:first-child").width(this.element.find(".e-workcellstab tr td:first-child").width());
                    else
                        this.element.find(".e-workcellstab tr td:not(:first-child)").width(this.element.find(".e-horizontmonthheaderdaytd").first().width() + 1);
                else if (this._browserInfo.name == "webkit")
                    this.element.find(".e-workcellstab tr td:first-child").width(this.element.find(".e-horizontaltimetd").first().width());
            }
            else {
                if (this.model.cellWidth != "")
                    this.element.find(".e-workcellstab tr td:first-child").width(this.element.find(".e-headercells").first().width());
                this._browserInfo.name == "webkit" && (this.currentView() == "month" ? this.element.find(".e-monthleftindent").width(this.element.find(".e-monthempty").width() + 1) : this.element.find(".e-workleftindent").attr('style', "width:" + (this.element.find(".e-empty").width() + 1) + "px !important"));
            }
        },
        _phoneChanges: function () {
            this._appointmentAddWindow.ejDialog({
                allowDraggable: false,
                position: { X: "0", Y: "0" }
            });
        },
        _dateConvert: function (dtstr) {
            if (!ej.isNullOrUndefined(dtstr)) {
                var convdt = new Date(parseInt(dtstr.match(/\d+/).toString()));
                convdt = $.type(convdt) == "date" ? convdt : new Date();
            }
            else
                convdt = null;
            return convdt;
        },
        _calculateCustomDays: function () {
            if (!ej.isNullOrUndefined(this.model.renderDates) && this.currentView() == "customview" && !this._navigate) {
                var startdate = $.type(this.model.renderDates.start) != "date" ? Globalize.parseDate(this.model.renderDates.start, this._datepattern()) : new Date(this.model.renderDates.start);
                startdate = (!ej.isNullOrUndefined(startdate)) ? startdate : ((new Date(this.model.renderDates.start).toString() == "Invalid Date" || new Date(this.model.renderDates.start).toString() == "NaN") ? this._dateConvert(this.model.renderDates.start) : new Date(this.model.renderDates.start));
                var enddate = $.type(this.model.renderDates.end) != "date" ? Globalize.parseDate(this.model.renderDates.end, this._datepattern()) : new Date(this.model.renderDates.end);
                enddate = (!ej.isNullOrUndefined(enddate)) ? enddate : ((new Date(this.model.renderDates.end).toString() == "Invalid Date" || new Date(this.model.renderDates.end).toString() == "NaN") ? this._dateConvert(this.model.renderDates.end) : new Date(this.model.renderDates.end));;
                if (!ej.isNullOrUndefined(this.model.minDate) && !ej.isNullOrUndefined(this.model.maxDate)) {
                    startdate = (this.model.minDate >= startdate && this.model.maxDate <= startdate) ? startdate : (this.model.minDate >= startdate && this.model.maxDate >= startdate) ? this.model.minDate : (this.model.maxDate <= startdate) ? this.model.minDate : startdate;
                    enddate = (this.model.minDate >= enddate && this.model.maxDate <= enddate) ? enddate : (this.model.minDate <= enddate && this.model.maxDate <= enddate) ? this.model.maxDate : (this.model.minDate >= enddate) ? this.model.maxDate : enddate;
                }
                this._renderStart = !ej.isNullOrUndefined(startdate) ? new Date(startdate.getFullYear(), startdate.getMonth(), startdate.getDate()) : enddate;
                this._renderEnd = !ej.isNullOrUndefined(enddate) ? new Date(enddate.getFullYear(), enddate.getMonth(), enddate.getDate()) : startdate;
                if ((this._renderStart.getTime() < this.currentDate().getTime()) && (this.currentDate().getTime() < this._renderEnd.getTime()) && (Math.round((this.model.renderDates.end - this.model.renderDates.start) / 86400000) + 1) <= 7)
                    this.currentDate(new Date(this.currentDate()));
                else
                    this.currentDate(new Date(this._renderStart));
                this._renderDays = Math.round((this._renderEnd - this._renderStart) / 86400000) + 1;
                if (this._renderDays > 7) {
                    var firstDay = this.model.orientation == "vertical" ? new Date(this._firstweekdate(this._renderStart)) : new Date(this._renderStart);
                    var endDay = new Date(this._renderEnd); var day = endDay.getDay();
                    (this._firstdayofweek == 0 && day != 6) ? endDay.setDate(new Date(this._renderEnd).getDate() + (6 - day)) : endDay.setDate(new Date(this._renderEnd).getDate() + (6 - day) + this._firstdayofweek);
                    endDay = this.model.orientation == "vertical" ? endDay : new Date(this._renderEnd);
                    this._numDays = Math.round((endDay - firstDay) / 86400000) + 1;
                }
            }
            return this;
        },

        _isCustomView: function () {
            if (this.currentView() == "customview" && this._dateRender.length > 7) {
                this._oneWeek = this._dateRender.length > 7 ? false : true;
                return true;
            }
            else
                if (this.currentView() == "customview") this._oneWeek = this._dateRender.length > 7 ? false : true;
        },

        _headerbarWidth: function () {
            var cols = this.currentView() == "workweek" ? this.model.workWeek.length : this.currentView() == "day" ? 1 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : 7;
            var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
            var border = this.element.outerWidth() - (this.element.find('.e-draggableworkarea').offset().left - this.element.offset().left) < parseInt(this.model.cellWidth) * col ? 1 : 0;
            if (this._browserInfo.name == "webkit")
                this.element.find(".e-headrealldaytable").width(this.model.cellWidth ? (parseInt(this.model.cellWidth) * col + (border ? 0 : col)) : "100%");
            else
                this.element.find(".e-headrealldaytable").width(this.model.cellWidth ? (parseInt(this.model.cellWidth) * col) + col : "100%");
        },

        _renderScroller: function () {
            var _scrollDiv = $('div#' + this._id + "_scroller");
            var hghtdiff = this.scHeight - (this.element.find('.e-draggableworkarea').offset().top - this.element.offset().top);
            var proxy = this; var wdthdiff = (this.currentView() === "month") ? 0 : 4;
            //var scrollWidth = (this.model.orientation === "vertical" && !this.model.enableRTL) ? this.element.outerWidth() - (this.element.find('.e-draggableworkarea').offset().left - this.element.offset().left) : (this.model.orientation == "horizontal") && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.element.width() - this.element.find(".e-resourceheadertable").width() : (this.model.enableRTL) ? (this._mediaQuery && this.currentView() == "month") ? this.element.width() : this.element.width() - this.element.find('.e-scrolltimecells').parent().width() : this.element.width();
            var scrollWidth = (this.model.orientation === "vertical") ? this.element.width() - (this._mediaQuery && this.currentView() == "month" ? "" : this.element.find('.e-scrolltimecells').parent().outerWidth()) : (this.model.orientation == "horizontal") && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.element.width() - this.element.find(".e-resourceheadertable").width() : this.element.width();
            _scrollDiv.ejScroller({
                enableRTL: this.model.enableRTL, height: (hghtdiff - 1), width: scrollWidth - 2, cssClass: this.model.cssClass,
                thumbStart: function (e) {
                    var target = $(e.originalEvent.target);
                    if ((!proxy._mediaQuery) && target.hasClass("e-workcells") || target.parents(".e-appointment").hasClass("e-appointment") || target.parents(".e-monthappointment").hasClass("e-monthappointment") || target.hasClass("e-categorybar")) return false;
                },
                scroll: function (e) {
                    var scrollerObj = $('div#' + proxy._id + "_scroller").data("ejScroller");
                    if ((scrollerObj.isVScroll()) || (scrollerObj.isHScroll())) {
                        (proxy.model.orientation === "vertical") ? proxy.element.find(".e-scrolltimecells").css("top", (-e.scrollTop) + "px") : (proxy.currentView() !== "month" && !(proxy._isCustomView())) ? $(proxy.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", (-e.scrollLeft) + "px") : proxy.element.find(".e-headerdays").css("left", (-e.scrollLeft) + "px");
                        if (proxy.element.find('.e-vhandle').length == 1)
                            proxy.element.find(".e-horires").css("top", (-e.scrollTop) + "px");
                        if (proxy.model.orientation == "vertical") {
                            proxy.element.find(".e-headrealldaytable").css("left", (-e.scrollLeft) + "px");
                            if (proxy._timeMode === "12" && (proxy.currentView() !== "month" && !(proxy._isCustomView()))) {
                                var top = (e.scrollTop / (proxy.element.find(".e-workcells").height() * 2));
                                top = (Math.round(top) < top) ? Math.round(top) + 1 : Math.round(top);
                                var localizeAmPm = Globalize.culture(proxy.model.locale).calendar;
                                if (localizeAmPm.AM && localizeAmPm.PM) {
                                    proxy.element.find('.e-ampmstyle').each(function (i, val) {
                                        if (i == (top))
                                            (parseInt($(val)[0].id) < 12) ? $(val).text(localizeAmPm.AM[0]) : $(val).text(localizeAmPm.PM[0]);
                                        else if (($(val).text() == localizeAmPm.AM[0] && i > 0) || ($(val).text() == localizeAmPm.PM[0] && parseInt($(val)[0].id) > 12))
                                            $(val).text("00");
                                    });
                                }
                            }
                        }
                        else if (ej.scheduleFeatures.horizontal && proxy.model.orientation == "horizontal")
                            proxy._horizontalScroll(e, proxy);
                    }
                    if (proxy.model.showQuickWindow) {
                        (proxy._quickAppointWindow.ejDialog("close") && proxy._quickAppDetailsWindow.ejDialog("close"));
                    }
                }
            });
            (!this._mediaQuery) && this._nextPrevAppointment(this.model.enableAppointmentNavigation);
            this._renderHoriVerticalScroll();
            this.element.find(".e-vscroll").css("left", (this.model.enableRTL) ? 0 : this.element.outerWidth() - (_scrollDiv.ejScroller("isVScroll") ? 18 : 0));
            this.element.find(".e-mobileNavigation").css("left", (this.model.enableRTL) ? (_scrollDiv.ejScroller("isVScroll") ? 18 : 0) : this.element.outerWidth() - 36);
            if (this.model.orientation == "horizontal" && this.element.find(".e-vscroll").length > 0) {
                this.element.find(".e-scheduleheader td").first().attr("colspan", "3");
                this.element.find(".e-workareadiv").parent().attr("colspan", "2");
                var emptytd = this.element.find(".e-horizontaldaterender").parent();
                $(emptytd).append(ej.buildTag('td', "", {}, { width: (this.currentView() == "month" || this._isCustomView()) ? "18px" : (this._mediaQuery) ? "0px" : "18px" }).append(this._emptyCell()));
            }
        },
        _renderHoriVerticalScroll: function () {
            var _scrollDiv = $('div#' + this._id + "_scroller");
            var wdthdiff = (this.currentView() === "month") ? 0 : 4;
            var scrollWidth = (this.model.orientation === "vertical") ? this.element.width() - (this._mediaQuery && this.currentView() == "month" ? "" : this.element.find('.e-scrolltimecells').parent().outerWidth()) : (this.model.orientation == "horizontal") && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.element.width() - this.element.find(".e-resourceheadertable").width() : this.element.width();
            scrollWidth = scrollWidth - 1;
            var hghtdiff = this.scHeight - (this.element.find('.e-draggableworkarea').offset().top - this.element.offset().top);
            var cols = this.currentView() == "workweek" ? this.model.workWeek.length : this.currentView() == "day" ? 1 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : 7;
            var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
            if (this.model.orientation == "vertical") {
                _scrollDiv.ejScroller("isVScroll") ? this.element.find(".e-emptytd,.e-emptyalldaytd").show() : this.element.find(".e-emptytd,.e-emptyalldaytd").hide();
                if (this.model.cellWidth || this.model.cellHeight != "20px") {
                    (this.currentView() == "day") && this.element.find(".e-headerdaydisplay").css("width", this.model.cellWidth);
                    var rows = this._dateRender.length / 7;
                    var overallWidth = (parseInt(this.model.cellWidth) + 1) * col;
                    var overallHeight = parseInt(this.model.cellHeight) * rows;
                    var dragWidth = this.scWidth - this.element.find(".e-scrolltimecells").width() + wdthdiff;
                    var dragHeight = hghtdiff;
                    if (this.currentView() == "month" || (this._isCustomView())) {
                        this.element.find(".e-draggableworkarea").width((this.element.width() > overallWidth + this.element.find(".e-scrolltimecells").width() + 18) ? overallWidth - 1 : _scrollDiv.ejScroller("isVScroll") ? scrollWidth - 18 - 1 : scrollWidth - 1);
                        (this.model.cellHeight != "20px") ? this.element.find(".e-draggableworkarea").height(dragHeight > overallHeight ? overallHeight : _scrollDiv.ejScroller("isHScroll") ? dragHeight - 18 - 1 : dragHeight - 1) : this.element.find(".e-draggableworkarea").height(_scrollDiv.ejScroller("isHScroll") ? dragHeight - 18 - 1 : dragHeight - 1);
                        this.element.find(".e-emptytd").show();
                    }
                    else {
                        this.element.find(".e-draggableworkarea").width((this.element.width() > overallWidth + this.element.find(".e-scrolltimecells").width() + (_scrollDiv.ejScroller("isVScroll") ? 18 : 0)) ? overallWidth - 1 : scrollWidth - 1 - (_scrollDiv.ejScroller("isVScroll") ? 18 : 0));
                        if (!this.model.showTimeScale)
                            (this.model.cellHeight == "20px") ? this.element.find(".e-draggableworkarea").height((_scrollDiv.ejScroller("isHScroll")) ? dragHeight - 1 - (_scrollDiv.ejScroller("isHScroll") ? 18 : 0) : dragHeight - 1) : this.element.find(".e-draggableworkarea").height(parseInt(this.model.cellHeight));
                    }
                    (this.model.cellWidth) && this.element.find(".e-datecommondiv").parent().width(this.element.width() - 2);
                    this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());
                }
                else {
                    this.element.find(".e-draggableworkarea").height(hghtdiff - 1);
                    if (_scrollDiv.ejScroller("isVScroll") && !this._mediaQuery)
                        this.element.find(".e-draggableworkarea").width(scrollWidth - 1 - (_scrollDiv.ejScroller("isVScroll") ? 18 : 0));
                    else if ((this.currentView() == "month" || (this._isCustomView())) || !this.model.showTimeScale) {
                        this.element.find(".e-draggableworkarea").width(scrollWidth - 1);
                        this.element.find(".e-draggableworkarea").height(hghtdiff);
                    }
                    else if (this._mediaQuery && this.res1.length > 1)
                        this.element.find(".e-draggableworkarea").width(this.element.find(".e-draggableworkarea").width() * this.res1.length);
                    this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());
                    this.element.find(".e-scrolltimecells").height(this.element.find('.e-draggableworkarea').height());
                }
            }
            else {
                !(_scrollDiv.ejScroller("isHScroll")) ? this.element.find(".e-draggableworkarea").css("height", "100%") && this.element.find(".e-resfootertr").css("height", "0px") : "";
                !(_scrollDiv.ejScroller("isVScroll")) && !(_scrollDiv.ejScroller("isHScroll")) ? "" : !(_scrollDiv.ejScroller("isVScroll")) ? this.element.find(".e-draggableworkarea").css("height", hghtdiff - 18 - 1) : "";
                if ((this.currentView() == "month") && !(_scrollDiv.ejScroller("isHScroll")) && !(_scrollDiv.ejScroller("isVScroll")))
                    this.element.find(".e-draggableworkarea").css("height", "100%");
                else if (this.model.showTimeScale && this.currentView() == "customview" && this._dateRender.length > 7) {
                    this.element.find(".e-headercells").width(this.element.find(".e-workcells").width());
                    this.element.find(".e-horizontmonthheaderdaytd").width(this.element.find(".e-workcells").width());
                }
                this.element.find(".e-horizontresdiv").height(this.element.find('.e-draggableworkarea').height());
                if (this.model.cellWidth || this.model.cellHeight != "20px") {
                    var count = $(this.element.find(".e-workcells")[0]).parent().find("td").length;
                    !(_scrollDiv.ejScroller("isHScroll")) ? this.element.find(".e-draggableworkarea").width(parseInt(this.model.cellWidth) * count) : "";
                    (this.currentView() == "month") && this.element.find(".e-headerdays").children().css("width", (this.element.find(".e-draggableworkarea").children().width() + "px"));
                    (this.model.cellHeight != "20px" && !this.model.resources) && this.element.find('.e-draggableworkarea').height(parseInt(this.model.cellHeight) - 1);
                }
                else {
                    if (!(_scrollDiv.ejScroller("isHScroll"))) {
                        this.element.find('.e-horizontaltabletimecell').width(this.element.find('.e-workcellstab').width());
                        if (this.element.find(".e-horizontmonthheaderdaytd").length == 0) {
                            var timearray = this.element.find(".e-horizontaltimetd");
                            var alternateArray = this.element.find(".e-horizontalalternatetd");
                            for (var i = 0; i < this.element.find('.e-childworkcell').length / 2; i++) {
                                $(timearray[i]).width($(this.element.find(".e-workcells")[(i * 2)]).width());
                                if (this.element.find('.e-childworkcell').length == (i * 2) + 2)
                                    $(alternateArray[i]).width($(this.element.find(".e-workcells")[(i * 2) + 1]).width() - 2);
                                else
                                    $(alternateArray[i]).width($(this.element.find(".e-workcells")[(i * 2) + 1]).width());
                            }
                        }
                        else {
                            var timearray = this.element.find(".e-horizontmonthheaderdaytd");
                            for (var i = 0; i < $(this.element.find('.e-workcellstab').find("tr")[0]).find(".e-workcells").length - 1; i++) {
                                $(timearray[i]).width($(this.element.find(".e-workcells")[i]).width());
                            }
                        }
                    }
                }
                if (!this.model.showTimeScale && this.currentView() != "month" && !this._isCustomView()) {
                    if (!this.model.cellWidth && this._browserInfo.name != "webkit") {
                        this.element.find(".e-dateheadercell").width(this.element.find(".e-workcells").width());
                        this.element.find(".e-workcells").width(this.element.find(".e-workcells").width());
                    }
                    else if (this._browserInfo.name == "webkit" && this.currentView() != "month") {
                        (this.model.cellWidth) && this.element.find(".e-headerdays").width(this.model.cellWidth);
                        this.element.find(".e-workcells").width(this.element.find(".e-draggableworkarea").width / this._dateRender.length);
                        this.element.find(".e-dateheadercell").width(this.element.find(".e-workcells").width());
                        this.element.find(".e-headerdays table tr td:not(:first-child)").find(".e-dateheadercell").width(this.element.find(".e-workcells").width() - 1);
                        this.element.find(".e-workcells").width(this.element.find(".e-workcells").width());
                    }
                    else
                        this.element.find(".e-headerdays").width(this.model.cellWidth);
                }
            }
        },

        _highlightBusinessHours: function () {
            if (this.currentView() == "agenda") return false;
            if (this.currentView() !== "month" && !(this._isCustomView()) && this.model.showTimeScale) {
                this.element.find(".e-workcells").removeClass("e-businesshighlightworkcells");
                var renderDates = this.dateRender;
                var target = this.element.find(".e-workcells");
                var b_startHour = this.model.workHours.start >= this.model.startHour ? this.model.workHours.start - this.model.startHour : 0;
                var b_endHour = this.model.workHours.end <= this.model.endHour ? this.model.workHours.end - this.model.startHour : this.model.endHour - this.model.startHour;
                var timeRowsCount = 60 / 30;
                this.res1 = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.res1 : [0];
                var colCount = this.currentView() === "week" ? 7 * this.res1.length : this.currentView() === "workweek" ? this.model.workWeek.length * this.res1.length : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length * this.res1.length : 1 * this.res1.length;
                if (this.model.orientation == "vertical") {
                    for (var i = (b_startHour * timeRowsCount) ; i < (b_endHour * timeRowsCount) ; i++) {
                        for (var j = 0; j < colCount; j++) {
                            for(var k = 0; k < this.model.workWeek.length; k++) {
                            if(this._dayNamesArray.indexOf(this.model.workWeek[k]) == new Date(renderDates[j]).getDay())
                                    $(target[(i * colCount) + j]).addClass("e-businesshighlightworkcells");
                            }
                        }
                    }
                    this._businessHourScroller();
                }
                else
                    this._horizontalBusinessHighlight(this._dateRender, b_startHour, b_endHour, timeRowsCount, target);
            }
            else {
                var target, monthDays;
                if (this.model.orientation == "vertical") {
                    monthDays = this.dateRender;
                    target = this.element.find(".e-monthcells");
                    target.removeClass("e-monthcellhighlight");
                    for (var i = 0; i < monthDays.length; i++) {
                        for(var j = 0; j < this.model.workWeek.length; j++) {
                        if(this._dayNamesArray.indexOf(this.model.workWeek[j]) == new Date(monthDays[i]).getDay())
                                $(target[i]).addClass("e-monthcellhighlight");
                        }
                    }
                }
                else
                    this._horizontalBusinessHighlight(null, null, null, null, null);
            }
        },

        _onselect: function (e) {
            this._navigateClick(e, "DateNavigation");            
        },

        _renderHeaderBar: function (hTR) {
            var colsVal = this.model.orientation == "vertical" ? 1 : 2;
            this.hTD = ej.buildTag('td', "", {}, { colspan: colsVal });
            var commonDiv = ej.buildTag('div.e-datecommondiv').append(ej.buildTag('div.e-icon e-navigateprevious', "", {}, { title: this._getLocalizedLabels("Previous") })).append(ej.buildTag('div', "", {}, { "aria-live": "assertive", tabindex: 0 }).html(this._getCurrentDateHeaderValue()).addClass("e-datedisplay")).append(ej.buildTag("input#" + this._id + "_navigator").css("display", "none").addClass("dateCalendar")).append(ej.buildTag('div.e-icon e-navigatenext', "", {}, { title: this._getLocalizedLabels("Next") }));
            if (this._mediaQuery || this.element.width() < 600) {
                this._mediaQuery && (commonDiv = ej.buildTag('div.e-datecommondiv').append(ej.buildTag('div.e-icon e-navigateprevious', "", {}, { title: this._getLocalizedLabels("Previous") })).append(ej.buildTag('div.e-icon e-navigatenext', "", {}, { title: this._getLocalizedLabels("Next") }).css("margin-left", "5px")).append(ej.buildTag('div', "", {}, { "aria-live": "assertive", tabindex: 0 }).html(this._getCurrentDateHeaderValue()).addClass("e-datedisplay").css("font-size", "18px")).append(ej.buildTag("input#" + this._id + "_navigator").css("display", "none").addClass("dateCalendar")));
                var menuIcon = ej.buildTag('div#' + this._id + 'navDrawerIcon.e-mobileNavigation e-icon');
                hTR.append(this.hTD.append(ej.buildTag("div").css("width", "100%").append(commonDiv).append(menuIcon)));
            }
            else {
                var buttonsDiv = ej.buildTag('div.e-viewsdiv');
                hTR.append(this.hTD.append(ej.buildTag("div").css("width", "100%").append(commonDiv).append(buttonsDiv.append(this._renderViewNavigation(this.model.views)))));
            }
        },

        _renderViewNavigation: function (views) {
            var $viewTable = ej.buildTag('table.e-table e-viewstable', "", {}, { cellspacing: '0px', cellpadding: '0px' });
            var $viewBody = ej.buildTag('tbody');
            var $viewTR = ej.buildTag('tr');
            var _viewCollection = views, viewClassname;
            for (var i = 0; i < _viewCollection.length; i++) {
                viewClassname = (_viewCollection.length == 1) ? "e-" + _viewCollection[i].toLowerCase() + " e-singleview" : (i == 0) ? "e-" + _viewCollection[i].toLowerCase() + " e-firstview" : i == (_viewCollection.length - 1) ? "e-" + _viewCollection[i].toLowerCase() + " e-lastview" : "e-" + _viewCollection[i].toLowerCase();
                switch (_viewCollection[i]) {
                    case "day": $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels("Day"), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
                        break;
                    case "week": $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels("Week"), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
                        break;
                    case "workweek": $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels("WorkWeek"), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
                        break;
                    case "month": $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels("Month"), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
                        break;
                    case "customview": $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels("CustomView"), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
                        break;
                    case "agenda": $viewTR.append(ej.buildTag('td').append(ej.buildTag('div', this._getLocalizedLabels("Agenda"), {}, {}).addClass("e-commonviewbutton").addClass(viewClassname)));
                        break;
                }
            }
            $viewTable.append($viewBody.append($viewTR))
            return $viewTable;
        },

        _emptyCell: function () {
            var emptyCell;
            var scrollDiv = $('div#' + this._id + "_scroller"); var Resources = "";
            if (!ej.isNullOrUndefined(this.model.group)) {
                for (var i = 0; i < this.model.group.resources.length - 1; i++)
                    Resources = Resources + "<tr><td class='e-emptytd e-headerresourcecells' style='height: 27px'></td></tr>";
            }
            if (this.currentView() != "month" && (!this._isCustomView()) && this.model.orientation == "vertical") {
                emptyCell = "<table class='e-table' cellspacing='0' cellpadding='0'><tbody>";
                if (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources))
                    emptyCell += "<tr><td class='e-emptytd e-headerresourcecells' style='height: 27px'></td></tr>";
                if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1)
                    emptyCell += Resources;
                emptyCell += "<tr><td class='e-emptytd' style='height:29px'></td></tr>";
                if (this.model.showAllDayRow)
                    emptyCell += "<tr><td class='e-emptyalldaytd' style='height:27px'></td></tr>";
                emptyCell += "</tbody></table>";
            }
            else if ((this.currentView() == "month" || this._isCustomView()) && scrollDiv.ejScroller("isVScroll") && this.model.orientation == "vertical")
                emptyCell = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? Resources + "<tr><td class='e-emptytd e-headerresourcecells' style='height: 27px'></td></tr>" : "" + "<tr><td class='e-emptytd' style='height:29px'></td></tr>";
            else if (this.model.orientation == "horizontal" && scrollDiv.ejScroller("isVScroll")) {
                if (this.currentView() == "month" && scrollDiv.ejScroller("isVScroll")) {
                    emptyCell += "<tr><td class='e-emptytd e-headerresourcecells' style='height:30px'></td></tr>";
                    emptyCell += "<tr><td class='e-emptytd' style='height:29px'></td></tr>";
                }
                if (this.currentView() != "month" && scrollDiv.ejScroller("isVScroll")) {
                    emptyCell += "<tr><td class='e-emptytd e-headerresourcecells' style='height:30px'></td></tr>";
                    emptyCell += "<tr><td class='e-emptytd' style='height:25px'></td></tr>";
                }
                emptyCell += "</tbody></table>";
            }
            return emptyCell;

        },

        _recurrenceWindow: function () {
            var recurtype = ["Daily", "Weekly", "Monthly", "Yearly", "EveryWeekDay"];
            var culture = Globalize.culture(this.model.locale);
            var weekshort = this._dayShortNames;
            var weekfull = this._dayFullNames;
            var months = this._monthNames;
            var startday = ["First", "Second", "Third", "Fourth", "Last"];
            var margin = this.model.enableRTL ? "margin-right" : "margin-left";
            if (this._mediaQuery) {
                var $recurWin = "<div class='e-recurwindow e-windowmargin' style='display:none'>" +
                            "<form id='" + this._id + "_RecurForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
                $recurWin += "<tr><td width='16%' class='e-textlabel'>" + this._getLocalizedLabels("Recurrence") + ":</td>" +
                            "<td><input class='recurtype' id='recurtype' type='text' name='RecurrenceType' value='" + this._getLocalizedLabels(recurtype[0]) + "' /><div id='recurtypelist'><ul>";
                for (var type = 0; type < recurtype.length; type++) {
                    $recurWin += "<li>" + this._getLocalizedLabels(recurtype[type]) + "</li>";
                }
                $recurWin += "</ul></div></td></tr>";
                $recurWin += "<tr id='" + this._id + "repeatEvery'><td class='e-textlabel'>" + this._getLocalizedLabels("Every") + ":</td><td class='e-tdpadding'><table><tr><td><div class='e-floatleft'><input id='recurevery' class='e-recurevery' type='text' /></div></td>" +
                            "<td><div id='" + this._id + "recurtypes' class='e-appcheckbox'>" + this._getLocalizedLabels("RecurrenceDay") + "</div></td></tr></table></td></tr>";

                $recurWin += "<tr id='" + this._id + "weeklyrecur' class='" + this._id + "weeklyrecur' style='display:none'><td colspan='2' id='recurweek' class='e-textlabel'><div>" + this._getLocalizedLabels("RepeatOn") + ":</div></td></tr><tr class='" + this._id + "weeklyrecur'>" +
                            "<td colspan='2' id='recurweekcount'><table class='e-table' cellpadding='3'><tr>";
                for (var weekcount = 0; weekcount < weekshort.length; weekcount++) {
                    $recurWin += "<td><div class='e-weekday e-_" + weekshort[weekcount] + "'><input id='" + this._id + "_" + this._dayNames[weekcount] + "' class='weekdays e-weekly" + this._dayNames[weekcount] + "'name='weekdays' type='checkbox'/><label class='e-textmargin' for='" + this._id + "_" + this._dayNames[weekcount] + "'>" + weekshort[weekcount] + "</label></div></td>";
                }
                $recurWin += "</tr></table></td></tr>";
                $recurWin += "<tr id='" + this._id + "monthlyrecur' class='" + this._id + "monthlyrecur' style='display:none'><td colspan='2' id='recurmonthtext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td></tr><tr class='" + this._id + "monthlyrecur'>" +
                            "<td colspan='2' id='recurmonthcount'><table class='e-table' cellpadding='3'><tr><td><div><input id='" + this._id + "monthday' class='e-monthdaytype' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthday'>" + this._getLocalizedLabels("Day") + "</label></div></td>" +
                            "<td><div><input id='monthdate' class='e-monthdate' type='text'/></div></td></tr>" +
                            "<tr><td><div><input id='" + this._id + "monthon' class='e-monthposition' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthon'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                            "<td><div><input id='monthsrt' class='monthsrt' type='text' name='monthsrt' value=''/><div id='monthsrtlist'><ul>";
                for (var day = 0; day < startday.length; day++) {
                    $recurWin += "<li>" + this._getLocalizedLabels(startday[day]) + "</li>";
                }
                $recurWin += "</div></div></td>";
                $recurWin += "<td><div class='e-appcheckbox'><input id='monthsrtday' class='e-monthsrtday' type='text' name='monthsrtday' value=''/>" +
                    "<div id='monthsrtdaylist'><ul>";
                for (var monthday = 0; monthday < weekfull.length; monthday++) {
                    $recurWin += "<li>" + weekfull[monthday] + "</li>";
                }
                $recurWin += "</div></div></td></tr></table></td></tr>";
                $recurWin += "<tr id='" + this._id + "yearlyrecur' class='" + this._id + "yearlyrecur' style='display:none'><td colspan='2' id='recuryeartext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td></tr><tr class='" + this._id + "yearlyrecur'>" +
                            "<td colspan='2' id='recuryearcount' ><table class='e-table' cellpadding='3'><tr>" +
                            "<td><div><input id='" + this._id + "yearday' class='e-yearrecurday' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearday'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                            "<td><div class='e-controlalign'><input id='yearmonth' class='yearmonth' type='text' name='yearmonth' value=''/></div></td>" +
                            "<div id='yearmonthlist'><ul>";
                for (var monthcount = 0; monthcount < (months.length - 1) ; monthcount++) {
                    $recurWin += "<li>" + months[monthcount] + "</li>";
                }
                $recurWin += "</ul></div><td><input id='yeardate' class='yeardate' text='text'/></td></tr>" +
                        "<tr><td><div><input id='" + this._id + "yearother' class='e-yearrecurposi' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearother'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                        "<td><div><input id='yearsrt' class='yearsrt' type='text' name='yearsrt' value=''/><div id='yearsrtlist'><ul>";
                for (var yearstartday = 0; yearstartday < startday.length; yearstartday++) {
                    $recurWin += "<li>" + this._getLocalizedLabels(startday[yearstartday]) + "</li>";
                }
                $recurWin += "</ul></div></div></td><td><div class='e-controlalign'><input id='yearsrtday' class='yearsrtday' type='text' name='yearsrtday' value=''/>" +
                    "<div id='yearsrtdaylist'><ul>";
                for (var yearweek = 0; yearweek < weekfull.length; yearweek++) {
                    $recurWin += "<li>" + weekfull[yearweek] + "</li>";
                }
                $recurWin += "</ul></div></div></td><td><div><span>" + this._getLocalizedLabels("OfEvery").split(" ")[0] + "</span></div></td></tr><tr><td></td><td><div><input id='yearsrtmonth' class='yearsrtmonth' type='text' name='yearsrtmonth' value=''/>" +
                        "<div id='yearsrtmonthlist'><ul>";
                for (var yearmonth = 0; yearmonth < (months.length - 1) ; yearmonth++) {
                    $recurWin += "<li>" + months[yearmonth] + "</li>";
                }
                $recurWin += "</ul></div></div></td></tr></table></td></tr>";
                $recurWin += "<tr><td colspan='2' class='e-textlabel'>" + this._getLocalizedLabels("StartsOn") + ":</td></tr><tr>" +
                            "<td colspan='2'><input id='recurstartdate' class='recurstartdate' type='text' name='RecurStartDate' value=''/></td></tr>";
                $recurWin += "<tr><td colspan='2' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("Ends") + ":</div></td></tr><tr>" +
                            "<td colspan='2'><table class='e-table' cellpadding='3'><tr><td><div><input id='repeatendnever' class='recurends e-recurnoend' type='radio' name='repeatend' value='Never'/><label class='e-textmargin' for='repeatendnever'>" + this._getLocalizedLabels("Never") + "</label></div></td></tr>" +
                            "<tr><td><div><input id='" + this._id + "repeatendafter' class='recurends e-recurafter' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendafter'>" + this._getLocalizedLabels("After") + "</label>" +
                            "</div></td><td><div><input id='dailyoccurance' class='e-recurcount' type='text'/></div></td>" +
                            "<td><span style='"+ margin +": -80px;'>" + this._getLocalizedLabels("Occurence") + "</span></td></tr>" +
                            "<td><div><input id='" + this._id + "repeatendon' class='recurends e-recuruntil' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendon'>" + this._getLocalizedLabels("On") + "</label></div></td>" +
                            "<td><input id='daily' class='e-until' type='text' name='daily' value=''/></td></tr></table></td></tr>";
                $recurWin += "<tr style='display:none'><td><span class='e-textlabel'>" + this._getLocalizedLabels("Summary") + ":</span></td><td><span class=e-recurRule></span></td></tr></tbody></table></form>";
                $recurWin += "<div class='e-floatright'><table class='e-table' cellpadding='5'><tr><td><div class='e-btnmargin e-floatleft'><button id='donerecur' class='recurbutton'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                            "<td><div class='e-btnmargin e-floatleft'><button id='cancelrecur' class='e-cancelrecur recurbutton'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></tabled></div></div>";
                return $recurWin;
            }
            else {
                var $recurWin = "<div class='e-recurwindow e-windowmargin' style='display:none'>" +
                        "<form id='" + this._id + "_RecurForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
                $recurWin += "<tr><td width='16%' class='e-textlabel'>" + this._getLocalizedLabels("Recurrence") + ":</td>" +
                        "<td><input class='recurtype' id='recurtype' type='text' name='RecurrenceType' value='" + this._getLocalizedLabels(recurtype[0]) + "' /><div id='recurtypelist'><ul>";
                for (var type = 0; type < recurtype.length; type++) {
                    $recurWin += "<li>" + this._getLocalizedLabels(recurtype[type]) + "</li>";
                }
                $recurWin += "</ul></div></td></tr>";
                $recurWin += "<tr id='" + this._id + "repeatEvery'><td class='e-textlabel'>" + this._getLocalizedLabels("Every") + ":</td><td class='e-tdpadding'><table><tr><td><div class='e-floatleft'><input id='recurevery' class='e-recurevery' type='text' /></div></td>" +
                        "<td><div id='" + this._id + "recurtypes' class='e-appcheckbox'>" + this._getLocalizedLabels("RecurrenceDay") + "</div></td></tr></table></td></tr>";

                $recurWin += "<tr id='" + this._id + "weeklyrecur' class='" + this._id + "weeklyrecur' style='display:none'><td id='recurweek' class='e-textlabel'><div>" + this._getLocalizedLabels("RepeatOn") + ":</div></td>" +
                        "<td id='recurweekcount'><table class='e-table' cellpadding='3'><tr>";
                for (var weekcount = 0; weekcount < weekshort.length; weekcount++) {
                    $recurWin += "<td><div class='e-weekday e-_" + weekshort[weekcount] + "'><input id='" + this._id + "_" + this._dayNames[weekcount] + "' class='weekdays e-weekly" + this._dayNames[weekcount] + "'name='weekdays' type='checkbox'/><label class='e-textmargin' for='" + this._id + "_" + this._dayNames[weekcount] + "'>" + weekshort[weekcount] + "</label></div></td>";
                }
                $recurWin += "</tr></table></td></tr>";
                $recurWin += "<tr id='" + this._id + "monthlyrecur' class='" + this._id + "monthlyrecur' style='display:none'><td id='recurmonthtext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td>" +
                        "<td id='recurmonthcount'><table class='e-table' cellpadding='3'><tr><td><div><input id='" + this._id + "monthday' class='e-monthdaytype' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthday'>" + this._getLocalizedLabels("Day") + "</label></div></td>" +
                        "<td><div><input id='monthdate' class='e-monthdate' type='text'/></div></td></tr>" +
                        "<tr><td><div><input id='" + this._id + "monthon' class='e-monthposition' name='monthday' type='radio'/><label class='e-textmargin' for='" + this._id + "monthon'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                        "<td><div><input id='monthsrt' class='monthsrt' type='text' name='monthsrt' value=''/><div id='monthsrtlist'><ul>";
                for (var day = 0; day < startday.length; day++) {
                    $recurWin += "<li>" + this._getLocalizedLabels(startday[day]) + "</li>";
                }
                $recurWin += "</div></div></td>";
                $recurWin += "<td><div class='e-appcheckbox'><input id='monthsrtday' class='e-monthsrtday' type='text' name='monthsrtday' value=''/>" +
                    "<div id='monthsrtdaylist'><ul>";
                for (var monthday = 0; monthday < weekfull.length; monthday++) {
                    $recurWin += "<li>" + weekfull[monthday] + "</li>";
                }
                $recurWin += "</div></div></td></tr></table></td></tr>";
                $recurWin += "<tr id='" + this._id + "yearlyrecur' class='" + this._id + "yearlyrecur' style='display:none'><td id='recuryeartext' class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("RepeatBy") + ":</div></td>" +
                        "<td id='recuryearcount' ><table class='e-table' cellpadding='3'><tr>" +
                        "<td><div><input id='" + this._id + "yearday' class='e-yearrecurday' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearday'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                        "<td><div class='e-controlalign'><input id='yearmonth' class='yearmonth' type='text' name='yearmonth' value=''/></div></td>" +
                        "<div id='yearmonthlist'><ul>";
                for (var monthcount = 0; monthcount < (months.length - 1) ; monthcount++) {
                    $recurWin += "<li>" + months[monthcount] + "</li>";
                }
                $recurWin += "</ul></div><td><input id='yeardate' class='yeardate' text='text'/></td></tr>" +
                        "<tr><td><div><input id='" + this._id + "yearother' class='e-yearrecurposi' name='yearday' type='radio'/><label class='e-textmargin' for='" + this._id + "yearother'>" + this._getLocalizedLabels("The") + "</label></div></td>" +
                        "<td><div><input id='yearsrt' class='yearsrt' type='text' name='yearsrt' value=''/><div id='yearsrtlist'><ul>";
                for (var yearstartday = 0; yearstartday < startday.length; yearstartday++) {
                    $recurWin += "<li>" + this._getLocalizedLabels(startday[yearstartday]) + "</li>";
                }
                $recurWin += "</ul></div></div></td><td><div class='e-controlalign'><input id='yearsrtday' class='yearsrtday' type='text' name='yearsrtday' value=''/>" +
                    "<div id='yearsrtdaylist'><ul>";
                for (var yearweek = 0; yearweek < weekfull.length; yearweek++) {
                    $recurWin += "<li>" + weekfull[yearweek] + "</li>";
                }
                $recurWin += "</ul></div></div></td><td><div><span>" + this._getLocalizedLabels("OfEvery").split(" ")[0] + "</span></div></td><td><div><input id='yearsrtmonth' class='yearsrtmonth' type='text' name='yearsrtmonth' value=''/>" +
                        "<div id='yearsrtmonthlist'><ul>";
                for (var yearmonth = 0; yearmonth < (months.length - 1) ; yearmonth++) {
                    $recurWin += "<li>" + months[yearmonth] + "</li>";
                }
                $recurWin += "</ul></div></div></td></tr></table></td></tr>";
                $recurWin += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("StartsOn") + ":</td>" +
                       "<td><input id='recurstartdate' class='recurstartdate' type='text' name='RecurStartDate' value=''/></td></tr>";
                $recurWin += "<tr><td class='e-recurendslabel'><div class='e-recurendsalign'>" + this._getLocalizedLabels("Ends") + ":</div></td>" +
                        "<td><table class='e-table' cellpadding='3'><tr><td><div><input id='repeatendnever' class='recurends e-recurnoend' type='radio' name='repeatend' value='Never'/><label class='e-textmargin' for='repeatendnever'>" + this._getLocalizedLabels("Never") + "</label></div></td></tr>" +
                        "<tr><td><div><input id='" + this._id + "repeatendafter' class='recurends e-recurafter' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendafter'>" + this._getLocalizedLabels("After") + "</label>" +
                        "</div></td><td><div><input id='dailyoccurance' class='e-recurcount' type='text'/></div></td>" +
                        "<td><span style='"+ margin +": -80px;'>" + this._getLocalizedLabels("Occurence") + "</span></td></tr>" +
                        "<td><div><input id='" + this._id + "repeatendon' class='recurends e-recuruntil' type='radio' name='repeatend'/><label class='e-textmargin' for='" + this._id + "repeatendon'>" + this._getLocalizedLabels("On") + "</label></div></td>" +
                        "<td><input id='daily' class='e-until' type='text' name='daily' value=''/></td></tr></table></td></tr>";
                $recurWin += "<tr style='display:none'><td><span class='e-textlabel'>" + this._getLocalizedLabels("Summary") + ":</span></td><td><span class=e-recurRule></span></td></tr></tbody></table></form>";
                $recurWin += "<div class='e-floatright'><table class='e-table' cellpadding='5'><tr><td><div class='e-btnmargin e-floatleft'><button id='donerecur' class='recurbutton'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                        "<td><div class='e-btnmargin e-floatleft'><button id='cancelrecur' class='e-cancelrecur recurbutton'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></tabled></div></div>";
                return $recurWin;
            }
        },

        _renderNavigationDrawer: function () {
            if (ej.isNullOrUndefined(this._scheduleDrawer)) {
                this._scheduleDrawer = ej.buildTag('div#' + this._id + 'navDrawer.navDrawerMenu e-scheduledrawer');
                var $scheduleDrawer = "<ul><li data-ej-imageClass='e-icon e-mAddEvent' data-ej-text='" + this._getLocalizedLabels("AddEvent") + "' id='addEvent'></li>";
                for (var i = 0; i < this.model.views.length; i++) {
                    switch (this.model.views[i]) {
                        case "day": $scheduleDrawer += "<li data-ej-imageClass='e-icon e-m" + this.model.views[i] + " e-" + this.model.views[i] + "' data-ej-text='" + this._getLocalizedLabels("Day") + "' id='" + this.model.views[i] + "'></li>";
                            break;
                        case "week": $scheduleDrawer += "<li data-ej-imageClass='e-icon e-m" + this.model.views[i] + " e-" + this.model.views[i] + "' data-ej-text='" + this._getLocalizedLabels("Week") + "' id='" + this.model.views[i] + "'></li>";
                            break;
                        case "workweek": $scheduleDrawer += "<li data-ej-imageClass='e-icon e-m" + this.model.views[i] + " e-" + this.model.views[i] + "' data-ej-text='" + this._getLocalizedLabels("WorkWeek") + "' id='" + this.model.views[i] + "'></li>";
                            break;
                        case "month": $scheduleDrawer += "<li data-ej-imageClass='e-icon e-m" + this.model.views[i] + " e-" + this.model.views[i] + "' data-ej-text='" + this._getLocalizedLabels("Month") + "' id='" + this.model.views[i] + "'></li>";
                            break;
                        case "customview": $scheduleDrawer += "<li data-ej-imageClass='e-icon e-m" + this.model.views[i] + " e-" + this.model.views[i] + "' data-ej-text='" + this._getLocalizedLabels("CustomView") + "' id='" + this.model.views[i] + "'></li>";
                            break;
                        case "agenda": $scheduleDrawer += "<li data-ej-imageClass='e-icon e-m" + this.model.views[i] + " e-" + this.model.views[i] + "' data-ej-text='" + this._getLocalizedLabels("Agenda") + "' id='" + this.model.views[i] + "'></li>";
                            break;
                    }
                }
                $scheduleDrawer += "</ul>";
                this._scheduleDrawer.append($scheduleDrawer);
                this.element.find(".e-scheduleinner").append(this._scheduleDrawer);
                this._navDrawer = true;
                this._scheduleDrawer.ejNavigationDrawer({ type: "overlay", targetId: this._id + "navDrawerIcon", direction: (this.model.enableRTL) ? "left" : "right", enableListView: true, listViewSettings: { persistSelection: true, selectedItemIndex: this.model.views.toString().toLowerCase().split(',').indexOf(this.currentView()) + 1, width: 100, height: "auto", mouseUp: $.proxy(this._slideMenuClick, this) }, position: "normal" });
                this.model.enableRTL && this._scheduleDrawer.parent().css("left", "0px");
                this._navDrawer = false;
            }
        },

        _slideMenuClick: function (e) {
            if (this._navDrawer) return false;
            this._scheduleDrawer.ejNavigationDrawer("close");
            var menu = (e.item.find("div").hasClass("e-day")) ? "day" : (e.item.find("div").hasClass("e-week")) ? "week" : (e.item.find("div").hasClass("e-workweek")) ? "workweek" : (e.item.find("div").hasClass("e-month")) ? "month" : (e.item.find("div").hasClass("e-customview")) ? "customview" : (e.item.find("div").hasClass("e-agenda")) ? "agenda" : "open";
            if (menu != "open") {
                this._navView = this._navSelected = true;
                (this.currentView() != menu) && this._viewChange(null, this.currentView(), menu);
                this._navSelected = false;
            }
            else {
                if (this.currentView() != "agenda") {
                    var date = new Date(this.currentDate()), targetElement;
                    var column = 0;
                    date = new Date(date.setHours(0, 0, 0, 0));
                    for (var i = 0; i < this.dateRender.length; i++) {
                        if (new Date(this.dateRender[i]).getTime() == date.getTime()) {
                            column = i;
                            break;
                        }
                    }
                    var row = new Date().getHours();
                    row = this.model.startHour + (row * 2);
                    targetElement = (this.currentView() != "month" && this.currentView() != "customview") && this.model.showTimeScale ? this.element.find(".e-draggableworkarea tr")[row].cells[column] : this.element.find(".e-monthcells")[column];
                    $(targetElement).addClass(".e-selectedCell").trigger($.Event('click'));
                    this._appointmentWindow({ target: targetElement });
                }
            }
        },

        _renderAlertWindow: function () {
            this._alertWindow = ej.buildTag('div.e-alertdialog#' + this._id + "alertWindow");
            var $alertWindow = "<div><div class='e-alerttext'></div><div class='e-alertbtn'><button id='" + this._id + "alertok' type='button' class='e-alertbutton e-alertOk'>" + this._getLocalizedLabels("Ok") + "</button></div></div>";
            this._alertWindow.append($alertWindow);
            this.element.append(this._alertWindow);
            this._alertWindow.find(".e-alertbutton").ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "100px", click: $.proxy(this._alertClose, this), cssClass: this.model.cssClass });
            this._alertWindow.ejDialog({
                width: (this._mediaQuery) ? "75%" : 450,
                cssClass: this.model.cssClass,
                showOnInit: false,
                enableRTL: this.model.enableRTL, enableModal: true,
                title: this._getLocalizedLabels("RecurrenceAlert"),
                target: "#" + this._id,
                enableResize: false,
                allowKeyboardNavigation: false,
                close: $.proxy(this._alertClose, this)
            });
            $("#" + this._id + "alertWindow_wrapper").addClass("e-scheduledialog").find(".e-titlebar").addClass("e-dialogheader");
        },

        _alertClose: function (args) {
            this._alertWindow.find(".e-alerttext").html("");
            var value;
            if (args.type == "click" && args.model.text == this._getLocalizedLabels("Ok") && this._alertWindow.find(".e-alertCancel").length != 0) {
                this._deletingAppointment();
            }
            if ($("#" + this._id + "alertcancel").length != 0) {
                $("#" + this._id + "alertcancel").remove();
            }
            this._alertWindow.ejDialog("close");
        },

        _renderAppointmentWindow: function () {
            if (this._mediaQuery) {
                this._appointmentAddWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppointmentAddEditWindow");
                var $appWindow = "<div class='e-appwindow e-windowmargin'>" +
                                "<form id='" + this._id + "_AddEditForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
                $appWindow += "<tr><td colspan='3' class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("AppointmentSubject") + ":</td></tr><tr><td colspan='3' class='e-rightfields'>" +
                                  "<input id='" + this._id + "subject' tabIndex='0' class='subject e-inputtext' name='" + this.model.appointmentSettings["subject"] + "' type='text' /><input id='id' class='id' type='text' name='" + this.model.appointmentSettings["id"] + "' value='' style='display: none'/></td></tr>";
                if (this.model.showLocationField) {
                    $appWindow += "<tr><td colspan='3' class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("Location") + ":</td></tr><tr><td colspan='3' class='e-rightfields'>" +
                                      "<input id='" + this._id + "location' tabIndex='0' class='location e-inputtext' name='" + this.model.appointmentSettings["location"] + "' type='text' /></td></tr>";
                }
                $appWindow += "<tr><td class='e-textlabel' colspan='3'>" + this._getLocalizedLabels("StartTime") + ":</td></tr><tr><td width='37%'><input id='" + this._id + "startdate' class='startenddate' type='text' name='StartDate' value='' /></td><td width=31%'><input type='text' id='" + this._id + "starttime' class='startendtime' name='StartTime'/></td>" +
                                  "<td width='31%'><input id='" + this._id + "startTimeZone' class='startutc'  name='" + this.model.appointmentSettings["startTimeZone"] + "' type='text' value=''/><div id='Timezoneliststr'></div></td></tr>";
                $appWindow += "<tr><td class='e-textlabel' colspan='3'>" + this._getLocalizedLabels("EndTime") + ":</td></tr><tr><td><input id='" + this._id + "enddate' class='startenddate' type='text' name='EndDate' value='' /></td><td><input type=''text' id='" + this._id + "endtime' class='startendtime' name='EndTime'/></td>" +
                                  "<td><input  id='" + this._id + "endTimeZone' class='endutc' name='" + this.model.appointmentSettings["endTimeZone"] + "' type='text' value=''/><div id='Timezonelistend'></div></td></tr>";
                if (this.model.prioritySettings.enable) {
                    $appWindow += "<tr><td class='e-leftfields e-textlabel' colspan='3'>" + this._getLocalizedLabels("Priority") + ":</td></tr><tr><td colspan='3' class='e-rightfields'>" + "<input id= '" + this._id + "_priority' class='priority' type='text' name='" + this.model.appointmentSettings["priority"] + "' value=''/><div id='prioritylist'></div></td></tr>";
                }
                $appWindow += "<tr><td colspan='3'><table class='e-table' cellpadding='3'><tr><td><div class='e-allday e-checkboxmargin'><input id='" + this._id + "allday' tabIndex='0' class='allday' type='checkbox' name='" + this.model.appointmentSettings["allDay"] + "'/></div></td><td><div class='e-alldaytext'>" +
                                  "<label for='" + this._id + "allday'>" + this._getLocalizedLabels("AllDay") + "</label></div></td><td><div class='e-checkboxmargin e-recurrence'>" +
                                  "<input id='" + this._id + "recurrence' tabIndex='0' class='recurrence' type='checkbox' name='" + this.model.appointmentSettings["recurrence"] + "' value='recurrence'/></div></td><td><label class='e-repeatlbl' for='" + this._id + "recurrence'>" +
                                      this._getLocalizedLabels("Recurrence") + "</label></td></tr></table></td></tr>";
                $appWindow += "<tr id='" + this._id + "summary' class='" + this._id + "summary' style='display:none'><td class='e-textlabel'><label>" + this._getLocalizedLabels("Summary") + ":</label></td></tr><tr class='" + this._id + "summary' style='display:none'><td colspan='4'><table><tr><td><label class='e-recurFinalRule'></label></td><td><div class='e-recuredit'><a class='e-recuredit' href='#'>" + this._getLocalizedLabels("Edit") + "</a><span class='e-icon e-scheduleedit'></span></div></td></tr></table></td></tr>";
                if (this.model.categorizeSettings.enable) {
                    $appWindow += "<tr id='" + this._id + "categorize'><td class='e-leftfields e-textlabel' colspan='3'>" + this._getLocalizedLabels("Categorize") + ":</td></tr><tr><td colspan='3' class='e-rightfields'>" + "<input id= '" + this._id + "_categorize' class='e-categorizevalue' type='text' value=''/><div id='ownerlist'></div></td></tr>";
                }
                if (!ej.isNullOrUndefined(this.model.resources) && this.model.resources.length > 0 && ej.scheduleFeatures.resources)
                    $appWindow = this._renderResourceElements($appWindow);
                $appWindow += "<tr><td colspan='3'><b>Description:</b></td></tr><tr><td colspan='3'><textarea class='e-description e-inputtext' id='description' name='" + this.model.appointmentSettings["description"] + "'></textarea></td></tr>";
                $appWindow += "</tbody></table></form>";
                $appWindow += "<div class='e-floatright' style='width:100%'><table width='100%' class='e-table' cellpadding='5'><tr><td width='30%'><div class='e-btnmargin e-floatleft'><button id='mDelete' class='e-btndelete e-appdelete'>" + this._getLocalizedLabels("Delete") + "</button></div></td><td width='30%'><div class='e-btnmargin e-floatleft'><button id='done' class='e-btndone e-appsave'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                              "<td width='30%'><div class='e-btnmargin e-floatleft'><button id='cancel' class='e-appointcancel cancel'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></table></div></div>";
                this._appointmentAddWindow.append($appWindow).append(this._recurrenceWindow());

            }
            else {
                this._appointmentAddWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppointmentAddEditWindow");
                var $appWindow = "<div class='e-appwindow e-windowmargin'>" +
                                    "<form id='" + this._id + "_AddEditForm'><table class='e-table' width='100%' cellpadding='7'><tbody>";
                $appWindow += "<tr  id='" + this._id + "subjecttr' class='" + this._id + "subjecttr'><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("AppointmentSubject") + ":</td><td colspan='3' class='e-rightfields'>" +
                                 "<input id='" + this._id + "subject' tabIndex='0' class='subject e-inputtext' name='" + this.model.appointmentSettings["subject"] + "' type='text' /><input id='id' class='id' type='text' name='" + this.model.appointmentSettings["id"] + "' value='' style='display: none'/></td></tr>";
                if (this.model.showLocationField) {
                    $appWindow += "<tr id='" + this._id + "showLocation' class='" + this._id + "showLocation'><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("Location") + ":</td><td colspan='3' class='e-rightfields'>" +
                                     "<input id='" + this._id + "location' tabIndex='0' class='location e-inputtext' name='" + this.model.appointmentSettings["location"] + "' type='text' /></td></tr>";
                }
                $appWindow += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("StartTime") + ":</td><td width='25%'><input id='" + this._id + "startdate' class='startenddate' type='text' name='StartDate' value='' /></td><td width=22%'><input type='text' id='" + this._id + "starttime' class='startendtime' name='StartTime'/></td>" +
                                  "<td width='28%'><input  id='" + this._id + "startTimeZone'  class='startutc'  name='" + this.model.appointmentSettings["startTimeZone"] + "' type='text' value=''/><div id='Timezoneliststr'></div></td></tr>";
                $appWindow += "<tr><td class='e-textlabel'>" + this._getLocalizedLabels("EndTime") + ":</td><td><input id='" + this._id + "enddate' class='startenddate' type='text' name='EndDate' value='' /></td><td><input type=''text' id='" + this._id + "endtime' class='startendtime' name='EndTime'/></td>" +
                                  "<td><input id='" + this._id + "endTimeZone'  class='endutc'  name='" + this.model.appointmentSettings["endTimeZone"] + "'  type='text' value=''/><div id='Timezonelistend'></div></td></tr>";
                var win;
                if (this.model.prioritySettings.enable) {
                    win = "<td colspan='1' class='e-textlabel " + this._id + "priorityshow'>" + this._getLocalizedLabels("Priority") + ":</td><td colspan='1' class='" + this._id + "prioritylistshow'><input id='" + this._id + "_priority' class='priority' type='text' name='" + this.model.appointmentSettings["priority"] + "' value='' /><div id='prioritylist'></div></td>";
                }
                else {
                    win = "<td style='display:none'></td>";
                }
                $appWindow += "<tr class='" + this._id + "parrow'>" + win + "<td class='" + this._id + "allDay' colspan='4'><table class='e-table' cellpadding='3'><tr><td><div class='e-allday e-checkboxmargin'><input id='" + this._id + "allday' tabIndex='0' class='allday' type='checkbox' name='" + this.model.appointmentSettings["allDay"] + "'/></div></td><td><div class='e-alldaytext'>" +
                                 "<label for='" + this._id + "allday'>" + this._getLocalizedLabels("AllDay") + "</label></div></td><td><div class='e-checkboxmargin e-recurrence'>" +
                                  "<input id='" + this._id + "recurrence' tabIndex='0' class='recurrence' type='checkbox' name='" + this.model.appointmentSettings["recurrence"] + "' value='recurrence'/></div></td><td><label class='e-repeatlbl' for='" + this._id + "recurrence'>" +
                                  this._getLocalizedLabels("Recurrence") + "</label></td></tr></table></td></tr>";
                $appWindow += "<tr id='" + this._id + "summary' class='" + this._id + "summary' style='display:none'><td class='e-textlabel'><label>" + this._getLocalizedLabels("Summary") + ":</label></td><td colspan='3'><table><tr><td><label class='e-recurFinalRule'></label></td><td><div class='e-recuredit'><a class='e-recuredit' href='#'>" + this._getLocalizedLabels("Edit") + "</a><span class='e-icon e-scheduleedit'></span></div></td></tr></table></td></tr>";
                if (this.model.categorizeSettings.enable) {
                    $appWindow += "<tr id='" + this._id + "categorize'><td class='e-leftfields e-textlabel'>" + this._getLocalizedLabels("Categorize") + ":</td><td colspan='3' class='e-rightfields'>" + "<input id= '" + this._id + "_categorize' class='e-categorizevalue' type='text' value=''/><div id='ownerlist'></div></td></tr>";
                }
                if (!ej.isNullOrUndefined(this.model.resources) && this.model.resources.length > 0 && ej.scheduleFeatures.resources)
                    $appWindow = this._renderResourceElements($appWindow);
                $appWindow += "<tr><td colspan='4'><textarea class='e-description e-inputtext' id='description' name='" + this.model.appointmentSettings["description"] + "'></textarea></td></tr>";
                $appWindow += "</tbody></table></form>";
                $appWindow += "<div class='e-floatright'><table class='e-table' cellpadding='5'><tr><td><div class='e-btnmargin e-floatleft'><button id='done' class='e-btndone e-appsave'>" + this._getLocalizedLabels("Done") + "</button></div></td>" +
                              "<td><div class='e-btnmargin e-floatleft'><button id='cancel' class='e-appointcancel cancel'>" + this._getLocalizedLabels("Cancel") + "</button></div></td></tr></table></div></div>";
                this._appointmentAddWindow.append($appWindow).append(this._recurrenceWindow());
            }
            this.element.append(this._appointmentAddWindow);
            this._renderControls();
            this._appointmentAddWindow.ejDialog({
                width: (this._mediaQuery) ? "100%" : 600,
                cssClass: this.model.cssClass,
                showOnInit: false,
                enableRTL: this.model.enableRTL, enableModal: true,
                title: this._getLocalizedLabels("CreateAppointmentTitle"),
                target: "#" + this._id,
                enableResize: false,
                allowKeyboardNavigation: false,
                close: $.proxy(this._onclose, this)
            });
            $("#" + this._id + "AppointmentAddEditWindow_wrapper").appendTo("body").addClass("e-scheduledialog").find(".e-titlebar").addClass("e-dialogheader");
            $("#" + this._id + "_priority_popup_wrapper").addClass("e-schedulepriority");
            this._on(this._appointmentAddWindow, "click", "a.e-recuredit", $.proxy(this._editRecurrence, this));
        },

        _renderQuickAppWindow: function () {
            this._quickAppointWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppointmentQuickWindow");
            var $quickWin = "<div id='" + this._id + "_quickAppwindow' class='e-tableAppoint'>" +
                        "<table cellpadding='7' class='e-qappwindow'><tbody>";
            $quickWin += "<tr><td colspan='2'><div class='e-quickstartend'></div> <div title='" + this._getLocalizedLabels("Close") + "'class='e-icon e-scheduleclose e-schedclosemargin e-pointercursor'></div></td></tr><tr><td class='e-subjectlabel'>" + this._getLocalizedLabels("AppointmentSubject") + ":</td><td colspan='2' class='e-rightfields'>" +
                              "<input id='" + this._id + "subject' class='subject e-inputtext'  type='text' /></td></tr></tbody></table>";
            $quickWin += "<div style='margin-top:5px; padding-bottom: 30px;'><div tabIndex='0' class='e-detailedapp e-detailedwindow'> " + this._getLocalizedLabels("Detailed") + " >></div><div style='margin-right:-3px;' class='e-floatright'><div class='e-quickAppDone'><button id='done' class='e-btndone qAppDone' style='font-size:12px !important;'>" + this._getLocalizedLabels("CreateAppointmentTitle") + "</button></div>" +
                              "</div></div></div>";
            this._quickAppointWindow.append($quickWin);
            this.element.append(this._quickAppointWindow);
            this._quickAppointWindow.append(ej.buildTag("div.e-quickarrow").append("<div class='e-outerarrowline'></div>").append("<div class='e-innerarrow'></div>"));
            var control = this._quickAppointWindow;
            control.find('.e-btndone').ejButton({ showRoundedCorner: true, width: "130px", click: $.proxy(this._saveAppointment, this), cssClass: this.model.cssClass, enableRTL: this.model.enableRTL });
            this._on(control.find('.e-scheduleclose'), "click", $.proxy(this._quickWindowClose, this));
            this._on(control.find('.e-detailedapp'), "click", $.proxy(this._appointmentWindow, this));
            this._quickAppointWindow.ejDialog({
                width: 330, minHeight: 0,
                cssClass: this.model.cssClass,
                enableRTL: this.model.enableRTL,
                enableAnimation: false,
                showOnInit: false,
                enableResize: false,
                allowDraggable: false, showHeader: false,
                allowKeyboardNavigation: false
            });
            $("#" + this._id + "AppointmentQuickWindow_wrapper").addClass("e-scheduledialog e-schedulequickdialog").css("overflow", "visible").find("#" + this._id + "AppointmentQuickWindow").addClass("e-dialogouter");
        },

        _renderQuickAppDetailsView: function () {
            this._quickAppDetailsWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "AppDetailsWindow");
            var $quickAppWin = "<div id='" + this._id + "_quickAppDetailswindow' class='e-tableAppoint e-appdetails'>" +
                              "<table cellpadding='1' class='e-qappwindow'><tbody>";
            $quickAppWin += "<tr><td colspan='2'><div class='e-quicksubject'></div> <div title='" + this._getLocalizedLabels("Close") + "' class='e-icon e-scheduleclose e-schedclosemargin e-pointercursor'></div></td></tr><tr><td colspan='2' class='e-rightfields'>" +
                              "</td></tr><tr><td colspan='2'><div class='e-quickstartendtime'></div></td></tr></tbody></table>";
            $quickAppWin += "<div style='margin-top:5px;'><div title='" + this._getLocalizedLabels("Delete") + "' class='e-icon e-scheduledelete e-pointercursor' tabIndex='0'></div><div class='e-floatright'><div class='e-editevents'><div class='e-editevent e-textellipsis' tabIndex='0'>" + this._getLocalizedLabels("Editevent") + " | " + "</div><div class='e-editseries e-textellipsis' tabIndex='0'>" + this._getLocalizedLabels("Editseries") + " >></div> </div>" +
                              "</div></div></div>";
            this._quickAppDetailsWindow.append($quickAppWin);
            this.element.append(this._quickAppDetailsWindow);
            this._quickAppDetailsWindow.append(ej.buildTag("div.e-quickarrow").append("<div class='e-outerarrowline'></div>").append("<div class='e-innerarrow'></div>"));
            this._on(this._quickAppDetailsWindow.find('.e-scheduleclose'), "click", $.proxy(this._quickAppDetailWindowClose, this));
            this._on(this._quickAppDetailsWindow.find('.e-scheduledelete'), "click", $.proxy(this._deleteAppoint, this));
            this._on(this._quickAppDetailsWindow.find('.e-editevent,.e-editseries'), "click", $.proxy(this._editClick, this));
            this._quickAppDetailsWindow.ejDialog({
                width: 270, minHeight: 0,
                cssClass: this.model.cssClass,
                enableRTL: this.model.enableRTL,
                enableAnimation: false,
                showOnInit: false,
                enableResize: false,
                allowDraggable: false, showHeader: false,
                allowKeyboardNavigation: false
            });
            $("#" + this._id + "AppDetailsWindow_wrapper").addClass("e-scheduledialog e-schedulequickdialog").css("overflow", "visible").find("#" + this._id + "AppDetailsWindow").addClass("e-dialogouter");
        },

        _editClick: function (e) {
            if ($(e.target).hasClass("e-editseries") && (this._quickAppDetailsWindow.find('.e-editseries').hasClass("e-disable"))) return false;
            var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
            var templist = new ej.DataManager(this._processed).executeLocal(query);
            if ($(e.target).hasClass("e-editevent")) {
                if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
                    this._parentId = templist[0]["ParentId"];
                    this._currentAction = ej.Schedule.Actions.EditOcurrence;
                    this._showAppointmentDetails(this._appUid, true);
                }
                else if (ej.isNullOrUndefined(templist[0][this.model.appointmentSettings["recurrence"]])) {
                    this._currentAction = ej.Schedule.Actions.Save;
                    this._showAppointmentDetails(this._appUid, true);
                }
                else {
                    this._currentAction = ej.Schedule.Actions.Save;
                    this._showAppointmentDetails(templist[0]["Guid"], true);
                }
            }
            else if ($(e.target).hasClass("e-editseries")) {
                if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
                    this._parentId = (templist[0][this.model.appointmentSettings["recurrenceId"]] == null) ? templist[0]["ParentId"] : templist[0][this.model.appointmentSettings["recurrenceId"]];
                    this._currentAction = ej.Schedule.Actions.EditSeries;
                    this._showAppointmentDetails(this._parentId, false);
                }
            }
            this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
        },

        _deleteAppoint: function (e) {
            if (!this.model.readOnly) {
                if (!ej.isNullOrUndefined(e) && (ej.isNullOrUndefined(this._quickAppDetailsWindow) || !this._quickAppDetailsWindow.ejDialog("isOpen"))) {
                    var _target = $(e.target).parent();
                    var appoint = _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") || _target.hasClass("e-agendaappointment") ? _target : _target.parents(".e-appointment,.e-alldayappointment,.e-monthappointment,.e-agendaappointment");
                    this._appUid = appoint.attr("guid");
                    if (this._mediaQuery) this._mAppId = this._appUid;
                    var _editApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                    this._recurId = _editApp[0][this.model.appointmentSettings["recurrenceId"]];
                    if (_editApp[0][this.model.appointmentSettings["recurrence"]])
                        this._deletingAppointment();
                    else
                        this._deleteConfirmation();
                }
                else {
                    var _editApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                    this._recurId = _editApp[0][this.model.appointmentSettings["recurrenceId"]];
                    if (_editApp[0][this.model.appointmentSettings["recurrence"]]) {
                        this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("isOpen") && this._quickAppDetailsWindow.ejDialog("close");
                        this._deletingAppointment();
                    }
                    else
                        this._deleteConfirmation();
                }
            }
        },

        _deletingAppointment: function () {
            if (this._appUid) {
                this._currentAction = ej.Schedule.Actions.Delete;
                var parentData = this._getAppointmentByParentId((this._mediaQuery) ? ej.isNullOrUndefined(this._mAppId) ? this._appUid : this._mAppId : this._appUid);
                if (!parentData[this.model.appointmentSettings["recurrence"]])
                    this._deleteApp((this._mediaQuery) ? ej.isNullOrUndefined(this._mAppId) ? this._appUid : this._mAppId : this._appUid);
                else {
                    this._parentId = parseInt(parentData["AppTaskId"]);
                    this._recurEditWindow.ejDialog("open");
                    this._recurEditWindow.focus();
                    this._deleteBeforeOpen();
                }
            }
            this._alertWindow.ejDialog("close");
        },

        _deleteConfirmation: function () {
            this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
            ej.isNullOrUndefined(this._alertWindow) && this._renderAlertWindow();
            if ($(".e-alertbtn").find(".e-alertCancel").length == 0) {
                var canButton = "<button id='" + this._id + "alertcancel' class='e-alertbutton e-alertCancel'>" + this._getLocalizedLabels("Cancel") + "</button>";
                this._alertWindow.find(".e-alertbtn").append(canButton);
                this._alertWindow.find(".e-alertCancel").ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "100px", click: $.proxy(this._alertClose, this), cssClass: this.model.cssClass });
                if (this._mediaQuery)
                    this._alertWindow.find(".e-alertCancel").css({ 'margin': '0px 20px 15px 10px' });
                $("#" + this._id + "alertWindow").ejDialog({
                    width: (this._mediaQuery) ? "90%" : 450,
                    title: this._getLocalizedLabels("MouseOverDeleteTitle")
                });
                this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("DeleteConfirmation"));
                this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Delete"));
            }
            if (!this._alertWindow.ejDialog("isOpen")) {
                this._alertWindow.ejDialog("open");
                this._alertWindow.find(".e-alertOk").attr("tabIndex", 0).focus();
            }

        },

        _quickAppDetailWindowClose: function () {
            this._quickAppDetailsWindow.ejDialog("close");
            this._processFocus();
        },

        _alldayCheck: function () {
            if (this._appointmentAddWindow.find(".allday").ejCheckBox("option", "checked")) {
                this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker({ value: (this._timeMode == "12") ? "12:00 AM" : "00:00", enabled: false });
                this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker({ value: (this._timeMode == "12") ? "11:59 PM" : "23:59", enabled: false });
                this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList({ enabled: false });
            }
            else {
                if (this._tempStart != "" || this._tempEnd != "") {
                    this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker({ value: this._tempStart });
                    this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker({ value: this._tempEnd });
                    if (((this._timeMode == "12") ? (this._tempStart == "12:00 AM" && this._tempEnd == "11:59 PM") : (this._tempStart == "00:00" && this._tempEnd == "23:59"))) {
                        this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker({ value: Globalize.format(new Date("1/1/2000 " + this.model.workHours.start + ":00:00"), this._pattern.t) });
                        this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker({ value: Globalize.format(new Date("1/1/2000 " + this.model.workHours.start + ":30:00"), this._pattern.t) });
                    }
                }
                this._appointmentAddWindow.find(".startendtime").ejTimePicker({ enabled: true });
                this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList({ enabled: true });
            }
        },

        _renderRecurEditWindow: function () {
            this._recurEditWindow = ej.buildTag('div.e-scheduledialog#' + this._id + "RecurrenceEdit");
            var msgDiv = ej.buildTag('div.e-recureditmsg', this._getLocalizedLabels("RecurrenceEditMessage"));
            var btns = ej.buildTag('div.e-recureditbtns').append(ej.buildTag('button.e-editonly', this._getLocalizedLabels("RecurrenceEditOnly"), {}, { name: "recurrenceeditonly", value: this._getLocalizedLabels("RecurrenceEditOnly") })).append(ej.buildTag('button.e-editseries', this._getLocalizedLabels("RecurrenceEditSeries"), {}, { name: "recurrenceeditseries", value: this._getLocalizedLabels("RecurrenceEditSeries") }));
            this._recurEditWindow.append(msgDiv).append(btns);
            btns.find('.e-editonly,.e-editseries').ejButton({ cssClass: this.model.cssClass, enableRTL: this.model.enableRTL, showRoundedCorner: true });
            this._on(this._recurEditWindow, "click", ".e-editonly,.e-editseries", $.proxy(this._recurBtnClick, this));
            this._on(this._recurEditWindow, "click", "div.e-scheduleclose", $.proxy(this._recrConformClose, this));
            this._recurEditWindow.appendTo(this.element);
            if (this._mediaQuery) {
                this._recurEditWindow.find('.e-editonly,.e-editseries').css({ 'margin-top': '10px' });
            }
            this._recurEditWindow.ejDialog({
                width: (this._mediaQuery) ? "90%" : 360,
                cssClass: this.model.cssClass,
                enableRTL: this.model.enableRTL,
                title: this._getLocalizedLabels("RecurrenceEditTitle"),
                showOnInit: false,
                enableModal: true,
                enableResize: false,
                target: "#" + this._id,
                allowKeyboardNavigation: false,
                close: $.proxy(this._processFocus, this),
                beforeOpen: $.proxy(this._deleteBeforeOpen, this)
            });
            $("#" + this._id + "RecurrenceEdit_wrapper").appendTo("body").addClass("e-scheduledialog");
        },

        _recrConformClose: function () {
            this._recurEditWindow.ejDialog("close");
            this._processFocus();
        },
        _deleteBeforeOpen: function () {
            $('#' + this._id + 'RecurrenceEdit_title').addClass('e-dialogheader');
            $('#' + this._id + 'RecurrenceEdit_title').find('.e-title').html(this._currentAction == ej.Schedule.Actions.Delete ? this._getLocalizedLabels("RecurrenceDeleteTitle") : this._getLocalizedLabels("RecurrenceEditTitle"));
        },
        _getNewOverlapList: function (appoint) {
            if (this.currentView() !== "month" && !(this._isCustomView())) {
                var strDate = appoint[this.model.appointmentSettings["startTime"]];
                var index = this.currentView() == "day" ? 0 : (this.model.orientation == "horizontal" && this.currentView() == "month") ? $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this.monthDays) : $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this._dateRender);
                var resIndex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings["id"], appoint[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : 0;
                var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(this._dateRender[index]));
                predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(this._dateRender[index]).setHours(23, 59, 59)));
                predicate = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[resIndex][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
                var apps = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(predicate));
                this._appList = this._alldaySort(apps);
                this._newoverlapList = this._overlapApp(appoint["AppTaskId"], appoint[this.model.appointmentSettings["startTime"]], appoint[this.model.appointmentSettings["endTime"]], null);
                (this._currentAction == ej.Schedule.Actions.Delete) ? this._newoverlapList.splice(this._newoverlapList.indexOf(appoint), 1) : "";
            }
        },

        _recurBtnClick: function (args) {
            if ($(args.target).hasClass("e-editonly")) {
                this._editOnlyClicked = true;
                if (this._currentAction == ej.Schedule.Actions.Delete) {
                    var _newAddList = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid))[0];
                    this._getNewOverlapList(_newAddList);
                    if (_newAddList[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) {
                       var Parid =  _newAddList[this.model.appointmentSettings["recurrenceId"]] == null ? _newAddList["ParentId"] : _newAddList[this.model.appointmentSettings["recurrenceId"]]
                       var deleted = new ej.DataManager(this._currentAppointmentData).executeLocal(new ej.Query().where("AppTaskId", ej.FilterOperators.equal, Parid));
                       if (this.dataSource() instanceof ej.DataManager) {
                           var promise = this._dataManager.remove(this.model.appointmentSettings["id"], _newAddList[this.model.appointmentSettings["id"]]);
                       }
                       else
                           var promise = this._dataManager.remove("Guid", _newAddList["Guid"]);
                        var proxy = this;
                        if ($.isFunction(promise.promise)) {
                            promise.done(function (e) {
                                if (proxy._trigger("actionBegin", { id: proxy._appId, requestType: "appointmentDelete" }))
                                    return false;
                                if (proxy.model.enableLoadOnDemand)
                                    proxy._initDataSource("refresh");
                                else {
                                    var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                    proxy._currentAppointmentData.length = 0;
                                    proxy._dateFormat(value);
                                    proxy._bindAppointments = true;
                                    proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                    proxy._dataProcessing(value);
                                    proxy._renderAppointmentAll();
                                }
                                proxy._trigger("appointmentDeleted", { appointment: deleted });
                                proxy._trigger("actionComplete", { appointment: deleted, requestType: "appointmentDelete" });
                            });
                            promise.fail(function (e) { });
                        }
                        else {
                            if (this._trigger("actionBegin", { id: this._appUid, requestType: "appointmentDelete" }))
                                return false;
                            new ej.DataManager(this._processed).remove("Guid", this._appUid);
                            this.element.find('[guid=' + this._appUid + ']').remove();
                            this.element.find("#categorybar_" + this._appUid).remove();
                            (this.currentView() !== "month" && !(this._isCustomView())) ? this.currentView() == "agenda" ? this._renderAgendaContent() : this._reRenderApp(_newAddList) : this._renderAppointmentAll();
                            this._trigger("appointmentDeleted", { appointment: deleted });
                            this._trigger("actionComplete", { appointment: deleted, requestType: "appointmentDelete" });
                        }
                    }
                    else {
                        var Parid = _newAddList[this.model.appointmentSettings["recurrenceId"]] == null ? _newAddList["ParentId"] : _newAddList[this.model.appointmentSettings["recurrenceId"]]
                        var deleApp = this._deleteOcurrence(Parid, _newAddList[this.model.appointmentSettings["startTime"]]);
                        var temp = this._timeZoneAppointments(deleApp[0], "");
                        if (this.dataSource() instanceof ej.DataManager) {
                            var promise = this._dataManager.update(this.model.appointmentSettings["id"], temp);
                        }
                        else
                            var promise = this._dataManager.update("AppTaskId", temp);
                        var updateTemp = (this._timeZoneAppointments(temp, "reverse"));
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTemp);
                        var proxy = this;
                        if (promise && $.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                if (proxy._trigger("actionBegin", { id: proxy._appId, requestType: "appointmentDelete" }))
                                    return false;
                                if (proxy.model.enableLoadOnDemand)
                                    proxy._initDataSource("refresh");
                                else {
                                    var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                    proxy._currentAppointmentData.length = 0;
                                    proxy._dateFormat(value);
                                    proxy._bindAppointments = true;
                                    proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                    proxy._dataProcessing(value);
                                    proxy._renderAppointmentAll();
                                }
                                proxy._trigger("appointmentDeleted", { appointment: _newAddList });
                                proxy._trigger("actionComplete", { appointment: _newAddList, requestType: "appointmentDelete" });
                            });
                            promise.fail(function (e) {
                            });
                        }
                        else {
                            if (this._trigger("actionBegin", { id: this._appId, requestType: "appointmentDelete" }))
                                return false;
                            if ((new Date(new Date(updateTemp[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(updateTemp[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((updateTemp[this.model.appointmentSettings["endTime"]] - updateTemp[this.model.appointmentSettings["startTime"]]) / 3600000) < 24) && (updateTemp[this.model.appointmentSettings["allDay"]] !== true)) {
                                var splitApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                                for (i = 0; i < splitApp.length; i++) {
                                    var delId = splitApp[i]["Guid"];
                                    var dele = new ej.DataManager(this._processed).remove("Guid", delId);
                                    this.element.find('[guid=' + delId + ']').remove();
                                    this.element.find("#categorybar_" + delId).remove();
                                }
                            }
                            else {
                                var dele = new ej.DataManager(this._processed).remove("Guid", this._appUid);
                                this.element.find('[guid=' + this._appUid + ']').remove();
                                this.element.find("#categorybar_" + this._appUid).remove();
                            }
                            (this.currentView() !== "month" && !(this._isCustomView())) ? this.currentView() == "agenda" ? this._renderAgendaContent() : this._reRenderApp(_newAddList) : this._renderAppointmentAll();
                            this._trigger("appointmentDeleted", { appointment: _newAddList });
                            this._trigger("actionComplete", { appointment: _newAddList, requestType: "appointmentDelete" });
                        }
                    }
                    var appCount = this._appointmentCount();
                    if (appCount.length < 1) {
                        if (this.model.enableAppointmentNavigation)
                            this._nextPrevButton(appCount);
                    }
                }
                else {
                    this._currentAction = ej.Schedule.Actions.EditOcurrence;
                    this._showAppointmentDetails(this._appUid, true);
                }
            }
            else {
                this._editOnlyClicked = false;
                if (this._currentAction == ej.Schedule.Actions.Delete)
                    this._deleteApp(this._appUid);
                else {
                    this._currentAction = ej.Schedule.Actions.EditSeries;
                    this._showAppointmentDetails(this._parentId, false);
                }
            }
            this._recurEditWindow.ejDialog("close");
        },

        _repeat: function (e) {
            if (e.isChecked || this._recurChecked) {
                if (this._currentAction == ej.Schedule.Actions.EditSeries || this._appointmentAddWindow.find(".e-recurFinalRule").html() != "") {
                    this._appointmentAddWindow.find("." + this._id + "summary").css("display", "table-row");
                }
                else {
                    (this._mediaQuery) && this._appointmentAddWindow.parent().parent().find(".e-dialog-icon").css("display", "none");
                    this._appointmentAddWindow.find(".e-appwindow").toggle();
                    this._appointmentAddWindow.find(".e-recurwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 400);
                    this._recurClearFields();
                    var strDate = new Date(this._appointmentAddWindow.find('.startenddate').ejDatePicker("option", "value"));
                    this._appointmentAddWindow.find('.recurtype').ejDropDownList({ selectedItemIndex: 0 });
                    this._appointmentAddWindow.find('.recurtype').ejDropDownList({ selectedItemIndex: 1 });
                    this._appointmentAddWindow.find(".recurstartdate").ejDatePicker("option", "value", new Date(strDate));
                    this._appointmentAddWindow.find('.e-until').ejDatePicker({ value: new Date(strDate.setDate(strDate.getDate() + 7 * 10)) });
                    this._appointmentAddWindow.find(".e-recurnoend").ejRadioButton({ checked: true });
                }
            }
            else
                (this._currentAction == ej.Schedule.Actions.EditSeries || this._appointmentAddWindow.find(".e-recurFinalRule").html() != "") && this._appointmentAddWindow.find("." + this._id + "summary").css("display", "none");
            this._recurChecked = false;
        },

        _recurtype: function (e) {
            this._rRuleFreq = "";
            var _rule = "";
            var strDate = new Date(this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value"));
            if (e.itemId != null) {
                this._appointmentAddWindow.find("#" + this._id + "repeatEvery").css("display", "table-row");
                if (e.itemId == 0) {
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceDay");
                    this._appointmentAddWindow.find("." + this._id + "weeklyrecur,." + this._id + "monthlyrecur,." + this._id + "yearlyrecur").css("display", "none");
                    this._appointmentAddWindow.find("#" + this._id + "dailyrecur").css("display", "table-row");
                    this._rRuleFreq = "DAILY";
                    _rule = this._getLocalizedLabels("RecurrenceDay").toLowerCase();
                }
                else if (e.itemId == 1) {
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceWeek");
                    var curDay = this._dayNames[strDate.getDay()];
                    this._appointmentAddWindow.find(".e-weekly" + curDay).ejCheckBox({ checked: true });
                    _rule = this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._dayFullNames[strDate.getDay()];
                    this._rRuleFreq = "WEEKLY";
                    this._appointmentAddWindow.find("." + this._id + "monthlyrecur,." + this._id + "yearlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                    this._appointmentAddWindow.find("." + this._id + "weeklyrecur").css("display", "table-row");
                }
                else if (e.itemId == 2) {
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceMonth");
                    this._appointmentAddWindow.find("." + this._id + "weeklyrecur,." + this._id + "yearlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                    this._appointmentAddWindow.find("." + this._id + "monthlyrecur").css("display", "table-row");
                    this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox({ value: strDate.getDate() });
                    this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList({ selectedItemIndex: strDate.getDay() });
                    this._appointmentAddWindow.find('.monthsrt').ejDropDownList({ selectedItemIndex: this._getWeekIndex(strDate) });
                    this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton({ checked: true });
                    this._rRuleFreq = "MONTHLY";
                    _rule = this._getLocalizedLabels("RecurrenceMonth").toLowerCase();
                }
                else if (e.itemId == 3) {
                    this._appointmentAddWindow.find("#" + this._id + "recurtypes").get(0).innerHTML = this._getLocalizedLabels("RecurrenceYear");
                    this._appointmentAddWindow.find("." + this._id + "weeklyrecur,." + this._id + "monthlyrecur,#" + this._id + "dailyrecur").css("display", "none");
                    this._appointmentAddWindow.find("." + this._id + "yearlyrecur").css("display", "table-row");
                    this._appointmentAddWindow.find('.yearmonth').ejDropDownList({ selectedItemIndex: strDate.getMonth() });
                    this._appointmentAddWindow.find('.yeardate').ejNumericTextbox({ value: strDate.getDate() });
                    this._appointmentAddWindow.find('.yearsrtday').ejDropDownList({ selectedItemIndex: strDate.getDay() });
                    this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList({ selectedItemIndex: strDate.getMonth() });
                    this._appointmentAddWindow.find('.yearsrt').ejDropDownList({ selectedItemIndex: this._getWeekIndex(strDate) });
                    this._appointmentAddWindow.find('.e-yearrecurday').ejRadioButton({ checked: true });
                    this._rRuleFreq = "YEARLY";
                    _rule = this._getLocalizedLabels("RecurrenceYear").toLowerCase();
                }
                else if (e.itemId == 4) {
                    this._rRuleFreq = "WEEKDAYS";
                    this._appointmentAddWindow.find("." + this._id + "weeklyrecur,." + this._id + "yearlyrecur,#" + this._id + "dailyrecur,#" + this._id + "repeatEvery,." + this._id + "monthlyrecur").css("display", "none");
                    _rule = e.text;
                }
                this._appointmentAddWindow.find(".e-recurRule").html(_rule);
            }
        },

        _getRenderDates: function () {
            this._dateRender = []; var firstDay, endDay, newDate, week = [], workweek = [], workweeks = [];
            if (this.currentView() === "week" || this.currentView() === "workweek") {
                firstDay = this._firstweekdate(this.currentDate());
                if (firstDay.getDay() != 0 || this._firstdayofweek == 0 && this.model.workWeek.length > 0 && this.currentView() == "workweek" || this.currentView() == "week") {
                    for (var i = 0; i < 7; i++) {
                        newDate = new Date(firstDay);
                        newDate.setDate(firstDay.getDate() + i);
                        this._dateRender.push(new Date(newDate.getFullYear(), newDate.getMonth(), newDate.getDate()).getTime());
                    }
                }
                if ((this._firstdayofweek != 0 || this._firstdayofweek == 0) && this.currentView() == "workweek" && this.model.workWeek[0] == "monday" && this.model.workWeek[4] == "friday" && this.model.workWeek[5] != "saturday") {
                    for (var i = 0; i < 7; i++) {
                        newDate = new Date(this._dateRender[i]);
                        if (newDate.getDay() == 0)
                            this._dateRender.splice(i, 1);
                        else if (newDate.getDay() == 6)
                            this._dateRender.splice(i, 2);
                    }
                }
                else if (this.currentView() == "workweek" && this.model.workWeek) {
                    for (var i = 0, j = 0; i < this._dateRender.length; i++) {
                        if (this.model.workWeek[j] == this._dayNamesArray[i]) {
                            workweek.push(i);
                            j++;
                        }
                    }

                    if (this._firstdayofweek >= workweek[0]) {
                        for (var k = 0; k < workweek.length; k++) {
                            if (this._firstdayofweek <= workweek[k]) {
                                var firstday = workweek[k];
                                break;
                            }
                        }
                    }
                    else {
                        for (var k = 0; k < workweek.length; k++) {
                            if (this._firstdayofweek > workweek[k]) {
                                var firstday = workweek[k];
                                break;
                            }
                        }
                    }

                    for (var i = 0, j = 0; i < workweek.length; i++) {
                        if (firstday <= workweek[i]) {
                            workweeks[j] = workweek[i];
                            j++;
                        }
                    }
                    workweeks = $.merge(workweeks, workweek);

                    if (this._firstdayofweek != 0 && this._firstdayofweek <= firstday) workweeks = $.unique(workweeks);
                    else
                        workweeks = $.unique(workweek);

                    for (var i = 0, j = 0; i < this._dateRender.length; i++) {
                        if (workweeks[j] == new Date(this._dateRender[i]).getDay()) {
                            week.push(this._dateRender[i]);
                            j++;
                        }
                    }
                    this._dateRender.length = 0;
                    this._dateRender = $.merge(this._dateRender, week);
                }
            }
            else if (this.currentView() === "month") {
                this._dateRender = this._monthdates();
            }
            else if (this.currentView() === "customview") {
                if (!ej.isNullOrUndefined(this.model.renderDates)) {
                    var daysRender = this._renderDays <= 7 ? this._renderDays : this._numDays;
                    firstDay = this._renderDays <= 7 ? new Date(this.currentDate()) : this.model.orientation == "vertical" ? this._firstweekdate(this.currentDate()) : new Date(this.currentDate());
                    for (var i = 0; i < daysRender; i++) {
                        newDate = new Date(firstDay);
                        newDate.setDate(firstDay.getDate() + i);
                        this._dateRender.push(new Date(newDate.getFullYear(), newDate.getMonth(), newDate.getDate()).getTime());
                    }
                }
                else
                    this._dateRender = this._monthdates();
            }
            else {
                this._dateRender.push(new Date(new Date(this.currentDate()).setHours(0, 0, 0)));
            }
            return this._dateRender;
        },

        _getCurrentDateHeaderValue: function () {
            var localizedDay = Globalize.culture(this.model.locale);
            var dateValue, value = this._dateRender;
            if (this.currentView() === "day")
                dateValue = localizedDay.calendar.days.names[new Date(value[0]).getDay()].slice(0, 3) + " " + new Date(value[0]).getDate() + " " + Globalize.format(new Date(value[0]), "MMM") + " " + new Date(value[0]).getFullYear();
            else if (this.currentView() === "week" || this.currentView() === "workweek") {
                if (new Date(value[0]).getYear() === new Date(value[value.length - 1]).getYear()) {
                    if (new Date(value[0]).getMonth() === new Date(value[value.length - 1]).getMonth()) {
                        dateValue = Globalize.format(new Date(value[0]), "dd") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                    }
                    else if (new Date(value[0]).getMonth() !== new Date(value[value.length - 1]).getMonth()) {
                        dateValue = this.model.enableRTL ? Globalize.format(new Date(value[value.length - 1]), "MMM dd") + " - " + Globalize.format(new Date(value[0]), "dd MMM yyyy") :
                               Globalize.format(new Date(value[0]), "dd MMM") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                    }
                }
                else {
                    dateValue = Globalize.format(new Date(value[0]), "MMM dd yyyy") + " - " + Globalize.format(new Date(value[value.length - 1]), "MMM dd yyyy");
                }
            }
            else if (this.currentView() === "customview") {
                if (value.length == 1)
                    dateValue = localizedDay.calendar.days.names[new Date(value[0]).getDay()].slice(0, 3) + " " + new Date(value[0]).getDate() + " " + Globalize.format(new Date(value[0]), "MMM") + " " + new Date(value[0]).getFullYear();
                else if (value.length > 1 && value.length <= 7) {
                    if (new Date(value[0]).getYear() === new Date(value[value.length - 1]).getYear()) {
                        if (new Date(value[0]).getMonth() === new Date(value[value.length - 1]).getMonth()) {
                            dateValue = Globalize.format(new Date(value[0]), "dd") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                        }
                        else if (new Date(value[0]).getMonth() !== new Date(value[value.length - 1]).getMonth()) {
                            dateValue = this.model.enableRTL ? Globalize.format(new Date(value[value.length - 1]), "MMM dd") + " - " + Globalize.format(new Date(value[0]), "dd MMM yyyy") :
                                   Globalize.format(new Date(value[0]), "dd MMM") + " - " + Globalize.format(new Date(value[value.length - 1]), "dd MMM yyyy");
                        }
                    }
                    else {
                        dateValue = Globalize.format(new Date(value[0]), "MMM dd yyyy") + " - " + Globalize.format(new Date(value[value.length - 1]), "MMM dd yyyy");
                    }
                }
                else {
                    if (new Date(value[0]).getFullYear() == new Date(value[value.length - 1]).getFullYear()) {
                        if (new Date(value[0]).getMonth() == new Date(value[value.length - 1]).getMonth())
                            dateValue = Globalize.format(new Date(this.currentDate()), "MMM yyyy");
                        else
                            dateValue = Globalize.format(new Date(value[0]), "MMM") + " - " + Globalize.format(new Date(value[value.length - 1]), "MMM yyyy");
                    }
                    else {
                        dateValue = Globalize.format(new Date(value[0]), "MMM yyyy") + " - " + Globalize.format(new Date(value[value.length - 1]), "MMM yyyy");
                    }
                }
            }
            else if (this.currentView() === "agenda") {
                var days = this.model.agendaViewSettings.daysInAgenda;
                if (new Date(value[0]).getYear() === new Date(new Date(value[0]).setDate(value[0].getDate() + days)).getYear()) {
                    if (new Date(value[0]).getMonth() === new Date(new Date(value[0]).setDate(value[0].getDate() + days)).getMonth()) {
                        dateValue = Globalize.format(new Date(value[0]), "dd") + " - " + Globalize.format(new Date(new Date(value[0]).setDate(value[0].getDate() + days)), "dd MMM yyyy");
                    }
                    else if (new Date(value[0]).getMonth() !== new Date(new Date(value[0]).setDate(value[0].getDate() + days)).getMonth()) {
                        dateValue = this.model.enableRTL ? Globalize.format(new Date(new Date(value[0]).setDate(value[0].getDate() + days)), "MMM dd") + " - " + Globalize.format(new Date(value[0]), "dd MMM yyyy") :
                               Globalize.format(new Date(value[0]), "dd MMM") + " - " + Globalize.format(new Date(new Date(value[0]).setDate(value[0].getDate() + days)), "dd MMM yyyy");
                    }
                }
                else
                    dateValue = Globalize.format(new Date(value[0]), "MMM dd yyyy") + " - " + Globalize.format(new Date(new Date(value[0]).setDate(value[0].getDate() + days)), "MMM dd yyyy");
            }
            else
                dateValue = Globalize.format(new Date(this.currentDate()), "MMM yyyy");
            return dateValue;
        },

        _firstweekdate: function (currentDate) {
            var firstdayofweek = new Date(currentDate);
            var day = firstdayofweek.getDay();
            if (this._firstdayofweek == 0) {
                if (day != 0) {
                    firstdayofweek.setDate(firstdayofweek.getDate() - day);
                    if (!this.model.showTimeScale && (this.currentView() == "workweek" || this._oneWeek) && this._resize)
                        firstdayofweek = new Date(this._dateRender[0]);
                }
            }
            else if 
			(this._firstdayofweek != 0 && this.currentDate().getDay() >= this._firstdayofweek) {
                var day = (day - this._firstdayofweek);
                day = firstdayofweek.getDate() - day;
                firstdayofweek.setDate(day);
            }
            else if 
			 (this._firstdayofweek != 0 && this.currentDate().getDay() < this._firstdayofweek) {
                var day = (day - this._firstdayofweek);
                day = (firstdayofweek.getDate() - day) - 7;
                firstdayofweek.setDate(day);
            }
            return firstdayofweek;
        },

        _getHeaderAllDayCellsCount: function () {
            var weekcols = Globalize.culture(this.model.locale);
            var temp, count, columns = [], cols, curday; this.dateRender = []; this._value = []; this.value = [];
            var resCount = 1;
            cols = [];
            if (this.model.orientation == "vertical") {
                count = this._getColumnCount().length;
                if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) {
                    var getCount = this.__getMultipleResourceCellsCount(count);
                    resCount = getCount.resCount;
                }
                for (var i = 0; i < count; i++) {
                    curday = (this.currentView() === "month" || this.currentView() === "day" || (this._isCustomView())) ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()] : weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3) : weekcols.calendar.days.names[new Date(this._dateRender[i]).getDay()].slice(0, 3);
                    var headerCellClass = (new Date(new Date(this._dateRender[i]).setHours(0, 0, 0, 0)).getTime() === new Date(new Date().setHours(0, 0, 0, 0)).getTime()) ? "e-headerToday" : "";
                    temp = (this.currentView() === "month" || (this._isCustomView())) ? { currentDay: curday, currentDateClass: "", cellToday: headerCellClass } : this.currentView() === "day" ? (ej.isNullOrUndefined(this.model.resources) && ej.isNullOrUndefined(this.model.group)) ? { currentDay: curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMMM yyyy"), currentDateClass: "", cellToday: headerCellClass } : { currentDay: curday + " " + Globalize.format(new Date(this._dateRender[i]), "dd MMM yy"), currentDateClass: "", cellToday: headerCellClass } : { currentDay: curday + " " + new Date(this._dateRender[i]).getDate(), currentDateClass: new Date(this._dateRender[i]).getDate() === new Date(this.currentDate()).getDate() ? "e-activeview" : "", cellToday: headerCellClass };
                    columns.push(temp);
                    this._value.push(i);
                }
                for (var i = 0; i < resCount; i++) {
                    this.dateRender = $.merge(this.dateRender, this._dateRender);
                    cols = $.merge(cols, columns);
                    this.value = $.merge(this.value, this._value);
                }
            }
            else
                cols = this._getHorizontalheaderCellCount(weekcols);
            return cols;
        },

        _findResourceIndex: function (data, attr, value) {
            for (var i = 0; i < data.length; i++) {
                if (data[i][attr] === value) {
                    return i;
                }
            }
        },

        _renderHeaderAllDayTemplate: function () {
            if (this.currentView() != "agenda") {
                if (ej.scheduleFeatures.horizontal && this.model.orientation == "horizontal") {
                    if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                        this._renderMultipleResourceHeaderTemplate(null, null);
                    this._renderHorizontalHeaderBar();
                }
                else {
                    var headerDiv = ej.buildTag('div.e-headeralldaydiv', "", {}, { height: "100%" }).css("position", "relative").css("overflow", "hidden");
                    var resCount = 0;
                    var table = ej.buildTag('table.e-table e-headrealldaytable', "", {}, { cellspacing: "0px", cellpadding: "0px", width: "100%" });
                    var tbody = ej.buildTag('tbody');
                    var headTR = ej.buildTag('tr.e-dateheader');
                    var cols = this._getHeaderAllDayCellsCount();
                    var html = this.headTemplate.render({ cols: cols, view: this.currentView(), colspan: 1 });
                    if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) {
                        tbody = this._renderMultipleResourceHeaderTemplate(tbody, cols);
                        resCount = !ej.isNullOrUndefined(this.model.group) ? this.model.group.resources.length : 0;
                    }
                    var tempNavigate = ej.buildTag('div.e-navigate', "", {}, {});
                    var inner_table = ej.buildTag('table.e-table e-headertable', "", {}, { cellspacing: "0px", cellpadding: "0px" });
                    var tb = ej.buildTag('tbody');
                    var headT = ej.buildTag('tr.e-headertr');

                    headTrCount = (this.currentView() == "month") || (this._isCustomView()) ? 1 : 2;
                    var rowCount = resCount + headTrCount; var rows = [];
                    for (var i = 0; i < rowCount; i++) {
                        if (this.currentView() == "month" || (this._isCustomView()))
                            rows.push({ tdClass: (i == rowCount - 1) ? "e-monthleftcells" : "e-resourceheadercells", divClass: (i == rowCount - 1) ? "e-dateheadercell" : "" });
                        else
                            rows.push({ tdClass: (i == rowCount - 1) ? "e-leftindent" : (i == rowCount - 2) ? "e-settingsindent" : "e-resourceheadercells", divClass: (i == rowCount - 1) ? "e-schedulesettings e-textellipsis" : (i == rowCount - 2) ? "e-dateheadercell" : "" });
                    }
                    var widths = (this._mediaQuery) ? this.currentView() == "month" || (this._isCustomView()) ? "50px" : "30px" : this.currentView() == "month" || (this._isCustomView()) ? "50px" : "56px";
                    if (!this.model.showAllDayRow)
                        new ej.DataManager(rows).remove("tdClass", "e-leftindent");
                    var leftIndentHtml = this.leftIndent.render({ trs: rows, cswidth: widths });
                    if (this.model.showTimeScale)
                        this.aTR.append(ej.buildTag('td').append(tempNavigate.append(inner_table.append(tb.append(headT.append(ej.buildTag('td.e-empty', "", {}, { width: (this._mediaQuery) ? (this.currentView() == "month" || (this._isCustomView())) ? "0px" : "30px" : (this.currentView() == "month" || this._isCustomView()) ? "50px" : "56px" }).addClass((this.currentView() == "month" || this._isCustomView()) ? "e-monthempty" : "").append(leftIndentHtml)).append(ej.buildTag('td').append(headerDiv.append(table.append(tbody.append(headTR.append(html)))))).append(ej.buildTag('td', "", {}, { width: (this.currentView() == "month" || this._isCustomView()) ? "0px" : (this._mediaQuery) ? "0px" : "18px" })))))));
                    else
                        this.aTR.append(ej.buildTag('td').append(tempNavigate.append(inner_table.append(tb.append(headT.append(ej.buildTag('td').append(headerDiv.append(table.append(tbody.append(headTR.append(html)))))).append(ej.buildTag('td', "", {}, { width: (this.currentView() == "month" || this._isCustomView()) ? "0px" : (this._mediaQuery) ? "0px" : "18px" })))))));
                    if (this.currentView() !== "month" && !(this._isCustomView())) {
                        if (this.model.showAllDayRow) {
                            var alldayTR = ej.buildTag('tr');
                            var allHtml = this.alldayTemp.render({ cols: cols, colspan: 1 });
                            tbody.append(alldayTR.append(allHtml));
                        }
                    }
                }
            }
            else {
                var tempNavigate = ej.buildTag('div.e-navigate', "", {}, {});
                var inner_table = ej.buildTag('table.e-table e-headertable', "", {}, { cellspacing: "0px", cellpadding: "0px" });
                var tb = ej.buildTag('tbody');
                var headT = ej.buildTag('tr.e-headertr');
                var headerDiv = ej.buildTag('div.e-headeralldaydiv', "", {}, { height: "100%" }).css("position", "relative").css("overflow", "hidden");
                var table = ej.buildTag('table.e-table e-headrealldaytable', "", {}, { cellspacing: "0px", cellpadding: "0px", width: "100%" });
                var tbody = ej.buildTag('tbody tr');
                var colspan = 1;
                var cols = [{ headertext: this._getLocalizedLabels("Date"), className: "e-datecolumn" }, { headertext: this._getLocalizedLabels("Time"), className: "e-timecolumn" }, { headertext: this._getLocalizedLabels("Event"), className: "e-eventcolumn" }];
                if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) {
                    this._renderMultipleResourceHeaderTemplate(null, null);
                    var res = [];
                    for (var resLevel = 0; resLevel < this.render_Resources.length; resLevel++)
                        res.push({ headertext: "", className: "e-resourcecolumn" });
                    (!ej.isNullOrUndefined(this.model.group)) && (cols = $.merge(res, cols));
                }
                var html = this.agendaheadTemplate.render({ cols: cols });
                this.aTR.append(ej.buildTag('td').append(tempNavigate.append(inner_table.append(tb.append(headT.append(ej.buildTag('td').append(headerDiv.append(table.append(tbody.append(html))))).append(ej.buildTag('td.e-emptytd', "", {}, { width: "18px" })))))));
            }
        },

        _monthdates: function () {
            var currentDate = new Date(this.currentDate());
            var header = new Date(currentDate.getFullYear(), currentDate.getMonth(), 1);
            var day = header.getDay() + 1;
            if (this._firstdayofweek && this._firstdayofweek >= 5)
                header.setDate(header.getDate() - day + this._firstdayofweek - 7);
            else if (this._firstdayofweek < 5) {
                header.setDate(header.getDate() - day + this._firstdayofweek);
                if (header.getMonth() == new Date(currentDate.getFullYear(), currentDate.getMonth(), 1).getMonth())
                    header.setDate(new Date(currentDate.getFullYear(), currentDate.getMonth(), 1).getDate() - day + this._firstdayofweek - 7);
            }
            else
                header.setDate(header.getDate() - day);
            //nextmonth days.
            var daysInMonth = new Date(currentDate.getFullYear(), currentDate.getMonth() + 1, 0).getDate();
            if (this._firstdayofweek == 0 || this._firstdayofweek == 5 || this._firstdayofweek == 6) {
                var daysInNextMonth = (day + daysInMonth) % 7 != 0 ? (7 - (day + daysInMonth) % 7) : 0;
                var rows = parseInt((day + daysInMonth + daysInNextMonth) / 7);
            }
            else {
                var daysInNextMonth = 7 - new Date(currentDate.getFullYear(), currentDate.getMonth() + 1, 0).getDay();
                (daysInMonth == 30 || currentDate.getMonth() == 6) ? rows = 6 : rows = 5;
            }
            var dates = [];
            for (var row = 0; row < (rows * 7) ; row++) {
                dates.push(header.setDate(header.getDate() + 1));
            }
            if (new Date(dates[0]).getDate() == new Date(currentDate.getFullYear(), currentDate.getMonth(), 1).getDate()) {
                dates = dates.splice(0, 35);
            }
            return dates;
        },

        _getStartEndTime: function () {
            var timerows, startendTimes = [], timelist;
            var localizeAmPm = Globalize.culture(this.model.locale).calendar;
            if (this.model.orientation === "vertical") {
                if (this.currentView() == "month" || (this._isCustomView())) {
                    var row = this._dateRender.length;
                    for (var i = 0; i < (row / 7) ; i++) {
                        startendTimes.push(i);
                    }
                }
                else {
                    for (var i = this.model.startHour; i < this.model.endHour; i++) {
                        if (this._timeMode == "12") {
                            timelist = ((i === this.model.startHour || i === 12) && (localizeAmPm.AM && localizeAmPm.PM)) ? Globalize.format(new Date(new Date().setHours(i)), "hh tt") : Globalize.format(new Date(new Date().setHours(i)), "hh 00");
                            startendTimes.push({ time: timelist, id: i });
                        }
                        else
                            startendTimes.push({ time: (i <= 9 ? "0" + i : i) + " 00" });
                    }
                }
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                startendTimes = this._getHoriStartEndTime(localizeAmPm);
            return startendTimes;
        },

        _getWorkCellsCount: function () {
            var workcells = [];
            if (this.model.showTimeScale) {
                if (this.currentView() == "month" || (this._isCustomView())) {
                    var row = this._dateRender.length;
                    for (var i = 0; i < (row / 7) ; i++) {
                        workcells.push({ crow: i, cwidth: this.model.cellWidth, cheight: this.model.cellHeight });
                    }
                }
                else {
                    var strEndTime = this._getStartEndTime();
                    for (var i = 0; i < (2 * strEndTime.length) ; i++) {
                        workcells.push({ crow: i, cwidth: this.model.cellWidth, cheight: this.model.cellHeight });
                    }
                }
            }
            else {
                var row = this._dateRender.length;
                for (var i = 0; i < (row / 7) ; i++) {
                    workcells.push({ crow: i, cwidth: this.model.cellWidth, cheight: this.model.cellHeight });
                }
            }
            return workcells;
        },

        _renderContentAreaTemplate: function () {
            var $contentAreaDiv = ej.buildTag('div.e-workareadiv');
            var $contentTable = ej.buildTag('table.e-table e-worktable', "", {}, { height: "100%", cellpadding: "0px", cellspacing: "0px", width: "100%" });
            var $contentBody = ej.buildTag('tbody');
            var $contentTR = ej.buildTag('tr');
            var $WorkCellTD = ej.buildTag('td');
            var $scrollerbarDiv = ej.buildTag('div#' + this._id + "_scroller");
            this.$WorkCellDiv = ej.buildTag('div').css('position', "relative").attr('class', 'e-draggableworkarea').css("height", "100%");
            if (this.currentView() != "agenda") {
                if (ej.scheduleFeatures.horizontal && this.model.orientation == "horizontal")
                    this._renderHorizontalContent($contentAreaDiv, $contentTable, $contentBody, $contentTR, $WorkCellTD, $scrollerbarDiv);
                else {
                    var timeCellClassName = this.currentView() === "month" || (this._isCustomView()) ? "e-monthleftindent" : "e-workleftindent";
                    this.$TimeCellTD = ej.buildTag('td', "", {}, {}).addClass(timeCellClassName);
                    var colspan = [], index;
                    var cols = this.currentView() === "month" || (this._isCustomView()) ? 7 : this._getRenderDates().length;
                    var timeCellhtml;
                    var viewclass = this.currentView() == "month" || (this._isCustomView()) ? 1 : 0;
                    timeCellhtml = this.timeCellTemplate.render({ timerows: this._getStartEndTime(), cols1: [1, 2], viewclass: viewclass });
                    var workCellhtml;
                    if (this.currentView() !== "month" && !(this._isCustomView()) && this.model.showTimeScale)
                        workCellhtml = this.workcellTemp.render({ cellrows: this._getWorkCellsCount(), cols1: this.value, orientation: this.model.orientation });
                    else
                        workCellhtml = this.monthcellTemp.render({ cellrows: this._getWorkCellsCount(), cols1: this.value, orientation: this.model.orientation });
                    if (this.model.showTimeScale)
                        this.wTR.append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append(this.$TimeCellTD.append(timeCellhtml)).append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv.append(workCellhtml)))))))));
                    else
                        this.wTR.append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv.append(workCellhtml)))))))));
                }
            }
            else
                this.wTR.append(ej.buildTag('td').append($contentAreaDiv.append($contentTable.append($contentBody.append($contentTR.append($WorkCellTD.append($scrollerbarDiv.append(this.$WorkCellDiv))))))));
        },

        _renderAgendaContent: function () {
            this.element.find(".e-agendacellstab").remove();
            var dateCount = this._dateRender;
            var row = []; var col = []; this._app = [];
            var startTimeDisp, endTimeDisp, date, time, subject, colorDiv, resName, userAppTemplId, userTempHtml, userDateId, userDateHtml, userTimeId, userTimeHtml, userResourceId, userResourceHtml;
            if (!this.model.group) this.res1 = ["0"];
            var repeatText = this._getLocalizedLabels("Recurrence");
            var count = this.model.agendaViewSettings.daysInAgenda;
            for (var res = 0; res < this.res1.length; res++) {
                var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(dateCount[0]));
                predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(new Date(dateCount[0]).setDate(dateCount[0].getDate() + count)).setHours(23, 59, 59)));
                predicate = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
                var appList = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(predicate));
                this._app = $.merge(this._app, appList);
                if (appList.length > 0) {
                    var resGroup = [];
                    if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) {
                        var resgroupid = this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings["groupId"]];
                        var resLevels = this._resCollection;
                        for (var i = resLevels.length - 1; i > 0; i--) {
                            var resApp = new ej.DataManager(resLevels[i - 1].dataSource).executeLocal(new ej.Query().where(resLevels[i - 1].id, ej.FilterOperators.equal, resgroupid));
                            resGroup.push({ resId: "resLevel_" + i + "_" + resApp[0][resLevels[i - 1].id], text: resApp[0][resLevels[i - 1].text] });
                            resgroupid = resApp[0][resLevels[i - 1].groupId];
                        }
                        resGroup.reverse();
                    }
                    var resChild = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? true : false;
                    var resRowSpan = appList.length;
                    userResourceId = (this.model.resourceHeaderTemplateId) ? true : false;
                    userResourceHtml = userResourceId ? this._getResourceHeadTemplate(this.res1[res]) : "";
                    var day = 0;
                    if (new Date(dateCount[0]) < this.model.minDate)
                        day = Math.floor((new Date(this.model.minDate) - new Date(dateCount[0])) / (1000 * 60 * 60 * 24));
                    if (new Date(new Date(dateCount[0]).setDate(dateCount[0].getDate() + count)) > this.model.maxDate)
                        count = count - Math.floor((new Date(new Date(dateCount[0]).setDate(dateCount[0].getDate() + count)) - new Date(this.model.maxDate)) / (1000 * 60 * 60 * 24));
                    for (var day; day <= count; day++) {
                        var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(new Date(dateCount[0]).setDate(dateCount[0].getDate() + day)));
                        predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(new Date(dateCount[0]).setDate(dateCount[0].getDate() + day)).setHours(23, 59, 59)));
                        predicate = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
                        var dailyapplist = new ej.DataManager(appList).executeLocal(new ej.Query().where(predicate));
                        if (dailyapplist.length > 0) {
                            row.push(row.length + 1);
                            var dayRowSpan = dailyapplist.length; var dayChild = true; var dayBorderBottom = false;
                            for (var i = 0; i < dailyapplist.length; i++) {
                                if (i == dailyapplist.length - 1) dayBorderBottom = true;
                                startTimeDisp = this._timeMode == "12" ? Globalize.format(dailyapplist[i][this.model.appointmentSettings["startTime"]], this._pattern.t) : Globalize.format(dailyapplist[i][this.model.appointmentSettings["startTime"]], "HH:mm");
                                endTimeDisp = this._timeMode == "12" ? Globalize.format(dailyapplist[i][this.model.appointmentSettings["endTime"]], this._pattern.t) : Globalize.format(dailyapplist[i][this.model.appointmentSettings["endTime"]], "HH:mm");
                                date = "<strong class='e-agendaday'>" + Globalize.format(new Date(dailyapplist[i][this.model.appointmentSettings['startTime']]), 'dd') + "</strong><span class='e-agendaweek'>" + Globalize.format(new Date(dailyapplist[i][this.model.appointmentSettings['startTime']]), 'dddd') + "</span><span class='e-agendamonth'>" + Globalize.format(new Date(dailyapplist[i][this.model.appointmentSettings['startTime']]), 'MMM, yyyy') + "</span>";
                                time = dailyapplist[i][this.model.appointmentSettings["allDay"]] ? "<div class='e-icon e-scheduleallday'></div>" + this._getLocalizedLabels("AllDay") : startTimeDisp + " - " + endTimeDisp;
                                subject = dailyapplist[i][this.model.appointmentSettings["recurrence"]] ? "<span class='e-icon e-schedulerepeat' title='" + repeatText + "'/>" + dailyapplist[i][this.model.appointmentSettings["subject"]] : dailyapplist[i][this.model.appointmentSettings["subject"]];
                                colorDiv = (this.model.categorizeSettings.enable && !ej.isNullOrUndefined(dailyapplist[i][this.model.appointmentSettings["categorize"]]) && dailyapplist[i][this.model.appointmentSettings["categorize"]] != "") ? this._getCategorizeColor(dailyapplist[i]) : (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) ? this._getResourceColor(dailyapplist[i]) : { appointData: "", appointCustomcss: "", appointtextcolor: "" };
                                resName = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings["text"]] : "";
                                userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                                userTempHtml = userAppTemplId ? this._getUserAppointmentTemplate(dailyapplist[i]) : "";
                                userDateId = (this.model.agendaViewSettings.dateColumnTemplateId) ? true : false;
                                userDateHtml = userDateId ? this._getUserTemplate(this.model.agendaViewSettings.dateColumnTemplateId, dailyapplist[i]) : "";
                                userTimeId = (this.model.agendaViewSettings.timeColumnTemplateId) ? true : false;
                                userTimeHtml = userTimeId ? this._getUserTemplate(this.model.agendaViewSettings.timeColumnTemplateId, dailyapplist[i]) : "";
                                col.push({ readOnly: this.model.readOnly, edittext: this._getLocalizedLabels("Edit"), deletetext: this._getLocalizedLabels("Delete"), userResourceId: userResourceId, userResourceHtml: userResourceHtml, userDateId: userDateId, userDateHtml: userDateHtml, userTimeId: userTimeId, userTimeHtml: userTimeHtml, userAppTemplId: userAppTemplId, userTemplate: userTempHtml, resGroup: resGroup, resName: resName, resChild: resChild, resRowSpan: resRowSpan, id: dailyapplist[i]["AppTaskId"], uid: dailyapplist[i].Guid, dayRowSpan: dailyapplist.length, dayChild: dayChild, dayBorderBottom: dayBorderBottom, background: colorDiv.appointData, date: date, time: time, event: subject });
                                dayChild = false; resChild = false;
                            }
                        }
                    }
                }
            }
            var workCellhtml = this.agendacellTemp.render({ rows: row, cols: col });
            this.element.find(".e-draggableworkarea").append(workCellhtml);
            if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) {
                var resLevels = this.render_Resources;
                for (var i = 0; i < resLevels.length - 1; i++) {
                    for (var j = 0; j < resLevels[i].length; j++) {
                        var cname = "resLevel_" + (i + 1) + "_" + resLevels[i][j][this._resCollection[i].id];
                        this.element.find("." + cname).first().attr('rowspan', this.element.find("." + cname).length);
                        this.element.find("." + cname).slice(1).addClass("e-hide");
                        this.element.find(".e-hide").remove();
                        (this.model.resourceHeaderTemplateId) && (this.element.find("." + cname).children().empty().append(this._getResourceHeadTemplate(resLevels[i][j])));
                    }
                }
            }
            var _scrollDiv = $('div#' + this._id + "_scroller");
            if (ej.isNullOrUndefined(_scrollDiv.data("ejScroller"))) {
                _scrollDiv.ejScroller({
                    enableRTL: this.model.enableRTL, height: this.scHeight - (this.element.find('.e-draggableworkarea').offset().top - this.element.offset().top) - 1, width: this.element.width() - 2, cssClass: this.model.cssClass
                });
                (!this._mediaQuery) && this._nextPrevAppointment(this.model.enableAppointmentNavigation);
            }
            _scrollDiv.ejScroller('refresh');
            _scrollDiv.ejScroller("isVScroll") ? (this.element.find(".e-emptytd").show() && this.element.find('.e-draggableworkarea').height(this.element.find('.e-draggableworkarea').height() + 1)) : this.element.find(".e-emptytd").hide();
            this.element.find(".e-vscroll").css("left", (this.model.enableRTL) ? 0 : this.element.outerWidth() - (_scrollDiv.ejScroller("isVScroll") ? 18 : 0));
            this.element.find(".e-mobileNavigation").css("left", (this.model.enableRTL) ? (_scrollDiv.ejScroller("isVScroll") ? 18 : 0) : this.element.outerWidth() - 36);
            this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());
            (this.model.enableAppointmentNavigation) && this._nextPrevButton(this._app);
        },

        _getUserTemplate: function (id, record) {
            var cloneObj = $.extend(cloneObj, record);
            cloneObj.View = this.currentView();
            return $.templates($(id).html()).render(cloneObj);
        },

        _getColumnCount: function () {
            var weekcells = [];
            var daysCount = (this.currentView() === "month" || this._isCustomView()) ? this._isCustomView() ? this._renderDays <= 7 ? this._renderDays : 7 : 7 : this._getRenderDates().length;
            for (var i = 0; i < daysCount; i++) weekcells.push(i);
            return weekcells;
        },

        _bindAppointmentsData: function (currentAction) {
            if (!ej.isNullOrUndefined(this.model.appointmentSettings) && this.dataSource() != null) {
                if (this.model.enablePersistence) {
                    this.model.appointmentSettings.query = new ej.Query();
                    this._enablePersist = true;
                }
                if (!this.dataSource() instanceof ej.DataManager && this.dataSource().dataSource.offline)
                    this._dateFormat(this.dataSource());
                if (this.dataSource() instanceof ej.DataManager && !this.dataSource().dataSource.offline) {
                    this._initDataSource(currentAction);
                }
                else {
                    this._angular = (!this._bindapp && !this._setmodelvalue && this._viewchange == true && this.dataSource()[0][this.model.appointmentSettings["id"]] == this._obj[0][this.model.appointmentSettings["id"]]);
                    this._bindAppointments = (this.dataSource().length > 0) ? true : false;
                    this._true = this.model.isDST == true ? true : false;
                    if (ej.isNullOrUndefined(this.model.timeZone) && this.model.isDST == false)
                        this._timeZoneOffset = new Date().getTimezoneOffset() - this._stdTimezoneOffset();
                    else
                        this._timeZoneOffset = 0;
                    this._currentAppointmentData = this.dataSource();
                    $.extend(true, this._obj = [], this.dataSource());
                    !ej.isNullOrUndefined(this.dataSource().dataSource) && this.dataSource().dataSource.json && (this._currentAppointmentData = this.dataSource().dataSource.json);
                    this._dataCount = this._currentAppointmentData.length;
                    this._dataProcessing(this._currentAppointmentData);
                    this._renderAppointmentAll();
                    if (this._app.length < 1 && this.model.enableAppointmentNavigation)
                        this._nextPrevButton(this._app);
                    this.tzone = this.model.timeZone;
                    this._bindapp = false;
                    this._setmodelvalue = false;
                    this._viewchange = false;
                }
            }
        },

        _initDataSource: function (currentAction) {
            var query = this._columnToSelect(this.model.appointmentSettings, currentAction);
            var tempQuery=$.extend(true,[],query._params);
            query._params = [];
            query.addParams("CurrentDate", this.currentDate()).addParams("CurrentView", this.currentView()).addParams("CurrentAction", currentAction);
            for (var i = 0; i < tempQuery.length; i++) {
                if (tempQuery[i].key != "CurrentDate" && tempQuery[i].key != "CurrentView" && tempQuery[i].key != "CurrentAction") {
                    query.addParams(tempQuery[i].key, tempQuery[i].value);
                }
            }
            var proxy = this;
            var queryPromise = this.dataSource().executeQuery(query);
            queryPromise.done(function (e) {
                var today = new Date();
                proxy._true = proxy.model.isDST == true ? true : false;
                if (ej.isNullOrUndefined(proxy.model.timeZone) && proxy.model.isDST == false)
                    proxy._timeZoneOffset = today.getTimezoneOffset() - proxy._stdTimezoneOffset();
                else
                    proxy._timeZoneOffset = 0;
                proxy._dataCount = e.result.length;
                if (!ej.isNullOrUndefined(e.result.result)) {
                    proxy._dataCount = e.result.count + 1;
                    e.result = e.result.result;
                }
                proxy._bindAppointments = (e.result.length > 0) ? true : false;
                proxy._currentAppointmentData = e.result;
                proxy._overlapList = [];
                proxy._dataProcessing(e.result);
                proxy._renderAppointmentAll();
                if (proxy._app.length < 1 && proxy.model.enableAppointmentNavigation)
                    proxy._nextPrevButton(proxy._app);
            });
        },

        _bindCategorizeData: function () {
            if (!ej.isNullOrUndefined(this.model.categorizeSettings)) {
                this._categoryInfo = [];
                if (ej.isNullOrUndefined(this.model.categorizeSettings.query) || !(this.model.categorizeSettings.query instanceof ej.Query))
                    this.model.categorizeSettings.query = ej.Query();
                if (this.model.categorizeSettings.dataSource instanceof ej.DataManager)
                    this._initCategorizeData(this.model.categorizeSettings);
                else {
                    this._categoryInfo.push(this.model.categorizeSettings);
                }
            }
        },

        _initCategorizeData: function (fields, i) {
            var query = this._columnToSelect(fields);
            var proxy = this;
            var queryCategoryPromise = fields.dataSource.executeQuery(query);
            queryCategoryPromise.done(function (e) {
                fields.dataSource = e.result;
                proxy._categoryInfo.push(fields);
                proxy._renderCategoryItems(proxy._appointmentAddWindow);
            });
        },

        _bindPriorityData: function () {
            if (!ej.isNullOrUndefined(this.model.prioritySettings)) {
                this._priorityInfo = [];
                if (ej.isNullOrUndefined(this.model.prioritySettings.query) || !(this.model.prioritySettings.query instanceof ej.Query))
                    this.model.prioritySettings.query = ej.Query();
                if (this.model.prioritySettings.dataSource instanceof ej.DataManager)
                    this._initPriorityData(this.model.prioritySettings);
                else {
                    this._priorityInfo.push(this.model.prioritySettings);
                }
            }
        },

        _initPriorityData: function (fields, i) {
            var query = this._columnToSelect(fields);
            var proxy = this;
            var queryPriorityPromise = fields.dataSource.executeQuery(query);
            queryPriorityPromise.done(function (e) {
                fields.dataSource = e.result;
                proxy._priorityInfo.push(fields);
                proxy._renderPriorityItems(proxy._appointmentAddWindow);
            });
        },

        _dateFormat: function (appointments) {
            if (new Date(appointments[0][this.model.appointmentSettings["startTime"]]).toString() == "Invalid Date") {
                for (var i = 0; i < appointments.length; i++) {
                    if (new Date(appointments[i][this.model.appointmentSettings["startTime"]]).toString() == "Invalid Date" || new Date(appointments[i][this.model.appointmentSettings["startTime"]]).toString() == "NaN")
                        appointments[i][this.model.appointmentSettings["startTime"]] = new Date(parseInt(appointments[i][this.model.appointmentSettings["startTime"]].match(/\d+/).toString()));
                    else
                        appointments[i][this.model.appointmentSettings["startTime"]] = new Date(appointments[i][this.model.appointmentSettings["startTime"]]);
                    if (new Date(appointments[i][this.model.appointmentSettings["endTime"]]).toString() == "Invalid Date")
                        appointments[i][this.model.appointmentSettings["endTime"]] = new Date(parseInt(appointments[i][this.model.appointmentSettings["endTime"]].match(/\d+/).toString()));
                    else
                        appointments[i][this.model.appointmentSettings["endTime"]] = new Date(appointments[i][this.model.appointmentSettings["endTime"]]);
                }
            }
        },

        _columnToSelect: function (mapper, currentAction) {
            var column = [], queryManager = ej.Query();
            if (!ej.isNullOrUndefined(mapper.tableName) && !this.dataSource().dataSource.url.match(mapper.tableName + "$")) {
                queryManager = mapper.query;
                queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },

        _getColumns: function (mapper) {
            var column = []
            for (var col in mapper) {
                if (col !== "tableName" && col !== "query" && col !== "dataSource")
                    !ej.isNullOrUndefined(mapper[col]) && column.push(mapper[col]);
            }
            return column;
        },

        _sortAppById: function (appointments) {

            var proxy = this;
            appointments.sort(function (a, b) {
                var x = a["AppTaskId"]; var y = b["AppTaskId"];
                return ((x < y) ? -1 : ((x > y) ? 1 : 0));
            });
            return appointments;
        },
        _stdTimezoneOffset: function () {
            var today = new Date();
            var jan = new Date(today.getFullYear(), 0, 1);
            var jul = new Date(today.getFullYear(), 6, 1);
            return Math.max(jan.getTimezoneOffset(), jul.getTimezoneOffset());
        },
        _timeZoneCalculate: function (appointment) {
            var localutc = new Date().getTimezoneOffset();
            var utcdiff;
            if (this.model.timeZone == null)
                obj = (this.model.isDST == false) ? -localutc + this._timeZoneOffset : -localutc;
            else {
                if (this.model.timeZone.indexOf(':') !== -1) {
                    (ej.isNullOrUndefined(this.tzone)) && (this.tzone = this.model.timeZone);
                    var timezone = this.model.timeZone.indexOf("UTC") != -1 ? this.model.timeZone[3] !== " " ? this.model.timeZone.split(":") : this.model.timeZone.split(" ")[1].split(":") : this.model.timeZone.split(":");
                    var oldtimezone = this.tzone.indexOf("UTC") != -1 ? this.tzone[3] !== " " ? this.tzone.split(":") : this.tzone.split(" ")[1].split(":") : this.tzone.split(":");
                    if (($.isNumeric(parseInt(timezone[0])) && (parseInt(timezone[0])).toString.length <= 2) || timezone[0].indexOf("UTC") !== -1) {
                        var hour = this.model.timeZone.indexOf("-") !== -1 ? -parseInt(timezone[0].replace(/\D/g, '')) : parseInt(timezone[0].replace(/\D/g, ''));
                        var mins = parseInt(timezone[1]);
                        var modelutc = -((hour * 60) + mins);
                        if (this._setmodelvalue == true) {
                            if (this.tzone == this.model.timeZone) {
                                utcdiff = -localutc + modelutc;
                            }
                            else {
                                var oldhour = this.tzone.indexOf("-") !== -1 ? -parseInt(oldtimezone[0].replace(/\D/g, '')) : parseInt(oldtimezone[0].replace(/\D/g, ''));
                                var oldmins = parseInt(oldtimezone[1]);
                                var oldmodelutc = -((oldhour * 60) + oldmins);
                                utcdiff = -oldmodelutc + modelutc;
                            }
                            obj = -utcdiff;
                        }
                        else {
                            obj = -modelutc;
                        }
                    }
                    else {
                        obj = -localutc;
                    }
                }
                else {
                    obj = -localutc;
                }
            }
            return obj;
        },
        _appointTimeZone: function (appoint) {
            var startUtc = appoint[this.model.appointmentSettings["startTimeZone"]];
            var endUtc = appoint[this.model.appointmentSettings["endTimeZone"]];

            appoint.EndTimeZone = endUtc = ej.isNullOrUndefined(endUtc) && !ej.isNullOrUndefined(startUtc) ? startUtc : endUtc;
            appoint.StartTimeZone = startUtc = !ej.isNullOrUndefined(endUtc) && ej.isNullOrUndefined(startUtc) ? endUtc : startUtc;

            if (ej.isNullOrUndefined(startUtc) && appoint[this.model.appointmentSettings["startTime"]]) {
                var startTime = appoint[this.model.appointmentSettings["startTime"]];
                var endTime = appoint[this.model.appointmentSettings["endTime"]];
                startTime = appoint[this.model.appointmentSettings["startTime"]] + this._timeZoneCalculate(appoint);
                endTime = appoint[this.model.appointmentSettings["endTime"]] + this._timeZoneCalculate(appoint);

            }
            else if (ej.isNullOrUndefined(startUtc) || (startUtc == "" && endUtc == ""))
                startUtc = 0;
            else {
                startUtc = startUtc.indexOf("UTC") != -1 ? parseInt(startUtc.split(" ")[1].split(":")[0]) * 60 + parseInt(startUtc.split(" ")[1].split(":")[1]) : parseInt(startUtc.split(":")[0]) * 60 + parseInt(startUtc.split(":")[1]);
                endUtc = endUtc.indexOf("UTC") != -1 ? parseInt(endUtc.split(" ")[1].split(":")[0]) * 60 + parseInt(endUtc.split(" ")[1].split(":")[1]) : parseInt(endUtc.split(":")[0]) * 60 + parseInt(endUtc.split(':')[1]);
            }
            return startUtc;
        },
        _timeZoneAppointments: function (appoint, app) {
            if (!appoint[this.model.appointmentSettings["allDay"]]) {
                var tempstarttime, tempendtime;
                if (ej.isNullOrUndefined(this._dayLight)) {
                    localutc = new Date().getTimezoneOffset();
                    var impValue = this._appointTimeZone(appoint);
                    var difference = (ej.isNullOrUndefined(impValue) || impValue == 0 && (this.model.appointmentSettings["startTimeZone"] == null && this.model.appointmentSettings["endTimeZone"] == null)) ? this._timeZoneCalculate(appoint) : impValue - this._timeZoneCalculate(appoint);
                }
                else
                    var difference = this._timeValue;
                difference = (app == "") ? -difference : difference;
                tempstarttime = new Date(appoint[this.model.appointmentSettings["startTime"]]);
                tempendtime = new Date(appoint[this.model.appointmentSettings["endTime"]]);
                if (appoint[this.model.appointmentSettings["allDay"]] && (Math.round((new Date(appoint[this.model.appointmentSettings["endTime"]]) - new Date(appoint[this.model.appointmentSettings["startTime"]])) / 3600000) < 24)) {
                    return appoint;
                }
                else {
                    appoint[this.model.appointmentSettings["startTime"]] = new Date(tempstarttime.setMinutes(tempstarttime.getMinutes() + difference));
                    appoint[this.model.appointmentSettings["endTime"]] = new Date(tempendtime.setMinutes(tempendtime.getMinutes() + difference));
                    return appoint;
                }
            }
            return appoint;
        },

        _dataProcessing: function (appointments) {
            this._processed = []; 
            if (appointments.length > 0) {
                var appId = this._sortAppById(appointments);
                this._maxId = (ej.isNullOrUndefined(appointments[0]) || ej.isNullOrUndefined(appointments[0].AppTaskId)) ? 1 : appointments[0].AppTaskId;
                this.app = true;
                var recurEndDate;
                for (var app = 0; app < appointments.length; app++) {
                    if (this._bindAppointments == true && !this._angular && ej.isNullOrUndefined(this._enablePersist)) {
                        var appoint = this._timeZoneAppointments(appointments[app], "reverse");
                        this._appointmentProcessing(appoint);
                    }
                    else
                        this._appointmentProcessing(appointments[app]);
                }
                this._bindAppointments = false;
                this._processed = this._sortAppById(this._processed);
                return this._processed;
            }
        },
        _guidFormatGenerate: function () {
            function guid() {
                return Math.floor((1 + Math.random()) * 0x10000)
                  .toString(16)
                  .substring(1);
            }
            return guid() + guid() + '-' + guid() + '-' + guid() + '-' +
              guid() + '-' + guid() + guid() + guid();
        },
        _RecurrenceAppointment: function (appointment) {
            var day_start = new Date(appointment[this.model.appointmentSettings["startTime"]]);
            var duration = ((new Date(appointment[this.model.appointmentSettings["endTime"]]) - new Date(appointment[this.model.appointmentSettings["startTime"]])) / 1000);
            var hours = Math.floor(duration / 3600);
            var minutes = Math.floor((duration % 3600) / 60);
            this._rRule = {}, _count = 0, this._filt = true, _lastDay = false;
            appointment.AppTaskId = ej.isNullOrUndefined(appointment.AppTaskId) ? this._maxId : appointment.AppTaskId;
            (!ej.isNullOrUndefined(appointment[this.model.appointmentSettings["recurrence"]])) && (!ej.isNullOrUndefined(appointment[this.model.appointmentSettings["recurrenceRule"]])) && this.recurrenceRuleSplit(appointment[this.model.appointmentSettings["recurrenceRule"]], appointment[this.model.appointmentSettings["recurrenceExDate"]]);

            if (appointment[this.model.appointmentSettings["recurrence"]] == 1 && ej.isNullOrUndefined(this._rRule.recurEditId)) {
                //appointment.ParentId = appointment[this.model.appointmentSettings["id"]];
                var byDay = "", _interval = ej.isNullOrUndefined(this._rRule.interval) ? 1 : this._rRule.interval;
                !ej.isNullOrUndefined(this._rRule.weekDays) && (byDay = this._rRule.weekDays.split(','));
                if (!ej.isNullOrUndefined(this._rRule.until)) {
                    recurEndDate = new Date(this._rRule.until);
                    recurEndDate.setDate(recurEndDate.getDate() + 1);
                }
                else if (!ej.isNullOrUndefined(this._rRule.count))
                    switch (this._rRule.freq) {
                        case "DAILY":
                            recurEndDate = new Date(new Date(day_start).setDate(day_start.getDate() + this._rRule.count * (_interval)));
                            break;
                        case "WEEKLY":
                            var dayCount = day_start.getDate() + (((this._rRule.count / byDay.length) * 7)) * (_interval);
                            recurEndDate = new Date(new Date(day_start).setDate(Math.round(dayCount) < dayCount ? dayCount + 1 : dayCount));
                            break;
                        case "MONTHLY":
                            recurEndDate = new Date(new Date(day_start).setMonth(day_start.getMonth() + this._rRule.count * (_interval)));
                            break;
                        case "YEARLY":
                            recurEndDate = new Date(new Date(day_start).setFullYear(day_start.getFullYear() + this._rRule.count * (_interval)));
                            break;
                    }
                else
                    recurEndDate = new Date(new Date(this.currentDate()).setDate(day_start.getDate() + 42 * (_interval)));

                switch (this._rRule.freq) {
                    case "WEEKLY":
                        byDay = this._rRule.weekDays.split(',');
                        if (byDay.indexOf(this._dayNames[new Date(day_start).getDay()]) == -1)
                            day_start = this._weeklyDayFind(day_start, byDay);
                        break;
                    case "MONTHLY":
                        if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                            var _lastDay = new Date(day_start).getDate() == new Date(day_start.getFullYear(), day_start.getMonth() + 1, 0).getDate() ? true : false;
                            while (new Date(day_start).getDate() != this._rRule.monthDay) {
                                day_start = new Date(day_start.setDate(day_start.getDate() + 1));
                            }
                        }
                        else
                            day_start = new Date((this._getWeekIndex(day_start) + 1) > this._rRule.setPositions && this._rRule.setPositions != -1 ? new Date(day_start.getFullYear(), day_start.getMonth() + 1, 1) : day_start);
                        break;
                    case "YEARLY":
                        if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                            day_start = (day_start.getMonth() + 1 >= this._rRule.month) && day_start.getDate() > this._rRule.monthDay ? new Date(day_start.getFullYear() + 1, this._rRule.month - 1, this._rRule.monthDay) : new Date(day_start.getFullYear(), this._rRule.month - 1, this._rRule.monthDay);
                        }
                        else if (!ej.isNullOrUndefined(this._rRule.setPositions)) {
                            day_start = new Date(day_start.getFullYear(), this._rRule.month - 1, day_start.getDate());
                            day_start = new Date((this._getWeekIndex(day_start) + 1) > this._rRule.setPositions ? new Date(day_start.getFullYear(), day_start.getMonth() + 1, 1) : day_start);
                        }
                        break;
                }
                while (day_start <= recurEndDate) {
                    var orgStrTime = appointment[this.model.appointmentSettings["startTime"]];
                    var newAppointment = {}, orgEndTime = appointment[this.model.appointmentSettings["endTime"]];
                    ((this._rRule.freq == "MONTHLY" || this._rRule.freq == "YEARLY") && !ej.isNullOrUndefined(this._rRule.setPositions)) && (day_start = this._dayOfWeekInMonth(day_start, this._rRule.weekDays, this._rRule.setPositions));
                    var strTime = new Date(day_start.getFullYear(), day_start.getMonth(), day_start.getDate(), orgStrTime.getHours(), orgStrTime.getMinutes(), 0);
                    var endTime = new Date(day_start.getFullYear(), day_start.getMonth(), day_start.getDate(), orgStrTime.getHours() + hours, orgStrTime.getMinutes() + minutes, 0);
                    this._idChecking();
                    appointment[this.model.appointmentSettings["recurrenceId"]] = appointment.AppTaskId;
                    appointment["ParentId"] = appointment.AppTaskId;
                    appointment[this.model.appointmentSettings["recurrenceExDate"]] = appointment[this.model.appointmentSettings["recurrenceExDate"]] == null ? null : appointment[this.model.appointmentSettings["recurrenceExDate"]];
                    if (!ej.isNullOrUndefined(this._rRule.exDate)) {
                        if (this._rRule.exDate.indexOf(Globalize.format(new Date(day_start), this._pattern.d)) == -1) {
                            newAppointment = this._addAppData(appointment, this._maxId, strTime, endTime, appointment.AppTaskId);
                        }
                    }
                    else
                        newAppointment = this._addAppData(appointment, this._maxId, strTime, endTime, appointment.AppTaskId);
                    switch (this._rRule.freq) {
                        case "DAILY":
                            day_start = new Date(day_start.setDate(day_start.getDate() + _interval));
                            break;
                        case "WEEKLY":
                            var curDay = this._dayNames[new Date(day_start).getDay()];
                            if (byDay.length - 1 != byDay.indexOf(curDay)) {
                                day_start = this._weeklyDayFind(day_start, byDay);
                            }
                            else {
                                day_start = new Date(day_start.setDate(day_start.getDate() + (day_start.getDay() * -1)));
                                day_start = new Date(day_start.setDate(day_start.getDate() + (_interval * 7) - 1));
                                day_start = this._weeklyDayFind(day_start, byDay);
                            }
                            break;
                        case "MONTHLY":
                            day_start = _lastDay ? new Date(day_start.getFullYear(), (day_start.getMonth() + _interval) + 1, 0) :
                                ej.isNullOrUndefined(this._rRule.setPositions) ? new Date(day_start.setMonth(day_start.getMonth() + _interval)) : new Date(day_start.getFullYear(), day_start.getMonth() + _interval, 1);
                            break;
                        case "YEARLY":
                            day_start = new Date(day_start.setFullYear(day_start.getFullYear() + _interval));
                            break;
                    }

                    if ((new Date(new Date(appointment[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(appointment[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((appointment[this.model.appointmentSettings["endTime"]] - appointment[this.model.appointmentSettings["startTime"]]) / 3600000) < 24) && (appointment[this.model.appointmentSettings["allDay"]] !== true)) {
                        newAppointment.Guid = this._guidFormatGenerate();
                        this._multipleDayApp(newAppointment, this._maxId, newAppointment[this.model.appointmentSettings["startTime"]], endTime);
                    }
                    else if (Math.round((newAppointment[this.model.appointmentSettings["endTime"]] - newAppointment[this.model.appointmentSettings["startTime"]]) / 3600000) > 24) {
                        newAppointment.Guid = this._guidFormatGenerate();
                        this._multipleSplit(newAppointment, this._maxId, newAppointment[this.model.appointmentSettings["startTime"]], newAppointment[this.model.appointmentSettings["endTime"]]);
                    }
                    else {
                        if (!$.isEmptyObject(newAppointment)) {
                            newAppointment.Guid = this._guidFormatGenerate();
                            this._processed.push(newAppointment);
                        }
                    }
                    this._maxId++;
                    //appointment.AppTaskId++;
                    _count++;
                    if (!ej.isNullOrUndefined(this._rRule.count) && _count == this._rRule.count)
                        break;
                }
            }
            this._filt = false;
        },
        _appointmentProcessing: function (appointment) {
            this._maxId = !ej.isNullOrUndefined(this._appMainId) ? parseInt(this._appMainId) + 1 : parseInt(this._maxId);
            var day_start = new Date(appointment[this.model.appointmentSettings["startTime"]]);
            var duration = ((new Date(appointment[this.model.appointmentSettings["endTime"]]) - new Date(appointment[this.model.appointmentSettings["startTime"]])) / 1000);
            var hours = Math.floor(duration / 3600);
            var minutes = Math.floor((duration % 3600) / 60);
            this._rRule = {}, _count = 0, _lastDay = false;
            var newAppointment = {}
            var newAppointment = $.extend(true, {}, appointment);
            if (appointment[this.model.appointmentSettings["recurrence"]] == 1 && ej.isNullOrUndefined(this._rRule.recurEditId)) {
                this._RecurrenceAppointment(appointment);
            }
            if ((new Date(new Date(appointment[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(appointment[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((appointment[this.model.appointmentSettings["endTime"]] - appointment[this.model.appointmentSettings["startTime"]]) / 3600000) < 24) && (appointment[this.model.appointmentSettings["allDay"]] !== true) && ((appointment[this.model.appointmentSettings["recurrence"]] && !ej.isNullOrUndefined(this._rRule.recurEditId)) || !appointment[this.model.appointmentSettings["recurrence"]])) {         
                appointment[this.model.appointmentSettings["recurrenceId"]] = appointment[this.model.appointmentSettings["recurrenceId"]] == null ? null : appointment[this.model.appointmentSettings["recurrenceId"]];
                appointment[this.model.appointmentSettings["recurrenceExDate"]] = appointment[this.model.appointmentSettings["recurrenceExDate"]] == null ? null : appointment[this.model.appointmentSettings["recurrenceExDate"]];
                appointment.AppTaskId = appointment.AppTaskId == null ? this._maxId : appointment.AppTaskId;
                ej.isNullOrUndefined(appointment.ParentId) && (appointment.ParentId = appointment.AppTaskId);
                appointment.Guid = ej.isNullOrUndefined(appointment.Guid) ? this._guidFormatGenerate() : appointment.Guid;
                this._maxId = appointment.AppTaskId;
                this.app = false;
                this._multipleDayApp(appointment, this._maxId, appointment[this.model.appointmentSettings["startTime"]], appointment[this.model.appointmentSettings["endTime"]]);
            }
            else if ((((appointment[this.model.appointmentSettings["endTime"]] - appointment[this.model.appointmentSettings["startTime"]]) / 3600000) > 24)) {
                if (((appointment[this.model.appointmentSettings["allDay"]] === true) || ((appointment[this.model.appointmentSettings["allDay"]] !== true) && ((appointment[this.model.appointmentSettings["endTime"]] - appointment[this.model.appointmentSettings["startTime"]]) / 3600000) > 24)) && appointment[this.model.appointmentSettings["recurrence"]] != 1) {
                    this._filt = true;
                    appointment.AppTaskId = appointment.AppTaskId == null ? this._maxId : appointment.AppTaskId;
                    ej.isNullOrUndefined(appointment.ParentId) && (appointment.ParentId = appointment.AppTaskId);
                    appointment.Guid = ej.isNullOrUndefined(appointment.Guid) ? this._guidFormatGenerate() : appointment.Guid;
                    this._maxId = appointment.AppTaskId;
                    this.app = false;
                    this._multipleSplit(appointment, this._maxId, appointment[this.model.appointmentSettings["startTime"]], appointment[this.model.appointmentSettings["endTime"]]);
                }
                else if (!ej.isNullOrUndefined(this._rRule.recurEditId)) {
                    ej.isNullOrUndefined(appointment.ParentId) && (appointment.ParentId = this._rRule.recurEditId);
                    appointment.AppTaskId = appointment.AppTaskId == null ? this._maxId : appointment.AppTaskId;
                    appointment.Guid = ej.isNullOrUndefined(appointment.Guid) ? this._guidFormatGenerate() : appointment.Guid;
                    this._maxId = appointment.AppTaskId;
                    this.app = false;
                    this._multipleSplit(appointment, this._maxId, appointment[this.model.appointmentSettings["startTime"]], appointment[this.model.appointmentSettings["endTime"]]);
                }
                this._filt = false;
            }
            else if ((appointment[this.model.appointmentSettings["recurrence"]] == 0 || !ej.isNullOrUndefined(this._rRule.recurEditId))) {
                appointment.AppTaskId = appointment.AppTaskId == null ? this._maxId : appointment.AppTaskId;
                ej.isNullOrUndefined(appointment.ParentId) && (appointment.ParentId = appointment.AppTaskId);
                (!ej.isNullOrUndefined(this._rRule.recurEditId)) && (appointment.ParentId = this._rRule.recurEditId);
                appointment[this.model.appointmentSettings["recurrenceId"]] = appointment[this.model.appointmentSettings["recurrenceId"]] == null ? null : appointment[this.model.appointmentSettings["recurrenceId"]];
                appointment[this.model.appointmentSettings["recurrenceExDate"]] = appointment[this.model.appointmentSettings["recurrenceExDate"]] == null ? null : appointment[this.model.appointmentSettings["recurrenceExDate"]];
                appointment.Guid = ej.isNullOrUndefined(appointment.Guid) ? this._guidFormatGenerate() : appointment.Guid;
                this.app = false;
                this._processed.push(appointment);
            }
            else if (((appointment[this.model.appointmentSettings["recurrence"]] == undefined) || (appointment[this.model.appointmentSettings["allDay"]] == undefined)) && !appointment[this.model.appointmentSettings["recurrence"]]) {
                appointment.AppTaskId = appointment.AppTaskId == null ? this._maxId : appointment.AppTaskId;
                ej.isNullOrUndefined(appointment.ParentId) && (appointment.ParentId = appointment.AppTaskId);
                appointment.Guid = ej.isNullOrUndefined(appointment.Guid) ? this._guidFormatGenerate() : appointment.Guid;
                appointment.RecurrenceId = null;
                appointment.recurrenceExDate = null;
                newAppointment = $.extend(true, {}, appointment);
                appointment.Guid = ej.isNullOrUndefined(appointment.Guid) ? this._guidFormatGenerate() : appointment.Guid;
                this.app = false;
                this._processed.push(appointment);
            }
            this._appMainId = (ej.isNullOrUndefined(this._appMainId) || this._appMainId < this._maxId) ? this._maxId : this._appMainId;
        },
        _idChecking: function () {
            for (var i = 0; i < this._currentAppointmentData.length ; i++) {
                var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, this._maxId);
                var _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                if (_editApp.length == 0 || this.app == true) { this._maxId; this.app = false; break; }
                else
                    this._maxId++;
            }
            for (; ;) {
                var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, this._maxId);
                var _editApp = new ej.DataManager(this._processed).executeLocal(query);
                if (_editApp.length == 0 || this.app == true) { this._maxId; this.app = false; break; }
                else
                    this._maxId++;
            }
            return this._maxId
        },

        _multipleDayApp: function (app, maxId, startTime, endTime) {
            var dayStart = new Date(startTime);
            if ((this.model.orientation == "vertical") || (this.model.orientation == "horizontal") && (startTime.getDay() == new Date(this._dateRender[this._dateRender.length - 1]).getDay() || endTime.getDay() == new Date(this._dateRender[this._dateRender.length - 1]).getDay())) {
                var nextDay = new Date(new Date(startTime).getFullYear(), new Date(startTime).getMonth(), new Date(startTime).getDate());
                var dayEnd = new Date(new Date(nextDay).setHours(new Date(nextDay).getHours() + 24));
                dayEnd = new Date(new Date(dayEnd).setMinutes(new Date(dayEnd).getMinutes() - 1));
            }
            else
                var dayEnd = new Date(endTime);
            while (dayStart >= new Date(startTime) && dayStart <= dayEnd && dayEnd <= new Date(endTime)) {
                app[this.model.appointmentSettings["recurrenceId"]] = app[this.model.appointmentSettings["recurrenceId"]] == null ? null : app[this.model.appointmentSettings["recurrenceId"]];
                app[this.model.appointmentSettings["recurrenceExDate"]] = app[this.model.appointmentSettings["recurrenceExDate"]] == null ? null : app[this.model.appointmentSettings["recurrenceExDate"]];
                var appointment = this._addAppData(app, maxId, dayStart, dayEnd, app.ParentId);
                this._processed.push(appointment);
                dayStart = new Date(new Date(dayEnd).setMinutes(new Date(dayEnd).getMinutes() + 1));
                dayEnd = app[this.model.appointmentSettings["endTime"]];
                this._maxId = maxId = this._idChecking();
            }
        },

        _multipleSplit: function (app, maxId, startTime, endTime) {
            startTime = new Date(startTime);
            if (this.currentView() == "workweek") {
                var sunday = new Date((new Date(startTime)).setDate(startTime.getDate() + 1));
                var saturday = new Date((new Date(startTime)).setDate(startTime.getDate() + 2));
                startTime = (new Date(startTime).getDay() == 0) && this.model.workWeek[0] == "monday" ? sunday : (new Date(startTime).getDay() == 6) ? saturday : new Date(startTime);
            }
            this.starttime = ((new Date(this._dateRender[0]) <= startTime) || this._filt == true) ? new Date(new Date(startTime).getFullYear(), new Date(startTime).getMonth(), new Date(startTime).getDate(), new Date(startTime).getHours(), new Date(startTime).getMinutes(), new Date(startTime).getSeconds()) : new Date(this._dateRender[0]);
            var end = new Date(new Date(endTime).getFullYear(), new Date(endTime).getMonth(), new Date(endTime).getDate());
            var next = 0;
            while (this.starttime <= end) {
                (this.starttime >= end && !(((endTime - startTime) / 3600000) > 24)) ? (this._filt = false) : "";
                var differDate = 0;
                var day_start = this.starttime;
                if (this.currentView() != "agenda") {
                    if ((this.model.orientation == "horizontal" && this.currentView() == "month"))
                        differDate = new Date(day_start.getFullYear(), day_start.getMonth() + 1, 0).getDate() - (day_start.getDate());
                    else if (this.currentView() == "customview" && (this._renderDays <= 7 || this.model.orientation == "horizontal")) {
                        this.starttime = (new Date(new Date(day_start).getFullYear(), new Date(day_start).getMonth(), new Date(day_start).getDate()));
                        differDate = (this._dateRender.length - 1) - this._dateRender.indexOf(this.starttime.getTime());
                    }
                    else if (this.currentView() !== "day")
                        if (this._firstdayofweek == 0)
                            differDate = 6 - day_start.getDay();
                        else if (this._firstdayofweek != 0 || this.currentView() == "month")
                            differDate = 6 - day_start.getDay() + this._firstdayofweek;
                }
                var nextDay = new Date(new Date(this.starttime).getFullYear(), new Date(this.starttime).getMonth(), new Date(this.starttime).getDate() + differDate);
                var day_end = (new Date(nextDay).getDate() == new Date(end).getDate() && this.currentView() !== "month") ? new Date(new Date(nextDay).setHours(new Date(endTime).getHours())) : new Date(new Date(nextDay).setHours(23));
                day_end = (new Date(nextDay).getDate() == new Date(end).getDate() && this.currentView() !== "month") ? new Date(new Date(day_end).setMinutes(new Date(endTime.getMinutes()))) : new Date(new Date(day_end).setMinutes(59));
                if ((day_end >= new Date(endTime)) && this.currentView !== "day")
                    day_end = app[this.model.appointmentSettings["endTime"]];
                else if ((day_end.getDay() == 6 && this.currentView() == "workweek") || (day_end.getDay() == 7 && this.currentView() == "workweek")) {
                    var sunday = new Date((new Date(day_end)).setDate(day_end.getDate() - 2));
                    var saturday = new Date((new Date(day_end)).setDate(day_end.getDate() - 1));
                    day_end = (new Date(day_end).getDay() == 6) ? saturday : sunday;
                }
                else {
                    day_end = day_end;
                    var weekEnd = true;
                }
                app[this.model.appointmentSettings["recurrenceId"]] = app[this.model.appointmentSettings["recurrenceId"]] == null ? null : app[this.model.appointmentSettings["recurrenceId"]];
                app[this.model.appointmentSettings["recurrenceExDate"]] = app[this.model.appointmentSettings["recurrenceExDate"]] == null ? null : app[this.model.appointmentSettings["recurrenceExDate"]];
                appointment = ((((app[this.model.appointmentSettings["endTime"]] - app[this.model.appointmentSettings["startTime"]]) / 3600000) > 24) && app[this.model.appointmentSettings["recurrence"]] != 1) ? this._addAppData(app, maxId, day_start, day_end, app.AppTaskId) : this._addAppData(app, maxId, day_start, day_end, app["ParentId"]);
                this._processed.push(appointment);
                this._maxId = maxId = this._idChecking();
                next = (this.currentView() == "workweek") ? 3 : 1;
                this.starttime = new Date(new Date(day_end).getFullYear(), new Date(day_end).getMonth(), new Date(day_end).getDate() + next);
                var lastDate = this.currentView() == "agenda" ? new Date(new Date(new Date(this._dateRender[0]).setDate(this._dateRender[0].getDate() + this.model.agendaViewSettings.daysInAgenda)).setHours(23, 59, 59)) : new Date(this._dateRender[this._dateRender.length - 1]);
                if (lastDate < this.starttime) {
                    if (!this._filt)
                        break;
                }
            }
        },

        _deleteOcurrence: function (appId, date) {
            var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, appId);
            var _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
            if ((this._currentAction == "delete" || this._currentAction == "editOcurrence") && _editApp[0][this.model.appointmentSettings["endTime"]].getDay() > _editApp[0][this.model.appointmentSettings["startTime"]].getDay()) {
                var objDate = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.appointmentSettings["startTime"], ej.FilterOperators.equal, date));
                objDate = objDate.length == 0 ? this._objDate : objDate;
                var objDate1 = objDate;
                var objId = objDate[0]["AppTaskId"];
                objDate = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("AppTaskId", ej.FilterOperators.equal, objId));
            }
            if (!ej.isNullOrUndefined(_editApp) && !ej.isNullOrUndefined(_editApp[0][this.model.appointmentSettings["recurrenceRule"]])) {
                var _rRule = _editApp[0][this.model.appointmentSettings["recurrenceRule"]];
                this.recurrenceRuleSplit(_rRule, _editApp[0][this.model.appointmentSettings["recurrenceExDate"]]);
                if (ej.isNullOrUndefined(this._rRule.exDate)) {
                    if ((this._currentAction == "delete" || this._currentAction == "editOcurrence") && _editApp[0][this.model.appointmentSettings["endTime"]].getDay() > _editApp[0][this.model.appointmentSettings["startTime"]].getDay()) {
                        if (objDate[0][this.model.appointmentSettings["startTime"]].getDay() == objDate[0][this.model.appointmentSettings["endTime"]].getDay()) {
                            date = objDate[0][this.model.appointmentSettings["startTime"]];
                            _editApp[0][this.model.appointmentSettings["recurrenceRule"]] = _rRule + ";EXDATE=" + Globalize.format(date, this._pattern.d);
                            _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] = !ej.isNullOrUndefined(_editApp[0][this.model.appointmentSettings["recurrenceExDate"]]) ? _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] + "," + Globalize.format(date, this._pattern.d).toString() : Globalize.format(date, this._pattern.d).toString();
                        }
                        else {
                            _editApp[0][this.model.appointmentSettings["recurrenceRule"]] = _rRule + ";EXDATE=" + Globalize.format(date, this._pattern.d);
                            _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] = !ej.isNullOrUndefined(_editApp[0][this.model.appointmentSettings["recurrenceExDate"]]) ? _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] + "," + Globalize.format(date, this._pattern.d).toString() : Globalize.format(date, this._pattern.d).toString();
                        }
                    }
                    else {
                        _editApp[0][this.model.appointmentSettings["recurrenceRule"]] = _rRule + ";EXDATE=" + Globalize.format(date, this._pattern.d);
                        _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] = !ej.isNullOrUndefined(_editApp[0][this.model.appointmentSettings["recurrenceExDate"]]) ? _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] + "," + Globalize.format(date, this._pattern.d).toString() : Globalize.format(date, this._pattern.d).toString();
                    }
                }
                else {
                    var _rules = _rRule.split(';');
                    recurRule = "";
                    for (var i = 0; i < _rules.length; i++) {
                        if (recurRule != "")
                            recurRule += ";";
                        var keyValuePair = _rules[i].replace(/ /g, '').split('=');
                        if (keyValuePair[0] == "EXDATE") {
                            if ((this._currentAction == "delete" || this._currentAction == "editOcurrence") && _editApp[0][this.model.appointmentSettings["endTime"]].getDay() > _editApp[0][this.model.appointmentSettings["startTime"]].getDay()) {
                                if (objDate[0][this.model.appointmentSettings["startTime"]].getDay() == objDate[0][this.model.appointmentSettings["endTime"]].getDay()) {
                                    date = objDate[0][this.model.appointmentSettings["startTime"]];
                                    _rules[i] = _rules[i] + "," + Globalize.format(date, this._pattern.d);
                                }
                                else
                                    _rules[i] = _rules[i] + "," + Globalize.format(date, this._pattern.d);
                            }
                            else
                                _rules[i] = _rules[i] + "," + Globalize.format(date, this._pattern.d);
                        }
                        recurRule += _rules[i];
                    }
                    _editApp[0][this.model.appointmentSettings["recurrenceRule"]] = recurRule;
                    _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] = !ej.isNullOrUndefined(_editApp[0][this.model.appointmentSettings["recurrenceExDate"]]) ? _editApp[0][this.model.appointmentSettings["recurrenceExDate"]] + "," + Globalize.format(date, this._pattern.d).toString() : Globalize.format(date, this._pattern.d).toString();
                }
                return _editApp;
            }
        },

        _getWeekIndex: function (curDate) {
            var index = 0;
            var firstDate = new Date(curDate.getFullYear(), curDate.getMonth(), 1);
            var dayIndex = curDate.getDay();
            if (dayIndex != firstDate.getDay())
                do {
                    firstDate = new Date(firstDate.setDate(firstDate.getDate() + 1));
                } while (dayIndex != firstDate.getDay())
            while (firstDate.getDate() != curDate.getDate()) {
                firstDate = new Date(firstDate.setDate(firstDate.getDate() + 7));
                index++;
            }
            return index;
        },

        _addAppData: function (app, id, startTime, endTime, parentId) {
            var data = {};
            for (var item in this.model.appointmentSettings) {
                if (item != "dataSource" && item != "query" && item != "tableName" && item != "resourceFields")
                    data[this.model.appointmentSettings[item]] = app[this.model.appointmentSettings[item]];
                if (item == "resourceFields") {
                    for (var val in !ej.isNullOrUndefined(this.model.appointmentSettings.resourceFields) && this.model.appointmentSettings.resourceFields.split(','))
                        data[this.model.appointmentSettings.resourceFields.split(',')[val]] = app[this.model.appointmentSettings.resourceFields.split(',')[val]];
                }
            }
            (!ej.isNullOrUndefined(id)) && (data.AppTaskId = id);
            (!ej.isNullOrUndefined(startTime)) && (data[this.model.appointmentSettings["startTime"]] = startTime);
            (!ej.isNullOrUndefined(endTime)) && (data[this.model.appointmentSettings["endTime"]] = endTime);
            (!ej.isNullOrUndefined(parentId)) && (data["ParentId"] = parentId);
            var appdata = $.extend({}, app, data);
            return appdata;
        },

        _dayOfWeekInMonth: function (strDate, dayName, weekIndex) {
            var dayIndex = this._dayNames.indexOf(dayName);
            var curDate = weekIndex == -1 ? new Date(strDate.getFullYear(), strDate.getMonth() + 1, 0) : new Date(strDate.getFullYear(), strDate.getMonth(), 1);
            while (curDate.getDay() != dayIndex) {
                curDate = new Date(weekIndex == -1 ? curDate.setDate(curDate.getDate() - 1) : curDate.setDate(curDate.getDate() + 1));
            }
            return new Date(weekIndex == 1 || weekIndex == -1 ? curDate : curDate.setDate(curDate.getDate() + ((weekIndex - 1) * 7)));
        },

        _weeklyDayFind: function (strDate, days) {
            var newdayStr = strDate;
            do {
                newdayStr = new Date(newdayStr.setDate(newdayStr.getDate() + 1));
                var curDay = this._dayNames[new Date(newdayStr).getDay()];
            } while (days.indexOf(curDay) == -1);
            return newdayStr;
        },

        _renderAppointments: function () {
            var dateCount = this._dateRender;
            this._tempapp = []; this._app = [];
            var level = 1; this.level = 0;
            this.cellwidth = this.element.find(".e-workcells").width();
            this.leftPosition = 0;
            this._filtered = [];
            if (!this.model.group) this.res1 = ["0"];
            this.resIndex = 0;
            for (var res = 0; res < this.res1.length; res++) {
                var day = 0, diff, count = dateCount.length;
                if (new Date(this._dateRender[0]) < this.model.minDate)
                    day = Math.floor((new Date(this.model.minDate) - new Date(this._dateRender[0])) / (1000 * 60 * 60 * 24));
                if (new Date(this._dateRender[this._dateRender.length - 1]) > this.model.maxDate) {
                    diff = Math.floor((new Date(this._dateRender[this._dateRender.length - 1]) - new Date(this.model.maxDate)) / (1000 * 60 * 60 * 24));
                    count = this._dateRender.length - diff;
                }
                for (day; day < count; day++) {
                    this._getDayAppointments(day, res);
                }
            }
        },
		_findAllDayApp: function (apps) {
			if (apps[this.model.appointmentSettings["allDay"]] != true) {
			    var tempCurrent = new ej.DataManager(this._currentAppointmentData).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, apps.Guid));
				var Value = (!ej.isNullOrUndefined(tempCurrent[0])) ? (((tempCurrent[0][this.model.appointmentSettings["endTime"]] - tempCurrent[0][this.model.appointmentSettings["startTime"]]) / 3600000) > 24) ? true : false : false;
				return Value;
            }
        },
		
        _getDayAppointments: function (day, res) {
            this._appList = [];
            var dateCount = this._dateRender, appointmentsList = this._processed;
            var predicate2 = ej.Predicate(this.model.appointmentSettings["endTime"], ej.FilterOperators.greaterThanOrEqual, new Date(dateCount[day])).and(this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(dateCount[day]).setHours(23, 59, 59)));
            var predicate1 = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(dateCount[day])).and(this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(dateCount[day]).setHours(23, 59, 59)));
            var predicate = predicate2["or"](predicate1);
            predicate = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
            appointmentsList = new ej.DataManager(appointmentsList).executeLocal(new ej.Query().where(predicate));
            var renderedAlready = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(dateCount[day]));
            appointmentsList = new ej.DataManager(appointmentsList).executeLocal(new ej.Query().where(renderedAlready));
            this._appList = this._alldaySort(appointmentsList);
            appointmentsList = this._appointmentSort(appointmentsList);
            this._app = $.merge(this._app, appointmentsList);
            if (this.model.orientation == "vertical") {
                for (var app = 0; app < appointmentsList.length; app++) {
                    var isBigApp = (((appointmentsList[app][this.model.appointmentSettings["endTime"]] - appointmentsList[app][this.model.appointmentSettings["startTime"]]) / 3600000) >= 24);
                    var allDayCheck = appointmentsList[app][this.model.appointmentSettings["allDay"]];
                    if (((new Date(new Date(dateCount[day]).setHours(0, 0, 0, 0)).getTime() == (new Date(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime())) &&
                              allDayCheck) || isBigApp) {
                        this._setAllDayPos(appointmentsList[app], day, res);
                        this._filtered.push(appointmentsList[app]);
                    }
                    else if ((new Date(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(dateCount[day]).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime() >= new Date(new Date(dateCount[day]).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() == new Date(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((ej.isNullOrUndefined(allDayCheck) || allDayCheck == "") ? false : allDayCheck === true ? true : allDayCheck.toLowerCase() === 'false' ? false : allDayCheck) === false) && (new Date(new Date(appointmentsList[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointmentsList[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime())) {
                        var query = new ej.Query().where("Guid", ej.FilterOperators.equal, appointmentsList[app]["Guid"]);
                        var templist = new ej.DataManager(this._processed).executeLocal(query);
                        if (!allDayCheck) {
							var Value = this._findAllDayApp(appointmentsList[app]);
                        }
                        if (Value || templist[0][this.model.appointmentSettings["allDay"]] && Math.round((new Date(templist[0][this.model.appointmentSettings["endTime"]]) - new Date(templist[0][this.model.appointmentSettings["startTime"]])) / 3600000) >= 24) {
                            this._setAllDayPos(appointmentsList[app], day, res);
                            this._filtered.push(appointmentsList[app]);
                        }
                        else {
                            this._renderApp(appointmentsList[app], day, res);
                            this._filtered.push(appointmentsList[app]);
                        }
                    }
                }
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                this._getHorizontalDayAppointments(appointmentsList, dateCount, day, res);
        },

        _startUtc: function (e) {
            (!ej.isNullOrUndefined(this.startutcValue) && ej.isNullOrUndefined(this.tempValue)) ? this._utcValue(e, "startUtc") : e.model.value == "" ? "" : ((e.model.value == this._appointmentAddWindow.find("#" + this._id + "endTimeZone").ejDropDownList('model.value') && this._appointmentAddWindow.find("#" + this._id + "startTimeZone").attr('aria-expanded') == "true")) ? this._appointmentAddWindow.find("#" + this._id + "endTimeZone").ejDropDownList("option", { text: e.text, value: e.value }) : this._appointmentAddWindow.find("#" + this._id + "endTimeZone").val(e.text);
        },
        _endUtc: function (e) {
            (this._appointmentAddWindow.find("#" + this._id + "endTimeZone").attr('aria-expanded') == "true") ? this._utcValue(e, "endUtc") : "";
        },
        _utcValue: function (temp, getValue) {
            Globalize.culture(this.model.locale);
            getValue == "endUtc" ? this._appointmentAddWindow.find('.endutc').val(temp.text) : "";
            var utcValue = new Date(Globalize.format(this._appointmentAddWindow.find("#" + this._id + "startdate").val(), this._datepattern()) + " " + Globalize.format(this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("model.value")));
            var utcendValue = new Date(Globalize.format(this._appointmentAddWindow.find("#" + this._id + "enddate").val(), this._datepattern()) + " " + Globalize.format(this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("model.value")));
            var startDateutc = new Date(utcValue);
            var EndDateutc = new Date(utcendValue);
            var utcEnd = getValue == "Appoint" ? this.endTimeAppoint : getValue == "endUtc" ? temp.value : this._appointmentAddWindow.find("#" + this._id + "endTimeZone").ejDropDownList("model.value");
            var utcStart = getValue == "Appoint" ? this.startTimeAppoint : getValue == "startUtc" ? temp.value : this._appointmentAddWindow.find("#" + this._id + "startTimeZone").ejDropDownList("model.value");
            utcStart = utcStart == "" ? this._timezoneStringValue() : utcStart;
            utcEnd = utcEnd == "" ? this._timezoneStringValue() : utcEnd;
            utcEnd = utcEnd.indexOf("UTC") != -1 ? parseInt(utcEnd.split(" ")[1].split(":")[0]) * 60 + parseInt(utcEnd.split(" ")[1].split(":")[1]) : parseInt(utcEnd.split(":")[0]) * 60 + parseInt(utcEnd.split(":")[1]);
            utcStart = utcStart.indexOf("UTC") != -1 ? parseInt(utcStart.split(" ")[1].split(":")[0]) * 60 + parseInt(utcStart.split(" ")[1].split(":")[1]) : parseInt(utcStart.split(":")[0]) * 60 + parseInt(utcStart.split(':')[1]);
            var startendUtc = utcEnd - utcStart;
            utcValue = new Date(utcValue.setMinutes(utcValue.getMinutes() + startendUtc));
            if (getValue == "Appoint") {
                var dateNew = new Date(temp.StartTime);
                var endNew = new Date(temp.EndTime);
                var timeZoneValue = ej.isNullOrUndefined(this.model.timeZone) ? -new Date().getTimezoneOffset() + this._timeZoneOffset : this.model.timeZone.indexOf('UTC') != -1 ? ((parseInt(this.model.timeZone.split(" ")[1].split(":")[0]) * 60) + (parseInt(this.model.timeZone.split(" ")[1].split(":")[1]))) : ((parseInt(this.model.timeZone.split(":")[0]) * 60) + (parseInt(this.model.timeZone.split(":")[1])));
                var timeZoneStart = (utcStart == timeZoneValue) ? 0 : timeZoneValue - utcStart;
                timeZoneStart = new Date(dateNew.setMinutes(dateNew.getMinutes() - timeZoneStart));
                var value2 = this._timeMode == "12" ? Globalize.format(new Date(timeZoneStart), this._pattern.t) : Globalize.format(new Date(timeZoneStart), "HH:mm");
                this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker({ value: Globalize.format(new Date(timeZoneStart), this._datepattern()) });
                var timeZoneEnd = utcEnd - timeZoneValue;
                this._appointmentAddWindow.find('#' + this._id + "starttime").ejTimePicker({ value: value2 });
            }
            utcendValue = (getValue == "endUtc" || getValue == "Appoint") ? new Date(startDateutc.setMinutes(startDateutc.getMinutes() + startendUtc)) : utcValue;
            utcendValue = getValue == "Appoint" ? new Date(endNew.setMinutes(endNew.getMinutes() + timeZoneEnd)) : utcendValue;
            var different = this._timeMode == "12" ? Globalize.format(new Date(utcValue), this._pattern.t) : Globalize.format(new Date(utcValue), "HH:mm");
            if (getValue == "Appoint") {
                value = this._timeMode == "12" ? Globalize.format(new Date(endNew), this._pattern.t) : Globalize.format(new Date(endNew), "HH:mm");
            }
            else {
                var endTimeValue = new Date(utcendValue.setMinutes(utcendValue.getMinutes() + 30));
                var value = this._timeMode == "12" ? Globalize.format(new Date(endTimeValue), this._pattern.t) : Globalize.format(new Date(endTimeValue), "HH:mm");
            }
            this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker({ value: Globalize.format(new Date(utcendValue), this._datepattern()) });
            getValue == "Appoint" ? this._appointmentAddWindow.find('#' + this._id + "endtime").ejTimePicker({ value: value }) : this._appointmentAddWindow.find('#' + this._id + "endtime").ejTimePicker({ value: value });
            this.startutcValue = 1;
        },

        _rowHandlesOver: function (e) {
            this.model.enableAppointmentResize && $(e.target).addClass("e-rowcursor");
        },

        _colHandlesOver: function (e) {
            this.model.enableAppointmentResize && $(e.target).addClass("e-columncursor");
        },

        _renderApp: function (record, day, res) {
            var cellHeight = this.element.find(".e-workcells").outerHeight();
            var timeScaleRowCount = (60 / 30);
            var startHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.startHour) : 0;
            var endHour = parseInt(this.model.startHour) < parseInt(this.model.endHour) ? parseInt(this.model.endHour) : 24;
            var dateCount = this._dateRender;
            var rStartTime = record[this.model.appointmentSettings["startTime"]];
            var rEndTime = record[this.model.appointmentSettings["endTime"]];
            var appdate = false;
            if ((new Date(rStartTime).getHours()) < startHour) {
                var newStartTime = new Date(rStartTime);
                newStartTime.setHours(startHour, rStartTime.getMinutes(), rStartTime.getSeconds());
                rStartTime = new Date(newStartTime);
            }
            if (this.model.orientation == "vertical") {
                var appWidth, appHeight;

                if (rStartTime <= rEndTime && (new Date(dateCount[0]).getTime() <= rStartTime.getTime() && new Date(new Date(dateCount[dateCount.length - 1]).setHours(23, 59, 59)) >= rStartTime.getTime())) {
                    var startTime = ((new Date(rStartTime).getHours()) < startHour) ? (new Date(rStartTime).getHours() + (-new Date(rStartTime).getHours()) + (-new Date(rStartTime).getMinutes())) : (new Date(rStartTime).getHours() + (-startHour));
                    var endTime = ((new Date(rEndTime).getHours()) > endHour) ? (new Date(rEndTime).getHours() + (-new Date(rEndTime).getHours() + (endHour - startHour)) + (-new Date(rEndTime).getMinutes())) : (new Date(rEndTime).getHours() + (-startHour));
                    endTime = (new Date(rEndTime).getHours() == endHour) ? (new Date(endTime).getMinutes() + (-new Date(rEndTime).getMinutes())) : endTime;

                    if (endHour <= rEndTime.getHours()) {
                        var endDate = new Date(rEndTime.setHours(endHour, 00, 00));
                        appHeight = (parseFloat(((new Date(endDate).getHours() * 60) + (new Date(endDate).getMinutes())) - ((new Date(rStartTime).getHours() * 60) + (new Date(rStartTime).getMinutes()))) * ((cellHeight) * timeScaleRowCount) / 60) - 3;
                    }
                    else
                        appHeight = (parseFloat(((new Date(rEndTime).getHours() * 60) + (new Date(rEndTime).getMinutes())) - ((new Date(rStartTime).getHours() * 60) + (new Date(rStartTime).getMinutes()))) * ((cellHeight) * timeScaleRowCount) / 60) - 2;
                    var topPosition = parseFloat((new Date(rStartTime).getMinutes())) * (cellHeight) * timeScaleRowCount / 60;

                    if (this.model.startHour <= new Date(rStartTime).getHours()) {

                        var overlapList = this._overlapApp(record["AppTaskId"], record[this.model.appointmentSettings["startTime"]], record[this.model.appointmentSettings["endTime"]], null);
					    var alldayquery= ej.Predicate(this.model.appointmentSettings["allDay"], ej.FilterOperators.contains, false);
						overlapList=new ej.DataManager(overlapList).executeLocal(new ej.Query().where(alldayquery));
						var proxy = this;
                        if (overlapList.length > 1) {
                            overlapList = this._appointmentSort(overlapList);
                            var overlabcount = 1;
                            var flag = false;
                            for (i = 0; i < overlapList.length; i++) {
                                for (var j = i + 1; j < overlapList.length; j++) {
                                    if ((new Date(overlapList[i][this.model.appointmentSettings["endTime"]]) > new Date(overlapList[j][this.model.appointmentSettings["startTime"]])) && (new Date(overlapList[i][this.model.appointmentSettings["startTime"]]) < new Date(overlapList[j][this.model.appointmentSettings["endTime"]]))) {
                                        flag = true;
                                    }
                                }
                                if (flag == true) {
                                    overlabcount = overlabcount + 1;
                                    flag = false;
                                }
                            }
                            var index = overlapList.indexOf(record);
                            appWidth = (100 / overlabcount) + 2;
                            this.currentView() === "day" || (this.currentView() == "customview" && this._renderDays == 1) ? appWidth -= 7 : appWidth;
                            this.leftPosition = ((100 / overlapList.length) * (index)) + "%";
                        }
                        else {
                            this.leftPosition = "0";
                            appWidth = 95;
                        }
                        appWidth = (this.currentView() === "week" || this.currentView() === "workweek" || (this.currentView() == "customview" && this._renderDays <= 7)) ? (appWidth - 10) + "%" : appWidth + "%";
                        var colorAppointment = this._appointmentColor(record);
                        var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                        var userTempHtml = this._getUserAppointmentTemplate(record);
                        var startTimeDisp = this._timeMode == "12" ? Globalize.format(record[this.model.appointmentSettings["startTime"]], this._pattern.t) : Globalize.format(record[this.model.appointmentSettings["startTime"]], "HH:mm");
                        var endTimeDisp = this._timeMode == "12" ? Globalize.format(record[this.model.appointmentSettings["endTime"]], this._pattern.t) : Globalize.format(record[this.model.appointmentSettings["endTime"]], "HH:mm");
                       var timeCellhtml = this.appTemplate.render({ id: record["AppTaskId"], appResize: this.model.enableAppointmentResize, currentview: this.currentView(), appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp, left: this.leftPosition, top: topPosition, appWidth: appWidth, userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-appointment", appointData: colorAppointment.appointColor.appointData, appointCustomcss: colorAppointment.appointColor.appointCustomcss, appointtextcolor: colorAppointment.appointColor.appointtextcolor, value: colorAppointment.value, multiDiv: colorAppointment.multiDiv, orientation: this.model.orientation, priority: this.model.prioritySettings.enable ? record[this.model.appointmentSettings["priority"]] : '', enablePriority: this.model.prioritySettings["template"], priorityTemplate: this.model.prioritySettings["template"] ? this._priorityTemplate(record) : record[this.model.appointmentSettings['priority']], uid: record.Guid });
                        $(this.element.find("div.e-appointwrapper")[(startTime * 2 * dateCount.length * this.res1.length) + day + (dateCount.length * res)]).append(timeCellhtml);
                    }
                }
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                this._renderHorizontalApp(rStartTime, rEndTime, record, day, cellHeight, timeScaleRowCount, startHour, endHour);
        },

        _priorityTemplate: function (record) {
            return this.model.prioritySettings.template.replace("{", "").replace("$", "").replace("}", "").replace(this.model.prioritySettings.template.indexOf("value") != -1 ? this.model.prioritySettings.value : this.model.prioritySettings.text, record[this.model.appointmentSettings["priority"]]);
        },

        _getUserAppointmentTemplate: function (record) {
            var userTemp = $.templates($(this.model.appointmentTemplateId).html());
            var cloneObj = $.extend(cloneObj, record);
            cloneObj.View = this.currentView();
            var userTempHtml = userTemp.render(cloneObj);
            return userTempHtml;
        },
        _overlapApp: function (id, appStartTime, appEndTime, resValue) {
            this.overlapList = [];
            this._appList = this._appointmentSort(this._appList);
            if (this.model.orientation == "vertical") {
                for (var i = 0; i < this._appList.length; i++) {
                    var Value = this._findAllDayApp(this._appList[i]);
                    if ((new Date(this._appList[i][this.model.appointmentSettings["endTime"]]) > new Date(appStartTime)) && (new Date(this._appList[i][this.model.appointmentSettings["startTime"]]) < new Date(appEndTime))
                        && (new Date(new Date(this._appList[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() === new Date(new Date(appStartTime).setHours(0, 0, 0, 0)).getTime()) && !Value) {
                        this.overlapList.push(this._appList[i]);
                    }
                }
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
                for (var i = 0; i < this._appList.length; i++) {
                    if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources)) {
                        if ((new Date(this._appList[i][this.model.appointmentSettings["endTime"]]) > new Date(appStartTime)) && (new Date(this._appList[i][this.model.appointmentSettings["startTime"]]) <= new Date(appEndTime))
                            && (this._appList[i][this.model.resources[this.model.resources.length - 1]["field"]] == resValue)
                            && (new Date(new Date(this._appList[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() === new Date(new Date(appStartTime).setHours(0, 0, 0, 0)).getTime())) {
                            this.overlapList.push(this._appList[i]);
                        }
                    }
                    else {
                        if ((new Date(this._appList[i][this.model.appointmentSettings["endTime"]]) > new Date(appStartTime)) && (new Date(this._appList[i][this.model.appointmentSettings["startTime"]]) <= new Date(appEndTime))
                            && (new Date(new Date(this._appList[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() === new Date(new Date(appStartTime).setHours(0, 0, 0, 0)).getTime())) {
                            this.overlapList.push(this._appList[i]);
                        }
                    }
                }
            }
            return this.overlapList;
        },

        _getOverlapCount: function (record, day, dateRender, resValue) {
            var appLevelsInDay = [];
            var len = dateRender.length;
            var appointments = this._processed;
            var predicate2 = ej.Predicate(this.model.appointmentSettings["endTime"], ej.FilterOperators.greaterThanOrEqual, new Date(dateRender[day])).and(this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(dateRender[day]).setHours(23, 59, 59)));
            var predicate1 = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(dateRender[day])).and(this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(dateRender[day]).setHours(23, 59, 59)));
            var predicate = predicate2["or"](predicate1);
            appointments = new ej.DataManager(appointments).executeLocal(new ej.Query().where(predicate));
            appointments = this._appointmentSort(appointments);
            var startDate = new Date(dateRender[day]);
			var endDate = new Date(new Date(dateRender[day]).setHours(23,59,59));
            var filterData = [];
            if (this.model.orientation == "vertical" && ej.isNullOrUndefined(this.model.group)) {
                for (var i = 0; i < appointments.length; i++) {
                    var isBigApp = (((appointments[i][this.model.appointmentSettings["endTime"]] - appointments[i][this.model.appointmentSettings["startTime"]]) / 3600000) >= 24);
                    var allDayCheck = appointments[i][this.model.appointmentSettings["allDay"]];
                    if (((new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime() == (new Date(new Date(appointments[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime())) &&
                              allDayCheck) || isBigApp) {
                        filterData.push(appointments[i]);
                    }
                }
            }
            else if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group) && this.model.orientation == "vertical") {
                for (var i = 0; i < appointments.length; i++) {
                    var isBigApp = (((appointments[i][this.model.appointmentSettings["endTime"]] - appointments[i][this.model.appointmentSettings["startTime"]]) / 3600000) >= 24);
                    var allDayCheck = appointments[i][this.model.appointmentSettings["allDay"]];
                    if ((appointments[i][this.model.resources[this.model.resources.length - 1]["field"]] == resValue) && (((new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime() == (new Date(new Date(appointments[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime())) &&
                              allDayCheck) || isBigApp)){
                        filterData.push(appointments[i]);
                    }
                }
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal)
                filterData = this._getHorizontalOverlapCount(appointments, dateRender, day, record, resValue);
            var proxy = this;
            filterData = this._sortAppById(filterData);
            filterData = this._appointmentSort(filterData);
            var index = -1;
            for (var j = 0; j < filterData.length; j++) {
                var apps = this.element.find('[guid=' + filterData[j]["Guid"] + ']').height();
                apps = ej.isNullOrUndefined(apps) ? false : true;
                if (apps) {
                    this._appointmentLevel(filterData[j]);
                    appLevelsInDay.push(this.appLevel);
                }
                if (filterData[j]["Guid"] == record["Guid"]) {
                    var appLevel = (appLevelsInDay.length > 0) ? this._smallestMissingNumber(appLevelsInDay) : -1;
                    if (filterData.length == 1) index = j;
                    else index = appLevel;
                    break;
                }
            }
            return index == -1 ? 0 : index;
        },
        _smallestMissingNumber: function (array) {
            if (array.length == 0)
                return 0;
            var large = Math.max.apply(Math, array);
            for (var i = 0; i < large; i++) {
                if (array.indexOf(i) == -1) {
                    return i;
                }
            }
            return large + 1;
        },
        _appointmentLevel: function (appointment) {
            var appHeight = this.element.find('[guid=' + appointment["Guid"] + ']').css("height");
            var appTop = this.element.find('[guid=' + appointment["Guid"] + ']').css("top");
            if (this.model.orientation == "vertical") {
                if (!ej.isNullOrUndefined(this.model.resources))
                    var appEle = (this.currentView() == "month" || (this._isCustomView() && !this._oneWeek)) ? 0 : ((!ej.isNullOrUndefined(this.model.group)) && (!ej.isNullOrUndefined(this.model.group.resources))) ? (this.model.group.resources.length > 1) ? (this.element.find(".e-headerbar").outerHeight() + (this.element.find(".e-dateheader").outerHeight() * this.model.group.resources.length) + 1) : (this.element.find(".e-headerbar").outerHeight() + this.element.find(".e-dateheader").outerHeight() + 1) : (this.element.find(".e-headerbar").outerHeight() + this.element.find(".e-dateheader").outerHeight() + 1);
                else
                    var appEle = (this.currentView() == "month" || (this._isCustomView() && !this._oneWeek)) ? 0 : this.element.find(".e-dateheader").outerHeight() + 1;
            }
            if (this.model.orientation == "vertical")
                this.appLevel = (this.currentView() == "month" || (this._isCustomView() && !this._oneWeek)) ? Math.round((parseInt(appTop) / parseInt(appHeight))) : Math.round((parseInt(appTop) - appEle) / parseInt(appHeight));
            else
                this.appLevel = Math.round(parseInt(appTop) / parseInt(appHeight));
            return this.appLevel;
        },

        _setAllDayPos: function (record, day, res) {
            this.overlapList = [];
            var level = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this._getOverlapCount(record, day, this._dateRender, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : this._getOverlapCount(record, day, this._dateRender, null);
            var start = record[this.model.appointmentSettings["startTime"]];
            var end = record[this.model.appointmentSettings["endTime"]];
            if (new Date(start) <= new Date(end)) {
                var dateRender = this._dateRender;
                var appWidth, left = 0, top = 16, appHeight = 19;
                var r_start = new Date(dateRender[0]);
                var r_end = new Date(dateRender[dateRender.length - 1]);
                r_end.setHours(23, 59, 59);
                end = new Date(end) > new Date(r_end) ? r_end : end;
                var noOfDays = Math.ceil((new Date(start) - new Date(r_start)) / (1000 * 3600) / 24);
                if (this.currentView() == "week" || this.currentView() == "day" || this.currentView() == "workweek" && start.getDay() == end.getDay())
                    var diffInDays = Math.ceil((new Date(new Date(end).setHours(23, 59, 59)) - new Date(new Date(start).setHours(0, 0, 0))) / (1000 * 3600) / 24);
                else if (end.getDay() == 6 && this.currentView() == "workweek" && this._firstdayofweek != 0)
                    diffInDays = $.inArray(new Date(end.getFullYear(), end.getMonth(), end.getDate() - 1).getTime(), this._dateRender) + 1;
                else if (end.getDay() == 0 && this.currentView() == "workweek" && this._firstdayofweek != 0)
                    diffInDays = $.inArray(new Date(end.getFullYear(), end.getMonth(), end.getDate() - 2).getTime(), this._dateRender) + 2;
                else
                    diffInDays = $.inArray(new Date(end.getFullYear(), end.getMonth(), end.getDate()).getTime(), this._dateRender) - ($.inArray(new Date(start.getFullYear(), start.getMonth(), start.getDate()).getTime(), this._dateRender)) + 1;
                if(diffInDays <= 0) diffInDays = 1;
                var cellWidth = this.element.find(".e-alldaycells").width();
                var daysCount = this.currentView() === "week" ? 7 : this.currentView() === "workweek" ? this.model.workWeek.length : (this.currentView() == "customview" && this._renderDays <= 7) ? this._renderDays : 1;
                var alldayTr = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.model.group.resources.length : 0;
                var start_time = (new Date(new Date(start).getFullYear(), new Date(start).getMonth(), new Date(start).getDate()));
                var tdIndex = this.currentView() === "day" ? 0 : this.currentView() === "workweek" && this._firstdayofweek == 0 && this.model.workWeek[0] == "Monday" && this.model.workWeek[4] == "Friday" ? new Date(start).getDay() - 1 : this._firstdayofweek || this.model.workWeek.length > 0 || (this.currentView() == "customview" && (this._renderDays <= 7 || this.model.orientation == "horizontal")) ? this._dateRender.indexOf(start_time.getTime()) : new Date(start).getDay();
                if (!ej.isNullOrUndefined(this.element.find(".e-alldaycells")[tdIndex + (res * daysCount)]))
                    left = (this.model.enableRTL) ? this.element.find(".e-headrealldaytable").width() - this.element.find(".e-alldaycells")[tdIndex + (res * daysCount)].offsetLeft - this.element.find(".e-alldaycells").width() : this.element.find(".e-alldaycells")[tdIndex + (res * daysCount)].offsetLeft;
                appWidth = ((diffInDays * cellWidth) + diffInDays) - 2;
                top = (level * top) + (3 * (level + 1)) + this.element.find(".e-headercells").height() + 1 + (alldayTr * this.element.find(".e-headerbar").first().height());

                var colorAppointment = this._appointmentColor(record);
                var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                var userTempHtml = this._getUserAppointmentTemplate(record);

                var startTimeDisp = this._timeMode == "12" ? Globalize.format(record[this.model.appointmentSettings["startTime"]], this._pattern.t) : Globalize.format(record[this.model.appointmentSettings["startTime"]], "HH:mm");
                var endTimeDisp = this._timeMode == "12" ? Globalize.format(record[this.model.appointmentSettings["endTime"]], this._pattern.t) : Globalize.format(record[this.model.appointmentSettings["endTime"]], "HH:mm");
                var alldayCellhtml = this.appTemplate.render({ id: record["AppTaskId"], appResize: this.model.enableAppointmentResize, currentview: this.currentView(), top: top, appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp, appWidth: appWidth + "px", allday: true, left: left + "px", userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-alldayappointment", appointData: colorAppointment.appointColor.appointData, appointCustomcss: colorAppointment.appointColor.appointCustomcss, appointtextcolor: colorAppointment.appointColor.appointtextcolor, value: colorAppointment.value, multiDiv: colorAppointment.multiDiv, orientation: this.model.orientation, uid: record.Guid });
                if (this.level <= level) {
                    var cellHeight = (level + 1) * (appHeight) + 27;
                    if (!this._mediaQuery)
                        this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").css('height', cellHeight + "px");
                    else {
                        this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").attr('style', "height:" + ((level == 0) ? 40 : cellHeight) + "px !important");
                        if (!this.model.enableRTL)
                            this.element.find(".e-alldaycells").first().css("border-left", "none");
                        else
                            this.element.find(".e-alldaycells").first().css("border-right", "none");
                    }
                }
                this.element.find(".e-headrealldaytable").append(alldayCellhtml);
            }
            this._reRenderScroller();
            this._alldayLevel = false;
            this.level = this.level < level ? level : this.level;
            if (record[this.model.appointmentSettings["recurrence"]] != 1) {
                var id = (record[this.model.appointmentSettings["recurrence"]] == undefined || record.ParentId == undefined) ? record["AppTaskId"] : record.ParentId;
                var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, parseInt(id));
                var list = this._currentAppointmentData;
                var templist = new ej.DataManager(list).executeLocal(query);
                if ((new Date(record[this.model.appointmentSettings["endTime"]])) < (templist[0][this.model.appointmentSettings["endTime"]]))
                    if (this.model.enableRTL)
                        this.element.find('[guid='+record["Guid"]+']').children('.e-lefthandle').remove();
                    else this.element.find('[guid=' + record["Guid"] + ']').children(".e-righthandle").remove();
                if ((new Date(record[this.model.appointmentSettings["startTime"]])) > (templist[0][this.model.appointmentSettings["startTime"]]))
                    if (this.model.enableRTL)
                        this.element.find('[guid=' + record["Guid"] + ']').children(".e-righthandle").remove();
                    else this.element.find('[guid=' + record["Guid"] + ']').children('.e-lefthandle').remove();
            }
        },
        _reRenderScroller: function () {
            var dialogObj = this._appointmentAddWindow.data("ejDialog");
            dialogObj.option("enableAnimation", false);
            dialogObj.close();
            var _scrollDiv = $('div#' + this._id + "_scroller");
            var scrollerObj = this.element.find(".e-scroller").data("ejScroller");
            var hasScrollbar = (typeof window.innerWidth === 'number') && window.innerWidth > document.documentElement.clientWidth;
            var scrollWidth = (this.model.orientation === "vertical") ? ((hasScrollbar && this._mediaQuery) ? this.element.outerWidth() + 17 : this.element.outerWidth()) - (this._mediaQuery && this.currentView() == "month" ? "" : this.element.find('.e-scrolltimecells').parent().outerWidth()) : (this.model.orientation == "horizontal") && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.element.width() - this.element.find(".e-resourceheadertable").width() : this.element.width();
            scrollerObj.model.width = scrollWidth - 2;
            scrollerObj.model.height = this.scHeight - 1 - (this.element.find('.e-draggableworkarea').offset().top - this.element.offset().top);
            scrollerObj.refresh();
            if (this.model.cellWidth != "") {
                var cols = this.currentView() == "workweek" ? this.model.workWeek.length : this.currentView() == "day" ? 1 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : 7;
                var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
                this.element.find(".e-draggableworkarea").width((this.element.width() > ((parseInt(this.model.cellWidth) + 1) * col) + this.element.find(".e-scrolltimecells").parent().width() + (_scrollDiv.ejScroller("isVScroll") ? 18 : 0)) ? ((parseInt(this.model.cellWidth) + 1) * col) - 1 : scrollWidth - 2 - (_scrollDiv.ejScroller("isVScroll") ? 18 : 0));
                this.element.find(".e-headeralldaydiv").width(this.element.find(".e-draggableworkarea").width());
            }
            this.element.find(".e-headeralldaydiv").width(this.element.find(".e-draggableworkarea").width());
            this.element.find(".e-scrolltimecells").css("top", -(this.element.find(".e-draggableworkarea").scrollTop()) + "px");
            this.element.find(".e-emptytd").length < 1 && this.model.orientation == "vertical" ? this.aTR.find(".e-headeralldaydiv").parent().next().append(this.element.find(".e-vscroll").length > 0 ? this._emptyCell() : "") : "";
            this.element.find(".e-emptyalldaytd").height(this.element.find(".e-alldaycells").height());
            (this._mediaQuery) && this._appointmentAddWindow.ejDialog("close");
            (hasScrollbar && this._mediaQuery) && this.element.find(".e-draggableworkarea").width((hasScrollbar) ? this.element.find(".e-draggableworkarea").width() - 17 : this.element.find(".e-draggableworkarea").width());
            dialogObj.option("enableAnimation", true);
        },
        _appointmentSort: function (appointments) {
            var proxy = this;
            if(this.currentView()!="day"){
            appointments.sort(function (a, b) {
                var d1 = a[proxy.model.appointmentSettings["endTime"]] - a[proxy.model.appointmentSettings["startTime"]];
                var d2 = b[proxy.model.appointmentSettings["endTime"]] - b[proxy.model.appointmentSettings["startTime"]];
                var date1 = new Date(a[proxy.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0);
                var date2 = new Date(b[proxy.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0);
                var date3 = new Date(a[proxy.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0);
                var date4 = new Date(b[proxy.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0);
                var d3 = a[proxy.model.appointmentSettings["startTime"]];
                var d4 = b[proxy.model.appointmentSettings["startTime"]];
                if (date1 == date2 && date3==date4)
                    return d3 - d4 || a.Guid.localeCompare(b.Guid);
                else if (date1 == date2)
                    return d2 - d1 || d3 - d4 || a.Guid.localeCompare(b.Guid);
                else
                    return d3 - d4 || a.Guid.localeCompare(b.Guid);
            });
            }
            else {
                appointments.sort(function (a, b) {
                    var d3 = a[proxy.model.appointmentSettings["startTime"]];
                    var d4 = b[proxy.model.appointmentSettings["startTime"]];
                    var d1 = a[proxy.model.appointmentSettings["endTime"]] - a[proxy.model.appointmentSettings["startTime"]];
                    var d2 = b[proxy.model.appointmentSettings["endTime"]] - b[proxy.model.appointmentSettings["startTime"]];
                    return d3 - d4 || d2 - d1 || a.Guid.localeCompare(b.Guid);
                });
            }
            return appointments;
        },
        _alldaySort: function (appointments) {
            var proxy = this, value = [];
            for (var i = 0; i < appointments.length; i++) {
                if (((new Date(appointments[i][proxy.model.appointmentSettings["endTime"]]) - new Date(appointments[i][proxy.model.appointmentSettings["startTime"]])) / 3600000) < 24 && !appointments[i][proxy.model.appointmentSettings["allDay"]])
                    value.push(appointments[i]);
            }
            return value;
        },
        _appointmentRecurrenceSort: function (appointments) {
            var proxy = this;
            appointments.sort(function (a, b) {
                var d1 = a[proxy.model.appointmentSettings["startTime"]];
                var d2 = b[proxy.model.appointmentSettings["startTime"]];
                return d1 - d2;
            });
            return appointments;
        },

        _renderMonthAppointment: function () {
            this.filterData = []; var filtered = []; this._app = [];
            var renderDate = (this.model.orientation == "vertical") ? this._dateRender : this.monthDays;
            var appointments = this._processed;
            appointments = this._sortAppById(appointments);
            appointments = this._appointmentSort(appointments);
            var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(renderDate[0]));
            predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(renderDate[renderDate.length - 1]).setHours(23, 59, 59)));
            this._appointmentsList = new ej.DataManager(appointments).executeLocal(new ej.Query().where(predicate));
            appointments = this._appointmentsList;

            if (!this.model.group) this.res1 = ["0"];
            for (var res = 0; res < this.res1.length; res++) {
                var day = 0, diff, count = renderDate.length;
                if (new Date(renderDate[0]) < this.model.minDate)
                    day = Math.floor((new Date(this.model.minDate) - new Date(renderDate[0])) / (1000 * 60 * 60 * 24));
                if (new Date(renderDate[renderDate.length - 1]) > this.model.maxDate) {
                    diff = Math.floor((new Date(renderDate[renderDate.length - 1]) - new Date(this.model.maxDate)) / (1000 * 60 * 60 * 24));
                    count = renderDate.length - diff;
                }
                for (day; day < count ; day++) {
                    this.filterData = [];
                    var predicate12 = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(renderDate[day]));
                    predicate12 = predicate12["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(renderDate[day]).setHours(23, 59, 59)));
                    predicate12 = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[res][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
                    appointments = new ej.DataManager(this._appointmentsList).executeLocal(new ej.Query().where(predicate12));
                    var renderedAlready = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(renderDate[day]));
                    appointments = new ej.DataManager(appointments).executeLocal(new ej.Query().where(renderedAlready));
                    appointments = this._appointmentSort(appointments);
                    this._app = $.merge(this._app, appointments);
                    for (var app = 0; app < appointments.length; app++) {
                        if ((appointments[app][this.model.appointmentSettings["allDay"]] == true) || (((new Date(appointments[app][this.model.appointmentSettings["endTime"]]) - new Date(appointments[app][this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) {
                            if ((new Date(new Date(renderDate[day]).setHours(0, 0, 0, 0)).getTime() == new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime()) &&
                (new Date(new Date(renderDate[day]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) &&
                (new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime())) {
                                this._renderAppointmentMonth(appointments[app], day, res);
                                filtered.push(appointments[app]);
                            }
                        } else {
                            if ((new Date(new Date(renderDate[day]).setHours(0, 0, 0, 0)).getTime() == new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime()) &&
     (new Date(new Date(renderDate[day]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) &&
     (new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime())) {
                                this._renderAppointmentMonth(appointments[app], day, res);
                                filtered.push(appointments[app]);
                            }
                        }
                    }
                }
            }
            if (this.model.enableAppointmentNavigation)
                this._nextPrevButton(filtered);
        },

        _wireResizeEvents: function () {
            if (this.model.enableAppointmentResize && !this.model.readOnly) {
                this.element.find("div.e-lefthandle,div.e-righthandle,div.e-tophandle,div.e-bottomhandle").on("mousedown", $.proxy(this._appointmentResizeStart, this));
                this._on(this.element.find("div.e-lefthandle,div.e-righthandle"), "mouseover", $.proxy(this._colHandlesOver, this));
                this._on(this.element.find("div.e-tophandle,div.e-bottomhandle"), "mouseover", $.proxy(this._rowHandlesOver, this));
            }
        },

        _unWireResizeEvents: function () {
            if (this.model.enableAppointmentResize) {
                this.element.find("div.e-lefthandle,div.e-righthandle,div.e-tophandle,div.e-bottomhandle").off("mousedown", $.proxy(this._appointmentResizeStart, this));
                this._off(this.element.find("div.e-lefthandle,div.e-righthandle"), "mouseover", $.proxy(this._colHandlesOver, this));
                this._off(this.element.find("div.e-tophandle,div.e-bottomhandle"), "mouseover", $.proxy(this._rowHandlesOver, this));
            }
        },

        _recuCollectionIndex: function (recurColl, _appList) {
            var index, model = this.model.appointmentSettings;
            for (var i = 0; i < recurColl.length; i++) {
                if (recurColl[i]["Guid"] == _appList["Guid"]) {
                    if (recurColl[i]["AppTaskId"] == _appList["AppTaskId"]) {
                        index = i;
                        break;
                    }
                }
            }
            return index;
        },

        _getIndex: function (day, record, dateRender, appointments, resValue) {
            var appLevelsInDay = [];
            var proxy = this;
            appointments = this._sortAppById(appointments);
            appointments = this._appointmentSort(appointments);
            var len = dateRender.length;
            var data = [];
            for (var app = 0; app < appointments.length; app++) {
                if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group)) {
                    if ((appointments[app][this.model.resources[this.model.resources.length - 1]["field"]] == resValue) &&
                        (new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                        (new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime() >= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                        (new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime())) {
                        data.push(appointments[app]);
                    }
                }
                else {
                    if ((new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                        (new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime() >= new Date(new Date(dateRender[day]).setHours(0, 0, 0, 0)).getTime()) &&
                        (new Date(new Date(appointments[app][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() <= new Date(new Date(appointments[app][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime())) {
                        data.push(appointments[app]);
                    }
                }
            }
            this.overlapList = data;
            data = this._sortAppById(data);
            data = this._appointmentSort(data);
            var index = -1;
            if (data.length > 0) {
                for (var i = 0; i < data.length; i++) {
					if (this.model.showTimeScale) {
					    var apps = this.element.find('[id=Appointment_' + data[i]["AppTaskId"] + ']').height();
						apps = ej.isNullOrUndefined(apps) ? false : true;
						if (apps) {
						    this._appointmentLevel(data[i]);
						    appLevelsInDay.push(this.appLevel);
						}
						if (data[i]["Guid"] == record["Guid"]) {
						    var appLevel = (appLevelsInDay.length > 0) ? this._smallestMissingNumber(appLevelsInDay) : -1;
						    if (data.length == 1) index = i;
						    else index = appLevel;
							break;
                        }
                    }
					else 
					    if (data[i]["Guid"] == record["Guid"]) index = i;
                }
            }
            return index == -1 ? 0 : index;
        },

        _renderAppointmentMonth: function (record, day, res) {
            var renderDate = (this.model.orientation == "vertical") ? this._dateRender : this.monthDays;
            var predicate12 = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(renderDate[0]));
            predicate12 = predicate12["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(renderDate[renderDate.length - 1]).setHours(23, 59, 59)));
            appointments = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(predicate12));
            var overlapCount = (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this._getIndex(day, record, renderDate, appointments, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : this._getIndex(day, record, renderDate, appointments, null);
            var startTime = new Date(record[this.model.appointmentSettings["startTime"]]);
            var endTime = new Date(record[this.model.appointmentSettings["endTime"]]);
            var appWidth, topPosition = 0;
            this.leftPosition = 0;
            this.CellWidth = this.model.orientation == "vertical" ? this.element.find(".e-monthcells").width() : this.element.find(".e-workcells").width();
            this.CellHeight = this.model.orientation == "vertical" ? this.element.find(".e-monthcells").height() : this.element.find(".e-workcells").height();
            var start = new Date(startTime);
            var end = new Date(endTime);
            var r_end = new Date(renderDate[renderDate.length - 1]);
            end = end > r_end ? r_end : end;
            end.setHours(23, 59);
            var maxDays = this._firstdayofweek == 0 ? 7 - (new Date(start).getDay()) : 7 - (new Date(start).getDay()) + this._firstdayofweek;
            if (!this.model.showTimeScale && this._oneWeek)
                maxDays = 7 - renderDate.indexOf(start.getTime());
            var diffInDays = Math.ceil((new Date(end) - new Date(start)) / (1000 * 3600) / 24);
            if (startTime <= endTime) {
                if (this.model.orientation == "vertical") {
                    diffInDays = diffInDays >= maxDays ? maxDays : diffInDays;
                    var rowPosition = Math.ceil((day + 1) / 7);
                    appWidth = (diffInDays * this.CellWidth) - ((this.CellWidth * 5) / 100);
                }
                else {
                    appWidth = this.model.showTimeScale || this.currentView() == "month" || this._isCustomView() ? (diffInDays * this.CellWidth) : (this.CellWidth * diffInDays) - parseInt((this.CellWidth * 10) / 100);
                }
                var resIndex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings.id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : 0;
                if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                    var cellId = this.model.orientation == "vertical" ? this.currentView() == "day" ? resIndex + 1 : this.currentView() == "workweek" || this._isCustomView() && this._oneWeek ? (resIndex * (this._dateRender.length)) + (day + 1) : this.currentView() == "workweek" ? (resIndex * (this._dateRender.length) + (new Date(startTime).getDay() + 1)) + ((rowPosition - 1) * (this.res1.length * this._dateRender.length)) : this._firstdayofweek == 0 ? (resIndex * (7) + (new Date(startTime).getDay() + 1)) + ((rowPosition - 1) * (this.res1.length * 7)) : (this.model.orientation == "vertical" && this._firstdayofweek != 0) ? ((resIndex * (7) + (new Date(startTime).getDay() + 1)) + ((rowPosition - 1) * (this.res1.length * 7)) - this._firstdayofweek) : day + 1 : day + 1;
                else
                    var cellId = this.model.orientation == "vertical" && this._isCustomView() && !this._oneWeek ? (resIndex * (7) + (new Date(startTime).getDay() + 1)) + ((rowPosition - 1) * (this.res1.length * 7)) : day + 1;
                if (this.model.orientation == "vertical" && this._firstdayofweek != 0 && new Date(startTime).getDay() < this._firstdayofweek)
                    var cellId = (resIndex * (7) + (new Date(startTime).getDay() + 1)) + ((rowPosition - 1) * (this.res1.length * 7)) - this._firstdayofweek + 7;
                var tempAppWidth = appWidth - 1;
                var userAppTemplId = (this.model.appointmentTemplateId) ? true : false;
                var userTempHtml = this._getUserAppointmentTemplate(record);
                var appHeight = (!this.model.appointmentTemplateId) ? 21 : ($(userTempHtml).css('height') == "100%" ? (this.CellHeight - this.element.find('.e-monthheader').height() - 15) : $(userTempHtml).css('height') == "0px" ? this.element.find(".e-monthappointment").height() : $(userTempHtml).height());
                appHeight = userAppTemplId ? this.model.showTimeScale && this.model.orientation == "vertical" ? appHeight : (this.model.orientation == "vertical" && (this.currentView() == "month" || (this._isCustomView() && !this._oneWeek))) ? (this.CellHeight - this.element.find('.e-monthheader').height() - 15) : ((this.CellHeight * 10) / 100) : appHeight;
                appWidth = (!this.model.appointmentTemplateId) ? appWidth : ($(userTempHtml).css('width') == "100%" || $(userTempHtml).css('width') == "0px" ? appWidth : $(userTempHtml).width());
                topPosition = (overlapCount * appHeight);
                this.leftPosition = 0;
                if (!ej.isNullOrUndefined(this.model.resources))
                    var newResCollection = this._getResourceCollection();
                var groupIndex = (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) ? this._findResourceIndex(newResCollection[0].dataSource, newResCollection[0].id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]) + 1 : 0;
                var resValue = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this._findResourceIndex(this.level_Resources, this.model.resources[this.model.resources.length - 1].resourceSettings.id, record[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) + 1 : 1;
                var colorAppointment = this._appointmentColor(record);
                var colCount = parseInt(this.CellWidth / (this.CellWidth <= appWidth ? this.CellWidth : (appWidth + (this.element.find(".e-monthappointment").outerWidth(true) - this.element.find(".e-monthappointment").width()))));
                var rowCount = parseInt((this.CellHeight - this.element.find('.e-monthheader').height() - 10) / (this.element.find(".e-monthappointment").length > 0 ? this.element.find(".e-monthappointment").outerHeight(true) : appHeight));
                if (userAppTemplId && $(userTempHtml).css('height') != "0px" ? ((overlapCount + 1) <= (colCount * rowCount)) : (this.CellHeight > this.element.find('.e-monthheader').height() + ((overlapCount + 1) * appHeight) + 10)) {
                    if (appWidth < tempAppWidth)
                        userAppTemplId && $(this.element.find(".e-appointwrapper")[cellId - 1]).attr('style', "height:" + (parseInt(overlapCount / colCount) + 1) * appHeight + "px !important;width:" + this.CellWidth + "px !important;position:absolute !important;");
                    else
                        userAppTemplId && $(this.element.find(".e-appointwrapper")[cellId - 1]);
                    appHeight -= 2;
                    if (topPosition > 1) topPosition -= 2;
                    if (this.leftPosition > 1) this.leftPosition -= 1;
                    var startTimeDisp = this._timeMode == "12" ? Globalize.format(record[this.model.appointmentSettings["startTime"]], this._pattern.t) : Globalize.format(record[this.model.appointmentSettings["startTime"]], "HH:mm");
                    var endTimeDisp = this._timeMode == "12" ? Globalize.format(record[this.model.appointmentSettings["endTime"]], this._pattern.t) : Globalize.format(record[this.model.appointmentSettings["endTime"]], "HH:mm");
                    var timeCellhtml = this.appTemplate.render({ id: record["AppTaskId"], appResize: this.model.enableAppointmentResize, currentview: this.currentView(), appHeight: appHeight, rtl: this.model.enableRTL, subject: record[this.model.appointmentSettings["subject"]], startTime: startTimeDisp, endTime: endTimeDisp, left: this.leftPosition + "px", top: topPosition, appWidth: appWidth + "px", userAppTemplId: userAppTemplId, userTemplate: userTempHtml, appClass: "e-monthappointment", appointData: colorAppointment.appointColor.appointData, appointCustomcss: colorAppointment.appointColor.appointCustomcss, appointtextcolor: colorAppointment.appointColor.appointtextcolor, value: colorAppointment.value, multiDiv: colorAppointment.multiDiv, uid: record.Guid });
                    if (this.model.orientation == "vertical")
                        $(this.element.find("div.e-appointwrapper")[cellId - 1]).append(timeCellhtml);
                    else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
                        $($(this.element.find(".e-workcellstab tr")[resValue + groupIndex - 1]).find("div.e-appointwrapper")[cellId - 1]).append(timeCellhtml);
                        if (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 0) {
                            var appointColor = this._getResourceColor(record);
                            var categoryClass = (appointColor.appointData == "") ? "e-categorycolor" : "";
                            $($(this.element.find(".e-resourceheadertr")[groupIndex - 1]).find("div.e-appointwrapper")[cellId - 1]).append("<div class='e-categorybar " + categoryClass + "' id='categorybar_" + record["Guid"] + "'style='position:absolute;height:" + (this.CellHeight - 3) + "px;background:" + appointColor.appointData + ";width:" + (appWidth + 1) + "px;left:" + this.leftPosition + "px;'></div>");
                        }
                    }
                    timeCellhtml = "";
                }
                else {
                    if ($("#" + this._id + "_" + Globalize.format(new Date(record[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + (cellId - 1)).length == 0) {
                        if (this.model.orientation == "vertical") {
                            var count = cellId - 1;
                            var DateString = new Date(record[this.model.appointmentSettings["startTime"]]).getTime();
                            var endDay = Math.round(((record[this.model.appointmentSettings["endTime"]] - record[this.model.appointmentSettings["startTime"]]) / 3600000) / 24);
                            var i = count;
                            do{
                                $(this.element.find("div.e-appointwrapper")[i]).append('<div id="' + this._id + "_" + Globalize.format(new Date(DateString), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + (i) + '" class="e-icon e-appsoverflow"></div>');
                                DateString = DateString + (60 * 60 * 24 * 1000);
                                i++;
                            }
                            while (i < endDay + count);
                        }
                        else {
                            var count = cellId - 1;
                            var DateString = new Date(record[this.model.appointmentSettings["startTime"]]).getTime();
                            var endDay = Math.round(((record[this.model.appointmentSettings["endTime"]] - record[this.model.appointmentSettings["startTime"]]) / 3600000) / 24);
                            var i = count;
                            do{
                                $($(this.element.find(".e-workcellstab tr")[resValue + groupIndex - 1]).find("div.e-appointwrapper")[i]).append('<div id="' + this._id + "_" + Globalize.format(new Date(DateString), "MM_dd_yyyy") + "_" + (resValue + groupIndex - 1) + "_" + (i) + '" class="e-icon e-appsoverflow"></div>');
                                DateString = DateString + (60 * 60 * 24 * 1000);
                                i++;
                            }
                            while (i < endDay + count);
                        }
                        topPosition = this.CellHeight - (this.element.find('.e-monthheader').height() + (this.element.find('.e-appsoverflow').height()) - 2);
                        this.element.find(".e-icon.e-appsoverflow").css(this.model.enableRTL ? "right" : "left", (this.CellWidth + this.leftPosition - 25) + "px").css("top", topPosition + "px");
                    }
                }
            }
            if (!record[this.model.appointmentSettings["recurrence"]]) {
                var id = (record.ParentId == undefined) ? record["AppTaskId"] : record.ParentId;
                var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, id);
                var templist = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                var tempMultiApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, record.Guid));
                if ((new Date(record[this.model.appointmentSettings["endTime"]])) < (templist[0][this.model.appointmentSettings["endTime"]])) {
                    if (this.model.enableRTL)
                        $(this.element.find("div.e-appointwrapper")[cellId - 1]).find('[guid='+ templist[0]["Guid"]+']').children(".e-lefthandle").remove();
                    else $(this.element.find("div.e-appointwrapper")[cellId - 1]).find('[guid=' + templist[0]["Guid"] + ']').children(".e-righthandle").remove();
                }
                if ((new Date(record[this.model.appointmentSettings["startTime"]])) > (templist[0][this.model.appointmentSettings["startTime"]])) {
                    if (this.model.enableRTL)
                        $(this.element.find("div.e-appointwrapper")[cellId - 1]).find('[guid=' + record["Guid"] + ']').children(".e-righthandle").remove();
					else $(this.element.find("div.e-appointwrapper")[cellId - 1]).find('[guid=' + record["Guid"] + ']').children(".e-lefthandle").remove();
                }
            }
            if (!ej.isNullOrUndefined(tempMultiApp) && tempMultiApp > 1) {
                $(this.element.find("div.e-appointwrapper")[cellId - 1]).find('[guid=' + record["Guid"] + ']').children(".e-lefthandle").remove();
                $(this.element.find("div.e-appointwrapper")[cellId - 1]).find('[guid=' + record["Guid"] + ']').children(".e-righthandle").remove();
            }
        },

        _quickWindowClose: function () {
            this._quickAppointWindow.ejDialog("close");
            this._processFocus();
        },

        _appointmentWindow: function (e) {
            this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close");
            var startTime, endTime, _target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
            var event = { startTime: new Date(this.cur_StartTime), endTime: this.cur_EndTime, target: e, resources: ((this.model.appointmentWindowOpen != null || this.model.cellDoubleClick != null) && !ej.isNullOrUndefined(this.model.resources)) ? this._getResourceValue(_target) : null };
            if (this._trigger("cellDoubleClick", event) || _target.hasClass("e-categorybar"))
                return false;
            if ((_target.hasClass("e-workcells") || _target.hasClass("e-alldaycells") || _target.hasClass("e-monthcells") || this._cellClick || _target.hasClass("e-detailedapp")) && !_target.hasClass("e-resourceheadercells") && !_target.hasClass("e-schedulemouseclose") && !_target.hasClass("e-appointinnertext") && !_target.hasClass("e-apptext") && !_target.hasClass("e-apptime")) {
                this._currentAction = ej.Schedule.Actions.Add;
                this._appointmentAddWindow.find(".e-recurwindow").css("display", "none");
                this._appointmentAddWindow.find(".e-appwindow").css("display", "block");
                var index = this._cellIndex;
                var renderDate = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : this.model.orientation == "vertical" ? this.dateRender : this._dateRender;
                var curDate = new Date(renderDate[index]);
                if ((this.model.minDate > curDate || this.model.maxDate < curDate)) { return false; }
                if (!this._cellClick) {
                    this.currentDate(new Date(this.dateRender[index]));
                    if (_target.hasClass("e-workcells")) {
                        var time = this.model.startHour + (_target.parent().index() / 2);
                        var timemin = time.toString().split(".");
                        startTime = new Date(this.currentDate()).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                        endTime = new Date(new Date(startTime).setMinutes(new Date(startTime).getMinutes() + 30));
                    }
                    else if (_target.hasClass("e-monthcells")) {
                        startTime = new Date(new Date(this.currentDate()).setHours(9));
                        startTime = new Date(startTime).setMinutes(00);
                        endTime = new Date(new Date(startTime).setMinutes(30));
                    }
                    else {
                        startTime = new Date(this.currentDate());
                        endTime = new Date(startTime);
                        this._appointmentAddWindow.find(".allday ").ejCheckBox({ checked: true });
                    }
                } else {
                    startTime = new Date(this.cur_StartTime);
                    endTime = new Date(this.cur_EndTime);
                }
                if (this._trigger("appointmentWindowOpen", event))
                    return false;
                this._appointmentAddWindow.find("#" + this._id + "subject").val(this.model.showQuickWindow ? this._quickAppointWindow.find(".subject").val() : "");
                this._appointmentAddWindow.find("#" + this._id + "location").val(this.model.showQuickWindow ? this._quickAppointWindow.find(".location").val() : "");
                this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", { value: Globalize.format(new Date(startTime), this._datepattern()) });
                this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", { value: Globalize.format(new Date(endTime), this._datepattern()) });
                this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.t));
                this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.t));
                this._appointmentAddWindow.find("#" + this._id + "_priority").ejDropDownList("option", "value", this.model.prioritySettings.dataSource[0][this.model.prioritySettings["value"]]);
                ($(e.currentTarget).hasClass("e-alldaycells") || this.currentView() == "month" || (this._isCustomView())) && this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
                this._timeZoneCollection(null);
                this._tempStart = this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value");
                this._tempEnd = this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value");
                this._alldayCheck();
                if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                    this._renderResourceData(_target, this._cellIndex);
                if (!this.model.readOnly) {
                    this._appointmentAddWindow.find(".e-appsave,#donerecur").ejButton("enable", true);
                    (this._mediaQuery) && this._appointmentAddWindow.find(".e-btndelete").css("display", "none");
                    this._appointmentAddWindow.ejDialog("open");
                }
                (!this._mediaQuery) && this._appointmentAddWindow.find(".subject").focus();
                this._cellClick = false;
            }
            else {
                if (!_target.hasClass("e-resourceheadercells") && !_target.hasClass("e-schedulemouseclose")) {
                    if (_target.hasClass("e-monthheader") || _target.hasClass("e-appsoverflow")) return false;
                    var target = (this._editKey) ? $(e) : _target.hasClass("e-appointment") || _target.hasClass("e-monthappointment") || _target.hasClass("e-alldayappointment") || _target.hasClass("e-agendaappointment") ? _target : _target.parents('.e-monthappointment,.e-appointment,.e-alldayappointment,.e-agendaappointment');
                    if (!ej.isNullOrUndefined(target[0].id)) {
                        this._currentAction = ej.Schedule.Actions.Save;
                        this._appUid = target.attr('guid');
                        var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                        var templist = new ej.DataManager(this._processed).executeLocal(query);
                        if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
                            this._parentId = templist[0]["ParentId"];
                            this._recurEditWindow.ejDialog("open");
                            this._recurEditWindow.focus();
                            this._deleteBeforeOpen();
                        }
                        else {
                            var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                            var temp = new ej.DataManager(this._processed).executeLocal(query);
                            this._showAppointmentDetails(this._appUid, true);
                        }
                        this.model.showQuickWindow && this._quickAppDetailsWindow.ejDialog("close");
                    }
                    this._editKey = false;
                }
            }
        },

        _editRecurrence: function (args) {
            this._appointmentAddWindow.find(".e-appwindow").toggle();
            (this._mediaQuery) && this._appointmentAddWindow.parent().parent().find(".e-dialog-icon").css("display", "none");
            this._appointmentAddWindow.find(".e-recurwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 300);
        },

        _showAppointmentDetails: function (appId, editOcurrence) {
            var query = editOcurrence == true ? new ej.Query().where("Guid", ej.FilterOperators.equal, appId) : new ej.Query().where("AppTaskId", ej.FilterOperators.equal, appId);
            var list = editOcurrence == true ? this._processed : this._currentAppointmentData;
            var templist = this._sortAppById(new ej.DataManager(list).executeLocal(query));
			templist[0][this.model.appointmentSettings["endTime"]] = templist[templist.length - 1][this.model.appointmentSettings["endTime"]];
            this._appointmentAddWindow.find('.subject').val(templist[0][this.model.appointmentSettings["subject"]]);
            this._appointmentAddWindow.find('.location').val(templist[0][this.model.appointmentSettings["location"]]);
            this._appointmentAddWindow.find('.id').val(templist[0]["AppTaskId"]);
            this._appointmentAddWindow.find('.e-description').val(templist[0][this.model.appointmentSettings["description"]] ? templist[0][this.model.appointmentSettings["description"]] : "");
            var temp = templist[0];
                this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(temp[this.model.appointmentSettings["startTime"]], this._datepattern()));
                this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(temp[this.model.appointmentSettings["endTime"]], this._datepattern()));
                this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(temp[this.model.appointmentSettings["startTime"]], this._pattern.t));
                this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(temp[this.model.appointmentSettings["endTime"]], this._pattern.t));
            if (this.model.prioritySettings.enable) {
                if (!ej.isNullOrUndefined(temp[this.model.appointmentSettings["priority"]])) {
                    this._appointmentAddWindow.find("#" + this._id + "_priority").ejDropDownList("option", { value: templist[0][this.model.appointmentSettings["priority"]] });
                }
                else {
                    this._appointmentAddWindow.find("#" + this._id + "_priority").ejDropDownList("option", { value: this.model.prioritySettings.dataSource[0][this.model.prioritySettings.value] });
                }
            }
            this._timeZoneCollection(temp);
                this._tempStart = this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value");
                this._tempEnd = this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value");

            templist[0][this.model.appointmentSettings["allDay"]] == 1 && (this._appointmentAddWindow.find(".allday ").ejCheckBox({ checked: true }));
            if (templist[0][this.model.appointmentSettings["recurrence"]] == 1) {
                this._recRule = templist[0][this.model.appointmentSettings["recurrenceRule"]];
                if (editOcurrence == true) {
                    (!ej.isNullOrUndefined(templist[0][this.model.appointmentSettings["recurrenceRule"]]) && templist[0][this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) && (this._currentAction = ej.Schedule.Actions.Save);
                    this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: true, enabled: false });
                    this._appointmentAddWindow.find(".e-repeatlbl").addClass('e-disable');
                    this._appointmentAddWindow.find("." + this._id + "summary").css("display", "none");
                    this._appointmentAddWindow.find(".e-recurFinalRule").html("");                    
                }
                else {
                    this.recurrenceRuleSplit(templist[0][this.model.appointmentSettings["recurrenceRule"]], temp[this.model.appointmentSettings["recurrenceExDate"]]);
                    this._appointmentAddWindow.find(".recurstartdate").ejDatePicker({ value: Globalize.format(temp[this.model.appointmentSettings["startTime"]], this._datepattern()) });
                    this._appointmentAddWindow.find("." + this._id + "summary").css("display", "table-row");
                    this._appointmentAddWindow.find(".e-recurFinalRule").html(this._showRecurrenceFields(appId));
                    this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: true });
                    this._appointmentAddWindow.find(".e-recuredit").css("display", "inline");
                }
            }
            if (this.model.categorizeSettings.enable && !ej.isNullOrUndefined(temp[this.model.appointmentSettings["categorize"]])) {
                this._showCategorizeDetails(temp);
            }
            if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                this._showMultipleResourceDetails(temp);
            var data = { appointment: templist[0], edit: editOcurrence };
            if (this._trigger("appointmentWindowOpen", data))
                return false;
            this._alldayCheck();
            var enableDisable = (this.model.readOnly) ? "disable" : "enable";
            this._appointmentAddWindow.find(".e-appsave,#donerecur").ejButton(enableDisable, true);
            (this._mediaQuery) && this._appointmentAddWindow.find(".e-btndelete").css("display", "block");
            this._appointmentAddWindow.ejDialog("open");
            (!this._mediaQuery) && this._appointmentAddWindow.find(".subject").focus();
        },

        _showRecurrenceFields: function (appId) {
            var _recurRule = "", _recurOpt = "", _recurType = this._appointmentAddWindow.find('.recurtype');
            _recurRule = ej.isNullOrUndefined(this._rRule.interval) ? this._getLocalizedLabels("Every") + " " : this._getLocalizedLabels("Every") + " " + this._rRule.interval + " ";
            switch (this._rRule.freq) {
                case "DAILY":
                    _recurType.ejDropDownList({ selectedItemIndex: 0 });
                    _recurRule += this._getLocalizedLabels("RecurrenceDay").toLowerCase();
                    break;
                case "WEEKLY":
                    _recurType.ejDropDownList({ selectedItemIndex: 1 });
                    this._appointmentAddWindow.find('.weekdays').ejCheckBox({ checked: false });
                    var _days = this._rRule.weekDays.split(',');
                    for (var i = 0; i < _days.length; i++) {
                        this._appointmentAddWindow.find('.e-weekly' + _days[i]).ejCheckBox({ checked: true });
                        _recurOpt += this._dayShortNames[this._dayNames.indexOf(_days[i])] + (i == _days.length - 1 ? "" : ", ");
                    }
                    _recurRule += this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                    break;
                case "MONTHLY":
                    _recurType.ejDropDownList({ selectedItemIndex: 2 });
                    if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                        this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton({ checked: true });
                        this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox({ value: parseInt(this._rRule.monthDay) });
                        _recurOpt = parseInt(this._rRule.monthDay);
                    }
                    else {
                        this._appointmentAddWindow.find('.e-monthposition').ejRadioButton({ checked: true });
                        this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList({ selectedItemIndex: this._dayNames.indexOf(this._rRule.weekDays) });
                        this._appointmentAddWindow.find('.monthsrt').ejDropDownList({ selectedItemIndex: this._rRule.setPositions - 1 });
                        _recurOpt = this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[this._dayNames.indexOf(this._rRule.weekDays)];
                    }
                    _recurRule += this._getLocalizedLabels("RecurrenceMonth").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                    break;
                case "YEARLY":
                    _recurType.ejDropDownList({ selectedItemIndex: 3 });
                    if (!ej.isNullOrUndefined(this._rRule.monthDay)) {
                        this._appointmentAddWindow.find('.e-yearrecurday').ejRadioButton({ checked: true });
                        this._appointmentAddWindow.find('.yearmonth').ejDropDownList({ selectedItemIndex: this._rRule.month - 1 });
                        this._appointmentAddWindow.find('.yeardate').ejNumericTextbox({ value: parseInt(this._rRule.monthDay) });
                        _recurOpt = this._monthNames[this._rRule.month - 1] + " " + parseInt(this._rRule.monthDay);
                    }
                    else {
                        this._appointmentAddWindow.find('.e-yearrecurposi').ejRadioButton({ checked: true });
                        this._appointmentAddWindow.find('.yearsrtday').ejDropDownList({ selectedItemIndex: this._dayNames.indexOf(this._rRule.weekDays) });
                        this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList({ selectedItemIndex: this._rRule.month - 1 });
                        this._appointmentAddWindow.find('.yearsrt').ejDropDownList({ selectedItemIndex: this._rRule.setPositions - 1 });
                        _recurOpt = this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[this._dayNames.indexOf(this._rRule.weekDays)] + " " + this._monthNames[this._rRule.month - 1];
                    }
                    _recurRule += this._getLocalizedLabels("RecurrenceYear").toLowerCase() + " " + this._getLocalizedLabels("On").toLowerCase() + " " + _recurOpt;
                    break;
            }
            this._rRuleFreq = (ej.isNullOrUndefined(this._rRuleFreq) || this._rRuleFreq) ? (_recurType.ejDropDownList("option", "value").toUpperCase()) : this._rRuleFreq;
            this._appointmentAddWindow.find('.e-recurevery').ejNumericTextbox({ value: ej.isNullOrUndefined(this._rRule.interval) ? 1 : this._rRule.interval });
            if (!ej.isNullOrUndefined(this._rRule.count)) {
                this._appointmentAddWindow.find(".e-recurafter").ejRadioButton({ checked: true });
                this._appointmentAddWindow.find('.e-recurcount').ejNumericTextbox({ value: this._rRule.count });
                _recurRule += ", " + this._rRule.count + this._getLocalizedLabels("Times").toLowerCase();
            }
            else if (!ej.isNullOrUndefined(this._rRule.until)) {
                this._appointmentAddWindow.find(".e-recuruntil").ejRadioButton({ checked: true });
                this._appointmentAddWindow.find('.e-until').ejDatePicker({ value: new Date(this._rRule.until) });
                _recurRule += " " + this._getLocalizedLabels("Until").toLowerCase() + " " + Globalize.format(new Date(this._rRule.until), this._datepattern());
            }
            else
                this._appointmentAddWindow.find(".e-recurnoend").ejRadioButton({ checked: true });

            return _recurRule;
        },

        _closerecur: function (args) {
            if (this._appointmentAddWindow.find('.e-recuruntil').ejRadioButton("option", "checked") && args.model.text != this._getLocalizedLabels("Cancel") && this._appointmentAddWindow.find("#recurstartdate").ejDatePicker("option", "value") > this._appointmentAddWindow.find(".e-until").ejDatePicker("option", "value")) {
                this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("WrongPattern"));
                this._alertWindow.ejDialog("open");
                return false;
            }
            if (args.model.text == this._getLocalizedLabels("Cancel")) {
                this._appointmentAddWindow.find(".e-recurFinalRule").html() == "" && this._appointmentAddWindow.find("#" + this._id + "recurrence").ejCheckBox({ checked: false });
            }
            else {
                var interval, monthDate, endRecur = "", recurOption = "", _weekRule = "";
                interval = this._findInterval(this._appointmentAddWindow.find('.e-recurevery'));
                switch (this._rRuleFreq) {
                    case "DAILY":
                        this._recRule = "FREQ=" + this._rRuleFreq + ";INTERVAL=" + interval;
                        break;
                    case "WEEKDAYS":
                        this._recRule = "FREQ=WEEKLY;BYDAY=MO,TU,WE,TH,FR";
                        _weekRule = "MO,TU,WE,TH,FR";
                        break;
                    case "WEEKLY":
                        var _weekChkList = this._appointmentAddWindow.find("input[type='checkbox'][name='weekdays']");
                        var _dayName = "", _byDay = "";
                        for (var i = 0; i < _weekChkList.length; i++) {
                            $(_weekChkList[i]).ejCheckBox("option", "checked") == true ? _weekRule += (_weekRule != "" ? "," : "") + this._dayNames[i].toUpperCase() : "";
                            $(_weekChkList[i]).ejCheckBox("option", "checked") == true ? _dayName += (_dayName != "" ? ", " : "") + this._dayShortNames[i] : "";
                        }
                        if (_weekRule == "") {
                            if (this.model.enableRecurrenceValidation) {
                                $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                                this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("WrongPattern"));
                                this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                                this._alertWindow.ejDialog("open");
                                return false;
                            } else {
                                var date = this._appointmentAddWindow.find('.recurstartdate').ejDatePicker("option", "value");
                                _weekRule = this._dayNames[date.getDay()].toUpperCase();
                            }
                        }
                        var _rule = this._getLocalizedLabels("RecurrenceWeek").toLowerCase() + " on " + _dayName;
                        this._appointmentAddWindow.find(".e-recurRule").html(_rule);
                        this._recRule = "FREQ=" + this._rRuleFreq + ";INTERVAL=" + interval + ";" + "BYDAY=" + _weekRule;
                        break;
                    case "MONTHLY":
                        if (this._appointmentAddWindow.find('.e-monthdaytype').ejRadioButton("option", "checked")) {
                            monthDate = this._appointmentAddWindow.find('.e-monthdate').ejNumericTextbox("option", "value");
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYMONTHDAY=" + monthDate + ";INTERVAL=" + interval
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + monthDate;
                        }
                        else {
                            var _setPos = this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "selectedItemIndex");
                            var dayName = this._appointmentAddWindow.find('.e-monthsrtday').ejDropDownList("option", "selectedItemIndex");
                            _setPos = _setPos < 4 ? (_setPos + 1) : -1;
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYDAY=" + this._dayNames[dayName] + ";BYSETPOS=" + _setPos + ";INTERVAL=" + interval;
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._appointmentAddWindow.find('.monthsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[dayName];
                        }
                        break;
                    case "YEARLY":
                        if (this._appointmentAddWindow.find("#" + this._id + "yearday").ejRadioButton("option", "checked")) {
                            var _monthIndex = this._appointmentAddWindow.find('.yearmonth').ejDropDownList("option", "selectedItemIndex");
                            var _monthDate = this._findInterval(this._appointmentAddWindow.find('.yeardate'));
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYMONTHDAY=" + _monthDate + ";BYMONTH=" + (_monthIndex + 1) + ";INTERVAL=" + interval;
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._monthNames[_monthIndex] + " " + _monthDate;
                        }
                        else {
                            var _setPos = this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "selectedItemIndex");
                            var _monthIndex = this._appointmentAddWindow.find('.yearsrtmonth').ejDropDownList("option", "selectedItemIndex");
                            var dayName = this._appointmentAddWindow.find('.yearsrtday').ejDropDownList("option", "selectedItemIndex");
                            _setPos = _setPos < 4 ? (_setPos + 1) : -1;
                            this._recRule = "FREQ=" + this._rRuleFreq + ";BYDAY=" + this._dayNames[dayName] + ";BYMONTH=" + (_monthIndex + 1) + ";BYSETPOS=" + _setPos + ";INTERVAL=" + interval;
                            recurOption = " " + this._getLocalizedLabels("On").toLowerCase() + " " + this._appointmentAddWindow.find('.yearsrt').ejDropDownList("option", "value").toLowerCase() + " " + this._dayFullNames[dayName] + " " + this._monthNames[_monthIndex];
                        }
                        break;
                }
                if (this._appointmentAddWindow.find('.e-recurafter').ejRadioButton("option", "checked")) {
                    var _count = this._findInterval(this._appointmentAddWindow.find('.e-recurcount'));
                    this._recRule += ";COUNT=" + _count;
                    endRecur = _count ? ", " + _count + " " + this._getLocalizedLabels("Times") : ", ";
                }
                else if (this._appointmentAddWindow.find('.e-recuruntil').ejRadioButton("option", "checked")) {
                    var date = Globalize.format(new Date(this._appointmentAddWindow.find('.e-until').ejDatePicker("option", "value")), this._pattern.d);
                    //this._recRule += ";UNTIL=" + date;
                    //endRecur = " " + this._getLocalizedLabels("Until").toLowerCase() + " " + date;
                    var _count = this._findRecurCount(interval, _weekRule);
                    this._recRule += ";COUNT=" + _count;
                    endRecur = _count ? ", " + _count + " " + this._getLocalizedLabels("Times") : ", ";
                }
                var _every = (this._rRuleFreq != "WEEKDAYS" ? this._getLocalizedLabels("Every") : "") + " " + (interval > 1 ? interval + " " : "");
                var _rRuleStr = _every + this._appointmentAddWindow.find(".e-recurRule").html() + recurOption + endRecur;
                this._appointmentAddWindow.find("." + this._id + "summary").css("display", "table-row");
                this._appointmentAddWindow.find(".e-recurFinalRule").html(_rRuleStr);
                this._appointmentAddWindow.find(".e-recuredit").css("display", "inline");
                recurOption = "";
            }
            this._appointmentAddWindow.find(".e-recurwindow").toggle();
            this._appointmentAddWindow.find(".e-appwindow").css("margin-left", 700).toggle().animate({ "margin-left": 10 }, 300);
            (this._mediaQuery) && this._appointmentAddWindow.parent().parent().find(".e-dialog-icon").css("display", "block");
            this._appointmentAddWindow.find('.done').focus();
        },

        _findRecurCount: function (_interval, byDay) {
            var _count = 0, _interval, byDay;
            var day_start = new Date(this._appointmentAddWindow.find('.recurstartdate').ejDatePicker("option", "value"));
            var recurEndDate = new Date(this._appointmentAddWindow.find('.e-until').ejDatePicker("option", "value"));
            while (day_start <= recurEndDate) {
                switch (this._rRuleFreq) {
                    case "DAILY":
                        day_start = new Date(day_start.setDate(day_start.getDate() + _interval));
                        break;
                    case "WEEKLY":
                    case "WEEKDAYS":
                        var curDay = this._dayNames[new Date(day_start).getDay()];
                        if (byDay.length - 1 != byDay.indexOf(curDay)) {
                            day_start = this._weeklyDayFind(day_start, byDay);
                        }
                        else {
                            day_start = new Date(day_start.setDate(day_start.getDate() + (day_start.getDay() * -1)));
                            day_start = new Date(day_start.setDate(day_start.getDate() + (_interval * 7) - 1));
                            day_start = this._weeklyDayFind(day_start, byDay);
                        }
                        break;
                    case "MONTHLY":
                        day_start = new Date(day_start.setMonth(day_start.getMonth() + _interval));
                        break;
                    case "YEARLY":
                        day_start = new Date(day_start.setFullYear(day_start.getFullYear() + _interval));
                        break;
                }
                _count++;
            }
            return _count;
        },

        _recurDateChange: function (args) {
            this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", args.value);
            if (this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value") < this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value"))
                this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", args.value);
        },

        _findInterval: function (numericElement) {
            var interval;
            interval = numericElement.ejNumericTextbox("option", "value");
            return interval = ej.isNullOrUndefined(interval) ? 0 : interval;
        },

        _cancel: function () {
            this._appointmentAddWindow.ejDialog("close");
        },

        _timeChange: function (e) {
            var obj = this._appointmentAddWindow.find("#" + this._id + "starttime").data("ejTimePicker");
            if (obj.showDropdown && obj._id == this._id + "starttime") {
                if (this._appointmentAddWindow.find('.startutc').val() != this._appointmentAddWindow.find('.endutc').val()) {
                    this._utcValue("", "starttime");
                }
                else {
                    var end = this._timeFormat(this._appointmentAddWindow.find('#' + this._id + 'endtime').ejTimePicker("option", "value")).split(":");
                    var endTime = new Date(new Date("1/1/2000").setHours(parseInt(end[0], 10), parseInt(end[1], 10)));
                    var start = this._timeFormat(this._appointmentAddWindow.find('#' + this._id + 'starttime').ejTimePicker("option", "value")).split(":");
                    var startTime = new Date(new Date("1/1/2000").setHours(parseInt(start[0], 10), parseInt(start[1], 10)));
                    var prev = this._timeFormat(e.prevTime).split(":");
                    var prevtime = new Date(new Date("1/1/2000").setHours(parseInt(prev[0], 10), parseInt(prev[1], 10)));
                    var updatetime = new Date(new Date(startTime).setHours(startTime.getHours() + endTime.getHours() - prevtime.getHours(), startTime.getMinutes() + endTime.getMinutes() - prevtime.getMinutes()));
                    this._appointmentAddWindow.find('#' + this._id + 'endtime').ejTimePicker("option", "value", updatetime);
                }
            }
        },

        _renderControls: function (e) {
            var control = this._appointmentAddWindow;
            control.find('.allday').ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass, change: $.proxy(this._alldayCheck, this) });
            control.find('.startenddate').ejDatePicker({ buttonText: this._getLocalizedLabels("Today"), startDay: this._firstdayofweek, enableRTL: this.model.enableRTL, width: '100%', value: this.currentDate(), locale: this.model.locale, dateFormat: this.model.dateFormat, cssClass: this.model.cssClass, minDate: this.model.minDate, maxDate: this.model.maxDate });
            control.find("#" + this._id + "starttime").ejTimePicker({ enableRTL: this.model.enableRTL, width: "100%", value: this.currentDate(), locale: this.model.locale, change: $.proxy(this._timeChange, this), cssClass: this.model.cssClass, timeFormat: (this._timeMode == "12") ? this._pattern.t : "HH:mm" });
            control.find("#" + this._id + "endtime").ejTimePicker({ enableRTL: this.model.enableRTL, width: "100%", value: this.currentDate(), locale: this.model.locale, cssClass: this.model.cssClass, timeFormat: (this._timeMode == "12") ? this._pattern.t : "HH:mm" });

            control.find("#" + this._id + "startTimeZone").ejDropDownList({ enableRTL: this.model.enableRTL, dataSource: this.model.timeZoneCollection.dataSource, fields: { text: "text", id: "id", value: "value" }, width: "100%", change: $.proxy(this._startUtc, this), cssClass: this.model.cssClass });
            control.find("#" + this._id + "endTimeZone").ejDropDownList({ enableRTL: this.model.enableRTL, dataSource: this.model.timeZoneCollection.dataSource, fields: { text: "text", id: "id", value: "value" }, width: "100%", change: $.proxy(this._endUtc, this), cssClass: this.model.cssClass });
            if (this.model.prioritySettings.enable && !(this.model.prioritySettings.dataSource instanceof ej.DataManager)) {
                this._renderPriorityItems(control);
            }
            if (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                this._renderAppWindowResources(control);
            if (this.model.categorizeSettings.enable && !(this.model.categorizeSettings.dataSource instanceof ej.DataManager))
                this._renderCategoryItems(control);
            control.find('.recurrence').ejCheckBox({ enableRTL: this.model.enableRTL, change: $.proxy(this._repeat, this), cssClass: this.model.cssClass });
            control.find('.e-btndone').ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: (this._mediaQuery) ? "100%" : "100px", click: $.proxy(this._saveAppointment, this), cssClass: this.model.cssClass });
            control.find('.cancel').ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: (this._mediaQuery) ? "100%" : "100px", click: $.proxy(this._cancel, this), cssClass: this.model.cssClass });
            control.find('.recurtype').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "recurtypelist", width: "100%", select: $.proxy(this._recurtype, this), cssClass: this.model.cssClass });
            control.find('.dailyrecurtype').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            control.find('.e-dailyeveryday').ejRadioButton({ enableRTL: this.model.enableRTL, checked: true, cssClass: this.model.cssClass });
            control.find('.everydaycount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "everydaycount", minValue: 1, value: 1, width: "100px", float: "left", cssClass: this.model.cssClass });
            control.find('.recurends').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            control.find('.e-recurcount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "dailyoccurance", minValue: 1, value: 10, width: "65px", cssClass: this.model.cssClass, change: $.proxy(this._endAfterChange, this) });
            control.find('.weekcount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "weekcount", minValue: 1, value: 1, width: "100px", cssClass: this.model.cssClass });
            control.find('.e-recurevery').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "recurevery", minValue: 1, value: 1, width: "100px", cssClass: this.model.cssClass });
            control.find('.weekdays').ejCheckBox({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            control.find('.e-monthposition').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            control.find('.e-monthdaytype').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            control.find('.e-monthdate').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "monthdate", minValue: 1, maxValue: 31, value: 1, width: "70px", cssClass: this.model.cssClass, change: $.proxy(this._monthDateChange, this) });
            control.find('.monthnumber').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "monthnumber", minValue: 1, value: 1, width: "70px", cssClass: this.model.cssClass });
            control.find('.yearcount').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "yearcount", minValue: 1, value: 1, width: "50px", cssClass: this.model.cssClass });
            control.find('.e-yearrecurposi,.e-yearrecurday').ejRadioButton({ enableRTL: this.model.enableRTL, cssClass: this.model.cssClass });
            control.find('.yeardate').ejNumericTextbox({ enableRTL: this.model.enableRTL, name: "yeardate", minValue: 1, maxValue: 31, value: 1, width: "70px", cssClass: this.model.cssClass, change: $.proxy(this._yearDateChange, this) });
            control.find('.recurbutton').ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "100px", click: $.proxy(this._closerecur, this), cssClass: this.model.cssClass });
            control.find('.monthsrt').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "monthsrtlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._monthDayChange, this) });
            control.find('.e-monthsrtday').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "monthsrtdaylist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._monthDayChange, this) });
            control.find('.yearsrt').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearsrtlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
            control.find('.yearmonth').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearmonthlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDateChange, this) });
            control.find('.yearsrtday').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearsrtdaylist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
            control.find('.yearsrtmonth').ejDropDownList({ enableRTL: this.model.enableRTL, targetID: "yearsrtmonthlist", width: "100px", cssClass: this.model.cssClass, change: $.proxy(this._yearDayChange, this) });
            control.find('.recurstartdate').ejDatePicker({ buttonText: this._getLocalizedLabels("Today"), startDay: this._firstdayofweek, enableRTL: this.model.enableRTL, locale: this.model.locale, cssClass: this.model.cssClass, dateFormat: this.model.dateFormat, change: $.proxy(this._recurDateChange, this), minDate: this.model.minDate, maxDate: this.model.maxDate });
            control.find('.e-until').ejDatePicker({ buttonText: this._getLocalizedLabels("Today"), startDay: this._firstdayofweek, enableRTL: this.model.enableRTL, locale: this.model.locale, cssClass: this.model.cssClass, dateFormat: this.model.dateFormat, change: $.proxy(this._endOnChange, this) });
            (this._mediaQuery) && control.find('.e-btndelete').ejButton({ enableRTL: this.model.enableRTL, showRoundedCorner: true, width: "100%", click: $.proxy(this._mobileAppDelete, this), cssClass: this.model.cssClass });
        },

        _mobileAppDelete: function () {
            if ((this._mediaQuery) && !(ej.isNullOrUndefined(this._editOnlyClicked)) && !(this._editOnlyClicked) && this._appointmentAddWindow.find(".recurrence").ejCheckBox("option", "checked")) {
                this._deleteApp(this._appUid);
            }
            else
                this._deleteAppoint();
            this._appointmentAddWindow.ejDialog("close");
        },

        _renderPriorityItems: function (control) {
            if (!ej.isNullOrUndefined(control)) {
                control.find('.priority').ejDropDownList({
                    enableRTL: this.model.enableRTL,
                    template: ej.isNullOrUndefined(this.model.prioritySettings["template"]) ? "<div class='e-" + "${" + this.model.prioritySettings["value"] + "}'></div><div>${" + this.model.prioritySettings["text"] + "}</div>" : this.model.prioritySettings["template"] + "${" + this.model.prioritySettings["text"] + "}",
                    dataSource: this.model.prioritySettings.dataSource, fields: { text: this.model.prioritySettings["text"], value: this.model.prioritySettings["value"] },
                    width: "100%",
                    cssClass: this.model.cssClass
                });
            }
        },

        _renderCategoryItems: function (control) {
            if (!ej.isNullOrUndefined(control)) {
              control.find('.e-categorizevalue').ejAutocomplete({
                    enableRTL: this.model.enableRTL,
                    showPopupButton: true,
                    width: "100%", cssClass: this.model.cssClass,
                    multiSelectMode: this.model.categorizeSettings.allowMultiple? "visualmode":"none",
                    dataSource: this.model.categorizeSettings.dataSource, fields: { text: this.model.categorizeSettings["text"], value: this.model.categorizeSettings["color"], key: this.model.categorizeSettings["id"], fontColor: this.model.categorizeSettings["fontColor"] },
                    template: this._categorizeTemplate,
              });
            }
        },

        _appointmentColor: function (record) {
            var appointColor = (this.model.categorizeSettings.enable && !ej.isNullOrUndefined(record[this.model.appointmentSettings["categorize"]]) && record[this.model.appointmentSettings["categorize"]] != "") ? this._getCategorizeColor(record) : (!ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources) ? this._getResourceColor(record) : { appointData: "", appointCustomcss: "", appointtextcolor: "" };

            if (this.model.categorizeSettings.enable && this.model.categorizeSettings.allowMultiple && !ej.isNullOrUndefined(record[this.model.appointmentSettings["categorize"]])) {
                var categorizeItems = record[this.model.appointmentSettings["categorize"]].split(",");
                var categorizeItemsLength = categorizeItems.length;
                var value = (categorizeItemsLength > 1) ? true : false;
                var multiDiv = this._categorizeMultipleDiv(record, categorizeItems, categorizeItemsLength);
            }
            if (!ej.isNullOrUndefined(multiDiv) && appointColor.appointData.length > 0 && this.model.categorizeSettings.allowMultiple)
                return { appointColor: appointColor, value: value, multiDiv: multiDiv };
            else if (appointColor.appointData.length > 0 && !this.model.categorizeSettings.allowMultiple || !ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                return { appointColor: appointColor };
            else {
                appointData = "", appointCustomcss = "", appointtextcolor = "", value = "";
                return { appointColor: appointColor, value: value, multiDiv: multiDiv };
            }
        },

        _categorizeMultipleDiv: function (record, categorizeItems, categorizeItemsLength) {
            var renderData = [];
            for (i = 0; i <= categorizeItemsLength - 1; i++) {
                if (i != categorizeItemsLength - 1) {
                    var dataSet = this.model.categorizeSettings;
                    var categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, this.model.categorizeSettings["id"], categorizeItems[i]);
                    (ej.isNullOrUndefined(categorizeDataIndex)) && (categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, this.model.categorizeSettings["id"], parseInt(categorizeItems[i])));
                    if (ej.isNullOrUndefined(categorizeDataIndex)) {
                        record[this.model.appointmentSettings["categorize"]] = "";
                        return null;
                    }
                    else
                        renderData.push({ text: dataSet.dataSource[categorizeDataIndex][dataSet.text], value: dataSet.dataSource[categorizeDataIndex][dataSet.color], key: dataSet.dataSource[categorizeDataIndex][dataSet.id], fontColor: dataSet.dataSource[categorizeDataIndex][dataSet.fontColor] })
                }
            }
            var template = this._categorizeMultipleTemplate.render({ renderData: renderData });
            return template;
        },

        _getResCategorySaveData: function (categorizeVal, autoVal, obj, maxId, temp, starttime, endtime) {
            var args = [], newAppointment, data, prevdata;
            newAppointment = this._addAppData(obj, maxId, new Date(obj[this.model.appointmentSettings["startTime"]]), new Date(obj[this.model.appointmentSettings["endTime"]]), maxId);
            if (categorizeVal.length > 0) {
                var categorizeItemsLength = categorizeVal[categorizeVal.length - 1].length - 1; var idValue = "";
                for (var i = 0; i <= categorizeItemsLength; i++) {
                    var getId = categorizeVal[categorizeVal.length - 1][i][this.model.categorizeSettings["id"]];
                    idValue = (i != categorizeItemsLength) ? idValue + getId.toString() + "," : idValue + getId.toString();
                }
                newAppointment[this.model.appointmentSettings["categorize"]] = idValue;
            }
            if (autoVal.length > 0) {
                var categorizeVar = newAppointment[this.model.appointmentSettings["categorize"]];
                var resources = this.model.appointmentSettings.resourceFields.toString().split(',');
                var newResCollection = this._getResourceCollection();
                if (autoVal[autoVal.length - 1].length > 0) {
                    for (var j = 0; j < autoVal[autoVal.length - 1].length; j++) {

                        newAppointment = this._addAppData(obj, maxId, new Date(obj[this.model.appointmentSettings["startTime"]]), new Date(obj[this.model.appointmentSettings["endTime"]]), maxId);
                        newAppointment[this.model.appointmentSettings["recurrenceId"]] = obj[this.model.appointmentSettings["recurrenceId"]] == null ? null : maxId;
                        newAppointment[this.model.appointmentSettings["categorize"]] = categorizeVar;

                        if (autoVal[autoVal.length - 1].length > 1) {
						    obj.Guid = this._guidFormatGenerate();
                            obj["AppTaskId"] = maxId;
                            maxId++;
                        }
                        var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, maxId);
                        var templist = new ej.DataManager(this._processed).executeLocal(query);
                        maxId = (templist.length == 0) ? maxId : this._processed[this._processed.length - 1]["AppTaskId"] + 1;
                        newAppointment[resources[resources.length - 1]] = autoVal[resources.length - 1][j][newResCollection[newResCollection.length - 1]["id"]];

                        var id = autoVal[resources.length - 1][j][newResCollection[newResCollection.length - 1]["id"]];
                        for (var i = resources.length - 1; i > 0; i--) {
                            data = this.render_Resources[i] instanceof ej.DataManager ? this.render_Resources[i] : ej.DataManager(this.render_Resources[i]);
                            query = new ej.Query().where(newResCollection[i].id, ej.FilterOperators.equal, id);
                            appointId = data.executeLocal(query);
                            prevdata = this.render_Resources[i - 1] instanceof ej.DataManager ? this.render_Resources[i - 1] : ej.DataManager(this.render_Resources[i - 1]);
                            query1 = new ej.Query().where(newResCollection[i - 1].id, ej.FilterOperators.equal, appointId[0][newResCollection[i].groupId]);
                            appointGroupId = prevdata.executeLocal(query1);
                            id = appointGroupId[0][newResCollection[i - 1].id];
                            newAppointment[resources[i - 1]] = id;

                        }
                        args.push(newAppointment);
                    }
                }
                else {
                    var resources = this.model.appointmentSettings.resourceFields.toString().split(',');
                    newAppointment = this._addAppData(obj, maxId, new Date(obj[this.model.appointmentSettings["startTime"]]), new Date(obj[this.model.appointmentSettings["endTime"]]), maxId);
                    newAppointment[this.model.appointmentSettings["categorize"]] = categorizeVar;
                    if (resources.length > 1) {
                        for (i = 0; i < resources.length; i++) {
                            newAppointment[resources[i]] = this.model.resources[i].resourceSettings.dataSource[0].id
                        }
                    }
                    else
                        newAppointment[resources[resources.length - 1]] = this.model.resources[0].resourceSettings.dataSource[0].id;
                    args.push(newAppointment);
                }
            }

            if (autoVal.length == 0 && categorizeVal.length > 0)
                args.push(newAppointment);
            return args;
        },

        _timeZoneCollection: function (temp) {
            if (temp !== null) {
                this.startTimeAppoint = temp[this.model.appointmentSettings["startTimeZone"]];
                this.endTimeAppoint = temp[this.model.appointmentSettings["endTimeZone"]];
            }
            this.startutcValue = null;
            this.tempValue = null;
            if (temp !== null && this.endTimeAppoint) {
                this._utcValue(temp, "Appoint");
                this.tempValue = 1;
            }
            if (this.model.timeInfo != null) {
                var value = this._findResourceIndex(this.model.timeZoneCollection.dataSource, "id", this.model.timeInfo);
            }
            else {
                var rawTimezone = (this.model.timeZone == null) ? this._timezoneStringValue() : this.model.timeZone;
                var value1 = temp !== null ? ej.isNullOrUndefined(this.startTimeAppoint) ? rawTimezone : this.startTimeAppoint : rawTimezone;
                var firstValue1 = this.model.timeZoneCollection.dataSource[0][this.model.timeZoneCollection["value"]];
                var firstValue2 = (firstValue1.indexOf('UTC') == -1) && (value1.indexOf('UTC') != -1) ? value1.split(" ")[1] : (firstValue1.indexOf('UTC') != -1) && (value1.indexOf('UTC') == -1) ? 'UTC ' + value1 : value1;
                var firstValue3 = !ej.isNullOrUndefined(firstValue1.split(':')[2]) && !ej.isNullOrUndefined(value1.split(':')[2]) || ej.isNullOrUndefined(firstValue1.split(':')[2]) && ej.isNullOrUndefined(value1.split(':')[2]) ? firstValue2 : ej.isNullOrUndefined(firstValue1.split(':')[2]) && !ej.isNullOrUndefined(value1.split(':')[2]) ? firstValue2.split(':')[0] + ':' + firstValue2.split(':')[1] : firstValue2 + ':00';
                var firstValue4 = firstValue2.indexOf("UTC") != -1 ? (firstValue2.split(" ")[1].split(":")[0]).length == 1 ? "0" + firstValue2.split(" ")[1].split(":")[0] : firstValue2.split(" ")[1].split(":")[0] : (firstValue2.split(":")[0]).length == 1 ? "0" + firstValue2.split(":")[0] : firstValue2.split(":")[0];
                var firstValue5 = firstValue4[0] == "+" ? firstValue4.split("+") : firstValue4.split("-");
                var newValue0 = "0" + firstValue5[1];
                var newValue = newValue0.length == 2 ? newValue0 : firstValue5[1];
                var firstValue6 = firstValue3.indexOf("UTC") != -1 ? firstValue3.split(" ")[0] + " +" + newValue + ":" + firstValue3.split(" ")[1].split(":")[1] : "0" +newValue + ":" + firstValue3.split(" ")[1].split(":")[1];
                var firstValue7 = firstValue3.indexOf("UTC") != -1 ? firstValue3.split(" ")[0] + " -" + newValue + ":" + firstValue3.split(" ")[1].split(":")[1] : "0" +newValue + ":" + firstValue3.split(" ")[1].split(":")[1];
                var firstValue = firstValue4[0] == "+" ? firstValue6 : firstValue7;
                var value = this._findResourceIndex(this.model.timeZoneCollection.dataSource, "value", firstValue);
                value = ej.isNullOrUndefined(value) ? this._findResourceIndex(this.model.timeZoneCollection.dataSource, "value", firstValue.replace('+', '')) : value;
                value = (!ej.isNullOrUndefined(value)) ? value : "add";
            }
            if (this.model.timeZone == null && value == 0) {
                this._appointmentAddWindow.find("#" + this._id + "startTimeZone").val("UTC " + Globalize.format(new Date(), "zzz")) && this._appointmentAddWindow.find("#" + this._id + "startTimeZone_hidden").val("UTC " + Globalize.format(new Date(), "zzz"));
                this._appointmentAddWindow.find("#" + this._id + "endTimeZone").val("UTC " + Globalize.format(new Date(), "zzz")) && this._appointmentAddWindow.find("#" + this._id + "endTimeZone_hidden").val("UTC " + Globalize.format(new Date(), "zzz"));
            }
            else if (value !== "add") {
                this._appointmentAddWindow.find("#" + this._id + "startTimeZone").ejDropDownList("option", { text: this.model.timeZoneCollection.dataSource[value][this.model.timeZoneCollection["text"]], value: this.model.timeZoneCollection.dataSource[value][this.model.timeZoneCollection["value"]] });
                this._appointmentAddWindow.find("#" + this._id + "endTimeZone").ejDropDownList("option", { text: this.model.timeZoneCollection.dataSource[value][this.model.timeZoneCollection["text"]], value: this.model.timeZoneCollection.dataSource[value][this.model.timeZoneCollection["value"]] });
            }
            if (temp !== null && !ej.isNullOrUndefined(this.startTimeAppoint)) {
                this._appointmentAddWindow.find("#" + this._id + "startTimeZone").ejDropDownList("option", { value: (temp !== null && this.startTimeAppoint) ? this.startTimeAppoint : this.model.timeZoneCollection.dataSource[value][this.model.timeZoneCollection["value"]] });
                this._appointmentAddWindow.find("#" + this._id + "endTimeZone").ejDropDownList("option", { value: (temp !== null && this.endTimeAppoint) ? this.endTimeAppoint : this.model.timeZoneCollection.dataSource[value][this.model.timeZoneCollection["value"]] });
            }
            if ((this.model.timeZone != null || this.model.timeZone == null || temp != null) && value == "add") {
                var value1 = temp != null ? this.startTimeAppoint : (this.model.timeZone == null) ? this._timezoneStringValue() : this.model.timeZone;
                var firstValue1 = this.model.timeZoneCollection.dataSource[0][this.model.timeZoneCollection["value"]];
                var firstValue = (firstValue1.indexOf('UTC') == -1) && (value1.indexOf('UTC') != -1) ? value1.split(" ")[1] : (firstValue1.indexOf('UTC') != -1) && (value1.indexOf('UTC') == -1) ? 'UTC ' + value1 : value1;
                this.model.timeZoneCollection.dataSource.push({ text: firstValue, id: (this.model.timeZoneCollection.dataSource.length + 1).toString() + "_val", value: firstValue });
                this._appointmentAddWindow.find("#" + this._id + "startTimeZone").ejDropDownList({ dataSource: this.model.timeZoneCollection.dataSource });
                this._appointmentAddWindow.find("#" + this._id + "endTimeZone").ejDropDownList({ dataSource: this.model.timeZoneCollection.dataSource });
                temp != null ? this._timeZoneCollection(temp) : this._timeZoneCollection(null);
            }
        },

        _timezoneStringValue: function () {
            var sign = (-new Date().getTimezoneOffset()) + (this._timeZoneOffset);
            var value = sign < 0 ? "-" : "+";
            var offset = Math.abs(sign);
            var hours = ((Math.floor(offset / 60)) < 10) ? '0' + (Math.floor(offset / 60)) : (Math.floor(offset / 60));
            var minutes = ((offset % 60) < 10) ? '0' + (offset % 60) : (offset % 60);
            return value + hours + ":" + minutes;
        },

        _showCategorizeDetails: function (temp) {
            var categorizeItems = temp[this.model.appointmentSettings["categorize"]].split(",");
            var key;
            for (i = 0; i < categorizeItems.length; i++) {
                key = $.type(categorizeItems[i]) == 'string' ? categorizeItems[i] : parseInt(categorizeItems[i]);
                this._appointmentAddWindow.find('.e-categorizevalue').data("ejAutocomplete").selectValueByKey(key);
            }
        },

        _renderCategorizeTemplate: function () {
            var Text = this.model.enableRTL ? 'margin-right:30px;margin-top:-18px' : 'float:left';
          var Color = this.model.enableRTL ? 'float:none': 'float:left';
            if (!ej.isNullOrUndefined(this.model.categorizeSettings)) {
                this._categorizeTemplate = "<div class='e-categorizeTemplate' style='height:15px;margin-right:5px;margin-top:3px;"+ Color +";width:15px;background-color: ${" + this.model.categorizeSettings["color"] + "}'></div><div style= "+Text+">${" + this.model.categorizeSettings["text"] + "}</div>";
            }
        },

        _getCategorizeColor: function (record) {
            var categorizeDataIndex = null, appointData = "", appointCustomcss = "", appointtextcolor = "";
            var categorizeItems = record[this.model.appointmentSettings["categorize"]].split(",");
            var categorizeItemsLength = categorizeItems.length - 1;
            var dataSet = this.model.categorizeSettings;
            categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, this.model.categorizeSettings["id"], categorizeItems[categorizeItemsLength]);
            (ej.isNullOrUndefined(categorizeDataIndex)) && (categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, this.model.categorizeSettings["id"], parseInt(categorizeItems[categorizeItemsLength])));
            if (ej.isNullOrUndefined(categorizeDataIndex)) {
                record[this.model.appointmentSettings["categorize"]] = "";
                return { appointData: appointData, appointCustomcss: appointCustomcss, appointtextcolor: appointtextcolor };
            }
            else {
                appointData = !ej.isNullOrUndefined(dataSet.dataSource[categorizeDataIndex][dataSet.color]) ? dataSet.dataSource[categorizeDataIndex][dataSet.color] : appointData;
                appointtextcolor = !ej.isNullOrUndefined(dataSet.dataSource[categorizeDataIndex][dataSet.fontColor]) ? dataSet.dataSource[categorizeDataIndex][dataSet.fontColor] : appointtextcolor;
            }
            return { appointData: appointData, appointCustomcss: appointCustomcss, appointtextcolor: appointtextcolor }
        },

        _currentDateFormat: function (option) {
            var minDate, maxDate;
            if (!ej.isNullOrUndefined(option) && ($.type(this.currentDate()) != "date" || $.type(this.model.minDate) != "date" || $.type(this.model.maxDate) != "date")) {
                var date = Globalize.parseDate(this.currentDate(), this.model.dateFormat);
                minDate = Globalize.parseDate(this.model.minDate, this.model.dateFormat);
                maxDate = Globalize.parseDate(this.model.maxDate, this.model.dateFormat);
                this.currentDate((!ej.isNullOrUndefined(date)) ? date : ((new Date(this.currentDate()).toString() == "Invalid Date" || new Date(this.currentDate()).toString() == "NaN") ? this._dateConvert(this.currentDate()) : new Date(this.currentDate())));
                this.model.minDate = (!ej.isNullOrUndefined(minDate)) ? minDate : ((new Date(this.model.minDate).toString() == "Invalid Date" || new Date(this.model.minDate).toString() == "NaN") ? this._dateConvert(this.model.minDate) : new Date(this.model.minDate));
                this.model.maxDate = (!ej.isNullOrUndefined(maxDate)) ? maxDate : ((new Date(this.model.maxDate).toString() == "Invalid Date" || new Date(this.model.maxDate).toString() == "NaN") ? this._dateConvert(this.model.maxDate) : new Date(this.model.maxDate));
            }
            else if (option == "MM/dd/yyyy" && ($.type(this.currentDate()) != "date" || $.type(this.model.minDate) != "date" || $.type(this.model.maxDate) != "date")) {
                var curntDate = Globalize.parseDate(this.currentDate());
                minDate = Globalize.parseDate(this.model.minDate);
                maxDate = Globalize.parseDate(this.model.maxDate);
                (!ej.isNullOrUndefined(curntDate)) ? this.currentDate(curntDate) : this.currentDate(new Date());
                this.model.minDate = (!ej.isNullOrUndefined(minDate)) ? minDate : this.model.minDate;
                this.model.maxDate = (!ej.isNullOrUndefined(maxDate)) ? maxDate : this.model.maxDate;
            }
        },

        _datepattern: function () {
            return (this.model.dateFormat != "" && !ej.isNullOrUndefined(this.model.dateFormat)) ? this.model.dateFormat : this._pattern.d;
        },

        _businessHourScroller: function () {
            if (this.currentView() == "agenda") return false;
            var highScrol;
            if (this.model.orientation == "vertical") {
                highScrol = this.element.find('.e-businesshighlightworkcells').length != 0 && this.model.workHours.highlight == true ? this.element.find('.e-businesshighlightworkcells').first().parent()[0].rowIndex * this.element.find('.e-workcells').height() : 0;
                $('div#' + this._id + "_scroller").ejScroller({ scrollTop: highScrol });
                this.element.find('.e-scrolltimecells').css('top', -(this.element.find('.e-draggableworkarea').scrollTop()) + 'px');
            }
            else if (this.model.orientation == "horizontal" && this.model.showTimeScale) {
                if (this.element.find('.e-businesshighlightworkcells').length != 0 && this.model.workHours.highlight == true && this.currentView() != "month") {
                    if (this.currentView() === "week" || this.currentView() === "workweek" || (this.currentView() === "customview" && this._renderDays <= 7)) {
                        var currentDateIndex = this.element.find('.e-activeview.e-horizontalheaderdiv').parent().index();
                        if (!ej.isNullOrUndefined(currentDateIndex) && currentDateIndex >= 0) {
                            this.currentView() == "customview" ? this.currentDate(new Date(this._dateRender[0])) : this.currentDate(new Date(this._getRenderDates()[currentDateIndex]));
                            dateRender = new Date(this._getRenderDates()[currentDateIndex]).getDay();
                            highScrol = this.element.find(".e-workcells")[currentDateIndex * ((this.model.endHour - this.model.startHour) * 2)].offsetLeft;
                        }
                        if (this.currentView() === "week" && [0, 6].indexOf(dateRender) == -1 || (this.currentView() === "workweek"))
                            if (this.model.enableRTL)
                                highScrol = (highScrol + this.element.find('.e-workcells').width()) - this.model.workHours.start * 2 * this.element.find(".e-workcells").width();
                            else
                                highScrol = this.model.workHours.start * 2 * this.element.find(".e-workcells").width() + (highScrol - this.element.find('.e-workcells').width());
                    }
                    else {
                        highScrol = this.element.find('.e-businesshighlightworkcells')[0].offsetLeft;
                    }

                }
                else {
                    highScrol = (this.element.find('.e-activeview.e-horizontalheaderdiv').parent().index() != -1) ? this.element.find(".e-workcells")[(this.element.find('.e-activeview.e-horizontalheaderdiv').parent().index()) * ((this.model.endHour - this.model.startHour) * 2)].offsetLeft : 0;
                }
                if (this.model.enableRTL) {
                    highScrol = this.element.find(".e-workcells")[0].offsetLeft - highScrol;
                    $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: highScrol });
                    var maxLeft = this.element.find(".e-workcells")[0].offsetLeft + this.element.find(".e-workcells").width() - this.element.find('.e-draggableworkarea').width();
                    highScrol = highScrol >= maxLeft ? maxLeft : highScrol;
                    $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", highScrol + "px");
                }
                else {
                    $('div#' + this._id + "_scroller").ejScroller({ scrollLeft: highScrol });
                    $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", (-this.element.find('.e-draggableworkarea').scrollLeft()) + "px");
                }
            }
        },

        _yearDateChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "yearday").ejRadioButton("option", "checked", true);
        },

        _yearDayChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "yearother").ejRadioButton("option", "checked", true);
        },

        _monthDateChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "monthday").ejRadioButton("option", "checked", true);
        },

        _monthDayChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "monthon").ejRadioButton("option", "checked", true);
        },

        _endAfterChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "repeatendafter").ejRadioButton("option", "checked", true);
        },

        _endOnChange: function (e) {
            this._appointmentAddWindow.find("#" + this._id + "repeatendon").ejRadioButton("option", "checked", true);
        },

        _minToHour: function (mins) {
            var hours = parseInt(mins / 60);
            var mins = mins % 60;
            var obj = {
                "h": hours,
                "m": mins
            };
            return obj;
        },

        _onclose: function (args) {
            this._appointmentAddWindow.find(".e-recurwindow").hide();
            this._appointmentAddWindow.find(".e-appwindow").css("margin-left", 700).show().animate({ "margin-left": 10 }, 300);
            this._clearFields();
            this._processFocus();
        },

        _printAppointment: function () {
            var $element = ej.buildTag("div");
            var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
            var templist = new ej.DataManager(this._processed).executeLocal(query);
            var appointmentTable = "<table align='center' style='margin-top:50px;margin-left:50;'>";
            var proxy = this;
            $.each(templist[0], function (i, val) {
                if (i != "ParentId" && !ej.isNullOrUndefined(val) && !ej.isNullOrUndefined(i) && i != "null") {
                    if (i == proxy.model.appointmentSettings.startTime || i == proxy.model.appointmentSettings.endTime)
                        val = val;
                    appointmentTable += "<tr style='height:30px'><td style='width:300px'><b>" + i + ":</b></td><td  style='width:600px'>" + val + "</td></tr>";
                }
            });
            appointmentTable += "</table>";
            var ownerTextName = "";
            if (!ej.isNullOrUndefined(this.model.appointmentSettings.resourceFields) && this.model.appointmentSettings.resourceFields != "") {
                var s = proxy.model.appointmentSettings.resourceFields.split(',');
                $.each(this.model.resources[this.model.resources.length - 1].resourceSettings.dataSource, function (i, val) {
                    if (val.id == templist[0][s[s.length - 1]])
                        ownerTextName = val.text;
                });
            }
            ownerTextName = (ownerTextName == "") ? "Unknown" : ownerTextName;
            var appointmentWindowData = "<font size='20' style='margin-top:50px;margin-left:50;font-family:Times New Roman'><b>" + ownerTextName + "</b></font></br><hr style='margin-left:50' size='10'>"
            var printElement = appointmentWindowData + appointmentTable;
            $element.append(printElement);
            ej.print($element);
            return false;
        },

        _recurrenceValidate: function (rule, recurId, sTime, eTime) {
            (!ej.isNullOrUndefined(rule)) && this.recurrenceRuleSplit(rule, recurId);
            var appEnd, appStart;
            if (!ej.isNullOrUndefined(eTime))
                appEnd = eTime;
            else {
                var end = this._timeFormat(this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value")).split(':');
                appEnd = new Date(this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value").setHours(parseInt(end[0], 10), parseInt(end[1], 10)));
            }
            if (!ej.isNullOrUndefined(sTime))
                appStart = sTime;
            else {
                var start = this._timeFormat(this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value")).split(':');
                appStart = new Date(this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value").setHours(parseInt(start[0], 10), parseInt(start[1], 10)));
            }
            if (((appEnd - appStart) / (1000 * 3600) / 24 < 14)) {
                var day_start = !ej.isNullOrUndefined(sTime) ? sTime : this._appointmentAddWindow.find("#recurstartdate").ejDatePicker("option", "value");
                var _interval = this._rRule.interval;
                if (!ej.isNullOrUndefined(this._rRule.until)) {
                    recurEndDate = new Date(this._rRule.until);
                    recurEndDate.setDate(recurEndDate.getDate() + 1);
                } else if (!ej.isNullOrUndefined(this._rRule.count))
                    switch (this._rRule.freq) {
                        case "DAILY":
                            recurEndDate = new Date(new Date(day_start).setDate(day_start.getDate() + this._rRule.count * (_interval)));
                            break;
                        case "WEEKLY":
                            var byDay = this._rRule.weekDays.split(",");
                            var dayCount = day_start.getDate() + (((this._rRule.count / byDay.length) * 7)) * (_interval);
                            recurEndDate = new Date(new Date(day_start).setDate(Math.round(dayCount) < dayCount ? dayCount + 1 : dayCount));
                            break;
                        case "MONTHLY":
                            recurEndDate = new Date(new Date(day_start).setMonth(day_start.getMonth() + this._rRule.count * (_interval)));
                            break;
                        case "YEARLY":
                            recurEndDate = new Date(new Date(day_start).setFullYear(day_start.getFullYear() + this._rRule.count * (_interval)));
                            break;
                    }
                else {
                    recurEndDate = new Date(new Date(this.currentDate()).setDate(day_start.getDate() + 42 * (_interval)));
                }
                if (((recurEndDate - day_start) / (1000 * 3600) / 24) < ((this._rRule.freq == "DAILY") ? 2 : (this._rRule.freq == "WEEKLY") ? 7 : 1)) {
                    return true;
                } else {
                    switch (this._rRule.freq) {
                        case "DAILY":
                            var count = this._rRule.interval;
                            if ((appEnd - appStart) / (1000 * 3600) > parseInt(count) * 24)
                                return false;
                            else
                                return true;
                            break;
                        case "WEEKLY":
                            if (rule.split(";")[1].split("=")[0] == "INTERVAL") {
                                var types = this._rRule.weekDays.split(",");
                                var obj = { "SU": 0, "MO": 1, "TU": 2, "WE": 3, "TH": 4, "FR": 5, "SA": 6 };
                                var temp = [], tempDiff = [];
                                for (var i = 0; i < types.length * (parseInt(this._rRule.interval) + 1) ; i++) {
                                    temp[i] = (types.length > i) ? obj[types[i]] : temp[i - types.length] + (7 * parseInt(this._rRule.interval));
                                }
                                for (var i = 1; i < temp.length; i++) {
                                    tempDiff.push(temp[i] - temp[i - 1]);
                                }
                                if ((appEnd - appStart) / (1000 * 3600) > Math.min.apply(Math, tempDiff) * 24) {
                                    return false;
                                } else
                                    return true;
                            } else {
                                if ((appEnd - appStart) / (1000 * 3600) > 24)
                                    return false;
                                else
                                    return true;
                            }
                            break;
                        case "MONTHLY":
                            return true;
                            break;
                        case "YEARLY":
                            return true;
                            break;
                    }
                }
            } else
                return false;
        },

        _saveAppointment: function () {
            if (this._currentAction != "") {
                start = this._timeFormat(this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value")).split(":");
                end = this._timeFormat(this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value")).split(":");
                startTime = new Date(this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value").setHours(parseInt(start[0], 10), parseInt(start[1], 10)));
                endTime = new Date(this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value").setHours(parseInt(end[0], 10), parseInt(end[1], 10)));
                if (startTime > endTime) {
                    this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("StartEndError"));
                    this._alertWindow.ejDialog("open");
                    return false;
                }
            }
            if (this._appointmentAddWindow.find("#" + this._id + "recurrence").ejCheckBox("option", "checked") && this._recRule != "" && !ej.isNullOrUndefined(this._recRule) && this.model.enableRecurrenceValidation && this._currentAction == "add") {
                if (!this._recurrenceValidate(this._recRule)) {
                    $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                    this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("CreateError"));
                    this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                    this._alertWindow.ejDialog("open");
                    return false;
                }
            }
            var appointments = this._sortAppById(this._processed), maxId, query, query1, appointId, appointGroupId, _editApp;
            if (this._cellClick && this.model.showQuickWindow) {
                var argsValue, obj = {};
                maxId = !ej.isNullOrUndefined(this._appMainId) ? this._appMainId + 1 : 1;
                if (this.model.enableLoadOnDemand) maxId = this._dataCount;
                obj[this.model.appointmentSettings["id"]] = maxId;
                obj["AppTaskId"] = maxId;
                obj["ParentId"] = maxId;
                obj[this.model.appointmentSettings["subject"]] = this._quickAppointWindow.find('.subject').val();
                obj[this.model.appointmentSettings["startTime"]] = new Date(this.cur_StartTime);
                obj[this.model.appointmentSettings["endTime"]] = new Date(this.cur_EndTime);
                if (this.currentView() === "month" || (new Date(this.cur_StartTime).getHours() == 0 && new Date(this.cur_EndTime).getHours() == 23 && new Date(this.cur_EndTime).getMinutes() == 59) || (this._isCustomView()))
                    obj[this.model.appointmentSettings["allDay"]] = true;
                else
                    obj[this.model.appointmentSettings["allDay"]] = false;
				if (!obj[this.model.appointmentSettings["allDay"]]) {
					obj[this.model.appointmentSettings["startTimeZone"]] = this.model.timeZone == null ? ("UTC " + this._timezoneStringValue()) : this.model.timeZone;
					obj[this.model.appointmentSettings["endTimeZone"]] = this.model.timeZone == null ? ("UTC " + this._timezoneStringValue()) : this.model.timeZone;
				}
                obj[this.model.appointmentSettings["recurrence"]] = false;
                if (!ej.isNullOrUndefined(this.model.group) || !ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                    obj = this._getResourceFields(obj);
                ("null" in obj) && delete obj[null];
                var args = { appointment: obj };
                if (this._trigger("actionBegin", { data: args.appointment, requestType: "appointmentSave" })) {
                    this._quickAppointWindow.ejDialog("close");
                    return false;
                }
                if (this._trigger("appointmentSaved", args)) {
                    this._quickAppointWindow.ejDialog("close");
                    return false;
                }
                this._currentAction == ej.Schedule.Actions.Add;
                args.appointment.Guid = this._guidFormatGenerate();
                $.extend(true, argsValue = {}, args.appointment);
                var temp = this._timeZoneAppointments(argsValue, "");
                promise = this._dataManager.insert(temp);
                new ej.DataManager(this._currentAppointmentData).update("AppTaskId", args.appointment);
                var proxy = this;
                if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                    promise.done(function (e) {
                        if (proxy.model.enableLoadOnDemand)
                            proxy._initDataSource("refresh");
                        else {
                            var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                            proxy._currentAppointmentData.length = 0;
                            proxy._dateFormat(value);
                            proxy._bindAppointments = true;
                            proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                            proxy._dataProcessing(value);
                            proxy._renderAppointmentAll();
                        }
                        proxy._trigger("actionComplete", { data: args.appointment, requestType: "appointmentSave" });
                    });
                    promise.fail(function (e) {
                    });
                }
                else {
                    this._processed.push(args.appointment);
                    this._appMainId = obj["AppTaskId"] + 1;
                    this._renderSingleApp(obj, args.appointment);
                    this._appMainId = this._sortAppById(this._processed)[this._processed.length - 1]["AppTaskId"];
                    this._trigger("actionComplete", { data: args.appointment, requestType: "appointmentSave" });
                }
                this._cellClick = false;
                this._quickAppointWindow.ejDialog("close");
            }
            else {
                var formelement;
                obj = {};
                temp = {};
                var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                maxId = (this._currentAction == ej.Schedule.Actions.Add || this._currentAction == ej.Schedule.Actions.EditOcurrence) ? (this.model.enableLoadOnDemand ? this._dataCount + 1 : (!ej.isNullOrUndefined(this._appMainId) ? this._appMainId + 1 : parseInt(this._appId))) : tempApp[0]["AppTaskId"];         
                formelement = $("#" + this._id + "AppointmentAddEditWindow").find("#" + this._id + "_AddEditForm").get(0); var autoVal = []; var categorizeVal = [];
                for (var index = 0; index < formelement.length; index++) {
                    var columnName = formelement[index].name, $element = $(formelement[index]);
                    if (columnName != undefined) {
                        if (this.model.appointmentSettings["startTimeZone"] == null && this.model.appointmentSettings["endTimeZone"] == null && columnName == "")
                            ((columnName = formelement[index].id.replace(this._id, "")));
                        var value = formelement[index].value;
                        if ((columnName != "" && obj[columnName] == null)) {
                            if (columnName == this.model.appointmentSettings["id"]) {
                                if (value == "")
                                    value = parseInt(maxId);
                                else
                                    value = parseInt(value);
                            }
                            if ($(formelement[index]).hasClass("e-datepicker"))
                                value = $element.ejDatePicker("option", "value");
                            else if ($(formelement[index]).hasClass("e-datepicker"))
                                value = $element.ejTimePicker("option", "value");
                            else if ($element.data("ejDropDownList"))
                                value = $element.ejDropDownList("model.value");
                            else if ($element.data("ejAutocomplete")) {
                                if ($(formelement[index])[0].id == this._id + "_categorize")
                                    categorizeVal.push($element.data("ejAutocomplete").getSelectedItems());
                                else
                                    autoVal.push($element.data("ejAutocomplete").getSelectedItems());
                            }
                            if (columnName != "StartDate" && columnName != "EndDate" && columnName != "StartTime" && columnName != "EndTime") {
                                if (formelement[index].type != "checkbox")
                                    obj[columnName] = value;
                                else
                                    obj[columnName] = $(formelement[index]).is(':checked');
                            }
                            else {
                                temp[columnName] = columnName == "StartDate" || columnName == "EndDate" ? new Date(value) : value;
                            }
                        }
                        else {
                            if ($element.data("ejAutocomplete")) {
                                if ($(formelement[index])[0].id == this._id + "_categorize")
                                    categorizeVal.push($element.data("ejAutocomplete").getSelectedItems());
                                else
                                    autoVal.push($element.data("ejAutocomplete").getSelectedItems());
                            }
                        }
                    }
                }
                if (this.model.categorizeSettings.enable == true && this.model.categorizeSettings.allowMultiple != true) {
                    var dataSet = this.model.categorizeSettings;
                    categorizeDataIndex = this._findResourceIndex(dataSet.dataSource, this.model.categorizeSettings["text"], obj[this._id + "_categorize"]);
                    categorizeVal[0].push(this.model.categorizeSettings.dataSource[categorizeDataIndex]);
                    categorizeVal[0].shift();
                }
                ("null" in obj) && delete obj[null];
                var starttime = this._timeFormat(temp.StartTime).split(":");
                var endtime = this._timeFormat(temp.EndTime).split(":");
                obj[this.model.appointmentSettings["startTime"]] = new Date(temp.StartDate.setHours(parseInt(starttime[0], 10), parseInt(starttime[1], 10)));
                obj[this.model.appointmentSettings["endTime"]] = new Date(temp.EndDate.setHours(parseInt(endtime[0], 10), parseInt(endtime[1], 10)));
                (obj[this.model.appointmentSettings["recurrence"]]) && (obj[this.model.appointmentSettings["recurrenceRule"]] = this._recRule);
                this._publicMethodSave = true;
                var value = this._timeZoneSavingAppoint(obj, temp);
                obj = value.obj;
                var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                var _editApps = new ej.DataManager(this._processed).executeLocal(query);
                if (obj[this.model.appointmentSettings["recurrenceRule"]] != "" && !ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && this.model.enableRecurrenceValidation && (this._currentAction == "editSeries" || this._currentAction == "editOcurrence" || this._currentAction == "save")) {
                    if (this._currentAction == "editOcurrence" || this._currentAction == "save") {
                        var _appList = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid))[0];
                        var Parid = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? _appList["ParentId"] : _appList[this.model.appointmentSettings["recurrenceId"]];
                        var MapValue = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                        var recurColl = this._appointmentRecurrenceSort(new ej.DataManager(this._processed).executeLocal(new ej.Query().where(MapValue, ej.FilterOperators.equal, Parid)));
                        var index = this._recuCollectionIndex(recurColl, _appList);
                        var model = this.model.appointmentSettings;
                        if (((index == 0) ? (!ej.isNullOrUndefined(recurColl[index + 1]) ? !(new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(obj[model["endTime"]]).setHours(0, 0, 0, 0)).getTime()) : false) : (index == recurColl.length - 1) ? !(new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[model["startTime"]]).setHours(0, 0, 0, 0)).getTime()) : !((new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[model["startTime"]]).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(obj[model["endTime"]]).setHours(0, 0, 0, 0)).getTime())))) {
                            $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                            this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("CreateError"));
                            this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                            this._alertWindow.ejDialog("open");
                            return false;
                        }
                    }
                    else {
                        if (!this._recurrenceValidate(obj[this.model.appointmentSettings["recurrenceRule"]], _editApps[0][this.model.appointmentSettings["recurrenceExDate"]])) {
                            $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                            this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("CreateError"));
                            this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                            this._alertWindow.ejDialog("open");
                            return false;
                        }
                    }
                }
                if (this._currentAction == ej.Schedule.Actions.EditOcurrence) {
                    this._objDate = _editApps;
                    var MapValue = this._objDate[0][this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                    var _editApp = this._deleteOcurrence(this._objDate[0][MapValue], new Date(_editApps[0][this.model.appointmentSettings["startTime"]]));

                        new ej.DataManager(this._processed).remove("Guid", this._appUid);
                        obj[this.model.appointmentSettings["recurrence"]] = 1;
                        obj[this.model.appointmentSettings["recurrenceRule"]] = _editApp[0][this.model.appointmentSettings["recurrenceRule"]] + ";RECUREDITID=" + _editApp[0]["AppTaskId"];
                        obj[this.model.appointmentSettings["recurrenceId"]] = _editApp[0][this.model.appointmentSettings["recurrenceId"]];
                        obj["Guid"] = this._appUid;
                        obj[this.model.appointmentSettings["recurrenceExDate"]] = _editApp[0][this.model.appointmentSettings["recurrenceExDate"]];
                }
                else if (this._currentAction == ej.Schedule.Actions.EditSeries) {
                    var Parid = _editApps[0][this.model.appointmentSettings["recurrenceId"]] == null ? _editApps[0]["ParentId"] : _editApps[0][this.model.appointmentSettings["recurrenceId"]];
                    var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, Parid);
                    var _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                    obj["AppTaskId"] = _editApp[0]["AppTaskId"];
                    obj[this.model.appointmentSettings["id"]] = _editApp[0][this.model.appointmentSettings["id"]];
                    maxId = _editApp[0]["AppTaskId"];
                    obj[this.model.appointmentSettings["recurrenceId"]] = _editApp[0][this.model.appointmentSettings["recurrenceId"]];
                    obj["Guid"] = this._appUid;
                    obj[this.model.appointmentSettings["recurrenceExDate"]] = _editApp[0][this.model.appointmentSettings["recurrenceExDate"]];
                    //strDate = _editApp[0][this.model.appointmentSettings["startTime"]];
                    //this._appointmentAddWindow.find('.e-until').ejDatePicker({ value: new Date(strDate.setDate(strDate.getDate() + 7 * 10)) });
                    obj[this.model.appointmentSettings["recurrenceRule"]] = (ej.isNullOrUndefined(this._recRule) || this._recRule == "") ? _editApp[0][this.model.appointmentSettings["recurrenceRule"]] : this._recRule;

                }
                if (this._currentAction != ej.Schedule.Actions.Add) {
                    obj["Guid"] = _editApps[0]["Guid"];
                    obj["AppTaskId"] = _editApps[0]["AppTaskId"];
                    obj[this.model.appointmentSettings["id"]] = (this._currentAction != ej.Schedule.Actions.EditSeries) ? _editApps[0][this.model.appointmentSettings["id"]] : obj[this.model.appointmentSettings["id"]];
                    if (this._trigger("actionBegin", { data: obj, requestType: "appointmentEdit" })) {
                        this._appointmentAddWindow.ejDialog("close");
                        return false;
                    }
                    if (this._trigger("appointmentEdited", { appointment: obj })) {
                        this._appointmentAddWindow.ejDialog("close");
                        return false;
                    }
                }
                else {
                    obj.Guid = ej.isNullOrUndefined(obj.Guid) ? this._guidFormatGenerate() : obj.Guid;
                    if (this._trigger("actionBegin", { data: obj, requestType: "appointmentSave" })) {
                        this._appointmentAddWindow.ejDialog("close");
                        return false;
                    }
                    if (this._trigger("appointmentSaved", { appointment: obj })) {
                        this._appointmentAddWindow.ejDialog("close");
                        return false;
                    }
                }
                var args = [], newAppointment;
                (this._startTimezone != this._endTimezone) ? obj[this.model.appointmentSettings["endTime"]] = value.cloneEnddiffer : "";
                if (!ej.isNullOrUndefined(this.model.categorizeSettings) && categorizeVal.length > 0 || !ej.isNullOrUndefined(this.model.resources) && ej.scheduleFeatures.resources)
                    args = this._getResCategorySaveData(categorizeVal, autoVal, obj, maxId, temp, starttime, endtime);
                else {
                    var parId = this._currentAction == ej.Schedule.Actions.EditOcurrence || (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && obj[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) ? this._parentId : this._currentAction == (ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : maxId;
                    this._maxId = ((((new Date(obj[this.model.appointmentSettings["endTime"]]) - new Date(obj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24) && this._currentAction != ej.Schedule.Actions.EditOcurrence && (obj[this.model.appointmentSettings["allDay"]] == true)) ? parId : this._currentAction == (ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : this._appMainId + 1;
                    this._maxId = (((new Date(obj[this.model.appointmentSettings["endTime"]]) - new Date(obj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24) && (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && obj[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) && (this._currentAction == ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : this._maxId;

                    this._maxId = (this._currentAction == ej.Schedule.Actions.EditSeries) ? parId : this._maxId;
                    newAppointment = this._addAppData(obj, this._maxId, new Date(obj[this.model.appointmentSettings["startTime"]]), new Date(obj[this.model.appointmentSettings["endTime"]]), parId);
                    args.push(newAppointment);

                }
                if (args[this.model.appointmentSettings["startTime"]] > args[this.model.appointmentSettings["endTime"]]) {
                    return false;
                }
                if (this._currentAction == ej.Schedule.Actions.Add || this._currentAction == ej.Schedule.Actions.EditOcurrence) {
                    this._maxId = maxId;
                    var proxy = this, promise, temp = [];
                    for (var a = 0; a < args.length; a++) {
                        var startTimezone1 = args[a][this.model.appointmentSettings["startTimeZone"]];
                        (startTimezone1 != this.model.timeZone) ? temp.push(this._timeZoneAppointments(args[a], "")) : temp.push(args[a]);
                    }
                    for (var b = 0; b < args.length; b++) {
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", args[b]);
                    }
                    var _editApp = [];
                    if (this._currentAction == ej.Schedule.Actions.EditOcurrence) {
                        var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, this._parentId);
                        var _App = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", _App[0]);
                        _editApp.push(this._timeZoneAppointments(_App[0], ""));
                    }
                    else
                        _editApp = [];
                    if (this.dataSource() instanceof ej.DataManager) {
                        promise = this._dataManager.saveChanges({ added: temp, changed: _editApp, deleted: [] }, this.model.appointmentSettings["id"]);
                    }
                    else
                        promise = this._dataManager.saveChanges({ added: temp, changed: _editApp, deleted: [] }, "AppTaskId");
                    var updateTempNew = [];
                    for (var c = 0; c < temp.length; c++) {
                        updateTempNew = (startTimezone1 != this.model.timeZone) ? this._timeZoneAppointments(temp[c], "reverse") : temp[c];
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew);
                    }
                    var updateTempNew2 = [];
                    for (var c = 0; c < _editApp.length; c++) {
                        if (startTimezone1 != this.model.timeZone)
                            updateTempNew2 = this._timeZoneAppointments(_editApp[c], "reverse");
                        else
                            updateTempNew2 = _editApp[c];
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew2);
                    }
                    if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            if (proxy.model.enableLoadOnDemand)
                                proxy._initDataSource("refresh");
                            else {
                                var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                proxy._currentAppointmentData.length = 0;
                                proxy._dateFormat(value);
                                proxy._bindAppointments = true;
                                proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                proxy._dataProcessing(value);
                                proxy._renderAppointmentAll();
                            }
                            proxy._trigger("actionComplete", { data: args, requestType: "appointmentSave" });
                        });
                        promise.fail(function (e) {
                        });
                    }
                    else {
                        for (var i = 0; i < args.length; i++) {
                            (this.model.showTimeScale) && this._appointmentProcessing(args[i]);
                            if (this._currentAction == ej.Schedule.Actions.EditOcurrence) {
                                this.element.find('[guid=' + this._appUid + ']').remove();
                                this.element.find("#categorybar_" + this._appUid).remove();
                            }
                            if (!this.model.showTimeScale) {
                                var diffDays = Math.ceil(((obj["EndTime"] - obj["StartTime"]) / 3600000) / 24);
                                var multiDayApp = (diffDays > 1) ? true : false;
                                if (multiDayApp || this._mediaQuery) {
                                    this._appointmentProcessing(args[i]);
                                    this._renderAppointmentAll();
                                }
                                else if (obj["Recurrence"] == true || this._mediaQuery) {
                                    this._appointmentProcessing(args[i]);
                                    this._renderSingleApp(obj, args[i]);
                                }
                                else {
                                    this._processed.push(args[i]);
                                    this._renderSingleApp(obj, args[i]);
                                }
                                if (this._mediaQuery) {
                                    this._unWireResizeEvents();
                                    this._wireResizeEvents();
                                    this._dragdropAppointments();
                                }
                            }
                            else !(this.dataSource() instanceof ej.DataManager) && this._renderSingleApp(obj, args[i]);
                        }
                        this._trigger("actionComplete", { data: args, requestType: "appointmentSave" });
                    }
                }
                else if (this._currentAction == ej.Schedule.Actions.Save || this._currentAction == ej.Schedule.Actions.EditSeries) {
                    var newApp = [], editedApp = [], deleted = [];
                    for (var i = 0; i < args.length; i++) {
                        var GId = this._currentAction == ej.Schedule.Actions.Save ? args[i]["Guid"] : args[i][this.model.appointmentSettings["recurrenceId"]];
                        var idName = this._currentAction == ej.Schedule.Actions.Save ? "Guid" :  this.model.appointmentSettings["recurrenceId"];
                        query = new ej.Query().where(idName, ej.FilterOperators.equal, GId);
                        _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                        (!ej.isNullOrUndefined(_editApp) && _editApp.length > 0) ? editedApp.push(args[i]) : newApp.push(args[i]);
                    }
                    if (this._currentAction == ej.Schedule.Actions.EditSeries) {
                        var query, dele;
                        for (var i = 0; i < editedApp.length; i++) {
                            query = ej.Query().where(ej.Predicate(this.model.appointmentSettings["recurrenceRule"], ej.FilterOperators.contains, "RECUREDITID=" + args[i].ParentId, true));
                            dele = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                            deleted = deleted.concat(dele);
                            this._appointmentAddWindow.find('.recurbutton').click();
                            editedApp[i][this.model.appointmentSettings["recurrenceRule"]] = this._recRule;
                            editedApp[i][this.model.appointmentSettings["recurrenceExDate"]] = null;
                        }
                    }
                    var proxy = this, promise, newTemp = [], editTemp = [];
                    for (var a = 0; a < newApp.length; a++) {
                        newTemp.push(this._timeZoneAppointments(newApp[a], ""));
                    }
                    for (var b = 0; b < editedApp.length; b++) {
                        editTemp.push(this._timeZoneAppointments(editedApp[b], ""));
                    }
                    if (this.dataSource() instanceof ej.DataManager) {
                        var promise = this._dataManager.saveChanges({ added: newTemp, changed: editTemp, deleted: deleted }, this.model.appointmentSettings["id"]);
                    }
                    else
                        var promise = this._dataManager.saveChanges({ added: newTemp, changed: editTemp, deleted: deleted },"AppTaskId");
                    var updateTempNew;
                    for (var c = 0; c < newTemp.length; c++) {
                        updateTempNew = this._timeZoneAppointments(newTemp[c], "reverse");
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew);
                    }
                    var updateTemp;
                    for (var d = 0; d < editTemp.length; d++) {
                        updateTemp = this._timeZoneAppointments(editTemp[d], "reverse");
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTemp);
                    }
                    if (editedApp.length > 0) {
                        var proxy = this;
                        if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                if (proxy.model.enableLoadOnDemand)
                                    proxy._initDataSource("refresh");
                                else {
                                    var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                    proxy._currentAppointmentData.length = 0;
                                    proxy._dateFormat(value);
                                    proxy._bindAppointments = true;
                                    proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                    proxy._dataProcessing(value);
                                    proxy._renderAppointmentAll();
                                }
                                proxy._appMainId = proxy._processed.length > 0 ? proxy._sortAppById(proxy._processed)[proxy._processed.length - 1]["AppTaskId"] : proxy._appMainId;
                                proxy._trigger("actionComplete", { data: editedApp, requestType: "appointmentEdit" });
                            });
                            promise.fail(function (e) {
                            });
                        }
                        else {
                            for (var i = 0; i < editedApp.length; i++) {
                                if (!(editedApp[i][this.model.appointmentSettings["recurrence"]] || (new Date(new Date(editedApp[i][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(editedApp[i][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((new Date(editedApp[i][this.model.appointmentSettings["endTime"]]) - new Date(editedApp[i][this.model.appointmentSettings["startTime"]])) / 3600000) < 24) && (editedApp[i][this.model.appointmentSettings["allDay"]] !== true)))
                                new ej.DataManager(this._currentAppointmentData).update("AppTaskId", editedApp[i]);
                                this._dataProcessing(this._currentAppointmentData);
                                this._renderAppointmentAll();
                                this._appMainId = this._processed.length > 0 ? this._sortAppById(this._processed)[this._processed.length - 1]["AppTaskId"] : this._appMainId;
                            }
                            this._trigger("actionComplete", { data: editedApp, requestType: "appointmentEdit" });
                        }
                    }
                }
            }
            this._clearFields();
            this._appointmentAddWindow.ejDialog("close");
            if (this.element.find('div.e-draggableworkarea tr').find('td').hasClass("e-selectedCell"))
                this._processFocus();
            this._recRule = "";

        },

        _timeZoneSavingAppoint: function (obj, temp) {
            this._startTimezone = obj[this.model.appointmentSettings["startTimeZone"]];
            this._endTimezone = obj[this.model.appointmentSettings["endTimeZone"]];
            if (!ej.isNullOrUndefined(this._startTimezone) && this._startTimezone != "") {
                var utc = this._startTimezone.indexOf("UTC") != -1 ? this._startTimezone.split(" ")[1].split(":") : this._startTimezone.split(":");
                var hour = parseInt(utc[0]);
                var minutes = parseInt(utc[1]);
                var calcualtevalue = (hour * 60) + minutes;
                if (this.model.timeZone == null) {
                    var localtime = new Date().getTimezoneOffset() - this._timeZoneOffset;
                    var local = this._minToHour(localtime + ((hour * 60) + minutes));
                }
                else {
                    var timezone = this.model.timeZone.indexOf("UTC") != -1 ? this.model.timeZone.split(" ")[1].split(":") : this.model.timeZone.split(":");
                    var local = this._minToHour((hour * 60) + minutes);
                    local.h = local.h - (parseInt(timezone[0]));
                    local.m = local.m - (parseInt(timezone[1]));
                }
                new Date(obj[this.model.appointmentSettings["startTime"]].setHours(obj[this.model.appointmentSettings["startTime"]].getHours() - local.h, obj[this.model.appointmentSettings["startTime"]].getMinutes() - local.m));
                if (this._startTimezone != this._endTimezone && this._publicMethodSave) {
                    this.startutcValue = null;
                    this.tempValue = null;
                    var utc1 = this._endTimezone.indexOf("UTC") != -1 ? this._endTimezone.split(" ")[1].split(":") : this._endTimezone.split(":");
                    var hour1 = parseInt(utc1[0]);
                    var minutes1 = parseInt(utc1[1]);
                    var calculateend = (hour1 * 60) + minutes1;
                    calcualtevalue = calcualtevalue - calculateend;
                    var tempStart = new Date(temp.StartDate);
                    var CloneStart = new Date(obj[this.model.appointmentSettings["startTime"]]);
                    var cloneEnd = new Date(temp.EndDate);
                    duration = new Date(cloneEnd.setMinutes(cloneEnd.getMinutes() + calcualtevalue));
                    duration = new Date(duration.setMinutes(duration.getMinutes() - ((local.h * 60) + local.m)));
                    var cloneEnddiffer = duration;
                }
                else
                    new Date(obj[this.model.appointmentSettings["endTime"]].setHours(obj[this.model.appointmentSettings["endTime"]].getHours() - (local.h), obj[this.model.appointmentSettings["endTime"]].getMinutes() - local.m));
            }
            return { obj: obj, cloneEnddiffer: cloneEnddiffer };
        },

        _renderAllDayAppointments: function () {
            var appHeight = 16;
            var appWidth = this.cellwidth - 20;
            var level = 1; this.level = 0;
            var renderDate = this._dateRender;
            allDayCount = [];
            this.element.find(".e-alldayappointment").remove();
            var appoints = this._appointmentSort(this._processed);
            if (!this.model.group) this.res1 = ["0"];
            for (var res = 0; res < this.res1.length; res++) {
                for (var day = 0; day < renderDate.length; day++) {
                    for (var i = 0; i < appoints.length; i++) {
                        if (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) {
                            this._getMultipleResourceAlldayApp(renderDate, day, res, i);
                            !ej.isNullOrUndefined(this.allDayCount[0]) ? allDayCount.push(this.allDayCount[0]) : "";
                        }
                        else {
                            var parentAppoint = new ej.DataManager(this._currentAppointmentData).executeLocal(new ej.Query().where(ej.Predicate("Guid", ej.FilterOperators.equal, appoints[i]["Guid"])));
                            var isMultipleDay = (parentAppoint.length > 0) ? (((parentAppoint[0][this.model.appointmentSettings["endTime"]] - parentAppoint[0][this.model.appointmentSettings["startTime"]]) / 3600000) >= 24) : ((appoints[i][this.model.appointmentSettings["endTime"]] - appoints[i][this.model.appointmentSettings["startTime"]]) / 3600000) >= 24;
                            if ((Globalize.format(new Date(renderDate[day]), this._pattern.d) == Globalize.format(new Date(appoints[i][this.model.appointmentSettings["startTime"]]), this._pattern.d)) && (appoints[i][this.model.appointmentSettings["allDay"]] === true || isMultipleDay)) {
                                this._setAllDayPos(appoints[i], day, res);
                                this._filtered.push(appoints[i]);
                                allDayCount.push(i);
                            }
                        }
                        this._app = this._filtered;
                    }
                }
            }
             this.allDayLevel = allDayCount.length;
            if (allDayCount.length == 0 && this.level == 0) {
                if (!this._mediaQuery)
                    this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").css('height', 27 + "px");
                else
                    this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").attr('style', "height:40px !important");
                this.element.find(".e-alldaycells").first().css(this.model.enableRTL ? 'border-right' : 'border-left', "none");
            }
            this._reRenderScroller();
        },

        _renderSingleApp: function (obj, args) {
            var isBigApp = ((new Date(new Date(obj[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((obj[this.model.appointmentSettings["endTime"]] - obj[this.model.appointmentSettings["startTime"]]) / 3600000) < 24) && (obj[this.model.appointmentSettings["allDay"]] !== true) && obj[this.model.appointmentSettings["recurrence"]] != 1);
            this.element.find(".e-prevapp,.e-nextapp").css("display", "none");
            if (((obj[this.model.appointmentSettings["endTime"]] - obj[this.model.appointmentSettings["startTime"]]) / 3600000) > 24 || isBigApp || obj[this.model.appointmentSettings["recurrence"]] == 1) {
                this._dataProcessing(this._currentAppointmentData);
                this._renderAppointmentAll();
            }
            else {
                if (this.currentView() != "agenda") {
                    if (isBigApp || args[this.model.appointmentSettings["recurrence"]] == true && args[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1) {
                        var appointCount = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("ParentId", ej.FilterOperators.equal, args.ParentId));
                    }
                    else {
                        var appointCount = [args];
                        var appId = appointCount[0]["Guid"];
                        //this._appMainId = appointCount[0][this.model.appointmentSettings["id"]];
                    }
                    for (var i = 0; i < appointCount.length; i++) {
                        var strDate = appointCount[i][this.model.appointmentSettings["startTime"]];
                        var index = (this.currentView() == "day" && (new Date(new Date(this._dateRender[0]).setHours(0, 0, 0, 0)).getTime() == new Date(new Date(obj[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0)).getTime())) ? 0 : (this.model.orientation == "horizontal" && this.currentView() == "month") ? $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this.monthDays) : this.currentView() == "day" ? -1 : $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this._dateRender);
                        var resIndex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings["id"], args[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : 0;
                        var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(this._dateRender[index]));
                        predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(this._dateRender[index]).setHours(23, 59, 59)));
                        predicate = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[resIndex][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
                        var apps = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(predicate));
                        this._appList = (this.currentView() !== "month" && !(this._isCustomView()) && !obj[this.model.appointmentSettings["allDay"]]) ? this._alldaySort(apps) : apps;
                        if (this.model.showTimeScale) {
                            ((this.currentView() !== "month" && !(this._isCustomView())) ? ((this.model.orientation == "vertical") && index >= 0 && (obj[this.model.appointmentSettings["allDay"]] || (Math.round((obj[this.model.appointmentSettings["endTime"]] - obj[this.model.appointmentSettings["startTime"]]) / 3600000) >= 24)) ? this._renderAllDayAppointments() : this._renderApp(appointCount[i], index, resIndex)) : this._renderAppointmentMonth(appointCount[i], index, resIndex));
                        }
                        else {
                            this._renderAppointmentMonth(appointCount[i], index, resIndex);
                            if (appointCount[i][this.model.appointmentSettings["allDay"]] != true) {
                                (this._dragDrop) && this._renderMonthAppointment();
                                (this._dragDrop) && this._monthOverlap(appointCount[i]);
                            }
                        }
                        if (this.overlapList.length > 1) {
                            for (var j = 0; j < this.overlapList.length; j++) {
                                this.element.find("#Appointment_" + this.overlapList[j]["AppTaskId"]).remove();
                                this.element.find("#categorybar_" + this.overlapList[j]["AppTaskId"]).remove();
                                var strDate = this.overlapList[j][this.model.appointmentSettings["startTime"]];
                                var index = this.currentView() == "day" ? 0 : (this.model.orientation == "horizontal" && this.currentView() == "month") ? $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this.monthDays) : $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this._dateRender);
                                if (this.model.showTimeScale)
                                    this.currentView() == "month" || (this._isCustomView()) ? this._renderAppointmentMonth(this.overlapList[j], index, resIndex) : this._renderApp(this.overlapList[j], index, resIndex);
                                else
                                    this._renderAppointmentMonth(this.overlapList[j], index, resIndex);
                            }
                        }
                        if (this._currentAction == ej.Schedule.Actions.Save && !obj[this.model.appointmentSettings["allDay"]] && (this.currentView() !== "month" && !(this._isCustomView())))
                            this._reRenderApp(args);
                        this._currentAction = "";
                    }
                    this._unWireResizeEvents();
                    this._wireResizeEvents();
                    this._dragdropAppointments();
                } else
                    this._renderAgendaContent();
            }
        },

        _timeFormat: function (time) {
            var culture = Globalize.culture(this.model.locale);
            var hours = Number(time.match(/^(\d+)/)[1]);
            var minutes = Number(time.match(/:(\d+)/)[1]);
            if (!ej.isNullOrUndefined(time.split(" ")[1])) {
                var AMPM = time.match(/\s(.*)$/)[1];
                if (AMPM == culture.calendar.PM[0] && hours < 12) hours = hours + 12;
                if (AMPM == culture.calendar.AM[0] && hours == 12) hours = hours - 12;
            }
            var sHours = hours.toString();
            var sMinutes = minutes.toString();
            if (hours < 10) sHours = "0" + sHours;
            if (minutes < 10) sMinutes = "0" + sMinutes;
            return sHours + ":" + sMinutes;
        },

        _processEscapeKey: function () {
            this._processFocus();
            this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
        },

        _processFocus: function () {
            var selectApp = this.element.find("div.e-selectedAppointment");
            if (selectApp.length != 0)
                selectApp.focus();
            if (this.element.find("td.e-selectedCell").hasClass("e-workcells")) {
                var oRowCell = this.element.find('div.e-draggableworkarea tr');
                var rowcount = oRowCell.length - 1;
                var omincolIndex, omaxcolIndex, rowInx, maxrowInx, checkcolindex;
                var curr_View = this.currentView();
                var proxy = this;
                this.element.find('td.e-selectedCell').each(function (index, tdItem) {
                    var targetId = $(tdItem).index();
                    if (targetId != null && !isNaN(targetId)) {
                        if (omincolIndex == null) {
                            omincolIndex = targetId;
                            rowInx = $(tdItem).parent()[0].rowIndex;
                        }
                        else if (omincolIndex < targetId) {
                            omincolIndex = omincolIndex;
                        }
                        else {
                            if (omincolIndex != targetId) {
                                omincolIndex = targetId;
                                rowInx = $(tdItem).parent()[0].rowIndex;
                            }
                        }
                        if (omaxcolIndex == null) {
                            omaxcolIndex = targetId;
                            if (curr_View != "month") {
                                checkcolindex = parseInt(omaxcolIndex);
                                for (var i = 0; i <= rowcount; i++) {
                                    if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                        maxrowInx = i;

                                    }
                                }
                            }
                        }
                        else if (omaxcolIndex > targetId) {
                            omaxcolIndex = omaxcolIndex;
                        }
                        else {
                            if (omaxcolIndex != targetId) {
                                omaxcolIndex = targetId;
                                if (curr_View != "month") {
                                    checkcolindex = parseInt(omaxcolIndex);
                                    for (var i = 0; i <= rowcount; i++) {
                                        if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                            maxrowInx = i;

                                        }
                                    }
                                }
                            }
                        }
                    }
                });
                if (curr_View != "month") {
                    $(oRowCell[maxrowInx].cells[omaxcolIndex]).attr("tabIndex", 0).focus();
                }
                else {
                    var tdsel = oRowCell.find('td.e-selectedCell');
                    var tdlen = parseInt(tdsel.length) - 1;
                    $(tdsel[tdlen]).attr("tabIndex", 0).focus();
                }
            }
            else {
                this.element.focus();
                this.element.find("td.e-selectedCell").attr("tabIndex", 0).focus();
            }
        },

        _clearFields: function () {
            this._appointmentAddWindow.find('.subject').val("");
            this._appointmentAddWindow.find('.id').val("");
            this._appointmentAddWindow.find('.e-description').val("").css("height", "");
            this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(), this._pattern.d));
            this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(), this._pattern.d));
            this._appointmentAddWindow.find('.startendtime').ejTimePicker({ enabled: true });
            this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(), this._pattern.t));
            this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(), this._pattern.t));
            this._appointmentAddWindow.find(".recurrence,.weekdays").ejCheckBox({ checked: false, enabled: true });
            this._appointmentAddWindow.find(".e-repeatlbl").removeClass('e-disable');
            this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: false });
            this._appointmentAddWindow.find(".startutc,.endutc").ejDropDownList("clearText");
            this._appointmentAddWindow.find("." + this._id + "summary").css("display", "none");
            this._appointmentAddWindow.find(".e-recurFinalRule").html("");
            this._appointmentAddWindow.find(".e-recuredit").hide();
            this._appointmentAddWindow.find(".e-categorizevalue").ejAutocomplete("clearText");
        },

        _getNumericModel: function () {
            return numericModel = {
                width: "100px",
                minValue: 1,
                value: 1,
                cssClass: this.model.cssClass,
                enableRTL: this.model.enableRTL,
                localize: this.model.locale
            };
        },

        _recurClearFields: function () {
            var list = this._appointmentAddWindow.find("input[type='checkbox'][name='weekdays']");
            for (var i = 0; i < list.length; i++) {
                $(list[i]).ejCheckBox({ checked: false });
            }
            this._appointmentAddWindow.find('.e-recurevery').ejNumericTextbox({ value: 1 });
            this._appointmentAddWindow.find('.e-recurcount').ejNumericTextbox({ value: 10 });
            this._appointmentAddWindow.find(".e-recurRule").html("");
        },

        _deleteApp: function (id) {
            var level = this.level;
            var args, deleted;
            if (this._trigger("actionBegin", { id: id, requestType: "appointmentDelete" }))
                return false;
            var query = new ej.Query().where("Guid", ej.FilterOperators.equal, id);
            var removeItem = ej.DataManager(this._processed).executeLocal(query);
            if (removeItem.length == 0) return false;
            this._getNewOverlapList(removeItem[0]);
            args = { appointment: removeItem };
            if (this._trigger("appointmentDeleted", args))
                return false;
            var parentObj = this._getAppointmentByParentId(id);
            if (removeItem[0][this.model.appointmentSettings["recurrence"]]) {
                var Parid = removeItem[0][this.model.appointmentSettings["recurrenceId"]] == null ? removeItem[0]["ParentId"] : removeItem[0][this.model.appointmentSettings["recurrenceId"]];
                var MapValue = removeItem[0][this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                var query = new ej.Query().where(MapValue, ej.FilterOperators.equal, Parid);
                deleted = ej.DataManager(this._currentAppointmentData).executeLocal(query);
                if(this.dataSource() instanceof ej.DataManager)
                    var promise = this._dataManager.saveChanges({ added: [], changed: [], deleted: deleted }, this.model.appointmentSettings["id"]);
                else
                    var promise = this._dataManager.saveChanges({ added: [], changed: [], deleted: deleted },"AppTaskId");
            }
            else {
                var p_id = removeItem.length == 0 ? id : parentObj["AppTaskId"];
                if (this.dataSource() instanceof ej.DataManager)
                    var promise = this._dataManager.remove("Id", parentObj[this.model.appointmentSettings["id"]]);
                else
                    var promise = this._dataManager.remove("AppTaskId", p_id);

            }
            var proxy = this;
            if (!ej.isNullOrUndefined(promise)) {
                if ($.isFunction(promise.promise)) {
                    promise.done(function (e) {
                        if (proxy.model.enableLoadOnDemand)
                            proxy._initDataSource("refresh");
                        else {
                            var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                            proxy._currentAppointmentData.length = 0;
                            proxy._dateFormat(value);
                            proxy._bindAppointments = true;
                            proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                            proxy._dataProcessing(value);
                            proxy._renderAppointmentAll();
                        }
                        proxy._trigger("actionComplete", { data: parentObj, requestType: "appointmentDelete" });
                    });
                    promise.fail(function (e) {
                    });
                }
                else {
                    var isBigApp = ((new Date(new Date(parentObj[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(parentObj[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((parentObj[this.model.appointmentSettings["endTime"]] - parentObj[this.model.appointmentSettings["startTime"]]) / 3600000) < 24) && (parentObj[this.model.appointmentSettings["allDay"]] !== true) && parentObj[this.model.appointmentSettings["recurrence"]] != 1);
                    if (this.currentView() != "agenda") {
                        if (this._currentAppointmentData.length > 0)
                            if (this.model.showTimeScale) {
                                if ((parentObj[this.model.appointmentSettings["recurrence"]]) || isBigApp || (((new Date(parentObj[this.model.appointmentSettings["endTime"]]) - new Date(parentObj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) {
                          
                                    var removeItemValue = !parentObj[this.model.appointmentSettings["recurrence"]] ? removeItem[0]["AppTaskId"] : removeItem[0][this.model.appointmentSettings["recurrenceId"]] == null ? removeItem[0]["ParentId"] : removeItem[0][this.model.appointmentSettings["recurrenceId"]];
                                    new ej.DataManager(this._currentAppointmentData).remove("AppTaskId", parentObj["AppTaskId"])
                                    this._dataProcessing(this._currentAppointmentData);
                                    this._renderAppointmentAll();
                                }
                                else {
                                    new ej.DataManager(this._processed).remove("Guid", id);
                                    this.element.find('[guid=' + id + ']').remove();
                                    this.element.find("#categorybar_" + id).remove();
                                    (this.currentView() !== "month" && !(this._isCustomView()) ? (this.model.orientation == "vertical" && removeItem[0][this.model.appointmentSettings["allDay"]] == 1 ? this._reRenderAllDayAppointments() : this._reRenderApp(removeItem[0])) : this._renderAppointmentAll());
                                    level = this.allDayLevel;
                                }
                            }
                            else
                                ((parentObj[this.model.appointmentSettings["recurrence"]]) || isBigApp || (((new Date(parentObj[this.model.appointmentSettings["endTime"]]) - new Date(parentObj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24)) ? this._recurrenceIdDelete(removeItem[0].ParentId) && this._dataProcessing(this._currentAppointmentData) && this._renderAppointmentAll() : new ej.DataManager(this._processed).remove("Guid", id) && this.element.find('[guid=' + id + ']').remove() && this.element.find("#categorybar_" + id).remove() && this._renderAppointmentAll();
                        else {
                            var parentQuery = new ej.Query().where("ParentId", ej.FilterOperators.equal, parentObj.ParentId);
                            var templist = new ej.DataManager(this._processed).executeLocal(parentQuery);
                            for (i = 0; i < templist.length; i++) {
                                this.element.find('[guid=' + templist[i]["Guid"] + ']').remove();
                                this.element.find("#categorybar_" + templist[i]["Guid"]).remove();
                                new ej.DataManager(this._processed).remove("Guid", templist[i]["Guid"]);
                            }
                        }
                    }
                    else {
                        new ej.DataManager(this._processed).remove("Guid", id);
                        this._recurrenceIdDelete(removeItem[0].ParentId);
                        this._renderAgendaContent();
                    }
                    this._trigger("actionComplete", { data: parentObj, requestType: "appointmentDelete" });
                    if (this.model.showTimeScale && ((((args.appointment[0][this.model.appointmentSettings["endTime"]] - args.appointment[0][this.model.appointmentSettings["startTime"]]) / 3600000) > 24) || args.appointment[0][this.model.appointmentSettings["allDay"]]) && this.currentView() != "month" && !(this._isCustomView()) && this.currentView() != "agenda" && (level == 0)) {
                        if (!this._mediaQuery)
                            this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").css('height', 27 + "px");
                        else
                            this.element.find(".e-alldaycells,.e-leftindent,.e-emptyalldaytd,.e-schedulesettings").attr('style', "height:40px !important");
                        this.element.find(".e-alldaycells").first().css(this.model.enableRTL ? 'border-right' : 'border-left', "none");
                        this._reRenderScroller();
                    }                    
                }
            }
            var appCount = this._appointmentCount();
            if (appCount.length < 1) {
                if (this.model.enableAppointmentNavigation)
                    this._nextPrevButton(appCount);
            }
        },

        _appointmentCount: function () {
            var renderDate = this._dateRender;
            var appointments = this._processed;
            appointments = this._sortAppById(appointments);
            appointments = this._appointmentSort(appointments);
            var endDate = this.currentView() == "agenda" ? new Date(new Date(new Date(renderDate[0]).setDate(renderDate[0].getDate() + this.model.agendaViewSettings.daysInAgenda)).setHours(23, 59, 59)) : new Date(new Date(renderDate[renderDate.length - 1]).setHours(23, 59, 59));
            var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(renderDate[0]));
            predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, endDate);
            var newAppList = new ej.DataManager(appointments).executeLocal(new ej.Query().where(predicate));
            return newAppList;
        },

        _reRenderAllDayAppointments: function () {
            this._alldayLevel = (this.level == 1) ? true : false;
            (this.model.showAllDayRow) && this.element.find("div.e-alldayappointment").remove() && this._renderAllDayAppointments();
            this._unWireResizeEvents();
            this._wireResizeEvents();
            this._dragdropAppointments();
        },

        _recurrenceIdDelete: function (id) {
            var query = ej.Query().where(ej.Predicate("ParentId", ej.FilterOperators.equal, id));
            var _newAddList = new ej.DataManager(this._processed).executeLocal(query);
            for (var i = 0; i < _newAddList.length; i++) {
                new ej.DataManager(this._processed).remove("Guid", _newAddList[i]["Guid"]);
                this.element.find('[guid=' + _newAddList[i]["Guid"] + ']').remove();
                this.element.find("#categorybar_" + _newAddList[i]["Guid"]).remove();
            }
        },

        _appointmentResizeStart: function (e) {
            if (this.model.enableAppointmentResize) {
                var data = { element: $(e.target).parent() };
                this._appointdata = $(e.target).parent();
                if (this._trigger("actionBegin", { data: data, requestType: "appointmentResize" }))
                    return false;
                if (this._trigger("resizeStart", data))
                    return false;
                e.preventDefault();
                this.model.showQuickWindow && this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close");
                this.resizeStart = true;
                if ($(e.target).hasClass("e-lefthandle") || $(e.target).hasClass("e-righthandle")) {
                    this.resize = e.pageX;
                    this.appoint_width = $(e.target).parent().width();
                    this.appoint_left = parseInt($(e.target).parent()[0].style.left);
                    this._appointleft = parseInt($($(e.target).parent()[0]).offset().left);
                    this._appointright = parseInt($($(e.target).parent()[0]).css('right'));
                    if (this.model.orientation == "horizontal" && this.model.enableRTL)
                        $(e.target).hasClass("e-lefthandle") ? this._rightResizing = true : this._leftResizing = true;
                    else $(e.target).hasClass("e-lefthandle") ? this._leftResizing = true : this._rightResizing = true;
                }
                else if ($(e.target).hasClass("e-tophandle") || $(e.target).hasClass("e-bottomhandle")) {
                    this.resize = e.pageY;
                    this.appoint_height = $(e.target).parent().height();
                    this.appoint_top = parseInt($(e.target).parent()[0].style.top);
                    $(e.target).hasClass("e-tophandle") ? this._topResizing = true : this._bottomResizing = true;
                }
                this._curElmt = $(e.target).parent();
                this.element.find(".e-workcells").removeClass("e-selectedCell,e-mousehover");
                $(document).bind(ej.eventType.mouseMove, $.proxy(this._appointmentResizing, this));
                $(document).bind(ej.eventType.mouseUp, $.proxy(this._appointmentResizeStop, this));
                if ($(e.target).hasClass("e-rowcursor") || $(e.target).hasClass("e-columncursor"))
                    return false;
            }
        },

        _appointmentResizing: function (e) {
            var target = $(e.target);
            var data = { element: target.parent() };
            var appointDiv = $(e.target).hasClass("e-appointment") || $(e.target).hasClass("e-monthappointment") || $(e.target).hasClass("e-workcells") || $(e.target).hasClass("e-monthcells") || $(e.target).hasClass("e-alldayappointment") || $(e.target).hasClass("e-alldaycells") ? $(e.target) : $(e.target).parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
            var cellHeight = this.element.find(".e-workcells").height();
            var appointcellCount = ($(appointDiv[0]).height() / cellHeight);
            if (this._trigger("resize", data))
                return false;
            if (this._topResizing) {
                if (appointcellCount > 0 && !$(e.target).hasClass("e-alldaycells") && !$(e.target).hasClass("e-apptext")) {
                    this._curElmt.height(this.appoint_height + this.resize - e.pageY + 3);
                    this._curElmt.css("top", this.appoint_top - (this.resize - e.pageY) - 3);
                }
            }
            else if (this._bottomResizing) {
                this._curElmt.height(this.appoint_height + (e.pageY - this.resize) + 2);
            }
            else if (this._rightResizing) {
                if (this.model.orientation == "horizontal" && this.model.enableRTL) {
                    this._curElmt.width(this.appoint_width + this.resize - e.pageX + 3);
                    this._curElmt.css("left", this.appoint_left - (this.resize - e.pageX) - 3);
                }
                else {
                    this._curElmt.width(this.appoint_width + (e.pageX - this.resize) + 2);
                    this._curElmt.css("right", this._appointright - (e.pageX - this.resize) - 2);
                }
            }
            else if (this._leftResizing) {
                if (this.model.orientation == "horizontal" && this.model.enableRTL) {
                    this._curElmt.width(this.appoint_width + (e.pageX - this.resize) + 2);
                    this._curElmt.css("right", this._appointright - (e.pageX - this.resize) - 2);
                }
                else {
                    this._curElmt.width(this.appoint_width + this.resize - e.pageX + 3);
                    this._curElmt.css("left", this.appoint_left - (this.resize - e.pageX) - 3);
                }
            }
        },

        _appointmentResizeStop: function (e) {
            try {
                var dragArea = this.element.find(".e-draggableworkarea");
                this.resizeStart = false;
                $(document).unbind(ej.eventType.mouseMove, $.proxy(this._appointmentResizing, this));
                $(document).unbind(ej.eventType.mouseUp, $.proxy(this._appointmentResizeStop, this));
                var model = this.model.appointmentSettings;
                var appList = this._currentAppointmentData;
                var workcell = this.element.find(".e-workcellstab");
                var scheduleHeight = workcell.find("tr").length * this.element.find(".e-workcells").height();
                var appointDiv = $(e.target).hasClass("e-appointment") || $(e.target).hasClass("e-monthappointment") || $(e.target).hasClass("e-alldayappointment") ? $(e.target) : $(e.target).parents(".e-appointment,.e-alldayappointment,.e-monthappointment");
                (appointDiv.length == 0) && (appointDiv = this._appointdata);
                if (appointDiv[0].id != this._appointdata[0].id) { appointDiv = this._appointdata; }
                 this._appUid = appointDiv.attr('guid');
                 var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                 var twoDayApps = tempApp.length > 1 ? true : false;
                var appoint = ((tempApp[0][model["recurrence"]] && tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1)) ? this._getAppointmentByParentId(this._appUid) : (tempApp[0][model["recurrence"]] ? tempApp[0] : this._getAppointmentByParentId(this._appUid));
                var noOfDays = this.currentView() === "week" || this.currentView() === "month" || this._isCustomView() ? 7 : this.currentView() === "workweek" && this.model.workWeek.length > 0 ? this.model.workWeek.length : 1;
                noOfDays = (this.currentView() == "customview" && this._renderDays <= 7) ? this._renderDays : noOfDays;
                if (appoint != null || appoint !== undefined) {
                    var bigApps = appoint[this.model.appointmentSettings["startTime"]].getDate() < new Date(appoint[this.model.appointmentSettings["endTime"]]).getDate() ? true : false;
                    var newStartTime = new Date(appoint[this.model.appointmentSettings["startTime"]]);
                    var newEndTime = new Date(appoint[this.model.appointmentSettings["endTime"]]);
                    if (twoDayApps && tempApp[0][model["recurrence"]]) {
                        var splitObj = this._sortAppById(new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, tempApp[0].Guid)));
                        newStartTime = new Date(splitObj[0][this.model.appointmentSettings["startTime"]]);
                        newEndTime = new Date(splitObj[1][this.model.appointmentSettings["endTime"]]);
                    }
                    else if (tempApp[0][model["recurrence"]]) {
                        newStartTime = new Date(tempApp[0][this.model.appointmentSettings["startTime"]]);
                        newEndTime = new Date(tempApp[0][this.model.appointmentSettings["endTime"]]);
                    }
                    var StartTime = appoint[this.model.appointmentSettings["startTime"]];
                    var EndTime = appoint[this.model.appointmentSettings["endTime"]];
                    var resizeElmntHeight = $(appointDiv[0]).height();

                    if (this.currentView() !== "month" && !(this._isCustomView())) {
                        var appTop = $(appointDiv[0]).offset().top;
                        var cellHeight = this.element.find(".e-workcells").height();
                        var resizeMax = dragArea.height() + dragArea.scrollTop();
                        var topPos = appTop - dragArea.offset().top + dragArea.scrollTop();
                        var resizeMaxTop = (topPos - (cellHeight / ((60 / 30)))) < resizeMax ? topPos : resizeMax - resizeElmntHeight;
                    }
                    var cellCount = ($(appointDiv[0]).height() / cellHeight);
                    var cellDiff = (resizeMaxTop + (cellCount * cellHeight)) * 60 / ((60 / 30) * (cellHeight + 1)) / 30;
                    var difHeight = resizeElmntHeight - parseInt(cellDiff);
                    var apptotheight = resizeMaxTop + difHeight;
                    if (this._topResizing) {
                        var splitTopResize = this._topResizing;
                        if (resizeMaxTop > 0) {
                            var newTopTime;
                            newTopTime = resizeMaxTop * 60 / ((60 / 30) * (cellHeight + 1));
                            newStartTime.setHours(parseInt(this.model.startHour) + parseInt(newTopTime / 60));
                            newStartTime.setMinutes(parseInt(newTopTime % 60));
                            newStartTime.setMinutes((Math.round(newStartTime.getMinutes() / 10.0) * 10));
                        }
                        else {
                            newStartTime.setHours(0, 0, 0);
                        }
                        this._topResizing = false;
                    }
                    else if (this._bottomResizing) {
                        var splitBottomResize = this._bottomResizing;
                        if (resizeMaxTop >= 0 && apptotheight < scheduleHeight) {
                            var newTopTime;
                            var newcellCount = ($(appointDiv[0]).height() / cellHeight);
                            newTopTime = (resizeMaxTop + (newcellCount * cellHeight)) * 60 / ((60 / 30) * (cellHeight + 1));
                            newEndTime.setHours(parseInt(this.model.startHour) + parseInt(newTopTime / 60));
                            newEndTime.setMinutes(parseInt(newTopTime % 60));
                            newEndTime.setMinutes((Math.round(newEndTime.getMinutes() / 10.0) * 10));
                        }
                        else {
                            newEndTime.setHours(23, 59, 59);
                        }
                        this._bottomResizing = false;
                    }
                    else if (this._leftResizing) {
                        if (this.model.orientation == "vertical") {
                            var index = (!this.model.group) ? 0 : this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings["id"], appoint[this.model.appointmentSettings.resourceFields.split(',')[this.model.resources.length - 1]]);
                            if (this.currentView() !== "month" && !(this._isCustomView()) && this.model.showTimeScale) {
                                if (!this.model.enableRTL) {
                                    if ($(appointDiv[0]).offset().left > $(this.element.find(".e-alldaycells")[index * noOfDays]).offset().left) {
                                        newStartTime = new Date(this._dateRender[0]);
                                        var diffLeft = $(appointDiv[0]).offset().left - $(this.element.find(".e-alldaycells")[index * noOfDays]).offset().left;
                                        var diffDays = Math.round(diffLeft / (this.element.find(".e-alldaycells").width() + 1));
                                        newStartTime.setDate(new Date(newStartTime).getDate() + diffDays);
                                        newStartTime.setHours(new Date(StartTime).getHours());
                                        newStartTime.setMinutes(new Date(StartTime).getMinutes());
                                    }
                                }
                                else {
                                    var diffWidth = $(appointDiv[0]).width() - this.appoint_width;
                                    var diffDays = Math.round(diffWidth / (this.element.find(".e-alldaycells").width()));
                                    newEndTime = new Date(tempApp[0][model["endTime"]]);
                                    newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);
                                    newEndTime.setHours(new Date(EndTime).getHours(), new Date(EndTime).getMinutes());
                                }
                            }
                            else {
                                var cellsLeft = !this.model.enableRTL ? ($(this.element.find(".e-monthcells")[index * noOfDays]).offset().left) : $(this.element.find(".e-monthcells")[(index + 1) * noOfDays - 1]).offset().left;
                                if ($(appointDiv[0]).offset().left > cellsLeft) {
                                    if (!this.model.enableRTL) {
                                        this._resize = true;
                                        newStartTime = new Date(this._firstweekdate(new Date(tempApp[0][model["startTime"]])));
                                        this._resize = false;
                                        var diffDays = Math.round(($(appointDiv[0]).offset().left - $(this.element.find(".e-monthcells")[index * noOfDays]).offset().left) / this.element.find(".e-monthcells").width());
                                        newStartTime.setDate(newStartTime.getDate() + diffDays);
                                        newStartTime.setHours(new Date(StartTime).getHours(), new Date(StartTime).getMinutes());
                                    }
                                    else {
                                        var diffDays = Math.round(($(appointDiv[0]).width() - this.appoint_width) / (this.element.find(".e-monthcells").width()));
                                        newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);
                                        newEndTime.setHours(new Date(EndTime).getHours(), new Date(EndTime).getMinutes());
                                    }
                                }
                            }
                        }
                        else {
                            if ($(this.element.find('.e-workcells.e-childworkcell')[0]).offset().left < $(appointDiv[0]).offset().left)
                                var diffLeft = $(appointDiv[0]).offset().left - this._appointleft;
                            else
                                var diffLeft = $(this.element.find('.e-workcells.e-childworkcell')[0]).offset().left - this._appointleft;                            
                            if (this.model.enableRTL) {
                                var diff = $(appointDiv[0]).width() - this.appoint_width;
                                diffLeft = -diff;
                            }
                            if (this.currentView() == "month" || (this._isCustomView()) || !this.model.showTimeScale) {
                                var diffDays = Math.round(diffLeft / (this.element.find(".e-workcells").width() + 1));
                                newStartTime.setDate(new Date(newStartTime).getDate() + diffDays);
                            }
                            else {
                                var diffMinutes = (diffLeft * 60) / ((this.element.find(".e-workcells").width() * 2) + 1);
                                newStartTime.setHours(parseInt(new Date(newStartTime).getHours()) + parseInt(diffMinutes / 60));
                                newStartTime.setMinutes(new Date(newStartTime).getMinutes() + parseInt(diffMinutes % 60));
                                newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
								if (newStartTime < new Date(this._dateRender[0]))
									newStartTime = new Date(this._dateRender[0]);
                            }
                            newStartTime = new Date(newStartTime);
                        }
                        this._leftResizing = false;
                    }
                    else if (this._rightResizing) {
                        if (this.model.orientation == "vertical") {
                            var index = (!this.model.group) ? 0 : this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings["id"], appoint[this.model.appointmentSettings.resourceFields.split(',')[this.model.resources.length - 1]]);
                            if (this.currentView() !== "month" && !(this._isCustomView()) && this.model.showTimeScale) {
                                if (!this.model.enableRTL) {
                                    var diffLeft = appointDiv[0].offsetLeft + $(appointDiv[0]).width();
                                    var diffDays = Math.floor(diffLeft / (this.element.find(".e-alldaycells").width())) - (index * noOfDays);
                                    var cellsLeft = (index == this.res1.length - 1) ? $(this.element.find(".e-alldaycells")[(index + 1) * noOfDays - 1]).offset().left + this.element.find(".e-alldaycells").width() : $(this.element.find(".e-alldaycells")[(index + 1) * noOfDays]).offset().left;
                                    if (cellsLeft >= diffLeft) {
                                        if (this.model.workWeek.length > 0) {
                                            newEndTime = new Date(this._dateRender[0]);
                                            newEndTime.setDate(new Date(this._dateRender[diffDays]).getDate());
                                            newEndTime.setMonth(new Date(this._dateRender[diffDays]).getMonth());
                                            newEndTime.setHours(new Date(EndTime).getHours(), new Date(EndTime).getMinutes());
                                        }
                                        else {
                                            newEndTime = new Date(this._dateRender[0]);
                                            newEndTime.setDate(new Date(newEndTime).getDate() + ((diffDays >= noOfDays) ? diffDays - 1 : diffDays));
                                            newEndTime.setHours(new Date(EndTime).getHours(), new Date(EndTime).getMinutes());
                                        }
                                    }
                                }
                                else {
                                    var diffWidth = $(appointDiv[0]).width() - this.appoint_width;
                                    var diffDays = Math.round(diffWidth / (this.element.find(".e-alldaycells").width()));
                                    newStartTime = new Date(tempApp[0][model["startTime"]]);
                                    newStartTime.setDate(new Date(newStartTime).getDate() - diffDays);
                                    newStartTime.setHours(new Date(StartTime).getHours(), new Date(StartTime).getMinutes());
                                }
                            }
                            else {
                                var cellwidth = this.element.find(".e-monthcells").width();
                                var cellsLeft = this.model.enableRTL ? ($(this.element.find(".e-monthcells")[(index) * noOfDays]).offset().left + cellwidth) : (index == this.res1.length - 1) ? $(this.element.find(".e-monthcells")[(index + 1) * noOfDays - 1]).offset().left + cellwidth : $(this.element.find(".e-monthcells")[(index + 1) * noOfDays]).offset().left;
                                if (cellsLeft >= ($(appointDiv[0]).offset().left + $(appointDiv[0]).width())) {
                                    var diffDays = Math.round(($(appointDiv[0]).width() - this.appoint_width) / cellwidth);
                                    if (!this.model.enableRTL) {
                                        newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);
                                        newEndTime.setHours(new Date(EndTime).getHours(), new Date(EndTime).getMinutes());
                                    }
                                    else {
                                        newStartTime.setDate(new Date(newStartTime).getDate() - diffDays);
                                        newStartTime.setHours(new Date(StartTime).getHours(), new Date(StartTime).getMinutes());
                                    }
                                }
                            }
                        }
                        else {
                            var diffWidth = $(appointDiv[0]).width() - this.appoint_width;
                            if (this.currentView() == "month" || (this._isCustomView()) || !this.model.showTimeScale) {
                                var diffDays = Math.round(diffWidth / (this.element.find(".e-workcells").width() + 1));
                                newEndTime.setDate(new Date(newEndTime).getDate() + diffDays);
                            }
                            else {
                                var diffMinutes = (diffWidth * 60) / ((this.element.find(".e-workcells").width() * 2) + 1);
                                newEndTime.setHours(new Date(newEndTime).getHours() + parseInt(diffMinutes / 60));
                                newEndTime.setMinutes(new Date(newEndTime).getMinutes() + parseInt(diffMinutes % 60));
                                newEndTime.setMinutes(Math.round(newEndTime.getMinutes() / 10.0) * 10);
								if (newEndTime > new Date(new Date(this._dateRender[this._dateRender.length - 1]).setHours(23,59,59)))
									newEndTime = new Date(new Date(this._dateRender[this._dateRender.length - 1]).setHours(23,59,59));
                            }
                            newEndTime = new Date(newEndTime);
                        }
                        this._rightResizing = false;
                    }
                    if ((newStartTime >= newEndTime) || $(appointDiv[0]).width() == 0) {
                        newStartTime = tempApp[0][model["startTime"]];
                        newEndTime = tempApp[0][model["endTime"]];
                    }
                    var reverse = true;
                    if (appoint[model["recurrence"]] == 1 && appoint[model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1) {
                        var _newAddList = {}, _resizeStrTime, _resizeEndTime;
                        var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                        var _appList = new ej.DataManager(this._processed).executeLocal(query)[0];
                        var Parid = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? _appList["ParentId"] : _appList[this.model.appointmentSettings["recurrenceId"]];
                        var MapValue = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                        var recurColl = this._appointmentRecurrenceSort(new ej.DataManager(this._processed).executeLocal(new ej.Query().where(MapValue, ej.FilterOperators.equal, Parid)));
                        $.extend(_newAddList, tempApp[0]);
                        var maxdate = new Date(new Date(this.model.maxDate).getTime());
                        maxdate.setHours(23, 59, 59);
                        if ((newEndTime >= new Date(this.model.minDate)) && (newEndTime <= maxdate)) {
                            var index;
                            (this.model.enableRecurrenceValidation && _newAddList[model["recurrence"]]) && (index = this._recuCollectionIndex(recurColl, _appList));
                            this._processed = this._sortAppById(this._processed);
                            var maxId = this._appMainId + 1;
                            if ((this.model.enableLoadOnDemand)) maxId = this._dataCount;
                            if (this.model.enableRecurrenceValidation && _newAddList[model["recurrence"]] && ((index == 0) ? (!ej.isNullOrUndefined(recurColl[index + 1]) ? !(new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime()) : false) : (index == recurColl.length - 1) ? !(new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) : !((new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime())))) {
                                _newAddList[model["startTime"]] = _appList[model["startTime"]];
                                _newAddList[model["endTime"]] = _appList[model["endTime"]];
                                $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                                this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("DragResizeError"));
                                this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                                this._alertWindow.ejDialog("open");
                                reverse = true;
                            }
                            else {
                                var data = { appointment: _newAddList, target: e };
                                prvStartTime = _newAddList[model["startTime"]];
                                prvEndTime = _newAddList[model["endTime"]];
                                _newAddList[model["startTime"]] = new Date(newStartTime);
                                _newAddList[model["endTime"]] = new Date(newEndTime);
                                if (this._trigger("resizeStop", data)) {
                                    _newAddList[model["startTime"]] = new Date(prvStartTime);
                                    _newAddList[model["endTime"]] = new Date(prvEndTime);
                                    reverse = true;
                                }
                                else {
                                    reverse = false;
                                    var Parid = this._getAppointmentByParentId(this._appUid)[model["recurrenceId"]] == null ? this._getAppointmentByParentId(this._appUid)["ParentId"] : this._getAppointmentByParentId(this._appUid)[model["recurrenceId"]];
                                
                                    this._deleteOcurrence(Parid, new Date(_appList[model["startTime"]]));
                                    _newAddList[model["recurrenceRule"]] = tempApp[0][model["recurrenceRule"]] + ";RECUREDITID=" + _newAddList["ParentId"];
                                    _newAddList["AppTaskId"] = maxId;
                                }
                            }
                            var args = { data: _newAddList };
                            var temp = [];
                            temp.push(this._timeZoneAppointments(args.data, ""));

                            var _editApp = [];
                            var parid = _newAddList[this.model.appointmentSettings["recurrenceId"]] == null ? _newAddList["ParentId"] : _newAddList[this.model.appointmentSettings["recurrenceId"]];
                            var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, parid);
                            var _App = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                            new ej.DataManager(this._currentAppointmentData).update("AppTaskId", _App[0]);
                            _editApp.push(this._timeZoneAppointments(_App[0], ""));
                            var promise;
                            if(this.dataSource() instanceof ej.DataManager)
                                (!reverse) && (promise = this._dataManager.saveChanges({ added: temp, changed: _editApp, deleted: [] }, this.model.appointmentSettings["id"]));
                            else
                                (!reverse) && (promise = this._dataManager.saveChanges({ added: temp, changed: _editApp, deleted: [] },"AppTaskId"));
                            var updateTempNew = [];
                            for (var c = 0; c < temp.length; c++) {
                                updateTempNew = this._timeZoneAppointments(temp[c], "reverse");
                                new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew);
                            }
                            var updateTempNew1 = [];
                            for (var c = 0; c < _editApp.length; c++) {
                                updateTempNew1 = this._timeZoneAppointments(_editApp[c], "reverse");
                                new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew1);
                            }
                            var proxy = this;
                            if (!ej.isNullOrUndefined(promise) && $.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                                promise.done(function (e) {
                                    if (proxy.model.enableLoadOnDemand)
                                        proxy._initDataSource("refresh");
                                    else {
                                        var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                        proxy._currentAppointmentData.length = 0;
                                        proxy._dateFormat(value);
                                        proxy._bindAppointments = true;
                                        proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                        proxy._dataProcessing(value);
                                        proxy._renderAppointmentAll();
                                    }
                                    proxy._trigger("actionComplete", { data: _newAddList, requestType: "appointmentResize" });
                                });
                                promise.fail(function (e) {
                                    proxy._renderAppointmentAll();
                                });
                            }
                            else {
                                this._processed.splice(this._processed.indexOf(_appList), 1);
                                twoDayApps ? this._appointmentProcessing(args.data) : this._processed.push(args.data);
                                if (twoDayApps) {
                                    if (splitBottomResize) this.element.find('[guid=' + tempApp[0]["Guid"] + ']').remove();
                                    else this.element.find('[guid=' + tempApp[0]["Guid"] + ']').remove();
                                }
                                else
                                    this.element.find('[guid=' + tempApp[0]["Guid"] + ']').remove();
                                this._renderSingleApp(_newAddList, _newAddList);
                                this._trigger("actionComplete", { data: _newAddList, requestType: "appointmentResize" });
                            }
                        }
                        else this._renderAppointmentAll();
                    }
                    else {
                        var _prevobj = {};
                        if (twoDayApps)
                            var _updateList = {}, object = new ej.DataManager(this._currentAppointmentData).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, appoint.Guid));
                        else {
                            var query = new ej.Query().where("Guid", ej.FilterOperators.equal, appoint.Guid);
                            var _updateList = {}, object = new ej.DataManager(this._processed).executeLocal(query);
                        }
                        (!object[0][this.model.appointmentSettings["allDay"]]) && this._getNewOverlapList(object[0]);
                        var recurColl = this._appointmentRecurrenceSort(new ej.DataManager(this._processed).executeLocal(new ej.Query().where("ParentId", ej.FilterOperators.equal, object[0].ParentId)));
                        $.extend(_updateList, object);
                        _prevobj = $.extend(true, [], object[0]);
                        var maxdate = new Date(new Date(this.model.maxDate).setHours(23, 59, 59));
                        if (this.model.orientation == "horizontal" && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                            this.element.find("#categorybar_" + appoint["Guid"]).remove();
                        if ((newEndTime >= new Date(this.model.minDate)) && (newEndTime <= maxdate)) {
                            var index;
                            (this.model.enableRecurrenceValidation && _updateList[0][model["recurrence"]]) && (index = this._recuCollectionIndex(recurColl, object[0]));
                            if (this.model.enableRecurrenceValidation && _updateList[0][model["recurrence"]] && ((index == 0) ? (!ej.isNullOrUndefined(recurColl[index + 1]) ? !(new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime()) : false) : (index == recurColl.length - 1) ? !(new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) : !((new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime())))) {
                                _updateList[0][model["startTime"]] = object[0][model["startTime"]];
                                _updateList[0][model["endTime"]] = object[0][model["endTime"]];
                                $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                                this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("DragResizeError"));
                                this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                                this._alertWindow.ejDialog("open");
                                reverse = true;
                            }
                            else {
                                var data = { appointment: _updateList[0], target: e };
                                prevStartTime = _updateList[0][this.model.appointmentSettings["startTime"]];
                                prevEndTime = _updateList[0][this.model.appointmentSettings["endTime"]];
                                _updateList[0][this.model.appointmentSettings["startTime"]] = newStartTime;
                                _updateList[0][this.model.appointmentSettings["endTime"]] = newEndTime;
                                if (this._trigger("resizeStop", data)) {
                                    _updateList[0][this.model.appointmentSettings["startTime"]] = prevStartTime;
                                    _updateList[0][this.model.appointmentSettings["endTime"]] = prevEndTime;
                                    reverse = true;
                                }
                                else {
                                    reverse = false;
                                }
                            }
                            var temp = this._timeZoneAppointments(_updateList[0], "");
                            var promise;
                            if (this.dataSource() instanceof ej.DataManager)
                                (!reverse) && (promise = this._dataManager.update(this.model.appointmentSettings["id"], temp));
                            else
                                (!reverse) && (promise = this._dataManager.update("AppTaskId", temp));
                            var updateTemp = this._timeZoneAppointments(temp, "reverse");
                            new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTemp);
                        }
                        var proxy = this;
                        if (!ej.isNullOrUndefined(promise) && promise && $.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                if (proxy.model.enableLoadOnDemand)
                                    proxy._initDataSource("refresh");
                                else {
                                    var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                    proxy._currentAppointmentData.length = 0;
                                    proxy._dateFormat(value);
                                    proxy._bindAppointments = true;
                                    proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                    proxy._dataProcessing(value);
                                    proxy._renderAppointmentAll();
                                }
                                proxy._trigger("actionComplete", { data: _updateList, requestType: "appointmentResize" });
                            });
                            promise.fail(function (e) {
                                proxy._renderAppointmentAll();
                            });
                        }
                        else {
                            var isBigapp = ((new Date(_updateList[0][this.model.appointmentSettings["endTime"]]) - new Date(_updateList[0][this.model.appointmentSettings["startTime"]])) / 3600000) > 24;
                            if (_updateList[0][this.model.appointmentSettings["recurrence"]])
                                new ej.DataManager(this._processed).update("Guid", _updateList[0]);

                            else if (((_updateList[0][this.model.appointmentSettings["endTime"]] - _updateList[0][this.model.appointmentSettings["startTime"]]) / 3600000) < 24) {
                                dele = new ej.DataManager(this._processed).executeLocal(ej.Query().where("Guid", ej.FilterOperators.equal, _updateList[0]["Guid"]));
                                for (var j = 0; j < dele.length; j++) {
                                    new ej.DataManager(this._processed).remove("Guid", dele[j]["Guid"]);
                                    this.element.find('[guid=' + dele[j]["Guid"] + ']').remove();
                                    this.element.find("#categorybar_" + dele[j]["Guid"]).remove();

                                }
                                this._appointmentProcessing(_updateList[0]);
                            }
                            else
                                new ej.DataManager(this._processed).update("ParentId", _updateList[0]);
                            !twoDayApps && this.element.find('[guid=' + this._appUid + ']').remove();
                            if (this.currentView() !== "month" && !(this._isCustomView())) {
                                this._renderSingleApp(_updateList[0], _updateList[0]);
                                (!twoDayApps && !isBigapp) && this._reRenderApp(_prevobj);
                            }
                            else { (isBigapp) ? this._renderSingleApp(_updateList[0], _updateList[0]) : this._renderAppointmentAll() };
                            this._trigger("actionComplete", { data: _updateList, requestType: "appointmentResize" });
                        }
                        this._appMainId = this._sortAppById(this._processed)[this._processed.length - 1]["AppTaskId"];
                    }
                }
                this.element.find("div.e-tophandle,div.e-bottomhandle").removeClass("e-rowcursor");
                this.element.find("div.e-lefthandle,div.e-righthandle").removeClass("e-columncursor");
            } catch (e) {

            }
        },

        _reRenderApp: function (appoint) {
            var strDate = appoint[this.model.appointmentSettings["startTime"]];
            if (this.model.orientation == "horizontal") {
                var timeScaleRowCount = (60 / 30);
                var startTime = ((new Date(strDate).getHours()) < this.model.startHour) ? (new Date(strDate).getHours() + (-new Date(strDate).getHours()) + (-new Date(strDate).getMinutes())) : (new Date(strDate).getHours() + (-this.model.startHour));
                var groupIndex = (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) ? this._findResourceIndex(this.model.resources[0].resourceSettings.dataSource, this.model.resources[0].resourceSettings.id, appoint[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]) + 1 : 0;
            }
            var index = this.currentView() == "day" ? 0 : (this.model.orientation == "horizontal" && this.currentView() == "month") ? $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this.monthDays) : $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this._dateRender);
            var resIndex = this.model.orientation == "vertical" ? (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings["id"], appoint[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : 0 : (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.level_Resources, this.model.resources[this.model.resources.length - 1].resourceSettings["id"], appoint[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : 0;
            var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(this._dateRender[index]));
            predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(this._dateRender[index]).setHours(23, 59, 59)));
            predicate = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[resIndex][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
            var app = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(predicate));
            this._appList = (this.currentView() !== "month" && !(this._isCustomView()) && !appoint[this.model.appointmentSettings["allDay"]]) ? this._alldaySort(app) : app;
            var cellId = this.model.orientation == "vertical" ? (resIndex * (7) + (new Date(strDate).getDay() + 1)) + (((Math.ceil((index + 1) / 7)) - 1) * (this.res1.length * 7)) : (((this.model.endHour - this.model.startHour) * timeScaleRowCount) * (index)) + (startTime * timeScaleRowCount);
            this.model.orientation == "vertical" ? this.element.find("#" + this._id + "_" + Globalize.format(new Date(appoint[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resIndex) + "_" + (cellId - 1)).remove() : this.element.find("#" + this._id + "_" + Globalize.format(new Date(appoint[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resIndex + groupIndex) + "_" + (cellId)).remove();

            for (var i = 0; i < app.length; i++) {
                this.element.find("#Appointment_" + app[i]["AppTaskId"]).remove();
                this.element.find("#categorybar_" + app[i]["Guid"]).remove();
				var Value = this._findAllDayApp(app[i]);
				((this.model.showTimeScale && this.currentView() !== "month" && !(this._isCustomView())) ? ((index >= 0 && app[i][this.model.appointmentSettings["allDay"]] && this.model.orientation == "vertical") ||(this.model.orientation == "vertical" && (((app[i][this.model.appointmentSettings["endTime"]] - app[i][this.model.appointmentSettings["startTime"]]) / 3600000) > 24)) || Value ? this._setAllDayPos(app[i], index, resIndex) : this._renderApp(app[i], index, resIndex)) : this._renderAppointmentMonth(app[i], index, resIndex));
            }
            var appCount = this._appointmentCount();
            if (appCount.length == 0) {
                this._nextPrevButton(appCount);
            }
            this._unWireResizeEvents();
            this._wireResizeEvents();
            this._dragdropAppointments();
        },

        recurrenceRuleSplit: function (rRule, app) {
            this._rRule = {};
            _ruleExdate = app;
            var _rule, _ruleExdate, _rRuleSplit = rRule.split(";");
            for (var r = 0; r < _rRuleSplit.length; r++) {
                _rule = _rRuleSplit[r].split("=");
                switch ($.trim(_rule[0]).toUpperCase()) {
                    case "FREQ":
                        this._rRule.freq = $.trim(_rule[1]).toUpperCase();
                        break;
                    case "INTERVAL":
                        this._rRule.interval = parseInt(_rule[1], 10);
                        break;
                    case "COUNT":
                        this._rRule.count = parseInt(_rule[1], 10);
                        break;
                    case "UNTIL":
                        this._rRule.until = new Date(_rule[1]);
                        break;
                    case "BYDAY":
                        this._rRule.weekDays = _rule[1];
                        break;
                    case "BYMONTHDAY":
                        this._rRule.monthDay = parseInt(_rule[1], 10);
                        break;
                    case "BYMONTH":
                        this._rRule.month = parseInt(_rule[1], 10);
                        break;
                    case "BYSETPOS":
                        this._rRule.setPositions = parseInt(_rule[1], 10);
                        break;
                    case "WKST":
                        this._rRule.weekStart = _rule[1];
                        break;
                    case "EXDATE":
                        if (!ej.isNullOrUndefined(app) && !ej.isNullOrUndefined(_ruleExdate.split(","))) {
                            var _exDates = _ruleExdate.split(",");
                            this._rRule.exDate = [];
                            for (var i = 0; i < _exDates.length; i++) {
                                this._rRule.exDate[i] = _exDates[i];
                            }
                        }
                        else {
                            var _exDates = _rule[1].split(",");
                            this._rRule.exDate = [];
                            for (var i = 0; i < _exDates.length; i++) {
                                this._rRule.exDate[i] = _exDates[i];
                            }
                        }
                        break;
                    case "RECUREDITID":
                        this._rRule.recurEditId = parseInt(_rule[1]);
                        break;
                }
            }
            return this._rRule;
        },

        _dragdropAppointments: function () {
            if (this.model.readOnly)
                return false;
            this._topResizing = false;
            this._bottomResizing = false;
            this._leftResizing = false;
            this._rightResizing = false;
            var appointdiv;
            if (this.model.allowDragDrop) {
                var proxy = this;
                var $dragableElements = this.element.find(".e-appointment");
                $dragableElements.ejDraggable({
                    handle: ".e-appointment",
                    dragArea: proxy.element.find("div.e-draggableworkarea",(".e-headrealldaytable tr")[(!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.model.group.resources.length + 1 : 1]),
                    cursorAt: { top: 0, left: 0 },
                    dragStart: function (args) {
                        proxy._dragStart(args);
                    },
                    drag: function (args) {
                        proxy._dragOver(args);
                    },
                    dragStop: function (args) {
                        proxy._dragStop(args);
                    },
                    helper: function (event, ui) {
                        appointdiv = $(event.sender.target).hasClass("e-appointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-appointment");
                        return proxy._appointElement(appointdiv, proxy);
                    }

                });
                var $dragableElements = this.element.find(".e-alldayappointment");
                $dragableElements.ejDraggable({
                    handle: ".e-alldayappointment",
                    dragArea: proxy.element.find((".e-headrealldaytable tr")[(!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.model.group.resources.length + 1 : 1],"div.e-draggableworkarea"),
                    cursorAt: { top: 17, left: 0 },
                    dragStart: function (args) {
                        proxy._dragStart(args);
                    },
                    drag: function (args) {
                        proxy._dragOver(args);
                    },
                    dragStop: function (args) {
                        proxy._dragStop(args);
                    },
                    helper: function (event, ui) {
                        appointdiv = $(event.sender.target).hasClass("e-alldayappointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-alldayappointment");
                        return proxy._appointElement(appointdiv, proxy);
                    }

                });
                var $dragableElements = this.element.find(".e-monthappointment");
                $dragableElements.ejDraggable({
                    handle: ".e-monthappointment",
                    dragArea: proxy.element.find("div.e-draggableworkarea"),
                    cursorAt: { top: 0, left: 0 },
                    dragStart: function (args) {
                        proxy._dragStart(args);
                    },
                    drag: function (args) {
                        proxy._dragOver(args);
                    },
                    dragStop: function (args) {
                        proxy._dragStop(args);
                    },
                    helper: function (event, ui) {
                        appointdiv = $(event.sender.target).hasClass("e-monthappointment") ? $(event.sender.target) : $(event.sender.target).parents(".e-monthappointment");
                        return proxy._appointElement(appointdiv, proxy);
                    }

                });
            }
        },
        _appointElement: function (appointdiv, proxy) {
            var _cloneWidth = (proxy.model.orientation == "vertical" && appointdiv.hasClass("e-appointment") && this.currentView() != "day") ? appointdiv.closest('td').width() * 0.85 : appointdiv.width();            
            appointdiv.find('.e-schedulemouseclose').removeClass('e-schedulemouseclose e-icon');
            appointdiv.find('.e-tophandle').removeClass('e-tophandle e-appointresizer-top e-icon e-nsexpand');
            appointdiv.find('.e-bottomhandle').removeClass('e-bottomhandle e-appointresizer-bottom e-icon e-nsexpand');
            appointdiv.find('.e-lefthandle').removeClass('e-lefthandle e-appointresizer-left e-icon e-ewexpand');
            appointdiv.find('.e-righthandle').removeClass('e-righthandle e-appointresizer-right e-icon e-ewexpand');
            appointdiv.clone().prop('id', 'cloneElement').appendTo(appointdiv.parent());
            return appointdiv.addClass("e-draggableAppointment e-scheduleAppGlassy").removeClass("e-schedulemouseclose e-tophandle e-bottomhandle").css("width", _cloneWidth + "px").appendTo(proxy.element);
        },
        _dragStart: function (args) {
            if ($(args.element[0]).hasClass("e-appointment") || $(args.element[0]).hasClass("e-monthappointment") || $(args.element[0]).hasClass("e-alldayappointment")) {
                var target = args.element;
                var data = { target: target };
                if (this._trigger("actionBegin", { data: data, requestType: "appointmentDrag" }))
                    return false;
                if (this._trigger("dragStart", data))
                    return false;
            }
            else
                args.cancel = true;
            this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
        },

        _dragOver: function (args) {
            var $target = $(args.element);
            var data = { target: $target };
            this._trigger("drag", data);
            this.element.find(".e-workcells").removeClass("e-selectedCell");
            this.element.find(".e-workcells").removeClass("e-mousehover");
        },

        _dragStop: function (args) {
			this.model.showQuickWindow && (this._quickAppDetailsWindow.ejDialog("close"));
            var proxy = this;
            var model = proxy.model.appointmentSettings; var dragArea = proxy.element.find(".e-draggableworkarea");;
            this._appUid = args.element.attr("guid");
            var tempApp = this._sortAppById(new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid)));
            if (tempApp.length > 1 && !tempApp[0][model["recurrence"]]) {
                var twoDayApps = true;
                var appoint = new ej.DataManager(this._currentAppointmentData).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid))[0];
            }
            else {
                var twoDayApps = false;
				var templist = $.extend(true,[],tempApp);
				templist[0][model["endTime"]] = templist[templist.length - 1][model["endTime"]];
                var appoint = (tempApp[0][model["recurrence"]] && (tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1)) ? this._getAppointmentByParentId(this._appUid) : (tempApp[0][model["recurrence"]] ? (tempApp.length > 1) ? templist[0] : tempApp[0] : this._getAppointmentByParentId(this._appUid));
                if (tempApp.length > 1) {
                    var appId = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("AppTaskId", ej.FilterOperators.equal, parseInt(args.element[0].id.split("_")[1])))[0];
                    var splitObj = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid));
                    var twoDayApps = true;
                }
            }
			var apps = $.extend(true,{},appoint);
			var splitDrag = ((Math.ceil((apps[model["endTime"]] - apps[model["startTime"]]) / 3600000) < 24) && (apps[model["endTime"]].getDay() != apps[model["startTime"]].getDay())) ? true : false;
            var appList = this._currentAppointmentData;
            if (this.model.orientation == "vertical") {
                if ($(args.element[0]).hasClass("e-appointment")) {
                    var dragMax = dragArea.height() + dragArea.scrollTop();
                    var dragElmntHeight = args.element[0].offsetHeight;
                    var cellHeight = this.element.find(".e-workcells").height();
                    var topPos = $(args.element[0]).offset().top - dragArea.offset().top + dragArea.scrollTop();
                    var dragMaxTop = (topPos - (cellHeight / (60 / 30))) < dragMax ? topPos : dragMax - dragElmntHeight;
                    if ((dragMaxTop > 0 || !this.model.showAllDayRow) && (args.target.classList[0] != "e-alldaycells")) {
                        var newTopTime;
                        newTopTime = (dragMaxTop > 0) ? dragMaxTop * 60 / ((60 / 30) * (cellHeight + 1)) : 1 * 60 / ((60 / 30) * (cellHeight + 1));

                        var twoDayApps = (new Date(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(appoint[proxy.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) ? true : false;
                        var startTime = appoint[proxy.model.appointmentSettings["startTime"]];
                        var endTime = appoint[proxy.model.appointmentSettings["endTime"]];
                        if (twoDayApps && tempApp[0][model["recurrence"]]) {
                            startTime = splitObj[0][this.model.appointmentSettings["startTime"]];
                            endTime = splitObj[1][this.model.appointmentSettings["endTime"]];
                        }
                        var duration = (endTime - startTime) / 1000;
                        var hours = Math.floor(duration / 3600);
                        var minutes = Math.floor((duration % 3600) / 60);
                        var newStartTime, newEndTime, cellstrtime, cellendtime;

                        if (proxy.currentView() !== "month" && !(proxy._isCustomView())) {
                            var firstDayOfWeek = new Date(proxy._firstweekdate(proxy.currentDate()));
                            if (proxy.currentView() === "workweek" || (proxy.currentView() == "customview" && proxy._renderDays == 5)) {
                                var _day = new Date(firstDayOfWeek).getDay();
                                var _dayAdd = _day == 0 ? 1 : 0;
                                firstDayOfWeek.setDate(firstDayOfWeek.getDate() + _dayAdd);
                            }
                            newStartTime = firstDayOfWeek;
                            var dragAreaLength = dragArea.width();
                            var noOfCols = proxy.currentView() === "week" ? 7 : proxy.currentView() === "workweek" ? this.model.workWeek.length : proxy.currentView() == "customview" ? proxy._renderDays : 1;
                            noOfCols = !ej.isNullOrUndefined(proxy.model.group) ? noOfCols * proxy.res1.length : noOfCols;
                            var cellIndex = Math.floor(($(args.element[0]).offset().left + dragArea.scrollLeft() - dragArea.offset().left) / this.element.find(".e-workcells").width());
                            newStartTime = (this.model.enableRTL) ? new Date(this.dateRender[noOfCols - cellIndex - 1]) : new Date(this.dateRender[cellIndex]);
                            newEndTime = newStartTime;
                        }
                        else {
                            newStartTime = appoint[proxy.model.appointmentSettings["startTime"]];
                            newEndTime = new Date(newStartTime);
                        }

                        if (twoDayApps) {
                            newStartTime = proxy._getNewStartTime(newTopTime, newStartTime);
                            cellstrtime = new Date(newStartTime);
                            newEndTime = proxy._getNewEndTime(newStartTime, hours, minutes);
                            cellendtime = new Date(newEndTime);
                        }
                        else {
                            newStartTime.setHours(parseInt(proxy.model.startHour) + Math.round(newTopTime / 60));
                            newStartTime.setMinutes(parseInt(newTopTime % 60));
                            newStartTime = (new Date(newStartTime).getMinutes() >= 30) ? new Date(new Date(newStartTime).setHours(new Date(newStartTime).getHours() - 1)) : new Date(newStartTime);
                            newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
                            cellstrtime = new Date(newStartTime);

                            var totMin = (appoint[proxy.model.appointmentSettings["endTime"]].getHours() * 60 + appoint[proxy.model.appointmentSettings["endTime"]].getMinutes()) -
                     (appoint[proxy.model.appointmentSettings["startTime"]].getHours() * 60 + appoint[proxy.model.appointmentSettings["startTime"]].getMinutes());

                            newEndTime.setHours(new Date(newStartTime).getHours() + parseInt(totMin / 60));
                            newEndTime.setMinutes(new Date(newStartTime).getMinutes() + parseInt(totMin % 60));
                            newEndTime.setMinutes(Math.round(newEndTime.getMinutes() / 10.0) * 10);
                            cellendtime = new Date(newEndTime);
                        }
                    }
                    else {
                        var dragAreaLength = dragArea.width();
                        var noOfCols = proxy.currentView() === "week" ? 7 : proxy.currentView() === "workweek" ? this.model.workWeek.length : proxy.currentView() == "customview" ? proxy._renderDays : 1;
                        noOfCols = !ej.isNullOrUndefined(proxy.model.group) ? noOfCols * proxy.res1.length : noOfCols;
                        var cellIndex = Math.floor(($(args.element[0]).offset().left + dragArea.scrollLeft() - dragArea.offset().left) / this.element.find(".e-workcells").width());
                        newStartTime = (this.model.enableRTL) ? new Date(this.dateRender[noOfCols - cellIndex - 1]) : new Date(this.dateRender[cellIndex]);
                        newStartTime.setHours(0);
                        newStartTime.setMinutes(0);
                        cellstrtime = new Date(newStartTime);
                        newEndTime = newStartTime;
                        newEndTime.setHours(23);
                        newEndTime.setMinutes(59);
                        cellendtime = new Date(newEndTime);
                        var allday = true;
                    }
                }
                else if ($(args.element[0]).hasClass("e-alldayappointment")) {
                    if (args.target.classList[0] == "e-workcells") {
                        var cellIndex = args.target.cellIndex;
                        newStartTime = new Date(this.dateRender[cellIndex]);
                        var dragMax = dragArea.height() + dragArea.scrollTop();
                        var dragElmntHeight = args.element[0].offsetHeight;
                        var cellHeight = this.element.find(".e-workcells").height();
                        var topPos = $(args.element[0]).offset().top - dragArea.offset().top + dragArea.scrollTop();
                        var dragMaxTop = (topPos - (cellHeight / (60 / 30))) < dragMax ? topPos : dragMax - dragElmntHeight;
                        var newTopTime = (dragMaxTop > 0) ? dragMaxTop * 60 / ((60 / 30) * (cellHeight + 1)) : 1 * 60 / ((60 / 30) * (cellHeight + 1));
                        newStartTime.setHours(parseInt(this.model.startHour) + Math.round(newTopTime / 60));
                        newStartTime.setMinutes(parseInt(newTopTime % 60));
                        newStartTime = (new Date(newStartTime).getMinutes() >= 30) ? new Date(new Date(newStartTime).setHours(new Date(newStartTime).getHours() - 1)) : new Date(newStartTime);
                        newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
                        cellstrtime = new Date(newStartTime);
                        newEndTime = new Date(newStartTime).setMinutes(newStartTime.getMinutes() + 30);
                        cellendtime = new Date(newEndTime);
                        var allday = false;
                        this.alldayDrag = true;
                    }
                    else {
                        newStartTime = appoint[this.model.appointmentSettings["startTime"]];
                        newEndTime = appoint[this.model.appointmentSettings["endTime"]];
                        if ((tempApp[0][model["recurrence"]] && tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1)) {
                            newStartTime = tempApp[0][this.model.appointmentSettings["startTime"]];
                            newEndTime = tempApp[0][this.model.appointmentSettings["endTime"]];
                        }
                        var diffLeft = $(args.element[0]).offset().left + dragArea.scrollLeft() - this.element.find(".e-alldaycells").offset().left - 1;
                        diffLeft = (this.model.enableRTL) ? Math.abs(diffLeft) : diffLeft;
                        var cellIndex = Math.round(diffLeft / (this.element.find(".e-alldaycells").width() + 1));
                        var count = Math.round($(args.element[0]).width() / (this.element.find(".e-alldaycells").width() + 1));
                        var overall = Math.ceil((newEndTime - newStartTime) / (1000 * 60 * 60 * 24));
                        var endInd = Math.round($(args.element[0]).width() / (this.element.find(".e-alldaycells").width() + 1));
                        var cols = proxy.currentView() === "week" ? 7 : proxy.currentView() === "workweek" ? this.model.workWeek.length : proxy.currentView() == "customview" ? proxy._renderDays : 1;
						newStartTime =new Date(this._dateRender[cellIndex]);
						newEndTime =new Date(this._dateRender[cellIndex]);
                        newEndTime.setHours(appoint[proxy.model.appointmentSettings["endTime"]].getHours(), appoint[proxy.model.appointmentSettings["endTime"]].getMinutes(), appoint[proxy.model.appointmentSettings["endTime"]].getSeconds());
                        if (cellIndex < 0) {
                            if (newStartTime <= new Date(this._dateRender[0])) {
                                cellstrtime = new Date(newStartTime);
                                cellstrtime.setDate(cellstrtime.getDate() + cellIndex);
                                cellstrtime.setHours(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getHours());
                                cellstrtime.setMinutes(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getMinutes());
                                cellstrtime = new Date(cellstrtime);
                            }
                            else {
                                var startdate = new Date(proxy._dateRender[0]);
                                startdate.setDate(startdate.getDate() + cellIndex);
                                startdate.setHours(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getHours());
                                startdate.setMinutes(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getMinutes());
                                cellstrtime = new Date(startdate);
                            }
                        }
                        else {
                            if (newStartTime < new Date(this._dateRender[0])) {
                                var Index = (cellIndex > cols - 1) ? (cellIndex % cols) : cellIndex;
                                newStartTime.setDate(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getDate() + Index);
                                newStartTime.setHours(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getHours());
                                newStartTime.setMinutes(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getMinutes());
                                cellstrtime = new Date(newStartTime);
                            }
                            else {
                                var Index = (cellIndex > cols - 1) ? (cellIndex % cols) : cellIndex;
                                cellstrtime = new Date(new Date(this._dateRender[Index]));
                                cellstrtime.setHours(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getHours());
                                cellstrtime.setMinutes(new Date(appoint[proxy.model.appointmentSettings["startTime"]]).getMinutes());
                            }
                        }
                        var starttime = new Date(cellstrtime);
                        starttime.setDate(starttime.getDate() + overall - 1);
                        starttime.setHours(new Date(appoint[proxy.model.appointmentSettings["endTime"]]).getHours());
                        starttime.setMinutes(new Date(appoint[proxy.model.appointmentSettings["endTime"]]).getMinutes());
                        cellendtime = new Date(starttime);
                    }
                }
                else if ($(args.element[0]).hasClass("e-monthappointment")) {
                    var oldStartTime = appoint[proxy.model.appointmentSettings["startTime"]];
                    var oldEndTime = appoint[proxy.model.appointmentSettings["endTime"]];
                    if ((tempApp[0][model["recurrence"]] && tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1)) {
                        oldStartTime = tempApp[0][this.model.appointmentSettings["startTime"]];
                        oldEndTime = tempApp[tempApp.length - 1][this.model.appointmentSettings["endTime"]];
                    }
                    newStartTime = new Date(new Date(proxy.model.currentDate).getFullYear(), new Date(proxy.model.currentDate).getMonth(), 1);
                    var daysInPrevMonth = newStartTime.getDay();
                    newStartTime.setDate(newStartTime.getDate() - daysInPrevMonth);
                    if (this.currentView() == "month" || (this.currentView() == "customview" && !this._oneWeek)) {
                        var cellIndex = parseInt(($(args.element[0]).offset().left + dragArea.scrollLeft() - dragArea.offset().left) / this.element.find(".e-monthcells").width());
                        cellIndex = (this.model.enableRTL) ? 7 - cellIndex - 1 : cellIndex;
                    }
                    else {
						var noOfCols = !ej.isNullOrUndefined(proxy.model.group) ? proxy._dateRender.length * proxy.res1.length : proxy._dateRender.length;
                        var cellIndex = parseInt(($(args.element[0]).offset().left - dragArea.offset().left) / (Math.round(dragArea.width() / noOfCols)));
                    }
					if (splitDrag && (parseInt(args.element.attr("id").split("_")[1]) == tempApp[tempApp.length - 1]["AppTaskId"])) cellIndex -= 1;
                    var newTopDate = parseInt(($(args.element[0]).offset().top + dragArea.scrollTop() - dragArea.offset().top) / (this.element.find(".e-monthcells").outerHeight()));
                    var dateCount = this.currentView() == "month" || this._isCustomView() && !this._oneWeek ? 7 : this._dateRender.length;
                    var index = (this.model.enableRTL) ? (newTopDate * dateCount) + (cellIndex - (Math.floor(cellIndex / dateCount) * dateCount)) - (oldEndTime.getDay() - oldStartTime.getDay()) : (newTopDate * dateCount) + (cellIndex - (Math.floor(cellIndex / dateCount) * dateCount));
                    newStartTime = new Date(new Date(proxy._dateRender[index]).setHours(oldStartTime.getHours(), oldStartTime.getMinutes(), oldStartTime.getSeconds()));
                    if (Math.ceil((appoint[model["endTime"]] - appoint[model["startTime"]]) / 3600000) > 24) {
                        var trgtApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("AppTaskId", ej.FilterOperators.equal, parseInt(args.element.attr('id').split("_")[1])));
                        var multiDif = new Date(trgtApp[0][model["startTime"]]) - new Date(tempApp[0][model["startTime"]]);
                        cellstrtime = new Date(newStartTime - multiDif);
                        var diffTime = (newStartTime.getTime() - new Date(oldStartTime).getTime()) - new Date(multiDif).getTime();
                    }
                    else {
                        cellstrtime = new Date(newStartTime);
                        var diffTime = newStartTime.getTime() - new Date(oldStartTime).getTime();
                    }
                    newEndTime = new Date(new Date(oldEndTime).getTime() + diffTime);
                    cellendtime = new Date(newEndTime);
                }
            }
            else if (this.model.orientation == "horizontal" && ej.scheduleFeatures.horizontal) {
                var diffTimeLeft;
                var cellstrtime = new Date(appoint[proxy.model.appointmentSettings["startTime"]]);
                var cellendtime = new Date(appoint[proxy.model.appointmentSettings["endTime"]]);
                if (this.model.enableRTL) {
					 var colIndex = (document.elementFromPoint( $(args.element[0]).offset().left,$(args.element[0]).offset().top-1)).cellIndex;
                    colIndex -= Math.floor(($(args.element[0]).width() / 30) - 1);
                }
                else
                    var colIndex = Math.floor(($(args.element[0]).offset().left + dragArea.scrollLeft() - dragArea.offset().left) / this.element.find(".e-workcells").width());
                var rowIndex = parseInt(($(args.element[0]).offset().top + dragArea.scrollTop() - dragArea.offset().top) / (this.element.find(".e-workcells").outerHeight()));
                if (this.element.find('.e-resourceheadertable .e-resourceexpand').length > 0)
                    rowIndex += $(this.element.find('.e-resourceheadertable').find('tr').filter(function () { return $(this).css('display') !== 'none'; })[rowIndex]).prevUntil("table.e-resourceheadertable").filter(function () { return $(this).css('display') == 'none'; }).length;
                var target = $(this.element.find(".e-workcellstab tr")[rowIndex]).find("td")[colIndex];
                if (!$(target).hasClass("e-resourceheadercells") && !ej.isNullOrUndefined(target)) {
                    var newStartTime = new Date(this._dateRender[0]);
                    diffTimeLeft = this.model.enableRTL ? (colIndex * 30) : (($(args.element[0]).offset().left + dragArea.scrollLeft() - dragArea.offset().left) * (30)) / $(this.element.find(".e-workcells")[1]).outerWidth();
                    newStartTime.setHours(parseInt(proxy.model.startHour) + Math.round(diffTimeLeft / 60));
                    newStartTime.setMinutes(parseInt(diffTimeLeft % 60));
                    newStartTime = (new Date(newStartTime).getMinutes() >= 30) ? new Date(new Date(newStartTime).setHours(new Date(newStartTime).getHours() - 1)) : new Date(newStartTime);
                    newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
                    if(this.model.workWeek.length != 5){
                    var index =  Math.floor(colIndex / ((this.model.endHour - this.model.startHour) * (60/30)))
                    newStartTime.setDate(new Date(this._dateRender[index]).getDate());}

                    cellstrtime = new Date(newStartTime);
                    var startTime = appoint[proxy.model.appointmentSettings["startTime"]];
                    var endTime = appoint[proxy.model.appointmentSettings["endTime"]];
                    var duration = (endTime - startTime) / 1000;
                    var hours = Math.floor(duration / 3600);
                    var minutes = Math.floor((duration % 3600) / 60);
                    newEndTime = proxy._getNewEndTime(newStartTime, hours, minutes);
                    cellendtime = new Date(newEndTime);
                    this._cellIndex = (this.model.enableRTL) ? this.element.find(".e-workcells").length - (Math.round($(args.element[0]).offset().left / this.element.find(".e-workcells").width())) : Math.floor(($(args.element[0]).offset().left + dragArea.scrollLeft() - dragArea.offset().left) / this.element.find(".e-workcells").width());
                    var cellIndex = rowIndex;
                    if ($(args.element[0]).hasClass("e-monthappointment")) {
                        var oldStartTime = new Date(appoint[proxy.model.appointmentSettings["startTime"]]);
                        var oldEndTime = new Date(appoint[proxy.model.appointmentSettings["endTime"]]);
                        if ((tempApp[0][model["recurrence"]] && tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") == -1)) {
                            oldStartTime = tempApp[0][this.model.appointmentSettings["startTime"]];
                            oldEndTime = tempApp[0][this.model.appointmentSettings["endTime"]];
                        }
                        else if ((tempApp[0][model["recurrence"]] && tempApp[0][model["recurrenceRule"]].toString().indexOf("RECUREDITID") > 0)) {
                            appoint = tempApp[0];
                            oldStartTime = tempApp[0][this.model.appointmentSettings["startTime"]];
                            oldEndTime = tempApp[0][this.model.appointmentSettings["endTime"]];
                        }
                        newStartTime = new Date(proxy.monthDays[this._cellIndex]);
                        newStartTime.setHours(oldStartTime.getHours(), oldStartTime.getMinutes(), oldStartTime.getSeconds());
                        cellstrtime = new Date(newStartTime);
                        var diffTime = newStartTime.getTime() - new Date(oldStartTime).getTime();
                        newEndTime = new Date(new Date(oldEndTime).getTime() + diffTime);
                        cellendtime = new Date(newEndTime);
                    }
                }
            }
			var reverse = true;
            if (appoint[model["recurrence"]] == 1 && appoint[model["recurrenceRule"]].indexOf("RECUREDITID") == -1) {
                var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                var _appList = this._sortAppById(new ej.DataManager(this._processed).executeLocal(query))[0];
                if (twoDayApps) var appListDetails = new ej.DataManager(this._processed).executeLocal(query);
                var Parid = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? _appList["ParentId"] : _appList[this.model.appointmentSettings["recurrenceId"]];
                var MapValue = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                var recurColl = this._appointmentRecurrenceSort(new ej.DataManager(this._processed).executeLocal(new ej.Query().where(MapValue, ej.FilterOperators.equal, Parid)));
                if (twoDayApps) {
                    for (i = 0; i < appListDetails.length; i++) {
                        this.element.find('[guid=' + appListDetails[i]["Guid"] + ']').remove();
                        new ej.DataManager(this._processed).remove("Guid", appListDetails[i]["Guid"]);
                    }
                }
                var _newAddList = {};
                $.extend(_newAddList, tempApp[0]);
                var maxdate = new Date(new Date(this.model.maxDate).getTime());
                maxdate.setHours(23, 59, 59);
                if ((cellendtime >= new Date(this.model.minDate)) && (cellendtime <= maxdate)) {
                    var index;
                    (this.model.enableRecurrenceValidation && _newAddList[model["recurrence"]]) && (index = this._recuCollectionIndex(recurColl, ej.isNullOrUndefined(appId) ? _appList : appId));
                    this._processed = this._sortAppById(this._processed);
                    var maxId = this._appMainId + 1;
                    this._appMainId = maxId;
                    if ((this.model.enableLoadOnDemand)) maxId = this._dataCount;
                    if (this.model.enableRecurrenceValidation && _newAddList[model["recurrence"]] && ((index == 0) ? (!ej.isNullOrUndefined(recurColl[index + 1]) ? !(new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime()) : false) : (index == recurColl.length - 1) ? !(new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) : !((new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime())))) {
                        _newAddList[model["startTime"]] = _appList[model["startTime"]];
                        _newAddList[model["endTime"]] = _appList[model["endTime"]];
                        $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                        this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("DragResizeError"));
                        this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                        this._alertWindow.ejDialog("open");
                        reverse = true;
                    }
                    else {
                        precellstrtime = _appList[model["startTime"]];
                        precellendtime = _appList[model["endTime"]];
                        _newAddList[model["startTime"]] = cellstrtime;
                        _newAddList[model["endTime"]] = cellendtime;
						 if (!ej.isNullOrUndefined(this.model.resources))
						{
							 oldownerId=_newAddList.ownerId;
						 if(this.model.appointmentSettings.resourceFields.split(',').length>1)
							oldroomId=_newAddList.roomId;
						}
						if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                           _newAddList = this._getDragDropResourceData(proxy, cellIndex, _newAddList);
                        if (proxy._trigger("dragStop", { appointment: _newAddList })) {
                            _newAddList[model["startTime"]] = precellstrtime;
                            _newAddList[model["endTime"]] = precellendtime;
                            if (!ej.isNullOrUndefined(this.model.resources))
							{
								_newAddList.ownerId= oldownerId;
							if(this.model.appointmentSettings.resourceFields.split(',').length>1)
								_newAddList.roomId= oldroomId;
							}
							reverse = true;
                        }
                        else {
                            reverse = false;
                            var Parid = appoint[model["recurrenceId"]] == null ? appoint["ParentId"] : appoint[model["recurrenceId"]];
                            proxy._deleteOcurrence(Parid, _appList[model["startTime"]]);
                            _newAddList[model["recurrenceRule"]] = appoint[model["recurrenceRule"]] + ";RECUREDITID=" + _newAddList["ParentId"];
                            _newAddList[model["recurrenceExDate"]] = appoint[model["recurrenceExDate"]];
                            _newAddList["AppTaskId"] = maxId;
                           if (!ej.isNullOrUndefined(allday)) _newAddList[model["allDay"]] = allday;
                        }
                    }
                    var argsAdd = { data: _newAddList };
                    var temp = [];
                    temp.push(this._timeZoneAppointments(argsAdd.data, ""));
                    var _editApp = [];
                    var Parid = _newAddList[this.model.appointmentSettings["recurrenceId"]] == null ? appoint["ParentId"] : _newAddList[this.model.appointmentSettings["recurrenceId"]];
                    var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, Parid);
                    var _App = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", _App[0]);
                    _editApp.push(this._timeZoneAppointments(_App[0], ""));
                    var promise;
                    if (this.dataSource() instanceof ej.DataManager)
                        (!reverse) && (promise = this._dataManager.saveChanges({ added: temp, changed: _editApp, deleted: [] }, this.model.appointmentSettings["id"]));
                    else
                        (!reverse) && (promise = this._dataManager.saveChanges({ added: temp, changed: _editApp, deleted: [] }, "AppTaskId"));
                    var updateTempNew = [];
                    for (var c = 0; c < temp.length; c++) {
                        updateTempNew = this._timeZoneAppointments(temp[c], "reverse");
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew);
                    }
                    var updateTempNew1 = [];
                    for (var c = 0; c < _editApp.length; c++) {
                        updateTempNew1 = this._timeZoneAppointments(_editApp[c], "reverse");
                        new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew1);
                    }
                    if (!ej.isNullOrUndefined(promise) && $.isFunction(promise.promise) && proxy.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            if (proxy.model.enableLoadOnDemand)
                                proxy._initDataSource("refresh");
                            else {
                                var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                proxy._currentAppointmentData.length = 0;
                                proxy._dateFormat(value);
                                proxy._bindAppointments = true;
                                proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                proxy._dataProcessing(value);
                                proxy._renderAppointmentAll();
                            }
                            proxy._trigger("actionComplete", { appointment: _newAddList, requestType: "appointmentDrag" });
                        });
                        promise.fail(function (e) {
                            proxy._renderAppointmentAll();
                        });
                    }
                    else {
                        !twoDayApps && proxy._processed.splice(proxy._processed.indexOf(_appList), 1);
                        twoDayApps ? this._appointmentProcessing(argsAdd.data) : this._processed.push(argsAdd.data);
                        this._renderAppointmentAll();
                        proxy._trigger("actionComplete", { appointment: _newAddList, requestType: "appointmentDrag" });
                    }
                }
                else this._renderAppointmentAll();

            }
            else {
                var _prevObj = {};
                var prvcellstrtime = tempApp[0][proxy.model.appointmentSettings["startTime"]];
                var prvcellendtime = tempApp[tempApp.length - 1][proxy.model.appointmentSettings["endTime"]];
                if (!ej.isNullOrUndefined(this.model.resources))
				{	
				   var oldownerId=tempApp[0].ownerId;				
					if(this.model.appointmentSettings.resourceFields.split(',').length>1)
					var oldroomId=tempApp[0].roomId;
				}
				if (twoDayApps)
                    var object = new ej.DataManager(this._currentAppointmentData).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, tempApp[0]["Guid"]));
                else {
                    var object = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, tempApp[0]["Guid"]));
                    var _prevObj = $.extend(true, [], object[0]);
                }
                var recurColl = this._appointmentRecurrenceSort(new ej.DataManager(this._processed).executeLocal(new ej.Query().where("ParentId", ej.FilterOperators.equal, object[0].ParentId)));
                var maxdate = new Date(new Date(this.model.maxDate).getTime());
                maxdate.setHours(23, 59, 59);
                if ((cellstrtime >= new Date(this.model.minDate)) && (cellendtime <= maxdate)) {
                    var index;
                    object[0][proxy.model.appointmentSettings["startTime"]] = cellstrtime;
                    object[0][proxy.model.appointmentSettings["endTime"]] = cellendtime;
                    (this.model.enableRecurrenceValidation) && (index = this._recuCollectionIndex(recurColl, object[0]));
                    if (this.model.enableRecurrenceValidation && object[0][model["recurrence"]] && ((index == 0) ? (!ej.isNullOrUndefined(recurColl[index + 1]) ? !(new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime()) : false) : (index == recurColl.length - 1) ? !(new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) : !((new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(newStartTime).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(newEndTime).setHours(0, 0, 0, 0)).getTime())))) {
                        object[0][model["startTime"]] = prvcellstrtime;
                        object[0][model["endTime"]] = prvcellendtime;
                        $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                        this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("DragResizeError"));
                        this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                        this._alertWindow.ejDialog("open");
                        reverse = true;
                    }
                    else {
						 if (!ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                                object[0] = this._getDragDropResourceData(proxy, cellIndex, object[0]);
                        if (proxy._trigger("dragStop", { appointment: object[0] })) {
                            object[0][proxy.model.appointmentSettings["startTime"]] = prvcellstrtime;
                            object[0][proxy.model.appointmentSettings["endTime"]] = prvcellendtime;
                            if (!ej.isNullOrUndefined(this.model.resources))
							{
								 object[0].ownerId= oldownerId;
							  if(this.model.appointmentSettings.resourceFields.split(',').length>1)
								object[0].roomId= oldroomId;
							
							}
							reverse = true;
                        }
                        else {
                            reverse = false;
                           	if (this.model.orientation == "horizontal" && object[0][this.model.appointmentSettings["allDay"]] && ej.isNullOrUndefined(cellIndex))
								object[0].ownerId = this.res1[rowIndex].id;
                        }
                        if (!ej.isNullOrUndefined(allday)) object[0][proxy.model.appointmentSettings["allDay"]] = allday;
                    }
                }
                if (twoDayApps) var _prevObj = $.extend(true, [], object[0]);
                var bg = (this.model.orientation == "vertical" && (new Date(new Date(prvcellstrtime).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(prvcellendtime).setHours(0, 0, 0, 0)).getTime()) && (((prvcellendtime - prvcellstrtime) / 3600000) < 24) && (object[0][this.model.appointmentSettings["allDay"]] !== true));
                var diffDays = Math.ceil(((object[0]["EndTime"] - object[0]["StartTime"]) / 3600000) / 24);
                var multiDayApp = (diffDays > 1) ? true : false;
                
                this.element.find('[guid=' + this._appUid + ']').remove();
                if (this.model.orientation == "horizontal" && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources)
                    proxy.element.find("#categorybar_" + appoint["Guid"]).remove();
                var temp = this._timeZoneAppointments(object[0], "");
                var promise;
                if (this.dataSource() instanceof ej.DataManager) {
                    (!reverse) && (promise = proxy._dataManager.update(this.model.appointmentSettings["id"], temp));
                }
                else
                    (!reverse) && (promise = proxy._dataManager.update("AppTaskId", temp));

                var updateTemp = this._timeZoneAppointments(temp, "reverse");
                if (twoDayApps)
                    new ej.DataManager(this._currentAppointmentData).update("Guid", updateTemp);         
                else
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTemp);
                
                if (!ej.isNullOrUndefined(promise) && promise && $.isFunction(promise.promise) && proxy.dataSource() instanceof ej.DataManager) {
                    promise.done(function (e) {
                        if (proxy.model.enableLoadOnDemand)
                            proxy._initDataSource("refresh");
                        else {
                            var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                            proxy._currentAppointmentData.length = 0;
                            proxy._dateFormat(value);
                            proxy._bindAppointments = true;
                            proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                            proxy._dataProcessing(value);
                            proxy._renderAppointmentAll();
                        }
                        proxy._trigger("actionComplete", { appointment: object, requestType: "appointmentDrag" });
                    });
                    promise.fail(function (e) {
                        proxy._renderAppointmentAll();
                    });
                }
                else {
                    if (object[0][this.model.appointmentSettings["recurrence"]])
                        new ej.DataManager(this._processed).update("AppTaskId", object[0]);
                    var newObj = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("ParentId", ej.FilterOperators.equal, object[0].ParentId));
                    var appLen = newObj.length == 0 ? false : true;
                    if (twoDayApps) {
                        var objValue = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, object[0]["Guid"]));
                        for (var i = 0; i < objValue.length; i++) {
                            new ej.DataManager(this._processed).remove("Guid", objValue[i]["Guid"]);
                            this.element.find('[guid=' + objValue[i]["Guid"] + ']').remove();
                        }
                    }
                    var isBigapp = ((new Date(object[0][this.model.appointmentSettings["endTime"]]) - new Date(object[0][this.model.appointmentSettings["startTime"]])) / 3600000) > 24;
                    if (this.model.showTimeScale && this.model.orientation == "vertical" && this.currentView() !== "month" && !(isBigapp) && !(this._isCustomView()) && object[0][model["allDay"]] == 1) {
                        (this.model.showAllDayRow) && this._renderAppointmentAll();
                    }
                    else {
                        if ((isBigapp) || (this.currentView() == "month" || this._isCustomView())) {
                            this._dragDrop = false;
                            if (this.model.showTimeScale)
                                this._renderSingleApp(object[0], object[0]);
                            else {
                                if (multiDayApp) {
                                    this._appointmentProcessing(object[0]);
                                    this._renderSingleApp(object[0], object[0]);
                                }
                                else if (object[0][this.model.appointmentSettings["recurrence"]])
                                    this._renderAppointmentAll();
                                else this._renderSingleApp(object[0], object[0]);
                                (this.model.orientation == "vertical") && this._unWireResizeEvents();
                                (this.model.orientation == "vertical") && this._wireResizeEvents();
                                (this.model.orientation == "vertical") && this._dragdropAppointments();
                            }
                            (this.model.showTimeScale && (this.currentView() == "month" || this._isCustomView()) && !twoDayApps) && this._monthOverlap(_prevObj);
                        }
                        else {
                            this._dragDrop = false;
                            twoDayApps && this._appointmentProcessing(object[0]);
                            this._renderSingleApp(object[0], object[0]);
                            (!twoDayApps && !isBigapp && this.model.showTimeScale && !this.alldayDrag) && this._reRenderApp(_prevObj);
							(splitDrag) && this._reRenderApp(_prevObj);
                            if (this.alldayDrag) {
                                this._renderAllDayAppointments();
                                this._unWireResizeEvents();
                                this._wireResizeEvents();
                                this._dragdropAppointments();
                                this.alldayDrag = false;
                            }
                            if (this.model.orientation == "vertical" && !this.model.showTimeScale) {
                                (!splitDrag) && this._monthOverlap(_prevObj);
                                this._unWireResizeEvents();
                                this._wireResizeEvents();
                                this._dragdropAppointments();
                            }
                        }
                    }
                    this._trigger("actionComplete", { appointment: object, requestType: "appointmentDrag" });
                    this._appMainId = this._sortAppById(this._processed)[this._processed.length - 1]["AppTaskId"];
                }
            }
            (this.element.find('#cloneElement').length > 0) && this.element.find('#cloneElement').remove();
        },

        _monthOverlap: function (appoint) {
            var strDate = appoint[this.model.appointmentSettings["startTime"]];
            if (this.model.orientation == "horizontal" && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) {
                var timeScaleRowCount = (60 / 30);
                var startTime = ((new Date(strDate).getHours()) < this.model.startHour) ? (new Date(strDate).getHours() + (-new Date(strDate).getHours()) + (-new Date(strDate).getMinutes())) : (new Date(strDate).getHours() + (-this.model.startHour));
                var groupIndex = (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) ? this._findResourceIndex(this.model.resources[0].resourceSettings.dataSource, this.model.resources[0].resourceSettings.id, appoint[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]) + 1 : 0;
            }
            var index = this.currentView() == "day" ? 0 : (this.model.orientation == "horizontal" && this.currentView() == "month") ? $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this.monthDays) : $.inArray(new Date(strDate.getFullYear(), strDate.getMonth(), strDate.getDate()).getTime(), this._dateRender);
            var resIndex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) ? this._findResourceIndex(this.res1, this.model.resources[this.model.resources.length - 1].resourceSettings["id"], appoint[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1]]) : 0;
            var groupIndex = (!ej.isNullOrUndefined(this.model.group) && this.model.group.resources.length > 1) ? this._findResourceIndex(this.model.resources[0].resourceSettings.dataSource, this.model.resources[0].resourceSettings.id, appoint[this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 2]]) + 1 : 0;
            var predicts = this.model.orientation == "vertical" ? this._dateRender[index] : this.monthDays[index];
            var predicate = ej.Predicate(this.model.appointmentSettings["startTime"], ej.FilterOperators.greaterThanOrEqual, new Date(predicts));
            predicate = predicate["and"](this.model.appointmentSettings["startTime"], ej.FilterOperators.lessThanOrEqual, new Date(new Date(predicts).setHours(23, 59, 59)));
            predicate = !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources ? predicate["and"](this.model.appointmentSettings.resourceFields.toString().split(',')[this.model.resources.length - 1], ej.FilterOperators.equal, this.res1[resIndex][this.model.resources[this.model.resources.length - 1].resourceSettings["id"]]) : predicate;
            var app = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(predicate));
            var cellId = this.model.orientation == "vertical" ? (resIndex * (7) + (new Date(strDate).getDay() + 1)) + (((Math.ceil((index + 1) / 7)) - 1) * (this.res1.length * 7)) : index + 1;
            this.model.orientation == "vertical" ? this.element.find("#" + this._id + "_" + Globalize.format(new Date(appoint[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resIndex + groupIndex) + "_" + (cellId - 1)).remove() : this.element.find("#" + this._id + "_" + Globalize.format(new Date(appoint[this.model.appointmentSettings["startTime"]]), "MM_dd_yyyy") + "_" + (resIndex + groupIndex) + "_" + (cellId - 1)).remove();
            for (var i = 0; i < app.length; i++) {
                this.element.find('[guid=' + app[i]["Guid"] + ']').remove();
                this.element.find("#categorybar_" + app[i]["Guid"]).remove();
                this.model.showTimeScale ? this.currentView() == "month" || (this._isCustomView()) ? this._renderAppointmentMonth(app[i], index, resIndex) : "" : this._renderAppointmentMonth(app[i], index, resIndex);
            }
            this._unWireResizeEvents();
            this._wireResizeEvents();
            this._dragdropAppointments();
        },

        _unWireDragDrop: function () {
            var dragableElements = this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment");
            dragableElements.ejDraggable("destroy");
        },

        _getAppointmentByParentId: function (appId) {
            var tempApp, appoint, appList = this._currentAppointmentData;
            var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, appId));
            var appoint = new ej.DataManager(appList).executeLocal(new ej.Query().where("AppTaskId", ej.FilterOperators.equal, parseInt(tempApp[0]["ParentId"])));
            return !$.isEmptyObject(appoint) ? appoint[0] : appoint;
        },

        _renderAppointmentAll: function () {
            if (this.currentView() == "agenda")
                this._renderAgendaContent();
            else {
                Globalize.culture(this.model.locale);
                this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,.e-appsoverflow").remove();
                this.element.find("div.e-categorybar").remove();
                if (this.currentView() != "month" && (!this._isCustomView()) && this.model.showTimeScale) {
                    this._renderAppointments();
                    if (this.model.enableAppointmentNavigation)
                        this._nextPrevButton(this._filtered);
                }
                else
                    this._renderMonthAppointment();
                this._wireResizeEvents();
                this._dragdropAppointments();
            }
        },

        _getNewEndTime: function (newStartTime, hours, minutes) {
            var newEndTime = new Date(newStartTime);
            newEndTime.setHours(newEndTime.getHours() + hours);
            newEndTime.setMinutes(newEndTime.getMinutes() + minutes);
            return newEndTime;
        },

        _getNewStartTime: function (newTopTime, newStartTime) {
            newStartTime.setHours((newStartTime.getHours()) + parseInt(this.model.startHour) + parseInt(newTopTime / 60));
            newStartTime.setMinutes((newStartTime.getMinutes()) + parseInt(newTopTime % 60));
            newStartTime.setMinutes(Math.round(newStartTime.getMinutes() / 10.0) * 10);
            return newStartTime;
        },

        _createTemplate: function (elt, templateId) {
            var scriptElt = document.createElement("script");
            scriptElt.id = (this._id + templateId + "_Template");
            scriptElt.type = "text/x-jsrender";
            scriptElt.text = elt;
            $("body").append(scriptElt);
            return scriptElt;
        },

        _documentOnKeyDown: function (event) {
            if (this.model.allowKeyboardNavigation) {
                var oEl = event.target || document.activeElement || event.srcElement;
                if (event.which == 27) {
                    this._processEscapeKey();
                }
                else if (event.which == 13) {
                    this._processEnterKey(oEl);
                }
                else if (event.shiftKey == false && event.which == 9) {
                    this._processWindowTab(oEl);
                }
            }
        },

        _processWindowTab: function (oEl) {
            if ($(oEl).hasClass("qAppDone"))
                this._quickAppointWindow.find(".subject").focus();
            else if ($(oEl).hasClass("e-appointcancel") || $(oEl).hasClass("e-cancelrecur"))
                this._appointmentAddWindow.find(".subject").focus();
            else if ($(oEl).hasClass("e-editseries"))
                this.model.showQuickWindow && this._quickAppDetailsWindow.focus();
        },

        _onKeyDown: function (event) {
            if (this.model.allowKeyboardNavigation) {
                this.model.showQuickWindow && (this._quickAppointWindow.ejDialog("close") && this._quickAppDetailsWindow.ejDialog("close"));
                var oRowCell = this.element.find('div.e-draggableworkarea tr');
                var oEl = event.target || document.activeElement || event.srcElement;
                $(oEl).hasClass("e-alldaycells") && (oRowCell = this.element.find(".e-alldaycells").parent());
                var index = $(oEl).hasClass("e-workcells") ? $(oEl).index() : $(oEl).hasClass("e-alldaycells") ? $(oEl).index() : 7 - ((parseInt($(oEl).index() / 7) + 1) * 7 - $(oEl).index()) + ($(oEl).parent().index() * 7);
                if (this.model.orientation == "horizontal")
                    index = this.currentView() !== "month" ? Math.floor(index / ((this.model.endHour - this.model.startHour) * 2)) : index;
                var renderDate = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : this._dateRender;
                (this.currentView() == "agenda") && (index = 0);
                var curDate = new Date(renderDate[index]);
                (event.which == 39) && curDate.setDate(curDate.getDate() + 1);
                (event.which == 37) && curDate.setDate(curDate.getDate() - 1);
                if (($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-alldaycells") || $(oEl).hasClass("e-monthcells")) && (curDate >= this.model.minDate) && (curDate <= this.model.maxDate)) {
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                    if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-alldaycells") || $(oEl).hasClass("e-monthcells"))
                        $(oEl).addClass("e-selectedCell");
                    if (event.shiftKey == false && event.which == 38) { //up arrow navigation
                        this._stopDefaultAction(event);
                        this._processUpArrow(oEl, oRowCell);
                    }
                    else if (event.shiftKey == false && event.which == 40) { //down arrow for cell navigation
                        this._stopDefaultAction(event);
                        this._processDown(oEl, oRowCell);
                    }
                    else if (event.ctrlKey == false && event.shiftKey == false && event.which == 39) { //right arrow cell navigation
                        this._stopDefaultAction(event);
                        this._processRight(oEl, oRowCell);
                    }
                    else if (event.ctrlKey == false && event.shiftKey == false && event.which == 37) { //left arrow cell navigation
                        this._stopDefaultAction(event);
                        this._processLeft(oEl, oRowCell);
                    }
                    else if (event.shiftKey == true && event.which == 38) { //shift + up arrow
                        this._stopDefaultAction(event);
                        this._processShiftUp(oEl, event, oRowCell);
                    }
                    else if (event.shiftKey == true && event.which == 40) { //shift + down arrow
                        this._stopDefaultAction(event);
                        this._processShiftDown(oEl, event, oRowCell);
                    }
                    else if (event.shiftKey == true && event.which == 37) { //shift+left
                        this._stopDefaultAction(event);
                        this._processShiftLeft(oEl, event, oRowCell);
                    }
                    else if (event.shiftKey == true && event.which == 39) { //shift+right
                        this._stopDefaultAction(event);
                        this._processShiftRight(oEl, event, oRowCell);
                    }
                }
                else if (this.currentView() == "agenda") {
                    this._stopDefaultAction(event);
                    if (event.which == 38 || event.which == 40) {
                        this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                        var rowcount = oRowCell.length - 1;
                        var currrowindex = this.element.find(".e-selectedCell").parent().index();
                        var newcur = (event.which == 38) ? parseInt(currrowindex) - 1 : parseInt(currrowindex) + 1;
                        if (newcur >= 0 && newcur <= rowcount && currrowindex != "undefined") {
                            $(oRowCell).find(".e-eventcolumn,.e-timecolumn").removeClass("e-selectedCell");
                            $(oRowCell[newcur]).find(".e-eventcolumn,.e-timecolumn").addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                }
                if (event.altKey == true && event.which == 67) { //Alt+C To show/hide the calendar pane
                    this._stopDefaultAction(event);
                    this.element.find(".e-datedisplay").click();
                }
                else if (event.ctrlKey == true && event.which == 37) { //Previous button
                    this._stopDefaultAction(event);
                    this.element.find("div.e-navigateprevious").click();
                    this.element.find('div.e-navigateprevious').attr("tabIndex", 0).focus();
                    this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                }
                else if (event.ctrlKey == true && event.which == 39) { //Next button
                    this._stopDefaultAction(event);
                    this.element.find("div.e-navigatenext").click();
                    this.element.find('div.e-navigatenext').attr("tabIndex", 0).focus();
                    this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                }
                else if ((event.altKey == true && event.which == 107) || (event.altKey == true && event.which == 187) || (event.altKey == true && event.which == 61)) { //Alt + "+" to navigate through the toolbar items
                    this._stopDefaultAction(event);
                    this._navigateToolbarRight(event);
                }
                else if ((event.altKey == true && event.which == 189) || (event.altKey == true && event.which == 109) || (event.altKey == true && event.which == 45) || (event.altKey == true && event.which == 173)) { // Alt + "-" to navigate back through the toolbar items
                    this._stopDefaultAction(event);
                    this._navigateToolbarLeft(event);
                }
                else if (event.ctrlKey == true && event.which == 67) { //Cancel Ctrl+C
                    this._stopDefaultAction(event);
                    this._cancel();
                }
                else if (event.which == 32) { //spacebar
                    if ($(document.activeElement)[0] === this.element.find("div.e-navigateprevious")[0]) {
                        this._stopDefaultAction(event);
                        this.element.find("div.e-navigateprevious").click();
                        this.element.find('div.e-navigateprevious').attr("tabIndex", 0).focus();
                        this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                    }
                    else if ($(document.activeElement)[0] === this.element.find("div.e-navigatenext")[0]) {
                        this._stopDefaultAction(event);
                        this.element.find("div.e-navigatenext").click();
                        this.element.find('div.e-navigatenext').attr("tabIndex", 0).focus();
                        this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
                    }
                }
                else if (event.altKey == true && event.which == 78) { //New appointment window
                    this._stopDefaultAction(event);
                    this.currentView() != "agenda" && this._newAppWindow(oEl, oRowCell);
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                }
                else if (event.ctrlKey == true && event.which == 69) { //(E)Edit appointment window
                    this._stopDefaultAction(event);
                    var elmt = this.element.find(".e-selectedAppointment");
                    if (this.currentView() == "agenda" && this.element.find('.e-selectedCell').length > 0) {
                        selCell = this.element.find(".e-selectedCell").parent().find(".e-agendaappointment");
                        selCell.hasClass("e-agendaappointment") && (elmt = selCell);
                    }
                    if (elmt.length > 0) {
                        this._editKey = true;
                        this._cellClick = false;
                        this._appointmentWindow(elmt);
                        this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                    }
                }
                else if (event.which == 46) { //Delete
                    this._stopDefaultAction(event);
                    var delap = this.element.find('.e-selectedAppointment');
                    if (this.currentView() == "agenda" && this.element.find('.e-selectedCell').length > 0) {
                        var selCell = this.element.find(".e-selectedCell").parent().find(".e-agendaappointment");
                        selCell.hasClass("e-agendaappointment") && (delap = selCell);
                    }
                    if (delap.length > 0) {
                        this._appUid = delap.attr('guid');
                        this._deleteAppoint();
                    }
                }
                else if (event.shiftKey == true && event.which == 9) { //Shift + tab for reverse navigation of appointments
                    this._stopDefaultAction(event);
                    this._reverseTab(oEl, event, oRowCell);
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                }
                else if (event.shiftKey == false && event.which == 9) { // For tab navigation of appointments
                    this._stopDefaultAction(event);
                    this._processTab(oEl, event, oRowCell);
                    this.element.find("#" + this._id + "_navigator").ejDatePicker("hide");
                }
                var proxy = this;
                this.currentView() != "agenda" && this._moveScrollContent(oEl, proxy);
            }
        },

        _processShiftUp: function (oEl, event, oRowCell) {
            if (this.model.orientation == "horizontal") return false;
            if ($(oEl).hasClass('e-workcells')) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    var newcur = parseInt(currrowindex) - 1;
                    if (newcur >= 0) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            $(oRowCell[newcur].cells[currcolindex]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                        else {
                            $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell");
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                    this._currentRowIndex = newcur;
                    this._currentColIndex = currcolindex;
                }
                else {
                    $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                    this._currentRowIndex = currrowindex;
                    this._currentColIndex = currcolindex;
                }

            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var resourceIndex = Math.floor($(oEl).index() / 7);
                var rowcount = oRowCell.length - 1, resmaxcolindex = 7 * (resourceIndex + 1), mincolindex = 7 * (parseInt(resourceIndex));
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    var newcur = parseInt(currrowindex) - 1;
                    if (newcur >= 0) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            for (var i = currcolindex; i >= mincolindex; i--) {
                                $(oRowCell[currrowindex].cells[i]).attr("tabIndex", 0).focus();
                                $(oRowCell[currrowindex].cells[i]).removeClass("e-selectedCell");
                            }
                            for (var j = resmaxcolindex - 1; j >= currcolindex; j--) {
                                if (newcur >= 0) {
                                    $(oRowCell[newcur].cells[j]).attr("tabIndex", 0).focus();
                                    $(oRowCell[newcur].cells[j]).removeClass("e-selectedCell");
                                }
                            }
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell");
                        }
                        else {
                            for (var i = currcolindex; i >= mincolindex; i--)
                                $(oRowCell[currrowindex].cells[i]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            for (var j = resmaxcolindex - 1; j >= currcolindex; j--) {
                                if (newcur >= 0)
                                    $(oRowCell[newcur].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                        }
                    }
                    else
                        $(oRowCell[newcur + 1].cells[currcolindex]).attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else
                return false;

        },

        _processShiftDown: function (oEl, event, oRowCell) {
            if (this.model.orientation == "horizontal") return false;
            if ($(oEl).hasClass('e-workcells')) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    var newcur = parseInt(currrowindex) + 1;
                    if (newcur <= rowcount) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            $(oRowCell[newcur].cells[currcolindex]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                        else {
                            $(oRowCell[currrowindex].cells[currcolindex]).addClass("e-selectedCell");
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                        this._currentRowIndex = newcur;
                        this._currentColIndex = currcolindex;
                    }
                    else {
                        $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                        this._currentRowIndex = currrowindex;
                        this._currentColIndex = currcolindex;
                    }

                }
            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var resourceIndex = Math.floor($(oEl).index() / 7);
                var rowcount = oRowCell.length - 1, resmaxcolindex = 7 * (resourceIndex + 1), mincolindex = 7 * (parseInt(resourceIndex));
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    var newcur = parseInt(currrowindex) + 1;
                    if (newcur < (this._dateRender.length) / 7) {
                        if ($(oRowCell[newcur].cells[currcolindex]).hasClass("e-selectedCell")) {
                            for (var i = currcolindex; i <= resmaxcolindex - 1; i++) {
                                $(oRowCell[currrowindex].cells[i]).attr("tabIndex", 0).focus();
                                $(oRowCell[currrowindex].cells[i]).removeClass("e-selectedCell");
                            }
                            for (var j = mincolindex; j <= currcolindex; j++) {
                                if (newcur < (this._dateRender.length) / 7) {
                                    $(oRowCell[newcur].cells[j]).attr("tabIndex", 0).focus();
                                    $(oRowCell[newcur].cells[j]).removeClass("e-selectedCell");
                                }
                            }
                            $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell");
                        }
                        else {
                            for (var i = currcolindex; i <= resmaxcolindex - 1; i++)
                                $(oRowCell[currrowindex].cells[i]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            for (var j = mincolindex; j <= currcolindex; j++) {
                                if (newcur < (this._dateRender.length) / 7)
                                    $(oRowCell[newcur].cells[j]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                        }
                    }
                    else
                        $(oRowCell[newcur - 1].cells[currcolindex]).attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else
                return false;
        },

        _processShiftLeft: function (oEl, event, oRowCell) {
            var rowcount = oRowCell.length - 1;
            if ($(oEl).hasClass('e-workcells')) {
                var targetElement = oEl;
                var count = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? this.model.workWeek.length : 7;
                var viewCount = this.model.orientation == "vertical" ? count : this.currentView() == "month" ? this.totdays.length : count * (this.model.endHour - this.model.startHour) * 2;
                var resIndex = Math.floor($(targetElement).index() / viewCount);
                var resmaxcolindex = (viewCount * resIndex);
                var currColInx = $(targetElement)[0].cellIndex;
                var currRowInx = $(targetElement).parent()[0].rowIndex;
                var leftColInx = currColInx - 1;
                var irowcount = this.model.orientation == "vertical" ? 0 : currRowInx;
                var Hrowcount = this.model.orientation == "vertical" ? rowcount : currRowInx;
                var flag = this.model.orientation == "vertical" && this.model.showAllDayRow ? true : false;
                var ad = this.element.find(".e-alldaycells").parent();
                if (currColInx > 0 && leftColInx > parseInt(resmaxcolindex) - 1 && currColInx != "undefined") {
                    if ($(oRowCell[currRowInx].cells[leftColInx]).hasClass("e-selectedCell")) {
                        for (var i = irowcount; i <= parseInt(currRowInx) ; i++)
                            $(oRowCell[i].cells[currColInx]).removeClass("e-selectedCell");
                        for (var j = Hrowcount; j >= parseInt(currRowInx) ; j--)
                            $(oRowCell[j].cells[leftColInx]).removeClass("e-selectedCell");
                        flag && $(ad[0].cells[currColInx]).removeClass("e-selectedCell").attr("tabIndex", 0).focus();
                        $(oRowCell[currRowInx].cells[leftColInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else {
                        for (var i = irowcount; i <= parseInt(currRowInx) ; i++)
                            $(oRowCell[i].cells[currColInx]).addClass("e-selectedCell");
                        for (var j = Hrowcount; j >= parseInt(currRowInx) ; j--)
                            $(oRowCell[j].cells[leftColInx]).addClass("e-selectedCell");
                        if (flag) {
                            $(ad[0].cells[currColInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            $(ad[0].cells[leftColInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                        $(oRowCell[currRowInx].cells[leftColInx]).attr("tabIndex", 0).focus();
                    }
                    this._currentRowIndex = currRowInx;
                    this._currentColIndex = leftColInx;
                }
                else {
                    $(oRowCell[currRowInx].cells[leftColInx + 1]).attr("tabIndex", 0).focus();
                    this._currentRowIndex = currRowInx;
                    this._currentColIndex = leftColInx + 1;
                }
            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var resIndex = Math.floor($(oEl).index() / 7);
                var rowcount = oRowCell.length - 1, mcolind = 7 * (parseInt(resIndex) + 1);
                var mincolindex = 7 * (parseInt(resIndex));
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                if (currcolindex >= mincolindex && currcolindex <= mcolind && currcolindex != "undefined") {
                    newcur = parseInt(currcolindex) - 1;
                    if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                        if (currcolindex == mincolindex && currrowindex >= 1) {
                            var newrow = parseInt(currrowindex) - 1;
                            var newcol = parseInt(mcolind) - 1;
                            if (newrow >= 0) {
                                $(oRowCell[newrow].cells[newcol]).attr("tabIndex", 0).focus();
                                $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell");
                                $(oRowCell[currrowindex].cells[mincolindex]).removeClass("e-selectedClass");
                            }
                        }
                        else if (currcolindex < mcolind) {
                            newcur = parseInt(currcolindex) - 1;
                            $(oRowCell[currrowindex].cells[newcur]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                    }
                    else {
                        if (currcolindex == mincolindex && currrowindex >= 0) {
                            var newrow = parseInt(currrowindex) - 1;
                            var newcol = parseInt(mcolind) - 1;
                            if (newrow >= 0)
                                $(oRowCell[newrow].cells[newcol]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            else
                                $(oRowCell[newrow + 1].cells[currcolindex]).attr("tabIndex", 0).focus();
                        }
                        else if (currcolindex < mcolind) {
                            newcur = parseInt(currcolindex) - 1;
                            $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                }
                else
                    return;
            }
            else if ($(oEl).hasClass('e-alldaycells')) {
                var count = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? this.model.workWeek.length : 7;
                var resIndex = Math.floor($(oEl).index() / count);
                var resmaxcolindex = (count * resIndex);
                var currColInx = $(oEl)[0].cellIndex;
                var leftColInx = currColInx - 1;
                if (currColInx > 0 && leftColInx > parseInt(resmaxcolindex) - 1 && currColInx != "undefined") {
                    if ($(oRowCell[0].cells[leftColInx]).hasClass("e-selectedCell")) {
                        $(oRowCell[0].cells[currColInx]).removeClass("e-selectedCell");
                        $(oRowCell[0].cells[leftColInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[0].cells[leftColInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    this._currentRowIndex = 0;
                    this._currentColIndex = leftColInx;
                }
                else {
                    $(oRowCell[0].cells[leftColInx + 1]).attr("tabIndex", 0).focus();
                    this._currentRowIndex = 0;
                    this._currentColIndex = leftColInx + 1;
                }
            }
            else
                return false;
        },

        _processShiftRight: function (oEl, event, oRowCell) {
            if ($(oEl).hasClass('e-workcells')) {
                var rowcount = oRowCell.length - 1;
                var count = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? this.model.workWeek.length : 7;
                var viewCount = this.model.orientation == "vertical" ? count : this.currentView() == "month" ? this.totdays.length : count * (this.model.endHour - this.model.startHour) * 2;
                var resIndex = Math.floor($(oEl).index() / viewCount);
                var resmaxcolindex = (viewCount * (resIndex + 1));
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                var irowcount = this.model.orientation == "vertical" ? 0 : currrowindex;
                var Hrowcount = this.model.orientation == "vertical" ? rowcount : currrowindex;
                var ad = this.element.find(".e-alldaycells").parent();
                var flag = this.model.orientation == "vertical" && this.model.showAllDayRow ? true : false;
                if (currcolindex >= 0 && currcolindex < parseInt(resmaxcolindex) - 1 && currcolindex != "undefined") {
                    if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                        for (var i = parseInt(currrowindex) ; i <= Hrowcount; i++)
                            $(oRowCell[i].cells[currcolindex]).removeClass("e-selectedCell");
                        for (var j = irowcount; j <= currrowindex; j++)
                            $(oRowCell[j].cells[newcur]).removeClass("e-selectedCell").attr("tabIndex", 0).focus();
                        flag && $(ad[0].cells[currcolindex]).removeClass("e-selectedCell");
                        $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell");
                    }
                    else {
                        for (var i = parseInt(currrowindex) ; i <= Hrowcount; i++)
                            $(oRowCell[i].cells[currcolindex]).addClass("e-selectedCell");
                        for (var j = irowcount; j <= currrowindex; j++)
                            $(oRowCell[j].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        if (flag) {
                            $(ad[0].cells[newcur]).addClass("e-selectedCell");
                            $(ad[0].cells[currcolindex]).addClass("e-selectedCell");
                        }
                    }
                    this._currentRowIndex = currrowindex;
                    this._currentColIndex = newcur;
                }
                else
                    $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass('e-monthcells')) {
                var resIndex = Math.floor($(oEl).index() / 7);
                var rowcount = oRowCell.length - 1, mcolind = resmaxcolindex = 7 * (parseInt(resIndex) + 1), mincolindex = 7 * (parseInt(resIndex));;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (currcolindex >= 0 && currcolindex <= parseInt(resmaxcolindex) - 1 && currcolindex != "undefined") {
                    if ($(oRowCell[currrowindex].cells[newcur]).hasClass("e-selectedCell")) {
                        if (newcur < mcolind) {
                            $(oRowCell[currrowindex].cells[newcur]).attr("tabIndex", 0).focus();
                            $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell");
                        }
                        if (currcolindex == parseInt(resmaxcolindex) - 1 && currrowindex < (this._dateRender.length) / 7) {
                            var newrow = parseInt(currrowindex) + 1;
                            currcolindex = mincolindex;
                            if (newrow < (this._dateRender.length) / 7) {
                                $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell").attr("tabIndex", 0).focus();
                                if (newcur < mcolind) {
                                    $(oRowCell[newrow].cells[newcur]).attr("tabIndex", 0).focus();
                                    $(oRowCell[newrow].cells[currcolindex]).removeClass("e-selectedCell");
                                }
                            }
                        }
                    }
                    else {
                        if (newcur < mcolind)
                            $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        if (currcolindex == parseInt(resmaxcolindex) - 1 && currrowindex < (this._dateRender.length) / 7) {
                            var newrow = parseInt(currrowindex) + 1;
                            currcolindex = mincolindex;
                            if (newrow < (this._dateRender.length) / 7) {
                                $(oRowCell[newrow].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                                if (newcur < mcolind)
                                    $(oRowCell[newrow].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                            }
                            else
                                $(oRowCell[newrow - 1].cells[newcur - 1]).attr("tabIndex", 0).focus();
                        }
                    }
                }
                else
                    return false;
            }
            else if ($(oEl).hasClass('e-alldaycells')) {
                var count = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? this.model.workWeek.length : 7;
                var resIndex = Math.floor($(oEl).index() / count);
                var resmaxcolindex = (count * (resIndex + 1));
                var currcolindex = $(oEl)[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (currcolindex >= 0 && currcolindex < parseInt(resmaxcolindex) - 1 && currcolindex != "undefined") {
                    if ($(oRowCell[0].cells[newcur]).hasClass("e-selectedCell")) {
                        $(oRowCell[0].cells[currcolindex]).removeClass("e-selectedCell");
                        $(oRowCell[0].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[0].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    this._currentRowIndex = 0;
                    this._currentColIndex = newcur;
                }
                else
                    $(oRowCell[0].cells[currcolindex]).attr("tabIndex", 0).focus();
            }
            else
                return false;
        },

        _processEnterKey: function (oEl) {
            if ($(oEl).hasClass("e-workcells") || $(oEl).hasClass("e-monthcells") || $(oEl).hasClass("e-alldaycells")) {
                var tds = this.element.find('td.e-selectedCell');
                var nt = tds.length - 1;
                if (nt == 0) {
                    this._cellSelection(oEl);
                }
                else {
                    this._multipleCellAppointCreation(oEl);
                }
            }
            else if ($(oEl).hasClass("e-detailedapp e-detailedwindow")) {
                this._appointmentWindow(oEl);
            }
            else if ($(oEl).parent().hasClass("e-allday")) {
                var e = this._appointmentAddWindow.find("#" + this._id + "allday").ejCheckBox("option", "checked");
                var checkedVal = e ? false : true;
                this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: checkedVal });
            }
            else if ($(oEl).parent().hasClass("e-recurrence")) {
                var e = this._appointmentAddWindow.find("#" + this._id + "recurrence").ejCheckBox("option", "checked");
                var checkedVal = e ? false : true;
                this._appointmentAddWindow.find(".recurrence").ejCheckBox({ checked: checkedVal });
                this._recurChecked = checkedVal ? true : false;
                this._repeat(oEl);
            }
            else if ($(oEl).hasClass("e-scheduledelete")) {
                this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-scheduledelete').click();
            }
            else if ($(oEl).hasClass("e-editevent"))
                this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-editevent').click();
            else if ($(oEl).hasClass(".e-editseries"))
                this.model.showQuickWindow && this._quickAppDetailsWindow.find('.e-editseries').click();
            else if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment"))
                this._cellSelection(oEl);
            else if ($(oEl).parent().hasClass("e-weekday")) {
                var e = this._appointmentAddWindow.find("#" + $(oEl).children()[0].id).ejCheckBox("option", "checked");
                var checkedVal = e ? false : true;
                this._appointmentAddWindow.find("#" + $(oEl).children()[0].id).ejCheckBox({ checked: checkedVal });
            }
        },

        _findMinMaxCellIndex: function () {
            var oRowCell = this.element.find('div.e-draggableworkarea tr');
            var rowcount = oRowCell.length - 1;
            var omincolIndex, omaxcolIndex, rowInx, maxrowInx, checkcolindex;
            var curr_View = this.currentView();
            var viewCount = curr_View == "day" ? 1 : curr_View == "workweek" ? this.model.workWeek.length : 7;
            var proxy = this;
            this.element.find('td.e-selectedCell').each(function (index, tdItem) {
                var targetId = $(tdItem).index();
                if (targetId != null && !isNaN(targetId)) {
                    if (omincolIndex == null) {
                        omincolIndex = targetId;
                        rowInx = $(tdItem).parent()[0].rowIndex;
                    }
                    else if (omincolIndex < targetId) {
                        omincolIndex = omincolIndex;
                    }
                    else {
                        if (omincolIndex != targetId) {
                            omincolIndex = targetId;
                            rowInx = $(tdItem).parent()[0].rowIndex;
                        }
                    }
                    if (curr_View != "month") {
                        for (var i = 0; i <= rowcount; i++) {
                            if ($(oRowCell[i].cells[omincolIndex]).hasClass("e-selectedCell")) {
                                rowInx = i;
                                break;
                            }
                        }
                    }
                    if (omaxcolIndex == null) {
                        omaxcolIndex = targetId;
                        if (curr_View != "month") {
                            checkcolindex = parseInt(omaxcolIndex);
                            for (var i = 0; i <= rowcount; i++) {
                                if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                    maxrowInx = i;

                                }
                            }
                        }
                    }
                    else if (omaxcolIndex > targetId) {
                        omaxcolIndex = omaxcolIndex;
                    }
                    else {
                        if (omaxcolIndex != targetId) {
                            omaxcolIndex = targetId;
                            if (curr_View != "month") {
                                checkcolindex = parseInt(omaxcolIndex);
                                for (var i = 0; i <= rowcount; i++) {
                                    if ($(oRowCell[i].cells[checkcolindex]).hasClass("e-selectedCell")) {
                                        maxrowInx = i;
                                    }
                                }
                            }
                        }
                    }
                }
            });
            var _minMaxIndexes = { rowInx: rowInx, maxrowInx: maxrowInx, omincolIndex: omincolIndex, omaxcolIndex: omaxcolIndex };
            return _minMaxIndexes;
        },

        _multipleCellAppointCreation: function (e) {
            var target = !ej.isNullOrUndefined(e.target) ? $(e.target) : $(e);
            this._cellIndex = $(target).hasClass("e-workcells") ? $(target).index() : $(target).hasClass("e-alldaycells") ? $(target).index() : 7 - ((parseInt($(target).index() / 7) + 1) * 7 - $(target).index()) + ($(target).parent().index() * 7);
            if (this.model.orientation == "horizontal") {
                this._cellIndex = this.currentView() !== "month" ? Math.floor(this._cellIndex / ((this.model.endHour - this.model.startHour) * 2)) : this._cellIndex;
                this._workCellIndex = $(target).parent().index();
            }
            var indexes = this._findMinMaxCellIndex();
            var renderDates = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : this.model.orientation == "vertical" ? this.dateRender : this._dateRender;
            var omincolIndex = indexes.omincolIndex, omaxcolIndex = indexes.omaxcolIndex, rowInx = indexes.rowInx, maxrowInx = indexes.maxrowInx;
            var minStartDate = renderDates[omincolIndex]; var maxEndDate = renderDates[omaxcolIndex];
            var localizeDays = Globalize.culture(this.model.locale);
            if (this.model.orientation == "vertical") {
                if ($(target).hasClass("e-workcells")) {
                    var minStartTime = (this.model.startHour + (rowInx / 2)).toString().split(".");
                    var maxEndTime = (this.model.startHour + (maxrowInx / 2)).toString().split(".");
                    this.cur_StartTime = new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(new Date(this.cur_EndTime).setMinutes(new Date(this.cur_EndTime).getMinutes() + 30));
                }
                else if ($(target).hasClass("e-monthcells")) {
                    var selCell = this.element.find('td.e-selectedCell');
                    var nsc = selCell.length - 1;
                    var startIndex = ($(selCell[0]).parent().index() * 7) + ($(selCell[0]).index() - (Math.floor($(selCell[0]).index() / 7) * 7));
                    var endIndex = ($(selCell[nsc]).parent().index() * 7) + ($(selCell[nsc]).index() - (Math.floor($(selCell[nsc]).index() / 7) * 7));
                    var startDate = new Date(this._dateRender[startIndex]);
                    var endDate = new Date(this._dateRender[endIndex]);
                    this.cur_StartTime = new Date(startDate.setHours(0, 0, 0));
                    this.cur_EndTime = new Date(endDate.setHours(23, 59, 59));
                }
                else if ($(target).hasClass("e-alldaycells")) {
                    this.cur_StartTime = new Date(minStartDate);
                    this.cur_EndTime = new Date(new Date(maxEndDate).setHours(23, 59, 59));
                }
            }
            else {
                if (this.currentView() != "month" && !this._isCustomView() && this.model.showTimeScale) {
                    var minStartDate = renderDates[0]; var maxEndDate = renderDates[0];
                    var minStartTime = (this.model.startHour + (omincolIndex / 2)).toString().split(".");
                    var maxEndTime = (this.model.startHour + (omaxcolIndex / 2)).toString().split(".");
                    this.cur_StartTime = new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(new Date(this.cur_EndTime).setMinutes(new Date(this.cur_EndTime).getMinutes() + 30));
                }
                else {
                    var minStartDate = this.totdays[omincolIndex]; var maxEndDate = this.totdays[omaxcolIndex];
                    var minStartTime = (this.model.startHour).toString().split(".");
                    var maxEndTime = (this.model.startHour).toString().split(".");
                    this.cur_StartTime = new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                    this.cur_EndTime = new Date(new Date(this.cur_EndTime).setHours(23, 59, 59));
                }
            }
            if (this.model.showQuickWindow && e.type !== "beforeOpen") {
                this._quickAppointWindow.find("#" + this._id + "_quickAppwindow").css("display", "block");
                if (new Date(new Date(this.cur_StartTime).setHours(0, 0, 0, 0)).getTime() == new Date(new Date(this.cur_EndTime).setHours(0, 0, 0, 0)).getTime())
                    this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(this.cur_StartTime).getDay()] + ", " + Globalize.format(new Date(this.cur_StartTime), "MMMM") + " " + new Date(this.cur_StartTime).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), this._timeMode == "12" ? this._pattern.t : "HH:mm") + " - " + Globalize.format(new Date(this.cur_EndTime), this._timeMode == "12" ? this._pattern.t : "HH:mm"));
                else
                    this._quickAppointWindow.find('.e-quickstartend').html(localizeDays.calendar.days.names[new Date(this.cur_StartTime).getDay()] + ", " + Globalize.format(new Date(this.cur_StartTime), "MMMM") + " " + new Date(this.cur_StartTime).getDate() + ", " + Globalize.format(new Date(this.cur_StartTime), this._timeMode == "12" ? this._pattern.t : "HH:mm") + " - " + localizeDays.calendar.days.names[new Date(this.cur_EndTime).getDay()] + ", " + Globalize.format(new Date(this.cur_EndTime), "MMMM") + " " + new Date(this.cur_EndTime).getDate() + ", " + Globalize.format(new Date(this.cur_EndTime), this._timeMode == "12" ? this._pattern.t : "HH:mm"));
                this._quickAppointWindow.find('.subject').val("");
                if (!this.model.readOnly)
                    this._quickAppointWindow.ejDialog("open");
                var x = $(target).offset().left - ((this._quickAppointWindow.width() - $(target).width()) / 2) + 10;
                var y = $(target).offset().top - this._quickAppointWindow.height() - 10;
                this._quickAppointWindow.ejDialog({ position: { X: x, Y: y } });
                this._quickAppointWindow.find(".subject").focus();
                if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                    this._getResourceDetails($(target));
            }
            this._cellClick = true;

        },

        _addCellSelection: function (oRowCell) {
            var tds = this.element.find('td.e-selectedCell');
            var nt = tds.length - 1;
            if (nt > 0) {
                this.element.find('td.e-selectedCell').removeClass("e-selectedCell");
                $(oRowCell[this._currentRowIndex].cells[this._currentColIndex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
        },

        _processUpArrow: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            var allda = this.element.find(".e-alldaycells");
            if ($(oEl).hasClass("e-workcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) - 1;
                if (currrowindex > 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    if ($(oRowCell[currrowindex - 1].cells[currcolindex]).hasClass("e-resourceheadercells")) return false;
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (currrowindex == 0) {
                    if (this.model.orientation == "vertical") {
                        var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                        var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(this.element).attr("tabIndex", 0).focus();
                        $(allda[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[0].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                return false;
            }

            else if ($(oEl).hasClass("e-monthcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) - 1;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    if (newcur >= 0) {
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[0].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
            else
                return false;

        },

        _processDown: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            if ($(oEl).hasClass("e-workcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) + 1;
                if (currrowindex >= 0 && currrowindex <= rowcount && currrowindex != "undefined") {
                    if (newcur <= rowcount) {
                        if ($(oRowCell[currrowindex + 1].cells[currcolindex]).hasClass("e-resourceheadercells")) return false;
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[currrowindex].cells[currcolindex]).attr("tabIndex", 0).focus();
                }
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex;
                var allda = this.element.find(".e-alldaycells");
                $(allda[allDayInx]).removeClass("e-selectedCell").blur();
                $(this.element.find(".e-workcells")[allDayInx]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }

            else if ($(oEl).hasClass("e-monthcells")) {
                var rowcount = oRowCell.length - 1;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currrowindex) + 1;
                if (currrowindex >= 0 && currrowindex < rowcount && currrowindex != "undefined") {
                    if (newcur <= 7) {
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(oRowCell[newcur].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else
                        $(oRowCell[newcur - 1].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
            }
            else
                return false;
        },

        _processRight: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            if ($(oEl).hasClass("e-workcells")) {
                var maxcolindex = oRowCell[0].cells.length;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (currcolindex >= 0 && currcolindex < parseInt(maxcolindex) - 1 && currcolindex != "undefined") {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (currcolindex == parseInt(maxcolindex) - 1) {
                    if (this.currentView() != "month") {
                        var maxrowInx1 = $(oEl).parent()[0].rowIndex;
                        this.element.find("div.e-navigatenext").click();
                        $(oRowCell[maxrowInx1].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    } else {
                        this.element.find("div.e-navigatenext").click();
                        $(this.element.find('div.e-draggableworkarea tr')[currrowindex].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                }
                else
                    return false;
            }
            else if ($(oEl).hasClass("e-monthcells")) {
                var resIndex = Math.floor($(oEl).index() / 7);
                var rowcount = oRowCell.length - 1, mcolind = resmaxcolindex = 7 * (parseInt(resIndex) + 1), mincolindex = 7 * (parseInt(resIndex));;
                var currrowindex = $(oEl).parent()[0].rowIndex;
                var currcolindex = $(oEl)[0].cellIndex;
                var newcur = parseInt(currcolindex) + 1;
                if (newcur < mcolind) {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                if (currcolindex == parseInt(resmaxcolindex) - 1 && currrowindex < (this._dateRender.length) / 7) {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    var newrow = parseInt(currrowindex) + 1;
                    currcolindex = mincolindex;
                    if (newrow < (this._dateRender.length) / 7) {
                        $(oRowCell[newrow].cells[currcolindex]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        if (newcur < mcolind) {
                            $(oRowCell[newrow].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                        }
                    }
                    else {
                        var previndex = currcolindex / 7;
                        this.element.find("div.e-navigatenext").click();
                        var index = (!ej.isNullOrUndefined(this.model.group) ? previndex * 7 : 0);
                        $(this.element.find(".e-monthcells")[index]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                }
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex;
                var nall = parseInt(this.element.find(".e-alldaycells").length) - 1;
                if (allDayInx >= 0 && allDayInx < nall) {
                    $(this.element.find(".e-alldaycells")[allDayInx]).removeClass("e-selectedCell").blur();
                    $(this.element.find(".e-alldaycells")[parseInt(allDayInx) + 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (allDayInx == nall) {
                    this.element.find("div.e-navigatenext").click();
                    $(this.element.find(".e-alldaycells")[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
        },

        _processLeft: function (oEl, oRowCell) {
            if ($(oEl).hasClass("e-selectedAppointment")) {
                this.element.find('.e-selectedAppointment').removeClass("e-selectedAppointment");
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-selectedCell"))
                this._addCellSelection(oRowCell);
            if ($(oEl).hasClass("e-workcells")) {
                var maxcolindex = oRowCell[0].cells.length;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;
                var newcur = parseInt(currcolindex) - 1;
                if (currcolindex > 0 && currcolindex <= parseInt(maxcolindex) - 1 && currcolindex != "undefined") {
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (currcolindex == 0) {
                    if (this.currentView() != "month") {
                        var maxrowInx1 = $(oEl).parent()[0].rowIndex;
                        this.element.find("div.e-navigateprevious").click();
                        $(oRowCell[maxrowInx1].cells[maxcolindex - 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else {
                        this.element.find("div.e-navigateprevious").click();
                        $(this.element.find('div.e-draggableworkarea tr')[currrowindex].cells[this.totdays.length - 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                }
                else
                    return false;
            }
            else if ($(oEl).hasClass("e-monthcells")) {
                var resIndex = Math.floor($(oEl).index() / 7);
                var rowcount = oRowCell.length - 1, mcolind = 7 * (parseInt(resIndex) + 1);
                var mincolindex = 7 * (parseInt(resIndex));
                var count = this.currentView() == "day" ? 1 : this.currentView() == "workweek" ? this.model.workWeek.length : 7;
                var colIndex = this.model.orientation == "vertical" ? count : this.currentView() == "month" ? this.totdays.length : count * (this.model.endHour - this.model.startHour) * 2;
                maxcolindex = (!ej.isNullOrUndefined(this.model.resources) && !ej.isNullOrUndefined(this.model.group) && ej.scheduleFeatures.resources) && this.model.orientation == "vertical" ? this.render_Resources[this.render_Resources.length - 1].length * colIndex : colIndex;
                var currrowindex = this.element.find("td.e-selectedCell").parent()[0].rowIndex;
                var currcolindex = this.element.find("td.e-selectedCell")[0].cellIndex;

                if (currcolindex == mincolindex && currrowindex >= 0) {
                    var newrow = parseInt(currrowindex) - 1;
                    var newcol = parseInt(mcolind) - 1;
                    if (newrow >= 0) {
                        $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                        $(oRowCell[newrow].cells[newcol]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                    else {
                        var previndex = currcolindex / 7;
                        this.element.find("div.e-navigateprevious").click();
                        var rowmax = this._dateRender.length / 7;
                        var index = (!ej.isNullOrUndefined(this.model.group) ? previndex * 7 : 0);
                        $(this.element.find('div.e-draggableworkarea tr')[rowmax - 1].cells[index + 6]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                    }
                }
                else if (currcolindex < mcolind) {
                    newcur = parseInt(currcolindex) - 1;
                    $(oRowCell[currrowindex].cells[currcolindex]).removeClass("e-selectedCell").blur();
                    $(oRowCell[currrowindex].cells[newcur]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
            }
            else if ($(oEl).hasClass("e-alldaycells")) {
                var allDayInx = this.element.find('td.e-selectedCell')[0].cellIndex;
                var nall = parseInt(this.element.find(".e-alldaycells").length) - 1;
                if (allDayInx > 0 && allDayInx <= nall) {
                    $(this.element.find(".e-alldaycells")[allDayInx]).removeClass("e-selectedCell").blur();
                    $(this.element.find(".e-alldaycells")[parseInt(allDayInx) - 1]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else if (allDayInx == 0) {
                    this.element.find("div.e-navigateprevious").click();
                    $(this.element.find(".e-alldaycells")[nall]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                }
                else
                    return false;
            }
        },

        _navigateToolbarRight: function (event) {
            var views = this.model.views;
            var curViewIndex = this.element.find(".e-commonviewbutton.e-" + this.currentView()).parent().index();
            if (curViewIndex < views.length - 1) {
                var newView = views[curViewIndex + 1].toLowerCase(); this._navView = true;
                this._viewChange(event, this.currentView(), newView);
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            }
        },

        _navigateToolbarLeft: function (event) {
            var curViewIndex = this.element.find(".e-commonviewbutton.e-" + this.currentView()).parent().index();
            if (curViewIndex > 0) {
                var newView = this.model.views[curViewIndex - 1].toLowerCase(); this._navView = true;
                this._viewChange(event, this.currentView(), newView);
                this.element.find("#" + this._id + "_navigator").ejDatePicker({ value: new Date(this.currentDate()) });
            }
        },

        _newAppWindow: function (contextElement, oRowCell) {
            if ($(contextElement).hasClass("e-selectedCell")) {
                this._currentAction = ej.Schedule.Actions.Add;
                var targetElement = contextElement == null ? event : contextElement;
                var rowIndex = $(targetElement).parent().index();
                var columnIndex = $(targetElement).index();
                var resourceIndex = 0;
                var renderDates = (this.model.orientation == "horizontal" && this.currentView() == "month") ? this.monthDays : (this.model.orientation == "vertical" || this.currentView() == "customview" && this._dateRender.length <= 7) ? this.dateRender : this._dateRender;
                renderDates = this.model.orientation == "horizontal" && this.currentView() == "customview" && this._dateRender.length <= 7 ? this._dateRender : renderDates;
                var ns = parseInt(this.element.find('td.e-selectedCell').length) - 1;
                _target = $(targetElement);
                if (this.model.showTimeScale)
                    this._cellIndex = _target.hasClass("e-workcells") ? _target.index() : _target.hasClass("e-alldaycells") ? _target.index() : this._isCustomView() && this._oneWeek ? _target.index() : 7 - ((parseInt(_target.index() / 7) + 1) * 7 - _target.index()) + (_target.parent().index() * 7);
                else
                    this._cellIndex = this.model.orientation == "vertical" && this.currentView() == "month" || (this.currentView() == "customview" && this._dateRender.length >= 7) ? 7 - ((parseInt(_target.index() / 7) + 1) * 7 - _target.index()) + (_target.parent().index() * 7) : _target.index();
                if (this.model.orientation == "horizontal") {
                    this._cellIndex = this.model.showTimeScale ? this.currentView() !== "month" && !(this._isCustomView()) ? Math.floor(this._cellIndex / ((this.model.endHour - this.model.startHour) * 2)) : this._cellIndex : _target.index();
                    this._workCellIndex = _target.parent().index();
                }
                if (ns == 0) {
                    var curDate = new Date(renderDates[this._cellIndex]);
                    if ((this.model.minDate > curDate || this.model.maxDate < curDate)) { return false; }
                    if (_target.hasClass("e-workcells") && (this.model.showTimeScale) && this.currentView() !== "month" && !(this._isCustomView())) {
                        var time = this.model.orientation == "vertical" ? this.model.startHour + (_target.parent().index() / 2) : this.model.startHour + ((_target.index() - (((this.model.endHour - this.model.startHour) * 2) * this._cellIndex)) / 2);
                        var timemin = time.toString().split(".");
                        startTime = new Date(curDate).setHours(parseInt(timemin[0]), parseInt(timemin[1]) == 5 ? 30 : 00);
                        var min = (parseInt(new Date(startTime).getHours()) == 23 && parseInt(new Date(startTime).getMinutes()) == 30) ? new Date(startTime).getMinutes() + 29 : new Date(startTime).getMinutes() + 30;
                        endTime = new Date(new Date(startTime).setMinutes(min));
                        this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: false });
                    }
                    else {
                        startTime = new Date(new Date(curDate).setHours(0, 0, 0));
                        endTime = new Date(new Date(curDate).setHours(23, 59, 59));
                        this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
                    }
                }
                else if (ns > 0) {
                    var indexes = this._findMinMaxCellIndex();
                    var omincolIndex = indexes.omincolIndex, omaxcolIndex = indexes.omaxcolIndex, rowInx = indexes.rowInx, maxrowInx = indexes.maxrowInx;
                    if (this.model.orientation == "vertical") {
                        if ($(contextElement).hasClass('e-workcells')) {
                            var minStartDate = renderDates[omincolIndex]; var maxEndDate = renderDates[omaxcolIndex];
                            var minStartTime = (this.model.startHour + (rowInx / 2)).toString().split(".");
                            var maxEndTime = (this.model.startHour + (maxrowInx / 2)).toString().split(".");
                            startTime = new Date(new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00));
                            endTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                            endTime = new Date(new Date(endTime).setMinutes(new Date(endTime).getMinutes() + 30));
                        }
                        else if ($(contextElement).hasClass('e-monthcells')) {
                            var selCell = this.element.find('td.e-selectedCell');
                            var startIndex = ($(selCell[0]).parent().index() * 7) + ($(selCell[0]).index() - (Math.floor($(selCell[0]).index() / 7) * 7));
                            var endIndex = ($(selCell[ns]).parent().index() * 7) + ($(selCell[ns]).index() - (Math.floor($(selCell[ns]).index() / 7) * 7));
                            var startDate = new Date(this._dateRender[startIndex]);
                            var endDate = new Date(this._dateRender[endIndex]);
                            startTime = new Date(new Date(startDate).setHours(0, 0, 0));
                            endTime = new Date(new Date(endDate).setHours(23, 59, 59));
                        }
                        else if ($(contextElement).hasClass('e-alldaycells')) {
                            startTime = new Date(new Date(renderDates[omincolIndex]).setHours(0, 0, 0));
                            endTime = new Date(new Date(renderDates[omaxcolIndex]).setHours(23, 59, 59));
                        }
                    }
                    else {
                        if (this.currentView() != "month" && !this._isCustomView() && this.model.showTimeScale) {
                            var minStartDate = renderDates[0]; var maxEndDate = renderDates[0];
                            var minStartTime = (this.model.startHour + (omincolIndex / 2)).toString().split(".");
                            var maxEndTime = (this.model.startHour + (omaxcolIndex / 2)).toString().split(".");
                            startTime = new Date(new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00));
                            endTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                            endTime = new Date(new Date(endTime).setMinutes(new Date(endTime).getMinutes() + 30));
                        }
                        else {
                            var minStartDate = this.totdays[omincolIndex]; var maxEndDate = this.totdays[omaxcolIndex];
                            var minStartTime = (this.model.startHour).toString().split(".");
                            var maxEndTime = (this.model.startHour).toString().split(".");
                            startTime = new Date(minStartDate).setHours(parseInt(minStartTime[0]), parseInt(minStartTime[1]) == 5 ? 30 : 00);
                            endTime = new Date(maxEndDate).setHours(parseInt(maxEndTime[0]), parseInt(maxEndTime[1]) == 5 ? 30 : 00);
                            endTime = new Date(new Date(endTime).setMinutes(new Date(endTime).getMinutes() + 30));
                            endTime = new Date(new Date(endTime).setHours(23, 59, 59));
                        }
                    }
                }
                var event = { startTime: new Date(startTime), endTime: new Date(endTime), target: _target, resources: ((this.model.appointmentWindowOpen != null || this.model.cellDoubleClick != null) && !ej.isNullOrUndefined(this.model.resources)) ? this._getResourceValue(_target) : null };
                if (this._trigger("appointmentWindowOpen", event))
                    return false;
                if (ej.scheduleFeatures.resources && !ej.isNullOrUndefined(this.model.resources))
                    this._renderResourceData($(targetElement), this._cellIndex);
                this._appointmentAddWindow.find("#" + this._id + "startdate").ejDatePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.d));
                this._appointmentAddWindow.find("#" + this._id + "enddate").ejDatePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.d));
                this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value", Globalize.format(new Date(startTime), this._pattern.t));
                this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value", Globalize.format(new Date(endTime), this._pattern.t));
                ($(contextElement).hasClass("e-alldaycells") || this.currentView() == "month" || (this._isCustomView())) && this._appointmentAddWindow.find(".allday").ejCheckBox({ checked: true });
                this._timeZoneCollection(null);
                this._tempStart = this._appointmentAddWindow.find("#" + this._id + "starttime").ejTimePicker("option", "value");
                this._tempEnd = this._appointmentAddWindow.find("#" + this._id + "endtime").ejTimePicker("option", "value");
                this._alldayCheck();
                if (!this.model.readOnly) {
                    this._appointmentAddWindow.find(".e-appsave,#donerecur").ejButton("enable", true);
                    (this._mediaQuery) && this._appointmentAddWindow.find(".e-btndelete").css("display", "none");
                    this._appointmentAddWindow.ejDialog("open");
                }
                this._appointmentAddWindow.find('.subject').focus();
                this._cellClick = false;
            }

        },

        _showToolTip: function (event) {
            if (this.model.tooltipSettings.enable && !ej.isTouchDevice()) {
			  if ($(event.target).is(".e-tophandle,.e-bottomhandle,.e-schedulemouseclose,.e-righthandle,.e-lefthandle,.e-schedulerepeat,.e-editapp,.e-deleteapp")) return false;
                var appData = this._getAppDetails(event);
                var tooltipdiv = $('.e-scheduletooltip');
                if (tooltipdiv.length == 0) {
                    tooltipdiv = $("<div class='e-scheduletooltip' style='position: absolute; z-index: 105; display: block;'></div>");
                    $(document.body).append(tooltipdiv);
                }
                if (ej.isNullOrUndefined(this.model.tooltipSettings.template)) {
                    var startTime = this._timeMode == "12" ? Globalize.format(appData.StartTime, this._pattern.t) : Globalize.format(appData.StartTime, "HH:mm");
                    var endTime = this._timeMode == "12" ? Globalize.format(appData.EndTime, this._pattern.t) : Globalize.format(appData.EndTime, "HH:mm");
                    var subject = appData.Subject == "" ? this._getLocalizedLabels("NoTitle") : appData.Subject;
                    var appDetails = subject + ":&nbsp;(" + startTime + "-" + endTime + ")";
                    tooltipdiv.html(appDetails);
                } else {
                    var appDetails = $(this.model.tooltipSettings.template).render(appData);
                    tooltipdiv.html(appDetails.trim());
                }
                var xPos = event.pageX;
                var yPos = event.pageY;
				var tooltipdivelmt = $(tooltipdiv);
                xPos = ((xPos + tooltipdivelmt.width()) < this.model.width) ? (xPos) : (xPos - tooltipdivelmt.width());
                yPos = ((yPos + tooltipdivelmt.height()) < this.model.height) ? (yPos) : (yPos - tooltipdivelmt.height());
                $(tooltipdiv).css("left", xPos);
                $(tooltipdiv).css("top", yPos);
                if (this.model.enableRTL == true) tooltipdivelmt.addClass("e-rtl");
                $('.e-scheduletooltip').show();
            }
        },

        _hideToolTip: function () {
            if (this.model.tooltipSettings.enable)
                $('.e-scheduletooltip').remove();
        },

        _processTab: function (oEl, event, oRowCell) {
            var appo = this.element.find('div.e-appointment,div.e-alldayappointment,div.e-monthappointment,div.e-agendaappointment');
            if (oEl.id == this._id || $(oEl).hasClass("e-datedisplay")) {
                if (appo.length != 0)
                    $(appo[0]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                else
                    $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            if ($(oEl).hasClass("e-selectedCell")) {
                if (appo.length != 0) {
                    var selcel = this.element.find('td.e-selectedCell');
                    var n = parseInt(selcel.length) - 1;
                    if (n > 0) {
                        for (var i = 0; i <= n; i++) {
                            selcel[i].blur();
                            $(selcel[i]).removeClass("e-selectedCell");
                        }
                    }
                    else
                        $(oEl).removeClass("e-selectedCell").blur();
                    $(appo[0]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                }
            }
            if ($(oEl).hasClass("e-navigateprevious")) {
                this.element.find('.e-navigatenext').attr("tabIndex", 0).focus();
            }
            else if ($(oEl).hasClass("e-navigatenext")) {
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                if ($(oRowCell[0].cells[0]).hasClass("e-resourceheadercells"))
                    $(oRowCell[1].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment") || $(oEl).hasClass("e-agendaappointment")) {
                var appcount = appo.length;
                var appindex = this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,div.e-agendaappointment").index(this.element.find(".e-selectedAppointment"));
                if (appindex < appcount && appindex >= 0) {
                    $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                    var newappindex = parseInt(appindex) + 1;
                    if (newappindex < appcount)
                        $(appo[newappindex]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                    else if (newappindex == appcount)
                        this.element.find('.e-navigateprevious').attr("tabIndex", 0).focus();
                }
                else if (appindex == appcount)
                    this.element.find('.e-navigateprevious').attr("tabIndex", 0).focus();
            }
        },

        _reverseTab: function (oEl, event, oRowCell) {
            var appo = this.element.find('div.e-appointment,div.e-alldayappointment,div.e-monthappointment,div.e-agendaappointment');
            if (oEl.id == this._id || $(oEl).hasClass("e-datedisplay")) {
                var len = parseInt(appo.length) - 1;
                if (appo.length != 0)
                    $(appo[len]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
            }
            var appcount = appo.length;
            if ($(oEl).hasClass("e-selectedCell")) {
                var len = appo.length - 1;
                if (appo.length != 0) {
                    var selcel = this.element.find('td.e-selectedCell');
                    var n = parseInt(selcel.length) - 1;
                    if (n > 0) {
                        for (var i = 0; i <= n; i++)
                            $(selcel[i]).removeClass("e-selectedCell").blur();
                    }
                    else
                        $(oEl).removeClass("e-selectedCell").blur();
                    $(appo[len]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                }
            }
            if ($(oEl).hasClass("e-navigatenext"))
                $(this.element).find('.e-navigateprevious').attr("tabIndex", 0).focus();
            else if ($(oEl).hasClass("e-navigateprevious")) {
                $(oRowCell[0].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
                if ($(oRowCell[0].cells[0]).hasClass("e-resourceheadercells"))
                    $(oRowCell[1].cells[0]).addClass("e-selectedCell").attr("tabIndex", 0).focus();
            }
            if ($(oEl).hasClass("e-appointment") || $(oEl).hasClass("e-alldayappointment") || $(oEl).hasClass("e-monthappointment") || $(oEl).hasClass("e-agendaappointment")) {
                var appindex = this.element.find("div.e-appointment,div.e-alldayappointment,div.e-monthappointment,div.e-agendaappointment").index(this.element.find(".e-selectedAppointment"));
                if (appindex < appcount && appindex > 0) {
                    $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                    var newappindex = parseInt(appindex) - 1;
                    if (newappindex < appcount && appindex >= 0)
                        $(appo[newappindex]).addClass("e-selectedAppointment").attr("tabIndex", 0).focus();
                    else if (newappindex == 0)
                        $(this.element).find('.e-navigatenext').attr("tabIndex", 0).focus();
                }
                else if (appindex == 0) {
                    $(appo[appindex]).removeClass("e-selectedAppointment").blur();
                    $(this.element).find('.e-navigatenext').attr("tabIndex", 0).focus();
                }
            }
        },

        _stopDefaultAction: function (e) {
            e.cancelBubble = true; e.returnValue = false; e.stopPropagation(); e.preventDefault();
        },

        _getLocalizedLabels: function (property) {
            return ej.Schedule.Locale[this.model.locale][property] === undefined ? ej.Schedule.Locale["en-US"][property] : ej.Schedule.Locale[this.model.locale][property];
        },

        refreshAppointments: function () {
            !ej.isNullOrUndefined(this.model.appointmentSettings) && (this._dataManager = this.dataSource() instanceof ej.DataManager ? this.dataSource() : this.dataSource() != null ? ej.DataManager(this.dataSource()) : null);
            this._bindAppointmentsData("Refresh");
        },

        refresh: function () {
            this._destroy();
            this._init();
        },

        saveAppointment: function (obj) {
            if (!this.model.readOnly) {
                var appointments = this._sortAppById(this._processed), maxId;
                this._publicMethodSave = false;
                obj[this.model.appointmentSettings["startTimeZone"]] = this.model.timeZone == null ? ("UTC " + Globalize.format(new Date(), "zzz")) : this.model.timeZone;
                obj[this.model.appointmentSettings["endTimeZone"]] = this.model.timeZone == null ? ("UTC " + Globalize.format(new Date(), "zzz")) : this.model.timeZone;
				var savingTimeZone = this._timeZoneSavingAppoint(obj, "");
                obj = savingTimeZone.obj;
                var temp = this._timeZoneAppointments(obj, "");
                if (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["id"]]) || !obj[this.model.appointmentSettings["id"]] == "")
                {
                    var tempApp = new ej.DataManager(this._processed).executeLocal(new ej.Query().where(this.model.appointmentSettings["id"], ej.FilterOperators.equal, obj[this.model.appointmentSettings["id"]]));
                    if (!ej.isNullOrUndefined(tempApp[0]))
                    {
                        obj.Guid = tempApp[0].Guid;
                        obj.AppTaskId = tempApp[0].AppTaskId;
                        obj.RecurrenceId = tempApp[0].recurrenceId;
                        obj.recurrenceExDate = tempApp[0].recurrenceExDate;
                        obj.ParentId = tempApp[0].ParentId;
                    }
                }
                var proxy = this;
                maxId = !ej.isNullOrUndefined(this._appMainId) ? this._appMainId + 1 : 1;
                if ((ej.isNullOrUndefined(obj["AppTaskId"]) || obj["AppTaskId"] == "") && (ej.isNullOrUndefined(obj[this.model.appointmentSettings["id"]]) || obj[this.model.appointmentSettings["id"]] == "")) {
                    obj[this.model.appointmentSettings["id"]] = maxId;
                    temp[this.model.appointmentSettings["id"]] = maxId;
                    temp["AppTaskId"] = maxId;
                    obj["AppTaskId"] = maxId;
                    temp["ParentId"] = maxId;
                    obj["ParentId"] = maxId;
                   obj.Guid = this._guidFormatGenerate();
                    if (this._trigger("actionBegin", { data: obj, requestType: "appointmentSave", methodType: "public" }))
                        return false;
                    if (this._trigger("appointmentSaved", { appointment: obj, methodType: "public" }))
                        return false;
                    var promise = this._dataManager.insert(temp);
                   
                    updateTempNew = this._timeZoneAppointments(temp, "reverse");
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew);
                    if (!ej.isNullOrUndefined(promise) && $.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            if (proxy.model.enableLoadOnDemand)
                                proxy._initDataSource("refresh");
                            else {
                                var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                proxy._currentAppointmentData.length = 0;
                                proxy._dateFormat(value);
                                proxy._bindAppointments = true;
                                proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                proxy._dataProcessing(value);
                                proxy._renderAppointmentAll();
                            }
                            proxy._trigger("actionComplete", { data: temp, requestType: "appointmentSave", methodType: "public" });
                        });
                    }
                    else {
                        this._processed.push(obj);
                        this._renderSingleApp(obj, obj);
                        this._trigger("actionComplete", { data: obj, requestType: "appointmentSave", methodType: "public" });
                    }
                }
                else if (this._currentAction == "editOcurrence") {
                    if (this._trigger("actionBegin", { data: obj, requestType: "appointmentEdit" }))
                                return false;
                            if (this._trigger("appointmentEdited", { appointment: obj })) 
                                return false;
                    var _appList = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid))[0];
                    var Parid = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? _appList["ParentId"] : _appList[this.model.appointmentSettings["recurrenceId"]];
                    var MapValue = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                    var recurColl = this._appointmentRecurrenceSort(new ej.DataManager(this._processed).executeLocal(new ej.Query().where(MapValue, ej.FilterOperators.equal, Parid)));
                    var index = this._recuCollectionIndex(recurColl, _appList);
                    var model = this.model.appointmentSettings;
                    if (((index == 0) ? (!ej.isNullOrUndefined(recurColl[index + 1]) ? !(new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(obj[model["endTime"]]).setHours(0, 0, 0, 0)).getTime()) : false) : (index == recurColl.length - 1) ? !(new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[model["startTime"]]).setHours(0, 0, 0, 0)).getTime()) : !((new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[model["startTime"]]).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(obj[model["endTime"]]).setHours(0, 0, 0, 0)).getTime())))) {
                        $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                        this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("CreateError"));
                        this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                        this._alertWindow.ejDialog("open");
                        return false;
                    }
                    var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                    var _editApps = new ej.DataManager(this._processed).executeLocal(query);
                    this._objDate = _editApps;
                    var MapValue = this._objDate[0][this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                    var _editApp = this._deleteOcurrence(this._objDate[0][MapValue], new Date(_editApps[0][this.model.appointmentSettings["startTime"]]));
                    new ej.DataManager(this._processed).remove("Guid", this._appUid);
                    obj[this.model.appointmentSettings["recurrence"]] = 1;
                    obj[this.model.appointmentSettings["recurrenceRule"]] = _editApp[0][this.model.appointmentSettings["recurrenceRule"]] + ";RECUREDITID=" + _editApp[0]["AppTaskId"];
                    obj[this.model.appointmentSettings["recurrenceId"]] = _editApp[0][this.model.appointmentSettings["recurrenceId"]];
                    obj["Guid"] = this._appUid;
                    obj[this.model.appointmentSettings["recurrenceExDate"]] = _editApp[0][this.model.appointmentSettings["recurrenceExdate"]];
                    obj["Guid"] = _editApps[0]["Guid"];
                    obj[this.model.appointmentSettings["id"]] = _editApps[0][this.model.appointmentSettings["id"]];
                    var args = [], newAppointment;
                    var parId = this._currentAction == ej.Schedule.Actions.EditOcurrence || (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && obj[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) ? this._parentId : this._currentAction == (ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : maxId;
                    this._maxId = ((((new Date(obj[this.model.appointmentSettings["endTime"]]) - new Date(obj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24) && this._currentAction != ej.Schedule.Actions.EditOcurrence && (obj[this.model.appointmentSettings["allDay"]] == true)) ? parId : this._currentAction == (ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : this._appMainId + 1;
                    this._maxId = (((new Date(obj[this.model.appointmentSettings["endTime"]]) - new Date(obj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24) && (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && obj[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) && (this._currentAction == ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : this._maxId;
                    this._maxId = (this._currentAction == ej.Schedule.Actions.EditSeries) ? parId : this._maxId;
                    newAppointment = this._addAppData(obj, this._maxId, new Date(obj[this.model.appointmentSettings["startTime"]]), new Date(obj[this.model.appointmentSettings["endTime"]]), parId);
                    args.push(newAppointment);
                    var temp = [];
                    var startTimezone1 = args[0][this.model.appointmentSettings["startTimeZone"]];
                    (startTimezone1 != this.model.timeZone) ? temp.push(this._timeZoneAppointments(args[0], "")) : temp.push(args[0]);
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", args[0]);
                    var _editApp = [];
                    var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, this._parentId);
                    var _App = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", _App[0]);
                    _editApp.push(this._timeZoneAppointments(_App[0], ""));
                    if (this.dataSource() instanceof ej.DataManager) {
                        promise = this._dataManager.saveChanges({ added: args, changed: _editApp, deleted: [] }, this.model.appointmentSettings["id"]);
                    }
                    else
                        var promise = this._dataManager.saveChanges({ added: args, changed: _editApp, deleted: [] }, "AppTaskId");
                    var updateTempNew = [];
                    updateTempNew = (startTimezone1 != this.model.timeZone) ? this._timeZoneAppointments(temp[0], "reverse") : temp[0];
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew);
                    var updateTempNew2 = [];
                    if (startTimezone1 != this.model.timeZone)
                        updateTempNew2 = this._timeZoneAppointments(_editApp[0], "reverse");
                    else
                        updateTempNew2 = _editApp[0];
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTempNew2);
                    if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            if (proxy.model.enableLoadOnDemand)
                                proxy._initDataSource("refresh");
                            else {
                                var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                proxy._currentAppointmentData.length = 0;
                                proxy._dateFormat(value);
                                proxy._bindAppointments = true;
                                proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                proxy._dataProcessing(value);
                                proxy._renderAppointmentAll();
                            }
                            proxy._trigger("actionComplete", { data: args, requestType: "appointmentSave" });
                        });
                        promise.fail(function (e) {
                        });
                    }
                    else {
                        (this.model.showTimeScale) && this._appointmentProcessing(args[0]);
                        this.element.find('[guid=' + this._appUid + ']').remove();
                        this.element.find("#categorybar_" + this._appUid).remove();
                        this._renderSingleApp(obj, args[0]);
                        this._trigger("actionComplete", { data: args, requestType: "appointmentEdit", methodType: "public" });
                    }
                }
                else if ((this._currentAction == "save" && obj[this.model.appointmentSettings["recurrence"]]) || (this._currentAction == ej.Schedule.Actions.EditSeries)) {
                    if (this._trigger("actionBegin", { data: obj, requestType: "appointmentEdit" }))
                        return false;
                    if (this._trigger("appointmentEdited", { appointment: obj }))
                        return false;
                    var query = new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid);
                    var _editApps = new ej.DataManager(this._processed).executeLocal(query);
                    if (this._currentAction != ej.Schedule.Actions.EditSeries) {
                        if (!ej.isNullOrUndefined(_editApps[0][this.model.appointmentSettings["recurrenceRule"]]))
                            obj[this.model.appointmentSettings["recurrenceRule"]] = _editApps[0][this.model.appointmentSettings["recurrenceRule"]];
                        var _appList = new ej.DataManager(this._processed).executeLocal(new ej.Query().where("Guid", ej.FilterOperators.equal, this._appUid))[0];
                        var Parid = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? _appList["ParentId"] : _appList[this.model.appointmentSettings["recurrenceId"]];
                        var MapValue = _appList[this.model.appointmentSettings["recurrenceId"]] == null ? "ParentId" : this.model.appointmentSettings["recurrenceId"];
                        var recurColl = this._appointmentRecurrenceSort(new ej.DataManager(this._processed).executeLocal(new ej.Query().where(MapValue, ej.FilterOperators.equal, Parid)));
                        var index = this._recuCollectionIndex(recurColl, _appList);
                        var model = this.model.appointmentSettings;
                        if (((index == 0) ? (!ej.isNullOrUndefined(recurColl[index + 1]) ? !(new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(obj[model["endTime"]]).setHours(0, 0, 0, 0)).getTime()) : false) : (index == recurColl.length - 1) ? !(new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[model["startTime"]]).setHours(0, 0, 0, 0)).getTime()) : !((new Date(new Date(recurColl[index - 1][model["endTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[model["startTime"]]).setHours(0, 0, 0, 0)).getTime()) && (new Date(new Date(recurColl[index + 1][model["startTime"]]).setHours(0, 0, 0, 0)).getTime() > new Date(new Date(obj[model["endTime"]]).setHours(0, 0, 0, 0)).getTime())))) {
                            $("#" + this._id + "alertWindow").ejDialog("instance").option("title", this._getLocalizedLabels("RecurrenceAlert"));
                            this._alertWindow.find(".e-alerttext").html(this._getLocalizedLabels("CreateError"));
                            this._alertWindow.find(".e-alertOk").html(this._getLocalizedLabels("Ok"));
                            this._alertWindow.ejDialog("open");
                            return false;
                        }
                    }
                    else {
                        var Parid = _editApps[0][this.model.appointmentSettings["recurrenceId"]] == null ? _editApps[0]["ParentId"] : _editApps[0][this.model.appointmentSettings["recurrenceId"]];
                        var query = new ej.Query().where("AppTaskId", ej.FilterOperators.equal, Parid);
                        var _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                        obj["AppTaskId"] = _editApp[0]["AppTaskId"];
                        obj[this.model.appointmentSettings["id"]] = _editApp[0][this.model.appointmentSettings["id"]];
                        maxId = _editApp[0]["AppTaskId"];
                        obj[this.model.appointmentSettings["recurrenceId"]] = _editApp[0][this.model.appointmentSettings["recurrenceId"]];
                        obj["Guid"] = this._appUid;
                        obj[this.model.appointmentSettings["recurrenceExDate"]] = _editApp[0][this.model.appointmentSettings["recurrenceExate"]];
                        obj[this.model.appointmentSettings["recurrenceRule"]] = (ej.isNullOrUndefined(this._recRule) || this._recRule == "") ? _editApp[0][this.model.appointmentSettings["recurrenceRule"]] : this._recRule;
                    }
                    obj["Guid"] = _editApps[0]["Guid"];
                    obj[this.model.appointmentSettings["id"]] = (this._currentAction != ej.Schedule.Actions.EditSeries) ? _editApps[0][this.model.appointmentSettings["id"]] : obj[this.model.appointmentSettings["id"]];
                    var args = [], newAppointment;
                    var parId = this._currentAction == ej.Schedule.Actions.EditOcurrence || (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && obj[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) ? this._parentId : this._currentAction == (ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : maxId;
                    this._maxId = ((((new Date(obj[this.model.appointmentSettings["endTime"]]) - new Date(obj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24) && this._currentAction != ej.Schedule.Actions.EditOcurrence && (obj[this.model.appointmentSettings["allDay"]] == true)) ? parId : this._currentAction == (ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : this._appMainId + 1;
                    this._maxId = (((new Date(obj[this.model.appointmentSettings["endTime"]]) - new Date(obj[this.model.appointmentSettings["startTime"]])) / 3600000) > 24) && (!ej.isNullOrUndefined(obj[this.model.appointmentSettings["recurrenceRule"]]) && obj[this.model.appointmentSettings["recurrenceRule"]].toString().indexOf("RECUREDITID") != -1) && (this._currentAction == ej.Schedule.Actions.Save) ? _editApps[0].AppTaskId : this._maxId;
                    this._maxId = (this._currentAction == ej.Schedule.Actions.EditSeries) ? parId : this._maxId;
                    newAppointment = this._addAppData(obj, this._maxId, new Date(obj[this.model.appointmentSettings["startTime"]]), new Date(obj[this.model.appointmentSettings["endTime"]]), parId);
                    args.push(newAppointment);
                    var newApp = [], editedApp = [], deleted = [];
                    var GId = this._currentAction == ej.Schedule.Actions.Save ? args[0]["Guid"] : args[0][this.model.appointmentSettings["recurrenceId"]];
                    var idName = this._currentAction == ej.Schedule.Actions.Save ? "Guid" : this.model.appointmentSettings["recurrenceId"];
                    query = new ej.Query().where(idName, ej.FilterOperators.equal, GId);
                    var _editApp = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                    (!ej.isNullOrUndefined(_editApp) && _editApp.length > 0) ? editedApp.push(args[0]) : newApp.push(args[0]);
                    if (this._currentAction == ej.Schedule.Actions.EditSeries) {
                        var query = ej.Query().where(ej.Predicate(this.model.appointmentSettings["recurrenceRule"], ej.FilterOperators.contains, "RECUREDITID=" + args[0].ParentId, true));
                        var dele = new ej.DataManager(this._currentAppointmentData).executeLocal(query);
                        deleted = deleted.concat(dele);
                        this._appointmentAddWindow.find('.recurbutton').click();
                        editedApp[0][this.model.appointmentSettings["recurrenceRule"]] = this._recRule;
                        editedApp[0][this.model.appointmentSettings["recurrenceExDate"]] = null;
                    }

                    var proxy = this, promise, newTemp = [], editTemp = [];
                    if (newApp.length > 0)
                        newTemp.push(this._timeZoneAppointments(newApp[0], ""));
                    if (editedApp.length > 0)
                        editTemp.push(this._timeZoneAppointments(editedApp[0], ""));
                    if (this.dataSource() instanceof ej.DataManager) {
                        var promise = this._dataManager.saveChanges({ added: newTemp, changed: editTemp, deleted: deleted }, this.model.appointmentSettings["id"]);
                    }
                    else
                        var promise = this._dataManager.saveChanges({ added: newTemp, changed: editTemp, deleted: deleted }, "AppTaskId");
                    if (editedApp.length > 0)
                        var updateTemp = this._timeZoneAppointments(editTemp[0], "reverse");
                    if (newApp.length > 0)
                        var updateTemp = this._timeZoneAppointments(newTemp[0], "reverse");
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTemp);
                    if (editedApp.length > 0) {
                        var proxy = this;
                        if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                if (proxy.model.enableLoadOnDemand)
                                    proxy._initDataSource("refresh");
                                else {
                                    var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                    proxy._currentAppointmentData.length = 0;
                                    proxy._dateFormat(value);
                                    proxy._bindAppointments = true;
                                    proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                    proxy._dataProcessing(value);
                                    proxy._renderAppointmentAll();
                                }
                                proxy._appMainId = proxy._processed.length > 0 ? proxy._sortAppById(proxy._processed)[proxy._processed.length - 1]["AppTaskId"] : proxy._appMainId;
                                proxy._trigger("actionComplete", { data: editedApp, requestType: "appointmentEdit" });
                            });
                            promise.fail(function (e) {
                            });
                        }
                        else {
                            if (!(editedApp[0][this.model.appointmentSettings["recurrence"]] || (new Date(new Date(editedApp[0][this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(editedApp[0][this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((new Date(editedApp[0][this.model.appointmentSettings["endTime"]]) - new Date(editedApp[0][this.model.appointmentSettings["startTime"]])) / 3600000) < 24) && (editedApp[0][this.model.appointmentSettings["allDay"]] !== true)))
                                new ej.DataManager(this._currentAppointmentData).update("AppTaskId", editedApp[0]);
                            dele = new ej.DataManager(this._processed).executeLocal(ej.Query().where("ParentId", ej.FilterOperators.equal, editedApp[0]["AppTaskId"]));
                            dele.length == 0 ? new ej.DataManager(this._processed).update("AppTaskId", editedApp[0]) : "";
                            for (var j = 0; j < dele.length; j++) {
                                new ej.DataManager(this._processed).remove("Guid", dele[j]["Guid"]);
                                this.element.find('[guid=' + dele[j]["Guid"] + ']').remove();
                                this.element.find("#categorybar_" + dele[j]["Guid"]).remove();
                            }
                            this.element.find('[guid=' + editedApp[0]["Guid"] + ']').remove();
                            if (dele.length != 0)
                                this._appointmentProcessing(editedApp[0]);
                            this._renderSingleApp(editedApp[0], editedApp[0]);
                            this._appMainId = this._processed.length > 0 ? this._sortAppById(this._processed)[this._processed.length - 1]["AppTaskId"] : this._appMainId;
                        }
                        this._trigger("actionComplete", { data: editedApp, requestType: "appointmentEdit", methodType: "public" });
                    }
                    if (newApp.length > 0) {
                        if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                            promise.done(function (e) {
                                if (proxy.model.enableLoadOnDemand)
                                    proxy._initDataSource("refresh");
                                else {
                                    var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                    proxy._currentAppointmentData.length = 0;
                                    proxy._dateFormat(value);
                                    proxy._bindAppointments = true;
                                    proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                    proxy._dataProcessing(value);
                                    proxy._renderAppointmentAll();
                                }
                            });
                            promise.fail(function (e) {
                            });
                        }
                        else {
                            for (var i = 0; i < newApp.length; i++) {
                                this._appointmentProcessing(newApp[i]);
                                !(this.dataSource() instanceof ej.DataManager) && this._renderSingleApp(newApp[i], newApp[i]);
                            }
                        }
                    }
                }
                else {
                    if (this._trigger("actionBegin", { data: obj, requestType: "appointmentEdit", methodType: "public" }))
                        return false;
                    if (this._trigger("appointmentEdited", { appointment: obj, methodType: "public" }))
                        return false;
                    if (this.dataSource() instanceof ej.DataManager) {
                        var promise = this._dataManager.update(this.model.appointmentSettings["id"], temp);
                    }
                    else
                        var promise = this._dataManager.update("AppTaskId", temp);
                    var updateTemp = this._timeZoneAppointments(temp, "reverse");
                    new ej.DataManager(this._currentAppointmentData).update("AppTaskId", updateTemp);
                    var id = this._appId == 1 ? obj["AppTaskId"] : this._appId;
                    if ($.isFunction(promise.promise) && this.dataSource() instanceof ej.DataManager) {
                        promise.done(function (e) {
                            if (proxy.model.enableLoadOnDemand)
                                proxy._initDataSource("refresh");
                            else {
                                var value = (ej.isNullOrUndefined(e.record)) ? e : e.record;
                                proxy._currentAppointmentData.length = 0;
                                proxy._dateFormat(value);
                                proxy._bindAppointments = true;
                                proxy._currentAppointmentData = $.merge(proxy._currentAppointmentData, value);
                                proxy._dataProcessing(value);
                                proxy._renderAppointmentAll();
                            }
                            proxy._trigger("actionComplete", { data: e, requestType: "appointmentEdit", methodType: "public" });
                        });
                    }
                    else {
                        if (!(obj[this.model.appointmentSettings["recurrence"]] || (new Date(new Date(obj[this.model.appointmentSettings["startTime"]]).setHours(0, 0, 0, 0)).getTime() < new Date(new Date(obj[this.model.appointmentSettings["endTime"]]).setHours(0, 0, 0, 0)).getTime()) && (((new Date(obj[this.model.appointmentSettings["endTime"]]) - new Date(obj[this.model.appointmentSettings["startTime"]])) / 3600000) < 24) && (obj[this.model.appointmentSettings["allDay"]] !== true)))
                            new ej.DataManager(this._processed).update("AppTaskId", updateTemp) && this.element.find('[guid=' + this._appUid + ']').remove() && this.element.find("#categorybar_" + this._appUid).remove();
                        this._renderSingleApp(updateTemp, updateTemp);
                        this._trigger("actionComplete", { data: updateTemp, requestType: "appointmentEdit", methodType: "public" });
                    }
                }
            }
        },

        print: function () {
            var cloneElement = this.element.clone();
            if (this.currentView() == "agenda") {
                cloneElement.width(this.element.find('.e-agendacellstab').width());
                cloneElement.height($("#" + this._id + " .e-headertr").height() + this.element.find('.e-agendacellstab').height() + $("#" + this._id + " .e-scheduleheader").height() + 2);
                $(cloneElement.find('.e-scroller')).ejScroller({ width: this.element.find('.e-agendacellstab').width(), height: $("#" + this._id + " .e-agendacellstab").height() + 2 });
                $(cloneElement).find('.e-commonviewbutton').css({ 'margin-top': '0px' });
                $(cloneElement).find('.e-viewstable').css({ 'padding-top': '15px' });
            }
            else {
                if (this.model.orientation == "vertical") {
                    cloneElement.width(this.element.find('.e-workcellstab').width() + $('.e-workleftindent').width());
                    cloneElement.height($("#" + this._id + " .e-headertr").height() + $("#" + this._id + " .e-workcellstab").height() + $("#" + this._id + " .e-scheduleheader").height() + 2);
                    $(cloneElement.find('.e-scroller')).ejScroller({ width: this.element.find('.e-workcellstab').width(), height: $("#" + this._id + " .e-workcellstab").height() + 2 });
                    $(cloneElement).find('.e-scrolltimecells').css({ top: 0 });
                    $(cloneElement).find('.e-commonviewbutton').css({ 'margin-top': '0px' });
                    $(cloneElement).find('.e-viewstable').css({ 'padding-top': '15px' });
                    $(cloneElement).children().first().css({ border: "0px solid #bbbcbb" });
                    cloneElement.css({ border: "1px solid #bbbcbb" });
                    if (this.model.currentView == "month")
                        $(cloneElement).find('.e-draggableworkarea').css({ 'height': $(this.element).find('.e-timecells').height() * $(this.element).find('.e-timecells').length + 2 });
                }
                else {
                    cloneElement.width($(this.element).find('.e-workcellstab').width());
                    $(cloneElement.find('.e-scroller')).ejScroller({ width: $(this.element).find('.e-workcellstab').width() });
                    $(cloneElement).find('.e-headerdays').find('tr:first').next().css({ left: "0px" });
                    $(cloneElement).find('.e-commonviewbutton').css({ 'margin-top': '0px' });
                    $(cloneElement).find('.e-viewstable').css({ 'padding-top': '15px' });
                    $(cloneElement).find('.e-draggableworkarea').css({ 'height': $(this.element).height() - $(this.element).find('.e-scheduleheader').height() + $(this.element).find('.e-scheduleheader').next().height() });
                }
            }
            ej.print(cloneElement);
        },

        deleteAppointment: function (id) {
            if (!this.model.readOnly)
                this._deleteApp(id);
        },

        getAppointments: function () {
            var appList = this.dataSource();
            return appList;
        },

        searchAppointments: function (searchString, fields, filterOperator, ignoreCase) {
            var queryManagar = ej.Query();
            var result, _fields = ej.isNullOrUndefined(fields) ? this._getColumns(this.model.appointmentSettings) : fields;
            filterOperator = ej.isNullOrUndefined(filterOperator) ? "contains" : filterOperator;
            ignoreCase = ej.isNullOrUndefined(ignoreCase) ? true : ignoreCase;
            if (typeof searchString == "string") {
                if (searchString.length) {
                    queryManagar.search(searchString, _fields, filterOperator, ignoreCase);
                    if ((this.dataSource() instanceof ej.DataManager)) {
                        result = queryManagar;
                    }
                    else
                        result = this._dataManager.executeLocal(queryManagar);
                }
            }
            else if (typeof searchString == "object")
                result = this.filterAppointments(searchString);
            return result;
        },

        filterAppointments: function (filterConditions) {
            var result, queryManagar = ej.Query(), matchcaseVal;
            if (!ej.isNullOrUndefined(filterConditions)) {
                var firstFilterCondition = filterConditions[0];
                matchcaseVal = ej.isNullOrUndefined(firstFilterCondition.matchcase) ? true : firstFilterCondition.matchcase;
                var predicate = ej.Predicate(firstFilterCondition.field, firstFilterCondition.operator, firstFilterCondition.value, matchcaseVal);
                for (var i = 1; i < filterConditions.length; i++) {
                    matchcaseVal = ej.isNullOrUndefined(filterConditions[i].matchcase) ? true : filterConditions[i].matchcase;
                    predicate = predicate[filterConditions[i].predicate](filterConditions[i].field, filterConditions[i].operator, filterConditions[i].value, matchcaseVal);
                }
                queryManagar.where(predicate);
                if ((this.dataSource() instanceof ej.DataManager)) {
                    result = queryManagar;
                }
                else
                    result = this._dataManager.executeLocal(queryManagar);
            }
            return result;
        },

        refreshScroller: function () {
            var scrollerObj = this.element.find(".e-scroller").data("ejScroller");
            var scrollWidth = (this.model.orientation === "vertical") ? this.element.width() - (this._mediaQuery && this.currentView() == "month" ? "" : this.element.find('.e-scrolltimecells').parent().outerWidth()) : (this.model.orientation == "horizontal") && (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? this.element.width() - this.element.find(".e-resourceheadertable").width() : this.element.width();
            scrollerObj.model.width = scrollWidth - 2;
            scrollerObj.model.height = this.scHeight - 1 - (this.element.find('.e-draggableworkarea').offset().top - this.element.offset().top);
            scrollerObj.refresh();
            if (this.model.orientation == "vertical") {
                if (this.model.cellWidth) {
                    (this.currentView() == "day") && this.element.find(".e-headerdaydisplay").css("width", this.model.cellWidth);
                    var cols = this.currentView() == "workweek" ? this.model.workWeek.length : this.currentView() == "day" ? 1 : (this.currentView() === "customview" && this._renderDays <= 7) ? this._dateRender.length : 7;
                    var col = (!ej.isNullOrUndefined(this.model.group) && !ej.isNullOrUndefined(this.model.resources)) ? (this.res1.length * cols) : cols;
                    var rows = this._dateRender.length / 7;
                    var overallWidth = parseInt(this.model.cellWidth) * col;
                    var overallHeight = parseInt(this.model.cellHeight) * rows;
                    this.element.find(".e-draggableworkarea").width((this.element.width() > overallWidth + this.element.find(".e-scrolltimecells").width() + 18) ? overallWidth : scrollerObj.model.width - 18);
                }
                this.element.find(".e-headrealldaytable").parent().width(this.element.find(".e-draggableworkarea").width());
                this.element.find(".e-datecommondiv").parent().width(this.element.width()-2);
                this.element.find(".e-headrealldaytable").css("left", -(this.element.find(".e-draggableworkarea").scrollLeft()) + "px");
            }
            else {
                $(this.element.find(".e-horizontaltimecells").find("table tr")[2]).css("left", -(scrollerObj.model.scrollLeft) + "px");
                this.element.find(".e-horizontresdiv").height(this.element.find('.e-draggableworkarea').height());
            }
            if (this.currentView() == "month" && this.model.orientation == "vertical")
                this._renderCurrentView();
        },

        exportSchedule: function (action, serverEvent, id) {
            var modelClone = JSON.parse(JSON.stringify(this.model));
            var appointmentClone = JSON.parse(JSON.stringify(this.model.appointmentSettings));
            var attr = { action: action, method: 'post' };
            var form = ej.buildTag('form', "", null, attr);
            if (this._ignoreFromAppointmentOnExport) {
                for (var i = 0; i < this._ignoreFromAppointmentOnExport.length; i++) {
                    delete appointmentClone[this._ignoreFromAppointmentOnExport[i]];
                }
            }
            if (ej.raiseWebFormsServerEvents) {
                var args = { model: modelClone, originalEventType: serverEvent };
                var clientArgs = { model: JSON.stringify(appointmentClone), appId: id };
                ej.raiseWebFormsServerEvents(serverEvent, args, clientArgs);
            }
            else {
                var inputAttr = { name: 'ScheduleModel', type: 'hidden', value: JSON.stringify(appointmentClone) }
                var input = ej.buildTag('input', "", null, inputAttr);
                if (!ej.isNullOrUndefined(id)) {
                    var idAttr = { name: 'AppointmentId', type: 'hidden', value: id }
                    var inputId = ej.buildTag('input', "", null, idAttr);
                    form.append(inputId);
                }
                form.append(input);
                form.append(this);
                $('body').append(form);
                form.submit();
            }
            return true;
        }
    });

    if (ej.scheduleFeatures.resources)
        $.extend(ej.Schedule.prototype, ej.scheduleFeatures.resources);
    if (ej.scheduleFeatures.horizontal)
        $.extend(ej.Schedule.prototype, ej.scheduleFeatures.horizontal);

    ej.Schedule.Locale = {};
    ej.Schedule.Locale["en-US"] = {
        ReminderWindowTitle: "Reminder window",
        CreateAppointmentTitle: "Create Appointment",
        RecurrenceEditTitle: "Edit Repeat Appointment",
        RecurrenceEditMessage: "How would you like to change the appointment in the series?",
        RecurrenceEditOnly: "Only this appointment",
        RecurrenceEditSeries: "Entire series",
        PreviousAppointment: "Previous Appointment",
        NextAppointment: "Next Appointment",
        AppointmentSubject: "Subject",
        StartTime: "Start Time",
        EndTime: "End Time",
        AllDay: "All day",
        StartTimeZone: "Start TimeZone",
        EndTimeZone: "End TimeZone",
        Today: "Today",
        Recurrence: "Repeat",
        Done: "Done",
        Cancel: "Cancel",
        Ok: "Ok",
        RepeatBy: "Repeat by",
        RepeatEvery: "Repeat every",
        RepeatOn: "Repeat on",
        StartsOn: "Starts on",
        Ends: "Ends",
        Summary: "Summary",
        Daily: "Daily",
        Weekly: "Weekly",
        Monthly: "Monthly",
        Yearly: "Yearly",
        Every: "Every",
        EveryWeekDay: "Every weekday",
        Never: "Never",
        After: "After",
        Occurence: "Occurrence(s)",
        On: "On",
        Edit: "Edit",
        RecurrenceDay: "Day(s)",
        RecurrenceWeek: "Week(s)",
        RecurrenceMonth: "Month(s)",
        RecurrenceYear: "Year(s)",
        The: "The",
        OfEvery: "of every",
        First: "First",
        Second: "Second",
        Third: "Third",
        Fourth: "Fourth",
        Last: "Last",
        WeekDay: "Weekday",
        WeekEndDay: "Weekendday",
        Subject: "Subject",
        Categorize: "Categories",
        DueIn: "Due In",
        DismissAll: "Dismiss All",
        Dismiss: "Dismiss",
        OpenItem: "Open Item",
        Snooze: "Snooze",
        Day: "Day",
        Week: "Week",
        WorkWeek: "Work Week",
        Month: "Month",
        AddEvent: "Add Event",
        CustomView: "Custom View",
        Agenda: "Agenda",
        Detailed: "Edit Appointment",
        EventBeginsin: "Appointment Begins in",
        Editevent: "Edit Appointment",
        Editseries: "Edit Series",
        Times: "times",
        Until: "until",
        Eventwas: "Appointment was",
        Hours: "hrs",
        Minutes: "mins",
        Overdue: "Overdue Appointment",
        Days: "day(s)",
        Event: "Event",
        Select: "select",
        Previous: "prev",
        Next: "next",
        Close: "close",
        Delete: "Delete",
        Date: "Date",
        Showin: "Show in",
        Gotodate: "Go To Date",
        Resources: "RESOURCES",
        RecurrenceDeleteTitle: "Delete Repeat Appointment",
        Location: "Location",
        Priority: "Priority",
        RecurrenceAlert: "Alert",
        NoTitle: "No Title",
        WrongPattern: "The recurrence pattern is not valid",
        CreateError: "The duration of the appointment must be shorter than how frequently it occurs. Shorten the duration, or change the recurrence pattern in the Appointment Recurrence dialog box.",
        DragResizeError: "Cannot reschedule an occurrence of the recurring appointment if it skips over a later occurrence of the same appointment.",
        StartEndError: "End time should be greater than start time",
        MouseOverDeleteTitle: "Delete Appointment",
        DeleteConfirmation: "Are you sure you want to delete this appointment?",
        Time: "Time"
    };

    ej.Schedule.Actions = {
        Add: "add",
        Save: "save",
        Delete: "delete",
        EditOcurrence: "editOcurrence",
        EditSeries: "editSeries"
    };

    ej.Schedule.CurrentView = {
        Day: "day",
        Week: "week",
        Workweek: "workweek",
        Month: "month",
        CustomView: "customview",
        Agenda: "agenda"
    };

    ej.Schedule.TimeMode = {
        Hour12: "12",
        Hour24: "24"
    };

    ej.Schedule.Orientation = {
        Vertical: "vertical",
        Horizontal: "horizontal"
    };
    ej.Schedule.DayOfWeek = {
        Sunday: "Sunday",
        Monday: "Monday",
        Tuesday: "Tuesday",
        Wednesday: "Wednesday",
        Thursday: "Thursday",
        Friday: "Friday",
        Saturday: "Saturday"
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to create the Gantt
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
   
    ej.widget("ejGantt", "ej.Gantt", {

        // widget element will be automatically set in this
        _rootCSS: "e-gantt",

        element: null,

        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,
        _tags: [{
            tag: "holidays",
            attr: ["day", "background", "label"],
            content: "template"
        },
        {
            tag: "stripLines",
            attr: ["day", "label", "lineStyle", "lineColor", "lineWidth"],
            content: "template"
        },
         {
             tag: "editDialogFields",
             attr: ["field", "editType"],
             content: "template"
         },
         {
             tag: "addDialogFields",
             attr: ["field", "editType"],
             content: "template"
         },
         {
            tag: "sortSettings.sortedColumns",
            attr: ["field","direction"]
         }
        ],
        _holidays: function (index, property, value, old) {
            this._$ganttchartHelper.ejGanttChart("refreshHolidays");
            this._trigger("refresh");
        },
        _stripLines: function (index, property, value, old) {
            this._$ganttchartHelper.ejGanttChart("refreshStripLines");
            this._trigger("refresh");
        },
        _editDialogFields: function (index, property, value, old) {
            this._trigger("refresh");
        },
        _addDialogFields: function (index, property, value, old) {
            this._trigger("refresh");
        },
        defaults: {

            allowSorting: false,
            allowColumnResize: false,
            allowSelection: true,
            dataSource: null,
            showColumnChooser: false,
            query: null,
            splitterPosition: null,
            taskIdMapping: "",
            parentTaskIdMapping: "",
            taskNameMapping: "",
            startDateMapping: "",
            endDateMapping: "",
            baselineStartDateMapping: "",
            baselineEndDateMapping: "",
            childMapping: "",
            durationMapping: "",
            milestoneMapping: "",
            progressMapping: "Progress",
            predecessorMapping: "",
            resourceInfoMapping: "",           
            resources: [],
            holidays: [],
            highlightWeekends: true,
            scheduleStartDate: null,
            scheduleEndDate: null,
            enableProgressBarResizing: true,
            rowHeight: 30,
            includeWeekend: true,
            toolbarSettings: {
                showToolbar: false,
                toolbarItems: []
            },
            stripLines: [],
            workingTimeScale: 'TimeScale8Hours',
            roundOffDayworkingTime: true,
            durationUnit: 'day',
            scheduleHeaderSettings:
            {
                weekHeaderFormat: "MMM dd , yyyy",
                dayHeaderFormat: "ddd",
                yearHeaderFormat: "yyyy",
                monthHeaderFormat: "MMM",
                hourHeaderFormat: 'HH',
                scheduleHeaderType: 'week',
                minutesPerInterval: "auto",
                weekendBackground: ''
            },
            taskbarBackground: "",
            progressbarBackground: "",
            connectorLineBackground: "",
            parentTaskbarBackground: "",
            parentProgressbarBackground: "",
            connectorlineWidth: 1,
            showTaskNames: true,
            showGridCellTooltip: false,
            showGridExpandCellTooltip: false,
            showProgressStatus: true,
            showResourceNames: true,
            enableTaskbarDragTooltip: true,
            enableTaskbarTooltip: true,
            editSettings: {
                allowEditing: false,
                allowAdding: false,
                allowDeleting: false,
                editMode: "normal",
            },
            flatRecords: [],
            parentRecords: [],
            currentViewData: [],
            updatedRecords: [],
            ids: [],
            allowKeyboardNavigation: true,
            cssClass: "",
            locale: "en-US",
            allowMultiSorting: false,
            sortSettings: {
                sortedColumns: []
            },
            enableAltRow: true,
            enableVirtualization: false,
            progressbarHeight: 100,
            taskbarTooltipTemplate: null,
            progressbarTooltipTemplate: null,
            taskbarTooltipTemplateId: null,
            dateFormat: "",
            resourceIdMapping: "",
            resourceNameMapping: "",
            progressbarTooltipTemplateId: null,
            taskbarEditingTooltipTemplateId: null,
            taskbarEditingTooltipTemplate: null,
            selectedRowIndex: -1,
            allowGanttChartEditing: true,
            sizeSettings: {
                height: null,
                width: null,
            },
            selectedItem: null,
            weekendBackground: '#F2F2F2',
            baselineColor: '#fba41c',
            treeColumnIndex: 0,
            editDialogFields: [],
            addDialogFields: [],
            renderBaseline: false,
            enableContextMenu: false,
            enableResize: false,
            enableCollapseAll: false,
            rowSelecting: null,
            rowSelected: null,
            queryCellInfo: null,
            queryTaskbarInfo: null,            
            beginEdit: null,
            endEdit: null,
            rowDataBound: null,
            expanding: null,
            expanded: null,
            collapsing: null,
            collapsed: null,
            actionBegin: null,
            actionComplete: null,
            taskbarEditing: null,
            taskbarEdited: null,
            load: null,
            contextMenuOpen: null,
        },

        dataTypes: {
            allowSorting: "boolean",
            allowColumnResize: "boolean",
            allowSelection: "boolean",
            dataSource: "data", //inline or remote data
            query: "data",
            taskIdMapping: "string",
            taskNameMapping: "string",
            startDateMapping: "string",
            endDateMapping: "string",
            baselineStartDateMapping: "string",
            baselineEndDateMapping: "string",
            childMapping: "string",
            durationMapping: "string",
            milestoneMapping: "string",
            progressMapping: "string",
            predecessorMapping: "string",
            resourceInfoMapping: "string",
            resources: "array",
            holidays: "array",
            highlightWeekends: "boolean",
            enableProgressBarResizing: "boolean",
            rowHeight: "number",
            includeWeekend: "boolean",
            toolbarSettings: {
                showToolbar: "boolean",
                toolbarItems: "array"
            },
            stripLines: "array",
            scheduleHeaderSettings: "data",
            taskbarBackground: "string",
            progressbarBackground: "string",
            connectorLineBackground: "string",
            parentTaskbarBackground: "string",
            parentProgressbarBackground: "string",
            connectorlineWidth: "number",
            showTaskNames: "boolean",
            showProgressStatus: "boolean",
            showResourceNames: "boolean",
            enableTaskbarDragTooltip: "boolean",
            enableTaskbarTooltip: "boolean",
            editSettings: "data",
            flatRecords: "array",
            parentRecords: "array",
            currentViewData: "array",
            ids: "array",
            updatedRecords: "array",
            allowKeyboardNavigation: "boolean",
            cssClass: "string",
            locale: "string",
            allowMultiSorting: "boolean",
            showGridExpandCellTooltip: "boolean",
            sortSettings: {
                sortedColumns: "array"
            },
            enableAltRow: "boolean",
            enableVirtualization: "boolean",
            progressbarHeight: "number",
            dateFormat: "string",
            resourceIdMapping: "string",
            resourceNameMapping: "string",
          //  selectedRowIndex: "number",
            allowGanttChartEditing: "boolean",
            size: {
                height: "string",
                width: "string"
            },
            selectedItem: "data",
            weekendBackground: "string",
            baselineColor: "string",
            renderBaseline: "boolean",
            showGridCellTooltip: "boolean",
            treeColumnIndex: "number",
            editDialogFields: "array",
            addDialogFields:"array",
            enableResize: "boolean"
        },

        observables: ["selectedItem", "selectedRowIndex", "splitterPosition", "dataSource"],
        selectedItem: ej.util.valueFunction("selectedItem"),
        selectedRowIndex: ej.util.valueFunction("selectedRowIndex"),
        splitterPosition : ej.util.valueFunction("splitterPosition"),
        dataSource: ej.util.valueFunction("dataSource"),

        //Retruns column value
        getColumns: function() {
            return this._columns;
        },
        //set column value
        setColumns: function(value) {
            this._columns = value;
        },

        // Initialize values to header Text and Gantt records for custom columns alone
        customColumnFields: function () {
            var proxy = this, model = proxy.model,
                totalColumns = proxy._columns, totalColumnsLength = totalColumns.length;

            for (var i = 0; i < totalColumnsLength; i++)
            {
                if (!proxy._columnHeaderTexts[totalColumns[i].field]) {
                    proxy._columnHeaderTexts[totalColumns[i].field] = totalColumns[i].headerText;
                }
            }           
        },

        //Control Rendering is initiated
        _init: function () {
            var proxy = this,
                model = proxy.model, startDate, endDate,interval = model.scheduleHeaderSettings.minutesPerInterval,
                durationUnit = model.durationUnit;

            //Default data source for ASP designer
            if (model.isdesignMode) {
                model.taskIdMapping = "Id";
                model.taskNameMapping = "Name";
                model.startDateMapping = "StartDate";
                model.durationMapping = "Duration";
                model.progressMapping = "Progress";
                model.scheduleStartDate = "02/23/2014";
                model.scheduleEndDate = "03/10/2014";
                this.dataSource([{ Id: 1, Name: "Task 1", StartDate: "02/23/2014", Duration: 5, Progress: 40 },
                { Id: 2, Name: "Task 2", StartDate: "02/24/2014", Duration: 5, Progress: 40 },
                { Id: 3, Name: "Task 3", StartDate: "02/25/2014", Duration: 5, Progress: 40 }]);
                model.sizeSettings.height = "300px";
				model.sizeSettings.width = "800px";
            }

            if (!(model.scheduleStartDate || model.scheduleEndDate) && interval == "auto")
                model.scheduleHeaderSettings.minutesPerInterval = ej.Gantt.minutesPerInterval.FiveMinutes;
            proxy._initPrivateProperties();
            //Load event triggered
            proxy._trigger("load");
            proxy.element.addClass("e-gantt-core");

            if (model.durationUnit == "hours") {
                model.durationUnit = "hour";
            }
            if (model.scheduleHeaderSettings.scheduleHeaderType == "hour") {
                if (durationUnit != "hour")
                    model.durationUnit = "minute";
            }
            if (this.dataSource() !== null) {
                this._checkDataBinding();
            }
            else {
                proxy._initialize();
            }
        },

        //clear selected item
        _deSelectRowItem: function () {
            var proxy = this, model = proxy.model;
            proxy._$ganttchartHelper.ejGanttChart("selectRows", -1);
            proxy._$treegridHelper.ejTreeGrid("selectRows", -1);
            this.selectedRowIndex(-1);
            model.selectedItem = null;
        },
        
        //populate gantt records from data source
        _checkDataBinding: function () {
            var proxy = this, model = proxy.model;

            if (this.dataSource() instanceof ej.DataManager) {
                this._initDataSource();
            }
            else if (this.dataSource().length > 0) {
                if ((model.taskIdMapping.length > 0) && (model.parentTaskIdMapping.length > 0)) {

                    //cloning the datasource
                    var dataArray = this.dataSource().slice(0);
                    var data = [];
                    $.each(dataArray, function (indx, value) {
                        data.push(jQuery.extend(true, {}, value));
                    });
                    proxy._reconstructDatasource(data);
                    proxy._createGanttRecords(proxy.secondaryDatasource);
                }
                else {
                    proxy._createGanttRecords(this.dataSource());
                }
                proxy._initialize();
            }
            else {
                proxy._initialize();
            }
        },

        //Create gantt records and render gantt if data source is remote data
        _initDataSource: function () {
            var proxy = this, model = proxy.model;     
            var query = this._columnToSelect();
            var proxy = this;
            var queryPromise = this.dataSource().executeQuery(query);
            queryPromise.done(ej.proxy(function (e) {
                if (proxy.dataSource().dataSource.offline) {
                    proxy._retrivedData = proxy.dataSource().dataSource.json;
                } else {
                    proxy._retrivedData = e.result;
                }
                if ((model.taskIdMapping.length > 0) && (model.parentTaskIdMapping.length > 0)) {
                    //cloning the datasource
                    var dataArray = e.result.slice(0);
                    var data = [];
                    $.each(dataArray, function (indx, value) {
                        data.push(jQuery.extend(true, {}, value));
                    });
                    proxy._reconstructDatasource(data);
                }
                else {
                    proxy._reconstructDatasource(e.result);
                }
                proxy._createGanttRecords(proxy.secondaryDatasource);
                proxy._initialize();
            }));
        },

        //create ejQuery when remote data is initiated
        _columnToSelect: function () {
            var proxy = this,
                model = proxy.model,
                column = [], queryManager = ej.Query();

            queryManager = model.query ? model.query : queryManager;
            return queryManager;
        },

        //If parentIdMapping is given data source is changed in hierarchical order
        _reconstructDatasource: function (datasource) {
            var proxy = this, model = proxy.model,
                childItems, data,tempRecord=[];
            
            for (var i = 0; i < datasource.length; i++) {
                var data = {};
                data = ej.DataManager(datasource).executeLocal(ej.Query().where(model.parentTaskIdMapping,
                    ej.FilterOperators.equal,
                    datasource[i][model.taskIdMapping]));
               

                if ((ej.isNullOrUndefined(datasource[i][model.parentTaskIdMapping]) || datasource[i][model.parentTaskIdMapping] == 0)
                    && data.length > 0) {
                    datasource[i][model.childMapping] = data;
                    proxy.secondaryDatasource.push(datasource[i]);
                }
                else if (ej.isNullOrUndefined(datasource[i][model.parentTaskIdMapping]) && !(data && data.length > 0)) {
                    proxy.secondaryDatasource.push(datasource[i]);
                }

                else if (data && data.length > 0 && !(ej.isNullOrUndefined(data[0][model.parentTaskIdMapping]))) {
                    proxy._appendChildDataItems(proxy.secondaryDatasource, data);
                }
                else if ((ej.isNullOrUndefined(datasource[i][model.parentTaskIdMapping]) || datasource[i][model.parentTaskIdMapping] == 0)
                    && data.length == 0) {
                    proxy.secondaryDatasource.push(datasource[i]);
                }

                if (data && data.length > 0) {

                    proxy._datasourceChildItems = proxy._reconstructDatasource(data);
                    if (ej.isNullOrUndefined(datasource[i][model.childMapping])) {
                        datasource[i][model.childMapping] = proxy._datasourceChildItems;
                        proxy._getParentIndex(datasource);
                    }
                    proxy._datasourceChildItems = [];

                }
                
                else if (!(ej.isNullOrUndefined(datasource[i][model.parentTaskIdMapping]))) {
                    tempRecord.push(datasource[i]);
                }
            }

            if (tempRecord && tempRecord.length > 0) {
                return tempRecord;
            }
        },

        //add child item in parent record in reference data source
        _appendChildDataItems: function (parentDatasource, data) {
            var proxy = this,
                model = proxy.model;

            for (var count = 0; count < parentDatasource.length; count++) {

                if (parentDatasource[count][model.taskIdMapping] === data[0][model.parentTaskIdMapping]) {
                    parentDatasource[count][model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[count][model.childMapping])) &&
                    parentDatasource[count][model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[count], data);
                }
            }

            if ((ej.isNullOrUndefined(parentDatasource.length)) && parentDatasource) {
                if (parentDatasource[model.taskIdMapping] === data[0][model.parentTaskIdMapping]) {
                    parentDatasource[model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[model.childMapping])) &&
                    parentDatasource[model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[model.childMapping], data);
                }
            }

        },
        //Get corresponding parent index while reconstrunting referencing data source
        _getParentIndex: function (datasource) {
            var proxy = this, model = proxy.model,
                dataArr = proxy.secondaryDatasource, parentIndex = 0;

            if (datasource.length > 0) {
                parentIndex = datasource[0][model.parentTaskIdMapping];
            }
            for (var i = 0; i < dataArr.length; i++) {
                if (dataArr[i][model.taskIdMapping] == parentIndex && ej.isNullOrUndefined(dataArr[i][model.childMapping])) {
                    dataArr[i][model.childMapping] = (datasource);
                    return true;
                }
                else {
                    var childItems = dataArr[i][model.childMapping];
                    if (childItems) {
                        for (var i = 0; i < childItems.length; i++) {
                            if (childItems[i][model.taskIdMapping] == parentIndex && ej.isNullOrUndefined(childItems[i][model.childMapping])) {
                                childItems[i][model.childMapping] = (datasource);
                                return true;
                            }
                        }
                    }
                }
            }

        },

        //refresh the chart with updated schedule dates
        updateScheduleDates: function (startDate, endDate) {
            
            var proxy = this, model = this.model;
                model.scheduleStartDate = startDate;
                model.scheduleEndDate = endDate;
                proxy._scheduleDays = [];
                proxy._scheduleMonths = [];
                proxy._scheduleWeeks = [];
                proxy._scheduleYears = [];
                proxy._scheduleHours = [];
                proxy._scheduleMinutes = [];
                proxy._calculateHeaderDates();
                var chartObject = $("#ejGanttChart" + proxy._id).ejGanttChart("instance");
                var chartModel = chartObject.model;
                /* No need to re render parent item , because all the records are rerendered*/
                proxy._isTreeGridRendered = false;
                proxy._isGanttChartRendered = false;
                proxy._updateGanttRecords();
                chartModel.projectStartDate = proxy._getDateFromFormat(proxy._projectStartDate);
                chartModel.projectEndDate = proxy._getDateFromFormat(proxy._projectEndDate);
                chartModel.scheduleDays = proxy._scheduleDays;
                chartModel.scheduleMonths = proxy._scheduleMonths;
                chartModel.scheduleWeeks = proxy._scheduleWeeks;
                chartModel.scheduleYears = proxy._scheduleYears;
                chartModel.scheduleHours = proxy._scheduleHours;
                chartModel.scheduleMinutes = proxy._scheduleMinutes;
                chartModel.renderBaseline = model.renderBaseline;
                $("#ejGanttChart" + proxy._id).ejGanttChart("refreshChartHeader", proxy._getDateFromFormat(startDate), proxy._getDateFromFormat(endDate));
                $("#ejGanttChart" + proxy._id).ejGanttChart("refreshContainersWidth");
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedRecords, proxy._totalCollapseRecordCount);
                if (model.predecessorMapping) {
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._isValidationEnabled = false;
                    proxy._createConnectorLinesCollection();
                    proxy._isValidationEnabled = true;
                }
                chartObject._$bodyContainer.ejScroller("refresh");

                proxy._isTreeGridRendered = true;
                proxy._isGanttChartRendered = true;
        },
        //used to change the schedule mode dynamically
        
        reRenderChart: function (scheduleMode) {
            var proxy = this, model = proxy.model,
                startDate = proxy._getDateFromFormat(model.scheduleStartDate),
                endDate = proxy._getDateFromFormat(model.scheduleEndDate);
            model.flatRecords = [];
            model.parentRecords = [];
            $("#ejGanttChart" + proxy._id).empty();

            switch (scheduleMode) {
                case ej.Gantt.ScheduleHeaderType.Week:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Week;
                    proxy._perDayWidth = 30;
                    proxy._calculateWeekSplit(startDate, endDate);
                    break;

                case ej.Gantt.ScheduleHeaderType.Day:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Day;
                    model.scheduleHeaderSettings.dayHeaderFormat = "dd, MMM yy";
                    proxy._perHourWidth = 20;
                    if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                        proxy._perDayWidth = (proxy._perHourWidth * 24);//24 hours
                    else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours)
                        proxy._perDayWidth = (proxy._perHourWidth * 24);//9 hours
                    proxy._calculateDaySplit(startDate, endDate);
                    break;

                case ej.Gantt.ScheduleHeaderType.Month:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Month;
                    model.scheduleHeaderSettings.monthHeaderFormat = "MMM yyyy";
                    model.scheduleHeaderSettings.weekHeaderFormat = "M/dd";
                    proxy._perDayWidth = 10;
                    proxy._perWeekWidth = 70;
                    proxy._calculateMonthSplit(startDate, endDate);
                    break;

                case ej.Gantt.ScheduleHeaderType.Year:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Year;
                    proxy._perDayWidth = 3;
                    proxy._perMonthWidth = 90;
                    proxy._calculateYearSplit(startDate, endDate);
                    break;
                case ej.Gantt.ScheduleHeaderType.Hour:
                    model.scheduleHeaderSettings.scheduleHeaderType = ej.Gantt.ScheduleHeaderType.Hour;
                    var minutesPerInterval = model.scheduleHeaderSettings.minutesPerInterval,
                        intervalValue = ej.Gantt.minutesPerInterval;
                    proxy._perMinuteWidth = 20;
                    if (minutesPerInterval) {
                        if (minutesPerInterval == intervalValue.Auto) {
                            var twoDaysTimeSpan = 2 * 86400000,// Time in milliseconds
                                sevenDaysTimeSpan = 7 * 86400000,//Time in milliseconds
                                tenDaysTimeSpan = 10 * 86400000,
                                diff = endDate - startDate;

                            if (diff > tenDaysTimeSpan) {
                                proxy._minuteInterval = 30;
                                proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 2);//24 hours
                            }
                            else if (diff > sevenDaysTimeSpan) {
                                proxy._minuteInterval = 15;
                                proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 4);//24 hours
                            }
                            else if (diff > twoDaysTimeSpan) {
                                proxy._minuteInterval = 5;
                                proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 12);//24 hours
                            }
                            else {
                                proxy._minuteInterval = 1;
                                proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 60);//24 hours
                            }
                        }
                        else if (minutesPerInterval == intervalValue.OneMinute) {
                            proxy._minuteInterval = 1;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 60)
                        }
                        else if (minutesPerInterval == intervalValue.FiveMinutes) {
                            proxy._minuteInterval = 5;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 12);//24 hours
                        }
                        else if (minutesPerInterval == intervalValue.FifteenMinutes) {
                            proxy._minuteInterval = 15;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 4);//24 hours
                        }
                        else if (minutesPerInterval == intervalValue.ThirtyMinutes) {
                            proxy._minuteInterval = 30;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 2);//24 hours
                        }
                    }
                    proxy._calculateHourSplit(startDate, endDate);
                    break;
            }

            proxy.element.ejGantt("destroy").ejGantt(model)
            proxy._isTreeGridRendered = false;
            proxy._isGanttChartRendered = false;

        },
        //update the all gantt records width,progresswidth and left value
        _updateGanttRecords: function () {

            var proxy = this, model = proxy.model,
                flatRecords = model.flatRecords,
                length = flatRecords.length,
                currentRecord, parentItem, childRecords;
          
            for (var i = 0; i < length; i++) {
                currentRecord = flatRecords[i];
                if (currentRecord.hasChildRecords === false) {
                    currentRecord.left = currentRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidaysList, model);
                    currentRecord.width = currentRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                    currentRecord.width = currentRecord.isMilestone == true ? 26 : model.flatRecords[i].width;
                    currentRecord.progressWidth = currentRecord._calculateProgressWidth(currentRecord.width, currentRecord.status);
                }
                if (model.renderBaseline && currentRecord.baselineStartDate && currentRecord.baselineEndDate) {
                    currentRecord.baselineLeft = currentRecord._calculateBaselineLeft(proxy._projectStartDate, proxy._perDayWidth,
                        model.includeWeekend, model.dateFormat, model, model.endDateMapping, proxy._holidaysList);
                    currentRecord.baselineWidth = currentRecord._calculateBaseLineWidth(proxy._perDayWidth, model);
                }

                parentItem = currentRecord.parentItem;
                childRecords = parentItem && parentItem.childRecords;

                if (parentItem && childRecords.indexOf(currentRecord) == childRecords.length - 1 && currentRecord.hasChildRecords === false) {
                        proxy._updateParentItem(currentRecord);
                }
            }

        },
        
        //Method for initialize private peoperties
        _initPrivateProperties: function () {

            var proxy = this;
            model = proxy.model;
            scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;
            proxy._isFromGantt = true;
            proxy._disabledToolItems = [],
            proxy._id = proxy.element.attr('id'),
            proxy._scheduleWeeks = [],
            proxy._duplicate = false,          
            proxy._wrongenddate=false,
            proxy._scheduleYears = [],//new property for year schedule modes

            proxy._scheduleMonths = [], //new property for month

            proxy._scheduleYears = [],//property for schedule years
            proxy._scheduleDays = [],//property for schedule days
            proxy._scheduleHours=[],
            proxy._projectStartDate, //property instead of scheduleWeeks[0] and scheduleYears[0]
            proxy._projectEndDate,

            proxy._columns = [],
            proxy._storedIndex = -1,
            proxy._perDayWidth,
            proxy._prevDayWidth = 0,


            proxy._perMonthWidth,//new property for month
            proxy._perWeekWidth,//new property for month-week schedule

            proxy._$treegridPane = null,
            proxy._$ganttchartPane = null,
            proxy._$treegrid = null,
            proxy._$ganttchart = null,
            proxy._$ejGantt = null,
            proxy._$treegridHelper = null,
            proxy._$ganttchartHelper = null;
            proxy._tdsOffsetWidth = [];
            proxy._subPosx = [];
            proxy._predecessorsCollection = [];
            proxy._connectorLinesCollection = [];            
            proxy._updatedConnectorLineCollection = [];
            proxy._connectorlineIds = [];
            proxy._gridRows = null;

            //variables used for ExpandCollapse action in non Virtualization 
            proxy._collapsedRecordCount = 0;
            proxy._expandedRecordsCount = 0;
            proxy._totalCollapseRecordCount = 0;
            proxy._isInExpandCollapse = false;

            proxy._hiddenRecordCount = 0;
            
            proxy.prevPredecessorTO = null;
            proxy.prevPredecessorFrom = null;
            proxy.prevPredecessorcount = 0;
            proxy._isValidationEnabled = true;
            proxy._isTreeGridRendered = false;//indicate treegrid part is rendered or not for refresh row
            proxy._isGanttChartRendered = false;//indicate gantChart part is rendered or not
            proxy._holidaysList = proxy._getHoliday();
            proxy._stringHolidays = proxy._getStringHolidays();
            proxy._isinBeginEdit = false;
            proxy._isMileStoneEdited = false;
            proxy._cellEditColumn = null;
            proxy._isinAddnewRecord = false;
           
            //variables for context menu items
            proxy._contextMenuItems = null; //proxy._getContextMenuItems();
            proxy._subContextMenuItems = null;  //proxy._getsubContextMenuItems();
            proxy._totalBorderWidth = 2;//Left and Right border width

            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                proxy._totalBorderHeight = model.toolbarSettings.showToolbar ? 3 : 2;//top,bottom,toolbar div
            } else {
                proxy._totalBorderHeight = model.toolbarSettings.showToolbar ? 1 : 0;//top box-sizing:border-box includes it's border
            }
            //private variable decalartion for localization
            proxy._columnHeaderTexts = null;
            proxy._predecessorEditingTexts = null;
            proxy._editDialogTexts = null;
            proxy._toolboxTooltipTexts = null;
            proxy._durationUnitTexts = null;
            proxy._contextMenuTexts = null;
            proxy._columnMenuTexts = null;
            proxy._newTaskTexts = null;
            proxy._months = null;  
            proxy._days = null;

            /*alert text localization*/
            proxy._alertTexts = null;

            proxy._commonColumnWidth = 150;
            proxy._searchString = "",
            proxy._emptyDataColumns = [],
            proxy.secondaryDatasource = [];
            proxy._datasourceChildItems = [];
            proxy._$predecessorTable = null;
            proxy._mappingItems = [];
            proxy._editedDialogRecord = null;
            proxy._calculateDimensions();
            proxy._retrivedData = this.dataSource();
            proxy._isInExpandCollapseAll = false;
            proxy._preTableCollection = [];
            if (scheduleMode == "week") {
                proxy._perDayWidth = 30;
                proxy._perMinuteWidth = proxy._perDayWidth / (24 * 60);
            }
            else if (scheduleMode == "year") {
                proxy._perDayWidth = 3;
                proxy._perMonthWidth = proxy._perDayWidth * 30;
                proxy._perMinuteWidth = proxy._perDayWidth / (24 * 60);
            }
            else if (scheduleMode == "month") {
                proxy._perDayWidth = 10;
                proxy._perWeekWidth = proxy._perDayWidth * 7;
                proxy._perHourWidth = proxy._perDayWidth / 24;
                proxy._perMinuteWidth = proxy._perDayWidth / (24 * 60);
            }
            else if (scheduleMode == "day") {
                proxy._perHourWidth = 20;
                proxy._perMinuteWidth = proxy._perHourWidth / 60;
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                    proxy._perDayWidth = (proxy._perHourWidth * 24);//24 hours
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours)
                    proxy._perDayWidth = (proxy._perHourWidth * 24);//9 hours
            }
            else if (scheduleMode == "hour") {
                var minutesPerInterval = model.scheduleHeaderSettings.minutesPerInterval,
                    intervalValue = ej.Gantt.minutesPerInterval;
                proxy._perMinuteWidth = 20;
                if (minutesPerInterval) {
                    if (minutesPerInterval == intervalValue.Auto) {
                        var scheduleStartDate = proxy._getDateFromFormat(model.scheduleStartDate),
                            scheduleEndDate = proxy._getDateFromFormat(model.scheduleEndDate),
                            twoDaysTimeSpan = 2 * 86400000,// Time in milliseconds
                            sevenDaysTimeSpan = 7 * 86400000,//Time in milliseconds
                            tenDaysTimeSpan = 10 * 86400000,
                            diff = scheduleEndDate - scheduleStartDate;

                        if (diff > tenDaysTimeSpan) {
                            proxy._minuteInterval = 30;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 2);//24 hours
                        }
                        else if (diff > sevenDaysTimeSpan) {
                            proxy._minuteInterval = 15;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 4);//24 hours
                        }
                        else if (diff > twoDaysTimeSpan) {
                            proxy._minuteInterval = 5;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 12);//24 hours
                        }
                        else {
                            proxy._minuteInterval = 1;
                            proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 60);//24 hours
                        }
                    }
                    else if (minutesPerInterval == intervalValue.OneMinute) {
                        proxy._minuteInterval = 1;
                        proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 60)
                    }
                    else if (minutesPerInterval == intervalValue.FiveMinutes) {
                        proxy._minuteInterval = 5;
                        proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 12);//24 hours
                    }
                    else if (minutesPerInterval == intervalValue.FifteenMinutes) {
                        proxy._minuteInterval = 15;
                        proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 4);//24 hours
                    }
                    else if (minutesPerInterval == intervalValue.ThirtyMinutes) {
                        proxy._minuteInterval = 30;
                        proxy._perDayWidth = (proxy._perMinuteWidth * 24 * 2);//24 hours
                    }
                }                
            }
            //culture info assigned for private variables
            proxy._setCultureInfo();
            //Column collection populated
            proxy.setColumns(proxy.createTreeGridColumns());
            /* flag for refreshing newly added record */
            proxy._isRefreshAddedRecord = false;
            /* Used as flag for refresh chart and grid rows on adding when parent item is in collpased state*/
            proxy._isInAdd = false;
        },
        //calculate height and width for gantt at load time
        _calculateDimensions: function ()
        {
            var proxy = this,
                sizeSettings = this.model.sizeSettings,
                sizeSettingHeight = sizeSettings.height,
                sizeSettingsWidth = sizeSettings.width,
                element = proxy.element,
                elementStyleHeight = element[0].style.height,
                elementStyleWidth = element[0].style.width,
                parentHeight = element.parent().height(),
                parentWidth = element.parent().width();

            /* check given value and assign default value for width and height */
            if (!sizeSettingHeight && !elementStyleHeight) {
                elementStyleHeight = "450px";
                element.css("height", "450px");
            }

            if (!sizeSettingsWidth && !elementStyleWidth) {
                elementStyleWidth = "100%";
                proxy.element.css("width", "100%");
            }
            proxy._ganttHeight = sizeSettingHeight ? sizeSettingHeight : elementStyleHeight,
            proxy._ganttWidth = sizeSettingsWidth ? sizeSettingsWidth : elementStyleWidth,
            proxy._ganttHeight = proxy._ganttHeight.toString().indexOf('%') !== -1 ? (parentHeight * parseInt(proxy._ganttHeight) / 100) :parseInt(proxy._ganttHeight),
            proxy._ganttWidth = proxy._ganttWidth.toString().indexOf('%') !== -1 ? (parentWidth * parseInt(proxy._ganttWidth) / 100) : parseInt(proxy._ganttWidth);

            if (proxy._ganttHeight === 0) {
                proxy._ganttHeight = 450;
                element.css("450px");
            }
        },
        _setCultureInfo:function() {
             var proxy = this,
                model = proxy.model,
                culture = model.locale,
                localization = ej.Gantt.localization[culture],
                defaultLocalization = ej.Gantt.localization["en-US"];
            model.dateFormat = !model.dateFormat ? ((localization && localization["calendars"] &&
                    localization["calendars"]["standard"]["patterns"]) ?
                (localization["calendars"]["standard"]["patterns"]["d"]) :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["patterns"]["d"] :
                    defaultLocalization["calendars"]["standard"]["patterns"]["d"]) : model.dateFormat;

            
            //EmptyRecord text to be displayed with dataSource is null
            proxy._emptyRecordText = (localization && localization["emptyRecord"]) ?
                localization["emptyRecord"] : defaultLocalization["emptyRecord"];
            
            //ColumnHeaderText to be displayed in treegrid
            proxy._columnHeaderTexts = ( localization && localization["columnHeaderTexts"] ) ?
                localization["columnHeaderTexts"] : defaultLocalization["columnHeaderTexts"];
            
            proxy._columnHeaderTexts = $.extend({}, defaultLocalization["columnHeaderTexts"], proxy._columnHeaderTexts);

            //EditDialog Text to be displayed in popup window
            proxy._editDialogTexts = (localization && localization["editDialogTexts"]) ?
                localization["editDialogTexts"] : defaultLocalization["editDialogTexts"];

            proxy._editDialogTexts = $.extend({}, defaultLocalization["editDialogTexts"], proxy._editDialogTexts);

            //Tooltip Text to be displayed in the Toolbar
            proxy._toolboxTooltipTexts = (localization && localization["toolboxTooltipTexts"]) ?
                localization["toolboxTooltipTexts"] :
                defaultLocalization["toolboxTooltipTexts"];

            proxy._toolboxTooltipTexts = $.extend({}, defaultLocalization["toolboxTooltipTexts"], proxy._toolboxTooltipTexts);
            //Tooltip Text to be displayed on the Taskbar for duration units
            proxy._durationUnitTexts = (localization && localization["durationUnitTexts"]) ?
                localization["durationUnitTexts"] : defaultLocalization["durationUnitTexts"];

            proxy._durationUnitTexts = $.extend({}, defaultLocalization["durationUnitTexts"],proxy._durationUnitTexts);
            //Context Menu Text
            proxy._contextMenuTexts = (localization && localization["contextMenuTexts"]) ?
                localization["contextMenuTexts"] : defaultLocalization["contextMenuTexts"];

            proxy._contextMenuTexts = $.extend({}, defaultLocalization["contextMenuTexts"], proxy._contextMenuTexts);
            //Name for New Task while adding
            proxy._newTaskTexts = (localization && localization["newTaskTexts"]) ?
                localization["newTaskTexts"] : defaultLocalization["newTaskTexts"];
            
            //Month names to be displayed in schedule
            proxy._months = (localization && localization["calendars"]) ?
                localization["calendars"]["standard"]["months"]["namesAbbr"] :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["months"]["namesAbbr"] :
                defaultLocalization["calendars"]["standard"]["months"]["namesAbbr"];
            
            //Day names to displayed in schedule
            proxy._days = (localization && localization["calendars"]) ?
                localization["calendars"]["standard"]["months"]["namesAbbr"] :
                Globalize.culture(culture) ? Globalize.culture(culture)["calendars"]["standard"]["days"]["namesAbbr"] :
                defaultLocalization["calendars"]["standard"]["days"]["namesAbbr"];

            //Column Menu Text
            proxy._columnMenuTexts = (localization && localization["columnMenuTexts"]) ?
                localization["columnMenuTexts"] : defaultLocalization["columnMenuTexts"];

            proxy._columnMenuTexts = $.extend({}, defaultLocalization["contextMenuTexts"], proxy._columnMenuTexts);

            //Predecssor Editing Text
            proxy._predecessorEditingTexts = localization && localization["predecessorEditingTexts"] ?
                localization["predecessorEditingTexts"] : defaultLocalization["predecessorEditingTexts"];

            proxy._predecessorEditingTexts = $.extend({}, defaultLocalization["contextMenuTexts"], proxy._predecessorEditingTexts);


            proxy._alertTexts = localization && localization["alertTexts"] ?
                localization["alertTexts"] : defaultLocalization["alertTexts"];

            proxy._alertTexts = $.extend({}, defaultLocalization["alertTexts"], proxy._alertTexts);
            
        },

        //Populate available fields
        _getMappingItems: function () {
            var proxy = this,
                model = proxy.model,
                mappingItems = [],
                length = proxy._columns.length;

            for (var i = 0; i < length; i++) {
                mappingItems.push(proxy._columns[i].mappingName);
            }
            return mappingItems;
        },

        //populate the context menu with items
        _getContextMenuItems: function () {
            var contextMenuItems = [],
                proxy = this,
                contextMenuLabel = this._contextMenuTexts;

            contextMenuItems = [{
                headerText: contextMenuLabel["taskDetailsText"],
                eventHandler: null,
                isDefault: true,
                menuId: "Task"

            }, {
                headerText: contextMenuLabel["addNewTaskText"],
                eventHandler: null,
                isDefault: true,
                menuId: "Add"
            },
             {
                headerText: contextMenuLabel["indentText"],
                eventHandler: null,
                isDefault: true,
                menuId: "Indent"
            }, {
                headerText: contextMenuLabel["outdentText"],
                eventHandler: null,
                isDefault: true,
                menuId: "Outdent"
            }, {
                headerText: contextMenuLabel["deleteText"],
                eventHandler: null,
                isDefault: true,
                menuId: "Delete"
            }         
           ];
            return contextMenuItems;
        },

        //populate sub context menu items
        _getsubContextMenuItems: function () {
            var contextMenuItems = [],
                subContextMenuLabel = this._contextMenuTexts;

            contextMenuItems = [{
                iconPath: null,
                headerText: subContextMenuLabel["aboveText"],
                eventHandler: null,
                menuId: "Above"
            }, {
                iconPath: null,
                headerText: subContextMenuLabel["belowText"],
                eventHandler: null,
                menuId: "Below"
            }
            ];
            return contextMenuItems;
        },

        //rendering is initiated here
        _initialize: function () {

            if (this._isFromSetModel)
                return;
            else {
                var proxy = this, model = proxy.model;
                proxy._$treegridPane = ej.buildTag('div.e-treegridPane');
                proxy._$ganttchartPane = ej.buildTag('div.e-ganttchartPane');
                proxy._$ejGantt = ej.buildTag('div.e-gantt-Spliter#e-ejSpliter' + proxy._id, "", {}, { 'unselectable': "on" });
                model.predecessorMapping && proxy._updatePredecessors();
                proxy._render();
                model.updatedRecords = model.flatRecords.slice();
                model.currentViewData = model.flatRecords.slice();
                model.predecessorMapping && proxy._updatedRecordsDateByPredecessor();
                proxy._calculateScheduleDates();
                proxy._calculateHeaderDates();
                proxy._updateGanttRecords();
                proxy._checkDataManagerUpdate();
                proxy._renderTreeGrid();
                model.updatedRecords = proxy.getUpdatedRecords();
                model.currentViewData = proxy.getCurrentViewData();
                proxy._renderGanttChart();
                proxy._enableCreateCollection = true;
                proxy._isValidationEnabled = false;          //no validation require here
                model.predecessorMapping && proxy._createConnectorLinesCollection();//connectorline oboject creates here
                proxy._isValidationEnabled = true;//reassign this value
                proxy._$treegridPane.css({ 'overflow': 'hidden' });
                //Dialog templates Rendering initiated
                proxy._initiateDialogTemplates();
                proxy._isTreeGridRendered = true;
                proxy._isGanttChartRendered = true;
                proxy._gridRows = proxy.getRows();
                proxy._ganttChartRows = proxy.getGanttChartRows();
                proxy._createSubContextMenuTemplate();
                proxy._createContextMenuTemplate();
                proxy._wireEvents();
                proxy._initialEndRendering();
            }
        },
        //apply row Selection and enable collapse all while loading
        _initialEndRendering:function()
        {
            var proxy = this,
                model = this.model;

            //Call collapse All
            if (model.enableCollapseAll) {
                if (this.selectedRowIndex() != -1) {
                    proxy._deSelectRowItem();
                }
                proxy._collapseAll();
            }

            //Select item by selected row index
            if (proxy.selectedRowIndex() >= 0 && proxy.selectedRowIndex() < proxy.model.updatedRecords.length) {
                var rowSelectingArgs = {};
                rowSelectingArgs.recordIndex = proxy.selectedRowIndex();
                rowSelectingArgs.previousIndex = model.selectedItem ? model.updatedRecords.indexOf(model.selectedItem) : -1;
                if (proxy.rowSelecting(rowSelectingArgs)) {
                    proxy.selectRows(proxy.selectedRowIndex());
                    proxy._$treegrid.focus();
                }
            }  
        },

        //Check if data manager is updatable
        _checkDataManagerUpdate: function ()
        {
            var proxy = this,
                dataSource =this.dataSource();
            proxy._isDataManagerUpdate = false;
            proxy._jsonData = null;

            if (dataSource instanceof ej.DataManager) {
                if (dataSource.dataSource.offline && dataSource.dataSource.json) {
                    proxy._isDataManagerUpdate = true;
                    proxy._jsonData = dataSource.dataSource.json;
                }
                else if (!dataSource.dataSource.offline) {
                    proxy._isDataManagerUpdate = true;
                    proxy._jsonData = proxy._retrivedData;
                }
            }
        },
        //Render add,edit dialog templates
        _initiateDialogTemplates:function()
        {
            var proxy = this, model = this.model;

            proxy.customColumnFields();
            if (model.editSettings.allowEditing || model.editSettings.editMode == "normal" || model.editSettings.editMode == "dialogTemplate") {
                proxy.editDialogTemplate();
                if ($("#" + this._id + "_dialogEdit").length === 0)
                    proxy.element.append(proxy._renderDialog());
            }
            if (model.editSettings.allowAdding) {
                proxy.addDialogTemplate();
                if ($("#" + this._id + "_dialogAdd").length === 0) {
                    var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogAdd" });
                    proxy.element.append($dialog);
                }
            }
        },

        //Toolbar,Gantt and treegrid parts are rendered and templates for add and edit dialog are created
        _render: function () {
            var proxy = this, element = proxy.element, model = proxy.model;
            if (proxy.model.toolbarSettings.showToolbar) {
                element.append(proxy._renderToolBar());
            }
            proxy._ganttHeight = parseInt(proxy._ganttHeight) - proxy._getToolBarHeight() - proxy._totalBorderHeight;
            proxy._$treegrid = ej.buildTag("div.ejTreeGrid#ejTreeGrid" + proxy._id, "", { "height": proxy._ganttHeight }, {});
            proxy._$ganttchart = ej.buildTag("div.ejGanttChart#ejGanttChart" + proxy._id, "", { "height": proxy._ganttHeight }, {});
            proxy._$ganttchartPane.append(proxy._$ganttchart);
            proxy._$treegridPane.append(proxy._$treegrid);
            proxy._$ejGantt.append(proxy._$treegridPane);
            proxy._$ejGantt.append(proxy._$ganttchartPane);    
            
            element.append(proxy._$ejGantt);
            proxy._renderGantt();
        },

        //Return dialog tag
        _renderDialog: function () {
            var $dialog = ej.buildTag("div.e-dialog e-dialog-content e-shadow e-widget-content", "", { display: "none" }, { id: this._id + "_dialogEdit" });
            return $dialog;
        },
        //Create Template for edit dialog
        editDialogTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = proxy._columns,
                length = columns.length,
                emptyDataColumns = [], model = proxy.model, column, predecessorColoumnIndex = -1,
                treeGridObj = proxy._$treegridHelper.data("ejTreeGrid"),
                columnIndex;

            proxy._mappingItems = proxy._getMappingItems();

            if (length == 0)
                return;

            if (model.editDialogFields.length > 0) {
                var filteredColumns = [];
                var count=0,resultColumn;
                for(;count<model.editDialogFields.length;count++)
                {
                    resultColumn = $.grep(proxy._columns, function (val) {
                        return val.mappingName === model.editDialogFields[count].field;
                    });
                    resultColumn.length && filteredColumns.push(resultColumn[0]);
                } 
                columns = filteredColumns;
            }
           
            var $tbody = ej.buildTag('div', "", {}, { 'unselectable': 'on' });
            $form = ej.buildTag('form', "", { 'height': 'auto', 'width': 'auto', 'font-family': 'Segoe UI', 'font-size': '14px' }, { id: proxy._id + "EditForm" });
            $table = ej.buildTag('table', "", { width: "500px" }, { 'unselectable': "on" });
            $tr = ej.buildTag('tr');
            for (var i = 0; i < columns.length; i++) {
                if ($.inArray(columns[i].mappingName, proxy._mappingItems) != -1) {
                    if (columns[i].mappingName !== model.predecessorMapping)
                    {
                        if ($tr.children("td").length < 2) {
                            $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '5px 15px 0', 'font-weight': 'normal', "outline": "none" }, { 'unselectable': "on" });
                            $innerTable = ej.buildTag('table', "", { "outline": "none" }, { 'unselectable': "on" });

                            $inTr = ej.buildTag('tr');

                            $inTd = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });
                            $inTd.append("<label for='" + columns[i].field + "' style='font-weight:normal;'>" + proxy._columnHeaderTexts[columns[i].field] + "</label>");

                            $inTr.append($inTd);

                            $innerTable.append($inTr);

                            $inTr2 = ej.buildTag('tr');
                            $inTd2 = ej.buildTag('td.e-editValue');

                            columnIndex = proxy._columns.indexOf(columns[i])
                            ej.TreeGrid._initCellEditType(treeGridObj, $inTd2, proxy._id, columnIndex);

                            $inTr2.append($inTd2);
                            $innerTable.append($inTr2);

                            $td.append($innerTable);
                            $tr.append($td);
                        }
                        else {
                            $table.append($tr);
                            $tr = $tempTr = ej.buildTag('tr');
                            i--;
                        }
                    }else
                    {
                        predecessorColoumnIndex=i;
                    }
                }
            }
            $table.append($tr);
            //      $("body").append($table);
            $tr = ej.buildTag("tr");

            if (model.predecessorMapping && predecessorColoumnIndex != -1) {

                $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '10px 15px 0', 'font-weight': 'normal',"outline":"none" }, { colspan: 2, 'unselectable': "on" });

                $innerTable = ej.buildTag('table#' + proxy._id + 'predecessoreditTable', "", { "width": "100%", "outline": "none" }, { 'unselectable': "on" });

                $inTr3 = ej.buildTag('tr');
                $inTd3 = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });
                $inTd3.append("<label for='predecessor' style='font-weight:normal;'>" + proxy._columnHeaderTexts["predecessor"] + "</label>");

                $inTr3.append($inTd3);
                $innerTable.append($inTr3);

                $inTr = ej.buildTag('tr');
                $inTd = ej.buildTag('td.editLabel', "", {"outline":"none"}, { 'unselectable': "on" });
                
                $inTd.append("<span class='e-addpre e-icon e-enable e-edit-dialog' style='cursor:pointer; width: auto;'>" + proxy._editDialogTexts["addPredecessor"] + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class='e-deletepre e-icon e-disable e-edit-dialog' style='cursor:pointer; width: auto;'>&nbsp;&nbsp;" + proxy._editDialogTexts["removePredecessor"] + "</span>");

                $inTr.append($inTd);
                $innerTable.append($inTr);

                $inTr2 = ej.buildTag('tr');
                $inTd2 = ej.buildTag('td.e-editValue');

                $preDiv = ej.buildTag("div#treegrid" + proxy._id + "predecessoredit", "", { "width": "100%", "height": "150px" }, {});

                $inTd2.append($preDiv);
             
                $inTr2.append($inTd2);
                $innerTable.append($inTr2);

                $td.append($innerTable);
                $tr.append($td);
                $table.append($tr);
            }

            
            $form.append($table);
            $form.appendTo($tbody);
           
            $tbody = proxy.renderDiaglogButton($form, $tbody);
            $.templates(proxy._id + "_JSONDialogEditingTemplate", $tbody.html());
        },
        //create template for add dialog
        addDialogTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = proxy._columns,
                length = columns.length,
                emptyDataColumns = [], model = proxy.model, column, predecessorColoumnIndex = -1,
                treeGridObj = proxy._$treegridHelper.data("ejTreeGrid"),
                columnIndex;

            proxy._mappingItems = proxy._getMappingItems();

            if (length == 0)
                return;

            if (model.addDialogFields.length > 0) {
                var filteredColumns = [];
                var count = 0, resultColumn;
                for (; count < model.addDialogFields.length; count++) {
                    resultColumn = $.grep(proxy._columns, function (val) {
                        return val.mappingName === model.addDialogFields[count].field;
                    });
                    resultColumn.length && filteredColumns.push(resultColumn[0]);
                }
                columns = filteredColumns;
            }

            var $tbody = ej.buildTag('div', "", {}, { 'unselectable': 'on' });
            $form = ej.buildTag('form', "", { 'height': 'auto', 'width': 'auto', 'font-family': 'Segoe UI', 'font-size': '14px' }, { id: proxy._id + "AddForm" });
            $table = ej.buildTag('table', "", { width: "500px" }, { 'unselectable': "on" });
            $tr = ej.buildTag('tr');
            for (var i = 0; i < columns.length; i++) {
                if ($.inArray(columns[i].mappingName, proxy._mappingItems) != -1) {
                    if (columns[i].mappingName !== model.predecessorMapping) {
                        if ($tr.children("td").length < 2) {
                            $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '5px 15px 0', 'font-weight': 'normal', "outline": "none" }, { 'unselectable': "on" });
                            $innerTable = ej.buildTag('table', "", { "outline": "none" }, { 'unselectable': "on" });

                            $inTr = ej.buildTag('tr');

                            $inTd = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });
                            $inTd.append("<label for='" + columns[i].field + "' style='font-weight:normal;'>" + proxy._columnHeaderTexts[columns[i].field] + "</label>");

                            $inTr.append($inTd);

                            $innerTable.append($inTr);

                            $inTr2 = ej.buildTag('tr');
                            $inTd2 = ej.buildTag('td.e-editValue');

                            columnIndex = proxy._columns.indexOf(columns[i])
                            ej.TreeGrid._initCellEditType(treeGridObj, $inTd2, proxy._id, columnIndex, "Add");
                            //proxy.initCellEditType(column, $inTd2, "Add");

                            $inTr2.append($inTd2);
                            $innerTable.append($inTr2);

                            $td.append($innerTable);
                            $tr.append($td);

                        }
                        else {
                            $table.append($tr);
                            $tr = $tempTr = ej.buildTag('tr');
                            i--;
                        }
                    } else {
                        predecessorColoumnIndex = i;
                    }
                }
            }
            $table.append($tr);

            $tr = ej.buildTag("tr");

            if (model.predecessorMapping && predecessorColoumnIndex != -1) {

                $td = ej.buildTag('td', "", { "text-align": "left", 'padding': '10px 15px 0', 'font-weight': 'normal', "outline": "none" }, { colspan: 2, 'unselectable': "on" });

                $innerTable = ej.buildTag('table#' + proxy._id + 'predecessoraddTable', "", { "width": "100%", "outline": "none" }, { 'unselectable': "on" });

                $inTr3 = ej.buildTag('tr');
                $inTd3 = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });
                $inTd3.append("<label for='predecessor' style='font-weight:normal;'>" + proxy._columnHeaderTexts["predecessor"] + "</label>");

                $inTr3.append($inTd3);
                $innerTable.append($inTr3);
                
                $inTr = ej.buildTag('tr');
                $inTd = ej.buildTag('td.editLabel', "", { "outline": "none" }, { 'unselectable': "on" });

                $inTd.append("<span class='e-addpre e-icon e-enable e-add-dialog' style='cursor:pointer; width: auto;'>" + proxy._editDialogTexts["addPredecessor"] + "</span>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<span class='e-deletepre e-icon e-disable e-add-dialog' style='cursor:pointer; width: auto;'>&nbsp;&nbsp;" + proxy._editDialogTexts["removePredecessor"] + "</span>");

                $inTr.append($inTd);
                $innerTable.append($inTr);

                $inTr2 = ej.buildTag('tr');
                $inTd2 = ej.buildTag('td.e-editValue');

                $preDiv = ej.buildTag("div#treegrid" + proxy._id + "predecessorAdd", "", { "width": "100%", "height": "150px" }, {});

                $inTd2.append($preDiv);

                $inTr2.append($inTd2);
                $innerTable.append($inTr2);

                $td.append($innerTable);
                $tr.append($td);
                $table.append($tr);
            }


            $form.append($table);
            $form.appendTo($tbody);

            $tbody = proxy.renderDiaglogButton($form, $tbody);
            $.templates(proxy._id + "_JSONDialogAddingTemplate", $tbody.html());
        },
        //remove contextmenu from DOM
        _clearContextMenu: function () {
            //variables for context menu items
         
            $('.e-contextmenu').remove();
            $('.e-innerContextmenu').remove();
        },

        //JSRender Template for ContextMenu
        _createContextMenuTemplate: function () {
            var proxy = this;
            var helpers = {};
            helpers["_" + proxy._id + "getHeaderName"] = $.proxy(proxy._getHeaderName, proxy);

            $.views.helpers(helpers);
            var menuItemList = "<li style='list-style-type:none;margin:0px;'>";
            var listChild = "<div class='e-menuitem' id={{:menuId}}  style='display:table;cursor:pointer;'>" +
                            "<div class='e-icon' style='display:inline-block;{{if !iconPath}}padding:5px;{{else}}margin-left:8px{{/if}};" +
                            "background-image:{{:iconPath}};background-repeat:no-repeat;'/>" +
                            "<div style='display:inline-block;padding:5px'>" +
                            "<span style='font-family:Segoe UI;font-size:12px;padding:5px;'>{{:headerText}}</span></div>" +
                            "{{if ~_" + proxy._id + "getHeaderName(#data)}}<div class='e-icon e-expander' style='display:inline-block;padding:5px;float:right;'/> {{/if}}";                            

            menuItemList += listChild;
            menuItemList += "</div></li>";
            var templates = {};
            templates[proxy._id + "contextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },

        //helper methods for context menu template
        _getHeaderName: function (data) {
            if (data.menuId == "Add" && this._subContextMenuItems.length > 0) {
                return true;
            }
            return false;
        },

        //JSRender Template for Inner Context Menu
        _createSubContextMenuTemplate: function () {
            var proxy = this;

            var menuItemList = "<li style='list-style-type:none;margin:0px;'>";
            var listChild = "<div class='e-menuitem' id={{:menuId}}  style='display:table;cursor:pointer;'>" +
                             "<div style='left:10px;height:20px;width:16px;position:absolute;padding-top:5px;" +
                             "background-image:{{:iconPath}};background-repeat:no-repeat;'/>" +
                            "<div style='display:inline-block;padding:5px 60px 5px 20px;'>" +
                            "<span style='font-family:Segoe UI;font-size:12px;padding:5px;'>{{:headerText}}</span></div></div>";


            menuItemList += listChild;
            menuItemList += "</li>";

            var templates = {};
            templates[proxy._id + "subContextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },
        //create templates for all columns edit types
        initCellEditType: function (column, element,editTarget) {

            var proxy = this,
                model = proxy.model,
                id,    
                helpers = {};
            id = editTarget ? editTarget : "";
            if (ej.isNullOrUndefined(column["editType"]))
                column["editType"] = "stringedit";

            switch (column["editType"]) {
                case "stringedit":
                    if (column.mappingName === model.predecessorMapping) {
                        helpers["_" + proxy._id + "predecessor"] = proxy._getPredecessorsValue;
                        $.views.helpers(helpers);
                        element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:~_" + proxy._id + "predecessor('" + proxy._id + "Object','" + model.predecessorMapping + "','" + column.field + "')}}", id: proxy._id + column.field, name: column.field }));
                    } else {
                        element.html(ej.buildTag('input.e-field e-ejinputtext', "", {}, { value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget }));
                    }
                    break;
                case "booleanedit":
                    element.html('{{if ' + column.field + '}} <input class="e-field e-checkbox" type ="checkbox" id=' + proxy._id + column.field + ' name=' + column.field + ' checked="checked"></input>{{else}} <input class="e-field e-checkbox" type ="checkbox" id=' + proxy._id + column.field + ' name=' + column.field + ' > {{/if}}');
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + column["editType"] + ' e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget });
                    element.append($datePicker);
                    break;
                case "dropdownedit":
                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist' + ' e-field', "", {}, { type: "text", value: "{{:#data['" + column.field + "']}}", id: proxy._id + column.field + id, name: column.field, dialog: editTarget });
                    element.append($dropDownList);                    
                    break;
                case "maskedit":
                    var $maskEdit = ej.buildTag('input .e-field e-maskedit', "", {},
                                        {
                                            value: "{{:#data['" + column.field + "']}}",
                                            id: proxy._id + column.field + id,
                                            name: column.field,
                                            dialog: editTarget
                                        });
                    element.append($maskEdit);
                    break;
            }
        },

        //returns predecessor value
        _getPredecessorsValue: function (gridObject, mappingName) {
            return this.data.item ? this.data.item[mappingName] : "";
        },

        //reduce the date when date is fall on holidays
        _onHoliDayCheckBefore: function (startDate) {
            var proxy = this,
                holidays = proxy.model.holidays,
                includeWeekend = proxy.model.includeWeekend;


            if (holidays && holidays.length) {
                var holidayIndex = 0,
                    holidayslength = holidays.length,
                    holiday, flag;
                    for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                        holiday = proxy._getDateFromFormat(holidays[holidayIndex].day);
                        if (holiday.getTime() == startDate.getTime()) {
                            startDate.setDate(startDate.getDate() - 1);
                            if (this.model.includeWeekend === false) {
                                proxy._setBeforeWeekend(startDate);
                            }
                        }
                        else {
                            return startDate;
                        }
                    }
                    return startDate;
            }
            else {
                return startDate;
            }
        },
        //check date is on holidays
        _onHoliDayCheck: function (startDate)
        {
            var proxy = this,
                holidays = proxy.model.holidays,
                includeWeekend = proxy.model.includeWeekend;
                

            if (holidays && holidays.length) {
                var holidayIndex = 0,
                    holidayslength = holidays.length,
                    holiday, flag;

                    for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                        holiday = proxy._getDateFromFormat(holidays[holidayIndex].day);
                        if (holiday.getTime()==startDate.getTime()) {
                            startDate.setDate(startDate.getDate() + 1);
                            if (this.model.includeWeekend === false) {
                                if (startDate.getDay() == 0) {
                                    startDate.setDate(startDate.getDate() + 1);
                                } else if (startDate.getDay() == 6) {
                                    startDate.setDate(startDate.getDate() + 2);
                                }
                            }
                        }
                        else
                        {
                            return startDate;
                        }
                    }
                    return startDate;
            }
            else
            {
                return startDate;
            }
        },
        //calculate end date and duration on date change in edit dialog
        _editStartDateChange:function(element,args)
        {
            var proxy = this,
                id = proxy._id,
                model = proxy.model,
                targetId = element.attr("id"),
                holidays = this.model.holidays,
                dialog = element.attr("dialog"),
                selectedItem = this.selectedItem(),
                editedObj = this._editedDialogRecord,
                startDateId, endDateId, durationId;
               

                if (dialog === "Add")
                {
                    startDateId = "#" + id + "startDateAdd";
                    endDateId = "#" + id + "endDateAdd";
                    durationId = "#" + id + "durationAdd";
                }
                else
                {
                    startDateId = "#" + id + "startDate";
                    endDateId = "#" + id + "endDate";
                    durationId = "#" + id + "duration";
                }

                var startDate = $(startDateId).val();
                endDate = $(endDateId).val();
                duration = parseInt($(durationId).val());

                if (!startDate)
                {
                    startDate = editedObj?editedObj.startDate:undefined;
                }
                if (isNaN(duration)) {
                    duration = editedObj ? editedObj.duration : undefined;
                }
                if (!endDate) {
                    endDate = editedObj ? editedObj.endDate : undefined;
                }

                if ((targetId === id + "startDate") || (targetId === id + "startDateAdd"))
                { 
                    startDate = proxy._getDateFromFormat(args.value);
                    var tempEndDate = new Date();
                    stratDate=proxy._onHoliDayCheck(startDate);
                    //check start date is in week end days
                    if (this.model.includeWeekend === false) {
                        if (startDate.getDay() === 0) {
                            startDate.setDate(startDate.getDate() + 1);
                        }
                        else if (startDate.getDay() === 6) {
                            startDate.setDate(startDate.getDate() + 2);
                        }
                    }
                    tempEndDate = startDate;
                    startDate = new Date(startDate);
                    //$("#" + id + "startDate").val(startDate);
                    if (editedObj)
                        editedObj.startDate = new Date(startDate);

                if (model.dateFormat.indexOf("hh")!=-1)
                    $(startDateId).ejDateTimePicker("option", "value", startDate);
                else
                    $(startDateId).ejDatePicker("option", "value", startDate);

                if (!endDate || isNaN(duration))
                    return;

                if (model.durationMapping) {

                    //check week end is inside the period

                    if (this.model.includeWeekend === false) {
                        for (count = 1; count < duration; count++) {
                            if (tempEndDate.getDay() == 0)
                                tempEndDate.setDate(tempEndDate.getDate() + 1);
                            else if (tempEndDate.getDay() == 6)
                                tempEndDate.setDate(tempEndDate.getDate() + 2);
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        }
                    } else if (duration > 0) {
                        tempEndDate.setDate(tempEndDate.getDate() + duration - 1);
                    }

                    if (this.model.includeWeekend === false) {
                        if (tempEndDate.getDay() == 0)
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        else if (tempEndDate.getDay() == 6)
                            tempEndDate.setDate(tempEndDate.getDate() + 2);
                    }

                    //holiday collection checking in end date
                    if (holidays && holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = proxy._getDateFromFormat(holidays[holidayIndex].day);
                            if (holiday >= proxy._getDateFromFormat(new Date(startDate), this.model.dateFormat) && holiday <= tempEndDate) {
                                tempEndDate.setDate(tempEndDate.getDate() + 1);
                                if (this.model.includeWeekend === false) {
                                    if (tempEndDate.getDay() == 0) {
                                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                                    } else if (tempEndDate.getDay() == 6) {
                                        tempEndDate.setDate(tempEndDate.getDate() + 2);
                                    }
                                }
                            }
                        }
                    }

                    if (this.model.includeWeekend === false) {
                        if (tempEndDate.getDay() == 0)
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        else if (tempEndDate.getDay() == 6)
                            tempEndDate.setDate(tempEndDate.getDate() + 2);
                    }

                    tempEndDate = new Date(tempEndDate);

                    //temp =this._calculateFormatedDate(temp, this.model.dateFormat);
                    if (editedObj)
                        editedObj.endDate = new Date(tempEndDate);

                    if (model.dateFormat.indexOf("hh") != -1)
                        $(endDateId).ejDateTimePicker("option", "value", tempEndDate);
                    else
                        $(endDateId).ejDatePicker("option", "value", tempEndDate);

                    //$("#" + id + "endDate").val(tempEndDate);
                    //$("#" + id + "endDate").attr("value", tempEndDate);
                }
                else {
                    var dateDiff = proxy._daydiff(new Date(this.selectedItem().startDate), new Date(editedObj.startDate));
                    editedObj.duration -= dateDiff;
                }
            }

            else if((targetId=== id + "endDate") ||(targetId===id + "endDateAdd"))
            {
                endDate = proxy._getDateFromFormat(args.value);

                endDate = proxy._onHoliDayCheck(endDate);
                if (this.model.includeWeekend === false) {
                    if (endDate.getDay() === 0) {
                        endDate.setDate(endDate.getDate() - 2);
                    }
                    else if (endDate.getDay() === 6) {
                        endDate.setDate(endDate.getDate() - 1);
                    }
                }
                endDate = new Date(endDate);

                if (editedObj)
                    editedObj.endDate = new Date(endDate);

                if (model.dateFormat.indexOf("hh") != -1)
                    $(endDateId).ejDateTimePicker("option", "value", endDate);
                else
                    $(endDateId).ejDatePicker("option", "value", endDate);
             
                if (!startDate || isNaN(duration))
                    return;

                startDate = proxy._getDateFromFormat(startDate);
                endDate = proxy._getDateFromFormat(endDate);

                if (startDate.getTime() <= endDate.getTime())
                {
                        
                    var tempDuration = 0;
                    dateDiff = proxy._daydiff(proxy._getDateFromFormat(startDate), proxy._getDateFromFormat(endDate));

                    if (this.model.includeWeekend === false) {
                        offset = proxy._calculateDateDifference(new Date(startDate), dateDiff);
                        offset += 1;
                    }
                    else
                        offset = dateDiff + 1;

                    for (var count = 0; count < holidays.length; count++) {
                        if (proxy._getDateFromFormat(holidays[count].day) >= new Date(startDate) && proxy._getDateFromFormat(holidays[count].day) <= new Date(endDate)) {
                            offset--;
                        }
                    }
                    tempDuration = offset;
                    if (editedObj)
                        editedObj.duration = tempDuration;

                    $(durationId).ejNumericTextbox("option", "value", tempDuration);
                }
                else {
                    if (editedObj)
                        editedObj.endDate = this._getDateFromFormat(args.prevDate);

                    if (model.dateFormat.indexOf("hh") != -1)
                        $(endDateId).ejDateTimePicker("option", "value", args.prevDate);
                    else
                        $(endDateId).ejDatePicker("option", "value", args.prevDate);

                }
            }
            else if ((targetId === id + "duration") || (targetId === id + "durationAdd"))
            {
                startDate = proxy._getDateFromFormat(startDate);
                var tempEndDate = new Date(startDate);

                if (!endDate || !startDate)
                    return;

                duration = args.value;

                //check week end is inside the period
                if (this.model.includeWeekend === false) {
                    for (count = 1; count < duration; count++) {
                        if (tempEndDate.getDay() == 0)
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                        else if (tempEndDate.getDay() == 6)
                            tempEndDate.setDate(tempEndDate.getDate() + 2);
                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                    }
                } else if (duration > 0) {
                    tempEndDate.setDate(tempEndDate.getDate() + duration - 1);
                }

                if (this.model.includeWeekend === false) {
                    if (tempEndDate.getDay() == 0)
                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                    else if (tempEndDate.getDay() == 6)
                        tempEndDate.setDate(tempEndDate.getDate() + 2);
                }
                //holiday collection checking in end date
                if (holidays && holidays.length) {
                    var holidayIndex = 0,
                        holidayslength = holidays.length,
                        holiday;
                    for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                        holiday = proxy._getDateFromFormat(holidays[holidayIndex].day);
                        if (holiday >= proxy._getDateFromFormat(startDate, this.model.dateFormat) && holiday <= tempEndDate) {
                            tempEndDate.setDate(tempEndDate.getDate() + 1);
                            if (this.model.includeWeekend === false) {
                                if (tempEndDate.getDay() == 0) {
                                    tempEndDate.setDate(tempEndDate.getDate() + 1);
                                } else if (tempEndDate.getDay() == 6) {
                                    tempEndDate.setDate(tempEndDate.getDate() + 2);
                                }
                            }
                        }
                    }
                }

                if (this.model.includeWeekend === false) {
                    if (tempEndDate.getDay() == 0)
                        tempEndDate.setDate(tempEndDate.getDate() + 1);
                    else if (tempEndDate.getDay() == 6)
                        tempEndDate.setDate(tempEndDate.getDate() + 2);
                }
                tempEndDate = new Date(tempEndDate);

                if (editedObj) {
                    editedObj.endDate = new Date(tempEndDate);
                    editedObj.duration = duration;
                }
                if (model.dateFormat.indexOf("hh") != -1)
                    $(endDateId).ejDateTimePicker("option", "value", tempEndDate);
                else
                    $(endDateId).ejDatePicker("option", "value", tempEndDate);
            }
        },

        //Create task name collection from records for edit dialog
        _taskNameCollection:function()
        {
            var proxy = this,
            records = proxy.model.flatRecords,
            names = [];
            idsCollection = [];
            proxy._preTableCollection = [];
            for(var i=0;i<records.length;i++)
            {
                var temp = { id: records[i].taskId.toString(), text: (records[i].taskId.toString() + "-" + records[i].taskName), value: (records[i].taskId.toString() + "-" + records[i].taskName)};
                var ids = { id: (records[i].taskId.toString() + "-" + records[i].taskName), text: records[i].taskId.toString(), value: records[i].taskId.toString() };
                names.push(temp);
                idsCollection.push(ids);
            }            
            proxy._preTableCollection.push(idsCollection);
            proxy._preTableCollection.push(names);
        }
        ,

        //get ID array from predecessor string
        _idFromPredecessor:function(pre)
        {
            var str = "123FS+2";
            var preArray = pre.split(',');
            var preIdArray = [];
           

            for (var j = 0; j < preArray.length; j++)
            {
                var strArray = [];
                for (var i = 0; i < preArray[j].length; i++) {
                    if (!isNaN(preArray[j].charAt(i))) {
                        strArray.push(preArray[j].charAt(i));
                    }
                    else {
                        break;
                    }
                }
                preIdArray.push((strArray.join('')));
            }
            return preIdArray;
        },

        //Change the id name while task name is selected in dialog
        _preIdChangByName:function()
        {
            var proxy = this;
            var $form = $("#" + proxy._id + "PreEditForm");
            $targetTr = $form.closest("tr");
            $val = $form.find(".onEdit");
            var temp = $val.val();
            var names = proxy._taskNameCollection();

            var id = $.map(names, function (obj, index) {
                if (obj.text === $val.val()) {
                    return obj.id;
                }
            });
            id = id[0];
            $targetTr.find(".id").html(id);
        },

        //handle end edit event in dialog box predecessor editing
        _preEndEdit:function(editDialogId,args)
        {
            var proxy = this,
                dataSource = args.model.dataSource,
                selectedItem = args.model.selectedItem,
                tempId;
            proxy._taskNameCollection();
            if (args.columnName === "name" && args.value.length > 0)
            {
                tempId = args.value.split('-');
                var id = $.map(proxy._preTableCollection[1], function (obj, index) {
                    if (obj.id === tempId[0]) {
                        return obj.id;
                    }
                });
                id = id[0];
                selectedItem.id = id;
                selectedItem.name = args.value;
                var curent = dataSource.indexOf(selectedItem.item);
                if (curent === -1) {
                    dataSource.push(selectedItem.item);
                    curent = dataSource.indexOf(selectedItem.item);
                }
                dataSource[curent].id = id;
                dataSource[curent].name = args.value;
            }
            if (args.columnName === "id" && args.value.length > 0) {

                var name = $.map(proxy._preTableCollection[0], function (obj, index) {
                    if (obj.value === args.value) {
                        return obj.id;
                    }
                });
                name = name[0];
                selectedItem.name = name; 
                selectedItem.id = id;
                var curent = dataSource.indexOf(selectedItem.item);
                if (curent === -1) {
                    dataSource.push(selectedItem.item);
                    curent = dataSource.indexOf(selectedItem.item);
                }
                dataSource[curent].name = name;
            }    
                
            if (!selectedItem.type && !ej.isNullOrUndefined(curent)) {
                    selectedItem.type = "Finish-Start";
                    dataSource[curent].type = "Finish-Start";
            }
            $(editDialogId).ejTreeGrid("refreshRow", args.model.selectedRowIndex);
            $(editDialogId).ejTreeGrid("selectRows", args.model.selectedRowIndex);
            proxy.enbleDisableAddPreButton('enable');
        },
        //udapte the status of predecessor buttons in dialog
        _deleteDisableEnable:function(args)
        {
            var proxy = this;
            $(".e-deletepre").unbind("click", $.proxy(proxy._deletepredecessor, proxy));
            $(".e-deletepre").bind("click", $.proxy(proxy._deletepredecessor, proxy));
            $(".e-deletepre").removeClass("e-disable");
            $(".e-deletepre").addClass("e-enable");
        },

        //prepare data source for dropdown in predecessor edit in dialog box
        _predecessorEditCollection:function(){
        
            var proxy = this,
               model = proxy.model,
               collection,
               gridId,
               types,
               data = [],
               selectedItem;
            gridId = "#" + proxy._id + "predecessor";
            types = [{ id: "SS", text: "Start-Start", value: "Start-Start" },
                     { id: "SF", text: "Start-Finish", value: "Start-Finish" },
                     { id: "FS", text: "Finish-Start", value: "Finish-Start" },
                     { id: "FF", text: "Finish-Finish", value: "Finish-Finish" }];
            data = [];
            selectedItem = model.selectedItem;
            proxy._taskNameCollection();
            collection = proxy._preTableCollection[1];
            if (!ej.isNullOrUndefined(selectedItem) && !ej.isNullOrUndefined(selectedItem.predecessor) && !ej.isNullOrUndefined(selectedItem.item[model.predecessorMapping]))
            {
                var predecessor = selectedItem.predecessor,
                    predecessorLength = predecessor.length,
                    typeLength = types.length,
                    collectionLength = collection.length;
                for (var i = 0; i < predecessorLength; i++) {
                    var id = predecessor[i].from;
                    if (selectedItem.taskId.toString()!==id.toString())
                    {
                        var taskName, type, lags = 0;
                        for (var j = 0; j < collectionLength; j++) {
                            if (collection[j].id === id) {
                                taskName = collection[j].value;
                                break;
                            }
                        }
                        for (var k = 0; k < typeLength; k++) {
                            if (types[k].id == predecessor[i].predecessorsType) {
                                type = types[k].text;
                            }
                        }
                        lags = predecessor[i].offset;
                        var obj = { id: id, name: taskName, type: type, offset: lags };
                        data.push(obj);
                    }
                }
            }
            return data;
        },

        //render control in diaog box as per template created
        _refreshEditForm: function (args) {
            var proxy = this,
                elementFocused = false,
                $formElement,
                percent = 86,
                i = 0, form, length,
                $element,
                inputWidth = "165px",
                model = proxy.model,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat, newIdInstant,
                formatVal, editTreeGridId;
                selectedItem = model.selectedItem;

                if (args.requestType == "add") {
                    form = document.getElementById(proxy._id + "AddForm");
                    editTreeGridId = "#treegrid" + proxy._id + "predecessorAdd";
                    if (!ej.isNullOrUndefined(model.dataSource) && model.dataSource.length == 0 && model.predecessorMapping && $(editTreeGridId).length > 0)
                        $(form).find("#" + proxy._id + "predecessoraddTable").parent()[0].hidden = true;
                    newIdInstant = proxy.model.flatRecords.length;
                    newIdInstant = proxy._getNewTaskId(newIdInstant);
                }
                else {
                    form = document.getElementById(proxy._id + "EditForm");
                    editTreeGridId = "#treegrid" + proxy._id + "predecessoredit";
                    if (model.flatRecords.length <= 2 && model.parentRecords.length == 1
                        && model.predecessorMapping && $(editTreeGridId).length > 0)
                        $(form).find("#" + proxy._id + "predecessoreditTable").parent()[0].hidden = true;

                }

                $formElement = $(form).find("input,select");
                length = $formElement.length;
                for (i; i < length; i++)
                {
                    $element = $formElement.eq(i);
                    params = {};
                    inputWidth = 210;
                    column = ej.TreeGrid.getColumnByField(proxy._columns, $element.prop("name"));
                    if ($element.hasClass("e-numerictextbox")) {
               
                        width = inputWidth;
                        value = $element.val();
                        params.width = width;                    
                        params.showSpinButton = true;
                        params.cssClass = model.cssClass;
                        //for parent editing 
                        var tempId = $element[0].id;
                        tempId=tempId.replace(proxy._id, '');
                        if (selectedItem !== null && selectedItem.hasChildRecords && (tempId == "duration" || tempId == "status"))
                        {
                            params.readOnly = true;
                            $element.closest("td").css("opacity", 0.5);
                        }
                        if (tempId == "status" || tempId == "statusAdd") {
                            params.maxValue = 100;
                            params.minValue = 0;
                        }
                        else if (tempId == "duration" || tempId == "durationAdd") {
                            params.maxValue = 100000;
                            params.change = $.proxy(proxy._editStartDateChange, proxy, $element);
                            params.minValue = 0;
                        }
                    
                        if (value.length)
                            params.value = parseFloat(value);
                        else
                            params.value = 0;

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);
                        $element.ejNumericTextbox(params);
                        $element.prop("name", $element.prop("name").replace(proxy._id, ""));
                    }
                    else if ($element.hasClass("e-datepicker")) {

                        params.width = inputWidth;
                        params.cssClass = model.cssClass;
                        params.dateFormat = model.dateFormat;
                        params.locale = model.locale;
                        params.change = function (args) {
                            this.option("value", args.value);
                            proxy._editStartDateChange(this.element,args);
                        };
                        var tempId = $element[0].id;
                        tempId = tempId.replace(proxy._id, '');
                        if (selectedItem !== null && selectedItem.hasChildRecords && (tempId == "startDate" || tempId == "endDate") && args.requestType !== "add" ) {
                            params.readOnly = true;
                            $element.closest("td").css("opacity", 0.5);
                        }
                        if ($element.val().length)
                            params.value = proxy._getDateFromFormat($element.val());
                        else
                            params.value = proxy._getDateFromFormat(model.scheduleStartDate);

                       
                        if (column["format"] !== undefined && column.format.length > 0) {
                            toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                            formatVal = toformat.exec(column.format);
                            params.dateFormat = formatVal[2];
                            params.value = Globalize.format(new Date(params.value), params.dateFormat);
                            $element.val(params.value);
                        }
                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                            $element.ejDatePicker(params);
                    }
                    else if ($element.hasClass("e-datetimepicker")) {

                        params = {
                            width: inputWidth,
                            rtl: model.rtl,
                            locale:model.locale,
                            cssClass: model.css,
                            dateTimeFormat: model.dateFormat,
                        };
                        if ($element.val().length)
                            params.value = proxy._getDateFromFormat($element.val());
                        else
                            params.value = proxy._getDateFromFormat(model.scheduleStartDate);

                        if (column["format"] !== undefined && column.format.length > 0) {
                            toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                            formatVal = toformat.exec(column.format);
                            params.dateTimeFormat = formatVal[2];
                            params.value = Globalize.format(new Date($element.val()), params.dateTimeFormat);
                            $element.val(params.value);
                        }
                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                            $element.ejDateTimePicker(params);
                    }
                    else if ($element.hasClass("e-dropdownlist")) {

                        var dataSource = column.dropdownData;
                        var checkReadonly = false;
                        if (column.field === "resourceInfo") {
                            if (selectedItem !== null && selectedItem.hasChildRecords && args.requestType !== "add") {
                                checkReadonly = true;
                                $element.closest("td").css("opacity", 0.5);
                            }
                            $element.ejDropDownList({
                                cssClass: model.cssClass,
                                width: inputWidth,
                                showCheckbox: true,
                                dataSource: dataSource,
                                readOnly: checkReadonly,
                                fields: { id: model.resourceIdMapping, text: model.resourceNameMapping, value: model.resourceNameMapping },
                                selectedItems: proxy.getIndexofresourceInfo(dataSource, args.data)
                            });
                        }
                        else {
                            var controlArgs = {};
                            controlArgs.cssClass = model.cssClass;
                            controlArgs.width = inputWidth;
                            controlArgs.dataSource = dataSource;
                            if (!ej.isNullOrUndefined(column["editParams"]))
                                $.extend(controlArgs, column["editParams"]);

                            $element.ejDropDownList(controlArgs);

                            var obj = $element.ejDropDownList("instance");
                            obj._setValue($element.attr("cellValue"));;

                        }
                    }
                    else if ($element.hasClass("e-maskedit"))
                    {
                        var controlArgs = {};
                        controlArgs.locale = model.locale;
                        controlArgs.cssClass = model.cssClass;
                        controlArgs.width = inputWidth;
                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(controlArgs, column["editParams"]);

                        $element.ejMaskEdit(controlArgs);
                    }
                    else {
                        switch ($element.prop('tagName')) {
                            case "INPUT":

                                if ($element.attr("type") != "checkbox") {
                                    //For ID field id is read only

                                    if ($element.attr("name") === "taskId") {
                                        if (args.requestType !== "add") {
                                            $element.attr("readonly", "readonly");
                                            $element.css("opacity", 0.5);
                                        }
                                        else if (args.requestType === "add") {
                                            $element.val(newIdInstant);
                                        }
                                    }
                                    else if ($element.attr("name") === "taskName" && args.requestType === "add") {
                                        $element.val(proxy._newTaskTexts["newTaskName"] +" " +newIdInstant);
                                    }

                                    var tempId = $element[0].id;
                                    tempId = tempId.replace(proxy._id, '');

                                    $element.css("text-align", $element.attr("name") != null && ej.TreeGrid.getColumnByField(proxy._columns, $element.prop("name")) != null ?
                                        ej.TreeGrid.getColumnByField(proxy._columns, $element.attr("name")).textAlign : "center");
                                    $element.outerWidth(inputWidth).height(25);
                                } else {
                                    var controlArgs = {};
                                    controlArgs.cssClass = model.cssClass;
                                    controlArgs.size = "medium";
                                    if (!ej.isNullOrUndefined(column["editParams"]))
                                        $.extend(params, column["editParams"]);
                                    $element.ejCheckBox(controlArgs);
                                }
                                break;

                            case "SELECT":
                                $element.width(inputWidth).height(23);
                                break;
                        }
                    }
                    if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                        if (!proxy._isEnterKeyPressed) {
                            proxy._focusElements($element.closest('td'));
                            elementFocused = true;
                        }
                    }
                }

            //render prdecessor table in edit dialog
            if (model.predecessorMapping && $(editTreeGridId).length) {
                var types = [{ id: "SS", text: "Start-Start", value: "Start-Start" }, { id: "SF", text: "Start-Finish", value: "Start-Finish" }, { id: "FS", text: "Finish-Start", value: "Finish-Start" }, { id: "FF", text: "Finish-Finish", value: "Finish-Finish" }];
                var ds = [];
                if (args.requestType !== "add")
                {
                    ds = proxy._predecessorEditCollection();
                }
                proxy._taskNameCollection();
                $(editTreeGridId).ejTreeGrid({
                    dataSource:ds,
                    allowSorting: false,
                    allowAdding: true,
                    columns: [{ headerText:proxy._columnHeaderTexts["taskId"], field: "id", editType: ej.TreeGrid.EditingType.Dropdown, dropdownData: proxy._preTableCollection[0], width: "50px" },
                              { headerText: proxy._columnHeaderTexts["taskName"], field: "name", editType: ej.TreeGrid.EditingType.Dropdown, dropdownData: proxy._preTableCollection[1], width: "170px" },
                              { headerText: proxy._columnHeaderTexts["type"], field: "type", editType: ej.TreeGrid.EditingType.Dropdown, dropdownData: types, width: "80px" },
                              { headerText: proxy._columnHeaderTexts["offset"], field: "offset", editType: ej.TreeGrid.EditingType.Numeric, width: "50px" }],
                    enableAltRow: true,
                    allowColumnResize: true,
                    editSettings: {
                        allowAdding: true,
                        allowDeleting: true,
                        allowEditing: true,
                        editMode: "cellEditing",
                    },
                    locale: model.locale,
                    endEdit: $.proxy(proxy._preEndEdit, proxy, editTreeGridId),
                    beginEdit: $.proxy(proxy._preBeginEdit, proxy),
                    rowSelected: $.proxy(proxy._deleteDisableEnable, proxy),
                    treeColumnIndex: 5,
                    predecessorTable: proxy._isFromGantt
                });
                
                if (model.selectedItem && model.selectedItem.hasChildRecords && args.requestType!=="add")
                {
                    proxy.enbleDisableDeletePreButton('disable');
                    proxy.enbleDisableAddPreButton('disable');
                }
                else {
                    proxy.enbleDisableDeletePreButton('disable');
                    proxy.enbleDisableAddPreButton('enable');
                }
            }
        },
        //updated the status of predecessor button in dialog box
        enbleDisableDeletePreButton:function(args)
        {
            var proxy = this;
            if(args==="enable")
            {
                $(".e-deletepre").unbind("click", $.proxy(proxy._deletepredecessor, proxy));
                $(".e-deletepre").bind("click", $.proxy(proxy._deletepredecessor, proxy));
                $('.e-deletepre').addClass('e-enable');
                $('.e-deletepre').removeClass('e-disable');
            }
            else {
                $(".e-deletepre").unbind("click", $.proxy(proxy._deletepredecessor, proxy));
                $('.e-deletepre').addClass('e-disable');
                $('.e-deletepre').removeClass('e-enable');
            }
        },
        //updated the status of predecessor button in dialog box
        enbleDisableAddPreButton:function(args)
        {
            var proxy = this;
            if (args === "enable") {
               $(".e-addpre").unbind("click", $.proxy(proxy._addpredecessor, proxy));
               $(".e-addpre").bind("click", $.proxy(proxy._addpredecessor, proxy));
                $('.e-addpre').addClass('e-enable');
                $('.e-addpre').removeClass('e-disable');
            }
            else {
                $(".e-addpre").unbind("click", $.proxy(proxy._addpredecessor, proxy));
                $('.e-addpre').addClass('e-disable');
                $('.e-addpre').removeClass('e-enable');
            }
        },
        //updated the status of predecessor button in dialog box
        _disableAddButton: function (treeGridId) {
            var proxy = this;
            var table = $(treeGridId +"e-table");
            var flag = false;
            var len = table[0].rows.length;
            proxy.enbleDisableAddPreButton('disable');
            for (var j = 0; j < len; j++)
            {
                var currentRow = table[0].rows[j];
                if(currentRow.cells.length===4)
                if(currentRow.cells[1].innerText==="")
                {
                    flag = true;
                }
            }
            if (!flag) {
                proxy.enbleDisableAddPreButton('enable');
            }
        },

        //add empty row in predecessor grid in dialog box
        _addpredecessor: function (e) {
            var proxy = this;
            var args = {}, treeGridId,
            item = { id: "", name: "", type: "", offset: "" };
            proxy.enbleDisableAddPreButton('disable');
            if ($(e.target).hasClass("e-edit-dialog"))
                treeGridId="#treegrid" + proxy._id + "predecessoredit";
            else
                treeGridId = "#treegrid" + proxy._id + "predecessorAdd";
            $(treeGridId).ejTreeGrid("addRow", item);
        },
        //delete predecessor in dialog box 
        _deletepredecessor:function(e)
        {
            var proxy = this, treeGridId;

            if ($(e.target).hasClass("e-edit-dialog"))
                treeGridId = "#treegrid" + proxy._id + "predecessoredit";
            else
                treeGridId = "#treegrid" + proxy._id + "predecessorAdd";

            var selectedItem = $(treeGridId).ejTreeGrid("option", "selectedRowIndex");
            var args = {};
            args["requestType"] = ej.TreeGrid.Actions.Delete;
            if (selectedItem !== -1) {
                $(treeGridId).ejTreeGrid("deleteRow");
                proxy.enbleDisableDeletePreButton('disable');
            }
            proxy._disableAddButton(treeGridId);
        },

        //returns column value by field name
        getColumnIndexByField: function (field) {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                column = 0;

            for (column; column < length; column++) {

                if (columns[column]["field"] == field) break;

            }

            return column;
        },

        //returns selected resource index from rource value
        getIndexofresourceInfo: function (dataSource,data) {
            var resourceInfo = data.resourceInfo,
                count = 0, length = resourceInfo && resourceInfo.length, resourceIndex = [];

            for (count; count < length; count++)
                resourceIndex.push(dataSource.indexOf(resourceInfo[count]));

            return resourceIndex;

        },
        //focus elements on editing
        _focusElements: function ($currentCell) {
            if ($currentCell.length) {
                $currentCell.focus();
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length)
                    $childElem.find(".e-ddl").focus();
                else
                    $childElem.find('input,select').select().focus();
            }
        },
        //add button templates for add and edit dialog
        renderDiaglogButton: function (form, tbody) {

            var btnId,
                proxy = this,
                model = proxy.model;
            if (form[0].id === (proxy._id + "EditForm"))
                btnId = "EditDialog_";
            else
                btnId = "AddDialog_";

            var savebtn = ej.buildTag('input', "",
                 {
                     'border-radius':'3px'
                 },
                 { type: "button", id: btnId + proxy._id + "_Save" }),
             saveText = proxy._editDialogTexts["saveButton"];
            
            savebtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: saveText,
                    width: "70"
                });

            var cancelText = proxy._editDialogTexts["cancelButton"],
             cancelbtn = ej.buildTag('input', "",
                 {
                     'margin-left': '20px',
                     'border-radius': '3px'
                 },
                 {
                     type: "button",
                     id: btnId + proxy._id + "_Cancel"
                 });
            
            cancelbtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: cancelText,
                    width: "70"
                });

            //Sample delete button
            var deleteText = proxy._editDialogTexts["deleteButton"],
                delbtn = ej.buildTag('input', "",
                {
                    'margin-left': '20px',
                    'border-radius': '3px'
                },
                {
                    type: "button",
                    id: "EditDialog_" + proxy._id + "_Delete"
                });

            delbtn.ejButton(
                {
                    cssClass: model.cssClass,
                    text: deleteText,
                    width: "70"
                });



            var btnDiv = ej.buildTag('div', "", { 'margin-top': '20px','float':'right','margin-bottom':'15px' ,'margin-right':'15px'}, { 'class': "e-editform-btn" });

            btnDiv.append(savebtn);
            if(form[0].id===(proxy._id+"EditForm") && model.editSettings.allowDeleting)
                btnDiv.append(delbtn);
            btnDiv.append(cancelbtn);
            form.appendTo(tbody);
            btnDiv.appendTo(tbody);
            
            return tbody;
        },
        //Tool bar rendering initiated
        _renderToolBar: function() {
            var proxy = this,
                model = proxy.model,
                $div = ej.buildTag("div.e-gantttoolbar#e-gantttoolbar" + proxy._id, "", { 'height': '50px'}, { id: proxy._id + "_toolbarItems"}),
                $ul = ej.buildTag("ul", "", { 'width': proxy._ganttWidth }, {});
            (!ej.isNullOrUndefined(model.toolbarSettings.toolbarItems) && model.toolbarSettings.toolbarItems.length) && proxy._renderLi($ul);
            $div.append($ul);
            var $customUl = ej.buildTag("ul");
            $div.append($customUl);
            var helper = {};
            helper.click = proxy._toolBarClick;
            helper.cssClass = "";
            helper.rtl = false;
            helper.itemSeparator = false;
            helper.width = proxy._ganttWidth;
            helper.fields = {
                id: "",
                tooltipText: "",
                imageUrl: "",
                text: "",
                imageAttributes: "",
                spriteCSS: "",
                htmlAttributes: "",
            };
            $div.ejToolbar(helper);
            $div.ejToolbar("disableItem", proxy._disabledToolItems);
            proxy._disabledToolItems = $();
            return $div;
        },
        //craete LI elements for tool bar
        _renderLi: function ($ul) {
            var proxy = this,
                toolBarItems = proxy.model.toolbarSettings.toolbarItems,
                length = toolBarItems.length,
                i = 0,
                $li;
            for (i; i < length; i++) {

                $li = ej.buildTag("li", "", {}, {
                    id: proxy._id + "_" + toolBarItems[i],                    
                    "tabindex": "0"
                });

                if (toolBarItems[i] !== "search")
                    $li.css({ 'width': '30px', 'height': '24px' });

                proxy._renderLiContent($li, toolBarItems[i]);
                $ul.append($li);
            }
        },

        //add attributes for tool bar li elements
        _renderLiContent: function ($li, item) {
            var $a, $input;
            var selectedRowIndex = this.selectedRowIndex();
            var editSetting = this.model.editSettings;
            switch (item) {

                case "add":
                    $a = ej.buildTag("span.e-addnewitem e-toolbaricons e-icon e-gantt-add", "", {});
                    //title: toolBarItems[i],
                    var addTitle = this._toolboxTooltipTexts["addTool"];
                    $li.attr("title", addTitle);
                    if (!this.model.editSettings.allowAdding)
                        this._disabledToolItems.push($li.get(0));
                    break;

                case "edit":
                    $a = ej.buildTag("span.e-edititem e-toolbaricons e-icon e-gantt-edit", "", {});
                    var editTitle = this._toolboxTooltipTexts["editTool"];
                    $li.attr("title", editTitle);
                    if (!this.model.editSettings.allowEditing || selectedRowIndex == -1) {
                        this._disabledToolItems.push($li.get(0));
                    }
                    
                    break;

                case "delete":
                    var deleteTitle = this._toolboxTooltipTexts["deleteTool"];
                    $li.attr("title", deleteTitle);
                    $a = ej.buildTag("span.e-deleteitem e-toolbaricons e-icon e-gantt-delete", "", {});
                    
                    if (!this.model.editSettings.allowDeleting || selectedRowIndex == -1) {
                        this._disabledToolItems.push($li.get(0));
                    }
                    
                    break;

                case "update":
                    var saveTitle = this._toolboxTooltipTexts["saveTool"];
                    $li.attr("title", saveTitle);
                    $a = ej.buildTag("span.e-saveitem e-toolbaricons e-icon e-gantt-save e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "cancel":
                    var cancelTitle = this._toolboxTooltipTexts["cancelTool"];
                    $li.attr("title", cancelTitle);
                    $a = ej.buildTag("span.e-cancel e-toolbaricons e-icon e-gantt-cancel e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "indent":
                    var indentTitle = this._toolboxTooltipTexts["indentTool"];
                    $li.attr("title", indentTitle);
                    $input = ej.buildTag("span.e-indent e-toolbaricons e-icon e-disabletool e-gantt-indent", "", {}, {});
                    $li.append($input);
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "outdent":
                    var outdentTitle = this._toolboxTooltipTexts["outdentTool"];
                    $li.attr("title", outdentTitle);
                    $input = ej.buildTag("span.e-outdent e-toolbaricons e-icon e-disabletool e-gantt-outdent", "", {}, {});
                    $li.append($input);
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "expandAll":
                    var expandTitle = this._toolboxTooltipTexts["expandAllTool"];
                    $li.attr("title", expandTitle);
                    $input = ej.buildTag("span.e-expandall e-toolbaricons e-icon e-gantt-expandall", "", {}, {});
                    $li.append($input);
                    break;

                case "collapseAll":
                    var collapseTitle = this._toolboxTooltipTexts["collapseAllTool"];
                    $li.attr("title", collapseTitle);
                    $input = ej.buildTag("span.e-collapseall e-toolbaricons e-icon e-gantt-collapseall", "", {}, {});
                    $li.append($input); 
                    break;
                case "prevTimeSpan":
                    var prevTitle = this._toolboxTooltipTexts["prevTimeSpanTool"];
                    $li.attr("title", prevTitle);
                    $input = ej.buildTag("span.e-prevtimespan e-toolbaricons e-icon e-gantt-prevtimespan", "", {}, {});
                    $li.append($input);
                    break;
                case "nextTimeSpan":
                    var nextTitle = this._toolboxTooltipTexts["nextTimeSpanTool"];
                    $li.attr("title", nextTitle);
                    $input = ej.buildTag("span.e-nexttimespan e-toolbaricons e-icon e-gantt-nexttimespan", "", {}, {});
                    $li.append($input);
                    break;
                case "search":
                    var searchTitle = this._toolboxTooltipTexts["searchTool"];
                    $input = ej.buildTag("input.e-ejinputtext", "", { 'margin': '4px', 'text-indent': '4px' }, { type: "text", placeholder: searchTitle }, { 'color': "#E8E9E9" });
                    $li.css({ 'float': 'right', 'padding': '2px' });
                    $li.attr("title", searchTitle);
                    $li.addClass('e-search');
                    $li.append($input);
                    this.model.allowSearching = true;

                    $input.keydown($.proxy(this._keyDown, this));

                    break;
            }
            $li.append($a);
        },

        //handle key down for searching in gantt
        _keyDown: function (e) {

            if (e.which ===13) { //13 is key code for Enter Key 
                this._searchString = e.currentTarget.value;

                this._toolbarOperation(this._id + "_search", e.currentTarget.value);
				e.preventDefault();
				return false;//prevent post back
            }          
        },

        //update the status of tool bar elements 
        _updateToolbarOptions:function(){

            var proxy = this,
                model = proxy.model,
                editSettings = model.editSettings,
                toolbar = $("#" + proxy._id + "_toolbarItems"),
                $addElement = $(toolbar).find(".e-addnewitem").parent()[0],
                $editElement = $(toolbar).find(".e-edititem").parent()[0],
                $deleteElement = $(toolbar).find(".e-deleteitem").parent()[0],
                $indentElement = $(toolbar).find(".e-indent").parent()[0],
                $outdentElement = $(toolbar).find(".e-outdent").parent()[0],                
                disabledItems = [], enabledItems = [];
                
            if (editSettings.allowAdding)
                $addElement && enabledItems.push($addElement);
            else
                $addElement && disabledItems.push($($addElement).removeClass('e-disable')[0]);

            if (!model.selectedItem) {

                if (editSettings.allowEditing) {
                    $editElement && disabledItems.push($($editElement).removeClass('e-disable')[0]);
                }

                if (editSettings.allowDeleting) {
                    $deleteElement && disabledItems.push($($deleteElement).removeClass('e-disable')[0]);
                }

                if ($indentElement)
                    $indentElement && disabledItems.push($($indentElement).removeClass('e-disable')[0]);

                if ($outdentElement)
                    $outdentElement && disabledItems.push($($outdentElement).removeClass('e-disable')[0]);
            }
            else {
                
                if (editSettings.allowEditing) {
                    $editElement && enabledItems.push($editElement);
                }

                if (editSettings.allowDeleting) {
                    $deleteElement && enabledItems.push($deleteElement);
                }

                proxy.updateIndentOutdentOption(model.selectedItem);
            }
            
            $(toolbar).ejToolbar('disableItem', disabledItems);
            $(toolbar).ejToolbar('enableItem', enabledItems);
        },

        //Splitter control is initiated
        _renderGantt: function() {
            
            var proxy = this,
                leftPaneSize,
                rightPaneSize,
                ganttWidth = parseInt(proxy._ganttWidth),
                splitterPosition = proxy.splitterPosition(),
                isPercentage = false;
            //calculate treegrid pane size
            if(!ej.isNullOrUndefined(splitterPosition)){
                if (splitterPosition.indexOf("%") != -1){
                    leftPaneSize = parseInt(splitterPosition);
                    isPercentage= true;
                }
                else
                {
                    leftPaneSize = parseInt(splitterPosition) / proxy._ganttWidth * 100;
                    isPercentage = false;
                }
                // set grid width value is zero when grid width is less than zero or NAN.
                if (isNaN(leftPaneSize) || leftPaneSize < 0)
                    leftPaneSize = 0;
                // check the left pane size is greather than 100% of gantt width
                if (leftPaneSize > 100) {
                    leftPaneSize = (2 * proxy._commonColumnWidth + 3) / ganttWidth * 100;
                    leftPaneSize = leftPaneSize < 30 ? leftPaneSize : 30;
                    isPercentage = true;
                }
            }
            else
            {
                leftPaneSize = (2*proxy._commonColumnWidth+3) / ganttWidth * 100;
                leftPaneSize = leftPaneSize < 30 ? leftPaneSize : 30;
                isPercentage = true;
            }
            //Here 7 is the splitter span width
            rightPaneSize = 100 - leftPaneSize - (9 / ganttWidth * 100);
            // Check the right pane size is less than or euqal to 0.
            if (rightPaneSize <= 0)
            {
                if (isPercentage)
                    proxy.splitterPosition("100%");
                else
                    proxy.splitterPosition(ganttWidth.toString());
                leftPaneSize = 100 - (9 / ganttWidth * 100);
                rightPaneSize = 0;
            }
            else if (rightPaneSize > 0) {
                if (isPercentage)
                    proxy.splitterPosition(Math.round(leftPaneSize) + "%");
                else
                    proxy.splitterPosition(Math.round((leftPaneSize * proxy._ganttWidth / 100)).toString());
            }
            $("#e-ejSpliter" + proxy._id).ejSplitter({
                //border width and height excluded
                height: proxy._ganttHeight,
                width: proxy._ganttWidth,
                orientation: ej.Orientation.Horizontal,
                properties: [{ paneSize: leftPaneSize + "%",collapsible:false }, { paneSize: rightPaneSize + "%",collapsible:false }],
                enableAutoResize: false,
                resize: $.proxy(proxy._onResize,proxy)
            });
        },

        //Tree Grid part is initiated
        _renderTreeGrid: function() {
            var proxy = this, model = proxy.model;
            proxy._$treegridHelper = $("#ejTreeGrid" + proxy._id).ejTreeGrid({
                enableAltRow: model.enableAltRow,
                allowColumnResize: model.allowColumnResize,
                enableVirtualization: model.enableVirtualization,
                allowSorting: model.allowSorting,
                showColumnChooser: model.showColumnChooser,
                allowMultiSorting: model.allowMultiSorting,
                sortSettings: model.sortSettings,
                allowSelection: model.allowSelection,
                selectionType:"single",
                selectedRowIndex: this.selectedRowIndex(),
                allowSearching: model.allowSearching,
                parentIdMapping:model.parentTaskIdMapping,
                allowKeyboardNavigation: model.allowKeyboardNavigation,
                cssClass: model.cssClass,
                locale: model.locale,
                editSettings: model.editSettings,
                toolbarSettings:model.toolbarSettings,
                rowTemplate: model.rowTemplate,
                rowDataBound: model.rowDataBound,
                queryCellInfo: model.queryCellInfo,
                dataSource:this.dataSource(),
                flatRecords: model.flatRecords,
                parentRecords: model.parentRecords,
                ids: proxy.model.ids,
                columns: proxy._columns,
                dateFormat: model.dateFormat,
                resourceInfoMapping: model.resourceInfoMapping,
                resourceNameMapping: model.resourceNameMapping,
                resourceIdMapping:model.resourceIdMapping,
                taskIdMapping: model.taskIdMapping,
                showGridCellTooltip: model.showGridCellTooltip,
                showGridExpandCellTooltip: model.showGridExpandCellTooltip,
                taskNameMapping: model.taskNameMapping,
                startDateMapping: model.startDateMapping,
                endDateMapping: model.endDateMapping,
                childMapping: model.childMapping,
                durationMapping: model.durationMapping,
                progressMapping: model.progressMapping,
                predecessorMapping: model.predecessorMapping,
                rowHeight: model.rowHeight,
                emptyRecordText: proxy._emptyRecordText,
                isFromGantt: proxy._isFromGantt,
                treeColumnIndex: model.treeColumnIndex,
                workingTimeScale:model.workingTimeScale,
                columnMenuTexts: proxy._columnMenuTexts,
                dataManagerUpdate: { isDataManagerUpdate: proxy._isDataManagerUpdate, jsonData: proxy._jsonData },
            });
        },
        //Chart part is initiated
        _renderGanttChart: function() {
            var proxy = this,
                model = this.model;
            proxy._$ganttchartHelper = $("#ejGanttChart" + proxy._id).ejGanttChart(
                {
                    dataSource: this.dataSource(),
                    scheduleStartDate: model.scheduleStartDate,
                    scheduleEndDate: model.scheduleEndDate,
                    startDateMapping: model.startDateMapping,
                    endDateMapping: model.endDateMapping,
                    progressMapping: model.progressMapping,
                    durationMapping: model.durationMapping,
                    childPropertyName: model.childMapping,
                    predecessorMapping: model.predecessorMapping,
                    enableVirtualization: model.enableVirtualization,
                    highlightWeekends: model.highlightWeekends,
                    milestoneMapping: model.milestoneMapping,
                    enableRowHover: model.enableRowHover,
                    perDayWidth: proxy._perDayWidth,
                    workingTimeScale: model.workingTimeScale,
                    roundOffDayworkingTime: model.roundOffDayworkingTime,
                    durationUnit: model.durationUnit,
                    
                    perMonthWidth: proxy._perMonthWidth,//new property for year - month schedule
                    perWeekWidth: proxy._perWeekWidth,//new property for week - month schedule
                    perHourWidth: proxy._perHourWidth,
                    perMinuteWidth: proxy._perMinuteWidth,
                    perTopHourWidth: proxy._perTopHourWidth,
                    minuteInterval:proxy._minuteInterval,
                    includeWeekend: model.includeWeekend,
                    connectorlineWidth: model.connectorlineWidth,
                    scheduleHeaderSettings: model.scheduleHeaderSettings,
                    taskbarBackground: model.taskbarBackground,
                    progressbarBackground: model.progressbarBackground,
                    connectorLineBackground: model.connectorLineBackground,
                    parentTaskbarBackground: model.parentTaskbarBackground,
                    parentProgressbarBackground: model.parentProgressbarBackground,
                    holidays: model.holidays,
                    dateFormat: model.dateFormat,
                    locale: model.locale,
                    enableTaskbarTooltip: model.enableTaskbarTooltip,
                    enableTaskbarDragTooltip:model.enableTaskbarDragTooltip,
                    flatRecords: model.flatRecords,
                    parentRecords: model.parentRecords,
                    scheduleWeeks: proxy._scheduleWeeks,

                    projectStartDate: proxy._projectStartDate, //new property instead of scheduleWeeks[0] and scheduleYears[0]
                    projectEndDate: proxy._projectEndDate,

                    scheduleYears: proxy._scheduleYears, //new property for year
                    scheduleMonths: proxy._scheduleMonths, //new property for month
                    scheduleDays:proxy._scheduleDays,
                    scheduleHours:proxy._scheduleHours,
                    progressbarHeight: model.progressbarHeight,
                    tooltipTemplate: model.taskbarTooltipTemplate,
                    tooltipTemplateId: model.taskbarTooltipTemplateId,
                    progressbarTooltipTemplate: model.progressbarTooltipTemplate,
                    progressbarTooltipTemplateId: model.progressbarTooltipTemplateId,
                    queryTaskbarInfo: model.queryTaskbarInfo,
                    showTaskNames: model.showTaskNames,
                    showProgressStatus: model.showProgressStatus,
                    showResourceNames: model.showResourceNames,
                    resourceInfoMapping: model.resourceInfoMapping,
                    resourceNameMapping: model.resourceNameMapping,
                    enableProgressBarResizing: model.enableProgressBarResizing,
                    allowGanttChartEditing: model.allowGanttChartEditing,
                    taskbarEditingTooltipTemplateId: model.taskbarEditingTooltipTemplateId,
                    taskbarEditingTooltipTemplate:model.taskbarEditingTooltipTemplate,
                    ids: proxy.model.ids,
                    stripLines: model.stripLines,
                    _predecessorCollection: proxy._predecessorsCollection,
                    weekendBackground: model.weekendBackground,
                    allowKeyboardNavigation: model.allowKeyboardNavigation,
                    updatedRecords: proxy.model.updatedRecords,
                    renderBaseline: model.renderBaseline,
                    baselineColor:model.baselineColor,
                    editSettings: model.editSettings,
                    columnHeaderTexts: proxy._columnHeaderTexts,
                    durationUnitTexts:proxy._durationUnitTexts,
                    predecessorEditingTexts: proxy._predecessorEditingTexts,
                    localizedDays: proxy._days,
                    localizedMonths:proxy._months,
                    rowHeight: model.rowHeight,
                    allowSelection:model.allowSelection
                });
        },
        //Populate columns for treegrid from mapping names
        createTreeGridColumns: function () {

            var proxy = this, model = proxy.model, column, columns = [];

            var mapping = (model.taskIdMapping)//?model.taskIdMapping:"Task Id";

            if (mapping.length) {
                if(model.showColumnChooser){
                    column = {
                        field: "taskId",
                        headerText: proxy._columnHeaderTexts["taskId"],
                        width: 40,
                        editType: ej.Gantt.EditingType.String,
                        mappingName: mapping
                    };
                }
                else
                {
                    column = {
                        field: "taskId",
                        headerText: proxy._columnHeaderTexts["taskId"],
                        width: 30,
                        editType: ej.Gantt.EditingType.String,
                        mappingName: mapping
                    };
                }

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.taskNameMapping;// "Task Name";

            if (mapping.length) {
                column = {
                    field: "taskName",
                    headerText: proxy._columnHeaderTexts["taskName"],
                    width: 150,
                    editType: ej.Gantt.EditingType.String,
                    mappingName: mapping
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.startDateMapping;// "Start Date";

            if (mapping.length) {
                column = {
                    field: "startDate",
                    headerText:proxy._columnHeaderTexts["startDate"],
                    width: 150,
                    editType: (model.dateFormat.indexOf("hh") == -1) ?
                              ej.Gantt.EditingType.DatePicker : ej.Gantt.EditingType.DateTimePicker,
                    mappingName: mapping,
                    format: "{0:" + model.dateFormat + "}"
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }


            mapping = model.endDateMapping;// "End Date";

            if (mapping.length) {
                column = {
                    field: "endDate",
                    headerText: proxy._columnHeaderTexts["endDate"],
                    width: 150,
                    editType: (model.dateFormat.indexOf("hh") == -1) ?
                             ej.Gantt.EditingType.DatePicker : ej.Gantt.EditingType.DateTimePicker,
                    mappingName: mapping,
                    format: "{0:" + model.dateFormat + "}"
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.resourceInfoMapping;//"Resources";

            if (mapping.length) {
                column = {
                    field: "resourceInfo",
                    headerText:proxy._columnHeaderTexts["resourceInfo"],
                    width: 150,
                    editType: ej.Gantt.EditingType.Dropdown,
                    mappingName: mapping,
                    dropdownData: model.resources //model.resources is null
                };
                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.durationMapping;// "Duration";

            if (mapping.length) {
                column = {
                    field: "duration",
                    headerText: proxy._columnHeaderTexts["duration"],
                    width: 150,
                    editType: ej.Gantt.EditingType.Numeric,
                    mappingName:mapping
                };

                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }

            mapping = model.progressMapping;// "Progress";

            if (mapping.length) {
                column = {
                    field: "status",
                    headerText: proxy._columnHeaderTexts["status"],
                    width: 150,
                    editType: ej.Gantt.EditingType.Numeric,
                    mappingName:mapping
                };
                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }
            mapping = model.predecessorMapping;//)?model.predecessorMapping: "Predecessor";

            if (mapping.length) {
                column = {
                    field: "predecessor",
                    headerText:proxy._columnHeaderTexts["predecessor"],
                    width: 150,
                    editType: ej.Gantt.EditingType.String,
                    mappingName: mapping
                };
                columns.push(column);
                //proxy._emptyDataColumns.push(column.field);
            }
            
            //mapping = (model.baselineStartDateMapping )?model.baselineStartDateMapping: "Baseline StartDate";

            //if (mapping.length) {
            //    column = {
            //        field: "baselineStartDate",
            //        headerText: proxy._columnHeaderTexts["baselineStartDate"],
            //        width: 150,
            //        editType: ej.Gantt.EditingType.DatePicker,
            //        mappingName: mapping
            //    };
            //    columns.push(column);
            //    proxy._emptyDataColumns.push(column.field);
            //}
            //mapping = (model.baselineEndDateMapping)?model.baselineEndDateMapping: "Baseline EndDate";

            //if (mapping.length) {
            //    column = {
            //        field: "baselineEndDate",
            //        headerText:proxy._columnHeaderTexts["baselineEndDate"],
            //        width: 150,
            //        editType: ej.Gantt.EditingType.DatePicker,
            //        mappingName: mapping
            //    };
            //    columns.push(column);
            //    proxy._emptyDataColumns.push(column.field);
            //}
            return columns;
        },
        
        //Create gannt record from given data source
        _createGanttRecords: function(data) {
            var proxy = this,
                model = proxy.model,
                ganttRecords = model.flatRecords,
                length = data.length,
                count = 0,
                parentGanttRecord,
                parentRecords = model.parentRecords,
                enableSorting = model.allowSorting,
                enableAltRow = model.enableAltRow,
                ids = proxy.model.ids;


            for (count; count < length; count++) {
                parentGanttRecord = proxy._createGanttRecord(data[count], 0, null,undefined,"Load");
                proxy._storedIndex++;
                //if (enableAltRow)
                parentGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                parentGanttRecord.index = proxy._storedIndex;
                
                ids[proxy._storedIndex] = parentGanttRecord["taskId"].toString();
                ganttRecords.push(parentGanttRecord);
                //if (enableSorting)
                parentRecords.push(parentGanttRecord);
                parentGanttRecord.childRecords && proxy._addNestedGanttRecords(parentGanttRecord.childRecords);
            }
        },
        
        //Calculate schedule dates from GanttRecord Collections
        _calculateScheduleDates:function(editArgs)
        {
            var proxy = this,
                minStartDate,
                maxEndDate,
                tempStartDate,
                tempEndDate,
                model = this.model,
                records = model.flatRecords,
                updatedDates;
            if (((!model.scheduleEndDate || !model.scheduleStartDate) && records.length > 0)  || editArgs) {

                $.each(records, function (index, value) {
                        tempStartDate = new Date(value.startDate);
                        tempEndDate = new Date(value.endDate);
                        baseLineStartDate = value.baselineStartDate ? new Date(value.baselineStartDate) : null;
                        baseLineEndDate = value.baselineEndDate ? new Date(value.baselineEndDate) : null;
                        //minimum startDate
                        if (minStartDate) {
                            if (minStartDate.getTime() >= tempStartDate.getTime()) {
                                minStartDate = tempStartDate;
                            }
                            if (baseLineStartDate && model.renderBaseline && (minStartDate.getTime() >= baseLineStartDate.getTime())) {
                                minStartDate = baseLineStartDate;
                            }
                        } else {
                            if (baseLineStartDate && model.renderBaseline && tempStartDate.getTime() >= baseLineStartDate.getTime())
                                minStartDate = baseLineStartDate;
                            else
                                minStartDate = tempStartDate;
                        }
                        //Maximum endDate
                        if (maxEndDate) {
                            if (maxEndDate.getTime() <= tempEndDate.getTime()) {
                                maxEndDate = tempEndDate;
                            }
                            if (baseLineEndDate && model.renderBaseline && (maxEndDate.getTime() <= baseLineEndDate.getTime())) {
                                maxEndDate = baseLineEndDate;
                            }

                        } else {
                            if (baseLineEndDate && model.renderBaseline && tempEndDate.getTime() <= baseLineEndDate.getTime())
                                maxEndDate = baseLineEndDate;
                            else
                                maxEndDate = tempEndDate;
                        }
                });
                updatedDates = proxy._updateScheduleDatesByTaskLables(minStartDate, maxEndDate);
                minStartDate = updatedDates.minStartDate;
                maxEndDate = updatedDates.maxEndDate;
            }
            else if ((!model.scheduleEndDate || !model.scheduleStartDate) && records.length === 0) {
                minStartDate = proxy._getDateFromFormat(new Date());
                maxEndDate = proxy._getDateFromFormat(new Date(minStartDate));
                maxEndDate.setDate(maxEndDate.getDate() + 20);
            }
            if (!editArgs) {
                model.scheduleStartDate = minStartDate ? proxy.getFormatedDate(minStartDate) : model.scheduleStartDate;
                model.scheduleEndDate = maxEndDate ? proxy.getFormatedDate(maxEndDate) : model.scheduleEndDate;
            }
            else {
                editArgs.minStartDate = minStartDate;
                editArgs.maxEndDate = maxEndDate;
            }

        },

        //update schedule dates as per showTaskName and showResourceName values  and 
        //holidays collection and strip line collections at loadtime and edit time
        _updateScheduleDatesByTaskLables: function (startDate, endDate)
        {
            var proxy = this,
                model = this.model,
                scheduleEndDate,
                scheduleHeaderType = model.scheduleHeaderSettings.scheduleHeaderType,
                scheduleHeaderValue = ej.Gantt.ScheduleHeaderType,
                resourceStringWidth = proxy._getResourceStringWidth(),
                numOfDays, returnValues = {},
                scheduleEndDate = model.scheduleEndDate && this._getDateFromFormat(model.scheduleEndDate);
              
            if (scheduleHeaderType == scheduleHeaderValue.Week) {
                //if (model.showTaskNames) {
                //    startDate.setDate(startDate.getDate() - 7);//one week before
                //}
                if (!model.scheduleEndDate || (scheduleEndDate.getTime() !== endDate.getTime()))
                {
                    numOfDays = resourceStringWidth > 0 ? (resourceStringWidth / proxy._perDayWidth)+7 : 7;//one week after
                    endDate.setDate(endDate.getDate() + numOfDays);
                }
            }
            if (scheduleHeaderType == scheduleHeaderValue.Year) {
                //if (model.showTaskNames) {
                //    startDate.setMonth(startDate.getMonth() - 2);//2 month before
                //}
                if (!model.scheduleEndDate || (scheduleEndDate.getTime() !== endDate.getTime())) {
                    numOfDays = resourceStringWidth > 0 ? (resourceStringWidth / proxy._perMonthWidth) + 1 : 1;//1 month after
                    endDate.setMonth(endDate.getMonth() + numOfDays);
                }
            }
             

            if (scheduleHeaderType == scheduleHeaderValue.Month) {
                //if (model.showTaskNames) {
                //    startDate.setDate(startDate.getDate() - 14);//two week before
                //}
                if (!model.scheduleEndDate || (scheduleEndDate.getTime() !== endDate.getTime())) {
                    numOfDays = resourceStringWidth > 0 ? (resourceStringWidth / proxy._perDayWidth) + 7 : 7;//one week after
                    endDate.setDate(endDate.getDate() + numOfDays);
                }
            }


            if (scheduleHeaderType == scheduleHeaderValue.Day) {
                //if (model.showTaskNames) {
                //    startDate.setHours(startDate.getHours() - 8);//8hrs before
                //}
                if (!model.scheduleEndDate || (scheduleEndDate.getTime() !== endDate.getTime())) {
                    numOfDays = resourceStringWidth > 0 ? (resourceStringWidth / proxy._perHourWidth) + 7 : 7;//7hrs after
                    endDate.setHours(endDate.getHours() + numOfDays);
                }
            }
            if (scheduleHeaderType == scheduleHeaderValue.Hour) {
                if (!model.scheduleEndDate || (scheduleEndDate.getTime() !== endDate.getTime())) {
                    numOfHours = resourceStringWidth > 0 ? (resourceStringWidth / proxy._perMinuteWidth) + 5 : 5;//5hrs after
                    endDate.setHours(endDate.getHours() + numOfHours);
                    startDate.setHours(startDate.getHours() - 1);//Add 1 hour before.
                }
            }

            /* Update schedule dates as per holiday and strip line collection */
            returnValues = proxy._updateScheduleDatesByStripLines(startDate, endDate);

            startDate = returnValues.startDate;
            endDate = returnValues.endDate;


            returnValues = proxy._updateScheduleDatesByHolidays(startDate, endDate);

            startDate = returnValues.startDate;
            endDate = returnValues.endDate;

            returnValues.minStartDate = startDate;
            returnValues.maxEndDate = endDate;
           
            return returnValues;

        },

         //update Schedule dates by holidays collection */
        _updateScheduleDatesByHolidays: function (startDate, endDate)
        {
            var proxy = this, model = this.model, returnValues = {};

            //Check holidays are in range of schedule dates
            if (proxy._holidaysList.length > 0) {
                $.each(proxy._holidaysList, function (idx, value) {
                    if (!(value.getTime() > startDate.getTime()) || !(value.getTime() < endDate.getTime())) {
                        if (value.getTime() < startDate.getTime())
                            startDate = new Date(value);
                        else
                            endDate = new Date(value);
                    }
                });
            }

            returnValues.startDate = startDate;
            returnValues.endDate = endDate;

            return returnValues;


        },

        //update Schedule dates by stripLines collection */
        _updateScheduleDatesByStripLines: function (startDate, endDate) {
            var proxy = this, model = this.model, returnValues = {};

            //Check striplines are in range of schedule dates
            if (model.stripLines.length > 0) {
                $.each(model.stripLines, function (idx, value) {
                    var currentValue = proxy._getDateFromFormat(value.day)
                    if (!(currentValue.getTime() > startDate.getTime()) || !(currentValue.getTime() < endDate.getTime())) {
                        if (currentValue.getTime() < startDate.getTime())
                            startDate = new Date(currentValue);
                        else
                            endDate = new Date(currentValue);
                    }
                });
            }
            returnValues.startDate = startDate;
            returnValues.endDate = endDate;

            return returnValues;
        },

        //Get reosurce collections width
        _getResourceStringWidth:function(){
            var proxy = this,
                model = this.model,
                resource = model.resources,
                totalString = "",width;
            if (resource.length > 0 && model.showResourceNames) {
                for (var count = 0; count < resource.length; count++) {
                    totalString += resource[count][model.resourceNameMapping] + " ,";
                }
                var $span = ej.buildTag("span", "", { "font-family": "Segoe UI", "font-size": "12px" }, {});
                $span.text(totalString);
                $(document.body).append($span);
                width = $span.width();
                $span.remove();
                return width;
            } else {
                return 0;
            }
        },

        //calculate project dates by schedule mode types
        _calculateHeaderDates: function () {

            var proxy = this,
                model=this.model,
                startDate = proxy._getDateFromFormat(model.scheduleStartDate),
                endDate = proxy._getDateFromFormat(model.scheduleEndDate),
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                headerValue = ej.Gantt.ScheduleHeaderType;

            if (headerType == headerValue.Week)
                proxy._calculateWeekSplit(startDate, endDate);

            if (headerType == headerValue.Year) {
                proxy._calculateYearSplit(startDate, endDate);//method for year split
            }

            if (headerType == headerValue.Month)
                proxy._calculateMonthSplit(startDate, endDate);//method for Month split

            if (headerType == headerValue.Day)
                proxy._calculateDaySplit(startDate, endDate);

            if (headerType == headerValue.Hour)
                proxy._calculateHourSplit(startDate, endDate);
        },
        //AddNested Gantt Records
        _addNestedGanttRecords: function (data) {
            var proxy = this,
               
                ids = proxy.model.ids,
                model = proxy.model,
                ganttRecords = model.flatRecords,
                count = 0,
                length = data.length,
                records = [],
                record,
                enableAltRow = model.enableAltRow;
            
            for (count; count < length; count++) {
                record = data[count];
                proxy._storedIndex++;
                //if (enableAltRow)
                record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;;
                record.index = proxy._storedIndex;
              
                ids[proxy._storedIndex] = record["taskId"].toString();
                ganttRecords.push(record);
                records = record.childRecords;
                if (records) {
                    var j = 0, recordlength = records.length, childGanttRecord = null;
                    for (j; j < recordlength; j++) {
                        childGanttRecord = records[j];
                        proxy._storedIndex++;
                        //if (enableAltRow)
                        childGanttRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        childGanttRecord.index = proxy._storedIndex;
                     
                        ids[proxy._storedIndex] = childGanttRecord["taskId"].toString();
                        ganttRecords.push(childGanttRecord);
                        childGanttRecord.childRecords && proxy._addNestedGanttRecords(childGanttRecord.childRecords);
                    }
                }

            }
            
        },
        //returns gantt record from data
        _createGanttRecord: function (data, level, parentItem, expanded, creatAt) {
            var proxy = this, ganttRecord, model = proxy.model, //column = proxy._emptyDataColumns,
                columnRecords = proxy._columns, columnRecordsLength = columnRecords.length,
                child = data[model.childMapping],
                duration = data[model.durationMapping] ? (!isNaN(parseFloat(data[model.durationMapping])) ? parseFloat(data[model.durationMapping]) : 0) : 0,
                status = data[model.progressMapping] ? (!isNaN(parseFloat(data[model.progressMapping])) ? parseFloat(data[model.progressMapping]) : 0) : 0,
                startDateType = proxy._getDateFromFormat(data[model.startDateMapping]),
                endDateType = proxy._getDateFromFormat(data[model.endDateMapping]),
                resourceInfo = data[model.resourceInfoMapping],
                predecessors = data[model.predecessorMapping],
                baselineStartDateType = proxy._getDateFromFormat(data[model.baselineStartDateMapping]),
                baselineEndDateType = proxy._getDateFromFormat(data[model.baselineEndDateMapping]),
                resourceName = [],
                predecessorscol = [];

            var sTime = startDateType ? new Date(startDateType.getFullYear(), startDateType.getMonth(),
                startDateType.getDate(), startDateType.getHours(), startDateType.getMinutes()).getTime() : null,
                eTime = endDateType ? new Date(endDateType.getFullYear(), endDateType.getMonth(),
                endDateType.getDate(), endDateType.getHours(), endDateType.getMinutes()).getTime() : null;

            ganttRecord = new ej.Gantt.GanttRecord();
            ganttRecord.taskId = data[model.taskIdMapping],
            ganttRecord.taskName = data[model.taskNameMapping],
            ganttRecord.startDate = startDateType,
            ganttRecord.endDate = endDateType,

            ganttRecord.taskbarBackground = null,
            ganttRecord.progressbarBackground = null,
            ganttRecord.parentProgressbarBackground = null,
            ganttRecord.parentTaskbarBackground = null,

            ganttRecord.cellBackgroundColor = null,
            ganttRecord.rowBackgroundColor = null,
            ganttRecord.treeMappingName = [],

            ganttRecord.baselineStartDate = baselineStartDateType,
            ganttRecord.baselineEndDate = baselineEndDateType,
            ganttRecord.duration = duration;

            // updating Gantt record for custom columns while adding new row
            if (columnRecordsLength) {
                for (var i = 0; i < columnRecordsLength; i++) {
                    if (!ganttRecord[columnRecords[i].field])
                        ganttRecord[columnRecords[i].field] = data[columnRecords[i].mappingName];
                }
            }

            if (endDateType && (ej.isNullOrUndefined(duration) || sTime != eTime)) {
                ganttRecord._calculateDuration(startDateType, endDateType, model.includeWeekend, model.holidays, data[model.milestoneMapping],model);
                duration = ganttRecord.duration;
            }
            ganttRecord.isMilestone = !ej.isNullOrUndefined(data[model.milestoneMapping]) ? data[model.milestoneMapping] : (duration == 0 ? true : false);

            // To avoid the milestone taskbar for the same start and end date task item
            if (sTime == eTime && !ganttRecord.isMilestone)
                ganttRecord.duration = 1; // Also for non-milestone child item, duration is required to calculate  parent progress of the item
            else
                ganttRecord.duration = (ganttRecord.isMilestone && duration != 0) ? 0 : duration;

            ganttRecord.status = status,
            ganttRecord.predecessor = predecessors && ganttRecord._calculatePredecessor(predecessors);
            ganttRecord.predecessorsName = predecessors;

            ganttRecord.resourceInfo = resourceInfo && ganttRecord._setResourceInfo(data[(model.resourceInfoMapping)], model.resourceIdMapping, model.resourceNameMapping, model.resources);
            proxy._updateResourceName(ganttRecord);
            ganttRecord.parentItem = parentItem;
            var tempData;
            if (model.parentTaskIdMapping && creatAt)
            {
                tempData = ej.DataManager(proxy._retrivedData).executeLocal(ej.Query().where(model.taskIdMapping,
                ej.FilterOperators.equal,
                ganttRecord.taskId));
                ganttRecord.item = tempData[0];
            }
            else
            {
                if (this.dataSource() instanceof ej.DataManager && this.dataSource().dataSource.json && this.dataSource().dataSource.offline && creatAt) {
                var tempData = ej.DataManager(proxy._retrivedData).executeLocal(ej.Query().where(model.taskIdMapping,
                ej.FilterOperators.equal,
                ganttRecord.taskId));
                if (tempData.length > 0) {
                    ganttRecord.item = tempData[0];
                } else {
                    ganttRecord.item = data;
                }
                }
                else {
                    ganttRecord.item = data;
                }
            }
            ganttRecord.isSelected = false,
            ganttRecord.level = level;
            if(!ganttRecord.endDate)
            {
                ganttRecord._calculateEndDate(model.includeWeekend, this._holidaysList, model, proxy._perDayWidth);
            }

            ganttRecord.left = ganttRecord._calculateLeft(proxy._projectStartDate,
                               proxy._perDayWidth, proxy._holidaysList, model);

            if (model.renderBaseline && ganttRecord.baselineStartDate && ganttRecord.baselineEndDate)
                ganttRecord.baselineLeft = ganttRecord._calculateBaselineLeft(proxy._projectStartDate, proxy._perDayWidth, model.includeWeekend, model.dateFormat, model.endDateMapping, proxy._holidaysList);
            else
                model.renderBaseline = false;

            if (!creatAt) {
                ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                ganttRecord.width = ganttRecord.width == 0 ? 26 : ganttRecord.width;
                ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, status);

                if (model.renderBaseline && ganttRecord.baselineStartDate && ganttRecord.baselineEndDate) {
                    ganttRecord.baselineWidth = ganttRecord._calculateBaseLineWidth(proxy._perDayWidth, model);
                }
            }

            ganttRecord.childRecords = (child && child.length > 0) && proxy._createChildRecords(child, level + 1, ganttRecord),
            ganttRecord.hasChildRecords = (child && child.length > 0) ? true : false,
            ganttRecord.isMilestone = ganttRecord.hasChildRecords ? false : ganttRecord.isMilestone,
            ganttRecord.expanded = expanded !== undefined ? expanded : child ? child.length > 0 : false;
            if (model.enableVirtualization === false)
            {
                ganttRecord.isExpanded=true;
            }

            if (predecessors)
                proxy._predecessorsCollection.push(ganttRecord);
            return ganttRecord;
        },
        /* udpate resourceNames attribute of record*/
        _updateResourceName: function (ganttRecord) {
            var proxy = this, resourceInfo = ganttRecord.resourceInfo,
                resourceName = [], length;

            if (resourceInfo) {
                var length = resourceInfo.length;
                for (var i = 0; i < length; i++)
                    resourceName.push(resourceInfo[i][this.model.resourceNameMapping]);
                ganttRecord.resourceNames = resourceName.join(',');
            }
        },
        _getHoliday: function () {
            if (this.model.holidays != null) {
                var holidayList = this.model.holidays,
                    holidays = [],
                    holidayLength = holidayList.length;
                if (holidayLength > 0) {
                    for (var i = 0; i < holidayLength; i++) {
                        holidays[i] = this._getDateFromFormat(holidayList[i].day);
                    }
                    return holidays;
                }
            }
            return false;
        },

        _getStringHolidays: function() {
            if (this._holidaysList != null && this._holidaysList.length > 0) {
                var holidayList = this._holidaysList,
                    holidays = [],
                    holidayLength = holidayList.length;
                if (holidayLength > 0) {
                    for (var i = 0; i < holidayLength; i++) {
                        holidays[i] = Globalize.format(holidayList[i], this.model.dateFormat);
                    }
                    return holidays;
                }
            }
            return [];
        },
        
        //create childRecords for particular GanttRecord
        _createChildRecords: function (data, level, parentItem) {
            var ganttRecords = [],
                proxy = this,
                i = 0,
                length = data.length;

            for (i = 0; i < length; i++) {
                if (data[i]) {
                    var parentRecord = proxy._createGanttRecord(data[i], level, parentItem,undefined,"Load");
                    ganttRecords.push(parentRecord);
                }
            }
            return ganttRecords;
        },

        _createResourceInfoCollection: function (data) {
            var proxy = this,
                count = 0,
                length = data.length;
            for (count; count < length; count++) {
                proxy._resourceInfoCollection.push(proxy._createResourceInfo(data[count]));
            }
        },

        _createResourceInfo: function (data) {
            var model = this.model, resourceInfo = {};
            resourceInfo.resourceId = data[model.resourceIdMapping];
            resourceInfo.resourceName = data[model.resourceNameMapping];
            return resourceInfo;
        },

        _calculateWeekSplit: function (startdate, enddate) {
            var proxy = this;
            var roundOffStartDate = startdate.getDay() == 0 ? (startdate.getDate()) - 7 :
                (startdate.getDate()) - startdate.getDay();
            startdate.setDate(roundOffStartDate);
            do {
                proxy._scheduleWeeks.push(new Date(startdate));
                startdate.setDate(startdate.getDate() + 7);
            } while (!(startdate > enddate));

            proxy._projectStartDate= new Date(proxy._scheduleWeeks[0]);
            proxy._projectEndDate = new Date(proxy._scheduleWeeks[proxy._scheduleWeeks.length - 1]);
        },

        //method for calculatting years (year-month schedule mode)
        _calculateYearSplit: function (startdate, enddate) {
            var proxy = this;
            startdate.setMonth(0);
            startdate.setDate(1);
            do {
                proxy._scheduleYears.push(new Date(startdate));
                startdate.setMonth(startdate.getMonth() + 12);
            } while (!(startdate >= enddate));

            var projectStartDate, projectEndDate;
            projectStartDate = new Date(proxy._scheduleYears[0].getFullYear(), 0, 1);
            projectEndDate = new Date(proxy._scheduleYears[proxy._scheduleYears.length - 1].getFullYear(), 11, 31);
            proxy._projectStartDate = projectStartDate;
            proxy._projectEndDate = projectEndDate;
        },

        //Method for calculating days (day-hour schedule mode)
        _calculateDaySplit: function (startdate, enddate) {
            var proxy = this;
            startdate.setHours(0);
            do {
                proxy._scheduleDays.push(new Date(startdate));
                startdate.setDate(startdate.getDate() + 1);
            } while ((startdate <= enddate));
            proxy._projectStartDate = new Date(proxy._scheduleDays[0]);
            proxy._projectEndDate = new Date(proxy._scheduleDays[proxy._scheduleDays.length - 1]);
        },

        //Method for calculating hours (Hour-Minute schedule mode)
        _calculateHourSplit: function (startdate, enddate) {
            var proxy = this;
            startdate.setMinutes(0);
            do {
                proxy._scheduleHours.push(new Date(startdate));               
                startdate.setHours(startdate.getHours() + 1);                            
            } while (startdate <= enddate);
            proxy._projectStartDate = new Date(proxy._scheduleHours[0]);          
            proxy._projectEndDate = new Date(proxy._scheduleHours[proxy._scheduleHours.length - 1]);
        },
        //new method for calculatting months
        _calculateMonthSplit: function (startdate, enddate) {
            var proxy = this;
            dt = new Date(startdate.getFullYear(), startdate.getMonth() - 1, 1);
            startdate = dt;

            do {
                proxy._scheduleMonths.push(new Date(startdate));
                if (startdate.getMonth() == 0 || startdate.getMonth() == 2 || startdate.getMonth() == 4 || startdate.getMonth() == 6 || startdate.getMonth() == 7 || startdate.getMonth() == 9 || startdate.getMonth() == 11)
                    startdate.setDate(startdate.getDate() + 31);
                else if (startdate.getMonth() == 3 || startdate.getMonth() == 5 || startdate.getMonth() == 8 || startdate.getMonth() == 10)
                    startdate.setDate(startdate.getDate() + 30);
                else if (startdate.getMonth() == 1) {
                    if(startdate.getFullYear()%4==0)
                        startdate.setDate(startdate.getDate() + 29);
                    else
                        startdate.setDate(startdate.getDate() + 28);
                }
            } while (!(startdate >= enddate));

            var projectStartDate, projectEndDate;
            projectStartDate = new Date(proxy._scheduleMonths[0]);
            projectEndDate = new Date(proxy._scheduleMonths[proxy._scheduleMonths.length - 1]);
            projectEndDate = new Date(projectEndDate.getFullYear(), projectEndDate.getMonth() + 1, 0);
            proxy._projectStartDate = projectStartDate;
            proxy._projectEndDate = projectEndDate;
        },



        _updatePredecessors: function () {
            var proxy = this,
                count = 0,
                predecessorsCollection = proxy._predecessorsCollection,
                length = predecessorsCollection.length,
                ganttRecord,
                connectorsCollection,
            i = 0,
            successorGanttRecord,
            connectorCount;
            for (count; count < length; count++) {
                ganttRecord = predecessorsCollection[count];
                connectorsCollection = ganttRecord["predecessor"];
                connectorCount = connectorsCollection.length;
                for (i=0; i < connectorCount; i++) {
                    var connector = connectorsCollection[i];
                    successorGanttRecord = proxy.model.flatRecords[proxy.model.ids.indexOf(connector.from)];
                    if (connector.from !== ganttRecord.taskId.toString()) {
                        if (successorGanttRecord) {
                            if (successorGanttRecord.predecessor) {
                                successorGanttRecord.predecessor.push(connector);
                            } else {
                                successorGanttRecord.predecessor = [];
                                successorGanttRecord.predecessor.push(connector);
                                predecessorsCollection.push(successorGanttRecord);
                            }
                        }
                    }

                }                
            }
        },
        
        getDateType: function (date) {
            if (date != null) {
                if (typeof (date) == "object") {
                    return date;
                }
                else {
                    var hyphenRegex = new RegExp("\\-", "g"),
                    dotRegex = new RegExp("\\.", "g");
                    date = date.replace(hyphenRegex, "/");
                    date = date.replace(dotRegex, "/");
                    return new Date(date);
                }
            }
            return null;
        },

        getFormatedDate: function(date) {
            return Globalize.format(date, this.model.dateFormat);
        },

        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },

        
        _wireEvents: function () {
            var proxy = this,
                model=proxy.model,
                treeGrid = $("#ejTreeGrid" + proxy._id),
                ganttChart = $("#ejGanttChart" + proxy._id),
                gantt = proxy._$ejGantt;

            proxy._on(proxy.element, "click", proxy.clickHandler);
            if (model.enableContextMenu) {
                proxy._on(proxy.element, "contextmenu", proxy._rightClick);
                proxy._on(proxy.element, "keyup", this._preventContextMenu);
            }

            if (model.enableResize) {
                proxy._on($(window), "resize", proxy.windowResize);
            }

            treeGrid.ejTreeGrid({ rowSelecting: $.proxy(proxy.rowSelecting, proxy) });
            treeGrid.ejTreeGrid({ rowSelected: $.proxy(proxy.rowSelected, proxy) });
            //treeGrid.ejTreeGrid({ rowHover: $.proxy(proxy._treeGridRowHover, proxy) });
            treeGrid.ejTreeGrid({ actionBegin: $.proxy(proxy.actionBegin, proxy) });
            treeGrid.ejTreeGrid({ actionComplete: $.proxy(proxy.actionComplete, proxy) });
            treeGrid.ejTreeGrid({ expandAllCollapseAllRequest: $.proxy(proxy.expandAllCollapseAllRequest, proxy) });
            treeGrid.ejTreeGrid({ contextMenuAction: $.proxy(proxy.contextMenuAction, proxy) });
            treeGrid.ejTreeGrid({ subContextMenuAction: $.proxy(proxy.subContextMenuAction, proxy) });
            treeGrid.ejTreeGrid({ setInitialData: $.proxy(proxy.setInitialData, proxy) });
            treeGrid.ejTreeGrid({ customContextMenuEvent: $.proxy(proxy.customContextMenuEvent, proxy) });
            

            if (model.queryCellInfo) {
                treeGrid.ejTreeGrid({ queryCellInfo: $.proxy(proxy._queryCellInfo, proxy) });
            }
            
            if (model.rowDataBound) {
                treeGrid.ejTreeGrid({ rowDataBound: $.proxy(proxy._rowDataBound, proxy) });
            }
            
            treeGrid.ejTreeGrid({ beginEdit: $.proxy(proxy._beginEdit, proxy) });
            treeGrid.ejTreeGrid({ endEdit: $.proxy(proxy._endEdit, proxy) });            
            
            treeGrid.ejTreeGrid({ expanding: $.proxy(proxy.expanding, proxy) });
            treeGrid.ejTreeGrid({ collapsing: $.proxy(proxy.collapsing, proxy) });
            treeGrid.ejTreeGrid({ expanded: $.proxy(proxy.expanded, proxy) });
            treeGrid.ejTreeGrid({ collapsed: $.proxy(proxy.collapsed, proxy) });
            treeGrid.ejTreeGrid({ refreshRow: $.proxy(proxy.refreshRow, proxy) });
            treeGrid.ejTreeGrid({ cancelEditCell: $.proxy(proxy.cancelEditCell, proxy) });
            
            
            ganttChart.ejGanttChart({ rowSelecting: $.proxy(proxy.rowSelecting, proxy) });
            ganttChart.ejGanttChart({ rowSelected: $.proxy(proxy.rowSelected, proxy) });
            ganttChart.ejGanttChart({ rowHover: $.proxy(proxy._ganttChartRowHover, proxy) });
            ganttChart.ejGanttChart({ actionBegin: $.proxy(proxy.actionBegin, proxy) });
            ganttChart.ejGanttChart({ actionComplete: $.proxy(proxy.chartactionComplete, proxy) });
            ganttChart.ejGanttChart({ refreshRow: $.proxy(proxy.refreshRowData, proxy) }); 
            ganttChart.ejGanttChart({ zooming: $.proxy(proxy.zoomingChart, proxy) });
            ganttChart.ejGanttChart({ queryTaskbarInfo: $.proxy(proxy.queryTaskbarInfo, proxy) });
            ganttChart.ejGanttChart({ expanding: $.proxy(proxy.expanding, proxy) });
            ganttChart.ejGanttChart({ collapsing: $.proxy(proxy.collapsing, proxy) });
            ganttChart.ejGanttChart({ expanded: $.proxy(proxy.expanded, proxy) });
            ganttChart.ejGanttChart({ collapsed: $.proxy(proxy.collapsed, proxy) });
            ganttChart.ejGanttChart({ expandAllCollapseAllRequest: $.proxy(proxy.expandAllCollapseAllRequest, proxy) });
            ganttChart.ejGanttChart({ taskbarEditing: $.proxy(proxy.taskbarEditing, proxy) });
            ganttChart.ejGanttChart({ taskbarEdited: $.proxy(proxy.taskbarEdited, proxy) });
            ganttChart.ejGanttChart({ clearColumnMenu: $.proxy(proxy.clearColumnMenu, proxy) });
            ganttChart.ejGanttChart({ deleteRow: $.proxy(proxy.deleteRow, proxy) });
            gantt.ejSplitter({ allowResizing: $.proxy(proxy._onResize, proxy) });
            proxy._on(proxy.element, "mousedown", proxy._mousedownhandler);
            proxy._enableEditingEvents();
        },
 
        /* prevent context menu action by menu option key*/
        _preventContextMenu: function (e) {
            if (e.keyCode == 93) {
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        },
        _onResize:function(args){
            var proxy = this,
                isPercentage = false,
                splitterPosition = proxy.splitterPosition();
            if (splitterPosition.indexOf("%") != -1) 
                isPercentage = true;
            else 
                isPercentage = false;
            if (isPercentage)
                proxy.splitterPosition(Math.round(args.prevPane.size / proxy._ganttWidth * 100)+ "%");
            else
                proxy.splitterPosition(Math.round(args.prevPane.size).toString());
            proxy._$treegridHelper.ejTreeGrid("refreshScroller", args.prevPane.size);
            proxy._$ganttchartHelper.ejGanttChart("refreshScroller", args.nextPane.size - 1);// chart content parent div added with 1px right border

        },

        clickHandler: function (e) {
            var proxy = this;
            var $target = $(e.target);

            //if ($target.closest(".e-popup").length == 0 &&
            //  $target.closest(".e-rowcell").find("#" + proxy._id + "EditForm").length == 0) {
                
            //}


            proxy._clearContextMenu();

            //Skip the scroll bar click in Gantt.
            if ($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vhandlespace") || $target.hasClass("e-vdown"))
                return;
            if ($target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hup") || $target.hasClass("e-hdown") || $target.hasClass("e-hhandlespace"))
                return;
            if (proxy._isinBeginEdit) {

                if (!$target.closest('td').hasClass("e-editedcell") && !$target.hasClass('e-date') && !$target.hasClass('e-arrow') && !$target.hasClass('e-down-arrow') && $target.closest("form#ejTreeGrid" + proxy._id + "EditForm").length == 0) {
                    proxy._$treegridHelper.ejTreeGrid("cancelEditCell");
                    proxy._isinBeginEdit = false;
                }
            }
            proxy.clearColumnMenu();
        },

        windowResize: function (e) {
            var proxy = this,
                ejGanttSplit = ("#e-ejSpliter" + this._id),
                toolbar = ("#" + this._id + "_toolbarItems"),
                ganttchart = ("#ejGanttChart" + this._id),
                treegrid = ("#ejTreeGrid" + this._id),
                ganttbody = (".e-ganttviewerbodyContianer"),
                gantthead = (".e-ganttviewerheaderContainer"),
                treegridhelp = ("#ejTreeGrid" + this._id),
                treegridcontent = ("#ejTreeGrid" + this._id + "e-gridcontent"),
                sizeSettingsWidth = proxy.model.sizeSettings.width,
                sizeSettingsHeight = proxy.model.sizeSettings.height,
                elementStyleHeight = proxy.element[0].style.height,
                elementStyleWidth = proxy.element[0].style.width,
                width, height, maxScrollWidth;
            //Width calculation
            if ((sizeSettingsWidth && (typeof(sizeSettingsWidth) != "number" && sizeSettingsWidth.indexOf("%") != -1)) || elementStyleWidth.indexOf("%") != -1) {

                var ganttWidth = sizeSettingsWidth ? sizeSettingsWidth : elementStyleWidth;
                var containerWidth = $(proxy.element).parent().width() ? $(proxy.element).parent().width() : $(proxy.element).width();
                width = (containerWidth / 100) * parseInt(ganttWidth);
                
            }
            else
                width = $(proxy.element).width();
            if (proxy.splitterPosition().indexOf("%") == -1){
                var splitterPosition = parseInt(proxy.splitterPosition()) + 7 + 2; // here 7 is splitter width, 2 is border width
                proxy._ganttWidth = width = Math.max(width, splitterPosition);
            }
            else
                proxy._ganttWidth = width;
            //height calculation
            if ((sizeSettingsHeight && (typeof(sizeSettingsWidth) != "number" && sizeSettingsHeight.indexOf("%") != -1 )) || elementStyleHeight.indexOf("%") != -1) {
                var ganttHeight = sizeSettingsHeight ? sizeSettingsHeight : elementStyleHeight;
                var containerHeight = $(proxy.element).parent().height() ? $(proxy.element).parent().height() : $(proxy.element).height();
                var tempHeight;
                if ($(proxy.element).parent().height())
                    tempHeight = (containerHeight / 100) * parseInt(ganttHeight);
                else
                    tempHeight = $(proxy.element).height();
                height = tempHeight;
            }
            else
                height = $(proxy.element).height();

            //Exclude bordor from width and height
            height = height - proxy._totalBorderHeight;
            width = Math.round(width);

            var ganttWidth, treeGridWidth, top,left;
            $(ejGanttSplit).css("width", width);
            $(ejGanttSplit).css("height", (height - $(toolbar).height()));
            $(ejGanttSplit).ejSplitter("refresh");
            $(toolbar).css("width", width);
            $(toolbar).children(".e-ul:first-child").css("width", width);

            ganttWidth = $(ganttchart).width();
            treeGridWidth = $(treegrid).width();
            $(ganttchart).height(height - $(toolbar).height());

            top = $(ganttbody).ejScroller("option", "scrollTop");
            left = $(ganttbody).ejScroller("option", "scrollLeft");
            //set height for scroller div
            $(ganttbody).ejScroller({
                width: ganttWidth,
                height: (height - $(toolbar).height() - $(gantthead).height() - parseInt($(gantthead).css("border-bottom-width"))),
            });
            $(ganttbody).ejScroller("refresh");
            maxScrollWidth = proxy._$ganttchartHelper.ejGanttChart("getMaxScrollWidth");
            if (left > maxScrollWidth)
                left = maxScrollWidth > 0 ? maxScrollWidth : 0;

            $(ganttbody).ejScroller("option", "scrollTop", top);
            $(ganttbody).ejScroller("option", "scrollLeft", left);

            $(treegridhelp).height(height - $(toolbar).height());
            $(treegridcontent).height((height - $(toolbar).height() - $(gantthead).height() - proxy._totalBorderHeight));
            left = $(treegridcontent).ejScroller("option", "scrollLeft");
            //$(treegridcontent).ejScroller("destroy";
            $(treegridcontent).width(treeGridWidth);
            //$(treegridcontent).ejScroller({
            //    width: treeGridWidth,
            //    scrollTop: top,
            //    height: 0
            //});
            $(treegridcontent).ejScroller("refresh");
            maxScrollWidth = proxy._$treegridHelper.ejTreeGrid("getMaxScrollWidth");
            if (left > maxScrollWidth)
                left = maxScrollWidth > 0 ? maxScrollWidth : 0;

            $(treegridcontent).ejScroller("option", "scrollLeft", left);

            proxy._$treegridHelper.ejTreeGrid("updateViewPortHeight");
            proxy._$ganttchartHelper.ejGanttChart("updateViewPortHeight");
            if(proxy.model.enableVirtualization)
            {
                proxy._$treegridHelper.ejTreeGrid("cancelRowEditCell");
                proxy._$treegridHelper.ejTreeGrid("processBindings");
                var tempArgs = {};
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", tempArgs);
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedRecords);
                proxy._$ganttchartHelper.ejGanttChart("onScrollHelper", top);
            }
            proxy.setSplitterPosition(proxy.splitterPosition());
        },

        
        // handler for mouse right click
        _rightClick: function (e) {
            e.preventDefault();
            var proxy = this,
                $target = $(e.target),
                div = $target.closest('tr'),
                $ganttGridRows,
                row,
                recordIndexr,
                item,selectIndex,
                args = {};
            proxy._$treegridHelper.ejTreeGrid("disableTooltip");
            proxy.clearColumnMenu();
            if ($(div).hasClass("e-chartcell") || $(div).hasClass("e-ganttrowcell")) {
                $ganttGridRows = proxy._$ganttchartHelper.ejGanttChart("getGanttChartRows");
                args.target = "ejTreeGrid";
            } else {
                $ganttGridRows = $(proxy._$treegridHelper.ejTreeGrid("getRows"));
                args.target = "ejGanttChart";
            }
            row = $target.closest('tr');
            recordIndexr = $ganttGridRows.index(row);
            proxy.model.currentViewData = proxy.getCurrentViewData();
            item = proxy.model.currentViewData[recordIndexr];
            recordIndexr = proxy.model.updatedRecords.indexOf(item);
            args.data = item;
            args.recordIndex = recordIndexr;

            if (item && proxy.model.enableContextMenu) {
                proxy._contextMenuItems = [];
                proxy._contextMenuItems = proxy._getContextMenuItems();
                proxy._subContextMenuItems = [];
                proxy._subContextMenuItems = proxy._getsubContextMenuItems();
                proxy._updateIndentOutdentContextmenuOption(item);
                proxy._clearContextMenu();
                if (proxy._contextMenuItems.length > 0)
                    proxy._renderContextMenu(e, recordIndexr, item);

                if (recordIndexr != proxy.model.selectedRowIndex)
                    proxy.rowSelected(args);
                proxy._$treegridHelper.ejTreeGrid("selectRows", recordIndexr);
                proxy._$ganttchartHelper.ejGanttChart("selectRows", recordIndexr);
            }
            else if (proxy.model.flatRecords.length === 0 && proxy.model.enableContextMenu) {
                proxy._contextMenuItems = [];
                proxy._subContextMenuItems = [];
                proxy._contextMenuItems = proxy._getContextMenuItems();
                proxy._updateIndentOutdentContextmenuOption(item);
                proxy._clearContextMenu();
                if (proxy._contextMenuItems.length > 0)
                    proxy._renderContextMenu(e, recordIndexr, item);
            }
        },
		 // Mouse down handler for gantt
        _mousedownhandler:function(e)
        {
            var $target = $(e.target),
            proxy = this;
            proxy._clearContextMenu();
            if ($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vdown") || $target.hasClass("e-vhandlespace") || $target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hdown") || $target.hasClass("e-hhandlespace") || $target.hasClass("e-splitbar")) {
                
                proxy.clearColumnMenu();
            }
        },
        
        //remove corresponding menu item in contextmenu collection by it menuId
        _removeContextMenuItem: function (id) {
            var proxy = this,
                index, contextMenuItems = proxy._contextMenuItems,
                filteredMenuItem;

            filteredMenuItem = contextMenuItems.filter(function (value) {
                if (value.menuId === id)
                    return true;
            });

            if (filteredMenuItem.length > 0) {
                index = contextMenuItems.indexOf(filteredMenuItem[0]);
                contextMenuItems.splice(index, 1);
            }
        },

        //Update contextmenu item according to selected item and editSettings API values
        _updateIndentOutdentContextmenuOption: function (ganttRecord) {
            var proxy = this,
                flatRecords = this.model.flatRecords,
                recordIndex = flatRecords.indexOf(ganttRecord);

            if (recordIndex === 0) {
                proxy._removeContextMenuItem("Indent");
                proxy._removeContextMenuItem("Outdent");
            }
            else if (recordIndex > 0) {
                if (ganttRecord.level === 0) {
                    proxy._removeContextMenuItem("Outdent");
                }
                else {
                    var previousGanttRecord = flatRecords[recordIndex - 1];
                    if (ganttRecord.level - previousGanttRecord.level === 1) {
                        proxy._removeContextMenuItem("Indent");
                    }
                }
            }
            else if (recordIndex < 0) {
                proxy._removeContextMenuItem("Task");
                proxy._removeContextMenuItem("Delete");
                proxy._removeContextMenuItem("Indent");
                proxy._removeContextMenuItem("Outdent");
            }

            if (!model.editSettings.allowAdding) {
                proxy._removeContextMenuItem("Add");
            }

            if (!model.editSettings.allowDeleting)
            {
                proxy._removeContextMenuItem("Delete");
            }

            if (!model.editSettings.allowEditing)
            {
                proxy._removeContextMenuItem("Task");
            }
        },
      
        queryTaskbarInfo: function (args) {        
            this._trigger("queryTaskbarInfo", args);
        },
        

        zoomingChart: function (args) {
            var proxy = this;
            proxy.reRenderChart(args.delta);
        },

        refreshRowData: function (args) {
            var proxy = this,
                treegridObject = proxy._$treegridHelper.ejTreeGrid("instance");
            ej.TreeGrid.refreshRow(treegridObject, args.index);
        },

        expanding: function (args) {
            var proxy = this;           
            if (!proxy._trigger("expanding", args)) {
                if (proxy.model.enableVirtualization) {                    
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                } else {
                    proxy._isInExpandCollapse = true;
                    //If need refresh chart if record is added after searching or sorting
                    if (proxy._isRefreshAddedRecord) {
                        args.data.expanded = true;
                        proxy._$treegridHelper.ejTreeGrid("updateExpandStatus", args.data, true);
                        proxy._refreshChartAndGridRows();
                        proxy._isRefreshAddedRecord = false;
                        return;
                    }
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    proxy._isInExpandCollapse = false;

                }
                proxy.updateSelectedItemIndex();
            }
        },

        collapsing: function (args) {
            var proxy = this, model= this.model,
                toolbar = $("#" + proxy._id + "_toolbarItems");
            if (!proxy._trigger("collapsing", args)) {
                if (proxy.model.enableVirtualization) {                    
                    ej.TreeGrid.sendExpandCollapseRequest(proxy,args);
                } else {
                    proxy._isInExpandCollapse = true;
                    //If need refresh chart if record is added after searching or sorting
                    if (proxy._isRefreshAddedRecord) {
                        args.data.expanded = false;
                        proxy._$treegridHelper.ejTreeGrid("updateExpandStatus", args.data, false);
                        proxy._refreshChartAndGridRows();
                        proxy._isRefreshAddedRecord = false;
                        return;
                    }
                    proxy._gridRows = proxy.getRows();
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    proxy._isInExpandCollapse = false;
                }
                var selectItem = proxy.selectedItem(), index;
                if (selectItem) {
                    var expanded = proxy.getExpandStatus(selectItem);
                    if (!expanded && !args.expanded) {
                        proxy._deSelectRowItem();
                        if (model.toolbarSettings.showToolbar) {
                            if (proxy.model.toolbarSettings.toolbarItems.indexOf("delete") !== -1) {
                                $(toolbar).ejToolbar('disableItem', $(toolbar).find(".e-deleteitem").parent()[0]);
                                if (proxy.model.toolbarSettings.toolbarItems.indexOf("indent") !== -1)
                                    $(toolbar).ejToolbar('disableItem', $(toolbar).find(".e-indent").parent()[0]);
                                if (proxy.model.toolbarSettings.toolbarItems.indexOf("outdent") !== -1)
                                    $(toolbar).ejToolbar('disableItem', $(toolbar).find(".e-outdent").parent()[0]);
                                if (proxy.model.toolbarSettings.toolbarItems.indexOf("edit") !== -1)
                                    $(toolbar).ejToolbar('disableItem', $(toolbar).find(".e-edititem").parent()[0]);
                            }
                        }
                    }
                    else {
                        proxy.updateSelectedItemIndex();
                    }
                }
            }
        },

        refreshRow: function (args) {
            var proxy = this;
            proxy._$ganttchartHelper.ejGanttChart("refreshRow", args.recordIndex);
        },

        _enableEditingEvents: function () {
            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing|| model.editSettings.editMode == "dialogTemplate" ||
                (model.toolbarSettings.showToolbar && ((model.editSettings.allowAdding && model.toolbarSettings.toolbarItems.indexOf("add") !== -1) ||
                    (model.editSettings.allowEditing && model.toolbarSettings.toolbarItems.indexOf("index") !== -1)))) {
                
                proxy._on($("#" + proxy._id + "_dialogEdit"),
                    "click keypress", "#EditDialog_" + proxy._id + "_Save ,#EditDialog_" + proxy._id + "_Cancel,#EditDialog_" + proxy._id + "_Delete", proxy._buttonClick);

                proxy._on($("#" + proxy._id + "_dialogAdd"),
                   "click keypress", "#AddDialog_" + proxy._id + "_Save ,#AddDialog_" + proxy._id + "_Cancel", proxy._buttonClick);
                // $(proxy.element).on("click", ".e-icon", $.proxy(proxy._buttonClick, proxy));
            }
        },
        
        cancelEditCell: function () {

            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("cancelEditCell");

        },

        _keyPressed: function (action, target, e) {
        },

        _buttonClick: function (e) {

            if (e.keyCode !== undefined && e.keyCode != 13)
                return true;
            
            var proxy = this,   
                model = proxy.model,
                args = {};
            
            if (model.editSettings.allowEditing || model.editSettings.editMode == "dialogTemplate" ||
                (model.toolbarSettings.showToolbar && ((model.editSettings.allowAdding && model.toolbarSettings.toolbarItems.indexOf("add") !== -1) ||
                    (model.editSettings.allowEditing && model.toolbarSettings.toolbarItems.indexOf("index") !== -1)))) {

                if (e.target.id == "EditDialog_" + proxy._id + "_Save") {
                    if (!proxy._sendSaveRequest("Edit"))
                        $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                } else if (e.target.id == "EditDialog_" + proxy._id + "_Cancel") {
                    $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                }
                    //For delete option in editing dialog window
                else if (e.target.id == "EditDialog_" + proxy._id + "_Delete") {

                    proxy._$treegridHelper.ejTreeGrid("deleteRow");
                    $("#" + proxy._id + "_dialogEdit").ejDialog("close");
                }
                else if (e.target.id == "AddDialog_" + proxy._id + "_Save") {
                    if (!proxy._sendSaveRequest("Add"))
                        if (!this._duplicate&&!this._wrongenddate) {
                            $("#" + proxy._id + "_dialogAdd").ejDialog("close");                            
                        }
                }
                else if (e.target.id == "AddDialog_" + proxy._id + "_Cancel") {
                  //  proxy._ids.splice(proxy._ids.length - 1, 1);
                    $("#" + proxy._id + "_dialogAdd").ejDialog("close");
                }
            }
            else {
                proxy._sendCancelRequest();
            }

            return false;
        },
        //trigger row selecting client side event
        rowSelecting: function (args) {
            var proxy = this, eventArgs = {}, model = this.model;
            if (proxy.model.rowSelecting) {
                eventArgs.recordIndex = args.recordIndex;
                eventArgs.previousIndex = args.previousIndex;
                eventArgs.previousData = model.updatedRecords[args.previousIndex];
                eventArgs.data = model.updatedRecords[args.recordIndex];
                eventArgs.previousChartRow = proxy._$ganttchartHelper.ejGanttChart("getRowByIndex", args.previousIndex);
                eventArgs.previousGridRow = ej.TreeGrid.getRowByIndex(this, args.previousIndex);
                eventArgs.targetChartRow = proxy._$ganttchartHelper.ejGanttChart("getRowByIndex", args.recordIndex);
                eventArgs.targetGridRow = ej.TreeGrid.getRowByIndex(this, args.recordIndex);
                if (proxy._trigger("rowSelecting", eventArgs))
                    return false;
            }
            return true;
        },

        rowSelected: function (args) {
            var proxy = this, model = this.model,
                toolbar = $("#" + proxy._id + "_toolbarItems");

            if (args.target == "ejTreeGrid")
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            if (args.target == "ejGanttChart")
                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
            var idx = args.recordIndex;
            this.selectedItem(args.data);
            this.selectedRowIndex(idx);
            if (proxy.model.toolbarSettings.showToolbar) {

                if (proxy.model.toolbarSettings.toolbarItems.indexOf("indent") !== -1 || proxy.model.toolbarSettings.toolbarItems.indexOf("outdent") !== -1) {

                    proxy.updateIndentOutdentOption(args.data);
                }

                var enableItems = [];

                if (proxy.model.editSettings.allowDeleting) {
                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("delete") !== -1)
                        enableItems.push($(toolbar).find(".e-deleteitem").parent()[0]);
                }
                if (proxy.model.editSettings.allowAdding) {
                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("add") !== -1)
                        enableItems.push($(toolbar).find(".e-addnewitem").parent()[0]);
                }
                if (proxy.model.editSettings.allowEditing) {
                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("edit") !== -1)
                        enableItems.push($(toolbar).find(".e-edititem").parent()[0]);
                }

                $(toolbar).ejToolbar('enableItem', enableItems);
            }

            var eventArgs = {};
            eventArgs.recordIndex = args.recordIndex;
            data = $.extend({}, model.updatedRecords[args.recordIndex]);
            /* Delete the childRecords and parentItem from data because of avoid circular reference
               while performing postback in ASP.NET */
            delete data.childRecords; 
            delete data.parentItem;
            eventArgs.data = data;
            eventArgs.targetChartRow = proxy._$ganttchartHelper.ejGanttChart("getRowByIndex", args.recordIndex);
            eventArgs.targetGridRow = ej.TreeGrid.getRowByIndex(this, args.recordIndex);
            proxy._trigger("rowSelected", eventArgs);
        },

        _treeGridRowHover: function (args) {
            var proxy = this;
            if (args.reason == "mouseenter")
                proxy._$ganttchartHelper.ejGanttChart("addRowHover", args.index);
            else if (args.reason == "mouseleave")
                proxy._$ganttchartHelper.ejGanttChart("removeRowHover");
        },

        updateIndentOutdentOption: function (ganttRecord) {
            var proxy = this,
                flatRecords = proxy.model.flatRecords;

            var recordIndex = flatRecords.indexOf(ganttRecord),
                previousGanttRecord;

            var toolbar = $("#" + proxy._id + "_toolbarItems"),
                indentIconElement = $(toolbar).find(".e-indent").parent()[0],
                outentIconElement = $(toolbar).find(".e-outdent").parent()[0];


            if (recordIndex === 0) {
                if (indentIconElement)
                    $(toolbar).ejToolbar('disableItem', indentIconElement);

                if (outentIconElement)
                    $(toolbar).ejToolbar('disableItem', outentIconElement);
            }
            else {
                if (ganttRecord.level === 0) {
                    if (indentIconElement)
                        $(toolbar).ejToolbar('enableItem', indentIconElement);
                    if (outentIconElement)
                        $(toolbar).ejToolbar('disableItem', outentIconElement);
                }
                else {
                    previousGanttRecord = flatRecords[recordIndex - 1];
                    if (ganttRecord.level - previousGanttRecord.level === 1) {
                        if (outentIconElement)
                            $(toolbar).ejToolbar('enableItem', outentIconElement);
                        if (indentIconElement)
                            $(toolbar).ejToolbar('disableItem', indentIconElement);
                    }
                    else {
                        if (indentIconElement)
                            $(toolbar).ejToolbar('enableItem', indentIconElement);
                        if (outentIconElement)
                            $(toolbar).ejToolbar('enableItem', outentIconElement);
                    }
                    
                }
            }
        },

        _ganttChartRowHover: function (args) {
            var proxy = this;
            if (args.reason == "mouseenter")
                proxy._$treegridHelper.ejTreeGrid("addRowHover", args.index);
            else if (args.reason == "mouseleave")
                proxy._$treegridHelper.ejTreeGrid("removeRowHover");
        },
        _removeChildRecordsPredecessor:function(record)
        {
            var proxy = this;
            if (record.hasChildRecords) {
                var length = record.childRecords.length, index = 0;
                for (index; index < length; index++) {
                    record.childRecords[index].predecessor && proxy._removePredecessor(record.childRecords[index].predecessor, record.childRecords[index]);
                    if(record.childRecords[index].hasChildRecords)
                    {
                        proxy._removeChildRecordsPredecessor(record.childRecords[index]);
                    }
                }
            }
        },
        getExpandedRecordCount: function (record, count)
        {
            var currentRecord, proxy = this;
            if (!record.hasChildRecords)
                return 0;
            for (var i = 0; i < record.childRecords.length; i++) {
                currentRecord = record.childRecords[i];
                if (proxy.getExpandStatus(currentRecord))
                    count++;
                
                if (currentRecord.hasChildRecords) {
                    count = proxy.getExpandedRecordCount(currentRecord, count);
                }
            }
            return count;
        },

        _updateExpandStatus:function()
        {
            var proxy = this,
                records = this.model.flatRecords,
            length = records.length, oldValue;
            for (var i = 0; i < length; i++) {
                oldValue = records[i].isExpanded;
                if (proxy.getExpandStatus(records[i]) !== records[i].isExpanded) {
                    records[i].isExpanded = proxy.getExpandStatus(records[i]);
                    if (records[i].isExpanded) {
                        proxy._totalCollapseRecordCount--;
                    }
                    else {
                        proxy._totalCollapseRecordCount++;
                    }
                }
            }
        },
        actionComplete: function (args) {
            var proxy = this,model = proxy.model,
                addedIndex = -1;
            proxy.model.updatedRecords = proxy.getUpdatedRecords();
            proxy.model.currentViewData = proxy.getCurrentViewData();
            proxy._gridRows = proxy.getRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();
            proxy._totalCollapseRecordCount = proxy._$treegridHelper.ejTreeGrid("getCollapsedRecordCount");

            if (args.requestType === "searching" || args.requestType === "sorting") {
                proxy._isRefreshAddedRecord = false;
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedRecords, proxy._totalCollapseRecordCount);
                proxy._isValidationEnabled = false;
                proxy._connectorlineIds = [];
                proxy._connectorLinesCollection = [];
                proxy._createConnectorLinesCollection();
            }

            if (args.requestType === "delete") {
                proxy._isRefreshAddedRecord = false;
                this.dataSource(proxy._$treegridHelper.ejTreeGrid("getDataSource"));
                if (proxy._isDataManagerUpdate)
                    this._jsonData = proxy._$treegridHelper.ejTreeGrid("getUpdatedDataManagerData");
                proxy.model.flatRecords = proxy._$treegridHelper.ejTreeGrid("getFlatRecords");
                proxy.model.ids = proxy._$treegridHelper.ejTreeGrid("getUpdatedIds");
                proxy.model.updatedRecords = proxy.getUpdatedRecords();
                model.selectedItem = null;
                this.selectedRowIndex(-1);
                var childCount = 0,
                    expandedRecordCount=0;
                //Delete from parent collection if item is in parentsCollection
                if (model.parentRecords.indexOf(args.data) !== -1) {
                    model.parentRecords.splice(model.parentRecords.indexOf(args.data), 1);
                }
                //Delete predecessor from child records
                if (args.data.hasChildRecords) {
                    proxy._removeChildRecordsPredecessor(args.data)
                }
                //Delete Predecessor from record
                args.data.predecessor && proxy._removePredecessor(args.data.predecessor, args.data);
                //Update parent item if item has parent
                if (args.data.parentItem)
                {
                    parentRecord=args.data.parentItem;
                    if(args.data.parentItem.childRecords.length>0)
                    {
                        proxy._updateParentItem(args.data);
                    }else
                    {
                        parentRecord.expanded=false;
                        parentRecord.hasChildRecords = false;
                        proxy.refreshGanttRecord(parentRecord);
                    }
                }
               
                proxy._isValidationEnabled = false;
                if (proxy.model.predecessorMapping) {
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._predecessorsCollection = [];
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._createConnectorLinesCollection();
                }
                //Scroller is not correctly updated before connector line cleared.
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.getCurrentViewData(), proxy.getUpdatedRecords(), proxy._totalCollapseRecordCount);

                proxy._updateToolbarOptions(); // for uypdating toolbar items after delete operation
                proxy.updateAltRow(proxy.getUpdatedRecords(), 0, 1);//for set e-alt-row style after deletion of element  
            }

            if (args.requestType === "selection") {
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            }

            if (args.requestType === "rowHover") {
                proxy._$ganttchartHelper.ejGanttChart("addRowHover", args.index);
            }
            
            if (args.requestType === "scroll") {
                
                proxy._$ganttchartHelper.ejGanttChart("onScrollHelper", args.delta);
            }

            if (args.requestType === "save") {
                if (args._cAddedRecord) {

                    proxy._renderAddedRow(args.index, args._cAddedRecord);
                   

                    if (args._cAddedRecord.parentItem)
                        proxy._updateParentItem(args._cAddedRecord);
                    proxy.refreshGanttRecord(args._cAddedRecord)

                    ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);

                    //proxy.updateAltRow(model.updatedRecords[0], 0, 0);

                    /* select newly added item */
                    proxy.selectRows(model.updatedRecords.indexOf(args._cAddedRecord));
                    addedIndex = proxy.selectedRowIndex();


                } else if (args._cModifiedData) {
                    args._cModifiedData.left = args._cModifiedData._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidaysList, model);
                    args._cModifiedData.width = args._cModifiedData._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                    args._cModifiedData.progressWidth = args._cModifiedData._getProgressWidth(args._cModifiedData.width, args._cModifiedData.status);
                    
                    proxy.refreshGanttRecord(args._cModifiedData);

                    if (args._cModifiedData.parentItem)
                        proxy._updateParentItem(args._cModifiedData);

                    if (args._cModifiedData.predecessor) {
                        if (args.previousValue) {
                            proxy._removeConnectorLine(args.previousValue, args._cModifiedData);
                        }
                    }
                }
                
                if (proxy.model.predecessorMapping) {

                    if ((args._cModifiedData && args._cModifiedData.predecessor)) {
                        proxy._isValidationEnabled = true;
                    }
                    else
                        proxy._isValidationEnabled = false;
                    if (args._cAddedRecord) {
                        if (args._cAddedRecord.predecessor) {
                            proxy._validatePredecessorDates(args._cAddedRecord);
                            proxy._addConnectorLine(args._cAddedRecord);
                        }
                        proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._createConnectorLinesCollection();
                    } else {
                        if (args._cModifiedData) {
                            if (args.previousValue) {
                                proxy._removeConnectorLine(args.previousValue, args._cModifiedData);
                            }
                            proxy._addConnectorLine(args._cModifiedData);
                            if (args._cModifiedData.predecessor) {
                                proxy._isMileStoneEdited = args._cModifiedData.isMilestone;
                                proxy._updatedConnectorLineCollection = [];
                                proxy._connectorlineIds = [];
                                proxy._validatePredecessor(args._cModifiedData);
                                if (proxy._updatedConnectorLineCollection.length > 0) {
                                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                                }
                            }
                            if (args._cModifiedData.parentItem)
                                proxy._updateParentItem(args._cModifiedData);
                        }
                    }
                }
            }

            if (args.requestType === "validatePredecessor") {
                var result = proxy._editedPredecessorValidation(args);
                args.result = result;
                return;
            }
            proxy._ganttActionCompleteTrigger(args);
            // Update the scrollbar for focus the newly added record
            if (addedIndex >= 0) {
                var ganttbody = proxy.element.find(".e-ganttviewerbodyContianer"),
                    scroller = ganttbody.ejScroller("instance"),
                    taskbarLeft = model.updatedRecords[addedIndex].left,
                    width = scroller.model.width,
                    scrollLeft = scroller.model.scrollLeft;
                    hScrollbar = scroller._hScrollbar;
                if (hScrollbar) {
                    if (hScrollbar.model.maximum < scrollLeft)
                        scrollLeft = hScrollbar.model.maximum;
                    if (scrollLeft > taskbarLeft || (scrollLeft + width) < taskbarLeft) {
                        var left = taskbarLeft - (width / 2);
                        left = left < 0 ? 0 : left;
                        ganttbody.ejScroller("scrollX", left, true);
                    }
                }
                proxy._$ganttchartHelper.ejGanttChart("updateScrollBar");
            }
        },

        expandAllCollapseAllRequest:function(args)
        {
            var proxy = this,
                $gridEle = $(proxy.element),//.closest(".e-gantt"),
                gridInstance = $gridEle.ejGantt("instance"),
                gridId = $gridEle.attr('id');
                gridInstance._toolbarOperation(gridId + "_" + args.requestType);
        },

        chartactionComplete: function (args) {
            var proxy = this;

            proxy.model.updatedRecords = proxy.getUpdatedRecords();
            proxy.model.currentViewData = proxy.getCurrentViewData();

            if (args.requestType === "updateConnectors") {
                if (proxy.model.predecessorMapping) {
                    proxy._isValidationEnabled = false;
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._createConnectorLinesCollection();
                }
            }

            if (args.requestType === "rowHover") {
                proxy._$treegridHelper.ejTreeGrid('addRowHover', args.index);
            }
            
            if (args.requestType === "scroll") {

                proxy._isTreeGridRendered = false;
                proxy._isGanttChartRendered = false;
                if (args.vscrollExsist && args.delta !== undefined)
                    proxy._$treegridHelper.ejTreeGrid('onScrollHelper', args.delta);

                proxy.model.updatedRecords = proxy.getUpdatedRecords();
                proxy.model.currentViewData = proxy.getCurrentViewData();

                if (proxy.model.enableVirtualization) {

                    if (proxy.model.predecessorMapping) {

                        if (proxy._connectorLinesCollection.length > 0) {
                            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                        }
                        proxy._isValidationEnabled = false;
                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._createConnectorLinesCollection();
                    }
                }
                proxy._isTreeGridRendered = true;
                proxy._isGanttChartRendered = true;
            }
            
            if (args.requestType === "selection") {
                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
            }
            //for validate the predecessor on mouse hover and mouse up of second drag point
            if (args.requestType === "validatePredecessor") {
                var flag= proxy._editedPredecessorValidation(args);
                args.predecessorValidation = flag;
            }

            //on mouse up if validation is true then draw connector line
            if (args.requestType === "drawConnectorLine") {

                var currentPredecessor = args.toItem._calculatePredecessor(args.predecessor);
                args.toItem.item[proxy.model.predecessorMapping] = args.predecessorString[0];
                args.toItem.predecessorsName = args.toItem.item[proxy.model.predecessorMapping];
                if (!args.fromItem.predecessor) {// for parent item
                    args.fromItem['predecessor'] = currentPredecessor;
                }
                else {
                    args.fromItem.predecessor.push(currentPredecessor[0]);
                }
                if (args.toItem.predecessor)
                {
                    args.toItem.predecessor.push(currentPredecessor[0]);
                }
                else
                {
                    args.toItem.predecessor=[];
                    args.toItem.predecessor.push(currentPredecessor[0]);
                }

                proxy._isMileStoneEdited = args.toItem.isMilestone;
                proxy._updatedConnectorLineCollection = [];
                proxy._connectorlineIds = [];
                proxy._validatePredecessor(args.toItem);
                if (proxy._updatedConnectorLineCollection.length > 0) {
                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                }
                /*update schedule dates on predecessor editing */
                var checkArgs = {};
                checkArgs.data = args.toItem;
                proxy._updateScheduleDatesOnEditing(checkArgs);
            }
            
            proxy._ganttActionCompleteTrigger(args);
        },

        //Public method for hidding column using header text
        // argument headerText is header text of the column

        
        hideColumn:function(headerText)
        {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("hideColumn",headerText);
        },

		//Public method for showing column using header text
        // argument headerText is header text of the column

        
        showColumn: function (headerText) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("showColumn", headerText);
        },
		 // public method for accessing tree grid clearColumnMenu method
        clearColumnMenu:function()
        {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("clearColumnMenu");
        },
        // public method for deleting row
        deleteRow: function () {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("deleteRow");
        },
        taskbarEditing: function (args) {
            
            this._trigger("taskbarEditing", args);
        },
        // public method for accessing _setInitialData Private method.
        setInitialData: function () {
            this._setInitialData();
        },

        // Method used for trigger custom context menu event
        customContextMenuEvent: function (choice) {
            var proxy = this,
                eventArgs = {};
            eventArgs.data = proxy._contextMenuSelectedItem;
            proxy._triggerMenuEventHandler(choice.event, eventArgs);
            proxy._clearContextMenu();
        },
        _refreshParent: function (item) {
            this.refreshGanttRecord(item);
            if (item.parentItem) {
                this._refreshParent(item.parentItem);
            }
        },

        taskbarEdited: function (args) {
            var proxy = this,
                ganttRecord = args.data, model = proxy.model,
                projectStartDate = new Date(proxy._projectStartDate),
                remainDays,
                day,
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                headerValue = ej.Gantt.ScheduleHeaderType,
                durationUnitValue = ej.Gantt.DurationUnit,
                startDateMinute,
                offset = 0;
            
            proxy._isValidationEnabled = false;
            
            if (args.dragging) {
                var editMode = "dragging";               
                if (args.previousData.left !== ganttRecord.left) {
                    remainDays = ganttRecord.left % proxy._perDayWidth,
                           day = (ganttRecord.left - remainDays) / proxy._perDayWidth,
                           offset = 0;
                    
                    if (remainDays > 0 && (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {
                        if (headerType == headerValue.Day)
                            offset = remainDays > ((proxy._perDayWidth / 2) + (3 * proxy._perHourWidth)) ? 1 : 0;
                        else
                            offset = remainDays >= proxy._perDayWidth / 2 ? 1 : 0;
                    }
                    //Hour calculation
                    var remainingHours = Math.round(remainDays / (proxy._perDayWidth / 24));
                    //Minute calculation
                    var remainingMinutes = Math.round(remainDays / (proxy._perDayWidth / (24 * 60)));
                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                    if (headerType == headerValue.Day && model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                        if (model.dateFormat.indexOf("hh") != -1) {
                            if (model.durationUnit == durationUnitValue.Minute)
                                projectStartDate.setMinutes(remainingMinutes);
                            else if(model.durationUnit == durationUnitValue.Hour)
                                projectStartDate.setHours(remainingHours);
                        }                      
                    }
                    else if (headerType == headerValue.Hour) {
                        projectStartDate.setMinutes(remainingMinutes);
                        //Update the start date after editing the taskbar in "hour" duration unit. 
                        if (model.durationUnit == durationUnitValue.Hour) {
                            var startDateMinutes = projectStartDate.getMinutes();
                            if (startDateMinutes > 30)
                                projectStartDate.setMinutes(60);
                            else
                                projectStartDate.setMinutes(0);
                        }
                    }

                    else if (headerType == headerValue.Week && model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours &&
                       (model.durationUnit == durationUnitValue.Day ||model.dateFormat.indexOf("hh:") == -1))
                        ganttRecord.left = day * proxy._perDayWidth;

                    else if (model.dateFormat.indexOf("hh:") != -1 &&
                     model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        //Update the duration only in the range between 8 hour to 16 hour.
                        if (remainingHours >= 8 && remainingHours <=16)
                            projectStartDate.setHours(remainingHours);
                        else
                            projectStartDate.setHours(8);                        
                    }
                    else if (model.dateFormat.indexOf("hh:") != -1) {
                        if (model.durationUnit == durationUnitValue.Hour)
                            projectStartDate.setHours(remainingHours);
                        else if (model.durationUnit == durationUnitValue.Minute)
                            projectStartDate.setMinutes(remainingMinutes);
                    }

                    if ((model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {
                        //Calculate the left value of gantt record after dragging the record in eight hour time scale.
                        if (headerType == headerValue.Day && model.durationUnit == durationUnitValue.Hour) {
                            var totalDay = offset + day;
                            var intervalHourWidth = 8;
                            if (remainingHours >= 8 && remainingHours <16)
                                intervalHourWidth = remainingHours;
                            if ((remainingHours < 8 || remainingHours > 15) && (args.previousData.left > ganttRecord.left)) {
                                projectStartDate.setDate(projectStartDate.getDate() - 1);
                                projectStartDate.setHours(15);
                                ganttRecord.left = 15 * proxy._perHourWidth;
                            }                                
                            else if(totalDay > 0)
                                ganttRecord.left = (totalDay * proxy._perDayWidth) + (intervalHourWidth * proxy._perHourWidth);
                            else
                                ganttRecord.left = ganttRecord.left;                                
                            }
                        else if (remainDays >= (proxy._perDayWidth / 2)) {
                             ganttRecord.left = ganttRecord.left + proxy._perDayWidth;//- remainDays;
                                if (model.roundOffDayworkingTime)
                                    ganttRecord.left -= remainDays;
                           }
                        else {
                             ganttRecord.left = ganttRecord.left;// - remainDays;
                             if (model.roundOffDayworkingTime)
                                 ganttRecord.left -= remainDays;
                         }
                    }
                    
                    if (this._stringHolidays &&
                     this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }
                    
                    if (this.model.includeWeekend == false) {
                        if (projectStartDate.getDay() === 0) {
                            projectStartDate.setDate(projectStartDate.getDate() + 1);
                            ganttRecord.left += proxy._perDayWidth;
                        } else if (projectStartDate.getDay() === 6) {
                            projectStartDate.setDate(projectStartDate.getDate() + 2);
                            ganttRecord.left += (2 * proxy._perDayWidth);
                        }
                    }

                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }
                    //To taskbar edit when duration unit of hour minute schedule mode is "hour".
                    if (headerType == headerValue.Hour && model.durationUnit == durationUnitValue.Hour) {
                        var previousDataStartDate = proxy._getDateFromFormat(args.previousData.startDate),
                            ganttStartDate = proxy._getDateFromFormat(ganttRecord.startDate),
                            startDateHour = previousDataStartDate.getHours(),
                            ganttDateHour = ganttStartDate.getHours(),
                            diff = Math.abs(previousDataStartDate - ganttStartDate),
                            hourDifference = Math.round(diff / 3600000),//Milliseconds to hour                                               
                            startDateMinute = ganttStartDate.getMinutes(),
                            remainingMinuteLeft = startDateMinute / proxy._minuteInterval,
                            remainingMinuteRight = (proxy._perDayWidth / (proxy._perMinuteWidth * 24)) - remainingMinuteLeft;

                        if (args.previousData.left > ganttRecord.left) {
                            if (startDateMinute > 30)
                                ganttRecord.left = args.previousData.left - ((hourDifference) * (proxy._perDayWidth / 24));
                            else
                                ganttRecord.left = ganttRecord.left - (proxy._perMinuteWidth * remainingMinuteLeft);
                        }
                        else if (args.previousData.left < ganttRecord.left) {
                            if (startDateMinute < 30)
                                ganttRecord.left = args.previousData.left + ((hourDifference) * (proxy._perDayWidth / 24));
                            else
                                ganttRecord.left = ganttRecord.left + (proxy._perMinuteWidth * remainingMinuteRight);
                        }
                    }                    
                    ganttRecord.startDate = projectStartDate;//_calculateFormatedDate
                    if (model.startDateMapping)
                        ganttRecord.item[model.startDateMapping] = ganttRecord.startDate;
                    ganttRecord._calculateEndDate(model.includeWeekend, this._holidaysList, model, proxy._perDayWidth, editMode);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth,model);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }

            } else if (args.rightResizing) {
                var editMode = "Resizing";
                if (args.previousData.width !== ganttRecord.width) {
                    day = ganttRecord.width / proxy._perDayWidth;
                    var rem = ganttRecord.width % proxy._perDayWidth;                    
                    offset = 0;
                    if (rem > 0 && (model.workingTimeScale != "TimeScale24Hours")) {
                        if (headerType == headerValue.Day)
                            offset = rem > ((proxy._perDayWidth / 2) + (3 * proxy._perHourWidth)) ? 1 : 0;
                        else
                            offset = rem >= (proxy._perDayWidth / 2) ? 1 : 0;
                    }

                    var totalLength = ganttRecord.left + ganttRecord.width;
                    var remainingLength = totalLength % proxy._perDayWidth;

                    var remainingHours = (remainingLength / (proxy._perDayWidth / 24)),//hour calculation
                        remainingMinutes = (remainingLength / (proxy._perDayWidth / (24*60)));
                    var startDate = proxy._getDateFromFormat(ganttRecord.startDate);

                    if (headerType == headerValue.Day) {
                        if (model.dateFormat.indexOf("hh") != -1) {
                            var startDateHour = ganttRecord.left / proxy._perHourWidth,
                                hours = (totalLength / (proxy._perHourWidth));
                            hours -= startDateHour;
                            var minutes = Math.round((hours - Math.floor(hours)) * 60);
                            if (model.durationUnit == durationUnitValue.Hour)
                                var endDate = ganttRecord._dateUpdate(startDate, 0,Math.round(hours));
                            else if (model.durationUnit == durationUnitValue.Minute)
                                var endDate = ganttRecord._dateUpdate(startDate, 0, hours, minutes);
                            else
                                var endDate = ganttRecord._dateUpdate(startDate, day, 0, 0);
                        }
                        else
                            var endDate = ganttRecord._dateUpdate(startDate, day, 0, 0);
                    }
                    else if ((headerType == headerValue.Hour &&
                            (model.durationUnit == durationUnitValue.Minute || model.durationUnit == durationUnitValue.Hour))) {
                       var startDateMinute = (ganttRecord.left % proxy._perDayWidth) / (proxy._perMinuteWidth) * (proxy._minuteInterval),
                           minutes = (totalLength / (proxy._perMinuteWidth)) * (proxy._minuteInterval);

                        minutes -= startDateMinute;
                        minutes = minutes % (24 * 60);                        
                        var endDate = ganttRecord._dateUpdate(startDate, day, 0, minutes);
                    }
                    else if (headerType == headerValue.Week) {
                        var startDateHour = Math.round(ganttRecord.left / (proxy._perDayWidth / 24)),
                               remainingHourDiff = Math.round(totalLength / (proxy._perDayWidth / 24)),
                               remainingMinuteDiff = remainingMinutes % 60;
                            remainingHourDiff -= startDateHour;
                        if (model.dateFormat.indexOf("hh") != -1) {
                            if (model.durationUnit == durationUnitValue.Hour)
                                endDate = ganttRecord._dateUpdate(startDate, 0, remainingHourDiff);
                            else if (model.durationUnit == durationUnitValue.Minute)
                                endDate = ganttRecord._dateUpdate(startDate, 0, remainingHourDiff, remainingMinuteDiff);
                            else
                                endDate = ganttRecord._dateUpdate(startDate, Math.floor(day - 1 + offset), 0);
                        }
                        else if (model.durationUnit == durationUnitValue.Day)
                            endDate = ganttRecord._dateUpdate(startDate, Math.floor(day - 1 + offset), 0);
                        else
                            endDate = ganttRecord._dateUpdate(startDate, Math.floor(day + offset), 0);
                    }

                    else {
                        if (model.durationUnit == durationUnitValue.Hour) {
                            if (model.dateFormat.indexOf("hh") != -1)
                                endDate = ganttRecord._dateUpdate(startDate, day - 1 + offset, remainingHours);
                            if (model.dateFormat.indexOf("hh") == -1)
                                var endDate = ganttRecord._dateUpdate(startDate, Math.floor(day + offset), 0);
                        }
                        else if (model.durationUnit == durationUnitValue.Minute) {
                            if (model.dateFormat.indexOf("hh") != -1)
                                endDate = ganttRecord._dateUpdate(startDate, day + offset, 0, remainingMinutes);
                            if (model.dateFormat.indexOf("hh") == -1)
                                endDate = ganttRecord._dateUpdate(startDate, Math.floor(day + offset), 0);
                        }
                        else
                            var endDate = ganttRecord._dateUpdate(startDate, Math.floor(day - 1 + offset), 0);
                    }

                    if (headerType == headerValue.Day && model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours &&
                         model.durationUnit == durationUnitValue.Hour && model.dateFormat.indexOf("hh") != -1)
                        endDate.setHours(Math.round(remainingHours));
                    if (model.dateFormat.indexOf("hh:") != -1 && headerType == headerValue.Day &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        //Update the end date by one day when end date when taskbar exceeds the 8 hour limit.
                        if (endDate.getHours() > 16)
                            endDate.setHours(Math.round(24 + 9));
                        else if (endDate.getHours() < 8)
                            endDate.setHours(Math.round(9));
                    }

                    if (model.holidays && model.holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = model.holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = proxy._getDateFromFormat(this._holidaysList[holidayIndex]);
                            if (endDate === holiday) {
                                endDate.setDate(endDate.getDate() - 1);
                            }
                        }
                    }

                    if (this.model.includeWeekend == false) {
                        if (endDate.getDay() == 0) {
                            endDate.setDate(endDate.getDate() + 1);
                        } else if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() + 2);
                        }
                    }
                    if ((headerType == headerValue.Week && model.dateFormat.indexOf("hh") != -1 &&
                        model.durationUnit == durationUnitValue.Day))
                        endDate.setHours(24);
                    if (this._stringHolidays &&
                   this._stringHolidays.indexOf(ganttRecord._getFormatedDate(endDate, model.dateFormat)) != -1) {
                        endDate.setDate(endDate.getDate() + 1);
                        if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() + 2);
                        }
                    }
                    
                    ganttRecord.endDate = new Date(endDate);
                    if (model.endDateMapping)
                        ganttRecord.item[model.endDateMapping] = ganttRecord.endDate;
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays, ganttRecord.isMilestone,model, editMode);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth,model);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }

            } else if (args.leftResizing) {
                var editMode = "Resizing";
                if (args.previousData.left !== ganttRecord.left) {
                    var diffLeft = args.previousData.left - ganttRecord.left;
                    remainHours = diffLeft / proxy._perHourWidth;
                    remainDays = ganttRecord.left % proxy._perDayWidth;
                    day = (ganttRecord.left - remainDays) / proxy._perDayWidth;
                    offset = 0;
                    if (remainDays > 0 && (model.workingTimeScale != "TimeScale24Hours")) {
                        if (headerType == headerValue.Day)
                            offset = remainDays > ((proxy._perDayWidth / 2) + (3 * proxy._perHourWidth)) ? 1 : 0;
                        else
                            offset = remainDays >= proxy._perDayWidth / 2 ? 1 : 0;
                    }
                    var remainingHours =(remainDays / (proxy._perDayWidth / 24));//Hour calculation
                    var remainingMinutes = Math.round(remainDays / (proxy._perDayWidth / (24 * 60)));//Minute Calculation
                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);

                    if (headerType == headerValue.Day ||
                        headerType == headerValue.Week){
                        if (model.dateFormat.indexOf("hh") != -1) {
                            if (model.durationUnit == durationUnitValue.Hour)
                                projectStartDate.setHours(Math.round(remainingHours));
                            else if (model.durationUnit == durationUnitValue.Minute) {
                                var minutes = Math.round((remainingHours - Math.floor(remainingHours)) * 60);
                                projectStartDate.setHours(remainingHours);
                                projectStartDate.setMinutes(minutes);
                            }
                            if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours &&
                                (model.durationUnit == durationUnitValue.Hour || model.durationUnit == durationUnitValue.Day)) {
                                if (remainingHours >= 8 && remainingHours <= 15)
                                    projectStartDate.setHours(remainingHours);
                                else {
                                    //Reduce one day when start date moves below 8 hour.
                                    projectStartDate.setDate(projectStartDate.getDate() - 1);
                                    projectStartDate.setHours(15);
                                }
                            }
                        }
                    }
                    else if (headerType == headerValue.Hour) {
                        projectStartDate.setMinutes(remainingMinutes);
                        //To update the start date after editing the taskbar in "hour" duration unit. 
                        if (model.durationUnit == durationUnitValue.Hour) {
                            var startDateMinutes = projectStartDate.getMinutes();
                            if (startDateMinutes > 30)
                                projectStartDate.setMinutes(60);
                            else
                                projectStartDate.setMinutes(0);
                        }
                    }
                    else if (model.dateFormat.indexOf("hh:") != -1 &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainDays / (proxy._perDayWidth / 9));
                        projectStartDate.setHours(8);
                    }
                    else {
                        if (model.durationUnit == durationUnitValue.Hour && model.dateFormat.indexOf("hh") != -1)
                            projectStartDate.setHours(remainingHours);
                        else if (model.durationUnit == durationUnitValue.Minute && model.dateFormat.indexOf("hh") != -1)
                            projectStartDate.setMinutes(remainingMinutes);
                    }
                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }
                    
                    if (this.model.includeWeekend == false) {
                        if (projectStartDate.getDay() === 0) {
                            projectStartDate.setDate(projectStartDate.getDate() + 1);
                            ganttRecord.left += proxy._perDayWidth;
                            day += 1;
                        } else if (projectStartDate.getDay() === 6) {
                            projectStartDate.setDate(projectStartDate.getDate() + 2);
                            ganttRecord.left += (2 * proxy._perDayWidth);
                            day += 2;
                        }
                    }
                    
                    if (this._stringHolidays &&
                    this._stringHolidays.indexOf(ganttRecord._getFormatedDate(projectStartDate, model.dateFormat)) != -1) {
                        projectStartDate.setDate(projectStartDate.getDate() + 1);
                        ganttRecord.left += proxy._perDayWidth;
                    }

                    
                    if (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours &&
                        headerType == headerValue.Day && model.durationUnit != durationUnitValue.Hour) {

                        if (remainDays >= (proxy._perDayWidth / 2)) {
                            ganttRecord.left = ganttRecord.left + proxy._perDayWidth;// - remainDays;
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        } else {
                            ganttRecord.left = ganttRecord.left;//- (remainingHours * 20);
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        }
                    }
                    else if (headerType == headerValue.Week && model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                        if (model.durationUnit == durationUnitValue.Day || model.dateFormat.indexOf("hh") == -1)
                            ganttRecord.left = day * (proxy._perDayWidth);                       
                    }

                    else if (headerType != headerValue.Day &&
                        model.dateFormat.indexOf("hh") == -1) {
                        if (remainDays >= (proxy._perDayWidth / 2)) {
                            ganttRecord.left = ganttRecord.left + proxy._perDayWidth;// - remainDays;
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        } else {
                            ganttRecord.left = ganttRecord.left;//- (remainingHours * 20);
                            if (model.roundOffDayworkingTime)
                                ganttRecord.left -= remainDays;
                        }
                    }
                      


                    else if (headerType == headerValue.Day
                        && (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours ||
                        model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)) {
                        if (model.durationUnit == durationUnitValue.Hour)
                            var remainingHourDiff = ganttRecord.left % proxy._perHourWidth;
                        else if (model.durationUnit == durationUnitValue.Minute)
                            var remainingHourDiff = ganttRecord.left % proxy._perMinuteWidth;
                        var totalDay = day + offset;
                        if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours && (remainingHours < 8 || remainingHours > 15)) {                            
                            ganttRecord.left = ((totalDay-1) * proxy._perDayWidth) + (15 * proxy._perHourWidth);
                        }
                        if (remainingHourDiff >= (proxy._perHourWidth / 2)) {
                            ganttRecord.left = ganttRecord.left + proxy._perHourWidth;// - remainDays;

                            ganttRecord.left -= remainingHourDiff;
                        } else {
                            ganttRecord.left -= remainingHourDiff;
                        }
                    }
                    //To taskbar edit when duration unit of hour minute schedule mode is "hour".
                    if (headerType == headerValue.Hour && model.durationUnit == durationUnitValue.Hour) {
                        var previousDataStartDate = proxy._getDateFromFormat(args.previousData.startDate),
                            ganttStartDate = proxy._getDateFromFormat(ganttRecord.startDate),
                            startDateHour = previousDataStartDate.getHours(),
                            ganttDateHour = ganttStartDate.getHours(),
                            diff = Math.abs(previousDataStartDate - ganttStartDate),
                            hourDifference = Math.round(diff / 3600000),//Milliseconds to hour.
                            startDateMinute = ganttStartDate.getMinutes(),
                            remainingMinuteLeft = startDateMinute / proxy._minuteInterval,
                            remainingMinuteRight = (proxy._perDayWidth / (proxy._perMinuteWidth * 24)) - remainingMinuteLeft;

                        if (args.previousData.left > ganttRecord.left) {
                            if (startDateMinute > 30)
                                ganttRecord.left = args.previousData.left - ((hourDifference) * (proxy._perDayWidth / 24));
                            else
                                ganttRecord.left = ganttRecord.left - (proxy._perMinuteWidth * remainingMinuteLeft);
                        }
                        else if (args.previousData.left < ganttRecord.left) {   
                            if (startDateMinute < 30)
                                ganttRecord.left = args.previousData.left + ((hourDifference) * (proxy._perDayWidth / 24));
                            else
                                ganttRecord.left = ganttRecord.left + (proxy._perMinuteWidth * remainingMinuteRight);
                        }
                    }
                    ganttRecord.startDate = new Date(projectStartDate);
                    if (model.startDateMapping)
                        ganttRecord.item[model.startDateMapping] = ganttRecord.startDate;
                    if (model.includeWeekend == false && (headerType == headerValue.Hour || headerType == headerValue.Day))
                        ganttRecord._calculateEndDate(model.includeWeekend, model.holidays, model, model.perDayWidth, editMode);
                    ganttRecord._calculateDuration(projectStartDate, proxy._getDateFromFormat(ganttRecord.endDate), model.includeWeekend, model.holidays, ganttRecord.isMilestone,model,editMode);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model);
                    ganttRecord.progressWidth = ganttRecord._getProgressWidth(ganttRecord.width, ganttRecord.status);
                    proxy._isValidationEnabled = true;
                }
            } else if (args.progressResizing) {
                ganttRecord.status = ganttRecord._getProgressPercent(ganttRecord.width, ganttRecord.progressWidth);
                if (ganttRecord.parentItem) {
                    ganttRecord._updateParentProgress(ganttRecord.parentItem, model.progressMapping);
                }
                proxy._refreshParent(ganttRecord);
                //proxy.refreshGanttRecord(ganttRecord);
                //proxy.refreshGanttRecord(ganttRecord.parentItem);
            }
            if (proxy._isValidationEnabled) {
                proxy._updateEditedGanttRecord(ganttRecord);
                if (ganttRecord.predecessor) {
                    proxy._$ganttchartHelper.ejGanttChart("updateConnectorLineOffset", ganttRecord);
                    proxy._isMileStoneEdited = ganttRecord.isMilestone;
                    proxy._updatedConnectorLineCollection = [];
                    proxy._connectorlineIds = [];
                    proxy._editedTaskBarItem = ganttRecord;
                    proxy._validatePredecessor(ganttRecord);
                    //proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    //proxy._createConnectorLinesCollection();

                    if (proxy._updatedConnectorLineCollection.length > 0) {
                        proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                    }
                }
                proxy._editedTaskBarItem = null;
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord, editMode);
                proxy.refreshGanttRecord(ganttRecord);
            }
            proxy._trigger("taskbarEdited", args);
            if (!args.cancel && (args.leftResizing || args.rightResizing || args.dragging)) {
                proxy._updateScheduleDatesOnEditing(args);
            }
        },

        actionBegin: function (args) {

            var proxy = this;

            if (!proxy._trigger("actionBegin", args)) {

                if (args.requestType === "beginedit") {
                    proxy._sendEditRequest();
                }   
                
            }

        },

        _ganttActionCompleteTrigger: function (args) {
            this._trigger("actionComplete", args);
            if (!args.cancel && args.requestType === 'save') {
                if (args._cModifiedData) {
                    args.data = args._cModifiedData;
                }
                if (args._cAddedRecord) {
                    args.data = args._cAddedRecord;
                }
                this._updateScheduleDatesOnEditing(args);
            }
        },

        _queryCellInfo: function (args) {
            this._trigger("queryCellInfo", args);
        },

        _beginEdit: function (args) {
            var proxy = this,
                toolbar = $("#" + proxy._id + "_toolbarItems");
            if (!proxy._trigger("beginEdit", args)) {
                proxy._isinBeginEdit = true;
                if (proxy.model.toolbarSettings.showToolbar) {

                    var toolBarEnabledItems = [];

                    var cancelElement = $(toolbar).find(".e-cancel").parent()[0];
                    var saeveElement = $(toolbar).find(".e-saveitem").parent()[0];

                    if (cancelElement) {
                        toolBarEnabledItems.push(cancelElement);
                    }

                    if (saeveElement) {
                        toolBarEnabledItems.push(saeveElement);
                    }

                    $(toolbar).ejToolbar('enableItem', toolBarEnabledItems);
                    var disableToolItems = [];
                    if (proxy.model.editSettings.allowAdding && proxy.model.toolbarSettings.toolbarItems.indexOf("add") !== -1)
                        disableToolItems.push($(toolbar).find(".e-addnewitem").parent()[0]);
                    if (proxy.model.editSettings.allowEditing && proxy.model.toolbarSettings.toolbarItems.indexOf("edit") !== -1)
                        disableToolItems.push($(toolbar).find(".e-edititem").parent()[0]);
                    if (proxy.model.editSettings.allowDeleting && proxy.model.toolbarSettings.toolbarItems.indexOf("delete") !== -1)
                        disableToolItems.push($(toolbar).find(".e-deleteitem").parent()[0]);
                    $(toolbar).ejToolbar('disableItem', disableToolItems);
                }
            }
        },

        _endEdit: function (args) {
            var proxy = this,model=this.model;
            var toolbar = $("#" + proxy._id + "_toolbarItems");
            proxy._isinBeginEdit = false;

            if (proxy._trigger("endEdit", args)) {
                var predecessorString = "";
                args.data[args.columnName] = args.previousValue;
                if (args.columnName === "predecessor") {
                    predecessorString = proxy._predecessorToString(args.data.predecessor, args.data);
                    args.data.item[args.columnObject.mappingName] = predecessorString;
                    args.data.predecessorsName = predecessorString;
                } else {
                    args.data.item[args.columnObject.mappingName] = args.previousValue;
                    if (args.columnName === "resourceInfo") {
                        proxy._updateResourceName(args.data);
                    }
                }
            }
            else {

                if (args.isModified) {
                    proxy.model.currentViewData = proxy._$treegridHelper.ejTreeGrid("getCurrentViewData");
                    proxy._isValidationEnabled = true;
                    proxy._updateEditedGanttRecords(args);
                }

                if (args.columnName === "resourceInfo") {
                    proxy._updateResourceName(args.data);
                }
            }
            proxy.refreshGanttRecord(args.data);
            
            delete args.isModified;
            if (proxy.model.toolbarSettings.showToolbar) {
                var toolBarDisableItems = [];

                if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                    toolBarDisableItems.push($(toolbar).find(".e-cancel").parent()[0]);
                }

                if (proxy.model.toolbarSettings.toolbarItems.indexOf("update") !== -1) {
                    toolBarDisableItems.push($(toolbar).find(".e-saveitem").parent()[0]);
                }
                
                $(toolbar).ejToolbar('disableItem', toolBarDisableItems);
            }

            if (!args.cancel && (args.columnName === "startDate" || args.columnName === "endDate" || args.columnName === "duration")) {
                proxy._updateScheduleDatesOnEditing(args);
            }


            // proxy._trigger("endEdit", args);
        },

        //updated the schedule dates while cellediting ,taskBarediting and dialogEditing and dialogAdding
        _updateScheduleDatesOnEditing:function(args)
        {
            var proxy = this,
                model = this.model,
                startDate = proxy._getDateFromFormat(args.data.startDate),
                endDate = proxy._getDateFromFormat(args.data.endDate),
                scheduleStartDate = proxy._projectStartDate,    //_getDateFromFormat(model.scheduleStartDate),
                scheduleEndDate = proxy._getDateFromFormat(model.scheduleEndDate),
                minStartDate, maxEndDate,
                isDateChanged = false,
                tempStartDate = model.scheduleStartDate,
                updatedDates,
                tempEndDate = model.scheduleEndDate;
                proxy._calculateDatesForScheduleCheck(startDate);
            if (!model.predecessorMapping) {
                if (!(startDate.getTime() >= scheduleStartDate.getTime()) || !(startDate.getTime() <= scheduleEndDate.getTime())) {
                    if (startDate.getTime() < scheduleStartDate.getTime()) {
                        minStartDate = startDate;
                        scheduleStartDate = startDate;
                    }
                    else {
                        maxEndDate = proxy._getDateFromFormat(args.data.startDate);
                        scheduleEndDate = proxy._getDateFromFormat(args.data.startDate);
                    }
                }
                if (!(endDate.getTime() >= scheduleStartDate.getTime()) || !(endDate.getTime() <= scheduleEndDate.getTime())) {
                    if (endDate.getTime() < scheduleStartDate.getTime())
                        minStartDate = endDate;
                    else
                        maxEndDate = endDate;
                }
                if (maxEndDate) {
                    isDateChanged = true;
                    tempEndDate = args.data._getFormatedDate(maxEndDate, model.dateFormat);
                }
                if (minStartDate) {
                    isDateChanged = true;
                    tempStartDate = args.data._getFormatedDate(minStartDate, model.dateFormat);
                }
                if (isDateChanged) {
                    updatedDates = proxy._updateScheduleDatesByTaskLables(proxy._getDateFromFormat(tempStartDate), proxy._getDateFromFormat(tempEndDate));
                    proxy.updateScheduleDates(args.data._getFormatedDate(updatedDates.minStartDate, model.dateFormat), args.data._getFormatedDate(updatedDates.maxEndDate, model.dateFormat));
                }
            }
            else {
                var editArgs = {};
                editArgs.onEditing = true;
                proxy._calculateScheduleDates(editArgs);
                proxy._calculateDatesForScheduleCheck(editArgs.minStartDate);
                if (editArgs.minStartDate.getTime() < scheduleStartDate) {
                    tempStartDate = args.data._getFormatedDate(editArgs.minStartDate, model.dateFormat);
                    isDateChanged = true;
                }
                if(editArgs.maxEndDate.getTime() > scheduleEndDate)
                {
                    tempEndDate = args.data._getFormatedDate(editArgs.maxEndDate, model.dateFormat);
                    isDateChanged = true;
                }
                if (isDateChanged)
                {
                    proxy.updateScheduleDates(tempStartDate, tempEndDate);
                }
            }
        },
        //add dates before edited dates for scheduleupdate 
        _calculateDatesForScheduleCheck:function(startdate)
        {
            var proxy = this, model = this.model,
                scheduleHeaderType = model.scheduleHeaderSettings.scheduleHeaderType,
                scheduleHeaderValue = ej.Gantt.ScheduleHeaderType;
               
            if (scheduleHeaderType == scheduleHeaderValue.Week) {

                startdate.setDate(startdate.getDate() - 3);
            }

            if (scheduleHeaderType == scheduleHeaderValue.Year) {
                startdate.setDate(startdate.getDate() - 5);
            }
            if (scheduleHeaderType == scheduleHeaderValue.Month) {
                startdate.setDate(startdate.getDate() - 3);
            }

            if (scheduleHeaderType == scheduleHeaderValue.Day) {
                startdate.setHours(startdate.getHours() - 2);
            }
            if (scheduleHeaderType == scheduleHeaderValue.Hour) {
                startdate.setMinutes(startdate.getMinutes() - 5);
            }
        }
        ,
        //update schedule dates by tool bar operation
        _updateScheduleDatesByToolBar:function(args)
        {
            var proxy = this, model = this.model,
                 startDate = args === "prevTimeSpan" ? proxy._projectStartDate : proxy._getDateFromFormat(proxy.model.scheduleStartDate);
                 endDate = proxy._getDateFromFormat(proxy.model.scheduleEndDate),
                 chartObject = $("#ejGanttChart" + proxy._id).ejGanttChart("instance"),
                 scheduleHeaderType = model.scheduleHeaderSettings.scheduleHeaderType,
                 scheduleHeaderValue = ej.Gantt.ScheduleHeaderType,
                 maxScrollWidth = 0;

            if (scheduleHeaderType == scheduleHeaderValue.Week) {
                if (args === "prevTimeSpan") {
                    startDate.setDate(startDate.getDate() - 6);
                }
                else {
                    endDate.setDate(endDate.getDate() + 7);
                }
            }

            if (scheduleHeaderType == scheduleHeaderValue.Year) {
                if (args === "prevTimeSpan")
                    startDate.setDate(startDate.getDate() - 360);
                else
                    endDate.setDate(endDate.getDate() + 365);
            }
            if (scheduleHeaderType == scheduleHeaderValue.Month) {
                
                if (args === "prevTimeSpan")
                    startDate.setDate(startDate.getDate());
                else
                    endDate.setDate(endDate.getDate() + 30);
            }
            if (scheduleHeaderType == scheduleHeaderValue.Day) {
                if (args === "prevTimeSpan")
                    startDate.setDate(startDate.getDate() - 1);
                else
                    endDate.setDate(endDate.getDate() + 1);
            }
            if (scheduleHeaderType == scheduleHeaderValue.Hour) {
                if (args === "prevTimeSpan")
                    startDate.setHours(startDate.getHours() - 1);
                else
                    endDate.setHours(endDate.getHours() + 1);
            }                                    
            if (args === "prevTimeSpan") {
                proxy.updateScheduleDates(proxy._getFormatedDate(startDate, model.dateFormat), proxy._getFormatedDate(endDate, model.dateFormat));
                chartObject._$bodyContainer.ejScroller("scrollX", 0, true);
            }
            else if (args === "nextTimeSpan") {
                proxy.updateScheduleDates(proxy._getFormatedDate(startDate, model.dateFormat), proxy._getFormatedDate(endDate, model.dateFormat));
                maxScrollWidth = chartObject.getMaxScrollWidth();
                chartObject._$bodyContainer.ejScroller("scrollX", maxScrollWidth, true);
            }
        },
        _rowDataBound: function (args) {
            this._trigger("rowDataBound", args);
        },
        _refreshToolBar: function () {
            var $toolbar = $("#" + this._id + "_toolbarItems");
            $toolbar.find("li").removeClass("e-hover");
        },

        _toolBarClick: function (sender) {
            var proxy = this,
                model=proxy.model,
                $gridEle = $(this.itemsContainer).closest(".e-gantt"),
                gridInstance = $gridEle.ejGantt("instance"),
                gridId = $gridEle.attr('id'),
                toolbarObj = $("#" + gridInstance._id + "_toolbarItems");

            gridInstance._clearContextMenu();
            if (sender.event == undefined && sender.target.tagName == "INPUT" && sender.currentTarget.id == gridId + "_search")
                return false;

            if (gridInstance.model.dateFormat.indexOf("hh") != -1)
                $.isFunction($.fn.ejDateTimePicker) && $("#" + gridId + "EditForm").find(".e-datetimepicker").ejDateTimePicker("hide");
            else
                $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");


            if ($gridEle.ejGantt("instance")._trigger("toolBarClick", sender))
                return false;
            $(toolbarObj).ejToolbar('deselectItem', sender.currentTarget);
            switch (sender.currentTarget.id) {

                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add");
                    break;

                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit");
                    break;

                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete");
                    break;

                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update");
                    break;

                case gridId + "_cancel":
                    gridInstance._toolbarOperation(gridId + "_cancel");
                    break;

                case gridId + "_search":
                    gridInstance._toolbarOperation(gridId + "_search", $(sender.currentTarget).find("input").val());
                    break;

                case gridId + "_indent":
                    gridInstance._toolbarOperation(gridId + "_indent");
                    break;

                case gridId + "_outdent":
                    gridInstance._toolbarOperation(gridId + "_outdent");
                    break;
                    
                case gridId + "_expandAll":
                    gridInstance._toolbarOperation(gridId + "_expandAll");
                    break;
                 
                case gridId + "_collapseAll":
                    gridInstance._toolbarOperation(gridId + "_collapseAll");
                    break;
                case gridId + "_prevTimeSpan":
                    gridInstance._toolbarOperation(gridId + "_prevTimeSpan");
                    break;
                case gridId + "_nextTimeSpan":
                    gridInstance._toolbarOperation(gridId + "_nextTimeSpan");
                    break;
            }
            return false;
        },

        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejGantt("instance"), gridId = $gridEle.attr('id'), proxy = this,
                rowindex = proxy.selectedRowIndex();
            var toolbar = $("#" + proxy._id + "_toolbarItems");
            switch (operation) {

                case gridId + "_add":
                    proxy._sendAddRequest();
                    break;

                case gridId + "_edit":
                    proxy._sendEditRequest();
                    break;

                case gridId + "_delete":
                    proxy._$treegridHelper.ejTreeGrid("deleteRow");
                    proxy._isRefreshAddedRecord = false;

                    break;

                case gridId + "_update":
                    proxy._$treegridHelper.ejTreeGrid("endEdit");
                    //gridObject.sendSaveRequest();

                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                        proxy._disabledToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
                    }

                    if (proxy.model.toolbarSettings.toolbarItems.indexOf("update") !== -1) {
                        proxy._disabledToolItems.push($(toolbar).find(".e-saveitem").parent()[0]);
                    }
                    $(toolbar).ejToolbar('disableItem', proxy._disabledToolItems);
                    
                    proxy._disabledToolItems = [];
                    
                    break;

                case gridId + "_cancel":
                    proxy._$treegridHelper.ejTreeGrid("cancelEditCell");                                                            
                    break;

                case gridId + "_search":
                    proxy._$treegridHelper.ejTreeGrid("search", searchEle);
                    proxy._searchString = searchEle;
                    proxy.updateSelectedItemIndex();
                    proxy._updateToolbarOptions();
                    proxy._isRefreshAddedRecord = false;
                    break;

                case gridId + "_indent":
                    proxy._sendIndentRequest();
                    proxy._$treegrid.focus();
                    proxy._isRefreshAddedRecord = false;
                    break;

                case gridId + "_outdent":
                    proxy._sendOutdentRequest();
                    proxy._$treegrid.focus();
                    proxy._isRefreshAddedRecord = false;
                    break;

                case gridId + "_expandAll":
                    proxy._expandAll();
                    proxy.updateSelectedItemIndex();
                    proxy._$treegrid.focus();
                    break;

                case gridId + "_collapseAll":                   
                    proxy._collapseAll();
                    proxy.updateSelectedItemIndex();
                    proxy._$treegrid.focus();
                    break;
                case gridId + "_prevTimeSpan":
                    proxy._updateScheduleDatesByToolBar("prevTimeSpan");
                    break;
                case gridId + "_nextTimeSpan":
                    proxy._updateScheduleDatesByToolBar("nextTimeSpan");
                    break;

            }
            proxy._refreshToolBar();
            return false;
        },

        /* update selected ietm index when selected item row position changed */
        updateSelectedItemIndex: function () {
            var proxy = this, model = this.model, updatedRecords = model.updatedRecords,
                index, expandRecord;
            if (model.enableVirtualization)
                expandRecord = updatedRecords;
            else
                expandRecord = proxy._$treegridHelper.ejTreeGrid("getExpandedRecords", updatedRecords);
            
            if(!ej.isNullOrUndefined(this.selectedItem()) &&
                this.selectedRowIndex() != expandRecord.indexOf(this.selectedItem()))
            {
                if (proxy.getExpandStatus(this.selectedItem())) {
                    index = updatedRecords.indexOf(this.selectedItem());
                    proxy.selectRows(index);
                }
                else
                {
                    proxy.selectRows(-1);
                }
            }
        },
        //select rows index of this item
        selectRows:function(index)
        {
            var proxy = this, model = this.model;

            if (index !== -1 && !ej.isNullOrUndefined(index) && index < model.updatedRecords.length) {
                var args = {};
                args.recordIndex = index;
                args.data = model.updatedRecords[args.recordIndex];
                args.target = "ejTreeGrid";
                proxy.rowSelected(args);
                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
            }
            else if (index === -1) {
                proxy._deSelectRowItem();
                proxy._updateToolbarOptions();
            }
        },

        _setModel: function (options) {
            
            var proxy = this, option,
                model = proxy.model;
            
            for (option in options) {
                switch (option) {
                    case "sortSettings":
                        proxy._sendSortingRequest(options[option]);
                        break;
                    case "allowSorting":
                        proxy._sortingRequest(options[option]);
                        break;
                    case "destroy":
                        proxy.destroy();
                        break;
                    case "treeColumnIndex":
                        proxy._columnIndex(options[option]);
                        break;
                    case "allowGanttChartEditing":
                        proxy._ganttChartEditing(options[option]);
                        break;
                    case "enableProgressBarResizing":
                        proxy._resizeProgressbar(options[option]);
                        break;
                    case "enableTaskbarTooltip":
                        proxy._showTooltip(options[option]);
                        break;
                    case "enableTaskbarDragTooltip":
                        proxy._showEditingTooltip(options[option]);
                        break;
                    case "enableAltRow":
                        proxy._updateAltRow(options[option]);
                        break;
                    case "locale":
                        model.flatRecords = [];
                        model.parentRecords = [];
                        proxy._clearContextMenu();
                        model.locale = options[option];
                        proxy.element.ejGantt("destroy").ejGantt(model);
                        break;
                    case "allowSelection":
                        model.allowSelection = options[option];
                        if (model.allowSelection) {
                            proxy._$treegridHelper.ejTreeGrid("instance").model.allowSelection = true;
                            proxy._$ganttchartHelper.ejGanttChart("instance").model.allowSelection = true;
                        }
                        else {
                            proxy._deSelectRowItem();
                            proxy._$treegridHelper.ejTreeGrid("instance").model.allowSelection = false;
                            proxy._$ganttchartHelper.ejGanttChart("instance").model.allowSelection = false;
                        }
                        proxy._updateToolbarOptions();
                        break;
                    case "editSettings":
                        $.extend(model.editSettings, options[option]);
                        proxy._updateEditSettings(options[option]);
                        break;
                    case "taskbarBackground":
                        proxy._updateTaskbarBackground(options[option]);
                        break;
                    case "progressbarBackground":
                        proxy._updateProgressbarBackground(options[option]);
                        break;
                    case "parentTaskbarBackground":
                        proxy._updateParentTaskbarBackground(options[option]);
                        break;
                    case "parentProgressbarBackground":
                        proxy._updateParentProgressbarBackground(options[option]);
                        break;
                    case "showTaskNames":
                        proxy._updateRenderTaskNames(options[option]);
                        break;
                    case "showProgressStatus":
                        proxy._updateRendeProgressStatus(options[option]);
                        break;
                    case "showResourceNames":
                        proxy._updateRenderResourceNames(options[option]);
                        break;
                    case "progressbarHeight":
                        proxy._updateProgressbarHeight(options[option]);
                        break;
                    case "baselineColor":
                        proxy._updateBaselineColor(options[option]);
                        break;
                    case "renderBaseline":
                        proxy._updateRenderBaseline(options[option]);
                        break;
                    case "allowColumnResize":
                        proxy._updateAllowColumnResize(options[option]);
                        break;
                    case "allowKeyboardNavigation":
                        proxy._updateAllowKeyboardNavigation(options[option]);
                        break;
                    case "selectedItem":
                        proxy._updateSelectedItem(options[option]);
                        break;
                    case "dataSource":
                        this._refreshDataSource(options[option]);
                         break;
                    case "showGridCellTooltip":
                        proxy._updateShowGridCellTooltip(options[option]);
                        break;
                    case "showGridExpandCellTooltip":
                        proxy._updateShowGridExpandCellTooltip(options[option]);
                        break;
                    case "highlightWeekends":
                        proxy._updateHighlightWeekends(options[option]);
                        break;
                    case "connectorLineBackground":
                        proxy._updateConnectorLineBackground(options[option]);
                        break;
                    case "connectorlineWidth":
                        proxy._updateConnectorlineWidth(options[option]);
                        break;
                    case "weekendBackground":
                        proxy._updateWeekendBackground(options[option]);
                        break;
                    case "enableContextMenu":
                        model.enableContextMenu = options[option];
                        proxy._clearContextMenu();
                        proxy._off(proxy.element, "contextmenu", proxy._rightClick);
                        if (model.enableContextMenu) {
                            proxy._on(proxy.element, "contextmenu", proxy._rightClick);
                        }
                        break;
                    case "enableResize":
                        if (model.enableResize) {
                            proxy.model.enableResize = options[option];
                            proxy._on($(window), "resize", proxy.windowResize);
                        }
                        else {
                            proxy._off($(window), "resize", proxy.windowResize);
                        }
                        break;
                    case "selectedRowIndex":
                        if (this.selectedRowIndex() === -1 || this.selectedRowIndex() === "") {
                            proxy._deSelectRowItem();
                            proxy._updateToolbarOptions();
                        }
                        else if (this.selectedRowIndex() !== -1 && !ej.isNullOrUndefined(this.selectedRowIndex()) && this.selectedRowIndex() < proxy.model.updatedRecords.length) {
                            var rowSelectingArgs = {};
                            rowSelectingArgs.recordIndex = proxy.selectedRowIndex();
                            rowSelectingArgs.previousIndex = model.selectedItem ? model.updatedRecords.indexOf(model.selectedItem) : -1;
                            if (proxy.rowSelecting(rowSelectingArgs)) {
                                var args = {};
                                args.recordIndex = this.selectedRowIndex();
                                args.data = proxy.model.updatedRecords[args.recordIndex];
                                args.target = "ejTreeGrid";
                                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
                                proxy.rowSelected(args);
                            }
                        }
                        break;
                    case "showColumnChooser":
                        model.showColumnChooser = options[option];
                        proxy._$treegridHelper.ejTreeGrid("setModel", {"showColumnChooser":model.showColumnChooser});
						break;
                    case "splitterPosition":
                        proxy.setSplitterPosition(proxy.splitterPosition());
                        break;
                    case "addDialogFields":
                        if (model.editSettings.allowAdding) {
                            model.addDialogFields = options[option];
                            proxy.addDialogTemplate();
                        }
                        break;

                    case "editDialogFields":
                        if (model.editSettings.allowEditing) {
                            model.editDialogFields = options[option];
                            proxy.editDialogTemplate();
                        }
                        break;
                    case "toolbarSettings":
                        $.extend(proxy.model.toolbarSettings, options[option]);
                        proxy._updateToolbar();
                        break;
                    case "holidays":
                        model.holidays = options[option];
                        proxy._refreshHolidays();
                        break;
                    case "stripLines":
                        model.stripLines = options[option];
                        proxy._updateStripLines();
                        break;
                }
            }
        },

        _refreshDataSource: function (dataSource) {
            var proxy = this, model = this.model, args = {};
            this.resetModelCollections();
            this.dataSource(dataSource);
            this._isFromSetModel = true;
            this._checkDataBinding();
            this._updatePredecessors();
            this._$treegridHelper.ejTreeGrid("setUpdatedRecords", model.flatRecords, model.updatedRecords, model.ids, model.parentRecords, this.dataSource());
            this._$treegridHelper.ejTreeGrid("processBindings");
            proxy.model.updatedRecords = proxy.getUpdatedRecords();
            proxy.model.currentViewData = proxy.getCurrentViewData();
            proxy._gridRows = proxy.getRows();
            proxy._totalCollapseRecordCount = proxy._$treegridHelper.ejTreeGrid("getCollapsedRecordCount");
            this._$ganttchartHelper.ejGanttChart("setUpdatedRecords", model.currentViewData, model.updatedRecords, model.flatRecords, model.ids);
            this._calculateScheduleDates();
            this.updateScheduleDates(model.scheduleStartDate, this.model.scheduleEndDate);
            args.requestType = ej.TreeGrid.Actions.Refresh;
            proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", args);
            this._isFromSetModel = false;
        },

        resetModelCollections: function () {
            var proxy = this,
                model = proxy.model;
            model.dataSource = [];
            model.flatRecords = [];
            model.parentRecords = [];
            model.updatedRecords = [];
            model.ids = [];
            model.scheduleStartDate = model.scheduleStartDate;
            model.scheduleEndDate = model.scheduleEndDate;
            proxy._storedIndex = -1;
            proxy._predecessorsCollection = [];
        },
        /* refresh holidays collections */
        _refreshHolidays: function () {
            var proxy = this,
                model = this.model, updatedDates;
            proxy._holidaysList = proxy._getHoliday();
            proxy._stringHolidays = proxy._getStringHolidays();

            updatedDates = proxy._updateScheduleDatesByHolidays(new Date(proxy._projectStartDate), new Date(proxy._projectEndDate));
            if (updatedDates.startDate.getTime() !== proxy._projectStartDate.getTime() || updatedDates.endDate.getTime() !== proxy._projectEndDate.getTime()) {
                var ganttObject = proxy._$ganttchartHelper.data("ejGanttChart");
                ganttObject.model.holidays = model.holidays;
                proxy.updateScheduleDates(updatedDates.startDate, updatedDates.endDate);
            }
            else {
                /* prevent refresh parent Item on update the child records */
                proxy._isTreeGridRendered = false;
                proxy._isGanttChartRendered = false;

                /* Update Left and Width Value of all records with new holidays*/
                proxy._updateGanttRecords();

                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedRecords, proxy._totalCollapseRecordCount);
                /* refresh the connector line collection */
                if (model.predecessorMapping) {
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._isValidationEnabled = false;
                    proxy._createConnectorLinesCollection();
                    proxy._isValidationEnabled = true;
                }
                /*refresh the holiday container */
                this._$ganttchartHelper.ejGanttChart("refreshHolidays", model.holidays);
                proxy._isTreeGridRendered = true;
                proxy._isGanttChartRendered = true;
            }

            /* refresh chart and Treegrid part */
            proxy._$treegridHelper.ejTreeGrid("renderRecords");
        },

        /* refreh the strip lines collection */
        _updateStripLines : function()
        {
            var proxy = this, model = this.model, updatedDates;

            updatedDates = proxy._updateScheduleDatesByStripLines(new Date(proxy._projectStartDate), new Date(proxy._projectEndDate));

            if (updatedDates.startDate.getTime() !== proxy._projectStartDate.getTime() || updatedDates.endDate.getTime() !== proxy._projectEndDate.getTime()) {
                var ganttObject = proxy._$ganttchartHelper.data("ejGanttChart");
                ganttObject.model.stripLines = model.stripLines;
                proxy.updateScheduleDates(updatedDates.startDate, updatedDates.endDate);
            }
            else {
                this._$ganttchartHelper.ejGanttChart("refreshStripLines", this.model.stripLines);
            }
        },
        _updateWeekendBackground: function (bgcolor) {
            var proxy = this;
            this.model.weekendBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateWeekendBackground", bgcolor);

        },

        _updateConnectorlineWidth: function (lineWidth) {
            var proxy = this;
            this.model.connectorlineWidth = lineWidth;
            proxy._$ganttchartHelper.ejGanttChart("updateConnectorlineWidth", lineWidth);
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines",
                proxy._connectorLinesCollection);

        },

        _updateConnectorLineBackground: function (bgcolor) {
            var proxy = this;
            this.model.connectorLineBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateConnectorLineBackground", bgcolor);
            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
            proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines",
                proxy._connectorLinesCollection);

        },

        _updateHighlightWeekends: function (bool) {
            var proxy = this;
            this.model.highlightWeekends = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateHighlightWeekends", bool);
        },

        _updateShowGridCellTooltip: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateShowGridCellTooltip", bool);
        },

        _updateShowGridExpandCellTooltip: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateShowGridExpandCellTooltip", bool);
        },

        _updateSelectedItem: function (item) {
            var proxy = this,
                args = {};
            if (item[proxy.model.taskIdMapping] != null) {


                args.recordIndex =proxy.model.ids.indexOf((item[proxy.model.taskIdMapping]).toString());
                args.item = proxy.model.flatRecords[args.recordIndex];
                args.target = "ejTreeGrid";
                proxy.rowSelected(args);
                proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
                proxy._$ganttchartHelper.ejGanttChart("updateSelectedItem", args.recordIndex);
            }
        },

        /* udpate allowKeyboardNavigation API value in treegrid and chart side */
        _updateAllowKeyboardNavigation: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateAllowKeyboardNavigation", bool);
            proxy._$ganttchartHelper.ejGanttChart("updateAllowKeyboardNavigation", bool);
        },
        
        _updateAllowColumnResize: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateAllowColumnResize", bool);
        },
        
        _updateProgressbarHeight: function (height) {
            var proxy = this;
            this.model.progressbarHeight = height;
            proxy._$ganttchartHelper.ejGanttChart("updateProgressbarHeight", height);
        },
        
        _updateBaselineColor: function (color) {
            var proxy = this;
            this.model.baselineColor = color;
            proxy._$ganttchartHelper.ejGanttChart("updateBaselineColor", color);
        },
        
        _updateRenderBaseline: function (bool) {
            var proxy = this;
            this.model.renderBaseline = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderBaseline", bool);
            proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines",
                proxy._connectorLinesCollection);
        },
        
        _updateRenderTaskNames: function (bool) {
            var proxy = this;
            this.model.showTaskNames = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderTaskNames", bool);
        },
        
        _updateRendeProgressStatus: function (bool) {
            var proxy = this;
            this.model.showProgressStatus = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRendeProgressStatus", bool);
        },
        
        _updateRenderResourceNames: function (bool) {
            var proxy = this;
            this.model.showResourceNames = bool;
            proxy._$ganttchartHelper.ejGanttChart("updateRenderResourceNames", bool);
        },
        
        _updateTaskbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.taskbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateTaskbarBackground", bgcolor);
        },

        _updateProgressbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.progressbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateProgressbarBackground", bgcolor);
        },

        _updateParentTaskbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.parentTaskbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateParentTaskbarBackground", bgcolor);
        },

        _updateParentProgressbarBackground: function (bgcolor) {
            var proxy = this;
            this.model.parentProgressbarBackground = bgcolor;
            proxy._$ganttchartHelper.ejGanttChart("updateParentProgressbarBackground", bgcolor);
        },
        
        _updateEditSettings: function (edit) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateEditSettings", edit);
            
        },
        
        _updateRowSelection: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateRowSelection", bool);
        },

        _updateAltRow: function (bool) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("updateAltRow", bool);
        },

        //Insert templated row in DOM for newly added record
        _renderAddedRow: function (index, data)
        {
            var proxy = this, model = this.model;
            model.currentViewData = proxy._$treegridHelper.ejTreeGrid("getUpdatedCurrentViewData");
            proxy._$ganttchartHelper.ejGanttChart("setUpdatedRecords", model.currentViewData, model.updatedRecords, model.flatRecords, model.ids);
            if (model.enableVirtualization) {
                var tempArgs = {};
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", tempArgs);
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", model.currentViewData, model.updatedRecords, proxy._totalCollapseRecordCount);
            }
            else {
                proxy._$treegridHelper.ejTreeGrid("renderNewAddedRow", index, data);
                proxy._$ganttchartHelper.ejGanttChart("renderNewAddedRow", index, data);
            }
            proxy._gridRows = proxy.getRows();
            proxy._ganttChartRows = proxy.getGanttChartRows();
        },
        
        /* get visible child record count in udpated records */
        _getVisibleChildRecordCount:function(data,count,collection)
        {
            var proxy = this, model = this.model, childRecords, length;;

            if (data.hasChildRecords) {
                childRecords = data.childRecords;
                length = childRecords.length;
                for (var i = 0; i < length; i++) {
                    if (collection.indexOf(childRecords[i]) !== -1)
                        count++;
                    if (childRecords[i].hasChildRecords) {
                        count = proxy._getVisibleChildRecordCount(childRecords[i], count, collection);
                    }
                }
            }
            else {
                if (collection.indexOf(data) !== -1) {
                    count++;
                }
            }
            return count;
        },

        /* validate predecessor value of adding record*/
        _validateAddRecordPrecessorValue :function(args){
            var proxy = this, data = args.data, model = this.model;
            /* Validate the predecessor value*/
            if (model.predecessorMapping && data[model.predecessorMapping] && data[model.predecessorMapping].length > 0) {
                var validateArgs = {};
                validateArgs.currentRecord = args.currentRecord;
                validateArgs.predecessorString = data[model.predecessorMapping].split(",");
                if (!proxy._editedPredecessorValidation(validateArgs)) {
                    data[model.predecessorMapping] = "";
                    args.currentRecord.predecessor = null;
                    args.currentRecord.predecessorsName = "";
                    return false;
                }
                else {
                    return true;
                }
            } else {
                return true;
            }
        },
        
        openAddDialog: function() {
            this._sendAddRequest();
        },

        
        openEditDialog:function() {
            this._sendEditRequest();
        },
        
        
        searchItem: function (searchString) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("search", searchString);
            proxy._searchString = searchString;
            $("#gantt_search input").val(searchString);
        },
        
        
        addRecord: function (data, rowPosition) {
            var proxy = this, model = this.model,
                selectedRowIndex = this.selectedRowIndex(),
                selectedItem = this.selectedItem(), _cAddedRecord,
                taskIdMapping = model.taskIdMapping,
                updatedRecords = model.updatedRecords,
                flatRecords = model.flatRecords,
                parentRecords = model.parentRecords,
                ids = model.ids,
                dataSource =this.dataSource(),
                level = 0, insertIndex, validateArgs = {},
                parentItem, nextBelowItemInflatRecords, nextBelowItemInUpdatedRecords;


            if (ej.isNullOrUndefined(dataSource) || ej.isNullOrUndefined(dataSource[0]))
            {
                this.dataSource([]);
                dataSource = this.dataSource();
            }
            if (Object.prototype.toString.call(data) !== "[object Object]") {
                data = {};
            }

            if (selectedRowIndex === -1 && (rowPosition === ej.Gantt.AddRowPosition.AboveSelectedRow
                 || rowPosition === ej.Gantt.AddRowPosition.BelowSelectedRow
                 || rowPosition === ej.Gantt.AddRowPosition.Child) || !rowPosition) {
                rowPosition = ej.Gantt.AddRowPosition.Top;
            }

            /*Validate Task Id of data*/
            if (data[taskIdMapping]) {
                if(model.ids.indexOf(data[taskIdMapping].toString()) != -1) {
                    data[taskIdMapping] = null;
                } else {
                    data[taskIdMapping] = isNaN(parseInt(data[taskIdMapping])) ? null : parseInt(data[taskIdMapping]);
                }
            }

            //Add default value for missing fields
            proxy._updateAddData(data);
            //get updated json data of data manager from TreeGrid 
            if (proxy._isDataManagerUpdate)
                proxy._jsonData = proxy._$treegridHelper.ejTreeGrid("getUpdatedDataManagerData");

            switch (rowPosition) {
                case ej.Gantt.AddRowPosition.Top:
                    level = 0;
                    parentItem = null;
                    _cAddedRecord = proxy._createGanttRecord(data, level);
                    _cAddedRecord.index = _cAddedRecord.taskId;

                    //validate predecessor of new record
                    validateArgs.data = data;
                    validateArgs.currentRecord = _cAddedRecord;
                    if (!proxy._validateAddRecordPrecessorValue(validateArgs) && proxy._isinAddnewRecord) {
                        return false;
                    }

                    /*record update*/
                    flatRecords.splice(0, 0, _cAddedRecord);
                    updatedRecords.splice(0, 0, _cAddedRecord);
                    ids.splice(0, 0, _cAddedRecord.taskId.toString());
                    parentRecords.splice(0, 0, _cAddedRecord);
                    /*data source update*/
                    if (proxy._isDataManagerUpdate) {
                        proxy._jsonData.splice(0, 0, _cAddedRecord.item);
                    } else {
                        dataSource.splice(0, 0, _cAddedRecord.item);
                    }
                    insertIndex = 0;
                    break;

                case ej.Gantt.AddRowPosition.Bottom:
                    level = 0;
                    parentItem = null;
                    _cAddedRecord = proxy._createGanttRecord(data, level);
                    _cAddedRecord.index = _cAddedRecord.taskId;

                    //validate predecessor of new record
                    validateArgs.data = data;
                    validateArgs.currentRecord = _cAddedRecord;
                    if (!proxy._validateAddRecordPrecessorValue(validateArgs) && proxy._isinAddnewRecord) {
                        return false;
                    }

                    /*record update*/
                    flatRecords.push(_cAddedRecord);
                    updatedRecords.push(_cAddedRecord);
                    ids.push(_cAddedRecord.taskId.toString());
                    parentRecords.push(_cAddedRecord);
                    /*data source update*/
                    if (proxy._isDataManagerUpdate) {
                        proxy._jsonData.push(_cAddedRecord.item);
                    } else {
                        dataSource.push(_cAddedRecord.item);
                    }
                    insertIndex = updatedRecords.indexOf(_cAddedRecord);
                    break;

                case ej.Gantt.AddRowPosition.AboveSelectedRow:
                    level = selectedItem.level;
                    parentItem = selectedItem.parentItem;
                    _cAddedRecord = proxy._createGanttRecord(data, level, parentItem);
                    _cAddedRecord.index = _cAddedRecord.taskId;

                    //validate predecessor of new record
                    validateArgs.data = data;
                    validateArgs.currentRecord = _cAddedRecord;
                    if (!proxy._validateAddRecordPrecessorValue(validateArgs) && proxy._isinAddnewRecord) {
                        return false;
                    }
                    var childIndex, recordIndex, updatedCollectionIndex;
                    /*Record Updates*/
                    recordIndex = flatRecords.indexOf(selectedItem);
                    updatedCollectionIndex = updatedRecords.indexOf(selectedItem);
                    flatRecords.splice(recordIndex, 0, _cAddedRecord);
                    updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                    ids.splice(recordIndex, 0, _cAddedRecord.taskId.toString());

                    if (!ej.isNullOrUndefined(parentItem)) {
                        childIndex = parentItem.childRecords.indexOf(selectedItem);
                        /*Child collection update*/
                        parentItem.childRecords.splice(childIndex, 0, _cAddedRecord);
                        if (!model.parentTaskIdMapping) {
                            parentItem.item[model.childMapping].splice(childIndex, 0, _cAddedRecord.item);
                        }
                        else {
                            _cAddedRecord.item[model.parentTaskIdMapping] = _cAddedRecord.parentItem.taskId;
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.push(_cAddedRecord.item);

                            } else {
                                dataSource.push(_cAddedRecord.item);
                            }
                        }
                    } else {
                        /* Parent records collection and data source update*/
                        parentRecords.splice(parentRecords.indexOf(selectedItem), 0, _cAddedRecord);
                        if (proxy._isDataManagerUpdate) {
                            proxy._jsonData.splice(proxy._jsonData.indexOf(selectedItem.item), 0, _cAddedRecord.item);
                        }
                        else {
                            dataSource.splice(dataSource.indexOf(selectedItem.item), 0, _cAddedRecord.item);
                        }
                    }
                    insertIndex = updatedCollectionIndex;
                    break;
                case ej.Gantt.AddRowPosition.BelowSelectedRow:
                    level = selectedItem.level;
                    parentItem = selectedItem.parentItem;
                    _cAddedRecord = proxy._createGanttRecord(data, level, parentItem);
                    _cAddedRecord.index = _cAddedRecord.taskId;

                    //validate predecessor of new record
                    validateArgs.data = data;
                    validateArgs.currentRecord = _cAddedRecord;
                    if (!proxy._validateAddRecordPrecessorValue(validateArgs) && proxy._isinAddnewRecord) {
                        return false;
                    }

                    //find next item position
                    var currentItemIndex = flatRecords.indexOf(selectedItem)

                    if (selectedItem.hasChildRecords) {
                        dataChildCount = proxy._$treegridHelper.ejTreeGrid("getChildCount", selectedItem, 0);
                        recordIndex = currentItemIndex + dataChildCount + 1;
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + proxy._getVisibleChildRecordCount(selectedItem, 0, updatedRecords) + 1;

                    } else {
                        recordIndex = currentItemIndex + 1;
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + 1;
                    }

                    /* Record collection update */
                    flatRecords.splice(recordIndex, 0, _cAddedRecord);
                    updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                    ids.splice(recordIndex, 0, _cAddedRecord.taskId.toString());

                    /* data Source update */
                    if (!ej.isNullOrUndefined(parentItem)) {
                        childIndex = parentItem.childRecords.indexOf(selectedItem);
                        /*Child collection update*/
                        parentItem.childRecords.splice(childIndex + 1, 0, _cAddedRecord);
                        if (!model.parentTaskIdMapping) {
                            parentItem.item[model.childMapping].splice(childIndex + 1, 0, _cAddedRecord.item);
                        }
                        else {
                            _cAddedRecord.item[model.parentTaskIdMapping] = _cAddedRecord.parentItem.taskId;
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.push(_cAddedRecord.item);

                            } else {
                                dataSource.push(_cAddedRecord.item);
                            }
                        }
                    } else {
                        /* Parent records collection and data source update*/
                        parentRecords.splice(parentRecords.indexOf(selectedItem) + 1, 0, _cAddedRecord);
                        if (proxy._isDataManagerUpdate) {
                            proxy._jsonData.splice(proxy._jsonData.indexOf(selectedItem.item) + 1, 0, _cAddedRecord.item);
                        }
                        else {
                            dataSource.splice(dataSource.indexOf(selectedItem.item) + 1, 0, _cAddedRecord.item);
                        }
                    }
                    insertIndex = updatedCollectionIndex;
                    break;
                case ej.Gantt.AddRowPosition.Child:
                    level = selectedItem.level + 1;
                    parentItem = selectedItem;
                    _cAddedRecord = proxy._createGanttRecord(data, level, parentItem);
                    _cAddedRecord.index = _cAddedRecord.taskId;

                    //validate predecessor of new record
                    validateArgs.data = data;
                    validateArgs.currentRecord = _cAddedRecord;
                    if (!proxy._validateAddRecordPrecessorValue(validateArgs) && proxy._isinAddnewRecord) {
                        return false;
                    }

                    //find next item position
                    var currentItemIndex = flatRecords.indexOf(selectedItem)

                    if (selectedItem.hasChildRecords) {
                        dataChildCount = proxy._$treegridHelper.ejTreeGrid("getChildCount", selectedItem, 0);
                        recordIndex = currentItemIndex + dataChildCount + 1;
                        //Expand Add record's parent item 
                        if (!selectedItem.expanded) {
                            var expanargs = {};
                            expanargs.expanded = true;
                            expanargs.data = selectedItem;
                            proxy._isInAdd = true;
                            proxy.expanding(expanargs);
                            proxy._isInAdd = false;
                            updatedRecords = proxy.getUpdatedRecords();
                        }
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + proxy._getVisibleChildRecordCount(selectedItem, 0, updatedRecords) + 1;

                    } else {
                        selectedItem.hasChildRecords = true;
                        selectedItem.childRecords = [];
                        selectedItem.expanded = true;
                        if (!model.parentTaskIdMapping) {
                            selectedItem.item[model.childMapping] = [];
                        }
                        selectedItem.isMilestone = false;
                        recordIndex = currentItemIndex + 1;
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + 1;
                        selectedItem.predecessor && proxy._updatePredecessorOnIndentOutdent(selectedItem);
                    }
                  
                    /* Record collection update */
                    flatRecords.splice(recordIndex, 0, _cAddedRecord);
                    updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                    ids.splice(recordIndex, 0, _cAddedRecord.taskId.toString());

                    /* data Source update */
                    if (!ej.isNullOrUndefined(parentItem)) {
                        childIndex = parentItem.childRecords.indexOf(selectedItem);
                        /*Child collection update*/
                        parentItem.childRecords.splice(childIndex + 1, 0, _cAddedRecord);
                        if (!model.parentTaskIdMapping) {
                            parentItem.item[model.childMapping].splice(childIndex + 1, 0, _cAddedRecord.item);
                        }
                        else {
                            _cAddedRecord.item[model.parentTaskIdMapping] = _cAddedRecord.parentItem.taskId;
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.push(_cAddedRecord.item);

                            } else {
                                dataSource.push(_cAddedRecord.item);
                            }
                        }
                    }
                    insertIndex = updatedCollectionIndex;
                    break;
            }

            //Clear previously selected Item
            proxy._deSelectRowItem()

            /*Updated record collections in treegrid side*/
            proxy._$treegridHelper.ejTreeGrid("setUpdatedRecords", model.flatRecords, model.updatedRecords, model.ids, model.parentRecords, this.dataSource());

            /* Trigger action complete event for further updated*/
            var eventArgs = {};
            eventArgs.requestType = "save";
            eventArgs._cAddedRecord = eventArgs.addedRecord = _cAddedRecord;
            eventArgs.index = insertIndex;
            proxy.actionComplete(eventArgs);

            /*Set flag for refresh newly added record in non virtalization mode*/
            var treeGridObject = proxy._$treegridHelper.ejTreeGrid("instance");
            if (model.enableVirtualization === false &&
                (treeGridObject.model.sortSettings.sortedColumns.length > 0 || treeGridObject._searchString.length > 0)) {
                proxy._isRefreshAddedRecord = true;
            }
            /* scroll to new added record */
            proxy._$treegridHelper.ejTreeGrid("updateScrollBar");
            //For dialog add reference
            return true;
        },
        
        deleteItem: function () {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("deleteRow");
        },


        

        expandAllItems:function() {
            this._expandAll();
        },

        
        collapseAllItems:function() {
            this._collapseAll();
        },
        
          
        indentItem: function () {
            var proxy = this, model = this.model,
                flatRecords = proxy.model.flatRecords,
                ganttRecord = model.selectedItem, indentFlag = true;

            if (ganttRecord) {
                var recordIndex = flatRecords.indexOf(ganttRecord),
                previousGanttRecord;
                if (recordIndex === 0) {
                    indentFlag = false;
                }
                else {
                    if (ganttRecord.level === 0) {
                        indentFlag = true;
                    }
                    else {
                        previousGanttRecord = flatRecords[recordIndex - 1];
                        if (ganttRecord.level - previousGanttRecord.level === 1) {
                            indentFlag = false;
                        }
                        else {
                            indentFlag = true;
                        }
                    }
                }
            }
            if (indentFlag) {
                this._sendIndentRequest();
            } else {
                return false;
            }
        },

        
        outdentItem:function() {
            this._sendOutdentRequest();
        },
        

        
        cancelEdit:function() {
            this.sendCancelRequest();
        },
        

        
        expandCollapseRecord: function (taskId) {
            var proxy = this, args = {},
                record, model = this.model, expandStatus,
                currentViewIndex, $gridRow, $chartRow;

            record = model.flatRecords[model.ids.indexOf(taskId.toString())];

            if (!ej.isNullOrUndefined(record) && record.hasChildRecords) {
                var args = {};
                args.expanded = record.expanded == true ? false : true;
                expandStatus = proxy.getExpandStatus(record);

                /*If the record is within expanded parent item*/
                if (expandStatus) {
                    args.data = record;
                    if (args.expanded) {
                        proxy.expanding(args);
                    } else {
                        proxy.collapsing(args);
                    }
                }
                /*If the record is within collapsed parent item*/
                else {
                    record.expanded = args.expanded;
                    currentViewIndex = model.currentViewData.indexOf(record);
                    if (currentViewIndex != -1 && !model.enableVirtualization) {
                        $gridRow = $(proxy.getRows()[currentViewIndex]);
                        if (record.expanded) {
                            $gridRow.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
                            $gridRow.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');
                        } else {
                            $gridRow.find(".e-treegridexpand").removeClass('e-treegridexpand').addClass('e-treegridcollapse');
                            $gridRow.removeClass('e-treegridrowexpand').addClass('e-treegridrowcollapse');
                        }
                    }
                }
            }
        },

        
        saveEdit: function () {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("endEdit");
        },

       

        setSplitterPosition: function(width)
        {
            var proxy = this,
                leftPansize = 0,
                rightPaneSize = 0;
                ganttWidth = parseInt(proxy._ganttWidth) - proxy._totalBorderWidth,
                isPercentage = false;
            if (!ej.isNullOrUndefined(width)) {
                if (width.indexOf("%") != -1){
                    leftPaneSize = parseInt(width);
                    isPercentage = true;
                }
                else
                {
                    leftPaneSize = parseInt(width) / proxy._ganttWidth * 100;
                    isPercentage = false;
                }
                leftPaneSize = isNaN(leftPaneSize) ? 0 : leftPaneSize;
                
                //Check the grid width is less than or equal to 100% of gantt width.
                if (leftPaneSize <= 100)
                {
                    leftPaneSize = leftPaneSize < 0 ? 0 : leftPaneSize;
                    //Here 7 is the splitter span width
                    rightPaneSize = 100 - leftPaneSize - (9 / ganttWidth * 100);
                    // Check the right pane size is less than or equal to 0.
                    // if it is less than 0 mean adjust rightpane size is 0 and left pane size is 100
                    if (rightPaneSize <= 0)
                    {
                        if (isPercentage)
                            proxy.splitterPosition("100%");
                        else
                            proxy.splitterPosition(ganttWidth.toString());
                        leftPaneSize = 100 - (9 / ganttWidth * 100);
                        rightPaneSize = 0;
                    }
                    else if (rightPaneSize > 0) {
                        if (isPercentage)
                            proxy.splitterPosition(Math.round(leftPaneSize) + "%");
                        else
                            proxy.splitterPosition(Math.round((leftPaneSize * proxy._ganttWidth / 100)).toString());
                    }
                    var splitter = $("#e-ejSpliter" + proxy._id).ejSplitter("instance");
                    // Set left and right pane size to ejSplitter
                    splitter.model.properties[0].paneSize = leftPaneSize+"%";
                    splitter.model.properties[1].paneSize = rightPaneSize + "%";
                    splitter.refresh();
                    proxy._$treegridHelper.ejTreeGrid("refreshScroller", Math.round((leftPaneSize * ganttWidth) / 100));
                    proxy._$ganttchartHelper.ejGanttChart("refreshScroller", splitter.model.properties[1].paneSize - 1);
                    proxy._clearContextMenu();
                }
                else
                {
                    var splitter = $("#e-ejSpliter" + proxy._id).ejSplitter("instance");
                    var leftPaneSize = splitter.model.properties[0].paneSize;
                    var rightPaneSize = splitter.model.properties[1].paneSize;
                    if (parseInt(rightPaneSize) == 0)
                    {
                        if (isPercentage)
                            proxy.splitterPosition("100%");
                        else
                            proxy.splitterPosition(ganttWidth.toString());
                    }
                    else
                    {
                        if (isPercentage)
                            proxy.splitterPosition(Math.round((parseInt(leftPaneSize) / ganttWidth * 100)) + "%");
                        else
                            proxy.splitterPosition((leftPaneSize).toString());
                    }

                }
            }
        },

        _showTooltip: function (boolValue) {
            var proxy = this;
            this.model.enableTaskbarTooltip = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("showTooltip", boolValue);
        },

        _showEditingTooltip: function (boolValue) {
            var proxy = this;
            this.model.enableTaskbarDragTooltip = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("showEditingTooltip", boolValue);
        },
        
        _ganttChartEditing:function(boolValue) {
            var proxy = this;
            proxy.model.allowGanttChartEditing = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("ganttChartEditing", boolValue);
        },
        
        _resizeProgressbar: function (boolValue) {
            var proxy = this;
            proxy.model.enableProgressBarResizing = boolValue;
            proxy._$ganttchartHelper.ejGanttChart("resizeProgressbar", boolValue);

        },
        
        _sendSortingRequest: function(sortSettings) {
            var proxy = this, columnName, columnSortDirection;
            columnName = sortSettings.sortedColumns[0].field;
            columnSortDirection = sortSettings.sortedColumns[0].direction;
            proxy._$treegridHelper.ejTreeGrid("sortColumn", columnName, columnSortDirection);
        },
        
        _sortingRequest: function (boolValue) {
            var proxy = this, model = proxy.model, sorting;
            proxy.model.allowSorting = boolValue;           
            proxy._$treegridHelper.ejTreeGrid("sortSetting", boolValue);
        },
        _columnIndex: function (value) {
            var proxy = this;
            proxy._$treegridHelper.ejTreeGrid("columnIndex", value);
        },
        sendCancelRequest: function () {
            var proxy = this,
                toolbar = $("#" + proxy._id + "_toolbarItems");
            proxy._$treegridHelper.ejTreeGrid("cancelEditCell");

            if (proxy.model.toolbarSettings.toolbarItems.indexOf("cancel") !== -1) {
                proxy._disabledToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
            }

            if (proxy.model.toolbarSettings.toolbarItems.indexOf("save") !== -1) {
                proxy._disabledToolItems.push($(toolbar).find(".e-saveitem").parent()[0]);
            }

            $(toolbar).ejToolbar('disableItem', proxy._disabledToolItems);
            proxy._disabledToolItems = [];
        },
        
        _sendAddRequest: function () {
            
            var proxy = this,                
                columns = proxy._columns,
                length = columns.length,
                columnCount = 0,
                cloneData = {},
                args = {};
            proxy._isinAddnewRecord = true;
            
            for (columnCount; columnCount < length; columnCount++)
                cloneData[columns[columnCount].mappingName] = "";

            args.data = cloneData;
            args.requestType = "add";            
            //  proxy._$treegridHelper.ejTreeGrid("clearSelection");
            proxy.renderedEditDialog(args);
        },

        _sendEditRequest: function ($tr) {
            var proxy = this,
                model = proxy.model,
                args,
                data;
            
            if (ej.isNullOrUndefined($tr)) {
                proxy._currentTrIndex = this.selectedRowIndex();
                proxy._$currentTr = $(proxy._$treegridHelper.ejTreeGrid("getRows")).eq(this.selectedRowIndex());
            } else {
                proxy._currentTrIndex = $tr.index();
                proxy._$currentTr = $tr;
            }
            data = proxy.getUpdatedRecords()[this.selectedRowIndex()];

            args = { rowElement: proxy._$currentTr, data: data };
            proxy._trigger("beginEdit", args);
            args.requestType = "beginedit";
            if(args.data)
                proxy.renderedEditDialog(args);
        },
        _updateAddData:function(obj)
        {
            var proxy = this, model = this.model, id, startDate;
           
            if (!obj[model.taskIdMapping])
            {
                id= model.flatRecords.length;
                id = proxy._getNewTaskId(id);
                obj[model.taskIdMapping] = id;
            }

            if (model.taskNameMapping && !obj[model.taskNameMapping]) {
                obj[model.taskNameMapping] = proxy._newTaskTexts["newTaskName"] + " " + obj[model.taskIdMapping];
            }

            if (!obj[model.startDateMapping]) {
                obj[model.startDateMapping] = model.scheduleStartDate;
            }

            if (model.durationMapping && ej.isNullOrUndefined(obj[model.durationMapping])) {
                if (!obj[model.endDateMapping])
                    obj[model.durationMapping] = "5";
            }
            if (model.progressMapping) {
                obj[model.progressMapping] = obj[model.progressMapping] ? (obj[model.progressMapping] > 100 ? 100 : obj[model.progressMapping]) : 0;
            }

            if (!obj[model.endDateMapping] && model.endDateMapping) {

                if (!obj[model.durationMapping]) {
                    var startDate = proxy._getDateFromFormat(model.scheduleStartDate);
                    startDate.setDate(startDate.getDate() + 4);
                    obj[model.endDateMapping] = Globalize.format(startDate, this.model.dateFormat);
                }
            }
        },
        _sendSaveRequest: function (saveType) {
            var obj = {},
                doc = document,
                proxy = this,
                model = proxy.model,
                index = 0,
                formelement,
                length,
                editedRowWrap,
                columnName,
                $element,
                startdate,
                enddate,
                column,
                value, dialogId, treeGridId,
                args = {};
            this._duplicate = false;
            this._wrongenddate = false;
            if (saveType === "Add")
            {
                formelement = doc.getElementById(proxy._id + "AddForm");
                length = formelement && formelement.length;
                dialogId = "Add";
                treeGridId = "#treegrid" + proxy._id + "predecessorAdd";
                $element = $("#" + proxy._id + "taskId" + dialogId);             
                column = ej.TreeGrid.getColumnByField(proxy._columns, "taskId");
                proxy._cellEditColumn = column;
                if (column && $element.length>0)
                    value = proxy.getCurrentEditCellData($element, column.dropdownData);

                if (proxy.model.ids.indexOf(value) != -1 && value)
                    this._duplicate = true;
                else if (value) {
                    this._duplicate = false;
                }

                if (!this.addFormValidate())
                    return true;
            }
            else
            {
                formelement = doc.getElementById(proxy._id + "EditForm");
                length = formelement && formelement.length;
                treeGridId = "#treegrid" + proxy._id + "predecessoredit";
                dialogId = "";

                if (!this.editFormValidate())
                    return true;
            }
            editedRowWrap = $(formelement).closest('div');                    
                      
            if (!this._duplicate) {


                for (var count = 0; count < proxy._columns.length; count++) {
                    $element = $(formelement).find("#" + proxy._id + proxy._columns[count].field + dialogId);
                    if ($element.length > 0) {
                        proxy._cellEditColumn = proxy._columns[count];
                        value = proxy.getCurrentEditCellData($element, proxy._columns[count].dropdownData); // formelement[index].value;
                        if (proxy._isinAddnewRecord === false && proxy._columns[count].mappingName === proxy.model.predecessorMapping) {
                            args.previousValue = proxy._updatePredecessorValue(obj, value);
                        }
                        obj[proxy._columns[count].mappingName] = value;
                    }
                }
                date1 = proxy._getDateFromFormat(obj[model.startDateMapping]);
                date2 = proxy._getDateFromFormat(obj[model.endDateMapping]);
                var diffdays=0;
                if (date1 && date2)
                    diffdays = date2.getTime() - date1.getTime();
                if (diffdays < 0)
                    this._wrongenddate = true;
                else
                    this._wrongenddate = false;
                if (!this._wrongenddate) {
                    // when adding new task resource value changes.
                if (obj[model.resourceIdMapping] && editedRowWrap.hasClass("e-addedrow"))
                {
                        var resource = [];
                        $.each(obj[model.resourceIdMapping], function (indx, value) { resource.push(value[model.resourceIdMapping]); })
                        obj[model.resourceIdMapping] = resource;
                }

                if (!obj[model.taskIdMapping] && saveType === "Edit")
                    obj[model.taskIdMapping] = proxy._editedDialogRecord.taskId;

                args.data = obj;

                //getting predecessor from edit dialog table    
                var prdecessor_validation_flag = false;
                var results = [], resultFlag = false;
                if (model.predecessorMapping)
                {
                    if (saveType === "Edit") {
                        resultFlag = model.editDialogFields.length === 0 ? true : false;
                        results = $.grep(model.editDialogFields, function (val, idx) {
                            return (val.field === model.predecessorMapping);
                        });
                    } else if (saveType === "Add")
                    {
                        resultFlag = model.addDialogFields.length === 0 ? true : false;
                        results = $.grep(model.addDialogFields, function (val, idx) {
                            return (val.field === model.predecessorMapping);
                        });
                    }
                }
                if (results.length > 0 || resultFlag)
                {
                    var pre = [];
                    var taskNameFlag = true;

                    /* save tree grid if it is in edit mode */
                    if ($(treeGridId).ejTreeGrid("model.isEdit")) {
                        $(treeGridId).ejTreeGrid("saveCell");
                    }

                    pre = $(treeGridId).ejTreeGrid("option", "dataSource");
                    $.each(pre, function (idx, value) {
                        if (!(value.id.length > 0 && value.name.length && value.type.length))
                        {
                            taskNameFlag = false;
                        }
                    });
                    if (taskNameFlag) {
                        var predecessorColl = [], predecessorString = [];//from,to,offset,predecessorType,isdrawn
                        var types = [{ id: "SS", text: "Start-Start", value: "Start-Start" }, { id: "SF", text: "Start-Finish", value: "Start-Finish" }, { id: "FS", text: "Finish-Start", value: "Finish-Start" }, { id: "FF", text: "Finish-Finish", value: "Finish-Finish" }];
                        var tempType;
                        for (var i = 0; i < pre.length; i++) {
                            var predecessor = {};
                            predecessor["from"] = obj.taskID;
                            predecessor["to"] = pre[i].id;
                            predecessor["isdrawn"] = false;
                            predecessor["offset"] = isNaN(parseInt(pre[i].offset)) ? 0 : parseInt(pre[i].offset);
                            for (var j = 0; j < types.length; j++) {
                                if (pre[i].type == types[j].text) {
                                    predecessor["predecessorType"] = types[j].id;
                                    tempType = types[j].id;
                                    break;
                                }
                            }
                            var tem = pre[i].id + tempType//+data[i].lags.toString();
                            pre[i].offset = isNaN(parseInt(pre[i].offset)) ? 0 : parseInt(pre[i].offset);
                            if (pre[i].offset !== 0) {
                                if (pre[i].offset < 0)
                                    tem += pre[i].offset.toString();
                                else
                                    tem += "+" + pre[i].offset.toString();
                            }
                            predecessorString.push(tem);
                            predecessorColl.push(predecessor);
                        }
                        args.predecessorString = predecessorString;
                    } else {
                        //Break here and give notification to users
                        alert(proxy._alertTexts["predecessorAddingValidationAlert"]);
                        return true;
                    }
                    //Validate predecessoe for editing
                    if (proxy._isinAddnewRecord === false) {
                        args.currentRecord = this.selectedItem();
                        if(proxy._editedPredecessorValidation(args))
                        {
                            args.previousValue = proxy._updatePredecessorValue(obj, predecessorString.join(','));
                        }
                        else {
                            alert(proxy._alertTexts["predecessorEditingValidationAlert"]);
                            return true;
                        }
                    }
                    obj[model.predecessorMapping] = predecessorString.join(',');
                }

                args.requestType = ej.TreeGrid.Actions.Save;
                args.recordIndex = (this.selectedRowIndex() !== -1) ? this.selectedRowIndex() : -1;
                if (proxy._trigger("actionBegin", args))
                    return true;
                if (editedRowWrap.hasClass("e-editedrow")) {
                    args._cModifiedData = proxy._updateRecord(obj);
                    if (proxy._editedDialogRecord) {
                        if (!args._cModifiedData.startDate) {
                            args._cModifiedData.startDate = new Date(proxy._editedDialogRecord.startDate);
                        }
                        if (!args._cModifiedData.endDate) {
                            args._cModifiedData.endDate = new Date(proxy._editedDialogRecord.endDate);
                        }
                        if (!args._cModifiedData.duration) {
                            args._cModifiedData.duration = proxy._editedDialogRecord.duration;
                        }
                        proxy._editedDialogRecord = null;
                    }
                    if (args.data[model.endDateMapping] && !args.data[model.durationMapping]) {
                        args._cModifiedData._calculateDuration(proxy._getDateFromFormat(args._cModifiedData.startDate), proxy._getDateFromFormat(args._cModifiedData.endDate), model.includeWeekend, model.holidays, args._cModifiedData.isMilestone, model);
                    }
                    args.modifiedRecord = args._cModifiedData;
                    proxy.actionComplete(args);
                }
                else if (editedRowWrap.hasClass("e-addedrow")) {
                    if (!proxy.addRecord(obj, ej.Gantt.AddRowPosition.BelowSelectedRow)) {
                        alert(proxy._alertTexts["predecessorEditingValidationAlert"]);
                        return true;
                    }
                }  
                proxy._cModifiedData = null;
                proxy._cAddedRecord = null;
                proxy._primaryKeyValues = [];
                proxy._isinAddnewRecord = false;
                return false;
                }
                else {
                    alert(proxy._alertTexts["dateValidationAlert"]);
                    return true;
                }
            }
            else {
                alert(proxy._alertTexts["idValidationAlert"]);
                return true;
            }   
        },

        _recordReturnerByID:function(id)
        {
            var proxy = this,
                records = proxy.model.flatRecords;
            for (var i = 0; i < records.length; i++) {
                if (records[i].taskId.toString() === id.toString()) {
                    return records[i]
                }
            }
            return null;

        },
        _editedPredecessorValidation: function (args) {

            var proxy = this,
                flag = true,
                recordId = args.currentRecord.taskId, currentId,
                predecessor = args.predecessorString.join(','), predecessorIdArray;
            if (predecessor.length > 0) {
                predecessorIdArray = proxy._idFromPredecessor(predecessor);
                for (var count = 0; count < predecessorIdArray.length; count++) {
                    //Check edited item has parent item in predecessor collection
                    var editingData = args.currentRecord;
                    if (editingData && editingData.parentItem) {
                        if (predecessorIdArray.indexOf(editingData.parentItem.taskId.toString()) !== -1) {
                            return false;
                        }
                    }
                    // Check if predecessor exist more then one 
                    var tempIdArray = predecessorIdArray.slice(0),checkArray=[],countFlag=true;
                    $.each(tempIdArray, function (index, value) { 
                        if(checkArray.indexOf(value)===-1)
                            checkArray.push(value);
                        else
                            countFlag = false;
                    });
                    if (countFlag === false) {
                        return false;

                    }
                    //Cyclick check  
                    currentId = predecessorIdArray[count];
                    var visitedIdArray = [];
                    var predecessorCollection = predecessorIdArray.slice(0);
                    predecessorCollection.splice(count, 1);


                    while (currentId !== null) {
                        var currentIdArray = [], currentIdIndex;
                        if (visitedIdArray.indexOf(currentId) === -1)
                        {
                            currentRecord = proxy._recordReturnerByID(currentId);
                            //Predecessor id not in records collection
                            if (ej.isNullOrUndefined(currentRecord))
                                return false;
                          //  var currentPredecessor="";
                            if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length>0)
                            {
                                $.each(currentRecord.predecessor, function (index, value) {
                                    if (currentRecord.taskId.toString() !== value.from) {
                                        currentIdArray.push(value.from.toString());
                                        currentIdIndex = index;
                                    }
                                });
                            //    currentPredecessor=currentRecord.predecessor[0].from
                            }
                            if (recordId.toString() === currentRecord.taskId.toString() || currentIdArray.indexOf(recordId.toString()) !== -1) { // || predecessorCollection.indexOf(currentPredecessor) !== -1
                                //cycylic occurs//break;
                                return false;
                            }
                            visitedIdArray.push(currentId);
                            if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length > 0) {
                                var result;
                                if (currentIdArray.length > 1) {
                                    result=this._predecessorValidation(currentIdArray, args);
                                }
                                else if (currentIdArray.length == 1)
                                    currentId = currentRecord.predecessor[currentIdIndex].from;

                                if (result === false) {
                                    return false;
                                }
                            }
                            else
                                break;
                        }
                        else
                        {
                            break;
                        }
                    }
                }
            }
            return flag;
        },

        _predecessorValidation: function (predecessor,args) {
            var proxy = this;
            var recordId = args.currentRecord.taskId;
            for (var count = 0; count < predecessor.length; count++) {
                currentId = predecessor[count];
                var visitedIdArray = [];
                var predecessorCollection = predecessor.slice(0);
                predecessorCollection.splice(count, 1);

                while (currentId !== null) {
                    var currentIdArray = [];
                    if (visitedIdArray.indexOf(currentId) === -1) {
                        currentRecord = proxy._recordReturnerByID(currentId);
                        //Predecessor id not in records collection
                        if (ej.isNullOrUndefined(currentRecord))
                            return false;

                        if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length > 0) {
                            $.each(currentRecord.predecessor, function (index, value) {
                                if (currentRecord.taskId.toString() !== value.from)
                                    currentIdArray.push(value.from.toString());
                            });
                        }
                        if (recordId.toString() === currentRecord.taskId.toString() || currentIdArray.indexOf(recordId.toString()) !== -1) { // || predecessorCollection.indexOf(currentPredecessor) !== -1
                            //cycylic occurs//break;
                            return false;
                        }
                        visitedIdArray.push(currentId);
                        if (!ej.isNullOrUndefined(currentRecord.predecessor) && currentRecord.predecessor.length > 0) {
                            currentId = currentRecord.predecessor[0].from;
                        }
                        else
                            break;
                    }
                    else {
                        break;
                    }
                }
            }
            return true;
            
        },
        _sendCancelRequest: function () {
            var proxy = this,
                model=proxy.model,
                args = {};

            if (model.dateFormat.indexOf("hh") != -1)
                $("#" + proxy._id + "EditForm").find(".e-datepicker").ejDateTimePicker("destroy");
            else
                $("#" + proxy._id + "EditForm").find(".e-datepicker").ejDatePicker("destroy");

            args.requestType = ej.TreeGrid.Actions.Cancel;
            proxy.processBindings(args);
            proxy._primaryKeyValues = [];
            proxy.model.currentData = null;
        },
        
        _updatePredecessorValue: function (obj, value) {
            var proxy = this,
                prevPredecessors,
                modifiedPredecessors,
                length=0,
                predecessor,
                ganttPredecessor = [],
                count = 0;
            prevPredecessors = proxy.selectedItem()["predecessor"];

            if (value.length > 0) {
                modifiedPredecessors = proxy.selectedItem()._calculatePredecessor(value);
            }
            if (modifiedPredecessors)
                length = modifiedPredecessors.length;

            var index = 0, prevPrdecessorlength=0;
            if(prevPredecessors)
                prevPrdecessorlength = prevPredecessors.length;


            for (index = 0; index < prevPrdecessorlength; index++) {
                predecessor = prevPredecessors[index];
                if (predecessor.from === proxy.selectedItem().taskId.toString()) {
                    ganttPredecessor.push(predecessor);
                }
            }
            for (count = 0; count < length; count++) {
                ganttPredecessor.push(modifiedPredecessors[count]);
            }
            proxy.selectedItem().predecessor = ganttPredecessor;
            if (proxy.model.predecessorMapping) {
                proxy.selectedItem().item[proxy.model.predecessorMapping] = value;
                proxy.selectedItem().predecessorsName = value;
            }

            return prevPredecessors;          //
        },


        getCurrentEditCellData: function ($element, dropdownData) {
            var proxy = this, model = proxy.model, date;
            if ($("#" + proxy._id + "EditForm").length || $("#" + proxy._id + "AddForm").length) {
                var cellValue="";// $element = $("#" + proxy._id + proxy._columnFieldName), 
                switch (proxy._cellEditColumn.editType) {
                    case ej.TreeGrid.EditingType.String:
                        cellValue = $element.val();
                        break;
                    case ej.TreeGrid.EditingType.Numeric:
                        cellValue = $element.ejNumericTextbox("getValue");
                        break;
                    case ej.TreeGrid.EditingType.Dropdown:
                        if ($element.attr("id").indexOf("resourceInfo") > 0) {
                            cellValue = proxy._getSelectedItem($element.ejDropDownList("model.selectedItems"), dropdownData);
                        } else
                            cellValue = $element.ejDropDownList("model.value");
                        break;
                    case ej.TreeGrid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:
                        date = $element.ejDatePicker("model.value");

                        if (proxy._cellEditColumn && !proxy._cellEditColumn.format) {
                            cellValue = proxy.getFormatedDate(date);
                        } else {
                            cellValue = date;
                        }
                        break;
                    case ej.TreeGrid.EditingType.DateTimePicker:
                        date = $element.ejDateTimePicker("model.value");
                        if (proxy._cellEditColumn && !proxy._cellEditColumn.format) {
                            cellValue = cellValue = proxy.getFormatedDate(date);
                        } else {
                            cellValue = date;
                        }
                        break;
                    case ej.TreeGrid.EditingType.Maskedit:
                        cellValue = $element.ejMaskEdit("model.value");
                        break;
                }
                return cellValue;
            }
            return null;
        },


        _getSelectedItem: function (indexArray, dropDownData) {
            var count = 0,
                length = indexArray.length,
                    selectedItems = [];
            if (dropDownData) {
                for (count = 0; count < length; count++) {
                    $.each(dropDownData, function (index, resourceInfo) {
                        if (indexArray[count] === index) {
                            selectedItems.push(resourceInfo);
                        }
                    });
                }
            }

            return selectedItems;
        },
        _updateRecord: function (data) {
            var proxy = this,model=this.model,
                columns=proxy._columns,
               length,
                index = 0,
                ganttRecord = proxy.getUpdatedRecords()[this.selectedRowIndex()];

            if (model.editDialogFields.length > 0) {
                var filteredColumns = [];
                var count = 0, resultColumn;
                for (; count < model.editDialogFields.length; count++) {
                    resultColumn = $.grep(proxy._columns, function (val) {
                        return val.mappingName === model.editDialogFields[count].field;
                    });
                    resultColumn.length && filteredColumns.push(resultColumn[0]);
                }
                columns = filteredColumns;
            }
            length = columns.length;

            for (index = 0; index < length; index++) {

                if (columns[index].field.toString() === "predecessor")
                    continue;
                //for predecessor from edit dialog window 
                
                ganttRecord[columns[index].field] = data[columns[index].mappingName];
                ganttRecord.item[columns[index].mappingName] = data[columns[index].mappingName];//field

                if (columns[index].field.toString() === "resourceInfo") {
                    proxy._updateResourceName(ganttRecord);
                }
            }
            if (ganttRecord.duration == 0 && ganttRecord.startDate.getTime() == ganttRecord.endDate.getTime())
                ganttRecord.isMilestone = true;
            else
                ganttRecord.isMilestone = false;

            return ganttRecord;
            
        },

        _sendIndentRequest: function () {

            var proxy = this,
                model = proxy.model,
                data = proxy.getCurrentData(),
                args = {},
                eveargs = {},
                previousGanttRecord,
                isRefresh = false,
                treeGridObject = proxy._$treegridHelper.ejTreeGrid("instance");

            if (ej.isNullOrUndefined(data)) {
                alert(proxy._alertTexts["indentAlert"]);
            }
            else {
                previousGanttRecord = model.flatRecords[model.flatRecords.indexOf(data) - 1];
                args.data = data;
                args.requestType = "indent"
                if (!proxy._trigger("actionBegin", args)) {

                    if (data.parentItem) {
                        var childIndex = data.parentItem.childRecords.indexOf(data);
                        data.parentItem.childRecords.splice(childIndex, 1);
                        if (!model.parentTaskIdMapping)
                            data.parentItem.item[model.childMapping].splice(childIndex, 1);
                    } else {
                        if (this.dataSource() instanceof ej.DataManager) {
                            if (this.dataSource().dataSource.offline && this.dataSource().dataSource.json) {
                                var dataSourceIndex = this.dataSource().dataSource.json.indexOf(data.item);
                                if (!model.parentTaskIdMapping)
                                    this.dataSource().dataSource.json.splice(dataSourceIndex, 1);
                            }
                            else if (proxy._isDataManagerUpdate) {
                                var dataSourceIndex = proxy._jsonData.indexOf(data.item);
                                if (!model.parentTaskIdMapping) {
                                    proxy._jsonData = proxy._$treegridHelper.ejTreeGrid("getUpdatedDatamanagerData");
                                    proxy._jsonData.splice(dataSourceIndex, 1);
                                }
                            }
                        }
                        else {
                            var dataSourceIndex = this.dataSource().indexOf(data.item);
                            if (!model.parentTaskIdMapping)
                                this.dataSource().splice(dataSourceIndex, 1);
                        }
                    }
                    if (data.level === previousGanttRecord.level) {
                        if (previousGanttRecord.childRecords) {
                            if (!previousGanttRecord.expanded) {
                                previousGanttRecord.childRecords.splice(previousGanttRecord.childRecords.length, 0, data);
                                data.parentItem = previousGanttRecord;
                                if (!model.parentTaskIdMapping)
                                    previousGanttRecord.item[model.childMapping].splice(previousGanttRecord.childRecords.length, 0, data.item);
                                else
                                    data.item[model.parentTaskIdMapping] = previousGanttRecord.taskId;
                            }
                        } else {
                            previousGanttRecord.childRecords = [];
                            previousGanttRecord.childRecords.push(data);
                            previousGanttRecord.expanded = true;
                            data.parentItem = previousGanttRecord;
                            previousGanttRecord.item[model.childMapping] = [];
                            if (!model.parentTaskIdMapping)
                                previousGanttRecord.item[model.childMapping].push(data.item);
                            else
                                data.item[model.parentTaskIdMapping] = previousGanttRecord.taskId;
                        }
                    } else if (previousGanttRecord.level - data.level === 1) {
                        previousGanttRecord.parentItem.childRecords.splice(previousGanttRecord.parentItem.childRecords.length, 0, data);
                        if (!model.parentTaskIdMapping)
                            previousGanttRecord.parentItem.item[model.childMapping].splice(previousGanttRecord.parentItem.childRecords.length, 0, data.item);
                        else
                            data.item[model.parentTaskIdMapping] = previousGanttRecord.parentItem.taskId;
                        data.parentItem = previousGanttRecord.parentItem;
                    } else if (previousGanttRecord.level - data.level > 1) {

                        var parentItem = proxy.getParentItem(previousGanttRecord, data.level);
                        parentItem.childRecords.splice(parentItem.childRecords.length, 0, data);
                        if (!model.parentTaskIdMapping)
                            parentItem.item[model.childMapping].splice(parentItem.childRecords.length, 0, data.item);
                        else
                            data.item[model.parentTaskIdMapping] = parentItem.taskId;
                        data.parentItem = parentItem;

                    }

                    if (data.parentItem.isMilestone) {
                        data.parentItem.isMilestone = false;
                    }
                    previousGanttRecord.hasChildRecords = previousGanttRecord.childRecords ? previousGanttRecord.childRecords.length > 0 : false;
                    data.level += 1;

                    if (treeGridObject._searchString.length > 0 || treeGridObject.model.sortSettings.sortedColumns.length > 0) {
                        isRefresh = true;
                    }
                    proxy.updateLevel(data, +1, isRefresh);

                    if (!isRefresh)
                        proxy.refreshGanttRecord(data);

                    //update parent recordsCollections
                    if (previousGanttRecord.level === 0) {
                        if (model.parentRecords.indexOf(previousGanttRecord) === -1) {
                            model.parentRecords.push(previousGanttRecord);
                        }
                    }
                    if (data.level > 0) {
                        if (model.parentRecords.indexOf(data) !== -1) {
                            var idx = model.parentRecords.indexOf(data);
                            model.parentRecords.splice(idx, 1);
                        }
                    }
                    //Update parentitem
                    if (data.parentItem) {
                        proxy._updateParentItem(data);
                    }
                    //Remove child predecessor from Parent item
                    if (previousGanttRecord.predecessor && previousGanttRecord.predecessor.length > 0 && model.predecessorMapping && previousGanttRecord.hasChildRecords) {
                        proxy._updatePredecessorOnIndentOutdent(previousGanttRecord);
                    }
                    if (!isRefresh)
                        proxy.refreshGanttRecord(previousGanttRecord);



                    if (!proxy.getExpandStatus(data)) {
                        var expanargs = {};
                        expanargs.expanded = true;
                        expanargs.data = data.parentItem;
                        proxy.expanding(expanargs);
                    }
                    proxy._$treegridHelper.ejTreeGrid("setUpdatedRecords", model.flatRecords, model.updatedRecords, model.ids, model.parentRecords, this.dataSource());
                    if (isRefresh) {
                        

                        args.requestType = "indent";
                        //update oudented reocords as per searching and sorting
                        proxy._$treegridHelper.ejTreeGrid("processBindings", args);
                        var tempArgs = {};
                        tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                        proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", tempArgs);
                        proxy.model.updatedRecords = proxy.getUpdatedRecords();
                        proxy.model.currentViewData = proxy.getCurrentViewData();
                        proxy._$ganttchartHelper.ejGanttChart("setUpdatedRecords", model.currentViewData, model.updatedRecords, model.flatRecords, model.ids);
                        proxy._$ganttchartHelper.ejGanttChart("refreshHelper", model.currentViewData, model.updatedRecords, proxy._totalCollapseRecordCount);
                        proxy.updateAltRow(model.updatedRecords, 0, 1);
                    }

                    if (model.predecessorMapping) {
                        proxy._isValidationEnabled = false;
                        proxy._connectorlineIds = [];
                        proxy._connectorLinesCollection = [];
                        proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                        proxy._createConnectorLinesCollection();
                    }

                    //row selection
                    var rowToindex = model.updatedRecords.indexOf(data);
                    var $ganttChartRows = proxy.getGanttChartRows();
                    args = {
                        rowElement: $ganttChartRows.eq(rowToindex),
                        data: proxy.model.selectedItem,
                        recordIndex: rowToindex
                    };

                    proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
                    proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
                    this.selectedItem(args.data);
                    this.selectedRowIndex(args.recordIndex);
                    proxy.updateIndentOutdentOption(data);
                    eveargs.data = data;
                    eveargs.requestType = "indent";
                    this._trigger("actionComplete", eveargs);
                }
            }
        },
        
        _updatePredecessorOnIndentOutdent:function(parentRecord)
        {

            var proxy = this, len = parentRecord.predecessor.length,
                predecessorCollection = parentRecord.predecessor,
                predecessorIndex,
                childRecord,
                id,
                updatedPredecessor = [];
            for (var count = 0; count < len; count++) {
                //remove predecessor connect with parent item
                if (predecessorCollection[count].to === parentRecord.taskId.toString()) {
                    childRecord = proxy.model.flatRecords[proxy.model.ids.indexOf(predecessorCollection[count].from)];
                    predecessorIndex = childRecord.predecessor.indexOf(predecessorCollection[count]);
                    childRecord.predecessor.splice(predecessorIndex, 1);
                    id = "parent" + childRecord['taskId'] + "child" + parentRecord['taskId'];
                    proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", id);
                }
                    //remove predecessor if parent record as predecessor of it's child record
                else if (predecessorCollection[count].from === parentRecord.taskId.toString()) {
                    childRecord = proxy.model.flatRecords[proxy.model.ids.indexOf(predecessorCollection[count].to)];
                    if (parentRecord.childRecords.indexOf(childRecord) !== -1) {
                        predecessorIndex = childRecord.predecessor.indexOf(predecessorCollection[count]);
                        childRecord.predecessor.splice(predecessorIndex, 1);
                        var stringPredecessor = proxy._predecessorToString(childRecord.predecessor, childRecord);
                        childRecord.item[model.predecessorMapping] = stringPredecessor;
                        childRecord.predecessorsName = stringPredecessor;
                        id = "parent" + parentRecord['taskId'] + "child" + childRecord['taskId'];
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", id);
                        proxy.refreshGanttRecord(childRecord);
                    }
                    else
                    {
                        updatedPredecessor.push(predecessorCollection[count]);
                    }
                }
            }
            parentRecord.predecessor = updatedPredecessor;
            parentRecord.item[model.predecessorMapping] = "";
            parentRecord.predecessorsName = "";

            //if (updatedPredecessor.length > 0) {
            //    proxy._updatedConnectorLineCollection = [];
            //    proxy._connectorlineIds = [];
            //    proxy._validatePredecessor(parentRecord);
            //    if (proxy._updatedConnectorLineCollection.length > 0) {
            //        proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
            //    }
            //}
        },
        
        _removePredecessor:function(predecessor,record)
        {
            var proxy = this, length = predecessor.length, count = 0, model = proxy.model, parentGanttRecord, childGanttRecord;
            for (count; count < length; count++)
            {
                if (predecessor[count].from !== record.taskId.toString())
                {
                    childGanttRecord = proxy.model.flatRecords[proxy.model.ids.indexOf(predecessor[count].from)];  //index st
                }
                else if (predecessor[count].to !== record.taskId.toString())
                {
                    childGanttRecord = proxy.model.flatRecords[proxy.model.ids.indexOf(predecessor[count].to)]
                }

                if (childGanttRecord) {
                    var cIndex = childGanttRecord.predecessor.indexOf(predecessor[count]);
                    if (cIndex !== -1)
                        childGanttRecord.predecessor.splice(cIndex, 1);

                    if (childGanttRecord.item[model.predecessorMapping]) {
                        childGanttRecord.item[model.predecessorMapping] = proxy._predecessorToString(childGanttRecord.predecessor, childGanttRecord);
                        childGanttRecord.predecessorsName = childGanttRecord.item[model.predecessorMapping];
                        proxy.refreshGanttRecord(childGanttRecord);
                    }
                }
            }

        },

        _predecessorToString:function(predecessorCollection,record)
        {
            var proxy = this, predecessorString=[],count=0,length=predecessorCollection.length;
            
            for (count; count < length;count++)
            {
                if(record.taskId.toString()!==predecessorCollection[count].from)
                    {
                        var tem = predecessorCollection[count].from + predecessorCollection[count].predecessorsType;
                        predecessorCollection[count].offset = isNaN(predecessorCollection[count].offset) ? 0 : predecessorCollection[count].offset;

                        if (predecessorCollection[count].offset !== 0) {
                            if (predecessorCollection[count].offset < 0)
                                tem += predecessorCollection[count].offset.toString();
                            else if (predecessorCollection[count].offset > 0)
                                tem += "+" + predecessorCollection[count].offset.toString();
                        }
                        predecessorString.push(tem);
                }
            }
           
            return predecessorString.join(',');
        },
        //get the parentItem of selected ganttRecord to perform the indent operation
        
        getParentItem:function(ganttRecord,level) {

            var proxy = this;

            if (ganttRecord.parentItem.level === level)
                return ganttRecord.parentItem;

            return proxy.getParentItem(ganttRecord.parentItem, level);
        },

        //Change the position or index for the outdent element 
        getSubChangeIndex: function (itemLevel, itemIndex, item) {
            var proxy = this, changetoIndex;
            for (var items = 0; items < proxy.model.updatedRecords.length; items++) {
                if (proxy.model.updatedRecords[items].level == itemLevel && proxy.model.updatedRecords[items].index > itemIndex) {
                    if (item.parentItem && proxy.model.updatedRecords[items].parentItem.taskId == item.parentItem.taskId)
                        changetoIndex = proxy.model.updatedRecords[items].index;
                    break;
                }
            }
            return changetoIndex;
        },

        //get the next sample level item from 
        _nextSameLevelItem:function(data,totRecords)
        {
            var proxy = this,
                level = data.parentItem.level, dataIndex = totRecords.indexOf(data), nextItem;
            for (i = dataIndex; i < totRecords.length; i++)
            {
                if (level === totRecords[i].level || level > totRecords[i].level) {
                    nextItem = totRecords[i];
                    break;
                }
            }
            return nextItem;
        },

        //Indent operation for gantt records
        _sendOutdentRequest: function () {
            var proxy = this,
                model = proxy.model,
                data = proxy.getCurrentData(),
                args = {},
                eveargs = {},
                updateNextItem,
                rowNextItem,
                updateInsertIndex,
                rowInsertIndex,
                dataUpdatedRecordIndex,
                dataRowIndex,
                childDataIndex,
                insertParentItem,
                rowNextItemChildIndex,
                dataChildCount, splicedRows, splicedIds;

            if (ej.isNullOrUndefined(data)) {
                alert(proxy._alertTexts["outdentAlert"]);
            } else {
                args.data = data;
                args.requestType = "outdent";
                if (!proxy._trigger("actionBegin", args)) {
                    if (data.level === 0)
                        return;
                    else {
                        //find where to insert data
                        rowNextItem = proxy._nextSameLevelItem(data, model.flatRecords);
                        dataRowIndex = model.flatRecords.indexOf(data);

                        childDataIndex = data.parentItem.childRecords.indexOf(data);
                        insertParentItem = data.parentItem.parentItem;
                        rowNextItemChildIndex = insertParentItem && data.parentItem.parentItem.childRecords.indexOf(rowNextItem);

                        //remove data from current parent item
                        data.parentItem.childRecords.splice(childDataIndex, 1);

                        if(!model.parentTaskIdMapping)
                            data.parentItem.item[model.childMapping].splice(childDataIndex, 1);

                        if (data.parentItem.childRecords.length == 0) {

                            data.parentItem.childRecords = null;
                            data.parentItem.expanded = false;
                            data.parentItem.hasChildRecords = false;
                        }
                        else
                        {
                            proxy._updateParentItem(data.parentItem.childRecords[0]);
                        }

                        //add to next parent item
                        if (insertParentItem) {
                            insertParentItem.childRecords.splice(rowNextItemChildIndex, 0, data);
                            if (!model.parentTaskIdMapping)
                                insertParentItem.item[model.childMapping].splice(rowNextItemChildIndex, 0, data.item);
                            else
                                data.item[model.parentTaskIdMapping] = insertParentItem.taskId;

                            data.parentItem = insertParentItem;
                        } else {

                            if (this.dataSource() instanceof ej.DataManager) {
                                if (this.dataSource().dataSource.offline && this.dataSource().dataSource.json) {
                                    if (!model.parentTaskIdMapping)
                                        this.dataSource().dataSource.json.push(data.item);
                                }
                                else if (proxy._isDataManagerUpdate) {
                                    if (!model.parentTaskIdMapping) {
                                        proxy._jsonData = proxy._$treegridHelper.ejTreeGrid("getUpdatedDatamanagerData");
                                        proxy._jsonData.push(data.item);
                                    }
                                }
                            } else {
                                if (!model.parentTaskIdMapping)
                                    if (rowNextItem) {
                                        var index = this.dataSource().indexOf(rowNextItem.item);
                                        this.dataSource().splice(index, 0, data.item);
                                    }
                                    else
                                        this.dataSource().push(data.item);
                            }
                            data.parentItem = null;
                            if (model.parentTaskIdMapping)
                                data.item[model.parentTaskIdMapping] = null;

                        }

                        //Records updates _rows collections
                        dataChildCount = proxy._$treegridHelper.ejTreeGrid("getChildCount", data, 0);
                        splicedRows = model.flatRecords.splice(dataRowIndex, dataChildCount + 1);
                        splicedIds = model.ids.splice(dataRowIndex, dataChildCount + 1);

                        if (rowNextItem) {
                            rowInsertIndex = model.flatRecords.indexOf(rowNextItem);
                        }
                        else {
                            rowInsertIndex = model.flatRecords.length;
                        }

                        if (data.hasChildRecords && data.childRecords.length > 0) {

                            for (i = 0; i < splicedRows.length; i++) {
                                model.flatRecords.splice(rowInsertIndex, 0, splicedRows[i]);
                                model.ids.splice(rowInsertIndex, 0, splicedIds[i]);
                                splicedRows[i].level -= 1;
                                rowInsertIndex++;
                            }
                        }
                        else {
                            model.flatRecords.splice(rowInsertIndex, 0, splicedRows[0]);
                            model.ids.splice(rowInsertIndex, 0, splicedIds[0]);
                            splicedRows[0].level -= 1;
                        }
                        //update parentitem of selected item after changing of its child 
                        if (data.parentItem) {
                            //change the structure of milestone after outdent
                            if (data.parentItem.isMilestone) {
                                data.parentItem.isMilestone = false;
                            }
                            data.parentItem.hasChildRecords && proxy._updateParentItem(data.parentItem.childRecords[0]);
                        }

                        if (data.hasChildRecords) {
                            proxy._updateParentItem(data.childRecords[0]);
                            data._updateParentProgress(data, model.progressMapping);
                        }

                        //update parent recordsCollections
                        if (data.level === 0) {
                            if (model.parentRecords.indexOf(data) === -1) {
                                model.parentRecords.push(data);
                            }
                        }

                        //update predecessor
                        if (data.predecessor && data.predecessor.length > 0 && model.predecessorMapping && data.hasChildRecords) {
                            proxy._updatePredecessorOnIndentOutdent(data)
                        }

                        if (model.enableVirtualization) {
                            model.updatedRecords = proxy._$treegridHelper.ejTreeGrid("getExpandedRecords", model.flatRecords);
                        }
                        else {
                            model.updatedRecords = model.flatRecords.slice();
                        }

                        //Refreshing the Gantt and TreeGrid data
                        proxy._$treegridHelper.ejTreeGrid("setUpdatedRecords", model.flatRecords, model.updatedRecords, model.ids, model.parentRecords,this.dataSource());
                        args.requestType = "outdent";
                        //update oudented reocords as per searching and sorting
                        proxy._$treegridHelper.ejTreeGrid("processBindings", args);
                        var tempArgs = {};
                        tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                        proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", tempArgs);

                        proxy.model.updatedRecords = proxy.getUpdatedRecords();
                        proxy.model.currentViewData = proxy.getCurrentViewData();
                        proxy._$ganttchartHelper.ejGanttChart("setUpdatedRecords", model.currentViewData, model.updatedRecords, model.flatRecords, model.ids);
                        if (proxy.model.predecessorMapping) {
                            proxy._isValidationEnabled = false;
                            proxy._connectorlineIds = [];
                            proxy._connectorLinesCollection = [];
                            proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                            proxy._createConnectorLinesCollection();
                        }

                        proxy._$ganttchartHelper.ejGanttChart("refreshHelper", model.currentViewData, model.updatedRecords, proxy._totalCollapseRecordCount);
                        proxy.updateAltRow(model.updatedRecords, 0, 1);

                        //   Making the outdented row as selected row 
                        var rowToindex = model.updatedRecords.indexOf(data);
                        var $ganttChartRows = proxy.getGanttChartRows();
                        args = {
                            data: proxy.model.selectedItem,
                            recordIndex: rowToindex
                        };

                        proxy._$ganttchartHelper.ejGanttChart("selectRows", args.recordIndex);
                        proxy._$treegridHelper.ejTreeGrid("selectRows", args.recordIndex);
                        this.selectedItem(args.data);
                        this.selectedRowIndex(args.recordIndex);
                        proxy.updateIndentOutdentOption(data);
                        eveargs.data = data;
                        eveargs.requestType = "outdent";
                        this._trigger("actionComplete", eveargs);
                    }
                }
            }
        },
        
        _refreshChildLevel: function (ganttRecord) {

            var proxy = this,
                count = 0,
                childRecords = ganttRecord.childRecords,
                length = childRecords.length,
                record;

            for (count = 0; count < length; count++) {
                record = childRecords[count];                
                proxy.refreshGanttRecord(record);
                if (record.hasChildRecords) {
                    proxy._refreshChildLevel(record);
                }
            }
        },

        _updateChildLevel:function(ganttRecord) {

            var proxy = this,
                count = 0,
                childRecords = ganttRecord.childRecords,
                length = childRecords.length,
                record;
            
            for (count = 0; count < length; count++) {
                record = childRecords[count];                
                record.level = record.level - 1;
                // proxy.refreshGanttRecord(record);
                if (record.hasChildRecords) {
                    proxy._updateChildLevel(record);
                }
            }
        },

        _removeChildRecords:function(ganttRecord,index){

            if (ganttRecord.parentItem) {
                var proxy = this,
                    model = proxy.model,
                    childIndex = 0,
                    record,
                    addedChildGanttRecordCount = 0,
                    length = 0;
                parentChildRecords = ganttRecord.parentItem.childRecords,
                parentChildRecordsLength = parentChildRecords && parentChildRecords.length;

                for (childIndex = index; childIndex < parentChildRecordsLength; childIndex++) {

                    record = parentChildRecords[childIndex];


                    if (record) {

                        if (ganttRecord.childRecords) {

                            length = ganttRecord.childRecords.length;
                            ganttRecord.childRecords.splice(length, 0, record);
                            if (!model.parentTaskIdMapping)
                                ganttRecord.item[model.childMapping].splice(length, 0, record.item);
                            else
                                record.item[model.parentTaskIdMapping] = ganttRecord.taskId;
                        } else {
                            ganttRecord.childRecords = [];
                            ganttRecord.item[model.childMapping] = [];
                            ganttRecord.childRecords.push(record);
                            if (!model.parentTaskIdMapping)
                                ganttRecord.item[model.childMapping].push(record.item);
                            else
                                record.item[model.parentTaskIdMapping] = ganttRecord.taskId;
                        }
                        record.parentItem = ganttRecord;                                             
                        addedChildGanttRecordCount++;
                    }
                }

                if (ganttRecord.parentItem.childRecords) {

                    ganttRecord.parentItem.childRecords.splice(index, addedChildGanttRecordCount);
                    if(!model.parentTaskIdMapping)
                        ganttRecord.parentItem.item[model.childMapping].splice(index, addedChildGanttRecordCount);
                    if (ganttRecord.parentItem.childRecords.length == 0) {
                        ganttRecord.parentItem.childRecords = null;
                        ganttRecord.parentItem.expanded = false;
                        ganttRecord.parentItem.hasChildRecords = false;
                        proxy.refreshGanttRecord(ganttRecord.parentItem);
                    }
                }
            }
        },

        updateChildGanttRecordLevel: function (data,index) {
            var childIndex = 0, proxy = this, model = proxy.model,
                parentChildRecords = data.parentItem&& data.parentItem.parentItem&& data.parentItem.parentItem.childRecords,
                parentChildRecordsLength =parentChildRecords&&parentChildRecords.length,
                addedChildRecordCount = 0;
            for (childIndex = index; childIndex < parentChildRecordsLength; childIndex++) {
                var childGanttRecord = parentChildRecords[childIndex];
                if (childGanttRecord.childRecords) {
                    childGanttRecord.childRecords.splice(childGanttRecord.childRecords.length, 0, childGanttRecord);
                    childGanttRecord.item[model.childMapping].splice(childGanttRecord.childRecords.length, 0, childGanttRecord.item);
                }
                else {
                    childGanttRecord.childRecords = [];
                    childGanttRecord.item[model.childMapping] = [];
                    if (childGanttRecord) {
                        childGanttRecord.childRecords.push(childGanttRecord);
                        childGanttRecord.item[model.childMapping].push(childGanttRecord.item);
                    }
                }
                proxy.refreshGanttRecord(data.parentItem);
                data.parentItem = childGanttRecord;
                ++addedChildGanttRecordCount;
            }
            data.parentItem.childRecords.splice(index, addedChildGanttRecordCount);
            if (data.parentItem.childRecords.length == 0) {
                data.parentItem.childRecords = null;
                data.parentItem.expanded = false;
                data.parentItem.hasChildRecords = false;
            }

            proxy.refreshGanttRecord(data.parentItem);

        },

        updateLevel: function (childGanttRecord, offset, isRefresh) {
            var proxy = this,
                count = 0,
                childRecords = childGanttRecord.childRecords,
                length =childRecords&& childRecords.length,
                ganttRecord;
            for (count; count < length; count++) {
                ganttRecord = childRecords[count];
                ganttRecord.level += offset;
                if (!isRefresh)
                    proxy.refreshGanttRecord(ganttRecord);
                if (ganttRecord.childRecords) 
                    proxy.updateLevel(ganttRecord, offset, isRefresh);
            }
        },

        addNewGanttRecord:function() {
            var proxy = this, eventArgs = {},newRecord;
            newRecord = this.model.flatRecords[this.model.flatRecords.length].item;
            eventArgs.requestType = "save";
            eventArgs._cAddedRecord = eventArgs.addedRecord = jQuery.extend(true, {}, newRecord);
            eventArgs.recordIndex = index + 1;
            proxy.subContextMenuAction(eventArgs);
            proxy._clearContextMenu();
        },
        getExpandStatus: function (record) {

            var parentRecord = record.parentItem;

            if (parentRecord != null) {

                if (parentRecord.expanded === false) {
                    return false;
                } else if (parentRecord.parentItem) {

                    if (parentRecord.parentItem.expanded === false) {
                        return false;
                    } else {
                        return this.getExpandStatus(parentRecord.parentItem);
                    }

                } else return true;
            } else return true;
        },
        subContextMenuAction: function (args) {
            var proxy = this,
            data = $.extend(true, {}, proxy._contextMenuSelectedItem.item), rowPosition;
            delete data[model.childMapping];
            delete data[model.predecessorMapping];
            delete data[model.taskNameMapping];
            
            if (args.position === "Below") {
                rowPosition = ej.Gantt.AddRowPosition.BelowSelectedRow;
            } else {
                rowPosition = ej.Gantt.AddRowPosition.AboveSelectedRow;
            }
            proxy.addRecord(data, rowPosition);
        },

        _getNewTaskId: function (taskId) {
            var proxy = this, tempTaskId;
            if (proxy.model.ids.indexOf(taskId.toString()) == -1) {              
                return taskId == 0 ? 1 : taskId;
            } else {
                taskId++;
                tempTaskId = proxy._getNewTaskId(taskId);
                return tempTaskId;
            }
        },

        contextMenuAction: function (args) {
            var proxy = this;
            if (args.requestType == "contextMenuAdd")
                proxy._sendAddRequest();
            if (args.requestType == "contextMenuIndent")
                proxy._sendIndentRequest();
            if (args.requestType == "contextMenuOutdent")
                proxy._sendOutdentRequest();
            if (args.requestType == "contextMenuDelete")
                proxy._$treegridHelper.ejTreeGrid("deleteRow");
            if (args.requestType == "contextMenuTaskDetails")
                proxy._sendEditRequest();
        },

        _renderContextMenu: function (e, index, item) {

            var proxy = this,
                eventArgs = {},
                args = {},
                posx,
				subposx,
                posy, addItemPositionY,addItemIndex,
                contextMenu,
                contextMenuUList,
                subContextMenu,
                subContextMenuUList, length, columnIndex;
            proxy._contextMenuSelectedIndex = index;
            proxy._contextMenuSelectedItem = item;
            proxy._contextMenuEvent = e;
            columnIndex = proxy._$treegridHelper.ejTreeGrid("getCellIndex", e);
            //Initialize the contextmenu item to avoid duplicates
            //proxy._contextMenuItems = proxy._getContextMenuItems();
            //proxy._subContextMenuItems = proxy._getsubContextMenuItems();
            if (!e) e = window.event;
            if (e.pageX || e.pageY) {
                subposx = posx = e.pageX;
                posy = e.pageY;
            } else if (e.clientX || e.clientY) {
                subposx = posx = e.clientX + document.body.scrollLeft
                    + document.documentElement.scrollLeft;
                posy = e.clientY + document.body.scrollTop
                    + document.documentElement.scrollTop;
            }

            var ganttOffset = this._$treegridHelper.ejTreeGrid("getOffsetRect", proxy.element[0]);
            ganttOffset.bottom = ganttOffset.top + this.element[0].offsetHeight;
            ganttOffset.right = ganttOffset.left + $(this.element).width();            
            args.requestType = "ContextMenuOpen";
            args.contextMenuItems = proxy._contextMenuItems;
            args.columnIndex = columnIndex;
            args.innerContextMenuItems = proxy._subContextMenuItems;
            if (proxy._trigger("contextMenuOpen", args))
                return;
            length = proxy._contextMenuItems.length;

            for (var count = 0 ; count < length; count++)
            {
                if (proxy._contextMenuItems[count].menuId === "Add") {
                    addItemIndex = count;
                    break;
                }
            }
            addItemPositionY = posy + (addItemIndex * 30); //30-menu items height

            contextMenu = ej.buildTag("div.e-contextmenu", "", {
                'display': 'table',
                'position': 'absolute',
                'border': '1px solid #bbbcbb',
                'z-index': '10033',
            }, {"id":proxy._id+"_ContextMenu"});

            contextMenuUList = ej.buildTag("ul", $.render[proxy._id + "contextMenuTemplate"](args.contextMenuItems), {
                'margin': '0px',
                'padding-left': '0px'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });

            subContextMenu = ej.buildTag("div.e-innerContextmenu", "", {
                'display': 'table',
                'position': 'absolute',
                'top': (addItemPositionY) + 'px',
                'border': '1px solid #bbbcbb',
                'z-index': '10',
                'visibility': 'hidden',
            }, { "id": proxy._id + "_SubContextMenu"});

            subContextMenuList = ej.buildTag("ul", $.render[proxy._id + "subContextMenuTemplate"](args.innerContextMenuItems), {
                'margin': '0px',
                'padding-left': '0px'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });
            contextMenu.append(contextMenuUList);
            $(document.body).append(contextMenu);

            if (args.innerContextMenuItems.length > 0) {
                subContextMenu.append(subContextMenuList);
                $(document.body).append(subContextMenu);
            }
         
            var contextMenuWidth = $(contextMenu).width(),
                contextMenuHeight = $(contextMenu).height(),
                subContextMenuWidth = $(subContextMenu).width(),
                contextMenuItems = contextMenu.find(".e-menuitem"),
                subContextMenuItems = subContextMenu.find(".e-menuitem"),
                contextMenuSelectItem = contextMenuItems.eq(0);

            if (ganttOffset.left > posx || (ganttOffset.right < (posx + contextMenuWidth))) {

                if (ganttOffset.right < (posx + contextMenuWidth)) {
                    posx = posx - contextMenuWidth;
                    subposx = posx - subContextMenuWidth - 2;
                }
                if (posx < 0) {
                    posx = ganttOffset.left + 10;
                    subposx = posx + contextMenuWidth + 2;
                }
            } else {
                subposx = subposx + contextMenuWidth + 2;
                if (ganttOffset.right < (subposx + contextMenuWidth)) {
                    var tempSubPosX = posx - -subContextMenuWidth - 2;
                    if (tempSubPosX > 0)
                        subposx = tempSubPosX;
                }
            }

            if (ganttOffset.bottom < posy + (30 * proxy._contextMenuItems.length)) {
                var tempPosY = posy - (30 * proxy._contextMenuItems.length);
                if (tempPosY > 0) {
                    posy = tempPosY;
                }
            }
           
            proxy._subPosx = subposx;
            treegridObject = proxy._$treegridHelper.ejTreeGrid("instance");
            treegridObject._subPosx = subposx;
            if ($(window).height() < posy + contextMenuHeight)
                posy = posy - contextMenuHeight;

            contextMenu.css({ 'left': posx + 'px', 'top': posy + 'px', });
            subContextMenu.css({ 'top': posy + (addItemIndex * 30) + 'px', })
            contextMenuItems.css({ 'width': contextMenuWidth + 2 });
            subContextMenuItems.css({ 'width': subContextMenuWidth });
            proxy._$treegridHelper.focus();
            $(contextMenu).css({ 'height': 'auto' });
            proxy._on(contextMenu, "contextmenu", function (argsE) {
                argsE.preventDefault();
            });

            //click event handlers for contextmenu items and subcontextmenu items

                $('.e-menuitem').click(function () { 
                    var choice = $(this).attr('id');
                    switch (choice) {
                        case "Task":
                            eventArgs.requestType = "contextMenuTaskDetails";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Indent": eventArgs.requestType = "contextMenuIndent";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Outdent": eventArgs.requestType = "contextMenuOutdent";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Delete": eventArgs.requestType = "contextMenuDelete";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Above": eventArgs.requestType = "save";
                            eventArgs.recordIndex = index == 0 ? 0 : index;
                            eventArgs.position = "Above";
                            proxy.subContextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Below": eventArgs.requestType = "save";
                            eventArgs.recordIndex = index;
                            eventArgs.position = "Below";
                            proxy.subContextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Open":
                            eventArgs.requestType = "contextMenuAdd";
                            proxy.contextMenuAction(eventArgs);
                            proxy._clearContextMenu();
                            break;
                        case "Add":
                            proxy._setInitialData();
                            break;
                        default:
                            eventArgs.data = item;
                            proxy._triggerMenuEventHandler(choice, eventArgs);
                            proxy._clearContextMenu();
                            break;
                    }
                });
                var contextMenu = $("#" + proxy._id + "_ContextMenu"),
                    subContextMenu = $("#" + proxy._id + "_SubContextMenu"),
                    contextMenuItems = contextMenu.find(".e-menuitem"),
                    subContextMenuItems = subContextMenu.find(".e-menuitem"),
                    contextMenuSelectItem = contextMenuItems.eq(0);
                contextMenuSelectItem.addClass("e-contextmenu-mouseover");
                subContextMenuItems.eq(0).addClass("e-contextmenu-mouseover");
                //Mouseenter and mouse leave handler for contextmenu items and Subcontextmenuitems
                $(contextMenuItems).mouseenter(function () {
                    if (!$(this).hasClass("e-disable")) {
                        contextMenu.find(".e-contextmenu-mouseover").removeClass("e-contextmenu-mouseover");
                        $(this).addClass("e-contextmenu-mouseover");
                        if ($(this).attr('id') == "Add") {
                            $(subContextMenu).css({ 'visibility': 'visible', 'left': (proxy._subPosx + 2) });
                            subContextMenuItems.removeClass("e-contextmenu-mouseover");
                            subContextMenuItems.eq(0).addClass("e-contextmenu-mouseover");
                            $(contextMenu).find('#Add').addClass("e-contextmenu-mouseover");
                        }
                        else {
                            $(subContextMenu).css({ 'visibility': 'hidden' });
                        }
                    }
                });
                $(subContextMenuItems).mouseenter(function () {
                    +$("#" + proxy._id + "_SubContextMenu").find(".e-contextmenu-mouseover").removeClass("e-contextmenu-mouseover");
                    +$(this).addClass("e-contextmenu-mouseover");
                });
                e.preventDefault();
        },


        //Call custom context menu event handlers
        _triggerMenuEventHandler:function(menuId,eventArgs){
            var proxy = this,
                contextMenuItems = proxy._contextMenuItems,
                currentMenuItemCollection, menuItem;

            if (!ej.isNullOrUndefined(contextMenuItems)) {
                currentMenuItemCollection = contextMenuItems.filter(function (value) {
                    //convert menu id to string since id accepts only string values
                    if (value.menuId.toString() === menuId)
                        return true;
                });

                if (currentMenuItemCollection.length > 0) {
                    menuItem = currentMenuItemCollection[0];
                    var fn = menuItem.eventHandler;
                    if (fn) {
                        if (typeof fn === "string") {
                            fn = ej.util.getObject(fn, window);
                        }
                        if ($.isFunction(fn)) {
                            args = ej.event("customContextMenuHandler", this.model, eventArgs);
                            fn.call(this, args);
                        }
                    }

                }
            }
        },
        _setInitialData: function () {
            var eventArgs = {}, proxy = this,
                model = proxy.model;
            if (model.flatRecords.length == 0) {
                proxy.addRecord();
                proxy._clearContextMenu();
            }
        },

        //Updated the all child records expandstatus on expandCollapse
        updateConnectorLines: function ($gridRows, expanded) {
            var proxy = this,
                length = $gridRows.length / 2,
                count = 0,
                row,
                index,
                record;
            
            for (count = 0; count < length; count++) {
                row = $gridRows[count];
                index = $(proxy._gridRows).index(row);
                if (index !== -1) {
                    record = proxy.model.currentViewData[index];
                    record.isExpanded = expanded;                  
                }
            }
                   
        },

        updateAltRow: function (ganttRecord, recordIndex, offset) {
            var proxy = this,
                count = 0,
                currentViewData = proxy.model.currentViewData,
                length = currentViewData.length,
                isAltRow = ganttRecord.isAltRow,
                record,
                $gridRows = $(proxy._gridRows);
            for (count = recordIndex + offset; count < length; count++) {
                if ($($gridRows[count]).css('display') === "none")
                    continue;
                record = currentViewData[count];
                record.isAltRow = !isAltRow;
                isAltRow = record.isAltRow;
                if (isAltRow)
                    $($gridRows[count]).addClass('e-alt-row');
                else
                    $($gridRows[count]).removeClass('e-alt-row');
            }
        },
        renderedEditDialog: function (args) {
            var proxy = this;
            switch (args.requestType) {
                case ej.TreeGrid.Actions.Add:
                    var temp = document.createElement('div');
                    $(temp).addClass("e-addedrow");
                    temp.innerHTML = $.render[proxy._id + "_JSONDialogAddingTemplate"](args.data);
                    $("#" + proxy._id + "_dialogAdd").html($(temp));
                    var evntArgs = {};
                    evntArgs.cssClass = this.model.cssClass,
                    evntArgs.enableModal = true,
                    evntArgs.width = "auto",
                    evntArgs.enableResize = false,
                    evntArgs.contentSelector = "#" + proxy._id,
                    evntArgs.rtl = proxy.model.rtl,
                    evntArgs.allowKeyboardNavigation = false,
                    title = proxy._editDialogTexts["addFormTitle"];
                    evntArgs.title = title;
                    $("#" + proxy._id + "_dialogAdd").ejDialog(evntArgs);
                    $("#" + proxy._id + "_dialogAdd").ejDialog("open");
                    proxy._editedDialogRecord = null;
                    break;
                case ej.TreeGrid.Actions.BeginEdit:
                    temp = document.createElement('div');
                    $(temp).addClass("e-editedrow");
                    temp.innerHTML = $.render[proxy._id + "_JSONDialogEditingTemplate"](args.data);
                    $("#" + proxy._id + "_dialogEdit").html($(temp));
                    var evntArgs = {};
                    evntArgs.cssClass = proxy.model.cssClass;
                    evntArgs.rtl = model.rtl;
                    evntArgs.width = "530px";
                    evntArgs.enableResize = false;
                    evntArgs.contentSelector = "#" + proxy._id;
                    evntArgs.enableModal = true;
                    evntArgs.allowKeyboardNavigation = false;
                    var title = proxy._editDialogTexts["editFormTitle"] + " " + proxy.selectedItem()["taskName"];
                    evntArgs.open = function () {
                        var dialog = this, id = this._id;
                        var titleElement = $("#" + id + "_title").children(".e-title");
                        titleElement.width("450px");
                        titleElement.css({ "word-wrap": "normal", "text-overflow": "ellipsis", "white-space": "nowrap", "display": "inline-block", "overflow-x": "hidden", "float": "left" });
                        titleElement.attr("title", title);
                    };
                    evntArgs.title = title;
                    $("#" + proxy._id + "_dialogEdit").ejDialog(evntArgs);
                    $("#" + proxy._id + "_dialogEdit").ejDialog("open");
                    proxy._editedDialogRecord=jQuery.extend({}, this.selectedItem());
                    break;                    
            }
            proxy._refreshEditForm(args);
            //proxy.setValidation();
        },

        editFormValidate: function () {
            if ($.isFunction($.validator))
                return $("#" + this._id + "EditForm").validate().form();
            return true;
        },

        addFormValidate: function () {
            if ($.isFunction($.validator))
                return $("#" + this._id + "AddForm").validate().form();
            return true;
        },
        getCurrentData: function () {
            var proxy = this,
                updatedRecords = proxy.getUpdatedRecords();
            return updatedRecords && updatedRecords[this.selectedRowIndex()];
        },

        getCurrentViewData: function () {
            return this._$treegridHelper.ejTreeGrid("getCurrentViewData");
        },

        getUpdatedCurrentViewData: function () {
            return this._$treegridHelper.ejTreeGrid("getUpdatedCurrentViewData");
        },

        getRows: function () {
            return this._$treegridHelper.ejTreeGrid("getRows");
        },
        
        getGanttChartRows: function() {
            return this._$ganttchartHelper.ejGanttChart("getGanttChartRows");
        },

        getUpdatedRecords: function () {
            return this._$treegridHelper.ejTreeGrid("getUpdatedRecords");
        },

        _getToolBarHeight: function () {
            var toolbarH = 0;
            if (this.model.toolbarSettings.showToolbar) {
                toolbarH = $(".e-gantttoolbar").height();
            }
            return toolbarH;
        },

        _initValidator: function () {
            var gridObject = this;
            $("#" + this._id + "EditForm").validate({
                errorClass: 'e-field-validation-error',
                errorElement: 'div',
                wrapper: "div",
                errorPlacement: function (error, element) {
                    var $td = element.closest("td"), $container = $(error).addClass("e-error")
                    , $tail = ej.buildTag("div.e-errortail e-toparrow");
                    $td.find(".e-error").remove();
                    element.hasClass("e-numerictextbox") ? $container.insertAfter(element.closest(".e-numeric")) : $container.insertAfter(element);
                    $container.prepend($tail);
                    gridObject.model.editSettings.editMode != "normal" && $container.offset({ left: element.offset().left });
                    $container.fadeIn("slow");
                },

            });
        },

        setValidation: function () {
            this._initValidator();
            for (var i = 0; i < this._columns.length; i++) {
                if (!ej.isNullOrUndefined(this._columns[i]["validationRules"])) {
                    this.setValidationToField(this._columns[i].field, this._columns[i].validationRules);
                }
            }
        },

        setValidationToField: function (name, rules) {
            var ele = $("#" + this._id + name).length > 0 ? $("#" + this._id + name) : $("#" + name);
            ele.rules("add", rules);
            var validator = $("#" + this._id + "EditForm").validate();
            if (!ej.isNullOrUndefined(rules["required"])) {
                validator.settings.messages["Name"] = {};
                validator.settings.messages["Name"]["required"] = ej.TreeGrid.getColumnByField(proxy._columns, name).headerText + " is required";
            }
        },

        _createConnectorLinesCollection: function (args) {

            var proxy = this,
                model = this.model,
                recordLength = model.currentViewData.length,
                count,
                prdecessorIndex = -1,
                predecessorsCollection,
                ganttRecord,
                connectorLinesCollection = proxy._predecessorsCollection,
                connectorLinesLength = connectorLinesCollection.length,
                x = connectorLinesLength / 2,
                connectorLinesCount,
                enableVirtualization = model.enableVirtualization,
                record;


            if (model.enableVirtualization && args === "Load") {
                //THis is done for validate all the record during load time
                recordLength = model.updatedRecords.length;
                for (count = 0; count < recordLength; count++) {
                    ganttRecord = model.updatedRecords[count];
                    predecessorsCollection = ganttRecord["predecessor"];
                    if (predecessorsCollection) {
                        record = ganttRecord;
                        proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);
                    }
                }
            }
            else {

                for (count = 0; count < recordLength; count++) {
                    ganttRecord = model.currentViewData[count];
                    if (enableVirtualization === false && ganttRecord.isExpanded === false)
                        continue;
                    if (model.currentViewData.indexOf(ganttRecord) !== -1) {
                        predecessorsCollection = ganttRecord["predecessor"];
                        if (predecessorsCollection) {
                            record = ganttRecord;
                            proxy._addPredecessorsCollection(ganttRecord, predecessorsCollection);
                        }
                    }
                }

            }
            proxy._$ganttchartHelper && proxy._$ganttchartHelper.ejGanttChart("renderConnectorLines", proxy._connectorLinesCollection);
        },

        _addPredecessorsCollection: function (ganttRecord, predecessorsCollection) {
            var proxy = this,
                predecessorsLength,
                predecessorCount,
                predecessor,
                parentGanttRecord,
                childGanttRecord,
                model = proxy.model,
                connectorObj = {},
                predecessorValue = [],
                flatRecords = model.flatRecords,
                updatedRecords = model.updatedRecords,
                ids = model.ids;

            if (predecessorsCollection) {

                predecessorsLength = predecessorsCollection.length;

                for (predecessorCount = 0; predecessorCount < predecessorsLength; predecessorCount++) {

                    predecessor = predecessorsCollection[predecessorCount];


                    parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
                    childGanttRecord = flatRecords[ids.indexOf(predecessor.to)];

                        if (!model.enableVirtualization) {
                            if ((parentGanttRecord && parentGanttRecord.isExpanded === false) || 
                                (childGanttRecord && childGanttRecord.isExpanded === false))
                                continue;
                        }

                        if (proxy._isValidationEnabled && !proxy._isInExpandCollapse && childGanttRecord)
                           proxy._validatePredecessorDates(childGanttRecord);
                }

                //for calculate auto schedule only validation is enough  
                if (!proxy._enableCreateCollection)
                    return;

				// for correct re draw the alreadey created line in multiple predecessor for same task
                var count = 0;
                 $.each(predecessorsCollection, function (inx, value) {

                    if (value.to === ganttRecord.taskId.toString()) {
                        count++;
                    }
                    return count;
                });
				
				
				
                    for (predecessorCount = 0; predecessorCount < predecessorsLength; predecessorCount++) {

                        predecessor = predecessorsCollection[predecessorCount];
						if (count > 1)
                        {
                            predecessor.isdrawn = false;
                        }
						
						parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
						childGanttRecord = flatRecords[ids.indexOf(predecessor.to)];

                    if (updatedRecords.indexOf(parentGanttRecord) === -1 || updatedRecords.indexOf(childGanttRecord) == -1)
                                continue;

                            if (!model.enableVirtualization) {
                                if (parentGanttRecord.isExpanded === false || childGanttRecord.isExpanded === false)
                                    continue;
                            }

                            //if (proxy._isValidationEnabled && !proxy._isInExpandCollapse)
                            //    proxy._validateChildGanttRecord(parentGanttRecord, childGanttRecord, predecessor);

                        if (parentGanttRecord && childGanttRecord) {

                            connectorObj = proxy._createConnectorLineObject(parentGanttRecord, childGanttRecord, predecessor);
                          
                            if (connectorObj) {

                                if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId) == -1) {

                                    proxy._connectorLinesCollection.push(connectorObj);
                                    proxy._connectorlineIds.push(connectorObj.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.length == 0) {
                                    proxy._connectorLinesCollection.push(connectorObj);
                                    proxy._connectorlineIds.push(connectorObj.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId) != -1) {
                                    var index = proxy._connectorlineIds.indexOf(connectorObj.ConnectorLineId);
                                    proxy._connectorLinesCollection[index] = connectorObj;
                                }
                                predecessor.isdrawn = true;
                            }
                        }
                   // }
                }

            }
          
        },
        //create connector line object collection for rendering
        _createConnectorLineObject: function (parentGanttRecord, childGanttRecord, predecessor) {
            var connectorObj = {},
                proxy = this,
                model = proxy.model,
                parentTop,
                childTop,
                updatedRecords = model.updatedRecords,
                parentIndex = updatedRecords.indexOf(parentGanttRecord),
                childIndex = updatedRecords.indexOf(childGanttRecord),
                count = 0,
                index = 0;
            
            if (parentIndex === -1 || childIndex === -1){
                return;
            }
            if (!model.enableVirtualization) {
                var collapsedCount = 0;
                if (childIndex > parentIndex) {
                    
                    for (count = 0; count < parentIndex; count++) {
                        if (updatedRecords[count].isExpanded===false)
                            ++collapsedCount;
                    }
                    index = parentIndex;
                    parentIndex -= collapsedCount;
                    
                    for (index; index < childIndex; index++) {
                        if (updatedRecords[index].isExpanded === false)
                            ++collapsedCount;
                    }
                    childIndex -= collapsedCount;
                }
                else if (parentIndex > childIndex) {
                    for (count = 0; count < childIndex; count++) {
                        if (updatedRecords[count].isExpanded === false)
                            ++collapsedCount;
                    }
                    index = childIndex;
                    childIndex -= collapsedCount;
                    
                    for (index ; index < parentIndex; index++) {
                        if (updatedRecords[index].isExpanded === false)
                            ++collapsedCount;
                    }
                    parentIndex -= collapsedCount;
                }
            }

            if (parentIndex != -1 && childIndex != -1) {
                parentTop = parentIndex * model.rowHeight;
                childTop = childIndex * model.rowHeight;
                connectorObj["ParentLeft"] = parentGanttRecord['left'];
                connectorObj["ChildLeft"] = childGanttRecord['left'];
                connectorObj["ParentWidth"] = parentGanttRecord['width'] == 0 ? 26 : parentGanttRecord['width'];
                connectorObj["ChildWidth"] = childGanttRecord['width'] == 0 ? 26 : childGanttRecord['width'];
                connectorObj["ParentTop"] = parentTop;
                connectorObj["ChildTop"] = childTop;
                connectorObj["Height"] = parentTop > childTop ? parentTop - childTop : childTop - parentTop;
                connectorObj["Type"] = predecessor.predecessorsType;
                connectorObj["ConnectorLineId"] = "parent"+ parentGanttRecord['taskId'] + "child" + childGanttRecord['taskId'];                
                connectorObj["milestoneParent"] = parentGanttRecord.isMilestone ? true : false;
                connectorObj["milestoneChild"] = childGanttRecord.isMilestone ? true : false;
                
                return connectorObj;
            }
        },

        /* get date after weekend and holidays */
        _getDateAfterWeekendHolidays: function (date)
        {
            var proxy = this, model = this.model,
                holidays = this._getStringHolidays(),
                tempDate = new Date(date),
                isDateUpdated = false,
                formatedDate;
            tempDate.setHours(0,0,0,0);
            formatedDate = this._getFormatedDate(tempDate, model.dateFormat);
            
            /* Holiday checking */
            if (holidays.indexOf(formatedDate) != -1) {
                date.setDate(date.getDate() + 1);
                isDateUpdated = true;
            }

            /* Weekend updates */
            if (model.includeWeekend === false) {
                if (date.getDay() == 0) {
                    date.setDate(date.getDate() + 1);
                    isDateUpdated = true;
                }
                else if (date.getDay() == 6) {
                    date.setDate(date.getDate() + 2);
                    isDateUpdated = true;
                }
            }

            /* if Date modified check for another turn */
            if (isDateUpdated)
                return this._getDateAfterWeekendHolidays(date)
            else
                return date;
        },

        /*get date before holidays and weekends */
        _getDateBeforeWeekendHolidays: function (date) {
            var proxy = this, model = this.model,
                holidays = this._getStringHolidays(),
                tempDate = new Date(date),
                isDateUpdated = false,
                formatedDate;
            tempDate.setHours(0, 0, 0, 0);
            formatedDate = this._getFormatedDate(tempDate, model.dateFormat);

            /* Holiday checking */
            if (holidays.indexOf(formatedDate) != -1) {
                date.setDate(date.getDate() - 1);
                isDateUpdated = true;
            }

            /* Weekend updates */
            if (model.includeWeekend === false) {
                if (date.getDay() == 0) {
                    date.setDate(date.getDate() - 2);
                    isDateUpdated = true;
                }
                else if (date.getDay() == 6) {
                    date.setDate(date.getDate() - 1);
                    isDateUpdated = true;
                }
            }

            /* if Date modified check for another turn */
            if (isDateUpdated)
                return this._getDateBeforeWeekendHolidays(date)
            else
                return date;
        },
        /* update date as per offset value */
        _updateDateByOffset: function (date, offsetValue)
        {
            var proxy = this,
                offsetUpdate,
                holidaysCollections = proxy._stringHolidays,
                includeWeekend = this.model.includeWeekend;
                
            /*update offset value for date update */
            if (offsetValue < 0) {
                offsetValue *= -1;
                offsetUpdate = -1;
            } else {
                offsetUpdate = 1;
            }

            if (offsetValue) {
                for (var i = 0; i < offsetValue; i++) {
                    if (offsetUpdate < 0) {
                        date = this._getDateBeforeWeekendHolidays(date);
                    }
                    if (offsetUpdate > 0) {
                        date = this._getDateAfterWeekendHolidays(date);
                    }
                    date.setDate(date.getDate() + offsetUpdate);
                }
            }

            if (offsetUpdate < 0)
                date = this._getDateBeforeWeekendHolidays(date);
            else
                date = this._getDateAfterWeekendHolidays(date);

            return date;
        },

        /* update StartDate by using duration and enddate with weekends and holidays*/
        _setStartDateByDurationEndDate:function(ganttRecord)
        {
            var proxy = this,
                model = this.model,
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                headerValue = ej.Gantt.ScheduleHeaderType,
                duartionUnit = model.durationUnit,
                duration = duartionUnit == ej.Gantt.DurationUnit.Day ? ganttRecord.duration - 1 : ganttRecord.duration;

            if (ganttRecord.isMilestone)
                ganttRecord.startDate = ganttRecord.endDate;
            else {
                var tempEndDate = new Date(ganttRecord.endDate);
                //To calculate the Child Gantt record start date from its end date.
                for (var i = 0; i < duration; i++) {
                    if (duartionUnit == ej.Gantt.DurationUnit.Hour)
                        tempEndDate.setHours(tempEndDate.getHours() - 1);
                    else if (duartionUnit == ej.Gantt.DurationUnit.Day)
                        tempEndDate.setDate(tempEndDate.getDate() - 1);
                    else if (duartionUnit == ej.Gantt.DurationUnit.Minute)
                        tempEndDate.setMinutes(tempEndDate.getMinutes() - 1);
                    tempEndDate = this._getDateBeforeWeekendHolidays(tempEndDate);
                }
                ganttRecord.startDate = new Date(tempEndDate);
            }
        },
        //Validate the child Gantt Record
        _validateChildGanttRecord: function (parentGanttRecord, childGanttRecord, predecessor) {
            var proxy = this,
                model = proxy.model,
                childStartDate,
                parentStartDate,
                childEndDate,
                parentEndDate,
                offset, holiday,
                includeWeekend = model.includeWeekend,
                predecessorOffset = 0,
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                predecessorOffset = predecessor.offset ? parseInt(predecessor.offset) : 0,
                holidaysCollections = proxy._stringHolidays,
                durationUnit = model.durationUnit;

                if (proxy._editedTaskBarItem === childGanttRecord)
                {
                    return;
                }
            switch (predecessor.predecessorsType) {

                case 'SS':

                    parentStartDate = new Date(parentGanttRecord.startDate);
                    childStartDate = new Date(childGanttRecord.startDate);

                    /*update start date by predecessor */                    
                    parentStartDate = proxy._updateDateByOffset(parentStartDate, predecessorOffset);

                    childGanttRecord.startDate = new Date(parentStartDate);
                    var currentTaskId = childGanttRecord['taskId'].toString(),
                        predecessorsCollection = childGanttRecord['predecessor'],
                        childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });
                    if (childPredecessor.length > 1) {
                        var minStartDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);
                        if (model.includeWeekend === false) {
                            proxy._setBeforeWeekend(minStartDate);
                        }
                        childGanttRecord.startDate = new Date(minStartDate);
                    }

                    childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate,
                            proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.progressWidth = childGanttRecord._calculateProgressWidth(childGanttRecord.width, childGanttRecord.status);

                    break;

                case 'SF':

                    parentStartDate = new Date(parentGanttRecord.startDate);
                    childEndDate = new Date(childGanttRecord.endDate);                    
                    if (durationUnit == ej.Gantt.DurationUnit.Day)
                        parentStartDate.setDate(parentStartDate.getDate() - 1);

                    parentStartDate = this._getDateBeforeWeekendHolidays(parentStartDate);

                    /*update start date by predecessor */
                    parentStartDate = proxy._updateDateByOffset(parentStartDate, predecessorOffset);
                    childGanttRecord.endDate = new Date(parentStartDate);
                    /* udpate record startDate by endDate and duration*/ 
                    this._setStartDateByDurationEndDate(childGanttRecord);

                    var currentTaskId = childGanttRecord['taskId'].toString(),
                        predecessorsCollection = childGanttRecord['predecessor'],
                        childPredecessor = predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                    if (childPredecessor.length > 0) {
                        var minStartDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);
                        if (durationUnit == ej.Gantt.DurationUnit.Day)
                            minStartDate.setDate(minStartDate.getDate() - 1);

                        minStartDate = this._getDateBeforeWeekendHolidays(minStartDate);

                        childGanttRecord.endDate = new Date(minStartDate);
                        /*update start date by duration and end date */
                        this._setStartDateByDurationEndDate(childGanttRecord);
                    }
                    childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.progressWidth = childGanttRecord._calculateProgressWidth(childGanttRecord.width, childGanttRecord.status);
                    break;

                case 'FS':
                    parentEndDate = new Date(parentGanttRecord.endDate);
                    childStartDate = new Date(childGanttRecord.startDate);
                    if (!childGanttRecord.isMilestone) {
                        if (durationUnit == ej.Gantt.DurationUnit.Day)
                            parentEndDate.setDate(parentEndDate.getDate() + 1);
                    }

                    parentEndDate = this._getDateAfterWeekendHolidays(parentEndDate);
                    /* update date by predecessor offset */
                    parentEndDate = proxy._updateDateByOffset(parentEndDate, predecessorOffset);

                    childGanttRecord.startDate = new Date(parentEndDate);

                    if (proxy._editedTaskBarItem !== childGanttRecord) {
                        var predecessorsCollection = childGanttRecord['predecessor'],
                                currentTaskId = childGanttRecord['taskId'].toString(),
                                childPredecessor = predecessorsCollection && predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                        if (childPredecessor && childPredecessor.length > 1) {
                            var maxEndDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);
                            if (!childGanttRecord.isMilestone) {
                                if(durationUnit == ej.Gantt.DurationUnit.Day)
                                    maxEndDate.setDate(maxEndDate.getDate() + 1);
                            }
                            maxEndDate = this._getDateAfterWeekendHolidays(maxEndDate);
                            childGanttRecord.startDate = new Date(maxEndDate);
                        }
                    }
                    childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.progressWidth = childGanttRecord._calculateProgressWidth(childGanttRecord.width, childGanttRecord.status);
                    break;

                case 'FF':
                    parentEndDate = new Date(parentGanttRecord.endDate);
                    childEndDate = new Date(childGanttRecord.endDate);
                    /* update date by predecessor offset */
                    parentEndDate = this._updateDateByOffset(parentEndDate, predecessorOffset);

                    childGanttRecord.endDate = new Date(parentEndDate);

                    /* udpate record startDate by endDate and duration*/
                    this._setStartDateByDurationEndDate(childGanttRecord);


                    var currentTaskId = childGanttRecord['taskId'].toString(),
                                predecessorsCollection = childGanttRecord['predecessor'],
                              childPredecessor = predecessorsCollection && predecessorsCollection.filter(function (data) { return data.to === currentTaskId });

                    if (childPredecessor && childPredecessor.length > 1) {

                        var maxEndDate = proxy._getPredecessorDate(childGanttRecord, childPredecessor, predecessor.predecessorsType);
                        maxEndDate = this._getDateAfterWeekendHolidays(maxEndDate);
                        childGanttRecord.endDate = new Date(maxEndDate);
                        /*update start date by duration and end date */
                        this._setStartDateByDurationEndDate(childGanttRecord);
                    }

                    childGanttRecord.left = childGanttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.width = childGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                    childGanttRecord.progressWidth = childGanttRecord._calculateProgressWidth(childGanttRecord.width, childGanttRecord.status);
                    break;
            }

            if (childGanttRecord.parentItem)
                proxy._updateParentItem(childGanttRecord);

            if (proxy._isTreeGridRendered || proxy._isGanttChartRendered) {
                proxy.refreshGanttRecord(childGanttRecord);
            }

        },
        //Get maximum or minimum date as per predecessor type
        _getPredecessorDate: function (ganttRecord, predecessorsCollection, predecessorType) {
            var minStartDate = null, proxy = this, model = proxy.model,
                    maxEndDate = null,
                    length,
                    i,
                    predecessorOffset,
                    flatRecords = model.flatRecords,
                    ids = model.ids,
                    tempStartDate, tempEndDate,
                    validatedpredecessor = predecessorsCollection.filter(function (data) {
                        if (data.to === ganttRecord.taskId.toString() && data.predecessorsType === predecessorType)
                            return true;
                    });

            if (validatedpredecessor) {

                length = validatedpredecessor.length;

                    for (i = 0; i < length; i++) {

                        predecessor = validatedpredecessor[i];

                        parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
                        childGanttRecord = flatRecords[ids.indexOf(predecessor.to)];
                        predecessorOffset = predecessor.offset ? parseInt(predecessor.offset) : 0;

                        tempStartDate = proxy._getDateFromFormat(parentGanttRecord.startDate);
                        tempEndDate = proxy._getDateFromFormat(parentGanttRecord.endDate);

                        if (predecessorOffset) {
                            tempStartDate = this._updateDateByOffset(tempStartDate, predecessorOffset);// minStartDate.setDate(minStartDate.getDate() + parseInt(predecessor.offset));
                            tempEndDate = this._updateDateByOffset(tempEndDate, predecessorOffset);
                        }

                        if (minStartDate === null) {
                            minStartDate = tempStartDate;
                        }

                        if (maxEndDate === null) {
                            maxEndDate = tempEndDate;
                        }

                        if (tempStartDate.getTime() < minStartDate.getTime()) {
                            minStartDate = tempStartDate;
                       }

                        if (tempEndDate.getTime() > maxEndDate.getTime()) {
                            maxEndDate = tempEndDate;
                        }
                }
            }
            if (predecessorType == "FS" || predecessorType == "FF") {
                return maxEndDate;
            }
            else if (predecessorType == "SF" || predecessorType == "SS") {
                return minStartDate;
            }

        },

        _setBeforeWeekend: function(date) {
            if (date.getDay() == 0)
                date.setDate(date.getDate() - 2);
            else if (date.getDay() == 6)
                date.setDate(date.getDate() - 1);
        },
        
        _setAfterWeekend: function(date) {
            if (date.getDay() == 0)
                date.setDate(date.getDate() + 1);
            else if (date.getDay() == 6)
                date.setDate(date.getDate() + 2);
        },
        
        _updateEditedGanttRecords: function (args) {
            var proxy = this, model = proxy.model,
                ganttRecord = args.data,
                startDate = new Date(ganttRecord.startDate),
                isPredecessorsModified = false;

            if (args.columnName === "startDate") {
               
                if (this._stringHolidays &&
                      this._stringHolidays.indexOf(ganttRecord._getFormatedDate(startDate, model.dateFormat)) != -1) {
                    startDate.setDate(startDate.getDate() + 1);
                }
                
                if (proxy.model.includeWeekend === false) {
                    this._setAfterWeekend(startDate);
                    if (this._stringHolidays &&
                      this._stringHolidays.indexOf(ganttRecord._getFormatedDate(startDate, model.dateFormat)) != -1) {
                        startDate.setDate(startDate.getDate() + 1);
                    }
                }
                ganttRecord.startDate = new Date(startDate);
                ganttRecord.left = ganttRecord._calculateGanttRecordLeft(startDate, proxy._projectStartDate, proxy._perDayWidth);
                ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);// model.holidays
                ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, ganttRecord.status);
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
            }

            else if (args.columnName === "endDate") {
                //validate start date
                var endDate = new Date(ganttRecord.endDate);
                if (startDate.getTime() <= endDate.getTime()) {

                    if (this._stringHolidays &&
                          this._stringHolidays.indexOf(ganttRecord._getFormatedDate(endDate, model.dateFormat)) != -1) {
                        endDate.setDate(endDate.getDate() + 1);
                        if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() + 2);
                        }
                    }

                    if (model.includeWeekend === false) {
                        if (endDate.getDay() == 0) {
                            endDate.setDate(endDate.getDate() - 2);
                        } else if (endDate.getDay() == 6) {
                            endDate.setDate(endDate.getDate() - 1);
                        }
                    }
                    if (startDate.getTime() == endDate.getTime())
                        ganttRecord.isMilestone = true;
                    ganttRecord.endDate = new Date(endDate);
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays, ganttRecord.isMilestone, model);
                    ganttRecord.width = ganttRecord._calculateTaskbarWidth(proxy._perDayWidth, model);
                    ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, ganttRecord.status);
                    if (ganttRecord.parentItem)
                        proxy._updateParentItem(ganttRecord);
                }
                else
                {
                    ganttRecord.endDate = args.previousValue;
                    ganttRecord.item[model.endDateMapping] = args.previousValue;
                }
                if (ganttRecord.duration === 0)
                    ganttRecord.isMilestone = true;
                else
                    ganttRecord.isMilestone = false;
                
            }
            
            else if (args.columnName == "duration") {
                var currentDuration = ganttRecord.duration,
                    offset = currentDuration - args.previousValue,
                modifiedEndDate = proxy._calculateUpdatedDate(proxy._getDateFromFormat(ganttRecord.endDate), offset);

                if (currentDuration === 0) {
                    ganttRecord.isMilestone = true;
                } else {
                    ganttRecord.isMilestone = false;
                }

                ganttRecord.endDate = modifiedEndDate;
                ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);
                ganttRecord.progressWidth = ganttRecord._calculateProgressWidth(ganttRecord.width, ganttRecord.status);
                if(model.endDateMapping)
                    ganttRecord.item[model.endDateMapping] = modifiedEndDate;
                if (ganttRecord.parentItem)
                    proxy._updateParentItem(ganttRecord);
            }

            else if (args.columnName == "predecessor") {
                isPredecessorsModified = true;
                if (args.previousValue) {
                    proxy._removeConnectorLine(args.previousValue, ganttRecord);
                }
                proxy._addConnectorLine(ganttRecord);
            }

            else if (args.columnName == "status") {
                ganttRecord.status = ganttRecord.status > 100 ? 100 : ganttRecord.status;
                ganttRecord.progressWidth = proxy._getProgressWidth(ganttRecord.width, ganttRecord.status);
                if (ganttRecord.parentItem) {
                    if (ganttRecord.parentItem)
                        proxy._updateParentItem(ganttRecord);
                    
                }
            }

            //proxy._updateEditedGanttRecord(ganttRecord);
            if (ganttRecord.predecessor) {
                if (!isPredecessorsModified)
                    proxy._$ganttchartHelper.ejGanttChart("updateConnectorLineOffset", ganttRecord);
                proxy._isMileStoneEdited = ganttRecord.isMilestone;
                proxy._updatedConnectorLineCollection = [];
                proxy._connectorlineIds = [];
                proxy._validatePredecessor(ganttRecord);
                if (proxy._updatedConnectorLineCollection.length > 0) {
                    proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                }
            }
            proxy._isValidationEnabled = false;
            
            proxy.refreshGanttRecord(ganttRecord);
        },

        _removeConnectorLine: function (predecessor,ganttRecord) {
            var proxy = this,
                model = this.model,
                connectorLineId,
                parentGanttRecord,
                record,
                i = 0,
                length = predecessor && predecessor.length,
                index = -1,
                flatRecords = model.flatRecords,
                ids = model.ids;
                
            for (i = 0; i < length; i++) {
                parentGanttRecord = flatRecords[ids.indexOf(predecessor[i].from)];
                record = flatRecords[ids.indexOf(predecessor[i].to)];
                
                if (ganttRecord.taskId.toString() === predecessor[i].to.toString()) {
                    
                    index = parentGanttRecord.predecessor.indexOf(predecessor[i]);
                    index >= 0 && parentGanttRecord.predecessor.splice(index, 1);
                }

                if (parentGanttRecord && record) {
                    connectorLineId = "parent"+parentGanttRecord['taskId'] + "child" + record['taskId'];
                    proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                }
            }
        },

        _addConnectorLine:function(ganttRecord) {
            var proxy = this,
                count = 0,
                model = proxy.model,
                predecessors = ganttRecord.predecessor,
                length,
                predecessor,
                flatRecords = model.flatRecords,
                ids = model.ids,
                successorGanttRecord;

            if (predecessors) {

                length = predecessors.length;

                for (count = 0; count < length; count++) {

                    predecessor = predecessors[count];
                    
                    if (predecessor.to === ganttRecord.taskId.toString()) {
                        successorGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
                    }

                    if (successorGanttRecord) {
                        if (successorGanttRecord.predecessor) {
                            successorGanttRecord.predecessor.push(predecessor);
                        } else {
                            successorGanttRecord.predecessor = [];
                            successorGanttRecord.predecessor.push(predecessor);
                            proxy._predecessorsCollection.push(successorGanttRecord);
                        }
                    }
                }
            }
        },

        _calculateUpdatedDate: function (modifiedDate, offset) {
            var proxy = this, model = proxy.model,
                durationUnitValue = ej.Gantt.DurationUnit,
                i = 0;
            if (offset > 0) {
                for (i; i < offset; i++) {
                    if (model.durationUnit == durationUnitValue.Day) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, 1, 0);
                    }
                    else if (model.durationUnit == durationUnitValue.Hour) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, 0, 1);
                    }
                    else if (model.durationUnit == durationUnitValue.Minute) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, 0, 0, 1);
                    }
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() + 1);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() + 2);
                }
            } else {
                var dateOffset = (-1) * offset;
                for (i = 0; i < dateOffset; i++) {
                    if (model.durationUnit == durationUnitValue.Day) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, -1, 0);
                    }
                    else if (model.durationUnit == durationUnitValue.Hour) {
                        modifiedDate = proxy.dateUpdate(modifiedDate, 0, -1);
                    }

                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() - 2);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() - 1);
                }
            }
            modifiedDate = new Date(modifiedDate);
            return modifiedDate;
        },

        _updateEditedGanttRecord: function (ganttRecord) {
            var proxy = this;
            if (ganttRecord.predecessor) {
                var predecessorsCollection = ganttRecord.predecessor,
                    count = 0,
                    length = predecessorsCollection.length,
                    predecessor,
                    parentGanttRecord,
                    record,
                    connectorLineId,
                    connectorLineObject,
                    model = proxy.model,
                    flatRecords = model.flatRecords,
                    ids = model.ids;

                for (count = 0; count < length; count++) {
                    proxy._updatedConnectorLineCollection = [];
                    proxy._connectorlineIds = [];
                    predecessor = predecessorsCollection[count];
                    if (predecessor.to === ganttRecord['taskId'].toString()) {

                        parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
                        record = flatRecords[ids.indexOf(predecessor.to)];
                        
                        if (parentGanttRecord && record) {
                            connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                            proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                            connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                            if (connectorLineObject) {
                                if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                                    proxy._updatedConnectorLineCollection.push(connectorLineObject);
                                    proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                                }
                                else if (proxy._connectorlineIds.length == 0) {
                                    proxy._updatedConnectorLineCollection.push(connectorLineObject);
                                    proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                                }
                                proxy._$ganttchartHelper.ejGanttChart("appendConnectorLine", proxy._updatedConnectorLineCollection);
                            }
                        }
                        
                    }
                }
            }
        },
        

        _updateGanttRecord: function (ganttRecord) {
            var proxy = this,
                model = proxy.model;            
            ganttRecord.left = ganttRecord._calculateLeft(proxy._projectStartDate, proxy._perDayWidth, proxy._holidaysList, model);
            ganttRecord.width = ganttRecord._calculateWidth(proxy._perDayWidth,model.holidays, model);
            proxy.refreshGanttRecord(ganttRecord);            
            if (ganttRecord.parentItem)
                proxy._updateParentItem(ganttRecord);
        },

        refreshGanttRecord: function (ganttRecord) {
            var proxy = this;
            if (proxy._isTreeGridRendered) {
                proxy.model.currentViewData = proxy.getCurrentViewData();
                var index = proxy.model.currentViewData.indexOf(ganttRecord);
                var treegridObject = proxy._$treegridHelper.ejTreeGrid("instance");
                ej.TreeGrid.refreshRow(treegridObject, index);
                if (proxy._isGanttChartRendered) {
                    proxy._$ganttchartHelper.ejGanttChart("refreshRow", index);
                }
                //Client side event is triggerd for upadte the server side database
                var eventArgs = {};
                eventArgs.data = ganttRecord;
                eventArgs.item = ganttRecord.item;
                eventArgs.requestType = "recordUpdate";
                proxy._trigger("actionComplete",eventArgs);
            }
        },

        _updateParentItem: function (ganttRecord, editmode) {

            var proxy = this,
                parentGanttRecord = ganttRecord.parentItem,                
                model = proxy.model,
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                headerValue = ej.Gantt.ScheduleHeaderType,
                prevStartDate = new Date(parentGanttRecord.startDate),
                currentStartDate = new Date(ganttRecord.startDate),
                prevEndDate = new Date(parentGanttRecord.endDate),
                currentEndDate = new Date(ganttRecord.endDate);
                            
            
            if (currentStartDate.getTime() < prevStartDate.getTime()) {
                parentGanttRecord.startDate = ganttRecord.startDate;
                if (model.startDateMapping)
                    parentGanttRecord.item[model.startDateMapping] = parentGanttRecord.startDate;                
            }

            if (currentEndDate.getTime() > prevEndDate.getTime()) {                
                dateDiff = proxy._daydiff(prevEndDate, currentEndDate);
                parentGanttRecord.endDate = ganttRecord.endDate;
                if(model.endDateMapping)
                    parentGanttRecord.item[model.endDateMapping] = parentGanttRecord.endDate;

            }

            var count = 0,
                childRecords = parentGanttRecord.childRecords,
                length = childRecords && childRecords.length,
                childGanttRecord,
                index = null,
                minStartDate = null,
                maxEndDate = null,
                parentProgress = 0,
                milestoneCount = 0,
                    totalDuration = 0,
                    completedDuration = 0;

            for (count = 0; count < length; count++) {

                childGanttRecord = childRecords[count];

                if (minStartDate === null) {
                    minStartDate = new Date(childGanttRecord.startDate);
                }
                if (maxEndDate === null) {
                    maxEndDate = new Date(childGanttRecord.endDate);
                    index = model.flatRecords.indexOf(childGanttRecord);
                }
                if (childGanttRecord.endDate.getTime() > maxEndDate.getTime()) {
                    maxEndDate = new Date(childGanttRecord.endDate);
                    index = model.flatRecords.indexOf(childGanttRecord);
                }
                if (childGanttRecord.startDate.getTime() < minStartDate.getTime()) {
                    minStartDate = new Date(childGanttRecord.startDate);
                }

                if (!childGanttRecord.isMilestone) {
                    //converting days into hours .
                    //By now 24 hours is default//...
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                        totalDuration += (childGanttRecord.duration*8);
                        //summation of progress done of child
                        completedDuration += ((childGanttRecord.duration*8) * childGanttRecord.status) / 100;
                    }
                    else if (model.durationUnit == ej.Gantt.DurationUnit.Minute) {
                        totalDuration += (childGanttRecord.duration * 60 );
                        //summation of progress done of child
                        completedDuration += ((childGanttRecord.duration * 60) * childGanttRecord.status) / 100;
                    }
                    else {
                        totalDuration += (childGanttRecord.duration * 24);
                        //summation of progress done of child
                        completedDuration += ((childGanttRecord.duration * 24) * childGanttRecord.status) / 100;
                    }


                }

                else
                    milestoneCount++;
            }

            if (prevStartDate.getTime() !== minStartDate.getTime()) {
                parentGanttRecord.startDate = new Date(minStartDate);
                if (model.startDateMapping)
                    parentGanttRecord.item[model.startDateMapping] = parentGanttRecord.startDate;
            }
            
            if (prevEndDate.getTime() != maxEndDate.getTime()) {                                                       
                parentGanttRecord.endDate = new Date(maxEndDate);
                if(model.endDateMapping)
                    parentGanttRecord.item[model.endDateMapping] = parentGanttRecord.endDate;

            }

            
                parentGanttRecord._calculateDuration(new Date(parentGanttRecord.startDate),
                    new Date(parentGanttRecord.endDate), model.includeWeekend, model.holidays,
                    parentGanttRecord[model.milestoneMapping], model);
                if (ej.isNullOrUndefined(editmode) && model.durationMapping && headerType != headerValue.Hour) {
                    if ((model.durationUnit == ej.Gantt.DurationUnit.Hour || model.durationUnit == ej.Gantt.DurationUnit.Minute)
                        && model.dateFormat.indexOf("hh:") == -1)
                        parentGanttRecord.duration = model.flatRecords[index].duration;
                }
           
            //Reduce the duration of parent gantt record according to week end count when include week end is false.
            if (model.includeWeekend == false) {
                var dayDiff = proxy._daydiff(parentGanttRecord.startDate, parentGanttRecord.endDate),                    
                    weekEndCount = proxy._calculateWeekendCount(new Date(parentGanttRecord.startDate), dayDiff);               
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour)
                        parentGanttRecord.duration = parentGanttRecord.duration - (weekEndCount * 24);
                }            

            if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours ||
                model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) &&
           (headerType == headerValue.Day || headerType == headerValue.Hour)) {
                var beginDate = new Date(parentGanttRecord.startDate);
                //beginDate.setHours(8);
                var diffDate = (beginDate - proxy._projectStartDate) / 86400000;
            }
            else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours && model.roundOffDayworkingTime) {
                var diffDate = Math.round((parentGanttRecord.startDate - proxy._projectStartDate) / 86400000);
            }
            else {
                var diffDate = ((new Date(parentGanttRecord.startDate) - proxy._projectStartDate) / 86400000);
            }
            if (model.durationUnit == "hour" || model.durationUnit == "minute") {
                if (model.dateFormat.indexOf("hh:") == -1 && model.endDateMapping &&
                    model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours)
                    parentGanttRecord.left = (diffDate * proxy._perDayWidth) + (8 * proxy._perHourWidth);
                else
                    parentGanttRecord.left = (diffDate * proxy._perDayWidth);
            }
            else
                parentGanttRecord.left = (diffDate * proxy._perDayWidth);            
            parentGanttRecord.width = parentGanttRecord._calculateWidth(proxy._perDayWidth, proxy._holidaysList, model);//model.holidays

            //Calculate progressWidth after Left and width calculation
            parentProgress = totalDuration > 0 ? (completedDuration / totalDuration) * 100 : 0;
            parentGanttRecord.progressWidth = proxy._getProgressWidth(parentGanttRecord.width, parentProgress);
            parentGanttRecord.status = Math.round(parentProgress);

            if (model.progressMapping)
                parentGanttRecord.item[model.progressMapping] = parentGanttRecord.status;

            if (proxy._isTreeGridRendered || proxy._isGanttChartRendered)
                proxy.refreshGanttRecord(parentGanttRecord);

            if (parentGanttRecord.parentItem) {
                proxy._updateParentItem(parentGanttRecord);
            }

            
        },

        /* updated all records dates as per predecessor and successor collections */
        _updatedRecordsDateByPredecessor:function()
        {
            var proxy = this, model = this.model,
                flatRecords = model.flatRecords, length = flatRecords.length;
            for (var count = 0; count < length ; count++) {
                flatRecords[count].predecessor && proxy._validatePredecessorDates(flatRecords[count]);
            }
        },

        /* updated the record dates as per predecessor and successor collections */
        _validatePredecessorDates: function (childGanttRecord) {

            var proxy = this, model = proxy.model;

            if (childGanttRecord.predecessor) {

                var predecessorsCollection = childGanttRecord['predecessor'],
                    count,
                    parentGanttRecord,
                    record = null,
                    isValidated = false,
                    predecessor,
                    lowestOffset,
                    flatRecords = model.flatRecords,
                    ids = model.ids;

                var currentTaskId = childGanttRecord['taskId'].toString(),
                    predecessors = predecessorsCollection.filter(function (data) { return data.to === currentTaskId }),
                    successors = predecessorsCollection.filter(function (data) { return data.from === currentTaskId });
                var predecessorLength = predecessors.length;

                for (count = 0; count < predecessorLength; count++) {
                    predecessor = predecessors[count];
                    parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
                    record = flatRecords[ids.indexOf(predecessor.to)];
                    proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);
                }

                var length = successors.length;

                for (count = 0; count < length; count++) {

                    predecessor = successors[count];
                    parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
                    record = flatRecords[ids.indexOf(predecessor.to)];
                    proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);
                    record && proxy._validatePredecessorDates(record);

                }
            }
        },


        _validatePredecessor: function (childGanttRecord) {

            var proxy = this, model = proxy.model;

            if (childGanttRecord.predecessor) {

                var predecessorsCollection = childGanttRecord['predecessor'],
                    count,                    
                    parentGanttRecord,
                    record=null,
                    connectorLineId,
                    isValidated = false,
                    predecessor,
                    connectorLineObject,
                    lowestOffset,
                    flatRecords = model.flatRecords,
                    ids = model.ids;

                var currentTaskId = childGanttRecord['taskId'].toString(),

                    predecessors = predecessorsCollection.filter(function (data) { return data.to === currentTaskId }),
                    successors = predecessorsCollection.filter(function (data) { return data.from === currentTaskId });
               var predecessorLength = predecessors.length;

                for (count = 0; count < predecessorLength; count++) {
                    predecessor = predecessors[count];
                    parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];//THis is going to exception when id is not corrct order
                    record = flatRecords[ids.indexOf(predecessor.to)];
                    connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                    if (proxy._$ganttchartHelper)
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);

                    if (childGanttRecord.taskId.toString() === predecessor.to)
                        proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);


                    if (childGanttRecord.taskId.toString() === predecessor.from)
                        proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);

                }
                for (count = 0; count < predecessorLength; count++) {
                    predecessor = predecessors[count];
                    parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];//THis is going to exception when id is not corrct order
                    record = flatRecords[ids.indexOf(predecessor.to)];
             
                    if ((model.enableVirtualization === false && (parentGanttRecord.isExpanded === false || record.isExpanded == false)))//(model.enableVirtualization === false && !parentGanttRecord.isExpanded && record.isExpanded === false)
                        continue;
                    connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                    if (connectorLineObject) {
                        if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.length == 0) {
                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) != -1) {
                            var index = proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId);
                            proxy._updatedConnectorLineCollection[index] = connectorLineObject;
                        }
                    }                   

                }

                var length = successors.length;

                for (count = 0; count < length; count++) {

                    predecessor = successors[count];
                    parentGanttRecord = flatRecords[ids.indexOf(predecessor.from)];
                    record = flatRecords[ids.indexOf(predecessor.to)];
                    connectorLineId = "parent" + parentGanttRecord['taskId'] + "child" + record['taskId'];
                    if (proxy._$ganttchartHelper)
                        proxy._$ganttchartHelper.ejGanttChart("removeConnectorline", connectorLineId);
                    proxy._validateChildGanttRecord(parentGanttRecord, record, predecessor);
                    if ((model.enableVirtualization === false && (parentGanttRecord.isExpanded === false || record.isExpanded == false)))
                        continue;
                    connectorLineObject = proxy._createConnectorLineObject(parentGanttRecord, record, predecessor);
                    if (connectorLineObject) {
                        if (proxy._connectorlineIds.length > 0 && proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) == -1) {

                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.length == 0) {
                            proxy._updatedConnectorLineCollection.push(connectorLineObject);
                            proxy._connectorlineIds.push(connectorLineObject.ConnectorLineId);
                        }
                        else if (proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId) != -1) {
                            var index = proxy._connectorlineIds.indexOf(connectorLineObject.ConnectorLineId);
                            proxy._updatedConnectorLineCollection[index] = connectorLineObject;
                        }
                    }

                    record &&  proxy._validatePredecessor(record);

                }                
            }
        },
       
        _calculateDateDifference: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                offset = 0;
            
            if (dateDiff > 0)
                offset = differenceCount - this._calculateWeekendCount(emptyDate, dateDiff);
            else
                offset = dateDiff + this._calculateWeekendCount(emptyDate, dateDiff);
            return offset;
        },

        _calculateWeekendCount: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                weekendCount = 0,
                i = 0;
            
            if (dateDiff > 0) {
                for (i = 0; i < dateDiff; i++) {
                    if (emptyDate.getDay() == 0) {
                        weekendCount++;
                        emptyDate.setDate(emptyDate.getDate() + 1);
                    }
                    else if (emptyDate.getDay() == 6) {
                        weekendCount += 2;
                        ++i;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    } else
                        emptyDate.setDate(emptyDate.getDate() + 1);
                }
            }
            else {
                for (i = 0; i < differenceCount; i++) {
                    emptyDate.setDate(emptyDate.getDate() - 1);
                    if (emptyDate.getDay() == 0 || emptyDate.getDay() == 6) {
                        weekendCount += 1;
                    }
                }
            }
            return weekendCount;
        },

        _daydiff: function (first, second) {
            return ((new Date(second) - new Date(first)) / (1000 * 60 * 60 * 24));
        },

        


        dateUpdate: function (date, days, hours, minutes) {
            if (!minutes)
                minutes = 0;
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours() + hours,
                date.getMinutes() + minutes,
                date.getSeconds(),
                date.getMilliseconds()
            );
        },
        
        //refresh treegrid and gantt rows
        _refreshChartAndGridRows: function ()
        {
            var proxy = this, model = this.model;
            proxy._$treegridHelper.ejTreeGrid("processBindings");
            var tempArgs = {};
            tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
            proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", tempArgs);
            proxy.model.updatedRecords = proxy.getUpdatedRecords();
            proxy.model.currentViewData = proxy.getCurrentViewData();
            collapsedRecordCount = proxy._$treegridHelper.ejTreeGrid("getCollapsedRecordCount");
            proxy._totalCollapseRecordCount = collapsedRecordCount;
            height = (proxy.model.updatedRecords.length - collapsedRecordCount) * model.rowHeight;
            proxy._$ganttchartHelper.ejGanttChart("setUpdatedRecords", model.currentViewData, model.updatedRecords, model.flatRecords, model.ids);
            proxy._$ganttchartHelper.ejGanttChart("refreshHelper", model.currentViewData, model.updatedRecords, proxy._totalCollapseRecordCount);
            if (!proxy._isInAdd) {
                proxy._$treegridHelper.ejTreeGrid("updateHeight");
                proxy._$ganttchart.ejGanttChart("updateHeight", height);
                proxy._$ganttchartHelper.ejGanttChart("setCollapsedRecordCount", collapsedRecordCount);
                proxy._$ganttchartHelper.ejGanttChart("refreshGridLinesTable", proxy.model.updatedRecords.length - collapsedRecordCount);
                if (model.enableAltRow && model.currentViewData.length > 0)
                    ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);

                if (model.predecessorMapping) {
                    proxy._isValidationEnabled = false;
                    proxy._connectorlineIds = [];
                    proxy._connectorLinesCollection = [];
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._createConnectorLinesCollection();
                }
            }
        },
        //Refresh gantt after expnadAll or collapseAll operation
        _refreshGanttOnExpandCollapseAll:function()
        {
            var proxy = this,
                args = {},
                model=this.model,
                collapsedRecordCount = 0,
                height = 0;

            if (model.enableVirtualization) {
                args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
                proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", args);
                model.updatedRecords = proxy.getUpdatedRecords();
                model.currentViewData = proxy.getCurrentViewData();
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedRecords);
                proxy._$treegridHelper.ejTreeGrid("refreshHeight");
            } else {
                //If need refresh chart if record is added after searching or sorting
                if (proxy._isRefreshAddedRecord) {
                    proxy._refreshChartAndGridRows();
                    proxy._isRefreshAddedRecord = false;
                    return;
                }
                collapsedRecordCount = proxy._$treegridHelper.ejTreeGrid("getCollapsedRecordCount");
                proxy._totalCollapseRecordCount = collapsedRecordCount;
                height = (proxy.model.updatedRecords.length - collapsedRecordCount) * model.rowHeight;
                proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                proxy._$treegridHelper.ejTreeGrid("updateHeight");
                proxy._$ganttchart.ejGanttChart("updateHeight", height);
                proxy._$ganttchartHelper.ejGanttChart("setCollapsedRecordCount", collapsedRecordCount);
                proxy._$ganttchartHelper.ejGanttChart("refreshGridLinesTable", proxy.model.updatedRecords.length - collapsedRecordCount);
                if (model.enableAltRow && model.currentViewData.length > 0)
                    ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);
            }
            if (model.predecessorMapping) {
                proxy._isValidationEnabled = false;
                proxy._connectorlineIds = [];
                proxy._connectorLinesCollection = [];
                proxy._createConnectorLinesCollection();
            }
        },
        //Collapse all parent and inner level parent records
        _collapseAll: function () {
            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization,
                $gridRows = proxy.getRows(),
                $rowElement = null;

            proxy._$treegridHelper.ejTreeGrid("cancelEditCell"); // Cancel the edit cell, if it is in edit mode.
            proxy._gridRows = $gridRows;
            proxy._isInExpandCollapse = true;
            proxy._isInExpandCollapseAll = true;
            //loop excecute for iterate the GanttRecords preseent in the Gantt Control            
            for (count = 0; count < proxy.model.parentRecords.length; count++) {
                    ganttRecord = proxy.model.parentRecords[count];
                    args.data = ganttRecord;
                    args.recordIndex = count;
                    args.expanded = false;
                    if (ganttRecord.hasChildRecords) {
                        proxy._expandCollapseInnerLevelRecord(ganttRecord, args.expanded);
                        ganttRecord.expanded && ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    }
            }
            if (model.flatRecords.length > 0)
                proxy._refreshGanttOnExpandCollapseAll();

            proxy._isInExpandCollapse = false;
            proxy._isInExpandCollapseAll = false;
        },
        //expand or collapse all inner level parent records
        _expandCollapseInnerLevelRecord:function(record,expanded)
        {
            var proxy = this,
                length = record.childRecords.length,
                tempArgs = {};

            for (var count = 0; count < length; count++) {
                if (record.childRecords[count].hasChildRecords) {
                    proxy._expandCollapseInnerLevelRecord(record.childRecords[count], expanded);
                    tempArgs.data = record.childRecords[count];
                    tempArgs.expanded = expanded;
                    if(record.childRecords[count].expanded !== expanded)
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, tempArgs);

                }
            }
        },


        //Method for expand all the GanttRecords
        
        _expandAll: function () {
            
            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization,
                parentRecords = model.parentRecords,
                length = parentRecords.length;

            proxy._isInExpandCollapse = true;
            proxy._isInExpandCollapseAll = true;
            //loop excecute for iterate the GanttRecords present in the Gantt Control

            for (count = 0; count < length; count++) {
                ganttRecord = parentRecords[count];
                args.data = ganttRecord;
                args.recordIndex = count;
                args.expanded = true;
                if (ganttRecord.hasChildRecords ) {
                    proxy._expandCollapseInnerLevelRecord(ganttRecord, args.expanded);
                    if(!ganttRecord.expanded)
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                }
            }
            if (model.flatRecords.length > 0)
                proxy._refreshGanttOnExpandCollapseAll();

            proxy._isInExpandCollapse = false;
            proxy._isInExpandCollapseAll = false;
        },
        _getDateFromFormat: function (date) {
            
            if (typeof date === "object") {
                return new Date(date);
            }
            if (date) {
                return Globalize.parseDate(date, this.model.dateFormat) == null ?
                    new Date(date) : Globalize.parseDate(date, this.model.dateFormat);
            }
        },
        _getFormatedDate: function (dateObject, dateformat) {
            return Globalize.format(dateObject, dateformat);
        },

        
        _destroy: function () {
            var proxy = this;
            proxy.element.empty().removeClass("e-gantt-core e-gantt " + proxy.model.cssClass);
        },
        _updateToolbar: function () {
            var proxy = this,
                model = proxy.model;
            $("#" + proxy._id + "_toolbarItems").remove();
            if (model.toolbarSettings.showToolbar) {
                proxy._renderToolBar().insertBefore($("#e-ejSpliter" + proxy._id));
                proxy._updateToolbarOptions("");
            }
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                proxy._totalBorderHeight = model.toolbarSettings.showToolbar ? 3 : 2;//top,bottom,toolbar div
            } else {
                proxy._totalBorderHeight = model.toolbarSettings.showToolbar ? 1 : 0;//top box-sizing:border-box includes it's border
            }
            proxy._$treegridHelper.ejTreeGrid("cancelEditCell");
            //Calculate the viewport height based on toolbar height
            //Change the tree grid and gantt chart container height based on view port height
            var toolbar = $("#" + proxy._id + "_toolbarItems"),
                ganttbody = proxy.element.find(".e-ganttviewerbodyContianer"),
                gantthead = proxy.element.find(".e-ganttviewerheaderContainer"),
                treegridcontent = ("#ejTreeGrid" + proxy._id + "e-gridcontent"),
                splitter = $("#e-ejSpliter" + proxy._id),
                height = $(proxy.element).height() - proxy._totalBorderHeight,
                viewPortHeight = height - gantthead.height() - 2 - toolbar.height(), // 2 is border width
                top = ganttbody.ejScroller("option", "scrollTop"),
                left = ganttbody.ejScroller("option", "scrollLeft");
            ganttbody.ejScroller({
                height: viewPortHeight
            });
            ganttHeight = height - toolbar.height();
            splitter.height(ganttHeight);
            $("#ejTreeGrid" + proxy._id).height(ganttHeight);
            $("#ejGanttChart" + proxy._id).height(ganttHeight);
            $(treegridcontent).height(viewPortHeight);
            ganttbody.height(viewPortHeight);
            $(treegridcontent).ejScroller("refresh");
            ganttbody.ejScroller("option", "scrollTop", top);
            ganttbody.ejScroller("option", "scrollLeft", left);
            proxy._$treegridHelper.ejTreeGrid("updateViewPortHeight");
            proxy._$ganttchartHelper.ejGanttChart("updateViewPortHeight");
            if (proxy.model.enableVirtualization) {
                proxy._$treegridHelper.ejTreeGrid("processBindings");
                var tempArgs = {};
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", tempArgs);
                proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedRecords);
                proxy._$ganttchartHelper.ejGanttChart("onScrollHelper", top);
            }
            proxy._clearContextMenu();
        }
        


    });
    
    ej.Gantt.EditingType = {
        String: "stringedit",
        Boolean: "booleanedit",
        Numeric: "numericedit",
        Dropdown: "dropdownedit",
        DatePicker: "datepicker",
        DateTimePicker: "datetimepicker",
        Maskedit: "maskedEdit"
    };

    ej.Gantt.ToolbarItems = {
        Add: "add",
        Edit: "edit",
        Delete: "delete",
        Update: "update",
        Cancel: "cancel",
        Search: "search",
        Indent: "indent",
        Outdent: "outdent",
        ExpandAll: "expandAll",
        CollapseAll: "collapseAll",
        PrevTimeSpan: "prevTimeSpan",
        NextTimeSpan:"nextTimeSpan",
    };

    ej.Gantt.ScheduleHeaderType = {
        Year: "year",
        Month: "month",
        Week: "week",
        Day: "day",
        Hour:"hour"
    };

    ej.Gantt.minutesPerInterval = {
        Auto:"auto",
        OneMinute: "oneMinute",
        FiveMinutes: "fiveMinutes",
        FifteenMinutes: "fifteenMinutes",
        ThirtyMinutes:"thirtyMinutes"
    };

    ej.Gantt.DurationUnit = {
        Day: "day",
        Hour: "hour",
        Minute:"minute"
    };

    ej.Gantt.workingTimeScale = {
        TimeScale8Hours: "TimeScale8Hours",
        TimeScale24Hours: "TimeScale24Hours",
    };

    ej.Gantt.AddRowPosition = {
        Top: "top",
        Bottom: "bottom",
        AboveSelectedRow: "aboveselectedrow",
        BelowSelectedRow: "belowselectedrow",
        Child:"child"
    };

    ej.Gantt.localization = {};

    ej.Gantt.localization["en-US"] = {

        //string to display with dataSource contains 0 objects
        emptyRecord: "No records to display",

        alertTexts: {
            indentAlert: "There is no gantt record is selected to perform the Indent",
            outdentAlert: "There is no gantt record is selected to perform the Outdent",
            predecessorEditingValidationAlert: "Cyclic Dependency Occured, Please Check The Predecessor",
            predecessorAddingValidationAlert: "Fill all the columns in predecessor table",
            idValidationAlert: "Duplicate ID",
            dateValidationAlert: "Invalid End date"
        },

        //headerText to be displayed in treegrid
        columnHeaderTexts: {
            taskId: "ID",
            taskName: "Task Name",
            startDate: "Start Date",
            endDate: "End Date",
            resourceInfo: "Resources",
            duration: "Duration",
            status: "Progress",
            predecessor: "Predecessors",
            type: "Type",
            offset: "Offset",
            baselineStartDate: "Baseline Start Date",
            baselineEndDate: "Baseline End Date"
        },

        //string to display in dialog 
        editDialogTexts: {
            addFormTitle: "New Task",
            editFormTitle: "Edit Task",
            saveButton: "Save",
            deleteButton: "Delete",
            cancelButton: "Cancel",
            addPredecessor: "Add New",
            removePredecessor: "Remove"
        },

        //string to be displayed in Toolbox's tooltip 
        toolboxTooltipTexts: {
            addTool: "Add",
            editTool: "Edit",
            saveTool: "Update",
            deleteTool: "Delete",
            cancelTool: "Cancel",
            searchTool: "Search",
            indentTool: "Indent",
            outdentTool: "Outdent",
            expandAllTool: "ExpandAll",
            collapseAllTool: "CollapseAll",
            nextTimeSpanTool: "Next TimeSpan",
            prevTimeSpanTool: "Previous TimeSpan"
        },

        //string to be displayed in taskbar tooltip for duration unit 
        durationUnitTexts: {           
            days: "days",
            hours: "hours",
            minutes:"minutes"
        },

        //string to be displayed in context menu 
        contextMenuTexts: {
            taskDetailsText: "Task Details...",
            addNewTaskText: "Add New Task",
            indentText: "Indent",
            outdentText: "Outdent",
            deleteText: "Delete",
            aboveText: "Above",
            belowText: "Below"
        },

        //Name to be displayed for New Task while adding  
        newTaskTexts: {           
            newTaskName: "New Task"
        },

        calendars: {
            standard: {
                days: {
                    // full name of days
                    names: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
                    // abbreviated names of days
                    namesAbbr: ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
                },
                months: {
                    // full name of months
                    names: ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
                    // abbreviated name of months
                    namesAbbr: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
                },

                // set of predefined date and time patterns used by the culture.
                patterns: {
                    d: "M/d/yyyy",
                    D: "dddd, MMMM dd, yyyy",
                    F: "dddd, MMMM dd, yyyy h:mm:ss tt",
                    g: "M/d/yyyy h:mm tt",
                    G: "M/d/yyyy h:mm:ss tt",
                    m: "MMMM dd",
                    M: "MMMM dd",
                    s: "yyyy'-'MM'-'ddTHH':'mm':'ss",
                    t: "h:mm tt",
                    T: "h:mm:ss tt",
                    u: "yyyy'-'MM'-'dd HH':'mm':'ss'Z'",
                    y: "MMMM, yyyy",
                    Y: "MMMM, yyyy"
                }
            }
        },

        //string to be displayed in column menu 
        columnMenuTexts: {
            sortAscendingText: "Sort Ascending",
            sortDescendingText: "Sort Descending",
            columnsText: "Columns"
        },
        predecessorEditingTexts: {
            fromText: "From",
            toText : "To"
        }
    };

    /*-----Initialize the GanttRecord object--------------*/

    ej.Gantt.GanttRecord = function() {
        var proxy = this;
        proxy.taskId = null,
        proxy.taskName = null,
        proxy.startDate = null,
        proxy.endDate = null,
        proxy.duration = null,
        proxy.isMilestone = false,
        proxy.status = null,
        proxy.predecessor = null,
        proxy.resourceInfo = null,
        proxy.parentItem = null,
        proxy.isSelected = false,
        proxy.childRecords = null,
        proxy.hasChildRecords = false,
        proxy.expanded = false,
        proxy.level = 0,
        proxy.left = 0,
        proxy.width = 0,
        proxy.progressWidth = 0,
        proxy.item = null,
        proxy.baselineLeft = 0,
        proxy.baselineWidth = 0,
        proxy.baselineStartDate = null,
        proxy.baselineEndDate = null,
        proxy.isReadOnly = false;
        proxy.hasFilteredChildRecords = true;
    }

    ej.Gantt.GanttRecord.prototype = {

        //calculate the left position of the taskbar
        _calculateLeft: function (projectStartDate, dayWidth, holidays, model) {

            var proxy = this,
                diffDate,
                enddatecount = 0,
                beginDate = new Date(proxy.startDate),
                endDate = new Date(proxy.endDate),
                endDateHours = endDate && endDate.getHours(),
                endDateMinutes = endDate && endDate.getMinutes(),
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                headerValue = ej.Gantt.ScheduleHeaderType,
                durationUnitValue = ej.Gantt.DurationUnit,
                duration = 0, timeScale = model.workingTimeScale;

            if (endDate) {
                endDateHours = endDate.getHours();
                endDateMinutes = endDate.getMinutes();
            }
            if (headerType == headerValue.Hour && model.durationUnit == durationUnitValue.Hour) {                
                        var totalHours = beginDate.getHours() + proxy.duration;
                        endDateHours = (parseInt(totalHours % (24)) > 0) ? parseInt(totalHours % (24)) : 0;
                        duration = endDateHours > 0 ? parseInt(proxy.duration / (24)) + 1 : parseInt(proxy.duration / (24));
            }
           else if (model.durationUnit == durationUnitValue.Minute) {
               var totalMinutes = beginDate.getMinutes() + proxy.duration + (beginDate.getHours() * 60);
                endDateMinutes = (parseInt(totalMinutes % (60)) > 0) ? parseInt(totalMinutes % (60)) : 0;
                duration = totalMinutes > 0 ? parseInt(totalMinutes / (24 * 60)) + 1 : parseInt(totalMinutes / (24 * 60));
            }
            else if (model.durationUnit == durationUnitValue.Hour) {
                if (model.durationMapping && !endDate) {
                    if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                        endDateHours = (parseInt(proxy.duration % 24) > 0) ? parseInt(proxy.duration % 24) : 0;
                        duration = endDateHours > 0 ? parseInt(proxy.duration / 24) + 1 : parseInt(proxy.duration / 24);

                    }
                    else if (timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        endDateHours = (parseInt(proxy.duration % 8) > 0) ? parseInt(proxy.duration % 8) : 0;
                        duration = endDateHours > 0 ? parseInt(proxy.duration / 8) + 1 : parseInt(proxy.duration / 8);
                    }
                }
                    //To update Tooltip end date when the task bar is ended in next day.
                else {
                    if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                        duration = endDateHours > 0 ? parseInt((proxy.duration + beginDate.getHours()) / 24) + 1 : parseInt(proxy.duration / 24);
                    else if (timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        if (model.dateFormat.indexOf("hh:") == -1)
                            beginDate.setHours(8);
                        duration = proxy.duration % 8 != 0 ? Math.floor(proxy.duration / 8) + 1 : Math.round(proxy.duration / 8);
                        if (beginDate.getHours() > 15 && model.durationMapping) {
                            remainDate = 24-(24 - beginDate.getHours()) + proxy.duration;
                            if (remainDate > 24)
                                duration += 1;
                        }
                    }
                }
            }
            else {
                duration = parseInt(proxy.duration);
            }

            if (endDate == null) {

                endDate = new Date(proxy.startDate);
                endDate.setDate(endDate.getDate() + duration);
            }


            if (model.includeWeekend == false) {

                if (beginDate.getDay() == 0) {
                    beginDate.setDate(beginDate.getDate() + 1);
                }
                if (beginDate.getDay() == 6) {
                    beginDate.setDate(beginDate.getDate() - 1);
                }

                proxy._updateHoliday(beginDate, holidays, model);

                var emptyDate = new Date(beginDate);
                for (var i = 0; i < duration; i++) {
                    if (emptyDate.getDay() == 0) {
                        enddatecount++;
                    } else if (emptyDate.getDay() == 6) {
                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    }
                    emptyDate.setDate(emptyDate.getDate() + 1);
                }
                endDate = new Date(beginDate);
                enddatecount = duration === 0 ? 1 : enddatecount;
                if (headerType == headerValue.Hour) {
                    if (model.durationUnit == durationUnitValue.Minute)
                        endDate.setMinutes(endDate.getMinutes() + proxy.duration);
                    else
                        endDate.setHours(endDate.getHours() + proxy.duration);                    
                }                    
                else
                    endDate.setDate(endDate.getDate() + (duration + enddatecount - 1));


                if (endDate.getDay() == 0) {
                    endDate.setDate(endDate.getDate() + 1);
                }
                if (endDate.getDay() == 6) {
                    endDate.setDate(endDate.getDate() + 2);
                }
            }

            else if (model.includeWeekend == true && headerType != headerValue.Hour) {
                proxy._updateHoliday(beginDate, holidays, model);
                endDate = new Date(beginDate);
                if (duration > 0) {
                    endDate.setDate(endDate.getDate() + (duration - 1));
                }

            }
            if (model.includeWeekend == false && headerType == headerValue.Hour) {
                if (endDate.getDay() == 0)
                    endDateHours = endDateHours + 24;
                if (endDate.getDay() == 6)
                    endDateHours = endDateHours + 48;                
            }
            if (endDateHours) {
                if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                    model.durationUnit == durationUnitValue.Hour) {
                    if (endDateHours > 0) {                        
                        if (endDateHours>24)
                            endDate.setHours(8 + endDateHours);
                        else
                            endDate.setHours(endDateHours);
                    }
                    else
                        endDate.setHours(17);
                }
                else
                    endDate.setHours(endDateHours);
            }
            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours && model.durationUnit == durationUnitValue.Hour)) {
                endDate.setHours(17);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours && model.dateFormat.indexOf("hh") != -1) ||
                (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours && model.durationUnit == durationUnitValue.Hour)) {
                if (headerType != headerValue.Hour && (headerType != headerValue.Week && model.durationUnit != durationUnitValue.Day))
                endDate.setHours(24);
            }
            if (endDateMinutes > 0) {
                if (model.durationUnit == durationUnitValue.Minute)
                    endDate.setMinutes(endDateMinutes);
            }
            // endDate.setMinutes(endDateMinutes);
            proxy.startDate = new Date(beginDate);
            proxy.endDate = new Date(endDate);

            if (model.endDateMapping)
                proxy.item[model.endDateMapping] = proxy.endDate;

            if (model.startDateMapping)
                proxy.item[model.startDateMapping] = proxy.startDate;
            diffDate = Math.round((beginDate - projectStartDate) / 86400000);

            if ((timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                && model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day) {
                diffDate = ((beginDate - projectStartDate) / 86400000);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours ) &&
                model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                model.dateFormat.indexOf("hh") == -1) {
                diffDate = ((beginDate - projectStartDate) / 86400000);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours || timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                  model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                   model.dateFormat.indexOf("hh") != -1) {

                diffDate = ((beginDate - projectStartDate) / 86400000);
            }

            else if ((timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) && model.roundOffDayworkingTime) {
                diffDate = Math.round((beginDate - projectStartDate) / 86400000);
            }


            return (diffDate * dayWidth);
        },

        //calculate the left margin of the baseLine element.
        _calculateBaselineLeft: function (projectStartDate, dayWidth, includeWeekend, dateformat, model) {

            var diffDate,
                proxy = this,
                beginDate = new Date(proxy.baselineStartDate),
                duration = parseInt(proxy.duration);

            diffDate = (beginDate - projectStartDate) / 86400000;
            if (model.durationUnit != ej.Gantt.DurationUnit.Minute)
                diffDate = Math.ceil(diffDate);
            return (diffDate * dayWidth);
        },

        _updateHoliday: function (date, holidays, model) {
            var j = 0,
                length = holidays && holidays.length;

            if (length > 0) {

                for (j = 0; j < length; j++) {
                    var tempDate = new Date(date);

                    tempDate.setHours(0, 0, 0, 0);

                    if (this._getFormatedDate(holidays[j], model.dateFormat) == this._getFormatedDate(date, model.dateFormat)) {
                        date.setDate(date.getDate() + 1);
                    }
                }
                if ((model.includeWeekend == false) && (date.getDay() == 6)) {
                    date.setDate(date.getDate() + 2);
                }
            }
        },

        _calculateGanttRecordLeft: function (startDate, projectStartDate, dayWidth) {
            var proxy = this,
                diffDate = proxy._daydiff(projectStartDate, startDate);
            return (diffDate * dayWidth);
        },

        _calculateReverseLeft: function (projectStartDate, dayWidth, includeWeekend, dateformat) {
            var diffDate,
                proxy = this,
                enddatecount = 0,
                startdate = proxy.startDate,
                duration = proxy.duration && parseInt(proxy.duration);

            if (startdate == null) {

                startdate = new Date(proxy.endDate);
                startdate.setDate(startdate.getDate() - duration);
            }
            if (includeWeekend == false) {

                var emptyDate = new Date(proxy.endDate);

                for (var i = 0; i < duration; i++) {

                    if (emptyDate.getDay() == 0) {
                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() - 2);
                    }
                    else if (emptyDate.getDay() == 6) {
                        enddatecount++;
                    }
                    emptyDate.setDate(emptyDate.getDate() - 1);
                }
                startdate = new Date(proxy.endDate);
                enddatecount = duration === 0 ? 1 : enddatecount;
                startdate.setDate(startdate.getDate() - (duration + enddatecount - 1));
            }
            proxy.startDate = new Date(startdate);
            diffDate = Math.ceil((startdate - projectStartDate) / 86400000);
            return (diffDate * dayWidth);
        },

        _calculateReverseWidth: function (dayWidth, dateFormat, includeWeekend, holidays) {

            var proxy = this,
                enddatecount = 0,
                diffDate,
                emptyDate,
                i = 0,
                startdate = proxy.startDate,
                duration = proxy.duration && parseInt(proxy.duration),
                enddate = proxy.endDate;

            if (startdate == null) {
                startdate = new Date(enddate);
                startdate.setDate(startdate.getDate() - duration + 1);

            }

            if (includeWeekend == false) {

                emptyDate = new Date(enddate);

                for (i = 0; i < duration; i++) {

                    if (emptyDate.getDay() == 0) {

                        enddatecount += 2;
                        emptyDate.setDate(emptyDate.getDate() - 2);

                    } else if (emptyDate.getDay() == 6) {
                        enddatecount++;
                    }

                    emptyDate.setDate(emptyDate.getDate() - 1);

                }

                startdate = new Date(enddate);
                startdate.setDate(startdate.getDate() - (duration + enddatecount - 1));
            }

            if (holidays != null && holidays.length > 0) {

                emptyDate = new Date(startdate);

                for (i = 0; i < duration; i++) {

                    emptyDate.setDate(emptyDate.getDate() + 1);

                    for (var j = 0; j < holidays.length; j++) {

                        if (proxy._getDateFromFormat(holidays[j].day).getDate() == emptyDate.getDate()) {
                            enddatecount++;
                        }

                    }

                    if ((includeWeekend == false) && (emptyDate.getDay() == 0 || emptyDate.getDay() == 6)) {
                        enddatecount++;
                    }

                }

                enddate = new Date(proxy.startDate);
                enddate.setDate(enddate.getDate() + (duration + enddatecount - 1));
            }

            diffDate = Math.ceil((new Date(enddate) - (proxy.startDate)) / 86400000) + 1;

            return (diffDate) * dayWidth;
        },

        _calculateWidth: function (dayWidth, holidays, model) {

            var proxy = this,
                diffDate,
                startdate =new Date(proxy.startDate),
                enddate = new Date(proxy.endDate),
                duration = proxy.duration;

            if (enddate == null) {
                duration = parseInt(duration);
                enddate = new Date(startdate);
                enddate.setDate(enddate.getDate() + duration - 1);
            }

            proxy._calculateEndDate(model.includeWeekend, holidays, model, dayWidth);

            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                    //To Get proper width of child Gantt Record when include weekend is false in Day-Hour Schedule mode.
                    if (model.includeWeekend == false && model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Hour) {
                        if (startdate.getDay() == 0)
                            startdate.setDate(startdate.getDate() + 1);
                        else if (startdate.getDay() == 6)
                            startdate.setdate(startdate.getDate() + 2);
                    }
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = proxy.duration;
                    }
                    else {
                        diffDate = proxy._hourdiff(new Date(startdate),new Date(proxy.endDate));
                    }
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        if (model.durationMapping) {
                            startdate.setHours(8);
                            enddate.setHours(8 + proxy.duration);
                            diffDate = proxy._hourdiff(startdate, enddate);
                        }
                        else
                            diffDate = (proxy.duration / 8) * 24 + (proxy.duration % 8) - 15;                                          
                    }
                    else {
                        diffDate = proxy._hourdiff(new Date(proxy.startDate),new Date(proxy.endDate));
                    }
                }
                return (diffDate * (dayWidth / 24));
            }
            else if (model.durationUnit == ej.Gantt.DurationUnit.Minute) {
                startdate = new Date(startdate);
                enddate = new Date(enddate);
                if (model.endDateMapping)
                    proxy._calculateDuration(startdate, enddate, model.includeWeekend, model.holidays, proxy.isMilestone, model);
                diffDate = proxy.duration;
                return (diffDate * (dayWidth / (24*60)));
            }
            else {
                if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                      && model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day &&
                    (model.dateFormat.indexOf("hh:") != -1)) {
                    diffDate = ((new Date(proxy.endDate) - new Date(proxy.startDate)) / 86400000);// + 1;//Math.ceil
                }

                else if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours)
                  && model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day &&
                   (model.dateFormat.indexOf("hh:") == -1)) {
                    diffDate = ((new Date(proxy.endDate) - new Date(proxy.startDate)) / 86400000) + 1;//Math.ceil
                }
                    //Calculating width for 8 hour timescale  for dateformats without time
                else if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                    model.durationUnit != ej.Gantt.DurationUnit.Hour) {

                    var tStartDate = new Date(proxy.startDate),
                        tEndDate = new Date(proxy.endDate);
                    tStartDate.setHours(8);
                    tEndDate.setHours(17);

                    diffDate = ((tEndDate - tStartDate) / 86400000);
                }

                else if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                         model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day &&
                         model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                    var tStartDate = new Date(proxy.startDate),
                       tEndDate = new Date(proxy.endDate);

                    diffDate = ((tEndDate - tStartDate) / 86400000);
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours && (model.dateFormat.indexOf("hh:") != -1)
                    && model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week)
                    diffDate = Math.round((new Date(proxy.endDate) - new Date(proxy.startDate)) / 86400000);

                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours &&
                   (model.dateFormat.indexOf("hh:") == -1) || model.roundOffDayworkingTime)
                    diffDate = Math.round((new Date(proxy.endDate) - new Date(proxy.startDate)) / 86400000) + 1;

                return (diffDate) * dayWidth;
            }
        },


        _calculateTaskbarWidth: function (dayWidth, model) {
            var proxy = this,
                diffDate = 0,
                dateFormat = model.dateFormat,
                durationUnitValue = ej.Gantt.DurationUnit,
                timeScale = model.workingTimeScale;
            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Hour) {
                diffDate = proxy.duration;
                if (model.durationUnit == durationUnitValue.Minute)
                return (diffDate * (dayWidth / (24 * 60)));
            else
                return(diffDate*(dayWidth/24));
        }
            else if (model.durationUnit == durationUnitValue.Hour) {
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {

                    if (model.durationUnit == durationUnitValue.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = proxy.duration;
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy.startDate, proxy.endDate);
                    }
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    if (model.durationUnit == durationUnitValue.Hour && model.dateFormat.indexOf("hh") == -1) {
                        var startdate = new Date(proxy.startDate),
                            enddate = new Date(proxy.endDate);
                        startdate.setHours(8);
                        if (model.durationMapping)
                            enddate.setHours(8 + proxy.duration);
                        else
                            enddate.setHours(16);
                        diffDate = proxy._hourdiff(startdate, enddate);
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy.startDate, proxy.endDate);
                    }
                }
                return (diffDate * (dayWidth / 24));
            }
            else {
                if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours && model.durationUnit == durationUnitValue.Day &&
                    model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day && model.dateFormat.indexOf("hh") == -1) {
                    diffDate = proxy._daydiff(proxy.startDate, proxy.endDate) + 1;
                }

                else if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                    diffDate = proxy._daydiff(proxy.startDate, proxy.endDate);
                }//+ 1;

                else if (timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    diffDate = Math.round(proxy._daydiff(proxy.startDate, proxy.endDate, dateFormat) + 1);
                }
                else {
                    diffDate = proxy._daydiff(proxy.startDate, proxy.endDate, dateFormat);//+ 1;
                }
                return (diffDate * dayWidth);
            }
        },
        _calculateBaseLineWidth: function (dayWidth, model)
        {
            var proxy = this,
                diffDate = 0,
                dateFormat = model.dateFormat,
                timeScale = model.workingTimeScale;

            if (model.durationUnit == ej.Gantt.DurationUnit.Hour) {
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {

                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = proxy.duration;
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy.baselineStartDate, proxy.baselineEndDate);
                    }
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    if (model.durationUnit == ej.Gantt.DurationUnit.Hour && model.dateFormat.indexOf("hh") == -1) {
                        diffDate = proxy.duration;
                    }
                    else {
                        diffDate = proxy._hourdiff(proxy.baselineStartDate, proxy.baselineEndDate);
                    }
                }
                return (diffDate * (dayWidth / 24));
            }
            else {
                if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours &&
                    model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day && model.dateFormat.indexOf("hh") == -1) {
                    diffDate = proxy._daydiff(proxy.baselineStartDate, proxy.baselineEndDate) + 1;
                }

                else if (timeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                    diffDate = proxy._daydiff(proxy.baselineStartDate, proxy.baselineEndDate);
                }//+ 1;

                else if (timeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    diffDate = Math.round(proxy._daydiff(proxy.baselineStartDate, proxy.baselineEndDate) + 1);
                }
                else {
                    diffDate = proxy._daydiff(proxy.baselineStartDate, proxy.baselineEndDate);//+ 1;
                }
                return (diffDate * dayWidth);
            }
        },
        _getFormatedDate: function (dateObject, dateformat) {
            return Globalize.format(dateObject, dateformat);
        },

        _getDateFromFormat: function (dateString, dateformat) {
            if (dateString) {
                return Globalize.parseDate(dateString, dateformat) == null ?
                    new Date(dateString) : Globalize.parseDate(dateString, dateformat);
            }
        },

        _calculateProgressWidth: function (parentWidth, percent) {
            return (parentWidth * percent) / 100;
        },

        _setResourceInfo: function (resourceIdcollection, resourceIdMapping, resourceNameMapping, resourcecollection) {

            var count = 0,
            length = resourceIdcollection.length,
            resources = [];

            for (count; count < length; count++) {

                var resource = resourcecollection.filter(function (resourceInfo) {
                    return resourceIdcollection[count] === resourceInfo[resourceIdMapping];
                });

                resource.length && resources.push(resource[0]);
            }
            return resources;
        },

        _updateEndDate: function (prevStartDate, includeWeekend, dateFormat) {
            var proxy = this,
                formatedPrevStartDate = new Date(prevStartDate),
                formatedStartDate = new Date(proxy.startDate),
                diff = proxy._daydiff(formatedPrevStartDate, formatedStartDate),
            endDate = new Date(proxy.endDate),
            weekendCount = 0;

            if (includeWeekend == false) {
                var count = 0;
                for (count; count <= diff; count++) {
                    endDate.setDate(endDate.getDate() + 1);
                    if (endDate.getDay() == 0 || endDate.getDay() == 6)
                        weekendCount += 1;
                }
            }

            var modifiedEndDate = proxy.endDate;
            modifiedEndDate.setDate(modifiedEndDate.getDate() + diff + weekendCount);
            proxy.endDate = new Date(modifiedEndDate);
        },

        _calculateFormatedDate: function (date, format) {
            return Globalize.format(date, format);
        },

        _updateParentProgress: function (parent,progressMapping) {
            var parentProgress = 0,
                childCount = parent.childRecords && parent.childRecords.length,
                totalDuration = 0,
                completedDuration = 0;


            if (parent.childRecords) {
                for (var i = 0; i < childCount; i++) {

                    //converting days into hours .
                    //By now 24 hours is default
                    totalDuration += (parent.childRecords[i].duration * 24);

                    //summation of progress done of child
                    completedDuration += ((parent.childRecords[i].duration * 24) * parent.childRecords[i].status) / 100;
                  
                }

                parentProgress = (completedDuration / totalDuration) * 100;
                if (isNaN(parentProgress))
                    parentProgress = 0;
                parent.progressWidth = this._getProgressWidth(parent.width, parentProgress);
                parent.status = Math.round(parentProgress);
                if (progressMapping)
                    parent.item[progressMapping] = parent.status;

            }
            if (parent.parentItem) {
                this._updateParentProgress(parent.parentItem, progressMapping);
            }
        },


        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },

        _calculatePredecessor: function (string) {
            var collection = [],
                match,
                obj = {},
                values,
                to = this.taskId.toString(),
                offsetvalue;
            string.split(',').forEach(function (el) {
                values = el.split('+');
                offsetvalue = '+';
                if (el.indexOf('-') >= 0) {
                    values = el.split('-');
                    offsetvalue = '-';
                }
                match = values[0].match(/(\d+|[A-z]+)/g);
                obj = {
                    from: match[0],
                    predecessorsType: match.length > 1 ? match[1].toUpperCase() : "FS",
                    offset: values.length > 1 ? offsetvalue + "" + values[1] : "",
                    isdrawn: false,
                    to: to
                };
                collection.push(obj);
            });
            return collection;
        },

        _updateGanttRecord: function (startDateMappingName, endDateMappingName, durationMappingName, includeWeekend, scheduleStartDay, perDayWidth, dateFormat, holidays) {
            var proxy = this,
                remainDays = proxy.left % perDayWidth,
                day = (proxy.left - remainDays) / perDayWidth,holiday
                startDate = new Date(scheduleStartDay);

            startDate.setDate(scheduleStartDay.getDate() + day);

            if (startDate.getDay() == 0)
                startDate.setDate(startDate.getDate() + 1);
            else if (startDate.getDay() == 6)
                startDate.setDate(startDate.getDate() + 2);

            proxy.startDate = new Date(startDate);
            proxy.left = proxy._calculateLeft(scheduleStartDay, perDayWidth, includeWeekend, dateFormat, endDateMappingName);
            proxy.width = proxy._calculateWidth(perDayWidth, includeWeekend, holidays, dateFormat);
            if (proxy.parentItem)
                proxy._updateParentItem(proxy, startDateMappingName, endDateMappingName, durationMappingName, includeWeekend, scheduleStartDay, perDayWidth, dateFormat, holidays);
        },

        _calculateUpdatedDate: function (modifiedDate, offset) {
            var proxy = this,
                i = 0;
            if (offset > 0) {
                for (i; i < offset; i++) {
                    modifiedDate = proxy._dataSubtract(modifiedDate, 1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() + 1);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() + 2);
                }
            } else {
                var dateOffset = (-1) * offset;
                for (i = 0; i < dateOffset; i++) {
                    modifiedDate = proxy._dataSubtract(modifiedDate, -1);
                    if (modifiedDate.getDay() == 0)
                        modifiedDate.setDate(modifiedDate.getDate() - 2);
                    else if (modifiedDate.getDay() == 6)
                        modifiedDate.setDate(modifiedDate.getDate() - 1);
                }
            }
            modifiedDate = new Date(modifiedDate);
            return modifiedDate;
        },

        _dataSubtract: function (date, days) {
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours(),
                date.getMinutes(),
                date.getSeconds(),
                date.getMilliseconds()
            );
        },

        _calculateEndDate: function (includeWeekend, holidays, model, perDayWidth, editMode) {
            var proxy = this,
                count = 0,
                dateFormat = model.dateFormat,
                startDate = new Date(proxy.startDate),
                endDate = new Date(proxy.endDate),
                endDateHours = 0, endDateMinutes = 0, day = 0,
                duration = 0,// proxy.duration && parseInt(proxy.duration),
                remainingHours = 0, totalLength,
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                headerValue = ej.Gantt.ScheduleHeaderType,
                durationUnitValue = ej.Gantt.DurationUnit,
                remainingLength;

            if (ej.isNullOrUndefined(editMode))
                editMode = "";
            if (endDate) {
                endDateHours = endDate.getHours();
                endDateMinutes = endDate.getMinutes();
            }
            if (headerType == headerValue.Hour && model.durationUnit == durationUnitValue.Hour) {                              
                    var totalMinutes = startDate.getMinutes() + proxy.duration;                   
                    if (!model.endDateMapping || editMode == "dragging") {
                        var totalHours = parseInt(startDate.getHours() + Math.round(proxy.duration));                      
                        endDateHours = (parseInt(totalHours % (24)) > 0) ? parseInt(totalHours % (24)) : 0;                       
                        endDay = parseInt(totalHours / 24);                        
                    }
                    else {
                        if (endDate) {
                            endDateHours = endDate.getHours();
                            endDateMinutes = endDate.getMinutes();
                        }
                        else {
                            endDateHours= (parseInt(totalMinutes % 24) > 0) ? parseInt(totalMinutes % 24) : 0;
                        }
                    }                                          
                        duration = endDateHours > 0 ? parseInt((totalHours) / 24) + 1 :
                                parseInt(totalHours) / 24;                                            
                }
            else if (model.durationUnit == durationUnitValue.Hour) {
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                    if (!model.endDateMapping || editMode == "dragging") {
                        var totalHours = startDate.getHours() + proxy.duration;
                        endDateHours = (parseInt(totalHours % 24) > 0) ? parseInt(totalHours % 24) : 0;
                    } else {
                        if (endDate) {
                            endDateHours = endDate.getHours();
                            endDateMinutes = endDate.getMinutes();
                        } else {
                            endDateHours = (parseInt(proxy.duration % 24) > 0) ? parseInt(proxy.duration % 24) : 0;
                        }
                    }
                    duration = endDateHours > 0 ? parseInt((startDate.getHours() + proxy.duration) / 24) + 1 :
                        parseInt(startDate.getHours() + proxy.duration) / 24;
                   
                }
                else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    if (!model.endDateMapping || editMode == "dragging") {
                        var totalHours = startDate.getHours() + proxy.duration,                           
                        endDateHours = (parseInt(totalHours % 8) > 0) ? parseInt(totalHours % 8) : 0;
                        endDateHours = proxy._calculateTimeScale8Hour(startDate.getHours(), endDateHours) % 24;
                        if (editMode == "dragging" && totalHours > 16 && model.durationMapping)
                            endDateHours += 16;
                    }
                    else {
                        if (endDate) {
                            endDateHours = endDate.getHours();
                            if (model.dateFormat.indexOf("hh:") == -1)
                                endDateHours = 16;
                            endDateMinutes = endDate.getMinutes();
                        } else {
                            endDateHours = proxy._calculateTimeScale8Hour(startDate.getHours(), proxy.duration) % 24;
                        }
                    }
                    var endHour = proxy._calculateTimeScale8Hour(startDate.getHours(), proxy.duration);                   
                    var checkEndDate = proxy._dateUpdate(startDate, 0, (endHour-startDate.getHours()));
                    var calculatedEndHour = proxy.duration % 8 > 0 ? Math.floor(proxy.duration / 8) + 1 : Math.round(proxy.duration / 8);                    
                         
                     if ((24 - startDate.getHours()) < 8 && model.durationMapping)
                         calculatedEndHour += 1;                     
                    duration = endDateHours > 0 ? Math.round(calculatedEndHour)
                        : parseInt(proxy.duration) / 8;

                }
                 
                //duration = parseInt(proxy.duration);
            }
           else if (model.durationUnit == durationUnitValue.Minute) {
               var totalMinutes = startDate.getMinutes() + proxy.duration + (startDate.getHours() * 60);
                if (!model.endDateMapping || editMode == "dragging" || editMode == "Resizing") {
                    var totalMinutes = startDate.getMinutes() + proxy.duration;
                    endDateMinutes = (parseInt(totalMinutes % (60)) > 0) ? parseInt(totalMinutes % (60)) : 0;
                    endDateHours = startDate.getHours() + (parseInt(totalMinutes / 60));
                }
                else {
                    if (endDate) {
                        endDateHours = endDate.getHours();
                        endDateMinutes = endDate.getMinutes();
                    }
                    else {
                        endDateMinutes = (parseInt(totalMinutes % 60) > 0) ? parseInt(totalMinutes % 60) : 0;
                    }
                }
                var reduceDuration = parseInt((startDate.getMinutes() + proxy.duration) / (60 * 24));
                duration = totalMinutes >= 0 ? parseInt((totalMinutes) / (60 * 24)) + 1 :
                        parseInt(totalMinutes) / (60 * 24);
                if (model.durationMapping && model.endDateMapping && editMode != "dragging")
                    duration = duration;
                else if (model.durationMapping || editMode == "dragging" || editMode == "Resizing")
                    duration = duration - reduceDuration;
            }
            else {
                duration = parseInt(proxy.duration);
            }

            
            if (proxy.width > 0) {
                totalLength = proxy.left + proxy.width;
                remainingLength = totalLength % perDayWidth;
                remainingHours = (remainingLength / (perDayWidth / 24));//hour calculation
            }

            if (duration === 0) {
                proxy.endDate = proxy.startDate;
                return;
            }

            if (includeWeekend === false && headerType != headerValue.Hour) {
                for (count = 1; count < duration; count++) {
                    if (startDate.getDay() == 0)
                        startDate.setDate(startDate.getDate() + 1);
                    else if (startDate.getDay() == 6)
                        startDate.setDate(startDate.getDate() + 2);

                    startDate.setDate(startDate.getDate() + 1);
                }
            } else {
                startDate.setDate(startDate.getDate() + duration - 1);
            }

            if (includeWeekend === false && headerType != headerValue.Hour) {
                if (startDate.getDay() == 0)
                    startDate.setDate(startDate.getDate() + 1);
                else if (startDate.getDay() == 6)
                    startDate.setDate(startDate.getDate() + 2);
            }

            if (holidays && holidays.length) {
                var holidayIndex = 0,
                    holidayslength = holidays.length,
                    holiday;
                for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                    holiday = proxy._getDateFromFormat(holidays[holidayIndex]);
                    if (holiday >= proxy.startDate && holiday <= startDate) {
                        startDate.setDate(startDate.getDate() + 1);
                        if (includeWeekend === false) {
                            if (startDate.getDay() == 0) {
                                startDate.setDate(startDate.getDate() + 1);
                            } else if (startDate.getDay() == 6) {
                                startDate.setDate(startDate.getDate() + 2);
                            }
                        }
                    }
                }
            }

            if (includeWeekend === false && headerType != headerValue.Hour) {
                if (startDate.getDay() == 0)
                    startDate.setDate(startDate.getDate() + 1);
                else if (startDate.getDay() == 6)
                    startDate.setDate(startDate.getDate() + 2);
            }
            if (remainingHours > 0 && model.durationUnit != "minute")//To restrict the over updation of date during dragging.
                startDate.setHours(remainingHours);//endDateHours
            if (endDateHours >= 0) {
                if ((model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) &&
                    model.durationUnit == durationUnitValue.Hour) {
                    startDate.setHours(endDateHours);
                }
                else
                    startDate.setHours(endDateHours);
            }
            if (model.durationUnit == durationUnitValue.Minute)
                 startDate.setMinutes(endDateMinutes);          
            
            if (model.dateFormat.indexOf("hh:") !== -1 &&
                      model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours &&
                model.scheduleHeaderSettings.scheduleHeaderType != ej.Gantt.ScheduleHeaderType.Day) {
                startDate.setHours(17);
            }
            else if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours
                && model.durationUnit == durationUnitValue.Hour) {
                if (endDateHours > 0) {
                    startDate.setHours(endDateHours);
                }
                else
                    startDate.setHours(24);
            }

            proxy.endDate = startDate;

            if (model.endDateMapping)
                proxy.item[model.endDateMapping] = proxy.endDate;
        },

        _calculateTimeScale8Hour: function (startHour, duration) {
              if ((24 - startHour) > 8) {           
                    if (this.duration < 8)
                        duration = this.duration;
                    else
                        duration = this.duration % 8;             
            }
            if ((24 - startHour) < 8 || this.duration > 8)
                startHour = 8;
            if (duration == 0)
                duration = 8;
            var endHour = startHour;

            do {
                if ((startHour >= 8 && startHour < 13) || (startHour > 13 && startHour <= 17)) {
                    endHour++;
                    duration--;
                    startHour = startHour == 24 ? 0 : startHour + 1;
                }
                else {
                    startHour = startHour == 24 ? 0 : startHour + 1;
                    
                    endHour++;
                    duration--;
                }
            } while (duration > 0);

            return endHour;
        },

        _dateUpdate: function (date, days, hours, minutes) {
            if (!minutes)
                minutes = 0;           
            return new Date(
                date.getFullYear(),
                date.getMonth(),
                date.getDate() + days,
                date.getHours() + hours,
                date.getMinutes() + minutes,
                date.getSeconds(),
                date.getMilliseconds()
            );
        },


        _calculateDuration: function (startDate, endDate, includeWeekend, holidays, milestone, model) {
            var proxy = this,
                dateDiff,
                durationUnitValue = ej.Gantt.DurationUnit,
                headerType = model.scheduleHeaderSettings.scheduleHeaderType,
                headerValue = ej.Gantt.ScheduleHeaderType;
            if (model.durationUnit == durationUnitValue.Hour) {
                if (model.dateFormat.indexOf("hh:") == -1 && model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                    startDate.setHours(8);
                    endDate.setHours(16);
                }
                var dayDiff =((endDate - startDate) / (1000 * 60 * 60 * 24));
                dateDiff = proxy._hourdiff(startDate, endDate);               
                if (dateDiff < 0) {
                    //dateDiff = dateDiff * -1;
                }
                if (model.endDateMapping)
                    dayDiff = dayDiff < 1 ? 0.5 : dayDiff;
                if (model.durationMapping)  
                    dayDiff = Math.round(dayDiff);                
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours){
                    var remainDiff = startDate.getHours() - 8;                    
                    if (startDate.getHours() == 0)
                        remainDiff += 8;
                    if (dayDiff >= 1)
                        proxy.duration = (dateDiff) - (dayDiff * 16) + remainDiff;
                        else
                            proxy.duration = dateDiff;
                    if (startDate.getHours() > 16) {
                        if (model.durationMapping)
                            proxy.duration -= (8 - (24 - startDate.getHours()));
                        if (model.endDateMapping)
                            proxy.duration += (8 - (24 - startDate.getHours()));
                    }
                        proxy.duration = Math.round(proxy.duration);
                        return proxy.duration;
                    }
                }            
            else if (model.durationUnit == durationUnitValue.Minute) {
                dateDiff = proxy._minutediff(startDate, endDate);               
            }
            else {
                //To restrict the over updation of parent record duration in date-time date format.
                if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours)
                    endDate.setHours(0);
                dateDiff = proxy._daydiff(startDate, endDate);
            }

            if (includeWeekend === false && model.durationUnit != durationUnitValue.Minute && model.durationUnit != durationUnitValue.Hour)
                offset = proxy._calculateDateDifference(new Date(startDate), dateDiff);
            else {
                if (model.durationUnit != durationUnitValue.Hour && model.durationUnit != durationUnitValue.Minute)
                    offset = dateDiff + 1;
                else
                    offset = dateDiff;
            }

            for (var count = 0; count < holidays.length; count++) {
                if (proxy._getDateFromFormat(holidays[count].day) >= startDate && proxy._getDateFromFormat(holidays[count].day) <= endDate) {
                    if (model.durationUnit == durationUnitValue.Hour)
                        offset -= 24;
                    else
                        offset--;
                }
            }

            if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours ||
                model.durationUnit == durationUnitValue.Hour) {

                var decimal = (offset - parseInt(offset, 10));
                decimal = (decimal * 10);
                if (decimal == 5) { offset = (parseInt(offset, 10) + 0.5); }

                else {
                    offset = Math.round(offset);
                }
            }
            //else if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day) {

            //    offset = Math.round(offset) - 1;
            //}
            else
                offset = Math.round(offset);

            if (ej.isNullOrUndefined(milestone))
                proxy.duration = offset;
            else
                proxy.duration = (offset == 0 && milestone == false) ? 1 : offset;
        },

        _calculateDateDifference: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                offset = 0;

            if (dateDiff > 0)
                offset = differenceCount - this._calculateWeekendCount(emptyDate, dateDiff) + 1;
            else
                offset = dateDiff + this._calculateWeekendCount(emptyDate, dateDiff);
            return offset;
        },

        _calculateWeekendCount: function (emptyDate, dateDiff) {
            var differenceCount = dateDiff >= 0 ? dateDiff : (-1 * dateDiff),
                weekendCount = 0,
                i = 0;

            if (dateDiff > 0) {
                for (i = 0; i < dateDiff; i++) {
                    if (emptyDate.getDay() == 0) {
                        weekendCount++;
                        emptyDate.setDate(emptyDate.getDate() + 1);
                    }
                    else if (emptyDate.getDay() == 6) {
                        weekendCount += 2;
                        ++i;
                        emptyDate.setDate(emptyDate.getDate() + 2);
                    } else
                        emptyDate.setDate(emptyDate.getDate() + 1);
                }
            }
            else {
                for (i = 0; i < differenceCount; i++) {
                    emptyDate.setDate(emptyDate.getDate() - 1);
                    if (emptyDate.getDay() == 0 || emptyDate.getDay() == 6) {
                        weekendCount += 1;
                    }
                }
            }
            return weekendCount;
        },
        //Get day difference from two dates time
        _daydiff: function (first, second) {
            return ((second - first) / (1000 * 60 * 60 * 24));
        },
         //Get hour difference from two dates time      
        _hourdiff: function (first, second) {
            //if(first.getTime()>second.getTime())
            //    return ((first - second) / (1000 * 60 * 60));
            //else
                return ((second - first) / (1000 * 60 * 60));
        },
        //Get progress value from taskbar width and progrssbar width
        _getProgressPercent: function (parentwidth, progresswidth) {
            return Math.ceil(((progresswidth / parentwidth) * 100).toFixed(2));
        },
        //Get minute difference from two dates time 
        _minutediff: function (first, second) {
            return ((second - first) / (1000 * 60));
        },
    };
    //Get predecessor value
    ej.Gantt._getPredecessorsValue = function (mappingName) {
        return this.data.item ? this.data.item[mappingName] : "";
    };    
    //Returns cell value
    ej.Gantt._getCellValue = function (columnName) {

        var cellValue = this.data[columnName];

        if (cellValue) {
            return cellValue;
        } else {
            return this.data.item && this.data.item[columnName];
        }


    };

})(jQuery, Syncfusion);;
/**
 * @fileOverview Plugin to style the Html TreeGrid elements
 * @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
 *  Use of this code is subject to the terms of our license.
 *  A copy of the current license can be obtained at any time by e-mailing
 *  licensing@syncfusion.com. Any infringement will be prosecuted under
 *  applicable laws. 
 * @version 12.1 
 * @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
 */

(function ($, ej, undefined) {
   
    ej.widget("ejTreeGrid", "ej.TreeGrid", {
        // widget element will be automatically set in this
        _rootCSS: "e-treegrid",

        // widget element will be automatically set in this
        element: null,

        validTags: ["div"],

        // user defined model will be automatically set in this
        model: null,

        _tags: [
        {
            tag: "columns",
            attr: ["field", "headerText", "allowSorting", "editType", "allowFiltering", "filterEditType","width", "visible", "editParams", "format"],
            content: "template"
        },
        {
            tag: "summaryRows",
            attr: ["title", "summaryColumns"],
            content: "template"
        },
         {
             tag: "summaryRows.summaryColumns",
             attr: ["summaryType", "dataMember", "displayColumn", "prefix", "suffix", "format"],
             content: "template"
         },
        {
            tag: "sortSettings.sortedColumns",
            attr: ["field", "direction"]
        },
        {
            tag: "sizeSettings",
            attr:["height","width"]
        }
        ],
        _columns: function (index, property, value, old){
            var $header = this.element.find(".e-headercelldiv");
            $header[index.columns].innerHTML=value;
            this._trigger("refresh");
        },
        keyConfigs: {
            focus: "e",
            editRecord: "113", //F2
            saveRequest: "13", // enter
            cancelRequest: "27", //Esc         
            firstRowSelection: "36", //"CtrlPlusHome",
            lastRowSelection: "35", //"CtrlPlusEnd",
            leftArrow:"37",//Left arrow
            upArrow: "38", //Up arrow
            rightArrow:"39",//Right arrow
            downArrow: "40", //Down arrow
            moveCellRight: "9", //tab
            moveCellLeft: "shift+9", //shifttab
            selectedRowExpand: "alt+40", //"AltPlusDownArrow",
            totalRowExpand: "ctrl+40", //"CtrlPlusDownArrow",
            selectedRowCollapse: "alt+38", //"AltPlusUpArrow",
            totalRowCollapse: "ctrl+38", //"CtrlPlusUpArrow"
            deleteRecord: "46", // delete
            spaceBar: "32" // spacebar
        },
      
        defaults: {
            
            allowFiltering: false,            
            allowDragAndDrop: false,            
            dragTooltip: {                
                showTooltip: false,                
                tooltipItems:[],                
                tooltipTemplate: null,
            },  
            allowSorting: false,            
            allowColumnResize: false, 
            allowSelection: true,   
            dataSource: null,
            query: ej.Query(),            
            idMapping: "",            
            parentIdMapping: "",  
            showGridCellTooltip: false,
            showGridExpandCellTooltip: false,
            showColumnChooser: false,
            enableAltRow: true,			
            selectedItem: null,            
            toolbarSettings: {                
                showToolbar: false,                
                toolbarItems: []
            },         
            editSettings: {       
                allowEditing: false,  
                allowAdding: true,
                allowDeleting: true,                
                editMode: "cellEditing",                
                rowPosition: "top",
            },                   
            enableVirtualization: false,   
            allowMultiSorting: false,
            sortSettings: {                
                sortedColumns: []
            },
            filterSettings: {
                filterBarMode: "immediate",
                filteredColumns: []
            },
            selectionType: "single",    
            selectedRowIndex: -1,
            showSummaryRow: false,
            summaryRows: [],            
            showTotalSummary: false,    
            allowKeyboardNavigation: true,
            cssClass: "",           
            locale: "en-US",
            isEdit: false,
            allowScrolling: false,
            groupSettings: {
                groupedColumns:[]
            },            
            sizeSettings: {                
                height: null,                
                width: null,
            },            
            enableResize: false,            
            showDetailsRow: false,            
            showDetailsRowInfoColumn: false,            
            detailsTemplate: null,            
            detailsRowHeight: 100,    
            rowDataBound: null,                        
            load: null,            
            queryCellInfo: null,            
            rowSelecting: null,           
            rowSelected: null,           
            beginEdit: null,            
            endEdit: null,            
            expanding: null,            
            expanded: null,            
            collapsing: null,            
            collapsed: null,            
            actionComplete: null,            
            actionBegin: null,
            currentViewData: [],
            flatRecords: [],
            parentRecords: [],
            updatedRecords:[],
            summaryRowRecords: [],
            ids: [],      
            columns: [
            {
                field: "",
                headerText: "",
                editType: "",
                filterEditType: "",
                allowFiltering: false,
                allowFilteringBlankContent: true,
                allowSorting: false,
                    visible: "",

                    textAlign: ej.TextAlign.Left,
                    headerTextAlign: ej.TextAlign.Left
                },
            ],
            commonWidth: 150,
            dateFormat: "MM/dd/yyyy",   
            rowHeight: 30,
            emptyRecordText: "No Records To Display",            
            treeColumnIndex: 0,
            //API for toggle between datepicker and datetimepicker
            workingTimeScale: 'TimeScale8Hours',            
            childMapping: "",             
            enableCollapseAll:false,
            rowTemplateID: "",
            altRowTemplateID: "",            
            contextMenuSettings: {                
                showContextMenu: false,                
                contextMenuItems: []
            },            
            contextMenuOpen: null,            
            rowDragStart: null,            
            rowDrag: null,            
            rowDragStop: null,       
            detailsDataBound: null,            
            detailsShown: null,            
            detailsHidden: null,
            toolbarClick: null,
            exportToPdfAction: "",
            allowMultipleExporting: false
        },

        dataTypes: {
            columns: "array",
            summaryRows: "array",
            summaryRows: {
                summaryColumns: "array"
            },
            sortSettings: {
                sortedColumns: "array"
            },
            filterSettings: {
                filteredColumns: "array"
            },
            dataSource: "data",
            query: "data",
            currentViewData: "array",
            flatRecords: "array",
            parentRecords: "array",
            updatedRecords:"array",
            summaryRowRecords: "array",
            ids: "array",
            contextMenuSettings: {
                contextMenuItems: "array"
            },
            toolbarSettings: {
                toolbarItems: "array"
            },
            selectedItem: "data",
            summaryRows: "array"
        },
        ignoreOnExport: [
            "isEdit", "toolbarClick", "query", "queryCellInfo", "selectionType", "currentViewData", "enableRTL", "rowDataBound", "rowTemplate",
             "detailsTemplate", "editSettings", "localization", "cssClass", "dataSource", "allowKeyboardNavigation"
        ],
        observables: ["selectedRowIndex", "dataSource"],
        selectedItem: ej.util.valueFunction("selectedItem"),
        selectedRowIndex: ej.util.valueFunction("selectedRowIndex"),
        dataSource: ej.util.valueFunction("dataSource"),
        //GET THE GRID HEADER TABLE FOR COLUMNRESIZE
        getHeaderTable: function () {
            return this._$gridHeaderTable;
        },

        //SET THE GRID HEADER TABLE
        setGridHeaderTable: function (value) {
            this._$gridHeaderTable = value;
        },

        //GET THE GRIDHEADERCONTENT
        getHeaderContent: function () {
            return this._$gridHeaderContent;
        },

        //SET THE GRIDHEADERCONTENT
        setGridHeaderContent: function (value) {
            this._$gridHeaderContent = value;
        },

        //GET THE GRID CONTENT TABLE
        getContentTable: function () {
            return this._$gridContentTable;
        },

        getContent: function () {
            return this._$gridContent;
        },
        
        //SET THE GRID CONTENT TABLE
        setGridContentTable: function (value) {
            this._$gridContentTable = value;
        },

        //GET TREEGRID ROWS
        getRows: function () {
            var gridRows = this.getTreeGridRows(), $gridRows = this._excludeDetailRows();
            return $gridRows;
        },

        //SET TREEGRID ROWS
        setGridRows: function (value) {
            this._$gridRows = value;
        },

        //GET CURRENTVIEWPORT DATA
        getCurrentViewData: function () {
            return this.model.currentViewData;
        },

        //GET UPDATED CURRENT VIEW DATA
        getUpdatedCurrentViewData: function () {
            this._updateCurrentViewData();
            return this.model.currentViewData;
        },

        //GET FLAT RECORDS
        getFlatRecords: function () {
            return this.model.flatRecords;
        },

        //SET THE HEIGHT BASED ON THE RECORD COUNT
        setHeight: function (value) {
            this._totalHeight = value;
        },

        //GET THE UPDATEDRECORDS[SORTED,FILTERED,EXPANDCOLLAPSED]
        getUpdatedRecords: function () {
            return this.model.updatedRecords;
        },

        //GET THE SUMMARY ROW RECORDS
        getsummaryRowRecords: function () {
            return this.model.summaryRowRecords;
        },
        //GET THE UPDATED DATASOURCE
        getDataSource: function () {
            return this.dataSource();
        },

        //get the ids collection after perform the addition/delete operation
        //GET THE IDS COLLECTION AFTER PERFORM THE ADDITION/DELETE OPERATION
        getUpdatedIds: function () {
            return this.model.ids;
        },

        //GET THE RECORDS COUNT
        getRecordsCount: function () {
            return this._recordsCount;
        },

        //SET THE RECORDS COUNT
        setRecordsCount: function (value) {
            this._recordsCount = value;
        },

        //update the collapseRecount by updatedRecord collection
        updateCollapsedRecordCount:function()
        {
            var proxy = this,
                model = this.model,
                resultRecord = [];
            resultRecord = model.updatedRecords.filter(function (record) {
                return proxy.getExpandStatus(record) == false;
            });
            proxy._totalCollapsedRecordCount = resultRecord.length;
        },

        //In virtualization get expaned records from total record collection only
        _getDetailsExpandedRecords: function (records) {
            var proxy = this;
            var resultRecord = records.filter(function (record) {
                return record.isDetailsExpanded == true;
            });
            return resultRecord;
        },

        /* Get expanded details row height */
        _getExpandedDetailsRowHeight:function()
        {
            var proxy = this,
                model = this.model,
                expandedRecords = this.getExpandedRecords(model.updatedRecords),
                detailExpadedRecords = this._getDetailsExpandedRecords(expandedRecords);
            detailExpadedRecords = $(detailExpadedRecords).not(model.summaryRowRecords);
            return (proxy._detailsRowHeight * detailExpadedRecords.length);
        },
        //return collapsed recount length in updatedRecord collectio
        getCollapsedRecordCount: function (records) {
            return this._totalCollapsedRecordCount
        },
        
        _colgroupRefresh: function () {
            var gridheaderCol = $(this.getHeaderTable()).find('colgroup')[0];
            var gridcontentCol = $(this.getContentTable()).find('colgroup')[0];
            var headerColClone = $(gridheaderCol).clone();
            var contentColClone = $(gridcontentCol).clone();
            $(gridcontentCol).remove();
            $(gridheaderCol).remove();
            $(headerColClone).prependTo(this.getHeaderTable());
            $(contentColClone).prependTo(this.getContentTable());
        },
        _detailColsRefresh: function () {
            this._$headerCols = this.getHeaderTable().children("colgroup").find("col");
            this._$contentCols = this.getContentTable().children("colgroup").find("col");
            var colCount = this.model.columns.length;
            if (this._$headerCols.length > colCount) this._$headerCols.splice(colCount -1, 1);
            if (this._$contentCols.length > colCount) this._$contentCols.splice(colCount - 1, 1);
        },
        /*CONSTRUCTOR FUNCTION*/
        _init: function () {

            var proxy = this;

            /* default value for asp designer */
            if (proxy.model.isdesignMode) {
                proxy.model.columns = [
                    { "field": "Column1", "HeaderText": "Column1" },
                    { "field": "Column2", "HeaderText": "Column2" },
                    { "field": "Column3", "HeaderText": "Column3" },
                    { "field": "Column4", "HeaderText": "Column4" }];
                proxy.model.childMapping = "Children";
                var tempDataSource = [{ Column1: "Value 1", Column2: "Value 2", Column3: "Value 3", Column4: "Value 4", Children: [] }];

                for (var count = 1; count < 4; count++) {
                    var data = { Column1: "Value 1", Column2: "Value 2", Column3: "Value 3", Column4: "Value 4" };
                    tempDataSource[0].Children.push(data);
                }
                this.dataSource(tempDataSource);
                this.model.sizeSettings.height = "250px";
                this.model.sizeSettings.width = "600px";
            }
            proxy._trigger("load");
            proxy._initPrivateProperties();
            proxy._setCultureInfo();
            proxy._processEditing();
            proxy._initDatasource();
            proxy._createDragTooltipTemplate();
            proxy._createTooltipTemplate();
        },
        /* update allow keyboard navigation value*/
        updateAllowKeyboardNavigation: function (bool)
        {
            this.model.allowKeyboardNavigation = bool;
        },
        //#region RUNTIME UPDATE METHOD FOR TREEGRID
        //METHOD FOR UPDATE THE TREEGRID IN RUN TIME
        _setModel: function (options) {

            var proxy = this,
                model = proxy.model;

            for (var prop in options) {

                switch (prop) {

                    case "enableAltRow":
                        model.enableAltRow = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        break;
                    case "allowFiltering":
                        model.allowFiltering = options[prop];
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        //Re render the grid header
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        proxy.renderRecords();
                        //Update the scrollbar
                        proxy._$gridContent.ejScroller("refresh");
                        proxy._updateScrollCss();
                        if (model.showColumnChooser)
                            proxy._renderColumnChooserList(true);
                        break;
                    case "enableResize":
                        proxy.model.enableResize = options[prop];
                        if (model.enableResize) {
                            proxy._on($(window), "resize", proxy._windowResize);
                        }
                        else {
                            proxy._off($(window), "resize", proxy._windowResize);
                        }
                        break;
                    case "enableVirtualization":
                        model.enableVirtualization = options[prop];
                        var previousRecord = proxy._previousIndex && model.updatedRecords[proxy._previousIndex],
                            selectedRecord = proxy.selectedRowIndex() && model.updatedRecords[proxy.selectedRowIndex()],
                            updateRecords = model.updatedRecords,
                            selectedRowsIndexesLength = proxy._selectedRowsIndexes.length,
                            sortedColumnsLength = model.sortSettings.sortedColumns.length;
                        proxy.processBindings();
                        // change the table top position is 0 and change the update records for non-virtulization mode.
                        if (!model.enableVirtualization) {
                            proxy._$tableContent.css({
                                "top": 0
                            });
                        }
                        //Change the previous index, selectedRow index based on virtulization and non-virtulization mode.
                        proxy._previousIndex = previousRecord && model.updatedRecords.indexOf(previousRecord);
                        selectedRecord && proxy.selectedRowIndex(model.updatedRecords.indexOf(selectedRecord));

                        for (var i = 0; i < selectedRowsIndexesLength; i++) {
                            if (sortedColumnsLength > 0) {
                                if (proxy.selectedRowIndex() != proxy._selectedRowsIndexes[i]){
                                    var selectRecord = updateRecords[proxy._selectedRowsIndexes[i]];
                                    proxy._selectedRowsIndexes[i] = model.updatedRecords.indexOf(selectRecord);
                                    ej.TreeGrid.getRowByIndex(proxy, proxy._selectedRowsIndexes[i]).attr("aria-selected", "true").find('.e-rowcell').addClass(" e-selectionbackground e-active").attr("tabindex", "0");
                                }
                            }
                            else
                            {
                                var selectRecord = updateRecords[proxy._selectedRowsIndexes[i]];
                                proxy._selectedRowsIndexes[i] = model.updatedRecords.indexOf(selectRecord);
                                ej.TreeGrid.getRowByIndex(proxy, proxy._selectedRowsIndexes[i]).attr("aria-selected", "true").find('.e-rowcell').addClass(" e-selectionbackground e-active").attr("tabindex", "0");
                            }
                        }
                        var args = {};
                        args.requestType = ej.TreeGrid.Actions.Refresh;
                        proxy.sendDataRenderingRequest(args);
                        //Set background color for selected rows
                        for (var i = 0; i < selectedRowsIndexesLength; i++)
                            ej.TreeGrid.getRowByIndex(proxy, proxy._selectedRowsIndexes[i]).attr("aria-selected", "true").find('.e-rowcell').addClass(" e-selectionbackground e-active").attr("tabindex", "0");
                        proxy._isRowEdit = model.isEdit = false;
                        proxy._cancelSaveTools();
                        proxy._clearColumnMenu();
                        proxy._clearContextMenu();
                        this._removeDetailsRow();
                        break;

                    case "allowColumnResize":
                        model.allowColumnResize = options[prop];
                        if (model.allowColumnResize)
                            proxy._resizer = new ej.gridFeatures.gridResize(proxy);
                        proxy._enableColumnResizeEvents();
                        break;

                    case "allowSorting":
                        model.allowSorting = options[prop];
                        proxy.sortSetting(options[prop]);
                        break;
                   
                    case "dragTooltip":
                        $.extend(model.dragTooltip, options[prop]);
                        proxy.refreshContent();
                        break;

                    case "allowDragAndDrop":
                        model.allowDragAndDrop = options[prop];
                        var matched = jQuery.uaMatch(navigator.userAgent);
                        if (model.allowDragAndDrop) {
                            //Events binded for Row drag and drop
                            proxy._on(proxy.element, "mousedown", ".e-treegridrows, .e-templatecell", proxy.dragRecord);
                            proxy._on($(document), "mouseup", proxy.dragMouseUp);
                            // touch events binded for row drag and drop 
                            if (matched.browser.toLowerCase() == "chrome") {
                                proxy._on(proxy.element, "touchstart", ".e-treegridrows, .e-templatecell", proxy.dragRecord);
                                $(document.body).bind("touchmove", function (event) {
                                    event.preventDefault();
                                });
                                $(document.body).bind("touchmove", $.proxy(proxy.dragToolTip, proxy));
                                proxy._on(proxy.element, "touchend", ".e-treegridrows, .e-templatecell", proxy.dragMouseUp);
                                proxy._on(proxy.element, "touchleave", ".e-treegridrows, .e-templatecell", proxy.dragMouseUp);
                                $(proxy._$bodyContainer).bind("touchend", $.proxy(proxy.dragMouseUp, proxy));
                                $(proxy._$bodyContainer).bind("touchleave", $.proxy(proxy.dragMouseUp, proxy));
                            }
                        }
                        else {
                            proxy._off(proxy.element, "mousedown", ".e-treegridrows, .e-templatecell", proxy.dragRecord);
                            proxy._off($(document), "mouseup", proxy.dragMouseUp);

                            if (matched.browser.toLowerCase() == "chrome") {
                                proxy._off(proxy.element, "touchstart", ".e-treegridrows, .e-templatecell", proxy.dragRecord);
                                $(document.body).unbind("touchmove", function (event) {
                                    event.preventDefault();
                                });
                                $(document.body).unbind("touchmove", $.proxy(proxy.dragToolTip, proxy));
                                proxy._off(proxy.element, "touchend", ".e-treegridrows, .e-templatecell", proxy.dragMouseUp);
                                proxy._off(proxy.element, "touchleave", ".e-treegridrows, .e-templatecell", proxy.dragMouseUp);
                                $(proxy._$bodyContainer).unbind("touchend", $.proxy(proxy.dragMouseUp, proxy));
                                $(proxy._$bodyContainer).unbind("touchleave", $.proxy(proxy.dragMouseUp, proxy));
                            }
                        }
                        proxy.refreshContent();
                        break;

                    case "sortSettings":
                        $.extend(model.sortSettings, options[prop]);
                        proxy.refreshContent();
                        refreshArgs = {};
                        refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
                        proxy.sendDataRenderingRequest(refreshArgs);                       
                        break;

                    case "allowSelection":                      
                        model.allowSelection = options[prop];
                        if (!model.allowSelection) {                                                   
                            proxy.clearSelection(-1);
                            proxy.selectedRowIndex(-1);
                            proxy.model.selectedItem = null;
                            proxy._cancelSaveTools();
                        }
                        break;

                    case "selectionType":
                        proxy.clearSelection(-1);
                        proxy.selectedRowIndex(-1);
                        proxy.model.selectedItem = null;
                        proxy._cancelSaveTools();
                        model.selectionType = options[prop];
                        break;
                    case "showSummaryRow":
                        model.showSummaryRow = options[prop];
                        if (model.showSummaryRow)                           
                                proxy._createSummaryRow();                        
                        else {
                            $(proxy.element).find(".e-summaryrow").hide();
                            model.updatedRecords = $(model.updatedRecords).not(model.summaryRowRecords).get();
                            proxy.updateHeight();
                        }                        
                        break;
                    case "showTotalSummary":
                        model.showTotalSummary = options[prop];
                        if (model.showTotalSummary) {
                            proxy._createTotalSummaryRow();
                            var contentHeight = proxy._viewPortHeight - $(proxy._$totalSummaryRowContainer).height();
                            proxy._$gridContent.ejScroller({ height: contentHeight });
                            proxy._updateScrollCss();
                        }
                        else {
                            $(proxy.element).find(".e-footersummaryrowdiv").remove();
                            proxy._$gridContent.ejScroller({ height: proxy._viewPortHeight });
                            proxy._updateScrollCss();
                        }
                        break;
                    case "selectedRowIndex":
                        if (!this.isEdit && !this._isRowEdit) {
                            if (this.selectedRowIndex() >= 0
                                && !ej.isNullOrUndefined(this.selectedRowIndex()) && model.allowSelection && model.updatedRecords.length > this.selectedRowIndex()
                                && this.getExpandStatus(model.updatedRecords[this.selectedRowIndex()])) {
                                var index = this.selectedRowIndex();
                                if (!proxy._rowSelectingEventTrigger(this._previousIndex, index)) {
                                    if (model.showSummaryRow) {
                                        var recordsWithOutSummary = $(model.updatedRecords).not(model.summaryRowRecords).get(),
                                            currentSelectedRecord = recordsWithOutSummary[index];
                                        index = model.updatedRecords.indexOf(currentSelectedRecord);
                                    }
                                    proxy.selectRows(index);
                                    proxy._rowSelectedEventTrigger(index);
                                }
                            } else {
                                proxy.clearSelection(-1);
                                proxy.selectedRowIndex(-1);
                                proxy.model.selectedItem = null;
                                proxy._previousIndex = -1;
                                proxy._selectedRowsIndexes = [];

                            }
                            proxy._cancelSaveTools();
                        } else {
                            this.selectedRowIndex(proxy._previousIndex);
                        }
                        break;

                    case "editSettings":
                        $.extend(model.editSettings, options[prop]);
                        proxy._processEditing();
                        proxy._enableEditingEvents();
                        proxy._updateToolbarItems();
                        break;

                    case "allowKeyboardNavigation":
                        model.allowKeyboardNavigation = options[prop];
                        break;

                    case "dataSource":
                        if (!proxy._isRowEdit && !proxy.model.isEdit) {
                            proxy._refreshDataSource();
                        }
                        break;

                    case "columns":
                        model.columns = options[prop].slice(0);
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        proxy._processEditing();
                        //Rerendering the column header with new columns.;
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        var args = {};
                        args.requestType = ej.TreeGrid.Actions.Refresh;

                        /* for details rows detailsDataBound event can be called after the width update of column */
                        proxy._isRendered = false;
                        //Rerendering the content of the tree grid
                        proxy.sendDataRenderingRequest(args);
                        proxy.setWidthToColumns();
                        proxy._isRendered = true;
                        /* trigger queryCellInfo ,detailsDataBoundEvent after update the width of the columns */
                        proxy._trigger("refresh");
                        this._eventBindings();
                        this._hideCollapsedDetailsRows();

                        // Update the scrollbar
                        proxy._$gridContent.ejScroller("refresh");
                        proxy._updateScrollCss();
                        if(model.showColumnChooser)
                            proxy._renderColumnChooserList(true);
                        break;
                    
                    case "rowHeight":
                        model.rowHeight = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        break;

                    case "treeColumnIndex":
                        proxy.columnIndex(options[prop]);
                        break;

                    case "showColumnChooser":
                        model.showColumnChooser = options[prop];
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        proxy.setWidthToColumns();
                        proxy._$gridContent.ejScroller("refresh");
                        proxy._updateScrollCss();
                        if (!model.showColumnChooser){
                            proxy._clearColumnMenu();
                            $("#" + proxy._id + "ccDiv_wrapper").remove();
                        }
                        else
                            proxy._renderColumnChooser();
                        break;
                    case "locale":
                        proxy._clearContextMenu();
                        proxy._clearColumnMenu();
                        proxy._removeDetailsRow();
                        model.locale = options[prop];
                        proxy._setCultureInfo();
                        proxy._processEditing();
                        proxy._addInitTemplate();
                        proxy.refreshContent();
                        //Re render the grid header(for updating filtering elements also) and content
                        proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                        proxy.renderRecords(args);
                        // Update the scrollbar
                        proxy._$gridContent.ejScroller("refresh");
                        proxy._updateScrollCss();
                        // Update the column chooser
                        if (model.showColumnChooser)
                            proxy._renderColumnChooserList(true);
                        // Update the toolbar
                        if (model.toolbarSettings.showToolbar)
                            proxy._updateToolbar();
                        break;
                    case "toolbarSettings": 
                        $.extend(proxy.model.toolbarSettings, options[prop]);
                        proxy._updateToolbar();
                        break;
                    case "contextMenuSettings":
                        $.extend(proxy.model.contextMenuSettings, options[prop]);
                        if (!model.contextMenuSettings.showContextMenu)
                            proxy._clearContextMenu();
                        this._removeDetailsRow();
                        break;
                    case "enableCollapseAll":
                        model.enableCollapseAll = options[prop];
                        if (model.enableCollapseAll)
                            proxy.collapseAll();
                        else
                            proxy._expandAll();
                        break;
                    case "showDetailsRow":
                        model.showDetailsRow = options[prop];
                        this._updateDetailsRowProperties("showDetailsRow");
                        break;
                    case "detailsRowHeight":
                        model.detailsRowHeight = options[prop];
                        this._updateDetailsRowProperties("detailsRowHeight");
                        break;
                    case "detailsTemplate":
                        model.detailsTemplate = options[prop];
                        this._updateDetailsRowProperties("detailsTemplate");
                        break;
                    case "showDetailsRowInfoColumn":
                        model.showDetailsRowInfoColumn = options[prop];
                        this._updateDetailsRowProperties("showDetailsRowInfoColumn");
                        break;
                    case "altRowTemplateID":
                        this._initiateSetModel();
                        model.altRowTemplateID = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        this.updateHeight();
                        break;
                    case "rowTemplateID":
                        this._initiateSetModel();
                        model.rowTemplateID = options[prop];
                        proxy._addInitTemplate();
                        proxy.renderRecords(args);
                        this.updateHeight();
                        break;
                    case "sizeSettings":
                        this._initiateSetModel();
                        $.extend(model.sizeSettings, options[prop]);

                        this._calculateDimensions();
                        if (this._getViewPortHeight() <= 0) {
                            model.sizeSettings.height = "450px";
                            this._updateElementHeight();
                        }
                        this._windowResize();
                        break;
                    case "showGridCellTooltip":
                        model.showGridCellTooltip = options[prop];
                        break;

                    case "showGridExpandCellTooltip":
                        model.showGridExpandCellTooltip = options[prop];
                        break;

                    case "allowMultiSorting":
                        model.allowMultiSorting = options[prop];
                        break;

                    case "query":
                        var proxy = this;
                        if (!ej.isNullOrUndefined(options[prop]))
                            model.query = options[prop];
                        else
                            model.query = ej.Query();
                        proxy._refreshDataSource();
                        break;
                    case "filterSettings":
                        var proxy = this,
                            filterSettings = proxy.model.filterSettings,
                            filterBarMode = filterSettings.filterBarMode,
                            args = {};
                        // Get the filteredColumn value before going to clear the filter columns
                        var filteredColumn = $.extend([], filterSettings.filteredColumns),
                            newFilterColumn = options[prop].filteredColumns;
                        proxy.clearFilter(); //Clear the all filtered Columns
                        $.extend(filterSettings, options[prop]);
                        if (ej.isNullOrUndefined(newFilterColumn)) {
                            filterSettings.filteredColumns = filteredColumn;
                        }
                        args.requestType = "filtering";
                        //Render the record with sorting and filtering
                        proxy._ensureDataSource(args);
                        //Method binding for filtering data.
                        if (filterBarMode == "immediate") {
                            proxy._on(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                            proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox")._on(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
                        }
                        else {
                            proxy._off(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                            proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
                        }
                        break;
                }
            }
        },
        /* clear contextMenu, columnChooser, detailsTemplate and save edited item before change
        property by setModel*/
        _initiateSetModel: function ()
        {
            var model = this.model;
            if (model.isEdit)
                this.saveCell();
            else if (this._isRowEdit)
                this._saveRow();
            this._clearContextMenu();
            this._clearColumnMenu();
            this._removeDetailsRow();
        },
        /* update the tree grid control as per new detailsRow fetaures API values by setModel */
        _updateDetailsRowProperties: function (property) {

            var proxy = this,
                model = this.model;

            /* cance edited cell or row */
            if(model.isEdit || this._isRowEdit){
                this.cancelRowEditCell();
                this._cancelSaveTools();
            }
            /* clear columnchooser menu and contextmenu and detail wrapper */
            this.clearColumnMenu();
            this._clearContextMenu();
            this._removeDetailsRow();
             
            /* update the detailsRowHeight private property */
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn) {
                proxy._detailsRowHeight = model.detailsRowHeight;
            } else {
                proxy._detailsRowHeight = 0;
            }

            /* update the details expand status as per details feture API values */
            model.flatRecords.forEach(function (record) {
                if (model.showDetailsRow && model.detailsTemplate) {
                    record.isDetailsExpanded = model.showDetailsRowInfoColumn ? false : true;
                } else {
                    record.isDetailsExpanded = false;
                }
            });
            
            /* update the tree grid control as per new detailsRow API's */
            if ((property == "detailsTemplate" || property == "detailsRowHeight") && model.showDetailsRowInfoColumn) {
                proxy._addInitTemplate();
            } else {
                proxy._$gridHeaderContent.replaceWith(proxy._renderGridHeader());
                proxy._addInitTemplate();
                /* while changing details row features API scroll top is assigned to beacause of rowHeight changes*/
                this.onScrollHelper(0);
                proxy.refreshContent();

                /* for details rows detailsDataBound event can be called after the width update of column */
                proxy._isRendered = false;
                //Rerendering the content of the tree grid
                var refreshArgs = {};
                refreshArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.sendDataRenderingRequest(refreshArgs);
                proxy.setWidthToColumns();
                proxy._isRendered = true;
                proxy._trigger("refresh");
                /* trigger queryCellInfo ,detailsDataBoundEvent after update the width of the columns */
                this._eventBindings();
                this._hideCollapsedDetailsRows();

                proxy._$gridContent.ejScroller("refresh");
                proxy._updateScrollCss();
            }
        },
        /* hide the collapsed state detail rows for prevent alignement issue for rendering sf control in  display none state*/
        _hideCollapsedDetailsRows:function()
        {
            var proxy = this, model = this.model;
            /* Collapse hidden expanded details rows */
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn)
                $(proxy.getTreeGridRows()).filter("tr.collapsedrowexpandeddetailsrow").css("display", "none");
        },
        _updateToolbar: function () {
            var proxy = this,
                model = proxy.model,
                scroller = proxy._$gridContent.ejScroller("instance"),
                           left = scroller.scrollLeft(),
                           top = scroller.scrollTop();
            //$.extend(proxy.model.toolbarSettings, options[prop]);
            $("#" + proxy._id + "_toolbarItems").remove();
            if (model.toolbarSettings.showToolbar)
                proxy._renderToolbarTemplate().insertBefore($("#" + proxy._id + "e-gridheader"));
            if (model.isEdit)
                proxy.cancelEditCell();
            if (proxy._isRowEdit) {
                proxy.cancelRowEditCell();
                proxy._cancelSaveTools();
            }
            proxy._viewPortHeight = proxy._getViewPortHeight();
            scroller.option("height", proxy._viewPortHeight);
            if (model.enableVirtualization) {
                proxy.processBindings();
                proxy.renderRecords();
            }
            scroller.option("scrollTop", top);
            scroller.option("scrollLeft", left);
            proxy._clearContextMenu();
            this._removeDetailsRow();

        },
        //refresh data source on set model     
        _refreshDataSource:function()
        {
            var proxy = this, model = this.model;
            //Cancelling the editable state of the row
            if (proxy._isRowEdit || proxy.model.isEdit) {
                proxy.cancelRowEditCell();
            }
            proxy.resetModelCollections();
            proxy._createdAt = "load";
            if (this.dataSource() instanceof ej.DataManager) {
                var queryPromise = this.dataSource().executeQuery(model.query);
                queryPromise.done(ej.proxy(function (e) {
                    if (proxy.dataSource().dataSource.offline) {
                        proxy._retrivedData = proxy.dataSource().dataSource.json;
                    } else {
                        proxy._retrivedData = e.result;
                    }
                    if (model.idMapping && model.parentIdMapping) {
                        //cloning the datasource
                        var dataArray = e.result.slice(0);
                        var data = [];
                        $.each(dataArray, function (indx, value) {
                            data.push(jQuery.extend(true, {}, value));
                        });
                        proxy._reconstructDatasource(data);
                    } else {
                        proxy._reconstructDatasource(e.result);
                    }
                    proxy._createRecords(proxy.secondaryDatasource);

                    args = {};
                    args["requestType"] = ej.TreeGrid.Actions.RefreshDataSource;
                    proxy._ensureDataSource(args);
                    if (proxy.selectedRowIndex() !== -1 && proxy.selectedRowIndex() < proxy.model.currentViewData.length) {
                        proxy.selectRows(proxy.selectedRowIndex());
                    }
                    if (proxy._$gridContent.hasClass("e-scroller"))
                        proxy._$gridContent.ejScroller("refresh");
                    proxy._createdAt = null;
                }));
            }
            else if (this.dataSource()) {
                proxy._retrivedData = proxy.dataSource();
                if (model.idMapping && model.parentIdMapping) {
                    if ((model.idMapping.length > 0) && (model.parentIdMapping.length > 0)) {
                        var dataArray = this.dataSource().slice(0);
                        proxy._reconstructDatasource(dataArray);
                        proxy._createRecords(proxy.secondaryDatasource);
                    }
                }
                else if (model.flatRecords.length === 0 && this.dataSource() !== null) {
                    proxy._createRecords(this.dataSource());
                }
                args = {};
                args["requestType"] = ej.TreeGrid.Actions.RefreshDataSource;
                proxy._ensureDataSource(args);
                if (this.selectedRowIndex() !== -1 && this.selectedRowIndex() < proxy.model.currentViewData.length) {
                    proxy.selectRows(this.selectedRowIndex());
                }
                if (this._$gridContent.hasClass("e-scroller"))
                    proxy._$gridContent.ejScroller("refresh");
                proxy._createdAt = null;
            }
            else if (this.dataSource() == null) {
                args = {};
                args["requestType"] = ej.TreeGrid.Actions.RefreshDataSource;
                proxy._ensureDataSource(args);
                proxy._createdAt = null;
            }
            //Render Records in collapsed state
            if (!proxy.model.isFromGantt && proxy.model.enableCollapseAll) {
                proxy.collapseAll();
            }
            $(".e-tooltiptable").remove();
            if(model.showSummaryRow || model.showTotalSummary)
                proxy._summaryRow();
        },
        //REFRESH THE TREEGRIDCONTENT
        refreshContent: function () {

            var proxy = this,
                args = {};     
            args.requestType = ej.TreeGrid.Actions.Refresh;
            proxy.processBindings(args);
        },


        //RESET THE COLLECTIONS WHILE CHANGING THE DATASOURCE
       
        resetModelCollections: function () {

            var proxy = this,
            model = proxy.model;

            // model.sortSettings.sortedColumns = [];
            model.currentViewData = [];
            model.flatRecords = [];
            model.parentRecords = [];
            model.updatedRecords = [];
            model.summaryRowRecords = [];
            model.ids = [];
            proxy.secondaryDatasource = [];
            proxy._parentRecords = [];
            proxy._storedIndex = -1;

        },        

        //#endregion

        //DECLARE THE PRIVATE VARIABLES
        _initPrivateProperties: function () {

            var proxy = this,
                model = this.model;

            proxy._id = proxy.element.attr("id"),
            proxy._columnsWidthCollection = [],
            proxy._cSortedColumn = null,
            proxy._cSortedDirection = null,
            proxy._dataManager = null,
            proxy._disabledSortableColumns = [],
            proxy._filteredRecords = [],
            proxy._disabledToolItems = [],
            proxy_currentFilterColumn = [];
            proxy._fieldColumnNames = [],
            proxy._multiSortRequest = null,
            proxy._$gridContainer = null,
            proxy._$gridContent = null,
            proxy._$gridContentTable = null,
            proxy._$gridHeaderTable = null,
            proxy._$gridHeaderContainer = null,
            proxy._$totalSummaryRowContainer = null,
            proxy._recordsCount = null,
            proxy._$gridRows = null,
            proxy._gridRows = null,
            proxy._calculateDimensions(),
            proxy._headerColumnNames = [],
            proxy._hiddenColumns = [],
            proxy._multiSelectCtrlRequest = false,
            proxy._multiSelectShiftRequest = false,
            proxy._offset = 0,
            proxy._previousIndex = -1,
            proxy._prevRBottom = 0,
            proxy._prevRTop = 0,
            proxy._prevScrollTop = 0,
            proxy._prevScrollLeft = 0,
            proxy._prevVBottom = 0,
            proxy._resizer = null,
            proxy._scrollTop = 0,
            proxy._scrollLeft = 0,
            proxy._searchString = "",
            proxy._fieldName = "",
            proxy._selectedRowsIndexes = [],
            proxy._sortedRecords = [],
            proxy._storedIndex = -1,
            proxy._rowIndex = -1,
            proxy._$tableContent = null,
            proxy._$footertableContent = null,
             proxy._$footerContainer = null,
            proxy._tempsortedrecords = [],
            proxy._totalHeight = 0,
            proxy._viewPortHeight = 0,
            proxy._recordIndexCount = 0,
            proxy._totalCollapsedRecordCount = 0,
            proxy._visibleColumns = [],
            proxy._visibleRange = null,
            proxy._vScrollDir = 1,
            proxy._vScrollDist = 0,
            proxy._cellEditTemplate = $(),
            proxy._tooltipTimer,
            proxy._totalBorderWidth = 2;//left and right border width
            proxy._totalBorderHeight = 1;
            proxy._toolboxTooltipTexts = null;
            proxy._contextMenuTexts = null;
            proxy._columnMenuTexts = null;
            proxy._dropDownListBlanksText = null;
            proxy._dropDownListClearText = null;
            proxy._trueText = null;
            proxy._falseText = null;
            proxy._rowDragIndexes = [];
            proxy._dropCancel = false;
            proxy._cellEditingDetails = {
                cellValue: null,
                rowIndex: -1,
                columnIndex: -1,
                fieldName: null,
                _data: null,
                cellEditType: "",
                cancelSave: false,
                defaultData: null,
                insertedTrCollection: [],
                data: null
            },
            proxy._tdsOffsetWidth = [],
            proxy._isEnterKeyPressed = false,
            proxy._scrollBarHeight = 18,
            proxy._removedCount = 0,
            proxy._$gridHeaderContent = null,
            proxy.secondaryDatasource = [],
            //variables for context menu items
            proxy._contextMenuItems = null;
            proxy._subContextMenuItems = null;
           
            proxy._createSubContextMenuTemplate();
            proxy._createContextMenuTemplate();
            proxy._createColumnMenuTemplate();
            proxy._retrivedData = this.dataSource();
            proxy._createdAt = "load";
            proxy._isInExpandCollapseAll = false;
            proxy._dataManager = proxy.dataSource() instanceof ej.DataManager ? proxy.dataSource() :
                proxy.dataSource() != null ? ej.DataManager(proxy.dataSource()) : null;
            proxy._dragMouseDown = false;
            proxy._dragTooltip = false;
            proxy._dragMouseLeave = false;
            _timerDragDown = null;
            _timerDragUp = null;  
            /* Check data manger is updatable */ 
            proxy._isDataManagerUpdate = false;
            proxy._jsonData = null;

            /* flag for refreshing newly added record */
            proxy._isRefreshAddedRecord = false;
            /* Used as flag for refresh chart and grid rows on adding when parent item is in collpased state*/
            proxy._isInAdd = false;         
            /* get unique index for tree grid add whil add */
            proxy._maxRowIndex = 0;
           /* used to check treegrid rendering is completed on load time*/
            proxy._isRendered = false;

            proxy._isSummaryRow = false;
            proxy._flatChildRecords = [];
            proxy._footerSummaryRecord = [];
            proxy._filteredChildRecords = [];
            proxy._parentRecords = [];
            proxy._detailsRowHeight = 0;
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn)
            {
                proxy._detailsRowHeight = model.detailsRowHeight;
            }
        },
        //set size to treegrid core by user values
        _calculateDimensions: function ()
        {
            var proxy = this, model = this.model;

            if (!model.isFromGantt) {
                if (model.sizeSettings.width)
                    proxy.element.css("width", model.sizeSettings.width);

                if (model.sizeSettings.height)
                    proxy.element.css("height", model.sizeSettings.height);
                else
                {
                    if (!proxy.element[0].style.height) {
                        proxy.element.height(450);
                    }
                }
            }
            proxy._gridHeight = proxy.element.height();
            proxy._gridWidth = proxy.element.width();

        },
        //DATASOURCE CHECKING
        _checkDataBinding: function () {

            var proxy = this,
                model = proxy.model;
            proxy._createdAt = null;
            proxy.element.addClass('e-treegrid-core');
            proxy.element.attr("tabindex", "0");
            proxy._renderToolbar();
            if(proxy.model.showColumnChooser)
                proxy._renderColumnChooser();
            proxy._renderAfterColumnInitialize();
            proxy._viewPortHeight = proxy._getViewPortHeight();

            /* check if viewport heigth value*/
            if (!model.isFromGantt && proxy._viewPortHeight < 0) {
                proxy._updateElementHeight();
            }

            proxy._ensureDataSource();
            proxy._initGridRender();
            proxy._$gridContent = $("#" + proxy._id + "e-gridcontent");
            proxy._$gridContainer = $("#" + proxy._id + "e-gridcontainer");
            proxy._$tableContent = $("#" + proxy._id + "e-table");
            proxy._$gridContent.css({ "height": proxy._viewPortHeight + "px" })
            proxy.updateHeight();
            proxy._wireEvents();            
            
            proxy.setWidthToColumns();
            proxy._summaryRow();
            proxy._renderScroller();
            //Render Records in collapsed state
            if (!proxy.model.isFromGantt && proxy.model.enableCollapseAll) {
                proxy.collapseAll();
            }
            proxy._updateScrollCss();
            proxy._resizeFilteringElements();
            proxy._initialEndRendering();
            if (model.isFromGantt) {
                proxy._addEmptyColumntoGrid();
            }
            if (!model.isFromGantt) {
                proxy._checkDataManagerUpdate();
            } else {
                proxy._isDataManagerUpdate = model.dataManagerUpdate.isDataManagerUpdate;
                proxy._jsonData = model.dataManagerUpdate.jsonData;
            }
        },
        //Updated records of online data manager 
        getUpdatedDataManagerData: function () {
            var proxy = this, data;
            data = proxy._jsonData;
            return data;
        },

        //Check if data manager is updatable
        _checkDataManagerUpdate: function () {
            var proxy = this,
                dataSource = this.dataSource();
            proxy._isDataManagerUpdate = false;
            proxy._jsonData = null;

            if (dataSource instanceof ej.DataManager) {
                if (dataSource.dataSource.offline && dataSource.dataSource.json) {
                    proxy._isDataManagerUpdate = true;
                    proxy._jsonData = dataSource.dataSource.json;
                }
                else if (!dataSource.dataSource.offline) {
                    proxy._isDataManagerUpdate = true;
                    proxy._jsonData = proxy._retrivedData;
                }
            }
        },

        _initDatasource: function () {
            var proxy = this,
                model = proxy.model,retriveData;
            if (!model.isFromGantt) {
            if (this.dataSource() instanceof ej.DataManager) {

                var queryPromise = this.dataSource().executeQuery(model.query);
                queryPromise.done(ej.proxy(function (e) {
                    if (proxy.dataSource().dataSource.offline) {
                        proxy._retrivedData = proxy.dataSource().dataSource.json;
                    } else {
                        proxy._retrivedData = e.result;
                    }
                    if (model.idMapping && model.parentIdMapping) {
                        //cloning the datasource
                        var dataArray = e.result.slice(0);
                        var data = [];
                        $.each(dataArray, function (indx, value) {
                            data.push(jQuery.extend(true, {}, value));
                        });
                        proxy._reconstructDatasource(data);
                    } else {
                        proxy._reconstructDatasource(e.result);
                    }
                    proxy._createRecords(proxy.secondaryDatasource);
                    proxy._checkDataBinding();
                }));
            }

            else if (this.dataSource()) {
                proxy._retrivedData = proxy.dataSource();
                if (model.idMapping && model.parentIdMapping) {
                        //cloning the datasource
                        var dataArray = this.dataSource().slice(0);
                        var data = [];
                        $.each(dataArray, function (indx, value) {
                            data.push(jQuery.extend(true, {}, value));
                        });
                        proxy._reconstructDatasource(data);
                        proxy._createRecords(proxy.secondaryDatasource);
                }
                else if (model.flatRecords.length === 0 && this.dataSource() !== null) {
                    proxy._createRecords(this.dataSource());
                }
                proxy._checkDataBinding();
            }
             else {
                    proxy._checkDataBinding();
            }
            } else {
                proxy._checkDataBinding();
            }
            
        },

        _reconstructDatasource: function (datasource) {
            var proxy = this, model = proxy.model,
                childItems, data, tempRecord = [];

            for (var i = 0; i < datasource.length; i++) {
                var data = {};
                data = ej.DataManager(datasource).executeLocal(ej.Query().where(model.parentIdMapping,
                    ej.FilterOperators.equal,
                    datasource[i][model.idMapping]));


                if ((ej.isNullOrUndefined(datasource[i][model.parentIdMapping]) || datasource[i][model.parentIdMapping] == 0)
                    && data.length > 0) {
                    datasource[i][model.childMapping] = data;
                    proxy.secondaryDatasource.push(datasource[i]);
                }
                else if (ej.isNullOrUndefined(datasource[i][model.parentIdMapping]) && !(data && data.length > 0)) {
                    proxy.secondaryDatasource.push(datasource[i]);
                }

                else if (data && data.length > 0 && !(ej.isNullOrUndefined(data[0][model.parentIdMapping]))){
                    proxy._appendChildDataItems(proxy.secondaryDatasource, data);
                }
                else if ((ej.isNullOrUndefined(datasource[i][model.parentIdMapping]) || datasource[i][model.parentIdMapping] == 0)
                    && data.length == 0) {
                    proxy.secondaryDatasource.push(datasource[i]);
                }

                if (data && data.length > 0) {

                    proxy._datasourceChildItems = proxy._reconstructDatasource(data);
                    if (ej.isNullOrUndefined(datasource[i][model.childMapping])) {
                        datasource[i][model.childMapping] = proxy._datasourceChildItems;
                        proxy._getParentIndex(datasource);
                    }
                    proxy._datasourceChildItems = [];

                }

                else if (!(ej.isNullOrUndefined(datasource[i][model.parentIdMapping]))) {
                    tempRecord.push(datasource[i]);
                }
            }

            if (tempRecord && tempRecord.length > 0) {
                return tempRecord;
            }
        },

        _appendChildDataItems: function (parentDatasource,data) {
            var proxy = this,
                model = proxy.model;
            
            for (var count = 0; count < parentDatasource.length; count++) {
                
                if (parentDatasource[count][model.idMapping] === data[0][model.parentIdMapping]) {
                    parentDatasource[count][model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[count][model.childMapping])) &&
                    parentDatasource[count][model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[count], data);
                }
            }

            if ((ej.isNullOrUndefined(parentDatasource.length)) && parentDatasource) {
                if (parentDatasource[model.idMapping] === data[0][model.parentIdMapping]) {
                    parentDatasource[model.childMapping] = data;
                }
                else if (!(ej.isNullOrUndefined(parentDatasource[model.childMapping])) &&
                    parentDatasource[model.childMapping].length > 0) {
                    proxy._appendChildDataItems(parentDatasource[model.childMapping], data);
                }
            }

        },
        
        _getParentIndex: function (datasource) {
            var proxy = this, model = proxy.model,
                dataArr = proxy.secondaryDatasource, parentIndex = 0;

            if (datasource.length > 0) {
                parentIndex = datasource[0][model.parentIdMapping];
            }
            for (var i = 0; i < dataArr.length; i++) {
                if (dataArr[i][model.idMapping] == parentIndex && ej.isNullOrUndefined(dataArr[i][model.childMapping])) {
                    dataArr[i][model.childMapping] = (datasource);
                    return true;
                }
                else {
                    var childItems = dataArr[i][model.childMapping];
                    if (childItems) {
                        for (var k = 0; k < childItems.length; k++) {
                            if (childItems[k][model.idMapping] == parentIndex && ej.isNullOrUndefined(childItems[k][model.childMapping])) {
                                childItems[k][model.childMapping] = (datasource);
                                return true;
                            }
                        }
                    }
                }
            }
        },
        //Window resize event
        _windowResize:function(e)
        {
            var proxy = this,
               model = this.model,
               treeGridHeader = proxy._$gridHeaderContent,
               totalSummaryRowContent = proxy._$totalSummaryRowContainer,
               treeGridContent = $("#" + proxy._id + "e-gridcontent"),
               treegridToolbar = $("#" + proxy._id + "_toolbarItems"),
               sizeSettingsHeight = model.sizeSettings.height,
               sizeSettingsWidth = model.sizeSettings.width,
               elementStyleWidth = proxy.element[0].style.width,
               elementStyleHeight = proxy.element[0].style.height,
               width, height;

            proxy._clearContextMenu();
            proxy._clearColumnMenu();
            $("#" + this._id + "detailscellwrapper").length > 0 && this._removeDetailsRow();

            //Width calculation
            if ((sizeSettingsWidth && (typeof(sizeSettingsWidth) != "number" && sizeSettingsWidth.indexOf("%") != -1)) || elementStyleWidth.indexOf("%") != -1) {
                var ganttWidth = sizeSettingsWidth ? sizeSettingsWidth : elementStyleWidth;
                var containerWidth = $(proxy.element).parent().width() ? $(proxy.element).parent().width() : $(proxy.element).width();
                var tempWidth = (containerWidth / 100) * parseInt(ganttWidth);
                width = tempWidth;
            }
            else
                width = $(proxy.element).width();
            proxy._gridWidth = width;
            //height calculation
            if ((sizeSettingsHeight && (typeof (sizeSettingsHeight) != "number" && sizeSettingsHeight.indexOf("%") != -1)) || elementStyleHeight.indexOf("%") != -1) {
                var ganttHeight = sizeSettingsHeight ? sizeSettingsHeight : elementStyleHeight;
                var containerHeight = $(proxy.element).parent().height() ? $(proxy.element).parent().height() : $(proxy.element).height();
                var tempHeight;
                if ($(proxy.element).parent().height())
                    tempHeight = (containerHeight / 100) * parseInt(ganttHeight);
                else
                    tempHeight = $(proxy.element).height();
                height = tempHeight;
            }
            else
                height = $(proxy.element).height();

            //excludes border 
            height = height - proxy._totalBorderHeight;
            if (proxy._$totalSummaryRowContainer)
                height -= proxy._$totalSummaryRowContainer.height();
            width = Math.round(width - proxy._totalBorderWidth);
            var treeGridWidth,headerHeight,headerWidth, top, left;
            top = proxy._$gridContent.ejScroller("option", "scrollTop");
            left = proxy._$gridContent.ejScroller("option", "scrollLeft");
            proxy._$gridContent.ejScroller({
                width: width,
                height: height - $(treeGridHeader).outerHeight() - treegridToolbar.outerHeight(),//for border-width of gridcontent
            });
            proxy._updateScrollCss();
            if ($(treeGridHeader).hasClass("e-scrollcss") && proxy._$gridContent.children(".e-vscrollbar").length > 0) {
                $(treeGridHeader).width(width - 17);
            }
            else {
                $(treeGridHeader).width(width);
            }

            if (proxy._$totalSummaryRowContainer) {
                if ($(totalSummaryRowContent).hasClass("e-scrollcss") && proxy._$gridContent.children(".e-vscrollbar").length > 0)
                    $(totalSummaryRowContent).width(width - 17);
                else
                    proxy._$totalSummaryRowContainer.ejScroller("refresh");
            }
            if (model.toolbarSettings.showToolbar && treegridToolbar.length > 0) {
                treegridToolbar.width(width);
                treegridToolbar.children(".e-ul:first-child").width(width);
            }

            proxy._$gridContent.ejScroller("refresh");
            proxy.updateViewPortHeight();         
            proxy._$gridContent.ejScroller("option", "scrollTop",top);
            proxy._$gridContent.ejScroller("option", "scrollLeft", left);
            proxy._$gridHeaderContainer.scrollLeft(left);
            if (model.enableVirtualization) {
                proxy.cancelRowEditCell(); // To hide or destroy popup menu
                proxy.processBindings();
                var tempArgs = {},
                    rowHeight = model.rowHeight;
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.sendDataRenderingRequest(tempArgs);
                /* check detail template and its row height */
                if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn)
                {
                    rowHeight += model.detailsRowHeight;
                }
                
                if ((model.currentViewData.length * rowHeight) < proxy._viewPortHeight) {

                    var currentPosition = top - (proxy._viewPortHeight - model.currentViewData.length * rowHeight);
                    if (currentPosition < 0)
                        currentPosition = 0;
                    proxy._$gridContent.ejScroller("scrollY",currentPosition, true);
                }
            }
            proxy._resizeFilteringElements();
            
        },

        //PROCESSBINDING
        processBindings: function (args) {

            var proxy = this,
                model = proxy.model;

            model.query = new ej.Query();

            if (proxy._trigger("actionBegin", args)) {
                return true;
            }

            proxy._ensureDataSource(args);

        },
        //updated the treegrid record collection after record update in Gantt
        setUpdatedRecords: function (flatRecords, updatedRecords, ids, parentRecords, dataSource) {
            var model = this.model;
            model.flatRecords = flatRecords;
            model.updatedRecords = updatedRecords;
            model.ids = ids;
            model.parentRecords = parentRecords;
            this.dataSource(dataSource);
        },
        //In virtualization get expaned records from total record collection only
        getExpandedRecords: function (records) {
            var proxy = this;
            var resultRecord = records.filter(function (record) {
                return proxy.getExpandStatus(record) == true;
            });
            return resultRecord;
        },

        //update the expandStatus of all child records in non-virtualization mode
        updateExpandStatus:function(record , expanded)
        {
            var proxy = this,
                childRecords = record.childRecords;
                for (var count = 0; count < childRecords.length; count++) {
                    childRecords[count].isExpanded = expanded;
                    if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                        proxy.updateExpandStatus(childRecords[count], expanded);
                    }
                }
        },
        //Expand collapse, sort, delete, filter search the record collections
        _ensureDataSource: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns,
                filteredColumns = model.filterSettings.filteredColumns,
                isupdatedHeight = false,
                index,
                dataSource =this.dataSource(),
                childIndex;
                proxy._filteredRecords = [];

            model.query.requiresCount();
            proxy._queryManagar = model.query;

            if ($.type(dataSource) !== "array" && !(dataSource instanceof ej.DataManager)) {
                this.dataSource([]);
                dataSource = this.dataSource();
            }

            //Expand Collapse Opearation
            if (args && args.requestType === ej.TreeGrid.Actions.ExpandCollapse) {

                isupdatedHeight = true;
                proxy._filteredRecords = [];
                args.data.expanded = args.expanded;

                proxy._filteredRecords = model.flatRecords.filter(function (record) {
                    return proxy.getExpandStatus(record) == true;
                });
                model.updatedRecords = proxy._filteredRecords;
            }

            //Delete selected record
            if (args && (model.editSettings.allowDeleting || args.isDragAndDropDelete) && args.requestType === ej.TreeGrid.Actions.Delete) {

                var deletedRow = args.data,
                    recordIndex,
                    rowindex;
                recordIndex = model.flatRecords.indexOf(deletedRow);
                if (deletedRow.parentItem) {

                    var childRecords = deletedRow.parentItem.childRecords;
                    childIndex = childRecords.indexOf(deletedRow);

                    deletedRow.parentItem.childRecords.splice(childIndex, 1);
                    //Delete the summary row of one parent after deleted the all child records of the parent
                    if (model.showSummaryRow) {
                        var summaryRowLength = model.summaryRows.length;
                        if (summaryRowLength == childRecords.length) {
                            deletedRow.parentItem.childRecords.splice(0, summaryRowLength);
                            model.flatRecords.splice(recordIndex, summaryRowLength);
                        }
                    }
                    if (!model.parentIdMapping)
                        deletedRow.parentItem.item[model.childMapping].splice(childIndex, 1);
                }
                //Delete from parent collection if item is parent
                var parentIndex=model.parentRecords.indexOf(deletedRow);
                if (parentIndex!== -1)
                {
                    model.parentRecords.splice(parentIndex, 1);
                }
                if (recordIndex !== -1) {
                    var deletedRecordCount = proxy.getChildCount(deletedRow, 0);
                    model.flatRecords.splice(recordIndex, deletedRecordCount + 1);
                    model.ids && model.ids.splice(recordIndex, deletedRecordCount + 1);
                }

                model.updatedRecords = model.enableVirtualization ? proxy.getExpandedRecords(model.flatRecords) : model.flatRecords.slice();
                proxy.updateCollapsedRecordCount();
                proxy.updateHeight();
                isupdatedHeight = true;
               
                
                if(!model.parentIdMapping)
                {
                    if (dataSource instanceof ej.DataManager) {
                        if (dataSource.dataSource.offline && dataSource.dataSource.json) {
                            rowindex = dataSource.dataSource.json.indexOf(deletedRow.item);
                            if (rowindex !== -1)
                                dataSource.dataSource.json.splice(rowindex, 1);
                        }
                        else if (proxy._isDataManagerUpdate) {
                            rowindex = proxy._jsonData.indexOf(deletedRow.item);
                            proxy._jsonData.splice(rowindex, 1);
                        }
                    } else {
                        rowindex = dataSource.indexOf(deletedRow.item);
                        if (rowindex !== -1)
                            dataSource.splice(rowindex, 1);
                    }
                }
                else
                {
                    //remove child item
                    if (deletedRow.hasChildRecords && deletedRow.childRecords.length > 0)
                    {
                        proxy._removeChildItem(deletedRow);
                    }
                    //remove deleting item

                    if (dataSource instanceof ej.DataManager) {
                        if (dataSource.dataSource.offline && dataSource.dataSource.json) {
                            var idx = dataSource.dataSource.json.indexOf(deletedRow.item);
                            if (idx !== -1)
                                dataSource.dataSource.json.splice(idx, 1);
                        }
                        else if (proxy._isDataManagerUpdate) {
                            var idx = proxy._jsonData.indexOf(deletedRow.item);
                            if (idx !== -1)
                                proxy._jsonData.splice(idx, 1);
                        }
                    }
                    else
                    {
                        var idx = dataSource.indexOf(deletedRow.item);
                        if (idx !== -1)
                            dataSource.splice(idx, 1);
                    }
                }
                model.selectedItem = null;
                this.selectedRowIndex(-1);
                if (!model.isFromGantt && deletedRow.parentItem)
                    if (deletedRow.parentItem.childRecords.length == 0) {
                        deletedRow.parentItem.expanded = false;
                        deletedRow.parentItem.hasChildRecords = false;
                    }
                //Update the summary and total summary row after deleted a record.
                if (model.showSummaryRow)
                    proxy._updateSummaryRow(args);
                if (model.showTotalSummary)
                    proxy._updateTotalSummaryRow(args);
            }

            // Condition for filtering columns.
            if ((args && args.requestType == "filtering") || (model.allowFiltering && !model.isFromGantt && filteredColumns.length)) {
                //Removed the summary rows before filtering the datasource.
                if (model.showSummaryRow) {
                    model.updatedRecords = $(model.updatedRecords).not(model.summaryRowRecords).get();
                    model.flatRecords = $(model.flatRecords).not(model.summaryRowRecords).get();
                }
                if ((model.allowFiltering && !model.isFromGantt &&filteredColumns.length)) {
                    var predicate, firstFilterCondition =filteredColumns[0];
                    // Condition to collect the details of filtering column.
                    if (firstFilterCondition.value == proxy._dropDownListBlanksText) {
                        predicate = ej.Predicate(firstFilterCondition.field, "equal", null);
                        predicate = predicate["or"](firstFilterCondition.field, "equal", "");
                    }
                    else if (!firstFilterCondition.isComplex)
                        predicate = ej.Predicate(firstFilterCondition.field, firstFilterCondition.operator, firstFilterCondition.value, !firstFilterCondition.matchcase);
                    else
                        predicate = firstFilterCondition;
                    for (var i = 1; i <filteredColumns.length; i++) {
                        //Loop to gather all the column condition as array.
                        if (!filteredColumns[i].isComplex)
                            predicate = predicate[filteredColumns[i].predicate](filteredColumns[i].field,filteredColumns[i].operator,filteredColumns[i].value, !filteredColumns[i].matchcase);
                        else
                            predicate = predicate["and"](filteredColumns[i]);
                    }
                    proxy._queryManagar.where(predicate);
                    proxy._filteredRecords = model.flatRecords.slice();
                    var dataManager = new ej.DataManager(proxy._filteredRecords);
                    proxy._tempfilteredrecords = dataManager.executeLocal(proxy._queryManagar).result;
                    proxy._updateFilteredRecords(proxy._tempfilteredrecords);
                    model.updatedRecords = proxy._filteredRecords;
                    if (model.allowSorting && model.sortSettings.sortedColumns.length > 0) {
                        proxy._sortingRecords();
                    }
                } else {
                    model.updatedRecords = model.enableVirtualization ? proxy.getExpandedRecords(model.flatRecords) : model.flatRecords.slice();
                    proxy._updateHasFilteredChildRecordsStatus(model.updatedRecords);
                }

                proxy.updateCollapsedRecordCount();
                proxy.updateHeight();
                isupdatedHeight = true;
                if (model.enableAltRow) {
                    proxy.updateAltRow();
                }
            }

            if ((model.allowSearching && proxy._searchString.length) || (args && args.requestType==="searching")) {

                proxy._queryManagar.search(proxy._searchString, proxy.getColumnFieldNames(),
                    ej.FilterOperators.contains, true);

                proxy._filteredRecords = model.flatRecords.filter(function (record) {
                    return record.hasChildRecords === false;
                });

                var dataManager = new ej.DataManager(proxy._filteredRecords);
                proxy._tempfilteredrecords = dataManager.executeLocal(proxy._queryManagar).result;
                proxy._updateFilteredRecords(proxy._tempfilteredrecords);
                model.updatedRecords = proxy._filteredRecords;

                if (model.allowSorting && model.sortSettings.sortedColumns.length > 0) {
                    proxy._sortingRecords();
                }
                proxy.updateCollapsedRecordCount();
                proxy.updateHeight();
                isupdatedHeight = true;

                if (model.enableAltRow) {

                    proxy.updateAltRow();
                }

            }
            else if (model.allowSorting && sortcolumns.length > 0) {
                
                proxy._sortingRecords();
                if (!isupdatedHeight)
                {
                    proxy.updateCollapsedRecordCount();
                    proxy.updateHeight();
                    isupdatedHeight = true;
                }
            }

            if (args && (args.requestType === "outdent" || args.requestType === "indent")) {
                args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;
                isupdatedHeight = true;
            }

            if (!isupdatedHeight) {
                model.updatedRecords = model.enableVirtualization ? proxy.getExpandedRecords(model.flatRecords) : model.flatRecords.slice();
                proxy.updateCollapsedRecordCount();
                proxy.updateHeight();
            }

            if (!args) {
                args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;
            }

            proxy._updateCurrentViewData();

            if ((!model.isFromGantt && args.requestType === "save") || args.requestType === "sorting" || args.requestType === "searching"
                || args.requestType === "delete" || args.requestType === "refreshDataSource" || args.requestType === "filtering"
                || args.requestType === "dragAndDrop") {
				    proxy._isRefreshAddedRecord = false; /* flag new added record is sorted or filtered in treegrid */
                proxy.renderRecords(args);
                proxy._setScrollTop();
                proxy._$gridContent.ejScroller("refresh");
                proxy._updateScrollCss();
            }
            //update the summary and total summary row after filtering the records.
            if (!model.isFromGantt && args.requestType === "filtering") {
                if (model.showSummaryRow)
                    proxy._createSummaryRow(args);
                if (model.showTotalSummary)
                    proxy._updateTotalSummaryRow(args);
            }
            //update the summary row during sorting the records.
            if (!model.isFromGantt && args.requestType === "sorting") {
                if (model.showSummaryRow)
                    proxy._createSummaryRow(args);               
            }
        },
        //remove child record in data source on self reference
        _removeChildItem: function (record) {
            var currentRecord, proxy = this;
            for (var i = 0; i < record.childRecords.length; i++) {
                currentRecord = record.childRecords[i];
                if (this.dataSource() instanceof ej.DataManager) {
                    if (this.dataSource().dataSource.offline && this.dataSource().dataSource.json) {
                        var idx = this.dataSource().dataSource.json.indexOf(currentRecord.item);
                        if(idx!==-1)
                            this.dataSource().dataSource.json.splice(idx, 1);

                        if (currentRecord.hasChildRecords) {
                            proxy._removeChildItem(currentRecord);
                        }
                    }
                    else if (proxy._isDataManagerUpdate) {
                        var idx = proxy._jsonData.indexOf(currentRecord.item);
                        if (idx !== -1)
                            proxy._jsonData.splice(idx, 1);
                        if (currentRecord.hasChildRecords) {
                            proxy._removeChildItem(currentRecord);
                        }
                    }
                } else {
                    var idx = this.dataSource().indexOf(currentRecord.item);
                    if (idx !== -1)
                        this.dataSource().splice(idx, 1);

                    if (currentRecord.hasChildRecords) {
                        proxy._removeChildItem(currentRecord);
                    }
                }
            }
        },

        //CHECK EDITING OPTIONS 
        _processEditing:function(){

            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing || model.editSettings.allowAdding) {
                if (model.editSettings.editMode.toLowerCase() === "cellediting" || model.editSettings.editMode.toLowerCase() === "rowediting") {
                    proxy._addCellEditTemplate();
                }
            }

        },

        _toolBarClick: function (sender) {

            //Skip the mouse right click and mouse wheel
            if ((sender.event.which && (sender.event.which == 3 || sender.event.which == 2)) ||
                     (sender.event.button && sender.event.button == 2))
                return false;

            var proxy = this,
               model = proxy.model,
               $gridEle = $(this.itemsContainer).closest(".e-treegrid"),
               gridInstance = $gridEle.ejTreeGrid("instance"),
               gridId = $gridEle.attr('id'),
                toolbarObj = $("#" + gridInstance._id + "_toolbarItems");
            
            if (sender.event == undefined && sender.target.tagName == "INPUT" && sender.currentTarget.id == gridId + "_search")
                return false;

            if (gridInstance.model.dateFormat.indexOf("hh") != -1)
                $.isFunction($.fn.ejDateTimePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDateTimePicker("hide");
            else
                $.isFunction($.fn.ejDatePicker) && $("#" + gridId + "EditForm").find(".e-datepicker").ejDatePicker("hide");
            var currentTarget = sender.currentTarget; var target = sender.target;
            var args = {
                itemName: currentTarget.title,
                currentTarget: currentTarget,
                model: gridInstance.model,
            };
            if ($gridEle.ejTreeGrid("instance")._trigger("toolbarClick", args))
                return false;
            $(toolbarObj).ejToolbar('deselectItem', sender.currentTarget);
            switch (sender.currentTarget.id) {

                case gridId + "_add":
                    gridInstance._toolbarOperation(gridId + "_add", sender);
                    break;

                case gridId + "_edit":
                    gridInstance._toolbarOperation(gridId + "_edit", sender);
                    break;

                case gridId + "_delete":
                    gridInstance._toolbarOperation(gridId + "_delete", sender);
                    break;

                case gridId + "_update":
                    gridInstance._toolbarOperation(gridId + "_update", sender);
                    break;

                case gridId + "_cancel":
                    gridInstance._toolbarOperation(gridId + "_cancel", sender);
                    break;

                case gridId + "_expandAll":
                    gridInstance._toolbarOperation(gridId + "_expandAll", sender);
                    break;

                case gridId + "_collapseAll":
                    gridInstance._toolbarOperation(gridId + "_collapseAll", sender);
                    break;
                case gridId + "_pdfExport":
                    gridInstance._toolbarOperation(gridId + "_pdfExport",sender);
                    break;
            }
            return false;
        },
        _toolbarOperation: function (operation, searchEle) {
            var $gridEle = this.element, gridObject = $gridEle.ejTreeGrid("instance"), gridId = $gridEle.attr('id'), proxy = this;
            var selectedrow = this.selectedItem();
            var parentEle = searchEle.target
            var index = this.selectedRowIndex();
            gridObject._exportTo = gridObject["export"];
            switch (operation) {

                case gridId + "_add":
                    proxy._startAdd(parentEle,null);
                    break;
                    
                case gridId + "_edit":
                    proxy.updateScrollBar();
                    proxy._editRow(index);
                    break;

                case gridId + "_delete":
                    proxy.deleteRow();
                    proxy._cancelSaveTools();
                    proxy._isRefreshAddedRecord = false;/* flag new added record is sorted or filtered in treegrid */
                    break;

                case gridId + "_update":
                    editMode = proxy.model.editSettings.editMode;
                    if (editMode.toLowerCase() == "cellediting") {
                        if (proxy._isRowEdit)
                            proxy._saveRow();
                        else proxy.saveCell();
                    }
                    else if (editMode.toLowerCase() == "rowediting")
                        proxy._saveRow();
                    proxy._clearContextMenu();
                    this._removeDetailsRow();
                    $(proxy.element).focus();

                    break;

                case gridId + "_cancel":
                    var editMode = proxy.model.editSettings.editMode;
                    if (editMode.toLowerCase() == "cellediting") {
                        if (proxy._isRowEdit)
                            proxy.cancelRowEditCell();
                        else proxy.cancelEditCell();
                    }
                    else if (editMode.toLowerCase() == "rowediting")
                        proxy.cancelRowEditCell();
                    proxy._cancelSaveTools();
                    proxy._clearContextMenu();
                    this._removeDetailsRow();
                    break;

                case gridId + "_expandAll":
                    proxy._expandAll();
                    break;

                case gridId + "_collapseAll":
                    proxy.collapseAll();
                    break;
                case gridId + "_pdfExport":
                    gridObject._exportTo(gridObject.model.exportToPdfAction, 'pdfExporting', gridObject.model.allowMultipleExporting);
                    break;
            }
            proxy._refreshToolBar();
            return false;
        },
		 // Perform Tree grid context menu operations.
        _contextMenuOperations:function(option){
            var proxy = this,
                model = proxy.model,
                index = proxy._contextMenuSelectedIndex,
                e = proxy._contextMenuEvent,
                item = proxy._contextMenuSelectedItem;
            switch (option)
            {
                case "Delete":
                    proxy.deleteRow();
                    proxy._cancelSaveTools();
                    proxy._clearContextMenu();
                    break;
                case "Above":
                    proxy._subContextMenuAction(item, index, "Above");
                    $(proxy.element).focus();
                    break;
                case "Below":
                    proxy._subContextMenuAction(item, index, "Below");
                    $(proxy.element).focus();
                    break;
                case "Add":
                    proxy._setInitialData();
                    proxy._clearContextMenu();
                    break;
                case "Edit":
                    if (model.editSettings.editMode.toLowerCase() == "cellediting") {
                        proxy.element.focus();
                        proxy._cellEditingDetails.columnIndex = proxy.getCellIndex(e);
                        proxy._editedRowIndex = proxy._cellEditingDetails.rowIndex = proxy.getRowIndex(e);
                        fieldName = proxy._cellEditingDetails.columnIndex >= 0 && model.columns[proxy._cellEditingDetails.columnIndex].field;
                        if (fieldName) {
                            model.editSettings.allowEditing &&
                                model.editSettings.editMode == ej.TreeGrid.EditMode.CellEditing &&
                                proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName);
                            proxy._editAddTools();
                        }
                    }
                    else if (model.editSettings.editMode.toLowerCase() == "rowediting")
                        proxy._editRow(index);
                    proxy._clearContextMenu();
                    break;
                case "Save":
                    if (model.editSettings.editMode.toLowerCase() == "cellediting") {
                        if (proxy._isRowEdit)
                            proxy._endEdit();
                        else proxy.saveCell();
                    }
                    else if (model.editSettings.editMode.toLowerCase() == "rowediting")
                        proxy._saveRow();
                    proxy._clearContextMenu();
                    $(proxy.element).focus();
                    break;
                case "Cancel":
                    var editMode = proxy.model.editSettings.editMode;
                    if (editMode.toLowerCase() == "cellediting") {
                        if (proxy._isRowEdit)
                            proxy.cancelRowEditCell();
                        else if (model.isEdit) proxy.cancelEditCell();
                    }
                    else if (editMode.toLowerCase() == "rowediting")
                        proxy.cancelRowEditCell();
                    proxy._clearContextMenu();
                    $(proxy.element).focus();
                    break;
                default:
                    var eventArgs = {};
                    eventArgs.data = item;
                    proxy._triggerMenuEventHandler(option, eventArgs);
                    proxy._clearContextMenu();
                    break;
            }
            if (model.enableAltRow && model.currentViewData[0])
                ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);            
        },
        //Change the text based on culture information.
        _setCultureInfo: function () {
            var proxy = this,
              model = proxy.model,
              culture = model.locale,
              localization, 
              defaultLocalization = ej.TreeGrid.localization["en-US"];
            if (!model.predecessorTable)
                localization = ej.TreeGrid.localization[culture];
            else
                localization = ej.Gantt.localization[culture];
            //Tooltip Text to be displayed in the Toolbar
            proxy._toolboxTooltipTexts = (localization && localization["toolboxTooltipTexts"])?
                localization["toolboxTooltipTexts"] : defaultLocalization["toolboxTooltipTexts"];

            //Context Menu Text
            proxy._contextMenuTexts = (localization && localization["contextMenuTexts"]) ?
                localization["contextMenuTexts"] : defaultLocalization["contextMenuTexts"];

            if (proxy.model.isFromGantt)
                proxy._columnMenuTexts = proxy.model.columnMenuTexts;
            else
            {
                //Column Menu Text
                proxy._columnMenuTexts = (localization && localization["columnMenuTexts"]) ?
                    localization["columnMenuTexts"] : defaultLocalization["columnMenuTexts"];
                
                //String to be displayed in drop down list for filtering blank items
                proxy._dropDownListBlanksText = (localization && localization["dropDownListBlanksText"]) ?
                    localization["dropDownListBlanksText"] : defaultLocalization["dropDownListBlanksText"];

                //String to be displayed in drop down list to clear filtered items
                proxy._dropDownListClearText = (localization && localization["dropDownListClearText"]) ?
                    localization["dropDownListClearText"] : defaultLocalization["dropDownListClearText"];

                //Text to be displayed in drop down list for TRUE value in boolean edit type column
                proxy._trueText = (localization && localization["trueText"]) ?
                    localization["trueText"] : defaultLocalization["trueText"];

                //Text to be displayed in drop down list for FALSE value in boolean edit type column
                proxy._falseText = (localization && localization["falseText"]) ?
                    localization["falseText"] : defaultLocalization["falseText"];
                
                //EmptyRecord text to be displayed with dataSource is null
                proxy.model.emptyRecordText = (localization && localization["emptyRecord"]) ?
                    localization["emptyRecord"] : defaultLocalization["emptyRecord"];
            }
        },

        //Create the instances for newly added row.
        _createNewRowInstance: function (obj) {
            var keys = [];
            for (var key in obj) {
                keys.push(key);
            }
            return keys;
        },
		
        //Public method to pass a datasource from customer side.

        //REFRESH THE TREEGRIDCONTENT
        

        refresh:function(datasource, query){
            var proxy = this, model = proxy.model;
            proxy.resetModelCollections();
            if (!ej.isNullOrUndefined(datasource)) {
                model.dataSource = datasource;
                model.query = query;
                proxy.element.ejTreeGrid("destroy").ejTreeGrid(model);
            }
            else
                proxy.refreshContent();
        },		
        /* get index for new record */
        _getNewIndex: function () {
            this._maxRowIndex += 1;
            return (this._maxRowIndex);
        },
        //Add new record in treeGrid control
        _addRecord: function (data, rowPosition, isEdit) {
            var proxy = this;

            /*Cancel if treegrid is in edit*/
            if (proxy.model.isEdit || proxy._isRowEdit) {
                proxy.cancelRowEditCell();
            }

            var model = this.model,
                selectedRowIndex = this.selectedRowIndex(),
                selectedItem = this.selectedItem(), _cAddedRecord,
                updatedRecords = model.updatedRecords,
                flatRecords = model.flatRecords,
                parentRecords = model.parentRecords,
                columns = model.columns,
                dataSource = this.dataSource(),
                level = 0, insertIndex, validateArgs = {},
                isRecordAdded = false,
                isToolbarAdd = false,
                newIndex = this._getNewIndex(),
                parentItem, nextBelowItemInflatRecords, nextBelowItemInUpdatedRecords;

            if ($.type(dataSource) !== "array" && !(dataSource instanceof ej.DataManager)) {
                this.dataSource([]);
                dataSource = this.dataSource();
            }
            if (selectedRowIndex === -1 && (rowPosition === ej.TreeGrid.RowPosition.Above
                 || rowPosition === ej.TreeGrid.RowPosition.Below
                 || rowPosition === ej.TreeGrid.RowPosition.Child) || !rowPosition) {
                rowPosition = ej.TreeGrid.RowPosition.Top;
            }

            if (ej.isNullOrUndefined(data)) {
                var data = {};
                for (var count = 0; count < columns.length; count++) {
                    data[columns[count].field] = "";
                }
                isToolbarAdd = true;
            }
            if (data[model.childMapping] && data[model.childMapping].length)
                delete data[model.childMapping]; // Remove the child items for externally adding row

            //Clear previously selected Item
            //proxy.clearSelection()
              proxy.selectRows(-1);

            switch (rowPosition) {
                case ej.TreeGrid.RowPosition.Top:
                    level = 0;
                    parentItem = null;
                    _cAddedRecord = proxy._createRecord(data, level);
                    _cAddedRecord.index = newIndex;

                    if (model.idMapping && model.parentIdMapping) {
                        _cAddedRecord[model.idMapping] = newIndex;
                        _cAddedRecord.item[model.idMapping] = newIndex;
                    }
                    /*record update*/
                    flatRecords.splice(0, 0, _cAddedRecord);
                    updatedRecords.splice(0, 0, _cAddedRecord);
                    parentRecords.splice(0, 0, _cAddedRecord);

                    /*data source update*/
                    if (proxy._isDataManagerUpdate) {
                        proxy._jsonData.splice(0, 0, _cAddedRecord.item);
                    } else {
                        dataSource.splice(0, 0, _cAddedRecord.item);
                    }
                    insertIndex = 0;
                    break;

                case ej.TreeGrid.RowPosition.Bottom:
                    level = 0;
                    parentItem = null;
                    _cAddedRecord = proxy._createRecord(data, level);
                    _cAddedRecord.index = newIndex;
                    if (model.idMapping && model.parentIdMapping) {
                        _cAddedRecord[model.idMapping] = newIndex;
                        _cAddedRecord.item[model.idMapping] = newIndex;
                    }
                    /*record update*/
                    flatRecords.push(_cAddedRecord);
                    updatedRecords.push(_cAddedRecord);
                    //ids.push(newIndex);
                    parentRecords.push(_cAddedRecord);
                    /*data source update*/
                    if (proxy._isDataManagerUpdate) {
                        proxy._jsonData.push(_cAddedRecord.item);
                    } else {
                        dataSource.push(_cAddedRecord.item);
                    }
                    insertIndex = updatedRecords.indexOf(_cAddedRecord);
                    break;

                case ej.TreeGrid.RowPosition.Above:
                    level = selectedItem.level;
                    parentItem = selectedItem.parentItem;
                    _cAddedRecord = proxy._createRecord(data, level, parentItem);
                    _cAddedRecord.index = newIndex;
                    if (model.idMapping && model.parentIdMapping) {
                        _cAddedRecord[model.idMapping] = newIndex;
                        _cAddedRecord.item[model.idMapping] = newIndex;
                    }

                    var childIndex, recordIndex, updatedCollectionIndex;
                    /*Record Updates*/
                    recordIndex = flatRecords.indexOf(selectedItem);
                    updatedCollectionIndex = updatedRecords.indexOf(selectedItem);
                    flatRecords.splice(recordIndex, 0, _cAddedRecord);
                    updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                    //ids.splice(recordIndex, 0, newIndex);

                    if (!ej.isNullOrUndefined(parentItem)) {
                        childIndex = parentItem.childRecords.indexOf(selectedItem);
                        /*Child collection update*/
                        parentItem.childRecords.splice(childIndex, 0, _cAddedRecord);
                        if (!model.parentIdMapping) {
                            parentItem.item[model.childMapping].splice(childIndex, 0, _cAddedRecord.item);
                        }
                        else {
                            _cAddedRecord.item[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                            _cAddedRecord[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.push(_cAddedRecord.item);

                            } else {
                                dataSource.push(_cAddedRecord.item);
                            }
                        }
                    } else {
                        /* Parent records collection and data source update*/
                        parentRecords.splice(parentRecords.indexOf(selectedItem), 0, _cAddedRecord);
                        if (proxy._isDataManagerUpdate) {
                            proxy._jsonData.splice(proxy._jsonData.indexOf(selectedItem.item), 0, _cAddedRecord.item);
                        }
                        else {
                            dataSource.splice(dataSource.indexOf(selectedItem.item), 0, _cAddedRecord.item);
                        }
                    }
                    insertIndex = updatedCollectionIndex;
                    break;
                case ej.TreeGrid.RowPosition.Below:
                    level = selectedItem.level;
                    parentItem = selectedItem.parentItem;
                    _cAddedRecord = proxy._createRecord(data, level, parentItem);
                    _cAddedRecord.index = newIndex;
                    if (model.idMapping && model.parentIdMapping) {
                        _cAddedRecord[model.idMapping] = newIndex;
                        _cAddedRecord.item[model.idMapping] = newIndex;
                    }
                    //find next item position
                    var currentItemIndex = flatRecords.indexOf(selectedItem)

                    if (selectedItem.hasChildRecords) {
                        dataChildCount = proxy.getChildCount(selectedItem, 0);
                        recordIndex = currentItemIndex + dataChildCount + 1;
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + proxy._getVisibleChildRecordCount(selectedItem, 0, updatedRecords) + 1;

                    } else {
                        recordIndex = currentItemIndex + 1;
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + 1;
                    }

                    /* Record collection update */
                    flatRecords.splice(recordIndex, 0, _cAddedRecord);
                    updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                    //ids.splice(recordIndex, 0, newIndex);

                    /* data Source update */
                    if (!ej.isNullOrUndefined(parentItem)) {
                        childIndex = parentItem.childRecords.indexOf(selectedItem);
                        /*Child collection update*/
                        parentItem.childRecords.splice(childIndex + 1, 0, _cAddedRecord);
                        if (!model.parentIdMapping) {
                            parentItem.item[model.childMapping].splice(childIndex + 1, 0, _cAddedRecord.item);
                        }
                        else {
                            _cAddedRecord.item[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                            _cAddedRecord[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.push(_cAddedRecord.item);

                            } else {
                                dataSource.push(_cAddedRecord.item);
                            }
                        }
                    } else {
                        /* Parent records collection and data source update*/
                        parentRecords.splice(parentRecords.indexOf(selectedItem) + 1, 0, _cAddedRecord);
                        if (proxy._isDataManagerUpdate) {
                            proxy._jsonData.splice(proxy._jsonData.indexOf(selectedItem.item) + 1, 0, _cAddedRecord.item);
                        }
                        else {
                            dataSource.splice(dataSource.indexOf(selectedItem.item) + 1, 0, _cAddedRecord.item);
                        }
                    }
                    insertIndex = updatedCollectionIndex;
                    break;
                case ej.TreeGrid.RowPosition.Child:
                    level = selectedItem.level + 1;
                    parentItem = selectedItem;
                    _cAddedRecord = proxy._createRecord(data, level, parentItem);
                    _cAddedRecord.index = newIndex;
                    if (model.idMapping && model.parentIdMapping) {
                        _cAddedRecord[model.idMapping] = newIndex;
                        _cAddedRecord.item[model.idMapping] = newIndex;
                    }
                    //find next item position
                    var currentItemIndex = flatRecords.indexOf(selectedItem)

                    if (selectedItem.hasChildRecords) {
                        dataChildCount = proxy.getChildCount(selectedItem, 0);
                        recordIndex = currentItemIndex + dataChildCount + 1;
                        //Expand Add record's parent item 
                        if (!selectedItem.expanded) {
                            var expanargs = {};
                            expanargs.expanded = true;
                            expanargs.data = selectedItem;
                            proxy._isInAdd = true;
                            /* refresh Added row on expand collapse */
                            if (proxy._isRefreshAddedRecord) {
                                expanargs.data.expanded = true;
                                proxy.updateExpandStatus(args.data, args.expanded);
                                proxy.refreshContent();
                                proxy._isRefreshAddedRecord = false;
                            } else {
                                ej.TreeGrid.sendExpandCollapseRequest(proxy, expanargs);
                            }
                            proxy._isInAdd = false;
                            updatedRecords = proxy.getUpdatedRecords();
                        }
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + proxy._getVisibleChildRecordCount(selectedItem, 0, updatedRecords) + 1;

                    } else {
                        selectedItem.hasChildRecords = true;
                        selectedItem.childRecords = [];
                        selectedItem.expanded = true;
                        if (!model.parentIdMapping) {
                            selectedItem.item[model.childMapping] = [];
                        }
                        selectedItem.isMilestone = false;
                        recordIndex = currentItemIndex + 1;
                        updatedCollectionIndex = updatedRecords.indexOf(selectedItem) + 1;
                    }

                    /* Record collection update */
                    flatRecords.splice(recordIndex, 0, _cAddedRecord);
                    updatedRecords.splice(updatedCollectionIndex, 0, _cAddedRecord);
                    //ids.splice(recordIndex, 0, newIndex);

                    /* data Source update */
                    if (!ej.isNullOrUndefined(parentItem)) {
                        childIndex = parentItem.childRecords.indexOf(selectedItem);
                        /*Child collection update*/
                        parentItem.childRecords.splice(childIndex + 1, 0, _cAddedRecord);
                        if (!model.parentIdMapping) {
                            parentItem.item[model.childMapping].splice(childIndex + 1, 0, _cAddedRecord.item);
                        }
                        else {
                            _cAddedRecord.item[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                            _cAddedRecord[model.parentIdMapping] = _cAddedRecord.parentItem.item[model.idMapping];
                            if (proxy._isDataManagerUpdate) {
                                proxy._jsonData.push(_cAddedRecord.item);
                            } else {
                                dataSource.push(_cAddedRecord.item);
                            }
                        }
                    }
                    insertIndex = updatedCollectionIndex;
                    break;
            }

            if (!isToolbarAdd && (ej.TreeGrid.RowPosition.Above || ej.TreeGrid.RowPosition.Below)) {
                var args = {},
                    currentValue = {},
                    columns = model.columns,
                    length = columns.length;

                args.rowIndex = insertIndex;
                args.requestType = "addNewRow";
                args.addedRow = data;

                if (!proxy._trigger("actionComplete", args)) {
                    proxy._renderAddedRow(insertIndex, _cAddedRecord);
                    if (model.showSummaryRow)
                        proxy._updateSummaryRow(args);
                    if (model.showTotalSummary)
                        proxy._updateTotalSummaryRow(args);
                    isRecordAdded = true;
                }
                else {
                    updatedRecords.splice(insertIndex, 1);
                }
            }
            else {
                proxy._renderAddedRow(insertIndex, _cAddedRecord);
                isRecordAdded = true;
            }

            /*Set flag for refresh newly added record in non virtalization mode*/
            if (model.enableVirtualization === false &&
                (model.sortSettings.sortedColumns.length > 0 || model.filterSettings.filteredColumns.length > 0)) {
                proxy._isRefreshAddedRecord = true;
            }
            /* select add row */
            if (model.allowSelection) {
                var currentRowIndex = isRecordAdded ? updatedRecords.indexOf(_cAddedRecord) : selectedRowIndex;
                if (!proxy._rowSelectingEventTrigger(this._previousIndex, currentRowIndex)) {
                    proxy.selectRows(currentRowIndex);
                    proxy._rowSelectedEventTrigger(currentRowIndex);
                }
            }
            /* scroll to new added record */
            proxy.updateScrollBar();
           
            if (isEdit == "isEditRow") {
                proxy._editRow(updatedRecords.indexOf(_cAddedRecord), _cAddedRecord);
            }
            else {
                ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(_cAddedRecord));
                proxy._cancelSaveTools();
                if (model.enableAltRow)
                    ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);
            }
            //refresh parent item for expand collapse icons
            if (_cAddedRecord.parentItem && rowPosition == ej.TreeGrid.RowPosition.Child && (_cAddedRecord.parentItem.childRecords.length == 1 || !_cAddedRecord.parentItem.hasFilteredChildRecords)) {
                _cAddedRecord.parentItem.hasFilteredChildRecords = true;
                ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(_cAddedRecord.parentItem));
            }
        },


         
         
        addRow: function (data, rowPosition) {
            var proxy = this, model = proxy.model;

            if (!data) {
                proxy._addRecord(null, rowPosition, "isEditRow");
            } else {
                proxy._addRecord(data, rowPosition);
            } 
        },

        _startAdd: function (parentEle, rowPos) {
            var proxy = this, model = this.model, rowPosition;
            proxy._addRecord(null, model.editSettings.rowPosition, "isEditRow");
        },
        _getRecordIndexByItem: function (item, collection) {
            var record = ej.DataManager(collection).executeLocal(ej.Query().where("item", ej.FilterOperators.equal, item));
            if (record.length > 0)
                return collection.indexOf(record[0]);
            else
                return -1;
        },
        /* update the reord's expand status in expanded state*/
        _expandRecord: function (record)
        {
            var parentItem = record.parentItem, args = {};
            while (parentItem != null) {
                if (parentItem.expanded == false) {
                    args.expanded = true;
                    args.data = parentItem;
                    if (!parentItem.parentItem)
                        this._isInExpandCollapseAll = false;
                    else
                        this._isInExpandCollapseAll = true;
                    ej.TreeGrid.sendExpandCollapseRequest(this, args);
                }
                parentItem = parentItem.parentItem;
            }
        },

        /* Public method for scroll to the corresponding offset value */
        scrollOffset: function (left, top) {
            var scrollerObj = this._$gridContent.data("ejScroller"),
                model = this.model;
            if (model.isEdit)
                this.saveCell();
            else if (this._isRowEdit)
                this._saveRow();
            if ((typeof top == "number" || typeof parseInt(top) == "number")) {
                top = parseInt(top);
                if(top >= 0 && this.getMaxScrollHeight() > top)
                    scrollerObj.scrollY(top, true);
            }
            if (typeof left == "number" || typeof parseInt(left) == "number") {
                left = parseInt(left);
                if (left >= 0 && this.getMaxScrollWidth() > left)
                    scrollerObj.scrollX(left, true);
            } 
        },
        /* Public method for get scrollTop value*/
        getScrollTopOffset: function () {
            return this._$gridContent.ejScroller("model.scrollTop");
        },
        /* Public method for get scrollLeft value*/
        getScrollLeftOffset: function () {
            return this._$gridContent.ejScroller("model.scrollLeft");
        },
        /* Public method for scroll to first row of treegrid*/
        scrollToTop:function()
        {
            var model = this.model,
                updatedRecords, record, rowIndex = -1;
            if (model.isEdit)
                this.saveCell();
            else if (this._isRowEdit)
                this._saveRow();
            updatedRecords = this.getExpandedRecords(model.updatedRecords);
            if (updatedRecords.length > 0) {
                rowIndex = 0;
                record = updatedRecords[0];
                rowIndex = model.updatedRecords.indexOf(record);
                if (model.allowSelection && !this._rowSelectingEventTrigger(this.selectedRowIndex(), rowIndex)) {
                    this.selectRows(rowIndex);
                    this._rowSelectedEventTrigger(rowIndex);
                }
                this.updateScrollBar(rowIndex);
            }
        },
        /* Public method for scroll to last row of treegrid*/
        scrollToBottom: function ()
        {
            var model = this.model,
                updatedRecords, record, rowIndex = -1;

            if (model.isEdit)
                this.saveCell();
            else if (this._isRowEdit)
                this._saveRow();
            updatedRecords = this.getExpandedRecords(model.updatedRecords);

            if (updatedRecords.length > 0) {
                rowIndex = updatedRecords.length - 1;
                record = updatedRecords[rowIndex];
                rowIndex = model.updatedRecords.indexOf(record);
                if (model.allowSelection && !this._rowSelectingEventTrigger(this.selectedRowIndex(), rowIndex)) {
                    this.selectRows(rowIndex);
                    this._rowSelectedEventTrigger(rowIndex);
                }
                this.updateScrollBar(rowIndex);
            }
        },

        //ENABLE AND DISABLE THE CORRESPONDING TOOLS WHILE ADDING AND EDITING THE ROW RECORD.
        _editAddTools: function () {
            var proxy = this, model = proxy.model;
            var toolbar = $("#" + proxy._id + "_toolbarItems");
            var editSettings = proxy.model.editSettings;
            toolbarItems = model.toolbarSettings.toolbarItems;
            var disableToolItems = [];
            if (editSettings.allowAdding && toolbarItems.indexOf("add") !== -1 && !$(toolbar).find(".e-addnew").parent("li").hasClass("e-disable"))
                disableToolItems.push($(toolbar).find(".e-addnew").parent()[0]);
            if (editSettings.allowEditing && toolbarItems.indexOf("edit") !== -1 && !$(toolbar).find(".e-edit").parent("li").hasClass("e-disable"))
                disableToolItems.push($(toolbar).find(".e-edit").parent()[0]);
            if (editSettings.allowDeleting && toolbarItems.indexOf("delete") !== -1 && !$(toolbar).find(".e-delete").parent("li").hasClass("e-disable"))
                disableToolItems.push($(toolbar).find(".e-delete").parent()[0]);
            $(toolbar).ejToolbar('disableItem', disableToolItems);

            var enabledToolItems = [];
            if (toolbarItems.indexOf("cancel") !== -1)
                enabledToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
            if (toolbarItems.indexOf("update") !== -1)
                enabledToolItems.push($(toolbar).find(".e-save").parent()[0]);
            $(toolbar).ejToolbar('enableItem', enabledToolItems);

        },
        //ENABLE AND DISABLE THE CORRESPONDING TOOLS WHILE CANCEL AND SAVING THE ROW RECORD.
        _cancelSaveTools: function () {
            var proxy = this;
                model = proxy.model,
                editSettings = proxy.model.editSettings,
                toolbar = $("#" + proxy._id + "_toolbarItems"),
                parentRecords = [],
                toolbarItems = model.toolbarSettings.toolbarItems;
            if (!model.isFromGantt) {
                var enabledToolItems = [];
                var disableToolItems = [];
                if (editSettings.allowAdding) {
                    if (toolbarItems.indexOf("add") !== -1)
                        enabledToolItems.push($(toolbar).find(".e-addnew").parent()[0]);
                }
                
                if (editSettings.allowEditing && toolbarItems.indexOf("edit") !== -1) {
                    if (proxy.selectedRowIndex() != -1)
                        enabledToolItems.push($(toolbar).find(".e-edit").parent()[0]);
                    else if (!$(toolbar).find(".e-edit").parent("li").hasClass("e-disable"))
                        disableToolItems.push($(toolbar).find(".e-edit").parent()[0]);
                }
                if (editSettings.allowDeleting && toolbarItems.indexOf("delete") !== -1) {                   
                        if (proxy.selectedRowIndex() != -1)
                            enabledToolItems.push($(toolbar).find(".e-delete").parent()[0]);
                        else if (!$(toolbar).find(".e-delete").parent("li").hasClass("e-disable"))
                            disableToolItems.push($(toolbar).find(".e-delete").parent()[0]);
                }

                parentRecords = proxy._getParentRecords(model.updatedRecords, proxy);
               
                if (parentRecords.length <= 0) {
                    if (toolbarItems.indexOf("expandAll") !== -1 && !$(toolbar).find(".e-expandall").parent("li").hasClass("e-disable"))
                        disableToolItems.push($(toolbar).find(".e-expandall").parent()[0]);
                    if (toolbarItems.indexOf("collapseAll") !== -1 && !$(toolbar).find(".e-collapseall").parent("li").hasClass("e-disable"))
                        disableToolItems.push($(toolbar).find(".e-collapseall").parent()[0]);
                }
                else {
                    if (toolbarItems.indexOf("expandAll") !== -1)
                        enabledToolItems.push($(toolbar).find(".e-expandall").parent()[0]);
                    if (toolbarItems.indexOf("collapseAll") !== -1)
                        enabledToolItems.push($(toolbar).find(".e-collapseall").parent()[0]);
                }

                if (toolbarItems.indexOf("cancel") !== -1 && !$(toolbar).find(".e-cancel").parent("li").hasClass("e-disable"))
                    disableToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
                if (toolbarItems.indexOf("update") !== -1 && !$(toolbar).find(".e-save").parent("li").hasClass("e-disable"))
                    disableToolItems.push($(toolbar).find(".e-save").parent()[0]);

                $(toolbar).ejToolbar('disableItem', disableToolItems);
                $(toolbar).ejToolbar('enableItem', enabledToolItems);
            }
            else {
                var toolbar = $("#" + proxy._id.replace("ejTreeGrid", "") + "_toolbarItems");
                var enabledToolItems = [];
                if (editSettings.allowAdding && toolbarItems.indexOf("add") !== -1)
                    enabledToolItems.push($(toolbar).find(".e-addnewitem").parent()[0]);
                
                if (editSettings.allowEditing && toolbarItems.indexOf("edit") !== -1) {
                    if (proxy.selectedRowIndex() != -1)
                        enabledToolItems.push($(toolbar).find(".e-edititem").parent()[0]);
                }
                if (editSettings.allowDeleting && toolbarItems.indexOf("delete") !== -1) {
                    if (proxy.selectedRowIndex() != -1)
                        enabledToolItems.push($(toolbar).find(".e-deleteitem").parent()[0]);
                }
                $(toolbar).ejToolbar('enableItem', enabledToolItems);

                var disableToolItems = [];
                if (toolbarItems.indexOf("cancel") !== -1)
                    disableToolItems.push($(toolbar).find(".e-cancel").parent()[0]);
                if (toolbarItems.indexOf("update") !== -1)
                    disableToolItems.push($(toolbar).find(".e-saveitem").parent()[0]);

                $(toolbar).ejToolbar('disableItem', disableToolItems);
            }

        },

        
        _checkIsEmptyRow:function(item){
            for (var key in item) {
                var val = item[key];
                if (val === " " || val === null || val === "" || val === false)
                    continue;
                    return false;                
            }
            return true;
        },
              
        _beginEdit: function () {
            var proxy = this;
            proxy._trigger("beginEdit", args);
            proxy._isinBeginEdit = true;           
        },

        _endEdit: function () {
            var proxy = this;

            proxy._isinBeginEdit = false;
            var index = this.selectedRowIndex();
            var tr = ej.TreeGrid.getRowByIndex(proxy, index);
            if ($('tr').hasClass("e-addedrow") || $('tr').hasClass("e-rowedit"))
                proxy._saveRow();
            else
                proxy.saveCell();

        },
        //Refresh the treegrid content after expandAll or collapseAll
        _refreshTreeGridOnExpandCollapseAll:function(args)
        {
            var proxy = this,
                model = this.model;
            if (model.enableVirtualization) {
                if (model.enableAltRow)
                    proxy.updateAltRow();
                proxy.sendDataRenderingRequest(args);
                proxy._setScrollTop();
                proxy.updateHeight();
            } else {
                //If need refresh chart if record is added after searching or sorting
                if (proxy._isRefreshAddedRecord) {
                    proxy.processBindings();
                    this.renderRecords();
                    proxy._isRefreshAddedRecord = false;
                    return;
                }
                proxy.updateHeight();
                if (model.enableAltRow && model.currentViewData.length > 0)
                    ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);
            }
        },
        //expand or collapse all inner level child records on expandAll or collapseAll
        _expandCollapseInnerLevelRecord: function (record, expanded) {
            var proxy = this,
                length = record.childRecords.length,
                tempArgs = {};

            for (var count = 0; count < length; count++) {
                if (record.childRecords[count].hasChildRecords) {
                    proxy._expandCollapseInnerLevelRecord(record.childRecords[count], expanded);
                    tempArgs.data = record.childRecords[count];
                    tempArgs.expanded = expanded;
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, tempArgs);
                }
            }
        },
        _expandParentLevelRecord: function (record, expanded) {
            var proxy = this,               
                tempArgs = {};
            var parentExpandState = proxy.getExpandStatus(treegridRecord);
            if (!record.expanded || !parentExpandState) {
                tempArgs.data = record;
                tempArgs.expanded = expanded;
                ej.TreeGrid.sendExpandCollapseRequest(proxy, tempArgs);
                if (tempArgs.data.parentItem) {
                    proxy._expandParentLevelRecord(tempArgs.data.parentItem, expanded);
                }
            }
        },
        // expand the specified level record
        expandAtLevel: function (index) {
            var proxy = this, args = {};
            var parentRecords = proxy._getParentRecords(model.updatedRecords, proxy);
            for (count = 0; count < parentRecords.length; count++) {
                treegridRecord = parentRecords[count];
                if (treegridRecord.level == index) {
                    args.data = treegridRecord;
                    args.expanded = true;
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    // expand its parent if it is collapsed.
                    if (treegridRecord.parentItem) {
                        proxy._expandParentLevelRecord(treegridRecord.parentItem, args.expanded);
                    }
                    
                }
            }
        },
        // collapse the specified level record
        collapseAtLevel: function (index) {
            var proxy = this, args = {};
            var parentRecords = proxy._getParentRecords(model.updatedRecords, proxy);
            for (count = 0; count < parentRecords.length; count++) {
                treegridRecord = parentRecords[count];                
                if (treegridRecord.level == index && treegridRecord.expanded) {
                    var parentExpandState = proxy.getExpandStatus(treegridRecord);
                    if (parentExpandState) {
                        args.data = treegridRecord;
                        args.expanded = false;
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    }
                }
            }
        },
        //Expand the child records from parent.
        _expandAll: function () {

            var proxy = this,
                model = proxy.model,
                count = 0,
                treegridRecord,
                args = {};

            if (proxy._isRowEdit || proxy.model.isEdit) {
                proxy.cancelRowEditCell();
            }
            args.requestType = ej.TreeGrid.Actions.ExpandCollapse;

            proxy._isInExpandCollapseAll = true;
            //loop excecute for iterate the GanttRecords preseent in the Gantt Control

            for (count = 0; count < proxy.model.parentRecords.length; count++) {
                treegridRecord = proxy.model.parentRecords[count];
                args.data = treegridRecord;
                args.recordIndex = count;
                args.expanded = true;
                if (treegridRecord.hasChildRecords) {
                    proxy._expandCollapseInnerLevelRecord(treegridRecord, args.expanded);
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                }
            }
            if (model.flatRecords.length > 0)
                proxy._refreshTreeGridOnExpandCollapseAll(args);

            proxy._isInExpandCollapseAll = false;
        },

           

        _refreshToolBar: function () {
            var $toolbar = $("#" + this._id + "_toolbarItems");
            $toolbar.find("li").removeClass("e-hover");
        },

        //RENDER TOOL BAR
        _renderToolbar: function () {
            var proxy = this,
            model = proxy.model;
            if (!model.isFromGantt) {
                var proxy = this, element = proxy.element, model = proxy.model;
                if (proxy.model.toolbarSettings.showToolbar) {
                    if (proxy.model.toolbarSettings.toolbarItems.length == 0)
                        proxy.model.toolbarSettings.toolbarItems = [ej.TreeGrid.ToolbarItems.Add,
                        ej.TreeGrid.ToolbarItems.Edit, ];
                    element.append(proxy._renderToolbarTemplate());
                }
            }
        },

        //RENDER TOOLBAR TEMPLATE
        _renderToolbarTemplate: function () {
            var proxy = this,
                model = proxy.model,
                $div = ej.buildTag("div.e-treegridtoolbar#" + proxy._id + "_toolbarItems", "", { 'height': '36px', 'border': '1px solid' }, { "unselectable": "on" }),
                $ul = ej.buildTag("ul", "", { 'width': proxy._gridWidth, 'box-sizing': ' ' }, { "unselectable": "on" });
            (!ej.isNullOrUndefined(model.toolbarSettings.toolbarItems) && model.toolbarSettings.toolbarItems.length) && proxy._renderLi($ul);
            $div.append($ul);
            var $customUl = ej.buildTag("ul", "", {}, { "unselectable": "on" });
            $div.append($customUl);
            var helper = {};
            helper.click = proxy._toolBarClick;
            helper.cssClass = "";
            helper.rtl = false;
            helper.itemSeparator = false;
            helper.itemLeave = proxy._toolbarItemLeave;
            helper.width= proxy._gridWidth,
            helper.fields = {
                id: "",
                tooltipText: "",
                imageUrl: "",
                text: "",
                imageAttributes: "",
                spriteCSS: "",
                htmlAttributes: "",
            };
            $div.ejToolbar(helper);
            $div.ejToolbar("disableItem", proxy._disabledToolItems);
            proxy._disabledToolItems = $();
            return $div;
        },
        //Render list to display the tools in toolbar.
        _renderLi: function ($ul) {
            var proxy = this,
                toolBarItems = proxy.model.toolbarSettings.toolbarItems,
                length = toolBarItems.length,
                i = 0,
                $li;
            for (i; i < length; i++) {

                $li = ej.buildTag("li", "", {}, {
                    id: proxy._id + "_" + toolBarItems[i],
                    "tabindex": "0"
                });

                if (toolBarItems[i] !== "search")
                    $li.css({ 'width': '30px', 'height': '24px', });

                proxy._renderLiContent($li, toolBarItems[i]);
                $ul.append($li);
            }
        },

        //RENDER TOOLBAR ICONS
        _renderLiContent: function ($li, item) {
            var $a, $input;
            var selectedRowIndex = this.selectedRowIndex();
            var editSetting = this.model.editSettings;
            switch (item) {

                case "add":
                    $a = ej.buildTag("span.e-addnew e-toolbaricons e-icon e-treegrid-add", "", {});
                    var addTitle = this._toolboxTooltipTexts["addTool"];
                    $li.attr("title", addTitle);
                    if (!this.model.editSettings.allowAdding)
                        this._disabledToolItems.push($li.get(0));
                    break;

                case "edit":
                    $a = ej.buildTag("span.e-edit e-toolbaricons e-icon e-treegrid-edit", "", {});
                    var editTitle = this._toolboxTooltipTexts["editTool"];
                    $li.attr("title", editTitle);
                    if (!this.model.editSettings.allowEditing || selectedRowIndex == -1) {
                        this._disabledToolItems.push($li.get(0));
                    }

                    break;

                case "delete":
                    var deleteTitle = this._toolboxTooltipTexts["deleteTool"];
                    $li.attr("title", deleteTitle);
                    $a = ej.buildTag("span.e-delete e-toolbaricons e-icon e-treegrid-delete", "", {});

                    if (!this.model.editSettings.allowDeleting || selectedRowIndex == -1) {
                        this._disabledToolItems.push($li.get(0));
                    }

                    break;

                case "update":
                    var updateTitle = this._toolboxTooltipTexts["updateTool"];
                    $li.attr("title", updateTitle);
                    $a = ej.buildTag("span.e-save e-toolbaricons e-icon e-treegrid-save e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "cancel":
                    var cancelTitle = this._toolboxTooltipTexts["cancelTool"];
                    $li.attr("title", cancelTitle);
                    $a = ej.buildTag("span.e-cancel e-toolbaricons e-icon e-treegrid-cancel e-disabletool", "", {});
                    this._disabledToolItems.push($li.get(0));
                    break;

                case "expandAll":
                    var expandAllTitle = this._toolboxTooltipTexts["expandAllTool"];
                    $li.attr("title", expandAllTitle);
                    $input = ej.buildTag("span.e-expandall e-toolbaricons e-icon e-treegrid-expandall", "", {}, {});
                    $li.append($input);
                    break;

                case "collapseAll":
                    var collapseAllTitle = this._toolboxTooltipTexts["collapseAllTool"];
                    $li.attr("title", collapseAllTitle);
                    $input = ej.buildTag("span.e-collapseall e-toolbaricons e-icon e-treegrid-collapseall", "", {}, {});
                    $li.append($input);
                    break;

                case "pdfExport":
                    var pdfExportTitle = this._toolboxTooltipTexts["pdfExportTool"];
                    $li.attr("title", pdfExportTitle);
                    $input = ej.buildTag("span.e-pdfIcon e-toolbaricons e-icon", "", {}, {});
                    $li.append($input);
                    break;
            }

            $li.append($a);
        },

        //update the toolbar items when dynamically update the editSettings property
        _updateToolbarItems: function () {
            var proxy = this,
                model = proxy.model,
                toolbarItems = model.toolbarSettings.toolbarItems,
                editSettings = model.editSettings,
                toolbar = $("#" + proxy._id + "_toolbarItems"),
                selectedRowIndex = proxy.selectedRowIndex(),
                editMode = editSettings.editMode;
                disableToolItems = [],
                enableToolItems = [], toolbarItems = model.toolbarSettings.toolbarItems;
                if (editMode.toLowerCase() == "cellediting") {
                    /* We can able perform rowEditing using toolbar, whether the editMode also in cellEditing.
                       So cancel the editing based on edit type.*/
                    if (proxy._isRowEdit)
                        proxy.cancelRowEditCell();
                    else if (model.isEdit)
                        proxy.cancelEditCell();
                }
                else if (editMode.toLowerCase() == "rowediting")
                    proxy.cancelRowEditCell();
                proxy._cancelSaveTools();
                if (toolbarItems.indexOf(ej.TreeGrid.ToolbarItems.Add) != -1) {
                    var addTool = $(toolbar).find(".e-addnew");
                    if (editSettings.allowAdding) {
                        if (addTool.parent("li").hasClass("e-disable"))
                            enableToolItems.push(addTool.parent()[0]);
                    }
                    else {
                        if (!addTool.parent("li").hasClass("e-disable"))
                            disableToolItems.push(addTool.parent()[0]);
                    }
                }
                if (toolbarItems.indexOf(ej.TreeGrid.ToolbarItems.Edit) != -1) {
                    var editTool = $(toolbar).find(".e-edit");
                    if (editSettings.allowEditing) {
                        if (editTool.parent("li").hasClass("e-disable") && selectedRowIndex > 0)
                            enableToolItems.push(editTool.parent()[0]);
                    }
                    else {
                        if (!editTool.parent("li").hasClass("e-disable"))
                            disableToolItems.push(editTool.parent()[0]);
                    }
                }
                if (toolbarItems.indexOf(ej.TreeGrid.ToolbarItems.Delete) != -1) {
                    var deleteTool = $(toolbar).find(".e-delete");
                    if (editSettings.allowDeleting) {
                        if (deleteTool.parent("li").hasClass("e-disable"))
                            enableToolItems.push(deleteTool.parent()[0]);
                    }
                    else {
                        if (!deleteTool.parent("li").hasClass("e-disable"))
                            disableToolItems.push(deleteTool.parent()[0]);
                    }
                }
                enableToolItems.length && $(toolbar).ejToolbar('enableItem', enableToolItems);
                disableToolItems.length && $(toolbar).ejToolbar('disableItem', disableToolItems);
        },


        //#region TREEGRIDHEADER RENDERIGN

        //TREEGRID HEADER RENDER INITIALIZATION
        _renderAfterColumnInitialize:function(){
            var proxy = this;
            proxy.element.append(proxy._renderGridHeader());
        },

        //TO CHECK FILTER FORM ELEMENTS EDIT TYPE
        filterEditType: function (column, element, j) {

            var proxy = this,
                model = proxy.model;
               
            if (ej.isNullOrUndefined(column["filterEditType"]))
                column["filterEditType"] = column["editType"];

            switch (column["filterEditType"]) {
                case "stringedit":
                    element.html(ej.buildTag('input.e-ejinputtext e-filtertext e-field e-filterwidth', "", {}, { id: proxy._id + "_" + column.field + "_filterbarcell", name: column.field }));
                    break;
                case "booleanedit":
                    element.html('<input class="e-field e-checkbox e-filtertext" type ="checkbox" id=' + proxy._id + "_" + column.field + "_filterbarcell" + ' name=' + column.field + ' />');
                    break;
                case "numericedit":
                    var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", { "width": column.width - 15 }, { type: "text", id: proxy._id + "_" + column.field + "_filterbarcell", name: column.field });
                    element.append($numericText);
                    break;
                case "datepicker":
                case "datetimepicker":
                    var $datePicker = ej.buildTag('input.e-' + column["filterEditType"] + ' e-field', "", {}, { type: "text", id: proxy._id + "_" + column.field + "_filterbarcell", name: column.field });
                    element.append($datePicker);
                    break;
                case "dropdownedit":
                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist e-field', "", {}, { type: "text", id: proxy._id + "_" + column.field + "_filterbarcell", name: column.field });
                    element.append($dropDownList);
                    break;
            }
        },


        //RENDER FILTER BAR ELEMENTS ACCORDING TO ITS TYPE
        _filteringElements: function (args) {
            var proxy = this,
                model=proxy.model,
                elementFocused = false,
                $formElement,
                form, length,
                $element,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat,
                formatVal;
            selectedItem = model.selectedItem;

            if (args.requestType == "filtering") {
                form = document.getElementById(proxy._id + "_filterBar" + args.fieldname);
            }
            
            var column = customParams = ej.TreeGrid.getColumnByField(model.columns,args.fieldname)
            $formElement = $(form).find("input,select");
            length = $formElement.length;
            for (var i=0; i < length; i++) {
                $element = $formElement.eq(i);
                params = {};
                //inputWidth = column.width ? column.width - 2 : proxy.model.commonWidth - 2;
                inputWidth = $element.closest("div").width()-2;
                if ($element.hasClass("e-numerictextbox")) {
                    width = inputWidth;
                    value = $element.val();
                    params.width = width;                    
                    params.showSpinButton = true;
                    params.cssClass = model.cssClass;
                    params.height = 28;
                    params.focusIn = $.proxy(proxy._cancelEditBoxes, proxy, $element);
                    //for parent editing 
                    var tempId = $element[0].id;
                    tempId=tempId.replace(proxy._id, '');
                    if (model.filterSettings.filterBarMode == "immediate")
                        params.change = $.proxy(proxy._filterBarHandler, proxy, $element);
                    else
                        params.change = null;
                    if (value.length)
                        params.value = parseFloat(value);
                    else
                        params.value = null;
                    customParams = ej.TreeGrid.getColumnByField(proxy.model.columns, $element.prop("name"));
                    if (!ej.isNullOrUndefined(customParams) && !ej.isNullOrUndefined(customParams["editParams"]))
                        $.extend(params, customParams["editParams"]);
                    $element.ejNumericTextbox(params);
                    $element.prop("name", $element.prop("name").replace(proxy._id, ""));
                }
                else if ($element.hasClass("e-datepicker")) {
                    width = inputWidth;
                    params.width = width;                    
                    params.cssClass = model.cssClass;
                    params.dateFormat = model.dateFormat;
                    params.locale = model.locale;
                    params.height = 28;
                    params.focusIn = $.proxy(proxy._cancelEditBoxes, proxy, $element);
                    params.open = $.proxy(proxy._popupHide, proxy, $element);
                    var tempId = $element[0].id;
                    tempId = tempId.replace(proxy._id, '');
                    //To filter the records once the date seleted.
                    if (model.filterSettings.filterBarMode == "immediate")
                        params.select = $.proxy(proxy._filterBarHandler, proxy, $element);
                    else
                        params.select = null;
                    column = ej.TreeGrid.getColumnByField(proxy.model.columns, $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);

                    if (model.dateFormat.indexOf("hh") != -1)
                        $element.ejDateTimePicker(params);
                    else
                        $element.ejDatePicker(params);
                    proxy._preventDoubleClick(column); //To prevent double click event
                }
                else if ($element.hasClass("e-datetimepicker")) {
                    width = inputWidth;
                    params = {
                        width: width,
                        rtl: model.rtl,
                        locale:model.locale,
                        cssClass: model.cssClass,
                        showButton: false,
                        focusIn : $.proxy(proxy._cancelEditBoxes, proxy, $element),
                        open : $.proxy(proxy._popupHide, proxy, $element)
                    };
                    params.height = 28;
                    column = ej.TreeGrid.getColumnByField(proxy.model.columns, $element.prop("name"));
                    if (column["format"] !== undefined && column.format.length > 0) {
                        toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                        formatVal = toformat.exec(column.format);
                        params.dateTimeFormat = formatVal[2];
                    }
                    if (!ej.isNullOrUndefined(column["editParams"]))
                        $.extend(params, column["editParams"]);

                    $element.ejDateTimePicker(params);
                    proxy._preventDoubleClick(column); //To prevent double click event

                }
                else if ($element.hasClass("e-dropdownlist")) {
                    column = ej.TreeGrid.getColumnByField(model.columns, args.fieldname);
                    var clear, emptyString, dropData, blank, selectAll;
                    //Collecting the drop down data for drop down filter bar
                    if (column.dropdownData && column.dropdownData.length > 0)
                        dropData = $.extend(true, [], column.dropdownData);
                    else {
                        //Default drop down data for boolean editType columns
                        if (column.editType == "booleanedit")
                            dropData = [
                                { text: proxy._trueText, value: "true" },
                                { text: proxy._falseText, value: "false"},
                            ];
                        else
                            dropData = proxy._dropDownArrayCollection(column, args.fieldname);  //Gathering dropdown data from datasource                       
                    }
                    //To append selectAll(clear) label in the drop down list                   
                    clear = { text: proxy._dropDownListClearText, value: "" };
                        dropData.splice(0, 0, clear);
                    //To append (Blanks) label in the drop down list
                        blank = column.allowFilteringBlankContent == undefined ? true : column.allowFilteringBlankContent;
                    if (blank) {
                        emptyString = { text: proxy._dropDownListBlanksText, value: proxy._dropDownListBlanksText };
                        dropData.push(emptyString);
                    }
                    var param = {}, inputValue = $element.val();
                    if ($element.hasClass("e-input")) {
                        var dropObject = $("#" + $element[0].id).data("ejDropDownList"),
                            $dropDownObj = $(dropObject.popupList),
                            dropVal = dropObject._selectedValue;
                    }
                    
                    param.width = inputWidth;
                    param.showCheckbox = false;
                    param.dataSource = dropData;
                    param.height = 28;
                    param.popupShown = $.proxy(proxy._cancelEditBoxes, proxy, $element);
                    param.fields = { text: "text", value: "value" };
                    //To filter the records once the item has been selected.
                    if (model.filterSettings.filterBarMode == "immediate")
                        param.popupHide = $.proxy(proxy._filterBarHandler, proxy, $element);
                    else
                        param.popupHide = null;
                    $element.ejDropDownList(param);
                    //Apply italic font style for "(Clear Filter)" and "(Blanks)" label
                    if ($dropDownObj) {
                        $dropDownObj.find(".e-ul :first-child").css("font-style", "italic");
                        if(blank)
                            $dropDownObj.find(".e-ul :last-child").css("font-style", "italic");
                    }
                    if (inputValue == proxy._dropDownListClearText)
                        inputValue = "";
                    $element.ejDropDownList("setSelectedText", inputValue);//To retain the selected value in dropdown    
                    proxy._preventDoubleClick(column); //To prevent double click event                 
                } else {
                    switch ($element.prop('tagName')) {
                        case "INPUT":
                            $element.width(inputWidth - 4);
                            if (column.filterEditType == "stringedit")
                                $("#" + proxy._id).on("mouseup", "#" + proxy._id + "_" + column.field + "_filterbarcell", function (e) {
                                    proxy._cancelEditBoxes(e); //To cancel the edit boxes while clicking string edit filter box
                                    e.stopImmediatePropagation();
                                });
                            break;

                        case "SELECT":
                            $element.width(inputWidth).height(15);
                            break;
                    }
                }
                if (!$element.is(":disabled") && !elementFocused && (!$element.is(":hidden") || typeof $element.data("ejDropDownList") == "object")) {
                    if (!proxy._isEnterKeyPressed) {
                        elementFocused = true;
                    }
                }
            }
        },

        // Method for retriving drop down data to filterbar from data source
        _dropDownArrayCollection: function (column, fieldItems) {
            var proxy = this,
                model = proxy.model,
                fieldName = fieldItems,
                dropDownRecords = [],
                uniqueDropDownRecords = [],
                dropdownData = [],
                columnEditType = column.editType;

            dropDownRecords = $.map(model.flatRecords, function (record) {
                if (record.item[fieldName])
                    return record.item[fieldName];
            });
            $.each(dropDownRecords, function (i, el) {
                if ($.inArray(el, uniqueDropDownRecords) === -1) uniqueDropDownRecords.push(el);
            });
            if (columnEditType != "numericedit")
                uniqueDropDownRecords.sort(proxy.stringCompare);
            else {
                uniqueDropDownRecords = uniqueDropDownRecords.sort(function (a, b) {
                    return a - b;
                });
            }
            $.each(uniqueDropDownRecords, function (i, val) {
                dropdownData.push({ text: val, value: val });
            });
            return dropdownData;
        },

        stringCompare: function (s1, s2) {
            var str1 = [], str2 = [];

            s1.replace(/(\d+)|(\D+)/g, function (_, $1, $2) { str1.push([$1 || Infinity, $2 || ""]) });
            s2.replace(/(\d+)|(\D+)/g, function (_, $1, $2) { str2.push([$1 || Infinity, $2 || ""]) });
    
            while (str1.length && str2.length) {
                var st1 = str1.shift();
                var st2 = str2.shift();
                var val = (st1[0] - st2[0]) || st1[1].localeCompare(st2[1]);
                if(val) return val;
            }

            return str1.length - str2.length;
        },
        // Method to prevent double click event during dropDown list and date picker filtering
        _preventDoubleClick: function (col) {
            var proxy = this,
                filterBoxTool = $("#" + proxy._id + "_" + col.field + "_filterbarcell_popup");
            if (filterBoxTool.length != 0) {
                $(filterBoxTool).on('click', function (e) {
                    proxy._off(proxy.element, "dblclick", ".e-gridcontent", proxy._editdblClickHandler);
                    setTimeout(function () {
                        proxy._on(proxy.element, "dblclick", ".e-gridcontent", proxy._editdblClickHandler);
                    }, 300);
                });
            }
        },
        // METHOD TO RENDER FILDERING ELEMENTS
        _renderFiltering: function () {
            var proxy = this, model = this.model,
            columns = model.columns;
            var $tr = ej.buildTag('tr.e-filterbar', "", {}, {});
            if (proxy.model.allowFiltering) {
                $.each(columns, function (i, column) {
                    if (ej.isNullOrUndefined(column["allowFiltering"]))
                        column["allowFiltering"] = true;
                });
                var filterEnabled = false;
                //To check whether atleast any one of the column's allowFiltering has been enabled
                $.each(columns, function (i, column) {
                    if (column.allowFiltering == true) {
                        filterEnabled = true;
                        return false;
                    }
                });
                if (filterEnabled) {
                    for (var column = 0; column < this.model.columns.length; column++) {

                    var $th = ej.buildTag('th.e-filterbarcell .e-editValue', "", {}, {});
                    columns[column]["visible"] === false && $th.addClass("e-hide");
                    $div = ej.buildTag('div.e-filterdiv', "", {}, { id: proxy._id + "_filterBar" + columns[column].field });
                        $tr.append($th.append($div));
                        //Condition to check whether allow filtering enabled for the individual column before rendering filtering elements
                        if (columns[column].allowFiltering == false)
                            continue;
                        else {
                       
                        this.filterEditType(columns[column], $div, column);
                        var $span = ej.buildTag('span.e-cancel e-icon e-hide e-spanstyle');
                        $div.append($span);
                    }                   
                }          
                }
            }
            /* add details column to last in column header */
            if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn) {
                $tr.append(ej.buildTag('th.e-filterbarcell e-detailheadercell', '<div style="width:35px;"></div>'));
            }
            return $tr;
        },


        _getIndentCol: function () {
            return ej.buildTag("col", "", { width: "35px" });
        },

        //METHOD FOR RENDERING TREEGRID HEADER
        _renderGridHeader: function () {

            var proxy = this,
                model = proxy.model,
                doc = document,
                $innerDiv = ej.buildTag('div.e-gridheadercontainer', "", { "overflow": "hidden" }, { "unselectable": "on" }),
                $table = ej.buildTag('table.e-table', "", {}, { "cellspacing": "0px", "border-spacing": "0px" }),
                $thead = ej.buildTag('thead'),
                $tbody = ej.buildTag('tbody.e-hide'),
                $columnHeader = ej.buildTag('tr.e-columnheader'),
                $colGroup = $(doc.createElement('colgroup')),
                $rowBody = $(doc.createElement('tr')),
                columnCount = 0,
                columns = model.columns,
                length = columns.length,
                $headerCell,
                bodyCell,
                $headerCellDiv,
                col,
                $div = model.isFromGantt ? ej.buildTag('div.e-gridheader#' + proxy._id + "e-gridheader", "", {
                    "border-left-style": "solid",
                    "border-left-width": "1px"
                }, "") :
                    ej.buildTag('div.e-gridheader#' + proxy._id + "e-gridheader", "",
                {
                    "border-top-style": "solid",
                    "border-left-style": "solid",
                    "border-right-style": "solid",
                    "border-top-width": "1px",
                    "border-left-width": "1px",
                    "border-right-width": "1px",
                    "width": proxy._gridWidth,
                }, "");
            

            proxy.setGridHeaderContent($div);
            proxy._$gridHeaderContainer = $innerDiv;
            proxy._hiddenColumns = [];
            proxy._visibleColumns = [];
            //CHECK SORTING ENABLED OR NOT
            if (model.allowSorting || model.showColumnChooser) {
                $columnHeader.css({ 'cursor': 'pointer' });
            }

            for (columnCount; columnCount < length; columnCount++) {
                if (columns[columnCount]) {
                    var headerTextAlign = columns[columnCount].headerTextAlign ? columns[columnCount].headerTextAlign
                                                           : (columns[columnCount].textAlign?columns[columnCount].textAlign: ej.TextAlign.Left);
                   // set padding right 35px for displaying column menu icon
                   if(proxy.model.showColumnChooser)
                        $headerCell = ej.buildTag('th.e-headercell', "", { "height": "45px","padding-right":"35px" }, {});
                   else
                        $headerCell = ej.buildTag('th.e-headercell', "", { "height": "45px" }, {});
                    bodyCell = doc.createElement('td');

                    $headerCellDiv = ej.buildTag(
                        'div.e-headercelldiv',
                        columns[columnCount]["headerText"] == undefined ?
                        columns[columnCount]["headerText"] = columns[columnCount]["field"] :
                        columns[columnCount]["headerText"], { "text-align": headerTextAlign }, { 'unselectable': "on" }
                    );

                    $headerCell.append($headerCellDiv);
					// Add column menu icon in header columns
                    if(proxy.model.showColumnChooser){
                        $columnChooser = ej.buildTag('div.e-columnmenu-icon e-icon e-columnicon', "", { "cursor": "pointer" });
                        $columnChooser.data("isClicked", false);
                        $headerCell.append($columnChooser);
                    }
                    
                    col = doc.createElement('col');

                    $rowBody.append(bodyCell);
                    $columnHeader.append($headerCell);
                    $colGroup.append(col);


                    //CHECK VISIBLE PROPERTY OF THE COLUMN
                    if (columns[columnCount]["visible"] === false && columnCount!=proxy.model.treeColumnIndex) {
                        $headerCell.addClass("e-hide") && $(col).css("display", "none") &&
                        proxy._hiddenColumns.push(columns[columnCount].headerText);

                    } else {
                        proxy._visibleColumns.push(columns[columnCount].headerText);
                        columns[columnCount]["visible"] = true;
                    }

                    if (columns[columnCount]["allowSorting"] === false) {
                        proxy._disabledSortableColumns.push(columns[columnCount].headerText);
                        $headerCell.css({ 'cursor': 'normal' });
                    }

                    if (!ej.isNullOrUndefined(columns[columnCount]["headerTemplateID"])) {
                        $headerCellDiv.html($(columns[columnCount]["headerTemplateID"]).hide().html()).parent().addClass("headertemplate");
                    }

                    if (columns[columnCount]["width"] == undefined && model.commonWidth !== undefined) {
                        proxy._columnsWidthCollection[columnCount] = model.commonWidth;
                    } else {
                        proxy._columnsWidthCollection.push(columns[columnCount]["width"]);
                    }

                    proxy._fieldColumnNames[columns[columnCount].headerText] = columns[columnCount].field;
                    proxy._headerColumnNames[columns[columnCount].field] = columns[columnCount].headerText;
                }
            }
            /* add detail column to last*/
            if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn) {
                $columnHeader.append(ej.buildTag('th.e-headercell e-detailheadercell', '<div style="width:35px;"></div>', {}, {}));
                $colGroup.append(this._getIndentCol());
            }

            // Condition to check whether filtering enabled to render filter elements.
            $filterHeader = ej.buildTag('tr.e-filterbar');
            if (this.model.allowFiltering && !proxy.model.isFromGantt)
            {
                $filterHeader = this._renderFiltering();
            }

            $thead.append($columnHeader).append($filterHeader);
            $table.append($colGroup).append($thead);

            $innerDiv.html($table);
            $div.html($innerDiv);

            proxy.setGridHeaderTable($table);

            return $div;
        },

        //#endregion

        //#region TREEGRIDCONTENT RENDERING
        //TREEGRID CONTENT RENDER INITIALIZATION
        _initGridRender: function () {

            var proxy = this;
            proxy._addInitTemplate();
            proxy._renderGrid();

            //CHECK COLUMN RESIZE IS ENABLED OR NOT AND INITIALIZE THE RESIZER
            if (proxy.model.allowColumnResize) {
                proxy._resizer = new ej.gridFeatures.gridResize(proxy);
            }

        },
        //Create a summary row.
        _summaryRow: function () {
            var proxy = this, model = proxy.model;
            if (model.showSummaryRow)
                proxy._createSummaryRow();
            if (model.showTotalSummary) {
                proxy._createTotalSummaryRow();
                proxy._viewPortHeight = proxy._viewPortHeight - proxy._$totalSummaryRowContainer.height();
            }
        },


        //ADD GRIDCONTENT TO THE TREEGRID
        _renderGrid: function () {
            
            var proxy = this;
            proxy._renderGridContent().insertAfter(proxy.element.children(".e-gridheader"));
            

        },

        //Add a total summary row after treegrid content.
        _createTotalSummaryRow: function () {
            var proxy = this, $summaryDiv, $table, model = proxy.model, flatRecords = model.flatRecords,
                summaryRows = model.summaryRows, summaryRowLength = summaryRows.length,
                updatedRecords = model.updatedRecords, parentRecords = model.parentRecords,
                columns = model.columns, item = {}, $tbody = ej.buildTag('tbody');

            proxy._footerSummaryRecord = [];
            proxy._footerSummaryRows = [];
            $("#" + proxy._id + "-footersummaryrow").remove();
            $summaryDiv = ej.buildTag('div.e-footersummaryrowdiv#' + proxy._id + '-footersummaryrow', '',
                {
                    "overflow": "hidden",
                    "cursor": "default"
                });
            var $footerContainer = ej.buildTag('div.e-footercontainer', '',
               {                                  
                   "overflow": "hidden"
               });
            var $table = ej.buildTag('table.e-table#' + proxy._id + "e-table", "",
                { top: "0px" }, { 'cellspacing': '0px' });
            $table.append(proxy.getContentTable().find('colgroup').clone()).append($tbody);
            if (proxy.model.showSummaryRow)
                proxy._flatChildRecords = $(model.updatedRecords).not(model.summaryRowRecords).get();
            else
                proxy._flatChildRecords = updatedRecords;

            for (var summaryRowIndex = 1; summaryRowIndex <= summaryRowLength; summaryRowIndex++) {
                for (var columnLength = 0; columnLength < columns.length; columnLength++) {
                    item[columns[columnLength].field] = "";
                }
                item = proxy._createSummaryItem(item, summaryRows[summaryRowIndex - 1]);
                var record = proxy._createRecord(item, 0);
                record.footerSummaryRowRecord = true;
                record.index = summaryRowIndex;
                proxy._footerSummaryRecord.push(record);
                GridRow = $.render[this._id + "_Template"](record);
                proxy._footerSummaryRows.push(GridRow);
                $tbody.append(GridRow);
            }
            proxy._$totalSummaryRowContainer = $summaryDiv;
            proxy._$footertableContent = $table;
            proxy._$footerContainer = $footerContainer;
            $footerContainer.append($table);            
            $summaryDiv.append($footerContainer);
            proxy.element.append($summaryDiv);
            proxy._flatChildRecords = [];
            if ($footerContainer.height() > 90)
                proxy._$totalSummaryRowContainer.ejScroller({ height: 90 });
            else
                proxy._$totalSummaryRowContainer.addClass("e-scrollcss");

        },

        //Add a summary row for every parent item in Tree grid.
        _createSummaryRow: function (args) {
            var proxy = this, model = proxy.model, parentRecords,
                recordIndex,
                item = {},
                docsummaryRow,
                totalChildRecordsLength,
                summaryRows = model.summaryRows,
                summaryRowLength = summaryRows.length,
                columns = model.columns;
            if (args && (args.requestType == "dragAndDrop" || args.requestType == "addNewRow" ||
                 args.requestType == "sorting"))
                parentRecords = proxy._getParentRecords(model.updatedRecords, proxy);
            else
                parentRecords = proxy._parentRecords;
            var length = parentRecords.length;
            if (args && ej.isNullOrUndefined(args.filterCollection))
                args.filterCollection = [];
            model.updatedRecords = $(model.updatedRecords).not(model.summaryRowRecords).get();
            model.flatRecords = $(model.flatRecords).not(model.summaryRowRecords).get();
            model.currentViewData = $(model.currentViewData).not(model.summaryRowRecords).get();
            $(proxy.element).find(".e-summaryrow").remove();                       
            this._$gridRows = $(this.getTreeGridRows()).not(".e-summaryrow");
            proxy._flatChildRecords = [];
            for (var parentIndex = 0; parentIndex < length; parentIndex++) {
                parentRecord = parentRecords[parentIndex];
                totalChildRecordsLength = proxy._getChildRecordsLength(parentRecord, model.updatedRecords);              
                if (totalChildRecordsLength == 0)
                    continue;
                for (var summaryRowIndex = 1; summaryRowIndex <= summaryRowLength; summaryRowIndex++) {
                    for (var columnLength = 0; columnLength < columns.length; columnLength++) {
                        item[columns[columnLength].field] = "";
                    }
                    item = proxy._createSummaryItem(item, summaryRows[summaryRowIndex - 1], parentRecord);
                    var currentIndex = model.updatedRecords.indexOf(parentRecord) + totalChildRecordsLength + summaryRowIndex,
                        summaryRecord = proxy._createRecord(item, currentIndex);
                    proxy._isSummaryRow = true;
                    summaryRecord.level = parentRecord.level + 1;
                    summaryRecord.parentItem = parentRecord;
                    summaryRecord.isSummaryRow = true;
                    summaryRecord.index = model.flatRecords.length + 1;
                    if (model.currentViewData[currentIndex])
                        model.currentViewData.splice(currentIndex, 0, summaryRecord);
                    if (model.enableVirtualization) {
                        if (parentRecord.expanded) {
                            model.updatedRecords.splice(currentIndex, 0, summaryRecord);
                            if (!args || (args && args.filterCollection))
                                model.summaryRowRecords.push(model.updatedRecords[currentIndex]);
                        }
                    }
                    else {
                        model.updatedRecords.splice(currentIndex, 0, summaryRecord);
                        model.summaryRowRecords.push(model.updatedRecords[currentIndex]);
                    }
                    model.flatRecords.splice(currentIndex, 0, summaryRecord);
                    parentRecord.childRecords.push(summaryRecord);
                    proxy._renderAddedRow(currentIndex, summaryRecord);
                    ej.TreeGrid.refreshRow(proxy, currentIndex);
                    item = {};
                    if (summaryRowIndex == summaryRowLength && proxy.getRows()[currentIndex]) {
                        var lastSummaryRow = proxy.getRows()[currentIndex];
                        if (lastSummaryRow)
                            $(lastSummaryRow.childNodes).css({
                                "border-bottom-width": "1px",
                            });
                    }
                }
                proxy._flatChildRecords = [];
            }
            proxy._isSummaryRow = false;
        },
        //Get all parent records from updated records.
        _getParentRecords: function (updatedRecords, proxy) {
            var recordLength = updatedRecords.length, record, parentRecords = [];
            for (var length = 0; length < recordLength; length++) {
                record = updatedRecords[length]
                if (record.childRecords && record.childRecords.length > 0)
                    parentRecords.push(record);
            }
            return parentRecords;
        },
        //To get all child and grand child of one parent record when the datasource is in hierarchy form.
        _getChildRecordsLength: function (parentRecord, updatedRecords) {
            var proxy = this,
                recordLength = updatedRecords.length, record;
            for (var length = 0; length < recordLength; length++) {
                record = updatedRecords[length];
                if (record.summaryRow)
                    continue;
                if (parentRecord == record.parentItem) {
                    proxy._flatChildRecords.push(record);
                    if (record.hasChildRecords)
                        proxy._getChildRecordsLength(record, updatedRecords);
                }
            }
            return proxy._flatChildRecords.length;
        },

        //Assign the values for item properties to create a summary record.
        _createSummaryItem: function (item, targetSummaryRow, parentRecord) {
            var proxy = this,
                summaryColumns = targetSummaryRow.summaryColumns,
                summaryColumnsLength = summaryColumns.length;
            for (var columnLength = 0; columnLength < summaryColumnsLength; columnLength++) {
                var targetSummaryColumn = summaryColumns[columnLength],
                    prefix = targetSummaryColumn.prefix,
                    suffix = targetSummaryColumn.suffix,
                    displayColumn = targetSummaryColumn.displayColumn;

                if (ej.isNullOrUndefined(displayColumn))
                    displayColumn = targetSummaryColumn.dataMember;
                if (ej.isNullOrUndefined(prefix))
                    prefix = "";
                if (ej.isNullOrUndefined(suffix))
                    suffix = "";
                item.summaryColumn = targetSummaryColumn;
                item.summaryRow = targetSummaryRow;

                for (var key in item) {
                    if (key == displayColumn) {
                        if (item[key])
                            item[key] = item[key] + " " + prefix + proxy._getSummaryValues(targetSummaryColumn, proxy._flatChildRecords) + suffix;
                        else
                            item[key] = prefix + proxy._getSummaryValues(targetSummaryColumn, proxy._flatChildRecords) + suffix;
                    }
                }
            }
            return item;
        },

        //Calculate the summary values.
        _getSummaryValues: function (summaryColumn, summaryData) {
            var dataMember = summaryColumn.dataMember, format = summaryColumn.format,
                jsonData = summaryData;
            var dbMgr, $value;
            if (jsonData instanceof ej.DataManager) {
                dbMgr = jsonData;
                jsonData = jsonData.dataSource.json;
            } else
                dbMgr = ej.DataManager(jsonData);
            switch (summaryColumn.summaryType) {
                case ej.TreeGrid.SummaryType.Sum:
                    $value = ej.sum(jsonData, dataMember);
                    break;
                case ej.TreeGrid.SummaryType.Average:
                    $value = ej.avg(jsonData, dataMember);
                    break;
                case ej.TreeGrid.SummaryType.Maximum:
                    var obj = ej.max(jsonData, dataMember);
                    $value = parseInt(obj[summaryColumn.dataMember]);
                    break;
                case ej.TreeGrid.SummaryType.Minimum:
					var filteringData = jsonData.filter(function (data) {
                        if (data[dataMember] !== "")
                            return true;
                    });
                    var obj = ej.min(filteringData, dataMember);
                    $value = parseInt(obj[summaryColumn.dataMember]);
                    break;
                case ej.TreeGrid.SummaryType.MaximumDate:
                    $value = this._getMaxDate(jsonData, dataMember);
                    break;
                case ej.TreeGrid.SummaryType.MinimumDate:
                    $value = this._getMinDate(jsonData, dataMember);
                    break;
                case ej.TreeGrid.SummaryType.Count:
                    $value = jsonData.length;
                    break;
                case ej.TreeGrid.SummaryType.TrueCount:
                    var predicate = ej.Predicate(summaryColumn.dataMember, "equal", true);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
                case ej.TreeGrid.SummaryType.FalseCount:
                    var predicate = ej.Predicate(summaryColumn.dataMember, "equal", false);
                    $value = dbMgr.executeLocal(ej.Query().where(predicate)).length;
                    break;
            }
            if (!ej.isNullOrUndefined(format))
                $value = this.formatting(format, $value, this.model.locale);
            return $value;
        },
        // To Get Maximum Date of Particular Datamember in JSON data.
        _getMaxDate: function (jsonData, dataMember) {
            var max = 0;
            var maxDate = null;
            for (var i = 0; i < jsonData.length; i++) {
                var item = jsonData[i],
                    date = new Date(item[dataMember]),
                    time = date.getTime();
                if (time > max) {
                    max = time;
                    maxDate = item[dataMember];
                }
            }

            return maxDate;
        },
        // To Get Minimum Date of Particular Datamember in JSON data.
        _getMinDate: function (jsonData, dataMember) {
            var firstMinDate = new Date(jsonData[0][dataMember]),
                min = firstMinDate.getTime();
            var minDate = jsonData[0][dataMember];
            for (var i = 0; i < jsonData.length; i++) {
                var item = jsonData[i],
                    date = new Date(item[dataMember]),
                    time = date.getTime();
                if (time < min) {
                    min = time;
                    minDate = item[dataMember];
                }
            }

            return minDate;
        },
        //Update the summary after add, edit or delete the record.
        _updateSummaryRow: function (args) {
            var proxy = this, model = proxy.model, summaryRow,
                editedRecord, editedRecordIndex, updatedSummaryRecords = [],
                _pRecords = [],cellvalue;
            if (args && args.requestType == "dragAndDrop") {
                args.rowIndex = model.updatedRecords.indexOf(proxy._droppedRecord);
            }
            if (args && args.requestType == "delete") {
                proxy._updateAfterDeleteRecord(args);
                return;
            }

            if (args && args.editType == "celledit") {
                if (args.columnObject.editType == "numericedit") {
                    cellvalue = parseInt(args.value);
					args.value = isNaN(cellvalue) ? "" : cellvalue;
                    args.data[args.columnName] = args.value;
                }
                args.currentValue = args.value;
                args.rowIndex = proxy.getRows().index(args.rowElement);
            }
            if ((args.currentValue != args.previousValue) || args.requestType == "dragAndDrop"
                || args.requestType == "addNewRow") {
                editedRecord = model.updatedRecords[args.rowIndex];
                if (editedRecord.parentItem)
                    _pRecords = proxy._getpRecords(editedRecord, proxy);
                if (ej.isNullOrUndefined(_pRecords))
                    return;
                for (var parentLength = 0; parentLength < _pRecords.length; parentLength++) {
                    var summaryRowRecords = model.summaryRowRecords;
                    proxy._getChildRecordsLength(_pRecords[parentLength], model.updatedRecords);
                    $.each(summaryRowRecords, function (index, record) {
                        summaryRow = record.summaryRow;
                        if (record.parentItem === _pRecords[parentLength]) {
                            var summaryColumnLength = summaryRow.summaryColumns.length;
                            for (var length = 0; length < summaryColumnLength; length++) {
                                var summaryColumn = summaryRow.summaryColumns[length],
                                    prefix = summaryColumn.prefix,
                                    suffix = summaryColumn.suffix;
                                if (ej.isNullOrUndefined(prefix))
                                    prefix = "";
                                if (ej.isNullOrUndefined(suffix))
                                    suffix = "";
                                if (args.editType == "rowedit") {
                                    args.columnName = summaryColumn.dataMember;
                                        var columnList = model.columns,
                                            dataMemberColumnIndex = proxy.getColumnIndexByField(args.columnName);
                                    //To convert empty string to integer while adding the new row.    
                                        if (model.columns[dataMemberColumnIndex].editType == "numericedit")
                                            editedRecord[args.columnName] = parseInt(editedRecord[args.columnName]);                                   
                                }
                                var displayColumn = summaryColumn.displayColumn;
                                if (!displayColumn)
                                    displayColumn = summaryColumn.dataMember;
                                if (model.enableVirtualization)
                                    recordindex = model.currentViewData.indexOf(record);
                                else
                                    recordindex = model.updatedRecords.indexOf(record);
                                columnIndex = proxy.getColumnIndexByField(displayColumn);
                                args.data = record;
                                args.rowElement = $(proxy.getRows()[recordindex]);
                                args.cellElement = args.rowElement.find(".e-rowcell").eq(columnIndex);

                                value = proxy._getSummaryValues(summaryColumn, proxy._flatChildRecords);
                                record[displayColumn] = prefix + value + suffix;
                                args.cellElement.empty().html(prefix + value + suffix);
                                updatedSummaryRecords.push(record);
                            }
                        }
                    })
                    proxy._flatChildRecords = [];
                }
            }
        },
        //Update the Total summary after add, edit or delete the record.
        _updateTotalSummaryRow: function (args) {
            if (args.requestType == "dragAndDrop" || args.isDragAndDropDelete)
                return;
            var proxy = this, model = proxy.model, summaryRows = model.summaryRows, summaryColumns, 
                summaryColumnsLength, record, summaryRowLength = summaryRows.length;
            if (model.updatedRecords.length == 0) {
                $("#" + proxy._id + "-footersummaryrow").hide();
                return;
            }
            else
                $("#" + proxy._id + "-footersummaryrow").show();
            for (var length = 0; length < summaryRowLength; length++) {
                summaryRow = summaryRows[length];
                summaryColumns = summaryRow.summaryColumns;
                summaryColumnsLength = summaryColumns.length;
                record = proxy._footerSummaryRecord[length];
                for (var columnLength = 0; columnLength < summaryColumnsLength; columnLength++) {
                    var summaryColumn = summaryColumns[columnLength],
                        prefix = summaryColumn.prefix,
                        suffix = summaryColumn.suffix;
                    if (ej.isNullOrUndefined(prefix))
                        prefix = "";
                    if (ej.isNullOrUndefined(suffix))
                        suffix = "";
                    if (args.editType == "rowedit")
                        args.columnName = summaryColumn.dataMember;
                    if (!displayColumn)
                        displayColumn = summaryColumn.dataMember;
                    if (summaryColumn.dataMember == args.columnName || args.requestType == "delete"
                        || args.requestType == "filtering" || args.requestType == "addNewRow") {
                        var displayColumn = summaryColumn.displayColumn,
                            recordindex = proxy._footerSummaryRecord.indexOf(record),
                            columnIndex = proxy.getColumnIndexByField(displayColumn);
                        args.data = record;
                        args.rowElement = $("#" + proxy._id).find(".e-footersummaryrow")[recordindex];
                        args.cellElement = args.rowElement.childNodes[columnIndex];
                        var footerUpdatedRecords = $(model.updatedRecords).not(model.summaryRowRecords).get();
                        value = proxy._getSummaryValues(summaryColumn, footerUpdatedRecords);
                        record[displayColumn] = prefix + value + suffix;
                        args.cellElement.innerHTML = prefix + value + suffix;
                    }
                }
            }
        },
        //Update summary row after deleted the record.
        _updateAfterDeleteRecord: function (args) {
            var proxy = this, model = proxy.model, summaryRow,
                editedRecord, editedRecordIndex, updatedSummaryRecords = [],
                _pRecords;
            deletedRecord = args.data;
            if (deletedRecord.parentItem)
                _pRecords = proxy._getpRecords(deletedRecord, proxy);
            if (ej.isNullOrUndefined(_pRecords))
                return;
            for (var parentLength = 0; parentLength < _pRecords.length; parentLength++) {
                var summaryRowRecords = model.summaryRowRecords,
                    childRecords = _pRecords[parentLength].childRecords;
                if (childRecords.length == 0)
                    continue;
                proxy._getChildRecordsLength(_pRecords[parentLength], model.updatedRecords);
                $.each(summaryRowRecords, function (index, record) {
                    summaryRow = record.summaryRow;
                    if (record.parentItem === _pRecords[parentLength]) {
                        var summaryColumnLength = summaryRow.summaryColumns.length;
                        for (var length = 0; length < summaryColumnLength; length++) {
                            var summaryColumn = summaryRow.summaryColumns[length],
                                displayColumn = summaryColumn.displayColumn,
                                recordindex = model.updatedRecords.indexOf(record),
                                columnIndex = proxy.getColumnIndexByField(displayColumn),
                                 prefix = summaryColumn.prefix,
                                 suffix = summaryColumn.suffix;
                            if (ej.isNullOrUndefined(prefix))
                                prefix = "";
                            if (ej.isNullOrUndefined(suffix))
                                suffix = "";
                            args.data = record;
                            args.tr = $(proxy.getRows()[recordindex]);
                            args.cellElement = args.tr.find(".e-rowcell").eq(columnIndex);
                            if (childRecords.length == 0) {

                            }
                            value = proxy._getSummaryValues(summaryColumn, proxy._flatChildRecords);
                            record[displayColumn] = prefix + value + suffix;
                            args.cellElement.empty().html(prefix + value + suffix);
                        }
                        updatedSummaryRecords.push(record);
                    }

                })
                proxy._flatChildRecords = [];
            }
        },
        //To get parent and grand Parent of one Child record.
        _getpRecords: function (record, proxy) {
            var recordParents = [];
            do {
                record = record.parentItem;
                recordParents.push(record);
            } while (record.parentItem);
            return recordParents;
        },

        //RENDER THE GRIDCONTENT
        _renderGridContent:function(){
            
            var proxy = this,
                doc = document,
                model = proxy.model,
                $tbody = $(doc.createElement('tbody')),
                $div = ej.buildTag("div.e-gridcontent#" + proxy._id + "e-gridcontent", "",
                {                    
                    "height": proxy._viewPortHeight
                },
                {"unselectable":"on"}),
                $innderDiv = ej.buildTag('div.e-gridcontainer#' + proxy._id + "e-gridcontainer", "", {}, {});
                //Insert e-contet div for ejScroller
                $scrolllerContentDiv = ej.buildTag('div', {}, { "width": "auto", "height": "auto" });
                $scrolllerContentDiv.append($innderDiv);
            //CHECK VIRTUALIZATION ENABLED OR NOT
            if (model.enableVirtualization) {
                $innderDiv.css({ 'height': proxy._totalHeight });
            }

            var $table = ej.buildTag('table.e-table#' + proxy._id + "e-table", "",
                {top: "0px" }, { 'cellspacing': '0px' });

            $table.append(proxy.getHeaderTable().find('colgroup').clone()).append($tbody);
            $innderDiv.html($table);
            $div.html($scrolllerContentDiv);
            proxy.setGridContentTable($table);

            //CHECK DATASOURCE IS DEFINED OR NOT
            if (this.dataSource() === null || this.model.currentViewData.length == 0) {
                var $emptyTd = ej.buildTag('td', model.emptyRecordText, {}, { colSpan: model.columns.length });
                $tbody.append($(doc.createElement("tr")).append($emptyTd));
                $innderDiv.css('height', model.rowHeight);
            } else
            {
                proxy.renderRecords();
            }

            if (!model.isFromGantt) {
                $div.css({
                    "border-bottom-style": "solid",
                    "border-bottom-width": "1px",
                    "border-left-style": "solid",
                    "border-left-width": "1px",
                    "border-right-style": "solid",
                    "border-right-width": "1px",
                    "width": "auto",
                    "height": "auto",
                    "cursor": "default"
                });
            } else {
                $div.css({
                    "border-left-style": "solid",
                    "border-left-width": "1px"
                });
            }

            $table.css({ 'position': 'relative' });
            return $div;

        },


        //AFTER INITIALIZATION END RENDERING
        _initialEndRendering: function () {

            var proxy = this, model = proxy.model, currentSelectingRecord,
                expandedRecords = proxy.getExpandedRecords(model.updatedRecords),
                index = proxy.selectedRowIndex();

            if (index != -1 && model.allowSelection && !model.isFromGantt) {
                if (model.showSummaryRow) {
                    var recordsWithOutSummary = $(model.updatedRecords).not(model.summaryRowRecords).get(),
                        currentSelectedRecord = recordsWithOutSummary[index];
                    index = model.updatedRecords.indexOf(currentSelectedRecord);                    
                }
                if (!proxy._rowSelectingEventTrigger(this._previousIndex, index)) {
                    proxy.selectRows(index);
                    proxy._rowSelectedEventTrigger(index);

                }
            }

            /* treegrid rendering completed here at load time*/
            proxy._isRendered = true;

            proxy._trigger("refresh");
            proxy._eventBindings();
            proxy._hideCollapsedDetailsRows();
        },

        //#endregion
        
        //#region TEMPLATE

        //ADD CELLEDITING TEMPLATE
        _addCellEditTemplate: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                $outerDiv = ej.buildTag('div', "", { display: 'none' }, { id: proxy._id + "_CellEditTemplate" }),
                $innerDiv,
                length =columns&& columns.length;

            for (columnCount = 0; columnCount < length; columnCount++) {
                if (columns[columnCount]) {
                    $innerDiv = ej.buildTag('div', "", {}, { id: columns[columnCount].field + "_CellEdit" });
                    ej.TreeGrid._initCellEditType(proxy, $innerDiv, proxy._id, columnCount);
                    $outerDiv.append($innerDiv);
                }
            }

            if ($outerDiv.children().length) {
                proxy._cellEditTemplate = $outerDiv;
            }

        },

        _getState: function () {
            if (this.data.parentItem)
                return false;
            else
                return true;
        },

        _getrowName: function () {
            var rowClass = "gridrowIndex",
                proxy = this;
            rowClass += proxy.data.index.toString() + "level" + proxy.data.level.toString();
            return rowClass;
        },
        /* get details row col span with showDetailROwInfoColumn API value */
        _getDetailsColSpan:function()
        {
            var model = this.model;
            return model.showDetailsRowInfoColumn ? this._visibleColumns.length + 1 : this._visibleColumns.length;
        },
        /* expand status for details rows */
        _getDetailsExpandStatus: function (data) {
            var proxy = this;
            if (proxy.getExpandStatus(data) && data.isDetailsExpanded) {
                return true;
            } else
            {
                return false;
            }
        },
        _createDetailsRowTemplate: function () {
            var proxy = this,
                model = this.model,
                helpers = {};

            helpers["_" + proxy._id + "getDetailsColSpan"] = $.proxy(proxy._getDetailsColSpan, proxy);
            helpers["_" + proxy._id + "getDetailsExpandStatus"] = $.proxy(proxy._getDetailsExpandStatus, proxy);
            helpers["_" + proxy._id + "getExpandStatus"] = $.proxy(proxy.getExpandStatus, proxy);
            
            $.views.helpers(helpers);

            var tr = '{{if !isSummaryRow && !footerSummaryRowRecord}}' + "<tr class='e-detailsrow e-treegridrows detailsrow{{:~_" + proxy._id + "rowClassName()}} {{if isDetailsExpanded}}" +
                      "e-detailsrowexpanded{{else}}e-detailsrowcollapsed{{/if}}{{if !~_" + proxy._id + "getExpandStatus(#data) && isDetailsExpanded}} collapsedrowexpandeddetailsrow{{/if}}' " +
                      "style='{{if ~_" + proxy._id + "getDetailsExpandStatus(#data)}}display:table-row;{{else}}visibility:none;{{/if}}height:" + model.detailsRowHeight + "px;'>" +
                      "<td class='e-detailsrowcell' colspan='{{:~_" + proxy._id + "getDetailsColSpan()}}'>" +
                      "<div class='e-detailscellwrapper' style='height:" + (model.detailsRowHeight - 1) + "px;'>";

            var templateString = "";
            if (model.detailsTemplate && model.detailsTemplate.length > 0) {

                if (document.getElementById(model.detailsTemplate))
                {
                    templateString = $("#" + model.detailsTemplate)[0].innerHTML;
                } else
                {
                    templateString = model.detailsTemplate;
                }
            }
            $("#treeGridDetailsRowTemplateElement" + proxy._id).remove();
            var scriptElement = this._createDetailRowTemplateElement(templateString);
            tr += "{{:~_treeGridTemplating('" + scriptElement.id + "')}}";
            tr += "</div></td></tr>";
            tr += "{{/if}}";

            var templates = {};
            templates[proxy._id + "_detailRowTemplate"] = tr;
            $.templates(templates);
            return tr;

        },
        //ADD TEMPLATE FOR RENDERING GRIDCONTENT
        _addInitTemplate:function(){

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                columnCount = 0,
                length = columns.length,
                columnName,
                column,
                tdCell,
                summaryTitle,
                helpers = {};

            //CHECK THE COLUMNS ARRAY CONTAIN THE COLUMN OBJECT
            if (columns.length === 0) {
                return false;
            }

            helpers["_" + proxy._id + "rowClassName"] = ej.TreeGrid._getrowClassName;
            helpers["_" + proxy._id + "SummaryRowtdClassName"] = ej.TreeGrid._getSummaryRowtdClassName;
            helpers["_" + proxy._id + "summaryTitle"] = proxy._getSummaryTitle;
            // helpers for drag row indicator icon class
            helpers["_" + proxy._id + "rowName"] = proxy._getrowName;
            helpers["getState"] = proxy._getState;

            helpers["_" + proxy._id + "expandStatus"] = $.proxy(proxy._getExpandStatusRecord, proxy);         
            helpers["_" + proxy._id + "cellValue"] = proxy._getCellValue;
            helpers["_" + proxy._id + "isTreeColumnIndex"] = $.proxy(proxy._isTreeColumnIndex, proxy);
            helpers["_" + proxy._id + "getIndentWidth"] = $.proxy(proxy._getIndentWidth, proxy);
            helpers["_" + proxy._id + "checkColumn"] = $.proxy(proxy._getCellColumn, proxy);
            helpers["_" + proxy._id + "formatting"] = $.proxy(proxy.formatting, proxy);
            helpers["_treeGridTemplating"] = ej.proxy(proxy._gridTemplate, null, proxy);
            helpers["_" + proxy._id + "itemDetails"] = $.proxy(proxy._convertItemToString, proxy);

            $.views.helpers(helpers);

            
            if (model.rowTemplateID.length > 0 || model.altRowTemplateID.length > 0) {
                
                var altRowTemplateElement = ej.buildTag("tbody", "", {}, {});
                var rowTemplateElement = ej.buildTag("tbody", "", {}, {});
                var rowTemplateHTML = "";
                var altRowTemplateHTML = "";


                var altRowTemplate = model.altRowTemplateID.length > 0 ? $("#" + model.altRowTemplateID)[0].innerHTML : $("#" + model.rowTemplateID)[0].innerHTML;
                var rowTemplate = model.rowTemplateID.length > 0 ? $("#" + model.rowTemplateID)[0].innerHTML : $("#" + model.altRowTemplateID)[0].innerHTML;

                $(altRowTemplateElement[0]).html(altRowTemplate);
                $(rowTemplateElement[0]).html(rowTemplate);

                if (rowTemplateElement[0].childNodes.length > 0) {
                    for (var i = 0; i < rowTemplateElement[0].childNodes.length; i++) {
                        if (rowTemplateElement[0].childNodes[i].nodeName == 'TR') {

                            rowTemplateElement[0].className = rowTemplateElement[0].childNodes[i].className;
                            rowTemplateElement[0].style.cssText = rowTemplateElement[0].childNodes[i].style.cssText;

                            if (rowTemplateElement[0].childNodes[i].childNodes.length > 0) {
                                var tdCount = 0;
                                for (var j = 0; j < rowTemplateElement[0].childNodes[i].childNodes.length; j++) {

                                    var childNode = rowTemplateElement[0].childNodes[i].childNodes[j];

                                    if (childNode.nodeName == 'TD') {
                                        var innerHtml = childNode.innerHTML,
                                            templateId = proxy._id + "RowTemplateElement" + tdCount;

                                        $("#" + templateId).remove();
                                        var scriptElement = this._createRowTemplateElement(innerHtml, templateId);
                                        childNode.innerHTML = "{{:~_treeGridTemplating('" + scriptElement.id + "')}}";
                                        childNode.className += " e-rowcell e-templatecell";


                                        if (columns[tdCount]) {
                                            column = columns[tdCount];
                                            columnName = column.field;
                                            var visible = column.visible == undefined ? true : column.visible;
                                            //To create a seperate div in summary cell to display a summary title.
                                            var summaryTitlediv = "";
                                            if (column.headerText == proxy._visibleColumns[0]) {
                                                summaryTitlediv = '{{if isSummaryRow || footerSummaryRowRecord}}' + "<div " +
                                                    "class='e-summarytitle'" + "style='float:left;display:block;'>" + "{{:~_" + proxy._id + "summaryTitle()}}"
                                                     + "</div>" + '{{/if}}';
                                            }

                                            if (tdCount == model.treeColumnIndex) {
                                                var indentDiv = "<div class='e-inner-treecolumn-container' style='height:20px;width:{{:(level+1)*30}}px;' unselectable='on'>" +
                                                "<div class='intend' style='height:1px; float:left;width:{{:~_" + proxy._id + "getIndentWidth(#data)}}; display:inline-block;'>" +
                                                 //indicator icons included for drag and drop
                                                "{{if !~getState() || !hasChildRecords }}" +
                                                "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                                "<span class='aboveIcon e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='belowIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='childIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                                "<span class='cancelIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "</div>" +
                                                "{{if ~getState() && hasChildRecords}}" +
                                                "<div class='intendparent'>" +
                                                "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                                "<span class='aboveIcon e-icon'  style='position:relative;float:left;display:none;'></span>" +
                                                "<span class='belowIcon e-icon' style='float:left;display:none;'></span>" +
                                                "<span class='childIcon e-icon' style='float:left;display:none;'></span>" +
                                                "<span class='cancelIcon e-icon' style='float:left;display:none;'></span>" +
                                                "</div>" +
                                                "</div>" +
                                                "{{/if}}" +
                                                "<div class='{{if expanded && hasFilteredChildRecords}}e-treegridexpand e-icon{{else hasChildRecords && hasFilteredChildRecords}}e-treegridcollapse e-icon{{/if}}' " +
                                                "style='height:20px;width:30px;margin:auto;float:left;margin-left:10px;display:inline-block;' unselectable='on'>" +
                                                "</div>" +
                                                "<div class='e-cell' style='display:inline-block;width:100%' unselectable='on'>";

                                                rowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected}} e-selectionbackground{{/if}}{{if " + !visible + "}} e-hide{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                    + childNode.style.cssText + "'>" + indentDiv + "{{else}}'>{{/if}}" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}';

                                                     if (!ej.isNullOrUndefined(column["format"]))
                                                         rowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}" + '{{/if}}';
                                                     else
                                                         rowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" + '{{/if}}';
                                                     rowTemplateHTML += "</div></div></td>";
                                            }
                                            else {
                                                rowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected}} e-selectionbackground{{/if}}{{if " + !visible + "}} e-hide{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                    + childNode.style.cssText + "{{/if}}'>" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}';

                                                if (!ej.isNullOrUndefined(column["format"]))
                                                    rowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}" + '{{/if}}';
                                                else
                                                    rowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" + '{{/if}}';
                                                rowTemplateHTML += "</td>";
                                            }
                                            tdCount++;

                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if (altRowTemplateElement[0].childNodes.length > 0) {
                    for (var i = 0; i < altRowTemplateElement[0].childNodes.length; i++) {
                        if (altRowTemplateElement[0].childNodes[i].nodeName == 'TR') {
                            altRowTemplateElement[0].className = altRowTemplateElement[0].childNodes[i].className;
                            altRowTemplateElement[0].style.cssText = altRowTemplateElement[0].childNodes[i].style.cssText;
                            var tdCount = 0;

                            for (var j = 0; j < altRowTemplateElement[0].childNodes[i].childNodes.length; j++) {

                                var childNode = altRowTemplateElement[0].childNodes[i].childNodes[j];

                                if (childNode.nodeName == 'TD') {

                                    var innerHtml = childNode.innerHTML,
                                        templateId = proxy._id + "AltRowTemplateElement" + tdCount;
                                    $("#" + templateId).remove();
                                    var scriptElement = this._createRowTemplateElement(innerHtml, templateId);
                                    childNode.innerHTML = "{{:~_treeGridTemplating('" + scriptElement.id + "')}}";

                                    childNode.className += " e-rowcell e-templatecell";
                                    if (columns[tdCount]) {
                                        column = columns[tdCount];
                                        columnName = column.field;
                                        var visible = column.visible == undefined ? true : column.visible;
                                        //To create a seperate div in summary cell to display a summary title.
                                        var summaryTitlediv = "";
                                        if (column.headerText == proxy._visibleColumns[0]) {
                                            summaryTitlediv = '{{if isSummaryRow || footerSummaryRowRecord}}' + "<div " +
                                                "class='e-summarytitle'" + "style='float:left;display:block;'>" + "{{:~_" + proxy._id + "summaryTitle()}}"
                                                 + "</div>" + '{{/if}}';
                                        }
                                        if (tdCount == model.treeColumnIndex) {
                                            var indentDiv = "<div class='e-inner-treecolumn-container' style='height:20px;width:{{:(level+1)*30}}px;' unselectable='on'>" +
                                            "<div class='intend' style='height:1px; float:left;width:{{:~_" + proxy._id + "getIndentWidth(#data)}}; display:inline-block;'>" +
                                             //indicator icons included for drag and drop
                                            "{{if !~getState() || !hasChildRecords }}" +
                                            "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                            "<span class='aboveIcon e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                            "<span class='belowIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                            "<span class='childIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                            "<span class='cancelIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                            "</div>" +
                                            "{{/if}}" +
                                            "</div>" +
                                            "{{if ~getState() && hasChildRecords}}" +
                                            "<div class='intendparent'>" +
                                            "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                            "<span class='aboveIcon e-icon'  style='position:relative;float:left;display:none;'></span>" +
                                            "<span class='belowIcon e-icon' style='float:left;display:none;'></span>" +
                                            "<span class='childIcon e-icon' style='float:left;display:none;'></span>" +
                                            "<span class='cancelIcon e-icon' style='float:left;display:none;'></span>" +
                                            "</div>" +
                                            "</div>" +
                                            "{{/if}}" +
                                            "<div class='{{if expanded&& hasFilteredChildRecords}}e-treegridexpand e-icon{{else hasChildRecords && hasFilteredChildRecords}}e-treegridcollapse e-icon{{/if}}' " +
                                            "style='height:20px;width:30px;margin:auto;float:left;margin-left:10px;display:inline-block;' unselectable='on'></div>" +
                                            "<div class='e-cell' style='display:inline-block;width:100%' unselectable='on'>";

                                            altRowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected}} e-selectionbackground{{/if}}{{if " + !visible + "}} e-hide{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                 + childNode.style.cssText + "'>" + indentDiv + "{{else}}'>{{/if}}" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}';

                                            if (!ej.isNullOrUndefined(column["format"]))
                                                altRowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}" + '{{/if}}';
                                            else
                                                altRowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" + '{{/if}}';
                                            altRowTemplateHTML += "</div></div></td>";

                                        }
                                        else {
                                            altRowTemplateHTML += "<td class='" + childNode.className + "{{if isSummaryRow || footerSummaryRowRecord}} e-summaryrowcell{{/if}}{{if isSelected}} e-selectionbackground{{/if}}{{if " + !visible + "}} e-hide{{/if}}' style='{{if !isSummaryRow && !footerSummaryRowRecord}}"
                                                    + childNode.style.cssText + "{{/if}}'>" +
                                                    '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                                     childNode.innerHTML +
                                                    '{{else}}' + summaryTitlediv + '{{/if}}';

                                            if (!ej.isNullOrUndefined(column["format"]))
                                                altRowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}" + '{{/if}}';
                                            else
                                                altRowTemplateHTML += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" + '{{/if}}';
                                            altRowTemplateHTML += "</td>";
                                        }
                                        tdCount++;
                                    }
                                }
                            }
                        }
                    }
                }
               
                /* add details column to end of headers */
                if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn) {
                    altRowTemplateHTML += "<td class='{{if isSelected}}e-selectionbackground{{/if}} {{if isDetailsExpanded}}e-detailsrowexpand{{else}}e-detailsrowcollapse{{/if}}  " +
                                          "e-detailsrowcell'><div class='e-detailsiconcell e-icon {{if isDetailsExpanded}}e-detailsinfoiconhide{{else}}e-detailsinfoiconshow{{/if}}'></div>";
                    rowTemplateHTML += "<td class='{{if isSelected}}e-selectionbackground{{/if}} {{if isDetailsExpanded}}e-detailsrowexpand{{else}}e-detailsrowcollapse{{/if}}  " +
                                        "e-detailsrowcell'><div class='e-detailsiconcell e-icon {{if isDetailsExpanded}}e-detailsinfoiconhide{{else}}e-detailsinfoiconshow{{/if}}'></div>";
                }

                var trRow = "{{if isAltRow}}" + "<tr class='{{:~_" + proxy._id + "rowClassName()}} " +
                    "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} e-treegridrowcollapse {{/if}} e-templaterow "
                    + altRowTemplateElement[0].className + "' style='{{if !isSummaryRow && !footerSummaryRowRecord}}" + altRowTemplateElement[0].style.cssText + "{{else}}height:" + model.rowHeight + "px;{{/if}}display:{{:~_" + proxy._id + "expandStatus(#data)}}'" + "'>" + altRowTemplateHTML + "</tr>"

                    + "{{else}}" + "<tr class='{{:~_" + proxy._id + "rowClassName()}} " +
                    "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} e-treegridrowcollapse {{/if}} e-templaterow "
                    + rowTemplateElement[0].className + "' style='{{if !isSummaryRow && !footerSummaryRowRecord}}" + rowTemplateElement[0].style.cssText + "{{else}}height:" + model.rowHeight + "px;{{/if}}display:{{:~_" + proxy._id + "expandStatus(#data)}}'" + "'>" + rowTemplateHTML + "</tr>" +
                    "{{/if}}";

            }

            else {

                var trRow = "<tr class='e-treegridrows {{:~_" + proxy._id + "rowClassName()}} ";

                if (model.enableAltRow) {
                    trRow += "{{if isAltRow}}e-alt-row{{/if}} ";
                }

              
                var tdCellCollections = "";
                for (columnCount = 0; columnCount < length; columnCount++) {
                    if (columns[columnCount]) {
                        columnName = columns[columnCount].field;
                        column = columns[columnCount];
                        var visible = columns[columnCount].visible == undefined ? true : columns[columnCount].visible;
                        var textAlign = columns[columnCount].textAlign;
                        //To create a seperate div in summary cell to display a summary title.
                        summaryTitlediv = "";
                        if (column.headerText == proxy._visibleColumns[0]) {
                            summaryTitlediv = '{{if isSummaryRow || footerSummaryRowRecord}}' + "<div " +
                                "class='e-summarytitle'" + "style='float:left;display:block;'>" + "{{:~_" + proxy._id + "summaryTitle()}}"
                                 + "</div>" + '{{/if}}';
                        }

                        //CHECK THE template COLUMN OR NOT
                        if (column["isTemplateColumn"]) {

                            trRow += "e-templaterow "

                            var templateId = column["templateID"],
                                template = column["template"],
                                angularTemplate = column["angularTemplate"];

                            if (angularTemplate) {
                                $("#" + proxy._id + column.headerText + columnCount + "_Template").remove();
                                var scriptElement = this._createTemplateElement(column);

                                tdCell = "<td " +
                                     "class='{{if !isSummaryRow && !footerSummaryRowRecord}} e-intend{{/if}} e-rowcell e-templatecell{{if isSelected}} e-selectionbackground{{/if}}{{if " + !visible + "}} e-hide{{/if}}'" +
                                     "style='text-Align:" + textAlign + "'" +
                                     "role='gridcell' tabindex='-1'>" +
                                     '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                     "{{:~_treeGridTemplating('" + scriptElement.id + "')}}" +
                                     '{{else}}' + summaryTitlediv + '{{/if}}';

                                     if (!ej.isNullOrUndefined(column["format"]))
                                         tdCell += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}" + '{{/if}}';
                                     else
                                         tdCell += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" + '{{/if}}';
                                     tdCell += "</td>";
                            }
                            else if (templateId) {
                                
                                tdCell = "<td "+
                                     "class='{{if !isSummaryRow && !footerSummaryRowRecord}} e-intend{{/if}} e-rowcell e-templatecell{{if isSelected}} e-selectionbackground{{/if}}{{if " + !visible + "}} e-hide{{/if}}'" +
                                     " style='text-Align:" + textAlign + ";padding-left:10px;'" +
                                     " role='gridcell'" +
                                     " tabindex='-1'" +
                                     " aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                                     '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                      $("#" + templateId)[0].innerHTML +
                                      '{{else}}' + summaryTitlediv + '{{/if}}';

                                if (!ej.isNullOrUndefined(column["format"]))
                                    tdCell += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}" + '{{/if}}';
                                else
                                    tdCell += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" + '{{/if}}';

                                tdCell += "</td>";

                            }
                            else if (template) {

                                tdCell = "<td " +
                                     "class='{{if !isSummaryRow && !footerSummaryRowRecord}} e-intend{{/if}} e-rowcell e-templatecell{{if isSelected}} e-selectionbackground{{/if}}{{if " + !visible + "}} e-hide{{/if}}'" +
                                     " style='text-Align:" + textAlign + "';padding-left:10px;" +
                                     " role='gridcell'" +
                                     " tabindex='-1'" +
                                     " aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                                     '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                     template +
                                       '{{else}}' + summaryTitlediv + '{{/if}}';

                                if (!ej.isNullOrUndefined(column["format"]))
                                    tdCell += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}" + '{{/if}}';
                                else
                                    tdCell += '{{if isSummaryRow || footerSummaryRowRecord}}' + "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}" + '{{/if}}';

                                tdCell += "</td>";
                            }

                            //EXPAND COLLAPSE COLUMN BASED ON THE TREECOLUMNINDEX
                        } else if (columnCount === model.treeColumnIndex) {

                            tdCell = "<td " +
                                 "class='{{if isSelected}}e-rowcell e-selectionbackground e-intend {{else}}e-rowcell{{if !isSummaryRow && !footerSummaryRowRecord}} e-intend{{/if}} {{/if}} {{:~_" + proxy._id + "SummaryRowtdClassName()}}'" +
                                 "style='padding-left:{{if ~_" + proxy._id + "isTreeColumnIndex('" + columnName + "') && isSummaryRow }}{{:(level*20)}}" + "px{{/if}};background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                 "{{:cellBackgroundColor}}{{else}}none{{/if}};'" +
                                 "role='gridcell'" +
                                 "tabindex='-1'" +
                                 "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                                 '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                 "<div class='e-inner-treecolumn-container' style='height:20px;width:{{if isSummaryRow}}100%{{else}}{{:(level+1)*30}}px{{/if}};' unselectable='on'>" +
                                 "{{if hasChildRecords && hasFilteredChildRecords}}" +
                                 "<div " +
                                 "class='intend'" +
                                 "style='height:1px; float:left; width:{{:level*20}}" + "px; display:inline-block;'>" +
                                 //indicators include for drag and drop
                                 "{{if !~getState() || !hasChildRecords}}" +
                                  "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                 "<span class='aboveIcon e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='belowIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='childIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='cancelIcon e-icon' style='position:relative;float:right;display:none;'></span>" +
                                 "</div>" +
                                 "{{/if}}"+
                                 "</div>" +
                                 "{{else !hasChildRecords || (!hasFilteredChildRecords && hasChildRecords)}}" +
                                 "<div " +
                                 "class='intend'" +
                                 "style='height:1px; float:left; width:{{:(level+1)*20}}" + "px; display:inline-block;'>" +
                                   //indicator icons included for drag and drop
                                 "{{if !~getState() || !hasChildRecords}}" +
                                 "<div class={{:~_" + proxy._id + "rowName()}} >"+
                                 "<span class='aboveIcon e-icon'  style='position:relative;float:right;display:none;'></span>" +
                                 "<span class='belowIcon e-icon' style='float:right;display:none;'></span>" +
                                 "<span class='childIcon e-icon' style='float:right;display:none;'></span>" +
                                 "<span class='cancelIcon e-icon' style='float:right;display:none;'></span>" +
                                 "</div>" +
                                  "{{/if}}" +
                                 "</div>" +
                                 "{{/if}}" +
                                 //indicator icons included for drag and drop
                                 "{{if ~getState() && hasChildRecords}}" +
                                 "<div " +
                                 "class='intendparent'>" +
                                 "<div class={{:~_" + proxy._id + "rowName()}} >" +
                                 "<span class='aboveIcon e-icon'  style='position:relative;float:left;display:none;'></span>" +
                                 "<span class='belowIcon e-icon' style='float:left;display:none;'></span>" +
                                 "<span class='childIcon e-icon' style='float:left;display:none;'></span>" +
                                 "<span class='cancelIcon e-icon' style='float:left;display:none;'></span>" +
                                 "</div>" +
                                 "</div>" +
                                 "{{/if}}" +
                                 "<div " +
                                 "class='{{if expanded && hasFilteredChildRecords}}e-treegridexpand e-icon{{else hasChildRecords && hasFilteredChildRecords}}" +
                                 "e-treegridcollapse e-icon{{/if}}'" +
                                 "style='float:left;display:inline-block; font-size:10px;' unselectable='on'>" +//unselectable on icon div
                                 "</div>" +
                                 "<div class='e-cell'" +
                                 "style='display:inline-block;width:100%;padding-left:{{if ~_" + proxy._id + "isTreeColumnIndex('" + columnName + "') && isSummaryRow}}{{:(level+1)*20}}" + "px{{/if}};' unselectable='on'>" +
                                '{{/if}}';
                            if (!ej.isNullOrUndefined(column["format"]))
                                tdCell += "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}";
                            else {                               
                                tdCell += "{{:#data['" + columnName + "']}}" + summaryTitlediv +
                                '{{if !isSummaryRow && !footerSummaryRowRecord}}' +
                                "</div>" +
                                "</div>" +
                                "</td>" +
                                '{{/if}}';
                                summaryTitlediv = "";
                            }
                            //CHECK RESOURCE COLUMN FROM GANTT OR NOT
                        } else if (model.isFromGantt && column.mappingName === model.resourceInfoMapping) {
                            helpers = {};
                            helpers["_" + proxy._id + "resourceName"] = proxy._getResourceName;
                            $.views.helpers(helpers);

                            tdCell = "<td  " +
                                 "class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}} {{if " + !visible + "}}e-hide{{/if}}'" +
                                 "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                 "{{:cellBackgroundColor}}{{else}}none{{/if}};'" +
                                 "role='gridcell'" +
                                 "tabindex='-1'" +
                                 "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                                 "{{:~_" + proxy._id + "resourceName('" + proxy._id + "Object','"
                                 + model.resourceNameMapping + "','" + columnName + "')}}" +
                                 "</td>";
                            //CHECK PREDECESSOR COLUMN FROM GANTT OR NOT
                        } else if (model.isFromGantt && column.mappingName === model.predecessorMapping) {

                            helpers = {};
                            helpers["_" + proxy._id + "predecessor"] = proxy._getPredecessorsValue;
                            $.views.helpers(helpers);

                            tdCell = "<td " +
                                "class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell {{/if}} {{if " + !visible + "}}e-hide{{/if}}' " +
                                "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                "{{:cellBackgroundColor}}{{else}}none{{/if}};'" +
                                "role='gridcell'" +
                                "tabindex='-1'" +
                                "aria-label={{:~_" + proxy._id + "itemDetails(#data)}}>" +
                                "{{:~_" + proxy._id + "predecessor('" + proxy._id + "Object','"
                                + model.predecessorMapping + "','" + columnName + "')}}" +
                                "</td>";

                        }
                        else {
                            tdCell = "<td " +
                                "class='{{if isSelected}}e-rowcell e-selectionbackground{{else}}e-rowcell{{/if}} {{if " + !visible + "}}e-hide{{/if}} {{:~_" + proxy._id + "SummaryRowtdClassName()}}'" +
                                "style='background-color:{{if cellBackgroundColor && ~_" + proxy._id + "checkColumn('" + columnName + "',#data)=='" + columnName + "'}}" +
                                "{{:cellBackgroundColor}}{{else}}none{{/if}};text-Align:" + textAlign + "'" +
                                "role='gridcell'" +
                                "tabindex='-1'" +
                                "aria-label={{:~_" + proxy._id + "itemDetails(#data)}} unselectable='on'>" + summaryTitlediv;

                            if (!ej.isNullOrUndefined(column["format"]))
                                tdCell += "{{:~_" + proxy._id + "formatting('" + column["format"] + "',~_" + proxy._id + "cellValue('" + columnName + "'),'" + this.model.locale + "')}}";
                            else {
                                tdCell += "{{:~_" + proxy._id + "cellValue('" + columnName + "')}}";

                            }    
                            tdCell+= "</td>";//unselectable for 
                            summaryTitlediv = "";
                        }

                        tdCellCollections += tdCell;
                    }
                }

                /* add details column to end of header cell*/
                if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn) {
                    tdCellCollections += "<td class='{{if isSelected}}e-selectionbackground{{/if}} {{if isDetailsExpanded}}e-detailsrowexpand{{else}}e-detailsrowcollapse{{/if}}  " +
                                         "e-detailsrowcell'>{{if !isSummaryRow && !footerSummaryRowRecord}}<div class='e-detailsiconcell e-icon {{if isDetailsExpanded}}e-detailsinfoiconhide{{else}}e-detailsinfoiconshow{{/if}}'></div>{{/if}}";
                }

                trRow += "{{if expanded && hasFilteredChildRecords}}e-treegridrowexpand {{else hasChildRecords && hasFilteredChildRecords}} " +
                      "e-treegridrowcollapse {{/if}} '" +
                      "style='background-color:{{if rowBackgroundColor }}{{:rowBackgroundColor}}{{else}}none{{/if}}; height:" + model.rowHeight + "px;" +
                      "display:{{:~_" + proxy._id + "expandStatus(#data)}}'" +
                      "role='row'>";

                trRow += tdCellCollections;

                if (model.isFromGantt) {
                    trRow += "<td class='e-extendcolumn'></td></tr>";
                } else {
                    trRow += "</tr>";
                }
            }

            if (model.showDetailsRow && model.detailsTemplate)
            {
                trRow +="{{if isDetailsExpanded}}" + proxy._createDetailsRowTemplate() + "{{/if}}";
            }

            var templates = {};
                templates[proxy._id + "_Template"] = trRow;
            $.templates(templates);

            

        },
        
        //#endregion template

        //#region template METHODS

        _gridTemplate: function (self, templateId) {
            return self._renderEjTemplate("#" + templateId, this.data, this.data.index);
        },

        //method for calculating the indent width for expander column
        _getIndentWidth:function(data){
            var level = data.level;
            if (data.hasChildRecords && data.hasFilteredChildRecords) {
                return (level * 20) + "px";
            }
            else {
                return ((level + 1) * 20) + "px";
            }
        },


        //creating template element for angulartemplate in details row template
        _createDetailRowTemplateElement: function (innerHtml) {
            var scriptElement = document.createElement("script");
            scriptElement.id = ("treeGridDetailsRowTemplateElement" + this._id).split("").join("");
            scriptElement.type = "text/x-template";
            scriptElement.text = innerHtml;
            $("body").append(scriptElement);
            return scriptElement;
        },


        //creating template element for angulartemplate in row template
        _createRowTemplateElement: function (innerHtml, id) {
            var scriptElement = document.createElement("script");
            scriptElement.id = id;
            scriptElement.type = "text/x-template";
            scriptElement.text = innerHtml;

            $("body").append(scriptElement);
            return scriptElement;
        },

        //creating a template element for angularjs template
        _createTemplateElement: function (column) {
            var scriptElement = document.createElement("script");
            scriptElement.id = ("treeGrid" + column.headerText + $.inArray(column, this.model.columns) + "_Template").split(" ").join("");
            scriptElement.type = "text/x-template";
            scriptElement.text = $(column["angularTemplate"]).html();



            $("body").append(scriptElement);
            return scriptElement;
        },

        //CONVERT RECORD OBJECT TO STRING
        _convertItemToString: function (data) {

            var str = '',
                labelObject = {},
                columns = this.model.columns;

            for (var i = 0; i < columns.length; i++) {
                if (columns[i]) {
                    labelObject[columns[i].field] = data.item[columns[i].field];
                }
            }

            for (var p in labelObject) {
                if (labelObject.hasOwnProperty(p)) {
                    str += p + '' + labelObject[p] + '\n';
                }
            }

            return str;
        },

        _getCellValue: function (columnName) {

            var cellValue = this.data[columnName];

            if (cellValue || cellValue===0) {
                return cellValue;
            } else {
                return this.data.item && this.data.item[columnName];
            }


        },
        _isTreeColumnIndex: function (columnName) {
            var proxy = this; model = proxy.model,
            currentColumn = model.columns[model.treeColumnIndex];
            if (currentColumn && currentColumn.field == columnName)
                return true;
            else
                return false;
        },
        _getSummaryTitle: function () {
            var proxy = this;
            var summaryTitleValue = proxy.data.summaryRow.title;
            if (summaryTitleValue)
                return summaryTitleValue;
            else
                return proxy.data.item && proxy.data.item.summaryRow.title;
        },

        _getCellColumn: function (columnName,data) {
            var proxy = this;

            if (data.treeMappingName.indexOf(columnName) !== -1) {
                return columnName;
            }
            else {
                return "none"
            }


        },

        //GET RESOURCE NAME FOR RESOURCE COLUMN IN THE GANTT CONTROL
        _getResourceName: function (gridObject, mappingName, columnName) {

            var proxy = this,
                data = proxy.data[columnName],
                count = 0,
                length = data && data.length,
                resourceName = [];

            if (data && data.length > 0) {
                for (count; count < length; count++) {
                    resourceName.push(data[count][mappingName]);
                }
            }
            return resourceName;
        },


        //GET PREDECESSOR VALUE FOR PREDECESSOR COLUMN IN THE GANTT CONTROL
        _getPredecessorsValue: function (gridObject, mappingName) {

            return this.data.item[mappingName];

        },


        //GET THE EXPAND COLLAPSE STATUS OF TABLE ROW
        _getExpandStatusRecord: function (data) {

            var proxy = this;

            if (proxy.getExpandStatus(data)) {

                return 'table-row';

            }

            return 'none';
        },

        //#endregion TEMPLATE METHODS

        //#region CONVERT HIERARCHICAL RECORDS TO FLAT RECORDS
        //CREATE FLATRECORDS FROM HIERARCHICAL DATASOURCE
        _createRecords: function (dataSource) {

            var proxy = this,
                model = proxy.model,
                ids=proxy.model.ids,
                flatRecords = model.flatRecords,
                length = dataSource.length,
                count = 0,
                parentRecord,
                parentRecords = model.parentRecords;

            for (count; count < length; count++) {

                parentRecord = proxy._createRecord(dataSource[count], 0, null);
                proxy._storedIndex++;

                //CHECK ALTROW ENABLED OR NOT
                
                parentRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                

                //CHECK SORTING ENABLED OR NOT
                
                parentRecords.push(parentRecord);
                

                parentRecord.index = proxy._storedIndex;
                ids[proxy._storedIndex] = parentRecord.index;
                flatRecords.push(parentRecord);
                if (parentRecord.hasChildRecords)
                    proxy._parentRecords.push(parentRecord);
                parentRecord.childRecords && proxy._addNestedRecords(parentRecord.childRecords);

            }
            /* assign records length as max index */
            proxy._maxRowIndex = this.model.flatRecords.length;
        },


        //CREATE RECORD OBJECT
        _createRecord: function (data, level, parentItem, expanded) {

            var proxy = this,
                record,
                childDataSource = data[proxy.model.childMapping], model = this.model;

            //CLONE THE DATA OBJECT
            record = $.extend({}, data);
            var tempData;
            if (model.parentIdMapping && proxy._createdAt==="load") {
                tempData = ej.DataManager(proxy._retrivedData).executeLocal(ej.Query().where(model.idMapping,
                ej.FilterOperators.equal,
                record[model.idMapping]));
                record.item = tempData[0];
            }
            else {

                if (this.dataSource() instanceof ej.DataManager && this.dataSource().dataSource.json && this.dataSource().dataSource.offline && proxy._createdAt === "load") {
                    var tempData = ej.DataManager(proxy._retrivedData).executeLocal(ej.Query().where(model.idMapping,
                    ej.FilterOperators.equal,
                    record[model.idMapping]));
                    if (tempData.length > 0) {
                        record.item = tempData[0];
                    } else {
                        record.item = data;
                    }
                }
                else {
                    record.item = data;
                }
            }
            record.parentItem = parentItem;
            record.childRecords = childDataSource && proxy._createChildRecords(childDataSource, level + 1, record);
            record.hasChildRecords = childDataSource && childDataSource.length ? true : false;
            record.expanded = expanded !== undefined ? expanded : childDataSource ? childDataSource.length > 0 : false;
            record.dragState = true;
            record.isSelected = false;
            record.hasFilteredChildRecords = true;//Used to Indicates Parent records can expandable or not on Filtering operation
            record.level = level;
            record.cellBackgroundColor = null;
            record.rowBackgroundColor = null;
            record.treeMappingName = [];
            if (model.showDetailsRow && model.detailsTemplate) {
                record.isDetailsExpanded = model.showDetailsRowInfoColumn ? false : true;
            } else {
                record.isDetailsExpanded = false;
            }
            delete record[model.childMapping];
            return record;
        },


        //CREATE CHILD GANTT RECORDS
        _createChildRecords: function (childDataSource, level, parentItem) {

            var proxy = this,
                records = [],
                count = 0,
                length = childDataSource.length,
                record = null,
                childRecord;

            for (count = 0; count < length; count++) {

                record = childDataSource[count];

                if (record) {
                    childRecord = proxy._createRecord(record, level, parentItem);
                    records.push(childRecord);
                }
            }

            return records;

        },


        //ADD NESTED RECORDS TO FLATRECORDS
        _addNestedRecords: function (childDataSource) {

            var proxy = this,
                model = proxy.model,
                ids=proxy.model.ids,
                flatRecords = model.flatRecords,
                updatedRecords = model.updatedRecords,
                count = 0,
                length = childDataSource.length,
                records = [],
                record;
                

            for (count = 0; count < length; count++) {

                record = childDataSource[count];
                proxy._storedIndex++;

                //CHECK ALTROW ENABLED OR NOT
                
                record.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                

                record.index = proxy._storedIndex;
                ids[proxy._storedIndex] = record.index;
                flatRecords.push(record);
                updatedRecords.push(record);
                if (record.hasChildRecords)
                    proxy._parentRecords.push(record);
                records = record.childRecords;

                if (records) {

                    var j = 0,
                        recordlength = records.length,
                        childRecord = null;

                    for (j = 0; j < recordlength; j++) {

                        childRecord = records[j];
                        proxy._storedIndex++;     
                        childRecord.index = proxy._storedIndex;
                        ids[proxy._storedIndex] = childRecord.index;
                        //CHECK ALTROW ENABLED OR NOT
                        
                        childRecord.isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        
                        flatRecords.push(childRecord);
                        updatedRecords.push(record);
                        if (childRecord.hasChildRecords)
                            proxy._parentRecords.push(childRecord);
                        childRecord.childRecords && proxy._addNestedRecords(childRecord.childRecords);
                    }
                }
            }
        },

        //#endregion CONVERT HIERARCHICAL RECORDS TO FLAT RECORDS

        //#region METHOD FOR EVENTS

        //INITIALIZE THE EVENTS
        _wireEvents: function () {

            var proxy = this,
                model = proxy.model,
                matched = jQuery.uaMatch(navigator.userAgent);

            proxy._$gridContent.bind("scroll", $.proxy(proxy._onScroll, proxy));
            proxy._on(proxy.element, "click", proxy._onClick);
            proxy._on(proxy.element, "click", "#" + proxy._id + "e-gridheader", proxy._onHeaderClick);
            proxy._on(proxy.element, "mousedown", "#" + proxy._id + "e-gridheader", proxy._headerMouseDown);
            proxy._on($(document), "mousedown", proxy._mouseDownHandler);
            if (!model.isFromGantt && model.allowDragAndDrop) {
                proxy._on(proxy.element, "mousedown", ".e-gridcontent", proxy._contentMouseDown);
               
                //Events binded for Row drag and drop
                proxy._on(proxy.element, "mousedown", ".e-treegridrows, .e-templatecell", proxy.dragRecord);
                proxy._on($(document), "mouseup", proxy.dragMouseUp);

                // touch events binded for row drag and drop 
                if (matched.browser.toLowerCase() == "chrome") {
                    proxy._on(proxy.element, "touchstart", ".e-treegridrows, .e-templatecell", proxy.dragRecord);
                    $(document.body).bind("touchmove", function (event) {
                        event.preventDefault();
                    });
                    $(document.body).bind("touchmove", $.proxy(proxy.dragToolTip, proxy));
                    proxy._on(proxy.element, "touchend", ".e-treegridrows, .e-templatecell", proxy.dragMouseUp);
                    proxy._on(proxy.element, "touchleave", ".e-treegridrows, .e-templatecell", proxy.dragMouseUp);
                    $(proxy._$bodyContainer).bind("touchend", $.proxy(proxy.dragMouseUp, proxy));
                    $(proxy._$bodyContainer).bind("touchleave", $.proxy(proxy.dragMouseUp, proxy));
                }
            }
            //Method binding for filtering data.
            if (model.filterSettings.filterBarMode == "immediate") {
                proxy._on(proxy.element, "keyup", ".e-filterbarcell input", proxy._filterBarHandler);
                proxy._off(proxy.element, "click", ".e-filterbarcell .e-checkbox")._on(proxy.element, "click", ".e-filterbarcell .e-checkbox", proxy._filterBarHandler);
            }
            proxy._on(proxy.element, "focus click", ".e-filterbarcell input", proxy._filterBarClose);
            proxy._on(proxy.element, "click", ".e-filterbarcell .e-cancel", proxy._filterBarClose);
            if (proxy.model.showGridCellTooltip && !proxy.model.showGridExpandCellTooltip) {

                if (ej.browserInfo().name == "msie") {
                    proxy._on(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
                }
                else {
                    proxy._on(proxy.element, "mousemove", ".e-rowcell", proxy._mouseHover);
                }
            }
            else {
                proxy._on(proxy.element, "mouseenter", ".e-rowcell", proxy._mouseHover);
            }
            proxy._on(proxy.element, "mouseleave", ".e-rowcell", proxy._cellMouseLeave);
            proxy._on(proxy.element, "mouseleave", proxy._cellMouseLeave);
            proxy._enableColumnResizeEvents();

            //MOUSE WHEEL EVENT FOR GANTT CONTROL
            if (model.isFromGantt) {
                proxy._on(proxy.element, "mousewheel DOMMouseScroll", proxy._mouseWheel);
            }

            if (model.enableResize && !model.isFromGantt) {
                proxy._on($(window), "resize", proxy._windowResize);
            }
            proxy._enableEditingEvents();
            //Mouse right click event for tree grid
            if (!proxy.model.isFromGantt && model.contextMenuSettings.showContextMenu)
                proxy._on(proxy.element, "contextmenu", proxy._rightClick);

            if (model.contextMenuSettings.showContextMenu) {
                proxy._on(proxy.element, "keyup", this._preventContextMenu);
            }

        },
        /* prevent context menu action by menu option key*/
        _preventContextMenu: function (e) {
            if (e.keyCode == 93) {
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        },
        _mouseDownHandler:function(e)
        {
            var proxy = this,
                target = $(e.target);
            //Check the whether the mouse down on column menu and column menu items
            if (target.closest("div.e-columnmenuitem").length == 0 && target.closest("div.e-columnMenuListDiv").length == 0 && !target.hasClass("e-columnmenu-icon"))
                proxy._clearColumnMenu();
            //Check the whether the mouse down on context menu items
            if (target.closest("div.e-menuitem").length == 0)
                proxy._clearContextMenu();
            //Check the detail template items on mouse down event
            if (target.closest("div.e-detailscellwrapperfly").length == 0)
                proxy._removeDetailsRow(e);
            //Save the edited mode cells when click on the document.
            if (target.closest("div.e-datepicker").length == 0 && target.closest("div.e-datetime-popup").length == 0 &&
                 target.closest("div.e-ddl-popup").length == 0 && !target.hasClass("e-tooltxt") && !target.hasClass("e-toolbaricons") && target.closest("div.e-menuitem").length == 0) {
                if (proxy._isRowEdit || proxy.model.isEdit)
                    proxy._endEdit();                               
            }
        },
        // Auto scroll the content if mouse move out of grid area while dragging a row.
        _mouseout: function (e) {
            var proxy = this,
                model = proxy.model,
                topvalue,
                position = proxy.getOffsetRect($("#" + proxy._id)[0]),
                height = proxy._$gridHeaderContainer.height() + position.top;

            if (model.toolbarSettings.showToolbar)
                height += $("#" + proxy._id + "_toolbarItems").height();
            if (proxy._dragMouseDown) {
                if (e.originalEvent.pageX || e.originalEvent.pageY) {
                    posx = e.originalEvent.pageX;
                    posy = e.originalEvent.pageY;
                }
                else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                    posx = e.originalEvent.clientX + document.body.scrollLeft
                       + document.documentElement.scrollLeft;
                    posy = e.originalEvent.clientY + document.body.scrollTop
                        + document.documentElement.scrollTop;
                }
                else if (e.originalEvent && e.originalEvent.changedTouches
                   && e.originalEvent.changedTouches.length > 0) {
                    posx = e.originalEvent.changedTouches[0].pageX;
                    posy = e.originalEvent.changedTouches[0].pageY;
                }
                if ((posx + 200) > proxy._windowWidth) {
                    posx -= (posx + 200) - proxy._windowWidth;
                }
                if ((posy + model.rowHeight) > proxy._windowHeight) {
                    posy -= (posy + model.rowHeight) - proxy._windowHeight;
                }

                var max = proxy.isVScroll() ? proxy._$gridContent.find(".e-content").get(0).scrollHeight - 1 : 0;
                //Condition to check whether mouse move above the content or below the content.
                if (posy > (proxy._viewPortHeight + height) || (posy < height && proxy._scrollTop != 0)) {
                    //condition to whether the mouse pointer below the content.

                    if (posy > proxy._viewPortHeight + height) {
                        //Condition to check whether the scroller reaches the end.

                        if (max > (proxy._scrollTop + proxy._viewPortHeight)) {
                            if (!_timerDragDown) {

                                //Timer started for scrolling the content till the end.
                                _timerDragDown = window.setInterval(function () {
                                    if (max > (proxy._scrollTop + proxy._viewPortHeight)) {
                                        topvalue = proxy._scrollTop;
                                        topvalue = topvalue + model.rowHeight;
                                        proxy._$gridContent.ejScroller("scrollY", topvalue, true);                       
                                    }
                                }, 100);
                            }
                        }
                        else {
                            _timerDragDown && (_timerDragDown = window.clearInterval(_timerDragDown));
                        }
                    }
                    else {
                        _timerDragDown  && (_timerDragDown = window.clearInterval(_timerDragDown));                        
                    }
                    //condition to check if the mouse points above to content area.
                    if (posy < height && proxy._scrollTop != 0) {
                        if (proxy._scrollTop > 0) {
                            if (!_timerDragUp) {
                                _timerDragUp = window.setInterval(function () {
                                    if (proxy._scrollTop > 0) {
                                        topvalue = proxy._scrollTop;
                                        topvalue = topvalue - model.rowHeight;
                                        if (topvalue < 0)
                                            topvalue = 0;
                                        proxy._$gridContent.ejScroller("scrollY", topvalue, true);
                                    }
                                }, 100);
                            }
                        }
                        else {
                            _timerDragUp && (_timerDragUp = window.clearInterval(_timerDragUp));
                        }
                    }
                    else {
                        _timerDragUp && (_timerDragUp = window.clearInterval(_timerDragUp));                        
                    }
                }
                else {
                    // cleared timer once the mouse enters into the grid area.
                    _timerDragDown && (_timerDragDown = window.clearInterval(_timerDragDown));
                    _timerDragUp && (_timerDragUp = window.clearInterval(_timerDragUp));
                }
            }
        },
        // row selection while selecting row for drag and moving row to destination.
        _dragRowSelection: function (e) {
            var proxy = this,
                model = this.model,
                $target = $(e.target),
                $tr = $target.closest('tr'),
                recordIndex = proxy.getRows().index($target.closest('tr'));
            if (recordIndex != -1) {
                record = model.currentViewData[recordIndex];
                var args = { rowElement: $tr, recordIndex: this.selectedRowIndex() };
                if (!proxy._trigger('rowSelecting', args)) {
                    model.selectedItem = record;
                    proxy._rowDragIndexes = [];
                    proxy._rowDragIndexes.push(model.updatedRecords.indexOf(record));
                    this.selectedRowIndex(model.updatedRecords.indexOf(record));
                    proxy.selectRows(this.selectedRowIndex());
                    args = {
                        rowElement: $tr,
                        data: model.selectedItem,
                        target: "ejTreeGrid",
                        recordIndex: proxy.selectedRowIndex(),
                    };
                    proxy._trigger("rowSelected", args);
                }
            }
        },
        // This method is to get the row range to enable the drag row indicators consequently.
        getRange: function (record, index,$tr) {
            
            var proxy = this,
                model = proxy.model;
                roundOff = 0,
                rowHeight = $tr[0].offsetHeight,
                position = proxy.getOffsetRect($("#" + proxy._id)[0]),
                height = proxy._$gridHeaderContainer.height() + position.top;
                rowHeight += proxy._detailsRowHeight;

            if (model.toolbarSettings.showToolbar)
                height += $("#" + proxy._id + "_toolbarItems").height();
            if(proxy._scrollTop!=0)
                roundOff = rowHeight - proxy._scrollTop % rowHeight;
            // 7 is to equalize the height.
            var rowTop = (index * rowHeight) + height + roundOff + 7,
                rowBottom = rowTop + $tr[0].offsetHeight,
                difference = rowBottom - rowTop,
                divide = difference / 3,
                topRowSegment = rowTop + divide,
                middleRowSegment = topRowSegment + divide,
                bottomRowSegment = middleRowSegment + divide,
                position = {
                    topRowSegment: topRowSegment,
                    middleRowSegment: middleRowSegment,
                    bottomRowSegment: bottomRowSegment
                };
            return position;
        },
        //Child validation to display the child indicator.
        _childValidation: function (record) {            
            var childIndex,
                proxy = this,
                draggedRecord = proxy._draggedRecord,
                parentItem = draggedRecord.parentItem,
                childRecords;
            if (parentItem) {
                childRecords = parentItem.childRecords;
                childIndex = childRecords.indexOf(this._draggedRecord);
                if (record == parentItem && childIndex != -1)
                    this._childItem = true;
            }
        },
        
        //This method is to render the tooltip while dragging.
        _renderToolTip: function (e,element,args) {
            var proxy = this,
                $target = $(e.target),
                model = this.model,
                positions = [],
                doc = document,
                item,
                columns = proxy.model.columns,
                tooltipItemLength = model.dragTooltip.tooltipItems.length,
                $tr = $target.closest('tr'),
                $td,
                recordIndex = proxy.getRows().index($tr),
                positions = {}, records, dragRecIndex, index;
            proxy._insertAbove = false;
            proxy._insertAsChild = false;
            proxy._insertBelow = false;
            proxy._childItem = false;
            proxy._currentRecord = model.currentViewData[recordIndex];
            item = proxy._draggedRecord;
            // disable all the icons initially.
            $(".intend").find(".e-icon").css({ "display": "none" });
            $(".intendparent").find(".e-icon").css({ "display": "none" });
            $(".e-treegridexpand").css({ "display": "inline-block" });
            $(".e-treegridcollapse").css({ "display": "inline-block" });           
               
            // to get the record index to find its mouse position.
            if (!model.rowTemplateID) {
                records = proxy.getExpandedRecords(model.updatedRecords);
                proxy.getVisibleRange();
                    currentViewData = records.slice(proxy._visibleRange.top,
                proxy._visibleRange.bottom);
                    if (proxy._scrollTop != 0) {
                        dragRecIndex = model.updatedRecords.indexOf(proxy._draggedRecord) - 1;
                        index = currentViewData.indexOf(proxy._currentRecord) - 1;
                    }
                    else {
                        dragRecIndex = model.updatedRecords.indexOf(proxy._draggedRecord);
                        var index = currentViewData.indexOf(proxy._currentRecord);
                    }
                }
                else {
                    records = model.updatedRecords;
                    index = model.updatedRecords.indexOf(proxy._currentRecord);
            }
            var currentIndex = model.updatedRecords.indexOf(proxy._currentRecord),
                isNextRecord = proxy._checkPrevNextRecord(proxy._draggedRecord, currentIndex, proxy._currentRecord.level, "next"),
                isPrevRecord = proxy._checkPrevNextRecord(proxy._draggedRecord, currentIndex, proxy._currentRecord.level, "prev");
            
            
            positions = proxy.getRange(proxy._currentRecord, index, $tr);
            if (e.originalEvent.pageX || e.originalEvent.pageY) {
                posx = e.originalEvent.pageX;
                posy = e.originalEvent.pageY;
            }                
            else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                posx = e.originalEvent.clientX + document.body.scrollLeft
                   + document.documentElement.scrollLeft;
                posy = e.originalEvent.clientY + document.body.scrollTop
                    + document.documentElement.scrollTop;
            }
            else if (e.originalEvent && e.originalEvent.changedTouches
                    && e.originalEvent.changedTouches.length > 0) {
                posx = e.originalEvent.changedTouches[0].pageX;
                posy = e.originalEvent.changedTouches[0].pageY;
            }
            if ((posx + 200) > proxy._windowWidth) {
                posx -= (posx + 200) - proxy._windowWidth;
            }
            if ((posy + model.rowHeight) > proxy._windowHeight) {
                posy -= (posy + model.rowHeight) - proxy._windowHeight;
            }
            var isTopRowSegment = posy <= positions.topRowSegment,
                isMiddleRowSegment = (posy > positions.topRowSegment && posy <= positions.middleRowSegment),
                isBottomRowSegment = (posy > positions.middleRowSegment && posy <= positions.bottomRowSegment);
            /* check current record or else render previous records tootip*/
            if (proxy._currentRecord) {
                proxy.data = proxy._currentRecord;
                var classname = this._getrowName();
                proxy._childValidation(proxy._currentRecord);
                // This condition is to disapper the expand collapse icon if it is top most level.
                if (!proxy._currentRecord.parentItem) {
                    if (proxy._currentRecord.hasChildRecords &&
                            !((isTopRowSegment && isPrevRecord) || (isMiddleRowSegment && proxy._childItem) || (isBottomRowSegment && isNextRecord)))
                        if (proxy._currentRecord.expanded)
                            $tr.find(".e-treegridexpand").css({ "display": "none" });
                        else
                            $tr.find(".e-treegridcollapse").css({ "display": "none" });
                }
                else {
                    if (proxy._currentRecord.hasChildRecords)
                        if (proxy._currentRecord.expanded)
                            $tr.find(".e-treegridexpand").css({ "display": "inline-block" });
                        else
                            $tr.find(".e-treegridcollapse").css({ "display": "inline-block" });
                }

                //To render the indicators according to its position.
                if (proxy._currentRecord.dragState == true && args.canDrop != false) {
                    if (isTopRowSegment && !isPrevRecord) {
                        $(proxy.element).find("." + classname).find(".aboveIcon").css({ "display": "block" });
                        proxy._insertAbove = true;
                    }
                    else if (isMiddleRowSegment && proxy._childItem == false) {
                        $(proxy.element).find("." + classname).find(".childIcon").css({ "display": "block" });
                        proxy._insertAsChild = true;
                    }
                    else if (isBottomRowSegment && !isNextRecord) {
                        $(proxy.element).find("." + classname).find(".belowIcon").css({ "display": "block" });
                        proxy._insertBelow = true;
                    }
                }
                else {
                    $(proxy.element).find("." + classname).find(".cancelIcon").css({ "display": "block" });
                    proxy.cancelDrop = true;
                }
            }

            var width = proxy._$gridContainer[0].scrollWidth;

            tooltiptable = ej.buildTag("table", "", { 'padding': '1', "width": "100%","border-collapse":"collapse" }, { 'cellspacing': '1' });
            //to render the tooltip according the option given in the model.
            if (proxy.model.dragTooltip.showTooltip) {
                if (proxy.model.dragTooltip.tooltipTemplate == null && proxy.model.dragTooltip.tooltipItems.length == 0) {
                    if (proxy._dragTooltip == false) {
                        tooltipbody = ej.buildTag("tbody", "", {}, {});
                        element.find(".e-treegridexpand").css({ "display": "none" });
                        element.find("td.e-detailsrowcell").css({ "display": "none" });
                        element.find(".e-treegridcollapse").css({ "display": "none" });
                        $(element.find(".intend")).css({ "width": 0 });
                        element.find(".e-intend").removeClass("e-intend");
                        element.find(".e-rowcell ").removeClass("e-rowcell ").addClass("e-dragrowcell");
                        element.addClass("e-dragtooltipbackground");
                        tooltipbody.append(element);
                        tooltiptable.append(tooltipbody);
                        proxy._dragmouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltiptreegrid" + proxy._id + "", tooltiptable,
                            {
                                "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                'position': 'absolute',
                                'z-index': '5',
                                'padding': '0',
                                'border-radius': '3px',
                                'border-right': '0',
                                'border-bottom': '0',
                                'width': width,
                            }, {});
                        $(proxy._dragmouseOverTooltip).addClass("e-dragtooltipbackground").addClass("e-dragrowcell");
                        $(document.body).append(proxy._dragmouseOverTooltip);
                        proxy._dragTooltip = true;
                    }
                    else {
                        element.find(".e-treegridexpand").css({ "display": "none" });
                        element.find(".e-treegridcollapse").css({ "display": "none" });
                        element.find("td.e-detailsrowcell").css({ "display": "none" });
                        $(proxy._dragmouseOverTooltip).css({
                            "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                            'position': 'absolute',
                            'z-index': '5',
                            'padding': '0',
                            'border-radius': '3px',
                            'border-right': '0',
                            'border-bottom': '0',
                            'width': width,
                        });
                    }
                }
                else if (proxy.model.dragTooltip.tooltipItems != 0) {
                    if (proxy._dragTooltip == false) {
                        tooltipbody = ej.buildTag("tbody", "", {}, {});
                        tooltiptr = ej.buildTag("tr.e-dragtooltipbackground", "", {}, {});
                        for (var j = 0; j < tooltipItemLength; j++) {
                            for (var i = 0; i < columns.length; i++) {
                                //Skip the columns which is not visible in TreeGrid.
                                if (columns[i].visible == false)
                                    continue;
                                if (model.dragTooltip.tooltipItems[j] == columns[i].field) {
                                    var value = !ej.isNullOrUndefined(proxy._draggedRecord[columns[i].field])?proxy._draggedRecord[columns[i].field].toString():false;
                                    if (value) {
                                        $td = ej.buildTag("td.e-dragrowcell", value.toString(), { "padding": "5px", "width": proxy._columnsWidthCollection[i], "height": model.rowHeight }, {});
                                        tooltiptr.append($td);
                                    }                                 
                                    tooltiptr.append($td);
                                    break;
                                }
                            }
                        }
                        tooltipbody.append(tooltiptr);
                        tooltiptable.append(tooltipbody);
                        proxy._dragmouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltiptreegrid" + proxy._id + "", tooltiptable,
                            {
                                "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                'position': 'absolute',
                                'z-index': '5',
                                'padding': '0',
                                'border-right': '0',
                                'border-bottom': '0',
                                'border-radius': '3px',
                                'height': height,
                            }, {});
                        $(proxy._dragmouseOverTooltip).addClass("e-dragtooltipbackground").addClass("e-dragrowcell");
                        $(document.body).append(proxy._dragmouseOverTooltip);
                        proxy._dragTooltip = true;
                    }
                    else {
                        $(proxy._dragmouseOverTooltip).css({
                            "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                            'position': 'absolute',
                            'z-index': '5',
                            'padding': '0',
                            'border-right': '0',
                            'border-bottom': '0',
                            'border-radius': '3px',
                            'height': height,
                        });
                    }
                }
                else if (proxy.model.dragTooltip.tooltipTemplate) {
                    if (proxy._dragTooltip == false) {
                        proxy._dragmouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltiptreegrid" + proxy._id + "", "",
                            {
                                "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                'position': 'absolute',
                                'z-index': '5',
                                'padding': '0',
                                'border-right': '0',
                                'border-bottom': '0',
                                'border-radius': '3px',
                                'height': height,
                            }, {});
                        $(document.body).append(proxy._dragmouseOverTooltip);
                        proxy.tooltipState = "TemplateID";
                        var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "dragtooltipTemplate"](item), {}, {});
                        proxy._dragmouseOverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                        proxy._dragTooltip = true;
                    }
                    else {
                        $(proxy._dragmouseOverTooltip).css({
                            "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                            'position': 'absolute',
                            'z-index': '5',
                            'padding': '0',
                            'border-right': '0',
                            'border-bottom': '0',
                            'border-radius': '3px',
                            'height': height,
                        });
                    }
                }
            }
        },

        //Used for check whether the dragged record is previous or next to current record based on position argument
        //If it is previous or next to current record, this method return true otherwise return false.
        _checkPrevNextRecord:function(record,index,level, position)
        {
            var proxy = this,
                model = proxy.model,
                updatedRecords = model.updatedRecords,
                currentIndex = index;
            position == "next" ? index++ : index--;
            while(updatedRecords[index])
            {
                if (updatedRecords[index].level == level)
                {
                    if (updatedRecords[index] == record && record.parentItem == updatedRecords[currentIndex].parentItem)
                        return true;
                    return false;
                }
                position == "next" ? index++ : index--;
            }
            return false;
        },
        //Tooltip template for row drag and drop
        _createDragTooltipTemplate: function () {
            var proxy = this,
                td,               
                template = proxy.model.dragTooltip.tooltipTemplate,
                parentTr;
            if (template) {
                var templateString = template.charAt(0);
             
                if (templateString == "#")
                     parentTr = $(proxy.model.dragTooltip.tooltipTemplate)[0].innerHTML;
                else
                     parentTr = proxy.model.dragTooltip.tooltipTemplate;
            }            
            var templates = {};
            templates[proxy._id + "dragtooltipTemplate"] = parentTr;
            $.templates(templates);
        },       

        //render the cancel icon if drag over its child.
		_updateDragStateFalse: function (record, count) {
		    var currentRecord, proxy = this, length;
		    record.dragState = false;		   
		    if (record.hasChildRecords) {
		        length = record.childRecords.length;
		        for (var i = 0; i < length; i++) {
		            currentRecord = record.childRecords[i];
		            currentRecord.dragState = false;
		            if (currentRecord.hasChildRecords) {
		                count = proxy._updateDragStateFalse(currentRecord, count);
		            }
		        }
		    }
		    return;
		},

        //Turn all the records dragState to true once dragging finished.
		_updateDragStateTrue: function (record, count) {
		    var currentRecord, proxy = this, length;
		    if (!ej.isNullOrUndefined(record)) {
		        record.dragState = true;
		        if (record.hasChildRecords) {
		            length = record.childRecords.length;
		            for (var i = 0; i < length; i++) {
		                currentRecord = record.childRecords[i];
		                currentRecord.dragState = true;
		                if (currentRecord.hasChildRecords) {
		                    count = proxy._updateDragStateTrue(currentRecord, count);
		                }
		            }
		        }
		    }
		    return;
		},
        //Mouse down action for to drag a record   
		dragRecord: function (e) {
            /*verify ctlkey and shift key and mouse button*/
		    if (e.ctrlKey || e.shiftKey || e.which == 2 || e.which == 3)
		        return true;

		    var $target = $(e.target),
               proxy = this,
               model=proxy.model,
               recordIndex,
               args = {},
               length,
               $tr = $target.closest('tr');
            
		    if (!$tr.hasClass("e-editedrow")) {
		        proxy._clearContextMenu();
		        proxy._clearColumnMenu();
		        length = proxy._selectedRowsIndexes.length;

		        proxy._posx = e.pageX;
		        proxy._posy = e.pageY;
		        if (!model.enableVirtualization)
		            proxy._currentIndex = proxy.getRows().index($tr);
		        else {
		            proxy._currentIndex = proxy.getRows().index($tr);
		            record = model.currentViewData[proxy._currentIndex];
		            proxy._currentIndex = model.updatedRecords.indexOf(record);
		        }
		        recordIndex = proxy.getRows().index($target.closest('tr'));
		        proxy._draggedRecord = proxy.model.currentViewData[recordIndex];
		        args.draggedRowIndex = recordIndex;
		        args.draggedRow = proxy._draggedRecord;
		        proxy._trigger("rowDragStart", args);
		        if (args.draggedRow && args.draggedRow.isSummaryRow)
		            args.cancel = true;
		        // For normal treegrid records
		        proxy._on(proxy.element, "mousemove", ".e-treegridrows", proxy.dragToolTip);
		        // For Template treegrid records.
		        proxy._on(proxy.element, "mousemove", ".e-templatecell  ", proxy.dragToolTip);

		        proxy._on($(document), "mousemove", proxy._mouseout);

		        if (proxy._draggedRecord) {
		            proxy._updateDragStateFalse(proxy._draggedRecord);
		            proxy._dragMouseDown = true;
		        }
		        if (args.cancel) {
		            proxy._dragMouseDown = false;
		            proxy._updateDragStateTrue(proxy._draggedRecord);
		        }
		    }
		},

        //Mouse Move action to rendering the tooltip for drag and drop
        dragToolTip: function (e) {
            var proxy = this,
                args = {},
                draggedrows = [],
                $target = $(e.target),
                $tr = $target.closest('tr'),
                length,
                recordIndex, item,
                $trClone;
            proxy._popupHide($target);

            if (proxy._dragMouseDown == true && proxy.model.allowDragAndDrop == true && e.pageX != proxy._posx && e.pageY != proxy._posy) {
                /*remove the details row while drag and drop */
                this._removeDetailsRow();
                if (!args.cancel) {
                    length=proxy._rowDragIndexes.length;
                    for (i = 0; i < length; i++)
                        ej.TreeGrid.getRowByIndex(proxy, this._rowDragIndexes[i]).removeAttr("aria-selected").find(".e-selectionbackground")
                            .removeClass("e-selectionbackground").removeClass("e-active");
                    if (proxy.model.allowSelection)
                        proxy._dragRowSelection(e);
                    recordIndex = proxy.getRows().index($target.closest('tr'));
                    if (recordIndex != -1) {
                        item = proxy.model.currentViewData[recordIndex];
                        $tr = ej.TreeGrid.getRowByIndex(proxy, proxy._currentIndex);
                        $trClone = $tr.clone();
                        //Remove the hided columns.
                        $trClone.find(".e-hide").remove();                      
                        args.targetRowIndex = item.index;
                        args.targetRow = item;
                        args.canDrop = true;
                        args.draggedRow = proxy._draggedRecord;
                        args.draggedRowIndex = proxy._draggedRecord.index;
                        proxy._trigger("rowDrag", args);
                        proxy._renderToolTip(e, $trClone, args);
                        if (args.targetRow.isSummaryRow) {
                            proxy._draggedRecord.canDrop = false;
                            $(".intend").find(".e-icon").css({ "display": "none" });
                            $(".intendparent").find(".e-icon").css({ "display": "none" });
                        }
                        else if (args.canDrop == false) {
                            proxy._draggedRecord.canDrop = false;
                        }
                        else {
                            proxy._draggedRecord.canDrop = true;
                        }
                    } else if (proxy._draggedRecord && proxy._dragMouseDown)
                    {
                        item = null;
                        $tr = ej.TreeGrid.getRowByIndex(proxy, proxy._currentIndex);
                        $trClone = $tr.clone();
                        args.targetRowIndex = null;
                        args.targetRow = null;
                        args.canDrop = true;
                        args.draggedRow = proxy._draggedRecord;
                        args.draggedRowIndex = proxy._draggedRecord.index;
                        proxy._trigger("rowDrag", args);
                        proxy._renderToolTip(e, $trClone, args);
                    }
                }
            }
        },
         
        //Mouse up event for drag and drop
        dragMouseUp: function (e) {            
            var $target = $(e.target),
                count = 0,
                args = {},
                arg = {},      
                proxy = this,
                model=proxy.model,
                draggedRecord = proxy._draggedRecord,
                currentRecord = proxy._currentRecord,
                $tr = $target.closest('tr'),    
                canDrop,
                length = proxy._selectedRowsIndexes.length,

                dropPosition;
            _timerDragDown && (_timerDragDown = window.clearInterval(_timerDragDown));
            _timerDragUp && (_timerDragUp = window.clearInterval(_timerDragUp));

            // unbind the drag events once row dropped
            proxy._off(proxy.element, "mousemove", ".e-treegridrows", proxy.dragToolTip);
            proxy._off(proxy.element, "mousemove", ".e-templatecell  ", proxy.dragToolTip);

            proxy._off($(document), "mousemove", proxy._mouseout);

            proxy._dragMouseDown = false;
            if (ej.isNullOrUndefined(draggedRecord))
               canDrop = true;
            else
               canDrop = draggedRecord.canDrop;
            $(proxy._dragmouseOverTooltip).remove();
            proxy._dragTooltip = false;
            $(".intend").find(".e-icon").css({ "display": "none" });
            $(".intendparent").find(".e-icon").css({ "display": "none" });
            if (this.cancelDrop == true &&  !ej.isNullOrUndefined(draggedRecord) && draggedRecord.parentItem == null) {
                if (draggedRecord.expanded)
                    $tr.find(".e-treegridexpand").css({ "display": "inline-block" });
                else
                    $tr.find(".e-treegridcollapse").css({ "display": "inline-block" });
            }
            proxy._dragMouseLeave = true;           
            if ($target.closest("div#" + proxy._id).length == 1 && !ej.isNullOrUndefined(draggedRecord) && proxy._dropCancel == false) {               
                proxy._dragMouseDown = false;
                if (model.allowDragAndDrop && !model.isFromGantt && canDrop ) {
                    proxy._dragMouseLeave = false;
                    proxy._dragMouseDown = false;
                    var recordIndex = proxy.getRows().index($tr);
                    if (recordIndex != proxy._currentIndex && recordIndex != -1) {
                        proxy._droppedRecord = model.currentViewData[recordIndex];
                        droppedRecord = proxy._droppedRecord;
                        args.previousItem = draggedRecord;
                        var flatRecords = model.flatRecords;
                        args.previousItemIndex = flatRecords.indexOf(args.previousItem);
                        args.previousParentItem = draggedRecord.parentItem;
                        if (!ej.isNullOrUndefined(draggedRecord.parentItem)) {
                            args.parentChildState = draggedRecord.parentItem.hasChildRecords;
                            args.parentExpandedState = draggedRecord.parentItem.expanded;
                        }
                        args.hasChildRecords = droppedRecord.hasChildRecords;
                        args.expanded = droppedRecord.expanded;
                        args.childIndex = args.previousParentItem ? args.previousParentItem.childRecords.indexOf(draggedRecord) : 0;
                        args.previousLevel = draggedRecord.level;
                        var draggedrows = [];

                        if (droppedRecord.dragState == true) {
                            proxy._updateDragStateTrue(draggedRecord);

                            if (proxy._insertAbove == true || proxy._insertBelow == true || proxy._insertAsChild == true) {
                                proxy._cancelSaveTools();
                                proxy._deleteDragRow();                                
                                var recordIndex1 = flatRecords.indexOf(droppedRecord);

                                for (i = 0; i < length; i++)
                                    ej.TreeGrid.getRowByIndex(proxy, this._selectedRowsIndexes[i]).removeAttr("aria-selected").find(".e-selectionbackground").removeClass("e-selectionbackground").removeClass("e-active");

                                //Condition to splice above to the dropped record.
                                if (proxy._insertAbove == true) {
                                    dropPosition = "insertAbove";
                                    args.index = recordIndex1;
                                    draggedRecord.parentItem = flatRecords[recordIndex1].parentItem;
                                    draggedRecord.level = flatRecords[recordIndex1].level;
                                    flatRecords.splice(recordIndex1, 0, draggedRecord);
                                    model.updatedRecords.splice(recordIndex1, 0, draggedRecord);
                                    if (draggedRecord.hasChildRecords) {
                                        var level = 1;
                                        proxy._updateChildRecord(draggedRecord, recordIndex1);
                                        proxy._updateChildRecordLevel(draggedRecord, level);
                                    }
                                    if (droppedRecord.parentItem)
                                        droppedRecord.parentItem.childRecords.splice(droppedRecord.parentItem.childRecords.indexOf(droppedRecord), 0, draggedRecord);
                                }

                                //condition to splice below to the dropped record.
                                else if (proxy._insertBelow == true) {
                                    dropPosition = "insertBelow";
                                    args.index = recordIndex1 + 1;
                                    if (!droppedRecord.hasChildRecords) {
                                        flatRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                        model.updatedRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                    }
                                    else {
                                        count = proxy.getChildCount(droppedRecord, 0);
                                        flatRecords.splice(recordIndex1 + count + 1, 0, draggedRecord);
                                        model.updatedRecords.splice(recordIndex1 + count + 1, 0, draggedRecord);
                                    }
                                    draggedRecord.parentItem = flatRecords[recordIndex1].parentItem;
                                    draggedRecord.level = flatRecords[recordIndex1].level;
                                    if (draggedRecord.hasChildRecords) {
                                        var level = 1;
                                        proxy._updateChildRecordLevel(draggedRecord, level);
                                        proxy._updateChildRecord(draggedRecord, recordIndex1 + count + 1);
                                    }
                                    if (droppedRecord.parentItem)
                                        droppedRecord.parentItem.childRecords.splice(droppedRecord.parentItem.childRecords.indexOf(droppedRecord) + 1, 0, draggedRecord);
                                }

                                // Condition to insert as child to the dropped Record.
                                else if (proxy._insertAsChild == true) {
                                    dropPosition = "insertAsChild";
                                    args.index = recordIndex1 + 1;
                                    if (!draggedRecord.hasChildRecords) {
                                        flatRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                        model.updatedRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                        proxy._recordLevel(recordIndex);
                                    }
                                    else {
                                        flatRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                        model.updatedRecords.splice(recordIndex1 + 1, 0, draggedRecord);
                                        proxy._recordLevel(recordIndex);
                                        proxy._updateChildRecord(draggedRecord, recordIndex1 + 1, droppedRecord.expanded);
                                    }                                    
                                }
                            }                                                    
                            if (draggedRecord.parentItem == null)
                                model.parentRecords[model.parentRecords.length] = draggedRecord;
                            args.requestType = "dragAndDrop";
                            args.draggedRow = draggedRecord;
                            args.targetRow = droppedRecord;
                            proxy._updateDataSource();
                            proxy.processBindings(args);
                            arg.targetRowIndex = recordIndex;
                            arg.targetRow = droppedRecord;
                            arg.draggedRowIndex = draggedRecord.index;
                            arg.draggedRow = draggedRecord;
                            arg.requestType = dropPosition;
                            proxy._trigger("rowDragStop", arg);                                                     
                            if (model.enableAltRow && model.currentViewData.length > 0)
                                ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);                           
                            if (arg.cancel)
                                proxy._revertDragging(args);
                            if (model.allowSelection)
                                proxy._selectDraggedRow($tr);
                            if (model.showSummaryRow && proxy._insertAsChild == true)
                                proxy._createSummaryRow(args);
                        }
                        else
                            //If drag state false then drop will cancel so have change the drag state.
                            proxy._updateDragStateTrue(draggedRecord);                        
                    }
                    else
                        // If drop record not found means condition failed and drag state should change
                        proxy._updateDragStateTrue(draggedRecord);
                }
                else {
                    // if canDrop true then drop will cancelled and state should change
                    proxy._updateDragStateTrue(draggedRecord);
                    return 0;
                }
            }
            else
                // if esc key pressed or user drops a record other the tree grid content then 
                //drop cancelled and have to change the drag state.
                proxy._updateDragStateTrue(draggedRecord);            
            // to enable expand and collapse icon of top most record if user mouse up outside of the grid area. 
            if (!ej.isNullOrUndefined(proxy._currentRecord)) {
                if (proxy._currentRecord.parentItem == null) {
                    $tr = ej.TreeGrid.getRowByIndex(proxy, model.updatedRecords.indexOf(proxy._currentRecord));
                    if (proxy._currentRecord.expanded)
                        $tr.find(".e-treegridexpand").css({ "display": "inline-block" });
                    else
                        $tr.find(".e-treegridcollapse").css({ "display": "inline-block" });
                }
            }            
            //Update the summary row after perform the drag and drop action.
            if (model.showSummaryRow && args.requestType == "dragAndDrop" && proxy._insertAsChild != true)
                proxy._updateSummaryRow(args);
            proxy._dropCancel = false;
            proxy._resetPrivateProperties();
        },

        _resetPrivateProperties: function () {
            _timerDragDown && (_timerDragDown = window.clearInterval(_timerDragDown));
            _timerDragUp && (_timerDragUp = window.clearInterval(_timerDragUp));
            var proxy = this;
            proxy.cancelDrop = false;
            proxy._draggedRecord = null;
            proxy._dragMouseDown = false;
            _timerDragDown = null;
            _timerDragUp = null;
            proxy._childItem = false;
            proxy._dragMouseLeave = false;
            proxy._currentRecord = null;
            proxy._currentIndex = null;
            proxy._droppedRecord = null;
            proxy._insertAsChild = false;
            proxy._insertAbove = false;
            proxy._insertBelow = false;
        },
        // To select the dragged Record after drag stopped.
        _selectDraggedRow: function ($tr) {
            var proxy = this, model = proxy.model;
            this.selectedRowIndex(model.updatedRecords.indexOf(proxy._draggedRecord));
            proxy.selectRows(this.selectedRowIndex());
            proxy.updateScrollBar()
            args = {
                rowElement: $tr,
                data: model.selectedItem,
                target: "ejTreeGrid",
                recordIndex: proxy.selectedRowIndex(),
            };
            proxy._cancelSaveTools();
            proxy._trigger("rowSelected", args);
        },
        //Add child records to the data source
        _addChildItem: function (record) {
            var currentRecord, proxy = this, length=0;
            length = record.childRecords.length;
            for (var i = 0; i < length; i++) {
                currentRecord = record.childRecords[i];
                if (proxy.dataSource() instanceof ej.DataManager) {
                    if (proxy.dataSource().dataSource.offline && proxy.dataSource().dataSource.json) {
                        proxy.dataSource().dataSource.json.push(currentRecord.item);
                        if (currentRecord.hasChildRecords) {
                            proxy._addChildItem(currentRecord);
                        }
                    }
                } else {
                    proxy.dataSource().push(currentRecord.item);                     
                    if (currentRecord.hasChildRecords) {
                        proxy._addChildItem(currentRecord);
                    }
                }
            }
        },
        //Update the changes in DataSource.
        _updateDataSource: function () {
            var proxy = this,
                model = proxy.model,
                draggedRecord = proxy._draggedRecord,
                droppedRecord = proxy._droppedRecord,
                tempDataSource, idx, childIndex;
            if (proxy.dataSource() instanceof ej.DataManager) {
                if (proxy.dataSource().dataSource.offline && proxy.dataSource().dataSource.json) {
                    tempDataSource = proxy.dataSource().dataSource.json;
                }
            }
            else 
                tempDataSource = proxy.dataSource();
            
            if (tempDataSource && !droppedRecord.parentItem) {
                idx = tempDataSource.indexOf(droppedRecord.item);
                if (proxy._insertAbove) {
                    if (!model.parentIdMapping)
                        tempDataSource.splice(idx, 0, draggedRecord.item);
                }
                else if (proxy._insertBelow) {
                    if (!model.parentIdMapping)
                        tempDataSource.splice(idx + 1, 0, draggedRecord.item);
                }
            }
            else if (!model.parentIdMapping && droppedRecord.parentItem) {
                childIndex = droppedRecord.parentItem.childRecords.indexOf(draggedRecord);
                if (proxy._insertAbove) 
                    droppedRecord.parentItem.item[proxy.model.childMapping].splice(childIndex, 0, draggedRecord.item);
                else if (proxy._insertBelow)
                    droppedRecord.parentItem.item[proxy.model.childMapping].splice(childIndex, 0, draggedRecord.item);
            }

            if (model.parentIdMapping) {
                if (draggedRecord.parentItem) {
                    if (proxy._insertAbove || proxy._insertBelow) {
                        draggedRecord[model.parentIdMapping] = droppedRecord[proxy.model.parentIdMapping];
                        draggedRecord.item[model.parentIdMapping] = droppedRecord[proxy.model.parentIdMapping];
                    }
                    else {
                        draggedRecord[model.parentIdMapping] = droppedRecord[proxy.model.idMapping];
                        draggedRecord.item[model.parentIdMapping] = droppedRecord[proxy.model.idMapping];
                    }
                }
                else {
                    draggedRecord.item[model.parentIdMapping] = null;
                    draggedRecord[model.parentIdMapping] = null;
                }

                if (proxy.dataSource() instanceof ej.DataManager) {
                    if (proxy.dataSource().dataSource.offline && proxy.dataSource().dataSource.json) {
                        proxy.dataSource().dataSource.json.push(draggedRecord.item);
                    }
                } else {

                    if (proxy._insertAbove || proxy._insertBelow || proxy._insertAsChild)
                        proxy.dataSource().push(draggedRecord.item);
                    if (draggedRecord.hasChildRecords) {
                        proxy._addChildItem(draggedRecord);
                    }
                }
                proxy.updateScrollBar();
            }
        },

        //This method is to delete the dragged record 
        _deleteDragRow: function ($tr) {
            var proxy = this,
                model=proxy.model,
                 deletedRowIndex,
                 deletedRow,
                 args = {};
            deletedRowIndex = proxy.model.updatedRecords.indexOf(proxy._draggedRecord);
            deletedRow = proxy.model.updatedRecords[deletedRowIndex];
            args.tr = $tr;
            args.data = deletedRow;
            args.requestType = ej.TreeGrid.Actions.Delete;
            args.isDragAndDropDelete = true;
            if (proxy._trigger("actionBegin", args)) {
                return true;
            }
            proxy.processBindings(args);           
            proxy._selectedRowsIndexes = [];
            proxy._previousIndex = -1;
        },
        // To revert the changes if drop get cancelled.
        _revertDragging: function (args) {
            var proxy = this,
                model=proxy.model,
                flatRecords = model.flatRecords;
            proxy._draggedRecord = args.draggedRow;
            proxy._deleteDragRow();
            if (!proxy._draggedRecord.hasChildRecords) {
                flatRecords.splice(args.previousItemIndex, 0, proxy._draggedRecord);
                if (!ej.isNullOrUndefined(flatRecords[args.previousItemIndex].parentItem)) {
                    flatRecords[args.previousItemIndex].parentItem = args.previousParentItem;
                    if (!ej.isNullOrUndefined(args.previousParentItem))
                        flatRecords[args.previousItemIndex].parentItem.childRecords.splice(args.childIndex, 0, proxy._draggedRecord);
                }
                flatRecords[args.previousItemIndex].level = args.previousLevel;
            }
            else {
                flatRecords.splice(args.previousItemIndex, 0, proxy._draggedRecord);
                if (!ej.isNullOrUndefined(flatRecords[args.previousItemIndex].parentItem)) {
                    flatRecords[args.previousItemIndex].parentItem = args.previousParentItem;
                    if (!ej.isNullOrUndefined(args.previousParentItem))
                        flatRecords[args.previousItemIndex].parentItem.childRecords.splice(args.childIndex, 0, proxy._draggedRecord);
                }
                proxy._updateChildRecord(proxy._draggedRecord, args.previousItem.index);
                proxy._draggedRecord.level = args.previousLevel;
                proxy._updateChildRecordLevel(proxy._draggedRecord, args.previousLevel);
            }
            if (!ej.isNullOrUndefined(args.previousItem.parentItem)) {
                args.previousItem.parentItem.hasChildRecords = args.parentChildState;
                args.previousItem.parentItem.expanded = args.parentExpandedState;
                args.previousItem.parentItem.item[proxy.model.childMapping].splice(args.childIndex, 0, args.previousItem.item);
            }
            args.targetRow.hasChildRecords = args.hasChildRecords;
            args.targetRow.expanded = args.expanded;            
            proxy.processBindings(args);
            if (model.currentViewData.length > 0)
                ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);
        },

        //To splice the child records if user drag and dropped the parent item.
        _updateChildRecord: function (record, count,expandState){
            var currentRecord,
                proxy = this,
                model = proxy.model,
                length;
            if (!record.hasChildRecords)
                return 0;
            length = record.childRecords.length;
            for (var i = 0; i < length; i++) {
                currentRecord = record.childRecords[i];
                count++;
                model.flatRecords.splice(count, 0, currentRecord);               
                if (currentRecord.hasChildRecords) {
                    count = proxy._updateChildRecord(currentRecord, count);
                }
            }
            return count;
        },
        //This method is to leveling the records when it is inserted as child.
        _recordLevel: function (recordIndex) {
            var j = 0, proxy = this,
                model = proxy.model,
                draggedRecord=proxy._draggedRecord,
                droppedRecord = proxy._droppedRecord,
                childItem = proxy.model.childMapping;
            if (!droppedRecord.hasChildRecords) {
                droppedRecord.hasChildRecords = true;
                if (ej.util.isNullOrUndefined(droppedRecord.childRecords) || $.isEmptyObject(droppedRecord.childRecords)) {
                    droppedRecord.childRecords = [];
                    if (!model.parentIdMapping)
                        droppedRecord.item[childItem] = [];
                }
            }
            if (proxy._insertAsChild) {
                draggedRecord.parentItem = droppedRecord;
                droppedRecord.childRecords.splice(0, 0, draggedRecord);
                if (!ej.isNullOrUndefined(draggedRecord) && !ej.isNullOrUndefined(droppedRecord.item[childItem]) && !model.parentIdMapping) {
                    droppedRecord.item[childItem].splice(0, 0, draggedRecord.item);
                }
                if (!draggedRecord.hasChildRecords)
                    draggedRecord.level = droppedRecord.level + 1;
                else {
                    var level = 1;
                    draggedRecord.level = droppedRecord.level + 1;
                    proxy._updateChildRecordLevel(draggedRecord, level);
                }
                droppedRecord.expanded = true;
                proxy._parentRecords.push(droppedRecord);
            }
        },
        // This method is to level the parent record at the end of it hierarchy.
        _updateChildRecordLevel: function (record,level) {
            var proxy = this,
                length=0;
            level++;
            if (!record.hasChildRecords)
                return 0;
            length = record.childRecords.length;
            for (var i = 0; i < length ; i++) {
                currentRecord = record.childRecords[i];
                currentRecord.level = record.parentItem ? record.parentItem.level + level : record.level + 1;
                if (currentRecord.hasChildRecords) {
                    level--;
                    level = proxy._updateChildRecordLevel(currentRecord, level);
                }
            }
            return level;
        },
                   
        //To renter cancel icon in filtering elements.
        _filterBarClose: function (e) {
            var $target = $(e.target);
            if (e.type == "click" && $target.hasClass("e-cancel")) {
                var $targetText = $target.prev();
                    $targetText.focus().val("");
                $targetText.prop('checked', false);
                $targetText.trigger("keyup");
                e.stopPropagation();
            }
            if (e.type == "focusin") { 
                $target = $(e.target).next();
                this.getHeaderTable().find(".e-cancel").addClass("e-hide");
                $target.removeClass("e-hide");
            }
        },

        checkIdenticalValue: function(val){
            var proxy = this,
                model = proxy.model,
                filteredColumns = model.filterSettings.filteredColumns,
                columnLength = filteredColumns.length;
            for (var col = 0; col < columnLength; col++) {
                if (filteredColumns[col].value == val) {
                    var flag = true;
                    break;
                }               
            }
            return flag;            
        },
        //To wireup method while user enter text to the filter textbox.
        _filterBarHandler: function (e) {
            var proxy = this,
                model = proxy.model;
            //cancel the edited cell and row
            if (proxy.model.isEdit)
                proxy.cancelEditCell();
            else if (proxy._isRowEdit)
                proxy.cancelRowEditCell();
            /* flag new added record is sorted or filttered in treegrid */
            proxy._isRefreshAddedRecord = false;

            /* clear selction*/
            proxy.selectRows(-1);
            proxy._cancelSaveTools();

            if (e.target == undefined)
                var $target =$(e);
            else
                var $target = $(e.target);
            
            if (this.model.allowFiltering ) {
                var cols=proxy.model.columns;
                var matchcase;
                matchcase = proxy._cellEditingDetails;
                var key = proxy._queryManagar.queries[0];
                var splits = $target[0].id.split('_'), filteringColumn;
                proxy._fieldName = splits[1];
                this._currentFilterColumn = ej.TreeGrid.getColumnByField(model.columns, splits[1]);
                if ($target.hasClass("e-checkbox"))
                    proxy._searchString = $target[0].checked;
                else if ($target.hasClass("e-ddl")) {
                    // To get the value from dropdown filter (onEnter) mode
                    $target = $target.find("input.e-dropdownlist");
                    var dropDownElement = $("#" + $target[0].id).data("ejDropDownList");
                    if (dropDownElement.selectedIndexValue == -1 || (proxy.checkIdenticalValue(dropDownElement._selectedValue, this._currentFilterColumn)))
                        return true;
                    proxy._searchString = dropDownElement._selectedValue;
                }
                else if ($target.hasClass("e-dropdownlist")) {
                    // To get the value from dropdown filter (immediate) mode
                    var dropdownobject = $("#" + $target[0].id).data("ejDropDownList");
                    if (dropdownobject.selectedIndexValue == -1 || (proxy.checkIdenticalValue(dropdownobject._selectedValue, this._currentFilterColumn)))
                        return true;
                    proxy._searchString = dropdownobject._selectedValue;
                }
                else
                    proxy._searchString = $target.closest("input").val();
                if (proxy._searchString == "") {
                    filteringColumn = model.filterSettings.filteredColumns.filter(function (column) {
                        if (column.field === proxy._fieldName)
                            return true;
                    });
                    if (!filteringColumn.length) {
                        proxy._resizeFilteringElements();
                        return true;
                    }
                }
               
                proxy._validateFilterValue(proxy._searchString);
                proxy.filterColumn(proxy._fieldName, this._operator, proxy._searchString, this._predicate);                
                if (proxy.model.currentViewData.length > 0 && proxy.model.enableAltRow)
                    ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);                
            }
        },
        //Method to validate search string type and assign operators accordingly.
        _validateFilterValue: function (_value) {
            var proxy = this;
            this._predicate = "and";
            switch (this._currentFilterColumn.filterEditType) {
                case "numericedit":
                    this._operator = ej.FilterOperators.equal;
                    break;
                case "datepicker":
                    this._operator = ej.FilterOperators.equal;
                    
                    var _format;
                    if (this._currentFilterColumn.format == "" || this._currentFilterColumn.format == undefined)
                        _format = Globalize.culture().calendar.patterns.d; //System Date format
                    else
                        _format = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                    if (this._currentFilterbarValue != "")
                        this._currentFilterbarValue = Globalize.format(_value, _format);
                    break;
                case "datetimepicker":
                    this._operator = ej.FilterOperators.equal;
                  
                    var _format;
                    if (this._currentFilterColumn.format == "")
                        _format = Globalize.culture().calendar.patterns.f; //System DateTime format
                    else
                        _format = this._currentFilterColumn.format.split(':')[1].replace('}', "");
                    if (this._searchstring != "")
                        this._searchstring = Globalize.format(_value, _format);
                    break;
                case "stringedit":
                    this._operator = ej.FilterOperators.startsWith;
                    break;
                case "dropdownedit":
                    var filterColumn = this._currentFilterColumn;
                    if (filterColumn.editType == "dropdownedit" && filterColumn.editParams && filterColumn.editParams.showCheckbox)
                        this._operator = ej.FilterOperators.contains;
                    else
                        this._operator = ej.FilterOperators.equal;
                    break;
                case "booleanedit":
                    if (proxy._searchString == true || proxy._searchString == "1")
                        proxy._searchString = true;
                    else if (proxy._searchString == "false" || proxy._searchString == "0")
                        proxy._searchString = false;
                    this._operator = ej.FilterOperators.equal;
                    break;
                default:
                    this._operator = ej.FilterOperators.equal;
            }
        },
        
        _mouseHover: function (e) {
            e.preventDefault();
            var proxy = this,
                model = proxy.model;
            if (model.showGridCellTooltip) {
                if (this._dragMouseDown != true) {
                    var $target = $(e.target), proxy = this,
                        posx = 0,
                        posy = 0,
                        $ganttGridRows = proxy.getRows(),
                        row = $target.closest('tr'),
                        column = $target.closest('td.e-rowcell'),
                        recordIndex = $ganttGridRows.index(row),
                        item = proxy.model.currentViewData[recordIndex],
                        tooltiptable, tooltipbody, textContent, tooltipWidth,
                        tooltipHeight, containerWidth, containerHeight,
                        columns = proxy.model.columns;
                    
                    if (recordIndex >= 0) {
                        columnIndex = $($ganttGridRows[recordIndex].childNodes).index(column);
                        proxy._cellMouseLeave();

                        if (!e) e = window.event;
                        if (e.originalEvent.pageX || e.originalEvent.pageY) {
                            posx = e.originalEvent.pageX;
                            posy = e.originalEvent.pageY;
                        }
                        else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                            posx = e.originalEvent.clientX + document.body.scrollLeft
                                + document.documentElement.scrollLeft;
                            posy = e.originalEvent.clientY + document.body.scrollTop
                                + document.documentElement.scrollTop;
                        }

                        if (item) {
                            if (columnIndex != proxy.model.treeColumnIndex)
                                textContent = column.children().length ? column.children().clone() : column.clone(); // Copying the DOM element of particular cell.
                            else
                                textContent = column.find(".e-cell").clone();
                            tooltiptable = ej.buildTag("table.e-tooltiptable", "", { 'padding': '0' }, { 'cellspacing': '1' });

                            if (!model.showGridExpandCellTooltip || (model.showGridExpandCellTooltip && columnIndex == proxy.model.treeColumnIndex)) {
                                tooltipbody = ej.buildTag("tbody", textContent, {}, {});
                            }
                            tooltiptable.append(tooltipbody);

                            var isNonEmptyText = $(textContent).text().replace(/[ \t\r]+/g, "").length > 0 ? true : false;

                            if (tooltipbody && $(column).find("#" + proxy._id + "EditForm").length == 0) {
                                if (!(proxy.model.showGridExpandCellTooltip) && isNonEmptyText
                                    || (proxy.model.showGridExpandCellTooltip)) {
                                    var z_index = $("#" + this._id + "detailscellwrapper").length > 0 ? 110 : 5;
                                    proxy._mouseOverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                                        {
                                            "top": (posy + 10) + "px", "left": (posx + 15) + "px",
                                            'position': 'absolute',
                                            'z-index': z_index,
                                            'padding': '0',
                                            'border-radius': '3px'
                                        }, {});

                                    //Providing a time delay to render the tooltip to improve UI
                                    proxy._tooltipTimer = setTimeout(function () {
                                        $(document.body).append(proxy._mouseOverTooltip);
                                        if (!model.isFromGantt) {
                                            var position = proxy.getOffsetRect(proxy.element[0]);
                                            containerWidth = $(proxy.element).width() + position.left;
                                            containerHeight = $(proxy.element).height() + position.top;
                                        }
                                        else
                                        {
                                            var element = $("#" + proxy._id.replace("ejTreeGrid", "")),
                                                position = proxy.getOffsetRect(element[0]);
                                            containerWidth = $(element).width() + position.left;
                                            containerHeight = $(element).height() + position.top;
                                        }
                                        tooltipWidth = $(proxy._mouseOverTooltip).width();
                                        tooltipHeight = $(proxy._mouseOverTooltip).height();
                                        // Calculation for reposition the tooltip when tooltip is exist the container height or width
                                        if ((posx + tooltipWidth + 15) >= containerWidth) {
                                            posx = posx - tooltipWidth - 15;
                                        }

                                        if ((posy + tooltipHeight + 10) >= containerHeight) {
                                            posy = posy - tooltipHeight -10;
                                        }
                                        $(proxy._mouseOverTooltip).css({ "top": posy, "left": posx })
                                    }, 700);
                                }
                            }
                        }
                    }
                }
            }
        },

        _cellMouseLeave: function () {

            var proxy = this;
            if (proxy._mouseOverTooltip != null) {
                $("#tooltipgantt").remove();
                window.clearTimeout(proxy._tooltipTimer);
                $(proxy._mouseOverTooltip).remove();
            }
        },

        disableTooltip: function () {
            this._cellMouseLeave();
        },
       
        _createTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                helpers = {
                    _getTaskName: proxy._tooltipTaskName
                };

            $.views.helpers(helpers);
            var parentTr = "<tr class='e-tooltip_rowcell'>";

            td = "{{if ~_getTaskName()}}" +
                "<td class='e-tooltiptaskname' style='height:auto;width:auto;font-weight:normal;'>{{:ttiptaskname}}</td>{{/if}}";

            parentTr += td;
            parentTr += "</tr>";

            var templates = {};
            templates[proxy._id + "tooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _tooltipTaskName: function () {
            return this.data.ttiptaskname;
        },

        //SCROLL EVENT FOR VIRTUALIZATION
        _onScroll: function (args) {

            var proxy = this,
                hScrollDist;
            proxy._popupHide();
            proxy._clearContextMenu();

            $("#" + this._id + "detailscellwrapper").length > 0 && this._removeDetailsRow();

            if (proxy.model.showGridCellTooltip) {
                proxy._cellMouseLeave();
            }

            if (!proxy.isVScroll() && !proxy.model.isFromGantt && args.source === "wheel") {
                args.cancel = true;
                return;
            }

            if (args.scrollLeft != undefined) {
                proxy._scrollLeft = args.scrollLeft;
                hScrollDist = Math.abs(proxy._scrollLeft - proxy._prevScrollLeft);
            }

            

            if (hScrollDist) {

                //CHANGE SCROLL LEFT VALUE OF HEADER  BASED ON THE SCROLLING OF CONTENT DIV
                proxy._$gridHeaderContainer.scrollLeft(proxy._scrollLeft);
                if (proxy.model.showTotalSummary)
                    proxy._$footerContainer.scrollLeft(proxy._scrollLeft);
                proxy._prevScrollLeft = proxy._scrollLeft;
            }
            
            if (args.scrollTop != undefined) {

                /*Cancel if treegrid is in edit*/
                if (proxy.model.isEdit) {
                    proxy.cancelEditCell();
                    $(proxy.element).focus();
                }
                else if (proxy._isRowEdit) {
                    proxy.cancelRowEditCell();
                    $(proxy.element).focus();
                }

                proxy._scrollTop = args.scrollTop; //Maintain the scroll top for dynamic update of virtualization and non-virtualization mode.
                proxy._vScrollDist = Math.abs(proxy._scrollTop - proxy._prevScrollTop);
                //CHECK VIRTUALIZATION IS ENABLED OR NOT
                if (proxy.model.enableVirtualization) {

                    proxy._vScrollDir = proxy._prevScrollTop <= proxy._scrollTop ? 1 : -1;
                    //console.log("VScrollDist:" + proxy._vScrollDist);
               
                    if (proxy._vScrollDist) {
                        proxy._updateCurrentViewData();
                        proxy._prevScrollTop = proxy._scrollTop;
                    }
                }
                proxy._scrollTop = args.scrollTop;
            }
        },

        //Save the edited row

        _saveRow:function()
        {
            var proxy = this,
                model = proxy.model,
                form = $(document.getElementById(proxy._id + "EditForm")),
                formElement = $(form).find("td"),
                index = this.getIndexByRow(form.closest("tr")),
                isAddRow = form.closest("tr").hasClass("e-addedrow"),
                columns = model.columns,
                columnLength = columns.length,
                currentItem =model.currentViewData[index],
                currentValue = {},
                previousValue = {},
                args = {};

            proxy.element[0].focus();

            if (form.length <= 0 && index == -1)
                return;

            for (i = 0; i < columnLength; i++) {
                column = columns[i];
                if (!ej.isNullOrUndefined(column)) {
                    if (column.visible) {
                        value = proxy.getCurrentEditCellDataForRowEdit(column["field"], column["editType"], i);
                        previousValue[column["field"]] = currentItem.item[column["field"]];
                        currentValue[column["field"]] = currentItem[column["field"]] = currentItem.item[column["field"]] = value;
                    }
                    else
                        previousValue[column["field"]] = currentValue[column["field"]] = currentItem[column["field"]];
                }
            }

            if (!isAddRow) {
                args.rowIndex = proxy.selectedRowIndex();
                args.previousValue = previousValue;
                args.requestType = "update";
                args.currentValue = currentItem.item;
                args.rowElement = form.closest("tr");
                if (proxy._trigger("endEdit", args)) {
                    for (i = 0; i < columnLength; i++) {
                        column = columns[i];
                        if (!ej.isNullOrUndefined(column))
                            if (column.visible)
                                currentItem[column["field"]] = currentItem.item[column["field"]] = previousValue[column["field"]];
                    }
                }
            }
            else {
                args.rowIndex = proxy.selectedRowIndex();
                args.previousValue = previousValue;
                args.requestType = "addNewRow";
                args.addedRow = proxy._checkIsEmptyRow(currentItem.item) ? null : currentItem.item;
                args.rowElement = form.closest("tr");
                if (proxy._trigger("actionComplete", args)) {
                    for (i = 0; i < columnLength; i++) {
                        column = columns[i];
                        if (!ej.isNullOrUndefined(column))
                            if (column.visible)
                                currentItem[column["field"]] = currentItem.item[column["field"]] = previousValue[column["field"]];
                    }
                }
            }

            /* delete the row if new added row is not have any values */
            if (proxy._checkIsEmptyRow(currentItem.item) && isAddRow) {
                var deleteValueUpdated = false;
                if (!model.editSettings.allowDeleting) {
                    deleteValueUpdated = true;
                    model.editSettings.allowDeleting = true;
                }
                proxy.deleteRow();
                if (deleteValueUpdated) {
                    model.editSettings.allowDeleting = false;
                }
            } else {
                // Save the edited row           
                ej.TreeGrid.refreshRow(proxy, index);
                if (isAddRow && currentItem.parentItem && currentItem.parentItem.childRecords.length == 1) {
                    ej.TreeGrid.refreshRow(proxy, model.currentViewData.indexOf(currentItem.parentItem));
                }
            }
            proxy._isRowEdit = false;
            if (model.currentViewData.length > 0 && model.enableAltRow)
                ej.TreeGrid.updateAltRow(proxy, model.currentViewData[0], 0, 0);
            if (model.showSummaryRow) {
                args.editType = "rowedit";
                if (args.requestType == "addNewRow" && model.editSettings.rowPosition == "child")
                    proxy._createSummaryRow(args);
                proxy._updateSummaryRow(args);
            }
            if (model.showTotalSummary) {
                args.editType = "rowedit";
                proxy._updateTotalSummaryRow(args);
            }
            proxy._cancelSaveTools();
            var colObject = columns.filter(function (column) {
                if (column.filterEditType == "dropdownedit" && column.editType != "dropdownedit")
                    return true;
            });
            if (model.allowFiltering && colObject.length)
                proxy._resizeFilteringElements();

        },
        _excludeDetailRows : function()
        {
            var $gridRows, model = this.model;
            if (model.showDetailsRow && model.detailsTemplate)
                $gridRows = $(this.getTreeGridRows()).not(".e-detailsrow");
            else
                $gridRows = $(this.getTreeGridRows());

            return $gridRows;
        },

        getIndexByRow: function ($tr) {
            var $gridRows = this._excludeDetailRows();
            return $gridRows.index($tr);
        },
        /*get all treegrid rows with details rows*/
        getTreeGridRows: function () {
            return this._$gridRows;
        },
        //CLICK HANDLER FOR PERFORM ROWSELECTION AND EXPANDCOLLAPSE THE RECORD
        _onClick: function (e) {

            var proxy = this,
                model = proxy.model,
                $target = $(e.target), currentRowIndex, args = {};
            /* check inner treegrid click events and  */
            if ($target.closest(".e-treegrid").attr("id") !== this._id) return true;

            proxy._popupHide($target);
            proxy._clearContextMenu();
            this._removeDetailsRow(e);
            proxy._cellIndex = proxy.getCellIndex(e);
            this.getHeaderTable().find(".e-cancel").addClass("e-hide");

            if ($target.hasClass('e-treegridexpand') || $target.hasClass("e-treegridcollapse")) {

                //Set the focus to the element when row or column not in edit mode.
                if ($("#" + proxy._id + "EditForm").length == 0) {
                    proxy.element[0].focus();
                }
                var $tr = $target.closest('tr'),
                    $gridRows = proxy.getRows(),
                    rowIndex = $gridRows.index($tr),
                    record = model.currentViewData && model.currentViewData[rowIndex],
                    recordIndex = proxy.model.updatedRecords.indexOf(record),
                    args = {},
                    isExpandCollapseEnabeled;

                args.data = record;
                args.recordIndex = recordIndex;
                args.expanded = $tr.hasClass("e-treegridrowexpand") ? false : true;

                if (args.expanded) {

                    isExpandCollapseEnabeled = proxy._trigger("expanding", args);

                } else {

                    isExpandCollapseEnabeled = proxy._trigger("collapsing", args);
                    
                }

                if (!isExpandCollapseEnabeled&&!model.isFromGantt) {

                    /* refresh Added row on expand collapse */
                    if (proxy._isRefreshAddedRecord) {
                        args.data.expanded = args.expanded;
                        proxy.updateExpandStatus(args.data, args.expanded);
                        proxy.refreshContent();
                        proxy.renderRecords();
                        proxy._isRefreshAddedRecord = false;
                    } else {
                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                    }
                }
                
            } else if ($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell") || ($target.closest("td").hasClass("e-rowcell")) || $target.parent().hasClass("e-detailsrowcell") || $target.hasClass("e-detailsrowcell")
                || ($target.closest("td").hasClass("e-rowcell")) && $target.hasClass("e-cell")) {

                //Set the focus to the element when row or column not in edit mode.
                if ($("#" + proxy._id + "EditForm").length == 0) {
                    proxy.element[0].focus();
                }
                //Check selction is enabled or not
                if (model.allowSelection) {

                    if (model.selectionType == ej.TreeGrid.SelectionType.Multiple) {

                        if (e.ctrlKey) proxy._multiSelectCtrlRequest = true;

                        if (e.shiftKey) {
                            currentRowIndex = proxy.getRows().index($target.closest('tr'));
                            proxy._multiSelectShiftRequest = true;
                            if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), currentRowIndex)) {
                                if (proxy.getRows() && proxy._previousIndex != -1) {
                                    proxy.selectRows(proxy._previousIndex, currentRowIndex);
                                }
                                else if (proxy.getRows() && proxy._previousIndex == -1) {
                                    proxy.selectRows(currentRowIndex, currentRowIndex);
                                }
                                proxy._cancelSaveTools();
                                proxy._rowSelectedEventTrigger(currentRowIndex);
                            }
                        }
                    }

                    if (proxy._isRowEdit) {
                        $target.focus();
                        if ($target.parents("form").length == 0)
                            proxy._saveRow();
                    }
                    if (!proxy._multiSelectShiftRequest && $("#" + proxy._id + "EditForm").length == 0) {

                        recordIndex = proxy.getRows() ? proxy.getRows().index($target.closest('tr')) : -1;

                        if (recordIndex != -1) {

                            record = model.currentViewData[recordIndex];
                            currentRowIndex = model.updatedRecords.indexOf(record);

                            if (this.selectedRowIndex() != currentRowIndex && !proxy._rowSelectingEventTrigger(this.selectedRowIndex(), currentRowIndex)) {
                                proxy.selectRows(currentRowIndex);
                                proxy._cancelSaveTools();
                                proxy._rowSelectedEventTrigger(currentRowIndex);
                            }
                        }
                    }
                    proxy._multiSelectShiftRequest = false;
                }
            }

            /* detail row show and hide */
            if (($target.hasClass("e-detailsrowcell") || $target.parent("td").hasClass("e-detailsrowcell")) && !$target.closest("tr").hasClass('e-detailsrow')) {
                if (!e.ctrlKey && !e.shiftKey) {
                    if (proxy._isRowEdit) {
                        this._saveRow();
                    } else
                    {
                        proxy._detailsExpandCollapse($target);
                    }
                }
            }

            if (proxy._isRowEdit)
                $target.focus();
            if (!$target.closest("#" + proxy._id + "_ColumnMenu").hasClass("e-columnmenu"))
            {
                proxy._clearColumnMenu();
            }
        },
         /* remove child detail row wraper when detail row contains another treegrid*/
        _removeInnerDetailRows: function ($target, eventCheck)
        {
            /* check if target is in any of detail rows */
            var detailRows = $(".e-detailscellwrapperfly"),
                detailRowsLength = detailRows.length, count = 0, containsFlag = false;

            for (count; count < detailRowsLength; count++) {
                if ($.contains(detailRows[count], $target[0]) || $target == detailRows[count]) {
                    containsFlag = true;
                    break;
                }
            }

            var countFlag = false;
            if (ej.isNullOrUndefined(detailRows[count])) {
                count = detailRows.index($("#" + this._id + "detailscellwrapper"));
                countFlag = true;
            }
            var countFlagString = countFlag ? "" : ",";
            var parentTreeGridId = $(detailRows[count]).attr("parentTreeGridIds");
            var parentDetailWrapperElement = $("[parentTreeGridIds$='" + countFlagString + parentTreeGridId + "']"),
                length = parentDetailWrapperElement.length, i = 0;
            this._triggerDetailRowHideEvent(parentDetailWrapperElement, eventCheck);
        },

        /* trigger detail row collaspse event */
        _triggerDetailRowHideEvent: function (elements, eventCheck)
        {
            var proxy = this,
                model = this.model;
            var length = elements.length, i = 0;

            if (length > 0 && $("#" + this._id + "detailscellwrapper").length > 0)
            {
                var detailElement = $("#" + this._id + "detailscellwrapper"),
                    selectedItem = model.selectedItem;
                var detailsCollapseEventArgs = {};
                detailsCollapseEventArgs.rowData = selectedItem;

                if (!this._trigger("detailsHidden", detailsCollapseEventArgs) || eventCheck) {
                    for (i; i < length; i++) {
                        var treeGridContainerId = $(elements[i]).attr('id').replace("detailscellwrapper", "");
                       
                            $("#" + treeGridContainerId).find(".e-detailsrowexpand").removeClass("e-detailsrowexpand").addClass("e-detailsrowcollapse");
                            $("#" + treeGridContainerId).find(".e-detailsinfoiconhide").removeClass("e-detailsinfoiconhide").addClass("e-detailsinfoiconshow");
                        
                            $(elements[i]).remove();
                    }
                }
            }
        },
        /* remove all child detail row */
        _removeDetailsRow: function (e, eventCheck)
        {
            var model= this.model;
            
                if (e) {
                    var target = $(e.target);
                    var detailCellElement = target;
                    if (target.parent("td").hasClass("e-detailsrowcell"))
                        detailCellElement = target.parent("td");

                    if (target.closest(".e-popup").length == 0 && target[0] != $('html').get(0)) {

                        /* check if target is in any of detail rows */
                        var detailRows = $(".e-detailscellwrapperfly"),
                            detailRowsLength = detailRows.length, count = 0, containsFlag = false;

                        for (count; count < detailRowsLength; count++) {
                            if ($.contains(detailRows[count], target[0]) || target[0] == detailRows[count]) {
                                containsFlag = true;
                                break;
                            }
                        }

                        if (!target.hasClass("e-detailscellwrapperfly") && !detailCellElement.hasClass("e-detailsrowexpand") && !detailCellElement.hasClass("e-detailsrowcollapse")
                            && !containsFlag) {
                            this._removeInnerDetailRows($("#" + this._id + "detailscellwrapper"), eventCheck);
                        }

                        if (target.hasClass("e-detailscellwrapperfly") || detailCellElement.hasClass("e-detailsrowcollapse") || (!detailCellElement.hasClass("e-detailsrowexpand") && containsFlag)) {
                            var countFlag = false;
                            if (ej.isNullOrUndefined(detailRows[count])) {
                                count = detailRows.index($("#" + this._id + "detailscellwrapper"));;
                                countFlag = true;
                            }
                            var countFlagString = countFlag ? "" : ",";
                            var parentTreeGridId = $(detailRows[count]).attr("parentTreeGridIds");
                            var parentDetailWrapperElement = $("[parentTreeGridIds$='" + countFlagString + parentTreeGridId + "']");
                            this._triggerDetailRowHideEvent(parentDetailWrapperElement, eventCheck);
                        }
                    }
                } else {
                    $("#" + this._id + "detailscellwrapper").length > 0 && this._removeInnerDetailRows($("#" + this._id + "detailscellwrapper")[0], eventCheck);
                    this._triggerDetailRowHideEvent($("#" + this._id + "detailscellwrapper"), eventCheck);
                }
            
        },

        /* show hide detail rows public method*/
        
        showHideDetailsRow:function(rowIndex)
        {
            var proxy = this, model = this.model;
            /* save if tree grid is in edit state */
            if (proxy._isRowEdit) {
                this._saveRow();
            }
            if (model.isEdit)
            {
                this.saveCell();
            }

            if (model.showDetailsRow && model.detailsTemplate && model.showDetailsRowInfoColumn && !ej.isNullOrUndefined(rowIndex)
                 && rowIndex <= model.updatedRecords.length && proxy.getExpandStatus(model.updatedRecords[rowIndex])) {
                if (rowIndex != this.selectedRowIndex()) {
                    if (rowIndex != -1 && $.inArray(rowIndex, proxy._selctedRowsIndexes == -1) && model.allowSelection && model.updatedRecords.length > rowIndex) {
                            proxy.selectRows(rowIndex);
                            this.selectedRowIndex(rowIndex);
                    }
                }

                this.updateScrollBar();
                var targetRowElement = $(ej.TreeGrid.getRowByIndex(proxy, rowIndex)), targetCellElement;
                if (targetRowElement.length > 0 && proxy.getExpandStatus(model.selectedItem))
                {
                    targetCellElement = targetRowElement[0].cells[model.columns.length];
                    if ($(targetCellElement).length > 0 && $(targetCellElement).hasClass("e-detailsrowcell"))
                    {
                        if (this.getRows().find(".e-detailsrowexpand")[0] != targetCellElement)
                            this._removeDetailsRow();

                            this._detailsExpandCollapse($(targetCellElement))
                    }
                }
            }
        },
        /* expand collapse the detail row */
        _detailsExpandCollapse:function($target)
        {
            var proxy = this,
                model = this.model;
            
            if ($target.parent("td").hasClass("e-detailsrowcell"))
                $target = $target.parent("td");
            var rowElement = $target.closest('tr'),
                detailsIconElement = rowElement.find(".e-detailsrowcell");

            if (detailsIconElement.hasClass("e-detailsrowcollapse")) {

                var rowIndexValue = this.getIndexByRow(rowElement),
                   rowData = this.model.currentViewData[rowIndexValue];
                if (ej.isNullOrUndefined(rowData) || rowData.isSummaryRow)
                    return;
                var rowBottom = this.getOffsetRect(rowElement[0]),
                   detailtr = $($.render[proxy._id + "_detailRowTemplate"](rowData)),
                   detailsCellElement = $(detailtr).find(".e-detailscellwrapper").attr({ "id": this._id + "detailscellwrapper" }),
                   width = proxy._gridWidth - (proxy.element.find(".e-scrollcss").length > 0 ? 18 : 0) - 2;//2 - treegrid border

                /* add treegrid attribute for detail row wraaper element */
                var rowWrapperElement = this.element.parents().filter(".e-detailscellwrapperfly"),
                    wrapperElementID = this._id, tempTreeGridId = "";
                while (rowWrapperElement.length > 0) {
                    tempTreeGridId = rowWrapperElement.attr("id").replace("detailscellwrapper", "");
                    wrapperElementID += "," + tempTreeGridId;
                    rowWrapperElement = $("#" + tempTreeGridId).parents().filter(".e-detailscellwrapperfly");
                }

                detailsCellElement.attr("parentTreeGridIds", wrapperElementID);
                
                //Edge detection for detail rows
                var treeGridContentOffset = this.getOffsetRect(proxy._$gridContent[0]), topPosition, bottomPosition;
                treeGridContentOffset.bottom = treeGridContentOffset.top + proxy._$gridContent[0].offsetHeight;
                topPosition = rowBottom.top + rowElement[0].offsetHeight;
                bottomPosition = topPosition + model.detailsRowHeight;
                if (topPosition > treeGridContentOffset.bottom || bottomPosition > treeGridContentOffset.bottom)
                {
                    var tempTopPosition = rowBottom.top - model.detailsRowHeight;
                    if (tempTopPosition > treeGridContentOffset.top)
                        topPosition = tempTopPosition;
                }

                detailsCellElement.removeClass("e-detailscellwrapper").addClass("e-detailscellwrapperfly");
                var scrollerLeft = proxy._$gridContent.ejScroller("isHScroll") ? proxy._$gridContent.ejScroller("option", "scrollLeft") : 0;
                detailsCellElement.css({
                    "top": topPosition, "left": rowBottom.left + scrollerLeft,
                                       "z-index": 100, position: "absolute", width: width }).appendTo("body");
                this._trigger("refresh");
                var eventArgs = {};
                eventArgs.detailsElement = detailsCellElement;
                eventArgs.data = rowData;
                eventArgs.rowIndex = this.selectedRowIndex();
                this._trigger("detailsDataBound", eventArgs);

                /* details show/expand event trigger*/
                var expandEventArgs = {};
                expandEventArgs.detailsElement = detailsCellElement;
                expandEventArgs.data = rowData;
                expandEventArgs.rowIndex = this.selectedRowIndex();
                expandEventArgs.appendTarget = null;
                if (this._trigger("detailsShown", expandEventArgs)) {
                    this._removeInnerDetailRows($target);
                }else
                {
                    detailsIconElement.find(".e-detailsinfoiconshow").removeClass("e-detailsinfoiconshow").addClass("e-detailsinfoiconhide");
                    detailsIconElement.removeClass("e-detailsrowcollapse").addClass("e-detailsrowexpand");
                    if (expandEventArgs.appendTarget)
                    {
                        $(detailsCellElement).css({ "top": "auto", left: "auto", position: "auto" });
                        $(detailsCellElement).detach().appendTo($("#" + expandEventArgs.appendTarget));
                    }

                }
            }else
            {
                this._removeInnerDetailRows($target);
            }
        },

        //CLICK HANDLER FOR PERFORM THE SORTING ACTION
        _onHeaderClick: function (e) {

            var proxy = this,
                model = proxy.model;
            proxy._cellMouseLeave();
            proxy._clearContextMenu();
            this._removeDetailsRow();
            if ($(e.target).is(".e-ascending, .e-descending")) e.target = e.target.parentNode;

            var $target = $(e.target),
                columnName,
                columnSortDirection;

            proxy.getHeaderTable().find(".e-columnheader").find(".e-headercellactive").removeClass("e-headercellactive");

            if ($target.hasClass("e-headercelldiv") || $target.hasClass("e-headercell")) {

                if (!model.allowSorting) return false;
                proxy._isRowEdit = false;
                columnName = $.trim($target.text());

                if (this.getColumnByHeaderText(columnName).allowSorting == false) 
                    return false;

                if ($target.find('span').hasClass("e-ascending")) {
                    columnSortDirection = ej.sortOrder.Descending;
                } else {
                    columnSortDirection = ej.sortOrder.Ascending;
                }

                if (model.allowMultiSorting && e.ctrlKey) {
                    proxy._multiSortRequest = true;
                }
                proxy._isRefreshAddedRecord = false;

                proxy.sortColumn(columnName, columnSortDirection);
                var index = proxy.selectedRowIndex();
                proxy.selectRows(index);
                if (proxy.model.isFromGantt && this.selectedRowIndex()>=0) {
                    var args = {};
                    args.data = proxy.model.selectedItem;
                    args.target = "ejTreeGrid";
                    args.recordIndex = this.selectedRowIndex();
                    proxy._trigger("rowSelected", args);
                }
            }

            // Show the column menu while clicking the column menu icon

            if ($target.hasClass("e-columnmenu-icon"))
            {
                
                if (!$target.data("isClicked")) {
                    proxy._renderColumnMenu(e);
                    $(".e-columnicon").data("isClicked", false);
                    $target.data("isClicked", true);
                }
                else{
                    proxy._clearColumnMenu();
                    $target.data("isClicked", false);
                }

            }
            else
                proxy._clearColumnMenu();
            if ($target.hasClass("e-checkbox"))
                return true
            else
                return false;
        },


        //HEADER MOUSE DOWN EVENT FOR COLUMN RESIZING
        _headerMouseDown: function (e) {

            var proxy = this,
                model = proxy.model;
            proxy._cellMouseLeave();

            if (model.allowColumnResize) proxy._resizer._mouseDown(e);

            if (($(e.target).hasClass("headercelldiv") && $(e.target).hasClass("headercell"))) {

                var $headercell = $(e.target).hasClass("headercelldiv") ? $(e.target).parent() : $(e.target);
                model.headerEffects && $headercell.addClass("headercellactive");

            }
        },

        //TREE GRID CONTENT MOUSE DOWN EVENT

        _contentMouseDown: function (e) {
            var $target = $(e.target),
                proxy = this;
            if ($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vdown")
                  || $target.hasClass("e-vhandlespace") || $target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hdown")
                  || $target.hasClass("e-hhandlespace"))
            {
                proxy._clearContextMenu();
                proxy._clearColumnMenu();
            }
        },


        //EVENT FOR COLUMN RESIZE
        _enableColumnResizeEvents: function () {

            var proxy = this;

            if (proxy.model.allowColumnResize) {
                proxy._on(proxy.element, "mousemove", proxy._mouseMove);
                proxy._on(proxy.element, "mouseup", proxy._mouseUp);
            } else {
                proxy._off(proxy.element, "mousemove", proxy._mouseMove);
                proxy._off(proxy.element, "mouseup", proxy._mouseUp);
            }
        },


        _mouseMove: function (e) {

            var proxy = this, model = this.model;

            if (proxy.model.allowColumnResize) {
                proxy._resizer._mouseMove(e);

                if ($(e.target).parent().css("cursor") !== "col-resize") {
                    if ((model.allowSorting || model.showColumnChooser) && !$(e.target).closest("th").hasClass("e-detailheadercell")) {
                        proxy.getHeaderTable().find(".e-columnheader").css({ 'cursor': 'pointer' });
                    } else {
                        proxy.getHeaderTable().find(".e-columnheader").css({ 'cursor': 'default' });
                    }
                }
                proxy._$gridHeaderContainer.scrollLeft(proxy._$gridContent.ejScroller("option", "scrollLeft"));
            }
        },


        _mouseUp: function (e) {

            var proxy = this,
                model = proxy.model;

            if (model.allowColumnResize) {
                
                if ($(e.target).hasClass('e-headercelldiv') || $(e.target).hasClass('e-reSizeColbg')) {
                    if (proxy.model.isEdit)
                        proxy.saveCell();
                    else if (proxy._isRowEdit)
                        proxy._saveRow();
                    (proxy.element).focus();
                    proxy._clearColumnMenu();
                }

                // if treegrid column resizer in initialted
                if (proxy._resizer._expand) {
                    proxy._resizer._mouseUp(e);
                    if (model.isFromGantt)
                        proxy.refreshScroller(proxy._$gridContent.width());
                    else {
                        proxy._$gridContent.ejScroller("refresh");
                        proxy._$gridHeaderContainer.scrollLeft(proxy._$gridContent.ejScroller("option", "scrollLeft"));
                    }
                    proxy._updateScrollCss();
                    proxy._resizeFilteringElements();
                    proxy._resizeFooterSummaryColumn();
                }
            }
        },

        // resize the filtering elements after column resized.

        _resizeFilteringElements:function()
        {
            var proxy = this;
            if (proxy.model.allowFiltering) {
                var args = {}, colLength = proxy.model.columns.length;
                ej.TreeGrid.Actions.Filter = "filtering";
                args.requestType = ej.TreeGrid.Actions.Filter;
                for (var item = 0; item < colLength ; item++) {
                    proxy.model.columns[item].width = proxy._columnsWidthCollection[item];
                    args.fieldname = proxy.model.columns[item].field;
                    this._filteringElements(args);
                }
            }
        },

        //Resize the total summary row column after resized the header column.
        _resizeFooterSummaryColumn: function () {
            var proxy = this;
            if (proxy.model.showSummaryRow) {
                var SummaryCol = proxy._$totalSummaryRowContainer.find("colgroup").find("col");
                for (var i = 0; i < proxy.model.columns.length; i++)
                    SummaryCol.eq(i).css("width", proxy._columnsWidthCollection[i]);
            }
        },



        //ENABLE THE EDITING EVENTS
        _enableEditingEvents: function () {

            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing) {

                if (model.editSettings.editMode.toLowerCase() == "cellediting" || model.editSettings.editMode.toLowerCase() == "rowediting") {
                    proxy._on(proxy.element, "mousedown", ".e-gridcontent", proxy._saveCellHandler);
                }

                proxy._on(this.element, "dblclick", ".e-gridcontent", proxy._editdblClickHandler);
            } else {

                if (model.editSettings.editMode.toLowerCase() == "cellediting" || model.editSettings.editMode.toLowerCase() == "rowediting") {
                    proxy._off(proxy.element, "mousedown", ".e-gridcontent", proxy._saveCellHandler);
                }

                proxy._off(this.element, "dblclick", ".e-gridcontent", proxy._editdblClickHandler);
            }
        },


        //ENABLE THE MOUSEDOWNEVENT HANDLER FOR SAVE CELL
        _saveCellHandler: function (e) {

            var $target = $(e.target),
                proxy = this, model = proxy.model, targetIndex;
            if (model.isFromGantt && $target.closest("tr").length) {
                var $targetElement = $target.closest("tr"),
                    $gridRows = proxy.getRows(),
                    rowIndex = $gridRows.index($targetElement),
                    record = model.currentViewData && model.currentViewData[rowIndex];
                targetIndex = model.updatedRecords.indexOf(record);             
            }
            //SKIP THE SCROLL BAR CLICK IN TREE GRID.
            if ($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vdown") || $target.hasClass("e-vhandlespace"))
                return;
            if ($target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hup") || $target.hasClass("e-hdown") || $target.hasClass("e-hhandlespace"))
                return;
           e.stopPropagation();
           if (proxy._isRowEdit) {
               if ($target.closest("form#" + proxy._id + "EditForm").length == 0)
                   proxy._saveRow();
           }else if ($target.closest(".e-popup").length == 0 &&
                $target.closest(".e-rowcell").find("#" + proxy._id + "EditForm").length == 0) {
                proxy.saveCell();
           }

            //Row selection for Gantt alone here
            //Why because, To select the clicked row after saving the edited cell (Due to predecessor and Row refresh)
           if (model.isFromGantt && model.updatedRecords.length && targetIndex) {
               if (this.selectedRowIndex() != targetIndex && !proxy._rowSelectingEventTrigger(this.selectedRowIndex(), targetIndex)) {
                   proxy.selectRows(targetIndex);
                   proxy._rowSelectedEventTrigger(targetIndex);
               }
           }
        },

        //Mouse right click (Context Menu) handler

        _rightClick: function (e) {
            e.preventDefault();
            var proxy = this,
                model = proxy.model,
               $target = $(e.target),
               div = $target[0].parentNode,
               $ganttGridRows,
               row,
               recordIndexr,
               item, selectIndex,
               args = {},
               checkEditMenu = false;

            /* check inner treegrid click events and  */
            if ($target.closest(".e-treegrid").attr("id") !== this._id) return true;

            if (proxy.model.contextMenuSettings.showContextMenu){
                proxy._clearContextMenu();
            this._removeDetailsRow();
            proxy.disableTooltip();
            if ($target.hasClass("e-summaryrowcell"))
                return;
            if ($target.hasClass("e-cell") || $target.hasClass("e-treegridexpand"))
                div = $target.closest('tr');
            if ($(div).hasClass("e-treegridrows") || $(div).hasClass("e-rowcell"))
                $ganttGridRows = proxy.getRows();
            else
                $ganttGridRows = $(proxy.getRows());
            row = $target.closest('tr');
            recordIndexr = $ganttGridRows.index(row);
            proxy.model.currentViewData = proxy.getCurrentViewData();
            item = proxy.model.currentViewData[recordIndexr];
            recordIndexr = proxy.model.updatedRecords.indexOf(item);
            args.data = item;
            args.recordIndex = recordIndexr;
            if (model.isEdit)
                checkEditMenu = ($target.closest(".e-rowcell").find("form#" + proxy._id + "EditForm").length > 0);
            else if (this._isRowEdit)
                checkEditMenu = ($target.closest(".e-rowedit").find("form#" + proxy._id + "EditForm").length > 0);
            //Context menu for edited row or empty records
            if ((proxy.dataSource() == null || proxy.dataSource().length == 0 || checkEditMenu || model.flatRecords.length == 0)) {
                proxy._contextMenuItems = [];
                proxy._contextMenuItems = proxy._getContextMenuItems();
                proxy._subContextMenuItems = [];
                proxy._updateContextmenuOption(recordIndexr, item);
                proxy._clearContextMenu();
                if (proxy._contextMenuItems.length > 0)
                    proxy._renderContextMenu(e, recordIndexr, item);
                
            } else if (item) {
                e.preventDefault();
                proxy._contextMenuItems = [];
                proxy._contextMenuItems = proxy._getContextMenuItems();
                proxy._subContextMenuItems = [];
                proxy._subContextMenuItems = proxy._getsubContextMenuItems();
                proxy._updateContextmenuOption(recordIndexr, item);
                proxy._clearContextMenu();
                if (proxy._contextMenuItems.length > 0)
                    proxy._renderContextMenu(e, recordIndexr, item);
                proxy.selectRows(recordIndexr);
                proxy._cancelSaveTools();
            }
           }
			if (!proxy.model.allowSelection)
                proxy.clearSelection(recordIndexr);
            proxy._clearColumnMenu();
        },

        //DOUBLE CLICK EVENT HANDLER FOR DIALOG EDITING

        _editdblClickHandler: function (e) {

            var proxy = this,
                model = proxy.model,
                $form = $("#" + proxy._id + "EditForm"),
                args = {},
                $target = $(e.target),
                row = $target.closest('tr'),
                record,
                cellEditingDetails = proxy._cellEditingDetails;
            if ($target.hasClass("e-vhandle") || $target.hasClass("e-vscrollbar") || $target.hasClass("e-vup") || $target.hasClass("e-vdown") || $target.hasClass("e-vhandlespace") ||
                $target.hasClass("e-treegridexpand") || $target.hasClass("e-treegridcollapse"))
                return;
            if ($target.hasClass("e-hhandle") || $target.hasClass("e-hscrollbar") || $target.hasClass("e-hup") || $target.hasClass("e-hdown") || $target.hasClass("e-hhandlespace"))
                return;
            /* check inner treegrid click events and  */
            if ($target.closest(".e-treegrid").attr("id") !== this._id) return true;

            if ($target.hasClass("e-rowcell") || $target.parent().hasClass("e-rowcell") || ($target.closest("td").hasClass("e-rowcell"))
                || ($target.closest("td").hasClass("e-rowcell")) && $target.hasClass("e-cell")) {

                if (model.editSettings.editMode === "normal") {
                    args.requestType = ej.TreeGrid.Actions.BeginEdit;
                    proxy._trigger("actionBegin", args);
                }

                if (proxy._cellEditingDetails.cancelSave) {

                    proxy._cellEditingDetails.cancelSave = false;
                    return;

                }

                if ($form.length === 0 && proxy.getRows() !== null) {

                    if (model.editSettings.editMode.toLowerCase() == "cellediting") {
                        proxy.element.focus();
                        proxy._cellEditingDetails.columnIndex = proxy.getCellIndex(e);
                        proxy._cellEditingDetails.rowIndex = proxy.getRowIndex(e);
                        record = model.currentViewData[proxy._cellEditingDetails.rowIndex];
                        if (record.summaryRow)
                            return;
                        model.selectedItem = record;
                        if (model.columns[cellEditingDetails.columnIndex].allowEditing == false)
                            return;
                        fieldName = proxy._cellEditingDetails.columnIndex >= 0 && model.columns[proxy._cellEditingDetails.columnIndex].field;
                        if (fieldName) {
                            model.editSettings.allowEditing &&
                                model.editSettings.editMode.toLowerCase() == ej.TreeGrid.EditMode.CellEditing.toLowerCase() &&
                                proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName);
                        }
                    }
                    else if (model.editSettings.editMode.toLowerCase() == "rowediting") {
                        model.currentViewData = proxy.getCurrentViewData();
                        if (proxy.getCellIndex(e) != -1 && proxy.getRowIndex(e) != -1) {
                            var rowIndex = proxy.getRowIndex(e);
                            if (model.enableVirtualization)
                                rowIndex = model.updatedRecords.indexOf(model.selectedItem);
                            proxy._editRow(rowIndex);
                        }
                    }
                    $form = $("#" + proxy._id + "EditForm");
                    if ($form.length > 0)
                        proxy._editAddTools();
                }

                if (!proxy.model.allowSelection)
                    row.find(".e-active").removeClass("e-active").removeClass("e-selectionbackground");
                proxy._clearContextMenu();
                this._removeDetailsRow();
            }
        },

        //To hide the datepicker and dropdown list.
        _popupHide: function (element) {
            var proxy = this,
             column = proxy.model.columns,
             object = null,
             dateobj = null,
             tdElement = element && $(element).closest("td");
            if (proxy.model.allowFiltering) {
                tdElement = element && $(element).closest("td, th");
            }
            var columnChooser = $("#" + proxy._id + "_ColumnMenu").length
            if (columnChooser > 0)
                proxy._clearColumnMenu();
            for (i = 0; i < column.length; i++) {
                object = null;

                if (column[i]) {
                    if (column[i].editType == "datepicker" || column[i].editType == "datetimepicker")
                        object = $("#" + proxy._id + column[i].field).data("ejDatePicker");
                    else if (column[i].editType == "dropdownedit") {
                        var dropdownobject = $("#" + proxy._id + column[i].field).data("ejDropDownList");
                        if (dropdownobject && $(tdElement).find("#" + dropdownobject._id).length == 0)
                            dropdownobject.hidePopup();
                    }                    
                    if (object && $(tdElement).find("#" + object._id).length == 0)
                        object.hide();

                    if (column[i].filterEditType == "datepicker" || column[i].filterEditType == "datetimepicker") {
                        dateobj = $("#" + proxy._id + "_" + column[i].field + "_filterbarcell").data("ejDatePicker");
                        if (dateobj && $(tdElement).find("#" + dateobj._id).length == 0)
                            dateobj.hide();
                    }
                    if (column[i].filterEditType == "dropdownedit") {
                        var dropdownobject = $("#" + proxy._id + "_" + column[i].field + "_filterbarcell").data("ejDropDownList");
                        if (dropdownobject && $(tdElement).find("#" + dropdownobject._id).length == 0)
                            dropdownobject.hidePopup();
                    }
                }
            }
        },
        //PERFORM SCROLLING IN TREEGRID
        _mouseWheel: function (e) {

            var proxy = this,
                delta = null,
                ori = e;            
            proxy._popupHide();
            
            e = e.originalEvent;

            try {
                e.preventDefault ? e.preventDefault() : ori.preventDefault();
            }
            catch (err) {
            }
			
			 if (proxy._$gridContent.height() > proxy._$gridContainer.height()) { 
                proxy._$gridContent.ejScroller("scrollY", 0, true);
                return;
            }

            if (e.wheelDelta) {
                delta = -e.wheelDelta / 120;
                if (window.opera) {
                    if (parseFloat(window.opera.version, 10) < 10)
                        delta = -delta;
                }
            } else if (e.detail) {
                delta = e.detail / 3;
            }
	
            if (!delta) return false;

            var scrollTop = proxy._$gridContent.ejScroller("option", "scrollTop"),

                change = scrollTop + (delta * 57);  

            var h = parseInt(proxy.model.updatedRecords.length * 30) - (proxy._viewPortHeight-18);

            
            
            if (change> h) {
                change = h;                
            }

            if (change < 0) {
                change = 0;
            }
            
                proxy._updateScrollTop(change);
           
            return true;

        },
        _getCollapsedParentItem:function(record)
        {
            var parentRecord = record.parentItem;
            if (parentRecord) {
                if (this.getExpandStatus(parentRecord) === true) {
                    return parentRecord;
                } else {
                    return this._getCollapsedParentItem(parentRecord)
                }
            }
            else {
                return null;
            }
        },


        // Update the scroll bar while edit, expand and collapse the record for focusing the row.
        updateScrollBar: function (rowIndex) {
            var proxy = this,
                model = proxy.model, horizontalScrollBarHeight = 0,
                currentUpdatedRecords, recordIndex,
                rowHeight = model.rowHeight + proxy._detailsRowHeight,
                isInViewPortAbove, isInViewPortBelow;
            currentUpdatedRecords = proxy.getExpandedRecords(model.updatedRecords);
            if (rowIndex) {
                recordIndex = currentUpdatedRecords.indexOf(model.updatedRecords[rowIndex]);
            } else {
                recordIndex = currentUpdatedRecords.indexOf(this.selectedItem());
            }

            var rowTop = recordIndex * rowHeight,
                rowBottom = rowTop + model.rowHeight,
                scrollTop,
                containerWidth,
                scrollerExist = proxy._$gridContent.children(".e-content").length;
   
            if (scrollerExist)
                scrollTop = proxy._$gridContent.children(".e-content").scrollTop();
            else
                scrollTop = proxy._$gridContent.scrollTop();
    
            if(proxy._$gridContent.ejScroller("isHScroll")  || proxy.element.find(".e-borderbox").length > 0)
            {
                horizontalScrollBarHeight = 18; /* default horizontal scrollbar height*/
            }

            isInViewPortAbove = (rowTop < scrollTop);
            isInViewPortBelow = (scrollTop + proxy._viewPortHeight - horizontalScrollBarHeight) < rowBottom;


            if (isInViewPortAbove || isInViewPortBelow) {
    
                if ((rowTop + proxy._viewPortHeight) > currentUpdatedRecords.length * rowHeight) {
                    rowTop = currentUpdatedRecords.length * rowHeight - proxy._viewPortHeight;

                    if (isInViewPortBelow)
                        rowTop += horizontalScrollBarHeight;
                }

                    var args = {
                         requestType: "scroll",
                         delta: rowTop
                    };
                    if (scrollerExist)
                        proxy._updateScrollTop(rowTop);
                    else {
                        proxy._$gridContent.scrollTop(args.delta);
                        proxy._completeAction(args);
                    }
                    if (model.enableVirtualization) {
                        proxy.selectRows(proxy.selectedRowIndex());
                        proxy.element[0].focus();
                    }
            }
        },

        //KEY PRESSED EVENT FOR KEY BOARD INTERACTION IN TREEGRID CONTROL

        _keyPressed: function (action, target, e) {

            var proxy = this,
            model = this.model,
            lastRowIndex,
            selectingRowIndex,
            updatedRecords = model.updatedRecords,
            expandedRecords = proxy.getExpandedRecords(updatedRecords),
            currentSelectingRecord;
            this._removeDetailsRow();
            //CHECK THE ALLOWKEYBOARDNAVIGATION
            if (!model.allowKeyboardNavigation){
                return false;
            }

            var returnValue = false,
                $target = $(target);
            $form = $("#" + proxy._id + "EditForm");
            if (action !="downArrow" && action != "upArrow" && action != "saveRequest" && action != "moveCellLeft" && action != "moveCellRight"
                   && action != "cancelRequest" && $target.prop("tagName") == "INPUT") // Skip the keypress event for filter bar input box except enter
                return true;
            var contextMenu, subContextMenu,columnMenu,columnChooserMenu,isVisible,isVisibleColumnChooserMenu;
            if (model.isFromGantt) {
                contextMenu = $("#" + proxy._id.replace("ejTreeGrid", "") + "_ContextMenu");
                subContextMenu = $("#" + proxy._id.replace("ejTreeGrid", "") + "_SubContextMenu");
            }
            else {
                contextMenu = $("#" + proxy._id + "_ContextMenu");
                subContextMenu = $("#" + proxy._id + "_SubContextMenu");
            }
            columnMenu = $("#" + proxy._id + "_ColumnMenu");
            columnChooserMenu = $("#" + proxy._id + "ccDiv_wrapper");
            isVisible = subContextMenu.css("visibility") == "visible" ? true : false;
            isVisibleColumnChooserMenu = columnChooserMenu.is(":visible");
            switch (action) {

                //save the current edited cellValue on Enter key press
                case "saveRequest":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) {
                            proxy._endEdit();
                            proxy.element[0].focus();
                        }
                        if (model.filterSettings.filterBarMode == "onEnter" && model.allowFiltering && $target.closest("th").hasClass("e-filterbarcell"))
                            proxy._filterBarHandler(target);
                    }
                    else
                    {
                        if (contextMenu.length)
                        {
                            var option = null;
                            if (isVisible) {
                                var subContextMenuItem = subContextMenu.find("div.e-contextmenu-mouseover");
                                if (subContextMenuItem.length > 0) {
                                    option = subContextMenuItem[0].id;
                                }
                            }
                            else
                            {
                                var contextMenuItem = contextMenu.find("div.e-contextmenu-mouseover");
                                if (contextMenuItem.length > 0) {
                                    option = contextMenuItem[0].id;
                                }
                            }
                            if(!ej.isNullOrUndefined(option))
                            {
								 // Context Menu operation for Gantt
                                if (model.isFromGantt) {
                                    var eventArgs = {};
                                    switch (option) {
                                        case "Task":
                                            eventArgs.requestType = "contextMenuTaskDetails";
                                            proxy._trigger("contextMenuAction", eventArgs);
                                            proxy._clearContextMenu();
                                            break;
                                        case "Indent": eventArgs.requestType = "contextMenuIndent";
                                            proxy._trigger("contextMenuAction", eventArgs);
                                            proxy._clearContextMenu();
                                            break;
                                        case "Outdent": eventArgs.requestType = "contextMenuOutdent";
                                            proxy._trigger("contextMenuAction", eventArgs);
                                            proxy._clearContextMenu();
                                            break;
                                        case "Delete": eventArgs.requestType = "contextMenuDelete";
                                            proxy._trigger("contextMenuAction", eventArgs);
                                            proxy._clearContextMenu();
                                            break;
                                        case "Add":
                                            proxy._trigger("setInitialData");
                                            break;
                                        case "Above": eventArgs.requestType = "save";
                                            eventArgs.position = "Above";
                                            proxy._trigger("subContextMenuAction", eventArgs);
                                            proxy._clearContextMenu();
                                            break;
                                        case "Below": eventArgs.requestType = "save";
                                            eventArgs.position = "Below";
                                            proxy._trigger("subContextMenuAction", eventArgs);
                                            proxy._clearContextMenu();
                                            break;
                                        default:
                                            eventArgs.event = option;
                                            proxy._trigger("customContextMenuEvent", eventArgs);
                                            break;
                                    }
                                }
								 // Context Menu operation for Tree grid
                                else
                                {
                                    var index = proxy._contextMenuSelectedIndex,
                                        item = proxy._contextMenuSelectedItem;
                                    switch (option) {
                                        case "Delete":
                                            proxy._contextMenuOperations("Delete");
                                            break;
                                        case "Above":
                                            proxy._contextMenuOperations("Above", item);
                                            break;
                                        case "Below":
                                            proxy._contextMenuOperations("Below", item);
                                            break;
                                        case "Add":
                                            proxy._contextMenuOperations("Add");
                                            break;
                                        case "Edit":
                                            proxy._contextMenuOperations("Edit");
                                            break;
                                        case "Save":
                                            proxy._contextMenuOperations("Save");
                                            break;
                                        case "Cancel":
                                            proxy._contextMenuOperations("Cancel");
                                            break;
                                        default:
                                            proxy._contextMenuOperations(option);
                                            break;
                                    }
                                }
                            }
                        }
                        else if (columnMenu.length)
                        {
                            var option = null,
                                columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection");                            
                            if (isVisibleColumnChooserMenu) {                                
                                option = columnChooserMenuItem.data("column");
                            }
                            else
                            {
                                var columnMenuItem = columnMenu.find("div.e-columnmenuselection");
                                if (columnMenuItem.length > 0) {
                                    option = columnMenuItem[0].id;
                                }
                            }
                            if (!ej.isNullOrUndefined(option))
                            {
                                $target = proxy._columnMenuTarget;
                                switch(option)
                                {
                                    case proxy._id + "_SortAscendingChooser":
                                        var columnName = $.trim($target.prev("div.e-headercelldiv").text());
                                        if (proxy.model.allowMultiSorting) proxy._multiSortRequest = true;
                                        proxy.sortColumn(columnName, ej.sortOrder.Ascending);
                                        proxy._multiSortRequest = false;
                                        break;
                                    case proxy._id + "_SortDescendingChooser":
                                        var columnName = $.trim($target.prev("div.e-headercelldiv").text());
                                        if (proxy.model.allowMultiSorting) proxy._multiSortRequest = true;
                                        proxy.sortColumn(columnName, ej.sortOrder.Descending);
                                        proxy._multiSortRequest = false;
                                        break;
                                    default:
                                        var isColumnVisible = columnChooserMenuItem.find("span").attr("aria-checked");
                                        isColumnVisible == "true" ? proxy.hideColumn(option) : proxy.showColumn(option);
                                        break;
                                }
                            }
                        }

                    }
                    break;

                    //cancel the cell editing in the current cell Esc key press
                case "cancelRequest":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if ($form.length > 0) {
                            var editMode = model.editSettings.editMode;
                            if (editMode.toLowerCase() == "cellediting") {
                                if (proxy._isRowEdit)
                                    proxy.cancelRowEditCell();
                                else
                                    proxy.cancelEditCell();
                            }
                            else if (editMode.toLowerCase() == "rowediting") {
                                proxy.cancelRowEditCell();
                            }
                            proxy._cancelSaveTools();
                        }
                        proxy.element[0].focus();
                        if (proxy._dragMouseDown == true) {
                            $(proxy._dragmouseOverTooltip).remove();
                            $(".intend").find(".e-icon").css({ "display": "none" });
                            $(".intendparent").find(".e-icon").css({ "display": "none" });
                            proxy._dropCancel = true;
                            proxy._dragMouseDown = false;
                        }
                    }
                    else {
                        proxy._clearContextMenu();
                        proxy._clearColumnMenu();
                    }
                    break;
                case "editRecord":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        var editSettings = model.editSettings,
                            editMode = editSettings.editMode;
                        if (editSettings.allowEditing && proxy._selectedRowsIndexes.length == 1) {
                            if (editMode.toLowerCase() == "cellediting") {
                                proxy._cellEditingDetails.rowIndex = this.selectedRowIndex();
                                proxy._cellEditingDetails.columnIndex = proxy._cellIndex;//proxy.getCellIndex(e, target);
                                if (proxy._cellEditingDetails.columnIndex > -1) {
                                    proxy.updateScrollBar();
                                    fieldName = proxy.model.columns[proxy._cellEditingDetails.columnIndex].field;
                                    fieldName && proxy.cellEdit(proxy._cellEditingDetails.rowIndex, fieldName);
                                    proxy._editAddTools();
                                }
                            }
                            else if (editMode.toLowerCase() == "rowediting" && !proxy._isRowEdit) {
                                var index = proxy.selectedRowIndex();
                                if (index >= 0) {
                                    if (model.enableVirtualization)
                                        proxy.updateScrollBar();
                                    proxy._editRow(index)
                                }
                            }
                        }
                    }
                    break;

                    //select the first row while press the Home key
                case "firstRowSelection":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit)
                            return true;

                        if (updatedRecords.length > 0) {
                            if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), 0)) {
                                proxy.selectRows(0);
                                proxy.updateScrollBar();
                                proxy.element[0].focus();
                                proxy._rowSelectedEventTrigger(0);
                            }
                        }
                    }
                    break;

                case "lastRowSelection":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit)
                            return true;
                        if (updatedRecords.length > 0) {
                            lastRowIndex = expandedRecords.length - 1;
                            currentSelectingRecord = expandedRecords[lastRowIndex];
                            selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                            if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), selectingRowIndex)) {
                                proxy.selectRows(selectingRowIndex);
                                proxy.element[0].focus();
                                proxy.updateScrollBar();
                                proxy._rowSelectedEventTrigger(selectingRowIndex);
                            }
                        }
                    }
                    break;
                    //set the edit mode to previous cell by press shift +tab key
                case "moveCellLeft":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        //Row edit Element focusing for Shift + tab key press
                        if (proxy._isRowEdit) {
                            previousElement = $target.closest("td").prev("td");
                            if (previousElement.length)
                                proxy._focusElementsForRowEdit(previousElement);
                        } else if (model.editSettings.allowEditing && model.editSettings.editMode.toLowerCase() == "cellediting" && $form.length > 0) {
                            returnValue = proxy._moveCurrentCell("left");
                            $(proxy.element).focus();
                        }
                    }
                    break;

                    //set the edit mode to next cell by tab key
                case "moveCellRight":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit) {
                            nextElement = $target.closest("td").next("td");
                            if (nextElement.length)
                                proxy._focusElementsForRowEdit(nextElement);
                        } else if (model.editSettings.allowEditing && model.editSettings.editMode == "cellEditing" && $target && $form.length > 0) {
                            returnValue = proxy._moveCurrentCell("right");
                            $(proxy.element).focus();
                        }
                    }
                    break;

                    //select the next row by press down arrow key
                case "downArrow":
                    if (model.allowSelection) {
                        if (contextMenu.length == 0 && columnMenu.length == 0) {
                            if (proxy._isRowEdit || proxy.model.isEdit)
                                return true;

                            if (updatedRecords.length > 0 && model.selectedItem) {
                                //get the last row index of the ganttchart control
                                lastRowIndex = updatedRecords.length - 1;
                                if (proxy.selectedRowIndex() != lastRowIndex) {

                                    selectingRowIndex = expandedRecords.indexOf(model.selectedItem);
                                    currentSelectingRecord = expandedRecords[selectingRowIndex + 1];
                                        //To skip the key Navigation from summary row.
                                    if (currentSelectingRecord && currentSelectingRecord.isSummaryRow)
                                        currentSelectingRecord = proxy._getNextRecord(selectingRowIndex + 1, expandedRecords);
                                    selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);
                                    if (currentSelectingRecord && selectingRowIndex <= lastRowIndex && !proxy._rowSelectingEventTrigger(this.selectedRowIndex(), selectingRowIndex)) {
                                        proxy.selectRows(selectingRowIndex);
                                        proxy.updateScrollBar();
                                        proxy.element[0].focus();
                                        proxy._rowSelectedEventTrigger(proxy.selectedRowIndex());
                                    }
                                }
                            }
                        }
                        else {
                            if (contextMenu.length) {
                                if (isVisible) {
                                    var subContextMenuItem = subContextMenu.find("div.e-contextmenu-mouseover"),
                                       nextItem = proxy._findContextMenuItem(subContextMenuItem, "next");
                                    if (nextItem.length > 0) {
                                        subContextMenuItem.removeClass("e-contextmenu-mouseover");
                                        nextItem.addClass("e-contextmenu-mouseover");
                                    }
                                }
                                else {
                                    var contextMenuItem = contextMenu.find("div.e-contextmenu-mouseover"),
                                        nextItem = proxy._findContextMenuItem(contextMenuItem, "next");
                                    if (nextItem.length > 0) {
                                        contextMenuItem.removeClass("e-contextmenu-mouseover");
                                        nextItem.addClass("e-contextmenu-mouseover");
                                    }
                                }
                            }
                            else if (columnMenu.length) {
                                if (isVisibleColumnChooserMenu) {
                                    var columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection"),
                                       nextItem = columnChooserMenuItem.next("div.e-columnMenuListDiv");
                                    if (nextItem.length > 0) {
                                        columnChooserMenuItem.removeClass("e-columnmenuselection");
                                        nextItem.addClass("e-columnmenuselection");
                                    }
                                }
                                else {
                                    var columnMenuItem = columnMenu.find("div.e-columnmenuselection"),
                                        nextItem = columnMenuItem.closest("li").next("li").find("div.e-columnmenuitem");
                                    if (nextItem.length > 0) {
                                        columnMenuItem.removeClass("e-columnmenuselection");
                                        nextItem.addClass("e-columnmenuselection");
                                    }
                                }
                            }
                        }
					}
                    break;
                // expand the child task by press right arrow
                case "rightArrow":
					if (model.allowSelection) {
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) return true;
                        var rowIndex = proxy.selectedRowIndex(),
                        record = proxy.model.updatedRecords[rowIndex],
                        args = {},
                        isExpandCollapseEnabeled;
                        args.data = record;
                        args.recordIndex = rowIndex;
                        args.expanded = true;
                        if (record) {
                            if (record.hasChildRecords && !record.expanded) {
                                if (proxy.selectedRowIndex() >= 0) {
                                    isExpandCollapseEnabeled = proxy._trigger("expanding", args);
                                    if (!isExpandCollapseEnabeled && !model.isFromGantt) {
                                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                                    }
                                    if (model.enableVirtualization)
                                        proxy.element[0].focus();
                                    proxy.updateScrollBar();
                                }
                            }
                        }
                    }
                    else
                    {
                        if (contextMenu.length && !isVisible) {
                            var contextMenuItem = contextMenu.find("div.e-contextmenu-mouseover");
                            if (contextMenuItem.length > 0) {
                                menuId = contextMenuItem[0].id;
                                if (menuId == "Add") {
                                    var subContextMenuItems = subContextMenu.find(".e-menuitem");
                                    subContextMenu.css({ 'visibility': 'visible', 'left': proxy._subPosx });
                                    subContextMenuItems.removeClass("e-contextmenu-mouseover");
                                    subContextMenuItems.eq(0).addClass("e-contextmenu-mouseover");
                                }
                            }
                        }
                        else if (columnMenu.length && !isVisibleColumnChooserMenu) {
                            var columnMenuItem = columnMenu.find("div.e-columnmenuselection"),
                                columnChooserMenuList = columnChooserMenu.find("div.e-columnMenuListDiv")
                                columnListPosX = proxy._columnListPosX,
                                columnChooserListIndex = proxy._columnChooserListIndex,
                                windowWidth = $(document).width();
                            if (columnMenuItem.length > 0 && columnMenuItem[0].id == proxy._id + "_ColumnsChooser") {
                                var columnchooser = $("#" + proxy._id + "ccDiv").ejDialog("instance"),
                                    columnListWidth = $("#" + proxy._id + "ccDiv_wrapper").width();
                                columnchooser.open();
                                if (windowWidth < columnListPosX + columnMenu.width()) {
                                    columnListPosX = columnListPosX - columnMenu.width() - columnListWidth;
                                }
                                var position = {
                                    X: columnListPosX,
                                    Y: posY + columnChooserListIndex * 30 + (columnChooserListIndex == 0 ? 0 : 1)
                                }
                                columnchooser.option({ "position": position });
                                columnMenu.css("z-index", "10033");
                                columnChooserMenuList.removeClass("e-columnmenuselection");
                                columnChooserMenuList.eq(0).addClass("e-columnmenuselection");
                                proxy.element.focus();
                            }
                        }
                    }
					}
                    break;
				
                //collapse the child task by press left arrow 
                case "leftArrow":
					if (model.allowSelection) {
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) return true;
                        var rowIndex = proxy.selectedRowIndex(),
                        record = proxy.model.updatedRecords[rowIndex],
                        args = {},
                        isExpandCollapseEnabeled;
                        args.data = record;
                        args.recordIndex = rowIndex;
                        args.expanded = false;
                        if (record) {
                            if (record.hasChildRecords && record.expanded) {
                                if (proxy.selectedRowIndex() >= 0) {
                                    isExpandCollapseEnabeled = proxy._trigger("collapsing", args);
                                    if (!isExpandCollapseEnabeled && !model.isFromGantt) {
                                        ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                                    }
                                    if (model.enableVirtualization)
                                        proxy.element[0].focus();
                                    proxy.updateScrollBar();
                                }
                            }
                        }
                    }
                    else if (contextMenu.length && isVisible)
                        subContextMenu.css({ 'visibility': 'hidden' });
                    else if (columnMenu.length && isVisibleColumnChooserMenu)
                        $("#" + proxy._id + "ccDiv").ejDialog("close");
					}
                    break;
				
                    //select the previous row by press down arrow key
                case "upArrow":
					if (model.allowSelection) {
					    if (contextMenu.length == 0 && columnMenu.length == 0) {
					       if (proxy._isRowEdit || proxy.model.isEdit)
                                return true;

                        if (this.selectedRowIndex() !== 0 && updatedRecords.length > 0 && model.selectedItem) {

                            selectingRowIndex = expandedRecords.indexOf(model.selectedItem);
                            currentSelectingRecord = expandedRecords[selectingRowIndex - 1];
                                if (currentSelectingRecord.isSummaryRow)
                                    currentSelectingRecord = proxy._getNextRecord(selectingRowIndex - 1, expandedRecords, action);
                                selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);

                            if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), selectingRowIndex)) {
                                proxy.selectRows(selectingRowIndex);
                                proxy.updateScrollBar();
                                proxy.element[0].focus();
                                proxy._rowSelectedEventTrigger(proxy.selectedRowIndex());
                            }
                        }
                    }
                    else
                    {
                        if (contextMenu.length) {
                            if (isVisible) {
                                var subContextMenuItem = subContextMenu.find("div.e-contextmenu-mouseover"),
                                   prevItem = proxy._findContextMenuItem(subContextMenuItem, "prev");
                                if (prevItem.length > 0) {
                                    subContextMenuItem.removeClass("e-contextmenu-mouseover");
                                    prevItem.addClass("e-contextmenu-mouseover");
                                }
                            }
                            else {
                                var contextMenuItem = contextMenu.find("div.e-contextmenu-mouseover"),
                                    prevItem = proxy._findContextMenuItem(contextMenuItem, "prev");
                                if (prevItem.length > 0) {
                                    contextMenuItem.removeClass("e-contextmenu-mouseover");
                                    prevItem.addClass("e-contextmenu-mouseover");
                                }
                            }
                        }
                        else if (columnMenu.length) {
                            if (isVisibleColumnChooserMenu) {
                                var columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection"),
                                  prevItem = columnChooserMenuItem.prev("div.e-columnMenuListDiv");
                                if (prevItem.length > 0) {
                                    columnChooserMenuItem.removeClass("e-columnmenuselection");
                                    prevItem.addClass("e-columnmenuselection");
                                }
                            }
                            else {
                                var columnMenuItem = columnMenu.find("div.e-columnmenuselection"),
                                    prevItem = columnMenuItem.closest("li").prev("li").find("div.e-columnmenuitem");
                                if (prevItem.length > 0) {
                                    columnMenuItem.removeClass("e-columnmenuselection");
                                    prevItem.addClass("e-columnmenuselection");
                                }
                            }
                        }
                    }
					}
                    break;
                case "deleteRecord":
                    if (model.editSettings.allowDeleting && contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy._isRowEdit || proxy.model.isEdit) return true;
                        if (proxy.selectedRowIndex() >= 0) {
                            proxy.deleteRow();
                            proxy._cancelSaveTools();
                            proxy.element[0].focus();
                        }
                    }
                    break;
                case "totalRowCollapse":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy.model.isFromGantt) {
                            args = {};
                            args.requestType = "collapseAll";
                            proxy._trigger("expandAllCollapseAllRequest", args);
                        }
                        else
                            proxy.collapseAll();
                        break;
                    }
                case "totalRowExpand":
                    if (contextMenu.length == 0 && columnMenu.length == 0) {
                        if (proxy.model.isFromGantt) {
                            args = {};
                            args.requestType = "expandAll";
                            proxy._trigger("expandAllCollapseAllRequest", args);
                        }
                        else
                            proxy._expandAll();
                    }
                    break;
                case "spaceBar":
                var option = null;
                if (columnMenu.length && isVisibleColumnChooserMenu) {
                    var columnChooserMenuItem = columnChooserMenu.find("div.e-columnmenuselection");
                    option = columnChooserMenuItem.data("column");
                    if (!ej.isNullOrUndefined(option)) {
                        var isColumnVisible = columnChooserMenuItem.find("span").attr("aria-checked");
                        isColumnVisible == "true" ? proxy.hideColumn(option) : proxy.showColumn(option);
                    }
                }
                break;
                default:
                    returnValue = true;
            }

            return returnValue;
        },

        //Method to bind all the filter condition and splice the over writed condition.
        filterColumn: function (fieldName, filterOperator, filterValue, predicate, matchcase, actualFilterValue) {
           
            var proxy = this,
                model = proxy.model,
                filteredColumns = model.filterSettings.filteredColumns;
            if (!this.model.allowFiltering)
                return;
            var args = {};
            ej.TreeGrid.Actions.Filter = "filtering";
            args.requestType = ej.TreeGrid.Actions.Filter;
            args.currentFilterObject = [];           
            if (!$.isArray(filterOperator))
                filterOperator = $.makeArray(filterOperator);
            if (!$.isArray(filterValue))
                filterValue = $.makeArray(filterValue);
            var firstLoop = false;
            var filterCol = this._filterCollection;
            if (ej.util.isNullOrUndefined(this._currentFilterColumn))
                this._currentFilterColumn = this.getColumnByField(fieldName);
            for (var index = 0; index < filterOperator.length; index++) {
                    var filterObject = {
                        field: fieldName,
                        operator: filterOperator[index],
                        value: filterValue[index],
                        matchcase: matchcase,
                        predicate: predicate,
                        actualFilterValue: actualFilterValue
                    };
                this._$colType = "string";
                if (filteredColumns.length == 0 && filterObject.value !== "") {
                    if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                        this._setDateFilters(filterObject);
                    else
                        filteredColumns.push(filterObject);
                } else {
                    var proxy = this;
                    if (!firstLoop) {
                        var dataManger = ej.DataManager(filteredColumns);
                        var query = new ej.Query().where("field", ej.FilterOperators.equal, filterObject.field);
                        var object = dataManger.executeLocal(query);
                        for (var i = 0; i < object.length; i++) {
                            var objectIndex = $.inArray(object[i], filteredColumns)
                            if (objectIndex != -1)
                                filteredColumns.splice(objectIndex, 1);
                        }
                    }
                    if (filterObject.value !== "") {
                        if (this._$colType == "date" && (filterOperator == "equal" || filterOperator == "notequal") && typeof (filterObject.value) !== "string")
                            this._setDateFilters(filterObject);
                        else
                            filteredColumns.push(filterObject);
                    }
                }
                firstLoop = true;
                args.currentFilterObject.push(filterObject);
            }
            args.filterCollection = filteredColumns;
            args.currentFilteringColumn = fieldName;
            args.fieldName = fieldName;
            var returnValue = this.processBindings(args);
            if (returnValue) {
                filteredColumns.reverse().splice(0, filterOperator.length);
                filteredColumns.reverse();
            }
        },
        //ROWSELECTING EVENT METHOD
        _rowSelectingEventTrigger: function (previousIndex, currentIndex) {

            var proxy = this,
                model = proxy.model,
                args = {},
                selectedItem,
                $gridRows;
                args.targetRow = ej.TreeGrid.getRowByIndex(proxy, currentIndex);
                args.previousTreeGridRow = ej.TreeGrid.getRowByIndex(proxy, previousIndex);
                args.data = model.updatedRecords[currentIndex];
                args.previousData = model.updatedRecords[previousIndex];
                args.recordIndex = currentIndex;
                args.previousIndex = previousIndex;
                return proxy._trigger('rowSelecting', args);
        },


        //ROWSELECTED EVENT METHOD
        _rowSelectedEventTrigger: function (index) {

            var proxy = this, model = proxy.model,
                args = {};
            var record = $.extend({}, model.selectedItem);
            /*Remove childRecords and parentItem from record object, 
              To avoid the cyclic dependency when performing postback action in ASP.NET */
            delete record.childRecords;
            delete record.parentItem;

            args = {
                targetRow: ej.TreeGrid.getRowByIndex(proxy, index),
                recordIndex: index
            };
            if (proxy.model.isFromGantt) {
                args.target = "ejTreeGrid";
                args.data = model.selectedItem;
            }
            else
                args.data = record;

                proxy._trigger("rowSelected", args);
        },


        //QUERYCELLINFO,ROWDATABOUND EVENT BINDING FOR TREEGRID
        _eventBindings: function () {

            var proxy = this,
                model = proxy.model,
                count = 0,
                gridRows = proxy.getRows(),
                length = gridRows.length;

            if (model.queryCellInfo != null || model.rowDataBound != null || model.detailsDataBound) {

                for (count; count < length; count++) {
                    proxy._rowEventTrigger(gridRows[count], model.currentViewData[count]);
                }
            }
        },

        //ROWDATABOUND EVENT TRIGGER
        _rowEventTrigger: function (row, data) {

            var proxy = this,
                model = this.model;

            if (model.rowDataBound) {
                proxy._trigger("rowDataBound", {
                    rowElement: row,
                    data: data
                });
            }
           
            var tdCells = row.cells,
                $tdRowcells = $(row).find(".e-rowcell"),
                cellIndex = 0,
                length = tdCells.length,
                column=null,
                columns = proxy.model.columns;
            if (model.queryCellInfo) {
                for (cellIndex; cellIndex < length; cellIndex++) {

                    if ($(tdCells[cellIndex]).hasClass("e-rowcell")) {

                        column = columns[$tdRowcells.index(tdCells[cellIndex])];

                    }

                    if (column) {

                        proxy._cellEventTrigger(tdCells[cellIndex], data, column);

                    }
                }
            }

            if (model.detailsDataBound && $(row).next("tr").hasClass("e-detailsrow")) {
                this._trigger("detailsDataBound", { detailsElement: $(row).next("tr"), data: data });
            }
        },


        //QUERYCELLINFO EVENT TRIGGER
        _cellEventTrigger: function (cell, data, column) {

            var args = {
                cellElement: cell,
                data: data,
                column: column,

            };

            args.cellValue = args.column && data[args.column.field];
            this._trigger("queryCellInfo", args);
        },


        //ACTION COMPLETE EVENT TRIGGER
        _completeAction: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns;

            model.isEdit = false;

            if ((ej.TreeGrid.Actions.Sorting == args.requestType && model.allowSorting)
                || (ej.TreeGrid.Actions.Refresh == args.requestType)) {

                if (model.allowSorting) {

                    if (!proxy._multiSortRequest) {
                        proxy.getHeaderTable().find(".e-columnheader").
                            find(".e-headercelldiv").find(".e-ascending,.e-descending").remove();

                        proxy.getHeaderTable().find("[aria-sort]").removeAttr("aria-sort");
                    }

                    for (var i = 0; i < sortcolumns.length; i++) {
                        proxy._addSortElementToColumn(sortcolumns[i].field, sortcolumns[i].direction);
                    }

                }

                proxy._multiSortRequest = false;
            } else if (ej.TreeGrid.Actions.BeginEdit == args.requestType || ej.TreeGrid.Actions.Add == args.requestType) {

                var $form = $("#" + proxy._id + "EditForm");

                proxy.model.isEdit = true;
                proxy.setWidthToColumns();

                if (ej.TreeGrid.Actions.Add == args.requestType) {

                    $form.find(".e-field:disabled").not(".e-identity").removeAttr("disabled").removeClass("e-disable");

                }

            }
            else if (args.requestType === "scroll")
            {
                proxy._updateCurrentViewData();
            }
            proxy._trigger("actionComplete", args);
        },


        //#endregion

        //#region METHOD FOR DATA PROCESSING

        //RENDERING THE DATA
        sendDataRenderingRequest: function (args) {

            var proxy = this, model = this.model,
                temp;
            if (proxy.model.currentViewData.length) {

                switch (args.requestType) {

                    case ej.TreeGrid.Actions.Delete:
                    case ej.TreeGrid.Actions.Refresh:
                    case ej.TreeGrid.Actions.Save:
                    case ej.TreeGrid.Actions.Filter:
                    case ej.TreeGrid.Actions.Sorting:
                    case ej.TreeGrid.Actions.Searching:
                    case ej.TreeGrid.Actions.ExpandCollapse:
                    case ej.TreeGrid.Actions.RefreshDataSource:
                    case ej.TreeGrid.Actions.DragAndDrop:
                        temp = document.createElement('div');
                        proxy.getContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                        var $tbody = proxy.getContentTable().children('tbody');
                        $tbody.empty();                        
                        temp.innerHTML = ['<table>', $.render[proxy._id + "_Template"](proxy.model.currentViewData),
                            '</table>'].join("");
                        proxy.getContentTable().get(0).replaceChild(temp.firstChild.firstChild,
                            proxy.getContentTable().get(0).lastChild);
                        proxy._gridRows = proxy.getContentTable().get(0).rows;
                        proxy.setGridRows($(proxy.getContentTable().get(0).rows));

                        /* check treee grid rows are append in DOM and Trigger refresh and querycellInfo events */
                        if (proxy._$gridContent && proxy._isRendered){
                            proxy._trigger("refresh");
                            proxy._eventBindings();
                            this._hideCollapsedDetailsRows();
                        }
                        break;
                }

            } else {

                proxy.getContentTable().find('tbody').empty().append(proxy._getEmptyTbody());
                proxy._$gridContainer && proxy._$gridContainer.css({ height: "30px" }); // for render no record text
                proxy._gridRows = null;
                proxy.setGridRows(null);

            }

            if (proxy.model.showGridCellTooltip || proxy.model.showGridExpandCellTooltip)
                proxy._cellMouseLeave();

            proxy._completeAction(args);

        },
        //Public method to change the allowSorting value dynamically.
        sortSetting: function (value) {
            var proxy = this;
                ascendingIcon = $("#" + proxy._id).find(".e-ascending.e-icon"),
                descendingIcon = $("#" + proxy._id).find(".e-descending.e-icon");
                proxy.model.allowSorting = value;               
                if (ascendingIcon.length > 0) {
                    for (var length = 0 ; length < ascendingIcon.length; length++) {
                        $(ascendingIcon[length]).css("visibility", "hidden");
                    }
                }                                      
            if (descendingIcon.length > 0)
                for (var length = 0 ; length < descendingIcon.length; length++) {
                    $(descendingIcon[length]).css("visibility", "hidden");
                }
        },
        //Public method to change the treeColumnIndex value dynamically.
        columnIndex:function(value){
            var proxy = this, model = proxy.model;
            if (!isNaN(parseInt(value)) && parseInt(value) >= 0 && parseInt(value) < model.columns.length) {
                model.treeColumnIndex = parseInt(value);
                proxy._addInitTemplate();
                this.renderRecords();
                if (model.showColumnChooser)
                    proxy._renderColumnChooserList(true);
            }
        },


        //SORTING THE DATA BASED ON THE PARTICULAR FIELDS
        
        sortColumn: function (columnName, columnSortDirection) {

            var proxy = this,
                model = proxy.model,
                sortColumns = model.sortSettings.sortedColumns,
                args = {},
                count = 0,
                length = sortColumns.length;

            if (!model.allowSorting || $.inArray(columnName, proxy._disabledSortableColumns) != -1
                || columnName.length == 0) {
                return;
            }

            if (!proxy._multiSortRequest) {
                model.sortSettings.sortedColumns = [];
            }

            args.requestType = ej.TreeGrid.Actions.Sorting;
            args.columnName = columnName;

            proxy._cSortedColumn = proxy.getFieldNameByHeaderText(columnName);
            proxy._cSortedDirection = args.columnSortDirection = columnSortDirection === undefined ?
                ej.sortOrder.Ascending : columnSortDirection;

            for (count; count < length; count++) {
                
                if (sortColumns[count].field == proxy._cSortedColumn) {
                    sortColumns.splice(count, 1);
                    break;
                }

            }

            model.sortSettings.sortedColumns.push({
                field: proxy._cSortedColumn,
                direction: proxy._cSortedDirection
            });

            var returnValue = proxy.processBindings(args);

            if (returnValue) {

                proxy._cSortedDirection = proxy._cSortedColumn = null;

            }
            if (proxy.selectedRowIndex() >= 0 && proxy._isRendered) {
                var previousIndex = proxy._selectedRowsIndexes.indexOf(proxy.selectedRowIndex());
                proxy.selectedRowIndex(proxy._sortedRecords.indexOf(this.selectedItem()));
                proxy._previousIndex = proxy.selectedRowIndex();
                proxy._selectedRowsIndexes[previousIndex] = proxy._previousIndex;
            }
            proxy._clearColumnMenu();
        },


        //DELETE THE PARTICULAR RECORD AND MULTIPLE RECORD
        deleteRow: function ($tr) {

            var proxy = this,
                deletedRowIndex,
                deletedRow,
                args = {},
                eveArgs = {},
            length = proxy._selectedRowsIndexes.length;

            if (!this.model.editSettings.allowDeleting)
                return false;

            proxy._selectedRowsIndexes.sort();
            var deletedrowCount = 0;
            for (var del = 0; del < length; del++) {

                if (ej.isNullOrUndefined($tr))
                    $tr = ej.TreeGrid.getRowByIndex(proxy, proxy._selectedRowsIndexes[del]);
                else
                    $tr = ej.TreeGrid.getRowByIndex(proxy, proxy._selectedRowsIndexes[del] - deletedrowCount);

                deletedRowIndex = proxy.getRows().index($tr);
                if (deletedRowIndex > -1)
                    deletedRow = proxy.model.currentViewData[deletedRowIndex];
                else{
                    if (ej.isNullOrUndefined($tr))
                        deletedRow = proxy.model.updatedRecords[proxy._selectedRowsIndexes[del]];
                    else
                        deletedRow = proxy.model.updatedRecords[proxy._selectedRowsIndexes[del] - deletedrowCount];
                }

            args.tr = $tr;
            args.data = deletedRow;
            args.requestType = ej.TreeGrid.Actions.Delete;   

            if (proxy._trigger("actionBegin", args)) {
                return true;
            }

            proxy.processBindings(args);
            deletedrowCount++;
            if (proxy.model.currentViewData.length > 0 && proxy.model.enableAltRow)
                ej.TreeGrid.updateAltRow(proxy, proxy.model.currentViewData[0], 0, 0);
            }
            proxy._selectedRowsIndexes = [];
            proxy._previousIndex = -1;
        },


        //SEARCH THE TASK FROM TREE GRID CONTROL

        

        search: function (searchString) {

            var proxy = this,
                args = {};

            args.requestType = ej.TreeGrid.Actions.Searching;
            args.keyValue = searchString;

            proxy._searchString = searchString;

            proxy.processBindings(args);
        },

        //SEND SAVE REQUEST
        endEdit: function () {

            var proxy = this;

            if (proxy.model.editSettings.editMode.toLowerCase() == "cellediting" && !proxy._isRowEdit)
                return proxy.saveCell();

            return true;

        },
        updateShowGridExpandCellTooltip: function (bool) {
            var proxy = this;
            proxy.model.showGridExpandCellTooltip = bool;
            proxy.refreshContent();
        },
        updateShowGridCellTooltip: function (bool) {
            var proxy = this;
            proxy.model.showGridCellTooltip = bool;
            proxy.refreshContent();
        },

        //#endregion

        //#region PUBLIC METHODS

        //CALCULATE THE VISIBLE RANGE OF RECORDS TO DISPLAYED IN CURRENT VIEWPORT
        getVisibleRange: function () {

            var proxy = this,
                model = proxy.model,
                calcHeight = (proxy.model.rowHeight + proxy._detailsRowHeight),
                top = proxy._scrollTop / calcHeight,
                coeff = top - Math.floor(top),
                topIndex, bottomIndex;

                proxy._offset = coeff * calcHeight;
                topIndex = Math.floor(top);
                bottomIndex = proxy._getRowPosition(proxy._scrollTop + proxy._viewPortHeight);
                topIndex = Math.max(0, topIndex);
                bottomIndex = Math.min(proxy.model.updatedRecords.length, bottomIndex);

                proxy._visibleRange = {
                    top: topIndex,
                    bottom: bottomIndex
                };

            return proxy._visibleRange;

        },

        //SET THE COLUMNWIDTH TO EACH COLUMN IN TREEGRID
        setWidthToColumns: function () {

            var proxy = this,
                model = proxy.model,
                $cols1 = proxy.getContentTable().children("colgroup").find("col"),
                $cols2 = proxy.getHeaderTable().children("colgroup").find("col"),
                width = proxy._gridWidth,
                colCount = model.columns.length;

            for (var i = 0; i < $cols2.length; i++) {

                if (!ej.isNullOrUndefined(proxy._columnsWidthCollection[i])) {

                    $cols1.eq(i).width(proxy._columnsWidthCollection[i]);
                    $cols2.eq(i).width(proxy._columnsWidthCollection[i]);

                } else {
                    var colWidth = (width / colCount).toFixed(2) + "px";
                    $cols1.eq(i).css("width", colWidth);
                    $cols2.eq(i).css("width", colWidth);
                    proxy._columnsWidthCollection[i] = (width / colCount).toFixed(2);
                }
            }
            /* width fro details col group cell */
            if ($cols1.length > colCount) {
                $cols1.eq(colCount).width(35);
            }

            if ($cols2.length > colCount) {
                $cols2.eq(colCount).width(35);
            }
        },

        //ENTER EDIT MODE INTO THE TREEGRID CELL
        cellEdit: function (index, fieldName) {

            var proxy = this,
                $form,
                model = proxy.model;

            if (proxy.model.showGridCellTooltip) {
                proxy._cellMouseLeave();
            }

            proxy.model.isEdit && proxy.saveCell();

            $form = $("#" + proxy._id + "EditForm");

            if ($form.length > 0) return true;

            if (!proxy.model.selectedItem) return false;

            if (model.isFromGantt && ((fieldName === "status" ||
                fieldName === "startDate" ||
                fieldName === "endDate" ||
                fieldName === "duration" ||
                fieldName === "predecessor") &&
                proxy.model.selectedItem.hasChildRecords) ||
                (fieldName === "baselineStartDate" ||
                    fieldName === "baselineEndDate") || (fieldName==="taskId")) {

                return false;

            }
            if (model.enableVirtualization)
                index = model.currentViewData.indexOf(model.selectedItem);
            if (proxy.getRows()[index]) {
                var $targetTr = $(proxy.getRows()[index]);
                columnIndex = proxy.getColumnIndexByField(fieldName),
                $targetTd = $targetTr.find(".e-rowcell").eq(columnIndex),
                column = proxy.model.columns[columnIndex],
                data = proxy.model.selectedItem,
                beginEditArgs = {},
                args = {
                    validationRules: ej.isNullOrUndefined(column.validationRules) ? {} : $.extend(true, {},
                        column.validationRules),
                    columnName: column.field,
                    value: !ej.isNullOrUndefined(data[fieldName]) ? data[fieldName] : data.item && data.item[fieldName],
                    data: data,
                    columnObject: column,
                    cell: $targetTd
                },
                isEditable = true;

            if (model.isFromGantt && args.columnName === "predecessor") {

                var predecessor = args.data.item[proxy.model.predecessorMapping];
                args.value = predecessor ? predecessor : "";

            }

            beginEditArgs.data = data;
            beginEditArgs.columnIndex = columnIndex;
            beginEditArgs.rowElement = $targetTr;
            beginEditArgs.cellElement = $targetTd;

          

                proxy._cellEventTrigger($targetTr[0].cells[proxy._cellEditingDetails.columnIndex], data, args.columnObject);

                if (proxy._trigger("beginEdit", beginEditArgs)) return false;               

                if ($targetTr.hasClass("e-insertedrow") && (!args.columnObject.isTemplateColumn)) isEditable = true;

                if (isEditable) {

                    $.extend(proxy._cellEditingDetails, {
                        rowIndex: index,
                        cellValue: args.value,
                        columnIndex: columnIndex,
                        fieldName: fieldName,
                        cellEditType: args.columnObject.editType,
                        data: data
                    });
                }

                proxy._renderCellEditObject(args, $targetTd);
                if (!$targetTr.hasClass("e-templaterow")) {
                    $targetTr.addClass("e-editedrow");
                }

                args.cell && args.cell.addClass("e-editedcell");

                if (proxy._cellEditingDetails && proxy._cellEditingDetails.cellEditType == "dropdownedit")
                    proxy.element[0].focus(); //To block keypress (enter & spacebar) event on dropdownlist
                 proxy._updateHorizontalScrollBar(fieldName);
                return true;
            }
        },

        _updateHorizontalScrollBar:function(fieldName)
        {
            var proxy = this,
                gridContent = proxy._$gridContent,
                isHorizontalScroll = gridContent.ejScroller("isHScroll");
            //Check if the horizontal scrollbar is exists or not
            if (isHorizontalScroll) {
                var model = proxy.model,
                columns = model.columns,
                columnLength = columns.length,
                columnIndex = proxy.getColumnIndexByField(fieldName),
                form = $("#" + proxy._id + "EditForm"),
                row = $(form).closest("tr.e-treegridrows"),
                cells = $(row).find("td.e-rowcell"),
                width = 0, startWidth = 0,
                viewPortWidth = gridContent.find(".e-content").width(),
                scrollLeft = gridContent.ejScroller("option", "scrollLeft");
                //Calculate the start and end width for edited cell
                for (var index = 0; index <= columnIndex; index++) {
                    if (index == columnIndex)
                        startWidth = width;
                    if (columns[index].visible) {
                        var cell = $(cells[index]);
                        width = width + cell.width() + parseFloat(cell.css("padding-left")) + parseFloat(cell.css("padding-right")) + 1; // 1 for border
                    }
                }
                // Check if the edited cell in viewport.
                if (startWidth < scrollLeft || width > scrollLeft + viewPortWidth) {
                    var left = width - viewPortWidth + 3;// 3 for slightly adjust the left scrollbar
                    gridContent.ejScroller("option", { "scrollLeft": left });
                    proxy._$gridHeaderContainer.scrollLeft(left);
                }
            }
        },
        updateAllowColumnResize: function (bool) {
            var proxy = this,
                model = proxy.model;
            model.allowColumnResize = bool;
            if (model.allowColumnResize)
                proxy._resizer = new ej.gridFeatures.gridResize(proxy);
            proxy._enableColumnResizeEvents();
        },

        //validate predecessor string

        _validatePredecessorString: function (string) {
            var proxy = this, result;
            if (string.length > 0) {
                string.split(',').forEach(function (el) {
                    values = el.split('+');
                    offsetvalue = '+';
                    if (el.indexOf('-') >= 0) {
                        values = el.split('-');
                        offsetvalue = '-';
                    }
                    match = values[0].match(/(\d+|[A-z]+)/g);
                    if (proxy.model.ids.indexOf(match[0]) != -1) {
                        if (match[1]) {
                            if (match[1].toUpperCase() == 'FS' || match[1].toUpperCase() == 'FF' || match[1].toUpperCase() == 'SF' || match[1].toUpperCase() == 'SS') {
                            }
                            else {
                                result = false;
                            }
                        }
                    } else {
                        result = false;
                    }
                });
            } else {
                return true;
            }
            if (result === undefined || result)
                return true;
            else
                return false;
        },

        //SAVE THE EDITED CELL
        
        saveCell: function () {

            var proxy = this,
                model = proxy.model;

            if ($("#" + proxy._id + "EditForm").length > 0) {

                if (!proxy.editFormValidate()) return true;

                if (proxy._cellEditingDetails.columnIndex >= 0) {
                    proxy.element[0].focus();

                    var $form = $("#" + proxy._id + "EditForm"),
                        $targetTr = $form.closest("tr"),
                        $targetTd = $form.closest("td"),
                        formattedValue, args = {},
                        column = model.columns[proxy._cellEditingDetails.columnIndex],
                        $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName),
                        cellElement;

                    args = {
                        columnName: column.field,
                        value: proxy.getCurrentEditCellData(),
                        data: proxy._cellEditingDetails.data,
                        previousValue: proxy._cellEditingDetails.cellValue,
                        columnObject: column,
                        cellElement: $targetTd,
                        rowElement: $targetTr,
                    };

                    if (proxy._cellEditingDetails.cellEditType == "datetimepicker" ||
                        proxy._cellEditingDetails.cellEditType == "dropdownedit" ||
                        proxy._cellEditingDetails.cellEditType == "datepicker") {

                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");                        

                    }

                    if (!ej.isNullOrUndefined(column.format)) {

                        formattedValue = proxy.formatting(column.format, args.value , model.locale);
                        args.cellElement.empty().html(formattedValue);

                    } else {

                        if (args.columnObject.type == "boolean") {

                            var cellData = {};
                            cellData[args.columnObject.field] = args.value;
                            args.cellElement.empty().html(
                                $($.templates.Grid_JSONTemplate.render(cellData))[0].cells[proxy._cellEditingDetails.columnIndex].innerHTML
                            );

                        } else args.cellElement.empty().html(args.value);
                    }

                    var isValueModified = ((proxy._cellEditingDetails.cellEditType == "datepicker" ||
                            proxy._cellEditingDetails.cellEditType == "datetimepicker") &&
                        args.value instanceof Date && args.previousValue instanceof Date) ?
                        (args.value.getTime() !== args.previousValue.getTime()) :
                        (args.value !== args.previousValue);

                    model.isEdit = false;

                    var fieldName = proxy._cellEditingDetails["fieldName"];

                    if (model.isFromGantt && fieldName === "predecessor") {

                        var eventArg = {},stringResult,logicResult;
                        stringResult = proxy._validatePredecessorString(args.value);
                        if (stringResult) {
                            eventArg.predecessorString = args.value.split(',');
                            eventArg.currentRecord = args.data;
                            eventArg.requestType = "validatePredecessor";
                            proxy._trigger("actionComplete", eventArg);
                            logicResult = eventArg.result;
                            if (!logicResult) {
                                args.value = args.previousValue;
                            }
                        }
                        else {
                            args.value = args.previousValue;
                        }
                        args.value = args.value.toUpperCase();
                        var ganttPredecessor = [],
                        prevPredecessors = [],
                        modifiedPredecessors = [],
                        count = 0,
                        length, predecessor;
                        args.previousValue = args.data.predecessor;
                        prevPredecessors = args.previousValue;
                        if (args.value.length > 0) {
                            modifiedPredecessors = args.data._calculatePredecessor(args.value);
                        }
                        length = modifiedPredecessors.length;
                        var index = 0,
                            prevPrdecessorlength = args.previousValue && args.previousValue.length;
                        for (index = 0; index < prevPrdecessorlength; index++) {
                            predecessor = prevPredecessors[index];
                            if (predecessor.from === args.data.taskId.toString()) {
                                ganttPredecessor.push(predecessor);
                            }
                        }
                        for (count = 0; count < length; count++) {
                            ganttPredecessor.push(modifiedPredecessors[count]);
                        }
                        args.data[proxy._cellEditingDetails["fieldName"]] = ganttPredecessor;
                        args.data.item[model.predecessorMapping] = args.value;
                        args.data.predecessorsName = args.value;

                    } else if (isValueModified) {
                        if (model.isFromGantt) {

                            switch (fieldName) {

                                case "taskId":
                                    args.data.item[model.taskIdMapping] = args.value;
                                    break;
                                case "taskName":
                                    args.data.item[model.taskNameMapping] = args.value;
                                    break;
                                case "startDate":
                                    args.data.item[model.startDateMapping] = args.value;
                                    break;
                                case "endDate":
                                    args.data.item[model.endDateMapping] = args.value;
                                    break;
                                case "resourceInfo":
                                    args.data.item[model.resourceInfoMapping] = proxy._getResourceId( args.value);                                    
                                    break;
                                case "duration":
                                    args.data.item[model.durationMapping] = args.value;
                                    break;
                                case "status":
                                    args.data.item[model.progressMapping] = args.value;
                                    break;
                                default:
                                    args.data.item[proxy._cellEditingDetails.fieldName] = args.value;
                                    break;
                            }

                        } else {
                            args.data.item[proxy._cellEditingDetails.fieldName] = args.value;
                        }
                        
                        args.data[proxy._cellEditingDetails.fieldName] = args.value;
                        
                    }

                    if (model.isFromGantt && (args.columnName === "startDate" ||
                        args.columnName === "endDate" ||
                        args.columnName === "duration" ||
                        args.columnName === "predecessor" ||
                        args.columnName == "status")) {

                        args.isModified = true;

                    }
                    
                    cellElement = $($.render[proxy._id + "_Template"](args.data))[0].cells[proxy._cellEditingDetails.columnIndex];
                    args.cellElement.removeClass("e-editedcell").replaceWith(cellElement);
                    /* refresh details Row */
                    if (model.showDetailsRow && !model.showDetailsRowInfoColumn && model.detailsTemplate) {
                        this._refreshDetailsRow(model.currentViewData.indexOf(args.data));
                    }

                    proxy._trigger("refresh");
                    if (!args.data.treeMappingName)
                        args.data.treeMappingName = [];
                    args.data.treeMappingName.push(args.columnName);
                    if (proxy._trigger("endEdit", args)) {
                        args.data[args.columnName] = args.previousValue;
                        args.data.item[args.columnObject.field] = args.previousValue;
                        proxy.selectRows(this.selectedRowIndex());
                        ej.TreeGrid.refreshRow(proxy, this.selectedRowIndex());
                    }
                    
                    proxy._cellEventTrigger(args.rowElement[0].cells[proxy._cellEditingDetails.columnIndex], args.data, args.columnObject);
                    $targetTr.removeClass("e-editedrow");                    
                }
                if (model.showSummaryRow) {
                    args.editType = "celledit";
                    proxy._updateSummaryRow(args);
                }
                if (model.showTotalSummary) {
                    args.editType = "celledit";
                    proxy._updateTotalSummaryRow(args);
                }
                proxy._cancelSaveTools();
                var colObject = args.columnObject;
                if (model.allowFiltering && colObject && colObject.filterEditType == "dropdownedit" && colObject.editType != "dropdownedit")
                    proxy._resizeFilteringElements();
            }
            return false;
        },

        //VALIDATE THE MODIFIED CELLVALUE
        editFormValidate: function () {

            if ($.isFunction($.validator))
                return $("#" + this._id + "EditForm").validate({
                    onsubmit: false
                }).form();

            return true;
        },


        //GET THE CURRENT CELL DATA
        getCurrentEditCellData: function () {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                column, cellDate;

            if ($("#" + proxy._id + "EditForm").length) {

                var $element = $("#" + proxy._id + proxy._cellEditingDetails.fieldName),
                    cellValue;
                column = columns[proxy._cellEditingDetails.columnIndex];
                switch (proxy._cellEditingDetails.cellEditType) {

                    case ej.TreeGrid.EditingType.String:
                        cellValue = $element.val();
                        break;

                    case ej.TreeGrid.EditingType.Numeric:
                        cellValue = parseFloat($element.val());
                        if (model.isFromGantt && (column.field == "status" || column.field == "duration")) {
                            cellValue = isNaN(cellValue) ? proxy._cellEditingDetails.cellValue : cellValue;
                        } else {
                            cellValue = isNaN(cellValue) ? "" : cellValue;
                        }
                        break;

                    case ej.TreeGrid.EditingType.Maskedit:
                        cellValue = $element.ejMaskEdit("model.value");
                        break;
                    case ej.TreeGrid.EditingType.Dropdown:
                        if (model.isFromGantt && proxy._cellEditingDetails.fieldName  === "resourceInfo") {
                            cellValue = proxy._getSelectedItem($element.ejDropDownList("model.selectedItems"));
                        } else
                            cellValue = $element.ejDropDownList("model.value");
                        break;

                    case ej.TreeGrid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:
                        if (column && !column.format) {
                            cellValue = Globalize.format($element.ejDatePicker("model.value"), this.model.dateFormat)
                        }
                        else {
                            cellValue = $element.ejDatePicker("model.value");
                        }
                        if (cellValue == "" && model.isFromGantt)
                            cellValue = proxy._cellEditingDetails.cellValue;
                        break;

                    case ej.TreeGrid.EditingType.DateTimePicker:

                        var dateTimePickerValue = $element.ejDateTimePicker("model.value"),
                            dateTimePickerFormat = $element.ejDateTimePicker("model.dateTimeFormat"),
                            tempValue;

                        if (column && !column.format) {
                            cellValue = Globalize.format($element.ejDateTimePicker("model.value"), this.model.dateFormat)
                        }
                        else {
                            cellValue = $element.ejDateTimePicker("model.value");
                        }
                        cellDate = Globalize.format($element.val(), dateTimePickerFormat);
                        tempValue = Globalize.format(dateTimePickerValue, dateTimePickerFormat);
                        if (cellDate !== tempValue && model.isFromGantt)
                            cellValue = proxy._cellEditingDetails.cellValue;
                        break;

                }
                if (typeof cellValue == "string" && cellValue.length &&
                    model.columns[proxy._cellEditingDetails.columnIndex].type == "number") {

                    cellValue = parseFloat(cellValue);

                }

                return cellValue;
            }

            return null;
        },
        //Get the value from edited row

        getCurrentEditCellDataForRowEdit: function (field,type,cellIndex) {

            var proxy = this,
                model = proxy.model,
                columns = model.columns,
                column;

            if ($("#" + proxy._id + "EditForm").length) {

                var $element = $("#" + proxy._id + field),
                    cellValue, dateValue;
                column = columns[cellIndex];

                switch (type) {

                    case ej.TreeGrid.EditingType.String:
                        cellValue = $element.val();
                        break;

                    case ej.TreeGrid.EditingType.Numeric:
                        cellValue = parseFloat($element.val());
                        cellValue = isNaN(cellValue) ? "" : cellValue;
                        break;

                    case ej.TreeGrid.EditingType.Dropdown:
                        cellValue = $element.ejDropDownList("model.value");
                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        break;

                    case ej.TreeGrid.EditingType.Boolean:
                        cellValue = $element.is(':checked');
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:

                        if (model.dateFormat.indexOf("hh") == -1)
                            dateValue = $element.ejDatePicker("model.value");
                        else {
                            dateValue = $element.ejDateTimePicker("model.value");
                        }

                        if (column && !column.format) {
                            cellValue = proxy.getFormatedDate(dateValue);
                        }
                        else {
                            cellValue = dateValue;
                        }
                       
                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        break;

                    case ej.TreeGrid.EditingType.DateTimePicker:

                        if (column && !column.format) {
                            cellValue = Globalize.format($element.ejDateTimePicker("model.value"), this.model.dateFormat)
                        }
                        else {
                            cellValue = $element.ejDateTimePicker("model.value");
                        }
                        $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
                        break;

                    case ej.TreeGrid.EditingType.Maskedit:
                        cellValue = $element.ejMaskEdit("model.value");
                        break;

                }
                if (typeof cellValue == "string" && cellValue.length &&
                    model.columns[cellIndex].type == "number") {
                    cellValue = parseFloat(cellValue);
                }

                return cellValue;
            }

            return null;
        },

        _getResourceId:function(dataSource){

            var resourceIds = [],
                count = 0, length = dataSource.length;

            for (count = 0; count < length; count++) {
                resourceIds.push(dataSource[count][this.model.resourceIdMapping]);
            }
            return resourceIds;
        },

        _getSelectedItem: function (indexArray) {

            var proxy = this,
                count = 0,
                length = indexArray.length,
                dropDownData = proxy.model.columns[proxy._cellEditingDetails.columnIndex].dropdownData,
                selectedItems = [];

            if (dropDownData) {

                for (count = 0; count < length; count++) {

                    $.each(dropDownData, function (index, resourceInfo) {
                        if (indexArray[count] === index) {
                            selectedItems.push(resourceInfo);
                        }
                    });

                }
            }

            return selectedItems;
        },

        //GET THE FORMATED DATE 
        getFormatedDate: function (date) {
            return Globalize.format(date, this.model.dateFormat);
        },
        //Get formated string from date object
        getDateFromFormat: function (date) {

            if (typeof date === "object") {
                return new Date(date);
            }
            if (date) {
                return Globalize.parseDate(date, this.model.dateFormat) == null ?
                    new Date(date) : Globalize.parseDate(date, this.model.dateFormat);
            }
        },

        //GET THE FORMATTED TEXT
        formatting: function (formatstring, str, locale) {
            formatstring = formatstring.replace(/%280/g, "\"").replace(/&lt;/g, "<").replace(/&gt;/g, ">");
            locale = Globalize.cultures[locale] ? locale : "en-US";
            var s = formatstring;
            var frontHtmlidx, FrontHtml, RearHtml, lastidxval;
            frontHtmlidx = formatstring.split("{0:");
            lastidxval = formatstring.split("}");
            FrontHtml = frontHtmlidx[0];
            RearHtml = lastidxval[1];
            if (typeof (str) == "string" && $.isNumeric(str))
                str = Number(str);
            if (formatstring.indexOf("{0:") != -1) {
                var toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                var formatVal = toformat.exec(formatstring);
                if (formatVal != null && str != null) {
                    if (FrontHtml != null && RearHtml != null)
                        str = FrontHtml + Globalize.format(str, formatVal[2], locale) + RearHtml;
                    else
                        str = Globalize.format(str, formatVal[2], locale);
                } else if (str != null)
                    str = str;
                else
                    str = "";
                return str;
            } else if (this.data != null && this.data.Value == null) {
                $.each(this.data, function (dataIndex, dataValue) {
                    s = s.replace(new RegExp('\\{' + dataIndex + '\\}', 'gm'), dataValue);
                });
                return s;
            } else {
                return this.data.Value;
            }
        },


        //GET THE COLUMNINDEX BY USIGN THE FIELD NAME
        getColumnIndexByField: function (fieldName) {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                column = 0;

            for (column; column < length; column++) {
                if (columns[column]["field"] == fieldName)
                    break;
            }

            return column == columns.length ? -1 : column; // if field name not available in columns collection, it return -1 value.
        },

        //GET THE BROWSER DETAILS
        getBrowserDetails: function () {
            var b = navigator.userAgent.match(/(firefox|chrome|opera|msie|safari)\s?\/?(\d+(.\d+)*)/i);
            if (!!navigator.userAgent.match(/Trident\/7\./))
                return { browser: "msie", version: jQuery.uaMatch(navigator.userAgent).version };
            return { browser: b[1].toLowerCase(), version: b[2] };
        },


        setValidation: function () {

            var proxy = this,
                columns = proxy.model.columns,
                length = columns.length,
                count = 0;

            proxy._initValidator();

            for (count; count < length; count++) {

                if (!ej.isNullOrUndefined(columns[count]["validationRules"])) {

                    proxy.setValidationToField(columns[count].field, columns[count].validationRules);

                }
            }
        },

        //SET THE VALIDATION TO THE PARTICULAR FIELD
        setValidationToField: function (name, rules) {

            var proxy = this,
                ele = $("#" + proxy._id + name).length > 0 ? $("#" + proxy._id + name) : $("#" + name);
            ele.rules("add", rules);

            var validator = $("#" + proxy._id + "EditForm").validate({
                onsubmit: false

            });

            if (!ej.isNullOrUndefined(rules["required"])) {

                validator.settings.messages[name] = {};
                validator.settings.messages[name]["required"] = ej.TreeGrid.getColumnByField(proxy.model.columns, name).headerText + " is required";

            }
        },


        //GET THE FIELDNAME OF COLUMN BY USING THE HEADERTEXT
        getFieldNameByHeaderText: function (headerText) {

            var proxy = this;

            if (ej.isNullOrUndefined(proxy._fieldColumnNames[headerText])) return null;

            return proxy._fieldColumnNames[headerText];
        },


        //CLEAR THE SELECTION

        

        clearSelection: function (index) {

            var proxy = this,
                model = proxy.model;

            if (this.selectedRowIndex() != -1 || (this.selectedRowIndex() == -1 && proxy._previousIndex != -1 )) {
                                                 //To clear the row selection of previously selected row via set model (selectedRowIndex)
                var $gridRows = proxy.getRows(),
                    index;

                if (!ej.isNullOrUndefined(index)) {
                    ej.TreeGrid.getRowByIndex(proxy, index).removeAttr("aria-selected").find(".e-selectionbackground").removeClass("e-selectionbackground").removeClass("e-active");
                    var record=model.updatedRecords[index];
                    if (record){   // For delete all records
                        record.isSelected = false;
                    }
                    else {

                        if ($gridRows) {
                            $gridRows.removeAttr("aria-selected").find(".e-rowcell,.e-detailsrowcell").removeClass("e-selectionbackground").removeClass("e-active");
                        }
                        var dataManager = new ej.DataManager(model.flatRecords);

                        var query = new ej.Query();
                        query.search(true, ['isSelected'], ej.FilterOperators.Equals, true);

                        var records = dataManager.executeLocal(query);
                        var length = records && records.length;

                        if (length > 0) {

                            var count = 0;

                            for (count = 0; count < length; count++) {
                                records[count].isSelected = false;
                            }

                        }

                    }
                    //Clearing the selected item and index while clearing the row selection itself
                    if (!proxy._selectedRowsIndexes.length || (proxy._selectedRowsIndexes.length && model.selectionType == "single")) {
                        proxy._previousIndex = index;
                        proxy.selectedRowIndex(-1);
                        model.selectedItem = null;//For single selection only
                        proxy._cancelSaveTools();
                    }
                }
            }
        },


        //COUNT THE CHILDRECORDS COUNT
        getChildRecordsCount: function (record) {

            var proxy = this,
                childRecords = record.childRecords,
                count = 0,
                length = childRecords ? childRecords.length : 0;

            for (count; count < length; count++) {

                if (proxy._filteredRecords.length > 0) {

                    var recordIndex = proxy._filteredRecords.indexOf(childRecords[count]);

                    if (recordIndex != -1) {

                        ++proxy._removedCount;

                        if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                            proxy.getChildRecordsCount(childRecords[count]);
                        }

                    }
                } else {

                    ++proxy._removedCount;

                    if (childRecords[count].hasChildRecords && childRecords[count].expanded) {
                        proxy.getChildRecordsCount(childRecords[count]);
                    }

                }
            }

            return proxy._removedCount;
        },


        //GET THE EXPAND STATUS OF RECORD
        getExpandStatus: function (record) {

            var parentRecord = record.parentItem;

            if (parentRecord != null) {

                if (parentRecord.expanded === false) {
                    return false;
                } else if (parentRecord.parentItem) {

                    if (parentRecord.parentItem.expanded === false) {
                        return false;
                    } else {
                        return this.getExpandStatus(parentRecord.parentItem);
                    }

                } else return true;
            } else return true;
        },


        //RETURN THE COLUMN FIELD NAMES
        getColumnFieldNames: function () {

            var columnNames = [],
                columns = this.model.columns,
                count = 0,
                length = columns.length;

            for (count; count < length; count++) {

                if (columns[count]["field"] && (columns[count]["field"] != "resourceInfo") && (columns[count]["field"] != "predecessor")) { 
                    columnNames.push(columns[count]["field"]);
                }
                else if (columns[count]["field"] == "resourceInfo") {
                    
                    columnNames.push("resourceNames");
                }
                else if (columns[count]["field"] == "predecessor")
                {
                    
                    columnNames.push("predecessorsName");
                }

            }

            return columnNames;
        },


        //RENDERED  RECORDS
        renderRecords: function (args) {

            var proxy = this;

            if (!args) {

                args = {};
                args.requestType = ej.TreeGrid.Actions.Refresh;

            }

            this.sendDataRenderingRequest(args);
        },       


        //SELECT THE ROW IN TREEGRID
        selectRows: function (rowIndex, toIndex) {

            var proxy = this,
                model = proxy.model;

            var $gridRows = proxy.getRows(),
            selectedItem;
            if (ej.TreeGrid.getRowByIndex(proxy, rowIndex).hasClass("e-summaryrow"))
                return;
            if ((model.editSettings.allowEditing || model.editSettings.allowAdding) && !proxy._isRowEdit) {
                if ($("#" + proxy._id + "EditForm").length > 0 && proxy.endEdit()) {
                    $gridRows = proxy.getRows();
                    return;
                }
            }

            if (ej.isNullOrUndefined(toIndex) || ej.isNullOrUndefined(rowIndex)) {

                rowIndex = ej.isNullOrUndefined(rowIndex) ? toIndex : rowIndex;        

                    switch (model.selectionType) {

                        case ej.TreeGrid.SelectionType.Multiple:

                            if (proxy._multiSelectCtrlRequest) {
                                var selectedRowIndex = $.inArray(rowIndex, proxy._selectedRowsIndexes);
                                if (selectedRowIndex != -1) {
                                    proxy.clearSelection(rowIndex);
                                    proxy._selectedRowsIndexes.splice(selectedRowIndex, 1);
                                }

                                if (selectedRowIndex == -1) {
                                    proxy._selectedRowsIndexes.push(rowIndex);                                   
                                    ej.TreeGrid.getRowByIndex(proxy, rowIndex).attr("aria-selected", "true").find('.e-rowcell,.e-detailsrowcell').addClass(" e-selectionbackground e-active").attr("tabindex", "0");
                                }
                                break;
                            }
                            

                        case ej.TreeGrid.SelectionType.Single:
                            proxy.clearSelection(-1);
                            model.selectedItem = null;
                            if (rowIndex <= proxy.model.updatedRecords.length - 1) {
                                proxy._selectedRowsIndexes = [];
                                (rowIndex != -1) && proxy._selectedRowsIndexes.push(rowIndex);
                                ej.TreeGrid.getRowByIndex(proxy, rowIndex).attr("aria-selected", "true").find('.e-rowcell,.e-detailsrowcell').addClass(" e-selectionbackground e-active").attr("tabindex", 0);
                                if (proxy.model.updatedRecords[rowIndex]) {
                                    proxy.model.updatedRecords[rowIndex].isSelected = true;  //Selection issue Changes
                                    model.selectedItem = proxy.model.updatedRecords[rowIndex];
                                }
                                
                                if (proxy.model.updatedRecords[proxy._previousIndex] && rowIndex != proxy._previousIndex) {
                                    proxy.model.updatedRecords[proxy._previousIndex].isSelected = false;
                                }

                            }
                            break;
                    }
              
            } else {

                if (model.selectionType == ej.TreeGrid.SelectionType.Multiple)
                {
                proxy.clearSelection(-1);
                proxy._selectedRowsIndexes = [];
                var rows = rowIndex - toIndex < 0 ? $gridRows.slice(rowIndex, toIndex + 1) : $gridRows.slice(toIndex, rowIndex + 1),
                    count = 0,
                    length = rows.length;

                for (count; count < length; count++) {
                    var rowIndex = $gridRows.index(rows[count]);
                    proxy._selectedRowsIndexes.push(rowIndex);
                    model.flatRecords[rowIndex].isSelected = true;
                }
                $(rows).attr("aria-selected", "true").find('.e-rowcell,.e-detailsrowcell').addClass(" e-selectionbackground e-active").attr("tabindex", "0");
                }
            }

            if (proxy._selectedRowsIndexes.length && proxy._selectedRowsIndexes.indexOf(rowIndex) != -1)
            {
                proxy._previousIndex = rowIndex;
                if (this.selectedRowIndex() != rowIndex) {
                    this.selectedRowIndex(rowIndex);
                }
            }
            else if (proxy._selectedRowsIndexes.length == 0 && proxy._previousIndex != -1 && this.selectedRowIndex() != -1) {
                proxy._previousIndex = -1;
                this.selectedRowIndex(-1);
            } else {
                proxy._previousIndex = proxy._selectedRowsIndexes[proxy._selectedRowsIndexes.length-1];
                this.selectedRowIndex(proxy._previousIndex);
            }
            proxy._multiSelectCtrlRequest = false;
            return false;
        },


        //GET THE CURRENTCELLINDEX
        getCellIndex: function (e, target) {

            var $target = $(e.target),
                cellIndex = -1,
                model =this.model;

            if (target) $target = $(target);

            if ($target.prop("tagName") == "TD") {
                cellIndex =  $target.index();
            } else if ($target.prop('tagName') == "DIV" && $target.hasClass('e-cell')) {

                var $td = $target.closest('td');
                cellIndex = $td.index();

            } else if ($target.prop('tagName') == "DIV" && $($target.closest('td')[0]).hasClass("e-templatecell")) {

                var $td = $target.closest('td');
                cellIndex = $td.index();
            }
            else if ($($target.closest('td')[0]).hasClass("e-templatecell") && $($target.closest("tr")[0]).hasClass("e-templaterow")) {

                var $td = $target.closest('td');
                cellIndex = $td.index();
            }
            else if ($target.parent().prop("tagName") == "TD") {
                var $td = $target.closest('td');
                cellIndex = $target.parent().index();
            } 

            return cellIndex;
        },


        //GET THE CURRENT ROW INDEX
        getRowIndex: function (e) {

            var proxy = this,
                $target = $(e.target),
                $gridRows = proxy.getRows(),
                index = $gridRows.index($target.parent());

            if (index == -1) {

                var row = $target.closest('tr');
                index = $gridRows.index(row);

            }

            return index;
        },

        //Cancel the edited cell and row
        _cancelEditBoxes: function (e) {
            var proxy = this;
            if (proxy.model.isEdit)
                proxy.cancelEditCell();
            else if (proxy._isRowEdit)
                proxy.cancelRowEditCell();
        },

        //cancel the editMode of the cell
        cancelEditCell: function (e) {

            var proxy = this,
                model = proxy.model,
                tr,
                cellData = {},
                cell;
            if (model.isFromGantt && !model.isEdit)
                return true;
            tr = proxy.getRows()[proxy._cellEditingDetails.rowIndex],//proxy._cellEditingDetails.rowIndex
            cellData[proxy._cellEditingDetails.fieldName] = proxy._cellEditingDetails.cellValue,
            data = proxy.model.selectedItem,
            cellIndex = proxy._cellEditingDetails.columnIndex;


            if ($(tr).hasClass("e-insertedrow")) cell = tr.cells[cellIndex];
            else cell = tr.cells[cellIndex];

            $(cell).removeClass("e-editedcell").empty().html(
                $($.render[proxy._id + "_Template"](data))[0].cells[cellIndex].innerHTML
            );

            proxy.model.isEdit = false;
            proxy._cancelSaveTools();
            proxy._trigger("refresh");
        },

        // To destroy the edit forms and popup menu
        _destroyWidgetElements:function()
        {
            var proxy = this, model = this.model, column = model.columns, columnLength = column.length
            for (i = 0; i < columnLength; i++) {
                var $element = $("#" + proxy._id + column[i].field);
                $element.data("ejWidgets") && $element[$element.data("ejWidgets")[0]]("destroy");
            }
        },

        //Cancel row edit.
        cancelRowEditCell: function () {

            var proxy = this, model = this.model,
            //  To delete the empty added row when allowDeleting is false.
            temp = proxy.model.editSettings.allowDeleting,
            form = $(document.getElementById(proxy._id + "EditForm")), $tr;
            if(form.length>0)
                this._destroyWidgetElements();

            $tr = $(form.closest("tr"));
            rowIndex = $tr.index();
            data = model.selectedItem;
            if (rowIndex == -1 && form.length == 0)
                return;
            
            if ($tr.hasClass('e-addedrow')) {
                if (!proxy.model.editSettings.allowDeleting)
                    proxy.model.editSettings.allowDeleting = true;
            }
            
            //Cancel the edited row.
            if ($tr.hasClass('e-editedrow')) {
                $tr.removeClass("e-editedrow").empty().html(
                    $($.render[proxy._id + "_Template"](data))[0].innerHTML);
            }
            else if ($tr.hasClass('e-rowedit')) {
                $tr.removeClass("e-rowedit").empty().html(
                    $($.render[proxy._id + "_Template"](data))[0].innerHTML);
                index = rowIndex;
            }               
            //Cancel the newly added row.
            else if ($tr.hasClass('e-addedrow')) {
                proxy._isRefreshAddedRecord = false;
                proxy.deleteRow();
            }           
            model.isEdit = proxy._isRowEdit = false;
            model.editSettings.allowDeleting = temp;
            proxy._cancelSaveTools();
            proxy._trigger("refresh");
        },

        //RERENDERED THE EDITED RECORD
        refreshRow: function (index) {
            var proxy = this,
                model = proxy.model,
                data = model.updatedRecords[index], eventArgs = {},
                editIndex = proxy.getIndexByRow($("#" + proxy._id + "EditForm").closest("tr"))
            if (model.enableVirtualization)
            {
                index = model.currentViewData.indexOf(data);
            }
            if (data && index > -1) {
                var currentRefreshingRow = $(proxy.getRows()[index]);

                if (model.showDetailsRow && model.detailsTemplate && data.isDetailsExpanded) {
                    var detailsRow = currentRefreshingRow.next("tr");
                    if (detailsRow.hasClass("e-detailsrow"))
                        detailsRow.remove();
                }
                currentRefreshingRow.replaceWith($($.render[proxy._id + "_Template"](data)));
                proxy.setGridRows($(proxy.getContentTable().get(0).rows));

                eventArgs.rowElement = $(proxy.getRows()[index])[0];
                eventArgs.data = data;
                proxy._trigger("refresh");
                proxy._refreshedCellEventTrigger(eventArgs);

                // Check whether the edited index and refreshed index are same
                // If it is same either rowEdit or cellEdit is enable in particular row.
                // So update the _isRowEdit and isEdit value is false, and also update the toolbar item.s
                if (editIndex == index) {
                    proxy._isRowEdit = proxy.model.isEdit = false;
                    proxy._cancelSaveTools();
                }
            }
        },
        /* refresh the details row while cell editing */
        _refreshDetailsRow: function (index) {
            var proxy = this,
                model = this.model,
                data = proxy.model.currentViewData[index], eventArgs = {};
            if (data) {
                var currentRefreshingRow = $(this.getRows()[index]).next("tr");
                if (currentRefreshingRow.hasClass("e-detailsrow")) {
                    currentRefreshingRow.remove();
                    var detailsRow = $($.render[proxy._id + "_detailRowTemplate"](data));
                    $(this.getRows()[index]).after(detailsRow);
                    proxy.setGridRows($(proxy.getContentTable().get(0).rows));
                    proxy._trigger("refresh");
                    if (model.detailsDataBound) {
                        this._trigger("detailsDataBound", { detailsElement: detailsRow, data: data });
                    }
                }
            }
        },
        //Trigger rowDataBound and queryCellInfo event when row is refreshed
        _refreshedCellEventTrigger:function(eventArgs)
        {
            var proxy = this, model = this.model;
            if (model.queryCellInfo != null || model.rowDataBound != null || model.detailsDataBound != null) {
                if (model.rowDataBound != null){
                    proxy._trigger("rowDataBound", {
                        rowElement: eventArgs.rowElement,
                        data: eventArgs.data
                    });
                }

                var tdCells = eventArgs.rowElement.cells,
                $tdRowcells = $(eventArgs.rowElement).find(".e-rowcell"),
                cellIndex = 0,
                length = tdCells.length,
                column = null,
                columns = proxy.model.columns;

                if (model.queryCellInfo != null) {

                    for (cellIndex; cellIndex < length; cellIndex++) {
                        if ($(tdCells[cellIndex]).hasClass("e-rowcell")) {
                            column = columns[$tdRowcells.index(tdCells[cellIndex])];
                        }
                        if (column) {
                            proxy._cellEventTrigger(tdCells[cellIndex], eventArgs.data, column);
                        }
                    }
                }
                if (model.detailsDataBound && $(eventArgs.rowElement).next("tr").hasClass("e-detailsrow")) {
                    this._trigger("detailsDataBound", { detailsElement: $(eventArgs.rowElement).next("tr"), data: eventArgs.data });
                }
            }
        },
        /* get visible child record count in udpated records */
        _getVisibleChildRecordCount: function (data, count, collection) {
            var proxy = this, model = this.model, childRecords, length;;

            if (data.hasChildRecords) {
                childRecords = data.childRecords;
                length = childRecords.length;
                for (var i = 0; i < length; i++) {
                    if (collection.indexOf(childRecords[i]) !== -1)
                        count++;
                    if (childRecords[i].hasChildRecords) {
                        count = proxy._getVisibleChildRecordCount(childRecords[i], count, collection);
                    }
                }
            }
            else {
                if (collection.indexOf(data) !== -1) {
                    count++;
                }
            }
            return count;
        },
        //Insert templated row in DOM for newly added record
        _renderAddedRow: function (index, data) {
            var proxy = this, model = this.model;
            proxy._updateCurrentViewData();
            if (model.enableVirtualization) {
                var tempArgs = {};
                tempArgs.requestType = ej.TreeGrid.Actions.Refresh;
                proxy.sendDataRenderingRequest(tempArgs);
            }
            else {
                proxy.renderNewAddedRow(index, data);
            }
            proxy.updateHeight();
        },

        //insert new item in DOM 
        renderNewAddedRow:function(index, data)
        {
            var proxy = this, model = this.model, length,
                gridRows = proxy.getRows(),
                addedGridRow = $.render[this._id + "_Template"](data);
                
            if (gridRows !== null && gridRows.length > 0) {
                if (gridRows.length == index) {
                    var insertPositionRow = gridRows.eq(index - 1);
                    /*check whether the next row is detail row or not */
                    if($(insertPositionRow).next("tr").hasClass("e-detailsrow"))
                    {
                        $(insertPositionRow).next("tr").after(addedGridRow)
                    }else
                    {
                        $(insertPositionRow).after(addedGridRow)
                    }
                } else {
                    gridRows.eq(index).before(addedGridRow);
                }
            } else {
                proxy.getContentTable().find('tbody').empty().append(proxy._getEmptyTbody());
                var temp = document.createElement('div');
                proxy.getContentTable().find("colgroup").first().replaceWith(proxy._getMetaColGroup());
                var $tbody = proxy.getContentTable().children('tbody');
                $tbody.empty();
                temp.innerHTML = ['<table>', $.render[proxy._id + "_Template"](proxy.model.currentViewData),
                    '</table>'].join("");
                proxy.getContentTable().get(0).replaceChild(temp.firstChild.firstChild,
                    proxy.getContentTable().get(0).lastChild);
            }
            proxy._gridRows = proxy.getContentTable().get(0).rows;
            proxy.setGridRows($(proxy.getContentTable().get(0).rows));
          
        },


        
        onScrollHelper: function (scrolltop) {
            var proxy = this;            
            proxy._$gridContent.ejScroller("scrollY", scrolltop, true);
            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");
            if (!isHorizontalScroll)
                proxy._$gridContent.scrollTop(scrolltop);
        },

        isVScroll: function () {
            if (this._$gridContent.hasClass("e-scroller")) {
                return this._$gridContent.ejScroller("isVScroll");
            }
            else
            {
                return false;
            }
        },

        //REFRESH THE SCROLLER FOR TREEGRIDCONTENT
        refreshScroller:function(panSize){
            
            var proxy = this;                       
            proxy._$gridContent.removeClass("e-borderbox");
            
            var scrollTop = proxy._$gridContent.ejScroller("option", "scrollTop");
            var scrollLeft = proxy._$gridContent.ejScroller("option", "scrollLeft");

            proxy._$gridContent.ejScroller("option", { "persist": true });
            proxy._$gridContent.ejScroller("option", { "width": panSize });
            
            var maxScrollWidth = proxy.getMaxScrollWidth();
            if (scrollLeft > maxScrollWidth)
                scrollLeft = maxScrollWidth > 0 ? maxScrollWidth : 0;

            proxy._$gridContent.css("height", proxy._viewPortHeight);
            proxy._$gridContent.ejScroller("refresh");
            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");

            if (proxy.model.isFromGantt)
                if (!isHorizontalScroll) {
                    proxy._$gridContent.addClass("e-borderbox");
                }
                else {
                    proxy._$gridContent.removeClass("e-borderbox");
                }
            proxy._$gridContent.ejScroller("option", { "scrollTop": scrollTop });
            proxy._$gridContent.ejScroller("option", { "scrollLeft": scrollLeft });
            proxy._$gridHeaderContainer.scrollLeft(scrollLeft);

            if (!isHorizontalScroll)
                proxy._$gridContent.scrollTop(scrollTop);
            else {
                proxy._$gridContent.scrollTop(0)
            }
        },
        getMaxScrollHeight: function () {
            var proxy = this;
            return proxy._$gridContent.children(".e-content").children().height() - proxy._$gridContent.children(".e-content").height();
        },
        getMaxScrollWidth:function()
        {
            var proxy=this;
            return proxy._$gridContent.children(".e-content").children(".e-gridcontainer").children(".e-table").width() - proxy._$gridContent.children(".e-content").width();
        },
        refreshHeight:function(){

            var proxy = this;
            proxy._$gridContent.ejScroller("refresh");            

        },

		// Get the column using header text
        // argument headerText is column header text value

        getColumnByHeaderText: function (headerText) {
            var proxy = this;
            var length = proxy.model.columns.length;
            var columns = proxy.model.columns;
            for (var column = 0; column < length; column++) {
                if (columns[column]["headerText"] == headerText)
                    break;
            }
            return column == columns.length ? null : columns[column];
        },

        //Hide the column by using header text
        //argument headerText is column header text value
        
        hideColumn: function (headerText) {
            var proxy = this,
                model = proxy.model,
                column = this.getColumnByHeaderText(headerText),
                index = $.inArray(column, model.columns),
                hiddenColumnIndex = $.inArray(headerText, proxy._hiddenColumns);
            if (index != -1 && index != model.treeColumnIndex && hiddenColumnIndex == -1) {
                proxy._hiddenColumns.push(headerText);
                proxy.model.columns[index].visible = false;
               
				//Table column width not properly set in firefox, so just remove and add the e-table class from both header and content table
                proxy.getHeaderTable().removeClass("e-table");
                proxy.getContentTable().removeClass("e-table");
                var headerColgroup = proxy.getHeaderTable().find('colgroup').find("col");
                $(headerColgroup).eq(index).hide();
                var headerRows = proxy.getHeaderTable().find('thead').find("th.e-headercell");
                $(headerRows).eq(index).addClass("e-hide");
                if (model.allowFiltering) {
                    var headerrows = proxy.getHeaderTable().find('thead').find("th.e-filterbarcell");
                    $(headerrows).eq(index).addClass("e-hide");
                }
                var gridrows = proxy.getContentTable().find("colgroup").find("col");
                $(gridrows).eq(index).hide();
                gridrows = proxy.getRows();
				//hidding the specified column
                gridrows.each(function () {
                    var td = $(this).children("td").eq(index);
                    $(td).addClass("e-hide");
                    if (model.showSummaryRow && $(td).hasClass('e-summaryrowcell'))
                        proxy._updateSummaryRowTitleCell(headerColgroup, td[0].parentElement);
                });
                //Hide the specific column from total summary row.
                if (model.showTotalSummary) {
                    var footerRows = proxy._$footertableContent.find("tbody").find("tr"),
                        footerColGroup = proxy._$footertableContent.find("colgroup").find("col");
                    $(footerColGroup).eq(index).hide();
                    footerRows.each(function () {
                        var footerSummarytd = $(this).children("td").eq(index);
                        $(footerSummarytd).addClass("e-hide");
                        if ($(footerSummarytd).hasClass('e-footersummaryrowcell'))
                            proxy._updateSummaryRowTitleCell(footerColGroup, footerSummarytd[0].parentElement);
                    });
                }

                /* update the colspan for details rows */
                proxy._gridRows && proxy.getTreeGridRows().find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", proxy._visibleColumns.length - 1);

                proxy._updateScrollCss();
                proxy.getHeaderTable().addClass("e-table");
                proxy.getContentTable().addClass("e-table");
                var gridContent = proxy._$gridContent;
                if (model.isFromGantt) {
                    var isHorizontalScroll = gridContent.ejScroller("isHScroll");
                    if (!isHorizontalScroll)
                        gridContent.addClass("e-borderbox");
                    else
                        gridContent.removeClass("e-borderbox");
                }
                gridContent.ejScroller("refresh");
                proxy._resizeFilteringElements();
                proxy._$gridHeaderContainer.scrollLeft(gridContent.ejScroller("option", "scrollLeft"));
                index = $.inArray(headerText, proxy._visibleColumns);
                if (index != -1)
                    proxy._visibleColumns.splice(index, 1);
                if (model.showColumnChooser)
                    proxy._renderColumnChooserList(true,headerText);
                proxy._addInitTemplate();
            }
        },

        //Clear the filter based on fieldName.
        //clearFilter method without argumentment means, it clear the all filter in tree grid.
        clearFilter: function (fieldName) {
            var proxy = this,
                model = proxy.model;
            if (ej.isNullOrUndefined(fieldName)) {
                var filteredColumns = model.filterSettings.filteredColumns,
                    length = filteredColumns && filteredColumns.length;
                //Loop for clearing all the filter in tree grid
                while (filteredColumns && filteredColumns.length > 0) {
                    proxy.clearFilter(filteredColumns[0].field);
                }

            } else {
                var column = proxy.getColumnByField(fieldName),
                    filteredColumn = $.grep(model.filterSettings.filteredColumns, function (filterColumn, index) {
                        return filterColumn.field === fieldName;
                    });
                if (column && filteredColumn.length > 0) {
                    /*Skip the filterColumn method for numeric textbox in immediate filterbar mode.
                      Because while set the empty value to numeric text box, it automatically triiger filterBarHanlder method*/
                    if (!(model.filterBarMode == "immediate" && column.filterEditType == "numericedit")) {
                        proxy._searchString = "";
                        proxy._fieldName = "";
                        proxy.filterColumn(fieldName, "equal", "", "and");
                    }
                    proxy._clearFilterElementValue(column);
                }
            }
        },

        //Get the column details based on the given field in tree grid
        getColumnByField: function (fieldName) {
            var proxy = this,
              model = proxy.model,
              columns = model.columns;
            for (var index = 0; index < columns.length; index++) {
                if (columns[index]["field"] == fieldName)
                    break;
            }
            return index == columns.length ? null : columns[index]; //If field name not available in column collection, it returns null value.
        },

        //Set the empty value to filter input elements
        _clearFilterElementValue: function (column) {
            var proxy = this,
                element = $("#" + proxy._id + "_" + column.field + "_filterbarcell");
            switch (column.filterEditType) {
                case "stringedit":
                    $(element).val("");
                    break;
                case "booleanedit":
                    $(element).prop("checked", false);
                    break;
                case "numericedit":
                    $(element).ejNumericTextbox("instance").option("value", "");
                    break;
                case "dropdownedit":
                    $(element).ejDropDownList("instance").option("value", "");
                    break;
                case "datepicker":
                    $(element).ejDatePicker("instance").option("value", "");
                    break;
                case "datetimepicker":
                    $(element).ejDateTimePicker("instance").option("value", "");
                    break;
            }
        },

        //To maintain the summary row title after hide/show the column using columnChooser.
        _updateSummaryRowTitleCell: function (colgroup, summaryRow) {
            var length = colgroup.length;
            for (var cell = 0; cell < length; cell++) {
                if ($(colgroup[cell]).css('display') == 'none')
                    continue;
                var summaryTitleElement = summaryRow.getElementsByClassName("e-summarytitle")[0],
                    currentColumnSummaryTitle = summaryRow.childNodes[cell].getElementsByClassName("e-summarytitle")[0];
                if (currentColumnSummaryTitle)
                    return;
                else
                    $(summaryRow.childNodes[cell]).append(summaryTitleElement);
                return;

            }

        },

        //Show the column by using header text
        // argument headerText is column header text value

        
        showColumn: function (headerText) {
            var proxy = this,
                model = proxy.model,
                column = this.getColumnByHeaderText(headerText),
                index = $.inArray(column, model.columns),
                visibleColumnIndex = $.inArray(headerText, proxy._visibleColumns);
            if (index != -1 && visibleColumnIndex == -1) {
                proxy._visibleColumns.push(headerText);
                proxy.model.columns[index].visible = true;
                var headerColgroup = proxy.getHeaderTable().find('colgroup').find("col");
                $(headerColgroup).eq(index).show();
                var headerRows = proxy.getHeaderTable().find('thead').find("th.e-headercell");
                $(headerRows).eq(index).removeClass("e-hide");
                if (model.allowFiltering) {
                    var headerrows = proxy.getHeaderTable().find('thead').find("th.e-filterbarcell");
                    $(headerrows).eq(index).removeClass("e-hide");
                }
                var gridrows = proxy.getContentTable().find("colgroup").find("col");
                $(gridrows).eq(index).show();
                gridrows = proxy.getRows();
				//Display the specified column
                gridrows.each(function () {
                    var td = $(this).children("td").eq(index);
                    $(td).removeClass("e-hide");
                    if (model.showSummaryRow && $(td).hasClass('e-summaryrowcell'))
                        proxy._updateSummaryRowTitleCell(headerColgroup, td[0].parentElement);
                });
                //Show the specific column from total summary row.
                if (model.showTotalSummary) {
                    var footerRows = proxy._$footertableContent.find("tbody").find("tr"),
                        footerColGroup = proxy._$footertableContent.find("colgroup").find("col");
                    $(footerColGroup).eq(index).show();
                    footerRows.each(function () {
                        var footerSummarytd = $(this).children("td").eq(index);
                        $(footerSummarytd).removeClass("e-hide");
                        if ($(footerSummarytd).hasClass('e-footersummaryrowcell'))
                            proxy._updateSummaryRowTitleCell(footerColGroup, footerSummarytd[0].parentElement);
                    });
                }
                /* update the colspan for details rows */
                proxy._gridRows && proxy.getTreeGridRows().find(".e-detailscellwrapper").parent(".e-detailsrowcell").attr("colspan", proxy._visibleColumns.length);
                proxy._updateScrollCss();
                proxy._addInitTemplate();
                var gridContent = proxy._$gridContent;
                if (model.isFromGantt) {
                    var isHorizontalScroll = gridContent.ejScroller("isHScroll");
                    if (!isHorizontalScroll) {
                        gridContent.addClass("e-borderbox");
                    }
                    else {
                        gridContent.removeClass("e-borderbox");
                    }
                }
                gridContent.ejScroller("refresh");
                proxy._$gridHeaderContainer.scrollLeft(gridContent.ejScroller("option", "scrollLeft"));
                index = $.inArray(headerText, proxy._hiddenColumns);
                if (index != -1)
                    proxy._hiddenColumns.splice(index, 1);
                if(model.showColumnChooser)
                    proxy._renderColumnChooserList(true, headerText);
            }
        },

        // Get the hidden columns detail in tree grid
        getHiddenColumns: function () {
            var proxy = this,
                model = proxy.model,
                hiddenColumn = proxy._hiddenColumns,
                length = hiddenColumn.length,
                hiddenColumns = [];
            for (var i = 0; i < length; i++)
                hiddenColumns.push(proxy.getColumnByHeaderText(hiddenColumn[i])); //Get the column by using header text
            return hiddenColumns;

        },

        // Get the column details based on the given column index in tree grid
        getColumnByIndex: function (index) {
            var proxy = this,
                model = proxy.model,
                columns = model.columns;
            if (columns.length > index)
                return columns[index];
            return null;
        },

        // Get the visible columns detail in tree grid
        getVisibleColumns: function () {
            var proxy = this,
                model = proxy.model,
                visibleColumn = proxy._visibleColumns,
                length = visibleColumn.length,
                visibleColumns = [];
            for (var i = 0; i < length; i++)
                visibleColumns.push(proxy.getColumnByHeaderText(visibleColumn[i])); //Get the column by using header text
            return visibleColumns;

        },

        //Cancel the edit cell or row based on editing.
        cancelEdit: function () {
            if (this._isRowEdit)
                this.cancelRowEditCell();
            else if (this.model.isEdit)
                this.cancelEditCell();
        },

        //Rerender the tree grid content.
        redraw: function () {
            var args = {};
            args.requestType = ej.TreeGrid.Actions.Refresh;
            this.sendDataRenderingRequest(args);
        },

        // Public method for removing column menu
		clearColumnMenu:function()
        {
            var proxy=this;
            proxy._clearColumnMenu();
		},

		"export": function (action, serverEvent, multipleExport) {
		    var proxy = this,
                model = proxy.model;
		    var updatedRecords = model.updatedRecords,
               currentViewData = model.currentViewData,
               flatRecords = model.flatRecords,
               parentRecords = model.parentRecords,
               summaryRowRecords = model.summaryRowRecords,
               selectedItem = model.selectedItem;
		    proxy._contextMenuOperations("Cancel"); // Cancel the edit row or empty newly added row.
            // Delete the below internal items in model for avoid the circular reference while extend the model
		    delete model.updatedRecords;
		    delete model.currentViewData;
		    delete model.flatRecords;
		    delete model.parentRecords;
		    delete model.summaryRowRecords;
		    delete model.selectedItem;
		    var modelClone = $.extend(true, {}, model);
		    model.updatedRecords = updatedRecords;
		    model.currentViewData = currentViewData;
		    model.flatRecords = flatRecords;
		    model.parentRecords = parentRecords;
		    model.selectedItem = selectedItem;
		    model.summaryRowRecords = summaryRowRecords;
		    var columns = modelClone.columns,
                columnLength = columns.length,
                filteredColumns = modelClone.filterSettings.filteredColumns,
                filterColumnLength = filteredColumns.length;
		    for (var i = 0; i < columnLength; i++) {
		        if (modelClone.columns[i].editType != undefined) {
		            switch (columns[i].editType) {
		                case "stringedit":
		                    columns[i].editType = "string";
		                    break;
		                case "numericedit":
		                    columns[i].editType = "numeric";
		                    break;
		                case "dropdownedit":
		                    columns[i].editType = "dropdown";
		                    break;
		                case "booleanedit":
		                    columns[i].editType = "boolean";
		                    break;
		                default:
		                    break;
		            }
		        }
		        if (columns[i].filterEditType != undefined) {
		            switch (columns[i].filterEditType) {
		                case "stringedit":
		                    columns[i].filterEditType = "string";
		                    break;
		                case "numericedit":
		                    columns[i].filterEditType = "numeric";
		                    break;
		                case "dropdownedit":
		                    columns[i].filterEditType = "dropdown";
		                    break;
		                case "booleanedit":
		                    columns[i].filterEditType = "boolean";
		                    break;
		                default:
		                    break;
		            }
		        }
		    }
		    for (var i = 0; i < filterColumnLength; i++) {
		        if (filteredColumns[i].operator == "equal")
		            filteredColumns[i].operator = "equals";
		        else if (modelClone.filterSettings.filteredColumns[i].operator == "notequal")
		            filteredColumns[i].operator = "notequals";
		    }
		    var attr = { action: action, method: 'post', "data-ajax": "false" };
		    $('form#' + proxy._id + 'export').remove();
		    var form = ej.buildTag('form#'+proxy._id+'export', "", null, attr);
            var ignoreOnExport = proxy.ignoreOnExport,
                igonreLength = ignoreOnExport.length;
		    if (this.ignoreOnExport) {
		        for (var i = 0; i < igonreLength; i++) {
		            delete modelClone[ignoreOnExport[i]];
		        }
		    }
		    if (ej.raiseWebFormsServerEvents) {
		        var args = { model: modelClone, originalEventType: serverEvent };
		        var clientArgs = { model: JSON.stringify(modelClone) };
		        ej.raiseWebFormsServerEvents(serverEvent, args, clientArgs);
		    }
		    else {
		        var treegridObjectArray = {};
		        if (multipleExport) {
		            $('body').find('.e-treegrid').each(function (index, object) {
		                var gridObject = $(object).data('ejTreeGrid');
		                if (!ej.isNullOrUndefined(gridObject)) {
		                    var gridmodel = gridObject.model;
		                    updatedRecords = gridmodel.updatedRecords;
		                    flatRecords = gridmodel.flatRecords;
		                    currentViewData = gridmodel.currentViewData;
		                    parentRecords = gridmodel.parentRecords;
		                    summaryRowRecords = gridmodel.summaryRowRecords;
		                    selectedItem = gridmodel.selectedItem;
		                    // Delete the below internal items in model for avoid the circular reference while extend the model
		                    delete gridmodel.updatedRecords;
		                    delete gridmodel.flatRecords;
		                    delete gridmodel.currentViewData;
		                    delete gridmodel.parentRecords;
		                    delete gridmodel.summaryRowRecords;
		                    delete gridmodel.selectedItem;
		                    var modelClone = JSON.parse(JSON.stringify(gridmodel));
		                    gridmodel.updatedRecords = updatedRecords;
		                    gridmodel.flatRecords = flatRecords;
		                    gridmodel.currentViewData = currentViewData;
		                    gridmodel.parentRecords = parentRecords;
		                    gridmodel.summaryRowRecords = summaryRowRecords;
		                    gridmodel.selectedItem = selectedItem;

		                    columns = modelClone.columns;
		                    columnLength = columns.length;
		                    filteredColumns = modelClone.filterSettings.filteredColumns,
                            filterColumnLength = filteredColumns.length;
                            for (var i = 0; i < columnLength; i++) {
		                        if (columns[i].editType != undefined) {
		                            switch (columns[i].editType) {
		                                case "stringedit":
		                                    columns[i].editType = "string";
		                                    break;
		                                case "numericedit":
		                                    columns[i].editType = "numeric";
		                                    break;
		                                case "dropdownedit":
		                                    columns[i].editType = "dropdown";
		                                    break;
		                                case "booleanedit":
		                                    columns[i].editType = "boolean";
		                                    break;
		                                default:
		                                    break;
		                            }
		                        }
		                        if (columns[i].filterEditType != undefined) {
		                            switch (columns[i].filterEditType) {
		                                case "stringedit":
		                                    columns[i].filterEditType = "string";
		                                    break;
		                                case "numericedit":
		                                    columns[i].filterEditType = "numeric";
		                                    break;
		                                case "dropdownedit":
		                                    columns[i].filterEditType = "dropdown";
		                                    break;
		                                case "booleanedit":
		                                    columns[i].filterEditType = "boolean";
		                                    break;
		                                default:
		                                    break;
		                            }
		                        }
		                    }
                            for (var i = 0; i < filterColumnLength; i++) {
		                        if (filteredColumns[i].operator == "equal")
		                            filteredColumns[i].operator = "equals";
		                        else if (filteredColumns[i].operator == "notequal")
		                            filteredColumns[i].operator = "notequals";
                            }
		                    if (ignoreOnExport) {
		                        for (var i = 0; i < igonreLength; i++) {
		                            delete modelClone[ignoreOnExport[i]];
		                        }
		                        treegridObjectArray[index] = JSON.stringify(modelClone);
		                        var inputAttr = { name: 'TreeGridModel', type: 'hidden', value: JSON.stringify(modelClone) }
		                        var input = ej.buildTag('input', "", null, inputAttr);
		                        form.append(input);
		                    }
		                }
		            });
		        }
		        else {
		            var inputAttr = { name: 'TreeGridModel', type: 'hidden', value: JSON.stringify(modelClone) }
		            var input = ej.buildTag('input', "", null, inputAttr);
		            form.append(input);
		            form.append(this);
		        }
		        $('body').append(form);
		        form.submit();
		    }
		    return true;
		},
        //#endregion

        //#region PRIVATE METHODS

        //CALCULATE THE VIEWPORTHEIGHT
        _getViewPortHeight: function () {

            var proxy = this,
                height = 0,borderTopWidth=0;
                borderTopWidth = proxy.model.isFromGantt ? 0 : 1;
                height = proxy.element.height() -
                proxy._$gridHeaderContent.height() -
                parseFloat(borderTopWidth) -
                parseFloat(proxy._$gridHeaderContent.css("border-bottom-width"));
                if (proxy.model.toolbarSettings.showToolbar) {
                    height -= $("#" + proxy._id + "_toolbarItems").outerHeight();
                }

            if (!proxy.model.isFromGantt)
                height -= 1;//in gantt gridcontent does't contain border-width but have in treeGrid
            return height;

        },

        updateViewPortHeight: function () {
            var proxy = this;
            proxy._viewPortHeight = proxy._getViewPortHeight();
        },

        _updateScrollCss: function () {
            var proxy = this,
                model = proxy.model;
            if (!model.isFromGantt) {
                var isVscroll = proxy.isVScroll();
                var isSafari = /Safari/.test(navigator.userAgent) && /Apple Computer/.test(navigator.vendor);
                if (isSafari)
                    proxy._$gridHeaderContent.removeClass("e-gridheader");
                if (isVscroll) {
                        proxy._$gridHeaderContent.addClass("e-scrollcss");                      
                } else {
                    proxy._$gridHeaderContent.removeClass("e-scrollcss");
                    if (model.showTotalSummary)
                        proxy._$totalSummaryRowContainer.removeClass("e-scrollcss");

                }

                // Tree grid header not properly aligned in safari browser.
                // Because dynamically add or remove any css class it not affected immediately in safari browser
                // So just refresh the tree grid header
                if (isSafari) {
                    proxy._$gridHeaderContent.width(proxy._gridWidth);
                    proxy._$gridHeaderContent.addClass("e-gridheader");
                }    
                proxy._resizeFilteringElements();
            }
        },



        //UPDATE CURRENT VIEWPORT DATA FROM FLATRECORDS/UPDATED RECORDS
        _updateCurrentViewData: function () {

            var proxy = this,
                model = proxy.model,
                args = {},
                tMargin,
                isRangeModified = false;

            //CHECK VIRTUALIZATION IS ENABLED OR NOT 
            if (model.enableVirtualization) {

                proxy.getVisibleRange();
                model.currentViewData = proxy.model.updatedRecords.slice(proxy._visibleRange.top,
                    proxy._visibleRange.bottom);



                if (proxy._vScrollDist !== 0) {

                    tMargin = proxy._scrollTop - proxy._offset;

                    

                    if ((proxy._prevRTop - proxy._visibleRange.top !== 0) ||
                        (proxy._prevRBottom - proxy._visibleRange.bottom !== 0)) {

                        args.requestType = ej.TreeGrid.Actions.Refresh;
                        proxy.sendDataRenderingRequest(args);
                        isRangeModified = true;

                    }

                        proxy._$tableContent.css({
                            "top": tMargin
                        });

                    proxy._prevRTop = proxy._visibleRange.top;
                    proxy._prevRBottom = proxy._visibleRange.bottom;

                }

            } else {
                model.currentViewData = proxy.model.updatedRecords.slice();
            }

            if (proxy._isEnterKeyPressed) {

                proxy._isEnterKeyPressed = false;
                if (model.allowSelection && model.editSettings.allowEditing) {

                    if (!isRangeModified) proxy._cellEditingDetails.rowIndex += 1;

                    if (!proxy._rowSelectingEventTrigger(this.selectedRowIndex(), proxy._cellEditingDetails.rowIndex)) {

                        proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                        proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);

                    }

                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, proxy.model.columns[proxy._cellEditingDetails.columnIndex].field);

                }
            }
        },


        //GET THE TABLE ROW POSITION
        _getRowPosition: function (y) {
            return Math.ceil((y) / (this.model.rowHeight + this._detailsRowHeight));
        },


        //GET THE COLUMNGROUP FOR HEADER TABLE
        _getMetaColGroup: function () {

            var $colgroup = this.getHeaderTable().find('colgroup').clone();
            return $colgroup;

        },


        //CREATE EMPTY ROW WHILE DATASOURCE OBJECT AS NULL
        _getEmptyTbody: function () {

            var proxy = this,
                $emptyTd = ej.buildTag('td', proxy.model.emptyRecordText, {}, { colSpan: proxy.model.columns.length });
            return $(document.createElement("tr")).append($emptyTd);

        },


        //ADD SORTICON TO THE COLUMN HEADER
        _addSortElementToColumn: function (field, direction) {

            var proxy = this, model = this.model,
                column = ej.TreeGrid.getColumnByField(proxy.model.columns, field),
                index = $.inArray(column, proxy.model.columns);

            var $headerCellDiv = proxy.getHeaderTable().find("thead").
                find(".e-headercell").eq(index).find(".e-headercelldiv");
            $headerCellDiv.find(".e-ascending,.e-descending").remove();
            $headerCellDiv.append(proxy._createSortElement().addClass("e-" + direction).addClass("e-icon"));
            $headerCellDiv.parent().attr("aria-sort", direction);
        },


        //CREATE SORT ELEMENT        
        _createSortElement: function () {
            return ej.buildTag('span', "&nbsp;");
        },


        //RENDERED EDITED CELL
        _renderCellEditObject: function (cellEditArgs, $td) {

            var proxy = this,
                $form = ej.buildTag("form.#" + proxy._id + "EditForm", "", {}, {}),
                $cellEditTemplate = proxy._cellEditTemplate,
                $element,
                htmlString,
                cellData = {},
                cellValue,
                columnObject = cellEditArgs.columnObject;

            cellValue = cellEditArgs.value;
            if (columnObject.editType == "dropdownedit" && cellValue == false)
                cellValue = cellValue.toString();
            cellData[columnObject.field] = cellValue,
            columnIndex = proxy.getColumnIndexByField(columnObject.field);

            if (columnIndex===proxy.model.treeColumnIndex) {
                $td.find('.e-cell').empty();
            } else $td.empty();

            htmlString = $cellEditTemplate.find("#" + columnObject.field + "_CellEdit").html();
            if (columnObject.field === "predecessor")
                $element = $($.templates(htmlString).render(cellEditArgs.data));
            else
                 $element = $($.templates(htmlString).render(cellData));

            if ($element.get(0).tagName == "SELECT") {

                $element.val(cellData[columnObject.field]);
                $element.val() == null && $element.val($element.find("option").first().val());

            }

            $form.append($element);

           if (columnIndex===proxy.model.treeColumnIndex) {
                $td.find('.e-cell').append($form);
            } else $td.append($form);

            proxy._setoffsetWidth();
            proxy._refreshEditForm();

            if ($.isFunction($.validator) && !$.isEmptyObject(cellEditArgs.validationRules)) {

                proxy._initValidator();
                proxy.setValidationToField(columnObject.field, cellEditArgs.validationRules);

            }
            proxy.model.isEdit = true;

        },


        //SET WIDTH FOR EDITING ELEMENTS
        _setoffsetWidth: function () {

            var proxy = this,
                model = proxy.model,
                tds, $form = $("#" + proxy._id + "EditForm"),
                i = 0,
                rowIndex = proxy._cellEditingDetails.rowIndex,
                length, child;

            if (model.editSettings.editMode.toLowerCase() == "cellediting" && !proxy._isRowEdit) {
                tds = $form.closest("td");
            } else {
                tds = $form.find("tr").find(".e-rowcell");
            }

            length = tds.length;

            for (i; i < length; i++) {

                child = $(tds.children()[0]).children();

                if (child.length > 1) {
                    if (model.currentViewData[rowIndex].level == 0 && model.currentViewData[rowIndex].hasChildRecords)
                        proxy._tdsOffsetWidth[i] = tds.get(i).offsetWidth - child[0].offsetWidth - child[2].offsetWidth;
                    else
                        proxy._tdsOffsetWidth[i] = tds.get(i).offsetWidth - child[0].offsetWidth - child[1].offsetWidth;
                } else {
                    proxy._tdsOffsetWidth[i] = tds.get(i).offsetWidth;
                }
            }

        },


        //get Index of resourceInfo
        getIndexofresourceInfo: function (dataSource) {

            var proxy = this,
                data = proxy._cellEditingDetails.data,
                resourceInfo = data.resourceInfo,
                count = 0,
                length = resourceInfo && resourceInfo.length,
                resourceIndex = [];

            for (count; count < length; count++)
                resourceIndex.push(dataSource.indexOf(resourceInfo[count]));

            return resourceIndex;

        },

        //REFRESH THE EDITED FORM
        _refreshEditForm: function () {

            var proxy = this,
                form = document.getElementById(proxy._id + "EditForm"),
                elementFocused = false,
                $formElement = $(form).find("input,select"),
                percent = 86,
                i = 0,
                length = $formElement.length,
                $element, inputWidth,
                model = proxy.model,
                width,
                params = {},
                value,
                column,
                customParams,
                toformat,
                formatVal,
                rowHeight = proxy.model.rowHeight,
                cellEditType;

            for (i; i < length; i++) {

                $element = $formElement.eq(i);

                if (model.editSettings.editMode.toLowerCase() == "cellediting")
                    percent = 95;
                if (proxy._isRowEdit && $element.attr("type") != "checkbox")
                    inputWidth = "100%";
                else
                    inputWidth = proxy._tdsOffsetWidth[i] * (percent / 100);

                cellEditType = $element.attr("edittype");
                column = ej.TreeGrid.getColumnByField(model.columns, $element.prop("name"));

                switch (cellEditType) {

                    case ej.TreeGrid.EditingType.Numeric:
                        params = {};
                        width = inputWidth;
                        value = $element.val();
                        params.width = width;
                        params.height = rowHeight - 5;
                        params.showSpinButton = true;
                        params.cssClass = model.cssClass;
                  
                        if (value.length) params.value = parseFloat(value);

                        if (model.isFromGantt && (column.field === "duration" || column.field === "status"))
                        {
                            params.minValue = 0;
                            if (column.field == "status")
                                params.maxValue = 100;
                        }

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                        $element.ejNumericTextbox(params);
                        $element.prop("name", $element.prop("name").replace(proxy._id, ""));
                        break;

                    case ej.TreeGrid.EditingType.DatePicker:
                        params = {};
                        params.width = inputWidth;
                        params.height = rowHeight - 5;
                        params.locale = model.locale;
                        params.cssClass = model.cssClass;
                        params.dateFormat = model.dateFormat;
                        params.enableStrictMode = true;
                        params.change = function (args) {
                            this.option("value", args.value)
                        };
                        if ($element.val().length) params.value = proxy.getDateFromFormat($element.val());

                        if (column["format"] !== undefined && column.format.length > 0) {

                            toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                            formatVal = toformat.exec(column.format);
                            params.dateFormat = formatVal[2];
                            if ($element.val().length) {
                                params.value = Globalize.format(new Date($element.val()), params.dateFormat);
                                $element.val(params.value);
                            }
                        }

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                        if (model.dateFormat.indexOf("hh") == -1)
                            $element.ejDatePicker(params);
                        else {
                            $element.ejDateTimePicker(params);
                        }
                        break;
                    case ej.TreeGrid.EditingType.DateTimePicker:
                        params = {};
                        params.width = inputWidth;
                        params.height = rowHeight - 5;
                        params.locale = model.locale;
                        params.cssClass = model.cssClass;
                        params.displayDefaultDate = true;
                        params.dateTimeFormat = model.dateFormat;
                        params.enableStrictMode = true;                       
                        params.change = function (args) {
                            this.option("value", args.value)
                        };
                        if ($element.val().length) params.value = proxy.getDateFromFormat($element.val());

                        if (column["format"] !== undefined && column.format.length > 0) {

                            toformat = new RegExp("\\{0(:([^\\}]+))?\\}", "gm");
                            formatVal = toformat.exec(column.format);
                            params.dateTimeFormat = formatVal[2];
                            params.value = Globalize.format(new Date($element.val()), params.dateTimeFormat);
                            $element.val(params.value);
                        }

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                            $element.ejDateTimePicker(params);
                        break;
                    case ej.TreeGrid.EditingType.Dropdown:

                        var dataSource, fieldName;

                        if (proxy._isRowEdit)
                        {
                            dataSource = model.columns[i].dropdownData;
                            fieldName = model.columns[i].field;
                        }
                        else{
                            dataSource = model.columns[proxy._cellEditingDetails.columnIndex].dropdownData;
                            fieldName = model.columns[proxy._cellEditingDetails.columnIndex].field;
                        }

                        if(model.isFromGantt && fieldName === "resourceInfo"){
                            $element.ejDropDownList({
                                width: inputWidth + "px",
                                height: rowHeight - 5,
                                showCheckbox: true,
                                dataSource: dataSource,
                                fields: {
                                    id: model.resourceIdMapping,
                                    text: model.resourceNameMapping,
                                    value: model.resourceNameMapping
                                },
                                selectedItems: proxy.getIndexofresourceInfo(dataSource)
                            });
                        
                        } else {
                            var controlArgs = {};
                            controlArgs.cssClass = model.cssClass;
                            controlArgs.width = inputWidth;
                            controlArgs.height = rowHeight - 5;
                            controlArgs.dataSource = dataSource;
                            if (!ej.isNullOrUndefined(column["editParams"]))
                                $.extend(controlArgs, column["editParams"]);

                            $element.ejDropDownList(controlArgs);

                            var obj = $element.ejDropDownList("instance");
                            obj._setValue($element.attr("cellValue"));
                           
                        }
                        break;

                    case ej.TreeGrid.EditingType.String:
                        $element.outerWidth(inputWidth).height(model.rowHeight - 9);
                        break;
                    case ej.TreeGrid.EditingType.Boolean:
                        var controlArgs = {};
                        controlArgs.cssClass = model.cssClass;
                        controlArgs.size = "small";

                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(params, column["editParams"]);

                        $element.ejCheckBox(controlArgs);
                        ////center align of check box control
                        var parentElement = $element.parent(".e-chkbox-wrap"),
                            formElement = parentElement.parent("form");

                        if (formElement.length > 0)
                            formElement.css("margin-left", "45%");
                        else
                            parentElement.css("margin-left", "45%");
                       
                        break;
                   
                    case ej.TreeGrid.EditingType.Maskedit:
                        var controlArgs = {};
                        controlArgs.locale = model.locale;
                        controlArgs.cssClass = model.cssClass;
                        controlArgs.width = inputWidth;
                        controlArgs.height = rowHeight - 5;
                        if (!ej.isNullOrUndefined(column["editParams"]))
                            $.extend(controlArgs, column["editParams"]);

                        $element.ejMaskEdit(controlArgs);
                        break;
                }

                if (!$element.is(":disabled") && !elementFocused &&
                   (!$element.is(":hidden") || typeof ($element.data("ejDropDownList") || $element.data("ejNumericTextbox") ||
                   $element.data("ejDatePicker") || $element.data("ejCheckBox")) == "object")) {

                    if (!proxy._isEnterKeyPressed) {

                        proxy._focusElements($element.closest('td'));
                        elementFocused = true;

                    }
                }
            }
        },


        //INITIALIZE THE VALIDATOR
        _initValidator: function() {

            var proxy = this;

            $("#" + proxy._id + "EditForm").validate({
                onclick: false,
                onfocusout: false,
                errorClass: 'e-field-validation-error',
                errorElement: 'div',
                wrapper: "div",

                errorPlacement: function(error, element) {
                    var $td = element.closest("td"),
                        $container = $(error).addClass("e-error"),
                        $tail = ej.buildTag("div.e-errortail e-toparrow");

                    $td.find(".e-error").remove();
                    element.hasClass("e-numerictextbox") ? $container.insertAfter(element.closest(".e-numeric")) :
                                                          $container.insertAfter(element);
                    $container.prepend($tail);
                    proxy.model.editSettings.editMode != "normal" && $container.offset({
                        left: element.offset().left
                    });

                    $container.fadeIn("slow");
                }
            });
        },


        //SET FOCUS TO EDITED ELEMENT
        _focusElements: function ($currentCell) {
            var proxy = this;

            if ($currentCell.length) {

                $currentCell.focus();
                var $childElem = $currentCell.children();

                if ($childElem[0]) {

                    if ($childElem[0].tagName.toLowerCase() == "select" || 
                        $childElem[0].tagName.toLowerCase() == "input") {

                        $childElem.focus().select();
                        $childElem[0].focus();

                    } else if ($childElem.find(".e-field.e-dropdownlist").length) {

                        $childElem.find(".e-ddl").focus();

                    } else if ($childElem.find(".e-chkbox-wrap.e-widget").length) {

                        $childElem.find(".e-chkbox-wrap.e-widget").focus();

                    }
                    else {

                        $childElem.find('input,select').select().focus();
                    }
                }
            }
			   proxy._$gridHeaderContainer.scrollLeft(proxy._$gridContent.children('.e-content').scrollLeft());
        },
        //Set the focus for edit row elements
        _focusElementsForRowEdit: function ($currentCell) {
            if ($currentCell.length) {
                var $childElem = $currentCell.children();
                if ($childElem[0].tagName.toLowerCase() == "select" || $childElem[0].tagName.toLowerCase() == "input") {
                    $childElem.focus().select();
                    $childElem[0].focus();
                }
                else if ($childElem.find(".e-field.e-dropdownlist").length)
                    $childElem[0].focus();
                else
                    $childElem.find('input:visible,select').first().select().focus();
            }
        },
        //UPDATE THE GRID CONTAINER HEIGHT
        //By updatedRecords and CollapsedRecord count length
        updateHeight: function () {

            var proxy = this,
                model = this.model,
                height = 0,
                detailsRowHeight = 0;

            if (model.enableVirtualization) {
                height = model.updatedRecords.length * model.rowHeight;
            } else {
                height = model.rowHeight * (model.updatedRecords.length - proxy.getCollapsedRecordCount());
            }

            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn) {
                detailsRowHeight = proxy._getExpandedDetailsRowHeight();
            }

            height += detailsRowHeight;

            if (proxy._$gridContainer && proxy._$gridContent) {

                    proxy._$gridContainer.css({
                        "height": height + "px",
                        "width": "auto"
                    });

                if (proxy._$gridContent.hasClass("e-scroller")) {
                    proxy._$gridContent.ejScroller("refresh");
                    proxy._updateScrollCss();
                }
            }
        },
        //check scroller position in virtualization
        _setScrollTop: function () {

            var proxy = this,
            model = this.model,
            currentPosition = 0,
            top = proxy._scrollTop,
            rowHeight = model.rowHeight,
            length = model.currentViewData.length;

            /* check detail template and its row height */
            if (model.showDetailsRow && model.detailsTemplate && !model.showDetailsRowInfoColumn) {
                rowHeight += proxy._detailsRowHeight;
            }
            if (top !== 0 && !model.isFromGantt) {
                if(model.enableVirtualization){
                    if ((length * rowHeight) < proxy._viewPortHeight) {
                        if (length > 0) {
                            currentPosition = top - (proxy._viewPortHeight - ((length - 1) * rowHeight));
                            if (currentPosition < 0)
                                currentPosition = 0;
                        }
                        proxy._$gridContent.ejScroller("scrollY", currentPosition, true);
                    }
                }
                else
                    proxy._$gridContent.ejScroller("scrollY", top, true);
            }
        },

        getChildCount:function(record,count)
        {
            var currentRecord, proxy = this;
            if (!record.hasChildRecords)
                return 0;
            for (var i = 0; i < record.childRecords.length;i++)
            {
                currentRecord= record.childRecords[i];
                count++;
                if(currentRecord.hasChildRecords)
                {
                    count=proxy.getChildCount(currentRecord, count);
                }
            }
            return count;
        },
        //UPDATE THE FILTERED RECORDS FOR ADDING PARENTRECORDS
        _updateFilteredRecords: function (data) {

            var proxy = this,
                count = 0,
                length = data.length,
                record,virtualization=proxy.model.enableVirtualization;

            proxy._filteredRecords = [];
            proxy._filteredGanttRecords = [];

            for (count; count < length; count++) {

                record = data[count];

                if (record.parentItem != null) {

                    proxy._addParentRecord(record.parentItem);

                    if ($.inArray(record, proxy._filteredRecords) === -1) {
                        if (virtualization)
                        {
                            if (proxy.getExpandStatus(record))
                            {
                                proxy._filteredRecords.push(record);
                            }
                        }
                        else
                        {
                            proxy._filteredRecords.push(record);
                        }
                           
                    }

                } else {

                    if ($.inArray(record, proxy._filteredRecords) === -1) {
                        if (virtualization) {
                            if (proxy.getExpandStatus(record)) {
                                proxy._filteredRecords.push(record);
                            }
                        }
                        else {
                            proxy._filteredRecords.push(record);
                        }
                    }
                }
                if (!this.model.isFromGantt) {
                    if (record.hasChildRecords && record.childRecords.length > 0) {
                        if (!this._checkChildExsist(record, data)) {
                            record.hasFilteredChildRecords = false;
                        } else {
                            record.hasFilteredChildRecords = true;
                        }
                    }
                    if (record.parentItem) {
                        record.parentItem.hasFilteredChildRecords = true;
                    }
                }
            }
        },
        /* Check child records of filtered parent records exists or not in updated records*/
        _checkChildExsist: function (record, collection)
        {
            var childRecords = record.childRecords,
                length = childRecords.length, isExsist = false;
            for (var count = 0; count < length; count++) {
                if ($.inArray(childRecords[count], collection) != -1) {
                    isExsist = true;
                    break;
                } else if (childRecords[count].hasChildRecords) {
                    isExsist = this._checkChildExsist(childRecords[count], collection);
                }
            }
            return isExsist;
        },
        /* While clear the filter, update hasFilteredChildRecords status of all records*/
        _updateHasFilteredChildRecordsStatus: function (collection)
        {
            var count=0,length=collection.length,currentRecord;
            for (count; count < length; count++) {
                currentRecord = collection[count];
                if (currentRecord.hasChildRecords && !currentRecord.hasFilteredChildRecords)
                    currentRecord.hasFilteredChildRecords = true;
            }
        },
        //ADD PARENTRECORD FOR FILTERED OTH LEVEL ITEMS
        _addParentRecord: function (record) {

            var proxy = this,
            virtualization = proxy.model.enableVirtualization;

            if (record.parentItem) {
                proxy._addParentRecord(record.parentItem);

                if ($.inArray(record, proxy._filteredRecords) === -1) {

                    if (virtualization) {
                        if (proxy.getExpandStatus(record)) {
                            proxy._filteredRecords.push(record);
                        }
                    }
                    else {
                        proxy._filteredRecords.push(record);
                    }
                     

                }
            } else {

                if ($.inArray(record, proxy._filteredRecords) === -1) {

                    if (virtualization) {
                        if (proxy.getExpandStatus(record)) {
                            proxy._filteredRecords.push(record);
                        }
                    }
                    else {
                        proxy._filteredRecords.push(record);
                    }

                }
            }
        },


        //UPDATEEXPANDCOLLAPSE FOR SORTED RECORDS
        _updateExpandCollapseForSortedRecords: function (Record, expanded) {

            var proxy = this,
                count = 0,
                length = proxy._sortedRecords.length,
                record,
                index = proxy._sortedRecords.indexOf(Record);

            if (expanded) {

                proxy._addNestedSortRecords(Record, index);

            } else {

                var removedCount = 0;

                for (count = index + 1; count < length; count++) {

                    record = proxy._sortedRecords[count];

                    if (record.level === Record.level) {
                        break;
                    } else if (!proxy.getExpandStatus(record)) {
                        removedCount++;
                    }

                }

                proxy._sortedRecords.splice(index + 1, removedCount);

            }
        },


        //ADD NESTEDSORTRECORDS
        _addNestedSortRecords: function (record, index) {

            var proxy = this,
                childRecords,
                sortColumns = proxy.model.sortSettings.sortedColumns,
                length;

            if (sortColumns.length) {

                length = sortColumns.length - 1;
                proxy._queryManagar.queries = [];

                for (var i = length; i >= 0; i--) {

                    proxy._queryManagar.sortBy(sortColumns[i].field, sortColumns[i].direction);

                }

            }
            if (record.hasChildRecords) {

                var dataManager = ej.DataManager(record.childRecords),
                    count,
                    childrecord;

                childRecords = dataManager.executeLocal(proxy._queryManagar).result;
                length = childRecords.length;

                for (count = 0; count < length; count++) {

                    childrecord = childRecords[count];

                    if (proxy._filteredRecords.length) {

                        if ($.inArray(childrecord, proxy._filteredRecords) !== -1) {
                            proxy._sortedRecords.splice(++index, 0, childrecord);
                        }

                    } else {
                        proxy._sortedRecords.splice(++index, 0, childrecord);
                    }

                    if (childrecord.hasChildRecords && childrecord.expanded) {
                        proxy._addNestedSortRecords(childrecord, index);
                    }
                }
            }
        },

        //UPDATE Edit Settings
        updateEditSettings: function (edit) {

            var proxy = this;

            if (edit) {
                if (edit.editMode)
                    proxy.model.editSettings.editMode = edit.editMode;
                if (edit.allowAdding)
                    proxy.model.editSettings.allowAdding = edit.allowAdding;
                if (edit.allowEditing)
                    proxy.model.editSettings.allowEditing = edit.allowEditing;
                if (edit.allowDeleting)
                    proxy.model.editSettings.allowDeleting = edit.allowDeleting;
            }
        },

        //UPDATE ALTROW
        updateAltRow: function () {

            var proxy = this,
                count = 0,
                currentViewData = proxy.model.updatedRecords,
                length = currentViewData.length,
                record;

            for (count = 0; count < length; count++) {
                record = currentViewData[count];
                if (record.isSummaryRow)
                    continue;
                record.isAltRow = count % 2 == 0 ? true : false;
            }

        },


        
        //COUNT THE NUMBER OF CHILD RECORDS PRESENT IN THE GANTTRECORD
        _getFilteredChildRecordsCount: function (Record) {

            var proxy = this,
                model=proxy.model,
                length = Record&&Record.childRecords && Record.childRecords.length,
                i = 0,
                record;

            for (i = 0; i < length; i++) {

                record = Record.childRecords[i];
                if (model.enableVirtualization)
                {
                    if(proxy.getExpandStatus(record))
                    {
                        proxy._removedCount++;
                    }
                }
                else
                {
                    proxy._removedCount++;
                }
                   
                if (record.childRecords) {
                    proxy._getFilteredChildRecordsCount(record);
                }
            }

            return proxy._removedCount;
        },


        //SORTING THE RECORDS
        _sortingRecords: function (args) {

            var proxy = this,
                model = proxy.model,
                sortcolumns = model.sortSettings.sortedColumns;

            proxy._queryManagar.queries = [];

            if (sortcolumns.length) {

                var length = sortcolumns.length - 1;

                for (var i = length; i >= 0; i--) {
                    var fieldName = sortcolumns[i].field;
                    if (model.isFromGantt) {
                            if (fieldName === "resourceInfo")
                                fieldName = "resourceNames";
                            else if (fieldName === "predecessor")
                                fieldName = "predecessorsName";
                    }

                    proxy._queryManagar.sortBy(fieldName, sortcolumns[i].direction);
                }
            }

            var dataManager = new ej.DataManager(model.parentRecords),
                records = dataManager.executeLocal(proxy._queryManagar).result;

            proxy._sortedRecords = [];
            proxy._storedIndex = -1;
            proxy._rowIndex = -1;

            proxy._createSortedRecords(records);
            proxy._tempsortedrecords = proxy._sortedRecords;
            proxy.model.updatedRecords = proxy._sortedRecords;            

            return true;
        },


        /**
        * Create sorted collection argumented data
        * @param {data} collection of records to sorting
        */
        _createSortedRecords: function (data) {

            var proxy = this, model = proxy.model,
                sortedRecords = proxy._sortedRecords,
                dataManager,
                enableAltRow = proxy.model.enableAltRow,model=this.model;
            data = $(data).not(model.summaryRowRecords).get();
            $.each(data, function (index, record) {
                //Skip the summary record from sorting.
                if (record.isSummaryRow) {
                    var recordIndex = record.parentItem.childRecords.indexOf(record);
                    record.parentItem.childRecords.splice(recordIndex, 1);
                    return;
                }
                if (proxy._searchString.length > 0 || (model.filterSettings.filteredColumns.length > 0)) {

                    if (proxy._filteredRecords.length > 0 && proxy._filteredRecords.indexOf(record) !== -1) {

                        proxy._storedIndex++;
                        proxy._rowIndex++;

                        if (enableAltRow) {

                            record.isAltRow = proxy._rowIndex % 2 == 0 ? false : true;

                        }

                        sortedRecords[proxy._storedIndex] = record;

                    }
                } else {

                    proxy._storedIndex++;
                    proxy._rowIndex++;

                    if (enableAltRow) {

                        record.isAltRow =proxy._rowIndex % 2 == 0 ? false : true;

                    }

                    sortedRecords[proxy._storedIndex] = record;

                }

                if (record.hasChildRecords && record.expanded) {
                    record.childRecords = $(record.childRecords).not(model.summaryRowRecords).get();
                    dataManager = ej.DataManager(record.childRecords);
                    var childRecords = dataManager.executeLocal(proxy._queryManagar).result;
                    proxy._createSortedRecords(childRecords);

                }
                else if (record.hasChildRecords && !record.expanded && !model.enableVirtualization) {
                    record.childRecords = $(record.childRecords).not(model.summaryRowRecords).get();
                    proxy._setChildRecords(record, sortedRecords);
                }
            });
        },

        _setChildRecords:function(record,sortedRecords)
        {
            var proxy = this,
                model = proxy.model,
            enableAltRow = proxy.model.enableAltRow;
            var length = record.childRecords.length;

            dataManager = ej.DataManager(record.childRecords);
            var childRecords = dataManager.executeLocal(proxy._queryManagar).result;
            for(var count=0;count<length;count++)
            {
                if (proxy._searchString.length > 0 || (model.filterSettings.filteredColumns.length > 0)) {

                    if (proxy._filteredRecords.length > 0 && proxy._filteredRecords.indexOf(childRecords[count]) !== -1) {
                        proxy._storedIndex++;
                        if (enableAltRow) {
                            childRecords[count].isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                        }
                        sortedRecords[proxy._storedIndex] = childRecords[count];
                        if (childRecords[count].hasChildRecords) {
                            this._setChildRecords(childRecords[count], sortedRecords);
                        }
                    }
                } else
                {
                    proxy._storedIndex++;
                    if (enableAltRow) {
                        childRecords[count].isAltRow = proxy._storedIndex % 2 == 0 ? false : true;
                    }
                    sortedRecords[proxy._storedIndex] = childRecords[count];
                    if (childRecords[count].hasChildRecords) {
                        this._setChildRecords(childRecords[count], sortedRecords);
                    }
                }
            }
        },
        //UPDAT THE SORTED RECORDS
        _updateSortedRecords: function (data) {

            var proxy = this,
                count = 0,
                length = data.length,
                record,model=this.model;

            for (count; count < length; count++) {

                record = data[count];

                if (!model.enableVirtualization) {

                    if (proxy._filteredRecords.length > 0) {

                        if ($.inArray(record, proxy._filteredRecords) !== -1 && $.inArray(record, proxy._sortedRecords) === -1) {
                            proxy._sortedRecords.push(record);
                        }

                    } else {

                        if ($.inArray(record, proxy._sortedRecords) === -1) {
                            proxy._sortedRecords.push(record);
                        }

                    }

                    if (record.hasChildRecords && record.childRecords.length > 0) {

                        var dataManager = ej.DataManager(record.childRecords),
                            sortedChildRecords = dataManager.executeLocal(proxy._queryManagar).result;
                        proxy._updateSortedRecords(sortedChildRecords);

                    }
                }
            }
        },


        //REMOVE SORT ICON FORM COLUMN HEADER
        _removeSortElementFromColumn: function (field) {

            var proxy = this,
                column = ej.TreeGrid.getColumnByField(proxy.model.columns, field),
                index = $.inArray(column, proxy.model.columns),
                $headerCellDiv = proxy.getHeaderTable().find("thead").find(".e-headercell").eq(index).find(".e-headercelldiv");

            $headerCellDiv.find(".ascending,.descending").remove();
            $headerCellDiv.parent().removeAttr("aria-sort");
        },


        //ADD EMPTYCOLUMN TO THE TREEGRID
        _addEmptyColumntoGrid: function () {

            var proxy = this;
            $(proxy.element).find('.e-gridheadercontainer table colgroup').append(ej.buildTag('col', "", {
                "width": "auto"
            },
            {}));

            $(proxy.element).find('.e-gridheadercontainer table thead tr').append(ej.buildTag('th.e-headercell'));
        },


        //UPDATE THE SCROLLTOP OF THE GRIDCONTAINER
        _updateScrollTop: function (y) {

            var args = {},
                proxy = this;

            args.requestType = "scroll";
            args.delta = y;

            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");

            if (!isHorizontalScroll) {
                proxy._$gridContent.scrollTop(y);//grid container
            }
            else
            {
                proxy._$gridContent.scrollTop(0)
            }
            proxy._$gridContent.ejScroller("scrollY", y, true);
            proxy._completeAction(args);
            proxy.element[0].focus();
            
        },


        //MOVE CURRENT CELL FOR KEYBOARD INTERACTION
        _moveCurrentCell: function (direction) {

            var proxy = this,
                rowIndex = proxy._cellEditingDetails.rowIndex,
                columnIndex = proxy._cellEditingDetails.columnIndex,
                model = proxy.model;

            if (rowIndex == -1 && columnIndex == -1) return false;

            if (proxy.model.isEdit) {

                proxy.saveCell();
                proxy.model.isEdit = false;

            }
            if (model.enableVirtualization) {
                var record = model.currentViewData[rowIndex];
                proxy._cellEditingDetails.rowIndex = rowIndex = model.updatedRecords.indexOf(record);
            }
            switch (direction) {

                case "right":
                    var columns = model.columns,
                        columnLength = columns.length,
                        hiddenColumn = proxy.getHiddenColumns(),
                        visibleColumn = $.extend([], columns),
                        nextColumnIndex, newColumnIndex, visibleColumnLength;
                    if(hiddenColumn)
                    hiddenColumn.forEach(function (item) {
                        visibleColumn.splice(visibleColumn.indexOf(item), 1);
                    });
                    newColumnIndex = visibleColumn.indexOf(columns[columnIndex]);
                    visibleColumnLength = visibleColumn.length;
                    if (rowIndex == model.updatedRecords.length - 1 && newColumnIndex == visibleColumnLength - 1) {
                        return true;
                    }
                    if (newColumnIndex == visibleColumnLength - 1) {
                        nextColumnIndex = 0;
                        while (!columns[nextColumnIndex].visible) {
                            nextColumnIndex++;
                        }
                        proxy._cellEditingDetails.columnIndex = nextColumnIndex;
                        if(proxy._getNextRow(direction, rowIndex, proxy))
                            return true;
                        
                    }
                    else {
                        nextColumnIndex = columnIndex + 1;
                        while (!columns[nextColumnIndex].visible) {
                            nextColumnIndex++;
                        }
                        proxy._cellEditingDetails.columnIndex = nextColumnIndex;
                    }
                    while (model.columns[nextColumnIndex].allowEditing == false || !columns[nextColumnIndex].visible) {
                        if ((nextColumnIndex + 1) == model.columns.length) {
                            proxy._cellEditingDetails.columnIndex = nextColumnIndex = 0;
                            if (proxy._getNextRow(direction, rowIndex, proxy))
                                return true;
                            continue;
                        }
                        proxy._cellEditingDetails.columnIndex = ++nextColumnIndex;
                    }
                    proxy.updateScrollBar();
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex,
                        model.columns[proxy._cellEditingDetails.columnIndex].field);
                    proxy._editAddTools();
                    break;

                case "left":
                    var columns = model.columns,
                       columnLength = columns.length,
                       hiddenColumn = proxy.getHiddenColumns(),
                       visibleColumn = $.extend([], columns),
                       prevColumnIndex, newColumnIndex, visibleColumnLength;
                    if (hiddenColumn)
                        hiddenColumn.forEach(function (item) {
                            visibleColumn.splice(visibleColumn.indexOf(item), 1);
                        });
                    newColumnIndex = visibleColumn.indexOf(columns[columnIndex]);
                    visibleColumnLength = visibleColumn.length;
                    if ((rowIndex == 0 && newColumnIndex == 0)) return true;
                    if (newColumnIndex == 0) {
                        prevColumnIndex = model.columns.length - 1;
                        while (!columns[prevColumnIndex].visible) {
                            prevColumnIndex--;
                        }
                        proxy._cellEditingDetails.columnIndex = prevColumnIndex;
                        if(proxy._getNextRow(direction, rowIndex, proxy))
                            return true;
                    } else {
                        prevColumnIndex = columnIndex - 1;
                        while (!columns[prevColumnIndex].visible) {
                            prevColumnIndex--;
                        }
                        proxy._cellEditingDetails.columnIndex = prevColumnIndex;
                    }

                    while (model.columns[prevColumnIndex].allowEditing == false || !columns[prevColumnIndex].visible) {

                        if ((prevColumnIndex - 1) == -1) {
                            proxy._cellEditingDetails.columnIndex = prevColumnIndex = columnLength - 1;
                            if (proxy._getNextRow(direction, rowIndex, proxy))
                                return true;
                            continue;
                        }
                        proxy._cellEditingDetails.columnIndex = --prevColumnIndex;
                    }
                    proxy.updateScrollBar();
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                    proxy._editAddTools();
                    break;

                case "up":
                    if (rowIndex == 0) return true;
                    proxy._cellEditingDetails.rowIndex = rowIndex - 1;
                    proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                    proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
                    proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                    break;

                case "down":
                    if (rowIndex + 1 == proxy.model.updatedRecords.length) {
                        proxy.element.focus();
                        proxy._isEnterKeyPressed = true;
                    }
                    if (model.showSummaryRow) {
                        if (rowIndex + 1 == model.updatedRecords.length - model.summaryRows.length)
                            proxy._isEnterKeyPressed = true;
                    }
                    if (!proxy._isEnterKeyPressed) {
                        if (proxy.model.updatedRecords.length > (rowIndex + 1))
                            if (!model.enableVirtualization) {
                                var currentRecord = model.updatedRecords[rowIndex],
                                    expandedRecords = proxy.getExpandedRecords(model.updatedRecords),
                                    nextRecord = expandedRecords[expandedRecords.indexOf(currentRecord) + 1];
                                //Skip the summary row from keyboard navigation.
                                if (nextRecord && nextRecord.isSummaryRow)
                                    nextRecord = proxy._getNextRecord(rowIndex + 1, expandedRecords);
                                var index = model.updatedRecords.indexOf(nextRecord);
                                if ((rowIndex == expandedRecords.length - 1 && columnIndex == model.columns.length - 1) || index == -1) {
                                    return true;
                                }
                                proxy._cellEditingDetails.rowIndex = index;
                            }
                            else {
                                var nextRecord = model.updatedRecords[rowIndex + 1];
                                //Skip the summary row from keyboard navigation.
                                if (nextRecord && nextRecord.isSummaryRow)
                                    nextRecord = proxy._getNextRecord(rowIndex + 1, model.updatedRecords);
                                index = model.updatedRecords.indexOf(nextRecord);
                                if (index == -1)
                                    return true;
                                proxy._cellEditingDetails.rowIndex = index;
                            }
                        if (!proxy._rowSelectingEventTrigger(proxy.selectedRowIndex(), proxy._cellEditingDetails.rowIndex)) {
                            proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                            proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
                        }
                        proxy.updateScrollBar();
                        args = {};
                        args.requestType = "scroll";
                        args.delta = proxy._$gridContent.children('.e-content').scrollTop();
                        proxy._trigger("actionComplete", args);
                        proxy.cellEdit(proxy._cellEditingDetails.rowIndex, model.columns[proxy._cellEditingDetails.columnIndex].field);
                        proxy._editAddTools();
                    }
                    else
                        proxy._isEnterKeyPressed = false;
                    break;
            }

            return false;

        },
        //To skip the summary record. 
        _getNextRecord: function (index, expandedRecords, action) {
            do {
                nextRecord = expandedRecords[index];
                if (action == "upArrow")
                    index--
                else
                    index++;
            } while (nextRecord && nextRecord.isSummaryRow);
            return nextRecord;
        },

        _getNextRow: function (direction, rowIndex, proxy) {
            var cellIndex,
            model = proxy.model;
            if (direction == "right")
                cellIndex = 1;
            else
                cellIndex = -1;

            if (!model.enableVirtualization) {
                var currentRecord = model.updatedRecords[rowIndex],
                  expandedRecords = proxy.getExpandedRecords(model.updatedRecords),
                  nextRecord = expandedRecords[expandedRecords.indexOf(currentRecord) + cellIndex];
                //Skip the summary row from keyboard navigation.
                if (nextRecord && nextRecord.isSummaryRow)
                    nextRecord = proxy._getNextRecord(rowIndex + cellIndex, expandedRecords);
                index = model.updatedRecords.indexOf(nextRecord);
                if(index == -1) {
                    return true;
                }
                proxy._cellEditingDetails.rowIndex = index;
            }
            else {
                var nextRecord = model.updatedRecords[rowIndex + cellIndex];
                //Skip the summary row from keyboard navigation.
                if (nextRecord && nextRecord.isSummaryRow)
                    nextRecord = proxy._getNextRecord(rowIndex + cellIndex, model.updatedRecords);
                index = model.updatedRecords.indexOf(nextRecord);
                if (index == -1)
                    return true;
                proxy._cellEditingDetails.rowIndex = index;
            }
            if (!proxy._rowSelectingEventTrigger(proxy.selectedRowIndex(), proxy._cellEditingDetails.rowIndex)) {
                proxy.selectRows(proxy._cellEditingDetails.rowIndex);
                proxy._rowSelectedEventTrigger(proxy._cellEditingDetails.rowIndex);
            }

        },

        //FIND THE COLUMNWIDTH OF THE TREEGRID COLUMN
        _findColumnsWidth: function (extra) {

            var proxy = this,
                length = proxy.model.columns.length,
                j = proxy.getHeaderTable().find("colgroup").find("col");
            
            for (var i = 0; i < length; i++)
                proxy._columnsWidthCollection[i] = j.eq(i).width();

        },

        //CALCULATE THE TOTAL WIDTH OF THE COLUMNS
        _calculateWidth: function () {

            var j = this.getHeaderTable().find("colgroup").find("col"),
                width = 0;

            for (var i = 0; i < j.length; i++)
                width += j.eq(i).width();

            return width;
        },

        //update element height when viewport height is <0
        _updateElementHeight:function()
        {
            var proxy = this;
            proxy.element.height(450);
            proxy._calculateDimensions();
            proxy.updateViewPortHeight();
        },
        _renderScroller: function () {

            var proxy = this,
            model = proxy.model;
            var containerHeight = model.flatRecords.length * model.rowHeight;
            var contentHeight = proxy._viewPortHeight;

            if (!proxy._$gridContent.hasClass("e-scroller")) {
                var height = proxy._gridHeight ? contentHeight : "auto";
                var width = proxy.element.width();

                proxy._$gridContent.ejScroller({
                    enableTouchScroll: false,
                    height:proxy.model.isFromGantt ? 0 :height,
                    width: proxy.model.isFromGantt ? width : width - proxy._totalBorderWidth,
                    scroll: $.proxy(proxy._onScroll, proxy)
                });
            }
            else {
                proxy._$gridContent.ejScroller("refresh");
            }

            proxy._$gridContent.ejScroller("model.keyConfigs", { up: "", down: "" });
            proxy._updateScrollCss();
            var isHorizontalScroll = proxy._$gridContent.ejScroller("isHScroll");
            if (proxy.model.isFromGantt)
                if (!isHorizontalScroll) {
                    proxy._$gridContent.addClass("e-borderbox");
                }
                else {
                    proxy._$gridContent.removeClass("e-borderbox");
                }

        },
        

        
        //ALL EVENTS BOUND USING THIS._ON WILL BE UNBIND AUTOMATICALLY
        _destroy: function () {

            var proxy = this;
            proxy.element.empty().removeClass("e-treegrid-core e-treegrid " + proxy.model.cssClass);
            
        },

        

        collapseAll: function () {
            var proxy = this,
                model = proxy.model,
                count = 0,
                ganttRecord,
                args = {},
                enableVirtualization = model.enableVirtualization,
                $gridRows = proxy.getRows(),
                $rowElement = null;

            
            //To cancel the cell edit or row edit mode when collapse the all record.
            if (proxy._isRowEdit || proxy.model.isEdit) {
                proxy.cancelRowEditCell();
                proxy.selectRows(-1);                          
                proxy._cancelSaveTools();
            }
            args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
            //loop excecute for iterate the GanttRecords preseent in the Gantt Control
            proxy._isInExpandCollapseAll = true;

            for (count = 0; count < proxy.model.parentRecords.length; count++) {
                treegridRecord = proxy.model.parentRecords[count];
                args.data = treegridRecord;
                args.recordIndex = count;
                args.expanded = false;
                if (treegridRecord.hasChildRecords) {
                    proxy._expandCollapseInnerLevelRecord(treegridRecord, args.expanded);
                    ej.TreeGrid.sendExpandCollapseRequest(proxy, args);
                }
            }
            if (model.flatRecords.length > 0)
                proxy._refreshTreeGridOnExpandCollapseAll(args);

            proxy._isInExpandCollapseAll = false;

        },
        //populate the context menu with items
        _getContextMenuItems: function () {
            var contextMenuItems = [];
            proxy = this,
            contextMenuLabel = proxy._contextMenuTexts;
            contextMenuItems = [ {
                headerText: contextMenuLabel["addRowText"],
                eventHandler: null,
                isDefault: true,
                disable: false,
                menuId: "Add"
            },
             {
                 iconClass: "e-edit",
                 headerText: contextMenuLabel["editText"],
                 eventHandler: null,
                 isDefault: true,
                 isEdit: true,
                 disable: false,
                 menuId: "Edit"
             },
            {
                headerText: contextMenuLabel["deleteText"],
                eventHandler: null,
                isDefault: true,
                disable: false,
                menuId: "Delete"
            },
            {
                iconClass: "e-save",
                headerText: contextMenuLabel["saveText"],
                eventHandler: null,
                isDefault: true,
                isEdit: true,
                disable: false,
                menuId: "Save"
            },
             {
                 iconClass: "e-cancel",
                 headerText: contextMenuLabel["cancelText"],
                 eventHandler: null,
                 isDefault: true,
                 isEdit: true,
                 disable: false,
                 menuId: "Cancel"
             }
            ];
            return contextMenuItems;
        },

        //populate sub context menu items
        _getsubContextMenuItems: function () {
            var contextMenuItems = [];
            proxy = this,
            contextMenuLabel = proxy._contextMenuTexts;
            contextMenuItems = [{
                iconPath: null,
                headerText: contextMenuLabel["aboveText"],
                eventHandler: null,
                menuId: "Above"
            }, {
                iconPath: null,
                headerText: contextMenuLabel["belowText"],
                eventHandler: null,
                menuId: "Below"
            }
            ];
            return contextMenuItems;
        },
        _clearContextMenu: function () {
            //remove context menu items
            $('.e-contextmenu').remove();
            $('.e-innerContextmenu').remove();
        },
        // create and append a context menu and also bind the events for context menu option
        _renderContextMenu: function (e, index, item) {

            var proxy = this,
            model = proxy.model,
                eventArgs = {},
                args = {},
                posx,
				subposx,
                posy,
                subposx,
                contextMenu,
                contextMenuUList,
                subContextMenu,
                subContextMenuUList;
            proxy._contextMenuSelectedIndex = index;
            proxy._contextMenuSelectedItem = item;
            proxy._contextMenuEvent = e;
            if (!e) e = window.event;
            if (e.pageX || e.pageY) {
               subposx = posx = e.pageX;
                posy = e.pageY;
            } else if (e.clientX || e.clientY) {
                subposx = posx = e.clientX + document.body.scrollLeft
                    + document.documentElement.scrollLeft;
                posy = e.clientY + document.body.scrollTop
                    + document.documentElement.scrollTop;
            }
            //Edge detection for context menu
            var treeGridOffset = this.getOffsetRect(proxy.element[0]);
            treeGridOffset.bottom = treeGridOffset.top + this.element[0].offsetHeight;
            treeGridOffset.right = treeGridOffset.left + $(this.element).width();
           
            args.targetElement = e;
            args.item = item;
            args.index = index;
            args.columnIndex = proxy.getCellIndex(e);
            args.requestType = "ContextMenuOpen";
            args.contextMenuItems = proxy._contextMenuItems;
            args.innerContextMenuItems = proxy._subContextMenuItems;
            proxy._trigger("contextMenuOpen", args);
            contextMenu = ej.buildTag("div.e-contextmenu", "", {
                'display': 'table',
                'position': 'absolute',
                'border': '1px solid #bbbcbb',
                'z-index': '10033',
            }, {"id":proxy._id+"_ContextMenu"});

            contextMenuUList = ej.buildTag("ul", $.render[proxy._id + "contextMenuTemplate"](args.contextMenuItems), {
                'margin': '0px',
                'padding-left': '0px'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });
            
            subContextMenu = ej.buildTag("div.e-innerContextmenu", "", {
                'display': 'table',
                'position': 'absolute',
                'border': '1px solid #bbbcbb',
                'z-index': '10',
                'visibility': 'hidden',
            }, {"id":proxy._id+"_SubContextMenu"});

            subContextMenuUList = ej.buildTag("ul", $.render[proxy._id + "subContextMenuTemplate"](args.innerContextMenuItems), {
                'margin': '0px',
                'padding-left': '0px'
            }, {
                'data-icon': false,
                'type': 'none',
                'data-role': 'list-divider'
            });
            contextMenu.append(contextMenuUList);
            $(document.body).append(contextMenu);

            if (args.innerContextMenuItems.length > 0) {
                subContextMenu.append(subContextMenuUList);
                $(document.body).append(subContextMenu);
            }

            var contextMenuWidth = $(contextMenu).width(),
                contextMenuHeight = $(contextMenu).height(),
                subContextMenuWidth = $(subContextMenu).width(),
                contextMenuItems = contextMenu.find(".e-menuitem"),
                subContextMenuItems = subContextMenu.find(".e-menuitem"),
                contextMenuSelectItem = contextMenuItems.eq(0);

            if (treeGridOffset.left > posx || (treeGridOffset.right < (posx + contextMenuWidth))) {
                if (treeGridOffset.right < (posx + contextMenuWidth)) {
                    posx = posx - contextMenuWidth;
                    subposx = posx - subContextMenuWidth - 2;
                }
                if (posx < 0) {
                    posx = treeGridOffset.left + 10;
                    subposx = posx + contextMenuWidth + 2;
                }

            } else {
                subposx = subposx + contextMenuWidth + 2;
                if (treeGridOffset.right < (subposx + contextMenuWidth)) {
                    var tempSubPosX = posx - subContextMenuWidth - 2;
                    if (tempSubPosX > 0)
                        subposx = tempSubPosX;
                }
            }

            if (treeGridOffset.bottom < posy + (30 * proxy._contextMenuItems.length)) {
                var tempPosY = posy - (30 * proxy._contextMenuItems.length);
                if (tempPosY > 0) {
                    posy = tempPosY;
                }
            }
            proxy._subPosx = subposx;           
            contextMenu.css({ 'left': posx + 'px', 'top': posy + 'px', });
            subContextMenu.css({ 'top': posy + (proxy._contextAddMenuIndex * 30) + 'px', })
            contextMenuItems.css({ 'width': contextMenuWidth + 2 });
            subContextMenuItems.css({ 'width': subContextMenuWidth });
            $(proxy.element).focus();
            $(contextMenu).css({ 'height': 'auto' });
            proxy._on(contextMenu, "contextmenu", function (argsE) {
                argsE.preventDefault();
            });
            //click event handlers for contextmenu items and subcontextmenu items
            $('.e-menuitem').click(function () {
                var choice = $(this).attr('id');
                if(!$(this).hasClass("e-disable")){
                switch (choice) {
                    case "Delete":
                        proxy._contextMenuOperations("Delete");
                        break;
                    case "Above":
                        proxy._contextMenuOperations("Above");
                        break;
                    case "Below":
                        proxy._contextMenuOperations("Below");
                        break;
                    case "Add":
                        proxy._contextMenuOperations("Add");
                        break;
                    case "Edit":
                        proxy._contextMenuOperations("Edit");
                        break;
                    case "Save":
                        proxy._contextMenuOperations("Save");
                        break;
                    case "Cancel":
                        proxy._contextMenuOperations("Cancel");
                        break;
                    default:
                        proxy._contextMenuOperations(choice);
                        break;
                } 
                }
            });

            if (!contextMenuSelectItem.hasClass("e-disable"))
                contextMenuSelectItem.addClass("e-contextmenu-mouseover");
            else
            {
                contextMenuSelectItem = proxy._findContextMenuItem(contextMenuSelectItem, "next");
                if(contextMenuSelectItem.length > 0)
                    contextMenuSelectItem.addClass("e-contextmenu-mouseover");
            }
            subContextMenuItems.eq(0).addClass("e-contextmenu-mouseover");
            //Mouseenter and mouse leave handler for contextmenu items and Subcontextmenuitems
            $(contextMenuItems).mouseenter(function () {
                if (!$(this).hasClass("e-disable")) {
                    contextMenu.find(".e-contextmenu-mouseover").removeClass("e-contextmenu-mouseover");
                    $(this).addClass("e-contextmenu-mouseover");
                    if ($(this).attr('id') == "Add") {
                        $(subContextMenu).css({ 'visibility': 'visible', 'left': subposx });
                        subContextMenuItems.removeClass("e-contextmenu-mouseover");
                        subContextMenuItems.eq(0).addClass("e-contextmenu-mouseover");
                        $(contextMenu).find('#Add').addClass("e-contextmenu-mouseover");
                    }
                    else {
                        $(subContextMenu).css({ 'visibility': 'hidden' });
                    }
                }
            });
            $(subContextMenuItems).mouseenter(function () {
                $("#" + proxy._id + "_SubContextMenu").find(".e-contextmenu-mouseover").removeClass("e-contextmenu-mouseover");
                $(this).addClass("e-contextmenu-mouseover");
            });
            e.preventDefault();
        },

        //Call custom context menu event handlers
        _triggerMenuEventHandler: function (menuId, eventArgs) {
            var proxy = this,
                contextMenuItems = proxy._contextMenuItems,
                currentMenuItemCollection, menuItem;

            if (!ej.isNullOrUndefined(contextMenuItems)) {
                currentMenuItemCollection = contextMenuItems.filter(function (value) {
                    //convert menu id to string if it is given as number
                    if (value.menuId.toString() === menuId)
                        return true;
                });

                if (currentMenuItemCollection.length > 0) {
                    menuItem = currentMenuItemCollection[0];
                    var fn = menuItem.eventHandler;
                    if (fn) {
                        if (typeof fn === "string") {
                            fn = ej.util.getObject(fn, window);
                        }
                        if ($.isFunction(fn)) {
                            args = ej.event("customContextMenuHandler", this.model, eventArgs);
                            fn.call(this, args);
                        }
                    }

                }
            }
        },

        _createContextMenuTemplate: function () {
            var proxy = this;
            var helpers = {};
            helpers["_" + proxy._id + "getHeaderName"] = $.proxy(proxy._getHeaderName, proxy);

            $.views.helpers(helpers);

            var menuItemList = "<li style='list-style-type:none;margin:0px;'>";
            var listChild = "<div class='e-menuitem {{if disable}}e-disable{{/if}}' id={{:menuId}}  style='display:table;cursor:{{if disable}}default{{else}}pointer{{/if}};'>" +
                            "{{if !isEdit}}" +
                           "<div class='e-icon' style='display:inline-block;{{if !iconPath}}padding-left:5px;{{else}}margin-left:8px{{/if}};" +
                            "background-image:{{:iconPath}};background-repeat:no-repeat;'/> " + 
                            "{{else}}" +
                            "<span class='e-icon {{:iconClass}}' style='display:inline-block;padding:0px 5px 0px 7px;'/>" +
                            "{{/if}}" +
                            "<div style='display:inline-block;padding:5px'>" +
                            "<span style='font-family:Segoe UI;font-size:12px;padding:5px;'>{{:headerText}}</span></div>" +
                            "{{if ~_" + proxy._id + "getHeaderName(#data)}}<div class='e-icon e-expander' style='display:inline-block;padding:5px;float:right;'/> {{/if}}";                            

            menuItemList += listChild;
            menuItemList += "</div></li>";
            var templates = {};
            templates[proxy._id + "contextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },
        _createSubContextMenuTemplate: function () {
            var proxy = this;

            var menuItemList = "<li style='list-style-type:none;margin:0px;'>";
            var listChild = "<div class='e-menuitem' id={{:menuId}}  style='display:table;cursor:pointer;'>" +
                            "<div style='display:inline-block;padding:5px 60px 5px 20px;'>" +
                            "<span style='font-family:Segoe UI;font-size:12px;padding:5px;'>{{:headerText}}</span></div>";


            menuItemList += listChild;
            menuItemList += "</li>";

            var templates = {};
            templates[proxy._id + "subContextMenuTemplate"] = menuItemList;
            $.templates(templates);
        },
        _getHeaderName: function (data) {
            if (data.menuId == "Add" && this._subContextMenuItems.length > 0) {
                return true;
            }
            return false;
        },

        // Remove the speace from column menu name for chaning name to id.

        _getColumnId: function (data) {
            return data.name.replace(/\s/g, "");
        },

        //Change the context menu based on context menu item list.
        _updateContextmenuOption: function (recordIndex) {
            //removing empty string and undefined from contextmenu items.
            Array.prototype.clean = function (deleteValue) {
                for (var i = 0; i < this.length; i++) {
                    if (this[i] == deleteValue) {
                        this.splice(i, 1);
                        i--;
                    }
                }
                return this;
            };
            var proxy = this;
            
            if(proxy._isRowEdit || proxy.model.isEdit)
            {
                //Remove add, delete and edit context menu when row in edit mode.
                proxy._contextMenuItems.splice(0, 3);
            }
            else
            {
                proxy._contextMenuItems.splice(3, 2); //Remove save and cancel when row not in edit mode
                var editSetting = proxy.model.editSettings
                var contextMenuItems = $.extend(true, [], proxy._contextMenuItems);
                var contextMenuList = proxy.model.contextMenuSettings.contextMenuItems.clean("").clean(undefined);
                var contextMenuItemList = [];
                //removing duplicate conetxt menu items
                $.each(contextMenuList, function (i, el) {
                    if ($.inArray(el.toLowerCase(), contextMenuItemList) === -1) contextMenuItemList.push(el.toLowerCase());
                });
                proxy._contextMenuItems = [];
                if (recordIndex == -1) {
                    if (contextMenuItemList.indexOf("add") != -1 || contextMenuItemList.length==0)
                        proxy._contextMenuItems[0] = contextMenuItems[0];
                    if (!editSetting.allowAdding)
                        proxy._contextMenuItems[0].disable = true;
                    proxy._contextAddMenuIndex = 0;
                }
                //Adding default context menu when user not specify the context menu items.
                else if (contextMenuItemList.length == 0)
                {
                    proxy._contextMenuItems[0] = contextMenuItems[0];
                    if (!editSetting.allowAdding)
                        proxy._contextMenuItems[0].disable = true;
                    proxy._contextAddMenuIndex = 0;
                    proxy._contextMenuItems[1] = contextMenuItems[2];
                    if (!editSetting.allowDeleting)
                        proxy._contextMenuItems[1].disable = true;
                }
                //Change the context menu order based on context menu items order
                else {
                for(i=0;i<contextMenuItemList.length;i++)
                {
                    var item=contextMenuItemList[i];
                    switch(item.toLowerCase())
                    {
                        case "add":
                            proxy._contextMenuItems[i] = contextMenuItems[0];
                            proxy._contextAddMenuIndex = i;
                            if (!editSetting.allowAdding)
                                proxy._contextMenuItems[i].disable = true;
                            break;
                        case "delete":
                            proxy._contextMenuItems[i] = contextMenuItems[2];
                            if (!editSetting.allowDeleting)
                                proxy._contextMenuItems[i].disable = true;
                            break;
                        case "edit":
                            proxy._contextMenuItems[i] = contextMenuItems[1];
                            if (!editSetting.allowEditing)
                                proxy._contextMenuItems[i].disable = true;
                            break;
                    }
                }
                }
            }
        },
        //Assign default value to columns based on column edit type
        _setInitialData:function()
        {
            var eventArgs = {}, proxy = this,
               model = proxy.model, data = [], item, initialData = {};
            for (i = 0; i < model.columns.length; i++)
            {
                var column = model.columns[i];
                switch(column.editType)
                {
                    case "booleanedit":
                        initialData[column.field] = false;
                        break;
                    case "stringedit":
                        initialData[column.field] = null;
                        break;
                    case "datepicker":
                        initialData[column.field] = Globalize.format(new Date(), this.model.dateFormat);
                        break;
                    case "datetimepicker":
                        today = new Date();
                        initialData[column.field] = Globalize.format(new Date(), "MM/dd/yyyy hh:mm tt");
                        break;
                    case "numericedit":
                        initialData[column.field] = 0;
                        break;
                    default:
                        initialData[column.field] = null;

            }
            }
            data.push(initialData);
            if (proxy.dataSource() == null || proxy.dataSource().length == 0 || model.flatRecords.length == 0) {
                
                item = proxy._createRecord(data[0], 0, null);
                item.index = 0;
                proxy._addRecord(item, ej.TreeGrid.RowPosition.Top);
                proxy._clearContextMenu();
                this._removeDetailsRow();
            }
        },
        
        // Perform add new row operation
           
        _subContextMenuAction: function (item, index, position)
        {
            var proxy = this, model = proxy.model,
            eventArgs = {},
            newItem, selectionIndex;

            eventArgs.requestType = "save";
            newItem = jQuery.extend(true, {}, item.item);
            newItem[proxy.model.childMapping] = null;
            if (position == "Above") {
                proxy._addRecord(newItem, ej.TreeGrid.RowPosition.Above);
            } else if (position == "Below") {
                proxy._addRecord(newItem, ej.TreeGrid.RowPosition.Below);
            }
            proxy._clearContextMenu();
        },

        //Create editable row with specific edit type.

        _editRow: function (index, addrow) {
            var proxy = this,
                model = proxy.model;
                rowHeight = proxy.model.rowHeight;
            proxy._editedRowIndex = index;
            var $targetTr = ej.TreeGrid.getRowByIndex(proxy, index);
            if ($targetTr.hasClass("e-summaryrow")) {
                proxy._editAddTools();
                return;
            }
            $form = ej.buildTag("form.#" + proxy._id + "EditForm", "", { "margin": "-2px" }, {})
            $table = ej.buildTag('table.e-table', "", {}, { "cellspacing": "0px", "cellpadding": "0" });
            $table.append(proxy.getHeaderTable().find('colgroup').clone());
            var $targetTr = ej.TreeGrid.getRowByIndex(proxy, index);
            var $detailCell;
            if (model.showDetailsRow && model.showDetailsRowInfoColumn && model.detailsTemplate) {
                $detailCell = $targetTr.find("td.e-detailsrowcell");
            }

            $targetTr.empty();
            if (addrow) {
                $targetTr.addClass('e-addedrow');
                if ($targetTr.next('tr').hasClass('e-detailsrow'))
                    $targetTr.next('tr').remove();
            }
            else
                $targetTr.addClass('e-rowedit');
            $mainTd = ej.buildTag("td", "", {}, { "colspan": $detailCell ? proxy._visibleColumns.length + 1 : proxy._visibleColumns.length });
            
            var $tr = ej.buildTag('tr.e-treegridrows e-selectionbackground e-active', { "height": rowHeight }, {});
            $tbody = ej.buildTag('tbody');
            $cellEditTemplate = proxy._cellEditTemplate;
            var columns = model.columns,
                columnLength = columns.length;
            for (var column = 0; column < columnLength; column++) {
                if (!ej.isNullOrUndefined(column)) {
                    if (columns[column].visible) {
                        var data = {};
                        //Create a html string for column field with specific edit type using cellEditTemplate
                        htmlString = $cellEditTemplate.find("#" + columns[column].field + "_CellEdit").html();
                        data[columns[column]["field"]] = model.updatedRecords[index][columns[column]["field"]];
                        if (columns[column]["editType"] == "dropdownedit" && data[columns[column]["field"]] == false)
                            data[columns[column]["field"]] = data[columns[column]["field"]].toString();
                        $element = $($.templates(htmlString).render(data));
                        var $td = ej.buildTag('td.e-rowcell e-editedcell', "", { "height": rowHeight, "vertical-align": "middle;", "overflow": "hidden" }, {});

                        if (columns[column].allowEditing == false)
                            $element.attr('disabled', true).addClass('e-disable');

                        if ($element.get(0).tagName == "SELECT") {
                            $element.val(data[columns[column]["field"]]);
                            $element.val() == null && $element.val($element.find("option").first().val());
                        }
                        $tr.append($td.append($element));
                        $tbody.append($tr);
                    }
                }
            }
            if ($detailCell) {
                $tr.append($detailCell);
            }
            $table.append($tbody);
            $form.append($table)
            $mainTd.append($form);
            $targetTr.append($mainTd);
            proxy._isRowEdit = true;
            proxy._setoffsetWidth();
            proxy._editAddTools();
            proxy._refreshEditForm();
        },

        //Creating column chooser menu

        _renderColumnChooser: function () {
            var proxy = this,
                $outerDiv = ej.buildTag("div.e-columnSelector", '',"", { id: this._id + "ccDiv" });
            $("#" + proxy._id + "ccDiv_wrapper").remove();
            proxy._renderColumnChooserList(false);
            $outerDiv.append(proxy._columnChooserList);
            $outerDiv.insertAfter(proxy.element)
            $outerDiv.ejDialog({ showOnInit: false, allowKeyboardNavigation: false, enableResize: false, "enableRTL": false, showHeader: false, width: "auto",
                                 position: { X: 1, Y: 1 }, enableAnimation: false, minWidth:"0px" });
            $("#" + proxy._id + "ccDiv_wrapper").removeClass("e-dialog").removeClass("e-shadow").find("div.e-widget-content").removeClass("e-widget-content");
        },

        /*  Creating Column Chooser menu list with column header name
		    argument refresh in boolean data type
		    If it is false, rendering the column chooser list
            If is is true, rerendering the column chooser list
        */

        _renderColumnChooserList: function (refresh,headerText) {
            var proxy = this,
                length = proxy.model.columns.length,
                columns = proxy.model.columns,
                $listBox = ej.buildTag("div", '', { 'margin-left': '0px', "width": "auto" });
            for (var index = 0; index < length; index++) {
                if (proxy.model.treeColumnIndex != index) {
                    var column = columns[index];
                    if (!ej.isNullOrUndefined(column)) {
                        var $innerDiv = ej.buildTag('div', '',"", { 'class': 'e-columnMenuListDiv' }),
                            styleAttr = {},
                            colValue = ej.isNullOrUndefined(column.headerText) ? column.field : column.headerText,
                            $input = ej.buildTag('input', '', styleAttr, { 'id': proxy._id+"_columnMenu_"+column.field, 'value': colValue, 'type': 'checkbox' }),
                            label = ej.buildTag('label', colValue, { 'font-size': '12px', "width": "auto" }, { 'for': proxy._id + column.field });
                        $innerDiv.append($input);
                        $innerDiv.append(label);
                        $innerDiv.data("column", column.headerText);
                        $listBox.append($innerDiv);
                        if (column.headerText == headerText)
                            $innerDiv.addClass("e-columnmenuselection");
                        $input.ejCheckBox({
                            checked: !ej.isNullOrUndefined(column.visible) ? column.visible : true,
                            change: function(args)
                            {
                                if (!args.isChecked)
                                    proxy.hideColumn(args.model.value);
                                else
                                    proxy.showColumn(args.model.value);
                            }
                        });
                    }
                }
            }
            if (!refresh) 
                proxy._columnChooserList = $listBox;
            else {
                proxy._columnChooserList.empty().append($listBox.children());
                proxy.element.focus();
            }
            var columnChooserListItem = proxy._columnChooserList.find("div.e-columnMenuListDiv");
            $(columnChooserListItem).mouseenter(function () {
                columnChooserListItem.removeClass("e-columnmenuselection");
                $(this).addClass("e-columnmenuselection");
            });
        },

        //Template for column menu

        _createColumnMenuTemplate:function()
        {
            var proxy = this;
            var columnMenuList = "<li style='list-style-type:none;margin:1px;'>" +
                                 "<div class='e-columnmenuitem' style='display:table;'" +
                                    " id='" + proxy._id + "_{{:menuId}}Chooser'>" +
                                "<div class='e-icon {{:icon}}' style='display:inline-block;padding:5px;'></div>" +
                                "<div style='display:inline-block;font-family:Segoe UI;font-size:12px;padding:5px;'>" +
                                "<span>{{:name}}</span></div>" +
                                "{{if (menuId =='Columns')}}<div class='e-icon e-expander' style='display:inline-block;padding:5px;float:right;'/> {{/if}}"+                            
                                "</div>" +
                                "</li>"                               
            var templates = {};
            templates[proxy._id + "columnMenuTemplate"] = columnMenuList;
            $.templates(templates);
        },

        //Creating Column Menu

        _renderColumnMenu: function (e) {
            var proxy = this,
                model = proxy.model,
                posX = columnListPosX = posY = 0,
                $target = $(e.target),
                targetWidth = targetHeight = 0,
                columnMenuItems = [],
                columnChooserListIndex = 0,
                targetWidth = $target[0].offsetWidth,
                targetHeight = $target[0].offsetHeight,
                columnMenuText = proxy._columnMenuTexts,
                columnName = $.trim($target.prev("div.e-headercelldiv").text()),
                column = proxy.getColumnByHeaderText(columnName),
                allowSorting = column.allowSorting == undefined ? true : column.allowSorting;
            proxy._clearColumnMenu();
            proxy._columnMenuTarget = $target;
            if (model.allowSorting && allowSorting)
            {
                columnMenuItems.push({ "name": columnMenuText["sortAscendingText"], "icon": "e-columnmenu-ascending", "menuId": "SortAscending" });
                columnMenuItems.push({ "name": columnMenuText["sortDescendingText"], "icon": "e-columnmenu-descending", "menuId": "SortDescending" });
            }
            if (model.showColumnChooser) {
                columnChooserListIndex = columnMenuItems.length;
                columnMenuItems.push({ "name": columnMenuText["columnsText"], "icon": "e-columnchooser-icon", "menuId": "Columns" });
            }
            var position = proxy.getOffsetRect($target[0]);            
            var elementPosition = proxy.getOffsetRect(proxy.element[0]);           
            if (!proxy.model.isFromGantt)
                posY = position.top + targetHeight;
            else
                posY = position.top + targetHeight;            
            proxy._columnChooserListIndex = columnChooserListIndex;
            var $outerDiv = ej.buildTag("div.e-columnmenu", "", {
                "border": "1px solid", "display": "table",
                "position": "absolute", "top": posY, "z-index": "10033"
            }, { id: proxy._id + "_ColumnMenu" }),
                $outerUl = ej.buildTag("ul", "", { "padding": "0px", "margin": "0px" }, {}),                                       
                $columnMenuList = $.render[proxy._id + "columnMenuTemplate"](columnMenuItems);
            $outerUl.append($columnMenuList);
            $outerDiv.append($outerUl);
            $(document.body).append($outerDiv);
            var columnMenuWidth = $outerDiv.width();
            posX = position.left + targetWidth - columnMenuWidth;
            if (posX <= elementPosition.left)
                posX = elementPosition.left;
            columnListPosX = posX + columnMenuWidth + 4;
            proxy._columnListPosX = columnListPosX;
            $outerDiv.css({ 'left': posX })
            $outerDiv.find(".e-columnmenuitem").css({ 'width': columnMenuWidth });

            //Edge detection for column menu
            var element = !model.isFromGantt ? $(this.element) : $(this.element).closest(".e-gantt"),
                containerOffset = this.getOffsetRect(element[0]);
            containerOffset.bottom = containerOffset.top + element[0].offsetHeight;
            containerOffset.right = containerOffset.left + $(element).width();

            $("#" + proxy._id + "_ColumnsChooser").mouseenter(function () {
                var columnchooser = $("#" + proxy._id + "ccDiv").ejDialog("instance"),
                    columnChooserList = $("#" + proxy._id + "ccDiv_wrapper"),
                    columnChooserMenuList = columnChooserList.find("div.e-columnMenuListDiv");
                columnchooser.open();
                if (containerOffset.right < columnListPosX + columnChooserList.width()) {
                    columnListPosX = columnListPosX - columnChooserList.width() - columnMenuWidth - 4;
                    if (columnListPosX <= (columnMenuWidth - 4)) {
                        columnListPosX = posX + columnMenuWidth - 4;
                    }
                }
                proxy._columnListPosX = columnListPosX;
                var position={
                    X:columnListPosX,
                    Y: posY + columnChooserListIndex * 30 + (columnChooserListIndex == 0 ? 0 : 1)
                }
                columnchooser.option({ "position": position });
                columnChooserList.css("z-index", "10033");
                columnChooserMenuList.removeClass("e-columnmenuselection");
                columnChooserMenuList.eq(0).addClass("e-columnmenuselection");
                proxy.element.focus();
            });
            $("#" + proxy._id + "_SortAscendingChooser, #" + proxy._id + "_SortDescendingChooser").mouseenter(function () {
                $("#" + proxy._id + "ccDiv").ejDialog("close");
            });
            $("#" + proxy._id + "_SortAscendingChooser").click(function () {
                if (model.allowMultiSorting)  proxy._multiSortRequest = true;
                proxy.sortColumn(columnName, ej.sortOrder.Ascending);
                proxy._multiSortRequest = false;
            });
            $("#" + proxy._id + "_SortDescendingChooser").click(function () {
                if(model.allowMultiSorting) proxy._multiSortRequest = true;
                proxy.sortColumn(columnName, ej.sortOrder.Descending);
                proxy._multiSortRequest = false;
            });
            var columnMenu = $("#" + proxy._id + "_ColumnMenu"),
                columnMenuItem = columnMenu.find("div.e-columnmenuitem");
            columnMenuItem.eq(0).addClass("e-columnmenuselection");
            $(columnMenuItem).mouseenter(function () {
                columnMenuItem.removeClass("e-columnmenuselection");
                $(this).addClass("e-columnmenuselection");
            }
            );
        },

        //Clear the column menu and menu list

        _clearColumnMenu:function()
        {
            var proxy = this;
            $("div[id$='_ColumnMenu']").remove();
            $("div[id$='ccDiv']").each(function () {
                $(this).data("ejDialog") && $(this).ejDialog("close");
            });
            $(".e-columnicon").data("isClicked", false);
        },

        //Get the element Left and Top position

        getOffsetRect:function(elem) {
            var box = elem.getBoundingClientRect(),
                body = document.body,
                docElem = document.documentElement,
                scrollTop = window.pageYOffset || docElem.scrollTop || body.scrollTop,
                scrollLeft = window.pageXOffset || docElem.scrollLeft || body.scrollLeft,
                clientTop = docElem.clientTop || body.clientTop || 0 ,
                clientLeft = docElem.clientLeft || body.clientLeft || 0 ,
                top  = box.top +  scrollTop - clientTop ,
                left = box.left + scrollLeft- clientLeft;
            return { top: Math.round(top), left: Math.round(left) }
        },
        // Find the next context menu item to select
        _findContextMenuItem: function (item, move) {
            var proxy = this;
            if (move === "next") {
                nextItem = item.closest("li").next("li").find("div.e-menuitem");
                if (nextItem.length > 0) {
                    if (!nextItem.hasClass("e-disable")) {
                        return nextItem;
                    }
                    else
                        return proxy._findContextMenuItem(nextItem, "next");
                }
                else
                    return nextItem;
            }
            else {
                prevItem = item.closest("li").prev("li").find("div.e-menuitem");
                if (prevItem.length > 0) {
                    if (!prevItem.hasClass("e-disable")) {
                        return prevItem;
                    }
                    else
                        return proxy._findContextMenuItem(prevItem, "prev");
                }
                else
                    return prevItem;
            }
        }
    });

    //ENUM FOR EACH ACTION IN TREEGRID
    ej.TreeGrid.Actions = {
        Sorting: "sorting",
        BeginEdit: "beginedit",
        Save: "save",
        Add: "add",
        Delete: "delete",
        Cancel: "cancel",
        Refresh: "refresh",
        Searching: "searching",
        ExpandCollapse: "expandcollapse",
        Selection: "selection",
        rowHover: "rowHover",
        Scroll: "scroll",
        ContextMenuAdd: "contextMenuAdd",
        RefreshDataSource: "refreshDataSource",
        DragAndDrop:"dragAndDrop"
    };

    //ENUM FOR SELECTION MODE IN TREEGRID
    ej.TreeGrid.SelectionType = {
        Single: "single",
        Multiple: "multiple"
    };

    //ENUM FOR DIFFERENT EDIT MODE IN TREEGRID
    ej.TreeGrid.EditMode = {
        CellEditing: "cellEditing",
        RowEditing:"rowEditing",
    };

    //Enum for position of new added row.
    ej.TreeGrid.RowPosition = {
        Top: "top",
        Bottom: "bottom",
        Above: "aboveSelectedRow",
        Below: "belowSelectedRow",
        Child : "child"
    }
    /* enum for six different columnEditType for treegrid */
    ej.TreeGrid.EditingType = {
        String: "stringedit",
        Boolean: "booleanedit",
        Numeric: "numericedit",
        Dropdown: "dropdownedit",
        DatePicker: "datepicker",
        DateTimePicker: "datetimepicker",
        Maskedit : "maskedit"
    };
    /* enum for different context menu in treegrid */
    ej.TreeGrid.ContextMenuItems = {
        Add: "add",
        Edit: "edit",
        Delete: "delete",
    };
    
    ej.TreeGrid = ej.TreeGrid || {};


    //INITIALIZE THE CELLEDIT ELEMENT FOR DIFFERENT TYPES OF EDITORS
    ej.TreeGrid._initCellEditType = function (instance, $element, id, columnCount , dialogId) {

        var proxy=instance,
            model=proxy.model,
            columns=model.columns,
            column=columns[columnCount],
            dialogId = dialogId ? dialogId : "";
        
        if (ej.isNullOrUndefined(columns[columnCount]["editType"])) {
            column["editType"] = "stringedit";
        }

        switch (column["editType"]) {
            
            case "stringedit":

                if (model.isFromGantt) {
                    
                    var helpers={};
                    
                    helpers["_" + id + "cellValue"] = ej.TreeGrid._getCellValue;
                    
                    if(column.field==="predecessor"){
                        helpers["_" + id + "predecessorCell"] = ej.Gantt._getPredecessorsValue;
                    }
                    
                    $.views.helpers(helpers);


                    if(column.field==="predecessor"){ //get the predecessor value from item of GanttRecord
                     
                        var $input=ej.buildTag('input.e-field e-ejinputtext',
                                       "",{},
                                       {
                                           value:"{{:~_" + id + "predecessorCell('"+model.predecessorMapping+"')}}",
                                           id: id + column.field + dialogId,
                                           name: column.field,
                                           dialog: dialogId
                                       });

                    }else { /*get the cellValue if the column does not exist in the GanttRecord object and
                           present in the item of GanttRecord*/
                        $input= ej.buildTag('input.e-field e-ejinputtext', 
                                            "",
                                            {},
                                            {
                                                value: "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}",
                                                id: id + model.columns[columnCount].field + dialogId,
                                                name: model.columns[columnCount].field,
                                                dialog: dialogId
                                            });
                       
                    }

                }else { /*Get the cellvalue of the normal gridcell*/
                    $input=ej.buildTag('input.e-field e-ejinputtext',
                                          "",
                                          {}, 
                                          {
                                              value: "{{:#data['" + column.field + "']}}", 
                                              id: id + column.field + dialogId,
                                              name: column.field,
                                              dialog: dialogId
                                          });
                }
                $input.attr("edittype", columns[columnCount].editType)
                $element.html($input);
            
                break;

            case "maskedit":
                $input = ej.buildTag('input.e-field e-maskedit', "", {},
                                         {
                                             value: "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}",
                                             id: id + column.field + dialogId,
                                             name: column.field,
                                             dialog: dialogId
                                         });
                $input.attr("edittype", columns[columnCount].editType)
                $element.html($input);
                break;

            case "booleanedit":
                $element.html('{{if true===~_' + proxy._id + 'cellValue("' + columns[columnCount].field + '")}}' +
                         '<input type ="checkbox" ' +
                         'id="' + id + columns[columnCount].field + dialogId +
                         '" dialog ="' + dialogId +
                         '" name="' + columns[columnCount].field +
                         '" checked="checked" edittype="' +columns[columnCount].editType +'"></input>' +
                         ' {{else}} ' +
                         ' <input' +
                         ' type ="checkbox"' +
                         ' id="' + id + columns[columnCount].field + dialogId +
                         '" dialog ="'+ dialogId +
                         '" name="' + columns[columnCount].field + '" edittype="' + columns[columnCount].editType + '">' +
                         '{{/if}}');

                break;
            
            case "numericedit":

                var $numericText = ej.buildTag('input.e-numerictextbox e-field', "", {}, {
                    type: "text",
                    value: "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}",
                    id: id + columns[columnCount].field + dialogId,
                    name: columns[columnCount].field,
                    dialog: dialogId
                });
                $numericText.attr("edittype", columns[columnCount].editType)
                $element.append($numericText);

                break;

            case "datepicker":
                
                var $datePicker = ej.buildTag('input.e-datepicker e-field', "",
                                             {}, 
                                             {
                                                 type: "text",
                                                 value: "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}",
                                                 id: id + columns[columnCount].field + dialogId,
                                                 name: columns[columnCount].field,
                                                 dialog: dialogId
                                             });
                $datePicker.attr("edittype", columns[columnCount].editType)
                $element.append($datePicker);

                break;
            case "datetimepicker":

                var $datePicker = ej.buildTag('input.e-datetimepicker e-field', "",
                                             {},
                                             {
                                                 type: "text",
                                                 value: "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}",
                                                 id: id + columns[columnCount].field + dialogId,
                                                 name: columns[columnCount].field,
                                                 dialog: dialogId
                                             });
                $datePicker.attr("edittype", columns[columnCount].editType)
                $element.append($datePicker);

                break;
            case "dropdownedit":

                    var $dropDownList = ej.buildTag('input.e-field e-dropdownlist',
                        "", 
                        {}, 
                        {
                            type: "text",
                            id: id + columns[columnCount].field + dialogId,
                            name: columns[columnCount].field,
                            dialog: dialogId
                        });
                    $dropDownList.attr("edittype", columns[columnCount].editType)
                        .attr("cellValue", "{{:~_" + proxy._id + "cellValue('" + columns[columnCount].field + "')}}");
                    $element.append($dropDownList);

                break;
        }
    };
    
    ej.TreeGrid._getCellValue= function (columnName) {

        var cellValue = this.data[columnName];

        if (cellValue) {
            return cellValue;
        } else {
            return this.data.item && this.data.item[columnName];
        }


    };
    //Initialize tree grid toolbar item.
    ej.TreeGrid.ToolbarItems = {
        Add: "add",
        Edit: "edit",
        Delete: "delete",
        Update: "update",
        Cancel: "cancel",
        ExpandAll: "expandAll",
        CollapseAll: "collapseAll",
        PdfExport:"pdfExport"
    };

    //Enum Values of summaryType
    ej.TreeGrid.SummaryType = {
        Sum: "sum",
        Average: "average",
        Maximum: "maximum",
        Minimum: "minimum",
        Count: "count",
        MinimumDate: "minimumDate",
        MaximumDate: "maximumDate",
        TrueCount: "trueCount",
        FalseCount: "falseCount",
    };

    ej.TreeGrid.localization = {};

    ej.TreeGrid.localization["en-US"] = {
        //string to be displayed in Toolbox's tooltip 
        toolboxTooltipTexts: {
            addTool: "Add",
            editTool: "Edit",
            updateTool: "Update",
            deleteTool: "Delete",
            cancelTool: "Cancel",
            expandAllTool: "ExpandAll",
            collapseAllTool: "CollapseAll",
            pdfExportTool:"PDF Export"
        },
        //string to be displayed in context menu 
        contextMenuTexts: {
            addRowText: "Add Row",
            editText: "Edit",
            deleteText: "Delete",
            saveText: "Save",
            cancelText : "Cancel",
            aboveText: "Above",
            belowText: "Below"
        },
        //string to be displayed in column menu 
        columnMenuTexts: {
            sortAscendingText: "Sort Ascending",
            sortDescendingText: "Sort Descending",
            columnsText: "Columns"
        },
        //String to be displayed in drop down list for filtering blank items
        dropDownListBlanksText: "(Blanks)",
        //String to be displayed in drop down list to clear filtered items
        dropDownListClearText: "(Clear Filter)",
        //Text to be displayed in drop down list for FALSE value in boolean edit type column
        trueText: "True",
        //Text to be displayed in drop down list for True value in boolean edit type column
        falseText: "False",

        emptyRecord: "No records to display"
    };

    //GET ROW CLASS NAME FOR EXPAND COLLAPSE ACTION
    ej.TreeGrid._getrowClassName = function () {

        var rowClass = "gridrowIndex",
            proxy = this;

        if (proxy.data.parentItem) {

            rowClass += proxy.data.parentItem.index.toString();

        }

        rowClass += "level";
        rowClass += proxy.data.level.toString();
        if (proxy.data.isSummaryRow) {
            rowClass += " ";
            rowClass += "e-summaryrow";
        }
        if (proxy.data.footerSummaryRowRecord) {
            rowClass += " ";
            rowClass += "e-footersummaryrow";
        }

        return rowClass;
    };
    ej.TreeGrid._getSummaryRowtdClassName = function () {
        proxy = this;
        var tdClass = "";
        if (proxy.data.isSummaryRow) {
            tdClass += " ";
            tdClass += "e-summaryrowcell";
        }
        if (proxy.data.footerSummaryRowRecord) {
            tdClass += " ";
            tdClass += "e-footersummaryrowcell";
        }
        return tdClass;
    };

    //GET COLUMN BY FIELD NAME
    ej.TreeGrid.getColumnByField = function (columns, field) {

        var column = 0;

        for (column; column < columns.length; column++) {

            if (columns[column]["field"] == field) break;

        }

        return column == columns.length ? null : columns[column];
    };


    ej.TreeGrid.getColumnByMappingName = function (columns, field) {
        var column = 0;
        for (column; column < columns.length; column++) {
            if (columns[column]["mappingName"] == field) break;
        }
        return column == columns.length ? null : columns[column];
    };

    //EXPAND COLLAPSE THE PARTICULAR RECORD OBJECT
    ej.TreeGrid.sendExpandCollapseRequest = function (element, args) {

        var proxy = element,
             model = proxy.model,
             toolbarItems = model.toolbarSettings.toolbarItems;

        args.requestType = ej.TreeGrid.Actions.ExpandCollapse;
        //To cancel the cell edit or row edit mode when expand the all record.
        if (proxy._isRowEdit || proxy.model.isEdit) {
            proxy.cancelRowEditCell();
        }
        if (model.enableVirtualization) {

            if (proxy._isFromGantt) {
                proxy._$treegridHelper.ejTreeGrid("processBindings", args);
            } else {
                proxy.processBindings(args);
            }

            if (proxy._isFromGantt) {
                if (proxy._isInExpandCollapseAll === false) {
                    if (model.enableAltRow) {
                        proxy._$treegridHelper.ejTreeGrid("updateAltRow");
                    }
                    proxy._$treegridHelper.ejTreeGrid("sendDataRenderingRequest", args);
                    proxy._$treegridHelper.ejTreeGrid("updateHeight");
                    proxy.model.updatedGanttRecords = proxy.getUpdatedRecords();
                    proxy.model.currentViewData = proxy.getCurrentViewData();
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._$ganttchartHelper.ejGanttChart("refreshHelper", proxy.model.currentViewData, proxy.model.updatedGanttRecords);
                  
                }
            }
            else {
                if (proxy._isInExpandCollapseAll === false) {
                    if (model.enableAltRow) {
                        proxy.updateAltRow();
                    }
                    proxy.sendDataRenderingRequest(args);
                    proxy._setScrollTop();
                    proxy.updateHeight();
                }
            }
            
        } else {
            var record = args.data,
                index,
                $row,
                height = 0,
                isModified = record.expanded !== args.expanded;
                index = proxy.model.currentViewData.indexOf(record);
            

                $row = proxy.getRows() && $(proxy.getRows()[index]);

                if (args.expanded) {
                    record.expanded = args.expanded;
                    $row && ej.TreeGrid.expandRecord(proxy, record, args.expanded, args);
                    if (record.hasFilteredChildRecords) {
                        $row && $row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
                        $row && $row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');
                    }
                    if (model.enableAltRow && proxy._isInExpandCollapseAll === false) {
                        ej.TreeGrid.updateAltRow(proxy, record, index, 1);
                    }
                }
                else {
                    proxy._collapsedRecordCount = 0;//For update alt row
                    $row && ej.TreeGrid.collapseRecord(proxy, record, args.expanded);
                    record.expanded = args.expanded;
                    if (record.hasFilteredChildRecords) {
                        $row && $row.find(".e-treegridexpand").removeClass('e-treegridexpand').addClass('e-treegridcollapse');
                        $row && $row.removeClass('e-treegridrowexpand').addClass('e-treegridrowcollapse');
                    }
                    if (model.enableAltRow && proxy._isInExpandCollapseAll === false)
                        ej.TreeGrid.updateAltRow(proxy, record, index, proxy._collapsedRecordCount);
                }

            var collapsedRecordCount= 0;
            if (proxy._isFromGantt) {
                proxy._$treegridHelper.ejTreeGrid("updateExpandStatus", record, args.expanded);
                proxy._$treegridHelper.ejTreeGrid("updateCollapsedRecordCount");
                collapsedRecordCount =  proxy._$treegridHelper.ejTreeGrid("getCollapsedRecordCount");
            } else {
                proxy.updateExpandStatus(record, args.expanded);
                proxy.updateCollapsedRecordCount();
                collapsedRecordCount = proxy.getCollapsedRecordCount();
            }

            height = (proxy.model.updatedRecords.length - collapsedRecordCount) * model.rowHeight;
            
            if (proxy._isFromGantt)
            {
                if (proxy._isInExpandCollapseAll === false) {
                    proxy._totalCollapseRecordCount = collapsedRecordCount;
                    proxy._$ganttchartHelper.ejGanttChart("clearConnectorLines");
                    proxy._$treegridHelper.ejTreeGrid("updateHeight");
                    proxy._$ganttchartHelper.ejGanttChart("updateHeight", height);
                    proxy._$ganttchartHelper.ejGanttChart("setCollapsedRecordCount", collapsedRecordCount);
                    proxy._$ganttchartHelper.ejGanttChart("refreshGridLinesTable", proxy.model.updatedRecords.length - collapsedRecordCount);
                }
                
            } else {
                if (proxy._isInExpandCollapseAll === false) {
                    proxy.updateHeight();
                }
            }
        }

        
        if (proxy._isFromGantt && proxy.model.predecessorMapping && proxy._isInExpandCollapseAll === false) {
            proxy._isValidationEnabled = false;
            proxy._connectorlineIds = [];
            proxy._connectorLinesCollection = [];
            proxy._createConnectorLinesCollection();
        }

        if (proxy._isFromGantt) {
            var treeGrid = proxy._$treegridHelper.ejTreeGrid("instance");
            var chart = proxy._$ganttchart.ejGanttChart("instance");
            var top = treeGrid._$gridContent.ejScroller("option", "scrollTop");
            chart._$bodyContainer.ejScroller("option", "scrollTop", top);
        }

        if (args.expanded) {
            proxy._trigger("expanded", args);
        } else {
            proxy._trigger("collapsed", args);
        }

        var selectItem = proxy.selectedItem();
        if (!proxy._isFromGantt && selectItem) {            
            var expanded = proxy.getExpandStatus(selectItem);
            if (!expanded && !args.expanded) {
                proxy.selectRows(-1);
                proxy.selectedRowIndex(-1);
                selectItem.isSelected = false;
                model.selectedItem = null;
                proxy._cancelSaveTools();
            }
            if (model.enableVirtualization && selectItem && model.updatedRecords.indexOf(selectItem) !== proxy.selectedRowIndex())
                proxy.selectRows(model.updatedRecords.indexOf(selectItem));
        }
    };

    //EXPAND THE RECORD
    ej.TreeGrid.expandRecord = function (controlObject, Record, expanded, args) {
  
        var proxy = controlObject,
            rowClassName = ".gridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            detailsRowClassName = ".detailsrowgridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            $rows = $(proxy.element).find(rowClassName),
            count = 0,
            length = $rows.length,
            row,
            index,
            record,
            model = proxy.model,
            $detailRows = $(proxy.element).find(detailsRowClassName),
            $expandedRows = $(proxy.element).find(rowClassName),
            index,
            $gridRows = proxy.getRows(),
            $row = $($gridRows[index]);

            index = model.currentViewData.indexOf(Record);

        if (Record.hasChildRecords && Record.childRecords.length > 0) {
            $row.find(".e-treegridcollapse").removeClass('e-treegridcollapse').addClass('e-treegridexpand');
            $row.removeClass('e-treegridrowcollapse').addClass('e-treegridrowexpand');
        }
        proxy._expandedRecordsCount += proxy._isFromGantt ? $expandedRows.length / 2 : $expandedRows.length;
        $expandedRows.css({ 'display': 'table-row' });
        $detailRows.filter("tr.e-detailsrowexpanded").css('display', 'table-row');
        if (proxy._isFromGantt) {
            length = length / 2;
        }

        for (count; count < length; count++) {
            row = $rows[count];
            index = $gridRows.index(row);
            if (index !== -1) {
                record = proxy._isFromGantt ? proxy.model.updatedRecords[index] : proxy.model.updatedRecords[index];
                record.isExpanded = true;
                if (record.expanded)
                    ej.TreeGrid.expandRecord(proxy,record, expanded);
            }
        }
    },
    
    //COLLAPSE THE RECORD
    ej.TreeGrid.collapseRecord = function (controlObject, Record, expanded) {

        var proxy = controlObject,
            model = proxy.model,
            rowClassName = ".gridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            detailsRowClassName = ".detailsrowgridrowIndex" + Record.index.toString() + "level" + (Record.level + 1).toString(),
            $rows = $(proxy.element).find(rowClassName),
            count = 0,
            length = $rows.length,
            row,
            index,
            record,
            $detailRows = $(proxy.element).find(detailsRowClassName),
            $collapsedRows = $(proxy.element).find(rowClassName),
        $gridRows = proxy.getRows();

        proxy._collapsedRecordCount += proxy._isFromGantt ? $collapsedRows.length / 2 : $collapsedRows.length;
        $collapsedRows.css({ 'display': 'none' });
        $detailRows.css({ 'display': 'none' });

        if (proxy._isFromGantt) {
            length = length / 2;
        }

        for (count; count < length; count++) {

            row = $rows[count];
            index = proxy.getRows().index(row);


            if (index !== -1) {
                record = proxy.getUpdatedRecords()[index];

                if (record.expanded)
                    ej.TreeGrid.collapseRecord(proxy,record, expanded);
            }
        }
    },

    //UPDATE THE ALT ROW FUNCTION
    ej.TreeGrid.updateAltRow = function (controlObject, Record, recordIndex, offset) {

        var proxy = controlObject,
            count = 0,
            model = proxy.model,
            currentViewData = model.currentViewData,
            length = currentViewData.length,
            isAltRow = Record.isAltRow,
            record,
            $gridRows = proxy.getRows();

        /* for rowTemplate no need set alt row class*/
        if (model.rowTemplateID || model.altRowTemplateID)
            return;

        for (count = recordIndex + offset; count < length; count++) {

            if ($($gridRows[count]).hasClass("e-summaryrow"))
                return;

            if ($($gridRows[count]).css('display') === "none")
                continue;

            record = currentViewData[count];
            if(count!=0)
            record.isAltRow = !isAltRow;
            isAltRow = record.isAltRow;

            if (isAltRow)
                $($gridRows[count]).addClass('e-alt-row');
            else
                $($gridRows[count]).removeClass('e-alt-row');

        }
    };

    //RERENDERED THE EDITED RECORD
    ej.TreeGrid.refreshRow = function (controlObject, index) {
        var proxy = controlObject,
            model = proxy.model,
            data = proxy.model.currentViewData[index], eventArgs = {};
        if (data) {
            var currentRefreshingRow = $(proxy.getRows()[index]);

            if (model.showDetailsRow && model.detailsTemplate && data.isDetailsExpanded) {
                var detailsRow = currentRefreshingRow.next("tr");
                if (detailsRow.hasClass("e-detailsrow"))
                    detailsRow.remove();
            }
            currentRefreshingRow.replaceWith($($.render[proxy._id + "_Template"](data)));
            proxy.setGridRows($(proxy.getContentTable().get(0).rows));

            eventArgs.rowElement = $(proxy.getRows()[index])[0];
            eventArgs.data = data;
            proxy._trigger("refresh");
            proxy._refreshedCellEventTrigger(eventArgs);
        }
    };

    //Return corressponding treegrid rows jquery object
    ej.TreeGrid.getRowByIndex = function (controlObject, from, to) {

        try {
            var proxy;
            if (controlObject.pluginName == "ejTreeGrid")
                proxy = controlObject;
            else
                proxy = controlObject._$treegridHelper.data("ejTreeGrid");

            var $gridRows = proxy.getRows(),
               // gridRows=proxy._gridRows,
                $row = $();

            if (proxy.model.enableVirtualization) {
                var recordstart = proxy.model.updatedRecords[from];
                from = proxy.model.currentViewData.indexOf(recordstart);
            }

            if (ej.isNullOrUndefined(to)) {
                if (proxy.model.enableVirtualization) {
                    var recordend = proxy.model.updatedRecords[to];
                    to = proxy.model.currentViewData.indexOf(recordend);
                }
                return $($gridRows[from]);
            } else {
                return $($gridRows.slice(from, to));
            }
        } catch (e) {
            return $();
        }
    }

})(jQuery, Syncfusion);;;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    ej.widget("ejRibbon", "ej.Ribbon", {
        element: null,
        validTags: ["div"],
        model: null,
        _rootCSS: "e-ribbon",
        defaults: {

            width: null,

            applicationTab: {

                type: "menu",
				
				backstageSettings:{

                text: null,

                height: null,

                width: null,
				
				headerWidth:null,

                pages: [{

                    id: null,

                    text: null,

                    itemType: "tab",

                    contentID: null,

                    enableSeparator: false
                }]
				
				},

                menuItemID: null,

                menuSettings: {}
            },

            tabs: [{

                id: null,

                text: null,

                groups: [{

                    text: null,

                    type: null,

                    contentID: null,

                    customContent: null,

                    alignType: "rows",

                    enableGroupExpander: false,

                    content: [{

                        groups: [{

                            id: null,

                            text: null,

                            toolTip: null,

                            customToolTip: {

                                title: null,

                                content: null,

                                prefixIcon: null,
                            },

                            columns: null,

                            itemHeight: null,

                            itemWidth: null,

                            expandedColumns: null,

                            type: "button",

                            galleryItems: [{

                                text: null,

                                toolTip: null,

                                customToolTip: {},

                                buttonSettings: {}
                            }],

                            customGalleryItems: [{

                                text: null,

                                toolTip: null,

                                customToolTip: {},

                                customItemType: "button",

                                buttonSettings: {},

                                menuId: "",

                                menuSettings: {}
                            }],

                            contentID: null,

                            enableSeparator: false,

                            isBig: null,

                            cssClass: null,

                            buttonSettings: {},

                            splitButtonSettings: {},

                            toggleButtonSettings: {},

                            dropdownSettings: {},
                        }],

                        defaults: {},
                    }],
                }]
            }],

            contextualTabs: [{

                backgroundColor: null,

                borderColor: null,

                tabs: []
            }],

            tabSelect: null,

            tabClick: null,

            tabAdd: null,

            tabRemove: null,

            beforeTabRemove: null,

            tabCreate: null,

            create: null,

            groupClick: null,

            galleryItemClick: null,

            backstageItemClick : null,

            collapse: null,
			
			expand: null,

            toggleButtonClick: null,

            groupExpand: null,

            selectedItemIndex: 1,

            enabledItemIndex: [],

            disabledItemIndex: [0],

            allowResizing: false,
			
            locale: "en-US",

            destory: null,
        },
        dataTypes: {
            tabs: "array",
            contextualTabs: "array",
            disabledItemIndex: "data",
            enabledItemIndex: "data",
            selectedItemIndex: "number",
        },

        _destroy: function () {
            this.element.ejTab('destroy');
            this.element.find('.e-contextul').removeAttr("style");
        },
        _tags: [{
			  tag: "applicationTab.backstageSettings.pages", attr: ["id", "text", "itemType", "contentID", "enableSeparator"]
        },
		{
		    tag: "tabs", attr: ["id", "text", [{
		        tag: "groups", attr: ["text", "alignType", "type", "contentID", [{
		            tag: "content", attr: ["defaults.type", "defaults.width", "defaults.height", "defaults.isBig", [{
		                tag: "groups", attr: ["id", "columns", "itemHeight", "itemWidth", "expandedColumns", "contentID", "text", "type", "toolTip", "width",
                        "enableSeparator",
					"buttonSettings.contentType", "buttonSettings.type", "buttonSettings.imagePosition", "buttonSettings.prefixIcon", "buttonSettings.click", "buttonSettings.width", "buttonSettings.cssClass", "buttonSettings.enabled", "buttonSettings.showRoundedCorner", "buttonSettings.enableRTL", "buttonSettings.repeatButton", "buttonSettings.size", "buttonSettings.height", "buttonSettings.text", "buttonSettings.timeInterval", "buttonSettings.create", "buttonSettings.click", "buttonSettings.destroy", "splitButtonSettings.contentType", "splitButtonSettings.prefixIcon", "splitButtonSettings.targetID",
                        "splitButtonSettings.buttonMode", "splitButtonSettings.arrowPosition", "splitButtonSettings.type", "splitButtonSettings.click",
						"splitButtonSettings.enabled", "splitButtonSettings.showRoundedCorner", "splitButtonSettings.enableRTL", "splitButtonSettings.size", "splitButtonSettings.height", "splitButtonSettings.imagePosition", "splitButtonSettings.width", "splitButtonSettings.text", "splitButtonSettings.suffixIcon", "splitButtonSettings.create", "splitButtonSettings.itemMouseOver", "splitButtonSettings.itemSelected", "splitButtonSettings.beforeOpen", "splitButtonSettings.destroy", "splitButtonSettings.cssClass",
						"dropdownSettings.dataSource", "dropdownSettings.text", "dropdownSettings.value", "dropdownSettings.select", "dropdownSettings.width", "dropdownSettings.showRoundedCorner", "dropdownSettings.showPopupOnLoad", "dropdownSettings.allowMultiSelection", "dropdownSettings.enableRTL", "dropdownSettings.enabled", "dropdownSettings.caseSensitiveSearch", "dropdownSettings.showCheckbox", "dropdownSettings.checkAll", "dropdownSettings.uncheckAll", "dropdownSettings.enablePersistence", "dropdownSettings.enableIncrementalSearch", "dropdownSettings.readOnly", "dropdownSettings.allowGrouping", "dropdownSettings.fields",
						"dropdownSettings.selectedItems", "dropdownSettings.itemsCount", "dropdownSettings.height", "dropdownSettings.cssClass", "dropdownSettings.itemValue", "dropdownSettings.popupHeight", "dropdownSettings.targetID", "dropdownSettings.waterMarkText", "dropdownSettings.template", "dropdownSettings.cascadeTo", "dropdownSettings.query", "dropdownSettings.create", "dropdownSettings.popupHide", "dropdownSettings.popupShown", "dropdownSettings.beforePopupShown", "dropdownSettings.change", "dropdownSettings.checkChange", "dropdownSettings.enableAnimation", "dropdownSettings.destroy",
						"toggleButtonSettings.contentType", "toggleButtonSettings.defaultText", "toggleButtonSettings.activeText", "toggleButtonSettings.type", "toggleButtonSettings.defaultPrefixIcon", "toggleButtonSettings.activePrefixIcon", "toggleButtonSettings.click", "toggleButtonSettings.enabled", "toggleButtonSettings.showRoundedCorner", "toggleButtonSettings.enableRTL", "toggleButtonSettings.enablePersistence", "toggleButtonSettings.toggleState",
						"toggleButtonSettings.preventToggle", "toggleButtonSettings.size", "toggleButtonSettings.imagePosition", "toggleButtonSettings.height", "toggleButtonSettings.width", "toggleButtonSettings.cssClass",
						"toggleButtonSettings.defaultText", "toggleButtonSettings.activeText", "toggleButtonSettings.defaultPrefixIcon",
						"toggleButtonSettings.defaultSuffixIcon", "toggleButtonSettings.activePrefixIcon", "toggleButtonSettings.create", "toggleButtonSettings.change", "toggleButtonSettings.destroy",
						"customToolTip.title", "customToolTip.content", "customToolTip.prefixIcon", [{
						    tag: "galleryItems", attr: ["text", "toolTip", "customToolTip.title", "customToolTip.content", "customToolTip.prefixIcon", "buttonSettings.contentType", "buttonSettings.type", "buttonSettings.imagePosition", "buttonSettings.prefixIcon", "buttonSettings.click", "buttonSettings.width", "buttonSettings.cssClass", "buttonSettings.enabled", "buttonSettings.showRoundedCorner", "buttonSettings.enableRTL", "buttonSettings.repeatButton", "buttonSettings.size", "buttonSettings.height", "buttonSettings.text", "buttonSettings.timeInterval", "buttonSettings.create", "buttonSettings.click", "buttonSettings.destroy"]
						}, {
						    tag: "customGalleryItems", attr: ["text", "toolTip", "customToolTip.title", "customToolTip.content", "customToolTip.prefixIcon", "menuId", "customItemType",
    "buttonSettings.contentType", "buttonSettings.type", "buttonSettings.imagePosition", "buttonSettings.prefixIcon", "buttonSettings.click", "buttonSettings.width", "buttonSettings.cssClass", "buttonSettings.enabled", "buttonSettings.showRoundedCorner", "buttonSettings.enableRTL", "buttonSettings.repeatButton", "buttonSettings.size", "buttonSettings.height", "buttonSettings.text", "buttonSettings.timeInterval", "buttonSettings.create", "buttonSettings.click", "buttonSettings.destroy",
                            "menuSettings.openOnClick", "menuSettings.enableCenterAlign", "menuSettings.showRooltLevelArrows", "menuSettings.showSubLevelArrows", "menuSettings.enableSeparator", "menuSettings.enabled", "menuSettings.orientation", "menuSettings.menuType", "menuSettings.animationType", "menuSettings.subMenuDirection", "menuSettings.fields", "menuSettings.cssClass", "menuSettings.contextMenuTarget", "menuSettings.excludeTarget", "menuSettings.height", "menuSettings.width", "menuSettings.titleText", "menuSettings.create", "menuSettings.beforeOpen", "menuSettings.open", "menuSettings.close", "menuSettings.mouseover", "menuSettings.mouseout", "menuSettings.keydown", "menuSettings.destroy"
						    ]
						}]]
		            }]]
		        }]]
		    }]]
		}, {
		    tag: "contextualTabs", attr: ["backgroundColor", "borderColor", [{
		        tag: "tabs", attr: ["id", "text", [{
		            tag: "groups", attr: ["text", "alignType", "type", "contentID", [{
		                tag: "content", attr: ["defaults.type", "defaults.width", "defaults.height", "defaults.isBig", [{
		                    tag: "groups", attr: ["id", "columns", "itemHeight", "itemWidth", "expandedColumns", "contentID", "text", "type", "toolTip", "width",
                            "enableSeparator",
                            "buttonSettings.contentType", "buttonSettings.type", "buttonSettings.imagePosition", "buttonSettings.prefixIcon", "buttonSettings.click", "buttonSettings.width", "buttonSettings.cssClass", "buttonSettings.enabled", "buttonSettings.showRoundedCorner", "buttonSettings.enableRTL", "buttonSettings.repeatButton", "buttonSettings.size", "buttonSettings.height", "buttonSettings.text", "buttonSettings.timeInterval", "buttonSettings.create", "buttonSettings.click", "buttonSettings.destroy", "splitButtonSettings.contentType", "splitButtonSettings.prefixIcon", "splitButtonSettings.targetID",
                            "splitButtonSettings.buttonMode", "splitButtonSettings.arrowPosition", "splitButtonSettings.type", "splitButtonSettings.click",
                            "splitButtonSettings.enabled", "splitButtonSettings.showRoundedCorner", "splitButtonSettings.enableRTL", "splitButtonSettings.size", "splitButtonSettings.height", "splitButtonSettings.imagePosition", "splitButtonSettings.width", "splitButtonSettings.text", "splitButtonSettings.suffixIcon", "splitButtonSettings.create", "splitButtonSettings.itemMouseOver", "splitButtonSettings.itemSelected", "splitButtonSettings.beforeOpen", "splitButtonSettings.destroy", "splitButtonSettings.cssClass",
                            "dropdownSettings.dataSource", "dropdownSettings.text", "dropdownSettings.value", "dropdownSettings.select", "dropdownSettings.width", "dropdownSettings.showRoundedCorner", "dropdownSettings.showPopupOnLoad", "dropdownSettings.allowMultiSelection", "dropdownSettings.enableRTL", "dropdownSettings.enabled", "dropdownSettings.caseSensitiveSearch", "dropdownSettings.showCheckbox", "dropdownSettings.checkAll", "dropdownSettings.uncheckAll", "dropdownSettings.enablePersistence", "dropdownSettings.enableIncrementalSearch", "dropdownSettings.readOnly", "dropdownSettings.allowGrouping", "dropdownSettings.fields",
                            "dropdownSettings.selectedItems", "dropdownSettings.itemsCount", "dropdownSettings.height", "dropdownSettings.cssClass", "dropdownSettings.itemValue", "dropdownSettings.popupHeight", "dropdownSettings.targetID", "dropdownSettings.waterMarkText", "dropdownSettings.template", "dropdownSettings.cascadeTo", "dropdownSettings.query", "dropdownSettings.create", "dropdownSettings.popupHide", "dropdownSettings.popupShown", "dropdownSettings.beforePopupShown", "dropdownSettings.change", "dropdownSettings.checkChange", "dropdownSettings.enableAnimation", "dropdownSettings.destroy",
                            "toggleButtonSettings.contentType", "toggleButtonSettings.defaultText", "toggleButtonSettings.activeText", "toggleButtonSettings.type", "toggleButtonSettings.defaultPrefixIcon", "toggleButtonSettings.activePrefixIcon", "toggleButtonSettings.click", "toggleButtonSettings.enabled", "toggleButtonSettings.showRoundedCorner", "toggleButtonSettings.enableRTL", "toggleButtonSettings.enablePersistence", "toggleButtonSettings.toggleState",
                            "toggleButtonSettings.preventToggle", "toggleButtonSettings.size", "toggleButtonSettings.imagePosition", "toggleButtonSettings.height", "toggleButtonSettings.width", "toggleButtonSettings.cssClass",
                            "toggleButtonSettings.defaultText", "toggleButtonSettings.activeText", "toggleButtonSettings.defaultPrefixIcon",
                            "toggleButtonSettings.defaultSuffixIcon", "toggleButtonSettings.activePrefixIcon", "toggleButtonSettings.create", "toggleButtonSettings.change", "toggleButtonSettings.destroy",
                            "customToolTip.title", "customToolTip.content", "customToolTip.prefixIcon", [{
                                tag: "galleryItems", attr: ["text", "toolTip", "customToolTip.title", "customToolTip.content", "customToolTip.prefixIcon", "buttonSettings.contentType", "buttonSettings.type", "buttonSettings.imagePosition", "buttonSettings.prefixIcon", "buttonSettings.click", "buttonSettings.width", "buttonSettings.cssClass", "buttonSettings.enabled", "buttonSettings.showRoundedCorner", "buttonSettings.enableRTL", "buttonSettings.repeatButton", "buttonSettings.size", "buttonSettings.height", "buttonSettings.text", "buttonSettings.timeInterval", "buttonSettings.create", "buttonSettings.click", "buttonSettings.destroy"]
                            }, {
                                tag: "customGalleryItems", attr: ["text", "toolTip", "customToolTip.title", "customToolTip.content", "customToolTip.prefixIcon", "menuId", "customItemType",
        "buttonSettings.contentType", "buttonSettings.type", "buttonSettings.imagePosition", "buttonSettings.prefixIcon", "buttonSettings.click", "buttonSettings.width", "buttonSettings.cssClass", "buttonSettings.enabled", "buttonSettings.showRoundedCorner", "buttonSettings.enableRTL", "buttonSettings.repeatButton", "buttonSettings.size", "buttonSettings.height", "buttonSettings.text", "buttonSettings.timeInterval", "buttonSettings.create", "buttonSettings.click", "buttonSettings.destroy",
                                "menuSettings.openOnClick", "menuSettings.enableCenterAlign", "menuSettings.showRooltLevelArrows", "menuSettings.showSubLevelArrows", "menuSettings.enableSeparator", "menuSettings.enabled", "menuSettings.orientation", "menuSettings.menuType", "menuSettings.animationType", "menuSettings.subMenuDirection", "menuSettings.fields", "menuSettings.cssClass", "menuSettings.contextMenuTarget", "menuSettings.excludeTarget", "menuSettings.height", "menuSettings.width", "menuSettings.titleText", "menuSettings.create", "menuSettings.beforeOpen", "menuSettings.open", "menuSettings.close", "menuSettings.mouseover", "menuSettings.mouseout", "menuSettings.keydown", "menuSettings.destroy"
                                ]
                            }]]
		                }]]
		            }]]
		        }]]
		    }]
		    ]
		}],
        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "selectedItemIndex":
                        if (this.model.selectedItemIndex > 0 && this._tabObj.model.disabledItemIndex.indexOf(this.model.selectedItemIndex) === -1 && this._ribbonTabs.eq(this.model.selectedItemIndex).is(':visible'))
                            this._tabObj.option({ selectedItemIndex: this.model.selectedItemIndex });
                        break;
                    case "disabledItemIndex":
                        var conParent, disableLen = this.model.disabledItemIndex.length;
                        var curDisabled = this.model.disabledItemIndex, aTag = this.element.find('.e-link'), activeTab;
                        this.model.disabledItemIndex.push(0);
                        this._tabObj.option({ disabledItemIndex: this.model.disabledItemIndex });
                        for (var k = 0; k < disableLen; k++) {
                            conParent = this._ribbonTabs.eq(curDisabled[k]).parent().parent();
                            if (conParent.hasClass('e-contextli') || conParent.hasClass('e-contextliset'))
                                conParent.addClass("e-disable");
                            if (this._tabObj.model.selectedItemIndex != 0 && this._tabObj.model.selectedItemIndex === curDisabled[k]) {
                                activeTab = aTag.eq(curDisabled[k]).attr("href");
                                $(activeTab).append(this.element.find("#" + this._id + "_disabled"));
                                $(activeTab).css("position", "relative");
                                this.element.find("#" + this._id + "_disabled").show();
                            }
                        }
                        break;
                    case "enabledItemIndex":
                        var conParent, enableLen = this.model.enabledItemIndex.length, conTabs = this._tabUl.find(".e-contextualtabset");
                        for (var k = 0; k < enableLen; k++) {
                            if (this.model.enabledItemIndex[k] === 0 || !(this._ribbonTabs.eq(this.model.enabledItemIndex[k]).is(':visible'))) {
                                this.model.enabledItemIndex.splice(k, 1);
                                --enableLen;
                                --k;
                            }
                        }

                        this._conTabsRemove();
                        this._tabObj.option({ enabledItemIndex: this.model.enabledItemIndex });
                        this._contextualTabs();
                        for (var i = 0; i < this.model.enabledItemIndex.length; i++) {
                            this._tabText.eq(this.model.enabledItemIndex[i]).parent().removeClass("e-disable");
                            this._tabText.eq(this.model.enabledItemIndex[i]).parent().css("position", "");
                            conParent = this._ribbonTabs.eq(this.model.enabledItemIndex[i]).parent().parent();
                            if (conParent.hasClass('e-contextli') || conParent.hasClass('e-contextliset'))
                                conParent.removeClass("e-disable");
                            if (this.model.enabledItemIndex[i] == this._tabObj.model.selectedItemIndex)
                                this.element.find("#" + this._id + "_disabled").hide();
                        }
                        for (var j = 0; j < conTabs.length; j++) {
                            if ($(conTabs[j]).hasClass('e-disable'))
                                $(conTabs[j]).parents(".e-contextliset").addClass('e-disable');
                        }
                        break;
                    case "allowResizing":
                        this._ribbonWindowResize();
                        break;
                    case "width":
                        this.element.width(options[key]);
                        this._ribbonWindowResize();
                        break;
                }
            }
        },


        _setTabContentHeight: function () {
            var maxGrpHeight = 0, tabContent = this.element.find('.e-content'),ctrlSeparatorCol,ctrlSeparatorLen;
            $(tabContent).addClass('e-content-show');
			ctrlSeparatorCol=this.element.find('.e-separatordivrow'),ctrlSeparatorLen=ctrlSeparatorCol.length;
			for(var i=0;i<ctrlSeparatorLen;i++)
			  $(ctrlSeparatorCol).eq(i).height($(ctrlSeparatorCol).eq(i).prev().height());
            this.element.find('.e-ribGroupContent').each(function () { maxGrpHeight = Math.max(maxGrpHeight, $(this).height()); });
            this.element.find(".e-ribGroupContent").height(maxGrpHeight);
            $(tabContent).removeClass('e-content-show');
        },

        _init: function () {
            if (!ej.isNullOrUndefined(this.element)) {
                this._render();
                this._wireEvents();
            }
        },
        _render: function () {
            this._renderTab();
            this._initPrivateProperties();
            this._groupingControls();
            this._customization();
		    this._initialRender=true;
			this._tabObj.initialRender=false;
            this.element.append(ej.buildTag("div#" + this._id + "_disabled"));
            var content = this.element.find(".e-content");
            this.element.find("#" + this._id + "_disabled").height($(content).height()).width(this.model.width);
            this.element.find("#" + this._id + "_disabled").css({ top: 0, left: 0, position: "absolute" }).addClass("e-disable");
            this.element.find("#" + this._id + "_disabled").hide();
            if (this.model.disabledItemIndex.length === 0) {
                this.model.disabledItemIndex.push(0);
                this._tabObj.option({ disabledItemIndex: this.model.disabledItemIndex });
            }
            var resizeDiv = ej.buildTag("div#" + this._id + "_resize").addClass("e-resizediv").hide().click($.proxy(this._onResizeDivClick, this));
            this.element.append(resizeDiv);
            var toolTipDiv = ej.buildTag("div#" + this._id + "_tooltip").addClass("e-tooltipdiv").hide();
            var title = ej.buildTag("div#" + this._id + "_toolTip_title").addClass('e-tooltiptitle');
            var description = ej.buildTag("div#" + this._id + "_toolTip_Desc").addClass('e-tooltipdesc');
            toolTipDiv.append(title);
            toolTipDiv.append(description);
            this.element.append(toolTipDiv);
            if (this._applicationTab.hasClass('e-active')) {
                this._applicationTab.removeClass("e-active");
                this.element.find('.e-content').remove();
                this._tabUl.find('.e-expandcollapse').remove();
            }
            var emptyTabContent = this.element.find('.e-empty-content'), firstGroupDiv = this.element.find('.e-groupdiv').eq(0);
            if ($(emptyTabContent).length > 0 && $(firstGroupDiv).length > 0)
                $(emptyTabContent).height($(firstGroupDiv).parent().height());
        },
        _tooltip: function (e) {
            var control = this.element.find('.e-controlclicked');
            if (control.length <= 0 || $(control).find('.e-ribbonbtn').length > 0) {
                if (this.element.find("#" + e.data.ribbonId + "_resize").is(':visible') || this.element.find('.e-gallexpandcontent').is(':visible')) {
                    if ($(e.target).parents(".e-resizediv").length > 0 || $(e.target).parents('.e-gallexpandcontent').length > 0)
                        this._toolTipShow(e);
                }
                else
                    this._toolTipShow(e);
            }
        },
        _toolTipShow: function (e) {
            var toolTipDiv = this.element.find("#" + e.data.ribbonId + "_tooltip"), control,controlOffset, tipTop, resizeDiv, tipDesc, tipTitle, expandContent, title, cntOverflow,ribHeight;
            if (toolTipDiv.find('.e-tooltiptitle').length <= 0) {
                title = ej.buildTag("div#" + e.data.ribbonId + "_toolTip_title").addClass('e-tooltiptitle');
                toolTipDiv.prepend(title);
            }
            toolTipDiv.removeClass("e-tooltipdivwithimg");
            tipTitle = toolTipDiv.find('.e-tooltiptitle');
            tipTitle.children().remove();
            if (!ej.isNullOrUndefined(e.data.value.customToolTip.title))
                tipTitle.append(e.data.value.customToolTip.title);
            else
                tipTitle.remove();

            if (!ej.isNullOrUndefined(e.data.value.customToolTip.content)) {
                if (tipTitle.children().length <= 0)
                    tipTitle.empty().append("<b>" + e.data.value.customToolTip.title + "</b>");
                tipDesc = toolTipDiv.find('.e-tooltipdesc');
                tipDesc.children().remove();
                tipDesc.append(e.data.value.customToolTip.content);
                if (tipDesc.children().length <= 0)
                    tipDesc.empty().append("<h6>" + e.data.value.customToolTip.content + "</h6>");
                if (!ej.isNullOrUndefined(e.data.value.customToolTip.prefixIcon)) {
                    tipDesc.prepend("<span class=e-tooltipimg></span>");
                    tipDesc.find('.e-tooltipimg').addClass(e.data.value.customToolTip.prefixIcon);
                    tipDesc.children().not('.' + e.data.value.customToolTip.prefixIcon).addClass('e-tooltipcntwithimg');
                    toolTipDiv.addClass("e-tooltipdivwithimg");
                }
                else
                    tipDesc.children().addClass('e-tooltipcontent');
            }
            controlOffset = this.element.find("#" + e.delegateTarget.id).offset();
			control=this.element.find("#" + e.delegateTarget.id);
            toolTipDiv.stop(true).delay(700).show(0).addClass("e-rbntooltipshadow");
            resizeDiv = this.element.find("#" + e.data.ribbonId + "_resize");
            expandContent = $(e.target).parents('.e-gallexpandcontent');
            this.element.css({ 'position': 'relative' });
            cntOverflow = (this.element.width() + this.element.offset().left) - (controlOffset.left + parseInt($(toolTipDiv).width(), 10));
            if (cntOverflow <= 0)
                controlOffset.left = controlOffset.left + cntOverflow - this.element.offset().left;
            else
                controlOffset.left = controlOffset.left - this.element.offset().left;
			ribHeight=this._tabUl.height()+this.element.find('.e-active-content').height();
			
            if (resizeDiv.is(':visible') && !expandContent.is(':visible')){
			   toolTipDiv.offset({ left: controlOffset.left });
			   if(control.parents('.e-galleryrow').length>0)
			    tipTop=ribHeight+control.position().top+control.parents('.e-galleryrow').height();
			   else
				tipTop=ribHeight+control.position().top+control.height();
				toolTipDiv.css({'top':tipTop});
			}
            else if (expandContent.is(':visible')) {
			    var galRow;
			    if(control.parents('.e-galleryrow').length>0)
				  galRow=control.parents('.e-galleryrow');
				else if(control.parents('.e-galleryexpanderrow').length>0)
				  galRow=control.parents('.e-galleryexpanderrow');
				else
				  galRow=control;
                if ($(expandContent).parents('.e-resizediv').length > 0)
                    tipTop = galRow.height() + ribHeight + control.position().top+12;
                else
                    tipTop = galRow.height() + this._tabUl.height() + control.position().top+12;
				toolTipDiv.offset({ left: controlOffset.left });
				toolTipDiv.css({'top':tipTop});
            }
            else{
                tipTop = ribHeight+3;
                toolTipDiv.offset({ top: tipTop, left: controlOffset.left });
			}
        },
        _toolTipOut: function (e) {
            var toolTipDiv = this.element.find("#" + e.data.ribbonId + "_tooltip");
            toolTipDiv.hide(0);
            toolTipDiv.css({ top: '', left: '' });
            toolTipDiv.find('.e-tooltiptitle').children().remove();
            toolTipDiv.find('.e-tooltipdesc').children().remove();
        },
        _onGroupClick: function (args) {
            this._toolTipCustomization(args),parentElem=$(args.target).parents('.e-resizebtnselect');
			this.element.find('.e-resizebtnselect').removeClass('e-resizebtnselect e-active');
			if($(args.target).hasClass('e-resizebtnselect'))
			 $(args.target).removeClass('e-resizebtnselect e-active');
			else{
			 if(parentElem.length>0)
			  parentElem.removeClass('e-resizebtnselect e-active');
			 else{ 
			  parentElem=$(args.target).parents('.e-resizebtn');
			  if(parentElem.length>0)
			   parentElem.addClass('e-resizebtnselect e-active');
			  else if($(args.target).hasClass('e-resizebtn'))
			   $(args.target).addClass('e-resizebtnselect e-active');
			 }
			}
            if ($(args.target).parents('.e-ribbongallerycontrol').length === 0) {
                this._trigger("groupClick", args);
                if (!args.cancel && ej.raiseWebFormsServerEvents && !ej.isNullOrUndefined(this.model.serverEvents)) {
                    if (this.model.serverEvents.indexOf("groupClick") != -1) {
                        var modelRibbon = $.extend(true, {}, this.model), controlId;
                        controlId = (args.target.id) ? args.target.id : args.target.parentNode.id;
                        var argsRibbon = { model: modelRibbon, originalEventType: "groupClick", id: controlId };
                        var clientArgs = { model: modelRibbon, id: controlId };
                        ej.raiseWebFormsServerEvents("groupClick", argsRibbon, clientArgs);
                    }
                }
            }
        },
		_onResizeDivClick: function (args) {
           this._toolTipCustomization(args);
        },
		_toolTipCustomization: function (args) {
		    var control = $(args.target).parents('.e-controlpadding');
            if ($(control).find('.e-ribbongallerycontrol').length <= 0 && $(args.target).find('.e-disable').length <= 0 && (!$(args.target).hasClass('e-disable'))) {
                this.element.find(".e-tooltipdiv").hide(0);
                if ($(control).hasClass('e-controlclicked'))
                    $(control).removeClass('e-controlclicked');
                else
                    $(control).addClass('e-controlclicked');
            }
		},
        _onTabClick: function (args) {
            var expandCollapse = this.element.find('.e-rarrowup-2x'), active = this.element.find('li.e-active');
            var collapse = this.element.find('.e-collapseactive'), aTag, href;
            var activeContent = this.element.find('.e-active-content');

            if (active.length > 0) {
                active.removeClass("e-active");
				active.children().removeClass("e-active");
                activeContent.removeClass("e-active-content").hide();
            }
            else {
                if (collapse.length > 0) {
                    if (args.activeHeader.textContent === collapse.find('.e-link').text()) {
                        collapse.addClass("e-active").removeClass("e-collapseactive");
                        activeContent.removeClass("e-collapse-content").slideDown("fast", "swing", $.proxy(this._ribExpand, this));
                    }
                    else{
                        aTag = this.element.find('.e-link');
                        collapse.removeClass("e-collapseactive");
                        this.element.find('.e-collapse-content').removeClass("e-collapse-content");
                        activeContent.removeClass("e-active-content");
                        href = aTag.eq(args.activeIndex).attr("href");
                        aTag.eq(args.activeIndex).addClass("e-active");
                        $(href).slideDown("fast", "swing", $.proxy(this._ribExpand, this));
                    }
                }
            }
            if (expandCollapse.length > 0 && this.element.find(".e-collapsed").length>0)
			    this._addRibbonPin();
			if(!this._initialRender)
             this._trigger("tabClick", args);
        },
        _onTabSelect: function (args) {
            var active = this.element.find('li.e-active'), aTag, href, disable,expCollapse=this.element.find('.e-expandcollapse');
            this.model.selectedItemIndex = args.activeIndex;
            if (!active.hasClass("e-disable")) {
                aTag = this.element.find('.e-link');
                href = aTag.eq(args.activeIndex).attr("href");
                disable = $(href).find("#" + this._id + "_disabled");
                if (disable)
                    this.element.find("#" + this._id + "_disabled").hide();
            }
			if(expCollapse.length>0)
			  expCollapse.appendTo(this.element.find('.e-active-content'));
            this._ribbonResize();
			if(!this._initialRender){
            this._trigger("tabSelect", args);
            if (ej.isOnWebForms && ej.raiseWebFormsServerEvents && !ej.isNullOrUndefined(this.model.serverEvents)) {
                if (this.model.serverEvents.indexOf("tabSelect") != -1) {
                    var modelRibbon = $.extend(true, {}, this.model);
                    var argsRibbon = { model: modelRibbon, originalEventType: "tabSelect", activeIndex: args.activeIndex, prevActiveIndex: args.prevActiveIndex };
                    var clientArgs = { model: modelRibbon, activeIndex: args.activeIndex, prevActiveIndex: args.prevActiveIndex };
                    ej.raiseWebFormsServerEvents("tabSelect", argsRibbon, clientArgs);
                }
            }
		  }
		  this._initialRender=false;
        },
        _create: function (args) {
            this._trigger("tabCreate", args);
        },
        _onGalMoveUpClick: function (args) {
            var gallery = this.element.find("#" + args.data.galleryId);
            var prevElem = gallery.find(".e-gallerycontent").children(':visible').first().prev();
            if (this.element.find('.e-gallexpandcontent:visible').length > 0)
                this._ribbonGalleryShow();
            if (prevElem.length > 0) {
                gallery.find(".e-gallerycontent").children(':visible').last().hide();
                prevElem.show().width(gallery.find('.e-galleryrow').first().width());
                if ($(prevElem).hasClass('e-gryfirstrow'))
                    gallery.find(".e-moveupdiv").addClass("e-disablegrymovebtn");
                gallery.find(".e-movedowndiv").removeClass("e-disablegrymovebtn");
            }
        },
        _onGalMoveDownClick: function (args) {
            var gallery = this.element.find("#" + args.data.galleryId);
            var nextElem = gallery.find(".e-gallerycontent").children(':visible').last().next();
            if (this.element.find('.e-gallexpandcontent:visible').length > 0)
                this._ribbonGalleryShow();
            if (nextElem.length > 0) {
                gallery.find(".e-gallerycontent").children(':visible').first().hide();
                nextElem.show().width(gallery.find('.e-galleryrow').first().width());
                if ($(nextElem).hasClass('e-grylastrow'))
                    gallery.find(".e-movedowndiv").addClass("e-disablegrymovebtn");
                gallery.find(".e-moveupdiv").removeClass("e-disablegrymovebtn");
            }
        },
        _onGalContentClick: function (args) {
            var gallery = this.element.find("#" + args.data.galleryId);
            if ($(args.target).hasClass("e-gallerybtn")) {
                gallery.find(".e-galleryselect").removeClass("e-galleryselect");
                $(args.target).addClass("e-galleryselect");
            }
            else if ($(args.target).parents(".e-gallerybtn")) {
                gallery.find(".e-galleryselect").removeClass("e-galleryselect");
                $(args.target).parents(".e-gallerybtn").addClass("e-galleryselect");
            }
        },
        _onExpandContentClick: function (args) {
            var gallery = this.element.find("#" + args.data.galleryId);
            if ($(args.target).hasClass("e-gallerybtn")) {
                gallery.parent().find(".e-galleryselect").removeClass("e-galleryselect");
                $(args.target).addClass("e-galleryselect");
            }
            else if ($(args.target).parents(".e-gallerybtn")) {
                gallery.parent().find(".e-galleryselect").removeClass("e-galleryselect");
                $(args.target).parents(".e-gallerybtn").addClass("e-galleryselect");
            }
        },
        _onExpandGalleryClick: function (args) {
            var rowLen, gallery = this.element.find("#" + args.data.galleryId), expcolumns = args.data.expandedColumns;
            if (this.element.find('.e-gallexpandcontent:visible').length > 0)
                this._ribbonGalleryShow();
            if (ej.isNullOrUndefined(args.data.expandedColumns))
                expcolumns = args.data.columns;
            rowLen = Math.floor(gallery.find('.e-galleryrow').children().length / expcolumns);
            if (gallery.find('.e-galleryrow').children().length % expcolumns > 0)
                ++rowLen;
            for (var m = 1; m <= rowLen; m++) {
                var galleryExpanderRow = ej.buildTag("div#" + gallery.attr("id") + "_galleryExpanderRow_" + m).addClass('e-galleryexpanderrow').click({ model: gallery }, $.proxy(this._onGalleryItemClick, this));
                for (var n = 0; n < expcolumns; n++)
                    gallery.find('.e-galleryrow').children().eq(0).appendTo(galleryExpanderRow);
                galleryExpanderRow.appendTo(this.element.find("#" + gallery.attr("id")).parent().find('.e-expandercontent'));
            }
            gallery.hide();
            for (var i = 0; i < args.data.columns; i++)
                gallery.find('.e-galleryrow').eq(0).append("<div></div>");
            if (this.element.find('.e-resizediv').is(":visible")) {
                this.element.find('.e-resizediv').css('visibility', 'hidden');
                this.element.find('.e-resizediv .e-gallexpandcontent').css('visibility', 'visible');
            }
            this.element.find("#" + gallery.attr("id")).parent().find(".e-gallexpandcontent").show();
            if (this.element.find("#" + gallery.attr("id")).parent().find('.e-expandercontent').height() > 160)
                this.element.find("#" + gallery.attr("id")).parent().find(".e-gallscrollcontent").ejScroller({ height: 160 });
        },
        _onGalleryItemClick: function (e) {
            var args, target, galleryModel, cancel;
            args = { target: e.target, galleryModel: e.data.model, cancel: false };
            if (!$(args.target).hasClass("e-galleryexpanderrow")) {
                this._trigger("galleryItemClick", args);
                e.preventDefault();
                if (ej.raiseWebFormsServerEvents && !ej.isNullOrUndefined(this.model.serverEvents)) {
                    if (this.model.serverEvents.indexOf("galleryItemClick") != -1) {
                        var modelRibbon = $.extend(true, {}, this.model), controlId;
                        controlId = (args.target.id) ? args.target.id : args.target.parentNode.id;
                        var argsRibbon = { model: modelRibbon, originalEventType: "galleryItemClick", id: controlId };
                        var clientArgs = { model: e.data.model, id: controlId };
                        ej.raiseWebFormsServerEvents("galleryItemClick", argsRibbon, clientArgs);
                    }
                }
            }
        },
		_ribbonKeyDown: function (e) {
		    if (e.keyCode === 40 || e.keyCode === 39) 
                if (this._tabUl.find('.e-tab:visible,.e-contextualtabset:visible').length === this._tabObj.model.selectedItemIndex)
                    e.stopImmediatePropagation();
		},
        _OnKeyDown: function (e) {
		    if(e.keyCode === 27){
			    var resizeDiv=this.element.find('.e-resizediv');
 			    if(this.element.find('.e-ribbonbackstagepage').is(':visible'))
 			      this.hideBackstage();
				else if($('.e-menu-wrap').children('.e-split:visible').length <= 0 && resizeDiv.find('.e-ddl').parents('.e-controlclicked').length <= 0 && resizeDiv.is(':visible'))
 				  this._resizeDivHide();
 				else if(this.element.find('.e-gallexpandcontent').is(':visible'))
 				  this._ribbonGalleryShow();
				if(resizeDiv.is(':visible'))
				 resizeDiv.find('.e-controlclicked').removeClass('e-controlclicked');
 			}
        },
        _ribCollapse: function (args) {
            this._trigger("collapse", args);
        },
        _ribExpand: function (args) {
		  if(this.element.find('.e-ribbonpin').length>0)
		   this._tabContents.width(this.element.width());
            this._trigger("expand", args);
        },
        collapse: function () {
            var activeContent = this.element.find('.e-active-content'), togglecontent = this.element.find("#" + this._id + "_togglebutton");
            activeContent.slideUp("fast", "swing", $.proxy(this._ribCollapse, this));
            this.element.find('li.e-active').removeClass("e-active").addClass("e-collapseactive e-select");
            activeContent.addClass("e-collapse-content");
            this._selectedItemIndex = this._tabObj.model.selectedItemIndex;
            this._tabObj.option({ selectedItemIndex: 0 });
            this._tabObj.model.selectedItemIndex = 0;
            this.model.selectedItemIndex = 0;
            togglecontent.removeClass("e-expanded").addClass("e-collapsed");
            this._isCollapsed = true;
        },

        expand: function () {
            var activeContent = this.element.find('.e-active-content'), togglecontent = this.element.find("#" + this._id + "_togglebutton");
            activeContent.slideDown("fast", "swing", $.proxy(this._ribExpand, this));
            this.element.find('.e-collapseactive').addClass("e-active").removeClass("e-collapseactive");
            activeContent.removeClass("e-collapse-content");
            this._tabObj.model.selectedItemIndex = this._selectedItemIndex;
            this.model.selectedItemIndex = this._selectedItemIndex
            togglecontent.removeClass("e-collapsed").addClass("e-expanded");
            togglecontent.find("span").addClass("e-rarrowup-2x");
            this._ribbonResize();
			this._addRibbonPin();
        },
		_addRibbonPin:function(){
			if(this.element.find('.e-ribbonpinicon').length<=0){
			 var expandCollapse=this.element.find('.e-expandcollapse'),expandIcon=this.element.find('.e-rarrowup-2x');
             expandIcon.parent().removeClass("e-collapsed").addClass("e-expanded e-ribbonpin");
             expandIcon.removeClass("e-rarrowup-2x").addClass("e-ribbonpinicon");
             this._isCollapsed = false;
			}
		   this._tabContents.css({'position':'absolute'}).width(this.element.width());
		},
		_removeRibbonPin:function(){
		  var togglecontent = this.element.find("#" + this._id + "_togglebutton");
		  this._tabContents.css({'position':'','width':''});
		  togglecontent.removeClass("e-ribbonpin");
		  togglecontent.find("span").removeClass("e-ribbonpinicon").addClass("e-rarrowup-2x");
		},
		_onRbnPinDivClick: function (args) {
		  this._removeRibbonPin();
		},
        _onGroupExpandClick: function (e) {
            this._trigger("groupExpand", e);
        },
        _onBackStageItemClick: function (args) {
            var iconHeight, backStageId, backStageText, argsBackStage, backStageHeight;
            args.preventDefault();
            if ($(args.target).hasClass("e-backstagebutton"))
                this.element.find(".e-ribbonbackstagepage").hide();
            this.element.find('.e-backstageactive-content').removeClass("e-backstageactive-content").hide();
            this.element.find(".e-backstageactive").removeClass("e-backstageactive");
            this.element.find(args.data.contentDivId).addClass("e-backstageactive-content").show();
            if ($(args.target).hasClass("e-backstageli"))
                $(args.target).addClass("e-backstageactive");
            else
                $(args.target).parents(".e-backstageli").addClass("e-backstageactive");
            iconHeight = this.element.find('.e-ribbonbackstagetop').height();
            this.element.find(".e-backstagescrollcontent").ejScroller({ height: 0 });
            backStageHeight = args.data.height;
            this._backStageHeightCalculate(backStageHeight, iconHeight);
            if (!ej.isNullOrUndefined($(args.target).text()))
                backStageText = $(args.target).text();
            else
                backStageText = $(args.target).children('a').text();
            if (!ej.isNullOrUndefined($(args.target).attr('id')))
                backStageId = $(args.target).attr('id');
            else
                backStageId = $(args.target).parent().attr('id');
            argsBackStage = { id: backStageId, text: backStageText, target: args.target, type: "backstageItemClick", cancel: false };
			this.element.find(".e-backstagetitlecontent").text('').append(backStageText);
            this._trigger("backstageItemClick", argsBackStage);
        },
        _onBackStageTopIcon: function (args) {
            this.hideBackstage();
        },
        _backStageHeightCalculate: function (backStageHeight, iconHeight) {
            if (!ej.isNullOrUndefined(backStageHeight)) {
			    if(backStageHeight.toString().endsWith('%'))
				  backStageHeight=this.element.find(".e-ribbonbackstagepage").height();
				else if(backStageHeight.toString().endsWith('px'))
                  backStageHeight=parseInt(backStageHeight,10);
				 if(this.model.applicationTab.backstageSettings.height !='100%'){
                if (this.element.find(".e-backstageactive-content").height() > backStageHeight - iconHeight)
                    this.element.find(".e-backstagescrollcontent").ejScroller({ height: backStageHeight - iconHeight });
            else {
                if (this.element.find(".e-backstageactive-content").height() > this.element.height() - iconHeight)
                    this.element.find(".e-backstagescrollcontent").ejScroller({ height: this.element.height() - iconHeight });
             }
		    }
		  } 
        },
        _onApplicationTabClick: function (args) {
            this.showBackstage();
        },

        showBackstage: function () {
            var headerWidth, argsBackStage, iconHeight, backStageHeight, target, backStageText,backStageLi;
            if (this.model.applicationTab.type === ej.Ribbon.applicationTabType.backstage) {
                if (ej.isNullOrUndefined(this.model.applicationTab.backstageSettings.height)) {
                    this.element.find(".e-ribbonbackstagepage").height(this.element.height()).width(this.element.width());
                    this.element.find(".e-ribbonbackstagebody").height(this.element.height() - 100);
                    this.element.find(".e-backstageheader").height(this.element.height() - 100);
                }
                this.element.css({ "position": "relative" });
                this.element.find(".e-ribbonbackstagepage").show();
                headerWidth = this.element.find('.e-backstageheader').width();
                this.element.find('.e-backstagetopcontent').width(headerWidth);
                this.element.find('.e-backstageheader').width(headerWidth);
				backStageLi=this.element.find(".e-backstageli");
				backStageLi.width(backStageLi.width());
                iconHeight = this.element.find('.e-ribbonbackstagetop').height();
                this.element.find(".e-backstageactive-content").removeClass("e-backstageactive-content").hide();
                this.element.find(".e-backstageactive").removeClass("e-backstageactive");
                backStageLi.first().addClass("e-backstageactive");
                this.element.find('.e-backstagecontent').first().addClass("e-backstageactive-content").show();
                this.element.find(".e-backstagescrollcontent").ejScroller({ height: 0 });
                backStageHeight = this.model.applicationTab.backstageSettings.height;
                this._backStageHeightCalculate(backStageHeight, iconHeight);
                target = this.element.find('.e-backstageli').first();
                if (!ej.isNullOrUndefined($(target).text()))
                    backStageText = $(target).text();
                else
                    backStageText = $(target).children('a').text();
                argsBackStage = { id: $(target).attr('id'), text: backStageText, target: $(target).children(), type: "backstageItemClick", cancel: false };
				this.element.find(".e-backstagetitlecontent").text('').append(backStageText);
                this._trigger("backstageItemClick", argsBackStage);
            }
        },
        hideBackstage: function () {
            this.element.find(".e-ribbonbackstagepage").hide();
        },
        _initPrivateProperties: function () {
		    this._initialRender=false;
            this._tabUl = this.element.find('.e-header');
            this._applicationTab = this.element.find('.e-apptab');
            this._ribbonTabs = this._tabUl.find(":not('a')");
            this._tabText = this.element.find('.e-link');
            this._id = this.element.attr("id");
            this._tabObj = this.element.data("ejTab");
            this._isCollapsed = false;
            this._contextualTabSet = "";
            this._resizeWidth = 0;
            this._selectedItemIndex = 0;
			this._tabContents=this.element.children().attr('role','tabpanel').not('.e-header,.e-disable,.e-resizediv,.e-tooltipdiv,.e-ribbonbackstagepage');
        },
        _getLocalizedLabels: function (property) {
            return ej.Ribbon.Locale[this.model.locale][property] === undefined ? ej.Ribbon.Locale["en-US"][property] : ej.Ribbon.Locale[this.model.locale][property];
        },
        _customization: function () {
            var toggleelm, model = {}, li;
            var backStagePage, backStageContent, backStageUl, backStageLi, aTag, backStageScrollContent, backStageTop, backStageBody, backStageTopContent, backStageTitleContent, backStageTopIcon, bottomSeparator;
            this._contextualTabs();
			toggleelm = '<div id=' + this._id + '_togglebutton class="e-expanded"><span class="e-icon e-rarrowup-2x"></span></div>';
            li = ej.buildTag("li#" + this._id + "_" + "expandCollapse.e-expandcollapse", toggleelm);
			this.element.find('.e-active-content').append(li);
            if (this.model.applicationTab.type === ej.Ribbon.applicationTabType.menu) {
                this._applicationTab.find('.e-link').hide();
                if (this.model.applicationTab.menuSettings)
                    model = this.model.applicationTab.menuSettings;
				if(this.model.applicationTab.menuItemID){
                $("#" + this.model.applicationTab.menuItemID).appendTo(this._applicationTab);
                $("#" + this.model.applicationTab.menuItemID).ejMenu(model);
				}
                $("#" + this._id + "_" + this.model.applicationTab.type.replace(/\s/g, '')).hide();
            }
            else {
                this._applicationTab.find('.e-link').text(this.model.applicationTab.backstageSettings.text);
            }
            this._applicationTab.click($.proxy(this._onApplicationTabClick, this));
            this.element.find('.e-content').css("box-sizing", "content-box");
            if (this.model.applicationTab.type === ej.Ribbon.applicationTabType.backstage) {
			    var backStageHeight=this.model.applicationTab.backstageSettings.height;
			    if(backStageHeight.toString().endsWith('%'))
				  backStageHeight=$(document).height()*(parseInt(backStageHeight.slice(0,-1),10)/100);
				else if(backStageHeight.toString().endsWith('px'))
                  backStageHeight=parseInt(backStageHeight,10);
                backStagePage = ej.buildTag("div#" + this._id + "_BackStage").addClass('e-ribbonbackstagepage').height(backStageHeight).width(this.model.applicationTab.backstageSettings.width).hide();
                backStageBody = ej.buildTag("div#" + this._id + "_BackStageBody").addClass('e-ribbonbackstagebody').height(backStageHeight - 50);
                backStageUl = ej.buildTag("ul#" + this._id + "_BackStageHeader").addClass('e-backstageheader').height(backStageHeight - 50);
                for (var i = 0; i < this.model.applicationTab.backstageSettings.pages.length; i++) {
                    backStageLi = ej.buildTag("li").addClass('e-backstageli').click({ contentDivId: "#" + this._id + "_BackStage_" + this.model.applicationTab.backstageSettings.pages[i].id.replace(/\s/g, ''), height: this.model.applicationTab.backstageSettings.height }, $.proxy(this._onBackStageItemClick, this));
                    if (ej.isNullOrUndefined(this.model.applicationTab.backstageSettings.pages[i].itemType))
                        this.model.applicationTab.backstageSettings.pages[i].itemType = ej.Ribbon.itemType.tab;
                    if (this.model.applicationTab.backstageSettings.pages[i].itemType == ej.Ribbon.itemType.tab) {
                        aTag = ej.buildTag("a", this.model.applicationTab.backstageSettings.pages[i].text, {}, { href: "#" + this._id + "_BackStage_" + this.model.applicationTab.backstageSettings.pages[i].id.replace(/\s/g, '') });
                        backStageLi.append(aTag).attr('id', this.model.applicationTab.backstageSettings.pages[i].id + "_backStageTab");
                    }
                    else
                        backStageLi.append(this.model.applicationTab.backstageSettings.pages[i].text).addClass('e-backstagebutton').attr('id', this.model.applicationTab.backstageSettings.pages[i].id + "_backStageBtn");
                    backStageUl.append(backStageLi);
                    if (this.model.applicationTab.backstageSettings.pages[i].enableSeparator) {
                        bottomSeparator = ej.buildTag("li").addClass('e-backstageseparator');
                        backStageUl.append(bottomSeparator);
                    }
                    if (this.model.applicationTab.backstageSettings.pages[i].itemType == ej.Ribbon.itemType.tab) {
                        backStageContent = ej.buildTag("div#" + this._id + "_BackStage_" + this.model.applicationTab.backstageSettings.pages[i].id.replace(/\s/g, '')).addClass('e-backstagecontent').css({ "width": "auto" });
                        if (i > 0)
                            backStageContent.hide();
                        if (this.model.applicationTab.backstageSettings.pages[i].contentID) {
                            $('#' + this.model.applicationTab.backstageSettings.pages[i].contentID).find('*').addClass('e-backstagechild');
                            $(backStageContent).append($('#' + this.model.applicationTab.backstageSettings.pages[i].contentID));
                        }
                        backStageBody.append(backStageContent);
                    }
                }
                backStageBody.prepend(backStageUl);
                backStageTop = ej.buildTag("div#" + this._id + "_BackStageTop").addClass('e-ribbonbackstagetop').height(100);
                backStageTopContent = ej.buildTag("div#" + this._id + "_BackStageTopContent").addClass('e-backstagetopcontent').appendTo(backStageTop);
                backStageTitleContent = ej.buildTag("div#" + this._id + "_BackStageTitleContent").addClass('e-backstagetitlecontent').appendTo(backStageTop);
				backStageTopIcon = ej.buildTag("div#" + this._id + "_BackStageTopIcon", "<span></span>").addClass('e-backstagetopicon').click($.proxy(this._onBackStageTopIcon, this)).appendTo(backStageTopContent);
                backStagePage.append(backStageTop);
                backStageScrollContent = ej.buildTag("div#" + this._id + "_backStageScrollContent", "<div></div>").addClass("e-backstagescrollcontent");
                backStageScrollContent.children().append(backStageBody);
                backStagePage.append(backStageScrollContent);
                this._applicationTab.addClass('e-backstagetab');
                this._applicationTab.find('a').addClass('e-apptabanchor');
				if(!ej.isNullOrUndefined(this.model.applicationTab.backstageSettings.headerWidth)){
				  backStageTopContent.width(this.model.applicationTab.backstageSettings.headerWidth);
				  backStageUl.width(this.model.applicationTab.backstageSettings.headerWidth);
				}
                this.element.prepend(backStagePage);
            }
        },
        _conTabsRemove: function () {
            var cntxTabs = this.element.find(".e-contextli,.e-contextliset"), $cntxTab;
            for (var i = 0; i < cntxTabs.length; i++) {
                $cntxTab = $(cntxTabs[i]);
                $cntxTab.is(":visible") ? $cntxTab.find("li").insertAfter($cntxTab.eq(0)) : $cntxTab.find("li").hide().insertAfter($cntxTab.eq(0));
                $cntxTab.remove();
            }
        },
        _contextualTabs: function () {
            var curcontextualTabSet, li, ul;
            if (this.model.contextualTabs) {
                for (var i = 0; i < this.model.contextualTabs.length; i++) {
                    curcontextualTabSet = this.model.contextualTabs[i];
                    if (curcontextualTabSet.tabs.length > 0) {
                        li = ej.buildTag("li.e-contextliset");
                        this._contextualTabSet = this._tabUl.children('.e-contextualtabset').filter(function (index) { return $(this).text() === curcontextualTabSet.tabs[0].text; });
                        this._contextualTabSet.before(li);
                        this._tabUl.children('.e-contextualtabset').find('a').css("color", curcontextualTabSet.borderColor);
                        ul = ej.buildTag("ul.e-contextul", "", {
                            'background': curcontextualTabSet.backgroundColor,
                            'border-top-color': curcontextualTabSet.borderColor
                        });
                        for (var j = 0; j < curcontextualTabSet.tabs.length; j++) {
                            ul.append(this._tabUl.children('.e-contextualtabset').filter(function (index) {
                                return $(this).text() ===
                                curcontextualTabSet.tabs[j].text;
                            }));
                        }
                        ul.appendTo(li);
                    }
                }
            }
        },
        showContextualTab: function (text) {
            this.element.find(".e-contexttitle").filter(function (index) { return $(this).text() === text; }).parent().parent().show();
        },
        hideContextualTab: function (text) {
            this.element.find(".e-contexttitle").filter(function (index) { return $(this).text() === text; }).parent().parent().hide();
            this._tabObj.option({ selectedItemIndex: this._tabObj.model.selectedItemIndex + 1 });
        },

        hideTab: function (text) {
            var index = this._tabText.map(function (index) { if ($(this).text() === text) return index; })[0], updatedindex, disLen = this._tabObj.model.disabledItemIndex.length, hide = true, conTab;
            var hiddenTabId = this.element.find('.e-link').eq(index).attr('href');
            conTab = this._tabText.filter(function (index) { return $(this).text() === text; }).parents(".e-contextualtabset");
            for (var i = 0; i < disLen; i++) {
                if (this._tabObj.model.disabledItemIndex[i] === index)
                    hide = false;
            }
            if (hide) {
                if (conTab.length > 0)
                    conTab.hide();
                else
                    this._tabText.filter(function (index) { return $(this).text() === text; }).parent().hide();
                if ($(hiddenTabId).hasClass('e-active-content')) {
                    $(hiddenTabId).hide();
                    $(hiddenTabId).removeClass('e-active-content');
                }
            }
            if (this._tabUl.find('.e-active').is(':hidden')) {
                updatedindex = this._getVisibleItemIndex(this._tabText);
                if (updatedindex) {
                    var currentTab = this.element.find('.e-link').eq(updatedindex).attr('href');
                    if ($(this._tabText[updatedindex]).parent().hasClass('e-disable')) {
                        $(this._tabText[index]).parent().removeClass('e-active');
                        $(this._tabText[updatedindex]).parent().addClass('e-active')
                        $(currentTab).append(this.element.find("#" + this._id + "_disabled"));
                        $(currentTab).css("position", "relative");
                        this.element.find("#" + this._id + "_disabled").show();
                        $(currentTab).show();
                        this._tabObj.option("selectedItemIndex", updatedindex);
                    }
                    else
                        this._tabObj.option("selectedItemIndex", updatedindex);
                    this._tabObj.model.selectedItemIndex = updatedindex;
                    this.model.selectedItemIndex = updatedindex;
                }
            }
        },

        showTab: function (text) {
            var index = this._tabText.map(function (index) { if ($(this).text() === text) return index; })[0], conTab;
            var Tab = this._tabText.filter(function (index) { return $(this).text() === text; }).parent();
            conTab = this._tabText.filter(function (index) { return $(this).text() === text; }).parents(".e-contextualtabset");
            if (conTab.length > 0)
                conTab.show();
            else
                this._tabText.filter(function (index) { return $(this).text() === text; }).parent().show();
            if (!Tab.hasClass('e-active'))
                Tab.addClass('e-select');
        },
        _getVisibleItemIndex: function (items) {
            var i = 1;
            while (i < items.length) {
                if ($(items[i]).is(":visible"))
                    return i;
                i++;
            }
        },

        removeTab: function (tabIndex) {
            var curTab, href,args;
            if (this._tabObj.model) {
                if (tabIndex > 0) {
				    args={index:tabIndex};
				    this._trigger("beforeTabRemove",args);
                    this._ribbonTabs = this._tabUl.find(".e-link").parent();
                    href = this._ribbonTabs.eq(tabIndex).children().attr("href");
                    curTab = this._ribbonTabs.eq(tabIndex);
                    if (curTab.hasClass("e-contextualtab"))
                        curTab.parent().remove();
                    else if (curTab.hasClass("e-contextualtabset")) {
                        if (curTab.siblings('.e-contextualtabset').length === 0)
                            curTab.parent().remove();
                        else
                            curTab.remove();
                    }
                    else
                        curTab.remove();
                    $(href).remove();
                    if (this._tabObj.model.selectedItemIndex === tabIndex) {
                        this._ribbonTabs = this._tabUl.find(".e-link").parent();
                        curTab = this._ribbonTabs.eq(tabIndex);
                        curTab.addClass("e-active");
                        href = curTab.children().attr("href");
                        $(href).addClass("e-active-content e-activetop").show();
                    }
					args={removedIndex:tabIndex};
				    this._trigger("tabRemove",args);
                }
            }
        },

        addTabGroup: function (tabIndex, tabGroup, grpIndex) {
            var tabContent, conTabIndex = 0, grpLen = 0, tabs, curTab, tabGroupText, contDiv, groupDiv;
            curTab = this._tabUl.find('.e-select,.e-active').eq(tabIndex);
            tabContent = curTab.children('a').attr('href');
            if ($(curTab).hasClass('e-tab')) {
                for (var i = 0; i < this.model.tabs.length; i++) {
                    if (i === tabIndex - 1) {
                        grpLen = this.model.tabs[i].groups.length;
                        if (ej.isNullOrUndefined(grpIndex))
                            grpIndex = grpLen;
                        this.model.tabs[i].groups.splice(grpIndex, 0, tabGroup);
                        tabs = this.model.tabs[i];
                    }
                }
            }
            else {
                conTabIndex = this.model.tabs.length;
                for (var i = 0; i < this.model.contextualTabs.length; i++) {
                    for (var j = 0; j < this.model.contextualTabs[i].tabs.length; j++) {
                        ++conTabIndex;
                        if (conTabIndex === tabIndex) {
                            grpLen = this.model.contextualTabs[i].tabs[j].groups.length;
                            if (ej.isNullOrUndefined(grpIndex))
                                grpIndex = grpLen;
                            this.model.contextualTabs[i].tabs[j].groups.splice(grpIndex, 0, tabGroup);
                            tabs = this.model.contextualTabs[i].tabs[j];
                        }
                    }
                }
            }
            if (!ej.isNullOrUndefined(tabs.groups[grpIndex])) {
                tabGroupText = ej.isNullOrUndefined(tabs.groups[grpIndex].text) ? tabs.groups[grpIndex].text : tabs.groups[grpIndex].text.replace(/\s/g, '');
                contDiv = this._createControls(grpIndex, tabs, tabGroupText);
                groupDiv = this._addControlsToGroup(grpIndex, tabs, tabGroupText, contDiv);
                if ($(tabContent).children().eq(grpIndex).length > 0)
                    $(tabContent).children().eq(grpIndex).before(groupDiv);
                else
                    $(tabContent).append(groupDiv);
            }
        },

        addTabGroupContent: function (tabIndex, grpIndex, subGrpIndex, content, contentIndex) {
            var tabContent, conTabIndex = 0, tabs, curTab, tabGroupText, contDiv, grpContent, subGroup, separator, grpContentDiv, grpHeight;
            curTab = this._tabUl.find('.e-select,.e-active').eq(tabIndex);
            tabContent = curTab.children('a').attr('href');
            if (curTab.hasClass('e-tab'))
                for (var i = 0; i < this.model.tabs.length; i++)
                    if (i === tabIndex - 1)
                        if (!ej.isNullOrUndefined(this.model.tabs[i].groups[grpIndex].content)) {
                            if (ej.isNullOrUndefined(contentIndex))
                                contentIndex = this.model.tabs[i].groups[grpIndex].content[subGrpIndex].groups.length;
                            this.model.tabs[i].groups[grpIndex].content[subGrpIndex].groups.splice(contentIndex, 0, content);
                            tabs = this.model.tabs[i];
                        }
                        else {
                            conTabIndex = this.model.tabs.length;
                            for (var i = 0; i < this.model.contextualTabs.length; i++)
                                for (var j = 0; j < this.model.contextualTabs[i].tabs.length; j++) {
                                    ++conTabIndex;
                                    if (conTabIndex === tabIndex)
                                        if (!ej.isNullOrUndefined(this.model.contextualTabs[i].tabs[j].groups[grpIndex].content)) {
                                            if (ej.isNullOrUndefined(contentIndex))
                                                contentIndex = this.model.contextualTabs[i].tabs[j].groups[grpIndex].content[subGrpIndex].groups.length;
                                            this.model.contextualTabs[i].tabs[j].groups[grpIndex].content[subGrpIndex].groups.splice(contentIndex, 0, content);
                                            tabs = this.model.contextualTabs[i].tabs[j];
                                        }
                                }
                        }
            if (!ej.isNullOrUndefined(tabs.groups[grpIndex])) {
                tabGroupText = ej.isNullOrUndefined(tabs.groups[grpIndex].text) ? tabs.groups[grpIndex].text : tabs.groups[grpIndex].text.replace(/\s/g, '');
                contDiv = this._createControls(grpIndex, tabs, tabGroupText);
                grpContentDiv = ej.buildTag("div");
                grpContent = contDiv.find(".e-innerdiv,.e-innerdivrow").eq(subGrpIndex).find(".e-controlpadding").eq(contentIndex);
                separator = contDiv.find("#" + grpContent.attr("id")).next();
                if (separator.hasClass("e-separatordivrow"))
                    grpContentDiv.append(separator);
                grpContentDiv.prepend(grpContent);
                subGroup = $(tabContent).children().eq(grpIndex).find(".e-innerdiv,.e-innerdivrow").eq(subGrpIndex);
                grpHeight = $(subGroup).parent().height();
                if (subGroup.find(".e-controlpadding").eq(contentIndex).length > 0)
                    subGroup.find(".e-controlpadding").eq(contentIndex).before(grpContentDiv.children());
                else
                    subGroup.append(grpContentDiv.children());
                if ($(subGroup).parent().height() > grpHeight)
                    this.element.find(".e-content").height(this.element.find(".e-content").height() + ($(subGroup).parent().height() - grpHeight));
                contDiv.children().remove();
            }
        },

        addTab: function (tabText, ribbonGroups, index) {
            var rTab = { id: tabText.replace(/\s/g, ''), text: tabText, groups: ribbonGroups }, tabLen = this.model.tabs.length, ribbonModel,newTab,tabcont,args;
            if (ej.isNullOrUndefined(index))
                index = this._tabUl.find(".e-select,.e-active").length;
            if (index > 0) {
                this._conTabsRemove();
                this._tabObj.addItem("#" + this._id + "_" + tabText.replace(/\s/g, ''), tabText, index);
                this._contextualTabs();
                this._tabUl.find(".e-select,.e-active").eq(index).addClass("e-tab");
                this._tabUl.find(".e-select > a,.e-active > a ").eq(index).addClass("e-link");
                this.model.tabs.splice(index - 1, 0, rTab);
                for (i = 0; i < 1; i++) {
                    tabs = this.model.tabs[index - 1];
                    for (j = 0; j < tabs.groups.length; j++) {
                        if (!ej.isNullOrUndefined(tabs.groups[j])) {
                            var tabGroupText = ej.isNullOrUndefined(tabs.groups[j].text) ? tabs.groups[j].text : tabs.groups[j].text.replace(/\s/g, '');
                            var contDiv = this._createControls(j, tabs, tabGroupText);
                            var groupDiv = this._addControlsToGroup(j, tabs, tabGroupText, contDiv);
                            tabcont = this.element.find("#" + this._id + "_" + tabs.id);
                            tabcont.append(groupDiv).addClass("e-parentdiv");
                        }
                    }
                }
                this._setTabContentHeight();
                this.element.find('.e-content').css("box-sizing", "content-box");
				newTab=this._tabUl.find(".e-select,.e-active").eq(index);
				args={tabHeader:newTab,tabContent:tabcont};
				this._trigger("tabAdd",args);
            }
        },

        addContextualTabs: function (contextualTabSet, index) {
            var tabSetLen = this.model.contextualTabs.length, ribbonModel;
            if (ej.isNullOrUndefined(index))
                index = this._tabUl.find(".e-select,.e-active").length;
            if (index > 0) {
                this._conTabsRemove();
                for (var i = 0; i < contextualTabSet.tabs.length; i++) {
                    this._tabObj.addItem("#" + this._id + "_" + contextualTabSet.tabs[i].id, contextualTabSet.tabs[i].text, index);
                    this._tabUl.find(".e-select,.e-active").eq(index).addClass("e-contextualtabset");
                    this._tabUl.find(".e-select > a ,.e-active > a ").eq(index).addClass("e-link");
                    ++index;
                }
                if (this.model.contextualTabs[0].tabs.length > 0)
                    this.model.contextualTabs.splice(index - 1, 0, contextualTabSet);
                else
                    this.model.contextualTabs[0] = contextualTabSet;
                this._contextualTabs();
                for (i = 0; i < contextualTabSet.tabs.length; i++) {
                    tabs = contextualTabSet.tabs[i];
                    for (j = 0; j < tabs.groups.length; j++) {
                        if (!ej.isNullOrUndefined(tabs.groups[j])) {
                            var tabGroupText = ej.isNullOrUndefined(tabs.groups[j].text) ? tabs.groups[j].text : tabs.groups[j].text.replace(/\s/g, '');
                            var contDiv = this._createControls(j, tabs, tabGroupText);
                            var groupDiv = this._addControlsToGroup(j, tabs, tabGroupText, contDiv);
                            tabcont = this.element.find("#" + this._id + "_" + tabs.id);
                            tabcont.append(groupDiv).addClass("e-parentdiv");
                        }
                    }
                }
                this._setTabContentHeight();
                this.element.find('.e-content').css("box-sizing", "content-box");
            }
        },

        setTabText: function (oldText, newText) {
            this._tabText.filter(function (index) { return $(this).text() === oldText; }).text(newText);
        },

        getTabText: function (index) {
            var tabText = this._tabText.eq(index).text();
            return tabText;
        },

        isVisible: function (text) {
            var tab = this._tabText.filter(function (index) { return $(this).text() === text; }).parent();
            return (tab.is(':visible'));
        },

        isEnable: function (text) {
            var isEnable = true, tabIndex = this._tabText.filter(function (index) { return $(this).text() === text; }).parent().index();
            for (var i = 0; i < this.model.disabledItemIndex.length; i++) {
                if (this.model.disabledItemIndex[i] == tabIndex)
                    isEnable = false;
            }
            return isEnable;
        },
        _renderTab: function () {
            var ul = ej.buildTag("ul"), li, aTag, appTab = this.model.applicationTab.type, tabs, contextualTabs, contextualTabSet, i, j;
            if (appTab != null)
                aTag = ej.buildTag("a", "", {}, { href: "#" + this._id + "_" + appTab.replace(/\s/g, '') });
            li = ej.buildTag("li.e-apptab", aTag);
            ul.append(li);
            if (this.model.tabs.length >= 1 && !ej.isNullOrUndefined(this.model.tabs[0].id) && !ej.isNullOrUndefined(this.model.tabs[0].text)) {
                for (i = 0; i < this.model.tabs.length; i++) {
                    tabs = this.model.tabs[i];
                    aTag = ej.buildTag("a", tabs.text, {}, { href: "#" + this._id + "_" + tabs.id });
                    li = ej.buildTag("li", aTag).addClass("e-tab");
                    ul.append(li);
                }
            }
            if (this.model.contextualTabs) {
                for (i = 0; i < this.model.contextualTabs.length; i++) {
                    if (this.model.contextualTabs[i].tabs) {
                        for (j = 0; j < this.model.contextualTabs[i].tabs.length; j++) {
                            contextualTabSet = this.model.contextualTabs[i].tabs[j];
                            aTag = ej.buildTag("a", contextualTabSet.text, {}, { href: "#" + this._id + "_" + contextualTabSet.id });
                            li = ej.buildTag("li", aTag).addClass("e-contextualtabset");
                            ul.append(li);
                        }
                    }
                }
            }
            this.element.append(ul);
            this.element.ejTab({ width: this.model.width,allowKeyboardNavigation:false, disabledItemIndex: this.model.disabledItemIndex, enabledItemIndex: this.model.enabledItemIndex, enableTabScroll: false, itemActive: $.proxy(this._onTabSelect, this), beforeActive: $.proxy(this._onTabClick, this), create: $.proxy(this._create, this), selectedItemIndex: this.model.selectedItemIndex });
        },
        _createControls: function (j, tabs, tabGroupText) {
            var innerDiv, innerDivCustom, innerDivChild, contDiv, model = {};
            var split, button, dropdown, custom, toggle, separatorDiv;
			tabGroupText=tabGroupText.replace(/\&/g, '');
            contDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_content").addClass('e-ribGroupContent');
            if (tabs.groups[j].type === "custom") {
                if (!tabs.groups[j].contentID)
                    innerDivCustom = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + tabs.groups[j].type, tabs.groups[j].customContent).addClass("e-innerdiv");
                else {
                    innerDivCustom = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + tabs.groups[j].type).addClass("e-innerdiv");
                    $("#" + tabs.groups[j].contentID).appendTo(innerDivCustom);
                }
                contDiv.append(innerDivCustom);
            }
            else {
                if (tabs.groups[j].content) {
                    for (var l = 0; l < tabs.groups[j].content.length; l++) {
                        if (tabs.groups[j].content[l] != undefined) {
                            var subGroup = tabs.groups[j].content[l].groups;
                            var defaults = tabs.groups[j].content[l].defaults;
                            if (tabs.groups[j].content.length == 1)
                                centerAlign = true;
                            if (ej.isNullOrUndefined(tabs.groups[j].alignType))
                                tabs.groups[j].alignType = ej.Ribbon.alignType.rows;
                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns)
                                innerDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + (l + 1)).addClass("e-innerdiv");
                            else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                innerDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + (l + 1)).addClass("e-innerdivrow");
                                ++m;
                            }
                            for (var m = 0; m < subGroup.length; m++) {
                                if (!ej.isNullOrUndefined(subGroup[m])) {
                                    if (!ej.isNullOrUndefined(defaults)) {
                                        if (!ej.isNullOrUndefined(defaults.type))
                                            subGroup[m].type = defaults.type;
                                    }
                                    if (ej.isNullOrUndefined(subGroup[m].type))
                                        subGroup[m].type = ej.Ribbon.type.button;
                                    if (ej.isNullOrUndefined(subGroup[m].isBig) && !ej.isNullOrUndefined(defaults))
                                        subGroup[m].isBig = defaults.isBig;
                                    if (ej.isNullOrUndefined(subGroup[m].height) && !ej.isNullOrUndefined(defaults))
                                        subGroup[m].height = defaults.height;
                                    if (ej.isNullOrUndefined(subGroup[m].width) && !ej.isNullOrUndefined(defaults))
                                        subGroup[m].width = defaults.width;
                                    if (subGroup[m].type === ej.Ribbon.type.custom) {
                                        custom = subGroup[m];
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + custom.contentID).addClass("e-controlpadding");
                                        }
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + custom.contentID).addClass("e-innerdivchild e-controlpadding");
                                        }
                                        if (!$.isEmptyObject(custom.customToolTip))
                                            innerDivChild.mouseover({ value: custom, ribbonId: this._id }, $.proxy(this._tooltip, this)).mouseout({ ribbonId: this._id }, $.proxy(this._toolTipOut, this));
                                        else if (!ej.isNullOrUndefined(custom.toolTip))
                                            innerDivChild.attr("title", custom.toolTip);
                                        $("#" + custom.contentID).appendTo(innerDivChild);
                                        if (!ej.isNullOrUndefined(subGroup[m])) {
                                            if (!ej.isNullOrUndefined(subGroup[m].height))
                                                innerDivChild.children().height(subGroup[m].height);
                                            if (!ej.isNullOrUndefined(subGroup[m].width))
                                                innerDivChild.children().width(subGroup[m].width);
                                        }
                                        if (!ej.isNullOrUndefined(custom.isBig)) {
                                            (custom.isBig) ? innerDivChild.children().addClass("e-big") : innerDivChild.children().addClass("e-small");
                                        }
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + custom.toolTip).addClass("e-separatordivrow");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    else if (subGroup[m].type === ej.Ribbon.type.splitButton) {
                                        split = subGroup[m];
                                        this._splitButtonControl = ej.buildTag("button#" + this._id + "_" + split.id, split.text, {}, { type: "button" }).addClass(split.cssClass);
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + split.id, this._splitButtonControl).addClass("e-controlpadding");
                                        }
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + split.id, this._splitButtonControl).addClass("e-innerdivchild e-controlpadding");
                                        }
                                        if (!$.isEmptyObject(split.customToolTip))
                                            innerDivChild.mouseover({ value: split, ribbonId: this._id }, $.proxy(this._tooltip, this)).mouseout({ ribbonId: this._id }, $.proxy(this._toolTipOut, this));
                                        else if (!ej.isNullOrUndefined(split.toolTip))
                                            innerDivChild.attr("title", split.toolTip);
                                        model = {}
                                        if (split.splitButtonSettings)
                                            model = split.splitButtonSettings;
                                        model.height = ej.isNullOrUndefined(model.height) ? subGroup[m].height : model.height;
                                        model.width = ej.isNullOrUndefined(model.width) ? subGroup[m].width : model.width;
										if(model.cssClass)
										 model.cssClass=model.cssClass.concat(" e-rbn-splitbtn");
										else
										 model.cssClass="e-rbn-splitbtn";
                                        this._splitButtonControl.ejSplitButton(model);
                                        if (!ej.isNullOrUndefined(split.isBig)) {
                                            (split.isBig) ? innerDivChild.children().removeAttr("style").addClass("e-big") : innerDivChild.children().removeAttr("style").addClass("e-small");
                                        }
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + split.id).addClass("e-separatordivrow");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    else if (subGroup[m].type === ej.Ribbon.type.toggleButton) {
                                        toggle = subGroup[m];
                                        this._toggleButtonControl = ej.buildTag("input#" + this._id + "_" + toggle.id, "", "", { type: "checkbox" }).addClass(toggle.cssClass);
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + toggle.id, this._toggleButtonControl).addClass("e-controlpadding");
                                        }
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + toggle.id, this._toggleButtonControl).addClass("e-innerdivchild e-controlpadding");
                                        }
                                        if (!$.isEmptyObject(toggle.customToolTip))
                                            innerDivChild.mouseover({ value: toggle, ribbonId: this._id }, $.proxy(this._tooltip, this)).mouseout({ ribbonId: this._id }, $.proxy(this._toolTipOut, this));
                                        else if (!ej.isNullOrUndefined(toggle.toolTip))
                                            innerDivChild.attr("title", toggle.toolTip);
                                        model = {}
                                        if (toggle.toggleButtonSettings)
                                            model = toggle.toggleButtonSettings;
                                        model.height = ej.isNullOrUndefined(model.height) ? subGroup[m].height : model.height;
                                        model.width = ej.isNullOrUndefined(model.width) ? subGroup[m].width : model.width;
                                        this._toggleButtonControl.ejToggleButton(model).addClass('e-ribbonbtn');
                                        if (!ej.isNullOrUndefined(toggle.isBig)) {
                                            (toggle.isBig) ? innerDivChild.find('.e-togglebutton.e-tbtn.e-btn').addClass("e-big") : innerDivChild.find('.e-togglebutton.e-tbtn.e-btn').addClass("e-small");
                                        }
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + toggle.id).addClass("e-separatordivrow");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    else if (subGroup[m].type === ej.Ribbon.type.button) {
                                        button = subGroup[m];
                                        if (button.buttonSettings) {
                                            if (button.buttonSettings.type)
                                                this._buttonControl = ej.buildTag("button#" + this._id + "_" + button.id, button.text, {}, { type: button.buttonSettings.type }).addClass(button.cssClass);
                                            else
                                                this._buttonControl = ej.buildTag("button#" + this._id + "_" + button.id, button.text).addClass(button.cssClass);
                                        }
                                        else
                                            this._buttonControl = ej.buildTag("button#" + this._id + "_" + button.id, button.text).addClass(button.cssClass);
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + button.id, this._buttonControl).addClass("e-controlpadding");
                                        }
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + button.id, this._buttonControl).addClass("e-innerdivchild e-controlpadding");
                                        }
                                        if (!$.isEmptyObject(button.customToolTip))
                                            innerDivChild.mouseover({ value: button, ribbonId: this._id }, $.proxy(this._tooltip, this)).mouseout({ ribbonId: this._id }, $.proxy(this._toolTipOut, this));
                                        else if (!ej.isNullOrUndefined(button.toolTip))
                                            innerDivChild.attr("title", button.toolTip);
                                        model = {}
                                        if (button.buttonSettings)
                                            model = button.buttonSettings;
                                        model.height = ej.isNullOrUndefined(model.height) ? subGroup[m].height : model.height;
                                        model.width = ej.isNullOrUndefined(model.width) ? subGroup[m].width : model.width;
                                        this._buttonControl.ejButton(model).addClass('e-ribbonbtn');
                                        if (!ej.isNullOrUndefined(button.isBig)) {
                                            (button.isBig) ? innerDivChild.children().addClass("e-big") : innerDivChild.children().addClass("e-small");
                                        }
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + button.id).addClass("e-separatordivrow");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    else if (subGroup[m].type === ej.Ribbon.type.dropDownList) {
                                        dropdown = subGroup[m];
                                        this._dropdownControl = ej.buildTag("input#" + this._id + "_" + dropdown.id, { type: "text" }).addClass(dropdown.cssClass);
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns && dropdown.dropdownSettings) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + dropdown.id, this._dropdownControl).append($("#" + dropdown.dropdownSettings.targetID)).addClass("e-controlpadding");
                                        }
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows && dropdown.dropdownSettings) {
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + dropdown.id, this._dropdownControl).append($("#" + dropdown.dropdownSettings.targetID)).addClass("e-innerdivchild e-controlpadding");
                                        }
                                        if (!$.isEmptyObject(dropdown.customToolTip))
                                            innerDivChild.mouseover({ value: dropdown, ribbonId: this._id }, $.proxy(this._tooltip, this)).mouseout({ ribbonId: this._id }, $.proxy(this._toolTipOut, this));
                                        else if (!ej.isNullOrUndefined(dropdown.toolTip))
                                            innerDivChild.attr("title", dropdown.toolTip);
                                        model = {};
                                        if (dropdown.dropdownSettings)
                                            model = dropdown.dropdownSettings;
                                        model.height = ej.isNullOrUndefined(model.height) ? subGroup[m].height : model.height;
                                        model.width = ej.isNullOrUndefined(model.width) ? subGroup[m].width : model.width;
										if(model.cssClass)
										 model.cssClass=model.cssClass.concat(" e-rbn-ddl");
										else
										 model.cssClass="e-rbn-ddl";
                                        this._dropdownControl.ejDropDownList(model);
                                        innerDiv.append(innerDivChild);
                                        if (subGroup[m].enableSeparator) {
                                            if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows) {
                                                separatorDiv = ej.buildTag("div#" + this._id + "_" + "separator" + "_" + dropdown.id).addClass("e-separatordivrow");
                                                innerDiv.append(separatorDiv);
                                            }
                                        }
                                    }
                                    else if (subGroup[m].type === ej.Ribbon.type.gallery) {
                                        var galleryContentDiv, galleryPagesDiv, prevPage, nextPage, expandGallery, visibleRowCnt, galleryRows;
                                        gallery = subGroup[m];
                                        this._galleryControl = ej.buildTag("div#" + this._id + "_" + gallery.id + "_gallerycontrol").addClass("e-ribbongallerycontrol");
                                        this._gallery = ej.buildTag("div#" + this._id + "_" + gallery.id).addClass("e-ribbongallery");
                                        var galleryContentDiv = ej.buildTag("div#" + this._id + "_" + gallery.id + "_ContentDiv").addClass("e-gallerycontent").height(65).width(gallery.columns * gallery.itemWidth).click({ galleryId: this._gallery.attr("id") }, $.proxy(this._onGalContentClick, this));

                                        galleryRows = Math.floor(gallery.galleryItems.length / gallery.columns);

                                        if ((gallery.galleryItems.length % gallery.columns) > 0)
                                            ++galleryRows;

                                        for (var m = 1; m <= galleryRows; m++) {
                                            var galleryRow = ej.buildTag("div#" + this._id + "_" + gallery.id + "_galleryrow_" + m).addClass('e-galleryrow').click({ model: gallery }, $.proxy(this._onGalleryItemClick, this));
                                            if (m === 1)
                                                galleryRow.addClass('e-gryfirstrow');
                                            else if (m === galleryRows)
                                                galleryRow.addClass('e-grylastrow');

                                            for (var n = 1; n <= gallery.columns; n++) {
                                                var contentLen = galleryContentDiv.find('.e-galleryrow').children().length + galleryRow.children().length, btnCtrl;
                                                if (!ej.isNullOrUndefined(gallery.galleryItems[contentLen])) {
                                                    btnCtrl = ej.buildTag("button#" + gallery.id + "_galleryItem_" + (contentLen + 1), gallery.galleryItems[contentLen].text, {}, { type: "button" }).addClass('e-gallerybtn');
                                                    if (!$.isEmptyObject(gallery.galleryItems[contentLen].customToolTip))
                                                        btnCtrl.mouseover({ value: gallery.galleryItems[contentLen], ribbonId: this._id }, $.proxy(this._tooltip, this)).mouseout({ ribbonId: this._id }, $.proxy(this._toolTipOut, this));
                                                    else if (!ej.isNullOrUndefined(gallery.galleryItems[contentLen].toolTip))
                                                        btnCtrl.attr("title", gallery.galleryItems[contentLen].toolTip);

                                                    model = {}
                                                    if (gallery.galleryItems[contentLen].buttonSettings)
                                                        model = gallery.galleryItems[contentLen].buttonSettings;
                                                    if (gallery.itemHeight <= 65)
                                                        model.height = gallery.itemHeight;
                                                    else
                                                        model.height = 65;
                                                    model.width = gallery.itemWidth;
                                                    btnCtrl.ejButton(model);
                                                    btnCtrl.appendTo(galleryRow);
                                                }
                                            }
                                            if (gallery.itemHeight > 65)
                                                visibleRowCnt = 1;
                                            else
                                                visibleRowCnt = (galleryContentDiv.height() / gallery.itemHeight);
                                            if (m > visibleRowCnt)
                                                galleryRow.hide();
                                            galleryRow.appendTo(galleryContentDiv);
                                        }
                                        this._gallery.append(galleryContentDiv);

                                        var galleryPagesDiv = ej.buildTag("div#" + this._id + "_" + gallery.id + "_PagesDiv").addClass("e-gallerymovediv");
                                        var prevPage = ej.buildTag("div#" + this._id + "_" + gallery.id + "_prevPage", "<span class='e-icon e-galleryup e-gallerymoveicon'></span>").click({ galleryId: this._gallery.attr("id"), rowCnt: visibleRowCnt }, $.proxy(this._onGalMoveUpClick, this)).addClass('e-moveupdiv e-disablegrymovebtn');
                                        var nextPage = ej.buildTag("div#" + this._id + "_" + gallery.id + "_nextPage", "<span class='e-icon e-gallerydown e-gallerymoveicon'></span>").click({ galleryId: this._gallery.attr("id") }, $.proxy(this._onGalMoveDownClick, this)).addClass('e-movedowndiv');
                                        var expandGallery = ej.buildTag("div#" + this._id + "_" + gallery.id + "_expandGallery", "<span class='e-icon e-galleryexpand e-gallerymoveicon'></span>").click({ galleryId: this._gallery.attr("id"), columns: gallery.columns, expandedColumns: gallery.expandedColumns, itemWidth: gallery.itemWidth }, $.proxy(this._onExpandGalleryClick, this)).addClass('e-expgallerydiv');

                                        galleryPagesDiv.append(prevPage);
                                        galleryPagesDiv.append(nextPage);
                                        galleryPagesDiv.append(expandGallery);
                                        this._gallery.append(galleryPagesDiv);
                                        var galleryExpandContent = ej.buildTag("div#" + this._id + "_" + gallery.id + "_ExpandContent").addClass("e-gallexpandcontent").hide();
                                        var scrollContent = ej.buildTag("div#" + this._id + "_" + gallery.id + "_scrollContent", "<div></div>").addClass("e-gallscrollcontent");
                                        var expanderContent = ej.buildTag("div#" + this._id + "_" + gallery.id + "_ExpanderContent").addClass("e-expandercontent").click({ galleryId: this._gallery.attr("id") }, $.proxy(this._onExpandContentClick, this));
                                        scrollContent.children().append(expanderContent);
                                        galleryExpandContent.append(scrollContent);
                                        if (!ej.isNullOrUndefined(gallery.customGalleryItems)) {
                                            var expanderExtraContent = ej.buildTag("div#" + this._id + "_" + gallery.id + "_ExtraContent").addClass("e-extracontent").click({ model: gallery }, $.proxy(this._onGalleryItemClick, this));
                                            for (var l = 0; l < gallery.customGalleryItems.length; l++) {
                                                var btnCtrlExtra;
                                                if (ej.isNullOrUndefined(gallery.customGalleryItems[l].customItemType))
                                                    gallery.customGalleryItems[l].customItemType = ej.Ribbon.customItemType.button;
                                                if (gallery.customGalleryItems[l].customItemType === ej.Ribbon.customItemType.button) {
                                                    btnCtrlExtra = ej.buildTag("button#" + gallery.id + "_" + "_extraItem_" + l, gallery.customGalleryItems[l].text, {}, { type: "button" }).addClass('e-galleryextrabtn');
                                                    if (!$.isEmptyObject(gallery.customGalleryItems[l].customToolTip))
                                                        btnCtrlExtra.mouseover({ value: gallery.customGalleryItems[l], ribbonId: this._id }, $.proxy(this._tooltip, this))
                                                        .mouseout({ ribbonId: this._id }, $.proxy(this._toolTipOut, this));
                                                    else if (!ej.isNullOrUndefined(gallery.customGalleryItems[l].toolTip))
                                                        btnCtrlExtra.attr("title", gallery.customGalleryItems[l].toolTip);
                                                    model = {}
                                                    if (gallery.customGalleryItems[l]) {
                                                        if (gallery.customGalleryItems[l].buttonSettings)
                                                            model = gallery.customGalleryItems[l].buttonSettings;
                                                        btnCtrlExtra.ejButton(model);
                                                    }
                                                    btnCtrlExtra.appendTo(expanderExtraContent);
                                                }
                                                else {
                                                    model = {}
                                                    if (gallery.customGalleryItems[l].menuSettings)
                                                        model = gallery.customGalleryItems[l].menuSettings;
                                                    model.menuType = ej.MenuType.NormalMenu;
                                                    model.orientation = ej.Orientation.Vertical;
                                                    $("#" + gallery.customGalleryItems[l].menuId).ejMenu(model).addClass('e-gallerymenu');
                                                    $("#" + gallery.customGalleryItems[l].menuId).appendTo(expanderExtraContent).css("width", "100%");
                                                }
                                            }
                                            galleryExpandContent.append(expanderExtraContent);
                                        }
                                        this._galleryControl.append(this._gallery);
                                        this._galleryControl.append(galleryExpandContent);
                                        if (tabs.groups[j].alignType === ej.Ribbon.alignType.columns)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + gallery.id, this._galleryControl).addClass("e-controlpadding");
                                        else if (tabs.groups[j].alignType === ej.Ribbon.alignType.rows)
                                            innerDivChild = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText + "_" + gallery.id, this._galleryControl).addClass("e-innerdivchild e-controlpadding");
                                        innerDiv.append(innerDivChild);
                                    }
                                    contDiv.append(innerDiv);
                                }
                            }
                        }
                    }
                }
            }
            return contDiv;
        },
        _groupingControls: function () {
            var tabLen = this.model.tabs.length, contextualTab, tabs, groupDiv, contDiv, tabcont;
            var i, j, tabGroupText, ribbonModel = $.extend(true, {}, this.model);
            if (this.model.contextualTabs) {
                for (var k = 0; k < this.model.contextualTabs.length; k++) {
                    contextualTab = this.model.contextualTabs[k];
                    if (contextualTab.tabs) {
                        for (var l = 0; l < contextualTab.tabs.length; l++) {
                            if (contextualTab.tabs[l].groups.length > 0) {
                                this.model.tabs[tabLen] = contextualTab.tabs[l];
                                ++tabLen;
                            }
                        }
                    }
                }
            }
            for (i = 0; i < this.model.tabs.length; i++) {
                tabs = this.model.tabs[i];
                if (!ej.isNullOrUndefined(tabs.groups)) {
                    for (j = 0; j < tabs.groups.length; j++) {
                        if (!ej.isNullOrUndefined(tabs.groups[j])) {
                            tabGroupText = ej.isNullOrUndefined(tabs.groups[j].text) ? tabs.groups[j].text : tabs.groups[j].text.replace(/\s/g, '');
                            contDiv = this._createControls(j, tabs, tabGroupText);
                            groupDiv = this._addControlsToGroup(j, tabs, tabGroupText, contDiv);
                            tabcont = this.element.find("#" + this._id + "_" + tabs.id);
                            tabcont.append(groupDiv).addClass("e-parentdiv");
                        }
                    }
                }
                else
                    this.element.find('.e-content').eq(i + 1).addClass('e-empty-content');
            }
            this._setTabContentHeight();
            this.model = ribbonModel;
        },
        _addControlsToGroup: function (j, tabs, tabGroupText, contDiv) {
            var contentbottom = ej.buildTag("div.e-contentbottom"), centerAlign = false, groupDiv, groupexpander;
            var captionArea = ej.buildTag("div.e-captionarea", tabs.groups[j].text);
            contentbottom.append(captionArea);
            if (tabs.groups[j].enableGroupExpander) {
                groupexpander = ej.buildTag("div#" + this._id + "_" + tabs.groups[j].text + "_e-groupexpander.e-icon e-expander e-groupexpander");
                contentbottom.append(groupexpander);
            }
            groupDiv = ej.buildTag("div#" + this._id + "_" + tabs.id + "_" + tabGroupText).addClass("e-groupdiv").click($.proxy(this._onGroupClick, this));
            if (!ej.isNullOrUndefined(tabs.groups[j].content))
                if (tabs.groups[j].content.length == 1)
                    centerAlign = true;
            tabs.groups[j].width && groupDiv.width(tabs.groups[j].width);
            groupDiv.append(contDiv);
            if (centerAlign) {
                groupDiv.addClass("e-centeralign");
                centerAlign == false;
            }
            groupDiv.append(contentbottom);
            return groupDiv;
        },
        _tabExpandCollapse: function (e) {
            var target, args, action;
            target = e.target.tagName == "div" ? e.target : e.target.parentNode;
            action = target.className == "e-expanded" ? "collapse" : "expand";
            args = { target: $(target), type: e.type, action: action, cancel: false };
            this._trigger("toggleButtonClick", args);
            if (args.cancel)
                return;
			if(this.element.find('.e-rarrowup-2x').length >0)
			 this.collapse();
			else
			 this._removeRibbonPin();
        },
		_ribbonWindowResize: function () {
          if (this.model.allowResizing) {
			if(this.element.find('.e-ribbonpin').length>0 && this.element.find('.e-active-content').is(':visible')){
			 this.collapse();
			 }
			else
			  this._ribbonResize();
		  }
		  if(this.element.find('.e-ribbonpin').length>0)
		   this._tabContents.width(this.element.width());
		},
        _ribbonResize: function () {
                var tabContent, resize, tabIndex, tabLeft, tabWidth, tabHeadLen, curHead, curTextLen, grpLen, contentWidth, grpsWidth, resizeBtnLen = 0, resizeBtn, m = 0, tabArray = [], resizeHead, toolTip, tabUl, curGroup;
                var ribTabContents, ribActiveCon = this.element.find('.e-active-content'), ribTabs, ribTabsVisible, ribResizeDiv, resizeTab,ribbonPin=this.element.find('.e-ribbonpin');
                ribTabsVisible = this._tabUl.find('li.e-select:visible,li.e-active:visible'), ribResizeDiv = this.element.find('.e-resizediv'),expandDiv=this.element.find('.e-expandcollapse');
                    tabUl = this.element.find('.e-header');
					if(ribbonPin.length>0){
					 this._removeRibbonPin();
					 expandDiv.children().addClass('e-ribbonpin');
					 }
                    if (tabUl.is(":hidden") && ribActiveCon.is(":hidden")) {
                        if (this._resizeWidth <= this.element.width() + this.element.offset().left) {
                            tabUl.show();
							if(this.element.find('.e-expandcollapse').children().hasClass('e-expanded') || tabUl.find('.e-active').length > 0){
							 if(ribbonPin.length<=0){
                              ribActiveCon.show();
							  expandDiv.children().addClass('e-expanded').removeClass('e-collapsed');
						   }
						   else if(ribbonPin.length>0)
						    this.collapse();
                        }
                      }
					}
                    ribTabs = tabUl.find('li.e-select,li.e-active');
                    tabHeadLen = ribTabs.length;
                    if (tabUl.is(":visible")) {
                        for (var t = tabHeadLen - 1; t > 0; t--) {
                            curHead = ribTabs.eq(t).children();
                            curTextLen = curHead.text().length - 1;
                            for (var s = curTextLen; s > 1; s--) {
                                if (ribTabsVisible.length > 0) {
                                    tabLeft = ribTabsVisible.last().offset().left;
                                    tabWidth = ribTabsVisible.last().width();
                                    if (((this.element.width() + this.element.offset().left) - 30) - (tabLeft + tabWidth) <= 0) {
                                        if (!curHead.hasClass("e-resizeHead"))
                                            curHead.text(curHead.text().slice(0, -1)).addClass("e-resizeHead");
                                        else
                                            curHead.text(curHead.text().slice(0, -1));
                                    }
                                }
                            }
                        }
                        resizeTab = tabUl.find('.e-resizeHead');
                        if (tabHeadLen - 1 === resizeTab.length && resizeTab.eq(0).text().length <= 2) {
                            if (ribTabsVisible.length > 0) {
                                tabLeft = ribTabsVisible.last().offset().left;
                                tabWidth = ribTabsVisible.last().width();
                                if (((this.element.width() + this.element.offset().left) - 30) - (tabLeft + tabWidth) <= 0) {
                                    this._resizeWidth = this.element.width() + this.element.offset().left;
                                    tabUl.hide();
                                    ribActiveCon.hide();
									expandDiv.children().addClass('e-collapsed').removeClass('e-expanded');
                                }
                            }
                        }
                    }
                    tabIndex = this._tabObj.model.selectedItemIndex;
                    ribTabContents = this.element.find('.e-content');
                    tabContent = ribTabContents.eq(tabIndex).attr('id');
                    grpLen = ribTabContents.eq(tabIndex).children('.e-groupdiv').length;
                    resize = ribTabContents.eq(tabIndex).children('.e-groupdiv')[grpLen - 1];
                    curGroup = ribTabContents.eq(tabIndex).children().eq(grpLen - 1).find('.e-resizebtn');
					if(!ej.isNullOrUndefined(resize)){
                    for (var j = grpLen - 1; j >= 0; j--) {
                        curGroup = ribTabContents.eq(tabIndex).children().eq(j).find('.e-resizebtn');
                        if ((this.element.width() + this.element.offset().left-25) - (resize.offsetWidth + $(resize).offset().left) <= 0) {
                            if (curGroup.length === 0)
                                this._createResizeBtn(tabIndex, j);
                        }
                    }
                    if ((this.element.width() + this.element.offset().left-25) - (resize.offsetWidth + $(resize).offset().left) <= 0) {
                        if (grpLen === ribTabContents.eq(tabIndex).find('.e-resizebtn').length) {
                            this._resizeWidth = this.element.width() + this.element.offset().left;
                            tabUl.hide();
                            ribActiveCon.hide();
							expandDiv.children().addClass('e-collapsed').removeClass('e-expanded');
                        }
                      }
					}
                    if (ribResizeDiv.is(":visible")) {
                        var resizeCon = ribResizeDiv.children().eq(0).attr("id");
                        resizeCon = "#" + resizeCon.replace("_content", "");
                        ribResizeDiv.children().hide().appendTo(resizeCon);
                        ribResizeDiv.hide();
                    }
                    for (var l = 0; l < this.model.tabs.length; l++) {
                        var tabText = this.model.tabs[l].text;
                        tabArray.push(tabText);
                    }

                    if (this.model.contextualTabs) {
                        for (var i = 0; i < this.model.contextualTabs.length; i++) {
                            if (this.model.contextualTabs[i].tabs) {
                                for (var j = 0; j < this.model.contextualTabs[i].tabs.length; j++) {
                                    tabText = this.model.contextualTabs[i].tabs[j].text;
                                    tabArray.push(tabText);
                                }
                            }
                        }
                    }

                    resizeHeadLen = tabUl.find(".e-resizeHead").length;
                    if (tabUl.is(":visible")) {
                        for (var t = 0; t < resizeHeadLen; t++) {
                            curHead = tabUl.find(".e-resizeHead").eq(0);
                            if (curHead.hasClass("e-resizeHead")) {
                                var aTagLen = ribTabs.find(".e-link").not('.e-resizeHead').length - 1;
                                var textLen = curHead.text().length;
                                for (var s = textLen; s <= tabArray[aTagLen].length; s++) {
                                    if (ribTabsVisible.length > 0) {
                                        tabLeft = ribTabsVisible.last().offset().left;
                                        tabWidth = ribTabsVisible.last().width() + 15;
                                        if (((this.element.width() + this.element.offset().left) - 30) >= (tabLeft + tabWidth)) {
                                            if (curHead.text().length === tabArray[aTagLen].length)
                                                curHead.removeClass("e-resizeHead");
                                            else
                                                curHead.text(tabArray[aTagLen].slice(0, curHead.text().length + 1));
                                        }
                                    }
                                }
                            }
                        }
                    }
                    resizeBtnLen = ribTabContents.eq(tabIndex).find('.e-resizebtn').length;
                    for (var i = 0; i < resizeBtnLen; i++) {
                        resizeBtn = ribTabContents.eq(tabIndex).find('.e-resizebtn').eq(0);
                        if ($(resizeBtn).is(':visible')) {
                            grpsWidth = this.element.width() + this.element.offset().left-25;
                            contentWidth = resizeBtn.offset().left + resizeBtn.parent().siblings().not('.e-contentbottom').width();
                            if (resizeBtnLen > 1) {
                                for (var k = 1; k < resizeBtnLen; k++)
                                    contentWidth = contentWidth + ribTabContents.eq(tabIndex).find('.e-resizebtn').eq(k).parents('.e-groupdiv').width() + 7;
                            }
                            if (grpsWidth > contentWidth) {
                                var rez = resizeBtn.parent().siblings();
                                resizeBtn.parent().remove();
                                rez.show();
                                rez.not('.e-contentbottom').css("width", "");
                            }
                        }
                    }
                    toolTip = this.element.find('.e-tooltipdiv');
                    toolTip.hide();
                    toolTip.find('.e-tooltiptitle').children().remove();
                    toolTip.find('.e-tooltipdesc').children().remove();
					if(this.element.find('.e-ribbonpin').length > 0 && this.element.find('.e-active-content').is(':visible')){
					 this._addRibbonPin();
		             this._tabContents.width(this.element.width());
		            }
					this.element.find('.e-resizebtnselect').removeClass('e-resizebtnselect e-active');
        },
        _createResizeBtn: function (i, j) {
            var groupText, grpWidth, ribTabContents = this.element.find('.e-content'), curGroup, grpContents, grpHeight;
            curGroup = ribTabContents.eq(i).children('.e-groupdiv').eq(j);
            grpHeight = curGroup.height();
            groupText = curGroup.find('.e-contentbottom').text();
            grpContents = curGroup.children().not('.e-contentbottom');
            grpWidth = grpContents.width();
            curGroup.children().hide();
            grpContents.css("width", grpWidth);
            var rBtnDiv = ej.buildTag("div#" + this._id + "_" + "resizeDiv" + "_" + groupText);
            var rBtn = ej.buildTag("button#" + this._id + "_" + "resizebtn" + "_" + groupText, groupText, {}, { type: "button" }).addClass("e-resizebtn")
            var data = { target: ribTabContents.eq(i).children('.e-groupdiv').eq(j).children() };
            rBtn.ejButton({
                click: $.proxy(this._resizeBtnClick, data),
                contentType: ej.ContentType.TextAndImage,
                imagePosition: ej.ImagePosition.ImageBottom,
                prefixIcon: "e-ribbonresize"
            });
            curGroup.height(grpHeight);
            rBtn.find('div').prepend("<span class='e-icon e-" + groupText + "'></span>").height(grpHeight-5);
            rBtn.appendTo(rBtnDiv);
            rBtnDiv.appendTo(ribTabContents.eq(i).children('.e-groupdiv').eq(j));
        },
        _resizeBtnClick: function (args) {

        },
        _ribbonDocClick: function (e) {
            var resizeDiv, resizeContent, expClick, gallery, gryColumns, contentHeight, toolTip, control, disabled;
            resizeDiv = $(e.target).parents('.e-resizediv');
            if (resizeDiv.length <= 0) {
                if ($(e.target).hasClass('e-resizebtn'))
                    resizeDiv = $(e.target);
                else if (!ej.isNullOrUndefined($(e.target).parents('.e-resizebtn')))
                    resizeDiv = $(e.target).parents('.e-resizebtn');
            }
            if (this.element.find('.e-resizediv').is(':visible') && this.element.find('.e-resizediv .e-gallexpandcontent').is(':visible')) {
                if ($(e.target).hasClass('e-gallerymoveicon') || $(e.target).hasClass('e-gallerymovediv') || $(e.target).hasClass('e-expgallerydiv') || $(e.target).hasClass('e-scrollbar') || $(e.target).parents().hasClass('e-scrollbar'))
                    expClick = e.target;
                else if ($(e.target).parents().hasClass('e-gallerymenu')) {
                    if ($(e.target).hasClass('e-haschild') || $(e.target).hasClass('aschild') || $(e.target).hasClass('e-arrows'))
                        expClick = e.target;
                }
                if (ej.isNullOrUndefined(expClick)) {
                    this._ribbonGalleryShow();
                    this.element.find('.e-resizediv').show().css('visibility', '');
                    this.element.find('.e-resizediv .e-gallexpandcontent').hide().css('visibility', '');
                }
            }
            else if (resizeDiv.length <= 0)
                this._resizeDivHide();
            else {
                if ($(resizeDiv).hasClass('e-resizebtn')) {
                    if (this.element.find('.e-resizediv').is(":visible")) {
                        if (resizeDiv.parent().siblings('.e-contentbottom').length <= 0)
                            this._resizeDivHide();
                        else {
                            this._resizeDivHide();
                            this._resizeDivShow(resizeDiv);
                        }
                    }
                    else
                        this._resizeDivShow(resizeDiv);
                }
            }
            if ($(e.target).hasClass('e-gallerymoveicon') || $(e.target).hasClass('e-gallerymovediv') || $(e.target).hasClass('e-expgallerydiv') || $(e.target).hasClass('e-scrollbar') || $(e.target).parents().hasClass('e-scrollbar'))
                expClick = e.target;
            else if ($(e.target).parents().hasClass('e-gallerymenu')) {
                if ($(e.target).hasClass('e-haschild') || $(e.target).hasClass('aschild') || $(e.target).hasClass('e-arrows'))
                    expClick = e.target;
            }
            if (ej.isNullOrUndefined(expClick))
                this._ribbonGalleryShow();
            toolTip = this.element.find('.e-tooltipdiv');
            disabled = $(e.target).find('.e-disable');
            if (toolTip.is(':visible') && $(disabled).length <= 0 && (!$(e.target).hasClass('e-disable')))
                toolTip.hide();
            if ($(e.target).hasClass('e-backstagechild') || $(e.target).hasClass('e-ribbonbackstagepage') || $(e.target).hasClass('e-apptab') || $(e.target).parents('.e-ribbonbackstagepage').length > 0 || $(e.target).parents('.e-apptab').length > 0)
                expClick = e.target;
            if (ej.isNullOrUndefined(expClick))
                this.element.find(".e-ribbonbackstagepage").hide();
		    if(!$(e.target).hasClass('e-disable-item') && $(e.target).parents('.e-disable-item').length<=0)
             this.element.find(".e-controlclicked").removeClass('e-controlclicked');
            control = $(e.target).parents('.e-controlpadding');
            if (control.length > 0 && $(disabled).length <= 0 && (!$(e.target).hasClass('e-disable')))
                if ($(control).find('.e-ribbongallerycontrol').length <= 0)
                    $(control).addClass('e-controlclicked');
			if(this.element.find('.e-ribbonpin').is(':visible') && $(e.target).parents('.e-ribbon').length <=0 && !$(e.target).hasClass('e-scrollbar') && !$(e.target).parents().hasClass('e-scrollbar'))
               this.collapse();			
			if(this.element.find('.e-resizediv').is(':hidden'))
			 this.element.find('.e-resizebtnselect').removeClass('e-resizebtnselect e-active');
			this._tabUl.find('.e-link.e-active').removeClass('e-active');
        },
        _ribbonGalleryShow: function () {
            var startIndex = 0;
            if (this.element.find('.e-gallexpandcontent:visible').length > 0) {
                var visibleRows = 0, n = 0;
                gallery = this.element.find('.e-gallexpandcontent:visible').parent().find('.e-ribbongallery');
                for (var i = 1; i <= gallery.find('.e-galleryrow').length; i++) {
                    if (gallery.find('.e-galleryrow').eq(i - 1).css('display') === "block") {
                        if (visibleRows === 0)
                            startIndex = i - 1;
                        ++visibleRows;
                    }
                }
                gallery.find('.e-galleryrow').hide();
                gallery.parent().find('.e-gallexpandcontent').hide();
                gallery.show();
                gryColumns = gallery.find('.e-galleryrow').eq(0).children().length;
                gallery.find('.e-galleryrow').eq(0).children().remove();
                for (var k = 0; k < gallery.find('.e-galleryrow').length; k++) {
                    for (var l = 0; l < gryColumns; l++) {
                        var activeBtn = gallery.parent().find('.e-galleryexpanderrow').children().eq(0);
                        if ($(activeBtn).hasClass("e-galleryselect")) {
                            gallery.find('.e-galleryrow').eq(k).append(gallery.parent().find('.e-galleryexpanderrow').children().eq(0)).show();
                            --visibleRows;
                            for (var i = 1; i <= visibleRows; i++) {
                                if (gallery.find('.e-galleryrow').eq(k + i).length > 0)
                                    gallery.find('.e-galleryrow').eq(k + i).show();
                                else {
                                    ++n;
                                    gallery.find('.e-galleryrow').eq(k - n).show();
                                }
                            }
                            if (!gallery.find('.e-galleryrow').last().is(':visible'))
                                gallery.parent().find('.e-movedowndiv').removeClass('e-disablegrymovebtn');
                            else
                                gallery.parent().find('.e-movedowndiv').addClass('e-disablegrymovebtn');
                            if (gallery.find('.e-galleryrow').first().is(':visible'))
                                gallery.parent().find('.e-moveupdiv').addClass('e-disablegrymovebtn');
                            else
                                gallery.parent().find('.e-moveupdiv').removeClass('e-disablegrymovebtn');
                        }
                        else
                            gallery.find('.e-galleryrow').eq(k).append(gallery.parent().find('.e-galleryexpanderrow').children().eq(0));
                    }
                }
                if (gallery.find('.e-galleryselect').length <= 0) {
                    for (var i = 0; i < visibleRows; i++) {
                        gallery.find('.e-galleryrow').eq(startIndex).show();
                        ++startIndex;
                    }
                }
                gallery.parent().find('.e-galleryexpanderrow').remove();
            }
        },
        _resizeDivHide: function () {
            var resizeArea = this.element.find('.e-resizediv'), resizeCon;
            if ($(resizeArea).is(":visible")) {
                resizeCon = $(resizeArea).children().eq(0).attr("id");
                if (!ej.isNullOrUndefined(resizeCon)) {
                    resizeCon = "#" + resizeCon.slice(0, -8);
                    $(resizeArea).children().hide().appendTo(resizeCon);
                    $(resizeArea).hide();
                }
				if(this.element.find('.e-ribbonpin').length>0)
				  $(resizeArea).css({'top':''});
            }
        },
        _resizeDivShow: function (resizeDiv) {
            var contentWidth, contentLeft, curGroup, rDivPadding, resizeArea = this.element.find('.e-resizediv');
            curGroup = this.element.find("#" + resizeDiv[0].id).parent().siblings();
            contentWidth = curGroup.not('.e-contentbottom').width() + curGroup.parent().offset().left;
            this.element.css({ "position": "relative" });
			$(resizeArea).css({'left':''});
            if (contentWidth > this.element.width() + this.element.offset().left) {
                contentLeft = curGroup.parent().position().left - (contentWidth - (this.element.width() + this.element.offset().left));
                rDivPadding = parseInt($(resizeArea).css('padding-left'), 10) + parseInt($(resizeArea).css('padding-right'), 10);
                $(resizeArea).offset({ left: contentLeft - rDivPadding });
            }
            else
                $(resizeArea).offset({ left: curGroup.parent().position().left });
			if(this.element.find('.e-ribbonpin').length>0)
			 $(resizeArea).offset({ top:this.element.height()+this.element.find('.e-active-content').height()});
            $(resizeArea).width(curGroup.not('.e-contentbottom').css("width"));
            $(resizeArea).append(curGroup.show()).show();
        },
        _wireEvents: function () {
            this._on($(document), "keydown", this._OnKeyDown);
			this._on(this.element, "keydown", this._ribbonKeyDown);
            this._on(this.element.find("#" + this._id + "_expandCollapse"), ej.eventType.click, this._tabExpandCollapse);
            this._on(this.element.find(".e-groupexpander"), ej.eventType.click, this._onGroupExpandClick);
            $(window).bind("resize", $.proxy(this._ribbonWindowResize, this));
            this._on($(document), 'click', this._ribbonDocClick);
        }
    });
    ej.Ribbon.alignType = {
        rows: "rows",
        columns: "columns"
    };
    ej.Ribbon.applicationTabType = {
        menu: "menu",
        backstage: "backstage"
    };
    ej.Ribbon.type = {
        button: "button",
        splitButton: "splitbutton",
        dropDownList: "dropdownlist",
        custom: "custom",
        toggleButton: "togglebutton",
        gallery: "gallery"
    };
    ej.Ribbon.customItemType = {
        button: "button",
        menu: "menu"
    };
    ej.Ribbon.itemType = {
        button: "button",
        tab: "tab"
    };
    ej.Ribbon.Locale = {};

    ej.Ribbon.Locale["en-US"] = {
        ExpandCollapse: "Expand Collapse"
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.widget("ejSpreadsheet", "ej.Spreadsheet", {
        _rootCSS: "e-spreadsheet",
        // widget element will be automatically set in this
        element: null,
        _excelFilter: null,
        validTags: ["div"],
        // user defined model will be automatically set in this
        model: null,
        defaults:
        {
            allowFreezing: true,
            enableContextMenu: true,
            allowEditing: true,
            allowResizing: true,
            allowMerging: true,
            allowWrap: true,
            allowUndoRedo: true,
            allowClipboard: true,
            allowDataValidation: true,
            allowConditionalFormats: true,
            allowSearching: true,
            allowKeyboardNavigation: true,
            allowAutoFill: true,
            allowSelection: true,
            allowSorting: true,
            allowDragAndDrop: true,
            allowCharts: true,
            enablePersistence: false,
            allowCellFormatting: true,
            allowFormulaBar: true,
            showRibbon: true,
            allowComments: true,
            allowFiltering: true,
            allowInsert: true,
            allowDelete: true,
            allowFormatPainter: true,
            allowHyperlink: true,
            allowImport: true,
            allowFormatAsTable: true,
            allowAutoSum: true,
            allowPivotTable: false,
            allowLockCell: true,
            columnCount: 21,
            rowCount: 20,
            rowHeight: 20,
            columnWidth: 64,
            sheetCount: 1,
            undoredoStep: 20,
            activeSheetIndex: 1,
            apWidth: 200,
            cssClass: "",
            userName: "",
            locale: "en-US",
            pageSize: "A4",
            sheets: [],
            nameManager: [],
            currentViewData: null,
            query: null,
            actionBegin: null,
            actionComplete: null,
            beforeCellFormat: null,
            cellFormatting:null,
            beforeOpen: null,
            beforeDrop: null,
            drop: null,
            cellEdit: null,
            pagerClick: null,
            cellClick: null,
            _pivotPrint: null,
            tabClick: null,
            tabSelect: null,
            _createPivotTable: null,
            beforePanelOpen: null,
            load: null,
            loadComplete: null,
            beforePrint: null,
            editRangeBegin: null,
            editRangeComplete: null,
            cellHover: null,
            menuClick: null,
            contextMenuClick: null,
            ribbonClick: null,
            dragStart: null,
            drag: null,
            cellSave: null,
            beforeCellSelect: null,
            autoFillBegin: null,
            autoFillComplete: null,
            seriesRendering: null,
            openFailure:null,
            scrollSettings:
           {
               width: 1300,
               height: 550,
               allowScrolling: true,
               scrollMode: "infinite",
               isResponsive: false,
               allowVirtualScrolling: false
           },
            pageSettings: {
                pageCount: 10,
                currentPage: 1,
                pagerHeight: 26
            },
            exportSettings:
           {
               allowExporting: true,
               excelUrl: null,
               csvUrl: null,
			   password:null
           },
            pictureSettings:
           {
               allowPictures: true,
               height: 220,
               width: 440
           },
            selectionSettings: {
                selectionType: "default",
                selectionUnit: "multirange",
                enableAnimation: false,
                animationType: "easeOutQuart",
                animationTime: 0.001,
                activeCell: ""
            },
            chartSettings: {
                height: 220,
                width: 440
            },
            formatSettings: {
                allowFontFamily: true,
                allowDecimalPlaces: true,
                allowCellBorder: true
            },
            autoFillSettings: {
                fillType: "fillseries",
                showFillOptions: true
            },
            "importSettings": {
                importMapper: "",
                importUrl: "",
                password: ""
            },
            printSettings: {
                allowPageSetup: true,
                allowPageSize: false,
                allowPrinting: true
            }
        },

        _dataSource: function (index) {
            return this.getSheet(index).dataSource;
        },

        getSheet: function (index) {
            return this.model.sheets[index];
        },

        getSheetElement: function (index) {
            return this.element.find("#" + this._id + "_" + this.model.sheets[index]._sheetName.value);
        },

        getDataSource: function (index) {
            return this.getSheet(index).dataSource;
        },

        setDataSource: function (index, value) {
            this.getSheet(index).dataSource = value;
        },

        _getJSSheetHeader: function (sheetIdx) {
            return this._sheets[sheetIdx]._sheetHeader;
        },

        _setJSSheetHeader: function (value, sheetIdx) {
            this._sheets[sheetIdx]._sheetHeader = value;
        },

        _getInputBox: function () {
            return this._inputBox;
        },

        _setInputBox: function (elem) {
            this._inputBox = elem;
        },

        _getIndexBox: function () {
            return this._indexBox;
        },

        _setIndexBox: function (elem) {
            this._indexBox = elem;
        },

        _getJSSheetRowHeader: function (sheetIdx) {
            return this._sheets[sheetIdx]._sheetRowHeader;
        },

        _setJSSheetRowHeader: function (value, sheetIdx) {
            this._sheets[sheetIdx]._sheetRowHeader = value;
        },

        _getJSSheetRowHeaderContent: function (sheetIdx) {
            return this._sheets[sheetIdx]._sheetRowHeaderContent;
        },

        _setJSSheetRowHeaderContent: function (value, sheetIdx) {
            this._sheets[sheetIdx]._sheetRowHeaderContent = value;
        },

        _getJSSheetContent: function (sheetIdx) {
            return this._sheets[sheetIdx]._sheetContent;
        },

        _setJSSheetContent: function (value, sheetIdx) {
            this._sheets[sheetIdx]._sheetContent = value;
        },

        _getContent: function (sheetIdx) {
            return this._sheets[sheetIdx]._content;
        },

        _setContent: function (value, sheetIdx) {
            this._sheets[sheetIdx]._content = value;
        },

        getPager: function () {
            return this._spreadSheetPager;
        },

        setPager: function (value) {
            this._spreadSheetPager = value;
        },

        getFooter: function () {
            return this._footer;
        },

        setFooter: function (value) {
            this._footer = value;
        },

        getMainPanel: function () {
            return this._mainPanel;
        },

        setMainPanel: function (value) {
            this._mainPanel = value;
        },

        getGroupPanel: function () {
            return this._groupPanel;
        },

        setGroupPanel: function (value) {
            this._groupPanel = value;
        },

        getActivationPanel: function () {
            return this._activationPanel;
        },

        setActivationPanel: function (value) {
            this._activationPanel = value;
        },

        getActiveSheetIndex: function () {
            return this.model.activeSheetIndex;
        },

        setActiveSheetIndex: function (value) {
            this.model.activeSheetIndex = value;
        },

        getRows: function (index) {
            return this._sheets[index]._Rows;
        },

        setRows: function (value, sheetIdx) {
            this._sheets[sheetIdx]._Rows = value;
        },

        getFrozenRows: function (sheetIdx) {
            return this.getSheet(sheetIdx)._frozenRows;
        },

        setFrozenRows: function (value, sheetIdx) {
            this.getSheet(sheetIdx)._frozenRows = value;
        },

        getFrozenColumns: function (sheetIdx) {
            return this.getSheet(sheetIdx)._frozenColumns;
        },

        setFrozenColumns: function (value, sheetIdx) {
            this.getSheet(sheetIdx)._frozenColumns = value;
        },

        setFocusTopElem: function (elem) {
            this._focusTopElem = elem;
        },

        getFocusTopElem: function () {
            return this._focusTopElem;
        },

        setFocusRightElem: function (elem) {
            this._focusRightElem = elem;
        },

        getFocusRightElem: function () {
            return this._focusRightElem;
        },

        setFocusBottomElem: function (elem) {
            this._focusBottomElem = elem;
        },

        getFocusBottomElem: function () {
            return this._focusBottomElem;
        },

        setFocusLeftElem: function (elem) {
            this._focusLeftElem = elem;
        },

        getFocusLeftElem: function () {
            return this._focusLeftElem;
        },

        setAutoFillElem: function (elem) {
            this._autoFillElem = elem;
        },

        getAutoFillElem: function () {
            return this._autoFillElem;
        },

        _setAutoFillOptElem: function (elem) {
            this._autoFillOptElem = elem;
        },

        _getAutoFillOptElem: function () {
            return this._autoFillOptElem;
        },

        setActiveCell: function (rowIdx, colIdx, sheetIdx) {
            var sheet = this.getSheet(sheetIdx ? sheetIdx : this.getActiveSheetIndex()), range;
            if (typeof (rowIdx) === "string") {
                range = this.getRangeIndices(rowIdx);
                rowIdx = range[0];
                colIdx = range[1];
            }
            sheet._activeCell = ej.isNullOrUndefined(rowIdx) && ej.isNullOrUndefined(colIdx) ? sheet._startCell : { rowIndex: rowIdx, colIndex: colIdx };
        },

        getActiveCell: function (sheetIdx) {
            return this.getSheet(sheetIdx ? sheetIdx : this.getActiveSheetIndex())._activeCell;
        },

        _setFormulaSuggElem: function (elem) {
            this._formulaSuggElem = elem;
        },

        _getFormulaSuggElem: function () {
            return this._formulaSuggElem;
        },

        _setRowHdrTBody: function (elem, sheetIdx) {
            this._sheets[sheetIdx]._rowHdrTBody = elem;
        },

        _getRowHdrTBody: function (sheetIdx) {
            return this._sheets[sheetIdx]._rowHdrTBody;
        },

        _setContTBody: function (elem, sheetIdx) {
            this._sheets[sheetIdx]._contTBody = elem;
        },

        _getContTBody: function (sheetIdx) {
            return this._sheets[sheetIdx]._contTBody;
        },

        _init: function () {
            var importReq, settings = this.model.importSettings;
            ej.support.stableSort = false;
            this._initialLoad = true;            
            if (this.model.allowImport && settings.importUrl.length) {
                importReq = new FormData();
                importReq.append("Url", settings.importUrl);
                importReq.append("Password", settings.password);
                this["import"](importReq);
            }
            else
                this._initBase();
            this._trigger("loadComplete", { model: this.model });
            this._initialLoad = false;
        },

        _initBase: function () {
            this._browserDetails = ej.browserInfo();
            if (this.model.rowHeight === 20 && this._browserDetails.name === "mozilla")
                this.model.rowHeight++;
            this.element.attr("tabindex", 0).css({ height: this.model.scrollSettings.height, width: this.model.scrollSettings.width });
            this.element.ejWaitingPopup();
            this._initProperties();
            this._initCSSTag();
            this._initLayout();
            this._initDependencies();
            this._wireEvents();
        },

        _initProperties: function () {
            var model = this.model, features = ej.spreadsheetFeatures;
            this.model.sheets.splice(0, 0, {});
            this._initPrivateProperties();
            this._updateFormulaCollection();
            this._updateCellType();
            this.XLClipboard = new features.clipboard(this);
            this.XLEdit = new features.editing(this);
            this.XLPrint = new features.print(this);
            if (model.showRibbon) {
                this.XLRibbon = new features.ribbon(this);
                this.XLRibbon._setButtonHeights();
            }
            if (model.allowDragAndDrop)
                this.XLDragDrap = new features.dragAndDrop(this);
            if (model.allowKeyboardNavigation)
                this.XLCellNav = new features.cellNavigation(this);
            if (model.allowFiltering)
                this.XLFilter = new features.filter(this);
            if (model.allowAutoFill)
                this.XLDragFill = new features.dragFill(this);
            if (model.allowCellFormatting || model.allowFormatAsTable || model.allowFormatPainter)
                this.XLFormat = new features.cellFormatting(this);
            if (model.allowConditionalFormats)
                this.XLCFormat = new features.cFormat(this);
            if (model.allowSearching)
                this.XLFndRp = new features.findnreplace(this);
            if (model.allowCharts)
                this.XLChart = new features.chart(this);
            if (model.exportSettings.allowExporting)
                this.XLExport = new features.exporting(this);
            if (model.scrollSettings.allowScrolling)
                this.XLScroll = new features.scroller(this);
            if (model.allowSorting)
                this.XLSort = new features.sorting(this);
            if (model.allowFreezing)
                this.XLFreeze = new features.freezepane(this);
            if (model.allowComments)
                this.XLComment = new features.comments(this);
            if (model.allowSelection)
                this.XLSelection = new features.selection(this);
            if (model.allowCharts || model.pictureSettings.allowPictures)
                this.XLShape = new features.shape(this);
            if (model.allowResizing)
                this.XLResize = new features.resizing(this);
            if (model.enableContextMenu)
                this.XLCMenu = new features.contextmenu(this);
            if (model.allowDataValidation)
                this.XLValidate = new features.validation(this);
            if (features.math)
                $.extend(ej.Spreadsheet.prototype, features.math);
        },

        _initPrivateProperties: function () {
            var i = 1, obj = { rowIndex: 0, colIndex: 0 }, text = this._getLocStr("AutoFillOptions").split("/"),
                sheetIdx = this.getActiveSheetIndex(), numFormat = Globalize.culture(this.model.locale).numberFormat;
            this._dataContainer = { sheets: {}, hashCode: [], sharedData: [], cellType: [], valData: [], cFormatData: [], pictures: [] };
            this._newSIndex = 1;
            this._sheets = this.importSheet ? this._sheets : [];
            while (i <= this.model.sheetCount) {
                this._sheetSettings(i);
                this._dataContainer.sheets[i] = {};
                this._sheets[i] = { _contentColGroup: null, _content: null, _sheetRowHeaderContent: null, _sheetHeader: null, _sheetRowHeader: null, _sheetContent: null, _Rows: null, _calcEngine: null, _contTBody: null, _rowHdrTBody: null };
                this._newSIndex++;
                i++;
            }
            this._detachTable = null;
            this._isPrint = false;
            this._isSheetInsert = false;
            this._isSheetRename = false;
            this._sheetCopy = { isSheetCopy: false, sheetContent: null };
            this._currentJsonData = {};
            this._alertDialog = { element: null, action: null };
            this._dataManager = this.getDataSource(sheetIdx) instanceof ej.DataManager ? this.getDataSource(sheetIdx) : this.getDataSource(sheetIdx) ? ej.DataManager(this.getDataSource(sheetIdx)) : null;
            this._isUndo = false;
            this._undoCollection = [],
            this._redoCollection = [],
            this._hasComment = false;
            this._tableLayout = '';
            this._ctrlKeyCount = 0;
            this._formulaRange = [];
            this._formulaHeight = 36;
            this._isSort = false;
            this._isPaste = false;
            this._undoredoStep = 20,
            this._isFAT = false;
            this._isFATResize = false;
            this._isDisplayHeader = false;
            this._currencySymbol = numFormat.currency.symbol || "$";
            this._percentSymbol = numFormat.percent.symbol || "%";
            this.operators = {
                '<': function (a, b) { return a < b },
                '>': function (a, b) { return a > b },
                '>=': function (a, b) { return a >= b },
                '<=': function (a, b) { return a <= b },
                '==': function (a, b) { return a === b },
                '!=': function (a, b) { return a !== b },
                '!': function (a) { return !a },
                '=': function (a) { return a }
            };
            this.parse = {
                "parseInt": function (a) { return parseInt(a); },
                "parseFloat": function (a) { return parseFloat(a); },
                "parseDate": function (a) { return new Date(a).getTime(); },
                "parseTime": function (a) { return new Date("01/01/1990 " + a).getTime(); },
                "length": function (a) { return (a.charAt(0) === "^") ? parseInt(a.split("^")[1]) : a.length; }
            };
            this._selMergeCells = [];
            this._mergeType = "merge-cells";
            this._delCells = [];
            this._deletedText = [];
            this._insDelStatus = "";
            this._isUndoRedo = false;
            this._indexBox = null;
            this._inputBox = null;
            this._focusTopElem = null;
            this._focusRightElem = null;
            this._focusBottomElem = null;
            this._focusLeftElem = null;
            this._dStartCell = obj;
            this._dEndCell = obj;
            this._pStartCell = obj;
            this._pEndCell = obj;
            this._pFillCell = obj;
            this._borderFocus = ["e-focusright", "e-focusbottom"];
            this._cutFocus = ["e-cutright", "e-cutbottom"];
            this._autoFillFocus = ["e-autofillright", "e-autofillbottom"];
            this._formulaBorder = [["e-bborderright", "e-bborderbottom"], ["e-rborderright", "e-rborderbottom"], ["e-vborderright", "e-vborderbottom"], ["e-gborderright", "e-gborderbottom"], ["e-pborderright", "e-pborderbottom"], ["e-oborderright", "e-oborderbottom"]];
            this._ctrlFormulaBorder = [["e-bctrlborderright", "e-bctrlborderbottom"], ["e-rctrlborderright", "e-rctrlborderbottom"], ["e-vctrlborderright", "e-vctrlborderbottom"], ["e-gctrlborderright", "e-gctrlborderbottom"], ["e-pctrlborderright", "e-pctrlborderbottom"], ["e-octrlborderright", "e-octrlborderbottom"]];
            this._borderTypes = [
               { id: "bottomborder", text: this._getLocStr("BottomBorder"), parentId: null, sprite: "e-icon e-ss-bottomborder" },
               { id: "topborder", text: this._getLocStr("TopBorder"), parentId: null, sprite: "e-icon e-ss-topborder" },
               { id: "leftborder", text: this._getLocStr("LeftBorder"), parentId: null, sprite: "e-icon e-ss-leftborder" },
               { id: "rightborder", text: this._getLocStr("RightBorder"), parentId: null, sprite: "e-icon e-ss-rightborder" },
               { id: "noborder", text: this._getLocStr("NoBorder"), parentId: null, sprite: "e-icon e-ss-noborder" },
               { id: "allborder", text: this._getLocStr("AllBorder"), parentId: null, sprite: "e-icon e-ss-allborder" }
            ];
            this._isApply = false;
            this._datePattern = ["year", "month", "day", "hours", "minutes", "seconds"];
            this._styleColl = {};
            this.importSheet = this.importSheet ? true : false;
            this._borderStyle = "solid";
            this._borderColor = "#000000";
            this._borderStyles = ["solid", "dashed", "dotted"];
            this._mergeCenter = false;
            this._ddlCell = null;
            this._cOpt = {};
            this._shapeChange = false;
            this._dupDetails = false;
            this._paste = false;
            this._autoFillElem = null;
            this._autoFillOptElem = null;
            this._rowCount = this.model.rowCount;
            this._colCount = this.model.columnCount;
            this._fillOptionMenuData = [
              { id: "copycells", text: text[0], parentId: null, sprite: "e-icon e-ss-fillselect" },
              { id: "fillseries", text: text[1], parentId: null, sprite: "e-icon e-ss-filloption" },
              { id: "fillformattingonly", text: text[2], parentId: null, sprite: "e-icon e-ss-filloption" },
              { id: "fillwithoutformatting", text: text[3], parentId: null, sprite: "e-icon e-ss-filloption" },
              { id: "flashfill", text: text[4], parentId: null, sprite: "e-icon e-ss-filloption" }
            ];
            this._formulaSuggElem = null;
            this._isFormulaSuggestion = false;
            this._ctrlKeyEnabled = false;
            this._shiftKeyEnabled = false;
            this._preventctrlkey = false;
            this._rAlign = "e-ralign";
            this._cAlign = "e-calign";
            this._chartBorder = [["e-rcbright", "e-rcbbottom"], ["e-vcbright", "e-vcbbottom"], ["e-bcbright", "e-bcbbottom"]];
            this._selectionBorder = ["e-selecttop", "e-selectright", "e-selectbottom", "e-selectleft"];
            this._autofillBorder = ["e-autofilltop", "e-autofillright", "e-autofillbottom", "e-autofillleft"];
            this._thinBorder = ["e-thinright", "e-thinbottom"];
            this._rcRegx = new RegExp('\\b' + "e-rc" + '.*\\b', 'g');
            this._mcRegx = new RegExp('\\b' + "e-mc" + '.*\\b', 'g');
            this._formatRegx = new RegExp('\\b' + "e-format" + '.*?\\b', 'g');
            this._borderRegx = new RegExp('\\b' + "e-border" + '.*?\\b', 'g');
            this._wrapRow = false;
            this._formulaChanged = [];
            this._ref = [];
            this._prevText = "";
            this._selectDataval = {};
            this._undoInputobj = {};
            this._cureditedCell = '';
            this._colHeaderHeight = 22;
            this._scrollerHeight = 18;
            this._scrollAdjust = false;
            this._canAdjustScroll = false;
            this._responsiveWidth = this.model.scrollSettings.width;
            this._calcID = CalcEngine.createSheetFamilyID();
        },

        _sheetSettings: function (idx) {
            var obj = { rowIndex: 0, colIndex: 0 }, sheet = this.getSheet(idx), sheetName = this._generateSheetName(this._newSIndex);
            this.model.sheets[idx] = {
                colCount: sheet && sheet.colCount || this.model.columnCount,
                rowCount: sheet && sheet.rowCount || this.model.rowCount,
                _formulaCollection: {},
                columnsWidthCollection: [],
                rowsHeightCollection: [],
                _spreadSheetPager: null,
                showHeader: true,
                showGridlines: true,
                dataSource: sheet && sheet.dataSource || null,
                columns: sheet && sheet.columns || [],
                _excelFilter: null,
                filterSettings:
                {
                    filteredColumns: [],
                    maxFilterChoices: 1000,
                    filteredRange: [],
                    range: "",
                    tableRange: []
                },
                _frozenRows: sheet && sheet._frozenRows || 0,
                _frozenColumns: sheet && sheet._frozenColumns || 0,
                _selectedRange: {},
                _selectedCellColors: [],
                columnWidth: this.model.columnWidth,
                rowHeight: this.model.rowHeight,
                _currentTopFrozenRow: 0,
                _currentLeftFrozenColumn: 0,
                _frozenHeight: 0,
                _frozenWidth: 0,
                _firstRow: { idx: 0, value: 0 },
                _firstCol: { idx: 0, value: 0 },
                _topRow: { idx: 0, value: 0 },
                _leftCol: { idx: 0, value: 0 },
                _filterColl: {},
                _mergeColl: {},
                _selectedCells: [],
                selectedRange: [],
                _tableManager: {},
                _header: {},
                _selectedCellIndexes: [],
                _startCell: obj,
                _endCell: obj,
                _activeCell: obj,
                _autoFillCell: obj,
                rangeSettings: sheet && sheet.rangeSettings || [],
                _sheetName: { text: sheetName, value: this._getLocStr("Sheet") + this._newSIndex, isVisible: true },
                _multipleFiltering: false,
                _multifilterIdx: [],
                _filteredByColor: "",
                staticWidth: 0,
                _fRow: 0,
                _fCol: 0,
                hideColsCollection: [],
                hideRowsCollection: [],
                filteredRowsCollection: [],
                _rHeightCollection: [],
                _cWidthCollection: [],
                _hiddenHeight: 0,
                _hiddenWidth: 0,
                _frHiddenHeight: 0,
                _fpHiddenHeight: 0,
                _fpHiddenWidth: 0,
                _pTarget: {},
                _cSpan: 1,
                _rSpan: 1,
                _usedRange: { rowIndex: 0, colIndex: 0 },
                _mergedCells: [],
                _shapeMngr: { picture: [], chart: [], sharedPics: [] },
                _nmValue: {},
                _nmEdit: false,
                _importRendered: false,
                _defRowCount: 0,
                _defColCount: 0,
                _wrapHtCollection: {},
                _isChartBorderDrawn: false,
                _virtualBlockCnt: 0,
                _virtualBlockRowCnt: 0,
                _virtualContBlockIdx: 1,
                _virtualTopTBodyHgt: 0,
                _virtualBottomTBodyHgt: 0,
                _virtualTBodyHgt: 0,
                _scrollTop: 0,
                _scrollLeft: 0,
                _virtualRowIdx: [],
                _virtualTopTBodyOffset: { top: 0, bottom: 0 },
                _virtualContTBodyOffset: { top: 0, bottom: 0 },
                _virtualBottomTBodyOffset: { top: 0, bottom: 0 },
                _isVirtualTopReached: true,
                _isVirtualEndReached: false,
                _isLoaded: false,
                _isRowSelected: false,
                _isColSelected: false,
                _isSheetSelected: false,
                _isSheetProtected: false
            };
        },

        _generateSheetName: function (index) {
            var i, len, sName, sheet, shtNo = 1, sheetModel = this.model.sheets, txt = this._getLocStr("Sheet"),
                sheet = sheetModel[index];
            sName = sheet && sheet.sheetName || (txt + index);
            for (i = 1, len = sheetModel.length; i < len; i++) {
                if (sheetModel[i]._sheetName && sName === sheetModel[i]._sheetName.text) {
                    i = 1;
                    sName = sheet && sheet.sheetName ? (sheet.sheetName + shtNo++) : (txt + ++index);
                }
            }
            return sName;
        },

        _initCSSTag: function () {
            this._initializeStyleTag();
            this._initCssRules();
        },

        _initializeStyleTag: function () {
            if (!document.getElementById(this._id + '_sscustomformat')) {
                var rtag, stag, str, len, i = 0;
                str = 's:11pt/b:#FFC7CE;c:#82012A;f:calibri;s:11pt/b:#C6EED2;c:#0A7043;f:calibri;s:11pt/b:#FEEC9B;c:#826717;f:calibri;s:11pt/f:Calibri;w:bold;v:top;s:11pt/f:calibri;s:18pt/b:#DDE5F6;f:calibri;s:11pt/b:#FDE0D2;f:calibri;s:11pt/b:#EEEEEC;f:calibri;s:11pt/b:#FEF4CD;f:calibri;s:11pt/b:#9CC0E7;f:calibri;s:11pt/b:#F9AD7B;f:calibri;s:11pt/b:#C9C9C9;f:calibri;s:11pt/b:#FED965;f:calibri;s:11pt/b:#559AD9;c:#FFFFFF;f:calibri;s:11pt/b:#EB7E2F;c:#FFFFFF;f:calibri;s:11pt/b:#A5A5A5;c:#FFFFFF;f:calibri;s:11pt/b:#FDBF01;c:#FFFFFF;f:calibri;s:11pt'.split('/');
                rtag = document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0];
                stag = document.createElement('style');
                stag.setAttribute('id', this._id + '_sscustomformat');
                stag.type = 'text/css';
                rtag.appendChild(stag);
                if (this.XLFormat) {
                    len = str.length;
                    while (i < len) {
                        this.XLFormat._createFormatClass(this._getStyleAsJSON(str[i]));
                        i++;
                    }
                }
            }
        },

        _getStyleAsJSON: function (str) {
            var istr, lwn, i = 0, obj = {}, prop = { b: 'background-color', c: 'color', f: 'font-family', s: 'font-size', w: 'font-weight', v: 'vetical-align', bo: 'border' };
            str = str.split(';'), len = str.length;
            while (i < len) {
                istr = str[i].split(':');
                obj[prop[istr[0]]] = istr[0] === 'bo' ? this.XLRibbon._getBorderObj(istr[1].split('-')[0], istr[1].split('-')[1]) : istr[1];
                i++;
            }
            return obj;
        },

        _initCssRules: function () {
            var stag, prop;
            stag = document.getElementById(this._id + '_sscustomformat');
            prop = '.e-spreadsheet .e-hide, .e-spreadsheet .e-mc-hide, .e-spreadsheet .e-fr-hide, .e-spreadsheet .e-r-hide{ display: none}';
            prop += '.e-spreadsheet .e-rowcell.e-redft { background-color: #FFC7CE; color: #9C0055;} .e-spreadsheet .e-rowcell.e-yellowft { background-color: #FFEB9C; color: #9C6500;}';
            prop += '.e-spreadsheet .e-rowcell.e-greenft { background-color: #C6EFCE;   color: #006100;} .e-spreadsheet .e-rowcell.e-redf { background-color: #FFC7CE;} .e-spreadsheet .e-rowcell.e-redt { color: #9C0055;}';
            prop += '.e-spreadsheet .e-rowcell.e-hlcell { background-color: #FFFF00; color: #FF0000;} .e-spreadsheet .e-rowcell.e-goto-blank { background-color: #C6C6C6;}';
            prop += '.e-fc-hide, .e-c-hide  {width: 0 !important;}';
            stag.styleSheet ? stag.styleSheet.cssText += prop : stag.appendChild(document.createTextNode(prop));
        },

        _initLayout: function () {
            if (this.model.showRibbon) {
                this.XLRibbon._renderRibbon();
                this._hideOtherControls();
            }            
            if(this.model.allowFormulaBar)
                this._renderFormulaBar();
            this._renderLayout();
            this._renderFooter();
        },

        _renderLayout: function () {
            var i, cnt;
            this._addInitTemplate();
            this._renderPanel();
            if (this.model.scrollSettings.allowVirtualScrolling)
                i = cnt = this.getActiveSheetIndex()
            else{
                i = 1;
                cnt = this.model.sheetCount;
            }
            while (i <= cnt) {
                this._renderSheet(i);
                i++;
            }
        },

        _addInitTemplate: function () {
            var sheet, idx = 1;
            while (idx <= this.model.sheetCount) {
                sheet = this.getSheet(idx);
                if(sheet.dataSource)
                    this._initColumns(idx, sheet.dataSource[0]);
                idx++;
            }
        },

        _initColumns: function (index, object) {
            var idx, value, field, $field, sheet = this.getSheet(index);
            while (object.length)
                object = object.items[0];
            if (sheet.columns.length === 0 && object) {
                for (field in object) {
                    if (object.hasOwnProperty(field) && (typeof (object[field]) != "object" || object[field] instanceof Date)) {
                        value = object[field];
                        sheet.columns.push({
                            field: field,
                            type: value != null ? (value.getDay ? "date" : typeof (value)) : null
                        });
                    }
                }
            }
            else {
                for (idx = 0, len = sheet.columns.length; idx < len; idx++) {
                    if (ej.isNullOrUndefined(sheet.columns[idx].type)) {
                        $field = !ej.isNullOrUndefined(sheet.columns[idx].field) ? ej.getObject(sheet.columns[idx].field, object) : null;
                        sheet.columns[idx].type = $field != null ? ($field.getDay ? "date" : typeof ($field)) : null;
                    }
                }
            }
            sheet.colCount = sheet.columns.length;
        },

        _renderPanel: function () {
            var $mainDiv, div, settings = this.model.scrollSettings, css = { height: settings.height, width: settings.width };
            $mainDiv = ej.buildTag('div.e-spreadsheetgrppanel', "", css, { id: this._id + "GroupPanel" });
            this.setGroupPanel($mainDiv);
            div = ej.buildTag('div.e-spreadsheetmainpanel', this._renderDIV("e-spreadsheet-list", ""), css, { id: this._id + "MainPanel" });
            $mainDiv.append(div);
            this.setMainPanel(div);
            div = ej.buildTag('div.e-spreadsheetactpanel', "", { height: settings.height, width: this.model.apWidth, display: "none" }, { id: this._id + "ActivationPanel" });
            $mainDiv.append(div);
            this.setActivationPanel(div);
            this.element.append($mainDiv);
        },
        
        _renderSheet: function (sheetIdx) {
            var div, settings = {}, sheet = this.getSheet(sheetIdx), scroll = this.model.scrollSettings,
                name = sheet._sheetName.value, elem = this.element.find('#' + this._id + '_' + name);
            this._trigger("load", { sheetIndex: sheetIdx, model: this.model });
            div = elem.length === 1 ? elem : ej.buildTag('div.e-sheet ', '', {}, { id: this._id + '_' + name });
            div.append(this._renderContent(sheetIdx));
            if(!elem.length)
                this.element.find(".e-spreadsheet-list").append(div);
            this._refreshContent(sheetIdx);
            if (this.model.enableContextMenu)
                this.XLCMenu._initContextMenu();
            this.XLEdit._processEditing(sheetIdx);
            this._initializePicture(sheetIdx);
            this._sheets[sheetIdx]._calcEngine = new CalcEngine(this, this._calcID);
            this._sheets[sheetIdx]._calcEngine.setUseDependencies(true);
            this._sheets[sheetIdx]._calcEngine.registerGridAsSheet(name, name, this._calcID);
            // should be in refresh content after renderviewport(due to edit template issue)
            settings.width = scroll.width;
            settings.height = scroll.height;           
            this._setSheetWidthHeight(sheetIdx, settings, null, "refresh");
            if(sheetIdx !== this.getActiveSheetIndex())
                div.hide();
            sheet._isLoaded = true;
        },

        _renderContent: function (sheetIdx) {
            var elems = [], div = ej.buildTag('div.e-spreadsheetheader'), innerDiv = ej.buildTag('div.e-headercontent'),
               colHeader = ej.buildTag('div.e-spreadsheetcolumnheader e-icon e-ss-selectall  e-ss-cursor', "", { "float": "left", "width": 29, "height": 20.5 }, {}),
               contentContainer = ej.buildTag('div.e-spreadsheetcontentcontainer'), cont = ej.buildTag("div.e-content e-ss-cursor"),
               rowHeader = ej.buildTag('div.e-spreadsheetrowheader', "", { "float": "left", "width": "30px" }, {}), rowHdrCont = ej.buildTag("div.e-rowheadercontent");
            this._setJSSheetHeader(div, sheetIdx);
            div.append(colHeader);
            div.append(innerDiv);
            elems.push(div);
            div = ej.buildTag("div.e-spreadsheetcontent");
            this._setJSSheetContent(div, sheetIdx);
            this._setContent(contentContainer, sheetIdx);
            this._setJSSheetRowHeader(rowHeader, sheetIdx);
            this._setJSSheetRowHeaderContent(rowHdrCont, sheetIdx);
            rowHeader.append(rowHdrCont);
            contentContainer.append(cont);
            div.append(rowHeader);
            div.append(contentContainer);
            elems.push(div);
            return elems;
        },

        _refreshContent: function (sheetIdx) {
            var contentrows, rowhdrrows, sheet = this.getSheet(sheetIdx), scroll = this.model.scrollSettings,
                isVirtualScroll = scroll.allowVirtualScrolling;
            this._calculateRowAndColumnCount(sheetIdx);
            sheet.rowsHeightCollection = [this.model.rowHeight];
            sheet._rowHeightCollection = [{ idx: 0, value: 0 }];
            sheet._colWidthCollection = [{ idx: 0, value: 0 }];
            if (sheet.dataSource && !this._sheetCopy.isSheetCopy) {
                sheet._isTemplate = true;
                if (!this.importSheet)
                    sheet.rowCount = sheet.dataSource.length;
                if (sheet.columns.length * sheet.columnWidth < scroll.width) {
                    colWidth = scroll.width / sheet.columns.length - 1;
                    sheet.columnWidth = colWidth;
                }
            }
            if(!this._sheetCopy.isSheetCopy)
                this._refreshTemplates(sheetIdx);
            this._getJSSheetHeader(sheetIdx).find(".e-headercontent").append(this._renderHeaderColumns(sheetIdx));
            this._getJSSheetRowHeaderContent(sheetIdx).append(this._renderHdrRows(sheetIdx));
            this._getContent(sheetIdx).find(".e-content").append(this._renderContTable(sheetIdx));
            if(isVirtualScroll){
                this._refreshViewport(0, sheetIdx);
                contentrows = this._getContent(sheetIdx).find(".e-table tbody:eq(1) tr");
                rowhdrrows = this._getJSSheetRowHeader(sheetIdx).find(".e-table tbody:eq(1) tr");
            }
            else {
                rowhdrrows = this._getJSSheetRowHeader(sheetIdx).find(".e-table").get(0).rows;
                contentrows = this._getContent(sheetIdx).find(".e-table").get(0).rows;
            }
            this.setRows([rowhdrrows, contentrows], sheetIdx);
        },

        _renderHeaderColumns: function (sheetIdx) {
            var columnHeader, colGroup, valuecollection, thead = $(document.createElement('thead')),
                table = ej.buildTag('table.e-table', "", {}, { cellspacing: "0.25px" });
            valuecollection = this._renderHeaderCol(sheetIdx, this.getSheet(sheetIdx).colCount, 0);
            colGroup = valuecollection.pop();
            columnHeader = valuecollection.pop();
            thead.append(columnHeader);
            table.append(colGroup).append(thead);
            return table;
        },

        _renderHeaderCol: function (sheetIdx, colCount, startIndex) {
            var $columnHeader = ej.buildTag('tr.e-columnheader'), field, value, $colGroup = $(document.createElement('colgroup'));
            var $headerCell, cell, $headerCellDiv, col, i, sheet = this.getSheet(sheetIdx), valuecollection = [], colWidth = sheet.columnWidth;
            for (i = startIndex ; i < colCount ; i++) {
                this._sheetCopy.isSheetCopy && (colWidth = !ej.isNullOrUndefined(sheet.columnsWidthCollection[i]) ? sheet.columnsWidthCollection[i] : sheet.columnWidth);
                $headerCell = ej.buildTag('th.e-headercell', "", {}, {});
                $headerCellDiv = sheet._isTemplate ? ej.buildTag('div.e-headercelldiv', this._generateHeaderText(sheetIdx, i), {}, {}) : ej.buildTag('div.e-headercelldiv', this._generateHeaderText(sheetIdx, i + 1), {}, {});
                $headerCell.append($headerCellDiv);
                field = $headerCellDiv.text();
                col = ej.buildTag('col', "", { width: colWidth }, {});
                $columnHeader.append($headerCell);
                $colGroup.append(col);
                sheet.columnsWidthCollection[i] = colWidth;
                sheet._colWidthCollection.push({ idx: i + 1, value: sheet._colWidthCollection[sheet._colWidthCollection.length - 1].value + colWidth });
                if (!sheet._isTemplate) {
                    sheet.columns.push({
                        field: field,
                        type: value != null ? (value.getDay ? "date" : typeof (value)) : null,
                        width: colWidth
                    });
                }
            }
            valuecollection.push($columnHeader);
            valuecollection.push($colGroup);
            return valuecollection;
        },

        _renderHdrRows: function (sheetIdx) {
            var i, cnt, table, isVirtualScroll = this.model.scrollSettings.allowVirtualScrolling, sheet = this.getSheet(sheetIdx),
                rhColgroup = $(document.createElement("colgroup")), col = document.createElement("col");
            col.style["width"] = "30px";
            rhColgroup.append(col);
            if (sheet.dataSource && !this.importSheet)
                sheet.rowCount = sheet.dataSource.length;
            table = ej.buildTag("table.e-table", rhColgroup.get(0).outerHTML);
            if (!this._sheetCopy.isSheetCopy) {
                i = 0, cnt = sheet.rowCount - 1;
                while (i < cnt) {
                    sheet.rowsHeightCollection.push(this.model.rowHeight);
                    sheet._rowHeightCollection.push({ idx: i + 1, value: sheet._rowHeightCollection[sheet._rowHeightCollection.length - 1].value + this.model.rowHeight });
                    i++;
                }
            }
            if (isVirtualScroll)
                table.append("<tbody class='e-ss-virtualtop'></tbody><tbody></tbody><tbody class='e-ss-virtualbottomm'></tbody></table></div>");            
            else
                table.append("<tbody>" + this._renderRowHdr(null, sheetIdx) + "</tbody>");
            this._setRowHdrTBody(table.find("tbody").eq(isVirtualScroll ? 1 : 0), sheetIdx);
            return table;
        },

        _renderRowHdr: function (rowIdxArr, sheetIdx) {
            var idx, obj, i = 0, data = [], sheet = this.getSheet(sheetIdx),
                cnt = this.model.scrollSettings.allowVirtualScrolling ? rowIdxArr.length : sheet.rowCount;
            while (i < cnt) {
                idx = this.model.scrollSettings.allowVirtualScrolling ? rowIdxArr[i] : i;
                obj = { height: sheet.rowsHeightCollection[idx] < this.model.rowHeight ? this.model.rowHeight : sheet.rowsHeightCollection[idx], rowidx: idx };
                obj.class = this.XLEdit.getPropertyValue(idx, 0, "hRow") ? "e-rowheader e-hrow " : "e-rowheader";
                obj.wrap = this.XLEdit.getPropertyValue(idx, 0, "wrapRow") ? "e-sswraprow" : "";
                if (this._inRow(sheet.selectedRange, idx)) {
                    obj.class += " e-rowhighlight";
                    if(sheet.selectedRange[1] === 0 && sheet.selectedRange[3] === sheet.colCount - 1)
                        obj.class += " e-rowselected";
                }
                data.push(obj);
                i++;
            }
            return $.render[this._id + "JSONRowHdrTemplate" + sheetIdx](data);
        },

        _renderContTable: function (sheetIdx) {
            var table, isVirtualScroll = this.model.scrollSettings.allowVirtualScrolling;
            if (this._sheetCopy.isSheetCopy)
                table = this._sheetCopy.sheetContent.outerHTML;
            else {
                table = ej.buildTag("table.e-table", this._sheets[sheetIdx]._contentColGroup.outerHTML);
                if (isVirtualScroll)
                    table.append("<tbody class='e-ss-virtualtop'></tbody><tbody></tbody><tbody class='e-ss-virtualbottom'></tbody>");
                else
                    table.append("<tbody>" + this._renderData(null, sheetIdx) + "</tbody>");
            }
            this._setContTBody(table.find("tbody").eq(isVirtualScroll ? 1 : 0), sheetIdx);
            return table;
        },

        _initDependencies: function () {
            var i = 1, model = this.model;
            if (model.allowSelection) {
                this._createFocusElement();
                this._initializeSelection();
            }
            this.XLClipboard._initCopyPaste();            
            if (model.pictureSettings.allowPictures)
                this._renderFUpload();
            this._initDialogs();
            while (i <= model.sheetCount) {
                //if(model.sheets[i].rangeSettings.dataSource)
                    //this.updateRange(i, model.sheets[i].rangeSettings); temp
                i++;
            }
            this.clearUndoRedo();
            if(model.showRibbon)
                this.XLRibbon._disableButtons(['clearfilter'], 'ejButton');
        },

        _initDialogs: function () {
            var model = this.model;
            if (model.allowFiltering) {
                this.XLFilter._renderExcelFilter();
                this.XLFilter._renderFilterDialogs();
            }
            if(model.showRibbon){
                if (model.allowDataValidation) {
                    this.XLRibbon._initValDialog();
                    this.XLRibbon._renderValDialog();
                }
                if(model.allowConditionalFormats)
                    this.XLRibbon._initCFDialog();
                this.XLRibbon._initHLDialog();
                this.XLRibbon._renderHLDialog();
				this._passwordDlg();
				this._importPasswordDlg();
                if (model.allowSearching) {
                    this.XLRibbon._initFRDialog();
                    this.XLRibbon._renderFRDialog();
                    this.XLRibbon._initGoToDialog();
                    this.XLRibbon._renderGoToDialog();
                }
                this._initMoveCopyDlg();
                this._initUnhideDlg();
                this._renderInsertDeleteDialog();
                this._renderAlertdlg();
            }
            if(model.allowCharts)
                this.XLChart._renderChartRangeDialog();
            if(model.enableContextMenu)
                this.XLCMenu._initCMenuFooter();
        },

        //_renderSheets: function () { //old
        //    var model = this.model;
        //    if (!this._initialLoad && this.showFormulaBar)
        //        $("#" + this._id + "_inputbox").data("ejAutocomplete").option("dataSource", this._formulaCollection);                        
        //},


        _generateHeaderText: function (index, colIndex) {
            var alphabet = "Z", sheet;
            if (index) {
                sheet = this.getSheet(index);
                if (sheet._isTemplate) {
                    if (sheet.columns[colIndex] != undefined)
                        return sheet.columns[colIndex].field;
                }
                else {
                    if (colIndex / 26 > 1)
                        return this._generateHeaderText(index, (colIndex % 26 === 0) ? (colIndex / 26 - 1) : Math.floor(colIndex / 26)) + String.fromCharCode((colIndex % 26) === 0 ? alphabet.charCodeAt("Z") : 64 + (colIndex % 26));
                    else
                        return String.fromCharCode(64 + (colIndex));
                }
            }
        },

        _generateColCount: function (text) {
            var colCount = 0;
            text = text.split('').reverse().join('');
            for (var i = text.length - 1; i >= 0  ; i--) {
                colCount += (text[i].charCodeAt(text[i]) - 64) * (Math.pow(26, i));
            }
            return colCount;
        },

        _getSheetIndex: function (sheetIdx) {
            return sheetIdx ? sheetIdx : this.getActiveSheetIndex();
        },

        _getSheetIdxFromName: function (name) {
            var i = 1, sheets = this.model.sheets, len = sheets.length - 1;
            while (i <= len) {
                if (sheets[i]._sheetName.value === name)
                    return i;
                i++;
            }
            return -1;
        },

        _getAlphaRange: function (sindex, srindex, scindex, erindex, ecindex) {
            srindex = srindex + 1, scindex = scindex + 1, erindex = erindex + 1, ecindex = ecindex + 1;
            return srindex === erindex && scindex === ecindex ? this._generateHeaderText(sindex, scindex) + srindex : this._generateHeaderText(sindex, scindex) + srindex + ":" + this._generateHeaderText(sindex, ecindex) + erindex;
        },

        _getProperAlphaRange: function (sindex, minrindex, mincindex, maxrindex, maxcindex) {
            var range = this._getAlphaRange(sindex, minrindex, mincindex, maxrindex, maxcindex);
            return range.indexOf(':') === -1 ? range + ':' + range : range;
        },

        _getDollarAlphaRange: function (range, checkRange) {
            var sheetIdx = this.getActiveSheetIndex();
            if(checkRange && !this.isRange(range))
                return this.getSheet(sheetIdx)._sheetName.value + "!$" + this._generateHeaderText(sheetIdx, range[1] + 1) + "$" + (range[0] + 1);
            else
                return this.getSheet(sheetIdx)._sheetName.value + "!$" + this._generateHeaderText(sheetIdx, range[1] + 1) + "$" + (range[0] + 1) + ":$" + this._generateHeaderText(sheetIdx, range[3] + 1) + "$" + (range[2] + 1);
        },

        _getWorkbookMaxRowColCnt: function(sheets, fromUsedRange){
            var range, rowCount, colCount, sheet, rcnt = 0, ccnt = 0, i = 1, sheets = sheets ? sheets : this.model.sheets, len = sheets.length;
            while(i < len){
                sheet = sheets[i];
                if(fromUsedRange){
                    if(!sheet.usedRange)
                        continue;
                    range = this.getRangeIndices(sheet.usedRange);
                    rowCount = range[2];
                    colCount = range[3];
                }
                else{
                    rowCount = sheet.rowCount;
                    colCount = sheet.colCount;
                }
                if(rcnt < rowCount)
                    rcnt = rowCount;
                if(ccnt < colCount)
                    ccnt = colCount;
                i++;
            }
            return {rowCount: rcnt, colCount: ccnt };
        },

        _generateEJMenu: function (type, orientation, cssClass) {
            return {
                menuType: (type === "normal") ? ej.MenuType.NormalMenu : ej.MenuType.ContextMenu,
                orientation: (orientation === "vertical") ? ej.Orientation.Vertical : ej.orientation.Horizontal,
                cssClass: cssClass
            };
        },

        _generateEJDD: function (datasource, fields, selectedIndex, width, height, evt) {
            return {
                dataSource: datasource,
                fields: fields,
                selectedItemIndex: selectedIndex,
                width: width,
                height: height,
                cssClass:"e-"+this._id+"-ddl",
                change: $.proxy(this.XLRibbon._frTypeChange, this, evt)
            };
        },

        _refreshWidthHeight: function (sheetIdx, settings, ribbonHeight, action) {
            var footer, colHdrHeight = 0, rowHdrHeight = 0, $content = this._getContent(sheetIdx), contentHt, formulaHeight, sheetRowHdr = this._getJSSheetRowHeader(sheetIdx), borderDiff = 0;
            if(this.model.scrollSettings.isResponsive) {
                if(action === "tabClick" || action === "toggleBtn")
                    borderDiff = this.XLRibbon._ribbonState ? 2 : 1; //Based on the Ribbon Borders
            }
            formulaHeight = this.model.allowFormulaBar ? this._formulaHeight : 0;
            ej.isNullOrUndefined(ribbonHeight) && (ribbonHeight = $("#" + this._id + "_Ribbon").height());
            if (this.model.sheets[sheetIdx].showHeader) {
                colHdrHeight = 20 + 2; // column header 20 + 2 //Border
                rowHdrHeight = sheetRowHdr.width();
            }
            contentHt = settings.height - (ribbonHeight + formulaHeight);
            this.getGroupPanel().height(contentHt);
            this.getMainPanel().height(contentHt);
            settings.height = settings.height - (colHdrHeight + this.model.pageSettings.pagerHeight + formulaHeight + ribbonHeight); // is for column Header
            if ($content.find("#hscrollBar").length > 0) {
                $content.find("#hscrollBar").remove();
                sheetRowHdr.removeClass("e-scrollrowcss");
                this._getJSSheetRowHeaderContent(sheetIdx).height(settings.height);
                $content.find(".e-content").height(settings.height - borderDiff);
            }
            if ($content.find("#vscrollBar").length > 0) {
                $content.find("#vscrollBar").remove();
                sheetRowHdr.removeClass("e-scrollcss");
                $content.find(".e-content").width(settings.width - borderDiff);
            }
            $content.height(settings.height - borderDiff);
            sheetRowHdr.height(settings.height - borderDiff);
            this._getJSSheetContent(sheetIdx).width(settings.width);
            $(this.getFooter()).width(settings.width);			
            settings.width = settings.width - rowHdrHeight; // is for row Header  
			this._alignFormulaBar(settings.width + rowHdrHeight);
            return settings;			
        },
        		
        _setSheetWidthHeight: function (sheetIdx, settings, ribbonHeight, status, action) {
            var content = $(document), pHt, pWt, cWt, cHt, $content = this._getContent(sheetIdx), contentHt, sheetRowHdr = this._getJSSheetRowHeader(sheetIdx);
            var prevSettings, isWPer = (settings.width.toString().indexOf("%") > -1), isHPer = (settings.height.toString().indexOf("%") > -1);
            if (this.model.scrollSettings.isResponsive) {
                pHt = settings.height = isHPer ? this.element.parent().height() : settings.height;
                pWt = settings.width = isWPer ? this.getMainPanel().width() : settings.width;
                if (status === "refresh") {
                    if (this._browserDetails.name.indexOf("msie") > -1) 
                        content = (this.model.showRibbon) ? $(document) : $('body');
                }
                prevSettings = this._refreshWidthHeight(sheetIdx, $.extend(true, {}, settings), ribbonHeight, action);
                action = "repeat";
                settings.height = cHt = isHPer ? this.element.parent().height() : settings.height;
                settings.width = cWt = isWPer ? this.getMainPanel().width() : settings.width;
                if(pWt != cWt || pHt != cHt)
                    settings = this._refreshWidthHeight(sheetIdx, settings, ribbonHeight, action);
                if(pWt === cWt)
                    settings.width = prevSettings.width;
                if(pHt === cHt)
                    settings.height = prevSettings.height;
            }
            else
                settings = this._refreshWidthHeight(sheetIdx,  $.extend(true, {}, settings), ribbonHeight, action);            
            this.model.scrollSettings.allowScrolling && this._renderScroller(sheetIdx, settings);
            if ($content.find("#vscrollBar").length > 0)
                this._getJSSheetHeader(sheetIdx).addClass("e-scrollcss");
            if ($content.find("#hscrollBar").length > 0) {
                contentHt = $content.height()
                sheetRowHdr.height(contentHt - 18);
                this._getJSSheetRowHeaderContent(sheetIdx).height(contentHt - 18);
                sheetRowHdr.addClass("e-scrollrowcss");
            }
        },

        setWidthToColumns: function (widthColl) {
            var col, hdrcol, staticWidth = 0, sheetIdx = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIdx);
            for (var i = 0, len = widthColl.length; i < len; i++) {
                col = this._getContent(sheetIdx).find("col")[widthColl[i].cellIndex];
                hdrcol = this._getJSSheetHeader(sheetIdx).find("col")[widthColl[i].cellIndex];
                if(widthColl[i].width === 0)
                    widthColl[i].width = 64;
                hdrcol.style["width"] = widthColl[i].width + "px";
                col.style["width"] = widthColl[i].width + "px";
                staticWidth += widthColl[i].width;
                sheet.columnsWidthCollection[widthColl[i].cellIndex] = widthColl[i].width;
                sheet.columns[widthColl[i].cellIndex]["width"] = widthColl[i].width;
            }
            this.model.sheets[sheetIdx].staticWidth = staticWidth;
            if (this.model.scrollSettings.allowScrolling) {
                this.XLScroll._getColWidths(sheetIdx, 1);
                this.XLScroll._refreshScroller(sheetIdx, "refresh", "horizontal");
            }
            if (!this.model.sheets[sheetIdx]._importRendered) {
                this.XLSelection._refreshBorder();
                this.model.allowAutoFill &&  this.XLDragFill.positionAutoFillElement();
            }
        },

        _calculateRowAndColumnCount: function (sheetIdx) {
            var diffCount, colUpd, rowUpd, totalWidth, conWidth, diff, i = 0, canUpdt = false, sheet = this.getSheet(sheetIdx), actPnlWidth = 0,
                ribbonHgt = 0, formulaHgt = 0, vPortHgt = 0, totalHeight = sheet.rowCount * this.model.rowHeight,
                conHeight = this.model.scrollSettings.isResponsive ? this.element.parent().height() : this.model.scrollSettings.height,
                scrollerHgt = this.model.scrollSettings.allowScrolling ? this._scrollerHeight : 0, colHdrHgt = sheet.showHeader ? this._colHeaderHeight : 0;
            if (this.model.showRibbon)
                ribbonHgt = $("#" + this._id + "_Ribbon").height();
            if (this.model.allowFormulaBar)
                formulaHgt = this._formulaHeight;
            if (this.model.scrollSettings.allowVirtualScrolling) {
                vPortHgt = conHeight - (ribbonHgt + formulaHgt + colHdrHgt + this.model.pageSettings.pagerHeight + scrollerHgt);
                sheet._virtualBlockRowCnt = Math.ceil(vPortHgt / this.model.rowHeight);
                sheet._virtualBlockRowCnt += 5;
                if (sheet.rowCount < sheet._virtualBlockRowCnt) {
                    diff = sheet._virtualBlockRowCnt - sheet.rowCount;
                    sheet.rowCount = sheet.rowCount + diff;
                    canUpdt = true;
                }
                if (!canUpdt) {
                    diff = sheet.rowCount % sheet._virtualBlockRowCnt;
                    if (diff)
                        sheet.rowCount = sheet.rowCount + (sheet._virtualBlockRowCnt - diff);
                    canUpdt = true;
                }
                if(canUpdt)
                    totalHeight = sheet.rowCount * this.model.rowHeight;                
                sheet._virtualBlockCnt = sheet.rowCount / sheet._virtualBlockRowCnt;
                sheet._virtualTBodyHgt = totalHeight;
            }
            else {
                colUpd = false, rowUpd = false, totalWidth = this.model.columnCount * this.model.columnWidth,
                conWidth = this.element.find(".e-spreadsheetmainpanel").width() + 2;
                if (this.getActivationPanel().is(":visible"))
                    actPnlWidth = this.getActivationPanel().width();
                if ((conWidth - actPnlWidth) > totalWidth) {
                    diffCount = (conWidth - totalWidth) / this.model.columnWidth;
                    this.model.columnCount += Math.ceil(diffCount) + 5;
                    colUpd = true;
                }
                if (conHeight - (ribbonHgt + formulaHgt) > totalHeight) {
                    diffCount = (conHeight - (ribbonHgt + formulaHgt) - totalHeight) / this.model.rowHeight;
                    this.model.rowCount += Math.ceil(diffCount) + 5;
                    rowUpd = true;
                }
                if (colUpd) {
                    if (this.model.columnCount > sheet.colCount)
                        sheet.colCount = this.model.columnCount;
                    else
                        this.model.columnCount = sheet.colCount;
                }
                if (rowUpd) {
                    if (this.model.rowCount > sheet.rowCount)
                        sheet.rowCount = this.model.rowCount;
                    else
                        this.model.rowCount = sheet.rowCount;
                }
                sheet._defRowCount = parseInt((conHeight - (ribbonHgt + formulaHgt)) / this.model.rowHeight) + 10; // for default Row Count
                sheet._defColCount = parseInt((conWidth - actPnlWidth) / this.model.columnWidth) + 10; // for default Column Count
            }
        },

        _refreshTemplates: function (index, skipRowHdr) {
            var i, col, $tdCell, colHdr, data, templates = {}, sheet = this.getSheet(index), tbody = ej.buildTag('tbody'),
                colgrp = ej.buildTag("colgroup"), columns = sheet.columns,
                $tr = $("<tr style='height:{{:height}}px' idx= '{{:rowidx}}'>");
            for (i = 0, len = sheet.colCount; i < len; i++) {
                colHdr = this._generateHeaderText(index, i + 1);
				col = document.createElement("col");
                col.style["width"] = sheet.columnWidth + "px";
                $tdCell = $("<td class='{{:" + colHdr + "_className}}' rowspan=' {{:" + colHdr + "_rspan}}' colspan=' {{:" + colHdr + "_cspan}}'>{{:" + colHdr + "_value}}</td>");
                $tr.append($tdCell);
                colgrp.append(col);
            }
            tbody.get(0).appendChild($tr.get(0));
            templates[this._id + "JSONTemplate" + index] = tbody.html();
            if(!skipRowHdr)
                templates[this._id + "JSONRowHdrTemplate" + index] = "<tr class='{{:wrap}}' style='height:{{:height}}px' idx='{{:rowidx}}'><td class='{{:class}}'>{{:rowidx + 1}}</td></tr>";
            $.templates(templates);
            this._sheets[index]._contentColGroup = colgrp.get(0);
        },

        _renderData: function (rangeData, sheetIdx) {
            var i, j, len, length, rowCount, rowIdx, colIdx, formattedVal, rowCount, newObj, colHdr, cls, val, data, eCol = 0, viewData = [],
                sheetIdx = this._getSheetIndex(sheetIdx) , sheet = this.getSheet(sheetIdx), colCount = sheet.colCount, fltrStr, isVisCmnt,
                dataContainer = this._dataContainer.sheets[sheetIdx], sharedData = this._dataContainer.sharedData, rSpan, cSpan, mSpan,
                hashCode = this._dataContainer.hashCode, cFormat;
            if (rangeData) {                
                data = rangeData.data;
                rowCount = data.length / colCount;
                for (i = 0; i < rowCount; i++) {
                    newObj = {};
                    rowIdx = rangeData.rowIdx[i];
                    colIdx = 0;
                    for (j = eCol; j < (eCol + colCount) ; j++) {
                        colHdr = this._generateHeaderText(sheetIdx, colIdx + 1);
                        colIdx++;
                        cls = colHdr + "_className";
                        val = colHdr + "_value";
                        rSpan = colHdr + "_rspan";
                        cSpan = colHdr + "_cspan";
                        newObj[cls] = "e-rowcell e-overflow ";
                        if (this.inRange(sheet.selectedRange, rowIdx, colIdx - 1)) {
                            newObj[cls] += "e-selected ";
                            if(sheet._activeCell.rowIndex === rowIdx && sheet._activeCell.colIndex === colIdx - 1)
                                newObj[cls] += "e-activecell ";
                        }
                        if ("format" in data[j])
                            newObj[cls] += data[j].format + " ";
                        if ("border" in data[j])
                            newObj[cls] += data[j].border + " ";
                        if (["text", "general"].indexOf(data[j].type) !== -1) {
                            formattedVal = data[j].value2;
                            if (this._isDateTime(formattedVal))
                                formattedVal = this._dateToInt(formattedVal);
                            this.isNumber(formattedVal) && (newObj[cls] += "e-ralign ");
                        }
                        if ("isMHide" in data[j])
                            newObj[cls] += "e-mc-hide ";
                        if ("cFormatRule" in data[j]) {
                            cFormat = data[j].cFormatRule[0].split("_");
                            if(cFormat[7])
                                newObj[cls] += "e-" + cFormat[4] + " ";
                        }
                        if ("wrap" in data[j]) {
                            newObj[cls] += "e-sswraptext ";
                        }
                        if ("merge" in data[j]) {
                            mSpan = data[j].merge.mSpan.split(":");
                            newObj[rSpan] = mSpan[0];
                            newObj[cSpan] = mSpan[1];
                        }
                        newObj[val] = data[j].value2 ? data[j].value2 : "";
                        if ("hyperlink" in data[j]) {
                            if (!data[j].hyperlink.webAddr)
                                newObj[val] = String.format("<a href=\{0}\ class='e-hyperlinks'></a>", "#")
                            else
                                newObj[val] = String.format("<a href=\{0}\ class='e-hyperlinks' targrt='_blank'>\{1}\</a>", data[j].hyperlink.webAddr, data[j].value2);
                        }
                        if ("comment" in data[j]) {
                            newObj[cls] += "e-commentcell ";
                            isVisCmnt = data[j].comment.isVisible ? "visible" : "none";
                            newObj[val] += String.format("<span class='e-comment'><span class='e-celltoparrow'></span><textarea class='e-comment-txtarea' style='display:\{0}\;'>\{1}\</textarea></span>", isVisCmnt, data[j].comment.value);
                        }
                        if ("rule" in data[j])
                            newObj[cls] += data[j].rule.isApply ? "e-hlcell" : "";
                        if ("isFilterHeader" in data[j]) {
                            fltrStr = 'e-icon e-spanfilter e-filterspan ' + this.XLFilter._filterColl[i][j];
                            newObj[val] += String.format("<span class=\"{0}\"></span>", fltrStr);
                        }
                    }
                    newObj["rowidx"] = rangeData.rowIdx[i];
                    newObj["height"] = sheet.rowsHeightCollection[rangeData.rowIdx[i]] < this.model.rowHeight ? this.model.rowHeight : sheet.rowsHeightCollection[rangeData.rowIdx[i]];
                    eCol = j;
                    viewData.push(newObj);
                }
            }
            else {
                rowCount = this.model.scrollSettings.allowVirtualScrolling ? sheet._virtualBlockRowCnt : sheet.rowCount;
                if (sheet.dataSource)
                    viewData = sheet.dataSource;
                else {
                    for (i = 0, len = rowCount; i < len; i++) {
                        newObj = {};
                        for (j = 0, length = sheet.colCount; j < length; j++) {
                            colHdr = this._generateHeaderText(sheetIdx, j + 1);
                            cls = colHdr + "_className";
                            newObj[cls] = "e-rowcell e-overflow";
                            newObj["rowidx"] = i;
                            newObj["height"] = sheet.rowsHeightCollection[i] < this.model.rowHeight ? this.model.rowHeight : sheet.rowsHeightCollection[i];
                        }
                        viewData.push(newObj);
                    }
                }
            }
            sheet.dataSource = viewData;
            return $.render[this._id + "JSONTemplate" + sheetIdx](viewData);
        },

        _refreshViewport: function (top, sheetIdx) {
            var rangeData, rowCnt, prevIdx = 0, sheet = this.getSheet(sheetIdx), idx = this.XLShape._getCellIndexFromOffset(top, 0);
            sheet._virtualContBlockIdx = this._getVirtualBlockIdx(sheet, idx.rowIndex);
            currIdx = (sheet._virtualContBlockIdx - 1) * sheet._virtualBlockRowCnt;
            nextIdx = sheet._virtualContBlockIdx * sheet._virtualBlockRowCnt;
            sheet._virtualContTBodyOffset.top = this._getRowOffsetTop(sheet, currIdx);
            if (sheet._virtualBlockCnt === 1) {
                sheet._virtualTopTBodyOffset.top = sheet._virtualTopTBodyOffset.bottom = sheet._virtualTopTBodyHgt = 0;
                sheet._virtualBottomTBodyOffset.top = sheet._virtualBottomTBodyOffset.bottom = sheet._virtualBottomTBodyHgt = 0;
                sheet._virtualContTBodyOffset.bottom = this._getRowOffsetBottom(nextIdx - 1, sheetIdx);
                rowCnt = sheet._virtualBlockRowCnt;
            }
            else if (sheet._virtualContBlockIdx === 1) {
                sheet._virtualTopTBodyOffset.top = sheet._virtualTopTBodyOffset.bottom = sheet._virtualTopTBodyHgt = 0;
                sheet._virtualContTBodyOffset.bottom = sheet._virtualBottomTBodyOffset.top = this._getRowOffsetTop(sheet, nextIdx);
                sheet._virtualBottomTBodyOffset.bottom = this._getRowOffsetBottom(nextIdx + (sheet._virtualBlockRowCnt - 1), sheetIdx);
                sheet._virtualBottomTBodyHgt = sheet._virtualTBodyHgt - ((sheet._virtualContTBodyOffset.bottom - sheet._virtualContTBodyOffset.top) + (sheet._virtualBottomTBodyOffset.bottom - sheet._virtualBottomTBodyOffset.top));
                rowCnt = sheet._virtualBlockRowCnt * 2
                sheet._isVirtualTopReached = true;
            }
            else if (sheet._virtualContBlockIdx === sheet._virtualBlockCnt) {
                prevIdx = (sheet._virtualContBlockIdx - 2) * sheet._virtualBlockRowCnt;
                sheet._virtualBottomTBodyOffset.top = sheet._virtualBottomTBodyOffset.bottom = sheet._virtualBottomTBodyHgt = 0;
                sheet._virtualContTBodyOffset.bottom = this._getRowOffsetBottom(currIdx + (sheet._virtualBlockRowCnt - 1), sheetIdx);
                sheet._virtualTopTBodyOffset.top = this._getRowOffsetTop(sheet, prevIdx);
                sheet._virtualTopTBodyOffset.bottom = sheet._virtualContTBodyOffset.top;
                sheet._virtualTopTBodyHgt = sheet._virtualTBodyHgt - ((sheet._virtualContTBodyOffset.bottom - sheet._virtualContTBodyOffset.top) + (sheet._virtualTopTBodyOffset.bottom - sheet._virtualTopTBodyOffset.top));
                rowCnt = sheet._virtualBlockRowCnt * 2;
                sheet._isVirtualEndReached = true;
            }
            else {
                prevIdx = currIdx - sheet._virtualBlockRowCnt;
                sheet._virtualTopTBodyOffset.top = this._getRowOffsetTop(sheet, prevIdx);
                sheet._virtualTopTBodyOffset.bottom = sheet._virtualContTBodyOffset.top = this._getRowOffsetTop(sheet, currIdx);
                sheet._virtualContTBodyOffset.bottom = sheet._virtualBottomTBodyOffset.top = this._getRowOffsetTop(sheet, nextIdx);
                sheet._virtualBottomTBodyOffset.bottom = this._getRowOffsetBottom(nextIdx + (sheet._virtualBlockRowCnt - 1), sheetIdx);
                sheet._virtualTopTBodyHgt = top - ((sheet._virtualTopTBodyOffset.bottom - sheet._virtualTopTBodyOffset.top) + (top - sheet._virtualContTBodyOffset.top));
                sheet._virtualBottomTBodyHgt = sheet._virtualTBodyHgt - (sheet._virtualTopTBodyHgt + (sheet._virtualTopTBodyOffset.bottom - sheet._virtualTopTBodyOffset.top) + (sheet._virtualContTBodyOffset.bottom - sheet._virtualContTBodyOffset.top) + (sheet._virtualBottomTBodyOffset.bottom - sheet._virtualBottomTBodyOffset.top));
                rowCnt = sheet._virtualBlockRowCnt * 3;
                sheet._isVirtualTopReached = sheet._isVirtualEndReached = false;
            }
            rangeData = this.getRangeData(null, null, ["value2", "comment", "hyperlink", "format", "border", "type", "isFilterHide", "isRHide", "isFHide", "isFilterHeader", "filterState", "isMHide", "mSpan", "hRow", "isLocked"], sheetIdx, false, false, true, prevIdx, rowCnt, true);
            this._getRowHdrTBody(sheetIdx).html(this._renderRowHdr(rangeData.rowIdx, sheetIdx));
            this._getContTBody(sheetIdx).html(this._renderData(rangeData, sheetIdx));
            this._refreshTBodiesHgt(sheetIdx);
            sheet._virtualRowIdx = rangeData.rowIdx;
        },

        _getVirtualBlockIdx: function (sheet, rowIdx) {
            var i = 1;
            while (i <= sheet._virtualBlockCnt) {
                if (rowIdx < (i * sheet._virtualBlockRowCnt))
                    return i;
                i++;
            }
        },

        _getRowOffsetTop: function (sheet, rowIdx) {
            return sheet._rowHeightCollection[rowIdx].value;
        },

        _getRowOffsetBottom: function (rowIdx, sheetIdx) {
            return this.getSheet(sheetIdx)._rowHeightCollection[rowIdx].value + this.getSheet(sheetIdx).rowsHeightCollection[rowIdx];
        },

        _getCellIdx: function (cell) {
            return { rowIndex: Number(cell.parentNode.attributes.getNamedItem("idx").value), colIndex: cell.cellIndex };
        },

        _removeRange: function (idx, cnt) {
            var sheetIdx = this.getActiveSheetIndex(), rowHdrTBody = this._getRowHdrTBody(sheetIdx), rowHdrRows = $(this._getRowHdrTBody(sheetIdx)[0].rows),
                contTBody = this._getContTBody(sheetIdx), contRows = $(this._getContTBody(sheetIdx)[0].rows);
            rowHdrRows.splice(idx, cnt);
            contRows.splice(idx, cnt);
            contTBody.html(contRows);
            rowHdrTBody.html(rowHdrRows);
        },

        _refreshTBodiesHgt: function (sheetIdx) {
            var px = "px", sheetIdx = this._getSheetIndex(sheetIdx), sheet = this.getSheet(sheetIdx), hdrTBody = this._getJSSheetRowHeaderContent(sheetIdx)[0].getElementsByTagName("tbody"),
                contTBody = this._getContent(sheetIdx)[0].getElementsByTagName("tbody");
            hdrTBody[0].style.height = contTBody[0].style.height = sheet._virtualTopTBodyHgt + px;
            hdrTBody[2].style.height = contTBody[2].style.height = sheet._virtualBottomTBodyHgt + px;
        },

        _renderTD: function (className, styles, innerHtml) {
            return String.format("<td class='{0}' style='{1}' >{2}</td>", className, styles, innerHtml);
        },

        _renderDIV: function (className, innerHtml, title) {
            return title ? String.format("<div class='{0}' title='{2}'>{1}</div>", className, innerHtml, title) : String.format("<div class='{0}'>{1}</div>", className, innerHtml);
        },

        _renderTR: function (tdData, i, sheetIdx) {
            var trHeight = this._sheetCopy.isSheetCopy ? this.getSheet(sheetIdx).rowsHeightCollection[i] : this.model.rowHeight,
                height = "height:" + trHeight + "px";
            return String.format("<tr style='{0}'>" + tdData + "</tr>", height);
        },

        _renderFooter: function () {
            var div = ej.buildTag('div.e-spreadsheetfooter'),
            table = ej.buildTag("table.e-table", "", {}, {}),
            tbody = $(document.createElement("tbody")),
            tr = $(document.createElement("tr")),
            //Pager
            td = $(document.createElement("td"));
            td.append(this._renderSpreadSheetPager());
            //AddSheet
            td.find(".e-pagercontainer").append(ej.buildTag("div.e-new e-icon e-default", "", null, { title: this._getLocStr("NewSheet") }));
            tr.append(td);
            tbody.append(tr);
            table.append(tbody);
            div.append(table);
            this.getMainPanel().append(div);
            div.width(this.model.scrollSettings.width);
            div.height(this.model.pageSettings.pagerHeight);
            this.setFooter(div);
            this._refreshSpreadSheetPager();
        },

        //Pager
        gotoPage: function (sheetIdx, newSheet) {
            var insertIndex, modelSheet = this.model.sheets, pagerObj = this.getPager(), opt = "option", chkd = "checked",
                sheet = modelSheet[sheetIdx], args = { reqType: "gotoSheet", gotoIndex: sheetIdx, newSheet: newSheet };
            if (this._trigActionBegin(args))
                return;
            if ((!newSheet && sheetIdx > this.model.sheetCount) || sheetIdx < 1)
                return;
            this.setActiveSheetIndex(sheetIdx);
            this.element.find(".e-sheet").hide();
            if(newSheet)
                this._addSheet(sheetIdx);            
            if (this._isSheetInsert) {
                insertIndex = pagerObj.ejPager("option", "currentPage");
                this._swapSheetDtCntr(modelSheet.length - 1, insertIndex);
                this.model.activeSheetIndex = sheetIdx = insertIndex;
                this._isSheetInsert = false;
            }
            if (this.model.scrollSettings.allowVirtualScrolling && !sheet._isLoaded)
                this._renderSheet(sheetIdx);                            
            else
                this.element.find("#" + this._id + "_" + sheet._sheetName.value).show();
            if (!sheet._importRendered && this.model.enableContextMenu)
                 this.XLCMenu.hideCMenu();
            this._refreshSpreadSheetPager();
            if (sheet._importRendered) {
                this.showWaitingPopUp();
                this._updateImportedSheets(sheetIdx);
                this.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
                if(sheet._frozenRows || sheet._frozenColumns)
                    this._setFreezePane(sheet._frozenRows, sheet._frozenColumns);
                this.hideWaitingPopUp();
            }
            this._initializeSelection();
            if (this.getSheetElement(pageIndex).find(".e-ss-activeimg").length)
                this.XLSelection._cleanUp(true);
            modelSheet[pageIndex]._importRendered = false;
            if (this.model.showRibbon) {
                this._updateRibbonTab(this.getActiveCellElem(sheetIdx)[0]);
                if (!this.importSheet) {
                    if (this.model.allowLockCell) {
                        modelSheet[pageIndex]._isSheetProtected ? this.XLRibbon._disableRibbonIcons() : this.XLRibbon._enableRibbonIcons();
                        this.XLRibbon._updateRibbonIcons();
                    }
                    if (this.model.allowComments)
                        this.XLComment._updateCmntRibIcons();
                }


                $("#" + this._id + "_Ribbon_Vheading").ejCheckBox(opt, chkd, sheet.showHeader);
                $("#" + this._id + "_Ribbon_Vgridlines").ejCheckBox(opt, chkd, sheet.showGridlines);
            }
            args = { activeSheet: sheetIdx, reqType: "gotoSheet" };
            this._trigActionComplete(args);
        },

        insertSheet: function () {
            if (!this.model.allowInsert)
                return;
            var pager = this.getPager(), insertPage = pager.ejPager("option", "totalRecordsCount") + 1;
            pager.ejPager("option", { totalRecordsCount: insertPage });
            this._isSheetInsert = true;
            this.gotoPage(insertPage, true);
            this.setSheetFocus();
        },

        _renameSheet: function () {
            var trgt = $(".e-currentitem"), input = ej.buildTag("input#" + this._id + "_Sheet_RenamePanel", "", { width: trgt.width() + 5 + "px", padding: 0 }, { type: "input", value: trgt.text(), maxlength: "31" });
            trgt.hide();
            input.insertAfter(trgt).select().focus();
            this._isSheetRename = true;
            if(this.model.showRibbon)
                this.XLRibbon._disableRibbonIcons();
        },

        deleteSheet: function (index) {
            var actShtIdx = this.getActiveSheetIndex(), sheetCount = this._getSheetNames(true).length;
            index = index || actShtIdx;
            if (sheetCount > 1) {
                if (index !== actShtIdx)
                    this.gotoPage(index, false);
                if (this.getObjectLength(this._dataContainer.sheets[index]))
                    this._showAlertDlg("", "SheetDeleteAlert", "sheetDeleteActn", 470);
                else
                    this._removeSheet(index);
            }
            else
                this._showAlertDlg("Alert", "SheetDeleteErrorAlert", "", 430);                            
        },

        hideSheet: function (shtInfo) {
            var visibeSht = this._getSheetNames(true), shtNames = this._getSheetNames(),
                idx = $.isNumeric(shtInfo) ? shtInfo : this._getSheetIndexByName(shtInfo) || this.getActiveSheetIndex();
            if (visibeSht.length > 1) {
                shtNames[idx - 1].isVisible = false;
                this.gotoPage(this._getNxtVisibleSheet(idx, null, visibeSht));
            }
            else
                this._showAlertDlg("Alert", "SheetDeleteErrorAlert", "", 377);
        },

        unhideSheet: function (shtInfo) {
            var shtNames = this._getSheetNames(), idx = $.isNumeric(shtInfo) ? shtInfo : this._getSheetIndexByName(shtInfo);
            if (!shtNames[idx - 1].isVisible) {
                shtNames[idx - 1].isVisible = true;
                this.gotoPage(idx);
            }
        },

        _getNxtVisibleSheet: function (idx, isReverseDir, visibleSheets) {
            var shtNames = this._getSheetNames(), limit = shtNames.length, optr1 = "<=", optr2 = 1, visibeShts = this._getSheetNames(true),
                isLastSht = shtNames[idx - 1].value === (visibleSheets && visibleSheets[visibleSheets.length - 1].value || visibeShts[visibeShts.length - 1].value);
            if (isReverseDir || isLastSht) {
                limit = 0;
                optr1 = ">=";
                optr2 = -1;
            }
            while (this.operators[optr1](idx, limit)) {
                if (shtNames[idx - 1].isVisible)
                    return idx;
                idx = idx + optr2;
            }
        },

        _renderSpreadSheetPager: function () {
            var pagerModel = {}, div = $(document.createElement("div"));
            this.model.pageSettings.click = this._spreadSheetPagerClickHandler;
            this.model.pageSettings.totalRecordsCount = this.model.sheetCount;
            this.model.pageSettings.locale = this.model.locale
            $.extend(pagerModel, this.model.pageSettings);
            pagerModel.currentPage = this.model.activeSheetIndex;
            pagerModel.masterObject = this;
            pagerModel.pageSize = 1;
            this.setPager(div);
            div.ejPager(pagerModel);
            div.ejPager("refreshPager");
            inst = div.ejPager('instance');
            inst._$first[0].title = this._getLocStr("Firstsheet");
            inst._$last[0].title = this._getLocStr("Lastsheet");
            inst._$next[0].title = this._getLocStr("Nextsheet");
            inst._$prev[0].title = this._getLocStr("Prevsheet");
            div.find(".e-parentmsgbar").remove();
            div.find("a.e-PP").appendTo(div.find(".e-pagercontainer"));
            div.find("#NumericContainer").appendTo(div.find(".e-pagercontainer"));
            div.find("a.e-NP").appendTo(div.find(".e-pagercontainer"));
            return div;
        },

        _getPageCount: function () {
            var i, len, pager = this.getPager(), width = 0, currentItem = pager.find('.e-currentitem'),
                totalwidth = pager.width() - (pager.find('.e-pagercontainer').width() - pager.find('#NumericContainer').width()),
                visibleLinks = pager.find('.e-link.e-numericitem:visible').not(".e-PP, .e-NP"), sheetNames = this._getSheetNames(),
                startPgIdx = $(visibleLinks[0]).data("index"), cavsElem = ej.buildTag('canvas'), cavsTxt = cavsElem[0].getContext("2d");
            cavsTxt.font = currentItem.css('font-size') + " " + currentItem.css("font-family");
            for (i = 0, len = this.model.pageSettings.pageCount; i < len; i++) {
                if (this.isUndefined(sheetNames[(startPgIdx - 1)]))
                    continue;
                width += cavsTxt.measureText(sheetNames[startPgIdx - 1].text).width + 25;
                if (totalwidth <= width)
                    return i;
                startPgIdx++;
            }
            return i;
        },

        _spreadSheetPagerClickHandler: function (sender) {
            var div, newPage, addnew = false, masterObj = this.model.masterObject, args, trgt = $(sender.event.target);
            if (((masterObj.model.allowComments && masterObj.XLComment._isCommentEdit) || masterObj.XLEdit._isEdit) && (sender.event && (trgt.hasClass("e-pager") || trgt.hasClass("e-new")) || (masterObj.element.find(".e-field-validation-error").length)))
                return;
            else {
                masterObj.model.allowComments &&  masterObj.XLComment._isCommentEdit && masterObj.XLComment._updateCurCmntVal();
                masterObj.XLEdit._isEdit && masterObj.XLEdit.saveCell();
            }
            if (!ej.isNullOrUndefined(sender.event) && (trgt.hasClass("e-prevpage") || trgt.hasClass("e-nextpage") || trgt.hasClass("e-firstpage") || trgt.hasClass("e-lastpage")))
                sender.currentPage = masterObj._getNxtVisibleSheet(sender.currentPage, (trgt.hasClass("e-prevpage") || trgt.hasClass("e-lastpage")));
            args = {
                activeSheet: masterObj.getActiveSheetIndex(),
                gotoSheet: sender.currentPage,
                newSheet: sender.event && trgt.hasClass("e-new")
            };
      		 if (args.newSheet) {
                div = masterObj.getPager();
                newPage = div.ejPager("option", "totalRecordsCount") + 1;
                args.gotoSheet = newPage;
            }
            else
                args.gotoSheet = sender.currentPage;
            sender.currentPage = args.gotoSheet;
            if ((!ej.isNullOrUndefined(sender.event) && sender.event.target.id.indexOf("_Sheet_RenamePanel") > -1) || (masterObj._isSheetRename && !masterObj._updateSheetNames(null, false)) || (this._prevPageNo === sender.currentPage))
                return;
            else if (args.newSheet) {               
                div.ejPager("option", { totalRecordsCount: newPage, currentPage: newPage });
                sender.currentPage = newPage;
                addnew = true;
            }
			masterObj.model.allowFiltering && masterObj._filterClose();			
            masterObj.gotoPage(sender.currentPage, addnew);		
			if (masterObj._trigger("pagerClick", args))
                return;
			if(masterObj.model.scrollSettings.isResponsive)  
				masterObj._refreshSheetonResize();
            !$(sender.event.target).hasClass("e-pager") && masterObj._completeAction({ reqType: "paging", sheetIndex: args.activeSheet, nSheetIndex: sender.currentPage });
            return false;
        },

        _pagerdblClickHandler: function (sender) {
            if (this.XLEdit && this.XLEdit._isEdit)
                if (this.getActiveCellElem().find(".e-error").length)
                    return;
            !this._isSheetRename && this._renameSheet();
        },

        _createTemplateElement: function (index, column) {
            var scriptElement = document.createElement("script");
            scriptElement.id = (this._id + index + column["field"] + $.inArray(column, this.getSheet(index).columns) + "_Template").split(" ").join("");
            scriptElement.type = "text/x-jsrender";
            scriptElement.text = column["templateContent"] ? column["templateContent"] : $(column["templateID"]).html();
            $("body").append(scriptElement);
            $(column["templateID"]).hide();
            return scriptElement;
        },

        _refreshSpreadSheetPager: function () {
            var pager = this.getPager(), model = {};
            if (this.getPager() != null && !this.importSheet)
                model.currentPage = this.getActiveSheetIndex();
            else {
                model.currentPage = this.getActiveSheetIndex();
                model.totalRecordsCount = this.model.sheetCount;
            }
            model.pageCount = this._getPageCount();
            pager.ejPager("option", model).ejPager("refreshPager");
            this._updatePageContainer(pager, model.currentPage);
        },

        _updatePageContainer: function (pagerContainer, currentPage) {
            var pagerObj = pagerContainer.data("ejPager"), pager = pagerContainer.find('.e-link.e-numericitem:visible').not(".e-PP, .e-NP"), $sheetNames = this.model.sheets, isFirst, isLast, i, len;
            for (i = 0, len = pager.length; i < len; i++) {
                $(pager[i]).text($sheetNames[$(pager[i]).data("index")]._sheetName.text);
                if ($sheetNames[$(pager[i]).data("index")]._sheetName.isVisible)
                    isLast = $(pager[i]).data("index");
                else
                    $(pager[i]).hide();
                !isFirst && $sheetNames[$(pager[i]).data("index")]._sheetName.isVisible && (isFirst = $(pager[i]).data("index"));
            }
            pagerObj._$prev.removeClass("e-prevpagedisabled e-disable").addClass("e-prevpage e-default");
            if (isFirst === currentPage) {
                pagerObj._$first.addClass("e-firstpagedisabled e-disable").removeClass("e-firstpage e-default");
                pagerObj._$prev.addClass("e-prevpagedisabled e-disable").removeClass("e-prevpage e-default");
            }
            if (isLast === currentPage) {
                pagerObj._$last.addClass("e-lastpagedisabled e-disable").removeClass("e-lastpage e-default");
                pagerObj._$next.addClass("e-nextpagedisabled e-disable").removeClass("e-nextpage e-default");
            }
        },

        _updateSheetNames: function (updatedValue, isCancel) {
            var pagerContainer = this.getPager(), i, len, $sheet = pagerContainer.find(".e-currentitem"), modelSheet = this.model.sheets, $input = $("#" + this._id + "_Sheet_RenamePanel"), isValidName = true;
            ej.isNullOrUndefined(updatedValue) && (updatedValue = $input.val().trim());

            //Cancel rename action
            if (isCancel) {
                $input.remove() && $sheet.show();
                this.model.showRibbon && this.XLRibbon._enableRibbonIcons();
                this._isSheetRename = false;
                return;
            }
            if (updatedValue.match(new RegExp(".*[\\[\\]\\*\\\\\/\\^].*")))
                updatedValue = false;

            if (!updatedValue) {
                this._showAlertDlg("Alert", "SheetRenameEmptyAlert", "sheetRenameActn", 500);
                return false;
            }

            //Check isValid name
            for (i = 1, len = modelSheet.length; i < len; i++) {
                $sheet.text() !== modelSheet[i]._sheetName.text && updatedValue === modelSheet[i]._sheetName.text && (isValidName = false);
            }

            if (isValidName) {
                (modelSheet[$sheet.data("index")]._sheetName.text = updatedValue) && $input.remove() && $sheet.show() && (this._isSheetRename = false);
                this._refreshSpreadSheetPager();
                this.model.showRibbon && this.XLRibbon._enableRibbonIcons();
                return true;
            }
            else {
                this._showAlertDlg("Alert", "SheetRenameAlert", "sheetRenameActn", 325);
                return false;
            }
        },

        _getSheetIndexByName: function (sheetName) {
            var sheetNameCol = this._getSheetNames(), i, len;
            for (i = 0, len = sheetNameCol.length; i < len; i++) {
                if (sheetNameCol[i].text === sheetName)
                    return (i + 1);
            }
            return false;
        },

        _removeSheet: function (index) {
            var sheetCount = this.model.sheetCount, shtDtCntr = this._dataContainer.sheets,i,len;
            if (sheetCount > 1) {
                this._getAutoFillOptElem().appendTo(this.element);
                this.element.find("#" + this._id + "_" + this.model.sheets[index]._sheetName.value).remove();
                this.setActiveSheetIndex((sheetCount === this.getActiveSheetIndex()) ? (index - 1) : index);
                this._swapSheetDtCntr(index, sheetCount);
                sheetCount in shtDtCntr && delete shtDtCntr[sheetCount];
                this._sheets.splice(sheetCount, 1);
                this.model.sheets.splice(sheetCount, 1);
                sheetCount = --this.model.sheetCount;

                this.getPager().ejPager("option", { totalRecordsCount: sheetCount, pageCount: sheetCount });
                this.gotoPage(this.getActiveSheetIndex(), false);
                this._getAutoFillOptElem().appendTo(this._getContent(this.getActiveSheetIndex()).find(".e-content"));
                //Rebind sheet scroller event after delete sheets(index changed)
                this._rebindScrollEvnt();
		        for(i = 0, len = this._undoCollection.length; i < len; i++)
	     	    {
		            if (this._undoCollection[i] && this._undoCollection[i].sheetIndex == index)
		                this._undoCollection.splice(i, 1);
		        }
		        for(i = 0, len = this._redoCollection.length; i < len; i++)
		        {
		            if(this._redoCollection[i] && this._redoCollection[i].sheetIndex == index)
			            this._redoCollection.splice(i, 1);
		        }
            }
        },

        _rebindScrollEvnt: function () {
            for (var i = 1; i <= this.model.sheetCount; i++) {
                this._getContent(i).find(".e-content").off("scroll");
                this.element.find("#" + this._id + "_" + this.model.sheets[i]._sheetName.value).find(".e-headercontent").off("scroll");
                this._refreshScrollerEvent(i);
            }
        },

        _getSheetNames: function (visible) {
            var sheetNames = [], modelSheet = this.model.sheets, i, len;
            for (i = 1, len = modelSheet.length; i < len; i++) {
                if (!this.isUndefined(visible) && ((visible && !modelSheet[i]._sheetName.isVisible) || (!visible && modelSheet[i]._sheetName.isVisible)))
                    continue;
                sheetNames.push(modelSheet[i]._sheetName);
            }
            return sheetNames;
        },

        _initMoveCopyDlg: function () {
            var dlgId = this._id + "_moveCopyDlg", $dlg, $content, $okBtn, $cancelBtn, $btndiv, $listUl, $chkbox, $listCtn, $btnctnr;
            $dlg = ej.buildTag("div#" + dlgId, "", { display: "none" });
            $content = ej.buildTag("div#" + dlgId + "_sheetContent.e-dlgctndiv e-dlg-fields", ej.buildTag("label.e-dlg-fields", this._getLocStr("Beforesheet")));
            $listCtn = ej.buildTag("div.e-dlg-fields"), $listUl = ej.buildTag("ul#" + dlgId + "_list");
            $content.append($listCtn.append($listUl));
            $chkbox = ej.buildTag("input#" + dlgId + "_createcopy");
            $content.append($chkbox);
            $chkbox.ejCheckBox({ cssClass: "e-dlg-fields" });
            $chkbox.parent().append(ej.buildTag("label", this._getLocStr("CreateaCopy"), { "margin-left": 6 }));
            $dlg.append($content);
            $btndiv = ej.buildTag("div#" + dlgId + "_btndiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $okBtn = ej.buildTag("input#" + dlgId + "_okbtn");
            $cancelBtn = ej.buildTag("input#" + dlgId + "_cnlbtn");
            $okBtn.ejButton({ text: this._getLocStr("Ok"), showRoundedCorner: true, width: "25%", click: ej.proxy(this._moveCopyBtnClick, this, "ok") });
            $cancelBtn.ejButton({ text: this._getLocStr("Cancel"), click: ej.proxy(this._moveCopyBtnClick, this, "cancel"), showRoundedCorner: true, width: "25%" });
            $btndiv.append($btnctnr.append($okBtn, $cancelBtn));
            $dlg.append($btndiv);
            this.element.append($dlg);
            $dlg.ejDialog({ showOnInit: false, enableModal: true, enableResize: false, allowKeyboardNavigation: false, title: this._getLocStr("MoveorCopy"), width: 280, height: "auto", cssClass: "e-ss-dialog e-"+this._id+"-dlg" });
            $listUl.ejListBox({
                dataSource: this._getSheetNames(),
                height: "197px",
                width: "250px",
                fields: { text: "text", value: "value" }
            });
        },

        _renderMoveCopyDlg: function () {
            var $dlg = $("#" + this._id + "_moveCopyDlg"), lbObj = $("#" + this._id + "_moveCopyDlg_list").data("ejListBox"), lbDataSrc = this._getSheetNames();
            lbDataSrc.push({ text: this._getLocStr('MoveToEnd'), value: this._getLocStr('MoveToEnd') });
            $dlg.ejDialog("open");
            lbObj.model.dataSource = lbDataSrc;
            lbObj.option("fields", { dataSource: lbDataSrc });
            lbObj.unSelectAll();
            lbObj.selectItemByIndex(0);
        },

        _initUnhideDlg: function () {
            var dlgId = this._id + "_UnhideDlg", $dlg, $content, $okBtn, $cancelBtn, $btndiv, $listUl, $btnctnr;
            $dlg = ej.buildTag("div#" + dlgId, "", { display: "none" });
            $content = ej.buildTag("div#" + dlgId + "_sheetContent.e-dlgctndiv e-dlg-fields", ej.buildTag("label.e-dlg-fields", this._getLocStr("UnhideDlgText")));
            $listUl = ej.buildTag("ul#" + dlgId + "_list");
            $content.append($listUl);
            $dlg.append($content);
            $btndiv = ej.buildTag("div#" + dlgId + "_btndiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $okBtn = ej.buildTag("input#" + dlgId + "_okbtn");
            $cancelBtn = ej.buildTag("input#" + dlgId + "_cnlbtn");
            $okBtn.ejButton({ text: this._getLocStr("Ok"), showRoundedCorner: true, width: "25%", click: ej.proxy(this._unhideBtnClick, this, "ok") });
            $cancelBtn.ejButton({ text: this._getLocStr("Cancel"), click: ej.proxy(this._unhideBtnClick, this, "cancel"), showRoundedCorner: true, width: "25%" });
            $btndiv.append($btnctnr.append($okBtn, $cancelBtn));
            $dlg.append($btndiv);
            this.element.append($dlg);
            $dlg.ejDialog({ showOnInit: false, enableModal: true, enableResize: false, allowKeyboardNavigation: false, title: this._getLocStr("UnhideSheet"), width: 280, height: "auto", cssClass: "e-ss-dialog e-"+this._id+"-dlg" });
            $listUl.ejListBox({
                dataSource: this._getSheetNames(),
                height: "197px",
                width: "250px",
                fields: { text: "text", value: "value" }
            });
        },

        _renderUnhideDlg: function () {
            var dlg = $("#" + this._id + "_UnhideDlg"), lbObj = $("#" + this._id + "_UnhideDlg_list").data("ejListBox"), lbDataSrc = this._getSheetNames(false);
            if (lbDataSrc.length > 0) {
                dlg.ejDialog("open");
                lbObj.model.dataSource = lbDataSrc;
                lbObj.option("fields", { dataSource: lbDataSrc });
                lbObj.unselectAll();
                lbObj.selectItemByIndex(0);
            }
        },

        _moveCopyBtnClick: function (action, e) {
            switch (action) {
                case "ok":
                    var spliceIndex, i, len, cpySheetName, isValid = false, sheetNameCount = 2, index, selectedItem = $(this.element).find(".e-currentitem"), lbObj = $("#" + this._id + "_moveCopyDlg_list").data("ejListBox"), checkObj = $("#" + this._id + "_moveCopyDlg_createcopy").data("ejCheckBox"), sText = lbObj.getSelectedItems()[0].text, mainIndex = lbObj.getIndexByText(sText), modelSheet = this.model.sheets;

                    //Check is same sheet
                    if (!checkObj.isChecked() && (sText === selectedItem.text())) {
                        $("#" + this._id + "_moveCopyDlg").ejDialog("close");
                        this.setSheetFocus();
                        return;
                    }

                    //Getting splice index and sheet position index
                    for (i = 1; i < modelSheet.length; i++) {
                        (sText === modelSheet[i]._sheetName.text) && (spliceIndex = i);
                        (selectedItem.text() === modelSheet[i]._sheetName.text) && (index = i);
                    }

                    //Check is Copy sheet
                    if (checkObj.isChecked()) {
                        // Getting unique sheet name
                        if (selectedItem.text().match('.*[ ]+[(]+[0-9]+[)]$') === null) {
                            cpySheetName = selectedItem.text() + " (" + sheetNameCount + ")";
                        }
                        else {
                            if (selectedItem.text().lastIndexOf(' ') !== -1) {
                                sheetNameCount = parseInt(selectedItem.text().substring(selectedItem.text().lastIndexOf("(") + 1, selectedItem.text().lastIndexOf(")")));
                                cpySheetName = selectedItem.text().substring(0, selectedItem.text().lastIndexOf(" ")) + " (" + sheetNameCount + ")";
                            }
                        }
                        while (!isValid) {
                            cpySheetName = cpySheetName.substring(0, cpySheetName.lastIndexOf(" ")) + " (" + sheetNameCount + ")";
                            for (i = 1, len = modelSheet.length; i < len; i++) {
                                (cpySheetName === modelSheet[i]._sheetName.text) && sheetNameCount++;
                            }
                            (sheetNameCount === parseInt(cpySheetName.substring(cpySheetName.lastIndexOf("(") + 1, cpySheetName.lastIndexOf(")")))) && (isValid = true);
                        }

                        //Copy Sheet Process      
                        this._copySheetProcess(index, cpySheetName);

                        index = this.model.sheetCount;
                        ej.isNullOrUndefined(spliceIndex) && (spliceIndex = index);
                    }
                    else {
                        !ej.isNullOrUndefined(spliceIndex) && spliceIndex > index && (spliceIndex = spliceIndex - 1);
                    }

                    ej.isNullOrUndefined(spliceIndex) && (spliceIndex = mainIndex);

                    //Sheets[] updates
                    if (!checkObj.isChecked() && spliceIndex === index) {
                        $("#" + this._id + "_moveCopyDlg").ejDialog("close");
                        this.setSheetFocus();
                        break;
                    }

                    //Update DataContainer positions
                    this._swapSheetDtCntr(index, spliceIndex);

                    $("#" + this._id + "_moveCopyDlg").ejDialog("close");

                    spliceIndex <= index && (mainIndex = mainIndex + 1);
                    this.getPager().ejPager("option", { currentPage: mainIndex });
                    this.gotoPage(mainIndex, false);
                    this._rebindScrollEvnt();

                    //uncheck copy sheet checkbox
                    checkObj._unChecked();
                    checkObj._isChecked = false;

                    this.setSheetFocus();
                    break;
                case "cancel":
                    $("#" + this._id + "_moveCopyDlg").ejDialog("close");
                    break;
            }
        },

        _unhideBtnClick: function (action, e) {
            if (action === "ok") {
                var lbObj = $("#" + this._id + "_UnhideDlg_list").data("ejListBox");
                this.unhideSheet(lbObj.getSelectedItems()[0].text);
            }
            $("#" + this._id + "_UnhideDlg").ejDialog("close");

        },

        _swapSheetDtCntr: function (index, spliceIdx) {
            var sheetDtMngr = this._dataContainer.sheets, modelSheet = this.model.sheets, dtSheets = this._sheets, shtValue = this._getLocStr("Sheet"), backupDtCntr = [{}], newIdx, i, urLen, undoCol = [], redoCol = [],
                bckupModel = this._getSheetNames();
            for (var ind in sheetDtMngr) {
                if (sheetDtMngr.hasOwnProperty(ind))
                    backupDtCntr[parseInt(modelSheet[ind]._sheetName.value.replace(shtValue, ""))] = sheetDtMngr[ind];
            }

            modelSheet.splice(spliceIdx, 0, modelSheet.splice(index, 1)[0]);
            dtSheets.splice(spliceIdx, 0, dtSheets.splice(index, 1)[0]);
            
            for (var indx = 1, len = modelSheet.length; indx < len; indx++) {
                newIdx = parseInt(modelSheet[indx]._sheetName.value.replace(shtValue, ""));
                this._dataContainer.sheets[indx] = backupDtCntr[newIdx];

                // Swap appropriate undo & redo collection sheetIndex
                for (i = 0, urLen = this._undoCollection.length; i < urLen; i++) {
                    if (undoCol.indexOf(i) === -1 && parseInt(bckupModel[this._undoCollection[i].sheetIndex - 1].value.replace(shtValue, "")) === newIdx) {
                        this._undoCollection[i].sheetIndex = indx;
                        undoCol.push(i);
                    }
                }

                for (i = 0, urLen = this._redoCollection.length; i < urLen; i++) {
                    if (redoCol.indexOf(i) === -1 && parseInt(bckupModel[this._redoCollection[i].sheetIndex - 1].value.replace(shtValue, "")) === newIdx) {
                        this._redoCollection[i].sheetIndex = indx;
                        redoCol.push(i);
                    }
                }
            }
        },

        _copySheetProcess: function (index, cpySheetName) {
            var pager = this.getPager(), newPage = pager.ejPager("option", "totalRecordsCount") + 1, modelSheet = this.model.sheets, newSheetIdx;
            pager.ejPager("option", { totalRecordsCount: newPage, currentPage: newPage }), cellIdx;

            //Sheet settings Copy
            modelSheet.push(jQuery.extend(true, {}, modelSheet[index]));
            this._sheetCopy = { isSheetCopy: true, sheetContent: this._getContent(index).find(".e-table").clone(true).get(0) };

            newSheetIdx = this.model.sheetCount + 1;
            modelSheet[newSheetIdx]._sheetName.text = cpySheetName;
            modelSheet[newSheetIdx]._sheetName.value = this._getLocStr("Sheet") + this._newSIndex;
            this._newSIndex++;
            this.setActiveSheetIndex(newSheetIdx);

            //Copy Data Container
            this._dataContainer.sheets[newSheetIdx] = $.extend(true, {}, this._dataContainer.sheets[index]);
            this._sheets[newSheetIdx] = { _contentColGroup: null, _content: null, _sheetRowHeaderContent: null, _sheetHeader: null, _sheetRowHeader: null, _sheetContent: null, _Rows: null, _calcEngine: null };

            this._createSheets(newSheetIdx);
            this.model.sheetCount++;

            this._sheetCopy = { isSheetCopy: false, sheetContent: null };

            //Comments update
            var commentCells = this._getContent(newSheetIdx).find(".e-commentcell"), elem, i, len;
            for (i = 0, len = commentCells.length; i < len; i++) {
                elem = $(commentCells[i]);
                cellIdx = this._getCellIdx($(".e-commentcell")[0]);
                elem.removeClass("e-commentcell") && this.XLComment.setComment([cellIdx.rowIndex, cellIdx.colIndex, cellIdx.rowIndex, cellIdx.colIndex], null, false);
            }

            //Update activeCell - selection
            if(this.model.allowSelection)
				this.XLSelection.selectRange(modelSheet[newSheetIdx]._startCell, modelSheet[newSheetIdx]._endCell);
        },

        showActivationPanel: function (rangeName) {
            var rangeInfo, sindex = this.getActiveSheetIndex(), actpanel = this.getActivationPanel(), settings = this.model.scrollSettings;
            if (this.isUndefined(rangeName)){
                if (actpanel.is(':visible')) {
                    actpanel.hide();
                    this._setSheetWidthHeight(sindex, { width: settings.width, height: settings.height }, null, "resize");
                    this.getMainPanel().width(this.getMainPanel().width() + this.model.apWidth);
                }
            }
			else {
                rangeInfo = this.getSheet(sindex).rangeSettings[rangeName];
                if (rangeInfo.showPanel) {
                    if (this._trigger("beforePanelOpen", { activationPanel: actpanel, range: rangeInfo, sheetIndex: sindex }))
                        return;
                    this._setSheetWidthHeight(sindex, { width: (settings.width - this.model.apWidth), height: settings.height }, null, "resize");
                    actpanel.width(this.model.apWidth).height(this.getMainPanel().height());
					this.getMainPanel().width(settings.width - this.model.apWidth);
					actpanel.show();
                }
            }
        },

        _hideOtherControls: function () {
            $("#" + this._id + "_Ribbon_CElement").hide();
            $("#" + this._id + "_Ribbon_Paste").hide();
            $("#" + this._id + "_Ribbon_Border").hide();
            $("#" + this._id + "_Ribbon_Merge").hide();
            $("#" + this._id + "_CFormat").hide();
            $("#" + this._id + "_formatastable").hide();
            $("#" + this._id + "_cellstyles").hide();
            $("#" + this._id + "_Ribbon_SortFilter").hide();
            $("#" + this._id + "_Ribbon_Clear").hide();
            $("#" + this._id + "_Ribbon_ColumnChart").hide();
            $("#" + this._id + "_Ribbon_BarChart").hide();
            $("#" + this._id + "_Ribbon_Validation").hide();
            $("#" + this._id + "_Ribbon_PageSize").hide();
            $("#" + this._id + "_Ribbon_FPane").hide();
            $("#" + this._id + "_Ribbon_Ins").hide();
            $("#" + this._id + "_Ribbon_Del").hide();
            $("#" + this._id + "_Ribbon_FindRep").hide();
            $("#" + this._id + "_Ribbon_PastePic").hide();
            $("#" + this._id + "_Ribbon_PictureBorder").hide();
            $("#" + this._id + "_ctxtmenu").hide();
            $("#" + this._id + "_Ribbon_AutoSum").hide();
            $("#" + this._id + "_Ribbon_ResetPic").hide();
            $("#" + this._id + "_Ribbon_PictureBorder").hide();
            $("#" + this._id + "_Ribbon_StockChart").hide();
            $("#" + this._id + "_Ribbon_LineChart").hide();
            $("#" + this._id + "_Ribbon_AreaChart").hide();
            $("#" + this._id + "_Ribbon_PieChart").hide();
            $("#" + this._id + "_Ribbon_ScatterChart").hide();
			$("#" + this._id + "_Ribbon_color_Presets").hide();
 			$("#" + this._id + "_Ribbon_PictureColor_Presets").hide();
			$("#" + this._id + "_Ribbon_BorderColor_Presets").hide();
			$("#" + this._id + "_Ribbon_background-color_Presets").hide();
        },

        _addSheet: function (index) {
            this._sheetSettings(index);
            this._sheets[index] = { _contentColGroup: null, _content: null, _sheetRowHeaderContent: null, _sheetHeader: null, _sheetRowHeader: null, _sheetContent: null, _Rows: null };
            this._dataContainer.sheets[index] = {};
            this._createSheets(index);
            this._newSIndex++;
            this.model.sheetCount++;
        },

        addNewSheet: function () {
            this.element.find(".e-pager .e-new").click();
        },

        _createFocusElement: function () {
            this.setFocusTopElem($("<div class='e-focustop e-hide' ></div>"));
            this.setFocusRightElem($("<div class='e-focusright e-hide' ></div>"));
            this.setFocusBottomElem($("<div class='e-focusbottom e-hide' ></div>"));
            this.setFocusLeftElem($("<div class='e-focusleft e-hide' ></div>"));
        },

        _createAutoFillElement: function () {
            var html, sheetCont = this._getContent(this.getActiveSheetIndex()).find(".e-content");
            if (this.model.allowAutoFill) {
                html = $("<div class='e-autofill e-hide'></div>");
                sheetCont.append(html);
                this.setAutoFillElem(html);
            }
            if (this.model.autoFillSettings.showFillOptions) {
                this.element.append(ej.buildTag("ul.e-spreadsheet", "", { "width": "auto" }, { id: this._id + "_ctxtmenu" }));
                html = $("<div class='e-splitspan e-hide' title=" + this._getLocStr("AutoFillTitle") + "><button id=" + this._id + "_dragfilloptionbtn></button></div>");
                sheetCont.append(html);
                this._setAutoFillOptElem(html);
            }
            this._initializeSplitButton();
        },

        _initializeSplitButton: function () {
            $("#" + this._id + "_dragfilloptionbtn").ejSplitButton({
                size: "normal",
                contentType: "imageonly",
                targetID: this._id + "_ctxtmenu",
                prefixIcon: "e-icon e-ss-autofill",
                isSeparator: false,
                buttonMode: ej.ButtonMode.Dropdown,
                imagePosition: ej.ImagePosition.ImageLeft,
                arrowPosition: ej.ArrowPosition.Right,
                fields: { dataSource: this._fillOptionMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" },
                beforeOpen: $.proxy(this.model.allowAutoFill && this.XLDragFill._autoFillClick, this),
                itemSelected: $.proxy(this.model.allowAutoFill && this.XLDragFill._autoFillOptionClick, this),
                width:"42px",
                height: "24px",
                cssClass:"e-spreadsheet e-autofillbtn"
            });
        },

        // formula bar
        _renderFormulaBar: function () {
            var inputBox;
            this.element.append("<div id=" + this._id + "_formulabar class='e-formulabar' ><div id=" + this._id + "_indexbox class='e-indexbox' title='" + this._getLocStr("NameBox") + "'></div><div class='e-inputbox' ><input id=" + this._id + "_inputbox type='text' title='" + this._getLocStr("FormulaBar") + "'/></div></div>");
            this._setIndexBox($("#" + this._id + "_indexbox"));
            inputBox = $("#" + this._id + "_inputbox");
            this._setInputBox(inputBox);
            if(this.model.allowEditing)
                this.XLEdit._renderAutoComplete(inputBox, "", 23);
            this._alignFormulaBar();
        },

        _alignFormulaBar: function (width) {
            width = width || this._responsiveWidth;
			this.element.find('.e-formulabar').width(width);
            width = parseInt(width);
            var idxWt, iptWt, inputBox;
            idxWt = width / 10;
            iptWt = width - (idxWt + 25); // 25 is for border and padding for the index box and input box
            inputBox = this.element.find(".e-inputbox");
            inputBox.hide();
            inputBox.width(Math.floor(iptWt));
            this.element.find(".e-indexbox").width(Math.floor(idxWt));
            inputBox.show();
        },

        hideFormulaBar: function () {
            $("#" + this._id + "_formulabar").hide();
        },

        showFormulaBar: function () {
            if (this.model.allowFormulaBar) {
                $("#" + this._id + "_formulabar").show();
                this.updateFormulaBar();
            }
        },

        updateFormulaBar: function () {
            var val, cellData, sheetIdx = this.getActiveSheetIndex(), activecell = this.getSheet(sheetIdx)._activeCell, text;
            var $trgt = this._getContent(sheetIdx).find(".e-ss-activeimg");
            cellData = this.getRangeData([activecell.rowIndex, activecell.colIndex, activecell.rowIndex, activecell.colIndex], false, ["value", "value2","isLocked"])[0];
            val = ej.isNullOrUndefined(cellData.value) ? "" : cellData.value;
            if (this._isDateTime(val)) // to avoid date obj editing issue
                val = cellData.value2;
            this._getInputBox().val(val);
            if (this.XLEdit.getPropertyValue(activecell.rowIndex, activecell.colIndex, "isReadonly") || (this.model.allowLockCell && this.getSheet(sheetIdx)._isSheetProtected && cellData.isLocked))
                this._getInputBox().attr("readonly", "readonly");
            else
                this._getInputBox().removeAttr("readonly");
            if (!ej.isNullOrUndefined($trgt[0]) && ($trgt[0].id.indexOf(this._id + "_picture") > -1 || $trgt[0].id.indexOf(this._id + "_chart") > -1))
                text = $trgt[0].id.split("_")[1];
            else
                text = this._getAlphaRange(sheetIdx, activecell.rowIndex, activecell.colIndex, activecell.rowIndex, activecell.colIndex);
            this._getIndexBox().html(text);
        },

        _initializeSelection: function () {
            var range, model = this.model, sheetIdx = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIdx),
                scont = this._getContent(sheetIdx).find(".e-content"), type = model.selectionSettings.selectionType,
                types = ej.Spreadsheet.SelectionType, actCell = model.selectionSettings.activeCell;
            scont.append(this.getFocusTopElem(), this.getFocusRightElem(), this.getFocusBottomElem(), this.getFocusLeftElem());          
            if (model.allowSelection) {
                if (actCell) {
                    range = this.getRangeIndices(actCell);
                    if (!this.isRange(range) && (range[0] < sheet.rowCount && range[1] < sheet.colCount))
                        sheet._startCell = sheet._endCell = sheet._activeCell = { rowIndex: range[0], colIndex: range[1] };
                }
                if (!sheet._importRendered && sheet._selectedCells.length)
                    this.XLSelection._refreshBorder();
                else {
                    this.XLSelection._processBoundary();
                    if (type === types.Row)
                        this.XLSelection.selectRows(sheet._startCell.rowIndex, sheet._endCell.rowIndex);
                    else if (type === types.Column)
                        this.XLSelection.selectColumns(sheet._startCell.colIndex, sheet._endCell.colIndex);
                    else
                        this.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                }
            }
            if(!this.getAutoFillElem())
                this._createAutoFillElement();
            if (model.allowAutoFill) {
                this.getAutoFillElem().appendTo(scont);
                this.XLDragFill.positionAutoFillElement();
            }
            if (model.autoFillSettings.showFillOptions && model.allowAutoFill) {
                this._getAutoFillOptElem().appendTo(scont);
                this.XLDragFill.hideAutoFillOptions();
            }
        },

        _updateRibbonTab: function (trgt) {
            var actShape, robj = $("#" + this._id + "_Ribbon").data("ejRibbon"), cname, ctabs = this.element.find(".e-ribbon .e-contextul a:visible"), len = ctabs.length, ribTabPinEle = robj.element.find("#" + this._id + "_Ribbon_togglebutton");
            (!ej.isNullOrUndefined(trgt)) && (cname = trgt.className)
            if (len) {
                while (len--)
                    robj.hideTab(ctabs[len].text);
            }

            if (ribTabPinEle.hasClass("e-ribbonpin"))
                this._refreshSheetonResize();

            actShape = this._getContent(this.getActiveSheetIndex()).find(".e-ss-activeimg");
            if (actShape.length)
                actShape[0].id.indexOf("chart") > -1 ? this.XLRibbon._chartDesignTabUpdate(actShape) : this.XLRibbon._formatTabUpdate();
            else if (!ej.isNullOrUndefined(trgt) && this._hasClass(trgt, "e-table"))
                this.XLRibbon._designTabUpdate(this._getTableID(cname), trgt);
            else
                return;
        },

        _updateCellType: function () {
            var i, ctype = ej.Spreadsheet.CellType;
            for (i in ctype)
                this._dataContainer.cellType.push(ctype[i]);
        },        

        _updateFormulaCollection: function () {
            var i, libraryFunc = new CalcEngine().getLibraryFunctions(), arr = [];
            for (i in libraryFunc.items)
                arr.push({ text: "=" + i, display: i });
            this._formulaCollection = arr;
        },

        _renderScroller: function (sheetIdx, settings) {
            this.XLScroll._createScroller(sheetIdx, settings);
            this._refreshScrollerEvent(sheetIdx);
        },

        _refreshScrollerEvent: function (sheetIdx) {
            this._getContent(sheetIdx).find(".e-content").scroll(ej.proxy(function (e) {
                this._getJSSheetHeader(sheetIdx).find(".e-headercontent").scrollLeft($(e.currentTarget).scrollLeft());
                this._getJSSheetRowHeaderContent(sheetIdx).scrollTop($(e.currentTarget).scrollTop());
            }, this));

            this.element.find("#" + this._id + "_" + this.model.sheets[sheetIdx]._sheetName.value).find(".e-headercontent").scroll(ej.proxy(function (e) {
                var $currentTarget = $(e.currentTarget);
                this._getContent(sheetIdx).find(".e-content").first().scrollLeft($currentTarget.scrollLeft());
            }, this));
        },

        _rClickHandler: function () {
            if (!ej.isNullOrUndefined(this._excelFilter)) {
                this.XLFilter._isSearchEdit = false;
                this._excelFilter.closeXFDialog();
            }
        },

        _cellClick: function (e) {
            var args, target = e.target, sheetIdx = this.getActiveSheetIndex(), colIndex, rowIndex, column;
            if (e.target.tagName === "TD" && $(e.target).parents("div")[0].className.indexOf("e-content") > -1) {
                colIndex = target.cellIndex, rowIndex = $.inArray($(target).closest("tr").get(0), this.getRows(sheetIdx)[1]);
                column = this.getSheet(sheetIdx).columns[colIndex];
                args = {
                    columnName: column.field, value: $(target).data("value"), columnIndex: colIndex, rowIndex: rowIndex, columnObject: column, cell: target, eventArgs: e, event: e,
                    target: e.target
                };
                if (this._trigger("cellClick", args))
                    return;
            }

            else if (!ej.isNullOrUndefined($(e.target).parents("div")) && $(e.target).parents("div")[0].className.indexOf("e-content") > -1) {
                target = $(e.target).parents("td").get(0);
                if (!ej.isNullOrUndefined(target)) {
                    colIndex = target.cellIndex, rowIndex = $.inArray($(target).closest("tr").get(0), this.getRows(sheetIdx)[1]);
                    column = this.getSheet(sheetIdx).columns[colIndex];
                    args = {
                        columnName: column.field, value: $(target).data("value"), columnIndex: colIndex, rowIndex: rowIndex, columnObject: column, cell: target, eventArgs: e, event: e,
                        target: e.target
                    };
                    if (this._trigger("cellClick", args))
                        return;

                }
            }
        },

        _wireEvents: function () {
            var elem = this.element, evnt = ej.eventType, id = this._id;
            this._on(elem, evnt.mouseDown, ".e-ribbon, .e-spreadsheetcontentcontainer .e-vscroll, .e-spreadsheetcontentcontainer .e-hscroll", this._rClickHandler);
            this._on(elem, evnt.mouseDown, ".e-spreadsheetcontentcontainer .e-content,.e-spreadsheetrowheader,.e-spreadsheetheader,.e-spreadsheetactpanel", this._mouseDownHandler);
            this._on(elem, "keydown", this._keyDownHandler);
            this._on(elem, "keyup", this._keyUpHandler);
            this._on(elem, "click", ".e-spreadsheetcontentcontainer .e-content,.e-spreadsheetrowheader,.e-spreadsheetheader,.e-spreadsheetactpanel, .e-ss-object", this._cellClick);
            this._on(elem, "dblclick", ".e-spreadsheetcontentcontainer,.e-spreadsheetheader,.e-spreadsheetrowheader", this._recorddblClickHandler);
            this._on(elem, "dblclick", ".e-numericcontainer .e-numericitem", this._pagerdblClickHandler);
            this._on(elem, evnt.mouseMove, this._mouseMove);
            this._on(elem, evnt.mouseUp, this._sheetMouseUp);
            this.model.allowResizing && this._on(elem, evnt.mouseLeave, ".e-spreadsheetrowheader,.e-spreadsheetheader", this._mouseLeave);
            this._on(elem, evnt.click, ".e-ssfiltered, .e-ssfilter, .e-ssfiltered-asc, .e-ssfilter-asc, .e-ssfiltered-dsc, .e-ssfilter-dsc", this._filterDownHandler);
            this._on(elem, evnt.mouseDown, ".e-ssddl", this._ddlopen);
            this._on(elem, "mouseover", ".e-rowcell", this._cellHoverHandler);
			this._on( elem, "mouseover", ".e-viewhdrlbl, .e-vheading", this._headingchkboxHover);
			this._on( elem, "mouseover", ".e-viewlinlbl, .e-vgridlines", this._gridlineschkboxHover);
			this._on( elem, "mouseleave", ".e-viewhdrlbl, .e-vheading", this._headingchkboxLeave);
			this._on( elem, "mouseleave", ".e-viewlinlbl, .e-vgridlines", this._gridlineschkboxLeave);
            this._on(this._getInputBox(), "input", this.XLEdit._formulaBoxInputChange);
            this._on($('#' + id + '_cellstyles'), evnt.click, this._cellStyleClick);
            this._on($('#' + id + '_formatastable'), evnt.click, this._formatAsTableClick);
            this._on($('#' + id + '_nmuseinformula'), evnt.click, ".e-nmuseinformularow", this._useInFormulaClick);
            if (this.model.scrollSettings.allowScrolling)
                this.element.find(".e-spreadsheet-list").bind('mousewheel DOMMouseScroll', $.proxy(this.XLScroll._spreadMouseWheel, this.XLScroll));
            this._on($(".e-menu .e-chartcontent .e-chartcell"), evnt.click, this._chartClickHandler);
            this._on($("#" + id + "_allchart").find(".e-chartimg"), evnt.click, this._allChartType);            
			if(this.model.scrollSettings.isResponsive) {
				var resizeTimer, proxy = this;
				window.onresize = function(){
					if (resizeTimer){
						clearTimeout(resizeTimer);
					} 
					resizeTimer = setTimeout(function(){
						proxy._spreadsheetonresize();
					}, 100);
				};
			}
        },

		_headingchkboxHover: function(e) {
			this.element.find(".e-vheading .e-chk-act, .e-vheading .e-chk-inact").addClass("e-chkhover");
		},
		
		_gridlineschkboxHover: function(e) {
			this.element.find(".e-vgridlines .e-chk-act,.e-vgridlines .e-chk-inact").addClass("e-chkhover");
		},
		
		_headingchkboxLeave: function(e) {
			this.element.find(".e-vheading .e-chk-act, .e-vheading .e-chk-inact").removeClass("e-chkhover");
		},
		
		_gridlineschkboxLeave: function(e) {
			this.element.find(".e-vgridlines .e-chk-act,.e-vgridlines .e-chk-inact").removeClass("e-chkhover");
		},
		
        _allChartType: function (e) {
            $("#" + this._id + "_allchart").find(".e-chartimg").removeClass("e-chartselect");
            $(e.target).addClass("e-chartselect");
        },

        _spreadsheetonresize: function () {
            this._refreshSheetonResize();
        },

        _refreshSheetonResize: function () {
            var sheetIdx = this.getActiveSheetIndex(), settings, actpanel = this.getActivationPanel(), popupElem = $('.e-menu:visible[id^=' + this._id + '], .e-ddl-popup:visible[id^=' + this._id + ']'), elemId, sElem,
                popupIds = ["CFormat", "Validation", "PageSize", "FPane", "Ins", "Del", "FindRep"], pEleIds = ["cFormat", "validationsbtn", "sizesbtn", "fPane", "inssbtn", "delsbtn", "find"];
            this.element.find(".e-inputbox").hide();
            settings = { width: this.model.scrollSettings.width, height: this.model.scrollSettings.height };
            settings.width = actpanel.is(":visible") ? settings.width - this.model.apWidth : settings.width;
            this._setSheetWidthHeight(sheetIdx, settings, null, "resize");
            this.element.find(".e-inputbox").show();
            if(this.model.scrollSettings.allowScrolling)
				this.XLScroll._scrollSelectedPosition(sheetIdx, this.getActiveCell());
            //pop-up hide
            if (popupElem.length) {
                elemId = popupElem[0].id.split("_", 3);
                elemId.indexOf("Ribbon") === -1 && elemId.splice(1, 0, "Ribbon");
                elemId[2] = popupIds.indexOf(elemId[2]) > -1 ? pEleIds[popupIds.indexOf(elemId[2])] : elemId[2].toLowerCase();
                elemId = elemId.join("_");
                if (elemId.length) {
                    sElem = $("#" + elemId);
                    if (sElem.hasClass("e-splitbutton"))
                        sElem.ejSplitButton("hide");
                    if (sElem.hasClass("e-dropdownlist"))
                        sElem.ejDropDownList("hidePopup");
                }
            }
        },

        _chartClickHandler: function (e) {
            var trgt = $(e.target), prop = $.extend(true, {}, $(e.target).data());
            trgt.parents().eq(1).hide();
            this.XLChart.createChart(null, prop);
        },

        _useInFormulaClick: function (e) {  // defined names click
            var proxy = this, sheetIdx = proxy.getActiveSheetIndex(), actCell = proxy.getActiveCellElem(sheetIdx)[0];
            proxy.XLEdit.editCell(actCell.parentNode.rowIndex, actCell.cellIndex, true);
            $("#" + proxy._id + "_" + proxy.XLEdit._EditCellDetails.fieldName).val("=" + e.target.textContent);
            $("#" + proxy._id + "_nmuseinformula").hide();
        },

        _formatAsTableClick: function (e) {
            var cname, fatName, proxy = this, target = e.target, open = 'open',
            sheet = proxy.getSheet(proxy.getActiveSheetIndex()), scells = sheet._selectedCells;
            if (scells.length === 0)
                return;
            if (target.parentNode.className === 'e-formatastablecell') {
                cname = proxy.getCell(scells[0].rowIndex, scells[0].colIndex)[0].className;
                if (cname.indexOf('e-table') > -1) {
                    proxy.XLFormat._formatAsTableStyle = $(target.parentNode).data();
                    proxy.XLFormat._createTable(proxy._getTableID(cname), proxy.XLFormat._formatAsTableStyle);
                }
                else if (scells.length > 1) {
                    fatName = $("#" + proxy._id + '_fatname');
                    $('#' + proxy._id + '_fatnamedlg').ejDialog(open);
                    $("#" + proxy._id + "_fatheader").attr("checked", false);
                    fatName.val('Table ' + proxy.getObjectLength(sheet._tableManager)).focus().setInputPos(fatName.val().length);
                    if(proxy.model.allowCellFormatting) proxy.XLFormat._formatAsTableStyle = $(target.parentNode).data();
                }
                else 
                    proxy._showAlertDlg("Alert", "FormatAsTableAlert", null, 445);
                $('#' + proxy._id + '_Ribbon_formatastable').data("ejSplitButton")._hidePopup();
            }
        },

        _getTableID: function (classname, name) {
            var i, tmgr = this.getSheet(this.getActiveSheetIndex())._tableManager;
            if (name) {
                for (i in tmgr)
                    if (name === tmgr[i].name)
                        return Number(i);
            }
            else {
                for (i in tmgr) {
                    if (classname.indexOf('e-table' + i) > -1)
                        return Number(i);
                }
            }
        },

        _cellStyleClick: function (e) {
            var elem, cname = e.target.className, formatstr = this.XLFormat.getFormatClass(cname);
            var details = { sheetIndex: this.getActiveSheetIndex(), reqType: "cell-styles", beforeFormat: this.XLFormat.getHashCodeClassAsArray() };
            if (cname.indexOf('e-cellstylecell') > -1)
                formatstr.indexOf('e-format') > -1 ? this.XLFormat.updateUniqueFormat(formatstr) : this.XLFormat.clearFormat();
            details.afterFormat = this.XLFormat.getHashCodeClassAsArray();
            details.range = this.model.sheets[this.getActiveSheetIndex()].selectedRange;
            if (!this.isUndoRedo)
                this._completeAction(details);
            $('#' + this._id + "_Ribbon_cellstyles").data("ejSplitButton")._hidePopup();
        },


        _cellHoverHandler: function (e) {
            var args = { target: e.target };
            this._trigger("cellHover", args);
        },


        _filterDownHandler: function (e) {
            if (e.target.className.indexOf("e-ssfilter") > -1) {
                if (this._excelFilter._openedFltr && $(this._excelFilter._openedFltr).is(":visible"))
                    this._excelFilter.closeXFDialog();
                else
                    this.XLFilter._filterClickHandler(e);
            }
        },

        _mouseDownHandler: function (e) {
			e.target = this._changeTargetWithOffset(e);
            var cname, str = "e-ss-object", trgt = e.target, $trgt = $(trgt), sheetIdx = this.getActiveSheetIndex(), actSheet = this.getSheet(sheetIdx), tid, proxy = this, isNavigate = true, robj = $('#' + this._id + '_Ribbon').data('ejRibbon'), imgNavigate = true, cell;
            this.model.allowCharts && actSheet._isChartBorderDrawn && this.XLChart._clearChartRange(this._arrayAsString(this._chartBorder));
			if(this.getSheetElement(sheetIdx).find(".e-ss-activeimg").length)
				this.model.showRibbon && this.XLRibbon._toggleFormatTab();
            if ((this.model.pictureSettings.allowPictures || this.model.allowCharts) && ($trgt.hasClass(str) || $($trgt).parents("." + str).length)) {
                if (!$trgt.hasClass(str))
                    trgt = e.target = $($trgt).parents("." + str)[0];
                this.XLShape._shapeMouseDown(e);
            }
            else {
                imgNavigate = false;
                !(this.XLShape._shapeROStart || this.XLShape._shapeRSStart) && this._getContent(sheetIdx).find(".e-ss-activeimg").removeClass("e-ss-activeimg e-ss-imgcresize e-ss-imgrresize");
            }
            cname = trgt.className;
            //Click the Chart
            if (this.model.showRibbon && trgt.id.indexOf("chart") > -1)
                this.XLRibbon._chartDesignTabUpdate($(trgt));
            else if (this.model.showRibbon && robj.model.selectedItemIndex === this.XLRibbon._getTabIndex("cdesign"))
                this.XLRibbon._toggleChartDesignTab();
            if (this.XLEdit._isEdit && $(this.getCell(actSheet._activeCell.rowIndex, actSheet._activeCell.colIndex)).index($(trgt).parents("td")) > -1)
                return;
            if (this.model.allowDragAndDrop && this.XLDragDrap._isDragAndDropStart && cname !== "e-autofill")
                this.XLDragDrap._dMouseDown(e);
            if (this.model.allowResizing && !this.XLEdit._isEdit) {
                if ($(trgt).hasClass("e-headercelldiv") || $(trgt).hasClass("e-headercell"))
                    this.XLResize._headerMouseDown(e);
                else if ($(trgt).hasClass("e-rowheader"))
                    this.XLResize._rowMouseDown(e);
                else if ($(trgt).hasClass("e-spreadsheetactpanel"))
                    this.XLResize._apMouseDown(e);
            }
            if ($(trgt).parents("div.e-spreadsheetactpanel").length > 0)
                return;
            isNavigate = this.XLEdit._isEdit ? (this._isFormulaSuggestion || !this.XLEdit._isFormulaEdit) && this.XLEdit.saveCell() : true;
            if (!isNavigate && !this.XLEdit._isFormulaEdit)
                return false;
            if (trgt.tagName === "A") {
                cell = $trgt.parent("td")[0];
                var obj = this._getCellIdx(cell);
                actSheet._activeCell = obj;
                this.XLSelection.selectRange(obj, obj);
                e.which !== 3 && this._hlClick(this.XLEdit.getPropertyValue(obj.rowIndex, obj.colIndex, "hyperlink", sheetIdx));
            }
            else if ((e.button !== 2 || e.which !== 3) || this.getSheetElement(sheetIdx).find(".e-selected, .e-colselected .e-headercelldiv, .e-rowselected").index(trgt) === -1) { // for context menu behavior                          
                if (this.model.allowSelection && !imgNavigate) {
                    if (ej.isNullOrUndefined(this.XLDragDrap) || !this.XLDragDrap._allowDragAndDrop) {
                        if (!(this.XLResize && this.XLResize._resizeStart) && !$trgt.hasClass("e-comment-txtarea"))
                            this.XLSelection._selectionMouseDownHandler(e);
                    }
                }
            }
            this.model.allowFiltering && this._filterClose();
			 this._undoInputobj = {};
            this._setRowHdrHeight(sheetIdx, this.model.sheets[sheetIdx]._activeCell.rowIndex);
            if (trgt.tagName === "TD" && $(trgt).parents("div")[0].className === "e-content")
                this.displayActivationPanel(this.XLEdit.getPropertyValueByElem($(trgt), "range"));
            if ((trgt.tagName === "TD") && trgt.className.indexOf("e-rowheader") < 0) {
                var val = $("#" + this._id + "ddl").val();
                cell = this._ddlCell;
                if (!ej.isNullOrUndefined(cell)) {
                    cell.find("span#" + this._id + "ddl_wrapper").remove();
                    $("#" + this._id + "ddlspan").remove();
                    $("#" + this._id + "ddl").remove();
                    cell.text(val);
                    this.XLEdit.updateCell(cell, cell.text());
                    this._ddlCell = null;
                }
            }
            if (!ej.isNullOrUndefined(this.XLEdit.getPropertyValue($(trgt).closest('tr').index(), $(trgt)[0].cellIndex, "rule", sheetIdx)))
                this._setddlCell(e);
            //Comment
            this.model.allowComments && !$trgt.hasClass("e-pager") && this.getSheetElement(sheetIdx).find(".e-commentcell").length > 0 && this.XLComment._commentMouseDownHandler(e);
            //Sheet Rename
            this._isSheetRename && this._updateSheetNames(null, false);
            this.element.find("#" + this._id + "ddl").ejDropDownList("hidePopup");
            if (cname.indexOf('e-rowcell') > -1 && cname.indexOf('e-table') > -1) {
                tid = proxy._getTableID(cname);
                proxy.isNumber(tid) && proxy.XLRibbon._designTabUpdate(tid, trgt);
            }
            else if (this.model.showRibbon && (robj.model.selectedItemIndex === this.XLRibbon._getTabIndex("design") || robj.isVisible(this._getLocStr("Design"))))
                this.XLRibbon._toggleDesignTab(trgt);
            if (this.model.allowEditing) {
                var cell = this.getSheet(this.getActiveSheetIndex())._activeCell;
                if (this.XLEdit._isEdit && e.target.id !== this._id + "_inputbox") {
                    var elem = ($(e.target).hasClass("e-ss-input") || !this.XLEdit._isFBarFocused) ? this.element.find("#" + this._id + "_Edit") : this._getInputBox();
                    if (elem) {
                        e.target.id !== elem[0].id && elem.focus().setInputPos(elem[0].selectionStart, elem[0].selectionEnd);
                        elem.hasClass("e-ss-input") && (this.XLEdit._isFBarFocused = false);
                    }
                }
                if (!(this.model.allowComments && this.XLComment._isCommentEdit) && !this._isSheetRename && e.target.id === this._id + "_inputbox") {
                    if (e.target.readOnly)
                        this.setSheetFocus();
                    else {
                        !this.XLEdit._isEdit && this.XLEdit.editCell(cell.rowIndex, cell.colIndex, true);
                        this._getInputBox().focus();
                        this.XLEdit._isFBarFocused = true;
                    }
                }
            }
        },
        _setddlCell: function (e) {
            var i, elepos, len, value, listdatsrc = [], check, customvalue, cell, trgt, sheetIdx = this.getActiveSheetIndex(), pos;
            if (!ej.isNullOrUndefined(e) && e.type === "mousedown") {
                cell = e.target;
                trgt = e.currentTarget;
            }
            else {
                cell = this.getSheetElement(sheetIdx).find(".e-activecell")[0];
                trgt = $("#" + this._id + "GroupPanel").find("div.e-content")[0];
            }
            customvalue = this.XLEdit.getPropertyValue($(cell).closest("tr").index(), cell.cellIndex, "rule", sheetIdx);
            if (customvalue && customvalue.type === "list") {
                this._ddlCell = $(cell);
                check = customvalue.customVal.split("_");
                var dataSrc = check[0], i, len, values = [];
                if (dataSrc.startsWith("=") && dataSrc.indexOf(",") < 0) {                    
                    var value = $.trim(dataSrc.replace("=", "")), splitStr;
                    if (value.indexOf("!") > -1) {
                        splitStr = value.split("!");
                        sheetIdx = this._getSheetIndexByName(splitStr[0]);
                        value = splitStr[1];
                    }
                    value.indexOf("$") > -1 && (value = value.split("$").join(""));
                    value.indexOf(":") < 0 && (value = value + ":" + value);
                    range = this.getRangeIndices(value);
                    var cells = this.getRange(range[0], range[1], range[2], range[3]);
                    for (i = 0, len = cells.length; i < len; i++) {
                        if (!ej.isNullOrUndefined($(cells[i])) && this.XLEdit.getPropertyValueByElem($(cells[i]), null, sheetIdx))
                            values.push(this.XLEdit.getPropertyValueByElem($(cells[i]), null, sheetIdx));
                        else
                            values.push("");
                    }
                    if (!values.length)
                        values.push(dataSrc);
                }
                else if (dataSrc.indexOf(",") > -1) 
                     values = dataSrc.split(",");                                                    
                else {
                    values = [];
                    values.push(dataSrc);
                }
                for (i = 0, len = values.length; i < len; i++)
                    listdatsrc[i] = { id: i + 1, text: values[i].toString() };
                value = this._ddlCell.text();
                this._ddlCell.text("");
                elepos = { "top": this._ddlCell[0].offsetTop, "left": this._ddlCell[0].offsetLeft };
                pos = this._ddlCell.height() - 18;
                $(cell).append('<input id="' + this._id + 'ddl" type="text" style="line-height:0px;text-align:end;padding-left:0px;background-color:#ffffff;color:#333333;font-family:calibri;" value=' + value + '>');
                trgt.insertAdjacentHTML("beforeEnd", "<span id='" + this._id + "ddlspan' class='e-icon e-ssddl e-spanddl e-ddlspan' role='button'></span>");
                $("#" + this._id + "ddlspan").css({ top: elepos.top + pos, left: elepos.left + this._ddlCell.width() + 3 });
                $("#" + this._id + "ddl").ejDropDownList({ dataSource: listdatsrc, fields: { id: "Types", text: "text", value: "text" }, height: "16", width: "auto", beforePopupShown: $.proxy(this._drppopupshow, this), change: $.proxy(this._changeddl, this) });
                $("#" + this._id + "ddl_dropdown").remove();
                $("#" + this._id + 'ddl_container').removeClass('e-in-wrap e-box');
            }
        },

        _drppopupshow: function () {
            $("#" + this._id + "ddl_popup").css({ "width": this.getActiveCellElem().width() + 20, "font-size": "11px" });
        },
        _ddlPosition: function () {
            var elepos = { "top": this._ddlCell[0].offsetTop, "left": this._ddlCell[0].offsetLeft };
            $("#" + this._id + "ddlspan").css({ top: elepos.top + this._ddlCell.height() - 19, left: elepos.left + this._ddlCell.width() + 3 });
        },

        _changeddl: function (args) {
            this.XLEdit.updateCellValue($("#" + this._id + "ddl").parents("td"), args.selectedText);
            this.getActiveCellElem().hasClass("e-hlcell") && this.XLValidate.highlightInvalidData();
        },

        _ddlopen: function (e) {
            var actionObj = $("#" + this._id + "ddl").data("ejDropDownList");
            e.stopPropagation();
            if (!ej.isNullOrUndefined(actionObj)) {
                if (actionObj._isPopupShown()) {
                    actionObj._hideResult();
                }
                else {
                    actionObj._showResult();
                    $("#" + this._id + "ddl_popup").css({ "width": this.getActiveCellElem().width() + 20, "font-size": "11px" });
                }
            }
        },

        _mouseMove: function (e) {
			e.target = this._changeTargetWithOffset(e);
            var trgt = $(e.target);
            if (this.model.allowResizing && (this.XLResize._resizeType === "Col" || trgt.hasClass("e-headercelldiv") || trgt.hasClass("e-headercell") || trgt.hasClass("e-reSizeColbg")) && !this.XLEdit._isEdit && !(this.XLCMenu && this.XLCMenu._isMenuOpened))
                this.XLResize._cMouseMove(e);
            else if (this.model.allowResizing && (this.XLResize._resizeType === "Row" || trgt.hasClass("e-rowheader") || trgt.hasClass("e-reSizeRowbg")) && !this.XLEdit._isEdit && !(this.XLCMenu && this.XLCMenu._isMenuOpened))
                this.XLResize._rMouseMove(e);
            else if (this.model.allowResizing && (this.XLResize._resizeType === "AP" || trgt.hasClass("e-spreadsheetactpanel") || trgt.parents("e-spreadsheetactpanel").length > 0))
                this.XLResize._apMouseMove(e);
            else if ((this.model.pictureSettings.allowPictures || this.model.allowCharts) && (trgt.hasClass("e-ss-object") || trgt.parents(".e-ss-object").length || this.XLShape._shapeROStart || this.XLShape._shapeRSStart || trgt.hasClass("e-ss-imgvisual")))
                this.XLShape._imgMouseMove(e);
            if (this.model.allowDragAndDrop && ej.spreadsheetFeatures.dragAndDrop && trgt.hasClass("e-rowcell") && !this.XLEdit._isEdit && this._getJSSheetHeader(this.getActiveSheetIndex())[0].querySelector('.e-spreadsheetcolumnheader').className.indexOf('e-sheetselected') === -1)
                this.XLDragDrap._dMouseMove(e);
            if (this.model.allowComments && !this.XLComment._isCommentEdit && (this.getSheetElement(this.getActiveSheetIndex()).find(".e-commentcell").length > 0))
                this.XLComment._commentMouseMove(e);
			if (!ej.isNullOrUndefined(e.target.parentNode))
				((e.target.id.indexOf('_svg') < 0) && e.target.parentNode.className.indexOf('e-content') < 0 && !$(e.target).is('td.e-rowcell')) && this.XLShape._shapeMouseUp(e);
        },

        _mouseLeave: function (e) {
            $(".e-ss-colresize, .e-ss-rowresize").removeClass("e-ss-colresize e-ss-rowresize");
        },

        _sheetMouseUp: function (e) {
			if( this.model.allowResizing)
			{
            if (this.XLResize._resizeType === "Col")
                this.XLResize._cMouseUp(e);
            else if (this.XLResize._resizeType === "Row")
                this.XLResize._rMouseUp(e);
            else if (this.XLResize._resizeType === "AP")
                this.XLResize._apMouseUp(e);
			}
            if (this.model.allowDragAndDrop && this.XLDragDrap._allowDragAndDrop)
                this.XLDragDrap._dMouseUp(e);           
            this.model.allowComments && this.XLComment._isCommentEdit && this.getActiveCellElem().find("textarea").focus();
            this._isSheetRename && $("#" + this._id + "_Sheet_RenamePanel").focus();
            this.model.pictureSettings.allowPictures && this.XLShape._shapeMouseUp(e);
        },

        _keyDownHandler: function (e) {
            var isNavigate = true, imgNavigate = false, sheetIdx = this.getActiveSheetIndex(), activeCell, tid, cname, robj = $('#' + this._id + '_Ribbon').data('ejRibbon');
            if (this._getContent(sheetIdx).find(".e-ss-activeimg").length) {
                imgNavigate = true;
                var actElemId = document.activeElement.id;
                actElemId.indexOf(this._id + "_pictureheight") < 0 && actElemId.indexOf(this._id + "_picturewidth") < 0 && this.XLShape._imgKeyDown(e);
            }
            if (e.target.id.indexOf("_Custom") < 0 && e.target.id.indexOf("_Ribbon") < 0) {
                e.stopImmediatePropagation();
                if (this.model.allowFiltering && this.XLFilter._isSearchEdit && e.keyCode !== 27)
                    return true;               
                if ($(e.target).parents(".e-formulabar").length && this.XLEdit._isEdit && this.model.allowKeyboardNavigation)
                    this.XLCellNav._isNavigate = e.keyCode === 13;
				!this._isSheetRename && !this.XLEdit._isEdit && !(this.XLComment && this.XLComment._isCommentEdit) && this.model.allowClipboard && this.XLClipboard._copyPasteHandler(e);
                if (!imgNavigate && !this._isSheetRename)
                    !(this.XLComment && this.XLComment._isCommentEdit) ? this.model.allowEditing && (document.activeElement.id.indexOf('_inputbox') === -1 || e.keyCode === 27) && this.XLEdit._editingHandler(e) : this.model.allowKeyboardNavigation && (this.XLCellNav._isNavigate = false);
                this.model.allowSearching && this.XLFndRp._fRKeyDownHandler(e);
                if (this._isFormulaSuggestion && (e.keyCode === 39 || e.keyCode === 37 || (e.keyCode === 13 && !e.altKey)))
                    this.XLEdit.saveCell();
                else if (!this.XLEdit._isFormulaEdit && (e.keyCode === 9 || (e.keyCode === 13 && !e.altKey) || e.keyCode === 37 || e.keyCode === 38 || e.keyCode === 39 || e.keyCode === 40))
                    isNavigate = this.XLEdit._isEdit && this.model.allowKeyboardNavigation && this.XLCellNav._isNavigate ? this.XLEdit.saveCell() : true;
                if ($(e.target).hasClass("e-field-validation-error") || (e.target.id == this._id + '_Ribbon_font-family_wrapper' || e.target.id == this._id + '_Ribbon_cellformat_wrapper' || e.target.id == this._id + '_Ribbon_fontsize_wrapper'))
                    return true;
                if ((!isNavigate && !this.XLEdit._isFormulaEdit) || (e.keyCode===13 && e.altKey))
                    return false;
                if (this.model.allowKeyboardNavigation && !imgNavigate && !this._isSheetRename) {
                    if (!(this.model.allowComments && this.XLComment._isCommentEdit))
                        if ((!$(e.target).parents(".e-formulabar").length || e.keyCode === 9 || (e.keyCode===13 && !e.altKey)))
                            this.XLCellNav._selectionKeyDownHandler(e);
                    this._keyboardShortCuts(e);
                }
                if (this.model.allowUndoRedo)
                    if (this.model.allowComments && !this.XLComment._isCommentEdit)
                        this._undoRedoOperation(e);
                    else if (!this.model.allowComments)
                        this._undoRedoOperation(e);
                if (this.XLEdit._isFormulaEdit && e.keyCode === 9 && !this.XLCellNav._canKeyBoardNavigate) // enabling tab key while selecting formula
                    this.XLCellNav._canKeyBoardNavigate = true;
            }
            if ((e.target.tagName === "DIV") && !this.XLEdit._isEdit) {
                var elem = this.element.find("#" + this._id + "ddl"), val = elem.val(), cell = this._ddlCell;
                if (!ej.isNullOrUndefined(cell)) {
                    elem.ejDropDownList("hidePopup");
                    cell.find("span#" + this._id + "ddl_wrapper").remove();
                    this.element.find("#" + this._id + "ddlspan").remove();
                    elem.remove();
                    cell.text(val);
                    this.XLEdit.updateCell(cell, cell.text());
                    this._ddlCell = null;
                }
            }
            activeCell = this.getActiveCellElem(), cname = !ej.isNullOrUndefined(activeCell) && activeCell.length ? activeCell[0].className : "";
            if (cname.indexOf('e-rowcell') > -1 && cname.indexOf("e-table") > -1) {
                tid = this._getTableID(cname);
                this.isNumber(tid) && this.XLRibbon._designTabUpdate(tid, e.target);
            }
            else if (this.model.showRibbon && (robj.model.selectedItemIndex === this.XLRibbon._getTabIndex("design") || robj.isVisible(this._getLocStr("Design"))))
                this.XLRibbon._toggleDesignTab(e.target);
            if (activeCell.length && !this.XLEdit._isEdit && !ej.isNullOrUndefined(this.XLEdit.getPropertyValue(activeCell.closest("tr").index(), activeCell[0].cellIndex, 'rule', sheetIdx)))
                this._setddlCell(e);

            if (this._isSheetRename && (e.keyCode === 13 || e.keyCode === 9 || e.keyCode === 27)) {
                e.preventDefault();
                this._updateSheetNames(null, e.keyCode === 27) && this.setSheetFocus();
            }
        },

        _keyUpHandler: function (e) {
            if (this.model.allowEditing && this.XLEdit._isEdit) {                             
                if (this.element.find("#" + this._id + "_Edit").text().indexOf("=") > -1)
                    this.XLEdit._captureEditing();                
            }
        },
        _recorddblClickHandler: function (e) {
            var sheetCont, trgt, cellIdx;
			e.target = this._changeTargetWithOffset(e);
            if (!this.model.allowEditing || !this.model.allowSelection)
                return;            
            if (this.XLSelection._isOutsideBordering || this.XLSelection._isGridBordering) {
                sheetCont = this._getContent(this.getActiveSheetIndex()).find("div:first-child");
                sheetCont.addClass("e-ss-cursor");
                sheetCont.removeClass("e-ss-drwbrdrcursor e-ss-drwbrdrgridcursor");
                this.XLSelection._isOutsideBordering = this.XLSelection._isGridBordering = false;
                return;
            }
            if (!this.XLEdit._isEdit && ($(e.target).hasClass("e-rowcell") || $(e.target).parents("td").hasClass("e-rowcell")) && !$(e.target).hasClass("e-readonly")) {
                if($('.e-cdata').length)
                    $('.e-cutright, .e-cutbottom').removeClass('e-cutright e-cutbottom');
                trgt = $(e.target);
                if (trgt.closest("#" + this._id + "EditForm").length)
                    return;
                cellIdx = this._getCellIdx(e.target);
                this.XLEdit.editCell(cellIdx.rowIndex, cellIdx.colIndex, true);
                if (this.model.allowKeyboardNavigation)
                    this.XLCellNav._isNavigate = false;
            }
            if(this.model.allowResizing && ($(e.target).hasClass("e-headercell") || $(e.target).hasClass("e-headercelldiv")))
                this.XLResize._resizeToFitWidth(e);
            if ($(e.target).is("span") && this.model.allowResizing && this._hasClass($(e.target.parentNode)[0], "e-ss-rowresize"))
                this.XLResize._resizeToFitHeight(e.target.parentNode);
            else if(this.model.allowResizing && $(e.target).hasClass("e-ss-rowresize"))
                this.XLResize._resizeToFitHeight(e);        
        },

        _keyboardShortCuts: function (e) {
            if (this.XLEdit._isEdit)
                return false;
            var activeCell, startCell, endCell, xlFormat = this.XLFormat, sheetIdx = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIdx), selCells = sheet._selectedCells, protect = false;
            if (this.model.allowLockCell && sheet._isSheetProtected)
                protect = true;
            if (e.ctrlKey) {
                switch (e.keyCode) {
                    case 65:  // Ctrl + A
                        e.preventDefault();
                        if (this.model.allowSelection) {
                            activeCell = this.getActiveCellElem(), startCell = { rowIndex: 0, colIndex: 0 }, endCell = { rowIndex: sheet.rowCount - 1, colIndex: sheet.colCount - 1 };
                            this.XLSelection.selectRange(startCell, endCell);
                            sheet._startCell = startCell;
                            sheet._endCell = endCell;
                            !ej.isNullOrUndefined(this.XLCellNav) && this.XLCellNav._updateActiveCell($(activeCell[0]).closest('tr').index(), activeCell[0].cellIndex);
                            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
                        }
                        break;
                    case 66:  // Ctrl + B
                        e.preventDefault();
                        if (!protect)
                            this._toggleFontPrpty("bold");
                        break;
                    case 68: // Ctrl + D
                    case 82: // Ctrl + R
                        e.preventDefault();
                        if (this.model.allowAutoFill)
                            this.XLDragFill._fillRange(e.keyCode === 68);
                        break;
                    case 73: // Ctrl + I
                        e.preventDefault();
                        if (!protect)
                            this._toggleFontPrpty("italic");
                        break;
                    case 75: // Ctrl + K  
                        e.preventDefault();
                        if (this.model.allowHyperlink && !protect)
                            this._hlDlgOpen();
                        break;
                    case 85: // Ctrl + U
                        e.preventDefault();
                        if (!protect)
                            this._toggleFontPrpty("underline");
                        break;
                    case 80: // Ctrl + P
                        e.preventDefault();
                        this.XLPrint.printSheet();
                        break;
                    case 79: //Ctrl + O
                        e.preventDefault();
                        if (this.model.allowImport) {
                            if (this._isSheetsDirty())
                                this._showAlertDlg("", "ImportAlert", "Open", 470, 135);
                            else
                                $("#" + this._id + "_file .e-uploadinput").click();
                        }
                        break;
                    case 83: //Ctrl + S
                        e.preventDefault();
                        this.XLExport["export"]("Excel");
                        break;
                    case 32: // Ctrl + space
                        e.preventDefault();
                        if (!this.model.allowSelection)
                            return;
                        var sColIdx = selCells[0].colIndex, eColIdx = selCells[selCells.length - 1].colIndex;
                        startCell = { rowIndex: 0, colIndex: sColIdx }, endCell = { rowIndex: sheet.rowCount - 1, colIndex: eColIdx };
                        this.model.allowSelection && this.XLSelection.selectRange(startCell, endCell);
                        sheet._startCell = startCell;
                        sheet._endCell = endCell;
                        !ej.isNullOrUndefined(this.XLCellNav) && this.XLCellNav._updateActiveCell(selCells[0].rowIndex, sColIdx);
                        this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
                        break;
                    case 53: // Ctrl + 5
                        e.preventDefault();
                        if (!protect)
                            this._toggleFontPrpty("line-through");
                        break;
                    case 56: // Ctrl + 8 
                        if (!e.shiftKey && !protect) {
                            e.preventDefault();
                            this.hideColumn(sheet._startCell.colIndex, sheet._endCell.colIndex);
                        }
                        break;
                    case 57: // Ctrl + 9 
                        if (!e.shiftKey) {
                            e.preventDefault();
                            if (!protect)
                                this.hideRow(sheet._startCell.rowIndex, sheet._endCell.rowIndex);
                        }
                        break;
                    case 220: // Ctrl + \ 
                        if (!e.shiftKey) {
                            e.preventDefault();
                            if (this.model.allowSearching)
                                this.XLFndRp._applyGotoRowDiff();
                        }
                        break;
                    case 90:
                        e.preventDefault();
                        break;
                    case 89:
                        e.preventDefault();
                        break;
                    case 114://ctrl + f3
                        e.preventDefault();
                        if (this.model.showRibbon && !protect)
                            this.XLRibbon._openNameManagerDlg();
                        break;
                }
                if (e.shiftKey) //Ctrl + Shift
                {
                    switch (e.keyCode) {
                        case 192:  // Ctrl + Shift + ~
                            e.preventDefault();
                            if (!protect)
                                xlFormat.format({ "type": "general" });
                            break;
                        case 52:  // Ctrl + Shift + $
                            e.preventDefault();
                            if (!protect)
                                xlFormat.format({ "type": "currency" });
                            break;
                        case 53:  // Ctrl + Shift + %
                            e.preventDefault();
                            if (!protect)
                                xlFormat.format({ "type": "percentage" });
                            break;
                        case 54:  // Ctrl + Shift + ^
                            e.preventDefault();
                            if (!protect)
                                xlFormat.format({ "type": "scientific" });
                            break;
                        case 51:  // Ctrl + Shift + #
                            e.preventDefault();
                            if (!protect)
                                xlFormat.format({ "type": "shortdate" });
                            break;
                        case 50: // Ctrl + Shift + @
                            e.preventDefault();
                            if (!protect)
                                xlFormat.format({ "type": "time" });
                            break;
                        case 49:  // Ctrl + Shift + ! 
                            e.preventDefault();
                            if (!protect)
                                xlFormat.format({ "type": "number" });
                            break;
                        case 55: // Ctrl + Shift + &    
                            e.preventDefault();
                            if (this.model.allowCellFormatting && this.model.formatSettings.allowCellBorder && !protect)
                                this.XLFormat.applyBorder({ top: '1px solid #000000', right: '1px solid #000000', bottom: '1px solid #000000', left: '1px solid #000000' });
                            break;
                        case 173: // Ctrl + Shift + _
                            e.preventDefault();
                            if (this.model.allowCellFormatting && this.model.formatSettings.allowCellBorder && !protect)
                                this.XLFormat.applyBorder('noborder');
                            break;
                        case 73: // Ctrl + Shift + I   
                            e.preventDefault();
                            if (this.model.allowInsert && !protect)
                                this._renderInsertDialog();
                            break;
                        case 68: // Ctrl + Shift + D 
                            e.preventDefault();
                            if (this.model.allowDelete && !protect)
                                this._renderDeleteDialog();
                            break;
                        case 113: // Ctrl + Shift + F2
                            e.preventDefault();
                            var rObj = $('#' + this._id + '_Ribbon').data('ejRibbon');
                            if (this.model.showRibbon)
                                rObj.element.find('.e-rarrowup-2x').length ? rObj.collapse() : rObj.expand();
                            break;
                        case 56: // Ctrl + Shift + 8  
                            e.preventDefault();
                            if (!protect)
                                this.showColumn(selCells[0].colIndex, selCells[selCells.length - 1].colIndex);
                            break;
                        case 57: // Ctrl + Shift + 9   
                            e.preventDefault();
                            if (!protect)
                                this.showRow(selCells[0].rowIndex, selCells[selCells.length - 1].rowIndex);
                            break;
                        case 79: // Ctrl + Shift + O
                            e.preventDefault();
                            this.XLFndRp._applyGotoComments();
                            break;
                        case 220: // Ctrl + Shift + | 
                            e.preventDefault();
                            this.XLFndRp._applyGotoColDiff();
                            break;
                    }
                }
            }
            else if (e.shiftKey) {
                if (e.keyCode === 32) {  // Shift + space           
                    e.preventDefault();
                    if (this.model.allowSelection) {
                        var sRowIdx = selCells[0].rowIndex, eRowIdx = selCells[selCells.length - 1].rowIndex;
                        startCell = { rowIndex: sRowIdx, colIndex: 0 }, endCell = { rowIndex: eRowIdx, colIndex: sheet.colCount - 1 };
                        this.XLSelection.selectRange(startCell, endCell);
                        sheet._startCell = startCell;
                        sheet._endCell = endCell;
                        !ej.isNullOrUndefined(this.XLCellNav) && this.XLCellNav._updateActiveCell(sRowIdx, selCells[0].colIndex);
                        this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
                    }
                }
                else if (e.keyCode === 113) {  // Shift + F2
                    e.preventDefault();
                    activeCell = this.getActiveCellElem();
                    if (!protect)
                        activeCell.hasClass("e-commentcell") ? this.XLComment.editComment() : this.XLComment.setComment();
                }
                else if (e.keyCode === 122) {  // Shift + F11    
                    e.preventDefault();
                    this.addNewSheet();
                }
            }
            else if (e.keyCode === 27) { // Esc
                if (this.model.allowComments && this.XLComment._isCommentEdit) {
                    this.XLComment._updateCurCmntVal();
                    this.XLComment._updateCmntRibIcons();
                }
                if (this.model.allowFiltering)
                    if (this._excelFilter._openedFltr && $(this._excelFilter._openedFltr).is(":visible")) {
                        var mObj = this._excelFilter._menuWrap.data("ejMenu");
                        if (mObj.element.find(".e-mhover li:visible").length) {
                            if (!mObj._isContextMenuOpen)
                                mObj._closeMenu();
                            return;
                        }
                        this.XLFilter._isSearchEdit = false;
                        this._excelFilter.closeXFDialog();
                    }
                this.setSheetFocus();
            }
            else if (e.altKey) {
                if (e.keyCode === 112 && this.model.showRibbon && this.model.allowCharts && !protect)//alt +f1-Create chart
                {
                    e.preventDefault();
                    this.XLChart.createChart(null, { type: "column", enable3D: false, marker: { visible: false } });
                }
                else if (e.keyCode === 84 && this.model.showRibbon && !this.XLRibbon._isDirtySelect && !protect)//alt+T-create a table
                {
                    e.preventDefault();
                    this.XLRibbon._openFATDlg();
                }
            }
        },

        _triggerTBtnClick: function (id) {
            var btnObj = $('#' + id).data("ejToggleButton");
            btnObj.buttontag.click();
        },

        _toggleFontPrpty: function (type) {
            var actCell = this.getActiveCell(), xlFormat = this.XLFormat, style = { "style": {} }, formatCode, fontPrpty, format;
            formatCode = this.XLEdit.getPropertyValue(actCell.rowIndex, actCell.colIndex, "format");
            format = !this.isUndefined(formatCode) && this.XLFormat.getFormatFromHashCode(formatCode);
            if (type === "bold")
                fontPrpty = "font-weight";
            else if (type === "italic")
                fontPrpty = "font-style";
            else
                fontPrpty = "text-decoration";
            if (format[fontPrpty] === type) {
                this.model.showRibbon && this.element.find("#" + this._id + "_Ribbon_" + type).ejToggleButton("option", "toggleState", false);
                style.style[fontPrpty] = "none";
                xlFormat.format(style);
            }
            else {
                this.model.showRibbon && this.element.find("#" + this._id + "_Ribbon_" + type).ejToggleButton("option", "toggleState", true);
                style.style[fontPrpty] = type;
                xlFormat.format(style);
            }
        },

        _getLocStr: function (prop) {
            var locale = ej.Spreadsheet.locale[this.model.locale];
            return locale && locale[prop] ? locale[prop] : ej.Spreadsheet.locale["en-US"][prop];
        },

        _checkIndicesInContainer: function (sheetIdx, rowIdx, colIdx, property) {
            var sheets = this._dataContainer.sheets;
            if (sheetIdx in sheets && rowIdx in sheets[sheetIdx] && colIdx in sheets[sheetIdx][rowIdx]) {
                if (property) {
                    if (property in sheets[sheetIdx][rowIdx][colIdx])
                        return true;
                    else
                        return false;
                }
                else
                    return true;
            }
            return false;
        },

        //Text Wrapping
        _textFormatting: function (operation) {
            switch (operation) {
                case 'wrapText':
                    this.setWrapText(null, "");
                    break;
            }
        },

        setWrapText: function (range, operation) {
            var selCell, cell, sheetIdx = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIdx), cellheight = 0, filterIcon, rows = this.getRows(sheetIdx)[1], rowhdrrows = this.getRows(sheetIdx)[0], aRange, j,
                rowHtColl = sheet.rowsHeightCollection, rowHt = 0, wrapHtColl = this.getSheet(sheetIdx)._wrapHtCollection, isWrap = false, rge;
            if (ej.isNullOrUndefined(range))
                selCell = sheet._selectedCells;
            else {
                var r = this.getRangeIndices(range);
                selCell = this.XLSelection._getSelectedRange({ rowIndex: r[0], colIndex: r[1] }, { rowIndex: r[2], colIndex: r[3] });
            }
            if(selCell.length > 1)
                aRange = this._getProperAlphaRange(sheetIdx, selCell[0].rowIndex, selCell[0].colIndex, selCell[selCell.length - 1].rowIndex, selCell[selCell.length - 1].colIndex)
            else if(selCell.length > 0) 
                aRange = this._getAlphaRange(sheetIdx, selCell[0].rowIndex, selCell[0].colIndex, selCell[selCell.length - 1].rowIndex, selCell[selCell.length - 1].colIndex);
            for (var i = 0, len = selCell.length; i < len; i++) {
                isWrap = this.XLEdit.getPropertyValue(selCell[i].rowIndex, selCell[i].colIndex, "wrap");
                if (isWrap) {
                    if (this._isRowViewable(sheetIdx, selCell[i].rowIndex)) {
                        cell = this.getCell(selCell[i].rowIndex, selCell[i].colIndex);
                        this._removeClass(cell[0], "e-sswraptext");
                        this._removeClass(rowhdrrows[selCell[i].rowIndex], "e-sswraprow");
                        operation = "unwrap";
                        cellheight = wrapHtColl[selCell[i].rowIndex];
                        if (cell.find(".e-filterspan").length > 0) {
                            filterIcon = cell.find(".e-filterspan");
                            this.model.allowFiltering && this.XLFilter._postionFilterIcon(cell, filterIcon);
                        }
                    }
                    delete this._dataContainer.sheets[sheetIdx][selCell[i].rowIndex][selCell[i].colIndex]["wrap"];
                    delete this._dataContainer.sheets[sheetIdx][selCell[i].rowIndex][0]["wrapRow"];
                }
                else {
                    if (this._isRowViewable(sheetIdx, selCell[i].rowIndex)) {
                        cell = this.getCell(selCell[i].rowIndex, selCell[i].colIndex);
                        operation = "wrap";
                        if (cell.is(":visible")) {
                            this.addClass(cell[0], "e-sswraptext");
                            !($(rowhdrrows[selCell[i].rowIndex]).hasClass("e-sswraprow")) && $(rowhdrrows[selCell[i].rowIndex]).addClass("e-sswraprow");
                            if (cell[0].rowSpan > 1)
                                for (j = selCell[i].rowIndex; j < selCell[i].rowIndex + cell[0].rowSpan; j++)
                                    rowHt = rowHt + rowHtColl[j];
                            if (cell.find(".e-filterspan").length > 0) {
                                filterIcon = cell.find(".e-filterspan");
                                this.model.allowFiltering && this.XLFilter._postionFilterIcon(cell, filterIcon);
                            }
                            cellheight = cell[0].offsetHeight;
                            if (cellheight <= rowHt)
                                cellheight = rowHt - cellheight;
                        }
                        wrapHtColl[selCell[i].rowIndex] = (i > 0 && (selCell[i].rowIndex === selCell[i - 1].rowIndex)) ? wrapHtColl[selCell[i - 1].rowIndex] : wrapHtColl[selCell[i].rowIndex] = $(rowhdrrows[selCell[i].rowIndex]).height();
                    }
                    this.XLEdit._updateDataContainer({ rowIndex: selCell[i].rowIndex, colIndex: selCell[i].colIndex }, { wrap: true });
                    if(ej.isNullOrUndefined(this.XLEdit.getPropertyValue(selCell[i].rowIndex, 0, "wrapRow")))
                        this.XLEdit._updateDataContainer({ rowIndex: selCell[i].rowIndex, colIndex: 0 }, { wrapRow: true });
                }
                if (cellheight > 0) {
                    rge = this.model.sheets[sheetIdx].selectedRange;
                    if (selCell[i].colIndex === rge[3]) {
                        if (this._isRowViewable(sheetIdx, selCell[i].rowIndex)) {
                            $(rowhdrrows[selCell[i].rowIndex]).height(cellheight);
                            $(rows[selCell[i].rowIndex]).height(cellheight);
                            rowHtColl[selCell[i].rowIndex] = cellheight;
                            if (this.getFrozenRows(sheetIdx) >= selCell[i].rowIndex) {
                                if (rowHt >= cellheight) {
                                    cellheight = rowHt - cellheight;
                                    this._refreshFreezeRows(sheetIdx, selCell[i].rowIndex, cellheight, "delete");
                                }
                                else {
                                    cellheight = cellheight - rowHt;
                                    this._refreshFreezeRows(sheetIdx, selCell[i].rowIndex, cellheight, "insert");
                                }
                            }
                        }
                    }
                    rowHtColl[selCell[i].rowIndex] = cellheight;
                }
                cellheight = 0;
            }
			this.model.allowComments && this.XLComment._updateCmntArrowPos(); //Comments position update
            if (!sheet._importRendered) {
                var details = { sheetIndex: sheetIdx, selectedCells: selCell, reqType: "textwrap", operation: operation, range: aRange };
                if(this.model.scrollSettings.allowScrolling)
					this.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
                this.XLSelection._refreshBorder();
                if (!this._isUndoRedo && !this._dupDetails)
                    this._completeAction(details);
            }
            if(this.model.scrollSettings.allowScrolling)
				this.XLScroll._getRowHeights(sheetIdx, selCell[0].rowIndex + 1);
        },
		
        _setWrapTextForImport: function (sheetIdx, rowIdx, colIdx, rows) {
            var tr, wrapHtColl = this.getSheet(sheetIdx)._wrapHtCollection;
            tr = rows[rowIdx];
            this.addClass(tr.cells[colIdx], "e-sswraptext");
            wrapHtColl[rowIdx] = 20;
            this._wrapRow = true;
        },

        _setWrapHeight: function (sheetIdx, rowIdx, detachedRows, rowHdrs, emptyTable, status) {
            var tr = detachedRows[rowIdx], sheet = this.getSheet(sheetIdx), hideRows = sheet.hideRowsCollection,
                isHidden = false, rowHtColl = sheet.rowsHeightCollection, newRow, rHt;
            isHidden = (hideRows.indexOf(rowIdx) > -1);
            if (rowHtColl[rowIdx] > 0)
                rHt = rowHtColl[rowIdx];
            else {
                if (!isHidden || status === "visible") {
                    newRow = $(detachedRows[rowIdx].outerHTML).attr("id", "ss");
                    emptyTable[0].insertBefore(newRow[0], emptyTable[0].childNodes[0]);
                    rHt = emptyTable[0].rows[0].getBoundingClientRect().height;
                    emptyTable[0].rows[0].parentNode.removeChild(emptyTable[0].rows[0]);
                }
            }
            rHt = rHt < 20 ? 20 : rHt;
            if (parseInt(tr.style.height) < rHt) {
                tr.style.height = rHt + "px";
                $(rowHdrs[rowIdx])[0].style.height = rHt + "px";
                sheet.rowsHeightCollection[rowIdx] = rHt;
            }
        },

        _setRowHdrHeightResize: function (sheetIdx, cellIndex) {
            var cell, shtRows = this.getRows(sheetIdx), rowHdrs = shtRows[0], rows = shtRows[1];
            for (var i = 0, len = rows.length; i < len; i++) {
                cell = this.getCell(i, cellIndex);
                if (cell[0].className.indexOf("e-rc") > 0 || cell.prop('rowspan') > 1 || cell.prop('colspan') > 1)
                    continue;
                $(rowHdrs[i]).height(cell[0].offsetHeight);
            }
        },

        _setRowHdrHeight: function (sheetIdx, rowIndex) {
            $(this.getRows(sheetIdx)[0][rowIndex]).height($(this.getRows(sheetIdx)[1][rowIndex]).height());
            //this.XLSelection.refreshSelection();
            //if (!this.model.sheets[sheetIdx]._importRendered && this.model.allowAutoFill)
            //    this.XLDragFill.positionAutoFillElement();
        },

        _refreshRangeTRHght: function (sheetIdx, range) {
            var sIdx = range[0] < 1 ? range[0] : range[0] - 1, i; // - 1 for border refresh
            for (i = sIdx; i <= range[2]; i++)
                this._setRowHdrHeight(sheetIdx, i);
            !this.model.sheets[sheetIdx]._importRendered && this.XLSelection._refreshBorder();
        },

        //Alert Dialog
        _renderAlertdlg: function () {
            var dlgId = this._id + "_alertdlg";
            if ($(dlgId).length)
                return;
            var $dlg = ej.buildTag("div#" + dlgId + ".e-alertdlg", "", { display: "none" }),
            contentdiv = ej.buildTag("div#" + dlgId + "_dlgcontent .e-alertdlgcontent"),
            btncontentdiv = ej.buildTag("div#" + dlgId + "_dlgbtncontent .e-dlg-btnfields"),
            confirmbtndiv = ej.buildTag("div#" + dlgId + "_condiv .e-dlg-btnctnr"),
            alertbtndiv = ej.buildTag("div#" + dlgId + "_alertdiv .e-dlg-btnctnr");
            $dlg.append(contentdiv);
            var okButton = ej.buildTag("input#" + this._id + "_dialog_Ok"),
            canButton = ej.buildTag("input#" + this._id + "_dialog_Can", "", { "margin-left": 10 }),
            alertokBtn = ej.buildTag("input#" + this._id + "_dialog_alertOk");
            okButton.ejButton({ text: this._getLocStr("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._alertOkClick, this), enabled: true });
            canButton.ejButton({ text: this._getLocStr("Cancel"), showRoundedCorner: true, width: 60, click: ej.proxy(this._alertCanClick, this) });
            alertokBtn.ejButton({ text: this._getLocStr("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._alertOkClick, this), enabled: true });
            btncontentdiv.append(confirmbtndiv.append(okButton, canButton));
            btncontentdiv.append(alertbtndiv.append(alertokBtn));
            $dlg.append(btncontentdiv);
            this.element.append($dlg);
        },

        _renderAlertDlgContent: function ($dlg, type, content, targetName, dlgWidth, className) {
            var contentdiv = $dlg.find("#" + this._id + "_alertdlg_dlgcontent");
            var confirmdiv = $dlg.find("#" + this._id + "_alertdlg_condiv");
            var alertdiv = $dlg.find("#" + this._id + "_alertdlg_alertdiv");
            dlgWidth = dlgWidth || 400;
            className = className || "";
            if (type === "Alert") {
                confirmdiv.hide();
                alertdiv.show();
            }
            else {
                confirmdiv.show();
                alertdiv.hide();
            }
            contentdiv.text("");
            contentdiv.html(content);
            $dlg.ejDialog({ showOnInit: false, enableModal: true, enableResize: false, allowKeyboardNavigation: false, title: this._getLocStr("Title"), width: dlgWidth, height: "auto", minHeight: 0, cssClass: "e-ss-dialog e-"+this._id+"-dlg" + className, open: ej.proxy(this._alertDlgOpen) });
            this._alertDialog = { element: $dlg, action: targetName };
        },

        _alertDlgOpen: function () {
            this.element.find("input:visible:first").focus();
        },

        _alertOkClick: function ($dlg) {
            var sheetIdx = this.getActiveSheetIndex(), shtModel;
            this._alertDialog.element.ejDialog("close");
            switch (this._alertDialog.action) {
                case "findPrevComment":
                    this.gotoPage(this.model.sheetCount, false);
                    shtModel = this.model.sheets[this.model.sheetCount];
                    this.XLComment._updateCurrentCell(shtModel.rowCount - 1, shtModel.colCount - 1);
                    this.XLComment.findPrevComment();
                    this._alertDialog.action = null;
                    break;
                case "findNextComment":
                    this.gotoPage(1, false);
                    this.XLComment._updateCurrentCell(0, 0);
                    this.XLComment.findNextComment();
                    this._alertDialog.action = null;
                    break;
                case "sheetRenameActn":
                    $("#" + this._id + "_Sheet_RenamePanel").select().focus();
                    break;
                case "sheetDeleteActn":
                    this._removeSheet(sheetIdx);
                    this._alertDialog.action = null;
                    break;
                case "New":               
                    this.importSheet = true;
                    this._destroy();
                    this.model.sheetCount = 1;
                    this.model.activeSheetIndex = 1;
                    this._renderSheets();
                    this.XLRibbon._toggleFormatTab();
                    this.XLRibbon._toggleChartDesignTab();
                    this._refreshSpreadSheetPager();
                    this.importSheet = false;
                    break;
                case "Open":
                    $("#" + this._id + "_file .e-uploadinput").click();
                    break;
                case "MissingParenthesis":
                    this.XLEdit._focusElements(this.getActiveCellElem(sheetIdx));
                    break;
                case "MergeCells":
                    this._mergeCells(sheetIdx, this.model.sheets[sheetIdx]._selectedCells, {}, []);
                    this._alertDialog.action = null;
                    break;
                case "MergeAcrossCells":
                    this._mergeAcrossCells(sheetIdx, this.model.sheets[sheetIdx]._selectedCells);
                    this._alertDialog.action = null;
                    break;
                case "CtrlKeyErrorAction":
                    this.setSheetFocus();
                    break;
				case "InvalidChartTitleActn":
                    $('#' + this._id + '_chartnamedlg').find("input:first").focus();
                    break;
            }
        },

        _alertCanClick: function ($dlg) {
			if (this._alertDialog.action == "findNextComment" || this._alertDialog.action == "findPrevComment")
                $(this.getActiveCellElem()).find('.e-comment-txtarea').css('display', 'inline-table');
            this._alertDialog.element.ejDialog("close");
            this.hideWaitingPopUp();
        },

        _showAlertDlg: function (type, prop, targetName, dlgWidth, className) {
            var alertDlg = $("#" + this._id + "_alertdlg"), alert = "Alert",
            text = (prop.indexOf("T-") === 0) ? prop.slice(2) : this._getLocStr(prop);
            if (type === alert)
                this._renderAlertDlgContent(alertDlg, alert, text, targetName, dlgWidth, className);
            else
                this._renderAlertDlgContent(alertDlg, "", text, targetName, dlgWidth, className);
            alertDlg.ejDialog("open");
        },

        _isSheetsDirty: function () {
            var i = 1, sheets = this._dataContainer.sheets, len = this.getObjectLength(sheets) + 1;
            if (this.model.nameManager.length)
                return true;
            while (i < len) {
                if (this.getObjectLength(sheets[i]))
                    return true;
                i++;
            }
            return false;
        },

        getCell: function (rowindex, columnindex, sheetIdx) {
            sheetIdx = this._getSheetIndex(sheetIdx);
            var sheet;
            if (this.model.scrollSettings.allowVirtualScrolling) {
                sheet = this.getSheet(sheetIdx);
                rowindex = sheet._virtualRowIdx.indexOf(rowindex);
                if (rowindex === -1)
                    return null;
            }
            if ((this.importSheet || this.model.sheets[sheetIdx]._importRendered || this._isPrint) && this._detachTable)
                return $(this._detachTable[0].tBodies[1].rows[rowindex].cells[columnindex]);
            else
                return $(this._getContTBody(sheetIdx)[0].rows[rowindex].cells[columnindex]);
        },

        getRange: function (startRIndex, startCIndex, endRIndex, endCIndex, sheetIdx, skipHiddenRow) {
            sheetIdx = this._getSheetIndex(sheetIdx);
            var i, rowIdx, rows, rowCells, temp, len, tempCells, sheet, cells = [];
            if ((this.importSheet || this.model.sheets[sheetIdx]._importRendered) && this._detachTable)
                rows = $(this._detachTable[0].tBodies[1].rows);
            else
                rows = $(this._getContTBody(sheetIdx)[0].rows);
            if (startRIndex > endRIndex) {
                temp = startRIndex;
                startRIndex = endRIndex;
                endRIndex = temp;
            }
            if (startCIndex > endCIndex) {
                temp = startCIndex;
                startCIndex = endCIndex;
                endCIndex = temp;
            }
            if (this.model.scrollSettings.allowVirtualScrolling) {
                sheet = this.getSheet(sheetIdx);
                for (i = startRIndex; i <= endRIndex; i++) {
                    rowIdx = sheet._virtualRowIdx.indexOf(i);
                    if (rowIdx > -1) {
                        rowCells = $(rows[rowIdx].cells);
                        tempCells = (endCIndex === startCIndex) ? rowCells.eq(endCIndex) : rowCells.slice(startCIndex, endCIndex + 1);
                        ej.merge(cells, tempCells);
                    }
                }
            }
            else {
                rows = (endRIndex !== startRIndex) ? rows.slice(startRIndex, endRIndex + 1) : rows.eq(endRIndex);
                len = rows.length;
                for (i = 0; i < len; i++) {
                    if (!skipHiddenRow || !this._isHiddenRow(i, sheetIdx)) {
                        tempCells = (endCIndex !== startCIndex) ? $(rows[i].cells).slice(startCIndex, endCIndex + 1) : $(rows[i].cells).eq(endCIndex);
                        ej.merge(cells, tempCells);
                    }
                }
            }
            return $(cells);
        },

        getRangeIndices: function (range) {
            range = range.indexOf(":") === -1 ? range + ":" + range : range;
            var rangeIndex = range.split(":"), rangeData, cells = [];
            rangeData = rangeIndex[0].split(/[A-Z]/i);
            cells.push(parseInt(rangeData[rangeData.length - 1]) - 1);
            cells.push(this._generateColCount(rangeIndex[0].split(/[0-9]/)[0].toUpperCase()) - 1);
            rangeData = rangeIndex[1].split(/[A-Z]/i);
            cells.push(parseInt(rangeData[rangeData.length - 1]) - 1);
            cells.push(this._generateColCount(rangeIndex[1].split(/[0-9]/)[0].toUpperCase()) - 1);
            this.swapRange(cells);
            return cells;
        },

        swapRange: function (cells) {
            if (cells[0] > cells[2])
                ej.dataUtil.swap(cells, 0, 2);
            if (cells[1] > cells[3])
                ej.dataUtil.swap(cells, 1, 3);
            return cells;
        },

        updateRange: function (sheetIdx, rangeSettings) {
            var dataSource = rangeSettings.dataSource, rangeData, startCell, range = [], data = [], obj;
            var rowCount = dataSource.length, colCount = this.getObjectLength(dataSource[0]);
            startCell = rangeSettings.startCell;
            rangeData = startCell.split(/[A-Z]/i);
            range.push(parseInt(rangeData[rangeData.length - 1]) - 1);
            range.push(this._generateColCount(startCell[0].split(/[0-9]/)[0].toUpperCase()) - 1);
            range.push(range[0] + rowCount - (rangeSettings.showHeader ? 0 : 1));
            range.push(range[1] + colCount - 1);
            this.XLScroll._refreshRowCol(range, sheetIdx);
            var rangeCells = this.getRange(range[0], range[1], range[2], range[3], sheetIdx);
            var contentTable = this._getContent(sheetIdx).find(".e-content >table");
            contentTable.detach();
            if (rangeSettings.showHeader) {
                for (obj in dataSource[0])
                    data.push(obj);
            }
            for (var r = 0; r < rowCount; r++) {
                if (dataSource[r] != undefined)
                    for (obj in dataSource[0])
                        data.push(dataSource[r][obj]);
            }
            for (var i = 0; i < rangeCells.length; i++) {
                this.XLEdit.updateCell($(rangeCells[i]), data[i], sheetIdx);
                if (rangeSettings.showHeader && i < colCount) {
                    range = rangeSettings.startCell + ":" + (this._generateHeaderText(sheetIdx, rangeCells[rangeCells.length - 1].cellIndex + 1) + ($(rangeCells[0]).parent().index() + 1));
                    if (rangeSettings.headerStyles)
                        this.XLFormat.format({ style: rangeSettings.headerStyles }, range);
                }
            }
            this._getContent(sheetIdx).find(".e-content").append(contentTable);

        },

        editRange: function (rangeName, fn) {
            var rangeInfo = this.getSheet(this.getActiveSheetIndex()).rangeSettings[rangeName], cell, content, i, j, len;
            var range = this.getRangeIndices(rangeInfo.range);
            var rangeCells = this.getRange(range[0], range[1], range[2], range[3]);
            this._trigger("editRangeBegin", { range: rangeInfo, sheetIndex: this.getActiveSheetIndex() });
            rangeInfo.cssClass && rangeCells.addClass(rangeInfo.cssClass);
            rangeInfo.readOnly && rangeCells.addClass("e-readonly");
            var contentTable = this._getContent(this.getActiveSheetIndex()).find(".e-content >table");
            contentTable.detach();
            for (i = 0, len = rangeCells.length; i < len; i++) {
                cell = rangeCells[i];
                $.trim(rangeName).length && this.XLEdit._updateDataContainer({rowIndex: cell.parentNode.rowIndex, colIndex: cell.cellIndex}, { range: rangeName });
                content = fn(cell, cell.cellIndex, i);
                if (typeof content === 'string')
                    this.XLEdit.updateCell($(cell), content);
                else if (typeof content === 'object') {
                    if (content.Value.indexOf("=") !== 0)
                        this.XLEdit.updateCell($(cell), content.Value);
                    else
                        this.XLEdit.updateCellValue($(cell), content.Value);
                    $(cell).data("value2", content.Value2);
                    $(cell).data("type", content.NumberFormat);
                    content.StyleIndex > 1 && $(cell).addClass(this._styleColl[content.StyleIndex]);
                }
            }
            this._getContent(this.getActiveSheetIndex()).find(".e-content").append(contentTable);
            for (j = range[0]; j < range[2] + 1 ; j++)
                this._setRowHdrHeight(this.getActiveSheetIndex(), j);
            this._trigger("editRangeComplete", { range: rangeInfo, sheetIndex: this.getActiveSheetIndex() });
        },

        removeRange: function (rangeName) {
            this.clearRange(rangeName);
            delete this.getSheet(this.getActiveSheetIndex()).rangeSettings[rangeName];
        },

        clearRange: function (rangeName) {
            var rangeInfo = this.getSheet(this.getActiveSheetIndex()).rangeSettings[rangeName];
            var range = this.getRangeIndices(rangeInfo.range);
            var rangeCells = this.getRange(range[0], range[1], range[2], range[3]);
            rangeInfo.cssClass && rangeCells.removeClass(rangeInfo.cssClass);
            rangeInfo.readOnly && rangeCells.removeClass("e-readonly");
            this.clearRangeData(null, ["range", "value", "type", "value2"], rangeCells);
            rangeCells.empty();
        },

        getObjectLength: function (obj) {
            var i, cnt = 0;
            obj = Object.prototype.toString.call(obj) === Object.prototype.toString() ? obj : {};
            if (Object.keys)
                return Object.keys(obj).length;
            for (i in obj)
                obj.hasOwnProperty(i) && cnt++;
            return cnt;
        },

        getObjectKeys: function (obj) {
            var i, keys = [];
            obj = Object.prototype.toString.call(obj) === Object.prototype.toString() ? obj : {};
            if (Object.keys)
                return Object.keys(obj);
            for (i in obj)
                obj.hasOwnProperty(i) && keys.push(i);
            return keys;
        },

        toArrayLowerCase: function (args) {
            for (var i = 0; i < args.length; i++)
                args[i] = args[i].toString().toLowerCase();
            return args;
        },

        _isValueAlreadyExist: function (arrObj, key, value) {
            var i = 0, length = arrObj.length;
            while (i < length) {
                if (arrObj[i][key] === value) return true;
                i++;
            }
            return false;
        },

        addClass: function (elem, classname) {
            if (!elem)
                return;
            var classes = elem.className;
            classes.indexOf(classname) === -1 && (elem.className = classes + " " + classname);
        },

        _removeClass: function (elem, className) {
            if (!elem)
                return;
            elem.className = elem.className.replace(new RegExp(className, "g"), '').replace(/ +/g, ' ');
        },

        _hasClass: function (elem, className) {
            if (!elem)
                return;
            return elem.className.indexOf(className) > -1;
        },

        isUndefined: function (val) {
            return val === undefined;
        },

        _isObject: function (obj) {
            return Object.prototype.toString.call(obj) === "[object Object]";
        },

        _isBool: function (val) {
            val = val + '';
            val = val.toUpperCase();
            return (val === "TRUE" || val === "FALSE");
        },

        isRange: function (range) {
            return range[0] !== range[2] || range[1] !== range[3];
        },

        inRange: function (range, rowIdx, colIdx) {
            return rowIdx >= range[0] && rowIdx <= range[2] && colIdx >= range[1] && colIdx <= range[3];
        },

        _isUniqueRange: function (oldRange, newRange) {
            return oldRange.toString() === newRange.toString();
        },

        _inRow: function(range, rowIdx){
            return rowIdx >= range[0] && rowIdx <= range[2];
        },

        hasSpan: function (cellIdx) {            
            return this.XLEdit.getPropertyValue(cellIdx.rowIndex, cellIdx.colIndex, "merge") ? true : false;
        },

        _getMergedIdx: function (rowIdx, colIdx) {
            var merge = this.XLEdit.getPropertyValue(rowIdx, colIdx, "merge");
            if (merge && merge.mSpan) {
                return { rowIndex: rowIdx + merge.mSpan.rowSpan, colIndex: colIdx + merge.mSpan.colSpan };
            }
            return { rowIndex: rowIdx, colIndex: colIdx };
        },

        _getCellInfo: function (cellIdx, sheetIdx) {
            var sheet = this.getSheet(this._getSheetIndex(sheetIdx));
            return {
                top: sheet._rowHeightCollection[cellIdx.rowIndex].value,
                left: sheet._colWidthCollection[cellIdx.colIndex].value,
                height: sheet.rowsHeightCollection[cellIdx.rowIndex],
                width: sheet.columnsWidthCollection[cellIdx.colIndex]
            };
        },

        _arrayAsString: function (arr) {
            return arr.join(" ").replace(/,/g, " ");
        },

        _rangeHasData: function (range) {
            var cells, i;
            range = this.swapRange(range);
            cells = this.getRange(range[0], range[1], range[2], range[3]);
            i = cells.length;
            while (i--) {
                if (cells[i].innerHTML.length)
                    return true;
            }
            return false;
        },

        _isHiddenRow: function (rowIdx, sheetIdx) {
            sheetIdx = this._getSheetIndex(sheetIdx);
            var col;
            if (!ej.isNullOrUndefined(this._dataContainer.sheets[sheetIdx][rowIdx]))
                col = this._dataContainer.sheets[sheetIdx][rowIdx][0];
            return (col && (col.isFilterHide || col.isRHide) || false);
        },

        getRangeDataAsObject: function (startcell, endcell, skipHiddenRow) {
            var i, columnName, celldata, temp, formatTemp, ctype = ej.Spreadsheet.CellType, container = this._dataContainer, sheetdata = container.sheets[this.getActiveSheetIndex()], minrindex, maxrindex, mincindex, maxcindex, rangeData = [], formattedRangeData = [], data = [];
            startcell.rowIndex > endcell.rowIndex ? (minrindex = endcell.rowIndex, maxrindex = startcell.rowIndex) : (minrindex = startcell.rowIndex, maxrindex = endcell.rowIndex);
            startcell.colIndex > endcell.colIndex ? (mincindex = endcell.colIndex, maxcindex = startcell.colIndex) : (mincindex = startcell.colIndex, maxcindex = endcell.colIndex);
            while (minrindex <= maxrindex) {
                if (!skipHiddenRow || !this._isHiddenRow(minrindex)) {
                    temp = {}, formatTemp = {};
                    for (i = mincindex; i <= maxcindex; i++) {
                        columnName = this.getSheet(this.getActiveSheetIndex()).columns[i].field;
                        if (minrindex in sheetdata && i in sheetdata[minrindex]) {
                            celldata = sheetdata[minrindex][i];
                            temp[columnName] = container.sharedData[celldata.value];
                            formatTemp[columnName] = [ctype.General, ctype.Text].indexOf(container.cellType[celldata.type]) === -1 ? celldata.value2 : container.sharedData[celldata.value2];
                        }
                    }
                    rangeData.push(temp);
                    formattedRangeData.push(formatTemp);
                }
                minrindex++;
            }
            data.push(rangeData);
            data.push(formattedRangeData);
            return data;
        },

        _getColorFromFormat: function (format) {
            var color = [], hashCode, fgcolor, bgcolor;
            hashCode = format.replace("e-format", "").toLowerCase();
            (hashCode.startsWith("6n")) ? fgcolor = "#" + hashCode.substr(2, 6) : fgcolor = "#" + hashCode.substr(6, 6);
            bgcolor = "#" + hashCode.substr(0, 6);
            color.push(bgcolor);
            color.push(fgcolor);
            return color;
        },

        getRangeData: function (range, valueonly, property, sheetIdx, skipDateTime, skipFormula, skipHiddenRow, virtualRowIdx, virtualRowCount, withRowIdx) {
            sheetIdx = sheetIdx ? sheetIdx : this.getActiveSheetIndex();
            property = property ? property : ["value", "value2", "type", "formatStr", "decimalPlaces", "thousandSeparator", "range", "format", "border", "comment", "hyperlink", "picture", "cFormatRule", "rule", "chart", "isLocked"];
            var i, prop, cellData, minc, obj, val, minr, maxr, maxc, isHiddenRow, rowIdx = [], arr = [], value = "value", skipVirtualHiddenRow = false,
                sheet = this.getSheet(sheetIdx), container = this._dataContainer, sheetdata = container.sheets[sheetIdx], sharedData = container.sharedData;
            range = range ? this.swapRange(range) : sheet.selectedRange;
            if (this.model.scrollSettings.allowVirtualScrolling && skipHiddenRow) {
                skipVirtualHiddenRow = true;
                minr = virtualRowIdx;
                maxr = minr + (virtualRowCount - 1);
                maxc = sheet.colCount - 1;
            }
            else {
                minr = range[0];
                maxr = range[2];
                maxc = range[3];
            }
            while (minr <= maxr) {
                isHiddenRow = this._isHiddenRow(minr, sheetIdx);
                if (skipVirtualHiddenRow && isHiddenRow)
                    maxr++;
                else if (!skipHiddenRow || !isHiddenRow) {
                    minc = skipVirtualHiddenRow ? 0 : range[1];
                    if (minr in sheetdata) {
                        if (valueonly) {
                            while (minc <= maxc) {
                                val = "";
                                if (minc in sheetdata[minr] && value in sheetdata[minr][minc]) {
                                    val = sharedData[sheetdata[minr][minc][value]];
                                    if (skipDateTime && this._isDateTime(val))
                                        val = this.XLEdit.getPropertyValue(minr, minc, "value2", sheetIdx);
                                    else if (skipFormula && this.isFormula(val))
                                        val = this.XLEdit.getPropertyValue(minr, minc, "calcValue", sheetIdx);
                                }
                                arr.push(val);
                                minc++;
                            }
                        }
                        else {
                            while (minc <= maxc) {
                                if (minc in sheetdata[minr]) {
                                    i = property.length;
                                    obj = {};
                                    cellData = sheetdata[minr][minc];
                                    while (i--) {
                                        prop = property[i];
                                        if (!this.isUndefined(cellData[prop]))
                                            obj[prop] = this.XLEdit.getPropertyValue(minr, minc, prop, sheetIdx);
                                    }
                                    arr.push(obj);
                                }
                                else
                                    arr.push({});
                                minc++;
                            }
                        }
                    }
                    else {
                        while (minc <= maxc) {
                            valueonly ? arr.push("") : arr.push({});
                            minc++;
                        }
                    }
                    rowIdx.push(minr);
                }
                minr++;
            }
            return withRowIdx ? { rowIdx: rowIdx, data: arr } : arr;
        },

        _processBindings: function (args) {
            this.model.query = new ej.Query();
            if (this._trigger("actionBegin", args))
                return true;
            this._ensureDataSource(args);
            args.action === "sorting" && this.XLSort._sortComplete(args);
            if (args.action === "sorting")
                !this._isUndoRedo && this._completeAction(args);
            else
                this._completeAction(args);
        },

        _ensureDataSource: function (args) {
            var sheetIndex = this.getActiveSheetIndex(), dataSource, queryManagar = this.model.query, predicate, predicates, obj, result;
            if (args.action === "filtering" || args.action === "clearfiltering" || args.requestType === "filterbycell") {
                dataSource = (this.model.sheets[sheetIndex].filterSettings.filteredRange == null) ? this._dataSource(sheetIndex) : this.model.sheets[sheetIndex].filterSettings.filteredRange;
                var tblrange = this.model.sheets[sheetIndex].filterSettings.tableRange;
                for (var i = 0, len = tblrange.length; i < len; i++) {
                    if (tblrange[i].tableID === args.tableID) {
                        tblrange[i].filteredRange = dataSource;
                        break;
                    }
                }
                this._dataManager = ej.DataManager(dataSource);
                if (!(dataSource instanceof ej.DataManager))
                    this.model.currentViewData = dataSource;
                this.model.query.requiresCount();
                queryManagar = this.model.query;
                if (args.filteredcolumns.length) {
                    predicates = this._excelFilter._predicates[sheetIndex];
                    var dataSrc = dataSource[0];
                    for (var prop in predicates) {
                        (dataSrc.hasOwnProperty(prop)) && (obj = predicates[prop]);
                        obj = predicates[prop];
                        predicate = predicate != undefined ? predicate["and"](obj) : obj;
                    }
                    (!ej.isNullOrUndefined(predicate)) && queryManagar.where(predicate);
                    if (!(dataSource instanceof ej.DataManager)) {
                        this._filteredRecordsCount = this._dataManager.executeLocal(queryManagar).count;
                        result = this._dataManager.executeLocal(queryManagar);
                        this.model.currentViewData = result.result;
                    }
                }
            }
            else if (args.requestType === "sorting") {
                this._dataManager = ej.DataManager(args.rangeData);
                !(args.rangeData instanceof ej.DataManager) && (this.model.currentViewData = args.rangeData);
                queryManagar.sortBy(args.columnName, args.sortDirection);
                if (!(args.rangeData instanceof ej.DataManager)) {
                    result = this._dataManager.executeLocal(queryManagar);
                    this.model.currentViewData = result;
                }
            }
        },

        _toArrayData: function (args) {
            var prop, obj, i = 0, temp = [], len = args.length;
            while(i < len){
                obj = args[i];
                if (this.getObjectLength(obj)) {
                    for (prop in obj)
                        temp.push(obj[prop]);
                }
                else
                    temp.push("");
                i++;
            }
            return temp;
        },

        updateRangeData: function (args, range, data) {
            var cells = this.XLSelection._getSelectedRange(range.startCell, range.endCell);
            var dataArray = this._toArrayData(data);
            if (dataArray.length === cells.length) {
                for (var i = 0; i < cells.length; i++) {
                    if (!ej.isNullOrUndefined(dataArray[i])) {
                        this.XLEdit.updateCellValue(this.getCell(cells[i].rowIndex, cells[i].colIndex), dataArray[i]);
                        if (!ej.isNullOrUndefined(args.filterIcon))
                            for (var j = 0; j < args.filterIcon.length; j++)
                                if ((args.filterIcon[j].rowIndex === cells[i].rowIndex) && (args.filterIcon[j].colIndex === cells[i].colIndex)) {
                                    this.getCell(cells[i].rowIndex, cells[i].colIndex).append(args.filterIcon[j].filterIcon);
                                    break;
                                }
                    }
                }
            }
        },

        updateData: function (data, range) {
            range = range ? this.swapRange(range) : this.getSheet(this.getActiveSheetIndex()).selectedRange;
            var cell, i, val, hlData, $cell, cells = $.makeArray(this.getRange(range[0], range[1], range[2], range[3])).reverse(), rindex, cindex;
            if (data.length === cells.length) {
                i = data.length;
                data = data.reverse();
                while (i--) {
                    cell = cells[i];
                    rindex = cell.parentNode.rowIndex, cindex = cell.cellIndex;
                    this.XLEdit._updateDataContainer({rowIndex: rindex, colIndex: cindex}, data[i]);
                    if ("hyperlink" in data[i]) {
                        hlData = data[i].hyperlink;
                        $cell = $(cell);
                        if (!this.isUndefined(hlData))
                            hlData.webAddr ? ($cell.html(ej.buildTag("a", data[i].value2, "", { href: hlData.webAddr, target: "_blank", "class": "e-hyperlinks" }))) : ($cell.html(ej.buildTag("a", data[i].value2, "", { href: "#", "class": "e-hyperlinks" })));
                    }
                    if ("comment" in data[i])
                        this.model.allowComments && this.XLComment._updateCmntFrmCntnr(null, null, $(cell));
                    if ("value" in data[i]) {
                        val = data[i].value;
                        this.XLEdit._refreshCalcEngine(rindex, cindex, this.isFormula(val), val);
                    }
                }
            }
        },

        updateUniqueData: function (data, range) {
            var minr, maxr, minc, maxc;
            range = range ? this.swapRange(range) : this.getSheet(this.getActiveSheetIndex()).selectedRange;
            for (minr = range[0], maxr = range[2]; minr <= maxr; minr++) {
                for (minc = range[1], maxc = range[3]; minc <= maxc; minc++){
                    this.XLEdit._updateDataContainer({ rowIndex: minr, colIndex: minc }, data);
                }
            }
        },

        clearRangeData: function (range, property, cells, skipHiddenRow) {
            var i, j, isText = false, val, isFormula, sheetIdx = this.getActiveSheetIndex(), sheetdata = this._dataContainer.sheets[sheetIdx], rindex, cindex;
            property = property ? property : ["value", "value2", "type", "formatStr", "decimalPlaces", "thousandSeparator", "hyperlink"];
            if (!cells) {
                range = range ? this.swapRange(range) : this.getSheet(this.getActiveSheetIndex()).selectedRange,
                cells = this.XLSelection._getSelectedRange({rowIndex: range[0], colIndex: range[1]}, {rowIndex: range[2], colIndex: range[3]});
            }
            i = cells.length;
            while (i--) {
                rindex = cells[i].rowIndex, cindex = cells[i].colIndex;
                if ((!skipHiddenRow || !this._isHiddenRow(rindex)) && rindex in sheetdata && cindex in sheetdata[rindex]) {
                    val = this.XLEdit.getPropertyValue(rindex, cindex, "value");
					val && (isText = true);
                    if (this.isFormula(val)) {
                        isFormula = true;
                        val = "";
                    }
                    else
                        isFormula = false;
                    j = property.length;
                    while (j--)
                        delete sheetdata[rindex][cindex][property[j]];
                    if (property.indexOf("value2") > -1 && this._isRowViewable(sheetIdx, rindex))
                        cells[i].innerHTML = "";
                    !this.getObjectLength(sheetdata[rindex][cindex]) && (delete sheetdata[rindex][cindex]);
                    !this.getObjectLength(sheetdata[rindex]) && (delete sheetdata[rindex]);
                    if (property.indexOf("value") > -1) {
                        this.XLEdit._refreshCalcEngine(rindex, cindex, isFormula, val);
                        this.model.allowCharts &&  this.XLChart._refreshChartElements(rindex, cindex, sheetIdx);
                    }
					if(isText && !this._dupDetails){
						this._textClip(rindex, cindex, 'delete');
						isText = false;
					} 
                }
            }
        },

        _getArrayRangeCells: function (range) {  // return cells if range specified else selected cells will returned
            return range ? (range = this.swapRange(range), this.getRange(range[0], range[1], range[2], range[3])) : this._getContent(this.getActiveSheetIndex()).find(".e-selected");
        },

        _getAlphaRangeCells: function (range) { // get range cells by alpha range input
            return range ? (range = this.getRangeIndices(range), this.getRange(range[0], range[1], range[2], range[3])) : this._getContent(this.getActiveSheetIndex()).find(".e-selected");
        },

        _completeAction: function (args) {
            //this._trigger("actionComplete", args);
            var sheetIndex = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIndex), details, fltrdIdxes, fltrdVal;
            if (args.reqType === "filter") {
                if (args.operation !== "apply-filter" && args.operation !== "remove-filter") {
					fltrdVal = args.model.currentViewData;
                    details = {
                        sheetIndex: sheetIndex,
                        range: sheet._selectedRange,
                        currentViewData: fltrdVal,
                        fieldName: args.currentFilteringColumn,
                        ejpredicate: args.ejpredicate,
                        activeCell: args.activeCell,
                        filteredValue: args.filteredValue,
                        filterCollection: args.filteredcolumns,
                        requestType: args.requestType,
                        reqType: args.reqType,
                        selCells: args.selCells,
                        action: args.action,
                        operation: "filter",
                        selectedDataSource: sheet.selectedDataSource,
                        tableID: args.tableID,
						filteredCells: args.filteredCells,
						fltrdIdxes: [],
                    };
					for (i = 0, len = fltrdVal.length; i < len; i++)
						details.fltrdIdxes.push(details.range.startRow + sheet.filterSettings.filteredRange.indexOf(fltrdVal[i]));
                }
                else if(args.operation === "remove-filter")
					details = args;
				else
                    details = { sheetIndex: sheetIndex, filteredCells: args.filteredCells, reqType: "filter", operation: "apply-filter", tableID: args.tableID, fRange: this.getSheet(sheetIndex).filterSettings.range };
            }
            else if (args.requestType === ej.Spreadsheet.Actions.Sorting) {               
                var prevIcon, currIcon, fltrspan = this.getCell(args.filterIcon.length > 0 ? args.range[0] - 1 : args.range[0], this.XLEdit.getColumnIndexByField(args.columnName)).find(".e-filterspan");
                if (fltrspan[0]) {
                    if (fltrspan[0].className.indexOf("-asc") > -1)
                        prevIcon = "-asc";
                    else if (fltrspan[0].className.indexOf("-dsc") > -1)
                        prevIcon = "-dsc";
                    else
                        prevIcon = "";
                } else
                    prevIcon = "";
                currIcon = args.sortDirection === "ascending" ? "-asc" : "-dsc";
                details = { sheetIndex: sheetIndex, prevIcon: prevIcon, isSortByFilter: args.isSortByFilter, colSelected: args.colSelected, currIcon: currIcon, range: args.range, columnName: args.columnName, rangeData: args.rangeData, reqType: "sort", direction: args.sortDirection, operation: "unsort" };
            }
            else if (args.requestType === "sortbycolor" || args.requestType === "filterbycolor") {
                details = { sheetIndex: sheetIndex, operation: args.operation, requestType: args.requestType, action: args.action, bgcolor: args.bgcolor, fgcolor: args.fgcolor, tableID: args.tableID, reqType: args.requestType };
                details.columnName = (args.requestType === "sortbycolor") ? args.sortColumn : args.filterColumn;
				var color;
				if(args.requestType === "filterbycolor"){
					for (i = range.endRow; i >= range.startRow; i--) {
						color = this._getColorFromFormat(this.XLEdit.getPropertyValue(i, this._generateColCount(details.columnName) - 1, "format"));
						if (!ej.isNullOrUndefined(color)) {
							if (args.operation === "filterbgcolor")
								(args.bgcolor === color[0].toLowerCase()) && fltrdIdxes.push(i);
							else
								(args.fgcolor === color[1].toLowerCase()) && fltrdIdxes.push(i);
						}
					}
					details.fltrdIdxes = fltrdIdxes;
				}
			}
            else
                details = args;
            if(this.model.allowUndoRedo  && !(details.requestType === "sortbycolor"))
                this._updateUndoRedoCollection(details);
            (args.operation !== "apply-filter" && args.operation !== "remove-filter") && this.sendDataRenderingRequest(details, this.getActiveSheetIndex());
            if(this.model.allowAutoFill && args.reqType !== "find-replace")
                this.XLDragFill.positionAutoFillElement();
            this._updateUndoRedoIcons();
            this._trigger("actionComplete", args);
        },

        _updateUndoRedoCollection: function (details) {
            if (!this._isUndoRedo) {
                this._undoCollection.push(details);
                this._redoCollection = [];
            }
            if (this._undoCollection.length > this._undoredoStep)
                this._undoCollection.splice(0, 1);
        },

        _trigActionBegin: function (args) {
            return this._trigger("actionBegin", args);
        },

        _trigActionComplete: function (args) {
            return this._trigger("actionComplete", args);
        },

        _updateUndoRedoIcons: function () {
            if (!this.model.showRibbon || !this.model.allowUndoRedo)
                return;
            var args, undoBtn = $("#" + this._id + "_Ribbon_undo").data("ejButton"), redoBtn = $("#" + this._id + "_Ribbon_redo").data("ejButton");
            this._undoCollection.length ? undoBtn.enable() : undoBtn.disable();
            this._redoCollection.length ? redoBtn.enable() : redoBtn.disable();
            if (!this.XLEdit._isEdit && !(this.XLComment && this.XLComment._isCommentEdit) && !this._isSheetRename && (document.activeElement && document.activeElement.id.indexOf(this._id + "_Ribbon") < 0))
                this.setSheetFocus();
            args = { reqType: "picture", action: "add" };
            this._trigActionComplete(args);
        },

        _filterMergeCells: function (row, status) {
            var rcregx = this._rcRegx, rc, mCell, isMerge = false, mCells = row.find('.e-mc-hide');
			for(var i = 0, len = mCells.length; i < len; i++) {
				rc = mCells[i].className.match(rcregx)[0].split(" ")[0].replace("e-rc-", "").split(":");
				isMerge = true;
			}   
            if (isMerge) {
                mCell = this.getCell(rc[0], rc[1])[0];
				mCell.rowSpan = (status === "incr") ? (mCell.rowSpan + 1) : (mCell.rowSpan - 1);
            }			
        },

		_selectFirstCell: function(sheetIdx, rowIdx, colIdx, isSelect) {
			if(!this.getSheet(sheetIdx)._importRendered){
				if(!isSelect) {
					isSelect = !isSelect;
					this.performSelection({rowIndex: rowIdx, colIndex: colIdx}, {rowIndex: rowIdx, colIndex: colIdx});
				}
			}
			return isSelect;
		},

		_refreshMergeCells: function(rowIdx, status) {
		    var sheetIdx = this.getActiveSheetIndex(), mCells = this.getSheet(sheetIdx)._mergeColl, mergeObj , mRange, range, container = this._dataContainer.sheets[sheetIdx];
		    if(rowIdx in mCells){
		        mKeys = this.getObjectKeys(mCells[rowIdx]);
		        for (var i = 0, len = mKeys.length; i < len; i++) {
		            mergeObj = this.XLEdit.getPropertyValue(rowIdx, mKeys[i], "merge");
		            if (mergeObj.mRange) {
		                this.unmergeCells(mergeObj.mRange);
		                range = this.getRangeIndices(mergeObj.mRange);
		                range[0] = (status === "hide") ? range[0] + 1 : range[0] - 1;
		                mRange = this._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]);
		                if (status === "hide") {
		                    container[range[0] - 1][range[1]].merge.mRange = mRange;
		                }
		                else {
		                    if (!ej.isNullOrUndefined(container[range[0] + 1][range[1]]))
		                        delete container[range[0] + 1][range[1]]["merge"]["mRange"];
		                }
		                (range[0] < range[2]) && this.mergeCells(mRange, true);
		            }
		        }
		    }		    
		},
		
		_refreshAllMergeCells: function(sheetIdx, rowIdx, rowCount, colIdx, colCount, status) {
			var mCells = this.getSheet(sheetIdx)._mergeColl, mRange, rKeys, cKeys, range, container = this._dataContainer.sheets[sheetIdx];
			this._dupDetails = true;
			rKeys = this.getObjectKeys(mCells);
			for (var i = 0, len = rKeys.length; i < len; i++) {
			    cKeys = this.getObjectKeys(mCells[rKeys[i]]);
			    for (var j = 0; j < cKeys.length; j++) {
			        mergeObj = this.XLEdit.getPropertyValue(rKeys[i], cKeys[j], "merge");
			        range = this.getRangeIndices(mergeObj.mRange);
			        if (((rowIdx > -1) && (rowIdx <= range[0])) || ((colIdx > -1) && (colIdx <= range[1]))) {
			            if (!ej.isNullOrUndefined(mRange)) {
			                if (rowIdx > -1) {
			                    if ((rowIdx + 1) <= range[0])
			                        range[0] = (status === "insert") ? range[0] + rowCount : range[0] - rowCount;
			                    range[2] = (status === "insert") ? range[2] + rowCount : range[2] - rowCount;
			                }
			                if (colIdx > -1) {
			                    if (colIdx + 1 <= range[1])
			                        range[1] = (status === "insert") ? range[1] + colCount : range[1] - colCount;
			                    range[3] = (status === "insert") ? range[3] + colCount : range[3] - colCount;
			                }
			                mRange = this._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]);
			                this.unmergeCells(mRange);
			                this.mergeCells(mRange, true);
			            }
			        }
			    }		   
			}
			this._dupDetails = false;
		},
		
        sendDataRenderingRequest: function (args, sheetIdx) {
            var i, j, k, isSelect = false, len, ln, ralign = this._rAlign, rcregx = this._rcRegx, rowsColl = this.getRows(sheetIdx), rowHdrs = rowsColl[0], rows = rowsColl[1], sheet = this.getSheet(sheetIdx),
                range = sheet._selectedRange, colIdx, format = "", fltrdIdxes, cellIdx, columns, isClear, activeCell, filteredRowsColl = sheet.filteredRowsCollection, hideRowsColl = sheet.hideRowsCollection,
                multiFltrIdx, tabIdx, tab = sheet.filterSettings.tableRange, regx = this._formatRegx, hiddenIdx = [], avble = false, filterColumn, predicate, valColl = [], filtered, hCell, mClassName, isVirtualScroll = this.model.scrollSettings.allowVirtualScrolling,
                isMerge, mCell, rowCount = sheet.rowCount, hRowCount = 0, defRowCount = sheet._defRowCount, frHiddenHeight = sheet._frHiddenHeight, stRow;
            for (i = 0, len = tab.length; i < len; i++) {
                if (tab[i].tableID == args.tableID) {
                    tabIdx = i;
                    break;
                }
            }
            this._dupDetails = true;
            if (!(ej.isNullOrUndefined(args)) && (args.reqType == "filter")) {
                cellIdx = this._generateColCount(args.fieldName) - 1;
                range = args.range;
                hiddenIdx = tab[tabIdx].hiddenIdx;
                tab[tabIdx].fltrdIdxes = fltrdIdxes = args.fltrdIdxes;
                if (args.requestType.indexOf("filtering") > -1 || args.requestType.indexOf("filterbycell") > -1 || args.action.indexOf("clearfiltering") === -1) {
                    stRow = (range.startRow === 0) ? 0 : range.startRow;
                    for (k = stRow; k <= range.endRow; k++) {
                        filtered = false;
                        for (j = 0, ln = fltrdIdxes.length; j < ln; j++)
                            (fltrdIdxes[j] == k) && (filtered = true);
                        if (filtered) {
                            if (!ej.isNullOrUndefined(this._dataContainer.sheets[sheetIdx][k])) {
                                if (!ej.isNullOrUndefined(this._dataContainer.sheets[sheetIdx][k][0])) 
                                    delete this._dataContainer.sheets[sheetIdx][k][0]["isFilterHide"];
                            }
                            if (!isVirtualScroll) {
                                this._removeClass(rows[k], "e-hide");
                                this._removeClass(rowHdrs[k], "e-hide");
                                this._refreshMergeCells(k, "show");
                                rowHt = $(rows[k]).height();
                                frHiddenHeight = frHiddenHeight - rowHt;
                                this._filterMergeCells($(rows[k]), "incr");
                                (hiddenIdx.indexOf(k)) && hiddenIdx.splice(hiddenIdx.indexOf(k), 1);
                                (filteredRowsColl.indexOf(k)) && filteredRowsColl.splice(filteredRowsColl.indexOf(k), 1);
                                this._refreshFreezeRows(sheetIdx, k, rowHt, "unhide");
                                if (!this.getSheet(sheetIdx)._importRendered) {
                                    if (!isSelect) {
                                        isSelect = !isSelect;
                                        this.performSelection({ rowIndex: k, colIndex: cellIdx }, { rowIndex: k, colIndex: cellIdx });
                                    }
                                }
                            }
                        }
                        else {
                            if(ej.isNullOrUndefined(this.XLEdit.getPropertyValue(k, 0, "isFilterHide")))
                                this.XLEdit._updateDataContainer({ rowIndex: k, colIndex: 0 }, { "isFilterHide": true });
                            if (!isVirtualScroll) {
                                if (!this._hasClass(rows[k], "e-hide")) {
                                    this._filterMergeCells($(rows[k]), "decr");
                                    rowHt = $(rows[k]).height();
                                    frHiddenHeight = frHiddenHeight + rowHt;
                                    this._refreshFreezeRows(sheetIdx, k, rowHt, "hide");
                                    this._refreshMergeCells(k, "hide");
                                }
                                this.addClass(rows[k], "e-hide");
                                this.addClass(rowHdrs[k], "e-hide");
                                (hiddenIdx.indexOf(k) === -1) && hiddenIdx.push(k);
                                (filteredRowsColl.indexOf(k) === -1) && filteredRowsColl.push(k);
                                hiddenRowColl = filteredRowsColl.length + hideRowsColl.length;
                                hRowCount = (hiddenRowColl > rowCount) ? (hiddenRowColl - rowCount) : (rowCount - hiddenRowColl);
                                if (defRowCount >= hRowCount && this.model.scrollSettings.allowScrolling)
                                    this.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                            }
                        }
                    }
                    if (isVirtualScroll) {
                        if (!this.getSheet(sheetIdx)._importRendered) {
                            this._refreshViewport(this.getSheet(sheetIdx)._scrollTop, sheetIdx);
                            this.performSelection({ rowIndex: fltrdIdxes[0], colIndex: cellIdx }, { rowIndex: fltrdIdxes[0], colIndex: cellIdx });
                        }
                    }
                    columns = this.model.sheets[sheetIdx].columns;
                    if (args.action === "clearfiltering") {
                        this.model.sheets[sheetIdx]._filteredByColor = -1;
                        this._excelFilter.closeXFDialog();
                    }
                    stRow = (range.startRow === 0) ? 0 : range.startRow - 1;
                    this.XLFilter._updateFilterIcons(stRow, range.startRow, cellIdx, args.action);
                    tab[tabIdx].hiddenIdx = $.extend(true, [], hiddenIdx);
                    hiddenIdx = [];
                }
			
            }
            else if (!(ej.isNullOrUndefined(args)) && (args.requestType === "sortbycolor" || args.requestType === "filterbycolor")) {
                var startRow = range.startRow, count = [], color, tableFilteredColumn = [], newIdx, cellIdx = this._generateColCount(args.columnName) - 1, rng, rangeData, cells;
                args.prev = {}; args.cur = {};
				if(this._filterColl[range.startRow - 1][cellIdx].indexOf("e-ssfilter") > -1 && args.requestType === "filterbycolor"){
					if(this._isRowViewable(sheetIdx, range.startRow - 1)) {
						activeCell = this.getCell(range.startRow - 1, cellIdx);
						activeCell.removeClass(ralign);
						this.XLFilter._changeFilterIcon({rowIndex: range.startRow - 1, colIndex: cellIdx}, "filter");
					}
                }
                else if (this._filterColl[range.startRow - 1][cellIdx].indexOf("-asc")) {					
                    args.prev.icon = this._filterColl[range.startRow - 1][cellIdx].split(" ")[3];
					if(this._isRowViewable(sheetIdx, range.startRow - 1)) {
						activeCell = this.getCell(range.startRow - 1, cellIdx);
						this.XLFilter._changeSortIcon({rowIndex: range.startRow - 1, colIndex: cellIdx}, "ascending");
					}
                    args.cur.icon = this._filterColl[range.startRow - 1][cellIdx].split(" ")[3];
                }
                if (args.requestType === "sortbycolor") {
                    rng = [range.startRow, cellIdx, range.endRow, cellIdx], rangeData = this.getRangeData(rng, null, null, null, null, null, true);
                    for (var i = 0; i < rangeData.length; i++) {
                        if (!ej.isNullOrUndefined(rangeData[i].format) && !ej.isNullOrUndefined(rangeData[i].format.match(regx))) {
                            color = this._getColorFromFormat(rangeData[i].format.match(regx)[0]);
                            if (args.operation === "sortbgcolor" || args.operation === "filterbgcolor")
                                (args.bgcolor === color[0].toLowerCase()) && count.push(i);
                            else
								(args.fgcolor === color[1].toLowerCase()) && count.push(i);
                        }
                    }
					var multifilterIdx, remData, totData, j, c, len;
					if(this.isUndefined(args.tableID)) {
						multifilterIdx = [];
						for(var col = rng[1]; col <= rng[3]; col++)
							multifilterIdx.push(col);
					}
					else
						multifilterIdx = tab[tabIdx].multifilterIdx;
					rng = [range.startRow, multifilterIdx[0], range.endRow, multifilterIdx[multifilterIdx.length - 1]];
					rangeData = this.getRangeData(rng, null, null, null, null, null, true);
					args.prev.data = $.extend(true, [], rangeData);
					rng.colIndex = cellIdx;
					args.range = rng;
					for(j = 0, len = multifilterIdx.length; j < len; j++) {
						remData = [], totData = [];
						rng = [range.startRow, multifilterIdx[j], range.endRow, multifilterIdx[j]];
						rangeData = this.getRangeData(rng, null, null, null, null, null, true);
						for (c = count.length - 1; c >= 0; c--)
							remData.push(rangeData.splice(count[c], 1)[0]);
						totData = remData.reverse().concat(rangeData);
						if(remData.length > 0) {
							this._isSort = true;
							this.XLComment.deleteComment(rng, null, true);
							this._removeHyperlink(null, null, true);
							this.clearRangeData(rng, ["value", "value2", "type", "formatStr", "decimalPlaces", "thousandSeparator", "comment", "hyperlink", "format"], null, true);
							this.XLFormat.clearFormat(rng, "format", true);
							this._isSort = false;
							this.XLEdit.updateCellWithContainer(totData, rng, sheetIdx, true);
						}
					}
                    args.cur.data = $.extend(true, [], this.getRangeData(rng, null, null, null, null, null, true));
                    this._updateUndoRedoCollection(args);
                }
                else {
                    tableFilteredColumn = [], hiddenIdx = [];
                    tab[tabIdx].fltrdIdxes = fltrdIdxes = args.fltrdIdxes;
					stRow = (range.startRow === 0) ? 0 : range.startRow;
                    for (k = stRow ; k <= range.endRow; k++) {
                        filtered = false;
                        for (j = 0, ln = fltrdIdxes.length; j < ln; j++)
                            (fltrdIdxes[j] == k) && (filtered = true);
                        if (filtered) {
                            filterColumn = { field: args.columnName, value: $(rows[k].cells[cellIdx]).text(), predicate: "or", operator: "equal", matchcase: false };
                            predicate = ej.Predicate(args.columnName, "equal", $(rows[k].cells[cellIdx]).text());
                            if (!ej.isNullOrUndefined(this._dataContainer.sheets[sheetIdx][k])) {
                                if (!ej.isNullOrUndefined(this._dataContainer.sheets[sheetIdx][k][0])) 
                                    delete this._dataContainer.sheets[sheetIdx][k][0]["isFilterHide"];
                            }
                            if (!isVirtualScroll) {
                                this._removeClass(rows[k], "e-hide");
                                this._removeClass(rowHdrs[k], "e-hide");
                                this._refreshMergeCells(k, "show");
                                this._filterMergeCells($(rows[k]), "incr");
                                (hiddenIdx.indexOf(k)) && hiddenIdx.splice(hiddenIdx.indexOf(k), 1);
                                (filteredRowsColl.indexOf(k)) && filteredRowsColl.splice(filteredRowsColl.indexOf(k), 1);
                                rowHt = $(rows[k]).height();
                                frHiddenHeight = frHiddenHeight - rowHt;
                                this._refreshFreezeRows(sheetIdx, k, rowHt, "unhide");
                                tableFilteredColumn.push(filterColumn);
                                valColl.push(filterColumn);
                                if (!this.getSheet(sheetIdx)._importRendered) {
                                    if (!isSelect) {
                                        isSelect = !isSelect;
                                        this.performSelection({ rowIndex: k, colIndex: cellIdx }, { rowIndex: k, colIndex: cellIdx });
                                    }
                                }
                            }
                        }
                        else {
                            if (!ej.isNullOrUndefined(this._dataContainer.sheets[sheetIdx][k])) {
                                if (!ej.isNullOrUndefined(this._dataContainer.sheets[sheetIdx][k][0])) 
                                    delete this._dataContainer.sheets[sheetIdx][k][0]["isFilterHide"];
                            }
                            this.XLEdit._updateDataContainer({ rowIndex: k, colIndex: 0 }, { "isFilterHide": true });
                            if (!isVirtualScroll) {
                                this._filterMergeCells($(rows[k]), "decr");
                                rowHt = $(rows[k]).height();
                                frHiddenHeight = frHiddenHeight + rowHt;
                                this._refreshFreezeRows(sheetIdx, k, rowHt, "hide");
                                this._refreshMergeCells(k, "hide");
                                this.addClass(rows[k], "e-hide");
                                this.addClass(rowHdrs[k], "e-hide");
                                (hiddenIdx.indexOf(k) === -1) && hiddenIdx.push(k);
                                (filteredRowsColl.indexOf(k) === -1) && filteredRowsColl.push(k);
                                hiddenRowColl = filteredRowsColl.length + hideRowsColl.length;
                                hRowCount = (hiddenRowColl > rowCount) ? (hiddenRowColl - rowCount) : (rowCount - hiddenRowColl);
                                if (defRowCount >= hRowCount && this.model.scrollSettings.allowScrolling)
                                    this.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                            }
                        }
                    }
                    if (isVirtualScroll) {
                        if (!this.getSheet(sheetIdx)._importRendered) {
                            this._refreshViewport(this.getSheet(sheetIdx)._scrollTop, sheetIdx);
                            this.performSelection({ rowIndex: fltrdIdxes[0], colIndex: cellIdx }, { rowIndex: fltrdIdxes[0], colIndex: cellIdx });
                        }
                    }
                    this.XLFilter._changeFilterIcon({rowIndex: range.startRow - 1, colIndex: cellIdx}, "filter");
                    var firstVal = valColl[0], isTake = firstVal["from"];
                    if (!ej.isNullOrUndefined(value)) {
                        predicate = firstVal["ejpredicate"] ? firstVal["ejpredicate"] : ej.Predicate(firstVal.field, firstVal.operator, firstVal.value, !firstVal.matchcase);
                        for (j = 1; j < valColl.length; j++)
                            predicate = valColl[j].ejpredicate != undefined ? predicate[valColl[j].predicate](valColl[j].ejpredicate) : predicate[valColl[j].predicate](valColl[j].field, valColl[j].operator, valColl[j].value, !valColl[j].matchcase);
                        if (ej.isNullOrUndefined(tab[tabIdx].predicate[args.columnName]))
                            tab[tabIdx].predicate[args.columnName] = {};
                        tab[tabIdx].filteredColumns = tableFilteredColumn;
                        tab[tabIdx].predicate[args.columnName] = !isTake ? predicate : { from: firstVal["from"], take: firstVal["take"] };
                        tab[tabIdx].hiddenIdx = $.extend(true, [], hiddenIdx);
                        hiddenIdx = [];
                    }
                }
            }
            this._dupDetails = false;
            if (this.model.showRibbon && this.model.allowFiltering) {
                var btnObj = $("#" + this._id + "_Ribbon_clearfilter").data("ejButton"), target = { rowIndex: this.getActiveCell().rowIndex, colIndex: this.getActiveCell().colIndex };
                this.XLRibbon._changeClrFltrStatus(btnObj, "button", target);
            }
            if(this.model.scrollSettings.allowScrolling)
				this.XLScroll._refreshScroller(sheetIdx, "refresh", "vertical", "filter");
        },
		
        _filterClose: function(){
                if (this._excelFilter._openedFltr && $(this._excelFilter._openedFltr).is(":visible")) {
                    this.XLFilter._isSearchEdit = false;
                    this._excelFilter.closeXFDialog();
                }
		},

        _setModel: function (options) {
            var option, expProp, sheetIdx = this.getActiveSheetIndex(), checkObj, model = this.model, xlId = this._id, rObj = $("#" + xlId + "_Ribbon").ejRibbon("instance"), rObjElem = rObj.element;
            for (option in options) {
                model[option] = options[option];
                switch (option) {
                    case "allowClipboard":
                        if (options[option]) {
                            if (!model.allowFormatPainter || !model.allowCellFormatting)
                                rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_1").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_cut").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_copy").show();
                            if (ej.isNullOrUndefined(this.XLClipboard))
                            this.XLClipboard = new ej.spreadsheetFeatures.clipboard(model);
                        }
                        else {
                            if (!model.allowFormatPainter || !model.allowCellFormatting)
                                rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_1").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_cut").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_copy").hide();
                        }
                        break;
                    case "activeSheetIndex":
                        this.gotoPage(options[option], false);
                        break;
                    case "pageSettings":
                        !this.isUndefined(options.pageSettings.currentPage) && this.gotoPage(options.pageSettings.currentPage, false);
                        if (!this.isUndefined(options.pageSettings.pageCount)) {
                            this.getPager().ejPager("option", "pageCount", options.pageSettings.pageCount);
                            this._refreshSpreadSheetPager();
                        }
                        break;
                    case "allowComments":
                        if (options[option]) {
                            if (ej.isNullOrUndefined(this.XLComment))
                                this.XLComment = new ej.spreadsheetFeatures.comments(model);
                            if(rObj.length)
                            rObj.showTab("REVIEW");
                        }
                        else if(rObj.length)
                            rObj.hideTab("REVIEW");
                        break;
                    case "allowFormatPainter":
                        if (model.allowCellFormatting) {
                            if (options[option]) {
                                if (!model.allowClipboard)
                                    rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard").show();
                                if (ej.isNullOrUndefined(this.XLFormat))
                                    this.XLFormat = new ej.spreadsheetFeatures.cellFormatting(model);
                                rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_formatpainter").show();
                            }
                            else {
                                if (!model.allowClipboard)
                                    rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard").hide();
                                rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_formatpainter").hide();
                            }
                        }
                        break;
                    case "allowHyperlink":
                        if (options[option])
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Links").show();
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Links").hide();
                        break;
                    case "enableContextMenu":
                        if (options[option] && ej.isNullOrUndefined(this.XLCMenu))
                            this.XLCMenu = new ej.spreadsheetFeatures.contextmenu(model);
                        break;
                    case "allowEditing":
                        if (options[option]) {
                            if (ej.isNullOrUndefined(this.XLEdit))
                                this.XLEdit = new ej.spreadsheetFeatures.editing(model);
                            rObjElem.find("#" + xlId + "_Ribbon_home_Editing").show();
                        }
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_home_Editing").hide();
                        break;
                    case "allowResizing":
                        if (options[option] && ej.isNullOrUndefined(this.XLResize))
                            this.XLResize = new ej.spreadsheetFeatures.resizing(model);
                        break;
                    case "allowDataValidation":
                        if(options[option]) {
                            rObjElem.find("#" + xlId + "_Ribbon_data_DataTools").show();
                            if (ej.isNullOrUndefined(XLValidate))
                                this.XLValidate = new ej.spreadsheetFeatures.validation(model);
                          }
                        else
                        rObjElem.find("#" + xlId + "_Ribbon_data_DataTools").hide();
                        break;
                    case "allowConditionalFormats":
                        if (options[option]) {
                            rObjElem.find("#" + xlId + "_Ribbon_home_Styles_cFormat").show();
                            if (ej.isNullOrUndefined(this.XLCFormat))
                                this.XLCFormat = new ej.spreadsheetFeatures.cFormat(model);
                        }
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_home_Styles_cFormat").hide();
                            break;
                    case "allowSearching":
                        this.model.allowSearching = options[option];
                        if (options[option]) {
                            if (ej.isNullOrUndefined(this.XLFndRp))
                                this.XLFndRp = new ej.spreadsheetFeatures.findnreplace(model);
                                rObjElem.find("#" + xlId + "_Ribbon_others_Editing").show();
                        }
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_others_Editing").hide();
                        
                        break;
                    case "pictureSettings":
                        for (var picProp in options[option]) {
                            switch (picProp) {
                                case "allowPictures":
                                    model.pictureSettings.allowPictures = options[option][picProp];
                                    if (options[option][picProp]) {
                                        if (ej.isNullOrUndefined(this.XLShape))
                                            this.XLShape = new ej.spreadsheetFeatures.shape(model);
                                        rObjElem.find("#" + xlId + "_Ribbon_insert_Illustrations").show();
                                    }
                                    else
                                        rObjElem.find("#" + xlId + "_Ribbon_insert_Illustrations").hide();
                                    break;
                            }
                        }
                        break;
                    case "showRibbon":
                        this.model.showRibbon = options[option];
                        if (options[option])
                            $("#" + xlId + "_Ribbon").show();
                        else
                            $("#" + xlId + "_Ribbon").hide();
                        break;
                    case "exportSettings":
                        for (expProp in options[option]) {
                            switch (expProp) {
                                case "allowExporting":
                                    model.exportSettings.allowExporting = options[option][expProp];
                                    if (options[option][expProp]) {
                                        rObjElem.find("#saveas_backStageTab").show();
                                        if (ej.isNullOrUndefined(this.XLExport))
                                            this.XLExport = new ej.spreadsheetFeatures.exporting(model);
                                    }
                                    else
                                        rObjElem.find("#saveas_backStageTab").hide();
                                    break;
                                case "excelUrl":
                                    model.exportSettings.excelUrl = options[option][expProp];
                                    break;
                                case "csvUrl":
                                    model.exportSettings.csvUrl = options[option][expProp];
                                    break;
                            }
                        }
                        break;
                    case "allowKeyboardNavigation":
                        if (options[option] && ej.isNullOrUndefined(this.XLCellNav))
                            this.XLCellNav = new ej.spreadsheetFeatures.cellNavigation(model);
                        break;
                    case "allowAutoFill":
                        if (options[option] && ej.isNullOrUndefined(this.XLDragFill))
                            this.XLDragFill = new ej.spreadsheetFeatures.dragFill(model);
                        break;
                    case "allowSelection":
                        if (options[option] && ej.isNullOrUndefined(this.XLSelection))
                            this.XLSelection = new ej.spreadsheetFeatures.selection(model);
                        break;
                    case "allowDragAndDrop":
                        if (options[option] && ej.isNullOrUndefined(this.XLDragDrap))
                            this.XLDragDrap = new ej.spreadsheetFeatures.dragAndDrop(model);
                        break;
                    case "allowCellFormatting":
                   if (options[option]) {
                            if (model.allowConditionalFormats || model.allowFormatAsTable)
                                rObjElem.find("#" + xlId + "_Ribbon_home_Styles").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Font").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Number").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_top").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_middle").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_bottom").show();
                            rObjElem.find("#" + xlId + "_Ribbon_separator_bottom").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_left").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_center").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_right").show();
                            rObjElem.find("#" + xlId + "_Ribbon_separator_right").show();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Styles_cellstyles").show();
                            if (model.allowFormatPainter)
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_formatpainter").show();
                            if (model.allowClipboard)
                                rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard").show();
                            if (ej.isNullOrUndefined(this.XLFormat))
                                this.XLFormat = new ej.spreadsheetFeatures.cellFormatting(model);
                        }
                        else {
                            rObjElem.find("#" + xlId + "_Ribbon_home_Font").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Number").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_top").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_middle").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_bottom").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_separator_bottom").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_left").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_center").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_right").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_separator_right").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Styles_cellstyles").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard_formatpainter").hide();
                            if (!model.allowClipboard)
                                rObjElem.find("#" + xlId + "_Ribbon_home_Clipboard").hide();
                            if (!model.allowConditionalFormats && !model.allowFormatAsTable)
                                rObjElem.find("#" + xlId + "_Ribbon_home_Styles").hide();
                        }
                        break;
                    case "allowImport":
                        if (options[option])
                            rObjElem.find("#new_backStageTab").show();
                        else
                            rObjElem.find("#new_backStageTab").hide();
                        break;
                    case "importSettings":
                        for (importProp in options[option]) {
                            switch (expProp) {
                                case "importMapper":
                                    model.importSettings.importMapper = options[option][importProp];
                                    break;
                                case "importUrl":
                                    model.importSettings.importUrl = options[option][importProp];
                                    break;
                                case "password":
                                    model.importSettings.password = options[option][importProp];
                                    break;
                            }
                        }
                        break;
                   case "allowSorting":
                        if (options[option]) {
                            if (ej.isNullOrUndefined(this.XLSort))
                                this.XLSort = this.XLSort = new ej.spreadsheetFeatures.sorting(model);
                            rObjElem.find("#" + xlId + "_Ribbon_data_SortFilter_1").show();
							rObjElem.find("#" + xlId + "_Ribbon_data_Sort").show();
							rObjElem.find("#" + xlId + "_Ribbon_home_Editing_1").show();
							
                       }
                        else{
                            rObjElem.find("#" + xlId + "_Ribbon_data_SortFilter_1").hide();
							if(!this.model.allowFiltering) {
								rObjElem.find("#" + xlId + "_Ribbon_data_Sort").hide();
								rObjElem.find("#" + xlId + "_Ribbon_home_Editing_1").hide();
							}
						}
                    case "formatSettings":
                        if (model.allowCellFormatting) {
                            for (var formatSetting in options[option]) {
                                switch (formatSetting) {
                                    case "allowFontFamily":
                                        model.formatSettings.allowFontFamily = options[option][formatSetting];
                                        if (options[option][formatSetting])
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Font_font-family").show();
                                        else 
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Font_font-family").hide();
                                        break;
                                    case "allowDecimalPlaces":
                                        model.formatSettings.allowDecimalPlaces = options[option][formatSetting];
                                        if (options[option][formatSetting]) {
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Number_increment").show();
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Number_decrement").show();
                                            rObjElem.find("#" + xlId + "_Ribbon_separator_usegrouping").show();
                                        }
                                        else {
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Number_increment").hide();
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Number_decrement").hide();
                                            rObjElem.find("#" + xlId + "_Ribbon_separator_usegrouping").hide();
                                        }
                                        break;
                                    case "allowCellBorder":
                                        model.formatSettings.allowCellBorder = options[option][formatSetting];
                                        if (options[option][formatSetting]) {
                                            rObjElem.find("#" + xlId + "_Ribbon_separator_border").show();
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Font_border").show();
                                        }
                                        else {
                                            rObjElem.find("#" + xlId + "_Ribbon_separator_border").hide();
                                            rObjElem.find("#" + xlId + "_Ribbon_home_Font_border").hide();
                                        }
                                        break;
                                }
                            }
                        }
                        break;
                    case "showGridlines":
                        if (this.model.showRibbon) {
                            checkObj = this.element.find("#" + this._id + "_Ribbon_Vgridlines").data("ejCheckBox");
                            (options[option]) ? checkObj._checked() : checkObj._unChecked();
                        }
                        this.showGridlines(options[option]);
                        break;
                    case "scrollSettings":
                        var settings = {};
                        if (!ej.isNullOrUndefined(options["scrollSettings"])) {
                            if (!ej.isNullOrUndefined(options["scrollSettings"]["allowScrolling"]))
                                model.scrollSettings.allowScrolling = options["scrollSettings"]["allowScrolling"];
                            if (!ej.isNullOrUndefined(options["scrollSettings"]["scrollMode"]))
                                if (options["scrollSettings"]["scrollMode"] === "normal")
                                    model.scrollSettings.scrollMode = ej.Spreadsheet.scrollMode.Normal;
                                else
                                    model.scrollSettings.scrollMode = ej.Spreadsheet.scrollMode.Infinite;
                            if (!ej.isNullOrUndefined(options["scrollSettings"]["width"]))
                                model.scrollSettings.width = options["scrollSettings"]["width"];
                            if (!ej.isNullOrUndefined(options["scrollSettings"]["height"]))
                                model.scrollSettings.height = options["scrollSettings"]["height"];
                        }
                        $.extend(true, settings, this.model.scrollSettings);
                        this._setSheetWidthHeight(sheetIdx, settings);
                        break;
                    case "allowFiltering":
                        if (options[option]) {
                            rObjElem.find("#" + xlId + "_Ribbon_data_SortFilter_2").show();
							rObjElem.find("#" + xlId + "_Ribbon_data_Sort").show();
							rObjElem.find("#" + xlId + "_Ribbon_home_Editing_1").show();
                            if (ej.isNullOrUndefined(this.XLFilter))
                                this.XLFilter = new ej.spreadsheetFeatures.filter(model);
							this.XLFilter._renderExcelFilter();
							this.XLFilter._renderFilterDialogs();
                        }
                        else {
							if(!this.model.allowSorting) {
								rObjElem.find("#" + xlId + "_Ribbon_data_Sort").hide();
								rObjElem.find("#" + xlId + "_Ribbon_home_Editing_1").hide();
							}
                            rObjElem.find("#" + xlId + "_Ribbon_data_SortFilter_2").hide();
						}
                        break;
                    case "allowInsert":
                        if (options[option]) {
                            if (!model.allowDelete)
                                rObjElem.find("#" + xlId + "_Ribbon_others_Cells").show();
                            rObjElem.find("#" + xlId + "_Ribbon_others_Cells_1").show();
                           
                        }
                        else {
                            if (!model.allowDelete)
                                rObjElem.find("#" + xlId + "_Ribbon_others_Cells").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_others_Cells_1").hide();
                        }
                        break;
                    case "allowDelete":
                        if (options[option]) {
                            if (!model.allowInsert)
                                rObjElem.find("#" + xlId + "_Ribbon_others_Cells").show();
                            rObjElem.find("#" + xlId + "_Ribbon_others_Cells_2").show();
                        }
                        else {
                            if (!model.allowInsert)
                                rObjElem.find("#" + xlId + "_Ribbon_others_Cells").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_others_Cells_2").hide();
                        }
                        break;
                    case "allowFreezing":
                        if (options[option]) {
                            if (ej.isNullOrUndefined(this.XLFreeze))
                                this.XLFreeze = new ej.spreadsheetFeatures.freezepane(model);
                            rObjElem.find("#" + xlId + "_Ribbon_others_Window").show();
                        }
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_others_Window").hide();
                        break;
                   case "allowMerging":
                        if (options[option])
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_merge").show();
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_merge").hide();
                        break;
                    case "freezeFirstColumn":
                        case "allowWrap":
                        if (options[option])
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_wraptext").show();
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_home_Alignment_wraptext").hide();
                        break; 
                    case "allowUndoRedo":
                        if (options[option])
                            rObjElem.find("#" + xlId + "_Ribbon_home_Actions").show();
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_home_Actions").hide();
                    case "freezeLeftColumn":
                        this.XLFreeze.freezeLeftColumn(sheetIdx);
                        this.XLFreeze._updateFPaneMenu("freeze");
                        break;
                    case "freezeRows":
                        case "allowFormatAsTable":
                        if (options[option]) {
                            if (!model.allowPivotTable)
                                rObjElem.find("#" + xlId + "_Ribbon_insert_Tables").show();
                            if (ej.isNullOrUndefined(this.XLFormat))
                                this.XLFormat = new ej.spreadsheetFeatures.cellFormatting(model);
                            rObjElem.find("#" + xlId + "_Ribbon_home_Styles_formatastable").show();
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Tables_table").show();
                        }
                        else {
                            if (!model.allowPivotTable)
                                rObjElem.find("#" + xlId + "_Ribbon_insert_Tables").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_home_Styles_formatastable").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Tables_table").hide();
                        }
                        break;
                    case "freezeColumns":
                        case "allowPivotTable":
                        if (options[option]) {
                            if (!model.allowFormatAsTable)
                                rObjElem.find("#" + xlId + "_Ribbon_insert_Tables").show();
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Tables_pivotTable").show();
                        }
                        else {
                            if (!model.allowFormatAsTable)
                                rObjElem.find("#" + xlId + "_Ribbon_insert_Tables").hide();
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Tables_pivotTable").hide();
                        }
                        break;
                        case "allowAutoSum":
                       if (options[option])
                           rObjElem.find("#" + xlId + "_Ribbon_home_Editing_autosum").show();
                       else
                           rObjElem.find("#" + xlId + "_Ribbon_home_Editing_autosum").hide();
                        break;
                        case "allowCharts":
                            if (options[option]) {
                                if (ej.isNullOrUndefined(this.XLChart) || ej.isNullOrUndefined(this.XLShape))
                                this.XLShape = new ej.spreadsheetFeatures.shape(model);
                                this.XLChart = new ej.spreadsheetFeatures.chart(model);
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Charts").show();
                        }
                        else
                            rObjElem.find("#" + xlId + "_Ribbon_insert_Charts").hide();
                        break;
                    case "allowSortandFilter":
						case "printSettings":
                        for (var pageProp in options[option]) {
                            switch (pageProp) {
                                case "allowPrinting":
                                    model.printSettings.allowPrinting = options[option][pageProp];
                                    if (options[option][pageProp]) {
                                        this.XLPrint = new ej.spreadsheetFeatures.print(model);
                                        rObjElem.find("#" + xlId + "_Ribbon_pagelayout_Print").show();
                                        rObjElem.find("#print_backStageTab").show();
                                    }
                                    else {
                                        rObjElem.find("#print_backStageTab").show();
                                        rObjElem.find("#" + xlId + "_Ribbon_pagelayout_Print").hide();
                                    }
                                    break;
                                case "allowPageSize":
                                    model.printSettings.allowPageSize = options[option][pageProp];
                                    if (options[option][pageProp])
                                        rObjElem.find("#" + xlId + "_Ribbon_pagelayout_PageLayout").show();
                                    else
                                        rObjElem.find("#" + xlId + "_Ribbon_pagelayout_PageLayout").hide();
                                   break;
                            }
                        }
                        break;
                    case "allowLockCell":
                        if (options[option]) {
                            rObjElem.find(".e-ssr-protectsheetbtn").show();
                            rObjElem.find(".e-ssr-lockbtn").show();
                        }
                        else {
                            rObjElem.find(".e-ssr-protectsheetbtn").hide();
                            rObjElem.find(".e-ssr-lockbtn").hide();
                        }
                }
            }
            if (rObj.length) {
                if (!model.allowAutoSum && !model.allowFiltering && !model.allowUndoRedo && !model.allowSorting && !model.allowClipboard && !model.allowFormatPainter && !model.allowCellFormatting && !model.allowMerging && !model.allowWrap && !model.allowConditionalFormats && !model.allowFormatAsTable)
                    rObj.hideTab("HOME");
                else
                    rObj.showTab("HOME");
                if (!model.allowFormatAsTable && !model.allowPictures && !model.allowHyperlink && !model.allowCharts)
                    rObj.hideTab("INSERT");
                else
                    rObj.showTab("INSERT");
                if (!model.allowDataValidation && !model.allowFiltering && !model.allowSorting)
                    rObj.hideTab("DATA");
                else
                    rObj.showTab("DATA");
                if (!model.allowPrinting)
                    rObj.hideTab("PAGELAYOUT");
                else
                    rObj.showTab("PAGELAYOUT");
                if (!model.allowComments)
                    rObj.hideTab("REVIEW");
                else
                    rObj.showTab("REVIEW");
                if (!model.allowFreezing && !model.allowInsert && !model.allowDelete && !model.allowSearching)
                    rObj.hideTab("OTHERS");
                else
                    rObj.showTab("OTHERS");
            }
        },

        // Importing
        "import": function (importRequest) {
            if (!this.model.allowImport)
                return;
            var proxy = this;
            proxy.importSheet = proxy._initialLoad ? false : true;
            proxy.showWaitingPopUp();
            $.ajax({
                type: "POST",
                data: importRequest,
                processData: false,
                contentType: false,
                async: false,
                url: proxy.model["importSettings"].importMapper,
                success: function (response) {
                    proxy._importSuccess(response);
                    proxy.importSheet = false;
                },
                error: function (err, type) {
                    proxy._importFailed({ type: type, xhr: { status: err.status, statusText: err.statusText } });
                }
            });
            proxy.hideWaitingPopUp();
        },

        _importSuccess: function (response) {
            if (!this.model.allowImport)
                return;
            var error = ["UnsupportedFile", "IncorrectPassword", "InvalidUrl", "NeedPassword"];
            switch (response) {
                case error[0]:
                case error[1]:
                case error[2]:
                    this._showAlertDlg("Alert", response, "", null, 110);
                    break;
				case error[3]:
                    $("#" + this._id + "_ImportPasswordDialog").ejDialog("open");
					this.model.importSettings.password = null;
                    break;
                default:
                    this.loadFromJSON(JSON.parse(response));
            }
        },

        loadFromJSON: function (response) {
            var j, actCell, hashcode, cformat, vData,resSheet, pxySheet, usedRange, i = 1,
            cnts = this._getWorkbookMaxRowColCnt(response.model.sheets, true);
            if (response.dataContainer.sharedData.length < 50001 && cnts.rowCount < 10001 && cnts.colCount < 126) {
                this._rowCount = this.model.rowCount;
                this._colCount = this.model.columnCount;
                hashcode = response.dataContainer.hashCode;                
                !this._initialLoad && this._destroy();
                this.model.sheetCount = response.model.sheetCount;
                this._updateImportedSheetModel(response.model);
                this.model.sheets.splice(0, 0, {});
                this.model.activeSheetIndex = response.model.activeSheetIndex;
                this._renderSheets();
                this._importedCondFormat = response.dataContainer.cFormatData;
                this._importedValData = response.dataContainer.valData;
                response.dataContainer.cFormatData = [];
                response.dataContainer.hashCode = this._dataContainer.hashCode;
                response.dataContainer.valData = [];
                this._dataContainer = response.dataContainer;
                this.XLFormat._formatAsTableStyle = this.XLFormat._getTableLayoutFromName("TableStyleLight1");
                this._updateImportedStyles(hashcode);
                this._updateNameManager(response.model.nameManager);
                i = response.model.sheets.length;
                while (i--) {
                    resSheet = response.model.sheets[i];
                    pxySheet = this.getSheet(i);
                    if (resSheet.usedRange) {
                        pxySheet._importRendered = true;

                        for (j = 0; j < resSheet.columnsWidthCollection.length; j++) {
                            if (!this.isUndefined(resSheet.columnsWidthCollection[j])) pxySheet.columnsWidthCollection[j] = resSheet.columnsWidthCollection[j];
                        }
                        for (j = 0; j < resSheet.rowsHeightCollection.length; j++) {
                            if (!this.isUndefined(resSheet.rowsHeightCollection[j])) pxySheet.rowsHeightCollection[j] = resSheet.rowsHeightCollection[j];
                        }
                        pxySheet._shapeMngr = resSheet._shapeMngr;
                        if (resSheet._mergedCells)
                            pxySheet._mergedCells = resSheet._mergedCells;
                        usedRange = this.getRangeIndices(resSheet.usedRange);
                        pxySheet._usedRange = { rowIndex: usedRange[2], colIndex: usedRange[3] };
                        if (resSheet._tableManager)
                            pxySheet._tableManager = resSheet._tableManager;
                        if (resSheet.filterSettings.range)
                            pxySheet.filterSettings.range = resSheet.filterSettings.range;
                        if (resSheet._isSheetProtected)
                            pxySheet._isSheetProtected = resSheet._isSheetProtected;
                        pxySheet.hideRowsCollection = resSheet.hideRowsCollection;
                        pxySheet.hideColsCollection = resSheet.hideColsCollection;
                        if (resSheet._frozenRows || resSheet._frozenColumns) {
                            pxySheet._frozenRows = resSheet._frozenRows;
                            pxySheet._frozenColumns = resSheet._frozenColumns;
                        }
                        if (!resSheet.showHeader)
                            pxySheet.showHeader = resSheet.showHeader;
                        if (!resSheet.showGridlines)
                            pxySheet.showGridlines = resSheet.showGridlines;
                    }
                }
                this._updateDataValidation(vData);
                this._updateConditionalFormats(cformat);
                this.gotoPage(this.model.activeSheetIndex, false);
                actCell = this.getCell(this.getActiveCell().rowIndex, this.getActiveCell().colIndex)[0];
                this.XLRibbon._toggleDesignTab(actCell);
                if (this.model.showRibbon) {
                    if (this.model.allowLockCell && this.getSheet(this.getActiveSheetIndex())._isSheetProtected)
                        this.XLRibbon._disableRibbonIcons();
                    this.XLRibbon._enableRibbonIcons();
                }
                this.XLComment._isCommentEdit = false;
                this.clearUndoRedo();
                this._trigger("loadComplete", { model: this.model });
                var sheet = this.getSheet(this.model.activeSheetIndex);
                (sheet._frozenRows || sheet._frozenColumns) && this._setFreezePane(sheet._frozenRows, sheet._frozenColumns);
                this._detachTable = null;
                this.model.rowCount = this._rowCount;
                this.model.columnCount = this._colCount;
            }
            else
                this._showAlertDlg("Alert", "HugeDataAlert");
        },

        _importFailed: function (response) {
            var args = { failureType: response.type };
            if (response.xhr) {
                args.status = response.xhr.status;
                args.statusText = response.xhr.statusText;
            }
            else
                args.statusText = response.error;
            this.hideWaitingPopUp();
            this._trigger("openFailure", args);
        },

        _updateImportedSheets: function (sheetIdx) {
            var range, data, sheet = this.getSheet(sheetIdx), obj = { rowIndex: 0, colIndex: 0 };
            this._setRowColumnSize(sheet);
            range = [0, 0, sheet._usedRange.rowIndex, sheet._usedRange.colIndex];
            data = this.getRangeData(range, false, ["value", "value2", "format", "hyperlink", "comment", "border", "picture", "wrap", "isMerge", "mRange", "isCenterAlign", "isLocked"]);
            this._updateImportedData(data, range);
            this._updateConditionalFormats();
            this._updateDataValidation();
            this._updateImportedChart(sheet._shapeMngr.chart);
            this._updateImportedTable(sheet._tableManager);
            sheet.filterSettings.range.length && this.XLFilter._filterCellRange(sheetIdx, sheet.filterSettings.range);
            this._updateHiddenRowsandColumns(sheet);
            sheet._activeCell = obj;
            !sheet.showHeader && this.showHeaders(sheet.showHeader);
            !sheet.showGridlines && this.showGridlines(sheet.showGridlines);
            !this.importSheet && this.XLSelection.selectRange(obj, obj);
        },

        _updateImportedMergedCells: function (sheet) {
            sheet._mergedCells = sheet._mergedCells;
            for (var j = 0, len = sheet._mergedCells.length; j < len; j++) {
                sheet._mergedCells[j].isCenterAlign && (this._mergeCenter = true);
                this.mergeCells(sheet._mergedCells[j].range, true);
                this._mergeCenter = false;
            }
            sheet._mergedCell = [];
        },

        _updateImportedData: function (data, range, sheetIdx, dataOnly) {
            sheetIdx = sheetIdx ? sheetIdx : this.getActiveSheetIndex();
            range = range ? range : this.getSheet(sheetIdx).selectedRange;
            var $span = ej.buildTag('span'), rowHdrs= this.getRows(sheetIdx)[0], cellIdx, rows, colgrp = $(this._getJSSheetHeader(sheetIdx).find('colgroup')[0].outerHTML).attr("id", "ss"), emptyTable = ej.buildTag("table#wrapTable.e-table", {});
			emptyTable.append(colgrp);
            this._detachTable = this._getContent(sheetIdx).find(".e-content >table");
            rows = this._detachTable[0].tBodies[0].rows;
            this._detachTable.detach();
			this.element[0].insertBefore(emptyTable[0], this.element[0].childNodes[0]);
            var colIdx, rowIdx, cellData, cell, cells = this.getRange(range[0], range[1], range[2], range[3], sheetIdx), i = 0, length = cells.length;
			this._getContent(sheetIdx).find(".e-content").append($span);
            while (i < length) {
                cell = cells[i];
                cellIdx = this._getCellIdx(cell);
                rowIdx = cellIdx.rowIndex;
                colIdx = cellIdx.coIndex;
                cellData = data[i];
                this.XLEdit._updateDataContainer(cellIdx, { value: this.XLEdit._parseValue(cellData.value).value }, sheetIdx);
				if(cellData.value) {
					$span.text(cellData.value);
					$span[0].className = cellData.format ? cellData.format : "";
					this._textClip(rowIdx, colIdx, 'add', $span.width());
				} 
                if (this.isFormula(cellData.value))
                    this.XLEdit._refreshCalcEngine(rowIdx, colIdx, true, cellData.value);
                else {
                    cell.innerHTML = cellData.value2 || "";
                    this.XLEdit._refreshCellAlignment(cell, cellData.value);
                    this.XLEdit._refreshCalcEngine(rowIdx, colIdx);
                }
                if (!dataOnly) {
                    cell.className = cell.className + (cellData.format ? " " + cellData.format : "");
                    if (cellData.hyperlink)
                        "cellAddr" in cellData.hyperlink ? $(cell).html(ej.buildTag('a', cellData.value2, '', { href: '#' })) : $(cell).html(ej.buildTag('a', cellData.value2, '', { href: cellData.hyperlink.webAddr, target: '_blank' }));
                    cellData.comment && this.XLComment.setComment([rowIdx, colIdx, rowIdx, colIdx], null, false);
                    cellData.border && this.XLFormat.applyBorder(this._getBorderInput(cellData.border), this._getAlphaRange(sheetIdx, cell.parentNode.rowIndex, cell.cellIndex, cell.parentNode.rowIndex, cell.cellIndex));
                    cellData.picture && this.XLShape._copyImages(cell, cellData.picture, sheetIdx);
                    if (cellData.isMerge) {
                        cellData.isCenterAlign && (this._mergeCenter = true);
                        this.mergeCells(cellData.mRange, true);
                        this._mergeCenter = false;
                    }
                    cellData.wrap && this._setWrapTextForImport(sheetIdx, rowIdx, colIdx, rows);
                }
				if(colIdx === range[3] && this._wrapRow){
					this._setWrapHeight(sheetIdx, rowIdx, rows, rowHdrs, emptyTable, "hide");
					!(this._hasClass(rowHdrs[rowIdx], "e-sswraprow")) && this.addClass(rowHdrs[rowIdx], "e-sswraprow");
                    this._wrapRow = false;
				}
                i++;
            }
			$span.remove();
			emptyTable.remove();
            this._getContent(sheetIdx).find(".e-content").append(this._detachTable);
            !dataOnly && this.XLComment._updateCmntArrowPos();
        },

        _updateImportedStyles: function (hashcode) {
            var format, code, i = hashcode.length;
            if (i) {
                hashcode = hashcode.reverse();
                while (i--) {
                    code = hashcode[i];
                    if (code.indexOf("e-format") === -1)
                        this._dataContainer.hashCode.push(code);
                    else {
                        format = this.XLFormat.getFormatFromHashCode(code);
                        this.XLFormat._createFormatClass(format);
                    }
                }
            }
        },

        _updateImportedTable: function (tables) {
            var range, format, table, i = this.getObjectLength(tables);
            if (i) {
                while (i--) {
                    table = tables[i];
                    range = table.range;
                    format = this.XLFormat._getTableLayoutFromName(table.formatName);
                    if (!format) {
                        format = this.XLFormat._formatAsTableStyle;
                        table.formatName = format.formatName;
                    }
                    this.XLFormat.createTable({ header: true, name: table.name, format: format.format, formatName: table.formatName }, this._getAlphaRange(this.getActiveSheetIndex(), range[0], range[1], range[2], range[3]));
                }
            }
        },

        _updateConditionalFormats: function () {
            var split, i, sheetIdx = this.getActiveSheetIndex(), len =  this._importedCondFormat.length;
            for(i = 0; i < len; i++){
                split = this._importedCondFormat[i].split("_");
                idx = Number(split[4]);
                if(idx === sheetIdx){
                    this.XLCFormat._cFormat(split[0], split[1], split[2], split[3], null, idx, this.getRangeIndices(split[5]));
                    this._importedCondFormat.splice(i, 1);
                    len = this._importedCondFormat.length;
                    i--;
                }
            }
        },

        _updateImportedSheetModel: function (model) {
            var sheet, i = 1, len = model.sheets.length;
            this.model.sheets = [];
            while (i <= len) {
                sheet = model.sheets[i];
                this.model.sheets.push({ sheetName: sheet._sheetName.text, rowCount: sheet.rowCount || this.model.rowCount, colCount: sheet.colCount || this.model.columnCount });
                i++;
            }
        },

        _setRowColumnSize: function (sheet) {
            var arr = [], coll = sheet.columnsWidthCollection, j = coll.length;
            if (j) {
                while (j--)
                    coll[j] !== 64 && arr.push({ cellIndex: j, width: coll[j] });  // 64 is default col width for excel
                this.setWidthToColumns(arr);
            }
            coll = sheet.rowsHeightCollection;
            j = coll.length;
            arr = [];
            if (j) {
                while (j--)
                    (coll[j] > 20) && arr.push({ rowIndex: j, height: coll[j] });
                this.setHeightToRows(arr);
            }
        },

        setHeightToRows: function (arrheight) {
            var xlObj = this, sheetIdx = xlObj.getActiveSheetIndex(), rHContent = xlObj._getJSSheetRowHeaderContent(sheetIdx);
            var tBody = rHContent.find(".e-table")[0].tBodies[0], rows = tBody.rows, cRows = xlObj._getJSSheetContent(sheetIdx).find(".e-content > .e-table")[0].tBodies[0].rows;
            for (var i = 0, len = arrheight.length; i < len; i++) {
                $(rows[arrheight[i].rowIndex]).outerHeight(arrheight[i].height);
                $(cRows[arrheight[i].rowIndex]).height(arrheight[i].height);
            }
			this.XLScroll._getRowHeights(sheetIdx, 1);
			this.XLScroll._refreshScroller(sheetIdx, "refresh", "vertical");
        },

        _updateNameManager: function (nameMgr) {
            nameMgr = nameMgr || [];
            var i = nameMgr.length;
            if (i) {
                while (i--)
                    this.XLRibbon._updateNamedRanges(nameMgr[i].name, nameMgr[i].refersto);
                $("#" + this._id + "_nmgrid").ejGrid("option", { dataSource: nameMgr });
                this.model.nameManager = nameMgr;
                this.XLRibbon._updateUseInFormulaTrgt();
            }
        },

        _setFreezePane: function (rowIdx, colIdx) {
            if (rowIdx && colIdx)
                this.XLFreeze.freezePanes({ rowIndex: rowIdx, colIndex: colIdx });
            else if (rowIdx)
                this.XLFreeze.freezeRows(rowIdx);
            else if (colIdx)
                this.XLFreeze.freezeColumns(colIdx);
        },

        _updateHiddenRowsandColumns: function (model) {
            var coll = model.hideRowsCollection, i = coll.length;
            if (i) {
                while (i--)
                    this._hideRow(coll[i], coll[i]);
            }
            coll = model.hideColsCollection;
            i = coll.length;
            if (i) {
                while (i--)
                    this._hideColumn(coll[i], coll[i]);
            }
        },

        _updateImportedChart: function (charts) {
            var sheet, range, data, chart, i = 0, len = charts.length;
            while (i < len) {
                chart = charts[i];
                sheet = this.getSheet(chart.dataSheetIdx);
                if (chart.dataSheetIdx != this.getActiveSheetIndex() && sheet._importRendered) { // for other sheet data issue
                    range = this.getRangeIndices(sheet._usedRange);
                    data = this.getRangeData(range, false, ["value", "value2"], chart.dataSheetIdx);
                    this._updateImportedData(data, range, chart.dataSheetIdx, true);
                }
                this.XLChart.createChart(chart.range, { type: chart.commonSeriesOptions.type, enable3D: chart.enable3D, marker: chart.commonSeriesOptions.marker, top: chart.top, left: chart.left, heigth: chart.size.height, width: chart.size.width, title: chart.title, legend: chart.legend, primaryYAxis: chart.primaryYAxis, primaryXAxis: chart.primaryXAxis, dataSheetIdx: chart.dataSheetIdx, _id: chart.dummyId });
                i++;
            }
        },

        _getBorderInput: function (str) {
            str = str.split("_");
            var i = 4, obj = {}, arr = ["top", "right", "bottom", "left"];
            while (i--)
                str[i].length && (obj[arr[i]] = str[i]);
            return obj;
        },

        _refreshHashCode: function (hashCode) {
            hashCode = hashCode || this._dataContainer.hashCode;
            var code, i = hashCode.length;
            while (i--) {
                code = hashCode[i];
                if (code.indexOf("e-format") === -1 && code.indexOf("e-border") === -1)
                    hashCode[i] = "";
            }
        },

        clearUndoRedo: function () {
            this._undoCollection = [];
            this._redoCollection = [];
            this._updateUndoRedoIcons();
        },

        _initializePicture: function (sheetIdx) {
            this._getContent(sheetIdx).find(".e-content").append(ej.buildTag("div.e-ss-imgvisual"));
        },

        _getShapePositionInManager: function (rowIndex, colIndex, id, shapeType) {
            var sheetIdx = this.getActiveSheetIndex(), cellObj = this._dataContainer.sheets[sheetIdx][rowIndex][colIndex], i, len;
            for (i = 0, len = cellObj.length; i < len; i++)
                if (cellObj[shapeType][i].id === id)
                    break;
            return i;
        },

        _swapVal: function (val) {
            var temp = val.prev;
            val.prev = val.cur;
            val.cur = temp;
        },

        _updateDataValidation: function () {
            var val, i, idx, sheetIdx = this.getActiveSheetIndex(), len = this._importedValData.length;
            for (i = 0; i < len; i++) {
                val = this._importedValData[i].split("_");
                idx = Number(val[val.length - 1]);
                if (idx === sheetIdx) {
                    if (this._importedValData[i].indexOf("list") === -1)
                        this.XLValidate.applyDVRules(val[5], [this._getValue(this.XLRibbon._actions, val[1]), val[2], val[3]], val[4], val[0] === "1" ? true : false, true);
                    else {
                        this.XLValidate.applyDVRules(val[3], ["list", val[1]], "list", val[0] === "1" ? true : false, true);
                    }
                    this._importedValData.splice(i, 1);
                    len = this._importedValData.length;
                    i--;
                }
            }
        },

        _getValue: function (coll, key) {
            var obj, i = coll.length;
            while (i--) {
                obj = coll[i];
                if (obj.value === key)
                    return obj.value;
            }
        },

        //Undo Redo Operations
        setUndoRedoSteps: function (value) {
            this._undoredoStep = value;
        },

        _isRowViewable: function (sheetIdx, rowIdx) {
            var isInView = true;
            if (this.model.scrollSettings.allowVirtualScrolling) {
                sheetIdx = this._getSheetIndex(sheetIdx);
                isInView = this.getSheet(sheetIdx)._virtualRowIdx.indexOf(sheetIdx) > -1;
            }
            return isInView;
        },
		
        _undoRedoOperation: function (evtobj) {
            if ((evtobj.which === 90 || evtobj.keyCode === 90) && evtobj.ctrlKey) {
				 evtobj.preventDefault();
                if ($(evtobj.target).is(":input")){
					if($(evtobj.target).val() !== this._cureditedCell) {
                    	var undoVal = $(evtobj.target).val();
				    	$(evtobj.target).val(this._cureditedCell);
						this._getInputBox().val(this._cureditedCell);	
						this._undoInputobj = {value:undoVal, rowIndex: $(evtobj.target).parents('tr')[0].rowIndex, colIndex: $(evtobj.target).parents('td')[0].cellIndex};
					}
                }
                else {
                    this._isUndo = true;
                    this._performUndoRedo();
                }
            }
            else if ((evtobj.which === 89 || evtobj.keyCode === 89) && evtobj.ctrlKey) {
                evtobj.preventDefault();
                if ($(evtobj.target).is(":input")){
					if($(evtobj.target).parents('td')[0].cellIndex === this._undoInputobj.colIndex && $(evtobj.target).parents('tr')[0].rowIndex === this._undoInputobj.rowIndex) {
				   		$(evtobj.target).val(this._undoInputobj.value);
				   		this._getInputBox().val(this._undoInputobj.value);
						}
				}
                else {
                    this._isUndo = false;
                    this._performUndoRedo();
                }
            }
        },

        _performUndoRedo: function () {
            var val = (this._isUndo) ? this._undoCollection.pop() : this._redoCollection.pop();
            if (!ej.isNullOrUndefined(val)) {
                if (val.sheetIndex !== this.getActiveSheetIndex())
                    this.gotoPage(val.sheetIndex, null);
                this._isUndoRedo = true;
                switch (val.reqType) {
                    case "auto-fill":
                        this._undoForAutoFill(val);
                        break;
                    case "auto-sum":
                        this._undoForAutoSum(val);
                        break;
                    case "cell-format":
                        this._undoForFormat(val);
                        break;
                    case "cell-styles":
                        this._undoForStyle(val);
                        break;
                    case "clear-all":
                        this._undoForClAll(val);
                        break;
                    case "clear-comment":
                        this._undoForClComment(val);
                        break;
                    case "clear-content":
                        this._undoForClContent(val);
                        break;
                    case "clear-format":
                        this._undoForClFormat(val);
                        break;
                    case "clear-link":
                        this._undoForCLLink(val);
                        break;
                    case "comment":
                        this._undoForComment(val);
                        break;
                    case "cond-format":
                        this._undoForCondFormat(val);
                        break;
                    case "copy-paste":
                        this._undoForCopyPaste(val);
                        break;
                    case "cut-paste":
                        this._undoForCutPaste(val);
                        break;
                    case "delete":
                        this._undoForDel(val);
                        break;
                    case "edit":
                        this._undoForEdit(val);
                        break;
                    case "filter":
                        this._undoForFilter(val);
                        break;
                    case "filterbycolor":
                        this._undoForFilterByColor(val);
                        break;
                    case "find-replace":
                        this._undoForFind(val);
                        break;
                    case "format-painter":
                        this._undoForFormatPaint(val);
                        break;
                    case "format-table":
                        this._undoForFormatTable(val);
                        break;
                    case "hyperlink":
                        this._undoForHyperlink(val);
                        break;
                    case "insert":
                        this._undoForIns(val);
                        break;
                    case "merge-cells":
                        this._undoForMerge(val);
                        break;
                    case "paging":
                        var tmpIdx = val.nSheetIndex;
                        val.nSheetIndex = val.sheetIndex;
                        val.sheetIndex = tmpIdx;
                        break;
                    case "pivotgrid":
                        this._undoForPivot(val);
                        break;
                    case "resize-column":
                        this._undoForResizeCol(val);
                        break;
                    case "resize-row":
                        this._undoForResizeRow(val);
                        break;
                    case "resize-fit-height":
                        this._undoForResizeFHeight(val);
                        break;
                    case "resize-fit-width":
                        this._undoForResizeFWidth(val);
                        break;
                    case "reorder-cells":
                        this._undoForReOrder(val);
                        break;
                    case "show-hide":
                        this._undoForShowHide(val);
                        break;
                    case "shape":
                        this._undoForShape(val);
                        break;
                    case "sort":
                        this._undoForSort(val);
                        break;
                    case "sortbycolor":
                        this._undoForSortByColor(val);
                        break;
                    case "textwrap":
                        this._undoForTextWrap(val);
                        break;
                    case "validation":
                        this._undoForValidation(val);
                        break;

                }
                (this._isUndo) ? this._redoCollection.push(val) : this._undoCollection.push(val);
                if (this._undoCollection.length > this._undoredoStep)
                    this._undoCollection.splice(0, 1);
                if (this._redoCollection.length > this._undoredoStep)
                    this._redoCollection.splice(0, 1);
                this._updateUndoRedoIcons();
                this.XLRibbon._updateRibbonIcons();
                this._isUndoRedo = false;
                this._isUndo = false;
                (val.reqType != "shape") && this.XLSelection._refreshBorder();
            }
        },

        _undoForAutoFill: function (val) {
            var sheet = this.getSheet(val.sheetIndex);
            if (this._isUndo) {
                this.clearRangeData(val.range);
                if(this.model.allowCellFormatting)
					this.XLFormat.updateFormat(val.bFormat, val.range);
                this.updateData($.extend(true, [], val.bData), val.range);
                val.actionType == "flash-fill" && this.updateData($.extend(true, [], val.bdData), val.bRange);
                if (val.colSel)
                    this.XLSelection.selectColumns(sheet._startCell.colIndex, sheet._endCell.colIndex);
                else if (val.rowSel)
                    this.XLSelection.selectRows(sheet._startCell.rowIndex, sheet._endCell.rowIndex);
                else
                    this.performSelection({ rowIndex: val.bRange[0], colIndex: val.bRange[1] }, { rowIndex: val.bRange[2], colIndex: val.bRange[3] });
            }
            else {
				if(this.model.allowCellFormatting)
					this.XLFormat.updateFormat(val.aFormat, val.range);
                this.updateData($.extend(true, [], val.aData), val.range);
                val.actionType == "flash-fill" && this.updateData($.extend(true, [], val.adData), val.bRange);
                this.performSelection({ rowIndex: val.bRange[0], colIndex: val.bRange[1] }, { rowIndex: val.range[2], colIndex: val.range[3] });
            }
        },

        _undoForAutoSum: function (val) {
            var container = this._dataContainer.sheets[val.sheetIndex], cell;
            if (this._isUndo) {
				for(var i = 0, len = val.updCell.length; i < len; i++ ) {
					cell = this.getCell(val.updCell[i].rowIndex, val.updCell[i].colIndex, val.sheetIndex);
					this._clearFormula(val.updCell[i].rowIndex, val.updCell[i].colIndex);
					container[val.updCell[i].rowIndex][val.updCell[i].colIndex] = val.updCell[i].pObj;
					cell.text(val.updCell[i].pValue);
				}
            }
            else
                this.XLRibbon.autoSum(val.oprType, val.range);
        },

        _undoForClAll: function (val) {
            var activeCell, tblRange, tab, sheet = this.model.sheets[val.sheetIndex], rowIdx, colIdx;
            if (this._isUndo) {
                this.updateData($.extend(true, [], val.bData), val.range);
                if(this.model.allowCellFormatting)
					this.XLFormat.updateFormat(val.bFormat, val.range);
                if(this.model.allowConditionalFormats)
					this.XLCFormat.refreshCFormat(val.range);
            }
            else {
                if(this.model.allowCellFormatting)
					this.XLFormat.clearFormat(val.range);
                if(this.model.allowConditionalFormats)
					this.XLCFormat.clearCF(true, val.range);
                this.clearRangeData(val.range);
            }
            for (var i = 0, len = val.selCells.length; i < len; i++) {
                rowIdx = val.selCells[i].rowIndex; colIdx = val.selCells[i].colIndex;
                if (val.selCells[i].prevComment) {
                    if (this._isUndo) {
                        this.XLEdit._updateDataContainer({rowIndex: rowIdx, colIndex: colIdx}, { "comment": val.selCells[i].prevComment });
                        if(this.model.allowComments)
                            this.XLComment.setComment([rowIdx, colIdx, rowIdx, colIdx], null, false);
                    }
                    else {
                        if(this.model.allowComments)
                            this.XLComment.deleteComment([rowIdx, colIdx, rowIdx, colIdx], "clear");
                    }
                    if(this._isRowViewable(val.sheetIndex, rowIdx)){
                        activeCell = this.getCell(rowIdx, colIdx, val.sheetIndex)[0];
                        activeCell[0].className = activeCell[0].className.replace("e-commentcell", "");
                    }
                }
                if (val.selCells[i].prevLink)
                    if (this._isUndo) {
                        this.XLEdit._updateDataContainer({rowIndex: rowIdx, colIndex: colIdx}, { "hyperlink": val.selCells[i].prevLink });
                        ej.isNullOrUndefined(val.selCells[i].prevLink.webAdr) ? $(activeCell).html(ej.buildTag("a.e-hyperlinks", val.selCells[i].prevText, "", { href: "#" })) : $(activeCell).html(ej.buildTag("a.e-hyperlinks", val.selCells[i].prevText, "", { href: val.selCells[i].prevLink.webAdr, target: "_blank" }));
                    }
                if (val.selCells[i].mRange)
                    if (this._isUndo) {
                        this._dupDetails = true;
                        val.selCells[i].mergeCenter && (this._mergeCenter = true);
                        this.mergeCells(val.selCells[i].mRange);
                        this._mergeCenter = false;
                        this._dupDetails = false;
                    }
                    else {
                        this._dupDetails = true;
                        this.unmergeCells(val.selCells[i].mRange);
                        this._dupDetails = false;
                    }
                if (val.selCells[i].wrap) {
                    this._dupDetails = true;
                    this.setWrapText(this._getAlphaRange(val.sheetIndex, val.selCells[i].rowIndex, val.selCells[i].colIndex, val.selCells[i].rowIndex, val.selCells[i].colIndex));
                    this._dupDetails = false;
                }
            }
            if (val.tabId > -1) {
                if (this._isUndo) {
                    sheet._tableManager[val.tabId] = val.tableObj;
                    tblRange = sheet._tableManager[val.tabId].range;
                    this.performSelection({ rowIndex: tblRange[0], colIndex: tblRange[1] }, { rowIndex: tblRange[2], colIndex: tblRange[3] });
                    this.element.find(".e-selected").addClass("e-table" + val.tabId);
                    this.XLFilter._filterTable(val.sheetIndex, val.tabId);
                }
                else {
                    tblRange = sheet._tableManager[val.tabId].range, tab = sheet.filterSettings.tableRange;
                    this.performSelection({ rowIndex: tblRange[0], colIndex: tblRange[1] }, { rowIndex: tblRange[2], colIndex: tblRange[3] });
                    this.element.find(".e-selected").removeClass("e-filterhdr e-table" + val.tabId);
                    delete sheet._tableManager[val.tabId];
                    for (i = 0, len = tab.length; i < len; i++) {
                        if (tab[i].tableID === val.tabId) {
                            tab.splice(i, 1);
                            break;
                        }
                    }
                    this.XLRibbon._toggleDesignTab();
                }
            }
        },

        _undoForClComment: function (val) {
            var cell, startCell, endCell;
            startCell = { rowIndex: val.selCells[0].rowIndex, colIndex: val.selCells[0].colIndex };
            endCell = { rowIndex: val.selCells[val.selCells.length - 1].rowIndex, colIndex: val.selCells[val.selCells.length - 1].colIndex };
            if (this._isUndo) {
                for (var i = 0, len = val.selCells.length; i < len; i++) {
                    cell = { rowIndex: val.selCells[i].rowIndex, colIndex: val.selCells[i].colIndex };
                    this.XLEdit._updateDataContainer({rowIndex: val.selCells[i].rowIndex, colIndex: val.selCells[i].colIndex}, { "comment": val.selCells[i].prevComment });
                    this.XLSelection.selectRange(cell, cell);
                    this.XLComment.setComment();
                    this.XLComment._updateCurCmntVal();
                }
            }
            else {
                this.XLSelection.selectRange(startCell, endCell);
                this.XLComment.deleteComment(undefined, "clear");
            }
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
        },

        _undoForClContent: function (val) {
            var selCells = this.getRange(val.range[0], val.range[1], val.range[2], val.range[3]);
            if (this._isUndo) {
                this.updateData($.extend(true, [], val.bData), val.range)
                if (val.bData.length) {
                    for (i = 0, len = selCells.length; i < len; i++) {
                        if (!ej.isNullOrUndefined(val.bData[i]) && val.bData[i].hyperlink) {
                            hldata = val.bData[i].hyperlink;
                            hldata && (!(hldata.webAddr) ? selCells.eq(i).html(ej.buildTag("a", val.bData[i].value, "", { href: "#", "class": "e-hyperlinks" })) : selCells.eq(i).html(ej.buildTag("a", val.bData[i].value, "", { href: hldata.webAddr, target: "_blank", "class": "e-hyperlinks" })));
                        }
                    }
                }
            }
            else
                this.clearRangeData(val.range, ["value", "value2"]);
            for (var i = val.range[0]; i < val.range[2] + 1; i++) {
                for (var j = val.range[1]; j < val.range[3] + 1; j++) {
                    this.model.allowComments && this.XLComment._updateCmntFrmCntnr(i, j);
                }
            }
        },

        _undoForClFormat: function (val) {
            var aRange, len = val.wRange.length > val.hRange.length + 1 ? val.wRange.length : val.hRange.length, actCell;
            len = len > val.cFormatRule.length + 1 ? len : val.cFormatRule.length;
            if(this.model.allowCellFormatting)
            if (this._isUndo)
                this.XLFormat.updateFormat(val.bFormat, val.range);
            else
                this.XLFormat.clearFormat(val.range);
            for (var i = 0; i < len; i++) {
                if (val.wRange[i]) {
                    aRange = this._getAlphaRange(val.sheetIndex, val.wRange[i].rowIndex, val.wRange[i].colIndex, val.wRange[i].rowIndex, val.wRange[i].colIndex);
                    if (this._isUndo)
                        this.setWrapText(aRange);
                    else
                        this.setWrapText(aRange, "unwrap");
                }
                if (val.hRange[i]) {
                    actCell = this.getCell(val.hRange[i].rowIndex, val.hRange[i].colIndex);
                    if (this._isUndo)
                        actCell.find("a").css({ "color": "#428bca", "text-decoration": "underline" }).addClass("e-hyperlinks");
                    else
                        actCell.find("a").css({"color": "#444445", "text-decoration": "none" }).removeClass("e-hyperlinks");
                }
                if (this.model.allowConditionalFormats && val.cFormatRule[i].rule) {
                    if (this._isUndo)
                        this.XLCFormat.setCFRule(val.cFormatRule[i].rule);
                    else {
                        this.performSelection({ rowIndex: val.cFormatRule[i].rowIndex, colIndex: val.cFormatRule[i].colIndex }, { rowIndex: val.cFormatRule[i].rowIndex, colIndex: val.cFormatRule[i].colIndex });
                        this.XLCFormat.clearCF(true);
                    }
                }
            }
        },

        _undoForCLLink: function (val) {
            var startCell = { rowIndex: val.selCells[0].rowIndex, colIndex: val.selCells[0].colIndex }, temp, currentCell,
            endCell = { rowIndex: val.selCells[val.selCells.length - 1].rowIndex, colIndex: val.selCells[val.selCells.length - 1].colIndex };
			this.setActiveCell(val.selCells[0].rowIndex, val.selCells[0].colIndex);
            this.XLSelection.selectRange(startCell, endCell);
            if (this._isUndo) {
				for (var i = 0, len = val.selCells.length; i < len; i++) {
					if (val.selCells[i].prevLink) {
						currentCell = this.getCell(val.selCells[i].rowIndex, val.selCells[i].colIndex);
						(!(val.selCells[i].prevLink.webAddr) ? currentCell.html(ej.buildTag("a.e-hyperlinks", val.selCells[i].prevText, "", { href: "#" })) : currentCell.html(ej.buildTag("a.e-hyperlinks", val.selCells[i].prevText, "", { href: val.selCells[i].prevLink.webAddr, target: "_blank" })));
					}
				}
			}
			else
				this._removeHyperlink("clear", val.selCells);
            this.XLSelection.selectRange(startCell, endCell);
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
            for (var i = 0, len = val.selCells.length; i < len; i++) {
                temp = val.selCells[i].prevLink;
                val.selCells[i].prevLink = val.selCells[i].curLink;
                val.selCells[i].curLink = temp;
            }
        },

        _undoForComment: function (val) {
            var cell, startCell, endCell, cmnt;
            startCell = { rowIndex: val.selCells[0].rowIndex, colIndex: val.selCells[0].colIndex };
            endCell = { rowIndex: val.selCells[val.selCells.length - 1].rowIndex, colIndex: val.selCells[val.selCells.length - 1].colIndex };
            for (var i = 0, len = val.selCells.length; i < len; i++) {
                cell = { rowIndex: val.selCells[i].rowIndex, colIndex: val.selCells[i].colIndex };
                this.XLSelection.selectRange(cell, cell);
                this.getSheet(val.sheetIndex)._startCell = cell;
                if (val.action === "add") {
                    val.selCells[i].prevComment = this.XLEdit.getPropertyValue(cell, "comment");
                    this.XLComment.deleteComment();
                }
                else if (val.action === "edit") {
                    if(this._isRowViewable(val.sheetIndex, val.selCells[i].rowIndex))
                        this.getCell(cell.rowIndex, cell.colIndex).find('.e-comment-txtarea').val(val.selCells[i].prevComment.value);
                    this.XLComment._updateCurCmntVal();
                    cmnt = val.selCells[i].prevComment;
                    val.selCells[i].prevComment = val.selCells[i].curComment;
                    val.selCells[i].curComment = cmnt;
                }
                else {
                    this.XLEdit._updateDataContainer(cell, { "comment": val.selCells[i].prevComment });
                    this.XLComment.setComment([cell.rowIndex, cell.colIndex, cell.rowIndex, cell.colIndex], null, false);
                    this.XLSelection.selectRange(cell, cell);
                    this.XLComment._updateCurCmntVal();
                    cmnt = val.selCells[i].prevComment;
                    val.selCells[i].prevComment = val.selCells[i].curComment;
                    val.selCells[i].curComment = cmnt;
                }
            }
            if (val.action === "add")
                val.action = "delete";
            else if (val.action === "delete")
                val.action = "add";
            this.XLSelection.selectRange(startCell, endCell);
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
        },

        _undoForCondFormat: function (val) {
            var i, j, len, ln, sRule, rules, rule, totRules = [], allRules, actCell;
            if (this._isUndo) {
                if (val.action === "add") {
                    var selCells = this.XLSelection._getSelectedRange({rowIndex: val.range[0], colIndex:  val.range[1]},{rowIndex: val.range[2], colIndex: val.range[3]});
                    rules = this.XLEdit.getPropertyValue(selCells[0].rowIndex, selCells[0].colIndex, 'cFormatRule');
                    val.poppedRule = rules[rules.length - 1];
                    for (i = 0, len = selCells.length; i < len; i++) {
                        allRules = this.XLEdit.getPropertyValue(selCells[i].rowIndex, selCells[i].colIndex, 'cFormatRule');
                        if (!this.isUndefined(allRules)) {
                            allRules.pop();
                            for (j = 0, ln = allRules.length; j < ln; j++) {
                                sRule = allRules[j];
                                if (totRules.length === 0) {
                                    rule = sRule.indexOf("true") > -1 ? sRule.replace("_true", "") : sRule.replace("_false", "");
                                    totRules.push(rule);
                                }
                                else
                                    totRules = this._addCondFormatRule(totRules, sRule);
                            }
                        }
                    }
                    if(this._isRowViewable(val.sheetIndex, selCells.rowIndex)){
                        actCell = this.getCell(selCells.rowIndex, selCells.colIndex, sheetIndex);
                        actCell.removeClass("e-redft e-yellowft e-greenft e-redf e-redt");
                    }
                    this.clearRangeData(null, ["cFormatRule"], selCells);
                    for (i = 0, len = totRules.length; i < len; i++) {
                        rule = totRules[i].split("_");
                        this.XLCFormat._cFormat(rule[0], rule[2], rule[3], rule[4], rule[5]);
                    }
                }
                else if (val.action === "clear") {
                    for (i = 0, len = val.rule.length; i < len; i++)
                        this.XLCFormat.setCFRule(val.rule[i][0]);
                }
            }
            else {
                if (val.action === "add") {
                    if (!ej.isNullOrUndefined(val.poppedRule)) {
                        rule = val.poppedRule.split("_");
                        this.XLCFormat._cFormat(rule[0], rule[2], rule[3], rule[4], rule[5]);
                    }
                }
                else if (val.action === "clear") {
                    this.XLCFormat.clearCF(val.selection);
                }
            }
        },

        _undoForCopyPaste: function (val) {
            var shapeObj = {}, container = this._dataContainer, type, shapeIdx, shapeMgr = this.getSheet(val.pSheetIndex)._shapeMngr, cellObj, m, rLen;
            if (val.operation === "shape") {
                type = val.pShapeId.indexOf("picture") > -1 ? "picture" : "chart";
                shapeMgr[type].some(function (obj, i) {
                    if (val.pShapeId === obj.id) {
                        shapeIdx = i;
                        return true;
                    }
                    return false;
                });
                cellObj = $.extend(true, {}, val.pasteCell.obj);
                if (!ej.isNullOrUndefined(cellObj)) {
                    this.XLShape._picCellIdx = { rowIndex: val.pasteCell.rowIndex, colIndex: val.pasteCell.colIndex };
                    if (type === "chart") {
                        if ("chart" in cellObj) {
                            if (this._isUndo) {
                                shapeObj = $.extend(true, {}, shapeMgr.chart[shapeIdx]);
								if (ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCell.rowIndex]))
									container.sheets[val.pSheetIndex][val.pasteCell.rowIndex] = {};
								container.sheets[val.pSheetIndex][val.pasteCell.rowIndex][val.pasteCell.colIndex] = cellObj;
                                this.XLShape._deleteShape(val.pSheetIndex, this.element.find("#" + shapeObj.id));
                            }
                            else {
                                this.getCell(val.pasteCell.rowIndex, val.pasteCell.colIndex, val.pSheetIndex).addClass("e-activecell");
                                shapeObj = $.extend(true, {}, shapeMgr.chart[shapeIdx]);
                                range = shapeObj.range;
								!(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                this.XLChart.createChart(this._getAlphaRange(val.sheetIndex, range[0], range[1], range[2], range[3]), shapeObj);
                                this.XLSelection.selectRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, val.cSheetIndex);
                            }
                        }
                    }
                    else {
                        if ("picture" in cellObj) {
                            if (this._isUndo) {
                                shapeObj = $.extend(true, {}, shapeMgr.picture[shapeIdx]);
                                if (ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCell.rowIndex]))
                                    container.sheets[val.pSheetIndex][val.pasteCell.rowIndex] = {};
                                container.sheets[val.pSheetIndex][val.pasteCell.rowIndex][val.pasteCell.colIndex] = cellObj;
                                !(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                this.XLShape._deleteShape(val.pSheetIndex, this.element.find("#" + shapeObj.id));
                            }
                            else {
                                shapeObj = $.extend(true, {}, shapeMgr.picture[shapeIdx]);
                                repStr = this._id + "_picture";
								this.XLChart._shapeCnt++;
								this.XLShape._createPicture(val.pSheetIndex, {rowIndex: val.pasteCell.rowIndex, colIndex: val.pasteCell.colIndex}, shapeMgr.sharedPics[shapeObj.data], shapeObj.top, shapeObj.left, shapeObj.width, shapeObj.height, parseInt(shapeObj.id.replace(repStr, "")));
                            }
                        }
                    }
                }
            }
            else {
                var i, j, len, ln, cell, container = this._dataContainer, selTxt = [], repStr, rowIdx, colIdx, range, pObj, nBrdr, m, rLen;
                for (i = 0, len = val.text.length; i < len; i++)
                    for (j = 0, ln = val.text[i].split("\t").length; j < ln; j++)
                        selTxt.push(val.text[i].split("\t")[j]);
                this.element.find(".e-cdata").length && $('td.e-cutright, td.e-cutbottom').removeClass('e-cutright e-cutbottom');
                this.setActiveSheetIndex(val.pSheetIndex);
                for (i = 0, len = val.pasteCells.length; i < len; i++) {
                    if(this._isRowViewable(val.pSheetIndex, val.pasteCells[i].rowIndex)) {
                        cell = this.getCell(val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex, val.pSheetIndex)[0];
                        cell.textContent = (this._isUndo) ? "" : selTxt[i];
                    }
                    cellObj = ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex]) ? {} : container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex];
                    if (!this._isUndo) {
                        if ("isMerge" in val.pasteCells[i].prevObj) {
                            this._mergeCenter = val.pasteCells[i].prevObj.isCenterAlign;
                            this.mergeCells(val.pasteCells[i].prevObj.mRange, true);
                        }
                    }
                    if (!ej.isNullOrUndefined(cellObj)) {
                        if ("picture" in cellObj) {
                            this.XLShape._picCellIdx = { rowIndex: val.pasteCells[i].rowIndex, colIndex: val.pasteCells[i].colIndex };
                            for (m = 0, rLen = cellObj.picture.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.picture[cellObj.picture[m]]);
                                this.XLShape._deleteShape(val.pSheetIndex, this.element.find("#" + shapeObj.id));
                            }
                        }
                        if ("chart" in cellObj) {
                            this.XLShape._picCellIdx = { rowIndex: val.pasteCells[i].rowIndex, colIndex: val.pasteCells[i].colIndex };
                            for (m = 0, rLen = cellObj.chart.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.chart[cellObj.chart[m]]);
                                this.XLShape._deleteShape(val.pSheetIndex, this.element.find("#" + shapeObj.id));
                            }
                        }
                        if ("isMerge" in cellObj) {
                            if (this._isUndo) {
                                val.pasteCells[i].newObj = $.extend(true, {}, container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]);
                                this.unmergeCells(this._getAlphaRange(val.pSheetIndex, val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex, val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex));
                                this.getCell(val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex, val.pSheetIndex).removeAttr("rowspan").removeAttr("colspan");
                                if ("isMerge" in val.pasteCells[i].prevObj) {
                                    this._mergeCenter = val.pasteCells[i].prevObj.isCenterAlign;
                                    this.mergeCells(val.pasteCells[i].prevObj.mRange, true);
                                    this._mergeCenter = false;
                                }
                            }
                        }
                    }
                    ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex]) && (container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex] = {});
                    !ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]) && (overflow = container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]['overflow']);
					container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex] = $.extend(true, {}, val.pasteCells[i].prevObj);
					(overflow) && (container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]['overflow'] = overflow);
                    pObj = val.pasteCells[i].prevObj;
                    val.pasteCells[i].prevObj = val.pasteCells[i].newObj;
                    val.pasteCells[i].newObj = pObj;
                    this._setRowHdrHeight(val.pSheetIndex, val.pasteCells[i].rowIndex);
                    cellObj = !ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex]) ? container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex] : {};
                    rowIdx = val.pasteCells[i].rowIndex; colIdx = val.pasteCells[i].colIndex;
                    this._applyCommentLink(cellObj, {rowIndex: val.pasteCells[i].rowIndex, colIndex: val.pasteCells[i].colIndex}, rowIdx, colIdx);
                    this._isPaste = true;
                    if (!ej.isNullOrUndefined(cellObj)) {
                        if ("picture" in cellObj) {
                            for (m = 0, rLen = cellObj.picture.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.picture[cellObj.picture[m]]);
                                repStr = this._id + "_picture";
                                !(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                this.XLShape._createPicture(val.pSheetIndex, {rowIndex: val.pasteCells[i].rowIndex, colIndex: val.pasteCells[i].colIndex}, shapeMgr.sharedPics[shapeObj.data], shapeObj.top, shapeObj.left, shapeObj.width, shapeObj.height, parseInt(shapeObj.id.replace(repStr, "")));
                            }
                        }
                        if ("chart" in cellObj) {
                            range = cellObj.chart[0].range;
                            this.getCell(val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex, val.pSheetIndex).addClass("e-activecell");
                            for (m = 0, rLen = cellObj.chart.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.chart[cellObj.chart[m]]);
                                !(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                this.XLChart.createChart(this._getAlphaRange(val.pSheetIndex, range[0], range[1], range[2], range[3]), shapeObj);
                            }
                            this.XLSelection.selectRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, val.pSheetIndex);
                        }
                        this._isPaste = false;
                    }
                }
            }
            if (val.isSpecial) {
                if (this.model.allowCellFormatting) {
                    this.XLFormat.clearFormat(val.pasteBorder.range, "format");
                    !this.isUndefined(val.pasteBorder.prevBorder) && this.XLFormat.updateFormat(val.pasteBorder.prevBorder, val.pasteBorder.range, "format"); //commented due to border issues
                }
                var nBrdr = val.pasteBorder.prevBorder;
                val.pasteBorder.prevBorder = val.pasteBorder.newBorder;
                val.pasteBorder.newBorder = nBrdr;
            }
            //table and filter
            if (!ej.isNullOrUndefined(val.pfat)) {
                var tab = val.pfat.ptable;
                for (var m = 0, rLen = tab.length; m < rLen; m++) {
                    if (!ej.isNullOrUndefined(val.pSheetIndex)) {
                        this.setActiveSheetIndex(val.pSheetIndex);
                        if (this._isUndo)
                            this.XLFormat.removeTable($.trim(tab[m].name.replace("Table", "")));
                        else
                        {
                            tab[m].header = true;
                            this.XLFormat.createTable(tab[m], tab[m].range);
                        }
                    }
                }
            }
            if (!ej.isNullOrUndefined(val.filter)) {
                this.XLFilter.clearFilter();
                this.XLFilter.filter(details.filter);
            }
            if (this._isUndo) {
                (!ej.isNullOrUndefined(val.cSheetIndex)) && this.setActiveSheetIndex(val.cSheetIndex);
                (val.cutCells.length > 0) && this.XLSelection._focusBorder(val.cutCells[0], val.cutCells[val.cutCells.length - 1], this._cutFocus);
                this.setActiveSheetIndex(val.pSheetIndex);
                this.XLSelection.selectRange(val.pasteCells[0], val.pasteCells[val.pasteCells.length - 1]);
            }
            else {
                (!ej.isNullOrUndefined(val.cSheetIndex)) && this.setActiveSheetIndex(val.cSheetIndex);
                (val.cutCells.length > 0) && this.XLSelection._focusBorder(val.cutCells[0], val.cutCells[val.cutCells.length - 1], this._cutFocus);
                this.setActiveSheetIndex(val.pSheetIndex);
                (val.pasteCells.length > 0) && this.XLSelection.selectRange(val.pasteCells[0], val.pasteCells[val.pasteCells.length - 1]);

            }
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
        },

        _undoForCutPaste: function (val) {
            var cellObj, overflow, m, rLen, shapeObj = {}, container = this._dataContainer, shapeMgr = this.getSheet(val.pSheetIndex)._shapeMngr, repStr, cell;
            if (val.operation === "shape") {
                var type, shapeIdx;
                type = val.pShapeId.indexOf("picture") > -1 ? "picture" : "chart";
                shapeMgr[type].some(function (obj, i) {
                    if (val.pShapeId === obj.id) {
                        shapeIdx = i;
                        return true;
                    }
                    return false;
                });
                cellObj = $.extend(true, {}, val.pasteCell.obj);
                if (!ej.isNullOrUndefined(cellObj)) {
                    this.XLShape._picCellIdx = { rowIndex: val.pasteCell.rowIndex, colIndex: val.pasteCell.colIndex };
                    shapeObj = $.extend(true, {}, shapeMgr[type][shapeIdx]);
					if (ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCell.rowIndex]))
                        container.sheets[val.pSheetIndex][val.pasteCell.rowIndex] = {};
                    container.sheets[val.pSheetIndex][val.pasteCell.rowIndex][val.pasteCell.colIndex] = cellObj;
                    this.XLShape._deleteShape(val.pSheetIndex, this.element.find("#" + shapeObj.id));
                }
                cellObj = $.extend(true, {}, val.cutCell.obj);
                type = val.cShapeId.indexOf("picture") > -1 ? "picture" : "chart";
                shapeMgr[type].some(function (obj, i) {
                    if (val.cShapeId === obj.id) {
                        shapeIdx = i;
                        return true;
                    }
                    return false;
                });
                if (!ej.isNullOrUndefined(cellObj)) {
                    this.getCell(val.cutCell.rowIndex, val.cutCell.colIndex, val.cSheetIndex).addClass("e-activecell");
                    if (type === "chart") {
                        if ("chart" in cellObj) {
                            shapeObj = $.extend(true, {}, shapeMgr.chart[shapeIdx]);
                            range = shapeObj.range;
							!(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                            this.XLChart.createChart(this._getAlphaRange(val.sheetIndex, range[0], range[1], range[2], range[3]), shapeObj);
                            this.XLSelection.selectRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, val.cSheetIndex);
                        }
                    }
                    else {
                        if ("picture" in cellObj) {
                            cell = this.getCell(val.cutCell.rowIndex, val.cutCell.colIndex, val.cSheetIndex)[0];
                            shapeObj = $.extend(true, {}, shapeMgr.picture[shapeIdx]);
                            repStr = this._id + "_picture";
							(this._isUndo) ? (!(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--) : this.XLChart._shapeCnt++;
							this.XLShape._createPicture(val.cSheetIndex, {rowIndex: val.pasteCells.rowIndex, colIndex: val.pasteCells.colIndex}, shapeMgr.sharedPics[shapeObj.data], shapeObj.top, shapeObj.left, shapeObj.width, shapeObj.height, parseInt(shapeObj.id.replace(repStr, "")));
                        }
                    }
                }
                this.XLShape._picCellIdx = { rowIndex: val.cutCell.rowIndex, colIndex: val.cutCell.colIndex };
                var pasteCell = val.pasteCell;
                val.pasteCell = val.cutCell;
                val.cutCell = pasteCell;
                var pShapeId = val.pShapeId;
                val.pShapeId = val.cShapeId;
                val.cShapeId = pShapeId;

            }
            else {
                var curCell, pObj, rowIdx, colIdx, cell, container = this._dataContainer, selTxt = [], i, j, len, ln, range;
                this.element.find(".e-cdata").length && $('td.e-cutright, td.e-cutbottom').removeClass('e-cutright e-cutbottom');
                for (i = 0, len = val.text.length; i < len; i++)
                    for (j = 0, ln = val.text[i].split("\t").length; j < ln; j++)
                        selTxt.push(val.text[i].split("\t")[j]);
                for (i = 0, len = val.pasteCells.length; i < len; i++) {
                    if (!ej.isNullOrUndefined(val.pSheetIndex)) {
                        if (this._isRowViewable(val.pSheetIndex, val.pasteCells[i].rowIndex)) {
                            cell = this.getCell(val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex, val.pSheetIndex)[0];
                            cell.textContent = "";
                        }
                        if(!this.isUndefined(container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex]))
                            cellObj = container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex];
                        if (!this.isUndefined(cellObj)) {
                            if ("picture" in cellObj) {
                                this.XLShape._picCellIdx = { rowIndex: val.pasteCells[i].rowIndex, colIndex: val.pasteCells[i].colIndex };
                                for (m = 0, rLen = cellObj.picture.length; m < rLen; m++) {
                                    shapeObj = $.extend(true, {}, shapeMgr.picture[cellObj.picture[m]]);
                                    this.XLShape._deleteShape(val.pSheetIndex, this.element.find("#" + shapeObj.id));
                                }
                            }
                            if ("chart" in cellObj) {
                                this.XLShape._picCellIdx = { rowIndex: val.pasteCells[i].rowIndex, colIndex: val.pasteCells[i].colIndex };
                                for (m = 0, rLen = cellObj.chart.length; m < rLen; m++) {
                                    shapeObj = $.extend(true, {}, shapeMgr.chart[cellObj.chart[m]]);
                                    this.XLShape._deleteShape(val.pSheetIndex, this.element.find("#" + shapeObj.id));
                                }
                            }
                            if ("isMerge" in cellObj) {
                                if (this._isUndo) {
                                    $.extend(true, val.pasteCells[i].newObj, container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]);
                                    this.setActiveSheetIndex(val.pSheetIndex);
                                    this.unmergeCells(cellObj.mRange);
                                    this.getCell(val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex, val.pSheetIndex).removeAttr("rowspan").removeAttr("colspan");
                                    if ("isMerge" in val.pasteCells[i].prevObj) {
                                        this._mergeCenter = val.pasteCells[i].prevObj.isCenterAlign;
                                        this.mergeCells(val.pasteCells[i].prevObj.mRange, true);
                                        this._mergeCenter = false;
                                    }
                                }
                                else {
                                    this.setActiveSheetIndex(val.pSheetIndex);
                                    this.unmergeCells(cellObj.mRange, true);
                                    if ("isMerge" in val.pasteCells[i].prevObj) {
                                        this._mergeCenter = val.pasteCells[i].prevObj.isCenterAlign;
                                        this.mergeCells(val.pasteCells[i].prevObj.mRange, true);
                                        this._mergeCenter = false;
                                    }
                                    this.setActiveSheetIndex(val.cSheetIndex);
                                    this.unmergeCells(this._getAlphaRange(val.pSheetIndex, val.cutCells[i].rowIndex, val.cutCells[i].colIndex, val.cutCells[i].rowIndex, val.cutCells[i].colIndex));
                                    range = this.getRangeIndices(val.cutCells[i].prevObj.mRange);
                                    this.XLSelection.selectRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, val.sheetIndex);
                                }
                            }
                        }
                        if(this.isUndefined(container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex]))
                            container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex] = {};
                        !ej.isNullOrUndefined(container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]) && (overflow = container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]['overflow']);
						container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex] = $.extend(true, {}, val.pasteCells[i].prevObj);
						(overflow) && (container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex]['overflow'] = overflow);
                        pObj = val.pasteCells[i].prevObj;
                        val.pasteCells[i].prevObj = val.pasteCells[i].newObj;
                        val.pasteCells[i].newObj = pObj;
                        this._setRowHdrHeight(val.pSheetIndex, val.pasteCells[i].rowIndex);
                        cellObj = container.sheets[val.pSheetIndex][val.pasteCells[i].rowIndex][val.pasteCells[i].colIndex];
                        rowIdx = val.pasteCells[i].rowIndex, colIdx = val.pasteCells[i].colIndex;
                        this._applyCommentLink(cellObj,  {rowIndex: rowIdx, colIndex: colIdx}, rowIdx, colIdx);
                        this._isPaste = true;
                        if (!ej.isNullOrUndefined(cellObj)) {
                            if ("picture" in cellObj) {
                                for (m = 0, rLen = cellObj.picture.length; m < rLen; m++) {
                                    shapeObj = $.extend(true, {}, shapeMgr.picture[cellObj.picture[m]]);
                                    repStr = this._id + "_picture";
                                    !(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                    this.XLShape._createPicture(val.pSheetIndex, {rowIndex: rowIdx, colIndex: colIdx}, shapeMgr.sharedPics[shapeObj.data], shapeObj.top, shapeObj.left, shapeObj.width, shapeObj.height, parseInt(shapeObj.id.replace(repStr, "")));
                                }
                            }
                            if ("chart" in cellObj) {
                                this.getCell(val.pasteCells[i].rowIndex, val.pasteCells[i].colIndex, val.pSheetIndex).addClass("e-activecell");
                                for (m = 0, rLen = cellObj.chart.length; m < rLen; m++) {
                                    shapeObj = $.extend(true, {}, shapeMgr.chart[cellObj.chart[m]]);
                                    range = shapeObj.range;
                                    !(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                    this.XLChart.createChart(this._getAlphaRange(val.sheetIndex, range[0], range[1], range[2], range[3]), shapeObj);
                                }
                                this.XLSelection.selectRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, val.pSheetIndex);
                            }
                        }
                        this._isPaste = false;
                    }
                }
                if (!ej.isNullOrUndefined(val.cSheetIndex)) {
                    if (this._isRowViewable(val.cSheetIndex, val.cutCells[i].rowIndex)) {
                        curCell = this.getCell(val.cutCells[i].rowIndex, val.cutCells[i].colIndex, val.cSheetIndex)[0];
                        curCell.textContent = selTxt[i];
                    }
                    this.setActiveSheetIndex(val.cSheetIndex);
                    cellObj = ej.isNullOrUndefined(container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex]) ? {} : container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex][val.cutCells[i].colIndex];
                    if (!ej.isNullOrUndefined(cellObj)) {
                        if ("picture" in cellObj) {
                            this.XLShape._picCellIdx = { rowIndex: val.cutCells[i].rowIndex, colIndex: val.cutCells[i].colIndex };
                            for (m = 0, rLen = cellObj.picture.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.picture[cellObj.picture[m]]);
                                this.XLShape._deleteShape(val.cSheetIndex, this.element.find("#" + shapeObj.id));
                            }
                        }
                        if ("chart" in cellObj) {
                            this.XLShape._picCellIdx = { rowIndex: val.cutCells[i].rowIndex, colIndex: val.cutCells[i].colIndex };
                            for (m = 0, rLen = cellObj.chart.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.chart[cellObj.chart[m]]);
                                this.XLShape._deleteShape(val.cSheetIndex, this.element.find("#" + shapeObj.id));
                            }
                        }
                    }
                    (ej.isNullOrUndefined(container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex])) && (container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex] = {});
                    !ej.isNullOrUndefined(container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex][val.cutCells[i].colIndex]) && (overflow = container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex][val.cutCells[i].colIndex]['overflow']);
					container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex][val.cutCells[i].colIndex] = $.extend(true, {}, val.cutCells[i].prevObj);
					(overflow) && (container.sheets[val.cSheetIndex][val.cutCells[i].rowIndex][val.cutCells[i].colIndex]['overflow'] = overflow);
                    pObj = val.cutCells[i].prevObj;
                    val.cutCells[i].prevObj = val.cutCells[i].newObj;
                    val.cutCells[i].newObj = pObj;
                    rowIdx = val.cutCells[i].rowIndex; colIdx = val.cutCells[i].colIndex;
                    this._setRowHdrHeight(val.cSheetIndex, rowIdx);
                    cellObj = ej.isNullOrUndefined(container.sheets[val.cSheetIndex][rowIdx]) ? {} : container.sheets[val.cSheetIndex][rowIdx][colIdx];
                    this._applyCommentLink(cellObj, {rowIndex: rowIdx, colIndex: colIdx}, rowIdx, colIdx);
                    this._isPaste = true;
                    if (!ej.isNullOrUndefined(cellObj)) {
                        if ("picture" in cellObj) {
                            for (m = 0, rLen = cellObj.picture.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.picture[cellObj.picture[m]]);
                                repStr = this._id + "_picture";
                                !(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                this.XLShape._createPicture(val.cSheetIndex, {rowIndex: rowIdx, colIndex: colIdx}, shapeMgr.sharedPics[shapeObj.data], shapeObj.top, shapeObj.left, shapeObj.width, shapeObj.height, parseInt(shapeObj.id.replace(repStr, "")));
                            }
                        }
                        if ("chart" in cellObj) {
                            this.getCell(val.cutCells[i].rowIndex, val.cutCells[i].colIndex, val.cSheetIndex).addClass("e-activecell");
                            for (m = 0, rLen = cellObj.chart.length; m < rLen; m++) {
                                shapeObj = $.extend(true, {}, shapeMgr.chart[cellObj.chart[m]]);
                                range = shapeObj.range;
                                !(this.XLChart._shapeCnt === 1) && this.XLChart._shapeCnt--;
                                this.XLChart.createChart(this._getAlphaRange(val.cSheetIndex, range[0], range[1], range[2], range[3]), shapeObj);
                            }
                            this.XLSelection.selectRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, val.cSheetIndex);
                        }
                        if ("isMerge" in cellObj) {
                            this._mergeCenter = cellObj.isCenterAlign;
                            this.mergeCells(cellObj.mRange, true);
                            this._mergeCenter = false;
                        }
                    }
                    this._isPaste = false;
                }
            }
            if (val.isSpecial) {
                if (this.model.allowCellFormatting && !ej.isNullOrUndefined(val.pSheetIndex)) {
                    this.setActiveSheetIndex(val.pSheetIndex);
                    this.XLFormat.clearFormat(val.pasteBorder.range, "format");
                    !this.isUndefined(val.pasteBorder.prevBorder) && this.XLFormat.updateFormat(val.pasteBorder.prevBorder, val.pasteBorder.range, "format");      //commented due to border issues
                }
                nBrdr = val.pasteBorder.prevBorder;
                val.pasteBorder.prevBorder = val.pasteBorder.newBorder;
                val.pasteBorder.newBorder = nBrdr;
                if (this.model.allowCellFormatting && !ej.isNullOrUndefined(val.cSheetIndex)) {
                    this.setActiveSheetIndex(val.cSheetIndex);
                    this.XLFormat.clearFormat(val.cutBorder.range, "format");
                    !this.isUndefined(val.pasteBorder.prevBorder) && this.XLFormat.updateFormat(val.cutBorder.prevBorder, val.cutBorder.range, "format");
                }
                nBrdr = val.cutBorder.prevBorder;
                val.cutBorder.prevBorder = val.cutBorder.newBorder;
                val.cutBorder.newBorder = nBrdr;
            }
            //table and filter
            if (!ej.isNullOrUndefined(val.pfat)) {
                var rng, tab = val.pfat.table, tblRange, minRowIdx = val.pfat.minRowIdx, minColIdx = val.pfat.minColIdx;
                range = val.pfat.range;
                for (var m = 0, rLen = tab.length; m < rLen; m++) {
                    tblRange = tab[m].range;
                    if (!ej.isNullOrUndefined(val.pSheetIndex)) {
                        this.setActiveSheetIndex(val.pSheetIndex);
                        this.XLFormat.removeTable($.trim(tab[m].tblObj.name.replace("Table", "")));
                    }
                    if (!ej.isNullOrUndefined(val.cSheetIndex)) {
                        this.setActiveSheetIndex(val.cSheetIndex);
                        if (this._isUndo)
                            this.XLFormat.createTable(tab[m].tblObj, tab[m].range);
                        else {
                            rng = this.getRangeIndices(this._getAlphaRange(val.cSheetIndex, tblRange[0] + minRowIdx - range[0], tblRange[1] + minColIdx - range[1], minRowIdx - range[0] + tblRange[2], minColIdx - range[1] + tblRange[3]));
                            this.XLFormat.createTable(tab[m].tblObj, rng);
                        }
                    }
                }
            }
            if (!ej.isNullOrUndefined(val.filter)) {
                this.XLFilter.clearFilter();
                this.XLFilter.filter(details.filter);
            }
            if (this._isUndo) {
                if (!ej.isNullOrUndefined(val.cSheetIndex)) {
                    this.setActiveSheetIndex(val.cSheetIndex);
                    this.XLSelection.selectRange(val.cutCells[0], val.cutCells[val.cutCells.length - 1]);
                }
                if (!ej.isNullOrUndefined(val.pSheetIndex)) {
                    this.setActiveSheetIndex(val.pSheetIndex);
                    this.XLSelection.selectRange(val.pasteCells[0], val.pasteCells[val.pasteCells.length - 1]);
                }
            }
            else {
                if (!ej.isNullOrUndefined(val.pSheetIndex)) {
                    this.setActiveSheetIndex(val.pSheetIndex);
                    this.XLSelection._focusBorder(val.pasteCells[0], val.pasteCells[val.pasteCells.length - 1], this._cutFocus);
                    this.XLSelection.selectRange(val.pasteCells[0], val.pasteCells[val.pasteCells.length - 1]);
                }
                if (!ej.isNullOrUndefined(val.cSheetIndex)) {
                    this.setActiveSheetIndex(val.cSheetIndex);
                    this.XLSelection.selectRange(val.cutCells[0], val.cutCells[val.cutCells.length - 1]);
                }
            }
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
            var pasteCells = val.pasteCells;
            val.pasteCells = val.cutCells;
            val.cutCells = pasteCells;
            var pasteBorder = val.pasteBorder;
            val.pasteBorder = val.cutBorder;
            val.cutBorder = pasteBorder;
            var tempIdx = val.cSheetIndex;
            val.cSheetIndex = val.pSheetIndex;
            val.pSheetIndex = tempIdx;       
        },
        
        _undoForDel: function (val) {
            $.extend(true, this._delCells, val.delCells);
            this._deletedText = $.extend(true, [], val.text);
            switch (val.action) {
                case "deleteShiftUp":
                    this.deleteShiftUp(val.startCell, val.endCell);
                    break;
                case "deleteShiftLeft":
                    this.deleteShiftLeft(val.startCell, val.endCell);
                    break;
                case "deleteRow":
                    this.deleteEntireRow(val.startCell, val.endCell);
                    break;
                case "deleteColumn":
                    this.deleteEntireColumn(val.startCell, val.endCell);
                    break;
            }
        },

        _undoForEdit: function (val) {
            var hLink, curVal, temp, cFormatStr, cFormatRule = val.cFormatRule, selCell = { rowIndex: val.rowIndex, colIndex: val.colIndex }, cmnt, cell, textDecoration;
            this.setActiveCell(val.rowIndex, val.colIndex, val.sheetIndex);
            this.XLSelection.selectRange(selCell, selCell);
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
			this._dupDetails = true;
			if(cFormatRule && cFormatRule.length) {
				for (i = 0, len = cFormatRule.length; i < len; i++) {
					cFormatStr = cFormatRule[i].split("_");
                    this.XLCFormat._cFormat(cFormatStr[0], cFormatStr[2], cFormatStr[3], cFormatStr[4], cFormatStr[5]);
                }
            }
			this._dupDetails = false;
			if(this._isRowViewable(val.sheetIndex, val.rowIndex)){
			    cell = this.getCell(val.rowIndex, val.colIndex);
			    textDecoration = cell.find("a").css("text-decoration");
			}
			hLink =  this.XLEdit.getPropertyValue(val.rowIndex, val.colIndex, "hyperlink", val.sheetIndex);
			if (!ej.isNullOrUndefined(hLink)) {
			    curVal = this.XLEdit.getPropertyValue(val.rowIndex, val.colIndex, "value2");
			    val.action = (curVal.length < 1) ? "add" : "delete";
                val.reqType = "edit";
                selCell.prevLink = hLink;
                selCell.prevText = curVal;
                val.selCells = [];
            }
            cmnt = this.XLEdit.getPropertyValue(val.rowIndex, val.colIndex, "comment");
            this.XLEdit.updateCellValue({rowIndex: val.rowIndex, colIndex: val.colIndex}, val.pValue);
			this._textClip(val.rowIndex, val.colIndex, (val.pValue.length > 0) ? 'add' : 'delete');
            if (!ej.isNullOrUndefined(cmnt)) {
                this.XLEdit._updateDataContainer({rowIndex: val.rowIndex, colIndex: val.colIndex}, { "comment": cmnt });
                this.XLComment.setComment();
                this.XLComment._updateCurCmntVal();
            }
            if(this._isRowViewable(val.sheetIndex, val.rowIndex)){
                hLink && (!(hLink.webAdr) ? cell.html(ej.buildTag("a", cell.text(), "", { href: "#", "class": "e-hyperlinks" })) : cell.html(ej.buildTag("a", cell.text(), "", { href: hldata.webAdr, target: "_blank", "class": "e-hyperlinks" })));
                cell.find("a").css("text-decoration", textDecoration);
                cell.removeClass("e-commentcell");
            }
			if (!ej.isNullOrUndefined(hLink)) {
                selCell.curLink = this.XLEdit.getPropertyValue(val.rowIndex, val.colIndex, "hyperlink");
                selCell.curText = this.XLEdit.getPropertyValue(val.rowIndex, val.colIndex, "value2");
                val.selCells.push(selCell);
            }
            this._setRowHdrHeight(val.sheetIndex, val.rowIndex);
            temp = val.cValue;
            val.cValue = val.pValue;
            val.pValue = temp;
            this.XLEdit._isEdit = false;
            temp = val.previousValue;
            val.previousValue = val.value;
            val.value = temp;
        },

        _undoForFilter: function (val) {
            var fltrChk = false, filterdCol = [], prevFltr, cell, selCells = [], i, len, fltr;
            if (val.operation === "filter" && val.requestType !== "filterbycell") {
                if (this._isUndo) {
                    for (i = this._undoCollection.length - 1; i >= 0 ; i--) {
                        if (this._undoCollection[i].reqType === "filter" && this._undoCollection[i].tableID === val.tableID && this._undoCollection[i].action === "filtering" && this._undoCollection[i].status !== "apply-filterIcon") {
                            delete this._excelFilter._predicates[val.sheetIndex][val.fieldName];
                            prevFltr = this._undoCollection[i];
                            this.XLFilter._clearFilterColumn(val.fieldName, "", "", "Or", null, null, prevFltr.tableID);
                            for (i = 0, len = prevFltr.filterCollection.length; i < len; i++)
                                filterdCol.push(prevFltr.filterCollection[i]);
                            this.model.sheets[prevFltr.sheetIndex].filterSettings.filteredColumns = filterdCol;
                            if (ej.isNullOrUndefined(this._excelFilter._predicates[prevFltr.sheetIndex][prevFltr.fieldName]))
                                this._excelFilter._predicates[prevFltr.sheetIndex][prevFltr.fieldName] = {};
                            this._excelFilter._predicates[prevFltr.sheetIndex][prevFltr.fieldName] = prevFltr.ejpredicate;
                            this.model.sheets[prevFltr.sheetIndex]._selectedRange = prevFltr.range;
                            this.XLFilter._filterHandler(prevFltr);
                            fltr = "undo";
                            fltrChk = true;
                            break;
                        }
                        if (this._undoCollection[i].requestType === "filterbycell") {
                            prevFltr = this._undoCollection[i];
                            delete this._excelFilter._predicates[prevFltr.sheetIndex][prevFltr.fieldName];
                            this.XLFilter._clearFilterColumn(prevFltr.fieldName, "", "", "Or", null, null, prevFltr.tableID);
                            $.extend(true, selCells, prevFltr.selCells);
                            this.XLFilter._filterActiveCell(prevFltr.sheetIndex, prevFltr.activeCell, selCells, prevFltr.filteredValue);
                            fltr = "undo";
                            fltrChk = true;
                            break;
                        }
                    }
                    if (!fltrChk) {
                        delete this._excelFilter._predicates[val.sheetIndex][val.fieldName];
                        this.XLFilter._clearFilterColumn(val.fieldName, "", "", "Or", null, null, val.tableID);
                    }
                }
                else {
                    filterdCol = [];
                    delete this._excelFilter._predicates[val.sheetIndex][val.fieldName];
                    this.XLFilter._clearFilterColumn(val.fieldName, "", "", "Or", null, null, val.tableID);
                    for (i = 0, len = val.filterCollection.length; i < len; i++)
                        filterdCol.push(val.filterCollection[i]);
                    this.model.sheets[val.sheetIndex].filterSettings.filteredColumns = filterdCol;
                    (ej.isNullOrUndefined(this._excelFilter._predicates[val.sheetIndex][val.fieldName])) && (this._excelFilter._predicates[val.sheetIndex][val.fieldName] = {});
                    this._excelFilter._predicates[val.sheetIndex][val.fieldName] = val.ejpredicate;
					if(val.filterCollection.length)
						this.XLFilter._filterHandler(val);
                    fltr = "redo";
                }
            }
            else {
                if (this._isUndo) {
                    if (val.operation === "apply-filter" && val.requestType !== "filterbycell") {
                        for (i = 0, len = val.filteredCells.length; i < len; i++) {
                            val.filteredCells[i].find(".e-ssfilter").remove();
							this._removeClass(val.filteredCells[i][0], "e-filterhdr");
                            (this.isNumber(val.filteredCells[i].text()) || this.XLEdit._parseValue(val.filteredCells[i].text()).type ==="datetime" || this.XLEdit._parseValue(val.filteredCells[i].text()).type ==="currency" || this.XLEdit._parseValue(val.filteredCells[i].text()).type ==="percentage") && val.filteredCells[i].addClass(this._rAlign);
                        }
					}
					if(val.requestType === "filterbycell") {
						if(!this.isUndefined(val.filteredCells)){
							for (i = 0, len = val.filteredCells.length; i < len; i++) {
								val.filteredCells[i].find(".e-ssfiltered, .e-ssfilter").remove();
								this._removeClass(val.filteredCells[i][0], "e-filterhdr");
								(this.isNumber(val.filteredCells[i].text()) || this.XLEdit._parseValue(val.filteredCells[i].text()).type ==="datetime" || this.XLEdit._parseValue(val.filteredCells[i].text()).type ==="currency" || this.XLEdit._parseValue(val.filteredCells[i].text()).type ==="percentage") && val.filteredCells[i].addClass(this._rAlign);
							}
						}
						delete this._excelFilter._predicates[val.sheetIndex][val.fieldName];
						this.XLFilter._clearFilterColumn(val.fieldName, "", "", "Or", null, null, val.tableID);
					}
                    this.model.sheets[val.sheetIndex].filterSettings.range = "";
					if((val.operation === "apply-filter" && val.requestType != "filterbycell") || (val.requestType === "filterbycell" && !this.isUndefined(val.filteredCells))) {
						var tabRange = this.model.sheets[val.sheetIndex].filterSettings.tableRange, tabIdx;
						for(var i = 0, len = tabRange.length; i < len; i++) {
							if(tabRange[i].tableID === val.tableID) {
								tabIdx = i;
								break;
							}
						}
						val.filterRange = tabRange.splice(tabIdx, 1)[0];
					}
					if(val.operation === "remove-filter") {
						var filterIcon;
                        for (i = 0, len = val.filteredCells.length; i < len; i++) {
                            filterIcon = this.XLFilter._getFilterIcon();
                            this.XLFilter._applyFilterIcon(val.filteredCells[i], filterIcon);
                            val.filteredCells[i].removeClass(this._rAlign);
                        }
                        this.model.sheets[val.sheetIndex].filterSettings.range = val.fRange;
						this.model.sheets[val.sheetIndex].filterSettings.tableRange.push(val.filterRange);
					}				                   
                }
                else {
                    if (val.operation === "remove-filter") {
                        if (val.requestType === "filterbycell") {
                            if(this._isRowViewable(val.sheetIndex, val.activeCell.rowIndex)) {
                                cell = this.getCell(val.activeCell.rowIndex, val.activeCell.colIndex);
                                cell.find("span.e-ssfilter").remove();
                            }
                            delete this._excelFilter._predicates[val.sheetIndex][val.fieldName];
                            this.XLFilter._clearFilterColumn(val.fieldName, "", "", "Or", null, null, val.tableID);
                            $.extend(true, selCells, val.selCells);
                            this.XLFilter._filterActiveCell(val.sheetIndex, val.activeCell, selCells, val.filteredValue);
                        }
                        else 
                            this.XLFilter.clearFilter("remove");
                    }
					if(val.operation === "filter") {
						if (val.requestType === "filterbycell") {
							delete this._excelFilter._predicates[val.sheetIndex][val.fieldName];
                            this.XLFilter._clearFilterColumn(val.fieldName, "", "", "Or", null, null, val.tableID);
                            $.extend(true, selCells, val.selCells);
                            this.XLFilter._filterActiveCell(val.sheetIndex, val.activeCell, selCells, val.filteredValue);
						}
					}
					if(val.operation === "apply-filter") {
						var filterIcon;
                        for (i = 0, len = val.filteredCells.length; i < len; i++) {
                            filterIcon = this.XLFilter._getFilterIcon();
                            this.XLFilter._applyFilterIcon({rowIndex: val.filteredCells[i].rowIndex, colIndex: val.filteredCells[i].colIndex}, filterIcon);
                        }
                        this.model.sheets[val.sheetIndex].filterSettings.range = val.fRange;
						this.model.sheets[val.sheetIndex].filterSettings.tableRange.push(val.filterRange);
					}
                }
            }
        },

        _undoForFilterByColor: function (val) {
            var avble = false, filterColl, coll = [];
			coll.push(val);
            filterColl = (this._isUndo) ? this._undoCollection : coll;
			delete this._excelFilter._predicates[val.sheetIndex][val.columnName];
            this.XLFilter._clearFilterColumn(val.columnName, "", "", "Or", null, null, val.tableID);
            for (var i = filterColl.length - 1; i >= 0 ; i--) {
                if (filterColl[i].reqType === "filterbycolor" && filterColl[i].tableID === val.tableID) {
                    this.XLFilter._filterHandler(filterColl[i]);
                    break;
                }
            }
        },

        _undoForFind: function (val) {
            var cell, i, len, t;
            switch (val.operation) {
                case "replace": {
                    this.XLEdit.updateCell({rowIndex: val.rowIndex, colIndex: val.colIndex}, val.prevData.value, val.sheetIndex);
                    if(this._isRowViewable(val.sheetIndex, val.rowIndex)) {
                        cell = this.getCell(val.rowIndex, val.colIndex, val.sheetIndex);
                        cell.text(val.prevData.value);
                    }
                    t = val.prevData;
                    val.prevData = val.newData;
                    val.newData = t;
                    this.XLSelection.selectRange({ rowIndex: val.rowIndex, colIndex: val.colIndex }, { rowIndex: val.rowIndex, colIndex: val.colIndex });
                    break;
                }
                case "replaceAll": {
                    for (i = 0, len = val.selCells.length; i < len; i++) {
                        this.XLEdit.updateCell({rowIndex: val.selCells[i].rowIndex, colIndex: val.selCells[i].colIndex}, val.selCells[i].prevData.value, val.selCells[i].sheetIndex);
                        if(this._isRowViewable(val.sheetIndex, val.rowIndex)) {
                            cell = this.getCell(val.selCells[i].rowIndex, val.selCells[i].colIndex, val.selCells[i].sheetIndex);
                            cell.text(val.selCells[i].prevData.value);
                        }
                        t = val.selCells[i].prevData;
                        val.selCells[i].prevData = val.selCells[i].newData;
                        val.selCells[i].newData = t;
                    }
                    break;
                }
                case "blanks":
                case "conditions":
                case "validations":
                case "formulas":
                case "rowdiff":
                case "coldiff":
                case "comments":
                case "constants": {
                    for (i = 0, len = val.selCells.length; i < len; i++) {
                        cell = this.getCell(val.selCells[i].rowIndex, val.selCells[i].colIndex, val.sheetIndex);
                        if (this._isUndo) {
                            if (cell.operation === "coldiff" || cell.operation === "rowdiff")
                                (val.selCells[i].isApply) ? cell.removeClass("e-selected") : cell.addClass("e-selected");
                            else 
                                (val.selCells[i].isApply) ? cell.addClass("e-selected") : cell.removeClass("e-selected");
                        }
                        else {
                            cell.removeClass("e-selected");
                        }
                    }
                    break;
                }
            }
        },

        _undoForFormat: function (val) {
            var data = [], range, cell, cElem, container = this._dataContainer, val2, s;
            if (ej.isNullOrUndefined(val.process)) {
                //Cell Format
                if (ej.isNullOrUndefined(val.action) || val.action !== "decimal") {
                    this.XLFormat.clearFormat(val.range);
                    this.XLFormat.updateFormat(val.beforeFormat, val.range);
                    s = val.beforeFormat;
                    val.beforeFormat = val.afterFormat;
                    val.afterFormat = s;
                }
                //Number Format
                range = { startCell: val.selCell[0], endCell: val.selCell[val.selCell.length - 1] };
                for (var i = 0, len = val.selCell.length; i < len; i++) {
                    cell = val.selCell[i];
                    cElem = this.getCell(cell.rowIndex, cell.colIndex);
                    if (!ej.isNullOrUndefined(container.sheets[val.sheetIndex][cell.rowIndex])) {
                        if (!ej.isNullOrUndefined(container.sheets[val.sheetIndex][cell.rowIndex][cell.colIndex])) {
                            container.sheets[val.sheetIndex][cell.rowIndex][cell.colIndex] = val.selCell[i].beforeFormat;
                            val2 = this.XLEdit.getPropertyValueByElem(cElem, "value2");
                            data.push({ data: val2 });
                            cElem[0].innerHTML = ej.isNullOrUndefined(val2) ? "" : val2;
                        }
                    }
					if ((val.selCell[0].colIndex === val.selCell[i].colIndex) && this.model.allowResizing)
                        (this._isUndo) ? this.XLResize.setRowHeight(val.selCell[i].rowIndex, val.selCell[i].bRowHeight) : this.XLResize.setRowHeight(val.selCell[i].rowIndex, val.selCell[i].aRowHeight);
                    s = val.selCell[i].beforeFormat;
                    val.selCell[i].beforeFormat = val.selCell[i].afterFormat;
                    val.selCell[i].afterFormat = s;
                }
            }
            if (!ej.isNullOrUndefined(val.process) && val.process.indexOf("border") > -1) {
                this.XLFormat.clearFormat(val.range);
                this.XLFormat.updateFormat(val.beforeFormat, val.range);
                s = val.beforeFormat;
                val.beforeFormat = val.afterFormat;
                val.afterFormat = s;
            }
            val.status = "undo-cellformat";
        },

        _undoForFormatPaint: function (val) {
            var s, cell, fregx = this._formatRegx, bregx = this._borderRegx;
            for (var i = 0, len = val.selCells.length; i < len; i++) {
                cell = this.getCell(val.selCells[i].rowIndex, val.selCells[i].colIndex)[0];
                cell.className.indexOf("e-format") && (cell.className = cell.className.replace(fregx, ''));
                $(cell).addClass(val.selCells[i].bFormat);
                cell.className.indexOf("e-border") && (cell.className = cell.className.replace(bregx, ''));
                $(cell).addClass(val.selCells[i].bBorder);
                $(cell).text(val.selCells[i].bVal);
                $(cell).data(val.selCells[i].bData);
                s = val.selCells[i].bFormat;
                val.selCells[i].bFormat = val.selCells[i].aFormat;
                val.selCells[i].aFormat = s;
                s = val.selCells[i].bBorder;
                val.selCells[i].bBorder = val.selCells[i].aBorder;
                val.selCells[i].aBorder = s;
                s = val.selCells[i].bData;
                val.selCells[i].bBorder = val.selCells[i].aData;
                val.selCells[i].aBorder = s;
                s = val.selCells[i].bVal;
                val.selCells[i].bVal = val.selCells[i].aVal;
                val.selCells[i].aVal = s;
            }
        },

        _undoForFormatTable: function (val) {
            var j, ln, hdrCol, rangeData, sheetIdx = val.sheetIndex, tmgr = this.getSheet(sheetIdx)._tableManager, tclass, proxy = this, header = val.header;
            switch (val.action) {
                case "convertorange":
                    if (this._isUndo) {
                        tmgr[val.tableID] = { name: "Table" + val.tableID, range: val.beforeRange, format: val.format, formatName: val.formatName };
                        this.XLFilter._filterCellRange(sheetIdx, this._getProperAlphaRange(sheetIdx, tmgr[val.tableID].range[0], tmgr[val.tableID].range[1], tmgr[val.tableID].range[2], tmgr[val.tableID].range[3]));
                        for (var i = tmgr[val.tableID].range[1], len = tmgr[val.tableID].range[3] + 1; i < len; i++)
                            for (j = tmgr[val.tableID].range[0], ln = tmgr[val.tableID].range[2] + 1; j < ln; j++)
                                this.getCell(j, i, sheetIdx).addClass("e-table" + val.tableID).find('span').addClass("e-ft" + val.tableID);
                        this.model.sheets[sheetIdx].filterSettings.tableRange[val.tableID] = val.tableRange;
						tmgr[val.tableID]["isFilter"] = val.isFilter;
						tmgr[val.tableID]["firstColumn"] = val.firstColumn;
						tmgr[val.tableID]["lastColumn"] = val.lastColumn;
						tmgr[val.tableID]["totalRow"] = val.totalRow;
                    }
                    else {
                        if (val.tableID in tmgr) {
                            tclass = 'e-table' + val.tableID;
                            proxy.element.find('.' + tclass).removeClass(tclass);
                            proxy.XLFilter._clearFilterTable(sheetIdx, val.tableID);
                            delete tmgr[val.tableID];
                            delete this.model.sheets[sheetIdx]._header[val.tableID];
                        }
                    }
                    break;
                case "filtericon":
                    var fIcon = $("#" + val.id).data("ejCheckBox");
                    (val.check) ? fIcon.option("checked", false) : fIcon.option("checked", true);
                    val.check = !(val.check);
                    this.XLFilter._clearFilterTableIcon(val.sheetIndex, parseInt(val.tableId));
                    break;
                case "firstlastcolumn":
                    var cc = $("#" + val.id).data("ejCheckBox");
                    if (val.check) {
                        this.XLFormat.format({ style: { 'font-weight': 'normal' } }, val.range);
                        cc.option("checked", false);
                    }
                    else {
                        this.XLFormat.format({ style: { 'font-weight': 'bold' } }, val.range);
                        cc.option("checked", true);
                    }
                    val.check = !(val.check);
                    break;
                case "totalrow":
                    var tr = $("#" + val.id).data("ejCheckBox");
                    if (val.check) {
                        this.XLEdit.updateCellValue($(val.cell[0]), val.pcells[0].value2);
                        this.XLEdit.updateCellValue($(val.cell[val.cell.length - 1]), val.pcells[1].value2);
                        "totalRow" in val.tmgr[val.tid] && delete val.tmgr[val.tid]["totalRow"];
                        tr.option("checked", false);
                    }
                    else {
                        this.XLEdit.updateCellValue($(val.cell[0]), 'Total');
                        this.XLEdit.updateCellValue($(val.cell[val.cell.length - 1]), '=SUBTOTAL(' + tmgr[val.tid]["fnNumber"] + ', ' + val.range + ')');
                        val.tmgr[val.tid]["totalRow"] = true;
                        tr.option("checked", true);
                    }
                    val.check = !(val.check);
                    break;
                case "resizetable":
                    if (this._isUndo) {
                        this._isFATResize = true;
                        this.XLFormat.removeTable(val.tableId);
                        this.XLFormat.clearFormat(val.afterRange);
                        hdrCol = val.beforeRange[3] - val.beforeRange[1] + 1;
                        rangeData = header[val.tableId].header ? this.getRangeData(val.beforeRange) : this.getRangeData(val.beforeRange).splice(hdrCol, this.getRangeData(val.beforeRange).length);
                        this.clearRangeData(val.beforeRange);
                        header[val.tableId].header ? this.updateData(rangeData, [val.beforeRange[0], val.beforeRange[1], val.beforeRange[2], val.beforeRange[3]]) : this.updateData(rangeData, [val.beforeRange[0], val.beforeRange[1], val.beforeRange[2] - 1, val.beforeRange[3]]);
                        tmgr[val.tableId] = { header: header[val.tableId].header, name: "Table" + val.tableId, range: val.beforeRange, format: val.format, formatName: val.formatName };
                        header[val.tableId].header ? this.XLFormat.createTable(tmgr[val.tableId], val.beforeRange) : this.XLFormat.createTable(tmgr[val.tableId], [val.beforeRange[0], val.beforeRange[1], val.beforeRange[2] - 1, val.beforeRange[3]]);
                    }
                    else {
                        this._isFATResize = true;
                        this.XLFormat.removeTable(val.tableId);
                        this.XLFormat.clearFormat(val.beforeRange);
                        hdrCol = val.beforeRange[3] - val.beforeRange[1] + 1;
                        rangeData = header[val.tableId].header ? this.getRangeData(val.afterRange) : this.getRangeData(val.afterRange).splice(hdrCol, this.getRangeData(val.afterRange).length);
                        this.clearRangeData(val.afterRange);
                        header[val.tableId].header ? this.updateData(rangeData, val.afterRange) : this.updateData(rangeData, [val.afterRange[0], val.afterRange[1], val.afterRange[2] - 1, val.afterRange[3]]);
                        tmgr[val.tableId] = { header: header[val.tableId].header, name: "Table" + val.tableId, range: val.beforeRange, format: val.format, formatName: val.formatName };
                        header[val.tableId].header ? this.XLFormat.createTable(tmgr[val.tableId], val.afterRange) : this.XLFormat.createTable(tmgr[val.tableId], [val.afterRange[0], val.afterRange[1], val.afterRange[2] - 1, val.afterRange[3]]);
                    }
                    break;
                default:
                    if (this._isUndo) {
                        this.XLFormat.removeTable(val.tableID);
                        this.XLFormat.clearFormat(val.afterRange);
                        if (!val.tableStyle.header) {
                            hdrCol = val.afterRange[3] - val.afterRange[1] + 1;
                            this.clearRangeData(val.afterRange);
                            this.updateData($.extend(true, [], val.beforeRangeData), val.afterRange);
                        }
						else
						{
							for(j = 0, ln = val.beforeRangeHeader.length; j < ln; j++)
							{
								if(ej.isNullOrUndefined(val.beforeRangeHeader[j].value2))
									this.XLEdit.updateCellValue($(val.pcells[j]), "");
								else 
									this.isNumber(val.pcells[j].innerHTML) && this.XLEdit._refreshCellAlignment(val.pcells[j])
							}
						}
                        this.XLFormat.updateFormat(val.beforeFormat, val.afterRange);
                    }
                    else {
                        tmgr[val.tableID] = { name: "Table" + val.tableID, range: val.beforeRange };
                        this.XLFormat.createTable(val.tableStyle, val.beforeRange);
                    }
                    break;
            }
        },

        _undoForHyperlink: function (val) {
            var endCell = { rowIndex: val.selCells[val.selCells.length - 1].rowIndex, colIndex: val.selCells[val.selCells.length - 1].colIndex },
            startCell = { rowIndex: val.selCells[0].rowIndex, colIndex: val.selCells[0].colIndex };
			this.setActiveCell(val.selCells[0].rowIndex, val.selCells[0].colIndex);
            this.XLSelection.selectRange(startCell, endCell);
            if (val.action === "add") {
                this.XLSelection.selectRange(startCell, endCell);
                this._removeHyperlink(val.reqType, val.selCells);
            }
            else if (val.action === "edit") {
                this.XLSelection.selectRange(startCell, endCell);
                this._setLink(val.selCells);
            }
            else {
                this.XLSelection.selectRange(startCell, endCell);
                this._setLink(val.selCells);
            }
            for (var i = 0, len = val.selCells.length; i < len; i++) {
                var lnk = val.selCells[i].prevLink;
                val.selCells[i].prevLink = val.selCells[i].curLink;
                val.selCells[i].curLink = lnk;
                var txt = val.selCells[i].prevText;
                val.selCells[i].prevText = val.selCells[i].curText;
                val.selCells[i].curText = txt;
            }
            if (val.action === "add")
                val.action = "delete";
            else if (val.action === "delete")
                val.action = "add";
            this.XLSelection.selectRange(startCell, endCell);
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
        },

        _undoForIns: function (val) {
            $.extend(true, this._delCells, val.delCells);
            $.extend(true, this._deletedText, val.text);
            switch (val.action) {
                case "insertShiftRight":
                    this.insertShiftRight(val.startCell, val.endCell);
                    break;
                case "insertShiftBottom":
                    this.insertShiftBottom(val.startCell, val.endCell);
                    break;
                case "insertRow":
                    this.insertEntireRow(val.startCell, val.endCell);
                    break;
                case "insertColumn":
                    this.insertEntireColumn(val.startCell, val.endCell);
                    break;
            }
        },

        _undoForMerge: function (val) {
            var rng, selectedCells, hiddenCount, pObj, cObj;
            this._delCells = val.hideCells;
            this._selMergeCells = [];
            switch (val.action) {
                case "merge-all":
                case "merge-across":
                case "un-merge":
                    for (var i = 0, len = val.mCells.length; i < len; i++) {
                        pObj = ej.isNullOrUndefined(val.mCells[i].prevObj) ? {} : ej.isNullOrUndefined(val.mCells[i].prevObj.merge) ? {} : val.mCells[i].prevObj.merge;
                        cObj = ej.isNullOrUndefined(val.mCells[i].curObj) ? {} : ej.isNullOrUndefined(val.mCells[i].curObj.merge) ? {} : val.mCells[i].curObj.merge;
                        if (val.action === "un-merge") {
                            if (!ej.isNullOrUndefined(pObj.mRange)) {
                                rng = this.getRangeIndices(pObj.mRange);
                                selectedCells = this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
                                this._mergeCenter = pObj.isCenterAlign;
                                this._mergeCells(val.sheetIndex, selectedCells, {}, []);
                            }
                        }
                        else {
                            hiddenCount = this._getHiddenRowColumnCount(val.sheetIndex, cObj.mRange);
                            this._unMergeAllCells(val.sheetIndex, i, val.mCells, [], hiddenCount.hRow, hiddenCount.hCol);
                        }
                        var nObj = val.mCells[i].prevObj;
                        val.mCells[i].prevObj = val.mCells[i].curObj;
                        val.mCells[i].curObj = nObj;
                    }
                    if (val.action !== "un-merge") {
                        for (var i = 0, len = val.mCells.length; i < len; i++) {
                            pObj = ej.isNullOrUndefined(val.mCells[i].prevObj) ? {} : ej.isNullOrUndefined(val.mCells[i].prevObj.merge) ? {} : val.mCells[i].prevObj.merge;
                            cObj = ej.isNullOrUndefined(val.mCells[i].curObj) ? {} : ej.isNullOrUndefined(val.mCells[i].curObj.merge) ? {} : val.mCells[i].curObj.merge;
                            if (!ej.isNullOrUndefined(cObj.mRange)) {
                                rng = this.getRangeIndices(cObj.mRange);
                                selectedCells = this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
                                this._mergeCenter = cObj.isCenterAlign;
                                this._mergeCells(val.sheetIndex, selectedCells, {}, []);
                            }
                        }
                    }
                    val.action = (val.action === "un-merge") ? "merge-all" : "un-merge";
                    break;
            }
            rng = this._getSelectedCells(val.sheetIndex, val.range);
            selectedCells = this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
            this.performSelection(selectedCells[0], selectedCells[selectedCells.length - 1]);
            this._delCells = [];
        },

        _undoForReOrder: function (val) {
			 var sheet = this.getSheet(val.sheetIndex);
            if (this._isUndo) {
                this.XLComment.deleteComment();
                this.clearRangeData(val.destinationRange);
                this.XLFormat.clearFormat(val.destinationRange);
                this.updateData($.extend(true, [], val.sourceData), val.sourceRange);
                this.updateData($.extend(true, [], val.destinationData), val.destinationRange);
                this.XLFormat.updateFormat(val.sourceFormat, val.sourceRange);
                this.XLFormat.updateFormat(val.destinationFormat, val.destinationRange);
                this.setActiveCell(val.sourceRange[0], val.sourceRange[1]);
                this.XLSelection.selectRange({ rowIndex: val.sourceRange[0], colIndex: val.sourceRange[1] }, { rowIndex: val.sourceRange[2], colIndex: val.sourceRange[3] }, val.sheetIndex);
				 sheet._startCell = {rowIndex: val.sourceRange[0], colIndex: val.sourceRange[1]};
				 sheet._endCell = {rowIndex: val.sourceRange[2], colIndex: val.sourceRange[3]};
                this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
            }
            else {
                this.XLSelection.selectRange({ rowIndex: val.sourceRange[0], colIndex: val.sourceRange[1] }, { rowIndex: val.sourceRange[2], colIndex: val.sourceRange[3] }, val.sheetIndex);
                this.XLComment.deleteComment();
                this.clearRangeData(val.sourceRange);
                this.clearRangeData(val.destinationRange);
                this.XLFormat.clearFormat(val.sourceRange);
                this.XLFormat.clearFormat(val.destinationRange);
                this.updateData($.extend(true, [], val.sourceData), val.destinationRange);
                this.XLFormat.updateFormat(val.sourceFormat, val.destinationRange);
                this.setActiveCell(val.destinationRange[0], val.destinationRange[1]);
                this.XLSelection.selectRange({ rowIndex: val.destinationRange[0], colIndex: val.destinationRange[1] }, { rowIndex: val.destinationRange[2], colIndex: val.destinationRange[3] }, val.sheetIndex);
				 sheet._startCell = {rowIndex: val.destinationRange[0], colIndex: val.destinationRange[1]};
				 sheet._endCell = {rowIndex: val.destinationRange[2], colIndex: val.destinationRange[3]};
                this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
            }
        },

        _undoForPivot: function (val) {
            var coll = (this._isUndo) ? this._undoCollection : this._redoCollection;
            var args = { curOperation: val, prevOperation: coll[coll.length - 1], isUndo: this._isUndo };
            this._trigger("_readUndoRedo", args);
        },

        _undoForResizeCol: function (val) {
            this.XLResize.setColWidth(val.colIndex, val.oldWidth);
            var temp = val.oldWidth;
            val.oldWidth = val.newWidth;
            val.newWidth = temp;
        },

        _undoForResizeRow: function (val) {
            this.XLResize.setRowHeight(val.rowIndex, val.oldHeight);
            var temp = val.oldHeight;
            val.oldHeight = val.newHeight;
            val.newHeight = temp;
        },

        _undoForResizeFHeight: function (val) {
            this.XLResize.setRowHeight(val.rowIndex, val.newHeight);
			(this.getFrozenRows(val.sheetIndex) >= val.rowIndex) && this._refreshFreezeColumns(val.sheetIndex, val.rowIndex, val.newHeight - val.oldHeight, "resize");
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
        },

        _undoForResizeFWidth: function (val) {
            var cell = this.getCell(0, val.colIndex).get(0);
            var newWidth = val.newWidth;
            val.newWidth = (cell.offsetWidth === val.oldWidth) ? val.newWidth : val.oldWidth;
            val.oldWidth = newWidth;
            this.XLResize._applyWidth(val.target, val.sheetIndex, val.colIndex, val.newWidth);
			(this.getFrozenColumns(val.sheetIndex) >= this._currentHCell) && this._refreshFreezeColumns(val.sheetIndex, val.colIndex, val.newWidth - val.oldWidth, "resize");
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
        },

        _undoForShape: function (val) {
            var cObj, dataVal, chartId, ddObj, idx = 0, chartThemes = this.XLRibbon._chartThemes, shapeMgr = this.getSheet(val.sheetIndex)._shapeMngr;
            if (val.action !== "create" && val.action !== "remove" && val.action !== "edit" && val.action !== "border" && val.action !== "resetpic" && val.action !== "changepic" && val.action !== "picturesize" && val.action !== "chartTheme" && val.action !== "chartType" && val.action !== "selectData") {
                cObj = $("#" + val.cid).data("ejChart");
                dataVal = this.getSheet(val.sheetIndex)._shapeMngr.chart[this.XLEdit.getPropertyValue(val.rowIndex, val.colIndex, "chart")[0]];
            }
            switch (val.action) {
                case "remove":
                    this.XLShape._picCellIdx = this.XLShape._getCellIndexFromOffset(val.position.top, val.position.left);
                    this.XLShape._deleteShape(val.sheetIndex, $("#" + val.id));
                    val.action = "create";
                    break;
                case "create":
                    if (val.shapeType === "picture") {
                        this.XLShape._createPicture(val.sheetIndex, val.actCell, val.img, val.position.top, val.position.left, val.position.width, val.position.height, val.cnt);
                        $(val.actCell).addClass("e-activecell");
                    }
                    else {
                        this.XLSelection.selectRange({ rowIndex: val.range[0], colIndex: val.range[1] }, { rowIndex: val.range[0], colIndex: val.range[1] }, val.sheetIndex);
                        this.getCell(val.range[0], val.range[1], val.sheetIndex).addClass("e-activecell");
                        this.XLChart.createChart(this._getAlphaRange(val.sheetIndex, val.range[0], val.range[1], val.range[2], val.range[3]), val.options, val.id);
						 this.XLShape._updateShapeMngr(this.getCell(val.range[0], val.range[1], val.sheetIndex)[0], { chart: val.options }, "chart");
                        this.getCell(val.range[0], val.range[1], val.sheetIndex).addClass("e-activecell");
                    }
                    val.action = "remove";
                    break;
                case "edit":
                    var rng, container = this._dataContainer.sheets[val.sheetIndex], shapeType = val.id.split("_")[1].replace(/[0-9]/g, ''), imgElem = {}, objIdx, prevCellIdx;
                    $("#" + val.id).css({ left: val.prev.left, top: val.prev.top, height: val.prev.height, width: val.prev.width });
                    (shapeType === "chart") && this.XLChart.resizeChart(val.id, val.prev.height, val.prev.width);
                    val.visualImg.hide();
                    this.XLShape._picCellIdx = this.XLShape._getCellIndexFromOffset(val.prev.top, val.prev.left), prevCellIdx = this.XLShape._getCellIndexFromOffset(val.cur.top, val.cur.left);
                    rng = this._getShapePositionInManager(prevCellIdx.rowIndex, prevCellIdx.colIndex, val.id, shapeType);
                    imgElem[shapeType] = val.obj[shapeType][rng];
                    this.XLEdit._updateDataContainer({ rowIndex: this.XLShape._picCellIdx.rowIndex, colIndex: this.XLShape._picCellIdx.colIndex }, imgElem);
                    if (val.actionType != "resize") {
                        if (this.getObjectLength(container[prevCellIdx.rowIndex][prevCellIdx.colIndex]) < 2 && container[prevCellIdx.rowIndex][prevCellIdx.colIndex][shapeType].length < 2)
                            delete container[prevCellIdx.rowIndex][prevCellIdx.colIndex];
                        else if (container[prevCellIdx.rowIndex][prevCellIdx.colIndex][shapeType].length < 2)
                            delete container[prevCellIdx.rowIndex][prevCellIdx.colIndex][shapeType];
                        else {
                            container[prevCellIdx.rowIndex][prevCellIdx.colIndex][shapeType].some(function (obj, i) { if (imgElem.id === obj.id) { objIdx = i; return true; } });
                            container[prevCellIdx.rowIndex][prevCellIdx.colIndex][shapeType].splice(objIdx, 1);
                        }
                    }
                    this.getCell(this.XLShape._picCellIdx.rowIndex, this.XLShape._picCellIdx.colIndex, val.sheetIndex).addClass("e-activecell");
                    if(this.model.showRibbon)
						shapeType === "picture" ? this.XLRibbon._toggleFormatTab() : this.XLRibbon._toggleChartDesignTab();
                    this._swapVal(val);
                    break;
                case "PHAxis":
                case "PVAxis":
                    var pAxis = (val.action == "PHAxis") ? "primaryXAxis" : "primaryYAxis", axis = (val.action === "PHAxis") ? "xAxis" : "yAxis";
                    val.visibility = !(val.visibility);
                    cObj.option(pAxis, { visible: val.visibility });
                    dataVal[axis].visible = val.visibility;
                    break;
                case "PX":
                case "PY":
                    var pAxis = (val.action == "PX") ? "primaryXAxis" : "primaryYAxis", axis = (val.action === "PX") ? "xAxis" : "yAxis";
                    ej.isNullOrUndefined(val.prev.text) && (val.prev.text = "");
                    cObj.option(pAxis, { title: { text: val.prev.text } });
                    dataVal[axis].title.text = val.prev.text;
                    this._swapVal(val);
                    break;
                case "CT":
                    cObj.option("title", { textAlignment: val.prev.align, text: val.prev.text });
                    dataVal.title = { text: val.prev.text, textAlignment: val.prev.align };
                    this._swapVal(val);
                    break;
                case "DLNone":
                    cObj.option({ series: val.prev.series });
                    dataVal.dataLabel = { visible: val.prev.series[0].marker.dataLabel.visible };
                    this._swapVal(val);
                    break;
                case "DL":
                    cObj.option({ series: val.prev.series });
                    dataVal.dataLabel = { visible: val.prev.series[0].marker.dataLabel.visible, vAlign: val.prev.series[0].marker.dataLabel.verticalTextAlignment, tPosn: val.prev.series[0].marker.dataLabel.textPosition };
                    this._swapVal(val);
                    break;
                case "PMajorH":
                    val.visibility = !(val.visibility);
                    cObj.option("primaryYAxis", { majorGridLines: { visible: val.visibility } });
                    dataVal.yAxis.majorGridLines = { visible: val.visibility };
                    break;
                case "PMajorV":
                    val.visibility = !(val.visibility);
                    cObj.option("primaryXAxis", { majorGridLines: { visible: val.visibility } });
                    dataVal.xAxis.majorGridLines = { visible: val.visibility };
                    break;
                case "PMinorH":
                    val.visibility = !(val.visibility);
                    cObj.option("primaryYAxis", { minorGridLines: { visible: val.visibility }, minorTicksPerInterval: 5, minorTickLines: { visible: val.visibility } });
                    dataVal.yAxis.minorGridLines = { visible: val.visibility };
                    break;
                case "PMinorV":
                    val.visibility = !(val.visibility);
                    cObj.option("primaryXAxis", { minorGridLines: { visible: val.visibility }, minorTicksPerInterval: 5, minorTickLines: { visible: val.visibility } });
                    dataVal.xAxis.minorGridLines = { visible: val.visibility };
                    break;
                case "LNone":
                    val.visibility = !(val.visibility);
                    cObj.option("legend", { visible: val.visibility });
                    dataVal.legend.visible = val.visibility;
                    break;
                case "Legend":
                    cObj.option("legend", { position: val.prev.position });
                    dataVal.legend.position = val.prev.position;
                    this._swapVal(val);
                    break;
                case "width":
                    var opt = { width: val.prev.width, height: val.prev.height };
                    var chartEle = $("#" + cObj._id);
                    cObj.option("size", $.extend(true,{},opt));
                    chartEle[0].style.height = val.prev.height + "px";
                    chartEle[0].style.width = val.prev.width+ "px";
                    dataVal.width = val.prev.width;
                    dataVal.height = val.prev.height;
                    this.XLRibbon._setChartWidthHeight(opt);
                    this._swapVal(val);
                    break;
                case "height":
                    var opt = { width: val.prev.width, height: val.prev.height };
                    var chartEle = $("#" + cObj._id);
                    cObj.option("size", $.extend(true,{},opt));
                    chartEle[0].style.height=val.prev.height+"px";
                    chartEle[0].style.width=val.prev.width+"px";
                    dataVal.width = val.prev.width;
                    dataVal.height = val.prev.Height;
                    this.XLRibbon._setChartWidthHeight(opt);
                    this._swapVal(val);
                    break;
                case "srcolumn":
                    var obj, chOptions;
                    cObj.model.series.length = 0;
                    dataVal.isRowColSwitched = !dataVal.isRowColSwitched;
                    obj = { xRange: dataVal.xRange, yRange: dataVal.yRange, lRange: dataVal.lRange, isRowColSwitched: dataVal.isRowColSwitched, dataSheetIdx: dataVal.dataSheetIdx };
                    chOptions = this.XLChart._processChartSeries(obj);
                    cObj.option({ series: chOptions.series });
                    this.getActiveSheetIndex() === dataVal.dataSheetIdx && this.XLChart._focusChartRange(chOptions.xRange, chOptions.yRange, chOptions.lRange);
                    dataVal.xAxis.range = chOptions.xRange;
                    dataVal.yAxis.range = chOptions.yRange;
                    dataVal.legend.range = chOptions.lRange;
                    break;
                case "border":
                    var temp, style, shapeObj = this._dataContainer.sheets[val.sheetIndex][val.cellIdx.rowIndex][val.cellIdx.colIndex].picture[val.picIdx], elem = $("#" + val.id);
                    this._removeClass(elem[0], val.curClass);
                    this.addClass(elem[0], val.prevClass);
                    style = this.XLShape._getImgStyleFromHashCode(val.prevClass);
                    shapeObj = { bcolor: style["border-color"], bstyle: style["border-style"], bwidth: style["border-width"] };
                    temp = val.prevClass;
                    val.prevClass = val.curClass;
                    val.curClass = temp;
                    break;
                case "resetpic":
                    var elem = $("#" + val.id), regx = new RegExp("\\b" + "e-shapebdr" + ".*?\\b", "g");
                    if (this._isUndo)
                        (ej.isNullOrUndefined(elem[0].className.match(regx))) && this.XLShape._updateImgBorder(elem[0], val.prev.bwidth, val.prev.bstyle, val.prev.bcolor);
                    else
                        elem[0].className = elem[0].className.replace(regx, "");
                    if (val.actionType !== "resetpicture") {
                        elem.css({ width: val.prev.width, height: val.prev.height });
                        this.XLShape._selectImg(elem);
                        this.XLShape._updateShapeObj(elem[0]);
                    }
                    this._swapVal(val);
                    break;
                case "changepic":
                    var num, container = this._dataContainer;
                    $("#" + val.id).css("background-image", "url('" + val.prev.img + "')");
					 shapeMgr.picture[val.prev.pcIdx].data = val.prev.data;
                    this._swapVal(val);
                    break;
                case "picturesize":
                    var elem = $("#" + val.id);
                    elem.css({ "width": val.prev.width, "height": val.prev.height });
                    this._swapVal(val);
                    break;
                case "chartTheme":
                    chartId = this.element.find("#" + this._id + "_chart").val(), ddObj = $("#" + this._id + "_Ribbon_chartTheme").data("ejDropDownList");
  			        (val.prev.theme.indexOf("dark") > -1) ? ($("#" + chartId).addClass('e-ss-charttheme')) : ($("#"+chartId).removeClass('e-ss-charttheme'));
                    this.XLChart.refreshChart(chartId, { theme: val.prev.theme });
					for(var i = 0, len = chartThemes.length; len > i ; i++)
					{
						if (val.prev.theme === chartThemes[i].value.toLowerCase()) {
                            idx = i;
                            break;
                        }
					}
					 ddObj.option({ selectedItemIndex: idx })
                    this._swapVal(val);
                    break;
                case "chartType":
                    this.XLChart.refreshChart(val.chartId, { type: val.prev.chartType, enable3D: val.prev.enable3D, marker: val.prev.marker });
                    this._swapVal(val);
                    break;
				case "selectData":
					this.XLChart._processChartRange(val.prev.xRange, val.prev.yRange, val.prev.lRange, document.getElementById(this._id + "_chart").value);
					this._swapVal(val);
					break;
			}
        },

        _undoForShowHide: function (val) {
			var i, len, rowHdrs = $(this.getRows(val.sheetIndex)[0]).find('td'), colHdrs = this._getJSSheetHeader(val.sheetIndex).find('th');
			if(val.operation === "row") {
				for(i = 0, len = val.hideRows.length; i < len; i++)
					(val.action === "show") ? this.hideRow(val.hideRows[i], val.hideRows[i]) : this.showRow(val.hideRows[i], val.hideRows[i]);
				for(i = 0, len = val.remBRows.length; i < len; i++)
					this.addClass(rowHdrs[val.remBRows[i]], "e-hrow");
				for(i = 0, len = val.addBRows.length; i < len; i++)
					this._removeClass(rowHdrs[val.addBRows[i]], "e-hrow");
				this.setActiveCell(val.bSelect.startRow, 0);
				(val.bSelect.startRow === val.bSelect.endRow) ? this.XLSelection.selectRow(val.bSelect.startRow, val.sheetIndex) : this.XLSelection.selectRows(val.bSelect.startRow, val.bSelect.endRow, val.sheetIndex);
				var addBRows = val.addBRows;
				val.addBRows = val.remBRows;
				val.remBRows = addBRows;
			}
			else{
				for(i = 0, len = val.hideCols.length; i < len; i++) 
					(val.action === "show") ? this.hideColumn(val.hideCols[i], val.hideCols[i]) : this.showColumn(val.hideCols[i], val.hideCols[i]);
				for(i = 0, len = val.remBCols.length; i < len; i++)
					this.addClass(colHdrs[val.remBCols[i]], "e-hcol");
				for(i = 0, len = val.addBCols.length; i < len; i++)
					this._removeClass(colHdrs[val.addBCols[i]], "e-hcol");
				(val.bSelect.startCol === val.bSelect.endCol) ? this.XLSelection.selectColumn(val.bSelect.startCol, val.sheetIndex) : this.XLSelection.selectColumns(val.bSelect.startCol, val.bSelect.endCol, val.sheetIndex);
				var addBCols = val.addBCols;
				val.addBCols = val.remBCols;
				val.remBCols = addBCols;
			}
			val.action = (val.action === "show") ? "hide" : "show";
			var bSelect = val.bSelect;
			val.bSelect = val.aSelect;
			val.aSelect = bSelect;
			this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
		},

        _undoForSort: function (val) {
            var colIdx = this.XLEdit.getColumnIndexByField(val.columnName), actCell, fSpan;
			if(val.colSelected < 1){
				actCell = this.getCell(val.range[0] - 1, colIdx);
				fSpan = actCell.find('.e-filterspan')[0];
			}
            if (val.operation === "unsort") {
				this.XLComment.deleteComment(val.range, null, true);
                this._isSort = true;
				this._removeHyperlink(null, null, true);
                this._isSort = false;
				this.clearRangeData(val.range, ["value", "value2", "type", "formatStr", "decimalPlaces", "thousandSeparator", "comment", "hyperlink", "format"], null, true);
				this.XLFormat.clearFormat(val.range, "format", true);
                this.XLEdit.updateCellWithContainer(val.rangeData, val.range, val.sheetIndex, true);
                val.operation = "sort";
                if (!this.isUndefined(fSpan) && val.isSortByFilter) 
                    fSpan.className = (fSpan.className.indexOf("-asc") > 1) ? fSpan.className.replace("-asc", val.prevIcon) : fSpan.className.replace("-dsc", val.prevIcon);
            }
            else {
				this.XLSort._isSortByFilter = val.isSortByFilter;
                this.XLSort.sortByRange(val.range, val.columnName, val.direction);
                val.operation = "unsort";
				if (!this.isUndefined(fSpan) && val.isSortByFilter)
					this.model.allowFiltering && this.XLFilter._changeSortIcon({rowIndex: val.range[0] - 1, colIndex: colIdx}, val.direction);
            }
            var prevIcon = val.prevIcon;
            val.prevIcon = val.currIcon;
            val.currIcon = prevIcon;
        },

        _undoForSortByColor: function (val) {
            var rng = val.range, pData = val.prev.data, i, activeCell = this.getCell(rng[0] - 1, rng[1]), format;
			this.XLComment.deleteComment(rng, null, true);
			this._isSort = true;
			this._removeHyperlink(null, null, true);
			this._isSort = false;
			this.clearRangeData(rng, ["value", "value2", "type", "formatStr", "decimalPlaces", "thousandSeparator", "comment", "hyperlink", "format"], null, true);
			this.XLFormat.clearFormat(rng, "format", true);
			this.XLEdit.updateCellWithContainer(pData, rng, val.sheetIndex, true);
            var filterSpan = activeCell.find(".e-filterspan");
            if (filterSpan.length > 0)
                filterSpan[0].className = filterSpan[0].className.replace(val.cur.icon, val.prev.icon);
            this._swapVal(val);
        },

        _undoForStyle: function (val) {
            this.XLFormat.clearFormat(val.range);
            this.XLFormat.updateFormat(val.beforeFormat, val.range);
            var s = this.changePosition(val.beforeFormat, val.afterFormat);
            this._refreshRangeTRHght(this.getActiveSheetIndex(), val.range);
            val.beforeFormat = s[1];
            val.afterFormat = s[0];
            this._isUndoRedo = true;
            val.status = "undo-cellstyles";
        },

        _undoForTextWrap: function (val) {
            if (val.operation === "wrap") {
                this.setWrapText(val.range, "unwrap");
                val.operation = "unwrap";
            }
            else {
                this.setWrapText(val.range, "wrap");
                val.operation = "wrap";
            }
            this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
        },

        _undoForValidation: function (val) {
            var i, len, range = this.getRangeIndices(val.range);
			this.performSelection({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] });
            if (val.operation === "cl-hl-data") {
                this.XLValidate.highlightInvalidData(val.range);
                val.operation = "hl-data";
            }
            else if (val.operation === "apply-validation") {
                var selCells = val.selectedCells, rule, curRule, selected = [];
                for (i = 0, len = selCells.length; i < len; i++) {
                    curRule = this.XLEdit.getPropertyValue(selCells[i].rowIndex, selCells[i].colIndex, "rule");
                    rule = this.changePosition(selCells[i].prevRule, curRule);
                    val.selectedCells[i].prevRule = rule[1];
                    if (!ej.isNullOrUndefined(rule[0])) {
                        rule[0].customVal = rule[0].customVal;
                        this.XLEdit._updateDataContainer({rowIndex: selCells[i].rowIndex, colIndex: selCells[i].colIndex}, { "rule": rule[0] });
                    }
                    selected.push(cell[0]);
                }
                if (ej.isNullOrUndefined(rule[0]))
                    this.clearRangeData(null, ["rule"], selected);
            }
            else if (val.operation === "hl-data") {
                this.XLValidate.clearHighlightedValData(val.range);
                val.operation = "cl-hl-data";
            }
            else {
                if (this._isUndo) {
                    for (i = 0, len = val.selCells.length; i < len; i++)
                        this.XLEdit._updateDataContainer({rowIndex: selCells[i].rowIndex, colIndex: selCells[i].colIndex}, { "rule": val.selCells[i].rules });
                }
                else
                    this.XLValidate.clearDV(val.range);
            }
        },

        _applyCommentLink: function (cellObj, cell, rowIdx, colIdx) {
            var container = this._dataContainer, hldata, cFormatData = container.cFormatData, actCell, isViewed = false;
            if(this._isRowViewable(this.getActiveSheetIndex(), cell.rowIndex)){
                actCell = this.getCell(cell.rowIndex, cell.colIndex);
                isViewed = true;
                $(actCell).removeClass("e-redft e-yellowft e-greenft e-redf e-redt");
            }
            if (cellObj) {
                var value2 = this.isNumber(cellObj.value2) ? container.sharedData[cellObj.value2] : cellObj.value2;
                this.XLEdit.updateCellValue(cell, value2);
                if ("comment" in cellObj) {
                    isViewed && actCell.removeClass("e-commentcell");
                    this.XLComment.setComment([cell.rowIndex, cell.colIndex, cell.rowIndex, cell.colIndex], null, false);
                }
                if ("cFormatRule" in cellObj) {
                    var cFormatRule = cellObj.cFormatRule, splitStr;
                    for (var m = 0, rLen = cFormatRule.length; m < rLen; m++) {
                        splitStr = cFormatData[m].split("_");
                        this.XLCFormat._cFormat(splitStr[0], splitStr[2], splitStr[3], splitStr[4], null, null, [rowIdx, colIdx, rowIdx, colIdx]);
                    }
                }
            }
        },

        changePosition: function (first, second) {
            var swap = [];
            swap.push(second);
            swap.push(first);
            ej.swap(swap, 0, 1);
            return swap;
        },

        _addValidationRule: function (totRules, rule) {
            var dupRule = [];
            $.extend(true, dupRule, totRules);
            for (var i = 0, len = totRules.length; i < len; i++)
                if (totRules[i].customVal !== rule.customVal && totRules[i].required !== rule.required && totRules[i].Number !== rule.number)
                    dupRule.push(rule);
            totRules = dupRule;
            return totRules;
        },

        _addCondFormatRule: function (totRules, rule) {
            var avble = false, dupRule = [];
            $.extend(true, dupRule, totRules);
            for (var i = 0, len = totRules.length; i < len; i++) {
                rule = rule.indexOf("true") > -1 ? rule.replace("_true", "") : rule.replace("_false", "");
                if (totRules[i] === rule) {
                    avble = true;
                    break;
                }
            }
            (!avble) && dupRule.push(rule);
            totRules = dupRule;
            return totRules;
        },

        getCellClass: function (curCell) {
            var regx = this._formatRegx;
            var currClass = curCell && (curCell.className.indexOf("e-format") > -1) ? curCell.className.match(regx)[0] : "";
            return currClass;
        },


        //Insert Or Delete Rows and Columns
        _renderInsertDeleteDialog: function () {
            var dlgId = this._id + "_insertdeletedlg", $dlg, $content, $okBtn, $canBtn, $btndiv, $btnctn;
            $dlg = ej.buildTag("div#" + dlgId, "", { display: "none" });
            $content = ej.buildTag("div#" + dlgId + "_content.e-dlg-fields e-dlgctndiv");
            $dlg.append($content);
            $btndiv = ej.buildTag("div#" + dlgId + "_btndiv.e-dlg-btnfields");
            $btnctn = ej.buildTag("div");
            $okBtn = ej.buildTag("input#" + dlgId + "_okbtn", "", { "margin-left": 4 });
            $canBtn = ej.buildTag("input#" + dlgId + "_cantn");
            $okBtn.ejButton({ text: this._getLocStr("Ok"), showRoundedCorner: true, width: "40%", click: ej.proxy(this._insertDeleteOk, this), enabled: true });
            $canBtn.ejButton({ text: this._getLocStr("Cancel"), click: ej.proxy(this._insertDeleteCancel, this), showRoundedCorner: true, width: "40%" });
            $btndiv.append($btnctn.append($okBtn, $canBtn));
            $dlg.append($btndiv);
            $dlg.ejDialog({ showOnInit: false, enableModal: true, enableResize: false, allowKeyboardNavigation: false, width: 187, height: "auto", cssClass: "e-ss-dialog e-"+this._id+"-dlg", minWidth: 150});
        },

        _renderInsertDialog: function () {
            var $dlg = $("#" + this._id + "_insertdeletedlg"), $content, text;
            $dlg.ejDialog({ title: this._getLocStr("Insert") });
            $dlg.ejDialog("open");
            this._insDelStatus = "insert";
            $content = $("#" + $dlg[0].id + "_content");
            text = ["shiftright", "shiftbottom", "entirerow", "entirecolumn"];
            $content.children().empty();
            this._createRadioButton($dlg, $content, text);
            $("#" + $dlg[0].id + "_" + "shiftbottom").ejRadioButton({ checked: true });
            $dlg.find("input[type='submit']:first").focus();
        },

        _renderDeleteDialog: function () {
            var $dlg = $("#" + this._id + "_insertdeletedlg"), $content, text;
            $dlg.ejDialog({ title: this._getLocStr("Delete") });
            $dlg.ejDialog("open");
            this._insDelStatus = "delete";
            $content = $("#" + $dlg[0].id + "_content");
            text = ["shiftleft", "shiftup", "entirerow", "entirecolumn"];
            $content.children().empty();
            this._createRadioButton($dlg, $content, text);
            $("#" + $dlg[0].id + "_shiftleft").ejRadioButton({ checked: true });
            $dlg.find("input[type='submit']:first").focus();
        },

        _createRadioButton: function ($dlg, $content, text) {
            var input, lbl, table, tr1, td1;
            table = ej.buildTag("table");
            for (var i = 0, len = text.length; i < len; i++) {
                tr1 = ej.buildTag("tr");
                td1 = ej.buildTag("td");
                lbl = ej.buildTag("label", this._getLocStr(text[i]), { "padding-bottom": 4 }, { "for": $dlg[0].id + "_" + text[i] });
                input = ej.buildTag("input#" + $dlg[0].id + "_" + text[i], "", {}, { type: "radio", name: "inserdelete" });
                td1.append(input);
                tr1.append(td1);
                td1 = ej.buildTag("td");
                td1.append(lbl);
                input.ejRadioButton({ size: "medium" });
                tr1.append(td1);
                table.append(tr1);
            }
            $content.append(table);
        },

        _getSelectedOption: function (text) {
            var dlgId = this._id + "_insertdeletedlg", selIdx = 0;
            for (var i = 0; i < 4; i++) {
                var rbtn = $("#" + dlgId + "_" + text[i]).data("ejRadioButton");
                if (rbtn.model.checked) {
                    selIdx = i;
                    break;
                }
            }
            return selIdx;
        },

        _insertDeleteOk: function () {
            this._isUndo = false, this._isUndoRedo = false;
            var text, selIdx, sheetIdx = this.getActiveSheetIndex(),
                startCell = this.model.sheets[sheetIdx]._selectedCells[0], endCell = this.model.sheets[sheetIdx]._selectedCells[this.model.sheets[sheetIdx]._selectedCells.length - 1];
            if (this._insDelStatus.indexOf("insert") > -1) {
                text = ["shiftright", "shiftbottom", "entirerow", "entirecolumn"];
                selIdx = this._getSelectedOption(text);
                if (text[selIdx].indexOf("shiftright") > -1)
                    this.insertShiftRight(startCell, endCell);
                else if (text[selIdx].indexOf("shiftbottom") > -1)
                    this.insertShiftBottom(startCell, endCell);
                else if (text[selIdx].indexOf("entirerow") > -1)
                    this.insertEntireRow(startCell, endCell);
                else
                    this.insertEntireColumn(startCell, endCell);
            }
            else {
                text = ["shiftleft", "shiftup", "entirerow", "entirecolumn"];
                selIdx = this._getSelectedOption(text);
                if (text[selIdx].indexOf("shiftleft") > -1)
                    this.deleteShiftLeft(startCell, endCell);
                else if (text[selIdx].indexOf("shiftup") > -1)
                    this.deleteShiftUp(startCell, endCell);
                else if (text[selIdx].indexOf("entirerow") > -1)
                    this.deleteEntireRow(startCell, endCell);
                else
                    this.deleteEntireColumn(startCell, endCell);
            }
            $("#" + this._id + "_insertdeletedlg").ejDialog("close");
            var sheetChart = this.model.sheets[this.model.activeSheetIndex]._shapeMngr.chart;
            for (var i = 0; i < sheetChart.length; i++)
                this.XLChart._refreshChart(sheetChart[i]);
        },

        _insertDeleteCancel: function () {
            $("#" + this._id + "_insertdeletedlg").ejDialog("close");
        },

        _undoredoDetails: function (sheetIndex, startCell, endCell, reqType, action) {
            var details = { sheetIndex: sheetIndex, startCell: startCell, endCell: endCell, reqType: reqType, action: action, delCells: this._delCells, text: this._deletedText };
            !this.importSheet && this._completeAction(details);
        },

        _getRangeFrmFormula: function (formula) {
            var pAlpha = formula.substring(formula.indexOf("(") + 1, formula.indexOf(")")), regx = new RegExp('^\\b[a-zA-Z][0-9]*:[a-zA-Z][0-9]*\\b$', 'g');
            if (regx.test(pAlpha))
                return pAlpha;
            else if (pAlpha.indexOf(',') > 1)
                if (regx.test(pAlpha.split(',')[1]))
                    return pAlpha.split(',')[1];
                else if (regx.test(pAlpha.replace(',', ':')))
                    return pAlpha.replace(',', ':');
        },

        _refreshFormula: function (rowIdx, colIdx, count, status, type, startIndex, isShift) {
            var sheetIdx = this.getActiveSheetIndex(), diff, diff1, val = this.XLEdit.getPropertyValue(rowIdx, colIdx, "value"), pAlpha, nAlpha, range, cellArgs, actSheet = this.getSheet(sheetIdx);
            if (this._isUndo && (this._formulaChanged.pop() === "true"))
                return;
            if (this.isFormula(val)) {
                pAlpha = this._getRangeFrmFormula(val);
                if (ej.isNullOrUndefined(pAlpha))
                    return;
                range = this.getRangeIndices(pAlpha);
                diff = (type === "column") ? (status === "insert") ? range[3] + count : range[3] - count : (status === "insert") ? range[2] + count : range[2] - count;
                diff1 = (type === "column") ? (status === "insert") ? range[1] + count : range[1] - count : (status === "insert") ? range[0] + count : range[0] - count;
                (type === "column") ? (startIndex > range[1] && (diff1 = range[1])) : (startIndex > range[0] && (diff1 = range[0]));
                (type === "column") ? (startIndex > range[3] && (diff = range[3])) : (startIndex > range[2] && (diff = range[2]));
                if (isShift) {
                    if (!(range[1] === range[3]) && (range[0] === range[2]))
                        (type === "column") ? (range[1] === range[3]) && (diff = range[3], diff1 = range[1]) : (range[0] === range[2]) && (diff = range[2], diff1 = range[0]);
                    (type === "column") ? (diff < range[1]) && (diff = range[3]) : (diff < range[0]) && (diff = range[2]);
                    (type === "column") ? (diff1 < range[1]) && (diff1 = range[1]) : (diff1 < range[0]) && (diff1 = range[0]);
                    type === "row" && ((status === "delete") ? ((diff1 < range[2] && range[1] != colIdx) && (diff = range[2], diff1 = range[0])) : ((range[1] != colIdx) && (diff = range[2], diff1 = range[0])));
                    type === "column" && ((status === "delete") ? ((diff1 < range[3] && range[0] != rowIdx) && (diff = range[3], diff1 = range[1])) : ((range[0] != rowIdx) && (diff = range[3], diff1 = range[1])));
                }
                nAlpha = (type === "column") ? this._getAlphaRange(sheetIdx, range[0], diff1, range[2], diff) : this._getAlphaRange(sheetIdx, diff1, range[1], diff, range[3]);
                (rowIdx === range[0] && colIdx === range[1]) && (nAlpha = "#REF!", this._ref.push(pAlpha));
                if (this._isUndo && pAlpha === "#REF!")
                    nAlpha = this._ref.pop();
                val = val.replace(pAlpha, nAlpha);
                cellArgs = new ValueChangedArgs(rowIdx + 1, colIdx + 1, val);
                this._sheets[sheetIdx]._calcEngine.valueChanged(actSheet._sheetName.value, cellArgs);
                this.XLEdit._updateDataContainer({rowIndex: rowIdx, colIndex: colIdx}, { value: val });
                this._applyFormula(sheetIdx, rowIdx, colIdx);
                !this._isUndo && ((pAlpha === nAlpha) ? this._formulaChanged.push("true") : this._formulaChanged.push("false"));
            }
        },

        _applyFormula: function (sheetIdx, rowIdx, colIdx) {
            (ej.isNullOrUndefined(this.model.sheets[sheetIdx]._formulaCollection[rowIdx])) && (this.model.sheets[sheetIdx]._formulaCollection[rowIdx] = {});
            var formulaColl = this._sheets[sheetIdx]._calcEngine.getFormulaInfoTable().values();
            this.model.sheets[sheetIdx]._formulaCollection[rowIdx][colIdx] = formulaColl[formulaColl.length - 1];
        },

        _clearFormula: function (i, j) {
            var formulaInfo, formulaIdx, formulaColl, sheetIdx = this.getActiveSheetIndex(), val = this.XLEdit.getPropertyValue(i, j, "value");
            if (this.isFormula(val)) {
                //clear formula for the particular cell
					formulaInfo = this.model.sheets[sheetIdx]._formulaCollection[i][j];
					formulaColl = this._sheets[sheetIdx]._calcEngine.getFormulaInfoTable().values();
					formulaIdx = formulaColl.indexOf(formulaInfo);
					formulaColl.splice(formulaIdx, 1);
				    this._sheets[sheetIdx]._calcEngine.getFormulaInfoTable().clear();
					for (var k = 0, len = formulaColl.length; k < len; k++)
						this._sheets[sheetIdx]._calcEngine.getFormulaInfoTable().add(formulaColl[k]);
					delete this.model.sheets[sheetIdx]._formulaCollection[i][j];
				}
        },
        _refreshValAndCF: function (i, j, vRule, cRule, rCount, cCount) {
            var container = this._dataContainer.sheets[this.getActiveSheetIndex()];
            if (this.isUndefined(vRule) && this.isUndefined(cRule))
                delete container[i][j];
            else {
                !this.isUndefined(vRule) && (container[i][j] = { rule: vRule });
                !this.isUndefined(cRule) && (container[i][j] = { cFormatRule: cRule });
            }
        },

        _refreshDataContainer: function (sIdx, value) {
            var stRow, enRow, stCol, enCol, vRule, cRule, i, j, k, len, ln, maxCol = 0, lt = 0, nContainer, maxRow, container = this._dataContainer.sheets[sIdx], cCount, rCount, sCol, remCells, rmColl, columns, count, picture, chart, index;
            maxRow = parseInt(this.getObjectKeys(container)[this.getObjectLength(container) - 1]);
            for (i = 0; i <= maxRow; i++) {
                if (!ej.isNullOrUndefined(container[i])) {
                    nContainer = this.getObjectKeys(container[i]);
                    lt = parseInt(nContainer[nContainer.length - 1]);
                    (lt > maxCol) && (maxCol = lt);
                }
            }
            sCol = (value.startCol <= maxCol) ? value.colCount + value.startCol : maxCol - 1;
            cCount = (value.status.indexOf("shift") > -1) ? sCol : maxCol - 1;
            rCount = (value.status.indexOf("shift") > -1) ? value.rowCount + value.startRow : maxRow;
            stRow = value.startRow; enRow = (value.startRow + value.rowCount) - 1;
            stCol = value.startCol; enCol = (value.startCol + value.colCount) - 1;
            switch (value.status) {
                case "insertRow":
                    for (i = maxRow, len = value.startRow; i >= len; i--) {
                        if (!ej.isNullOrUndefined(container[i])) {
                            container[i + value.rowCount] = $.extend(true, {}, container[i]);
                            if (!ej.isNullOrUndefined(container[i + value.rowCount])) {
                                columns = this.getObjectKeys(container[i + value.rowCount]);
                                for (j = 0, ln = columns.length; j < ln; j++) {
                                    this._clearFormula(i, columns[j]);
                                    //refresh Validation
                                    if (i >= stRow && i <= enRow) {
                                        this._checkIndicesInContainer(sIdx, stRow, columns[j], "rule") && (vRule = this._dataContainer.sheets[sIdx][stRow][columns[j]]["rule"]);
                                        this._checkIndicesInContainer(sIdx, stRow, columns[j], "cFormatRule") && (cRule = this._dataContainer.sheets[sIdx][stRow][columns[j]]["cFormatRule"]);                                        
                                        this._refreshValAndCF(i, columns[j], vRule, cRule, value.rowCount, 0);
                                        !(stRow < 1) && this.addClass(this.getCell(i, columns[j])[0], this.XLFormat.getFormatClass(this.getCell(stRow - 1, columns[j])[0].className, false));
                                    }
                                    else
                                        delete container[i][columns[j]];
                                    this._refreshFormula(i + value.rowCount, parseInt(columns[j]), value.rowCount, "insert", "row", value.startRow, false);
                                    this.XLEdit._refreshCalcEngine(i + value.rowCount, parseInt(columns[j]));
                                    this.XLComment._updateCmntFrmCntnr(i + value.rowCount, parseInt(columns[j]));
                                }
                            }
                            (this.getObjectLength(container[i]) < 1) && delete container[i];
                        }
                    }
                    if (this._isUndo && this._delCells.length > 0) {
                        remCells = this._delCells;
                        count = remCells[remCells.length - 1].row - remCells[0].row;
                        for (i = 0, len = remCells.length; i < len; i++) {
                            container[remCells[i].row] = remCells[i].value;
                            columns = this.getObjectKeys(container[remCells[i].row]);
                            for (j = 0, ln = columns.length; j < ln; j++) {
                                this._refreshFormula(remCells[i].row, parseInt(columns[j]), count, "insert", "row","", false);
                                this.XLEdit._refreshCalcEngine(remCells[i].row, parseInt(columns[j]));
                            }
                        }
                    }
                    break;
                case "insertColumn":
                    for (i = 0 ; i <= rCount; i++) {
                        for (j = maxCol, len = value.startCol; j >= len ; j--) {
                            picture = chart = null;
                            if (!ej.isNullOrUndefined(container[i])) {
                                if (!ej.isNullOrUndefined(container[i][j])) {
                                    if (!ej.isNullOrUndefined(container[i][j + value.colCount])) {
                                        if (!ej.isNullOrUndefined(container[i][j + value.colCount].picture))
                                            picture = container[i][j + value.colCount].picture;
                                        if (!ej.isNullOrUndefined(container[i][j + value.colCount].chart))
                                            chart = container[i][j + value.colCount].chart;
                                    }
                                    container[i][j + value.colCount] = container[i][j];
                                    if (!ej.isNullOrUndefined(container[i][j].picture) || !ej.isNullOrUndefined(container[i][j].chart)) {
                                        if (!ej.isNullOrUndefined(container[i][j].picture))
                                            picture = container[i][j].picture;
                                        if (!ej.isNullOrUndefined(container[i][j].chart))
                                            chart = container[i][j].chart;
                                    }
                                    if (!ej.isNullOrUndefined(container[i][j + value.colCount])) {
                                        if (!ej.isNullOrUndefined(picture))
                                            container[i][j + value.colCount].picture = picture;
                                        if (!ej.isNullOrUndefined(chart))
                                            container[i][j + value.colCount].chart = chart;
                                    }

                                }
                                this._clearFormula(i, j);
                                //refresh Validation
                                if (j >= stCol && j <= enCol) {
                                    this._checkIndicesInContainer(sIdx, i, stCol, "rule") && (vRule = this._dataContainer.sheets[sIdx][i][stCol]["rule"]);
                                    this._checkIndicesInContainer(sIdx, i, stCol, "cFormatRule") && (cRule = this._dataContainer.sheets[sIdx][i][stCol]["cFormatRule"]);                                   
                                    this._refreshValAndCF(i, j, vRule, cRule, 0, value.colCount);
                                    (!stCol < 1) && this.addClass(this.getCell(i, j)[0], this.XLFormat.getFormatClass(this.getCell(i, stCol - 1)[0].className, false));
                                }
                                else
                                    delete container[i][j];
                                if (ej.isNullOrUndefined(container[i][j]) && (!ej.isNullOrUndefined(picture) || !ej.isNullOrUndefined(chart)))
                                    container[i][j] = {};
                                if (!ej.isNullOrUndefined(picture))
                                    container[i][j].picture = picture;
                                if (!ej.isNullOrUndefined(chart))
                                    container[i][j].chart = chart;
                                this._refreshFormula(i, j + value.colCount, value.colCount, "insert", "column", value.startCol, false);
                                this.XLEdit._refreshCalcEngine(i, j + value.colCount);
                                this.XLComment._updateCmntFrmCntnr(i, j + value.colCount);
                            }
                        }
                    }
                    if (this._isUndo && this._delCells.length > 0) {
                        remCells = this._delCells;
                        count = remCells[remCells.length - 1].col - remCells[0].col;
                        for (k = 0, len = remCells.length; k < len; k++) {
                            if (ej.isNullOrUndefined(container[remCells[k].row]))
                                container[remCells[k].row] = {};
                            container[remCells[k].row][remCells[k].col] = remCells[k].value;
                            this._refreshFormula(remCells[k].row, remCells[k].col, count, "insert", "column", "", false);
                            this.XLEdit._refreshCalcEngine(remCells[k].row, remCells[k].col);
                            this.XLComment._updateCmntFrmCntnr(remCells[k].row, remCells[k].col);
                        }
                    }
                    break;
                case "deleteRow":
                    this._delCells = [];
                    rmColl = [];
                    for (i = value.startRow; i <= maxRow; i++) {
                        if (i < (value.startRow + value.rowCount)) {
                            this._delCells.push({ row: i, value: $.extend(true, {}, container[i]) });
                            if (!ej.isNullOrUndefined(container[i])) {
                                columns = this.getObjectKeys(container[i]);
                                for (j = 0, len = columns.length; j < len; j++) {
                                    this._clearFormula(i, columns[j]);
                                    container[i][columns[j]] = {};
                                }
                            }
                            container[i] = {};
                            rmColl.push({ row: i });
                        }
                        if (!ej.isNullOrUndefined(container[i + value.rowCount])) {
                            container[i] = $.extend(true, {}, container[i + value.rowCount]);
                            columns = this.getObjectKeys(container[i]);
                            for (j = 0, len = columns.length; j < len; j++) {
                                this._clearFormula(i + value.rowCount, columns[j]);
                                container[i + value.rowCount][columns[j]] = {};
                                this._refreshFormula(i, parseInt(columns[j]), value.rowCount, "delete", "row", value.startRow, false);
                                this.XLComment._updateCmntFrmCntnr(i, parseInt(columns[j]));
                            }
                            container[i + value.rowCount] = {};
                            rmColl.push({ row: i + value.rowCount });
                        }
                    }
                    for (k = 0, len = rmColl.length; k < len; k++) {
                        if (this.getObjectLength(container[rmColl[k].row]) === 0) {
                            this._clearFormula(rmColl[k].row, rmColl[k].col);
                            delete container[rmColl[k].row];
                        }
                    }
                    break;
                case "deleteColumn":
                    this._delCells = [];
                    rmColl = [];
                    for (i = 0 ; i <= rCount; i++) {
                        for (j = value.startCol; j <= maxCol; j++) {
                            picture = chart = null;
                            if (j < (value.startCol + value.colCount)) {
                                if (!ej.isNullOrUndefined(container[i])) {
                                    if (!ej.isNullOrUndefined(container[i][j])) {
                                        this._delCells.push({ row: i, col: j, value: $.extend(true, {}, container[i][j]) });
                                        this._clearFormula(i, j);
                                        container[i][j] = {};
                                        if (!ej.isNullOrUndefined(this._delCells[this._delCells.length - 1])) {
                                            index = this._delCells.length - 1;
                                            if (!ej.isNullOrUndefined(this._delCells[index].value.picture))
                                                container[i][j].picture = this._delCells[index].value.picture;
                                            if (!ej.isNullOrUndefined(this._delCells[index].value.chart))
                                                container[i][j].chart = this._delCells[index].value.chart;
                                        }
                                        rmColl.push({ row: i, col: j });
                                    }
                                }
                            }
                            if (!ej.isNullOrUndefined(container[i]) && (!ej.isNullOrUndefined(container[i][j + value.colCount]))) {
                                container[i][j] = $.extend(true, {}, container[i][j + value.colCount]);
                                if (!ej.isNullOrUndefined(container[i][j + value.colCount].picture))
                                    picture = container[i][j + value.colCount].picture;
                                if (!ej.isNullOrUndefined(container[i][j + value.colCount].chart))
                                    chart = container[i][j + value.colCount].chart;
                                this._clearFormula(i, j + value.colCount);
                                container[i][j + value.colCount] = {};
                                if (!ej.isNullOrUndefined(picture))
                                    container[i][j + value.colCount].picture = picture;
                                if (!ej.isNullOrUndefined(chart))
                                    container[i][j + value.colCount].chart = chart
                                if (!ej.isNullOrUndefined(this._delCells[this._delCells.length - 1])) {
                                    index = this._delCells.length - 1;
                                    if (!ej.isNullOrUndefined(this._delCells[index].value.picture))
                                        container[i][j].picture = this._delCells[index].value.picture;
                                    if (!ej.isNullOrUndefined(this._delCells[index].value.chart))
                                        container[i][j].chart = this._delCells[index].value.chart;
                                }
                                rmColl.push({ row: i, col: j + value.colCount });
                                this._refreshFormula(i, j, value.colCount, "delete", "column", value.startCol, false);
                                this.XLComment._updateCmntFrmCntnr(i, j);
                            }
                        }
                        for (k = 0, len = rmColl.length; k < len; k++)
                            if (this.getObjectLength(container[rmColl[k].row][rmColl[k].col]) === 0) {
                                this._clearFormula(rmColl[k].row, rmColl[k].col);
                                delete container[rmColl[k].row][rmColl[k].col];
                            }
                        rmColl = [];
                    }
                    break;
                case "shiftRight":
                    var cValue;
                    for (i = value.startRow ; i < rCount; i++) {
                        for (j = maxCol, len = value.startCol; j >= len ; j--) {
                            if (!ej.isNullOrUndefined(container[i])) {
                                if (!ej.isNullOrUndefined(container[i][j])) {
                                    container[i][j + value.colCount] = $.extend(true, {}, container[i][j]);
                                    this._clearFormula(i, j);
                                    //refresh Validation
                                    if (j >= stCol && j <= enCol) {                                        
                                        this._checkIndicesInContainer(sIdx, i, stCol, "rule") && (vRule = this._dataContainer.sheets[sIdx][i][stCol]["rule"]);
                                        this._checkIndicesInContainer(sIdx, i, stCol, "cFormatRule") && (cRule = this._dataContainer.sheets[sIdx][i][stCol]["cFormatRule"]);
                                        this._refreshValAndCF(i, j, vRule, cRule, 0, value.colCount);
                                        !(stCol < 1) && this.addClass(this.getCell(i, j)[0], this.XLFormat.getFormatClass(this.getCell(i, stCol - 1)[0].className, false));
                                    }
                                    else
                                        delete container[i][j];
                                    this._refreshFormula(i, j + value.colCount, value.colCount, "insert", "column", value.startCol, true);
                                    this.XLEdit._refreshCalcEngine(i, j + value.colCount);
                                    this.XLEdit._refreshCalcEngine(i, j);
                                    this.XLComment._updateCmntFrmCntnr(i, j + value.rowCount);
                                }
                            }
                        }
                    }
                    if (this._isUndo && this._delCells.length > 0) {
                        remCells = this._delCells;
                        count = remCells[remCells.length - 1].col - remCells[0].col;
                        for (k = 0, len = remCells.length; k < len; k++) {
                            container[remCells[k].row][remCells[k].col] = remCells[k].value;
                            this._refreshFormula(remCells[k].row, remCells[k].col, count, "insert", "column", value.startRow, false);
                            this.XLEdit._refreshCalcEngine(remCells[k].row, remCells[k].col);
                            this.XLComment._updateCmntFrmCntnr(remCells[k].row, remCells[k].col);
                        }
                    }
                    break;
                case "shiftLeft":
                    this._delCells = [];
                    rmColl = [];
                    for (i = value.startRow ; i < rCount; i++) {
                        for (j = value.startCol; j <= maxCol; j++) {
                            if (j < (value.startCol + value.colCount) && !ej.isNullOrUndefined(container[i])) {
                                if (!ej.isNullOrUndefined(container[i][j])) {
                                    this._delCells.push({ row: i, col: j, value: $.extend(true, {}, container[i][j]) });
                                    this._clearFormula(i, j);
                                    container[i][j] = {};
                                    rmColl.push({ row: i, col: j });
                                }
                            }
                            if (!ej.isNullOrUndefined(container[i]))
                                if (!ej.isNullOrUndefined(container[i][j + value.colCount])) {
                                    container[i][j] = $.extend(true, {}, container[i][j + value.colCount]);
                                    this.clearTextNode(this.getCell(i, j + value.colCount));
                                    this.clearTextNode(this.getCell(i, j), this.XLEdit.getPropertyValue(i, j, "value2"));
									rmColl.push({ row: i, col: j + value.colCount });
                                    this._clearFormula(i, j + value.colCount);
                                    container[i][j + value.colCount] = {};
                                    this._refreshFormula(i, j, value.colCount, "delete", "column", value.startCol, true);
                                    this.XLComment._updateCmntFrmCntnr(i, j);
                                }
                            this.XLEdit._refreshCalcEngine(i, j);
                        }
                        for (k = 0, len = rmColl.length; k < len; k++) {
                            if (this.getObjectLength(container[rmColl[k].row][rmColl[k].col]) === 0) {
                                this._clearFormula(rmColl[k].row, rmColl[k].col);
                                this.XLEdit._refreshCalcEngine(rmColl[k].row, rmColl[k].col);
                                delete container[rmColl[k].row][rmColl[k].col];
                            }
                        }
                        rmColl = [];
                    }
                    break;
                case "shiftBottom":
                    for (i = maxRow, len = value.startRow; i >= len; i--) {
                        for (j = value.startCol; j < cCount; j++) {
                            if (!ej.isNullOrUndefined(container[i]))
                                if (!ej.isNullOrUndefined(container[i][j])) {
                                    if (ej.isNullOrUndefined(container[i + value.rowCount]))
                                        container[i + value.rowCount] = {};
                                    container[i + value.rowCount][j] = $.extend(true, {}, container[i][j]);
                                    this._clearFormula(i, j);
                                    //refresh Validation
                                    if (i >= stRow && i <= enRow) {
                                        this._checkIndicesInContainer(sIdx, stRow, j, "rule") && (vRule = this._dataContainer.sheets[sIdx][stRow][j]["rule"]);
                                        this._checkIndicesInContainer(sIdx, stRow, j, "cFormatRule") && (cRule = this._dataContainer.sheets[sIdx][stRow][j]["cFormatRule"]);                                      
                                        this._refreshValAndCF(i, j, vRule, cRule, value.rowCount, 0);
                                        !(stRow < 1) && this.addClass(this.getCell(i, j)[0], this.XLFormat.getFormatClass(this.getCell(stRow - 1, j)[0].className, false));
                                    }
                                    else
                                        delete container[i][j];
                                    this._refreshFormula(i + value.rowCount, j, value.rowCount, "insert", "row", value.startRow, true);
                                    this.XLComment._updateCmntFrmCntnr(i + value.rowCount, j);
                                }
                            this.XLEdit._refreshCalcEngine(i, j);
                            this.XLEdit._refreshCalcEngine(i + value.rowCount, j);
                        }
                        if (!ej.isNullOrUndefined(container[i]))
                            (this.getObjectLength(container[i]) === 0) && delete container[i];
                    }
                    if (this._isUndo && this._delCells.length > 0) {
                        remCells = this._delCells;
                        count = remCells[remCells.length - 1].row - remCells[0].row;
                        for (i = 0, len = remCells.length; i < len; i++) {
                            if (ej.isNullOrUndefined(container[remCells[i].row]))
                                container[remCells[i].row] = {};
                            container[remCells[i].row][remCells[i].col] = remCells[i].value;
                            this._refreshFormula(remCells[i].row, remCells[i].col, count, "insert", "row", value.startRow, true);
                            this.XLEdit._refreshCalcEngine(remCells[i].row, remCells[i].col);
                            this.XLComment._updateCmntFrmCntnr(remCells[i].row, remCells[i].col);
                        }
                    }
                    break;
                case "shiftUp":
                    rmColl = [];
                    for (i = value.startRow; i <= maxRow; i++) {
                        for (j = value.startCol; j < cCount; j++) {
                            if (i < (value.startRow + value.rowCount) && !ej.isNullOrUndefined(container[i]))
                                if (!ej.isNullOrUndefined(container[i][j])) {
                                    this._delCells.push({ row: i, col: j, value: $.extend(true, {}, container[i][j]) });
                                    this._clearFormula(i, j);
                                    container[i][j] = {};
                                    rmColl.push({ row: i, col: j });
                                }
                            if (!ej.isNullOrUndefined(container[i + value.rowCount]))
                                if (!ej.isNullOrUndefined(container[i + value.rowCount][j])) {
                                    if (ej.isNullOrUndefined(container[i]))
                                        container[i] = {};
                                    container[i][j] = $.extend(true, {}, container[i + value.rowCount][j]);
                                    this.clearTextNode(this.getCell(i + value.rowCount, j));
                                    this.clearTextNode(this.getCell(i, j), this.XLEdit.getPropertyValue(i, j, "value2"));
									this._clearFormula(i + value.rowCount, j);
                                    container[i + value.rowCount][j] = {};
                                    rmColl.push({ row: i + value.rowCount, col: j });
                                    this._refreshFormula(i, j, value.rowCount, "delete", "row", value.startRow, true);
                                    this.XLComment._updateCmntFrmCntnr(i, j);
                                }
                            this.XLEdit._refreshCalcEngine(i, j);
                        }
                    }
                    for (k = 0, len = rmColl.length; k < len; k++) {
                        if (this.getObjectLength(container[rmColl[k].row][rmColl[k].col]) === 0) {
                            this._clearFormula(rmColl[k].row, rmColl[k].col);
                            delete container[rmColl[k].row][rmColl[k].col];
                        }
                        (this.getObjectLength(container[rmColl[k].row]) === 0) && delete container[rmColl[k].row];
                    }
                    break;
            }
        },

        insertShiftRight: function (startCell, endCell) {
            var status, len, sheetIdx = this.getActiveSheetIndex(), value, count = endCell.colIndex - startCell.colIndex + 1, rows = this.getRows(sheetIdx)[1], hdrcolgrp = this._getJSSheetHeader(sheetIdx).find("tr").eq(0).parent().prev(), colgrp = $(this._getContent(sheetIdx).find("tbody").get(0)).prev();
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (this._isUndo) {
                for (var i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                    len = $(rows[i]).find('td').length - 1;
                    for (var j = startCell.colIndex; j <= len - count; j++) {
                        $(rows[i]).find('td:eq(' + j + ')')[0].outerHTML = $(rows[i]).find('td:eq(' + (j + count) + ')')[0].outerHTML;
                    }
                }
                for (var k = len; k > len - count; k--) {
                    $(hdrcolgrp).find('col').eq(k).remove();
                    $(colgrp).find('col').eq(k).remove();
                    for (var m = 0; m < rows.length; m++) {
                        $(rows[m]).find('td:eq(' + k + ')').remove();
                    }
                }
                status = "shiftLeft";
                this._insDelStatus = "delete";
            }
            else {
                for (k = 0; k < count; k++)
                    this.XLScroll._createNewColumn(sheetIdx, { rowIndex: -1, colIndex: -1 }, { rowIndex: -1, colIndex: -1 }, "insert");
                for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                    len = $(rows[i]).find('td').length - 1;
                    for (j = len - count; j >= startCell.colIndex; j--) {
                        $(rows[i]).find('td:eq(' + (j + count) + ')')[0].outerHTML = $(rows[i]).find('td:eq(' + j + ')')[0].outerHTML;
                        $(rows[i]).find('td:eq(' + j + ')')[0].innerHTML = "";
                    }
                }
                status = "shiftRight";
                this._insDelStatus = "insert";
            }
            this.performSelection(startCell, endCell);
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
            if(this.model.allowComments) this.XLComment._curCommentHCell = null;
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "insert", "insertShiftRight");
        },

        insertShiftBottom: function (startCell, endCell) {
            var i, j, k, len, sheetIdx = this.getActiveSheetIndex(), status, x, value, xTd, iTd, rowColl = this.getRows(sheetIdx), rows = rowColl[1], rowHdrs = rowColl[0];
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (!this._isUndo) {
                for (i = startCell.rowIndex; i <= endCell.rowIndex; i++)
                    this.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                for (i = rows.length - 1; i >= startCell.rowIndex  ; i--) {
                    x = i - ((endCell.rowIndex - startCell.rowIndex) + 1);
                    if (x >= startCell.rowIndex)
                        for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                            xTd = $(rows[x]).find('td').get(j);
                            iTd = $(rows[i]).find('td').get(j);
                            iTd.outerHTML = xTd.outerHTML;
                            xTd.innerHTML = "";
                            iTd.className = xTd.className;
                            xTd.className = "e-rowcell";
                        }
                }
                status = "shiftBottom";
            }
            else {
                rows = this._getContent(sheetIdx).find("tbody").get(0).rows;
                for (i = startCell.rowIndex, len = rows.length; i < len ; i++) {
                    x = i + ((endCell.rowIndex - startCell.rowIndex) + 1);
                    if (x < rows.length) {
                        for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                            $(rows[i]).find('td').get(j).outerHTML = $(rows[x]).find('td').get(j).outerHTML;
                            $(rows[x]).find('td').get(j).innerHTML = "";
                        }
                    }
                 }
            for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                for (k = len - 1, length = (len - 1) - ((endCell.rowIndex - startCell.rowIndex) + 1) ; k > length; k--) {
                    $(rows[k]).remove();
                    $(rowHdrs[k]).remove();
                    this.model.sheets[sheetIdx].rowCount = this.model.sheets[sheetIdx].rowCount - 1;
                }
            }
            this.XLScroll._refreshScroller(1, { rowIndex: -1, colIndex: -1 }, { rowIndex: -1, colIndex: -1 }, "all");
                status = "shiftUp";
            }
            this.performSelection(startCell, endCell);
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
            if(this.model.allowComments) this.XLComment._curCommentHCell = null;
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "insert", "insertShiftBottom");
        },

        insertEntireRow: function (startCell, endCell) {
            var i, len, status, sheetIdx = this.getActiveSheetIndex(), rows, rowHdrs, tbodyRowHdr, value, rowhdrSel = this._getJSSheetRowHeaderContent(sheetIdx).find(".e-rowselected");
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (!this._isUndo) {
                for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                    this.XLScroll._createNewRow(sheetIdx, i, startCell.colIndex, "insert");
					this._refreshFreezeRows(sheetIdx, i, this.model.rowHeight, "insert");
				}
                status = "insertRow";
            }
            else {
                rows = this._getContent(sheetIdx).find("tbody").get(0).rows, rowHdrs = this._getJSSheetRowHeaderContent(sheetIdx).find("tbody").get(0).rows;
                tbodyRowHdr = this._getJSSheetRowHeaderContent(sheetIdx).find("tbody").get(0);
                for (i = endCell.rowIndex; i >= startCell.rowIndex; i--) {
                    $(rows[i]).remove();
                    $(rowHdrs[i]).remove();
                    this.model.sheets[sheetIdx].rowCount = this.model.sheets[sheetIdx].rowCount - 1;
                    this.XLScroll._getRowHeights(sheetIdx, startCell.rowIndex);
					this._refreshFreezeRows(sheetIdx, i, parseInt(rowHdrs[i].style.height), "delete");
                }
                var thdr = $(tbodyRowHdr).find('td');
                for (i = 0, len = thdr.length; i < len; i++)
                    $(thdr[i]).text(i + 1);
                status = "deleteRow";
            }
            (rowhdrSel.length)?this.XLSelection.selectRows(startCell.rowIndex, endCell.rowIndex): this.performSelection(startCell, endCell);
            this.model.allowComments && this.XLComment._updateCmntArrowPos(); //Comments position update
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
			(!this._isUndo) ? this._refreshAllMergeCells(sheetIdx, startCell.rowIndex, value.rowCount, -1, 0, "insert"): this._refreshAllMergeCells(sheetIdx, startCell.rowIndex, value.rowCount, -1, 0, "delete");
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "insert", "insertRow");
        },

        insertEntireColumn: function (startCell, endCell) {
            var status, sheetIdx = this.getActiveSheetIndex(), value, colHdrSel = this._getJSSheetHeader(sheetIdx).find(".e-colselected");
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (this._isUndo) {
                this._insDelStatus = "delete";
                status = "deleteColumn";
            }
            else {
                status = "insertColumn";
                this._insDelStatus = "insert";
            }
            this.XLScroll._createNewColumn(sheetIdx, startCell, endCell, "insert");
            (colHdrSel.length)?this.XLSelection.selectColumns(startCell.colIndex, endCell.colIndex): this.performSelection(startCell, endCell);
            this.model.allowComments && this.XLComment._updateCmntArrowPos(); //Comments position update
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
			(!this._isUndo) ? this._refreshAllMergeCells(sheetIdx, -1, 0, startCell.colIndex, value.colCount, "insert"): this._refreshAllMergeCells(sheetIdx, -1, 0, startCell.colIndex, value.colCount, "delete");
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "insert", "insertColumn");
        },

        deleteShiftLeft: function (startCell, endCell) {
            var i, j, k, colCount = endCell.colIndex - startCell.colIndex + 1, sheetIdx = this.getActiveSheetIndex(), rows = this.getRows(sheetIdx)[1], status, text = [], value, uLength;
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (!this._isUndo) {
                this._insDelStatus = "delete";
                for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                    for (j = startCell.colIndex; j <= endCell.colIndex; j++)
                        text.push($(rows[i]).find('td').get(j).outerHTML);
                    for (k = startCell.colIndex, length = $(rows[i]).find('td').length; k < length - colCount; k++)
                        $(rows[i]).find('td:eq(' + k + ')')[0].outerHTML = $(rows[i]).find('td:eq(' + (k + colCount) + ')')[0].outerHTML;
                }
                status = "shiftLeft";
            }
            else {
                $.extend(true, text, this._deletedText);
                this._insDelStatus = "insert";
                for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                    uLength = $(rows[i]).find('td').length - 1;
                    for (k = uLength - colCount; k > startCell.colIndex - 1; k--)
                        $(rows[i]).find('td:eq(' + (k + colCount) + ')')[0].outerHTML = $(rows[i]).find('td:eq(' + k + ')')[0].outerHTML;
                    for (j = startCell.colIndex; j <= endCell.colIndex; j++)
                        $(rows[i]).find('td').get(j).outerHTML = text.shift();
                }
                status = "shiftRight";
            }
            this.performSelection(startCell, endCell);
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
            if(this.model.allowComments) this.XLComment._curCommentHCell = null;
            this._deletedText = text;
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "delete", "deleteShiftLeft");
        },

        deleteShiftUp: function (startCell, endCell) {
            var i, j, len, sheetIdx = this.getActiveSheetIndex(), rows = this.getRows(sheetIdx)[1], status, text = [], x, value;
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (!this._isUndo) {
                status = "shiftUp";
                for (i = startCell.rowIndex, len = rows.length; i < len ; i++) {
                    x = i + ((endCell.rowIndex - startCell.rowIndex) + 1);
                    if (x < rows.length) {
                        for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                            text.push($(rows[i]).find('td').get(j).outerHTML);
                            $(rows[i]).find('td').get(j).outerHTML = $(rows[x]).find('td').get(j).outerHTML;
                            $(rows[x]).find('td').get(j).innerHTML = "";
                        }
                    }
                }
            }
            else {
                $.extend(true, text, this._deletedText);
                for (i = rows.length - 1; i >= startCell.rowIndex  ; i--) {
                    x = i - ((endCell.rowIndex - startCell.rowIndex) + 1);
                    if (x >= startCell.rowIndex) {
                        for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                            $(rows[i]).find('td').get(j).outerHTML = $(rows[x]).find('td').get(j).outerHTML;
                            $(rows[x]).find('td').get(j).innerHTML = "";
                        }
                    }
                }
                for (i = startCell.rowIndex; i <= endCell.rowIndex; i++)
                    for (j = startCell.colIndex; j <= endCell.colIndex; j++)
                        $(rows[i]).find('td').get(j).outerHTML = text.shift();
                status = "shiftBottom";
            }
            this.performSelection(startCell, endCell);
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
            if(this.model.allowComments) this.XLComment._curCommentHCell = null;
            this._deletedText = text;
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "delete", "deleteShiftUp");
        },

        deleteEntireRow: function (startCell, endCell) {
            var i, len, table, thdr, tdHdr, tableRowHdr, status, text = [], sheetIdx = this.getActiveSheetIndex(), tr, value, rowColl, rows, rowHdrs;
            table = this._getContent(sheetIdx).find("table"), rowColl = this.getRows(sheetIdx), rows = rowColl[1], rowHdrs = rowColl[0];
            tableRowHdr = this._getJSSheetRowHeaderContent(sheetIdx).find("table"), selRowHdr = tableRowHdr.find(".e-rowselected");
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (!this._isUndo) {
                status = "deleteRow";
                for (i = endCell.rowIndex; i >= startCell.rowIndex; i--) {
                    text.push(table.find('tr').get(i).outerHTML);
					this._refreshFreezeRows(sheetIdx, i, parseInt(rowHdrs[i].style.height), "delete");
                    $(rows[i]).remove();
                    $(rowHdrs[i]).remove();
                    this.model.sheets[sheetIdx].rowCount = this.model.sheets[sheetIdx].rowCount - 1;
                    this.XLScroll._getRowHeights(sheetIdx, startCell.rowIndex + 1);
                    this.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                }
            }
            else {
                status = "insertRow";
                $.extend(true, text, this._deletedText);
                for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                    tr = text.pop();
                    $(rows[i]).before(tr);
                    tdHdr = this.XLScroll._rowHeaderTemplate(1);
                    $(tdHdr).height($(tr).height());
                    tableRowHdr.find('tr').eq(i).before(tdHdr);
                    this.model.sheets[sheetIdx].rowCount = this.model.sheets[sheetIdx].rowCount + 1;
                    this.XLScroll._getRowHeights(sheetIdx, startCell.rowIndex + 1);
					this._refreshFreezeRows(sheetIdx, i, $(tr).height(), "insert");
                }
            }
            thdr = tableRowHdr.find('td');
            for (i = 0, len = thdr.length; i < len; i++)
                $(thdr[i]).text(i + 1);
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
			(!this._isUndo) ? this._refreshAllMergeCells(sheetIdx, startCell.rowIndex, value.rowCount, -1, 0, "delete"): this._refreshAllMergeCells(sheetIdx, startCell.rowIndex, value.rowCount, -1, 0, "insert");
            (selRowHdr.length)?this.XLSelection.selectRows(startCell.rowIndex, endCell.rowIndex): this.performSelection(startCell, endCell);
            this._deletedText = text;
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "delete", "deleteRow");
        },

        deleteEntireColumn: function (startCell, endCell) {
            var i, j, len, sheetIdx = this.getActiveSheetIndex(), tbody, thdr, tbodyColHdr, colgrp, hdrcolgrp, tdColgrp, status, text = [], value, tdRowData;
            tbody = this._getContent(sheetIdx).find("tbody").get(0), tbodyColHdr = this._getJSSheetHeader(sheetIdx).find("tr").eq(0);
            colgrp = $(tbody).prev(), hdrcolgrp = tbodyColHdr.parent().prev(), tdColgrp = this.XLScroll._colGroupTemplate(1), colHdrSel = tbodyColHdr.find(".e-colselected");
            if (this.element.find(".e-selected").filter(".e-merge").length > 0 || this.element.find(".e-selected").filter(".e-mc-hide").length > 0)
                return;
            if (!this._isUndo) {
                status = "deleteColumn";
                for (j = endCell.colIndex; j >= startCell.colIndex; j--) {
					text.push($(hdrcolgrp).find('col').get(j).outerHTML);
					text.push($(colgrp).find('col').get(j).outerHTML);
					this._refreshFreezeColumns(sheetIdx, j, parseInt(colgrp.find('col')[j].style.width), "delete");
                    $(hdrcolgrp).find('col').eq(j).remove();
                    $(colgrp).find('col').eq(j).remove();
                    $(hdrcolgrp).append(tdColgrp);
                    $(colgrp).append(tdColgrp);
                }
                for (i = 0, len = tbody.rows.length; i < len; i++)
                    for (j = endCell.colIndex; j >= startCell.colIndex; j--) {
                        tdRowData = this.XLScroll._columnTemplate();
                        $(tbody.rows[i]).append(tdRowData);
                        text.push($(tbody.rows[i]).find('td').get(j).outerHTML);
                        $(tbody.rows[i]).find('td').eq(j).remove();
                    }
            }
            else {
                text = $.extend(true, text, this._deletedText);
                status = "insertColumn";
                for (i = tbody.rows.length - 1; i >= 0; i--)
                    for (j = startCell.colIndex; j <= endCell.colIndex; j++)
                        $(tbody.rows[i]).find('td').eq(j).before(text.pop());
				for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
					$(colgrp).find('col').eq(j).before(text.pop());
					tbodyColHdr.parent().find('th').eq(j).before("<th class='e-headercell'><div class='e-headercelldiv'></div></th>");
					$(hdrcolgrp).find('col').eq(j).before(text.pop());
					this._refreshFreezeColumns(sheetIdx, j, parseInt(colgrp.find('col')[j].style.width), "insert");
				}
            }
            thdr = $(tbodyColHdr).find('th');
            for (i = 0, len = thdr.length; i < len; i++)
                $(thdr[i]).find('div').text(this._generateHeaderText(sheetIdx, i + 1));      
            value = { startCol: startCell.colIndex, colCount: (endCell.colIndex - startCell.colIndex) + 1, startRow: startCell.rowIndex, rowCount: (endCell.rowIndex - startCell.rowIndex) + 1, status: status };
            this._refreshDataContainer(sheetIdx, value);
			(!this._isUndo) ? this._refreshAllMergeCells(sheetIdx, -1, 0, startCell.colIndex, value.colCount, "delete"): this._refreshAllMergeCells(sheetIdx, -1, 0, startCell.colIndex, value.colCount, "insert");
            (colHdrSel.length)? this.XLSelection.selectColumns(startCell.colIndex, endCell.colIndex):  this.performSelection(startCell, endCell);
            this._deletedText = text;
            (!this._isUndoRedo) && this._undoredoDetails(sheetIdx, startCell, endCell, "delete", "deleteColumn");
        },

		_refreshFreezeRows: function(sheetIdx, rowIdx, height, status) {
			if(rowIdx < this.getFrozenRows(sheetIdx)) {
				var fRowTop = parseInt(this._getContent(sheetIdx).find('.e-frow')[0].style.top);
				if(status === "hide" || status === "delete") {
					this._getContent(sheetIdx).find('.e-frow')[0].style.top = (fRowTop - height) + "px";
					this._getJSSheetRowHeaderContent(sheetIdx).find('.e-frow')[0].style.top = (fRowTop - height) + "px";
					(status === "delete") && (this.getSheet(sheetIdx)._frozenRows = this.getSheet(sheetIdx)._frozenRows - 1);
				}
				else if(status === "unhide" || status === "insert") {
					this._getContent(sheetIdx).find('.e-frow')[0].style.top = (fRowTop + height) + "px";
					this._getJSSheetRowHeaderContent(sheetIdx).find('.e-frow')[0].style.top = (fRowTop + height) + "px";
					(status === "insert") && (this.getSheet(sheetIdx)._frozenRows = this.getSheet(sheetIdx)._frozenRows + 1);
				}
				else{
					this._getContent(sheetIdx).find('.e-frow')[0].style.top = (fRowTop + height) + "px";
					this._getJSSheetRowHeaderContent(sheetIdx).find('.e-frow')[0].style.top = (fRowTop + height) + "px";
				}
			}
			this.getSheet(sheetIdx)._currentTopFrozenRow = this.getSheet(sheetIdx)._frozenRows;
		},
		
		_refreshFreezeColumns: function(sheetIdx, colIdx, width, status) {
			if(colIdx < this.getFrozenColumns(sheetIdx)) {
				var fRowLeft = parseInt(this._getContent(sheetIdx).find(".e-fcol")[0].style.left);
				if(status === "hide" || status === "delete") {
					this._getContent(sheetIdx).find(".e-fcol")[0].style.left = (fRowLeft - width) + "px";
					this._getJSSheetHeader(sheetIdx).find(".e-fcol")[0].style.left = (fRowLeft - width) + "px";
					(status === "delete") && (this.getSheet(sheetIdx)._frozenColumns = this.getSheet(sheetIdx)._frozenColumns - 1);
				}
				else if(status === "unhide" || status === "insert") {
					this._getContent(sheetIdx).find(".e-fcol")[0].style.left = (fRowLeft + width) + "px";
					this._getJSSheetHeader(sheetIdx).find(".e-fcol")[0].style.left = (fRowLeft + width) + "px";
					(status === "insert") && (this.getSheet(sheetIdx)._frozenColumns = this.getSheet(sheetIdx)._frozenColumns + 1);
				}
				else{
					this._getContent(sheetIdx).find(".e-fcol")[0].style.left = (fRowLeft + width) + "px";
					this._getJSSheetHeader(sheetIdx).find(".e-fcol")[0].style.left = (fRowLeft + width) + "px";
				}
			}
			this.getSheet(sheetIdx)._currentLeftFrozenColumn = this.getSheet(sheetIdx)._frozenColumns;
		},
		
        performSelection: function (startCell, endCell) {
            var sheetIdx = this.getActiveSheetIndex(), range, sheet = this.getSheet(sheetIdx);
            if (typeof (startCell) === "string") {
                range = this.getRangeIndices(startCell);
                startCell = { rowIndex: range[0], colIndex: range[1] };
                endCell = { rowIndex: range[2], colIndex: range[3] };
            }
            if(this.model.scrollSettings.allowScrolling)
                this.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
            this.setActiveCell(startCell.rowIndex, startCell.colIndex);
            this.XLSelection.selectRange(startCell, endCell);
            if(this.model.allowAutoFill)
				this.XLDragFill.positionAutoFillElement();
            sheet._startCell = startCell;
            sheet._endCell = endCell;
        },
		 _checkRowCol: function(start, end){
			if(start === 0 || end === 0)
				return;
			if(ej.isNullOrUndefined(end))
				end = start;
			return {start: start, end: end};
		 },
		 hideRow: function(startRow, endRow){
			var rowColl = this._checkRowCol(startRow, endRow);
			this._hideRow(rowColl.start - 1, rowColl.end - 1);
		 },
		 showRow: function(startRow, endRow){
			var rowColl = this._checkRowCol(startRow, endRow);
			this._showRow(rowColl.start - 1, rowColl.end - 1);
		 },
		 hideColumn: function(startCol, endCol){
			var hideCol = this._checkRowCol(startCol, endCol);
			this._hideColumn(hideCol.start - 1, hideCol.end - 1);
		 },

		 showColumn: function(startCol, endCol){
			var showCol = this._checkRowCol(startCol, endCol);
			this._showColumn(showCol.start - 1, showCol.end - 1);
		 },

        _hideRow: function (startRow, endRow) {
            var sheetIdx = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIdx), rowColl = this.getRows(sheetIdx), rows = rowColl[1], hdrRows = rowColl[0], isVirtualScroll = this.model.scrollSettings.allowVirtualScrolling,
                hideRowsColl = sheet.hideRowsCollection, hiddenHeight = sheet._hiddenHeight, rowHt, hRows = [];
            if (!this._isUndoRedo) {
                this.XLEdit._updateDataContainer({ rowIndex: startRow - 1, colIndex: 0 }, { hRow: true });
                if (!isVirtualScroll)
                    this.addClass(this._getJSSheetRowHeaderContent(sheetIdx).find('.e-rowheader')[startRow - 1], "e-hrow");
            }
            for (var i = startRow; i <= endRow; i++) {
                this.XLEdit._updateDataContainer({ rowIndex: i, colIndex: 0 }, { isRHide: true });
                hideRowsColl.push(i);
                hRows.push(i);
                if (isVirtualScroll)
                    this._refreshViewport(this.getSheet(sheetIdx)._scrollTop, sheetIdx);
                else {
                    if (this._isRowViewable(sheetIdx, i)) {
                        this.addClass(rows[i], "e-r-hide");
                        this.addClass(hdrRows[i], "e-r-hide");
                        rowHt = parseInt(rows[i].style.height);
                    }
                    else
                        rowHt = this.model.rowHeight;
                    hiddenHeight = hiddenHeight + rowHt;
                    this._refreshFreezeRows(sheetIdx, i, rowHt, "hide");
                    if (this.model.scrollSettings.allowScrolling && !this.model.sheets[sheetIdx]._importRendered)
                        this.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                }				
            }
            sheet.hideRowsCollection = hideRowsColl;   
            if (!this.model.sheets[sheetIdx]._importRendered) {
                this.model.allowComments && this.XLComment._updateCmntArrowPos();
                !isVirtualScroll && (sheet._hiddenHeight = hiddenHeight);
				if(this.model.scrollSettings.allowScrolling && !isVirtualScroll)
					this.XLScroll._refreshScroller(sheetIdx, "refresh", "vertical");
                if (!this._isUndoRedo) {
					this.getSheet(sheetIdx)._activeCell.rowIndex = endRow + 1;
					this.XLSelection.selectRow(endRow + 1, sheetIdx);
					if(this.model.allowAutoFill)
						this.XLDragFill.hideAutoFillElement();
                    var details = { sheetIndex: sheetIdx, reqType: "show-hide", action: "hide", operation: "row", bSelect: {startRow: startRow, endRow: endRow}, aSelect: {startRow: endRow + 1, endRow: endRow + 1}, addBRows: [startRow - 1], remBRows: [], hideRows: hRows };
                    this._completeAction(details);
                }
            }
        },

        _showRow: function (startRow, endRow) {
            var sheetIdx = this.getActiveSheetIndex(), strtIdx = (startRow - 1 > 0) ? startRow - 1 : 0, sheet = this.getSheet(sheetIdx), rowColl = this.getRows(sheetIdx), rows = rowColl[1], rowHeaders = rowColl[0], idx,
                hRows = [], addBRow = [], remBRow = [], hideRow = sheet.hideRowsCollection, hiddenHeight = sheet._hiddenHeight, colgrp = $(this._getJSSheetHeader(sheetIdx).find('colgroup')[0].outerHTML).attr("id", "ss"), 
				emptyTable = ej.buildTag("table#wrapTable.e-table", {}), dataContainer = this._dataContainer.sheets[sheetIdx], isVirtualScroll = this.model.scrollSettings.allowVirtualScrolling;
            if (!isVirtualScroll) {
                emptyTable.append(colgrp);
                this.element[0].insertBefore(emptyTable[0], this.element[0].childNodes[0]);
            }
			if (!this._isUndoRedo && ((startRow === endRow) && ej.isNullOrUndefined(this.XLEdit.getPropertyValue(strtIdx, 0, "isRHide"))))
                return;
            for (j = strtIdx; j <= endRow; j++) {
                hideValue = hideRow.indexOf(j);
                if (hideValue > -1) {
                    delete dataContainer[j][0].isRHide;
                    hideRow.splice(hideValue, 1);
                    hRows.push(j);
                    if (!isVirtualScroll) {
                        this._removeClass(rows[j], "e-r-hide");
                        this._removeClass(rowHeaders[j], "e-r-hide");
                        rowHt = parseInt(rows[i].style.height);
                        hiddenHeight = hiddenHeight + rowHt;
                        this._refreshFreezeRows(sheetIdx, j, rowHt, "unhide");
                    }                        
					if(!this._isUndoRedo && (j > 0)) {
					    remBRow.push(j - 1);
					    delete dataContainer[j - 1][0].hRow;
                        if(!isVirtualScroll)
						    this._removeClass($(rowHeaders[j - 1]).find('td')[0], "e-hrow");
					}
					else if (j === 0) {
					    delete dataContainer[0][0].hRow;
					    if (!isVirtualScroll) {
					        rowHdr.height(20.5);
					        this._removeClass(rowHdr[0], "e-hrow");
					    }
					}
                }
                if (!isVirtualScroll) {
                    this._setWrapHeight(sheetIdx, j, rows, rowHeaders, emptyTable, "visible");
                    emptyTable.remove();
                }
                this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();
            }
            if (isVirtualScroll)
                this._refreshViewport(this.getSheet(sheetIdx)._scrollTop, sheetIdx);
			if (!sheet._importRendered && !this._isUndoRedo){
				var aSelect;
				if (startRow < endRow) {
					this.setActiveCell(startRow, 0);
					this.XLSelection.selectRows(startRow, endRow, sheetIdx);
					aSelect =  {startRow: startRow, endRow: endRow};
				}
				else {
					this.setActiveCell(startRow - 1, 0);
					this.XLSelection.selectRow(startRow - 1, sheetIdx);
					aSelect = {startRow: startRow - 1, endRow: startRow - 1};
				}
				this.XLDragFill.positionAutoFillElement();
				var details = { sheetIndex: sheetIdx, reqType: "show-hide", action: "show", operation: "row", bSelect: {startRow: startRow, endRow: endRow}, aSelect: aSelect, addBRows: addBRow, remBRows : remBRow, hideRows: hRows };
                this._completeAction(details);
			}
            !sheet._importRendered && this.model.allowComments && this.XLComment._updateCmntArrowPos();
			sheet._hiddenHeight = hiddenHeight;
			if(this.model.scrollSettings.allowScrolling)
				this.XLScroll._refreshScroller(sheetIdx, "refresh", "vertical");
        },

		_colgroupRefresh: function (sheetIdx) {
            if (this.model.allowResizing) {
                var headerCol =  this._getJSSheetHeader(sheetIdx).find('colgroup');
                var contentCol = this._getContent(sheetIdx).find('colgroup');
            }
            var headerColClone = $(headerCol).clone();
            var contentColClone = $(contentCol).clone();
            $(contentCol).remove();
            $(headerCol).remove();
            $(headerColClone).prependTo(this._getJSSheetHeader(sheetIdx).find('.e-table'));
            $(contentColClone).prependTo(this._getContent(sheetIdx).find('.e-table'));
        },
		
        _hideColumn: function (startCol, endCol) {
            var i, j, len, sheetIdx = this.getActiveSheetIndex(), colgrpColl = this._getContent(sheetIdx).find('col'), hdrolgrp = this._getJSSheetHeader(sheetIdx).find("col"),
                filterSpan, isCHide, sheet = this.getSheet(sheetIdx), hideRowColl = sheet.hideColsCollection, hiddenWidth = sheet._hiddenWidth, colWt, hCols = [], addBCol = [], range,
                filterKeys;
            for (j = startCol; j <= endCol; j++) {
                isCHide = this.XLEdit.getPropertyValue(0, j, "isCHide");
                if (!ej.isNullOrUndefined(isCHide)) {
                    this.XLEdit._updateDataContainer({ rowIndex: 0, colIndex: j }, { isCHide: true });
                    if (this._isColumnViewable(sheetIdx, j))
                        isViewed = true;
                    if (isViewed) {
                        this.addClass(colgrpColl[j], "e-c-hide");
                        this.addClass(hdrolgrp[j], "e-c-hide");
                        range = this.getRange(0, j, this.model.rowCount - 1, j);
                        range.filter(".e-sswraptext").hide();
                    }
                    if (!sheet._importRendered) {
                        hideRowColl.push(j);
                        hCols.push(j);
                    }
                    colWt = parseInt(colgrpColl[j].style.width);
                    hiddenWidth = hiddenWidth + colWt;
                    this._refreshFreezeColumns(sheetIdx, j, colWt, "hide");
                    fltrKeys = this.getObjectKeys(filterColl);
                    for (i = fltrKeys[0], len = fltrKeys.length; i < len; i++){
                        if(!ej.isNullOrUndefined(filterColl[fltrKeys[i]])){
                            if(ej.isNullOrUndefined(this.XLEdit.getPropertyValue(i, j, "isSpanHide"))) {
                                this.XLEdit._updateDataContainer({rowIndex: i, colIndex: j}, {isSpanHide: true});
                                if(isViewed){
                                    filterSpan = this.getCell(i, j).find(".e-filterspan");
                                    this._addClass(filterSpan[0], "e-hide");
                                }
                            }
                        }
                    }
                }
            }
			if(!this._isUndoRedo){
			    addBCol.push(startCol - 1);
                if(isViewed){
					if(startCol === 0){
						colHdr.width(28);
						this.addClass(colHdr[0],"e-hfcol");
					}
					else
				    	this.addClass(this._getJSSheetHeader(sheetIdx).find('th:eq(' + (startCol - 1) + ')')[0], "e-hcol");
				}
			}
			sheet._hiddenWidth = hiddenWidth;
			if(this.model.scrollSettings.allowScrolling)
			    this.XLScroll._refreshScroller(sheetIdx, "refresh", "horizontal");
            if (!sheet._importRendered) {
                var aSelect;
                if (xlObj.getSheet(sheetIdx).columnsWidthCollection.length - 1 === endCol) {
					this.XLSelection.selectColumn(endCol - 1, sheetIdx);
					aSelect = {startCol: endCol - 1 , endCol: endCol - 1};
				}
				else {
					aSelect = {startCol: endCol + 1, endCol: endCol + 1};
					this.XLSelection.selectColumn(endCol + 1, sheetIdx); 
				}
				this.XLDragFill.positionAutoFillElement();
				if (!this._isUndoRedo) {
                    var details = { sheetIndex: sheetIdx, reqType: "show-hide", action: "hide", operation: "column", bSelect: {startCol: startCol, endCol: endCol}, aSelect: aSelect, hideCols: hCols, addBCols: addBCol, remBCols: [] };
                    this._completeAction(details);
				}
				if(this.model.allowComments)
					this.XLComment._updateCmntArrowPos();
            }
			else
				this.setActiveCell(0,0);
            if(this._browserDetails.name.indexOf("msie") > -1)
                this._colgroupRefresh(sheetIdx);
        },

        _showColumn: function (startCol, endCol) {
            var sheetIdx = this.getActiveSheetIndex(), colIdx = (startCol - 1 > 0) ? startCol - 1 : 0, idx, sheet = this.getSheet(sheetIdx), hideColumn = sheet.hideColsCollection, colgrpColl = this._getContent(sheetIdx).find('col'),
				hCols = [], addBCol = [], fltrKeys, isViewd = false, remBCol = [], hdrolgrp = this._getJSSheetHeader(sheetIdx).find("col"), filterColl = this.XLFilter._filterColl, filterSpan, hiddenWidth = sheet._hiddenWidth, 
                sheetHdr = this._getJSSheetHeader(sheetIdx).find('th'), colWt, range, colWtColl = sheet.columnsWidthCollection;
            
            if (!this._isUndoRedo && ((startCol === endCol) && ej.isNullOrUndefined(this.XLEdit.getPropertyValue(0, startCol, "isCHide"))))
                return;
            for (j = colIdx; j <= endCol; j++) {
                hideValue = hideColumn.indexOf(j);
                if (hideValue > -1) {
                    if(this._isColumnViewable(sheetIdx, j))
                        isViewd = true;
                    if(isViewed) {
                        this._removeClass(colgrpColl[j], "e-c-hide");
                        this._removeClass(hdrolgrp[j], "e-c-hide");
                    }
                    colWt = colWtColl[j];
                    hiddenWidth = hiddenWidth + colWt;
					this._refreshFreezeColumns(sheetIdx, j, colWt, "unhide");                   
                    fltrKeys = this.getObjectKeys(filterColl);
                    for(i = fltrKeys[0], len = fltrKeys.length; i < len; i++){
                        if(!ej.isNullOrUndefined(filterColl[fltrKeys[i]])){
                            if(this.XLEdit.getPropertyValue(i, j, "isSpanHide")) {
                                delete this._dataContainer.sheets[sheetIdx][0][i]["isSpanHide"];
                                if(isViewed){
                                    filterSpan = this.getCell(i, j).find(".e-filterspan");
                                    this._removeClass(filterSpan[0], "e-hide");
                                }
                            }
                        }
                    }
                    hideColumn.splice(hideValue, 1);
                    hCols.push(j);
                    if(isViewed) {
                        range = this.getRange(0, j, this.model.rowCount - 1, j);
                        range.filter(".e-sswraptext").show();
                    }
					if(!this._isUndoRedo && (j > 0)) {					
						isViewed && this._removeClass(sheetHdr.get(j - 1), "e-hcol");
						remBCol.push(j - 1);
					}
					else if(j === 0){
						colHdr.width(29);
						this._removeClass(colHdr[0],"e-hfcol");
					}
                }
            }
			if (!this._isUndoRedo && !sheet._importRendered) {
                if(startCol < endCol) {
					this.XLSelection.selectColumns(startCol, endCol, sheetIdx);
					aSelect = {startCol: startCol, endCol: endCol};
				}
				else{
					this.XLSelection.selectColumn(colIdx, sheetIdx);
					aSelect = {startCol: colIdx, colIdx: endCol};
				}
				this.XLDragFill.positionAutoFillElement();
                var details = { sheetIndex: sheetIdx, reqType: "show-hide", action: "show", operation: "column", bSelect: {startCol: startCol, endCol: endCol}, aSelect: aSelect, addBCols: addBCol, remBCols: remBCol, hideCols: hCols };
            }
            if(this._browserDetails.name.indexOf("msie") > -1)
                this._colgroupRefresh(sheetIdx);
            !this.model.sheets[sheetIdx]._importRendered && this.model.allowComments && this.XLComment._updateCmntArrowPos();
			this.getSheet(sheetIdx)._hiddenWidth = hiddenWidth;
			if(this.model.scrollSettings.allowScrolling)
				this.XLScroll._refreshScroller(sheetIdx, "refresh", "horizontal");
        },

        _destroySheet: function (index) {
            this.element.find("#" + this._id + "_" + this.model.sheets[index]._sheetName.value).empty();
        },

        _destroy: function () {
            var i = 1, name;
            this._unWireEvents();
            $("#" + this._id + "_sscustomformat").remove();
             if (!this.importSheet) {
               this._destroySubControls();
                this.element.empty();
                this.element.css({ "height": "", "width": "" });
                this.element.removeAttr("tabindex");
                this.element.removeEleEmptyAttrs();
             }
			 else{
		    this.element.find(".e-spreadsheet-list").empty();
            while (i <= this.model.sheetCount) {
                name = this.getSheet(i)._sheetName.value;
                delete $.templates[this._id + "JSONTemplate" + i];
                delete $.templates[this._id + "JSONRowHdrTemplate" + i];
                CalcEngine.unregisterGridAsSheet(name, name);
                i++;
            }
            this.model.sheets = [];
            this.model.nameManager = [];
            $("#" + this._id + "_nmgrid").ejGrid("option", { dataSource: [] });
            this.XLRibbon._updateUseInFormulaTrgt();
            CalcEngine._tokenCount = 0;
			 
			 }
        },

		 _destroySubControls:function(){
            var dlgs, id = this._id, ddl;
            //destroying dropdown list
            ddl = $("input.e-" + id + "-ddl");        
            if (ddl.length)
                ddl.ejDropDownList("destroy");
            if (this.model.allowFiltering)//destroy filter
                this._excelFilter.resetExcelFilter();
            if (this.model.showRibbon) {
                $("#" + this._id + "_nmgrid").ejGrid("destroy");
                $("#" + this._id + "_Ribbon_background-color").ejColorPicker("destroy");
                $("#" + this._id + "_Ribbon_color").ejColorPicker("destroy");
            }
            //Removing dialogs appended in body
            dlgs = $(".e-" + id + "-dlg");
            if (dlgs.length)
                dlgs.remove();
            if(this.model.allowFormulaBar)
            $("#" + this._id + "_inputbox").ejAutocomplete("destroy");          
            if(this.model.allowAutoFill)
            $("#" + this._id + "_dragfilloptionbtn").ejSplitButton("destroy")
            if (this.model.enableContextMenu) {
                $("#" + this._id + "_contextMenuCell").ejMenu("destroy").remove();
                $("#" + this._id + "_contextMenuColumnHeader").ejMenu("destroy").remove();
                $("#" + this._id + "_contextMenuRowHeader").ejMenu("destroy").remove();
                $("#" + this._id + "_contextMenuFooter").ejMenu("destroy").remove();
                $(".e-" + id + "-cmenu").remove();
            }
            $("#" + this._id + "_copyPaste").remove();
            $("#" + this._id).ejWaitingPopup("destroy");
        },

        _unWireEvents: function () {
            var elem = this.element, evnt = ej.eventType, id = this._id;
            this._off(elem, evnt.mouseDown, ".e-ribbon, .e-spreadsheetcontentcontainer .e-vscroll, .e-spreadsheetcontentcontainer .e-hscroll", this._rClickHandler);
            this._off(elem, evnt.mouseDown, ".e-spreadsheetcontentcontainer .e-content,.e-spreadsheetrowheader,.e-spreadsheetheader,.e-spreadsheetactpanel", this._mouseDownHandler);
            this._off(elem, "keydown", this._keyDownHandler);
            this._off(elem, "click", ".e-spreadsheetcontentcontainer .e-content,.e-spreadsheetrowheader,.e-spreadsheetheader,.e-spreadsheetactpanel, .e-ss-object", this._cellClick);
            this._off(elem, "dblclick", ".e-spreadsheetcontentcontainer,.e-spreadsheetheader,.e-spreadsheetrowheader", this._recorddblClickHandler);
            this._off(elem, "dblclick", ".e-numericcontainer .e-numericitem", this._pagerdblClickHandler);
            this._off(elem, evnt.mouseMove, this._mouseMove);
            this._off(elem, evnt.mouseUp, this._sheetMouseUp);
            if(this.model.allowResizing)
                this._off(elem, evnt.mouseLeave, ".e-spreadsheetrowheader,.e-spreadsheetheader", this._mouseLeave);
            this._off(elem, evnt.click, ".e-ssfiltered, .e-ssfilter, .e-ssfiltered-asc, .e-ssfilter-asc, .e-ssfiltered-dsc, .e-ssfilter-dsc", this._filterDownHandler);
            this._off(elem, evnt.mouseDown, ".e-ssddl", this._ddlopen);
            this._off(elem, "mouseover", ".e-rowcell", this._cellHoverHandler);
			this._off( elem, "mouseover", ".e-viewhdrlbl, .e-vheading", this._headingchkboxHover);
			this._off( elem, "mouseover", ".e-viewlinlbl, .e-vgridlines", this._gridlineschkboxHover);
			this._off( elem, "mouseleave", ".e-viewhdrlbl, .e-vheading", this._headingchkboxLeave);
			this._off( elem, "mouseleave", ".e-viewlinlbl, .e-vgridlines", this._gridlineschkboxLeave);
            this._off(this._getInputBox(), "input", this.XLEdit._formulaBoxInputChange);
            this._off($('#' + id + '_cellstyles'), evnt.click, this._cellStyleClick);
            this._off($('#' + id + '_formatastable'), evnt.click, this._formatAsTableClick);
            this._off($('#' + id + '_nmuseinformula'), evnt.click, ".e-nmuseinformularow", this._useInFormulaClick);
            this.element.find(".e-spreadsheet-list").unbind('mousewheel DOMMouseScroll', $.proxy(this.XLScroll._spreadMouseWheel, this.XLScroll));
            this._off($(".e-menu .e-chartcontent .e-chartcell"), evnt.click, this._chartClickHandler);
            this._off($("#" + id + "_allchart").find(".e-chartimg"), evnt.click, this._allChartType);
            if(this.model.scrollSettings.isResponsive)
                $(window).unbind("resize", $.proxy(this._spreadsheetonresize, this));
        },

        getValueRowCol: function (sheetIdx, rowindex, colindex) {
            var container = this._dataContainer, sheets = container.sheets;
            rowindex = rowindex - 1, colindex = colindex - 1;
            if (sheetIdx in sheets && rowindex in sheets[sheetIdx] && colindex in sheets[sheetIdx][rowindex])
                return container.sharedData[sheets[sheetIdx][rowindex][colindex].value];
        },

        setValueRowCol: function (sheetIdx, value, rowindex, colindex) {
            var cellInfo, value2, rIdx = rowindex - 1, cIdx = colindex - 1, cell = {rowIndex: rIdx, colIndex: cIdx};
            cellInfo = this.getRangeData([rIdx, cIdx, rIdx, cIdx], false, ["formatStr", "type", "thousandSeparator", "decimalPlaces"], sheetIdx)[0];
            if (cellInfo.formatStr)
                value2 = this.XLFormat._format(value, { formatStr: cellInfo.formatStr, type: cellInfo.type, thousandSeparator: cellInfo.thousandSeparator, decimalPlaces: cellInfo.decimalPlaces });
            if(cellInfo.type == "fraction")
                value2 = this.XLFormat._format(value, { type: cellInfo.type });
            this.XLEdit._updateDataContainer(cell, { value2: value2 || value, calcValue: this.XLEdit._parseValue(value).value });
            this.XLEdit._refreshCellAlignment(cell, value, cellInfo.type);
        },

        getActiveCellElem: function (sheetIdx) {
            sheetIdx = this._getSheetIndex(sheetIdx);
            var sheet = this.getSheet(sheetIdx);
            return this.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex, sheetIdx);
        },

        //HyperLink 
        _hlDlgOpen: function () {
            var sheetIdx = this.getActiveSheetIndex(), selected = this.getActiveCellElem(sheetIdx);
            if (!selected.length)
                return false;
            var xlId = this._id, hlTabObj, webAddrElem = $("#" + xlId + "_Ribbon_webAddress"), cellAddrElem = $("#" + xlId + "_Ribbon_cellAddress"), webValElem = $("#" + xlId + "_Ribbon_findText"), value;
            var data = this.XLEdit.getPropertyValue(selected.parent("tr").index(), selected[0].cellIndex, "hyperlink", sheetIdx), node, treeObj, cellValElem = $("#" + xlId + "_Ribbon_cellText");
            $("#" + xlId + "_HLDialog").ejDialog("open");
            treeObj = $("#" + xlId + "_Ribbon_sheentName").data("ejTreeView");
            treeObj.option("fields", { dataSource: this.XLRibbon._sheetData() });
            hlTabObj = $("#" + xlId + "_HLDialogTab").data("ejTab");
            cellValElem.attr("disabled", false);
            webValElem.attr("disabled", false);
            if (!this.isUndefined(data)) {
                this.isUndefined(data.webAddr) ? hlTabObj.option({ selectedItemIndex: 1 }) : hlTabObj.option({ selectedItemIndex: 0 });
                if (!this.isUndefined(data.cellAddr)) {
                    cellAddrElem.val(data.cellAddr.split("_")[0]);
                    node = parseInt(data.cellAddr.split("_")[1]) + 1;
                    treeObj.selectNode(node);
                    webAddrElem.val("http://");
                }
                else {
                    webAddrElem.val(data.webAddr);
                    cellAddrElem.val("A1");
                }
            }
            else {
                if (hlTabObj.model.selectedItemIndex === 0) {
                    webAddrElem.val("http://");
                    cellValElem.val("");
                }
                else {
                    treeObj.selectNode(sheetIdx + 1);
                    cellValElem.val(treeObj.getText(sheetIdx + 1) + "!A1");
                    cellAddrElem.val("A1");
                }
            }
            if ($.validator) {
                $("#" + xlId + "_Form_Ribbon_cellAddress").validate().resetForm();
                $("#" + xlId + "_Form_Ribbon_webAddress").validate().resetForm();
            }
            webValElem.val("");
            value = this.XLEdit.getPropertyValueByElem(selected);
            if (!this.isUndefined(value)) {
                if (this.isFormula(value) || (this.XLEdit.getPropertyValueByElem(selected, "type") && this.XLEdit.getPropertyValueByElem(selected, "type").indexOf("date") > -1) || $.isNumeric(value)) {
                    cellValElem.val(this._getLocStr("HyperLinkHide"));
                    webValElem.val(this._getLocStr("HyperLinkHide"));
                    cellValElem.attr("disabled", true);
                    webValElem.attr("disabled", true);
                }
                else {
                    cellValElem.val(value);
                    webValElem.val(value);
                }
            }
        },


        _setLink: function (selCells) {
            var cellText, cellAddr, webText, webAddr, range, sheetIdx, treeObj, actShtIdx = this.getActiveSheetIndex(), linkAdr, value, cell = this.getActiveCell(actShtIdx), cellVal, tabIdx;
            range = this.getSheet(actShtIdx).selectedRange;
            tabIdx = $("#" + this._id + "_HLDialogTab").ejTab("option", "selectedItemIndex");
            cellText = $("#" + this._id + "_Ribbon_cellText").val();
            cellAddr = $("#" + this._id + "_Ribbon_cellAddress").val();
            webText = $("#" + this._id + "_Ribbon_findText").val();
            webAddr = $("#" + this._id + "_Ribbon_webAddress").val();
            treeObj = $("#" + this._id + "_Ribbon_sheentName").data("ejTreeView"), sheetIdx = this._getSheetIndexByName(treeObj.getSelectedNode().text());
            if (!cellText.length && !webText.length) {
                cellText = treeObj.getSelectedNode().text() + "!" + cellAddr;
                webText = webAddr;
            }
            cellVal = this.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex);
   		    this._prevText = ej.isNullOrUndefined(cellVal) ? "" : cellVal; 
   		    if (this.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, "type") && this.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, "type").indexOf("date") > -1)
                value = this.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, "value2");
            else if ($.isNumeric(cellVal) || this.isFormula(cellVal))
                value = cellVal;
            else
                value = (tabIdx === 0) ? webText : cellText;
            this.XLEdit.updateCellValue(cell, value);
            (this.isFormula(value) || $.isNumeric(value)) && (value = this.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, "value2"));
            linkAdr = (tabIdx === 0) ? { webAddr: webAddr, text: value } : { cellAddr: cellAddr, text: value };
            this.setHyperlink(this._getAlphaRange(actShtIdx, range[0], range[1], range[2], range[3]), linkAdr, sheetIdx, selCells);
        },

        setHyperlink: function (range, linkRefer, sheetIdx, selCells) {
            if (this.model.allowHyperlink) {
                var args = {
                    range: range,
                    address: linkRefer.webAddr,
                    text: linkRefer.text,
                    sheetIndex: sheetIdx,
                    reqType: "hyperlink",
                    action: "add"
                };
                if (this._trigActionBegin(args))
                    return;
                range = args.range, linkRefer.webAddr = args.address, linkRefer.text = args.text, sheetIdx = args.sheetIndex;
                var cellText, cellAddr, webText, webAddr, actShetIdx = this.getActiveSheetIndex(), splitStr, selCell = [], data, val, i, len, selected = this._getAlphaRangeCells(range);
                var details = { sheetIndex: this.getActiveSheetIndex(), reqType: "hyperlink" }, cValue;
                if (ej.isNullOrUndefined(range)) {
                    rng = this.getRangeIndices(range);
                    selCellObj = this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] })
                }
                else {
                    selCellObj = this.getSheet(actShetIdx)._selectedCells;
                    rng = this.getSheet(actShetIdx).selectedRange;
                }
                details.action = ej.isNullOrUndefined(this.XLEdit.getPropertyValue({ rowIndex: selCellObj[0].rowIndex, colIndex: selCellObj[0].colIndex }, "hyperlink")) ? "add" : "edit";
                if (!this._isUndoRedo) {
                    if (!ej.isNullOrUndefined(linkRefer.webAddr)) {
                        webAddr = linkRefer.webAddr;
                        webText = !ej.isNullOrUndefined(linkRefer.text) ? linkRefer.text : this.XLEdit.getPropertyValue({ rowIndex: selCellObj[0].rowIndex, colIndex: selCellObj[0].colIndex }, "value2");
                        if (webAddr.indexOf("http") === -1)
                            webAddr = "http://" + webAddr;
                    }
                    else {
                        cellAddr = linkRefer.cellAddr;
                        cellText = !ej.isNullOrUndefined(linkRefer.text) ? linkRefer.text : this.XLEdit.getPropertyValue({ rowIndex: selCellObj[0].rowIndex, colIndex: selCellObj[0].colIndex }, "value2");
                        if (cellAddr.indexOf("!") > -1) {
                            splitStr = cellAddr.split("!");
                            sheetIdx = this._getSheetIndexByName(splitStr[0]);
                            cellAddr = splitStr[1];
                        }
                        cellAddr.indexOf("$") > -1 && (cellAdr = cellAddr.split("$").join(""));
                    }
                    for (i = 0, len = selCellObj.length; i < len; i++)
                        selCell.push({ rowIndex: selCellObj[i].rowIndex, colIndex: selCellObj[i].colIndex, prevLink: this.XLEdit.getPropertyValue({ rowIndex: selCellObj[i].rowIndex, colIndex: selCellObj[i].colIndex }, "hyperlink") });
                    (this.isUndefined(cellAddr)) ? this.updateUniqueData({ hyperlink: { "webAddr": webAddr } },rng) : this.updateUniqueData({ hyperlink: { "cellAddr": cellAddr + "_" + (sheetIdx) } }, rng);
                    for (i = 0, len = selCellObj.length; i < len; i++) {
                        val = this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "value2");
						selCell[i].prevText = (i === 0) ? this._prevText : val;
						if(this._isRowViewable(sheetIdx, selCellObj[i].rowIndex)) {
							$(selected.eq(i).find("a")).remove();
							$(selected.eq(i).contents()[0]).remove();
							selected.eq(i).removeClass("e-commentcell");
							isViewed = true;
						}
						(this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "comment", actShetIdx)) && this.XLComment.setComment([selCellObj[i].rowIndex, selCellObj[i].colIndex, selCellObj[i].rowIndex, selCellObj[i].colIndex], null, false);
                        data = this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "hyperlink", actShetIdx);
                        if (data.cellAddr) {
                            val = (i === 0) ? cellText : val;
                            isViewed && selected.eq(i).append(ej.buildTag("a", val, "", { href: "#" }));
                            selCell[i].curText = cellText;
                        }
                        else {
                            val = (i === 0) ? webText : val;
                            isViewed && selected.eq(i).append(ej.buildTag("a", val, "", { href: data.webAddr, target: "_blank" }));
                            selCell[i].curText = webText;
                        }
                        selCell[i].curLink = this.XLEdit.getPropertyValue(selCellObj.rowIndex, selCellObj.colIndex, "hyperlink");
						isViewed && $(selected[i]).find("a").addClass("e-hyperlinks");
                    }
                    this._dupDetails = true;
                    this.XLFormat.format({ "style": { "color": "#428bca" } }, range);
                    this._dupDetails = false;
                    details.selCells = selCell;
                    !this.model.sheets[actShetIdx]._importRendered && this._completeAction(details);
                }
                else {
                    for (i = 0, len = selCellObj.length; i < len; i++) {
                        if (!ej.isNullOrUndefined(selCells[i].prevLink)) {
                            this.XLEdit._updateDataContainer({rowIndex: selCellObj[i].rowIndex, colIndex: selCellObj[i].colIndex}, { "hyperlink": selCells[i].prevLink });
                            if(this._isRowViewable(sheetIdx, selCellObj[i].rowIndex)) {
								$(selected[i]).find("a").remove();
								if (i === 0)                               
									!(ej.isNullOrUndefined(this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "hyperlink").cellAdr)) ? $(selected[i]).text("").append(ej.buildTag("a", selCells[i].prevText, "", { href: "#" })) : $(selected[i]).text("").append(ej.buildTag("a", selCells[i].prevText, "", { href: this.XLEdit.getPropertyValueByElem($(selected[i]), "hyperlink").webAdr, target: "_blank" }));
								else {
									cValue = $(selected[i]).text();                    
									!(ej.isNullOrUndefined(this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "hyperlink").cellAdr)) ? $(selected[i]).text("").append(ej.buildTag("a", cValue, "", { href: "#" })) : $(selected[i]).text("").append(ej.buildTag("a", cValue, "", { href: this.XLEdit.getPropertyValueByElem($(selected[i]), "hyperlink").webAdr, target: "_blank" }));
								}
								$(selected[i]).find("a").addClass("e-hyperlinks");
							}							                           
                            this._dupDetails = true;
                            this.XLFormat.format({ style: { color: "#428bca" } });
                            this._dupDetails = false;
                        }
                    }
                }
            }
        },

        removeHyperlink: function (range, isClearHLink, status, cells, skipHiddenRow) {
            var i, len, rLink = [], selCellObj, rng, sheetIdx = this.getActiveSheetIndex(), selCell = [], cell, selected = this._getAlphaRangeCells(range);
            var details = { sheetIndex: sheetIdx, reqType: "hyperlink", action: "delete" };
			if (ej.isNullOrUndefined(range)) {
                rng = this.getRangeIndices(range);
                selCellObj = this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] })
            }
            else
                selCellObj = this.getSheet(sheetIdx)._selectedCells;
            for (i = 0, len = selCellObj.length; i < len; i++) {
				if (!skipHiddenRow || !this._isHiddenRow(selCellObj.rowIndex)) {
					selCell.push({ rowIndex: selCellObj[i].rowIndex, colIndex: selCellObj[i].colIndex, prevLink: this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "hyperlink"), prevText: this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "value2") });
					this._dupDetails = true;
					if (selCell.prevLink) {
						if (isClearHLink)
							this.XLFormat.format({ style: { "text-decoration": "underline" } }, this._getAlphaRange(sheetIdx, selCellObj[i].rowIndex, selCellObj[i].cellIndex, selCellObj[i].rowIndex, selCellObj[i].cellIndex));
						else
							this.XLFormat.format({ style: { color: "#444445" } }, this._getAlphaRange(sheetIdx, selCellObj[i].rowIndex, selCellObj[i].cellIndex, selCellObj[i].rowIndex, selCellObj[i].cellIndex));
					}
					this._dupDetails = false;
					this.clearRangeData(null, ["hyperlink"], [selCellObj[i]]);
					selCell[i].curLink = this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "hyperlink");
					if(this._isRowViewable(sheetIdx, selCellObj[i].rowIndex)) {
						$(selected[i]).find("a").contents().unwrap();
						$(selected[i]).find('a').removeClass("e-hyperlinks");
						isViewed = true;
					}
					if(this._isUndoRedo && !this._isSort)
						this.XLEdit._updateDataContainer({rowIndex: selCellObj[i].rowIndex, colIndex: selCellObj[i].colIndex}, { value: cells[i].prevText }, sheetIdx);
					if(this._isUndoRedo && !this._isSort) 
						isViewed && selected.eq(i).text(cells[i].prevText);
					selCell[i].curText = this.XLEdit.getPropertyValue(selCellObj[i].rowIndex, selCellObj[i].colIndex, "value2");
					if ((this._isUndoRedo && !this._isSort) && !ej.isNullOrUndefined(cells[i].prevLink)) {
						this.XLEdit._updateDataContainer({rowIndex: selCellObj[i].rowIndex, colIndex: selCellObj[i].colIndex}, { "hyperlink": cells[i].prevLink });
						cell = { rowIndex: cells[i].rowIndex, colIndex: cells[i].colIndex };
						this.XLSelection.selectRange(cell, cell);
						this._setLink([cells[i]]);
						rLink.push(i);
					}
				}
            }
            for (i = 0, len = rLink.length; i < len; i++)
                selCellObj.splice(rLink[i], 1);
            details.selCells = selCell;
            details.reqType = (!ej.isNullOrUndefined(status)) ? "clear-link" : "hyperlink";
            if (!this.model.sheets[sheetIdx]._importRendered && !this._isUndoRedo)
                (!this._isSort) && this._completeAction(details);

       },

        _removeHyperlink: function (status, cells, skipHiddenRow) {
            var sheetIdx = this.getActiveSheetIndex(), range = this.getSheet(sheetIdx).selectedRange;
            this.removeHyperlink(this._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]), status === "clear", status, cells, skipHiddenRow);
        },

        _openLink: function () {
            var sheetIdx = this.getActiveSheetIndex(), selected = this.getSheetElement(sheetIdx).find(".e-activecell");
            this._hlClick(this.XLEdit.getPropertyValue(selected.closest("tr").index(), selected[0].cellIndex, "hyperlink", sheetIdx));
        },

        _hlClick: function (data) {
            var range, sheetIdx, startCell, endCell, urlregex, actSheetIdx = this.getActiveSheetIndex(), alertText, alertMsg = this._getLocStr("HyperLinkAlert").split("-");
            this.model.enableContextMenu && this.XLCMenu.hideCMenu();
            if (data.cellAddr) {
                range = this.getRangeIndices(data.cellAddr), startCell = { rowIndex: range[0], colIndex: range[1] }, endCell = { rowIndex: range[2], colIndex: range[3] };
                sheetIdx = (data.cellAddr.split("_")[1] === 0) ? actSheetIdx : Number(data.cellAddr.split("_")[1]);
                this.gotoPage(sheetIdx, false);
                this.getSheet(actSheetIdx)._activeCell = startCell;
                this.XLSelection.selectRange(startCell, endCell, sheetIdx);
				 this.getSheet(actSheetIdx)._startCell = startCell;
				 this.getSheet(actSheetIdx)._endCell = endCell;
                this.XLScroll._scrollSelectedPosition(sheetIdx, startCell);
                this.model.allowAutoFill && this.XLDragFill.positionAutoFillElement();

            }
            else {
				 startCell = {rowIndex: this.getSheet(actSheetIdx)._activeCell.rowIndex, colIndex: this.getSheet(actSheetIdx)._activeCell.colIndex};
				 this.getSheet(actSheetIdx)._startCell = startCell;
				 this.getSheet(actSheetIdx)._endCell = startCell;
				 if (this._isValidUrl(data.webAddr))
				     window.open(data.webAddr, "_blank");
				 else
				     this._showAlertDlg("Alert", "HyperLinkAlert");
            }
        },

        _isValidUrl: function (url) {
            return /(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?/.test(url);
        },

        showWaitingPopUp: function () {
            $("#" + this._id).data("ejWaitingPopup").show();
        },

        hideWaitingPopUp: function () {
            $("#" + this._id).data("ejWaitingPopup").hide();
        },

        _getSelRange: function (selCells) {
            var sCell, eCell, sheetIdx = this.getActiveSheetIndex();
            sCell = selCells[0], eCell = selCells[selCells.length - 1];
            return (this._generateHeaderText(sheetIdx, sCell.colIndex + 1) + (sCell.rowIndex + 1) + ":" + this._generateHeaderText(sheetIdx, eCell.colIndex + 1) + (eCell.rowIndex + 1));
        },

        _getSelectedCells: function (sheetIdx, range) {
            var selectedCells, rng;
            if (ej.isNullOrUndefined(range)) {
                selectedCells = this.model.sheets[sheetIdx]._selectedCells;
				if(selectedCells.length > 0) {
					range = this._getSelRange(selectedCells);
					rng = this.getRangeIndices(range);
				}
            }
            else {
                rng = this.getRangeIndices(range);
                selectedCells = this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
            }
            this.model.sheets[sheetIdx]._selectedCells = selectedCells;
            return rng;
        },

        _getHiddenCells: function (sheetIdx, altCells, selectedCell, hideCells) {
            var container = this._dataContainer.sheets[sheetIdx], value;
            (ej.isNullOrUndefined(hideCells[selectedCell.rowIndex])) && (hideCells[selectedCell.rowIndex] = {});
            (ej.isNullOrUndefined(hideCells[selectedCell.rowIndex][selectedCell.colIndex])) && (hideCells[selectedCell.rowIndex][selectedCell.colIndex] = { value: {}, text: "" });
            if (!ej.isNullOrUndefined(container[selectedCell.rowIndex])) {
                hideCells[selectedCell.rowIndex][selectedCell.colIndex]["value"] = $.extend(true, {}, container[selectedCell.rowIndex][selectedCell.colIndex]);
                value = this.XLEdit.getPropertyValue(altCells.rowIndex, altCells.colIndex, "value2", sheetIdx)
				delete container[selectedCell.rowIndex][selectedCell.colIndex];				
                hideCells[selectedCell.rowIndex][selectedCell.colIndex]["text"] = (ej.isNullOrUndefined(value) ? "" : value); //Checking for empty td cell
            }
			if(this._isRowViewable(sheetIdx, altCells.rowIndex))
				altCells = this.getCell(altCells.rowIndex, altCells.colIndex, sheetIdx).empty();
            return hideCells;
        },

        _getRowSpan: function (sheetIdx, selectedCells) {
            var rows = this.getRows(sheetIdx)[0], rowSpan = 0;
            for (var i = selectedCells[0].rowIndex, len = selectedCells[selectedCells.length - 1].rowIndex; i <= len; i++) {
				if(ej.isNullOrUndefined(this.XLEdit.getPropertyValue(i, 0, "isRHide", sheetIdx)) && ej.isNullOrUndefined(this.XLEdit.getPropertyValue(i, 0, "isFHide", sheetIdx)) && ej.isNullOrUndefined(this.XLEdit.getPropertyValue(i, 0, "isFilterHide", sheetIdx)))
                    rowSpan = rowSpan + 1;
            }
            return rowSpan;
        },

		_getColSpan: function(sheetIdx, selectedCells) {
			var colSpan = 0, colgrp = this._getJSSheetHeader(sheetIdx).find("col");
			for (var i = selectedCells[0].colIndex, len = selectedCells[selectedCells.length - 1].colIndex; i <= len; i++) {
                if (!this._hasClass(colgrp[i], "e-fc-hide") && !this._hasClass(colgrp[i], "e-c-hide"))
                    colSpan = colSpan + 1;
            }
            return colSpan;
		},
		
        _getHiddenRowColumnCount: function (sheetIdx, range) {
            var rng = this.getRangeIndices(range), hr = 0, hc = 0, i, j, rows = this.getRows(sheetIdx)[0], cols = this._getContent(sheetIdx).find("col");
            for (i = rng[0]; i <= rng[2]; i++) {
				if(!ej.isNullOrUndefined(this.XLEdit.getPropertyValue(i, 0, "isRHide", sheetIdx)) || !ej.isNullOrUndefined(this.XLEdit.getPropertyValue(i, 0, "isFHide", sheetIdx)))
                    hr = hr + 1;
            }
            for (j = rng[1]; j <= rng[3]; j++) {
                if ($(cols[j]).hasClass("e-fc-hide") || $(cols[j]).hasClass("e-c-hide"))
                    hc = hc + 1;
            }
            return { hRow: hr, hCol: hc };
        },

        _updateMergeColl: function(sheetIdx, rowIdx, colIdx, type){
            var mergeColl = this.getSheet(sheetIdx)._mergeColl;
            if (ej.isNullOrUndefined(mergeColl[rowIdx]))
                mergeColl[rowIdx] = {};
            mergeColl[rowIdx][colIdx] = type;
        },

        mergeCells: function (range, alertStatus) {
            var i, mergeObj, rSpan = 1, cSpan = 1, len, cells = [], sheetIdx = this.getActiveSheetIndex(), alertdlg = $("#" + this._id + "_alertdlg"), selectedCells, rng, isMerged = false, actCell, hiddenCount;
			if(ej.isNullOrUndefined(range) && this.XLSelection.getSelectedCells(sheetIdx).length < 2)
				return;
			else if(this._getAlphaRangeCells(range).length < 2)
				return;
            this._selMergeCells = [];
            this._mergeType = "merge-cells";
            rng = this._getSelectedCells(sheetIdx, range);
			if(this.isUndefined(rng))
				return;
            selectedCells = (!ej.isNullOrUndefined(range)) ? this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] }) : this.model.sheets[sheetIdx]._selectedCells;
            if (!this.model.sheets[sheetIdx]._importRendered) {
                if (this._mergeCenter) {
                    isMerged = false;
                    for (i = 0, len = selectedCells.length; i < len; i++) {
                        mergeObj = this.XLEdit.getPropertyValue(selectedCells[i].rowIndex, selectedCells[i].colIndex, "merge", sheetIdx);
                        if (!ej.isNullOrUndefined(mergeObj)) {
                            rSpan = mergeObj.mSpan.rowSpan;
                            cSpan = mergeObj.mSpan.colSpan;
                            if (rSpan > 1 || cSpan > 1) {
                                hiddenCount = this._getHiddenRowColumnCount(sheetIdx, this.XLEdit.getPropertyValue(selectedCells[i].rowIndex, selectedCells[i].colIndex, 'merge').mRange);
                                cells = this._unMergeAllCells(sheetIdx, i, selectedCells, [], hiddenCount.hRow, hiddenCount.hCol);
                                if (!this._dupDetails && !this._isUndoRedo) {
                                    details = { sheetIndex: sheetIdx, cells: selectedCells, range: range, reqType: "merge-cells", action: "un-merge", operation: "un-merge", mCells: cells };
                                    this._completeAction(details);
                                }
                                isMerged = true;
                            }
                        }
                    }
                    this.performSelection(selectedCells[0], selectedCells[selectedCells.length - 1]);
                }
                else {
                    for (i = 0, len = selectedCells.length; i < len; i++) {
                        mergeObj = this.XLEdit.getPropertyValue(selectedCells[i].rowIndex, selectedCells[i].colIndex, "merge", sheetIdx);
                        if (!ej.isNullOrUndefined(mergeObj)) {
						    rSpan = mergeObj.mSpan.rowSpan;
						    cSpan = mergeObj.mSpan.colSpan;
						}
                        if (rSpan === 1 && cSpan === 1) {
                            isMerged = false;
                            break;
                        }
                    }
                }
            }
            if (!this.model.sheets[sheetIdx]._importRendered && !this._isUndoRedo && this._rangeHasData(rng) && !alertStatus && !isMerged && selectedCells.length > 1)
                this._showAlertDlg("", "MergeCellsAlert", "MergeCells", 498);
            else
                !isMerged && this._mergeCells(sheetIdx, selectedCells, {}, []);
			 this.XLRibbon._updateRibbonIcons();
        },

        _mergeCells: function (sheetIdx, selectedCells, hideCells, mCells) {
            var mSpan, container = this._dataContainer.sheets[sheetIdx], details, range, startrowIdx, startcolIdx, prevObj = {}, activeCell, altCells, cols = this._getJSSheetHeader(sheetIdx).find('col');
            startrowIdx = selectedCells[0].rowIndex; startcolIdx = selectedCells[0].colIndex;
			if((ej.isNullOrUndefined(this.XLEdit.getPropertyValue(startrowIdx, 0, "isFRHide")) || ej.isNullOrUndefined(this.XLEdit.getPropertyValue(startrowIdx, 0, "isRHide"))) && cols.eq(startcolIdx).hasClass("e-fc-hide"))
                return { hideCell: [], mCells: [] };
			mSpan = { rowSpan: this._getRowSpan(sheetIdx, selectedCells), colSpan: ((selectedCells[selectedCells.length - 1].colIndex - selectedCells[0].colIndex) + 1) };
			if (!ej.isNullOrUndefined(container[startrowIdx]) && !ej.isNullOrUndefined(container[startrowIdx][startcolIdx])) {
			    delete container[startrowIdx][startcolIdx]["isMHide"];
			    delete container[startrowIdx][startcolIdx]["mergeIdx"];
			}
			if(this._isRowViewable(sheetIdx, startrowIdx)) {
				activeCell = this.getCell(startrowIdx, startcolIdx);
				activeCell[0].colSpan = mSpan.colSpan;
				activeCell[0].rowSpan = mSpan.rowSpan;
				activeCell[0].className = activeCell[0].className.replace("e-mc-hide", "");
				this._mergeCenter && activeCell.addClass("e-calign ");
				if (activeCell.hasClass("e-commentcell"))
					this.XLComment._updateCmntFrmCntnr(null, null, activeCell);
			}			
            this._selMergeCells.push(selectedCells[0]);
            for (var i = 1, len = selectedCells.length; i < len; i++) {
    			if (this._isRowViewable(sheetIdx, selectedCells[i].rowIndex)) {
    			    altCells = this.getCell(selectedCells[i].rowIndex, selectedCells[i].colIndex);
    			    if (!this._hasClass(altCells[0], "e-mc-hide"))
				        this.addClass(altCells[0], "e-mc-hide");
				}
                hideCells = this._getHiddenCells(sheetIdx, {rowIndex: selectedCells[i].rowIndex, colIndex: selectedCells[i].colIndex}, selectedCells[i], hideCells);
                this.XLEdit._updateDataContainer({ rowIndex: selectedCells[i].rowIndex, colIndex: selectedCells[i].colIndex }, { isMHide: true, mergeIdx: { rowIndex: startrowIdx, colIndex: startcolIdx } }, sheetIdx);
                this._selMergeCells.push(selectedCells[i]);
            }            
            (!ej.isNullOrUndefined(container[startrowIdx])) && $.extend(true, prevObj, container[startrowIdx][startcolIdx]);
            range = this._getSelRange(selectedCells);
            this.model.sheets[sheetIdx]._activeCell = selectedCells[0];
            if (!this.model.sheets[sheetIdx]._importRendered) {
                this.performSelection(selectedCells[0], selectedCells[selectedCells.length - 1]);
                this.XLEdit._updateDataContainer({ rowIndex: startrowIdx, colIndex: startcolIdx }, { merge: { mSpan: mSpan, isMerge: true, mRange: range, isCenterAlign: this._mergeCenter } }); // mRange denones Merged Range
                this._updateMergeColl(sheetIdx, startrowIdx, startcolIdx, this._mergeCenter ? "center" : "merge");
            }
            mCells.push({ rowIndex: startrowIdx, colIndex: startcolIdx, prevObj: prevObj, curObj: $.extend(true, {}, container[startrowIdx][startcolIdx]) });
            if (!this.model.sheets[sheetIdx]._importRendered && !this._isUndoRedo && !this._dupDetails)
                if (!this._paste && !this._dupDetails)
                    if (this._mergeType === "merge-cells") {
                        details = { sheetIndex: sheetIdx, cells: [{ rowIndex: startrowIdx, colIndex: startcolIdx }], range: range, reqType: "merge-cells", action: "merge-all", hideCells: hideCells, mCells: mCells };
                        details.operation = (this._mergeCenter) ? "merge-center" : "merge-all";
                        (!this._paste) && this._completeAction(details);
                    }
            this._mergeCenter = false;
            this._paste = false;
            return { hideCells: hideCells, mCells: mCells };
        },

        mergeAcrossCells: function (range, alertStatus) {
            var sheetIdx = this.getActiveSheetIndex(), alertdlg = $("#" + this._id + "_alertdlg"), selectedCells, rng, alpRange;
		    if(ej.isNullOrUndefined(range) && this.XLSelection.getSelectedCells(sheetIdx).length < 2)
				return;
			else if(this._getAlphaRangeCells(range).length < 2)
				return;
            this._selMergeCells = [];
            rng = this._getSelectedCells(sheetIdx, range);
			if(this.isUndefined(rng))
				return;
            selectedCells = (!ej.isNullOrUndefined(range)) ? this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] }) : this.model.sheets[sheetIdx]._selectedCells;
            alpRange = this._getAlphaRange(sheetIdx, selectedCells[0].rowIndex, selectedCells[0].colIndex, selectedCells[selectedCells.length - 1].rowIndex,  selectedCells[selectedCells.length - 1].colIndex);
			if (!this._isUndoRedo && this._rangeHasData(rng) && !this.model.sheets[sheetIdx]._importRendered && !alertStatus && (selectedCells.length > 1 && (selectedCells[0].colIndex != selectedCells[selectedCells.length - 1].colIndex)))
                this._showAlertDlg("", "MergeCellsAlert", "MergeAcrossCells", 498);
            else if (selectedCells[0].colIndex != selectedCells[selectedCells.length - 1].colIndex)
                this._mergeAcrossCells(sheetIdx, selectedCells);
            this.performSelection(selectedCells[0], selectedCells[selectedCells.length - 1]);
			 this.XLRibbon._updateRibbonIcons();
        },

        _mergeAcrossCells: function (sheetIdx, selectedCells) {
            var i, j, len, mergeObj, rSpan = 1, cSpan = 1, isMerged = true, details, sCol, eCol, sRow, eRow, selCell = [], mValue = { mCells: [], hideCells: {} };
            this._mergeType = "merge-across";
            for (i = 0, len = selectedCells.length; i < len ; i++) {
                mergeObj = this.XLEdit.getPropertyValue(selectedCells[i].rowIndex, selectedCells[i].colIndex, "merge", sheetIdx);
                if (!ej.isNullOrUndefined(mergeObj)) {
                    rSpan = mergeObj.mSpan.rowSpan;
                    cSpan = mergeObj.mSpan.colSpan;
				}
                if (rSpan === 1 && cSpan === 1) {
                    isMerged = false;
                    break;
                }
                else if (rSpan > 1)
                    isMerged = false;
            }
            if (!isMerged) {
                sRow = selectedCells[0].rowIndex, eRow = selectedCells[selectedCells.length - 1].rowIndex;
                sCol = selectedCells[0].colIndex, eCol = selectedCells[selectedCells.length - 1].colIndex;
                for (i = sRow; i <= eRow; i++) {
                    selCell = [];
                    for (j = sCol; j <= eCol; j++)
                        selCell.push({ rowIndex: i, colIndex: j });
                    mValue = this._mergeCells(sheetIdx, selCell, mValue.hideCells, mValue.mCells);
                }
            }
            if (!this.model.sheets[sheetIdx]._importRendered && !this._isUndoRedo)
                if (!this._dupDetails) {
                    details = { sheetIndex: sheetIdx, cells: $.extend(true, [], this._selMergeCells), range: this._getSelRange(this._selMergeCells), reqType: "merge-cells", action: "merge-across", operation: "merge-across", hideCells: mValue.hideCells, mCells: mValue.mCells };
                    (!this._paste) && this._completeAction(details);
                    this._paste = false;
                }
            this.performSelection(selectedCells[0], selectedCells[selectedCells.length - 1]);
        },

        unmergeCells: function (range) {
            var sheetIdx = this.getActiveSheetIndex(), cells = [], selectedCells, details, rng, hiddenCount, mergeObj;
            rng = this._getSelectedCells(sheetIdx, range);
			if(this.isUndefined(rng))
				return;
            this._selMergeCells = [];
            selectedCells = (!ej.isNullOrUndefined(range)) ? this.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] }) : this.model.sheets[sheetIdx]._selectedCells;
            for (var k = 0, len = selectedCells.length; k < len; k++) {
                mergeObj = this.XLEdit.getPropertyValue(selectedCells[k].rowIndex, selectedCells[k].colIndex, "merge", sheetIdx);
                if (!this.isUndefined(mergeObj)) {
                    hiddenCount = this._getHiddenRowColumnCount(sheetIdx, mergeObj.mRange);
                    cells = this._unMergeAllCells(sheetIdx, k, selectedCells, cells, hiddenCount.hRow, hiddenCount.hCol);
                }
            }
            range = this._getSelRange(selectedCells);
            this.performSelection(selectedCells[0], selectedCells[selectedCells.length - 1]);
            if (!this.model.sheets[sheetIdx]._importRendered && !this._isUndoRedo)
                if (!this._dupDetails) {
                    details = { sheetIndex: sheetIdx, cells: selectedCells, range: range, reqType: "merge-cells", action: "un-merge", operation: "un-merge", mCells: cells };
                    this._completeAction(details);
                }
			this.XLRibbon._updateRibbonIcons();
			},

        _unMergeAllCells: function (sheetIdx, rowIdx, selectedCells, cells, hr, hc) {
            var i, j, hr, hc, mergeObj, rSpan = 1, cSpan = 1, mSpanObj, isViewable = false, container = this._dataContainer.sheets[sheetIdx], prevObj = {},
                activeCell, cell, startrowIdx, startcolIdx, mCells, mergeContainer; // mCells denotes merged cells
            startrowIdx = selectedCells[rowIdx].rowIndex; startcolIdx = selectedCells[rowIdx].colIndex, rowhdrrows = this.getRows(sheetIdx)[0];
            activeCell = this.getCell(startrowIdx, startcolIdx);
            mergeObj = this.XLEdit.getPropertyValue(startrowIdx, startcolIdx, "merge", sheetIdx);
            if (!ej.isNullOrUndefined(mergeObj)) {
			    rSpan = mergeObj.mSpan.rowSpan;
			    cSpan = mergeObj.mSpan.colSpan;
			}
			if (cSpan > 1 || rSpan > 1) {
			    mergeContainer = container[startrowIdx][startcolIdx];
			    $.extend(true, prevObj, mergeContainer);
			    delete mergeContainer["merge"];
			    delete this.getSheet(sheetIdx)._mergeColl[startrowIdx][startcolIdx];
			    if(this.XLFilter._isRowVisible(sheetIdx, startrowIdx)){
					activeCell = this.getCell(startrowIdx, startcolIdx);
					if (!ej.isNullOrUndefined(mergeObj.isCenterAlign))
						activeCell.removeClass("e-calign");
					activeCell[0].colSpan = activeCell[0].rowSpan = 1;
					if($(rowhdrrows[startrowIdx]).height() < activeCell[0].offsetHeight)
                        this._setRowHdrHeight(sheetIdx, startrowIdx);
				}
                cells.push({ rowIndex: startrowIdx, colIndex: startcolIdx, prevObj: prevObj, curObj: $.extend(true, {}, container[startrowIdx][startcolIdx]) });
                mCells = [];
                for (i = startrowIdx ; i < (startrowIdx + rSpan + hr) ; i++) {
                    for (j = startcolIdx ; j < (startcolIdx + cSpan + hc) ; j++) {
                        this._selMergeCells.push({ rowIndex: i, colIndex: j });
                        if (this._isRowViewable(sheetIdx, i)) {
                            cell = this.getCell(i, j, sheetIdx);
                            isViewable = true;
                        }
						if(!ej.isNullOrUndefined(this.XLEdit.getPropertyValue(i, j, "isMHide", sheetIdx))) {
							if (this._isUndoRedo) {
							    mSpanObj = this.XLEdit.getPropertyValue(i, j, "merge", sheetIdx);
							    if (!ej.isNullOrUndefined(mSpanObj)) {
							        if (mSpanObj.mergeSpan.rowSpan > 1 || mSpanObj.mergeSpan.colSpan > 1) {
							            mCells.push({ rowIndex: i, colIndex: j, rowSpan: mSpanObj.mergeSpan.rowSpan, colSpan: mSpanObj.mergeSpan.colSpan });
									    container[i][j].merge.mSpan = { rowSpan: 1, colSpan: 1 };						
									    cell[0].rowSpan = 1; cell[0].colSpan = 1;
									}
								}
							}
							mergeIdx = this.XLEdit.getPropertyValue(i, j, "mergeIdx", sheetIdx);
							if(!ej.isNullOrUndefined(mergeIdx))
							    if (mergeIdx.rowIndex === startrowIdx && mergeIdx.colIndex === startcolIdx) {
							        isViewable && cell.removeClass("e-mc-hide");
							        delete container[i][j]["isMHide"];
							        delete container[i][j]["mergeIdx"];
							    }
						}
                        if (this._isUndoRedo && !ej.isNullOrUndefined(this._delCells)) {
                            if (!ej.isNullOrUndefined(this._delCells[i]) && !ej.isNullOrUndefined(this._delCells[i][j])) {
                                if (!ej.isNullOrUndefined(container[i])) {
                                    container[i][j] = this._delCells[i][j]["value"];
                                    isViewable && cell.text(this._delCells[i][j]["text"]);
                                    mergeObj = this.XLEdit.getPropertyValue(i, j, "merge");
                                    if (!ej.isNullOrUndefined(mergeObj) && mergeObj.isMerge)
                                        this.mergeCells(mergeObj.mRange, true);
                                }
                            }
                        }
                        if (isViewable && cell.hasClass("e-commentcell"))
                            this.model.allowComments && this.XLComment._updateCmntFrmCntnr(null, null, cell);
                    }
                }
            }
            return cells;
        },

        showHeaders: function (status) {
            var sheetIdx = this.getActiveSheetIndex(), settings = {};
            this.model.sheets[sheetIdx].showHeader = status;
            this._isDisplayHeader = true;
            $.extend(true, settings, this.model.scrollSettings);
            if (this.model.sheets[sheetIdx].showHeader) {
                this._getJSSheetRowHeader(sheetIdx).show();
                this._getJSSheetHeader(sheetIdx).show();
				this._getContent(sheetIdx).find('.e-content').removeClass('e-viewbrdr');
            }
            else {
                this._getJSSheetRowHeader(sheetIdx).hide();
                this._getJSSheetHeader(sheetIdx).hide();
				this._getContent(sheetIdx).find('.e-content').addClass('e-viewbrdr');
            }
            this.XLRibbon._heightWidthCalculation(sheetIdx, settings, {action: ""});
        },

        showGridlines: function (status) {
            var sheetIdx = this.getActiveSheetIndex();
            this.model.sheets[sheetIdx].showGridlines = status;
            this.model.sheets[sheetIdx].showGridlines ? this._getContent(sheetIdx).find(".e-rowcell").removeClass('e-hborder') : this._getContent(sheetIdx).find(".e-rowcell").addClass("e-hborder");
        },

        //Spreadsheet Common Prop
        setSheetFocus: function () {
            $("#" + this._id).focus();
        },

        _diffNumbers: function (a, b) {
            return a - b;
        },

         _isValidDate: function (date) {
            var dateFormat = /^\d{1,4}[\.|\/|-]\d{1,2}[\.|\/|-]\d{1,4}$/;
            if (dateFormat.test(date)) {
                date = date.replace(/0*(\d*)/gi, "$1");
                var dateArray = date.split(/[\.|\/|-]/);
                // correct month value
                dateArray[1] = dateArray[1] - 1;
                // correct year value
                if (dateArray[2].length < 4) {
                    // correct year value
                    dateArray[2] = (parseInt(dateArray[2]) < 50) ? 2000 + parseInt(dateArray[2]) : 1900 + parseInt(dateArray[2]);
                }
                var testDate = new Date(dateArray[2], dateArray[1], dateArray[0]);
                if (testDate.getDate() != dateArray[0] || testDate.getMonth() != dateArray[1] || testDate.getFullYear() != dateArray[2]) {
                    return false;
                } else {
                    return true;
                }
            } else {
                return false;
            }
        },

        _isValidTime: function (value) {
            var colonCount = 0, status = false, mm, ss;
            for (var i = 0, len = value.length; i < len; i++) {
                var ch = value.substring(i, i + 1);
                if ((ch < "0") || (ch > "9")) {
                    if ((ch != ":") && (ch != " ") && (ch != "a") && (ch != "A") && (ch != "p") && (ch != "P") && (ch != "m") && (ch != "M")) {
                        return false;
                    }
                }
                if (ch == ":") { colonCount++; }
                if ((ch == "p") || (ch == "P") || (ch == "a") || (ch == "A")) { status = true; }
            }
            if ((colonCount < 1) || (colonCount > 2)) { return false; }
            var hh = value.substring(0, value.indexOf(":"));
            if ((parseFloat(hh) < 0) || (parseFloat(hh) > 23)) { return false; }
            if (status) {
                if ((parseFloat(hh) < 1) || (parseFloat(hh) > 12)) { return false; }
            }
            if (colonCount == 2) {
                mm = value.substring(value.indexOf(":") + 1, value.lastIndexOf(":"));
            } else {
                mm = value.substring(value.indexOf(":") + 1, value.length);
            }
            if ((parseFloat(mm) < 0) || (parseFloat(mm) > 59)) { return false; }
            if (colonCount == 2) {
                ss = value.substring(value.lastIndexOf(":") + 1, value.length);
            } else {
                ss = "00";
            }
            if ((parseFloat(ss) < 0) || (parseFloat(ss) > 59)) { return false; }
            return true;
        },

        _setXY: function (e) {
            var x = 0, y = 0;
            if (e.type === "mousemove" || e.type === "mousedown") {
                x = e.clientX;
                y = e.clientY;
            }
            else if (e.type === "touchmove") {
                x = e.originalEvent.changedTouches[0].clientX;
                y = e.originalEvent.changedTouches[0].clientY;
            }
            else if (e.type === "MSPointerMove") {
                x = e.originalEvent.clientX;
                y = e.originalEvent.clientY;
            }
            return [x, y];
        },

        _getSelectedItems: function (sheetIdx, range, addr) {
            var selected, sCIdx, eCIdx, actSheet;
            sheetIdx = sheetIdx ? sheetIdx : this.getActiveSheetIndex(), actSheet = this.getSheet(sheetIdx);
            if (ej.isNullOrUndefined(addr) && ej.isNullOrUndefined(range)) {
                selected = actSheet._selectedCells;
                sCIdx = actSheet._startCell, eCIdx = actSheet._endCell;
                addr = this._generateHeaderText(sheetIdx, sCIdx.colIndex + 1) + (sCIdx.rowIndex + 1) + ":" + this._generateHeaderText(sheetIdx, eCIdx.colIndex + 1) + (eCIdx.rowIndex + 1);
            }
            else {
                range = this.isUndefined(range) ? this.getRangeIndices(addr) : range;
                selected = this.getRange(range[0], range[1], range[2], range[3], sheetIdx);
                addr = this._generateHeaderText(sheetIdx, range[1] + 1) + (range[0] + 1) + ":" + this._generateHeaderText(sheetIdx, range[3] + 1) + (range[2] + 1);
            }
            return [selected, addr];
        },
        setBorder: function (property, range) {
            var str = "", borderStyle = property.style, borderType = property.ID, hexcode = property.hexaCode, arr1, arr2, arr3, border1, border2, prop, sprop, sheetCont = this._getContent(this.getActiveSheetIndex()).find("div:first-child");
            arr1 = ["bottomborder", "topborder", "leftborder", "rightborder", "outsideborder"];
            arr2 = ["thickboxborder", "thickbottomborder", "topandbottomborder", "topandthickbottomborder"];
            arr3 = ["allborder"];
            border1 = {
                topborder: { top: "" },
                rightborder: { right: "" },
                bottomborder: { bottom: "" },
                leftborder: { left: "" },
                outsideborder: { top: "", right: "", bottom: "", left: "" },
                thickboxborder: { top: "2px solid", right: "2px solid", bottom: "2px solid", left: "2px solid" },
                thickbottomborder: { bottom: "2px solid" },
                topandbottomborder: { top: "1px solid", bottom: "1px solid" },
                topandthickbottomborder: { top: "1px solid", bottom: "2px solid" }
            };
            border2 = { solid: "1px solid", dashed: "1px dashed", dotted: "1px dotted" };
            this._isGridBordering = this._isOutsideBordering = false;
            switch (true) {
                case arr1.indexOf(borderType) > -1:
                    prop = border1[borderType];
                    for (i in prop) {
                        str = "";
                        str = str + border2[borderStyle] + " " + hexcode;
                        prop[i] = str;
                    }
                    this.XLFormat.applyBorder(prop, range);
                    break;
                case arr2.indexOf(borderType) > -1:
                    prop = border1[borderType];
                    for (i in prop) {
                        str = "";
                        sprop = prop[i].split(" ");
                        str = sprop.length === 2 ? str + prop[i] + " " + hexcode : str + prop[i] + " " + borderStyle + " " + hexcode; // consider
                        prop[i] = str;
                    }
                    this.XLFormat.applyBorder(prop, range);
                    break;
                case arr3.indexOf(borderType) > -1:
                    str = str + border2[borderStyle] + " " + hexcode;
                    this.XLFormat.applyBorder(str, range);
                    break;
                case borderType === "noborder":
                    this.XLFormat.applyBorder("noborder");
                    break;
                case borderType === "drawborder":
                    this.XLSelection._isOutsideBordering = true;
                    sheetCont.removeClass("e-ss-cursor");
                    sheetCont.addClass("e-ss-drwbrdrcursor");
                    break;
                case borderType === "drawbordergrid":
                    this.XLSelection._isGridBordering = true;
                    sheetCont.removeClass("e-ss-cursor");
                    sheetCont.addClass("e-ss-drwbrdrgridcursor");
                    break;
            }
        },

        clearBorder: function (range) {
            range = this.getRangeIndices(range);
            this.XLFormat.clearFormat(range, "border");
        },

        clearAllFormat: function (aRange) {
			var activeElem = this.getActiveCellElem();
			if(!activeElem.length)
				return;
            var sheetIdx = this.getActiveSheetIndex(), range = this.isUndefined(aRange) ? this.getSheet(sheetIdx).selectedRange : this.getRangeIndices(aRange), nRange = [];
            var details = { sheetIndex: sheetIdx, reqType: "clear-format", range: range, bFormat: this.model.allowCellFormatting && this.XLFormat.getHashCodeClassAsArray(range), wRange: [], hRange: [], cFormatRule: [] };
            var rKeys, cKeys, cell, rules;
            this._dupDetails = true;
            this.model.allowCellFormatting && this.XLFormat.format({ type: "general" });
            rKeys = this.getObjectKeys(this._dataContainer.sheets[sheetIdx]);
            nRange[0] = (range[0] > parseInt(rKeys[0])) ? range[0] : parseInt(rKeys[0]);
            nRange[2] = (range[2] < parseInt(rKeys[rKeys.length - 1])) ? range[2] : parseInt(rKeys[rKeys.length - 1]);
            for (var i = nRange[0]; i < nRange[2] + 1; i++) {
                cKeys = this.getObjectKeys(this._dataContainer.sheets[sheetIdx][i]);
                nRange[1] = (range[1] > parseInt(cKeys[0])) ? range[1] : parseInt(cKeys[0]);
                nRange[3] = (range[3] < parseInt(cKeys[cKeys.length - 1])) ? range[3] : parseInt(cKeys[cKeys.length - 1]);
                for (var j = nRange[1]; j < nRange[3] + 1; j++) {
                    cell = this.getCell(i, j);
                    if (this._isCellProtected(i, j, true))
                        return;
                    cell[0].className.indexOf("e-sswraptext") > -1 && details.wRange.push({ rowIndex: i, colIndex: j });
                    rules = !ej.isNullOrUndefined(this.XLEdit.getPropertyValueByElem(cell, 'cFormatRule')) ? this.model.allowConditionalFormats && this.XLCFormat.getCFRule(i, j)[this.XLCFormat.getCFRule(i, j).length - 1] : "";
                    details.cFormatRule.push({ rowIndex: i, colIndex: j, rule: rules });
                    if (cell.find("a").hasClass("e-hyperlinks")) {
                        details.hRange.push({ rowIndex: i, colIndex: j, rowIndex: i, colIndex: j });
                        cell.find("a").css({ "color": "#444445", "text-decoration": "none" }).removeClass("e-hyperlinks");
                    }
                    if (cell.hasClass("e-commentcell"))
                        cell.removeClass("e-commentcell") && this.XLComment.setComment([i, j, i, j], null, false);
                }
            }
            if ((ej.isNullOrUndefined(rKeys)) || (ej.isNullOrUndefined(cKeys)))
                return;
            if (!this._checkTableRange(this._getAlphaRange(sheetIdx, nRange[0], nRange[1], nRange[2], nRange[3])))
                return;
            this.model.allowCellFormatting && this.XLFormat.clearFormat(range);
            this.model.allowConditionalFormats && this.XLCFormat.clearCF(true);
            this.setWrapText(aRange, "unwrap");
            this.XLRibbon._updateRibbonIcons();
            this._dupDetails = false;
            (!this._isUndoRedo) && this._completeAction(details);
        },

        clearContents: function (range) {
            var activeElem = this.getActiveCellElem();
			if(!activeElem.length)
                return;
            range = this.isUndefined(range) ? this.getSheet(sheetIdx).selectedRange : this.getRangeIndices(range);
            var sheetIdx = this.getActiveSheetIndex(), rangeData = this.getRangeData(range), details = { sheetIndex: sheetIdx, reqType: "clear-content", range: range, bData: rangeData }, hlData;
            for (var i = range[0]; i < range[2] + 1; i++) {
                for (var j = range[1]; j < range[3] + 1; j++) {
                    if (this._isCellProtected(i, j, true))
                        return;
                    hlData = this.XLEdit.getPropertyValue(i, j, "hyperlink", sheetIdx);
                    if (!this.isUndefined(hlData))
                        this.model.allowCellFormatting && this.XLFormat.format({ "style": { "text-decoration": "underline" } }, this._getAlphaRange(sheetIdx, i, j, i, j));
                    this.model.allowComments && this.XLComment._updateCmntFrmCntnr(i, j);
                }
            }
            this.clearRangeData(range, ["value", "value2", "hyperlink"]);
            this._completeAction(details);
            this.setSheetFocus();
        },

        clearAll: function (aRange) {
            var activeElem = this.getActiveCellElem();
			if(!activeElem.length)
                return;
            var sheetIdx = this.getActiveSheetIndex(), tabRegx = new RegExp('\\b' + "e-table" + '.*?\\b', 'g'), nRange = [], range = this.isUndefined(aRange) ? this.getSheet(sheetIdx).selectedRange : this.getRangeIndices(aRange), rangeData = this.getRangeData(range), details = { sheetIndex: sheetIdx, reqType: "clear-all", range: range, bFormat: this.model.allowCellFormatting && this.XLFormat.getHashCodeClassAsArray(range), bData: rangeData, selCells: [] }, container = this._dataContainer.sheets[sheetIdx],
            rKeys, cKeys, cell, rules, tab = this.model.sheets[sheetIdx].filterSettings.tableRange;
            this._dupDetails = true;
            rKeys = this.getObjectKeys(this._dataContainer.sheets[sheetIdx]);
            nRange[0] = (range[0] > parseInt(rKeys[0])) ? range[0] : parseInt(rKeys[0]);
            nRange[2] = (range[2] < parseInt(rKeys[rKeys.length - 1])) ? range[2] : parseInt(rKeys[rKeys.length - 1]);
            for (var i = nRange[0]; i < nRange[2] + 1; i++) {
                cKeys = this.getObjectKeys(this._dataContainer.sheets[sheetIdx][i]);
                nRange[1] = (range[1] > parseInt(cKeys[0])) ? range[1] : parseInt(cKeys[0]);
                nRange[3] = (range[3] < parseInt(cKeys[cKeys.length - 1])) ? range[3] : parseInt(cKeys[cKeys.length - 1]);
                for (var j = nRange[1]; j < nRange[3] + 1; j++) {
                    cell = this.getCell(i, j);
                    if (this._isCellProtected(i, j, true))
                        return;
                    rules = !ej.isNullOrUndefined(this.XLEdit.getPropertyValueByElem(cell, 'cFormatRule')) ? this.model.allowConditionalFormats && this.XLCFormat.getCFRule(i, j)[this.XLCFormat.getCFRule(i, j).length - 1] : "";
                    (!ej.isNullOrUndefined(container[i][j])) && details.selCells.push({ rowIndex: i, colIndex: j, prevLink: this.XLEdit.getPropertyValueByElem(cell, "hyperlink"), curLink: {}, prevText: cell.text(), prevComment: this.XLEdit.getPropertyValueByElem(cell, "comment"), mRange: !ej.isNullOrUndefined(container[i][j].mRange) ? container[i][j].mRange : "", mergeCenter: !ej.isNullOrUndefined(container[i][j].mRange) && container[i][j].isCenterAlign, wrap: !ej.isNullOrUndefined(container[i][j].wrap) ? container[i][j].wrap : "", cFormatRule: rules });
                }
            }
            if ((ej.isNullOrUndefined(rKeys)) || (ej.isNullOrUndefined(cKeys)))
                return;
            if (!this._checkTableRange(this._getAlphaRange(sheetIdx, nRange[0], nRange[1], nRange[2], nRange[3])))
                return;
            activeElem[0].className.indexOf("e-table") > -1 && (details.tabId = parseInt(activeElem[0].className.match(tabRegx)[0].replace("e-table", "")));
            if (details.tabId > -1) {
                details.tableObj = this.model.sheets[sheetIdx]._tableManager[details.tabId];
                delete this.model.sheets[sheetIdx]._tableManager[details.tabId];
                for (i = 0, len = tab.length; i < len; i++) {
                    if (tab[i].tableID === details.tabId) {
                        details.filterObj = tab[i];
                        tab.splice(i, 1);
                        break;
                    }
                }
                this.element.find(".e-selected").removeClass("e-filterhdr e-table" + details.tabId);
                this.XLRibbon._toggleDesignTab();
            }
            this.model.allowComments && this.XLComment.deleteComment(range);
            this.clearRangeData(range);
            this.model.allowCellFormatting && this.XLFormat.clearFormat(range);
            this.model.allowConditionalFormats && this.XLCFormat.clearCF(true);
            this.unmergeCells(aRange);
            this.setWrapText(aRange, "unwrap");
            this._dupDetails = false;
            this.XLRibbon._updateRibbonIcons();
            this._completeAction(details);
        },

        _checkTableRange: function (aRange) {
            var tAlphaRange, sheetIdx = this.getActiveSheetIndex(), tabRegx = new RegExp('\\b' + "e-table" + '.*?\\b', 'g'), tId, range = this.getRangeIndices(aRange);
            (this.getCell(range[0], range[1])[0].className.indexOf("e-table") > -1) && (tId = parseInt(this.getCell(range[0], range[1])[0].className.match(tabRegx)[0].replace("e-table", "")));
            if (ej.isNullOrUndefined(tId))
                return true;
            else {
                tRange = this.model.sheets[sheetIdx]._tableManager[tId].range;
                tAlphaRange = this._getAlphaRange(sheetIdx, tRange[0], tRange[1], tRange[2], tRange[3])
                if (aRange === tAlphaRange)
                    return true;
            }
        },

        clearTextNode: function (td, text) {
            var sheetIdx = this.getActiveSheetIndex(), rowIndex = td[0].parentNode.rowIndex, colIndex = td[0].cellIndex, container = this._dataContainer.sheets[sheetIdx], node = td[0].childNodes[td[0].childNodes.length - 1];
            text = text || "";
            if (this._checkIndicesInContainer(sheetIdx, rowIndex, colIndex, "rule"))
                if (container[rowIndex][colIndex].rule.type === "list")
                    return;
            if (ej.isNullOrUndefined(node) || node.hasChildNodes("a"))
                return;
            else
                td[0].lastChild.textContent = text;
        },
		
		_changeTargetWithOffset: function (e) {                       //get a cell by position of mouse pointer
            var trgt, offset = e.offsetX;
            if ($(e.target).is('a')) {
                trgt = e.target;
                e.target = e.target.parentNode;
            }
            if (offset > e.target.offsetWidth && $(e.target).is('td')) {
                var sheet = this.getSheet(1),
                 left = sheet._colWidthCollection[e.target.cellIndex].value - 1 + offset,
                 top = sheet._rowHeightCollection[e.target.parentNode.rowIndex].value;
                 trgt && this.addClass(trgt, 'e-rmanchor');
                 return this.getCell(this.XLShape._getCellIndexFromOffset(top, left).rowIndex, this.XLShape._getCellIndexFromOffset(top, left).colIndex)[0];
            }
            else {
                if (trgt) {
                    this._removeClass(trgt, 'e-rmanchor');
                    return trgt;
                }
                else {
                    this._removeClass($(e.target).find('a')[0], 'e-rmanchor');
                    return e.target;
                }
            }
        },
		
		_textClip: function (rowIndex, colIndex, action, width) {
            var endCell, chngd = false, val, isText = false, sheetIndex = this.getActiveSheetIndex(), tempCell, rowsColl = this.model.sheets[sheetIndex]._rowHeightCollection, colsColl = this.model.sheets[sheetIndex]._colWidthCollection, container = this._dataContainer.sheets[sheetIndex], xlEdit = this.XLEdit, cell = this.getCell(rowIndex, colIndex), overflow = xlEdit.getPropertyValue(rowIndex, colIndex, 'overflow');
			if(!this._dupDetails) {
				this._dupDetails = true;
				chngd = true;
			}
            if (action === 'add') {
                (!this._hasClass(cell[0], 'e-overflow')) && this.addClass(cell[0], 'e-overflow');
				endCell = this.XLShape._getCellIndexFromOffset(rowsColl[rowIndex].value - 1, (colsColl[colIndex].value - 1 + (width ? width : ((cell[0].scrollWidth === cell.outerWidth()) ? (cell[0].scrollWidth -1) : cell[0].scrollWidth))));
                val = endCell.colIndex - colIndex;
                if (!ej.isNullOrUndefined(overflow)) {
                    tempCell = this.getCell(overflow.rowIndex, overflow.colIndex);
                    this._removeClass(tempCell[0], 'e-overflow')
                    for (var j = 0, len = container[rowIndex][colIndex]['overflow']['cellCount']; j <= len; j++) {
                        (j !== 0) && this.clearRangeData([rowIndex, overflow.colIndex + j, rowIndex, overflow.colIndex + j], ['overflow']);
                        (j !== len) && this._removeClass(this.getCell(rowIndex, overflow.colIndex + j)[0],'e-ofbrdr');
                    }
                }
                for (var j = 1; j <= val; j++) {
                    if (xlEdit.getPropertyValue(rowIndex, colIndex + j, 'value2')) {
                        isText = true;
                        break;
                    }
                }
                if (!isText){
                    if(val!==0){
						for (var j = 0; j <= val; j++){
						(j !== 0) && xlEdit._updateDataContainer({rowIndex : rowIndex, colIndex: colIndex + j}, { overflow: { rowIndex: rowIndex, colIndex: colIndex, cellCount: val } });
							(j !== val) && this.addClass(this.getCell(rowIndex, colIndex + j)[0],'e-ofbrdr');
						}
					}
				}
                else
                    this._removeClass(cell[0], 'e-overflow');
            }
            else if (action === 'delete') {
                keys = this.getObjectKeys(container[rowIndex]);
                !this._hasClass(cell[0], 'e-overflow') && this.addClass(cell[0], 'e-overflow');
                if (!ej.isNullOrUndefined(xlEdit.getPropertyValue(rowIndex, colIndex + 1, 'overflow')))
                    for (var i = colIndex, len = colIndex + 1 + container[rowIndex][colIndex + 1]['overflow']['cellCount']; i < len; i++){
                        (i !== colIndex) && this.clearRangeData([rowIndex, i, rowIndex, i], ['overflow']);
                        (i !== len) && this._removeClass(this.getCell(rowIndex, i)[0],'e-ofbrdr');
                        }
                for (var i = keys.length - 1; i > -1; i--) {
                    if (keys[i] < colIndex) {
                        parseCell = this.getCell(rowIndex, keys[i]);
                        if (xlEdit.getPropertyValue(rowIndex, keys[i], 'value2')) {
                            if(!this._hasClass(parseCell[0], 'e-overflow')){
                            this.addClass(parseCell[0], 'e-overflow');
                            this._textClip(rowIndex, parseInt(keys[i]), 'add');
                            }
                            break;
                        }
                    }
                }
            }
			chngd && (this._dupDetails = false);
        },

        sheetRename: function (sheetName) {
            this._updateSheetNames(sheetName);
        },

        _isSubMenuOpen: function () {
            var arr = ["_formatastable", "_cellstyles", "_Ribbon_CElement", "_contextMenuCell", "_contextMenuColumnHeader", "_contextMenuRowHeader", "_contextMenuFooter"], i = 0, len = arr.length;
            while (i < len) {
                if ($("#" + this._id + arr[i]).is(":visible"))
                    return true;
                i++;
            }
           return false;
        },

        copySheet: function (fromIndex, toIndex, isCopySheet) {
            var sheetIdx = this._getSheetIndex(sheetIdx);
            var spliceIndex, i, len, cpySheetName, isValid = false, sheetNameCount = 2, index, selectedItem = this.element.find("#NumericContainer").children().eq(fromIndex - 1), mainIndex = toIndex, modelSheet = this.model.sheets;
            //Check is same sheet
            if (!isCopySheet && (modelSheet[mainIndex]._sheetName.text === selectedItem.text())) {
                this.setSheetFocus();
                return;
            }

            //Getting splice index and sheet position index
            for (i = 1, len = modelSheet.length; i < len; i++) {
                (modelSheet.length - 1 !== mainIndex) && (modelSheet[mainIndex]._sheetName.text === modelSheet[i]._sheetName.text) && (spliceIndex = i);
                (selectedItem.text() === modelSheet[i]._sheetName.text) && (index = i);

            }

            //Check is Copy sheet
            if (isCopySheet) {
                // Getting unique sheet name
                if (selectedItem.text().match('.*[ ]+[(]+[0-9]+[)]$') === null) {
                    cpySheetName = selectedItem.text() + " (" + sheetNameCount + ")";
                }
                else {
                    if (selectedItem.text().lastIndexOf(' ') > -1) {
                        sheetNameCount = parseInt(selectedItem.text().substring(selectedItem.text().lastIndexOf("(") + 1, selectedItem.text().lastIndexOf(")")));
                        cpySheetName = selectedItem.text().substring(0, selectedItem.text().lastIndexOf(" ")) + " (" + sheetNameCount + ")";
                    }
                }
                while (!isValid) {
                    cpySheetName = cpySheetName.substring(0, cpySheetName.lastIndexOf(" ")) + " (" + sheetNameCount + ")";
                    for (i = 1, len = modelSheet.length; i < len; i++) {
                        (cpySheetName === modelSheet[i]._sheetName.text) && sheetNameCount++;
                    }
                    (sheetNameCount === parseInt(cpySheetName.substring(cpySheetName.lastIndexOf("(") + 1, cpySheetName.lastIndexOf(")")))) && (isValid = true);
                }

                //Copy Sheet Process      
                this._copySheetProcess(index, cpySheetName);

                index = this.model.sheetCount;
                ej.isNullOrUndefined(spliceIndex) && (spliceIndex = index);
            }
            else {
                !ej.isNullOrUndefined(spliceIndex) && spliceIndex > index && (spliceIndex = spliceIndex - 1);
            }

            ej.isNullOrUndefined(spliceIndex) && (spliceIndex = mainIndex);

            //Sheets[] updates
            if (!isCopySheet && spliceIndex === index) {
                this.setSheetFocus();
            }
            else {
                //Update DataContainer positions
                this._swapSheetDtCntr(index, spliceIndex);
                this.getPager().ejPager("option", { currentPage: mainIndex });
                this.gotoPage(mainIndex, false);
                this.setSheetFocus();
            }
        },

        getHyperlink: function (cell) {
            return this.XLEdit.getPropertyValueByElem(cell, "hyperlink", this.getActiveSheetIndex());
        },

        getComment: function (cell) {
            return this.XLEdit.getPropertyValueByElem(cell, "comment", this.getActiveSheetIndex());
        },

        _isvalidRange:function(alphaRange)
        {
            var regx = new RegExp(/^\$?(?:\b[a-z]{1,3})\$?(?:\d{1,7}):\$?(?:\b[a-z]{1,3})\$?(?:\d{1,7})$/i);
            return regx.test(alphaRange);
        },

        // upload
        _renderFUpload: function () {
            this.element.append(ej.buildTag("div", "", "", { id: this._id + "_file", style: "display:none" }));
            $("#" + this._id + "_file").ejUploadbox({
                uploadName: "file",
                autoUpload: false,
                showFileDetails: false,
                dialogAction: { content: this.element },
                saveUrl: this.model["importSettings"].importMapper,
                fileSelect: $.proxy(this._fileSelect),
                complete: $.proxy(this._changeImport),                
                error: $.proxy(this._importFailed)
            });
            $("#" + this._id + "_file_SelectButton").hide(); //?
        },

        _fileSelect: function (args) {
            var xlObj = $("#" + $("#" + this._id).parents("div")[0].id).data("ejSpreadsheet"), file = args.files[0], extension = file.extension, formData;
            if ((xlObj._uploadImage && extension === ".xlsx") || (!xlObj._uploadImage && (!(xlObj._browserDetails.name === "msie" && parseInt(xlObj._browserDetails.version) < 10) ? (file.rawFile.type.indexOf("image") > -1) : (extension.indexOf("jpg") > -1 || extension.indexOf("png") > -1 || extension.indexOf("jpeg") > -1 || extension.indexOf("gif") > -1)))) {
                xlObj._showAlertDlg("Alert", "CorrectFormat", "", 290);
                return false;
            }
			formData = new FormData();
		    formData.append("Password","");
		    formData.append("file", file.rawFile);
		    xlObj["import"](formData);
            xlObj._uploadImage = false;
            //xlObj.showWaitingPopUp();
        },

        _changeImport: function (args) {
            var response, xlObj = $("#" + $("#" + this._id).parents("div")[0].id).data("ejSpreadsheet");
            if (!(xlObj._browserDetails.name === "msie" && parseInt(xlObj._browserDetails.version) < 10) ? (args.files.rawFile.type.indexOf("image") > -1) : (args.files[0].extension.indexOf("jpg") > -1 || args.files[0].extension.indexOf("png") > -1 || args.files[0].extension.indexOf("jpeg") > -1 || args.files[0].extension.indexOf("gif") > -1)) {
                this.updialog.ejDialog("close");
                xlObj.XLShape._insertPicture(args);
                xlObj.XLRibbon._formatTabUpdate();
            }
            else {
                response = xlObj._browserDetails.name === "msie" && parseInt(xlObj._browserDetails.version) < 10 ? args.responseText : args.xhr.response;
                this.updialog.ejDialog("close");
                xlObj.importSheet = true;
                xlObj._importSuccess(response);
                xlObj.importSheet = false;
            }
            xlObj.hideWaitingPopUp();
        },

        _importFailed: function (response) {
            var args = { failureType: response.type };
            if (response.xhr) {
                args.status = response.xhr.status;
                args.statusText = response.xhr.statusText;
            }
            else
                args.statusText = response.error;
            this.hideWaitingPopUp();
            this._trigger("openFailure", args);
        },
		_dlgBeforeOpen: function (args) {
            var xlObj = this.XLObj, dlgObj = $("#" + xlObj._id + "_PasswordDialog").data("ejDialog");
            xlObj.hideWaitingPopUp();
            $("#" + xlObj._id + "_workBookName").text("'" + $("#" + xlObj._id + "_file").data("ejUploadbox")._files[0].name + "'" + xlObj._getLocStr("Protect"));
        },
        _dlgOk: function (args) {
            var xlObj = this, tgleObj, dlgObj = $("#" + xlObj._id + "_PasswordDialog").data("ejDialog"), pwdEle = $("#" + xlObj._id + "_xlpassword"), cPwdEle = $("#" + xlObj._id + "_confirmpassword");
            tgleObj = $("#" + xlObj._id + "_Ribbon_protectWorkbook").data("ejToggleButton");
            if (dlgObj.model.title === "Protect Workbook") {
                if (pwdEle.val().length && cPwdEle.val().length) {
                    if (pwdEle.val() === cPwdEle.val()) {
                        xlObj.model.exportSettings.password = pwdEle.val();
                        $("#" + xlObj._id + "_confirm").hide();
                        dlgObj.option("title", xlObj._getLocStr("UnProtectWorkbook"));
                    }
                    else {
                        $("#" + xlObj._id + "_confirm").show();
                        xlObj._showAlertDlg("Alert", "PasswordAlert1");
                        tgleObj.option("toggleState", false);
                        dlgObj.option("title", xlObj._getLocStr("ProtectWorkbook"));
                    }
                }
                else {
                    $("#" + xlObj._id + "_confirm").show();
                    tgleObj.option("toggleState", false);
                    dlgObj.option("title", xlObj._getLocStr("ProtectWorkbook"));
                    xlObj._showAlertDlg("Alert", "PasswordAlert2");
                }
            }

            else if (dlgObj.model.title === "Unprotect Workbook") {
                if (xlObj.model.exportSettings.password === pwdEle.val()) {
                    xlObj.model.exportSettings.password = null;
                    $("#" + xlObj._id + "_confirm").show();
                    dlgObj.option("title", xlObj._getLocStr("ProtectWorkbook"));
                }
                else {
                    dlgObj.option("title", xlObj._getLocStr("UnProtectWorkbook"));
                    tgleObj.option("toggleState", true);
                    xlObj._showAlertDlg("Alert", "PasswordAlert3");
                }
            }
            pwdEle.val("");
            cPwdEle.val("");
            $("#" + xlObj._id + "_PasswordDialog").ejDialog("close");

        },
        _passwordDlg: function () {
            var xlObj = this, $dlg, $label, $okBtn, $canBtn, $btndiv, $div;
            $dlg = ej.buildTag("div", "", "", { id: xlObj._id + "_PasswordDialog" });
            $btndiv = ej.buildTag("div.e-dlg-btnfields");
            $div = ej.buildTag("div.e-dlg-btnctnr");
            $label = "<div class= 'e-dlgctndiv'><table><tr class= 'e-dlgtd-fields'><td><label>" + xlObj._getLocStr("Password") + ":</label></td><td><input id ='" + xlObj._id + "_xlpassword' type ='password' class = 'ejinputtext'/></td></tr><tr class= 'e-dlgtd-fields' id ='" + xlObj._id + "_confirm'><td><label>" + xlObj._getLocStr("ConfirmPassword") + "</label></td><td><input id ='" + xlObj._id + "_confirmpassword' type ='password'  class = 'ejinputtext'/></td></tr></table></div>";
            $dlg.append($label);
            $okBtn = ej.buildTag("input", "", "", { type: "submit" });
            $canBtn = ej.buildTag("input");
            $okBtn.ejButton({ text: xlObj._getLocStr("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._dlgOk, this), enabled: true });
            $canBtn.ejButton({ text: xlObj._getLocStr("Cancel"), click: ej.proxy(this._dlgCancel, this), showRoundedCorner: true, width: 60 });
            $btndiv.append($div.append($okBtn, $canBtn));
            $dlg.append($btndiv);
            $dlg.ejDialog({ enableModal: true, showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("ProtectWorkbook"), width: 420, height: "auto", cssClass: "e-ss-dialog", close: ej.proxy(this._dlgCancel, this) });
        },
        _importPasswordDlg: function () {
            var xlObj = this, $dlg, $label, $okBtn, $canBtn, $btndiv, $div;
            $dlg = ej.buildTag("div", "", "", { id: xlObj._id + "_ImportPasswordDialog" });
            $btndiv = ej.buildTag("div.e-dlg-btnfields");
            $div = ej.buildTag("div.e-dlg-btnctnr");
            $label = "<div class= 'e-dlgctndiv'><table><tr class= 'e-dlgtd-fields'><td><label id ='" + xlObj._id + "_workBookName' ></label></td></tr><tr class= 'e-dlgtd-fields'><td><label>" + xlObj._getLocStr("Password") + ":</label></td><td><input id ='" + xlObj._id + "_importpassword' type ='password'  class = 'ejinputtext'/></td></tr></table></div>";
            $dlg.append($label);
            $okBtn = ej.buildTag("input", "", "", { type: "submit" });
            $canBtn = ej.buildTag("input");
            $okBtn.ejButton({ text: xlObj._getLocStr("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._importDlgOk, this), enabled: true });
            $canBtn.ejButton({ text: xlObj._getLocStr("Cancel"), click: ej.proxy(this._dlgCancel, this), showRoundedCorner: true, width: 60 });
            $btndiv.append($div.append($okBtn, $canBtn));
            $dlg.append($btndiv);
            $dlg.ejDialog({ enableModal: true, showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("Password"), beforeOpen: $.proxy(this._dlgBeforeOpen, this), width: 420, height: "auto", cssClass: "e-ss-dialog", close: ej.proxy(this._dlgCancel, this) });
        },
        _dlgCancel: function (args) {
            var xlObj = this, dlgObj = $("#" + xlObj._id + "_PasswordDialog").data("ejDialog");
            if (dlgObj.model.title === "Protect Workbook")
                $("#" + xlObj._id + "_Ribbon_protectWorkbook").ejToggleButton("option", "toggleState", false);
            else if (dlgObj.model.title === "Unprotect Workbook")
                $("#" + xlObj._id + "_Ribbon_protectWorkbook").ejToggleButton("option", "toggleState", true);
            $("#" + xlObj._id + "_confirmpassword").val("");
            $("#" + xlObj._id + "_xlpassword").val("");
            $("#" + xlObj._id + "_importpassword").val("");
            $("#" + xlObj._id + "_PasswordDialog").ejDialog("close");
            $("#" + xlObj._id + "_ImportPasswordDialog").ejDialog("close");
        },
        _importDlgOk: function (args) {
            var xlObj = this, formData, pwdEle = $("#" + xlObj._id + "_importpassword");
            xlObj.model.importSettings.password = pwdEle.val();
            $("#" + xlObj._id + "_ImportPasswordDialog").ejDialog("close");
            formData = new FormData();
            formData.append("Password", xlObj.XLExport._encrypt(pwdEle.val()));
            formData.append("file", $("#" + xlObj._id + "_file").data("ejUploadbox")._files[0].rawFile);
            pwdEle.val("");
            xlObj["import"](formData);
		},

        lockCells: function (aRange, status) {
            var len, range, cells, i = 0, btn, sheetIdx = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIdx);
            if (this.model.allowLockCell || !sheet._isSheetProtected) {
                range = aRange ? this.getRangeIndices(aRange) : sheet.selectedRange;
                status ? this.updateUniqueData({ isLocked: true }, range) : this.clearRangeData(range, ["isLocked"]);
                if (this.model.showRibbon)
                    this.XLRibbon._updateRibbonIcons()
            }
        },

        protectSheet: function (status) {
            var sheetIdx = this.getActiveSheetIndex(), sheet = this.getSheet(sheetIdx), btn, rObj = this.XLRibbon;
            if (this.model.allowLockCell) {
                sheet._isSheetProtected = status;
                if (this.model.showRibbon) {

                    btn = $("#" + this._id + "_Ribbon_LockCell").data("ejToggleButton");
                    btn.option("enabled", !status);
                    status ? rObj._disableRibbonIcons() : rObj._enableRibbonIcons();
                    rObj._updateRibbonIcons();
                }
                if (this.model.showFormulaBar)
                    this.updateFormulaBar();
                if (this.model.allowUndoRedo) {
                    if (this.model.sheetCount > 1)
                        this._removeUndoRedoForCurSheet(sheetIdx, "undo");
                    else {
                        this._undoCollection = [];
                        this._redoCollection = [];
                    }
                }
            }
        },
        _israngeProtected: function (range) {
            var i = 0, flag = false;
            while (i < range.length) {
                if (this.XLEdit.getPropertyValue(range[i].rowIndex, range[i].colIndex, "isLocked")) {
                    flag = true;
                    break;
                }
                i++;
            }
            return flag;
        },
        _isCellProtected: function (rIndex, colIndex, alertStatus) {
            if (this.model.allowLockCell && this.getSheet(this.getActiveSheetIndex())._isSheetProtected && this.XLEdit.getPropertyValue(rIndex, colIndex, "isLocked")) {
                alertStatus && this._popUplockCellAlert();
                return true;
            }
        },

        _popUplockCellAlert: function () {
            this._showAlertDlg("Alert", "LockAlert", "", 500);
        },

        _removeUndoRedoForCurSheet: function (sheetIdx, state) {
            var collection = (state === "undo") ? this._undoCollection : this._redoCollection, len = collection.length;
            if (len) {
                while (len--) {
                    if (sheetIdx === collection[len].sheetIndex)
                        collection.splice(len, 1);
                }
            }
            if (state === "undo")
                this._removeUndoRedoForCurSheet(sheetIdx, "redo");
        }

    })

    ej.Spreadsheet.locale = {};

    ej.Spreadsheet.locale["en-US"] = {
        Cut: "Cut",
        Copy: "Copy",
        FormatPainter: "Format Painter",
        Paste: "Paste",
        PasteValues: "Paste Values Only",
        PasteSpecial: "Paste",
        Filter: "Filter",
        FilterContent: "Turn on filtering for the selected cells.",
        FilterSelected: "Filter by Selected Cell's value",
        Sort: "Sort",
        Clear: "Clear",
        ClearContent: "Delete everything in the cell, or remove just the formatting, contents, comments or hyperlinks.",
        ClearFilter: "Clear Filter",
        ClearFilterContent: "Clear the filter and sort state for the current range of data.",
        SortAtoZ: "Sort A to Z",
        SortAtoZContent: "Lowest to Highest.",
        SortZtoA: "Sort Z to A",
        SortZtoAContent: "Highest to Lowest.",
        SortSmallesttoLargest: "Sort Smallest to Largest",
        SortLargesttoSmallest: "Sort Largest to Smallest",
        SortOldesttoNewest: "Sort Oldest to Newest",
        SortNewesttoOldest: "Sort Newest to Oldest",
        Insert: "Insert",
		InsertTitle: "Insert Cells",
        InsertContent: "Add new cells, rows, or columns to your workbook.<br /><br />FYI: To insert multiple rows or columns at a time, select multiple rows or columns in the sheet, and click Insert.",
        InsertSBContent: "Add cells, rows, columns, or sheets to your workbook.",
        Delete: "Delete",
		DeleteTitle: "Delete Cells",
        DeleteContent: "Delete cells, rows, columns, or sheets from your workbook.<br /><br />FYI: To delete multiple rows or columns at a time, select multiple rows or columns in the sheet, and click Delete.",
        FindSelectTitle: "Find & Select",
        FindSelectContent: "Click to see options for finding text in your document.",
        Title: "Spreadsheet",
        Ok: "OK",
        Cancel: "Cancel",
        Alert: "We couldn't do this for the selected range of cells. Select a single cell within a range of data and then try again.",
        HeaderAlert: "The command could not be completed as you are attempting to filter with the filter header. Select a single cell in the filter range and try the command again.",
        FlashFillAlert: "All the data next to your selection was checked and there was no pattern for filling in values.",
        Formatcells: "Format Cells",
        FontFamily: "Font",
        FFContent: "Pick a new font for your text.",
        FontSize: "Font Size",
        FSContent: "Change the size of your text.",
        IncreaseFontSize: "Increase Font Size",
        IFSContent: "Make your text a bit bigger.",
        DecreaseFontSize: "Decrease Font Size",
        DFSContent: "Make your text a bit smaller.",
        Bold: "Bold",
        Italic: "Italic",
        Underline: "Underline",
        Linethrough: "Linethrough",
        FillColor: "Fill Color",
        FontColor: "Font Color",
        TopAlign: "Top Align",
        TopAlignContent: "Align text to the top.",
        MiddleAlign: "Middle Align",
        MiddleAlignContent: "Align text so that it is centered between the top and bottom of the cell.",
        BottomAlign: "Bottom Align",
        BottomAlignContent: "Align text to the bottom.",
        WrapText: "Wrap Text",
        WrapTextContent: "Wrap extra-long text into multiple lines so you can see all of it.",
        AlignLeft: "Align Left",
        AlignLeftContent: "Align your content to the left.",
        AlignCenter: "Center",
        AlignCenterContent: "Center your content.",
        AlignRight: "Align Right",
        AlignRightContent: "Align your content to the right.",      
        Undo: "Undo",
        Redo: "Redo",           
        NumberFormat: "Number Format",
        NumberFormatContent: "Choose the format for your cells, such as percentage, currency, date or time.",
        AccountingStyle: "Accounting Style",
        AccountingStyleContent: "Format as dollar accounting number format.",
        PercentageStyle: "Percent Style",
        PercentageStyleContent: "Format as a percent.",
        CommaStyle: "Comma Style",
        CommaStyleContent: "Format with a thousands separator.",
        IncreaseDecimal: "Increase Decimal",
        IncreaseDecimalContent: "Show more decimal places for a more precise value.",
        DecreaseDecimal: "Decrease Decimal",
        DecreaseDecimalContent: "Show fewer decimal places.",
        AutoSum: "AutoSum",
		AutoSumTitle: "Sum",
        AutoSumContent: "Automatically add a quick calculation to your worksheet, such as sum or average.",
        Fill: "Fill",
        ExportXL: "Excel",
        ExportCsv: "CSV",
        SaveXml: "Save XML",
        BackgroundColor: "Fill Color",
        BGContent: "Color the background of the cells to make them stand out.",
        Color: "Font Color",
        ColorContent: "Change the color of your text.",
        Border: "Border",
        BorderContent: "Apply borders to the currently selected cells.",
        BottomBorder: "Bottom Border",
        TopBorder: "Top Border",
        LeftBorder: "Left Border",
        RightBorder: "Right Border",
        OutsideBorder: "Outside Borders",
        NoBorder: "No Border",
        AllBorder: "All Borders",
        ThickBoxBorder: "Thick Box Border",
        ThickBottomBorder: "Thick Bottom Border",
        TopandThickBottomBorder: "Top and Thick Bottom Border",
        DrawBorderGrid: "Draw Border Grid",
        DrawBorder: "Draw Border",
        TopandBottomBorder: "Top and Bottom Border",
        BorderColor: "Line Color",
        BorderStyle: "Line Style",
        Number: "Number is used for general display of numbers.",
        General: "General format cells have no specific number format.",
        Currency: "Currency format is used for general monetary values.",
        Accounting: "Accounting format lines up the currency symbols and decimal points.",
        Text: "Text format cell is displayed exactly as entered.",
        Percentage: "This Format multiplies the cell value by 100 and displays with percent symbol.",
        CustomMessage: "Type number format code, using one of existing codes as a starting point.",
        Fractions: "",
        Scientific: "",
        Type: "Type",
        CustomFormatAlert: "Enter a valid format",
        Date: "Date formats display date and time serial numbers as date values",
        File: "FILE",
        New: "New",
        Open: "Open",
        SaveAs: "Save As",
        Print: "Print",
        PrintContent: "Print the current sheet.",
        PrintSheet: "Print Sheet",
        PrintSelected: "Print Selected",
        PrintSelectedContent: "Select an area on the sheet you would like to print.",
        HighlightVal: "Format Invalid Data",
        ClearVal: "Clear Validation",
        Validation: "Validation",
        DataValidation: "Data Validation",
        DVContent: "Pick from a list of rules to limit the type of data that can be entered in a cell.",
        A4: "A4",
        A3: "A3",
        Letter: "Letter",
        PageSize: "Page Size",
        PageSizeContent: "Choose a page size for your document.",
        FormatCells: "FormatCells",
        ConditionalFormat: "Conditional Formatting",
        CFContent: "Easily spot trends and patterns in your data using colors to visually highlight important values.",
        And: "and",
        With: "with",
        GTTitle: "Greater Than",
        GTContent: "Format cells that are GREATER THAN:",
        LTTitle: "Less Than",
        LTContent: "Format cells that are LESS THAN:",
        BWTitle: "Between",
        BWContent: "Format cells that are BETWEEN:",
        EQTitle: "Equal To",
        EQContent: "Format cells that are EQUAL TO:",
        DateTitle: "A Date Occurring",
        DateContent: "Format cells that contain a DATE:",
        ContainsTitle: "Text That Contains",
        ContainsContent: "Format cells that contain the text:",
        GreaterThan: "Greater Than",
        LessThan: "Less Than",
        Between: "Between",
        EqualTo: "Equal To",
        TextthatContains: "Text that Contains",
        DateOccurring: "A Date Occurring",
        ClearRules: "Clear Rules",
        ClearRulesfromSelected: "Clear Rules from Selected Cells",
        ClearRulesfromEntireSheets: "Clear Rules from Entire Sheet",
        CellStyles: "Cell Styles",
        CellStylesContent: "A colorful style is a great way to make important data stand out on the sheet.",
        CellStyleHeaderText: "Good, Bad and Neutral/Titles and Headings/Themed Cell Styles",
        Custom: "Custom",
        CellStyleGBN: "Normal/Bad/Good/Neutral",
        CellStyleTH: "Heading 4/Title",
        CellsStyleTCS: "20% - Accent1/20% - Accent2/20% - Accent3/20% - Accent4/60% - Accent1/60% - Accent2/60% - Accent3/60% - Accent4/Accent1/Accent2/Accent3/Accent4",
        Style: "Style",
        FormatAsTable: "Format As Table",
        FormatasTable: "Format as Table",
        FATContent: "Quickly convert a range of cells to a table with its own style.",
        FATHeaderText: "Light/Medium/Dark",
        FATNameDlgText: "Table Name :/My table has headers",
        InvalidReference: "The range you have specified is invalid",
        ResizeAlert: "The specified range is invalid. The top of the table must remain in the same row, and the resulting table must overlap the original table. Specify a valid range.",
        RangeNotCreated: "Increasing the row beyond the maximum sheet rowcount is restricted in Format as Table.",
        ResizeRestrictAlert:"Increase or decrease of column count and decrease of row count is restricted in Format as Table.",
        FATResizeTableText: "Enter new data range for your table:",
        FATReizeTableNote: "Note: The headers must remain in the same row, and the resulting table range must overlap the original table range.",
        FormatAsTableAlert: "Cannot create a table with a single row. A table must have at least two rows, one for the table header, and one for data",
        FormatAsTableTitle: "Light 1/Light 2/Light 3/Light 4/Light 5/Light 6/Light 7/Light 8/Light 9/Light 10/Light 11/Light 12/Medium 1/Medium 2/Medium 3/Medium 4/Medium 5/Medium 6/Medium 7/Medium 8/Dark 1/Dark 2/Dark 3/Dark 4",
        NewTableStyle: "New Table Style",
        ResizeTable: "Resize Table",
        ResizeTableContent: "Resize this table by adding or removing rows and columns.",
        ConvertToRange: "Convert to Range",
        ConvertToRangeContent: "Convert this table into a normal range of cells.",
        ConverToRangeAlert: "Do you want to convert the table to a normal range?",
        TableID: "Table ID:",
        Table: "Table",
        TableContent: "Create a table to organize and analyze related data.",
        TableStyleOptions: "First Column/Last Column/Total Row/Filter Button",
        Format: "Format",
        NameManager: "Name Manager",
        NameManagerContent: "Create, edit, delete and find all the names used in the workbook.<br /><br />Names can be used in formulas as substitutes for cell references.",
        DefinedNames: "Defined Names",
        DefineName: "Define Name",
        DefineNameContent: "Define and apply names.",
        UseInFormula: "Use In Formula",
        UseInFormulaContent: "Choose a name used in this workbook and insert it into the current formula.",
        Refersto: "Refers to",
        RefersTo: "Refers To",
        Name: "Name",
        Scope: "Scope",
        NMNameAlert: "The name that you entered is not valid./Reason for this can include:/The name does not begin with a letter or an underscore/The name contains a space or other invalid characters/The name conflicts with a spreadsheet built-in name or the name of another object in the workbook", // not used completely
        NMUniqueNameAlert: "The name entered already exists. Enter a unique name.",
        NMRangeAlert: "Enter a valid range",
        FORMULAS: "FORMULAS",
        Value: "Values",
		 DataValue: "Values:",
        Formula: "Formulas",
        MissingParenthesisAlert: "Your formula is missing a parenthesis--) or (. Check the formula, and then add the parenthesis in the appropriate place.",
        UnsupportedFile: "Unsupported File",
        IncorrectPassword: "Unable to open the file or worksheet with the given password",
        InvalidUrl: "Please specify proper URL",
        Up: "Up",
        Down: "Down",
        Sheet: "Sheet",
        Workbook: "Workbook",
        Rows: "By Rows",
        Columns: "By Columns",
        FindReplace: "Find Replace",
        FindnReplace: "Find and Replace",
        Find: "Find",
        Replace: "Replace",
        FindLabel: "Find what:",
        ReplaceLabel: "Replace with:",
        ReplaceAll: "Replace All",
        Close: "Close",
        FindNext: "Find Next",
        FindPrev: "Find Prev",
        Settings: "Settings",
        MatchCase: "Match case",
        MatchAll: "Match entire cell contents",
        Within: "Within:",
        Search: "Search:",
        Lookin: "Look in:",
        shiftright: "Shift cells right",
        shiftbottom: "Shift cells down",
        entirerow: "Entire row",
        entirecolumn: "Entire column",
        shiftup: "Shift cells up",
        shiftleft: "Shift cells left",
        Direction: "Direction:",
        GoTo: "Go To",
        GoToName: "Go to:",
        Reference: "Reference:",
        Special: "Special",
        Select: "Select",
        Comments: "Comments",
        Formulas: "Formulas",
        Constants: "Constants",
        RowDiff: "Row differences",
        ColDiff: "Column differences",
        LastCell: "Last cell",
        CFormat: "Conditional formats",
        Blanks: "Blanks",
        GotoError: "Error",
        GotoLogicals: "Logicals",
        GotoNumbers: "Numbers",
        GotoText: "Text",
        FindSelect: "Find & Select",
        Comment: "Comment",
        NewComment: "New",
        InsertComment: "Insert Comment",
        EditComment: "Edit",
        DeleteComment: "Delete Comment",
        DeleteCommentContent: "Delete the selected comment.",
        HideComment: "Hide Comment",
        Next: "Next",
        NextContent: "Jump to the next comment.",
        Previous: "Previous",
        PreviousContent: "Jump to the previous comment.",
        ShowHide: "Show/Hide Comment",
        ShowHideContent: "Show or hide the comment on the active cell.",
        ShowAll: "Show All Comments",
        ShowAllContent: "Display all comments in the sheet.",
        UserName: "User Name",
        Hide: "Hide",
        Unhide: "Unhide",
        Add: "Add",
        DropAlert: "Do you want to replace the existing data?",
        PutCellColor: "Put Selected Cell Color To The Top",
        PutFontColor: "Put Selected Font Color To The Top",
        WebPage: "Web Page",
        WorkSheet: "Worksheet Reference",
        SheetReference: "Sheet Reference",
        InsertHyperLink: "Insert Hyperlink",
        HyperLink: "Hyperlink",
        EditLink: "Editlink",
        OpenLink: "Openlink",
        HyperlinkText: "Text:",
        RemoveLink: "Removelink",
        WebAddress: "Web Address:",
        CellAddress: "Cell Reference:",
        SheetIndex: "Select a place in this document",
        ClearAll: "Clear All",
        ClearFormats: "Clear Formats",
        ClearContents: "Clear Contents",
        ClearComments: "Clear Comments",
        ClearHyperLinks: "Clear Hyperlinks",
        SortFilter: "Sort & Filter",
        SortFilterContent: "Organize your data so it's easier to analyze.",
        NumberStart: "Minimum:",
        NumberEnd: "Maximum:",
        DecimalStart: "Minimum",
        DecimalEnd: "Maximum:",
        DateStart: "Start Date:",
        DateEnd: "End Date:",
        ListStart: "Source:",
		FreeText: "Show error alert after invalid data is entered", 
        ListEnd: "Cell Reference:",
        TimeStart: "Start Time:",
        TimeEnd: "End Time:",
        TextLengthStart: "Minimum:",
        TextLengthEnd: "Maximum:",
        CommentFindEndAlert: "Spreadsheet reached the end of the workbook. Do you want to continue reviewing from the beginning of the workbook?",
        InsertSheet: "Insert",
        DeleteSheet: "Delete",
        RenameSheet: "Rename",
        MoveorCopy: "Move or Copy",
        HideSheet: "Hide",
        UnhideSheet: "Unhide",
        SheetRenameAlert: "That name is already taken. Try a different one.",
        SheetRenameEmptyAlert: "You typed an invalid name for a sheet. Make sure that:<ul><li>The name that you type does not exceed 31 characters.</li><li>The name does not contain any of the following characters: \ / ? * [ or  ]</li><li>You did not leave the name blank.</li></ul>",
        SheetDeleteAlert: "You can't undo deleting sheets, and you might be removing some data. If you don't need it, click OK to delete.",
        SheetDeleteErrorAlert: "A workbook must contain at least one visible worksheet. To hide, delete, or move the selected sheet, you must first insert a new sheet or unhide a sheet that is already hidden.",
        CtrlKeyErrorAlert: "That command cannot be used on  multiple selections.",
        MoveToEnd: "Move To End",
        Beforesheet: "Before sheet:",
        CreateaCopy: "Create a copy",
        AutoFillOptions: "Copy Cells/Fill Series/Fill Formatting Only/Fill Without Formatting/Flash Fill",
        NumberValidationMsg: "Enter only digits",
        DateValidationMsg: "Enter only date",
        Required: "Required",
        TimeValidationMsg: "The time you entered for the Time is invalid.",
        cellAddrsValidationMsg: "Reference is not valid.",
        PivotTable: "Pivot Table",
        PivotTableContent: "Easily arrange and summarize complex data in a PivotTable.",
        NumberTab: "Number",
        AlignmentTab: "Alignment",
        FontTab: "Font",
        BorderTab: "Border",
        FillTab: "Fill",
        textalignment: "Text alignment",
        horizontal: "Horizontal:",
        vertical: "Vertical:",
        indent: "Indent",
        textcontrol: "Text Control",
        wraptext: "Wrap text",
        fontgroup: "Font:",
        fontstyle: "Font style:",
        size: "Size:",
        Size: "Page size",
        effects: "Effects:",
        underline: "Underline",
        strikethrough: "Strikethrough",
        overline: "Overline",
        color: "Color:",
        normalfont: "Normal font",
        preview: "Preview",
        previewtext: "AaBbCcYyZz",
        line: "Line",
        style: "Style:",
        presets: "Presets",
        none: "None",
        outline: "Outline",
        border: "Border",
        allside: "All sides",
        backgroundcolor: "Background Color:",
        inscells: "Insert Cells",
        insrows: "Insert Sheet Rows",
        inscols: "Insert Sheet Columns",
        inssheet: "Insert Sheet",
        delcells: "Delete Cells",
        delrows: "Delete Sheet Rows",
        delcols: "Delete Sheet Columns",
        delsheet: "Delete Sheet",
        HyperLinkAlert: "The address of this site is not valid.Check the address and try again.",
        ReplaceData: "All done. We made / replacements.",
        NotFound: "We couldn't find what you were looking for. Select settings tab for more ways to search",
        Data: "Data:",
        Allow: "Allow:",
        IgnoreBlank: "Ignore blank",
        NotFind: "Unable to find the match to replace",
        freezePanes: "Freeze Panes",
        FreezeRow: "Freeze Top Row",
        FreezeColumn: "Freeze First Column",
        unfreezePanes: "Unfreeze Panes",
        DestroyAlert: "Are you sure you want to destroy the current workbook without saving and create a new workbook?",
        ImageValAlert: "Upload image files only",
        Pictures: "Pictures",
		PicturesTitle: "From File",
        PicturesContent: "Insert pictures from computer or from other computers that you are connected to.",
        ImportAlert: "Are you sure you want to destroy the current workbook without saving and open a new workbook?",
        UnmergeCells: "Unmerge Cells",
        MergeCells: "Merge Cells",
        MergeAcross: "Merge Across",
        MergeAndCenter: "Merge & Center",
        MergeAndCenterContent: "Combine and center the contents of the selected cells in a new larger cell.",
        MergeCellsAlert: "Merging Cells keeps only upper left cell value and discards the other values",
        Axes: "Axes",
        PHAxis: "Primary Horizontal",
        PVAxis: "Primary Vertical",
        AxisTitle: "Axis Title",
        CTNone: "None",
        CTCenter: "Center",
        CTFar: "Far",
        CTNear: "Near",
        DataLabels: "Data Labels",
        DLNone: "None",
        DLCenter: "Center",
        DLIEnd: "Inside End",
        DLIBase: "Inside Base",
        DLOEnd: "Outside End",
        ErrorBar: "Error Bars",
        Gridline: "Gridlines",
        PMajorH: "Primary Major Horizontal",
        PMajorV: "Primary Major Vertical",
        PMinorH: "Primary Minor Horizontal",
        PMinorV: "Primary Minor Vertical",
        Legend: "Legends",
        LNone: "None",
        LLeft: "Left",
        LRight: "Right",
        LBottom: "Bottom",
        LTop: "Top",
        ChartTitleDlgText: "Enter Title",
        ChartTitle: "Title",
		InvalidTitle: "You typed an invalid name for the Title.",
        CorrectFormat: "Select the correct Format File",
        ResetPicture: "Reset Picture",
        ResetPictureContent: "Discard all of the formatting changes made to this picture.",
        PictureBorder: "Picture Border",
        PictureBorderContent: "Pick the color, width, and line style for the outline of your shape.",
        ResetSize: "Reset Picture & Size",
        Height: "Height",
        Width: "Width",
        ThemeColor: "Theme Colors",
        NoOutline: "No Outline",
        Weight: "Weight",
        Dashes: "Dashes",
        ColumnChart: "2-D Column/3-D Column",
		ColumnChartTitle: "Insert Column Chart",
        ColumnChartContent: "Use this chart type to visually compare values across a few categories.",
        BarChart: "2-D Bar/3-D Bar",
		BarChartTitle: "Insert Bar Chart",
        BarChartContent: "Use this chart type to visually compare values across a few categories when the chart shows duration or the category text is long.",
        StockChart: "Radar",
		StockChartTitle: "Insert Radar Chart",
        StockChartContent: "Use this chart type to show values relative to a center point.",
        LineChart: "2-D Line",
		LineChartTitle: "Insert Line Chart",
        LineChartContent: "Use this chart type to show trends over time (years, months, and days) or categories.",
        AreaChart: "2-D Area/3-D Area",
		AreaChartTitle: "Insert Area Chart",
        AreaChartContent: "Use this chart type to show trends over time (years, months, and days) or categories. Use it to highlight the magnitude of change over time.",
        ComboChart: "Combo",
        PieChart: "Pie",
		PieChartTitle: "Insert Pie/Doughnut Chart",
        PieChartContent: "Use this chart type to show proportions of a whole. Use it when the total of your numbers is 100%.",
        ScatterChart: "Scatter",
		ScatterChartTitle: "Insert Scatter (X, Y) Chart",
        ScatterChartContent: "Use this chart type to show the relationship between sets of values.",
		ClusteredColumn: "Clustered&nbsp;Column",
		StackedColumn: "Stacked&nbsp;Column",
		Stacked100Column: "100%&nbsp;Stacked&nbsp;Column",
		Cluster3DColumn: "3-D&nbsp;Clustered&nbsp;Column",
		Stacked3DColumn: "3-D&nbsp;Stacked&nbsp;Column",
		Stacked100Column3D: "3-D&nbsp;100%&nbsp;Stacked&nbsp;Column",
		ClusteredBar: "Clustered&nbsp;Bar",
		StackedBar: "Stacked&nbsp;Bar",
		Stacked100Bar: "100%&nbsp;Stacked&nbsp;Bar",
		Cluster3DBar: "3-D&nbsp;Clustered&nbsp;Bar",
		Stacked3DBar: "3-D&nbsp;Stacked&nbsp;Bar",
		Stacked100Bar3D: "3-D&nbsp;100%&nbsp;Stacked&nbsp;Bar",
		Radar: "Radar",
		RadarMarkers: "Radar&nbsp;with&nbsp;Markers",
		Line: "Line",
		LineMarkers: "Line&nbsp;with&nbsp;Markers",
		Area: "Area",
		StackedArea: "Stacked&nbsp;Area",
		Stacked100Area: "100%&nbsp;Stacked&nbsp;Area",
		Pie: "Pie",
		Pie3D: "3-D&nbsp;Pie",
		Doughnut: "Doughnut",
		Scatter: "Scatter",
        ChartRange: "Chart Range",
        XAxisRange: "Enter X-axis range:",
        YAxisRange: "Enter Y-axis range:",
        LegendRange: "Enter legend range:",
        YAxisMissing: "Enter Y-axis range to create chart",
        InvalidYAxis: "Y-axis range must be within the selected range",
        InvalidXAxis: "X-axis range must be within the selected range",
        InvalidLegend: "Legend range must be within the selected range",
        InvalidXAxisColumns: "X-axis range should be within a single column",
        FreezePanes: "Freeze Panes",
        FreezePanesContent: "Freeze a portion of the sheet to keep it visible while you scroll through the rest of the sheet.",
        PasteTitle: "Paste (Ctrl+V)",
        PasteContent: "Add content on the Clipboard to your document.",
        PasteSplitContent: "Pick a paste option, such as keeping formatting or pasting only content.",
        CutTitle: "Cut (Ctrl+X)",
        CutContent: "Remove the selection and put it on the Clipboard so you can paste it somewhere else.",
        CopyTitle: "Copy (Ctrl+C)",
        CopyContent: "Put a copy of the selection on the Clipboard so you can paste it somewhere else.",
        FPTitle: "Format Painter",
        FPContent: "Like the look of a particular selection? You can apply that look to other content in the document.",
        BoldTitle: "Bold (Ctrl+B)",
        BoldContent: "Make your text bold.",
        ItalicTitle: "Italic (Ctrl+I)",
        ItalicContent: "Italicize your text.",
        ULineTitle: "Underline (Ctrl+U)",
        ULineContent: "Underline your text.",
        LineTrTitle: "Strike Through (Ctrl+L)",
        LineTrContent: "Cross something out by drawing a strike through it.",
        UndoTitle: "Undo (Ctrl+Z)",
        UndoContent: "Undo your last action.",
        RedoTitle: "Redo (Ctrl+Y)",
        RedoContent: "Redo your last action.",
        TableTitle: "Table (Ctrl+T)",
        HyperLinkTitle: "Add a Hyperlink (Ctrl+K)",
        HyperLinkContent: "Create a link in your document for quick access to webpages and files. <br /><br />Hyperlinks can also take you to places in your document.",
        NewCommentTitle: "Insert a Comment",
        NewCommentContent: "Add a note about this part of the document.",
        AddChartElement: "Add Chart Element",
        AddChartElementContent: "Add elements to the created chart.",
        SwitchRowColumn: "Switch Row/ Column",
        SwitchRowColumnContent: "Swap the data over the axis.",
        MergeAlert: "We can't do that to a merged cell.",
        UnhideDlgText: "Unhide Sheet:",
        ChartThemes: "Chart Themes",
        ChartThemesContent: "Pick a new theme for your chart.",
        ChangePicture: "Change Picture",
        ChangePictureContent: "Change to a different picture, preserving the formatting and size of the current picture.",
        ChangeChartType: "Change Chart Type",
        SelectData: "Select Data",
        SelectDataContent: "Change the data range included in the chart.",
        Sum: "Sum",
        Average: "Average",
        CountNumber: "Count Numbers",
        Max: "Max",
        Min: "Min",
        ChartType: "Change Chart Type",
        ChartTypeContent: "Change to a different chart type.",
        AllCharts: "All Charts",
        defaultfont: "Default",
        LGeneral: "General",
        LCurrency: "Currency",
        LAccounting: "Accounting",
        LDate: "Date",
        LTime: "Time",
        LPercentage: "Percentage",
        LFraction: "Fraction",
        LScientific: "Scientific",
        LText: "Text",
        LCustom: "Custom",
        Top: "Top",
        Center: "Center",
        Bottom: "Bottom",
        Left: "Left (Indent)",
        Right: "Right",
        Justify: "Justify",
        GeneralTxt: "General format cells have no specific number format.",
        NegativeNumbersTxt: "Negative Numbers",
        ThousandSeparatorTxt: "Use 1000 Separator by (,)",
        DecimalPlacesTxt: "Decimal Places:",
        TextTxt: "Text format cells are treated as text even when a number is in the cell. The cell is displayed exactly as entered.",
        BoldItalic: "Bold Italic",
        Regular: "Regular",
        HyperLinkHide: "<<Selection in Document>>",
        InvalidSheetIndex: "Specify proper SheetIndex",
        HugeDataAlert: "File too large to open.",
        ImportExportUrl: "Give import/export URL and try again.",
        TitleColumnChart: "Clustered Column/Stacked Column/100% Stacked Column/3D Clustered Column/3D Stacked Column/ 3D 100% Stacked Column",
        TitleBarChart: "Clustered Bar/Stacked Bar/100% Stacked Bar/3D Clustered Bar/3D Stacked Bar/3D 100% Stacked Bar",
        TitleRadarChart: "Radar/Radar with Markers",
        TitleLineChart: "Line/Line with Markers",
        TitleAreaChart: "Area/Stacked Area/100% Stacked Area",
        TitlePieChart: "Pie/3D Pie/Doughnut",
        TitleScatterChart: "Scatter",
        BetweenAlert: "The Maximum must be greater or equal to the Minimum.",
        BorderStyles: "Solid/Dashed/Dotted",
        FPaneAlert: "Freeze Pane is not applied for the First Cell",
        ReplaceNotFound: "Spreadsheet cannot find a match.",
        BlankWorkbook: "Blank workbook",
        SaveAsExcel: "Save As Excel",
        SaveAsCsv: "Save As CSV",
        PrintSheet: "Print Sheet",
        PrintSelected: "Print Selected",
        Design: "DESIGN",
        NewName: "New Name",
        FormulaBar: "Formula Bar",
        NameBox: "Name Box",
        NumberValMsg: "Decimal values cannot be used for number conditions.",
        NumberAlertMsg: "Enter only digits.",
        ListAlert: "Cell range is incorrect, Please enter correct cell range.",
        ListValAlert: "The list source must be a delimited list, or a reference to single row or column.",
        ListAlertMsg: "The value you entered is not valid",
        AutoFillTitle: "AutoFillOptions",
        NewSheet: "New Sheet",
        FullSheetCopyPasteAlert: "We can't paste because the Copy area and paste area aren't the same size.",
        Heading: "Headings",
        Gridlines: "Gridlines",
        Firstsheet: "Scroll to the first sheet",
        Lastsheet: "Scroll to the last sheet",
        Nextsheet: "Scroll to the next sheet",
        Prevsheet: "Scroll to the previous sheet",
		ProtectWorkbook: "Protect Workbook",
        UnProtectWorkbook: "Unprotect Workbook",
        ProtectWBContent: "Keep others from making structural changes to your workbook",
        Password: "Password",
        ConfirmPassword: "Reenter password to proceed:",
        PasswordAlert1: "Confirmation password is not identical.",
        PasswordAlert2: "Please enter a password.",
        PasswordAlert3: "The password you supplied is not correct. Verify that the CAPS LOCK key is off and be sure to use the correct capitalization.",
        Protect: "is protected.",
        Prevsheet: "Scroll to the previous sheet",
        Lock: "LockCell",
        Unlock: "UnlockCell",
        Protectsheet: "Protect Sheet",
        protectsheettooltip: "Prevent unwanted changes from others by limiting their ability to edit",
        Unprotect: "Unprotect Sheet",
        LockAlert: "The cell you are trying to change is on protected sheet. To make changes, click Unprotect Sheet in the Review tab."
    };

    ej.Spreadsheet.scrollMode = {
        Normal: "normal",
        Infinite: "infinite"
    };

    ej.Spreadsheet.AutoFillOptions = {
        CopyCells: "copycells",
        FillSeries: "fillseries",
        FillFormattingOnly: "fillformattingonly",
        FillWithoutFormatting: "fillwithoutformatting",
        FlashFill: "flashfill"
    };

    ej.Spreadsheet.autoFillDirection = {
        Up: "up",
        Right: "right",
        Down: "down",
        Left: "left"
    };

    ej.Spreadsheet.pageSize = {
        A4: "A4",
        A3: "A3",
        Letter: "Letter"
    };

    ej.Spreadsheet.Actions = {
        Sorting: "sorting",
        Selection: "selection",
        AutoFill: "autofill",
        CellNavigation: "cellnavigation",
        Filtering: "filtering"
    };

    ej.Spreadsheet.exportType = {
        Excel: "Excel",
        Csv: "Csv"
    };

    ej.Spreadsheet.CellType = {
        String: "string",
        Number: "number",
        Custom: "custom",
        Formula: "formula",
        Time: "time",
        LongDate: "longdate",
        ShortDate: "shortdate",
        Scientific: "scientific",
        Fraction: "fraction",
        Text: "text",
        General: "general",
        Accounting: "accounting",
        Currency: "currency",
        Percentage: "percentage",
        Date: "date",
        DateTime: "datetime"
    };

    ej.Spreadsheet.SupportedStyles = {
        FontFamily: "font-family",
        FontSize: "font-size",
        FontStyle: "font-style",
        FontWeight: "font-weight",
        TextAlign: "text-align",
        TextDecoration: "text-decoration",
        TextIndent: "text-indent",
        VerticalAlign: "vertical-align"
    };

    ej.Spreadsheet.Validation = {
        ">": "GreaterThan",
        ">=": "GreaterOrEqualTo",
        "<": "LessThan",
        "<=": "LessThanOrEqualTo",
        "==": "EqualTo",
        "!=": "NotEqualTo",
        "!": "Not",
        "=": ""
    };

    ej.Spreadsheet.ValidationText = {
        ">": "greater than",
        ">=": "greater than or equal to",
        "<": "less than",
        "<=": "less than or equal to",
        "==": "equal to",
        "!=": "not equal to",
        "!": " not",
        "=": ""
    };

    ej.Spreadsheet.msg = " ";

    ej.Spreadsheet.getMsg = function () {
        return ej.Spreadsheet.msg;
    };

    ej.Spreadsheet.SelectionType = {
        Row: "row",
        Column: "column",
        Default: "default"
    };

    ej.Spreadsheet.SelectionUnit = {
        Single: "single",
        Range: "range",
        MultiRange: "multirange"
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.selection = function (obj) {
        this.XLObj = obj;
        this._isRightClick = false;
        this._colhdrClick = false;
        this._rowhdrClick = false;
        this._aFillDownHandler = false;
        this._isOutsideBordering = false;
        this._isGridBordering = false;
        this._isRowSelected = false;
        this._isColSelected = false;
        this._isSheetSelected = false;
        this._isEndReached = true;
    };

    ej.spreadsheetFeatures.selection.prototype = {
        _selectionMouseDownHandler: function (e) {
            e.preventDefault();
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), trgt = e.target;
            xlObj._on($(document), "mouseup", this._mouseUpHandler);
            if ((e.which === 3 || e.button === 2) || xlObj.model.selectionSettings.selectionUnit === ej.Spreadsheet.SelectionUnit.Single)
                this._isRightClick = true;
            else
                xlObj._on(xlObj.element, "mousemove", this._mouseMoveHandler);
            if (trgt.tagName == "TD") {
                if (xlObj.XLEdit._isFormulaEdit) {
                    if (e.ctrlKey) {
                        this._clearBorder(xlObj._arrayAsString(xlObj._ctrlFormulaBorder));
                        xlObj._ctrlKeyCount = xlObj._formulaRange.length;
                    }
                    xlObj._dStartCell = xlObj._getCellIdx(trgt);
                }
                else if (e.shiftKey && sheet._startCell.rowIndex != -1)
                    sheet._endCell = xlObj._getCellIdx(trgt);
                else {
                    sheet._startCell = xlObj._getCellIdx(trgt);
                    sheet._activeCell = xlObj._getCellIdx(trgt);
                }
                if (trgt.className.indexOf("e-rowheader") > -1)
                    this._rowhdrClick = true;
            }
            else if (trgt.parentNode.tagName == "TH") {  // check for parentnode issue for all
                if (e.shiftKey && sheet._startCell.rowIndex != -1)
                    sheet._endCell = { rowIndex: 0, colIndex: trgt.parentNode.cellIndex };
                else {
                    sheet._startCell = { rowIndex: 0, colIndex: trgt.parentNode.cellIndex };
                    sheet._activeCell = { rowIndex: 0, colIndex: trgt.parentNode.cellIndex };
                }
                if (trgt.className.indexOf("e-headercelldiv") > -1)
                    this._colhdrClick = true;
            }
            else if (trgt.className == "e-autofill")
                this._aFillDownHandler = true;
        },

        _mouseMoveHandler: function (e) {
            var status, xlObj = this, selObj = xlObj.XLSelection, index = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(index),
                trgt = e.target, range, type = xlObj.model.selectionSettings.selectionType, types = ej.Spreadsheet.SelectionType;
            if (trgt.tagName == "TD") {
                this.XLSelection._isEndReached = false;
                xlObj._getContent(index).find("td").removeClass(xlObj._ctrlFormulaBorder[xlObj._ctrlKeyCount % 6].join(" "));
                if (xlObj.XLEdit._isFormulaEdit) {
                    if (e.ctrlKey) {
                        xlObj._dEndCell = xlObj._getCellIdx(trgt);
                        selObj._processFormulaRange(xlObj._ctrlFormulaBorder, true);
                    }
                    else {
                        xlObj._dEndCell = xlObj._getCellIdx(trgt);
                        selObj._processFormulaRange(xlObj._ctrlFormulaBorder, true);
                    }
                    return;
                }
                else if (selObj._aFillDownHandler && $(trgt).parents(".e-rowheadercontent").length === 0) {
                    xlObj.model.allowAutoFill && xlObj.XLDragFill._selectAutoFillRange(trgt);
                    return;
                }
                else {
                    sheet._startCell = sheet._activeCell;
                    sheet._endCell = xlObj._getCellIdx(trgt);
                    selObj._processBoundary();
                    if ($(trgt).parents(".e-spreadsheetcontentcontainer").length > 0) {
                        if (e.ctrlKey && xlObj.model.selectionSettings.selectionType === ej.Spreadsheet.SelectionType.Default && xlObj.model.selectionSettings.selectionUnit === ej.Spreadsheet.SelectionUnit.MultiRange) {
                            xlObj._getContent(xlObj.getActiveSheetIndex()).find("td").removeClass("e-activecell e-focusright e-focusbottom");
                            range = xlObj.swapRange([sheet._startCell.rowIndex, sheet._startCell.colIndex, sheet._endCell.rowIndex, sheet._endCell.colIndex]);
                            xlObj.getRange(range[0], range[1], range[2], range[3]).addClass("e-ctrlselected");
                            selObj._focusRange(sheet._startCell, sheet._endCell, xlObj._selectionBorder);
                            xlObj.model.allowAutoFill && xlObj.XLDragFill.hideAutoFillElement();
                            return;
                        }
                        else {
                            if (type === types.Row || selObj._rowhdrClick)
                                selObj.selectRows(sheet._startCell.rowIndex, sheet._endCell.rowIndex);
                            else if (type === types.Column || selObj._colhdrClick)
                                selObj.selectColumns(sheet._startCell.colIndex, sheet._endCell.colIndex);
                            else
                                selObj.selectRange(sheet._startCell, sheet._endCell, trgt);
                        }
                    }
                    else if ($(trgt).parents(".e-rowheadercontent").length > 0)
                        if (selObj._colhdrClick || selObj._rowhdrClick) {
                            if (type === types.Column)
                                selObj.selectColumns(0, sheet.colCount - 1);
                            else
                                selObj.selectRows(sheet._startCell.rowIndex, sheet._endCell.rowIndex);
                        }
                        else {
                            status = { position: "horizontal", action: "Decrement" };
                            selObj._scrollCalculation(index, trgt, status);
                        }
                    else
                        return;
                }
            }
            else if (!ej.isNullOrUndefined(trgt.parentNode) && trgt.parentNode.tagName == "TH") {
                sheet._endCell = { rowIndex: 0, colIndex: trgt.parentNode.cellIndex };
                sheet._startCell = sheet._activeCell;
                selObj._processBoundary();
                if (selObj._colhdrClick || selObj._rowhdrClick) {
                    if (type === types.Row)
                        selObj.selectRows(0, sheet.rowCount - 1);
                    else
                        selObj.selectColumns(sheet._startCell.colIndex, sheet._endCell.colIndex);
                }
                else {
                    status = { position: "vertical", action: "Decrement" };
                    selObj._scrollCalculation(index, trgt, status);
                }
            }
            else if (trgt.className.indexOf("e-hhandlespace") > -1 || trgt.className.indexOf("e-hhandle") > -1 || trgt.className.indexOf("e-hup") > -1 || trgt.className.indexOf("e-hdown") > -1) {
                status = { position: "vertical", action: "Increment" };
                selObj._scrollCalculation(index, trgt, status);
            }

            else if (trgt.className.indexOf("e-vhandlespace") > -1 || trgt.className.indexOf("e-vhandle") > -1 || trgt.className.indexOf("e-vup") > -1 || trgt.className.indexOf("e-vdown") > -1) {
                status = { position: "horizontal", action: "Increment" };
                selObj._scrollCalculation(index, trgt, status);
            }
            else
                return;

        },

        _mouseUpHandler: function (e) {
            var mergeBtn, style,prctRange,cells, currcell, autofillRange, xlObj = this, selObj = xlObj.XLSelection, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx),
                range = sheet.selectedRange, sheetElement = xlObj.getSheetElement(sheetIdx),scell=sheet._startCell,eCell= sheet._endCell, trgt = e.target, type = xlObj.model.selectionSettings.selectionType, types = ej.Spreadsheet.SelectionType;
            xlObj._off($(document), "mouseup", selObj._mouseUpHandler)._off(xlObj.element, "mousemove", selObj._mouseMoveHandler);
            e.target = xlObj._changeTargetWithOffset(e);
            trgt = e.target;
			if (trgt.tagName == "TD" || trgt.tagName == "INPUT" || (trgt.tagName == "DIV" && selObj._aFillDownHandler)) {
                $(trgt).parents(".e-rowheadercontent").length > 0 && (selObj._isOutsideBordering = selObj._isGridBordering = false);
                if (e.ctrlKey)
                    xlObj._preventctrlkey = true;
                else
                    xlObj._preventctrlkey = false;
                if (xlObj.XLEdit._isFormulaEdit) {
                    xlObj._getContent(xlObj.getActiveSheetIndex()).find("td").removeClass(xlObj._ctrlFormulaBorder.join(" ").replace(/,/g, " "));
                    if (e.ctrlKey) {
                        xlObj._dEndCell = xlObj._getCellIdx(trgt);
                        selObj._processFormulaRange(xlObj._formulaBorder);
                    }
                    else {
                        selObj._clearBorder(xlObj._formulaBorder.join(" ").replace(/,/g, " ") + " " + xlObj._ctrlFormulaBorder.join(" ").replace(/,/g, " "));
                        xlObj._ctrlKeyCount = 0;
                        xlObj._dEndCell = xlObj._getCellIdx(trgt);
                        selObj._processFormulaRange(xlObj._formulaBorder);
                        xlObj._formulaRange = [];
                    }
                    xlObj._formulaRange.push(xlObj._getProperAlphaRange(sheetIdx, xlObj._dStartCell.rowIndex, xlObj._dStartCell.colIndex, xlObj._dEndCell.rowIndex, xlObj._dEndCell.colIndex));
                    return;
                }
                else if (selObj._aFillDownHandler) {
                    currcell = trgt.tagName == "DIV" ? xlObj.XLShape._getCellIndexFromOffset(trgt.offsetTop, trgt.offsetLeft) : xlObj._getCellIdx(trgt);
                    autofillRange = xlObj.model.allowAutoFill && xlObj.XLDragFill._getAutoFillRange(currcell);
                    if (xlObj.model.allowLockCell && xlObj.getSheet(xlObj.getActiveSheetIndex())._isSheetProtected) {
                       prctRange = xlObj.swapRange(autofillRange.fillRange);
                       cells = xlObj.XLSelection._getSelectedRange({ rowIndex: prctRange[0], colIndex: prctRange[1] }, { rowIndex: prctRange[2], colIndex: prctRange[3] });
                       if (xlObj._israngeProtected(cells)) {
                           selObj._aFillDownHandler = false;
                           xlObj.XLSelection.selectRange(scell, eCell);
                           xlObj.XLDragFill.positionAutoFillElement();
                           xlObj._popUplockCellAlert();                        
                            return;
                        }
                    }
                    if (!autofillRange)
                        return;
                    else if ("direction" in autofillRange) {
                        xlObj._pStartCell = sheet._startCell;
                        xlObj._pEndCell = sheet._endCell;
                        xlObj._pFillCell = currcell;
                        if (xlObj.inRange(sheet.selectedRange, currcell.rowIndex, currcell.colIndex))
                            xlObj.clearRangeData(null, null, $(selObj.getSelectedCells()).not(".e-autofillcell"));
                        else
                            xlObj.model.allowAutoFill && xlObj.XLDragFill.autoFill({ dataRange: sheet.selectedRange, fillRange: autofillRange.fillRange, direction: autofillRange.direction });
                        sheet._startCell = autofillRange.startCell;
                        sheet._endCell = autofillRange.endCell;
                    }
                    else
                        selObj._aFillDownHandler = false;
                    selObj.refreshSelection(xlObj.swapRange([autofillRange.startCell.rowIndex, autofillRange.startCell.colIndex, autofillRange.endCell.rowIndex, autofillRange.endCell.colIndex]));
                }
                else if (selObj._isOutsideBordering || selObj._isGridBordering) {  // for border drawing
                    style = "1px " + xlObj._borderStyle + " " + xlObj._borderColor;
                    sheet._endCell = selObj._isRightClick ? sheet._startCell : xlObj._getCellIdx(trgt);
                    if ($(trgt).parents(".e-spreadsheetcontentcontainer").length > 0) {
                        selObj._isGridBordering ? xlObj.XLFormat.applyBorder(style) : xlObj.XLFormat.applyBorder({ top: style, right: style, bottom: style, left: style });
                        selObj._cleanUp();
                        xlObj.model.allowAutoFill && xlObj.XLDragFill.hideAutoFillElement();
                    }
                    else
                        return;
                }
                else {
                    if (trgt.tagName == "INPUT" && $(trgt).parents("tr").length > 0)
                        sheet._endCell = selObj._isRightClick ? sheet._startCell : xlObj._getCellIdx($(trgt).parents("td")[0]);
                    else {
                        if (selObj._isRightClick)
                            sheet._endCell = sheet._startCell;
                        else {
                            sheet._startCell = sheet._activeCell;
                            sheet._endCell = xlObj._getCellIdx(trgt);
                            selObj._processBoundary();
                        }
                    }
                    if ($(trgt).parents(".e-spreadsheetcontentcontainer").length > 0) {
                        if (e.ctrlKey && xlObj.model.selectionSettings.selectionType === ej.Spreadsheet.SelectionType.Default && xlObj.model.selectionSettings.selectionUnit === ej.Spreadsheet.SelectionUnit.MultiRange) {
                            xlObj._getContent(xlObj.getActiveSheetIndex()).find("td").removeClass("e-activecell e-focustop e-focusleft e-ctrlselected");
                            range = xlObj.swapRange([sheet._startCell.rowIndex, sheet._startCell.colIndex, sheet._endCell.rowIndex, sheet._endCell.colIndex]);
                            xlObj.getRange(range[0], range[1], range[2], range[3]).addClass("e-selected");
                            xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex).addClass('e-activecell');
                            selObj._focusRange(sheet._startCell, sheet._endCell, xlObj._selectionBorder);
                            xlObj.model.allowAutoFill && xlObj.XLDragFill.hideAutoFillElement();
                            xlObj.getSheet(xlObj.getActiveSheetIndex())._selectedCells.push(selObj._getSelectedRange(xlObj.getSheet(sheetIdx)._startCell, xlObj.getSheet(sheetIdx)._endCell));  // consider
                            xlObj.model.showRibbon && xlObj.model.allowComments && sheetElement.find(".e-selected.e-commentcell").length > 0 && xlObj.XLRibbon._enableButtons(["deleteComment"], "ejButton");
                            return;
                        }
                        else {
                            if (type === types.Row || selObj._rowhdrClick)
                                selObj.selectRows(sheet._startCell.rowIndex, sheet._endCell.rowIndex);
                            else if (type === types.Column || selObj._colhdrClick)
                                selObj.selectColumns(sheet._startCell.colIndex, sheet._endCell.colIndex);
                            else
                                selObj.selectRange(sheet._startCell, sheet._endCell, trgt);
                        }
                    }
                    else if ($(trgt).parents(".e-rowheadercontent").length > 0) {
                        if (selObj._colhdrClick || selObj._rowhdrClick) {
                            if (type === types.Column)
                                selObj.selectColumns(0, sheet.colCount - 1);
                            else
                                selObj.selectRows(sheet._startCell.rowIndex, sheet._endCell.rowIndex);
                            selObj._hdrClick = false;
                        }
                        selObj._isOutsideBordering = selObj._isGridBordering = false;
                    }
                    else
                        return;
                }
                if (trgt.tagName === "TD" && xlObj.model.showRibbon) {
                    var btnObj = $("#" + xlObj._id + "_Ribbon_clearfilter").data("ejButton");
                    !ej.isNullOrUndefined(btnObj) && xlObj.XLRibbon._changeClrFltrStatus(btnObj, "button", xlObj._getCellIdx(trgt));
                }
            }
            else if (!selObj._aFillDownHandler && !ej.isNullOrUndefined(trgt.parentNode) && trgt.parentNode.tagName == "TH") {
                sheet._startCell = sheet._activeCell;
                sheet._endCell = selObj._isRightClick ? sheet._startCell : { rowIndex: 0, colIndex: trgt.parentNode.cellIndex };
                selObj._processBoundary();
                if (selObj._colhdrClick || selObj._rowhdrClick) {
                    if (type === types.Row)
                        selObj.selectRows(0, sheet.rowCount - 1);
                    else
                        selObj.selectColumns(sheet._startCell.colIndex, sheet._endCell.colIndex);
                }
                selObj._isOutsideBordering = selObj._isGridBordering = false;
            }
            else if (typeof trgt.className != "object" && trgt.className && trgt.className.indexOf("e-spreadsheetcolumnheader") > -1) {
                selObj.selectSheet();
                selObj._isOutsideBordering = selObj._isGridBordering = false;
            }
            else {
                xlObj.model.showRibbon && xlObj.XLRibbon._updateRibbonIcons();
                return;
            }
            xlObj.model.allowCellFormatting && xlObj.XLFormat._formatEnable && xlObj.XLFormat._fPMouseUp(e);
            xlObj.model.showRibbon && (xlObj.XLEdit._isEdit ? xlObj.XLRibbon._disableRibbonIcons() : xlObj.XLRibbon._updateRibbonIcons());
            xlObj.model.allowAutoFill && xlObj.XLDragFill.positionAutoFillElement(selObj._aFillDownHandler);
            selObj._isRightClick = selObj._aFillDownHandler = false;
            selObj._colhdrClick = selObj._rowhdrClick = false;
            xlObj.setSheetFocus();
            if (xlObj.model.showRibbon) {
                mergeBtn = $("#" + xlObj._id + "_Ribbon_merge").data("ejSplitButton");
                if(!ej.isNullOrUndefined(mergeBtn) &&(xlObj.model.allowLockCell && !sheet._isSheetProtected))
					(xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-selected").is('[class*="e-table"]')) ? mergeBtn.option("enabled", false) : mergeBtn.option("enabled", true);
            }
            if (xlObj.model.allowEditing && this.XLEdit._isEdit) {
                var cell = xlObj.getSheet(xlObj.getActiveSheetIndex())._startCell;
                xlObj.getCell(cell.rowIndex, cell.colIndex).find('.e-ss-input').focus();
            }
        },

        _processBoundary: function () {
            var i, status, cell, cells, cname, idx, arr = [], str = "e-rc-", xlObj = this.XLObj, regx = xlObj._rcRegx,
                sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), sCell = sheet._startCell, eCell = sheet._endCell,
                range = xlObj.swapRange([sCell.rowIndex, sCell.colIndex, eCell.rowIndex, eCell.colIndex]);
            sheet._startCell = { rowIndex: range[0], colIndex: range[1] };
            sheet._endCell = { rowIndex: range[2], colIndex: range[3] };
            cells = $.makeArray(xlObj.getRange(range[0], range[1], range[0], range[3])); // top
            cells = cells.concat($.makeArray(xlObj.getRange(range[0], range[3], range[2], range[3]))); // right
            cells = cells.concat($.makeArray(xlObj.getRange(range[2], range[1], range[2], range[3]))); // bottom
            cells = cells.concat($.makeArray(xlObj.getRange(range[0], range[1], range[2], range[1]))); // left
            i = cells.length;
            while (i--) {
                cell = cells[i];
                cname = cell.className;
                idx = xlObj._getCellIdx(cell);
                idx = (idx.rowIndex + ":" + idx.colIndex).toString();
                status = xlObj.hasSpan(idx) && arr.indexOf(idx) === -1 && (arr.push(idx), this._updateRange(cell, range));
                if (cname.indexOf(str) > -1) {
                    idx = cname.match(regx)[0].split(" ")[0].replace(str, "");
                    if (arr.indexOf(idx) === -1) {
                        arr.push(idx);
                        idx = idx.split(":");
                        cell = xlObj.getCell(Number(idx[0]), Number(idx[1]))[0];
                        status = this._updateRange(cell, range);
                    }
                }
                if (status)
                    break;
            }
            if(status)
                this._processBoundary();
        },

        _updateRange: function (cell, range) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), temp = range.slice(0), idx = xlObj._getCellIdx(cell),
                minr = idx.rowIndex, minc = idx.colIndex, maxr = minr + (cell.rowSpan - 1), maxc = minc + (cell.colSpan - 1);
            minr < range[0] && (range[0] = minr);
            minc < range[1] && (range[1] = minc);
            maxr > range[2] && (range[2] = maxr);
            maxc > range[3] && (range[3] = maxc);
            if (range[0] != temp[0] || range[1] != temp[1] || range[2] != temp[2] || range[3] != temp[3]) {
                sheet._startCell = { rowIndex: range[0], colIndex: range[1] };
                sheet._endCell = { rowIndex: range[2], colIndex: range[3] };
                return true;
            }
        },

        selectRange: function (startCell, endCell, trgt) {
            var cells, sheetIdx, sheet, args, prevRange, currRange, xlObj = this.XLObj;
            if (!xlObj.model.allowSelection)
                return;
            sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            prevRange = sheet.selectedRange;
            currRange = xlObj.swapRange([startCell.rowIndex, startCell.colIndex, endCell.rowIndex, endCell.colIndex]);
            if (!xlObj.importSheet && !sheet._importRendered) {
                args = { prevRange: prevRange, currRange: currRange, sheetIdx: sheetIdx };
                if (trgt)
                    args.target = trgt;
                if (xlObj._trigger("beforeCellSelect", args))
                    return;
            }
            xlObj._dStartCell = startCell;
            xlObj._dEndCell = endCell;
            sheet.selectedRange = currRange;
            this._cleanUp(false);
            cells = xlObj.getRange(currRange[0], currRange[1], currRange[2], currRange[3]);
            cells.addClass("e-selected");
            sheet._activeCell = xlObj._getMergedIdx(sheet._activeCell.rowIndex, sheet._activeCell.colIndex);
            xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex).addClass("e-activecell");
            this._focusRange(startCell, endCell, xlObj._selectionBorder);
            this._highlightHeader(currRange);
            if(xlObj.model.allowDragAndDrop && ej.spreadsheetFeatures.dragAndDrop)
                xlObj.XLDragDrap._initDragAndDropRange(currRange);
            sheet._selectedCells = this._getSelectedRange(startCell, endCell);
            if (xlObj.model.allowFormulaBar)                
                    xlObj.updateFormulaBar();
            if(xlObj.model.enableContextMenu)
                xlObj.XLRibbon._dirtySelect(cells);
            this._isRowSelected = this._isColSelected = this._isSheetSelected = false;
            sheet._isRowSelected = sheet._isColSelected = sheet._isSheetSelected = false;
            if (!xlObj.importSheet && !sheet._importRendered) {
                args = { selectedRange: currRange, target: trgt, sheetIdx: sheetIdx };
                xlObj._trigger("cellSelected", args);
            }
        },

        selectRow: function (rowIdx) {
            var sheet, xlObj = this.XLObj;
            if (!xlObj.model.allowSelection)
                return;
            this._isRowSelected = true;
            sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());            
            sheet._startCell = { rowIndex: rowIdx, colIndex: 0 };
            sheet._endCell = { rowIndex: rowIdx, colIndex: sheet.colCount - 1 };
            this.selectRange(sheet._startCell, sheet._endCell);
            this._selectRow(rowIdx, "e-rowselected");            
            sheet._isRowSelected = true;
        },

        selectRows: function (startIndex, endIndex) {
            var sheet, xlObj = this.XLObj;
            if (!xlObj.model.allowSelection)
                return;
            this._isRowSelected = true;
            sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());            
            sheet._startCell = { rowIndex: startIndex, colIndex: 0 };
            sheet._endCell = { rowIndex: endIndex, colIndex: sheet.colCount - 1 };
            this.selectRange(sheet._startCell, sheet._endCell);
            this._markHeaderSelection([sheet._startCell.rowIndex, sheet._startCell.colIndex, sheet._endCell.rowIndex, sheet._endCell.colIndex], "row");
            sheet._isRowSelected = true;
        },

        selectColumn: function (colIdx) {
            var sheet, xlObj = this.XLObj;
            if (!xlObj.model.allowSelection)
                return;
            this._isColSelected = true;
            sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());
            sheet._startCell = { rowIndex: 0, colIndex: colIdx };
            sheet._endCell = { rowIndex: sheet.rowCount - 1, colIndex: colIdx };
            this.selectRange(sheet._startCell, sheet._endCell);
            this._selectColumn(colIdx, "e-colselected");
            sheet._isColSelected = true;
        },

        selectColumns: function (startIndex, endIndex) {
            var sheet, xlObj = this.XLObj;
            if (!xlObj.model.allowSelection)
                return;
            this._isColSelected = true;
            sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());
            sheet._startCell = { rowIndex: 0, colIndex: startIndex };
            sheet._endCell = { rowIndex: sheet.rowCount - 1, colIndex: endIndex };
            this.selectRange(sheet._startCell, sheet._endCell);
            this._markHeaderSelection([sheet._startCell.rowIndex, sheet._startCell.colIndex, sheet._endCell.rowIndex, sheet._endCell.colIndex], "column");
            sheet._isColSelected = true;
        },

        selectSheet: function () {
            var sheet, xlObj = this.XLObj;
            if (!xlObj.model.allowSelection)
                return;
            this._isSheetSelected = true;
            sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());
            sheet._activeCell = { rowIndex: 0, colIndex: 0 };
            sheet._startCell = { rowIndex: 0, colIndex: 0 };
            sheet._endCell = { rowIndex: sheet.rowCount - 1, colIndex: sheet.colCount - 1 };
            this.selectRange(sheet._startCell, sheet._endCell);
            xlObj.element.find(".e-spreadsheetcolumnheader").addClass("e-sheetselected");            
            sheet._isSheetSelected = true;
        },

        refreshSelection: function (range) {
            var xlObj = this.XLObj, range = range ? range : xlObj.getSheet(xlObj.getActiveSheetIndex()).selectedRange,
                minr = range[0], minc = range[1], maxr = range[2], maxc = range[3];
            if (this._isRowSelected)
                this.selectRows(minr, maxr);
            else if (this._isColSelected)
                this.selectColumns(minc, maxc);
            else if (this._isSheetSelected)
                this.selectSheet();
            else
                this.selectRange({ rowIndex: minr, colIndex: minc }, { rowIndex: maxr, colIndex: maxc });
        },

        getSelectedCells: function (sheetIdx) {
            return this.XLObj._getContent(this.XLObj._getSheetIndex(sheetIdx))[0].querySelectorAll("td.e-selected");
        },

        _select: function (args, classname) {
            var elem = this.XLObj.getCell(args.rowIndex, args.colIndex);
            if (elem)
                elem.addClass(classname);
        },

        _selectRow: function (args, classname) {
            var elem = this.XLObj._getJSSheetRowHeaderContent(this.XLObj.getActiveSheetIndex()).find("td:eq(" + args + ")");
            if (elem)
                elem.addClass(classname);
        },

        _selectRows: function (startRIndex, endRIndex, classname) {
            var elem = this.XLObj._getJSSheetRowHeaderContent(this.XLObj.getActiveSheetIndex()).find("td");
            elem = (endRIndex !== startRIndex) ? elem.slice(startRIndex, endRIndex + 1) : elem.eq(endRIndex);
            if (elem)
                elem.addClass(classname);
        },


        _selectColumn: function (args, classname) {
            var elem = this.XLObj._getJSSheetHeader(this.XLObj.getActiveSheetIndex()).find("th:eq(" + args + ")");
            if (elem)
                elem.addClass(classname);
        },
        _selectColumns: function (startCIndex, endCIndex, classname) {
            var elem = this.XLObj._getJSSheetHeader(this.XLObj.getActiveSheetIndex()).find("th");
            elem = (endCIndex !== startCIndex) ? elem.slice(startCIndex, endCIndex + 1) : elem.eq(endCIndex);
            if (elem)
                elem.addClass(classname);
        },

        _focusBorder: function (startcell, endcell, classes) {  // consider
            var xlObj = this.XLObj, range = xlObj.swapRange([startcell.rowIndex, startcell.colIndex, endcell.rowIndex, endcell.colIndex]),
            minr = range[0], minc = range[1], maxr = range[2], maxc = range[3];
            minr - 1 >= 0 && xlObj.getRange(minr - 1, minc, minr - 1, maxc).addClass(classes[1]); // top, original minr -1 in 3rd args, minc in 2nd args                            
            xlObj.getRange(minr, maxc, maxr, maxc).addClass(classes[0]); // right
            xlObj.getRange(maxr, minc, maxr, maxc).addClass(classes[1]); // bottom
            minc - 1 >= 0 && xlObj.getRange(minr, minc - 1, maxr, minc - 1).addClass(classes[0]); // left, original is minc - 1 in 2nd args, maxr in 3rd args
        },

        _focusRange: function (startcell, endcell, border) {
            var xlObj = this.XLObj;
            xlObj.getFocusTopElem().removeClass().addClass("e-focustop " + border[0]),
            xlObj.getFocusRightElem().removeClass().addClass("e-focusright " + border[1]),
            xlObj.getFocusBottomElem().removeClass().addClass("e-focusbottom " + border[2]),
            xlObj.getFocusLeftElem().removeClass().addClass("e-focusleft " + border[3]);
            this._refreshBorder(xlObj.swapRange([startcell.rowIndex, startcell.colIndex, endcell.rowIndex, endcell.colIndex]));
        },

        _refreshBorder: function (range) {
            range = range ? range : this.XLObj.getSheet(this.XLObj.getActiveSheetIndex()).selectedRange;
            var type, time, hide = "e-hide", xlObj = this.XLObj,
                minr = range[0], minc = range[1], focusInfo = this._focusInfo(range), topElem = xlObj.getFocusTopElem().removeClass(hide),
                rightElem = xlObj.getFocusRightElem().removeClass(hide), bottomElem = xlObj.getFocusBottomElem().removeClass(hide), leftElem = xlObj.getFocusLeftElem().removeClass(hide),
                ttop = focusInfo.topLeft.top, tleft = focusInfo.topLeft.left, twid = (focusInfo.topRight.left + focusInfo.topRight.width) - focusInfo.topLeft.left,
                rtop = focusInfo.topRight.top, rleft = focusInfo.topRight.left + focusInfo.topRight.width, rhgt = (focusInfo.bottomRight.top + focusInfo.bottomRight.height) - focusInfo.topRight.top,
                btop = focusInfo.bottomLeft.top + focusInfo.bottomLeft.height, bleft = focusInfo.topLeft.left, bwid = (focusInfo.bottomRight.left + focusInfo.bottomRight.width) - focusInfo.bottomLeft.left,
               ltop = focusInfo.topLeft.top, lleft = focusInfo.topLeft.left, lhgt = (focusInfo.bottomLeft.top + focusInfo.bottomLeft.height) - focusInfo.topLeft.top;
            if (xlObj.model.selectionSettings.enableAnimation) {
                type = xlObj.model.selectionSettings.animationType;
                time = xlObj.model.selectionSettings.animationTime;
                minr ? topElem.animate({ top: ttop.toString(), left: tleft.toString(), width: twid.toString() }, time, type) : topElem.addClass(hide);
                rightElem.animate({ top: rtop.toString(), left: rleft.toString(), height: rhgt.toString() }, time, type);
                bottomElem.animate({ top: btop.toString(), left: bleft.toString(), width: bwid.toString() }, time, type);
                minc ? leftElem.removeClass(hide).animate({ top: ltop.toString(), left: lleft.toString(), height: lhgt.toString() }, time, type) : leftElem.addClass(hide);
            }
            else {
                minr ? topElem.css({ top: ttop, left: tleft, width: twid }) : topElem.addClass(hide);
                rightElem.css({ top: rtop, left: rleft, height: rhgt });
                bottomElem.css({ top: btop, left: bleft, width: bwid });
                minc ? leftElem.css({ top: ltop, left: lleft, height: lhgt }) : leftElem.addClass(hide);
            }
        },

        _focusInfo: function (range) {
            var obj = {}, xlObj = this.XLObj;
            obj.topLeft = xlObj._getCellInfo({ rowIndex: range[0], colIndex: range[1] });
            obj.topRight = xlObj._getCellInfo(xlObj._getMergedIdx(range[0], range[3]));
            obj.bottomLeft = xlObj._getCellInfo(xlObj._getMergedIdx(range[2], range[1]));
            obj.bottomRight = xlObj._getCellInfo(xlObj._getMergedIdx(range[2], range[3]));
            return obj;
        },

        _highlightHeader: function (range) {
            this._selectRows(range[0], range[2], "e-rowhighlight");
            this._selectColumns(range[1], range[3], "e-colhighlight");
        },

        _markHeaderSelection: function (range, type) {
            var xlObj = this.XLObj, i;
            range = xlObj.swapRange(range);
            i = range[0];
            if (type === "row") {
                while (i <= range[2]) {
                    this._selectRow(i, "e-rowselected");
                    i++;
                }
            }
            else {
                i = range[1];
                while (i <= range[3]) {
                    this._selectColumn(i, "e-colselected");
                    i++;
                }
            }
        },

        _getSelectedRange: function (startcell, endcell) {
            var i, range, arr = [], xlObj = this.XLObj;
            range = xlObj.swapRange([startcell.rowIndex, startcell.colIndex, endcell.rowIndex, endcell.colIndex]);
            while (range[0] <= range[2]) {
                i = range[1];
                while (i <= range[3]) {
                    arr.push({ rowIndex: range[0], colIndex: i });
                    i = i + 1;
                }
                range[0] = range[0] + 1;
            }
            return arr;
        },

        _processFormulaRange: function (border, ismousemove) {
            var alpharange, xlObj = this.XLObj, sindex = xlObj.getActiveSheetIndex(), scell = xlObj._dStartCell, ecell = xlObj._dEndCell, editcell = xlObj._getContent(sindex).find(".e-editedcell");
            this._focusBorder(scell, ecell, border[xlObj._ctrlKeyCount % 6]);
            xlObj.model.allowAutoFill && xlObj.XLDragFill.hideAutoFillElement();
            alpharange = xlObj._getAlphaRange(sindex, scell.rowIndex, scell.colIndex, ecell.rowIndex, ecell.colIndex);
            xlObj.model.allowAutoFill && xlObj.XLEdit._updateFormulaCellRange(alpharange);
            xlObj.XLEdit._isCellEdit ? xlObj.XLEdit._focusElements(editcell) : xlObj._getInputBox().focus();
        },


        _scrollCalculation: function (sheetIndex, target, status) {
            var interval, xlObj = this.XLObj, sheet, diff;
            if (xlObj.model.scrollSettings.allowVirtualScrolling)
                return;
            sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());
            xlObj.XLSelection._isEndReached = true;
            interval = setInterval(function (e) {
                var reachedEnd, hScroll = xlObj._getContent(sheetIndex).find("#hscrollBar").data("ejScrollBar"), vScroll = xlObj._getContent(sheetIndex).find("#vscrollBar").data("ejScrollBar");
                if (!xlObj.XLSelection._isEndReached) {
                    clearInterval(interval);
                    return;
                }
                if (status.position === "horizontal") {
                    if (status.action == "Increment") {
                        reachedEnd = Math.ceil(parseFloat(hScroll.element.find(".e-hhandle").css('left'))) + Math.ceil(parseFloat(hScroll.element.find(".e-hhandle").width())) >= hScroll.element.find(".e-hhandlespace").width() - 1;
                        (reachedEnd) ? xlObj.XLScroll._scrollX(sheetIndex) : hScroll.scroll(hScroll.value() + xlObj.model.columnWidth, true);
                        if (!xlObj.XLShape._shapeROStart && !xlObj.XLShape._shapeRSStart)
                            sheet._endCell.colIndex = sheet.colCount - 1;
                    }
                    else {
                        diff = hScroll.value() - xlObj.model.columnWidth;
                        (diff < 0) && (diff = 0);
                        (hScroll.value() >= 0) && hScroll.scroll(diff, true);
                        if (xlObj.XLSelection._aFillDownHandler)
                            sheet._endCell.colIndex = sheet._leftCol.idx;
                    }
                }
                else if (status.position === "vertical") {
                    if (status.action == "Increment") {
                        reachedEnd = Math.ceil(parseFloat(vScroll.element.find(".e-vhandle").css('top'))) + Math.ceil(parseFloat(vScroll.element.find(".e-vhandle").height())) >= vScroll.element.find(".e-vhandlespace").height() - 1;
                        (reachedEnd) ? xlObj.XLScroll._scrollY(sheetIndex) : vScroll.scroll(vScroll.value() + xlObj.model.rowHeight, true);
                        if (!xlObj.XLShape._shapeROStart && !xlObj.XLShape._shapeRSStart)
                            sheet._endCell.rowIndex = sheet.rowCount - 1;
                    }
                    else {
                        diff = vScroll.value() - xlObj.model.rowHeight;
                        (diff < 0) && (diff = 0);
                        (vScroll.value() > 0) && vScroll.scroll(diff, true);
                        if (!xlObj.XLShape._shapeROStart && !xlObj.XLShape._shapeRSStart) {
                            if (sheet._topRow.idx < 1)
                                return;
                            sheet._startCell = sheet._activeCell;
                            sheet._endCell.rowIndex = sheet._topRow.idx - 1;
                            sheet._endCell.colIndex = target.parentNode.cellIndex;  //?
                        }
                    }
                }
                if (!xlObj.XLShape._shapeROStart && !xlObj.XLShape._shapeRSStart) {
                    xlObj.XLSelection._aFillDownHandler ? xlObj.XLDragFill._selectAutoFillRange(null, sheet._endCell) : xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                    xlObj.model.allowAutoFill && xlObj.XLDragFill.positionAutoFillElement(xlObj.XLSelection._aFillDownHandler)
                }
            }, 200);
            target.addEventListener("mouseup", function (e) {
                clearInterval(interval);
            });
            document.addEventListener("mouseup", function (e) {
                clearInterval(interval);
            });
        },

        _cleanUp: function (border) {
            var i, cells, hide = "e-hide", xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            if(xlObj.model.allowAutoFill)
                xlObj.XLDragFill._autoFillCleanUp();
            cells = xlObj.getSheetElement(sheetIdx)[0].querySelectorAll(".e-selected,.e-rowselected,.e-colselected,.e-rowhighlight,.e-colhighlight,.e-sheetselected,.e-activecell");
            i = cells.length;
            while (i--)
                xlObj._removeClass(cells[i], "e-activecell|e-selected|e-rowhighlight|e-colhighlight|e-rowselected|e-colselected|e-ddcell|e-sheetselected");
            if (border) {
                xlObj.addClass(xlObj.getFocusTopElem()[0], hide);
                xlObj.addClass(xlObj.getFocusRightElem()[0], hide);
                xlObj.addClass(xlObj.getFocusBottomElem()[0], hide);
                xlObj.addClass(xlObj.getFocusLeftElem()[0], hide);
            }
            sheet._selectedCells = [];
        },

        _clearBorder: function (classes) {
            var xlObj = this.XLObj;
            xlObj._getContent(xlObj.getActiveSheetIndex()).find("td").removeClass(classes);
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.filter = function (obj) {
        this.XLObj = obj;
        this._isSearchEdit = false;
        this._isDecimal = false;
    };

    ej.spreadsheetFeatures.filter.prototype = {
        //Filtering
        _renderExcelFilter: function () {
            var xlObj = this.XLObj, model = { instance: xlObj, showSortOptions: true, maxFilterLimit: xlObj.model.sheets[xlObj.getActiveSheetIndex()].filterSettings.maxFilterChoices, filterHandler: ej.proxy(this._filterHandler, this), cancelHandler: ej.proxy(this._cancelHandler, this), customFilterHandler: ej.proxy(this._customFilterHandler, this) };
            xlObj._excelFilter = new ej.excelFilter(model);
        },

        _customFilterHandler: function () {
            var xlObj = this.XLObj;
            if (this._isDecimal) {
                $("#" + xlObj._id + "number_CustomValue1").ejNumericTextbox("option", { decimalPlaces: 2 });
                $("#" + xlObj._id + "number_CustomValue2").ejNumericTextbox("option", { decimalPlaces: 2 });
            }
        },

        _renderFilterDialogs: function () {
            var xlObj = this.XLObj;
            xlObj._excelFilter.renderDialog("string");
            xlObj._excelFilter.renderDialog("number");
            xlObj._excelFilter.renderDialog("date");
            xlObj.element.find(".e-excelfilter.e-dlgcontainer").hide();
        },

        _createSelection: function (sheetIdx, startrowIndex, endrowIndex, colIndex, filteredIndex) {
            var xlObj = this.XLObj, rows = xlObj.getRows(sheetIdx)[1], endRowIdx = 0, sheetData = xlObj._dataContainer.sheets[sheetIdx];
            for (var i = endrowIndex, len = rows.length; i < len; i++) {
				if(i in sheetData) {				
				    if (ej.isNullOrUndefined(sheetData[i].isHide) && ej.isNullOrUndefined(i, colIndex, "value2", sheetIdx) && i > endrowIndex) {
				        if (i <= startrowIndex)
				            i = i + (startrowIndex - i) + 1;
				        return { endRowIndex: i, filteredIndex: filteredIndex };
				    }
				    else if (!ej.isNullOrUndefined(sheetData[i].isHide) && !ej.isNullOrUndefined(i, colIndex, "value2", sheetIdx)) {
				        endRowIdx = i + 1;
				        filteredIndex.push(i);
				    }
				    else
				        endRowIdx = endrowIndex + 1;
				}
            }
            return { endRowIndex: endRowIdx, filteredIndex: filteredIndex };
        },

        _getSelectedRangeData: function (startCell, endCell, tableId, cellIndex) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), filteredRangeIndex = [], columnName, i, j, cellData,
                regx = xlObj._formatRegx, selCellColor = [], className, color, cells, celldatatemp, rangeData = [], container = xlObj._dataContainer, sheetData = container.sheets[sheetIdx];
            if (tableId === -1)
                sheet.filterSettings.range = xlObj._generateHeaderText(sheetIdx, startCell.colIndex + 1) + startCell.rowIndex + ":" + xlObj._generateHeaderText(sheetIdx, endCell.colIndex + 1) + (endCell.rowIndex + 1);
            if (sheet._multipleFiltering)
                endCell.colIndex = sheet._multifilterIdx[sheet._multifilterIdx.length - 1];
            for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                temp = {};
                for (j = startCell.colIndex; j <= endCell.colIndex; j++) {                   					
                    columnName = xlObj._generateHeaderText(sheetIdx, j + 1);
					if(i in sheetData && (ej.isNullOrUndefined(sheetData[i].isRHide) || ej.isNullOrUndefined(sheetData[i].isFHide))) { 
                        if (i in sheetData && j in sheetData[i]) {
                            cellData = sheetData[i][j];
							if(!ej.isNullOrUndefined(cellData.format) && !ej.isNullOrUndefined(container.hashCode[cellData.format].match(regx))){
								className = container.hashCode[cellData.format].match(regx)[0];
								color = xlObj._getColorFromFormat(className);
								cells = { id: j, background: color[0], foreground: color[1] };
								selCellColor.push(cells);
							}
                            if (ej.isNullOrUndefined(container.sharedData[cellData.value]))
                                temp[columnName] = "";
                            else
                                temp[columnName] = (xlObj._isDateTime(container.sharedData[cellData.value]) || xlObj.isFormula(container.sharedData[cellData.value])) ? cellData.value2 : container.sharedData[cellData.value];
                        }
                        else
                            temp[columnName] = "";
                    }
                }
                if (xlObj.getObjectLength(temp) > 0) {
                    rangeData.push(temp);
                    filteredRangeIndex.push(i);
                }
            }
            xlObj.model.sheets[sheetIdx]._selectedCellColors = selCellColor;
            xlObj.model.sheets[sheetIdx]._selectedRange = { startRow: startCell.rowIndex, endRow: endCell.rowIndex, dataSourceIndexes: filteredRangeIndex };
            xlObj.model.sheets[sheetIdx].filterSettings.filteredRange = rangeData;
        },

        _getColumnType: function (sheetIdx, startRow, endRow, cellIndex) {
            var cellType = "string", value;
            for (var i = startRow; i < endRow; i++) {
                value = this.XLObj.XLEdit.getPropertyValue(i, cellIndex, "value", sheetIdx);
                if (!$.isNumeric(value)) {
                    cellType = "string";
                    return cellType;
                }
                else {
                    if (!this._isDecimal)
                        this._isDecimal = ((value % 1) != 0);
                    cellType = "number";
                }
            }
            return cellType;
        },

        _getColumnByField: function (sheetIdx, field) {
            var xlObj = this.XLObj, column, sheet = xlObj.getSheet(sheetIdx);
            for (column = 0; column < sheet.columns.length; column++)
                if (sheet.columns[column]["field"] === field)
                    break;
            return column === sheet.columns.length ? null : sheet.columns[column];
        },

        _checkEmptyText: function (sheetIdx) {
            var xlObj = this.XLObj, selectedCell = xlObj.model.sheets[sheetIdx]._selectedCells, empty = false;
            for (var i = 0, len = selectedCell.length; i < len; i++) {
				if(ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(selectedCell[i].rowIndex, selectedCell[i].colIndex, "value", sheetIdx))) {
                    empty = true;
                    return true;
                }
            }
            if (!empty) {
                xlObj._showAlertDlg("Alert", "Alert", null, 440);
                return false;
            }
        },

        _getFilterIcon: function () {
            var filterIcon = ej.buildTag("span .e-icon", "", {});
            filterIcon.addClass("e-spanfilter e-filterspan e-ssfilter");
            return filterIcon;
        },

        filterByActiveCell: function () {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), value;
            if (xlObj.model.allowFiltering) {
                var activeCell = xlObj.model.sheets[sheetIdx]._activeCell, activeElem = xlObj.getCell(activeCell.rowIndex, activeCell.colIndex), selCells = xlObj.model.sheets[sheetIdx]._selectedCells;
                if (this._checkEmptyText(sheetIdx)) {
                    if (activeElem.find("span").is('.e-ssfiltered, .e-ssfilter, .e-ssfilter-asc, .e-ssfiltered-asc, .e-ssfilter-dsc, .e-ssfiltered-dsc'))
                        xlObj._showAlertDlg("Alert", "HeaderAlert", "", 440);
                    else {
                        value = xlObj.XLEdit.getPropertyValue(activeCell.rowIndex, activeCell.colIndex, "value", sheetIdx);
                        value = xlObj._isDateTime(value) ? xlObj.XLEdit.getPropertyValue(activeCell.rowIndex, activeCell.colIndex, "value2", sheetIdx) : value;
                        this._filterActiveCell(sheetIdx, activeCell, selCells, value);
                    }
                }
            }
        },

        _filterActiveCell: function (sheetIdx, activeCell, selCells, value) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx);
            sheet._activeCell = activeCell;
            sheet._selectedCells = selCells;
            this._filterByValue(sheetIdx, null, sheet.columns[activeCell.colIndex].field, "or", "equal", value);
            xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
        },

        _filterByValue: function (sheetIdx, range, columnName, pred, operator, value) {
            var xlObj = this.XLObj, selCells, rng, activeCell, filterColumn, fQMgr, query, obj, objectIndex = -1, i, len, fltrVal,
                arg = {}, tabRange, details, colIdx = xlObj.XLEdit.getColumnIndexByField(columnName), sheet = xlObj.getSheet(sheetIdx);
            if (ej.isNullOrUndefined(range))
                selCells = sheet._selectedCells;
            else {
                rng = xlObj.getRangeIndices(range);
                selCells = xlObj.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
            }
            activeCell = selCells[0];
            if (!ej.isNullOrUndefined(range)) {
				fltrVal = xlObj.XLEdit.getPropertyValue(activeCell.rowIndex, activeCell.colIndex, "value2", sheetIdx);
				if(ej.isNullOrUndefined(fltrVal) || fltrVal !== value) {
                    for (i = rng[0]; i < rng[2]; i++) {
						fltrVal = xlObj.XLEdit.getPropertyValue(i, colIdx, "value2", sheetIdx);
						if(!ej.isNullOrUndefined(fltrVal) && fltrVal === value){
                            activeCell = { rowIndex: i, colIndex: colIdx };
                            break;
                        }
                    }
                }
            }
            details = { currentFilteringColumn: columnName, action: "filter", predicated: pred, requestType: "filterbycell", reqType: "filter", filteredValue: value, selCells: $.extend(true, [], selCells), activeCell: activeCell };
            tabRange = this._applyFilterByValue(sheetIdx, activeCell, selCells, value);
            xlObj._excelFilter._predicates[sheetIdx] = {};
            filterColumn = { field: columnName, value: value, predicate: pred, operator: operator, matchcase: false };
            arg.currentFilterCollection = value;
            xlObj._excelFilter._predicates[sheetIdx][columnName] = ej.Predicate(columnName, operator, value);
            sheet.filterSettings.filteredColumns.push(filterColumn);
            fQMgr = ej.DataManager(sheet.filterSettings.filteredColumns);
            query = new ej.Query().where("field", ej.FilterOperators.equal, columnName);
            obj = fQMgr.executeLocal(query);
            for (i = 0, len = obj.length; i < len; i++) {
                objectIndex = $.inArray(obj[i], sheet.filterSettings.filteredColumns);
                (objectIndex > -1) && sheet.filterSettings.filteredColumns.splice(objectIndex, 1);
            }
            sheet.filterSettings.filteredColumns.push(filterColumn);
            details.filteredcolumns = tabRange.filteredColumns = sheet.filterSettings.filteredColumns;
            tabRange.predicate = xlObj._excelFilter._predicates[sheetIdx];
            details.tableID = tabRange.tableID;
			details.filteredCells = tabRange.filteredCells;
            xlObj.XLSelection._refreshBorder();
            xlObj._processBindings(details) && (sheet.filterSettings.filteredColumns = temp);
        },


        _applyFilterByValue: function (sheetIdx, activeCell, selCells, value) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), cellIndex = [], endCell, startRowIdx, endRowIdx, filterIcon, filtrIconsCol, tableID, actCellCls = xlObj.getCell(activeCell.rowIndex, activeCell.colIndex)[0].className,
                tab = sheet.filterSettings.tableRange, filteredCells = [], tabRange, tabIdx, exists = false, activeObj, i, len, isVirtualScroll = xlObj.model.scrollSettings.allowVirtualScrolling, filterColl = xlObj.getSheet(sheetIdx)._filterColl, filterCollObj;
            tableID = xlObj._getTableId(actCellCls[0].className);
			//tableID = xlObj.XLEdit.getPropertyValue(activeCell.rowIndex, activeCell.colIndex, "tableID", sheetIdx);
            for (i = 0, len = tab.length; i < len; i++) {
                if (tab[i].tableID === tableID) {
                    tabIdx = i;
                    break;
                }
            }
            if (!ej.isNullOrUndefined(tabIdx) && ((tab[tabIdx].tableID === tableID) && tab[tabIdx].selectedCellIndexes.indexOf(activeCell.colIndex) !== -1) && ((tab[i].startRow <= activeCell.rowIndex) && (tab[i].endRow >= activeCell.rowIndex))) {
                tabRange = tab[tabIdx];
                sheet.filterSettings.filteredRange = tab[tabIdx].filteredRange;
                sheet._selectedRange = tab[tabIdx].selectedRange;
            }
            else {
                sheet._multipleFiltering = false;
                sheet._multifilterIdx = [];
				xlObj._dupDetails = true;
                this.clearFilter("remove");
				xlObj._dupDetails = false;
				var rowKeys = xlObj.getObjectKeys(filterColl), colKeys, cell;
				for(i = rowKeys.length - 1; i >= 0; i--){
					colKeys = xlObj.getObjectKeys(filterColl[i]);
					for(j = colKeys.length - 1; j >= 0; j--){
						if(xlObj._isRowViewable(sheetIdx, rowKeys[i])) {
							cell = xlObj.getCell(rowKeys[i], colKeys[j]);
							cell.find(".e-filterspan").remove();
							cell.removeClass("e-filterhdr");
						}
						delete filterColl[rowKeys[i]][colKeys[j]];
					}
				}
                if (selCells.length < 2 || (selCells[0].rowIndex === selCells[selCells.length - 1].rowIndex)) {
                    startRowIdx = -1;
                    endRowIdx = selCells[selCells.length - 1].rowIndex;
                    for (var i = selCells[0].rowIndex; i >= 0; i--) {
						if(ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(i, selCells[0].colIndex, "value2"))) {
                            startRowIdx = i;
                            break;
                        }
                    }
                    startRowIdx = (startRowIdx === -1) ? 1 : startRowIdx + 2; // For text cell is act as a header
                    endRowIdx = this._createSelection(sheetIdx, startRowIdx, endRowIdx, selCells[0].colIndex, []).endRowIndex - 1;
                    endCell = { rowIndex: endRowIdx, colIndex: activeCell.colIndex };
                    this._getSelectedRangeData({ rowIndex: startRowIdx, colIndex: activeCell.colIndex }, endCell, -1, activeCell.colIndex);
                }
                else {
                    startRowIdx = activeCell.rowIndex;
                    endRowIdx = this._createSelection(sheetIdx, startRowIdx, selCells[selCells.length - 1].rowIndex, selCells[0].colIndex, []).endRowIndex - 1;
                    endCell = { rowIndex: endRowIdx, colIndex: activeCell.colIndex };
                    this._getSelectedRangeData({ rowIndex: startRowIdx, colIndex: activeCell.colIndex }, endCell, -1, activeCell.colIndex);
                }
                for (i = selCells[0].colIndex, len = selCells[selCells.length - 1].colIndex; i <= len; i++) {
                    cellIndex.push(i);
                    filterIcon = this._getFilterIcon();
					startRowIdx = (startRowIdx === 0) ? 1 : startRowIdx;
					activeObj = {rowIndex: startRowIdx - 1, colIndex: i};
                    this._applyFilterIcon(activeObj, filterIcon);
					filteredCells.push(activeObj);
                }
                (cellIndex.length > 1) && (sheet._multipleFiltering = true);
                sheet._multifilterIdx = cellIndex;
                sheet._selectedCellIndexes = cellIndex;
                for (i = 0, len = tab.length; i < len; i++) {
                    if (tab[i].tableID === tableID) {
                        tabIdx = i;
                        exists = true;
                        break;
                    }
                }
                if (!exists) {
                    sheet.filterSettings.tableRange.push({
                        startRow: startRowIdx, endRow: endCell.rowIndex, tableID: -1, multiFiltering: xlObj.model.sheets[sheetIdx]._multipleFiltering, multifilterIdx: xlObj.model.sheets[sheetIdx]._multifilterIdx, selectedCellIndexes: xlObj.model.sheets[sheetIdx]._multifilterIdx, selectedRange: xlObj.model.sheets[sheetIdx]._selectedRange, filteredRange: xlObj.model.sheets[sheetIdx].filterSettings.filteredRange, hiddenIdx: []
                    });
                    tabRange = tab[tab.length - 1];
                }
                else {
                    tabRange = tab[tabIdx];
                    tabRange.startRow = activeCell.rowIndex;
                    tabRange.endRow = endCell.rowIndex;
                    tabRange.multiFiltering = xlObj.model.sheets[sheetIdx]._multipleFiltering;
                    tabRange.multifilterIdx = xlObj.model.sheets[sheetIdx]._multifilterIdx;
                    tabRange.selectedCellIndexes = xlObj.model.sheets[sheetIdx]._multifilterIdx;
                    tabRange.selectedRange = xlObj.model.sheets[sheetIdx]._selectedRange;
                    tabRange.filteredRange = xlObj.model.sheets[sheetIdx].filterSettings.filteredRange;
                    tabRange.hiddenIdx = [];
                    tabRange.tableID = tableID;
                }
				tabRange.filteredCells = filteredCells;
            }
            return tabRange;
        },

        clearFilter: function (status) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), columnName, fltrdcol, fltrcol, sheet = xlObj.getSheet(sheetIdx), filterColl = sheet._filterColl,
                tblRange = sheet.filterSettings.tableRange, tabIdx = -1, i, len, filterRange;          
			var rowKeys = xlObj.getObjectKeys(filterColl), colKeys, cell;
			for(i = rowKeys.length - 1; i >= 0; i--){
				colKeys = xlObj.getObjectKeys(filterColl[i]);
				for(j = colKeys.length - 1; j >= 0; j--){
					if(xlObj.XLEdit.getPropertyValue(rowKeys[i], colKeys[j], "tableID", sheetIdx) === -1) {
						fltrdCol = filterColl[rowKeys[i]][colKeys[j]];
						if(fltrdCol.status.indexOf("e-ssfiltered") > -1) {
							columnName = xlObj.model.sheets[sheetIdx].columns[colKeys[j]].field;
							delete xlObj._excelFilter._predicates[sheetIdx][columnName];
							this._clearFilterColumn(columnName, "", "", "Or", "", "", -1);
						}
						if(status === "remove") {
							delete filterColl[rowKeys[i]][colKeys[j]];
							cell = xlObj.getCell(rowKeys[i], colKeys[j]);
							if(cell.length) {
								$.isNumeric(cell.text()) && cell.addClass(xlObj._rAlign);
								cell.find(".e-filterspan").remove();
								cell.removeClass("e-filterhdr");
							}
						}
					}
				}
			}
            if (status === "remove") {
                for (i = 0; i < tblRange.length; i++) {
                    if (tblRange[i].tableID === -1) {
                        tabIdx = i;
                        break;
                    }
                }
                (tabIdx > -1) && (filterRange = tblRange.splice(tabIdx, 1)[0]);
                xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
				if(!xlObj._isUndoRedo && !xlObj._dupDetails) {
					var details = {sheetIndex: sheetIdx, operation: "remove-filter", process: "apply-filter", requestType: "filter", reqType: "filter", filteredCells: filteredCells, fRange: xlObj.getSheet(sheetIdx).filterSettings.range, filterRange: filterRange  };
					xlObj._completeAction(details);
				}
            }
        },

        filter: function (range) {
            var xlObj = this.XLObj;
            if (xlObj.model.allowFiltering) {
                var sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), applyFltr = false, activeCell = sheet._activeCell, tableId, colIdx,
					tabRange = sheet.filterSettings.tableRange, tabIdx, i, len, stRow, fltrStr;
                sheet._multipleFiltering = false;
                sheet._multifilterIdx = [];
				tableId = xlObj.XLEdit.getPropertyValue(activeCell.rowIndex, activeCell.colIndex, "tableID", sheetIdx);
				for(i = 0, len = tabRange.length; i < len; i++) {
					if(tabRange[i].tableID === -1)
						applyFltr = true;					
					tabIdx = i;
				}
				if(applyFltr) {
					xlObj._dupDetails = true;
					this.clearFilter("remove");
					xlObj._dupDetails = false;
					sheet.filterSettings.filteredRange = [];
					sheet._selectedRange = {};
					sheet._selectedCellIndexes = [];
					sheet.filterSettings.filteredColumns = [];
					xlObj._excelFilter._predicates[sheetIdx] = {};
				}
                if(!ej.isNullOrUndefined(tableId) && tableId > -1) {
					stRow = tableRange[tabIdx].startRow;
					colIdx = tabRange[tabIdx].multifilterIdx;
                    for (i = 0, len = colIdx.length; i < colLen; i++) {
						fltrStr = this._getFilterHeader(sheetIdx, stRow, colIdx[i]);
						if(!ej.isNullOrUndefined(fltrStr) && fltrStr.status.indexOf("e-ssfiltered") > -1)
							this._clearFilterColumn(sheet.columns[i].field, "", "", "", "", "", tableId);
                    }
                    this._clearFilterTableIcon(sheetIdx, tableId);
                    applyFltr = true;
				}
                (!applyFltr) && this._filterCellRange(sheetIdx, range);
                xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
            }
        },

		_isRowVisible: function(sheetIdx, rowIdx) {
		    var container = this.XLObj._dataContainer.sheets[sheetIdx];
		    if (rowIdx in container)
		        if (0 in container[rowIdx] && (container[rowIdx][0].isRHide || container[rowIdx][0].isFRHide))
					return false;
				else
					return true;
		},
		
		_applyFilterIcon: function (activeObj, filterIcon) {
		    var xlObj = this.XLObj, ralign = this.XLObj._rAlign, sheetIdx = xlObj.getActiveSheetIndex(), rowIdx = activeObj.rowIndex, colIdx = activeObj.colIndex, 
			    cols = xlObj._getContent(sheetIdx).find(".e-content col:eq(" + colIdx + ")"), isViewed = xlObj._isRowViewable(sheetIdx, rowIdx), filterColl = xlObj.getSheet(sheetIdx)._filterColl;
			if(isViewed) {
				activeEle = xlObj.getCell(rowIdx, colIdx, sheetIdx);
				activeEle[0].insertBefore(filterIcon[0], activeEle[0].childNodes[activeEle[0].childNodes.length - 1])
				var contWidth = xlObj.model.allowResizing && xlObj.XLResize._getContentWidth(colIdx, sheetIdx, activeEle.parent());
				if (cols.width() < contWidth + 19) {
				    (xlObj.getFrozenColumns(sheetIdx) >= colIdx) && xlObj._refreshFreezeColumns(sheetIdx, colIdx, (contWidth + 19) - cols.width(), "resize");
					cols.width(contWidth + 19);
					xlObj._getJSSheetHeader(sheetIdx).find("col:eq(" + colIdx + ")").width(contWidth + 19);
					xlObj.getSheet(sheetIdx).columnsWidthCollection[colIdx] = contWidth + 19;
					xlObj.XLScroll._refreshScroller(sheetIdx, "initial", "horizontal");
				}
				activeEle.addClass("e-filterhdr");
			}
			xlObj.XLEdit._updateDataContainer({rowIndex: rowIdx, colIndex: colIdx}, {isFilterHeader: true, filterState: "filter"}, sheetIdx)
			if (xlObj.isNumber(xlObj.model.allowFiltering && xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2", sheetIdx)))
                isViewed && xlObj._removeClass(activeEle[0], ralign);
			if(!(rowIdx in filterColl))
			    filterColl[rowIdx] = {};
			if (!(colIdx in filterColl[rowIdx]))
			    filterColl[rowIdx][colIdx] = {};
			filterColl[rowIdx][colIdx].status = "e-ssfilter";
			filterColl[rowIdx][colIdx].isTable = (filterIcon[0].className.indexOf("e-ft") > -1);
        },

        _postionFilterIcon: function (activeElem, filterIcon) {
            var xlObj = this.XLObj, actCell = activeElem || xlObj.getActiveCellElem(), fltrIcon = filterIcon || actCell.find(".e-filterspan"), contWidth, contHeight;
            if (xlObj.model.allowResizing){
                contWidth = xlObj.XLResize._getContentWidth(actCell.index(), null, actCell.parent());
                contHeight = xlObj.XLResize._getContentHeight(null, actCell);
            }
            fltrIcon.css("margin-top", (xlObj._hasClass(actCell[0], "e-sswraptext") && contWidth > actCell.width()) ? Math.floor(contHeight - 25) : ""); //25px - filter span height + padding
        },

        _filterCellRange: function (sheetIdx, range) {
            var xlObj = this.XLObj, selCells, addFilter = false, exists, tableIdx, tableRange, colCount = 0, cellVal, isVisible,
                filterIcon, prevText, i, j, k, len, ln, rng, endRowIdx, startCell, endCell, activeEle, tabRange, sheet = xlObj.getSheet(sheetIdx),
                cellIndex = [], rowIconIdx = -1, startRowIndex = 0, endRowIndex = 0, selFilterPosn = false, isMiddle = false, rowIndx,
                filterCellIndex = [], filteredCells, isEmpty = false, details = { sheetIndex: sheetIdx, reqType: "filter", operation: "apply-filter" },
                multifilterIdx = sheet._multifilterIdx, rows = xlObj.getRows(sheetIdx)[1], filteredRangeIndex = [];
            if (ej.isNullOrUndefined(range))
                selCells = sheet._selectedCells;
            else {
                rng = xlObj.getRangeIndices(range);
                selCells = xlObj.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
            }
            for (i = selCells[0].colIndex, len = selCells[selCells.length - 1].colIndex; i <= len ; i++) {
                cellIndex.push(i);
                filterIcon = this._getFilterIcon();
                rowIndx = selCells[0].rowIndex;
                if (selCells.length < 2) {
                    multifilterIdx.push(i);
					cellVal = xlObj.XLEdit.getPropertyValue(rowIndx, i, "value2", sheetIdx);
                    //Single Cell Filtering
                    if (ej.isNullOrUndefined(cellVal)) {
                        for (j = rowIndx; j >= 0 ; j--) {
							isVisibe = xlObj._isRowVisible(sheetIdx, j);
                            if (isVisibe && !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(j, i, "value2", sheetIdx))) {
                                isMiddle = true;
                                break;
							}
                        }
                        if (!isMiddle) {
                            //Active Element is Empty in the Top
                            for (j = rowIndx; j < (rowIndx + 2) ; j++) {
								isVisibe = xlObj._isRowVisible(sheetIdx, j);
								if (isVisibe && !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(j, i, "value2", sheetIdx))) {
                                    this._applyFilterIcon({rowIndex: j, colIndex: i}, filterIcon);
                                    addFilter = true;
                                    startRowIndex = endRowIndex = j;
                                    break;
                                }
                            }
                        }
                        //Active Element is Empty in the Bottom
                        if (!addFilter) {
                            prevText = false;
                            for (j = rowIndx; j > (rowIndx - 2) ; j--) {
                                if (j > -1) {
									isVisibe = this._isRowVisible(sheetIdx, j);
									if (isVisibe && ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(j, i, "value2", sheetIdx))) {                                  
                                        if (j > 0 && xlObj.XLEdit.getPropertyValue(j - 1, i, "value2", sheetIdx).length === 0)          
                                            prevText = true;
                                    }
                                }
                            }
                            if (!prevText) {
                                for (j = rowIndx - 2; j >= 0 ; j--) {
									isVisibe = this._isRowVisible(sheetIdx, j);
									if (isVisibe && (ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(j, i, "value2", sheetIdx)) || j === 0)) {
                                        k = (j === 0) ? j : j + 1;
                                        this._applyFilterIcon({rowIndex: k, colIndex: i}, filterIcon);
                                        addFilter = true;
                                        startRowIndex = endRowIndex = k;
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    else {
                        //Active Element is Middle of the Column
                        for (j = rowIndx; j >= 0 ; j--) {
							isVisibe = this._isRowVisible(sheetIdx, j);
						    if (isVisibe && ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(j, i, "value2", sheetIdx))) {          
                                k = j + 1;
                                this._applyFilterIcon({rowIndex: j + 1, colIndex: i}, filterIcon);
                                addFilter = true;
                                startRowIndex = endRowIndex = k;
                                break;
                            }
                        }
                        if (!addFilter) {
                            this._applyFilterIcon({rowIndex: 0, colIndex: i}, filterIcon);
                            addFilter = true;
                            startRowIndex = endRowIndex = 0;
                        }
                    }
                    if (!addFilter) {
                        //display an alert message to select the cell which is having a value
                        xlObj._showAlertDlg("Alert", "Alert");
                    }
                }
                else {
                    //Multi Cell Filtering
                    if (!selFilterPosn || cellIndex.length > 0)
                        for (j = rowIndx, ln = selCells[selCells.length - 1].rowIndex; j <= ln; j++) {
                            isVisibe = this._isRowVisible(sheetIdx, j);
							if (isVisibe && !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(j, i, "value2", sheetIdx))) {          
                                (rowIconIdx === -1 || rowIconIdx > j) && (rowIconIdx = j);
                                selFilterPosn = true;
                                addFilter = true;
                                isEmpty = true;
                            }
                            (rowIconIdx < j && colCount === 0) && filteredRangeIndex.push(j);
                        }
                    if (isEmpty) {
                        filterCellIndex.push(i);
                        sheet._multifilterIdx.push(i);
                        isEmpty = false;
                    }
                }
                colCount++;
            }
            (cellIndex.length > 1) && (sheet._multipleFiltering = true);
            filteredCells = [];
            if (addFilter) {
                if (selCells.length > 1) {
                    for (i = 0, len = filterCellIndex.length; i < len; i++) {
                        filterIcon = this._getFilterIcon();
                        this._applyFilterIcon({rowIndex: rowIconIdx, colIndex: filterCellIndex[i]}, filterIcon);
                        filteredCells.push({rowIndex: rowIconIdx, colIndex: filterCellIndex[i]});
                    }
                    startCell = { rowIndex: rowIconIdx + 1, colIndex: selCells[0].colIndex };
                    endCell = { rowIndex: selCells[selCells.length - 1].rowIndex };
                    sheet._selectedRange = { startRow: startCell.rowIndex, endRow: endCell.rowIndex, dataSourceIndexes: filteredRangeIndex };
                    this._getSelectedRangeData(startCell, selCells[selCells.length - 1], -1, startCell.colIndex);
                    sheet._selectedCellIndexes = cellIndex;
                }
                else {
                    if (selCells.length === 1)
                        filteredCells.push({rowIndex: startRowIndex, colIndex: cellIndex[0]});
                    var selection = this._createSelection(sheetIdx, startRowIndex + 1, endRowIndex, cellIndex[0], []);
                    endRowIdx = selection.endRowIndex - 1;
                    startCell = { rowIndex: startRowIndex + 1, colIndex: selCells[0].colIndex };
                    endCell = { rowIndex: endRowIdx, colIndex: selCells[0].colIndex };
                    sheet._selectedRange = { startRow: startCell.rowIndex, endRow: endCell.rowIndex, dataSourceIndexes: selection.filteredIndex };
                    this._getSelectedRangeData(startCell, endCell, -1, startCell.colIndex);
                    sheet._selectedCellIndexes.push(selCells[0].colIndex);
                }
                details = {
                    sheetIndex: sheetIdx,
                    filteredCells: filteredCells,
                    reqType: "filter",
                    operation: "apply-filter",
                    tableID: -1
                }
                tableRange = sheet.filterSettings.tableRange;
                for (i = 0, len = tableRange.length; i < len; i++) {
                    if (tableRange[i].tableID === -1) {
                        exists = true;
                        tableIdx = i;
                    }
                }
                if (!exists)
                    sheet.filterSettings.tableRange.push({ startRow: startCell.rowIndex, endRow: endCell.rowIndex, tableID: -1, multiFiltering: sheet._multipleFiltering, multifilterIdx: sheet._multifilterIdx, selectedCellIndexes: sheet._multifilterIdx, selectedRange: sheet._selectedRange, filteredRange: sheet.filterSettings.filteredRange, filteredColumns: sheet.filterSettings.filteredColumns, predicate: {}, hiddenIdx: [], fltrdIdxes: [] });
                else {
                    tabRange = sheet.filterSettings.tableRange[tableIdx];
                    tabRange.startRow = startCell.rowIndex;
                    tabRange.endRow = endCell.rowIndex;
                    tabRange.multiFiltering = sheet._multipleFiltering;
                    tabRange.multifilterIdx = sheet._multifilterIdx;
                    tabRange.selectedCellIndexes = sheet._multifilterIdx;
                    tabRange.selectedRange = sheet._selectedRange;
                    tabRange.filteredRange = sheet.filterSettings.filteredRange;
                    tabRange.filteredColumns = sheet.filterSettings.filteredColumns;
                    tabRange.predicate = xlObj._excelFilter._predicates[sheetIdx];
                    tabRange.hiddenIdx = [];
                    tabRange.flterdIdxes = [];
                }
                if (!sheet._importRendered) {
                    xlObj.XLSelection._refreshBorder();
                    xlObj._completeAction(details);
                }
            }
            else 
                xlObj._showAlertDlg("Alert", "Alert", "", 440);
            
        },

        _filterClickHandler: function (e) {
            var xlObj = this.XLObj, $target = $(e.target), sheetIdx = xlObj.getActiveSheetIndex(), isFiltered = false, endRowIdx, columnName, columns, colType, i, len, startIdx, endIdx,
                actCell, fltrVal, tableId, selectedRange, tableRange, tableFilteredColumn = [], tablePredicate = {}, docWidth, dlgWidth, xPos, startCell, endCell, tabIdx, selection, isVirtualScroll = xlObj.model.scrollSettings.allowVirtualScrolling;
            columnName = xlObj.model.sheets[sheetIdx].columns[e.target.parentNode.cellIndex].field;
            this._isDecimal = false;
            if (xlObj.model.allowLockCell && xlObj.getSheet(sheetIdx)._isSheetProtected)
                return;
            ($target[0].className.indexOf("e-ssfiltered") > -1) && (isFiltered = true);
            if ($target[0].className.indexOf("e-ssfilter") > -1) {
                tableRange = xlObj.model.sheets[sheetIdx].filterSettings.tableRange;
                if ($target[0].className.indexOf("e-ft") !== -1)
                    tableId = parseInt($target[0].className.substr($target[0].className.indexOf("e-ft") + 4, $target[0].className.length));
                else
                    tableId = -1;
                for (i = 0, len = tableRange.length; i < len; i++) {
                    if (tableRange[i].tableID === tableId) {
                        tabIdx = i;
                        if (tableRange[i].multiFiltering) {
                            startIdx = tableRange[i].multifilterIdx[0];
                            endIdx = tableRange[i].multifilterIdx[tableRange[i].multifilterIdx.length - 1];
                        }
                        else
                            startIdx = endIdx = e.target.parentNode.cellIndex;
                        startCell = { rowIndex: tableRange[i].startRow, colIndex: startIdx };
                        endCell = { rowIndex: tableRange[i].endRow, colIndex: endIdx };
                        xlObj.model.sheets[sheetIdx]._multipleFiltering = tableRange[i].multiFiltering;
                        xlObj.model.sheets[sheetIdx]._multifilterIdx = tableRange[i].multifilterIdx;
                        selectedRange = xlObj.model.sheets[sheetIdx]._selectedRange = tableRange[i].selectedRange;
                        xlObj.model.sheets[sheetIdx].filterSettings.filteredRange = tableRange[i].filteredRange;
                        xlObj.model.sheets[sheetIdx]._selectedCellIndexes = tableRange[i].selectedCellIndexes;
                    }
                }
                if (tableId === -1) {
                    selection = this._createSelection(sheetIdx, selectedRange.startRow, selectedRange.endRow, endIdx, selectedRange.dataSourceIndexes);
                    endRowIdx = selection.endRowIndex - 1;
                    if (selectedRange.endRow != endRowIdx) {
                        for (var i = selectedRange.endRow + 1; i <= endRowIdx; i++) {
                            fltrVal = { field: columnName, value: xlObj.XLEdit.getPropertyValue(i, endIdx, "value2", sheetIdx), matchcase: false, operator: "equal", predicate: "or" };
                            xlObj.model.sheets[sheetIdx].filterSettings.filteredColumns.push(fltrVal);
                            tableRange[tabIdx].filteredColumns.push(fltrVal);
                            tableRange[tabIdx].selectedRange.endRow = endRowIdx;
                        }
                    }
                    endCell = { rowIndex: endRowIdx, colIndex: endIdx };
                    tableRange[tabIdx].selectedRange = xlObj.model.sheets[sheetIdx]._selectedRange = { startRow: startCell.rowIndex, endRow: endCell.rowIndex, dataSourceIndexes: selection.filteredIndex };
                }
                this._getSelectedRangeData(startCell, endCell, tableId, e.target.parentNode.cellIndex);
                columns = xlObj.model.sheets[sheetIdx].columns;
                colType = this._getColumnType(sheetIdx, selectedRange.startRow, selectedRange.endRow, e.target.parentNode.cellIndex);
                for (i = 0, len = columns.length; i < len; i++) {
                    if (columns[i].field === columnName) {
                        xlObj._$colType = colType;
                        xlObj._$curFieldName = columns[i].field;
                        break;
                    }
                }
                var fltrDlg = xlObj.element.find(".e-excelfilter.e-dlgcontainer"), dlgHeight, docHeight = xlObj.element.height(), yPos;
                docWidth = xlObj.element.width();
                dlgHeight = fltrDlg.height(); dlgWidth = fltrDlg.width();
                xPos = e.clientX; yPos = e.clientY;
                (docWidth - e.clientX <= dlgWidth) && (xPos = docWidth - dlgWidth);
                (docHeight - e.clientY <= dlgHeight) && (yPos = docHeight - (dlgHeight + 20));
                tableRange = xlObj.model.sheets[sheetIdx].filterSettings.tableRange;
                for (i = 0, len = tableRange.length; i < len; i++) {
                    if (tableRange[i].tableID === tableId) {
                        tableFilteredColumn = tableRange[i].filteredColumns;
                        tablePredicate = tableRange[i].predicate;
                    }
                }
                xlObj._excelFilter._predicates[sheetIdx] = tablePredicate;
                this._isSearchEdit = true;
                if (xlObj.model.sheets[sheetIdx].filterSettings.filteredRange == null)
                    xlObj._excelFilter.openXFDialog({ field: columnName, dataSource: xlObj.model.sheets[sheetIdx].dataSource, position: { X: xPos, Y: yPos }, type: xlObj._$colType, format: xlObj._$colFormat, filteredColumns: tableFilteredColumn, sortedColumns: [], key: sheetIdx, tableID: tableId, selectedColors: xlObj.model.sheets[sheetIdx]._selectedCellColors, isFiltered: isFiltered, blank: "", enableColor: true, filteredByColor: xlObj.model.sheets[sheetIdx]._filteredByColor });
                else
                    xlObj._excelFilter.openXFDialog({ field: columnName, dataSource: xlObj.model.sheets[sheetIdx].filterSettings.filteredRange, position: { X: xPos, Y: yPos }, type: xlObj._$colType, format: xlObj._$colFormat, filteredColumns: tableFilteredColumn, sortedColumns: [], key: sheetIdx, tableID: tableId, selectedColors: xlObj.model.sheets[sheetIdx]._selectedCellColors, isFiltered: isFiltered, blank: "", enableColor: true, filteredByColor: xlObj.model.sheets[sheetIdx]._filteredByColor });
            }
        },

        _cancelHandler: function () {
            this._isSearchEdit = false;
        },

        _filterHandler: function (args) {
            var i, len, tab, avble = false, tabIdx = -1, colIdx, range, arg = {}, fQMgr, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), query, obj, objectIndex, filterObj, tableFilteredColumn = [], tablePredicate = {}, cell, isSelAllChkBox, isSrchVal;
            arg.requestType = (args.action === "sorting" || args.action === "filterbycolor") ? args.action : "filtering";
            tab = xlObj.model.sheets[sheetIdx].filterSettings.tableRange;
            xlObj.XLFilter._isSearchEdit = false;
            for (i = 0, len = tab.length; i < len; i++) {
                if (tab[i].tableID === args.tableID) {
                    $.extend(true, tableFilteredColumn, tab[i].filteredColumns);
                    tablePredicate = tab[i].predicate;
                    avble = true;
                    tabIdx = i;
                }
            }
            arg.tableID = args.tableID;
            if (args.action === "filtering" || args.action === "clearfiltering") {
                fQMgr = ej.DataManager(tableFilteredColumn);
                query = new ej.Query().where("field", ej.FilterOperators.equal, args.fieldName);
                obj = fQMgr.executeLocal(query);
                arg.filteredColumns = tableFilteredColumn;
                for (i = 0, len = obj.length; i < len; i++) {
                    objectIndex = $.inArray(obj[i], tableFilteredColumn);
                    (objectIndex >= 0) && tableFilteredColumn.splice(objectIndex, 1);
                }
                if (args.action === "clearfiltering")
                    delete xlObj._excelFilter._predicates[sheetIdx][args.fieldName];
                else
                    ej.merge(tableFilteredColumn, args.filterCollection);
                tablePredicate = xlObj._excelFilter._predicates[sheetIdx];
            }
            if (avble) {
                tab[tabIdx].filteredColumns = tableFilteredColumn;
                tab[tabIdx].predicate = tablePredicate;
            }
            if (args.action === "filtering" || args.action === "clearfiltering") {
                arg.currentFilteringColumn = args.fieldName;
                arg.ejpredicate = args.ejpredicate;
                arg.filterCollection = $.extend(true, [], args.filterCollection);
                arg.action = args.action;
                arg.reqType = "filter";
                arg.sheetIndex = sheetIdx;
                arg.filteredcolumns = tableFilteredColumn;
                arg.predicate = tablePredicate;
            }
            else if (args.action === "sorting") {
                arg.requestType = arg.action = args.action;
                colIdx = xlObj.XLEdit.getColumnIndexByField(args.sortDetails.field);
                range = xlObj.model.sheets[sheetIdx].filterSettings.tableRange[tabIdx].selectedRange;
                var colIndexs = xlObj.model.sheets[sheetIdx].filterSettings.tableRange[tabIdx].selectedCellIndexes;
				xlObj.XLSort._isSortByFilter = true;
                xlObj.XLSort.sortByRange(xlObj.swapRange([range.startRow, colIndexs[0], range.endRow, colIndexs[colIndexs.length - 1]]), args.sortDetails.field, args.sortDetails.direction);
				this._changeSortIcon(xlObj.getCell(range.startRow - 1, colIdx), args.sortDetails.direction);
                for (var i = 0, len = colIndexs.length; i < len; i++) {
                    cell = xlObj.getCell(range.startRow - 1, colIndexs[i]).find('.e-filterspan')[0];
                    if (colIndexs[i] != colIdx && (xlObj._hasClass(cell, "e-ssfilter-asc") || xlObj._hasClass(cell, "e-ssfilter-dsc"))) {
                        xlObj._hasClass(cell, "e-ssfilter-asc") && xlObj._removeClass(cell, "e-ssfilter-asc");
                        xlObj._hasClass(cell, "e-ssfilter-dsc") && xlObj._removeClass(cell, "e-ssfilter-dsc");
                        xlObj.addClass(cell, "e-ssfilter");
                    }
                }
            }
            else {
                arg.requestType = arg.action = args.action;
                arg.range = xlObj.model.sheets[sheetIdx]._selectedRange;
                var colors;
                if (args.action === "sortbycolor") {
                    arg.sortColumn = args.sortDetails.field;
                    arg.operation = args.sortDetails.operation;
                    colors = args.sortDetails.color.replace("rgb(", "").replace(")", "").split(",");
                }
                else {
                    if (xlObj._isUndoRedo) {
                        arg.filterColumn = args.columnName;
                        arg.operation = args.operation;
                    }
                    else {
                        arg.filterColumn = args.filterDetails.field;
                        arg.operation = args.filterDetails.operation;
                        colors = args.filterDetails.color.replace("rgb(", "").replace(")", "").split(",");
                    }
                    arg.tableID = args.tableID;
                    xlObj.model.sheets[sheetIdx]._filteredByColor = arg.filterColumn;
                }
                var r, g, b;
                if (!xlObj._isUndoRedo) {
                    r = colors[0];
                    g = colors[1];
                    b = colors[2];
                }
                if (arg.operation === "sortbgcolor" || arg.operation === "filterbgcolor")
                    arg.bgcolor = xlObj._isUndoRedo ? args.bgcolor : this._convertrgbtohex(parseInt(r), parseInt(g), parseInt(b));
                else
                    arg.fgcolor = xlObj._isUndoRedo ? args.fgcolor : this._convertrgbtohex(parseInt(r), parseInt(g), parseInt(b));
            }
            if (args.action !== "sorting")
                xlObj._processBindings(arg) && (tableFilteredColumn = temp);
            xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
            xlObj.XLSelection._refreshBorder();
        },

        _convertrgbtohex: function (r, g, b) {
            return "#" + ((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1);
        },

        _clearFilterColumn: function (fieldName, filterOperator, filterValue, predicate, matchcase, actualFilterValue, tableId) {
            var tabRange, args = {}, i, len, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), firstLoop = false, filterObject = {}, dataManger, query, obj, objectIndex,
                tab = xlObj.model.sheets[sheetIdx].filterSettings.tableRange, filteredcolumns = [];
            args.requestType = ej.Grid.Actions.Filtering;
            args.currentFilterObject = [];
            (!$.isArray(filterOperator)) && (filterOperator = $.makeArray(filterOperator));
            (!$.isArray(filterValue)) && (filterValue = $.makeArray(filterValue));
            this._currentFilterColumn = this._getColumnByField(sheetIdx, fieldName);
            for (var idx = 0, ln = filterOperator.length; idx < ln; idx++) {
                filterObject = { field: fieldName, operator: filterOperator[idx], value: filterValue[idx], matchcase: matchcase, predicate: predicate, actualFilterValue: actualFilterValue };
                if (!firstLoop) {
                    for (i = 0, len = tab.length; i < len; i++) {
                        if (tab[i].tableID === tableId) {
                            tabRange = tab[i];
                            $.extend(true, filteredcolumns, tab[i].filteredColumns);
                            xlObj._excelFilter._predicates[sheetIdx] = tab[i].predicate;
                            xlObj.model.sheets[sheetIdx].filterSettings.filteredRange = xlObj.model.sheets[sheetIdx].filterSettings.tableRange[i].filteredRange;
                            xlObj.model.sheets[sheetIdx]._selectedCellIndexes = tab[i].selectedCellIndexes;
                            xlObj.model.sheets[sheetIdx]._selectedRange = tab[i].selectedRange;
                            xlObj.model.sheets[sheetIdx]._multifilterIdx = tab[i].multifilterIdx;
                        }
                    }
                    dataManger = ej.DataManager(filteredcolumns);
                    query = new ej.Query().where("field", ej.FilterOperators.equal, filterObject.field);
                    obj = dataManger.executeLocal(query);
                    for (i = 0, len = obj.length; i < len; i++) {
                        objectIndex = $.inArray(obj[i], filteredcolumns);
                        if (objectIndex !== -1)
                            filteredcolumns.splice(objectIndex, 1);
                    }
                }
            }
			if(xlObj._isUndoRedo && xlObj.isUndefined(tabRange))
				return;
            firstLoop = true;
            delete xlObj._excelFilter._predicates[sheetIdx][fieldName];
            tabRange.filteredColumns = filteredcolumns;
            tabRange.predicate = xlObj._excelFilter._predicates[sheetIdx];
            args.currentFilterObject.push(filterObject);
            args.filterCollection = filteredcolumns;
            args.currentFilteringColumn = fieldName;
            args.filteredcolumns = filteredcolumns;
            args.action = "clearfiltering";
            args.reqType = "filter";
            args.tableID = tableId;
            if (xlObj._processBindings(args)) {
                xlObj.model.sheets[sheetIdx].filterSettings.filteredColumns.reverse().splice(0, filterOperator.length);
                xlObj.model.sheets[sheetIdx].filterSettings.filteredColumns.reverse();
            }
            xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
            this.XLObj.XLSelection._refreshBorder();
        },

        _getFilterHeader: function(sheetIdx, rowIdx, colIdx) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), filterColl = sheet._filterColl;
            if(!ej.isNullOrUndefined(filterColl[rowIdx]) && !ej.isNullOrUndefined(filterColl[rowIdx][colIdx]))
                return filterColl[rowIdx][colIdx];
        },

        //Format As Table
        _filterTable: function (sheetIdx, id) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), cellIndex = [], alfltr = false, multiFltr = false, tabRange, filterIcon, filClass,
                filterHdr;
            sheet._multipleFiltering = false;
            sheet._multifilterIdx = [];
            var selectedCells = $.extend(true, [], sheet._selectedCells), endCell = selectedCells[selectedCells.length - 1], startCell = selectedCells[0];
            for (var i = startCell.colIndex; i <= endCell.colIndex; i++) {
                filterHdr = this._getFilterHeader(sheetIdx, startCell.rowIndex, startCell.colIndex);
                if(!ej.isNullOrUndefined(filterHdr) && filterHdr.status.indexOf("e-ssfilter") > -1)
                    alfltr = true;
                else {
                    filterIcon = this._getFilterIcon();
                    filClass = "e-ft" + id;
                    filterIcon.addClass(filClass);
                    this._applyFilterIcon({rowIndex: startCell.rowIndex, colIndex: i}, filterIcon);
                    cellIndex.push(i);
					sheet._multifilterIdx.push(i);
                }
            }
            if (!alfltr) {
                if (cellIndex.length > 1) {
                    sheet._multipleFiltering = true;
                    multiFltr = true;
                }
                startCell.rowIndex = startCell.rowIndex + 1;
                this._getSelectedRangeData(startCell, endCell, id, startCell.colIndex);
                if (!xlObj._isFATResize)
                    sheet.filterSettings.tableRange.push({ startRow: startCell.rowIndex, endRow: endCell.rowIndex, tableID: id, multiFiltering: multiFltr, multifilterIdx: cellIndex, selectedCellIndexes: cellIndex, selectedRange: xlObj.model.sheets[sheetIdx]._selectedRange, filteredRange: xlObj.model.sheets[sheetIdx].filterSettings.filteredRange, filteredColumns: xlObj.model.sheets[sheetIdx].filterSettings.filteredColumns, predicate: {}, hiddenIdx: [] });
                else {
                    tabRange = sheet.filterSettings.tableRange;
                    if (tabRange.length)
                        for (var i = 0, len = tabRange.length; i < len; i++) {
                            if (tabRange[i].tableID === id) {
                                tabRange[i].endRow = endCell.rowIndex;
                                break;
                            }
                        }
                    xlObj._isFATResize = false;
                }
            }
        },

        _clearFilterTable: function (sheetIdx, tableId, rmFltr) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), tableRange = sheet.filterSettings.tableRange, fltrCell, cell, j, ln, field,
				multiIdx, startRow, isViewable;
            for (var i = 0, len = tableRange.length; i < len; i++) {
                if (tableRange[i].tableID === tableId) {
                    sheet._selectedRange = tableRange[i].selectedRange;
                    multiIdx = tableRange[i].multifilterIdx;
					startRow = tableRange[i].startRow;
                    for (j = 0, ln = multiIdx.length; j < ln; j++) {
                        field = xlObj.model.sheets[sheetIdx].columns[multiIdx[j]].field;
                        this._clearFilterColumn(field, "", "", "Or", "", "", tableRange[i].tableID);
                        if (!rmFltr) {
							if(xlObj._isRowViewable(sheetIdx, startRow - 1)) {
								fltrCell = xlObj.getCell(startRow - 1, multiIdx[j]);
								cell = fltrCell.find('.e-filterspan');
								$(cell).remove();
								fltrCell.removeClass("e-filterhdr");
							}
                        }
                    }
                    if (!rmFltr) {
                        tableRange.splice(i, 1);
                        len = tableRange.length;
                    }
                }
            }
        },

        _clearFilterTableIcon: function (sheetIdx, tableId) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), tableRange = sheet.filterSettings.tableRange, multiIdx, startRow, j, ln, cell;
            for (var i = 0, len = tableRange.length; i < len; i++) {
                if (tableRange[i].tableID === tableId) {
                    sheet._selectedRange = tableRange[i].selectedRange;
                    sheet.filterSettings.filteredRange = tableRange[i].filteredRange;
                    multiIdx = tableRange[i].multifilterIdx;
                    startRow = tableRange[i].startRow;
                    for (j = 0, ln = multiIdx.length; j < ln; j++) {
						//Need to Update in Container ( filter Icon is hidden or not )
						if(xlObj._isRowViewable(sheetIdx, startRow - 1)){
							cell = xlObj.getCell(startRow - 1, multiIdx[j]).find('.e-filterspan');
							cell.is(":visible") ? cell.hide() : cell.show();
						}
                    }
                }
            }
        },

        _insertFilterIcon: function (i, j, tabRange, fRow, fApplied) { //fRow- Filtered Row, fApplied - filter Applied
            var filterIcon, filClass;
            if (fApplied && i === fRow) {
                filterIcon = this._getFilterIcon();
                filClass = "e-ft" + tabRange.tableID;
                filterIcon.addClass(filClass);
                tabRange.multifilterIdx.push(tabRange.multifilterIdx[tabRange.multifilterIdx.length - 1] + 1);
                this._applyFilterIcon({rowIndex: i, colInde: j - 1}, filterIcon);
            }
        },

        _changeFilterIcon: function (cellObj, status, fltrState) {
            var fSpan, activeCell, fltrStr, isViewed = false, xlObj = this.XLObj, rowIdx = cellObj.rowIndex, colIdx = cellObj.colIndex, ralign = xlObj._rAlign, sheetIdx = xlObj.getActiveSheetIndex(), filterColl;
            filterColl = this._getFilterHeader(sheetIdx, rowIdx, colIdx);
            fltrStr = filterColl.status;
            if (xlObj._isRowViewable(sheetIdx, rowIdx)) {
                activeCell =  xlObj.getCell(rowIdx, colIdx);
                fSpan = activeCell.find('.e-filterspan')[0];
			    activeCell.removeClass(ralign);
				isViewed = true;
			}
			if (status === "filter") {
			    xlObj.XLEdit._updateDataContainer({ rowIndex: rowIdx, colIndex: colIdx }, { filterState: fltrState.replace("filter", "filtered")});
				xlObj.getSheet(sheetIdx)._filterColl[rowIdx][colIdx].status = fltrStr.replace("e-ssfilter", "e-ssfiltered");
				if(isViewed)
					fSpan.className = fSpan.className.replace("e-ssfilter", "e-ssfiltered");
			}
			else {
			    xlObj.XLEdit._updateDataContainer({ rowIndex: rowIdx, colIndex: colIdx }, { filterState: fltrState.replace("filtered", "filter") });
			    xlObj.getSheet(sheetIdx)._filterColl[rowIdx][colIdx].status = fltrStr.replace("e-ssfiltered", "e-ssfilter");
				if(isViewed)
					fSpan.className = fSpan.className.replace("e-ssfiltered", "e-ssfilter");
			}			
        },

        _changeSortIcon: function (cellObj, status) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), fSpan, idx, fltrStr, fStr, isViewed = false, filterColl = xlObj.getSheet(sheetIdx)._filterColl;
			fltrStr = filterColl[cellObj.rowIndex][cellObj.colIndex];
			idx = (fltrStr.indexOf("e-ssfiltered") > -1) ? 12 : 10;
			fStr = fltrStr.substr(0, idx);
			if(xlObj._isRowViewable(sheetIdx, cellObj.rowIndex)) {
				var fSpan = xlObj.getCell(cellObj.rowIndex, cellObj.colIndex).find('.e-filterspan')[0];
				fltrStr = fSpan.className;
				idx = fltrStr.indexOf("e-ssfilter") + idx;
				isViewed = true;
			}
            if (status === "ascending") {
				filterColl[cellObj.rowIndex][cellObj.colIndex]["status"] = fStr + "-asc";
				if(isViewed)
					fSpan.className = fltrStr.substr(0, idx) + "-asc" + fltrStr.substr(idx + 1, fltrStr.length);
			}
			else {
				filterColl[cellObj.rowIndex][cellObj.colIndex]["status"] = fStr + "-dsc";
				if(isViewed)
					fSpan.className = fltrStr.substr(0, idx) + "-asc" + fltrStr.substr(idx + 1, fltrStr.length);
			}
        },

        _extendFilterRange: function (tid, range) {//extend the filter range
            var xlObj = this.XLObj, dataSrcIdx, tabRange = xlObj.getSheet(xlObj.getActiveSheetIndex()).filterSettings.tableRange;
            if (tabRange.length) {
                for (var i = 0, len = tabRange.length; i < len; i++) {
                    if (tabRange[i].tableID == tid) {
                        if (tabRange[i].endRow < range[2]) {
                            dataSrcIdx = tabRange[i].selectedRange.dataSourceIndexes;
                            tabRange[i].endRow = tabRange[i].selectedRange.endRow = range[2];
                            if (dataSrcIdx[dataSrcIdx.length - 1] < tabRange[i].endRow) {
                                for (var j = dataSrcIdx[dataSrcIdx.length - 1], len = tabRange[i].endRow; j < len; j++)
                                    dataSrcIdx.push(j + 1);
                            }
                        }
                        break;
                    }
                }
            }
        },

        _checkFilteredCells: function (sheetIdx) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), selCells = sheet._selectedCells, cell, tabRegx = new RegExp('\\b' + "e-table" + '.*?\\b', 'g'), tabId, tables = [-1],
				tabRng = sheet.filterSettings.tableRange, selRng = sheet.selectedRange, isApplied = false, clrTbl = [],
				tblObj, tabObj2, tabMngr = sheet._tableManager;
            for (var i = 0, len = selCells.length; i < len; i++) {
                tabId = xlObj.XLEdit.getPropertyValue(selCells[i].rowIndex, selCells[i].colIndex, "tableID", sheetIdx);
				if(!ej.isNullOrUndefined(tabId))
                    (tables.indexOf(tabId) > -1) && tables.push(tabId);
            }
            for (var i = 0, len = tables.length; i < len; i++) {
                for (var j = 0, ln = tabRng.length; j < ln; j++) {
                    if (tabRng[j].tableID === tables[i]) {
                        if (tabRng[j].startRow === selRng[0] && tabRng[j].multifilterIdx[0] === selRng[1] && tabRng[j].endRow === selRng[2] && tabRng[j].multifilterIdx[tabRng[j].multifilterIdx.length] === selRng[3]) {
                            isApplied = true;
                            tblObj = $.extend(true, {}, tabMngr[tables[i]]);
                            tabObj2 = $.extednd(true, {}, tabRng[j]);
                            clrTbl.push({ tableID: tables[i], tabObj: tblObj, tabObj2: tabObj2 });
                        }
                        else
                            isApplied = false;
                        break;
                    }
                }
            }
            tabDetails = { isApplied: isApplied, clrTable: clrTbl };
            return tabDetails;
        },

        _moveFilteredCells: function (sheetIdx, tabDetails) {
            var clrTbl = tabDetails.clrTable, fName, cell, xlObj = this.XLObj;
            for (var i = 0, len = clrTbl.length; i < len; i++) {
                if (clrTbl[i].tableID > -1) {
                    xlObj.XLFormat.removeTable(clrTbl[i].tableID);
                    xlObj.XLFormat.createTable(clrTbl[i].tableObj, newRng)
                }
                else {
                    tabObj2 = clrTbl[i].tabObj2;
                    for (j = tabObj2.multifilterIdx[0], len = tabObj2.multifilterIdx[tabObj2.multifilterIdx.length - 1]; j <= len; j++) {
                        fName = xlObj._generateHeaderText(sheetIdx, tabObj2.multifilterIdx[j]);
                        delete xlObj._excelFilter._predicates[sheetIdx][fName];
                        xlObj.XLFilter._clearFilterColumn(fName, "", "", "Or", null, null, clrTbl[i].tableID);
						if(xlObj._isRowViewable(sheetIdx, tabObj2.startRow - 1)) {
							cell = xlObj.getCell(tabObj2.startRow - 1, j, sheetIdx);
							cell.find("span.e-ssfilter").remove();
							cell.addClass(this._rAlign);
						}
                    }
                }
            }
        },

        _updateFilterIcons: function (stRow, endRow, colIdx, action) {
            var xlObj = this.XLObj, fltrState, cellObj, numFltr = $("#" + xlObj._id + "numberSelectAll").ejCheckBox("instance"), isNumFltr = numFltr ? false : numFltr.model.checked,
                strFltr = $("#" + xlObj._id + "stringSelectAll").ejCheckBox("instance"), isStrFltr = strFltr ? false : strFltr.model.checked;
            for (var i = stRow; i <= endRow; i++) {
                cellObj = { rowIndex: i, colIndex: colIdx };
                fltrState = xlObj.XLEdit.getPropertyValue(i, colIdx, "filterState");
                if (fltrState.indexOf("filtered") > -1 && (action == "clearfiltering" || (isNumFltr || isStrFltr))) 
                    this._changeFilterIcon(cellObj, "filtered", fltrState);
                else if (fltrState.indexOf("filtered") < 0 && action != "clearfiltering")
                    this._changeFilterIcon(cellObj, "filter", fltrState);
                break;
            }
        },

    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.ribbon = function (obj) {
        this.XLObj = obj;
        this._appTabCollection = null;
        this._tabCollection = [];
        this._contextualTabCollection = [];
        this._smallBtnHeight = 25;
        this._mediumBtnHeight = 35;
        this._bigBtnHeight = 75;
        this._splitBtnHeight = 75;
        this._dropDownHeight = 25;
        this._ribbonState = true;
        this._isSetModel = false;
        this._isDesignTab = true;
        this._isDirtySelect = false;
        this._hasTitle = "";
        this._pageLaySptBtnIds = ["sizesbtn"];
        this._dataBtnIds = ["sortAtoZ", "sortZtoA", "filter", "clearfilter", "validationbtn"];
        this._dataSptBtnIds = ["validationsbtn"];
        this._toggleBtnIds = ["bold", "italic", "underline", "line-through", "left", "right", "center", "top", "middle", "bottom", "wraptext", "formatpainter"];
        this._homeBtnIds = ["pastebtn", "increasefontsize", "decreasefontsize", "increment", "decrement", "undo", "redo", "accounting", "usegrouping", "percentage", "cut", "copy"];
        this._homeSptBtnIds = ["pastesbtn", "border", "cFormat", "formatastable", "cellstyles", "find", "clear", "merge", "autosum"];
        this._protectStateBtnIds = ["cut", "copy", "find", "clear", "paste", "undo", "redo"];
        this._insertBtnIds = ["pivotTable", "table", "hyperlink", "pictures"];
        this._insertSptBtnIds = ["columnchart", "barchart", "stockchart", "linechart", "areachart", "combochart", "piechart", "scatterchart"],
        this._pageLayBtnIds = ["sizebtn", "print", "printselected"];
        this._pageLayChckBxIds = ["Vheading", "Vgridlines"];
        this._ddIds = ["cellformat", "fontsize", "font-family"];
        this._cpIds = ["background-color", "color"];
        this._splitBtnIds = ["border", "paste", "cFormat", "formatastable", "cellstyles", "find", "clear", "sortfilter"];
        this._pasteMenuData = [
                 { id: "PasteSpecial", text: obj._getLocStr("PasteSpecial"), parentId: null },
                 { id: "PasteValues", text: obj._getLocStr("PasteValues"), parentId: null }
        ];
        this._fontFamily = [{ value: "1", text: "Angsana New" }, { value: "2", text: "Arial" }, { value: "3", text: "Arial Black" }, { value: "4", text: "Batang" }, { value: "5", text: "Book Antiqua" }, { value: "6", text: "Browallia New" }, { value: "7", text: "Calibri" }, { value: "8", text: "Cambria" }, { value: "9", text: "Candara" }, { value: "10", text: "Century" }, { value: "11", text: "Comic Sans MS" }, { value: "12", text: "Consolas" }, { value: "13", text: "Constantia" }, { value: "14", text: "Corbel" }, { value: "15", text: "Cordia New" }, { value: "16", text: "Courier" }, { value: "17", text: "Courier New" }, { value: "18", text: "DilleniaUPC" }, { value: "19", text: "Dotum" }, { value: "20", text: "FangSong" }, { value: "21", text: "Garamond" }, { value: "22", text: "Georgia" }, { value: "23", text: "Gulim" }, { value: "24", text: "GungSuh" }, { value: "25", text: "KaiTi" }, { value: "26", text: "JasmineUPC" }, { value: "27", text: "Malgun Gothic" }, { value: "28", text: "Mangal" }, { value: "29", text: "Meiryo" }, { value: "30", text: "Microsoft JhengHei" }, { value: "31", text: "Microsoft YaHei" }, { value: "32", text: "MingLiu" }, { value: "33", text: "MingLiU_HKSCS" }, { value: "34", text: "MS Gothic" }, { value: "35", text: "MS Mincho" }, { value: "36", text: "MS PGothic" }, { value: "37", text: "MS PMincho" }, { value: "38", text: "PMingliU" }, { value: "39", text: "PMingLiU-ExtB" }, { value: "40", text: "SimHei" }, { value: "41", text: "SimSun" }, { value: "42", text: "SimSun-ExtB" }, { value: "43", text: "Tahoma" }, { value: "44", text: "Times New Roman" }, { value: "45", text: "Trebuchet MS" }, { value: "46", text: "Verdana" }, { value: "47", text: "Yu Gothic" }, { value: "48", text: "Yu Mincho" }];
        this._fontSize = [{ value: "8pt", text: "8" }, { value: "9pt", text: "9" }, { value: "10pt", text: "10" }, { value: "11pt", text: "11" }, { value: "12pt", text: "12" }, { value: "14pt", text: "14" }, { value: "16pt", text: "16" }, { value: "18pt", text: "18" }, { value: "20pt", text: "20" }, { value: "22pt", text: "22" }, { value: "24pt", text: "24" }, { value: "26pt", text: "26" }, { value: "28pt", text: "28" }, { value: "36pt", text: "36" }, { value: "48pt", text: "48" }, { value: "72pt", text: "72" }];
        this._mergeMenuData = [
                { id: "MergeAndCenter", text: obj._getLocStr("MergeAndCenter"), parentId: null, sprite: "e-icon e-ss-mergecenter" },
                { id: "MergeAcross", text: obj._getLocStr("MergeAcross"), parentId: null, sprite: "e-icon e-ss-mergeacross" },
                { id: "MergeCells", text: obj._getLocStr("MergeCells"), parentId: null, sprite: "e-icon e-ss-merge" },
                { id: "UnmergeCells", text: obj._getLocStr("UnmergeCells"), parentId: null, sprite: "e-icon e-ss-unmerge" }
        ];
        this._sortFltrMenuData = [
                { id: 'SortAtoZ', text: this.XLObj._getLocStr('SortAtoZ'), parentId: null, sprite: 'e-icon e-ssdrop-sortatoz' },
                { id: 'SortZtoA', text: this.XLObj._getLocStr('SortZtoA'), parentId: null, sprite: 'e-icon e-ssdrop-sortztoa' },
                { id: 'Filter', text: this.XLObj._getLocStr('Filter'), parentId: null, sprite: 'e-icon e-ssdrop-filter' },
                { id: 'ClearFilter', text: this.XLObj._getLocStr('ClearFilter'), parentId: null, sprite: 'e-icon e-ssdrop-clearfilter' }
        ];
        this._findMenuData = [
                { id: "Find", text: obj._getLocStr("Find") + "...", parentId: null, sprite: "e-icon e-ss-find" },
                { id: "Replace", text: obj._getLocStr("Replace") + "...", parentId: null, sprite: "e-icon e-ss-replace" },
                { id: "GoTo", text: obj._getLocStr("GoTo") + "...", parentId: null, sprite: "e-icon e-ss-goto" },
                { id: "GoToSpecial", text: obj._getLocStr("GoTo") + " " + obj._getLocStr("Special") + "...", parentId: null },
                { id: "Formulas", text: obj._getLocStr("Formulas"), parentId: null },
                { id: "Comments", text: obj._getLocStr("Comments"), parentId: null },
                { id: "CFormat", text: obj._getLocStr("ConditionalFormat"), parentId: null },
                { id: "Constants", text: obj._getLocStr("Constants"), parentId: null },
                { id: "DataValidation", text: obj._getLocStr("DataValidation"), parentId: null }
        ];
        this._valMenuData = [
                { id: "Validation", text: obj._getLocStr("DataValidation") + "...", parentId: null, sprite: "e-icon e-ss-validation" },
                { id: "HighlightVal", text: obj._getLocStr("HighlightVal"), parentId: null, sprite: "e-icon e-ss-hlval" },
                { id: "ClearHLVal", text: obj._getLocStr("ClearFormats"), parentId: null, sprite: "e-icon e-ss-clearval" },
                { id: "ClearVal", text: obj._getLocStr("ClearVal"), parentId: null, sprite: "e-icon e-ss-clearval" }
        ];
        this._pageSizes = [
                { id: "A4", text: obj._getLocStr("A4"), parentId: null, sprite: "e-icon e-ss-A4size" },
                { id: "A3", text: obj._getLocStr("A3"), parentId: null, sprite: "e-icon e-ss-A3size" },
                { id: "Letter", text: obj._getLocStr("Letter"), parentId: null, sprite: "e-icon e-ss-lettersize" }
        ];
        this._viewMenuData = [
                { id: "freezePanes", text: obj._getLocStr("freezePanes"), parentId: null, sprite: "e-icon e-ss-frzpane" },
                { id: "FreezeRow", text: obj._getLocStr("FreezeRow"), parentId: null, sprite: "e-icon e-ss-frzrow" },
                { id: "FreezeColumn", text: obj._getLocStr("FreezeColumn"), parentId: null, sprite: "e-icon e-ss-frzcln" }
        ];
        this._insertMenuData = [
                { id: 'InsertCells', text: obj._getLocStr('inscells') + "...", parentId: null, sprite: 'e-icon e-ssr-inscell' },
                { id: 'InsertSheetRows', text: obj._getLocStr('insrows'), parentId: null, sprite: 'e-icon e-ssr-insrow' },
                { id: 'InsertSheetColumns', text: obj._getLocStr('inscols'), parentId: null, sprite: 'e-icon e-ssr-inscol' },
                { id: 'InsertSheet', text: obj._getLocStr('inssheet'), parentId: null, sprite: 'e-icon e-ssr-inssheet' }
        ];
        this._resetPicture = [
                { id: "resetpicture", text: obj._getLocStr("ResetPicture"), parentId: null, sprite: "e-icon e-ss-resetpicture" },
                { id: "resetsize", text: obj._getLocStr("ResetSize"), parentId: null, sprite: "e-icon e-ss-resetpicture" }
        ];
        var borderstyle = obj._getLocStr("BorderStyles").split("/");
        this._borderMenuData = [
              { id: "bottomborder", text: obj._getLocStr("BottomBorder"), parentId: null, sprite: "e-icon e-ss-bottomborder" },
              { id: "topborder", text: obj._getLocStr("TopBorder"), parentId: null, sprite: "e-icon e-ss-topborder" },
              { id: "leftborder", text: obj._getLocStr("LeftBorder"), parentId: null, sprite: "e-icon e-ss-leftborder" },
              { id: "rightborder", text: obj._getLocStr("RightBorder"), parentId: null, sprite: "e-icon e-ss-rightborder" },
              { id: "noborder", text: obj._getLocStr("NoBorder"), parentId: null, sprite: "e-icon e-ss-noborder" },
              { id: "allborder", text: obj._getLocStr("AllBorder"), parentId: null, sprite: "e-icon e-ss-allborder" },
              { id: "outsideborder", text: obj._getLocStr("OutsideBorder"), parentId: null, sprite: "e-icon e-ss-outsideborder" },
              { id: "thickboxborder", text: obj._getLocStr("ThickBoxBorder"), parentId: null, sprite: "e-icon e-ss-thickboxborder" },
              { id: "thickbottomborder", text: obj._getLocStr("ThickBottomBorder"), parentId: null, sprite: "e-icon e-ss-thickbottomborder" },
              { id: "topandbottomborder", text: obj._getLocStr("TopandBottomBorder"), parentId: null, sprite: "e-icon e-ss-topandbottomborder" },
              { id: "topandthickbottomborder", text: obj._getLocStr("TopandThickBottomBorder"), parentId: null, sprite: "e-icon e-ss-topandthickbottomborder" },
              { id: "drawborder", text: obj._getLocStr("DrawBorder"), parentId: null, sprite: "e-icon e-ss-drawborder" },
              { id: "drawbordergrid", text: obj._getLocStr("DrawBorderGrid"), parentId: null, sprite: "e-icon e-ss-drawbordergrid" },
              { id: obj._id + "_bordercolor", text: obj._getLocStr("BorderColor"), parentId: null, sprite: "e-iconcolor" },
              { id: obj._id + "_Ribbon_bordercolor", text: "", parentId: obj._id + "_bordercolor" },
              { id: obj._id + "_Ribbon_borderstyle", text: obj._getLocStr("BorderStyle"), parentId: null },
              { id: "solid", text: borderstyle[0], parentId: obj._id + "_Ribbon_borderstyle" },
              { id: "dashed", text: borderstyle[1], parentId: obj._id + "_Ribbon_borderstyle" },
              { id: "dotted", text: borderstyle[2], parentId: obj._id + "_Ribbon_borderstyle" },
        ];
        this._cFormatMenuData = [
             { id: obj._id + "_HLCellRules", text: "Highlight Cell Rules", parentId: null, sprite: "e-icon e-ss-hlcellrules" },
             { id: "greaterthan", text: obj._getLocStr("GreaterThan") + "...", parentId: obj._id + "_HLCellRules", sprite: "e-icon e-ss-greaterthan" },
             { id: "lessthan", text: obj._getLocStr("LessThan") + "...", parentId: obj._id + "_HLCellRules", sprite: "e-icon e-ss-lessthan" },
             { id: "between", text: obj._getLocStr("Between") + "...", parentId: obj._id + "_HLCellRules", sprite: "e-icon e-ss-between" },
             { id: "equalto", text: obj._getLocStr("EqualTo") + "...", parentId: obj._id + "_HLCellRules", sprite: "e-icon e-ss-equalto" },
             { id: "textcontains", text: obj._getLocStr("TextthatContains") + "...", parentId: obj._id + "_HLCellRules", sprite: "e-icon e-ss-textcontains" },
             { id: "dateoccur", text: obj._getLocStr("DateOccurring") + "...", parentId: obj._id + "_HLCellRules", sprite: "e-icon e-ss-dateoccur" },
             { id: obj._id + "_ClearRules", text: obj._getLocStr('ClearRules'), parentId: null, sprite: "e-icon e-ss-clearrules" },
             { id: "clearselected", text: obj._getLocStr('ClearRulesfromSelected'), parentId: obj._id + "_ClearRules" },
             { id: "clearall", text: obj._getLocStr('ClearRulesfromEntireSheets'), parentId: obj._id + "_ClearRules" },
        ];
        this._textAlign = [{ value: "1", text: "Left" }, { value: "2", text: "Center" }, { value: "3", text: "Right" }];
        this._verticalAlign = [{ value: "1", text: "Top" }, { value: "2", text: "Middle" }, { value: "3", text: "Bottom" }];
        this._actions = [{ value: "Greater", text: "greater than", }, { value: "GreaterOrEqual", text: "greater or equal to" }, { value: "Less", text: "less than" }, { value: "LessOrEqual", text: "less than or equal to" }, { value: "Equal", text: "equal to" }, { value: "NotEqual", text: "not equal to" }, { value: "Between", text: "between" }, { value: "NotBetween", text: "not between" }];
        this._types = [{ value: "number", text: "Number", }, { value: "decimal", text: "Decimal" }, { value: "date", text: "Date" }, { value: "time", text: "Time" }, { value: "text", text: "TextLength" }, { value: "list", text: "List" }];
        this._cFormatData = { greaterthan: ["GTTitle", "GTContent"], lessthan: ["LTTitle", "LTContent"], between: ["BWTitle", "BWContent"], equalto: ["EQTitle", "EQContent"], textcontains: ["ContainsTitle", "ContainsContent"], dateoccur: ["DateTitle", "DateContent"] };
        this._styles = [{ value: "redft", text: "Light Red Fill with Dark Red Text", }, { value: "yellowft", text: "Yellow Fill with Dark Yellow Text" }, { value: "greenft", text: "Green Fill with Dark Green Text" }, { value: "redf", text: "Red Fill" }, { value: "redt", text: "Red Text" }];
        this._currentCFormat = "";
        this._addrList = [];
        this._within = [{ value: "sheet", text: obj._getLocStr("Sheet") }, { value: "workbook", text: obj._getLocStr("Workbook") }];
        this._sType = [{ value: "columns", text: obj._getLocStr("Columns") }, { value: "rows", text: obj._getLocStr("Rows") }];
        this._vType = [{ value: "value", text: obj._getLocStr("Value"), }, { value: "formula", text: obj._getLocStr("Formula") }, { value: "comment", text: obj._getLocStr("Comments") }];
        this._direction = [{ value: "down", text: obj._getLocStr("Down") }, { value: "up", text: obj._getLocStr("Up") }];
        this._gotoRBtnSet1 = ["Comments", "Blanks", "LastCell", "Formulas", "Constants"];
        this._gotoRBtnSet2 = ["RowDiff", "ColDiff", "CFormat", "DataValidation"];
        this._gotoChkBox = ["GotoNumbers", "GotoText", "GotoLogicals", "GotoError"];
        this._othersSptBtnIds = ["inssbtn", "delsbtn", "find", "fPane"];
        this._contextualBtnIds = ["resizetable", "convertorange"];
        this._contextualInputIds = ["tableid", "tablename"];
        this._contextualCheckBoxIds = ["tsofirstcolumn", "tsolastcolumn", "tsototalrow", "tsofiltercolumn"];
        this._numberFormat = [{ value: "1", text: "General" }, { value: "2", text: "Number" }, { value: "3", text: "Currency" }, { value: "4", text: "Accounting" }, { value: "5", text: "Percentage" }, { value: "6", text: "Text" }, { value: "7", text: "Shortdate" }, { value: "8", text: "Longdate" }, { value: "9", text: "Time" }, { value: "10", text: "Scientific" }, { value: "11", text: "Fraction" }];
        this._othersBtnIds = ["insbtn", "delbtn", "namemanager", "definename"];
        this._deleteMenuData = [
                { id: "DeleteCells", text: obj._getLocStr("delcells") + "...", parentId: null, sprite: "e-icon e-ssr-delcell" },
                { id: "DeleteSheetRows", text: obj._getLocStr("delrows"), parentId: null, sprite: "e-icon e-ssr-delrow" },
                { id: "DeleteSheetColumns", text: obj._getLocStr("delcols"), parentId: null, sprite: "e-icon e-ssr-delcol" },
                { id: "DeleteSheet", text: obj._getLocStr("delsheet"), parentId: null, sprite: "e-icon e-ssr-delsheet" }
        ];
        this._clearMenuData = [
                { id: "Clear_All", text: obj._getLocStr("ClearAll"), parentId: null, sprite: "e-icon e-ss-clear" },
                { id: "Clear_Formats", text: obj._getLocStr("ClearFormats"), parentId: null, sprite: "e-icon e-ss-clearformat" },
                { id: "Clear_Contents", text: obj._getLocStr("ClearContents"), parentId: null },
                { id: "Clear_Comments", text: obj._getLocStr("ClearComments"), parentId: null },
                { id: "Clear_Hyperlinks", text: obj._getLocStr("ClearHyperLinks"), parentId: null }
        ];
        this._autosumMenuData = [
                { id: "Sum", text: obj._getLocStr("Sum"), parentId: null, sprite: "e-icon e-ss-autosum" },
                { id: "Average", text: obj._getLocStr("Average"), parentId: null },
                { id: "Count", text: obj._getLocStr("CountNumber"), parentId: null },
                { id: "Max", text: obj._getLocStr("Max"), parentId: null },
                { id: "Min", text: obj._getLocStr("Min"), parentId: null }
        ];
        this._cmntTglBtnIds = ["showAllComments"];
        this._cmntBtnIds = ["newComment", "deleteComment", "previousComment", "nextComment", "showHideComment"];
        this._chartThemes = [{ value: "flatlight", text: "Flat Light" }, { value: "flatdark", text: "Flat Dark" }, { value: "azure", text: "Azure" }, { value: "azuredark", text: "Azure Dark" }, { value: "lime", text: "Lime" }, { value: "limedark", text: "Lime Dark" }, { value: "saffron", text: "Saffron" }, { value: "saffrondark", text: "Saffron Dark" }];
        this._isCustomCellStyleReset = false;
    };

    ej.spreadsheetFeatures.ribbon.prototype = {

        _renderChartTypeDialog: function () {
            var xlObj = this.XLObj, dlgId = xlObj._id + "_charttypedlg", $dlg, $tab, $okBtn, $canBtn, $btndiv, $ul, $li, $aTag, $divctnr;
            $dlg = ej.buildTag("div#" + dlgId + ".e-chartdialog");
            $tab = ej.buildTag("div#" + dlgId + "_allcharttab.e-chtdlgtab");
            $divctnr = ej.buildTag("div.e-dlg-fields e-dlgctndiv");
            $ul = ej.buildTag("ul", "", "");
            $aTag = ej.buildTag("a", xlObj._getLocStr("AllCharts"), {}, { href: "#" + xlObj._id + "_allchart" });
            $li = ej.buildTag("li", $aTag);
            $dlg.append($divctnr.append($tab.append($ul.append($li))));
            $btndiv = ej.buildTag("div#" + dlgId + "_btndiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $okBtn = ej.buildTag("input#" + dlgId + "_okbtn");
            $canBtn = ej.buildTag("input#" + dlgId + "_cantn");
            $okBtn.ejButton({ text: xlObj._getLocStr("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._chartTypeOk, this), enabled: true });
            $canBtn.ejButton({ text: xlObj._getLocStr("Cancel"), click: ej.proxy(this._chartTypeCancel, this), showRoundedCorner: true, width: 60 });
            $btndiv.append($btnctnr.append($okBtn, $canBtn));
            $dlg.append($btndiv);
            xlObj.element.append($dlg);
            $dlg.ejDialog({ enableModal: true, showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("ChartType"), width: 510, height: "auto", cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this) });
            $("#" + dlgId + "_allcharttab").ejTab({ cssClass: "e-ss-dlgtab", width: 480 });
            this._renderAllChartTab();
        },
        _renderAllChartTab: function () {
            var xlObj = this.XLObj, xlId = xlObj._id, charttabId = xlId + "_chartdlg_charttab", $rightdiv, $leftdiv, $contentdiv, $listUl, $li = "", chart, colChart, chartTitle;
            var i, j, len, chartCnt, chartLen, splt, content = "", chartName = ["Column-6", "Bar-6", "Radar-2", "Line-2", "Area-3", "Pie-3", "Scatter-1"];
            $rightdiv = ej.buildTag("div#" + charttabId + "_right.e-chtdlgright");
            $leftdiv = ej.buildTag("div#" + charttabId + "_left.e-chtdlgleft");
            $contentdiv = ej.buildTag("div#" + charttabId + "_content.e-chtdlgcontent", { "border-bottom": 0 });
            $listUl = ej.buildTag("ul#" + charttabId + "_list");
            for (chartCnt = 0, chartLen = chartName.length; chartCnt < chartLen; chartCnt++) {
                chart = chartName[chartCnt].split("-")[0];
                $li += "<li><span class='e-icon e-ss-dlg" + chart.toLowerCase() + "chart'></span>" + chart + "</li>";
            }
            $listUl.append($li);
            $listUl.ejListBox({ width: "240", selectIndexChanged: $.proxy(this._onChartChange, this), height: "300", allowMultiSelection: false, selectedItemIndex: 0 });
            $leftdiv.append($listUl);
            $contentdiv = ej.buildTag("div#" + charttabId + "_content.e-chtdlgcontent");
            $rightdiv.append($contentdiv);
            $("#" + xlId + "_allchart").append($leftdiv).append($rightdiv);
            for (j = 0, chartLen = chartName.length; j < chartLen; j++) {
                splt = chartName[j].split("-");
                chartTitle = xlObj._getLocStr("Title" + splt[0] + "Chart").split("/");
                chart = splt[0].toLowerCase();
                colChart = ej.buildTag("div#" + xlId + "_" + chart + "chart");
                for (i = 1, len = parseInt(splt[1]) ; i <= len; i++) {
                    content += "<div class='e-chartimg e-dlg" + chart + "chart" + i + "' title='" + chartTitle[i - 1] + "'></div>";
                }
                $contentdiv.append(colChart.append(content));
                $("#" + xlId + "_" + chart + "chart").find(".e-dlg" + chart + "chart1").addClass("e-chartselect");
                colChart.hide();
                content = "";
            }
            $("#" + xlId + "_columnchart").show();
        },
        _onChartChange: function (args) {
            var i, xlObj = this.XLObj, chart = args.model.value.toLowerCase(), allChart = ["column", "bar", "radar", "line", "area", "pie", "scatter"];
            for (i = 0; i < allChart.length; i++)
                $("#" + xlObj._id + "_" + allChart[i] + "chart").hide();
            $("#" + xlObj._id + "_" + chart + "chart").show();
            $("#" + xlObj._id + "_allchart").find(".e-chartimg").removeClass("e-chartselect");
            $("#" + xlObj._id + "_" + chart + "chart").find(".e-dlg" + chart + "chart1").addClass("e-chartselect");
        },

        _initValDialog: function () {
            var $dlg, $tab, $okBtn, $canBtn, $btndiv, $ul, $li, $aTag, xlObj = this.XLObj, $divctnr, $btnctnr;
            $dlg = ej.buildTag("div#" + xlObj._id + "_ValDialog");
            $tab = ej.buildTag("div#" + xlObj._id + "_ValDialog_valDlgTab");
            $divctnr = ej.buildTag("div.e-dlg-fields e-dlgctndiv");
            //create tab Content
            $ul = ej.buildTag("ul");
            $aTag = ej.buildTag("a", "Settings", {}, { href: "#" + xlObj._id + "_Validation" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $tab.append($ul);
            $tab.ejTab({ width: 325, height: 320, cssClass: "e-ss-dlgtab" });
            $dlg.append($divctnr.append($tab));
            $btndiv = ej.buildTag("div#" + xlObj._id + "btnDiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $okBtn = ej.buildTag("input#" + xlObj._id + "Dialog_OkBtn");
            $canBtn = ej.buildTag("input#" + xlObj._id + "Dialog_CancelBtn");
            $okBtn.ejButton({ text: xlObj._getLocStr("Ok"), showRoundedCorner: true, width: "19%", click: ej.proxy(this._valDlgBtnClick, this, "ok"), enabled: true });
            $canBtn.ejButton({ text: xlObj._getLocStr("Cancel"), click: ej.proxy(this._valDlgBtnClick, this, "cancel"), showRoundedCorner: true, width: "19%" });
            $btndiv.append($btnctnr.append($okBtn, $canBtn));
            $dlg.append($btndiv);
            $dlg.ejDialog({ enableModal: true, showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("DataValidation"), width: 355, height: "auto", cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this) });
        },

        _initCFDialog: function () {
            var xlObj = this.XLObj, $dlg, $div, $okBtn, $canBtn, $btndiv, $btnctnr;
            $dlg = ej.buildTag("div#" + xlObj._id + "_CFDialog");
            xlObj.element.append($dlg);
            $div = ej.buildTag("div#" + xlObj._id + "_CondFormat.e-dlgctndiv");
            $dlg.append($div);
            $btndiv = ej.buildTag("div#" + xlObj._id + "cfBtnDiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $okBtn = ej.buildTag("input#" + xlObj._id + "cfDialog_OkBtn");
            $canBtn = ej.buildTag("input#" + xlObj._id + "cfDialog_CancelBtn");
            $okBtn.ejButton({ text: xlObj._getLocStr("Ok"), showRoundedCorner: true, width: "25%", click: ej.proxy(this._dlgCFOk, this), enabled: true });
            $canBtn.ejButton({ text: xlObj._getLocStr("Cancel"), click: ej.proxy(this._dlgCFCancel, this), showRoundedCorner: true, width: "25%" });
            $btndiv.append($btnctnr.append($okBtn, $canBtn));
            $dlg.append($btndiv);
            $dlg.ejDialog({ enableModal: true, showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("ConditionalFormat"), width: 275, cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this) });
        },

        _initHLDialog: function () {
            var xlObj = this.XLObj, $dlg, $tab, $ul, $li, $aTag, $btndiv, $okBtn, $canBtn, $divctnr;
            $dlg = ej.buildTag("div#" + xlObj._id + "_HLDialog");
            $tab = ej.buildTag("div#" + xlObj._id + "_HLDialogTab");
            $divctnr = ej.buildTag("div.e-dlg-fields e-dlgctndiv");
            //create tab Content
            $ul = ej.buildTag("ul");
            $aTag = ej.buildTag("a", xlObj._getLocStr("WebPage"), {}, { href: "#" + xlObj._id + "_Web" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $aTag = ej.buildTag("a", xlObj._getLocStr("WorkSheet"), {}, { href: "#" + xlObj._id + "_Doc" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $tab.append($ul);
            $tab.ejTab({ width: 350, itemActive: ej.proxy(this._hlTabChange, this), cssClass: "e-ss-dlgtab" });
            $dlg.append($divctnr.append($tab));
            $btndiv = ej.buildTag("div#" + xlObj._id + "hlBtnDiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $okBtn = ej.buildTag("input#" + xlObj._id + "hlDialog_OkBtn");
            $canBtn = ej.buildTag("input#" + xlObj._id + "hlDialog_CancelBtn");
            $okBtn.ejButton({ text: xlObj._getLocStr("Ok"), showRoundedCorner: true, width: 60, click: ej.proxy(this._dlgHLOk, this), enabled: true });
            $canBtn.ejButton({ text: xlObj._getLocStr("Cancel"), click: ej.proxy(this._dlgHLCancel, this), showRoundedCorner: true, width: 60 });
            $btndiv.append($btnctnr.append($okBtn, $canBtn));
            $dlg.append($btndiv);
            $dlg.ejDialog({
                enableModal: true, showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("InsertHyperLink"), width: 380, cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this), open: ej.proxy(function (e) {
                    var webAddr = $("#" + xlObj._id + "_Ribbon_webAddress"), cellAddr = $("#" + xlObj._id + "_Ribbon_cellAddress");
                    webAddr.focus().setInputPos(webAddr.val().length);
                    cellAddr.focus().setInputPos(cellAddr.val().length);
                })
            });
        },

        _initFRDialog: function () {
            var xlObj = this.XLObj, $dlg, $tab, $findNextBtn, $findPrevBtn, $close, $btndiv, $btnctnr, $ul, $li, $ctnr, $aTag, $replaceBtn, $replaceAllBtn;
            $dlg = ej.buildTag("div#" + xlObj._id + "_FRDialog");
            $tab = ej.buildTag("div#" + xlObj._id + "_FRDialog_FPDlgTab");
            $ctnr = ej.buildTag("div.e-dlg-fields e-dlgctndiv");
            //create tab Content
            $ul = ej.buildTag("ul");
            $aTag = ej.buildTag("a", xlObj._getLocStr("Find"), {}, { href: "#" + xlObj._id + "_Find" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $aTag = ej.buildTag("a", xlObj._getLocStr("Replace"), {}, { href: "#" + xlObj._id + "_Replace" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $aTag = ej.buildTag("a", xlObj._getLocStr("Settings"), {}, { href: "#" + xlObj._id + "_Settings" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $tab.append($ul);
            $tab.ejTab({ width: 390, height: 110, itemActive: ej.proxy(this._findTabChange, this), cssClass: "e-ss-dlgtab" });
            $dlg.append($ctnr.append($tab));
            $btndiv = ej.buildTag("div#" + xlObj._id + "btnFRDiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $replaceBtn = ej.buildTag("input#" + xlObj._id + "FR_ReplaceBtn", "", { "margin-left": 0 });
            $replaceAllBtn = ej.buildTag("input#" + xlObj._id + "FR_ReplaceAllBtn");
            $findNextBtn = ej.buildTag("input#" + xlObj._id + "FR_NextBtn");
            $findPrevBtn = ej.buildTag("input#" + xlObj._id + "FR_PrevBtn");
            $close = ej.buildTag("input#" + xlObj._id + "FR_CloseBtn");
            $replaceBtn.ejButton({ text: xlObj._getLocStr("Replace"), showRoundedCorner: true, width: 55, height: 17, click: ej.proxy(this._btnFROnClick, this, "FR_ReplaceBtn"), enabled: true }).css("display", "none");
            $replaceAllBtn.ejButton({ text: xlObj._getLocStr("ReplaceAll"), showRoundedCorner: true, width: 70, height: 17, click: ej.proxy(this._btnFROnClick, this, "FR_ReplaceAllBtn"), enabled: true }).css("display", "none");
            $findNextBtn.ejButton({ text: xlObj._getLocStr("FindNext"), showRoundedCorner: true, width: 69, height: 17, click: ej.proxy(this._btnFROnClick, this, "FR_NextBtn"), enabled: true });
            $findPrevBtn.ejButton({ text: xlObj._getLocStr("FindPrev"), showRoundedCorner: true, width: 69, height: 17, click: ej.proxy(this._btnFROnClick, this, "FR_PrevBtn"), enabled: true });
            $close.ejButton({ text: xlObj._getLocStr("Close"), click: ej.proxy(this._btnFROnClick, this, "FR_CloseBtn"), showRoundedCorner: true, width: 47, height: 17 });
            $btndiv.append($btnctnr.append($replaceBtn, $replaceAllBtn, $findNextBtn, $findPrevBtn, $close));
            $dlg.append($btndiv);
            $dlg.ejDialog({
                showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("FindnReplace"), width: 420, height: "auto", cssClass: "e-ss-dialog e-ss-frdlg e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this), open: ej.proxy(function (e) {
                    var findTxt = $("#" + xlObj._id + "_Ribbon_textFind");
                    findTxt.focus().setInputPos(findTxt.val().length);
                })
            });
        },

        _initGoToDialog: function () {
            var xlObj = this.XLObj, $dlg, $tab, $btndiv, $ul, $li, $aTag, $okBtn, $canBtn, $ctnr;
            $dlg = ej.buildTag("div#" + xlObj._id + "_GoDialog");
            $tab = ej.buildTag("div#" + xlObj._id + "_GoToDlgTab");
            $ctnr = ej.buildTag("div.e-dlg-fields e-dlgctndiv");
            //create tab Content
            $ul = ej.buildTag("ul");
            $aTag = ej.buildTag("a", xlObj._getLocStr("GoTo"), {}, { href: "#" + xlObj._id + "_GoTo" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $aTag = ej.buildTag("a", xlObj._getLocStr("GoTo") + " " + xlObj._getLocStr("Special"), {}, { href: "#" + xlObj._id + "_GoToSp" });
            $li = ej.buildTag("li", $aTag);
            $ul.append($li);
            $tab.append($ul);
            $tab.ejTab({ width: 288, height: 340, itemActive: ej.proxy(this._gotoTabChange, this), cssClass: "e-ss-dlgtab" });
            $dlg.append($ctnr.append($tab));
            $btndiv = ej.buildTag("div#" + xlObj._id + "btnDiv.e-dlg-btnfields");
            $btnctnr = ej.buildTag("div.e-dlg-btnctnr");
            $dlg.append($btndiv.append($btnctnr));
            $okBtn = ej.buildTag("input#" + xlObj._id + "Dialog_GoOkBtn");
            $canBtn = ej.buildTag("input#" + xlObj._id + "Dialog_GoCancelBtn");
            $okBtn.ejButton({ text: xlObj._getLocStr("Ok"), showRoundedCorner: true, width: "21.5%", click: ej.proxy(this._gotoBtnClick, this, "ok") });
            $canBtn.ejButton({ text: xlObj._getLocStr("Cancel"), click: ej.proxy(this._gotoBtnClick, this, "cancel"), showRoundedCorner: true, width: "21.5%" });
            $btnctnr.append($okBtn, $canBtn);
            xlObj.element.append($dlg);
            $dlg.ejDialog({
                showOnInit: false, enableResize: false, allowKeyboardNavigation: false, title: xlObj._getLocStr("GoTo"), width: 318, height: "auto", cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this), open: ej.proxy(function (e) {
                    var gotoipt = $("#" + xlObj._id + "_textRef");
                    gotoipt.focus().setInputPos(gotoipt.val().length);
                })

            });
        },

        _dialogClose: function () {
            this.XLObj.setSheetFocus();
        },

        _beforeDlgClose: function (args) {
            var gridOlay = $("#" + this.XLObj._id + "_nmgrid_dialogEdit_overLay");
            gridOlay.length && (args.cancel = true);
        },

        _setButtonHeights: function () {
            this._smallBtnHeight = 25;
            this._mediumBtnHeight = 35;
            this._bigBtnHeight = 75;
            this._splitBtnHeight = 75;
            this._drpDownHeight = 25;
        },

        _homeTabObj: function () {
            var createdObj, xlObj = this.XLObj, imgtop = "imagetop", homeTab = { id: "home", text: "HOME", groups: [] }, xlId = xlObj._id, xlMod = xlObj.model;
            createdObj = {
                text: "Clipboard",
                alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
                        this._generateBtn("pastebtn", "paste", "textandimage", "e-icon e-ssr-paste", 45, this._mediumBtnHeight, "PasteTitle", "PasteContent", false, imgtop),
                        this._generateSplitBtn("pastesbtn", "Paste", "textandimage", xlId + "_Ribbon_Paste", { dataSource: this._pasteMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "", false, 45, this._mediumBtnHeight, "bottom", "", "dropdown", "e-ssr-pastesbtn", "PasteTitle", "PasteSplitContent")
                    ],
                },
                {
                    groups: [
                        this._generateBtn("cut", "Cut", "textandimage", "e-icon e-ssr-cut", 58, this._smallBtnHeight, "CutTitle", "CutContent"),
                        this._generateBtn("copy", "Copy", "textandimage", "e-icon e-ssr-copy", 67, this._smallBtnHeight, "CopyTitle", "CopyContent"),
                        this._generateToggleBtn("formatpainter", "FormatPainter", "textandimage", "e-icon e-ss-formatpainter", "FPTitle", 110, this._smallBtnHeight, "FPContent")
                    ],
                }]
            };
            homeTab.groups.push(createdObj);
            createdObj = {
                text: "Font",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
						 this._generateDD("font-family", "FontFamily", this._fontFamily, "7", 150, this._smallBtnHeight, "FontFamily", "FFContent"),
                        this._generateDD("fontsize", "FontSize", this._fontSize, "11pt", 65, this._smallBtnHeight, "FontSize", "FSContent"),
                        this._generateBtn("increasefontsize", "IncreaseFontSize", "imageonly", "e-icon e-ss-increasefontsize", "", this._smallBtnHeight, "IncreaseFontSize", "IFSContent"),
                        this._generateBtn("decreasefontsize", "DecreaseFontSize", "imageonly", "e-icon e-ss-decreasefontsize", "", this._smallBtnHeight, "DecreaseFontSize", "DFSContent")
                    ],
                },
                {
                    groups: [
                        this._generateToggleBtn("bold", "Bold", "imageonly", "e-icon e-ss-bold", "BoldTitle", "", this._smallBtnHeight, "BoldContent"),
                        this._generateToggleBtn("italic", "Italic", "imageonly", "e-icon e-ss-italic", "ItalicTitle", "", this._smallBtnHeight, "ItalicContent"),
                        this._generateToggleBtn("underline", "Underline", "imageonly", "e-icon e-ss-underline", "ULineTitle", "", this._smallBtnHeight, "ULineContent"),
                        this._generateToggleBtn("line-through", "Linethrough", "imageonly", "e-icon e-ss-linethrough", "LineTrTitle", 25, this._smallBtnHeight, "LineTrContent", true),
                        this._generateSplitBtn("border", "Border", "imageonly", xlId + "_Ribbon_Border", { dataSource: this._borderMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "e-icon e-ss-bottomborder", true, 37, this._smallBtnHeight, "", "", "", "e-borderbtn", "Border", "BorderContent"),
                        this._generateCustomControl(xlId + "_Ribbon_background-color", "BackgroundColor", "BackgroundColor", "BGContent"),
                        this._generateCustomControl(xlId + "_Ribbon_color", "Color", "Color", "ColorContent")
                    ],
                }]
            };
            homeTab.groups.push(createdObj);
            createdObj = {
                text: "Alignment",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateToggleBtn("top", "TopAlign", "imageonly", "e-icon e-ss-topalign", "TopAlign", "", "", "TopAlignContent"),
                        this._generateToggleBtn("middle", "MiddleAlign", "imageonly", "e-icon e-ss-middlealign", "MiddleAlign", "", "", "MiddleAlignContent"),
                        this._generateToggleBtn("bottom", "BottomAlign", "imageonly", "e-icon e-ss-bottomalign", "BottomAlign", "", "", "BottomAlignContent", true),
						this._generateToggleBtn("wraptext", "WrapText", "textandimage", "e-icon e-ss-wraptext", "WrapText", 85, 25, "WrapTextContent")
                    ],
                },
                {
                    groups: [
                        this._generateToggleBtn("left", "AlignLeft", "imageonly", "e-icon e-ss-alignleft", "AlignLeft", "", "", "AlignLeftContent"),
                        this._generateToggleBtn("center", "AlignCenter", "imageonly", "e-icon e-ss-aligncenter", "AlignCenter", "", "", "AlignCenterContent"),
                        this._generateToggleBtn("right", "AlignRight", "imageonly", "e-icon e-ss-alignright", "AlignRight", "", "", "AlignRightContent", true),
                        this._generateSplitBtn("merge", "MergeAndCenter", "textandimage", xlId + "_Ribbon_Merge", { dataSource: this._mergeMenuData, id: 'id', parentId: 'parentId', text: 'text', spriteCssClass: 'sprite' }, "e-icon e-ssr-merge", false, 136, 25, "right", "", "", "e-spreadsheet e-mergebtn", "MergeAndCenter", "MergeAndCenterContent")
                    ],
                }]
            };
            homeTab.groups.push(createdObj);
            createdObj = {
                text: "Actions",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateBtn("undo", "Undo", "textandimage", "e-ss-undo", 40, this._bigBtnHeight, "UndoTitle", "UndoContent", false, imgtop, ""),
                        this._generateBtn("redo", "Redo", "textandimage", "e-ss-redo", 40, this._bigBtnHeight, "RedoTitle", "RedoContent", false, imgtop, "")
                    ],
                }]
            };
            homeTab.groups.push(createdObj);
            createdObj = {
                text: "Number",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateDD("cellformat", "NumberFormat", this._numberFormat, "1", 125, this._smallBtnHeight, "NumberFormat", "NumberFormatContent")
                    ],
                },
                {
                    groups: [
                        this._generateBtn("accounting", "AccountingStyle", "imageonly", "e-icon e-ss-accounting", "", this._smallBtnHeight, "AccountingStyle", "AccountingStyleContent"),
                        this._generateBtn("percentage", "PercentageStyle", "imageonly", "e-icon e-ss-percentage", "", this._smallBtnHeight, "PercentageStyle", "PercentageStyleContent"),
                        this._generateBtn("usegrouping", "CommaStyle", "imageonly", "e-icon e-ss-comma", "", this._smallBtnHeight, "CommaStyle", "CommaStyleContent", true),
                        this._generateBtn("increment", "IncreaseDecimal", "imageonly", "e-icon e-ss-increasedecimal", "", this._smallBtnHeight, "IncreaseDecimal", "IncreaseDecimalContent"),
                        this._generateBtn("decrement", "DecreaseDecimal", "imageonly", "e-icon e-ss-decreasedecimal", "", this._smallBtnHeight, "DecreaseDecimal", "DecreaseDecimalContent")
                    ],
                }]
            };
            homeTab.groups.push(createdObj);
            createdObj = {
                text: "Styles",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateSplitBtn("cFormat", "ConditionalFormat", "textandimage", xlId + "_CFormat", { dataSource: this._cFormatMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "e-icon e-ssr-condformat", false, 75, this._splitBtnHeight, "bottom", imgtop, "dropdown", "e-spreadsheet e-cformatbtn", "ConditionalFormat", "CFContent"),
                        this._generateSplitBtn("formatastable", "FormatasTable", "textandimage", xlId + "_formatastable", "", "e-icon e-ssr-formatastable", false, 65, this._splitBtnHeight, "bottom", ej.ImagePosition.ImageTop, "dropdown", "e-spreadsheet e-formatastablebtn", "FormatAsTable", "FATContent"),
                        this._generateSplitBtn("cellstyles", "CellStyles", "textandimage", xlId + "_cellstyles", "", "e-icon e-ssr-cellstyles", false, 55, this._splitBtnHeight, "bottom", ej.ImagePosition.ImageTop, "dropdown", "e-spreadsheet e-cellstylebtn", "CellStyles", "CellStylesContent")
                    ],
                }]
            };
            homeTab.groups.push(createdObj);
            createdObj = {
                text: 'Editing',
                alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
                        this._generateSplitBtn("sortfilter", "SortFilter", "textandimage", xlId + "_Ribbon_SortFilter", { dataSource: this._sortFltrMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "e-icon e-ss-sortfilter", false, 50, this._splitBtnHeight, "bottom", "imagetop", "dropdown", "e-spreadsheet e-sortfltrbtn", "SortFilter", "SortFilterContent")
                    ],
                },
                {
                    groups: [
                        this._generateSplitBtn("autosum", "AutoSum", "textandimage", xlId + "_Ribbon_AutoSum", { dataSource: this._autosumMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "e-icon e-ssr-autosum", false, 100, 25, "right", "", "dropdown", "e-spreadsheet e-autosumbtn", "AutoSumTitle", "AutoSumContent"),
                        this._generateSplitBtn("clear", "Clear", "textandimage", xlId + "_Ribbon_Clear", { dataSource: this._clearMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "e-icon e-ssr-clear", false, 80, 25, "right", "", "dropdown", "e-spreadsheet e-clearbtn", "Clear", "ClearContent")
                    ],
                }]
            };
            homeTab.groups.push(createdObj);
            this._tabCollection.push(homeTab);
        },

        _insertTabObj: function () {
            var createdObj, xlObj = this.XLObj, imgtop = "imagetop", insertTab = { id: "insert", text: "INSERT", groups: [] }, xlId = xlObj._id;
            createdObj = {
                text: "Tables",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateBtn("pivotTable", "PivotTable", "textandimage", "e-icon e-ssr-pivottable", 65, this._bigBtnHeight, "PivotTable", "PivotTableContent", false, imgtop),
                        this._generateBtn("table", "Table", "textandimage", "e-icon e-ssr-table", 50, this._bigBtnHeight, "Table", "TableContent", false, imgtop)
                    ],
                }]
            };
            insertTab.groups.push(createdObj);
            createdObj = {
                text: "Illustrations",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateBtn("pictures", "Pictures", "textandimage", "e-icon e-ssr-pictures", 50, this._bigBtnHeight, "PicturesTitle", "PicturesContent", false, imgtop, "")
                    ],
                }]
            };
            insertTab.groups.push(createdObj);
            createdObj = {
                text: "Links",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateBtn("hyperlink", "HyperLink", "textandimage", "e-icon e-ssr-hyperlink", 60, this._bigBtnHeight, "HyperLinkTitle", "HyperLinkContent", false, imgtop, "")
                    ],
                }]
            };
            insertTab.groups.push(createdObj);
            createdObj = {
                text: "Charts",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateSplitBtn("columnchart", "ColumnChart", "imageonly", xlId + "_Ribbon_ColumnChart", "", "e-icon e-ss-colchart", false, 41, this._smallBtnHeight, "bottom", "ej.ImagePosition.ImageTop", "dropdown", "e-spreadsheet e-chartbtn", "ColumnChartTitle", "ColumnChartContent"),
                        this._generateSplitBtn("barchart", "BarChart", "imageonly", xlId + "_Ribbon_BarChart", "", "e-icon e-ss-barchart", false, 41, this._smallBtnHeight, "bottom", "ej.ImagePosition.ImageTop", "dropdown", "e-spreadsheet e-chartbtn", "BarChartTitle", "BarChartContent"),
                        this._generateSplitBtn("stockchart", "StockChart", "imageonly", xlId + "_Ribbon_StockChart", "", "e-icon e-ss-stockchart", false, 41, this._smallBtnHeight, "bottom", "ej.ImagePosition.ImageTop", "dropdown", "e-spreadsheet e-chartbtn", "StockChartTitle", "StockChartContent")
                    ],
                },
                {
                    groups: [
                        this._generateSplitBtn("linechart", "LineChart", "imageonly", xlId + "_Ribbon_LineChart", "", "e-icon e-ss-linechart", false, 41, this._smallBtnHeight, "bottom", "ej.ImagePosition.ImageTop", "dropdown", "e-spreadsheet e-chartbtn", "LineChartTitle", "LineChartContent"),
                        this._generateSplitBtn("areachart", "AreaChart", "imageonly", xlId + "_Ribbon_AreaChart", "", "e-icon e-ss-areachart", false, 41, this._smallBtnHeight, "bottom", "ej.ImagePosition.ImageTop", "dropdown", "e-spreadsheet e-chartbtn", "AreaChartTitle", "AreaChartContent"),
                        this._generateSplitBtn("piechart", "PieChart", "imageonly", xlId + "_Ribbon_PieChart", "", "e-icon e-ss-piechart", false, 41, this._smallBtnHeight, "bottom", "ej.ImagePosition.ImageTop", "dropdown", "e-spreadsheet e-chartbtn", "PieChartTitle", "PieChartContent")
                    ],
                },
                {
                    groups: [
                        this._generateSplitBtn("scatterchart", "ScatterChart", "imageonly", xlId + "_Ribbon_ScatterChart", "", "e-icon e-ss-scterchart", false, 41, this._smallBtnHeight, "bottom", "ej.ImagePosition.ImageTop", "dropdown", "e-spreadsheet e-chartbtn", "ScatterChartTitle", "ScatterChartContent")
                    ],
                }]
            };
            insertTab.groups.push(createdObj);
            this._tabCollection.push(insertTab);
        },

        _dataTabObj: function () {
            var createdObj, xlObj = this.XLObj, imgtop = "imagetop", dataTab = { id: "data", text: "DATA", groups: [] };
            createdObj = {
                text: "Sort & Filter",
                alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
                        this._generateBtn("sortAtoZ", "SortAtoZ", "textandimage", "e-icon e-ssr-sortatoz", 90, this._smallBtnHeight, "SortAtoZ", "SortAtoZContent", false, "", "e-datapadding"),
                        this._generateBtn("sortZtoA", "SortZtoA", "textandimage", "e-icon e-ssr-sortztoa", 90, this._smallBtnHeight, "SortZtoA", "SortZtoAContent", false, "", "e-datapadding")
                    ]
                },
                {
                    groups: [
                        this._generateBtn("filter", "Filter", "textandimage", "e-icon e-ssr-filter", 60, this._smallBtnHeight, "Filter", "FilterContent", false, "", "e-datapadding"),
                        this._generateBtn("clearfilter", "ClearFilter", "textandimage", "e-icon e-ss-clearfilter", 90, this._smallBtnHeight, "ClearFilter", "ClearFilterContent", false, "", "e-datapadding")
                    ],
                }]
            };
            dataTab.groups.push(createdObj);
            createdObj = {
                text: "Data Tools",
                alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
                        this._generateBtn("validationbtn", "Validation", "imageonly", "e-icon e-ssr-validation", 70, 39, "DataValidation", "DVContent", false, imgtop),
                        this._generateSplitBtn("validationsbtn", "DataValidation", "textandimage", xlObj._id + "_Ribbon_Validation", { dataSource: this._valMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "", false, 70, this._mediumBtnHeight, "bottom", "", "dropdown", "e-ssr-validationsbtn", "DataValidation", "DVContent")
                    ],
                }]
            };
            dataTab.groups.push(createdObj);
            this._tabCollection.push(dataTab);
        },

        _pageTabObj: function () {
            var createdObj, xlObj = this.XLObj, imgtop = "imagetop", pageTab = { id: "pagelayout", text: "PAGE LAYOUT", groups: [] };
            createdObj = {
                text: "Show",
                alignType: ej.Ribbon.alignType.rows,
                width: 105,
                content: [{
                    groups: [
                        { contentID: xlObj._id + "_Ribbon_pagesetupleft", type: ej.Ribbon.type.custom, height: 80 },
                    ],
                }]
            };
            pageTab.groups.push(createdObj);
            createdObj = {
                text: "Page Layout",
                alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
                        this._generateBtn("sizebtn", "size", "imageonly", "e-icon e-ssr-pagesize", 65, 44, "size", "PageSizeContent", false, imgtop, ""),
                        this._generateSplitBtn("sizesbtn", "PageSize", "textandimage", xlObj._id + "_Ribbon_PageSize", { dataSource: this._pageSizes, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "", false, 65, this._mediumBtnHeight, "bottom", "", "dropdown", "e-ssr-pagesizesbtn", "PageSize", "PageSizeContent")
                    ],
                }]
            };
            pageTab.groups.push(createdObj);
            createdObj = {
                text: "Print", alignType: ej.Ribbon.alignType.columns, width: 120,
                content: [{
                    groups: [
						this._generateBtn("print", "Print", "textandimage", "e-icon e-ssr-print", 50, this._bigBtnHeight, "Print", "PrintContent", false, imgtop, "")
                    ]
                },
                {
                    groups: [
                        this._generateBtn("printselected", "PrintSelected", "textandimage", "e-icon e-ssr-printselected", 55, this._bigBtnHeight, "PrintSelected", "PrintSelectedContent", true, imgtop, "")
                    ],
                }]
            };
            pageTab.groups.push(createdObj);
            this._tabCollection.push(pageTab);
        },

        _reviewTabObj: function () {
            var createdObj, xlObj = this.XLObj, imgtop = "imagetop", reviewTab = { id: "review", text: "REVIEW", groups: [] };
            createdObj = {
                text: "Comments", alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
						this._generateBtn("newComment", "New", "textandimage", "e-icon e-ssr-newcmnt", 49, this._bigBtnHeight, "NewCommentTitle", "NewCommentContent", false, imgtop)
                    ],
                },
                {
                    groups: [
                        this._generateBtn("deleteComment", "Delete", "textandimage", "e-icon e-ssr-deletecmnt", 49, this._bigBtnHeight, "DeleteComment", "DeleteCommentContent", false, imgtop)
                    ],
                },
                {
                    groups: [
                        this._generateBtn("previousComment", "Previous", "textandimage", "e-icon e-ssr-previouscmnt", 49, this._bigBtnHeight, "Previous", "PreviousContent", false, imgtop)
                    ],
                },
                {
                    groups: [
                        this._generateBtn("nextComment", "Next", "textandimage", "e-icon e-ssr-nextcmnt", 49, this._bigBtnHeight, "Next", "NextContent", false, imgtop)
                    ],
                },
                {
                    groups: [
                        this._generateBtn("showHideComment", "ShowHide", "textandimage", "e-icon e-ssr-showhidecmnt", 160, this._smallBtnHeight, "ShowHide", "ShowHideContent", false, "imageLeft", "e-ssr-cmntcolbtn"),
                        this._generateToggleBtn("showAllComments", "ShowAll", "textandimage", "e-icon e-ssr-showallcmnt", "ShowAll", 155, this._smallBtnHeight, "ShowAllContent", false, "e-ssr-cmntcolbtn")
                    ],
                }]
            };
            reviewTab.groups.push(createdObj);
            createdObj = {
                text: "Changes",
                alignType: ej.Ribbon.alignType.row,
                content: [{
                    groups: [
                          this._generateToggleBtn("ProtectSheet", "Protectsheet", "textandimage", "e-icon e-ssr-protectsheet", "Protectsheet", 60, this._bigBtnHeight, "protectsheettooltip", false, "e-ssr-protectsheetbtn", "imagetop", "Unprotect"),
                          this._generateToggleBtn("protectWorkbook", "ProtectWorkbook", "textandimage", "e-icon e-ssr-protect", "ProtectWorkbook", 60, this._bigBtnHeight, "ProtectWBContent", false, "e-ssr-cmntcolbtn", "imagetop"),
                          this._generateToggleBtn("LockCell", "Lock", "textonly", "e-icon e-ssr-protlockcell", "lock", 75, this._smallBtnHeight, "Lock", false, "e-ssr-lockbtn", "", "Unlock"),
                    ],
                },
			   
                ]
            };
            reviewTab.groups.push(createdObj);
            this._tabCollection.push(reviewTab);
        },

        _otherTabObj: function () {
            var createdObj, xlObj = this.XLObj, imgtop = "imagetop", otherTab = { id: "others", text: "OTHERS", groups: [] }, xlId = xlObj._id, conttype = ej.ContentType;
            createdObj = {
                text: "Window", alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
						this._generateSplitBtn("fPane", "FreezePanes", "textandimage", xlObj._id + "_Ribbon_FPane", { dataSource: this._viewMenuData, id: 'id', parentId: 'parentId', text: 'text', spriteCssClass: 'sprite' }, "e-icon e-ssr-frzpane", false, 55, this._splitBtnHeight, "bottom", imgtop, "dropdown", "e-spreadsheet e-fpanebtn", "FreezePanes", "FreezePanesContent")
                    ],
                }]
            };
            otherTab.groups.push(createdObj);
            createdObj = {
                text: xlObj._getLocStr("Formula"),
                alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
                        this._generateBtn("namemanager", "NameManager", conttype.TextAndImage, "e-icon e-ssr-namemngr", 55, this._bigBtnHeight, "NameManager", "NameManagerContent", false, imgtop, "")
                    ],
                },
                {
                    groups: [
                        this._generateBtn("definename", "DefineName", conttype.TextAndImage, "e-icon e-ssr-definename", 110, 25, "DefineName", "DefineNameContent"),
                        this._generateSplitBtn("useinformula", "UseInFormula", "textandimage", xlObj._id + "_nmuseinformula", "", "e-icon e-ssr-useformula", false, 125, 25, ej.ArrowPosition.Bottom, ej.ImagePosition.ImageLeft, "dropdown", "e-spreadsheet e-useinformulabtn", "UseInFormula", "UseInFormulaContent")
                    ],
                }]
            };
            otherTab.groups.push(createdObj);
            createdObj = {
                text: "Cells",
                alignType: ej.Ribbon.alignType.columns,
                content: [{
                    groups: [
                        this._generateBtn("insbtn", "Insert", "imageonly", "e-icon e-ssr-insert", 54, 40, "InsertTitle", "InsertContent", false, imgtop),
                        this._generateSplitBtn("inssbtn", "Insert", "textandimage", xlId + "_Ribbon_Ins", { dataSource: this._insertMenuData, id: 'id', parentId: 'parentId', text: 'text', spriteCssClass: 'sprite' }, "", false, 54, this._mediumBtnHeight, "bottom", "", "dropdown", "e-insertsbtn", "InsertTitle", "InsertSBContent")
                    ],
                },
                {
                    groups: [
                        this._generateBtn("delbtn", "Delete", "imageonly", "e-icon e-ssr-delete", 54, 40, "DeleteTitle", "DeleteContent", false, imgtop),
                        this._generateSplitBtn("delsbtn", "Delete", "textandimage", xlId + "_Ribbon_Del", { dataSource: this._deleteMenuData, id: 'id', parentId: 'parentId', text: 'text', spriteCssClass: 'sprite' }, "", false, 54, this._mediumBtnHeight, "bottom", "", "dropdown", "e-deletesbtn", "DeleteTitle", "DeleteContent")
                    ],
                }]
            };
            otherTab.groups.push(createdObj);
            createdObj = {
                text: "Editing",
                alignType: ej.Ribbon.alignType.rows,
                content: [{
                    groups: [
                        this._generateSplitBtn("find", "FindSelect", "textandimage", xlId + "_Ribbon_FindRep", { dataSource: this._findMenuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "e-icon e-ss-findselect", false, 55, this._splitBtnHeight, "bottom", "imagetop", "dropdown", "e-spreadsheet e-findbtn", "FindSelectTitle", "FindSelectContent")
                    ],
                }]
            };
            otherTab.groups.push(createdObj);
            this._tabCollection.push(otherTab);
        },

        _renderRibbon: function () {
            var xlObj = this.XLObj, xlId = xlObj._id, xlEle = xlObj.element, xlMod = xlObj.model, table, tr, td1, td2, click, tr1, td11, td12,
                ribbonElem = ej.buildTag("div", "", "", { id: xlId + "_Ribbon" });
            xlEle.append(ribbonElem);
            this._renderChartTypeDialog();
            if (xlMod.allowCellFormatting) {
                xlEle.append(ej.buildTag("input", "", "", { id: xlId + "_Ribbon_background-color", type: "text" }));
                ribbonElem.append(ej.buildTag("input", "", "", { id: xlId + "_Ribbon_BorderColor", type: "text" }));
            }
            xlEle.append(this._borderPicture());
            xlMod.allowCharts && xlEle.append(this._chartElementTag());
            xlMod.allowFormatAsTable && xlEle.append(this._renderFormatAsTable());
            xlEle.append(xlObj.XLRibbon._renderCellStyles());
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_FPane" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_Merge" }));
            var cid = '<input id=' + xlId + '_chart type="text" style="display:none;"/><input id=' + xlId + '_tableid type="text" style="display:none;"/>';
            ribbonElem.append(cid);
            this._renderNameManagerDlg(ribbonElem);
            if (xlMod.allowCharts) {
                var chartsizediv = ej.buildTag("div.e-ss-numeric", "", { width: "180", height: "80" }, { id: xlId + "_Ribbon_chartsize" });
                table = ej.buildTag("table"); tr = ej.buildTag("tr"); tr1 = ej.buildTag("tr"); td1 = ej.buildTag("td", "", { width: "100%" }); td2 = ej.buildTag("td"); td11 = ej.buildTag("td"); td12 = ej.buildTag("td");
                td1.append(ej.buildTag("label.e-icon e-ssr-chartheight", "", { width: "25", height: "25", display: "visible" }, { id: xlId + "_Ribbon_chartheightlbl" }));
                td2.append(ej.buildTag("input", "", { type: "text" }, { id: xlId + "_Ribbon_chartheight" }));
                tr.append(td1).append(td2);
                td11.append(ej.buildTag("label.e-icon e-ssr-chartwidth", "", { width: "25", height: "25", display: "visible" }, { id: xlId + "_Ribbon_chartwidthlbl" }));
                td12.append(ej.buildTag("input", "", { type: "text" }, { id: xlId + "_Ribbon_chartwidth" }));
                tr1.append(td11).append(td12); table.append(tr).append(tr1);
                chartsizediv.append(table);
                xlEle.append(chartsizediv);
            }
            var pagesetupdiv = ej.buildTag("div", "", { width: "140", height: "80", display: "none" }, { id: xlId + "_Ribbon_pagesetup" });
            var pagesetupleftdiv = ej.buildTag("div", "", { width: "70", height: "80", "float": "left" }, { id: xlId + "_Ribbon_pagesetupleft" });
            table = ej.buildTag("table"); tr = ej.buildTag("tr"); tr1 = ej.buildTag("tr"); td1 = ej.buildTag("td", "", { padding: "6px 0px 6px 6px" }); td2 = ej.buildTag("td", "", { padding: "6px 0px 6px 6px" }); td11 = ej.buildTag("td", "", { padding: "6px 0px 6px 6px" }); td12 = ej.buildTag("td", "", { padding: "6px 0px 6px 6px" });
            var gdlnviewlbl = ej.buildTag("label.e-viewlinlbl", "", { width: "40" }, { id: xlId + "_Ribbon_pagesetup_viewgrdlnlbl", "for": xlId + "_Ribbon_Vgridlines" });
            var hdrviewlbl = ej.buildTag("label.e-viewhdrlbl", "", { width: "40", display: "visible" }, { id: xlId + "_Ribbon_pagesetup_viewhdrlbl", "for": xlId + "_Ribbon_Vheading" });
            var headerVchk = ej.buildTag("input", "", "", { id: xlId + "_Ribbon_Vheading" });
            var gridVchk = ej.buildTag("input", "", "", { id: xlId + "_Ribbon_Vgridlines" });
            td1.append(gridVchk); td2.append(gdlnviewlbl); tr.append(td1).append(td2); td11.append(headerVchk); td12.append(hdrviewlbl); tr1.append(td11).append(td12); table.append(tr1).append(tr);
            pagesetupleftdiv.append(table);
            pagesetupdiv.append(pagesetupleftdiv);
            ribbonElem.append(pagesetupdiv);
            $("#" + xlId + "_Ribbon_pagesetup_viewhdrlbl").text(xlObj._getLocStr("Heading"));
            $("#" + xlId + "_Ribbon_pagesetup_viewgrdlnlbl").text(xlObj._getLocStr("Gridlines"));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_PageSize" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_Paste" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_FindRep" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_Validation" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_Border" }));
            xlEle.append(ej.buildTag("ul.e-spreadsheet", "", "", { id: xlId + "_CFormat" }));
            xlMod.allowCellFormatting && xlEle.append(ej.buildTag("input", "", "", { id: xlId + "_Ribbon_color", type: "text" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_Clear" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_SortFilter" }));
            xlEle.append(ej.buildTag("ul", "", { width: "auto" }, { id: xlId + "_Ribbon_Ins" }));
            xlEle.append(ej.buildTag("ul", "", { width: "auto" }, { id: xlId + "_Ribbon_Del" }));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_ResetPic" }));
            xlEle.append(this._chartType("ColumnChart", 6, 2, 2, ["column", "stackingcolumn", "stackingcolumn100", "column", "stackingcolumn", "stackingcolumn100"], ["ClusteredColumn", "StackedColumn", "Stacked100Column", "Cluster3DColumn", "Stacked3DColumn", "Stacked100Column3D"]));
            xlEle.append(this._chartType("BarChart", 6, 2, 2, ["bar", "stackingbar", "stackingbar100", "bar", "stackingbar", "stackingbar100"], ["ClusteredBar", "StackedBar", "Stacked100Bar", "Cluster3DBar", "Stacked3DBar", "Stacked100Bar3D"]));
            xlEle.append(this._chartType("StockChart", 2, 2, 2, ["radar", "radar"], ["Radar", "RadarMarkers"]));
            xlEle.append(this._chartType("LineChart", 2, 2, 3, ["line", "line"], ["Line", "LineMarkers"]));
            xlEle.append(this._chartType("AreaChart", 3, 3, 3, ["Area", "stackingarea", "stackingarea100"], ["Area", "StackedArea", "Stacked100Area"]));
            xlEle.append(this._chartType("PieChart", 3, 3, 4, ["pie", "pie", "doughnut"], ["Pie", "Pie3D", "Doughnut"]));
            xlEle.append(this._chartType("ScatterChart", 1, 2, 2, ["scatter"], ["Scatter"]));
            xlEle.append(ej.buildTag("ul", "", "", { id: xlId + "_Ribbon_AutoSum" }));
            this._renderAppTab();
            xlMod.allowFormatAsTable && this._renderDesignTab();
            this._renderFormatTab();
            this._homeTabObj();
            this._insertTabObj();
            this._dataTabObj();
            this._pageTabObj();
            this._reviewTabObj();
            this._otherTabObj();
            this._contextualTabObj();
            ribbonElem.ejRibbon({
                width: "100%",
                allowResizing: true,
                applicationTab: this._appTabCollection,
                tabs: this._tabCollection,
                contextualTabs: this._contextualTabCollection,
                tabClick: $.proxy(this._onTabClick, this),
                tabSelect: $.proxy(this._onTabSelect, this),
                create: $.proxy(this._onControlCreated, this),
                toggleButtonClick: $.proxy(this._onToggleButtonClick, this),
                collapse: $.proxy(this._onCollapse, this),
                backstageItemClick: $.proxy(this._onBackStageItemClick, this)
            });
            $("#" + xlId + "_bordercolor ul li").find("a").remove();
            $("#" + xlId + "_bordercolor ul li").append($("#" + xlId + "_Ribbon_BorderColor_popup"));
            $("#" + xlId + "picturecolor ul li").find("a").remove();
            $("#" + xlId + "picturecolor ul li").append($("#" + xlId + "_Ribbon_PictureColor_popup"));
            $("#" + xlId + "_Ribbon_review_Comments_5").css("vertical-align", "middle");
            xlMod.scrollSettings.isResponsive && $("#" + xlId + "_Ribbon").ejRibbon("option", { allowResizing: true });
            this._hideTabs();
            !xlMod.allowComments && xlObj._setAllowComments(false);
            click = ej.SplitButton.prototype._documentClick;
            ej.SplitButton.prototype._documentClick = function (e) { // for ejmenu document click issue(JS-10113)
                if (!$(e.target).is(".e-formatastable"))
                    click.call(this, e);
            };
            if (xlObj.model.allowFormatAsTable)
                $("#" + xlObj._id + "_formatastable").data("ejMenu").model.open = function (args) {
                    $("#" + xlObj._id + "_formatastable").scrollTop(0);
                };
            //Css update
            this._updateRibCustomCss();
        },

        _updateRibCustomCss: function () {
            var xlObj = this.XLObj, xlId = xlObj._id, xlEle = xlObj.element;
            xlEle.find("#" + xlId + "_Ribbon_background-colorWrapper .e-in-wrap, #" + xlId + "_Ribbon_colorWrapper .e-in-wrap").addClass("e-ssr-colpickHprt");
            xlEle.find(".e-ssr-colpickHprt .e-color-container,.e-ssr-colpickHprt .e-select").addClass("e-ssr-colorcontainer");
            xlEle.find(".e-mergebtn, .e-borderbtn").parent().addClass("e-ssr-horizontalprt");
            xlEle.find("#" + xlId + "_Ribbon_font-family").parents('.e-controlpadding').addClass("e-ssr-fontfamily");
            xlEle.find("#" + xlId + "_Ribbon_fontsize").parents('.e-controlpadding').addClass("e-ssr-fontsize");

            xlEle.find("#" + xlId + "_Ribbon_insbtn").parent().parent().prepend(ej.buildTag("span.e-ssr-verticalHparent e-ssr-insertbtnHprt", "", { display: "inherit" }));
            xlEle.find("#" + xlId + "_Ribbon_insbtn").parent().appendTo($('span.e-ssr-verticalHparent.e-ssr-insertbtnHprt'));
            xlEle.find("#" + xlId + "_Ribbon_inssbtn").closest("div").appendTo($('span.e-ssr-verticalHparent.e-ssr-insertbtnHprt'));

            xlEle.find("#" + xlId + "_Ribbon_delbtn").parent().parent().prepend(ej.buildTag("span.e-ssr-verticalHparent e-ssr-deletebtnHprt", "", { display: "inherit" }));
            xlEle.find("#" + xlId + "_Ribbon_delbtn").parent().appendTo($('span.e-ssr-verticalHparent.e-ssr-deletebtnHprt'));
            xlEle.find("#" + xlId + "_Ribbon_delsbtn").closest("div").appendTo($('span.e-ssr-verticalHparent.e-ssr-deletebtnHprt'));

            xlEle.find("#" + xlId + "_Ribbon_sizebtn").parent().parent().prepend(ej.buildTag("span.e-ssr-verticalHparent e-ssr-sizebtnHprt", "", { display: "inherit" }));
            xlEle.find("#" + xlId + "_Ribbon_sizebtn").parent().appendTo($('span.e-ssr-verticalHparent.e-ssr-sizebtnHprt'));
            xlEle.find("#" + xlId + "_Ribbon_sizesbtn").closest("div").appendTo($('span.e-ssr-verticalHparent.e-ssr-sizebtnHprt'));

            xlEle.find("#" + xlId + "_Ribbon_validationbtn").parent().parent().prepend(ej.buildTag("span.e-ssr-verticalHparent e-ssr-dtvaldtnHprt", "", { display: "inherit" }));
            xlEle.find("#" + xlId + "_Ribbon_validationbtn").parent().appendTo($('span.e-ssr-verticalHparent.e-ssr-dtvaldtnHprt'));
            xlEle.find("#" + xlId + "_Ribbon_validationsbtn").closest("div").appendTo($('span.e-ssr-verticalHparent.e-ssr-dtvaldtnHprt'));

            xlEle.find("#" + xlId + "_Ribbon_pastebtn").parent().parent().prepend(ej.buildTag("span.e-ssr-verticalHparent e-ssr-pastebtnHprt", "", { display: "inherit" }));
            xlEle.find("#" + xlId + "_Ribbon_pastebtn").parent().appendTo($('span.e-ssr-verticalHparent.e-ssr-pastebtnHprt'));
            xlEle.find("#" + xlId + "_Ribbon_pastesbtn").closest("div").appendTo($('span.e-ssr-verticalHparent.e-ssr-pastebtnHprt'));

            xlEle.find("#ej" + xlId + "_Ribbon_Vheading").addClass("e-vheading");
            xlEle.find("#ej" + xlId + "_Ribbon_Vgridlines").addClass("e-vgridlines");
        },

        // ribbon backstage
        _renderAppTab: function () {
            var htmlStr, xlObj = this.XLObj, bspage = [];
            // backstage content elements
            htmlStr = "<div id='" + xlObj._id + "_backstagetabnew' class='e-ssr-bstabnew'><div class='e-ssr-bscontent' title='" + xlObj._getLocStr("BlankWorkbook") + "'><div class='e-ssr-bsnewtmpl' ><button id='" + xlObj._id + "_bstab_new_blank' ></button></div></div></div>";
            htmlStr = htmlStr.concat("<div id='" + xlObj._id + "_backstagetabsaveas' class='e-ssr-bstabsaveas'><div class='e-ssr-bscontent'><div class='e-ssr-bssaveasopt'><button id='" + xlObj._id + "_bstab_saveas_excel'></button><span>" + xlObj._getLocStr("SaveAsExcel") + "</span></div><div class='e-ssr-bssaveasopt'><button id='" + xlObj._id + "_bstab_saveas_csv'></button><span>" + xlObj._getLocStr("SaveAsCsv") + "</span></div></div></div>");
            htmlStr = htmlStr.concat("<div id='" + xlObj._id + "_backstagetabprint' class='e-ssr-bstabprint'><div class='e-ssr-bscontent'><div class='e-ssr-bsprintopt'><button id='" + xlObj._id + "_bstab_print_sheet'></button><span>" + xlObj._getLocStr("PrintSheet") + "</span></div><div class='e-ssr-bsprintopt'><button id='" + xlObj._id + "_bstab_print_selected'></button><span>" + xlObj._getLocStr("PrintSelected") + "</span></div></div></div>");
            xlObj.element.append(htmlStr);
            // backstage object construction
            bspage.push({ id: "new", text: xlObj._getLocStr("New"), itemType: ej.Ribbon.itemType.tab, contentID: xlObj._id + "_backstagetabnew" });
            bspage.push({ id: "open", text: xlObj._getLocStr("Open") + "...", itemType: ej.Ribbon.itemType.button });
            bspage.push({ id: "saveas", text: xlObj._getLocStr("SaveAs"), itemType: ej.Ribbon.itemType.tab, contentID: xlObj._id + "_backstagetabsaveas" });
            bspage.push({ id: "print", text: xlObj._getLocStr("Print"), itemType: ej.Ribbon.itemType.tab, contentID: xlObj._id + "_backstagetabprint" });
            this._appTabCollection = { type: ej.Ribbon.applicationTabType.backstage, backstageSettings: { text: xlObj._getLocStr("File"), headerWidth: 124, height: xlObj.model.scrollSettings.height, width: xlObj.model.scrollSettings.width, pages: bspage } };
        },

        _onBackStageItemClick: function (args) {
            var xlObj = this.XLObj, evnt = { element: args.target };
            xlObj.XLEdit._isEdit && xlObj.XLEdit.saveCell();
            if (xlObj._trigger("menuClick", evnt))
                return;
            if (args.id === "open_backStageBtn") {
                if (xlObj._isSheetsDirty())
                    xlObj._showAlertDlg("confirm", "ImportAlert", "Open", 470);
                else if (xlObj.model.importSettings.importMapper.length < 1) {
                    xlObj._showAlertDlg("Alert", "ImportExportUrl");
					 return;	
                }
                else
                    $("#" + xlObj._id + "_file .e-uploadinput").click();
            }
        },

        _bSBtnClickHandler: function (args) {
            var alertdlg, sheetIdx, id = this._id.split("_"), xlObj = $("#" + id[0]).data("ejSpreadsheet"), tab = id[2], opt = id[3];
            $("#" + xlObj._id + "_Ribbon").ejRibbon("hideBackstage");
            switch (tab) {
                case "new":
                    switch (opt) {
                        case "blank":
                            xlObj._isSheetsDirty() && xlObj._showAlertDlg("confirm", "DestroyAlert", "New", 470);
                            break;
                    }
                    break;
                case "saveas":
                    xlObj.XLExport["export"](opt === "excel" ? ej.Spreadsheet.exportType.Excel : ej.Spreadsheet.exportType.Csv);
                    break;
                case "print":
                    sheetIdx = xlObj.getActiveSheetIndex();
                    switch (opt) {
                        case "sheet":
                            xlObj.XLPrint.printSheet(sheetIdx);
                            break;
                        case "selected":
                            xlObj.XLPrint.printSelection(sheetIdx);
                            break;
                    }
                    break;
            }
        },

        _contextualTabObj: function () {
            var xlObj = this.XLObj, xlId = xlObj._id, imgtop = "imagetop", createdObj;
            if (xlObj.model.allowFormatAsTable) {
                createdObj = {
                    backgroundColor: "#FCFBEB",
                    borderColor: "#F2CC1C",
                    tabs: [
                        {
                            id: "design",
                            text: xlObj._getLocStr("Design"),
                            groups: [
                                {
                                    text: "Properties",
                                    alignType: ej.Ribbon.alignType.rows,
                                    type: "custom",
                                    contentID: xlId + "_tableproperties"
                                },
                                {
                                    text: "Tools",
                                    alignType: ej.Ribbon.alignType.columns,
                                    content: [{
                                        groups: [
                                            this._generateBtn("resizetable", "ResizeTable", "textandimage", "e-icon e-ssr-resizetable", 101, this._smallBtnHeight, "ResizeTable", "ResizeTableContent"),
                                            this._generateBtn("convertorange", "ConvertToRange", "textandimage", "e-icon e-ssr-range", 130, this._smallBtnHeight, "ConvertToRange", "ConvertToRangeContent")
                                        ],
                                    }
                                    ]
                                },
                                {
                                    text: "Table Style Options",
                                    alignType: ej.Ribbon.alignType.rows,
                                    type: "custom",
                                    contentID: xlId + "_tablestyleoptions"
                                }
                            ]
                        }]
                };
                this._contextualTabCollection.push(createdObj);
            }
            if (xlObj.model.allowCharts) {
                createdObj = {
                    backgroundColor: "#EAF6BD",
                    borderColor: "#90AA3C",
                    tabs: [
                        {
                            id: "cdesign",
                            text: "Chart Design",
                            groups: [
                                {
                                    text: "Chart Layouts",
                                    alignType: ej.Ribbon.alignType.columns,
                                    content: [{
                                        groups: [
                                            this._generateSplitBtn("cElement", "AddChartElement", "textandimage", xlId + "_Ribbon_CElement", "", "e-icon e-ssr-celements", true, 65, this._bigBtnHeight, "bottom", "imagetop", "dropdown", "e-spreadsheet e-chartlayoutbtn", "AddChartElement", "AddChartElementContent")
                                        ]
                                    }]
                                },
                                {
                                    text: "Data",
                                    alignType: ej.Ribbon.alignType.rows,
                                    content: [{
                                        groups: [
                                            this._generateBtn("srcolumn", "SwitchRowColumn", "textandimage", "e-icon e-ssr-srcolumn", 72, this._bigBtnHeight, "SwitchRowColumn", "SwitchRowColumnContent", "", "imagetop"),
                                            this._generateBtn("selectData", "SelectData", "textandimage", "e-icon e-ssr-selectdata", 50, this._bigBtnHeight, "SelectData", "SelectDataContent", false, imgtop)
                                        ]
                                    }]
                                },
                                {
                                    text: "Type",
                                    alignType: ej.Ribbon.alignType.rows,
                                    content: [{
                                        groups: [
                                            this._generateBtn("charttype", "ChartType", "textandimage", "e-icon e-ssr-charttype", 65, this._bigBtnHeight, "ChartType", "ChartTypeContent", false, imgtop)
                                        ]
                                    }]
                                },
                                {
                                    text: "Chart Themes",
                                    alignType: ej.Ribbon.alignType.columns,
                                    content: [{
                                        groups: [
                                            this._generateDD("chartTheme", "ChartThemes", this._chartThemes, "1", 130, this._smallBtnHeight, "ChartThemes", "ChartThemesContent")
                                        ],
                                    }]
                                },
                                {
                                    text: "Size",
                                    alignType: ej.Ribbon.alignType.columns,
                                    content: [{
                                        groups: [{
                                            contentID: xlId + "_Ribbon_chartsize",
                                            enableSeparator: true,
                                            type: ej.Ribbon.type.custom,
                                            height: 80
                                        }]
                                    }]
                                }
                            ]
                        }]
                };
                this._contextualTabCollection.push(createdObj);
            }
            if (xlObj.model.pictureSettings.allowPictures) {
                createdObj = {
                    backgroundColor: "#FCFBEB",
                    borderColor: "#F2CC1C",
                    tabs: [
                        {
                            id: "format",
                            text: "Format",
                            groups: [
                                {
                                    text: "Adjust",
                                    alignType: ej.Ribbon.alignType.columns,
                                    content: [{
                                        groups: [
                                            this._generateBtn("changepic", "ChangePicture", "textandimage", "e-icon e-ssr-changepicture", 120, this._mediumBtnHeight, "ChangePicture", "ChangePictureContent"),
                                            this._generateSplitBtn("resetpic", "ResetPicture", "textandimage", xlId + "_Ribbon_ResetPic", { dataSource: this._resetPicture, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" }, "e-icon e-ssr-resetpicture", false, 120, this._mediumBtnHeight, "left", "", "dropdown", "e-ss-resetpictbtn", "ResetPicture", "ResetPictureContent")
                                        ],
                                    }
                                    ]
                                },
                                {
                                    text: "Border",
                                    alignType: ej.Ribbon.alignType.columns,
                                    content: [{
                                        groups: [
                                            this._generateSplitBtn("pictureborder", "PictureBorder", "textandimage", xlId + "_Ribbon_PictureBorder", "", "e-icon e-ssr-pictureborder", false, 120, this._mediumBtnHeight, "left", "", "dropdown", "e-ss-pictbrdrbtn", "PictureBorder", "PictureBorderContent")
                                        ],
                                    }
                                    ]
                                },
                                {
                                    text: "Size",
                                    alignType: ej.Ribbon.alignType.columns,
                                    type: "custom",
                                    contentID: xlId + "_picturesize"
                                }
                            ]
                        }]
                };
                this._contextualTabCollection.push(createdObj);
            }
        },

        _onToggleButtonClick: function (args) {
            var xlObj = this.XLObj, settings = $.extend(true, {}, xlObj.model.scrollSettings);
            this._ribbonState = !(args.action === "collapse");
            if (args.action === "expand") {
                args.action = "tabClick";
                this._heightWidthCalculation(xlObj.getActiveSheetIndex(), settings, args);
            }
        },

        _heightWidthCalculation: function (sheetIdx, settings, args) {
            var height, xlObj = this.XLObj, actpanel = xlObj.getActivationPanel(), isVisible = actpanel.is(':visible');
            (settings.width.toString().indexOf("%") !== -1) && (settings.width = xlObj.getMainPanel().width());
            if (!xlObj._isDisplayHeader) {
                if (this._ribbonState)
                    height = xlObj.element.find('.e-ribbon .e-header').height() + 109;
                else
                    height = xlObj.element.find('.e-ribbon .e-header').height();
            }
            else {
                height = $("#" + xlObj._id + "_Ribbon").height();
                xlObj._isDisplayHeader = false;
            }
            xlObj._setSheetWidthHeight(sheetIdx, { width: isVisible ? settings.width - xlObj.model.apWidth : settings.width, height: settings.height }, height, "resize", args.action);
            isVisible && actpanel.height(xlObj.getMainPanel().height());
        },

        _hideTabs: function () {  // to hide ribbon tabs
            var i, robj = $("#" + this.XLObj._id + "_Ribbon").ejRibbon("instance"), tnames = ["", this.XLObj._getLocStr("Design"), "Chart Design", "Format"];
            i = tnames.length;
            while (i = i - 1)
                robj.hideTab(tnames[i]);
        },

        // ribbon design tab
        _renderDesignTab: function () {
            var proxy = this.XLObj, htmlstr = '', tsotext = proxy._getLocStr('TableStyleOptions').split('/');
            htmlstr = '<div id=' + proxy._id + '_tableproperties ><div class="e-tablepropertiesrow" ><div class="e-tablepropertiescell">Table Name:</div></div><div class="e-tablepropertiesrow" ><div class="e-tablepropertiescell"><input type="text" id=' + proxy._id + '_tablename class="ejinputtext" style="width:86px;opacity:0.7;height: 18px;text-indent: 3px;" disabled="disabled" /></div></div></div>';
            htmlstr = htmlstr + '<div id=' + proxy._id + '_tablestyleoptions ><div class="e-tablestyleoptionsrow" ><div class="e-tablestyleoptionscell"><input id=' + proxy._id + '_Ribbon_tsofirstcolumn type="checkbox" /><label class="e-tablestyleoptionslabel" for="' + proxy._id + '_Ribbon_tsofirstcolumn">' + tsotext[0] + '</label></div><div class="e-tablestyleoptionscell" ><input id=' + proxy._id + '_Ribbon_tsolastcolumn type="checkbox" /><label class="e-tablestyleoptionslabel" for="' + proxy._id + '_Ribbon_tsolastcolumn">' + tsotext[1] + '</label></div></div><div class="e-tablestyleoptionsrow" ><div class="e-tablestyleoptionscell"><input id=' + proxy._id + '_Ribbon_tsototalrow type="checkbox" /><label class="e-tablestyleoptionslabel" for="' + proxy._id + '_Ribbon_tsototalrow">' + tsotext[2] + '</label></div><div class="e-tablestyleoptionscell" ><input id=' + proxy._id + '_Ribbon_tsofiltercolumn type="checkbox" /><label class="e-tablestyleoptionslabel" for="' + proxy._id + '_Ribbon_tsofiltercolumn">' + tsotext[3] + '</label></div></div></div>';
            proxy.element.append(htmlstr);
        },
        _renderFormatTab: function () {
            var xlObj = this.XLObj;
            xlObj.element.append("<div id=" + xlObj._id + "_picturesize class= e-ss-numeric><table><tr><td style='padding-right:4px;'>" + xlObj._getLocStr("Height") + ": </label></td><td style='padding-bottom:3px;'><input id='" + xlObj._id + "_pictureheight' type='text' /></td></tr><tr><td><labe>" + xlObj._getLocStr("Width") + ": </label></td><td><input id='" + xlObj._id + "_picturewidth' type='text' /></td></tr></table></div>");
        },


        // Name Manager(NM)
        _renderNameManagerDlg: function (ribbonElem) {
            var xlObj = this.XLObj, htmlStr = "<div id = '" + xlObj._id + "_nmdlg' ><div class='e-dlgctndiv' ><div class='e-dlg-fields'><div id='" + xlObj._id + "_nmgrid' ></div></div><div class='e-dlg-fields'><label class='e-dlg-fields'>" + xlObj._getLocStr("Refersto") + ":</label><input type='text' class='e-nmrange ejinputtext' id='" + xlObj._id + "_nmrange' disabled='true' /></div><div class='e-dlg-btnfields' ><div class='e-dlg-btnctnr'><button id='" + xlObj._id + "_nmclose' >" + xlObj._getLocStr("Close") + "</button></div></div></div></div>";
            htmlStr = htmlStr + "<div id='" + xlObj._id + "_nmeditortemplate' class='e-hide' ><div class='e-dlg-fields e-dlgctndiv'><table cellpadding='0' cellspacing='0'><tr><td>" + xlObj._getLocStr("Name") + ":</td><td><input id='name' type='text' class='ejinputtext' value='{{: name}}'/></td></tr><tr class='e-dlgtd-fields'><td>" + xlObj._getLocStr("Scope") + ":</td><td><input class='e-nmscope ejinputtext' id='scope' type='text' readonly value='Workbook'/></td></tr><tr class='e-dlgtd-fields'><td>" + xlObj._getLocStr("Comment") + ":</td><td><textarea id='comment' style='resize:none;height:44px;width:100%;' value='{{: comment}}'>{{: comment}}</textarea></td></tr><tr class='e-dlgtd-fields'><td>" + xlObj._getLocStr("Refersto") + ":</td><td><input type='text' class='ejinputtext' id='refersto' value='{{: refersto}}' /></td></tr></table></div></div>";
            htmlStr = htmlStr + "<div id='" + xlObj._id + "_nmuseinformula' class='e-nmuseinformula' style='width:125px'></div>";
            ribbonElem.append(htmlStr);
            $("#" + xlObj._id + "_nmclose").ejButton({ showRoundedCorner: true, width: 68, click: $.proxy(this._nmdlgClose, xlObj) });
            $("#" + xlObj._id + "_nmgrid").ejGrid({
                dataSource: xlObj.model.nameManager.slice(0),
                height: 162,
                width: 505,
                allowScrolling: true,
                allowSorting: true,
                enableAltRow: false,
                cssClass: "e-ss-dialog ej-ssgrid e-"+xlObj._id+"-dlg",
                selectionType: ej.Grid.SelectionType.Single,
                editSettings: { allowAdding: true, editMode: ej.Grid.EditMode.DialogTemplate, dialogEditorTemplateID: "#" + xlObj._id + "_nmeditortemplate" },
                toolbarSettings: { showToolbar: true, toolbarItems: [ej.Grid.ToolBarItems.Add] },
                actionBegin: $.proxy(this._nmActionBegin, xlObj),
                actionComplete: $.proxy(this._nmActionComplete, xlObj),
                rowSelected: $.proxy(this._nmRecordSelected, xlObj),
                columns: [
                       { field: "name", headerText: xlObj._getLocStr("Name"), isPrimaryKey: true, width: 120 },
                       { field: "refersto", headerText: xlObj._getLocStr("RefersTo"), width: 120 },
                       { field: "scope", headerText: xlObj._getLocStr("Scope"), width: 80 },
                       { field: "comment", headerText: xlObj._getLocStr("Comment"), width: 160 }
                ]
            });
            $("#" + xlObj._id + "_nmdlg").ejDialog({ showOnInit: false, title: xlObj._getLocStr('NameManager'), enableModal: true, enableResize: false, width: '537px', cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this), beforeClose: ej.proxy(this._beforeDlgClose, this) });
        },

        _updateNMRange: function () {
            var xlObj = this.XLObj;
            $("#" + xlObj._id + "_nmgridEditForm").find("#refersto").val("=" + xlObj._getDollarAlphaRange(xlObj.getSheet(xlObj.getActiveSheetIndex()).selectedRange, true));
        },

        _nmdlgClose: function () {
            $("#" + this._id + "_nmdlg").ejDialog("close");
        },

        _nmRecordSelected: function (args) {
            $("#" + this._id + "_nmdlg").find(".e-nmrange").val(args.data.refersto);
        },

        _nmActionBegin: function (args) {  // validating input
            var i = 0, canBreak = false, len, prop, data, sheet = this.getSheet(this.getActiveSheetIndex());
            if (args.requestType === "save") {
                data = args.data;
                data.sheetIndex = this._getSheetIdxFromName(args.data.refersto.split("!")[0].replace("=", ""));
                args.cancel = true;
                if (data.sheetIndex === -1) {
                    this._showAlertDlg("Alert", "InvalidSheetIndex", null, 275);
                    return;
                }
                if (sheet._nmEdit) {
                    for (prop in sheet._nmValue) {
                        len = this.model.nameManager.length;
                        while (i < len) {
                            if (this.model.nameManager[i].name === sheet._nmValue.name) {
                                this.model.nameManager.splice(i, 1);
                                this._sheets[this.getActiveSheetIndex()]._calcEngine.removeNamedRange(sheet._nmValue.name);
                                break;
                            }
                            i++;
                        }
                        i = this._formulaCollection.length;
                        while (i--) {
                            if (this._formulaCollection[i].display === sheet._nmValue.name) {
                                canBreak = true;
                                this._formulaCollection.splice(i, 1);
                                break;
                            }
                        }
                        if (canBreak)
                            break;
                    }
                }
                if (this.XLRibbon._validateNamedRange(data.name, data.refersto))
                    args.cancel = false;
                else
                    canBreak && this.model.nameManager.push(sheet._nmValue);
            }
            if (args.requestType === "beginedit") {
                sheet._nmEdit = true;
                sheet._nmValue = { name: args.model.dataSource[args.model.selectedRowIndex].name, refersto: args.model.dataSource[args.model.selectedRowIndex].refersto, comment: args.model.dataSource[args.model.selectedRowIndex].comment, sheetIndex: args.model.dataSource[args.model.selectedRowIndex].sheetIndex, scope: args.model.dataSource[args.model.selectedRowIndex].scope };
            }
        },

        _nmActionComplete: function (args) {
            var gridInst = $("#" + this._id + "_nmgrid").data("ejGrid"), reqType = args.requestType, editDlgInst, editDlgInpElem;
            switch (reqType) {
                case "add":
                    editDlgInst = $("#" + this._id + "_nmgrid_dialogEdit").data("ejDialog");
                    editDlgInst.model.open = $.proxy(this.XLRibbon._updateNMRange(), this);
                    editDlgInst.option("allowDraggable", false);
                    editDlgInst.option("title", this._getLocStr("NewName"));
                    editDlgInpElem = $("#" + this._id + "_nmgridEditForm").find("input[type='button']:first");
                    editDlgInpElem.data("ejButton").option("text", "OK");
                    editDlgInpElem.parent().addClass("e-dlg-btnfields").css("text-align", "right");
                    break;
                case "save":
                    this.XLRibbon._addNamedRange(gridInst, args.data.name, args.data.refersto);
                    break;
                case "delete":
                    this.XLRibbon._removeNamedRange(gridInst, args.data.name);
                    break;
                case "cancel":
                    gridInst.element.focus();
                    break;
            }
        },

        _updateNamedRanges: function (name, refersto) {
            var xlObj = this.XLObj;
            xlObj._sheets[xlObj.getActiveSheetIndex()]._calcEngine.addNamedRange(name, refersto.replace("=", ""));
            xlObj._formulaCollection.push({ text: "=" + name, display: name });
            $("#" + xlObj._id + "_inputbox").data("ejAutocomplete").option("dataSource", xlObj._formulaCollection);
        },

        _updateUseInFormulaTrgt: function () {
            var i = 0, xlObj = this.XLObj, htmlStr = "", trgt = $("#" + xlObj._id + "_nmuseinformula"), nameMgr = xlObj.model.nameManager, length = nameMgr.length;
            while (i < length) {
                htmlStr = htmlStr + "<div class='e-nmuseinformularow' style = 'color:#333333'>" + nameMgr[i].name + "</div>";
                i++;
            }
            length ? $("#" + xlObj._id + "_Ribbon_useinformula").ejSplitButton("enable") : $("#" + xlObj._id + "_Ribbon_useinformula").ejSplitButton("disable");
            trgt.html("<div>" + htmlStr + "</div>");
        },

        addNamedRange: function (name, refersTo, comment, sheetIdx) {
            var gridInst = $("#" + this.XLObj._id + "_nmgrid").data("ejGrid");
            if (this._validateNamedRange(name, refersTo)) {
                gridInst.addRecord({ name: name, refersto: refersTo, comment: comment ? comment : "", scope: "WorkBook", sheetIndex: sheetIdx });
                gridInst.refreshContent();
            }
        },

        _addNamedRange: function (gridInst, name, refersTo) {
            var xlObj = this.XLObj;
            xlObj.model.nameManager = gridInst.model.dataSource.slice(0);
            this._updateNamedRanges(name, refersTo);
            this._updateUseInFormulaTrgt();
        },

        removeNamedRange: function (name) {
            if (!name)
                return;
            var gridInst = $("#" + this.XLObj._id + "_nmgrid").data("ejGrid");
            gridInst.deleteRecord("name", { name: name });
        },

        _removeNamedRange: function (gridInst, name) {
            var i, nameMgr, xlObj = this.XLObj, nm = xlObj.model.nameManager[xlObj.model.nameManager.length - 1];
            xlObj.model.nameManager = gridInst.model.dataSource.slice(0);
            nameMgr = xlObj.model.nameManager;
            xlObj._sheets[xlObj.getActiveSheetIndex()]._calcEngine.removeNamedRange(name);
            xlObj._updateFormulaCollection();
            i = nameMgr.length;
            if (i) {
                while (i--)
                    xlObj._formulaCollection.push({ text: "=" + nameMgr[i].name, display: nameMgr[i].name });
            }
            this._updateUseInFormulaTrgt();
        },

        _validateNamedRange: function (name, refersTo) {
            var range, libMgr, nameMgr, adlg, text, xlObj = this.XLObj;
            range = this._validateDollarRange(refersTo);
            libMgr = /^[a-zA-Z_]+$/.test(name) && this._validateLibraryFunctions(name);
            nameMgr = this._validateNameManager(name);
            if (range && libMgr && nameMgr)
                return true;
            else {
                adlg = $("#" + xlObj._id + "_alertdlg");
                if (!range)
                    text = xlObj._getLocStr("NMRangeAlert");
                else if (!libMgr)
                    text = xlObj._getLocStr("NMNameAlert").split("/")[0];  // not used completely
                else
                    text = xlObj._getLocStr("NMUniqueNameAlert");
                xlObj._showAlertDlg("Alert", "T-" + text, null, 370);
            }
        },

        _validateLibraryFunctions: function (name) {
            var i, arr = [], libraryFunc = new CalcEngine().getLibraryFunctions();
            name = ej.isNullOrUndefined(name) ? "" : name.toString().toUpperCase();
            for (i in libraryFunc.items)  // for hashable issue (return false => always return true)
                arr.push(i);
            i = arr.length;
            while (i--)
                if (arr[i] === name) return false;
            return true;
        },

        _validateNameManager: function (name) {
            var i = 0, nameMgr = this.XLObj.model.nameManager, length = nameMgr.length;
            name = ej.isNullOrUndefined(name) ? "" : name.toString().toUpperCase();
            while (i < length) {
                if (nameMgr[i].name.toUpperCase() === name)
                    return false;
                i++;
            }
            return true;
        },

        _validateDollarRange: function (range) {
            try {
                if (range && range.indexOf("!") > -1) {
                    range = range.split("!")[1].replace(/\$/g, "");
                    this.XLObj.getRangeIndices(range);
                    return true;
                }
            }
            catch (err) {
                return false;
            }
        },

        // Format as table(FAT)
        _renderFormatAsTable: function () {
            var parent, child, xlObj = this.XLObj, i = 0, separator = [11, 19], j = 0, str = '', htmlstr = '',
            b = 'w:bold', c1 = 'c:#FFFFFF', c2 = 'c:#000000', c3 = 'c:#2F75B5', c4 = 'c:#ED7D31', c5 = 'c:#FFC000', b1 = '1px solid #000000', b2 = '1px solid #2f75b5', b3 = '1px solid #ED7D31', b4 = '1px solid #FFC000',
            d1 = 'b:#D9D9D9', d2 = 'b:#DDEBF7', d3 = 'b:#FCE4D6', d4 = 'b:#FFF2CC', d5 = 'b:#000000', d6 = 'b:#DBECF6', d7 = 'b:#FFE2C6', d8 = 'b:#FFF1BF', d9 = 'b:#000000', d10 = 'b:#D7D7D7', d11 = 'b:#FFC000', d12 = 'b:#5B9BD5', d13 = 'b:#Ed7D31',
            style = 'hl?' + b + ';' + c2 + ';bo:tb-' + b1 + '&cl?' + d1 + ';' + c2 + ',' + c2 + '&cb?b-' + b1 + '/hl?' + b + ';bo:tb-' + b2 + ';' + c3 + '&cl?' + d2 + ';' + c3 + ',' + c3 + '&cb?b-' + b2 + '/hl?' + b + ';bo:tb-' + b3 + ';' + c4 + '&cl?' + d3 + ';' + c4 + ',' + c4 + '&cb?b-' + b3 + '/hl?' + b + ';bo:tb-' + b4 + ';' + c5 + '&cl?' + d4 + ';' + c5 + ',' + c5 + '&cb?b-' + b4 + '/hl?' + b + ';' + d5 + ';' + c1 + ';bo:o-' + b1 + '&cl?bo:t-' + b1 + ';' + c2 + '&cb?o-' + b1 + '/hl?bo:o-' + b2 + ';' + b + ';' + d12 + ';' + c1 + '&cl?bo:t-' + b2 + ';' + c2 + '&cb?o-' + b2 + '/hl?' + b + ';b:#EB7E2F;' + c1 + ';bo:o-' + b3 + '&cl?bo:t-' + b3 + ';' + c2 + '&cb?o-' + b3 + '/hl?' + b + ';' + d11 + ';' + c1 + ';bo:o-' + b4 + '&cl?bo:t-' + b4 + ';' + c2 + '&cb?o-' + b4 + '/hl?' + b + ';bo:gb-' + b1 + ';' + c2 + '&cl?' + d1 + ';' + c2 + ',' + c2 + '&cb?gb-' + b1 + '/hl?' + b + ';bo:gb-' + b2 + ';' + c2 + '&cl?' + d2 + ';' + c2 + ',' + c2 + '&cb?gb-' + b2 + '/hl?' + b + ';bo:gb-' + b3 + ';' + c2 + '&cl?' + d3 + ';' + c2 + ',' + c2 + '&cb?gb-' + b3 + '/hl?' + b + ';bo:gb-' + b4 + ';' + c2 + '&cl?' + d4 + ';' + c2 + ',' + c2 + '&cb?gb-' + b4 + '/hl?' + b + ';' + d5 + ';' + c1 + '&cl?b:#A6A6A6;' + c2 + ',' + d1 + ';' + c2 + '/hl?' + b + ';' + d12 + ';' + c1 + '&cl?b:#BDD7EE;' + c2 + ',' + d2 + ';' + c2 + '/hl?' + b + ';' + d13 + ';' + c1 + '&cl?b:#F8CBAD;' + c2 + ',' + d3 + ';' + c2 + '/hl?' + b + ';' + d11 + ';' + c1 + '&cl?b:#FFE699;' + c2 + ',' + d4 + ';' + c2 + '/hl?' + b + ';' + d10 + ';' + c2 + ';bo:gb-' + b1 + '&cl?b:#A7A7A7;' + c2 + ',' + d10 + ';' + c2 + '&cb?gb-' + b1 + '/hl?' + b + ';' + d6 + ';' + c2 + ';bo:gb-' + b2 + '&cl?b:#BED8F1;' + c2 + ',' + d2 + ';' + c2 + '&cb?gb-' + b2 + '/hl?' + b + ';' + d7 + ';' + c2 + ';bo:gb-' + b3 + '&cl?b:#F6CAAD;' + c2 + ',' + d7 + ';' + c2 + '&cb?gb-' + b3 + '/hl?' + b + ';' + d8 + ';' + c2 + ';bo:gb-' + b4 + '&cl?b:#FDE18D;' + c2 + ',' + d8 + ';' + c2 + '&cb?gb-' + b4 + '/hl?' + b + ';' + d9 + ';' + c1 + '&cl?b:#404040;' + c1 + ',b:#737373;' + c1 + '/hl?' + b + ';' + d9 + ';' + c1 + '&cl?b:#2F75B5;' + c1 + ',' + d12 + ';' + c1 + '/hl?' + b + ';' + d9 + ';' + c1 + '&cl?b:#C65911;' + c1 + ',' + d13 + ';' + c1 + '/hl?' + b + ';' + d9 + ';' + c1 + '&cl?b:#BF8F00;' + c1 + ',' + d11 + ';' + c1,
            hdrtext = xlObj._getLocStr('FATHeaderText').split('/'),
            title = xlObj._getLocStr('FormatAsTableTitle').split('/'),
            name = ["Light1", "Light2", "Light3", "Light5", "Light8", "Light9", "Light10", "Light12", "Light15", "Light16", "Light17", "Light19", "Medium8", "Medium9", "Medium10", "Medium12", "Medium22", "Medium23", "Medium24", "Medium26", "Dark1", "Dark2", "Dark3", "Dark5"];
            style = style.split('/');
            while (i < 24) {  // totally 24 div elements has to creates
                str = str + xlObj._renderDIV('e-formatastablecell', '<div class="e-ss-cellstyles-' + (i + 1) + '" />'); // add image here
                i === separator[j] && (j = j + 1, str = str + '</br>');  // for split up purpose
                i = i + 1;
            }
            i = 0, str = str.split('</br>');
            while (i < hdrtext.length * 2) {
                htmlstr = htmlstr + xlObj._renderDIV(i % 2 === 0 ? 'e-formatastableheader' : 'e-formatastablecontent', i % 2 === 0 ? hdrtext[i / 2] : str[0]);
                i % 2 != 0 && str.shift();
                i = i + 1;
            }
            parent = $('<div id=' + xlObj._id + '_formatastable class="e-spreadsheet e-formatastable" style="display:none;" >' + htmlstr + '</div>');
            child = parent.find('.e-formatastablecell');
            i = 0;
            while (i < child.length) {
                $(child[i]).data({ format: this._getFormatAsTableInput(style[i]), formatName: "TableStyle" + name[i] }).attr("title", "Table Style " + title[i]);
                i = i + 1;
            }
            if (!ej.isNullOrUndefined(xlObj.XLFormat)) xlObj.XLFormat._formatAsTableStyle = $(child[0]).data(); // initializing this for insert table option
            this._renderFATNameDlg();
            this._renderChartNameDlg();
            this._renderFATResizeTableDlg();
            return parent;
        },

        _renderFATNameDlg: function () {
            var htmlstr, xlObj = this.XLObj, proptext = xlObj._getLocStr('FATNameDlgText').split('/'), obj = { showRoundedCorner: true, width: "27%", click: $.proxy(this._fatNameDlgBtnClick, this) };
            htmlstr = '<div id=' + xlObj._id + '_fatnamedlg><div class="e-dlg-fields e-dlgctndiv"><table cellspacing="0" cellpadding="0"><tr><td>' + proptext[0] + '</td><td><input id="' + xlObj._id + '_fatname" class="ejinputtext" /></td></tr><tr class="e-dlgtd-fields"><td></td><td><input type="checkbox" id="' + xlObj._id + '_fatheader" style="background-color:#ffffff"/> <label for="' + xlObj._id + '_fatheader" style="margin-left: 4px;">' + proptext[1] + '</label></td></tr></table></div><div class="e-dlg-btnfields"><div class="e-dlg-btnctnr"><button id=' + xlObj._id + '_fatnamedlgok>' + xlObj._getLocStr('Ok') + '</button><button id=' + xlObj._id + '_fatnamedlgcancel>' + xlObj._getLocStr('Cancel') + '</button></div></div></div>'
            xlObj.element.append(htmlstr);
            $("#" + xlObj._id + "_fatheader").ejCheckBox();
            $('#' + xlObj._id + '_fatnamedlgok').ejButton(obj);
            $('#' + xlObj._id + '_fatnamedlgcancel').ejButton(obj);
            $('#' + xlObj._id + '_fatnamedlg').ejDialog({ showOnInit: false, width: "286px", showRoundedCorner: true, title: xlObj._getLocStr('FormatAsTable'), enableModal: true, enableResize: false, cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this) });
        },

        _renderChartNameDlg: function () {
            var htmlstr, xlObj = this.XLObj, proptext = xlObj._getLocStr('ChartTitleDlgText'), obj = { showRoundedCorner: true, width: "23%", click: $.proxy(this._chartNameDlgBtnClick, this) };
            htmlstr = '<div id=' + xlObj._id + '_chartnamedlg ><div class="e-dlg-fields e-dlgctndiv"><table cellpadding="0" cellspacing="0"><tr><td>' + proptext + '</td><td><input id="' + xlObj._id + '_chartname" class="ejinputtext" /></td></tr></table></div><div class="e-dlg-btnfields"><div class="e-dlg-btnctnr"><button id=' + xlObj._id + '_chartnamedlgok >' + xlObj._getLocStr('Ok') + '</button><button id=' + xlObj._id + '_chartnamedlgcancel>' + xlObj._getLocStr('Cancel') + '</button></div></div>';
            xlObj.element.append(htmlstr);
            $('#' + xlObj._id + '_chartnamedlgok').ejButton(obj);
            $('#' + xlObj._id + '_chartnamedlgcancel').ejButton(obj);
            $('#' + xlObj._id + '_chartnamedlg').ejDialog({ showOnInit: false, width: "334px", minHeight: "auto", showRoundedCorner: true, title: xlObj._getLocStr('ChartTitle'), enableModal: true, enableResize: false, cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this) });
        },

        _renderFATResizeTableDlg: function () {
            var xlObj = this.XLObj, obj = { showRoundedCorner: true, width: "15%", click: $.proxy(this._fatResizeTableBtnClick, this) },
            htmlstr = '<div id=' + xlObj._id + '_fatresizetabledlg ><div class="e-dlgctndiv"><table><tr><td>' + xlObj._getLocStr('FATResizeTableText') + '</td><td style="width: 50%;"><input id="' + xlObj._id + '_fatresizetablerange" class="ejinputtext" style="margin-left: 2px;"></td></tr></table><div class="e-dlg-fields">' + xlObj._getLocStr('FATReizeTableNote') + '</div></div><div class="e-dlg-btnfields"><div class="e-dlg-btnctnr"><button id=' + xlObj._id + '_fatresizetabledlgok >' + xlObj._getLocStr('Ok') + '</button><button id=' + xlObj._id + '_fatresizetabledlgcancel >' + xlObj._getLocStr('Cancel') + '</button></div></div></div>';
            xlObj.element.append(htmlstr);
            $('#' + xlObj._id + '_fatresizetabledlgok').ejButton(obj);
            $('#' + xlObj._id + '_fatresizetabledlgcancel').ejButton(obj);
            $('#' + xlObj._id + '_fatresizetabledlg').ejDialog({ showOnInit: false, width: "494px", showRoundedCorner: true, title: xlObj._getLocStr('ResizeTable'), enableModal: true, enableResize: false, cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg", close: ej.proxy(this._dialogClose, this) });
        },


        _fatResizeTableBtnClick: function (args) {
            var xlObj = this.XLObj, alertText, rrange = document.getElementById(xlObj._id + '_fatresizetablerange').value, dataSrcIdx, sheetIndex = xlObj.getActiveSheetIndex();
            var details, tid = document.getElementById(xlObj._id + '_tableid').value, tmgr = xlObj.getSheet(sheetIndex)._tableManager;
            details = { sheetIndex: sheetIndex, reqType: "format-table", action: "resizetable", beforeRange: tmgr[tid].range, tableId: tid }, canclose = true;
            if (args.model.text === xlObj._getLocStr('Ok')) {
                if (!xlObj._isvalidRange(rrange)) {
                    canclose = false;
                    xlObj._showAlertDlg("Alert", "InvalidReference", "", 375);
                }
                else {
                    rrange = rrange.indexOf(':') != -1 ? xlObj.getRangeIndices(rrange) : [];
                    alertText = this._validateResizeRange(rrange, tmgr[tid].range);
                    if (!alertText) {
                        tmgr[tid].range = rrange;
                        xlObj.XLFormat._createTable(tid, { format: tmgr[tid].format });
                        xlObj.XLFilter._extendFilterRange(tid, rrange);
                        details.afterRange = rrange;
                        details.format = tmgr[tid].format;
                        details.formatName = tmgr[tid].formatName;
                        details.header = $.extend(true, {}, xlObj.model.sheets[sheetIndex]._header);
                        xlObj._completeAction(details);

                    }
                    else
                        xlObj._showAlertDlg("Alert", alertText, "", 500);
                }
            }
            canclose && $('#' + xlObj._id + '_fatresizetabledlg').ejDialog('close');
        },

        _fatNameDlgBtnClick: function (args) {
            var obj, xlObj = this.XLObj, sid = xlObj._id;
            $('#' + sid + '_fatnamedlg').ejDialog('close');
            if (args.model.text === xlObj._getLocStr('Ok')) {
                obj = { header: $("#" + sid + "_fatheader").ejCheckBox("checked"), name: document.getElementById(sid + '_fatname').value, format: xlObj.XLFormat._formatAsTableStyle.format };
                xlObj.XLFormat._formatAsTableStyle.formatName && (obj["formatName"] = xlObj.XLFormat._formatAsTableStyle.formatName);
                xlObj.XLFormat.createTable(obj);
            }
        },

        _validateResizeRange: function (rrange, trange) {
            var xlObj = this.XLObj;
            if (rrange.length) {
                if (rrange[0] != trange[0] || rrange[1] != trange[1])
                    return "ResizeAlert";
                else if (rrange[3] != trange[3] || rrange[2] < trange[2])
                    return "ResizeRestrictAlert";
                else if (rrange[2] > xlObj.getSheet(xlObj.getActiveSheetIndex()).rowCount)
                    return "RangeNotCreated"
            }
            else
                return "InvalidReference";
        },

        _chartNameDlgBtnClick: function (args) {
            var cObj, details, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), dataVal = xlObj.getSheet(sheetIdx)._shapeMngr.chart[xlObj.XLEdit.getPropertyValue(xlObj.XLShape._picCellIdx.rowIndex, xlObj.XLShape._picCellIdx.colIndex, 'chart')[0]], cname, sid = xlObj._id, cid = document.getElementById(sid + '_chart').value;
            cObj = $("#" + cid).data("ejChart");
            details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", cid: cid, rowIndex: xlObj.XLShape._picCellIdx.rowIndex, colIndex: xlObj.XLShape._picCellIdx.colIndex, prev: {}, cur: {} };
            cname = document.getElementById(sid + '_chartname').value;
            if (args.model.text === xlObj._getLocStr('Ok')) {
                if (!cname.length) {
                    xlObj._showAlertDlg("Alert", "InvalidTitle", "InvalidChartTitleActn", 375);
                    return;
                }
                switch (xlObj._hasTitle) {
                    case "PX":
                        cObj.option("primaryXAxis", { title: { text: cname } });
                        dataVal.xAxis.title.text = cname;
                        details.action = "PX";
                        break;
                    case "PY":
                        cObj.option("primaryYAxis", { title: { text: cname } });
                        dataVal.yAxis.title.text = cname;
                        details.action = "PY";
                        break;
                    case "CT":
                        cObj.option("title", { text: cname });
                        dataVal.title.text = cname;
                        details.action = "CT";
                        break;
                }
            }
            details.prev.text = xlObj._cOpt.prevText;
            if (xlObj._hasTitle == "CT") {
                details.prev.align = xlObj._cOpt.prevAlign;
                details.cur.align = xlObj._cOpt.curAlign;
            }
            details.cur.text = cname;
            xlObj._completeAction(details);
            xlObj._cOpt = {};
            document.getElementById(sid + '_chartname').value = "";
            $('#' + sid + '_chartnamedlg').ejDialog('close');
        },

        _designTabUpdate: function (tid, cellIdx) {
            var xlObj = this.XLObj;
            if (xlObj.model.showRibbon) {
                var xlId = xlObj._id, btnObj, robj = $('#' + xlObj._id + '_Ribbon').ejRibbon('instance'), tmgr = xlObj.getSheet(xlObj.getActiveSheetIndex())._tableManager;
                robj.showTab(xlObj._getLocStr('Design'));
                (this._isDesignTab && !robj._isCollapsed) && robj.option({ selectedItemIndex: this._getTabIndex("design") });
                document.getElementById(xlObj._id + '_tablename').value = tmgr[tid].name;
                document.getElementById(xlObj._id + '_tableid').value = tid;
                this._isSetModel = true;
                $("#" + xlId + "_Ribbon_tsototalrow").data("ejCheckBox").option({ checked: (tmgr[tid]["totalRow"] ? true : false) });
                $("#" + xlId + "_Ribbon_tsofirstcolumn").data("ejCheckBox").option({ checked: (tmgr[tid]["firstColumn"] ? true : false) });
                $("#" + xlId + "_Ribbon_tsolastcolumn").data("ejCheckBox").option({ checked: (tmgr[tid]["lastColumn"] ? true : false) });
                $("#" + xlId + "_Ribbon_tsofiltercolumn").data("ejCheckBox").option({ checked: (tmgr[tid]["isFilter"] ? true : false) });
                this._isSetModel = false;
                btnObj = $("#" + xlObj._id + "_Ribbon_clearfilter").data("ejButton");
                this._changeClrFltrStatus(btnObj, "button", cellIdx);
            }
        },

        _formatTabUpdate: function () {
            var xlObj = this.XLObj;
            if (xlObj.model.showRibbon) {
                var robj = $("#" + xlObj._id + "_Ribbon").ejRibbon("instance");
                robj.showTab("Format");
                xlObj._shapeChange = true;
                xlObj.element.find("#" + xlObj._id + "_pictureheight").ejNumericTextbox("option", "value", xlObj.element.find(".e-ss-activeimg").height());
                xlObj.element.find("#" + xlObj._id + "_picturewidth").ejNumericTextbox("option", "value", xlObj.element.find(".e-ss-activeimg").width());
                if (xlObj.model.showRibbon && !robj._isCollapsed) robj.option({ selectedItemIndex: xlObj.XLRibbon._getTabIndex("format") });
                xlObj._shapeChange = false;
            }
        },

        _chartDesignTabUpdate: function (chartElem) {
            var xlObj = this.XLObj;
            if (xlObj.model.showRibbon) {
                var robj = $('#' + xlObj._id + '_Ribbon').data('ejRibbon'), cid = chartElem.get(0).id, left = chartElem.get(0).offsetLeft, top = chartElem.get(0).offsetTop;
                xlObj.XLShape._picCellIdx = xlObj.XLShape._getCellIndexFromOffset(top, left);
                robj.showTab('Chart Design');
                !robj._isCollapsed && robj.option({ selectedItemIndex: xlObj.XLRibbon._getTabIndex("cdesign") });
                document.getElementById(xlObj._id + '_chart').value = cid;
                this._isSetModel = true;
                this._setChartWidthHeight({ height: chartElem.height(), width: chartElem.width() });
                robj.element.find("#" + xlObj._id + "_Ribbon_chartTheme").ejDropDownList("setSelectedValue", chartElem.ejChart("option", "theme"));
                this._isSetModel = false;
            }
        },

        _setChartWidthHeight: function (options) {
            var xlObj = this.XLObj, wNum = $("#" + xlObj._id + "_Ribbon_chartwidth").data("ejNumericTextbox"), hNum = $("#" + xlObj._id + "_Ribbon_chartheight").data("ejNumericTextbox");
            xlObj._shapeChange = true;
            hNum.option({ value: options.height });
            wNum.option({ value: options.width });
            xlObj._shapeChange = false;
        },

        _toggleDesignTab: function (target) {
            var xlObj = this.XLObj, btnObj;
            if (xlObj.model.showRibbon) {
                var rObj = $('#' + xlObj._id + '_Ribbon').data("ejRibbon");
                if (ej.isNullOrUndefined(target) || (!ej.isNullOrUndefined(target) && target.className.indexOf("e-ft") < 0)) {
                    rObj.hideTab(xlObj._getLocStr('Design'));
                    !rObj._isCollapsed && rObj.option({ selectedItemIndex: 1 });
                }
            }
            btnObj = $("#" + xlObj._id + "_Ribbon_clearfilter").data("ejButton");
            (ej.isNullOrUndefined(target)) && (target = xlObj.getCell(xlObj.getActiveCell().rowIndex, xlObj.getActiveCell().colIndex)[0]);
            this._changeClrFltrStatus(btnObj, "button", xlObj._getCellIdx(target));
        },

        _toggleChartDesignTab: function () {
            var xlObj = this.XLObj;
            if (xlObj.model.showRibbon) {
                var rObj = $('#' + xlObj._id + '_Ribbon').data("ejRibbon");
                rObj.hideTab('Chart Design');
                !rObj._isCollapsed && rObj.option({ selectedItemIndex: 1 });
            }
        },

        _toggleFormatTab: function () {
            var xlObj = this.XLObj;
            if (xlObj.model.showRibbon) {
                var rObj = $('#' + xlObj._id + '_Ribbon').data("ejRibbon");
                rObj.hideTab('Format');
                !rObj._isCollapsed && rObj.option({ selectedItemIndex: 1 });
            }
        },

        _getTabIndex: function (name) {
            if (!this.XLObj.model.showRibbon)
                return;
            var model = $('#' + this.XLObj._id + '_Ribbon').ejRibbon("model"), tabs = model.tabs, ctabs = model.contextualTabs, i = 0;
            while (i < ctabs.length) {
                tabs = tabs.concat(ctabs[i].tabs);
                i++;
            }
            i = tabs.length;
            while (i--)
                if (tabs[i].id === name) return ++i;
        },

        _getFormatAsTableInput: function (str) {
            var xlObj = this.XLObj, str1, str2, istr, cstr, i = 0, j = 0, arr = [], abbr = { hl: 'headerLayout', cl: 'contentLayout', hb: 'headerBorder', cb: 'contentBorder' }, obj = {};
            str = str.split('&'); // for layout input
            while (i < str.length) {
                istr = str[i].split('?'); // to separate headerlayout: '' like that
                str1 = istr[0], str2 = istr[1];
                if (str1 === 'hl')
                    obj[abbr[str1]] = xlObj._getStyleAsJSON(str2, true);
                else if (str1 === 'cl') {
                    cstr = str2.split(','); // for multiple content layout
                    while (j < cstr.length) {
                        arr.push(cstr[j].length ? xlObj._getStyleAsJSON(cstr[j], true) : {});
                        j = j + 1;
                    }
                    obj[abbr[str1]] = arr;
                }
                else if (str1 === 'cb') {
                    cstr = str2.split('-'); // for border
                    obj[abbr[str1]] = { border: this._getBorderObj(cstr[0], cstr[1]) };
                }
                i = i + 1;
            }
            return obj;
        },

        _getBorderObj: function (prop, value) {
            var i, border = { t: { top: '' }, r: { right: '' }, b: { bottom: '' }, l: { left: '' }, o: { top: '', right: '', bottom: '', left: '' }, tb: { top: '', bottom: '' } };
            if (prop === 'gb')
                return value;
            for (i in border[prop])
                border[prop][i] = value;
            return border[prop];
        },

        // Cell Style Feature
        _renderCellStyles: function () {
            var txt, i = 0, cdiv = [], str = '', index = [4, 2, 12], xlObj = this.XLObj, fcoll = xlObj._dataContainer.hashCode,
            cstylehdrtext = xlObj._getLocStr('CellStyleHeaderText').split('/'),
            gbntext = xlObj._getLocStr('CellStyleGBN').split('/'), gbnformat = [fcoll[0], fcoll[1], fcoll[2], fcoll[3]],
            thtext = xlObj._getLocStr('CellStyleTH').split('/'), thformat = fcoll.slice(4, 6),
            tcstext = xlObj._getLocStr('CellsStyleTCS').split('/'), tcsformat = fcoll.slice(6, 18);
            while (i < index[0]) {
                txt = gbntext[i];
                str = str + xlObj._renderDIV('e-cellstylecell ' + gbnformat[i], txt, txt);
                i = i + 1;
            }
            cdiv.push(str), str = '', i = 0;
            while (i < index[1]) {
                txt = thtext[i];
                if (i === 1)
                    str = str + '<div class="e-cellstylecell ' + thformat[i] + '"  style="font-size: 11pt;" title=' + txt + '>' + txt + '</div>';
                else
                    str = str + xlObj._renderDIV('e-cellstylecell ' + thformat[i], txt, txt);
                i = i + 1;
            }
            cdiv.push(str), str = '', i = 0;
            while (i < index[2]) {
                txt = tcstext[i];
                str = str + xlObj._renderDIV('e-cellstylecell ' + tcsformat[i], txt, txt);
                i = i + 1;
            }
            cdiv.push(str), str = '', i = 0;
            while (i < cstylehdrtext.length * 2) {
                str = str + xlObj._renderDIV(i % 2 === 0 ? 'e-cellstyleheader' : 'e-cellstylecontent', i % 2 === 0 ? cstylehdrtext[i / 2] : cdiv[0]);
                i % 2 != 0 && cdiv.shift();
                i++;
            }
            return $('<div id=' + xlObj._id + '_cellstyles class="e-spreadsheet e-cellstyles">' + str + '</div>');
        },

        _onTabClick: function (args) {
            var xlObj = this.XLObj, settings = $.extend(true, {}, xlObj.model.scrollSettings);
            args.prevActiveIndex !== -1 && xlObj.model.allowComments && xlObj.model.allowEditing && (xlObj.XLEdit._isEdit || xlObj.XLComment._isCommentEdit || xlObj._isSheetRename || !xlObj.getSheet(xlObj.getActiveSheetIndex())._isSheetProtected) && this._enableRibbonIcons(args.activeIndex);
            xlObj._trigger("tabClick", args);
        },

        _onCollapse: function (args, e) {
            var xlObj = this.XLObj, settings = $.extend(true, {}, xlObj.model.scrollSettings);
            this._ribbonState = false;
            args.action = "toggleBtn";
            this._heightWidthCalculation(xlObj.getActiveSheetIndex(), settings, args);
        },

        _onTabSelect: function (args) {
            var xlObj = this.XLObj, sIndex = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sIndex), activeEle = xlObj.getActiveCellElem(sIndex);
            (xlObj.XLEdit._isEdit || (xlObj.model.allowComments && xlObj.XLComment._isCommentEdit) || xlObj._isSheetRename || sheet._isSheetProtected) && this._disableRibbonIcons();
            this._updateRibbonIcons();
            if (activeEle.length && activeEle[0].className.indexOf("e-table") != -1)
                this._isDesignTab = (args.activeIndex == this._getTabIndex("design"));
            xlObj._trigger("tabSelect", args);
        },

        _valDTypeOnChange: function (args) {
            var xlObj = this.XLObj;
            if ($.validator)
                $("#" + xlObj._id + "_Form_ValDialog").validate().resetForm();
            if (args.value === "Between" || args.value === "NotBetween") {
                $("#" + xlObj._id + "_Ribbon_lbMax").show();
                $("#" + xlObj._id + "_Ribbon_btnMax").show();
                $("#" + xlObj._id + "_Ribbon_lbMin").text(xlObj._getLocStr("NumberStart"));
            }
            else {
                $("#" + xlObj._id + "_Ribbon_lbMax").hide();
                $("#" + xlObj._id + "_Ribbon_btnMax").hide();
                $("#" + xlObj._id + "_Ribbon_lbMin").text(xlObj._getLocStr("DataValue"));
            }
            this._dialogValidate("_ValDialog");
        },

        _valTypeOnChange: function (args) {
            var xlObj = this.XLObj, actionObj = $("#" + xlObj._id + "_Ribbon_ddAction").data("ejDropDownList");
            actionObj.enable();
            if (args.value === "List") {
                $("#" + xlObj._id + "_Ribbon_lbMax").hide();
                $("#" + xlObj._id + "_Ribbon_btnMax").hide();
                actionObj.disable();
            }
            else if (actionObj._selectedValue === "Between" || actionObj._selectedValue === "NotBetween") {
                $("#" + xlObj._id + "_Ribbon_lbMax").show();
                $("#" + xlObj._id + "_Ribbon_btnMax").show();
            }
            $("#" + xlObj._id + "_Ribbon_lbMin").text(xlObj._getLocStr(args.selectedText + "Start"));
            $("#" + xlObj._id + "_Ribbon_lbMax").text(xlObj._getLocStr(args.selectedText + "End"));
            this._dialogValidate("_ValDialog");
        },

        _valDlgBtnClick: function (id, args) {
            var xlObj = this.XLObj, alertdlg, sheetIdx;
            if (id === "ok" && $.validator && $("#" + xlObj._id + "_Form_ValDialog").valid()) {
                var cbObj = $("#" + xlObj._id + "_Ribbon_blank").data("ejCheckBox");
                var showErrorAlert = $("#" + xlObj._id + "_Ribbon_freetext").data("ejCheckBox").isChecked();
                var typeObj = $("#" + xlObj._id + "_Ribbon_ddType").data("ejDropDownList");
                var actionObj = $("#" + xlObj._id + "_Ribbon_ddAction").data("ejDropDownList");
                if (typeObj._selectedValue === "List") {
                    var value = $("#" + xlObj._id + "_Ribbon_btnMin").val(), val = value, splitStr, range, alertdlg = $("#" + xlObj._id + "_alertdlg"), getSheet;
                    if (value.indexOf("=") > -1) {
                        value = value.split("=")[1];
                        if (value.indexOf("!") > -1) {
                            splitStr = value.split("!");
                            sheetIdx = xlObj._getSheetIndexByName(splitStr[0]);
                            value = splitStr[1];
                        }
                        if (sheetIdx === false) {
                            xlObj._renderAlertDlgContent(alertdlg, "Alert", xlObj._getLocStr("ListAlert"), "ListValidation");
                            xlObj._alertDialog.element.ejDialog("open") && alertdlg.focus();
                            return;
                        }
                        value.indexOf("$") > -1 && (value = value.split("$").join(""));
                        value.indexOf(":") < 0 && (value = value + ":" + value);
                        sheetIdx = xlObj.isUndefined(sheetIdx) ? xlObj.getActiveSheetIndex() : sheetIdx;
                        getSheet = xlObj.getSheet(sheetIdx);
                        range = xlObj.getRangeIndices(value);
                        if (isNaN(range[0]) || isNaN(range[1]) || isNaN(range[2]) || isNaN(range[3]) || getSheet.colCount <= range[3] || getSheet.rowCount <= range[2]) {
                            xlObj._renderAlertDlgContent(alertdlg, "Alert", xlObj._getLocStr("ListAlert"), "ListValidation");
                            xlObj._alertDialog.element.ejDialog("open") && alertdlg.focus();
                            return;
                        }
                        if (range[2] === range[0] || range[3] === range[1]) {
                            xlObj.XLValidate.applyDVRules(null, ["list", val], this._types[typeObj.selectedIndexValue].value, cbObj.isChecked(), showErrorAlert);
                        }
                        else {
                            xlObj._renderAlertDlgContent(alertdlg, "Alert", xlObj._getLocStr("ListValAlert"), "ListValidation");
                            xlObj._alertDialog.element.ejDialog("open") && alertdlg.focus();
                            return;
                        }
                    }
                    else
                        xlObj.XLValidate.applyDVRules(null, ["list", val], this._types[typeObj.selectedIndexValue].value, cbObj.isChecked(), showErrorAlert);
                }
                else
                    xlObj.XLValidate.applyDVRules(null, [this._actions[actionObj.selectedIndexValue].value, $("#" + xlObj._id + "_Ribbon_btnMin").val(), $("#" + xlObj._id + "_Ribbon_btnMax").val()], this._types[typeObj.selectedIndexValue].value, cbObj.isChecked(), showErrorAlert);
                xlObj._setddlCell();
            }
            else if (!$.validator)
                id = "cancel";
            if (id === "cancel" || (id === "ok" && $.validator && $("#" + xlObj._id + "_Form_ValDialog").valid())) {
                $("#" + xlObj._id + "_Ribbon_ddType").data("ejDropDownList").option({ selectedItemIndex: 0 });
                $("#" + xlObj._id + "_Ribbon_ddAction").data("ejDropDownList").option({ selectedItemIndex: 6 });
                $("#" + xlObj._id + "_Ribbon_blank").data("ejCheckBox").option({ checked: true });
                $("#" + xlObj._id + "_Ribbon_freetext").data("ejCheckBox").option({ checked: true });
                $("#" + xlObj._id + "_Ribbon_btnMin").val("");
                $("#" + xlObj._id + "_Ribbon_btnMax").val("");
                $("#" + xlObj._id + "_ValDialog").ejDialog("close");
                xlObj.setSheetFocus();
            }

        },

        _renderValDialog: function () {
            var xlObj = this.XLObj;
            if ($("#" + xlObj._id + "_Ribbon_MainDiv").length < 1) {
                var $stgctnr = ej.buildTag("div.e-dlgctndiv"), $mainDiv = ej.buildTag("div.e-dlg-fields", "", {}, { id: xlObj._id + "_Ribbon_MainDiv" }), $subDiv;
                $("#" + xlObj._id + "_Validation").append($mainDiv);
                var $label = ej.buildTag("label.e-dlg-fields", xlObj._getLocStr("Allow"));
                $mainDiv.append($label);
                var $ddType = ej.buildTag("input", "", "", { id: xlObj._id + "_Ribbon_ddType", type: "text" });
                $mainDiv.append($ddType);
                $ddType.ejDropDownList({
                    dataSource: this._types,
                    fields: { id: "Types", text: "text", value: "text" }, selectedItemIndex: 1,
                    width: 235,
                    height: 25,
                    change: $.proxy(this._valTypeOnChange, this)
                });
                var $chkinput = ej.buildTag("input#" + xlObj._id + "_Ribbon_blank");
                var $chklabel = ej.buildTag("label", xlObj._getLocStr("IgnoreBlank"), {}, { "for": xlObj._id + "_Ribbon_blank" });
                var table = ej.buildTag("table");
                var tr = ej.buildTag("tr");
                var td = ej.buildTag("td");
                td.append($chkinput);
                tr.append(td);
                td = ej.buildTag("td");
                td.append($chklabel);
                tr.append(td);
                table.append(tr);
                $chkinput.ejCheckBox();
                $chkinput = ej.buildTag("input#" + xlObj._id + "_Ribbon_freetext");
                $chklabel = ej.buildTag("label#" + xlObj._id + "_Ribbon_lblfreetext", xlObj._getLocStr("FreeText"), {}, { "for": xlObj._id + "_Ribbon_lblfreetext" });
                tr = ej.buildTag("tr");
                td = ej.buildTag("td");
                td.append($chkinput);
                tr.append(td);
                td = ej.buildTag("td");
                td.append($chklabel);
                tr.append(td);
                table.append(tr);
                $mainDiv.append(table);
                $chkinput.ejCheckBox({ checked: true });
                $label = ej.buildTag("label.e-dlg-fields").text(xlObj._getLocStr("Data"));
                $mainDiv.append($label);
                var $ddAction = ej.buildTag("input", "", "", { id: xlObj._id + "_Ribbon_ddAction", type: "text" });
                $mainDiv.append($ddAction);
                $ddAction.ejDropDownList({
                    dataSource: this._actions,
                    fields: { id: "Types", text: "text", value: "value" }, selectedItemIndex: 6,
                    width: 235,
                    height: 25,
                    change: $.proxy(this._valDTypeOnChange, this)
                });
                $subDiv = ej.buildTag("div.e-dlgctndiv e-dlg-fields", "", {}, { id: xlObj._id + "_Ribbon_SubDiv" });
                table = ej.buildTag("table");
                tr = ej.buildTag("tr"), td = ej.buildTag("td");
                $label = ej.buildTag("label.e-dlg-fields", "", "", { id: xlObj._id + "_Ribbon_lbMin" }).text(xlObj._getLocStr("NumberStart"));
                table.append(tr.append(td.append($label)));
                tr = ej.buildTag("tr"), td = ej.buildTag("td");
                var $min = ej.buildTag("input.ejinputtext", "", { width: 235 }, { id: xlObj._id + "_Ribbon_btnMin", type: "text", name: xlObj._id + "_valMin" });
                table.append(tr.append(td.append($min)));
                tr = ej.buildTag("tr"), td = ej.buildTag("td");
                $label = ej.buildTag("label.e-dlg-fields", "", "", { id: xlObj._id + "_Ribbon_lbMax" }).text(xlObj._getLocStr("NumberEnd"));
                table.append(tr.append(td.append($label)));
                tr = ej.buildTag("tr"), td = ej.buildTag("td");
                var $max = ej.buildTag("input.ejinputtext", "", { width: 235 }, { id: xlObj._id + "_Ribbon_btnMax", type: "text", name: xlObj._id + "_valMax" });
                table.append(tr.append(td.append($max)));
                $stgctnr.append($mainDiv, $subDiv.append(table)).height(294);
                $("#" + xlObj._id + "_Validation").append($stgctnr);
                $("#" + xlObj._id + "_Ribbon_SubDiv").wrapInner('<form id="' + xlObj._id + '_Form_ValDialog" onsubmit="return false"></form>');
                this._dialogValidate("_ValDialog");
            }
        },

        _dialogValidate: function (dlgid, inputid) {
            var xlObj = this.XLObj, id = xlObj._id, obj, elem = $("#" + id + "_Form" + dlgid), val, i, len, cellAddr, webAddr, date, inputLength, input, number, time, splitStr, sheetName = true, type, condType, decimal;
            if ($.validator) {
                elem.validate({
                    errorElement: "div",
                    wrapper: "div",
                    errorPlacement: function (error, elem) {
                        error.insertAfter(elem.closest("input"));
                        error.children().addClass("e-dlg-field-validation-error");
                        error.css({ "position": "relative" });
                        error.offset({ left: elem.offset().left, top: elem.offset().top + 19 });
                    },
                    onfocusout: function (element) {
                        this.element(element);
                    },
                });
                type = $("#" + id + "_Ribbon_ddAction").ejDropDownList("getValue");
                switch (dlgid) {
                    case "_ValDialog":
                        obj = $("#" + id + "_Ribbon_ddType").data("ejDropDownList");
                        val = obj.getSelectedValue();
                        break;
                    case "_CFDialog":
                        val = inputid.charAt(0).toUpperCase() + inputid.slice(1).toLowerCase();
                        break;
                    case "_GoToDiv":
                        val = "cellAddr";
                        break;
                    case "_Ribbon_cellAddress":
                        val = "cellAddr";
                        break;
                    case "_Ribbon_webAddress":
                        val = "webAddr";
                        break;
                }
                $("#" + id + "_Form" + dlgid).validate().resetForm();
                input = new Array(), inputLength = $("#" + id + "_Form" + dlgid).find("input").length, date = { required: true, date: true, messages: { date: xlObj._getLocStr("DateValidationMsg"), required: xlObj._getLocStr("RequiredField") } }, cellAddr = { required: true, celladdr: true }, number = { required: true, numberVal: true };
                decimal = { required: true, number: true, messages: { required: xlObj._getLocStr("RequiredField"), number: xlObj._getLocStr("NumberValidationMsg") } }, time = { required: true, time: true }, webAddr = { required: true }, condType = { required: true, betweentype: true };
                $.validator.addMethod("time", function (value, elem, param) {
                    return value === "" || value.match(/^(([0-1]?[0-9])|([2][0-3])):([0-5]?[0-9])(:([0-5]?[0-9]))?(\s(AM|am|PM|pm))?$/);
                }, xlObj._getLocStr("TimeValidationMsg"));
                $.validator.addMethod("date", function (value, elem, param) {
                    var dateSplit = value.split("/");
                    return this.optional(elem) || !/Invalid|NaN/.test(new Date((/chrom(e|ium)/.test(navigator.userAgent.toLowerCase())) ? dateSplit[1] + "/" + dateSplit[0] + "/" + dateSplit[2] : value));
                }, xlObj._getLocStr("DateValidationMsg"));
                $.validator.addMethod("celladdr", function (value, elem, param) {
                    var range, xlObj = $("#" + elem.id.split("_")[0]).data("ejSpreadsheet"), sheetIdx = $("#" + xlObj._id + "_Ribbon_sheentName").ejTreeView("option", "selectedNode"), getSheet;
                    sheetIdx = sheetIdx < 0 ? xlObj.getActiveSheetIndex() : sheetIdx;
                    getSheet = xlObj.getSheet(sheetIdx);
                    (value.indexOf("=") > -1) && (value = value.split('=')[1]);
                    if (value.indexOf("!") > -1) {
                        splitStr = value.split("!");
                        sheetName = $("#" + elem.id.split("_textRef")[0]).data("ejSpreadsheet")._getSheetIndexByName(splitStr[0]);
                        value = splitStr[1];
                    }
                    value.indexOf("$") > -1 && (value = value.split("$").join(""));
                    range = xlObj.getRangeIndices(value);
                    range = (getSheet.colCount <= range[3] || getSheet.rowCount <= range[2]) ? null : value;
                    return (value.match(/^(?:\bxfXF[a-zA-D]|xX[a-eA-E][a-zA-Z]|[a-wA-W][a-zA-Z]{2}|[a-zA-Z]{2}|[a-zA-Z])\$?(?:104857[0-6]|10485[0-6]\d|1048[0-4]\d{2}|104[0-7]\d{3}|10[0-3]\d{4}|[1-9]\d{1,5}|[1-9]):(?:\bxfXF[a-zA-D]|xX[a-eA-E][a-zA-Z]|[a-wA-W][a-zA-Z]{2}|[a-zA-Z]{2}|[a-zA-Z])\$?(?:104857[0-6]|10485[0-6]\d|1048[0-4]\d{2}|104[0-7]\d{3}|10[0-3]\d{4}|[1-9]\d{1,5}|[1-9])$/) || value.match(/^(?:\bxfXF[a-zA-D]|xX[a-eA-E][a-zA-Z]|[a-wA-W][a-zA-Z]{2}|[a-zA-Z]{2}|[a-zA-Z])\$?(?:104857[0-6]|10485[0-6]\d|1048[0-4]\d{2}|104[0-7]\d{3}|10[0-3]\d{4}|[1-9]\d{1,5}|[1-9])$/) && (sheetName)) && range;
                }, xlObj._getLocStr("cellAddrsValidationMsg"));
                $.validator.addMethod("betweentype", function (value, elem, param) {
                    var id = elem.id.split("_")[0], val = $("#" + elem.id.split("_")[0] + "_Ribbon_ddType").ejDropDownList("getSelectedValue"), minVal = $("#" + id + "_Ribbon_btnMin").val(), maxVal = $("#" + elem.id.split("_")[0] + "_Ribbon_btnMax").val();
                    if (minVal === "" || maxVal === "" || parseFloat(minVal) > parseFloat(maxVal))
                        value = null;
                    if (val === "Date" && Date.parse(minVal) > Date.parse(maxVal))
                        value = null;
                    if (val === "Time" && new Date("01/01/1990 " + minVal).getTime() > new Date("01/01/1990 " + maxVal).getTime())
                        value = null;
                    return value;
                }, xlObj._getLocStr("BetweenAlert"));
                $.validator.addMethod("numberVal", function (value, elem, param) {
                    if ($.isNumeric(value)) {
                        if (parseFloat(value) % 1 !== 0)
                            value = null;
                        $.validator.messages.numberVal = xlObj._getLocStr("NumberValMsg");
                        return value;
                    }
                    else
                        $.validator.messages.numberVal = xlObj._getLocStr("NumberAlertMsg");
                }, $.validator.messages.numberVal);
                for (i = 0, len = inputLength; i < len ; i++) {
                    input[i] = $("#" + id + "_Form" + dlgid).find("input")[i].id;
                    switch (val) {
                        case "Date":
                            $("#" + input[i]).rules("remove");
                            $("#" + input[i]).rules("add", date);
                            break;
                        case "Time":
                            $("#" + input[i]).rules("remove");
                            $("#" + input[i]).rules("add", time);
                            break;
                        case "Number":
                        case "TextLength":
                            $("#" + input[i]).rules("remove");
                            $("#" + input[i]).rules("add", number);
                            break;
                        case "Decimal":
                            $("#" + input[i]).rules("remove");
                            $("#" + input[i]).rules("add", decimal);
                            break;
                        case "List":
                            $("#" + input[i]).rules("remove");
                            $("#" + input[i]).rules("add", webAddr);
                            break;
                        case "cellAddr":
                            $("#" + input[i]).rules("remove");
                            $("#" + input[i]).rules("add", cellAddr);
                            break;
                        case "webAddr":
                            $("#" + input[i]).rules("remove");
                            $("#" + input[i]).rules("add", webAddr);
                            break;
                    }
                    if (val !== "List" && (type === "between" || type === "not between"))
                        $("#" + id + "_Ribbon_btnMax").rules("add", condType);
                }
            }
        },

        _renderCFDialog: function (args) {
            var $mainDiv, $subDiv, $label, $input, dlgObj, xlObj = this.XLObj;
            dlgObj = $("#" + xlObj._id + "_CFDialog").data("ejDialog");
            dlgObj.option({ title: xlObj._getLocStr(this._cFormatData[args.ID][0]) });
            if ($("#" + xlObj._id + "_Ribbon_CFSubDiv").length < 1) {
                $subDiv = ej.buildTag("div.e-dlg-fields", "", {}, { id: xlObj._id + "_Ribbon_CFSubDiv" });
                $label = ej.buildTag("label.e-dlg-fields").text(xlObj._getLocStr("With"));
                $subDiv.append($label);
                var $ddStyle = ej.buildTag("input", "", {}, { id: xlObj._id + "_Ribbon_ddStyle", type: "text" });
                $subDiv.append($ddStyle);
                $ddStyle.ejDropDownList({
                    dataSource: this._styles,
                    fields: { id: "Types", text: "text", value: "value" }, selectedItemIndex: 0,
                    width: "100%",
                    height: 25,
                    change: $.proxy(this._ddOnChange, this),
                });
                $("#" + xlObj._id + "_CondFormat").append($subDiv);
                $("#" + xlObj._id + "_CondFormat").addClass("e-cfdiv");
            }
            if ($("#" + xlObj._id + "_Ribbon_CFMainDiv").length > 0)
                $("#" + xlObj._id + "_Ribbon_CFMainDiv").remove();
            $mainDiv = ej.buildTag("div.e-dlg-fields", "", "", { id: xlObj._id + "_Ribbon_CFMainDiv" });
            $("#" + xlObj._id + "_CondFormat").prepend($mainDiv);
            if (args.ID == "between") {
                $input = ej.buildTag("input.ejinputtext", "", {}, { id: xlObj._id + "_Ribbon_input2", type: "text" });
                $mainDiv.prepend($input);
                $label = ej.buildTag("label.e-dlg-fields").text(xlObj._getLocStr("And")).css('display', "table");
                $mainDiv.prepend($label);
            }
            $input = ej.buildTag("input.ejinputtext", "", {}, { id: xlObj._id + "_Ribbon_input1", type: "text" });
            $mainDiv.prepend($input);
            $label = ej.buildTag("label.e-dlg-fields").text(xlObj._getLocStr(this._cFormatData[args.ID][1]));
            $mainDiv.prepend($label);
            (args.ID === "aboveavg" || args.ID === "belowavg") && $input.hide();
            $("#" + xlObj._id + "_Ribbon_CFMainDiv").wrapInner('<form id="' + xlObj._id + '_Form_CFDialog" onsubmit="return false"></form>');
            $input.is(":visible") && $input.focus().setInputPos($input.val().length);
            this._dialogValidate("_CFDialog", args.ID);
        },

        _dlgCFOk: function () {
            var xlObj = this.XLObj;
            if ($("#" + xlObj._id + "_Form_CFDialog").valid()) {
                var ddObj = $("#" + xlObj._id + "_Ribbon_ddStyle").data("ejDropDownList");
                xlObj.XLCFormat._cFormat(xlObj.XLRibbon._currentCFormat, $("#" + xlObj._id + "_Ribbon_input1").val(), $("#" + xlObj._id + "_Ribbon_input2").val(), ddObj.getSelectedValue());
                $("#" + xlObj._id + "_CFDialog").ejDialog("close");
                xlObj.setSheetFocus();
            }
        },

        _dlgCFCancel: function () {
            var xlObj = this.XLObj;
            $("#" + xlObj._id + "_CFDialog").ejDialog("close");
            xlObj.setSheetFocus();
        },

        _sheetData: function (e) {
            var xlObj = this.XLObj, sheetname = new Array(), shtnameclt = new Array(), i, len, sheets = xlObj._getSheetNames();
            shtnameclt[0] = { id: 1, name: xlObj._getLocStr("SheetReference"), hasChild: true, expanded: true };
            for (i = 1, len = sheets.length; i <= len; i++) {
                if (sheets[i - 1].isVisible) {
                    sheetname[i] = sheets[i - 1].text;
                    shtnameclt[i] = { id: i + 1, pid: 1, name: sheetname[i], hasChild: true, expanded: true };
                }
            }
            return shtnameclt;
        },

        _renderHLDialog: function () {
            var xlObj = this.XLObj;
            if ($("#" + xlObj._id + "_Ribbon_WebDiv").length < 1) {
                var $WebDiv, $label, $find, $SheetDiv, $lb, $div, $table, $tr, $tr, $divctnr;
                $WebDiv = ej.buildTag("div.e-dlgctndiv", "", {}, { id: xlObj._id + "_Ribbon_WebDiv" });
                $table = ej.buildTag("table", "", {}, { "cellpadding": 0, "cellspacing": 0 }), $tr = ej.buildTag("tr"), $td = ej.buildTag("td", "", { width: "32%" });
                $table.append($tr.append($td));
                $label = ej.buildTag("label", xlObj._getLocStr("HyperlinkText"));
                $td.append($label);
                $td = ej.buildTag("td");
                $find = ej.buildTag("input.ejinputtext", "", {}, { id: xlObj._id + "_Ribbon_findText", type: "text" });
                $tr.append($td.append($find));

                $tr = ej.buildTag("tr.e-dlgtd-fields"), $td = ej.buildTag("td");
                $table.append($tr.append($td));
                $label = ej.buildTag("label", xlObj._getLocStr("WebAddress"), "", { id: xlObj._id + "_Ribbon_address" });
                $td.append($label);
                $td = ej.buildTag("td");
                $find = ej.buildTag("input.ejinputtext", "", {}, { id: xlObj._id + "_Ribbon_webAddress", type: "text" });
                $tr.append($td.append($find));
                $("#" + xlObj._id + "_Web").append($WebDiv.append($table));

                $SheetDiv = ej.buildTag("div.e-dlgctndiv", "", "", { id: xlObj._id + "_Ribbon_DocDiv" });
                $divctnr = ej.buildTag("div.e-dlg-fields");
                $table = ej.buildTag("table"), $tr = ej.buildTag("tr"), $td = ej.buildTag("td", "", { width: "32%" });
                $table.append($tr.append($td));
                $label = ej.buildTag("label", xlObj._getLocStr("HyperlinkText"));
                $td.append($label);
                $td = ej.buildTag("td");
                $find = ej.buildTag("input.ejinputtext", "", {}, { id: xlObj._id + "_Ribbon_cellText", type: "text" });
                $tr.append($td.append($find));

                $tr = ej.buildTag("tr.e-dlgtd-fields"), $td = ej.buildTag("td");
                $table.append($tr.append($td));
                $label = ej.buildTag("label", xlObj._getLocStr("CellAddress"));
                $td.append($label);
                $td = ej.buildTag("td");
                $find = ej.buildTag("input.ejinputtext", "", {}, { id: xlObj._id + "_Ribbon_cellAddress", type: "text" });
                $tr.append($td.append($find));
                $div = ej.buildTag("div.e-dlg-fields", "", "", { id: xlObj._id + "_Ribbon_Sheet" });
                $SheetDiv.append($divctnr.append($table), $div);
                $label = ej.buildTag("label", xlObj._getLocStr("SheetIndex"));
                $div.append($label);
                $lb = ej.buildTag("div", "", "", { id: xlObj._id + "_Ribbon_sheentName" });
                $div.append($lb);
                $lb.ejTreeView({ fields: { id: "id", parentId: "pid", text: "name", hasChild: "hasChild", dataSource: this._sheetData(), expanded: "expanded" }, nodeSelect: $.proxy(this._sheetSelect, xlObj), height: 140 });
                $("#" + xlObj._id + "_Doc").append($SheetDiv);
                $("#" + xlObj._id + "_Ribbon_cellAddress").wrap("<form id='" + xlObj._id + "_Form_Ribbon_cellAddress' onsubmit='return false'></form>");
                this._dialogValidate("_Ribbon_cellAddress");
                $("#" + xlObj._id + "_Ribbon_webAddress").wrap("<form id='" + xlObj._id + "_Form_Ribbon_webAddress' onsubmit='return false'></form>");
                this._dialogValidate("_Ribbon_webAddress");
            }
        },

        _sheetSelect: function (args) {
            var i, len, shtName = this._getSheetNames(), cellElem = $("#" + this._id + "_Ribbon_cellAddress"), btnElem = $("#" + this._id + "hlDialog_OkBtn").data("ejButton"), valElem = $("#" + this._id + "_Ribbon_cellText");
            if (args.model.selectedNode > 0) {
                cellElem.attr("disabled", false);
                btnElem.enable();
                cellElem.focus().setInputPos(1);
                for (i = 0, len = shtName.length; i < len ; i++) {
                    if (shtName[i].text === valElem.val().split("!")[0])
                        valElem.val(args.value + "!" + cellElem.val());
                }

            }
            else {
                cellElem.attr("disabled", true);
                btnElem.disable();
            }
        },


        _dlgHLOk: function () {
            var xlObj = this.XLObj;
            if ($("#" + xlObj._id + "_Form_Ribbon_cellAddress").valid() && $("#" + xlObj._id + "_Form_Ribbon_webAddress").valid()) {
                xlObj._setLink();
                this._dlgHLCancel();
            }
        },

        _dlgHLCancel: function () {
            $("#" + this.XLObj._id + "_HLDialog").ejDialog("close");
        },

        _renderFRDialog: function () {
            var $findDiv, $label, $find, $replace, $chkinput, $dd, $divleft, $divright, $maindiv, $settingsDiv, xlObj = this.XLObj, $table, $tr, $td;
            if ($("#" + xlObj._id + "_Ribbon_FindDiv").length < 1) {
                $findDiv = ej.buildTag("div.e-dlgctndiv", "", { height: 63 }, { id: xlObj._id + "_Ribbon_FindDiv" });
                $("#" + xlObj._id + "_Find").append($findDiv);
                $table = ej.buildTag("table");
                $tr = ej.buildTag("tr"), $td = ej.buildTag("td", "", { width: "26%" });
                $table.append($tr.append($td));
                $label = ej.buildTag("label", xlObj._getLocStr("FindLabel"));
                $td.append($label);
                $td = ej.buildTag("td");
                $find = ej.buildTag("input.ejinputtext", "", "", { id: xlObj._id + "_Ribbon_textFind", type: "text" });
                $td.append($find);
                $tr.append($td);
                $tr = ej.buildTag("tr.e-dlgtd-fields"), $td = ej.buildTag("td");
                $table.append($tr.append($td));
                $label = ej.buildTag("label", xlObj._getLocStr("ReplaceLabel"), "", { id: xlObj._id + "_Ribbon_lbReplace" });
                $td.append($label);
                $td = ej.buildTag("td");
                $replace = ej.buildTag("input.ejinputtext", "", "", { id: xlObj._id + "_Ribbon_textReplace", type: "text" });
                $td.append($replace);
                $tr.append($td);
                $findDiv.append($table);
                $("#" + xlObj._id + "_Ribbon_lbReplace").hide();
                $("#" + xlObj._id + "_Ribbon_textReplace").hide();

                $settingsDiv = ej.buildTag("div.e-dlgctndiv", "", {}, { id: xlObj._id + "_Ribbon_SettingsDiv" });
                $table = ej.buildTag("table");
                $tr = ej.buildTag("tr"), $td = ej.buildTag("td", "", { width: "50%" });
                $table.append($tr.append($td));
                $chkinput = ej.buildTag("input#" + xlObj._id + "_Ribbon_Case");
                $label = ej.buildTag("label", xlObj._getLocStr("MatchCase"), {}, { "for": xlObj._id + "_Ribbon_Case" });
                $td.append($chkinput);
                $td.append($label);
                $chkinput.ejCheckBox();
                $td = ej.buildTag("td");
                $chkinput = ej.buildTag("input#" + xlObj._id + "_Ribbon_AllContent");
                $label = ej.buildTag("label", xlObj._getLocStr("MatchAll"), {}, { "for": xlObj._id + "_Ribbon_AllContent" });
                $td.append($chkinput);
                $chkinput.ejCheckBox();
                $td.append($label);
                $tr.append($td);

                $tr = ej.buildTag("tr.e-dlgtd-fields"), $td = ej.buildTag("td");
                $table.append($tr.append($td));
                $label = ej.buildTag("label").text(xlObj._getLocStr("Within"));
                $td.append($label);
                $td = ej.buildTag("td");
                $label = ej.buildTag("label").text(xlObj._getLocStr("Search"));
                $td.append($label);
                $tr.append($td);

                $tr = ej.buildTag("tr.e-dlgtd-fields"), $td = ej.buildTag("td");
                $table.append($tr.append($td));
                $dd = ej.buildTag("input", "", "", { id: xlObj._id + "_Ribbon_ddWithin", type: "text" });
                $td.append($dd);
                $dd.ejDropDownList(xlObj._generateEJDD(this._within, { id: "Types", text: "text", value: "value" }, 0, 125, 25));
                $td = ej.buildTag("td");
                $dd = ej.buildTag("input", "", "", { id: xlObj._id + "_Ribbon_ddSearch", type: "text" });
                $td.append($dd);
                $dd.ejDropDownList(xlObj._generateEJDD(this._sType, { id: "Types", text: "text", value: "value" }, 0, 125, 25));
                $tr.append($td);

                $tr = ej.buildTag("tr.e-dlgtd-fields"), $td = ej.buildTag("td");
                $table.append($tr.append($td));
                $label = ej.buildTag("label", xlObj._getLocStr("Lookin"));
                $td.append($label);
                $td = ej.buildTag("td");
                $label = ej.buildTag("label", xlObj._getLocStr("Replace") + " " + xlObj._getLocStr("Direction"));
                $tr.append($td.append($label));

                $tr = ej.buildTag("tr.e-dlgtd-fields"), $td = ej.buildTag("td");
                $table.append($tr.append($td));
                $dd = ej.buildTag("input", "", "", { id: xlObj._id + "_Ribbon_ddLookin", type: "text" });
                $td.append($dd);
                $dd.ejDropDownList(xlObj._generateEJDD(this._vType, { id: "Types", text: "text", value: "value" }, 0, 125, 25, "Type"));
                $td = ej.buildTag("td");
                $dd = ej.buildTag("input", "", "", { id: xlObj._id + "_Ribbon_ddDirection", type: "text" });
                $tr.append($td.append($dd));
                $dd.ejDropDownList(xlObj._generateEJDD(this._direction, { id: "Types", text: "text", value: "value" }, 0, 125, 25));

                $("#" + xlObj._id + "_Settings").append($settingsDiv.append($table));
                $("#" + xlObj._id + "_Ribbon_FindRep").css("width", 174);
            }
        },

        _frTypeChange: function (name, args) {
            if (name === "Type" && (args.value === "comment")) {
                $("#" + this._id + "_Ribbon_textReplace").attr("disabled", true);
                $("#" + this._id + "FR_ReplaceBtn").ejButton("disable");
                $("#" + this._id + "FR_ReplaceAllBtn").ejButton("disable");
            }
            else {
                $("#" + this._id + "_Ribbon_textReplace").removeAttr("disabled");
                $("#" + this._id + "FR_ReplaceBtn").ejButton("enable");
                $("#" + this._id + "FR_ReplaceAllBtn").ejButton("enable");
            }
        },

        _hlTabChange: function (args) {
            var xlObj = this.XLObj, treeObj = $("#" + xlObj._id + "_Ribbon_sheentName").data("ejTreeView"), sheetIdx = xlObj.getActiveSheetIndex(), docTab = $("#" + xlObj._id + "_Doc"), webTab = $("#" + xlObj._id + "_Web");
            var webAddrElem = $("#" + xlObj._id + "_Ribbon_webAddress"), webElem = $("#" + xlObj._id + "_Ribbon_findText"), cellAddrElem = $("#" + xlObj._id + "_Ribbon_cellAddress"), cellElem = $("#" + xlObj._id + "_Ribbon_cellText");
            switch (args.activeIndex) {
                case 0:
                    webAddrElem.focus().setInputPos(webAddrElem.val().length);
                    webElem.val(cellElem.val());
                    docTab.hide();
                    webTab.show();
                    $("#" + xlObj._id + "hlDialog_OkBtn").ejButton("enable");
                    break;
                case 1:
                    !cellAddrElem.val().length && cellAddrElem.val("A1");
                    docTab.show();
                    webTab.hide();
                    treeObj.selectNode(sheetIdx + 1);
                    webElem.val().length ? cellElem.val(webElem.val()) : cellElem.val(treeObj.getText(sheetIdx + 1) + "!A1");
                    cellAddrElem.focus().setInputPos(cellAddrElem.val().length);
                    break;
            }
        },

        _findTabChange: function (args) {
            var $findDiv, xlObj = this.XLObj;
            switch (args.activeIndex) {
                case 0:
                    $findDiv = $("#" + xlObj._id + "_Ribbon_FindDiv");
                    $("#" + xlObj._id + "_Ribbon_lbReplace").hide();
                    $("#" + xlObj._id + "_Ribbon_textReplace").hide();
                    $("#" + xlObj._id + "FR_ReplaceBtn").hide();
                    $("#" + xlObj._id + "FR_ReplaceAllBtn").hide();
                    $("#" + xlObj._id + "_Find").append($findDiv);
                    $("#" + xlObj._id + "FR_PrevBtn").show();
                    $("#" + xlObj._id + "FR_NextBtn").show();
                    $("#" + xlObj._id + "_Ribbon_textFind").focus().setInputPos($("#" + xlObj._id + "_Ribbon_textFind").val().length);
                    break;
                case 1:
                    $findDiv = $("#" + xlObj._id + "_Ribbon_FindDiv");
                    $("#" + xlObj._id + "_Ribbon_lbReplace").show();
                    $("#" + xlObj._id + "_Ribbon_textReplace").show();
                    $("#" + xlObj._id + "FR_ReplaceBtn").show();
                    $("#" + xlObj._id + "FR_ReplaceAllBtn").show();
                    $("#" + xlObj._id + "FR_PrevBtn").show();
                    $("#" + xlObj._id + "FR_NextBtn").show();
                    $("#" + xlObj._id + "_Replace").append($findDiv);
                    $("#" + xlObj._id + "_Ribbon_textFind").focus().setInputPos($("#" + xlObj._id + "_Ribbon_textFind").val().length);
                    break;
                case 2:
                    $("#" + xlObj._id + "FR_PrevBtn").hide();
                    $("#" + xlObj._id + "FR_NextBtn").hide();
                    $("#" + xlObj._id + "FR_ReplaceBtn").hide();
                    $("#" + xlObj._id + "FR_ReplaceAllBtn").hide();
                    $("#" + xlObj._id + "_Ribbon_ddWithin").focus();
                    break;
            }
        },

        _gotoTabChange: function (args) {
            var xlObj = this.XLObj;
            if (args.activeIndex == 0)
                $("#" + xlObj._id + "_textRef").focus().setInputPos($("#" + xlObj._id + "_textRef").val().length);
        },

        _btnFROnClick: function (id, e) {
            var xlObj = this.XLObj;
            xlObj.element.ejWaitingPopup("show");
            var sheetIdx = xlObj.getActiveSheetIndex(), rowIndex = xlObj.getSheet(sheetIdx)._startCell.rowIndex, colIndex = xlObj.getSheet(sheetIdx)._startCell.colIndex, mCase = $("#" + xlObj._id + "_Ribbon_Case").data("ejCheckBox").isChecked(), mAll = $("#" + xlObj._id + "_Ribbon_AllContent").data("ejCheckBox").isChecked();
            var within = this._within[$("#" + xlObj._id + "_Ribbon_ddWithin").data("ejDropDownList").selectedIndexValue].value, search = this._sType[$("#" + xlObj._id + "_Ribbon_ddSearch").data("ejDropDownList").selectedIndexValue].value, lookin = this._vType[$("#" + xlObj._id + "_Ribbon_ddLookin").data("ejDropDownList").selectedIndexValue].value, direction = this._direction[$("#" + xlObj._id + "_Ribbon_ddDirection").data("ejDropDownList").selectedIndexValue].value, isFinded;
            switch (id) {
                case "FR_NextBtn":
                case "FR_PrevBtn":
                    xlObj.XLFndRp._findCount = 0;
                    isFinded = id === "FR_NextBtn" ? xlObj.XLFndRp._findNext($("#" + xlObj._id + "_Ribbon_textFind").val(), sheetIdx, mCase, mAll, lookin, within, search) : xlObj.XLFndRp._findPrev($("#" + xlObj._id + "_Ribbon_textFind").val(), sheetIdx, mCase, mAll, lookin, within, search);
                    if (!isFinded) {
                        xlObj.XLFndRp._updateSelection(rowIndex, colIndex, sheetIdx);
                        xlObj.element.ejWaitingPopup("hide");
                        xlObj._showAlertDlg("Alert", "NotFound");
                    }
                    break;
                case "FR_ReplaceBtn":
                    xlObj.XLFndRp._replaceData($("#" + xlObj._id + "_Ribbon_textFind").val(), $("#" + xlObj._id + "_Ribbon_textReplace").val(), sheetIdx, mCase, mAll, lookin, direction, within, search);
                    break;
                case "FR_ReplaceAllBtn":
                    xlObj.XLFndRp._replaceAllData($("#" + xlObj._id + "_Ribbon_textFind").val(), $("#" + xlObj._id + "_Ribbon_textReplace").val(), mCase, mAll, lookin, within);
                    break;
                case "FR_CloseBtn":
                    $("#" + xlObj._id + "_FRDialog").ejDialog("close");
                    xlObj.setSheetFocus();
                    break;
            }
            xlObj.element.ejWaitingPopup("hide");
        },

        _renderGoToDialog: function () {
            var $gotoDiv, $gotoSpDiv, $label, $list, $input, xlObj = this.XLObj, $subdiv;
            if ($("#" + xlObj._id + "_GoToDiv").length < 1) {
                $gotoDiv = ej.buildTag("div.e-dlgctndiv", "", {}, { id: xlObj._id + "_GoToDiv" });
                $("#" + xlObj._id + "_GoTo").append($gotoDiv);
                $subdiv = ej.buildTag("div.e-dlg-fields");
                $label = ej.buildTag("label.e-dlg-fields", xlObj._getLocStr("GoToName"));
                $list = ej.buildTag("ul#" + xlObj._id + "_GotoAddr");
                $gotoDiv.append($subdiv.append($label, $list));
                $("#" + xlObj._id + "_GotoAddr").ejListBox({
                    dataSource: this._addrList,
                    height: "190px",
                    width: "247px",
                    selected: $.proxy(this._gotoValChange, this),
                    fields: { id: "Types", text: "text", value: "value" }
                });

                $subdiv = ej.buildTag("div.e-dlg-fields");
                $label = ej.buildTag("label.e-dlg-fields", xlObj._getLocStr("Reference"));
                $input = ej.buildTag("input.ejinputtext", "", "", { id: xlObj._id + "_textRef", type: "text" });
                $gotoDiv.append($subdiv.append($label, $input));

                $gotoSpDiv = ej.buildTag("div#" + xlObj._id + "_GoToSpDiv.e-dlgctndiv");
                $subdiv = ej.buildTag("div.e-dlg-fields e-dlgctndiv");
                $("#" + xlObj._id + "_GoToSp").append($gotoSpDiv.append($subdiv));
                $label = ej.buildTag("label.e-dlg-fields").text(xlObj._getLocStr("Select")).css('display', "table");
                $subdiv.append($label);
                this._renderGotoSp($subdiv, this._gotoRBtnSet1, this._gotoRBtnSet2, this._gotoChkBox);
                $('#' + xlObj._id + '_GoToDiv').wrapInner('<form id="' + xlObj._id + '_Form_GoToDiv" onsubmit="return false"></form>');
                this._dialogValidate('_GoToDiv');

            }
        },

        _renderGotoSp: function ($content, rBtnIdSet1, rBtnIdSet2, chkBoxIDs) {
            var input, lbl, table, tr, td, i, xlObj = this.XLObj, tabId = xlObj._id + "_gotoSpDiv", $chkinput;
            table = ej.buildTag("table");
            for (i = 0; i < rBtnIdSet1.length; i++) {
                tr = ej.buildTag("tr", "", { height: 35 });
                td = ej.buildTag("td");
                input = ej.buildTag("input#" + tabId + "_" + rBtnIdSet1[i], "", {}, { type: "radio", name: "gotospecial" });
                td.append(input);
                tr.append(td);
                input.ejRadioButton({ size: "medium", change: $.proxy(this._rBtnOnClick, this, rBtnIdSet1[i]) });
                td = ej.buildTag("td", "", { width: 100 });
                lbl = ej.buildTag("label", xlObj._getLocStr(rBtnIdSet1[i]), {}, { "for": tabId + "_" + rBtnIdSet1[i] });
                td.append(lbl);
                tr.append(td);
                if (!xlObj.isUndefined(rBtnIdSet2[i])) {
                    td = ej.buildTag("td");
                    input = ej.buildTag("input#" + tabId + "_" + rBtnIdSet2[i], "", {}, { type: "radio", name: "gotospecial" });
                    td.append(input);
                    tr.append(td);
                    input.ejRadioButton({ size: "medium", change: $.proxy(this._rBtnOnClick, this, rBtnIdSet2[i]) });
                    td = ej.buildTag("td");
                    lbl = ej.buildTag("label", xlObj._getLocStr(rBtnIdSet2[i]), {}, { "for": tabId + "_" + rBtnIdSet2[i] });
                    td.append(lbl);
                    tr.append(td);
                }
                table.append(tr);
            }
            for (i = 0; i < chkBoxIDs.length; i = i + 2) {
                tr = ej.buildTag("tr");
                td = ej.buildTag("td");
                $chkinput = ej.buildTag("input#" + xlObj._id + "_" + chkBoxIDs[i]);
                td.append($chkinput);
                tr.append(td);
                td = ej.buildTag("td");
                lbl = ej.buildTag("label", xlObj._getLocStr(chkBoxIDs[i]), {}, { "for": xlObj._id + "_" + chkBoxIDs[i] });
                td.append(lbl);
                tr.append(td);
                $chkinput.ejCheckBox({ checked: true, enabled: false });
                td = ej.buildTag("td");
                $chkinput = ej.buildTag("input#" + xlObj._id + "_" + chkBoxIDs[i + 1]);
                td.append($chkinput);
                tr.append(td);
                td = ej.buildTag("td");
                lbl = ej.buildTag("label", xlObj._getLocStr(chkBoxIDs[i + 1]), {}, { "for": xlObj._id + "_" + chkBoxIDs[i + 1] });
                td.append(lbl);
                tr.append(td);
                $chkinput.ejCheckBox({ checked: true, enabled: false });
                table.append(tr);

            }
            $content.append(table);
            $("#" + tabId + "_" + "Blanks").ejRadioButton("option", { checked: true });
        },

        _rBtnOnClick: function (id, e) {
            var xlObj = this.XLObj, i;
            if (id === "Formulas" || id === "Constants") {
                for (i = 0; i < this._gotoChkBox.length; i++)
                    $("#" + xlObj._id + "_" + this._gotoChkBox[i]).data("ejCheckBox").option({ enabled: true });
            }
            else {
                for (i = 0; i < this._gotoChkBox.length; i++)
                    $("#" + xlObj._id + "_" + this._gotoChkBox[i]).data("ejCheckBox").option({ enabled: false });
            }
        },

        _gotoValChange: function (args) {
            $("#" + this.XLObj._id + "_textRef").val(args.model.value);
        },

        _gotoBtnClick: function (id, e) {
            var xlObj = this.XLObj;
            switch (id) {
                case "ok":
                    if ($('#' + xlObj._id + '_Form_GoToDiv').valid()) {
                        var tabObj = $("#" + xlObj._id + "_GoToDlgTab").data("ejTab");
                        if (!tabObj.model.selectedItemIndex) {
                            var isApply, lbObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), startCell, endCell, value = $.trim($("#" + xlObj._id + "_textRef").val().toString()), sIdx, splitStr, txt = xlObj._getDollarAlphaRange(sheet.selectedRange);
                            if (value.indexOf("!") > -1) {
                                splitStr = value.split("!");
                                sIdx = xlObj._getSheetIndexByName(splitStr[0]);
                                value = splitStr[1];
                            }
                            value.indexOf("$") > -1 && (value = value.split("$").join(""));
                            value.indexOf(":") < 0 && (value = value + ":" + value);
                            value = value.replace("=", "");
                            sIdx && xlObj.gotoPage(sIdx, false);
                            var range = xlObj.getRangeIndices(value), rangeCells = xlObj.getRange(range[0], range[1], range[2], range[3]);
                            startCell = { rowIndex: $(rangeCells[0]).parent("tr").index(), colIndex: $(rangeCells[0]).index() };
                            endCell = { rowIndex: $(rangeCells[rangeCells.length - 1]).parent("tr").index(), colIndex: $(rangeCells[rangeCells.length - 1]).index() };
                            xlObj.setActiveCell(startCell.rowIndex, startCell.colIndex);
                            xlObj.XLSelection.selectRange(startCell, endCell);
                            xlObj.XLScroll._scrollSelectedPosition(xlObj.getActiveSheetIndex(), startCell);
                            xlObj.XLDragFill.positionAutoFillElement();
                            sheet._startCell = startCell;
                            sheet._endCell = endCell;
                            xlObj.setSheetFocus();
                            isApply = $.grep(this._addrList, function (e) {
                                return e.value === txt;
                            });
                            if (isApply.length < 1) {
                                this._addrList.push({ value: txt, text: txt });
                                lbObj = $("#" + xlObj._id + "_GotoAddr").data("ejListBox");
                                this._addrList = JSON.parse(JSON.stringify(this._addrList));
                                lbObj.model.dataSource = this._addrList;
                                lbObj.option("fields", { dataSource: this._addrList });
                            }
                            $("#" + xlObj._id + "_GoDialog").ejDialog("close");
                        }
                        else {
                            xlObj.XLFndRp._applyGoToRule($("#" + xlObj._id + "_GoToSpDiv").find(".e-rad-active").parents("div")[0].id.split("ej" + xlObj._id + "_gotoSpDiv_")[1].toLowerCase(), $("#" + xlObj._id + "_" + this._gotoChkBox[0]).data("ejCheckBox").isChecked(), $("#" + xlObj._id + "_" + this._gotoChkBox[1]).data("ejCheckBox").isChecked(), $("#" + xlObj._id + "_" + this._gotoChkBox[2]).data("ejCheckBox").isChecked(), $("#" + xlObj._id + "_" + this._gotoChkBox[3]).data("ejCheckBox").isChecked());
                            $("#" + xlObj._id + "_GoDialog").ejDialog("close");
                        }
                    }
                    break;
                case "cancel":
                    if ($.validator)
                        $('#' + xlObj._id + '_Form_GoToDiv').validate().resetForm();
                    $("#" + xlObj._id + "_GoDialog").ejDialog("close");
                    break;
            }
            xlObj.setSheetFocus();
        },

        _borderPicture: function () {
            var xlObj = this.XLObj;
            return "<ul id=" + xlObj._id + "_Ribbon_PictureBorder><li id='" + xlObj._id + "picturecolor'><a>" + xlObj._getLocStr("ThemeColor") + "</a><ul><li><input id='" + xlObj._id + "_Ribbon_PictureColor'></input></li></ul></li><li id='nooutline' class='picturecolor'><a>" + xlObj._getLocStr("NoOutline") + "</a></li><li id='weight' class='picturecolor'><a>" + xlObj._getLocStr("Weight") + "</a><ul><li id='1px'><a>1px</a></li><li id='2px'><a>2px</a></li><li id='3px'><a>3px</a></li></ul></li><li id='dashes' class='picturecolor'><a>" + xlObj._getLocStr("Dashes") + "</a><ul><li id='solid'><a><div style='top:9px;border:1px solid black;width:40px'></div></a></li><li id='dotted'><a><div style='top:9px;border:1px dotted black;width:40px'></div></a></li><li id='dashed'><a><div style='top:9px;border:1px dashed black;width:40px'></div></a></li></ul></li></ul>";
        },

        _chartElementTag: function () {
            var xlObj = this.XLObj, element = '<ul id=' + xlObj._id + '_Ribbon_CElement class="e-spreadsheet">';
            element += '<li class="axes" id="Axes"><a><span class="e-ss-chartimg e-icon e-ss-axes"></span>' + xlObj._getLocStr("Axes") + '</a><ul><li class="phaxis e-ss-cmenuitem" id="PHAxis"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-phaxis"></span>' + xlObj._getLocStr("PHAxis") + '</a></li><li class="pvaxis e-ss-cmenuitem" id="PVAxis"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-pvaxis"></span>' + xlObj._getLocStr("PVAxis") + '</a></li></ul></li>';
            element += '<li class="axistitle" id="AxisTitle"><a><span class="e-ss-chartimg e-icon e-ss-axistitle"></span>' + xlObj._getLocStr("AxisTitle") + '</a><ul><li class="phaxistitle e-ss-cmenuitem" id="PHAxisTitle"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-phaxistitle"></span>' + xlObj._getLocStr("PHAxis") + '</a></li><li class="pvaxistitle e-ss-cmenuitem" id="PVAxisTitle"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-pvaxistitle"></span>' + xlObj._getLocStr("PVAxis") + '</a></li></ul></li>';
            element += '<li class="charttitle" id="ChartTitle"><a><span class="e-ss-chartimg e-icon e-ss-charttitle"></span>' + xlObj._getLocStr("ChartTitle") + '</a><ul><li class="ctnone e-ss-cmenuitem" id="CTNone"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-ctnone"></span>' + xlObj._getLocStr("CTNone") + '</a></li><li class="ctcenter e-ss-cmenuitem" id="CTCenter"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-ctcenter"></span>' + xlObj._getLocStr("CTCenter") + '</a></li><li class="ctfar e-ss-cmenuitem" id="CTFar"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-ctfar"></span>' + xlObj._getLocStr("CTFar") + '</a></li><li class="ctnear e-ss-cmenuitem" id="CTNear"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-ctnear"></span>' + xlObj._getLocStr("CTNear") + '</a></li></ul></li>';
            element += '<li class="datalabels" id="DataLabels"><a><span class="e-ss-chartimg e-icon e-ss-datalabels"></span>' + xlObj._getLocStr("DataLabels") + '</a><ul><li class="dlnone e-ss-cmenuitem" id="DLNone"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-dlnone"></span>' + xlObj._getLocStr("DLNone") + '</a></li><li class="dlcenter e-ss-cmenuitem" id="DLCenter"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-dlcenter"></span>' + xlObj._getLocStr("DLCenter") + '</a></li><li class="dliend e-ss-cmenuitem" id="DLIEnd"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-dliend"></span>' + xlObj._getLocStr("DLIEnd") + '</a></li><li class="dlibase e-ss-cmenuitem" id="DLIBase"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-dlibase"></span>' + xlObj._getLocStr("DLIBase") + '</a></li><li class="dloend e-ss-cmenuitem" id="DLOEnd"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-dloend"></span>' + xlObj._getLocStr("DLOEnd") + '</a></li></ul></li>';
            element += '<li class="gridline" id="Gridline"><a><span class="e-ss-chartimg e-icon e-ss-gridline"></span>' + xlObj._getLocStr("Gridline") + '</a><ul><li class="pmajorh e-ss-cmenuitem" id="PMajorH"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-pmajorh"></span>' + xlObj._getLocStr("PMajorH") + '</a></li><li class="pmajorv e-ss-cmenuitem" id="PMajorV"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-pmajorv"></span>' + xlObj._getLocStr("PMajorV") + '</a></li><li class="pminorh e-ss-cmenuitem" id="PMinorH"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon  e-ss-pminorh"></span>' + xlObj._getLocStr("PMinorH") + '</a></li><li class="pminorv e-ss-cmenuitem" id="PMinorV"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-pminorv"></span>' + xlObj._getLocStr("PMinorV") + '</a></li></ul></li>';
            element += '<li class="legend" id="Legend"><a><span class="e-ss-chartimg e-icon e-ss-legend"></span>' + xlObj._getLocStr("Legend") + '</a><ul><li class="lnone e-ss-cmenuitem" id="LNone"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-lnone"></span>' + xlObj._getLocStr("LNone") + '</a></li><li class="lright e-ss-cmenuitem" id="LRight"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-lright"></span>' + xlObj._getLocStr("LRight") + '</a></li><li class="lleft e-ss-cmenuitem" id="LLeft"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-lleft"></span>' + xlObj._getLocStr("LLeft") + '</a></li><li class="lbottom e-ss-cmenuitem" id="LBottom"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-lbottom"></span>' + xlObj._getLocStr("LBottom") + '</a></li><li class="ltop e-ss-cmenuitem" id="LTop"><a class="e-ss-chartlayout"><span class="e-ss-chartimg e-icon e-ss-ltop"></span>' + xlObj._getLocStr("LTop") + '</a></li></ul></li>';
            return element;
        },

        _cpClickHandler: function (name, args) {
            var xlObj = this.XLObj;
            xlObj._isApply = true;
            if (name === "Ribbon_background-color")
                xlObj.XLFormat.format({ "style": { "background-color": $("#" + xlObj._id + "_Ribbon_background-color").ejColorPicker('option', 'value') } });
            else if (name === "Ribbon_color")
                xlObj.XLFormat.format({ "style": { "color": $("#" + xlObj._id + "_Ribbon_color").ejColorPicker('option', 'value') } });
        },

        _ribbonClickHandler: function (args) {
            var i, cfObj, hexcode, gotoTabObj, cellRange, startCell, endCell, info, details, elem, element, cname, selectedIndex, activeCell, fsObj, tid, tmgr, range, obj = {}, xlObj = $("#" + this._id.split("_Ribbon")[0]).data("ejSpreadsheet"), sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), temp = {}, style = {}, splitId = this._id.split("_"), sstyles = ej.Spreadsheet.SupportedStyles, size, arg, sheetCont, sheetPanel, dataVal;
            if (xlObj._shapeChange || xlObj.XLRibbon._isSetModel)
                return;
            sheetCont = xlObj._getContent(sheetIdx).find("div:first-child"), sheetPanel = xlObj.getMainPanel();
            cfObj = $("#" + xlObj._id + "_Ribbon_cellformat").data("ejDropDownList");
            fsObj = $("#" + xlObj._id + "_Ribbon_fontsize").data("ejDropDownList");
            activeCell = $(xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex));
            if (xlObj.model.allowSelection && ((xlObj.model.allowFormatAsTable && xlObj.model.allowCellFormatting))) {
                if ((xlObj.XLFormat._formatEnable || xlObj.XLSelection._isOutsideBordering || xlObj.XLSelection._isGridBordering)) {
                    if (xlObj._id + "_Ribbon_BorderColor" !== this._id && xlObj._id + "_Ribbon_bordercolor" !== args.ID && !(xlObj._borderStyles.indexOf(args.ID) > -1)) {
                        xlObj.XLSelection._isGridBordering = xlObj.XLSelection._isOutsideBordering = xlObj.XLFormat._formatEnable = false;
                        $("#" + xlObj._id + "_Ribbon_formatpainter").ejToggleButton("option", { toggleState: false });
                        if (args.type === "itemSelected" && !xlObj.isUndefined(args.model.prefixIcon) && args.model.prefixIcon.split("e-ss-")[1] === args.ID)
                            xlObj.XLSelection._isGridBordering = xlObj.XLSelection._isOutsideBordering = true;
                        if (args.type === "click" && !xlObj.isUndefined(args.model.prefixIcon) && (args.model.prefixIcon.indexOf("e-ss-drawborder") > -1 || args.model.prefixIcon.indexOf("e-ss-drawbordergrid") > -1))
                            xlObj.XLSelection._isGridBordering = xlObj.XLSelection._isOutsideBordering = true;
                        sheetCont.addClass("e-ss-cursor");
                        sheetCont.removeClass("e-ss-drwbrdrcursor e-ss-drwbrdrgridcursor e-ss-fpcursor");
                    }
                }
            }
            arg = { Id: this._id, isChecked: args.isChecked, status: args.status, prop: args.model, model: xlObj.model };
            if (xlObj._trigger("ribbonClick", arg)) {
                xlObj.XLRibbon._updateRibbonIcons();
                return;
            }
            if (!ej.isNullOrUndefined(xlObj._getAutoFillOptElem()) && xlObj.model.autoFillSettings.showFillOptions) {
                xlObj._getAutoFillOptElem().addClass("e-hide");
                $("#" + xlObj._id + "_ctxtmenu").hide();
            }
            switch (this._id) {
                case xlObj._id + "_Ribbon_cut":
                    xlObj.XLClipboard.cut();
                    break;
                case xlObj._id + "_Ribbon_copy":
                    xlObj.XLClipboard.copy();
                    break;
                case xlObj._id + "_Ribbon_pastebtn":
                case xlObj._id + "_Ribbon_pastesbtn":
                    xlObj.XLClipboard._isSpecial = !(args.ID === "PasteValues");
                    xlObj.XLClipboard.paste();
                    break;
                case xlObj._id + "_Ribbon_undo":
                    xlObj.XLClipboard._triggerKeyDown(90, true);
                    break;
                case xlObj._id + "_Ribbon_redo":
                    xlObj.XLClipboard._triggerKeyDown(89, true);
                    break;
                case xlObj._id + "_Ribbon_wraptext":
                    xlObj._textFormatting("wrapText");
                    break;
                case xlObj._id + "_Ribbon_exportXL":
                    xlObj.XLExport["export"](ej.Spreadsheet.exportType.Excel);
                    break;
                case xlObj._id + "_Ribbon_exportCsv":
                    xlObj.XLExport["export"](ej.Spreadsheet.exportType.Csv);
                    break;
                case xlObj._id + "_Ribbon_print":
                    xlObj.XLPrint.printSheet(sheetIdx);
                    break;
                case xlObj._id + "_Ribbon_printselected":
                    xlObj.XLPrint.printSelection(sheetIdx);
                    break;
                case xlObj._id + "_Ribbon_new":
                    xlObj._cellFormatHandler(this._id.replace(xlObj._id + "_Ribbon_", ""), args.selectedText);
                    break;
                case xlObj._id + "_Ribbon_bold":
                    style[sstyles.FontWeight] = args.model.toggleState ? "bold" : "normal";
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_italic":
                    style[sstyles.FontStyle] = args.model.toggleState ? "italic" : "normal";
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_underline":
                case xlObj._id + "_Ribbon_line-through":
                    var txtDStyle = xlObj.XLFormat.getFormatFromHashCode(xlObj.XLFormat.getFormatClass(activeCell[0].className))["text-decoration"] || "";

                    if (args.model.toggleState) {
                        txtDStyle = txtDStyle.replace("none", "");
                        style[sstyles.TextDecoration] = txtDStyle ? txtDStyle.indexOf("underline") === -1 ? txtDStyle + " " + splitId[splitId.length - 1] : splitId[splitId.length - 1] + " " + txtDStyle : splitId[splitId.length - 1];
                    }
                    else {
                        txtDStyle = $.trim(txtDStyle.replace(splitId[splitId.length - 1], ""));
                        style[sstyles.TextDecoration] = txtDStyle.length ? txtDStyle : "none";
                    }
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_fontsize":
                    style[sstyles.FontSize] = args.value;
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_increasefontsize":
                case xlObj._id + "_Ribbon_decreasefontsize":
                    if (this._id == xlObj._id + "_Ribbon_increasefontsize") {
                        selectedIndex = !ej.isNullOrUndefined(fsObj.selectedIndexValue) ? fsObj.selectedIndexValue + 1 : 6; //6-default increase fonsize index
                        !ej.isNullOrUndefined(xlObj.XLRibbon._fontSize[selectedIndex]) && fsObj.option({ selectedItemIndex: selectedIndex });
                    }
                    else if (this._id == xlObj._id + "_Ribbon_decreasefontsize") {
                        selectedIndex = !ej.isNullOrUndefined(fsObj.selectedIndexValue) ? fsObj.selectedIndexValue - 1 : 4; //6-default decrease fonsize index
                        !ej.isNullOrUndefined(xlObj.XLRibbon._fontSize[selectedIndex]) && fsObj.option({ selectedItemIndex: selectedIndex });
                    }
                    style[sstyles.FontSize] = fsObj.value();
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_left":
                case xlObj._id + "_Ribbon_right":
                case xlObj._id + "_Ribbon_center":
                    element = $(this.element.parents("div").eq(1)).find(".e-text-align");
                    if (!ej.isNullOrUndefined(element[0])) {
                        $("#" + element[0].id).ejToggleButton("option", "toggleState", false);
                        element.removeClass("e-text-align");
                    }
                    this.element.addClass("e-text-align");
                    style[sstyles.TextAlign] = args.model.toggleState ? splitId[splitId.length - 1] : "left";
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_top":
                case xlObj._id + "_Ribbon_middle":
                case xlObj._id + "_Ribbon_bottom":
                    element = $(this.element.parents("div").eq(1)).find(".e-vertical-align");
                    if (!ej.isNullOrUndefined(element[0])) {
                        $("#" + element[0].id).ejToggleButton("option", "toggleState", false);
                        element.removeClass("e-vertical-align");
                    }
                    this.element.addClass("e-vertical-align");
                    style[sstyles.VerticalAlign] = args.model.toggleState ? splitId[splitId.length - 1] : "bottom";
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_font-family":
                    style[sstyles.FontFamily] = args.text;
                    temp["style"] = style;
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_Vheading":
                    xlObj.showHeaders(args.isChecked);
                    break;
                case xlObj._id + "_Ribbon_Vgridlines":
                    xlObj.showGridlines(args.isChecked);
                    break;
                case xlObj._id + "_Ribbon_sizebtn":
                case xlObj._id + "_Ribbon_sizesbtn":
                    var value = !ej.isNullOrUndefined(args.ID) ? args.ID : args.model.prefixIcon.substring(12);
                    xlObj.XLPrint._printSetting(value);
                    break;
                case xlObj._id + "_Ribbon_cellformat":
                    temp["type"] = args.text.toLowerCase();
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_increment":
                case xlObj._id + "_Ribbon_decrement":
                    if (activeCell.data("type") != "number" && $.isNumeric(xlObj.XLEdit.getPropertyValueByElem(activeCell))) {
                        xlObj.XLRibbon._isSetModel = true;
                        cfObj.option({ selectedItemIndex: 1 }); //number type
                        xlObj.XLRibbon._isSetModel = false;
                    }
                    xlObj.XLFormat.updateDecimalPlaces(splitId[splitId.length - 1]);
                    break;
                case xlObj._id + "_Ribbon_accounting":
                case xlObj._id + "_Ribbon_percentage":
                    this._id === xlObj._id + "_Ribbon_accounting" ? cfObj.option({ selectedItemIndex: 3 }) : cfObj.option({ selectedItemIndex: 4 });
                    temp["type"] = cfObj.selectedTextValue.toLowerCase();
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_usegrouping":
                    cfObj.option({ selectedItemIndex: 1 });
                    temp["type"] = cfObj.selectedTextValue.toLowerCase();
                    xlObj.XLFormat.format(temp);
                    break;
                case xlObj._id + "_Ribbon_sortAtoZ":
                    xlObj.XLSort._sortHandler("SortAtoZ");
                    break;
                case xlObj._id + "_Ribbon_sortZtoA":
                    xlObj.XLSort._sortHandler("SortZtoA");
                    break;
                case xlObj._id + "_Ribbon_filter":
                    xlObj.XLFilter.filter();
                    break;
                case xlObj._id + "_Ribbon_clearfilter":
                    cname = xlObj.getCell(xlObj.getActiveCell().rowIndex, xlObj.getActiveCell().colIndex)[0].className;
                    if (cname.indexOf("e-table") < 0)
                        xlObj.XLFilter.clearFilter();
                    else {
                        tid = xlObj._getTableID(cname);
                        xlObj.XLFilter._clearFilterTable(xlObj.getActiveSheetIndex(), parseInt(tid), true);
                    }
                    break;
                case xlObj._id + "_Ribbon_BorderColor":
                    obj = $('#' + xlObj._id + '_Ribbon_BorderColor').data('ejColorPicker');
                    !ej.isNullOrUndefined(obj) && (obj.hide());
                    !ej.isNullOrUndefined(args.value) && (xlObj._borderColor = args.value);
                    break;
                case xlObj._id + "_Ribbon_border":
                    var alphRange;
                    if ((xlObj._borderStyles.indexOf(args.ID) < 0)) {
                        if (ej.isNullOrUndefined(args.ID))
                            args.ID = args.model.prefixIcon.substring(12);
                        else
                            args.ID.indexOf("bordercolor") < 0 && $(this.element).find(".e-icon ").removeClass().addClass("e-icon e-ss-" + args.ID) && this.option({ prefixIcon: "e-icon e-ss-" + args.ID });
                    }
                    else
                        xlObj.XLRibbon._updateBordeStyle(args);
                    hexcode = xlObj._borderColor || '#000000';
                    startCell = sheet._startCell;
                    endCell = sheet._endCell;
                    alphRange = xlObj._getAlphaRange(sheetIdx, startCell.rowIndex, startCell.colIndex, endCell.rowIndex, endCell.colIndex);
                    if (xlObj.XLSelection._isOutsideBordering || xlObj.XLSelection._isGridBordering)
                        !(xlObj._borderStyles.indexOf(args.ID) > -1) && (args.ID.indexOf("bordercolor") < 0) && (xlObj.XLSelection._isOutsideBordering = xlObj.XLSelection._isGridBordering = false);
                    else
                        xlObj.setBorder({ "ID": args.ID, "hexaCode": hexcode, "style": xlObj._borderStyle }, alphRange);
                    break;
                case xlObj._id + "_Ribbon_validationsbtn":
                case xlObj._id + "_Ribbon_validationbtn":
                    if (args.ID == "ClearVal")
                        xlObj.XLValidate.clearDV();
                    else if (args.ID == "HighlightVal")
                        xlObj.XLValidate.highlightInvalidData();
                    else if (args.ID == "ClearHLVal")
                        xlObj.XLValidate.clearHighlightedValData();
                    else {
                        $("#" + xlObj._id + "_ValDialog").ejDialog("open");
                        if ($.validator)
                            $("#" + xlObj._id + "_Form_ValDialog").validate().resetForm();
                        $("#" + xlObj._id + "_Ribbon_ddType_wrapper").focus();
                        var rules = xlObj.XLEdit.getPropertyValueByElem(xlObj.getActiveCellElem(sheetIdx), "rule");
                        if (rules) {
                            var rule = rules.customVal, splitStr = rule.split("_");
                            if (rule.indexOf("list") > -1) {
                                $("#" + xlObj._id + "_Ribbon_ddType").ejDropDownList('option', 'selectedItemIndex', 5);
                                $("#" + xlObj._id + "_Ribbon_btnMin").val(splitStr[0]);
                            }
                            else {
                                var index;
                                xlObj.XLRibbon._types.some(function (element, i) { if (splitStr[4] === element.value) { index = i; return true; } });
                                $("#" + xlObj._id + "_Ribbon_ddType").ejDropDownList('option', 'selectedItemIndex', index);
                                xlObj.XLRibbon._actions.some(function (element, i) { if (splitStr[0] === element.value) { index = i; return true; } });
                                $("#" + xlObj._id + "_Ribbon_ddAction").ejDropDownList('option', 'selectedItemIndex', index);
                                $("#" + xlObj._id + "_Ribbon_btnMin").val(splitStr[1].replace("^", ""));
                                $("#" + xlObj._id + "_Ribbon_btnMax").val(splitStr[2].replace("^", ""));
                            }
                            $("#" + xlObj._id + "_Ribbon_blank").ejCheckBox("option", "checked", !rules.required);
                            $("#" + xlObj._id + "_Ribbon_freetext").ejCheckBox("option", "checked", JSON.parse(splitStr[splitStr.length - 3]));
                        }
                        else {
                            $("#" + xlObj._id + "_Ribbon_ddType").ejDropDownList('option', 'selectedItemIndex', 1);
                            $("#" + xlObj._id + "_Ribbon_ddAction").ejDropDownList('option', 'selectedItemIndex', 6);
                            $("#" + xlObj._id + "_Ribbon_btnMin").val("");
                            $("#" + xlObj._id + "_Ribbon_btnMax").val("");
                            $("#" + xlObj._id + "_Ribbon_blank").ejCheckBox("option", "checked", true);
                            $("#" + xlObj._id + "_Ribbon_freetext").ejCheckBox("option", "checked", true);
                        }

                    }
                    break;
                case xlObj._id + "_Ribbon_cFormat":
                    if (!args.ID.length)
                        break;
                    xlObj.XLRibbon._currentCFormat = args.ID;
                    if (args.ID.startsWith("clear"))
                        xlObj.XLCFormat.clearCF(args.ID === "clearselected");
                    else {
                        if (!(args.ID.indexOf("_HLCellRules") > 0 || args.ID.indexOf("_ClearRules") > 0)) {
                            $("#" + xlObj._id + "_CFDialog").ejDialog("open");
                            xlObj.XLRibbon._renderCFDialog(args);
                        }
                    }
                    break;
                case xlObj._id + "_Ribbon_find":
                    var fRTabObj;
                    gotoTabObj = $("#" + xlObj._id + "_GoToDlgTab").data("ejTab"), fRTabObj = $("#" + xlObj._id + "_FRDialog_FPDlgTab").data("ejTab");
                    switch (args.ID) {
                        case "Find":
                            $("#" + xlObj._id + "_FRDialog").ejDialog("open");
                            fRTabObj.option({ selectedItemIndex: 0 });
                            $("#" + xlObj._id + "_Ribbon_textFind").focus().setInputPos($("#" + xlObj._id + "_Ribbon_textFind").val().length);
                            break;
                        case "Replace":
                            $("#" + xlObj._id + "_FRDialog").ejDialog("open");
                            fRTabObj.option({ selectedItemIndex: 1 });
                            $("#" + xlObj._id + "_Ribbon_textFind").focus().setInputPos($("#" + xlObj._id + "_Ribbon_textFind").val().length);
                            break;
                        case "GoTo":
                            $("#" + xlObj._id + "_GoDialog").ejDialog("open");
                            gotoTabObj.option({ selectedItemIndex: 0 });
                            $("#" + xlObj._id + "_textRef").focus().setInputPos($("#" + xlObj._id + "_Ribbon_textFind").val().length);
                            if ($.validator)
                                $('#' + xlObj._id + '_Form_GoToDiv').validate().resetForm();
                            break;
                        case "GoToSpecial":
                            $("#" + xlObj._id + "_GoDialog").ejDialog("open");
                            gotoTabObj.option({ selectedItemIndex: 1 });
                            $("#" + xlObj._id + "_Blanks").focus();
                            break;
                        default:
                            xlObj.XLFndRp._applyGoToRule(args.ID.toLowerCase(), true, true, true, true);
                            break;
                    }
                    break;
                case xlObj._id + "_Ribbon_formatpainter":
                    if (!args.model.toggleState)
                        return false;
                    xlObj.XLFormat._formatEnable = true;
                    sheetCont.removeClass("e-ss-cursor");
                    sheetCont.addClass("e-ss-fpcursor");
                    xlObj.XLFormat._formatPainter();
                    break;
                case xlObj._id + '_Ribbon_convertorange':
                    if (confirm(xlObj._getLocStr("ConverToRangeAlert"))) {
                        tid = parseInt(document.getElementById(xlObj._id + '_tableid').value);
                        tmgr = xlObj.getSheet(sheetIdx)._tableManager
                        var tableRange = xlObj.model.sheets[sheetIdx].filterSettings.tableRange, tableId;
                        for (i = 0; i < tableRange.length; i++)
                            tableRange[i].tableID == tid && (tableId = i);
                        details = { sheetIndex: sheetIdx, reqType: "format-table", action: "convertorange", beforeFormat: xlObj.XLFormat.getHashCodeClassAsArray() };
                        details.tableID = tid;
                        details.format = tmgr[tid].format;
                        details.formatName = tmgr[tid].formatName;
                        details.tableRange = xlObj.model.sheets[sheetIdx].filterSettings.tableRange[tableId];
                        details.beforeRange = tmgr[tid].range;
                        details.isFilter = tmgr[tid].isFilter;
                        details.firstColumn = tmgr[tid].firstColumn;
                        details.lastColumn = tmgr[tid].lastColumn;
                        details.totalRow = tmgr[tid].totalRow;
                        xlObj.isNumber(tid) && xlObj.XLFormat.removeTable(tid);
                        var actCell = xlObj.getCell(xlObj.getActiveCell().rowIndex, xlObj.getActiveCell().colIndex)[0];
                        xlObj.XLRibbon._toggleDesignTab(actCell);
                        xlObj._completeAction(details);

                    }
                    break;
                case xlObj._id + '_Ribbon_resizetable':
                    tid = parseInt(document.getElementById(xlObj._id + '_tableid').value);
                    range = xlObj.model.sheets[sheetIdx]._tableManager[tid].range;
                    $('#' + xlObj._id + '_fatresizetablerange').val(xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]));
                    $('#' + xlObj._id + '_fatresizetabledlg').ejDialog('open');
                    break;
                case xlObj._id + '_Ribbon_tsofirstcolumn':
                case xlObj._id + '_Ribbon_tsolastcolumn':
                    tmgr = sheet._tableManager, tid = document.getElementById(xlObj._id + '_tableid').value;
                    if (tid in tmgr) {
                        range = tmgr[tid].range;
                        range = this._id === xlObj._id + '_Ribbon_tsofirstcolumn' ? xlObj._getProperAlphaRange(sheetIdx, range[0] + 1, range[1], range[2], range[1]) : xlObj._getProperAlphaRange(sheetIdx, range[0] + 1, range[3], range[2], range[3]);
                        if (this._id === xlObj._id + '_Ribbon_tsofirstcolumn')
                            args.isChecked ? tmgr[tid]["firstColumn"] = true : delete tmgr[tid]["firstColumn"];
                        else
                            args.isChecked ? tmgr[tid]["lastColumn"] = true : delete tmgr[tid]["lastColumn"];
                        xlObj._dupDetails = true;
                        xlObj.XLFormat.format({ style: { 'font-weight': args.isChecked ? 'bold' : 'normal' } }, range);
                        xlObj._dupDetails = false;
                        details = { sheetIndex: sheetIdx, reqType: "format-table", action: "firstlastcolumn", range: range, check: args.isChecked, id: args.model.id };
                        xlObj._completeAction(details);
                    }
                    break;
                case xlObj._id + '_Ribbon_tsototalrow':
                    var cells, aRange, pcells = [], rData, i, isString = false;
                    tmgr = sheet._tableManager, tid = document.getElementById(xlObj._id + '_tableid').value;
                    if (tid in tmgr) {
                        range = tmgr[tid].range;
                        cells = xlObj.getRange(range[2] + 1, range[1], range[2] + 1, range[3]);
                        pcells.push(xlObj.getRangeData([range[2] + 1, range[1], range[2] + 1, range[3]]))
                        aRange = xlObj._getProperAlphaRange(sheetIdx, range[0] + 1, range[3], range[2], range[3]);
                        range = args.isChecked ? xlObj._getProperAlphaRange(sheetIdx, range[0] + 1, range[3], range[2], range[3]) : '';
                        if (cells.length > 1)
                            xlObj.XLEdit.updateCellValue($(cells[0]), args.isChecked ? 'Total' : '');

                        if (!tmgr[tid]["fnNumber"] && args.isChecked) {
                            rData = ej.distinct(xlObj.getRangeData(xlObj.getRangeIndices(aRange), true));
                            rData = rData.filter(Boolean);
                            i = rData.length;
                            if (i) {
                                while (i--) {
                                    if (!xlObj.isNumber(rData[i])) {
                                        isString = true;
                                        break;
                                    }
                                }
                            }
                            else
                                isString = true;
                            tmgr[tid]["fnNumber"] = isString ? 103 : 109;
                        }
                        xlObj.XLEdit.updateCellValue($(cells[cells.length - 1]), args.isChecked ? '=SUBTOTAL(' + tmgr[tid]["fnNumber"] + ',' + range + ')' : '');
                        args.isChecked ? tmgr[tid]["totalRow"] = true : "totalRow" in tmgr[tid] && delete tmgr[tid]["totalRow"];
                        details = { sheetIndex: sheetIdx, reqType: "format-table", action: "totalrow", range: aRange, check: args.isChecked, id: args.model.id, cell: cells, tmgr: tmgr, tid: tid, pcells: pcells[0] };
                        xlObj._completeAction(details);
                    }
                    break;
                case xlObj._id + '_Ribbon_tsofiltercolumn':
                    tmgr = sheet._tableManager, tid = document.getElementById(xlObj._id + '_tableid').value;
                    xlObj.XLFilter._clearFilterTableIcon(sheetIdx, parseInt(document.getElementById(xlObj._id + '_tableid').value));
                    tmgr[tid]["isFilter"] = args.isChecked;
                    details = { sheetIndex: sheetIdx, reqType: "format-table", action: "filtericon", tableId: document.getElementById(xlObj._id + '_tableid').value, id: args.model.id, check: args.isChecked };
                    xlObj._completeAction(details);
                    break;
                case xlObj._id + '_Ribbon_table':
                    xlObj.XLRibbon._openFATDlg();
                    break;
                case xlObj._id + "_Ribbon_newComment":
                    $("#" + xlObj._id + "_Ribbon_newComment").text() === xlObj._getLocStr("NewComment") ? xlObj.XLComment.setComment() : xlObj.XLComment.editComment();
                    break;
                case xlObj._id + "_Ribbon_deleteComment":
                    xlObj.XLComment.deleteComment();
                    break;
                case xlObj._id + "_Ribbon_showAllComments":
                    xlObj.XLComment.showAllComments();
                    break;
                case xlObj._id + "_Ribbon_showHideComment":
                    xlObj.XLComment.showHideComment();
                    break;
                case xlObj._id + "_Ribbon_nextComment":
                    xlObj.XLComment.findNextComment();
                    break;
                case xlObj._id + "_Ribbon_previousComment":
                    xlObj.XLComment.findPrevComment();
                    break;
                case xlObj._id + "_Ribbon_hyperlink":
                    xlObj._hlDlgOpen();
                    break;
                case xlObj._id + "_Ribbon_sortfilter":
                    switch (args.ID) {
                        case "SortAtoZ":
                        case "SortZtoA":
                            xlObj.model.allowSorting && xlObj.XLSort._sortHandler(args.ID);
                            break;
                        case "Filter":
                            xlObj.model.allowFiltering && xlObj.XLFilter.filter();
                            break;
                        default:
                            cname = xlObj.getCell(xlObj.getActiveCell().rowIndex, xlObj.getActiveCell().colIndex)[0].className;
                            if (cname.indexOf("e-table") < 0)
                                xlObj.XLFilter.clearFilter();
                            else {
                                tid = xlObj._getTableID(cname);
                                xlObj.XLFilter._clearFilterTable(xlObj.getActiveSheetIndex(), parseInt(tid), true);
                            }
                            break;
                    }
                    break;
                case xlObj._id + "_Ribbon_insbtn":
                case xlObj._id + "_Ribbon_inssbtn":
                    var selCells = xlObj.model.sheets[sheetIdx]._selectedCells;
                    if (selCells.length < 1)
                        return;
                    startCell = selCells[0], endCell = selCells[selCells.length - 1];
                    switch (args.ID) {
                        case "InsertCells":
                            if (xlObj._getJSSheetRowHeaderContent(sheetIdx).find('.e-rowselected').length)
                                xlObj.insertEntireRow(startCell, endCell);
                            else if (xlObj._getJSSheetHeader(sheetIdx).find('.e-colselected').length) {
                                xlObj._insDelStatus = "insert";
                                xlObj.insertEntireColumn(startCell, endCell);
                            }
                            else
                                xlObj._renderInsertDialog();
                            break;
                        case "InsertSheetRows":
                            xlObj.insertEntireRow(startCell, endCell);
                            break;
                        case "InsertSheetColumns":
                            xlObj._insDelStatus = "insert";
                            xlObj.insertEntireColumn(startCell, endCell);
                            break;
                        case "InsertSheet":
                            xlObj.insertSheet();
                            break;
                        default:
                            if (xlObj.element.find('.e-rowselected').length > 0)
                                xlObj.insertEntireRow(startCell, endCell);
                            else if (xlObj.element.find('.e-colselected').length > 0) {
                                xlObj._insDelStatus = "insert";
                                xlObj.insertEntireColumn(startCell, endCell);
                            }
                            else
                                xlObj.insertShiftBottom(startCell, endCell);
                            break;
                    }
                    break;
                case xlObj._id + "_Ribbon_delbtn":
                case xlObj._id + "_Ribbon_delsbtn":
                    var selCells = xlObj.model.sheets[sheetIdx]._selectedCells;
                    if (selCells.length < 1)
                        return;
                    startCell = selCells[0], endCell = selCells[selCells.length - 1];
                    switch (args.ID) {
                        case "DeleteCells":
                            if (xlObj._getJSSheetRowHeaderContent(sheetIdx).find('.e-rowselected').length)
                                xlObj.deleteEntireRow(startCell, endCell, []);
                            else if (xlObj._getJSSheetHeader(sheetIdx).find('.e-colselected').length) {
                                xlObj._insDelStatus = "delete";
                                xlObj.deleteEntireColumn(startCell, endCell, []);
                            }
                            else
                                xlObj._renderDeleteDialog();
                            break;
                        case "DeleteSheetRows":
                            xlObj.deleteEntireRow(startCell, endCell, []);
                            break;
                        case "DeleteSheetColumns":
                            xlObj._insDelStatus = "delete";
                            xlObj.deleteEntireColumn(startCell, endCell, []);
                            break;
                        case "DeleteSheet":
                            xlObj.deleteSheet(sheetIdx);
                            break;
                        default:
                            if (xlObj.element.find('.e-rowselected').length > 0)
                                xlObj.deleteEntireRow(startCell, endCell);
                            else if (xlObj.element.find('.e-colselected').length > 0) {
                                xlObj._insDelStatus = "insert";
                                xlObj.deleteEntireColumn(startCell, endCell);
                            }
                            else
                                xlObj.deleteShiftUp(startCell, endCell);
                            break;
                    }
                    break;
                case xlObj._id + "_Ribbon_clear":
                    range = sheet.selectedRange;
                    switch (args.ID) {
                        case "Clear_All":
                            range = xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]);
                            xlObj.clearAll(range);
                            break;
                        case "Clear_Formats":
                            range = xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]);
                            xlObj.clearAllFormat(range);
                            break;
                        case "Clear_Contents":
                            range = xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]);
                            xlObj.clearContents(range);
                            break;
                        case "Clear_Comments":
                            xlObj.model.allowComments && xlObj.XLComment.deleteComment(undefined, "clear");
                            break;
                        default:
                            xlObj._removeHyperlink("clear");
                            break;
                    }
                    xlObj.XLSelection._clearBorder(xlObj._arrayAsString(xlObj._cutFocus), sheetIdx);
                    break;
                case xlObj._id + "_Ribbon_pivotTable":
                    info = { sheetIndex: sheetIdx, model: xlObj.model.sheets };
                    if (xlObj._trigger("_createPivotTable", info))
                        return;
                    break;
                case xlObj._id + "_Ribbon_charttype":
                    $("#" + xlObj._id + "_charttypedlg").ejDialog("open");
                    break;
                case xlObj._id + "_Ribbon_pictures":
                    if (xlObj.model.importSettings.importMapper.length < 1) {
                        xlObj._showAlertDlg("Alert", "ImportExportUrl");
                        return;
                    }
                    xlObj._uploadImage = true;
                    $("#" + xlObj._id + "_file .e-uploadinput").click();
                    xlObj.XLRibbon._toggleFormatTab();
                    break;
                case xlObj._id + "_Ribbon_namemanager":
                    xlObj.XLRibbon._openNameManagerDlg();
                    break;
                case xlObj._id + "_Ribbon_definename":
                    var grid = $("#" + xlObj._id + "_nmgrid").data("ejGrid");
                    grid.getContentTable().find('td').addClass("e-ss-emptyrecord")
                    if (grid.model.dataSource.length < 1)
                        grid.getContentTable().find('td').addClass("e-ss-emptyrecord");
                    else
                        grid.getContentTable().find('.e-ss-emptyrecord').removeClass("e-ss-emptyrecord");
                    $("#" + xlObj._id + "_nmdlg").ejDialog("open"); // due to grid issue
                    $("#" + xlObj._id + "_nmgrid").ejGrid("instance").addRecord();
                    ($("#" + xlObj._id + "_nmgridEditForm input")[0]).focus();
                    break;
                case xlObj._id + "_Ribbon_fPane":
                    switch (args.ID) {
                        case "freezePanes":
                            xlObj.XLFreeze.freezePanes(xlObj.model.sheets[sheetIdx]._activeCell);
                            break;
                        case "FreezeRow":
                            xlObj.XLFreeze.freezeTopRow();
                            break;
                        case "FreezeColumn":
                            xlObj.XLFreeze.freezeLeftColumn();
                            break;
                        case "unfreezePanes":
                            xlObj.XLFreeze.unfreezePanes();
                        default:
                            break;
                    }
                    break;
                case xlObj._id + "_Ribbon_merge":
                    if (ej.isNullOrUndefined(args.ID)) {
                        xlObj._mergeCenter = true;
                        xlObj.mergeCells();
                    }
                    switch (args.ID) {
                        case "MergeCells":
                            xlObj.mergeCells();
                            break;
                        case "MergeAcross":
                            xlObj.mergeAcrossCells();
                            break;
                        case "MergeAndCenter":
                            xlObj._mergeCenter = true;
                            xlObj.mergeCells();
                            break;
                        case "UnmergeCells":
                            xlObj.unmergeCells();
                            break;
                    }
                    break;
                case xlObj._id + "_Ribbon_autosum":
                    cellRange = sheet.selectedRange;
                    xlObj.XLRibbon.autoSum(args.ID.toUpperCase(), xlObj._getAlphaRange(sheetIdx, cellRange[0], cellRange[1], cellRange[2], cellRange[3]));
                    break;
                case xlObj._id + "_Ribbon_cElement":
                    dataVal = xlObj.getSheet(sheetIdx)._shapeMngr.chart[xlObj.XLEdit.getPropertyValue(xlObj.XLShape._picCellIdx.rowIndex, xlObj.XLShape._picCellIdx.colIndex, 'chart')[0]];
                    var visiblity, cid = document.getElementById(xlObj._id + "_chart").value, series, cObj = $("#" + cid).ejChart("instance");
                    details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", cid: cid, rowIndex: xlObj.XLShape._picCellIdx.rowIndex, colIndex: xlObj.XLShape._picCellIdx.colIndex };
                    switch (args.ID) {
                        case "PHAxis":
                            details.visibility = !(cObj.model.primaryXAxis.visible);
                            cObj.option("primaryXAxis", { visible: details.visibility });
                            dataVal.xAxis.visible = details.visibility;
                            details.action = args.ID;
                            break;
                        case "PVAxis":
                            details.visibility = !(cObj.model.primaryYAxis.visible);
                            cObj.option("primaryYAxis", { visible: details.visibility });
                            dataVal.yAxis.visible = details.visibility;
                            details.action = args.ID;
                            break;
                        case "PHAxisTitle":
                            xlObj._cOpt.prev = { text: dataVal.xAxis.title.text };
                            if (cObj.model.primaryXAxis.title.text.length != 0) {
                                cObj.option("primaryXAxis", { title: { text: "" } });
                                dataVal.xAxis.title.text = "";
                                details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "PX", cid: cid, rowIndex: xlObj.XLShape._picCellIdx.rowIndex, colIndex: xlObj.XLShape._picCellIdx.colIndex, prev: { text: xlObj._cOpt.prev.text }, cur: { text: "" } };
                                xlObj._completeAction(details);
                            }
                            else {
                                xlObj._hasTitle = "PX";
                                document.getElementById(xlObj._id + '_chartname').value = dataVal.xAxis.title.text;
                                $('#' + xlObj._id + '_chartnamedlg').ejDialog("open");
                                $("#" + xlObj._id + '_chartname').focus().setInputPos(dataVal.xAxis.title.text.length);
                                xlObj._cOpt = { prevText: dataVal.title.text, prevAlign: cObj.model.title.textAlignment, curAlign: cObj.model.title.textAlignment };
                            }
                            break;
                        case "PVAxisTitle":
                            xlObj._cOpt.prev = { text: dataVal.yAxis.title.text };
                            if (cObj.model.primaryYAxis.title.text.length != 0) {
                                cObj.option("primaryYAxis", { title: { text: "" } });
                                dataVal.yAxis.title.text = "";
                                details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "PY", cid: cid, rowIndex: xlObj.XLShape._picCellIdx.rowIndex, colIndex: xlObj.XLShape._picCellIdx.colIndex, prev: { text: xlObj._cOpt.prev.text }, cur: { text: "" } };
                                xlObj._completeAction(details);
                            }
                            else {
                                xlObj._hasTitle = "PY";
                                document.getElementById(xlObj._id + '_chartname').value = dataVal.yAxis.title.text;
                                $('#' + xlObj._id + '_chartnamedlg').ejDialog("open");
                                $("#" + xlObj._id + '_chartname').focus().setInputPos(dataVal.yAxis.title.text.length);
                                xlObj._cOpt = { prevText: dataVal.title.text, prevAlign: cObj.model.title.textAlignment, curAlign: cObj.model.title.textAlignment };
                            }
                            break;
                        case "CTNone":
                            xlObj._cOpt.prev = { text: dataVal.title.text };
                            if (cObj.model.title.text.length) {
                                cObj.option("title", { text: "" });
                                dataVal.title.text = "";
                                details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "CT", cid: cid, rowIndex: xlObj.XLShape._picCellIdx.rowIndex, colIndex: xlObj.XLShape._picCellIdx.colIndex, prev: { text: xlObj._cOpt.prev.text }, cur: { text: "" } };
                                xlObj._completeAction(details);
                            }
                            break;
                        case "CTCenter":
                            xlObj._hasTitle = "CT";
                            xlObj._cOpt = { prevText: cObj.model.title.text, prevAlign: cObj.model.title.textAlignment };
                            document.getElementById(xlObj._id + '_chartname').value = dataVal.title.text;
                            $('#' + xlObj._id + '_chartnamedlg').ejDialog("open");
                            $("#" + xlObj._id + '_chartname').focus().setInputPos(dataVal.title.text.length);
                            cObj.option("title", { textAlignment: "center" });
                            xlObj._cOpt.curAlign = cObj.model.title.textAlignment;
                            dataVal.title.align = "center";
                            break;
                        case "CTFar":
                            xlObj._hasTitle = "CT";
                            xlObj._cOpt = { prevText: cObj.model.title.text, prevAlign: cObj.model.title.textAlignment };
                            document.getElementById(xlObj._id + '_chartname').value = dataVal.title.text;
                            $('#' + xlObj._id + '_chartnamedlg').ejDialog("open");
                            $("#" + xlObj._id + '_chartname').focus().setInputPos(dataVal.title.text.length);
                            cObj.option("title", { textAlignment: "far" });
                            xlObj._cOpt.curAlign = cObj.model.title.textAlignment;
                            dataVal.title.align = "far";
                            break;
                        case "CTNear":
                            xlObj._hasTitle = "CT";
                            xlObj._cOpt = { prevText: cObj.model.title.text, prevAlign: cObj.model.title.textAlignment };
                            document.getElementById(xlObj._id + '_chartname').value = dataVal.title.text;
                            $('#' + xlObj._id + '_chartnamedlg').ejDialog("open");
                            $("#" + xlObj._id + '_chartname').focus().setInputPos(dataVal.title.text.length);
                            cObj.option("title", { textAlignment: "near" });
                            xlObj._cOpt.curAlign = cObj.model.title.textAlignment;
                            dataVal.title.align = "near";
                            break;
                        case "DLNone":
                            series = [];
                            details.action = "DL";
                            details.prev = { series: $.extend(true, [], cObj.model.series) };
                            visiblity = (cObj.model.series[0].marker.dataLabel.visible) ? false : true;
                            for (i = 0; i < cObj.model.series.length; i++)
                                series.push({ marker: { dataLabel: { visible: visiblity, verticalTextAlignment: "far", textPosition: "top" } } });
                            cObj.option({ series: series });
                            details.cur = { series: $.extend(true, [], cObj.model.series) };
                            dataVal.dataLabel = { visible: visiblity };
                            break;
                        case "DLCenter":
                            series = [];
                            details.action = "DL";
                            details.prev = { series: $.extend(true, [], cObj.model.series) };
                            for (i = 0; i < cObj.model.series.length; i++)
                                series.push({ marker: { dataLabel: { visible: true, verticalTextAlignment: "center", textPosition: "middle" } } });
                            cObj.option({ series: series });
                            details.cur = { series: $.extend(true, [], cObj.model.series) };
                            dataVal.dataLabel = { visible: true, vAlign: "center", tPosn: "middle" };
                            break;
                        case "DLIBase":
                            series = [];
                            details.action = "DL";
                            details.prev = { series: $.extend(true, [], cObj.model.series) };
                            for (i = 0; i < cObj.model.series.length; i++)
                                series.push({ marker: { dataLabel: { visible: true, verticalTextAlignment: "far", textPosition: "bottom" } } });
                            cObj.option({ series: series });
                            details.cur = { series: $.extend(true, [], cObj.model.series) };
                            dataVal.dataLabel = { visible: true, vAlign: "far", tPosn: "bottom" };
                            break;
                        case "DLIEnd":
                            series = [];
                            details.action = "DL";
                            details.prev = { series: $.extend(true, [], cObj.model.series) };
                            for (i = 0; i < cObj.model.series.length; i++)
                                series.push({ marker: { dataLabel: { visible: true, verticalTextAlignment: "near", textPosition: "top" } } });
                            cObj.option({ series: series });
                            details.cur = { series: $.extend(true, [], cObj.model.series) };
                            dataVal.dataLabel = { visible: true, vAlign: "near", tPosn: "top" };
                            break;
                        case "DLOEnd":
                            series = [];
                            details.action = "DL";
                            details.prev = { series: $.extend(true, [], cObj.model.series) };
                            for (i = 0; i < cObj.model.series.length; i++)
                                series.push({ marker: { dataLabel: { visible: true, verticalTextAlignment: "far", textPosition: "top" } } });
                            cObj.option({ series: series });
                            details.cur = { series: $.extend(true, [], cObj.model.series) };
                            dataVal.dataLabel = { visible: true, vAlign: "far", tPosn: "top" };
                            break;
                        case "PMajorH":
                            details.visibility = !(cObj.model.primaryYAxis.majorGridLines.visible);
                            cObj.option("primaryYAxis", { majorGridLines: { visible: details.visibility } });
                            dataVal.yAxis.majorGridLines = { visible: details.visibility };
                            details.action = args.ID;
                            break;
                        case "PMajorV":
                            details.visibility = !(cObj.model.primaryXAxis.majorGridLines.visible);
                            cObj.option("primaryXAxis", { majorGridLines: { visible: details.visibility } });
                            dataVal.xAxis.majorGridLines = { visible: details.visibility };
                            details.action = args.ID;
                            break;
                        case "PMinorH":
                            details.visibility = !(cObj.model.primaryYAxis.minorGridLines.visible);
                            cObj.option("primaryYAxis", { minorGridLines: { visible: details.visibility }, minorTicksPerInterval: 5, minorTickLines: { visible: details.visibility } });
                            dataVal.yAxis.minorGridLines = { visible: details.visibility };
                            details.action = args.ID;
                            break;
                        case "PMinorV":
                            details.visibility = !(cObj.model.primaryXAxis.minorGridLines.visible);
                            cObj.option("primaryXAxis", { minorGridLines: { visible: details.visibility }, minorTicksPerInterval: 5, minorTickLines: { visible: details.visibility } });
                            dataVal.xAxis.minorGridLines = { visible: details.visibility };
                            details.action = args.ID;
                            break;
                        case "LNone":
                            details.visibility = !(cObj.model.legend.visible);
                            cObj.option("legend", { visible: details.visibility });
                            dataVal.legend.visible = details.visibility;
                            details.action = args.ID;
                            break;
                        case "LLeft":
                            details.prev = { position: cObj.model.legend.position };
                            cObj.option("legend", { position: "left", visible: true });
                            dataVal.legend.position = "left";
                            details.cur = { position: cObj.model.legend.position };
                            details.action = "Legend";
                            break;
                        case "LRight":
                            details.prev = { position: cObj.model.legend.position };
                            cObj.option("legend", { position: "right", visible: true });
                            dataVal.legend.position = "right";
                            details.cur = { position: cObj.model.legend.position };
                            details.action = "Legend";
                            break;
                        case "LBottom":
                            details.prev = { position: cObj.model.legend.position };
                            cObj.option("legend", { position: "bottom", visible: true });
                            dataVal.legend.position = "bottom";
                            details.cur = { position: cObj.model.legend.position };
                            details.action = "Legend";
                            break;
                        case "LTop":
                            details.prev = { position: cObj.model.legend.position };
                            cObj.option("legend", { position: "top" });
                            dataVal.legend.position = "top";
                            details.cur = { position: cObj.model.legend.position };
                            details.action = "Legend";
                            break;
                    }
                    if (!ej.isNullOrUndefined(details.action))
                        xlObj._completeAction(details);
                    break;
                case xlObj._id + "_Ribbon_chartwidth":
                    xlObj.XLRibbon._changeChartSize("height", args.value);
                    break;
                case xlObj._id + "_Ribbon_chartheight":
                    xlObj.XLRibbon._changeChartSize("width", args.value);
                    break;
                case xlObj._id + "_Ribbon_srcolumn":
                    var chOptions;
                    cid = document.getElementById(xlObj._id + "_chart").value, cObj = $("#" + cid).ejChart("instance"), dataVal = xlObj.XLChart._getShapeObj(cid, "chart"), details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "srcolumn", cid: cid, rowIndex: xlObj.XLShape._picCellIdx.rowIndex, colIndex: xlObj.XLShape._picCellIdx.colIndex };
                    cObj.model.series.length = 0;
                    dataVal.isRowColSwitched = !dataVal.isRowColSwitched;
                    obj = { xRange: dataVal.xRange, yRange: dataVal.yRange, lRange: dataVal.lRange, isRowColSwitched: dataVal.isRowColSwitched, dataSheetIdx: dataVal.dataSheetIdx };
                    chOptions = xlObj.XLChart._processChartSeries(obj);
                    cObj.option({ series: chOptions.series });
                    xlObj.getActiveSheetIndex() === dataVal.dataSheetIdx && xlObj.XLChart._focusChartRange(chOptions.xRange, chOptions.yRange, chOptions.lRange);
                    dataVal.xAxis.range = chOptions.xRange;
                    dataVal.yAxis.range = chOptions.yRange;
                    dataVal.legend.range = chOptions.lRange;
                    xlObj._completeAction(details);
                    break;
                case xlObj._id + "_Ribbon_PictureColor":
                case xlObj._id + "_Ribbon_pictureborder":
                    if (["weight", "dashes", xlObj._id + "picturecolor", ""].indexOf(args.ID) === -1) {
                        elem = xlObj.element.find(".e-ss-activeimg");
                        var regx = new RegExp("\\b" + "e-shapebdr" + ".*?\\b", "g"), style;
                        if (elem[0].className.match(regx))
                            style = xlObj.XLShape._getImgStyleFromHashCode(elem[0].className.match(regx)[0]);
                        else
                            style = { "border-color": "transparent", "border-width": "1px", "border-style": "solid" };
                        var color = style["border-color"], width = style["border-width"], style = style["border-style"];
                        if (elem[0]) {
                            switch (true) {
                                case this._id.indexOf("PictureColor") > -1:
                                    color = args.value;
                                    break;
                                case args.ID.indexOf("px") > -1:
                                    width = args.ID;
                                    break;
                                case args.ID === "nooutline":
                                    color = "transparent";
                                    break;
                                default:
                                    style = args.ID;
                                    break;
                            }
                            xlObj.XLShape._updateImgBorder(elem[0], width, style, color);
                        }
                    }
                    break;
                case xlObj._id + "_Ribbon_resetpic":
                    elem = xlObj.element.find(".e-ss-activeimg"), regx = new RegExp("\\b" + "e-shapebdr" + ".*?\\b", "g");
                    details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "picture", action: "resetpic", id: $(elem).attr("id"), actionType: args.ID };
                    xlObj._dupDetails = true;
                    if (!ej.isNullOrUndefined(elem[0].className.match(regx))) {
                        imgStyle = xlObj.XLShape._getImgStyleFromHashCode(elem[0].className.match(regx)[0]);
                    }
                    details.prev = { bcolor: imgStyle["border-color"], bstyle: imgStyle["border-style"], bwidth: imgStyle["border-width"] };
                    elem[0].className = elem[0].className.replace(regx, "");
                    details.cur = { bcolor: elem.css("border-color"), bstyle: elem.css("border-style"), bwidth: elem.css("border-width") };
                    elem[0].className = elem[0].className + " " + xlObj.XLShape._getImgBorderHashCode({ "border-color": details.cur.bcolor, "border-style": details.cur.bstyle, "border-width": details.cur.bwidth });
                    if (args.ID !== "resetpicture") {
                        details.prev.width = elem.css("width");
                        details.prev.height = elem.css("height");
                        elem.css({ width: xlObj.model.pictureSettings.width, height: xlObj.model.pictureSettings.height });
                        details.cur.width = elem.css("width");
                        details.cur.height = elem.css("height");
                        xlObj.XLShape._updateShapeObj(elem[0]);
                        xlObj.XLRibbon._formatTabUpdate();
                    }
                    xlObj._dupDetails = false;
                    xlObj._completeAction(details);
                    break;
                case xlObj._id + "_Ribbon_changepic":
                    xlObj._uploadImage = true;
                    xlObj.XLShape._changePicture = true;
                    xlObj.element.find("#" + xlObj._id + "_file .e-uploadinput").click();
                    xlObj.XLRibbon._toggleFormatTab();
                    break;
                case xlObj._id + "_Ribbon_selectData":
                    cid = document.getElementById(xlObj._id + "_chart").value, dataVal = xlObj.XLChart._getShapeObj(cid, "chart");
                    $("#" + xlObj._id + "_crxaxis").val(dataVal.xRange ? xlObj._getAlphaRange(sheetIdx, dataVal.xRange[0], dataVal.xRange[1], dataVal.xRange[2], dataVal.xRange[3]) : "");
                    $("#" + xlObj._id + "_cryaxis").val(dataVal.yRange ? xlObj._getAlphaRange(sheetIdx, dataVal.yRange[0], dataVal.yRange[1], dataVal.yRange[2], dataVal.yRange[3]) : "");
                    $("#" + xlObj._id + "_crlaxis").val(dataVal.lRange ? xlObj._getAlphaRange(sheetIdx, dataVal.lRange[0], dataVal.lRange[1], dataVal.lRange[2], dataVal.lRange[3]) : "");
                    xlObj._selectDataval = { xRange: $("#" + xlObj._id + "_crxaxis").val(), yRange: $("#" + xlObj._id + "_cryaxis").val(), lRange: $("#" + xlObj._id + "_crlaxis").val() }
                    $("#" + xlObj._id + "_chartrangedlg").ejDialog("open");
                    $("#" + xlObj._id + "_crxaxis").focus().setInputPos(0);
                    break;
                case xlObj._id + "_Ribbon_chartTheme":
                    var chartId = xlObj.element.find("#" + xlObj._id + "_chart").val();
                    details = { sheetIndex: sheetIdx, reqType: "shape", action: "chartTheme" };
                    details.prev = { theme: $("#" + chartId).data("ejChart").model.theme };
                    (this.selectedTextValue.indexOf("Dark") > -1) ? ($("#" + chartId).addClass('e-ss-charttheme')) : ($("#" + chartId).removeClass('e-ss-charttheme'));
                    xlObj.XLChart.refreshChart(chartId, { theme: this._selectedValue });
                    details.cur = { theme: this._selectedValue };
                    xlObj._completeAction(details);
                    break;
                case xlObj._id + "_Ribbon_LockCell":
                    xlObj.lockCells(null, args.model.toggleState);
                    break;
                case xlObj._id + "_Ribbon_ProtectSheet":
                    xlObj.protectSheet(args.model.toggleState);
                    break;
                case xlObj._id + "_Ribbon_protectWorkbook":
                    $("#" + xlObj._id + "_PasswordDialog").ejDialog("open");
                    break;
            }
            //xlObj.setSheetFocus(); //Comment Editing - focus missing
        },

        _changeChartSize: function (status, value) {
            var xlObj = this.XLObj, pSize, newVal, dataVal = xlObj.XLEdit.getPropertyValue(xlObj.XLShape._picCellIdx.rowIndex, xlObj.XLShape._picCellIdx.colIndex, 'chart')[0], cid = document.getElementById(xlObj._id + "_chart").value, cObj = $("#" + cid).ejChart("instance"), chartEle = $("#" + cid), details = { sheetIndex: xlObj.getActiveSheetIndex(), reqType: "shape", shapeType: "chart", action: status, cid: cid, rowIndex: xlObj.XLShape._picCellIdx.rowIndex, colIndex: xlObj.XLShape._picCellIdx.colIndex, prev: { width: cObj.model.size.width, height: cObj.model.size.height } };
            pSize = cObj.model.size;
            if (status === "height") {
                cObj.option("size", { width: value.toString() });
                dataVal.width = value;
                chartEle[0].style.width = value + "px";
                details.cur = { width: value.toString(), height: cObj.model.size.height };
            }
            else {
                cObj.option("size", { height: value.toString() });
                dataVal.height = value;
                chartEle[0].style.height = value + "px";
                details.cur = { width: cObj.model.size.width, height: value.toString() };
            }
            xlObj._completeAction(details);
        },

        _updateBordeStyle: function (args) {
            var xlObj = this.XLObj;
            xlObj._borderStyle = args.ID && args.ID.length > 0 && args.ID;
        },

        _colorPickerHandler: function (name, args) {
            var xlFormat, xlObj = this.XLObj;
            if (args.type === "open") {
                $("#" + xlObj._id + "_" + name + "_popup").find(".e-buttons, .e-button").hide();
                $("#" + xlObj._id + "_" + name + "_popup").focus();
            }
            else if (args.type === "change") { //color change  
                xlFormat = xlObj.XLFormat;
                var cpObj = $("#" + xlObj._id + "_" + name).data("ejColorPicker");
                if (name === "Ribbon_color") {
                    xlFormat.format({ "style": { "color": args.value } });
                    $("#" + xlObj._id + "_Ribbon_home_Font_Color").find(".e-selected-color").css("background-color", args.value);
                }
                else {
                    xlFormat.format({ "style": { "background-color": args.value } });
                    $("#" + xlObj._id + "_Ribbon_home_Font_Background").find(".e-selected-color").css("background-color", args.value);
                }
                cpObj.hide();
                cpObj.setValue(args.value);
            }
        },

        updateRibbonIcons: function () {
            this._updateRibbonIcons();
        },

        _updateRibbonIcons: function () { //scrip error
            if (!this.XLObj.model.showRibbon)
                return;
            var isLocked,lockbtObj, xlObj = this.XLObj, cfObj = $("#" + xlObj._id + "_Ribbon_cellformat").data("ejDropDownList"), sheetIdx = xlObj.getActiveSheetIndex(), actSheet = xlObj.getSheet(sheetIdx);
            var fsObj = $("#" + xlObj._id + "_Ribbon_fontsize").data("ejDropDownList");
            $("#" + xlObj._id + "_Ribbon_fontsize").css("width", "auto");
            var ffObj = $("#" + xlObj._id + "_Ribbon_font-family").data("ejDropDownList");
            var index = -1, element;
            var rObj = $("#" + xlObj._id + "_Ribbon").data("ejRibbon");
            switch (rObj.getTabText(rObj.model.selectedItemIndex)) {
                case "HOME":
                    var activeCell = $(xlObj.getCell(actSheet._activeCell.rowIndex, actSheet._activeCell.colIndex)), isWrap = xlObj.XLEdit.getPropertyValue(actSheet._activeCell.rowIndex, actSheet._activeCell.colIndex, "wrap"), isMerge = xlObj.XLEdit.getPropertyValue(actSheet._activeCell.rowIndex, actSheet._activeCell.colIndex, "isMerge");
                    var style = !ej.isNullOrUndefined(xlObj.XLFormat) && xlObj.XLFormat.getFormatFromHashCode(xlObj.XLFormat.getFormatClass(activeCell[0].className)), type =xlObj.XLEdit.getPropertyValue(actSheet._activeCell.rowIndex, actSheet._activeCell.colIndex, "type");
                    xlObj.isUndefined(isWrap) && (isWrap = false);
                    $("#" + xlObj._id + "_Ribbon_wraptext").ejToggleButton("option", { toggleState: isWrap });
                    if (isMerge) {
                        $("#" + xlObj._id + "_Ribbon_merge").addClass("e-ss-active");
                        $("#" + xlObj._id + "_Ribbon_mergedrpbtn").addClass("e-ss-active");
                    }
                    else {
                        $("#" + xlObj._id + "_Ribbon_merge").removeClass("e-ss-active");
                        $("#" + xlObj._id + "_Ribbon_mergedrpbtn").removeClass("e-ss-active");
                    }
                    type && (this._numberFormat.some(function (elem, i) {
                        if (type === elem.text.toLowerCase()) {
                            index = i;
                            return true;
                        }
                        return false;
                    }));
            		if(!ej.isNullOrUndefined(cfObj))
            		{
            			cfObj._initValue = true;
            			index > -1 ? cfObj.option({ selectedItemIndex: index }) : (!ej.isNullOrUndefined(cfObj.selectedIndexValue) && cfObj.selectedIndexValue != 0 && cfObj.option({ selectedItemIndex: 0 }));
            			cfObj._initValue = false;
            		}
                    if (style) {
                        !ej.isNullOrUndefined(style["font-size"]) ? (this._fontSize.some(function (elem, i) {
                            if (style["font-size"] === elem.value) { index = i; return true; }
                            return false;
                        })) : (index = -1);
                        fsObj._initValue = true;
                        index > -1 ? (fsObj.option({ selectedItemIndex: index }) || (index = -1)) : (!ej.isNullOrUndefined(fsObj.selectedIndexValue) && fsObj.selectedIndexValue != 3 && fsObj.option({ selectedItemIndex: 3 }));
                        fsObj._initValue = false;
                        !ej.isNullOrUndefined(style["font-family"]) ? (this._fontFamily.some(function (elem, i) {
                            if (style["font-family"] === elem.text.toLowerCase()) { index = i; return true; }
                            return false;
                        })) : (index = -1);
                        ffObj._initValue = true;
                        index > -1 ? ffObj.option({ selectedItemIndex: index }) : (!ej.isNullOrUndefined(ffObj.selectedIndexValue) && ffObj.selectedIndexValue != 6 && ffObj.option({ selectedItemIndex: 6 }));
                        ffObj._initValue = false;
                        style["font-weight"] == "bold" ? $("#" + xlObj._id + "_Ribbon_bold").data("ejToggleButton").option("toggleState", true) : $("#" + xlObj._id + "_Ribbon_bold").data("ejToggleButton").option("toggleState", false);
                        style["font-style"] == "italic" ? $("#" + xlObj._id + "_Ribbon_italic").data("ejToggleButton").option("toggleState", true) : $("#" + xlObj._id + "_Ribbon_italic").data("ejToggleButton").option("toggleState", false);
                        style["text-decoration"] = style["text-decoration"] || "";
                        $("#" + xlObj._id + "_Ribbon_underline").data("ejToggleButton").option("toggleState", style["text-decoration"].indexOf("underline") !== -1);
                        $("#" + xlObj._id + "_Ribbon_line-through").data("ejToggleButton").option("toggleState", style["text-decoration"].indexOf("line-through") !== -1);
                        element = $(xlObj.element).find(".e-text-align");
                        !ej.isNullOrUndefined(element[0]) && ($("#" + element[0].id).data("ejToggleButton").option("toggleState", false) || element.removeClass("e-text-align"));
                        !ej.isNullOrUndefined(style["text-align"]) && ($("#" + xlObj._id + "_Ribbon_" + style["text-align"]).data("ejToggleButton").option("toggleState", true) || $("#" + xlObj._id + "_Ribbon_" + style["text-align"]).addClass("e-text-align"));
                        element = $(xlObj.element).find(".e-vertical-align");
                        !ej.isNullOrUndefined(element[0]) && ($("#" + element[0].id).data("ejToggleButton").option("toggleState", false) || element.removeClass("e-vertical-align"));
                        !ej.isNullOrUndefined(style["vertical-align"]) ? ($("#" + xlObj._id + "_Ribbon_" + style["vertical-align"]).data("ejToggleButton").option("toggleState", true) || $("#" + xlObj._id + "_Ribbon_" + style["vertical-align"]).addClass("e-vertical-align")) : ($("#" + xlObj._id + "_Ribbon_bottom").data("ejToggleButton").option("toggleState", true) || $("#" + xlObj._id + "_Ribbon_bottom").addClass("e-vertical-align"));
                    }
                    break;
                case "REVIEW":
                    if (xlObj.model.allowLockCell) {
                        lockbtObj = $("#" + xlObj._id + "_Ribbon_LockCell").data("ejToggleButton");
                        isLocked = xlObj.XLEdit.getPropertyValue(actSheet._activeCell.rowIndex, actSheet._activeCell.colIndex, "isLocked");
                        xlObj.isUndefined(isLocked) && (isLocked = false)
                        lockbtObj.option("toggleState", isLocked);
                        $("#" + xlObj._id + "_Ribbon_ProtectSheet").data("ejToggleButton").option("toggleState", actSheet._isSheetProtected);
                        if (!actSheet._isSheetProtected)
                            lockbtObj.enable();
                        else
                            lockbtObj.disable();
                    }
                    if (!actSheet._isSheetProtected || !xlObj.model.allowLockCell)
                        (xlObj.model.allowComments && !xlObj.XLComment._isCommentEdit) && !xlObj.XLEdit._isEdit && !xlObj._isSheetRename && xlObj.XLComment._updateCmntRibIcons(xlObj.getActiveCellElem());

                    break;
            }
        },

        _disableButtons: function (idCollection, type) {
            var i, index = -1, len, elem, xlObj = this.XLObj;
            for (i = 0, len = idCollection.length; i < len; i++) {
                if (xlObj.model.allowLockCell)
                    if (!xlObj.XLEdit._isEdit && !xlObj._isSheetRename && xlObj.getSheet(xlObj.getActiveSheetIndex())._isSheetProtected)
                        index = this._protectStateBtnIds.indexOf(idCollection[i]);
                if (index === -1) {
                    elem = xlObj.element.find("#" + xlObj._id + "_Ribbon_" + idCollection[i]);
                    elem.length && !ej.isNullOrUndefined(elem.data(type)) && elem.data(type).disable();
                }
            }
        },

        _enableButtons: function (idCollection, type) {
            var i, index = 0, len, elem, xlObj = this.XLObj;
            for (i = 0, len = idCollection.length; i < len; i++) {
                if (xlObj.model.allowLockCell && !xlObj._isSheetRename && xlObj.getSheet(xlObj.getActiveSheetIndex())._isSheetProtected)
                    index = this._protectStateBtnIds.indexOf(idCollection[i]);
                if (index > -1) {
                    elem = xlObj.element.find("#" + xlObj._id + "_Ribbon_" + idCollection[i]);
                    elem.length && !ej.isNullOrUndefined(elem.data(type)) && elem.data(type).enable();
                }
            }
        },

        _disableInputs: function (idColl, disable) {
            for (var i = 0, len = idColl.length; i < len; i++)
                $("#" + this.XLObj._id + "_" + idColl[i]).attr("disabled", disable);
        },

        _enableRibbonIcons: function (rbnActTabIndex) {
            if (!this.XLObj.model.showRibbon)
                return;
            var index, xlObj = this.XLObj, rObj = $("#" + xlObj._id + "_Ribbon").data("ejRibbon");
            index = rbnActTabIndex ? rbnActTabIndex : rObj.model.selectedItemIndex;
            switch (rObj.getTabText(index)) {
                case "HOME":
                    this._enableButtons(this._homeBtnIds, "ejButton");
                    this._enableButtons(this._toggleBtnIds, "ejToggleButton");
                    this._enableButtons(this._ddIds, "ejDropDownList");
                    this._enableButtons(this._cpIds, "ejColorPicker");
                    this._enableButtons(this._splitBtnIds, "ejSplitButton");
                    this._enableButtons(this._homeSptBtnIds, "ejSplitButton");
                    break;
                case "INSERT":
                    this._enableButtons(this._insertBtnIds, "ejButton");
                    this._enableButtons(this._insertSptBtnIds, "ejSplitButton");
                    break;
                case "DATA":
                    this._enableButtons(this._dataBtnIds, "ejButton");
                    this._enableButtons(this._dataSptBtnIds, "ejSplitButton");
                    break;
                case "PAGE LAYOUT":
                    this._enableButtons(this._pageLaySptBtnIds, "ejSplitButton");
                    this._enableButtons(this._pageLayBtnIds, "ejButton");
                    this._enableButtons(this._pageLayChckBxIds, "ejCheckBox");
                    $("#" + rObj._id + "_pagesetup_viewhdrlbl").eq(0).removeClass("e-disable");
                    $("#" + rObj._id + "_pagesetup_viewgrdlnlbl").eq(0).removeClass("e-disable");
                    break;
                case "FORMULAS":
                    this._enableButtons(this._formulaBtnIds, "ejButton");
                    this._enableButtons(this._formulaSptBtnIds, "ejSplitButton");
                    break;
                case "REVIEW":
                    xlObj.model.allowComments && !xlObj.XLComment._isCommentEdit && xlObj.XLComment._updateCmntRibIcons();
                    break;
                case "OTHERS":
                    this._enableButtons(this._othersSptBtnIds, "ejSplitButton");
                    this._enableButtons(this._othersBtnIds, "ejButton");
                    break;
                case xlObj._getLocStr("Design"):
                    this._enableButtons(this._contextualBtnIds, "ejButton");
                    this._enableButtons(this._contextualCheckBoxIds, "ejCheckBox");
                    this._disableInputs(this._contextualInputIds, false);
                    break;

            }
            !xlObj.XLClipboard._copyBackup.cells && (xlObj.XLRibbon._disableButtons(["pastebtn"], "ejButton") || xlObj.XLRibbon._disableButtons(["pastesbtn"], "ejSplitButton"));
            xlObj._updateUndoRedoIcons();
        },

        _disableRibbonIcons: function () {
            if (!this.XLObj.model.showRibbon)
                return;
            var rObj = $("#" + this.XLObj._id + "_Ribbon").data("ejRibbon");
            switch (rObj.getTabText(rObj.model.selectedItemIndex)) {
                case "HOME":
                    this._disableButtons(this._homeBtnIds, "ejButton");
                    this._disableButtons(this._toggleBtnIds, "ejToggleButton");
                    this._disableButtons(this._ddIds, "ejDropDownList");
                    this._disableButtons(this._cpIds, "ejColorPicker");
                    this._disableButtons(this._splitBtnIds, "ejSplitButton");
                    this._disableButtons(this._homeSptBtnIds, "ejSplitButton");
                    break;
                case "INSERT":
                    this._disableButtons(this._insertBtnIds, "ejButton");
                    this._disableButtons(this._insertSptBtnIds, "ejSplitButton");
                    break;
                case "DATA":
                    this._disableButtons(this._dataBtnIds, "ejButton");
                    this._disableButtons(this._dataSptBtnIds, "ejSplitButton");
                    break;
                case "PAGE LAYOUT":
                    this._disableButtons(this._pageLaySptBtnIds, "ejSplitButton");
                    this._disableButtons(this._pageLayBtnIds, "ejButton");
                    this._disableButtons(this._pageLayChckBxIds, "ejCheckBox");
                    $("#" + rObj._id + "_pagesetup_viewhdrlbl").eq(0).addClass("e-disable");
                    $("#" + rObj._id + "_pagesetup_viewgrdlnlbl").eq(0).addClass("e-disable");
                    break;
                case "FORMULAS":
                    this._disableButtons(this._formulaBtnIds, "ejButton");
                    this._disableButtons(this._formulaSptBtnIds, "ejSplitButton");
                    break;
                case "REVIEW":
                    this._disableButtons(this._cmntBtnIds, "ejButton");
                    this._disableButtons(this._cmntTglBtnIds, "ejToggleButton");
                    break;
                case "OTHERS":
                    this._disableButtons(this._othersSptBtnIds, "ejSplitButton");
                    this._disableButtons(this._othersBtnIds, "ejButton");
                    break;
                case this.XLObj._getLocStr("Design"):
                    this._disableButtons(this._contextualBtnIds, "ejButton");
                    this._disableButtons(this._contextualCheckBoxIds, "ejCheckBox");
                    this._disableInputs(this._contextualInputIds, true);
                    break;
            }
        },

        _capitalize: function (s) {
            return s[0].toUpperCase() + s.slice(1);
        },

        _generateToggleBtn: function (id, text, contentType, prefixIcon, title, width, height, content, isSeparator, cssClass, imagePosition, actText) {
            return {
                id: id,
                toolTip: this.XLObj._getLocStr(text),
                type: ej.Ribbon.type.toggleButton,
                enableSeparator: isSeparator,
                toggleButtonSettings: {
                    width: width,
                    height: height,
                    contentType: contentType,
                    cssClass: cssClass,
                    defaultPrefixIcon: prefixIcon,
                    activePrefixIcon: prefixIcon,
                    defaultText: this.XLObj._getLocStr(text),
                    activeText: actText ? this.XLObj._getLocStr(actText) : this.XLObj._getLocStr(text),
                    imagePosition: imagePosition == "imagetop" ? ej.ImagePosition.ImageTop : ej.ImagePosition.ImageLeft,
                    click: $.proxy(this._ribbonClickHandler)
                },
                customToolTip:
                  {
                      title: this.XLObj._getLocStr(title),
                      content: "<h6>" + this.XLObj._getLocStr(content) + "</h6>"
                  }
            };
        },

        _generateBtn: function (id, text, contentType, prefixIcon, width, height, title, content, isSeparator, imagePosition, cssClass) {
            return {
                id: id,
                text: this.XLObj._getLocStr(text),
                toolTip: this.XLObj._getLocStr(text),
                cssClass: cssClass,
                enableSeparator: isSeparator,
                type: ej.Ribbon.type.button,
                buttonSettings: {
                    contentType: contentType,
                    width: width,
                    height: height,
                    imagePosition: imagePosition == "imagetop" ? ej.ImagePosition.ImageTop : ej.ImagePosition.ImageLeft,
                    prefixIcon: prefixIcon,
                    click: $.proxy(this._ribbonClickHandler)
                },
                customToolTip:
                  {
                      title: this.XLObj._getLocStr(title),
                      content: "<h6>" + this.XLObj._getLocStr(content) + "</h6>"
                  }
            };
        },

        _generateDD: function (id, text, data, value, width, height, title, content) {
            return {
                id: id,
                text: this.XLObj._getLocStr(text),
                toolTip: this.XLObj._getLocStr(text),
                type: ej.Ribbon.type.dropDownList,
                dropdownSettings: {
                    dataSource: data,
                    value: value,
                    popupHide: $.proxy(this._ribbonClickHandler),
                    width: width,
                    height: height - 1,
                    cssClass: "e-" + this.XLObj._id+"-ddl"
                },
                customToolTip:
                  {
                      title: this.XLObj._getLocStr(title),
                      content: "<h6>" + this.XLObj._getLocStr(content) + "</h6>"
                  }
            };
        },

        _generateCustomControl: function (contentId, toolTip, title, content) {
            return {
                type: ej.Ribbon.type.custom,
                contentID: contentId,
                toolTip: this.XLObj._getLocStr(toolTip),
                click: $.proxy(this._ribbonClickHandler),
                change: $.proxy(this._ribbonClickHandler),
                customToolTip:
                  {
                      title: this.XLObj._getLocStr(title),
                      content: "<h6>" + this.XLObj._getLocStr(content) + "</h6>"
                  }
            };
        },

        _generateSplitBtn: function (id, text, contentType, targetId, fields, prefixIcon, isSeparator, width, height, arrowPos, imagePosition, btnMode, cssClass, title, content) {
            return {
                id: id,
                toolTip: this.XLObj._getLocStr(text),
                text: this.XLObj._getLocStr(text),
                type: ej.Ribbon.type.splitButton,
                cssClass: cssClass,
                enableSeparator: isSeparator,
                splitButtonSettings: {
                    contentType: contentType,
                    targetID: targetId,
                    arrowPosition: arrowPos == "bottom" ? ej.ArrowPosition.Bottom : ej.ArrowPosition.Right,
                    imagePosition: imagePosition == "imagetop" ? ej.ImagePosition.ImageTop : ej.ImagePosition.ImageLeft,
                    buttonMode: btnMode == "dropdown" ? ej.ButtonMode.Dropdown : ej.ButtonMode.Split,
                    fields: fields,
                    prefixIcon: prefixIcon,
                    click: $.proxy(this._ribbonClickHandler),
                    itemSelected: $.proxy(this._ribbonClickHandler),
                    height: height,
                    width: width,
                    beforeOpen: $.proxy(this._splitBeforeOpen, this, id),
                    close: (id === "inssbtn" || id === "delsbtn" || id === "validationsbtn" || id === "pastesbtn") ? $.proxy(this._splitbtnClose, this, id) : null,
                },
                customToolTip:
                {
                    title: this.XLObj._getLocStr(title),
                    content: "<h6>" + this.XLObj._getLocStr(content) + "</h6>"
                }
            };
        },

        _splitBeforeOpen: function (id, args) {
            var xlObj = this.XLObj;
            if (id === "sortfilter")
                this._sfbeforeOpen(args);
            else if (id === "inssbtn" || id === "delsbtn")
                this._insdelbeforOpen(args);
            else if (id === "fPane")
                this._fpbeforeOpen(args);
            else if (id === "cElement")
                this._celementpbeforeOpen(args);
            else if (id === "validationsbtn" || id === "pastesbtn")
                this._validpasteOpen(args);
            else if (id === "clear")
                this._clearbeforeOpen(args);
        },

        _splitbtnClose: function (id, args) {
            this.XLObj.element.find("#" + this.XLObj._id + "_Ribbon_" + id).parents("div.e-controlpadding").siblings().children("button").removeClass("e-active");
        },

        _fpbeforeOpen: function (args) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), fpaneMenu = $("#" + xlObj._id + "_Ribbon_FPane").data("ejMenu"), dataSource = fpaneMenu.model.fields.dataSource.slice(0);
            if (xlObj.getFrozenRows(sheetIdx) > 0 || xlObj.getFrozenColumns(sheetIdx) > 0) {
                dataSource[0].id = "unfreezePanes";
                dataSource[0].text = xlObj._getLocStr("unfreezePanes");
            }
            else {
                dataSource[0].id = "freezePanes";
                dataSource[0].text = xlObj._getLocStr("freezePanes");
            }
            fpaneMenu.option("fields", { id: "id", dataSource: dataSource, parentId: "parentId" });
        },

        _sfbeforeOpen: function (args) {
            var xlObj = this.XLObj, menuObj = $("#" + xlObj._id + "_Ribbon_SortFilter").data("ejMenu");
            (!xlObj.model.allowFiltering) && menuObj.disableItemByID("Filter");
            if (!xlObj.model.allowSorting) {
                menuObj.disableItemByID("SortAtoZ");
                menuObj.disableItemByID("SortZtoA");
            }
            this._changeClrFltrStatus(menuObj, "menu", xlObj.getActiveCell());
        },

        _validpasteOpen: function (args) {
            var xlObj = this.XLObj;
            if (args.model.targetID === xlObj._id + "_Ribbon_Validation")
                xlObj.element.find("#" + xlObj._id + "_Ribbon_validationbtn").addClass("e-active");
            else
                xlObj.element.find("#" + xlObj._id + "_Ribbon_pastebtn").addClass("e-active");
        },

        _insdelbeforOpen: function (args) {
            var xlObj = this.XLObj, insMenuObj = $("#" + xlObj._id + "_Ribbon_Ins").data("ejMenu"), delMenuObj = $("#" + xlObj._id + "_Ribbon_Del").data("ejMenu"), colSel, rowSel;
            if (args.model.targetID === xlObj._id + "_Ribbon_Ins")
                xlObj.element.find("#" + xlObj._id + "_Ribbon_insbtn").addClass("e-active");
            else
                xlObj.element.find("#" + xlObj._id + "_Ribbon_delbtn").addClass("e-active");
            if (xlObj.element.find(".e-colselected").length) {
                insMenuObj.disableItemByID("InsertSheetRows");
                delMenuObj.disableItemByID("DeleteSheetRows");
            }
            else {
                insMenuObj.enableItemByID("InsertSheetRows");
                delMenuObj.enableItemByID("DeleteSheetRows");
            }
            if (xlObj.element.find(".e-rowselected").length) {
                insMenuObj.disableItemByID("InsertSheetColumns");
                delMenuObj.disableItemByID("DeleteSheetColumns");
            }
            else {
                insMenuObj.enableItemByID("InsertSheetColumns");
                delMenuObj.enableItemByID("DeleteSheetColumns");
            }
        },

        _celementpbeforeOpen: function (args) {
            var xlObj = this.XLObj; chartEle = $("#" + xlObj._id + "_chart").val(), chType = xlObj.XLChart._getShapeObj(chartEle, "chart").type,
            cEleMenuObj = $("#" + xlObj._id + "_Ribbon_CElement").data("ejMenu"), pieChart = ["Axes", "AxisTitle", "Gridline"],
            radarChart = ["AxisTitle", "PMinorH", "PMinorV"], menuEle = ["Axes", "AxisTitle", "Gridline", "PMinorH", "PMinorV"], i, len = 0;
            for (i = 0, len = menuEle.length; i < len; i++)
                cEleMenuObj.enableItemByID(menuEle[i]);
            if (chType === "pie" || chType === "doughnut") {
                for (i = 0, len = pieChart.length; i < len; i++)
                    cEleMenuObj.disableItemByID(pieChart[i]);
            }
            else if (chType === "radar") {
                for (i = 0, len = radarChart.length; i < len; i++)
                    cEleMenuObj.disableItemByID(radarChart[i]);
            }
        },

        _clearbeforeOpen: function (args) {
            var xlObj = this.XLObj, i = 0, clearMenuObj = $("#" + xlObj._id + "_Ribbon_Clear").data("ejMenu"), eleId = ["Clear_Hyperlinks", "Clear_Comments"];
            if (xlObj.model.allowLockCell) {
                if (xlObj.getSheet(xlObj.getActiveSheetIndex())._isSheetProtected) {
                    while (i < eleId.length) {
                        clearMenuObj.disableItemByID(eleId[i]);
                        i++;
                    }
                }
                else {
                    while (i < eleId.length) {
                        clearMenuObj.enableItemByID(eleId[i]);
                        i++;
                    }
                }
            }

        },

        _changeClrFltrStatus: function (obj, control, target) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), nfat = false, i, j, filterColl = xlObj.getSheet(sheetIdx)._filterColl;
            (control === "menu") ? obj.disableItemByID("ClearFilter") : obj.disable();
            //(target.className.indexOf("e-table") < 0) && (nfat = true);
            var rowKeys = xlObj.getObjectKeys(filterColl), colKeys, cell;
            for (i = rowKeys.length - 1; i >= 0; i--) {
                colKeys = xlObj.getObjectKeys(filterColl[i]);
                for (j = colKeys.length - 1; j >= 0; j--) {
                    if (filterColl[rowKeys[i]][colKeys[j]].status.indexOf("filtered")) {
                        if (nfat) {
                            if (!filterColl[rowKeys[i]][colKeys[j]].isTable) {
                                (control === "menu") ? obj.enableItemByID("ClearFilter") : obj.enable();
                                break;
                            }
                        }
                        else {
                            if (!filterColl[rowKeys[i]][colKeys[j]].isTable) {
                                (control === "menu") ? obj.enableItemByID("ClearFilter") : obj.enable();
                                break;
                            }
                        }
                    }
                }
            }
        },

        _onControlCreated: function () {
            var ht, wt, xlObj = this.XLObj, xlId = xlObj._id, bsBtnOpt = { height: 154, width: 163, text: xlObj._getLocStr("BlankWorkbook"), contentType: "textandimage", imagePosition: "imagetop", prefixIcon: "e-ss-blank", click: $.proxy(this._bSBtnClickHandler) };
            ht = (xlObj.isNumber(xlObj.model.scrollSettings.height) ? xlObj.model.scrollSettings.height : xlObj.element.height()) - (($("#" + xlId + "_Ribbon").height() - xlObj.element.find(".e-ribbon .e-content").eq(0).height()) + xlObj._formulaHeight) - 40;
            wt = xlObj.isNumber(xlObj.model.scrollSettings.width) ? (xlObj.model.scrollSettings.width - 40) : (xlObj.element.width() - 40);
            // backstage 
            $("#" + xlId + "_bstab_new_blank").ejButton(bsBtnOpt);
            delete bsBtnOpt.text;
            bsBtnOpt.height = bsBtnOpt.width = 72;
            bsBtnOpt.prefixIcon = "e-ss-exportxl";
            $("#" + xlId + "_bstab_saveas_excel").ejButton(bsBtnOpt);
            bsBtnOpt.prefixIcon = "e-ss-exportcsv";
            $("#" + xlId + "_bstab_saveas_csv").ejButton(bsBtnOpt);
            bsBtnOpt.prefixIcon = "e-ss-print";
            $("#" + xlId + "_bstab_print_sheet").ejButton(bsBtnOpt);
            bsBtnOpt.prefixIcon = "e-ss-printselected";
            $("#" + xlId + "_bstab_print_selected").ejButton(bsBtnOpt);
            this._hideRibbonElem();
            $("#" + xlId + "_Ribbon_background-color").ejColorPicker({ value: "#FFFF00", modelType: "palette", showSwitcher: false, cssClass: "e-ss-colorpicker", toolIcon: "e-ss-backgroundcolor", open: $.proxy(this._colorPickerHandler, this, "Ribbon_background-color"), change: $.proxy(this._colorPickerHandler, this, "Ribbon_background-color"), select: $.proxy(this._cpClickHandler, this, "Ribbon_background-color") });
            $("#" + xlId + "_Ribbon_color").ejColorPicker({ value: "#FF0000", modelType: "palette", showSwitcher: false, cssClass: "e-ss-colorpicker", toolIcon: "e-ss-fontcolor", open: $.proxy(this._colorPickerHandler, this, "Ribbon_color"), change: $.proxy(this._colorPickerHandler, this, "Ribbon_color"), select: $.proxy(this._cpClickHandler, this, "Ribbon_color") });
            $("#" + xlId + "_Ribbon_BorderColor").ejColorPicker({ modelType: "palette", presetType: "basic", cssClass: "e-ss-colorpicker e-ss-menuclrpkr", change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_Ribbon_BorderColorWrapper").hide();
            $("#" + xlId + "_Ribbon_BorderColor_popup").css({ "padding-left": "10px", "width": "197px", "display": "block" });
            $("#" + xlId + "_Ribbon_BorderCP").ejColorPicker({ value: "#278787", displayInline: true, modelType: "picker" });
            $("#" + xlId + "_Ribbon_BorderCP_Presets").hide();
            $("#" + xlId + "_Ribbon_Border").width(200);
            $("#" + xlId + "_CFormat").width(200);
            $("#" + xlId + "_Ribbon_Border").addClass("e-spreadsheet e-border");
            $("#" + xlId + "_Ribbon_color_Presets").css("width", "119px");
            $("#" + xlId + "_Ribbon_background-color_Presets").css("width", "119px");
            $("#" + xlId + "_Ribbon_pagesetupleft").show();
            $("#" + xlId + "_Ribbon_Vheading").ejCheckBox({ change: $.proxy(this._ribbonClickHandler), checked: true });
            $("#" + xlId + "_Ribbon_Vgridlines").ejCheckBox({ change: $.proxy(this._ribbonClickHandler), checked: true });
            $("#" + xlId + "_Ribbon_tsofirstcolumn").ejCheckBox({ change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_Ribbon_tsolastcolumn").ejCheckBox({ change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_Ribbon_tsototalrow").ejCheckBox({ change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_Ribbon_tsofiltercolumn").ejCheckBox({ change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_Ribbon_chartwidth").ejNumericTextbox({ name: "numeric", value: 5, minValue: 0, maxValue: wt, decimalPlaces: 1, incrementStep: 5, change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_Ribbon_chartheight").ejNumericTextbox({ name: "numeric", value: 5, minValue: 0, maxValue: ht, decimalPlaces: 1, incrementStep: 5, change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_pictureheight").ejNumericTextbox({ name: "numeric", value: 100, minValue: 25, incrementStep: 5, change: $.proxy(this._pictureSizeChange, this, "picturewidth") });
            $("#" + xlId + "_picturewidth").ejNumericTextbox({ name: "numeric", value: 100, minValue: 25, incrementStep: 5, change: $.proxy(this._pictureSizeChange, this, "pictureheight") });
            $("#" + xlId + "_Ribbon_PictureColor").ejColorPicker({ modelType: "palette", presetType: "basic", cssClass: "e-ss-colorpicker e-ss-menuclrpkr", change: $.proxy(this._ribbonClickHandler) });
            $("#" + xlId + "_Ribbon_PictureColorWrapper").hide();
            $("#" + xlId + "_Ribbon_PictureColor_popup").css({ "padding-left": "10px", "width": "197px", "display": "block" });
            if (!xlObj.model.allowInsert && xlObj.model.scrollSettings.allowScrolling) {
                this._disableButtons(["inssbtn"], "ejSplitButton");
                this._disableButtons(["insbtn"], "ejButton");
            }
            if (!xlObj.model.allowDelete && xlObj.model.scrollSettings.allowScrolling) {
                this._disableButtons(["delsbtn"], "ejSplitButton");
                this._disableButtons(["delbtn"], "ejButton");
            }
            $("#" + xlId + "_Ribbon_Validation").width(160);
            this._disableButtons(["pastebtn"], "ejButton");
            this._disableButtons(["pastesbtn", "useinformula"], "ejSplitButton");
            $("#" + xlId + "_Ribbon_background-color_popup").find(".e-buttons, .e-button").hide();
            $("#" + xlId + "_Ribbon_color_popup").find(".e-buttons, .e-button").hide();
        },

        _hideRibbonElem: function () {
            var xlObj = this.XLObj, rObj = $("#" + xlObj._id + "_Ribbon").ejRibbon("instance"), model = xlObj.model, rObjElem = rObj.element;		
			if(!model.allowAutoSum && !model.allowFiltering && !model.allowUndoRedo && !model.allowSorting && !model.allowClipboard && !model.allowFormatPainter && !model.allowCellFormatting && !model.allowMerging && !model.allowWrap && !model.allowConditionalFormats && !model.allowFormatAsTable)
                rObj.hideTab("HOME");
			if(!model.allowFormatAsTable && !model.allowPictures && !model.allowHyperlink && !model.allowCharts)
                rObj.hideTab("INSERT");
			if(!model.allowDataValidation && !model.allowFiltering && !model.allowSorting)
                rObj.hideTab("DATA");
			if(!model.allowPrinting)
				rObj.hideTab("PAGELAYOUT");
			if(!model.allowComments)
                rObj.hideTab("REVIEW");
			if(!model.allowFreezing && !model.allowInsert && !model.allowDelete && !model.allowSearching)
                rObj.hideTab("OTHERS");
            if (!model.allowClipboard && !model.allowFormatPainter)
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Clipboard").hide();
            else if (!model.allowFormatPainter)
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Clipboard_formatpainter").hide();
            else if (!model.allowClipboard) {
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Clipboard_1").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Clipboard_cut").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Clipboard_copy").hide();
            }
            if (!model.allowCellFormatting) {
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Font").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Number").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_top").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_middle").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_bottom").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_separator_bottom").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_left").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_center").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_right").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_separator_right").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Styles_cellstyles").hide();
                if (!model.allowConditionalFormats && !model.allowFormatAsTable)
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Styles").hide();
            }
            if (model.allowCellFormatting) {
                if (!model.formatSettings.allowFontFamily)
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Font_font-family").hide();
                if (!model.formatSettings.allowDecimalPlaces) {
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Number_increment").hide();
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Number_decrement").hide();
                    rObjElem.find("#" + xlObj._id + "_Ribbon_separator_usegrouping").hide();
                }
                if (!model.formatSettings.allowCellBorder) {
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Font_border").hide();
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Font_border").hide();
                }
            }
            if (!model.allowMerging)
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_merge").hide();
            if (!model.allowWrap)
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Alignment_wraptext").hide();
            if (!model.allowUndoRedo)
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Actions").hide();
            if (!model.allowPivotTable && !model.allowFormatAsTable)
                rObjElem.find("#" + xlObj._id + "_Ribbon_insert_Tables").hide();
            else if (!model.allowPivotTable)
                rObjElem.find("#" + xlObj._id + "_Ribbon_insert_Tables_pivotTable").hide();
            else if (!model.allowFormatAsTable) {
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Styles_formatastable").hide();
                rObjElem.find("#" + xlObj._id + "_Ribbon_insert_Tables_table").hide();
            }
            if (!model.allowConditionalFormats && !model.allowFormatAsTable && !model.allowCellFormatting)
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Styles").hide();
            if (!model.allowConditionalFormats)
                rObjElem.find("#" + xlObj._id + "_Ribbon_home_Styles_cFormat").hide();
            if (model.allowEditing) {
                if (!model.allowAutoSum)
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Editing_autosum").hide();
                if (!model.allowSorting && !model.allowFiltering)
                    rObjElem.find("#" + xlObj._id + "_Ribbon_home_Editing_1").hide();
            }
            if (!model.pictureSettings.allowPictures)
                rObjElem.find("#" + xlObj._id + "_Ribbon_insert_Illustrations").hide();
            if (!model.allowHyperlink)
                rObjElem.find("#" + xlObj._id + "_Ribbon_insert_Links").hide();
            if (!model.allowCharts)
                rObjElem.find("#" + xlObj._id + "_Ribbon_insert_Charts").hide();
            if (!model.allowFiltering)
                rObjElem.find("#" + xlObj._id + "_Ribbon_data_SortFilter_2").hide();
            if (!model.allowSorting)
                rObjElem.find("#" + xlObj._id + "_Ribbon_data_SortFilter_1").hide();
            if (!model.allowDataValidation)
                rObjElem.find("#" + xlObj._id + "_Ribbon_data_DataTools").hide();
            if (!model.printSettings.allowPrinting) {
                rObjElem.find("#" + xlObj._id + "_Ribbon_pagelayout_Print").hide();
                rObjElem.find("#print_backStageTab").hide();
            }
            if (!model.printSettings.allowPageSize)
                rObjElem.find("#" + xlObj._id + "_Ribbon_pagelayout_PageLayout").hide();
            if (!model.allowLockCell)
            {
                rObjElem.find(".e-ssr-protectsheetbtn").hide();
                rObjElem.find(".e-ssr-lockbtn").hide();
            }
            if (!model.allowComments && !model.allowLockCell)
                rObj.hideTab("REVIEW");
            if (!model.allowComments)
                rObjElem.find("#" + xlObj._id + "_Ribbon_review_Comments").hide();
            if (!model.allowFreezing)
                rObjElem.find("#" + xlObj._id + "_Ribbon_others_Window").hide();
            if (!model.allowFormulaBar) {
                rObjElem.find("#" + xlObj._id + "_Ribbon_others_Formulas").hide();
                xlObj.hideFormulaBar();
            }
            if (!model.allowInsert && !model.allowDelete)
                rObjElem.find("#" + xlObj._id + "_Ribbon_others_Cells").hide();
            else if (!model.allowInsert)
                rObjElem.find("#" + xlObj._id + "_Ribbon_others_Cells_1").hide();
            else if (!model.allowDelete)
                rObjElem.find("#" + xlObj._id + "_Ribbon_others_Cells_2").hide();
            if (!model.allowFiltering && !model.allowSorting)
                rObjElem.find("#" + xlObj._id + "_Ribbon_data_Sort").hide();
            if (!model.allowSearching)
                rObjElem.find("#" + xlObj._id + "_Ribbon_others_Editing").hide();
            if (!model.allowFormatAsTable && !model.allowPivotTable)
                rObjElem.find("#" + xlObj._id + "_Ribbon_insert_Tables").hide();
            if (!model.allowImport)
                rObjElem.find("#open_backStageBtn").hide();
            if (!model.exportSettings.allowExporting)
                rObjElem.find("#saveas_backStageTab").hide();
        },

        _pictureSizeChange: function (name, args) {
            var xlObj = this.XLObj, newVal, elem = xlObj.element.find(".e-ss-activeimg"), details = { sheetIndex: xlObj.getActiveSheetIndex(), reqType: "shape", shapeType: "picture", action: "picturesize", name: name, id: $(elem).attr("id"), prev: {}, cur: {} };
            if (xlObj._shapeChange || xlObj.XLRibbon._isSetModel)
                return;
            details.prev = { width: elem.width(), height: elem.height() };
            if (name === "pictureheight") {
                newVal = parseInt((elem.height() * args.value) / elem.width());
                elem.css({ width: args.value, height: newVal });
                details.cur = { width: args.value, height: newVal };
            }
            else {
                newVal = parseInt((elem.width() * args.value) / elem.height());
                elem.css({ height: args.value, width: newVal });
                details.cur = { height: args.value, width: newVal };
            }
            xlObj.XLRibbon._isSetModel = true;
            $("#" + xlObj._id + "_" + name).ejNumericTextbox("option", { value: newVal });
            xlObj.XLRibbon._isSetModel = false;
            xlObj._completeAction(details);
            xlObj.XLShape._updateShapeObj(elem[0]);
        },

        _chartTypeOk: function (args) {
            var xlObj = this.XLObj, id = xlObj.element.find("#" + xlObj._id + "_chart").val(), chartObj = $("#" + id).data('ejChart'), details, chartType, className, allChart, markerChart, option;
            className = $("#" + xlObj._id + "_allchart").find(".e-chartselect")[0].classList[1], chartType = className.split("e-dlg")[1].split("chart");
            allChart = ["e-dlgcolumnchart4", "e-dlgcolumnchart5", "e-dlgcolumnchart6", "e-dlgbarchart4", "e-dlgbarchart5", "e-dlgbarchart6", "e-dlgpiechart2"];
            markerChart = ["e-dlgstockchart2", "e-dlglinechart2", "dlgradarchart2"], option = { type: chartType[0], enable3D: false, marker: { visible: false } };
            if (chartType[0] === "column" || chartType[0] === "bar" || chartType[0] === "area") {
                if (chartType[1] === "1" || chartType[1] === "4")
                    option["type"] = chartType[0];
                else if (chartType[1] === "2" || chartType[1] === "5")
                    option["type"] = "stacking" + chartType[0];
                else if (chartType[1] === "3" || chartType[1] === "6")
                    option["type"] = "stacking" + chartType[0] + "100";
            }
            allChart.indexOf(className) > -1 && (option["enable3D"] = true);
            markerChart.indexOf(className) > -1 && (option["marker"] = { visible: true });
            className === "e-dlgpiechart3" && (option["type"] = "doughnut");
            details = { sheetIndex: xlObj.getActiveSheetIndex(), reqType: "shape", shapeType: "chart", action: "chartType", chartId: id };
            details.prev = { chartType: chartObj.model.series[0].type, enable3D: chartObj.model.enable3D, marker: chartObj.model.series[0].marker.visible };
            xlObj.XLChart.refreshChart(id, option);
            details.cur = { chartType: chartObj.model.series[0].type, enable3D: chartObj.model.enable3D, marker: chartObj.model.series[0].marker.visible };
            $("#" + xlObj._id + "_charttypedlg").ejDialog("close");
            xlObj._completeAction(details);

        },
        _chartTypeCancel: function (args) {
            var xlObj = this.XLObj;
            $("#" + xlObj._id + "_charttypedlg").ejDialog("close");
            xlObj.setSheetFocus();
        },

        _chartType: function (chart, len, cnt, typecnt, type, title) {
            var parent, i, j = 0, xlObj = this.XLObj, text = xlObj._getLocStr(chart).split("/"), chartIcon = chart.toLowerCase(), elem;
            parent = ej.buildTag("div#" + xlObj._id + "_Ribbon_" + chart + ".e-ss-dialog", "", { width: "auto", padding: "4px" });
            parent.append("<div class='e-chartheader'><span>" + text[0] + "</span></div>");
            elem = ej.buildTag("div.e-chartcontent");
            for (i = 0; i < len; i++) {
                elem.append("<div class='e-chartcell e-" + chartIcon + (i + 1) + "' title=" + xlObj._getLocStr(title[i]) + "></div>");
                $(elem).children().eq(i).data({ type: type[i], enable3D: false, marker: { visible: false } });
                if (i === cnt) {
                    parent.append(elem).append("<div class='e-chartheader'><span>" + text[1] + "</span></div>");
                    elem = ej.buildTag("div.e-chartcontent");
                }
                if (i > typecnt) {
                    $(elem).children().eq(j).data({ type: type[i], enable3D: true, marker: { visible: false } });
                    j++;
                }
            }
            parent.append(elem);
            elem = $(parent).find(".e-" + chartIcon + "2");
            (chart === "StockChart" || chart === "LineChart") && elem.data({ marker: { visible: true } });
            (chart === "PieChart") && elem.data({ enable3D: true });
            xlObj.element.append(parent);
        },

        autoSum: function (type, range) {
            if (!this.XLObj.model.allowAutoSum)
                return;
            var xlObj = this.XLObj, xlEdit = xlObj.XLEdit, sheetIdx = xlObj.getActiveSheetIndex(), elem, cellIdx = xlObj.getActiveCell(sheetIdx), sCell = false;
			if(ej.isNullOrUndefined(cellIdx))
                return;
            var i, inputElem, colLen, actCell = xlObj.getActiveCell(sheetIdx);
            var alphRange, rangeIdx = xlObj.getRangeIndices(range), selectCell = xlObj.XLSelection._getSelectedRange({ "rowIndex": rangeIdx[0], "colIndex": rangeIdx[1] }, { "rowIndex": rangeIdx[2], "colIndex": rangeIdx[3] }), cellCnt = selectCell.length - 1;
            var rowCnt = selectCell[cellCnt].rowIndex - selectCell[0].rowIndex, colCnt = selectCell[cellCnt].colIndex - selectCell[0].colIndex, details = { sheetIndex: sheetIdx, range: range, reqType: "auto-sum", oprType: type, updCell: [] };
            var rowIdx, colIdx, cell, textLength = xlObj.getRange(selectCell[0].rowIndex, selectCell[0].colIndex, selectCell[cellCnt].rowIndex, selectCell[cellCnt].colIndex).text().length, container = xlObj._dataContainer.sheets[sheetIdx];
            if (cellCnt === 0 || !textLength) {
                rowIdx = cellIdx.rowIndex; colIdx = cellIdx.colIndex;
                details.updCell.push({ rowIndex: rowIdx, colIndex: colIdx, pObj: {}, pValue: xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2") });
                (ej.isNullOrUndefined(container[rowIdx])) && (container[rowIdx] = {});
                (ej.isNullOrUndefined(container[rowIdx][colIdx])) && (container[rowIdx][colIdx] = {});
                $.extend(true, details.updCell[0].pObj, container[rowIdx][colIdx]);
                xlEdit.editCell(actCell.rowIndex, actCell.colIndex, true);
                xlObj._getInputBox().val("=" + type + "()");
                if (xlObj._isRowViewable(sheetIdx, rowIdx)) {
                    inputElem = xlObj.getCell(rowIdx, colIdx).find("input");
                    inputElem.val("=" + type + "()");
                    inputElem.focus().setInputPos(inputElem.val().length - 1);
                }
                sCell = true;
            }
            else if (rowCnt === 0 && colCnt > 0) {
                alphRange = xlObj._getAlphaRange(sheetIdx, selectCell[0].rowIndex, selectCell[0].colIndex, selectCell[cellCnt].rowIndex, selectCell[cellCnt].colIndex);
                rowIdx = selectCell[cellCnt].rowIndex; colIdx = selectCell[cellCnt].colIndex + 1;
                if (colIdx === xlObj.getSheet(sheetIdx).colCount)
                    xlObj.XLScroll._createNewColumn(sheetIdx, { rowIndex: -1, colIndex: -1 }, { rowIndex: -1, colIndex: -1 }, "insert");
                details.updCell.push({ rowIndex: rowIdx, colIndex: colIdx, pObj: {}, pValue: xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2") });
                (ej.isNullOrUndefined(container[rowIdx])) && (container[rowIdx] = {});
                (ej.isNullOrUndefined(container[rowIdx][colIdx])) && (container[rowIdx][colIdx] = {});
                $.extend(true, details.updCell[0].pObj, container[rowIdx][colIdx]);
                xlEdit.updateCellValue(cell, "=" + type + "(" + alphRange + ")");
                if (xlObj._isRowViewable(sheetIdx, rowIdx)) {
                    cell = xlObj.getCell(rowIdx, colIdx, sheetIdx);
                    cell.addClass("e-ralign");
                }
            }
            else {
                colLen = selectCell[cellCnt].colIndex - selectCell[0].colIndex;
                for (i = 0; i <= colLen; i++) {
                    alphRange = xlObj._getAlphaRange(sheetIdx, selectCell[0].rowIndex, selectCell[i].colIndex, selectCell[cellCnt].rowIndex, selectCell[i].colIndex);
                    rowIdx = selectCell[cellCnt].rowIndex + 1; colIdx = selectCell[i].colIndex;
                    if (selectCell[cellCnt].rowIndex === xlObj.getSheet(sheetIdx).rowCount - 1)
                        xlObj.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                    details.updCell.push({ rowIndex: rowIdx, colIndex: colIdx, pObj: {}, pValue: xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2") });
                    (ej.isNullOrUndefined(container[rowIdx])) && (container[rowIdx] = {});
                    (ej.isNullOrUndefined(container[rowIdx][colIdx])) && (container[rowIdx][colIdx] = {});
                    $.extend(true, details.updCell[i].pObj, container[rowIdx][colIdx]);
                    xlEdit.updateCellValue(cell, "=" + type + "(" + alphRange + ")");
                    if (xlObj._isRowViewable(sheetIdx, rowIdx)) {
                        cell = xlObj.getCell(rowIdx, colIdx, sheetIdx);
                        cell.addClass("e-ralign");
                    }
                }
            }
            if (!xlObj._isUndoRedo && !sCell)
                xlObj._completeAction(details);
        },

        _openNameManagerDlg: function () {
            var xlObj = this.XLObj, grid = $("#" + xlObj._id + "_nmgrid").data("ejGrid");
            grid.getContentTable().find('td').addClass("e-ss-emptyrecord")
            if (grid.model.dataSource.length < 1)
                grid.getContentTable().find('td').addClass("e-ss-emptyrecord");
            else
                grid.getContentTable().find('.e-ss-emptyrecord').removeClass("e-ss-emptyrecord");
            $("#" + xlObj._id + "_nmdlg").ejDialog("open");
            $("#" + xlObj._id + "_nmgrid").focus();
        },

        _openFATDlg: function () {
            var xlObj = this.XLObj;
            $('#' + xlObj._id + '_fatnamedlg').ejDialog('open');
            $("#" + xlObj._id + "_fatheader").ejCheckBox("option", "checked", false);
            $("#" + xlObj._id + "_fatname").val('Table ' + xlObj.getObjectLength(xlObj.getSheet(xlObj.getActiveSheetIndex())._tableManager)).focus().setInputPos($("#" + xlObj._id + "_fatname").val().length);
        },

        _dirtySelect: function (cells) {
            var len, filterIcon, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), menuObj = $("#" + xlObj._id + "_contextMenuCell").data("ejMenu"), splitBtnIds = ["sortfilter", "clear"], fatSplitBtn = ["formatastable"], btnIds = ["convertorange", "filter", "clearfilter"], tableBtn = ["table"], activeEle = xlObj.getActiveCellElem(sheetIdx);
            var colMenuObj = $("#" + xlObj._id + "_contextMenuColumnHeader").data("ejMenu"), rowMenuObj = $("#" + xlObj._id + "_contextMenuRowHeader").data("ejMenu");
            var enColIdx, enRowIdx, isDirtySelect = false, tabRange = xlObj.getSheet(sheetIdx).filterSettings.tableRange;
            if (xlObj.model.allowLockCell && xlObj.getSheet(sheetIdx)._isSheetProtected)
                return;
            if (cells.length > 0) {
                enRowIdx = $(cells[cells.length - 1]).parent().index();
                enColIdx = cells[cells.length - 1].cellIndex;
                for (var i = 0, len = tabRange.length; i < len; i++) {
                    if (tabRange[i].endRow > enRowIdx) {
                        isDirtySelect = true;
                        break;
                    }
                    if (tabRange[i].multifilterIdx[tabRange[i].multifilterIdx.length - 1] + 1 > enColIdx) {
                        isDirtySelect = true;
                        break;
                    }
                }
            }
            if (isDirtySelect) {
                if (xlObj.model.allowInsert && xlObj.model.scrollSettings.allowScrolling) {
                    menuObj.disableItemByID("Insert");
                    rowMenuObj.disableItemByID("Insert");
                    colMenuObj.disableItemByID("Insert");
                    this._disableButtons(["inssbtn"], "ejSplitButton");
                    this._disableButtons(["insbtn"], "ejButton");
                }
                if (xlObj.model.allowDelete && xlObj.model.scrollSettings.allowScrolling) {
                    this._disableButtons(["delsbtn"], "ejSplitButton");
                    this._disableButtons(["delbtn"], "ejButton");
                    menuObj.disableItemByID("Delete");
                    rowMenuObj.disableItemByID("Delete");
                    colMenuObj.disableItemByID("Delete");
                }
            }
            else {
                if (xlObj.model.allowInsert && xlObj.model.scrollSettings.allowScrolling) {
                    menuObj.enableItemByID("Insert");
                    rowMenuObj.enableItemByID("Insert");
                    colMenuObj.enableItemByID("Insert");
                    this._enableButtons(["inssbtn"], "ejSplitButton");
                    this._enableButtons(["insbtn"], "ejButton");
                }
                if (xlObj.model.allowDelete && xlObj.model.scrollSettings.allowScrolling) {
                    menuObj.enableItemByID("Delete");
                    rowMenuObj.enableItemByID("Delete");
                    colMenuObj.enableItemByID("Delete");
                    this._enableButtons(["delsbtn"], "ejSplitButton");
                    this._enableButtons(["delbtn"], "ejButton");
                }
            }
            if (cells.not("[class*='e-table']").length > 0 && cells.is("[class*='e-table']")) {
                this._disableButtons(splitBtnIds, "ejSplitButton");
                !xlObj._hasClass(activeEle[0], "e-table") && this._disableButtons(fatSplitBtn, "ejSplitButton");
                this._disableButtons(btnIds, "ejButton");
                menuObj.disableItemByID("Sort");
            }
            else {
                this._enableButtons(splitBtnIds, "ejSplitButton");
                this._enableButtons(fatSplitBtn, "ejSplitButton")
                this._enableButtons(btnIds, "ejButton");
                xlObj.model.allowSorting && menuObj.enableItemByID("Sort");
            }
            if (cells.is("[class*='e-table']")) {
                this._disableButtons(tableBtn, "ejButton");
                this._isDirtySelect = true;
            }
            else {
                this._enableButtons(tableBtn, "ejButton");
                this._isDirtySelect = false;
            }
            filterIcon = xlObj._getContent(sheetIdx).find(".e-ssfiltered, .e-ssfiltered-asc, .e-ssfiltered-dsc"), len = filterIcon.length;
            if (!cells.not("[class*='e-table']").length < 1 && !cells.is("[class*='e-table']")) {
                if (len > 0) {
                    for (var i = 0; i < len; i++)
                        if (filterIcon[i].parentNode.className.indexOf("e-table") > -1) {
                            if (cells.not("[class*='e-table']").length < 1 && cells.is("[class*='e-table']"))
                                this._enableButtons(["clearfilter"], "ejButton");
                            else
                                this._disableButtons(["clearfilter"], "ejButton");
                        }
                        else {
                            this._enableButtons(["clearfilter"], "ejButton");
                            if (cells.has("[class*='e-table']").length < 1)
                                return;
                        }
                }
                else
                    this._disableButtons(["clearfilter"], "ejButton");
            }
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.freezepane = function (obj) {
        this.XLObj = obj;
        this._isFirstRow = false;
        this._isFirstColumn = false;
    };

    ej.spreadsheetFeatures.freezepane.prototype = {
        // Frozen Rows and Columns
        freezeTopRow: function () {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());
            if (!xlObj.model.allowFreezing)
                return;		
            this._getFirstFreezeRow();
            this._isFirstRow = true;
            if(sheet._firstRow.idx === 0)
                sheet._firstRow = {idx: 1, value: sheet._rowHeightCollection[1]};
			this.freezeRows(sheet._firstRow.idx);
        },

        _getFirstFreezeRow: function () {
			this._updateRowHeightCollection(0);
			var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), rowHtColl = sheet._rHeightCollection, top = xlObj._getContent(sheetIdx).find(".e-content").scrollTop(), rHtColl = sheet.rowsHeightCollection;
			if(top === 0)
				 sheet._firstRow = { idx: 1, value: rowHtColl[1].value };
			else{
				for (var i = 0, len = rowHtColl.length; i < len; i++) {
					if (rowHtColl[i].value >= top) {
						sheet._firstRow = { idx: i + 1, value: rowHtColl[i + 1].value };
						break;
					}
				}
			}
			if (sheet._firstRow.idx !== 0)
			    this._updateRowHeightCollection(sheet._firstRow.idx);
        },

        freezeLeftColumn: function () {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex());
            if (!xlObj.model.allowFreezing)
                return;
            this._getFirstFreezeColumn();
            this._isFirstColumn = true;
            if(sheet._firstCol.idx === 0)
                sheet._firstCol = {idx: 1, value: sheet._colWidthCollection[1]};
            this.freezeColumns(sheet._firstCol.idx);
        },

        _getFirstFreezeColumn: function () {
			this._updateColWidthCollection(0);
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), colWtColl = sheet._cWidthCollection, left = xlObj._getContent(sheetIdx).find(".e-content").scrollLeft();
			if(left === 0)
				sheet._firstCol = { idx: 1, value: colWtColl[1].value };
			else{
				for (var i = 0, len = colWtColl.length; i < len; i++) {
					if (colWtColl[i].value >= left) {
						sheet._firstCol = { idx: i + 1, value: colWtColl[i + 1].value };
						break;
					}
				}
			}
			if (sheet._firstCol.idx !== 0)
			    this._updateColWidthCollection(sheet._firstCol.idx);
        },

        freezeRows: function (freezerow) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowFreezing)
                return;
            if (!this._isFirstRow)
                this._getFirstFreezeRow();
            var sheetIdx = xlObj.getActiveSheetIndex(), actSheet = xlObj.getSheet(sheetIdx), fRow = xlObj.model.sheets[sheetIdx]._firstRow,
				rowColl = xlObj.getRows(sheetIdx), rows = rowColl[1], vScroll = xlObj.XLScroll._vScroller(sheetIdx);
            if (xlObj.getFrozenRows(sheetIdx) > 0 || xlObj.getFrozenColumns(sheetIdx) > 0)
                this.unfreezePanes();
            actSheet._frozenHeight = 0;
            vScroll.element.find(".e-vhandle").css("top", 0);
            vScroll.value(0);
            if (freezerow > 0) {
                xlObj.setFrozenRows(freezerow, sheetIdx);
                this._updateRowHeightCollection(freezerow);
                actSheet._currentTopFrozenRow = freezerow;
                this._applyFreeze(sheetIdx, freezerow, 0);
            }
            for (var i = 0, len = fRow.idx;i < len; i++)
                actSheet._frozenHeight += parseInt(rows[i].style.height);
			xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "vertical");
        },

        freezeColumns: function (freezecolumn) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowFreezing)
                return;
            if (!this._isFirstColumn)
                this._getFirstFreezeColumn();
            var sheetIdx = xlObj.getActiveSheetIndex(), fCol = xlObj.model.sheets[sheetIdx]._firstCol, actSheet = xlObj.getSheet(sheetIdx),
				hScroll = xlObj.XLScroll._hScroller(sheetIdx), colgrp = xlObj._getContent(sheetIdx).find(".e-table").find('col');
            if (xlObj.getFrozenRows(sheetIdx) > 0 || xlObj.getFrozenColumns(sheetIdx) > 0)
                this.unfreezePanes(sheetIdx);
            this._updateColWidthCollection(freezecolumn);
            actSheet._frozenWidth = 0;
            hScroll.element.find(".e-hhandle").css("left", 0);
            hScroll.value(0);
            if (freezecolumn > 0) {
                xlObj.setFrozenColumns(freezecolumn, sheetIdx);
                actSheet._currentLeftFrozenColumn = freezecolumn;
                this._applyFreeze(sheetIdx, 0, freezecolumn);
                for (var i = 0, len = fCol.idx; i < len; i++)
                    actSheet._frozenWidth += parseInt(colgrp[i].style.width);
            }
			xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "horizontal");
        },

        freezePanes: function (cell) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowFreezing)
                return;
            var vScroll, hScroll, i, len, sheetIdx = xlObj.getActiveSheetIndex(), rows = xlObj.getRows(sheetIdx)[1], actSheet = xlObj.getSheet(sheetIdx), 
				rowHtColl = xlObj.getSheet(sheetIdx)._rowHeightCollection, colWtColl = xlObj.getSheet(sheetIdx)._colWidthCollection, top, left, fRow, fCol, freezerow = 0, freezecolumn = 0;
			this._isFirstRow = false;
			this._isFirstColumn = false;
			if (xlObj.getFrozenRows(sheetIdx) > 0 || xlObj.getFrozenColumns(sheetIdx) > 0)
			    this.unfreezePanes(sheetIdx);
            vScroll = xlObj.XLScroll._vScroller(sheetIdx);
            hScroll = xlObj.XLScroll._hScroller(sheetIdx);
            if (!ej.isNullOrUndefined(cell)) {
                if (typeof (cell) === "string") {
                    cell = xlObj.getRangeIndices(cell);
                    freezerow = cell[0];
                    freezecolumn = cell[1];
                }
                else {
                    freezerow = cell.rowIndex;
                    freezecolumn = cell.colIndex;
                }
            }
            if (freezerow > 0 && freezecolumn > 0) {
                actSheet._frozenHeight = 0;
                actSheet._frozenWidth = 0;
                top = vScroll.element.find(".e-vhandle")[0].offsetTop * vScroll._scrollData.onePx;
                for (i = 0, len = rowHtColl.length; i < len; i++) {
                    if (rowHtColl[i].value >= top) {
                        fRow = xlObj.getSheet(sheetIdx)._firstRow = { idx: i, value: rowHtColl[i].value };
                        break;
                    }
                }
                left = hScroll.element.find(".e-hhandle")[0].offsetLeft * hScroll._scrollData.onePx;
                for (i = 0, len = colWtColl.length; i < len; i++) {
                    if (colWtColl[i].value >= left) {
                        fCol = xlObj.model.sheets[sheetIdx]._firstCol = { idx: i, value: colWtColl[i].value };
                        break;
                    }
                }
                for (i = 0, len = fRow.idx; i < len; i++)
                    actSheet._frozenHeight += parseInt(rows[i].style.height);
                for (i = 0, len = fCol.idx; i < len; i++)
                    actSheet._frozenWidth += $(rows[0]).find("td")[i].offsetWidth;
                xlObj.setFrozenColumns(freezecolumn, sheetIdx);
                xlObj.setFrozenRows(freezerow, sheetIdx);
                actSheet._currentTopFrozenRow = freezerow;
                actSheet._currentLeftFrozenColumn = freezecolumn;
                this._updateRowHeightCollection(freezerow);
                this._updateColWidthCollection(freezecolumn);
                this._applyFreeze(sheetIdx, freezerow, freezecolumn);
                xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
                vScroll.element.find(".e-vhandle").css("top", 0);
                vScroll.value(0);
                hScroll.element.find(".e-hhandle").css("left", 0);
                hScroll.value(0);
            }
            else if (freezerow < 1 && freezecolumn > 0)
                this.freezeColumns(freezecolumn);
            else if (freezecolumn < 1 && freezerow > 0)
                this.freezeRows(freezerow);
            else {
                //var alertdlg = $("#" + xlObj._id + "_alertdlg"), text = xlObj._getLocStr("FPaneAlert");
                //xlObj._renderAlertDlgContent(alertdlg, "Alert", text, "", 272);
                //xlObj._alertDialog.element.ejDialog("open") && alertdlg.focus();
            }
        },

        _applyFreeze: function (sheetIdx, rowIdx, colIdx) {
            var xlObj = this.XLObj, cell, rowdiv = "", rowhdrdiv = "", coldiv = "", colhdrdiv = "", left, width = xlObj._getContent(sheetIdx).find('.e-table').width(), top, height = xlObj._getContent(sheetIdx).find('.e-table').height(),
			frRows = xlObj.getFrozenRows(sheetIdx), frCols = xlObj.getFrozenColumns(sheetIdx);
			if(this._isFirstColumn || this._isFirstRow) {
				if (frRows)
					top = xlObj.model.sheets[sheetIdx]._firstRow.value;
				if (frCols)
					left = xlObj.model.sheets[sheetIdx]._firstCol.value;
			}
			else{
				cell = xlObj.getCell(rowIdx, colIdx);
				left = cell[0].offsetLeft;
				top = cell[0].offsetTop;
			}
            if (frRows) {
                rowdiv = String.format("<div class=\"{0}\" style='width:\{1}px;\ left: \{2}px;\ top: \{3}px;'></div>", "e-frow", width, 0, top);
                rowhdrdiv = String.format("<div class=\"{0}\" style='width:\{1}px;\ left: \{2}px;\ top: \{3}px;'></div>", "e-frow", width, 0, top);
                xlObj._getContent(sheetIdx).find(".e-content").append($(rowdiv));
                xlObj._getJSSheetRowHeaderContent(sheetIdx).append($(rowhdrdiv));
            }
            if (frCols) {
                coldiv = String.format("<div class=\"{0}\" style='height:\{1}px;\ left: \{2}px;\ top: \{3}px;'></div>", "e-fcol", height, left, 0);
                colhdrdiv = String.format("<div class=\"{0}\" style='height:\{1}px;\ left: \{2}px;\ top: \{3}px;'></div>", "e-fcol", height, left, 0);
                xlObj._getContent(sheetIdx).find(".e-content").append($(coldiv));
                xlObj._getJSSheetHeader(sheetIdx).find(".e-headercontent").append($(colhdrdiv));
            }
			xlObj.setSheetFocus();
        },

        unfreezePanes: function () {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), rows = xlObj.getRows(sheetIdx), selRange = xlObj._getSelectedCells(sheetIdx);
            if (!xlObj.model.allowFreezing)
                return;
            var vScroll = xlObj.XLScroll._vScroller(sheetIdx), hScroll = xlObj.XLScroll._hScroller(sheetIdx);
            vScroll.element.find(".e-vhandle").css("top", 0);
            vScroll.value(0);
            hScroll.element.find(".e-hhandle").css("left", 0);
            hScroll.value(0);
            xlObj.setFrozenColumns(0, sheetIdx);
            xlObj.setFrozenRows(0, sheetIdx);
            xlObj._getContent(sheetIdx).find('.e-fcol, .e-frow').remove();
            xlObj._getJSSheetRowHeaderContent(sheetIdx).find(".e-frow").remove();
            xlObj._getJSSheetHeader(sheetIdx).find(".e-fcol").remove();
            $(rows[0]).removeClass("e-fr-hide");
            $(rows[1]).removeClass("e-fr-hide");
            xlObj._getContent(sheetIdx).find("col").removeClass("e-fc-hide");
            $(xlObj._getJSSheetHeader(sheetIdx).find("col")).removeClass("e-fc-hide");
            xlObj.model.sheets[sheetIdx]._frozenHeight = 0;
            xlObj.model.sheets[sheetIdx]._frozenWidth = 0;
            vScroll.scroll(0);
			xlObj.performSelection({rowIndex: selRange[0], colIndex: selRange[1]}, {rowIndex: selRange[2], colIndex: selRange[3]});
        },

        _getTopRow: function (offsetTop, sheetIdx) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), currentTopRow = xlObj.getFrozenRows(sheetIdx), idx = 0;
            for (var i = 0, len = sheet._rowHeightCollection.length; i < len; i++) {
                idx = i;
                if (sheet._rowHeightCollection[i].value > offsetTop) {
                    currentTopRow = sheet._rowHeightCollection[i].idx;
                    break;
                }
            }
            return { imaginaryIndex: currentTopRow, actualIndex: currentTopRow, i: idx };
        },

        _getLeftColumn: function (offsetLeft, sheetIdx) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), currentLeftColumn = xlObj.getFrozenColumns(sheetIdx), idx = 0;
            for (var i = 0, len = sheet._colWidthCollection.length; i < len; i++) {
                idx = i;
                if (sheet._colWidthCollection[i].value > offsetLeft) {
                    currentLeftColumn = sheet._colWidthCollection[i].idx;
                    break;
                }
            }
            return { imaginaryIndex: currentLeftColumn, actualIndex: currentLeftColumn, i: idx };
        },

        _showHideRow: function (sheetIdx, from, to, action, scrollPosition, rows, contentHt, tblHt) {
            var xlObj = this.XLObj, i, sheet = xlObj.getSheet(sheetIdx), fpHeight = 0, fRow = sheet._firstRow, fpHiddenHt = sheet._fpHiddenHeight, rowHt;
            if (action === "hide") {
                for (i = from; i <= to; i++) {
					rowHt = $(rows[0][i]).height();
                    if (contentHt > (tblHt - (fRow.value + rowHt)))
                        xlObj.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                    if (!xlObj._hasClass(rows[0][i], "e-fr-hide"))
                        fpHiddenHt = fpHiddenHt + rowHt;
                    xlObj.addClass(rows[0][i], "e-fr-hide");
                    xlObj.addClass(rows[1][i], "e-fr-hide");
                    sheet._currentTopFrozenRow = i;
                }
            }
            else {
                for (i = from; i <= to; i++) {
                    if (xlObj._hasClass(rows[0][i], "e-fr-hide")) {
                        xlObj._removeClass(rows[0][i], "e-fr-hide");
                        xlObj._removeClass(rows[1][i], "e-fr-hide");
                        fpHeight = $(rows[0][i]).height();
                        fpHiddenHt = (fpHiddenHt - fpHeight) <= 0 ? 0 : (fpHiddenHt - fpHeight);
                    }
                }
                sheet._currentTopFrozenRow = from;
            }
            sheet._fpHiddenHeight = fpHiddenHt;
            sheet._currentTopPosition = scrollPosition;
        },

        _hideRow: function (sheetIdx) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), rows = xlObj.getRows(sheetIdx), fpHiddenHt = sheet._fpHiddenHeight;
            if (!xlObj._hasClass(rows[0][sheet._currentTopFrozenRow], "e-fr-hide")) {
                xlObj.addClass(rows[0][sheet._currentTopFrozenRow], "e-fr-hide");
                xlObj.addClass(rows[1][sheet._currentTopFrozenRow], "e-fr-hide");
                sheet._fpHiddenHeight = fpHiddenHt + $(rows[0][sheet._currentTopFrozenRow]).height();
            }
            sheet._currentTopFrozenRow = sheet._currentTopFrozenRow + 1;
        },

        _showHideColumn: function (sheetIdx, from, to, action, scrollPosition, colgrp, hdrcolgrp, contentWt, tblWt) {
            var  xlObj = this.XLObj, rc, rcregx = xlObj._rcRegx, toHide = to, i, j, mCell, mRange, sheet = xlObj.getSheet(sheetIdx);
            if (action === "hide") {
                for (i = from; i <= to; i++) {
                    toHide = i;
					mCell = xlObj.getCell(0, i);
					if(xlObj._hasClass(mCell[0], "e-merge")) {
						mRange = xlObj.getRangeIndices(xlObj.XLEdit.getPropertyValueByElem(mCell, "mRange"));
						for(j = mRange[1]; j <= mRange[3]; j++) {
							this._refreshHideColumn(sheetIdx, j, colgrp, hdrcolgrp, contentWt, tblWt,  "hide");
							i = j;
						}
					}
					else
						this._refreshHideColumn(sheetIdx, i, colgrp, hdrcolgrp, contentWt, tblWt,  "hide");
                }
                sheet._currentLeftFrozenColumn = toHide;
            } else {
                for (i = from; i <= toHide; i++) {
					mCell =  xlObj.getCell(0, i);
					if (xlObj._hasClass(mCell[0], "e-mc-hide")) {
						rc = mCell[0].className.match(rcregx)[0].split(" ")[0].replace("e-rc-", "").split(":");
						mCell = xlObj.getCell(rc[0], rc[1]);
						mRange = xlObj.getRangeIndices(xlObj.XLEdit.getPropertyValueByElem(mCell, "mRange"));
						for(j = mRange[1]; j <= mRange[3]; j++) {
							this._refreshHideColumn(sheetIdx, j, colgrp, hdrcolgrp, contentWt, tblWt,  "show");
							i = j;
						}
					}
					else
						this._refreshHideColumn(sheetIdx, i, colgrp, hdrcolgrp, contentWt, tblWt,  "show");
                }
                sheet._currentLeftFrozenColumn = from;
            }
            if (sheet._currentLeftFrozenColumn <= xlObj.getFrozenColumns(sheetIdx))
                sheet._currentLeftFrozenColumn = xlObj.getFrozenColumns(sheetIdx);
			sheet._currentLeftPosition = scrollPosition;
        },

        _hideColumn: function (sheetIdx) {
            var xlObj = this.XLObj, rc, mRange, cCount = 0, mCell, rcregx = xlObj._rcRegx, sheet = xlObj.getSheet(sheetIdx), curFrozenCol = sheet._currentLeftFrozenColumn, colgrp = xlObj._getContent(sheetIdx).find("col"), hdrcolgrp = xlObj._getJSSheetHeader(sheetIdx).find("col");
			mCell =  xlObj.getCell(0, curFrozenCol);
			if (xlObj._hasClass(mCell[0], "e-mc-hide")) {
				rc = mCell[0].className.match(rcregx)[0].split(" ")[0].replace("e-rc-", "").split(":");
				mCell = xlObj.getCell(rc[0], rc[1]);
				mRange = xlObj.getRangeIndices(xlObj.XLEdit.getPropertyValueByElem(mCell, "mRange"));
				for(var j = mRange[1]; j <= mRange[3]; j++) {
					this._refreshHideColumn(sheetIdx, j, colgrp, hdrcolgrp, 0, 0,  "hide");
					cCount = cCount + 1;
				}
			}
			else{
				this._refreshHideColumn(sheetIdx, curFrozenCol, colgrp, hdrcolgrp, 0, 0,  "hide");
				cCount = cCount + 1;
			}
            sheet._currentLeftFrozenColumn = curFrozenCol + cCount;
        },

		_refreshHideColumn: function(sheetIdx, j, colgrp, hdrcolgrp, contentWt, tblWt,  status) {
			var k, xlObj = this.XLObj, len, range, fpHiddenWt = xlObj.getSheet(sheetIdx)._fpHiddenWidth, filterSpan = xlObj._getContent(sheetIdx).find(".e-filterspan"), fCol = xlObj.model.sheets[sheetIdx]._firstCol,
				 startCell = { rowIndex: -1, colIndex: -1 }, parTdIdx;
			if(status === "hide") {
				range = xlObj.getRange(0, j, xlObj.model.rowCount - 1, j);
				range.filter(".e-sswraptext").hide();
				$(colgrp[j]).addClass("e-fc-hide");
				$(hdrcolgrp[j]).addClass("e-fc-hide");
				fpHiddenWt = fpHiddenWt + $(colgrp[j]).width();
				if (contentWt > (tblWt - (fCol.value + xlObj.model.columnWidth)))
					xlObj.XLScroll._createNewColumn(sheetIdx, startCell, startCell, "insert");
				for (k = 0, len = filterSpan.length; k < len; k++) {
					parTdIdx = $(filterSpan[k]).parent()[0].cellIndex;
					if (parTdIdx === j)
						$(filterSpan[k]).addClass("e-hide");
				}
			}
			else {
				range = xlObj.getRange(0, j, xlObj.model.rowCount - 1, j);
				range.filter(".e-sswraptext").show();
				$(colgrp[j]).removeClass("e-fc-hide");
				$(hdrcolgrp[j]).removeClass("e-fc-hide");
				fpHiddenWt = fpHiddenWt - $(colgrp[j]).width();
				for (k = 0, len = filterSpan.length; k < len; k++) {
					parTdIdx = $(filterSpan[k]).parent()[0].cellIndex;
					if (parTdIdx === j)
						$(filterSpan[k]).removeClass("e-hide");
				}
			}
		},

        _changevHandlerPosition: function (top) {
            var xlObj = this.XLObj, vScroll = xlObj.XLScroll._vScroller(xlObj.getActiveSheetIndex());
            vScroll.value(top);
            if (vScroll !== null && top >= vScroll._scrollData.scrollable)
                top = vScroll._scrollData.scrollable;
            else
                top = top;
            if (vScroll !== null && top >= 0 && top <= vScroll._scrollData.scrollable) {
                top = top / vScroll._scrollData.onePx;
                vScroll.element.find(".e-vhandle").css("top", top - 1);
            }
        },

        _changehHandlerPosition: function (left) {
            var xlObj = this.XLObj, hScroll = xlObj.XLScroll._hScroller(xlObj.getActiveSheetIndex());
            hScroll.value(left);
            if (hScroll !== null && left >= hScroll._scrollData.scrollable)
                left = hScroll._scrollData.scrollable;
            else
                left = left;
            if (hScroll !== null && left >= 0 && left <= hScroll._scrollData.scrollable) {
                left = left / hScroll._scrollData.onePx;
                hScroll.element.find(".e-hhandle").css("left", left - 1);
            }
        },

        _updateRowHeightCollection: function (rowIdx) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), trs = xlObj.getRows(sheetIdx)[1], sheet = xlObj.getSheet(sheetIdx);
            if (!ej.isNullOrUndefined(trs)) {
                sheet._rHeightCollection = [{ idx: rowIdx, value: 1 }];
				sheet._rowHeightCollection = [{ idx: rowIdx, value: 1 }];
				for (var i = rowIdx + 1, len = trs.length; i < len; i++) {
				    if (!xlObj.model.scrollSettings.allowVirtualScrolling)
    					sheet._rHeightCollection.push({idx: i, value:  trs[i].offsetTop});
                    sheet._rowHeightCollection.push({ idx: i, value: sheet._rowHeightCollection[sheet._rowHeightCollection.length - 1].value + $(trs[i]).height() });
				}
			}
        },

        _updateColWidthCollection: function (colIdx) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            var col = xlObj._getJSSheetHeader(sheetIdx).find("col"), trs = xlObj.getRows(sheetIdx)[1];
            if (!ej.isNullOrUndefined(trs)) {
                sheet._colWidthCollection = [{ idx: colIdx, value: 1 }];
				sheet._cWidthCollection = [{ idx: colIdx, value: 1 }];
                for (var i = colIdx + 1, len = trs[0].cells.length; i < len; i++) {
                    sheet._colWidthCollection.push({ idx: i, value: sheet._colWidthCollection[sheet._colWidthCollection.length - 1].value + parseInt(col[i].style.width) });
                    if (!xlObj.model.scrollSettings.allowVirtualScrolling)
                        sheet._cWidthCollection.push({ idx: i, value: trs[0].cells[i].offsetLeft });
				}
			}
        },
		
		_refreshFreezeDiv: function(sheetIdx, position, size) {
			var xlObj = this.XLObj, fRow, fCol;
			if(position === "horizontal") {
				fRow = xlObj._getContent(sheetIdx).find('.e-frow');
				fRow.width(fRow.width() + size);
			}
			else{
				fCol = xlObj._getContent(sheetIdx).find('.e-fcol');
				fCol.height(fCol.height() + size);
			}
		},
    };
})(jQuery, Syncfusion);
;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};
    ej.spreadsheetFeatures.math = {
        toFraction: function (val) {
            if (this.isNumber(val)) {
                var input = val.toString(), integerval = input.split(".")[0], decimalval = input.split(".")[1];
                if (!decimalval)
                    return { integer: val };
                var wholeval = (+decimalval).toString(), placeval = this.getPlaceValue(decimalval, wholeval), gcd = this.getGcd(wholeval, placeval);
                return { integer: integerval, numerator: Number(wholeval) / gcd, denominator: Number(placeval) / gcd };
            }
            return null;
        },

        toExponential: function (val, decimalplaces) {
            if (this.isNumber(val)) {
                var input = Number(val);
                decimalplaces = this.isNumber(decimalplaces) ? decimalplaces : 2;
                var expval = input.toExponential(decimalplaces).split("e+");
                return expval[0] + "E+" + this.padZeros(expval[1]);
            }
            return null;
        },

        toAccounting: function (formatstr, value, locale) {
            if (this.isNumber(value) && formatstr) {
                var prefix, suffix, val = ej.widgetBase.formatting(formatstr, value, locale), symbol = this._currencySymbol,
                    trunval = $.trim(val.replace(symbol, "")), idx = val.indexOf(symbol);
                if (!idx || (value < 0 && idx === 1)) {
                    prefix = symbol;
                    suffix = !Number(value) ? "-" : trunval;
                }
                else {
                    prefix = !Number(value) ? "-" : trunval;
                    suffix = symbol;
                }
                value = prefix + "   " + suffix;
            }
            return value;
        },

        intToDate: function (num) {
            return new Date(((Number(num) - 2) * (1000 * 3600 * 24)) + new Date("01/01/1900").getTime());
        },

        getGcd: function (a, b) {  //make generic gcd of multiple no
            a = Number(a);
            b = Number(b);
            if (!b)
                return a;
            return this.getGcd(b, a % b);
        },

        isNumber: function (val) {
            return val - parseFloat(val) >= 0;
        },

        isTime: function (val) {
            return !val ? false : val.toString().indexOf(":") !== -1;
        },

        isFormula: function (formula) {
            var obrackets, cbrackets;
            if (formula) {
                if (!formula.toString().indexOf("=") && formula.length > 1) {
                    obrackets = formula.split("(").length - 1;
                    cbrackets = formula.split(")").length - 1;
                    return obrackets === cbrackets;
                }
            }
            return false;
        },

        getPlaceValue: function (val, digit) {
            var index = val.indexOf(digit) + digit.length;
            return "1" + Array(index + 1).join("0");
        },

        padZeros: function (val, digits, position) {
            if (this.isNumber(val)) {
                digits = this.isNumber(digits) ? digits : 2;
                position = ej.isNullOrUndefined(ej.Spreadsheet.autoFillDirection[position]) ? ej.Spreadsheet.autoFillDirection.Left : ej.Spreadsheet.autoFillDirection[position];
                if (position === ej.Spreadsheet.autoFillDirection.Left)
                    return val <= this.maxValue(digits) ? Array(digits - this.getDigits(val) + 1).join("0") + val.toString() : val.toString();
                else
                    return val < this.minValue(digits) ? val.toString() + Array(digits - this.getDigits(val) + 1).join("0") : val.toString();
            }
            return null;
        },

        maxValue: function (digits) {
            if (this.isNumber(digits))
                return Number(Array(digits + 1).join("9"));
            return null;
        },

        minValue: function (digits) {
            if (this.isNumber(digits))
                return Number("1" + Array(digits).join("0"));
            return null;
        },

        getDigits: function (val) {
            if (this.isNumber(val))
                return val.toString().length;
            return null;
        },

        _round: function round(val, digits) {
            return Number(Math.round(val + 'e' + digits) + 'e-' + digits);
        },

        _parseTime: function (val) {   // will return object as input of new date
            var len, sval, i = 0, j = 3, obj = {}, cval;
            if (this.isTime(val)) {
                val = val.toString();
                sval = val.split(" ");
                len = sval.length;
                while (i < len) {
                    if (this.isTime(sval[i])) {
                        cval = sval[i].split(":");
                        while (j < 6) {
                            obj[this._datePattern[j]] = cval[j - 3] || "0";
                            j++;
                        }
                    }
                    i++;
                }
            }
            return obj;
        },

        _dateToInt: function (date) {
            var date1 = new Date("01/01/1900"), date2 = this._isDateTime(date) ? date : new Date(date),
            timeDiff = (date2.getTime() - date1.getTime()),
            diffDays = (timeDiff / (1000 * 3600 * 24)) + 2;
            return diffDays;
        },

        _defTimeFormat: function (d) {
            var dformat = [d.getHours(),
                    d.getMinutes(),
                    d.getSeconds()].join(":");
            return dformat;
        },

        _isDate: function (text) {
            var regx = /^((0?[1-9]|1[012])[-.](0?[1-9]|[12][0-9]|3[01])[-.](19|20)?[0-9]{2})*$/, value = {};
            if (!ej.isNullOrUndefined(Globalize.parseDate(text.toString())))
                text = this._dateToInt(text.toString());
            if (regx.test(text.toString())) {
                text = text.toString().replace("-", "/", "g");
                text = this._dateToInt(text.toString());
                value.text = text;
                value.isDate = true;
            }
            else {
                value.text = text;
                value.isDate = false;
            }
            return value;
        },

        _defDateFormat: function (d) {
            var dformat = [(d.getMonth() + 1),
                        d.getDate(),
                        d.getFullYear()].join("/");
            return dformat;
        },

        _isCellReference: function (text) {
            return /^[a-z]+[0-9]+$/gi.test(text);
        },

        _isDateTime: function (date) {
            return Object.prototype.toString.call(date) === "[object Date]" && !isNaN(date.valueOf());
        }
    };
})(jQuery, Syncfusion);
;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.resizing = function (obj) {
        this.XLObj = obj;
        this._$visualElem = $();
        this._currentHCell = -1;
        this._colMinWidth = 15;
        this._rowMinHeight = 20;
        this._allowStart = false;
        this._resizeStart = false;
        this._resizeType = "";
        this._tableY = null;
        this._oldWidth = null;
        this._orgX = null;
        this._orgY = null;
        this._extra = null;
        this._initialTableWidth = null;
        this._resizeElt = null;
        this._tableX = null;
        this._oldHeight = null;
        this._initialTableHeight = null;
    };

    ej.spreadsheetFeatures.resizing.prototype = {
        //Resizing

        getColWidth: function (colIdx) {
            return this.XLObj.getSheet(this.XLObj.getActiveSheetIndex()).columnsWidthCollection[colIdx];
        },

        getRowHeight: function (rowIdx) {
            return this.XLObj.getSheet(this.XLObj.getActiveSheetIndex()).rowsHeightCollection[rowIdx];
        },

        setColWidth: function (colIdx, size) {
            if (!this.XLObj.model.allowResizing)
                return;
            this._currentHCell = colIdx;
            this._resizeColumnUsingDiff(size, this.XLObj._getJSSheetHeader(this.XLObj.getActiveSheetIndex()).find(".e-columnheader").find(".e-headercell")[this._currentHCell].offsetWidth);
        },

        setRowHeight: function (rowIdx, size) {
            if (!this.XLObj.model.allowResizing)
                return;
            this._currentHCell = rowIdx;
            this._resizeRowUsingDiff(size, this.XLObj._getJSSheetRowHeaderContent(this.XLObj.getActiveSheetIndex()).find("tbody").find(".e-rowheader")[this._currentHCell].offsetHeight);
        },

        _headerMouseDown: function (e) {
            e.preventDefault();
            this._colMouseDown(e);
        },

        _colMouseDown: function (e) {
            if (this._allowStart && ($(e.target).css("cursor") === "col-resize")) {
                var x = e.clientX, y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    x = e.pageX;
                    y = e.pageY;
                }
                x += document.documentElement.scrollLeft;
                if (e.button !== 2)
                    this._start(x, y);
                this._resizeType = "Col";
                e.preventDefault();
            }
        },

        _mouseHover: function (e) {
            var nlx, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), header = xlObj._getJSSheetHeader(sheetIdx);
            if (this._$visualElem.is(":visible"))
                return;
            var $trgt = $(e.target);
            if ($trgt.is(".e-headercelldiv"))
                $trgt = $($trgt.get(0).parentNode);
            if ($trgt.hasClass("e-headercell")) {
                var resCell = $trgt.get(0), location = resCell.getBoundingClientRect(), xy = xlObj._setXY(e), x = xy[0], y = xy[1];
                nlx = $(header.find(".e-table").get(0)).width() + $(header.find(".e-columnheader").get(0)).offset().left;
                if (((x >= (location.left + document.documentElement.scrollLeft + resCell.offsetWidth - 5)) || ((x <= (location.left + 3)))) && (x < nlx) && (x >= location.left) && (y <= location.top + document.documentElement.scrollTop + e.target.offsetHeight)) {
                    this._currentHCell = header.find(".e-headercell").index(resCell);
                    if (this._currentHCell < 1 && x < (location.left + resCell.offsetWidth - 5))
                        return;
                    $trgt.addClass("e-ss-colresize");
                    this._allowStart = true;
                }
                else {
                    $(".e-ss-colresize").removeClass("e-ss-colresize");
                    this._allowStart = false;
                    this._currentHCell = -1;
                }
            }
        },

        _start: function (x, y) {
            var xlObj = this.XLObj, height, sheetIdx = xlObj.getActiveSheetIndex(), header = xlObj._getJSSheetHeader(sheetIdx), row = header.find(".e-columnheader"), cells = row.find(".e-headercell"), cell, rect;
            if (this._currentHCell > -1 && this._currentHCell < cells.length)
                cell = cells[this._currentHCell];
            if (ej.isNullOrUndefined(cell))
                return;
            rect = cell.getBoundingClientRect();
            this._tableY = rect.top + parseInt(navigator.userAgent.indexOf("WebKit") > -1 ? document.body.scrollTop : document.documentElement.scrollTop);
            if (this._allowStart) {
                this._$visualElem = $(document.createElement("div"));
                height = xlObj._getJSSheetContent(sheetIdx).height() + header.height();
                this._$visualElem.addClass("e-reSizeColbg").appendTo(xlObj.element).css({ height: height + "px" }).addClass("e-ss-colresize");
                this._$visualElem.css({ left: x - 1, top: this._tableY });
                this._oldWidth = cell.offsetWidth;
                this._orgX = x;
                this._orgY = y;
                this._extra = x - this._orgX;
                this._resizeStart = true;
            }
            else {
                this._currentHCell = -1;
            }
        },

        _cMouseMove: function (e) {
            if (this._resizeStart) {
                var xy = this.XLObj._setXY(e), x = xy[0];
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    x = e.pageX;
                }
                x += document.documentElement.scrollLeft;
                e.preventDefault();
                this._moveVisual(x);
                $(e.target).addClass("e-ss-colresize");
            }
            else
                this._mouseHover(e);
        },

        _reSize: function (x) {
            // Function used for Resizing the column                     
            var oldWidth, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex();
            this._initialTableWidth = xlObj._getJSSheetHeader(sheetIdx).find(".e-headercontent").width();
            this._getResizableCell();
            if (this._currentHCell > -1 && this._resizeStart) {
                oldWidth = xlObj._getJSSheetHeader(sheetIdx).find(".e-columnheader").find(".e-headercell")[this._currentHCell].offsetWidth;
                this._resizeColumnUsingDiff(x - this._orgX + oldWidth, oldWidth);
                //need to refresh scroller
                if(xlObj.model.scrollSettings.allowScrolling){
                    xlObj.XLScroll._getColWidths(sheetIdx, this._currentHCell + 1);
                    xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
                }
				if(xlObj.model.allowAutoFill){
					xlObj.XLDragFill.positionAutoFillElement();
					xlObj.XLDragFill.hideAutoFillOptions();
				}
                if ($("#" + xlObj._id + "ddl").length)
                    xlObj._ddlPosition();
            }
            if (this._currentHCell > 0)
                xlObj._setRowHdrHeightResize(sheetIdx, this._currentHCell);
            xlObj.model.allowSelection && xlObj.XLSelection._refreshBorder();
            this._removeVisualElem();
        },

         _resizeColumnUsingDiff: function (newWidth, oldWidth) {
            var xlObj = this.XLObj, details = {}, sheetIdx = xlObj.getActiveSheetIndex(), header = xlObj._getJSSheetHeader(sheetIdx), $headerCol,$ContentCol,groupedColumn,currentCHCells=[];
            if (newWidth > 0) {
                newWidth = newWidth > this._colMinWidth ? newWidth : this._colMinWidth;
				var colLen=header.find(".e-colselected").length;
				var currentCHCellIdx=header.find(".e-colselected").index();
				for(var i=0;i<colLen;i++){
				if(this._currentHCell==currentCHCellIdx)
					groupedColumn=true;
				currentCHCells.push(currentCHCellIdx);
				currentCHCellIdx++;
				}
				if(!groupedColumn)
				currentCHCells = [this._currentHCell];
				for(var i=0,len=currentCHCells.length; i<len; i++)
				{
                $headerCol = header.find("col").eq(currentCHCells[i]);
                $ContentCol = xlObj._getJSSheetContent(sheetIdx).find(".e-spreadsheetcontentcontainer").find("col").eq(currentCHCells[i]);
                $headerCol.outerWidth(newWidth);
                $ContentCol.width(newWidth);
				}
				var filterIcons = xlObj._getContent(sheetIdx).find(".e-filterspan");
                for (var i = 0, len = filterIcons.length; i < len ; i++) {
                    var activeElem = $(filterIcons[i]).parent();
                    xlObj.XLFilter._postionFilterIcon(activeElem, $(filterIcons[i]));
                }
                this._findColumnsWidth();
                header.find(".e-columnheader").css("cursor", "default");
                details = { sheetIndex: sheetIdx, colIndex: this._currentHCell, newWidth: newWidth, oldWidth: oldWidth, reqType: "resize-column" };
                if (oldWidth !== newWidth)
                    xlObj._completeAction(details);
                if (xlObj.getFrozenColumns(sheetIdx) >= this._currentHCell)
                    xlObj._refreshFreezeColumns(sheetIdx, this._currentHCell, newWidth - oldWidth, "resize");
            }
        },

         _findColumnsWidth: function () {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), actSheet = xlObj.getSheet(sheetIdx), colgroup = xlObj._getJSSheetHeader(sheetIdx).find("col");
            for (var i = 0, len = actSheet.columns.length; i < len; i++) {
                if (!ej.isNullOrUndefined(actSheet.columns[i]))
                    actSheet.columnsWidthCollection[i] = colgroup.eq(i).width();
            }
        },

        _calculateWidth: function () {
            var xlObj = this.XLObj, colgroup = xlObj._getJSSheetHeader(xlObj.getActiveSheetIndex()).find("col"), width = 0;
            for (var i = 0; i < colgroup.length; i++)
                width += colgroup.eq(i).width();
            return width;

        },

        _cMouseUp: function (e) {
            if (this._resizeStart) {
                var x = e.clientX, xlObj = this.XLObj;
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    x = e.pageX;
                }
                x += document.documentElement.scrollLeft;
                this._reSize(x);
                xlObj.element.find("#" + xlObj._id + "ddl").ejDropDownList("hidePopup");
            }
        },

        _getResizableCell: function (e) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), row = xlObj._getJSSheetHeader(sheetIdx).find(".e-columnheader"), cell = row.find(".e-headercell").not(".e-hide");
            var scrollLeft = navigator.userAgent.indexOf("WebKit") > -1 ? document.body.scrollLeft : document.documentElement.scrollLeft, xlimit, point;
            for (var i = 0, len = cell.length; i < len; i++) {
                point = cell[i].getBoundingClientRect();
                xlimit = point.left + scrollLeft + 5;
                if (xlimit > this._orgX) {
                    if (xlObj.isUndefined(e))
                        this._currentHCell = i - 1;
                    else
                        e.target = xlObj._getJSSheetHeader(sheetIdx).find('th:eq(' + (i - 1) + ')')[0];
                    return;
                }
            }
        },

        _moveVisual: function (x) {
            /// Used to move the visual element in mouse move
            var xlObj = this.XLObj, bounds = xlObj._getJSSheetHeader(xlObj.getActiveSheetIndex()).find(".e-headercontent")[0].getBoundingClientRect();
            if ((bounds.left + document.documentElement.scrollLeft + bounds.width < x) || (x < bounds.left + document.documentElement.scrollLeft))
                this._$visualElem.remove();
            else if (this._currentHCell > -1)
                this._$visualElem.css({ left: x - 1, top: this._tableY });
        },

        _rowMouseDown: function (e) {
            if (this._allowStart && ($(e.target).css("cursor") === "row-resize")) {
                var x = e.clientX, y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    x = e.pageX;
                    y = e.pageY;
                }
                y += document.documentElement.scrollTop;
                if (e.button !== 2)
                    this._rowStart(x, y);
                this._resizeType = "Row";
                e.preventDefault();
            }
            return false;
        },

        _rowMouseHover: function (e) {
            if (this._$visualElem.is(":visible"))
                return;
            var $trgt = $(e.target);
            if ($trgt.hasClass("e-rowheader")) {
                var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), rHContent = xlObj._getJSSheetRowHeaderContent(sheetIdx);
                var resCell = $trgt.get(0), location = resCell.getBoundingClientRect(), xy = xlObj._setXY(e), x = xy[0], y = xy[1];
                var nly = xlObj._getJSSheetRowHeader(sheetIdx).find(".e-table").height() + rHContent.find("tbody").offset().top;
                if (((y >= (location.top + document.documentElement.scrollTop + resCell.offsetHeight - 5)) || ((y <= (location.top + 3)))) && (y < nly) && (y >= location.top) && (x <= location.left + document.documentElement.scrollLeft + e.target.offsetWidth)) {
                    this._resizeElt = $trgt.parent();
                    this._currentHCell = rHContent.find(".e-rowheader").index(resCell);
                    if (this._currentHCell < 1 && y < (location.top + resCell.offsetHeight - 5))
                        return;
                    $trgt.addClass("e-ss-rowresize");
                    this._allowStart = true;
                }
                else {
                    $(".e-ss-rowresize").removeClass("e-ss-rowresize");
                    this._allowStart = false;
                    this._currentHCell = -1;
                }
            }
        },

        _rowStart: function (x, y) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), cells = xlObj._getJSSheetRowHeader(sheetIdx).find(".e-rowheader"), cell, rect, width;
            if (this._currentHCell > -1 && this._currentHCell < cells.length)
                cell = cells[this._currentHCell];
            if (!cell)
                return;
            rect = cell.getBoundingClientRect();
            this._tableX = rect.left + parseInt(navigator.userAgent.indexOf("WebKit") > -1 ? document.body.scrollLeft : document.documentElement.scrollLeft);
            if (this._allowStart) {
                this._$visualElem = $(document.createElement("div"));
                width = xlObj._getJSSheetContent(sheetIdx).width();
                this._$visualElem.addClass("e-reSizeRowbg").appendTo(xlObj.element).css({ width: width + "px" }).addClass("e-ss-rowresize");
                this._$visualElem.css({ top: y - 1, left: this._tableX });
                this._oldHeight = cell.offsetHeight;
                this._orgX = x;
                this._orgY = y;
                this._extra = y - this._orgY;
                this._resizeStart = true;
            }
            else {
                this._currentHCell = -1;
            }
        },

        _rMouseMove: function (e) {
            if (this._resizeStart) {
                var xy = this.XLObj._setXY(e), y = xy[1];
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    y = e.pageY;
                }
                y += document.documentElement.scrollTop;
                e.preventDefault();
                $(e.target).addClass("e-ss-rowresize");
                this._moveRowVisual(y);
            }
            else
                this._rowMouseHover(e);
        },

        _rowResize: function (x, y) {
            // Function used for Resizing the row                      
            var oldHeight, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), rows = xlObj.getRows(sheetIdx)[1], isWrap = false;
            this._initialTableHeight = xlObj._getJSSheetRowHeader(sheetIdx).find(".e-rowheadercontent").height();
            this._getResizableRowCell();
            isWrap = $(rows[this._currentHCell]).find(".e-sswraptext").length > 0;
            if (this._currentHCell > -1 && this._resizeStart) {
                oldHeight = xlObj._getJSSheetRowHeaderContent(sheetIdx).find("tbody").find(".e-rowheader")[this._currentHCell].offsetHeight;
                if (!isWrap || (isWrap && (y - this._orgY + oldHeight) > oldHeight))
                    this._resizeRowUsingDiff(y - this._orgY + oldHeight, oldHeight);
                //need to refresh scroller
                if (xlObj.model.scrollSettings.allowScrolling) {
                    xlObj.XLScroll._getRowHeights(sheetIdx, this._currentHCell + 1);
                    xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
                }
				if(xlObj.model.allowAutoFill){
					xlObj.XLDragFill.positionAutoFillElement();
					xlObj.XLDragFill.hideAutoFillOptions();
				}
            }
            xlObj.XLSelection._refreshBorder();
            this._removeVisualElem();
        },

        _resizeRowUsingDiff: function (newHeight, oldHeight) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), details = {}, rHContent = xlObj._getJSSheetRowHeaderContent(sheetIdx), activeElem, curRCells = xlObj.getRows(sheetIdx)[1][this._currentHCell].cells, $headerCol, $ContentCol,groupedRow, currentRHCells=[];
            var contentHeight = this._getContentHeight(this._currentHCell, curRCells, sheetIdx) - 4;
            if (newHeight > 0) {
                newHeight = newHeight > contentHeight ? (newHeight > this._rowMinHeight ? newHeight : this._rowMinHeight) : contentHeight;
                var rowLen=rHContent.find(".e-rowselected").length;
				var currentRHCellIdx=rHContent.find(".e-rowselected").parent().index();
				for(var i=0;i<rowLen;i++){
				if(this._currentHCell==currentRHCellIdx)
							groupedRow=true;
				currentRHCells.push(currentRHCellIdx);
				currentRHCellIdx++;
				}
				if(!groupedRow)
				currentRHCells=[this._currentHCell];
				for(var i=0,len=currentRHCells.length; i<len;i++)
				{
				$headerCol = rHContent.find("tbody").children().eq(currentRHCells[i]);
                $ContentCol = xlObj._getJSSheetContent(sheetIdx).find(".e-content").find("tr").eq(currentRHCells[i]);
                $headerCol.outerHeight(newHeight);
                $ContentCol.height(newHeight);
				}
				rHContent.find("tbody").css("cursor", "default");
                rHContent.find("tbody").children().css("cursor", "default");
                this._findRowsHeight();
                var filterIcons = xlObj._getContent(sheetIdx).find(".e-filterspan");
                for (var i = 0, len = filterIcons.length; i < len; i++) {
                    activeElem = $(filterIcons[i]).parent();
                    xlObj.XLFilter._postionFilterIcon(activeElem, $(filterIcons[i]));
                }
                details = {
                    sheetIndex: sheetIdx,
                    rowIndex: this._currentHCell,
                    newHeight: newHeight,
                    oldHeight: oldHeight,
                    reqType: "resize-row"
                };
                if (oldHeight !== newHeight)
                    xlObj._completeAction(details);
                if (xlObj.getFrozenRows(sheetIdx) >= this._currentHCell)
                    xlObj._refreshFreezeRows(sheetIdx, this._currentHCell, newHeight - oldHeight, "resize");
            }
        },

        _findRowsHeight: function () {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), actSheet = xlObj.getSheet(sheetIdx), rows = xlObj._getJSSheetContent(sheetIdx).find("tbody").children();
            for (var i = 0, len = actSheet.rowCount; i < len; i++)
                if (xlObj._sheets[sheetIdx]._Rows[0])
                    actSheet.rowsHeightCollection[i] = rows.eq(i).height();
        },

        _calculateHeight: function () {
            var xlObj = this.XLObj, rows = xlObj._getJSSheetRowHeaderContent(xlObj.getActiveSheetIndex()).find("tbody").children(), height = 0;
            for (var i = 0, len = rows.length; i < len; i++)
                height += rows.eq(i).height();
            return height;
        },

        _rMouseUp: function (e) {
            if (this._resizeStart) {
                var x = e.clientX, y = e.clientY;
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    x = e.pageX;
                    y = e.pageY;
                }
                y += document.documentElement.scrollTop;
                this._rowResize(x, y);
                if ($("#" + this.XLObj._id + "ddl").length)
                    this.XLObj._ddlPosition();
            }
        },

        _getResizableRowCell: function (e) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), cell = xlObj._getJSSheetContent(sheetIdx).find(".e-rowheader"), scrollTop = navigator.userAgent.indexOf("WebKit") > -1 ? document.body.scrollTop : document.documentElement.scrollTop, ylimit, point, count = 1;
            for (var i = 0, len = cell.length; i < len; i++) {
                point = cell[i].getBoundingClientRect();
                if (point.top === 0) count++;
                ylimit = point.top + scrollTop + 5;
                if (ylimit > this._orgY) {
                    if (xlObj.isUndefined(e))
                        this._currentHCell = $(cell[i - 1]).is(":visible") ? i - 1 : i - count;
                    else
                        e.target = xlObj._getJSSheetRowHeader(sheetIdx).find('tr:eq(' + (i - 1) + ')').find("td")[0];
                    return;
                }
            }
        },

        _moveRowVisual: function (y) {
            /// Used to move the visual element in mouse move
            var xlObj = this.XLObj, bounds = xlObj._getJSSheetContent(xlObj.getActiveSheetIndex()).find(".e-rowheadercontent")[0].getBoundingClientRect();
            if ((bounds.top + document.documentElement.scrollTop + bounds.height < y) || (y < bounds.top + document.documentElement.scrollTop))
                this._$visualElem.remove();
            else if (this._currentHCell > -1)
                this._$visualElem.css({ top: y - 1, left: this._tableX });
        },

        //Activation panel resizing
        _apMouseDown: function (e) {
            if (this._allowStart && ($(e.target).closest("div").css("cursor") === "col-resize")) {
                var x = e.clientX;
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    x = e.pageX;
                }
                x += document.documentElement.scrollLeft;
                if (e.button !== 2)
                    this._apStart(x);
                this._resizeType = "AP";
                e.preventDefault();
            }
            return false;
        },

        _apMouseHover: function (e) {
            var xlObj = this.XLObj;
            if (this._$visualElem.is(":visible"))
                return;
            var $trgt = $(e.target), resizableElt = $trgt.get(0), location = resizableElt.getBoundingClientRect(), xy = xlObj._setXY(e);
            var x = xy[0];
            if ((x <= (location.left + 4)) && (x >= location.left)) {
                $trgt.parent().addClass("e-ss-apresize");
                this._allowStart = true;
            }
            else {
                xlObj.element.find(".e-ss-apresize").removeClass("e-ss-apresize");
                this._allowStart = false;
            }
        },

        _apStart: function (x) {
            var xlObj = this.XLObj, cell = xlObj.getActivationPanel()[0], rect = cell.getBoundingClientRect(), sheetIdx = xlObj.getActiveSheetIndex(), height;
            this._tableY = rect.top + parseInt(navigator.userAgent.indexOf("WebKit") > -1 ? document.body.scrollTop : document.documentElement.scrollTop);
            if (this._allowStart) {
                this._$visualElem = $(document.createElement("div"));
                height = xlObj._getJSSheetContent(sheetIdx).height() + xlObj.getJSSheetHeader(sheetIdx).height();
                this._$visualElem.addClass("e-reSizeAPbg").appendTo(xlObj.element).css({ height: height + "px" }).addClass("e-ss-apresize");
                this._$visualElem.css({ left: x, top: this._tableY });
                this._resizeStart = true;
            }
        },

        _apMouseMove: function (e) {
            if (this._resizeStart) {
                var xy = this.XLObj._setXY(e), x = xy[0];
                x += document.documentElement.scrollLeft;
                e.preventDefault();
                this._apMoveVisual(x);
            }
            else
                this._apMouseHover(e);
        },

        _apMouseUp: function (e) {
            if (this._resizeStart) {
                var x = e.clientX;
                if (navigator.userAgent.indexOf("WebKit") > -1) {
                    x = e.pageX;
                }
                x += document.documentElement.scrollLeft;
                this._apReSize(x);
            }
        },

        _apMoveVisual: function (x) {
            this._$visualElem.css({ left: x, top: this._tableY });
        },

        _apReSize: function (x) {
            var xlObj = this.XLObj;
            xlObj.model.apWidth = Math.abs(xlObj.model.scrollSettings.width - x);
            $(xlObj.getActivationPanel()).width(xlObj.model.apWidth);
            xlObj.setSheetWidthHeight(xlObj.getActiveSheetIndex(), { width: x, height: xlObj.model.scrollSettings.height });
            this._removeVisualElem();
        },

        _removeVisualElem: function () {
            var xlObj = this.XLObj;
            this._$visualElem.remove();
            xlObj.element.find(".e-reSizeRowbg").remove();
            this._resizeStart = false;
            this._currentHCell = -1;
            this._resizeType = "";
            this._allowStart = false;
            if (xlObj.model.allowComments)
                xlObj.XLComment._updateCmntArrowPos();
        },

        //Resize To Fit Width
        fitWidth: function (colIdxes) {
            //colIdxes is the array of column indexes to be resized
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex();
            for (var i = 0, len = colIdxes.length; i < len; i++)
                this._fitWidth(colIdxes[i], xlObj._sheets[sheetIdx]._Rows[1]);
            if (xlObj.model.scrollSettings.allowScrolling) {
                xlObj.XLScroll._getColWidths(xlObj.getActiveSheetIndex(), colIdxes[0] + 1);
                xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "horizontal");
            }
            xlObj.model.allowSelection && xlObj.XLSelection._refreshBorder();
        },

        _resizeToFitWidth: function (e) {
            var xlObj = this.XLObj, $trgt, sheetIdx = xlObj.getActiveSheetIndex(), rows = xlObj.getRows(sheetIdx)[1];
            if ($(e.target).css("cursor") === "col-resize") {
                if ($(e.target).is(".e-headercelldiv"))
                    e.target = e.target.parentNode;
                this._getResizableCell(e);
                $trgt = $(e.target);
                if ($trgt.hasClass("e-headercell")) {
                    this._fitWidth(e.target.cellIndex, rows);
                }
            }
            this._mouseHover(e);
            if (xlObj.model.scrollSettings.allowScrolling) {
                xlObj.XLScroll._getColWidths(xlObj.getActiveSheetIndex(), e.target.cellIndex + 1);
                xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "horizontal");
            }
            xlObj.model.allowSelection && xlObj.XLSelection._refreshBorder();
        },

        _fitWidth: function (colIdx, rows) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), trgt = xlObj._getJSSheetHeader(sheetIdx).find(".e-headercell")[colIdx], $cellDiv = $(trgt).children(".e-headercelldiv"), finalWidth = 0, headerWidth = 0, contentWidth = 0;
            rows = rows ? rows : xlObj.getRows(sheetIdx)[1];
            contentWidth = this._getContentWidth(colIdx, sheetIdx, rows);
            headerWidth = this._getHeaderContentWidth($cellDiv, sheetIdx);
            if (contentWidth > 1) {
                finalWidth = headerWidth > contentWidth ? headerWidth : contentWidth;
                finalWidth = finalWidth + 10; // for some spacing
                var details = { target: trgt, sheetIndex: xlObj.getActiveSheetIndex(), colIndex: colIdx, newWidth: finalWidth, oldWidth: trgt.offsetWidth, reqType: "resize-fit-width" };
                if (finalWidth > 0) {
                    if (finalWidth < 20)
                        finalWidth = 30;
                    this._applyWidth(trgt, sheetIdx, colIdx, finalWidth);
                    xlObj.model.allowAutoFill && xlObj.XLDragFill.positionAutoFillElement();
                    xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
                    if (!xlObj.model.sheets[sheetIdx]._importRendered && !xlObj._isUndoRedo)
                        xlObj._completeAction(details);
                    if (xlObj.getFrozenColumns(sheetIdx) >= colIdx)
                        xlObj._refreshFreezeColumns(sheetIdx, colIdx, finalWidth - details.oldWidth, "resize");
                }
            }
            xlObj.XLEdit._updateDataContainer({ rowIndex: 0, colIndex: colIdx }, { isFitWidth: true });
        },

        _applyWidth: function (trgt, activeIdx, cellIndex, finalWidth) {
            var xlObj = this.XLObj;
            xlObj._getJSSheetHeader(activeIdx).find("col").eq(cellIndex).width(finalWidth);
            xlObj._getJSSheetContent(activeIdx).find("col").eq(cellIndex + 1).width(finalWidth);
            xlObj.getSheet(activeIdx).columnsWidthCollection[cellIndex] = finalWidth;
            xlObj.getSheet(activeIdx).columns[cellIndex].width = finalWidth;
        },

        _getContentWidth: function (cellindx, sheetindex, rows) {
            var i, j, len, ln, contentWidth = 0, spanWidth, paddingWidth, tdWidth, xlObj = this.XLObj, cellSpan, td, spanColl, regx = xlObj._formatRegx, spanEle;
            for (j = 0, len = rows.length; j < len; j++) {
                td = $(rows[j]).find(".e-rowcell").eq(cellindx);
                spanColl = $(td).find("span:not(.e-comment,.e-celltoparrow)");
                spanWidth = 0; paddingWidth = 0;
                for (i = 0, ln = spanColl.length; i < ln; i++)
                    if (spanColl[i].className.indexOf("e-icon") > -1)
                        spanWidth += $(spanColl[i]).outerWidth(true) + parseInt($(spanColl[i]).css('margin-left')) + parseInt($(spanColl[i]).css('margin-right'));
                cellSpan = "<span style='display:none'>" + (ej.isNullOrUndefined(td[0].lastChild) ? "" : td[0].lastChild.nodeValue) + "</span>"; //Checking for empty td cell
                xlObj.element.append(cellSpan);
                spanEle = xlObj.element.find("span:last");
                if (td[0].className.indexOf("e-sswraptext") > -1)
                    spanEle.addClass("e-sswraptext");
                if (td[0].className.indexOf("e-format") > -1)
                    spanEle.addClass(td[0].className.match(regx)[0]);
                tdWidth = spanEle.width();
                spanEle.remove();
                if (parseInt(td.css('padding-left')) > -1)
                    paddingWidth += parseInt(td.css('padding-left'));
                if (parseInt(td.css('padding-right')) > -1)
                    paddingWidth += parseInt(td.css('padding-right'));
                tdWidth = tdWidth + spanWidth + paddingWidth;
                if (tdWidth > contentWidth)
                    contentWidth = tdWidth;
            }
            return contentWidth;
        },

        _getHeaderContentWidth: function ($cellDiv, sheetIndex) {
            var headerWidth = 0, $span = ej.buildTag("span", {}, {}), content = $cellDiv.html();
            $span.html(content);
            $cellDiv.html($span);
            headerWidth = $cellDiv.find("span:first").width();
            headerWidth = headerWidth + 10; // for some space
            $cellDiv.html(content);
            return headerWidth;
        },

        //Resize To Fit Height
        fitHeight: function (rowIdxes) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowResizing)
                return;
            var sheetIdx = xlObj.getActiveSheetIndex(), rowHdrs = xlObj.getRows(sheetIdx)[0]; // rowIdxes is the array of rows to be resized
            for (var i = 0, len = rowIdxes.length; i < len; i++)
                if (rowHdrs[i].className.indexOf("e-sswraprow") < 0)
                    this._fitHeight(rowIdxes[i]);
            if (xlObj.model.scrollSettings.allowScrolling) {
                xlObj.XLScroll._getRowHeights(sheetIdx, rowIdxes[0] + 1);
                xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "vertical");
            }
            if(xlObj.model.allowSelection)
				xlObj.XLSelection._refreshBorder();
        },

        _resizeToFitHeight: function (e) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex();
            this._getResizableRowCell(e);
            this._fitHeight(parseInt(e.target.textContent));
            if (xlObj.model.scrollSettings.allowScrolling) {
                xlObj.XLScroll._getRowHeights(xlObj.getActiveSheetIndex(), e.target.parentNode.rowIndex);
                xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "vertical");
            }
            if(xlObj.model.allowSelection)
				xlObj.XLSelection._refreshBorder();
        },

        _fitHeight: function (rowIndex) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), trgt = xlObj._getJSSheetRowHeader(sheetIdx).find(".e-rowheader")[rowIndex - 1], finalHeight = 0, $headerElement, contentHeight = 0, headerHeight, $span = ej.buildTag("span", {}, {}), curRCells = xlObj.getRows(sheetIdx)[1][rowIndex - 1].cells;
            contentHeight = this._getContentHeight(rowIndex, curRCells, sheetIdx);
            $headerElement = $(trgt);
            $span.html($headerElement.text());
            $headerElement.html($span);
            headerHeight = $headerElement.find("span:first").height();
            headerHeight = headerHeight + parseInt($($headerElement).css("padding-top")) + parseInt($($headerElement).css("padding-bottom")) + 1;
            finalHeight = (contentHeight < headerHeight) ? headerHeight : contentHeight;
            if (finalHeight < xlObj.model.rowHeight)
                finalHeight = xlObj.model.rowHeight;
            var details = { target: trgt, sheetIndex: xlObj.getActiveSheetIndex(), rowIndex: rowIndex, newHeight: finalHeight, oldHeight: trgt.offsetHeight, reqType: "resize-fit-height" };
            $(trgt.parentElement).height(finalHeight);
            $(xlObj._getContent(sheetIdx).find("tr")[rowIndex - 1]).height(finalHeight);
            xlObj.getSheet(sheetIdx).columns[rowIndex].height = finalHeight;
            if(xlObj.model.allowAutoFill)
				xlObj.XLDragFill.positionAutoFillElement();
            if(xlObj.model.allowComments)
				xlObj.XLComment._updateCmntArrowPos();
            if(xlObj.model.scrollSettings.allowScrolling)
				xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
            if (!xlObj.model.sheets[sheetIdx]._importRendered)
                xlObj._completeAction(details);
            if (xlObj.getFrozenRows(sheetIdx) >= rowIndex)
                xlObj._refreshFreezeRows(sheetIdx, rowIndex - 1, finalHeight - details.oldHeight, "resize");
            xlObj.XLEdit._updateDataContainer({ rowIndex: rowIndex, colIndex: 0 }, { "isFitHeight": true });
        },

        _getContentHeight: function (rowindx, curRCells, sheetindex) {
            var xlObj = this.XLObj, contentHeight = 0, $span = ej.buildTag("span.e-rowcell e-tspan", {}, {}), tdHeight, format = "", regx = xlObj._formatRegx, $td, content;
            for (var i = 0, len = curRCells.length; i < len; i++) {
                $td = $(curRCells[i]);
                content = $td.text();
                if (curRCells[i].className.indexOf("e-format") > -1)
                    format = curRCells[i].className.match(regx)[0];
                if (curRCells[i].className.indexOf("e-sswraptext") > -1)
                    $span.addClass("e-sswraptext");
                $span.addClass(format);
                $span.append(content);
                $td.append($span);
                tdHeight = $td.find("span.e-tspan").height();
                if (tdHeight > contentHeight)
                    contentHeight = tdHeight;
                $td.find("span:last").remove();
            }
            return (contentHeight > xlObj.model.rowHeight) ? contentHeight + 5 : contentHeight;// is for adding row's padding and margin with content.
        }

    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.editing = function (obj) {
        this.XLObj = obj;
        this._EditCellDetails = {
            value: null,
            rowIndex: -1,
            columnIndex: -1,
            fieldName: null,
            cell: null
        };        
        this._isEdit = false;
        this._isFormulaEdit = false;
        this._isFBarFocused = false;
        this._skipHTML = false;      
        this._editElem = null;
        this._filterIcon = null;
        this._isCellEdit = true;
        this._validOperators = ["+", "-", "*", "/", ","];
        this._invalidOperators = ["%"];
        this._validCharacters = ["+", "-", "*", "/", ",", "(", "="];
        this._textDecoration = false;
        this._editCell = null;
        this._isACPositioned = false;
        this._acPosition = { left: 0, top: 0 };
    };

    ej.spreadsheetFeatures.editing.prototype = {
        //Editing 
        getCurrentEditCellData: function () {
            var elem, value, xlObj = this.XLObj;
            if (this._isEdit) {
                elem = xlObj.element.find("#" + xlObj._id + "_Edit");
                value = elem.text();
                if (typeof value === "string" && value.length && xlObj.getSheet(xlObj.getActiveSheetIndex()).columns[this._EditCellDetails.columnIndex].type === "number")
                    value = parseFloat(value);
                return value;
            }
            return null;
        },

        getColumnIndexByField: function (field) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), column, len;
            for (column = 0, len = sheet.columns.length; column < len; column++) {
                if (sheet.columns[column]["field"] === field)
                    break;
            }
            return column;
        },

        _processEditing: function (sheetIdx) {
            var i, columnCount, xlObj = this.XLObj, sheetIdx = xlObj._getSheetIndex(sheetIdx),
                sheet = xlObj.getSheet(sheetIdx), $acElem, $form;
            if (sheet.columns.length === 0)
                return;
             this._editElem = ej.buildTag("div.e-field e-ss-input", "", {}, { id: xlObj._id + "_Edit", contenteditable: true });
             $acElem = ej.buildTag("input", "", { display: "none" }, { id: xlObj._id + "_AutoComplete" });
             xlObj.element.append($acElem);
             this._renderAutoComplete($acElem);
             $form = ej.buildTag("form", "", {}, { id: xlObj._id + "EditForm" }), $valElem = ej.buildTag("input", "", { width: 1, height: 1, overflow: "hidden", opacity: 0 }, { id: xlObj._id + "_ValElem" });
             $form.append($valElem);
             xlObj.element.append($form);
        },
		
        editCell: function (rowIdx, colIdx, oldData) {
            var sheetIdx, sheet, input, args, actCell, prevValue, trgtTd, selObj, xlObj = this.XLObj;
            if (!xlObj.model.allowEditing || !xlObj.model.allowSelection || xlObj._isCellProtected(rowIdx, colIdx, true))
                return;
            selObj = { rowIndex: rowIdx, colIndex: colIdx };
			this._textDecoration = xlObj.getCell(rowIdx, colIdx).find("a").css("text-decoration") === "none";
            sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), actCell = xlObj.getActiveCell(sheetIdx);
            trgtTd = xlObj.getCell(rowIdx, colIdx), column = sheet.columns[colIdx];
			this._editCell = trgtTd;
            this._isACPositioned = false;
            args = { 
                columnName: column.field,
                value: xlObj.getRangeData([rowIdx, colIdx, rowIdx, colIdx], true, null, sheetIdx, true)[0],
                columnObject: column,
                cell: trgtTd 
            };
            if (xlObj._trigger("cellEdit", args))
                return;
            xlObj._cureditedCell = args.value;
            if (actCell.rowIndex !== rowIdx || actCell.colIndex !== colIdx) {
                sheet._activeCell = selObj;
                if (sheet._startCell.rowIndex <= rowIdx && sheet._startCell.colIndex <= colIdx && sheet._endCell.rowIndex >= rowIdx && sheet._endCell.colIndex >= colIdx)
                    xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                else
                    xlObj.XLSelection.selectRange(selObj, selObj);
                if(xlObj.model.allowAutoFill)
					xlObj.XLDragFill.positionAutoFillElement();
            }
            this._EditCellDetails = {
                rowIndex: rowIdx,
                value: args.value,
                columnIndex: colIdx,
                fieldName: column.field,
                cell: trgtTd
            };
            prevValue = args.value;
            if(!oldData)
                args.value = "";
            this._renderBulkEditObject(args, trgtTd, prevValue);
            if(this.getPropertyValue(sheet._activeCell.rowIndex, sheet._activeCell.colIndex, "rule", sheetIdx))
                xlObj.XLValidate._setValidation();
            input = $("#" + xlObj._id + "_" + this._EditCellDetails.fieldName);
            if(this.getPropertyValue(rowIdx, colIdx, "hyperlink", sheetIdx))
                input.addClass("e-hyperlinks");
            if(this._textDecoration)
                input.css("text-decoration", "none");
            args.cell.addClass(xlObj._browserDetails.name === "msie" ? "e-editedcell e-msie-edit" : "e-editedcell");
            if(xlObj.isFormula(args.value))
                this._processFormulaEditRange(args.value);
        },

        _renderBulkEditObject: function (cellArgs, td, prevValue) {
            var htmlString, currHeight, filterIcon, cellData = {}, xlObj = this.XLObj, prevHeight = td.height(),
                tdSpan = td.find(".e-filterspan");
            cellData[cellArgs.columnObject.field] = cellArgs.value;
            if (tdSpan.length) {
                filterIcon = { filterIcon: tdSpan, rowIndex: tdSpan.parentsUntil("table").eq(1).index(), colIndex: tdSpan.parent().index() };
                this._filterIcon = filterIcon;
            }
            currHeight = td.height();
            td.empty();
            if (td.hasClass('e-sswraptext') && prevHeight !== currHeight)
                td.height(prevHeight);
            var $elem = this._editElem;	
            $elem.text("");
            if (!ej.isNullOrUndefined(prevValue))
                $elem.text(prevValue);
            $elem.height(currHeight);          
            xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-content").append($elem);
            $elem.css({ left: td[0].offsetLeft+2, top: td[0].offsetTop, position: "absolute", width: td.width() });     //+2px for right padding like excel          
            $elem.show();           
            this._refreshEditForm(td, $elem, cellArgs.value, prevValue, currHeight);			           
            this._isEdit = true;
            xlObj.XLRibbon._disableRibbonIcons();
        },       

        _refreshEditForm: function (cell, elem, cellvalue, prevValue,height) {
            var i, len, clsNm, splitStr, cFormatRule, cellType, cellObj = {}, xlObj = this.XLObj, form = document.getElementById(xlObj._id + "EditForm"),
                formElem = $(form).find("input,select"), sheet = xlObj._dataContainer.sheets[xlObj.getActiveSheetIndex()],
                rowIdx = cell.parent().index();
            if(sheet && sheet[rowIdx] && sheet[rowIdx][cell[0].cellIndex])
                cellObj = sheet[rowIdx][cell[0].cellIndex];                           
                if(xlObj.isNumber(prevValue))
                    elem.addClass("e-ralign");
                cellType = this.getPropertyValueByElem(cell, "type") || "";
                if(cellType.indexOf("date") > -1 || cellType.indexOf("time") > -1)
                    elem.addClass("e-ralign");
                elem.css({ "border": "none", "outline": "none" });
				elem[0].className = "";
                if(this.XLObj.model.allowCellFormatting)
                    clsNm = xlObj.XLFormat.getFormatClass(cell[0].className);
                elem.addClass(clsNm);
                if (xlObj.model.allowCellFormatting) {
                    if(!xlObj.XLFormat.getFormatFromHashCode(clsNm).color)
                        elem.css({ "color": "#333333" });
                    if(!xlObj.XLFormat.getFormatFromHashCode(clsNm)["background-color"])
                        elem.css({ "background-color": "#ffffff" });
                }
                if (cellObj && "cFormatRule" in cellObj) {
                    cFormatRule = $.extend(true, [], cellObj.cFormatRule);
                    for (var m = 0, rLen = cFormatRule.length; m < rLen; m++) {
                        splitStr = xlObj._dataContainer.cFormatData[cFormatRule[m]].split("_");
                        if($.parseJSON(splitStr[7]))
                            elem.css({ "background-color": cell.css("background-color"), "color": cell.css("color") });
                    }
                }
                elem.on("input", function (e) {
                    var sObj = $("#" + this.id.split("_")[0]).data("ejSpreadsheet");
                    sObj.XLEdit._isFormulaEdit = !$(e.target).text().indexOf("=");
                    if(sObj.XLEdit._isFormulaEdit)
                        sObj.XLEdit._processFormulaEditRange($(e.target).text());
                    sObj._getInputBox().val($(e.target).text());
                    sObj.XLEdit._isCellEdit = true;
                });
                elem.on("focus", function (e) {
                    var sObj = $("#" + this.id.split("_")[0]).data("ejSpreadsheet");
                    sObj.XLEdit._isCellEdit = true;
                });             
                xlObj._setFormulaSuggElem($("#" + xlObj._id+"_AutoComplete_suggestion"));            
            this._focusElements(elem);
        },

        _renderAutoComplete: function (elem, value, height) {
            var xlObj = this.XLObj;
            elem.ejAutocomplete({
                dataSource: xlObj._formulaCollection,
                value: value,
                cssClass: "e-ss-autocomplete e-ss-editinput",
                width: "100%",
                height: height,
                delaySuggestionTimeout: 10,
                minCharacter: 2,
                popupWidth: "170px",
                popupHeight: "150px",
                showEmptyResultText: false,
                autoFocus: true,
                template: "<span class='e-icon e-ss-function'> </span>" + "<div> ${display} </div>",
                select: $.proxy(this._formulaSelect, xlObj),
                open: $.proxy(this._formulaSuggestionStatus, xlObj),
                close: $.proxy(this._formulaSuggestionStatus, xlObj)
            });
            elem.data("ejAutocomplete")._bubbleEvent(true);
        },

		 _captureEditing: function () {
            var xlObj = this.XLObj, acElem, actElem = document.activeElement;
            if (actElem && actElem.tagName === "DIV") {
                acElem = xlObj.element.find("#" + xlObj._id + "_AutoComplete");
                acElem.val($(actElem).text());
                acElem.data("ejAutocomplete").search();
                if (!this._isACPositioned) {
                    var editElem = xlObj.element.find("#" + xlObj._id + "_Edit")[0];
                    var location = this._editCell[0].getBoundingClientRect();
                    this._acPosition.left = location.left + window.pageXOffset;
                    this._acPosition.top = location.top + window.pageYOffset + $(editElem).height();
                    this._isACPositioned = true;
                }
                $("#" + xlObj._id + "_AutoComplete_suggestion").css({ left: this._acPosition.left, top: this._acPosition.top });
             }
		 },
		
        saveCell: function () {
            var i, actCell, hldata, form, column, cFormatStr, trgtTd, elem, cFormatRule, len, rowIdx, colIdx, args, prevValue,
            details, sheetIdx, sheet, xlObj = this.XLObj;
            if (!xlObj.model.allowEditing)
                return;
            if (this._isEdit) {
                sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), form = $("#" + xlObj._id + "EditForm");
                trgtTd = this._editCell, column = sheet.columns[this._EditCellDetails.columnIndex];
                elem = xlObj.element.find("#" + xlObj._id + "_ValElem");
				var editElem = xlObj.element.find("#" + xlObj._id + "_Edit")[0], htmlVal=$(editElem).html();				 
                args = {
                    columnName: column.field,
                    value: this.getCurrentEditCellData(),
                    pValue: this._EditCellDetails.value,
                    columnObject: column,
                    cell: trgtTd
                };
				if (xlObj.model.allowDataValidation) {
                    elem.val(args.value);                   
                    if (!form.validate().element(elem[0]))
                        return false;
                }
                rowIdx = args.cell.parent().index(), colIdx = args.cell.index();
				if (xlObj._trigger("cellSave", args))
                    return;
                this.updateCellValue({rowIndex: rowIdx, colIndex: colIdx}, args.value);
                hldata = this.getPropertyValue(rowIdx, colIdx, "hyperlink", sheetIdx);
                if (hldata)
                    args.cell.html(ej.buildTag("a", args.cell.text(), "", hldata.webAdr ? { href: hldata.webAdr, target: "_blank", "class": "e-hyperlinks" } : { href: "#", "class": "e-hyperlinks" }));
                if(this._textDecoration)
                    args.cell.find("a").css("text-decoration", "none");				
				prevValue = this._EditCellDetails.value || "";
                details = { sheetIndex: sheetIdx, rowIndex: rowIdx, colIndex: colIdx, cValue: args.value, reqType: "edit", iconName: "", pValue: prevValue };
                if (xlObj.isFormula(args.value))
                    details.operation = "formula";
                xlObj._setRowHdrHeight(sheetIdx, rowIdx);                
                this._isEdit = this._isFormulaEdit = false;
                args.cell.removeClass("e-editedcell e-msie-edit");
                if (xlObj.model.allowKeyboardNavigation)
                    xlObj.XLCellNav._isNavigate = true;
                if (xlObj.model.showRibbon) {
                    xlObj.XLRibbon._enableRibbonIcons();
                    xlObj.XLRibbon._updateRibbonIcons();
                }
                xlObj._updateUndoRedoIcons();
                if (!xlObj.model.allowInsert) {
                    xlObj.XLRibbon._disableButtons(["inssbtn"], "ejSplitButton");
                    xlObj.XLRibbon._disableButtons(["insbtn"], "ejButton");
                }
                if (!xlObj.model.allowDelete) {
                    xlObj.XLRibbon._disableButtons(["delsbtn"], "ejSplitButton");
                    xlObj.XLRibbon._disableButtons(["delbtn"], "ejButton");
                }
                details.cFormatRule = cFormatRule = this.getPropertyValue(rowIdx, colIdx, "cFormatRule", sheetIdx);
				xlObj._dupDetails = true;
                if (xlObj.model.allowConditionalFormats && cFormatRule) {
                    for (i = 0, len = cFormatRule.length; i < len; i++) {
                        cFormatStr = cFormatRule[i].split("_");
                        xlObj.XLCFormat._cFormat(cFormatStr[0], cFormatStr[2], cFormatStr[3], cFormatStr[4], xlObj._generateHeaderText(sheetIdx, colIdx + 1) + (rowIdx + 1));
                    }
                }
				xlObj._dupDetails = false;
				if (!args.pValue || args.value !== args.pValue.toString())
                    xlObj._completeAction(details);
				actCell = xlObj.getActiveCellElem();
				if (actCell) {
				    xlObj._hasComment = true;
				    if(actCell.hasClass("e-commentcell"))
				        xlObj.XLComment._updateCmntFrmCntnr(null, null, actCell);
				    if (actCell.hasClass("e-hlcell"))
				        xlObj.XLValidate.highlightInvalidData();
				}
                xlObj._hasComment = this._shiftKeyEnabled = this._isFBarFocused = false;
                xlObj._ctrlKeyCount = 0;
                if (this._isFormulaEdit && xlObj.model.allowSelection)
                    xlObj.XLSelection._clearBorder(xlObj._formulaBorder.join(" ").replace(/,/g, " ") + " " + xlObj._ctrlFormulaBorder.join(" ").replace(/,/g, " "));
                this._editElem.hide();                
                xlObj.setSheetFocus();
                if (htmlVal.indexOf("<br>") > -1)
                    xlObj.setWrapText();
                return true;
            }
        },

        updateCellValue: function (cell, val, formatclass, sheetIdx) {  // change the cell args by discuss
            if (!cell)
                return;
            var xlObj = this.XLObj, colIdx = cell.colIndex, rowIdx = cell.rowIndex, actCell, prevVal = this.getPropertyValue(rowIdx, colIdx, 'value2');;
            sheetIdx = xlObj.getActiveSheetIndex();
            if (xlObj.isFormula(val)) {
                this._refreshCalcEngine(rowIdx, colIdx, true, val);
                this._updateDataContainer({rowIndex: rowIdx, colIndex: colIdx}, { value: val, type: ej.Spreadsheet.CellType.Formula });
                xlObj._applyFormula(sheetIdx, rowIdx, colIdx);
                this._updateUsedRange(rowIdx, colIdx);
                xlObj._isFormulaSuggestion = false;
            }
            else {
                this.updateCell({ rowIndex: rowIdx, colIndex: colIdx }, val);
                this._refreshCalcEngine(rowIdx, colIdx);
            }
            if (xlObj._isRowViewable(sheetIdx, cell.rowIndex)){
				if(xlObj.model.allowCellFormatting && xlObj.XLFormat._hasFormat(formatclass)) {
					actCell = xlObj.getCell(rowIdx, colIdx, sheetIdx);
					xlObj.XLFormat._updateFormatClass(actCell[0], formatclass);
				}
				(prevVal || val) && xlObj._textClip(rowIdx, colIdx, val ? 'add' : 'delete');
                !ej.isNullOrUndefined(this._filterIcon) && this._filterIconHandler(rowIdx, colIdx, cell);
            }
        },

        updateCell: function (cell, val, sheetIdx) {
            if (!cell)
                return;
            var xlObj = this.XLObj, rIdx = cell.rowIndex, cIdx = cell.colIndex, ctype = ej.Spreadsheet.CellType,
            // parse value
            valObj = this._parseValue(val),
            cellInfo = xlObj.getRangeData([rIdx, cIdx, rIdx, cIdx], false, ["formatStr", "type", "thousandSeparator", "decimalPlaces"], sheetIdx)[0];
            // cell type
            if (cellInfo.type && cellInfo.type != ctype.General) {
                valObj.type = cellInfo.type;
                delete valObj.formatStr;
            }
            // cell format
            if (cellInfo.formatStr || cellInfo.type === ctype.Scientific || cellInfo.type === ctype.Fraction)
                valObj.value2 = xlObj.XLFormat._format(valObj.value, { formatStr: cellInfo.formatStr, type: cellInfo.type, thousandSeparator: cellInfo.thousandSeparator, decimalPlaces: cellInfo.decimalPlaces });
            // data updation            
            this._updateDataContainer(cell, valObj, sheetIdx);
            this._updateUsedRange(rIdx, cIdx);
        },

        _parseValue: function (value) {
            value = this.XLObj.isNumber(value) ? Number(value) : value || "";
            var prop, temp, formatStr, xlObj = this.XLObj, locale = xlObj.model.locale, ctype = ej.Spreadsheet.CellType, value2 = value + "", type = ctype.General,
                dSymbol = Globalize.culture(locale).numberFormat["."], tSep = Globalize.culture(locale).numberFormat[","],
                patrns = Globalize.culture(locale).calendar.patterns, idx = value2.indexOf(xlObj._currencySymbol);
            if (idx === 0 || (idx > -1 && idx === value2.length - 1)){
                temp = Globalize.parseFloat(value2, 10, locale);
                if(temp)
                    return { value: temp, value2: value2, type: ctype.Currency };
            }
            if ($.trim(value2).indexOf(xlObj._percentSymbol) > -1) {
                temp = $.trim(value2).replace(xlObj._percentSymbol, "");
                if (xlObj.isNumber(temp))
                    return { value: Number(temp), value2: value2, type: ctype.Percentage };
            }
            if ((value2.indexOf(dSymbol) > -1 && value2.indexOf(tSep) === -1) || (value2.indexOf(dSymbol) > -1 && value2.indexOf(tSep) > -1)) {
                temp = Globalize.parseFloat(value2, 10, locale);
                if (temp)
                    return { value: temp, value2: value2, type: type };
            }
            for (prop in patrns) {
                temp = Globalize.parseDate(value2, patrns[prop], locale);
                if (temp) {
                    formatStr = "{0:" + patrns[prop] + "}";
                    value2 = xlObj.XLFormat._format(temp, { type: ctype.DateTime, formatStr: formatStr });
                    return { value: temp, value2: value2, type: ctype.DateTime, formatStr: formatStr };
                }
            }
            if (value2.toLowerCase() === "true" || value2.toLowerCase() === "false")
                value = value2 = value2.toUpperCase();
            return { value: value, value2: value2, type: type };
        },

        updateCellWithContainer: function (data, range, sheetIdx, skipHiddenRow) {
            var xlObj = this.XLObj;
            sheetIdx = sheetIdx ? sheetIdx : xlObj.getActiveSheetIndex(), range = range ? range : xlObj.getSheet(sheetIdx).selectedRange;
            var actCell, colIdx, rowIdx, cellData, cell, cells = this.XLSelection._getSelectedRange({rowIndex: range[0], colIndex: range[1]}, {rowIndex: range[2], colIndex: range[3]}), i = 0, j = 0, length = cells.length;
            while (i < length) {
                cell = cells[i];
                rowIdx = cell.rowIndex;
                if(!skipHiddenRow || !xlObj._isHiddenRow(rowIdx)){
                    colIdx = cell.colIndex;
                    cellData = data[j];
                    this._updateDataContainer(cell, cellData);
                    if(xlObj._isRowViewable(sheetIdx, cell.rowIndex)){
                        actCell = xlObj.getCell(cell.rowIndex, cell.colIndex, sheetIdx);
                        isViewed = true;
                        actCell.innerHTML = ej.isNullOrUndefined(cellData.value2) ? "" : cellData.value2.toString();
                    }
                    this._refreshCellAlignment(cell, cellData.value);
                    this._refreshCalcEngine(rowIdx, colIdx, xlObj.isFormula(cellData.value), cellData.value);
                    (isViewed && cellData.format) && xlObj.addClass(actCell, cellData.format);
                    if (isViewed && cellData.hyperlink)
                        "cellAdr" in cellData.hyperlink ? actCell.html(ej.buildTag('a.e-hyperlinks', cellData.value2, '', { href: '#' })) : actCell.html(ej.buildTag('a.e-hyperlinks', cellData.value2, '', { href: cellData.hyperlink.webAdr, target: '_blank' }));
                    cellData.comment && xlObj.XLComment.setComment([cell.rowIndex, cell.colIndex, cell.rowIndex, cell.colIndex], null, false);
                    cellData.border && xlObj.XLFormat.applyBorder(xlObj._getBorderInput(cellData.border), xlObj._getAlphaRange(sheetIdx, cell.rowIndex, cell.colIndex, cell.rowIndex, cell.colIndex));
                    j++;
                }
                i++;
            }
            xlObj.model.allowConditionalFormats && xlObj.XLCFormat.refreshCFormat(range);
        },

        _refreshCellAlignment: function (cell, value, type) {
            var xlObj = this.XLObj, $cell = $(cell), rAlign = xlObj._rAlign, cAlign = xlObj._cAlign, sheetIdx = xlObj.getActiveSheetIndex(), rowIdx = cell.parentNode.rowIndex, colIdx = cell.cellIndex;
            value = ej.isNullOrUndefined(value) ? this.getPropertyValue(rowIdx, colIdx, "value") : value;
            if (xlObj.isFormula(value))
                value = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "calcValue");
            type = type ? type : this.getPropertyValue(rowIdx, colIdx, "type");
            if (xlObj._isRowViewable(sheetIdx, rowIdx)) {
                if (!this.getPropertyValue(rowIdx, colIdx, "isCenterAlign"))
                    cell.className.indexOf(cAlign) > -1 && $cell.removeClass(cAlign);
                cell.className.indexOf(rAlign) > -1 && $cell.removeClass(rAlign);
                if ((xlObj.isNumber(value) || typeof value === "object") && type !== ej.Spreadsheet.CellType.Text)
                    xlObj.addClass(cell, rAlign);
                else if (xlObj._isBool(value))
                    xlObj.addClass(cell, cAlign);
            }
        },

        _filterIconHandler: function (rowIdx, colIdx, cell) {
            if (this._filterIcon.rowIndex === rowIdx && this._filterIcon.colIndex === colIdx) {
                (cell[0].className.indexOf(this.XLObj._rAlign) > -1) && cell.removeClass(this.XLObj._rAlign);
                cell[0].insertBefore(this._filterIcon.filterIcon[0], cell[0].childNodes[cell[0].childNodes.length - 1]);
                this.XLObj.XLFilter._postionFilterIcon(cell);
                this._filterIcon = null;
            }
        },

        _editingHandler: function (e) {
            var xlObj = this.XLObj, sheetIdx, sheet, $trgt;
            if ((e.keyCode === 61 || (!e.shiftKey && e.keyCode === 32) || (e.keyCode >= 48 && e.keyCode <= 59) || (e.keyCode >= 65 && e.keyCode <= 90) || (e.keyCode >= 96 && e.keyCode <= 111) || (e.keyCode === 173) || (e.keyCode >= 186 && e.keyCode <= 192) || (e.keyCode >= 219 && e.keyCode <= 222) || (e.keyCode >= 226)) && !e.ctrlKey && !e.altKey) {
                sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), $trgt = xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex);
                if (!this._isEdit && !$trgt.hasClass("e-readonly")) {                    
                    xlObj.element.find(".e-cdata").length && xlObj.element.find(".e-cutright, .e-cutbottom").removeClass("e-cutright e-cutbottom");                    
                    this.editCell($.inArray($trgt.closest("tr").get(0), xlObj.getRows(sheetIdx)[1]), $trgt.index(), false);
                }
            }
            else if (e.keyCode === 113 && !e.shiftKey && !e.ctrlKey) { //F2            
                sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), $trgt = xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex);
                if (!this._isEdit && !$trgt.hasClass("e-readonly")) {
                    xlObj.element.find(".e-cdata").length && xlObj.element.find(".e-cutright, .e-cutbottom").removeClass("e-cutright e-cutbottom");
                    this.editCell($.inArray($trgt.closest("tr").get(0), xlObj.getRows(sheetIdx)[1]), $trgt.index(), true);
                    xlObj.XLCellNav._isNavigate = false;
                }
            }
            else if (e.keyCode === 27 && this._isEdit) { //escape                
                var sheetDt, cellObj, rIdx, cIdx;
                sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), rIdx = sheet._activeCell.rowIndex, cIdx = sheet._activeCell.colIndex, $trgt = xlObj.getCell(rIdx, cIdx), sheetDt = xlObj._dataContainer.sheets[sheetIdx];
                cellObj = rIdx in sheetDt && cIdx in sheetDt[rIdx] ? sheetDt[rIdx][cIdx] : cellObj;
                $trgt.removeClass("e-editedcell e-msie-edit");
                $trgt.get(0).textContent = xlObj.isUndefined(cellObj) ? "" : typeof cellObj["value2"] === "string" ? cellObj["value2"] : xlObj._dataContainer.sharedData[cellObj["value"]];
                xlObj.setSheetFocus();
                xlObj._setRowHdrHeight(sheetIdx, sheet._selectedCells[0].rowIndex);
                this._isEdit = this._isFormulaEdit = this._isFBarFocused = false;
                xlObj._getContent(sheetIdx).find("td").removeClass(xlObj._formulaBorder.join(" ").replace(/,/g, " ") + " " + xlObj._ctrlFormulaBorder.join(" ").replace(/,/g, " "));
                if(xlObj.model.allowComments)
					$trgt.hasClass("e-commentcell") && xlObj.XLComment._updateCmntFrmCntnr(null, null, $trgt);
                if(xlObj.model.allowAutoFill)
					xlObj.XLDragFill.positionAutoFillElement();
                xlObj.model.showRibbon && xlObj.XLRibbon._enableRibbonIcons();
				!ej.isNullOrUndefined(this._filterIcon) && this._filterIconHandler(rIdx, cIdx, $trgt);
                if(!ej.isNullOrUndefined(xlObj.XLCellNav)) xlObj.XLCellNav._isNavigate = true;
            }
            else if (e.keyCode === 46) { //delete   
                if (!this._isEdit) {
					if (xlObj._preventctrlkey) {
						xlObj._showAlertDlg("Alert", "CtrlKeyErrorAlert", "CtrlKeyErrorAction", 450);
						return;
					}
                    sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), $trgt = xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex);
                    var cName, i, j, len, m, n, rLen, cFormatStr, aRange;
                    var regxFormat = new RegExp("\\b" + "e-format" + ".*?\\b", "g"), cFormatRule;
                    var selected = sheet._selectedCells, text = xlObj.getRangeData([selected[0].rowIndex, selected[0].colIndex, selected[selected.length - 1].rowIndex, selected[selected.length - 1].colIndex]);
					if(xlObj.model.allowLockCell && sheet._isSheetProtected)
					{
						if(xlObj._israngeProtected(selected))
						{
							xlObj._popUplockCellAlert();
							return;
						}
					}
                    if (!this._isEdit) {
                        xlObj.element.find(".e-cdata").length && xlObj.element.find(".e-cutright, .e-cutbottom").removeClass("e-cutright e-cutbottom");
                        var tblMgr = xlObj.model.sheets[sheetIdx]._tableManager, tblObj, tblRange, tblCells;
                        for (m = 0, rLen = xlObj.getObjectLength(tblMgr) ; m < rLen; m++) {
                            tblRange = tblMgr[m].range;
                            if (xlObj.XLClipboard._compareRange([selected[0].rowIndex, selected[0].colIndex, selected[selected.length - 1].rowIndex, selected[selected.length - 1].colIndex], tblRange)) {
                                aRange = xlObj._getAlphaRange(sheetIdx, tblRange[0], tblRange[1], tblRange[2], tblRange[3]);
                                xlObj.clearAll(aRange);
                                return;
                            }
                        }
						!$trgt.hasClass("e-readonly") && xlObj.clearRangeData(null, ["value2","value"]);
                        for (i = 0, len = selected.length; i < len; i++) {
                            $trgt = xlObj.getCell(selected[i].rowIndex, selected[i].colIndex);
                            if (!$trgt.hasClass("e-readonly")) {  
                                xlObj._getInputBox().val('');
								if(xlObj.model.allowComments)
									xlObj.XLComment._updateCmntFrmCntnr(selected[i].rowIndex, selected[i].colIndex);
                                cFormatRule = this.getPropertyValueByElem($trgt, 'cFormatRule');
                                if (xlObj.model.allowConditionalFormats && cFormatRule && cFormatRule.length) {
                                    for (j = 0, rLen = cFormatRule.length; j < rLen; j++) {
                                        cFormatStr = cFormatRule[j].split("_");
                                        xlObj.XLCFormat._cFormat(cFormatStr[0], cFormatStr[2], cFormatStr[3], cFormatStr[4], cFormatStr[5]);
                                    }
                                }
                                xlObj._setRowHdrHeight(sheetIdx, selected[i].rowIndex);
                            }
                        }
                        xlObj.XLSelection._refreshBorder();
                    }
                    var details = { sheetIndex: sheetIdx, range: [selected[0].rowIndex, selected[0].colIndex, selected[selected.length - 1].rowIndex, selected[selected.length - 1].colIndex], bData: text, reqType: "clear-content" };
                    xlObj._completeAction(details);
                }
                else {
                    if (xlObj._ctrlKeyCount > 0) {
                        xlObj._ctrlKeyCount = 0;
                        xlObj._formulaRange = [];
                        xlObj.XLSelection._clearBorder(xlObj._formulaBorder.join(" ").replace(/,/g, " ") + " " + xlObj._ctrlFormulaBorder.join(" ").replace(/,/g, " "));
                    }
                }
            }
            else if (!e.ctrlKey && !e.shiftKey && e.keyCode === 8) { //backspace
                sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), $trgt = xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex);
                if (!this._isEdit && !$trgt.hasClass("e-readonly")) {
                    xlObj.element.find(".e-cdata").length && xlObj.element.find(".e-cutright, .e-cutbottom").removeClass("e-cutright e-cutbottom");
                    $trgt = xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex);
                    $trgt.empty();
                    this.editCell($.inArray($trgt.closest("tr").get(0), xlObj.getRows(sheetIdx)[1]), $trgt.index(), false);
                    xlObj._setRowHdrHeight(sheetIdx, sheet._activeCell.rowIndex);
                    xlObj._getInputBox().val("");
                }
            }
            else if ((((e.keyCode === 13 && !e.altKey) || e.keyCode === 9)) && this._isEdit && xlObj.model.allowKeyboardNavigation && !xlObj.XLCellNav._isNavigate && !this._isFormulaEdit) { //enter  
                e.preventDefault();
                if(!ej.isNullOrUndefined(xlObj.XLCellNav)) xlObj.XLCellNav._isNavigate = true;
            }
            else if ((e.keyCode === 13 && e.altKey) && this._isEdit) { //alt + enter  
                e.preventDefault();
                var editElem = xlObj.element.find("#" + xlObj._id + "_Edit");
                editElem.html(editElem.html() + "<br> <br>");
                editElem.focus();
                editElem.focusEnd();
            }
        },

        _focusElements: function (cell) {
            $(cell).not(":hidden").focus().setInputPos($(cell).text().length);
        },

        _updateUsedRange: function (rowIdx, colIdx) {
            var xlObj = this.XLObj, usedRange = xlObj.model.sheets[xlObj.getActiveSheetIndex()]._usedRange;
            usedRange.rowIndex < rowIdx && (usedRange.rowIndex = rowIdx);
            usedRange.colIndex < colIdx && (usedRange.colIndex = colIdx);
        },

        _updateDataContainer: function (cellIdx, dataobj, sheetIdx) {
            var i, cell, ctype, colObj, rowobj, index, data, arr, cellData, colIdx, rowIdx, innerTag, container,
                canRefresh, sheet, sheetData, rowData, prefix, regx, format = "e-format", border = "e-border",
                xlObj = this.XLObj;
            if (xlObj.getObjectLength(dataobj)) {
                canRefresh = false, ctype = ej.Spreadsheet.CellType,
                container = xlObj._dataContainer, colIdx = cellIdx.colIndex, rowIdx = cellIdx.rowIndex;
                sheetIdx = xlObj._getSheetIndex(sheetIdx), sheetData = container.sheets[sheetIdx];
                arr = ["value", "value2", "type", "cFormatRule", "range", "thousandSeparator", "rule", "format", "border", "picture", "chart", "calcValue", "isSelected"];
                if (rowIdx in sheetData)
                    rowobj = sheetData[rowIdx];
                else
                    sheetData[rowIdx] = rowobj = {};
                rowData = sheetData[rowIdx];
                if (colIdx in rowData)
                    colObj = rowData[colIdx];
                else
                    rowData[colIdx] = colObj = {};
                if (xlObj._isRowViewable(sheetIdx, cellIdx.rowIndex))
                    cell = xlObj.getCell(cellIdx.rowIndex, cellIdx.colIndex, sheetIdx)[0];                
                for (i in dataobj) {
                    data = dataobj[i];
                    switch (i) {
                        case arr[0]:
                        case arr[4]:
                        case arr[5]:
                        case arr[11]:
                            if(container.sharedData.indexOf(data) === -1)
                                container.sharedData.push(data);
                            index = container.sharedData.indexOf(data);
                            colObj[i] = index;
                            if (!xlObj.model.sheets[sheetIdx]._importRendered && !xlObj._initialLoad) {
                                if (i === arr[0] && xlObj.model.allowCharts)
                                    canRefresh = true;
                            }
                            break;
                        case arr[1]:
                            if ([ctype.General, ctype.Text].indexOf(dataobj.type) === -1)
                                colObj[i] = dataobj.value2;
                            else {
                                if(container.sharedData.indexOf(dataobj.value2) === -1)
                                    container.sharedData.push(dataobj.value2);
                                colObj[i] = container.sharedData.indexOf(dataobj.value2);
                            }
                            if (cell) {
                                innerTag = cell.getElementsByTagName("a");
                                if (innerTag.length)
                                    innerTag[0].innerHTML = dataobj.value2;
                                else
                                    cell.innerHTML = this._skipHTML ? cell.innerHTML.concat(dataobj.value2) : dataobj.value2;
                                this._refreshCellAlignment(cell, dataobj.value, dataobj.type);
                            }
                            break;
                        case arr[2]:
                            colObj[i] = container.cellType.indexOf(dataobj.type);
                            break;
                        case arr[3]:
                            var j = 0, k = data.length;
                            while (j < k) {
                                if(container.cFormatData.indexOf(data[j]) === -1)
                                    container.cFormatData.push(data[j]);
                                index = container.cFormatData.indexOf(data[j]);
                                colObj[i] ? (colObj[i].indexOf(index) < 0 && colObj[i].push(index)) : colObj[i] = [index];
                                j++;
                            }
                            break;
                        case arr[6]:
                            if(container.valData.indexOf(data.customVal) === -1)
                                container.valData.push(data.customVal);
                            colObj[i] = JSON.parse(JSON.stringify(data));
                            colObj[i].customVal = container.valData.indexOf(data.customVal);
                            break;
                        case arr[7]:
                        case arr[8]:
                            if(cell){
                                prefix = arr[8] ? border : format, regx = new RegExp('\\b' + prefix + '.*?\\b', 'g');
                                cell.className = cell.className.replace(regx, "");
                                                                
                                xlObj.addClass(cell, data);
                            }                                                       
                            if(container.hashCode.indexOf(data) === -1)
                                container.hashCode.push(data);
                            colObj[i] = container.hashCode.indexOf(data);
                            break;
                        case arr[9]:
                        case arr[10]:
                            colObj[i] ? colObj[i].indexOf(data) < 0 && colObj[i].push(data) : colObj[i] = [data];
                            break;
                        case arr[12]:
                            colObj[i] = data;
                            if(cell)
                                xlObj.addClass(cell, "e-selected");
                            break;
                        default:
                            colObj[i] = data;
                    }
                }
                if (xlObj.getObjectLength(colObj) === 3) {
                    cellData = xlObj.getRangeData([rowIdx, colIdx, rowIdx, colIdx], false, arr.slice(0, 3))[0];
                    if (cellData.value === "" && cellData.value2 === "" && cellData.type === ej.Spreadsheet.CellType.General)
                        xlObj.clearRangeData(null, arr.slice(0, 3), cell);
                }
                if(canRefresh)
                    xlObj.XLChart._refreshChartElements(rowIdx, colIdx, sheetIdx);
            }
            else {
                if (cell)
                    cell.innerHTML = "";
            }
        },

        getPropertyValue: function (rowindex, colindex, property, sheetIdx) {
            var value, coldata, ctype = ej.Spreadsheet.CellType, xlObj = this.XLObj, container = xlObj._dataContainer, sheets = container.sheets;
            sheetIdx = sheetIdx ? sheetIdx : xlObj.getActiveSheetIndex();
            if (sheetIdx in sheets && rowindex in sheets[sheetIdx] && colindex in sheets[sheetIdx][rowindex]) {
                coldata = sheets[sheetIdx][rowindex][colindex];
                if (!ej.isNullOrUndefined(coldata)) {
                    property = property ? property : "value";
                    switch (property) {
                        case "value":
                        case "range":
                        case "thousandSeparator":
                        case "calcValue":
                            value = container.sharedData[coldata[property]];
                            break;
                        case "format":
                        case "border":
                            value = container.hashCode[coldata[property]];
                            break;
                        case "value2":
                            value = [ctype.General, ctype.Text].indexOf(container.cellType[coldata.type]) === -1 ? coldata.value2 : container.sharedData[coldata.value2];
                            break;
                        case "type":
                            value = container.cellType[coldata.type];
                            break;
                        case "cFormatRule":
                            var i = 0, j = 0;
                            value = [];
                            !xlObj.isUndefined(coldata[property]) && (j = coldata[property].length);
                            while (i < j) {
                                coldata[property][i] in container.cFormatData && value.push(container.cFormatData[coldata[property][i]]);
                                i++;
                            }
                            break;
                        case "rule":
                            if (!xlObj.isUndefined(coldata[property])) {
                                value = {};
                                $.extend(true, value, coldata[property]);
                                value.customVal = container.valData[value.customVal];
                            }
                            break;                      
                        default:
                            value = coldata[property];
                    }
                }
            }
            return value;
        },

        getPropertyValueByElem: function (elem, property, sheetIdx) {
            if (!ej.isNullOrUndefined(elem[0]))
                return (this.getPropertyValue(elem[0].parentNode.rowIndex, elem[0].cellIndex, property, sheetIdx));
        },

        _refreshCalcEngine: function (rowindex, colindex, isFormula, val) {
            var cellArgs, family, cellRef, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            if (isFormula) {
                cellArgs = new ValueChangedArgs(rowindex + 1, colindex + 1, val);
                xlObj._sheets[sheetIdx]._calcEngine.valueChanged(sheet._sheetName.value, cellArgs);
            }
            else {
                family = CalcEngine.getSheetFamilyItem(sheet._sheetName.value);
                cellRef = RangeInfo.getAlphaLabel(colindex + 1) + (rowindex + 1);
                if (xlObj._sheets[sheetIdx]._calcEngine.isSheetMember() && family.parentObjectToToken != null)
                    cellRef = family.parentObjectToToken.getItem(sheet._sheetName.value) + cellRef;
                xlObj._sheets[sheetIdx]._calcEngine.refresh(cellRef);
            }
        },

        _formulaSelect: function (args) {
            var len = this.model.nameManager.length, i = 0, isName = false, text, value = args.text;
            if (len) {
                while (i < len) {
                    if (this.model.nameManager[i].name === value.slice(1)) {
                        isName = true;
                        break;
                    }
                    i = i + 1;
                }
            }
            text = (!isName ? (value + "(") : value);
            this.getActiveCellElem().find("input").val(text);
            this._getInputBox().val(text);
            this.XLCellNav._canKeyBoardNavigate = false;
        },

        _formulaBoxInputChange: function (e) {
            var sheetIdx = this.getActiveSheetIndex(), acell = this.getSheet(sheetIdx)._activeCell;
            var ielem = this.element.find("#"+this._id+"_Edit");
            this.XLEdit._isFormulaEdit = !e.target.value.indexOf("=");
            this.XLEdit._isFormulaEdit && this.XLEdit._processFormulaEditRange(e.target.value, true);
            if (!this.XLEdit._isEdit) {
                this.XLEdit.editCell(acell.rowIndex, acell.colIndex, true);
                ielem = this.element.find("#" + this._id + "_Edit");
            }
            ielem.text(e.target.value);
            e.target.focus();
            this.XLEdit._isCellEdit = false;
        },

        _formulaSuggestionStatus: function (args) {
            this._isFormulaSuggestion = args.type === "open";
        },

        _updateFormulaCellRange: function (val, ctrlKey) {
            var idx, xlObj = this.XLObj, ielem = xlObj.element.find("#" + xlObj._id + "_Edit").not(":hidden"),
               otext = ielem.text(), lastChar = otext[otext.length - 1];
            if (ctrlKey && this._validCharacters.indexOf(lastChar) === -1)
                otext = otext + ",";
            else {
                idx = this._getCharPosition(otext);
                if (idx > -1)
                    otext = otext.substr(0, [idx + 1]);
            }
            val = otext + val;
            ielem.val(val);
            xlObj._getInputBox().val(val);
        },

        _getCharPosition: function (val) {
            var len = val.length;
            while (len--) {
                if (this._validCharacters.indexOf(val[len]) > -1)
                    return len;
            }
            return -1;
        },

        _processFormulaEditRange: function (val, isInputBox) {
            var str, lastChar, i = 0, xlObj = this.XLObj, parsedVal = xlObj.XLDragFill._parseFormula(val), len = parsedVal.length;
            xlObj._ctrlKeyCount = 0;
            xlObj._formulaRange = [];
            xlObj._ctrlKeyEnabled = false;
            xlObj.XLSelection._clearBorder(xlObj._arrayAsString(xlObj._ctrlFormulaBorder.concat(xlObj._formulaBorder)));
            while (i < len) {
                str = parsedVal[i];
                if (this._invalidOperators.indexOf(str) > -1)
                    break;
                if (xlObj._isCellReference(str)) {
                    if (parsedVal[i + 1] === ":") {
                        i++;
                        if (xlObj._isCellReference(parsedVal[i + 1])) {
                            str = str + ":" + parsedVal[i + 1];
                            i++;
                        }
                    }
                    this._updateFormulaEditRange(str, xlObj._ctrlKeyCount)
                    xlObj._ctrlKeyCount++;
                }
                i++;
            }
            lastChar = val.charAt(val.length - 1);
            if (lastChar === "(" && val.split("(").length === 2) {
                xlObj._isFormulaSuggestion = false;
                xlObj._getInputBox().val(val);
                isInputBox ? $("#" + xlObj._id + "_inputbox_suggestion").hide() : xlObj._getFormulaSuggElem().hide();
            }
            else if (lastChar === ")" && xlObj.isFormula(val)) {
                xlObj.XLEdit._isFormulaEdit = false;
                xlObj._formulaRange = [];
                xlObj._ctrlKeyCount = 0;
            }
            if (lastChar === ":") {
                xlObj._shiftKeyEnabled = true;
                xlObj._ctrlKeyCount--;
            }
            else if (this._validOperators.indexOf(lastChar) > -1) {
                xlObj._dStartCell = xlObj.getSheet(xlObj.getActiveSheetIndex())._activeCell;
                xlObj._ctrlKeyEnabled = true;
                xlObj._shiftKeyEnabled = false;
            }
        },

        _updateFormulaEditRange: function (str, i) {
            var xlObj = this.XLObj, indices = xlObj.getRangeIndices(str);
            xlObj._formulaRange[i] = str;
            xlObj._dStartCell = { rowIndex: indices[0], colIndex: indices[1] };
            xlObj._dEndCell = { rowIndex: indices[2], colIndex: indices[3] };
            xlObj.XLSelection._focusBorder(xlObj._dStartCell, xlObj._dEndCell, xlObj._formulaBorder[i % 6]);
        }
    };

    $.fn.focusEnd = function () {  //http://stackoverflow.com/questions/1181700/set-cursor-position-on-contenteditable-div
        $(this).focus();
        var tmp = $('<span />').appendTo($(this)),
            node = tmp.get(0),
            range = null,
            sel = null;

        if (document.selection) {
            range = document.body.createTextRange();
            range.moveToElementText(node);
            range.select();
        } else if (window.getSelection) {
            range = document.createRange();
            range.selectNode(node);
            sel = window.getSelection();
            sel.removeAllRanges();
            sel.addRange(range);
        }
        tmp.remove();
        return this;
    }

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.validation = function (obj) {
        this.XLObj = obj;
        this._ruleSymbols = { "Greater": ">", "GreaterOrEqual": ">=", "Less": "<", "LessOrEqual": "<=", "Equal": "==", "NotEqual": "!=", "Between": ">=&&<=&&=", "NotBetween": ">=&&<=&&!" };
    };

    ej.spreadsheetFeatures.validation.prototype = {
        //Validation
        _setValidation: function () {
            if (!this.XLObj.model.allowDataValidation || !$.validator)
                return;
            this._initValidator();
            this._setValidationToField();
        },

        _initValidator: function () {
            var elem = $("#" + this.XLObj._id + "EditForm");
            elem.validate({
                errorClass: "e-field-validation-error",
                errorElement: "div",
                wrapper: "div",
                errorPlacement: function (error, ele) {
                    var xlObj = $("#" + $(ele).parents("div")[0].id).data("ejSpreadsheet"), $content = xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-content");
                    var $td = xlObj.XLEdit._editCell, $container = $(error).addClass("e-error");                   
                    $content.append($container);
                    $container.prepend(ej.buildTag("div.e-errortail e-toparrow"));
                    $container.css({ left: $td[0].offsetLeft, top: $td[0].offsetTop + $td.height() });
                }
            });
        },

        _setValidationToField: function () {
            var xlObj = this.XLObj, rules = xlObj.XLEdit.getPropertyValueByElem(xlObj.getActiveCellElem(), "rule");
            delete rules.type;
            xlObj.element.find("#" + xlObj._id + "_ValElem").rules("add", rules);
            var validator = xlObj.element.find("#" + xlObj._id + "EditForm").validate();
            if (!ej.isNullOrUndefined(rules.required)) {
                validator.settings.messages[name] = {};
                validator.settings.messages[name].required = xlObj.XLEdit._EditCellDetails.fieldName + (xlObj.XLEdit._EditCellDetails.rowIndex + 1) + " is required";
            }
        },

        applyDVRules: function (range, values, type, required, showErrorAlert) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), isHLight = false, rng, selected;
            if (ej.isNullOrUndefined(range))
                selected = xlObj.getSheet(sheetIdx)._selectedCells;
            else {
				if (typeof (range) === "string")
                	rng = xlObj.getRangeIndices(range);
                selected = xlObj.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3]})
            }
            if (!xlObj.model.allowDataValidation)
                return;
            var rule = {}, $trgt;
            rule.required = !required;
            switch (type) {
                case "number":
                    values.push("parseInt");
                    rule.digits = true;
                    values.push("number");
                    break;
                case "decimal":
                    values.push("parseFloat");
                    rule.number = true;
                    values.push("decimal");
                    break;
                case "date":
                    values.push("parseDate");
                    rule.date = true;
                    values.push("date");
                    break;
                case "time":
                    values.push("parseTime");
                    values.push("time");
                    break;
                case "text":
                    values[1] = "^" + values[1];
                    values[2] = "^" + values[2];
                    values.push("length");
                    values.push("text");
                    break;
                case "list":
                    var dataSrc = values[1];
                    values = [];
                    values.push(dataSrc);
                    values.push("list");
                    break;
            }
            values.push(showErrorAlert);
            values.push(xlObj._getSelectedItems(sheetIdx, range)[1]);
            values.push(sheetIdx);
            rule.customVal = values.join("_");
            rule.isApply = false;
            rule.type = type;
            var selCells = [], cell = {}, cellObj;
            for (i = 0, len = selected.length; i < len; i++) {
                cell = { rowIndex: selected[i].rowIndex, colIndex: selected[i].colIndex, prevRule: xlObj.XLEdit.getPropertyValue(selected[i].rowIndex, selected[i].colIndex, "rule") };
                if(xlObj._isRowViewable(sheetIdx, selected[i].rowIndex)) {
                    $trgt = xlObj.getCell(selected[i].rowIndex, selected[i].colIndex);
                    $trgt.hasClass("e-hlcell") && (isHLight = true);
                    $trgt.removeData("rule");
                    $trgt.removeClass("e-hlcell");
                }         
                xlObj.XLEdit._updateDataContainer({ rowIndex: selected[i].rowIndex, colIndex: selected[i].colIndex }, { "rule": rule });
                selCells.push(cell);
            }
            if (isHLight === true)
                this.highlightInvalidData();
            if (!xlObj.model.sheets[sheetIdx]._importRendered && !xlObj._isUndoRedo) {
                var details = { sheetIndex: sheetIdx, reqType: "validation", operation: "apply-validation", selectedCells: selCells, range: xlObj._getAlphaRange(sheetIdx, selected[0].rowIndex, selected[0].colIndex, selected[selected.length - 1].rowIndex, selected[selected.length - 1].colIndex) };
                xlObj._completeAction(details);
            }
        },

        clearDV: function (range) {
            var xlObj = this.XLObj, selected;
            if (!xlObj.model.allowDataValidation)
                return;
            var $trgt, selectedCells, sheetIdx = xlObj.getActiveSheetIndex(), details = { sheetIndex: sheetIdx, reqType: "validation", operation: "clear-validation", selCells: [] };
            if (ej.isNullOrUndefined(range)) {
                selected = xlObj.model.sheets[sheetIdx]._selectedCells;
                selectedCells = xlObj.XLSelection._getSelectedRange({ rowIndex: selected[0].rowIndex, colIndex: selected[0].colIndex }, { rowIndex: selected[selected.length - 1].rowIndex, colIndex: selected[selected.length - 1].colIndex });
            }
            else {
                var r = xlObj.getRangeIndices(range);
                selectedCells = xlObj.XLSelection._getSelectedRange({ rowIndex: r[0], colIndex: r[1] }, { rowIndex: r[2], colIndex: r[3] });
            }
            for (var i = 0, len = selectedCells.length; i < len; i++) {
                details.selCells.push({ rowIndex: selectedCells[i].rowIndex, colIndex: selectedCells[i].colIndex, rules: xlObj.XLEdit.getPropertyValue(selectedCells[i].rowIndex , selectedCells[i].colIndex, "rule") });
                xlObj.clearRangeData(null, ["rule"], [{ rowIndex: selectedCells[i].rowIndex, colIndex: selectedCells[i].colIndex }]);
                if (xlObj._isRowViewable(selectedCells[i].rowIndex)) {
                    $trgt = xlObj.getCell(selectedCells[i].rowIndex, selectedCells[i].colIndex);
                    $trgt.removeClass("e-hlcell");
                }
            }
			details.range = xlObj._getAlphaRange(sheetIdx, selectedCells[0].rowIndex, selectedCells[0].colIndex, selectedCells[selectedCells.length - 1].rowIndex, selectedCells[selectedCells.length - 1].colIndex);
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        clearHighlightedValData: function (range) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowDataValidation)
                return;
            var selected, trgt, sheetIdx = xlObj.getActiveSheetIndex();
            if (ej.isNullOrUndefined(range))
                selected = xlObj.model.sheets[sheetIdx]._selectedCells;       
            else {
                var r = xlObj.getRangeIndices(range);
                selected = xlObj.XLSelection._getSelectedRange({ rowIndex: r[0], colIndex: r[1] }, { rowIndex: r[2], colIndex: r[3] });
            }
            for (var i = 0, len = selected.length; i < len; i++) {
                if (xlObj._isRowViewable(sheetIdx, selected[i].rowIndex)) {
                    trgt = xlObj.getCell(selected[i].rowIndex, selected[i].colIndex);
                    trgt.removeClass("e-hlcell");
                }
                if (!ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(0, 8, "rule")))
                    xlObj._dataContainer.sheets[sheetIdx][selected[i].rowIndex][selected[i].colIndex].rule.isApply = false;
            }
            if (!xlObj._isUndoRedo) {
                var details = { sheetIndex: sheetIdx, reqType: "validation", operation: "cl-hl-data", range: (ej.isNullOrUndefined(range)) ? this.XLObj._getAlphaRange(1, selected[0].rowIndex, selected[0].colIndex, selected[selected.length - 1].rowIndex, selected[selected.length - 1].colIndex) : range };
                xlObj._completeAction(details);
            }
        },

        _checkValidation: function (value, params) {
            var xlObj = this.XLObj, len, listVal;
            if (!JSON.parse(params[params.length - 3]))
                return true;
            value = this._convertValueByType(value, params[4]);
            params[1] = this._convertValueByType(params[1], params[4]);
            params[2] = this._convertValueByType(params[2], params[4]);
            if (value.length) {
                if (params[params.length - 4] === "list") {
                    var i, isMatch = false;
                    if (params[0].indexOf(",") > -1)
                        listVal = params[0].split(",");
                    else if (params[0].indexOf("=") > -1)
                        listVal = xlObj.getRangeData(xlObj.getRangeIndices(params[0].split("=")[1]), true);
                    else
                        listVal = value;
                    for (i = 0, len = listVal.length; i < len; i++) {
                        isMatch = listVal[i].toString() === value;
                        if (isMatch)
                            break;
                    }
                    ej.Spreadsheet.msg = xlObj._getLocStr("ListAlertMsg");
                    return isMatch;
                }
                else {
                    var val, range;
                    params[0] = this._ruleSymbols[params[0]];
                    if (params[1].indexOf("=") > -1) {
                        val = params[1].replace("=", "");
                        range = this.getRangeIndices(val + ":" + val);
                        params[1] = xlObj.XLEdit.getPropertyValue(range[0], range[1]).toString();
                    }
                    if (params[2].indexOf("=") > -1) {
                        val = params[2].replace("=", "");
                        range = this.getRangeIndices(val + ":" + val);
                        params[2] = xlObj.XLEdit.getPropertyValue(range[0], range[1]).toString();
                    }
                    if (params[0].length <= 2) {
                        ej.Spreadsheet.msg = "The " + params[4] + " should be " + ej.Spreadsheet.ValidationText[params[0]] + " " + params[1].replace("^", "");
                        return xlObj.operators[params[0]](xlObj.parse[params[3]](value), xlObj.parse[params[3]](params[1]));
                    }
                    else {
                        var op = [];
                        op = params[0].split("&&");
                        ej.Spreadsheet.msg = "The " + params[4] + " should" + ej.Spreadsheet.ValidationText[op[2]] + "be between" + " " + params[1].replace("^", "") + " and " + params[2].replace("^", "");
                        return xlObj.operators[op[2]]((xlObj.operators[op[0]](xlObj.parse[params[3]](value), xlObj.parse[params[3]](params[1])) && xlObj.operators[op[1]](xlObj.parse[params[3]](value), xlObj.parse[params[3]](params[2]))));
                    }
                }
            }
            else {
                return true;
            }
        },

        _convertValueByType: function (value, type) {
            if (value.length) {
                var xlObj = this.XLObj;
                switch (type) {
                    case "number":
                    case "decimal":
                        if (xlObj._isValidDate(value))
                            value = xlObj._dateToInt(value);
                        break;
                    case "date":
                        if ($.isNumeric(value))
                            value = xlObj.intToDate(value).toString();
                        break;
                }
            }
            return value;
        },

        highlightInvalidData: function (range) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowDataValidation)
                return;
            var len, value, selected, cell, isApply, i, r, sheetIdx = xlObj.getActiveSheetIndex(), rule;
            if (ej.isNullOrUndefined(range)) {
                selected = xlObj.model.sheets[sheetIdx]._selectedCells;
                range = xlObj._getAlphaRange(sheetIdx, selected[0].rowIndex, selected[0].colIndex, selected[selected.length - 1].rowIndex, selected[selected.length - 1].colIndex);
            }
            else {
                r = xlObj.getRangeIndices(range);
                selected = xlObj.XLSelection._getSelectedRange({ rowIndex: r[0], colIndex: r[1] }, { rowIndex: r[2], colIndex: r[3] });
            }
            for (i = 0, len = selected.length; i < len; i++) {
                rule = xlObj.XLEdit.getPropertyValue(selected[i].rowIndex, selected[i].colIndex, "rule");
                if (xlObj.XLEdit.getPropertyValue(selected[i].rowIndex, selected[i].colIndex) && !ej.isNullOrUndefined(rule)) {
                    value = xlObj.XLEdit.getPropertyValue(selected[i].rowIndex, selected[i].colIndex, "type") === "datetime" ? xlObj.XLEdit.getPropertyValue(selected[i].rowIndex, selected[i].colIndex, "value2").toString() : xlObj.XLEdit.getPropertyValue(selected[i].rowIndex, selected[i].colIndex).toString();
                    isApply = rule ? !(this._checkValidation(value, rule.customVal.split("_"))) : false;
                    xlObj._dataContainer.sheets[sheetIdx][selected[i].rowIndex][selected[i].colIndex].rule.isApply = isApply;
                }
                else if (!ej.isNullOrUndefined(rule))
                    isApply = rule.required;
                if(xlObj._isRowViewable(sheetIdx, selected[i].rowIndex)){
                    cell = xlObj.getCell(selected[i].rowIndex, selected[i].colIndex);
                    if (isApply)
                        cell.addClass("e-hlcell");
                    else
                        cell.removeClass("e-hlcell");
                }
            }
            if (!xlObj._isUndoRedo) {
                var details = { sheetIndex: sheetIdx, reqType: "validation", operation: "hl-data", range: range };
                xlObj._completeAction(details);
            }
        },

        _replaceRule: function (idx, newRule, range) {
            // replace the rules in shared data
            this.XLObj._dataContainer.cFormatData[idx] = newRule;
        }
    };

    !ej.isNullOrUndefined($.validator) && $.validator.addMethod("customVal", function (value, elem, params) {        
        var xlObj = $("#" + $(elem).parents("div")[0].id).data("ejSpreadsheet"), $content = xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-content");
        $content.find(".e-error").remove();
        if (!$("#" + xlObj._id + "_Ribbon_freetext").data("ejCheckBox").isChecked())
            return true;
        return xlObj.XLValidate._checkValidation(value, params.split("_"));
    }, ej.Spreadsheet.getMsg);

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.dragFill = function (obj) {
        this.XLObj = obj;
        this._customList = [
            ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
            ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"],
            ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
            ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
        ];
        this._fillInfo = null;
        this._verticalFill = false;
        this._uniqueOBracket = String.fromCharCode(129);
        this._uniqueCBracket = String.fromCharCode(130);
        this._uniqueCSeparator = String.fromCharCode(131);
        this._uniqueCOperator = String.fromCharCode(132);
        this._uniquePOperator = String.fromCharCode(133);
        this._uniqueSOperator = String.fromCharCode(134);
        this._uniqueMOperator = String.fromCharCode(135);
        this._uniqueDOperator = String.fromCharCode(136);
        this._uniqueModOperator = String.fromCharCode(137);
    };

    ej.spreadsheetFeatures.dragFill.prototype = {
        positionAutoFillElement: function (isdragfill) {
            var mergeIdx, rowIdx, colIdx, cellInfo, hdrInfo, sheetIdx, sheet, top = 0, left = 0, vdiff = 0, hdiff = 0,
                tdiff = 0, ldiff = 0, otdiff = 0, oldiff = 0, xlObj = this.XLObj, hide = "e-hide";
            if (!xlObj.model.allowAutoFill || !xlObj.model.allowSelection)
                return;
            sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            if (sheet._isRowSelected) {
                tdiff = -3;
                ldiff = -1;
                otdiff = 5;
                oldiff = 4;
                rowIdx = sheet.selectedRange[2];
                colIdx = sheet.selectedRange[1];
            }
            else if (sheet._isColSelected) {
                ldiff = -3;
                oldiff = 5;
                rowIdx = sheet.selectedRange[0];
                colIdx = sheet.selectedRange[3];
            }
            else if (sheet._isSheetSelected) {
                this.hideAutoFillElement();
                return;
            }
            else {
                tdiff = -3;
                ldiff = -3;
                otdiff = 1;
                oldiff = 3;
                rowIdx = sheet.selectedRange[2];
                colIdx = sheet.selectedRange[3];
            }
            mergeIdx = xlObj._getMergedIdx(rowIdx, colIdx);
            rowIdx = mergeIdx.rowIndex;
            colIdx = mergeIdx.colIndex;
            contInfo = xlObj._getContent(sheetIdx).offset();
            cellInfo = xlObj._getCellInfo({ rowIndex: rowIdx, colIndex: colIdx });
            vdiff = cellInfo.top - sheet._scrollTop;
            hdiff = cellInfo.left - sheet._scrollLeft;
            if (!sheet._isColSelected)
                top += cellInfo.height;
            if (!sheet._isRowSelected)
                left += cellInfo.width;
            top += contInfo.top + vdiff;
            left += contInfo.left + hdiff;
            xlObj.getAutoFillElem().removeClass(hide).offset({ top: Math.round(top) + tdiff, left: Math.round(left) + ldiff });
            if (xlObj.model.autoFillSettings.showFillOptions && isdragfill)
                xlObj._getAutoFillOptElem().removeClass(hide).offset({ top: top + otdiff, left: left + oldiff });
            sheet._autoFillCell = { rowIndex: rowIdx, colIndex: colIdx };
        },

        hideAutoFillElement: function () {
            var elem = this.XLObj._getAutoFillOptElem();
            if (elem)
                elem.addClass("e-hide");
        },

        hideAutoFillOptions: function () {
            var elem = this.XLObj._getAutoFillOptElem();
            if (elem) {
                elem.find("button").data("ejSplitButton")._hidePopup();
                elem.addClass("e-hide");
            }
        },

        autoFill: function (options) {
            options = options || {};
            if (!this.XLObj.model.allowAutoFill || !options.dataRange || !options.fillRange || !options.direction)
                return;
            var details, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), aOpt = ej.Spreadsheet.AutoFillOptions;
            this._fillInfo = this._getFillInfo(options);
            options.fillType = options.fillType || this._fillInfo.fillType;
            this._verticalFill = (options.direction === "down" || options.direction === "up");
            options.sheetIndex = sheetIdx;
            details = { sheetIndex: options.sheetIndex, reqType: "auto-fill", range: options.fillRange, bRange: options.dataRange, bData: xlObj.getRangeData(options.fillRange), bFormat: xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(options.fillRange) };
            details.rowSel = sheet._isRowSelected;
            details.colSel = sheet._isColSelected;
            if (xlObj._trigger("autoFillBegin", options))
                return;
            switch (options.fillType) {
                case aOpt.FillSeries:
                case aOpt.FillWithoutFormatting:
                    this._fillSeries(options);
                    break;
                case aOpt.CopyCells:
                case aOpt.FillFormattingOnly:
                    this._copyCells(options);
                    break;
                case aOpt.FlashFill:
                    details.actionType = "flash-fill";
                    details.bdData = xlObj.getRangeData(options.dataRange);
                    this._flashFill(options);
                    details.adData = xlObj.getRangeData(options.dataRange);
                    break;
            }
            details.aData = xlObj.getRangeData(options.fillRange);
            if (xlObj.model.allowCellFormatting)
                details.aFormat = xlObj.XLFormat.getHashCodeClassAsArray(options.fillRange);
            xlObj._completeAction(details);
            xlObj._trigger("autoFillComplete", options);
        },

        _fillSeries: function (options) {
            var val, plen, patterns, patrn, patrnRange, temp, j, k, tlen, tot, hasRef, format, flen, pStart, pEnd, fStart, fEnd,
                cells, clen, i = 0, xlObj = this.XLObj, pos = ej.Spreadsheet.autoFillDirection, ctype = ej.Spreadsheet.CellType,
                dminr = options.dataRange[0], dminc = options.dataRange[1], dmaxr = options.dataRange[2], dmaxc = options.dataRange[3],
                fminr = options.fillRange[0], fminc = options.fillRange[1], fmaxr = options.fillRange[2], fmaxc = options.fillRange[3],
                isVFill = [pos.Down, pos.Up].indexOf(options.direction) > -1, isRFill = [pos.Up, pos.Left].indexOf(options.direction) > -1,
                len = isVFill ? dmaxc - dminc : dmaxr - dminr, withFrmt = options.fillType === ej.Spreadsheet.AutoFillOptions.FillSeries;
            while (i <= len) {
                if (isVFill) {
                    pStart = { rowIndex: dminr, colIndex: dminc + i };
                    pEnd = { rowIndex: dmaxr, colIndex: dminc + i };
                    fStart = { rowIndex: fminr, colIndex: fminc + i };
                    fEnd = { rowIndex: fmaxr, colIndex: fminc + i };
                }
                else {
                    pStart = { rowIndex: dminr + i, colIndex: dminc };
                    pEnd = { rowIndex: dminr + i, colIndex: dmaxc };
                    fStart = { rowIndex: fminr + i, colIndex: fminc };
                    fEnd = { rowIndex: fminr + i, colIndex: fmaxc };
                }
                patrnRange = [pStart.rowIndex, pStart.colIndex, pEnd.rowIndex, pEnd.colIndex];
                patterns = this._getPattern(patrnRange, { isRFill: isRFill, isVFill: isVFill });
                if (!patterns)
                    return;
                plen = patterns.length;
                if (withFrmt) {
                    if (xlObj.model.allowCellFormatting)
                        xlObj.XLFormat.clearFormat([fStart.rowIndex, fStart.colIndex, fEnd.rowIndex, fEnd.colIndex]);
                    format = xlObj.getRangeData(patrnRange, null, ["format"]);
                    flen = format.length;
                }
                cells = xlObj.getRange(fStart.rowIndex, fStart.colIndex, fEnd.rowIndex, fEnd.colIndex);
                clen = cells.length;
                if (isRFill) {
                    cells = cells.toArray().reverse();
                    patterns = patterns.reverse();
                    patterns = this._ensurePattern(patterns);
                    if (withFrmt)
                        format = format.reverse();
                }                
                j = 0;
                while (j < clen) {
                    patrn = patterns[j % plen];
                    if (xlObj.isNumber(patrn))
                        patrn = patterns[patrn];
                    switch (patrn.type) {
                        case ctype.Number:
                            val = xlObj._round(patrn.regVal.a + (patrn.regVal.b * patrn.i), 5);
                            if (isRFill)
                                patrn.i--;
                            else
                                patrn.i++;
                            break;
                        case ctype.String:
                            val = patrn.val[patrn.i % patrn.val.length];
                            patrn.i++;
                            break;
                        case ctype.Formula:
                            hasRef = false;
                            val = "=";
                            k = 0;
                            tlen = patrn.val.length;
                            while (k < tlen) {
                                temp = patrn.val[k];
                                if (xlObj._isObject(temp)) {
                                    hasRef = true;
                                    tot = xlObj._round(temp.a + (temp.b * patrn.i), 0);
                                    if (tot < 1)
                                        val += "#REF!";
                                    else
                                        val += isVFill ? temp.c + tot : xlObj._generateHeaderText(options.sheetIdx, tot) + temp.c;
                                }
                                else
                                    val += temp;
                                k++;
                            }
                            if (hasRef && isRFill)
                                patrn.i--;
                            else
                                patrn.i++;
                            break;
                        case ctype.Custom:
                            val = xlObj._round(patrn.regVal.a + (patrn.regVal.b * patrn.i), 0);
                            if (val < 0)
                                val = (val % patrn.len) + patrn.len;
                            if (val >= patrn.len)
                                val = val % patrn.len;
                            val = patrn.val[val];
                            if (isRFill)
                                patrn.i--;
                            else
                                patrn.i++;
                            break;
                    }
                    xlObj.XLEdit.updateCellValue($(cells[j]), val, withFrmt ? format[j % flen].format : null);
                    if (xlObj.model.allowComments)
                        xlObj.XLComment._updateCmntFrmCntnr(null, null, $(cells[j]));
                    j++;
                }
                i++;
            }
        },

        _ensurePattern: function (patterns) {
            var patrn, idx = -1, i = patterns.length;
            while (i--) {
                patrn = patterns[i];
                if (this.XLObj._isObject(patrn)) {
                    idx = i;
                    if (patrn.type === ej.Spreadsheet.CellType.String)
                        patrn.val = patrn.val.reverse();
                }
                else
                    patterns[i] = idx;
            }
            return patterns;
        },

        _copyCells: function (options) {
            var i, j, k, l, patrnRange, fStart, fEnd, pStart, pEnd, data, flen, format, cells, clen, xlObj = this.XLObj, i = 0, pos = ej.Spreadsheet.autoFillDirection,
                dminr = options.dataRange[0], dminc = options.dataRange[1], dmaxr = options.dataRange[2], dmaxc = options.dataRange[3],
                fminr = options.fillRange[0], fminc = options.fillRange[1], fmaxr = options.fillRange[2], fmaxc = options.fillRange[3],
                isVFill = [pos.Down, pos.Up].indexOf(options.direction) > -1, isRFill = [pos.Up, pos.Left].indexOf(options.direction) > -1,
                len = isVFill ? dmaxc - dminc : dmaxr - dminr, formatOnly = options.fillType === ej.Spreadsheet.AutoFillOptions.FillFormattingOnly;
            while (i <= len) {
                if (isVFill) {
                    pStart = { rowIndex: dminr, colIndex: dminc + i };
                    pEnd = { rowIndex: dmaxr, colIndex: dminc + i };
                    fStart = { rowIndex: fminr, colIndex: fminc + i };
                    fEnd = { rowIndex: fmaxr, colIndex: fminc + i };
                }
                else {
                    pStart = { rowIndex: dminr + i, colIndex: dminc };
                    pEnd = { rowIndex: dminr + i, colIndex: dmaxc };
                    fStart = { rowIndex: fminr + i, colIndex: fminc };
                    fEnd = { rowIndex: fminr + i, colIndex: fmaxc };
                }
                patrnRange = [pStart.rowIndex, pStart.colIndex, pEnd.rowIndex, pEnd.colIndex];
                if (xlObj.model.allowCellFormatting)
                    xlObj.XLFormat.clearFormat([fStart.rowIndex, fStart.colIndex, fEnd.rowIndex, fEnd.colIndex]);
                if (!formatOnly)
                    data = xlObj.getRangeData(patrnRange, true, null, null, true);
                format = xlObj.getRangeData(patrnRange, null, ["format"]);
                flen = format.length;
                cells = xlObj.getRange(fStart.rowIndex, fStart.colIndex, fEnd.rowIndex, fEnd.colIndex);
                clen = cells.length;
                j = 0;
                if (isRFill) {
                    cells = cells.toArray().reverse();
                    format = format.reverse();
                    if (!formatOnly)
                        data = data.reverse();
                }
                if (formatOnly) {
                    while (j < clen) {
                        if (xlObj.model.allowCellFormatting)
                            xlObj.XLFormat._updateFormatClass(cells[j], format[j % flen].format);
                        j++;
                    }
                }
                else {
                    while (j < clen) {
                        var k = j % flen;
                        xlObj.XLEdit.updateCellValue($(cells[j]), data[k], format[k].format);
                        if (xlObj.model.allowComments)
                            xlObj.XLComment._updateCmntFrmCntnr(null, null, $(cells[j]));
                        j++;
                    }
                }
                i++;
            }
        },

        _flashFill: function (options) {
            var cells, data, i, xlObj = this.XLObj, dminr = options.dataRange[0], dmaxr = options.dataRange[2], fminr = options.fillRange[0];
            var fminc = options.fillRange[1], fmaxr = options.fillRange[2], fmaxc = options.fillRange[3], patrn = this._getFlashFillPattern(options.dataRange, "left");
            if (patrn) {
                data = xlObj.getRangeData([fminr, patrn.colIndex, fmaxr, patrn.colIndex], true, null, null, true);
                if (patrn.subStr > -1)
                    data = this._splitString(data, " ", patrn.subStr);
                cells = xlObj.getRange(fminr, fminc, fmaxr, fmaxc);
                i = cells.length;
                while (i--)
                    xlObj.XLEdit.updateCellValue($(cells[i]), data[i]);
            }
            else {
                if (fminr > dminr)
                    fminr = dminr;
                if (fmaxr < dmaxr)
                    fmaxr = dmaxr;
                xlObj._getAutoFillOptElem().addClass("e-hide");
                xlObj.updateUniqueData({ value: "", value2: "", type: "general" }, [fminr, fminc, fmaxr, fmaxc]);
                xlObj._showAlertDlg("Alert", "FlashFillAlert");
            }
        },

        _getFlashFillPattern: function (range, move) {
            var resp, value, splval, slen, j, i = 0, subStr = -1, xlObj = this.XLObj, data = xlObj.getRangeData(range, true, null, null, true),
                dlen = data.length, minr = range[0], right = ej.Spreadsheet.autoFillDirection.Right, isRight = move === right;
            if (!data.join().replace(/,/g, '').length)
                return;
            var val = data[0].toString();
            i = range[1];
            if (i) {
                while (isRight ? i++ : i--) {
                    value = xlObj.XLEdit.getPropertyValue(minr, i);
                    value = ej.isNullOrUndefined(value) ? "" : value.toString();
                    if (!value.length)
                        break;
                    else {
                        if (value === val) {
                            if (dlen > 1) {
                                resp = this._matchData(minr, i, data);
                                if (resp)
                                    return resp;
                            }
                            else
                                return { colIndex: i, subStr: subStr };
                        }
                        splval = value.split(" ");
                        slen = splval.length;
                        j = 0;
                        while (j < slen) {
                            if (val === splval[j]) {
                                subStr = j;
                                if (dlen > 1) {
                                    resp = this._matchData(minr, i, data);
                                    if (resp) {
                                        if (subStr > resp.subStr)
                                            resp.subStr = subStr;
                                        return resp;
                                    }
                                }
                                else
                                    return { colIndex: i, subStr: subStr };
                            }
                            j++;
                        }
                    }
                }
                if (!isRight)
                    return this._getFlashFillPattern(range, right);
            }
        },

        _matchData: function (rowIdx, colIdx, data) {
            var j, match = false, val, value, splval, slen, i = 1, subStr = -1, dlen = data.length;
            while (i < dlen) {
                match = false;
                val = data[i].toString();
                value = this.XLObj.XLEdit.getPropertyValue(rowIdx + i, colIdx);
                value = ej.isNullOrUndefined(value) ? "" : value.toString();
                if (val === value) {
                    match = true;
                    i++;
                    continue;
                }
                splval = value.split(" ");
                slen = splval.length;
                j = 0;
                while (j < slen) {
                    if (val === splval[j]) {
                        if (subStr < j)
                            subStr = j;
                        match = true;
                        break;
                    }
                    j++;
                }
                i++;
            }
            if (match && i === dlen)
                return { colIndex: colIdx, subStr: subStr };
        },

        _splitString: function (array, splitter, index) {
            var data, val, i = array.length, j;
            while (i--) {
                j = index;
                data = array[i].split(splitter);
                while (j > -1) {
                    val = data[j];
                    if (val)
                        break;
                    j--;
                }
                array[i] = val;
            }
            return array;
        },

        _getPattern: function (range, options) {
            var j, idx, temp, regVal, custColl, lCaseColl, lCaseVal, isLCase, diff, len, i = 0, pattern = [], xlObj = this.XLObj,
                ctype = ej.Spreadsheet.CellType, patrns = this._getDataPattern(range), plen = patrns.length, patrn;
            if (patrns) {
                while (i < plen) {
                    patrn = patrns[i];
                    switch (patrn.type) {
                        case ctype.Number:
                            idx = pattern.length;
                            len = patrn.val.length;
                            diff = options.isRFill ? -1 : len;
                            if (len === 1)
                                patrn.val.push(patrn.val[0] + 1);
                            regVal = this._getPredictionValue(patrn.val);
                            temp = { regVal: regVal, type: patrn.type, i: diff };
                            pattern.push(temp);
                            j = 1;
                            while (j < len) {
                                pattern.push(idx);
                                j++;
                            }
                            break;
                        case ctype.String:
                            idx = pattern.length;
                            temp = { val: patrn.val, type: patrn.type, i: 0 };
                            pattern.push(temp);
                            j = 1;
                            len = patrn.val.length;
                            while (j < len) {
                                pattern.push(idx);
                                j++;
                            }
                            break;
                        case ctype.Formula:
                            len = patrn.val.length;
                            patrn = this._getFormulaPattern(patrn.val, options);
                            diff = options.isRFill ? -1 : len;
                            if (patrn.isInPattern) {
                                idx = pattern.length;
                                temp = { val: patrn.val, type: ctype.Formula, i: diff };
                                pattern.push(temp);
                                j = 1;
                                while (j < len) {
                                    pattern.push(idx);
                                    j++;
                                }
                            }
                            else {
                                j = 0;
                                diff = options.isRFill ? -1 : 1;
                                while (j < len) {
                                    pattern.push({ val: patrn.val[j], type: ctype.Formula, i: diff });
                                    j++;
                                }
                            }
                            break;
                        default:
                            if (patrn.type.indexOf(ctype.Custom) > -1) {
                                idx = pattern.length;
                                len = patrn.val.length;
                                diff = options.isRFill ? -1 : len;
                                custColl = this._customList[Number(patrn.type.replace(ctype.Custom, ''))];
                                isLCase = custColl.indexOf(patrn.val[0]) === -1;
                                lCaseColl = xlObj.toArrayLowerCase(custColl.slice(0));
                                lCaseVal = xlObj.toArrayLowerCase(patrn.val);
                                regVal = this._getCustomPredictionValue(lCaseVal, lCaseColl);
                                temp = { val: isLCase ? lCaseColl : custColl, regVal: regVal, i: diff, type: ctype.Custom, len: custColl.length };
                                pattern.push(temp);
                                j = 1;
                                while (j < len) {
                                    pattern.push(idx);
                                    j++;
                                }
                            }
                    }
                    i++;
                }
                return pattern;
            }
        },

        _getCustomPredictionValue: function (data, coll) {
            var i = 0, temp = [], len = data.length;
            while (i < len) {
                temp.push(coll.indexOf(data[i]));
                i++;
            }
            if (temp.length === 1)
                temp.push(temp[0] + 1);
            return this._getPredictionValue(temp);
        },

        _getDataPattern: function (range) {
            var val, type, i = 0, obj = {}, patrn = [], data = this.XLObj.getRangeData(range, true, null, null, true), dlen = data.length;
            if (dlen) {
                while (i < dlen) {
                    val = data[i];
                    type = this._getType(val);
                    if (i === 0)
                        obj = { val: [val], type: type };
                    else if (type === obj.type)
                        obj.val.push(val);
                    else {
                        patrn.push(obj);
                        obj = { val: [val], type: type };
                    }
                    i++;
                }
                patrn.push(obj);
                return patrn;
            }
        },

        _getFormulaPattern: function (data, options) {
            var patrn, j, temp, xlObj = this.XLObj, patrns = [], i = 0, len = data.length;
            while (i < len) {
                patrns.push(this._parseFormula(data[i]));
                i++;
            }
            var isInPatrn = this._isInPattern(patrns, options.isVFill);
            if (isInPatrn) {
                patrn = patrns[0];
                i = patrn.length;
                while (i--) {
                    temp = patrn[i];
                    if (xlObj._isCellReference(temp))
                        patrn[i] = this._getCellRefPrediction(temp, options);
                }
                return { isInPattern: isInPatrn, val: patrn };
            }
            else {
                i = 0;
                while (i < len) {
                    patrn = patrns[i];
                    j = patrn.length;
                    while (j--) {
                        temp = patrn[j];
                        if (xlObj._isCellReference(temp))
                            patrns[i][j] = this._getCellRefPrediction(temp, options, len);
                    }
                    i++;
                }
                return { isInPattern: isInPatrn, val: patrns };
            }
        },

        _getCellRefPrediction: function (text, options, length) {
            text = text.toUpperCase();
            var arr = [], aRegx = new RegExp("[a-z]", "gi"), nRegx = new RegExp("[0-9]", "g"), eStr = "",
                str = options.isVFill ? text.replace(nRegx, eStr) : text.replace(aRegx, eStr),
                temp = options.isVFill ? Number(text.replace(aRegx, eStr)) : this.XLObj._generateColCount(text.replace(nRegx, eStr));
            if (length && length !== 1) {
                arr.push(temp);
                arr.push(temp + length);
            }
            else {
                arr.push(temp);
                arr.push(temp + 1);
            }
            temp = this._getPredictionValue(arr);
            temp.c = str;
            return temp;
        },

        _isInPattern: function (patrn, isVFill) {
            var oldPatrn, olen, newPatrn, nlen, oldStr, newStr, oldInt, newInt, xlObj = this.XLObj, eStr = "",
                i = 0, j = 1, plen = patrn.length, nregx = new RegExp("[0-9]", "g"), aregx = new RegExp("[a-z]", "gi");
            if (plen === 1)
                return false;
            while (j < plen) {
                oldPatrn = patrn[i];
                newPatrn = patrn[j];
                olen = oldPatrn.length;
                nlen = newPatrn.length;
                if (olen !== nlen)
                    return false;
                else {
                    while (olen--) {
                        oldStr = oldPatrn[olen];
                        newStr = newPatrn[olen];
                        if (xlObj._isCellReference(oldStr) === xlObj._isCellReference(newStr)) {
                            if (isVFill) {
                                oldInt = Number(oldStr.replace(aregx, eStr));
                                newInt = Number(newStr.replace(aregx, eStr));
                            }
                            else {
                                oldInt = xlObj._generateColCount(oldStr.replace(nregx, eStr));
                                newInt = xlObj._generateColCount(newStr.replace(nregx, eStr));
                            }
                            if (oldInt !== newInt - 1)
                                return false;
                        }
                        else if (oldStr !== newStr)
                            return false;
                    }
                }
                i++;
                j++;
            }
            return true;
        },

        _parseFormula: function (formula) {
            var temp, str, i = 0, arr = [];
            formula = this._markSpecialChar(formula.replace("=", ""));
            formula = formula.split(/\(|\)|,|:|\+|-|\*|\/|%/g);
            var len = formula.length;
            while (i < len) {
                temp = formula[i];
                if (!temp) {
                    i++;
                    continue;
                }
                if (temp.length === 1)
                    arr.push(this._isUniqueChar(temp) ? this._getUniqueCharVal(temp) : temp);
                else {
                    str = temp[0];
                    if (this._isUniqueChar(str)) {
                        arr.push(this._getUniqueCharVal(str));
                        arr.push(temp.substr(1));
                    }
                    else
                        arr.push(temp);
                }
                i++;
            }
            return arr;
        },

        _getUniqueCharVal: function (str) {
            switch (str) {
                case this._uniqueOBracket:
                    return "(";
                case this._uniqueCBracket:
                    return ")";
                case this._uniqueCSeparator:
                    return ",";
                case this._uniqueCOperator:
                    return ':';
                case this._uniquePOperator:
                    return "+";
                case this._uniqueSOperator:
                    return "-";
                case this._uniqueMOperator:
                    return "*";
                case this._uniqueDOperator:
                    return "/";
                case this._uniqueModOperator:
                    return "%";
            }
            return "";
        },

        _isUniqueChar: function (str) {
            var code = str.charCodeAt(str);
            return code >= 129 && code <= 137;
        },

        _markSpecialChar: function (formula) {
            formula = formula.replace(/\(/g, "(" + this._uniqueOBracket).replace(/\)/g, ")" + this._uniqueCBracket);
            formula = formula.replace(/,/g, "," + this._uniqueCSeparator).replace(/:/g, ":" + this._uniqueCOperator);
            formula = formula.replace(/\+/g, "+" + this._uniquePOperator).replace(/-/g, "-" + this._uniqueSOperator);
            formula = formula.replace(/\*/g, "*" + this._uniqueMOperator).replace(/\//g, "/" + this._uniqueDOperator);
            return formula.replace(/%/g, "%" + this._uniqueModOperator);
        },

        _getPredictionValue: function (args) {
            var i = 0, sumx = 0, sumy = 0, sumxy = 0, sumxx = 0, n = args.length;
            while (i < n) {
                sumx = sumx + i;
                sumy = sumy + Number(args[i]);
                sumxy = sumxy + (i * Number(args[i]));
                sumxx = sumxx + (i * i);
                i++;
            }
            var a = this.XLObj._round(((sumy * sumxx) - (sumx * sumxy)) / ((n * sumxx) - (sumx * sumx)), 5), b = this.XLObj._round(((n * sumxy) - (sumx * sumy)) / ((n * sumxx) - (sumx * sumx)), 5);
            return { a: a, b: b };
        },

        _getType: function (val) {
            var type = this._isCustomType(val), ctype = ej.Spreadsheet.CellType, xlObj = this.XLObj;
            if (type)
                type = type;
            else if (xlObj.isFormula(val))
                type = ctype.Formula;
            else if (xlObj.isNumber(val))
                type = ctype.Number;
            return type || ctype.String;
        },

        _isCustomType: function (val) {
            val = val + "";
            val = val.toLowerCase();
            var i = this._customList.length;
            while (i--) {
                if (this.XLObj.toArrayLowerCase(this._customList[i].slice(0)).indexOf(val) > -1)
                    return ej.Spreadsheet.CellType.Custom + i;
            }
            return false;
        },

        _selectAutoFillRange: function (trgt, idx) {
            if (trgt)
                idx = this.XLObj._getCellIdx(trgt);
            var i, xlObj = this.XLObj, range = this._getAutoFillRange(idx);
            if (!range)
                return;
            this._autoFillCleanUp();
            var cells = xlObj.getRange(range.startCell.rowIndex, range.startCell.colIndex, range.endCell.rowIndex, range.endCell.colIndex);
            i = cells.length;
            while (i--)
                xlObj.addClass(cells[i], "e-autofillcell");
            xlObj.XLSelection._focusRange(range.startCell, range.endCell, xlObj._autofillBorder);
        },

        _getAutoFillRange: function (idx) {
            var dir = ej.Spreadsheet.autoFillDirection, xlObj = this.XLObj, sheet = this.XLObj.getSheet(this.XLObj.getActiveSheetIndex()), scell = sheet._startCell, ecell = sheet._endCell, acell = sheet._autoFillCell,
                range = sheet.selectedRange, minr = range[0], minc = range[1], maxr = range[2], maxc = range[3], inRange = xlObj.inRange(range, idx.rowIndex, idx.colIndex),
                minIdx = { rowIndex: minr, colIndex: minc }, maxIdx = { rowIndex: maxr, colIndex: maxc };
            if (idx.rowIndex < acell.rowIndex) {// up 
                if ((minr - idx.rowIndex > idx.colIndex - maxc) && (minr - idx.rowIndex > minc - idx.colIndex))
                    return inRange ? { startCell: minIdx, endCell: { rowIndex: idx.rowIndex, colIndex: maxc } } : { startCell: maxIdx, endCell: { rowIndex: idx.rowIndex, colIndex: minc }, fillRange: [idx.rowIndex, minc, minr - 1, maxc], direction: dir.Up };
               else if (idx.colIndex > acell.colIndex )
                    return { startCell: minIdx, endCell: { rowIndex: maxr, colIndex: idx.colIndex }, fillRange: [minr, maxc + 1, maxr, idx.colIndex], direction: dir.Right };
                else if(idx.colIndex < acell.colIndex )
                    return inRange ? { startCell: minIdx, endCell: maxIdx } : { startCell: maxIdx, endCell: { rowIndex: minr, colIndex: idx.colIndex }, fillRange: [minr, idx.colIndex, maxr, minc - 1], direction: dir.Left };
              
            }
            else if (idx.colIndex > acell.colIndex) {// right
                if ((idx.rowIndex - maxr > idx.colIndex - maxc))
                    return { startCell: minIdx, endCell: { rowIndex: idx.rowIndex, colIndex: maxc }, fillRange: [maxr + 1, minc, idx.rowIndex, maxc], direction: dir.Down };
                else
                    return { startCell: minIdx, endCell: { rowIndex: maxr, colIndex: idx.colIndex }, fillRange: [minr, maxc + 1, maxr, idx.colIndex], direction: dir.Right };
            }

            else if (idx.colIndex < acell.colIndex) { // left
                if ((idx.rowIndex - maxr > maxc - idx.colIndex) || (idx.rowIndex - minr > maxc - idx.colIndex))
                    return { startCell: minIdx, endCell: { rowIndex: idx.rowIndex, colIndex: maxc }, fillRange: [maxr + 1, minc, idx.rowIndex, maxc], direction: dir.Down };
                else
                    return inRange ? { startCell: minIdx, endCell: maxIdx } : { startCell: maxIdx, endCell: { rowIndex: minr, colIndex: idx.colIndex }, fillRange: [minr, idx.colIndex, maxr, minc - 1], direction: dir.Left };
            }
            else if (idx.rowIndex > acell.rowIndex) // down                
                return { startCell: minIdx, endCell: { rowIndex: idx.rowIndex, colIndex: maxc }, fillRange: [maxr + 1, minc, idx.rowIndex, maxc], direction: dir.Down };
            else if (idx.rowIndex === acell.rowIndex && idx.colIndex === acell.colIndex)
                return { startCell: scell, endCell: ecell };
        },

        _getDirection: function (endCell, currcell, isVerticalFill) {
            var dir = ej.Spreadsheet.autoFillDirection;
            isVerticalFill = this.XLObj.isUndefined(isVerticalFill) ? this._verticalFill : isVerticalFill;
            if (isVerticalFill) {
                if (currcell.rowIndex < endCell.rowIndex) // up
                    return dir.Up;
                else if (currcell.rowIndex > endCell.rowIndex) // down
                    return dir.Down;
                else if (currcell.colIndex > endCell.colIndex) // right
                    return dir.Right;
                else if (currcell.colIndex < endCell.colIndex) // left
                    return dir.Left;
            }
            else {
                if (currcell.colIndex > endCell.colIndex) // right
                    return dir.Right;
                else if (currcell.colIndex < endCell.colIndex) // left
                    return dir.Left;
                else if (currcell.rowIndex < endCell.rowIndex) // up
                    return dir.Up;
                else if (currcell.rowIndex > endCell.rowIndex) // down
                    return dir.Down;
            }
            return null;
        },

        _getFillRange: function (pStartCell, pEndCell, pFillCell, direction) {
            var dir = ej.Spreadsheet.autoFillDirection;
            switch (direction) {
                case dir.Up:
                    return [pFillCell.rowIndex, pStartCell.colIndex, pStartCell.rowIndex - 1, pEndCell.colIndex];
                case dir.Right:
                    return [pStartCell.rowIndex, pEndCell.colIndex + 1, pEndCell.rowIndex, pFillCell.colIndex];
                case dir.Down:
                    return [pEndCell.rowIndex + 1, pStartCell.colIndex, pFillCell.rowIndex, pEndCell.colIndex];
                case dir.Left:
                    return [pStartCell.rowIndex, pFillCell.colIndex, pEndCell.rowIndex, pStartCell.colIndex - 1];
            }
            return null;
        },

        _autoFillOptionClick: function (args) {
            var elem = $("#" + args.model.targetID), autoElem = elem.find("#" + args.ID).find(">>"), sIdx = this.getActiveSheetIndex(), colSel = this._getJSSheetHeader(sIdx)[0].querySelectorAll("th.e-colselected"), rowSel = this._getJSSheetRowHeaderContent(sIdx)[0].querySelectorAll("td.e-rowselected");
            if (this._hasClass(autoElem[0], "e-ss-fillselect"))
                return;
            var dragFill = this.XLDragFill, sheet = this.getSheet(this.getActiveSheetIndex()), dir = dragFill._getDirection(this._pEndCell, this._pFillCell),
                dataRange = this.swapRange([this._pStartCell.rowIndex, this._pStartCell.colIndex, this._pEndCell.rowIndex, this._pEndCell.colIndex]),
                fillRange = dragFill._getFillRange(this._pStartCell, this._pEndCell, this._pFillCell, dir), sCell = sheet._startCell, eCell = sheet._endCell;
            this._isUndo = true;
            this._performUndoRedo();
            if (colSel.length < 1 && rowSel.length < 1) {
                sheet._startCell = sCell;
                sheet._endCell = eCell;
                this.XLSelection.selectRange(sheet._startCell, sheet._endCell);
            }
            dragFill.positionAutoFillElement(true);
            dragFill.autoFill({ dataRange: dataRange, fillRange: fillRange, direction: dir, fillType: args.ID });
            dragFill._fillInfo.fillType = args.ID;
            this._isUndo = false;
        },

        _autoFillCleanUp: function () {
            var i, cname, cell, hide = "e-hide", xlObj = this.XLObj, aElem = xlObj.getAutoFillElem(), aOpt = xlObj._getAutoFillOptElem();
            if (aElem)
                xlObj.addClass(aElem[0], hide);
            if (aOpt)
                xlObj.addClass(aOpt[0], hide);
            var cells = xlObj._getContent(xlObj.getActiveSheetIndex())[0].querySelectorAll("td.e-autofillcell");
            i = cells.length;
            while (i--) {
                cell = cells[i];
                cname = cell.className;
                cname = cname.replace(/e-autofillcell|e-blur/g, "");
                cname = cname.replace(/ +/g, " ");
                cell.className = cname;
            }
        },

        _autoFillClick: function () {
            var len, i = 0, elems = $("#" + this._id + "_ctxtmenu").find("li"), cname = "e-ss-fillselect", enabledItem;
            enabledItem = elems.find("." + cname);
            // to remove existing enabled item
            if (enabledItem.length) {
                enabledItem = enabledItem[0];
                this._removeClass(enabledItem, cname);
                this.addClass(enabledItem, "e-ss-filloption");
            }
            // to show all elements 
            elems.show();
            // to hide some options based on scenario
            len = this.XLDragFill._fillInfo.disableItems.length;
            while (i < len) {
                $(elems[this.XLDragFill._fillInfo.disableItems[i]]).hide();
                i++;
            }
            // to select fill type item
            elems.filter("#" + this.XLDragFill._fillInfo.fillType).find("span").addClass(cname);
        },

        _fillRange: function (verticalFill, range) {
            var dirc, isProperKey, pRange, cells, args = {}, xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), direction = ej.Spreadsheet.autoFillDirection;
            range = range ? range : sheet.selectedRange;
            dirc = this._getDirection({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, verticalFill);
            isProperKey = verticalFill ? dirc === direction.Down : dirc === direction.Right;
            if (xlObj.isRange(range) && isProperKey) {
                if (verticalFill) {
                    args.dataRange = [range[0], range[1], range[0], range[3]];
                    args.fillRange = [range[0] + 1, range[1], range[2], range[3]];
                }
                else {
                    args.dataRange = [range[0], range[1], range[2], range[1]];
                    args.fillRange = [range[0], range[1] + 1, range[2], range[3]];
                }
            }
            else {
                if (verticalFill) {
                    if (!range[0])
                        return;
                    args.dataRange = [range[0] - 1, range[1], range[0] - 1, range[3]];
                }
                else {
                    if (!range[1])
                        return;
                    args.dataRange = [range[0], range[1] - 1, range[2], range[1] - 1];
                }
                args.fillRange = range;
            }
            args.direction = verticalFill ? direction.Down : direction.Right;
            args.fillType = ej.Spreadsheet.AutoFillOptions.CopyCells;
            if (xlObj.model.allowLockCell && sheet._isSheetProtected) {
                pRange = xlObj.swapRange(args.fillRange);
                cells = xlObj.XLSelection._getSelectedRange({ rowIndex: pRange[0], colIndex: pRange[1] }, { rowIndex: pRange[2], colIndex: pRange[3] });
                if (xlObj._israngeProtected(cells)) {
                    xlObj._popUplockCellAlert();
                    return;
                }
            }


            this.autoFill(args);
        },

        _getFillInfo: function (options) {
            var i, val, isStringType = true, fillType = ej.Spreadsheet.AutoFillOptions.CopyCells, disableItems = [],
                xlObj = this.XLObj, pos = ej.Spreadsheet.autoFillDirection, isVFill = [pos.Down, pos.Up].indexOf(options.direction) > -1,
                data = xlObj.getRangeData(options.dataRange, true, null, null, true), len = data.join().replace(/,/g, '').length;
            if (xlObj.isRange(options.dataRange) && len) {
                i = data.length;
                while (i--) {
                    val = data[i];
                    if (xlObj.isNumber(val) || xlObj.isFormula(val) || this._isCustomType(val)) {
                        isStringType = false;
                        fillType = ej.Spreadsheet.AutoFillOptions.FillSeries;
                        break;
                    }
                }
            }
            else {
                val = data[0];
                if (xlObj.isNumber(val) || xlObj.isFormula(val) || this._isCustomType(val))
                    isStringType = false;
            }
            // to disable auto fill options
            if (!len || isStringType)
                disableItems.push(1);
            if (!isVFill || (isVFill && options.dataRange[1] !== options.dataRange[3]))
                disableItems.push(4);
            return { fillType: fillType, disableItems: disableItems };
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.cellNavigation = function (obj) {
        this.XLObj = obj;
        this._canKeyBoardNavigate = true;
        this._isNavigate = true;
        this._scrollReverse = false;
    };

    ej.spreadsheetFeatures.cellNavigation.prototype = {
        _selectionKeyDownHandler: function (e) {
            var val, cells, scell, ecell, range, classname, obj, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx),
                keyCode = e.keyCode, shiftKey = e.shiftKey, ctrlKey = e.ctrlKey,
                key = { keyCode: keyCode, shiftKey: shiftKey, ctrlKey: ctrlKey, altKey: e.altKey },
                activecell = sheet._activeCell, endCell = sheet._endCell,
                rowCount = sheet.rowCount, colCount = sheet.colCount;
            if (((keyCode === 9 && this._canKeyBoardNavigate) || keyCode === 13) && !ctrlKey) { // tab & Enter Key
                e.preventDefault(); // to prevent default browser actions
                val = e.target.value;
                if (xlObj.XLEdit._isFormulaEdit && !xlObj._isFormulaSuggestion) {
                    if (val.length > 1) {
                        if (!this._isValidParameter(val)) {
                            xlObj._showAlertDlg("Alert", "MissingParenthesisAlert", "MissingParenthesis", 440);
                            return;
                        }
                        else {
                            if (!xlObj.isFormula(val)) {
                                val = val + ")";
                                xlObj.getActiveCellElem(sheetIdx).find("input:visible").val(val);
                                xlObj._getInputBox().val(val);
                            }
                            xlObj.XLEdit.saveCell();
                            xlObj._getContent(sheetIdx).find("td").removeClass(xlObj._formulaBorder.join(" ").replace(/,/g, " ") + " " + xlObj._ctrlFormulaBorder.join(" ").replace(/,/g, " "));
                            xlObj.XLSelection._cleanUp();
                            xlObj.XLSelection.selectRange(sheet._activeCell, sheet._activeCell);
                            xlObj.XLEdit._isFormulaEdit = false;
                            xlObj._formulaRange = [];
                            xlObj._ctrlKeyCount = 0;
                        }
                    }
                    else {
                        xlObj.XLEdit.saveCell();
                        xlObj.XLEdit._isFormulaEdit = false;
                    }
                }
                cells = xlObj._getJSSheetRowHeaderContent(sheetIdx).find(".e-rowselected").length > 0 ? xlObj._getJSSheetRowHeaderContent(sheetIdx).find(".e-rowselected") : (xlObj._getJSSheetHeader(sheetIdx).find(".e-colselected").length > 0 ? xlObj._getJSSheetHeader(sheetIdx).find(".e-colselected") : xlObj._getContent(sheetIdx).find(".e-selected"));
                if (cells.length) {
                    classname = cells[0].className;
                    if (cells.length === 1 || cells.length - 1 === cells.filter(".e-mc-hide").length) {
                        if (classname.indexOf("e-rowcell") > -1) {
                            if (shiftKey) {
                                if (keyCode === 9 && activecell.colIndex > 0)
                                    this._navigateCell(activecell.rowIndex, activecell.colIndex - 1, key); //shifttab
                                else if (keyCode === 13 && activecell.rowIndex > 0)
                                    this._navigateCell(activecell.rowIndex - 1, activecell.colIndex, key); //shiftenter
                            }
                            else {
                                if (keyCode === 9 && activecell.colIndex < colCount)
                                    this._navigateCell(activecell.rowIndex, activecell.colIndex + 1, key); //tab
                                else if (keyCode === 13 && activecell.rowIndex < rowCount)
                                    this._navigateCell(activecell.rowIndex + 1, activecell.colIndex, key); //enter
                            }
                        }
                        else if (classname.indexOf("e-rowselected") > -1) {
                            if (shiftKey) {
                                if (activecell.colIndex > 0)
                                    this._navigateRow("Backward", key);
                            }
                            else {
                                if (activecell.colIndex < 39)
                                    this._navigateRow("Forward", key);
                            }
                        }
                        else if (classname.indexOf("e-colselected") > -1) {
                            if (shiftKey) {
                                if (activecell.rowIndex > 0)
                                    this._navigateCol("Backward", key);
                            }
                            else {
                                if (activecell.rowIndex < 39)
                                    this._navigateCol("Forward", key);
                            }
                        }
                    }
                    else {
                        if (shiftKey)
                            keyCode === 9 ? this._navigateRange("ShiftTab", key) : this._navigateRange("ShiftEnter", key);
                        else
                            keyCode === 9 ? this._navigateRange("Tab", key) : this._navigateRange("Enter", key);
                    }
                }
            } else if ((keyCode === 37 || (keyCode === 38 && !xlObj._isFormulaSuggestion) || keyCode === 39 || (keyCode === 40 && !xlObj._isFormulaSuggestion)) && this._isNavigate) { //left ,up, right & down arrow
                e.preventDefault(); // to prevent default browser actions
                if (xlObj.XLEdit._isFormulaEdit) { // for formula navigation
                    range = xlObj.swapRange([xlObj._dStartCell.rowIndex, xlObj._dStartCell.colIndex, xlObj._dEndCell.rowIndex, xlObj._dEndCell.colIndex]);
                    xlObj.getRange(!range[0] ? range[0] : range[0] - 1, !range[1] ? range[1] : range[1] - 1, range[2], range[3]).removeClass(xlObj._formulaBorder[xlObj._ctrlKeyCount % 6].join(' ') + " " + xlObj._ctrlFormulaBorder[xlObj._ctrlKeyCount % 6].join(' '));
                    if (ctrlKey) { //ctrl key support
                        scell = xlObj._dStartCell;
                        if (keyCode === 37 && scell.colIndex > 0)  // min column count reached? left arrow 
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: scell.rowIndex, colIndex: 0 };
                        else if (keyCode === 38 && scell.rowIndex > 0)  // min row count reached? up arrow
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: 0, colIndex: scell.colIndex };
                        else if (keyCode === 39 && scell.colIndex < sheet.colCount) // max column count reached ? right arrow
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: scell.rowIndex, colIndex: sheet.colCount };
                        else if (keyCode === 40 && scell.rowIndex < rowCount) //max row count reached? down arrow
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: sheet.colCount, colIndex: scell.colIndex };
                    }
                    if (shiftKey || xlObj._shiftKeyEnabled) { // shift key support
                        ecell = xlObj._dEndCell;
                        if (keyCode === 37 && ecell.colIndex > 0)
                            xlObj._dEndCell = { rowIndex: ecell.rowIndex, colIndex: ecell.colIndex - 1 };
                        else if (keyCode === 38 && ecell.rowIndex > 0)
                            xlObj._dEndCell = { rowIndex: ecell.rowIndex - 1, colIndex: ecell.colIndex };
                        else if (keyCode === 39 && ecell.colIndex < sheet.colCount)
                            xlObj._dEndCell = { rowIndex: ecell.rowIndex, colIndex: ecell.colIndex + 1 };
                        else if (keyCode === 40 && ecell.rowIndex < sheet.rowCount)
                            xlObj._dEndCell = { rowIndex: ecell.rowIndex + 1, colIndex: ecell.colIndex };
                    }
                    else {
                        scell = xlObj._dStartCell;
                        if (keyCode === 37 && scell.colIndex > 0)  // min column count reached? left arrow
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: scell.rowIndex, colIndex: scell.colIndex - 1 }; // left
                        else if (keyCode === 38 && scell.rowIndex > 0) // min row index reached?
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: scell.rowIndex - 1, colIndex: scell.colIndex }; // up
                        else if (keyCode === 39 && scell.colIndex < sheet.colCount) // max col count reached?
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: scell.rowIndex, colIndex: scell.colIndex + 1 }; // right
                        else if (keyCode === 40 && scell.rowIndex < sheet.rowCount)  // max row count reached?
                            xlObj._dStartCell = xlObj._dEndCell = { rowIndex: scell.rowIndex + 1, colIndex: scell.colIndex }; // down   
                    }
                    xlObj.XLSelection._processFormulaRange(xlObj._ctrlFormulaBorder);
                    xlObj._formulaRange[xlObj._ctrlKeyCount] = xlObj._getProperAlphaRange(sheetIdx, xlObj._dStartCell.rowIndex, xlObj._dStartCell.colIndex, xlObj._dEndCell.rowIndex, xlObj._dEndCell.colIndex);
                }
                else {
                    if (!xlObj.model.scrollSettings.allowVirtualScrolling && ctrlKey) { //ctrl key support
                        if (keyCode === 37 && activecell.colIndex > 0)  // min column count reached? left arrow 
                            this._navigateCell(activecell.rowIndex, 0, key);
                        else if (keyCode === 38 && activecell.rowIndex > 0)  // min row count reached? up arrow
                            this._navigateCell(0, activecell.colIndex, key);
                        else if (keyCode === 39 && activecell.colIndex < colCount - 1) // max column count reached ? right arrow
                            this._navigateCell(activecell.rowIndex, colCount - 1, key);
                        else if (keyCode === 40 && activecell.rowIndex < rowCount - 1) //max row count reached? down arrow
                            this._navigateCell(rowCount - 1, activecell.colIndex, key);
                    }
                    else if (!xlObj.model.scrollSettings.allowVirtualScrolling && shiftKey) { // shift key support(prevented for virtual scrolling)
                        if (keyCode === 37 && endCell.colIndex > 0)
                            sheet._endCell = { rowIndex: endCell.rowIndex, colIndex: endCell.colIndex - 1 };
                        else if (keyCode === 38 && endCell.rowIndex > 0)
                            sheet._endCell = { rowIndex: endCell.rowIndex - 1, colIndex: endCell.colIndex };
                        else if (keyCode === 39 && endCell.colIndex < colCount)
                            sheet._endCell = { rowIndex: endCell.rowIndex, colIndex: endCell.colIndex + 1 };
                        else if (keyCode === 40 && endCell.rowIndex < rowCount)
                            sheet._endCell = { rowIndex: endCell.rowIndex + 1, colIndex: endCell.colIndex };
                        if ((sheet.rowCount === sheet._endCell.rowIndex || sheet.colCount === sheet._endCell.colIndex) && xlObj.model.scrollSettings.scrollMode === "normal")
                            return;
						if ((sheet.rowCount === sheet._endCell.rowIndex || sheet.colCount === sheet._endCell.colIndex) && !xlObj.model.scrollSettings.allowScrolling)
						    return;
						if (xlObj.model.scrollSettings.allowScrolling) {
						    sheet.rowCount === sheet._endCell.rowIndex && xlObj.XLScroll._scrollY(sheetIdx);
						    sheet.colCount === sheet._endCell.colIndex && xlObj.XLScroll._scrollX(sheetIdx);
						}
                        if (this._isHiddenCell(sheet._endCell)) {
                            obj = { rowIndex: sheet._endCell.rowIndex, colIndex: sheet._endCell.colIndex };
                            this._skipHiddenRowsAndCols(obj, key);
                            sheet._endCell.rowIndex = obj.rowIndex;
                            sheet._endCell.colIndex = obj.colIndex;
                        }
                        xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                        if(xlObj.model.allowAutoFill)
							xlObj.XLDragFill.positionAutoFillElement();
                    }
                    else {
                        key = { keyCode: keyCode, shiftKey: false, ctrlKey: false, altKey: false };
                        if (keyCode === 37 && activecell.colIndex > 0)  // min column count reached? left arrow
                            this._navigateCell(activecell.rowIndex, activecell.colIndex - 1, key);//left
                        else if (keyCode === 38 && activecell.rowIndex > 0) // min row index reached?
                            this._navigateCell(activecell.rowIndex - 1, activecell.colIndex, key);//up
                        else if (keyCode === 39 && activecell.colIndex < colCount) // max col count reached?
                            this._navigateCell(activecell.rowIndex, activecell.colIndex + 1, key);//right
                        else if (keyCode === 40 && activecell.rowIndex < rowCount)  // max row count reached?
                            this._navigateCell(activecell.rowIndex + 1, activecell.colIndex, key);//down
                        else
                            return false;
                    }
                }
            }
        },

        _navigateCell: function (rowindex, colindex, key) { // select cell by rowindex and colindex
            var colgrp, isSpan, cell, actCell, obj = { rowIndex: rowindex, colIndex: colindex }, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), cont = xlObj._getContent(sheetIdx),
                hScroll = cont.find("#hscrollBar").data("ejScrollBar"), vScroll = cont.find("#vscrollBar").data("ejScrollBar"), content = cont.find(".e-content"), activeCell, scrollReach = false;
            if ((sheet.rowCount === obj.rowIndex || sheet.colCount === obj.colIndex) && xlObj.model.scrollSettings.scrollMode === "normal")
                return;
            if ((sheet.rowCount === obj.rowIndex || sheet.colCount === obj.colIndex) && !xlObj.model.scrollSettings.allowScrolling)
                return;
            if (sheet.rowCount === obj.rowIndex) {
                xlObj.XLScroll._scrollY(sheetIdx);
                scrollReach = true;
            }
            if (sheet.colCount === obj.colIndex) {
                xlObj.XLScroll._scrollX(sheetIdx);
                scrollReach = true;
            }
            activeCell = xlObj.getCell(obj.rowIndex, obj.colIndex);
            if(this._isHiddenCell(obj))
                this._skipHiddenRowsAndCols(obj, key);
            cell = xlObj.getCell(obj.rowIndex, obj.colIndex)[0];
            actCell = content.find(".e-activecell");
            if (actCell.hasClass("e-merge") && (cell.className.indexOf("e-rc-") > -1))
                this._updateMergeRange(actCell, obj, key);
            if (activeCell.length > 0)
                sheet._activeCell = sheet._endCell = sheet._startCell = obj;
            if (!ej.isNullOrUndefined(cell) && (cell.className.indexOf("e-rc-") > -1 || xlObj.hasSpan(obj))) {
                xlObj.XLSelection._processBoundary();
                sheet._activeCell = sheet._startCell;
                activeCell = xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex);
                isSpan = true;
            }
            if (activeCell.length < 1) {
                if (key.keyCode == 40 || key.keyCode == 13)
                    xlObj.XLScroll._scrollY(sheetIdx);
                else if (key.keyCode == 39 || key.keyCode == 9)
                    xlObj.XLScroll._scrollX(sheetIdx);
            } else {
                if (xlObj.model.scrollSettings.allowScrolling && content.offset().top + content[0].offsetHeight <= (activeCell.offset().top + activeCell[0].offsetHeight) && (key.keyCode == 40 || (key.keyCode == 13 && !key.shiftKey)) && !scrollReach) {
                    vScroll._scrollData.step = 1;
                    if (key.ctrlKey && key.keyCode == 40)
                        xlObj.XLScroll._scrollSelectedPosition(sheetIdx, xlObj.getActiveCell(sheetIdx));
                    else
                        vScroll.scroll(vScroll.value() + parseInt(activeCell.parent()[0].style.height), true);
                } else if (xlObj.model.scrollSettings.allowScrolling && ((activeCell.offset().top - sheet._frozenHeight) <= content.offset().top) && (key.keyCode == 38 || (key.keyCode == 13 && key.shiftKey)) && !scrollReach && !xlObj._hasClass(cont.find("tr").eq(activeCell[0].parentNode.rowIndex)[0], 'e-fr-hide')) {
                    vScroll._scrollData.step = -1;
                    if (activeCell.parent().index() > 0)
                        vScroll.scroll(vScroll.value() - parseInt(activeCell.parent()[0].style.height), true);
                    else
                        vScroll.scroll(0);
                }
                if (xlObj.model.scrollSettings.allowScrolling && content[0].offsetWidth <= (activeCell.offset().left + activeCell[0].offsetWidth) && (key.keyCode == 39 || (key.keyCode == 9 && !key.shiftKey)) && !scrollReach) {
                    hScroll._scrollData.step = 1;
                    if (key.ctrlKey && key.keyCode == 39)
                        xlObj.XLScroll._scrollSelectedPosition(sheetIdx, xlObj.getActiveCell(sheetIdx));
                    else
                        hScroll.scroll(hScroll.value() + activeCell[0].offsetWidth, true);
                } else if (xlObj.model.scrollSettings.allowScrolling && ((activeCell.offset().left - sheet._frozenWidth) <= content.offset().left) && (key.keyCode == 37 || (key.keyCode == 9 && key.shiftKey)) && !scrollReach) {
                    hScroll._scrollData.step = -1;
                    if (activeCell.get(0).cellIndex > 0)
                        hScroll.scroll(hScroll.value() - activeCell[0].offsetWidth, true);
                    else
                        hScroll.scroll(0);
                }
            }
            colgrp = cont.find("col");
            var rows = $(xlObj.getRows(sheetIdx)[1]), rowIdx = sheet._startCell.rowIndex, colIdx = sheet._startCell.colIndex;
            cell = xlObj.getCell(rowIdx, colIdx);
            if (cell.length > 0) {
                if ((content[0].offsetWidth >= cell.offset().left || content.offset().left + content[0].offsetWidth >= (cell.offset().left + cell[0].offsetWidth)) && (key.keyCode == 39 || (key.keyCode == 9 && !key.shiftKey))) {
                    xlObj.XLSelection.selectRange(sheet._startCell, sheet._startCell);
                    xlObj.model.showRibbon && xlObj.XLRibbon._updateRibbonIcons();
                    if(xlObj.model.allowAutoFill)
                    xlObj.XLDragFill.positionAutoFillElement();
                }
                if (!colgrp.eq(colIdx).hasClass("e-fc-hide") || (key.keyCode == 37 || (key.keyCode == 9 && key.shiftKey))) {
                    xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                    xlObj.model.showRibbon && xlObj.XLRibbon._updateRibbonIcons();
                    if(xlObj.model.allowAutoFill)
                    xlObj.XLDragFill.positionAutoFillElement();
                }
                if (!rows.eq(rowIdx).hasClass("e-fr-hide") && (key.keyCode == 38 || (key.keyCode == 13 && key.shiftKey))) {
                    xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                    xlObj.model.showRibbon && xlObj.XLRibbon._updateRibbonIcons();
                    if(xlObj.model.allowAutoFill)
                    xlObj.XLDragFill.positionAutoFillElement();
                }
                if (((content.offset().top + content[0].offsetHeight) >= cell.offset().top || content.offset().top + content[0].offsetHeight >= (cell.offset().top + cell[0].offsetHeight)) && (key.keyCode == 40 || (key.keyCode == 13 && !key.shiftKey))) {
                    xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                    xlObj.model.showRibbon && xlObj.XLRibbon._updateRibbonIcons();
                    if(xlObj.model.allowAutoFill)
                    xlObj.XLDragFill.positionAutoFillElement();
                }
            }
            if (isSpan) {
                if (key.keyCode === 37 || (key.shiftKey && key.keyCode === 9))
                    obj.colIndex = sheet._startCell.colIndex;
                else if (key.keyCode === 38 || (key.shiftKey && key.keyCode === 13))
                    obj.rowIndex = sheet._startCell.rowIndex;
                else if (key.keyCode === 39 || key.keyCode === 9)
                    obj.colIndex = sheet._endCell.colIndex;
                else if (key.keyCode === 40 || key.keyCode === 13)
                    obj.rowIndex = sheet._endCell.rowIndex;
                sheet._activeCell = obj;
            }
            sheet._startCell = { rowIndex: activeCell[0].parentNode.rowIndex, colIndex: activeCell[0].cellIndex };
            while ((key.keyCode === 37 || (key.shiftKey && key.keyCode === 9)) && xlObj._hasClass(colgrp[sheet._startCell.colIndex], 'e-c-hide')) {
                xlObj._removeClass(colgrp[sheet._startCell.colIndex], 'e-fc-hide');
                sheet._startCell.colIndex--;
            }
            while ((key.keyCode === 38 || (key.shiftKey && key.keyCode === 13)) && xlObj._hasClass(cont.find('tr')[sheet._startCell.rowIndex], 'e-r-hide')) {
                xlObj._removeClass(cont.find('tr')[sheet._startCell.rowIndex], 'e-fr-hide');
                sheet._startCell.rowIndex--;
            }
            while ((key.keyCode === 39 || key.keyCode === 9) && xlObj._hasClass(colgrp[sheet._startCell.colIndex], 'e-c-hide'))
                sheet._startCell.colIndex++;
            while ((key.keyCode === 40 || key.keyCode === 13) && xlObj._hasClass(cont.find('tr')[sheet._startCell.rowIndex], 'e-r-hide'))
                sheet._startCell.rowIndex++;
            rIdx = sheet._startCell.rowIndex, cIdx = sheet._startCell.colIndex;
            if (xlObj._hasClass(colgrp[cIdx], 'e-fc-hide') || xlObj._hasClass(cont.find('tr')[rIdx], 'e-fr-hide')) {
                sheet._activeCell = sheet._endCell = { rowIndex: rIdx, colIndex: cIdx };
                if (xlObj._hasClass(colgrp[cIdx], 'e-fc-hide') && (key.keyCode === 37 || (key.shiftKey && key.keyCode === 9))) {
                    this._scrollReverse = true;
                    xlObj._removeClass(colgrp[cIdx], 'e-fc-hide');
                    xlObj._removeClass(xlObj._getJSSheetHeader(sheetIdx).find('col')[cIdx], 'e-fc-hide');
                    hScroll.scroll(hScroll.value() - sheet.columnsWidthCollection[cIdx], true);
                    (hScroll.value() < 0 || !xlObj._hasClass(colgrp[cIdx - 1], 'e-fc-hide')) && hScroll.scroll(0);
                }
                else if (xlObj._hasClass(cont.find('tr')[rIdx], 'e-fr-hide') && (key.keyCode == 38 || (key.shiftKey && key.keyCode === 13))) {
                    this._scrollReverse = true;
                    xlObj._removeClass(cont.find('tr')[rIdx], 'e-fr-hide');
                    xlObj._removeClass(xlObj._getJSSheetRowHeader(sheetIdx).find('tr')[rIdx], 'e-fr-hide');
                    vScroll.scroll(vScroll.value() - sheet.rowsHeightCollection[rIdx], true);
                    (vScroll.value() < 0 || !xlObj._hasClass(cont.find('tr')[rIdx - 1], 'e-fr-hide')) && vScroll.scroll(0);
                }
                else if (xlObj._hasClass(colgrp[cIdx], 'e-fc-hide') && (key.keyCode === 39 || key.keyCode === 9)) {
                    xlObj.XLScroll._scrollDirection('hscrollBar', 0);
                }
                else if (xlObj._hasClass(cont.find('tr')[rIdx], 'e-fr-hide') && (key.keyCode === 40 || key.keyCode === 13)) {
                    xlObj.XLScroll._scrollDirection('vscrollBar', 0);
                }
                xlObj.XLSelection.selectRange({ rowIndex: rIdx, colIndex: cIdx }, { rowIndex: rIdx, colIndex: cIdx });
            if(xlObj.model.allowAutoFill)
                xlObj.XLDragFill.positionAutoFillElement();
            }
        },

        _navigateRange: function (args, key) { // active cell changes at range of cells,rows,columns
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), startcell = sheet._startCell, endcell = sheet._endCell, activecell = sheet._activeCell,
                minrowindex = startcell.rowIndex < endcell.rowIndex ? startcell.rowIndex : endcell.rowIndex,
                maxrowindex = startcell.rowIndex > endcell.rowIndex ? startcell.rowIndex : endcell.rowIndex,
                mincolindex = startcell.colIndex < endcell.colIndex ? startcell.colIndex : endcell.colIndex,
                maxcolindex = startcell.colIndex > endcell.colIndex ? startcell.colIndex : endcell.colIndex;
            if (args == "Tab") {
                if (activecell.colIndex < maxcolindex)
                    this._navigateRow("Forward", key);
                else if (activecell.rowIndex < maxrowindex)
                    this._updateActiveCell(activecell.rowIndex + 1, mincolindex, key, args);
                else if (activecell.rowIndex == maxrowindex && activecell.colIndex == maxcolindex)
                    this._updateActiveCell(minrowindex, mincolindex, key, args);
                else
                    return false;
            } else if (args == "Enter") {
                if (activecell.rowIndex < maxrowindex)
                    this._navigateCol("Forward", key);
                else if (activecell.colIndex < maxcolindex)
                    this._updateActiveCell(minrowindex, activecell.colIndex + 1, key, args);
                else if (activecell.rowIndex == maxrowindex && activecell.colIndex == maxcolindex)
                    this._updateActiveCell(minrowindex, mincolindex, key, args);
                else
                    return false;
            } else if (args == "ShiftTab") {
                if (activecell.colIndex > mincolindex)
                    this._navigateRow("Backward", key);
                else if (activecell.rowIndex > minrowindex)
                    this._updateActiveCell(activecell.rowIndex - 1, maxcolindex, key, args);
                else if (activecell.rowIndex == minrowindex && activecell.colIndex == mincolindex)
                    this._updateActiveCell(maxrowindex, maxcolindex, key, args);
                else
                    return false;
            } else { // shift enter
                if (activecell.rowIndex > minrowindex)
                    this._navigateCol("Backward", key);
                else if (activecell.colIndex > mincolindex)
                    this._updateActiveCell(maxrowindex, activecell.colIndex - 1, key, args);
                else if (activecell.rowIndex == minrowindex && activecell.colIndex == mincolindex)
                    this._updateActiveCell(maxrowindex, maxcolindex, key, args);
                else
                    return false;
            }
        },

        _navigateRow: function (args, key) { // active cell changes at single row
            var idx, action, xlObj = this.XLObj, activecell = xlObj.getSheet(xlObj.getActiveSheetIndex())._activeCell;
            if(args === "Forward"){
                idx = 1;
                action = "Tab";
            }
            else{
                idx = -1;
                action = "ShiftTab";
            }
            this._updateActiveCell(activecell.rowIndex, activecell.colIndex + idx, key, action);
        },

        _navigateCol: function (args, key) { // active cell changes at single column
            var idx, action, xlObj = this.XLObj, activecell = xlObj.getSheet(xlObj.getActiveSheetIndex())._activeCell;
            if (args === "Forward") {
                idx = 1;
                action = "Enter";
            }
            else {
                idx = -1;
                action = "ShiftEnter";
            }
            this._updateActiveCell(activecell.rowIndex + idx, activecell.colIndex, key, action);
        },

        _updateActiveCell: function (rowindex, colindex, key, args) { // to update active cell by removing existing active cell
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), activecell = sheet._activeCell;
            xlObj.getCell(activecell.rowIndex, activecell.colIndex).removeClass("e-activecell");
            sheet._activeCell = { rowIndex: rowindex, colIndex: colindex };
            if(key && this._isHiddenCell(sheet._activeCell))
                this._skipHiddenRowsAndCols(sheet._activeCell, key, true, args);
            if(this._isHiddenCell(sheet._activeCell))
              this._skipMergedCells(sheet._activeCell, key);
            xlObj.getCell(sheet._activeCell.rowIndex, sheet._activeCell.colIndex).addClass("e-activecell");
            xlObj.updateFormulaBar();
            xlObj.setSheetFocus();
        },

        _isValidParameter: function (val) {
            var index = val.indexOf("(");
            if (index > -1) {
                val = val.split("(");
                return val[1].length > 0;
            }
            return true;
        },

        _skipHiddenRowsAndCols: function (obj, key, inRange, args) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), canIterate = true;
            if (inRange)
                this._navigateRange(args, key);
            else {
                if ((key.keyCode === 37 || (key.keyCode === 9 && key.shiftKey) || (key.keyCode === 39 && key.ctrlKey)) && obj.colIndex)
                    obj.colIndex--;
                else if ((key.keyCode === 38 || (key.keyCode === 13 && key.shiftKey) || (key.keyCode === 40 && key.ctrlKey)) && obj.rowIndex)
                    obj.rowIndex--;
                else if ((key.keyCode === 39 || key.keyCode === 9 || (key.keyCode === 37 && key.ctrlKey)) && obj.colIndex < sheet.colCount)
                    obj.colIndex++;
                else if ((key.keyCode === 40 || key.keyCode === 13 || (key.keyCode === 38 && key.ctrlKey)) && obj.rowIndex < sheet.rowCount)
                    obj.rowIndex++;
                else
                    canIterate = false;
                if (this._isHiddenCell(obj) && canIterate)
                    this._skipHiddenRowsAndCols(obj, key);
                else if (!canIterate) {
                    if ((key.keyCode === 37 || (key.keyCode === 9 && key.shiftKey)) && !obj.colIndex)
                        obj.colIndex = (key.keyCode === 37 && key.shiftKey) ? sheet._endCell.colIndex + 1 : sheet._startCell.colIndex;
                    else if ((key.keyCode === 38 || (key.keyCode === 13 && key.shiftKey)) && !obj.rowIndex)
                        obj.rowIndex = (key.keyCode === 38 && key.shiftKey) ? sheet._endCell.rowIndex + 1 : sheet._startCell.rowIndex;
                }
            }
        },

        _isHiddenCell: function (cellIdx) {
            return this.XLObj.XLEdit.getPropertyValue(cellIdx.rowIndex, cellIdx.colIndex, "isMHide") || false;
        },

        _skipMergedCells: function (obj, key, xtendSelection) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(xlObj.getActiveSheetIndex()), startcell = sheet._startCell, endcell = sheet._endCell, canIterate = true, regx = xlObj._rcRegx, isSelected = false;
            minr = startcell.rowIndex < endcell.rowIndex ? startcell.rowIndex : endcell.rowIndex,
            maxr = startcell.rowIndex > endcell.rowIndex ? startcell.rowIndex : endcell.rowIndex,
            minc = startcell.colIndex < endcell.colIndex ? startcell.colIndex : endcell.colIndex,
            maxc = startcell.colIndex > endcell.colIndex ? startcell.colIndex : endcell.colIndex;
            if (key.shiftKey && key.keyCode === 9)
                obj.colIndex--;
            else if (key.keyCode === 9)
                obj.colIndex++;
            else if (key.shiftKey && key.keyCode === 13)
                obj.rowIndex--;
            else if (key.keyCode === 13)
                obj.rowIndex++;
            else
                canIterate = false;

            if (obj.colIndex > maxc) {
                obj.colIndex = minc;
                obj.rowIndex = (obj.rowIndex === maxr ? minr : obj.rowIndex + 1);
            }
            else if (obj.colIndex < minc) {
                obj.colIndex = maxc;
                obj.rowIndex = (obj.rowIndex === minr ? maxr : obj.rowIndex - 1);
            }
            else if (obj.rowIndex > maxr) {
                obj.rowIndex = minr;
                obj.colIndex = (obj.colIndex === maxc ? minc : obj.colIndex + 1);
            }
            else if (obj.rowIndex < minr) {
                obj.rowIndex = maxr;
                obj.colIndex = (obj.colIndex === minc ? maxc : obj.colIndex - 1);
            }
            if (this._isHiddenCell(obj) && canIterate)
                this._skipMergedCells(obj, key);

        },

        _updateMergeRange: function (actCell, obj, key) {
            var xlObj = this.XLObj, actCellEle = actCell[0], colspan = actCellEle.colSpan - 1, rowspan = actCellEle.rowSpan - 1;			
            if (!key.shiftKey && (key.keyCode === 39 || key.keyCode === 9))
                obj.colIndex = obj.colIndex + colspan;
            else if (!key.shiftKey && (key.keyCode === 40 || key.keyCode === 13))
                obj.rowIndex = obj.rowIndex + rowspan;
							
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.cellFormatting = function (obj) {
        this.XLObj = obj;
        this._formatEnable = false;
        this._thousandSeparator = Globalize.culture(this.XLObj.model.locale).numberFormat[","];
        this._formatAsTableStyle = {};
        this._rowLength = null;
        this._colLength = null;
        this._getFormat = null;
        this._getEformatClass = null;
        this._borderPosition = ["top", "right", "bottom", "left"];
        this._cellObj = null;
    };

    ej.spreadsheetFeatures.cellFormatting.prototype = {
        format: function (formatObj, range) {
            if (!this.XLObj.model.allowCellFormatting)
                return;
            var info, cell, isViewed = false, cssclass, formattedval, args, xlObj = this.XLObj, cellObj = {}, i = 0, selCells = [], style = "style",
                sheetIdx = xlObj.getActiveSheetIndex(), details = { sheetIndex: sheetIdx, reqType: "cell-format" }, ralign = xlObj._rAlign, container = xlObj._dataContainer,
                format = "format", sep = "thousandSeparator", cells, len;
            if(!range)
                cells = xlObj.getSheet(sheetIdx)._selectedCells;
            else{
                var rng = xlObj.getRangeIndices(range)
                cells = xlObj.XLSelection._getSelectedRange({rowIndex: rng[0], colIndex: rng[1]}, {rowIndex: rng[2], colIndex: rng[3]});
            }
            len = cells.length;
            if (xlObj._trigger("beforeCellFormat", { sheetIndex: sheetIdx, format: formatObj, cells: cells }))
                return;
            if (len > 0) {
                formatObj = this._getFormatObj(formatObj);
                if (style in formatObj || format in formatObj) {
                    details.beforeFormat = xlObj.isUndefined(range) ? this.getHashCodeClassAsArray() : this.getHashCodeClassAsArray(xlObj.getRangeIndices(range));
                    while (i < len) {
                        cell = cells[i], args = { sheetIndex: sheetIdx, format: formatObj, cell: cell };
                        info = { rowIndex: cell.rowIndex, colIndex: cell.colIndex, beforeFormat: {}, afterFormat: {}, bRowHeight: xlObj.model.allowResizing && xlObj.XLResize.getRowHeight(cell.rowIndex) };
                        if (!ej.isNullOrUndefined(container.sheets[sheetIdx][info.rowIndex]))
                            cellObj = ej.isNullOrUndefined(container.sheets[sheetIdx][info.rowIndex][info.colIndex]) ? {} : container.sheets[sheetIdx][info.rowIndex][info.colIndex];
                        else
                            cellObj = {};
                        $.extend(true, info.beforeFormat, cellObj);
                        if (style in formatObj) {
                            info.prevClass = xlObj.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, format);
                            info.prevClass = info.prevClass ? info.prevClass : "";
                            cssclass = this._getExtendedClass(info.prevClass, formatObj.style);
                            if (xlObj._isRowViewable(sheetIdx, cell.rowIndex)) {
                                $cell = xlObj.getCell(cell.rowIndex, cell.colIndex);
                                this._updateFormatClass(cell, cssclass);
                                isViewed = true;
                            }
                            info.className = cssclass;
                            if (("font-family" in formatObj["style"] || "font-size" in formatObj["style"]) && ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(cell.rowIdx, cell.colIndex, "isMHide")) && ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(cell.rowIdx, cell.colIndex, "isMerge"))) {
                                if (isViewed) {
                                    var cHght = $cell.outerHeight(), cWidth, rows = xlObj.getRows(sheetIdx)[0], val = xlObj.XLEdit.getPropertyValue(info.rowIndex, info.colIndex);
                                    $(rows[info.rowIndex]).outerHeight(cHght);
                                    xlObj.getSheet(sheetIdx).rowsHeightCollection[info.rowIndex] = cHght;
                                    if (xlObj.model.allowResizing && (xlObj.isNumber(val) || xlObj._isDateTime(val) || xlObj.isTime(val))) {
                                        cWidth = xlObj.XLResize._getContentWidth(info.colIndex, sheetIdx, [cell.parentElement]) + 2;
                                        (cWidth > $cell.width()) && xlObj.setWidthToColumns([{ cellIndex: info.colIndex, width: cWidth }]);
                                    }
                                }
                                xlObj.XLSelection._refreshBorder();
                            }
                            args.cssClass = cssclass;
                        }
                        if (format in formatObj && formatObj.type) {
                            formattedval = xlObj.XLEdit.getPropertyValue(info.rowIndex, info.colIndex);
                            formattedval = ej.isNullOrUndefined(formattedval) ? "" : formattedval;
                            this._cellObj = { value: formattedval, type: formatObj.type, formatStr: formatObj.format.formatStr, decimalPlaces: formatObj.format.decimalPlaces, thousandSeparator: formatObj.format[sep] ? true : false, canUpdate: true };
                            if (xlObj.isFormula(formattedval)) {
                                formattedval = xlObj.XLEdit.getPropertyValue(info.rowIndex, info.colIndex, "calcValue");
                                this._cellObj.canUpdate = false;
                            }
                            if (["text", "general"].indexOf(this._cellObj.type) !== -1) {
                                if (xlObj._isDateTime(formattedval)) {
                                    formattedval = xlObj._dateToInt(formattedval);
                                    this._cellObj.value = formattedval;
                                }
                                this._cellObj.type === ej.Spreadsheet.CellType.Text ? $cell.removeClass(ralign) : xlObj.isNumber(formattedval) && cell.className.indexOf(ralign) === -1 && $cell.addClass(ralign);
                            }
                            else
                                formattedval = this._format(formattedval, { formatStr: this._cellObj.formatStr, type: this._cellObj.type, thousandSeparator: this._cellObj.thousandSeparator, decimalPlaces: this._cellObj.decimalPlaces, cellObj: this._cellObj }); // thousand separtor may be undefined(for performance)
                            args.value = this._cellObj.value;
                            args.value2 = this._cellObj["value2"] = formattedval + "";
                            delete this._cellObj.canUpdate;
                            xlObj.XLEdit._updateDataContainer(info, this._cellObj);
                            info.beforeFormat.Oprtype = this._cellObj.type;
                            this._cellObj = null;
                        }
                        if (!ej.isNullOrUndefined(container.sheets[sheetIdx][info.rowIndex]))
                            cellObj = ej.isNullOrUndefined(container.sheets[sheetIdx][info.rowIndex][info.colIndex]) ? {} : container.sheets[sheetIdx][info.rowIndex][info.colIndex];
                        else
                            cellObj = {};
                        info.afterFormat = $.extend(true, {}, cellObj);
                        info.afterFormat.Oprtype = info.beforeFormat.Oprtype;
						info.aRowHeight = xlObj.model.allowResizing && xlObj.XLResize.getRowHeight(cell.rowIndex);
                        selCells.push(info);
						xlObj._trigger("cellFormatting", args);
						if(xlObj.XLEdit.getPropertyValue(info.rowIndex, info.colIndex, 'value2')){
                          xlObj._textClip(info.rowIndex, info.colIndex, 'delete');
                          xlObj._textClip(info.rowIndex, info.colIndex, 'add');
						}
                        i++;
                    }
                    details.selCell = selCells;
                    details.afterFormat = xlObj.isUndefined(range) ? this.getHashCodeClassAsArray() : this.getHashCodeClassAsArray(xlObj.getRangeIndices(range));
                    details.range = xlObj.getSheet(sheetIdx).selectedRange;
                }
                if ("border" in formatObj)
                    this.applyBorder(formatObj.border, range, details);
                else {
                    if (!xlObj.model.sheets[sheetIdx]._importRendered && !xlObj._isUndoRedo)
                        if (!xlObj._dupDetails)
                            if (!xlObj._isSort)
                                xlObj._completeAction(details);
                }
            }
        },

        _format: function (value, formatObj) {
            var xlObj = this.XLObj, formatstr = formatObj.formatStr,
                temp = xlObj.isUndefined(value) ? "" : value, type = formatObj.type || "", ctype = ej.Spreadsheet.CellType,
                locale = xlObj.model.locale, prefix = ["."].indexOf(this._thousandSeparator) > -1 ? '\\' : "", regx = new RegExp(prefix + this._thousandSeparator, "g");
            value = temp.toString();
            switch (type) {
                case ctype.Accounting:
                    if (xlObj._isDateTime(temp)) {
                        temp = xlObj._dateToInt(temp);
                        if (this._cellObj && this._cellObj.canUpdate)
                            this._cellObj.value = temp;
                    }
                    if (xlObj.isNumber(temp))
                        temp = xlObj.toAccounting(formatstr, temp, locale);
                    break;
                case ctype.Currency:
                case ctype.Number:
                case ctype.Percentage:
                    if (xlObj._isDateTime(temp)) {
                        temp = xlObj._dateToInt(temp);
                        if (this._cellObj && this._cellObj.canUpdate)
                            this._cellObj.value = temp;
                    }
                    if (xlObj.isNumber(temp)) {
                        temp = xlObj.formatting(formatstr, temp, locale);
                        if (!formatObj.thousandSeparator && [ctype.Number, ctype.Percentage].indexOf(type) > -1 && this._thousandSeparator)
                            temp = temp.replace(regx, "");
                    }
                    break;
                case ctype.Scientific:
                    if (xlObj._isDateTime(temp)) {
                        temp = xlObj._dateToInt(temp);
                        if (this._cellObj && this._cellObj.canUpdate)
                            this._cellObj.value = temp;
                    }
                    if (xlObj.isNumber(temp))
                        temp = xlObj.toExponential(temp, formatObj.decimalPlaces);
                    break;
                case ctype.Fraction:
                    if (xlObj._isDateTime(temp)) {
                        temp = xlObj._dateToInt(temp);
                        if (this._cellObj && this._cellObj.canUpdate)
                            this._cellObj.value = temp;
                    }
                    if (xlObj.isNumber(temp)) {
                        temp = xlObj.toFraction(temp);
                        temp = "numerator" in temp ? temp.integer + " " + temp.numerator + "/" + temp.denominator : temp.integer;
                    }
                    break;
                case ctype.LongDate:
                case ctype.ShortDate:
                case ctype.Date:
                case ctype.Time:
                case ctype.DateTime:
                    if (xlObj.isNumber(temp)) {
                        temp = xlObj.intToDate(temp);
                        if (this._cellObj && this._cellObj.canUpdate)
                            this._cellObj.value = temp;
                    }
                    else if (!xlObj._isDateTime(temp))
                        temp = Globalize.parseDate(temp);
                    if (xlObj._isDateTime(temp))
                        temp = xlObj.formatting(formatstr, temp, locale);
                    break;
            }
            return ej.isNullOrUndefined(temp) ? value : temp;
        },

        createTable: function (tableObj, range) {
            if (!this.XLObj.model.allowFormatAsTable)
                return;
            var details, pcells, trange, cname, tlength, xlObj = this.XLObj, obj = { format: tableObj.format }, sheetIdx = xlObj.getActiveSheetIndex(),
                sheet = xlObj.getSheet(sheetIdx), tname = tableObj.name || "", tmgr = xlObj.getSheet(sheetIdx)._tableManager, scell = sheet._startCell,
                ecell = sheet._endCell, str = "formatName", args = { sheetIndex: sheetIdx, tableStyle: tableObj, reqType: "format-table" };
            if (xlObj._trigActionBegin(args))
                return;
            tableObj = args.tableStyle;
            xlObj._isFAT = true;
            if (xlObj._isUndoRedo)
                trange = $.extend(true, [], range);
            else
                trange = range ? xlObj.getRangeIndices(range) : sheet.selectedRange;
            cname = xlObj.XLEdit.getPropertyValue(trange[0], trange[1], "tableName") || "";
            if (!sheet._importRendered) {
				pcells = xlObj.getRange(trange[0], trange[1], trange[0], trange[3]);
                details = { sheetIndex: xlObj.getActiveSheetIndex(), reqType: "format-table", beforeFormat: this.getHashCodeClassAsArray(), beforeRange: $.extend(true, [], trange), pcells: pcells, beforeRangeHeader: xlObj.getRangeData([trange[0], trange[1], trange[0], trange[3]])};
                details.beforeRangeData = tableObj.header ? xlObj.getRangeData([trange[0], trange[1], trange[2], trange[3]]) : xlObj.getRangeData([trange[0], trange[1], trange[2] + 1, trange[3]]);
            }
            if (str in tableObj)
                obj[str] = tableObj.formatName;
            if (!cname) {
                if (xlObj._isUndoRedo && xlObj.XLClipboard._isCut)
                    tlength = parseInt(tname.replace("Table", ""));
                else
                    tlength = sheet._importRendered ? this._getTableIdxFromName(tableObj.name) : xlObj.getObjectLength(tmgr);
                tname = tname.length ? tname : 'Table ' + tlength;
                xlObj.model.sheets[sheetIdx]._header[tlength] = { header: tableObj.header };
                if (tableObj.header) {
                    if (range) {
                        range = xlObj._isUndoRedo ? range : xlObj.getRangeIndices(range);
                        sheet._startCell = { rowIndex: range[0], colIndex: range[1] };
                        sheet._endCell = { rowIndex: range[2], colIndex: range[3] };
                        xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                        if (xlObj.model.allowAutoFill && !sheet._importRendered)
                            xlObj.XLDragFill.positionAutoFillElement();
                    }
                    else
                        range = xlObj.swapRange([scell.rowIndex, scell.colIndex, ecell.rowIndex, ecell.colIndex]);
                    if (range[0] === range[2]) {
                        trange = range;
                        trange[2] = trange[2] + 1;
                    }
                }
                else {
                    if (xlObj._isUndoRedo)
                        trange = $.extend(true, [], range);
                    else {
                        trange = range ? xlObj.getRangeIndices(range) : xlObj.swapRange([scell.rowIndex, scell.colIndex, ecell.rowIndex, ecell.colIndex]);
                        range = trange;
                    }
                    if(xlObj.model.allowDragAndDrop)
						xlObj.XLDragDrap.moveRangeTo(trange, [trange[0] + 1, trange[1], trange[2] + 1, trange[3]]);
                    sheet._startCell = { rowIndex: trange[0], colIndex: trange[1] };
                    sheet._endCell = { rowIndex: trange[2] + 1, colIndex: trange[3] };
                    xlObj.XLSelection.selectRange(sheet._startCell, sheet._endCell);
                    if (xlObj.model.allowAutoFill && !sheet._importRendered)
                        xlObj.XLDragFill.positionAutoFillElement();
                    trange[2] = trange[2] + 1;
                }
                if("dataSource" in tableObj)
                    xlObj.updateRange(sheetIdx, { showHeader: "showHeader" in tableObj ? tableObj.showHeader : true, startCell: xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[0], range[1]), dataSource: tableObj.dataSource });
                xlObj.updateUniqueData({ tableName: 'e-table' + tlength }, range);
                if (!sheet._importRendered)
                    tmgr[tlength] = { name: tname, range: trange};
                tmgr[tlength]["isFilter"] = true;
                xlObj.XLRibbon._isDesignTab = true;
                this._createTable(tlength, obj);
                //xlObj.XLFilter._filterTable(sheetIdx, tlength);
                this._updateTblStyleOptns(tlength, tableObj);
                if(!sheet._importRendered && !tableObj.hideTab)
                    xlObj.XLRibbon._designTabUpdate(tlength, { rowIndex: range[0], colIndex: range[1] });
                if(xlObj.model.allowComments)
                    xlObj.XLComment._updateCmntArrowPos();
            }
            else {
                this._formatAsTableStyle = obj;
                this._createTable(xlObj._getTableID(cname), obj);
                xlObj._undoCollection.pop();
                xlObj._undoCollection.pop();
            }
            xlObj._isFAT = false;
            if (!sheet._importRendered) {
                xlObj.XLRibbon._dirtySelect(xlObj.getRange(range[0], range[1], range[2], range[3]));
                details.tableID = tlength;
                details.tableStyle = tableObj;
                details.afterRange = trange;
                if (!xlObj._isPaste && !xlObj._isUndoRedo)
                    xlObj._completeAction(details);
            }
            xlObj.XLSelection._refreshBorder();
        },

        _createTable: function (tableId, tableObj) {
            var i, j, cells, cell, cellIdx, format, arange, border, len, val, txt, k = 0, xlObj = this.XLObj, tformat = tableObj.format, bStr = "border",
                sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
                var tmgr = sheet._tableManager, range = tmgr[tableId].range;
            tmgr[tableId]["format"] = tformat;
            if (!sheet._importRendered && tableObj.formatName)
                tmgr[tableId]["formatName"] = tableObj.formatName;
            this.clearFormat(range, "border");
            if ("contentLayout" in tformat) {
                i = range[0] + 1;
                len = tformat.contentLayout.length;
                while (i <= range[2]) {
                    format = $.extend(true, {}, tformat.contentLayout[k % len]);
                    if (bStr in format) {
                        arange = xlObj._getProperAlphaRange(sheetIdx, i, range[1], i, range[3]);
                        border = format.border;
                        delete format[bStr];
                    }
                    if (xlObj.getObjectLength(format)) {
                        for (j = range[1]; j <= range[3]; j++) {
                            cellIdx = { rowIndex: i, colIndex: j };
                            if (xlObj.XLEdit.getPropertyValue(i, j, "tableName") != "e-table" + tableId)
                                xlObj.XLEdit._updateDataContainer(cellIdx, { tableName: "e-table" + tableId }, sheetIdx);
                            if (xlObj.XLEdit.getPropertyValue(i, j, "hyperlink"))
                                delete format.color;                            
                            this._updateFormatClass(cellIdx, this._getExtendedClass(cellIdx, format));
                        }
                    }
                    if (border) {
                        border.tableID = tableId;
                        this.applyBorder(border, arange);
                        border = undefined;
                    }
                    k++;
                    i++;
                }
            }
            i = range[0], k =1, format = undefined;
            if ("headerLayout" in tformat) {
                format = $.extend(true, {}, tformat.headerLayout);
                if (bStr in tformat.headerLayout) {
                    arange = xlObj._getProperAlphaRange(sheetIdx, range[0], range[1], range[0], range[3]);
                    border = format.border;
                    delete format[bStr];
                }
            }
            for (j = range[1]; j <= range[3]; j++) {
                cellIdx = { rowIndex: i, colIndex: j };
                val = xlObj.XLEdit.getPropertyValue(i, j, "value2") || "";
                txt = "Column" + k;
                if (!val.length) {
                    xlObj.XLEdit.updateCell(cellIdx, txt);
                    k++;
                }
                else if (val.toLowerCase() === txt.toLowerCase())
                    k++;
                if (xlObj.getObjectLength(format))
                    this._updateFormatClass(cellIdx, this._getExtendedClass(cellIdx, format));
            }
            if (border) {
                border.tableID = tableId;
                this.applyBorder(border, arange);
                border = undefined;
            }
            if ("contentBorder" in tformat) {
                tformat.contentBorder.border.tableID = tableId;
                arange = xlObj._getProperAlphaRange(sheetIdx, range[0] + 1, range[1], range[2], range[3]);
                this.applyBorder(tformat.contentBorder.border, arange);
            }
            $("#" + xlObj._id + "_Ribbon_tsofiltercolumn").ejCheckBox("option", { checked: true });
        },

        _updateTblStyleOptns: function (tdId, tbObj) {
            var xlObj = this.XLObj, sheetIdx = xlObj._getSheetIndex(), sheet = xlObj.getSheet(sheetIdx), tmgr = sheet._tableManager, tmgrDt, tRange;

            if (tdId in tmgr) {
                tmgrDt = tmgr[tdId];
                tRange = tmgrDt.range;

                tmgrDt["isFilter"] = "isFilter" in tbObj ? tbObj.isFilter : true; //Filter Icon update
                if (!tmgrDt["isFilter"])
                    xlObj.XLFilter._clearFilterTableIcon(sheetIdx, tdId);

                xlObj._dupDetails = true;
                if ("firstColumn" in tbObj) { //First column update
                    tmgrDt["firstColumn"] = tbObj.firstColumn;
                    this.format({ style: { "font-weight": tmgrDt["firstColumn"] ? "bold" : "normal" } }, xlObj._getProperAlphaRange(sheetIdx, tRange[0] + 1, tRange[1], tRange[2], tRange[1]));
                }
                if ("lastColumn" in tbObj) { //Last column update
                    tmgrDt["lastColumn"] = tbObj.lastColumn;
                    this.format({ style: { "font-weight": tmgrDt["lastColumn"] ? "bold" : "normal" } }, xlObj._getProperAlphaRange(sheetIdx, tRange[0] + 1, tRange[3], tRange[2], tRange[3]));
                }
                xlObj._dupDetails = false;

                if ("totalRow" in tbObj) //Total row update
                    tmgrDt["totalRow"] = tbObj.totalRow;
            }
        },

        removeTable: function (tableId) { // remove filter
            if (!this.XLObj.model.allowFormatAsTable)
                return;
            var tclass, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), tmgr = xlObj.getSheet(sheetIdx)._tableManager;
            if (tableId in tmgr) {
                tclass = "e-table" + tableId;
                xlObj.element.find("." + tclass).removeClass(tclass);
                (!xlObj._isFATResize) && xlObj.XLFilter._clearFilterTable(sheetIdx, parseInt(tableId));
                delete tmgr[tableId];
                delete xlObj.model.sheets[sheetIdx]._header[tableId];
                $("#" + xlObj._id + "_Ribbon_tsofiltercolumn").ejCheckBox("option", { checked: false });
                xlObj.XLRibbon._toggleDesignTab();
            }
        },

        _getTableLayoutFromName: function (name) {
            if (!name)
                return;
            var data, tables = $("#" + this.XLObj._id + "_formatastable").find(".e-formatastablecell"), i = tables.length - 1;
            while (i--) {
                data = $(tables[i]).data();
                if (data.formatName === name)
                    return data;
            }
        },

        _getTableIdxFromName: function (name) {
            var xlObj = this.XLObj, tmgr = xlObj.getSheet(xlObj.getActiveSheetIndex())._tableManager, len = xlObj.getObjectLength(tmgr);
            if (len) {
                while (len--) {
                    if (tmgr[len].name === name)
                        return len;
                }
            }
            return -1;
        },

        customFormatParser: function (input) {  // if returned object is empty then parsing failed 
            if (!this.XLObj.model.allowCellFormatting)
                return;
            var str, type, splitter, formatObj = {}, ctype = ej.Spreadsheet.CellType, pre = "{0:", suf = "}", n = "N", p = "P", d = ".", c = ",", z = "0";
            input = ej.isNullOrUndefined(input) ? "" : input.toString();
            if (Number(input) === 0) {
                if (input.indexOf(d) > -1) {
                    splitter = input.split(d);
                    str = pre + n + splitter[1].length + suf;
                }
                else
                    str = pre + "D" + input.length + suf;
                type = ctype.Number;
            }
            else if (!input.indexOf("#") && input.indexOf(c) > -1 && input.lastIndexOf(z) === input.length - 1) {  // thousand separator
                if (input.indexOf(d) > -1) {
                    splitter = input.split(d);
                    str = pre + n + splitter[1].length + suf;
                }
                else
                    str = pre + n + z + suf;
                formatObj["thousandSeparator"] = true;
                type = ctype.Number;
            }
            else if (input.lastIndexOf("%") === input.length - 1 && Number(input.replace(/%/g, "")) === 0) {  // percentage
                if (input.indexOf(d) > -1) {
                    splitter = input.split(d);
                    str = pre + p + --splitter[1].length + suf;
                }
                else
                    str = pre + p + z + suf;
                type = ctype.Percentage;
            }
            else if (/^[dMy/-]+$/.test(input)) {  // date & time
                str = pre + input + suf;
                type = ctype.Date;
            }
            else if (/^[hmst:\s]+$/.test(input)) {
                str = pre + input + suf;
                type = ctype.Time;
            }
            str && (formatObj["formatStr"] = str, formatObj["type"] = type);
            return formatObj;
        },

        applyBorder: function (prop, range, details) {
            if (!this.XLObj.model.allowCellFormatting || (this.XLObj.model.allowCellFormatting && !this.XLObj.model.formatSettings.allowCellBorder) || (!this.XLObj.getObjectLength(prop) && $.type(prop) !== "string"))
                return;
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), scell = sheet._startCell, ecell = sheet._endCell;
            range = range ? xlObj.getRangeIndices(range) : xlObj.swapRange([scell.rowIndex, scell.colIndex, ecell.rowIndex, ecell.colIndex]);
            if (!sheet._importRendered) {
                !details && (details = { sheetIndex: sheetIdx, reqType: "cell-format", process: "border" });
                details.range = range;
                details.beforeFormat = this.getHashCodeClassAsArray(range);
            }
            xlObj.getObjectLength(prop) ? this._applyBorderCss(prop, range, details) : prop === "noborder" ? this.clearFormat(range, "border") : this._applyAllBorderCss(prop, range, details);
            xlObj._refreshRangeTRHght(sheetIdx, range);
            if (!sheet._importRendered && !xlObj._isUndoRedo)
                if (ej.isNullOrUndefined(prop.tableID) && !this._dupDetails) {
                    details.afterFormat = this.getHashCodeClassAsArray(range);
                    xlObj._completeAction(details);
                }
        },

        updateDecimalPlaces: function (type, range) {
            if (!this.XLObj.model.allowCellFormatting || (this.XLObj.model.allowCellFormatting && !this.XLObj.model.formatSettings.allowDecimalPlaces))
                return;
            var cell, rowIdx, colIdx, len, dataAttr, temp, i = 0, obj = {}, cellObj = {}, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(),
                container = xlObj._dataContainer, details = { action: "decimal", reqType: "cell-format", sheetIndex: sheetIdx }, selCells = [];
            range = xlObj._getAlphaRangeCells(range);
            len = range.length;
            if (len) {
                while (i < len) {
                    rowIdx = range[i].parentNode.rowIndex; colIdx = range[i].cellIndex;
                    dataAttr = xlObj.getRangeData([rowIdx, colIdx, rowIdx, colIdx])[0];
                    cell = { rowIndex: rowIdx, colIndex: colIdx, afterFormat: {}, beforeFormat: {} };
                    if (!ej.isNullOrUndefined(container.sheets[sheetIdx][rowIdx]))
                        cellObj = ej.isNullOrUndefined(container.sheets[sheetIdx][rowIdx][colIdx]) ? {} : container.sheets[sheetIdx][rowIdx][colIdx];
                    else
                        cellObj = {};
                    $.extend(true, cell.beforeFormat, cellObj);
                    if (xlObj.isNumber(dataAttr.value)) {
                        xlObj.isUndefined(dataAttr.decimalPlaces) && (dataAttr.decimalPlaces = 0);
                        obj["type"] = dataAttr.type === ej.Spreadsheet.CellType.General ? ej.Spreadsheet.CellType.Number : dataAttr.type;
                        obj["decimalPlaces"] = dataAttr.decimalPlaces = type === "decrement" ? dataAttr.decimalPlaces > 0 ? dataAttr.decimalPlaces - 1 : 0 : dataAttr.decimalPlaces + 1;
                        obj["formatStr"] = this._getFormatString(obj.type, obj.decimalPlaces);
                        temp = this._format(dataAttr.value, { formatStr: obj.formatStr, type: obj.type, thousandSeparator: dataAttr.thousandSeparator, decimalPlaces: obj.decimalPlaces });
                        obj["value2"] = temp;
                        xlObj.XLEdit._updateDataContainer({rowIndex: rowIdx, colIndex: colIdx}, obj);
                        cell.beforeFormat.Oprtype = obj.type;
                    }
                    if (!ej.isNullOrUndefined(container.sheets[sheetIdx][rowIdx]))
                        cellObj = ej.isNullOrUndefined(container.sheets[sheetIdx][rowIdx][colIdx]) ? {} : container.sheets[sheetIdx][rowIdx][colIdx];
                    else
                        cellObj = {};
                    $.extend(true, cell.afterFormat, cellObj);
                    cell.afterFormat.Oprtype = cell.beforeFormat.Oprtype;
                    selCells.push(cell);
                    i++;
                }
            }
            details.selCell = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        getFormatClass: function (classname, isborder) {
            var str = isborder ? "e-border" : "e-format", regx = new RegExp('\\b' + str + '.*?\\b', 'g');
            return classname.indexOf(str) !== -1 ? classname.match(regx)[0] : "";
        },

        getHashCodeClassAsArray: function (range) { //range as array indexes
            var i = 0, obj = { format: [] }, xlObj = this.XLObj, arr = ["topborder", "leftborder"], cname;
            range = range ? xlObj.swapRange(range) : xlObj.getSheet(xlObj.getActiveSheetIndex()).selectedRange;
            var cells = xlObj.getRange(range[0], range[1], range[2], range[3]), length = cells.length;
            while (i < length) {
                cname = cells[i].className;
                obj.format.push(this.getFormatClass(cname) + " " + this.getFormatClass(cname, true));
                i++;
            }
            i = 2;
            while (i--)
                obj[arr[i]] = this._getBorderArray(range, arr[i]);
            return obj;
        },

        _getBorderArray: function (range, position) {
            var length, i = 0, arr = [], xlObj = this.XLObj, str = "topborder", minindex = position === str ? range[0] - 1 : range[1] - 1, cells;
            if (minindex >= 0) {
                cells = position === str ? xlObj.getRange(range[0] - 1, range[1], range[0] - 1, range[3]) : xlObj.getRange(range[0], range[1] - 1, range[2], range[1] - 1);
                length = cells.length;
                while (i < length) {
                    arr.push(this.getFormatClass(cells[i].className, true));
                    i++;
                }
            }
            else {
                length = position === str ? (range[3] - range[1]) + 1 : (range[2] - range[0]) + 1;
                while (length--)
                    arr.push("");
            }
            return arr;
        },

        clearFormat: function (range, type, skipHiddenRow) {
            if (!this.XLObj.model.allowCellFormatting)
                return;
            var i, $cell, cell, cname, xlObj = this.XLObj, prop = [], pos = ["topborder", "leftborder"], frmt = "format", bdr = "border", format = !type || type === frmt, border = !type || type === bdr;
            range = range ? xlObj.swapRange(range) : xlObj.getSheet(xlObj.getActiveSheetIndex()).selectedRange;
            var cells = xlObj.getRange(range[0], range[1], range[2], range[3]);
            i = cells.length;
            while (i--) {
                cell = cells[i], $cell = $(cell);
				if (!skipHiddenRow || !xlObj._isHiddenRow($cell.parent().index())) {
					cname = cell.className;
					if(format)
					    $cell.removeClass(this.getFormatClass(cname));
					if(border)
					    $cell.removeClass(this.getFormatClass(cname, true));
				}
            }
            if(format)
                prop.push(frmt);
            if(border)
                prop.push(bdr);
            xlObj.clearRangeData(null, prop, cells, skipHiddenRow);
            if (border) {
                i = 2;
                while (i--)
                    this._clearBorderClass(range, pos[i], skipHiddenRow);
            }
        },

        _clearBorderClass: function (range, position, skipHiddenRow) {
            var i, cells, cell, obj, sepstr, cname, xlObj = this.XLObj, str = "topborder", index = position === str ? range[0] - 1 : range[1] - 1;
            if (index >= 0) {
                cells = position === str ? xlObj.getRange(range[0] - 1, range[1], range[0] - 1, range[3]) : xlObj.getRange(range[0], range[1] - 1, range[2], range[1] - 1);
                i = cells.length;
                while (i--) {
                    cell = cells[i];
					if (!skipHiddenRow || !xlObj._isHiddenRow($cell.parent().index())) {
						cname = this.getFormatClass(cell.className, true);
						if (cname.length) {
							sepstr = cname.split("S");
							obj = this._getBorderFromHashCode(sepstr[1]);
							"border" in obj && (delete obj.border[position === str ? "bottom" : "right"], sepstr[1] = this._getBorderHashCode(obj.border), (!xlObj.getObjectLength(obj.border) && delete obj["border"]));
							$.extend(true, obj, this._getBorderFromHashCode(sepstr[0].replace("e-border", "")));
							if (xlObj.getObjectLength(obj))
								this._updateFormatClass(cell, this._createFormatClass(obj, sepstr[0] + "S" + sepstr[1]), true);
							else {
								$(cell).removeClass(cname);
								xlObj.clearRangeData(null, ["border"], [cell], skipHiddenRow);
							}
						}
                    }
                }
            }
        },

        updateFormat: function (formatobj, range) {
            var i, pos = ["topborder", "leftborder"], xlObj = this.XLObj;
            range = range ? xlObj.swapRange(range) : xlObj.getSheet(xlObj.getActiveSheetIndex()).selectedRange;
            var cells = xlObj.getRange(range[0], range[1], range[2], range[3]), arr = formatobj.format;
            i = cells.length;
            while (i--)
                this._updateHashCodeClass(cells[i], arr[i]);
            i = 2;
            while (i--)
                this._updateBorderClass(range, formatobj[pos[i]], pos[i]);
        },

        _updateBorderClass: function (range, format, position) {
            var i, cells, xlObj = this.XLObj, str = "topborder", index = position === str ? range[0] - 1 : range[1] - 1;
            if (index >= 0) {
                cells = position === str ? xlObj.getRange(range[0] - 1, range[1], range[0] - 1, range[3]) : xlObj.getRange(range[0], range[1] - 1, range[2], range[1] - 1);
                i = cells.length;
                while (i--)
                    this._updateFormatClass(cells[i], format[i], true);
            }
        },

        updateUniqueFormat: function (formatclass, range) {
            var  xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), range = range ? range : xlObj.getSheet(sheetIdx).selectedRange, cells = xlObj._getArrayRangeCells(range), i = cells.length + 1;
            while (i = i - 1)                
                this._updateFormatClass(cells[i - 1], formatclass);           
            xlObj._refreshRangeTRHght(sheetIdx, xlObj.swapRange(range));
        },

        _hasFormat: function (formatclass) {
            if (formatclass)
                return !ej.isNullOrUndefined(formatclass.match(/e-format|e-border/));
            return false;
        },

        getFormatHashCode: function (style) {
            var code = "";
            code = "background-color" in style ? style["background-color"].replace("#", "") : "6N";
            code = "color" in style ? code + style["color"].replace("#", "") : code + "6N";
            code = "font-family" in style ? ej.isNullOrUndefined(this._getStyleCode("FontFamily", style["font-family"])) ? code + "2N" : code + this._getStyleCode("FontFamily", style["font-family"]) : code + "2N";
            code = "font-size" in style ? code + this._getFontCode(style["font-size"]) : code + "2N";  // consider
            code = "font-style" in style ? ej.isNullOrUndefined(this._getStyleCode("FontStyle", style["font-style"])) ? code + "1N" : code + this._getStyleCode("FontStyle", style["font-style"]) : code + "1N";
            code = "text-decoration" in style ? ej.isNullOrUndefined(this._getStyleCode("TextDecoration", style["text-decoration"])) ? code + "1N" : code + this._getStyleCode("TextDecoration", style["text-decoration"]) : code + "1N";
            code = "vertical-align" in style ? ej.isNullOrUndefined(this._getStyleCode("VerticalAlign", style["vertical-align"])) ? code + "1N" : code + this._getStyleCode("VerticalAlign", style["vertical-align"]) : code + "1N";
            code = "text-align" in style ? ej.isNullOrUndefined(this._getStyleCode("TextAlign", style["text-align"])) ? code + "1N" : code + this._getStyleCode("TextAlign", style["text-align"]) : code + "1N";
            code = "text-indent" in style ? code + this._getFontCode(style["text-indent"]) : code + "2N";
            code = "font-weight" in style ? ej.isNullOrUndefined(this._getStyleCode("FontWeight", style["font-weight"])) ? code + "1N" : code + this._getStyleCode("FontWeight", style["font-weight"]) : code + "1N";
            return "e-format" + code;
        },

        getFormatFromHashCode: function (code) {
            var style = {}, temp, i = 0, str = "e-format";
            code = code || "";
            if (!code.indexOf(str)) {
                temp = code.replace(str, "").split("N");
                temp[i].length === 1 ? i = i + 1 : (style["background-color"] = "#" + temp[i].slice(0, 6), temp[i] = temp[i].substr(6));
                temp[i].length === 1 ? i = i + 1 : (style["color"] = "#" + temp[i].slice(0, 6), temp[i] = temp[i].substr(6));
                temp[i].length === 1 ? i = i + 1 : (temp[i].length === 2 ? (style["font-family"] = this._getCodeStyle("FontFamily", temp[i].slice(0, 1) + "1N"), i = i + 1) : (style["font-family"] = this._getCodeStyle("FontFamily", temp[i].slice(0, 2)), temp[i] = temp[i].substr(2)));
                temp[i].length === 1 ? i = i + 1 : (temp[i].length === 2 ? (style["font-size"] = temp[i].slice(0, 1) + "pt", i = i + 1) : (style["font-size"] = temp[i].slice(0, 2) + "pt", temp[i] = temp[i].substr(2)));
                temp[i].length === 1 ? i = i + 1 : (style["font-style"] = this._getCodeStyle("FontStyle", temp[i].slice(0, 1)), temp[i] = temp[i].substr(1));
                temp[i].length === 1 ? i = i + 1 : (style["text-decoration"] = this._getCodeStyle("TextDecoration", temp[i].slice(0, 1)), temp[i] = temp[i].substr(1));
                temp[i].length === 1 ? i = i + 1 : (style["vertical-align"] = this._getCodeStyle("VerticalAlign", temp[i].slice(0, 1)), temp[i] = temp[i].substr(1));
                temp[i].length === 1 ? i = i + 1 : (style["text-align"] = this._getCodeStyle("TextAlign", temp[i].slice(0, 1)), temp[i] = temp[i].substr(1));
                temp[i].length === 1 ? i = i + 1 : (temp[i].length === 2 ? (style["text-indent"] = temp[i].slice(0, 1) + "px", i = i + 1) : (style["text-indent"] = temp[i].slice(0, 2) + "px", temp[i] = temp[i].substr(2)));
                temp[i] === "2" && (style["font-weight"] = "bold");
            }
            return style;
        },

        getBorderFromHashCode: function (code, isImaginary) {
            var obj = {};
            code = code || "";
            if (!code.indexOf("e-border")) {
                code = code.replace("e-border", "").split("S")[isImaginary ? 1 : 0];
                obj = this._getBorderFromHashCode(code);
            }
            return obj;
        },

        _getBorderFromHashCode: function (code) {
            var temp, i = 0, obj = {};
            code = code || "";
            if (code.length > 12) {
                temp = code.split("N"), obj = { border: {} };
                temp[i].length > 1 ? (obj.border["right"] = this._concatBorderCode(temp[i]), temp[i] = temp[i].substr(8)) : i = i + 3;
                temp[i].length > 1 && (obj.border["bottom"] = this._concatBorderCode(temp[i]))
            }
            return obj;
        },

        //FormatPainter
        _formatPainter: function () {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowFormatPainter || !xlObj.model.allowCellFormatting)
                return;
            var sheetIdx = xlObj.getActiveSheetIndex(), i, j, cellIdx = 0, startCell = xlObj.getSheet(sheetIdx)._startCell, endCell = xlObj.getSheet(sheetIdx)._endCell;
            this._rowLength = (endCell.rowIndex - startCell.rowIndex) + 1, this._colLength = (endCell.colIndex - startCell.colIndex) + 1;
            var rowLen = this._rowLength, colLen = this._colLength, getData;
            this._getFormat = this.getHashCodeClassAsArray([startCell.rowIndex, startCell.colIndex, endCell.rowIndex, endCell.colIndex]);
            getData = xlObj.getRangeData([startCell.rowIndex, startCell.colIndex, endCell.rowIndex, endCell.colIndex], null, ["value", "value2", "type","wrap","formatStr", "thousandSeparator", "decimalPlaces", "cFormatRule", "hyperlink"]);
            this._getEformatClass = new Array(rowLen);
            for (i = 0; i < rowLen; i++) {
                this._getEformatClass[i] = new Array(colLen);
                for (j = 0; j < colLen; j++) {
                    this._getEformatClass[i][j] = { "eFormat": this._getFormat.format[cellIdx], "rangeData": getData[cellIdx] };
                    cellIdx++;
                }
            }
        },

        _fPMouseUp: function (e) {
            var obj, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), getSheet = xlObj.getSheet(sheetIdx), sheetCont = xlObj._getContent(sheetIdx).find("div:first-child"), colcount = 0, topborder = 0, leftborder = 0, cFRule, fClass;
            var getCell = getSheet._selectedCells, startCell = getCell[0], endCell = getCell[getCell.length - 1], i, j, cellIndex = 0, cellCount = 0, cFormatStr, rule, cellAddr, len,k;
            var cells = xlObj._getArrayRangeCells([startCell.rowIndex, startCell.colIndex, endCell.rowIndex, endCell.colIndex]), cellRange = xlObj.XLSelection._getSelectedRange(startCell, endCell);
            var rowlength = (endCell.rowIndex - startCell.rowIndex), collength = (endCell.colIndex - startCell.colIndex), alphaRange = new Array(cellRange.length);
            var details = { sheetIndex: sheetIdx, reqType: "format-painter" }, selCell = [], cell, fregx = xlObj._formatRegx, bregx = xlObj._borderRegx;
            for (i = 0, len = cellRange.length ; i < len; i++) {
                alphaRange[i] = xlObj._generateHeaderText(sheetIdx, cellRange[i].colIndex + 1) + (cellRange[i].rowIndex + 1) + ":" + xlObj._generateHeaderText(sheetIdx, cellRange[i].colIndex + 1) + (cellRange[i].rowIndex + 1);
            }
			xlObj._dupDetails = true;
            for (i = 0; i <= rowlength; i++) {
                cellCount++;
                for (j = 0; j <= collength; j++) {
                    colcount++;
                    cell = { rowIndex: $(cells[cellIndex]).parent().index(), colIndex: cells[cellIndex].cellIndex };
                    cell.bVal = $(cells[cellIndex]).text();
                    $.extend(true, cell.bData, $(cells[cellIndex]).data());
                    cell.bFormat = (cells[cellIndex].className.indexOf("e-format") !== -1) ? cells[cellIndex].className.match(fregx)[0] : "";
                    //cell.bBorder = (cells[cellIndex].className.indexOf("e-border") !== -1) ? cells[cellIndex].className.match(bregx)[0] : "";
                    if (this._getEformatClass[i][j].eFormat === " ")
                        this.clearFormat([getCell[cellIndex].rowIndex, getCell[cellIndex].colIndex, getCell[cellIndex].rowIndex, getCell[cellIndex].colIndex], "format");
                    //this._updateFormatClass(cells[cellIndex], this._getEformatClass[i][j].eFormat);
                    fClass = this._getEformatClass[i][j].eFormat.split(" ");
                    if (fClass[0].indexOf("e-format") > -1)
                        this._updateFormatClass(cells[cellIndex], fClass[0]);
                    else if (fClass[1].indexOf("e-format") > -1)
                        this._updateFormatClass(cells[cellIndex], fClass[1]);
                    obj = { type: this._getEformatClass[i][j].rangeData.type ? this._getEformatClass[i][j].rangeData.type : "general" };
                    if(this._getEformatClass[i][j].rangeData.formatStr)
                        obj.formatStr = this._getEformatClass[i][j].rangeData.formatStr;
                    if(this._getEformatClass[i][j].rangeData.decimalPlaces)
                        obj.decimalPlaces = this._getEformatClass[i][j].rangeData.decimalPlaces;
                    if(this._getEformatClass[i][j].rangeData.thousandSeparator)
                        obj.thousandSeparator = this._getEformatClass[i][j].rangeData.thousandSeparator;
                    this.format(obj, alphaRange[cellIndex]);
                    xlObj.XLEdit.updateCell(cells.eq(cellIndex), cells.eq(cellIndex).text());
                    if (!ej.isNullOrUndefined(this._getEformatClass[i][j].rangeData.cFormatRule)) {
                        for (k = 0, len = this._getEformatClass[i][j].rangeData.cFormatRule.length; k < len; k++) {
                            cFRule = this._getEformatClass[i][j].rangeData.cFormatRule[k];
                            rule = cFRule.split("_");
                            cellAddr = rule[5];
                            rule[rule.indexOf(cellAddr)] = alphaRange[cellIndex];
                            xlObj._dataContainer.cFormatRule = rule.join("_");
                            cFormatStr = xlObj._dataContainer.cFormatRule.split("_");
                            xlObj.XLCFormat._cFormat(cFormatStr[0], cFormatStr[2], cFormatStr[3], cFormatStr[4], cFormatStr[5]);
                        }
                    }
                    if (cells[cellIndex].className.indexOf("e-sswraptext") > -1 && ej.isNullOrUndefined(this._getEformatClass[i][j].rangeData.wrap))
                        xlObj.setWrapText(alphaRange[cellIndex], "unwrap");
                    else if (cells[cellIndex].className.indexOf("e-sswraptext") < 0 && !ej.isNullOrUndefined(this._getEformatClass[i][j].rangeData.wrap))
                        xlObj.setWrapText(alphaRange[cellIndex], "wrap");

                    if (!ej.isNullOrUndefined(this._getEformatClass[i][j].rangeData.hyperlink) && xlObj.isUndefined(xlObj.XLEdit.getPropertyValue(getCell[cellIndex].rowIndex, getCell[cellIndex].colIndex, "hyperlink", sheetIdx))) {
                        this.format({ style: { "text-decoration": "underline" } }, alphaRange[cellIndex]);
                    }
                    else if (xlObj.isUndefined(this._getEformatClass[i][j].rangeData.hyperlink) && !xlObj.isUndefined(xlObj.XLEdit.getPropertyValue(getCell[cellIndex].rowIndex, getCell[cellIndex].colIndex, "hyperlink", sheetIdx)))
                        cells.eq([cellIndex]).find("a").css("text-decoration", "none");
        
                    cell.aVal = $(cells[cellIndex]).text()
                    cell.aData = $(cells[cellIndex]).data();
                    cell.aFormat = (cells[cellIndex].className.indexOf("e-format") !== -1) ? cells[cellIndex].className.match(fregx)[0] : "";
                    //cell.aBorder = (cells[cellIndex].className.indexOf("e-border") !== -1) ? cells[cellIndex].className.match(bregx)[0] : "";
                    selCell.push(cell);
                    cellIndex++;
                    //if (i === 0) {    // border not supported in formatpainter
                    //    startCell.rowIndex > 0 && this._updateFormatClass(xlObj.getCell(startCell.rowIndex - 1, startCell.colIndex + (colcount - 1), sheetIdx)[0], this._getFormat.topborder[topborder]);
                    //    topborder++;
                    //    (this._getFormat.topborder.length === topborder) && (topborder = 0);
                    //}
                    //if (j === 0) {
                    //    startCell.colIndex > 0 && this._updateFormatClass(xlObj.getCell(startCell.rowIndex + (cellCount - 1), startCell.colIndex - 1, sheetIdx)[0], this._getFormat.leftborder[leftborder]);
                    //    leftborder++;
                    //    (this._getFormat.topborder.length === leftborder) && (leftborder = 0);
                    //}
                    if (colcount === collength + 1) {
                        colcount = 0;
                        break;
                    }
                    (this._colLength === j + 1) && (j = -1);
                }
                if (cellCount === rowlength + 1)
                    break;
                (this._rowLength === i + 1) && (i = -1);
            }
            cells = xlObj.getCell(selCell[i].rowIndex, selCell[i].colIndex)[0];
            selCell[i].aBorder = (cells.className.indexOf("e-border") !== -1) ? cells.className.match(bregx)[0] : "";
            this._formatEnable = false;
            sheetCont.addClass("e-ss-cursor");
            sheetCont.removeClass("e-ss-fpcursor");
            $("#" + xlObj._id + "_Ribbon_formatpainter").ejToggleButton("option", { toggleState: false });
            details.selCells = selCell;
			xlObj._dupDetails = false;
			xlObj.XLSelection.refreshSelection();
			xlObj.XLDragFill.positionAutoFillElement();
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _getFormatObj: function (obj) {
            if ("type" in obj || "formatStr" in obj) {
                var format = {}, sep = "thousandSeparator";
                format["formatStr"] = "formatStr" in obj ? obj.formatStr : this._getFormatString(obj.type, obj.decimalPlaces);
                format["decimalPlaces"] = "decimalPlaces" in obj ? obj.decimalPlaces : ["currency", "accounting", "number"].indexOf(obj.type) !== -1 ? parseInt(format.formatStr.replace(/{0:|[A-Z]|}/g, "")) : 0;
                if (sep in obj)
                    format[sep] = obj[sep];
                else if (obj.type === "number")
                    format[sep] = true;
                obj["format"] = format;
            }
            if ("style" in obj && obj.style.border) {
                obj["border"] = obj.style.border;
                delete obj.style.border;
                !this.XLObj.getObjectLength(obj.style) && delete obj.style;
            }
            return obj;
        },

        _getFormatString: function (format, decimalplace) {
            var formatstr = ej.Spreadsheet.CellFormat[format];
            if (this.XLObj.isNumber(decimalplace) && !ej.isNullOrUndefined(formatstr))
                formatstr = "{0:" + formatstr.substr(3, 1) + decimalplace + "}";
            return formatstr || "";
        },

        _createFormatClass: function (args, hashcode) {
            var container = this.XLObj._dataContainer.hashCode;
            hashcode = hashcode ? hashcode : this.getFormatHashCode(args);
            if (container.indexOf(hashcode) !== -1)
                return hashcode;
            var properties = this._getCssPropertyAsString(args);
            this._writeCssRules(hashcode, properties);
            container.push(hashcode);
            return hashcode;
        },

        _writeCssRules: function (hashcode, rules) {
            var stag = document.getElementById(this.XLObj._id + "_sscustomformat"), prop = ".e-spreadsheet ." + hashcode + " {" + rules + "}";
            stag.styleSheet ? stag.styleSheet.cssText += prop : stag.appendChild(document.createTextNode(prop));
        },

        _getCssPropertyAsString: function (args) {
            var str = "", prop, j, k;
            if ("border" in args) {
                prop = args.border;
                for (j in prop)
                    args["border-" + j] = prop[j];
                delete args.border;
            }
            for (k in args)
                str = str + (k + ":" + args[k] + ";");
            return str;
        },

        _splitBorderCode: function (code) {
            code = code.split(" ");
            return code[0].replace("px", "") + this._getStyleCode("BorderStyle", code[1]) + code[2].replace("#", "");
        },

        _concatBorderCode: function (code) {
            return code.substr(0, 1) + "px " + this._getCodeStyle("BorderStyle", code.substr(1, 1)) + " #" + code.substr(2, 6);
        },

        _getCodeStyle: function (obj, value) {
            obj = ej.Spreadsheet[obj];
            for (var prop in obj) {
                if (obj.hasOwnProperty(prop)) {
                    if (obj[prop] === value)
                        return prop;
                }
            }
        },

        _getStyleCode: function (prop, val) {
            val = val || "";
            return ej.Spreadsheet[prop][val.toLowerCase()];
        },

        _getFontCode: function (args) {
            var temp = args.replace(/pt|-|\+/g, "");
            return temp.length === 2 ? temp : (temp.length === 1 ? temp + "1N" : "2N");
        },

        _getExtendedClass: function (cellIdx, style) {
            var format = this.XLObj.XLEdit.getPropertyValue(cellIdx.rowIndex, cellIdx.colIndex, "format") || "",
                obj = this.getFormatFromHashCode(format);
            $.extend(true, obj, style);
            return this._createFormatClass(obj);
        },

        _updateHashCodeClass: function (cell, cssclass) {
            var format, border, xlObj = this.XLObj, frmt = "e-format", bdr = "e-border", obj = {}, fregx = xlObj._formatRegx, bregx = xlObj._borderRegx;
            if (!ej.isNullOrUndefined(cssclass)) {
                cell.className = cell.className.replace(fregx, '').replace(bregx, '');
                format = cssclass.indexOf(frmt) > -1;
                border = cssclass.indexOf(bdr) > -1;
                if (format)
                    obj.format = cssclass.match(fregx)[0];
                if (border)
                    obj.border = cssclass.match(bregx)[0];
                $(cell).addClass(cssclass);
                (format || border) && xlObj.XLEdit._updateDataContainer({rowIndex: cell.parentNode.rowIndex, colIndex: cell.cellIndex}, obj);
            }
        },

        _updateFormatClass: function (cell, cssclass, isborder) {
            var format = "e-format", border = "e-border";
            if (cssclass && (cssclass.indexOf(format) > -1 || cssclass.indexOf(border) > -1))                                
                this.XLObj.XLEdit._updateDataContainer(cell, isborder ? { border: cssclass } : { format: cssclass });            
        },

        _applyBorderCss: function (prop, range, details) {
            var i, borderpos = this._borderPosition;
            if(borderpos[0] in prop && range[0] - 1 >= 0)
                this._updateBorder([range[0] - 1, range[1], range[0] - 1, range[3]], this._parseBorder(borderpos[2], prop.top), true);
            if(borderpos[1] in prop)
                this._updateBorder([range[0], range[3], range[2], range[3]], this._parseBorder(borderpos[1], prop.right));
            if(borderpos[2] in prop)
                this._updateBorder([range[2], range[1], range[2], range[3]], this._parseBorder(borderpos[2], prop.bottom));
            if(borderpos[3] in prop && range[1] - 1 >= 0)
                this._updateBorder([range[0], range[1] - 1, range[2], range[1] - 1], this._parseBorder(borderpos[1], prop.left), true);
        },

        _applyAllBorderCss: function (prop, range, details) {
            var borderpos = this._borderPosition;
            this._updateBorder(range, this._parseBorder(borderpos[1] + " " + borderpos[2], prop));
            range[0] - 1 >= 0 && this._updateBorder([range[0] - 1, range[1], range[0] - 1, range[3]], this._parseBorder(borderpos[2], prop), true);
            range[1] - 1 >= 0 && this._updateBorder([range[0], range[1] - 1, range[2], range[1] - 1], this._parseBorder(borderpos[1], prop), true);
        },

        _updateBorder: function (range, prop, isDuplicate) {
            var k, l, cellIdx, cssclass, i = range[0], j = range[2], xlObj = this.XLObj;
            while (i <= j) {
                k = range[1], l = range[3];
                while (k <= l) {
                    cellIdx = { rowIndex: i, colIndex: k };
                    if (!(isDuplicate && xlObj.hasSpan(cellIdx))) {
                        cssclass = this._extendBorder(xlObj.XLEdit.getPropertyValue(i, k, "border") || "", prop, isDuplicate);
                        this._updateFormatClass(cellIdx, cssclass, true);
                    }
                    k++;
                }
                i++;
            }
        },

        _extendBorder: function (classname, style, isduplicate) {
            var obj;
            classname = classname.length ? classname.replace("e-border", "").split("S") : "1N1N6N1N1N6NS1N1N6N1N1N6N".split("S");
            if (isduplicate) {
                obj = this._getBorderFromHashCode(classname[1]);
                $.extend(true, obj, style);
                classname[1] = this._getBorderHashCode(obj.border);
                obj = $.extend(true, this._getBorderFromHashCode(classname[0]), obj);
            }
            else {
                obj = this._getBorderFromHashCode(classname[0]);
                $.extend(true, obj, style);
                classname[0] = this._getBorderHashCode(obj.border);
                obj = $.extend(true, this._getBorderFromHashCode(classname[1]), obj);
            }
            return this._createFormatClass(obj, "e-border" + classname[0] + "S" + classname[1]);
        },

        _getBorderHashCode: function (args) {
            var code = "";
            code = "right" in args ? this._splitBorderCode(args["right"]) : "1N1N6N";
            code = code + ("bottom" in args ? this._splitBorderCode(args["bottom"]) : "1N1N6N");
            return code;
        },

        _parseBorder: function (prop, value) {
            var i = 0, obj = { border: {} };
            prop = prop.split(" ");
            while (i < prop.length) {
                obj.border[prop[i]] = value;
                i++;
            }
            return obj;
        }
    };

    ej.Spreadsheet.FontFamily = {
        "angsana new": "11N",
        "arial": "21N",
        "arial black": "31N",
        "batang": "41N",
        "book antiqua": "51N",
        "browallia new": "61N",
        "calibri": "71N",
        "cambria": "81N",
        "candara": "91N",
        "century": "10",
        "comic sans ms": "11",
        "consolas": "12",
        "constantia": "13",
        "corbel": "14",
        "cordia new": "15",
        "courier": "16",
        "courier new": "17",
        "dilleniaupc": "18",
        "dotum": "19",
        "fangsong": "20",
        "garamond": "21",
        "georgia": "22",
        "gulim": "23",
        "gungsuh": "24",
        "kaiti": "25",
        "jasmineupc": "26",
        "malgun gothic": "27",
        "mangal": "28",
        "meiryo": "29",
        "microsoft jhenghei": "30",
        "microsoft yahei": "31",
        "mingliu": "32",
        "mingliu_hkscs": "33",
        "ms gothic": "34",
        "ms mincho": "35",
        "ms pgothic": "36",
        "ms pmincho": "37",
        "pmingliu": "38",
        "pmingliu-extb": "39",
        "simhei": "40",
        "simsun": "41",
        "simsun-extb": "42",
        "tahoma": "43",
        "times": "44",
        "times new roman": "45",
        "trebuchet ms": "46",
        "verdana": "47",
        "yu gothic": "48",
        "yu mincho": "49"
    };

    ej.Spreadsheet.FontStyle = {
        "normal": "1",
        "italic": "2",
        "oblique": "3"
    };

    ej.Spreadsheet.TextAlign = {
        "left": "1",
        "right": "2",
        "center": "3",
        "justify": "4"
    };

    ej.Spreadsheet.VerticalAlign = {
        "top": "1",
        "middle": "2",
        "bottom": "3"
    };

    ej.Spreadsheet.TextDecoration = {
        "none": "1",
        "underline": "2",
        "overline": "3",
        "line-through": "4",
        "line-through underline": "5"
    };

    ej.Spreadsheet.FontWeight = {
        normal: "1",
        bold: "2"
    };

    ej.Spreadsheet.BorderStyle = {
        solid: "1",
        dashed: "2",
        dotted: "3"
    };

    ej.Spreadsheet.CellFormat = {
        number: "{0:N2}",
        currency: "{0:C2}",
        accounting: "{0:C2}",
        percentage: "{0:P2}",
        shortdate: "{0:MM/dd/yyyy}",
        longdate: "{0:dddd, MMMM dd, yyyy}",
        time: "{0:T}",
        scientific: "{0:N2}",
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.cFormat = function (obj) {
        this.XLObj = obj;
    };

    ej.spreadsheetFeatures.cFormat.prototype = {

        //Conditional formatting   

        getCFRule: function (rowIdx, colIdx) {
            var xlObj = this.XLObj;
            var ruleColl = xlObj._dataContainer.sheets[xlObj.getActiveSheetIndex()][rowIdx][colIdx].cFormatRule;
            var rules = [], names = xlObj._dataContainer.cFormatData, splitStr;
            if (ruleColl) {
                for (var i = 0, len = ruleColl.length; i < len; i++) {
                    splitStr = names[ruleColl[i]].split("_");
                    rules.push({ "action": splitStr[0], "input1": splitStr[2], "input2": splitStr[3], "color": splitStr[4], "range": splitStr[5], "isApplied": JSON.parse(splitStr[7]) });
                }
            }
            return rules;
        },

        setCFRule: function (rule) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowConditionalFormats)
                return;
            this._cFormat(rule.action, rule.input1, rule.input2, rule.color, rule.range, xlObj.getActiveSheetIndex());
        },

        _updateCFormatRule: function (rule, isApply) {
            rule = rule.indexOf("true") > -1 ? rule.replace("true", isApply) : rule.replace("false", isApply);
            return rule;
        },

        _getRuleIndex: function (rule, ruleCol) {
            if (ej.isNullOrUndefined(ruleCol))
                return -1;
            for (var i = 0, len = ruleCol.length; i < len; i++) {
                if (ruleCol[i] === rule)
                    return i;
            }
            return -1;
        },

        _cFormat: function (action, input1, input2, cFColor, addr, sheetIdx, range, status) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowConditionalFormats)
                return;
            xlObj.element.ejWaitingPopup("show");
            sheetIdx = sheetIdx ? sheetIdx : xlObj.getActiveSheetIndex();
            var sheet = xlObj.getSheet(sheetIdx), rangeData = sheet.selectedRange, details = { sheetIndex: sheetIdx, reqType: "cond-format", action: "add", range: rangeData };
            switch (action) {
                case "greaterthan":
                    this._applyGLEValues(">", input1, cFColor, action, addr, sheetIdx, range);
                    break;
                case "greaterequalto":
                    this._applyGLEValues(">=", input1, cFColor, action, addr, sheetIdx, range);
                    break;
                case "lessthan":
                    this._applyGLEValues("<", input1, cFColor, action, addr, sheetIdx, range);
                    break;
                case "lessequalto":
                    this._applyGLEValues("<=", input1, cFColor, action, addr, sheetIdx, range);
                    break;
                case "equalto":
                    this._applyGLEValues("==", input1, cFColor, action, addr, sheetIdx, range);
                    break;
                case "notequalto":
                    this._applyGLEValues("!=", input1, cFColor, action, addr, sheetIdx, range);
                    break;
                case "between":
                    this._applyBetweenValues("=", input1, input2, cFColor, action, addr, sheetIdx, range);
                    break;
                case "notbetween":
                    this._applyBetweenValues("!", input1, input2, cFColor, action, addr, sheetIdx, range);
                    break;
                case "textcontains":
                    this._applyContainsValues(input1, cFColor, action, addr, sheetIdx, range);
                    break;
                case "dateoccur":
                    this._applyDateContainsValues(input1, cFColor, action, addr, sheetIdx, range);
                    break;
            }
            if (!sheet._importRendered && !xlObj._isUndoRedo && status !== "paste" && !xlObj._dupDetails)
                xlObj._completeAction(details);
            xlObj.element.ejWaitingPopup("hide");
        },

        clearCF: function (isSelected, range) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), details = { sheetIndex: sheetIdx, reqType: "cond-format", action: "clear", selection: isSelected, rule: [] };
            if (!xlObj.model.allowConditionalFormats)
                return;
            var i, len, cFormatData = xlObj._dataContainer.cFormatData, selected;
            if (!xlObj.isUndefined(range))
                selected = xlObj._getSelectedItems(sheetIdx, range)[0];
            else
                selected = (isSelected) ? xlObj.element.find(".e-selected") : xlObj.element.find(".e-rowcell");
            selected.removeClass("e-redft e-yellowft e-greenft e-redf e-redt");
            for (i = 0, len = selected.length; i < len; i++) {
                if (!ej.isNullOrUndefined(xlObj._dataContainer.sheets[sheetIdx][$(selected[i]).parent().index()]))
                    if (!ej.isNullOrUndefined(xlObj._dataContainer.sheets[sheetIdx][$(selected[i]).parent().index()][selected[i].cellIndex]))
                        if (!ej.isNullOrUndefined(xlObj._dataContainer.sheets[sheetIdx][$(selected[i]).parent().index()][selected[i].cellIndex].cFormatRule)) {
                            details.rule.push(this.getCFRule($(selected[i]).parent().index(), selected[i].cellIndex));
                            xlObj.clearRangeData(null, ["cFormatRule"], $(selected[i]));
                        }
            }
            var rule = (isSelected) ? "clear_" + sheetIdx + "_" + xlObj._getSelectedItems()[1] : "clear_" + sheetIdx + "_" + "entire";
            if (cFormatData.indexOf(rule) === -1)
                cFormatData.push(rule);
            if (!xlObj._isUndoRedo && !xlObj._dupDetails)
                xlObj._completeAction(details);
        },

        _getValueForCFormat: function (cell, sheetIdx) {
            var xlObj = this.XLObj, rowIdx = cell.rowIndex, colIdx = cell.colIndex, cellObj = xlObj.getRangeData([rowIdx, colIdx, rowIdx, colIdx], false, null, sheetIdx);
            var value = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx), value2 = xlObj.XLEdit.getPropertyValueByElem(rowIdx, colIdx, "value2");
            value = value ? value.toString() : "";
            if (value2 && (typeof value2 == "string" && !cellObj.formatStr && value[0] === "=") || (xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "type") === "datetime"))
                value = value2;
            return value;
        },

        _applyGLEValues: function (op, input, cFColor, action, addr, sheetIdx, range) {
            var xlObj = this.XLObj, value, selected, rowIdx, colIdx, temp, isApply, prevStatus, tRule, ruleIdx, type, numRegx = new RegExp(/[^.0-9]+/g), txtRegx = new RegExp(/[^.-a-zA-Z 0-9]+/g);
            if(ej.isNullOrUndefined(range))
                selected = xlObj.getSheet(sheetIdx)._selectedCells;
            else{
                var rng = xlObj.getRangeIndices(range)
                selected = xlObj.XLSelection._getSelectedRange({rowIndex: rng[0], colIndex: rng[1]}, {rowIndex: rng[2], colIndex: rng[3]});
            }
            for (var i = 0, len = selected.length; i < len; i++) {
                rowIdx = selected[i].rowIndex; colIdx = selected[i].colIndex;
                if (xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isMHide") || xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isFilterHide"))
                    continue;
                value = this._getValueForCFormat({rowIndex: rowIdx, colIndex: colIdx}, sheetIdx);
                type = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "type") ? xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "type") : "general";
                prevStatus = true;
                ruleIdx = -1;
                if (ej.isNullOrUndefined(value) || !value.length)
                    isApply = (action === "lessthan" || action === "lessequalto");
                else if ($.isNumeric(input)) {
                    if ($.isNumeric(value)) {
                        value = parseFloat(value);
                        isApply = xlObj.operators[op](value, parseFloat(input.replace(txtRegx, "")));
                        if (type === "percentage")
                            isApply = xlObj.operators[op](value, 0);
                    }
                    else if (xlObj._isValidDate(value)) {
                        isApply = xlObj.operators[op](xlObj._dateToInt(value), parseFloat(input));
                    }
                    else if (xlObj._isValidTime(value))
                        continue;
                    else
                        isApply = xlObj.operators[op](value.toLowerCase(), input.toLowerCase());
                }
                else if (input.lastIndexOf("%") === input.length - 1 && $.isNumeric(input.replace(numRegx, ""))) {
                    if (type === "percentage")
                        isApply = xlObj.operators[op](parseFloat(value), parseFloat(input.replace(numRegx, "")));
                    else
                        isApply = $.isNumeric(value) ? xlObj.operators[op](parseFloat(value), 0) : xlObj.operators[op](value.length, 0);
                }
                else if (input.indexOf("$") === 0 && $.isNumeric(input.replace(numRegx, ""))) {
                    if (type === "accounting" || type !== "currency" || $.isNumeric(value))
                        isApply = xlObj.operators[op](parseFloat(value), parseFloat(input.replace(numRegx, "")));
                    else
                        isApply = ((xlObj.operators[op](value.length, 0) || xlObj._isValidDate(value)) && !xlObj._isValidTime(value));
                }
                else if (xlObj._isValidDate(input)) {
                    if (xlObj._isValidDate(value)) {
                        isApply = xlObj.operators[op](xlObj._dateToInt(new Date(value)), xlObj._dateToInt(new Date(input)));
                    }
                    else if (!xlObj._isValidTime(value)) {
                        if ($.isNumeric(value))
                            isApply = xlObj.operators[op](parseFloat(value), xlObj._dateToInt(input));
                        else
                            isApply = xlObj.operators[op](value.length, xlObj._dateToInt(input));
                    }
                }
                else if (xlObj._isValidTime(input)) {
                    if (xlObj._isValidTime(value))
                        isApply = xlObj.operators[op](xlObj.parse.parseTime(value), xlObj.parse.parseTime(input));
                    else
                        isApply = $.isNumeric(value) ? xlObj.operators[op](parseFloat(value), 0) : xlObj.operators[op](value.length, 0);
                }
                else
                    isApply = xlObj.operators[op](value.toLowerCase(), input.toLowerCase());
                tRule = action + "_" + op + "_" + input + "__" + cFColor + "_" + addr + "_" + xlObj.getActiveSheetIndex();
                this._applyCFormatRule({rowIndex: rowIdx, colIndex: colIdx}, tRule, ruleIdx, isApply, prevStatus, cFColor);
            }
        },

        _applyBetweenValues: function (op, input1, input2, cFColor, action, addr, sheetIdx, range) {
            var xlObj = this.XLObj, value, $trgt, selected, temp, isApply, prevStatus, tRule, ruleIdx, type, numRegx = new RegExp(/[^.0-9]+/g), txtRegx = new RegExp(/[^.-a-zA-Z 0-9]+/g), convDate;            
            if (!xlObj.operators["<"](input1, input2)) {
                temp = input2;
                input2 = input1;
                input1 = temp;
            }
            if(ej.isNullOrUndefined(range))
                selected = xlObj.getSheet(sheetIdx)._selectedCells;
            else{
                var rng = xlObj.getRangeIndices(range)
                selected = xlObj.XLSelection._getSelectedRange({rowIndex: rng[0], colIndex: rng[1]}, {rowIndex: rng[2], colIndex: rng[3]});
            }
            for (var i = 0, len = selected.length; i < len; i++) {
                rowIdx = selected[i].rowIndex; colIdx = selected[i].colIndex;
                if (xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isMHide") || xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isFilterHide"))
                    continue;
                value = this._getValueForCFormat({rowIndex: rowIdx, colIndex: colIdx}, sheetIdx);
                type = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "type") ? xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "type") : "general";
                prevStatus = true;
                ruleIdx = -1;
                if (ej.isNullOrUndefined(value) || !value.length)
                    isApply = false;
                else if ($.isNumeric(input1.replace(txtRegx, "")) && $.isNumeric(input2.replace(txtRegx, "")) && type !== "percentage" && type !== "currency" && type !== "accounting") {
                    input1 = input1.replace(txtRegx, "");
                    input2 = input2.replace(txtRegx, "");
                    if ($.isNumeric(value))
                        isApply = xlObj.operators[op](parseFloat(value) >= parseFloat(input1) && parseFloat(value) <= parseFloat(input2));
                    else if (xlObj._isValidDate(value)) {
                        isApply = xlObj.operators[op](xlObj._dateToInt(value) >= parseFloat(input1) && convDate <= parseFloat(input2));
                    }
                    else if (xlObj._isValidTime(value))
                        continue;
                    else
                        isApply = xlObj.operators[op](value.toLowerCase() >= input1.toLowerCase() && value.toLowerCase() <= input2.toLowerCase());
                }
                else if (input1.lastIndexOf("%") === input1.length - 1 && $.isNumeric(input1.replace(numRegx, "")) && input2.lastIndexOf("%") === input2.length - 1 && $.isNumeric(input2.replace(numRegx, "")) && type === "percentage")
                    isApply = xlObj.operators[op](parseFloat(value) >= parseFloat(input1.replace(numRegx, "")) && parseFloat(value) <= parseFloat(input2.replace(numRegx, "")));
                else if (input1.indexOf("$") === 0 && $.isNumeric(input1.replace(numRegx, "")) && input2.indexOf("$") === 0 && $.isNumeric(input2.replace(numRegx, "")) && (type === "accounting" || type !== "currency"))
                    isApply = xlObj.operators[op](parseFloat(value) >= parseFloat(input1.replace(numRegx, "")) && parseFloat(value) <= parseFloat(input2.replace(numRegx, "")));
                else if (xlObj._isValidDate(input1) && xlObj._isValidDate(input2)) {
                    var date1 = xlObj._dateToInt(new Date(value)), date2 = xlObj._dateToInt(new Date(input1)), date3 = xlObj._dateToInt(new Date(input2));
                    isApply = xlObj.operators[op](date1 >= date2 && date1 <= date3);
                }
                else if (xlObj._isValidTime(input1) && xlObj._isValidTime(input2) && xlObj._isValidTime(value))
                    isApply = xlObj.operators[op](xlObj.parse.parseTime(value) >= xlObj.parse.parseTime(input1) && xlObj.parse.parseTime(value) <= xlObj.parse.parseTime(input2));
                else
                    isApply = xlObj.operators[op](value.toLowerCase() >= input1.toLowerCase() && value.toLowerCase() <= input2.toLowerCase());
                tRule = action + "_" + op + "_" + input1 + "_" + input2 + "_" + cFColor + "_" + addr + "_" + xlObj.getActiveSheetIndex();
                this._applyCFormatRule({ rowIndex: rowIdx, colIndex: colIdx }, tRule, ruleIdx, isApply, prevStatus, cFColor);
            }
        },

        _applyContainsValues: function (input, cFColor, action, addr, sheetIdx, range) {
            var xlObj = this.XLObj, value, $trgt, selected, temp, isApply, prevStatus, tRule, ruleIdx, txtRegx = new RegExp(/[^.-a-zA-Z 0-9]+/g);
            if (ej.isNullOrUndefined(range))
                selected = xlObj.getSheet(sheetIdx)._selectedCells;
            else {
                var rng = xlObj.getRangeIndices(range)
                selected = xlObj.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
            }
            for (var i = 0, len = selected.length; i < len; i++) {
                rowIdx = selected[i].rowIndex; colIdx = selected[i].colIndex;
                if (xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isMHide") || xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isFilterHide"))
                    continue;
                value = this._getValueForCFormat({ rowIndex: rowIdx, colIndex: colIdx }, sheetIdx);
                type = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "type") ? xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "type") : "general";
                prevStatus = true;
                ruleIdx = -1;
                if (ej.isNullOrUndefined(value) || !value.length)
                    isApply = false;
                else if ($.isNumeric(input.replace(txtRegx, ""))) {
                    input = input.replace(txtRegx, "");
                    if (xlObj._isValidDate(value))
                        value = ((xlObj._dateToInt(value))).toString();
                    isApply = value.indexOf(input) > -1;
                }
                else if (xlObj._isValidDate(input)) {
                    if (xlObj._isValidDate(value))
                        value = xlObj._dateToInt(value).toString();
                    isApply = value.indexOf(xlObj._dateToInt(input)) > -1;
                }
                else if (xlObj._isValidTime(input)) {
                    if (xlObj._isValidTime(value))
                        value = xlObj._dateToInt("01/01/1990 " + value).toString();
                    isApply = value.indexOf(xlObj._dateToInt("01/01/1990 " + input)) > -1;
                }
                else
                    isApply = value.toLowerCase().indexOf(input.toLowerCase()) > -1;
                tRule = action + "__" + input + "__" + cFColor + "_" + addr + "_" + xlObj.getActiveSheetIndex();
                this._applyCFormatRule({rowIndex: rowIdx, colIndex: colIdx}, tRule, ruleIdx, isApply, prevStatus, cFColor);
            }
        },

        _applyDateContainsValues: function (input, cFColor, action, addr, sheetIdx, range) {
            var xlObj = this.XLObj, value, $trgt, isApply = false, tRule, ruleIdx;
            if (ej.isNullOrUndefined(range))
                selected = xlObj.getSheet(sheetIdx)._selectedCells;
            else {
                var rng = xlObj.getRangeIndices(range)
                selected = xlObj.XLSelection._getSelectedRange({ rowIndex: rng[0], colIndex: rng[1] }, { rowIndex: rng[2], colIndex: rng[3] });
            }
            for (var i = 0, len = selected.length; i < len; i++) {
                rowIdx = selected[i].rowIndex; colIdx = selected[i].colIndex;
                if (xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isMHide") || xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "isFilterHide"))
                    continue;
                value = this._getValueForCFormat({ rowIndex: rowIdx, colIndex: colIdx }, sheetIdx);
                ruleIdx = -1;
                if (ej.isNullOrUndefined(value) || !value.length)
                    isApply = false;
                else if (xlObj._isValidDate(input)) {
                    var date1 = new Date(value), date2 = new Date(input);
                    isApply = (date1 - date2) === 0;
                }
                tRule = action + "__" + input + "__" + cFColor + "_" + addr + "_" + xlObj.getActiveSheetIndex();
                this._applyCFormatRule({rowIndex: rowIdx, colIndex: colIdx}, tRule, ruleIdx, isApply, true, cFColor);
            }
        },

        refreshCFormat: function (range) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowConditionalFormats)
                return;
            var i, j, rules = [], sheetIdx = xlObj.getActiveSheetIndex(), cFormatStr, selected, cFormatRule, len, rLen, rowIdx, colIdx;
            if(ej.isNullOrUndefined(range))
                selected = xlObj.getSheet(sheetIdx)._selectedCells;
            else {
                if (typeof (range) === "string")
                    range = xlObj.getRangeIndices(range)
                selected = xlObj.XLSelection._getSelectedRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] });
            }
            for (i = 0, len = selected.length; i < len; i++) { //getting rules & clear rules
                rowIdx = selected[i].rowIndex; colIdx = selected[i].colIndex;
                cFormatRule = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "cFormatRule");
                if (!xlObj.isUndefined(cFormatRule))
                    for (j = 0, rLen = cFormatRule.length; j < rLen; j++) {
                        cFormatRule[j] = cFormatRule[j].replace("_true", "").replace("_false", "");
                        if (rules.indexOf(cFormatRule[j]) < 0)
                            rules.push(cFormatRule[j]);
                    }
                if(xlObj._isRowViewable(sheetIdx, rowIdx))
                    xlObj.getCell(rowIdx, colIdx, sheetIdx).removeClass("e-redft e-yellowft e-greenft e-redf e-redt");
                xlObj.clearRangeData(null, ["cFormatRule"], selected[i]);
            }
            //apply rules
            for (j = 0, rLen = rules.length; j < rLen; j++) {
                cFormatStr = rules[j].split("_");
                this._cFormat(cFormatStr[0], cFormatStr[2], cFormatStr[3], cFormatStr[4], cFormatStr[5]);
            }
        },

        _applyCFormatRule: function (cell, tRule, ruleIdx, isApply, prevStatus, cFColor) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), rowIdx = cell.rowIndex, colIdx = cell.colIndex, isViewed, actCell, cFormatRule = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "cFormatRule"), rule, idx;
            if (this._getRuleIndex(tRule + "_true", cFormatRule) > -1)
                ruleIdx = this._getRuleIndex(tRule + "_true", cFormatRule); //check already rule exists
            if (this._getRuleIndex(tRule + "_false", cFormatRule) > -1)
                ruleIdx = this._getRuleIndex(tRule + "_false", cFormatRule);
            if (ruleIdx > -1) { //change rule index in cellobj and container for rule true, false changes
                rule = cFormatRule[ruleIdx];
                idx = xlObj._dataContainer.cFormatData.indexOf(this._updateCFormatRule(rule, isApply));
                if (idx > -1)
                    xlObj._dataContainer.sheets[sheetIdx][rowIdx][colIdx].cFormatRule[ruleIdx] = idx;
                else {
                    xlObj._dataContainer.cFormatData.push(this._updateCFormatRule(rule, isApply));
                    xlObj._dataContainer.sheets[sheetIdx][rowIdx][colIdx].cFormatRule[ruleIdx] = xlObj._dataContainer.cFormatData.length - 1;
                }
            }
            cFormatRule = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "cFormatRule");
            if (xlObj._isRowViewable(sheetIdx, rowIdx)) {
                isViewed = true;
                actCell = xlObj.getCell(rowIdx, colIdx);
            }
            if (!xlObj.isUndefined(cFormatRule) && (cFormatRule.length > 0 && (!(tRule + "_true" === cFormatRule[ruleIdx] || tRule + "_false" === cFormatRule[ruleIdx]) || (cFormatRule.length > 1)))) { //check and change prev applied status for multirule
                for (var j = 0, len = cFormatRule.length; j < len; j++) {
                    rule = cFormatRule[j].split("_");
                    if (rule[7] === "true" && rule[4] === cFColor)
                        prevStatus = false;
                }
                if (isViewed) {
                    if (isApply)
                        actCell.removeClass("e-redft e-yellowft e-greenft e-redf e-redt").addClass("e-" + cFColor);
                    else if (prevStatus)
                        actCell.removeClass("e-" + cFColor);
                }
            } else {
                if (isViewed) {
                    if (isApply)
                        actCell.removeClass("e-redft e-yellowft e-greenft e-redf e-redt").addClass("e-" + cFColor);
                    else
                        actCell.removeClass("e-" + cFColor);
                }
            }
            if ((this._getRuleIndex(tRule + "_true", cFormatRule) < 0 && this._getRuleIndex(tRule + "_false", cFormatRule) < 0))  //apply rule for first time while applying        
                xlObj.XLEdit._updateDataContainer(cell, { "cFormatRule": [tRule + "_" + isApply] });
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.clipboard = function (obj) {
        this.XLObj = obj;
        this._cutCells = [];
        this._copyCells = [];
        this._copyBackup = {};
        this._cData = "";
        this._isCut = false;
        this._copyRange = [];
        this._isSpecial = true;
        this._isShape = false;
    };

    ej.spreadsheetFeatures.clipboard.prototype = {
        //Cut, copy and paste
        copy: function () {
            this._triggerKeyDown(67, true);
        },

        cut: function () {
            this._triggerKeyDown(88, true);
        },

        paste: function () {
            this._triggerKeyDown(86, true);
        },

        _initCopyPaste: function () {
            var xlId = this.XLObj._id, $div = ej.buildTag("div", {}, { position: "fixed", top: -1000, left: -1000 }, { id: xlId + "_copyPaste" });
            $("body").append($div);
            $div.append(ej.buildTag("textarea " + "." + xlId + "_copyTxt", {}, { width: 1, height: 1, overflow: "hidden", opacity: 0, resize: "none", cursor: "default" }, { id: xlId + "_hiddenTxt" }));
        },

        _copyPasteHandler: function (e) {
            var selCells,xlObj = this.XLObj,actSheet= xlObj.getSheet(xlObj.getActiveSheetIndex());
            xlObj._isUndoRedo = false;
            if ((e.keyCode === 67 || e.keyCode === 88 || e.keyCode === 86) && xlObj._preventctrlkey) {
                xlObj._showAlertDlg("Alert", "CtrlKeyErrorAlert", "CtrlKeyErrorAction", 450);
                return;
            }

            if (e.ctrlKey) {
                if (e.keyCode === 67) //copy
                {
                    this._getClipboard(e);
                    this._cutCells = [];
                    $.extend(true, this._copyCells,actSheet._selectedCells);
                    if (!this._copyCells.length)
                        this._copyCells = [xlObj.XLShape._picCellIdx];
                    setTimeout(function () {
                        xlObj.setSheetFocus();
                    }, 0);
                }
                else if (e.keyCode === 88) //cut
                {
                  
                    selCells= actSheet._selectedCells;
                    if (xlObj.model.allowLockCell && actSheet._isSheetProtected) {
                        if (xlObj._israngeProtected(selCells)) {
                            xlObj._popUplockCellAlert();
                            return;
                        }
                    }
                    $.extend(true, this._cutCells, selcells);
                    if (!this._cutCells.length)
                        this._cutCells = [xlObj.XLShape._picCellIdx];
                    this._getClipboard(e);
                    setTimeout(function () {
                        xlObj.setSheetFocus();
                    }, 0);
                }
                else if (e.keyCode === 86) //paste
                {
                    if (!xlObj.isUndefined(e.bubbles))
                        this._isSpecial = true;
                    $("#" + xlObj._id + "_hiddenTxt").select();
                    setTimeout(function () {
                        xlObj.XLClipboard._setClipboard();
                    }, 0);
                }
            }
            else if (e.keyCode === 27 && xlObj.element.find(".e-cdata").length)//escape          
                xlObj.element.find("td.e-cutright, td.e-cutbottom").removeClass("e-cutright e-cutbottom");
        },

        _triggerKeyDown: function (keyCode, ctrlKey) {
            var xlObj = this.XLObj;
            xlObj.setSheetFocus();
            var e = $.Event("keydown");
            e.keyCode = keyCode;
            e.ctrlKey = ctrlKey;
            $("#" + xlObj._id).trigger(e);
        },

        _getClipboard: function (e) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), actSheet = xlObj.getSheet(sheetIdx), startcell = actSheet._startCell, endcell = actSheet._endCell;
            var selected = xlObj._getContent(sheetIdx).find(".e-selected"), actCell = xlObj.getActiveCellElem(sheetIdx)[0];
            if (!xlObj.XLClipboard._copyBackup.cells) {
                xlObj.XLRibbon._enableButtons(["pastebtn"], "ejButton");
                xlObj.XLRibbon._enableButtons(["pastesbtn"], "ejSplitButton");
            }
            if (selected.length) {
                this._copyBackup = { cells: [], border: xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(), table: [], filterRange: [] };
                var textContent = this._getSelectedData(startcell, endcell);
                if (xlObj.element.find(".e-cdata").length)
                    xlObj.element.find("td.e-cutright, td.e-cutbottom").removeClass("e-cutright e-cutbottom");
                $(selected).addClass("e-cdata");
                xlObj.XLSelection._focusBorder(startcell, endcell, xlObj._cutFocus);
                if (selected.length < 2 && !textContent.length)
                    this._cData = textContent = " ";
                else
                    this._cData = textContent;
                this._isCut = false;
                $("#" + xlObj._id + "_hiddenTxt").val(textContent).select();
                this._copyBackup.actCellPos = { "top": actCell.offsetTop, "left": actCell.offsetLeft };
                this._isShape = false;
            }
            else {
                this._isShape = true;
                this._copyBackup = { elem: xlObj.getSheetElement(sheetIdx).find(".e-ss-activeimg"), isCut: e.keyCode === 88, sIdx: sheetIdx };
            }
        },

        _getSelectedData: function (startCell, endCell) {
            var xlObj = this.XLObj, textContent = "", cell, i, j, value, sheetIdx = xlObj.getActiveSheetIndex();
            if (startCell.rowIndex >= endCell.rowIndex) {
                var t = startCell;
                startCell = endCell;
                endCell = t;
            }
            this._copyRange = [];
            this._copyRange.push(xlObj.getActiveSheetIndex());
            this._copyRange.push(xlObj._getSelectedItems()[1]);
            for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                if (i !== startCell.rowIndex)
                    textContent += "\n";
                if (startCell.colIndex <= endCell.colIndex) {
                    for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                        cData = xlObj.getRangeData([i, j, i, j], false, ["type", "value", "value2"])[0];
                        if (cData.length)
                            value = cData.type !== "general" ? cData.value2 : cData.value;
                        else
                            value = "";
                        this._updateCBackupData(i, j);
                        //if (cell) {
                            if (j !== endCell.colIndex)
                                textContent += value + "\t";
                            else {
                                textContent += value;
                            }
                        }
                    }
                else {
                    for (j = endCell.colIndex; j <= startCell.colIndex; j++) {
                        cData = xlObj.getRangeData([i, j, i, j], false, ["type", "value", "value2"])[0];
                        if (cData.length)
                            value = cData.type !== "general" ? cData.value2 : cData.value;
                        else
                            value = "";
                        this._updateCBackupData(i, j);
                        //if (cell) {
                            if (j !== startCell.colIndex)
                                textContent += value + "\t";
                            else {
                                textContent += value;
                            }
                        //}
                        }
                    }
                }
            //get table details
            var tblMgr = xlObj.model.sheets[sheetIdx]._tableManager, tblObj, tblRange, tblName = [], selectedRange = xlObj.model.sheets[sheetIdx].selectedRange;
            for (var m = 0, rLen = xlObj.getObjectLength(tblMgr) ; m < rLen; m++) {
                tblObj = $.extend(true, {}, tblMgr[m]);
                tblRange = tblObj.range;
                if (this._compareRange(selectedRange, tblObj.totalRow ? [tblRange[0], tblRange[1], tblRange[2] + 1, tblRange[3]] : tblRange)) {
                    tblName.push(tblObj.name);
                    tblObj.header = true;
                    tblObj.name = "Table" + (parseInt($.trim(tblMgr[rLen - 1].name.replace("Table", ""))) + m);
                    this._copyBackup.table.push({ tblName: tblName, tblObj: tblObj, range: tblRange });
                }
            }
            //get filtering details
            var fltrRange = xlObj.getRangeIndices(xlObj.model.sheets[sheetIdx].filterSettings.range);
            if (this._compareRange(selectedRange, fltrRange))
                this._copyBackup.filterRange = fltrRange;
            return textContent;
        },

        _updateCBackupData: function (rowIdx, colIdx) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowClipboard)
                return;
            var sheetIdx = xlObj.getActiveSheetIndex(), format, container = xlObj._dataContainer.sheets[sheetIdx], cellObj, regx = new RegExp("\\b" + "e-format" + ".*?\\b", "g");
            if (rowIdx in container && colIdx in container[rowIdx]) {
                cellObj = $.extend(true, {}, container[rowIdx][colIdx]);
                format = cellObj.format ? cellObj.format : "";
            }
            else {
                cellObj = {};
                format = "";
            }
            this._copyBackup.cells.push({ cell: cellObj, format: format });
        },

        _setClipboard: function () {
            var cells, overflow, xlObj = this.XLObj, cellObj,prtctRange,row = [], cell = [], type, regx = new RegExp("\\b" + "e-format" + ".*?\\b", "g"), currentCell, isColor = false, currCell, sheetIdx = xlObj.getActiveSheetIndex(), details = { cSheetIndex: this._copyRange[0], pSheetIndex: sheetIdx, sheetIndex: sheetIdx }, pastedTable = [];
            var startCell = xlObj.getSheet(sheetIdx)._startCell, endCell = xlObj.getSheet(sheetIdx)._endCell, i, j, k = 0, m, rLen, addr, cFormatRule, splitStr, shapeObj = {}, shapeElem, fltr, currCellObj, shapeRegx = new RegExp("\\b" + "e-shapebdr" + ".*?\\b", "g"), bstyle = ["solid", "dashed", "dotted"];
            var minRowIdx = (startCell.rowIndex < endCell.rowIndex) ? startCell.rowIndex : endCell.rowIndex, tblName, tableObj = this._copyBackup.table, style, bcolor;
            var minColIdx = (startCell.colIndex < endCell.colIndex) ? startCell.colIndex : endCell.colIndex, container = xlObj._dataContainer, range, cellPos;

            if (xlObj.model.allowLockCell && xlObj.getSheet(sheetIdx)._isSheetProtected) {
                prtctRange=xlObj.swapRange(xlObj.getRangeIndices(this._copyRange[1]));
                cells = xlObj.XLSelection._getSelectedRange({ rowIndex: minRowIdx, colIndex: minColIdx }, { rowIndex: minRowIdx + (prtctRange[2] - prtctRange[0]), colIndex: minColIdx + (prtctRange[3] - prtctRange[1]) });
                    if (xlObj._israngeProtected(cells)) {
                        xlObj._popUplockCellAlert();
                        return;
                    }
                }

            if ((minRowIdx === 0 && minColIdx === 0 && this._copyCells.length === (this.XLObj.model.rowCount * this.XLObj.model.columnCount)) || this._copyCells.length !== (this.XLObj.model.rowCount * this.XLObj.model.columnCount)) {
                var actSheet = xlObj.model.sheets[sheetIdx], actCell = xlObj.getSheetElement(sheetIdx).find(".e-activecell"), shapeMgr = xlObj.getSheet(this._copyBackup.sIdx || this._copyRange[0] || sheetIdx)._shapeMngr, pRange, className, len, cLen, isCut, cutCells, cSheetIdx;
                if (this._isShape) {
                    var shapeIdx, id, cellIdx = xlObj.XLShape._picCellIdx, actCellIdx = xlObj.getActiveCell(sheetIdx);
                    shapeElem = this._copyBackup.elem;
                    id = shapeElem[0].id;
                    type = id.indexOf("picture") > -1 ? "picture" : "chart";
                    shapeMgr[type].some(function (obj, i) {
                        if (id === obj.id) {
                            shapeIdx = i;
                            return true;
                        }
                        return false;
                    });
                    details = { sheetIndex: sheetIdx, cSheetIndex: this._copyBackup.sIdx, pSheetIndex: sheetIdx, operation: "shape", cShapeId: id, reqType: (this._copyBackup.isCut) ? "cut-paste" : "copy-paste" };
                    cellObj = $.extend(true, {}, container.sheets[this._copyBackup.sIdx][cellIdx.rowIndex][cellIdx.colIndex]);
                    details.cutCell = { rowIndex: cellIdx.rowIndex, colIndex: cellIdx.colIndex, obj: cellObj };
                    cell = xlObj.getCell(actCellIdx.rowIndex, actCellIdx.colIndex);
                    $.extend(true, shapeObj, shapeMgr[type][shapeIdx]);
                    shapeElem = xlObj.element.find("#" + shapeObj.id);
                    if (type === "picture") {
                        className = shapeElem[0].className;
                        if (className.indexOf("e-shapebdr") > -1) {
                            className = shapeElem[0].className.match(shapeRegx)[0];
                            style = className.replace("e-shapebdr", "").split("N");
                            bcolor = style[2];
                            xlObj.XLShape._createPicture(sheetIdx, cell[0], shapeMgr.sharedPics[shapeObj.data], cell[0].offsetTop, cell[0].offsetLeft, shapeObj.width, shapeObj.height, null, "#" + bcolor, bstyle[parseInt(style[1]) - 1], style[0] + "px");
                        }
                        else
                            xlObj.XLShape._createPicture(sheetIdx, cell[0], shapeMgr.sharedPics[shapeObj.data], cell[0].offsetTop, cell[0].offsetLeft, shapeObj.width, shapeObj.height);
                    }
                    else {
                        range = shapeObj.range;
                        range = xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]);
                        if (range.indexOf(":") === -1)
                            range = range + ":" + range;
                        shapeObj.top = cell[0].offsetTop;
                        shapeObj.left = cell[0].offsetLeft;
                        xlObj.XLChart.createChart(range, $.extend(true, {}, shapeObj));
                    }
                    //container.sheets[sheetIdx][actCellIdx.rowIndex][actCellIdx.colIndex][type] = [shapeMgr[type].length - 1];
                    if (this._copyBackup.isCut)
                        shapeElem.remove();
                    this._isShape = this._copyBackup.isCut ? false : true;
                    this._cutCells = [];
                    if (!xlObj._isUndoRedo) {
                        var repStr = xlObj._id + "_" + type;
                        id = this.XLObj.XLChart._shapeCnt - 1;
                        details.pShapeId = repStr + id;
                        details.pasteCell = { rowIndex: actCellIdx.rowIndex, colIndex: actCellIdx.colIndex, obj: $.extend(true, {}, container.sheets[sheetIdx][actCellIdx.rowIndex][actCellIdx.colIndex]) };
                        xlObj._completeAction(details);
                    }
                }
                else {
                    details.reqType = (this._cutCells.length > 0) ? "cut-paste" : "copy-paste";
                    if (xlObj.element.find(".e-selected").hasClass("e-readonly")) {
                        if (xlObj.element.find(".e-cdata").length)
                            xlObj.element.find(".e-cutright, .e-cutbottom").removeClass("e-cutright e-cutbottom");
                        return;
                    }
                    details.text = row = $("#" + xlObj._id + "_hiddenTxt").val().split("\n");
                    isColor = this._cData === $("#" + xlObj._id + "_hiddenTxt").val();
                    if (this._cData.length < 1 && details.text.length < 1)
                        return;
                    if (!isColor && row[row.length - 1] === "")
                        row.splice(row.length - 1, 1);
                    var pasteCells = [], hldata, value2, rowIdx, colIdx, cRowIdx, cColIdx, rangeCells;
                    if (isColor) {
                        details.isSpecial = this._isSpecial;
                        range = xlObj.getRangeIndices(this._copyRange[1]);
                        xlObj.setActiveSheetIndex(this._copyRange[0]);
                        details.cutBorder = { range: range, prevBorder: xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(range) };
                        xlObj.setActiveSheetIndex(sheetIdx);
                    }
                    else
                        details.isSpecial = false;
                    if (this._cutCells.length) { //cut special
                        cutCells = [];
                        cSheetIdx = this._copyRange[0];
                        for (i = 0, len = this._cutCells.length; i < len; i++) {
                            rowIdx = this._cutCells[i].rowIndex;
                            colIdx = this._cutCells[i].colIndex;
                            currentCell = xlObj.getCell(rowIdx, colIdx, cSheetIdx);   //this._copyRange[0] is cutcell sheetIndex
                            cell = { rowIndex: rowIdx, colIndex: colIdx, prevObj: {}, newObj: {} };
                            if (!ej.isNullOrUndefined(container.sheets[cSheetIdx][rowIdx]))
                                $.extend(true, cell.prevObj, container.sheets[cSheetIdx][rowIdx][colIdx]);
                            else
                                cell.prevObj = {};
                            if (currentCell[0]) {
                                if (!("merge" in cell.prevObj))
                                    xlObj.XLEdit.updateCellValue({rowIndex: rowIdx, colIndex: colIdx}, "");
                                cellObj = ej.isNullOrUndefined(container.sheets[cSheetIdx][rowIdx]) ? {} : $.extend(true, {}, container.sheets[cSheetIdx][rowIdx][colIdx]);
                                if ("wrap" in cellObj && cellObj.wrap) {
                                    addr = xlObj._generateHeaderText(cSheetIdx, colIdx + 1) + (rowIdx + 1);
                                    xlObj.setWrapText(addr + ":" + addr, "wrap");
                                }
                                currentCell.removeClass("e-commentcell e-hyperlinks e-redft e-yellowft e-greenft e-redf e-redt e-hlcell");
                                if ((rowIdx in container.sheets[cSheetIdx]) && (colIdx in container.sheets[cSheetIdx][rowIdx])){
									!ej.isNullOrUndefined(container.sheets[cSheetIdx][rowIdx][colIdx]) && (overflow = container.sheets[cSheetIdx][rowIdx][colIdx]['overflow']);
                                    delete container.sheets[cSheetIdx][rowIdx][colIdx];
									if(overflow){
										container.sheets[cSheetIdx][rowIdx][colIdx] = {};
										container.sheets[cSheetIdx][rowIdx][colIdx]['overflow'] = overflow;
									} 
								}
                                if (currentCell[0].className.match(regx))
                                    currentCell[0].className = currentCell[0].className.replace(regx, "");
                                if (!ej.isNullOrUndefined(container.sheets[cSheetIdx][rowIdx]))
                                    $.extend(true, cell.newObj, container.sheets[cSheetIdx][rowIdx][colIdx]);
                                else
                                    cell.newObj = {};
                                xlObj._setRowHdrHeight(cSheetIdx, $(currentCell[0]).parent().index());
                            }
                            cutCells.push(cell);
                        }
                        for (m = 0, rLen = tableObj.length; m < rLen; m++) {
                            tblName = $.trim(tableObj[m].tblName);
                            xlObj.XLFormat.removeTable($.trim(tableObj[m].tblName[0].replace("Table", "")));
                        }
                        if (xlObj.model.allowCellFormatting)
                            xlObj.XLFormat.clearFormat(range, "format");
                        xlObj.setActiveSheetIndex(cSheetIdx);
                        //xlObj.model.allowCellFormatting && xlObj.XLFormat.clearFormat(range, "border");
                        xlObj.setActiveSheetIndex(sheetIdx);
                        details.cutCells = cutCells;
                        details.cfat = { table: tblName, range: range };
                        isCut = true;
                        if (xlObj.element.find(".e-cdata").length)
                            xlObj.element.find(".e-cutright, .e-cutbottom").removeClass("e-cutright e-cutbottom");
                    }
                    else {
                        var copyCells = [];
                        if (this._isSpecial) {
                            for (i = 0, len = this._copyCells.length; i < len; i++) {
                                rowIdx = this._copyCells[i].rowIndex;
                                colIdx = this._copyCells[i].colIndex;
                                cell = { rowIndex: rowIdx, colIndex: colIdx };
                                if (!ej.isNullOrUndefined(container.sheets[sheetIdx][rowIdx]))
                                    $.extend(true, cell.prevObj, container.sheets[sheetIdx][rowIdx][colIdx]);
                                else
                                    cell.prevObj = {};
                                copyCells.push(cell);
                            }
                        }
                        else
                            copyCells = this._copyCells;
                        details.cutCells = copyCells;
                    }
                    if (row.length)
                        pRange = [minRowIdx, minColIdx, minRowIdx + row.length - 1, minColIdx + row[0].split("\t").length - 1];
                    else {
                        pRange = [minRowIdx, minColIdx, minRowIdx, minColIdx];
                        i = j = 1;
                    }
                    details.pasteBorder = { range: pRange, prevBorder: xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(pRange) };
                    var content = xlObj._getContent(sheetIdx).find(".e-table")[0].tBodies[0];
                    for (i = 0, len = row.length; i < len; i++) {
                        cell = row[i].split("\t");
                        for (j = 0, cLen = cell.length; j < cLen; j++) {
                            currentCell = content.rows[minRowIdx + i] ? xlObj.getCell(minRowIdx + i, minColIdx + j)[0] : undefined;
                            if (ej.isNullOrUndefined(currentCell)) {
                                if (actSheet.colCount <= minColIdx + j)
                                    xlObj.XLScroll._createNewColumn(sheetIdx, { rowIndex: -1, colIndex: -1 }, { rowIndex: -1, colIndex: -1 }, "insert");
                                else {
                                    xlObj.XLScroll._createNewRow(sheetIdx, -1, -1, "insert");
                                }
                                xlObj.XLScroll._refreshScroller(sheetIdx, "refresh", "all");
                                currentCell = xlObj.getCell(minRowIdx + i, minColIdx + j);
                            }
                            else
                                currentCell = $(currentCell);
                            currCell = { rowIndex: minRowIdx + i, colIndex: minColIdx + j };
                            currCell.prevObj = (!ej.isNullOrUndefined(container.sheets[sheetIdx][minRowIdx + i])) ? $.extend(true, {}, container.sheets[sheetIdx][minRowIdx + i][minColIdx + j]) : {};
                            if (this._isCut && isColor)
                                return;
                            currentCellIdx = xlObj._getCellIdx(currentCell[0]);
                            if (this._isSpecial && isColor) { //paste special 
                                range = xlObj.getRangeIndices(this._copyRange[1]);
                                rangeCells = xlObj.getRange(range[0], range[1], range[2], range[3], this._copyRange[0]);
                                rowIdx = currentCellIdx.rowIndex;
                                colIdx = currentCellIdx.colIndex;
                                cRowIdx = $(rangeCells[k]).parent("tr").index();
                                cColIdx = rangeCells[k].cellIndex;
                                currentCell[0].className = currentCell[0].className.replace(regx, "");
                                className = this._copyBackup.cells[k].format;
                                if (!("merge" in this._copyBackup.cells[k].cell) && !ej.isNullOrUndefined(className))
                                    currentCell.addClass(className);
                                cellObj = $.extend(true, {}, this._copyBackup.cells[k].cell);
                                if (cellObj) {
                                    if (!(rowIdx in container.sheets[sheetIdx]))
                                        container.sheets[sheetIdx][rowIdx] = {};
                                    if (!(colIdx in container.sheets[sheetIdx][rowIdx]))
                                        container.sheets[sheetIdx][rowIdx][colIdx] = {};
                                    else {
                                        if ("comment" in container.sheets[sheetIdx][rowIdx][colIdx])
                                            xlObj.getCell(rowIdx, colIdx).removeClass("e-commentcell");
                                    }
									overflow = container.sheets[sheetIdx][rowIdx][colIdx]['overflow'];
                                    container.sheets[sheetIdx][rowIdx][colIdx] = cellObj;
									(overflow) && (container.sheets[sheetIdx][rowIdx][colIdx]['overflow'] = overflow);
                                    if (cellObj.value2 && typeof cellObj.value2 == "string" && !cellObj.formatStr)
                                        value2 = container.sharedData[cellObj.value][0] === "=" ? container.sharedData[cellObj.value] : cellObj.value2;
                                    else
                                        value2 = container.sharedData[cellObj.value];
                                    if (!("merge" in cellObj))
                                        xlObj.XLEdit.updateCellValue(currentCellIdx, value2);
                                    if ("merge" in cellObj) {
                                        var length = 0, pstCell, pstCellObj, rowSpan, colSpan, merge = cellObj.merge;
                                        range = xlObj.getRangeIndices(merge.mRange);
                                        var rowCnt = range[2] - range[0], colCnt = range[3] - range[1], selectCells = xlObj.getSheet(sheetIdx)._selectedCells;
                                        var cellRange = xlObj._getAlphaRange(sheetIdx, currCell.rowIndex, currCell.colIndex, currCell.rowIndex + rowCnt, currCell.colIndex + colCnt);
                                        var mCell = xlObj.getRange(range[0], range[1], range[2], range[3], sheetIdx).eq(0).attr("rowspan"), sRowCnt = endCell.rowIndex - startCell.rowIndex, sColCnt = endCell.colIndex - startCell.colIndex;//sRowCnt is selected cell row count & sColCnt is selectedcell Colcount
                                        pstCell = xlObj.XLSelection._getSelectedRange({ rowIndex: currCell.rowIndex, colIndex: currCell.colIndex }, { rowIndex: currCell.rowIndex + rowCnt, colIndex: currCell.colIndex + colCnt });
                                        if (rangeCells.length >= selectCells.length || !(((sColCnt + 1) % (colCnt + 1)) === 0 && ((sRowCnt + 1) % (rowCnt + 1)) === 0)) {
                                            for (m = 0, rLen = pstCell.length; m < rLen; m++) {
                                                if (ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(pstCell[m].rowIndex, pstCell[m].colIndex, "isMHide"))) {
                                                    pstCellObj = xlObj.XLEdit.getPropertyValue(pstCell[m].rowIndex, pstCell[m].colIndex, "merge");
                                                    length++;
                                                    if (pstCellObj) {
                                                        rowSpan = pstCellObj.mSpan.rowSpan;
                                                        colSpan = pstCellObj.mSpan.colSpan;
                                                    }
                                                    if (!ej.isNullOrUndefined(rowSpan) && !ej.isNullOrUndefined(colSpan))
                                                        length += (parseInt(rowSpan) * parseInt(colSpan)) - 1;
                                                    if (pstCellObj.isCenterAlign) {
                                                        if (xlObj._isRowViewable(sheetIdx, pstCell[m].rowIndex)) 
                                                            xlObj.getCell(pstCell[m].rowIndex, pstCell[m].colIndex).addClass("e-calign").removeClass("e-ralign");
                                                    }
                                                }
                                            }
                                            if (pstCell.length === length) {
                                                if (this._cutCells.length) {
                                                    xlObj.XLEdit.updateCellValue(xlObj._getAlphaRangeCells(cellObj.mRange), "");
                                                    xlObj.setActiveSheetIndex(cSheetIdx);
                                                    xlObj._unMergeAllCells(this._copyRange[0], 0, xlObj.XLSelection._getSelectedRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }), [], 0, 0);
                                                    xlObj.setActiveSheetIndex(sheetIdx);
                                                }
                                                xlObj._paste = true;
                                                if (!xlObj.isUndefined(currCell.prevObj.mRange))
                                                    xlObj.unmergeCells(currCell.prevObj.mRange);
                                                xlObj.unmergeCells(cellRange);
                                                xlObj._mergeCenter = cellObj.isCenterAlign;
                                                if (!xlObj.isUndefined(mCell) && mCell === "1")
                                                    xlObj.mergeAcrossCell(cellRange, true);
                                                else
                                                    xlObj.mergeCells(cellRange, true);
                                                xlObj.XLEdit.updateCellValue(currentCell, value2);
                                                if (!ej.isNullOrUndefined(className))
                                                    currentCell.addClass(className);
                                            }
                                            else {
                                                var obj = $.extend(true, {}, cellObj);
                                                obj.value = obj.value2 = value2;
                                                if (!ej.isNullOrUndefined(className))
                                                    xlObj.getCell(range[0], range[1], this._copyRange[0]).addClass(className);
                                                xlObj.XLEdit._updateDataContainer({rowIndex: range[0], colIndex: range[1]}, obj);
                                                xlObj._showAlertDlg("Alert", "MergeAlert", "", 250);
                                                return;
                                            }
                                        }
                                        xlObj.performSelection({ "rowIndex": currCell.rowIndex, "colIndex": currCell.colIndex }, { "rowIndex": currCell.rowIndex + rowCnt, "colIndex": currCell.colIndex + colCnt });
                                    }
                                    if ("hyperlink" in cellObj) {
                                        hldata = cellObj.hyperlink;
                                        if (hldata && !hldata.webAddr)
                                            currentCell.html(ej.buildTag("a.e-hyperlinks", currentCell.text(), "", { href: "#" }));
                                        else
                                            currentCell.html(ej.buildTag("a.e-hyperlinks", currentCell.text(), "", { href: hldata.webAddr, target: "_blank" }));
                                    }
                                    if ("comment" in cellObj) {
                                        currentCell.removeClass("e-commentcell");
                                        xlObj.XLComment.setComment([rowIdx, colIdx, rowIdx, colIdx], null, false);
                                    }
                                    if ("cFormatRule" in cellObj) {
                                        cFormatRule = $.extend(true, [], cellObj.cFormatRule);
                                        cellObj.cFormatRule = [];
                                        for (m = 0, rLen = cFormatRule.length; m < rLen; m++) {
                                            splitStr = xlObj._dataContainer.cFormatData[cFormatRule[m]].split("_");
                                            xlObj.XLCFormat._cFormat(splitStr[0], splitStr[2], splitStr[3], splitStr[4], null, null, [rowIdx, colIdx, rowIdx, colIdx], "paste");
                                        }
                                    }
                                    if ("wrap" in cellObj && cellObj.wrap) {
                                        addr = xlObj._generateHeaderText(sheetIdx, colIdx + 1) + (rowIdx + 1);
                                        xlObj.setWrapText(addr + ":" + addr, "wrap");
                                    }
                                    xlObj._isPaste = true;
                                    if ("picture" in cellObj) {
                                        for (m = 0, rLen = cellObj.picture.length; m < rLen; m++) {
                                            cell = this._copyCells.length ? this._copyCells[k] : this._cutCells[k];
                                            cellPos = this._copyBackup.actCellPos;
                                            currCellObj = xlObj.getRangeData([cell.rowIndex, cell.colIndex, cell.rowIndex, cell.colIndex], "", "", this._copyRange[0])[0];
                                            if ((!xlObj.isUndefined(currCellObj.picture) && currCellObj.picture.indexOf(cellObj.picture[m]) > -1) || !this._copyCells.length) {
                                                $.extend(true, shapeObj, shapeMgr.picture[cellObj.picture[m]]);
                                                shapeElem = xlObj.element.find("#" + shapeObj.id);
                                                if (shapeObj.rowIndex === cell.rowIndex && shapeObj.colIndex === cell.colIndex) {
                                                    shapeElem = xlObj.element.find("#" + shapeObj.id);
                                                    className = shapeElem[0].className;
                                                    if (className.indexOf("e-shapebdr") > -1) {
                                                        className = shapeElem[0].className.match(shapeRegx)[0];
                                                        style = className.replace("e-shapebdr", "").split("N");
                                                        bcolor = style[2];
                                                        xlObj.XLShape._createPicture(sheetIdx, currentCell[0], shapeMgr.sharedPics[shapeObj.data], parseInt(shapeElem.css("top")) - cellPos.top + actCell[0].offsetTop, parseInt(shapeElem.css("left")) - cellPos.left + actCell[0].offsetLeft, shapeObj.width, shapeObj.height, null, "#" + bcolor, bstyle[parseInt(style[1]) - 1], style[0] + "px");
                                                    }
                                                    else
                                                        xlObj.XLShape._createPicture(sheetIdx, currentCell[0], shapeMgr.sharedPics[shapeObj.data], parseInt(shapeElem.css("top")) - cellPos.top + actCell[0].offsetTop, parseInt(shapeElem.css("left")) - cellPos.left + actCell[0].offsetLeft, shapeObj.width, shapeObj.height);
                                                    if (this._copyRange[0] === sheetIdx)
                                                        container.sheets[sheetIdx][minRowIdx + i][minColIdx + j].picture[m] = shapeMgr.picture.length - 1;
                                                    if (this._cutCells.length)
                                                        shapeElem.remove();
                                                }
                                            }
                                        }
                                    }
                                    if ("chart" in cellObj) {
                                        currCellObj = xlObj.getRangeData([rowIdx, colIdx, rowIdx, colIdx]);
                                        for (m = 0, rLen = cellObj.chart.length; m < rLen; m++) {
                                            cell = this._copyCells.length ? this._copyCells[k] : this._cutCells[k];
                                            cellPos = this._copyBackup.actCellPos;
                                            currCellObj = xlObj.getRangeData([cell.rowIndex, cell.colIndex, cell.rowIndex, cell.colIndex])[0];
                                            if ((!xlObj.isUndefined(currCellObj.chart) && currCellObj.chart.indexOf(cellObj.chart[m]) > -1) || !this._copyCells.length) {
                                                $.extend(true, shapeObj, shapeMgr.chart[cellObj.chart[m]]);
                                                shapeElem = xlObj.element.find("#" + shapeObj.id);
                                                if (shapeObj.rowIndex === cell.rowIndex && shapeObj.colIndex === cell.colIndex) {
                                                    range = shapeObj.range;
                                                    range = xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]);
                                                    if (range.indexOf(":") < 0)
                                                        range = range + ":" + range;
                                                    shapeObj.top = parseInt(shapeElem.css("top")) - cellPos.top + actCell[0].offsetTop;
                                                    shapeObj.left = parseInt(shapeElem.css("left")) - cellPos.left + actCell[0].offsetLeft;
                                                    xlObj.XLChart.createChart(range, shapeObj);
                                                    container.sheets[sheetIdx][minRowIdx + i][minColIdx + j].chart[m] = shapeMgr.chart.length - 1;
                                                    if (this._cutCells.length)
                                                        shapeElem.remove();
                                                }
                                            }
                                        }
                                    }
                                }
                                else {
                                    xlObj.XLEdit.updateCellValue(currentCell, "");
                                    delete container.sheets[sheetIdx][rowIdx][colIdx];
                                }
                            }
                            else { //paste values only
                                xlObj.XLEdit.updateCellValue(currentCell, cell[j]);
                                if (currentCell.hasClass("e-commentcell"))
                                    xlObj.XLComment._updateCmntFrmCntnr(null, null, currentCell);
                                if (cell[j].indexOf("=") > -1)
                                    xlObj.XLEdit.updateCellValue(currentCell, currentCell.html()); //remove formula 
                            }
                            k++;
                            currCell.newObj = (!ej.isNullOrUndefined(container.sheets[sheetIdx][minRowIdx + i])) ? $.extend(true, {}, container.sheets[sheetIdx][minRowIdx + i][minColIdx + j]) : {};
                            if ((ej.isNullOrUndefined(currCell.newObj)))
                                currCell.newObj = {};
                            pasteCells.push(currCell);
                        }
                        xlObj._setRowHdrHeight(sheetIdx, currentCell.parent().index());
                    }
                    if (row.length < 1)
                        i = 1;
                    if (this._cutCells.length) {
                        this._isCut = isCut;
                        this._cutCells = [];
                    }
                    if (this._isSpecial && isColor) { //paste - border, filtering, table                        
                        var tblRange, table = this._copyBackup.table, tObj;
                        range = xlObj.getRangeIndices(this._copyRange[1]);
                        for (m = 0, rLen = table.length; m < rLen; m++) {
                            tblRange = this._copyBackup.table[m].range;
                            tObj = $.extend(true, {}, this._copyBackup.table[m].tblObj);
                            tObj.name = "";
                            xlObj.XLFormat.createTable(tObj, xlObj._getAlphaRange(this._copyRange[0], tblRange[0] + minRowIdx - range[0], tblRange[1] + minColIdx - range[1], minRowIdx -
                            range[0] + tblRange[2], minColIdx - range[1] + tblRange[3]));
                            pastedTable.push(this.XLObj.model.sheets[sheetIdx]._tableManager[this.XLObj.getObjectLength(this.XLObj.model.sheets[sheetIdx]._tableManager) - 1]);
                        }
                        pRange = [minRowIdx, minColIdx, (i + minRowIdx - 1), (j + minColIdx - 1)];
                        //xlObj.model.allowCellFormatting && xlObj.XLFormat.updateFormat(this._copyBackup["border"], pRange);             //commented due to border issues      
                        if (this._cutCells.length && this._copyBackup.filterRange.length > 0 && this._copyRange[0] === sheetIdx) {
                            var fltrRange = this._copyBackup.filterRange;
                            xlObj.XLFilter.clearFilter();
                            fltr = xlObj._generateHeaderText(sheetIdx, fltrRange[1] + minColIdx - range[1] + 1) + (fltrRange[0] + minRowIdx - range[0] + 1) + ":" + xlObj._generateHeaderText(sheetIdx, minColIdx - range[1] + fltrRange[3] + 1) + (minRowIdx - range[0] + fltrRange[2] + 1);
                            xlObj.XLFilter.filter(fltr);
                        }
                        details.pasteBorder.newBorder = xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(pRange);
                        xlObj.setActiveSheetIndex(this._copyRange[0]);
                        details.cutBorder.newBorder = xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(range);
                        xlObj.setActiveSheetIndex(sheetIdx);
                    }
                    details.pfat = { table: this._copyBackup.table, range: range, minRowIdx: minRowIdx, minColIdx: minColIdx, ptable: pastedTable };
                    details.pfilter = fltr;
                    details.pasteCells = pasteCells;
                    xlObj.XLSelection.selectRange(({ rowIndex: minRowIdx, colIndex: minColIdx }), ({ rowIndex: (i + minRowIdx - 1), colIndex: (j + minColIdx - 1) }));
                    xlObj.getSheet(sheetIdx)._startCell = ({ rowIndex: minRowIdx, colIndex: minColIdx });
                    xlObj.getSheet(sheetIdx)._endCell = ({ rowIndex: (i + minRowIdx - 1), colIndex: (j + minColIdx - 1) });
                    if (xlObj.model.allowAutoFill)
                        xlObj.XLDragFill.positionAutoFillElement();
                    details.sheetIndex = sheetIdx;
                    xlObj.setSheetFocus();
                    xlObj.XLRibbon._updateRibbonIcons();
                    if (!xlObj._isUndoRedo)
                        xlObj._completeAction(details);
                }
            }
            else
                xlObj._showAlertDlg("Alert", "FullSheetCopyPasteAlert", "", 375);
        },

        _compareRange: function (range1, range2) {
            return (range1[0] <= range2[0] && range1[1] <= range2[1] && range1[2] >= range2[2] && range1[3] >= range2[3]);
        }

    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.comments = function (obj) {
        this.XLObj = obj;
        this._isCommentEdit = false;
        this._isShowAllComments = false;
        this._curCommentHCell = null;
    };

    ej.spreadsheetFeatures.comments.prototype = {

        setComment: function (range, data, showEditPanel, showUserName) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowComments)
                return;
            var sheetIdx = xlObj.getActiveSheetIndex(), prevCmnt, isViewed = false, activeCell, cellObj, trgtCell, rowIdx, colIdx, cmntData, activeCmnt, userName;
            if (ej.isNullOrUndefined(range))
                cellObj = xlObj.getActiveCell(sheetIdx);
            else {
                if (typeof (range) === "string")
                    range = xlObj.getRangeIndices(range);
                cellObj = { rowIndex: range[0], colIndex: range[1] };
            }
            rowIdx = cellObj.rowIndex; colIdx = cellObj.colIndex;
            showEditPanel = ej.isNullOrUndefined(showEditPanel) ? true : showEditPanel;
            showUserName = ej.isNullOrUndefined(showUserName) ? true : showUserName;
            userName = (xlObj.model.userName.length > 0 ? xlObj.model.userName : xlObj._getLocStr("UserName")) + ":\n";
            prevCmnt = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx);
            if (!ej.isNullOrUndefined(data)){
				if (showUserName)
					data = userName + data;
                xlObj.XLEdit._updateDataContainer(cellObj, { comment: { value: data, isVisible: ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx)) ? showEditPanel : xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx).isVisible } });
            }
			cmntData = ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx)) ? (xlObj.model.userName.length > 0 ? xlObj.model.userName : xlObj._getLocStr("UserName")) + ":\n" : xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx).value;
            if (xlObj.model.allowComments && ej.isNullOrUndefined(prevCmnt) && !xlObj.XLEdit._isEdit && !xlObj._isSheetRename) {
                if (xlObj._isRowViewable(sheetIdx, rowIdx)) {
                    activeCell = xlObj.getCell(rowIdx, colIdx);
                    activeCell.prepend(ej.buildTag("span.e-comment", ej.buildTag("span.e-celltoparrow")).append(ej.buildTag("textarea.e-comment-txtarea", cmntData, { "z-index": 10 }))).addClass("e-commentcell");
                    this._updateCmntArrowPos(activeCell);
                    activeCmnt = activeCell.find(".e-comment-txtarea");
                    this._updateCmtAreaPos(activeCell, activeCmnt);
                    activeCmnt.focus().setInputPos(activeCmnt.val().length);
                    isViewed = true;
                }
                if (xlObj.model.sheets[sheetIdx]._importRendered)
                    xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx).value = cmntData;
                if (showEditPanel && isViewed) {
                    activeCmnt.focus();
                    if (this._curCommentHCell && !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValueByElem(this._curCommentHCell, "comment")) && !xlObj.XLEdit.getPropertyValueByElem(this._curCommentHCell, "comment").isVisible) {
                        this._curCommentHCell.find(".e-comment-txtarea").hide();
                        this._curCommentHCell = null;
                    }
                    if (!xlObj.XLDragDrap._isDragAndDropped)
                        this._updateCurrentCell(rowIdx, colIdx);
                    this._isCommentEdit = true;
                    if (xlObj.model.showRibbon)
                        xlObj.XLRibbon._disableRibbonIcons();
                }
                else if(isViewed) {
                    if (!xlObj._isShowAllComments && !xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx).isVisible)
                        activeCmnt.hide();
                    if (!xlObj.model.sheets[sheetIdx]._importRendered && !xlObj.XLDragDrap._isDragAndDropped)
						if (xlObj.model.showRibbon)
							this._updateCmntRibIcons(xlObj.XLClipboard._isSpecial ? xlObj.getActiveCellElem() : activeCell);
                }
            }
        },

        _updateCmtAreaPos: function (activeCell, cmtTxtArea) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), hScroll = xlObj._getContent(sheetIdx).find("#hscrollBar").offset(), vScroll = xlObj._getContent(sheetIdx).find("#vscrollBar").offset(), hdrOffset = xlObj._getJSSheetHeader(sheetIdx).offset(),
                txtOffset = cmtTxtArea.offset(), cHeight = cmtTxtArea.height(), cWidth = cmtTxtArea.width();
            if (xlObj.model.scrollSettings.allowScrolling) {
                if ((txtOffset.top + cHeight + 5) > hScroll.top && (txtOffset.left + cWidth) < vScroll.left)
                    cmtTxtArea.css({ top: -77 });
                else if ((txtOffset.left + cWidth) > vScroll.left && (txtOffset.top + cHeight + 5) > hScroll.top)
                    cmtTxtArea.css({ left: -136, top: -81 });
                else if ((txtOffset.top + cHeight) > hdrOffset.top && (txtOffset.left + cWidth) > vScroll.left)
                    cmtTxtArea.css({ left: -137, top: activeCell.height() + 8 });
            }
        },

        deleteComment: function (range, status, skipHiddenRow) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowComments)
                return;
            var actCell, selCell = [], activeShtIdx = xlObj.getActiveSheetIndex(), activeCells;
            var details = { sheetIndex: activeShtIdx, reqType: "comment", action: "delete" };

            if (typeof (range) === "string")
                range = xlObj.getRangeIndices(range);
            activeCells = ej.isNullOrUndefined(range) ? xlObj.getSheetElement(activeShtIdx).find(".e-selected.e-commentcell") : xlObj.getRange(range[0], range[1], range[2], range[3], activeShtIdx).filter(".e-commentcell");

            if (xlObj.model.allowComments && activeCells.hasClass("e-commentcell")) {
                for (var i = 0; i < activeCells.length; i++) {
                    actCell = $(activeCells[i]);
                    if (skipHiddenRow && xlObj._isHiddenRow(actCell.parent().index())) {
                        activeCells.splice(i, 1);
                        continue;
                    }
                    if (!ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValueByElem(actCell, "comment")))
                        selCell.push({ rowIndex: actCell.parent().index(), colIndex: activeCells[i].cellIndex, prevComment: xlObj.XLEdit.getPropertyValueByElem(actCell, "comment"), currComment: {} });
                }
                details.selCells = selCell;

                //Delete action
                activeCells.find(".e-comment").parent("td").removeClass("e-commentcell").end().remove();
                xlObj.clearRangeData(null, ["comment"], activeCells);
                if (xlObj.model.showRibbon && !xlObj.XLDragDrap._isDragAndDropped)
                    xlObj.XLComment._updateCmntRibIcons();

                details.reqType = ej.isNullOrUndefined(status) ? "comment" : "clear-comment";
                if (!xlObj._isUndoRedo && !xlObj._dupDetails)
                    xlObj._completeAction(details);
            }
        },

        editComment: function (trgtCell) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowComments)
                return;
            var $target = ej.isNullOrUndefined(trgtCell) ? xlObj.getActiveCellElem() : $(trgtCell);
            if (!xlObj.XLEdit._isEdit && !xlObj._isSheetRename) {
                if (xlObj.model.allowComments && $target.hasClass("e-commentcell")) {
                    $target.find(".e-comment-txtarea").css("display", "inline-table").focus();
                    this._isCommentEdit = true;
                }
                if (xlObj.model.showRibbon)
                    xlObj.XLRibbon._disableRibbonIcons();
            }
        },

        showHideComment: function (trgtCell) {
            var xlObj = this.XLObj, activeCell = ej.isNullOrUndefined(trgtCell) ? xlObj.getActiveCellElem() : $(trgtCell), $target = activeCell.find(".e-comment-txtarea"), visibility = false;

            if (!xlObj.model.allowComments)
                return;

            if (activeCell.hasClass("e-commentcell")) {
                if (xlObj.XLEdit.getPropertyValueByElem(activeCell, "comment").isVisible) {
                    $target.hide();
                    visibility = false;
                }
                else {
                    $target.css("display", "inline-table");
                    this._curCommentHCell = null;
                    visibility = true;
                }
            }
            if (!xlObj.isUndefined(visibility))
                xlObj.XLEdit.getPropertyValueByElem(activeCell, "comment").isVisible = visibility;
        },

        showAllComments: function () {
            var xlObj = this.XLObj, $getAllComments = xlObj.getMainPanel().find(".e-comment-txtarea"), showToglBtObj = $("#" + xlObj._id + "_Ribbon_showAllComments").data("ejToggleButton"), ribIconsIds = ["previousComment", "nextComment"], i, len, visibleStatus;
            if (!xlObj.model.allowComments)
                return;
            if (this._isShowAllComments) {
                $getAllComments.hide();
                visibleStatus = false;
            } else {
                $getAllComments.css("display", "inline-table");
                visibleStatus = true;
            }
            this._isShowAllComments = visibleStatus;
            for (i = 1; i <= xlObj.model.sheetCount; i++) {
                var j = 0, sheetCmts = xlObj.getSheetElement(i).find(".e-commentcell");
                len = sheetCmts.length;
                while (j < len) {
                    xlObj.XLEdit.getPropertyValueByElem($(sheetCmts[j]), "comment", i).isVisible = visibleStatus;
                    j++;
                }
            }
            if (xlObj.model.showRibbon && showToglBtObj.model.toggleState)
                xlObj.XLRibbon._disableButtons(ribIconsIds, "ejButton");
            else if (xlObj.model.showRibbon && xlObj.getMainPanel().find(".e-commentcell").length > 0)
                xlObj.XLRibbon._enableButtons(ribIconsIds, "ejButton");
        },

        findPrevComment: function () {
            var xlObj = this.XLObj, activeSheet = xlObj.model.activeSheetIndex, rowCount, colCount, rowIndex, colIndex, i, j, k, cell, insCmtEle = $("#" + xlObj._id + "_Ribbon_newComment"), commentPanel;
            if (!xlObj.model.allowComments)
                return false;
            if (xlObj.getMainPanel().find(".e-commentcell").length < 1)
                return false;
            for (i = activeSheet; i > 0; i--) {
                if (i !== activeSheet)
                    xlObj.gotoPage(i, false);
                if (xlObj.model.showRibbon)
                    $("#" + xlObj._id + "_Ribbon").ejRibbon("option", "selectedItemIndex", 5);
                rowCount = xlObj.model.sheets[xlObj.model.activeSheetIndex].rowCount - 1;
                colCount = xlObj.model.sheets[xlObj.model.activeSheetIndex].colCount - 1;
                if (i !== activeSheet)
                    this._updateCurrentCell(rowCount - 1, colCount - 1);
                rowIndex = xlObj.getSheet(xlObj.model.activeSheetIndex)._activeCell.rowIndex;
                colIndex = xlObj.getSheet(xlObj.model.activeSheetIndex)._activeCell.colIndex;
                for (j = rowIndex; j >= 0; j--) {
                    if (j !== rowIndex)
                        colIndex = colCount - 1;
                    for (k = colIndex; k >= 0; k--) {
                        cell = $(xlObj.getCell(j, k));
                        commentPanel = $(cell.find(".e-comment-txtarea"));
                        if (xlObj.getActiveCellElem()[0] !== cell[0] && !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValueByElem(cell, "comment")) && xlObj.XLEdit.getPropertyValueByElem(cell, "comment").isVisible)
                            commentPanel.hide();
                        if (cell.hasClass("e-commentcell") && !commentPanel.is(":visible")) {
                            if (commentPanel.css("display", "inline-table").focus())
                                this._updateCurrentCell(j, k);
                            if (xlObj.model.showRibbon) {
                                xlObj.XLRibbon._enableButtons(["deleteComment", "showHideComment"], "ejButton");
                                insCmtEle.find(".e-icon").removeClass("e-ssr-newcmnt").addClass("e-ssr-editcmnt");
                                    insCmtEle.find(".e-btntxt").text(xlObj._getLocStr("EditComment"));
                            }
                            xlObj.setSheetFocus();
                            return true;
                        }
                        if (!this._isShowAllComments && !this._isCommentEdit && cell.hasClass("e-commentcell") && !xlObj.XLEdit.getPropertyValueByElem(cell, "comment").isVisible)
                            commentPanel.hide();
                    }
                }
            }
            if (i === 0)
                xlObj._showAlertDlg("", "CommentFindEndAlert", "findPrevComment", 630);
        },

        findNextComment: function () {
            var xlObj = this.XLObj, activeSheet = xlObj.model.activeSheetIndex, rowIndex, colIndex, rowCount, colCount, i, j, k, cell, commentPanel, insCmtEle = $("#" + xlObj._id + "_Ribbon_newComment");
            if (!xlObj.model.allowComments)
                return false;
            if (xlObj.getMainPanel().find(".e-commentcell").length < 1)
                return false;
            for (i = activeSheet; i <= xlObj.model.sheetCount; i++) {
                if (i !== activeSheet)
                    xlObj.gotoPage(i, false);
                if (xlObj.model.showRibbon)
                    $("#" + xlObj._id + "_Ribbon").ejRibbon("option", "selectedItemIndex", 5);
                if (i !== activeSheet)
                    this._updateCurrentCell(0, 0);
                rowIndex = xlObj.getSheet(xlObj.model.activeSheetIndex)._activeCell.rowIndex;
                colIndex = xlObj.getSheet(xlObj.model.activeSheetIndex)._activeCell.colIndex;
                rowCount = xlObj.model.sheets[xlObj.model.activeSheetIndex].rowCount - 1;
                colCount = xlObj.model.sheets[xlObj.model.activeSheetIndex].colCount - 1;
                for (j = rowIndex; j < rowCount; j++) {
                    if (j !== rowIndex)
                        colIndex = 0;
                    for (k = colIndex; k < colCount; k++) {
                        cell = $(xlObj.getCell(j, k));
                        commentPanel = $(cell.find(".e-comment-txtarea"));
                        if (xlObj.getActiveCellElem()[0] !== cell[0] && !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValueByElem(cell, "comment")) && xlObj.XLEdit.getPropertyValueByElem(cell, "comment").isVisible)
                            commentPanel.hide();
                        if (cell.hasClass("e-commentcell") && !commentPanel.is(":visible")) {
                            commentPanel.css("display", "inline-table").focus();
                                this._updateCurrentCell(j, k);
                            if (xlObj.model.showRibbon) {
                                xlObj.XLRibbon._enableButtons(["deleteComment", "showHideComment"], "ejButton");
                                insCmtEle.find(".e-icon").removeClass("e-ssr-newcmnt").addClass("e-ssr-editcmnt");
                                    insCmtEle.find(".e-btntxt").text(xlObj._getLocStr("EditComment"));
                            }
                            xlObj.setSheetFocus();
                            return true;
                        }
                        if (!this._isShowAllComments && !this._isCommentEdit && cell.hasClass("e-commentcell") && !xlObj.XLEdit.getPropertyValueByElem(cell, "comment").isVisible)
                            commentPanel.hide();
                    }
                }
            }
            if (i === xlObj.model.sheetCount + 1)
                xlObj._showAlertDlg("", "CommentFindEndAlert", "findNextComment", 630);
        },

        _commentMouseMove: function (e) {
            var xlObj = this.XLObj, $target = $(e.target), commentPanel = $target.find(".e-comment-txtarea");
            if (!this._isCommentEdit) {
                if (!ej.isNullOrUndefined(this._curCommentHCell)) {
                    if ($target !== this._curCommentHCell) {
                        var cellData = xlObj.XLEdit.getPropertyValueByElem(this._curCommentHCell, "comment");
                        if (!ej.isNullOrUndefined(cellData) && !cellData.isVisible) {
                            this._curCommentHCell.find(".e-comment-txtarea").hide();
                            this._curCommentHCell = null;
                        }
                    }
                }
                if (e.target.tagName === "TD" && $target.hasClass("e-commentcell")) {
                    commentPanel.css("display", "inline-table");
                    this._curCommentHCell = $target;
                }
            }
        },

        _commentMouseDownHandler: function (e) {
            var xlObj = this.XLObj, activeCell = xlObj.getActiveCellElem(), cellData, commentPanel = $(activeCell.find(".e-comment-txtarea"));
            if (activeCell.hasClass("e-commentcell") && commentPanel.get()[0] !== e.target) {
                cellData = xlObj.XLEdit.getPropertyValueByElem(activeCell, "comment");
                if (this._isCommentEdit)
                    this._updateCurCmntVal({ rowIndex: activeCell.parent().index(), colIndex: activeCell.index() });
                if (!this._isShowAllComments && !xlObj.isUndefined(cellData) && !cellData.isVisible)
                    commentPanel.hide();
            }

            if ($(e.target).hasClass("e-comment-txtarea")) {
                if ($(e.target).focus().css("z-index", 10))
                    this._isCommentEdit = true;
                this._updateCurrentCell($(e.target).parents("td").parent().index(), $(e.target).parents("td").index());
                if (xlObj.model.showRibbon)
                    xlObj.XLRibbon._disableRibbonIcons();
            }
        },

        _updateCmntRibIcons: function (trgtCell) {
            var xlObj = this.XLObj, ribIconsIds = ["showHideComment", "deleteComment"], activeSheet = xlObj.getActiveSheetIndex(), sheetElement = xlObj.getSheetElement(activeSheet), insCmtEle = $('#' + xlObj._id + '_Ribbon_newComment'), actCell = xlObj.getActiveCellElem(), selectedCells = sheetElement.find('.e-selected');
            trgtCell = (xlObj.model.allowDragandDrop && !xlObj.XLDragDrap._isDragAndDropped) ? (trgtCell ? trgtCell : actCell) : xlObj.getCell(xlObj._dStartCell.rowIndex, xlObj._dStartCell.colIndex);
            xlObj.XLRibbon._enableButtons(["newComment"], "ejButton");
            xlObj.XLRibbon._enableButtons(xlObj.XLRibbon._cmntTglBtnIds, "ejToggleButton");
            if (trgtCell.hasClass("e-commentcell")) {
                xlObj.XLRibbon._enableButtons(ribIconsIds, "ejButton");
                insCmtEle.find(".e-icon").removeClass("e-ssr-newcmnt").addClass("e-ssr-editcmnt");
                insCmtEle.find(".e-btntxt").text(xlObj._getLocStr("EditComment"));
            }
            else {
                insCmtEle.find(".e-icon").removeClass("e-ssr-editcmnt").addClass("e-ssr-newcmnt");
                insCmtEle.find(".e-btntxt").text(xlObj._getLocStr("NewComment"));
                xlObj.XLRibbon._disableButtons(ribIconsIds, "ejButton");
            }
            if (this._isShowAllComments || xlObj.element.find(".e-commentcell").length === 0)
                xlObj.XLRibbon._disableButtons(["previousComment", "nextComment"], "ejButton");
            else
                xlObj.XLRibbon._enableButtons(["previousComment", "nextComment"], "ejButton");
            if (selectedCells.length > 1 && selectedCells.is(trgtCell) && (selectedCells.filter(".e-commentcell").length > 0))
                xlObj.XLRibbon._enableButtons(["deleteComment"], "ejButton");
        },

        _updateCurCmntVal: function (trgtCell, status) {
            status = ej.isNullOrUndefined(status) ? "" : status;
            var xlObj = this.XLObj, activeCell = ej.isNullOrUndefined(trgtCell) ? xlObj.getActiveCell() : trgtCell, commentPanel, cmntVal, selCell = [], cmntVal = "User Name:";
            var details = { sheetIndex: xlObj.getActiveSheetIndex(), reqType: "comment" }, rowIdx = activeCell.rowIndex, colIdx = activeCell.colIndex;
            if(xlObj._isRowViewable(details.sheetIndex, rowIdx)){
                commentPanel = xlObj.getCell(rowIdx, colIdx, xlObj.getActiveSheetIndex()).find(".e-comment-txtarea");
                isViewed = true;
            }
            selCell.push({ rowIndex: rowIdx, colIndex: colIdx });
            details.action = (ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment"))) ? "add" : "edit";
            details.selCells = selCell;
            details.selCells[0].prevComment = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment");
            isViewed && (cmntVal = commentPanel.css("z-index", 9).val());
            xlObj.XLEdit._updateDataContainer({rowIndex: rowIdx, colIndex: colIdx}, { comment: { value: cmntVal, isVisible: xlObj.isUndefined(xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment")) ? this._isShowAllComments : xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment").isVisible } });
            if (isViewed && !this._isShowAllComments && !xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment").isVisible)
                commentPanel.hide();
            xlObj.setSheetFocus();
            this._isCommentEdit = false;
            xlObj.XLCellNav._isNavigate = true;
            if (xlObj.model.showRibbon)
                xlObj.XLRibbon._enableRibbonIcons();
            xlObj.XLEdit._updateUsedRange(rowIdx, colIdx);
            details.selCells[0].curComment = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment");
            if (!xlObj._isUndoRedo)
                if (!xlObj._hasComment && status !== "paste")
                    xlObj._completeAction(details);
        },

        _updateCurrentCell: function (rIndex, cIndex, sheetIdx) {
            var xlObj = this.XLObj, cellObj = { rowIndex: rIndex, colIndex: cIndex };
            sheetIdx = sheetIdx ? sheetIdx : xlObj.model.activeSheetIndex;
            var shtModel = xlObj.getSheet(sheetIdx);
            shtModel._activeCell = cellObj;
            xlObj.XLSelection.selectRange(cellObj, cellObj, sheetIdx);
            shtModel._startCell = shtModel._endCell = cellObj;
            if(xlObj.model.allowAutoFill)
				xlObj.XLDragFill.positionAutoFillElement();
        },

        _updateCmntArrowPos: function (cells) {
            var xlObj = this.XLObj, commentCells = cells ? cells : xlObj.getSheetElement(xlObj.getActiveSheetIndex()).find(".e-commentcell"), i, activeCell, len, cmntElem;
            if (xlObj.model.allowComments) {
                for (i = 0, len = commentCells.length; i < len; i++) {
                    activeCell = $(commentCells[i]);
                    cmntElem = activeCell.find(".e-comment");
                    if (cmntElem.length) {
                        if (activeCell.width() < 5) { // 5 - comment arrow indicator width
                            cmntElem.hide();
                            continue;
                        }
                        if (cmntElem.is(":hidden"))
                            cmntElem.show();
                        cmntElem.css({ "top": activeCell.get()[0].offsetTop + "px", "left": (activeCell.get()[0].offsetLeft + activeCell.get()[0].offsetWidth - 5) + "px" });
                    }
                }
            }
        },

        _updateCmntFrmCntnr: function (i, j, cCell) {
            var xlObj = this.XLObj, cell = cCell || xlObj.getCell(i, j);
            i = xlObj.isUndefined(i) ? cell[0].parentNode.rowIndex : i;
            j = xlObj.isUndefined(j) ? cell[0].cellIndex : j;
            if (xlObj.XLEdit.getPropertyValue(i, j, "comment")) {
                xlObj._removeClass(cell[0], "e-commentcell");
                this.setComment([i, j, i, j], null, false);
            }
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.contextmenu = function (obj) {
        this.XLObj = obj;
        this._clrFltrEnable = false;
        this._isMenuOpened = false;
        this._uniqueClass = this.XLObj._id + "-cmenu";
        this._rowCellMenuData = [
				{ id: "Cut", text: obj._getLocStr("Cut"), parentId: null, sprite: "e-icon e-ss-cut" },
				{ id: "Copy", text: obj._getLocStr("Copy"), parentId: null, sprite: "e-icon e-ss-copy" },
				{ id: "Paste", text: obj._getLocStr("Paste"), parentId: null, sprite: "e-icon e-ss-paste" },
                { id: "Insert", text: obj._getLocStr("Insert") + "...", parentId: null },
                { id: "Delete", text: obj._getLocStr("Delete") + "...", parentId: null },
				{ id: "Filter", text: obj._getLocStr("Filter"), parentId: null, sprite: "e-icon e-ss-filter" },
				{ id: "Sort", text: obj._getLocStr("Sort"), parentId: null, sprite: "e-icon e-ss-sortatoz" },
                { id: "SortAtoZ", parentId: "Sort", text: obj._getLocStr("SortAtoZ"), sprite: "e-icon e-ss-sortatoz" },
				{ id: "SortZtoA", parentId: "Sort", text: obj._getLocStr("SortZtoA"), sprite: "e-icon e-ss-sortztoa" },
				{ id: "ctxComment", parentId: null, text: obj._getLocStr("Comment") },
                { id: "ctxInsrtCmnt", text: obj._getLocStr("InsertComment"), parentId: "ctxComment", sprite: "e-icon e-ss-newcmnt" },
                { id: "ctxEditCmnt", text: obj._getLocStr("EditComment"), parentId: "ctxComment", sprite: "e-icon e-ss-editcmnt" },
                { id: "ctxDeleteCmnt", text: obj._getLocStr("DeleteComment"), parentId: "ctxComment", sprite: "e-icon e-ss-deletecmnt" },
                { id: "ctxShwHdCmnt", text: obj._getLocStr("ShowHide"), parentId: "ctxComment", sprite: "e-icon" },
				{ id: "PutCellColor", parentId: "Sort", text: obj._getLocStr("PutCellColor"), sprite: "" },
				{ id: "PutFontColor", parentId: "Sort", text: obj._getLocStr("PutFontColor"), sprite: "" },
                { id: "CmHyperLink", text: obj._getLocStr("HyperLink"), parentId: null, sprite: "e-icon e-ss-hyperlink" },
                { id: "PasteSpecial", text: obj._getLocStr("PasteSpecial"), parentId: "Paste" },
                { id: "PasteValues", text: obj._getLocStr("PasteValues"), parentId: "Paste" },
                { id: "ClearContents", text: obj._getLocStr("ClearContents"), parentId: null, sprite: "e-icon e-ss-clear" },
                { id: "ClearFilter", parentId: "Filter", text: obj._getLocStr("ClearFilter"), sprite: "e-icon e-ss-filternone" },
				{ id: "FilterSelected", parentId: "Filter", text: obj._getLocStr("FilterSelected"), sprite: "e-icon e-ss-filter" },
                { id: "HyperLink", text: obj._getLocStr("HyperLink") + "...", parentId: "CmHyperLink", sprite: "e-icon e-ss-hyperlink" },
                { id: "EditLink", text: obj._getLocStr("EditLink") + "...", parentId: "CmHyperLink" },
                { id: "OpenLink", text: obj._getLocStr("OpenLink"), parentId: "CmHyperLink" },
                { id: "RemoveLink", text: obj._getLocStr("RemoveLink"), parentId: "CmHyperLink", sprite: "e-icon e-ss-removelink" }
        ];
        this._columnHeaderMenuData = [
				{ id: "Cut", text: obj._getLocStr("Cut"), parentId: null, sprite: "e-icon e-ss-cut" },
				{ id: "Copy", text: obj._getLocStr("Copy"), parentId: null, sprite: "e-icon e-ss-copy" },
				{ id: "Paste", text: obj._getLocStr("Paste"), parentId: null, sprite: "e-icon e-ss-paste" },
                { id: "Insert", text: obj._getLocStr("Insert"), parentId: null },
                { id: "Delete", text: obj._getLocStr("Delete"), parentId: null },
                { id: "ClearContents", text: obj._getLocStr("ClearContents"), parentId: null, sprite: "e-icon e-ss-clear" },
                { id: "Hide", text: obj._getLocStr("Hide"), parentId: null },
                { id: "Unhide", text: obj._getLocStr("Unhide"), parentId: null },
                { id: "PasteSpecial", text: obj._getLocStr("PasteSpecial"), parentId: "Paste" },
                { id: "PasteValues", text: obj._getLocStr("PasteValues"), parentId: "Paste" }
        ];

        this._rowHeaderMenuData = [
				{ id: "Cut", text: obj._getLocStr("Cut"), parentId: null, sprite: "e-icon e-ss-cut" },
				{ id: "Copy", text: obj._getLocStr("Copy"), parentId: null, sprite: "e-icon e-ss-copy" },
				{ id: "Paste", text: obj._getLocStr("Paste"), parentId: null, sprite: "e-icon e-ss-paste" },
                { id: "Insert", text: obj._getLocStr("Insert"), parentId: null },
                { id: "Delete", text: obj._getLocStr("Delete"), parentId: null },
                { id: "ClearContents", text: obj._getLocStr("ClearContents"), parentId: null, sprite: "e-icon e-ss-clear" },
                { id: "Hide", text: obj._getLocStr("Hide"), parentId: null },
                { id: "Unhide", text: obj._getLocStr("Unhide"), parentId: null },
                { id: "PasteSpecial", text: obj._getLocStr("PasteSpecial"), parentId: "Paste" },
                { id: "PasteValues", text: obj._getLocStr("PasteValues"), parentId: "Paste" }
        ];

        this._footerMenuData = [
            { id: "InsertSheet", text: obj._getLocStr("InsertSheet"), parentId: null },
            { id: "DeleteSheet", text: obj._getLocStr("DeleteSheet"), parentId: null, sprite: "e-icon e-ss-delete" },
            { id: "RenameSheet", text: obj._getLocStr("RenameSheet"), parentId: null, sprite: "e-icon " },
            { id: "MoveorCopy", text: obj._getLocStr("MoveorCopy") + "...", parentId: null, sprite: "e-icon " },
            { id: "HideSheet", text: obj._getLocStr("HideSheet"), parentId: null, sprite: "e-icon " },
            { id: "UnhideSheet", text: obj._getLocStr("UnhideSheet"), parentId: null, sprite: "e-icon " },
            { id: "ProtectSheet", text: obj._getLocStr("Protectsheet"), parentId: null, sprite: "e-icon e-ss-cmenuprotect" }
        ];
        this._chartMenuData = [
            { id: "Cut", text: obj._getLocStr("Cut"), parentId: null, sprite: "e-icon e-ss-cut" },
            { id: "Copy", text: obj._getLocStr("Copy"), parentId: null, sprite: "e-icon e-ss-copy" },
            { id: "ChartType", text: obj._getLocStr("ChangeChartType") + "...", parentId: null },
            { id: "SelectData", text: obj._getLocStr("SelectData") + "...", parentId: null }
        ];
        this._imgMenuData = [
            { id: "Cut", text: obj._getLocStr("Cut"), parentId: null, sprite: "e-icon e-ss-cut" },
            { id: "Copy", text: obj._getLocStr("Copy"), parentId: null, sprite: "e-icon e-ss-copy" },
            { id: "ChangePicture", text: obj._getLocStr("ChangePicture"), parentId: null }
        ];
        this._protectRowMenuIds = ["Insert", "Delete", "Filter", "ctxComment", "Sort", "LockCells", "CmHyperLink"];
        this._prtctHeaderIds = ["Insert", "Delete", "Hide", "Unhide"];
    };

    ej.spreadsheetFeatures.contextmenu.prototype = {
        //Contextmenu
        _initContextMenu: function () {
            var createMenu = ej.buildTag("ul .e-spreadsheet e-"+this._uniqueClass, " ", { display: "none" }, { id: this.XLObj._id + "_contextMenuCell" });
            this._createMenu(createMenu, this._rowCellMenuData, ".e-spreadsheetcontentcontainer");
            createMenu = ej.buildTag("ul .e-spreadsheet e-"+ this._uniqueClass, " ", { display: "none" }, { id: this.XLObj._id + "_contextMenuColumnHeader" });
            this._createMenu(createMenu, this._columnHeaderMenuData, ".e-columnheader");
            createMenu = ej.buildTag("ul .e-spreadsheet e-"+ this._uniqueClass, " ", { display: "none" }, { id: this.XLObj._id + "_contextMenuRowHeader" });
            this._createMenu(createMenu, this._rowHeaderMenuData, ".e-rowheadercontent");
        },

        _initCMenuFooter: function () {
            var createMenu = ej.buildTag("ul .e-spreadsheet e-"+this._uniqueClass, " ", { display: "none" }, { id: this.XLObj._id + "_contextMenuFooter" });
            this._createMenu(createMenu, this._footerMenuData, ".e-pagercontainer");
        },

        _createMenu: function (createMenu, data, target) {
            createMenu.ejMenu(
            {
                fields: { dataSource: data, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" },
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                beforeOpen: $.proxy(this._beforeOpen, this),
                close: $.proxy(this._menuClose, this),
                click: $.proxy(this._menuClick, this),
                contextMenuTarget: target
            });

        },
        hideCMenu: function () {
            var xlObj = this.XLObj;
            if (!xlObj.model.enableContextMenu)
                return;
            $("#" + xlObj._id + "_contextMenuCell").data("ejMenu").element.hide();
            $("#" + xlObj._id + "_contextMenuColumnHeader").data("ejMenu").element.hide();
            $("#" + xlObj._id + "_contextMenuRowHeader").data("ejMenu").element.hide();
            $("#" + xlObj._id + "_contextMenuFooter").data("ejMenu").element.hide();
        },

        _beforeOpen: function (args) {
            var xlObj = this.XLObj, fltrIcon,dataSource;
            this._clrFltrEnable = false;
            if (!xlObj.model.enableContextMenu)
                args.cancel = true;
            this.hideCMenu();
            var trgt = args.target,text,Locked, evnt = { target: trgt }, $trgt = $(trgt), sheetIdx = xlObj.getActiveSheetIndex(), actSheet = xlObj.getSheet(sheetIdx), actCell = xlObj.getActiveCellElem().length > 0 ? xlObj.getActiveCellElem() : $trgt, menuData = this._rowCellMenuData, menuObj = $("#" + xlObj._id + "_contextMenuCell").data("ejMenu"), footMenuObj = $("#" + xlObj._id + "_contextMenuFooter").data("ejMenu"), actElem = xlObj.getActiveCell(sheetIdx), tabId, tabIdx, tabRange;
            if (xlObj._trigger("beforeOpen", evnt))
                return;
            if (xlObj.XLEdit._isEdit || (xlObj.model.allowComments && xlObj.XLComment._isCommentEdit) || $trgt.hasClass("e-filterspan") || (args.events.which === 1 && $trgt.hasClass("e-hyperlinks")) || $trgt.parents("div").hasClass("e-vscroll") || $trgt.parents("div").hasClass("e-hscroll") || ($(trgt.parentNode).hasClass("e-pagercontainer") && !$trgt.hasClass("e-link")) || (!ej.isNullOrUndefined(xlObj.XLResize) && xlObj.XLResize._resizeStart) || args.target.id.indexOf("Sheet_RenamePanel") > -1 || $trgt.parents().hasClass("e-grid") || $trgt.hasClass("e-autofill"))
                args.cancel = true;
            if ($trgt.hasClass("e-rowcell") || $trgt.hasClass("e-hyperlinks")) {
                var sortData = [], colHdr = xlObj._getJSSheetHeader(sheetIdx).find('th'), rowHdr = xlObj._getJSSheetRowHeader(sheetIdx).find('td'), colMenuObj = $("#" + xlObj._id + "_contextMenuColumnHeader").data("ejMenu"), rowMenuObj = $("#" + xlObj._id + "_contextMenuRowHeader").data("ejMenu");
                var value = xlObj.XLEdit.getPropertyValueByElem(xlObj.getCell(actSheet._activeCell.rowIndex, actSheet._activeCell.colIndex));
                this._isMenuOpened = true;
                if (xlObj.model.allowLockCell && actSheet._isSheetProtected) 
                    this._disableMenuOpt(this._protectRowMenuIds);
                else {
                if ($trgt.hasClass("e-hyperlinks")) {
                    $trgt = $trgt.parent("td");
                    trgt = $trgt[0];
                }
                if (!ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValueByElem($trgt, "range"))) {
                    var rangeInfo = xlObj.model.sheets[sheetIdx].rangeSettings[xlObj.XLEdit.getPropertyValueByElem($trgt, "range")];                    
                    menuObj.option("fields", { dataSource: rangeInfo.contextMenuSettings.dataSource, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" });
                    menuObj.option("width", rangeInfo.contextMenuSettings.width);
                    menuObj.option("height", rangeInfo.contextMenuSettings.height);
                }
                else if ($(colHdr[$trgt.index()]).hasClass('e-colselected') || $(rowHdr[$trgt.parent().index()]).hasClass("e-rowselected")) {
                    menuData = $(colHdr[$trgt.index()]).hasClass('e-colselected') ? this._columnHeaderMenuData : this._rowHeaderMenuData;
                    if (JSON.stringify(menuData) !== JSON.stringify(args.model.fields.dataSource))
                        menuObj.option("fields", { dataSource: menuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" });
                }
                else {
                    if (JSON.stringify(menuData) !== JSON.stringify(args.model.fields.dataSource))
                        menuObj.option("fields", { dataSource: menuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" });
                    if (xlObj.model.allowHyperlink) {
                        this._enableMenuOpt(["CmHyperLink"]);
                        var data = xlObj.XLEdit.getPropertyValue($trgt.parent().index(), trgt.cellIndex, "hyperlink", sheetIdx), hlItem = ["EditLink", "OpenLink", "RemoveLink"];
                        if (ej.isNullOrUndefined(data)) {
                            this._disableMenuOpt(hlItem);
                            this._enableMenuOpt(["HyperLink"]);
                        }
                        else {
                            this._enableMenuOpt(hlItem);
                            this._disableMenuOpt(["HyperLink"]);
                        }
                    }
                    else {
                        this._disableMenuOpt(["CmHyperLink"]);
                    }
                    if (!xlObj.model.allowFiltering)
                        menuObj.disableItemByID("Filter");
                    else {
                        menuObj.enableItemByID("Filter");
                        tabRange = xlObj.model.sheets[sheetIdx].filterSettings.tableRange;
                        var actvCell = xlObj.getCell(actElem.rowIndex, actElem.colIndex, sheetIdx)[0];
                        if (tabRange.length > 0) {
                            tabId = (actvCell.className.indexOf("e-table") > -1) ? (actvCell.className[actvCell.className.indexOf("e-table") + 7]) : -1;
                            for (var i = 0, n = tabRange.length; i < n; i++)
                                if (tabRange[i].tableID === parseInt(tabId)) {
                                    tabIdx = i;
                                    break;
                                }
                            if (!ej.isNullOrUndefined(tabIdx) && actElem.rowIndex < tabRange[tabIdx].selectedRange.endRow + 1 && actElem.rowIndex > tabRange[tabIdx].selectedRange.startRow - 2)
                                if (actElem.colIndex < tabRange[tabIdx].multifilterIdx[tabRange[tabIdx].multifilterIdx.length - 1] + 1 && actElem.colIndex > tabRange[tabIdx].multifilterIdx[0] - 1) {
                                    fltrIcon = xlObj.getCell(tabRange[tabIdx].selectedRange.startRow - 1, actElem.colIndex, sheetIdx).find("span");
                                    if (fltrIcon.length && fltrIcon[0].className.indexOf("e-ssfiltered") > -1) {
                                        menuObj.enableItemByID("ClearFilter");
                                        this._clrFltrEnable = true;
                                    }
                                }
                        }
                        if (!this._clrFltrEnable)
                            menuObj.disableItemByID("ClearFilter");
                    }
                    if (xlObj.model.allowInsert && xlObj.model.scrollSettings.allowScrolling) {
                        rowMenuObj.enableItemByID("Insert");
                        colMenuObj.enableItemByID("Insert");
                        menuObj.enableItemByID("Insert");
                        xlObj.XLRibbon._enableButtons(["inssbtn"], "ejSplitButton");
						xlObj.XLRibbon._enableButtons(["insbtn"], "ejButton");
                    }
                    else if (!xlObj.model.allowInsert || !xlObj.model.scrollSettings.allowScrolling) {
                        rowMenuObj.disableItemByID("Insert");
                        colMenuObj.disableItemByID("Insert");
                        menuObj.disableItemByID("Insert");
                        xlObj.XLRibbon._disableButtons(["inssbtn"], "ejSplitButton");
                        xlObj.XLRibbon._disableButtons(["insbtn"], "ejButton");
                    }
                    if (xlObj.model.allowDelete && xlObj.model.scrollSettings.allowScrolling) {
                        rowMenuObj.enableItemByID("Delete");
                        colMenuObj.enableItemByID("Delete");
                        menuObj.enableItemByID("Delete");
						xlObj.XLRibbon._enableButtons(["delsbtn"], "ejSplitButton");
                        xlObj.XLRibbon._enableButtons(["delbtn"], "ejButton");
                    }
                    else if(!xlObj.model.allowDelete || !xlObj.model.scrollSettings.allowScrolling) {
                        rowMenuObj.disableItemByID("Delete");
                        colMenuObj.disableItemByID("Delete");
                        menuObj.disableItemByID("Delete");
                        xlObj.XLRibbon._disableButtons(["delsbtn"], "ejSplitButton");
                        xlObj.XLRibbon._disableButtons(["delbtn"], "ejButton");
                    }
                    if (xlObj.XLClipboard._copyBackup.cells || xlObj.XLClipboard._copyBackup.elem)
                        this._enableMenuOpt(["Paste"]);
                    else
                        this._disableMenuOpt(["Paste"]);
                    if (xlObj.model.allowSorting)
                        this._enableMenuOpt(["Sort"]);
                    else
                        this._disableMenuOpt(["Sort"]);
                    sortData = !ej.isNullOrUndefined(value) && $.isNumeric(value) ? [xlObj._getLocStr("SortSmallesttoLargest"), xlObj._getLocStr("SortLargesttoSmallest")] : (!ej.isNullOrUndefined(Globalize.parseDate(value)) ? [xlObj._getLocStr("SortOldesttoNewest"), xlObj._getLocStr("SortNewesttoOldest")] : [xlObj._getLocStr("SortAtoZ"), xlObj._getLocStr("SortZtoA")]);
                    //Comments
                    if (xlObj.model.allowComments) {
                        if (menuObj.element.find("#ctxComment").hasClass("e-disable-item"))
                            this._enableMenuOpt(["ctxComment"]);
                        var selectedCells = xlObj.getSheetElement(xlObj.getActiveSheetIndex()).find(".e-selected"), trgtCell = selectedCells.length > 1 ? actCell : $trgt;
                        if (trgtCell.hasClass("e-commentcell")) {
                            this._disableMenuOpt(["ctxInsrtCmnt"]);
                            if (xlObj.XLEdit.getPropertyValueByElem(trgtCell, "comment").isVisible)
                                menuData["13"].text = xlObj._getLocStr("HideComment");
                            else
                                menuData["13"].text = xlObj._getLocStr("ShowHide");
                            this._enableMenuOpt(["ctxEditCmnt", "ctxDeleteCmnt", "ctxShwHdCmnt"]);
                        }
                        else {
                            this._enableMenuOpt(["ctxInsrtCmnt"]);
                            this._disableMenuOpt(["ctxEditCmnt", "ctxDeleteCmnt", "ctxShwHdCmnt"]);
                        }
                        if (selectedCells.filter(trgt).length > 0 && selectedCells.filter(".e-commentcell").length > 0)
                            this._enableMenuOpt(["ctxDeleteCmnt"]);
                    }
                    else
                        this._disableMenuOpt(["ctxComment"]);
                    menuData["7"].text = sortData[0]; //change sort data 
                    menuData["8"].text = sortData[1]; //change sort data
                    if (JSON.stringify(menuData) !== JSON.stringify(args.model.fields.dataSource))
                        menuObj.option("fields", { dataSource: menuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" });
                }
                xlObj.XLRibbon._dirtySelect(xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-selected"));
                   
            }
                if (xlObj.model.allowInsert)
                    footMenuObj.enableItemByID("InsertSheet");
                else
                    footMenuObj.disableItemByID("InsertSheet");
                if (xlObj.model.allowDelete)
                    footMenuObj.enableItemByID("DeleteSheet");
                else
                    footMenuObj.disableItemByID("DeleteSheet");
                if (xlObj.model.allowClipboard)
                    this._enableMenuOpt(["Cut", "Copy"]);
                else
                    this._disableMenuOpt(["Cut", "Copy"]);
                if (xlObj.XLClipboard._copyBackup.cells || xlObj.XLClipboard._copyBackup.elem)
                    this._enableMenuOpt(["Paste"], cmenuId);
                else
                    this._disableMenuOpt(["Paste"], cmenuId);
            }
            else if ($trgt.hasClass("e-ss-imgvisual") || $trgt.hasClass("e-ss-object") || $trgt.closest("div").hasClass("e-ss-object")) {
                var chartMenuData = $.extend(true, [], this._chartMenuData);
                this._isMenuOpened = true;
                chartMenuData = this.XLObj.model.showRibbon ? this._chartMenuData : chartMenuData.splice(0, 2);
                menuData = xlObj.XLShape._shapeType === "chart" ? chartMenuData : this._imgMenuData;
                if (JSON.stringify(menuData) !== JSON.stringify(args.model.fields.dataSource))
                    menuObj.option("fields", { dataSource: menuData, id: "id", parentId: "parentId", text: "text", spriteCssClass: "sprite" });
            }
            else if ($trgt.hasClass("e-link")) {
                dataSource = footMenuObj.model.fields.dataSource.slice(0);
                if (xlObj._isSheetRename && !xlObj._updateSheetNames()) {
                    args.cancel = true;
                    return;
                }
                if (xlObj.model.allowLockCell && actSheet._isSheetProtected)
                    text = xlObj._getLocStr("Unprotect");
                else
                    text = xlObj._getLocStr("Protectsheet");
                dataSource[6].text = text;                 
                footMenuObj.option("fields", { id: "id", dataSource: dataSource, parentId: "parentId" });
                this._isMenuOpened = true;
                if (xlObj._isCommentEdit)
                    xlObj.XLComment._updateCurCmntVal();
                if ($trgt.not("a.e-currentitem, a.e-PP, a.e-NP").hasClass("e-numericitem"))
                    xlObj.gotoPage($trgt.data("index"));
                if (xlObj._getSheetNames(false).length > 0)
                    footMenuObj.enableItemByID("UnhideSheet");
                else
                    footMenuObj.disableItemByID("UnhideSheet");
                if (xlObj._getSheetNames(true).length > 1) {
                    footMenuObj.enableItemByID("DeleteSheet");
                    footMenuObj.enableItemByID("HideSheet");
                    footMenuObj.enableItemByID("MoveorCopy");
                }
                else {
                    footMenuObj.disableItemByID("DeleteSheet");
                    footMenuObj.disableItemByID("HideSheet");
                    footMenuObj.disableItemByID("MoveorCopy");
                }
                if (xlObj.model.allowLockCell)
                    footMenuObj.enableItemByID("ProtectSheet");
                else
                    footMenuObj.disableItemByID("ProtectSheet");
            }
            else if ($trgt.hasClass("e-headercelldiv") || $trgt.hasClass("e-headercell") || $trgt.hasClass("e-rowheader")) {
                var cmenuId = $trgt.hasClass("e-rowheader") ? "contextMenuRowHeader" : "contextMenuColumnHeader";
                if (xlObj.model.allowLockCell && actSheet._isSheetProtected)
                    this._disableMenuOpt(this._prtctHeaderIds, cmenuId);
                else
                    this._enableMenuOpt(this._prtctHeaderIds, cmenuId);
                if (xlObj.model.allowClipboard)
                    this._enableMenuOpt(["Cut", "Copy"], cmenuId);
                else
                    this._disableMenuOpt(["Cut", "Copy"], cmenuId);
                if (xlObj.XLClipboard._copyBackup.cells || xlObj.XLClipboard._copyBackup.elem)
                    this._enableMenuOpt(["Paste"], cmenuId);
                else
                    this._disableMenuOpt(["Paste"], cmenuId);
                xlObj.XLRibbon._dirtySelect(xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-selected"));
            }
        },

        _menuClick: function (args) {
            if (this.XLObj._trigger("contextMenuClick", args))
                return;
            this._processCMenu(args.ID, args.events.element);
        },

        _menuClose: function () {
            this._isMenuOpened = false;
        },

        _processCMenu: function (key, elem) {
            var cid, dataVal, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), actElem, columnName, selCols, selRows;
            switch (key) {
                case "Cut":
                    xlObj.XLClipboard.cut();
                    break;
                case "Copy":
                    xlObj.XLClipboard.copy();
                    break;
                case "PasteSpecial":
                case "PasteValues":
                    xlObj.XLClipboard._isSpecial = !(key === "PasteValues");
                    xlObj.XLClipboard.paste();
                    break;
                case "Insert":
                    xlObj._insDelStatus = "insert";
                    selCols = xlObj._getJSSheetHeader(sheetIdx).find(".e-colselected");
                    selRows = xlObj._getJSSheetRowHeader(sheetIdx).find(".e-rowselected");
                    if (selCols.length)
                        xlObj.insertEntireColumn({ rowIndex: 0, colIndex: selCols[0].cellIndex }, { rowIndex: 0, colIndex: selCols[selCols.length - 1].cellIndex });
                    else if (selRows.length)
                        xlObj.insertEntireRow({ rowIndex: $(selRows[0]).parent().index(), colIndex: 0 }, { rowIndex: $(selRows[selRows.length - 1]).parent().index(), colIndex: 0 });
                    else
                        xlObj._renderInsertDialog();
                    break;
                case "Delete":
                    selCols = $(xlObj._getJSSheetHeader(sheetIdx).find("tr")[0]).find(".e-colselected");
                    selRows = $(xlObj.getRows(sheetIdx)[0]).find(".e-rowselected");
                    if (elem.parentElement.id.indexOf("ColumnHeader") > -1 || selCols.length > 0)
                        xlObj.deleteEntireColumn({ rowIndex: 0, colIndex: selCols[0].cellIndex }, { rowIndex: 0, colIndex: selCols[selCols.length - 1].cellIndex }, []);
                    else if (elem.parentElement.id.indexOf("RowHeader") > -1 || selRows.length > 0)
                        xlObj.deleteEntireRow({ rowIndex: $(selRows[0]).parent().index(), colIndex: 0 }, { rowIndex: $(selRows[selRows.length - 1]).parent().index(), colIndex: 0 }, []);
                    else
                        xlObj._renderDeleteDialog();
                    break;
                case "FilterSelected":
                    xlObj.XLFilter.filterByActiveCell();
                    this._clrFltrEnable = false;
                    break;
                case "ClearFilter":
                    actElem = xlObj.getActiveCell(sheetIdx);
                    var clsname = xlObj.getCell(actElem.rowIndex, actElem.colIndex, sheetIdx)[0].className, tabId = clsname.indexOf("e-table");
                    if (tabId > -1) {
                        columnName = xlObj.model.sheets[sheetIdx].columns[actElem.colIndex].field;
                        xlObj.XLFilter._clearFilterColumn(columnName, "", "", "", "", "", parseInt(clsname[tabId + 7]));
                        delete xlObj._excelFilter._predicates[sheetIdx][columnName];
                    }
                    else
                        xlObj.XLFilter.clearFilter("apply");
                    this._clrFltrEnable = false;
                    break;
                case "SortAtoZ":
                case "SortZtoA":
                case "SortSmallesttoLargest":
                case "SortLargesttoSmallest":
                case "SortOldesttoNewest":
                case "SortNewesttoOldest":
                case "PutCellColor":
                case "PutFontColor":
                    if (xlObj.model.allowSorting)
                        xlObj.XLSort._sortHandler(key);
                    break;
                case "ClearContents":
                    var range = xlObj.model.sheets[sheetIdx].selectedRange;
                    xlObj.clearContents(xlObj._getAlphaRange(sheetIdx, range[0], range[1], range[2], range[3]));
                    break;
                case "InsertSheet":
                    xlObj.insertSheet();
                    break;
                case "DeleteSheet":
                    xlObj.deleteSheet(sheetIdx);
                    break;
                case "RenameSheet":
                    xlObj._renameSheet();
                    break;
                case "MoveorCopy":
                    xlObj._renderMoveCopyDlg();
                    break;
                case "Hide":
                    selCols = xlObj._getJSSheetHeader(sheetIdx).find(".e-colselected");
                    selRows = xlObj._getJSSheetRowHeader(sheetIdx).find(".e-rowselected");
                    if (selCols.length)
                        xlObj._hideColumn(selCols[0].cellIndex, selCols[selCols.length - 1].cellIndex);
                    if (selRows.length)
                        xlObj._hideRow($(selRows[0]).parent().index(), $(selRows[selRows.length - 1]).parent().index());
                    break;
                case "Unhide":
                    selCols = xlObj._getJSSheetHeader(sheetIdx).find(".e-colselected");
                    selRows = xlObj._getJSSheetRowHeader(sheetIdx).find(".e-rowselected");
                    if (selCols.length)
                        xlObj._showColumn(selCols[0].cellIndex, selCols[selCols.length - 1].cellIndex);
                    if (selRows.length)
                        xlObj._showRow($(selRows[0]).parent().index(), $(selRows[selRows.length - 1]).parent().index());
                    break;
                case "HyperLink":
                case "EditLink":
                    xlObj._hlDlgOpen();
                    break;
                case "RemoveLink":
                    xlObj._removeHyperlink();
                    break;
                case "OpenLink":
                    xlObj._openLink();
                    break;
                case "ctxInsrtCmnt":
                    xlObj.XLComment.setComment();
                    break;
                case "ctxEditCmnt":
                    xlObj.XLComment.editComment();
                    break;
                case "ctxDeleteCmnt":
                    xlObj.XLComment.deleteComment();
                    break;
                case "ctxShwHdCmnt":
                    xlObj.XLComment.showHideComment();
                    break;
                case "ChangePicture":
                    xlObj._uploadImage = true;
                    xlObj.XLShape._changePicture = true;
                    xlObj.element.find("#" + xlObj._id + "_file .e-uploadinput").click();
                    break;
                case "ChartType":
                    $("#" + xlObj._id + "_charttypedlg").ejDialog("open");
                    break;
                case "SelectData":
                    cid = document.getElementById(xlObj._id + "_chart").value;
                    dataVal = xlObj.XLChart._getShapeObj(cid, "chart");
                    $("#" + xlObj._id + "_crxaxis").val(dataVal.xRange ? xlObj._getAlphaRange(sheetIdx, dataVal.xRange[0], dataVal.xRange[1], dataVal.xRange[2], dataVal.xRange[3]) : "");
                    $("#" + xlObj._id + "_cryaxis").val(dataVal.yRange ? xlObj._getAlphaRange(sheetIdx, dataVal.yRange[0], dataVal.yRange[1], dataVal.yRange[2], dataVal.yRange[3]) : "");
                    $("#" + xlObj._id + "_crlaxis").val(dataVal.lRange ? xlObj._getAlphaRange(sheetIdx, dataVal.lRange[0], dataVal.lRange[1], dataVal.lRange[2], dataVal.lRange[3]) : "");
                    $("#" + xlObj._id + "_chartrangedlg").ejDialog("open");
                    break;
                case "HideSheet":
                    xlObj.hideSheet();
                    break;
                case "UnhideSheet":
                    xlObj._renderUnhideDlg();
                    break;
                case "ProtectSheet":
                    xlObj.protectSheet(!xlObj.getSheet(sheetIdx)._isSheetProtected);
                    break;
 }
        },

        _disableMenuOpt: function (idCollection, cMenuId) {
            var cName = cMenuId || "contextMenuCell", menuObj = $("#" + this.XLObj._id + "_" + cName).data("ejMenu");
            for (var i = 0, len = idCollection.length; i < len; i++)
                menuObj.disableItemByID(idCollection[i]);
        },

        _enableMenuOpt: function (idCollection, cMenuId) {
            var cName = cMenuId || "contextMenuCell", menuObj = $("#" + this.XLObj._id + "_" + cName).data("ejMenu");
            for (var i = 0, len = idCollection.length; i < len; i++)
                menuObj.enableItemByID(idCollection[i]);
        }
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.sorting = function (obj) {
        this.XLObj = obj;
		this._isSortByFilter = false;
    };

    ej.spreadsheetFeatures.sorting.prototype = {
        _sortHandler: function (args) {
            var i, ctype1, ctype2, val1, val2, direction, tableId, xlObj = this.XLObj, columnName = "", range = {}, regx = xlObj._formatRegx, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), rIdx, cIdx;
            columnName = sheet.columns[sheet._activeCell.colIndex].field;
			this._isSortByFilter = false;
            if (args != "PutCellColor" && args != "PutFontColor") {
                if (!xlObj.model.allowSorting || sheet._selectedCells.length == 0)
                    return;
                direction = args === "SortZtoA" ? ej.sortOrder.Descending : ej.sortOrder.Ascending;
                tableId = xlObj._getTableID(xlObj.getActiveCellElem()[0].className);
                if (tableId) { // for FAT
                    range = sheet._tableManager[tableId].range.slice(0);
                    range[0]++;
                } else {
                    range = sheet.selectedRange.slice(0), rIdx = range[0], cIdx = range[1];
                    if (rIdx === range[2])
                        return;
                    val1 = xlObj.XLEdit.getPropertyValue(rIdx, cIdx);
                    for (i = rIdx + 1; i <= range[2]; i++) {
                        if (!xlObj._isHiddenRow(i)) {
                            val2 = xlObj.XLEdit.getPropertyValue(i, cIdx);
                            break;
                        }
                    }
                    ctype1 = xlObj.isNumber(val1) ? ej.Spreadsheet.CellType.Number : ej.Spreadsheet.CellType.String;
                    ctype2 = xlObj.isNumber(val2) ? ej.Spreadsheet.CellType.Number : ej.Spreadsheet.CellType.String;
                    if (ctype1 != ctype2 || $(this.XLObj.getCell(range[0], range[1]))[0].className.indexOf("e-sswraptext") > -1)
                        range[0]++;
                }
                this.sortByRange(range, columnName, direction);
            } else {
                var alphRange, format, currCell = sheet._activeCell, color, cellIdx, actSheet = xlObj.getSheet(sheetIdx), startCell = actSheet._startCell, endCell = actSheet._endCell;                
                if(format)
                    color = xlObj._getColorFromFormat(format);
                alphRange = xlObj._getAlphaRange(sheetIdx, startCell.rowIndex, startCell.colIndex, endCell.rowIndex, endCell.colIndex);
                this.sortByColor(args, color, alphRange);
            }
        },

        sortByColor: function (args, color, range) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowSorting)
                return;
            var sheetIdx = xlObj.getActiveSheetIndex(), sortObj = {}, startRowIndex, endRowIndex, sheet = xlObj.getSheet(sheetIdx), columnName;
            range = xlObj.getRangeIndices(range);
            columnName = sheet.columns[range[1]].field;
            if (!ej.isNullOrUndefined(color)) {
                color = (color.length === 2 && args === "PutCellColor") ? color[0] : color[1];
                if (args === "PutCellColor") {
                    sortObj.operation = "sortbgcolor";
                    sortObj.bgcolor = color;
                } else {
                    sortObj.operation = "sortfgcolor";
                    sortObj.fgcolor = color;
                }
                sortObj.requestType = sortObj.action = "sortbycolor";
                sortObj.sortColumn = columnName;
                sortObj.selRange = { startCell: { "rowIndex": range[0], "colIndex": range[1] }, endCell: { "rowIndex": range[2], "colIndex": range[3] } };
                sortObj.rangeData = xlObj.getRangeDataAsObject(sortObj.selRange.startCell, sortObj.selRange.endCell, true)[0];
                startRowIndex = range[0]; //this._getSortColorRange({ "rowIndex": range[0], "colIndex": range[1] });
                if (!ej.isNullOrUndefined(startRowIndex)) {
                    endRowIndex = xlObj.XLFilter._createSelection(sheetIdx, startRowIndex, range[0], range[1], []).endRowIndex;
                    startRowIndex = (args === "PutCellColor" || args === "PutFontColor") ? startRowIndex + 1 : startRowIndex;
                    sortObj.range = xlObj.model.sheets[sheetIdx]._selectedRange = { startRow: startRowIndex, endRow: endRowIndex, dataSourceIndexes: [] };
                    xlObj._processBindings(sortObj);
                }
            }
        },

        _getSortColorRange: function (activeCell) {
            var xlObj = this.XLObj, actElem = xlObj.getCell(activeCell.rowIndex, activeCell.colIndex), addSort = false, k, j;
            var startRowIndex = 0, prevText = false, rowIndex = activeCell.rowIndex, colIndex = activeCell.colIndex;
            if (actElem.text().length < 1) {
                //Active Element is Empty in the Top
                for (j = rowIndex; j < (rowIndex + 2) ; j++) {
                    actElem = xlObj.getCell(j, colIndex);
                    if (actElem.is(":visible") && actElem.text().length > 0) {
                        addSort = true;
                        startRowIndex = j;
                        break;
                    }
                }

                //Active Element is Empty in the Bottom
                if (!addSort) {
                    for (j = rowIndex; j > (rowIndex - 2); j--) {
						if (j > -1) {
							actElem = xlObj.getCell(j, colIndex);
							if (actElem.text().length === 0) {
								if (j > 0) {
									actElem = xlObj.getCell(j - 1, colIndex);
									actElem.text().length == 0 && (prevText = true);
								}
							}
						}
                    }
                    if (!prevText) {
                        for (j = rowIndex - 2; j > 0; j--) {
							actElem = xlObj.getCell(j, colIndex);
							if (actElem.is(":visible") && actElem.text().length === 0) {
								k = j + 1;
								addSort = true;
								startRowIndex = k;
								break;
							}
                        }
                    }
                }
            } else {
                //Active Element is Middle of the Column
                for (j = rowIndex; j >= 0; j--) {
                    actElem = xlObj.getCell(j, colIndex);
                    if (actElem.is(":visible") && actElem.text().length === 0) {
                        k = j + 1;
                        addSort = true;
                        startRowIndex = k;
                        break;
                    }
                    if (!addSort && j == 0) {
                        startRowIndex = j;
                        addSort = true;
                    }
                }
            }
            if (!addSort) {
                //display an alert message to select the cell which is having a value
                xlObj._showAlertDlg("Alert", "Alert", null, 630);
            } else
                return startRowIndex;
        },

        sortByRange: function (range, columnName, direction) {
            var xlObj = this.XLObj;
            if (!xlObj.model.allowSorting)
                return;
            var len, sheetIdx = xlObj.getActiveSheetIndex(), i = 0, rangeData = [], args = {}, filterIcon = [], filterIcons, alpRange,
				colSelected = xlObj._getJSSheetHeader(sheetIdx).find(".e-colselected").length, fltrRng;
            if (!xlObj.model.allowSorting)
                return;
			if (typeof (range) === "string")
                range = xlObj.getRangeIndices(range);
			if(this._isSortByFilter && colSelected < 1) {
				fltrRng = this._isSortByFilter ? range[0] - 1 : range[0];
				alpRange = xlObj._getAlphaRange(sheetIdx, fltrRng, range[1], fltrRng + 1, range[1]);		
				filterIcons = xlObj._getAlphaRangeCells(alpRange).find(".e-filterspan");
				len = filterIcons.length;
				while (i < len) {
					filterIcon.push({ filterIcon: filterIcons.eq(i), colIndex: filterIcons.eq(i).parent().index(), rowIndex: filterIcons.eq(i).parents("tr").index() });
					i++;
				}
				if(filterIcons.length)
					range[0] = this._isSortByFilter ? range[0] : range[0] + 1;
			}
			rangeData = xlObj.getRangeDataAsObject({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] }, true)[0];
            args.requestType = ej.Spreadsheet.Actions.Sorting;
            args.range = range;
            args.columnName = columnName;
            args.rangeData = rangeData;
            args.action = "sorting";
            args.sortDirection = direction;
            args.filterIcon = filterIcon;
			args.isSortByFilter = this._isSortByFilter;
			args.colSelected = colSelected;
            xlObj._processBindings(args);
			if(colSelected && this._isSortByFilter)
				xlObj.XLFilter._changeSortIcon({rowIndex: range[0] - 1, colIndex: range[1]}, direction);			
        },

        _sortComplete: function (args) {
            var xlObj = this.XLObj, range = args.range, data = this._swapSortedDatas(args), swappedData = data[1];
            xlObj._isSort = true;
			args.rangeData = data[0];
            xlObj.XLComment.deleteComment(range, null, true);
			 xlObj._isSort = true;
            xlObj._removeHyperlink(null, null, true);
			 xlObj._isSort = false;
            xlObj.clearRangeData(range, ["value", "value2", "type", "formatStr", "decimalPlaces", "thousandSeparator", "comment", "hyperlink", "format"], null, true);
            xlObj.XLFormat.clearFormat(range, "format", true);
            xlObj.XLEdit.updateCellWithContainer(swappedData, range, null, true);
            for (var i = 0, n = args.filterIcon.length; i < n; i++)
                xlObj.XLFilter._applyFilterIcon(xlObj.getCell(args.filterIcon[i].rowIndex, args.filterIcon[i].colIndex), args.filterIcon[i].filterIcon);
            xlObj._isSort = false;
        },

        _swapSortedDatas: function (args) {
            var text, idx, oIdx, j, i = 0, oldInst = [], total =[], arr = [], viewData = args.model.currentViewData, vlen = viewData.length, colIdx = this.XLObj._generateColCount(args.columnName) - 1, olen,
                range = args.range, tCols = (range[3] - range[1]) + 1, ccIdx = colIdx - range[1],
                oldData = this.XLObj.getRangeData(range, false, ["value", "value2", "type", "thousandSeparator", "range", "format", "comment", "hyperlink"], null, false, false, true);
            $.extend(true, oldInst, oldData);
			total.push(oldInst);
			while (i < vlen) {
                text = viewData[i][args.columnName];
                j = ccIdx;
                olen = oldData.length;
                while (j < olen) {
                    if (oldData[j].value === text) {
                        idx = this._getMinIdx(j, tCols);
						arr = arr.concat(oldData.splice(idx, tCols));
                        break;
                    }
                    j += tCols;
                }
                i++;
            }
			total.push(arr);
            return total;
        },

        _getMinIdx: function (ccIdx, tCols) {
            var i, j;
            i = j = tCols;
            if (ccIdx < tCols)
                return 0;
            while (i <= ccIdx) {
                j += tCols;
                if (j > ccIdx)
                    return i;
                i += tCols;
            }
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.dragAndDrop = function (Obj) {
        this.XLObj = Obj;
        this._isDragAndDropStart = false;
        this._dragAndDropCell = { rowIndex: 0, colIndex: 0 };
        this._isDragAndDropped = false;
        this._allowDragAndDrop = false;
    };

    ej.spreadsheetFeatures.dragAndDrop.prototype = {
        moveRangeTo: function (sourcerange, destinationrange) { // sourcerange & destinationrange as array 
            var xlObj = this.XLObj;
            if (!xlObj.model.allowDragAndDrop)
                return;
            var sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            sourcerange = xlObj.swapRange(sourcerange);
            destinationrange = xlObj.swapRange(destinationrange);
            var dStartCell = { rowIndex: destinationrange[0], colIndex: destinationrange[1] }, dEndCell = { rowIndex: destinationrange[2], colIndex: destinationrange[3] };
            var data = xlObj.getRangeData(sourcerange), format = xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(sourcerange),
            details = { sheetIndex: sheetIdx, reqType: "reorder-cells", sourceData: $.extend(true, [], data), sourceRange: sourcerange, destinationRange: destinationrange, sourceFormat: format, destinationFormat: xlObj.model.allowCellFormatting && xlObj.XLFormat.getHashCodeClassAsArray(destinationrange), destinationData: xlObj.getRangeData(destinationrange) };
            var tabDetails = xlObj.model.allowFiltering && xlObj.XLFilter._checkFilteredCells(sheetIdx);
            xlObj.model.allowComments && xlObj.XLComment.deleteComment();
            if (tabDetails.isApplied)
                xlObj.XLFilter._moveFilteredCells(sheetIdx, tabDetails);
            else {
                xlObj.clearRangeData(sourcerange);
                xlObj.clearRangeData(destinationrange);
                xlObj.model.allowCellFormatting && xlObj.XLFormat.clearFormat(sourcerange);
                xlObj.updateData(data, destinationrange);
                xlObj.model.allowCellFormatting && xlObj.XLFormat.updateFormat(format, destinationrange);
            }
            // selection
            xlObj.setActiveCell(dStartCell.rowIndex, dStartCell.colIndex);
            if (dStartCell.rowIndex === 0 && dEndCell.rowIndex === sheet.rowCount - 1)
                xlObj.XLSelection.selectColumns(dStartCell.colIndex, dEndCell.colIndex);
            else if (dStartCell.colIndex === 0 && dEndCell.colIndex === sheet.colCount - 1)
                xlObj.XLSelection.selectRows(dStartCell.rowIndex, dEndCell.rowIndex);
            else
                xlObj.XLSelection.selectRange(dStartCell, dEndCell);
            if (!xlObj._isUndoRedo)
                if (!xlObj._isFAT)
                    xlObj._completeAction(details);
        },

        _dMouseHover: function (e) {
            var xlObj = this.XLObj, isLeft, isTop, isRight, isBottom, post, xy, x, y, target = e.target, cname = target.className;
            if (cname.indexOf('e-ddcell') > -1) {
                post = e.target.getBoundingClientRect();
                xy = xlObj._setXY(e);
                x = xy[0];
                y = xy[1];
                isLeft = x - xlObj.getFocusLeftElem().offset().left;
                isLeft = isLeft >= 0 && isLeft <= 5;
                isTop = y - xlObj.getFocusTopElem().offset().top;
                isTop = isTop >= 0 && isTop <= 5;
                isRight = xlObj.getFocusRightElem().offset().left - x;
                isRight = isRight >= 0 && isRight <= 5;
                isBottom = xlObj.getFocusBottomElem().offset().top - y;
                isBottom = isBottom >= 0 && isBottom <= 5;
                if ((isLeft && x <= post.left + 5 && x >= post.left)/*left*/ || (isTop && y <= post.top + 5 && y >= post.top)/*top*/ || (isRight && x >= post.right - 5 && x <= post.right)/*right*/ || (isBottom && y >= post.bottom - 5 && y <= post.bottom)/*bottom*/) {
                    xlObj.addClass(target, 'e-cursormove');
                    this._isDragAndDropStart = true;
                    return;
                }
            }
            if (this._isDragAndDropStart) {
                xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-cursormove").removeClass("e-cursormove");
                this._isDragAndDropStart = false;
            }
        },

        _dMouseDown: function (e) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            e.preventDefault();
            e.stopImmediatePropagation();
            var range = xlObj.swapRange([sheet._startCell.rowIndex, sheet._startCell.colIndex, sheet._endCell.rowIndex, sheet._endCell.colIndex]);
            this._dragAndDropCell = { rowIndex: e.target.parentNode.rowIndex, colIndex: e.target.cellIndex };
            xlObj._dStartCell = { rowIndex: range[0], colIndex: range[1] };
            xlObj._dEndCell = { rowIndex: range[2], colIndex: range[3] };
            if (xlObj._trigger('dragStart', { sheetIndex: sheetIdx, target: e.target, currentCell: this._dragAndDropCell, dragAndDropRange: { startCell: sheet._startCell, endCell: sheet._endCell } }))
                return;
            this._allowDragAndDrop = true;
        },

        _dMouseMove: function (e) {
            if (this._allowDragAndDrop) {
                var index = { rowIndex: e.target.parentNode.rowIndex, colIndex: e.target.cellIndex }, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
                if (xlObj._trigger("drag", { sheetIndex: sheetIdx, target: e.target, currentCell: index, dragAndDropRange: { startCell: sheet._startCell, endCell: sheet._endCell } }))
                    return;
                xlObj.XLSelection._focusBorder(sheet._startCell, sheet._endCell, xlObj._thinBorder);
                this._updateDCells(index);
                xlObj.XLSelection._focusRange(xlObj._dStartCell, xlObj._dEndCell, xlObj._autofillBorder);
            }
            else
                this._dMouseHover(e);
        },

        _dMouseUp: function (e) {
            if (this._isDragAndDropped) {
                var cells, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), canreplace = true, preventDrop = false, range = [xlObj._dStartCell.rowIndex, xlObj._dStartCell.colIndex, xlObj._dEndCell.rowIndex, xlObj._dEndCell.colIndex],
                args = { sheetIndex: sheetIdx, preventAlert: false, target: e.target, currentCell: this._dragAndDropCell, dragAndDropRange: { startCell: sheet._startCell, endCell: sheet._endCell } }, thinBorder = xlObj._arrayAsString(xlObj._thinBorder);
                if (xlObj._isUniqueRange(sheet.selectedRange, xlObj.swapRange(range)))
                    this._preventDrop(sheet);
                else {
                    if (xlObj.model.allowLockCell && sheet._isSheetProtected) {
                        cells = xlObj.XLSelection._getSelectedRange({ rowIndex: range[0], colIndex: range[1] }, { rowIndex: range[2], colIndex: range[3] });
                        cells = cells.concat(sheet._selectedCells);
                        if (xlObj._israngeProtected(cells)) {
                            this._preventDrop(sheet);
                            xlObj.XLSelection._clearBorder(thinBorder);
                            this._allowDragAndDrop = this._isDragAndDropStart = this._isDragAndDropped = false;
                            xlObj._popUplockCellAlert();
                            return;

                        }
                    }
                    preventDrop = xlObj._trigger("beforeDrop", args);
                    if (!args.preventAlert && !preventDrop && xlObj._rangeHasData(range))
                        canreplace = confirm(xlObj._getLocStr("DropAlert")); // has to be modeified when confirm dialog available	                
                    if (!preventDrop && canreplace) {
                        this.moveRangeTo([sheet._startCell.rowIndex, sheet._startCell.colIndex, sheet._endCell.rowIndex, sheet._endCell.colIndex], [xlObj._dStartCell.rowIndex, xlObj._dStartCell.colIndex, xlObj._dEndCell.rowIndex, xlObj._dEndCell.colIndex]);
                        sheet._startCell = xlObj._dStartCell;
                        sheet._endCell = xlObj._dEndCell;
                    }
                    else
                        this._preventDrop(sheet);
                    if (!preventDrop && canreplace)
                        xlObj._trigger("drop", { sheetIndex: sheetIdx, target: e.target, currentCell: this._dragAndDropCell, dragAndDropRange: { startCell: sheet._startCell, endCell: sheet._endCell } });
                }
                xlObj.XLSelection._clearBorder(thinBorder);
                if(xlObj.model.allowAutoFill)
					xlObj.XLDragFill.positionAutoFillElement();
            }
            this._allowDragAndDrop = this._isDragAndDropStart = this._isDragAndDropped = false;
        },

        _preventDrop: function (sheet) {
            var xlObj = this.XLObj;
            xlObj.XLSelection._focusRange(sheet._startCell, sheet._endCell, xlObj._selectionBorder);
            xlObj._dStartCell = sheet._startCell;
            xlObj._dEndCell = sheet._endCell;
        },

        _updateDCells: function (index) {
            var xlObj = this.XLObj, sheetIdx, sheet, sheadercont, srowcontent, slen;
            sheetIdx = xlObj.getActiveSheetIndex();
            sheet = xlObj.getSheet(sheetIdx);
            slen = sheet._selectedCells.length;
            sheadercont = xlObj._getJSSheetHeader(sheetIdx).find(".e-headercontent th");
            srowcontent = xlObj._getJSSheetRowHeaderContent(sheetIdx).find("td");
            if (sheadercont[sheet._startCell.colIndex].className.indexOf("e-colselected") === -1) {
                if (index.rowIndex < this._dragAndDropCell.rowIndex && xlObj._dStartCell.rowIndex) {
                    xlObj._dStartCell.rowIndex = xlObj._dStartCell.rowIndex - (this._dragAndDropCell.rowIndex - index.rowIndex);
                    xlObj._dEndCell.rowIndex = slen === 1 ? xlObj._dStartCell.rowIndex : xlObj._dEndCell.rowIndex - (this._dragAndDropCell.rowIndex - index.rowIndex);
                }
                else if (index.rowIndex > this._dragAndDropCell.rowIndex) {
                    xlObj._dStartCell.rowIndex = xlObj._dStartCell.rowIndex + (index.rowIndex - this._dragAndDropCell.rowIndex);
                    xlObj._dEndCell.rowIndex = slen === 1 ? xlObj._dStartCell.rowIndex : xlObj._dEndCell.rowIndex + (index.rowIndex - this._dragAndDropCell.rowIndex);
                }
            }
            if (srowcontent[sheet._startCell.rowIndex].className.indexOf("e-rowselected") === -1) {
                if (index.colIndex < this._dragAndDropCell.colIndex && xlObj._dStartCell.colIndex) {
                    xlObj._dStartCell.colIndex = xlObj._dStartCell.colIndex - (this._dragAndDropCell.colIndex - index.colIndex);
                    xlObj._dEndCell.colIndex = slen === 1 ? xlObj._dStartCell.colIndex : xlObj._dEndCell.colIndex - (this._dragAndDropCell.colIndex - index.colIndex);
                }
                else if (index.colIndex > this._dragAndDropCell.colIndex) {
                    xlObj._dStartCell.colIndex = xlObj._dStartCell.colIndex + (index.colIndex - this._dragAndDropCell.colIndex);
                    xlObj._dEndCell.colIndex = slen === 1 ? xlObj._dStartCell.colIndex : xlObj._dEndCell.colIndex + (index.colIndex - this._dragAndDropCell.colIndex);
                }
            }
            this._dragAndDropCell = index;
            this._isDragAndDropped = true;
        },

        _initDragAndDropRange: function (range) {
            var i = 0, cname = "e-ddcell", xlObj = this.XLObj;
            range = xlObj.swapRange(range);
            var cells = xlObj.getRange(range[0], range[1], range[2], range[3]), rowcount = (range[2] - range[0]) + 1, colcount = (range[3] - range[1]) + 1;
            while (i < colcount) {
                xlObj.addClass(cells[i], cname);
                i++;
            }
            i = colcount - 1;
            while (i < rowcount * colcount) {
                xlObj.addClass(cells[i], cname);
                i += colcount;
            }
            i = (rowcount - 1) * colcount;
            while (i < rowcount * colcount) {
                xlObj.addClass(cells[i], cname);
                i++;
            }
            i = 0;
            while (i < rowcount * colcount) {
                xlObj.addClass(cells[i], cname);
                i = i + colcount;
            }
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.exporting = function (obj) {
        this.XLObj = obj;
    };

    ej.spreadsheetFeatures.exporting.prototype = {
        //Exporting        
        "export": function (type) {
            var xlObj = this.XLObj;
            if (!xlObj.model.exportSettings.allowExporting)
                return;
            var model = {}, expSettings = xlObj.model.exportSettings, attr, form, inputAttr, input, url, contClone;
            $.extend(true, model, xlObj.model);
            var i, j, k, len, slen, chartObj, modelClone, series;
            url = type === ej.Spreadsheet.exportType.Excel ? expSettings.excelUrl : expSettings.csvUrl;
			 if(ej.isNullOrUndefined(url)){
				 xlObj._showAlertDlg("Alert", "ImportExportUrl");
				 return;	
			 }
            for (i = 1; i < model.sheets.length; i++) {
                model.sheets[i].chart = [];
                $.extend(true, model.sheets[i].chart, model.sheets[i]._shapeMngr.chart);
                model.sheets[i]._shapeMngr.chart = [];
                for (j = 0, len = model.sheets[i].chart.length; j < len; j++) {
                    chartObj = $("#" + model.sheets[i].chart[j].id).data("ejChart");
                    if (chartObj) {
                        modelClone = $.extend(true, {}, chartObj.model);
                        series = modelClone.series;
                        for (k = 0, slen = series.length; k < slen; k++) {
                            delete series[k].dataSource;
                            delete series[k].query;
                            series[k].fill = jQuery.type(series[k].fill) == "array" ? series[k].fill[0].color : series[k].fill;
                        }
                        model.sheets[i]._shapeMngr.chart.push(modelClone);
                    }
                }
            }
            model.sheets.shift();
            if (!expSettings.allowExporting || xlObj.isUndefined(url))
                return;
            contClone = $.extend(true, {}, xlObj._dataContainer);
            xlObj._refreshHashCode(contClone.hashCode);
            attr = { action: url, method: "post" };
            form = ej.buildTag("form", "", null, attr);
            inputAttr = { name: "sheetData", type: "hidden", value: JSON.stringify(contClone) };
            input = ej.buildTag("input", "", null, inputAttr);
            form.append(input);
            inputAttr = { name: "sheetModel", type: "hidden", value: JSON.stringify(this._dropElements(model)) };
            input = ej.buildTag("input", "", null, inputAttr);
            form.append(input);
			inputAttr = { name: "Password", type: "hidden", value: this._encrypt(expSettings.password) };
            input = ej.buildTag("input", "", null, inputAttr);
            form.append(input);
            $("body").append(form);
            form.submit();
        },
        _encrypt: function (str) {
            if (!ej.isNullOrUndefined(str)) {
                var i, xorKey = 129, result = "";
                for (i = 0; i < str.length; ++i)
                    result += String.fromCharCode(xorKey ^ str.charCodeAt(i));
                return result;
            }
        },

        _dropElements: function (model) {
            var actSheet = model.sheets, arr = ["_spreadSheetPager", "datasource"], i, j, content;        
            model.undoCollection = [];
            while (i in actSheet) {
                content = actSheet[i];
                while (j in content) {
                    if (arr.indexOf(j) > -1)
                        content[j] = null;
                }
            }
            return model;
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.findnreplace = function (obj) {
        this.XLObj = obj;
        this._findCount = 0;
        this.caseSen = {
            "true": function (a) { return a.toString(); },
            "false": function (a) { return a.toString().toLowerCase(); }
        };
        this.isEMatch = {
            "true": function (a, b) { return a == b; },
            "false": function (a, b) { return a.indexOf(b) > -1; }
        };
    };

    ej.spreadsheetFeatures.findnreplace.prototype = {
        //Find and Replace       

        replaceAllBySheet: function (findData, replaceData, isCSen, isEMatch) {
            if (!this.XLObj.model.allowSearching)
                return;
            this._replaceAllData(findData, replaceData, isCSen, isEMatch, "value", "sheet");
        },

        replaceAllByBook: function (findData, replaceData, isCSen, isEMatch) {
            if (!this.XLObj.model.allowSearching)
                return;
            this._replaceAllData(findData, replaceData, isCSen, isEMatch, "value", "workbook");
        },

        _findNext: function (input, sIndex, isCSen, isEMatch, type, mode, searchBy) {
            var xlObj = this.XLObj, args = {
                sheetIndex: sIndex,
                value: input,
                isCaseSensitive: isCSen,
                isEntireMatch: isEMatch,
                searchBy: searchBy,
                mode: mode,
                findType: type,
                reqType: "findNext"
            };
            if (xlObj._trigActionBegin(args))
                return true;
            input = args.value;
            sIndex = args.sheetIndex;
            isCSen = args.isCaseSensitive;
            isEMatch = args.isEntireMatch;
            type = args.findType; mode = args.mode;
            searchBy = args.searchBy;
            var sheetIdx = sIndex ? sIndex : xlObj.model.activeSheetIndex, actSheet = xlObj.getSheet(sheetIdx), rowIdx = actSheet._activeCell.rowIndex, colIdx = actSheet._activeCell.colIndex, usedRange = actSheet._usedRange, rowCount = usedRange.rowIndex + 1, colCount = usedRange.colIndex + 1;
            var matched, i, j, s, tmpSIndex, isFinded, searchRows = searchBy === "rows", tmp, k;
            if (!searchRows) {
                tmp = rowIdx;
                rowIdx = colIdx;
                colIdx = tmp;
                tmp = rowCount;
                rowCount = colCount;
                colCount = tmp;
            }
            if (mode === "sheet") {
                for (i = rowIdx; i < rowCount; i++) {
                    if (i !== rowIdx)
                        colIdx = 0;
                    for (j = colIdx; j < colCount; j++) {
                        matched = this._compareValues(searchRows ? i : j, searchRows ? j : i, sheetIdx, input, isCSen, isEMatch, type);
                        if (matched) {
                            if (!(i === rowIdx && j === colIdx)) {
                                this._updateSelection(searchRows ? i : j, searchRows ? j : i);
                                this._findCount++;
                                args = { reqType: "findNext", activeCell: xlObj.getActiveCellElem() };
                                xlObj._trigActionComplete(args);
                                return true;
                            }
                        }
                    }
                }
            }
            else {
                for (s = sheetIdx, k = xlObj.model.sheetCount; s <= k; s++) {
                    usedRange = xlObj.model.sheets[s]._usedRange;
                    if (s !== sheetIdx) {
                        colIdx = 0;
                        rowIdx = 0;
                        rowCount = searchRows ? usedRange.rowIndex : usedRange.colIndex;
                        colCount = searchRows ? usedRange.colIndex : usedRange.rowIndex;
                    }
                    for (i = rowIdx; i < rowCount + 1; i++) {
                        if (i !== rowIdx)
                            colIdx = 0;
                        for (j = colIdx; j < colCount + 1; j++) {
                            matched = this._compareValues(searchRows ? i : j, searchRows ? j : i, s, input, isCSen, isEMatch, type);
                            if (matched) {
                                if (!(s === sheetIdx && i === rowIdx && j === colIdx)) {
                                    this._updateSelection(searchRows ? i : j, searchRows ? j : i, s);
                                    this._findCount++;
                                    args = { reqType: "findNext", activeCell: xlObj.getActiveCellElem() };
                                    xlObj._trigActionComplete(args);
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
            if (!this._findCount) {
                tmpSIndex = (mode === "sheet") ? sheetIdx : 1;
                this._updateSelection(0, 0, tmpSIndex);
                this._findCount++;
                if (this._compareValues(0, 0, tmpSIndex, input, isCSen, isEMatch, type))
                    return true;
                else {
                    isFinded = this._findNext(input, tmpSIndex, isCSen, isEMatch, type, mode, searchBy);
                    return isFinded;
                }
            }
            return false;
        },

        _findPrev: function (input, sIndex, isCSen, isEMatch, type, mode, searchBy) {
            var xlObj = this.XLObj, args = {
                sheetIndex: sIndex,
                value: input,
                isCaseSensitive: isCSen,
                isEntireMatch: isEMatch,
                searchBy: searchBy,
                mode: mode,
                findType: type,
                reqType: "findPrevious"
            };
            if (xlObj._trigActionBegin(args))
                return true;
            input = args.value;
            sIndex = args.sheetIndex;
            isCSen = args.isCaseSensitive;
            isEMatch = args.isEntireMatch;
            type = args.findType;
            mode = args.mode;
            searchBy = args.searchBy;
            var sheetIdx = sIndex ? sIndex : xlObj.model.activeSheetIndex, actSheet = xlObj.getSheet(sheetIdx), rowIdx = actSheet._activeCell.rowIndex, colIdx = actSheet._activeCell.colIndex, usedRange = actSheet._usedRange, rowCount = usedRange.rowIndex + 1, colCount = usedRange.colIndex + 1;
            var matched, i, j, s, tmpSIndex, isFinded, searchRows = searchBy === "rows", tmp;
            if (!searchRows) {
                tmp = rowIdx;
                rowIdx = colIdx;
                colIdx = tmp;
                tmp = rowCount;
                rowCount = colCount;
                colCount = tmp;
            }
            if (mode === "sheet") {
                for (i = rowIdx; i >= 0; i--) {
                    if (i !== rowIdx)
                        colIdx = colCount;
                    for (j = colIdx; j >= 0; j--) {
                        matched = this._compareValues(searchRows ? i : j, searchRows ? j : i, sheetIdx, input, isCSen, isEMatch, type);
                        if (matched) {
                            if (!(i === rowIdx && j === colIdx)) {
                                this._updateSelection(searchRows ? i : j, searchRows ? j : i);
                                this._findCount++;
                                args = { reqType: "findPrevious", activeCell: xlObj.getActiveCellElem() };
                                xlObj._trigActionComplete(args);
                                return true;
                            }
                        }
                    }
                }
            }
            else {
                for (s = sheetIdx; s >= 1; s--) {
                    usedRange = xlObj.model.sheets[s]._usedRange;
                    if (s !== sheetIdx) {
                        rowCount = searchRows ? usedRange.rowIndex : usedRange.colIndex;
                        colCount = searchRows ? usedRange.colIndex : usedRange.rowIndex;
                        colIdx = colCount;
                        rowIdx = rowCount;
                    }
                    for (i = rowIdx; i >= 0; i--) {
                        if (i !== rowIdx)
                            colIdx = colCount + 1;
                        for (j = colIdx; j >= 0; j--) {
                            matched = this._compareValues(searchRows ? i : j, searchRows ? j : i, s, input, isCSen, isEMatch, type);
                            if (matched) {
                                if (!(s === sheetIdx && i === rowIdx && j === colIdx)) {
                                    this._updateSelection(searchRows ? i : j, searchRows ? j : i, s);
                                    this._findCount++;
                                    args = { reqType: "findPrevious", activeCell: xlObj.getActiveCellElem() };
                                    xlObj._trigActionComplete(args);
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
            if (!this._findCount) {
                tmpSIndex = (mode === "sheet") ? sheetIdx : xlObj.model.sheetCount;
                rowCount = xlObj.model.sheets[tmpSIndex]._usedRange.rowIndex;
                colCount = xlObj.model.sheets[tmpSIndex]._usedRange.colIndex;
                this._updateSelection(rowCount, colCount, tmpSIndex);
                this._findCount++;
                if (this._compareValues(rowCount, colCount, tmpSIndex, input, isCSen, isEMatch, type))
                    return true;
                else {
                    isFinded = this._findPrev(input, tmpSIndex, isCSen, isEMatch, type, mode, searchBy);
                    return isFinded;
                }
            }
            return false;
        },

        _compareValues: function (rowIdx, colIdx, sheetIdx, input, isCSen, isEMatch, type) {
            var xlObj = this.XLObj, value = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, null, sheetIdx) ? xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, null, sheetIdx).toString() : "";
            var isFormula = type === "formula" && !ej.isNullOrUndefined(value) && value.charAt(0) === "=" && !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2", sheetIdx)) && typeof xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2", sheetIdx) === "string";
            if (type === "value")
                value = value && !isFormula ? this.caseSen[isCSen](value) : "";
            else if (type === "formula") {
                value = isFormula ? this.caseSen[isCSen](value) : "";
            }
            else
                value = xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx) ? this.caseSen[isCSen](xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "comment", sheetIdx).value) : "";
            input = this.caseSen[isCSen](input);
            return (this.isEMatch[isEMatch](value, input));
        },

        _replace: function (cell, sIndex, input1, input2, isCSen) {
            var xlObj = this.XLObj, cmnt, cellElem, sheetIdx = xlObj.getActiveSheetIndex(), value = xlObj.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, null, sIndex) ? xlObj.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, null, sIndex).toString() : "", regx = new RegExp(input1.toString().replace(/[.*+?^${}()|[\]\\]/g, "\\$&"), "ig");
            var newData = (value !== "" && input1 !== "") ? (isCSen ? value.replace(input1, input2) : value.replace(regx, input2)) : input2;
            if (!(sIndex === sheetIdx))
                xlObj.gotoPage(sIndex);
            xlObj.XLEdit.updateCellValue(cell, newData);
            cmnt = xlObj.XLEdit.getPropertyValue(cell.rowIndex, cell.colIndex, "comment");
            if (!ej.isNullOrUndefined(cmnt)) {
                if (xlObj._isRowViewable(sheetIdx, cell.rowIndex)) {
                    cellElem = xlObj.getCell(cell.rowIndex, cell.colIndex);
                    cellElem.removeClass("e-commentcell");
                }
                xlObj.XLComment.setComment([cell.rowIndex, cell.colIndex, cell.rowIndex, cell.colIndex], null, false);
            }
        },

        _replaceData: function (input1, input2, sIndex, isCSen, isEMatch, type, direction, mode, searchBy) {
            var details, cell, xlObj = this.XLObj, sheetIdx = sIndex ? sIndex : xlObj.model.activeSheetIndex, actSheet = xlObj.getSheet(sheetIdx), rowIdx = actSheet._activeCell.rowIndex, colIdx = actSheet._activeCell.colIndex, isFinded;
            if (xlObj.model.allowLockCell && actSheet._isSheetProtected) {
                xlObj._popUplockCellAlert();
                return;
            }
            if (this._compareValues(rowIdx, colIdx, sheetIdx, input1, isCSen, isEMatch, type)) {
                details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "replace", rowIndex: rowIdx, colIndex: colIdx, sIndex: sIndex };
                details.prevData = { value: xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2") };
                this._replace({ rowIndex: rowIdx, colIndex: colIdx }, sIndex, input1, input2, isCSen);
                details.newData = { value: xlObj.XLEdit.getPropertyValue(rowIdx, colIdx, "value2") };
                this._findCount = 0;
                isFinded = direction === "up" ? this._findPrev(input1, sheetIdx, isCSen, isEMatch, type, mode, searchBy) : this._findNext(input1, sheetIdx, isCSen, isEMatch, type, mode, searchBy);
                if (!isFinded)
                    this._updateSelection(rowIdx, colIdx, sheetIdx);
                if (!xlObj._isUndoRedo)
                    xlObj._completeAction(details);
            }
            else
                xlObj._showAlertDlg("Alert", "ReplaceNotFound", "");

        },

        _replaceAllData: function (input1, input2, isCSen, isEMatch, type, sType) {
            var details, cell, cellInfo, selCells = [], skipCount = 0, sheetProtected = false, curSheet, xlObj = this.XLObj, sheetCount = xlObj.model.sheetCount, sheetIdx = xlObj.model.activeSheetIndex, actSheet = xlObj.getSheet(sheetIdx), usedRange = actSheet._usedRange, rowIdx = actSheet._activeCell.rowIndex, colIdx = actSheet._activeCell.colIndex, i, j, s, m, n, count = 0;
            details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "replaceAll" };
            if (sType === "sheet") {
                if (xlObj.model.allowLockCell && actSheet._isSheetProtected)
                    sheetProtected = true;
                for (i = 0, m = usedRange.rowIndex + 1; i < m; i++) {
                    for (j = 0, n = usedRange.colIndex + 1; j < n; j++)
                        if (this._compareValues(i, j, sheetIdx, input1, isCSen, isEMatch, type)) {
                            cellInfo = { rowIndex: i, colIndex: j, prevData: { value: xlObj.XLEdit.getPropertyValue(i, j, "value2") }, sheetIndex: sheetIdx };
                            this._replace({ rowIndex: i, colIndex: j }, sheetIdx, input1, input2, isCSen);
                            count++;
                            cellInfo.newData = { value: xlObj.XLEdit.getPropertyValue(i, j, "value2") };
                            selCells.push(cellInfo);
                        }
                }
            }
            else {
                for (s = 1; s <= sheetCount; s++) {
                    curSheet = xlObj.model.sheets[s];
                    if (xlObj.model.allowLockCell && curSheet._isSheetProtected) {
                        skipCount++;
                        continue;
                    }
                    usedRange = curSheet._usedRange;
                    for (i = 0, m = usedRange.rowIndex + 1; i < m; i++) {
                        for (j = 0, n = usedRange.colIndex + 1; j < n; j++)
                            if (this._compareValues(i, j, s, input1, isCSen, isEMatch, type)) {
                                cellInfo = { rowIndex: i, colIndex: j, prevData: { value: xlObj.XLEdit.getPropertyValue(i, j, "value2") }, sheetIndex: s };
                                this._replace(cell, s, input1, input2, isCSen);
                                count++;
                                cellInfo.newData = { value: xlObj.XLEdit.getPropertyValue(i, j, "value2") };
                                selCells.push(cellInfo);
                            }
                    }
                }
            }
            if (sheetProtected || skipCount === sheetCount) {
                xlObj._popUplockCellAlert();
                return;
            }
            details.selCells = selCells;
            this._updateSelection(rowIdx, colIdx);
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
            xlObj.element.ejWaitingPopup("hide");
            var alertText = xlObj._getLocStr("ReplaceData").split("/");
            alertText = "T-" + alertText[0] + count + " " + alertText[1];
            $("#" + xlObj._id + "_FRDialog").ejDialog("close");
            xlObj._showAlertDlg("Alert", alertText, "", 325);
        },

        _updateSelection: function (rowIdx, colIdx, sIndex) {
            var cellIdx = { rowIndex: rowIdx, colIndex: colIdx }, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx);
            if (!sIndex)
                sIndex = sheetIdx;
            if (sIndex && sIndex !== sheetIdx)
                xlObj.gotoPage(sIndex, false);
            if (xlObj.getSheetElement(sheetIdx).find(".e-selected:visible").length < 2) {
                xlObj.setActiveCell(cellIdx.rowIndex, cellIdx.colIndex, sIndex);
                xlObj.XLSelection.selectRange(cellIdx, cellIdx);
                sheet._startCell = cellIdx;
                sheet._endCell = cellIdx;
            }
            else
                xlObj.XLCellNav._updateActiveCell(rowIdx, colIdx);
            if (xlObj.model.scrollSettings.allowScrolling)
                xlObj.XLScroll._scrollSelectedPosition(sIndex, xlObj.getActiveCell(sIndex));
            if (xlObj.model.allowAutoFill)
                xlObj.XLDragFill.positionAutoFillElement();
        },

        _fRKeyDownHandler: function (e) {
            var xlObj = this.XLObj;
            if (e.ctrlKey) {
                if (e.keyCode === 70) { //Ctrl + F
                    e.preventDefault();
                    $("#" + xlObj._id + "_FRDialog").ejDialog("open");
                    $("#" + xlObj._id + "_FRDialog_FPDlgTab").ejTab("option", "selectedItemIndex", 0);
                }
                else if (e.keyCode === 72) { //Ctrl + H
                    e.preventDefault();
                    $("#" + xlObj._id + "_FRDialog").ejDialog("open");
                    $("#" + xlObj._id + "_FRDialog_FPDlgTab").ejTab("option", "selectedItemIndex", 1);
                }
                else if (e.keyCode === 71) { //Ctrl + G
                    e.preventDefault();
                    $("#" + xlObj._id + "_GoDialog").ejDialog("open");
                    $("#" + xlObj._id + "_GoToDlgTab").ejTab("option", "selectedItemIndex", 0);
                }
            }
        },

        //GoTo
        _getSelRangeDetails: function (selected) {
            var rowIdx = selected[selected.length - 1].rowIndex - selected[0].rowIndex, colIdx = selected[selected.length - 1].colIndex - selected[0].colIndex;
            var startRow = selected[0].rowIndex, startCol = selected[0].colIndex;
            return [rowIdx, colIdx, startRow, startCol];
        },

        _applyGoToRule: function (type, isNumber, isText, isLogical, isError) {
            var xlObj = this.XLObj;
            xlObj.element.ejWaitingPopup("show");
            switch (type) {
                case "blanks":
                    this._applyGotoBlanks();
                    break;
                case "formulas":
                    this._applyGotoFormulas(isNumber, isText, isLogical, isError);
                    break;
                case "comments":
                    this._applyGotoComments();
                    break;
                case "datavalidation":
                    this._applyGotoValidation();
                    break;
                case "cformat":
                    this._applyGotoCFormat();
                    break;
                case "lastcell":
                    var sheetIdx = xlObj.getActiveSheetIndex(), usedRange = xlObj.model.sheets[sheetIdx]._usedRange;
                    xlObj.XLSelection.selectRange({ rowIndex: usedRange.rowIndex, colIndex: usedRange.colIndex }, { rowIndex: usedRange.rowIndex, colIndex: usedRange.colIndex });
                    this._updateSelection(usedRange.rowIndex, usedRange.colIndex, sheetIdx);
                    break;
                case "constants":
                    this._applyGotoConstants(isNumber, isText, isLogical, isError);
                    break;
                case "rowdiff":
                    this._applyGotoRowDiff();
                    break;
                case "coldiff":
                    this._applyGotoColDiff();
                    break;
                case "visiblecells":
                    break;
            }
            xlObj.element.ejWaitingPopup("hide");
        },

        _applyGotoBlanks: function () {
            var i, j, cell, isViewed = false, selCells = [], temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), usedRange = xlObj.model.sheets[sheetIdx]._usedRange, value, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0;
            var details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "blanks" };
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
            }
            for (i = 0; i <= rowIdx; i++) {
                for (j = 0; j <= colIdx; j++) {
                    value = xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol) ? xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol).toString() : "";
                    if (!value.length) {
                        selCells.push({ rowIndex: i + startRow, colIndex: j + startCol, isApply: true });
                        if (xlObj._isRowViewable(sheetIdx, i + startRow)) {
                            cell = xlObj.getCell(i + startRow, j + startCol, sheetIdx);
                            cell.addClass("e-selected");
                            isViewed = true;
                        }
                    } else {
                        if (isViewed) {
                            cell.removeClass("e-selected");
                            isViewed = false;
                        }
                    }
                }
            }
            details.selCells = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _applyGotoComments: function () {
            var i, j, cell, isViewed = false, selCells = [], temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), usedRange = xlObj.getSheet(sheetIdx)._usedRange, value, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0;
            var details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "comments" };
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
            }
            for (i = 0; i <= rowIdx; i++) {
                for (j = 0; j <= colIdx; j++) {
                    value = xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol, "comment");
                    if (!ej.isNullOrUndefined(value)) {
                        selCells.push({ rowIndex: i + startRow, colIndex: j + startCol, isApply: true });
                        if (xlObj._isRowViewable(sheetIdx, i + startRow)) {
                            cell = xlObj.getCell(i + startRow, j + startCol, sheetIdx);
                            cell.addClass("e-selected");
                            isViewed = true;
                        }
                    } else {
                        if (isViewed) {
                            cell.removeClass("e-selected");
                            isViewed = false;
                        }
                    }
                }
            }
            details.selCells = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _applyGotoFormulas: function (isNumber, isText, isLogical, isError) {
            var i, j, cell, isViewed = false, selCells = [], temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), value, value2, isFormula, usedRange = xlObj.model.sheets[sheetIdx]._usedRange, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0;
            var details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "formulas" };
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
            }
            for (i = 0; i <= rowIdx; i++) {
                for (j = 0; j <= colIdx; j++) {
                    value = !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol)) ? xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol).toString() : "";
                    value2 = !ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol, "value2")) ? xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol, "value2").toString() : "";
                    isFormula = value.charAt(0) === "=" && typeof value2 === "string";
                    if (isFormula && ((isNumber && ($.isNumeric(value2))) || (isText && (!(value2.toLowerCase() === "true" || value2.toLowerCase() === "false") && !$.isNumeric(value2) && !(value2.length !== 0 && (value2.indexOf("#") === 0 && value2.lastIndexOf("?") === value2.length - 1) && value2.length > 0))) || (isLogical && (value2.toLowerCase() === "true" || value2.toLowerCase() === "false")) || (isError && value2.length !== 0 && (xlObj._sheets[sheetIdx]._calcEngine.getErrorStrings().indexOf("#VALUE!") > -1)))) {
                        selCells.push({ rowIndex: i + startRow, colIndex: j + startCol, isApply: true });
                        if (xlObj._isRowViewable(sheetIdx, i + startRow)) {
                            cell = xlObj.getCell(i + startRow, j + startCol, sheetIdx);
                            cell.addClass("e-selected");
                            isViewed = true;
                        }
                    } else {
                        if (isViewed) {
                            cell.removeClass("e-selected");
                            isViewed = false;
                        }
                    }
                }
            }
            details.selCells = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _applyGotoValidation: function () {
            var i, j, cell, isViewed = false, selCells = [], temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), usedRange = xlObj.model.sheets[sheetIdx]._usedRange, value, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0;
            var details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "validations" };
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
            }
            for (i = 0; i <= rowIdx; i++) {
                for (j = 0; j <= colIdx; j++) {
                    value = xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol, "rule");
                    if (!ej.isNullOrUndefined(value)) {
                        selCells.push({ rowIndex: i + startRow, colIndex: j + startCol, isApply: true });
                        if (xlObj._isRowViewable(sheetIdx, i + startRow)) {
                            cell = xlObj.getCell(i + startRow, j + startCol, sheetIdx);
                            cell.addClass("e-selected");
                            isViewed = true;
                        }
                    } else {
                        if (isViewed) {
                            cell.removeClass("e-selected");
                            isViewed = false;
                        }
                    }
                }
            }
            details.selCells = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _applyGotoCFormat: function () {
            var i, j, cell, isViewed = false, selCells = [], temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), usedRange = xlObj.model.sheets[sheetIdx]._usedRange, value, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0;
            var details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "conditions" };
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
            }
            for (i = 0; i <= rowIdx; i++) {
                for (j = 0; j <= colIdx; j++) {
                    value = xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol, "cFormatRule");
                    if (!ej.isNullOrUndefined(value) && value.length) {
                        selCells.push({ rowIndex: i + startRow, colIndex: j + startCol, isApply: true });
                        if (xlObj._isRowViewable(sheetIdx, i + startRow)) {
                            cell = xlObj.getCell(i + startRow, j + startCol, sheetIdx);
                            cell.addClass("e-selected");
                            isViewed = true;
                        }
                    } else {
                        if (isViewed) {
                            cell.removeClass("e-selected");
                            isViewed = false;
                        }
                    }
                }
            }
            details.selCells = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _applyGotoConstants: function (isNumber, isText, isLogical, isError) {
            var i, j, cell, isViewed = false, selCells = [], isFormula, temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), value, value2, usedRange = xlObj.model.sheets[sheetIdx]._usedRange, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0;
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
            }
            for (i = 0; i <= rowIdx; i++) {
                for (j = 0; j <= colIdx; j++) {
                    value = ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol)) ? "" : xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol).toString();
                    value2 = ej.isNullOrUndefined(xlObj.XLEdit.getPropertyValueByElem(cell, "value2")) ? "" : xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol, "value2").toString();
                    isFormula = value.charAt(0) === "=" && typeof value2 === "string";
                    if (!isFormula && ((isNumber && ($.isNumeric(value))) || (isText && (!(value.toLowerCase() === "true" || value.toLowerCase() === "false") && !$.isNumeric(value) && value.length > 0)) || (isLogical && (value.toLowerCase() === "true" || value.toLowerCase() === "false") || (isError && value2.length !== 0 && (value2.indexOf("#") === 0 && value2.lastIndexOf("?") === value2.length - 1))))) {
                        selCells.push({ rowIndex: i + startRow, colIndex: j + startCol, isApply: true });
                        if (xlObj._isRowViewable(sheetIdx, i + startRow)) {
                            cell = xlObj.getCell(i + startRow, j + startCol, sheetIdx);
                            cell.addClass("e-selected");
                            isViewed = true;
                        }
                    } else {
                        if (isViewed) {
                            cell.removeClass("e-selected");
                            isViewed = false;
                        }
                    }
                }
            }
        },

        _applyGotoColDiff: function () {
            var i, j, cell, isViewed = false, selCells = [], temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), usedRange = xlObj.model.sheets[sheetIdx]._usedRange, value, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0;
            var details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "coldiff" }, result;
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
                for (i = 0; i <= colIdx; i++) {
                    result = xlObj.XLEdit.getPropertyValue(startRow, i + startCol) ? xlObj.XLEdit.getPropertyValue(startRow, i + startCol).toString() : "";
                    for (j = 0; j <= rowIdx; j++) {
                        value = xlObj.XLEdit.getPropertyValue(j + startRow, i + startCol) ? xlObj.XLEdit.getPropertyValue(j + startRow, i + startCol).toString() : "";
                        if (value === result) {
                            if (xlObj._isRowViewable(sheetIdx, j + startRow)) {
                                cell = xlObj.getCell(j + startRow, i + startCol, sheetIdx);
                                cell.removeClass("e-selected");
                                isViewed = true;
                            }
                        }
                        else {
                            selCells.push({ rowIndex: j + startRow, colIndex: i + startCol, isApply: true });
                            if (isViewed) {
                                cell.addClass("e-selected");
                                isViewed = false;
                            }
                        }
                    }
                }
            }
            details.selCells = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _applyGotoRowDiff: function () {
            var i, j, cell, isViewed = false, selCells = [], temp, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), usedRange = xlObj.model.sheets[sheetIdx]._usedRange, value, rowIdx = usedRange.rowIndex, colIdx = usedRange.colIndex, selected = xlObj.getSheet(sheetIdx)._selectedCells, startRow = 0, startCol = 0, result;
            var details = { sheetIndex: sheetIdx, reqType: "find-replace", operation: "rowdiff" };
            if (selected.length > 1) {
                temp = this._getSelRangeDetails(selected);
                rowIdx = temp[0];
                colIdx = temp[1];
                startRow = temp[2];
                startCol = temp[3];
                for (i = 0; i <= rowIdx; i++) {
                    result = xlObj.XLEdit.getPropertyValue(i + startRow, startCol) ? xlObj.XLEdit.getPropertyValue(i + startRow, startCol).toString() : "";
                    for (j = 0; j <= colIdx; j++) {
                        value = xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol) ? xlObj.XLEdit.getPropertyValue(i + startRow, j + startCol).toString() : "";
                        if (value === result) {
                            if (xlObj._isRowViewable(sheetIdx, i + startRow)) {
                                cell = xlObj.getCell(i + startRow, j + startCol, sheetIdx);
                                cell.removeClass("e-selected");
                                isViewed = true;
                            }
                        }
                        else {
                            selCells.push({ rowIndex: i + startRow, colIndex: j + startCol, isApply: true });
                            if (isViewed) {
                                cell.addClass("e-selected");
                                isViewed = false;
                            }
                        }
                    }
                }
            }
            details.selCells = selCells;
            if (!xlObj._isUndoRedo)
                xlObj._completeAction(details);
        }
    };
})(jQuery, Syncfusion);

$.fn.setInputPos = function (start, end) {
    if (!end) end = start;
    if ($(this).is(":visible")) {
        return this.each(function () {
            if (this.setSelectionRange) {
                this.focus();
                this.setSelectionRange(start, end);
            } else if (this.createTextRange) {
                var range = this.createTextRange();
                range.collapse(true);
                range.moveEnd("character", end);
                range.moveStart("character", start);
                range.select();
            }
        });
    }
};;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.shape = function (obj) {
        this.XLObj = obj;
        this._imgX = 0;
        this._imgY = 0;
        this._imgOffsetleft = 0;
        this._imgOffsetTop = 0;
        this._shapeType = "Img";
        this._shapeROStart = false;
        this._shapeRSStart = false;
        this._imgRSWStart = false;
        this._imgRSEStart = false;
        this._imgRSNStart = false;
        this._imgRSSStart = false;
        this._changePicture = false;
        this._picCellIdx = {};
        this._insertPic = false;
    };

    ej.spreadsheetFeatures.shape.prototype = {
        _insertPicture: function (e) {
            var sheetIdx, left, top, imgElem, details, shapeMngr, shapeObj, picIdx, cellIdx, xlObj = this.XLObj, imgData = xlObj._browserDetails.name === "msie" && parseInt(xlObj._browserDetails.version) < 10 ? e.responseText : e.xhr.response;
            if (imgData.startsWith("Invalid"))
                xlObj._showAlertDlg("Alert", "ImageValAlert");
            else {
                sheetIdx = xlObj.getActiveSheetIndex();
                if (this._changePicture) {
                    imgElem = xlObj.element.find(".e-ss-activeimg").get(0);
                    left = imgElem.offsetLeft;
                    top = imgElem.offsetTop;
                    cellIdx = this._getCellIndexFromOffset(top, left);
                    shapeMngr = xlObj.getSheet(sheetIdx)._shapeMngr;
                    $(imgElem).css('background-image', "url('" + imgData + "')");
                    shapeMngr.picture.some(function (obj, i) {
                        if (imgElem.id === obj.id) {
                            picIdx = i;
                            return true;
                        }
                    });
                    shapeObj = shapeMngr.picture[picIdx];
                    details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "picture", action: "changepic", id: $(imgElem).attr("id"), cell: cellIdx, prev: { img: shapeMngr.sharedPics[shapeObj.data], data: shapeObj.data, pcIdx: picIdx }, cur: { img: imgData, pcIdx: picIdx } };
                    if (shapeMngr.sharedPics.indexOf(imgData) === -1)
                        shapeMngr.sharedPics.push(imgData);
                    shapeObj.data = shapeMngr.sharedPics.indexOf(imgData);
                    details.cur.data = shapeObj.data;
                    this._changePicture = false;
                    xlObj.setSheetFocus();
                    xlObj._completeAction(details);
                }
                else {
                    var activeCell = xlObj.getActiveCellElem()[0], picSettings = xlObj.model.pictureSettings;
                    var height = picSettings.height, width = picSettings.width;
                    if (ej.isNullOrUndefined(activeCell)) {
                        imgElem = xlObj.element.find(".e-ss-activeimg").get(0);
                        cellIdx = this._getCellIndexFromOffset(imgElem.offsetTop, imgElem.offsetLeft);
                        activeCell = xlObj.getCell(cellIdx.rowIndex, cellIdx.colIndex)[0];
                    }
                    left = activeCell.offsetLeft;
                    top = activeCell.offsetTop;
                    this._insertPic = true;
                    this._createPicture(sheetIdx, activeCell, imgData, top, left, width, height);
                }
            }
        },

        setPicture: function (rowIdx, colIdx, url, width, height, top, left) {
            var xlObj = this.XLObj;
            if (!xlObj.model.pictureSettings.allowPictures)
                return;
            var sheetIdx = xlObj.getActiveSheetIndex(), activeCell, picSettings = xlObj.model.pictureSettings;
            height = height ? height : picSettings.height;
            width = width ? width : picSettings.width;
            if(xlObj._isRowViewable(sheetIdx, rowIdx)){
                activeCell = xlObj.getCell(rowIdx, colIdx)[0];
                top = top ? top : activeCell.offsetTop;
                left = left ? left : activeCell.offsetLeft;
            }
            else {
                top = top ? top : 0;
                left = left ? left : 0;
            }
            this._createPicture(sheetIdx, {rowIndex: rowIdx, colIndex: colIdx}, url, top, left, width, height);
        },

        _createPicture: function (sheetIdx, activeCell, imgData, top, left, width, height, cnt, bcolor, bstyle, bwidth) {
            bcolor = bcolor ? bcolor : "transparent";
            bstyle = bstyle ? bstyle : "solid";
            bwidth = bwidth ? bwidth : "1px";
            var xlObj = this.XLObj, args = {
                sheetIndex: sheetIdx,
                targetCell: activeCell,
                top: top,
                left: left,
                width: width,
                height: height,
                bcolor: bcolor,
                bstyle: bstyle,
                bwidth: bwidth,
                reqType: "picture",
                add: "add"
            };
            if (xlObj._trigActionBegin(args))
                return;
            top = args.top;
            left = args.left;
            width = args.width;
            height = args.height;
            cnt = ej.isNullOrUndefined(cnt) ? (xlObj.model.allowCharts && xlObj.XLChart._shapeCnt) : cnt;
            var $elem, details;
            var rowIdx = activeCell.rowIndex, colIdx = activeCell.colIndex;
            xlObj._getContent(sheetIdx).find(".e-content").append(ej.buildTag("div.e-ss-object", "", "", { id: xlObj._id + "_picture" + cnt }));
            $elem = $("#" + xlObj._id + "_picture" + cnt);
            $elem.css({ 'background-image': "url('" + imgData + "')", height: height, width: width, left: left, top: top });
            var borderCss = bwidth + " " + bstyle + " " + bcolor;
            var hashCode = this._getImgBorderHashCode({ "border-color": bcolor, "border-style": bstyle, "border-width": bwidth });
            if(xlObj.model.allowCellFormatting)
				xlObj.XLFormat._writeCssRules(hashCode, xlObj.XLFormat._getCssPropertyAsString({ "border": { "left": borderCss, "right": borderCss, "top": borderCss, "bottom": borderCss } }));                        
            $elem.addClass(hashCode);
            if(!xlObj._isUndoRedo && xlObj.model.allowCharts)
				xlObj.XLChart._shapeCnt++;
            xlObj.model.sheets[sheetIdx]._importRendered ? xlObj.model.sheets[sheetIdx]._shapeMngr["picture"][cnt]["id"] = xlObj._id + "_picture" + cnt : this._updateShapeMngr(activeCell, { "picture": { data: imgData, id: xlObj._id + "_picture" + cnt, height: height, width: width, left: left, top: top, rowIndex: rowIdx, colIndex: colIdx } }, "picture");
            this._selectImg($elem);
            xlObj.setSheetFocus();
            if (!xlObj.model.sheets[sheetIdx]._importRendered && !xlObj._isUndoRedo)
                if (!xlObj._isPaste && !xlObj.XLClipboard._isShape) {
                    details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "picture", action: "remove", id: xlObj._id + "_picture" + cnt, actCell: activeCell, img: imgData, cnt: cnt };
                    details.position = { top: top, left: left, height: height, width: width };
                    xlObj._completeAction(details);
                }
        },

        _shapeMouseDown: function (e) {
            e.preventDefault();
            var xlObj = this.XLObj, actElemId = document.activeElement.id;
            if (actElemId.indexOf("pictureheight") > -1 || actElemId.indexOf("picturewidth") > -1)
                xlObj.setSheetFocus();
            var prop, id = e.target.id, $trgt = $(e.target), imgElem = $trgt.get(0), left = imgElem.offsetLeft, top = imgElem.offsetTop, xy = xlObj._setXY(e);
            this._selectImg($trgt);
            this._picCellIdx = this._getCellIndexFromOffset(top, left);
            this._imgOffsetleft = left;
            this._imgOffsetTop = top;
            this._imgX = xy[0];
            this._imgY = xy[1];
            this._shapeType = id.indexOf("chart") > -1 ? "chart" : "Img";
            if ($trgt.css("cursor") === "move")
                this._shapeROStart = true;
            else if ($trgt.css("cursor") === "col-resize" || $trgt.css("cursor") === "row-resize") {
                this._shapeRSStart = true;
                xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-ss-imgvisual").css({ left: left, top: top, height: $trgt.height(), width: $trgt.width() });
            }
            if (this._shapeType === "chart") {
                prop = xlObj.XLChart._getShapeObj(id, this._shapeType);
                if (xlObj.getActiveSheetIndex() === prop.dataSheetIdx)
                    xlObj.XLChart._focusChartRange(prop.xAxis.range, prop.yAxis.range, prop.legend.range);
            }
        },

        _imgKeyDown: function (e) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), $trgt = xlObj._getContent(sheetIdx).find(".e-ss-activeimg"), imgElem = $trgt.get(0);
            var left = imgElem.offsetLeft, top = imgElem.offsetTop, height, width;
            this._shapeType = $trgt[0].id.indexOf("chart") > -1 ? "chart" : "Img";
            this._picCellIdx = this._getCellIndexFromOffset(top, left);
            if (!e.shiftKey && !e.ctrlKey) {
                switch (e.keyCode) {
                    case 39:
                        //right
                        e.preventDefault();
                        $trgt.css({ left: left + 2 });
                        this._updateShapeObj(imgElem);
                        break;
                    case 37:
                        //left
                        e.preventDefault();
                        if (left > 1)
                            $trgt.css({ left: left - 2 });
                        this._updateShapeObj(imgElem);
                        break;
                    case 38:
                        //up
                        e.preventDefault();
                        if (top > 1)
                            $trgt.css({ top: top - 2 });
                        this._updateShapeObj(imgElem);
                        break;
                    case 40:
                        //down
                        e.preventDefault();
                        $trgt.css({ top: top + 2 });
                        this._updateShapeObj(imgElem);
                        break;
                }
                this._onKeyscrollShape(sheetIdx, $trgt, e);
            } else if (e.shiftKey && !e.ctrlKey) {
                if (e.keyCode === 39 || e.keyCode === 38) { //right or up
                    e.preventDefault();
                    height = $trgt.height() + 2;
                    width = $trgt.width() + 2;
                    if (height < 400 && width < 600)
                        $trgt.css({ width: width, height: height });
                    this._updateShapeObj(imgElem);
                    xlObj.XLRibbon._formatTabUpdate();
                } else if (e.keyCode === 37 || e.keyCode === 40) { //left or down
                    e.preventDefault();
                    height = $trgt.height() - 2;
                    width = $trgt.width() - 2;
                    if (height > 40 && width > 60)
                        $trgt.css({ width: width, height: height });
                    this._updateShapeObj(imgElem);
                    xlObj.XLRibbon._formatTabUpdate();
                }
            }
            if (e.keyCode === 46) { //delete
                e.preventDefault();
                var changed = false;
                if (xlObj.XLClipboard._isShape) {
                    xlObj.XLClipboard._isShape = false;
                    changed = true;
                }
                this._deleteShape(sheetIdx, $trgt);
                (changed) && (xlObj.XLClipboard._isShape = true);
            }
        },

        _shapeMouseUp: function (e) {
            var chart = "chart", details, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), $content = xlObj._getContent(sheetIdx), $actImg = $content.find(".e-ss-activeimg"), $visualImg = $content.find(".e-ss-imgvisual");
            var $trgt = $(e.target), imgElem = $trgt.get(0);
            if ($trgt.hasClass("e-ss-picture") || $trgt[0].id.indexOf(xlObj._id + "_picture") > -1)
                xlObj.XLRibbon._formatTabUpdate();
            if (imgElem.id.indexOf(chart) > -1 && imgElem.id.indexOf("Ribbon") < 0)
                imgElem = (imgElem.id.indexOf('_svg_') > -1) ? ($(imgElem).parents(".e-ss-object").get(0)) : imgElem;
            if (!(imgElem.className.indexOf('e-ss-imgvisual') > -1) && !(imgElem.className.indexOf('e-ss-object') > -1)) {
                if (this._shapeROStart)
                    imgElem = $actImg;
                if (this._shapeRSStart)
                    imgElem = $visualImg[0];
            }
            if (this._shapeROStart) {
                if (!xlObj._isUndoRedo) {
                    if (parseInt($actImg.css("top")) !== imgElem.offsetTop || parseInt($actImg.css("left")) !== imgElem.offsetLeft) {
                        details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "edit", actionType: "reorder", id: $($actImg).attr("id"), visualImg: $visualImg, trgt: $trgt };
                        details.prev = { top: parseInt($actImg.css("top")), left: parseInt($actImg.css("left")), height: parseInt($actImg.css("height")), width: parseInt($actImg.css("width")) };
                        details.cur = { top: imgElem.offsetTop, left: imgElem.offsetLeft, height: $trgt.height(), width: $trgt.width() };
                    }
                }
                if (!xlObj._isUndoRedo) {
                    if ((parseInt($actImg.css("top")) !== imgElem.offsetTop || parseInt($actImg.css("left")) !== imgElem.offsetLeft) && xlObj._checkIndicesInContainer(sheetIdx, this._picCellIdx.rowIndex, this._picCellIdx.colIndex)) {
                        details.obj = $.extend(true, {}, xlObj._dataContainer.sheets[sheetIdx][this._picCellIdx.rowIndex][this._picCellIdx.colIndex]);
                        xlObj._completeAction(details);
                    }
                }
                $actImg.css({ left: imgElem.offsetLeft, top: imgElem.offsetTop });
                this._updateShapeObj(imgElem);
                $visualImg.removeClass("e-ss-picture").hide();
            } else if (this._shapeRSStart) {
                if (!xlObj._isUndoRedo) {
                    if (imgElem.className.indexOf("e-ss-imgvisual") < 0 && $visualImg.length > 0)
                        (this._shapeROStart) && (imgElem = $visualImg[0]);
                    $trgt = $(imgElem);
                    if ((parseInt($actImg.css("height")) !== $trgt.height() || parseInt($actImg.css("width")) !== $trgt.width()) && xlObj._checkIndicesInContainer(sheetIdx, this._picCellIdx.rowIndex, this._picCellIdx.colIndex)) {
                        details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "edit", actionType: "resize", id: $($actImg).attr("id"), visualImg: $visualImg, trgt: $trgt };
                        details.prev = { top: parseInt($actImg.css("top")), left: parseInt($actImg.css("left")), height: parseInt($actImg.css("height")), width: parseInt($actImg.css("width")) };
                        details.cur = { top: imgElem.offsetTop, left: imgElem.offsetLeft, height: $trgt.height(), width: $trgt.width() };
                        details.obj = $.extend(true, {}, xlObj._dataContainer.sheets[sheetIdx][this._picCellIdx.rowIndex][this._picCellIdx.colIndex]);
                        xlObj._completeAction(details);
                    }
                }
                $actImg.css({ left: imgElem.offsetLeft, top: imgElem.offsetTop, height: $trgt.height(), width: $trgt.width() });
                this._updateShapeObj(imgElem);
                $visualImg.removeClass("e-ss-picture").hide();
                if (this._shapeType === "Img" && xlObj.model.showRibbon)
                    xlObj.XLRibbon._formatTabUpdate();
                if (this._shapeType === chart) {
                    xlObj.XLChart.resizeChart($actImg.attr("id"), $trgt.height(), $trgt.width());
                    if (xlObj.model.showRibbon)
                        xlObj.XLRibbon._chartDesignTabUpdate($("#" + $actImg.attr("id")));
                }
            }
            this._refreshImgResizing();
            this._shapeType = "Img";
            this._shapeROStart = this._shapeRSStart = false;
        },

        _imgMouseMove: function (e) {
            var xlObj = this.XLObj;
            if (xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-ss-activeimg").length) {
                e.preventDefault();
                var sheetIdx = xlObj.getActiveSheetIndex(), $content = xlObj._getContent(sheetIdx), $actImg = $content.find(".e-ss-activeimg"), $visualImg = $content.find(".e-ss-imgvisual");
                var imgElem = $actImg.get(0), location = imgElem.getBoundingClientRect(), xy = xlObj._setXY(e), x = xy[0], y = xy[1], extra, left, top;
                if (this._shapeROStart) {
                    $visualImg.show();
                    if (imgElem.id.indexOf("picture") > -1)
                        $visualImg.addClass("e-ss-picture");
                    left = x - (this._imgX - this._imgOffsetleft);
                    top = y - (this._imgY - this._imgOffsetTop);
                    if (left > -1 && top > -1)
                        $visualImg.css({ left: left, top: top, height: $actImg.height(), width: $actImg.width() });
                    //this._scrollShape(sheetIdx, $visualImg, x, y); // shape resize in scrolling - consider it later
                } else {
                    var width = $visualImg.width(), height = $visualImg.height(), offsetWidth = imgElem.offsetWidth, offsetHeight = imgElem.offsetHeight,
                        vCont = $content.find(".e-content"), hScroll = $content.find(".e-hscrollbar").data("ejScrollBar"), vScroll = $content.find(".e-vscrollbar").data("ejScrollBar");
                    if (this._shapeRSStart) {
                        $visualImg.show();
                        if (imgElem.id.indexOf("picture") > -1)
                            $visualImg.addClass("e-ss-picture");
                        if ($actImg.css("cursor") === "col-resize" || $actImg.css("cursor") === "row-resize") {
                            switch (true) {
                                case this._imgRSWStart:
                                    extra = this._imgX - x;
                                    left = $visualImg[0].offsetLeft - extra;
                                    if (left > -1) {
                                        $visualImg.css({ width: width + extra, left: left });
                                        this._imgX = x;
                                    }
                                    break;
                                case this._imgRSEStart:
                                    extra = this._imgX - x;
                                    if (xlObj.model.scrollSettings.allowScrolling && (width - extra + $visualImg[0].offsetLeft + 5) < (vCont.width() + hScroll.value())) {
                                        $visualImg.css({ width: width - extra });
                                        this._imgX = x;
                                    }
                                    break;
                                case this._imgRSNStart:
                                    extra = this._imgY - y;
                                    top = $visualImg[0].offsetTop - extra;
                                    if (top > -1) {
                                        $visualImg.css({ height: height + extra, top: top });
                                        this._imgY = y;
                                    }
                                    break;
                                case this._imgRSSStart:
                                    extra = this._imgY - y;
                                    if (xlObj.model.scrollSettings.allowScrolling && (height - extra + $visualImg[0].offsetTop + 5) < (vCont.height() + vScroll.value())) {
                                        $visualImg.css({ height: height - extra });
                                        this._imgY = y;
                                    }
                                    break;
                            }
                        }
                        // this._scrollShape(sheetIdx, $visualImg, x, y); // shape resize in scrolling - consider it later
                    } else {
                        this._refreshImgResizing();
                        if ((x <= (location.left + 4)) && (x >= location.left)) {
                            $actImg.addClass("e-ss-imgcresize");
                            this._imgRSWStart = true;
                        } else if ((x <= (location.left + offsetWidth)) && (x >= location.left + offsetWidth - 4)) {
                            $actImg.addClass("e-ss-imgcresize");
                            this._imgRSEStart = true;
                        } else {
                            $content.find(".e-ss-imgcresize").removeClass("e-ss-imgcresize");
                        }

                        if ((y <= (location.top + 4)) && (y >= location.top)) {
                            $actImg.addClass("e-ss-imgrresize");
                            this._imgRSNStart = true;
                        } else if ((y <= (location.top + offsetHeight)) && (y >= location.top + offsetHeight - 4)) {
                            $actImg.addClass("e-ss-imgrresize");
                            this._imgRSSStart = true;
                        } else {
                            $content.find(".e-ss-imgrresize").removeClass("e-ss-imgrresize");
                        }
                    }
                }
            }
        },

        _deleteShape: function (sheetIdx, $trgt) {
            var selCell, xlObj = this.XLObj, cellIdx = this._picCellIdx, container = xlObj._dataContainer, sheet = xlObj.getSheet(sheetIdx), shapeMngr = sheet._shapeMngr, cellObj = container.sheets[sheetIdx][cellIdx.rowIndex][cellIdx.colIndex], details;
            var shapeType = $trgt.get(0).id.split("_")[1].replace(/[0-9]/g, ''), imgElem = $trgt.get(0), objIdx, picIdx, shapeObj = shapeMngr[shapeType][cellObj[shapeType][0]];
            shapeMngr[shapeType].some(function (obj, i) {
                if (imgElem.id === obj.id) {
                    picIdx = i;
                    return true;
                }
                return false;
            });
            if (!xlObj._isUndoRedo)
                if (!xlObj._isPaste && !xlObj.XLClipboard._isShape) {
                    details = { sheetIndex: sheetIdx, id: imgElem.id, picIndex: picIdx, reqType: "shape", shapeType: shapeType, action: "create", options: shapeObj, range: shapeObj.range, img: shapeMngr.sharedPics[shapeObj.data] };
                    details.actCell = xlObj.getCell(cellIdx.rowIndex, cellIdx.colIndex, sheetIdx)[0];
                    details.position = { height: shapeObj.height, width: shapeObj.width, top: shapeObj.top, left: shapeObj.left };
                    details.cnt = $trgt.get(0).id.split("_")[1].replace(/[a-z]/g, '');
                    xlObj._completeAction(details);
                }
            cellObj[shapeType].some(function (obj, i) {
                if (imgElem.id === shapeMngr[shapeType][cellObj[shapeType][i]].id) {
                    objIdx = i;
                    return true;
                }
                return false;
            });
            if (Object.keys(cellObj).length < 2 && cellObj[shapeType].length < 2)
                delete cellObj;
            else if (cellObj[shapeType].length < 2)
                delete cellObj[shapeType];
            else
                cellObj[shapeType].splice(objIdx, 1);
            selCell = { rowIndex: cellIdx.rowIndex, colIndex: cellIdx.colIndex };
            if (sheetIdx === xlObj._getSheetIndex()) { // check for cut paste sheet to sheet
                xlObj.setActiveCell(cellIdx.rowIndex, cellIdx.colIndex);
                xlObj.performSelection(selCell, selCell);
            }
            $trgt.remove();
            xlObj.updateFormulaBar();
            if (xlObj.model.showRibbon) {
                if (shapeType === "picture")
                    xlObj.XLRibbon._toggleFormatTab();
                else
                    xlObj.XLRibbon._toggleChartDesignTab();
            }
            if (sheet._isChartBorderDrawn)
                xlObj.XLChart._clearChartRange(xlObj._arrayAsString(xlObj._chartBorder));
        },

        _selectImg: function (elem) {
            this.XLObj._getContent(this.XLObj.getActiveSheetIndex()).find(".e-ss-activeimg").removeClass("e-ss-activeimg");
            elem.addClass("e-ss-activeimg");
            this.XLObj.XLSelection._cleanUp(true);
        },

        _refreshImgResizing: function () {
            this._imgRSEStart = false;
            this._imgRSWStart = false;
            this._imgRSNStart = false;
            this._imgRSSStart = false;
        },

        _getShapePropeties: function (cellIdx, id, type) {
            var i, data = this.XLObj.XLEdit.getPropertyValue(cellIdx.rowIndex, cellIdx.colIndex, type);
            while (i in data) {
                if (data[i].id === id)
                    return data[i];
                else
                    return;
            }
        },

        _updateShapeMngr: function (cell, shapeObj, type) {
            var shapeIdx, index, obj = {}, insert = false, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), 
                shapeMngr = xlObj.getSheet(sheetIdx)._shapeMngr, shape = shapeMngr[type];            
            if (xlObj.model.sheets[sheetIdx]._importRendered && type === "chart") {
                for (var i = 0; i < shape.length; i++) {
                    if (shape[i].dummyId === shapeObj.chart._id) {
                        insert = true;
                        shape[i] = shapeObj.chart;
                        index = i;
                    }
                }
            }
            if (!insert)
                shape.push($.extend(true, {}, shapeObj[type]));
            if (type === "picture") {
                if(shapeMngr.sharedPics.indexOf(shapeObj[type].data) < 0)
                     shapeMngr.sharedPics.push(shapeObj[type].data);
                shapeIdx = shapeMngr.sharedPics.indexOf(shapeObj[type].data);
                shape[shape.length - 1].data = shapeIdx;
            }
            if (!xlObj._isPaste || this._insertPic || xlObj.XLClipboard._isShape) {
                obj[type] = insert ? index : shape.length - 1;
                xlObj.XLEdit._updateDataContainer(cell, obj);
                this._insertPic = false;
            }
        },

        _updateShapeObj: function (imgElem) {
            var oldElem, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), container = xlObj._dataContainer, shapeObj, cellObj, picIdx;
            var chart = "chart", type = this._shapeType === "Img" ? "picture" : chart, cellIdx = this._picCellIdx;
            var shapeMngr = xlObj.getSheet(sheetIdx)._shapeMngr, cellPicIdx, cObj = {};
            if (!xlObj.getObjectLength(this._picCellIdx))
                cellIdx = this._picCellIdx = this._getCellIndexFromOffset(imgElem.offsetTop, imgElem.offsetLeft);
            oldElem = xlObj._getContent(sheetIdx).find(".e-ss-activeimg");
            cellObj = container.sheets[sheetIdx][cellIdx.rowIndex][cellIdx.colIndex];
            if (!cellObj || !cellObj[type])
                return;
            shapeMngr[type].some(function (obj, i) {
                if (oldElem[0].id === obj.id) {
                    picIdx = i;
                    return true;
                }
                return false;
            });
            shapeObj = shapeMngr[type][picIdx];
            cellObj[type].some(function (obj, i) {
                if (oldElem[0].id === shapeMngr[type][cellObj[type][i]].id) {
                    cellPicIdx = i;
                    return true;
                }
                return false;
            });
            cObj[type] = cellObj[type][cellPicIdx];
            if (xlObj.getObjectLength(cellObj) < 2 && cellObj[type].length < 2)
                delete container.sheets[sheetIdx][cellIdx.rowIndex][cellIdx.colIndex];
            else if (cellObj[type].length < 2)
                delete container.sheets[sheetIdx][cellIdx.rowIndex][cellIdx.colIndex][type];
            else
                container.sheets[sheetIdx][cellIdx.rowIndex][cellIdx.colIndex][type].splice(cellPicIdx, 1);
            this._picCellIdx = cellIdx = this._getCellIndexFromOffset(imgElem.offsetTop, imgElem.offsetLeft);
            shapeObj.height = oldElem.height();
            shapeObj.width = oldElem.width();
            shapeObj.left = oldElem[0].offsetLeft;
            shapeObj.top = oldElem[0].offsetTop;
            shapeObj.rowIndex = cellIdx.rowIndex;
            shapeObj.colIndex = cellIdx.colIndex;
            xlObj.XLEdit._updateDataContainer({rowIndex: cellIdx.rowIndex, colIndex: cellIdx.colIndex}, cObj);
        },

        _updateImgBorder: function (imgElem, width, style, color) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), left = imgElem.offsetLeft, top = imgElem.offsetTop, details;
            var cellIdx = this._getCellIndexFromOffset(top, left), shapeMngr = xlObj.getSheet(sheetIdx)._shapeMngr, shapeObj, picIdx, regx = new RegExp("\\b" + "e-shapebdr" + ".*?\\b", "g");
            shapeMngr.picture.some(function (obj, i) {
                if (imgElem.id === obj.id) {
                    picIdx = i;
                    return true;
                }
                return false;
            });
            shapeObj = shapeMngr.picture[picIdx];
            details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "picture", action: "border", id: $(xlObj.element.find(".e-ss-activeimg")).attr("id"), prevClass: imgElem.className.match(regx)[0], picIdx: picIdx, cellIdx: cellIdx };
            var borderCss = width + " " + style + " " + color;
            var hashCode = this._getImgBorderHashCode({ "border-color": color, "border-style": style, "border-width": width });
            if(xlObj.model.allowCellFormatting)
				xlObj.XLFormat._writeCssRules(hashCode, xlObj.XLFormat._getCssPropertyAsString({ "border": { "left": borderCss, "right": borderCss, "top": borderCss, "bottom": borderCss } }));
            shapeObj.bcolor = color;
            shapeObj.bstyle = style;
            shapeObj.bwidth = width;
            if(imgElem.className.match(regx))
                imgElem.className = imgElem.className.replace(regx, "");
            $(imgElem).addClass(hashCode);
            details.curClass = hashCode;
            if (!xlObj._dupDetails && !xlObj._isUndoRedo)
                xlObj._completeAction(details);
        },

        _getCellIndexFromOffset: function (top, left) {
            var i, len, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), actSheet = xlObj.getSheet(sheetIdx),
                rowIdx = 0, colIdx = 0, value = 0;
            for (i = 0, len = actSheet.rowsHeightCollection.length; i < len; i++) {
                value += actSheet.rowsHeightCollection[i];
                if (value > top) {
                    rowIdx = i;
                    break;
                }
            }
            for (i = 0, value = 0, len = actSheet.columnsWidthCollection.length; i < len; i++) {
                value += actSheet.columnsWidthCollection[i];
                if (value > left) {
                    colIdx = i;
                    break;
                }
            }
            return { rowIndex: rowIdx, colIndex: colIdx };
        },

        _getImgBorderHashCode: function (style) {
            var xlObj = this.XLObj, code = "";
            "border-width" in style && (code = parseInt(style["border-width"]) + "N");
            if(xlObj.model.allowCellFormatting && "border-style" in style) 
				code = (code + xlObj.XLFormat._getStyleCode("BorderStyle", style["border-style"]) +"N");
            "border-color" in style && (code = code + style["border-color"].replace("#", ""));
            return "e-shapebdr" + code;
        },

        _getImgStyleFromHashCode: function (code) {
            var xlObj = this.XLObj;
            var style = {}, bStyle = ["", "solid", "dashed", "dotted"];
            code = code || "";
            code = code.replace("e-shapebdr", "").split("N");
            if (code.length > 1) {
                style["border-width"] = code[0] + "px";
                style["border-style"] = bStyle[code[1]];
                style["border-color"] = "#" + code[2];
            }
            return style;
        },

        _copyImages: function (cell, pics, sheetIdx) {
            var sheet = this.XLObj.model.sheets[sheetIdx], shapeMngr = sheet._shapeMngr, i, len, picObj;
            if(sheet._importRendered)
                pics = eval(pics);
            for (i = 0, len = pics.length; i < len; i++) {
                picObj = shapeMngr.picture[pics[i]];
                this._createPicture(sheetIdx, cell, shapeMngr.sharedPics[picObj.data], picObj.top, picObj.left, picObj.width, picObj.height, pics[i], picObj.bcolor, picObj.bstyle, picObj.bwidth);
            }
        },

        _scrollShape: function (sheetIdx, $visualImg, x, y) {
            var status = { position: "none", action: "none" }, xlObj = this.XLObj, $content = xlObj._getContent(sheetIdx);
            if (((parseInt($visualImg.css('width')) + ($visualImg.offset().left - xlObj._getContent(sheetIdx).find(".e-content").offset().left)) > xlObj._getContent(sheetIdx).find(".e-content").width()) && (this._imgX <= x))
                status = { position: "horizontal", action: "Increment" };
            else if (($visualImg.offset().left - $content.find(".e-content").offset().left) <= 0)
                status = { position: "horizontal", action: "Decrement" };
            else if (((parseInt($visualImg.css('height')) + ($visualImg.offset().top - xlObj._getContent(sheetIdx).find(".e-content").offset().top)) > xlObj._getContent(sheetIdx).find(".e-content").height()) && (this._imgY <= y))
                status = { position: "vertical", action: "Increment" };
            else if (($visualImg.offset().top - $content.find(".e-content").offset().top) <= 0)
                status = { position: "vertical", action: "Decrement" };
            xlObj.XLSelection._scrollCalculation(sheetIdx, $visualImg.get(0), status);
        },

        _onKeyscrollShape: function (sheetIdx, $trgt, key) {
            var xlObj = this.XLObj, content = xlObj._getContent(sheetIdx).find(".e-content"), hScroll = xlObj._getContent(sheetIdx).find("#hscrollBar").ejScrollBar("instance"), vScroll = xlObj._getContent(sheetIdx).find("#vscrollBar").ejScrollBar("instance");
            if (content.offset().top + content[0].offsetHeight <= ($trgt.offset().top + $trgt[0].offsetHeight) && (key.keyCode === 40 || (key.keyCode === 13 && !key.shiftKey)))
                vScroll.scroll(vScroll.value() + 2, true);
            else if ((($trgt.offset().top - xlObj.model.sheets[sheetIdx]._frozenHeight) <= content.offset().top) && (key.keyCode === 38 || (key.keyCode === 13 && key.shiftKey))) {
                if (vScroll.value() - 2 >= 0)
                    vScroll.scroll(vScroll.value() - 2, true);
            }
            if (content[0].offsetWidth <= ($trgt.offset().left + $trgt[0].offsetWidth) && (key.keyCode === 39 || (key.keyCode === 9 && !key.shiftKey)))
                hScroll.scroll(hScroll.value() + 2, true);
            else if ((($trgt.offset().left - xlObj.model.sheets[sheetIdx]._frozenWidth) <= content.offset().left) && (key.keyCode === 37 || (key.keyCode === 9 && key.shiftKey))) {
                if (hScroll.value() - 2 >= 0)
                    hScroll.scroll(hScroll.value() - 2, true);
            }
        }
    };

})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.chart = function (obj) {
        this.XLObj = obj;
        this._shapeCnt = 1;
    };

    ej.spreadsheetFeatures.chart.prototype = {
        createChart: function (range, options) {
            if (!this.XLObj.model.allowCharts)
                return;
            options = options || {};
            var cnt, chartElem, chartOptions, details, cellInfo, chartModel, chartRange, xlObj = this.XLObj, type = "chart",
                sheetIdx = xlObj._getSheetIndex(options.sheetIdx), sheet = xlObj.getSheet(sheetIdx);
            if (xlObj._isUndoRedo)
                cnt = options.id.split("_")[1].replace(/[a-z]/g, '');
            range = range ? xlObj.getRangeIndices(range) : sheet.selectedRange;            
            if (!ej.isNullOrUndefined(options.top) && !ej.isNullOrUndefined(options.left))
                options.activeCell = xlObj.XLShape._getCellIndexFromOffset(options.top, options.left);
            if (!options.activeCell)
                options.activeCell = sheet._activeCell;
            if (!xlObj.isRange(range) || !options.activeCell)
                return;
            cellInfo = xlObj._getCellInfo(options.activeCell);
            options.id = cnt ? xlObj._id + "_" + type + cnt : xlObj._id + "_" + type + this._shapeCnt;
            options.type = options.type ? options.type : "column";
            options.animation = options.animation ? options.animation : false;
            options.range = range;
            options.dataSheetIdx = options.dataSheetIdx ? options.dataSheetIdx : sheetIdx;
            options.height = options.height ? options.height : xlObj.model.chartSettings.height;
            options.width = options.width ? options.width : xlObj.model.chartSettings.width;
            options.top = options.top ? options.top : cellInfo.top;
            options.left = options.left ? options.left : cellInfo.left;
            options.isRowColSwitched = options.isRowColSwitched ? options.isRowColSwitched : false;
            if (xlObj.isUndefined(options.xRange)) {
                chartRange = this._processChartrange(range, options.dataSheetIdx);
                options.xRange = chartRange.xRange;
                options.yRange = chartRange.yRange;
                options.lRange = chartRange.lRange;
            }
            chartOptions = this._processChartSeries(options);
            options.xRange = chartOptions.xRange;
            options.yRange = chartOptions.yRange;
            options.lRange = chartOptions.lRange;
            if (xlObj.getActiveSheetIndex() === options.dataSheetIdx)
                this._focusChartRange(chartOptions.xRange, chartOptions.yRange, chartOptions.lRange);
            chartElem = this._renderBaseElem(type, options.top, options.left, cnt);
            chartElem.ejChart(this._processChartOptions(options, chartOptions.series));
            chartModel = chartElem.ejChart("model");
            options.theme = chartModel.theme;
            options.title = { text: chartModel.title.text.length ? chartModel.title.text : "" };
            options.xAxis = { min: chartModel.primaryXAxis.range.min, max: chartModel.primaryXAxis.range.max, interval: chartModel.primaryXAxis.range.interval, title: { text: chartModel.primaryXAxis.title.text.length ? chartModel.primaryXAxis.title.text : "" }, range: chartOptions.xRange };
            options.yAxis = { min: chartModel.primaryYAxis.range.min, max: chartModel.primaryYAxis.range.max, interval: chartModel.primaryYAxis.range.interval, title: { text: chartModel.primaryYAxis.title.text.length ? chartModel.primaryYAxis.title.text : "" }, range: chartOptions.yRange };
            options.legend = { visible: chartModel.legend.visible, range: chartOptions.lRange };
            if (!xlObj._isUndoRedo)
                xlObj.XLShape._updateShapeMngr(options.activeCell, { chart: options }, "chart");
            if (options.theme.indexOf("dark") > -1)
                chartElem.addClass('e-ss-charttheme');
            if (xlObj.model.showRibbon && !options.hideTab)
                xlObj.XLRibbon._chartDesignTabUpdate(chartElem);
            xlObj._on(chartElem, ej.eventType.mouseDown, xlObj._mouseDownHandler);
            xlObj._on(chartElem, ej.eventType.mouseMove, xlObj._mouseMove);
            if (!sheet._importRendered && !xlObj._isUndoRedo && !xlObj._isPaste && !xlObj.XLClipboard._isShape) {
                details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "remove", options: options, range: options.range, id: options.id, position: { top: options.top, left: options.left } };
                xlObj._completeAction(details);
            }
        },

        resizeChart: function (id, height, width) {
            var obj = {}, elem = $("#" + id);
            if (elem.length) {
                if (!ej.isNullOrUndefined(height))
                    obj.height = height.toString();
                if (!ej.isNullOrUndefined(width))
                    obj.width = width.toString();
                elem.ejChart("option", { size: obj });
            }
        },

        refreshChart: function (id, options) {// can update type, enable3D, marker and theme in chart           
            if (!this.XLObj.model.allowCharts)
                return;
            var i, marker = "marker", cProp = this._getShapeObj(id, "chart");
            if (cProp) {
                for (i in options)
                    cProp[i] = options[i];
                if (options.type) {
                    options.commonSeriesOptions = { type: options.type };
                    delete options.type;
                }
                if (marker in options) {
                    if(options.commonSeriesOptions)
                        options.commonSeriesOptions.marker = { visible: options.marker.visible };
                    else
                        options.commonSeriesOptions = { marker: options.marker };
                    delete options[marker];
                }
                $("#" + id).ejChart("option", options);
            }
        },

        _processChartOptions: function (options, chartSeries) {
            var opt = { commonSeriesOptions: { type: options.type, enableAnimation: options.animation }, series: chartSeries, canResize: true, size: { height: options.height.toString(), width: options.width.toString() }, enableRotation: false, depth: 100, wallSize: 2, tilt: 0, rotation: 34, perspectiveAngle: 90, sideBySideSeriesPlacement: true, seriesRendering: this.XLObj.model.seriesRendering };
            if (options.enable3D)
                opt.enable3D = true;
            if (options.marker)
                opt.commonSeriesOptions.marker = options.marker;
            if (options.theme)
                opt.theme = options.theme;
            opt.title = options.title || {};
            opt.legend = options.legend || {};
            opt.primaryYAxis = options.primaryYAxis || { minorTicksPerInterval: 5, minorGridLines: { visible: false }, minorTickLines: { visible: false } };
            opt.primaryXAxis = options.primaryXAxis || { minorTicksPerInterval: 5, minorGridLines: { visible: false }, minorTickLines: { visible: false }, valueType: "category" };
            return opt;
        },

        _processChartrange: function (range, dataSheetIdx) {
            var xlObj = this.XLObj, xRange, yRange, lRange, minr = range[0], minc = range[1], maxr = range[2], maxc = range[3],
                trVal = xlObj.XLEdit.getPropertyValue(minr, maxc, null, dataSheetIdx), blVal = xlObj.XLEdit.getPropertyValue(maxr, minc, null, dataSheetIdx),
                tlVal = xlObj.XLEdit.getPropertyValue(minr, minc, null, dataSheetIdx), isSingleRow = minr === maxr, isSingleCol = minc === maxc;
            if (xlObj.isUndefined(tlVal) && !isSingleRow && !isSingleCol) {
                xRange = [minr + 1, minc, maxr, minc];
                yRange = [minr + 1, minc + 1, maxr, maxc];
                lRange = [minr, minc + 1, minr, maxc];
            }
            else if ((!ej.isNullOrUndefined(blVal) && !xlObj.isNumber(blVal) && !isSingleRow && !isSingleCol)) {
                if (!ej.isNullOrUndefined(trVal) && !xlObj.isNumber(trVal)) {
                    xRange = [minr + 1, minc, maxr, minc];
                    yRange = [minr + 1, minc + 1, maxr, maxc];
                    lRange = [minr, minc + 1, minr, maxc];
                }
                else {
                    xRange = [minr, minc, maxr, minc];
                    yRange = [minr, minc + 1, maxr, maxc];
                }
            }
            else {
                yRange = [minr, minc, maxr, maxc];
                if ((!ej.isNullOrUndefined(trVal) && !xlObj.isNumber(trVal) && !xlObj._isDateTime(trVal))) {
                    lRange = [minr, minc, minr, maxc];
                    yRange[0] = yRange[0] + 1;
                }
                else if (ej.isNullOrUndefined(tlVal) && (isSingleRow || isSingleCol)) {
                    lRange = [minr, minc, minr, maxc];
                    if (isSingleRow) {
                        yRange[1] = yRange[1] + 1;
                        lRange[3] = lRange[1];
                    }
                    else
                        yRange[0] = yRange[0] + 1;
                }
            }
            return { xRange: xRange, yRange: yRange, lRange: lRange };
        },

        _processChartSeries: function (options) {
            options = options || {};
            var xlObj = this.XLObj, val, len, xRange = options.xRange, yRange = options.yRange, lRange = options.lRange, xValue, yValue,
                lValue, diff, rDiff, cDiff, pArr, pObj, j, inc, i = 0, yInc = 0, sArr = [], tArr = ["value2"];
            yValue = xlObj.getRangeData(yRange, true, null, options.dataSheetIdx, false, true);
            rDiff = (yRange[2] - yRange[0]) + 1;
            cDiff = (yRange[3] - yRange[1]) + 1;
            if (options.isRowColSwitched) {
                xValue = lRange ? xlObj._toArrayData(xlObj.getRangeData(lRange, false, tArr, options.dataSheetIdx)) : this._getVirtualXValues(cDiff + 1);
                if (xRange)
                    lValue = xlObj._toArrayData(xlObj.getRangeData(xRange, false, tArr, options.dataSheetIdx));
                diff = rDiff;
            }
            else {
                xValue = xRange ? xlObj._toArrayData(xlObj.getRangeData(xRange, false, tArr, options.dataSheetIdx)) : this._getVirtualXValues(rDiff + 1);
                if (lRange)
                    lValue = xlObj._toArrayData(xlObj.getRangeData(lRange, false, tArr, options.dataSheetIdx));
                diff = cDiff;
            }
            len = xValue.length;
            inc = options.isRowColSwitched ? 1 : diff;
            while (i < diff) {
                j = 0;
                pArr = [];
                yInc = options.isRowColSwitched ? yInc : i;
                while (j < len) {
                    val = yValue[yInc];
                    pArr.push({ x: xValue[j], y: xlObj.isNumber(val) ? val : 0 });
                    yInc += inc;
                    j++;
                }
                pObj = { points: pArr };
                if (lValue)
                    pObj.name = lValue[i];
                sArr.push(pObj);
                i++;
            }
            return { series: sArr, xRange: options.isRowColSwitched ? lRange : xRange, yRange: yRange, lRange: options.isRowColSwitched ? xRange : lRange };
        },

        _getVirtualXValues: function (limit) {
            var i = 1, arr = [];
            while (i < limit) {
                arr.push(i.toString());
                i++;
            }
            return arr;
        },

        _focusChartRange: function (xRange, yRange, lRange) {
            var xlObj = this.XLObj, border = xlObj._chartBorder;
            this._clearChartRange(xlObj._arrayAsString(border));
            if (lRange)
                xlObj.XLSelection._focusBorder({ rowIndex: lRange[0], colIndex: lRange[1] }, { rowIndex: lRange[2], colIndex: lRange[3] }, border[0]);
            if (xRange)
                xlObj.XLSelection._focusBorder({ rowIndex: xRange[0], colIndex: xRange[1] }, { rowIndex: xRange[2], colIndex: xRange[3] }, border[1]);
            xlObj.XLSelection._focusBorder({ rowIndex: yRange[0], colIndex: yRange[1] }, { rowIndex: yRange[2], colIndex: yRange[3] }, border[2]);
            xlObj.getSheet(xlObj.getActiveSheetIndex())._isChartBorderDrawn = true;
        },

        _clearChartRange: function (classes) {
            var xlObj = this.XLObj;
            xlObj.XLSelection._clearBorder(classes);
            xlObj.getSheet(xlObj.getActiveSheetIndex())._isChartBorderDrawn = false;
        },

        _renderBaseElem: function (type, top, left, cnt) {
            cnt = ej.isNullOrUndefined(cnt) ? this._shapeCnt : cnt;
            var xlObj = this.XLObj, div = $("<div id='" + xlObj._id + "_" + type + cnt + "' class='e-ss-object' style='top:" + top + "px;left:" + left + "px' ></div>");
            xlObj._getContent(xlObj.getActiveSheetIndex()).find(".e-content").append(div);
            xlObj.XLShape._selectImg(div);
            if (!xlObj._isUndoRedo)
                this._shapeCnt++;
            return div;
        },

        _refreshChartElements: function (rowIdx, colIdx, sheetIdx) {
            var charts, i, chartProp, xlObj = this.XLObj, j = 1, cnt = xlObj.model.sheetCount + 1;
            while (j < cnt) {
                charts = xlObj.getSheet(j)._shapeMngr.chart;
                i = charts.length;
                if (i) {
                    while (i--) {
                        chartProp = charts[i];
                        if (chartProp.dataSheetIdx === sheetIdx && this.XLObj.inRange(chartProp.range, rowIdx, colIdx))
                            this._refreshChart(chartProp);
                    }
                }
                j++;
            }
        },

        _refreshChart: function (chartProp) {
			if($("#" + chartProp.id).length !== 0){
				var cObj = $("#" + chartProp.id).data("ejChart"), obj = { xRange: chartProp.xRange, yRange: chartProp.yRange, lRange: chartProp.lRange, isRowColSwitched: chartProp.isRowColSwitched, dataSheetIdx: chartProp.dataSheetIdx }, cOptions;
				cObj.model.series.length = 0;
				cOptions = this._processChartSeries(obj);
				cObj.option({ series: cOptions.series });
			}
        },

        _getShapeObj: function (id, type) {
            var shape, shapeMngr = this.XLObj.getSheet(this.XLObj.getActiveSheetIndex())._shapeMngr[type], i = shapeMngr.length;
            if (i) {
                while (i--) {
                    shape = shapeMngr[i];
                    if (shape.id === id)
                        return shape;
                }
            }
        },

        _validateChartRange: function (range, xRange, yRange, lRange) {
            var xlObj = this.XLObj;
            if (!yRange)
                return { msg: xlObj._getLocStr("YAxisMissing") };
            if (!(xlObj.inRange(range, yRange[0], yRange[1]) || xlObj.inRange(range, yRange[2], yRange[3])))
                return { msg: xlObj._getLocStr("InvalidYAxis") };
            if (xRange) {
                if (!(xlObj.inRange(range, xRange[0], xRange[1]) || xlObj.inRange(range, range[2], range[3])))
                    return { msg: xlObj._getLocStr("InvalidXAxis") };
                else if (xRange[3] - xRange[1] !== 0)
                    return { msg: xlObj._getLocStr("InvalidXAxisColumns") };
            }
            if (lRange && !(xlObj.inRange(range, lRange[0], lRange[1]) || xlObj.inRange(range, lRange[2], lRange[3])))
                return { msg: xlObj._getLocStr("InvalidLegend") };
            return { status: true };
        },

        _renderChartRangeDialog: function () {
            var xlObj = this.XLObj, opt = { width: "25%", showRoundedCorner: true, click: $.proxy(this._chartRangeDlgBtnClick, this) }, htmlElem = $("<div id='" + xlObj._id + "_chartrangedlg'><div class='e-dlgctndiv e-dlg-fields'><table cellpadding='0' cellspacing='0'><tr class='e-dlgtd-fields'><td>" + xlObj._getLocStr("XAxisRange") + "</td><td><input type='text' class='ejinputtext' id='" + xlObj._id + "_crxaxis'/></td></tr><tr class='e-dlgtd-fields'><td>" + xlObj._getLocStr("YAxisRange") + "</td><td><input type='text' class='ejinputtext' id='" + xlObj._id + "_cryaxis' /></td></tr><tr class='e-dlgtd-fields'><td>" + xlObj._getLocStr("LegendRange") + "</td><td><input type='text' class='ejinputtext' id='" + xlObj._id + "_crlaxis'/></td></tr></table></div><div class='e-dlg-btnfields' ><div class='e-dlg-btnctnr'><button id='" + xlObj._id + "_crok' >" + xlObj._getLocStr("Ok") + "</button><button id='" + xlObj._id + "_crcancel'>" + xlObj._getLocStr("Cancel") + "</button></div></div></div>");
            xlObj.element.append(htmlElem);
            htmlElem.ejDialog({
                width: 310,
                height: "auto",
                enableResize: false,
                showOnInit: false,
                enableModal: true,
                cssClass: "e-ss-dialog e-"+xlObj._id+"-dlg",
                title: xlObj._getLocStr("ChartRange")
            });
            $("#" + xlObj._id + "_crok").ejButton(opt);
            $("#" + xlObj._id + "_crcancel").ejButton(opt);
        },

        _chartRangeDlgBtnClick: function (args) {
            var  cid, xRange, yRange, lRange, canDlgClose = true,  xlObj = this.XLObj, sheetIdx = xlObj._getSheetIndex(xlObj.getActiveSheetIndex()), sheet = xlObj.getSheet(sheetIdx),details, selectData;
            if (args.model.text === xlObj._getLocStr("Ok")) {
                cid = document.getElementById(xlObj._id + "_chart").value;
                xRange = $("#" + xlObj._id + "_crxaxis").val();
                yRange = $("#" + xlObj._id + "_cryaxis").val();
                lRange = $("#" + xlObj._id + "_crlaxis").val();
                selectData = { xRange: xRange, yRange: yRange, lRange: lRange };
                this._processChartRange(xRange, yRange, lRange, cid);
            }
            if (canDlgClose)
                $("#" + xlObj._id + "_chartrangedlg").ejDialog("close");
            if (!sheet._importRendered && !xlObj._isUndoRedo) {
                details = { sheetIndex: sheetIdx, reqType: "shape", shapeType: "chart", action: "selectData", id: cid, prev: xlObj._selectDataval, cur:selectData };
                xlObj._completeAction(details);
            }
        },
		_processChartRange: function(xRange, yRange, lRange, cid){
		    var resp, chartData, xlObj = this.XLObj, canDlgClose, xIdx, yIdx, lIdx;
			chartData = this._getShapeObj(cid, "chart");
			if (xRange.length)
                    xIdx = xlObj.getRangeIndices(xRange);
                if (yRange.length)
                    yIdx = xlObj.getRangeIndices(yRange);
                if (lRange.length)
                    lIdx = xlObj.getRangeIndices(lRange);
                resp = this._validateChartRange(chartData.range, xIdx, yIdx, lIdx);
                if (resp.status) {
                    chartData.xRange = xIdx;
                    chartData.yRange = yIdx;
                    chartData.lRange = lIdx;
                    this._refreshChart(chartData);
                    var cModel = $("#" + cid).ejChart("model");
                    chartData.xAxis = { min: cModel.primaryXAxis.range.min, max: cModel.primaryXAxis.range.max, interval: cModel.primaryXAxis.range.interval, title: { text: cModel.primaryXAxis.title.text.length ? cModel.primaryXAxis.title.text : "" }, range: xIdx };
                    chartData.yAxis = { min: cModel.primaryYAxis.range.min, max: cModel.primaryYAxis.range.max, interval: cModel.primaryYAxis.range.interval, title: { text: cModel.primaryYAxis.title.text.length ? cModel.primaryYAxis.title.text : "" }, range: yIdx };
                    chartData.legend = { visible: cModel.legend.visible, range: lIdx };
                    if (xlObj.getActiveSheetIndex() === chartData.dataSheetIdx)
                        this._focusChartRange(xIdx, yIdx, lIdx);
                }
                else {
                    canDlgClose = false;
                    xlObj._showAlertDlg("Alert", "T-" + resp.msg, "", 400);
                }
            }
    };
})(jQuery, Syncfusion);;
/// <reference path="ej.spreadsheet.print.js" />
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.print = function (obj) {
        this.XLObj = obj;
        this._printHeight = 800;
        this._pageHeight = 820;
        this._pageWidth = 750;
        this._marginTop = 30;
        this._marginBottom = 150;
        this._printRowLabels = false;
        this._isPrintSelection = false;
        this._stRow = 0;
        this._stColumn = 0;
    };
    ej.spreadsheetFeatures.print.prototype = {
        //Printing Feature
        _printSetting: function (value) {
            this.XLObj.model.pageSize = value;
            switch (value) {
                case ej.Spreadsheet.pageSize.A4:
                    this._printHeight = 800;
                    this._pageHeight = 820;
                    this._pageWidth = 750;
                    this._marginTop = 30;
                    this._marginBottom = 150;
                    break;
                case ej.Spreadsheet.pageSize.A3:
                    this._printHeight = 1350;
                    this._pageHeight = 1330;
                    this._pageWidth = 860;
                    this._marginTop = 30;
                    this._marginBottom = 150;
                    break;
                case ej.Spreadsheet.pageSize.Letter:
                    this._printHeight = 1000;
                    this._pageHeight = 970;
                    this._pageWidth = 860;
                    this._marginTop = 30;
                    this._marginBottom = 60;
                    break;
            }
        },

        printSelection: function () {
            var xlObj = this.XLObj;
            if (!xlObj.model.printSettings.allowPrinting)
                return;
            var i, j, sheetIdx = xlObj.getActiveSheetIndex(), selectedCells = xlObj.getSheet(sheetIdx)._selectedCells, endCell = selectedCells[selectedCells.length - 1], rowCount = endCell.rowIndex, cellCount = endCell.colIndex,
                trRows, cell, selTable = ej.buildTag("table"), colgroup = ej.buildTag("colgroup"), col = xlObj._getContent(sheetIdx).find("col"), printDiv = ej.buildTag("div#printDiv");
            xlObj._detachTable = xlObj._getContent(sheetIdx).find(".e-content >table");
            xlObj._detachTable.detach();
            xlObj._isPrint = true;
            for (j = selectedCells[0].colIndex; j < cellCount; j++)
                $(col[j].outerHTML).attr("id", "cc").appendTo(colgroup);
            selTable.append(colgroup);
            for (i = selectedCells[0].rowIndex; i <= rowCount; i++) {
                cell = xlObj.getCell(i, 0);
                trRows = (cell.parent().hasClass("e-fr-hide")) ? ej.buildTag("tr.e-fr-hide", "", { height: parseInt(cell[0].parentNode.style.height) }) : ej.buildTag("tr", "", { height: parseInt(cell[0].parentNode.style.height) });
                for (j = selectedCells[0].colIndex; j <= cellCount; j++) {
                    cell = xlObj.getCell(i, j);
                    if ($(col[j]).hasClass("e-fc-hide"))
                        xlObj.addClass(cell[0], "e-fc-hide");
                    $(cell[0].outerHTML).attr('id', "s").appendTo(trRows);
                }
                selTable.append(trRows);
            }
            this._stRow = selectedCells[0].rowIndex;
            this._stCol = selectedCells[0].colIndex;
            this._isPrintSelection = true;
            var table = $(selTable[0].outerHTML).attr("id", "print");
            xlObj._getContent(sheetIdx).find(".e-content").append(xlObj._detachTable);
            xlObj._isPrint = false;
            if(this._printRowLabels)
                this._calculateTabCount(printDiv, sheetIdx, table);
            else
                this._constructSheet(printDiv, sheetIdx, table, table.get(0).rows);
        },

        printSheet: function () {
            var xlObj = this.XLObj;
            if (!xlObj.model.printSettings.allowPrinting)
                return;
            var printDiv = ej.buildTag("div#printDiv"),sheetIdx = xlObj.getActiveSheetIndex(), table = $(xlObj._getContent(sheetIdx).find(".e-content .e-table")[0].outerHTML).attr("id", "print");
            if (this._printRowLabels)
                this._calculateTabCount(printDiv, sheetIdx, table);
            else
                this._constructSheet(printDiv, sheetIdx, table, $(xlObj._getContent(sheetIdx).find(".e-content .e-table")).get(0).rows);
        },

        _calculateTabCount: function (printDiv, sheetIdx, table) {
            var cColl = [], tCount = 0, actualWidth = 0,col = $(this.XLObj.getRows(sheetIdx)[1]).parents('table').find('col');
            for (var i = 0, len = col.length; i < len; i++) {
                actualWidth += parseInt(col[i].style.width);
                if (actualWidth >= this._pageWidth) {
                    cColl.push({ tCount: tCount + 1, cellCount: i });
                    actualWidth = 0;
                }
            }
            var args = { xlObj: this.XLObj, element: cColl, table: table, printDiv: printDiv, Index: sheetIdx };
            this.XLObj._trigger("beforePrint", args);
        },

        _calculateCellRowCount: function (rows, startCell, startRow) {
            var xlObj = this.XLObj, extra, widthColl = [], heightColl = [],  actualWidth = 0, actualHeight = 0, cellCount = 12, rowCount = 40, endRow = 0, endCell = 0, len, i, j, cols = xlObj._getContent(xlObj.getActiveSheetIndex()).find("col");
            for ( i = startCell, len = rows[0].cells.length; i < len; i++) {
                actualWidth += parseInt(cols[i].style.width);
                widthColl.push(parseInt(cols[i].style.width));
                if (actualWidth >= this._pageWidth) {
                    endCell = cellCount = i;
                    break;
                }
                else {
                    extra = Math.floor((this._pageWidth - actualWidth) / xlObj.model.columnWidth);
                    endCell = cellCount = rows[startRow].cells.length + extra;
                }
            }

            for ( j = startRow, len = rows.length; j < len; j++) {
                actualHeight += parseInt(rows[j].style.height);
                heightColl.push(parseInt(rows[j].style.height));
                if (actualHeight >= this._pageHeight) {
                    endRow = rowCount = j;
                    break;
                }
                else {
                    extra = Math.floor((this._pageHeight - actualHeight) / xlObj.model.rowHeight);
                    endRow = rowCount = rows.length + extra;
                }
            }
            var cells = { startCell: startCell, endCell: endCell, startRow: startRow, endRow: endRow, cellCount: cellCount, rowCount: rowCount };
            return cells;
        },

        _constructSheet: function (printDiv, sheetIdx, table, rows) {
            var maindiv = ej.buildTag("div .e-spreadsheet", "", {}), pageCount = 0, cells = this._calculateCellRowCount(rows, 0, 0);
            var totCellCount = rows[0].cells.length, totRowCount = rows.length;
            pageCount = Math.ceil(totCellCount / cells.cellCount);
            pageCount = pageCount * (Math.ceil(totRowCount / cells.rowCount));
            if(pageCount == 1)
                cells.rowCount = rows.length;
            this._constructPrintableElement(sheetIdx, printDiv, maindiv, table, rows, totRowCount, totCellCount, pageCount, cells.cellCount, cells.rowCount, cells.startRow, cells.startCell, cells.endRow, cells.endCell);
        },

        _constructPrintableElement: function (sheetIdx, printDiv, maindiv, maintable, rows, totRowCount, totCellCount, pageCount, cellCount, rowCount, startRow, startCell, endRow, endCell) {
            var trRow, tdTemplate, isFirst = false, table, xlObj = this.XLObj, duplicateDiv = ej.buildTag("div"), pageTable, rIdx,
                isEmpty = false, emptyTr = 0, currTable, i, j, cell, len, dupTab, cols = xlObj._getContent(sheetIdx).find('col');
            for (var h = pageCount; h > 0; h--) {
                pageTable = this._calculateCellRowCount(rows, startCell, startRow);
                startCell = pageTable.startCell; endCell = pageTable.endCell; startRow = pageTable.startRow; endRow = pageTable.endRow; cellCount = pageTable.cellCount; rowCount = pageTable.rowCount;
                table = this._constructTable(sheetIdx, startCell, endCell);
                if (xlObj.model.sheets[sheetIdx].showHeader)
                    this._constructHeaders(startCell, endCell, endRow, table);
				if (this._isPrintSelection)
				    if(startRow < this._stRow)
				        rIdx = this._stRow;
				    else
				        rIdx = startRow;
                else
                    rIdx = startRow;
                for (i = startRow; i < endRow; i++) {
                    if (!ej.isNullOrUndefined(maintable[0].rows[i])) {
                        cell = $(maintable[0].rows[i].cells[0]);
                        trRow =(cell.parent().height() > 0) ? ej.buildTag("tr", "", { height: cell.parent().height() }) : ej.buildTag("tr", "", { height: xlObj.model.rowHeight });
                    }
                    else {
                        cell = $(maintable[0].rows[0].cells[0]);
                        trRow = ej.buildTag("tr", "", { height: xlObj.model.rowHeight });
                    }
                    if (cell.parent().hasClass("e-fr-hide"))
                        trRow.addClass("e-fr-hide");
                    for (j = startCell; j < endCell; j++) {
                        if (xlObj.model.sheets[sheetIdx].showHeader && !(isFirst)) {
                            tdTemplate = "<td class=\"{0}\">{1}</td>";
                            $(String.format(tdTemplate, "e-rowcell e-header", (rIdx + i + 1))).appendTo(trRow);
                            isFirst = true;
                        }
                        if ($(rows[i]).length > 0 && $(rows[i].cells[j]).length > 0) {
                            cell = $(rows[i].cells[j].outerHTML).attr("id", "ss");
                            if ($(cols[j]).hasClass("e-fc-hide"))
                                cell.addClass("e-fc-hide");
                            cell.appendTo(trRow);
                        }
                        else
                            ej.buildTag("td.e-rowcell").appendTo(trRow);
                    }
                    isFirst = false;
                    table.append(trRow);
                }
                printDiv.append(table);
                if (endRow < totRowCount) {
                    startRow = endRow;
                    endRow = endRow + rowCount;
                }
                else if (endRow >= totRowCount) {
                    if (endCell >= totCellCount)
                        break;
                    startRow = 0;
                    endRow = rowCount;
                    startCell = endCell;
                    endCell = endCell + cellCount;
                }
            }
            for (i = printDiv.find("table").length - 1; i >= 0 ; --i) {
                currTable = $(printDiv.find("table")[i]);
                if (!isEmpty)
                    this._checkEmptyRow(sheetIdx, currTable, "first");
                emptyTr = xlObj.model.sheets[sheetIdx].showHeader ? currTable.find("tr.empty").length + 1 : currTable.find("tr.empty").length;
                if ((currTable.find("tr").length !== emptyTr) || isEmpty) {
                    $(currTable.get(0).outerHTML).attr("id", "cc").appendTo(duplicateDiv);
                    isEmpty = true;
                }

            }
            dupTab = duplicateDiv.find("table");
            for (i = dupTab.length - 1 ; i >= 0 ; --i) {
                if (dupTab.length - 1 !== i) {
                    if (xlObj.model.pageSize === ej.Spreadsheet.pageSize.A4)
                        dupTab.eq(i).css({ 'margin-top': 300, 'margin-bottom': 300 });
                    if (xlObj.model.pageSize === ej.Spreadsheet.pageSize.A3)
                        dupTab.eq(i).css({ 'margin-top': 120, 'margin-bottom': 110 });
                    if (xlObj.model.pageSize === ej.Spreadsheet.pageSize.Letter)
                        if(xlObj.model.sheets[sheetIdx].showHeader)
                            dupTab.eq(i).css({ 'margin-top': 70, 'margin-bottom': 50 });
                        else
                            dupTab.eq(i).css({ 'margin-top': 0, 'margin-bottom': 50 });
                }
                else if (dupTab.length - 1 === i) {
                    if(!xlObj.model.sheets[sheetIdx].showHeader)
                        dupTab.eq(i).css({ 'margin-top': 30, 'margin-bottom': 0 });
                    else
                        dupTab.eq(i).css({ 'margin-top': 30, 'margin-bottom': 20 });
                }

                if (i === 0) {
                    if (dupTab.length > 1)
                        dupTab.eq(i).css({ 'margin-top': 120, 'margin-bottom': 0 });
                    this._checkEmptyRow(sheetIdx, dupTab.eq(i), "last");
                }
                $(dupTab[i].outerHTML).attr("id", "cc").appendTo(maindiv);
            }
            var links = $("head").find("link"), stylesLink = [];
            for ( i = 0, len = links.length; i < len; i++) {
                if (links[i].href.indexOf("ej.pivotintegration.css") > -1) {
                    stylesLink.push(links[i]);
                    break;
                }
            }
            xlObj._trigger("_pivotPrint", { element: maindiv });
            this._isPrintSelection = false;
            this.print(maindiv, stylesLink, sheetIdx);
        },

        _checkEmptyRow: function (sheetIdx, currTable, tablePosn) {
            var emptyTd, emptyCnt, curTr = currTable.find("tr"), unformatted, unBordered;
            for (var j = curTr.length - 1; j >= 0 ; --j) {
				emptyTd = $(curTr[j]).find(".e-rowcell").length;
                emptyCnt = $(curTr[j]).find(".e-rowcell:empty").length;
                emptyCnt = this.XLObj.model.sheets[sheetIdx].showHeader ? emptyCnt + 1 : emptyCnt;
				unformatted = $(curTr[j]).find('td[class *= "e-format"]').length;
				unBordered = $(curTr[j]).find('td[class *= "e-border"]').length;
                if ((emptyCnt === emptyTd) && unformatted < 1 && unBordered < 1)
                    $(curTr[j]).addClass("empty");
                else {
                    $(curTr[j]).addClass("nonempty");
                    break;
                }
            }
            if (tablePosn === "last") {
                currTable.find("tr.empty").find("td").removeClass("e-rowcell");
                currTable.find("tr.empty").find("td.e-header").text("");
            }
        },

        _constructTable: function (sheetIdx, startCell, endCell) {
            var colgroup = ej.buildTag("colgroup"), xlObj = this.XLObj, table = ej.buildTag("table.e-table", "", { height: this._printHeight, 'margin-top': this._marginTop, 'margin-bottom': this._marginBottom}),
                col = xlObj._getContent(xlObj.getActiveSheetIndex()).find("col"), newcol;
            if (xlObj.model.sheets[sheetIdx].showHeader) {
                newcol = "<col class=\"{0}\" style= \"width: 30px;\" >{1}</col>";
                $(newcol).appendTo(colgroup);
            }
            for (var j = startCell; j < endCell; j++)
                if ($(col[j]).length > 0)
                    $(col[j].outerHTML).attr("id", "cc").appendTo(colgroup);
                else {
                    newcol = "<col class=\"{0}\" style= \"width: 60px;\" >{1}</col>";
                    $(newcol).appendTo(colgroup);
                }
            table.append(colgroup);
            return table;
        },

        _constructHeaders: function (startCell, endCell, endRow, table) {
            var trRow = ej.buildTag("tr", "", { height: 30 }), tdTemplate, divTemplate, divcell, divCellData, tdCellData, xlObj = this.XLObj, isFirst = false,
                cols = xlObj._getContent(xlObj.getActiveSheetIndex()).find('col'), rClass = "e-rowcell";
            if (this._isPrintSelection) {
                if (startCell < this._stCol) {
                    endCell = endCell + (this._stCol - startCell);
                    startCell = this._stCol;
                }
            }
            tdTemplate = "<td class=\"{0}\" style= \"width: 30px;\" >{1}</td>";
            divTemplate = "<div class = \"{0}\" > {1}</div>";
            for (var i = startCell; i <= endCell; i++) {
                if (!isFirst) {
                    divcell = "";
                    isFirst = true;
                }
                else {
                    divcell = xlObj._generateHeaderText(xlObj.getActiveSheetIndex(), i);
                    rClass = ($(cols[i - 1]).hasClass("e-fc-hide")) ? "e-rowcell e-fc-hide" : "e-rowcell e-headercell";
                }
                divCellData = String.format(divTemplate, "e-headercelldiv", divcell);
                tdCellData = String.format(tdTemplate, rClass, divCellData);
                $(trRow[0]).append(tdCellData);
            }
            table.append(trRow);
        },

        print: function (element, links, sheetIdx) {
            var xlObj = this.XLObj;
            if (!xlObj.model.printSettings.allowPrinting)
                return;
            var printWin = window.open('', 'print', "'height=452,width=1024,tabbar=no");
			if(xlObj.isUndefined(printWin))
				return;
			var styletext, sty, a, $div = ej.buildTag("div"), elementClone = element.clone();
			$div.append(elementClone);
			printWin.document.write("<!DOCTYPE html>");
			if (xlObj._browserDetails.name === "msie") {
                a = "";
                if (links.length > 0)
                    $(links).each(function (index, obj) {
                        $(obj).attr("href", obj.href);
                        a += obj.outerHTML;
                    });
                printWin.document.write('<html><head></head><body>' + a + $div[0].innerHTML + '</body></html>');
            }
            else {
                a = "";
                printWin.document.write("<html><head>");
                if (links.length > 0)
                    $(links).each(function (index, obj) {
                        $(obj).attr("href", obj.href);
                        a += obj.outerHTML;
                    });
                printWin.document.writeln(a + "</head><body>");
                printWin.document.writeln($div[0].innerHTML + "</body></html>");
            }
            $(printWin.document.head).append('<style></style>');
			if (xlObj.model.sheets[sheetIdx].showGridlines)
                styletext = ".e-spreadsheet .e-rowcell {border: 1px solid #000000;line-height: 14px;width: auto;cursor:cell;overflow:hidden;white-space: pre;}.e-spreadsheet .e-table{border-collapse: collapse}";
            else
                styletext = ".e-spreadsheet .e-rowcell {line-height: 14px;width: auto;cursor:cell;overflow:hidden;white-space: pre;}.e-spreadsheet .e-table{border-collapse: collapse}";
            styletext = styletext.concat($(document).find("style#" + this.XLObj._id + "_sscustomformat").text());
            sty = $(printWin.document.head).find("style");
            styletext = styletext.concat(".e-spreadsheet .e-fr-hide { display: none !important;}.e-spreadsheet .e-fc-hide { display: none !important;}.e-spreadsheet .e-sswraptext {word-break: break-all; white-space: normal;}.e-spreadsheet .e-headercell{text-align:center;}.e-spreadsheet .e-ralign{text-align:right;}");
            sty.append(styletext);
            printWin.focus();
            $("#print").remove();
            setTimeout(function () {
                if (!ej.isNullOrUndefined(printWin.window)) {
					if(!ej.isNullOrUndefined(printWin.location)){
						printWin.print();
						printWin.location.reload();
					}
                    setTimeout(function () { printWin.close(); }, 2000);
                }
            }, 2000);
        }
    };
})(jQuery, Syncfusion);;
(function ($, ej, undefined) {
    ej.spreadsheetFeatures = ej.spreadsheetFeatures || {};

    ej.spreadsheetFeatures.scroller = function (obj) {
        this.XLObj = obj;
    };

    ej.spreadsheetFeatures.scroller.prototype = {
        _getColWidths: function (sheetIdx, colIdx) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), trs = xlObj.getRows(sheetIdx)[1], col = xlObj._getContent(sheetIdx).find('col');
            if (!ej.isNullOrUndefined(trs)) {
                for (var i = colIdx, len = trs[0].cells.length; i < len ; i++) {
                    if (!xlObj.model.scrollSettings.allowVirtualScrolling && xlObj.getFrozenColumns(sheetIdx))
    					sheet._cWidthCollection[i] = {idx: i, value:  trs[0].cells[i].offsetLeft};
                    sheet._colWidthCollection[i] = { idx: i, value: sheet._colWidthCollection[i - 1].value + parseInt(col[i - 1].style.width)};
				}
				sheet._colWidthCollection[sheet._colWidthCollection.length - 1].value = sheet._colWidthCollection[sheet._colWidthCollection.length - 2].value + parseInt(col[col.length - 1].style.width);
			}
            return xlObj._colWidthCollection;
        },

        _getRowHeights: function (sheetIdx, rowIdx) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), trs = xlObj.getRows(sheetIdx)[1];
            if (!ej.isNullOrUndefined(trs)) {
                for (var i = rowIdx, len = trs.length; i < len ; i++) {
                    if (!xlObj.model.scrollSettings.allowVirtualScrolling && xlObj.getFrozenRows(sheetIdx))
					    sheet._rHeightCollection[i] = {idx: i, value:  trs[i].offsetTop};
                    sheet._rowHeightCollection[i] = { idx: i, value: sheet._rowHeightCollection[i - 1].value + parseInt(trs[i - 1].style.height) };
				}
				sheet._rowHeightCollection[sheet._rowHeightCollection.length - 1].value = sheet._rowHeightCollection[sheet._rowHeightCollection.length - 2].value + parseInt(trs[trs.length - 1].style.height);
            }
            return xlObj._rowHeightCollection;
        },

        _createScroller: function (sheetIdx, settings) {
            var xlObj = this.XLObj, $content = xlObj._getContent(sheetIdx), vscroll = ej.buildTag("div#vscrollBar"), hscroll = ej.buildTag("div#hscrollBar");
            $content.append(vscroll).append(hscroll);
            $content.find(".e-content").width(settings.width - 18).height(settings.height - 19); // 18 for scrollbar width, height
            vscroll.ejScrollBar({
                orientation: "vertical",
                height: settings.height - 18,
                minimum: 0,
                viewportsize: settings.height,
                infiniteScrolling: true,
                scroll: $.proxy(this._scrollSpreadY, this)
            });
            hscroll.ejScrollBar({
                orientation: "horizontal",
                width: settings.width - 17,
                minimum: 0,
                infiniteScrolling: true,
                viewportsize: settings.width,
                scroll: $.proxy(this._scrollSpreadX, this)
            });
            this._refreshScroller(sheetIdx, "initial", "all");
        },

        _hScroller: function (sheetIdx) {
            return this.XLObj._getContent(sheetIdx).find("#hscrollBar").data("ejScrollBar");
        },

        _vScroller: function (sheetIdx) {
            return this.XLObj._getContent(sheetIdx).find("#vscrollBar").data("ejScrollBar");
        },
		
        _refreshScroller: function (sheetIdx, status, type) {
            var xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), hScroll = this._hScroller(sheetIdx), noOfCol, noOfRow, colWt = sheet.columnWidth, rowHt = sheet.rowHeight, vScroll = this._vScroller(sheetIdx), modelWt,
				scrollWt, modelHt, scrollHt, content = xlObj._getContent(sheetIdx);
            /* ----------- Horizontal Scrollbar--------- */
            if (type === "all" || type === "horizontal") {
                modelWt = content.find('.e-content').width();
                scrollWt = (sheet._colWidthCollection[sheet._colWidthCollection.length - 1].value + 18) - sheet._frozenWidth;
				scrollWt = scrollWt - sheet._hiddenWidth;
				if (scrollWt < modelWt + 2 * colWt) {
					noOfCol =  Math.floor(((modelWt + (colWt * 5)) - scrollWt) / colWt);
                    for (var i = 0; i < noOfCol; i++){
						this._createNewColumn(sheetIdx, {rowIndex: -1, colIndex: -1}, {rowIndex: -1, colIndex: -1}, "insert");
						scrollWt = scrollWt + colWt;
					}
				}
                hScroll._scrollData.handleSpace = modelWt - (2 * hScroll.model.buttonSize);
                hScroll._scrollData.handle = Math.floor((modelWt - 18) / scrollWt * hScroll._scrollData.handleSpace);
                (hScroll._scrollData.handle < 15) && (hScroll._scrollData.handle = 15);
                hScroll.model.maximum = scrollWt - modelWt;
                hScroll._scrollData.scrollable = scrollWt - modelWt;
                hScroll._scrollData.onePx = hScroll._scrollData.scrollable / (hScroll._scrollData.handleSpace - hScroll._scrollData.handle);
                hScroll["e-hhandle"].width(hScroll._scrollData.handle);
                hScroll.model.scrollLeft = hScroll._scrollData.handleSpace - hScroll._scrollData.handle;
            }
            if (type === "all" || type === "vertical") {
                /* ----------- Vertical Scrollbar--------- */
                modelHt = content.find('.e-content').height();
                scrollHt = ((sheet._rowHeightCollection[sheet._rowHeightCollection.length - 1].value + 18) - sheet._frozenHeight) + sheet._fpHiddenHeight;
				scrollHt = scrollHt - sheet._hiddenHeight;
				if (scrollHt < modelHt + 2 * rowHt) {
					noOfRow =  Math.floor(((modelHt + (rowHt * 5)) - scrollHt) / rowHt);
                    for (var i = 0; i < noOfRow; i++){
						this._createNewRow(sheetIdx,  -1, -1, "insert");
						scrollHt = scrollHt + rowHt;
					}
				}
				vScroll._scrollData.handleSpace = modelHt - (2 * vScroll.model.buttonSize);
                vScroll._scrollData.handle = Math.floor((modelHt - 18) / scrollHt * vScroll._scrollData.handleSpace);
                (vScroll._scrollData.handle < 15) && (vScroll._scrollData.handle = 15);
                vScroll.model.maximum = scrollHt - modelHt;
                vScroll._scrollData.scrollable = scrollHt - modelHt;
                vScroll._scrollData.onePx = vScroll._scrollData.scrollable / (vScroll._scrollData.handleSpace - vScroll._scrollData.handle);
                vScroll["e-vhandle"].height(vScroll._scrollData.handle);
                vScroll.model.scrollTop = vScroll._scrollData.handleSpace - vScroll._scrollData.handle;
            }
        },


        _changeTopSpread: function (vScroll, d, step, source) {
            var start = vScroll.value(), t = start + step;
            d.step = step;
            (!d.enableRTL ? t > d.scrollable : t < d.scrollable) && (t = d.scrollable);
            (!d.enableRTL ? t < 0 : t > 0) && (t = 0);
            vScroll["scroll"](t, source);
        },

        _spreadMouseWheel: function (e) {
            var xlObj = this.XLObj;
            e.stopImmediatePropagation();
            e.preventDefault();
            if (xlObj.element.find(".e-excelfilter.e-dlgcontainer").is(":visible") || xlObj._isSubMenuOpen() || xlObj._isSheetRename)
                return;
            var delta = 0, ori = e, vScroll = this._vScroller(xlObj.getActiveSheetIndex());
            e = e.originalEvent;
            if (e.wheelDelta) {
                delta = -e.wheelDelta / 120;
                if (window.opera) {
                    if (parseFloat(window.opera.version, 10) < 10)
                        delta = -delta;
                }
            } else if (e.detail) delta = e.detail / 3;
            if (!delta) return;
            if (this._changeTopSpread(vScroll, vScroll._scrollData, (delta * vScroll._scrollData.scrollOneStepBy), "wheel"))
                e.preventDefault ? e.preventDefault() : ori.preventDefault();
        },

        _scrollSpreadX: function (args) {
			this._spreadHS(args);
		},

		_scrollDirection: function(scroller, pix){
            var xlObj = this.XLObj, scroller = xlObj._getContent(xlObj.getActiveSheetIndex()).find('#' + scroller).ejScrollBar('instance');
            if(scroller.value() < pix)
                scroller._scrollData.step = 1;
            else if(scroller.value() > pix)
                scroller._scrollData.step = -1;
            scroller.scroll(pix);
        },

		_spreadHS: function(args) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), sheet = xlObj.getSheet(sheetIdx), pix, hScroll = this._hScroller(sheetIdx), content = xlObj._getContent(sheetIdx), colgrp = content.find('col'), sheet = xlObj.model.sheets[sheetIdx];
            args["reachedEnd"] = Math.ceil(parseFloat(hScroll.element.find(".e-hhandle").css('left'))) + Math.ceil(parseFloat(hScroll.element.find(".e-hhandle").width())) >= xlObj.element.find(".e-hhandlespace").width() - 2;
            pix = args.scrollLeft;
            xlObj.model.enableContextMenu && xlObj.XLCMenu.hideCMenu();
            if (xlObj.getFrozenColumns(sheetIdx) > 0 && (!xlObj.XLCellNav._scrollReverse)) {
                args.cancel = true;
                if (args.scrollData != null && args.scrollData.dimension != "height") {
                    if (!args.reachedEnd || args.scrollData.step < 0) {
                        var indexes = xlObj.XLFreeze._getLeftColumn(pix, sheetIdx), currentLeftColumn = indexes.imaginaryIndex, hdrcolgrp = xlObj._getJSSheetHeader(sheetIdx).find("col"),
							tblWt = content.find('.e-table').height(), contentWt = content.height();
                        xlObj.XLFreeze._changehHandlerPosition(pix);
                        if (currentLeftColumn > sheet._currentLeftFrozenColumn) {
                            if (content.find(".e-content").width() <= (content.find(".e-table").width() - xlObj.model.columnWidth))
                                xlObj.XLFreeze._showHideColumn(sheetIdx, sheet._currentLeftFrozenColumn, currentLeftColumn, "hide", sheet._colWidthCollection[indexes.i].value, colgrp, hdrcolgrp, contentWt, tblWt);
                        }
                        else if (currentLeftColumn < sheet._currentLeftFrozenColumn)
                            xlObj.XLFreeze._showHideColumn(sheetIdx, currentLeftColumn, sheet._currentLeftFrozenColumn + 1, "show", sheet._colWidthCollection[indexes.i].value, colgrp, hdrcolgrp, contentWt, tblWt);
                    }
                }
            }
            if (xlObj.model.scrollSettings.scrollMode === ej.Spreadsheet.scrollMode.Infinite && args.reachedEnd && args.scrollData.step > -1 && !(xlObj.XLCellNav && xlObj.XLCellNav._scrollReverse)) {
 				var hold = false, btnClk = false, proxy = this, interval;
				if (!xlObj.isUndefined(args.scrollData.target)) {
					interval = setInterval(function (e) {
						hold = true;
						for(var i = 0; i < 5; i++) {
							proxy._scrollRight(sheetIdx);
							((xlObj.getFrozenColumns(sheetIdx) < 1) && (xlObj.getFrozenRows(sheetIdx) > 0)) && xlObj.XLFreeze._refreshFreezeDiv(sheetIdx, "horizontal", 20 * xlObj.getSheet(sheetIdx).columnWidth);
							if (xlObj.getFrozenColumns(sheetIdx) > 0 && (content.find(".e-content").width() <= (content.find(".e-table").width() - xlObj.model.sheets[sheetIdx]._firstCol.value)))
								xlObj.XLFreeze._hideColumn(sheetIdx);
						}
						pix = (hScroll._scrollData.handleSpace - hScroll._scrollData.handle) * hScroll._scrollData.onePx;
						proxy._refreshScroller(sheetIdx, "refresh", "horizontal");
						hScroll.element.find(".e-hhandle").css("left", (hScroll._scrollData.handleSpace - (hScroll._scrollData.handle + 1)));
						hScroll.scroll(pix);
						clearInterval(interval);
						return;						
					}, 1000);
					hScroll.element.find(".e-hdown")[0].addEventListener("mouseup", function (e) {
						clearInterval(interval);
						return;
					});
					document.addEventListener("mouseup", function (e) {
						clearInterval(interval);
						return;
					});
					btnClk = true;
				}
				args.cancel = true;
				if(!hold || !btnClk) {
					this._scrollRight(sheetIdx);
					((xlObj.getFrozenColumns(sheetIdx) < 1) && (xlObj.getFrozenRows(sheetIdx) > 0)) && xlObj.XLFreeze._refreshFreezeDiv(sheetIdx, "horizontal", xlObj.getSheet(sheetIdx).columnWidth);
					if (xlObj.getFrozenColumns(sheetIdx) > 0 && (content.find(".e-content").width() <= (content.find(".e-table").width() - xlObj.model.sheets[sheetIdx]._firstCol.value)))
						xlObj.XLFreeze._hideColumn(sheetIdx);
					pix = (hScroll._scrollData.handleSpace - hScroll._scrollData.handle) * hScroll._scrollData.onePx;
					proxy._refreshScroller(sheetIdx, "refresh", "horizontal");
					hScroll.element.find(".e-hhandle").css("left", (hScroll._scrollData.handleSpace - (hScroll._scrollData.handle + 1)));
					hScroll.value(pix);
				}
            }
            sheet._scrollLeft = args.scrollLeft = pix;
            if(xlObj.model.allowKeyboardNavigation)
				xlObj.XLCellNav._scrollReverse = false;
            if(xlObj.getFrozenColumns(sheetIdx) < 1) {
				content.find(".e-content").scrollLeft(args.scrollLeft, args.scrollData.onePx);
				hScroll.value(args.scrollLeft);
				if(xlObj.model.allowAutoFill && xlObj._getJSSheetHeader(sheetIdx)[0].querySelectorAll("th.e-colselected").length < 1)
					xlObj.XLDragFill.positionAutoFillElement();
			}
            this._getFirstColumn(sheetIdx);
            if (xlObj.getFrozenColumns(sheetIdx) > 0) {
                xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
                xlObj.XLSelection.refreshSelection();
                xlObj.model.allowAutoFill &&  xlObj.XLDragFill.positionAutoFillElement();
                (colgrp.get(xlObj.getActiveCellElem(sheetIdx).index()).className.indexOf("e-fc-hide") > -1) && xlObj.model.allowAutoFill &&  xlObj.XLDragFill.hideAutoFillElement();
            }
        },

        _scrollSpreadY: function (args) {
            this.XLObj.model.scrollSettings.allowVirtualScrolling ? this._scrollHandler(args) : this._spreadVS(args);
        },

        _scrollHandler: function (args) {
            var contTBody, rowHdrTBody, diff, rangeData, idx, height = 0, isScrollDown = true, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(),
                sheet = xlObj.getSheet(sheetIdx), prop = ["value2", "comment", "hyperlink", "format", "border", "type", "isFilterHide", "filterState", "isMHide", "isFHide", "mSpan", "isFilterHeader", "text", "general", "cFormatRule", "wrap", "wrapRow", "rule"];
            if (args.scrollTop != sheet._scrollTop) {
                isScrollDown = args.scrollTop >= sheet._scrollTop;
                rowHdrTBody = xlObj._getRowHdrTBody(sheetIdx);
                contTBody = xlObj._getContTBody(sheetIdx);
                if (isScrollDown) {
                    if (args.scrollTop >= sheet._virtualContTBodyOffset.bottom && args.scrollTop <= sheet._virtualBottomTBodyOffset.bottom) {
                        height = sheet._virtualTopTBodyOffset.bottom - sheet._virtualTopTBodyOffset.top;
                        sheet._virtualTopTBodyOffset.top = sheet._virtualContTBodyOffset.top;
                        sheet._virtualTopTBodyOffset.bottom = sheet._virtualContTBodyOffset.bottom;
                        sheet._virtualContTBodyOffset.top = sheet._virtualBottomTBodyOffset.top;
                        sheet._virtualContTBodyOffset.bottom = sheet._virtualBottomTBodyOffset.bottom;
                        sheet._virtualContBlockIdx++;
                        if (sheet._virtualContBlockIdx === sheet._virtualBlockCnt) {
                            sheet._virtualBottomTBodyOffset.top = sheet._virtualBottomTBodyOffset.bottom = 0;
                            sheet._isVirtualEndReached = true;
                        }
                        else {
                            sheet._virtualBottomTBodyOffset.top = sheet._virtualBottomTBodyOffset.bottom;
                            sheet._virtualBottomTBodyOffset.bottom = xlObj._getRowOffsetBottom(((sheet._virtualContBlockIdx * sheet._virtualBlockRowCnt) + (sheet._virtualBlockRowCnt - 1)), sheetIdx);
                            rangeData = xlObj.getRangeData(null, null, prop, sheetIdx, false, false, true, sheet._virtualContBlockIdx * sheet._virtualBlockRowCnt, sheet._virtualBlockRowCnt, true);
                            rowHdrTBody.append(xlObj._renderRowHdr(rangeData.rowIdx, sheetIdx));
                            contTBody.append(xlObj._renderData(rangeData, sheetIdx));
                            sheet._virtualRowIdx = sheet._virtualRowIdx.concat(rangeData.rowIdx);
                        }
                        if (!sheet._isVirtualTopReached) {
                            xlObj._removeRange(0, sheet._virtualBlockRowCnt);
                            sheet._virtualRowIdx.splice(0, sheet._virtualBlockRowCnt);
                            sheet._virtualTopTBodyHgt += height;
                        }
                        sheet._virtualBottomTBodyHgt = sheet._virtualTBodyHgt - (sheet._virtualTopTBodyHgt + contTBody.height());
                        xlObj._refreshTBodiesHgt();
                        sheet._isVirtualTopReached = false;
                    }
                    else if (!sheet._isVirtualEndReached && args.scrollTop >= sheet._virtualBottomTBodyOffset.bottom)
                        xlObj._refreshViewport(args.scrollTop, sheetIdx);
                }
                else {
                    if ((args.scrollTop + args.model.viewportsize) < sheet._virtualContTBodyOffset.top && (args.scrollTop + args.model.viewportsize) >= sheet._virtualTopTBodyOffset.top) {
                        height = sheet._virtualBottomTBodyOffset.bottom - sheet._virtualBottomTBodyOffset.top;
                        sheet._virtualBottomTBodyOffset.top = sheet._virtualContTBodyOffset.top;
                        sheet._virtualBottomTBodyOffset.bottom = sheet._virtualContTBodyOffset.bottom;
                        sheet._virtualContTBodyOffset.top = sheet._virtualTopTBodyOffset.top;
                        sheet._virtualContTBodyOffset.bottom = sheet._virtualTopTBodyOffset.bottom;
                        sheet._virtualContBlockIdx--;
                        sheet._virtualTopTBodyOffset.bottom = sheet._virtualContTBodyOffset.top;
                        if (sheet._virtualContBlockIdx === 1) {
                            sheet._virtualTopTBodyOffset.top = 0;
                            sheet._isVirtualTopReached = true;
                        }
                        else {
                            sheet._virtualTopTBodyOffset.top = xlObj._getRowOffsetTop(sheet, (sheet._virtualContBlockIdx - 2) * sheet._virtualBlockRowCnt);
                            rangeData = xlObj.getRangeData(null, null, prop, sheetIdx, false, false, true, (sheet._virtualContBlockIdx - 2) * sheet._virtualBlockRowCnt, sheet._virtualBlockRowCnt, true);
                            rowHdrTBody.prepend(xlObj._renderRowHdr(rangeData.rowIdx, sheetIdx));
                            contTBody.prepend(xlObj._renderData(rangeData, sheetIdx));
                            sheet._virtualRowIdx = rangeData.rowIdx.concat(sheet._virtualRowIdx);
                        }
                        if (!sheet._isVirtualEndReached) {
                            xlObj._removeRange(sheet._virtualContBlockIdx === 1 ? 2 * sheet._virtualBlockRowCnt : 3 * sheet._virtualBlockRowCnt, sheet._virtualBlockRowCnt);
                            sheet._virtualRowIdx.splice(sheet._virtualContBlockIdx === 1 ? 2 * sheet._virtualBlockRowCnt : 3 * sheet._virtualBlockRowCnt, sheet._virtualBlockRowCnt);
                            sheet._virtualBottomTBodyHgt += height;
                        }
                        sheet._virtualTopTBodyHgt = sheet._virtualTBodyHgt - (sheet._virtualBottomTBodyHgt + contTBody.height());
                        xlObj._refreshTBodiesHgt();
                        sheet._isVirtualEndReached = false;
                    }
                    else if (!sheet._isVirtualTopReached && (args.scrollTop + args.model.viewportsize) <= sheet._virtualTopTBodyOffset.top)
                        xlObj._refreshViewport(args.scrollTop, sheetIdx);
                }
                xlObj._getContent(sheetIdx).find(".e-content").scrollTop(args.scrollTop);
                sheet._scrollTop = args.scrollTop;
            }
            xlObj.XLComment._updateCmntArrowPos(xlObj._getContent(sheetIdx).find('.e-commentcell'));
            var clKeys, rwKeys, isFitWidth;
            clKeys = xlObj.getObjectKeys(xlObj._dataContainer.sheets[sheetIdx][0]);
            xlObj._dupDetails = true;
            for (var i = 0, len = clKeys.length; i < len; i++) {
                fitWidth = xlObj.XLEdit.getPropertyValue(0, clKeys[i], "isFitWidth");
                if (!ej.isNullOrUndefined(fitWidth))
                    xlObj.XLResize._fitWidth(clKeys[i]);
            }
        },
		
		_spreadVS: function(args) {
            var xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex(), pix, vScroll = this._vScroller(sheetIdx), sheet = xlObj.model.sheets[sheetIdx], content = xlObj._getContent(sheetIdx);
            args["reachedEnd"] = Math.ceil(parseFloat(vScroll.element.find(".e-vhandle").css('top'))) + Math.ceil(parseFloat(vScroll.element.find(".e-vhandle").height())) >= vScroll.element.find(".e-vhandlespace").height() - 2;
            pix = args.scrollTop;
            xlObj.model.enableContextMenu && xlObj.XLCMenu.hideCMenu();
            if (xlObj.getFrozenRows(sheetIdx) > 0 && !(xlObj.XLCellNav && xlObj.XLCellNav._scrollReverse)) {
                if (args.scrollData != null && args.scrollData.dimension != "width") {
                    args.cancel = true;
                    xlObj.XLFreeze._changevHandlerPosition(pix);
                    if (!args.reachedEnd || args.scrollData.step < 0) {
                        var indexes = xlObj.XLFreeze._getTopRow(pix, sheetIdx), currentTopRow = indexes.imaginaryIndex, rows = xlObj.getRows(sheetIdx), contentHt = xlObj._getContent(sheetIdx).find(".e-content").height(), tblHt = xlObj._getContent(sheetIdx).find(".e-table").height();
                        if (currentTopRow > sheet._currentTopFrozenRow) {
                            if (content.find(".e-content").height() <= (content.find(".e-table").height() - xlObj.model.rowHeight))
                                xlObj.XLFreeze._showHideRow(sheetIdx, sheet._currentTopFrozenRow, currentTopRow, "hide", sheet._rowHeightCollection[indexes.i].value, rows, contentHt, tblHt);
                        }
                        else if (currentTopRow < sheet._currentTopFrozenRow)
                            xlObj.XLFreeze._showHideRow(sheetIdx, currentTopRow, sheet._currentTopFrozenRow + 1, "show", sheet._rowHeightCollection[indexes.i].value, rows, contentHt, tblHt);
                    }
                }
            }
            if (xlObj.model.scrollSettings.scrollMode === ej.Spreadsheet.scrollMode.Infinite && args.reachedEnd && args.scrollData.step > -1 && !(xlObj.XLCellNav && xlObj.XLCellNav._scrollReverse)) {
				var hold = false, btnClk = false, proxy = this, interval;
				if (!xlObj.isUndefined(args.scrollData.target) && xlObj._hasClass(args.scrollData.target, "e-vdown")) {
					interval = setInterval(function (e) {
						hold = true;
						for(var i = 0; i < 20; i++) {
							proxy._scrollBottom(sheetIdx);
							((xlObj.getFrozenRows(sheetIdx) < 1) && (xlObj.getFrozenColumns(sheetIdx) > 0)) && xlObj.XLFreeze._refreshFreezeDiv(sheetIdx, "vertical", xlObj.getSheet(sheetIdx).rowHeight);
							(xlObj.getFrozenRows(sheetIdx) > 0 && content.find(".e-content").height() <= content.find(".e-table").height() - sheet._firstRow.value) && xlObj.XLFreeze._hideRow(sheetIdx);
						}
						pix = (vScroll._scrollData.handleSpace - vScroll._scrollData.handle) * vScroll._scrollData.onePx;
						proxy._refreshScroller(sheetIdx, "refresh", "vertical");
						vScroll.element.find(".e-vhandle").css("top", (vScroll._scrollData.handleSpace - (vScroll._scrollData.handle)));
						vScroll.scroll(pix);	
						clearInterval(interval);
						return;						
					}, 1000);
					vScroll.element.find(".e-vdown")[0].addEventListener("mouseup", function (e) {
						clearInterval(interval);
						return;
					});
					document.addEventListener("mouseup", function (e) {
						clearInterval(interval);
						return;
					});
					btnClk = true;
				}
				args.cancel = true;
				if(!hold || !btnClk) {
					this._scrollBottom(sheetIdx);
					((xlObj.getFrozenRows(sheetIdx) < 1) && (xlObj.getFrozenColumns(sheetIdx) > 0)) && xlObj.XLFreeze._refreshFreezeDiv(sheetIdx, "vertical", xlObj.getSheet(sheetIdx).rowHeight);
					(xlObj.getFrozenRows(sheetIdx) > 0 && content.find(".e-content").height() <= content.find(".e-table").height() - sheet._firstRow.value) && xlObj.XLFreeze._hideRow(sheetIdx);
					pix = (vScroll._scrollData.handleSpace - vScroll._scrollData.handle) * vScroll._scrollData.onePx;
					proxy._refreshScroller(sheetIdx, "refresh", "vertical");
					vScroll.element.find(".e-vhandle").css("top", (vScroll._scrollData.handleSpace - (vScroll._scrollData.handle)));
					vScroll.value(pix);
				}			
            }
            args.scrollTop = sheet._scrollTop = pix;
			if(xlObj.model.allowKeyboardNavigation)
				xlObj.XLCellNav._scrollReverse = false;
            (xlObj.getFrozenRows(sheetIdx) < 1) && content.find(".e-content").scrollTop(args.scrollTop, vScroll._scrollData.onePx);
			if(xlObj.model.allowAutoFill && xlObj._getJSSheetRowHeaderContent(sheetIdx)[0].querySelectorAll("td.e-rowselected").length < 1)
				xlObj.XLDragFill.positionAutoFillElement();
            this._getFirstRow(sheetIdx);
            if (xlObj.getFrozenRows(sheetIdx) > 0) {
                xlObj.model.allowComments && xlObj.XLComment._updateCmntArrowPos();
                xlObj.XLSelection.refreshSelection();
                xlObj.model.allowAutoFill && xlObj.XLDragFill.positionAutoFillElement();
                (xlObj.getActiveCellElem(sheetIdx).parent()[0].className.indexOf("e-fr-hide") > -1) && xlObj.model.allowAutoFill && xlObj.XLDragFill.hideAutoFillElement();
            }
        },

        _scrollX: function (sheetIdx) {
            var xlObj = this.XLObj, hScroll = this._hScroller(sheetIdx), pix, colWtColl = [1], colgroup = xlObj._getContent(sheetIdx).find("tbody").eq(0).prev();
            (xlObj.model.scrollSettings.scrollMode === ej.Spreadsheet.scrollMode.Infinite) && this._scrollRight(sheetIdx);
            hScroll.model.maximum = hScroll.model.maximum + xlObj.model.columnWidth;
            hScroll._scrollData.scrollable = hScroll._scrollData.scrollable + xlObj.model.columnWidth;
            this._refreshScroller(sheetIdx, "refresh", "horizontal");
            hScroll.element.find(".e-hhandle").css("left", (hScroll._scrollData.handleSpace - (hScroll._scrollData.handle + 1)));
            pix = (hScroll._scrollData.handleSpace - hScroll._scrollData.handle) * hScroll._scrollData.onePx;
            hScroll.value(pix);
            for (var i = 0, len = xlObj.model.sheets[sheetIdx]._fCol; i < len; i++)
                colWtColl.push(colWtColl[colWtColl.length - 1] + $(colgroup.find('col')[i]).width());
            (xlObj.getFrozenColumns(sheetIdx) > 0 && (xlObj._getContent(sheetIdx).find(".e-content").width() <= (xlObj._getContent(sheetIdx).find(".e-table").width() - (colWtColl[colWtColl.length - 1] + xlObj.model.columnWidth)))) && xlObj.XLFreeze._hideColumn(sheetIdx);
            (xlObj.getFrozenColumns(sheetIdx) < 1) && xlObj._getContent(sheetIdx).find(".e-content").scrollLeft(pix, hScroll._scrollData.onePx);
            this._getFirstColumn(sheetIdx);
        },

        _scrollY: function (sheetIdx) {
            var xlObj = this.XLObj, vScroll = this._vScroller(sheetIdx), pix, rowHtColl = [1], rows = xlObj.getRows(sheetIdx)[1];
            (xlObj.model.scrollSettings.scrollMode === ej.Spreadsheet.scrollMode.Infinite) && this._scrollBottom(sheetIdx);
            vScroll.model.maximum = vScroll.model.maximum + xlObj.model.rowHeight;
            vScroll._scrollData.scrollable = vScroll._scrollData.scrollable + xlObj.model.rowHeight;
            this._refreshScroller(sheetIdx, "refresh", "vertical");
            vScroll.element.find(".e-vhandle").css("top", (vScroll._scrollData.handleSpace - (vScroll._scrollData.handle + 1)));
            pix = (vScroll._scrollData.handleSpace - vScroll._scrollData.handle) * vScroll._scrollData.onePx;
            vScroll.value(pix);
            for (var i = 0, len = xlObj.model.sheets[sheetIdx]._fRow; i < len; i++)
                rowHtColl.push(rowHtColl[rowHtColl.length - 1] + $(rows[i]).height());
            (xlObj.getFrozenRows(sheetIdx) > 0 && xlObj._getContent(sheetIdx).find(".e-content").height() <= xlObj._getContent(sheetIdx).find(".e-table").height() - (rowHtColl[rowHtColl.length - 1] + xlObj.model.rowHeight)) && xlObj.XLFreeze._hideRow(sheetIdx);
            (xlObj.getFrozenRows(sheetIdx) < 1) && xlObj._getContent(sheetIdx).find(".e-content").scrollTop(pix, vScroll._scrollData.onePx);
            this._getFirstRow(sheetIdx);
        },

        _getFirstRow: function (sheetIdx) {
            var xlObj = this.XLObj, top, vScroll = this._vScroller(sheetIdx), rowHtColl = xlObj.model.sheets[sheetIdx]._rowHeightCollection;
            top = parseInt(vScroll.element.find(".e-vhandle").css('top')) * vScroll._scrollData.onePx;
            for (var i = 0, len = rowHtColl.length; i < len; i++)
                if (rowHtColl[i].value >= top) {
                    xlObj.model.sheets[sheetIdx]._topRow = { idx: i, value: rowHtColl[i].value };
                    break;
                }
        },

        _getFirstColumn: function (sheetIdx) {
            var xlObj = this.XLObj, left, hScroll = this._hScroller(sheetIdx), colWtColl = xlObj.model.sheets[sheetIdx]._colWidthCollection;
            left = parseInt(hScroll.element.find(".e-hhandle").css('left')) * hScroll._scrollData.onePx;
            for (var i = 0, len = colWtColl.length; i < len; i++)
                if (colWtColl[i].value >= left) {
                    xlObj.model.sheets[sheetIdx]._leftCol = { idx: i, value: colWtColl[i].value };
                    break;
                }
        },

        _scrollSelectedPosition: function (sheetIdx, aCell) {
            var leftPx, topPx, xlObj = this.XLObj, hScroll = this._hScroller(sheetIdx), vScroll = this._vScroller(sheetIdx), actCell = xlObj.getCell(aCell.rowIndex, aCell.colIndex);
            if (((parseInt(actCell.width()) + (actCell.offset().left - xlObj._getContent(sheetIdx).find(".e-content").offset().left)) > xlObj._getContent(sheetIdx).find(".e-content").width()) || (actCell.offset().left - xlObj._getContent(sheetIdx).find(".e-content").offset().left) <= 0) {
                leftPx = actCell.offset().left - (hScroll._scrollData.handleSpace + hScroll.model.buttonSize);
                (leftPx < 0) && (leftPx = 0);
                hScroll.scroll(leftPx);
            }
            if (((parseInt(actCell.height()) + (actCell.offset().top - xlObj._getContent(sheetIdx).find(".e-content").offset().top)) > xlObj._getContent(sheetIdx).find(".e-content").height()) || (actCell.offset().top - xlObj._getContent(sheetIdx).find(".e-content").offset().top) <= 0) {
                topPx = actCell.offset().top - (vScroll._scrollData.handleSpace + vScroll.model.buttonSize);
                (topPx < 0) && (topPx = 0);
                (topPx > vScroll._scrollData.scrollable) && (topPx = vScroll._scrollData.scrollable)
                vScroll.scroll(topPx);
            }
        },

        //Infinite Scrolling JS	
        _rowTemplate: function (sheetIdx) {
            var trRData, tdCell = "", xlObj = this.XLObj, i, len, rowCls = "e-rowcell e-overflow";
            for (i = 0, len = xlObj.model.sheets[sheetIdx].colCount; i < len; i++) {
                !xlObj.getSheet(sheetIdx).showGridlines && (rowCls = rowCls.concat(" e-hborder"));
                tdCell += String.format("<td class=\"{0}\" style=\"{2}\"  >{1}</td>", rowCls, "");
            }
            trRData = String.format("<tr style='{0}'>" + tdCell + "</tr>", "height:" + xlObj.model.rowHeight + "px;");
            return trRData;
        },

        _rowHeaderTemplate: function (value) {
            var tdRHdrCData, trRHdrData;
            tdRHdrCData = String.format("<td class=\"{0}\"  >{1}</td>", "e-rowheader", value);
            trRHdrData = String.format("<tr style='{0}'>" + tdRHdrCData + "</tr>", "height:" + this.XLObj.model.rowHeight + "px;");
            return trRHdrData;
        },

        _columnTemplate: function (rowIndex) {
            var rowCls = "e-rowcell e-overflow", tdCData, xlObj = this.XLObj, sheetIdx = xlObj.getActiveSheetIndex();
            !xlObj.getSheet(sheetIdx).showGridlines && (rowCls = rowCls.concat(" e-hborder"));
            tdCData = String.format("<td class=\"{0}\" style=\"{2}\" >{1}</td>", rowCls, "");
            return tdCData;
        },

        _colGroupTemplate: function (sheetIdx) {
            var tdCData = String.format("<col style = '{0}'></col>", "width:" + this.XLObj.getSheet(sheetIdx).columnWidth + "px;");
            return tdCData;
        },

        _columnHeaderTemplate: function (colCnt, sheetIdx) {
            var divcell, divCData, tdCData;
            divcell = this.XLObj._generateHeaderText(sheetIdx, colCnt);
            divCData = String.format("<div class = \"{0}\" > {1}</div>", "e-headercelldiv", divcell);
            tdCData = String.format("<th class=\"{0}\"  >{1}</th>", "e-headercell", divCData);
            return tdCData;
        },

        _createNewRow: function (sheetIdx, rowIdx, colIdx, status) {
            var tbody, trow, trRData, tbodyRHdr, thdr, xlObj = this.XLObj, isVirtualScroll = xlObj.model.scrollSettings.allowVirtualScrolling, xlSelc = xlObj.XLSelection, sheet = xlObj.getSheet(sheetIdx);
            trRData = this._rowTemplate(sheetIdx);
            if (!isVirtualScroll) {
                tbody = xlObj._getContent(sheetIdx).find("tbody").get(0);
                tbodyRHdr = xlObj._getJSSheetRowHeaderContent(sheetIdx).find("tbody").get(0);
            }
            else {
                tbody = xlObj._getContent(sheetIdx).find("tbody").get(1);
                tbodyRHdr = xlObj._getJSSheetRowHeaderContent(sheetIdx).find("tbody").get(1);
            }
            if (rowIdx === -1 || (colIdx > -1 && rowIdx > -1 && status === "shift")) {
                $(tbodyRHdr).append(this._rowHeaderTemplate(parseInt(tbodyRHdr.lastChild.textContent) + 1));
                $(tbody).append(trRData);
                if (status == "shift") {
                    $(tbody).find("tr:last td:last").remove();
                    $(tbody).find("tr: eq(' + rowIdx + ') td:eq(' + (colIdx - 1) + ')").after(this._columnTemplate());
                }
            }
            else {
                trow = tbody.insertRow(rowIdx);
                $(trow).height(xlObj.model.rowHeight);
                $(trow).append($(trRData)[0].innerHTML);
                trow = tbodyRHdr.insertRow(rowIdx);
                $(trow).height(xlObj.model.rowHeight);
                $(trow).append("<td class='e-rowheader'></td>");
            }
            sheet.rowCount = xlObj.model.sheets[sheetIdx].rowCount + 1;
            sheet.rowsHeightCollection.push(xlObj.model.rowHeight);
            xlObj.model.sheets[sheetIdx].dataSource.push({});
			if(rowIdx > -1) {
				thdr = $(tbodyRHdr).find('td');
				for (var i = 0, len = thdr.length; i < len; i++)
					$(thdr[i]).text(i + 1);
				xlObj.XLScroll._getRowHeights(sheetIdx, rowIdx + 1 );
			}
            if(!xlObj.model.scrollSettings.allowVirtualScrolling && xlObj.getFrozenRows(sheetIdx))
			    sheet._rHeightCollection.push({idx: sheet.rowCount - 1, value:  tbody.rows[sheet.rowCount - 1].offsetTop});
			sheet._rowHeightCollection.push({ idx: sheet.rowCount - 1, value: sheet._rowHeightCollection[sheet._rowHeightCollection.length - 1].value + xlObj.model.rowHeight });
			if (xlObj.model.allowSelection && (xlSelc._isColSelected || xlSelc._isSheetSelected))
			    xlSelc.refreshSelection();
        },

        _createNewColumn: function (sheetIdx, startCell, endCell, status) {
            var i, j, len, xlObj = this.XLObj, sheet = xlObj.getSheet(sheetIdx), xlSelc = xlObj.XLSelection, tbody, tbodyCHdr, colgrp, hdrcolgrp, colCnt, tdColgrp, tdRData, tdCHdrData, value,
            thdr, field, $outerDiv, $innerDiv, fIndex, fApplied = false, fRow, tabRange;
            tbody = xlObj._getContTBody(sheetIdx)[0],
            tbodyCHdr = xlObj._getJSSheetHeader(sheetIdx).find("tr").get(0);
            colgrp = xlObj._getJSSheetContent(sheetIdx).find(".e-spreadsheetcontentcontainer colgroup");
            hdrcolgrp = $(tbodyCHdr).parent().prev();
            colCnt = xlObj._generateColCount($.trim(tbodyCHdr.lastChild.textContent));
            tabRange = xlObj.model.sheets[sheetIdx].filterSettings.tableRange;
            for (i = 0, len = tabRange.length; i < len; i++) {
                fIndex = tabRange[i].multifilterIdx;
                if ((startCell.colIndex > fIndex[0]) && (endCell.colIndex < fIndex[fIndex.length - 1])) {
                    fApplied = true;
                    fRow = tabRange[i].startRow - 1;
                    tabRange = tabRange[i];
                    break;
                }
            }
            if (startCell.colIndex === -1 || (startCell.colIndex > -1 && startCell.rowIndex > -1 && status === "shift")) {
                if (startCell.colIndex === -1 && status === "insert") {
                    tdCHdrData = this._columnHeaderTemplate(colCnt + 1, sheetIdx);
                    $(tbodyCHdr).append(tdCHdrData);
                    tdColgrp = this._colGroupTemplate(sheetIdx);
                    $(hdrcolgrp).append(tdColgrp);
                    for (i = 0, len = tbody.rows.length; i < len; i++) {
                        tdRData = this._columnTemplate(i);
                        $(tbody.rows[i]).append(tdRData);
                    }
                    colgrp.append(tdColgrp);
                }
                if (status === "shift") {
                    var x = colCnt;
                    for (i = startCell.colIndex; i <= endCell.colIndex; i++) {
                        tdCHdrData = this._columnHeaderTemplate(x + 1, sheetIdx);
                        $(tbodyCHdr).append(tdCHdrData);
                        tdColgrp = this._colGroupTemplate(sheetIdx);
                        hdrcolgrp.append(tdColgrp);
                        colgrp.append(tdColgrp);
                        x = x + 1;
                    }
                    for (i = 0, len = tbody.rows.length; i < len ; i++)
                        for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                            tdRData = this._columnTemplate(i);
                            $(tbody.rows[i]).append(tdRData);
                        }
                    if (xlObj._insDelStatus.indexOf("insert") > -1)
                        for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                            for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                                $(tbody).find('tr:eq(' + i + ') td:last').remove();
                                tdRData = this._columnTemplate(i);
                                $(tbody).find("tr").eq(i).find('td:eq(' + j + ')').before(tdRData);
                            }
                            xlObj.model.allowFiltering && xlObj.XLFilter._insertFilterIcon(i, j, tabRange, fRow, fApplied);
                        }
                    else
                        for (i = startCell.rowIndex; i <= endCell.rowIndex; i++) {
                            for (j = endCell.colIndex; j >= startCell.colIndex; j--) {
                                $(tbody).find('tr:eq(' + i + ') td:eq(' + j + ')').remove();
                                tdRData = this._columnTemplate(i);
                                $(tbody).find('tr:eq(' + i + ')').append(tdRData);
                            }
                            xlObj.model.allowFiltering && xlObj.XLFilter._insertFilterIcon(i, j, tabRange, fRow, fApplied);
                        }
                }
				sheet.colCount = sheet.colCount + 1;
            }
            else {
                //insert a column in particular position
                if (xlObj._insDelStatus.indexOf("insert") > -1) {
                    for (i = startCell.colIndex; i <= endCell.colIndex; i++) {
                        tdCHdrData = this._columnHeaderTemplate(i + 1, sheetIdx);
                        $(tbodyCHdr).find('th:eq(' + i + ')').before(tdCHdrData);
                        tdColgrp = this._colGroupTemplate(sheetIdx);
                        hdrcolgrp.find('col:eq(' + i + ')').before(tdColgrp);
                        colgrp.find('col:eq(' + i + ')').before(tdColgrp);
                        xlObj._refreshFreezeColumns(sheetIdx, i, parseInt(colgrp.find('col')[i].style.width), "insert");
                    }
                    for (i = 0, len = tbody.rows.length; i < len  ; i++) {
                        for (j = startCell.colIndex; j <= endCell.colIndex; j++) {
                            tdRData = this._columnTemplate(i);
                            $(tbody.rows[i]).find('td:eq(' + j + ')').before(tdRData);
                        }
                        xlObj.model.allowFiltering && xlObj.XLFilter._insertFilterIcon(i, j, tabRange, fRow, fApplied);
                    }
					sheet.colCount = sheet.colCount + 1;
                }
                else {
                    for (i = endCell.colIndex; i >= startCell.colIndex; i--) {
                        xlObj._refreshFreezeColumns(sheetIdx, i, parseInt(colgrp.find('col')[i].style.width), "delete");
                        $(tbodyCHdr).find('th:eq(' + i + ')').remove();
                        hdrcolgrp.find('col:eq(' + i + ')').remove();
                        colgrp.find('col:eq(' + i + ')').remove();
                    }
                    for (i = 0, len = tbody.rows.length; i < len; i++)
                        for (j = endCell.colIndex; j >= startCell.colIndex; j--)
                            $(tbody.rows[i]).find('td:eq(' + j + ')').remove();
					sheet.colCount = sheet.colCount - 1;
                }
				thdr = $(tbodyCHdr).find('th');
                for (i = 0, len = thdr.length; i < len ; i++)
                    $(thdr[i]).find(".e-headercelldiv").text(xlObj._generateHeaderText(sheetIdx, i + 1));
            }
            field = $.trim($(tdCHdrData).text());
            value = $(tdRData).text();
            xlObj.model.sheets[sheetIdx].columns.push({
                field: field,
                type: value != null ? (value.getDay ? "date" : typeof (value)) : null,
                width: $(tdRData).width()
            });           
            sheet.columnsWidthCollection.push(xlObj.model.columnWidth);
            if (!xlObj.model.scrollSettings.allowVirtualScrolling && xlObj.getFrozenColumns(sheetIdx))
    			sheet._cWidthCollection.push({idx: sheet.colCount, value: tbody.rows[0].cells[sheet.colCount - 1].offsetLeft});
			sheet._colWidthCollection.push({ idx: sheet.colCount, value: sheet._colWidthCollection[sheet._colWidthCollection.length - 1].value + xlObj.model.columnWidth});
			(startCell.colIndex > -1) && xlObj.XLScroll._getColWidths(sheetIdx, startCell.colIndex + 1);
			if (xlObj.model.allowSelection && (xlSelc._isRowSelected || xlSelc._isSheetSelected))
			    xlSelc.refreshSelection();
			xlObj._refreshTemplates(sheetIdx, true);
        },

        _insertColHeaders: function (sheetIdx, colCnt, tbodyCHdr) {
            var tdCHdrData = this._columnHeaderTemplate(colCnt + 1, sheetIdx), tdColgrp;
            $(tbodyCHdr).append(tdCHdrData);
            tdColgrp = this._colGroupTemplate(sheetIdx);
            $(tbodyCHdr.parentNode.previousSibling).append(tdColgrp);
        },

        _scrollBottom: function (sheetIdx) {
            this._createNewRow(sheetIdx, -1, -1, "insert");
        },

        _scrollRight: function (sheetIdx) {
            var startCell = { rowIndex: -1, colIndex: -1 };
            this._createNewColumn(sheetIdx, startCell, startCell, "insert");
        },

        _refreshRowCol: function (range, sheetIdx) {
            sheetIdx = this.XLObj._getSheetIndex(sheetIdx);
            var i, sheet = this.XLObj.getSheet(sheetIdx), refreshScroll = false;
            if (sheet.colCount < (range[3] + 1)) {
                i = range[3] - (sheet.colCount - 1);
                while (i--)
                    this._scrollRight(sheetIdx);
                refreshScroll = true;
            }
            if (sheet.rowCount < (range[2] + 1)) {
                i = range[2] - (sheet.rowCount - 1);
                while (i--)
                    this._scrollBottom(sheetIdx);
                refreshScroll = true;
            }
            refreshScroll && this._refreshScroller(sheetIdx, "refresh", "all");
        }
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html ReportViewer elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/

(function ($, ej, undefined) {

    

    ej.widget("ejReportViewer", "ej.ReportViewer", {        
        _rootCSS: "e-reportviewer",        
        element: null,        
        model: null,
        validTags: ["div"],

        //#region Defaults
        defaults: {            
            reportServiceUrl: "",            
            reportPath: "",            
            reportServerUrl: "",            
            dataSources: [],            
            parameters: [],            
            exportSettings: {                
                exportOptions: 1 | 2 | 4 | 8,                
                excelFormat: "excel97to2003",                
                wordFormat: "doc"
            },            
            toolbarSettings: {                
                items: 1 | 2 | 4 | 8 | 16 | 32 | 64 | 128 | 256| 512,                
                showToolbar: true,                
                templateId: "",                
                click: "",               
                showTooltip: true
            },            
            locale: "en-US",            
            printMode: false,            
            renderMode: 1 << 0 | 1 << 1,
            enablePageCache: false,            
            pageSettings: {                
                orientation: null,               
                paperSize: null
            },            
            processingMode: "remote",            
            zoomFactor: 1,            
            reportLoaded: null,            
            renderingBegin: null,            
            renderingComplete: null,            
            reportError: null,
            reportExport: null,            
            drillThrough: null,            
            viewReportClick: null,            
            destroy: null                                                                           
        },

        dataTypes: {
            dataSources: "array",
            parameters: "array",
            toolbarSettings: {
                showToolbar: "boolean"
            }
        },
        //#endregion

        //#region Local Members
        _dataSources: null,
        _svg: true,
        _authenticationToken: null,
        _refresh: false,
        _isToolbarClick: false,
        _printMode: false,
        _pageSetup: false,
        _pageModel: null,
        _currentPage: 1,
        _pageLayoutPage: 1,
        _reportParameters: null,
        _reportDataSources: null,
        _reporDataSets: null,
        _zoomLevel: 2,
        _preZoomVal: 1,
        _actionUrl: null,
        _isDocumentMap: false,
        _isPageDocMap: false,
        _pageDocMapFlag: false,
        _childReportAuthentication: null,
        _parents: [],
        _parentPageXY: null,
        _isDevice: false,
        _zoomVal: 1,
        _originX: 0,
        _originY: 0,
        _paperName: null,
        _isPercentHeight: -1,
        _isPercentWidth: -1,
        _isHeight: false,
        _isWidth: false,
        _paperOrientation: null,
        _pageCache: {},
        _paperSetup: {
            paperHeight: null,
            paperWidth: null,
            MarginTop: 0.0,
            MarginRight: 0.0,
            MarginBottom: 0.0,
            MarginLeft: 0.0
        },
        _reportAction: {
            reportLoad: 'ReportLoad',
            getDataSourceCredential: 'GetDataSourceCredential',
            validateDSCredential: 'ValidateDSCredential',
            updateDSCredential: 'UpdateDSCredential',
            getParameters: 'GetParameters',
            setParameters: 'SetParameters',
            updateParameters: 'UpdateParameters',
            updateDataSource: 'UpdateDataSource',
            getPageModel: 'GetPageModel',
            getPrintModel: 'GetPrintModel',
            drillDown: 'DrillDown',
            clearCache: 'ClearCache',
            documentMap: 'DocumentMap',
            drillThrough: 'DrillThrough',
            sorting: 'Sorting'
        },
        //#endregion

        _parameters: function (eventArgs) {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");;
        },

        _dataSources: function (eventArgs) {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");
        },

        //#region angular
        _tags: [{
            tag: "parameters",
            attr: ["",
                [{
                    tag: "parameter", attr: ["name", "labels", "values", "nullable"]
                }]
            ]
        },
        {
            tag: "dataSources",
            attr: ["",
                [{
                    tag: "datasource", attr: ["name", "value"]
                }]
            ]
        }],
        //#endregion

        //#region Initialization
        _setModel: function (options) {
            var reload = false;
            var update = false;
            for (var prop in options) {
                switch (prop) {
                    case "reportPath":
                    case "reportServerUrl":
                    case "reportServiceUrl":
                        reload = true;
                        break;
                    case "dataSources":
                    case "parameters":
                        this._refresh = true;
                        this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.setParameters, 'parameters': options.parameters }), "_setParameters");
                        break;
                    case "processingMode":
                        update = true;
                        break;
                    case "zoomFactor":
                        this._zoomContainer(options[prop], false);
                        break;
                    case "printMode":
                        this._updatePreviewLayout(options[prop], false);
                        break;
                    case "toolbarSettings":
                        this._updateToolbarmodel();
                        break;
                    case "locale":
                        this._setCultureInfo();
                        break;
                    case "renderMode":
                        this._reInit();
                        break;
                }
            }
            if (update && !reload) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");
            }

            if (reload) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.clearCache }), '_reportReload');
            }
        },

        _reInit: function () {
            this._destroy();
            this._init();
        },

        _destroyEJObjects: function (_ejObjects) {
            for (var i = 0; i < _ejObjects.length; i++) {
                var _ejEle = $(_ejObjects[i]);
                var _ejName = _ejEle.data('ejWidgets');
                if (_ejName) {
                    var _ejObj = _ejEle.data(_ejName[0]);
                    if (_ejObj) {
                        var _ejInnerObjs = _ejObj.element.find('.e-js');
                        if (_ejInnerObjs && _ejInnerObjs.length > 0) {
                            this._destroyEJObjects(_ejInnerObjs);
                        }
                        _ejInnerObjs = null;
                        _ejObj.destroy();
                    }
                }
            }
        },

        _destroy: function () {

            var _ejObjects = this.element.find('.e-js');
            if (_ejObjects && _ejObjects.length > 0) {
                this._destroyEJObjects(_ejObjects);
            }
            _ejObjects = null;

            this._clearPageCache();
            $('#' + this._id + '_exportForm').remove();
            $('#' + this._id + '_toolbar_exportListTip').remove();
            $('#' + this._id + '_toolbar_fittoPagePopup').remove();
            $('#' + this._id + '_rptTooltip').remove();
            $(this.element).find('.e-reportviewer-viewer').remove();
            $('#' + this._id + '_pageInfoPopup').remove();
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.clearCache }), '_clearCurrentServerCache');
        },

        _init: function () {
            this._svg = (window.SVGSVGElement) ? true : false;
            this._actionUrl = this.model.reportServiceUrl + "/PostReportAction";
            this._authenticationToken = this._getAuthenticationToken();
            this._renderViewer();
            this._initViewer();
            this._on($(window), "resize", this._viewerResize);
            if (this.model.reportServiceUrl) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({
                    'reportAction': this._reportAction.reportLoad, 'controlId': this._id, 'reportPath': this.model.reportPath,
                    'reportServerUrl': this.model.reportServerUrl, 'processingMode': this.model.processingMode
                }), "_reportLoaded");
            }
        },

        _initViewer: function () {
            this._wireEvents();
            this._initToolbar();
            this._clearPageCache();
            this._currentPage = 1;
            this._pageLayoutPage = this._currentPage;

            if (this.model.pageSettings.paperSize) {
                this._paperName = this.model.pageSettings.paperSize;
                var paperDimension = this._getPaperSize(this.model.pageSettings.paperSize);
                this._paperOrientation = this.model.pageSettings.orientation == "Landscape" ? "Landscape" : "Portrait";
                if (this.model.pageSettings.orientation == "Landscape") {
                    this._paperSetup.paperHeight = paperDimension.width;
                    this._paperSetup.paperWidth = paperDimension.height;
                } else {
                    this._paperSetup.paperHeight = paperDimension.height;
                    this._paperSetup.paperWidth = paperDimension.width;
                }
            }

            if (!this._refresh) {
                this._preZoomVal = 1;
                this._zoomLevel = 2;
            }
            this._isDocumentMap = false;
            this._printMode = this.model.printMode;
            this._isPageDocMap = false;
            this._pageDocMapFlag = false;
            var tbSettings = this.model.toolbarSettings;
            if (!tbSettings.templateId) {
                this._showToolbar(tbSettings.showToolbar);
                this._showPrintButton(tbSettings.items & ej.ReportViewer.ToolbarItems.Print);
                this._showRefreshButton(tbSettings.items & ej.ReportViewer.ToolbarItems.Refresh);
                this._showZoomControl(tbSettings.items & ej.ReportViewer.ToolbarItems.Zoom);
                this._showFittoPage(tbSettings.items & ej.ReportViewer.ToolbarItems.FittoPage);
                this._showExportControls(tbSettings.items & ej.ReportViewer.ToolbarItems.Export);
                this._showPrintPageSetupButton(tbSettings.items & ej.ReportViewer.ToolbarItems.PageSetup);
                this._showExportList();
                this._showPageNavigationControls(tbSettings.items & ej.ReportViewer.ToolbarItems.PageNavigation);
                this._showParameterBlock(tbSettings.items & ej.ReportViewer.ToolbarItems.Parameters);
                this._showPreviewButton(tbSettings.items & ej.ReportViewer.ToolbarItems.PrintLayout);
                this._showDrillThrough(this._parents.length > 0);
                this._showDocumentMap(false);
            }
            this._showViewerBlock(false);
            this._setContainerSize();
            this._showloadingIndicator(true);
        },
        //#endregion

        //-------------------- Render the ReportViewer Ctrl [start] -------------------------//
        _renderPageInfoPopup: function () {
            var pagePopup = $('#' + this._id + '_pageInfoPopup');
            if (!(pagePopup.length > 0)) {
                pagePopup = ej.buildTag("div.e-reportviewer-pagepopupinfo", "", { 'WHITE-SPACE': 'nowrap' }, { id: this._id + '_pageInfoPopup' });
                var pageNoDiv = ej.buildTag("input.e-reportviewer-pagenumber e-reportviewer-elementalignments ejinputtext", "", {}, { 'type': 'text', 'value': this._currentPage, id: this._id + '_popupPageNo' });
                pageNoDiv.css('height', '25.5px');
                pageNoDiv.css('width', '25px');
                pageNoDiv.css('background-color', 'black');
                pageNoDiv.css('color', 'white');
                pageNoDiv.css('border-color', 'black');

                var pageSeparateSpan = ej.buildTag("span.e-reportviewer-popuptotalpage", "", {}, {});
                pageSeparateSpan.append(' / ' + this._pageModel.TotalPages);
                pagePopup.append(pageNoDiv);
                pagePopup.append(pageSeparateSpan);
                $('body').append(pagePopup);
                this._on($('#' + this._id + '_popupPageNo'), "click", this._popupClick);
                this._on($('#' + this._id + '_popupPageNo'), "keypress", this._onkeyPress);
            }

            if (pagePopup.css('display') == 'block') {
                pagePopup.stop().fadeIn();
                pagePopup.stop().fadeOut(3000);
                pagePopup.hide(3000);
            }
            else {
                pagePopup.show();
                pagePopup.stop().fadeIn();
                pagePopup.stop().fadeOut(3000);
                pagePopup.hide(3000);
            }

            var viewerContainer = $('#' + this._id + '_viewerContainer');
            var popupX = (viewerContainer.offset().left + (viewerContainer.width() / 2)) - (pagePopup.width() / 2);
            var popupY = (viewerContainer.offset().top + (viewerContainer.height() / 2)) - (pagePopup.height() / 2);
            pagePopup.css({ 'left': popupX, 'top': popupY });
            this._updatePageNo();
        },

        _renderToolTip: function () {
            var $divTooltip = ej.buildTag("div.e-reportviewer-tbdiv e-reportviewer-tooltip", "", { 'display': 'none' }, { 'id': this._id + '_rptTooltip' });
            var $tooltipHeader = ej.buildTag("span.e-reportviewer-headerspan", "", { 'display': 'block' }, { 'id': this._id + '_rptTooltip_Header' });
            var $tooltipContent = ej.buildTag("span.e-reportviewer-contentspan", "", { 'display': 'block' }, { 'id': this._id + '_rptTooltip_Content' });
            $divTooltip.append($tooltipHeader);
            $divTooltip.append($tooltipContent);
            $('body').append($divTooltip);
        },

        _renderToolBar: function (targetTag) {
            if (!this.model.toolbarSettings.templateId) {
                var div = ej.buildTag("div.e-reportviewer-toolbarcontainer .e-reportviewer-viewer", "", { 'width': '100%' }, { 'id': this._id + '_toolbarContainer' });
                targetTag.append(div);

                if (this._isDevice) {
                    var $ultoolbaritems = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ultoolbaritems, 'export');
                    this._appendToolbarItems($ultoolbaritems, 'gotoparent');
                    this._appendToolbarItems($ultoolbaritems, 'zoomin');
                    this._appendToolbarItems($ultoolbaritems, 'zoomout');
                    this._appendToolbarItems($ultoolbaritems, 'pagefit');
                    this._appendToolbarItems($ultoolbaritems, 'refresh');
                    this._appendToolbarItems($ultoolbaritems, 'documentmap');
                    this._appendToolbarItems($ultoolbaritems, 'parameter');
                    div.append($ultoolbaritems);
                } else {
                    var $ulprintExport = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulprintExport, 'print');
                    this._appendToolbarItems($ulprintExport, 'export');
                    this._appendToolbarItems($ulprintExport, 'preview');
                    this._appendToolbarItems($ulprintExport, 'pagesetup');
                    div.append($ulprintExport);

                    var $ulnavigate = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulnavigate, 'gotofirst');
                    this._appendToolbarItems($ulnavigate, 'gotoprevious');
                    this._appendToolbarItems($ulnavigate, 'gotopage');
                    this._appendToolbarItems($ulnavigate, 'gotonext');
                    this._appendToolbarItems($ulnavigate, 'gotolast');
                    this._appendToolbarItems($ulnavigate, 'gotoparent');
                    div.append($ulnavigate);

                    var $ulzoom = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulzoom, 'zoomin');
                    this._appendToolbarItems($ulzoom, 'zoomout');
                    this._appendToolbarItems($ulzoom, 'zoom');
                    this._appendToolbarItems($ulzoom, 'pagefit');
                    div.append($ulzoom);

                    var $ulro = ej.buildTag("ul.e-reportviewer-toolbarul", "", {});
                    this._appendToolbarItems($ulro, 'refresh');
                    this._appendToolbarItems($ulro, 'documentmap');
                    this._appendToolbarItems($ulro, 'parameter');
                    div.append($ulro);
                    $('#' + this._id + '_toolbar_zoomSelection').ejDropDownList({ height: "27px", width: "75px", change: this._zoomValChange, selectedItem: 2 });
                }
                div.ejToolbar({ enableSeparator: !this._isDevice, click: $.proxy(this._toolbarClick, this) });
            } else {
                var templateDiv = $('#' + this.model.toolbarSettings.templateId);
                targetTag.append(templateDiv);
                templateDiv.ejToolbar({ enableSeparator: true, height: templateDiv.height(), click: this.model.toolbarSettings.click });
                templateDiv.css('display', 'block');
            }
        },

        _appendToolbarItems: function (litag, eletype) {
            var $divouter;
            var localeObj = ej.ReportViewer.localization[this.model.locale] ? ej.ReportViewer.localization[this.model.locale] : ej.ReportViewer.localization["en-us"];
            switch (eletype) {
                case 'print':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanprint = ej.buildTag("span.e-reportviewer-icon e-reportviewer-print", "", {}, { 'id': this._id + '_toolbar_Print' });
                    $divouter.append($spanprint);
                    break;
                case 'export':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanexport = ej.buildTag("span.e-reportviewer-icon e-reportviewer-export", "", {}, { 'id': this._id + '_ejtb_export' });
                    $divouter.append($spanexport);
                    break;
                case 'pagesetup':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanpagesetup = ej.buildTag("span.e-reportviewer-icon e-reportviewer-pagesetup", "", {}, { 'id': this._id + '_ejtb_PageSetup' });
                    $divouter.append($spanpagesetup);
                    break;
                case 'gotofirst':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotofirst = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotofirst", "", {});
                    $divouter.append($spangotofirst);
                    break;
                case 'gotolast':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotolast = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotolast", "", {});
                    $divouter.append($spangotolast);
                    break;
                case 'gotonext':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotonext = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotonext", "", {});
                    $divouter.append($spangotonext);
                    break;
                case 'gotoprevious':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotoprevious = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotoprevious", "", {});
                    $divouter.append($spangotoprevious);
                    break;
                case 'gotoparent':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spangotoparentreport = ej.buildTag("span.e-reportviewer-icon e-reportviewer-gotoparent", "", {});
                    $divouter.append($spangotoparentreport);
                    break;
                case 'gotopage':
                    $divouter = ej.buildTag("div.e-reportviewer-tbpage", "", { 'display': 'block' }, {});
                    var $inputpageno = ej.buildTag("input.e-reportviewer-pagenumber e-reportviewer-elementalignments ejinputtext", "", {}, { 'type': 'text', 'value': '0', 'id': this._id + '_txtpageNo', 'data-role': 'none' });
                    var $spanpageno = ej.buildTag("span.e-reportviewer-labelpageno", "", {}, {});
                    $divouter.append($inputpageno);
                    $divouter.append($spanpageno);
                    break;
                case 'zoomin':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanzoomin = ej.buildTag("span.e-reportviewer-icon e-reportviewer-zoomin", "", {}, { 'id': this._id + '_toolbar_zoomin' });
                    $divouter.append($spanzoomin);
                    break;
                case 'zoomout':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanzoomout = ej.buildTag("span.e-reportviewer-icon e-reportviewer-zoomout", "", {}, { 'id': this._id + '_toolbar_zoomout' });
                    $divouter.append($spanzoomout);
                    break;
                case 'zoom':
                    $divouter = ej.buildTag("div.e-reportviewer-ejdropdownlist", "", {}, {});
                    var $sltzoom = ej.buildTag("select.e-reportviewer-tbdiv e-reportviewer-zoomlist", "", {}, { 'id': this._id + '_toolbar_zoomSelection', 'data-role': 'none' });
                    $sltzoom.append("<option>50%</option>");
                    $sltzoom.append("<option>75%</option>");
                    $sltzoom.append("<option Selected>100%</option>");
                    $sltzoom.append("<option>125%</option>");
                    $sltzoom.append("<option>150%</option>");
                    $sltzoom.append("<option>200%</option>");
                    $sltzoom.append("<option>400%</option>");
                    $divouter.append($sltzoom);
                    break;
                case 'pagefit':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanexport = ej.buildTag("span.e-reportviewer-icon e-reportviewer-pagefit", "", {}, { 'id': this._id + '_ejtb_fittopage' });
                    $divouter.append($spanexport);
                    break;
                case 'preview':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, { 'id': this._id + '_ejtb_preview' });
                    var $spanpreview = ej.buildTag("span.e-reportviewer-icon e-reportviewer-preview", "", {}, {});
                    $divouter.append($spanpreview);
                    break;
                case 'refresh':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, {});
                    var $spanrefresh = ej.buildTag("span.e-reportviewer-icon e-reportviewer-refresh", "", {}, { 'id': this._id + '_toolbar_refresh' });
                    $divouter.append($spanrefresh);
                    break;
                case 'documentmap':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, { 'id': this._id + '_ejtb_documentmap' });
                    var $spandocumentMap = ej.buildTag("span.e-reportviewer-icon e-reportviewer-documentmap", "", {});
                    $divouter.append($spandocumentMap);
                    break;
                case 'parameter':
                    $divouter = ej.buildTag("li.e-reportviewer-toolbarli", "", {}, { 'id': this._id + '_ejtb_parameter' });
                    var $spanparameter = ej.buildTag("span.e-reportviewer-icon e-reportviewer-parameter", "", {});
                    $divouter.append($spanparameter);
                    break;
                case 'pdf':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli e-pdf", "", {}, { 'id': this._id + '_pdf' });
                    $divouter.append(localeObj['toolbar']['exportformat']['Pdf']);
                    break;
                case 'xls':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli e-excel", "", {}, { 'id': this._id + '_xls' });
                    $divouter.append(localeObj['toolbar']['exportformat']['Excel']);
                    break;
                case 'word':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli e-word", "", {}, { 'id': this._id + '_word' });
                    $divouter.append(localeObj['toolbar']['exportformat']['Word']);
                    break;
                case 'html':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli e-html", "", {}, { 'id': this._id + '_html' });
                    $divouter.append(localeObj['toolbar']['exportformat']['Html']);
                    break;
                case 'fittopage':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli", "", {}, { 'id': this._id + '_fittoPage' });
                    break;
                case 'fittopagewidth':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli e-fitpagewidth", "", { 'padding-right': '5px' }, { 'id': this._id + '_pageWidth', 'isSelect': 'false' });
                    var $spanTagWidth = ej.buildTag("span.e-reportviewer-icon e-reportviewer-emptyconetent", "", { 'padding-right': '4px', 'cursor': 'pointer' }, { 'id': this._id + 'Pagewidthspan', 'fitType': 'PAGEWIDTH' });
                    $divouter.append($spanTagWidth);
                    $divouter.append(localeObj['toolbar']['fittopage']['pageWidth']);
                    break;
                case 'fittowholepage':
                    $divouter = ej.buildTag("li.e-reportviewer-popupli e-fitpageheight", "", { 'padding-right': '5px' }, { 'id': this._id + '_pageHeight', 'isSelect': 'false' });
                    var $spanTagHeight = ej.buildTag("span.e-reportviewer-icon e-reportviewer-emptyconetent", "", { 'padding-right': '4px', 'cursor': 'pointer' }, { 'id': this._id + 'pageheightSpan', 'fitType': 'WHOLEPAGE' });
                    $divouter.append($spanTagHeight);
                    $divouter.append(localeObj['toolbar']['fittopage']['pageHeight']);
                    break;
            }
            litag.append($divouter);
        },

        _renderToolTipExport: function () {
            var $tipExport = ej.buildTag("div.e-reportviewer-tbdiv e-reportviewer-icon e-reportviewer-exporttip", "", { 'display': 'none' }, { 'id': this._id + '_toolbar_exportListTip' });
            this._appendToolbarItems($tipExport, 'pdf');
            this._appendToolbarItems($tipExport, 'xls');
            this._appendToolbarItems($tipExport, 'word');
            this._appendToolbarItems($tipExport, 'html');
            return $tipExport;
        },

        _renderPageFitPopup: function () {
            var $tipFittoPage = ej.buildTag("div.e-reportviewer-tbdiv e-reportviewer-icon e-reportviewer-fittopagetip", "", { 'display': 'none' }, { 'id': this._id + '_toolbar_fittoPagePopup' });
            //this._appendToolbarItems($tipFittoPage, 'fittopage');
            this._appendToolbarItems($tipFittoPage, 'fittopagewidth');
            this._appendToolbarItems($tipFittoPage, 'fittowholepage');
            return $tipFittoPage;
        },

        _renderFitopagePopup: function () {
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            var pageViewerContainer = $('#' + this._id + '_viewerContainer');
            var _pageWidth = $('#' + this._id + '_pageWidth');
            var _pageHeight = $('#' + this._id + '_pageHeight');
            var w = pageViewerContainer.width();
            var h = pageViewerContainer.height();
            var pageW = pageViewline.width();
            var pageH = pageViewline.height();
            if (h > pageH && _pageHeight.attr('isSelect') == 'false') {
                _pageHeight.css('opacity', '0.4');
            } else {
                _pageHeight.css('opacity', '1');
            }
            if (w > pageW && _pageWidth.attr('isSelect') == 'false') {
                _pageWidth.css('opacity', '0.4');
            }
            else {
                _pageWidth.css('opacity', '1');
            }
        },

        //------------------------Render Print Page Setup [Start] ------------------------//

        _showPrintPageSetupButton: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-pagesetup').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-pagesetup').parent().css("display", "none");
            }
        },

        _renderPrintPageSetup: function () {
            var printPageWidth = null, printPageHeight = null;
            var browserInfo = ej.browserInfo();
            var $printPageSetup = ej.buildTag("div", "", { 'padding-left': '20px' }, { 'id': this._id + '_printPageSetup' });

            var $paperSetup = ej.buildTag("div", "", { 'padding-bottom': '10px' }, { 'id': this._id + '_paperSetup' });
            var $printAlignSetup = ej.buildTag("div", "", {}, { 'id': this._id + '_printAlignSetup' });
            var $defaultSetup = ej.buildTag("div", "", {}, { 'id': this._id + '_defaultSetup' });
            var position = $('#' + this._id + '.e-reportviewer')[0].getBoundingClientRect();

            if (browserInfo.name == "msie" && browserInfo.version == 8) {
                printPageWidth = position.right;
                printPageHeight = position.bottom;
            }
            else {
                printPageWidth = position.width;
                printPageHeight = position.height;
            }
            $printPageSetup.ejDialog({ width: 348, position: { X: (position.left + (printPageWidth) / 3) + 20, Y: (position.top + (printPageHeight / 2)) / 2 }, enableModal: false, title: "Page Setup", enableResize: false, enablePersistance: false });
            var $sizeHeader = ej.buildTag("div", "", { 'font-weight': 'bold' }, {});
            var $paperSizeType = ej.buildTag("div.e-reportviewer-ejdropdownlist", "", { 'width': '315px', 'padding-bottom': '10px' }, {});

            $sizeHeader.append('&nbsp;<span>Paper Size</span>');
            $paperSetup.append($sizeHeader);

            $paperSizeDDl = ej.buildTag("select", "", {}, { 'id': this._id + '_PaperSize' });
            $paperSizeDDl.append('<option value="A3">A3</option>');
            $paperSizeDDl.append('<option value="A4">A4</option>');
            $paperSizeDDl.append('<option value="B4(JIS)">B4(JIS)</option>');
            $paperSizeDDl.append('<option value="B5(JIS)">B5(JIS)</option>');
            $paperSizeDDl.append('<option value="Envelope #10">Envelope #10</option>');
            $paperSizeDDl.append('<option value="Envelope Monarch">Envelope Monarch</option>');
            $paperSizeDDl.append('<option value="Executive">Executive</option>');
            $paperSizeDDl.append('<option value="Legal">Legal</option>');
            $paperSizeDDl.append('<option value="Letter">Letter</option>');
            $paperSizeDDl.append('<option value="Tabloid">Tabloid</option>');
            $paperSizeDDl.append('<option value="Custom">Custom</option>');

            $paperSizeType.append($paperSizeDDl);
            $paperSizeDDl.ejDropDownList({ width: '95%', height: '26px', selectedIndex: 10, watermarkText: 'Select Option', change: $.proxy(this._setUpChange, this), enableIncrementalSearch: true });
            $paperSetup.append($paperSizeType);

            var $paperSizeContainer = ej.buildTag("div", "", { 'float': 'left', 'width': '100%', 'padding-left': '4px', 'padding-bottom': '20px', 'padding-top': '5px' }, {});
            var $paperSizeWidthLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '15%', 'margin-top': '4px', 'margin-left': '20px' }, {});
            var $paperSizeWidth = ej.buildTag("div", "", { 'float': 'left', 'width': '25%' }, {});
            var $paperSizeHeightLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '17%', 'margin-top': '4px' }, {});
            var $paperSizeHeight = ej.buildTag("div", "", { 'float': 'left', 'width': '25%' }, {});
            var $paperHeightUnitLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '5%', 'margin-top': '4px', 'font-weight': 'bold' }, {});
            var $paperWidthUnitLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '5%', 'margin-top': '4px', 'font-weight': 'bold' }, {});

            var $height = ej.buildTag("input.e-reportviewer-textbox", "", { 'margin-right': '5px' }, { 'type': 'text', 'id': this._id + "_paperHeight" });
            var $width = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'id': this._id + "_paperWidth" });
            $paperSizeHeight.append($height);
            $paperSizeWidth.append($width);

            $paperSizeHeightLabel.append('Height&nbsp;');
            $paperHeightUnitLabel.append('in&nbsp;');
            $paperSizeWidthLabel.append('Width&nbsp;');
            $paperWidthUnitLabel.append('in&nbsp;');

            $paperSizeContainer.append($paperSizeHeightLabel);
            $paperSizeContainer.append($paperSizeHeight);
            $paperSizeContainer.append($paperHeightUnitLabel);
            $paperSizeContainer.append($paperSizeWidthLabel);
            $paperSizeContainer.append($paperSizeWidth);
            $paperSizeContainer.append($paperWidthUnitLabel);

            $paperSetup.append($paperSizeContainer);
            $height.ejNumericTextbox({ enableStrictMode: true, width: '75px', minValue: 1, decimalPlaces: 2, enable: false });
            $width.ejNumericTextbox({ enableStrictMode: true, width: "75px", minValue: 1, decimalPlaces: 2, enable: false });

            var $marginHeader = ej.buildTag("div", "", { 'padding-top': '10px', 'font-weight': 'bold' }, {});
            var $margins = ej.buildTag("div", "", { 'padding-top': '10px', 'padding-left': '5px' }, {});

            var $tbsubrow1 = ej.buildTag("div", "", { 'float': 'left', 'width': '100%' }, {});
            var $tbsubrow1Cell1 = ej.buildTag("div", "", { 'float': 'left', 'width': '17%', 'margin-top': '4px' }, {});
            var $tbsubrow1Cell2 = ej.buildTag("div", "", { 'float': 'left', 'width': '25%' }, {});
            var $topUnitLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '5%', 'margin-top': '4px', 'font-weight': 'bold' }, {});
            var $tbsubrow1Cell3 = ej.buildTag("div", "", { 'float': 'left', 'width': '15%', 'margin-top': '4px', 'margin-left': '20px' }, {});
            var $tbsubrow1Cell4 = ej.buildTag("div", "", { 'float': 'left', 'width': '25%' }, {});
            var $rightUnitLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '5%', 'margin-top': '4px', 'font-weight': 'bold' }, {});

            var $tbsubrow2 = ej.buildTag("div", "", { 'float': 'left', 'width': '100%', 'padding-top': '12px', 'padding-bottom': '18px' }, {});
            var $tbsubrow2Cell1 = ej.buildTag("div", "", { 'float': 'left', 'width': '17%', 'margin-top': '4px' }, {});
            var $tbsubrow2Cell2 = ej.buildTag("div", "", { 'float': 'left', 'width': '25%' }, {});
            var $bottomUnitLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '5%', 'margin-top': '4px', 'font-weight': 'bold' }, {});
            var $tbsubrow2Cell3 = ej.buildTag("div", "", { 'float': 'left', 'width': '15%', 'margin-top': '4px', 'margin-left': '20px' }, {});
            var $tbsubrow2Cell4 = ej.buildTag("div", "", { 'float': 'left', 'width': '25%' }, {});
            var $leftUnitLabel = ej.buildTag("div", "", { 'float': 'left', 'width': '5%', 'margin-top': '4px', 'font-weight': 'bold' }, {});

            $marginHeader.append('&nbsp;<span>Margins</span>');
            $printAlignSetup.append($marginHeader);

            var $top = ej.buildTag("input.e-reportviewer-textbox", "", { 'margin-right': '5px' }, { 'type': 'text', 'id': this._id + "_paperMarginTop" });
            var $right = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'id': this._id + "_paperMarginRight" });
            var $bottom = ej.buildTag("input.e-reportviewer-textbox", "", { 'margin-right': '5px' }, { 'type': 'text', 'id': this._id + "_paperMarginBottom" });
            var $left = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'id': this._id + "_paperMarginLeft" });

            $tbsubrow1Cell1.append('Top&nbsp;');
            $topUnitLabel.append('in&nbsp;');
            $tbsubrow1Cell2.append($top);
            $tbsubrow1Cell3.append('Right&nbsp;');
            $rightUnitLabel.append('in&nbsp;');
            $tbsubrow1Cell4.append($right);
            $tbsubrow1.append($tbsubrow1Cell1);
            $tbsubrow1.append($tbsubrow1Cell2);
            $tbsubrow1.append($topUnitLabel);
            $tbsubrow1.append($tbsubrow1Cell3);
            $tbsubrow1.append($tbsubrow1Cell4);
            $tbsubrow1.append($rightUnitLabel);
            $margins.append($tbsubrow1);

            $tbsubrow2Cell1.append('Bottom&nbsp;');
            $bottomUnitLabel.append('in&nbsp;');
            $tbsubrow2Cell2.append($bottom);
            $tbsubrow2Cell3.append('Left&nbsp;');
            $leftUnitLabel.append('in&nbsp;');
            $tbsubrow2Cell4.append($left);
            $tbsubrow2.append($tbsubrow2Cell1);
            $tbsubrow2.append($tbsubrow2Cell2);
            $tbsubrow2.append($bottomUnitLabel);
            $tbsubrow2.append($tbsubrow2Cell3);
            $tbsubrow2.append($tbsubrow2Cell4);
            $tbsubrow2.append($leftUnitLabel);
            $margins.append($tbsubrow2);
            $printAlignSetup.append($margins);

            $top.ejNumericTextbox({ decimalPlaces: 2, width: "75px", minValue: 0, maxValue: 22 });
            $right.ejNumericTextbox({ decimalPlaces: 2, width: "75px", minValue: 0, maxValue: 22 });
            $bottom.ejNumericTextbox({ decimalPlaces: 2, width: "75px", minValue: 0, maxValue: 22 });
            $left.ejNumericTextbox({ decimalPlaces: 2, width: "75px", minValue: 0, maxValue: 22 });

            var $orientation = ej.buildTag("div", "", { 'padding-top': '10px', 'font-weight': 'bold' }, {});
            $orientation.append('&nbsp;<span>Orientation</span>');
            $printAlignSetup.append($orientation);
            var $radio = ej.buildTag("div", "", { 'width': '100%', 'padding-left': '5px', 'padding-top': '5px' }, {});

            var $radio1 = ej.buildTag("div", "", { 'width': '50%', 'float': 'left' }, {});
            var $radio1div1 = ej.buildTag("div", "", { 'float': 'left', 'margin-top': '2px' }, {});
            var $radio1div2 = ej.buildTag("div", "", { 'float': 'left' }, {});

            var $portraitradio = ej.buildTag("input", "", {}, { 'type': 'radio', 'id': this._id + '_portrait', 'value': 'Portrait' });

            $radio1div1.append($portraitradio);
            $radio1div2.append("&nbsp;<span>Portrait</span>");
            $radio1.append($radio1div1);
            $radio1.append($radio1div2);
            $radio.append($radio1);
            $portraitradio.ejRadioButton({ change: $.proxy(this._orientationChanged, this), name: 'orientation', checked: true });

            var $radio2 = ej.buildTag("div", "", { 'width': '50%', 'float': 'left' }, {});
            var $radio2div1 = ej.buildTag("div", "", { 'float': 'left', 'margin-top': '2px' }, {});
            var $radio2div2 = ej.buildTag("div", "", { 'float': 'left' }, {});

            var $landscaperadio = ej.buildTag("input", "", {}, { 'type': 'radio', 'id': this._id + '_landscape', 'value': 'Landscape' });

            $radio2div1.append($landscaperadio);
            $radio2div2.append("&nbsp;<span>Landscape</span>");
            $radio2.append($radio2div1);
            $radio2.append($radio2div2);
            $radio.append($radio2);

            $printAlignSetup.append($radio);
            $landscaperadio.ejRadioButton({ change: $.proxy(this._orientationChanged, this), name: 'orientation' });

            var $confirm = ej.buildTag("div", "", { 'padding-top': '10px', 'float': 'right', 'padding-right': '13px', 'padding-bottom': '10px' }, {});

            var $submitButton = ej.buildTag("input.e-reportviewer-viewreportbutton e-btn", "", { 'margin-left': '50px', 'margin-top': '10px', 'min-width': '65px' }, { 'type': 'button', 'value': 'Done', 'id': this._id + '_Submit' });
            $submitButton.ejButton({ showRoundedCorner: true, height: 30, click: $.proxy(this._pageSetupSubmit, this) });

            var $cancelButton = ej.buildTag("input.e-reportviewer-viewreportbutton e-btn", "", { 'margin-left': '10px', 'margin-top': '10px', 'min-width': '65px' }, { 'type': 'button', 'value': 'Cancel', 'id': this._id + '_Cancel' });
            $cancelButton.ejButton({ showRoundedCorner: true, height: 30, click: $.proxy(this._pageSetupCancel, this) });

            $confirm.append($submitButton);
            $confirm.append($cancelButton);
            $defaultSetup.append($confirm);

            $printPageSetup.append($paperSetup);
            $printPageSetup.append($printAlignSetup);
            $printPageSetup.append($defaultSetup);

            var browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie') {
                $('#' + this._id + '_PaperSize_input').css('margin-top', '-2px');
            } else if (browserInfo.name == 'opera' || browserInfo.name == 'webkit' || browserInfo.name == 'chrome' || browserInfo.name == 'mozilla') {
                $('#' + this._id + '_PaperSize_input').css('padding-bottom', '3px');
            }
        },

        //------------------------Render Print Page Setup [End] ------------------------//

        _renderViewerBlockinDevice: function (targetTag) {
            var div = ej.buildTag("div.e-reportviewer-viewer e-reportviewer-viewerblock e-reportviewer-blockstyle", "", {}, { 'id': this._id + '_viewBlockContainer' });
            targetTag.append(div);

            var $innerConetnt = ej.buildTag("div.e-reportviewer-viewerblockcellcontent", "", { 'margin': '1px', 'padding': '1px', 'width': '99%', 'height': '99%', 'display': 'inline-table' });
            var $blockContent = ej.buildTag("div.e-reportviewer-viewerblockcontent", "", { 'width': '100%', 'height': 'auto' });
            var $buttonConetnt = ej.buildTag("div.e-reportviewer-viewreport", "", { 'width': '100%', 'height': '30px', 'border-left': '0px', 'padding': '10px 0' });
            var $button = ej.buildTag("input.e-reportviewer-viewreportbutton e-btn", "", {}, { 'type': 'button', 'value': ej.ReportViewer.localization[this.model.locale] ? ej.ReportViewer.localization[this.model.locale]["viewButton"] : ej.ReportViewer.localization["en-us"]["viewButton"], 'id': this._id + '_viewReportClick', 'data-role': 'none' });
            $buttonConetnt.append($button);
            $innerConetnt.append($blockContent);
            $innerConetnt.append($buttonConetnt);

            $button.ejButton({ showRoundedCorner: true });

            div.append($innerConetnt);
            return div;
        },

        _renderViewerBlockinWeb: function (targetTag) {
            var div = ej.buildTag("div.e-reportviewer-viewer e-reportviewer-viewerblock", "", {}, { 'id': this._id + '_viewBlockContainer' });
            targetTag.append(div);

            var $innerConetnt = ej.buildTag("table.e-reportviewer-viewerblockcellcontent", "", { 'margin': '1px', 'padding': '5px 5px 10px' });
            var $tr = ej.buildTag("tr", "", { 'width': '100%' });
            var $tdleft = ej.buildTag("td.e-reportviewer-viewerblockcontent", "", {});
            var $tdright = ej.buildTag("td.e-reportviewer-viewreport", "", {});
            var $button = ej.buildTag("input.e-reportviewer-viewreportbutton e-btn", "", {'margin-top':'4px'}, { 'type': 'button', 'value': ej.ReportViewer.localization[this.model.locale] ? ej.ReportViewer.localization[this.model.locale]["viewButton"] : ej.ReportViewer.localization["en-US"]["viewButton"], 'id': this._id + '_viewReportClick', 'data-role': 'none' });
			$button.ejButton({ height: 30 });
            $tdright.append($button);
            $tr.append($tdleft);
            $tr.append($tdright);
            $innerConetnt.append($tr);

            $button.ejButton({ showRoundedCorner: true });

            div.append($innerConetnt);
            return div;
        },

        _renderViewerContainer: function (targetTag) {
            var div = ej.buildTag("div", "", {}, { 'id': this._id + '_reportviewerContainer' });
            targetTag.append(div);

            var $reportviewContainer = ej.buildTag("div.e-reportviewer-viewer e-reportviewer-scrollcontainer e-reportviewer-viewercontainer", "", { 'height': '100%', 'width': '100%', 'font-size': '8pt' }, { 'id': this._id + '_viewerContainer' });

            var $loadingindicator = ej.buildTag("div", "", { 'margin': '0px', 'height': '99.8%', 'width': '100%' }, { 'id': this._id + '_loadingIndicator' });
            var $loadingemptybackview = ej.buildTag("div", "", { 'margin': '0px', 'height': '99.8%', 'width': '100%', 'background-color': 'rgba(164, 183, 216, 0.18)', 'display': 'block' }, { 'id': this._id + '_loadingIndicatorBackView' });

            var $pageviewOuterContainer = ej.buildTag("div.e-reportviewer-pageviewcontainer", "", {}, { 'id': this._id + '_pageviewOuterContainer' });
            var $pageviewOuterline = ej.buildTag("div.e-reportviewer-pageouterline", "", {}, { 'id': this._id + '_pageviewOuterline' });
            var $pageviewContainer = ej.buildTag("div.e-reportviewer-pageview", "", { 'background-color': '#FFFFFF' }, { 'id': this._id + '_pageviewContainer' });

            var $pageviewheaderContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent' }, { 'id': this._id + '_pageviewheaderContainer' });
            var $pageviewbodyContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent' }, { 'id': this._id + '_pageviewbodyContainer' });
            var $pageviewfooterContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent' }, { 'id': this._id + '_pageviewfooterContainer' });

            var $pageviewheaderborder = ej.buildTag("div.pageHeaderBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF' });
            var $pageviewbodyborder = ej.buildTag("div.pageBodyBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF' });
            var $pageviewfooterborder = ej.buildTag("div.pageFooterBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF' });

            $pageviewheaderborder.append($pageviewheaderContainer);
            $pageviewbodyborder.append($pageviewbodyContainer);
            $pageviewfooterborder.append($pageviewfooterContainer);
            $pageviewContainer.append($pageviewheaderborder);
            $pageviewContainer.append($pageviewbodyborder);
            $pageviewContainer.append($pageviewfooterborder);

            $loadingindicator.append($loadingemptybackview);
            $reportviewContainer.append($loadingindicator);
            $pageviewOuterline.append($pageviewContainer);
            $pageviewOuterContainer.append($pageviewOuterline);
            $reportviewContainer.append($pageviewOuterContainer);

            if (this._isDevice) {
                this._renderViewerBlockinDevice($reportviewContainer);
            } else {
                var $documentmapcontainer = ej.buildTag("div.e-reportviewer-documentmapcontainer", "", {}, { 'id': this._id + '_documentmapContainer' });
                div.append($documentmapcontainer);
            }

            div.append($reportviewContainer);

            return div;
        },

        _renderViewer: function () {
            var height = this.element.height();
            var width = this.element.width();
            
            var viewer = ej.buildTag("div.e-reportviewer-viewer", "", {});

            if (!this.element[0].style.height && this.element[0].parentElement.clientHeight != 0) {
                this._isHeight = true;
            }

            if (!this.element[0].style.width && this.element[0].parentElement.clientWidth != 0) {
                this._isWidth = true;
            }

            if (height === 0 && this.element[0].parentElement.clientHeight != 0) {
                this.element.height(this.element[0].parentElement.clientHeight);
                //viewer.height(this.element[0].parentElement.clientHeight);
            }

            if (width === 0 && this.element[0].parentElement.clientWidth != 0) {
                this.element.width(this.element[0].parentElement.clientWidth);
                //viewer.width(this.element[0].parentElement.clientWidth);
            }

            this.element.append(viewer);
            if ((this.model.renderMode & ej.ReportViewer.RenderMode.Mobile) && (this.model.renderMode & ej.ReportViewer.RenderMode.Desktop)) {
                this._isDevice = this._isMobileDevice();
            } else if (this.model.renderMode & ej.ReportViewer.RenderMode.Mobile) {
                this._isDevice = true;
            } else {
                this._isDevice = false;
            }
            this._renderToolBar(viewer);
            if (!this._isDevice) {
                this._renderViewerBlockinWeb(viewer);
                this._renderToolTip();
            }
            this._renderViewerContainer(viewer);
        },
        //-------------------- Render the ReportViewer Ctrl [End] -------------------------//

        //-------------------- Render Credential Block and Actions [Start] -------------------------//
        _renderCredentialBlock: function (datasources) {
            var table = ej.buildTag("table", "", {}, {});
            for (var index = 0; index < datasources.length; index++) {
                var datasource = datasources[index];
                var $headertr = ej.buildTag("tr", "", {}, {});
                var $headertd = ej.buildTag("td", "", {}, { 'colspan': !this._isDevice ? '5' : '1' });
                $headertd.html(!this._isDevice ? datasource.Promt : 'Specify the ' + datasource.Name + ':');
                $headertr.append($headertd);

                if (!this._isDevice) {
                    var $row = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row, 'namelbl', datasource);
                    this._appendCredentialItems($row, 'nametxt', datasource);
                    $row.append(ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", { 'width': '7px' }, {}));
                    this._appendCredentialItems($row, 'passwdlbl', datasource);
                    this._appendCredentialItems($row, 'passwdtxt', datasource);

                    table.append($headertr);
                    table.append('<tr height="6px"/>');
                    table.append($row);
                    table.append('<tr height="6px"/>');
                } else {
                    table.append($headertr);
                    table.append('<tr height="4px"/>');

                    var $row1 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row1, 'namelbl', datasource);
                    var $row2 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row2, 'nametxt', datasource);
                    var $row3 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row3, 'passwdlbl', datasource);
                    var $row4 = ej.buildTag("tr", "", {}, {});
                    this._appendCredentialItems($row4, 'passwdtxt', datasource);

                    table.append($row1);
                    table.append($row2);
                    table.append($row3);
                    table.append($row4);
                    table.append('<tr height="6px"/>');
                }
            }
            var viewerBlock = $('#' + this._id + '_viewBlockContainer').find(".e-reportviewer-viewerblockcontent");
            viewerBlock.append(table);
            this._on($('#' + this._id + '_viewReportClick'), "click", { ds: datasources }, this._viewReportCredentialClick);
        },

        _appendCredentialItems: function (targetTag, eleType, datasource) {
            var $tbcell;
            switch (eleType) {
                case 'namelbl':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    $tbcell.append('Login Name&nbsp;');
                    break;
                case 'nametxt':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    var $loginname = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'autocomplete': 'off', 'id': datasource.ControlId + "_loginName" });
                    $tbcell.append($loginname);
                    break;
                case 'passwdlbl':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    $tbcell.append('Password&nbsp;');
                    break;
                case 'passwdtxt':
                    $tbcell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                    var $password = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'password', 'autocomplete': 'off', 'id': datasource.ControlId + "_password" });
                    $tbcell.append($password);
                    break;
            }
            targetTag.append($tbcell);
        },

        _viewReportCredentialClick: function (event) {
            var datasources = event.data.ds;
            var jsdatasource = [];
            for (var index = 0; index < datasources.length; index++) {
                var datasource = datasources[index];
                var username = $('#' + datasource.ControlId + '_loginName').val();
                var password = $('#' + datasource.ControlId + '_password').val();
                jsdatasource.push({ 'Name': datasource.Name, 'UserName': username, 'Password': password });
            }
            this._dataSources = jsdatasource;
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.validateDSCredential, 'datasoures': jsdatasource }), "_validateDSCredential");
        },
        //-------------------- Render Credential Block and Actions [End] -------------------------//

        //-------------------- Render Parameter Block and Actions [Start] -------------------------//
        _renderParameterBlock: function (parameters) {
            var table = ej.buildTag("table", "", {}, { 'isViewClick': false });
            if (this._isDevice) {
                table.css('width', '100%');
            }
            if (parameters != null && parameters.length > 0) {
                var viewerBlock = $('#' + this._id + '_viewBlockContainer').find(".e-reportviewer-viewerblockcontent");
                var isRender = this._onRenderingBegin({ reportParameters: $.extend(true, {}, parameters), parameterBlock: $('#' + this._id + '_viewBlockContainer') });
                if (!isRender) {
                    viewerBlock.html(table);
                    for (var index = 0; index < parameters.length; index++) {
                        var parameter = parameters[index];

                        var tablerow;
                        var $contentDiv;
                        var $tdCell01;
                        var $tdCell02;
                        if (!this._isDevice) {
                            if (index % 2 == 0) {
                                table.append('<tr height="10px"/>');
                                tablerow = ej.buildTag("tr", "", {}, {});
                                table.append(tablerow);
                            } else {
                                var $tdmiddle = ej.buildTag("td", "", { 'width': '15px' }, {});
                                tablerow.append($tdmiddle);
                            }
                            $tdCell01 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                            $tdCell01.append(parameter.Promt + '&nbsp;&nbsp;');
                            tablerow.append($tdCell01);
                            $tdCell02 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", { 'min-width': '130px', 'max-width': '185px' }, {});
                            tablerow.append($tdCell02);

                            $contentDiv = ej.buildTag("div", "", { 'width': '100%', 'display': 'flex', 'white-space': 'nowrap' }, { 'id': parameter.ControlId + '_div' });
                            $tdCell02.append($contentDiv);

                        } else {
                            table.append('<tr height="10px"/>');
                            tablerow = ej.buildTag("tr", "", {}, {});
                            table.append(tablerow);
                            $tdCell01 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
                            $tdCell01.append(parameter.Promt + '&nbsp;&nbsp;');
                            tablerow.append($tdCell01);

                            tablerow = ej.buildTag("tr", "", {}, {});
                            table.append(tablerow);
                            $tdCell02 = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", { 'min-width': '130px', 'max-width': '185px' }, {});
                            tablerow.append($tdCell02);

                            $contentDiv = ej.buildTag("div", "", { 'width': '100%', 'display': 'flex', 'white-space': 'nowrap' }, { 'id': parameter.ControlId + '_div' });
                            $tdCell02.append($contentDiv);
                        }
                        this._renderParameterElements(parameter, $contentDiv);
                    }
                }
            }
            else {
                this._showParameterBlock(false);
            }
            table.append('<tr height="10px"/>');
            this._onRenderingComplete({ reportParameters: $.extend(true, {}, parameters) });
            this._on($('#' + this._id + '_viewReportClick'), "click", { params: parameters }, this._viewReportParamsClick);
            return isRender;
        },

        _renderParameterElements: function (parameter, $contentDiv) {
            var indexddl;
            var $parameterSpan;
            var $parameterNull;
            var $optionText;
            if (parameter.IsMultiValue) {
                var itemindex = [];
                var $paramMultiValuedropdown = ej.buildTag("select", "", {}, { 'id': parameter.ControlId, 'name': parameter.Name, 'sf-name': parameter.Name });
                if (parameter.AvailableLabels) {
                    $optionText = $('<option value="parameter_SelectAll"> Select All</option>"');
                    $paramMultiValuedropdown.append($optionText);
                    var _selectedFieldVals = $.extend(parameter.DefaultValuesfields, true);
                    if (parameter.Values && parameter.Values.length > 0) {
                        _selectedFieldVals = parameter.Values;
                    }

                    for (var indexddl = 0; indexddl < parameter.AvailableLabels.length; indexddl++) {
                        $optionText = $('<option value="' + parameter.AvailableValues[indexddl] + '">' + parameter.AvailableLabels[indexddl] + '</option>');
                        if (jQuery.inArray(parameter.AvailableValues[indexddl], _selectedFieldVals) != -1) {
                            itemindex.push(indexddl + 1);
                        }
                        $paramMultiValuedropdown.append($optionText);
                    }
                    if (parameter.AvailableValues.length == itemindex.length) {
                        itemindex.push(0);
                    }
                    $contentDiv.append($paramMultiValuedropdown);
                    $paramMultiValuedropdown.ejDropDownList({ width: '95%', height: '26px', showCheckbox: true, selectedIndices: itemindex, watermarkText: 'Select Option', change: parameter.IsDependent ? this._paramsChangeEvent : '', checkChange: this._selectionChanged });
                }
                else if (parameter.DefaultValues) {
                    $optionText = $('<option value="parameter_SelectAll"> Select All</option>"');
                    $paramMultiValuedropdown.append($optionText);
                    var _selectedFieldVals = $.extend(parameter.DefaultValuesfields, true);
                    if (parameter.Values && parameter.Values.length > 0) {
                        _selectedFieldVals = parameter.Values;
                    }
                    for (var indexddl = 0; indexddl < parameter.DefaultValues.length; indexddl++) {
                        $optionText = $('<option value="' + parameter.DefaultValues[indexddl] + '">' + parameter.DefaultValues[indexddl] + '</option>');
                        if (jQuery.inArray(parameter.DefaultValues[indexddl], _selectedFieldVals) != -1) {
                            itemindex.push(indexddl + 1);
                        }
                        $paramMultiValuedropdown.append($optionText);
                    }
                    if (parameter.DefaultValues.length == itemindex.length) {
                        itemindex.push(0);
                    }
                    $contentDiv.append($paramMultiValuedropdown);
                    $paramMultiValuedropdown.ejDropDownList({ width: '95%', height: '26px', showCheckbox: true, selectedIndices: itemindex, watermarkText: 'Select Option', change: parameter.IsDependent ? this._paramsChangeEvent : '', checkChange: this._selectionChanged });
                }
            } else if (parameter.AvailableValues != null) {
                var $parameterdropdown = ej.buildTag("select", "", {}, { 'id': parameter.ControlId, 'name': parameter.Name, 'data': 'select', 'sf-name': parameter.Name });
                var selectindex = 9999;
                for (indexddl = 0; indexddl < parameter.AvailableLabels.length; indexddl++) {
                    if (parameter.AvailableLabels[indexddl] != 'Select a value') {
                        $optionText = $('<option value="' + parameter.AvailableValues[indexddl] + '">' + parameter.AvailableLabels[indexddl] + '</option>');
                        if (parameter.AvailableLabels[indexddl] == parameter.Label) {
                            selectindex = indexddl;
                        }
                        $parameterdropdown.append($optionText);
                    }
                }
                $contentDiv.append($parameterdropdown);
                var _dropDownJson = { width: '95%', height: '26px', watermarkText: 'Select a Value', change: parameter.IsDependent ? this._paramsChangeEvent : '', enableIncrementalSearch: true };
                if (selectindex != 9999) {
                    _dropDownJson.selectedIndex = selectindex;
                }
                $parameterdropdown.ejDropDownList(_dropDownJson);
            } else if (parameter.DataType == "DateTime") {
                var $parameterdatepicker = ej.buildTag("input.e-reportviewer-textbox e-reportviewer-DateTime", "", {}, { 'type': 'text', 'id': parameter.ControlId, 'maxlength': '256', 'name': parameter.Name });
                $contentDiv.append($parameterdatepicker);
                var datestring = ((parameter.Label.indexOf('-')) ? (parameter.Label.replace(/\-/g, "/")) : parameter.Label);
                $parameterdatepicker.ejDatePicker({ width: parameter.IsNullable ? "75%" : "95%", height: '26px', value: parameter.Label != null && parameter.Label.length > 0 ? new Date(datestring ? datestring : parameter.Label) : null, change: parameter.IsDependent ? this._paramsChangeEvent : '', locale: this.model.locale });
                if (parameter.IsNullable) {
                    $parameterSpan = ej.buildTag("span", "", { 'WHITE-SPACE': 'nowrap', 'display': 'block' }, {});
                    $parameterNull = ej.buildTag("input", "", {}, { 'type': 'checkbox', 'name': 'chkDateTime', 'id': parameter.ControlId + '~chk', 'value': this._id });
                    $parameterSpan.append($parameterNull);
                    $parameterSpan.append('&nbsp;<span class=e-reportviewer-label>Null</span>');
                    $contentDiv.append('&nbsp;');
                    $contentDiv.append($parameterSpan);
                    if (parameter.Label != null && parameter.Label.length > 0) {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    } else {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange, checked: true });
                        this._paramElementDisable(parameter.ControlId, 'chkDateTime', true);
                    }
                }
            } else if (parameter.DataType == "Boolean") {
                var $parametertable = ej.buildTag("table", "", {}, {});
                var $parametertr = ej.buildTag("tr", "", {}, {});
                $parametertable.append($parametertr);

                var propStat = false;
                if (parameter.Label != null && parameter.Label.length > 0) {
                    if (parameter.Label.toLowerCase() == "true") {
                        propStat = true;
                    }
                }

                var $parametertd01 = ej.buildTag("td", "", {}, {});
                var $parameterradio01 = ej.buildTag("input", "", {}, { 'type': 'radio', 'name': parameter.Name, 'id': parameter.ControlId + '_01' });
                $parametertd01.append($parameterradio01);
                $parametertd01.append("&nbsp;<span>True</span>");
                $parameterradio01.ejRadioButton({ checked: propStat });
                $parametertr.append($parametertd01);

                $parametertr.append('<td width=10px/>');

                var $parametertd02 = ej.buildTag("td", "", {}, {});
                var $parameterradio02 = ej.buildTag("input", "", {}, { 'type': 'radio', 'name': parameter.Name, 'id': parameter.ControlId + '_02' });
                $parametertd02.append($parameterradio02);
                $parametertd02.append("&nbsp;<span>False</span>");
                if (parameter.Label != null && parameter.Label.length > 0) {
                    propStat = !propStat;
                }
                $parameterradio02.ejRadioButton({ checked: propStat });
                $parametertr.append($parametertd02);

                $contentDiv.append($parametertable);

                if (parameter.IsNullable) {
                    $parameterSpan = ej.buildTag("span", "", { 'WHITE-SPACE': 'nowrap', 'display': 'block' }, {});
                    $parameterNull = ej.buildTag("input", "", {}, { 'type': 'checkbox', 'name': 'chkRadio', 'id': parameter.ControlId + '~chk', 'value': this._id });
                    $parameterSpan.append($parameterNull);
                    $parameterSpan.append('&nbsp;<span class=e-reportviewer-label>Null</span>');
                    $contentDiv.append('&nbsp;');
                    $contentDiv.append($parameterSpan);
                    $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    if (parameter.Label != null && parameter.Label.length > 0) {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    } else {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange, checked: true });
                        this._paramElementDisable(parameter.ControlId, 'chkRadio', true);
                    }
                }
            } else {
                var $parametertxtbx = ej.buildTag("input.e-reportviewer-textbox", "", {}, { 'type': 'text', 'autocomplete': 'off', 'id': parameter.ControlId, 'maxlength': '256', 'name': parameter.Name, 'value': parameter.Label });
                $contentDiv.append($parametertxtbx);
                if (parameter.IsNullable) {
                    $parametertxtbx.css('width', '75%');
                    $parameterSpan = ej.buildTag("span", "", { 'WHITE-SPACE': 'nowrap', 'display': 'block' }, {});
                    $parameterNull = ej.buildTag("input", "", {}, { 'type': 'checkbox', 'name': 'chkTextbox', 'id': parameter.ControlId + '~chk', 'value': this._id });
                    $parameterSpan.append($parameterNull);
                    $parameterSpan.append('&nbsp;<span class=e-reportviewer-label>Null</span>');
                    $contentDiv.append('&nbsp;');
                    $contentDiv.append($parameterSpan);
                    $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    if (parameter.Label != null && parameter.Label.length > 0) {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange });
                    } else {
                        $parameterNull.ejCheckBox({ "change": this._paramNullChange, checked: true });
                        this._paramElementDisable(parameter.ControlId, 'chkTextbox', true);
                    }
                } else {
                    $parametertxtbx.css('width', '96%');
                }

                if (parameter.IsDependent) {
                    this._on($parametertxtbx, "change", this._paramsChangeEvent);
                }
            }
        },

        _selectionChanged: function (event) {
            var activeItem = this._activeItem;
            if (this.selectedIndexValue == 0 && this.selectedTextValue == "Select All") {
                if (this.checkedStatus) {
                    for (var index = 1; index < this.listitems.length; index++) {
                        this._setModel({ selectedIndex: index });
                    };
                }
                else {
                    for (var index = 1; index < this.listitems.length; index++) {
                        this.unselectItemsByIndices(index);
                    };
                }
            }
            else if (this._selectedIndices.length == this.listitems.length - 1) {
                if (this.checkedStatus) {
                    this._setModel({ selectedIndex: 0 });
                }
                else {
                    this.unselectItemsByIndices(0);
                }
            }
            this._activeItem = activeItem;
        },

        _updateParamElements: function (parameters) {
            var isRender = this._onRenderingBegin({ reportParameters: $.extend(true, {}, parameters) });
            if (!isRender) {
                for (var index = 0; index < parameters.length; index++) {
                    var parameter = parameters[index];
                    var targetTag = $('#' + parameter.ControlId + '_div');
                    if (parameter.IsMultiValue || parameter.AvailableValues != null || parameter.DataType == "Boolean") {
                        targetTag.empty();
                        parameter.Labels = parameter.Values = null;
                        this._renderParameterElements(parameter, targetTag);
                    } else if (parameter.DataType == "DateTime") {
                        var ejDatePick = $('#' + parameter.ControlId).data("ejDatePicker");
                        ejDatePick.setModel({ value: parameter.Label });
                    } else {
                        $('#' + parameter.ControlId).val(parameter.Label);
                    }
                }
            }
            this._onRenderingComplete({ reportParameters: $.extend(true, {}, parameters) });
            return isRender;
        },

        _paramsChangeEvent: function (event) {
            var ejViewerId = this._id.substr(0, (this._id.lastIndexOf('Param') - 1));
            var proxy = $('#' + ejViewerId).data('ejReportViewer');
            var updateParam = {
                Name: '',
                Labels: [],
                Values: []
            };
            var targetTag = $('#' + this._id);
            updateParam.Name = targetTag.attr('name');
            if (this.pluginName == 'ejDropDownList') {
                if (updateParam.Name == undefined) {
                    updateParam.Name = targetTag.attr('sf-name');
                }
                if (this.model.showCheckbox) {
                    for (var optIndex = 0; optIndex < this.model.selectedIndices.length; optIndex++) {
                        if (this.model.selectedIndices[optIndex] != 0 && this.selectOptionItems[this.model.selectedIndices[optIndex]].value != "parameter_SelectAll") {
                            updateParam.Labels.push(this.selectOptionItems[this.model.selectedIndices[optIndex]].text);
                            updateParam.Values.push(this.selectOptionItems[this.model.selectedIndices[optIndex]].value);
                        }
                    }
                } else {
                    updateParam.Labels.push(this.selectOptionItems[this.model.selectedIndex].text);
                    updateParam.Values.push(this.selectOptionItems[this.model.selectedIndex].value);
                }
            } else if (this.pluginName == 'ejDatePicker') {
                var dateTime = targetTag.data("ejDatePicker").getValue();
                updateParam.Labels.push(dateTime);
                updateParam.Values.push(dateTime);
            } else if (this.pluginName == 'ejRadioButton') {
                var isCheck = data("ejRadioButton").model.checked;
                updateParam.Labels.push(isCheck);
                updateParam.Values.push(isCheck);
            } else {
                updateParam.Labels.push(targetTag.val());
                updateParam.Values.push(targetTag.val());
            }
            proxy.doAjaxPost("POST", proxy._actionUrl, JSON.stringify({ 'reportAction': proxy._reportAction.updateParameters, 'updateParam': updateParam }), "_updateParameters");
        },

        _viewReportParamsClick: function (event) {
            this._showloadingIndicator(true);
            var parameters = event.data.params;
            var reportParams = this._getParameterJson(parameters, true);
            if (reportParams) {
                this._viewReportEnableDisable(true);
                this._refresh = true;
                $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcontent table:first').attr('isviewclick', 'true');
                this._onViewReportClick(reportParams);
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.setParameters, 'parameters': reportParams }), "_setParameters");
            }
        },

        _viewReportEnableDisable: function (enable) {
            $('#' + this._id + '_viewReportClick').attr('disabled', enable)
        },

        _getDefaultParameters: function (defaultparameters) {
            var values = [];
            var labels = [];
            for (var parameterIndex = 0; parameterIndex < defaultparameters.length; defaultparameters++) {
                if (defaultparameters[parameterIndex].DefaultValues) {
                    for (var optIndex = 0; optIndex < defaultparameters[parameterIndex].DefaultValues.length; optIndex++) {
                        labels.push(defaultparameters[0].DefaultValues[optIndex]);
                        values.push(defaultparameters[0].DefaultValues[optIndex]);
                    }
                    var reportParams = [{ Name: defaultparameters[parameterIndex].Name, Values: values, Labels: labels, Nullable: defaultparameters[parameterIndex].IsNullable }];
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.setParameters, 'parameters': reportParams }), "_setParameters");
                }
            }
        },

        _getParameterJson: function (parameters, isUIUpdate) {
            var reportParams = [];
            for (var index = 0; index < parameters.length; index++) {
                var parameter = parameters[index];
                var values = [];
                var labels = [];
                var nullable = false;
                var isNull;
                if (parameter.IsMultiValue) {
                    var ddl = $('#' + parameter.ControlId).data("ejDropDownList");
                    var selectedIndex = ddl._selectedIndices; 
                    for (var optIndex = 0; optIndex < selectedIndex.length; optIndex++) {
                        if (selectedIndex[optIndex] != 0 && ddl.selectOptionItems[selectedIndex[optIndex]].value != "parameter_SelectAll") {
                            labels.push(ddl.selectOptionItems[selectedIndex[optIndex]].text);
                            values.push(ddl.selectOptionItems[selectedIndex[optIndex]].value);
                        }
                    }
                } else if (parameter.AvailableValues != null) {
                    if (parameter.IsNullable) {
                        nullable = parameter.IsNullable;                                                       
                    }
                    var ddlVal = $('#' + parameter.ControlId).data("ejDropDownList").getSelectedValue();
                    var ddltxt = $('#' + parameter.ControlId).data("ejDropDownList").getSelectedItem();
                    ddlVal = (ddlVal == "") ? parameter.Value : ddlVal;
                    ddltxt = (ddltxt && $(ddltxt).text() == "") ? parameter.Label : $(ddltxt).text();
                    labels.push(ddltxt);
                    values.push(ddlVal);
                } else if (parameter.DataType == "DateTime") {
                    isNull = $('#' + parameter.ControlId + '_div').find('.e-chkbox-wrap').attr('aria-checked');
                    if (isNull == 'true') {
                        nullable = true;
                    } else {
                        var dateTime = $('#' + parameter.ControlId).data("ejDatePicker").getValue();
                        labels.push(dateTime);
                        values.push(dateTime);
                    }
                } else if (parameter.DataType == "Boolean") {
                    var radio1 = $('#' + parameter.ControlId + '_01').data("ejRadioButton");
                    var radio2 = $('#' + parameter.ControlId + '_02').data("ejRadioButton");
                    isNull = $('#' + parameter.ControlId + '_div').find('.e-chkbox-wrap').attr('aria-checked');
                    if (isNull == 'true') {
                        nullable = true;
                    } else if (radio1.model.checked || radio2.model.checked) {
                        labels.push(radio1.model.checked);
                        values.push(radio1.model.checked);
                    }
                } else {
                    isNull = $('#' + parameter.ControlId + '_div').find('.e-chkbox-wrap').attr('aria-checked');
                    if (isNull == 'true') {
                        nullable = true;
                    } else {
                        var val = $('#' + parameter.ControlId).val();
                        if (isUIUpdate) {
                            if (parameter.DataType == "Float" && !val.match('^[-+]?[0-9]+\.[0-9]+$')) {
                                alert('Please give the float data type input');
                                return false;
                            } else if (parameter.DataType == "Integer" && !val.match('^[-+]?[0-9]+$')) {
                                alert('Please give the integer data type input');
                                return false;
                            }
                        }
                        labels.push(val);
                        values.push(val);
                    }
                }

                var parameterValue = { Name: parameter.Name, Values: values, Labels: labels, Nullable: nullable };
                reportParams.push(parameterValue);
            }
            return reportParams;
        },

        _paramNullChange: function (event) {
            var id = event.model.id.split('~');
            var ejReport = $('#' + event.model.value).data("ejReportViewer");
            ejReport._paramElementDisable(id[0], event.model.name, event.isChecked);
        },

        _paramElementDisable: function (elementId, elementType, isEnable) {
            if (elementType == 'chkDateTime') {
                var dateTime = $('#' + elementId).data("ejDatePicker");
                if (isEnable) {
                    dateTime.disable();
                } else {
                    dateTime.enable();
                }
            } else if (elementType == 'chkTextbox') {
                if (isEnable) {
                    $('#' + elementId).attr('disabled', 'disabled');
                    $('#' + elementId).css('opacity', '0.45');
                } else {
                    $('#' + elementId).removeAttr('disabled');
                    $('#' + elementId).css('opacity', '1');
                }
            } else if (elementType == 'chkRadio') {
                var radio1 = $('#' + elementId + '_01').data("ejRadioButton");
                var radio2 = $('#' + elementId + '_02').data("ejRadioButton");
                if (isEnable) {
                    radio1.disable();
                    radio2.disable();
                } else {
                    radio1.enable();
                    radio2.enable();
                }
            }
        },

        _selectparamToolItem: function (enable) {
            if (!this.model.toolbarSettings.templateId) {
                var _ejToolbar = $('#' + this._id + '_toolbarContainer').data("ejToolbar");
                if (enable) {
                    _ejToolbar.selectItemByID(this._id + '_ejtb_parameter');
                } else {
                    _ejToolbar.deselectItemByID(this._id + '_ejtb_parameter');
                }
            }
        },

        _showParameterBlock: function (enable) {
            if (enable) {
                $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display", "block");
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-parameter').parent().css("display", "block");
            } else {
                $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display", "none");
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-parameter').parent().css("display", "none");
            }

            if (this._isDevice) {
                this._showViewerBlock(enable);
                this._showViewerPage(!enable);
            }
        },

        _toggleParameterBlock: function () {
            this._selectparamToolItem(!($('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display") == 'block'));
            $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').toggle();
            this._setContainerSize();

            if (this._isDevice) {
                var isEnable = $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display") == "block";
                this._showViewerBlock(isEnable);
                this._showViewerPage(!isEnable);
            }
        },
        //-------------------- Render Parameter Block and Actions [End] -------------------------//

        //-------------------- Render Exception Block and Actions [Start] -------------------------//
        _renderExceptionBlock: function (errmsg) {
            this._resetExceptionBlock();
            $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').css("display", "none");
            var table = ej.buildTag("table.errmsg", "", {}, {});
            var $row = ej.buildTag("tr", "", {}, {});
            var $cell = ej.buildTag("td.e-reportviewer-viewerblockcellcontent", "", {}, {});
            $row.append($cell);
            $cell.append("<h5>Report Viewer encountered some issues loading this report. Please click <a id=" + this._id + '_viewBlockContainer_errmsg' + " href=javascript:void(0); >here</a>  to see the error details</h5>");
            table.append($row);
            $('#' + this._id + '_viewBlockContainer').append(table);
            this._on($('#' + this._id + '_viewBlockContainer_errmsg'), "click", { err: errmsg }, this._errorPopupblock);
        },

        _errorPopupblock: function (event) {
            alert(event.data.err);
        },

        _resetExceptionBlock: function () {
            $('#' + this._id + '_viewBlockContainer table.errmsg').remove();
        },

        _renderExcpetion: function (exception) {
            this._showloadingIndicator(false);
            //$('#' + this._id + '_viewerContainer').html('Exception:' + exception);
            this._renderExceptionBlock(exception);
            this._showViewerBlock(true);
        },
        //-------------------- Render Exception Block and Actions [End] -------------------------//

        //-------------------- Apply Page Style and Actions [Start] -------------------------//
        _setContainerSize: function () {
            var ctrlheight = $('#' + this._id).height();
            
            
            
            var viewerblockHeight = 0;
            if (!this._isDevice) {
                var viewerContainer = $('#' + this._id + '_viewBlockContainer');
                if (viewerContainer.css('display').toLowerCase() == 'block') {
                    viewerblockHeight = $('#' + this._id + '_viewBlockContainer').height();
                }
            }

            var toolbarHeight = this.model.toolbarSettings.templateId ? $('#' + this.model.toolbarSettings.templateId).height() : $('#' + this._id + '_toolbarContainer').height();

            var calcHeight = ctrlheight - viewerblockHeight - toolbarHeight - 4;
            $('#' + this._id + '_viewerContainer').css({ "height": calcHeight + "px" });
        },

        _setPageSize: function (pageHeight, pageWidth, headerHeight, footerHeight) {
            var pageSizeWidth = null, pageSizeHeight = null;
            var totalpageHeight = pageHeight + headerHeight + footerHeight + 3;
            var browserInfo = ej.browserInfo();

            $('#' + this._id + '_pageviewContainer').css({ "width": pageWidth + "px", "height": totalpageHeight + "px" });
            $('#' + this._id + '_pageviewheaderContainer').css({ "width": pageWidth + "px", "height": headerHeight + "px" });
            $('#' + this._id + '_pageviewfooterContainer').css({ "width": pageWidth + "px", "height": footerHeight + "px" });
            $('#' + this._id + '_pageviewbodyContainer').css({ "height": pageHeight + "px", "width": pageWidth + "px" });
            var scalePageSize = $('#' + this._id + '_pageviewContainer')[0].getBoundingClientRect();

            if (browserInfo.name == 'msie' && browserInfo.version == 8) {
                pageSizeWidth = scalePageSize.right;
                pageSizeHeight = scalePageSize.bottom;
            }
            else {
                pageSizeWidth = scalePageSize.width;
                pageSizeHeight = scalePageSize.height;
            }
            $('#' + this._id + '_pageviewOuterline').css({ "width": pageSizeWidth + "px", "height": pageSizeHeight + "px" });
        },

        _renderPageModels: function (pageData) {
            $('#' + this._id + '_pageviewheaderContainer').empty();
            $('#' + this._id + '_pageviewfooterContainer').empty();
            $('#' + this._id + '_pageviewbodyContainer').empty();
            if (pageData) {
                if (pageData.HeaderStyleModel) {
                    this._applyPageStyle($('#' + this._id + '_pageviewContainer .pageHeaderBorder'), pageData.HeaderStyleModel, false);
                }

                if (pageData.FooterStyleModel) {
                    this._applyPageStyle($('#' + this._id + '_pageviewContainer .pageFooterBorder'), pageData.FooterStyleModel, false);
                }

                if (pageData.BodyStyleModel) {
                    this._applyPageStyle($('#' + this._id + '_pageviewContainer .pageBodyBorder'), pageData.BodyStyleModel, false);
                }

                if (pageData.HeaderModel && pageData.HeaderModel.length > 0) {
                    this._renderPageControls(pageData.HeaderModel, $('#' + this._id + '_pageviewheaderContainer'), false, null);
                }

                if (pageData.FooterModel && pageData.FooterModel.length > 0) {
                    this._renderPageControls(pageData.FooterModel, $('#' + this._id + '_pageviewfooterContainer'), false, null);
                }

                if (pageData.PageModel && pageData.PageModel.length > 0) {
                    this._renderPageControls(pageData.PageModel, $('#' + this._id + '_pageviewbodyContainer'), false, null);
                }
            }

            if ($('.txtToggle_' + this._id).length > 0) {
                this._drillAction();
            }

            if ($('.drillAction_' + this._id).length > 0) {
                this._drillThroughAction();
            }

            if ($('.txtSorting_' + this._id).length > 0) {
                this._sortingAction();
            }

            if (this._isDocumentMap && !this._printMode) {
                this._documentMapAction();
                this._isDocumentMap = false;
                this._pageDocMapFlag = true;
            }

            if (this._parentPageXY) {
                $('#' + this._id + '_viewerContainer').animate({ scrollTop: this._parentPageXY.y, scrollLeft: this._parentPageXY.x }, 1000);
                this._parentPageXY = null;
            }

            if (this._isDevice) {
                this._renderPageInfoPopup();
            }

            if (this.model.printMode) {
                $('#' + this._id + '_pageviewContainer').css({ 'padding-left': this._pageModel.MarginLeft, 'padding-top': this._pageModel.MarginTop, 'padding-right': this._pageModel.MarginRight, 'padding-bottom': this._pageModel.MarginBottom });
            } else {
                $('#' + this._id + '_pageviewContainer').css('padding', '0px');
            }
        },

        _applyPageStyle: function (page, styleModel, printMode) {
            var resourceName = printMode ? 'PrintImage' : 'GetResource';
            page.css("background-color", styleModel.BackgroundColor);
            if (styleModel.BackgroundSrc) {
                page.css("background-image", "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + styleModel.BackgroundSrc + '&resourcetype=sfimg&isPrint=' + printMode + ")");
            }
            if (styleModel.Border) {
                this._applyBorderStyle(styleModel.Border, page);
            }
        },

        _renderPageControls: function (pageModelItems, renderArea, printMode, printPageId) {
            var $reportItemsContainer = ej.buildTag("div", "", {}, {});
            renderArea.html($reportItemsContainer);
            for (var index = 0; index < pageModelItems.length; index++) {
                var reportItem = pageModelItems[index];
                if (reportItem.ItemType == "TextBoxModel") {
                    var textboxControl = this._renderTextBoxControl(reportItem, false, null, printMode, printPageId);
                    $reportItemsContainer.append(textboxControl);
                }
                else if (reportItem.ItemType == "LineModel") {
                    var lineControl = this._renderLineControl(reportItem, null, printMode, printPageId);
                    $reportItemsContainer.append(lineControl);
                }
                else if (reportItem.ItemType == "ImageModel") {
                    this._renderImageControl(reportItem, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "TablixModel") {
                    this._renderTablixControl(reportItem, false, {}, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "GaugeModel") {
                    this._renderGaugeControl(reportItem, false, {}, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "RectangleModel") {
                    this._renderRectangleControl(reportItem, false, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "ChartModel") {
                    this._renderChartControl(reportItem, false, {}, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "MapModel") {
                    this._renderMapControl(reportItem, $reportItemsContainer, null, printMode, printPageId);
                }
                else if (reportItem.ItemType == "SubReportModel") {
                    this._renderSubReportControl(reportItem, false, $reportItemsContainer, null, printMode, printPageId);
                }
            }
        },
        //-------------------- Apply Page Style and Actions [End] -------------------------//

        //-------------------- Render the Page ReportItems [start] -------------------------//
        _controlKeyGenerator: function (currentKey, parentId, printMode, printPageId) {
            var objId = !ej.isNullOrUndefined(parentId) ? '^' + parentId : '';
            if (printMode) {
                objId = objId.length == 0 ? '^' + printPageId : objId + '_' + printPageId;
            }
            return currentKey + objId;
        },

        _applyBorderStyle: function (border, tag) {
            if (border.Default) {
                tag.css("border-width", border.Default.Thickness + 'px');
                tag.css("border-color", border.Default.BorderBrush);
                tag.css("border-style", border.Default.BorderStyle);
            }

            if (border.LeftBorder) {
                tag.css("border-left-width", border.LeftBorder.Thickness + 'px');
                tag.css("border-left-color", border.LeftBorder.BorderBrush);
                tag.css("border-left-style", border.LeftBorder.BorderStyle);
            }

            if (border.RightBorder) {
                tag.css("border-right-width", border.RightBorder.Thickness + 'px');
                tag.css("border-right-color", border.RightBorder.BorderBrush);
                tag.css("border-right-style", border.RightBorder.BorderStyle);
            }

            if (border.TopBorder) {
                tag.css("border-top-width", border.TopBorder.Thickness + 'px');
                tag.css("border-top-color", border.TopBorder.BorderBrush);
                tag.css("border-top-style", border.TopBorder.BorderStyle);
            }

            if (border.BottomBorder) {
                tag.css("border-bottom-width", border.BottomBorder.Thickness + 'px');
                tag.css("border-bottom-color", border.BottomBorder.BorderBrush);
                tag.css("border-bottom-style", border.BottomBorder.BorderStyle);
            }
        },

        _applyTextStyle: function (textboxModel, tag, textbox) {
            if (textboxModel.Padding) {
                if (textboxModel.Padding.Top) {
                    tag.css("padding-top", textboxModel.Padding.Top);
                }

                if (textboxModel.Padding.Right) {
                    tag.css("padding-right", textboxModel.Padding.Right);
                }

                if (textboxModel.Padding.Bottom) {
                    tag.css("padding-bottom", textboxModel.Padding.Bottom);
                }

                if (textboxModel.Padding.Left) {
                    tag.css("padding-left", textboxModel.Padding.Left);
                }
            }

            if (textboxModel.TextBoxBackgroundColor) {
                tag.css("backgroundColor", textboxModel.TextBoxBackgroundColor);
            }

            if (textboxModel.VerticalAlign && textboxModel.VerticalAlign != 'Default') {
                tag.css("vertical-align", textboxModel.VerticalAlign);
            }

            if (textboxModel.Border) {
                this._applyBorderStyle(textboxModel.Border, tag);
            }

            var tagName = tag.get(0).tagName;
            if (tagName.toLowerCase() == 'td') {
                var padwidthadjust = parseFloat(tag.css("padding-left"), 10) + parseFloat(tag.css("padding-right"), 10);
                var padheightadjust = parseFloat(tag.css("padding-top"), 10) + parseFloat(tag.css("padding-bottom"), 10);

                var borderwidthadjust = parseFloat(tag.css("border-left-width"), 10) + parseFloat(tag.css("border-right-width"), 10);
                var borderheightadjust = parseFloat(tag.css("border-top-width"), 10) + parseFloat(tag.css("border-bottom-width"), 10);

                var adjusWidth = (parseFloat(tag.width(), 10) - (padwidthadjust + borderwidthadjust));
                var adjusHeight = (parseFloat(tag.height(), 10) - (padheightadjust + borderheightadjust));

                tag.css('width', adjusWidth + 'px');
                tag.css('height', adjusHeight + 'px');
                if (!textboxModel.ToggleInfo) {
                    $(textbox).children().width(adjusWidth);
                    $(textbox).children().height(adjusHeight);
                }
            }
        },

        //-------------------- Render the Tablix ReportItem [start] -------------------------//
        _renderTablixControl: function (tablixmodel, isTablixCell, cellSize, renderarea, parentId, printMode, printPageId) {
            var proxy = this;
            var _height = tablixmodel.Height;
            var _width = tablixmodel.Width;
            var ctrlId = proxy._controlKeyGenerator(proxy._id + '_' + tablixmodel.Name, parentId, printMode, printPageId);
            var tablixDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            var _tablixDivCss = {};
            renderarea.append(tablixDiv);
            _tablixDivCss["top"] = tablixmodel.Top;
            _tablixDivCss["left"] = tablixmodel.Left;

            if (isTablixCell) {
                _height = cellSize.height;
                var borderwidthadjust = parseFloat(tablixDiv.css("border-left-width"), 10) + parseFloat(tablixDiv.css("border-right-width"), 10);
                var borderheightadjust = parseFloat(tablixDiv.css("border-top-width"), 10) + parseFloat(tablixDiv.css("border-bottom-width"), 10);
                _width = (_width - (borderwidthadjust));
                _height = (_height - (borderheightadjust));
                _tablixDivCss["position"] = 'relative';
            } else {
                _tablixDivCss["position"] = 'absolute';
            }

            _tablixDivCss["width"] = _width;
            _tablixDivCss["height"] = _height;

            var tablixtable = ej.buildTag("table", "", { 'border-collapse': 'collapse', 'font-size': '8pt' }, { 'cellspacing': '0', 'cellpadding': '0' });
            var _tablixtableCss = {};
            tablixDiv.append(tablixtable);

            if (tablixmodel.BackgroundColor) {
                _tablixtableCss["background-color"] = tablixmodel.BackgroundColor;
            }

            if (tablixmodel.StyleVal) {
                if (tablixmodel.StyleVal.Font) {
                    if (tablixmodel.StyleVal.Font.FontFamily) {
                        _tablixtableCss["font-family"] = tablixmodel.StyleVal.Font.FontFamily;
                    }

                    if (tablixmodel.StyleVal.Font.FontSize) {
                        _tablixtableCss["font-size"] = tablixmodel.StyleVal.Font.FontSize;
                    }

                    if (tablixmodel.StyleVal.Font.FontStyle) {
                        _tablixtableCss["font-style"] = tablixmodel.StyleVal.Font.FontStyle;
                    }

                    if (tablixmodel.StyleVal.Font.FontWeight) {
                        _tablixtableCss["font-weight"] = tablixmodel.StyleVal.Font.FontWeight + ' !important';
                    }
                }

                _tablixtableCss["color"] = tablixmodel.StyleVal.TextColor ? tablixmodel.StyleVal.TextColor : 'black';

                if (tablixmodel.StyleVal.TextDecoration) {
                    _tablixtableCss["text-decoration"] = tablixmodel.StyleVal.TextDecoration;
                }
            }

            if (tablixmodel.Padding) {
                if (tablixmodel.Padding.Left) {
                    _tablixtableCss["padding-left"] = tablixmodel.Padding.Left;
                }

                if (tablixmodel.Padding.Right) {
                    _tablixtableCss["padding-right"] = tablixmodel.Padding.Right;
                }

                if (tablixmodel.Padding.Top) {
                    _tablixtableCss["padding-top"] = tablixmodel.Padding.Top;
                }

                if (tablixmodel.Padding.Bottom) {
                    _tablixtableCss["padding-bottom"] = tablixmodel.Padding.Bottom;
                }
            }

            //if (tablixmodel.LineHeight) {
            //    tablixtable.css("line-height", tablixmodel.LineHeight);
            //}

            if (tablixmodel.TextAlign) {
                _tablixtableCss["text-align"] = tablixmodel.TextAlign;
            }

            if (tablixmodel.VerticalAlign) {
                _tablixtableCss["vertical-align"] = tablixmodel.VerticalAlign;
            }

            var resourceName = printMode ? 'PrintImage' : 'GetResource';
            if (tablixmodel.BackgroundImageSrc) {
                _tablixtableCss["background-image"] = "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + tablixmodel.BackgroundImageSrc + '&resourcetype=sfimg&isPrint=' + printMode + ")";
            }

            if (tablixmodel.Border) {
                var _topBorder = tablixmodel.Border.TopBorder;
                tablixmodel.Border.TopBorder = null;
                this._applyBorderStyle(tablixmodel.Border, tablixtable);
                if (_topBorder) {
                    _tablixDivCss["border-top-width"] = _topBorder.Thickness + 'px';
                    _tablixDivCss["border-top-color"] = _topBorder.BorderBrush;
                    _tablixDivCss["border-top-style"] = _topBorder.BorderStyle;
                }
                tablixmodel.Border.TopBorder = _topBorder;
            }
            tablixtable.css(_tablixtableCss);
            tablixDiv.css(_tablixDivCss);

            if (tablixmodel.CellModels && tablixmodel.CellModels.length > 0) {
                var emptyRow = ej.buildTag("tr", "", { 'padding': '0px', 'margin': '0px', 'height': '0px', 'visibility': 'collapse' }, {});
                tablixtable.append(emptyRow);
                var colindex;
                for (colindex = 0; colindex < tablixmodel.CellModels[0].length; colindex++) {
                    var emptyCol = ej.buildTag("td", "", { 'padding': '0px', 'margin': '0px', 'width': tablixmodel.ColWidths[colindex] + 'px' }, { 'rowSpan': '1', 'colSpan': '1' });
                    emptyRow.append(emptyCol);
                }

                for (var rowindex = 0; rowindex < tablixmodel.CellModels.length; rowindex++) {
                    var tableRow = ej.buildTag("tr", "", { 'padding': '0px', 'margin': '0px', 'text-align': 'left', 'height': tablixmodel.RowHeights[rowindex] + 'px' }, { 'vAlign': 'top' });
                    tablixtable.append(tableRow);
                    var row = tablixmodel.CellModels[rowindex];
                    for (colindex = 0; colindex < row.length; colindex++) {
                        var tableCol = ej.buildTag("td", "", { 'padding': '0px', 'margin': '0px', 'text-align': 'left', 'height': '100%' }, { 'rowSpan': '1', 'colSpan': '1' });
                        tableRow.append(tableCol);
                        var cell = row[colindex].ItemModel;
                        var border = row[colindex].Border;
                        var rowSpan = row[colindex].RowSpan;
                        var colSpan = row[colindex].ColSpan;
                        if (border) {
                            proxy._applyBorderStyle(border, tableCol);
                        }

                        if (cell) {
                            if (cell.ItemType == "TextBoxModel") {
                                var textboxControl = proxy._renderTextBoxControl(cell, true, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                                
                                textboxControl.css('max-height', '99.9% !important');
                                tableCol.css('height', tablixmodel.RowHeights[rowindex] + 'px');
                                proxy._applyTextStyle(cell, tableCol, textboxControl);
                                tableCol.append(textboxControl);
                            } else if (cell.ItemType == "LineModel") {
                                var lineControl = proxy._renderLineControl(cell, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                                lineControl.css("position", "relative");
                                tableCol.append(lineControl);
                            } else if (cell.ItemType == "ImageModel") {
                                proxy._renderImageControl(cell, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "TablixModel") {
                                proxy._renderTablixControl(cell, true, { height: tablixmodel.RowHeights[rowindex] }, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "GaugeModel") {
                                proxy._renderGaugeControl(cell, true, { height: tablixmodel.RowHeights[rowindex] }, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "RectangleModel") {
                                proxy._renderRectangleControl(cell, true, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "ChartModel") {
                                this._renderChartControl(cell, true, { height: tablixmodel.RowHeights[rowindex] }, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "MapModel") {
                                this._renderMapControl(cell, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            } else if (cell.ItemType == "SubReportModel") {
                                proxy._renderSubReportControl(cell, true, tableCol, ctrlId + 'tablixRow' + rowindex + 'xCol' + colindex, printMode, printPageId);
                            }

                            if (rowSpan > 1) {
                                tableCol.attr("rowspan", rowSpan);
                            }

                            if (colSpan > 1) {
                                tableCol.attr("colspan", colSpan);
                            }
                        } else {
                            tableCol.css('display', 'none');
                        }
                    }
                }
            }
        },
        //-------------------- Render the Tablix ReportItem [end] -------------------------//
        _renderTextBoxControl: function (textboxModel, isTablixCell, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + textboxModel.Name, parentId, printMode, printPageId);

            var textboxDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, title: textboxModel.ToolTip });
            var _textboxCss = {};
            if (!isTablixCell) {
                _textboxCss["position"] = 'absolute';
                _textboxCss["top"] = textboxModel.Top;
                _textboxCss["left"] = textboxModel.Left;
                _textboxCss["width"] = textboxModel.Width;
                _textboxCss["height"] = textboxModel.Height;
                this._applyTextStyle(textboxModel, textboxDiv, null);
            } else if (isTablixCell) {
                var browserInfo = ej.browserInfo();
                if (browserInfo.name == 'opera' || browserInfo.name == 'webkit') {
                    _textboxCss["word-wrap"] = 'break-word';
                    _textboxCss["white-space"] = 'pre-wrap';
                }  else if (browserInfo.name == 'msie') {
                    _textboxCss["word-wrap"] = 'break-word';
                    _textboxCss["white-space"] = 'pre-wrap';
                }
                if (!textboxModel.CanGrow) {
                    _textboxCss["overflow"] = 'hidden';
                }
            }

            var resourceName = printMode ? 'PrintImage' : 'GetResource';
            if (textboxModel.BackgroundSrc) {
                _textboxCss["background-image"] = "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + textboxModel.BackgroundSrc + '&resourcetype=sfimg&isPrint=' + printMode + ")";
            }

            if (textboxModel.ToggleInfo && !printMode) {
                var cssToggleName = textboxModel.IsToggle ? 'e-reportviewer-collapsetoggle' : 'e-reportviewer-expandtoggle';
                var toogleiconDiv = ej.buildTag('div.' + cssToggleName + ' txtToggle_' + this._id, '', { 'display': 'block', 'width': '14px', 'height': '14px', 'margin-Top': '1px' }, {});
                var _objToogleDivCss = {};
                var browserInfo = ej.browserInfo();
                if (browserInfo.name == 'msie') {
                    _objToogleDivCss["margin-right"] = '5px';
                    _objToogleDivCss["float"] = 'left';
                } else if (browserInfo.name == 'opera' || browserInfo.name == 'webkit' || browserInfo.name == 'chrome' || browserInfo.name == 'mozilla') {
                    _objToogleDivCss["display"] = 'table';
                    _objToogleDivCss["margin-right"] = '5px';
                    _objToogleDivCss["float"] = 'left';
                }

                toogleiconDiv.data('drillObj', textboxModel.ToggleInfo);
                toogleiconDiv.css(_objToogleDivCss);
                textboxDiv.append(toogleiconDiv);
                _textboxCss["display"] = browserInfo.name == 'msie' ? 'inline-block' : 'flex';
            }

            if (textboxModel.ActionInfo) {
                textboxDiv.data('actionObj', textboxModel.ActionInfo);
                textboxDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
            }

            if (textboxModel.Paragraphval) {
                for (var indexi = 0; indexi < textboxModel.Paragraphval.length; indexi++) {
                    var $textboxParaDiv = ej.buildTag("div.e-reportviewer-paragrap", "", {});
                    var _objParaDivCss = {};
                    var paragraph = textboxModel.Paragraphval[indexi];
                    for (var indexj = 0; indexj < paragraph.Runs.length; indexj++) {
                        var run = paragraph.Runs[indexj];
                        var $textboxSpanDiv = ej.buildTag("span", "", {});
                        var _objSpanDivCss = {};

                        //if (paragraph.HangingIndent && paragraph.HangingIndent != 0) {
                        //    $textboxSpanDiv.css("text-indent", paragraph.HangingIndent)
                        //}                    

                        if (run.ActionInfo) {
                            $textboxSpanDiv.data('actionObj', run.ActionInfo);
                            $textboxSpanDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
                        }

                        if (run.Style.Font.FontFamily) {
                            _objSpanDivCss["font-family"] = run.Style.Font.FontFamily;
                        }

                        if (run.Style.Font.FontSize) {
                            _objSpanDivCss["font-size"] = run.Style.Font.FontSize;
                        }

                        if (run.Style.Font.FontStyle) {
                            _objSpanDivCss["font-style"] = run.Style.Font.FontStyle;
                        }

                        if (run.Style.Font.FontWeight) {
                            _objSpanDivCss["font-weight"] = run.Style.Font.FontWeight;
                        }

                        if (run.Style.TextColor) {
                            _objSpanDivCss["color"] = run.Style.TextColor;
                        } else {
                            _objSpanDivCss["color"] = 'black';
                        }

                        if (run.Style.TextDecoration) {
                            _objSpanDivCss["text-decoration"] = run.Style.TextDecoration;
                        }

                        if (run.RunText) {
                            $textboxSpanDiv.append(run.RunText);
                        }
                        $textboxSpanDiv.css(_objSpanDivCss);
                        $textboxParaDiv.append($textboxSpanDiv);
                    }

                    if (paragraph.TextAlignment) {
                        _objParaDivCss["text-align"] = paragraph.TextAlignment;
                    }

                    if (textboxModel.Direction != "Default") {
                        if (textboxModel.Direction != "LTR") {
                            _objParaDivCss["text-align"] = 'right';
                        }
                    }

                    if (textboxModel.VerticalAlign != "Default") {
                        _objParaDivCss["vertical-align"] = textboxModel.VerticalAlign;
                        if (!isTablixCell) {
                            _textboxCss["display"] = 'table';
                            _objParaDivCss["display"] = 'table-cell';
                        }
                    }

                    if (paragraph.LeftIndent) {
                        _objParaDivCss["padding-left"] = paragraph.LeftIndent;
                    }

                    if (paragraph.RightIndent) {
                        if (paragraph.TextAlignment == "Right") {
                            _objParaDivCss["padding-right"] = paragraph.RightIndent;
                            _objParaDivCss["float"] = 'right';
                        }
                        else {
                            _objParaDivCss["padding-right"] = paragraph.RightIndent;
                        }                        
                    }

                    if (paragraph.SpaceBefore) {
                        _objParaDivCss["padding-top"] = paragraph.SpaceBefore;
                    }

                    if (paragraph.SpaceAfter) {
                        _objParaDivCss["padding-bottom"] = paragraph.SpaceAfter;
                    }

                    if (paragraph.ListStyle) {
                        _objParaDivCss["list-style-type"] = paragraph.ListStyle;
                    }

                    if (paragraph.HangingIndent) {
                        _objParaDivCss["text-indent"] = paragraph.HangingIndent;
                    }

                    if (isTablixCell) {
                        _objParaDivCss["min-width"] = '99.9%';
                    }

                    if (paragraph.ListLevel && paragraph.ListLevel != 0 && textboxModel.Direction != "RTL") {
                        _objParaDivCss["margin-left"] = paragraph.ListLevel * '30';
                    }
                    else if (paragraph.ListLevel != 0 && textboxModel.Direction == "RTL") {
                        _objParaDivCss["margin-Right"] = paragraph.ListLevel * '30';
                        _objParaDivCss["direction"] = 'rtl';
                    }

                    if (paragraph.ListStyle == "Numbered" && paragraph.ListLevel >= 1) {
                        if (paragraph.ListLevel == 1 || paragraph.ListLevel == 4 || paragraph.ListLevel == 7) {
                            _objParaDivCss["display"] = 'list-item';
                            _objParaDivCss["list-style-type"] = 'decimal';
                        }
                        else if (paragraph.ListLevel == 2 || paragraph.ListLevel == 5 || paragraph.ListLevel == 8) {
                            _objParaDivCss["display"] = 'list-item';
                            _objParaDivCss["list-style-type"] = 'lower-roman';
                        }
                        else if (paragraph.ListLevel == 3 || paragraph.ListLevel == 6 || paragraph.ListLevel == 9) {
                            _objParaDivCss["display"] = 'list-item';
                            _objParaDivCss["list-style-type"] = 'lower-alpha';
                        }
                    }
                    else if (paragraph.ListStyle == "Bulleted" && paragraph.ListLevel >= 1) {
                        _objParaDivCss["display"] = 'list-item';
                        _objParaDivCss["list-style-type"] = paragraph.ListStyle;
                    }
                    $textboxParaDiv.css(_objParaDivCss);
                    textboxDiv.append($textboxParaDiv);
                }
            }

            if (textboxModel.Sorting) {
                var _sortingType = "sortingUpDown";
                if (textboxModel.Sorting.Sorting == 'UpDown') {
                    _sortingType = "sortingUpDown";
                } else if (textboxModel.Sorting.Sorting == 'Up') {
                    _sortingType = "sortingUp";
                } else {
                    _sortingType = "sortingDown";
                }
                var sortingiconDiv = ej.buildTag('div.e-reportviewer-' + _sortingType + ' txtSorting_' + this._id, '', { 'display': 'block', 'width': '14px', 'height': '14px', 'padding-right': '1px', 'margin-Top': '1px', 'float': 'right', 'vertical-align': 'middle', 'position': 'relative' }, {});
                sortingiconDiv.data('sortingObj', textboxModel.Sorting);
                textboxDiv.find('.e-reportviewer-paragrap').append(sortingiconDiv);
            }
            textboxDiv.css(_textboxCss);
            return textboxDiv;
        },

        //-------------------- Render the Chart ReportItem [start] -------------------------//
        _renderChartControl: function (chartModel, isTablixCell, cellSize, parentObj, parentId, printMode, printPageId) {
            if (chartModel.ChartAreas) {
                var _height = chartModel.Height;
                var _width = chartModel.Width;
                var _parentID = 'chartCtrl';
                var ctrlId = this._controlKeyGenerator(this._id + '_' + chartModel.Name, parentId, printMode, printPageId);
                var chartDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, 'title': chartModel.ToolTip });
                parentObj.append(chartDiv);
                chartDiv.css("position", "absolute");

                if (chartModel.ChartStyle) {
                    if (chartModel.ChartStyle.Border) {
                        this._applyBorderStyle(chartModel.ChartStyle.Border, chartDiv);
                    }

                    if (chartModel.ChartStyle.FillStyle) {
                        chartDiv.css('background-color', chartModel.ChartStyle.FillStyle.BackgroundColor);
                        this._applyBackgroundGradientStyle(chartModel.ChartStyle.FillStyle, chartDiv);
                    }
                    var resourceName = printMode ? 'PrintImage' : 'GetResource';
                    if (chartModel.BackgroundImge) {
                        chartDiv.css({ 'background-image': 'url(' + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + chartModel.BackgroundImge + '&resourcetype=sfimg&isPrint=' + printMode + ')', 'background-size': '100% 100%' });
                    }
                }

                if (isTablixCell) {
                    _parentID = parentId;
                    _height = cellSize.height;
                    var borderwidthadjust = parseFloat(chartDiv.css("border-left-width"), 10) + parseFloat(chartDiv.css("border-right-width"), 10);
                    var borderheightadjust = parseFloat(chartDiv.css("border-top-width"), 10) + parseFloat(chartDiv.css("border-bottom-width"), 10);
                    _width = (_width - (borderwidthadjust));
                    _height = (_height - (borderheightadjust));
                    chartDiv.css("position", "relative");
                }

                var chartAreaHeight = ((_height) / (chartModel.ChartAreas.length));

                if (chartModel.Top) {
                    chartDiv.css("top", chartModel.Top);
                }

                if (chartModel.Left) {
                    chartDiv.css("left", chartModel.Left);
                }

                if (_width) {
                    chartDiv.css("width", _width);
                }

                if (_height) {
                    chartDiv.css("height", _height);
                }

                for (var areaIndex = 0; areaIndex < chartModel.ChartAreas.length; areaIndex++) {
                    var chartArea = chartModel.ChartAreas[areaIndex];
                    var chartChildDiv = null;
                    if (printMode) {
                        chartChildDiv = ej.buildTag("div", "", {}, { 'id': _parentID + '_' + this._id + '_' + chartModel.Name + '_' + chartArea.Name + '_' + printPageId, 'title': chartModel.ToolTip });
                    } else {
                        chartChildDiv = ej.buildTag("div", "", {}, { 'id': _parentID + '_' + this._id + '_' + chartModel.Name + '_' + chartArea.Name, 'title': chartModel.ToolTip });
                    }

                    chartDiv.append(chartChildDiv);
                    chartChildDiv.css("height", chartAreaHeight);
                    //chartChildDiv.css("margin", "1px");
                    chartChildDiv.ejChart();
                    var ejChartCtrl = chartChildDiv.data("ejChart");

                    var ejchartSeries = ejChartCtrl.model.series[0];
                    ejChartCtrl.model.series.pop(ejchartSeries);

                    if (chartArea.Style) {
                        if (chartArea.Style.FillStyle && chartArea.Style.FillStyle.BackgroundColor != "#00ffffff") {
                            ejChartCtrl.model.chartArea.background = chartArea.Style.FillStyle.BackgroundColor;
                        }
                       
                        var resourceName = printMode ? 'PrintImage' : 'GetResource';
                        if (chartArea.BackgroundImg) {
                            ejChartCtrl.model.backGroundImageUrl = (this.model.reportServiceUrl + '/' + resourceName + '/?key=' + chartArea.BackgroundImg+ '&resourcetype=sfimg&isPrint=' + printMode);
                        }

                        if (chartArea.Style.Border && chartArea.Style.Border.Default) {
                            if (chartArea.Style.Border.Default.BorderBrush) {
                                ejChartCtrl.model.chartArea.border.color = chartArea.Style.Border.Default.BorderBrush;
                            }
                            if (chartArea.Style.Border.Default.BorderStyle == "Default" || chartArea.Style.Border.Default.BorderStyle == "None") {
                                ejChartCtrl.model.chartArea.border.width = 0;
                                ejChartCtrl.model.chartArea.border.opacity = 0;
                            }
                            else {
                                ejChartCtrl.model.chartArea.border.width = chartArea.Style.Border.Default.Thickness;
                            }
                        }
                        else {
                            ejChartCtrl.model.chartArea.border.opacity = 1;
                            ejChartCtrl.model.chartArea.border.color = "Transparent";
                        }
                    }

                    if (areaIndex == 0 && chartModel.ChartTiles.length > 0) {
                        var charttile = chartModel.ChartTiles[areaIndex];
                        if (charttile.Caption && charttile.Visible) {
                            ejChartCtrl.model.title.text = charttile.Caption;
                            if (charttile.Style) {
                                if (charttile.Style.Color) {
                                    ejChartCtrl.model.title.font.color = charttile.Style.Color;
                                }

                                if (charttile.Style.Font) {
                                    if (charttile.Style.Font.FontFamily) {
                                        ejChartCtrl.model.title.font.fontFamily = charttile.Style.Font.FontFamily;
                                    }

                                    if (charttile.Style.Font.FontStyle) {
                                        ejChartCtrl.model.title.font.fontStyle = charttile.Style.Font.FontStyle;
                                    }

                                    if (charttile.Style.Font.FontWeight) {
                                        ejChartCtrl.model.title.font.fontWeight = charttile.Style.Font.FontWeight;
                                    }

                                    if (charttile.Style.Font.FontSize != 0) {
                                        ejChartCtrl.model.title.font.size = charttile.Style.Font.FontSize + 'px';
                                    }
                                }
                            }
                        }
                    }

                    for (var seriesIndex = 0; seriesIndex < chartArea.ChartSeries.length; seriesIndex++) {
                        var chartseries = chartArea.ChartSeries[seriesIndex];
                        var seriesObj = $.extend(true, {}, ejchartSeries);
                        var seriesPoints = seriesObj.points[0];
                        seriesObj.points = [];
                        seriesObj.type = chartseries.Type;
                        seriesObj.name = (chartseries.Legend && chartseries.Legend.LegendText) ? chartseries.Legend.LegendText : chartseries.Label;
                        seriesObj.name = (seriesObj.name == "" && chartseries.Name == "EmptySeriesName") ? chartseries.Name : seriesObj.name;

                        if (chartseries.Type == "explodedpie") {
                            seriesObj.type = "pie";
                            seriesObj.explodeAll = true;
                            seriesObj.explodeOffset = 8;
                        }

                        else if (chartseries.Type == "explodeddoughnut") {
                            seriesObj.type = "doughnut";
                            seriesObj.explodeAll = true;
                            seriesObj.explodeOffset = 10;
                        }

                        if (chartseries.Type == "stackingcolumn") {
                            seriesObj.isStacking = true;
                        } else {
                            seriesObj.isStacking = false;
                        }

                        var actionInfos = [];
                        for (var pointIndex = 0; pointIndex < chartseries.PointValues.length; pointIndex++) {
                            var pointObj = $.extend(true, {}, seriesPoints);
                            var point = chartseries.PointValues[pointIndex];
                            pointObj.YValues = [];
                            if (point.X) {
                                pointObj.X = pointObj.x = point.X;
                            } else {
                                pointObj.X = pointObj.x = "0";
                            }

                            if (point.Y) {
                                pointObj.Y = pointObj.y = point.Y;
                                pointObj.YValues.push(pointObj.Y);
                            } else {
                                pointObj.Y = pointObj.y = 0;
                                pointObj.YValues.push(pointObj.Y);
                            }

                            if (point.Low) {
                                pointObj.YValues.push(point.Low);
                            }

                            if (point.High) {
                                pointObj.YValues.push(point.High);
                            }

                            if (point.Start) {
                                pointObj.YValues.push(point.Start);
                            }

                            if (point.End) {
                                pointObj.YValues.push(point.End);
                            }

                            if (chartseries.Type == "bubble") {
                                pointObj.size = 0.5;
                            }

                            if (point.ChartDataLabel && !point.ChartDataLabel.Visible) {
                                pointObj.text = "";
                            }
                            else {
                                if (point.ChartDataLabel && point.ChartDataLabel.Format) {
                                    pointObj.text = point.ChartDataLabel.Format
                                }
                            }

                            if (point.ActionInfo) {
                                actionInfos.push(point.ActionInfo);
                            }

                            if (chartModel.ColorPalette == "Custom") {
                                for (var paletteIndex = 0; paletteIndex < chartModel.CustomPaletteColors.length; paletteIndex++) {
                                    ejChartCtrl.model.colors[paletteIndex] = chartModel.CustomPaletteColors[paletteIndex];
                                }
                            }
                            else {
                                if (!chartseries.Type == "doughnut" || !chartseries.Type == "pie") {
                                    ejChartCtrl.model.colors = [];
                                }
                                else if (point.Style && point.Style.Color) {
                                    ejChartCtrl.model.colors[pointIndex] = point.Style.Color;
                                }
                            }

                            if (point.Style && point.Style.Color) {
                                seriesObj.fill = point.Style.Color;
                            }

                            seriesObj.points.push(pointObj);

                        }

                        if (actionInfos.length > 0) {
                            ejChartCtrl.model.pointRegionClick = this._drillThroughClick;
                            ejChartCtrl.ejreportid = this._id;
                            chartChildDiv.data(ejChartCtrl._id + 'actionObj_Series' + seriesIndex, actionInfos);
                            ejChartCtrl.model.pointRegionMouseMove = function chartSeriesMouseHover(event) {
                                if (ej.isNullOrUndefined($('#' + this._id).data(this._id + 'actionObj_Series' + event.data.region.SeriesIndex))) {
                                    return false;
                                }
                                $('#' + this._id + '_svg_Series' + event.data.region.SeriesIndex + '_Point' + event.data.region.Region.PointIndex).css("cursor", "pointer");
                            };
                        }

                        seriesObj.visibility = chartseries.Visibility ? 'visible' : 'hide';
                        seriesObj.tooltip.format = " x: #point.x#<br/>y: #point.y#";
                        seriesObj.tooltip.visible = true;
                        seriesObj.enableAnimation = false;

                        if (chartModel.CustomPaletteColors && (chartModel.ColorPalette) && chartModel.ColorPalette == "Custom") {
                            seriesObj.fill = chartModel.CustomPaletteColors[seriesIndex];
                        }

                        if (chartseries.Style && chartseries.Style.Border && chartseries.Style.Border.Default) {
                            if (chartseries.Style.Border.Default.BorderBrush) {
                                seriesObj.border.color = chartseries.Style.Border.Default.BorderBrush;
                            }

                            if (chartseries.Style.Border.Default.Thickness != 0) {
                                seriesObj.border.width = chartseries.Style.Border.Default.Thickness;
                            }

                            if (chartseries.Style.Border.Default.BorderStyle == "Dashed") {
                                seriesObj.border.dashArray = "2,4";
                            }
                            else if (chartseries.Style.Border.Default.BorderStyle == "Dotted") {
                                seriesObj.border.dashArray = "2,2";
                            }
                        }

                        if (chartseries.DataPointsStyle && chartseries.DataPointsStyle.length > 0) {
                            if (chartseries.Type == "scatter") {
                                var datapointStyle = chartseries.DataPointsStyle[0];

                                if (datapointStyle.ChartMarker && datapointStyle.ChartMarker.MarkerType) {
                                    seriesObj.marker.shape = datapointStyle.ChartMarker.MarkerType;
                                } else {
                                    seriesObj.marker.shape = "Circle";
                                }
                                if (datapointStyle.ChartMarker.Color) {
                                    seriesObj.marker.fill = datapointStyle.ChartMarker.Color;
                                }

                                if (datapointStyle.ChartMarker.BorderColor) {
                                    seriesObj.marker.border.color = datapointStyle.ChartMarker.BorderColor;
                                }

                                if (datapointStyle.ChartMarker.Borderwidth) {
                                    seriesObj.marker.border.width = datapointStyle.ChartMarker.Borderwidth == 0 ? 2 : datapointStyle.ChartMarker.Borderwidth;
                                }

                                if (datapointStyle.ChartMarker.Size) {
                                    seriesObj.marker.size.height = seriesObj.marker.size.width = parseInt((datapointStyle.ChartMarker.Size == 0 || datapointStyle.ChartMarker.Size < 6)) ? 6 : (datapointStyle.ChartMarker.Size);
                                } else {
                                    seriesObj.marker.size.height = seriesObj.marker.size.width = 10;
                                }
                                seriesObj.marker.visible = true;
                            } else {
                                var datapointStyle = chartseries.DataPointsStyle[0];
                                if (datapointStyle.ChartMarker) {
                                    if (datapointStyle.ChartMarker.MarkerType) {
                                        seriesObj.marker.shape = datapointStyle.ChartMarker.MarkerType;
                                    }

                                    if (datapointStyle.ChartMarker.Color) {
                                        seriesObj.marker.fill = datapointStyle.ChartMarker.Color;
                                    }

                                    if (datapointStyle.ChartMarker.BorderColor) {
                                        seriesObj.marker.border.color = datapointStyle.ChartMarker.BorderColor;
                                    }

                                    if (datapointStyle.ChartMarker.Borderwidth) {
                                        seriesObj.marker.border.width = datapointStyle.ChartMarker.Borderwidth == 0 ? 2 : datapointStyle.ChartMarker.Borderwidth;
                                    }

                                    if (datapointStyle.ChartMarker.Size) {
                                        seriesObj.marker.size.height = seriesObj.marker.size.width = parseInt((datapointStyle.ChartMarker.Size == 0 || datapointStyle.ChartMarker.Size < 6)) ? 6 : (datapointStyle.ChartMarker.Size);
                                    }
                                }

                                if (chartseries.Visibility && datapointStyle.ChartMarker && datapointStyle.ChartMarker.MarkerType) {
                                    seriesObj.marker.visible = true;
                                }
                                else {
                                    seriesObj.marker.visible = false;
                                }

                                if (chartseries.PointValues.length > 0 && chartseries.PointValues[0].ChartDataLabel) {
                                    seriesObj.marker.dataLabel.visible = false;
                                    var pointDatalbl = chartseries.PointValues[0].ChartDataLabel;
                                    seriesObj.marker.dataLabel.visible = pointDatalbl.Visible;
                                    if (pointDatalbl.Visible) {
                                        seriesObj.marker.dataLabel.connectorLine.type = "line";
                                        seriesObj.marker.dataLabel.connectorLine.width = 0.5;
                                        seriesObj.marker.dataLabel.textPosition = (pointDatalbl.Position && pointDatalbl.Position == "Default") ? "top" : pointDatalbl.Position;
                                        seriesObj.marker.dataLabel.font.color = !pointDatalbl.TextColor ? "Black" : pointDatalbl.TextColor;
                                        if(pointDatalbl.Font){
                                            seriesObj.marker.dataLabel.font.fontFamily = pointDatalbl.Font.FontFamily;
                                            seriesObj.marker.dataLabel.font.fontStyle = pointDatalbl.Font.FontStyle;
                                            seriesObj.marker.dataLabel.font.fontWeight = pointDatalbl.Font.FontWeight;
                                            seriesObj.marker.dataLabel.font.size = pointDatalbl.Font.FontSize + "px";
                                        }
                                        if (pointDatalbl.BorderStyle && pointDatalbl.BorderStyle != "None") {
                                            seriesObj.marker.dataLabel.border.color = pointDatalbl.BorderColor;
                                            seriesObj.marker.dataLabel.border.width = pointDatalbl.BorderWidth;
                                            seriesObj.marker.dataLabel.shape = "Rectangle";
                                        }
                                        if (pointDatalbl.BackGroundColor && pointDatalbl.BackGroundColor != "Transparent") {
                                            seriesObj.marker.dataLabel.fill = pointDatalbl.BackGroundColor;
                                            seriesObj.marker.dataLabel.shape = "Rectangle";
                                        }
                                    }
                                }
                            }
                        }
                        ejChartCtrl.model.series.push(seriesObj);
                    }

                    if (chartModel.ChartLegends && chartModel.ChartLegends.length > 0) {
                        var chartLegend = chartModel.ChartLegends[0];
                        ejChartCtrl.model.legend.visible = chartLegend.Visibilty;

                        if (chartLegend.Alignment) {
                            ejChartCtrl.model.legend.alignment = chartLegend.Alignment;
                        }

                        if (chartLegend.Position) {
                            ejChartCtrl.model.legend.position = chartLegend.Position;
                        }

                        if (chartLegend.Style) {
                            if (chartLegend.Style.Color) {
                                ejChartCtrl.model.legend.font.color = chartLegend.Style.Color;
                            }

                            if (chartLegend.Style.Font) {
                                if (chartLegend.Style.Font.FontFamily) {
                                    ejChartCtrl.model.legend.font.fontFamily = chartLegend.Style.Font.FontFamily;
                                }

                                if (chartLegend.Style.Font.FontStyle) {
                                    ejChartCtrl.model.legend.font.fontStyle = chartLegend.Style.Font.FontStyle;
                                }

                                if (chartLegend.Style.Font.FontWeight) {
                                    ejChartCtrl.model.legend.font.fontWeight = chartLegend.Style.Font.FontWeight;
                                }

                                if (chartLegend.Style.Font.FontSize != 0) {
                                    ejChartCtrl.model.legend.font.size = chartLegend.Style.Font.FontSize + 'px';
                                }
                            }

                            if (chartLegend.Style.Border && chartLegend.Style.Border.Default) {
                                if (chartLegend.Style.Border.Default.BorderBrush) {
                                    ejChartCtrl.model.legend.border.color = chartLegend.Style.Border.Default.BorderStyle == "Default" ? "transparent" : chartLegend.Style.Border.Default.BorderBrush;
                                }

                                if (chartLegend.Style.Border.Default.Thickness != 0) {
                                    ejChartCtrl.model.legend.border.width = chartLegend.Style.Border.Default.Thickness;
                                }
                            }
                        }
                    }
                    else {
                        ejChartCtrl.model.legend.visible = false;
                    }

                    if (ejChartCtrl.model.series[0].type != 'polar' && ejChartCtrl.model.series[0].type != 'radar') {
                        for (var chartAreaXaxisIndex = 0; chartAreaXaxisIndex < chartArea.ChartAreaXAxis.length; chartAreaXaxisIndex++) {
                            var chartXAxis = chartArea.ChartAreaXAxis[chartAreaXaxisIndex];
                            if (chartXAxis.Name.toString().toLowerCase() == "primary") {
                                this._applyAxis(chartXAxis, ejChartCtrl.model.primaryXAxis, 'XAxis', 'primary', ejChartCtrl.model.series[0].type, chartArea.Chart3D ? chartArea.Chart3D.Enabled:null);
                            }
                            //else if (chartXAxis.Name.toString().toLowerCase() == "secondary") {
                            //    if (!ej.isNullOrUndefined(ejChartCtrl.model.axes)) {
                            //        var xaxesObj = $.extend(true, {}, ejChartCtrl.model._axes[0]);
                            //        this._applyAxis(chartXAxis, xaxesObj, 'XAxis', 'secondary');
                            //        xaxesObj.opposedPosition = true;
                            //        xaxesObj.orientation = 'Horizontal';
                            //        ejChartCtrl.model.axes.push(xaxesObj);
                            //    }
                            //}
                        }

                        for (var chartAreaYaxisIndex = 0; chartAreaYaxisIndex < chartArea.ChartAreaYAxis.length; chartAreaYaxisIndex++) {
                            var chartYAxis = chartArea.ChartAreaYAxis[chartAreaYaxisIndex];
                            if (chartYAxis.Name.toString().toLowerCase() == "primary") {
                                this._applyAxis(chartYAxis, ejChartCtrl.model.primaryYAxis, 'YAxis', 'primary', ejChartCtrl.model.series[0].type, chartArea.Chart3D ? chartArea.Chart3D.Enabled:null);
                            }
                            //else if (chartYAxis.Name.toString().toLowerCase() == "secondary") {
                            //    if (!ej.isNullOrUndefined(ejChartCtrl.model.axes)) {
                            //        var yaxesObj = $.extend(true, {}, ejChartCtrl.model._axes[0]);
                            //        yaxesObj.opposedPosition = true;
                            //        yaxesObj.orientation = "vertical";
                            //        this._applyAxis(chartYAxis, yaxesObj, 'YAxis', 'secondary');
                            //        ejChartCtrl.model.axes.push(yaxesObj);
                            //    }
                            //}
                        }
                    }

                    if (chartArea.Chart3D && chartArea.Chart3D.Enabled) {
                        ejChartCtrl.model.enable3D = chartArea.Chart3D.Enabled;
                        ejChartCtrl.model.depth = chartArea.Chart3D.DepthRatio;
                        ejChartCtrl.model.wallSize = chartArea.Chart3D.WallThickness;
                        ejChartCtrl.model.tilt = 0;
                        ejChartCtrl.model.rotation = 20;
                        ejChartCtrl.model.perspectiveAngle = chartArea.Chart3D.Perspective == 0 ? 90 : chartArea.Chart3D.Perspective;
                        ejChartCtrl.model.crosshair.visible = true;
                    }

                    if (chartseries.PointValues.length <= 0) {
                        ejChartCtrl.model.legend.visible = false;
                        ejChartCtrl.model.primaryXAxis.visible = false;
                        ejChartCtrl.model.primaryYAxis.visible = false;
                        ejChartCtrl.model.series = [];
                        ejChartCtrl.model.chartArea.visible = false;
                        var chartsubtile = chartModel.ChartNoDataMessage;
                        if (ejChartCtrl.model.title.text == "") {
                            ejChartCtrl.model.title.text = " ";
                        }
                        if (chartsubtile.Caption && chartsubtile.Visible) {
                            ejChartCtrl.model.title.subTitle.textAlignment = "center";
                            ejChartCtrl.model.title.subTitle.text = chartsubtile.Caption;
                            if (chartsubtile.Style) {
                                if (chartsubtile.Style.Color) {
                                    ejChartCtrl.model.title.subTitle.font.color = chartsubtile.Style.Color;
                                }

                                if (chartsubtile.Style.Font) {
                                    if (chartsubtile.Style.Font.FontFamily) {
                                        ejChartCtrl.model.title.subTitle.font.fontFamily = chartsubtile.Style.Font.FontFamily;
                                    }

                                    if (chartsubtile.Style.Font.FontStyle) {
                                        ejChartCtrl.model.title.subTitle.font.fontStyle = chartsubtile.Style.Font.FontStyle;
                                    }

                                    if (chartsubtile.Style.Font.FontWeight) {
                                        ejChartCtrl.model.title.subTitle.font.fontWeight = chartsubtile.Style.Font.FontWeight;
                                    }

                                    if (chartsubtile.Style.Font.FontSize != 0) {
                                        ejChartCtrl.model.title.subTitle.font.size = chartsubtile.Style.Font.FontSize + 'px';
                                    }
                                }
                            }
                        }
                    }
                    ejChartCtrl.redraw();
                }
            }
        },

        _applyBackgroundGradientStyle: function (style, tag) {
            var type;
            switch (style.BackgroundGradientType) {
                case "None":
                    break;
                case "Default":
                    break;
                case "LeftRight":
                    type = "to right";
                    tag.css('background', 'linear-gradient(' + type + ',' + style.BackgroundColor + ', ' + style.BackgroundGradientEndcolor + ')');
                    break;
                case "TopBottom":
                    tag.css('background', 'linear-gradient(' + style.BackgroundColor + ', ' + style.BackgroundGradientEndcolor + ')');
                    break;
                case "Center":
                    tag.css('background', 'radial-gradient(' + style.BackgroundColor + ', ' + style.BackgroundGradientEndcolor + ')');
                    break;
                case "DiagonalLeft":
                    type = "to bottom right";
                    tag.css('background', 'linear-gradient(' + type + ',' + style.BackgroundColor + ', ' + style.BackgroundGradientEndcolor + ')');
                    break;
                case "DiagonalRight":
                    type = "to bottom left";
                    tag.css('background', 'linear-gradient(' + type + ',' + style.BackgroundColor + ', ' + style.BackgroundGradientEndcolor + ')');
                    break;
                case "HorizontalCenter":
                    tag.css('background', 'linear-gradient(' + style.BackgroundColor + ',' + style.BackgroundGradientEndcolor + ', ' + style.BackgroundColor + ')');
                    break;
                case "VerticalCenter":
                    tag.css('background', 'linear-gradient(90deg,' + style.BackgroundColor + ',' + style.BackgroundGradientEndcolor + ', ' + style.BackgroundColor + ')');
                    break;
            }
        },

        _applyAxis: function (axisModelProp, axisName, axisSpecific, axistype, seriestype, threeDEnabled) {
            if (seriestype != 'pie' && !threeDEnabled) {
                axisName.labelPlacement = 'onticks';
            }

            if (axisModelProp.ChartAxisTitle) {

                if (axisModelProp.ChartAxisTitle.Caption) {
                    axisName.title.text = axisModelProp.ChartAxisTitle.Caption;
                }

                if (axisModelProp.ChartAxisTitle.TextColor) {
                    axisName.title.font.color = axisModelProp.ChartAxisTitle.TextColor;
                }

            if (axisModelProp.ChartAxisTitle.Font) {
                if (axisModelProp.ChartAxisTitle.Font.FontSize) {
                    axisName.title.font.size = axisModelProp.ChartAxisTitle.Font.FontSize;
                }

                if (axisModelProp.ChartAxisTitle.Font.FontFamily) {
                    axisName.title.font.fontFamily = axisModelProp.ChartAxisTitle.Font.FontFamily;
                }

                    if (axisModelProp.ChartAxisTitle.Font.FontStyle) {
                        axisName.title.font.fontStyle = axisModelProp.ChartAxisTitle.Font.FontStyle;
                    }
                }
            }

            if ((axisModelProp.Maximum && axisModelProp.Maximum != "NaN" && axisModelProp.Maximum != 0) && (axisModelProp.Minimum && axisModelProp.Minimum != "NaN" && axisModelProp.Minimum != 0)) {
                axisName.range.max = axisModelProp.Maximum;
                axisName.range.min = axisModelProp.Minimum;
            }
            if (axisModelProp.Interval != "NaN") {
                //axisName.range.interval = axisModelProp.Interval;
            }

            if (axisModelProp.IntervalType) {
                //axisName.intervalType = axisModelProp.IntervalType;
            }

            axisName.font.color = axisModelProp.LabelColor ? axisModelProp.LabelColor : "Black";
            if (axisModelProp.LabelFont) {
                axisName.font.fontFamily = axisModelProp.LabelFont.FontFamily;
                axisName.font.size = axisModelProp.LabelFont.FontSize;
                axisName.font.fontStyle = axisModelProp.LabelFont.FontStyle == "Default" ? "Normal" : axisModelProp.LabelFont.FontStyle;
                axisName.font.fontWeight = axisModelProp.LabelFont.FontWeight;
            }

            if (axisModelProp.ChartMajorGridLines) {
                this._applyGridLines(axisModelProp.ChartMajorGridLines, axisName.majorGridLines);
            }

            if (axisModelProp.ChartMinorGridLines) {
                this._applyGridLines(axisModelProp.ChartMinorGridLines, axisName.minorGridLines);
            }

            if (axisModelProp.ChartMajorTickMarks) {
                this._applyTickMarks(axisModelProp.ChartMajorTickMarks, axisName.majorTickLines);
            }

            if (axisModelProp.ChartMinorTickMarks) {
                this._applyTickMarks(axisModelProp.ChartMinorTickMarks, axisName.minorTickLines);
            }

            if (axisName.minorTicksPerInterval == null)
            {
                axisName.minorTicksPerInterval = 1;
            }

            if (axisModelProp.LabelsAutoFitEnabled) {
                if (axisModelProp.PrExventWordWrap) {
                    if (axisModelProp.PrExventLabelOffset) {
                        if (axisModelProp.AllowLabelRotation) {
                            axisName.labelIntersectAction = axisModelProp.AllowLabelRotation;
                        }
                        else {
                            axisName.labelIntersectAction = 'multiplerows';
                        }
                    }
                }
                else {
                    axisName.labelIntersectAction = 'none';
                }
            }
            else {
                axisName.labelRotation = axisModelProp.Angle;
            }

            if (axisModelProp.LineColor) {
                axisName.axisLine.color = axisModelProp.LineColor;
            }
            if (axisModelProp.LineWidth) {
                axisName.axisLine.width = axisModelProp.LineWidth;
            }

            if (axisModelProp.HideEndLabels) {
                axisName.edgeLabelPlacement = 'hide';
            }

            if (axisModelProp.VisibleStatus == "True" || (axistype == 'primary' && axisModelProp.VisibleStatus == 'Auto')) {
                axisName.visible = true;
                axisName.axisLine.visible = true;
            } else {
                axisName.visible = false;
                axisName.axisLine.visible = false;
            }
            axisName.rangePadding = 'none';
        },

        _applyGridLines: function (applySrc, applyTarget) {
            applyTarget.visible = applySrc.Enabled;
            if (applySrc.Enabled) {
                if (applySrc.Style && applySrc.Style.Default) {
                    if (applySrc.Style.Default.BorderBrush) {
                        applyTarget.color = applySrc.Style.Default.BorderBrush;
                    }

                    applyTarget.width = applySrc.Style.Default.BorderStyle == "None" ? applyTarget.width : applySrc.Style.Default.Thickness;

                    if (applySrc.Style.Default.BorderStyle == "Dashed") {
                        applyTarget.dashArray = "2,4";
                    }
                    else if (applySrc.Style.Default.BorderStyle == "Dotted") {
                        applyTarget.dashArray = "2,2";
                    }
                }
            }
        },

        _applyTickMarks: function (applySrc, applyTarget) {
            applyTarget.visible = applySrc.Enabled;

            if (applySrc.Style && applySrc.Style.Default) {
                if (applySrc.Style.Default.BorderBrush) {
                    applyTarget.color = applySrc.Style.Default.BorderBrush;
                }

                if (applySrc.Style.Default.Thickness) {
                    applyTarget.width = applySrc.Style.Default.Thickness;
                }
            }

            if (applySrc.Length) {
                applyTarget.size = applySrc.Length;
            }
        },
        //-------------------- Render the Chart ReportItem [end] -------------------------//

        _renderImageControl: function (imgModel, parnetObj, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + imgModel.Name, parentId, printMode, printPageId);
            var imageDiv = ej.buildTag("div", "", {}, { id: ctrlId, title: imgModel.ToolTip });
            var _imageDivCss = {};
            parnetObj.append(imageDiv);
            _imageDivCss["position"] = 'absolute';
            _imageDivCss["top"] = imgModel.Top;
            _imageDivCss["left"] = imgModel.Left;
            _imageDivCss["width"] = imgModel.Width;
            _imageDivCss["height"] = imgModel.Height;

            _imageDivCss["padding-left"] = imgModel.PaddingLeft;
            _imageDivCss["padding-right"] = imgModel.PaddingRight;
            _imageDivCss["padding-top"] = imgModel.PaddingTop;
            _imageDivCss["padding-bottom"] = imgModel.PaddingBottom;

            if (imgModel.IsTablixChild) {
                _imageDivCss["position"] = 'relative';
            }

            if (imgModel.ActionInfo) {
                imageDiv.data('actionObj', imgModel.ActionInfo);
                imageDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
            }

            if (imgModel.Border && !imgModel.IsTablixChild) {
                this._applyBorderStyle(imgModel.Border, imageDiv);
            }

            var resourceName = printMode ? 'PrintImage' : 'GetResource';
            var imgwidth = imgModel.IsTablixChild ? imgModel.Width : '100%';
            var imgheight = imgModel.IsTablixChild ? imgModel.Height : '100%';
            var imagetag = ej.buildTag("img", "", { 'width': imgwidth, 'height': imgheight }, { 'src': this.model.reportServiceUrl + '/' + resourceName + '/?key=' + imgModel.ImageUrl + '&resourcetype=sfimg&isPrint=' + printMode });
            imageDiv.append(imagetag);
            imageDiv.css(_imageDivCss);
        },

        _renderRectangleControl: function (rectModel, isTablixCell, parentObj, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + rectModel.Name, parentId, printMode, printPageId);
            var rectDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, title: rectModel.ToolTip });
            var _rectDivCss = {};            
            parentObj.append(rectDiv);
            _rectDivCss["position"] = isTablixCell ? 'relative' : 'absolute';
            _rectDivCss["top"] = rectModel.Top;
            _rectDivCss["left"] = rectModel.Left;
            _rectDivCss["width"] = rectModel.Width;
            _rectDivCss["height"] = rectModel.Height;
            _rectDivCss["background-color"] = rectModel.BackgroundColor;

            var resourceName = printMode ? 'PrintImage' : 'GetResource';
            if (rectModel.BackgroundImageSrc) {
                _rectDivCss["background-image"] = "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + rectModel.BackgroundImageSrc + '&resourcetype=sfimg&isPrint=' + printMode + ")";
            }

            if (rectModel.Border && !isTablixCell) {
                this._applyBorderStyle(rectModel.Border, rectDiv);
            }

            if (rectModel.IsTablixChild && rectModel.ReportModels && rectModel.ReportModels.length > 0) {
                for (var i = 0; i < rectModel.ReportModels.length; i++) {
                    var innerItem = rectModel.ReportModels[i];
                    if (innerItem) {
                        if (innerItem.ItemType == "TextBoxModel") {
                            var textboxControl = this._renderTextBoxControl(innerItem, false, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            rectDiv.append(textboxControl);
                        }
                        else if (innerItem.ItemType == "LineModel") {
                            var lineControl = this._renderLineControl(innerItem, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            rectDiv.append(lineControl);
                        }
                        else if (innerItem.ItemType == "ImageModel") {
                            this._renderImageControl(innerItem, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "TablixModel") {
                            this._renderTablixControl(innerItem, false, {}, rectDiv, ctrlId + '_' + innerItem.Name, printPageId);
                        }
                        else if (innerItem.ItemType == "GaugeModel") {
                            this._renderGaugeControl(innerItem, false, {}, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "RectangleModel") {
                            this._renderRectangleControl(innerItem, false, rectDiv, ctrlId + + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "ChartModel") {
                            this._renderChartControl(innerItem, false, {}, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "MapModel") {
                            this._renderMapControl(innerItem, rectDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        }
                        else if (innerItem.ItemType == "SubReportModel") {
                            this._renderSubReportControl(innerItem, false, rectDiv, ctrlId + + '_' + innerItem.Name, printMode, printPageId);
                        }
                    }
                }
            }
            rectDiv.css(_rectDivCss);
        },

        _renderMapControl: function (mapModel, parentObj, parentId, printMode, printPageId) {
            var mapContainer = mapModel.MapProperties;
            var ctrlId = this._controlKeyGenerator(this._id + '_' + mapModel.Name, parentId, printMode, printPageId);
            var mapDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            var titleContainerHeight = 0;
            parentObj.append(mapDiv);

            mapDiv.css("position", "absolute");
            mapDiv.css("top", mapModel.Top);
            mapDiv.css("left", mapModel.Left);
            mapDiv.css("height", mapModel.Height);
            mapDiv.css("width", mapModel.Width);
            if (mapContainer.Background && mapContainer.Background.BackgroundColor) {
                mapDiv.css("background-color", mapContainer.Background.BackgroundColor);
            }

            if (mapModel.IsTablixChild) {
                mapDiv.css("position", "relative");
            }

            if (mapContainer.ActionInfo) {
                mapDiv.data('actionObj', mapContainer.ActionInfo);
                mapDiv.addClass('drillAction_' + this._id + ' e-reportviewer-drillhover');
            }

            if (mapContainer.Border) {
                this._applyBorderStyle(mapContainer.Border, mapDiv);
            }
            var title = mapContainer.MapTitles;
            if (title && title.length > 0) {

                for (var i = 0; i < title.length; i++) {
                    var titleDiv = ej.buildTag("div", "", {}, { 'id': ctrlId + '_' + title[i].Name });
                    titleDiv.css("position", "relative");
                    titleDiv.css("left", "1px");
                    titleDiv.css("height", title[i].Style.Font.FontSize - (3 * title[i].Border.Thickness));
                    titleDiv.css("width", mapModel.Width - (3 * title[i].Border.Thickness));
                    titleDiv.css("text-align", "center");
                    titleDiv.css("background-color", title[i].BackGround.BackgroundColor);
                    titleDiv.css("border-color", title[i].Border.BorderBrush);
                    titleDiv.css("border-width", title[i].Border.Thickness);
                    titleDiv.css("border-style", title[i].Border.BorderStyle);

                    var span = ej.buildTag("span", "", {}, { 'id': ctrlId + '_' + title[i].Name + '_' + i });
                    span.css("font-family", title[i].Style.Font.FontFamily);
                    span.css("font-size", (title[i].Style.Font.FontSize / 1.33));
                    span.css("font-style", title[i].Style.Font.FontStyle);
                    span.css("font-weight", title[i].Style.Font.FontWeight);
                    span.append(title[i].Text);
                    titleDiv.append(span);
                    mapDiv.append(titleDiv);
                }
            }

            if (mapContainer.MapViewport) {
                var viewPort = mapContainer.MapViewport;
                var shapDiv = ej.buildTag("div", "", {}, { 'id': ctrlId + '_' + mapModel.Name });
                mapDiv.append(shapDiv);
                shapDiv.css("left", "1px");
                shapDiv.css("top", "1px");

                if (title && title.length > 0) {
                    for (var titleIndex = 0; titleIndex < title.length; titleIndex++) {
                        titleContainerHeight = titleContainerHeight + title[titleIndex].Style.Font.FontSize;
                    }
                    shapDiv.css("height", ((mapModel.Height - titleContainerHeight) - (3 * viewPort.Border.Thickness)));
                    shapDiv.css("top", "10px");
                }
                else {
                    shapDiv.css("height", (mapModel.Height - (3 * viewPort.Border.Thickness)));
                }

                shapDiv.css("width", mapModel.Width - (3 * viewPort.Border.Thickness));
                shapDiv.css("background-color", viewPort.BackGround.BackgroundColor);
                shapDiv.css("border-color", viewPort.Border.BorderBrush);
                shapDiv.css("border-width", viewPort.Border.Thickness);
                shapDiv.css("border-style", viewPort.Border.BorderStyle);

                shapDiv.ejMap({ layers: [{ layerType: "geometry" }] });
                var map = shapDiv.data("ejMap");
                map.model.zoomSettings.minValue = 1;
                map.model.zoomSettings.level = 1;
                map.model.zoomSettings.enableZoom = false;

                map.model.centerPosition = [viewPort.MapView.CenterX, viewPort.MapView.CenterY];

                if (mapContainer.MapPolygonLayers && mapContainer.MapPolygonLayers.length > 0) {
                    for (var polygonIndex = 0; polygonIndex < mapContainer.MapPolygonLayers.length; polygonIndex++) {
                        var polygon = mapContainer.MapPolygonLayers[polygonIndex];
                        var layer = $.extend(true, {}, map.model.layers[0]);
                        layer.layerType = "geometry";
                        layer.enableSelection = false;
                        layer.enableMouseHover = false;
                        layer.shapeData = polygon.MapFields;
                        if (polygon.MapBindingFieldPairs) {
                            layer.shapePropertyPath = polygon.MapBindingFieldPairs.FieldName;
                        }
                        layer.shapeDataPath = 'Value';
                        layer.dataSource = mapModel.MapModelData;

                        if (mapModel.MapModelData && mapModel.MapModelData.length > 0 && mapModel.MapModelData[0].ShapeActionInfo) {
                            map.ejreportid = this._id;
                            map.model.shapeSelected = this._drillThroughClick;
                            layer.enableSelection = true;
                            layer.enableMouseHover = true;
                            map.model.mouseover = function (event) {
                                if (event.originalEvent.data && event.originalEvent.data.ShapeActionInfo) {
                                    $(event.originalEvent.shape).css('cursor', 'pointer');
                                }
                            };
                        }

                        if (polygon.ShapeSettings) {
                            var shapesettings = $.extend(true, {}, layer.shapeSettings);
                            shapesettings.colorValuePath = polygon.ShapeSettings.ShapeColorValuePath;
                            shapesettings.valuePath = polygon.ShapeSettings.ShapeValuePath;
                            if (polygon.ShapeColorMappings && polygon.ShapeColorMappings.length > 1) {
                                shapesettings.colorMappings = { equalColorMapping: polygon.ShapeColorMappings };
                            }
                            else {
                                shapesettings.autoFill = true;
                                shapesettings.colorPalette = "palette1";
                            }
                            layer.shapeSettings = shapesettings;
                        }
                        if (polygon.MapCenterPointRules && polygon.MapCenterPointRules.length > 0 && polygon.MapCenterPointRules.MapSizeRule) {
                            var bubblesettings = $.extend(true, {}, layer.bubbleSettings);
                            bubblesettings.maxValue = polygon.BubbleSettings.MaxSize;
                            bubblesettings.minValue = polygon.BubbleSettings.MinSize;
                            bubblesettings.colorValuePath = polygon.BubbleSettings.ColorValuePath == null ? "Value" : polygon.BubbleSettings.ColorValuePath;
                            bubblesettings.valuePath = polygon.BubbleSettings.ValuePath;
                            if (polygon.BubbleColorMappings && polygon.BubbleColorMappings.length > 1) {
                                bubblesettings.colorMappings = { equalColorMapping: polygon.BubbleColorMappings };
                            }
                            layer.bubbleSettings = bubblesettings;
                        }

                        if (polygon.MapPolygonTemplate) {
                            var labelsettings = $.extend(true, {}, layer.labelSettings);
                            labelsettings.showLabels = true;
                            if (polygon.ShapeSettings) {
                                labelsettings.labelPath = polygon.ShapeSettings.LabelPath;
                            }
                            labelsettings.enableSmartLabel = false;
                            layer.labelSettings = labelsettings;
                        }

                        if (mapContainer.MapLegends && mapContainer.MapLegends.length > 0) {
                            var legendsettings = $.extend(true, {}, layer.legendSettings);
                            legendsettings.showLegend = true;
                            layer.legendSettings = legendsettings;
                        }
                        map.model.layers[polygonIndex] = layer;
                    }
                }
                map.refresh();
                shapDiv.css("position", "relative");
            }
        },

        //-------------------- Render the Gauge ReportItem [start] -------------------------//
        _renderGaugeControl: function (gaugeModel, isTablixCell, cellSize, parentObj, parentId, printMode, printPageId) {
            var gaugePanel = gaugeModel.GaugePanel;
            var ctrlId = this._controlKeyGenerator(this._id + '_' + gaugeModel.Name, parentId, printMode, printPageId);
            var gaugeDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            parentObj.append(gaugeDiv);
            gaugeDiv.css("position", "absolute");
            var containerHeight = gaugeModel.Height;
            var containerWidth = gaugeModel.Width;

            if (gaugeModel.Top) {
                gaugeDiv.css("top", gaugeModel.Top);
            }

            if (gaugeModel.Left) {
                gaugeDiv.css("left", gaugeModel.Left);
            }
            var panelOrientation = 'Horizontal';
            var itemCount = 0;

            if (gaugePanel.RadialGauges && gaugePanel.RadialGauges.length > 0) {
                itemCount = itemCount + gaugePanel.RadialGauges.length;
            }

            if (gaugePanel.LinearGauges && gaugePanel.LinearGauges.length > 0) {
                itemCount = itemCount + gaugePanel.LinearGauges.length;
            }

            if (gaugePanel.Indicator && gaugePanel.Indicator.length > 0) {
                itemCount = itemCount + gaugePanel.Indicator.length;
            }

            var resourceName = printMode ? 'PrintImage' : 'GetResource';
            if (gaugePanel.GaugeFrame && gaugePanel.GaugeFrame.FrameImage) {
                gaugeDiv.css("background-image", "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + gaugePanel.GaugeFrame.FrameImage + '&resourcetype=sfimg&isPrint=' + printMode + ")");
            }

            if (gaugePanel.Border) {
                this._applyBorderStyle(gaugePanel.Border, gaugeDiv);
            }

            if (isTablixCell) {
                _parentID = parentId;
                containerHeight = cellSize.height;
                var borderwidthadjust = parseFloat(gaugeDiv.css("border-left-width"), 10) + parseFloat(gaugeDiv.css("border-right-width"), 10);
                var borderheightadjust = parseFloat(gaugeDiv.css("border-top-width"), 10) + parseFloat(gaugeDiv.css("border-bottom-width"), 10);
                containerWidth = (containerWidth - (borderwidthadjust));
                containerHeight = (containerHeight - (borderheightadjust));
            }

            if (itemCount != 0) {
                if (containerHeight > containerWidth) {
                    containerHeight = containerHeight / itemCount;
                    panelOrientation = 'Vertical';
                } else {
                    containerWidth = containerWidth / itemCount;
                }
            }

            if (gaugePanel.GaugeLabel) {
                this._renderGaugeLabels(gaugePanel.GaugeLabel, gaugeDiv, gaugeModel.Name, parentId, printMode, printPageId);
            }

            if (gaugePanel.RadialGauges && gaugePanel.RadialGauges.length > 0) {
                this._renderRadialGauge(gaugePanel.RadialGauges, gaugeDiv, gaugeModel.Name, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId);
            }

            if (gaugePanel.LinearGauges && gaugePanel.LinearGauges.length > 0) {
                this._renderLinearGauge(gaugePanel.LinearGauges, gaugeDiv, gaugeModel.Name, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId);
            }

            if (gaugePanel.Indicator && gaugePanel.Indicator.length > 0) {
                this._renderIndicator(gaugePanel.Indicator, gaugeDiv, gaugeModel.Name, containerWidth, containerHeight, parentId, printMode, printPageId);
            }

            if (isTablixCell) {
                gaugeDiv.css({ 'position': 'relative', 'width': containerWidth, 'height': containerHeight, 'background-color': gaugePanel.BackgroundColor });
            } else {
                gaugeDiv.css({ 'width': gaugeModel.Width, 'height': gaugeModel.Height, 'background-color': gaugePanel.BackgroundColor });
            }
        },

        _renderIndicator: function (indicators, gaugeDiv, gaugeName, containerWidth, containerHeight, parentId, printMode, printPageId) {
            var svgNs = "http://www.w3.org/2000/svg";
            for (var guageIndex = 0; guageIndex < indicators.length; guageIndex++) {
                var svgelement = document.createElementNS(svgNs, "svg");
                var groupElement = document.createElementNS(svgNs, "g");
                var scaleXy = (containerWidth < containerHeight) ? containerWidth : containerHeight;
                var indicator = indicators[guageIndex];
                var indicatorDiv = ej.buildTag("div", "", { 'width': containerWidth, 'height': containerHeight, 'top': '0px', 'left': '0px' }, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + indicator.Name, parentId, printMode, printPageId), title: indicator.ToolTip });
                var indicatorinnerDiv = ej.buildTag("div", "", { 'width': '20px', 'height': '20px', 'position': 'relative', 'display': 'block' });
                //svgelement.setAttributeNS(null, 'width', 'auto');
                //svgelement.setAttributeNS(null, 'height', 'auto');
                svgelement.setAttributeNS(null, 'style', 'display: block;margin:auto');
                for (var state = 0; state < indicator.IndicatorState.length; state++) {
                    var style = indicator.IndicatorState[state].IndicatorStyle;
                    var startValue = indicator.IndicatorState[state].StartValue.Value;
                    var endValue = indicator.IndicatorState[state].EndValue.Value;

                    var temStart=startValue;
                    var temEnd=endValue;
                    if(startValue >= endValue){
                        startValue = temEnd;
                        endValue = temStart;
                    }

                    if ((indicator.IndicatorData.Value >= startValue && indicator.IndicatorData.Value <= endValue)) {
                        var fillColor = indicator.IndicatorState[state].FillColor;
                        var svgAttribute = 'display: block;';
                        var innerDivWidth = 17;
                        var innerDivHeight = 17;
                        switch (style) {
                            case "ArrowDown":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.60501098632813,16.3990020751953L14.06298828125,10.0289993286133 10.3890075683594,10.0289993286133 10.3890075683594,2.32599639892578 4.61099243164063,2.32599639892578 4.61099243164063,10.0289993286133 1.14801025390625,10.0289993286133 7.60501098632813,16.3990020751953z" });
                                innerDivWidth = 16;
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowDownIncline":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.5499877929688,13.8919982910156L13.6119995117188,4.822998046875 11.0130004882813,7.42100524902344 5.5670166015625,1.9739990234375 1.48101806640625,6.05900573730469 6.92901611328125,11.5059967041016 4.47900390625,13.9550018310547 13.5499877929688,13.8919982910156z" });
                                innerDivWidth = innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowSide":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.1960144042969,8.43499755859375L8.8280029296875,1.97899627685547 8.8280029296875,5.65299987792969 1.125,5.65299987792969 1.125,11.4300003051758 8.8280029296875,11.4300003051758 8.8280029296875,14.8919982910156 15.1960144042969,8.43499755859375z" });
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowUp":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.27099609375,1.47299957275391L0.81500244140625,7.84200286865234 4.48800659179688,7.84200286865234 4.48800659179688,15.5439987182617 10.2659912109375,15.5439987182617 10.2659912109375,7.84200286865234 13.7279968261719,7.84200286865234 7.27099609375,1.47299957275391z" });
                                innerDivWidth = 16;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "ArrowUpIncline":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.0830078125,2.04299926757813L4.0159912109375,1.98199462890625 6.61300659179688,4.58000183105469 1.16598510742188,10.0269927978516 5.25100708007813,14.1119995117188 10.697998046875,8.66499328613281 13.14599609375,11.1130065917969 13.0830078125,2.04299926757813z" });
                                innerDivWidth = 15;
                                innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-left:.5px;';
                                break;
                            case "BoxesAllFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                svgAttribute = svgAttribute + 'margin-top:2.5px;';
                                break;
                            case "BoxesNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "BoxesOneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "BoxesThreeFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "BoxesTwoFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M2.031005859375,7.5L8.031005859375,7.5 8.031005859375,1.5 2.031005859375,1.5 2.031005859375,7.5z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.96798706054688,7.5L14.9679870605469,7.5 14.9679870605469,1.5 8.96798706054688,1.5 8.96798706054688,7.5z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M2.031005859375,14.43701171875L8.031005859375,14.43701171875 8.031005859375,8.43798828125 2.031005859375,8.43798828125 2.031005859375,14.43701171875z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M8.96798706054688,14.43701171875L14.9679870605469,14.43701171875 14.9679870605469,8.43798828125 8.96798706054688,8.43798828125 8.96798706054688,14.43701171875z" });
                                innerDivHeight = 16;
                                break;
                            case "Circle":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                innerDivWidth = innerDivHeight = 17;
                                svgAttribute = svgAttribute + 'margin-left:0.5px;';
                                break;
                            case "Flag":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M1.53518676757813,1.97268676757813L1.71956861019135,1.87765264511108 2.22404956817627,1.65059661865234 3.90132141113281,1.14869689941406 5.98302745819092,1.16354370117188 6.99305105209351,1.58242082595825 7.88519287109375,2.39169311523438 9.55474090576172,4.09454298019409 11.1795692443848,5.14020156860352 12.5349636077881,5.63535928726196 13.3962097167969,5.68670654296875 13.5080919265747,5.78957939147949 13.3250951766968,6.16205215454102 12.1980743408203,7.28895568847656 10.2623682022095,8.2137336730957 9.06851482391357,8.33354949951172 7.76519775390625,8.08270263671875 5.64992141723633,7.59151554107666 4.50045776367188,7.73744964599609 4.02486801147461,8.11925792694092 3.93121337890625,8.335693359375 1.53518676757813,1.97268676757813z" });
                                innerDivWidth = 16;
                                innerDivHeight = 15;
                                break;
                            case "QuartersAllFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-left:0.5px;';
                                break;
                            case "QuartersNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-left:0.5px;';
                                break;
                            case "QuartersOneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M1.2958984375,8.71868896484375L1.864914894104,11.5364475250244 3.41652679443359,13.8374519348145 5.71757793426514,15.3888282775879 8.534912109375,15.9577026367188 11.353235244751,15.3888282775879 13.6545295715332,13.8374519348145 15.2060127258301,11.5364475250244 15.77490234375,8.71868896484375 15.2060127258301,5.90094757080078 13.6545295715332,3.59995269775391 11.353235244751,2.04858016967773 8.534912109375,1.47970581054688 5.71757793426514,2.04858016967773 3.41652679443359,3.59995269775391 1.864914894104,5.90094757080078 1.2958984375,8.71868896484375" });
                                createElement(groupElement, "path", { fill: fillColor, d: "M8.57321166992188,1.86428833007813L11.3244533538818,2.38370847702026 13.4967041015625,3.80480575561523 14.9227085113525,5.96914529800415 15.4352111816406,8.71829223632813 15.5302124023438,8.7222900390625 8.57321166992188,8.7222900390625 8.57321166992188,1.86428833007813z" });
                                svgAttribute = svgAttribute + 'margin-top:0.5px;';
                                break;
                            case "QuartersThreeFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M16.033203125,9.3330078125L15.4643287658691,12.150749206543 13.9129524230957,14.4517440795898 11.6119480133057,16.003116607666 8.794189453125,16.5719909667969 5.97586584091187,16.003116607666 3.6745719909668,14.4517440795898 2.12308931350708,12.150749206543 1.55419921875,9.3330078125 2.12308931350708,6.51524877548218 3.6745719909668,4.2142448425293 5.97586584091187,2.66286897659302 8.794189453125,2.093994140625 11.6119480133057,2.66286897659302 13.9129524230957,4.2142448425293 15.4643287658691,6.51524877548218 16.033203125,9.3330078125" });
                                createElement(groupElement, "path", { fill: "white", d: "M8.75729370117188,2.4794921875L6.00546979904175,2.99834585189819 3.83292007446289,4.41925430297852 2.40680551528931,6.58378267288208 1.894287109375,9.33349609375 1.79928588867188,9.33648681640625 8.75729370117188,9.33648681640625 8.75729370117188,2.4794921875z" });
                                svgAttribute = svgAttribute + 'margin-top:0.5px;';
                                break;
                            case "QuartersTwoFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.1328125,8.3330078125L14.563796043396,11.150749206543 13.0121841430664,13.4517440795898 10.7111330032349,15.003116607666 7.893798828125,15.5719909667969 5.07547521591187,15.003116607666 2.7741813659668,13.4517440795898 1.22269868850708,11.150749206543 0.65380859375,8.3330078125 1.22269868850708,5.51524877548218 2.7741813659668,3.2142448425293 5.07547521591187,1.66286897659302 7.893798828125,1.093994140625 10.7111330032349,1.66286897659302 13.0121841430664,3.2142448425293 14.563796043396,5.51524877548218 15.1328125,8.3330078125" });
                                createElement(groupElement, "path", { fill: "white", d: "M7.81930541992188,1.45999145507813L5.0907883644104,1.99271726608276 2.95392227172852,3.42375564575195 1.56049299240112,5.59241724014282 1.06228637695313,8.3380126953125 1.57205629348755,11.0836029052734 2.99092483520508,13.252254486084 5.1532301902771,14.6832828521729 7.893310546875,15.2160034179688 7.81930541992188,1.45999145507813z" });
                                svgAttribute = svgAttribute + 'margin-top:0.5px;margin-left:0.5px;';
                                break;
                            case "SignalMeterFourFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterThreeFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterOneFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "SignalMeterTwoFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M3.44400024414063,17.3070068359375L1.53201293945313,17.3070068359375 1.53201293945313,11.5700073242188 3.44400024414063,11.5700073242188 3.44400024414063,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M7.14999389648438,17.3070068359375L5.23699951171875,17.3070068359375 5.23699951171875,9.65701293945313 7.14999389648438,9.65701293945313 7.14999389648438,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M10.8550109863281,17.3070068359375L8.9429931640625,17.3070068359375 8.9429931640625,5.83200073242188 10.8550109863281,5.83200073242188 10.8550109863281,17.3070068359375z" });
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M14.56201171875,17.3070068359375L12.6489868164063,17.3070068359375 12.6489868164063,2.00698852539063 14.56201171875,2.00698852539063 14.56201171875,17.3070068359375z" });
                                innerDivHeight = 19;
                                svgAttribute = svgAttribute + 'margin-top:-10px;;margin-left:0.5px;';
                                break;
                            case "StarQuartersAllFilled":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M6.41500854492188,6.37100219726563L1.56698608398438,7.07400512695313 5.07501220703125,10.4939880371094 4.24700927734375,15.3210144042969 8.58200073242188,13.0409851074219 12.9169921875,15.3210144042969 12.0899963378906,10.4939880371094 15.5979919433594,7.07400512695313 10.75,6.37100219726563 8.58200073242188,1.97698974609375 6.41500854492188,6.37100219726563z" });
                                break;
                            case "StarQuartersNoneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M6.41500854492188,6.37100219726563L1.56698608398438,7.07400512695313 5.07501220703125,10.4939880371094 4.24700927734375,15.3210144042969 8.58200073242188,13.0409851074219 12.9169921875,15.3210144042969 12.0899963378906,10.4939880371094 15.5979919433594,7.07400512695313 10.75,6.37100219726563 8.58200073242188,1.97698974609375 6.41500854492188,6.37100219726563z" });
                                break;
                            case "StarQuartersOneFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.64109802246094,1.97750854492188L10.80810546875,6.37149047851563 15.6560974121094,7.07449340820313 12.1490936279297,10.4945068359375 12.9761047363281,15.3204956054688 8.64109802246094,13.0404968261719 4.30610656738281,15.3204956054688 5.13409423828125,10.4945068359375 1.6260986328125,7.07449340820313 6.47309875488281,6.37149047851563 8.64109802246094,1.97750854492188z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M6.22300720214844,6.37100219726563L1.50100708007813,7.07400512695313 5.07099914550781,10.4939880371094 4.06300354003906,15.531005859375 7.08399963378906,13.864013671875 7.072998046875,4.968994140625 6.22300720214844,6.37100219726563z" });
                                break;
                            case "StarQuartersThreeFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.60209655761719,1.97750854492188L10.7691040039063,6.37149047851563 15.6170959472656,7.07449340820313 12.110107421875,10.4945068359375 12.9371032714844,15.3204956054688 8.60209655761719,13.0404968261719 4.26710510253906,15.3204956054688 5.0950927734375,10.4945068359375 1.58709716796875,7.07449340820313 6.43409729003906,6.37149047851563 8.60209655761719,1.97750854492188z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M6.40199279785156,6.37100219726563L1.58700561523438,7.07400512695313 5.11000061035156,10.4939880371094 4.28999328613281,15.3210144042969 8.49000549316406,13.0989990234375 10.6970062255859,14.27099609375 10.6970062255859,6.2239990234375 8.60200500488281,1.97698974609375 6.40199279785156,6.37100219726563z" });
                                break;
                            case "StarQuartersTwoFilled":
                                createElement(groupElement, "path", { fill: "none", stroke: "lightgray", d: "M8.17819213867188,1.97750854492188L10.34521484375,6.37149047851563 15.1932067871094,7.07449340820313 11.6861877441406,10.4945068359375 12.5132141113281,15.3204956054688 8.17819213867188,13.0404968261719 3.84320068359375,15.3204956054688 4.67120361328125,10.4945068359375 1.1632080078125,7.07449340820313 6.01019287109375,6.37149047851563 8.17819213867188,1.97750854492188z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M6.010009765625,6.37100219726563L1.16299438476563,7.07400512695313 4.67098999023438,10.4939880371094 3.84298706054688,15.3210144042969 7.83499145507813,13.2219848632813 7.90399169921875,2.53201293945313 6.010009765625,6.37100219726563z" });
                                break;
                            case "ThreeSignsCircle":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-top:1px;margin-left:0.5px;';
                                break;
                            case "ThreeSignsDiamond":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M1.08401489257813,8.01901245117188L8.10400390625,15.1589965820313 15.2929992675781,8.01901245117188 8.10400390625,0.878997802734375 1.08401489257813,8.01901245117188z" });
                                innerDivWidth = 16;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:1px;';
                                break;
                            case "ThreeSignsTriangle":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M0.83099365234375,14.2340087890625L15.7950134277344,14.2340087890625 8.31298828125,1.27499389648438 0.83099365234375,14.2340087890625z" });
                                svgAttribute = svgAttribute + 'margin-top:1px;';
                                break;
                            case "ThreeSymbolCheck":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M12.15869140625,4.38958740234375L11.3836975097656,3.86260986328125 10.7337036132813,3.98660278320313 6.94171142578125,9.57958984375 5.19769287109375,7.83660888671875 4.53570556640625,7.83660888671875 3.87469482421875,8.49859619140625 3.87469482421875,9.16061401367188 6.5546875,11.8416137695313 7.1536865234375,12.1086120605469 7.71771240234375,11.7735900878906 12.28369140625,5.03860473632813 12.15869140625,4.38958740234375" });
                                svgAttribute = svgAttribute + 'margin-top:1px;margin-left:0.5px;';
                                break;
                            case "ThreeSymbolCross":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M1.10299682617188,7.94601440429688L1.66799592971802,10.7435569763184 3.20874404907227,13.0283813476563 5.49386739730835,14.569019317627 8.2919921875,15.1340026855469 8.2919921875,15.1340026855469 11.0901165008545,14.569019317627 13.3752403259277,13.0283813476563 14.9159889221191,10.7435569763184 15.4809875488281,7.94601440429688 15.4809875488281,7.94601440429688 14.9159889221191,5.14773225784302 13.3752403259277,2.86226272583008 11.0901165008545,1.32116842269897 8.2919921875,0.756011962890625 8.2919921875,0.756011962890625 5.49386739730835,1.32116842269897 3.20874404907227,2.86226272583008 1.66799592971802,5.14773225784302 1.10299682617188,7.94601440429688" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M10.0479125976563,7.94430541992188L11.9789123535156,6.0133056640625 11.9789123535156,5.31130981445313 10.9259033203125,4.25729370117188 10.222900390625,4.25729370117188 8.2908935546875,6.18930053710938 6.35989379882813,4.25729370117188 5.65689086914063,4.25729370117188 4.60488891601563,5.31130981445313 4.60488891601563,6.0133056640625 6.534912109375,7.94430541992188 4.60488891601563,9.87728881835938 4.60488891601563,10.5783081054688 5.65689086914063,11.6322937011719 6.35989379882813,11.6322937011719 8.2908935546875,9.70028686523438 10.2239074707031,11.6322937011719 10.9259033203125,11.6322937011719 11.9789123535156,10.5783081054688 11.9789123535156,9.87728881835938 10.0479125976563,7.94430541992188z" });
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:1px;';
                                break;
                            case "ThreeSymbolExclamation":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M0.983001708984375,7.94601440429688L1.54785919189453,10.7435569763184 3.08837127685547,13.0283813476563 5.37344741821289,14.569019317627 8.1719970703125,15.1340026855469 8.1719970703125,15.1340026855469 10.9699649810791,14.569019317627 13.2547416687012,13.0283813476563 14.7951431274414,10.7435569763184 15.3599853515625,7.94601440429688 15.3599853515625,7.94601440429688 14.7951431274414,5.14773225784302 13.2547416687012,2.86226272583008 10.9699649810791,1.32116842269897 8.1719970703125,0.756011962890625 8.1719970703125,0.756011962890625 5.37344741821289,1.32116842269897 3.08837127685547,2.86226272583008 1.54785919189453,5.14773225784302 0.983001708984375,7.94601440429688" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M7.59280395507813,9.60159301757813L8.71978759765625,9.60159301757813 9.00180053710938,9.60159301757813 9.12579345703125,2.8916015625 7.143798828125,2.8916015625 7.31280517578125,9.60159301757813 7.59280395507813,9.60159301757813z" });
                                createElement(groupElement, "path", { fill: "white", stroke: "lightgray", d: "M8.96728515625,10.69140625L8.17428588867188,10.3753967285156 7.37130737304688,10.6924133300781 7.03228759765625,11.4764099121094 7.37530517578125,12.244384765625 8.17428588867188,12.54541015625 8.96328735351563,12.244384765625 9.310302734375,11.4764099121094 8.96728515625,10.69140625" });
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:1px;';
                                break;
                            case "ThreeSymbolUnCircledCheck":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.8515930175781,2.2255859375L12.6495971679688,1.402587890625 12.1062202453613,1.29134368896484 11.6416015625,1.59561157226563 5.755615234375,10.32958984375 3.04959106445313,7.60760498046875 2.53897094726563,7.39609527587891 2.02459716796875,7.60760498046875 0.997589111328125,8.64260864257813 0.785346984863281,9.15935516357422 0.997589111328125,9.6776123046875 5.1575927734375,13.8635864257813 6.0845947265625,14.2785949707031 6.96060180664063,13.755615234375 14.0445861816406,3.2406005859375 14.1565895080566,2.69297027587891 13.8515930175781,2.2255859375" });
                                innerDivWidth = innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "ThreeSymbolUnCircledCross":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M9.36907958984375,6.86331176757813L12.423095703125,3.71530151367188 12.6540832519531,3.14280700683594 12.423095703125,2.5703125 10.7581176757813,0.851287841796875 10.2026062011719,0.613548278808594 9.6470947265625,0.851287841796875 6.59613037109375,4.00131225585938 3.5421142578125,0.851287841796875 2.98759841918945,0.613548278808594 2.43310546875,0.851287841796875 0.768096923828125,2.5703125 0.537109375,3.14205169677734 0.768096923828125,3.71530151367188 3.82012939453125,6.86331176757813 0.768096923828125,10.0133056640625 0.537109375,10.5845413208008 0.768096923828125,11.1572875976563 2.43310546875,12.8763122558594 2.98723220825195,13.1140518188477 3.5421142578125,12.8763122558594 6.59613037109375,9.72628784179688 9.6480712890625,12.8763122558594 10.2027282714844,13.1140518188477 10.7581176757813,12.8763122558594 12.423095703125,11.1572875976563 12.6540832519531,10.5845413208008 12.423095703125,10.0133056640625 9.36907958984375,6.86331176757813z" });
                                innerDivWidth = innerDivHeight = 16;
                                svgAttribute = svgAttribute + 'margin-top:5px;margin-left:1.5px;';
                                break;
                            case "ThreeSymbolUnCircledExclamation":
                                createElement(groupElement, "path", { fill: "none", stroke: "none", d: "M0.983001708984375,7.94601440429688L1.54785919189453,10.7435569763184 3.08837127685547,13.0283813476563 5.37344741821289,14.569019317627 8.1719970703125,15.1340026855469 8.1719970703125,15.1340026855469 10.9699649810791,14.569019317627 13.2547416687012,13.0283813476563 14.7951431274414,10.7435569763184 15.3599853515625,7.94601440429688 15.3599853515625,7.94601440429688 14.7951431274414,5.14773225784302 13.2547416687012,2.86226272583008 10.9699649810791,1.32116842269897 8.1719970703125,0.756011962890625 8.1719970703125,0.756011962890625 5.37344741821289,1.32116842269897 3.08837127685547,2.86226272583008 1.54785919189453,5.14773225784302 0.983001708984375,7.94601440429688" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M7.59280395507813,9.60159301757813L8.71978759765625,9.60159301757813 9.00180053710938,9.60159301757813 9.12579345703125,2.8916015625 7.143798828125,2.8916015625 7.31280517578125,9.60159301757813 7.59280395507813,9.60159301757813z" });
                                createElement(groupElement, "path", { fill: fillColor, stroke: "none", d: "M8.96728515625,10.69140625L8.17428588867188,10.3753967285156 7.37130737304688,10.6924133300781 7.03228759765625,11.4764099121094 7.37530517578125,12.244384765625 8.17428588867188,12.54541015625 8.96328735351563,12.244384765625 9.310302734375,11.4764099121094 8.96728515625,10.69140625" });
                                svgAttribute = svgAttribute + 'margin-top:1px;margin-left:1px;';
                                break;
                            case "TrafficLight":
                                createElement(groupElement, "path", { fill: "black", stroke: "lightgray", d: "M3.83200073242188,1.46800231933594L2.76542282104492,1.68336224555969 1.89412689208984,2.27062797546387 1.3065185546875,3.14157843589783 1.09100341796875,4.20799255371094 1.09100341796875,4.20799255371094 1.09100341796875,13.3419952392578 1.3065185546875,14.4091548919678 1.89412689208984,15.2807502746582 2.76542282104492,15.8684692382813 3.83200073242188,16.0839996337891 3.83200073242188,16.0839996337891 12.9670104980469,16.0839996337891 14.0340003967285,15.8684692382813 14.9052505493164,15.2807502746582 15.4926300048828,14.4091548919678 15.7080078125,13.3419952392578 15.7080078125,13.3419952392578 15.7080078125,4.20799255371094 15.4926300048828,3.14157843589783 14.9052505493164,2.27062797546387 14.0340003967285,1.68336224555969 12.9670104980469,1.46800231933594 12.9670104980469,1.46800231933594 3.83200073242188,1.46800231933594z" });
                                createElement(groupElement, "path", { fill: fillColor, d: "M3.78421020507813,8.77540588378906L4.14698934555054,6.97894716262817 5.13620376586914,5.51202392578125 6.60316896438599,4.52304029464722 8.39920043945313,4.160400390625 10.1958141326904,4.52304029464722 11.6630783081055,5.51202392578125 12.6524028778076,6.97894716262817 13.0151977539063,8.77540588378906 12.6524028778076,10.5718555450439 11.6630783081055,12.0387744903564 10.1958141326904,13.027756690979 8.39920043945313,13.3903961181641 6.60316896438599,13.027756690979 5.13620376586914,12.0387744903564 4.14698934555054,10.5718555450439 3.78421020507813,8.77540588378906" });
                                innerDivWidth = innerDivHeight = 18;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "TrafficLightUnrimmed":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M15.3310852050781,8.3330078125L14.7622108459473,11.150749206543 13.2108383178711,13.4517440795898 10.9098434448242,15.003116607666 8.09210205078125,15.5719909667969 5.27377843856812,15.003116607666 2.97248458862305,13.4517440795898 1.42100191116333,11.150749206543 0.85211181640625,8.3330078125 1.42100191116333,5.51524877548218 2.97248458862305,3.2142448425293 5.27377843856812,1.66286897659302 8.09210205078125,1.093994140625 10.9098434448242,1.66286897659302 13.2108383178711,3.2142448425293 14.7622108459473,5.51524877548218 15.3310852050781,8.3330078125" });
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "TriangleDash":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M13.9800109863281,7.56300354003906L0.501007080078125,7.56300354003906 0.501007080078125,4.67400360107422 13.9800109863281,4.67400360107422 13.9800109863281,7.56300354003906z" });
                                innerDivWidth = 15;
                                innerDivHeight = 9;
                                break;
                            case "TriangleDown":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M14.06201171875,0.788002014160156L7.60501098632813,7.15699768066406 1.14801025390625,0.788002014160156 14.06201171875,0.788002014160156z" });
                                innerDivWidth = 15;
                                innerDivHeight = 9;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                            case "TriangleUp":
                                createElement(groupElement, "path", { fill: fillColor, stroke: "lightgray", d: "M0.89599609375,7.30400085449219L7.35198974609375,0.93499755859375 13.8089904785156,7.30400085449219 0.89599609375,7.30400085449219z" });
                                innerDivWidth = 15;
                                innerDivHeight = 9;
                                svgAttribute = svgAttribute + 'margin-top:3px;margin-left:.5px;';
                                break;
                        }
                        indicatorinnerDiv.css({ 'width': innerDivWidth + 'px', 'height': innerDivHeight + 'px' });
                        svgelement.setAttributeNS(null, 'style', svgAttribute);
                        svgelement.appendChild(groupElement);
                        break;
                    }
                }
                indicatorinnerDiv.append(svgelement);
                indicatorDiv.append(indicatorinnerDiv);
                gaugeDiv.append(indicatorDiv);

                var innerScaleWidth = indicatorinnerDiv.width();
                var innerScaleHeight = indicatorinnerDiv.height();
                var indidenomintaorLen = innerScaleWidth < innerScaleHeight ? innerScaleWidth : innerScaleHeight;
                var browserInfo = ej.browserInfo();
                scaleXy = scaleXy / indidenomintaorLen;

                if (browserInfo.name == 'msie') {
                    indicatorinnerDiv.css('transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                } else if (browserInfo.name == 'opera') {
                    indicatorinnerDiv.css('-o-transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                } else if (browserInfo.name == 'mozilla') {
                    indicatorinnerDiv.css('-ms-transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                } else if (browserInfo.name == 'webkit' || browserInfo.name == 'chrome') {
                    indicatorinnerDiv.css('-webkit-transform', 'scale(' + scaleXy + ',' + scaleXy + ')');
                }

                if (innerScaleWidth < innerScaleHeight) {
                    indicatorinnerDiv.css({ 'left': '50%', 'margin-left': -(innerScaleWidth / 2), 'top': '50%', 'margin-top': -(innerScaleWidth / 2) });
                } else {
                    indicatorinnerDiv.css({ 'left': '50%', 'margin-left': -(innerScaleHeight / 2), 'top': '50%', 'margin-top': -(innerScaleHeight / 2) });
                }
            }

            function createElement(root, name, attrs) {
                var element = document.createElementNS(svgNs, name);
                for (var attr in attrs) {
                    if (attrs.hasOwnProperty(attr)) {
                        element.setAttribute(attr, attrs[attr]);
                    }
                }
                root.appendChild(element);
            }
        },

        _renderGaugeLabels: function (gaugeLabels, gaugeDiv, gaugeName, parentId, printMode, printPageId) {
            var guageIndex;
            for (guageIndex = 0; guageIndex < gaugeLabels.length; guageIndex++) {
                var labelModel = gaugeLabels[guageIndex];
                var labelGaugeDiv = ej.buildTag("div", "", {}, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + labelModel.Name, parentId, printMode, printPageId), title: labelModel.ToolTip });
                labelGaugeDiv.css({ "position": "absolute", "top": labelModel.Top, "left": labelModel.Left, "width": labelModel.length + 10, "height": labelModel.Height + 10, "background-color": labelModel.BackgroundColor, "border-color": labelModel.Style.BorderColor, "border-style": labelModel.Style.BorderStyle, "border-width": labelModel.Style.BorderWidth });
                var span = document.createElement("span");
                span.innerHTML = labelModel.Text;
                span.style.fontFamily = labelModel.Font.FontFamily;
                span.style.fontSize = labelModel.Font.FontSize;
                span.style.fontStyle = labelModel.Font.FontStyle;
                span.style.fontWeight = labelModel.Font.FontWeight;
                span.style.textAlign = labelModel.TextAlign;
                span.style.color = labelModel.TextColor;
                span.style.textDecoration = labelModel.TextDecoration;
                labelGaugeDiv.append(span);
                gaugeDiv.append(labelGaugeDiv);
            }
        },

        _renderRadialGauge: function (radialGauges, gaugeDiv, gaugeName, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId) {
            for (var guageIndex = 0; guageIndex < radialGauges.length; guageIndex++) {
                var radialGauge = radialGauges[guageIndex];
                var radialframe = radialGauge.GaugeFrame;
                var radialGaugeDiv = ej.buildTag("div", "", {}, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + radialGauge.Name, parentId, printMode, printPageId), title: radialGauge.ToolTip });
                gaugeDiv.append(radialGaugeDiv);
                radialGaugeDiv.ejCircularGauge();
                var resourceName = printMode ? 'PrintImage' : 'GetResource';
                if (radialframe.FrameImage) {
                    radialGaugeDiv.css("background-image", "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + radialframe.FrameImage + '&resourcetype=sfimg&isPrint=' + printMode + ")");
                }
                var circular = radialGaugeDiv.data("ejCircularGauge");
                var radialscaleobj = circular.model.scales[0];
                circular.model.scales.pop(circular.model.scales[0]);

                var radialheight = (containerWidth < containerHeight) ? (containerWidth / 2) - radialframe.FrameWidth : (containerHeight / 2) - radialframe.FrameWidth;
                circular.model.radius = radialheight;
                circular.model.width = containerWidth;
                circular.model.height = containerHeight;
                circular.model.backgroundColor = radialGauge.BackgroundColor;
                circular.model.enableAnimation = false;

                if (radialGauge.ScaleProperties && radialGauge.ScaleProperties.length > 0) {
                    var radialscaleLen = radialGauge.ScaleProperties.length;
                    for (var radialscaleIndex = 0; radialscaleIndex < radialscaleLen; radialscaleIndex++) {
                        var radialScale = radialGauge.ScaleProperties[radialscaleIndex];
                        var radialscale = $.extend(true, {}, radialscaleobj);
                        radialscale.radius = (radialheight * ((radialscaleLen - radialscaleIndex) / radialscaleLen)) - radialframe.FrameWidth;
                        radialscale.startAngle = radialScale.StartAngle + 90;
                        radialscale.sweepAngle = radialScale.SweepAngle;
                        radialscale.showRanges = true;
                        radialscale.showScaleBar = true;
                        radialscale.maximum = radialScale.Maximum;
                        radialscale.minimum = radialScale.Minimum;
                        radialscale.border.color = radialScale.BorderColor;
                        radialscale.border.width = radialScale.BorderWidth;
                        radialscale.showPointers = true;
                        radialscale.showTicks = true;
                        radialscale.showLabels = true;
                        radialscale.showIndicators = true;
                        radialscale.backgroundColor = radialScale.BackgroundColor;
                        radialscale.shadowOffset = radialScale.ShadowOffset;

                        if (radialScale.RadialPointer && radialScale.RadialPointer.length > 0) {
                            var radialpointerobj = radialscale.pointers[0];
                            radialscale.pointers.pop(radialscale.pointers[0]);
                            var radialpointerLen = radialScale.RadialPointer.length;
                            for (var radialpointerIndex = 0; radialpointerIndex < radialpointerLen; radialpointerIndex++) {
                                var radialpointer = $.extend(true, {}, radialpointerobj);
                                var scalePointer = radialScale.RadialPointer[radialpointerIndex];
                                radialpointer.value = scalePointer.Value;
                                radialpointer.length = radialheight * ((radialscaleLen - radialscaleIndex) / radialscaleLen) - 30;
                                radialpointer.backgroundColor = scalePointer.BackgroundColor;
                                radialpointer.type = scalePointer.Type == "Needle" ? "needle" : "marker";
                                radialpointer.showBackNeedle = false;
                                radialpointer.width = scalePointer.Width;
                                radialpointer.border.width = scalePointer.BorderWidth;
                                radialpointer.border.color = scalePointer.BorderColor;

                                switch (scalePointer.NeedleStyle) {
                                    case "Rectangular":
                                        radialpointer.needleType = "rectangle";
                                        break;
                                    case "Arrow":
                                        radialpointer.needleType = "arrow";
                                        break;
                                    default:
                                        radialpointer.needleType = "triangle";
                                        break;
                                }

                                if (scalePointer.MarkerStyle == "wedge" || scalePointer.MarkerStyle == "triangle" || scalePointer.MarkerStyle == "circle" || scalePointer.MarkerStyle == "diamond" || scalePointer.MarkerStyle == "pentagon" || scalePointer.MarkerStyle == "trapezoid") {
                                    radialpointer.markerType = scalePointer.MarkerStyle.toLowerCase();
                                } else {
                                    radialpointer.markerType = "rectangle";
                                }

                                radialpointer.distanceFromScale = scalePointer.distanceFromScale;
                                radialscale.pointerCap.backgroundColor = scalePointer.CapBackGroundColor;
                                radialscale.pointerCap.radius = scalePointer.CapRadius / 3;
                                radialscale.pointers.push(radialpointer);
                            }
                        } else {
                            radialscale.pointers = [];
                        }

                        if (radialScale.Range && radialScale.Range.length > 0) {
                            var radialrangeobj = radialscale.ranges[0];
                            radialscale.ranges.pop(radialscale.ranges[0]);
                            var raddialrangeLen = radialScale.Range.length;
                            for (var raddialrangeIndex = 0; raddialrangeIndex < raddialrangeLen; raddialrangeIndex++) {
                                var radialrange = $.extend(true, {}, radialrangeobj);
                                var radialscaleRange = radialScale.Range[raddialrangeIndex];
                                radialrange.distanceFromScale = radialscaleRange.DistanceFromScale;
                                radialrange.backgroundColor = radialscaleRange.BackgroundColor;
                                if (radialscaleRange.EndValue == 0 && radialscaleRange.StartValue == 0) {
                                    radialrange.startValue = 70;
                                    radialrange.endValue = 100;
                                } else {
                                    radialrange.startValue = radialscaleRange.StartValue;
                                    radialrange.endValue = radialscaleRange.EndValue;
                                }
                                radialrange.startWidth = radialscaleRange.StartWidth;
                                radialrange.endWidth = radialscaleRange.EndWidth;
                                radialrange.border.color = radialscaleRange.BorderColor;
                                radialrange.border.width = radialscaleRange.BorderWidth;
                                radialscale.ranges.push(radialrange);
                            }
                        } else {
                            radialscale.ranges = [];
                        }

                        var radialtick = radialScale.TickMark;
                        var radialmajortickobj = radialscale.ticks[0];
                        var radialminortickobj = radialscale.ticks[1];
                        radialscale.ticks.pop(radialscale.ticks[0]);
                        radialscale.ticks.pop(radialscale.ticks[1]);

                        if (!radialtick.MajorTickHide) {
                            radialmajortickobj.type = "major";
                            radialmajortickobj.width = radialtick.MajorTickWidth + 1;
                            radialmajortickobj.distanceFromScale = radialtick.MajorTickDistanceFromScale;
                            radialmajortickobj.height = radialtick.MajorTickLength - 2;
                            radialmajortickobj.color = radialtick.MajorTickColor == null ? "black" : radialtick.MajorTickColor;
                            radialmajortickobj.placement = "near";
                            radialscale.ticks.push(radialmajortickobj);
                        }

                        if (!radialtick.MinorTickHide) {
                            radialminortickobj.type = "minor";
                            radialminortickobj.width = radialtick.MinorTickWidth;
                            radialminortickobj.distanceFromScale = radialtick.MinorTickDistanceFromScale;
                            radialminortickobj.height = radialtick.MinorTickLength - 1;
                            radialminortickobj.color = radialtick.MinorTickColor == null ? "black" : radialtick.MinorTickColor;
                            radialminortickobj.placement = "near";
                            radialscale.ticks.push(radialminortickobj);
                        }

                        var radiallabelValue = radialScale.Label;
                        var radialscalelabel = radialscale.labels[0];
                        radialscale.labels.pop(radialscale.labels[0]);
                        radialscalelabel.font.fontFamily = radiallabelValue.FontFamily;
                        radialscalelabel.font.size = radiallabelValue.FontSize;
                        radialscalelabel.font.fontStyle = radiallabelValue.FontStyle;
                        radialscalelabel.distanceFromScale = radiallabelValue.DistanceFromScale;
                        radialscalelabel.color = radiallabelValue.Color;
                        radialscale.labels.push(radialscalelabel);
                        radialscale.indicators = null;
                        circular.model.scales.push(radialscale);
                    }
                }
                circular.refresh();
                circular.contextEl.beginPath();
                circular.contextEl.arc((containerWidth / 2), (containerHeight / 2), radialheight, 0, 2 * Math.PI, false);
                circular.contextEl.fillStyle = "transparent";
                circular.contextEl.fill();
                circular.contextEl.lineWidth = radialframe.FrameWidth;
                circular.contextEl.strokeStyle = (radialframe.FrameColor != null) ? radialframe.FrameColor : "Transparent";
                circular.contextEl.stroke();
                circular.contextEl.closePath();

                if (printMode) {
                    radialGaugeDiv.append('<img src=' + circular.contextEl.canvas.toDataURL("image/png") + ' width=100% height=100%/>');
                    $(circular.contextEl.canvas).remove();
                }
                radialGaugeDiv.css({ 'width': containerWidth, 'height': containerHeight, 'float': panelOrientation == 'Horizontal' ? 'left' : 'top' });
            }
        },

        _renderLinearGauge: function (linearGauges, gaugeDiv, gaugeName, panelOrientation, containerWidth, containerHeight, parentId, printMode, printPageId) {
            for (var guageIndex = 0; guageIndex < linearGauges.length; guageIndex++) {
                var linearGauge = linearGauges[guageIndex];

                if (guageIndex % 2 != 0) {
                    var emptyDiv = ej.buildTag("div", "", { 'width': panelOrientation == 'Horizontal' ? '3px' : containerWidth - (frame.FrameWidth * 2), 'float': panelOrientation == 'Horizontal' ? 'left' : 'top', 'height': panelOrientation == 'Horizontal' ? containerHeight - (frame.FrameWidth * 2) : '3px' }, {});
                    gaugeDiv.append(emptyDiv);
                }

                var frame = linearGauge.GaugeFrame;
                var linearGaugeDiv = ej.buildTag("div", "", {}, { 'id': this._controlKeyGenerator(this._id + '_' + gaugeName + '_' + linearGauge.Name, parentId, printMode, printPageId), title: linearGauge.ToolTip });
                var resourceName = printMode ? 'PrintImage' : 'GetResource';
                if (frame.FrameImage) {
                    linearGaugeDiv.css("background-image", "url(" + this.model.reportServiceUrl + '/' + resourceName + '/?key=' + frame.FrameImage + '&resourcetype=sfimg&isPrint=' + printMode + ")");
                }

                gaugeDiv.append(linearGaugeDiv);
                linearGaugeDiv.ejLinearGauge();
                var linear = linearGaugeDiv.data("ejLinearGauge");

                var scaleobj = linear.model.scales[0];
                linear.model.scales.pop(linear.model.scales[0]);
                linear.model.width = containerWidth - (frame.FrameWidth * 2);
                linear.model.height = containerHeight - (frame.FrameWidth * 2);

                linear.model.enableAnimation = false;
                linear.model.backgroundColor = linearGauge.BackgroundColor;
                linear.model.orientation = linearGauge.Orientation;

                if (linearGauge.Orientation == "Horizontal") {
                    linear.model.orientation = linearGauge.Orientation;
                } else if (linearGauge.Orientation == "Vertical") {
                    linear.model.orientation = linearGauge.Orientation;
                } else {
                    if (containerWidth > containerHeight) {
                        linear.model.orientation = "Horizontal";
                    } else {
                        linear.model.orientation = "Vertical";
                    }
                }

                if (linearGauge.ScaleProperties && linearGauge.ScaleProperties.length > 0) {
                    var scaleLen = linearGauge.ScaleProperties.length;

                    for (var scaleIndex = 0; scaleIndex < scaleLen; scaleIndex++) {
                        var linearScale = linearGauge.ScaleProperties[scaleIndex];
                        var scale = $.extend(true, {}, scaleobj);
                        scale.maximum = linearScale.Maximum;
                        scale.minimum = linearScale.Minimum;
                        scale.length = linear.model.orientation == "Horizontal" ? (containerWidth / 100) * 85 : (containerHeight / 100) * 85;                         scale.border.color = linearScale.BorderColor;
                        scale.border.width = linearScale.BorderWidth;
                        scale.backgroundColor = linearScale.BackgroundColor;
                        scale.shadowOffset = linearScale.ShadowOffset;
                        scale.width = linear.model.orientation == "Vertical" ? (linear.model.width / 4) : (linear.model.height / 4);

                        if (linearScale.PointerType && linearScale.PointerType == "thermometer") {
                            scale.type = linearScale.PointerType;
                        }

                        if (linearScale.LinearPointer && linearScale.LinearPointer.length > 0) {


                            var pointerobj;
                            if (linearScale.PointerType != "thermometer") {
                                pointerobj = $.extend(true, {}, scale.markerPointers[0]);
                                scale.markerPointers.pop(scale.markerPointers[0]);
                            } else {
                                pointerobj = $.extend(true, {}, scale.barPointers[0])
                                scale.barPointers.pop(scale.barPointers[0]);
                            }
                            var pointerLen = linearScale.LinearPointer.length;
                            for (var pointerIndex = 0; pointerIndex < pointerLen; pointerIndex++) {
                                var pointer = $.extend(true, {}, pointerobj);
                                var scaleMarkerPointer = linearScale.LinearPointer[pointerIndex];
                                pointer.value = scaleMarkerPointer.MarkerValue;
                                pointer.length = scaleMarkerPointer.MarkerLength;
                                pointer.backgroundColor = scaleMarkerPointer.MarkerBackgroundColor;
                                pointer.width = scaleMarkerPointer.MarkerWidth;
                                pointer.border.width = scaleMarkerPointer.MarkerBorderWidth;
                                pointer.border.color = scaleMarkerPointer.MarkerBorderColor;
                                pointer.type = scaleMarkerPointer.MarkerStyle.toLowerCase();
                                pointer.distanceFromScale = scaleMarkerPointer.MarkerDistanceFromScale;
                                if (linearScale.PointerType != "thermometer") {
                                    scale.markerPointers.push(pointer);
                                } else {
                                    scale.showBarPointers = true;
                                    scale.barPointers.push(pointer);
                                    scale.markerPointers[pointerIndex].opacity = 0;
                                }
                            }
                        } else {
                            scale.markerPointers = [];
                            scale.barPointers = [];
                        }

                        if (linearScale.Range && linearScale.Range.length > 0) {
                            var rangeobj = $.extend(true, {}, scale.ranges[0]);
                            scale.ranges.pop(scale.ranges[0]);
                            var rangeLen = linearScale.Range.length;

                            for (var rangeIndex = 0; rangeIndex < rangeLen; rangeIndex++) {
                                var range = $.extend(true, {}, rangeobj);
                                var scaleRange = linearScale.Range[rangeIndex];

                                range.distanceFromScale = scaleRange.DistanceFromScale;
                                range.backgroundColor = scaleRange.BackgroundColor;
                                range.startValue = scaleRange.StartValue;
                                range.startWidth = scaleRange.StartWidth;
                                range.endValue = scaleRange.EndValue;
                                range.endWidth = scaleRange.EndWidth;
                                range.border.color = scaleRange.BorderColor;
                                range.border.width = scaleRange.BorderWidth;
                                scale.ranges.push(range);
                            }
                        } else {
                            scale.ranges = [];
                        }

                        var tick = linearScale.TickMark;
                        var majortickobj = scale.ticks[0];
                        var minortickobj = scale.ticks[1];
                        scale.ticks.pop(scale.ticks[0]);
                        scale.ticks.pop(scale.ticks[1]);
                        majortickobj.type = "majorInterval";
                        majortickobj.width = tick.MajorTickWidth;
                        majortickobj.height = tick.MajorTickLength / 2;
                        majortickobj.color = tick.MajorTickColor == null ? "black" : tick.MajorTickColor;
                        scale.ticks.push(majortickobj);
                        minortickobj.type = "minorInterval";
                        minortickobj.width = tick.MinorTickWidth;
                        minortickobj.height = tick.MinorTickLength * 2;
                        minortickobj.color = tick.MinorTickColor == null ? "black" : tick.MinorTickColor;
                        scale.ticks.push(minortickobj);

                        var labelValue = linearScale.Label;
                        var scalelabel = scale.labels[0];
                        scale.labels.pop(scale.labels[0]);
                        scalelabel.font.fontFamily = labelValue.LinearLabelFontFamily;
                        scalelabel.font.size = labelValue.LinearLabelFontSize;
                        scalelabel.font.fontStyle = labelValue.LinearLabelFontStyle;
                        scalelabel.distanceFromScale = labelValue.LinearLabelDistanceFromScale;
                        scalelabel.textColor = labelValue.TextColor;
                        scale.labels.push(scalelabel);

                        scale.indicators = null;
                        scale.customLabels = [];

                        linear.model.scales.push(scale);
                    }
                    linear.refresh();

                    if (printMode) {
                        linearGaugeDiv.append('<img src=' + linear.contextEl.canvas.toDataURL("image/png") + ' width=100% height=100%/>');
                        $(linear.contextEl.canvas).remove();
                    }
                    linearGaugeDiv.css({ 'width': panelOrientation == 'Horizontal' ? (containerWidth - (frame.FrameWidth * 2) - 3) : containerWidth - (frame.FrameWidth * 2), 'height': panelOrientation == 'Horizontal' ? containerHeight - (frame.FrameWidth * 2) : ((containerHeight - (frame.FrameWidth * 2)) - 3), 'float': panelOrientation == 'Horizontal' ? 'left' : 'top', 'border-color': (frame.FrameColor != null) ? frame.FrameColor : "Transparent", 'border-width': frame.FrameWidth, 'border-style': 'solid', 'background-color': linearGauge.BackgroundColor });
                }
            }
        },
        //-------------------- Render the Gauge ReportItem [start] -------------------------//

        _renderLineControl: function (lineModel, parentId, printMode, printPageId) {
            var svgNs = "http://www.w3.org/2000/svg";
            var ctrlId = this._controlKeyGenerator(this._id + '_' + lineModel.Name, parentId, printMode, printPageId);
            var linDiv = ej.buildTag("div", "", {}, { 'id': ctrlId });
            var _linDivCss = {};
            _linDivCss["position"] = 'absolute';
            _linDivCss["top"] = lineModel.Top;
            _linDivCss["left"] = lineModel.Left;
            _linDivCss["width"] = lineModel.Width == 0 ? '11.00064' : (lineModel.Width);
            _linDivCss["height"] = lineModel.Height == 0 ? '16.00128' : (lineModel.Height);
            linDiv.css(_linDivCss);

            if (this._svg) {
                var svgelement = document.createElementNS(svgNs, "svg");
                svgelement.setAttributeNS(null, 'width', lineModel.Width == 0 ? "11.00064" : (lineModel.Width));
                svgelement.setAttributeNS(null, 'height', lineModel.Height == 0 ? "16.00128" : (lineModel.Height));

                var svgtagelement = document.createElementNS(svgNs, "line");
                svgtagelement.setAttributeNS(null, "x1", lineModel.X1 == 0 ? lineModel.LineWidth : (lineModel.X1 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "y1", lineModel.Y1 == 0 ? lineModel.LineWidth : (lineModel.Y1 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "x2", lineModel.X2 == 0 ? lineModel.LineWidth : (lineModel.X2 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "y2", lineModel.Y2 == 0 ? lineModel.LineWidth : (lineModel.Y2 - lineModel.LineWidth));
                svgtagelement.setAttributeNS(null, "stroke", lineModel.LineColor);
                svgtagelement.setAttributeNS(null, "stroke-width", lineModel.LineWidth);
                svgtagelement.setAttributeNS(null, "shape-rendering", "crispEdges");
                if (lineModel.LineStyle == "Dashed") {
                    svgtagelement.setAttributeNS(null, "stroke-dasharray", "" + (lineModel.LineWidth * 2) + "," + (lineModel.LineWidth * 2) + "");
                } else if (lineModel.LineStyle == "Dotted") {
                    svgtagelement.setAttributeNS(null, "stroke-dasharray", lineModel.LineWidth);
                }
                svgelement.appendChild(svgtagelement);
                linDiv.append(svgelement);
            } else {
                document.namespaces.add('v', 'urn:schemas-microsoft-com:vml');
                document.createStyleSheet().cssText = 'v\\:fill, v\\:path, v\\:polyline, v\\:line, v\\:rect,v\\:shape,v\\:oval, v\\:stroke' + '{ behavior:url(#default#VML); display: inline-block; } ';

                var vmlObject = document.createElement("<v:line id='line' from='" + lineModel.X1 + ',' + lineModel.Y1 + "' to='" + lineModel.X2 + ',' + lineModel.Y2 + "' strokecolor='" + lineModel.LineColor + "' strokeweight='" + lineModel.LineWidth + "pt'>" + "<v:stroke dashstyle='dot'></v:stroke>" + "</v:line>");
                var str = document.createElement("v:stroke");
                if (lineModel.LineStyle == "Dotted") {
                    str.setAttribute("dashstyle", "dot");
                } else if (lineModel.LineStyle == "Dashed") {
                    str.setAttribute("dashstyle", "dash");
                } else {
                    str.setAttribute("dashstyle", "solid");
                }
                str.outerHTML = str.outerHTML;
                vmlObject.appendChild(str);

                vmlObject.style.position = "absolute";
                vmlObject.style.width = lineModel.Width;
                vmlObject.style.height = lineModel.Height;
                vmlObject.style.visibility = "visible";

                linDiv.append(vmlObject);
            }
            return linDiv;
        },

        _renderSubReportControl: function (subreportModel, isTablixCell, parentObj, parentId, printMode, printPageId) {
            var ctrlId = this._controlKeyGenerator(this._id + '_' + subreportModel.Name, parentId, printMode, printPageId);
            var subReportDiv = ej.buildTag("div", "", {}, { 'id': ctrlId, title: subreportModel.ToolTip });

            parentObj.append(subReportDiv);
            subReportDiv.css('position', isTablixCell ? 'relative' : 'absolute');
            subReportDiv.css('top', subreportModel.Top);
            subReportDiv.css('left', subreportModel.Left);
            subReportDiv.css('width', subreportModel.Width);
            subReportDiv.css('height', subreportModel.Height);
            subReportDiv.css('background-color', subreportModel.BackgroundColor);

            if (subreportModel.Padding) {
                if (subreportModel.Padding.Left) {
                    subReportDiv.css("padding-left", subreportModel.Padding.Left);
                }

                if (subreportModel.Padding.Right) {
                    subReportDiv.css("padding-right", subreportModel.Padding.Right);
                }

                if (subreportModel.Padding.Top) {
                    subReportDiv.css("padding-top", subreportModel.Padding.Top);
                }

                if (subreportModel.Padding.Bottom) {
                    subReportDiv.css("padding-bottom", subreportModel.Padding.Bottom);
                }
            }

            if (subreportModel.Style) {
                if (subreportModel.Style.Font) {
                    if (subreportModel.Style.Font.FontFamily) {
                        subReportDiv.css("font-family", subreportModel.Style.Font.FontFamily);
                    }

                    if (subreportModel.Style.Font.FontSize) {
                        subReportDiv.css("font-size", subreportModel.Style.Font.FontSize);
                    }

                    if (subreportModel.Style.Font.FontStyle) {
                        subReportDiv.css("font-style", subreportModel.Style.Font.FontStyle);
                    }

                    if (subreportModel.Style.Font.FontWeight) {
                        subReportDiv.css("font-weight", subreportModel.Style.Font.FontWeight);
                    }
                }

                subReportDiv.css("color", subreportModel.Style.TextColor ? subreportModel.Style.TextColor : "black");

                if (subreportModel.Style.TextDecoration) {
                    subReportDiv.css("text-decoration", subreportModel.Style.TextDecoration);
                }
            }

            if (subreportModel.LineHeight) {
                subReportDiv.css("line-height", subreportModel.LineHeight);
            }

            if (subreportModel.TextAlign) {
                subReportDiv.css("text-align", subreportModel.TextAlign);
            }

            if (subreportModel.VerticalAlign) {
                subReportDiv.css("vertical-align", subreportModel.VerticalAlign);
            }

            if (subreportModel.Border && !isTablixCell) {
                this._applyBorderStyle(subreportModel.Border, subReportDiv);
            }

            if (subreportModel.ReportItemModels && subreportModel.ReportItemModels.length > 0) {
                for (var i = 0; i < subreportModel.ReportItemModels.length; i++) {
                    var innerItem = subreportModel.ReportItemModels[i];
                    if (innerItem) {
                        if (innerItem.ItemType == "TextBoxModel") {
                            var textboxControl = this._renderTextBoxControl(innerItem, false, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            subReportDiv.append(textboxControl);
                        } else if (innerItem.ItemType == "LineModel") {
                            var lineControl = this._renderLineControl(innerItem, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                            subReportDiv.append(lineControl);
                        } else if (innerItem.ItemType == "ImageModel") {
                            this._renderImageControl(innerItem, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "TablixModel") {
                            this._renderTablixControl(innerItem, false, {}, subReportDiv, ctrlId + '_' + innerItem.Name, printPageId);
                        } else if (innerItem.ItemType == "GaugeModel") {
                            this._renderGaugeControl(innerItem, false, {}, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "RectangleModel") {
                            this._renderRectangleControl(innerItem, false, subReportDiv, ctrlId + +'_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "ChartModel") {
                            this._renderChartControl(innerItem, false, {}, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "MapModel") {
                            this._renderMapControl(innerItem, subReportDiv, ctrlId + '_' + innerItem.Name, printMode, printPageId);
                        } else if (innerItem.ItemType == "SubReportModel") {
                            this._renderSubReportControl(innerItem, false, subReportDiv, ctrlId + +'_' + innerItem.Name, printMode, printPageId);
                        }
                    }
                }
            }
        },
        //-------------------- Render the Page ReportItems[End] -------------------------//

        //-------------------- Ajax Web API Call back methods[start] -------------------------//
        doAjaxPost: function (type, url, jsondata, onSuccess) {
            var proxy = this;
            var inVokemethod = onSuccess;
            $.ajax({
                type: type,
                url: url,
                crossDomain: true,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: jsondata,
                beforeSend: function (req) {
                    if (inVokemethod == "_getPageModel" || inVokemethod == "_getPreviewModel") {
                        if (!proxy._isToolbarClick) {
                            proxy._showloadingIndicator(true);
                        } else {
                            proxy._showNavigationIndicator(true);
                        }
                    }
                    req.setRequestHeader('ejAuthenticationToken', proxy._authenticationToken);
                },
                success: function (data) {
                    if (data && typeof (data.Data) != "undefined") {
                        data = data.Data;
                    }
                    if (typeof (data) == "string") {
                        if (data.indexOf("Sf_Exception") != -1) {
                            proxy._renderExcpetion(inVokemethod + ":" + data);
                            return;
                        }
                    }
                    proxy[inVokemethod](data);
                },
                error: function (msg, textStatus, errorThrown) {
                    if (inVokemethod != "_clearCurrentServerCache") {
                        if (msg.readyState == 0) {
                            return;
                        }
                        alert('Exception' + msg.responseText);
                    }
                }
            });
        },

        _reportLoaded: function (reportresult) {
            if (typeof (reportresult) == "object" && reportresult.isReportLoad) {
                if (reportresult) {
                    this._reportParameters = reportresult.parameters;
                    this._reportDataSources = reportresult.dataSources;
                    this._reporDataSets = reportresult.dataSets;
                    this._onReportLoaded();
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");
                }
                else {
                    this._renderExcpetion("Could not load the report");
                }
            } else {
                this._renderExcpetion("Report Loaded:" + reportresult);
            }
        },

        _getDataSourceCredential: function (jsondata) {
            if (ej.isNullOrUndefined(jsondata) || jsondata == "Sf_legacy") {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.updateDataSource }), "_updateDataSource");
            } else {
                this._renderCredentialBlock(jsondata);
                this._showViewerBlock(true);
            }
            this._showloadingIndicator(false);
            this._setContainerSize();
            this._showloadingIndicator(true);
        },

        _validateDSCredential: function (jsondata) {
            if (jsondata == "Sf_legacy") {
                this._showloadingIndicator(false);
                this._showViewerBlock(false);
                this._setContainerSize();
                this._showloadingIndicator(true);
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.updateDSCredential, 'datasoures': this._dataSources }), "_updateDSCredential");
                this._dataSources = null;
            }
            else {
                alert(jsondata);
            }
        },

        _updateDSCredential: function (jsondata) {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.updateDataSource }), "_updateDataSource");
        },

        _getParameters: function (jsondata) {
            if (jsondata != "Sf_legacy") {
                var isCustomBlock = this._renderParameterBlock(jsondata);
                if (!isCustomBlock) {
                    this._showViewerBlock(true);
                    this._showloadingIndicator(false);
                    this._setContainerSize();
                    this._showloadingIndicator(true);
                    var reportParams = this._getParameterJson(jsondata, false);
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.setParameters, 'parameters': reportParams }), "_setParameters");
                } else {
                    this._setContainerSize();
                    this._getDefaultParameters(jsondata);
                }
            }
            else {
                this._showParameterBlock(false);
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
            }
        },

        _setParameters: function (jsondata) {
            var isClick = $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcontent table:first').attr('isviewclick');
            if (typeof (jsondata) == "boolean") {
                if (jsondata == false) {
                    this._showloadingIndicator(true);
                    this._setContainerSize();
                    if (this._isDevice) {
                        this._showViewerBlock(false);
                        this._showViewerPage(true);
                        this._selectparamToolItem(false);
                    }
                    this._currentPage = 1;
                    this._clearPageCache();
                    this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                } else {
                    this._showloadingIndicator(false);
                    if (isClick == 'true') {
                        this._viewReportEnableDisable(false);
                        alert('Please select the parameter values.');
                    }
                }
            }
        },

        _updateParameters: function (jsondata) {
            if (jsondata != "Sf_legacy") {
                this._updateParamElements(jsondata);
            }
        },

        _updateDataSource: function (jsondata) {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getParameters }), "_getParameters");
        },

        _getPageModel: function (jsondata) {
            var jsData = jsondata;
            this._showNavigationIndicator(false);
            this._showloadingIndicator(false);
            this._viewReportEnableDisable(false);
            if (jsData) {
                if (jsData === 'IsDrillDown' || jsData === 'IsSorting') {
                    return;
                }
                if (jsData.isDocumentMap) {
                    this._isDocumentMap = this._isPageDocMap = jsData.isDocumentMap;
                }
                if (jsData.errmsg && jsData.errmsg.length > 0) {
                    var isError = this._onReportError({ errmsg: jsData.errmsg });
                    if (!isError) {
                        if (this._isDevice) {
                            alert(jsData.errmsg);
                        } else {
                            this._renderExceptionBlock(jsData.errmsg);
                            this._showViewerBlock(true);
                        }
                    }
                } else {
                    this._resetExceptionBlock();
                }
                if (jsData.pgmodel && jsData.pgmodel.PageData) {
                    this._pageModel = jsData.pgmodel;
                    if (!this._isToolbarClick) {
                        this._enableToolbarItems();
                    }
                    this._onRenderingBegin();
                    this._gotoPage(this._currentPage);
                    this._pageLayoutPage = this._currentPage;
                    if (this.model.enablePageCache) {
                        this._storePageCache(this._currentPage);
                    }
                    this._onRenderingComplete();
                    this._refresh = false;
                }
                if (jsData.pgsetting) {
                    this._updatePageSetup(jsData.pgsetting);
                }
            }
        },

        _getPreviewModel: function (jsonData) {
            this._showNavigationIndicator(false);
            this._showloadingIndicator(false);
            if (jsonData) {
                if (jsonData.isDocumentMap) {
                    this._isDocumentMap = jsonData.isDocumentMap;
                    this._isPageDocMap = this._isDocumentMap;
                    this._pageDocMapFlag = false;
                }
                if (jsonData.pgmodel && jsonData.pgmodel.PageData) {
                    this._pageModel = jsonData.pgmodel;
                    if (!this._isToolbarClick) {
                        this._selectPreviewToolItem(true);
                        this._enableToolbarItems();
                    }
                    this._gotoPage(this._currentPage);
                    this._refresh = false;
                    this._initialPageSetup(jsonData);
                }
            }
        },

        _getPrintModel: function (jsonData) {
            if (jsonData && jsonData.PageData && jsonData.PageData.length > 0) {
                this._printReport(jsonData);
            }
        },

        _documentMapModel: function (jsondata) {
            if (jsondata && jsondata.indexOf("<ul id=treeViewID>") != -1) {
                this._showDocumentMap(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.DocumentMap);
                this._containerSplit(jsondata);
            }
        },

        _drillThroughModel: function (jsondata) {
            var pageXy = { x: $('#' + this._id + '_viewerContainer').scrollLeft(), y: $('#' + this._id + '_viewerContainer').scrollTop() };
            var reportProperties = this._cloneReportViewerProperties();
            this._parents.push({ pagePos: pageXy, parentPro: reportProperties });
            this._setInitialization();
            this._authenticationToken = reportProperties._childReportAuthentication;
            this._unwireEvents();
            this._initViewer();
            this._reportLoaded(jsondata);
        },

        _gotoParentReportModel: function (jsondata) {
            this._setParentReportViewerProperties();
            var currentpgno = this._currentPage;
            this._unwireEvents();
            this._initViewer();
            this._currentPage = currentpgno;
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getParameters }), "_getParameters");
        },

        _clearCurrentServerCache: function (jsonData) {
        },
        //-------------------- Ajax Web API Call back methods[end] -------------------------//

        //-------------------- Print Actions[Start] -------------------------//
        _print: function () {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPrintModel }), "_getPrintModel");
        },

        _printReport: function (printModel) {
            var browserInfo = ej.browserInfo();
            var reportModel = printModel;

            var pageHeight = reportModel.PaperHeight;
            var pageWidth = reportModel.PaperWidth;
            var headerTop = reportModel.MarginTop;
            var contentLeft = reportModel.MarginLeft;
            var links = $('head').find('link');
            var isIe9 = (browserInfo.name == 'msie' && (parseInt(browserInfo.version) <= 9.0));
            var printWind = window.open('', 'pint', "tabbar=no,scrollbars = yes,resizable = yes");
            var pageStyle = '<style>h1 {display:block;height:0px;page-break-after:always;}@page { size:' + pageHeight + 'px ' + pageWidth + 'px;}@media print{ #header, #footer { display:none } }</style>';

            if (printWind != null) {
                printWind.document.write('<!DOCTYPE html>');

                if (browserInfo.name === "msie") {
                    var a = ""
                    links.each(function (index, obj) {
                        $(obj).attr('href', obj.href);
                        a += obj.outerHTML;
                    });
                    printWind.document.write('<html><head>' + pageStyle + '</head><body style="width:' + pageWidth + 'px">');
                }
                else {
                    var a = ""
                    printWind.document.write('<html><head>' + pageStyle);
                    links.each(function (index, obj) {
                        $(obj).attr('href', obj.href);
                        a += obj.outerHTML;
                    });
                    printWind.document.writeln(a + '</head><body>');
                }

                for (var i = 0; i < reportModel.TotalPages; i++) {
                    var pagdata = reportModel.PageData[i];
                    printWind.document.write('<div style="position:relative;height:' + pageHeight + 'px;width:' + pageWidth + 'px">');
                    var printContent = this._renderPrintContainer(pagdata, headerTop, contentLeft, i);
                    printWind.document.write(printContent);
                    printWind.document.write('</div>');

                    if (!isIe9) {
                        printWind.document.write('<h1></h1>');
                    }
                    else if (isIe9 && (i < reportModel.TotalPages)) {
                        printWind.document.write('<h1></h1>');
                    }
                }

                printWind.document.write('</body></html>');
                printWind.document.close();
                printWind.focus();
                setTimeout(function () {
                    if (!ej.isNullOrUndefined(printWind.window)) {
                        printWind.print();
                        //setTimeout(function () { printWind.close() }, 100);
                    }
                }, 200);
            }
        },

        _renderPrintContainer: function (pagedata, top, left, index) {
            var oDiv = document.createElement("div");
            document.body.appendChild(oDiv);
            $(oDiv).css({ 'overflow': 'auto', 'width': '1px', 'height': '1px' });
            //$(oDiv).hide();

            var $printviewheaderContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent', 'width': pagedata.Width + 'px', 'height': pagedata.HeaderHeight + 'px' }, {});
            var $printviewheaderborder = ej.buildTag("div.pageHeaderBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF', 'left': left, 'top': top });
            $printviewheaderborder.append($printviewheaderContainer);
            $(oDiv).append($printviewheaderborder);

            var $printviewbodyContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent', 'width': pagedata.Width + 'px', 'height': pagedata.Height + 'px' }, {});
            var $printviewbodyborder = ej.buildTag("div.pageBodyBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF', 'left': left, 'top': top });
            $printviewbodyborder.append($printviewbodyContainer);
            $(oDiv).append($printviewbodyborder);

            var $printviewfooterContainer = ej.buildTag("div", "", { 'position': 'relative', 'background-color': 'transparent', 'width': pagedata.Width + 'px', 'height': pagedata.FooterHeight + 'px' }, {});
            var $printviewfooterborder = ej.buildTag("div.pageFooterBorder", "", { 'position': 'relative', 'background-color': '#FFFFFF', 'left': left, 'top': top });
            $printviewfooterborder.append($printviewfooterContainer);
            $(oDiv).append($printviewfooterborder);

            if (pagedata.HeaderStyleModel) {
                this._applyPageStyle($printviewheaderborder, pagedata.HeaderStyleModel, true);
            }

            if (pagedata.FooterStyleModel) {
                this._applyPageStyle($printviewfooterborder, pagedata.FooterStyleModel, true);
            }

            if (pagedata.BodyStyleModel) {
                this._applyPageStyle($printviewbodyborder, pagedata.BodyStyleModel, true);
            }

            if (pagedata.HeaderModel && pagedata.HeaderModel.length > 0) {
                this._renderPageControls(pagedata.HeaderModel, $printviewheaderContainer, true, 'Print' + index);
            }

            if (pagedata.FooterModel && pagedata.FooterModel.length > 0) {
                this._renderPageControls(pagedata.FooterModel, $printviewfooterContainer, true, 'Print' + index);
            }

            if (pagedata.PageModel && pagedata.PageModel.length > 0) {
                this._renderPageControls(pagedata.PageModel, $printviewbodyContainer, true, 'Print' + index);
            }

            var htmlele = oDiv.innerHTML.replace(/PrintImage/g, 'GetResource');
            document.body.removeChild(oDiv);
            return htmlele;
        },

        _showPrintButton: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbar_Print').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbar_Print').parent().css("display", "none");
            }
        },
        //-------------------- Print Actions[end] -------------------------//

        //-------------------- Export Actions[Start] -------------------------//    
        _exportMenuClick: function (event) {
            var exportType;
            var exportformat = $(event.target);
            if (exportformat) {
                if (exportformat.hasClass('e-pdf')) {
                    exportType = 'PDF';
                } else if (exportformat.hasClass('e-word')) {
                    exportType = 'Word_' + this.model.exportSettings.wordFormat;
                } else if (exportformat.hasClass('e-excel')) {
                    exportType = 'Excel_' + this.model.exportSettings.excelFormat;
                } else if (exportformat.hasClass('e-html')) {
                    exportType = 'Html';
                }
                this._exportReport(exportType);
            }
        },

        _exportReport: function (exportType) {
            var _exportFormObj = null;
            if (exportType && exportType.length > 0) {
                var requrl = this.model.reportServiceUrl + '/PostReportAction';
                if ($('#' + this._id + '_exportForm').length > 0) {
                    var _exportForm = $('#' + this._id + '_exportForm');
                    _exportForm.attr("action", requrl);
                    $('#' + this._id + "_exportKey").val(this._authenticationToken);
                    $('#' + this._id + "_exportRes").val(exportType);
                    _exportFormObj = { exportAction: _exportForm };
                    this._onReportExport(_exportFormObj);
                    _exportForm.submit();
                } else {
                    var exportForm = document.createElement("form");
                    $(exportForm).attr({ "id": this._id + "_exportForm", "method": "post", "action": requrl, "data-ajax": "false" });

                    var exportKey = document.createElement("input");
                    $(exportKey).attr({ "type": "hidden", "id": this._id + "_exportKey", "name": "controlID", "value": this._authenticationToken });

                    var exportRes = document.createElement("input");
                    $(exportRes).attr({ "type": "hidden", "id": this._id + "_exportRes", "name": "resourcetype", "value": exportType });

                    var exportPrint = document.createElement("input");
                    $(exportPrint).attr({ "type": "hidden", "id": this._id + "_exportPrint", "name": "isPrint", "value": false });

                    var exportName = document.createElement("input");
                    $(exportName).attr({ "type": "hidden", "id": this._id + "_exportfileName", "name": "reportName", "value": "" });

                    $(exportForm).append(exportKey);
                    $(exportForm).append(exportRes);
                    $(exportForm).append(exportPrint);
                    $(exportForm).append(exportName);
                    $('body').append(exportForm);
                    $(exportForm).hide();
                    _exportFormObj = { exportAction: exportForm };
                    this._onReportExport(_exportFormObj);
                    $(exportForm).submit();
                }
            }
            $('#' + this._id + '_toolbar_exportListTip').hide();
        },

        _showExportList: function () {
            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Html) {
                $('#' + this._id + '_html').css('display', 'block');
            } else {
                $('#' + this._id + '_html').css('display', 'none');
            }

            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Word) {
                $('#' + this._id + '_word').css('display', 'block');
            } else {
                $('#' + this._id + '_word').css('display', 'none');
            }

            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Excel) {
                $('#' + this._id + '_xls').css('display', 'block');
            } else {
                $('#' + this._id + '_xls').css('display', 'none');
            }

            if (this.model.exportSettings.exportOptions & ej.ReportViewer.ExportOptions.Pdf) {
                $('#' + this._id + '_pdf').css('display', 'block');
            } else {
                $('#' + this._id + '_pdf').css('display', 'none');
            }
        },

        _showExportControls: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-export').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-export').parent().css("display", "none");
            }
        },

        _showTooltip: function () {
            var exportTooltip = $('#' + this._id + '_toolbar_exportListTip');
            if (!(exportTooltip.length > 0)) {
                exportTooltip = this._renderToolTipExport();
                $('body').append(exportTooltip);
                this._showExportList();
                this._on($('#' + this._id + '_toolbar_exportListTip .e-reportviewer-popupli'), "click", this._exportMenuClick);
            }

            if (exportTooltip.css('display') == 'block') {
                exportTooltip.css('display', 'none');
            } else {
                exportTooltip.css('display', 'block');
            }

            var exportTagHeight = $('#' + this._id + '_toolbarContainer .e-reportviewer-export').height();
            var expoTagPos = $($('#' + this._id + '_toolbarContainer .e-reportviewer-export')[0]).offset();
            exportTooltip.css({ 'top': (expoTagPos.top + exportTagHeight) - 3, 'left': (expoTagPos.left - 5) });
        },

        //-------------------- Export Actions[End] -------------------------//

        //-------------------- Page Navigation Actions[Start] -------------------------//
        _updatePageNavigation: function (pageNo, totalPage) {
            if (pageNo > 1 && pageNo < totalPage) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').removeClass('e-reportviewer-disabled');
            } else if (pageNo == 1 && pageNo < totalPage) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').removeClass('e-reportviewer-disabled');
            } else if (pageNo == totalPage && totalPage != 1) {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').removeClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').addClass('e-reportviewer-disabled');
            } else {
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').addClass('e-reportviewer-disabled');
                $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').addClass('e-reportviewer-disabled');
            }
        },

        _gotoRenderPage: function () {
            this._onRenderingBegin();
            this._pageModel = this._retrievePageCache(this._currentPage);
            this._gotoPage(this._currentPage);
            this._pageLayoutPage = this._currentPage;
            this._onRenderingComplete();
            this._refresh = false;
        },

        _gotoFirstPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._pageModel.TotalPages > 0) {
                    this._currentPage = 1;
                    this._refresh = false;
                    if (this.model.enablePageCache && this._pageContains(this._currentPage)) {
                        this._gotoRenderPage();
                    } else {
                        this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                    }
                }
            }
        },

        _gotoLastPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._pageModel.TotalPages > 0) {
                    this._currentPage = this._pageModel.TotalPages;
                    this._refresh = false;
                    if (this.model.enablePageCache && this._pageContains(this._currentPage)) {
                        this._gotoRenderPage();
                    } else {
                        this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                    }
                }
            }
        },

        _gotoNextPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._currentPage < this._pageModel.TotalPages) {
                    this._currentPage = this._currentPage + 1;
                    this._refresh = false;
                    if (this.model.enablePageCache && this._pageContains(this._currentPage)) {
                        this._gotoRenderPage();
                    } else {
                        this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                    }
                }
            }
        },

        _gotoPreviousPage: function () {
            if (this._pageModel && this._pageModel.TotalPages) {
                if (this._currentPage > 1) {
                    this._currentPage = this._currentPage - 1;
                    this._refresh = false;
                    if (this.model.enablePageCache && this._pageContains(this._currentPage)) {
                        this._gotoRenderPage();
                    } else {
                        this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                    }
                }
            }
        },

        _gotoPageNo: function (pageNo) {
            if (this._pageModel && this._pageModel.TotalPages) {
                var pagenumber = parseInt(pageNo);
                if (pagenumber >= 1 && pagenumber <= this._pageModel.TotalPages && this._currentPage != pagenumber) {
                    this._currentPage = pagenumber;
                    this._refresh = false;
                    if (this.model.enablePageCache && this._pageContains(this._currentPage)) {
                        this._gotoRenderPage();
                    } else {
                        this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
                    }
                }
            }
        },

        _gotoPage: function (pageNo) {
            if (this._pageModel && this._pageModel.PageData && this._pageModel.PageData.length > 0) {
                var pagenumber = parseInt(pageNo);
                this._updatePageNavigation(pagenumber, this._pageModel.TotalPages);
                if (this._pageModel.TotalPages > 0 && this._pageModel.TotalPages) {
                    $('#' + this._id + '_toolbarContainer span.e-reportviewer-labelpageno').html(" / " + this._pageModel.TotalPages);
                    $('#' + this._id + '_txtpageNo').val(pagenumber);
                }

                var _ejObjects = $('#' + this._id + '_viewerContainer').find('.e-js');
                if (_ejObjects && _ejObjects.length > 0) {
                    this._destroyEJObjects(_ejObjects);
                }
                _ejObjects = null;

                var pageData = this._pageModel.PageData[0];
                this._renderPageModels(pageData);
                if (pageData) {
                    this._setPageSize(pageData.Height, pageData.Width, pageData.HeaderHeight, pageData.FooterHeight);
                }
            }
        },

        _onkeyPress: function (event) {
            if (event.keyCode == 13) {
                try {
                    var val = parseInt($(event.currentTarget).val());
                    this._gotoPageNo(val);
                } catch (err) {
                }
            }
        },

        _showPageNavigationControls: function (enable) {
            if (enable) {
                $('#' + this._id + '_txtpageNo').parents(".e-reportviewer-toolbarul").css("display", "block");
            } else {
                $('#' + this._id + '_txtpageNo').parents(".e-reportviewer-toolbarul").css("display", "none");
            }
        },
        //-------------------- Page Navigation Actions[End] -------------------------//

        //-------------------- Print Page Setup [Start] -----------------------------//

        _showPrintPageSetup: function () {
            var pageSetup = $('#' + this._id + '_printPageSetup');
            var eDialog;
            if (!(pageSetup.length > 0)) {
                pageSetup = this._renderPrintPageSetup();
                this._bindPageSetupModel();
                $('body').append(pageSetup);
            } else {
                eDialog = $('#' + this._id + '_printPageSetup').data('ejDialog');
            }

            if (eDialog && eDialog.model) {
                var _isopen = eDialog.isOpen();
                if (_isopen) {
                    $('#' + this._id + '_printPageSetup').ejDialog("close");
                } else {
                    $('#' + this._id + '_printPageSetup').ejDialog("open");
                    this._bindPageSetupModel();
                }
            }
        },

        _updatePageSetup: function (_pageSetting) {
            if (!ej.isNullOrUndefined(_pageSetting)) {
                this._paperSetup.paperHeight = (_pageSetting.PaperHeight / 96).toFixed(2);
                this._paperSetup.paperWidth = (_pageSetting.PaperWidth / 96).toFixed(2);
                this._paperSetup.MarginTop = (_pageSetting.MarginTop / 96).toFixed(2);
                this._paperSetup.MarginRight = (_pageSetting.MarginRight / 96).toFixed(2);
                this._paperSetup.MarginBottom = (_pageSetting.MarginBottom / 96).toFixed(2);
                this._paperSetup.MarginLeft = (_pageSetting.MarginLeft / 96).toFixed(2);
                this._paperOrientation = this._paperSetup.paperHeight < this._paperSetup.paperWidth ? "Landscape" : "Portrait";
                this._paperName = this._getPaperName(this._paperSetup.paperWidth, this._paperSetup.paperHeight);
            }
        },

        _initialPageSetup: function (jsonData) {
            if (!ej.isNullOrUndefined(jsonData.pgmodel.PaperHeight)) {
                this._paperSetup.paperHeight = (jsonData.pgmodel.PaperHeight / 96).toFixed(2);
            }
            if (!ej.isNullOrUndefined(jsonData.pgmodel.PaperWidth)) {
                this._paperSetup.paperWidth = (jsonData.pgmodel.PaperWidth / 96).toFixed(2);
            }
            if (!ej.isNullOrUndefined(jsonData.pgmodel.MarginTop)) {
                this._paperSetup.MarginTop = (jsonData.pgmodel.MarginTop / 96).toFixed(2);
            }
            if (!ej.isNullOrUndefined(jsonData.pgmodel.MarginRight)) {
                this._paperSetup.MarginRight = (jsonData.pgmodel.MarginRight / 96).toFixed(2);
            }
            if (!ej.isNullOrUndefined(jsonData.pgmodel.MarginBottom)) {
                this._paperSetup.MarginBottom = (jsonData.pgmodel.MarginBottom / 96).toFixed(2);
            }
            if (!ej.isNullOrUndefined(jsonData.pgmodel.MarginLeft)) {
                this._paperSetup.MarginLeft = (jsonData.pgmodel.MarginLeft / 96).toFixed(2);
            }
        },

        _bindPageSetupModel: function () {
            var heightNum = $('#' + this._id + '_paperHeight').data('ejNumericTextbox');
            var widthNum = $('#' + this._id + '_paperWidth').data('ejNumericTextbox');
            var marginLeft = $('#' + this._id + '_paperMarginLeft').data('ejNumericTextbox');
            var marginTop = $('#' + this._id + '_paperMarginTop').data('ejNumericTextbox');
            var marginRight = $('#' + this._id + '_paperMarginRight').data('ejNumericTextbox');
            var marginBottom = $('#' + this._id + '_paperMarginBottom').data('ejNumericTextbox');
            if (this._paperName) {
                var paperDDL = $('#' + this._id + '_PaperSize').data("ejDropDownList");
                paperDDL.selectItemByText(this._paperName);

                var landscape = $('#' + this._id + '_landscape').data('ejRadioButton');
                var portrait = $('#' + this._id + '_portrait').data('ejRadioButton');

                if (this._paperName == "Custom") {
                    if (landscape.model.checked == true) {
                        heightNum.option('value', this._paperSetup.paperWidth ? this._paperSetup.paperWidth : 1);
                        widthNum.option('value', this._paperSetup.paperHeight ? this._paperSetup.paperHeight : 1);
                    } else {
                        heightNum.option('value', this._paperSetup.paperHeight ? this._paperSetup.paperHeight : 1);
                        widthNum.option('value', this._paperSetup.paperWidth ? this._paperSetup.paperWidth : 1);
                    }
                } else {
                    var dimension = this._getPaperSize(this._paperName);
                    if (this._paperOrientation == "Landscape") {
                        heightNum.option('value', dimension.width.toFixed(2));
                        widthNum.option('value', dimension.height.toFixed(2));
                        landscape.model.checked = true;
                    } else {
                        heightNum.option('value', dimension.height.toFixed(2));
                        widthNum.option('value', dimension.width.toFixed(2));
                        portrait.model.checked = true;
                    }
                }
                marginTop.option('value', this._paperSetup.MarginTop);
                marginRight.option('value', this._paperSetup.MarginRight);
                marginBottom.option('value', this._paperSetup.MarginBottom);
                marginLeft.option('value', this._paperSetup.MarginLeft);
            }
            else {
                var jsonheight = this._paperSetup.paperHeight;
                var jsonwidth = this._paperSetup.paperWidth;
                marginTop.option('value', this._paperSetup.MarginTop);
                marginRight.option('value', this._paperSetup.MarginRight);
                marginBottom.option('value', this._paperSetup.MarginBottom);
                marginLeft.option('value', this._paperSetup.MarginLeft);

                heightNum.option('value', jsonheight ? jsonheight : 1);
                widthNum.option('value', jsonwidth ? jsonwidth : 1);

                this._paperName = "Custom";
            }
        },

        _orientationChanged: function (event) {
            var heightNum = $('#' + this._id + '_paperHeight').data('ejNumericTextbox');
            var widthNum = $('#' + this._id + '_paperWidth').data('ejNumericTextbox');
            var marginLeft = $('#' + this._id + '_paperMarginLeft').data('ejNumericTextbox');
            var marginTop = $('#' + this._id + '_paperMarginTop').data('ejNumericTextbox');
            var marginRight = $('#' + this._id + '_paperMarginRight').data('ejNumericTextbox');
            var marginBottom = $('#' + this._id + '_paperMarginBottom').data('ejNumericTextbox');
            if (heightNum && widthNum) {
                var height = heightNum.getValue();
                var width = widthNum.getValue();
                heightNum.option('value', width);
                widthNum.option('value', height);

                if (event.model.id == this._id + '_landscape' && event.model.checked == true) {
                    var temp1 = marginLeft.getValue();
                    marginLeft.option('value', marginTop.getValue());
                    var temp2 = marginBottom.getValue();
                    marginBottom.option('value', temp1);
                    temp1 = marginRight.getValue();
                    marginRight.option('value', temp2);
                    marginTop.option('value', temp1);
                }
                else {
                    var temp1 = marginRight.getValue();
                    marginRight.option('value', marginTop.getValue());
                    var temp2 = marginBottom.getValue();
                    marginBottom.option('value', temp1);
                    temp1 = marginLeft.getValue();
                    marginLeft.option('value', temp2);
                    marginTop.option('value', temp1);
                }
            }
        },

        _setUpChange: function (event) {
            var paperType = event.value;
            var initVal = 0;
            var heightNum = $('#' + this._id + '_paperHeight').data('ejNumericTextbox');
            var widthNum = $('#' + this._id + '_paperWidth').data('ejNumericTextbox');
            var marginLeft = $('#' + this._id + '_paperMarginLeft').data('ejNumericTextbox');
            var marginTop = $('#' + this._id + '_paperMarginTop').data('ejNumericTextbox');
            var marginRight = $('#' + this._id + '_paperMarginRight').data('ejNumericTextbox');
            var marginBottom = $('#' + this._id + '_paperMarginBottom').data('ejNumericTextbox');
            var paperSize = this._getPaperSize(paperType);
            heightNum.option('value', paperSize.height);
            widthNum.option('value', paperSize.width);
            if (!marginTop.getValue()) {
                marginTop.option('value', initVal.toFixed(2));
            }
            if (!marginRight.getValue()) {
                marginRight.option('value', initVal.toFixed(2));
            }
            if (!marginBottom.getValue()) {
                marginBottom.option('value', initVal.toFixed(2));
            }
            if (!marginLeft.getValue()) {
                marginLeft.option('value', initVal.toFixed(2));
            }
            if (paperType == "Custom") {
                heightNum.enable();
                widthNum.enable();
            } else {
                heightNum.disable();
                widthNum.disable();
            }
        },

        _getPaperName: function (width, height) {
            if (width == 11.70 && height == 16.50) {
                return "A3";
            } else if (width == 8.27 && height == 11.69) {
                return "A4";
            } else if (width == 10.12 && height == 14.33) {
                return "B4(JIS)";
            } else if (width == 7.17 && height == 10.12) {
                return "B5(JIS)";
            } else if (width == 4.13 && height == 9.50) {
                return "Envelope #10";
            } else if (width == 3.88 && height == 7.50) {
                return "Envelope Monarch";
            } else if (width == 7.25 && height == 10.50) {
                return "Executive";
            } else if (width == 8.50 && height == 14.00) {
                return "Legal";
            } else if (width == 8.50 && height == 11.00) {
                return "Letter";
            } else if (width == 11.00 && height == 17.00) {
                return "Tabloid";
            } else {
                return "Custom";
            }
        },

        _getPaperSize: function (paperType) {
            var paperSize = {};
            var height;
            var width;
            switch (paperType) {
                case "A3":
                    width = 11.70;
                    height = 16.50;
                    break;
                case "A4":
                    width = 8.27;
                    height = 11.69;
                    break;
                case "B4(JIS)":
                    width = 10.12;
                    height = 14.33;
                    break;
                case "B5(JIS)":
                    width = 7.17;
                    height = 10.12;
                    break;
                case "Envelope #10":
                    width = 4.13;
                    height = 9.50;
                    break;
                case "Envelope Monarch":
                    width = 3.88;
                    height = 7.50;
                    break;
                case "Executive":
                    width = 7.25;
                    height = 10.50;
                    break;
                case "Legal":
                    width = 8.50;
                    height = 14.00;
                    break;
                case "Letter":
                    width = 8.50;
                    height = 11.00;
                    break;
                case "Tabloid":
                    width = 11.00;
                    height = 17.00;
                    break;
                case "Custom":
                    height = $('#' + this._id + '_paperHeight').data('ejNumericTextbox').getValue();
                    width = $('#' + this._id + '_paperWidth').data('ejNumericTextbox').getValue();
                    break;
            }
            paperSize.height = height;
            paperSize.width = width;
            return paperSize;
        },

        _pageSetupSubmit: function () {
            this._paperSetup.paperHeight = ($('#' + this._id + '_paperHeight').data('ejNumericTextbox').getValue() ? $('#' + this._id + '_paperHeight').data('ejNumericTextbox').getValue() : null);
            this._paperSetup.paperWidth = ($('#' + this._id + '_paperWidth').data('ejNumericTextbox').getValue() ? $('#' + this._id + '_paperWidth').data('ejNumericTextbox').getValue() : null);
            this._paperSetup.MarginTop = ($('#' + this._id + '_paperMarginTop').data('ejNumericTextbox').getValue() ? $('#' + this._id + '_paperMarginTop').data('ejNumericTextbox').getValue() : null);
            this._paperSetup.MarginRight = ($('#' + this._id + '_paperMarginRight').data('ejNumericTextbox').getValue() ? $('#' + this._id + '_paperMarginRight').data('ejNumericTextbox').getValue() : null);
            this._paperSetup.MarginBottom = ($('#' + this._id + '_paperMarginBottom').data('ejNumericTextbox').getValue() ? $('#' + this._id + '_paperMarginBottom').data('ejNumericTextbox').getValue() : null);
            this._paperSetup.MarginLeft = ($('#' + this._id + '_paperMarginLeft').data('ejNumericTextbox').getValue() ? $('#' + this._id + '_paperMarginLeft').data('ejNumericTextbox').getValue() : null);
            var paperSizeDDl = $('#' + this._id + '_PaperSize').data('ejDropDownList');
            this._paperName = paperSizeDDl.model.value;

            if ($('#' + this._id + '_portrait').attr('checked') == 'checked') {
                this._paperOrientation = "Portrait";
            } else {
                this._paperOrientation = "Landscape";
            }

            this._pageSetup = true;

            if ($('#' + this._id + '_ejtb_preview').hasClass('e-active')) {
                if (!ej.isNullOrUndefined(this._paperSetup.paperWidth) && !ej.isNullOrUndefined(this._paperSetup.paperHeight)) {
                    this._updatePreviewLayout(!this._id._printMode, true);
                }
            }

            $('#' + this._id + '_printPageSetup').ejDialog('close');
        },

        _pageSetupCancel: function (events) {
            var ejDialog = $('#' + this._id + '_printPageSetup').ejDialog('close');
        },

        //-------------------- Print Page Setup [End] -------------------------------//


        //-------------------- FitToPage[start] -------------------------//
        _showFittoPage: function (enable) {
            if (enable) {
                $('#' + this._id + '_ejtb_fittopage').parents("li.e-reportviewer-toolbarli").css("display", "block");
            } else {
                $('#' + this._id + '_ejtb_fittopage').parents("li.e-reportviewer-toolbarli").css("display", "none");
            }
        },

        _resetPage: function () {
            var pageContainerWidth = null, pageContainerHeight = null;
            var browserInfo = ej.browserInfo();

            var pagecontainer = $('#' + this._id + '_pageviewContainer');
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            pageViewline.css({ "width": pagecontainer.width(), "height": pagecontainer.height() });
            this._applyPageTransform(pagecontainer,this._preZoomVal, this._preZoomVal);

            if (browserInfo.name == 'msie' && browserInfo.version == 8) {
                pageContainerWidth = pagecontainer[0].getBoundingClientRect().right;
                pageContainerHeight = pagecontainer[0].getBoundingClientRect().bottom;
            }
            else {
                pageContainerWidth = pagecontainer[0].getBoundingClientRect().width;
                pageContainerHeight = pagecontainer[0].getBoundingClientRect().height;
            }
            pageViewline.css({ "width": pageContainerWidth, "height": pageContainerHeight });
        },

        _fitToPage: function (fitType) {
            var pageContainerWidth = null, pageContainerHeight = null;
            var browserInfo = ej.browserInfo();
            this._resetPage();
            var pageViewerContainer = $('#' + this._id + '_viewerContainer');
            var pagecontainer = $('#' + this._id + '_pageviewContainer');
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            var w = pageViewerContainer.width();
            var h = pageViewerContainer.height();
            var pageW = pageViewline.width();
            var pageH = pageViewline.height();
            var scaleX = 1;
            var scaleY = 1;

            if (browserInfo.name == 'msie' && browserInfo.version == 8) {
                pageContainerWidth = pagecontainer[0].getBoundingClientRect().right;
                pageContainerHeight = pagecontainer[0].getBoundingClientRect().bottom;
            }
            else {
                pageContainerWidth = pagecontainer[0].getBoundingClientRect().width;
                pageContainerHeight = pagecontainer[0].getBoundingClientRect().height;
            }

            if (fitType == "fitToPage") {
                scaleX = (w - 40) / pageW * this._preZoomVal;
                scaleY = (h - 40) / pageH * this._preZoomVal;
                if (h > pageH) {
                    scaleY = this._preZoomVal;
                }

                if (w > pageW) {
                    scaleX = this._preZoomVal;
                }

                this._applyPageTransform(pagecontainer,scaleX , scaleY , fitType);
                pageViewline.css({ "width": pageContainerWidth, "height": pageContainerHeight });
            } else if (fitType == "wholepage") {
                if (h < pageH) {
                    scaleY = (h - 40) / pageH * this._preZoomVal;
                    this._applyPageTransform(pagecontainer,this._preZoomVal , scaleY , fitType);
                    scaleX = this._preZoomVal;
                    pageViewline.css({ "width": pageContainerWidth, "height": pageContainerHeight });
                }
            } else if (fitType == "fitToWidth") {
                if (w < pageW) {
                    scaleX = (w - 40) / pageW * this._preZoomVal;
                     this._applyPageTransform(pagecontainer, scaleX,this._preZoomVal, fitType);
                    scaleY = this._preZoomVal;
                    pageViewline.css({ "width": pageContainerWidth, "height": pageContainerHeight });
                }
            }
            return { scalX: scaleX, scalY: scaleY }
        },

        _selectDropDownIndex: function (index) {
            var _selectedObj = $($('#' + this._id + '_toolbar_zoomSelection_popup li')[index]);
            _selectedObj.addClass('e-active');
            $('#' + this._id + '_toolbar_zoomSelection_input').val(_selectedObj.text());
        },

        _applyDropDownVal: function (scaleXY, orientationX, isFittoPage) {
            if (isFittoPage) {
                var _maxScale = scaleXY.scalX > scaleXY.scalY ? scaleXY.scalY : scaleXY.scalX;
                $('#' + this._id + '_toolbar_zoomSelection_input').val(Math.round((_maxScale * 100)) + "%");
                $('#' + this._id + '_toolbar_zoomSelection_popup li').removeClass('e-active');
            }
            else if (orientationX) {
                $('#' + this._id + '_toolbar_zoomSelection_input').val(Math.round((scaleXY.scalX * 100)) + "%");
                $('#' + this._id + '_toolbar_zoomSelection_popup li').removeClass('e-active');
            } else {
                $('#' + this._id + '_toolbar_zoomSelection_input').val(Math.round((scaleXY.scalY * 100)) + "%");
                $('#' + this._id + '_toolbar_zoomSelection_popup li').removeClass('e-active');
            }
        },

        _pageFitMenuClick: function (event) {
            var pagefitTxt;
            if ($(event.target).hasClass('e-reportviewer-icon')) {
                pagefitTxt = $(event.target).attr('fitType');
            } else {
                pagefitTxt = (event.target).className;
            }
            if (pagefitTxt) {
                var _anchorWidth = $('#' + this._id + '_pageWidth');
                var spanWidth = _anchorWidth.find('span.e-reportviewer-icon');
                var _anchorHeight = $('#' + this._id + '_pageHeight');
                var spanHeight = _anchorHeight.find('span.e-reportviewer-icon');
                var pageViewerContainer = $('#' + this._id + '_viewerContainer');
                var pageViewline = $('#' + this._id + '_pageviewOuterline');
                var viewContainer = $('#' + this._id + '_pageviewContainer')
                var w = pageViewerContainer.width();
                var h = pageViewerContainer.height();
                var pageW = pageViewline.width();
                var pageH = pageViewline.height();
                var previousPageWidth = viewContainer.width();
                var zoomddl = $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList');
                var _scaleXY;
                var _pageFitType = pagefitTxt;

                switch (_pageFitType) {
                    case "FITTOPAGE":
                        _scaleXY = this._fitToPage("fitToPage");
                        this._applyDropDownVal(_scaleXY, false, true);
                        break;
                    case "PAGEWIDTH":
                    case 'e-reportviewer-popupli e-fitpagewidth':
                        if (_anchorWidth.attr('isSelect') == 'true') {
                            if (_anchorHeight.attr('isSelect') == 'true') {
                                _scaleXY = this._fitToPage("fitToPage");
                                this._applyDropDownVal(_scaleXY, false, true);
                            } else {
                                this._resetPage();
                                this._selectDropDownIndex(this._zoomLevel);
                            }
                            spanWidth.removeClass('e-reportviewer-pagefittopage');
                            spanWidth.addClass('e-reportviewer-emptyconetent');
                            _anchorWidth.attr('isSelect', 'false');
                        } else {
                            if (w > pageW) {
                                if (_anchorHeight.attr('isSelect') == 'true' && pageW != previousPageWidth) {
                                    _scaleXY = this._fitToPage("fitToWidth");
                                    this._applyDropDownVal(_scaleXY, true, false);
                                    _anchorWidth.attr('isSelect', 'true');
                                    spanWidth.addClass('e-reportviewer-pagefittopage');
                                    spanWidth.removeClass('e-reportviewer-emptyconetent');
                                    spanHeight.addClass('e-reportviewer-emptyconetent');
                                    spanHeight.removeClass('e-reportviewer-pagefittopage');
                                    _anchorHeight.attr('isSelect', 'false');
                                }
                                return;
                            }
                            if (_anchorHeight.attr('isSelect') == 'true') {
                                _scaleXY = this._fitToPage("fitToPage");
                                this._applyDropDownVal(_scaleXY, false, true);
                            } else {
                                _scaleXY = this._fitToPage("fitToWidth");
                                this._applyDropDownVal(_scaleXY, true, false);
                            }
                            _anchorWidth.attr('isSelect', 'true');
                            spanWidth.addClass('e-reportviewer-pagefittopage');
                            spanWidth.removeClass('e-reportviewer-emptyconetent');
                        }
                        break;
                    case "WHOLEPAGE":
                    case 'e-reportviewer-popupli e-fitpageheight':
                        if (_anchorHeight.attr('isSelect') == 'true') {
                            if (_anchorWidth.attr('isSelect') == 'true') {
                                _scaleXY = this._fitToPage("fitToWidth");
                                this._applyDropDownVal(_scaleXY, true, false);
                            } else {
                                this._resetPage();
                                this._selectDropDownIndex(this._zoomLevel);
                            }
                            spanHeight.addClass('e-reportviewer-emptyconetent');
                            spanHeight.removeClass('e-reportviewer-pagefittopage');
                            _anchorHeight.attr('isSelect', 'false');
                        } else {
                            if (h > pageH) {
                                return;
                            }
                            _scaleXY = this._fitToPage("fitToPage");
                            this._applyDropDownVal(_scaleXY, false, true);
                            spanHeight.addClass('e-reportviewer-pagefittopage');
                            spanHeight.removeClass('e-reportviewer-emptyconetent');
                            _anchorHeight.attr('isSelect', 'true');
                            spanWidth.removeClass('e-reportviewer-pagefittopage');
                            spanWidth.addClass('e-reportviewer-emptyconetent');
                            _anchorWidth.attr('isSelect', 'false');
                        }
                        break;
                }
                $('#' + this._id + '_toolbar_fittoPagePopup').hide();
            }
        },

        _showFitToPagetip: function () {
            var fittoPage = $('#' + this._id + '_toolbar_fittoPagePopup');
            if (!(fittoPage.length > 0)) {
                fittoPage = this._renderPageFitPopup();
                $('body').append(fittoPage);
                this._on($('#' + this._id + '_toolbar_fittoPagePopup li.e-reportviewer-popupli'), "click", this._pageFitMenuClick);
            }

            if (fittoPage.css('display') == 'block') {
                fittoPage.css('display', 'none');
            } else {
                fittoPage.css('display', 'block');
                this._renderFitopagePopup();
            }

            var exportTagHeight = $('#' + this._id + '_toolbarContainer .e-reportviewer-pagefit').height();
            var expTagPos = $($('#' + this._id + '_toolbarContainer .e-reportviewer-pagefit')[0]).offset();
            fittoPage.css({ 'top': (expTagPos.top + exportTagHeight) - 3, 'left': (expTagPos.left - 5) });
        },
        //-------------------- FitToPage[end] -------------------------//

        //-------------------- Page Zoom[start] -------------------------//
        _zoomValChange: function (sender) {
            var ejViewer = $(this.element).closest(".e-reportviewer");
            var ejViewerInstance = ejViewer.ejReportViewer("instance");
            var zoomVal = parseInt(sender.value) / 100;
            ejViewerInstance._zoomLevel = sender.itemId;
            ejViewerInstance._zoomContainer(zoomVal, false);
        },

        _zoomIn: function () {
            if (this._zoomLevel >= 6) {
                this._zoomLevel = 6;
            } else {
                this._zoomLevel++;
            }
            if (this._isDevice) {
                this._mobileLayoutZoomChange();
            } else {
                var zoomddl = $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList');
                zoomddl.selectItemsByIndices(this._zoomLevel);
            }
        },

        _zoomOut: function () {
            if (this._zoomLevel <= 0) {
                this._zoomLevel = 0;
            } else {
                this._zoomLevel--;
            }
            if (this._isDevice) {
                this._mobileLayoutZoomChange();
            } else {
                var zoomddl = $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList');
                zoomddl.selectItemsByIndices(this._zoomLevel);
            }
        },

        _mobileLayoutZoomChange: function () {
            var _zoomLevels = [0.5, 0.75, 1, 1.25, 1.5, 2, 4];
            this._zoomContainer(_zoomLevels[this._zoomLevel], false);
        },

        _zoomInContainer: function (isFactor, scaleFactor) {
            var zoomFactor = scaleFactor;
            var pageViewerContainer = $('#' + this._id + '_viewerContainer');
            var pagecontainer = $('#' + this._id + '_pageviewContainer');
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            if (isFactor) {
                this._zoomVal = this._zoomVal + zoomFactor;
            }
            var w = pagecontainer.width();
            var h = pagecontainer.height();
            if (this.model.printMode) {
                w = w + (this._pageModel.MarginLeft + this._pageModel.MarginRight);
                h = h + (this._pageModel.MarginTop + this._pageModel.MarginBottom);
            }

            this._applyPageTransform(pagecontainer, this._zoomVal , this._zoomVal);
            pageViewline.css({ "width": w * this._zoomVal, "height": h * this._zoomVal });

            if (!this.model.printMode) {
                if (pageViewerContainer.width() < pageViewline.outerWidth()) {
                    pageViewerContainer.scrollLeft(this._originX + (((w + 10) * zoomFactor) / 2));
                }

                if (pageViewerContainer.height() < pageViewline.outerHeight()) {
                    pageViewerContainer.scrollTop(this._originY + (((h + 10) * zoomFactor) / 2));
                }
            }
        },

        _zoomOutContainer: function (isFactor, scaleFactor) {
            var zoomFactor = scaleFactor;
            var pageViewerContainer = $('#' + this._id + '_viewerContainer');
            var pagecontainer = $('#' + this._id + '_pageviewContainer');
            var pageViewline = $('#' + this._id + '_pageviewOuterline');
            if (isFactor) {
                this._zoomVal = this._zoomVal - zoomFactor;
            }
            var w = pagecontainer.width();
            var h = pagecontainer.height();
            this._applyPageTransform(pagecontainer, this._zoomVal , this._zoomVal);
            pageViewline.css({ "width": w * this._zoomVal, "height": h * this._zoomVal });

            if (pageViewerContainer.width() < pageViewline.outerWidth()) {
                pageViewerContainer.scrollLeft(this._originX - (((w + 10) * zoomFactor) / 2));
            }

            if (pageViewerContainer.height() < pageViewline.outerHeight()) {
                pageViewerContainer.scrollTop(this._originY - (((h + 10) * zoomFactor) / 2));
            }
        },

        _applyPageTransform: function (targetTag, transformX, transformY, fittype) {
            var transform = "scale(" + transformX + "," + transformY + ")";
            var browserInfo = ej.browserInfo();
			if (browserInfo.name == 'msie') {
				if (browserInfo.version == "8.0") {
					 targetTag.css({ 'zoom': transformX })
				}
				else {
						 targetTag.css({ '-ms-transform': transform, '-ms-transform-origin': '0 0' });
				}
                this._setZoomScrollerIE();
                var scrollContainer = $('#' + this._id + '_viewerContainer');
                switch (fittype) {
                    case 'fitToPage':
                        scrollContainer.css({ "overflow-x": "hidden", "overflow-y": "hidden" });
                        break;
                    case 'wholepage':
                        scrollContainer.css({ "overflow-y": "hidden" });
                        break;
                    case 'fitToWidth':
                        scrollContainer.css({ "overflow-x": "hidden" });
                        break;
                }
            } else if (browserInfo.name == 'opera') {
                targetTag.css({ '-o-transform': transform, '-o-transform-origin': '0 0' });
            }
            else if (browserInfo.name == 'mozilla') {
                if (browserInfo.version == '11.0') {
                    targetTag.css({ 'transform': transform, 'transform-origin': '0 0' });
                } else {
                    targetTag.css({ '-moz-transform': transform, '-moz-transform-origin': '0 0' });
                }
            } else if (browserInfo.name == 'webkit' || browserInfo.name == 'chrome') {
                targetTag.css({ '-webkit-transform': transform, '-webkit-transform-origin': '0 0' });
            }
        },

        _zoomContainer: function (zoomfactor, isMode) {
            this._zoomVal = zoomfactor;
            var _anchorWidth = $('#' + this._id + '_pageWidth');
            var _anchorHeight = $('#' + this._id + '_pageHeight');
            var spanWidth = _anchorWidth.find('span.e-reportviewer-icon');
            var spanHeight = _anchorHeight.find('span.e-reportviewer-icon');
            if (_anchorWidth.attr('isSelect') == 'true' || _anchorHeight.attr('isSelect') == 'true') {
                this._resetPage();
            }
            if (spanWidth) {
                spanWidth.removeClass('e-reportviewer-pagefittopage');
                spanWidth.addClass('e-reportviewer-emptyconetent');
            }
            if (spanHeight) {
                spanHeight.addClass('e-reportviewer-emptyconetent');
                spanHeight.removeClass('e-reportviewer-pagefittopage');
            }

            var scalefactor = Math.abs(this._preZoomVal - zoomfactor);
            if (scalefactor == 0) return false;
            if (this._preZoomVal > zoomfactor) {
                this._zoomOutContainer(isMode, scalefactor);
            } else {
                this._zoomInContainer(isMode, scalefactor);
            }
            this._preZoomVal = zoomfactor;
        },

        _setZoomScrollerIE: function (zoomfact) {
            var browserInfo = ej.browserInfo();
            if (browserInfo.name == 'msie') {
                if (this._pageModel && this._pageModel.PageData && this._pageModel.PageData.length > 0) {
                    var scrollContainer = $('#' + this._id + '_viewerContainer');
                    var pagdata = this._pageModel.PageData[0];
                    var pageHeight = pagdata.Height;
                    var pageWidth = pagdata.Width;

                    var headerHeight = pagdata.HeaderHeight;
                    var footerHeight = pagdata.FooterHeight;

                    var totalpageHeight = pageHeight + headerHeight + footerHeight;

                    var height = scrollContainer.height();
                    var width = scrollContainer.width();

                    totalpageHeight = totalpageHeight * zoomfact;
                    pageWidth = pageWidth * zoomfact;

                    scrollContainer.css({ "overflow": "hidden" });
                    scrollContainer.css({ "overflow-x": "Auto" });
                    scrollContainer.css({ "overflow-y": "Auto" });

                    if (totalpageHeight < height && pageWidth < width) {
                        scrollContainer.css({ "overflow-x": "hidden" });
                        scrollContainer.css({ "overflow-y": "hidden" });
                    } else if (pageWidth < width) {
                        scrollContainer.css({ "overflow-x": "hidden" });
                    } else if (totalpageHeight < height) {
                        scrollContainer.css({ "overflow-y": "hidden" });
                    }
                }
            }
        },

        _showZoomControl: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbar_zoomin').parents("li.e-reportviewer-toolbarli").css("display", "block");
                $('#' + this._id + '_toolbar_zoomout').parents("li.e-reportviewer-toolbarli").css("display", "block");
                $('#' + this._id + '_toolbar_zoomSelection').parents("div.e-reportviewer-ejdropdownlist").css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbar_zoomin').parents("li.e-reportviewer-toolbarli").css("display", "none");
                $('#' + this._id + '_toolbar_zoomout').parents("li.e-reportviewer-toolbarli").css("display", "none");
                $('#' + this._id + '_toolbar_zoomSelection').parents("div.e-reportviewer-ejdropdownlist").css("display", "none");
            }
        },
        //-------------------- Page Zoom[end] -------------------------//

        //-------------------- Refresh Report[Start] -------------------------//
        _refreshReport: function () {
            this._refresh = true;
            this._unwireEvents();
            this._initViewer();
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getDataSourceCredential, 'dataSourceCredentials': this.model.dataSourceCredentials, 'dataSources': this.model.dataSources, 'parameters': this.model.parameters }), "_getDataSourceCredential");
        },

        _showRefreshButton: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbar_refresh').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbar_refresh').parent().css("display", "none");
            }
        },
        //-------------------- Refresh Report[End] -------------------------//

        //-------------------- Toolbar Actions[Start] -------------------------//
        _wireEvents: function () {
            if (!this.model.toolbarSettings.templateId) {
                this._on($('#' + this._id + '_txtpageNo'), "keypress", this._onkeyPress);
                this._on($('#' + this._id + '_txtpageNo'), "click ", this._onToolbarItemClick);
                this._on($('#' + this._id + '_zoomSelection'), "change", this._zoomValChange);
                this._on($('#' + this._id + '_toolbar_zoomSelection_container'), "click ", this._onToolbarItemClick);
            }

            this._on($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
            this._on($('#' + this._id + '_pageviewOuterContainer'), "click", this._viewerClick);
            //this._on($('#' + this._id + '_viewerContainer'), "pinchin pinchout swipeleft swiperight", this._pinchAndSwipe);
            this._on($(document), "click", this._onReporClick);


            if (this._isDevice) {
                this._on($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
                this._on($('#' + this._id + '_viewerContainer'), "scrollstop", this._containerScrollStop);
            } else {
                this._on($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseover", this._showIconToolTip);
                this._on($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseout", this._hideIconToolTip);
            }
        },

        _unwireEvents: function () {
            if (!this.model.toolbarSettings.templateId) {
                this._off($('#' + this._id + '_txtpageNo'), "keypress", this._onkeyPress);
                this._off($('#' + this._id + '_txtpageNo'), "click ", this._onToolbarItemClick);
                this._off($('#' + this._id + '_zoomSelection'), "change", this._zoomValChange);
                this._off($('#' + this._id + '_toolbar_zoomSelection_container'), "click ", this._onToolbarItemClick);
            }

            this._off($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
            this._off($('#' + this._id + '_pageviewOuterContainer'), "click", this._viewerClick);
            //this._off($('#' + this._id + '_viewerContainer'), "pinchin pinchout swipeleft swiperight", this._pinchAndSwipe);
            this._off($(document), "click", this._onReporClick);
            this._off($('#' + this._id + '_viewReportClick'), "click", this._viewReportParamsClick);

            if (this._isDevice) {
                this._off($('#' + this._id + '_viewerContainer'), "scroll", this._scrollPage);
                this._off($('#' + this._id + '_viewerContainer'), "scrollstop", this._containerScrollStop);
            } else {
                this._off($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseover", this._showIconToolTip);
                this._off($('#' + this._id + '_toolbarContainer li,#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist,#' + this._id + '_toolbarContainer .e-reportviewer-tbpage'), "mouseout", this._hideIconToolTip);
            }
        },

        _onReporClick: function (event) {
            if (!($(event.target).hasClass('e-reportviewer-export') || $($(event.target).children()).hasClass('e-reportviewer-export'))) {
                $('#' + this._id + '_toolbar_exportListTip').css('display', 'none');
            } else {
                if ($('#' + this._id + '_toolbar_exportListTip').css('display') != "block") {
                    $('#' + this._id + '_toolbar_exportListTip').css('display', 'none');
                }
            }

            if (!($(event.target).hasClass('e-reportviewer-pagefit') || $($(event.target).children()).hasClass('e-reportviewer-pagefit'))) {
                $('#' + this._id + '_toolbar_fittoPagePopup').css('display', 'none');
            } else {
                if ($('#' + this._id + '_toolbar_fittoPagePopup').css('display') != "block") {
                    $('#' + this._id + '_toolbar_fittoPagePopup').css('display', 'none');
                }
            }
        },

        _onToolbarItemClick: function (event) {
            $('#' + this._id + '_rptTooltip').css('display', 'none');
        },

        _toolbarClick: function (event) {
            var targetItem = event.target
            var clickedItem = event.target;
            if (this._isToolbarClick) {
                if (this._isPageDialogOpened()) {
                    return;
                }

                $('#' + this._id + '_rptTooltip').css('display', 'none');
                if ($(clickedItem).hasClass('e-reportviewer-toolbarli')) {
                    clickedItem = $(clickedItem).find('span');
                }
                if ($(clickedItem).hasClass("e-reportviewer-print")) {
                    this._print();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-export")) {
                    this._showTooltip();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-pagesetup")) {
                    this._showPrintPageSetup();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotofirst") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoFirstPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotolast") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoLastPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotonext") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoNextPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotoprevious") && !($(clickedItem).hasClass("e-reportviewer-disabled"))) {
                    this._gotoPreviousPage();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-gotoparent")) {
                    this._gotoParentReport();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-documentmap") && !this._printMode) {
                    this._documentMapClick();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-zoomin")) {
                    this._zoomIn();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-zoomout")) {
                    this._zoomOut();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-pagefit")) {
                    this._showFitToPagetip();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-preview")) {
                    this._printlayout();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-refresh")) {
                    this._refreshReport();
                }
                else if ($(clickedItem).hasClass("e-reportviewer-parameter")) {
                    this._toggleParameterBlock();
                }
            }
        },

        _showToolbar: function (enableToolbar) {
            if (enableToolbar) {
                $('#' + this._id + '_toolbarContainer').css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').css("display", "none");
            }
        },

        _initToolbar: function () {
            this._isToolbarClick = false;

            if (!this.model.toolbarSettings.templateId) {
                if (!this._isDevice) {
                    var zoomddl = $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList');
                    if (this._refresh) {
                        zoomddl.selectItemsByIndices(zoomddl.model.selectedIndex);
                    } else {
                        zoomddl.selectItemsByIndices(2);
                    }
                    zoomddl.disable();

                    $('#' + this._id + '_txtpageNo').attr('disabled', 'disabled');

                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst').addClass('e-reportviewer-disabled');
                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious').addClass('e-reportviewer-disabled');
                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext').addClass('e-reportviewer-disabled');
                    $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast').addClass('e-reportviewer-disabled');
                }
            }
        },

        _enableToolbarItems: function () {
            this._isToolbarClick = true;
            if (!this.model.toolbarSettings.templateId) {
                if (!this._isDevice) {
                    $('#' + this._id + '_txtpageNo').removeAttr('disabled');
                    $('#' + this._id + '_toolbar_zoomSelection').data('ejDropDownList').enable();
                }
            }
        },
        //-------------------- Toolbar Actions[End] -------------------------//

        //-------------------- Tooltip Localization Actions[Start]------------------------//
        _showIconToolTip: function (event) {
            if ($('#' + this._id + '_rptTooltip').css('display') == 'none') {
                this._showTooltipContent(event);
            }
        },

        _hideIconToolTip: function () {
            $('#' + this._id + '_rptTooltip').css('display', 'none');
        },

        _viewpopupBlockNone: function (jqueryEement) {
            if (ej.isNullOrUndefined(jqueryEement) || jqueryEement.length == 0 || jqueryEement.css('display') != 'block') {
                return false;
            }
            return true;
        },

        _isPageDialogOpened: function () {
            var _ejPageDialog = $('#' + this._id + '_printPageSetup').data("ejDialog");
            if (!ej.isNullOrUndefined(_ejPageDialog) && _ejPageDialog.isOpen()) {
                return true;
            }
            return false;
        },

        _showTooltipContent: function (event) {
            var currentToolbarItem = event.target;
            if (this.model.toolbarSettings.showTooltip && !this._isDevice) {
                var TagPos;
                var toolTipText;
                var isShowTooltip = true;
                if ($(currentToolbarItem).hasClass('e-reportviewer-toolbarli')) {
                    currentToolbarItem = $(currentToolbarItem).find('span');
                }

                if (this._viewpopupBlockNone($('#' + this._id + '_toolbar_fittoPagePopup')) || this._viewpopupBlockNone($('#' + this._id + '_toolbar_exportListTip')) || this._isPageDialogOpened()) {
                    isShowTooltip = false;
                }

                if ($(currentToolbarItem).hasClass("e-reportviewer-print")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-print')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('print');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-export")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-export')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('export');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotofirst")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotofirst')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('first');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotolast")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotolast')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('last');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotonext")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotonext')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('next');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotoprevious")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoprevious')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('previous');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-documentmap")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-documentmap')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('documentmap');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-zoomin")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-zoomin')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('zoomin');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-zoomout")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-zoomout')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('zoomout');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-preview")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-preview')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('preview');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-refresh")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-refresh')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('refresh');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-parameter")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-parameter')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('parameter');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-gotoparent")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-gotoparent')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('back');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-tbpage") || $(currentToolbarItem).hasClass("e-reportviewer-pagenumber") || $(currentToolbarItem).hasClass("e-reportviewer-labelpageno")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-tbpage')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('pageIndex');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-ejdropdownlist") || $(currentToolbarItem).hasClass("e-dropdownlist") || $(currentToolbarItem).hasClass("e-select") || $(currentToolbarItem).hasClass("e-down-arrow")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-ejdropdownlist')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('zoom');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-pagefit")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-pagefit')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('fittopage');
                }
                else if ($(currentToolbarItem).hasClass("e-reportviewer-pagesetup")) {
                    TagPos = $('#' + this._id + '_toolbarContainer .e-reportviewer-pagesetup')[0].getBoundingClientRect();
                    toolTipText = this._getTootipText('pagesetup');
                }

                if (toolTipText && isShowTooltip) {
                    $('#' + this._id + '_rptTooltip_Header').html(toolTipText.header);
                    $('#' + this._id + '_rptTooltip_Content').html(toolTipText.content);
                    $('#' + this._id + '_rptTooltip').css({ 'top': (TagPos.top + TagPos.height) + 5, 'left': (TagPos.left + (TagPos.width / 2)), 'display': 'block', 'position': 'fixed' });
                }
            }
        },

        _getTootipText: function (tipType) {
            var localeObj = ej.ReportViewer.localization[this.model.locale] ? ej.ReportViewer.localization[this.model.locale] : ej.ReportViewer.localization["en-us"];
            var headerTxt = '';
            var contentTxt = '';
            switch (tipType) {
                case 'print':
                    headerTxt = localeObj['toolbar']['print']['headerText'];
                    contentTxt = localeObj['toolbar']['print']['contentText'];
                    break;
                case 'export':
                    headerTxt = localeObj['toolbar']['exportformat']['headerText'];
                    contentTxt = localeObj['toolbar']['exportformat']['contentText'];
                    break;
                case 'first':
                    headerTxt = localeObj['toolbar']['first']['headerText'];
                    contentTxt = localeObj['toolbar']['first']['contentText'];
                    break;
                case 'previous':
                    headerTxt = localeObj['toolbar']['previous']['headerText'];
                    contentTxt = localeObj['toolbar']['previous']['contentText'];
                    break;
                case 'next':
                    headerTxt = localeObj['toolbar']['next']['headerText'];
                    contentTxt = localeObj['toolbar']['next']['contentText'];
                    break;
                case 'last':
                    headerTxt = localeObj['toolbar']['last']['headerText'];
                    contentTxt = localeObj['toolbar']['last']['contentText'];
                    break;
                case 'zoomin':
                    headerTxt = localeObj['toolbar']['zoomIn']['headerText'];
                    contentTxt = localeObj['toolbar']['zoomIn']['contentText'];
                    break;
                case 'zoomout':
                    headerTxt = localeObj['toolbar']['zoomOut']['headerText'];
                    contentTxt = localeObj['toolbar']['zoomOut']['contentText'];
                    break;
                case 'preview':
                    headerTxt = localeObj['toolbar']['printLayout']['headerText'];
                    contentTxt = localeObj['toolbar']['printLayout']['contentText'];
                    break;
                case 'documentmap':
                    headerTxt = localeObj['toolbar']['documentMap']['headerText'];
                    contentTxt = localeObj['toolbar']['documentMap']['contentText'];
                    break;
                case 'refresh':
                    headerTxt = localeObj['toolbar']['refresh']['headerText'];
                    contentTxt = localeObj['toolbar']['refresh']['contentText'];
                    break;
                case 'parameter':
                    headerTxt = localeObj['toolbar']['parameter']['headerText'];
                    contentTxt = localeObj['toolbar']['parameter']['contentText'];
                    break;
                case 'back':
                    headerTxt = localeObj['toolbar']['back']['headerText'];
                    contentTxt = localeObj['toolbar']['back']['contentText'];
                    break;
                case 'pageIndex':
                    headerTxt = localeObj['toolbar']['pageIndex']['headerText'];
                    contentTxt = localeObj['toolbar']['pageIndex']['contentText'];
                    break;
                case 'zoom':
                    headerTxt = localeObj['toolbar']['zoom']['headerText'];
                    contentTxt = localeObj['toolbar']['zoom']['contentText'];
                    break;
                case 'fittopage':
                    headerTxt = localeObj['toolbar']['fittopage']['headerText'];
                    contentTxt = localeObj['toolbar']['fittopage']['contentText'];
                    break;
                case 'pagesetup':
                    headerTxt = localeObj['toolbar']['pagesetup']['headerText'];
                    contentTxt = localeObj['toolbar']['pagesetup']['contentText'];
                    break;
            }
            return { header: headerTxt, content: contentTxt };
        },
        //-------------------- Tooltip Localization Actions[End]------------------------//

        //-------------------- DrillDown Actions[Start] -------------------------//
        _drillAction: function () {
            if ($._data($('.txtToggle_' + this._id).get(0), 'events') == undefined) {
                this._on($('.txtToggle_' + this._id), "click", this._drillClick);
            }
        },

        _drillClick: function (event) {
            var drillInfo = $(event.currentTarget).data('drillObj');
            if (drillInfo) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.drillDown, 'pageindex': this._currentPage, 'toggleInfo': drillInfo, 'refresh': false, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
            }
        },
        //-------------------- DrillDown Actions[End] -------------------------//

        //-------------------- Sorting Actions[Start] -------------------------//
        _sortingAction: function () {
            if ($._data($('.txtSorting_' + this._id).get(0), 'events') == undefined) {
                this._on($('.txtSorting_' + this._id), "click", this._sortingClick);
            }
        },

        _sortingClick: function (event) {
            var sortingInfo = $(event.currentTarget).data('sortingObj');
            if (sortingInfo) {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.sorting, 'pageindex': this._currentPage, 'sortingInfo': sortingInfo, 'refresh': false, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
            }
        },
        //-------------------- Sorting Actions[End] -------------------------//

        //-------------------- Document Map Actions[Start]----------------------//
        _showDocumentMap: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-documentmap').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-documentmap').parent().css("display", "none");
            }
        },

        _appendDocumentMapDiv: function () {
            if (this._isDevice) {
                var documentMapDiv = $('#' + this._id + '_documentmapContainer');
                if (documentMapDiv.length == 0) {
                    documentMapDiv = ej.buildTag("div.e-reportviewer-documentmappopup", "", {}, { id: this._id + '_documentmapContainer' });
                    $('body').append(documentMapDiv);
                }
            }
        },

        _selectDocMapToolItem: function (enable) {
            if (!this.model.toolbarSettings.templateId) {
                var _ejToolbar = $('#' + this._id + '_toolbarContainer').data("ejToolbar");
                if (enable) {
                    _ejToolbar.selectItemByID(this._id + '_ejtb_documentmap');
                } else {
                    _ejToolbar.deselectItemByID(this._id + '_ejtb_documentmap');
                }
            }
        },

        _documentMapClick: function () {
            if (!this._isDevice) {
                var ejSplitterCtrl = $('#' + this._id + '_reportviewerContainer').data('ejSplitter');
                if ($(ejSplitterCtrl.panes[0]).hasClass('collapsed')) {
                    this._pageDocMapFlag = true;
                    ejSplitterCtrl.expand(0);
                    this._selectDocMapToolItem(true);
                } else {
                    this._pageDocMapFlag = false;
                    ejSplitterCtrl.collapse(0);
                    this._selectDocMapToolItem(false);
                }
            } else {
                var documentMapDiv = $('#' + this._id + '_documentmapContainer');
                if (documentMapDiv.css('display') == 'block') {
                    this._pageDocMapFlag = true;
                    documentMapDiv.css('display', 'none');
                    this._selectDocMapToolItem(false);
                } else {
                    this._pageDocMapFlag = false;
                    documentMapDiv.css('display', 'block');
                    this._selectDocMapToolItem(true);
                }
            }
        },

        _showDocumentMapContainer: function (enable) {
            if (!this._isDevice) {
                var ejSplitterCtrl = $('#' + this._id + '_reportviewerContainer').data('ejSplitter');
                if (enable) {
                    ejSplitterCtrl.expand(0);
                } else {
                    ejSplitterCtrl.collapse(0);
                }
            } else {
                if (enable) {
                    $('#' + this._id + '_documentmapContainer').css('display', 'block');
                } else {
                    $('#' + this._id + '_documentmapContainer').css('display', 'none');
                }
            }
        },

        _documentMapAction: function () {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.documentMap }), "_documentMapModel");
        },

        _containerSplit: function (jsondocmap) {
            var viewerContainer = $('#' + this._id + '_viewerContainer');
            this._appendDocumentMapDiv();
            jsondocmap = jsondocmap.replace("treeViewID", this._id + '_ejtreeView');
            $('#' + this._id + '_documentmapContainer').css("height", "100%");
            $('#' + this._id + '_documentmapContainer').html(jsondocmap);
            $('#' + this._id + '_ejtreeView').css("display", "none");
            $('#' + this._id + '_ejtreeView').ejTreeView({ height: viewerContainer.height(), nodeSelect: this._pageNavigate });

            if (this._isDevice) {
                $('#' + this._id + '_documentmapContainer').css({ 'position': 'absolute', 'left': viewerContainer.offset().left, 'top': viewerContainer.offset().top, 'height': viewerContainer.height(), 'width': viewerContainer.width() / 6 });
            } else {
                $('#' + this._id + '_reportviewerContainer').ejSplitter({ height: viewerContainer.height() + 1, orientation: ej.Orientation.Horizontal, properties: [{ paneSize: 140 }, {}] });
            }
            $('#' + this._id + '_ejtreeView').css("display", "block");
            this._selectDocMapToolItem(true);
        },

        _pageNavigate: function (args) {
            try {
                var jsondata = JSON.parse($(args.currentElement).attr('id'));
                var ejviewerCtrlId = this._id.replace('_ejtreeView', '');
                var ejReportViewer = $('#' + ejviewerCtrlId).data('ejReportViewer');
                if (ejReportViewer._isDevice) {
                    $('#' + ejviewerCtrlId + '_documentmapContainer').css('display', 'none');
                    this._selectDocMapToolItem(false);
                }
                ejReportViewer._gotoPageNo(jsondata.PageNo);
                $('#' + ejviewerCtrlId + '_viewerContainer').animate({ scrollTop: jsondata.TopPos - 12, scrollLeft: jsondata.LeftPos - 12 }, "600");
            } catch (err) {
            }
        },
        //-------------------- Document Map Actions[End]------------------------//

        //-------------------- DrillThrough Actions[Start]----------------------//
        _showDrillThrough: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-gotoparent').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-gotoparent').parent().css("display", "none");
            }
        },

        _drillThroughAction: function () {
            if ($._data($('.drillAction_' + this._id).get(0), 'events') == undefined) {
                this._on($('.drillAction_' + this._id), "click", this._drillThroughClick);
            }
        },

        _drillThroughClick: function (event) {
            var drillActionInfo = null;
            var proxy = null;
            if (this.pluginName == 'ejChart') {
                proxy = $('#' + this.ejreportid).data('ejReportViewer');
                $('#' + this._id + '_svg_TrackToolTip').css('display', 'none');
                var chartActionInfo = $('#' + this._id).data(this._id + 'actionObj_Series' + event.data.region.SeriesIndex);
                if (ej.isNullOrUndefined(chartActionInfo)) {
                    return false;
                }
                drillActionInfo = chartActionInfo[event.data.region.Region.PointIndex];
            } else if (this.pluginName == 'ejMap') {
                proxy = $('#' + this.ejreportid).data('ejReportViewer');
                if (event.originalEvent[0] == null || ej.isNullOrUndefined(event.originalEvent[0].data)) {
                    return;
                }
                drillActionInfo = event.originalEvent[0].data.ShapeActionInfo;
            } else {
                proxy = this;
                drillActionInfo = $(event.currentTarget).data('actionObj');
            }

            if (drillActionInfo) {
                var actionInfo = { actionInfo: $.extend(true, {}, drillActionInfo) };
                var isDrill = proxy._onDrillThrough(actionInfo);
                if (!isDrill) {
                    if (drillActionInfo.Hyperlink) {
                        window.open(drillActionInfo.Hyperlink);
                    } else if (drillActionInfo.ReportName) {
                        proxy._childReportAuthentication = proxy._getAuthenticationToken();
                        this._showloadingIndicator(true);
                        proxy.doAjaxPost("POST", proxy._actionUrl, JSON.stringify({ 'reportAction': proxy._reportAction.drillThrough, 'reportName': drillActionInfo.ReportName, 'parameters': drillActionInfo.Parameters, 'authKey': proxy._childReportAuthentication }), "_drillThroughModel");
                    }
                }
            }
        },

        _gotoParentReport: function () {
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.clearCache }), '_gotoParentReportModel');
        },

        _setInitialization: function () {
            this._dataSources = null;
            this._authenticationToken = null;
            this._childReportAuthentication = null;
            this._refresh = false;
            this._isToolbarClick = false;
            this._pageModel = null;
            this._currentPage = 1;
            this._reportParameters = null;
            this._reportDataSources = null;
            this._reporDataSets = null;
            this._zoomLevel = 2;
            this._isDocumentMap = false;
        },

        _cloneReportViewerProperties: function () {
            var reportProperties = {};
            reportProperties.model = $.extend(true, {}, this.model);
            reportProperties._dataSources = $.extend(true, {}, this._dataSources);
            reportProperties._authenticationToken = this._authenticationToken;
            reportProperties._childReportAuthentication = this._childReportAuthentication;
            reportProperties._refresh = this._refresh;
            reportProperties._isToolbarClick = this._isToolbarClick;
            reportProperties._pageModel = $.extend(true, {}, this._pageModel);
            reportProperties._currentPage = this._currentPage;
            reportProperties._reportParameters = $.extend(true, {}, this._reportParameters);
            reportProperties._reportDataSources = $.extend(true, {}, this._reportDataSources);
            reportProperties._reporDataSets = $.extend(true, {}, this._reporDataSets);
            reportProperties._zoomLevel = this._zoomLevel;
            reportProperties._preZoomVal = this._preZoomVal;
            reportProperties._isDevice = this._isDevice;
            reportProperties._originX = this._originX;
            reportProperties._originY = this._originY;
            reportProperties._printMode = this._printMode;
            reportProperties._pageSetup = this._pageSetup;
            reportProperties._isDocumentMap = this._isDocumentMap;
            return reportProperties;
        },

        _setParentReportViewerProperties: function () {
            var parentObj = this._parents[this._parents.length - 1];
            var parentProperties = parentObj.parentPro;
            this.model = parentProperties.model;
            this._dataSources = parentProperties._dataSources;
            this._authenticationToken = parentProperties._authenticationToken;
            this._childReportauthentication = parentProperties._childReportAuthentication;
            this._refresh = parentProperties._refresh;
            this._isToolbarClick = parentProperties._isToolbarClick;
            this._pageModel = parentProperties._pageModel;
            this._currentPage = parentProperties._currentPage;
            this._reportParameters = parentProperties._reportParameters;
            this._reportDataSources = parentProperties._reportDataSources;
            this._reporDataSets = parentProperties._reporDataSets;
            this._zoomLevel = parentProperties._zoomLevel;
            this._isDocumentMap = parentProperties._isDocumentMap;
            this._preZoomVal = parentProperties._preZoomVal;
            this._printMode = parentProperties._printMode;
            this._originX = parentProperties._originX;
            this._originY = parentProperties._originY;
            this._parentPageXY = parentObj.pagePos;
            this._parents.pop();
        },
        //-------------------- DrillThrough Actions[End]----------------------//

        //-------------------- Print Preview Layout[Start]----------------------//
        _printlayout: function () {
            this._updatePreviewLayout(!this._printMode, this._pageSetup);
        },

        _selectPreviewToolItem: function (enable) {
            if (!this.model.toolbarSettings.templateId) {
                var _ejToolbar = $('#' + this._id + '_toolbarContainer').data("ejToolbar");
                if (enable) {
                    _ejToolbar.selectItemByID(this._id + '_ejtb_preview');
                } else {
                    _ejToolbar.deselectItemByID(this._id + '_ejtb_preview');
                }
            }
        },

        _updatePreviewLayout: function (isPrintMode, isPageSetup) {
            this._printMode = isPrintMode;
            if (this._printMode) {
                this._pageLayoutPage = this._currentPage;
            } else {
                this._currentPage = this._pageLayoutPage;
            }

            this._selectPreviewToolItem(this._printMode);

            if (this._isPageDocMap && !this._isDocumentMap) {
                this._showDocumentMapContainer(this._printMode ? false : this._pageDocMapFlag);
            }
            this.model.printMode = this._printMode;
            if (isPageSetup) {
                var paperType = $('#' + this._id + '_PaperSize').data("ejDropDownList");
                var paperDimension = this._getPaperSize(paperType.model.value);
                var landscape = $('#' + this._id + '_landscape').data('ejRadioButton');
                var portrait = $('#' + this._id + '_portrait').data('ejRadioButton');

                if (landscape.model.checked == true) {
                    this._paperSetup.paperHeight = paperDimension.width;
                    this._paperSetup.paperWidth = paperDimension.height;
                } else {
                    this._paperSetup.paperHeight = paperDimension.height;
                    this._paperSetup.paperWidth = paperDimension.width;
                }

                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode, 'PageSetup': this._paperSetup }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
            } else {
                this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.getPageModel, 'refresh': this._refresh, 'pageindex': this._currentPage, 'pageInit': this._isToolbarClick, 'isPrint': this._printMode }), !this._printMode ? "_getPageModel" : "_getPreviewModel");
            }
        },

        _showPreviewButton: function (enable) {
            if (enable) {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-preview').parent().css("display", "block");
            }
            else {
                $('#' + this._id + '_toolbarContainer').find('.e-reportviewer-preview').parent().css("display", "none");
            }
        },
        //-------------------- Print Preview Layput[End]----------------------//

        //-------------------- SetCulture Layput[Start]----------------------//
        _setCultureInfo: function () {
            var culture = this.model.locale;
            var spanTagWidth = $('#' + this._id + 'Pagewidthspan');
            var spanTagHeight = $('#' + this._id + 'pageheightSpan');
            var _pageWidth = $('#' + this._id + '_pageWidth');
            var _pageHeight = $('#' + this._id + '_pageHeight');
            $('#' + this._id + '_viewReportClick').val(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]["viewButton"] : ej.ReportViewer.localization["en-US"]["viewButton"]);
            $('#' + this._id + '_pdf').html(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]['toolbar']['exportformat']["Pdf"] : ej.ReportViewer.localization["en-US"]['toolbar']['exportformat']["Pdf"]);
            $('#' + this._id + '_xls').html(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]['toolbar']['exportformat']["Excel"] : ej.ReportViewer.localization["en-US"]['toolbar']['exportformat']["Excel"]);
            $('#' + this._id + '_word').html(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]['toolbar']['exportformat']["Word"] : ej.ReportViewer.localization["en-US"]['toolbar']['exportformat']["Word"]);
            $('#' + this._id + '_html').html(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]['toolbar']['exportformat']["Html"] : ej.ReportViewer.localization["en-US"]['toolbar']['exportformat']["Html"]);
            _pageWidth.html(spanTagWidth);
            _pageWidth.append(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]['toolbar']['fittopage']["pageWidth"] : ej.ReportViewer.localization["en-US"]['toolbar']['fittopage']["pageWidth"]);
            _pageHeight.html(spanTagHeight);
            _pageHeight.append(ej.ReportViewer.localization[culture] ? ej.ReportViewer.localization[culture]['toolbar']['fittopage']["pageHeight"] : ej.ReportViewer.localization["en-US"]['toolbar']['fittopage']["pageHeight"]);

            var _dateTimePicker = $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').find('.e-reportviewer-DateTime');
            if (_dateTimePicker && _dateTimePicker.length > 0) {
                for (var i = 0; i < _dateTimePicker.length; i++) {
                    $('#' + $(_dateTimePicker[i]).attr('id')).data('ejDatePicker').setModel({ 'locale': this.model.locale });
                }
            }
        },
        //-------------------- SetCulture Layput[End]----------------------//

        //-------------------- ReportViewer Page Cache[start]----------------------//
        _storePageCache: function (key) {
            if (this._pageModel && this._pageModel.PageData && this._pageModel.PageData.length > 0) {
                this._pageCache[key] = $.extend(true, {}, this._pageModel);
            }
        },

        _retrievePageCache: function (key) {
            return this._pageCache[key];
        },

        _pageContains: function (key) {
            var _page = this._pageCache[key];
            return !ej.isNullOrUndefined(_page);
        },

        _clearPageCache: function () {
            this._pageCache = {};
        },
        //-------------------- ReportViewer Page Cache[end]----------------------//

        //-------------------- Common ReportViewer events & utils[start]----------------------//
        _itemfadeToggle: function (targetTag) {
            targetTag.fadeToggle();
        },

        _popupClick: function (event) {
            var pagePopup = $('#' + this._id + '_pageInfoPopup');
            pagePopup.fadeIn();
            pagePopup.finish();
        },

        _containerScrollStop: function (event) {
            var pagePopup = $('#' + this._id + '_pageInfoPopup');
            pagePopup.finish().fadeIn();
            pagePopup.finish().fadeOut(3000);
        },

        _viewerClick: function (event) {
            if (this._isDevice) {
                var pagePopup = $('#' + this._id + '_pageInfoPopup');
                pagePopup.finish().fadeIn(1000);
                pagePopup.finish().fadeOut(3000);
                pagePopup.hide(4000);
                $('#' + this._id + '_documentmapContainer').css('display', 'none');
            }
            $('#' + this._id + '_toolbar_exportListTip').css('display', 'none');
        },

        _pinchAndSwipe: function (event) {
            switch (event.type) {
                case "pinchin":
                    this._zoomOut();
                    break;
                case "pinchout":
                    this._zoomIn();
                    break;
                case "swipeleft":
                    this._gotoNextPage();
                    if (this._isDevice) {
                        this._updatePageNo();
                    }
                    break;
                case "swiperight":
                    this._gotoPreviousPage();
                    if (this._isDevice) {
                        this._updatePageNo();
                    }
                    break;
            }
        },

        _scrollPage: function (event) {
            var scrollContainer = $(event.target);
            var scrollY = this._orginY = scrollContainer.scrollTop();
            this._orginX = scrollContainer.scrollLeft();
            if (this._isDevice) {
                if (scrollY + scrollContainer.innerHeight() >= scrollContainer[0].scrollHeight) {
                    scrollContainer.scrollTop(2);
                    if (!this._pageModel || this._currentPage == this._pageModel.TotalPages) {
                        return;
                    }
                    this._gotoNextPage();
                    this._updatePageNo();
                } else if (scrollY == 0) {
                    scrollContainer.scrollTop(2);
                    if (!this._pageModel || this._currentPage == 1) {
                        return;
                    }
                    this._gotoPreviousPage();
                    this._updatePageNo();
                }
            }
        },

        _showViewerBlock: function (showblock) {
            if (showblock) {
                $('#' + this._id + '_viewBlockContainer').css("display", "block");
                var parameterBlock = $('#' + this._id + '_viewBlockContainer .e-reportviewer-viewerblockcellcontent').find('table');
                this._selectparamToolItem(parameterBlock.is('[isviewclick]'));
            } else {
                $('#' + this._id + '_viewBlockContainer').css("display", "none");
            }
            if (this._isDevice) {
                $('#' + this._id + '_viewBlockContainer.e-reportviewer-blockstyle').css('z-index', showblock ? '10' : '0');
            }
        },

        _showNavigationIndicator: function (isShow) {
            $('#' + this._id + '_viewerContainer').ejWaitingPopup({ showOnInit: isShow });
            $('#' + this._id + '_viewerContainer_WaitingPopup').addClass('e-reportviewer-waitingpopup');
        },

        _showloadingIndicator: function (isShow) {
            if (isShow) {
                $('#' + this._id + '_loadingIndicator').css('display', 'block');
                $('#' + this._id + '_pageviewOuterContainer').css('display', 'none');
            }
            else {
                $('#' + this._id + '_loadingIndicator').css('display', 'none');
                $('#' + this._id + '_pageviewOuterContainer').css('display', 'block');
            }
            $('#' + this._id + '_loadingIndicator').ejWaitingPopup({ showOnInit: isShow });
            $('#' + this._id + '_loadingIndicator_WaitingPopup').addClass('e-reportviewer-waitingpopup');
        },

        _isMobileDevice: function () {
            //return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));  
            return (/windows phone|android|iphone/i.test(navigator.userAgent.toLowerCase()));
        },

        _getAuthenticationToken: function () {
            function _guid() {
                function guid() {
                    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
                }
                return (guid() + guid() + "-" + guid() + "-" + guid() + "-" + guid() + "-" + guid() + guid() + guid()).toUpperCase();
            }
            return _guid() + "^" + this._id;
        },

        _updateToolbarmodel: function () {
            this._showToolbar(this.model.toolbarSettings.showToolbar);
            this._showPrintButton(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Print);
            this._showRefreshButton(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Refresh);
            this._showZoomControl(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Zoom);
            this._showFittoPage(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.FittoPage);
            this._showExportControls(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Export);
            this._showExportList();
            this._showPageNavigationControls(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.PageNavigation);
            this._showParameterBlock(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.Parameters);
            this._showPreviewButton(model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.PrintLayout);
            this._showPrintPageSetupButton(this.model.toolbarSettings.items & ej.ReportViewer.ToolbarItems.PageSetup);
        },

        _viewerResize: function (event) {
            var proxy = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                var elementWidth = proxy._isWidth ? proxy.element[0].parentElement.style.width : proxy.element[0].style.width;
                var elementHeight = proxy._isHeight ? proxy.element[0].parentElement.style.height : proxy.element[0].style.height;
                var _height = $(proxy.element).height();
                var _width = $(proxy.element).width();

                if (elementHeight.indexOf("%") != -1) {
                    proxy._isPercentHeight = parseInt(elementHeight);
                    var containerHeight = proxy._isHeight ? $(proxy.element).parent().height() : _height;
                    _height = !proxy._isHeight ? ((containerHeight / 100) * proxy._isPercentHeight) : containerHeight;
                } else if (proxy.element[0].parentElement.clientHeight != 0 && proxy._isPercentHeight != -1) {
                    _height = !proxy._isHeight ? ((proxy.element[0].parentElement.clientHeight / 100) * proxy._isPercentHeight) : proxy.element[0].parentElement.clientHeight;
                } else if (proxy._isHeight && proxy.element[0].parentElement.clientHeight != 0) {
                    _height = proxy.element[0].parentElement.clientHeight;
                }
                proxy.element.height(_height);

                if (elementWidth.indexOf("%") != -1) {
                    proxy._isPercentWidth = parseInt(elementWidth);
                    var containerWidth = proxy._isWidth ? $(proxy.element).parent().width() : _width;
                    _width = !proxy._isWidth ? (containerWidth / 100) * proxy._isPercentWidth : containerWidth;
                } else if (proxy.element[0].parentElement.clientWidth != 0 && proxy._isPercentWidth != -1) {
                    _width = !proxy._isWidth ? (proxy.element[0].parentElement.clientWidth / 100) * proxy._isPercentWidth : proxy.element[0].parentElement.clientWidth;
                } else if (proxy._isWidth && proxy.element[0].parentElement.clientWidth != 0) {
                    _width = proxy.element[0].parentElement.clientWidth;
                }
                proxy.element.width(_width);

                proxy._setContainerSize();
            }, 200);
        },

        _showViewerPage: function (isEnable) {
            if (isEnable) {
                $('#' + this._id + '_pageviewOuterline').css('display', 'block');
            } else {
                $('#' + this._id + '_pageviewOuterline').css('display', 'none');
            }
            if (this._isDevice) {
                $('#' + this._id + '_pageviewOuterline').css('z-index', isEnable ? '10' : '0');
            }
        },

        _updatePageNo: function () {
            $('#' + this._id + '_popupPageNo').val(this._currentPage);
            $('#' + this._id + '_pageInfoPopup .e-reportviewer-popuptotalpage').html(' / ' + this._pageModel.TotalPages);
        },

        _reportReload: function () {
            this._refresh = true;
            this._unwireEvents();
            this._initViewer();
            this.doAjaxPost("POST", this._actionUrl, JSON.stringify({ 'reportAction': this._reportAction.reportLoad, 'controlId': this._id, 'reportPath': this.model.reportPath, 'reportServerUrl': this.model.reportServerUrl, 'processingMode': this.model.processingMode }), "_reportLoaded");
        },
        //-------------------- Common ReportViewer events & utils[End]----------------------//

        /*---------------------client side methods[start]----------------------------------------------------------*/
        
        getDataSetNames: function () {
            var reportdatasets = $.extend(true, {}, this._reporDataSets);
            return reportdatasets;
        },

        
        getParameters: function () {
            var reportparameters = $.extend(true, {}, this._reportParameters);
            return reportparameters;
        },

        
        gotoPageIndex: function (pageNo) {
            this._gotoPageNo(pageNo);
        },

        
        gotoLastPage: function () {
            this._gotoLastPage();
        },

        
        gotoFirstPage: function () {
            this._gotoFirstPage();
        },

        
        gotoNextPage: function () {
            this._gotoNextPage();
        },

        
        gotoPreviousPage: function () {
            this._gotoPreviousPage();
        },

        
        print: function () {
            this._print();
        },

        
        printLayout: function () {
            this._printlayout();
        },

        
        exportReport: function (format) {
            this._exportReport(format);
        },

        
        refresh: function () {
            this._refreshReport();
        },

        
        fitToPage: function () {
            this._fitToPage('fitToPage');
        },

        
        fitToPageWidth: function () {
            this._fitToPage('fitToWidth');
        },

        fitToPageHeight: function () {
            this._fitToPage('wholepage');
        },
        /*---------------------client side methods[end]----------------------------------------------------------*/

        /*---------------------client side Events[start]----------------------------------------------------------*/
        _onReportLoaded: function () {
            this._trigger("reportLoaded");
        },

        _onRenderingBegin: function (args) {
            return this._trigger("renderingBegin", args);
        },

        _onRenderingComplete: function (args) {
            this._trigger("renderingComplete", args);
        },

        _onReportError: function (args) {
            return this._trigger("reportError", args);
        },

        _onViewReportClick: function (args) {
            return this._trigger("viewReportClick", args);
        },

        _onDrillThrough: function (args) {
            return this._trigger("drillThrough", args);
        },

        _onReportExport: function (args) {
            this._trigger("reportExport", args);
        }
        /*---------------------client side Events[end]----------------------------------------------------------*/
    });
    
    ej.ReportViewer.ExportOptions = {        
        Excel: 1 << 0,        
        Html: 1 << 1,        
        Pdf: 1 << 2,        
        Word: 1 << 3,        
        All: 1 << 0 | 1 << 1 | 1 << 2 | 1 << 3
    };
    ej.ReportViewer.ExcelFormats = {        
        Excel97to2003: "excel97to2003",        
        Excel2007: "excel2007",        
        Excel2010: "excel2010",        
        Excel2013: "excel2013"
    };    
    ej.ReportViewer.WordFormats = {        
        Doc: "doc",       
        Dot: "dot",        
        Docx: "docx",        
        Word2007: "word2007",        
        Word2010: "word2010",        
        Word2013: "word2013",        
        Word2007Dotx: "word2007dotx",        
        Word2010Dotx: "word2010dotx",       
        Word2013Dotx: "word2013dotx",        
        Word2007Docm: "word2007docm",        
        Word2010Docm: "word2010docm",        
        Word2013Docm: "word2013docm",        
        Word2007Dotm: "word2007dotm",       
        Word2010Dotm: "word2010dotm",        
        Word2013Dotm: "word2013dotm",        
        Rtf: "rtf",      
        Txt: "txt",        
        EPub: "epub",        
        Html: "html",        
        Xml: "xml",       
        Automatic: "automatic"
    };
    ej.ReportViewer.ProcessingMode = {        
        Remote: "remote",        
        Local: "local"
    };    
    ej.ReportViewer.Orientation = {        
        Portrait: "Portrait",       
        Landscape: "Landscape"
    };  
    ej.ReportViewer.PaperSize = {        
        A3: "A3",        
        A4: "A4",        
        B4_JIS: "B4(JIS)",        
        B5_JIS: "B5(JIS)",        
        Envelope_10: "Envelope #10",        
        Envelope_Monarch: "Envelope Monarch",        
        Executive: "Executive",        
        Legal: "Legal",        
        Letter: "Letter",
    };    
    ej.ReportViewer.ToolbarItems = {        
        Export: 1 << 0,        
        Zoom: 1 << 1,        
        PageNavigation: 1 << 2,        
        Refresh: 1 << 3,        
        Print: 1 << 4,        
        DocumentMap: 1 << 5,        
        Parameters: 1 << 6,        
        PrintLayout: 1 << 7,        
        FittoPage: 1 << 8,        
        PageSetup: 1 << 9,        
        All: 1 << 0 | 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6 | 1 << 7 | 1 << 8 | 1 << 9
    };    
    ej.ReportViewer.RenderMode = {        
        Default: 1 << 0 | 1 << 1,        
        Mobile: 1 << 0,       
        Desktop: 1 << 1
    };

    ej.ReportViewer.localization = {};

    ej.ReportViewer.localization["en-US"] = {
        toolbar: {
            print: {
                headerText: 'Print',
                contentText: 'Print the report.'
            },
            exportformat: {
                headerText: 'Export',
                contentText: 'Select the exported file format.',
                Pdf: 'PDF',
                Excel: 'Excel',
                Word: 'Word',
                Html: 'Html'
            },
            first: {
                headerText: 'First',
                contentText: 'Go to the first page of the report.'
            },
            previous: {
                headerText: 'Previous',
                contentText: 'Go to the previous page of the report.'
            },
            next: {
                headerText: 'Next',
                contentText: 'Go to the next page of the report.'
            },
            last: {
                headerText: 'Last',
                contentText: 'Go to the last page of the report.'
            },
            documentMap: {
                headerText: 'Document Map',
                contentText: 'Show or hide the document map.'
            },
            parameter: {
                headerText: 'Parameter',
                contentText: 'Show or hide the parameters pane.'
            },
            zoomIn: {
                headerText: 'Zoom-In',
                contentText: 'Zoom in to the report.'
            },
            zoomOut: {
                headerText: 'Zoom-Out',
                contentText: 'Zoom out of the report.'
            },
            refresh: {
                headerText: 'Refresh',
                contentText: 'Refresh the report.'
            },
            printLayout: {
                headerText: 'Print Layout',
                contentText: 'Change between print layout and normal modes.'
            },
            pageIndex: {
                headerText: 'Page Number',
                contentText: 'Current page number to view.'
            },
            zoom: {
                headerText: 'Zoom',
                contentText: 'Zoom in or out on the report.'
            },
            back: {
                headerText: 'Back',
                contentText: 'Go back to the parent report.'
            },
            fittopage: {
                headerText: 'Fit to Page',
                contentText: 'Fit the report page to the container.',
                pageWidth: 'Page Width',
                pageHeight: 'Whole Page'
            },
            pagesetup: {
                headerText: 'Page Setup',
                contentText: 'Choose page setup option to change paper size, orientation and margins.'
            }
        },
        viewButton: 'View Report'
    };
})(jQuery, Syncfusion);
;
(function ($, ej, undefined) {
    // ejGanttChart is the plugin name 
    // "ej.GanttChart" is "namespace.className" will hold functions and properties

    ej.widget("ejGanttChart", "ej.GanttChart", {
        // widget element will be automatically set in this
        element: null,

        // user defined model will be automatically set in this
        model: null,
        _rootCSS: "e-ganttchart",

        /* keyConfigurations for keyboard interaction in treegrid */
        keyConfigs: {
            focus: "e",

            //Insert
            insertRecord: "45",

            // delete
            deleteRecord: "46",

            //F2
            editRecord: "113",

            // Enter
            saveRequest: "13",

            //Esc   
            cancelRequest: "27",

            //Home,
            firstRowSelection: "36",

            //End,
            lastRowSelection: "35",

            //Left arrow
            leftArrow: "37",

            //Right arrow
            rightArrow: "39",

            //Up arrow
            upArrow: "38",

            //Down arrow
            downArrow: "40",

            //tab
            moveCellRight: "9",

            //shifttab
            moveCellLeft: "shift+9",

            //AltPlusDownArrow
            selectedRowExpand: "alt+40",

            //CtrlPlusDownArrow
            totalRowExpand: "ctrl+40",

            //AltPlusUpArrow
            selectedRowCollapse: "alt+38",

            //CtrlPlusUpArrow
            totalRowCollapse: "ctrl+38"
        },

        //default model
        defaults: {
            childPropertyName: "",
            taskIdMapping: "",
            taskNameMapping: "",
            startDateMapping: "",
            endDateMapping: "",
            childMapping: "",
            finishDateMapping: "",
            durationMapping: "",
            milestoneMapping: "",
            progressMapping: "",
            predecessorMapping: "",
            resourceInfoMapping: "",
            resourceNameMapping: "",
            resourceCollection: [], /*--------------Array containing the resource names */

            holidays: [
                {
                    day: null,
                    label: null,
                    background: null
                }
            ],

            stripLines: [
                {
                    day: null,
                    label: null,
                    lineStyle: "dotted",
                    lineColor: "#169dd7",
                    lineWidth: 2
                }
            ],

            enableTaskbarTooltip: true, /*------------showing tooltip on taskbar mouse over */
            enableTaskbarDragTooltip: true, /*--------showing taskbar while dragging taskbar */

            highlightWeekends: true, /*-------------Boolean value for highlighting weekends */

            scheduleStartDate: null, /*-------------start date of entire project */
            scheduleEndDate: null, /*---------------end date of entire project */

            enableProgressBarResizing: true, /*----------Boolean value for resizing progress bar */

            rowHeight: 30,
            progressbarHeight: 100,
            connectorlineWidth: 1,
            dateFormat: "MM/dd/yyyy",
            locale: "en-US",

            //background style
            taskbarBackground: "#DE8080", //"#5AC1E0",
            progressbarBackground: "#C44647", //"#169DD7",
            weekendBackground: '#F2F2F2',
            connectorLineBackground: "#383838",
            parentTaskbarBackground: "#383838",
            parentProgressbarBackground: "#1C1C1C",

            selectedItem: null,
            selectedRowIndex: -1,
            perDayWidth: null,

            perMonthWidth: null,//year-month schedule mode
            perWeekWidth: null,//month-week schedule mode
            perHourWidth:null,
            queryTaskbarInfo: null,

            taskbarTemplate: null,
            progressbarTemplate: null,
            parenttaskbarTemplate: null,
            parentprogressbarTemplate: null,

            //internal collections
            flatRecords: [],
            updatedRecords: [],
            ids: [],
            currentViewData: [],

            scheduleWeeks: [],

            scheduleYears: [],
            scheduleMonths: [], //new property for month schedule
            scheduleDays:[],
            scheduleHours:[],
            projectStartDate: null, //new property instead of scheduleWeeks[0] and scheduleYears[0]
            projectEndDate: null,

           

            //tooltip templates
            tooltipTemplate: null,
            tooltipTemplateId:null,
            progressbarTooltipTemplateId: null,
            taskbarEditingTooltipTemplateId: null,

            taskbarEditingTooltipTemplate: null,
            progressbarTooltipTemplate: null,

            //boolean values
            showTaskNames: true,
            showProgressStatus: true,
            showResourceNames: true,
            enableRowHover: false,
            includeWeekend: false,
            allowZooming: false,
            allowGanttChartEditing: true,
            allowKeyboardNavigation: true,

            //baselines
            renderBaseline: false,
            baselineColor: '#fba41c ',

            /* editing API's */
            editSettings: {
                allowEditing: false,
                allowAdding: false,
                allowDeleting: false,
                editMode: "normal",
                dialogEditorTemplateId: null
            },

            scheduleHeaderSettings:
            {
                weekHeaderFormat: "MMM dd , yyyy",//for week - Day schedule mode
                dayHeaderFormat: "ddd",
                weekendBackground: '#F2F2F2',

                yearHeaderFormat: "yyyy",
                monthHeaderFormat: "MMM",

                hourHeaderFormat: "HH",
                minuteHeaderFormat:"mm",
                topHourHeaderFormat:"HH:mm",
                scheduleHeaderType: "week"
            },
            workingTimeScale: "TimeScale8Hours",
            roundOffDayworkingTime: true,
            durationUnit: "day",

            localizedDays: null,
            localizedMonths: null,
            //localized string to be displayed in toolTip
            columnHeaderTexts: null,
            predecessorEditingTexts: null,
            enablePredecessorEditing:true,
        },

        updateHighlightWeekends: function (boolValue) {
            var proxy = this;
            proxy.model.highlightWeekends = boolValue;

            if (proxy.model.highlightWeekends == true)
                proxy._renderWeekends();
            else
                proxy._$weekendContainer.remove();
        },

        updateWeekendBackground: function (bgcolor) {
            var proxy = this;
            proxy.model.weekendBackground = bgcolor;
            proxy._$weekendContainer.remove();
            proxy._renderWeekends();
        },

        showTooltip: function (boolValue) {
            var proxy = this;
            proxy.model.enableTaskbarTooltip = boolValue;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        showEditingTooltip: function (boolValue) {
            var proxy = this;
            proxy.model.enableTaskbarDragTooltip = boolValue;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateConnectorLineBackground: function (bgcolor) {
            var proxy = this;
            proxy.model.connectorLineBackground = bgcolor;
            proxy._createConnectorLineTemplate();
        },

        updateConnectorlineWidth: function (lineWidth) {
            var proxy = this;
            proxy.model.connectorlineWidth = parseInt(lineWidth);
            proxy._createConnectorLineTemplate();
        },


        updateTaskbarBackground: function (bgcolor) {
            var proxy = this;
            proxy.model.taskbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateProgressbarBackground: function (bgcolor) {
            this.model.progressbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateParentTaskbarBackground: function (bgcolor) {
            this.model.parentTaskbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateParentProgressbarBackground: function (bgcolor) {
            this.model.parentProgressbarBackground = bgcolor;
            this._createTaskbarTemplate();
            this._refresh();
        },

        updateRenderTaskNames: function (bool) {
            var proxy = this;
            proxy.model.showTaskNames = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRendeProgressStatus: function (bool) {
            var proxy = this;
            proxy.model.showProgressStatus = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRenderResourceNames: function (bool) {
            var proxy = this;
            proxy.model.showResourceNames = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        ganttChartEditing: function (boolValue) {
            var proxy = this;
            proxy.model.allowGanttChartEditing = boolValue;
        },

        resizeProgressbar: function (boolValue) {
            var proxy = this;
            proxy.model.enableProgressBarResizing = boolValue;

        },
        /* udpate value of allowKeyboardNavigation API from Gantt */
        updateAllowKeyboardNavigation: function (bool) {
            this.model.allowKeyboardNavigation = bool;
        },

        updateProgressbarHeight: function (height) {
            var proxy = this;
            this.model.progressbarHeight = height;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateBaselineColor: function (color) {
            var proxy = this;
            this.model.baselineColor = color;
            proxy._createTaskbarTemplate();
            proxy._refresh();
        },

        updateRenderBaseline: function (bool) {
            var proxy = this;
            this.model.renderBaseline = bool;
            proxy._createTaskbarTemplate();
            proxy._refresh();
            proxy._createConnectorLineTemplate();
        },


        //returns the schedule year width
        _getYearWidth: function () {
            var proxy = this,count=0,width=0;
            for (count = 0; count < this.model.scheduleYears.length; count++) {
                width += this._getTemplateYearWidth(this.model.scheduleYears[count]);
            }
            return (width);
        },

        //returns the schedule month width
        _getMonthWidth: function () {
            var proxy = this,
                model = proxy.model,
                date, scheduleMonths = model.scheduleMonths,
                    monthWidth = 0;

            for (var i = 0; i < scheduleMonths.length; i++) {
                var startDate = scheduleMonths[i];

                if (typeof startDate === "object") {
                    date = new Date(startDate);
                }
                else {
                    date = Globalize.parseDate(startDate, "MM/dd/yyyy");
                }
                var monthLastDay = new Date(date.getFullYear(), date.getMonth() + 1, 0);

                var lastDate = monthLastDay.getDate();
                if (lastDate == 31)
                    monthWidth += (this.model.perDayWidth * 31);
                else if (lastDate == 30)
                    monthWidth += (this.model.perDayWidth * 30);
                else if (lastDate == 28)
                    monthWidth += (this.model.perDayWidth * 28);
            }
            return monthWidth;
        },



        //returns the schedule week width
        _getWeekWidth: function () {
            return (this.model.perDayWidth * 7);
        },

        _getNumberOftWeeks: function (startdate, enddate) {


            //var date1 = new Date(startdate), date2 = new Date(enddate);
            //// The number of milliseconds in one week
            //var ONE_WEEK = 1000 * 60 * 60 * 24 * 7;
            //// Convert both dates to milliseconds
            //var date1_ms = date1.getTime();
            //var date2_ms = date2.getTime();
            //// Calculate the difference in milliseconds
            //var difference_ms = Math.abs(date1_ms - date2_ms);
            //// Convert back to weeks and return hole weeks
            //return Math.floor(difference_ms / ONE_WEEK);
            var startDate = new Date(startdate),
                endDate = new Date(enddate);

            var totalWeeksCollection = [];
            do {
                totalWeeksCollection.push(new Date(startDate));
                startDate.setDate(startDate.getDate() + 7);
            } while (!(startDate > endDate));

            return totalWeeksCollection;
        },

        //returns the ganttchart container height
        _getHeight: function () {
            return this._containerHeight;
        },

        //sets the height of the ganttchart container
        _setHeight: function (value) {
            this._containerHeight = value;
        },

        //returns the rows in the ganttchart
        getGanttChartRows: function () {
            return this._ganttChartRows;
        },

        //returns the gantt record collection
        getUpdatedRecords: function () {
            return this.model.flatRecords;
        },

        //sets value for currentviewdata , ganttrecords and row collections
        setUpdatedRecords: function (currentViewData, updatedRecords, flatRecords,ids) {
            var proxy = this;
            proxy.model.currentViewData = currentViewData;
            proxy.model.updatedRecords = updatedRecords;
            proxy.model.flatRecords = flatRecords;
            proxy.model.ids = ids;
        },

        //returns the string date type 
        _getFormatedDate: function (date, format) {
            return Globalize.format(date, format);
        },

        //returns the viewport height
        _getViewportHeight: function () {
            var proxy = this,
                headerdiv = proxy._$headerContainer,
                height;

            height = proxy.element.height() - headerdiv.height() -
                    parseFloat(headerdiv.css("border-bottom-width"));
            return height;
        },

        updateViewPortHeight:function(){
            var proxy = this;
            proxy._viewportHeight=proxy._getViewportHeight();
        },

        //constructor function
        _init: function () {

            var proxy = this, model = proxy.model,
            scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;

            proxy.element.addClass("e-ganttchart-core");
            proxy.element.attr("tabindex", "0");
            proxy._initPrivateProperties();
            proxy._initialize();
            proxy._renderGanttChart();
            proxy._$bodyContainerParent.append(proxy._$bodyContainer);
            proxy._$bodyContainer.append(proxy._$bodyContent);
            proxy._$ganttChartContainer.append(proxy._$bodyContainerParent);

            $(proxy._$bodyContainer).css({ "height": proxy._viewportHeight + "px" });
            if (proxy.model.enableVirtualization)
                $(proxy._$bodyContent).css({ "height": (model.rowHeight * model.updatedRecords.length) + "px" });

            if (scheduleMode == "week") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleWeeks.length * proxy._scheduleWeekWidth) + "px" });
            }
            else if (scheduleMode == "year") {
                $(proxy._$bodyContent).css({ "width": (proxy._scheduleYearWidth) + "px" });

            }
            else if (scheduleMode == "month") {
                $(proxy._$bodyContent).css({ "width": (proxy._scheduleMonthWidth) + "px" });
            }
            else if (scheduleMode == "day") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleDays.length * (model.perHourWidth * 24)) + "px" });
            }
            else if (scheduleMode == "hour") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleHours.length * (model.perMinuteWidth * proxy._totalInterval)) + "px" });
            }
            proxy.createGridLinesTable();
            proxy.renderGanttRecords();
          //  Inner div for scroller
            var scrollerDiv = ej.buildTag('div');
            scrollerDiv.append(proxy._$bodyContent);
            proxy._$bodyContainer.append(scrollerDiv);
            proxy._$bodyContainer.ejScroller({
                width: proxy.element.width() - 1,// additional parent div element added with 1px right border
                enableTouchScroll:false,
                height: proxy._getViewportHeight(), scroll: function (args) {

                    var arg = {},
                        scrollTop = args.scrollTop;

                    args.requestType = "scroll";
                    args.delta = scrollTop;
//                    proxy._completeAction(args);
                    if (scrollTop != undefined) {// && this.isVScroll()
                        if (proxy.onScrollHelper(scrollTop) === false) {
                           args.cancel = true;
                        }
                    } else {
                        proxy._handleScroll(args);
                    }
                    
                }
            });
            proxy._$bodyContainer.ejScroller("model.keyConfigs", { up: "", down: "" });
            proxy._$bodyContainer.ejScroller("refresh");
            //check if Hscroller is render or not
            var isHorizontalScroll = proxy._$bodyContainer.ejScroller("isHScroll");
                if (!isHorizontalScroll) {
                    proxy._$bodyContainer.addClass("e-borderbox");
                }
                else {
                    proxy._$bodyContainer.removeClass("e-borderbox");
                }
            proxy._wireEvents();


        },

        //rendering gantt chart again after zoomingvalue changed
        reRenderGanttChart: function (daywidth, flatRecords) {

            var proxy = this, model = proxy.model,
                   scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;

            proxy._wireEvents();
            model.perDayWidth = daywidth;

            // proxy.model.perMonthWidth = monthwidth;

            model.flatRecords = flatRecords;
            proxy._scheduleWeekWidth = proxy._getWeekWidth();
            proxy._scheduleYearWidth = proxy._getYearWidth();
            proxy._scheduleMonthWidth = proxy._getMonthWidth();

            $(proxy._$headerContainer).empty();
            $(proxy._$scheduleDiv).empty();
            proxy._renderGanttChart();
            proxy._$bodyContainerParent.append(proxy._$bodyContainer);
            proxy._$bodyContainer.append(proxy._$bodyContent);
            proxy._$ganttChartContainer.append(proxy._$bodyContainerParent);

            if (scheduleMode == "week") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleWeeks.length * proxy._scheduleWeekWidth) + "px" });
            }
            else if (scheduleMode == "year") {
                $(proxy._$bodyContent).css({ "width": (proxy._scheduleYearWidth) + "px" });
            }
            else if (scheduleMode == "month") {
                $(proxy._$bodyContent).css({ "width": (proxy._scheduleMonthWidth) + "px" });
            } 
            else if (scheduleMode == "day") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleDays.length * (model.perHourWidth * 24)) + "px" });
            }
            else if (scheduleMode == "hour") {
                $(proxy._$bodyContent).css({ "width": (model.scheduleHours.length * (model.perMinuteWidth * proxy._totalInterval)) + "px" });
            }
        },

        //declaring private properties
        _initPrivateProperties: function () {
            var proxy = this;
            proxy._viewportHeight = 0,
            proxy._prevScrollTop = 0,
            proxy._scrollTop = 0,
            proxy._vScrollDir = 1,
            proxy._$ganttChartContainer = null,
            proxy._$headerContainer = null,
            proxy._$bodyContainer = null,
            proxy._$bodyContent = null,
            proxy._$dependencyViewContainer = null,
            proxy._$secondaryCanvas = null,
            proxy._$ganttViewTable = null,
            proxy._$ganttViewTablebody = null,
            proxy._weekdays = [],
            proxy._rowMargin = 0,
            proxy._renderedRange = null,
            proxy._visibleRange = null,
            proxy._vTop = null,
            proxy._vBottom = null,
            proxy._rTop = null,
            proxy._rBottom = null,
            proxy._vscrollDist = 0,
            proxy._containerHeight = 0,
            proxy._id = this.element.attr("id"),
            proxy._ganttChartRows = null,
            proxy._filteredRecords = [],
            proxy._sortedRecords = [],
            proxy._tempSortedRecords = [],
            proxy._scheduleHours = [],
            proxy._scheduleMinutes = [],
            proxy._updateScheduleWidth(),
            proxy._totalMonthWidth = 0,
            proxy._tempFilteredRecords = [],
            proxy._prevLeft = 0,
            proxy._allowDragging = false,
            proxy._leftResizing = false,
            proxy._rightResizing = false,
            proxy._progressResizing = false,
            proxy._editingContainer = null,
            proxy._allowExpandCollapse = true,
            proxy._leftResizer = null,
            proxy._taskbarItem = null,
            proxy._progressbarItem = null,
            proxy._progressResizer = null,
            proxy._rightResizer = null,
            proxy._tasknameContainer = null,
            proxy._mousePosX = 0,
            proxy._currMousePosX = 0,
            proxy._currMousePosY = 0,
            proxy._prevItem = {},
            proxy._parentCollectionPreviousValues = [],
            proxy._mouseTarget = null,
            proxy._mouseDown = false,
            proxy._$scheduleDiv = null,
            proxy._leftResizerGripper = null,
            proxy._rightResizerGripper = null,
            proxy._progressHandle = null,
            proxy._progressHandleChild = null,
            proxy._mouseHoverTooltip = document.getElementById(this.model.tooltipTemplate),
            proxy._progressBarTooltipID = document.getElementById(this.model.progressbarTooltipTemplateId),
            proxy._taskbarEditingTooltipID = document.getElementById(this.model.taskbarEditingTooltipTemplateId),
            proxy._scrollBarHeight = 18,
            proxy._windowWidth = $(window).width(),
            proxy._windowHeight = $(window).height(),
            proxy._editingItem = null,
            proxy._editingTarget = null,
            proxy.tooltipState = null,
            proxy._visibleRecordsCount = 0,
            proxy._tooltipTimer,
            proxy._$gridLinesTablebody = null,

            //Properties for connector line mouse
            proxy._leftConnectorPoint = null,
            proxy._rightConnectorPoint = null,
            proxy._childLeftConnectorPoint = null,
            proxy._childRightConnectorPoint = null,
            proxy._editPredecessor = false,
            proxy._connectorPointX = null,
            proxy._connectorPointY = null,
            proxy._predecessorTooltip = null,
            proxy._totalCollapsedRecordCount = 0;
        },
        //Calculate schedulw width for all modes
        _updateScheduleWidth:function(){
            var proxy = this;
            proxy._scheduleWeekWidth = proxy._getWeekWidth();
            proxy._numOfWeeks = proxy._getNumberOftWeeks(this.model.projectStartDate, this.model.projectEndDate);
            proxy._scheduleYearWidth = proxy._getYearWidth();
            proxy._scheduleMonthWidth = proxy._getMonthWidth();
            proxy._totalWeekWidth = proxy._scheduleWeekWidth * proxy._numOfWeeks.length;
            proxy._balanceWidth = proxy._totalWeekWidth - proxy._scheduleMonthWidth;
            proxy._scheduleMonthWidth += proxy._balanceWidth;
        },
        //Initialize ganttchart
        _initialize: function () {
            var proxy = this;
            proxy._$ganttChartContainer = ej.buildTag("div.e-ganttviewercontainer#ganttviewercontainer" + proxy._id, "", { width: "100%" }, {});
            proxy._$headerContainer = ej.buildTag("div.e-ganttviewerheaderContainer#ganttviewerheaderContainer" + proxy._id, "", {
                "border-right-style": "solid",
                "border-right-width": "1px"
            }, {});
            proxy._$bodyContainer = ej.buildTag("div.e-ganttviewerbodyContianer#ganttviewerbodyContianer" + proxy._id, "", {
                '-ms-touch-action': 'none',
                'touch-action': 'none'
            }, {});

            proxy._$bodyContainerParent = ej.buildTag("div.e-ganttviewerbodyContianerparent#ganttviewerbodyContianerParent" + proxy._id, "", {
                '-ms-touch-action': 'none',
                'touch-action': 'none',
                "border-right-style": "solid",
                "border-right-width": "1px",
                "box-sizing": "border-box",
                "width": "auto",
                "height": "auto"
            }, {});
            proxy._$bodyContent = ej.buildTag("div.e-ganttviewerbodyContent#ganttviewerbodyContent" + proxy._id, "", {
            }, {});

            proxy._$dependencyViewContainer = ej.buildTag("div.e-ganttviewerbodyContianer-dependencyViewContainer#dependencyViewContainer" + proxy._id,
                "", {
                    'z-index': '4',
                    'position': 'absolute'
                }, {});
            proxy._$secondaryCanvas = ej.buildTag("div.e-ganttviewerbodyContianer-secondaryCanvas", "", {
                'z-index': '3',
                'position': 'absolute'
            }, {});
            //Container for strip lines
            proxy._$stripLineContainer = ej.buildTag("div.e-ganttviewerbodyContianer-stripLines", "", {
                'z-index': '6',
                'position': 'absolute'
            }, {});

            proxy._$weekendsContainer = ej.buildTag("div.e-ganttviewerbodyContianer-weekendsContainer", "", {
                'z-index': '1',
                'position': 'absolute'
            }, {});

            proxy._$ganttViewTable = ej.buildTag("table.e-ganttviewerbodyContianer-ganttViewTable#ganttViewTable" + proxy._id, "", {
                'z-index': '5',
                'position': 'absolute'
            }, { 'cellspacing': '0px' });
            proxy._$scheduleDiv = ej.buildTag("div.e-gantt-schedule-container#gantt-schedule" + proxy._id + "", "", {}, {});
        },

        _getScheduleLength: function (scheduleMode) {
            var proxy = this,
                model = proxy.model;

            switch (scheduleMode) {
                case "week":                    
                    return (model.scheduleWeeks.length * proxy._scheduleWeekWidth);
                    break;
                case "year":                    
                    return (proxy._scheduleYearWidth);
                    break;
                case "month":
                    return (proxy._scheduleMonthWidth);
                    break;
                case "day":
                    return (model.scheduleDays.length * (proxy.model.perHourWidth * 24));
                    break;
                case "hour":
                    return ((model.scheduleHours.length) * (model.perMinuteWidth * proxy._totalInterval));
                    break;
                default:
                    return (model.scheduleWeeks.length * proxy._scheduleWeekWidth);

            }
        },

        //Rendering GanttChart
        _renderGanttChart: function () {
            var proxy = this, model = proxy.model,
                scheduleLength, $colsgroup, $column,
            scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;
            
        proxy._createSchedule(proxy._getDateType(model.scheduleStartDate),
            proxy._getDateType(model.scheduleEndDate));

            proxy._viewportHeight = proxy._getViewportHeight();
                       
            proxy._$ganttViewTable.css("width", proxy._getScheduleLength(scheduleMode) + 'px');
            
            $colsgroup = ej.buildTag("colgroup", "", {}, {});
            $column = $(document.createElement("col"));
            $column.css("width", proxy._getScheduleLength(scheduleMode) + 'px');
            
            $colsgroup.append($column);
            proxy._$ganttViewTable.append($colsgroup);

            proxy._createTooltipTemplate();
            proxy._createEditingTooltipTemplate();
            proxy._createProgressbarTooltipTemplate();

            proxy._createTaskbarTemplate();
            proxy._createConnectorLineTemplate();

            proxy._updateCurrentViewData();

            //Display weekends
            if (proxy.model.highlightWeekends == true
                && model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Week)
                proxy._renderWeekends();

            proxy._$stripLineContainer.empty();
            //Display striplines
            if (proxy.model.stripLines != null) {
                this._renderStripLines(proxy.model.stripLines);
            }

            proxy._$secondaryCanvas.empty();
            //Display Holidays
            if (proxy.model.holidays != null) {
                var holidayCount = proxy.model.holidays.length;
                for (var i = 0; i < holidayCount; i++) {
                    proxy._renderHoliday(proxy.model.holidays[i]);
                }
            }
        },

        //Rendering connectorlines from Gantt
        renderConnectorLines: function (connectorLinesCollection) {
            var proxy = this,
                connectorLineContainer = $.render[proxy._id + "ConnectorLineTemplate"](connectorLinesCollection);

            $(proxy._$dependencyViewContainer).empty();
            proxy._$dependencyViewContainer.append(connectorLineContainer);
            proxy._$bodyContent.append(proxy._$dependencyViewContainer);
        },

        //Empty the entire connectorline collection
        clearConnectorLines: function () {
            $(this._$dependencyViewContainer).empty();
        },

        //append single connectorline
        appendConnectorLine: function (connectorLineCollection) {
            var proxy = this,
            connectorLineContainer = $.render[proxy._id + "ConnectorLineTemplate"](connectorLineCollection);
            proxy._$dependencyViewContainer.append(connectorLineContainer);
        },
        //set Collapsed record Count

        setCollapsedRecordCount:function(count)
        {
            this._totalCollapsedRecordCount = count;
        },
        //get collapsed record count
        getCollapsedRecordCount:function()
        {
            return this._totalCollapsedRecordCount;
        },
        //remove a particular connector line
        removeConnectorline: function (lineId) {
            $("#ConnectorLine" + lineId).remove();
        },

        //Events
        _wireEvents: function () {
            var proxy = this,
                matched = jQuery.uaMatch(navigator.userAgent);

            //$(proxy._$bodyContainer).bind("scroll", $.proxy(proxy._handleScroll, proxy));

            proxy._on(proxy.element, "mouseenter", ".e-gantt-milestone-container, .e-childContainer, .e-parentContainer", this._mouseHover);//.gantt-viewer-bodyContianer tr td
            proxy._on(proxy.element, "mouseleave", ".e-gantt-milestone-container, .e-childContainer, .e-parentContainer", this._mouseLeave);//.gantt-viewer-bodyContianer tr td
            proxy._on(proxy.element, "click", proxy._cellClickHandler);

            proxy._on(proxy.element, "mouseenter", ".taskbarresizer-left", proxy._resizerleftOver);
            proxy._on(proxy.element, "mouseleave", ".taskbarresizer-left", proxy._resizerleftLeave);

            proxy._on(proxy.element, "mouseenter", ".taskbarresizer-right", proxy._resizerrightOver);
            proxy._on(proxy.element, "mouseleave", ".taskbarresizer-right", proxy._resizerrightLeave);

            proxy._on(proxy.element, "mouseenter", ".progressbarresizer-right", proxy._progressresizerOver);
            proxy._on(proxy.element, "mouseleave", ".progressbarresizer-right", proxy._progressresizerLeave);

            proxy._on(proxy.element, "mouseenter", ".e-connectorpoint-left", proxy._dragLeftOver);
            proxy._on(proxy.element, "mouseleave", ".e-connectorpoint-left", proxy._dragLeftLeave);

            proxy._on(proxy.element, "mouseenter", ".e-connectorpoint-right", proxy._dragRightOver);
            proxy._on(proxy.element, "mouseleave", ".e-connectorpoint-right", proxy._dragRightLeave);
          

            if (matched.browser.toLowerCase() != "msie") {
                proxy._on(proxy.element, "mousedown", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
            }
            else {
                proxy._on(proxy.element, "mousedown", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
            }

            $(proxy._$bodyContainer).bind("mouseup", $.proxy(proxy._handleMouseUp, proxy));
            $(document).bind("mouseup", $.proxy(proxy._tooltipMouseup, proxy));

            //if (matched.browser.toLowerCase() == "msie") {
            //    proxy._on(proxy.element, "MSPointerDown", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
            //    $(document.body).bind("MSPointerMove", function (event) {
            //        event.preventDefault();
            //    });
            //    $(document.body).bind("MSPointerMove", $.proxy(proxy.handleMouseMove, proxy));
            //    $(proxy._$bodyContainer).bind("MSPointerUp", $.proxy(proxy._handleMouseUp, proxy));
            //}


            if (matched.browser.toLowerCase() == "chrome") {
                proxy._on(proxy.element, "touchstart", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._mousedown);
                $(document.body).bind("touchmove", function (event) {
                    event.preventDefault();
                });
                $(document.body).bind("touchmove", $.proxy(proxy.handleMouseMove, proxy));

                proxy._on(proxy.element, "touchend", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                proxy._on(proxy.element, "touchleave", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                $(proxy._$bodyContainer).bind("touchend", $.proxy(proxy._handleMouseUp, proxy));
                $(proxy._$bodyContainer).bind("touchleave", $.proxy(proxy._handleMouseUp, proxy));
            }

            proxy._on(proxy.element, "mouseenter", ".e-stripline", proxy._stripLineMouseOver);
            proxy._on(proxy.element, "mouseleave", ".e-stripline", proxy._stripLineMouseLeave);
            proxy._enableEditingEvents();
        },

        //Event append for editDialogOpen while perform double click action
        _enableEditingEvents: function () {

            var proxy = this,
                model = proxy.model;

            if (model.editSettings.allowEditing) {
                if (model.editSettings.editMode === "normal") {
                    proxy._on(this.element, "dblclick", ".e-ganttviewerbodyContent", proxy._editdblClickHandler);
                }
            }
        },

        //Double click event handler for editing with editMode as normal
        _editdblClickHandler: function () {

            var proxy = this,
                args = {};

            if (proxy.model.editSettings.editMode === "normal") {
                args.requestType = ej.TreeGrid.Actions.BeginEdit;
                proxy._trigger("actionBegin", args);
            }

        },


        //Event to append RowHover effect
        _enableRowHover: function () {
            var proxy = this;
            if (proxy.model.enableRowHover)
                proxy._on(proxy.element, "mouseleave mouseenter",
                    "#ganttviewerbodyContent,.e-ganttviewerbodyContent tr,.e-ganttviewerbodyContent" + proxy._id,
                    proxy._rowHover);
            else
                proxy._on(proxy.element, "mouseleave mouseenter",
                    "#ganttviewerbodyContent,.e-ganttviewerbodyContent tr,.e-ganttviewerbodyContent" + proxy._id);
        },

        //MouseLeave event for striplines to remove tooltip
        _stripLineMouseLeave: function () {
            window.clearTimeout(this._tooltipTimer);
            this._clearTooltip();
        },

        //mouseHover event for stripline to render tooltip
        _stripLineMouseOver: function (e) {

            e.target.style.cursor = "pointer";
            var proxy = this, model = proxy.model,
                $target = $(e.target),
                parentNode, match,
                tooltiptable,
                tooltipbody,
                posx = 0, posy = 0;
            proxy._clearTooltip();
            if ($target.hasClass("e-striplinespan")) {
                parentNode = $target[0].parentNode;
            } else
                parentNode = $target[0];

            match = parentNode.id.match(/(\d+|[A-z]+)/g);

            if (e.type == "mouseenter" || e.type == "mousemove") {
                if (!e) e = window.event;
                if (e.originalEvent.pageX || e.originalEvent.pageY) {
                    posx = e.originalEvent.pageX;
                    posy = e.originalEvent.pageY;
                }
                else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                    posx = e.originalEvent.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                    posy = e.originalEvent.clientY + document.body.scrollTop
                        + document.documentElement.scrollTop;
                }

                tooltiptable = ej.buildTag("table.e-tooltiptable", "", { "width": "auto" },
                    { 'cellspacing': '2px', 'cellpadding': '2px' });
                tooltipbody = ej.buildTag("tbody",
                    "<tr><td>" + model.stripLines[match[1]].day + "</td></tr><tr><td>" +
                        model.stripLines[match[1]].label + "</td></tr>", {}, {});

                tooltiptable.append(tooltipbody);
                if (model.tooltipTemplate == null) {
                    proxy._mouseHoverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                        { "top": (posy) + "px", "left": (posx + 15) + "px", 'position': 'absolute', 'z-index': '6' }, {});
                } else {
                    if (typeof (model.tooltipTemplate) != "object") {
                        var parser = $.parseHTML(model.tooltipTemplate);
                        proxy._mouseHoverTooltip = parser[0];
                    }
                    else {
                        var markup = $(model.tooltipTemplate)[0].innerHTML;
                        parser = $.parseHTML(markup);
                        proxy._mouseHoverTooltip = parser[1];
                    }

                    $(proxy._mouseHoverTooltip).css({
                        "left": (posx + 15) + "px", "top": (posy) + "px",
                        "position": "absolute"
                    });
                    $(proxy._mouseHoverTooltip).addClass("customTooltip");
                    proxy._mouseHoverTooltip.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";
                }
                setTimeout(function () {
                    $(document.body).append(proxy._mouseHoverTooltip)
                }, 700);
            }
        },

        //Mouseup over tooltip 
        _tooltipMouseup: function () {
            var proxy = this;
            if (proxy._mouseTarget != null) {
                proxy._handleMouseUp(proxy._mouseTarget);
            }
        },

        //removes all the tooltip in ganttchart
        _clearTooltip: function () {
            var proxy = this;
            proxy._mouseHoverTooltip && $(proxy._mouseHoverTooltip).remove();
            proxy._taskbarEditingTooltipID && $(proxy._taskbarEditingTooltipID).remove();
            proxy._progressBarTooltipID && $(proxy._progressBarTooltipID).remove();
            $('.e-editingtooltip').remove();
            $("#tooltipgantt").remove();
            $(".e-progressbartooltip").remove();
            proxy._mouseHoverTooltip = null;

        },

        //mousewheel action for zooming ganttchart
        _mouseWheel: function (e) {
            if (this.model.allowZooming) {
                var proxy = this, args = {},
                    delta = null;

                e = e.originalEvent;
                e.preventDefault();

                if (e.wheelDelta)
                    delta = -e.wheelDelta / 120;
                else if (e.detail)
                    delta = e.detail / 3;

                if (!delta)
                    return;
                args.requestType = "scroll";
                args.delta = delta;
                proxy._enbleZooming(args);
            }
        },


        //mouseHover event on taskbar right helper element
        _resizerrightOver: function (e) {

            var proxy = this,
                $target = e.target,
                div = $target.parentNode;

            if (proxy.model.allowGanttChartEditing == true) {
                if (this._mouseDown == false)
                    e.target.style.cursor = "w-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for taskbar right helper element
        _resizerrightLeave: function () {

            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        //mouseHover event on taskbar left helper element
        _resizerleftOver: function (e) {

            var proxy = this,
                $target = e.target,
                div = $target.parentNode;
            if (proxy.model.allowGanttChartEditing == true) {
                if (this._mouseDown == false)
                    e.target.style.cursor = "e-resize";
                proxy._setEditingElements(div);
            }
        },


        //mouseLeave event for taskbar left helper element
        _resizerleftLeave: function () {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        //mouseHover event on taskbar progressbar helper element
        _progressresizerOver: function (e) {
            var proxy = this,
                $target = e.target,
                div = $target.parentNode;
            if (proxy.model.allowGanttChartEditing == true) {
                if (proxy.model.enableProgressBarResizing){
                    if (proxy._mouseDown == false)
                        $target.style.cursor = "col-resize";
                    proxy._setEditingElements(div);
                }
                else
                    $target.style.cursor = "move";
            }
        },


        //mouseLeave event for progressbar helper element
        _progressresizerLeave: function () {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true) {
                proxy._removeEditingElements();
            }
        },


        _secondMouseOverConnectorPoint: function (e, mouseUp)
        {
            var proxy = this, row, recordIndexr, item;

            if ((proxy._mouseDown && proxy._falseLine) || (mouseUp && proxy._falseLine)) {

                var $target = $(e.target),
                  $ganttGridRows = proxy.getGanttChartRows(),
                  row = $target.closest('tr'),
                  recordIndexr = $ganttGridRows.index(row),
                  item = proxy.model.currentViewData[recordIndexr];

                var fromItem = proxy._editingItem;
                var toItem = item, predecessor, fullPredecessor, currentTarget;

                if (proxy._prevItem.predecessorTarget === 'Start') {
                    predecessor = fromItem.taskId + 'S';
                } else if (proxy._prevItem.predecessorTarget === 'Finish') {
                    predecessor = fromItem.taskId + 'F';
                }

                if ($target.hasClass('e-connectorpoint-left')) {
                    predecessor += 'S';
                    currentTarget = "Start"
                }
                else if ($target.hasClass('e-connectorpoint-right')) {
                    predecessor += 'F';
                    currentTarget = "Finish"
                }

                //Append second item in predecessor tooltip

                $table = proxy._predecessorTooltip.find("table");
                $td = $table.find("tbody").find("tr:eq(1)").find("td:eq(2)").text(toItem.taskName);
                $td = $table.find("tbody").find("tr:eq(1)").find("td:eq(3)").text(currentTarget);
                var args = {};
                args.requestType = "validatePredecessor";
                args.fromItem = fromItem;
                args.toItem = toItem;

                if (args.toItem.item[proxy.model.predecessorMapping]) {
                    fullPredecessor = args.toItem.item[proxy.model.predecessorMapping] + "," + predecessor;
                }
                else {
                    fullPredecessor = predecessor;
                }
                args.predecessor = predecessor;
                args.currentRecord = toItem;
                args.predecessorString = [];
                args.predecessorString.push(fullPredecessor);
                var validation = proxy._completeAction(args);
                $td = $table.find("tbody").find("tr:eq(0)").find("td:eq(0)");
                if (validation) {
                   // $td.text("True");
                    $td.removeClass();
                    $td.addClass("e-predecessor-true");
                } else {
                  //  $td.text("False");
                    $td.removeClass();
                    $td.addClass("e-predecessor-false");
                }

                //Here also update on moueup//same code is required
                if (mouseUp) {
                    return args;
                }
                
            }
        },
        //mouse enter event for left-connectorpoint-point

        _dragLeftOver:function(e)
        {
            var proxy = this,
              $target = e.target,
              div = $target.parentNode;
            if (proxy.model.predecessorMapping && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing) {
                if (this._mouseDown == false ) {
                    $target.style.cursor = "pointer";
                }
                proxy._setEditingElements(div);
            }
            if(!$(div).hasClass('e-parentContainer'))
                proxy._secondMouseOverConnectorPoint(e);

        },
        //mouse leave event for left-connectorpoint-point
        _dragLeftLeave: function (e) {

            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true && proxy.model.enablePredecessorEditing) {
                proxy._removeEditingElements();
            }
        },

        //mouse enter event for right-connectorpoint-point

        _dragRightOver: function (e) {
            var proxy = this,
             $target = e.target,
             div = $target.parentNode;
            if (proxy.model.predecessorMapping && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing) {
                if (this._mouseDown == false) {
                    $target.style.cursor = "pointer";
                }
                proxy._setEditingElements(div);
            }
            if (!$(div).hasClass('e-parentContainer'))
                proxy._secondMouseOverConnectorPoint(e);
        },
        //mouse leave event for right-connectorpoint-point
        _dragRightLeave: function (e) {
            var proxy = this;
            if (proxy.model.allowGanttChartEditing == true && proxy.model.enablePredecessorEditing) {
                proxy._removeEditingElements();
            }
        },




        //adding styles for taskbar helper elements
        _setEditingElements: function (div) {
            var proxy = this, progressHandler;

            if (proxy._editingTarget && !proxy._editPredecessor && $(div).find(proxy._editingTarget).length == 0)
                return true;

            if ($(div).hasClass("e-childContainer")) {
                proxy._leftResizerGripper = div.childNodes[1],//[0]
                proxy._rightResizerGripper = div.childNodes[5];//[4]
                progressHandler = div.childNodes[4];//[3]
                proxy._progressHandle = progressHandler.firstChild;
                proxy._progressHandleChild = progressHandler.childNodes[1];

                $(proxy._leftResizerGripper).addClass("gripper");
                $(proxy._rightResizerGripper).addClass("gripper");

                if (proxy.model.progressMapping) {
                    $(proxy._progressHandle).addClass("e-progresshandle");
                    $(proxy._progressHandleChild).addClass("e-progresshandleafter");
                }
            }


            if (proxy.model.enablePredecessorEditing) {
                if ($(div).hasClass("e-childContainer") && proxy.model.predecessorMapping && !proxy._falseLine) {

                    proxy._leftConnectorPoint = div.childNodes[0];
                    proxy._rightConnectorPoint = div.childNodes[6];
                    $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-gantt-milestone-container") && proxy.model.predecessorMapping && !proxy._falseLine) {

                    proxy._leftConnectorPoint = div.childNodes[0];
                    proxy._rightConnectorPoint = div.childNodes[2];
                    $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-childContainer") && proxy.model.predecessorMapping && proxy._falseLine) {
                    proxy._childLeftConnectorPoint = div.childNodes[0];
                    proxy._childRightConnectorPoint = div.childNodes[6];
                    $(proxy._childLeftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._childRightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-gantt-milestone-container") && proxy.model.predecessorMapping && proxy._falseLine) {
                    proxy._childLeftConnectorPoint = div.childNodes[0];
                    proxy._childRightConnectorPoint = div.childNodes[2];
                    $(proxy._childLeftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._childRightConnectorPoint).addClass("e-connectorpoint-hover");
                }
                else if ($(div).hasClass("e-parentContainer") && proxy.model.predecessorMapping && !proxy._falseLine) {

                    proxy._leftConnectorPoint = div.childNodes[0];
                    proxy._rightConnectorPoint = div.childNodes[3];
                    $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                    $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");

                }
            }
        },


        //removing styles for taskbar helper elements
        _removeEditingElements: function () {
            var proxy = this;
            $(proxy._leftResizerGripper).removeClass("gripper");
            $(proxy._rightResizerGripper).removeClass("gripper");

            if (proxy.model.progressMapping) {
                $(proxy._progressHandle).removeClass("e-progresshandle");
                $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
            }


            //Remove Connector points in chart
            if (proxy.model.enablePredecessorEditing) {
                if (proxy.model.predecessorMapping && proxy._mouseDown == false) {

                    $(proxy._leftConnectorPoint).removeClass('e-connectorpoint-hover');
                    $(proxy._rightConnectorPoint).removeClass('e-connectorpoint-hover');
                    proxy._leftConnectorPoint = null;
                    proxy._rightConnectorPoint = null;
                }

                if (proxy.model.predecessorMapping && proxy._leftConnectorPoint !== null && proxy._mouseDown == true || proxy._childLeftConnectorPoint) {
                    $(proxy._childLeftConnectorPoint).removeClass("e-connectorpoint-hover");
                    $(proxy._childRightConnectorPoint).removeClass("e-connectorpoint-hover");
                    //remove curret target predecessor in tooltip    
                    if (proxy._predecessorTooltip) {
                        $table = proxy._predecessorTooltip.find("table");
                        $table.find("tbody").find("tr:eq(1)").find("td:eq(2)").text("");
                        $table.find("tbody").find("tr:eq(1)").find("td:eq(3)").text("");
                        $td = $table.find("tbody").find("tr:eq(0)").find("td:eq(0)").removeClass();
                        //$td.text("False");
                        $td.addClass('e-predecessor-false');
                    }
                }
            }
        },


        refreshHeight:function(){

            var proxy = this;
            proxy._$bodyContainer.ejScroller("refresh");
        },

        //mousedown event for ganttchart
        _mousedown: function (e) {
            if (e.which == 2 || e.which == 3) //2 for mouseWheel, 3 for rightClick button
                return false;
            var proxy = this;
            proxy._clearTooltip();
            proxy._mouseDown = true;
            args = {};
            //proxy._clearContextMenu();
            var $target = $(e.target),
            $ganttGridRows = proxy.getGanttChartRows(),
            row = $target.closest('tr'),
            recordIndexr = $ganttGridRows.index(row),
            item = proxy.model.currentViewData[recordIndexr],
            parentItem = item.parentItem,
            div = $target[0].parentNode,
            matched = jQuery.uaMatch(navigator.userAgent),
            model = this.model;

            if (ej.isNullOrUndefined(item)) {
                return;
            }
            proxy._editingItem = item;
            proxy._editingTarget = $target;

            if ($(div).hasClass("e-gantt-childtaskbar-progress") ||
                $(div).hasClass("progressbarresizer-right")) {
                div = div.parentNode;
            }

            if ($(div).hasClass("e-childContainer") ||
                ($(div).hasClass("e-gantt-milestone")) || $(div).hasClass('e-parentContainer') || $(div).hasClass("e-gantt-milestone-container")) {

                $(row).addClass("e-editmode");
                proxy._mouseTarget = e;
            }

            if (proxy.model.allowGanttChartEditing && ($(div).hasClass("e-childContainer")
                || ($(div).hasClass("e-gantt-milestone")) || $(div).hasClass("e-gantt-milestone-container") || $(div).hasClass('e-parentContainer'))) { //mouse down on drag points in child and milestone container and parent container also included 

                if (e.target.style.cursor == "e-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))
                    || (e.type == "touchstart" && e.target.className == "taskbarresizer-left" && matched.browser.toLowerCase() == "chrome")) {
                    proxy._updateEditingType();
                    proxy._leftResizing = true;
                }

                if (e.target.style.cursor == "w-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))
                    || (e.type == "touchstart" && e.target.className == "taskbarresizer-right" && matched.browser.toLowerCase() == "chrome")) {
                    proxy._updateEditingType();
                    proxy._rightResizing = true;

                }
                if (e.target.style.cursor == "col-resize" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))
                    || (e.type == "touchstart" && e.target.className == "progressbarresizer-right" && matched.browser.toLowerCase() == "chrome")) {
                    proxy._updateEditingType();
                    proxy._progressResizing = true;
                }
                if (e.target.style.cursor == "move" && ($(div).hasClass("e-childContainer")
                    || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._allowDragging = true;
                }
                if (e.target.style.cursor == "" && ($(div).hasClass("e-childContainer")
                  || ($(div).hasClass("e-gantt-milestone")))) {
                    proxy._updateEditingType();
                    proxy._allowDragging = true;
                }

                // ON drag points initiate the false line and edit type for predecessor

                if (proxy.model.enablePredecessorEditing) {
                    if (e.target.style.cursor == "pointer" && ($(div).hasClass("e-childContainer")
                     || ($(div).hasClass("e-gantt-milestone")) || $(div).hasClass("e-gantt-milestone-container") || $(div).hasClass('e-parentContainer'))) {
                        proxy._updateEditingType();
                        proxy._editPredecessor = true;
                        if ($target.hasClass('e-connectorpoint-left')) {
                            proxy._prevItem.predecessorTarget = 'Start';
                        }
                        else if ($target.hasClass('e-connectorpoint-right')) {
                            proxy._prevItem.predecessorTarget = 'Finish';
                        }
                    }
                }




                if (!e) e = window.event;
                if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                    e.originalEvent.pageX = e.originalEvent.clientX;
                    e.originalEvent.pageY = e.originalEvent.clientY;
                }

                if (e.originalEvent.pageX || e.originalEvent.pageY) {
                    proxy._currMousePosX = e.originalEvent.pageX;
                    proxy._currMousePosY = e.originalEvent.pageY;
                    proxy.dragPosX = e.originalEvent.pageX;
                }
                else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                    proxy.dragPosX = e.originalEvent.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                }
                else if (e.originalEvent && e.originalEvent.changedTouches
                    && e.originalEvent.changedTouches.length > 0) {
                    proxy._currMousePosX = e.originalEvent.changedTouches[0].pageX;
                    proxy._currMousePosY = e.originalEvent.changedTouches[0].pageY;
                    proxy.dragPosX = e.originalEvent.changedTouches[0].pageX;
                }

                proxy._prevItem.left = item.left;
                proxy._prevItem.status = item.status;
                proxy._prevItem.duration = item.duration;
                proxy._prevItem.startDate = item.startDate;
                proxy._prevItem.endDate = item.endDate;
                proxy._prevItem.width = item.width;

                proxy._parentCollectionPreviousValues = proxy._getParentCollectionPreviousValues(item);
                //For false line starting point
                if (proxy.model.enablePredecessorEditing) {
                    proxy._connectorPointX = proxy._currMousePosX - proxy._$bodyContent.offset().left;
                    proxy._connectorPointY = proxy._currMousePosY - proxy._$bodyContent.offset().top;
                }


                $(proxy._$bodyContainer).bind("mousemove", $.proxy(proxy.handleMouseMove, proxy));

                if (matched.browser.toLowerCase() == "chrome") {

                    proxy._on(proxy.element, "touchend", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                    proxy._on(proxy.element, "touchleave", ".e-tasklabel,.e-parentContainer,.e-childContainer,.e-gantt-milestone-container", proxy._handleMouseUp);
                    $(proxy._$bodyContainer).bind("touchend", $.proxy(proxy._handleMouseUp, proxy));
                    $(proxy._$bodyContainer).bind("touchleave", $.proxy(proxy._handleMouseUp, proxy));
                    $(document.body).bind("touchend", $.proxy(proxy._handleMouseUp, proxy));
                    $(document.body).bind("touchleave", $.proxy(proxy._handleMouseUp, proxy));
                }
            }

            if (($(div).hasClass("e-parentContainer")) && !$target.hasClass("e-connectorpoint-left") && !$target.hasClass("e-connectorpoint-right")) { // here check drag point class
                if (proxy._allowExpandCollapse && item && item.hasChildRecords) {
                    recordIndex = model.updatedRecords.indexOf(item);

                    if (model.selectedRowIndex !== recordIndex && model.allowSelection) {

                        if (!proxy._rowSelectingEventTrigger(model.selectedRowIndex, recordIndex)) {
                            proxy.selectRows(recordIndex);
                            proxy._rowSelectedEventTrigger(recordIndex);
                        }
                    }
                    proxy._expandCollapse(recordIndexr);
                }
            } else {
                proxy._appendTooltip($target, recordIndexr);
            }
            proxy._trigger("clearColumnMenu");
        },

        //Get previous parent item values
        _getParentCollectionPreviousValues: function (item) {
            var record = item,
                parentPrevItem = {},
                allParentRecords = [];
            do {               
                parentItem = record.parentItem;
                if (!ej.isNullOrUndefined(parentItem)) {
                    parentPrevItem.left = parentItem.left;
                    parentPrevItem.status = parentItem.status;
                    parentPrevItem.duration = parentItem.duration;
                    parentPrevItem.startDate = parentItem.startDate;
                    parentPrevItem.endDate = parentItem.endDate;
                    parentPrevItem.width = parentItem.width;
                    parentPrevItem.index = parentItem.index;
                    allParentRecords.push(parentPrevItem);
                    record = parentItem;
                    parentPrevItem = {};
                }
            } while (record.parentItem);
            return allParentRecords;
        },

        handleMouseMove: function (e) {
            this._updateOnmouseMove(e);

        },


        //Handler for MouseUp event in ganttchart
        _handleMouseUp: function (e) {

            var proxy = this,
                args = {},
                $ganttGridRows = proxy.getGanttChartRows(),
                $target, row, recordIndexr = 0, item;

            
            //Calculate and validate predecessor and draw connectorline after second connector point connects
            if (($(e.target).hasClass('e-connectorpoint-left') || $(e.target).hasClass('e-connectorpoint-right')) && proxy._falseLine && !$(e.target).parent().hasClass('e-parentContainer')) {
                var newArgs = proxy._secondMouseOverConnectorPoint(e, true);
                if (newArgs.predecessorValidation) {
                    newArgs.requestType = "drawConnectorLine";
                    proxy._completeAction(newArgs);
                }
            }

            if (proxy._editingTarget != null && proxy._editingItem != null) {
                $target = proxy._editingTarget,
                row = $target.closest('tr'),
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem;
                if ($(row).hasClass("e-editmode")) {
                    $(row).removeClass("e-editmode");
                    proxy._mouseTarget = null;
                }
            }


            proxy._editingTarget = null;
            proxy._editingItem = null;

            proxy._clearTooltip();
            proxy._$ganttChartContainer[0].style.cursor = "auto";

            proxy._allowExpandCollapse = true;

            if (item != null && (proxy._allowDragging || proxy._leftResizing || proxy._rightResizing || proxy._progressResizing)) {
                args.editingFields = { startdate: item.startDate, enddate: item.endDate, progress: item.status, duration: item.duration };
                args.data = item;
                args.previousData = proxy._prevItem;
                args.dragging = proxy._allowDragging;
                args.leftResizing = proxy._leftResizing;
                args.rightResizing = proxy._rightResizing;
                args.progressResizing = proxy._progressResizing;
                proxy._taskbarEdited(args);
            }

            if (args.cancel) {
                proxy._getPrevItem(item, recordIndexr);
                if (item.parentItem)
                    proxy._getParentPreviousValue();
                if (proxy.model.predecessorMapping) {
                    args.requestType = "updateConnectors";
                    proxy._trigger("actionComplete", args);
                }
            }

            $("#ganttviewerbodyContianer" + proxy._id).unbind("mousemove");
            proxy._updateEditingType();
            proxy._mouseDown = false;

            //Remove drag point and false lines on mouse up
            proxy._falseLine && proxy._falseLine.remove();
            proxy._falseLine = null;
            proxy._predecessorTooltip && proxy._predecessorTooltip.remove();
            proxy._predecessorTooltip = null;
            $(proxy._leftConnectorPoint).removeClass("e-connectorpoint-hover");
            $(proxy._rightConnectorPoint).removeClass("e-connectorpoint-hover");
            $(proxy._childRightConnectorPoint).removeClass("e-connectorpoint-hover");
            $(proxy._childLeftConnectorPoint).removeClass("e-connectorpoint-hover");
            proxy._leftConnectorPoint = null;
            proxy._rightConnectorPoint = null;
            proxy._childRightConnectorPoint = null;
            proxy._childLeftConnectorPoint = null;
            proxy._trigger("clearColumnMenu");
        },

        getChildCount: function (record, count) {
            var currentRecord, proxy = this;
            if (!record.hasChildRecords)
                return 0;
            for (var i = 0; i < record.childRecords.length; i++) {
                currentRecord = record.childRecords[i];
                count++;
                if (currentRecord.hasChildRecords) {
                    count = proxy.getChildCount(currentRecord, count);
                }
            }
            return count;
        },

        //get the Expand status of GridRecord
        _getCollapsedParentItem: function (record) {
            var parentRecord = record.parentItem;
            if (parentRecord) {
                if (this._getExpandStatus(parentRecord) === true) {
                    return parentRecord;
                } else {
                    return this._getCollapsedParentItem(parentRecord)
                }
            }
            else
            {
                return null;
            }
        },
        /* scroll to selected record */
        updateScrollBar: function () {
            var proxy = this,
                model = proxy.model, horizontalScrollBarHeight,
                currentUpdatedRecords, recordIndex,
                isInViewPortAbove, isInViewPortBelow;

            currentUpdatedRecords = proxy.getExpandedRecords(model.updatedRecords);
            recordIndex = currentUpdatedRecords.indexOf(model.selectedItem);

            if (!model.selectedItem || recordIndex == -1)
                return;

            var rowTop = recordIndex * model.rowHeight,
                rowBottom = rowTop + model.rowHeight,
                scrollTop,
                containerWidth,
                scrollerExist = proxy._$bodyContainer.children(".e-content").length;

            if (scrollerExist)
                scrollTop = proxy._$bodyContainer.children(".e-content").scrollTop();
            else
                scrollTop = proxy._$bodyContainer.scrollTop();

            if (proxy._$bodyContainer.ejScroller("isHScroll") || proxy.element.find(".e-borderbox").length > 0) {
                horizontalScrollBarHeight = 18; /* default horizontal scrollbar height*/
            }

            isInViewPortAbove = (rowTop < scrollTop);
            isInViewPortBelow = (scrollTop + proxy._viewportHeight - horizontalScrollBarHeight) < rowBottom;

            if (isInViewPortAbove || isInViewPortBelow) {

                if ((rowTop + proxy._viewportHeight) > currentUpdatedRecords.length * model.rowHeight) {
                    rowTop = currentUpdatedRecords.length * model.rowHeight - proxy._viewportHeight;

                    if (isInViewPortBelow)
                        rowTop += horizontalScrollBarHeight;
                }
                proxy._$bodyContainer.ejScroller("scrollY", rowTop, true);

                if (model.enableVirtualization) {
                    proxy.element[0].focus();
                }
            }
        },

        //In virtualization get expaned records from total record collection only
        getExpandedRecords: function (records) {
            var proxy = this;
            var resultRecord = records.filter(function (record) {
                return proxy._getExpandStatus(record) == true;
            });
            return resultRecord;
        },


        //keyPressed event for perform the keynavigate selection in gantt
        _keyPressed: function (action, target, arg, e) {

            var proxy = this,
                model = this.model,
                selectingRowIndex,
                updatedRecords = model.updatedRecords,
                expandedRecords = proxy.getExpandedRecords(updatedRecords),
                currentSelectingRecord;

            e.stopImmediatePropagation();
            e.preventDefault();

            if (!model.allowKeyboardNavigation)
                return true;

            switch (action) {

                case "downArrow":
                    if (updatedRecords.length > 0 && model.selectedItem) {
                        //get the last row index of the ganttchart control
                        lastRowIndex = updatedRecords.length - 1;
                        if (model.selectedRowIndex != lastRowIndex) {

                            selectingRowIndex = expandedRecords.indexOf(model.selectedItem);
                            currentSelectingRecord = expandedRecords[selectingRowIndex + 1];
                            selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);

                            if (currentSelectingRecord && selectingRowIndex <= lastRowIndex && !proxy._rowSelectingEventTrigger(model.selectedRowIndex, selectingRowIndex)) {
                                proxy.selectRows(selectingRowIndex);
                                proxy.element[0].focus();
                                proxy.updateScrollBar();
                                proxy._rowSelectedEventTrigger(model.selectedRowIndex);
                            }
                        }
                    }
                    break;

                    //select the previous row by press down arrow key
                case "upArrow":
                    if (model.selectedRowIndex !== 0 && updatedRecords.length > 0 && model.selectedItem) {

                        selectingRowIndex = expandedRecords.indexOf(model.selectedItem);
                        currentSelectingRecord = expandedRecords[selectingRowIndex - 1];
                        selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);

                        if (!proxy._rowSelectingEventTrigger(model.selectedRowIndex, selectingRowIndex)) {
                            proxy.selectRows(selectingRowIndex);
                            proxy.element[0].focus();
                            proxy.updateScrollBar();
                            proxy._rowSelectedEventTrigger(model.selectedRowIndex);
                        }
                    }
                    break;

                    //select the first row while press the Home key
                case "firstRowSelection":
                    if (updatedRecords.length > 0) {
                        if (!proxy._rowSelectingEventTrigger(model.selectedRowIndex, 0)) {
                            proxy.selectRows(0);
                            proxy.element[0].focus();
                            proxy.updateScrollBar();
                            proxy._rowSelectedEventTrigger(0);
                        }
                    }
                    break;

                case "lastRowSelection":

                    if (updatedRecords.length > 0) {

                        lastRowIndex = expandedRecords.length - 1;
                        currentSelectingRecord = expandedRecords[lastRowIndex];
                        selectingRowIndex = updatedRecords.indexOf(currentSelectingRecord);

                        if (!proxy._rowSelectingEventTrigger(model.selectedRowIndex, selectingRowIndex)) {
                            proxy.selectRows(selectingRowIndex);
                            proxy.element[0].focus();
                            proxy.updateScrollBar();
                            proxy._rowSelectedEventTrigger(selectingRowIndex);
                        }
                    }
                    break;

                // expand the child task by press right arrow
                case "rightArrow":
                    var rowIndex = proxy.model.selectedRowIndex,
                    record = proxy.model.updatedRecords[rowIndex],
                    args = {},
                    isExpandCollapseEnabeled;
                    args.data = record;
                    args.recordIndex = rowIndex;
                    args.expanded = true;
                    if (record) {
                        if (!proxy._rowSelectingEventTrigger() && record.hasChildRecords && !record.expanded) {
                            if (rowIndex >= 0)
                                isExpandCollapseEnabeled = proxy._trigger("expanding", args);
                            if (model.enableVirtualization)
                                proxy.element[0].focus();
                        }
                    }
                    break;
                //collapse the child task by press left arrow 
                case "leftArrow":
                    var rowIndex = proxy.model.selectedRowIndex,
                        record = proxy.model.updatedRecords[rowIndex],
                        args = {},
                        isExpandCollapseEnabeled;   
                        args.data = record;
                        args.recordIndex = rowIndex;
                        args.expanded = false;
                        if (record) {
                            if (!proxy._rowSelectingEventTrigger() && record.hasChildRecords && record.expanded) {
                                if (rowIndex >= 0)
                                    proxy._trigger("collapsing", args);
                                if (model.enableVirtualization)
                                    proxy.element[0].focus();
                            }
                        }
                    break;
                case "deleteRecord":
                    if (model.editSettings.allowDeleting && model.selectedRowIndex >= 0) {
                        proxy._trigger("deleteRow");
                        proxy.element[0].focus();
                    }
                    break;
                case "totalRowCollapse":
                    args = {};
                    args.requestType = "collapseAll";
                    proxy._trigger("expandAllCollapseAllRequest", args);
                    break;
                case "totalRowExpand":
                    args = {};
                    args.requestType = "expandAll";
                    proxy._trigger("expandAllCollapseAllRequest", args);
                    break;
            }

            return true;
        },

        _rowSelectingEventTrigger: function (previousIndex, recordIndex) {

            var proxy = this,
                model = proxy.model,
                args = {};

            args.previousIndex = previousIndex;
            args.recordIndex = recordIndex;
            return proxy._trigger('rowSelecting', args);
        },

        _rowSelectedEventTrigger: function (index) {
            var proxy = this,
                args;

            args = {
                data: proxy.model.selectedItem,
                target: "ejGanttChart",
                recordIndex: index
            };
            proxy._trigger("rowSelected", args);
        },

        _updateEditingType: function () {
            var proxy = this;
            proxy._leftResizing = false;
            proxy._allowDragging = false;
            proxy._rightResizing = false;
            proxy._progressResizing = false;
            proxy._editPredecessor = false;
        },
        /*Get Number of holidays in between startdate and enddate*/
        _getHolidaysCount: function (startDate, endDate) {
            var proxy = this,
            holidays = this.model.holidays, holidaysCount = 0;

            for (var count = 0; count < holidays.length; count++) {
                if (proxy._getDateFromFormat(holidays[count].day) >= startDate && proxy._getDateFromFormat(holidays[count].day) <= endDate) {
                    holidaysCount += 1;
                }
            }
            return holidaysCount;
        },

        //update offset values for predecessor
        updateConnectorLineOffset: function (item) {

            var parentGanttRecord,
                model = this.model,
                predecessor = item.predecessor,
                predecessorValue = [],
                parentStartDate,
                parentEndDate,
                childStartDate,
                childEndDate,
                includeWeekend = model.includeWeekend,
                weekendOffset, childGanttRecord, dayOffset;

            if (predecessor) {
                for (var i = 0; i < predecessor.length; i++) {
                    if ((predecessor[i].to === item['taskId'].toString())) {

                        parentGanttRecord = model.flatRecords[model.ids.indexOf(predecessor[i].from)];
                        childGanttRecord = model.flatRecords[model.ids.indexOf(predecessor[i].to)];
                        parentStartDate = new Date(parentGanttRecord.startDate);
                        parentEndDate = new Date(parentGanttRecord.endDate);
                        childStartDate = new Date(childGanttRecord.startDate);
                        childEndDate = new Date(childGanttRecord.endDate);

                        if (predecessor[i].predecessorsType == "FS") {

                            if (childGanttRecord.left < (parentGanttRecord.left + parentGanttRecord.width)) {

                                dayOffset = ((parentGanttRecord.left + parentGanttRecord.width)
                                    - childGanttRecord.left) / this.model.perDayWidth;

                                dayOffset = childGanttRecord.isMilestone == true ? (dayOffset - 1) : dayOffset;
                                predecessor[i].offset = Math.round(-dayOffset);
                            }
                            else if (childGanttRecord.left > (parentGanttRecord.left + parentGanttRecord.width)) {
                                dayOffset = (childGanttRecord.left - (parentGanttRecord.left + parentGanttRecord.width))
                                    / this.model.perDayWidth;

                                if (dayOffset >= 0)
                                    predecessor[i].offset = Math.round(dayOffset);
                            }
                            else {
                                predecessor[i].offset = 0;
                            }
                            /*WeekEnd , Holidays deduction*/
                            if (predecessor[i].offset > 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(parentEndDate, new Date(childStartDate))
                                    predecessor[i].offset -= weekEndDays;
                                }
                                predecessor[i].offset -= this._getHolidaysCount(parentEndDate, childStartDate);
                            } else if (predecessor[i].offset < 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childStartDate), parentEndDate)
                                    predecessor[i].offset += weekEndDays;
                                }
                                predecessor[i].offset += this._getHolidaysCount(childStartDate, parentEndDate);
                            }

                        }
                        if (predecessor[i].predecessorsType == "SS") {
                            if (childGanttRecord.left < parentGanttRecord.left) {
                                dayOffset = ((parentGanttRecord.left) - childGanttRecord.left) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(-dayOffset);
                            }
                            else if (childGanttRecord.left > parentGanttRecord.left) {
                                dayOffset = (childGanttRecord.left - (parentGanttRecord.left)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(dayOffset);
                            } else {
                                predecessor[i].offset = 0;
                            }

                            /* holidays and weekend days deduction*/
                            if (predecessor[i].offset > 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(parentStartDate, new Date(childStartDate))
                                    predecessor[i].offset -= weekEndDays;
                                }
                                predecessor[i].offset -= this._getHolidaysCount(parentStartDate, childStartDate);
                            } else if (predecessor[i].offset < 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childStartDate), parentStartDate)
                                    predecessor[i].offset += weekEndDays;
                                }
                                predecessor[i].offset += this._getHolidaysCount(childStartDate, parentStartDate);
                            }
                        }
                        if (predecessor[i].predecessorsType == "SF") {
                            if ((childGanttRecord.left + childGanttRecord.width) < parentGanttRecord.left) {
                                dayOffset = ((parentGanttRecord.left) - (childGanttRecord.left + childGanttRecord.width))
                                    / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(-dayOffset);
                            } else if ((childGanttRecord.left + childGanttRecord.width) > parentGanttRecord.left) {

                                dayOffset = ((childGanttRecord.left + childGanttRecord.width) -
                                    (parentGanttRecord.left)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(dayOffset);
                            }
                            else {
                                predecessor[i].offset = 0;
                            }
                            /* holidays and weekend days deduction*/
                            if (predecessor[i].offset > 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(parentStartDate, new Date(childEndDate))
                                    predecessor[i].offset -= weekEndDays;
                                }
                                predecessor[i].offset -= this._getHolidaysCount(parentStartDate, childEndDate);
                            } else if (predecessor[i].offset < 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childEndDate), parentStartDate)
                                    predecessor[i].offset += weekEndDays;
                                }
                                predecessor[i].offset += this._getHolidaysCount(childEndDate, parentStartDate);
                            }
                        }

                        if (predecessor[i].predecessorsType == "FF") {

                            if ((childGanttRecord.left + childGanttRecord.width) < (parentGanttRecord.left + parentGanttRecord.width)) {
                                dayOffset = ((parentGanttRecord.left + parentGanttRecord.width) -
                                    (childGanttRecord.left + childGanttRecord.width)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(-dayOffset);
                            }
                            else if ((childGanttRecord.left + childGanttRecord.width) > (parentGanttRecord.left + parentGanttRecord.width)) {
                                dayOffset = ((childGanttRecord.left + childGanttRecord.width) -
                                    (parentGanttRecord.left + parentGanttRecord.width)) / this.model.perDayWidth;
                                predecessor[i].offset = Math.round(dayOffset);
                            }
                            else {
                                predecessor[i].offset = 0;
                            }

                            /* holidays and weekend days deduction*/
                            if (predecessor[i].offset > 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(parentEndDate, new Date(childEndDate))
                                    predecessor[i].offset -= weekEndDays;
                                }
                                predecessor[i].offset -= this._getHolidaysCount(parentEndDate, childEndDate);
                            } else if (predecessor[i].offset < 0) {
                                if (!includeWeekend) {
                                    var weekEndDays = this._calculateWeekendDays(new Date(childEndDate), parentEndDate)
                                    predecessor[i].offset += weekEndDays;
                                }
                                predecessor[i].offset += this._getHolidaysCount(childEndDate, parentEndDate);
                            }
                        } 
                        

                        if (predecessor[i].offset < 0) {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType + ""
                                + predecessor[i].offset + "d");
                        }
                        else if (predecessor[i].offset >= 1) {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType + "+"
                                + predecessor[i].offset + "d");
                        }
                        else {
                            predecessorValue.push(predecessor[i].from + "" + predecessor[i].predecessorsType);
                        }
                    }
                }
                if (predecessorValue.length > 0) {
                    item.item[model.predecessorMapping] = predecessorValue.join(',');
                    //this.refreshRow(itemIndex, false);
                }
            }
        },
        _calculateWeekendDays: function (fromDate, toDate)
        {
                var weekendDayCount = 0;
                fromDate = new Date(fromDate);

                while(fromDate < toDate){
                    fromDate.setDate(fromDate.getDate() + 1);
                    if(fromDate.getDay() === 0 || fromDate.getDay() == 6){
                        ++weekendDayCount ;
                    }
                }
                return weekendDayCount ;
        },

        //Returns the previous item properties
        _getPrevItem: function (item, index) {
            var proxy = this;
            item.left = proxy._prevItem.left;
            item.width = proxy._prevItem.width;
            item.startDate = proxy._prevItem.startDate;
            item.endDate = proxy._prevItem.endDate;
            item.duration = proxy._prevItem.duration;
            item.status = proxy._prevItem.status;
            item.progressWidth = this._getProgressWidth(item.width, item.status);
            this.refreshRow(index);
        },

        //Returns the previous parent item properties
        _getParentPreviousValue: function () {
            var proxy = this, prevParent, index,
                model=proxy.model,
                length = proxy._parentCollectionPreviousValues.length;
            for (var i = 0; i < length; i++) {
                prevParent = proxy._parentCollectionPreviousValues[i];
                index = prevParent.index;
                parentItem = model.updatedRecords[index];
                parentItem.left = prevParent.left;
                parentItem.width = prevParent.width;
                parentItem.startDate = prevParent.startDate;
                parentItem.endDate = prevParent.endDate;
                parentItem.duration = prevParent.duration;
                parentItem.status = prevParent.status;
                parentItem.progressWidth = this._getProgressWidth(parentItem.width, parentItem.status);
                this.refreshRow(index);
            }

        },

        //handler for left resizing taskbar item
        _enableLeftResizing: function (e, item) {

            var proxy = this, model = this.model,
                scheduleHeaderType = model.scheduleHeaderSettings.scheduleHeaderType,
                scheduleHeaderValue = ej.Gantt.ScheduleHeaderType;
            e.target.style.cursor = "e-resize";
            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    if ((item.left + 30) < (item.left + item.width)) {
                        item.left -= proxy._mousePosX;
                    }
                    item.width += proxy._mousePosX;
                }
            }
            else {
                proxy._mousePosX -= proxy.dragPosX;
                if ((item.left + 30) < (item.left + item.width)) {
                    item.left += proxy._mousePosX;
                }
                item.width -= proxy._mousePosX;
            }
            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }
            if (scheduleHeaderType == scheduleHeaderValue.Day)
                item.width = item.width < proxy.model.perHourWidth ? proxy.model.perHourWidth : item.width;
            else if (scheduleHeaderType == scheduleHeaderValue.Hour)
                item.width = item.width < proxy.model.perMinuteWidth ? proxy.model.perMinuteWidth : item.width;
            else
                item.width = item.width < proxy.model.perDayWidth ? proxy.model.perDayWidth : item.width;
            item.progressWidth = proxy._getProgressWidth(item.width, item.status);

            proxy._setItemPosition(item);

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },

        //handler for progressbar resizing
        _enableProgressResizing: function (e, item) {
            var proxy = this,
                diff,
                radius;
            e.target.style.cursor = "col-resize";
            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy._progressResizer) {
                proxy._progressHandle = proxy._progressResizer.firstChild;
                proxy._progressHandleChild = proxy._progressResizer.childNodes[1];
            }

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.progressWidth -= proxy._mousePosX;
                }
            }
            else {
                proxy._mousePosX -= proxy.dragPosX;
                item.progressWidth += proxy._mousePosX;
            }
            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }
            item.progressWidth = item.progressWidth > item.width ? item.width : item.progressWidth;
            item.progressWidth = item.progressWidth < 0 ? 0 : item.progressWidth;

            diff = item.width - item.progressWidth;

            if (diff <= 4) {
                radius = 4 - diff;
            }
            else
                radius = 0;

            $(proxy._progressbarItem).css({
                "width": (item.progressWidth) + "px",
                "left": (item.left) + "px",
                "border-top-right-radius": radius + "px",
                "border-bottom-right-radius": radius + "px"
            });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });

            $(proxy._progressHandle).addClass("e-progresshandle");
            $(proxy._progressHandleChild).addClass("e-progresshandleafter");
        },


        //handler for right resizing taskbar
        _enableRightResizing: function (e, item) {

            var proxy = this, model = this.model;
            e.target.style.cursor = "w-resize";

            $(proxy._leftResizer).addClass("gripper");
            $(proxy._rightResizer).addClass("gripper");

            if (proxy.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.width -= proxy._mousePosX;
                }
            } else {
                proxy._mousePosX -= proxy.dragPosX;
                item.width += proxy._mousePosX;
            }
            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }

            if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Day)
                item.width = item.width < proxy.model.perHourWidth ? proxy.model.perHourWidth : item.width;
            else if (model.scheduleHeaderSettings.scheduleHeaderType == ej.Gantt.ScheduleHeaderType.Hour)
                item.width = item.width < proxy.model.perMinuteWidth ? proxy.model.perMinuteWidth : item.width;
            else
                item.width = item.width < proxy.model.perDayWidth ? proxy.model.perDayWidth : item.width;

            item.progressWidth = proxy._getProgressWidth(item.width, item.status);

            $(proxy._taskbarItem).css({ "left": (item.left) + "px", "width": (item.width) + "px" });
            $(proxy._progressbarItem).css({ "width": (item.progressWidth) + "px", "left": (item.left) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({ "left": (item.left) + "px", "width": (item.width) + "px" });

            $(proxy._leftConnectorPoint).css({ "left": (item.left - 10) + "px" });
            $(proxy._rightConnectorPoint).css({ "left": (item.left + item.width) + "px" });

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },


        //Handler for dragging taskbar
        _enableDragging: function (e, item) {
            var proxy = this;
            e.target.style.cursor = "move";

            if ($(e.target.parentNode).hasClass("e-parentContainer")) {
                proxy._allowExpandCollapse = false;
            }

            if (this.dragPosX > proxy._mousePosX) {
                proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                if (item.left > 0) {
                    item.left -= proxy._mousePosX;
                }
            } else {
                proxy._mousePosX -= proxy.dragPosX;
                item.left += proxy._mousePosX;
            }

            if (e.originalEvent.pageX == 0) {
                this.dragPosX = e.originalEvent.targetTouches[0].pageX;
            }
            else {
                this.dragPosX = e.originalEvent.pageX;
            }
            item.left = item.left < 0 ? 0 : item.left;

            $(proxy._progressbarItem).css({ "left": (item.left) + "px" });
            $(proxy._taskbarItem).css({ "left": (item.left) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._leftResizer).css({ "left": (item.left + 2) + "px" });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });

            $(proxy._leftConnectorPoint).css({ "left": (item.left-10) + "px" });
            $(proxy._rightConnectorPoint).css({ "left": (item.left + item.width) + "px" });

            if (proxy._taskbarItem && proxy._taskbarItem.childNodes.length > 0) {
                proxy._taskbarItem.removeChild(proxy._taskbarItem.lastChild);
            }
        },


        //Append tooltip while editing taskbar
        _appendTooltip: function (target, index) {

            var proxy = this, tooltipElement, tooltiptable, tooltipbody,
                $target = $(target),
                model = this.model,
                item = proxy.model.currentViewData[index],
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                predecessorEditingTexts = proxy.model.predecessorEditingTexts;
            if (item != null) {
               
                var div = $target[0];

                if ($(div).hasClass("e-parentContainer") || $(div).hasClass("e-childContainer")) {
                    proxy._editingContainer = div;
                    proxy._leftResizer = proxy._editingContainer.childNodes[0];
                    proxy._taskbarItem = proxy._editingContainer.childNodes[1];
                    proxy._progressbarItem = proxy._editingContainer.childNodes[2];
                    proxy._progressResizer = proxy._editingContainer.childNodes[3];
                    proxy._rightResizer = proxy._editingContainer.childNodes[4];
                }

                //Don't need to display the drag tooltip in Parent Taskbar
                if (this.model.enableTaskbarDragTooltip && (!$target.hasClass('e-parentContainer'))) {

                    if (this._progressResizing == true) {

                        proxy._$ganttChartContainer[0].style.cursor = "col-resize";

                        tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' });
                        tooltipbody = ej.buildTag("tbody", "", {}, {});
                        tooltipbody[0].innerHTML = "<tr><td>" + columnHeaderTexts["status"] + " : " + item.status + "%</td></tr>";
                        tooltiptable.append(tooltipbody);
                        if (proxy.model.progressbarTooltipTemplate) {
                            proxy._progressBarTooltipID = ej.buildTag("div.e-progressbartooltip#progressbartooltip" + proxy._id + "", "",
                              {
                                  'position': 'absolute',
                                  'z-index': '7'
                              }, {});
                            //proxy.tooltipState = "editingTemplateID";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "progressbarTooltipTemplate"](item), {}, {});
                            proxy._progressBarTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                        }
                        else if (proxy.model.progressbarTooltipTemplateId) {
                            proxy._progressBarTooltipID = ej.buildTag("div.e-progressbartooltip#progressbartooltip" + proxy._id + "", "",
                                    {
                                        'position': 'absolute',
                                        'z-index': '7'
                                    }, {});

                            //proxy.tooltipState = "editingTemplate";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "progressbarTooltipTemplate"](item), {}, {});
                            proxy._progressBarTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                        }
                        else {
                            proxy._progressBarTooltipID = ej.buildTag("div.e-progressbartooltip#progressbartooltip" + proxy._id + "",
                                tooltiptable,
                                {
                                    'position': 'absolute',
                                    'z-index': '7'
                                }, { 'onselectstart': 'return false' });
                        }
                        if (proxy._progressBarTooltipID) {
                            $(document.body).append(proxy._progressBarTooltipID);
                            proxy._updateTooltipPosition(proxy._progressBarTooltipID, proxy._currMousePosX, proxy._currMousePosY, true);
                        }
                    }
                    // for apend tooltip for connector line
                    else if (this._editPredecessor === true && proxy.model.enablePredecessorEditing) {
                        if (proxy._predecessorTooltip === null) {
                            proxy._predecessorTooltip = ej.buildTag("div.e-tooltipganttpredecessor#predecessortooltip" + proxy._id + "",
                                      "",
                                      {
                                          "top": (proxy._currMousePosY + 10) + "px",
                                          "left": (proxy._currMousePosX + 10) + "px",
                                          'position': 'absolute',
                                          'z-index': '7'
                                      }, { 'onselectstart': 'return false' });
                            tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' });
                            tooltipbody = ej.buildTag("tbody", "", {}, {});
                            var innerHtml = "<tr><td class='e-predecessor-false'></td><td >"+ predecessorEditingTexts["fromText"] +":</td><td class='e-tooltiptaskname'>" + item.taskName + "</td><td>" + proxy._prevItem.predecessorTarget + "</td> </tr>" +
                                "<tr><td></td><td>" + predecessorEditingTexts["toText"] + ":</td><td class='e-tooltiptaskname'></td><td></td></tr>";
                            $(tooltipbody).html(innerHtml);
                            tooltiptable.append(tooltipbody);
                            proxy._predecessorTooltip.append(tooltiptable);
                            $(document.body).append(proxy._predecessorTooltip);
                            proxy._updateTooltipPosition(proxy._predecessorTooltip, proxy._currMousePosX, proxy._currMousePosY);
                            //   proxy._$bodyContent.append(proxy._predecessorTooltip);
                        }
                    }
                    else if (proxy._leftResizing == true || proxy._rightResizing == true || proxy._allowDragging == true)
                    {
                    if (proxy._leftResizing == true) {
                        proxy._$ganttChartContainer[0].style.cursor = "e-resize";

                        tooltipElement = {
                            ttipstartDate: this._getFormatedDate(item.startDate, model.dateFormat),
                            ttipduration: item.duration,
                        };
                    } else if (proxy._rightResizing == true) {

                        proxy._$ganttChartContainer[0].style.cursor = "w-resize";

                        tooltipElement = {
                            ttipendDate: this._getFormatedDate(item.endDate, model.dateFormat),
                            ttipduration: item.duration,
                        };
                    } else if (proxy._allowDragging == true) {
                        proxy._$ganttChartContainer[0].style.cursor = "move";

                        tooltipElement = {
                            ttipstartDate: this._getFormatedDate(item.startDate, model.dateFormat),
                            ttipendDate: this._getFormatedDate(item.endDate, model.dateFormat),
                        };
                    }


                    tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '2px', 'cellpadding': '2px' });

                    if (proxy.model.taskbarEditingTooltipTemplate) {
                        proxy._taskbarEditingTooltipID = ej.buildTag("div.e-editingtooltip#editingtooltip" + proxy._id + "", "",
                           {
                               'position': 'absolute',
                               'z-index': '7'
                           }, {});
                        proxy.tooltipState = "editingTemplateID";
                        var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "editingTooltipTemplate"](item), {}, {});
                        proxy._taskbarEditingTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                    }
                    else if (proxy.model.taskbarEditingTooltipTemplateId) {
                        proxy._taskbarEditingTooltipID = ej.buildTag("div.e-editingtooltip#editingtooltip" + proxy._id + "", "",
                                {
                                    'position': 'absolute',
                                    'z-index': '7'
                                }, {});

                        proxy.tooltipState = "editingTemplate";
                        var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "editingTooltipTemplate"](item), {}, {});
                        proxy._taskbarEditingTooltipID[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";
                    }

                    else {
                        proxy.tooltipState = "";
                        if (tooltipElement) {
                            tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "editingTooltipTemplate"](tooltipElement), {}, {});
                            tooltiptable.append(tooltipbody);
                        }

                        proxy._taskbarEditingTooltipID = ej.buildTag("div.e-editingtooltip#editingtooltip" + proxy._id + "", tooltiptable,
                            {
                                'position': 'absolute',
                                'z-index': '7'
                            }, {});
                    }
                    if (proxy._taskbarEditingTooltipID) {
                        $(document.body).append(proxy._taskbarEditingTooltipID);
                        proxy._updateTooltipPosition(proxy._taskbarEditingTooltipID, proxy._currMousePosX, proxy._currMousePosY, true);
                    }
                }
                }
            }
        },

        _setItemPosition: function (item) {
            var proxy = this;
            $(proxy._leftResizer).css({ "left": (item.left + 2) + "px" });
            $(proxy._rightResizer).css({ "left": (item.left + item.width - 12) + "px" });
            $(proxy._taskbarItem).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });
            $(proxy._progressbarItem).css({
                "left": (item.left) + "px",
                "width": (item.progressWidth) + "px"
            });
            $(proxy._progressResizer).css({ "left": (item.left + item.progressWidth - 10) + "px" });
            $(proxy._tasknameContainer).css({
                "left": (item.left) + "px",
                "width": (item.width) + "px"
            });

            $(proxy._leftConnectorPoint).css({ "left": (item.left - 10) + "px" });
            $(proxy._rightConnectorPoint).css({ "left": (item.left + item.width) + "px" });


        },

        //returns the custom template for tooltip
        _getTemplate: function (template) {
            if (typeof (template) != "object") {
                var parser = $.parseHTML(this.model.tooltipTemplate);
                return parser[0];
            } else {
                var markup = $(template)[0].innerHTML;
                parser = $.parseHTML(markup);
                return parser[1];
            }
        },


        //Handles the mousemove after mousedown event
        _updateOnmouseMove: function (e) {

            var proxy = this, args = {}, posy,
                $target = (proxy._editingTarget) ? proxy._editingTarget : null,
                $ganttGridRows = proxy.getGanttChartRows(),
                row = ($target) ? $target.closest('tr') : null,
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem,
                div = ($target) ? $target[0].parentNode : null;

            if ($(div).hasClass("progressbarresizer-right") || $(div).hasClass("e-gantt-childtaskbar-progress")) {
                div = div.parentNode;
            }

            if (item != null) {

                args.editingFields = {
                    startdate: item.startDate,
                    enddate: item.endDate,
                    progress: item.status,
                    duration: item.duration
                };
                args.rowData = { rowData: item };

                proxy._taskbarEditing(args);

                if (!args.cancel) {

                    if (!e) e = window.event;

                    if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                        e.originalEvent.pageX = e.originalEvent.clientX;
                        e.originalEvent.pageY = e.originalEvent.clientY;
                    }

                    if (e.originalEvent.pageX || e.originalEvent.pageY) {
                        proxy._mousePosX = e.originalEvent.pageX;
                        posy = e.originalEvent.pageY;
                    } else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                        proxy._mousePosX = e.originalEvent.clientX + document.body.scrollLeft
                            + document.documentElement.scrollLeft;
                        posy = e.originalEvent.clientY + document.body.scrollTop
                            + document.documentElement.scrollTop;
                    }
                    else if (e.originalEvent.touches[0].pageX || e.originalEvent.touches[0].pageY) {
                        proxy._mousePosX = e.originalEvent.targetTouches[0].pageX;
                        posy = e.originalEvent.targetTouches[0].pageY;
                    }

                    if ($target.hasClass("e-gantt-milestone")) {
                        if ($(row).hasClass("e-editmode")) {
                            this._editMilestone(e);
                        }
                    } else {

                        if ($(div).hasClass("e-childContainer")) {
                            proxy._editingContainer = div;
                            proxy._leftResizer = proxy._editingContainer.childNodes[1]; //[0]
                            proxy._taskbarItem = proxy._editingContainer.childNodes[2];//[1]
                            proxy._progressbarItem = proxy._editingContainer.childNodes[3];//[2]
                            proxy._progressResizer = proxy._editingContainer.childNodes[4];//[3]
                            proxy._rightResizer = proxy._editingContainer.childNodes[5];//[4]
                            if (proxy.model.predecessorMapping && proxy.model.enablePredecessorEditing) {
                                proxy._leftConnectorPoint = div.childNodes[0];
                                proxy._rightConnectorPoint = div.childNodes[6];
                                $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                                $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                            }

                        }

                        if ($(row).hasClass("e-editmode") &&
                            (($(div).hasClass("e-parentContainer")) || $(div).hasClass("e-gantt-milestone-container") || $(div).hasClass("e-childContainer"))) {

                            if (proxy._leftResizing == true) {
                                proxy._enableLeftResizing(e, item);
                                proxy._updateEditedItem(item, "leftResizing");

                            } else if (proxy._progressResizing == true
                                && proxy.model.enableProgressBarResizing
                                && proxy.model.progressMapping) {
                                //Remove the progress bar status while resizing the progress bar status
                                
                                if (proxy._editingTarget.hasClass("progressbarresizer-right")) {
                                    proxy._editingTarget.prev(".progressbar").find(".e-tasklabel").text("")
                                }
                                proxy._enableProgressResizing(e, item);
                                proxy._updateEditedItem(item, "progressResizing");

                            } else if (proxy._rightResizing == true) {
                                proxy._enableRightResizing(e, item);
                                proxy._updateEditedItem(item, "rightResizing");

                            } else if (proxy._allowDragging == true) {
                                proxy._enableDragging(e, item);
                                proxy._updateEditedItem(item, "dragging");
                            }

                            //for predecessor , draw false line move the tooltip
                            else if (proxy._editPredecessor == true) {
                                proxy._drawFalseLine(e, item);
                            }
                        }
                    }
                    proxy._clearTooltip();
                    proxy._appendTooltip($target, recordIndexr);
                }
            }
        },


        //Handler for milestone editing
        _editMilestone: function (e) {

            var proxy = this,
                $target = proxy._editingTarget,
                div = $target[0].parentNode,
                $ganttGridRows = proxy.getGanttChartRows(),
                row = $target.closest('tr'),
                recordIndexr = $ganttGridRows.index(row),
                item = proxy._editingItem;

            if (item != null) {

                if (!e) e = window.event;
                if (e.originalEvent.pageX || e.originalEvent.pageY) {
                    proxy._mousePosX = e.originalEvent.pageX;
                } else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                    proxy._mousePosX = e.originalEvent.clientX + document.body.scrollLeft
                        + document.documentElement.scrollLeft;
                }
                else if (e.originalEvent.touches[0].pageX || e.originalEvent.touches[0].pageY) {
                    proxy._mousePosX = e.originalEvent.targetTouches[0].pageX;
                    posy = e.originalEvent.targetTouches[0].pageY;
                }

                e.target.style.cursor = "move";
                if (proxy.dragPosX > proxy._mousePosX) {
                    proxy._mousePosX = proxy.dragPosX - proxy._mousePosX;
                    if (item.left > 0) {
                        item.left -= proxy._mousePosX;
                    }
                } else {
                    proxy._mousePosX -= proxy.dragPosX;
                    item.left += proxy._mousePosX;
                }
                if (e.originalEvent.pageX == 0) {
                    this.dragPosX = e.originalEvent.targetTouches[0].pageX;
                }
                else {
                    this.dragPosX = e.originalEvent.pageX;
                }
                item.left = item.left < 0 ? 0 : item.left;

                $(div).css({ "left": (item.left) + "px" });

                $(proxy._leftConnectorPoint).css({ "left": (item.left - 10) + "px" });
                $(proxy._rightConnectorPoint).css({ "left": (item.left + 24) + "px" });

                proxy._updateEditedItem(item, "dragging");
                proxy._appendTooltip($target, recordIndexr);
            }
        },


        //Updates the edited item's values
        _updateEditedItem: function (ganttRecord, reason) {

            var proxy = this,
                model = proxy.model,
                scheduleHeaderType = model.scheduleHeaderSettings.scheduleHeaderType,
                scheduleHeaderValue = ej.Gantt.ScheduleHeaderType,
                durationUnitValue = ej.Gantt.DurationUnit,
                projectStartDate = ganttRecord._getFormatedDate(model.projectStartDate, model.dateFormat),
                projectStartDate = ganttRecord._getDateFromFormat(projectStartDate, model.dateFormat),
                projectStartDateHours = ganttRecord.startDate.getHours(),
                projectStartDateMinutes = ganttRecord.startDate.getMinutes(),

                remainDays, day, offset, rem, startDate, endDate;

            switch (reason) {

                case 'dragging':
                    remainDays = ganttRecord.left % model.perDayWidth,
                    day = (ganttRecord.left - remainDays) / model.perDayWidth,
                    offset = 0;
                    if (remainDays > 0 && (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {
                        if (scheduleHeaderType == scheduleHeaderValue.Day)
                            offset = remainDays > ((model.perDayWidth / 2) + (3 * model.perHourWidth)) ? 1 : 0;
                        else
                            offset = remainDays >= model.perDayWidth / 2 ? 1 : 0;                        
                    }
                    var remainingHours = Math.round(remainDays / (model.perDayWidth / 24)),//hour calculation
                           remainingMinutes = Math.round(remainDays / model.perMinuteWidth);
                    if (scheduleHeaderType == scheduleHeaderValue.Day ||
                        (scheduleHeaderType == scheduleHeaderValue.Week)) {
                        projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                        if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                            if (model.durationUnit == durationUnitValue.Minute && model.dateFormat.indexOf("hh:")!=-1)
                                projectStartDate.setMinutes(remainingMinutes);
                            else if (model.durationUnit == durationUnitValue.Hour && model.dateFormat.indexOf("hh:") != -1)
                                projectStartDate.setHours(remainingHours);
                        }
                        else {
                            projectStartDate.setHours(remainingHours);
                            //Update tooltip duration value only in the range between 8.00 AM to 4.00 PM. 
                            if (projectStartDate.getHours() > 8 && projectStartDate.getHours() < 16)
                                projectStartDate.setHours(remainingHours);
                            else
                                projectStartDate.setHours(8);
                        }
                            
                    }
                    else if (scheduleHeaderType == scheduleHeaderValue.Hour) {
                        var remainingMinutes = (remainDays / (model.perDayWidth / (24 * 60)));                       
                            projectStartDate.setMinutes(remainingMinutes);
                            projectStartDate.setDate(projectStartDate.getDate() + day + offset);                               
                    }
                    else if (model.dateFormat.indexOf("hh:") !== -1 &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainDays / (model.perDayWidth / 9));
                        projectStartDate.setHours(8 + remainingHours);
                        projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                    }
                    else {
                        projectStartDate.setDate(projectStartDate.getDate() + day + offset);
                        if (model.durationUnit == durationUnitValue.Hour)
                            projectStartDate.setHours(remainingHours);
                        else if (model.durationUnit == durationUnitValue.Minute)
                            projectStartDate.setMinutes(remainingMinutes);                                                  
                    }                                        
                    ganttRecord.startDate = new Date(projectStartDate);
                    ganttRecord._calculateEndDate(model.includeWeekend, model.holidays, model, model.perDayWidth, reason);
                   
                    //update on dataSource
                    if (model.startDateMapping)
                        ganttRecord.item[model.startDateMapping] = ganttRecord.startDate;
                    if (model.endDateMapping)
                        ganttRecord.item[model.endDateMapping] = ganttRecord.endDate;

                    break;

                case 'leftResizing':
                    var editMode = "Resizing";
                    remainDays = ganttRecord.left % model.perDayWidth,
                    day = (ganttRecord.left - remainDays) / model.perDayWidth,
                    offset = 0;
                    if (remainDays > 0 && (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours)) {// 
                        if (scheduleHeaderType == scheduleHeaderValue.Day)
                            offset = remainDays > ((model.perDayWidth / 2) + (3 * model.perHourWidth)) ? 1 : 0;
                        else
                            offset = remainDays >= model.perDayWidth / 2 ? 1 : 0;
                    }

                    var remainingHours = Math.round(remainDays / (model.perDayWidth / 24)),//hour calculation
                        remainingMinutes = Math.round(remainDays / model.perMinuteWidth);

                    projectStartDate.setDate(projectStartDate.getDate() + day + offset);

                    if (scheduleHeaderType == scheduleHeaderValue.Day ||
                        (scheduleHeaderType == scheduleHeaderValue.Week && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale24Hours) {
                            if (model.durationUnit == durationUnitValue.Minute)
                                projectStartDate.setMinutes(remainingMinutes);
                            else if (model.durationUnit == durationUnitValue.Hour)
                                projectStartDate.setHours(remainingHours);
                        }
                        else {
                            if (remainingHours >= 8 && remainingHours <= 15)
                                projectStartDate.setHours(remainingHours);
                            else {
                                projectStartDate.setDate(projectStartDate.getDate() - 1);
                                projectStartDate.setHours(15);
                            }
                        }                            
                    }
                    else if (scheduleHeaderType == scheduleHeaderValue.Hour) {
                        var remainingMinutes = remainDays / (model.perDayWidth / (24 * 60));
                        projectStartDate.setMinutes(remainingMinutes);                        
                    }
                    else if (model.dateFormat.indexOf("hh:") !== -1 &&
                        model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainDays / (model.perDayWidth / 9));
                        projectStartDate.setHours(8 + remainingHours);
                    }
                    else {                       
                        if (model.durationUnit == durationUnitValue.Hour)
                            projectStartDate.setHours(remainingHours);
                        else if (model.durationUnit == durationUnitValue.Minute)
                            projectStartDate.setMinutes(remainingMinutes);
                    }

                    //if (projectStartDate < new Date(ganttRecord.endDate)) {
                    ganttRecord.startDate = new Date(projectStartDate);
                    ganttRecord._calculateDuration(projectStartDate, new Date(ganttRecord.endDate),
                        model.includeWeekend, model.holidays, false, model);
                    //}
                    if (ganttRecord.startDate.getTime() == ganttRecord.endDate.getTime()
                        && ej.isNullOrUndefined(ganttRecord.isMilestone) && ganttRecord.isMilestone == false && ganttRecord.duration == 0) {
                        ganttRecord.duration = 1;
                    }

                    //update on dataSource
                    if (model.startDateMapping)
                        ganttRecord.item[model.startDateMapping] = ganttRecord.startDate;
                    if(model.durationMapping)
                        ganttRecord.item[model.durationMapping] = ganttRecord.duration;

                    break;

                case 'rightResizing':
                    var editMode = "Resizing";
                    day = ganttRecord.width / model.perDayWidth,
                    rem = ganttRecord.width % model.perDayWidth,
                    startDate = new Date(ganttRecord.startDate);                    
                    offset = 0;

                    if (rem > 0 && (model.workingTimeScale != "TimeScale24Hours")) {
                        if (scheduleHeaderType == scheduleHeaderValue.Day)
                            offset = rem > ((model.perDayWidth / 2) + (3 * model.perHourWidth)) ? 1 : 0;
                        else
                            offset = rem >= (model.perDayWidth / 2) ? 1 : 0;                        
                    }

                    var totalLength = ganttRecord.left + ganttRecord.width;
                    var remainingLength = totalLength % model.perDayWidth;

                    var remainingHours = Math.round(remainingLength / (model.perDayWidth / 24)),//hour calculation
                        remainingMinutes = Math.round(remainingLength / (model.perDayWidth / (24*60)));


                    if ((scheduleHeaderType == scheduleHeaderValue.Day
                        && model.dateFormat.indexOf("hh") == -1)) {

                        var startDateHour = (ganttRecord.left % model.perDayWidth) / model.perHourWidth;
                        var endDateHour = ((ganttRecord.left + ganttRecord.width) % model.perDayWidth) / model.perHourWidth;
                        var remainingHourDiff = Math.round(remainingLength % (model.perHourWidth));

                        endDate = day < 1 ? ganttRecord._dateUpdate(startDate, 0, (endDateHour)) :
                            ganttRecord._dateUpdate(startDate, Math.ceil(day - 1 + offset), endDateHour);

                        startDate.setHours(startDateHour);
                    }

                    else if (scheduleHeaderType == scheduleHeaderValue.Week)
                    {
                        var startDateHour = ganttRecord.left / (model.perDayWidth / 24);
                        var remainingHourDiff = totalLength / (model.perDayWidth / 24);
                        remainingHourDiff -= startDateHour;
                        var dayCount = day + offset;
                        dayCount = Math.floor(dayCount);
                        if (model.dateFormat.indexOf("hh") != -1) {                           
                            if (model.durationUnit == durationUnitValue.Hour)
                                endDate = ganttRecord._dateUpdate(startDate, 0, remainingHourDiff);
                            else if (model.durationUnit == durationUnitValue.Minute)
                                endDate = ganttRecord._dateUpdate(startDate, dayCount, 0, remainingMinutes);
                            else
                                endDate = ganttRecord._dateUpdate(startDate, dayCount, 0);
                        }
                        else
                            endDate = ganttRecord._dateUpdate(startDate, day - 1 + offset, 0);
                    }
                    else if ((scheduleHeaderType == scheduleHeaderValue.Hour && model.dateFormat.indexOf("hh") != -1)) {                                        
                        var startDateHour = Math.round((ganttRecord.left % model.perDayWidth) / (model.perDayWidth / (24 * 60)));
                        var remainingMinDiff = Math.round(totalLength / (model.perDayWidth / (24 * 60)));
                        remainingMinDiff -= startDateHour;
                        remainingMinDiff = remainingMinDiff % (24 * 60);                        
                        endDate = ganttRecord._dateUpdate(startDate, day, 0, remainingMinDiff);
                    }


                    else if ((scheduleHeaderType == scheduleHeaderValue.Day
                        && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.durationUnit == durationUnitValue.Hour) {
                            var startDateHour = ganttRecord.left / model.perHourWidth;
                            var remainingHourDiff = Math.round(totalLength / (model.perHourWidth));
                            remainingHourDiff -= startDateHour;

                            endDate = ganttRecord._dateUpdate(startDate, 0, (remainingHourDiff));
                        }
                        else {
                            var startDateHour = ganttRecord.left / model.perMinuteWidth;
                            var remainingMinuteDiff = Math.round(totalLength / (model.perMinuteWidth));
                            remainingMinuteDiff -= startDateHour;

                            endDate = ganttRecord._dateUpdate(startDate, 0, 0, remainingMinuteDiff);
                        }
                    }

                    else {
                        if (model.durationUnit == durationUnitValue.Hour && model.dateFormat.indexOf("hh") != -1)
                            endDate = ganttRecord._dateUpdate(startDate, day - 1 + offset, remainingHours);
                        else if (model.durationUnit == durationUnitValue.Minute && model.dateFormat.indexOf("hh") != -1)
                            endDate = ganttRecord._dateUpdate(startDate, day + offset, 0, remainingMinutes);                       
                        else
                            endDate = ganttRecord._dateUpdate(startDate, day + offset, 0);
                    }
                    
                    if (scheduleHeaderType == scheduleHeaderValue.Day ||
                        (scheduleHeaderType == scheduleHeaderValue.Week && model.dateFormat.indexOf("hh") != -1)) {
                        if (model.workingTimeScale != ej.Gantt.workingTimeScale.TimeScale24Hours) {
                            if (model.durationUnit == durationUnitValue.Hour) {
                                endDate.setHours(Math.round(remainingHours));
                                if (endDate.getHours() > 16)
                                    endDate.setHours(Math.round(24 + 9));
                                else if (endDate.getHours() < 8)
                                    endDate.setHours(Math.round(9));                                                                    
                            }
                            else
                                endDate.setHours(17);
                        }
                    }
                    else if (model.dateFormat.indexOf("hh:") != -1 &&
                       model.workingTimeScale == ej.Gantt.workingTimeScale.TimeScale8Hours) {
                        var remainingHours = Math.round(remainingLength / (model.perDayWidth / 9));
                        endDate.setHours(Math.round(8 + remainingHours));
                    }
                    ganttRecord.endDate = new Date(endDate);

                    if (model.holidays && model.holidays.length) {
                        var holidayIndex = 0,
                            holidayslength = model.holidays.length,
                            holiday;
                        for (holidayIndex = 0; holidayIndex < holidayslength; holidayIndex++) {
                            holiday = this._getDateFromFormat(model.holidays[holidayIndex]);
                            if (endDate === holiday) {
                                endDate.setDate(endDate.getDate() - 1);
                            }
                        }
                    }
                    if (this.model.includeWeekend == false) {
                        if (endDate.getDay() == 0)
                            endDate.setDate(endDate.getDate() + 2);
                        else if (endDate.getDay() == 6)
                            endDate.setDate(endDate.getDate() + 1);
                    }
                    ganttRecord._calculateDuration(startDate, endDate, model.includeWeekend, model.holidays, false, model);

                    //update on datasource
                    if(model.endDateMapping)
                        ganttRecord.item[model.endDateMapping] = ganttRecord.endDate;
                    if (model.durationMapping)
                        ganttRecord.item[model.durationMapping] = ganttRecord.duration;


                    break;

                case 'progressResizing':
                    ganttRecord.status = proxy._getProgressPercent(ganttRecord.width, ganttRecord.progressWidth);
                    if (model.progressMapping)
                        ganttRecord.item[model.progressMapping] = ganttRecord.status;
                    break;
            }
        },


        //false line implementation 
        _drawFalseLine: function (e, item) {

            var proxy = this,

              x1 = proxy._connectorPointX;
              y1 = proxy._connectorPointY;

              x2 = e.pageX - proxy._$bodyContent.offset().left,
              y2 = e.pageY - proxy._$bodyContent.offset().top;

            //var sy=proxy._$bodyContainer.ejScroller("option", "scrollTop");
            //var sx = proxy._$bodyContainer.ejScroller("option", "scrollLeft");

            var length = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
            var angle = Math.atan2(y2 - y1, x2 - x1) * 180 / Math.PI;
            var transform = 'rotate(' + angle + 'deg)';

            proxy._falseLine && proxy._falseLine.remove();

                proxy._falseLine = ej.buildTag("div.e-gantt-falseLine#ganttfalseline" + proxy._id, "", {}, {});
                proxy._falseLine.css({
                    'transform-origin': '0% 100%',
                    'right':'auto',
                    'position': 'absolute',
                    'transform': transform,
                    '-ms-transform':transform, /* IE 9 */
                    '-moz-transform':transform, /* Firefox */
                    '-webkit-transform':transform, /* Safari and Chrome */
                    '-o-transform':transform, /* Opera */
                    'border-top-width': '2px',
                    'border-top-style': 'dashed',
                    'z-index': '5'
                }).width(length - 3).offset({ left: x1, top: y1 });

                if (!(ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)) {
                    proxy._$bodyContent.append(proxy._falseLine);
                }

                //Change the position of predecesspr tooltip
                proxy._updateTooltipPosition(proxy._predecessorTooltip, e.pageX, e.pageY);
        },

        _mouseLeave: function () {

            var proxy = this;
            $(".progressbarresizer-right").css({ "background-image": "none" });
            window.clearTimeout(proxy._tooltipTimer);
            proxy._clearTooltip();
            

            if (this._mouseDown == false) {
                $(proxy._leftResizerGripper).removeClass("gripper");
                $(proxy._rightResizerGripper).removeClass("gripper");

                if (proxy.model.progressMapping) {
                    $(proxy._progressHandle).removeClass("e-progresshandle");
                    $(proxy._progressHandleChild).removeClass("e-progresshandleafter");
                }

                if (proxy.model.predecessorMapping && proxy.model.enablePredecessorEditing) {

                    $(proxy._leftConnectorPoint).removeClass('e-connectorpoint-hover');
                    $(proxy._rightConnectorPoint).removeClass('e-connectorpoint-hover');
                    proxy._leftConnectorPoint = null;
                    proxy._rightConnectorPoint = null;
                }
            }
        },

        _getOffsetRect: function (elem) {
            var box = elem.getBoundingClientRect(),
                body = document.body,
                docElem = document.documentElement,
                scrollTop = window.pageYOffset || docElem.scrollTop || body.scrollTop,
                scrollLeft = window.pageXOffset || docElem.scrollLeft || body.scrollLeft,
                clientTop = docElem.clientTop || body.clientTop || 0,
                clientLeft = docElem.clientLeft || body.clientLeft || 0,
                top = box.top + scrollTop - clientTop,
                left = box.left + scrollLeft - clientLeft;
            return { top: Math.round(top), left: Math.round(left) }
        },
        /* edge detection for tooltip in ganttchart*/
        _updateTooltipPosition: function (tooltip, posx, posy, isEditingTooltip) {
            if (tooltip && tooltip.length > 0) {
                var proxy = this,
                    containerBoundary = proxy._getOffsetRect(proxy._$bodyContainer[0]),
                    topEnd = containerBoundary.top + proxy._$bodyContainer.height(),
                    leftEnd = containerBoundary.left + proxy._$bodyContainer.width();
                //Left update
                if (((posx + tooltip.width() + 30) > leftEnd)) {
                    var pos = posx - tooltip.width() - 30;
                    tooltip.css('left', ((pos) + 'px'));
                }
                else {
                    if (!isEditingTooltip)
                        tooltip.css('left', (posx + 15) + 'px');
                    else
                        tooltip.css('left', (posx) + 'px');
                }
                //Top update
                if (isEditingTooltip || ((posy + tooltip.height() + 30) > topEnd)) {
                    var pos = posy - tooltip.height() - 30;
                    tooltip.css('top', ((pos) + 'px'));
                } else {
                    tooltip.css('top', (posy + 10) + 'px');
                }
            }
        },

        _mouseHover: function (e) {

            
            var $target = $(e.target), proxy = this,
                posx = 0,
                posy = 0,
                $ganttChartRows = proxy.getGanttChartRows(),
                row = $target.closest('tr'),
                div = $target[0].parentNode,
                progressHandler,
                recordIndexr = $ganttChartRows.index(row),
                item = proxy.model.currentViewData[recordIndexr],
                tooltipElement, tooltiptable, tooltipbody;

          
            //clearing rendered tooltips in Gantt
            proxy._clearTooltip();
            
            if (proxy._rightResizing == false && proxy._leftResizing == false &&
                proxy._progressResizing == false && proxy._mouseDown == false) {

                // Check whether any one of the following classes are available in the target to skip MOVE cursor
                if ($target.is('.e-gantt-parenttaskbar-innerdiv, .e-gantt-parenttaskbar-progress, .e-connectorpoint-left, .e-connectorpoint-right, .taskbarresizer-left, .taskbarresizer-right')) {
                }
                else if (proxy.model.allowGanttChartEditing) {
                    e.target.style.cursor = "move";
                }


                if (e.type == "mouseenter" || e.type == "mousemove") {

                    if (!e) e = window.event;
                    if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                        e.originalEvent.pageX = e.originalEvent.clientX;
                        e.originalEvent.pageY = e.originalEvent.clientY;
                    }

                    if (e.originalEvent.pageX || e.originalEvent.pageY) {

                        posx = e.originalEvent.pageX;
                        posy = e.originalEvent.pageY;

                    }
                    else if (e.originalEvent.clientX || e.originalEvent.clientY) {
                        posx = e.originalEvent.clientX + document.body.scrollLeft
                            + document.documentElement.scrollLeft;
                        posy = e.originalEvent.clientY + document.body.scrollTop
                            + document.documentElement.scrollTop;

                    }
                    else if (e.originalEvent && e.originalEvent.changedTouches
                        && e.originalEvent.changedTouches.length > 0) {
                        posx = e.originalEvent.changedTouches[0].pageX;
                        posy = e.originalEvent.changedTouches[0].pageY;
                    }

                  
                    if ($(div).hasClass("e-childContainer") && proxy.model.allowGanttChartEditing && !proxy._editingTarget) {
                        proxy._leftResizerGripper = div.childNodes[1],//[0]  // New div is added for drag points so other elemnts are changed
                        proxy._rightResizerGripper = div.childNodes[5];//[4]
                        progressHandler = div.childNodes[4];//[3]
                        proxy._progressHandle = progressHandler.firstChild;
                        proxy._progressHandleChild = progressHandler.childNodes[1];//[1]

                        $(proxy._leftResizerGripper).addClass("gripper");
                        $(proxy._rightResizerGripper).addClass("gripper");

                        if (proxy.model.progressMapping) {
                            $(proxy._progressHandle).addClass("e-progresshandle");
                            $(proxy._progressHandleChild).addClass("e-progresshandleafter");
                        }
                    }

                    //Before Mouse Down  , Apply style for drag points in child cotainer and miles stone container
                    if ($(div).hasClass("e-childContainer") && proxy.model.predecessorMapping  && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing && !proxy._editingTarget) {

                        proxy._leftConnectorPoint = div.childNodes[0];
                        proxy._rightConnectorPoint = div.childNodes[6];
                        $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                        $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                    }
                    else if ($(div).hasClass("e-gantt-milestone") && proxy.model.predecessorMapping && proxy.model.allowGanttChartEditing && proxy.model.enablePredecessorEditing && !proxy._editingTarget) {
                        var milesStoneContainer = div.parentNode;
                        proxy._leftConnectorPoint = milesStoneContainer.childNodes[0];
                        proxy._rightConnectorPoint = milesStoneContainer.childNodes[2];
                        $(proxy._leftConnectorPoint).addClass("e-connectorpoint-hover");
                        $(proxy._rightConnectorPoint).addClass("e-connectorpoint-hover");
                    }


                    if (item && proxy.model.enableTaskbarTooltip) {
                        tooltipElement = {
                            ttipstartDate: this._getFormatedDate(item.startDate,this.model.dateFormat),
                            ttipendDate: this._getFormatedDate(item.endDate, this.model.dateFormat),
                            ttipduration: item.duration,
                            ttipprogress: item.status.toString(),
                            ttiptaskname: item.taskName,
                        };

                        tooltiptable = ej.buildTag("table.e-tooltiptable", "", {}, { 'cellspacing': '5' });
                      

                        if (proxy.model.tooltipTemplate == null && proxy.model.tooltipTemplateId == null) {
                            proxy.tooltipState = "";
                            tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](tooltipElement), {}, {});
                            tooltiptable.append(tooltipbody);
                            proxy._mouseHoverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", tooltiptable,
                                {'position': 'absolute', 'z-index': '7', 'border-radius': '5px' }, {});

                            proxy._mouseHoverTooltip.innerHTML = "<table>" + tooltiptable[0].innerHTML + "</table>";

                            //Providing a time delay to render the tooltip to improve UI
                            proxy._tooltipTimer = setTimeout(function () {
                                $(document.body).append(proxy._mouseHoverTooltip)
                                proxy._updateTooltipPosition(proxy._mouseHoverTooltip, posx, posy);
                            }, 700);
                        }
                        else if (proxy.model.tooltipTemplate) {
                            proxy._mouseHoverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "","",
                                {'position': 'absolute', 'z-index': '7', 'border-radius': '5px' }, {});
                                                       
                            proxy.tooltipState = "Template";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](item), {}, {});
                            proxy._mouseHoverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";

                            //Providing a time delay to render the tooltip to improve UI
                            proxy._tooltipTimer = setTimeout(function () {
                                $(document.body).append(proxy._mouseHoverTooltip)
                                proxy._updateTooltipPosition(proxy._mouseHoverTooltip, posx, posy);
                            }, 700);
                        }
                        else if (proxy.model.tooltipTemplateId) {
                            proxy._mouseHoverTooltip = ej.buildTag("div.e-tooltipgantt#tooltipgantt" + proxy._id + "", "",
                                {'position': 'absolute', 'z-index': '7', 'border-radius': '5px' }, {});

                            proxy.tooltipState = "TemplateID";
                            var tooltipbody = ej.buildTag("tbody", $.render[proxy._id + "tooltipTemplate"](item), {}, {});
                            proxy._mouseHoverTooltip[0].innerHTML = "<table>" + tooltipbody[0].innerHTML + "</table>";

                            //Providing a time delay to render the tooltip to improve UI
                            proxy._tooltipTimer = setTimeout(function () {
                                $(document.body).append(proxy._mouseHoverTooltip);
                                proxy._updateTooltipPosition(proxy._mouseHoverTooltip, posx, posy);
                            }, 700);
                        }
                        $(proxy._mouseHoverTooltip).addClass("customTooltip");
                    }
                }
            }
        },
        
        _createEditingTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                durationUnitTexts=proxy.model.durationUnitTexts,
                helpers = {
                    _getStartDate: proxy._tooltipStartDate,
                    _getEndDate: proxy._tooltipEndDate,
                    _getDuration: proxy._tooltipDuration,
                    _getTaskName: proxy._tooltipTaskName,
                    _getProgress: proxy._tooltipProgress
                },
                getDurationUnit = proxy._getDurationUnits(),
                editingTemplateId = proxy.model.taskbarEditingTooltipTemplateId,
                editingTemplate = proxy.model.taskbarEditingTooltipTemplate;

            $.views.helpers(helpers);

            
            if (editingTemplateId) {
                var parentTr = $("#" + editingTemplateId)[0].innerHTML;
            }
            else if (editingTemplate) {
                var parentTr = editingTemplate;
            }
            else {
                var parentTr = "<tr class='e-tooltip_rowcell'>";

                td = "{{if ~_getTaskName()}}" +
                    "<td class='e-tooltiptaskname' style='width:40px;text-align:left;' colspan='3';>{{:ttiptaskname}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getStartDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["startDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipstartDate}}</td> {{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getEndDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["endDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipendDate}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getDuration()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["duration"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipduration}}" +" "+ durationUnitTexts[getDurationUnit] + "</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getProgress()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["status"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipprogress}}%</td>{{/if}}";

                parentTr += td;
                parentTr += "</tr>";
            }

            var templates = {};
            templates[proxy._id + "editingTooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _createProgressbarTooltipTemplate: function () {
            var proxy = this,
                  templateId = proxy.model.progressbarTooltipTemplateId,
            template = proxy.model.progressbarTooltipTemplate;

            if (templateId) {
                var parentTr = $("#" + proxy.model.progressbarTooltipTemplateId)[0].innerHTML;
            }
            else if (proxy.model.progressbarTooltipTemplate) {
                var parentTr = proxy.model.progressbarTooltipTemplate;
            }
            var templates = {};
            templates[proxy._id + "progressbarTooltipTemplate"] = parentTr;
            $.templates(templates);
        },


        //Template to create tooltip
        _createTooltipTemplate: function () {

            var proxy = this,
                td,
                columnHeaderTexts = proxy.model.columnHeaderTexts,
                durationUnitTexts = proxy.model.durationUnitTexts,
                helpers = {
                    _getStartDate: proxy._tooltipStartDate,
                    _getEndDate: proxy._tooltipEndDate,
                    _getDuration: proxy._tooltipDuration,
                    _getTaskName: proxy._tooltipTaskName,
                    _getProgress: proxy._tooltipProgress,
                    _getDurationUnit: proxy._getDurationUnits,
                },
                getDurationUnit = proxy._getDurationUnits(),
                templateId = proxy.model.tooltipTemplateId,
                template = proxy.model.tooltipTemplate;

            $.views.helpers(helpers);

            if (templateId ) {
                var parentTr = $("#" + proxy.model.tooltipTemplateId)[0].innerHTML;
            }
            else if (proxy.model.tooltipTemplate ) {
                var parentTr = proxy.model.tooltipTemplate;
            }
           
            else {
                var parentTr = "<tr class='e-tooltip_rowcell'>";

                td = "{{if ~_getTaskName()}}" +
                    "<td class='e-tooltiptaskname' style='width:40px;text-align:left;' colspan='3';>{{:ttiptaskname}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getStartDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["startDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipstartDate}}</td> {{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getEndDate()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["endDate"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipendDate}}</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getDuration()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["duration"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipduration}}"+ " " + durationUnitTexts[getDurationUnit]+"</td>{{/if}}" +
                    "</tr><tr>" +
                    "{{if ~_getProgress()}}" +
                    "<td class='e-tooltiptd-alignleft' style='width:30px;white-space:nowrap;'>" + columnHeaderTexts["status"] + "</td>" +
                    "<td class='e-tooltiptd-aligncenter' style='width:10px;font-weight:bold;'>:</td>" +
                    "<td class='e-tooltiptd-alignright' style='width:30px;white-space:nowrap;'>{{:ttipprogress}}%</td>{{/if}}";

                parentTr += td;
                parentTr += "</tr>";
            }

            var templates = {};
            templates[proxy._id + "tooltipTemplate"] = parentTr;
            $.templates(templates);
        },

        _tooltipTaskName: function () {
            return this.data.ttiptaskname;
        },

        _tooltipProgress: function () {
            return this.data.ttipprogress;
        },

        _tooltipStartDate: function () {
            return this.data.ttipstartDate;
        },

        _tooltipEndDate: function () {
            return this.data.ttipendDate;
        },

        _tooltipDuration: function () {
            return this.data.ttipduration;
        },

        _getDurationUnits: function (data) {
            var proxy = this;
            if (proxy.model.durationUnit == ej.Gantt.DurationUnit.Hour)
                return "hours";
            else if (proxy.model.durationUnit == ej.Gantt.DurationUnit.Minute)
                return "minutes";
            else
                return "days";

        },

        _cellClickHandler: function (e) {
            var $target = $(e.target),
                $row = $target.closest('tr'),
                proxy = this,
                model = this.model,
                args = {},
                selectedItem,
                index,
                $ganttGridRows = proxy.getGanttChartRows();

            index = $ganttGridRows.index($target.parent());

            if (index == -1) {
                index = $ganttGridRows.index($row);
            }

          
            if (index != -1 && model.allowSelection) {

                selectedItem = model.currentViewData[index];
                index = model.updatedRecords.indexOf(selectedItem);
                if (proxy._rowSelectingEventTrigger(model.selectedRowIndex, index))
                    return;
                proxy.selectRows(index);
                proxy.element[0].focus();
                proxy._rowSelectedEventTrigger(index);
            }

        },

        getRowByIndex: function (from, to) {
            try {
                var proxy = this;

                var gridRows = proxy.getGanttChartRows(),
                    $gridRows = $(gridRows),
                    $row = $();

                if (proxy.model.enableVirtualization) {
                    var recordstart = proxy.model.updatedRecords[from];
                    from = proxy.model.currentViewData.indexOf(recordstart);
                }

                if (ej.isNullOrUndefined(to)) {
                    if (proxy.model.enableVirtualization) {
                        var recordend = proxy.model.updatedRecords[to];
                        to = proxy.model.currentViewData.indexOf(recordend);
                    }
                    return $(gridRows[from]);
                } else {
                    return $($gridRows.slice(from, to));
                }
            } catch (e) {
                return $();
            }
        },

        selectRows: function (index) {
            var proxy = this,
                $ganttGridRows = proxy.getGanttChartRows(),
                selectedItem;
            if (proxy.prevSelectedItem) {
                proxy.prevSelectedItem.isSelected = false;
            }
            $ganttGridRows.removeClass("e-gantt-mouseclick");
            var $row = proxy.getRowByIndex(index);
            $row.addClass("e-gantt-mouseclick");
            proxy.model.selectedRowIndex = index;
            selectedItem = proxy.model.updatedRecords[index];
            if(selectedItem)
                selectedItem.isSelected = true;
            proxy.model.selectedItem = selectedItem;
            proxy.prevSelectedItem = selectedItem;
        },

        updateSelectedItem: function (recordIndex) {
            var proxy = this,
                $ganttGridRows = $(proxy.getGanttChartRows());

            $ganttGridRows.removeClass("e-gantt-mouseclick");
            proxy.model.updatedRecords[recordIndex].isSelected = true;
            proxy.model.selectedItem = proxy.model.updatedRecords[recordIndex];
            this._refresh();
            proxy._rowSelectedEventTrigger(recordIndex);

        },


        ganttrowHover: function (index) {
            var proxy = this;
            if (!proxy.model.enableRowHover)
                return;
            var $gridRows =proxy.getGanttChartRows();
            $gridRows.removeClass("e-gantt_mousehover").addClass("e-gantt_mouseleave");;
            $($gridRows[index]).addClass("e-gantt_mousehover").removeClass("e-gantt_mouseleave");
        },

        _rowHover: function (e) {
            var proxy = this,
                $target = $(e.target),
                $gridRows,
                args = {},
                index,
                row;
            if (!proxy.model.enableRowHover)
                return false;
            $gridRows = $(proxy.getGanttChartRows());
            index = $gridRows.index($target.parent());
            if (index == -1) {
                row = $target.closest('tr');
                index = $gridRows.index(row);
            }
            args.index = index;

            if (e.type == "mouseenter") {
                args.reason = "mouseenter";
                proxy.addRowHover(index);
            } else {
                args.reason = "mouseleave";
                proxy.removeRowHover();
            }
            proxy._trigger("rowHover", args);
            return false;
        },

        addRowHover: function (index) {
            var $gridRows = this.getGanttChartRows();
            if (index != null)
                index != -1 && $($gridRows[index]).addClass("e-enablerowhover");
        },

        removeRowHover: function () {
            var $gridRows = this.getGanttChartRows();
            $gridRows.removeClass("e-enablerowhover");
        },

        updateHeight: function (height) {
            var proxy = this,
                model = proxy.model;
            proxy._setScrollTop();
            proxy._containerHeight = height;
            $("#ganttviewerbodyContent" + proxy._id + "").css({
                 "height": proxy._containerHeight + "px"
            });

            if (model.stripLines) {
                $(proxy.element).find(".e-stripline,.e-stripLines").css({ height: height + "px" });
            }

            if (model.holidays) {
                $(proxy.element).find(".e-holidays,.e-holiday").css({ height: height + "px" });
            }
            $(".e-secondary_canvas").css({ height: height + "px" });
            $(proxy.element).find(".e-weekends").css({ height: height + "px" });

            var holidayLableSelector = ".e-holiday-label";
            /* for rotate holiday lable in ie8 */
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                holidayLableSelector = ".e-holiday-label-ie8";
            }

            if (height < proxy._viewportHeight) {
                if (height <= model.rowHeight) {
                    $(proxy.element).find(holidayLableSelector).css({ top: -height - 30 });
                }
                else
                    $(proxy.element).find(holidayLableSelector).css({ top: height / 2 });
            }
            else
                $(proxy.element).find(holidayLableSelector).css({ top: proxy._viewportHeight / 2 });
            
            //To enable the Horizontal scroller when all the records were deleted
            if (height == 0) {
                proxy._$bodyContent.css({ "height": "1px" });
            }
            proxy._$bodyContainer.ejScroller("refresh");

            /*While refreshing scroller the gantt chart container height is increased by 16px.
             When horizontal scrollbar not avaiable in gantt chart side.
             So check the chart container height and view port height.
             If it is varies set the view port height to chart container height*/
            if(parseInt(proxy._$bodyContainer.css("height")) != proxy._viewportHeight)
                proxy._$bodyContainer.css("height", proxy._viewportHeight);
        },


        //method to trigger actioncomplete event
        _completeAction: function (args) {
            var proxy = this;
            args.vscrollExsist = proxy._$bodyContainer.ejScroller("isVScroll");
            proxy._trigger("actionComplete", args);
            if (args.requestType === "validatePredecessor") {
                return args.predecessorValidation;
            }
        },


        //method to trigger taskbarediting event
        //to apply changes during taskbar editing
        _taskbarEditing: function (args) {
            var proxy = this;
            proxy._trigger("taskbarEditing", args);
        },


        //method to trigger taskbaredited event
        //to apply changes after taskbar edited
        _taskbarEdited: function (args) {
            var proxy = this;
            proxy._trigger("taskbarEdited", args);
        },


        //method to trigger querytaskbarinfo 
        //to apply changes at load time
        _queryTaskbarinfo: function (args) {
            this._trigger("queryTaskbarInfo", args);
            this._updateTaskbarItems(args);
        },


        //method to trigger refresh row
        _refreshRow: function (args) {
            var proxy = this;
            proxy._trigger("refreshRow", args);
        },

        _enbleZooming: function (args) {
            var proxy = this;
            proxy._trigger("zooming", args);
        },


        //Updates taskbar items styles when chart loads
        _updateTaskbarItems: function (args) {

            var $ganttChartRows = this.getGanttChartRows(),
                index = this.model.currentViewData.indexOf(args.data),
                taskbarContainer = $ganttChartRows[index], taskElement;

            if (taskbarContainer) {
                if (taskbarContainer.childNodes.length > 1)
                    taskElement = taskbarContainer.childNodes[1].firstChild;
                else
                    taskElement = taskbarContainer.childNodes[0].firstChild;

                if ($(taskElement).hasClass("e-parentContainer")) {

                    var parentTaskbar = taskElement.childNodes[1],
                        parentProgressbar = taskElement.childNodes[2],
                        parentTaskbarStyle = parentTaskbar.style,
                        parentProgressbarStyle = parentProgressbar.style;

                    if (parentTaskbarStyle.backgroundColor != args.parentTaskbarBackground) {
                        parentTaskbarStyle.backgroundColor = args.parentTaskbarBackground;
                    }
                    if (parentProgressbarStyle.backgroundColor != args.parentProgressbarBackground) {
                        parentProgressbarStyle.backgroundColor = args.parentProgressbarBackground;
                    }

                } else if ($(taskElement).hasClass("e-childContainer")) {

                    var taskbar = taskElement.childNodes[2],
                        progressbar = taskElement.childNodes[3],
                        taskbarStyle = taskbar.style,
                        progressbarStyle = progressbar.style;

                    if (taskbarStyle.backgroundColor != args.taskbarBackground) {
                        taskbarStyle.backgroundColor = args.taskbarBackground;
                    }
                    if (progressbarStyle.backgroundColor != args.progressbarBackground) {
                        progressbarStyle.backgroundColor = args.progressbarBackground;
                    }
                }
            }
        },

        //to perform expand collapse operation from ganttchart
        _expandCollapse: function (recordIndex) {

            var proxy = this,
                args = {},
                data = proxy.model.currentViewData[recordIndex];

            args.data = data;
            args.recordIndex = proxy.model.updatedRecords && proxy.model.updatedRecords.indexOf(data);
            args.source = "ganttChart";
            args.expanded = !data.expanded;
            if (data) {
                if (!args.expanded) {
                    proxy._trigger("collapsing", args);
                }
                else {
                    proxy._trigger("expanding", args);
                }
            }
        },


        //get the Expand status of GridRecord
        _getExpandStatus: function (record) {
            var parentRecord = record.parentItem;
            if (parentRecord != null) {
                if (parentRecord.expanded === false)
                    return false;
                else if (parentRecord.parentItem) {
                    if (parentRecord.parentItem.expanded === false)
                        return false;
                    else
                        return this._getExpandStatus(parentRecord.parentItem);
                }
                else
                    return true;
            } else
                return true;
        },



        //method to scroll ganttchart from gridtree scroll event       

        onScrollHelper: function (scrollTop) {

            var proxy = this;
			if (!proxy._$bodyContainer.ejScroller("isVScroll"))
			{
                return false;
            }
            proxy._mouseLeave();

            var div = $("#ganttviewerbodyContianer" + this._id);
            
            $(proxy.element).find(".e-ganttviewerbodyContent").scrollTop(scrollTop);

            //After added new div in body container ,treegrid wheel event is not updated in chart part.
            $(proxy.element).find(".e-ganttviewerbodyContent").parent(".e-content").scrollTop(scrollTop);

            var h = parseInt(proxy.model.updatedRecords.length * 30) - (proxy._viewportHeight - 18);
            

            if (scrollTop > h) {
                scrollTop = h;
            }

            if (scrollTop < 0) {
                scrollTop = 0;
            }

            var args = { scrollTop: scrollTop };

            proxy._handleScroll(args);
            
            //var div = $("#ganttviewerbodyContianer" + this._id);
            //div.scrollTop(scrollTop);
            
        },



        //Handler for scroll event from ganttchart
        _handleScroll: function (args) {

            var proxy = this,
                args =args|| {},
                headerdiv = proxy._$headerContainer,
                contentdiv = proxy._$bodyContainer,
                left = proxy._prevLeft;

            
            if (args.scrollTop !== undefined) {
                proxy._scrollTop = args.scrollTop;
            }

            if (args.scrollLeft !== undefined) {
                left = args.scrollLeft;
            }

            var diff = (left - proxy._prevLeft),
                vDiff = proxy._scrollTop - proxy._prevScrollTop;

            if (diff) {
                headerdiv.scrollLeft(left);
            }
            else {
                headerdiv.scrollLeft(proxy._prevLeft);
            }

            if (proxy.model.enableVirtualization) {
                proxy._vscrollDist = Math.abs(proxy._scrollTop - proxy._prevScrollTop);
                if (proxy._vscrollDist) {
                    proxy._vScrollDir = proxy._prevScrollTop <= proxy._scrollTop ? 1 : -1;
                        proxy._updateCurrentViewData();

                    proxy.renderGanttRecords();
                    proxy._prevScrollTop = proxy._scrollTop;
                }
            }
            args.requestType = "scroll";
            args.delta = proxy._scrollTop;
            proxy._prevLeft = left;
            proxy._prevScrollTop = proxy._scrollTop;
            if (diff === 0 && vDiff !== 0) {
                proxy._completeAction(args);
            }
        },


        //helper method to refresh the ganttchart content from gantt
        refreshHelper: function (currentviewdata, records,collapseRecordsCount) {

            var proxy = this;
            proxy.model.currentViewData = currentviewdata;
            proxy.model.updatedRecords = records;
            if (collapseRecordsCount != undefined)
                proxy._totalCollapsedRecordCount = collapseRecordsCount;
            else
                proxy._totalCollapsedRecordCount = 0;

            if (records) {
                proxy._refresh();
                proxy._setHeight(records.length * proxy.model.rowHeight);
                proxy.updateHeight((records.length - proxy._totalCollapsedRecordCount) * proxy.model.rowHeight);
            }
        },
        _setScrollTop: function () {

            var proxy = this,
            model = this.model,
            top = proxy._scrollTop;

            if (model.enableVirtualization && top!==0) {
                if ((proxy.model.currentViewData.length * proxy.model.rowHeight) < proxy._viewportHeight) {
                    var currentPosition = top - (proxy._viewportHeight - proxy.model.currentViewData.length * proxy.model.rowHeight);
                    if (currentPosition < 0)
                        currentPosition = 0;
                    proxy._$bodyContainer.ejScroller("scrollY", currentPosition, true);
                }
            }
        },
  
        refreshGridLinesTable:function(length)
        {
            var proxy = this;
                proxy._$gridLinesTablebody.empty().append(proxy._createGridLinesTableRow(length));
                proxy._$bodyContainer.ejScroller("refresh");

                /*While refreshing scroller the gantt chart container height is increased by 16px.
                When horizontal scrollbar not avaiable in gantt chart side.
                So check the chart container height and view port height.
                If it is varies set the view port height to chart container height*/
                if (parseInt(proxy._$bodyContainer.css("height")) != proxy._viewportHeight)
                    proxy._$bodyContainer.css("height", proxy._viewportHeight);
        },
        //to refresh the content of the ganttchart
        _refresh: function () {

            var proxy = this, model = proxy.model,
                data = $.render[proxy._id + "_CustomTemplate2"](proxy.model.currentViewData);

            proxy._$ganttViewTablebody = $("#ganttViewTablebody" + proxy._id + "");

            proxy._$ganttViewTablebody.empty().append(data);

            //var $gridLinesTablebody = $("#gridLinesTablebody" + proxy._id + "");
            
            if (model.enableVirtualization) {
                proxy._$gridLinesTablebody.empty().append(proxy._createGridLinesTableRow(proxy.model.currentViewData.length));
            }
            else {
                proxy._$gridLinesTablebody.empty().append(proxy._createGridLinesTableRow(proxy.model.currentViewData.length - proxy._totalCollapsedRecordCount));
            }


            if (proxy._$ganttViewTablebody[0] != null) {
                this._ganttChartRows = $(proxy._$ganttViewTablebody[0].childNodes);
            }
            else {
                proxy._$ganttViewTablebody = ej.buildTag("tbody.e-gantt-viewer-ganttViewTablebody#ganttViewTablebody" + proxy._id + "",
                    data, {}, {});
                proxy._ganttChartRows = $(proxy._$ganttViewTablebody[0].childNodes);
            }
            if(model.queryTaskbarInfo)
                proxy._eventBindings();

        },


        //to rerender a particular ganttchart row
        refreshRow: function (index) {

            var proxy = this, model = this.model,
                $tr = $(proxy.getGanttChartRows()[index]),
                selectedItem = proxy.model.currentViewData[index];

            if (index != -1 && selectedItem) {
                $tr.replaceWith($.render[proxy._id + "_CustomTemplate2"](selectedItem));
                proxy._setGanttChartRows($(proxy._$ganttViewTablebody[0].childNodes));
            }
            if (model.queryTaskbarInfo)
                proxy._refreshedRowEventBinding(index);

        },


        //Trigger queryTaskbarInfo while refresh the row
        _refreshedRowEventBinding: function (index)
        {
            var $ganttChartRows = $(this.getGanttChartRows()),
                            taskbarContainer,
                            args = {}, progressbar, taskbar,
                            parentProgressbar, parentTaskbar, taskElement;

                    taskbarContainer = $ganttChartRows[index];

                    if ($(taskbarContainer).hasClass("e-ganttrowcell")) {

                        if (taskbarContainer.childNodes.length > 1)
                            taskElement = taskbarContainer.childNodes[1].firstChild;
                        else
                            taskElement = taskbarContainer.childNodes[0].firstChild;

                        if ($(taskElement).hasClass("e-parentContainer")) {

                            parentTaskbar = taskElement.firstChild;
                            parentProgressbar = taskElement.childNodes[1];
                        } else if ($(taskElement).hasClass("e-childContainer")) {

                            taskbar = taskElement.childNodes[1];
                            progressbar = taskElement.childNodes[2];
                        }

                        args.data = this.model.currentViewData[index];
                        args.taskbar = taskbarContainer;

                        args.taskbarBackground = taskbar != null ?
                            taskbar.style.backgroundColor : null;

                        args.progressbarBackground = progressbar != null ?
                            progressbar.style.backgroundColor : null;

                        args.parentTaskbarBackground = parentTaskbar != null ?
                            parentTaskbar.style.backgroundColor : null;

                        args.parentProgressbarBackground = parentProgressbar != null ?
                            parentProgressbar.style.backgroundColor : null;
                        this._queryTaskbarinfo(args);
                    }

        },
        //REFRESH THE SCROLLER FOR TREEGRIDCONTENT
        refreshScroller: function (panSize) {

            var proxy = this,maxScrollWidth;
            //proxy._$gridContent.css({'width':panSize+"px"});
            proxy._$bodyContainer.removeClass("e-borderbox");
            var scrollTop = proxy._$bodyContainer.ejScroller("option", "scrollTop");
            var scrollLeft = proxy._$bodyContainer.ejScroller("option", "scrollLeft");
            proxy._$bodyContainer.ejScroller("option", { "width": panSize });
            proxy._$bodyContainer.ejScroller("refresh");
            maxScrollWidth=proxy.getMaxScrollWidth();
            
            if (scrollLeft > maxScrollWidth)
                scrollLeft = maxScrollWidth > 0 ? maxScrollWidth : 0;

            var isHorizontalScroll = proxy._$bodyContainer.ejScroller("isHScroll");
            if (!isHorizontalScroll) {
                proxy._$bodyContainer.addClass("e-borderbox");
            }
            else {
                proxy._$bodyContainer.removeClass("e-borderbox");
            }
            var isVerticalScroll = proxy._$bodyContainer.ejScroller("isVScroll");
            if (isVerticalScroll)
            {
                proxy._$bodyContainer.ejScroller("scrollY", scrollTop, true);
            }
            proxy._$bodyContainer.ejScroller("option", "scrollLeft", scrollLeft);
            proxy._$headerContainer.scrollLeft(scrollLeft);

        },
        getMaxScrollWidth:function()
        {
            var proxy = this;
            return proxy._$bodyContainer.children(".e-content").children().width() - proxy._$bodyContainer.children(".e-content").width();
        },

        /* refresh stripLines Collection*/
        refreshStripLines:function (stripLines)
        {
            var proxy = this;
            if (stripLines) {
                this.model.stripLines = stripLines;
            }
            proxy._$stripLineContainer.empty();
            this._renderStripLines(this.model.stripLines);

        },
        /* refresh holidays Collection*/
        refreshHolidays: function (holidays) {
            //Display Holidays
            var proxy = this;
            if (holidays) {
                proxy.model.holidays = holidays
            }
            proxy._$secondaryCanvas.empty();
            if (proxy.model.holidays != null) {
                var holidayCount = proxy.model.holidays.length;
                for (var i = 0; i < holidayCount; i++) {
                    proxy._renderHoliday(proxy.model.holidays[i]);
                }
            }
        },

        //insert new record in DOM 
        renderNewAddedRow:function(index, data)
        {
            var proxy = this, model = proxy.model,
            addedchartRow = $.render[this._id + "_CustomTemplate2"](data);
            if (proxy._ganttChartRows.length != 0) {
                if (proxy._ganttChartRows.length == index) {
                    proxy.getGanttChartRows().eq(index - 1).after(addedchartRow);
                } else {
                    proxy.getGanttChartRows().eq(index).before(addedchartRow);
                }
            } else {
                proxy._refresh();
            }
            proxy._setGanttChartRows($(proxy._$ganttViewTablebody[0].childNodes));
            proxy._$gridLinesTablebody.empty().append(proxy._createGridLinesTableRow(proxy.model.updatedRecords.length - proxy._totalCollapsedRecordCount));
            if (model.enableVirtualization === false) {
                proxy._setHeight((proxy.model.updatedRecords.length - proxy._totalCollapsedRecordCount) * proxy.model.rowHeight);

            } else {
                proxy._setHeight(proxy.model.updatedRecords.length * proxy.model.rowHeight);
            }
            proxy.updateHeight(proxy._containerHeight);

        },

        _setGanttChartRows: function (value) {
            this._ganttChartRows = value;
        },

        refreshChartHeader:function(startdate,enddate)
        {
            var proxy = this,
            model = proxy.model,
            scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType,
            $colsgroup, $column, $thead,
            $tr, $td, $div, $table;
            proxy._updateScheduleWidth();
            if (scheduleMode == "month") {
                proxy._createScheduleTopMonthTemplate();
            }
            
            startdate = startdate == null ? new Date() : startdate;
            enddate = enddate == null ? new Date(null) : enddate;
            proxy._$scheduleDiv.empty();
            $table = ej.buildTag("table.e-schedule-headerrow-week", "",
                { 'display': 'block' },
                { 'cell-spacing': '0' }
            );

            $colsgroup = ej.buildTag("colgroup", "", {}, {});
            $column = $(document.createElement("col"));
            $column.css("width", proxy._getScheduleLength(scheduleMode) + 'px');

            $colsgroup.append($column);

            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});

            if (scheduleMode == "week")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleWeekTemplate"](model.scheduleWeeks), {}, {});
            else if (scheduleMode == "year")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleYearTemplate"](model.scheduleYears), {}, {});
            else if (scheduleMode == "month")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopMonthTemplate"](model.scheduleMonths), {}, {});
            else if (scheduleMode == "day")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopDayTemplate"](model.scheduleDays), {}, {});
            else if (scheduleMode == "hour")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopHourTemplate"](model.scheduleHours), {}, {});

            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);

            $tr = null;

            $table = ej.buildTag("table.e-schedule-headerrow-day", "",
                {
                    'display': 'block',
                    'border-collapse': 'collapse'
                }, { 'cell-spacing': '0' });
            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});

            if (scheduleMode == "week")
                $tr = ej.buildTag("tr", $.render[this._id + "_CustomTemplate1"](proxy.model.scheduleWeeks), {}, {});
            if (scheduleMode == "year")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleMonthTemplate"](proxy.model.scheduleYears), {}, {});
            if (scheduleMode == "month") {
                proxy._numOfWeeks = proxy._getNumberOftWeeks(this.model.projectStartDate, this.model.projectEndDate);
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleBottomWeekTemplate"](proxy._numOfWeeks), {}, {});
            }
            if (scheduleMode == "day")
                $tr = ej.buildTag("tr", $.render[this._id + "scheduleHourTemplate"](proxy._getScheduleHours()), {}, {});
            if (scheduleMode == "hour")
                $tr = ej.buildTag("tr", $.render[this._id + "scheduleMinuteTemplate"](proxy._getScheduleMinutes()), {}, {});

            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);
        },


        //update all containers width
        refreshContainersWidth:function()
        {
            var proxy = this,
                width, model = this.model,
                scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType;
            width = proxy._getScheduleLength(scheduleMode),
            $gridLineContainerTable=$("#ganttgridLinesTable" + proxy._id);

            proxy._$bodyContent.width(width);//main body container
            if (scheduleMode === "week")
                proxy.refreshWeekEndContainer();//weekENd Container
            $gridLineContainerTable.width(width);
            $gridLineContainerTable.children("col").width(width);
            proxy._$ganttViewTable.width(width);
            proxy._$ganttViewTable.children("col").width(width);
            proxy._createTaskbarTemplate();
            proxy._$secondaryCanvas.empty();
            proxy._$stripLineContainer.empty();
            if (proxy.model.stripLines != null) {
                this._renderStripLines(proxy.model.stripLines);
            }
            if (proxy.model.holidays != null) {
                var holidayCount = proxy.model.holidays.length;
                for (var i = 0; i < holidayCount; i++) {
                    proxy._renderHoliday(proxy.model.holidays[i]);
                }
            }
        },
        //refresh week ends when schedule dates changes
        refreshWeekEndContainer:function()
        {
            var proxy = this,
                left = 0,
                width = (proxy.model.perDayWidth) + 1,
                height = ((proxy.model.updatedRecords.length) * this.model.rowHeight),
                count = proxy.model.scheduleWeeks.length,
                $sunday, $saturday,
                $weekendContainer = proxy._$weekendsContainer.children('.e-secondary_canvas');
                $weekendContainer.empty();


                for (var i = 0; i < count; i++) {
                    $sunday = ej.buildTag("div.e-weekends", "", {
                        'left': left + 'px',
                        'width': width + 'px',
                        'height': height + 'px',
                        'background-color': this.model.weekendBackground
                    }, {});
                    $saturday = ej.buildTag("div.e-weekends", "", {
                        'left': (left + (proxy.model.perDayWidth * 6)) + 'px',
                        'width': width + 'px',
                        'height': height + 'px',
                        'background-color': this.model.weekendBackground
                    }, {});
                    $weekendContainer.append($sunday);
                    $weekendContainer.append($saturday);
                    left = left + this._scheduleWeekWidth;
                }
                proxy._$weekendsContainer.append($weekendContainer);
        },
        //delete gantt chart rows without rerendering of whole page
        deleteChartRows: function (startIndex, count) {
            var proxy = this;
            var $chartRows = proxy.getGanttChartRows();
            for (var i = startIndex; i < startIndex + count; i++) {
                $chartRows.splice(i, 1);
            }
        },


        //helper method to bind querytaskbarinfo events to ganttchart rows
        _eventBindings: function () {

            var $ganttChartRows = $(this.getGanttChartRows()),
                rowsLength = $ganttChartRows.length,
                taskbarContainer,
                args = {}, progressbar, taskbar,
                parentProgressbar, parentTaskbar, taskElement;

            if (rowsLength != 0) {

                for (var row = 0; row < rowsLength; row++) {

                    taskbarContainer = $ganttChartRows[row];

                    if ($(taskbarContainer).hasClass("e-ganttrowcell")) {

                        if (taskbarContainer.childNodes.length > 1)
                            taskElement = taskbarContainer.childNodes[1].firstChild;
                        else
                            taskElement = taskbarContainer.childNodes[0].firstChild;

                        if ($(taskElement).hasClass("e-parentContainer")) {

                            parentTaskbar = taskElement.firstChild;
                            parentProgressbar = taskElement.childNodes[1];
                        } else if ($(taskElement).hasClass("e-childContainer")) {

                            taskbar = taskElement.childNodes[1];
                            progressbar = taskElement.childNodes[2];
                        }

                        args.data = this.model.currentViewData[row];
                        args.taskbar = taskbarContainer;

                        args.taskbarBackground = taskbar != null ?
                            taskbar.style.backgroundColor : null;

                        args.progressbarBackground = progressbar != null ?
                            progressbar.style.backgroundColor : null;

                        args.parentTaskbarBackground = parentTaskbar != null ?
                            parentTaskbar.style.backgroundColor : null;

                        args.parentProgressbarBackground = parentProgressbar != null ?
                            parentProgressbar.style.backgroundColor : null;
                        this._queryTaskbarinfo(args);
                    }
                }
            }
        },


        //to update the currentviewdata of ganttchart
        _updateCurrentViewData: function () {
            var proxy = this, margin = 0;
            if (!this.model.enableVirtualization) {
                this.model.currentViewData = this.model.updatedRecords;
            }
            else {
                this._getRenderedRowRange();

                //console.log("Gantt Chart Rendered Range Top: " + proxy._renderedRange.top + "Bottom :" + proxy._renderedRange.bottom);

                this.model.currentViewData = this.model.updatedRecords.slice(proxy._renderedRange.top,
                    proxy._renderedRange.bottom);
                if (proxy._vscrollDist !== 0) {
                    margin = proxy._scrollTop;
                    margin -= proxy._rowMargin;
                }
                $("#ganttViewTable" + proxy._id + "").css({ "top": margin + "px" });

                $("#ganttgridLinesTable" + proxy._id + "").css({ "top": margin + "px" });

                proxy._vTop = proxy._visibleRange.top;
                proxy._vBottom = proxy._visibleRange.bottom;
                proxy._rTop = proxy._renderedRange.top;
                proxy._rBottom = proxy._renderedRange.bottom;
            }

        },

       
        //to render ganttchart content
        renderGanttRecords: function () {

            var proxy = this;
            proxy._refresh();
            proxy._$ganttViewTable.append(proxy._$ganttViewTablebody);
            proxy._$bodyContent.append(proxy._$ganttViewTable);
        },

        _getRenderedRowRange: function () {

            var proxy = this;
            proxy._getVisibleRowRange();
            var topIndex = proxy._visibleRange.top;
            var bottomIndex = proxy._visibleRange.bottom;
            proxy._renderedRange = { top: topIndex, bottom: bottomIndex };
            return proxy._renderedRange;
        },

        _getVisibleRowRange: function () {

            var proxy = this,
                top = proxy._scrollTop / proxy.model.rowHeight,
                coeff = top - Math.floor(top),
                topIndex,
                bottomIndex, length;

            //console.log("GanttChart ScrollTop :" + proxy._scrollTop);

            proxy._rowMargin = coeff * proxy.model.rowHeight;
            topIndex = Math.floor(top);
            length = proxy.model.updatedRecords.length;
            bottomIndex = proxy._getrowposition(proxy._scrollTop + proxy._viewportHeight);
            topIndex = Math.max(0, topIndex);
            bottomIndex = Math.min(length, bottomIndex);
            proxy._visibleRange = { top: topIndex, bottom: bottomIndex };
        },

        _getrowposition: function (y) {
            return Math.ceil((y) / this.model.rowHeight);
        },


        //returns the date object
        _getDateType: function (date) {

            if (date != null) {
                if (typeof (date) == "object") {
                    return date;
                } else {
                    var re = new RegExp("\\-", "g");
                    var re1 = new RegExp("\\.", "g");
                    date = date.replace(re, "/");
                    date = date.replace(re1, "/");

                    return new Date(date);
                }
            }
            return null;
        },

        // Returns the date as per date format
        _getDateFromFormat: function (date) {

            if (typeof date === "object") {
                return new Date(date);
            }
            if (date) {
                return Globalize.parseDate(date, this.model.dateFormat) == null ?
                    new Date(date) : Globalize.parseDate(date, this.model.dateFormat);
            }
        },

        _createScheduleTopDayTemplate: function () {
            var proxy = this,
                model = proxy.model,
                dayFormat = model.scheduleHeaderSettings.dayHeaderFormat,
                helpers = {
                    renderDayTop: $.proxy(proxy._getDaySplit, proxy),
                };
            $.views.helpers(helpers);

            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";

            td = "<div class='e-schedule-week-headercell-content'  style='width:"
                + ((proxy.model.perHourWidth*24)-1) + "px;margin:auto;white-space:nowrap;'>"
                + "{{:~renderDayTop('" + dayFormat + "',#data)}}" + "</div>" +
            "</div>"

            parentTr += td;
            parentTr += "</th>";

            var templates = {};
            templates[proxy._id + "ScheduleTopDayTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper methood for day schedule mode

        _getDaySplit: function (dateformat,data) {
            var date = new Date(data),
               proxy = this,
               model = proxy.model,
               formatedDate;

            formatedDate = Globalize.format(date, dateformat, model.locale);
            return formatedDate;
        },


        // Month Week Schedule Mode

        _createScheduleTopMonthTemplate: function () {

            var proxy = this, td, i = 0,
                model = proxy.model,
                monthFormat = model.scheduleHeaderSettings.monthHeaderFormat,
                helpers = {
                    renderMonthTop: $.proxy(proxy._getMonthsplit, proxy),
                    setMonthWidth: $.proxy(proxy._setMonthWidth, proxy),
                    isLastMonth: $.proxy(proxy._checkLastMonthWidth, proxy),
                };

            $.views.helpers(helpers);
            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";

            td = "<div class='e-schedule-week-headercell-content'  style='margin:auto;white-space:nowrap;width:{{:~setMonthWidth(#data)-1}}" + "px;'>"
                + "{{:~renderMonthTop('" + monthFormat + "',#data)}}" + "</div>"
                 +"{{if ~isLastMonth(#data)}}"
                 + "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>"
                    + "<div class='e-schedule-week-headercell-content' style='margin:auto;white-space:nowrap;width:" + (proxy._balanceWidth-1) + "px;'>"
                +"</div></th>{{/if}}"
            
                parentTr += td;
                parentTr += "</th>";
                //if (proxy._balanceWidth) {
                //    parentTr += tdx;
                //    parentTr += "</th>";
                //}

            var templates = {};
            templates[proxy._id + "ScheduleTopMonthTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper method for calculating unique month width
        _setMonthWidth: function (data) {
            var proxy = this;
            var date;
            if (typeof data === "object") {
                date = new Date(data);
            }
            else {
                date = Globalize.parseDate(data, "MM/dd/yyyy");
            }

            var monthLastDay = new Date(date.getFullYear(), date.getMonth() + 1, 0);

            var lastDate = monthLastDay.getDate();
            if (lastDate == 31)
                return (this.model.perDayWidth * 31);
            else if (lastDate == 30)
                return (this.model.perDayWidth * 30);
            else if (lastDate == 28)
                return (this.model.perDayWidth * 28);
            else if (lastDate == 29)
                return (this.model.perDayWidth * 29);
        },

        _checkLastMonthWidth: function (data) {
            var proxy = this;
            if (data == proxy.model.scheduleMonths[proxy.model.scheduleMonths.length - 1] && proxy._balanceWidth>0) {
                return true;
            }
            else
            {
                return false;
            }
        },


        //helper method for _createScheduleMonthTemplate
        _getMonthsplit: function (monthFormat, data) {

            var date = new Date(data),
                proxy = this,
                model = proxy.model,
                formatedDate;

            formatedDate = Globalize.format(date, monthFormat, model.locale);
            return formatedDate;

        },


        _createScheduleYearTemplate: function () {

            var proxy = this, td,
                model = proxy.model,
                yearFormat = model.scheduleHeaderSettings.yearHeaderFormat,
                helpers = {
                    renderYear: $.proxy(proxy._getYearsplit, proxy),
                    getYearWidth: $.proxy(proxy._getTemplateYearWidth, proxy),
                };

            $.views.helpers(helpers);
            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";
            
            td = "<div class='e-schedule-week-headercell-content'  style='margin:auto;white-space:nowrap;width:{{:~getYearWidth(#data)-1}}px'>"
                + "{{:~renderYear('" + yearFormat + "',#data)}}" + "</div>";

            parentTr += td;
            parentTr += "</th>";

            var templates = {};
            templates[proxy._id + "ScheduleYearTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper method for _createScheduleYearTemplate
        _getYearsplit: function (yearFormat, data) {

            var date = new Date(data),
                proxy = this,
                model = proxy.model,
                formatedDate;

            formatedDate = Globalize.format(date, yearFormat, model.locale);
            return formatedDate;

        },
        _getTemplateYearWidth: function (date) {

            var year = date.getFullYear();
            if (new Date(year, 1, 29).getMonth() === 1) {
                return 366 * this.model.perDayWidth;
            }
            else
                return 365 * this.model.perDayWidth;
        },
        //Template to render schedule week view
        _createScheduleWeekTemplate: function () {

            var proxy = this, td,
                model = proxy.model,
                dateFormat = model.scheduleHeaderSettings.weekHeaderFormat,
                helpers = {
                    renderWeek: $.proxy(proxy._getWeeksplit, proxy)
                };

            $.views.helpers(helpers);
            var parentTr = "<th class='e-schedule-week-headercell' style='box-sizing:border-box;-moz-box-sizing:border-box;'>";

            td = "<div class='e-schedule-week-headercell-content'  style='width:"
                + (proxy._scheduleWeekWidth - 1) + "px;margin:auto;white-space:nowrap;'>"
                + "{{:~renderWeek('" + dateFormat + "',#data)}}" + "</div>";

            parentTr += td;
            parentTr += "</th>";

            var templates = {};
            templates[proxy._id + "ScheduleWeekTemplate"] = parentTr;
            $.templates(templates);
        },

        //helper method for _createScheduleWeekTemplate
        _getWeeksplit: function (dateFormat, data) {

            var date = new Date(data),
                proxy = this,
                model = proxy.model,
                formatedDate;
            //var values = dateFormat.split(' '),
            //    scheduleUnits = values[0].match(/(\d+|[A-z]+)/g),
            //    month = date.getMonth(),
            //    day = date.getDay() == 1 ? proxy._days("mon") : "",
            //    weekTemplate = [];

            //switch (month) {
            //    case 0:
            //        month = proxy._getLocalizedLabels("jan");
            //        break;
            //    case 1:
            //        month = proxy._getLocalizedLabels("feb");
            //        break;
            //    case 2:
            //        month = proxy._getLocalizedLabels("mar");
            //        break;
            //    case 3:
            //        month = proxy._getLocalizedLabels("apr");
            //        break;
            //    case 4:
            //        month = proxy._getLocalizedLabels("may");
            //        break;
            //    case 5:
            //        month = proxy._getLocalizedLabels("jun");
            //        break;
            //    case 6:
            //        month = proxy._getLocalizedLabels("jul");
            //        break;
            //    case 7:
            //        month = proxy._getLocalizedLabels("aug");
            //        break;
            //    case 8:
            //        month = proxy._getLocalizedLabels("sep");
            //        break;
            //    case 9:
            //        month = proxy._getLocalizedLabels("oct");
            //        break;
            //    case 10:
            //        month = proxy._getLocalizedLabels("nov");
            //        break;
            //    case 11:
            //        month = proxy._getLocalizedLabels("dec");
            //        break;
            //}

            //for (var i = 0; i < scheduleUnits.length; i++) {
            //    switch (scheduleUnits[i]) {//('0' + currentDate.getHours()).slice(-2)
            //        case "d":
            //        case "D":
            //            weekTemplate.push(('0' + date.getDate()).slice(-2));
            //            break;
            //        case "dd":
            //        case "DD":
            //            weekTemplate.push(date.getDate());
            //            break;
            //        case "ddd":
            //        case "DDD":
            //            weekTemplate.push(day.substring(0, 3));
            //            break;
            //        case "dddd":
            //        case "DDDD":
            //            weekTemplate.push(day);
            //            break;
            //        case "M":
            //        case "m":
            //            weekTemplate.push(('0' + date.getMonth() + 1).slice(-2));
            //            break;
            //        case "MM":
            //        case "mm":
            //            weekTemplate.push(date.getMonth() + 1);
            //            break;
            //        case "MMM":
            //        case "mmm":
            //            weekTemplate.push(month.substring(0, 3));
            //            break;
            //        case "MMMM":
            //        case "mmmm":
            //            weekTemplate.push(month);
            //            break;
            //        case "YY":
            //        case "yy":
            //            weekTemplate.push(('0' + date.getFullYear()).slice(-2));
            //            break;
            //        case "YYYY":
            //        case "yyyy":
            //            weekTemplate.push(date.getFullYear());
            //            break;
            //    }
            //}
            formatedDate = Globalize.format(date, dateFormat, model.locale);

            return formatedDate.toLowerCase().replace(/(-(\w)|(^| )(\w)|:(\w)|,(\w)|\/(\w)|\.(\w))/g, function (x) {
                return x.toUpperCase();
            });
        },

        _createScheduleHourTemplate: function () {
            var proxy = this, model = proxy.model,td,
                hourHeaderFormat = model.scheduleHeaderSettings.hourHeaderFormat,
                helpers = {
                    renderHour:$.proxy(proxy._getHourTemplate,proxy),
                };
            $.views.helpers(helpers);
            var parentTr = "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perHourWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderHour('" + hourHeaderFormat + "',#data)}}</div>";

            parentTr += td;

            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "scheduleHourTemplate"] = parentTr;
            $.templates(templates);
        },
        //Method to create template for upper part of schedule header in hour-minute schedule mode.
        _createScheduleTopHourTemplate: function () {
            var proxy = this, model = proxy.model, minuteInterval = model.minuteInterval, td, interval,
                hourHeaderFormat = model.scheduleHeaderSettings.topHourHeaderFormat,
                helpers = {
                    renderHour: $.proxy(proxy._getHourTemplate, proxy),
                };
            if (minuteInterval == 30)
                interval = 2;
            if (minuteInterval == 15)
                interval = 4;
            if (minuteInterval == 5)
                interval= 12;
            if (minuteInterval == 1)
                interval = 60;
            proxy._totalInterval = interval;//To calculate schedule length
            $.views.helpers(helpers);
            var parentTr = "<th  class='e-schedule-hour-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-hour-headercell-content' style='width:"
                + ((proxy.model.perMinuteWidth*interval)-1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderHour('" + hourHeaderFormat + "',#data)}}</div>";

            parentTr += td;

            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "ScheduleTopHourTemplate"] = parentTr;
            $.templates(templates);
        },
        //Method to create template for lower part of schedule header in hour-minute schedule mode.
        _createMinuteTemplate: function () {
            var proxy = this, model = proxy.model, td,
                hourHeaderFormat = model.scheduleHeaderSettings.minuteHeaderFormat,
                helpers = {
                    renderHour: $.proxy(proxy._getHourTemplate, proxy),
                };
            $.views.helpers(helpers);
            var parentTr = "<th  class='e-schedule-hour-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-hour-headercell-content' style='width:"
                + (proxy.model.perMinuteWidth-1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderHour('" + hourHeaderFormat + "',#data)}}</div>";

            parentTr += td;

            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "scheduleMinuteTemplate"] = parentTr;
            $.templates(templates);
        },
        _getHourTemplate: function (format,data) {
            var date = new Date(data),
              proxy = this,
              model = proxy.model,
              formatedDate;

            formatedDate = Globalize.format(date, format, model.locale);
            return formatedDate;
        },


        //Template to render schedule week view in month-week schedule mode
        _createScheduleBottomWeekTemplate: function () {

            var proxy = this, td, model = proxy.model,
                weekFormat = model.scheduleHeaderSettings.weekHeaderFormat,
                helpers = {
                    renderWeek: $.proxy(proxy._getMonthWeekTemplate, proxy)
                };

            $.views.helpers(helpers);

            var parentTr = "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;padding-left:2px;width:" + (proxy.model.perWeekWidth-1) + "px;'>" +
                "{{:~renderWeek('" + weekFormat + "',#data)}}</div>";

            parentTr += td;

            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "ScheduleBottomWeekTemplate"] = parentTr;
            $.templates(templates);
        },

        _getMonthWeekTemplate: function (weekFormat, data) {

            var date = new Date(data),
                proxy = this, model = proxy.model;

            //date.setDate(date.getDate() + (weekLength * 7));

            switch (weekFormat) {
                case "d":
                case "dd":
                    //return (('0' + date.getDate()).slice(-2));
                case "ddd":

                default: return Globalize.format(date, weekFormat, model.locale);
            }
        },




        //Template to render schedule day view
        _createScheduleMonthTemplate: function () {

            var proxy = this, td, model = proxy.model,
                monthFormat = model.scheduleHeaderSettings.monthHeaderFormat,
                helpers = {
                    renderMonth: $.proxy(proxy._getMonthTemplate, proxy),
                    getMonthWidth: $.proxy(proxy._setMonthWidthYearMode, proxy)
                };

            $.views.helpers(helpers);

            var parentTr = "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>";
            td = "<div class='e-schedule-day-headercell-content' style='"
                + "position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(0,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',0,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(1,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',1,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(2,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',2,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(3,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',3,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(4,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',4,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(5,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',5,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(6,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',6,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(7,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',7,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(8,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',8,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(9,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',9,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(10,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',10,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip;width:{{:~getMonthWidth(11,#data)-1}}px;'>" +
                "{{:~renderMonth('" + monthFormat + "',11,#data)}}</div>";

            
            parentTr += td;
            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "ScheduleMonthTemplate"] = parentTr;
            $.templates(templates);
        },

        _getMonthTemplate: function (monthFormat, monthLength, data) {

            var date = new Date(data),
                proxy = this, model = proxy.model;
            date.setMonth(date.getMonth() + monthLength);

            switch (monthFormat) {
                case "M":
                case "m":
                    return (('0' + date.getDate()).slice(-2));
                case "MM":
                case "mm":
                    return date.getDate();
                case "MMM":
                case "mmm":
                case "MMMM":
                case "mmmm":
                    return Globalize.format(date, monthFormat, model.locale);
                default: return Globalize.format(date, monthFormat, model.locale);
            }
        },

        //For year template set month width accordance with num of days
        _setMonthWidthYearMode: function (monthLength, data) {

            var proxy = this;
            var date=new Date(data);
            date.setMonth(date.getMonth() + monthLength);

            var monthLastDay = new Date(date.getFullYear(), date.getMonth() + 1, 0);

            var lastDate = monthLastDay.getDate();
            if (lastDate == 31)
                return (this.model.perDayWidth * 31);
            else if (lastDate == 30)
                return (this.model.perDayWidth * 30);
            else if (lastDate == 28)
                return (this.model.perDayWidth * 28);
            else if (lastDate == 29)
                return (this.model.perDayWidth * 29);

        },


        //Template to render schedule day view
        _createScheduleDayTemplate: function () {

            var proxy = this, td, model = proxy.model,
                dayFormat = model.scheduleHeaderSettings.dayHeaderFormat,
                helpers = {
                    renderDay: $.proxy(proxy._getDayTemplate, proxy)
                };

            $.views.helpers(helpers);

            var parentTr = "<th  class='e-schedule-day-headercell  e-headercell-weekend' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;" +
                "background-color:" + this.model.scheduleHeaderSettings.weekendBackground + "'>";
            td = "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',0,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',1,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',2,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',3,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell'" +
                " style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',4,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell'" +
                " style='-moz-box-sizing:border-box;box-sizing:border-box;'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',5,#data)}}</div>" +

                "<th  class='e-schedule-day-headercell  e-headercell-weekend' " +
                "style='-moz-box-sizing:border-box;box-sizing:border-box;" +
                "background-color:" + this.model.scheduleHeaderSettings.weekendBackground + "'>" +
                "<div class='e-schedule-day-headercell-content' style='width:"
                + (proxy.model.perDayWidth - 1) + "px;position:static;box-sizing:border-box;overflow:hidden;text-overflow:clip'>" +
                "{{:~renderDay('" + dayFormat + "',6,#data)}}</div>";

            parentTr += td;
            parentTr += "</th>";
            var templates = {};
            templates[proxy._id + "_CustomTemplate1"] = parentTr;
            $.templates(templates);
        },

        _getDayTemplate: function (dayFormat, dayLength, data) {

            var date = new Date(data),
                proxy = this, model = proxy.model;
            date.setDate(date.getDate() + dayLength);

            switch (dayFormat) {
                case "D":
                case "d":
                    return (('0' + date.getDate()).slice(-2));
                case "DD":
                case "dd":
                    return date.getDate();
                case "DDD":
                case "ddd":
                case "DDDD":
                case "dddd":
                    return Globalize.format(date, dayFormat, model.locale)[0].toUpperCase();
                default: return Globalize.format(date, dayFormat, model.locale)[0].toUpperCase();
            }
        },


        //Method to render schedule header
        _createSchedule: function (startdate, enddate) {

            var proxy = this,
                model=proxy.model,
                scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType,
                $colsgroup, $column, $thead,
                $tr, $td, $div, $table;

            startdate = startdate == null ? new Date() : startdate;
            enddate = enddate == null ? new Date(null) : enddate;

            proxy._createScheduleWeekTemplate();
            proxy._createScheduleYearTemplate();
            proxy._createScheduleTopMonthTemplate();//template for MONTH week schedule mode
            proxy._createScheduleTopDayTemplate();
            proxy._createScheduleTopHourTemplate();
            $table = ej.buildTag("table.e-schedule-headerrow-week", "",
                { 'display': 'block' },
                { 'cell-spacing': '0' }
            );

            $colsgroup = ej.buildTag("colgroup", "", {}, {});
            $column = $(document.createElement("col"));
            $column.css("width", proxy._getScheduleLength(scheduleMode) + 'px');
            
            $colsgroup.append($column);

            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});

            if (scheduleMode == "week")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleWeekTemplate"](proxy.model.scheduleWeeks), {}, {});
            else if (scheduleMode == "year")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleYearTemplate"](proxy.model.scheduleYears), {}, {});
            else if (scheduleMode == "month")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopMonthTemplate"](proxy.model.scheduleMonths), {}, {});
            else if (scheduleMode == "day")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopDayTemplate"](proxy.model.scheduleDays), {}, {});
            else if (scheduleMode == "hour")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleTopHourTemplate"](proxy.model.scheduleHours), {}, {});

            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);

            $tr = null;

            proxy._createScheduleDayTemplate();
            proxy._createScheduleMonthTemplate();//for year - month schedule mode
            proxy._createScheduleBottomWeekTemplate(); //for month - week schedule mode
            proxy._createMinuteTemplate();
            proxy._createScheduleHourTemplate();

            $table = ej.buildTag("table.e-schedule-headerrow-day", "",
                {
                    'display': 'block',
                    'border-collapse': 'collapse'
                }, { 'cell-spacing': '0' });
            $thead = ej.buildTag("thead.e-ejganttschedule", "",
                { 'display': 'block', 'border-collapse': 'collapse' }, {});
            
            if (scheduleMode == "week")
                $tr = ej.buildTag("tr", $.render[this._id + "_CustomTemplate1"](model.scheduleWeeks), {}, {});
            if (scheduleMode == "year")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleMonthTemplate"](model.scheduleYears), {}, {});
            if (scheduleMode == "month")
                $tr = ej.buildTag("tr", $.render[this._id + "ScheduleBottomWeekTemplate"](proxy._numOfWeeks), {}, {});
            if (scheduleMode == "day")
                $tr = ej.buildTag("tr", $.render[this._id + "scheduleHourTemplate"](proxy._getScheduleHours()), {}, {});
            if (scheduleMode == "hour")               
                 $tr = ej.buildTag("tr", $.render[this._id + "scheduleMinuteTemplate"](proxy._getScheduleMinutes()), {}, {});

            $td = ej.buildTag("th.schedule", "", {}, {});
            $div = ej.buildTag("div.schedule", "", { 'width': 20 + 'px' }, {});
            $td.append($div);
            $tr.append($td);

            $thead.append($tr);
            $table.append($colsgroup);
            $table.append($thead);
            proxy._$scheduleDiv.append($table);

            proxy._$headerContainer.append(proxy._$scheduleDiv);
            proxy._$ganttChartContainer.append(proxy._$headerContainer);
            $("#" + proxy._id).append(proxy._$ganttChartContainer);
        },

        _getScheduleHours: function () {
            var proxy = this, model = proxy.model,
                startHour = new Date(proxy.model.projectStartDate),
                endHour = new Date(model.projectEndDate);
                proxy._scheduleHours = [];
                endHour.setHours(24);
            do {
                proxy._scheduleHours.push(new Date(startHour));
                startHour.setHours(startHour.getHours() + 1);
            } while (!(startHour.getTime()>= endHour.getTime()));

            return proxy._scheduleHours;
        },
        //Calculate total number of minutes to render template.
        _getScheduleMinutes: function () {
            var proxy = this, model = proxy.model,                
                startDate = new Date(proxy.model.projectStartDate),
                endDate = new Date(model.projectEndDate);
            endDate.setMinutes(60);
            startDate.setMinutes(00);
            proxy._scheduleMinutes = [];
            do {
                proxy._scheduleMinutes.push(new Date(startDate));
                startDate.setMinutes(startDate.getMinutes() + model.minuteInterval);
            } while (!(startDate.getTime() >= endDate.getTime()));
        
            return proxy._scheduleMinutes;
        },
        createGridLinesTable: function () {
            var proxy=this,
                model = proxy.model,
                $gridLinesTable,
                gridLineTable,
                scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType,
                scheduleLength,
                tableRows;

            $gridLinesTable = ej.buildTag("table.e-ganttgridLinesTable#ganttgridLinesTable" + proxy._id, "", {
                'z-index': '2',
                'position': 'absolute',
            }, { 'cellspacing': '0px' });
            var $colsgroup = ej.buildTag("colgroup", "", {}, {});
            var $column = $(document.createElement("col"));

            switch (scheduleMode) {
                case "week":
                    scheduleLength = model.scheduleWeeks.length,
                    $gridLinesTable.css("width",(scheduleLength * proxy._scheduleWeekWidth) + 'px');
                    $column.css("width", (scheduleLength * this._scheduleWeekWidth) + 'px');
                    break;

                case "year":
                    scheduleLength = model.scheduleYears.length,                    
                    $gridLinesTable.css("width", (proxy._scheduleYearWidth) + 'px');
                    $column.css("width", (this._scheduleYearWidth) + 'px');

                    break;
                case "month":
                    scheduleLength = model.scheduleMonths.length,
                    $gridLinesTable.css("width", (proxy._scheduleMonthWidth) + 'px');
                    $column.css("width", (this._scheduleMonthWidth) + 'px');
                    break;
                case "day":
                    scheduleLength = model.scheduleDays.length,
                    $gridLinesTable.css("width", (scheduleLength * (proxy.model.perHourWidth*24)) + 'px');
                    $column.css("width", (scheduleLength * (proxy.model.perHourWidth * 24)) + 'px');
                    break;
                case "hour":
                    scheduleLength = model.scheduleHours.length,
                    $gridLinesTable.css("width", (scheduleLength * (proxy.model.perMinuteWidth * proxy._totalInterval)) + 'px');
                    $column.css("width", (scheduleLength * (proxy.model.perMinuteWidth * proxy._totalInterval)) + 'px');
                    break;
                default:
                    break;

            }

            $colsgroup.append($column);
            $gridLinesTable.append($colsgroup);

            tableRows = proxy._createGridLinesTableRow(model.updatedRecords.length);
            
            var $gridLinesTablebody = ej.buildTag("tbody.e-gantt-viewer-gridLinesTablebody#gridLinesTablebody" + proxy._id + "",
                tableRows, {}, {});
            proxy._$gridLinesTablebody = $gridLinesTablebody;
            $gridLinesTable.append(proxy._$gridLinesTablebody);
            proxy._$bodyContent.append($gridLinesTable);
        },


        _createGridLinesTableRow: function (length) {
            var proxy = this,
                tableRows="";

            for (var i = 0; i < length; i++) {
                tableRows += "<tr style='display:table-row;height:" + this.model.rowHeight + "px'><td class='e-ganttgridlines'></td></tr>"
            }
            return tableRows;
        },


        //Template to render taskbar and milestone items
        _createTaskbarTemplate: function () {

            var proxy = this,
                model = proxy.model,
                tdWidth,
                scheduleMode = model.scheduleHeaderSettings.scheduleHeaderType,
                tr, baselineHeight = 4, taskbarHeight,
                shrinkedTaskbarHeight = 0,
                baselineColor = model.baselineColor,
                helpers = {
                    _getType: proxy._getTypedata,
                    _expander: proxy._addRecordExpandCollapse,
                    _milestoneMapping: proxy._isMilestone,
                    _resources: $.proxy(proxy._getResourceInfo, proxy),
                    _rowClassName: ej.TreeGrid._getrowClassName,//proxy._getrowClassName,
                    _expandStatus: $.proxy(proxy._getExpandStatusRecord, proxy),
                    _borderRadius: proxy._getBorderRadius,
                };

            if (scheduleMode == "week") {
                tdWidth = model.scheduleWeeks.length * proxy._scheduleWeekWidth;
            }
            else if (scheduleMode == "year") {
                tdWidth = proxy._scheduleYearWidth;
            }
            else if (scheduleMode == "month") {
                tdWidth = proxy._scheduleMonthWidth;
            }
            else if (scheduleMode == "day") {
                tdWidth = model.scheduleDays.length * (model.perHourWidth*24);
            }
            else if (scheduleMode == "hour") {
                tdWidth = model.scheduleHours.length * (model.perMinuteWidth * proxy._totalInterval);
            }
            if (model.renderBaseline) {
                //taskbarHeight = model.rowHeight - 10;
                taskbarHeight = model.rowHeight - baselineHeight - 5;
                shrinkedTaskbarHeight = 0;
            } else {
                taskbarHeight = model.rowHeight - 10;
            }

            $.views.helpers(helpers);

            if (!model.progressMapping) {


                this.model.showProgressStatus = false;
            }

            if (!model.resourceInfoMapping) {
                model.showResourceNames = false;
            }

            var parentTr = "<tr class='{{:~_rowClassName()}}  {{if isSelected}}e-ganttrowcell {{:~_expander()}} e-gantt-mouseclick{{else}}e-ganttrowcell {{:~_expander()}}{{/if}}' id='chartrowcell"
                + proxy._id + "' style='display:{{:~_expandStatus(#data)}};background-color:{{if rowBGColor }}{{:rowBGColor}}{{else}} none{{/if}};'>";
            var radius = "{{:~_borderRadius()}}";

            tr = "{{if ~_getType()}}" +

                //Parent taskbar Item

                "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width}}px;width:{{:left+width}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;" +
                "font-weight:bold'>{{:taskName}}</span></div></td>{{/if}}" +

                "<td  class='e-chartcell' vertical-align='middle' style='width:" + tdWidth + "px;'>" +

               "<div class='e-parentContainer' unselectable='on' style='height:" + (this.model.rowHeight-1) + "px;z-index:3;'>" +

                "<div class='e-connectorpoint-left' style='left:{{:left-10}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +

                "<div class='e-gantt-parenttaskbar-innerdiv {{:~_expander()}}' style='left:{{:left}}px;margin-top:5px;width:{{:width}}px;height:"
                + (taskbarHeight) + "px;background-color:{{if parentTaskbarBackground }}{{:parentTaskbarBackground}}{{else}} " + this.model.parentTaskbarBackground + "{{/if}} ;'></div>" +
                "<div  class='e-gantt-parenttaskbar-progress {{:~_expander()}}  progressbar' id='progressbar' " +
                "style='margin-top:5px;border-style:{{if progressWidth}}solid{{else}}none{{/if}}; left:{{:left}}px;width:{{:progressWidth}}px;height:" + (taskbarHeight)
                + "px;position:absolute;background-color:{{if parentProgressbarBackground }}{{:parentProgressbarBackground}}{{else}} " + this.model.parentProgressbarBackground + "{{/if}};z-index:3;" +
                "border-top-right-radius:" + radius + "px;border-bottom-right-radius:" + radius + "px;'></div>" +

                  "<div class='e-connectorpoint-right' style='left:{{:left+width}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +

             "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div  class='e-baselinebar' style='margin-top:-4px;left:{{:baselineLeft}}px;" +
                "width:{{:baselineWidth}}px;height:" + baselineHeight + "px;background-color: " + baselineColor + ";" +
                "position:absolute;z-index:3;box-sizing:border-box;-moz-box-sizing:border-box;'></div>{{/if}}" +

                "</td>" +

               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='margin-top:5px;left:{{:left+width+30}}px;" +
                "margin:0;margin-left:5px;margin-top:-9px;position:absolute;background-color:transparent;font-weight:bold'>" +
                "<span class='e-resourcename'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;outline:none;'>"
                + "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}" +


               //Milestone Item

                "{{else ~_milestoneMapping()}}" +
                    
                "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width}}px;width:{{:left+width}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:taskName}}</span></div></td>{{/if}}" +

               "<td class='e-chartcell' style='width:" + tdWidth + "px;'>" +
                "<div class='e-gantt-milestone-container' style=height:" + (this.model.rowHeight - 1) + "px;>" +
                "<div class='e-connectorpoint-left' style='left:{{:left-10}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +
                "<div class='e-gantt-milestone' style='position:absolute;left:{{:left}}px;'>" +
                "<div class='e-gantt-milestone milestone-top'  style='top:" + (3) + "px;'></div>" +
                "<div class='e-gantt-milestone milestone-bottom'  style='top:" + (15) + "px;'></div>" +
                "</div>"+
                 "<div class='e-connectorpoint-right' style='left:{{:left+24}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +
                "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div class='e-baseline-gantt-milestone-container' >" +
                "<div class='e-baseline-gantt-milestone' style='position:absolute;left:{{:baselineLeft}}px;z-index:2;'>" +
                "<div class='e-baseline-gantt-milestone baseline-milestone-top'  " +
                "style=' border-bottom-color: " + baselineColor + ";z-index:2;top:" + (-26) + "px;'></div>" +
                "<div class='e-baseline-gantt-milestone baseline-milestone-bottom'  " +
                "style=' border-top-color: " + baselineColor + ";z-index:2;top:" + (-14) + "px;'></div>" +
                "</div></div>{{/if}}" +

                "</td>" +

               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='margin-top:5px;left:{{:left+56}}px;margin:0;margin-left:5px;margin-top:-9px;" +
                "position:absolute;background-color:transparent;'><span class='e-resourcename'  style='margin-top:"
                + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" + "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}" +

                //Child Taskbar Item

                "{{else}}" +

                //left label
               "{{if " + this.model.showTaskNames + "}}<td class='e-chartcell'>" +
                "<div class='e-tasknameContainer'  unselectable='on' style='left:-{{:width}}px;width:{{:left+width}}px;'>" +
                "<span class='e-taskbarname'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:taskName}}</span></div></td>{{/if}}" +

                //taskbar
               "<td class='e-chartcell' style='width:" + tdWidth + "px;'>" +

               

               "<div  class='e-childContainer'  unselectable='on' style='height:" + (this.model.rowHeight - 1) + "px;'>" +

               "<div class='e-connectorpoint-left' style='left:{{:left-10}}px;width:10px;margin-top:10px;height:" + (taskbarHeight/2) + "px;'></div>"+

              "<div class='taskbarresizer-left e-icon' style='left:{{:left+2}}px;width:10px;margin-top:5px;height:" + (taskbarHeight) + "px;'></div>" +
            
               "<div  class='e-gantt-childtaskbar' style='left:{{:left}}px;width:{{:width}}px;height:"
                + (taskbarHeight) + "px;background-color:{{if taskbarBackground }}{{:taskbarBackground}}{{else}} " + this.model.taskbarBackground + "{{/if}};" + "margin-top:5px;'></div>" +

               "<div  class='e-gantt-childtaskbar-progress  progressbar' id='progressbar' style='border-style:{{if progressWidth}}solid{{else}}none{{/if}}; left:{{:left}}px;" +
                "width:{{:progressWidth}}px;text-align:right;line-height:" + (taskbarHeight) + "px;margin-top:5px;" +
                //"margin-top:" + (((taskbarHeight) - (this.model.progressbarHeight * ((taskbarHeight) / 100))) / 2) + "px;" +
                "height:" + (this.model.progressbarHeight * ((taskbarHeight) / 100)) + "px;" +
                "position:absolute;background-color:{{if progressbarBackground }}{{:progressbarBackground}}{{else}} " + this.model.progressbarBackground + "{{/if}};" +
                "border-top-right-radius:" + radius + "px;border-bottom-right-radius:" + radius + "px;'>" +
                "{{if " + this.model.showProgressStatus + "}}<span class='e-tasklabel' style=''>{{:status}}%</span>{{/if}}</div>" +

               "<div class='progressbarresizer-right' style='left:{{:left+progressWidth-10}}px;width:20px;margin-top:5px;" +
                "height:" + (taskbarHeight) + "px;position:absolute;z-index:4;'>" +
               "<div class='progressbarresizer-right' style='top:" + (22 - (shrinkedTaskbarHeight / 2)) + "px;margin-top:5px;position:absolute;'><div class='e-progresshandler-element'></div></div>" +//-" + (shrinkedTaskbarHeight/2) + "
               "<div class='progressbarresizer-right' style='top:" + (17 - (shrinkedTaskbarHeight / 2)) + "px;margin-top:5px;position:absolute;'><div class='e-progresshandlerafter-element'></div></div></div>" +

               "<div class='taskbarresizer-right e-icon' style='left:{{:left+width-12}}px;width:10px;" +
                "height:" + (taskbarHeight) + "px;position:absolute;z-index:2;margin-top:5px;'></div>" +
               
                "<div class='e-connectorpoint-right' style='left:{{:left+width}}px;width:10px;margin-top:10px;height:" + (taskbarHeight / 2) + "px;'></div>" +
                //"<div class='e-connectorpoint-right' style='left:{{:left-20}}px;width:20px;margin-top:10px;background-color:black;height:" + (taskbarHeight) + "px;'></div>"+
                 "</div>" +

                "{{if " + this.model.renderBaseline + " }}" +
                "<div  class='e-baselinebar' style='margin-top:-4px;left:{{:baselineLeft}}px;" +
                "width:{{:baselineWidth}}px;height:" + baselineHeight + "px;background-color: " + baselineColor + ";" +
                "position:absolute;z-index:3;box-sizing:border-box;-moz-box-sizing:border-box;'></div>{{/if}}" +

                "</td>" +

                //right label
               "{{if " + this.model.showResourceNames + "}}<td class='e-chartcell' >" +
                "<div class='e-resourceinfo'  unselectable='on' style='left:{{:left+width+30}}px;margin:0;margin-left:5px;margin-top:-9px;" +
                "position:absolute;text-align:left;background-color:transparent;'><span class='e-resourcename'  style='margin-top:" + ((this.model.rowHeight) / 2) + "px;margin-right:30px;'>" +
                "{{:~_resources(#data)}}" + "</span></div></td>{{/if}}{{/if}}";

            parentTr += tr;
            parentTr += "</tr>";

            var templates = {};
            templates[this._id + "_CustomTemplate2"] = parentTr;
            $.templates(templates);

        },

        _getResourceInfo: function (data) {
            var resource = null;
            if (data.resourceInfo != null) {
                var resourcelength = data.resourceInfo.length;
                if (resourcelength > 0) {
                    for (var i = 0; i < resourcelength; i++) {
                        if (resource == null)
                            resource = data.resourceInfo[i][this.model.resourceNameMapping];
                        else
                            resource += " , " + data.resourceInfo[i][this.model.resourceNameMapping];
                    }
                    return resource;
                } else
                    return data.resourceInfo[0][this.model.resourceNameMapping];
            }
            return String.empty;
        },

        _getrowClassName: function () {
            var rowClass = "gridrowtaskId",
                proxy = this;
            if (proxy.data.parentItem)
                rowClass += proxy.data.parentItem.taskId.toString();
            rowClass += "level";
            rowClass += proxy.data.level.toString();
            return rowClass;
        },

        _getExpandStatusRecord: function (data) {
            var proxy = this;
            if (proxy._getExpandStatus(data)) {
                return 'table-row';
            }
            return 'none';
        },


        _isMilestone: function () {
            if (this.data.isMilestone)
                return true;
            else
                return false;
        },

        _addRecordExpandCollapse: function () {
            if (this.data.expanded)
                return 'e-chartexpand';
            else if (!this.data.expanded && this.data.hasChildRecords) {
                return 'e-chartcollapse';
            }
            return false;
        },

        _getTypedata: function () {
            return this.data.hasChildRecords;
        },


        _getBorderRadius: function () {

            var proxy = this,
                diff = proxy.data.width - proxy.data.progressWidth;

            if (diff <= 4) {
                return 4 - diff;
            }
            else
                return 0;

        },


        _createConnectorLineTemplate: function () {
            var proxy = this,
                lineStroke = (proxy.model.connectorlineWidth) > 4 ? 4 : proxy.model.connectorlineWidth,
                shrinkedTaskbarHeight = 0,
                deltaTop, taskbarMidpoint,
                lineColor = proxy.model.connectorLineBackground,
                helpers = {
                    _getWidthVal: proxy._getWidth,
                    _getTop: proxy._getPredecessorTop,
                    _getPosition: proxy._getParentPosition,
                    _setContainerWidthSSType2: proxy._getContainerWidthSSType2,
                    _setInnerChildWidthSSType2: proxy._getInnerChildWidthSSType2,
                    _setInnerElementLeftSSType2: proxy._getInnerElementLeftSSType2,
                    _setInnerElementWidthSSType2: proxy._getInnerElementWidthSSType2,
                    _isMilestone: proxy._isTaskMilestone,
                    _isMilestoneParent: proxy._isMilestoneParent,
                };

            if (proxy.model.renderBaseline) {
                deltaTop = shrinkedTaskbarHeight / 2;
                taskbarMidpoint = ((proxy.model.rowHeight) / 2) - deltaTop;
            } else {
                taskbarMidpoint = ((proxy.model.rowHeight) / 2);
            }

            $.views.helpers(helpers);
            var connectorContainer = "<div  id='ConnectorLine{{:ConnectorLineId}}' style='background-color=black'>";

            var div = "{{if ~_getPosition()=='FSType1'}}"//FS
            + "<div style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:(ChildLeft - (ParentLeft + ParentWidth + 20))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if milestoneParent}}-7px{{else}}0px{{/if}};width:{{if milestoneParent}}{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-2}}px{{else}}{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px{{/if}};" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px;" +
                "width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft-(ParentLeft + ParentWidth + 10))+" + lineStroke + "-10}}px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='FSType2'}}"//FS
            + "<div style='left:{{:ParentLeft}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:~_getWidthVal()}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if milestoneParent}}{{:ParentWidth-7}}px{{else}}{{:ParentWidth}}px{{/if}};width:{{if milestoneParent}}17px{{else}}10px{{/if}};height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:ParentWidth+10-" + lineStroke + "}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-14-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div><div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;" +
                "width:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30 )}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;width:" + lineStroke + "px;" +
                "height:{{:14-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 20))}}px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ParentWidth - (((ParentLeft + ParentWidth) - ChildLeft) + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;top:{{:-6-" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FSType3'}}"
            + "<div style='left:{{:ChildLeft-20}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentWidth+ChildWidth}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;" +
                "top:{{:-5-" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative;top:-12.5px;'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-14-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative;top:-12.5px;'></div>"
            + "<div class='e-line' style='width:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30)}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative;top:-12.5px;'></div>"
            + "<div class='e-line' style='left:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 30)}}px;width:" + lineStroke + "px;" +
                "height:{{:14-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative;" +
                "top:-12.5px;'></div>"
            + "<div class='e-line' style='{{if ~_isMilestoneParent()}}left:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 14)}}px;width:16px;{{else}}left:{{:(((ParentLeft + ParentWidth) - ChildLeft) + 20)}}px;width:10px;{{/if}}height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative;top:-12.5px;'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FSType4'}}"
            + "<div style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:(ChildLeft - (ParentLeft + ParentWidth + 20))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth + 10))}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-5-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='left:{{:(ChildLeft - (ParentLeft + ParentWidth) - 20)}}px;top:-13px;width:10px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;left:{{:(ChildLeft - (ParentLeft + ParentWidth) - 20)}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestoneParent()}}left:-6px;{{/if}}top:-12.5px;width:{{if ~_isMilestoneParent()}}{{:(ChildLeft-(ParentLeft + ParentWidth+20 )+ 6)+" + lineStroke + "}}px;{{else}}{{:(ChildLeft-(ParentLeft + ParentWidth + 20))+" + lineStroke + "}}px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType4'}}"
            + "<div  style='left:{{:ParentLeft-10}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ChildLeft-ParentLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:{{:ChildLeft-ParentLeft}}px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:ChildLeft-ParentLeft}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType3'}}"
            + "<div  style='left:{{:ChildLeft-20}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentLeft-ChildLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:ParentLeft-ChildLeft+20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType2'}}"
            + "<div  style='left:{{:~_setInnerElementLeftSSType2()}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:~_setContainerWidthSSType2()}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:{{:~_setInnerChildWidthSSType2()}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:~_setInnerElementWidthSSType2()}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:{{:~_setInnerElementWidthSSType2()}}px;" +
                "border-left: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;" +
                "height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SSType1'}}"
            + "<div  style='left:{{:ChildLeft-20}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ParentLeft-ChildLeft}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:{{:ParentLeft-ChildLeft+20}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='connectorline-rightarrow' style='left:10px;border-left: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType1'}}"
            + "<div class='FFtype1' style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft + ChildWidth)-(ParentLeft + ParentWidth))+30}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{if ~_isMilestoneParent()}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))-10}}px;{{else}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))}}px;{{/if}}" +
                "width:{{if ~_isMilestoneParent()}}30px;{{else}}20px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{if ~_isMilestoneParent()}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;{{else}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;{{/if}}width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{if ~_isMilestone()}}4px;{{else}}10px;{{/if}}width:{{if ~_isMilestone()}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+16}}px;{{else}}{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+10}}px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='{{if ~_isMilestone()}}left:-6px;{{/if}}border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType2'}}"
            + "<div style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='{{if ~_isMilestoneParent()}}left:-6px;{{/if}}width:{{if ~_isMilestoneParent()}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+26}}px;{{else}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;{{/if}}height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{if ~_isMilestone()}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+4}}px;{{else}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+10}}px;{{/if}}" +
                "width:{{if ~_isMilestone()}}16px;{{else}}10px;{{/if}}" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='left:{{if ~_isMilestone()}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))-6}}px;{{else}}{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))}}px;{{/if}}" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType3'}}"
            + "<div  style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='{{if ~_isMilestone()}}left:-6px;{{/if}}border-right: 10px solid " + lineColor + ";" +
                "top:{{:-5-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestone()}}left:4px;width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+16}}px;{{else}}left:10px;width:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+10}}px;{{/if}}top:-12.5px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))+20}}px;top:-12.5px;" +
                "width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestoneParent()}}left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))-6}}px;width:26px;{{else}}left:{{:((ParentLeft + ParentWidth) - (ChildLeft + ChildWidth))}}px;width:20px;{{/if}}top:-12.5px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='FFType4'}}"
            + "<div  style='left:{{:ParentLeft+ParentWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div  class='connectorline-leftarrow' style='{{if ~_isMilestone()}}left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))-6}}px;{{else}}left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))}}px;{{/if}}" +
                "border-right: 10px solid " + lineColor + ";top:{{:-5-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;" +
                "position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestone()}}left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+4}}px;width:16px;{{else}}left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+10}}px;width:10px;{{/if}}top:-12.5px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;top:-12.5px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestoneParent()}}left:-6px;width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+26}}px;{{else}}width:{{:((ChildLeft +ChildWidth)-(ParentLeft + ParentWidth))+20}}px;{{/if}}top:-12.5px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='SFType4'}}"//SF
            + "<div style='left:{{:ParentLeft-10}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;" +
                "height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='left:{{if ~_isMilestone()}}{{:((ChildLeft + ChildWidth) - (ParentLeft ))+4}}px;{{else}}{{:((ChildLeft + ChildWidth) - (ParentLeft ))+10}}px;{{/if}}" +
                "border-right: 10px solid " + lineColor + ";top:{{:-5-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;" +
                "position:relative'></div>"
            + "<div class='e-line' style='left:{{if ~_isMilestone()}}{{:((ChildLeft + ChildWidth) - (ParentLeft ))+14}}px;width:16px;{{else}}{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;width:10px;{{/if}}top:-13px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;top:-13px;width:" + lineStroke + "px;" +
                "height:{{:Height-15-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:" + lineStroke + "px;height:{{:15-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='top:-13px;width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SFType3'}}"//SF
            + "<div style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ChildTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft)-(ChildLeft + ChildWidth))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='connectorline-leftarrow' style='{{if ~_isMilestone()}}left:-6px;{{/if}}border-right: 10px solid " + lineColor + ";" +
                "top:{{:-5-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;" +
                "border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestone()}}left:4px;width:16px;{{else}}left:10px;width:10px;{{/if}}top:-13px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:20px;top:-13px;width:" + lineStroke + "px;height:{{:Height-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:20px;top:-13px;width:{{:(ChildLeft-(ParentLeft + ParentWidth + 20))+" + lineStroke + "}}px;" +
                "height:" + lineStroke + "px;background-color:" + lineColor + ";position:relative'></div>"
            + "</div>"

            + "{{else  ~_getPosition()=='SFType1'}}"//SF
            + "<div style='left:{{:ParentLeft-10}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:ChildLeft+ChildWidth+20}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='width:10px;height:" + lineStroke + "px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='width:" + lineStroke + "px;height:{{:Height-15-" + lineStroke + "}}px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='width:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+30}}px;width:" + lineStroke + "px;" +
                "height:{{:15-" + lineStroke + "}}px;background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestone()}}left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+(15-" + lineStroke +")}}px;width:16px;{{else}}left:{{:((ChildLeft + ChildWidth) - (ParentLeft ))+20}}px;width:10px;{{/if}}height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='left:{{if ~_isMilestone()}}{{:((ChildLeft + ChildWidth) - (ParentLeft ))+4}}px;{{else}}{{:((ChildLeft + ChildWidth) - (ParentLeft ))+10}}px;{{/if}}" +
                "border-right: 10px solid " + lineColor + ";top:{{:-6-" + lineStroke + "}}px;" +
                "border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;" +
                "width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{else ~_getPosition()=='SFType2'}}"//SF
            + "<div  style='left:{{:ChildLeft+ChildWidth}}px;top:{{:ParentTop+" + taskbarMidpoint + "}}px;" +
                "width:{{:((ParentLeft) - (ChildLeft + ChildWidth))}}px;height:{{:Height}}px;position:absolute'>"
            + "<div class='e-line' style='left:{{:((ParentLeft) - (ChildLeft + ChildWidth))-10}}px;width:10px;height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='e-line' style='left:{{:((ParentLeft) - (ChildLeft + ChildWidth))-10}}px;width:" + lineStroke + "px;" +
                "height:{{:Height-" + lineStroke + "}}px;background-color:" + lineColor + ";" +
                "position:relative'></div>"
            + "<div class='e-line' style='{{if ~_isMilestone()}}left:0px;width:{{:((ParentLeft) - (ChildLeft + ChildWidth))-10}}px;{{else}}left:10px;width:{{:((ParentLeft) - (ChildLeft + ChildWidth))-20}}px;{{/if}}height:" + lineStroke + "px;" +
                "background-color:" + lineColor + ";position:relative'></div>"
            + "<div class='connectorline-leftarrow' style='left:{{if ~_isMilestone()}}-6px;{{else}}0px;{{/if}}border-right: 10px solid " + lineColor + ";" +
                "top:{{:-6-" + lineStroke + "}}px;border-bottom-width:{{:5+" + lineStroke + "}}px;border-top-width:{{:5+" + lineStroke + "}}px;width:0;height:0;position:relative'></div>"
            + "</div>"

            + "{{/if}}";

            connectorContainer += div;
            connectorContainer += "</div>";

            var templates = {};
            templates[this._id + "ConnectorLineTemplate"] = connectorContainer;
            $.templates(templates);
        },

        _isTaskMilestone: function () {
            if (this.data.milestoneChild)
                return true;
            else
                return false;
            
        },

        _isMilestoneParent: function () {
            if (this.data.milestoneParent)
                return true;
            else
                return false;
        },

        _getInnerChildWidthSSType2: function () {
            if ((this.data.ParentLeft + this.data.ParentWidth) < this.data.ChildLeft) {
                return 10;
            }
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 20;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },

        _getInnerElementLeftSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return (this.data.ParentLeft - 20);
            }
            return (this.data.ParentLeft - 10);
        },

        _getInnerElementWidthSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 10;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },


        _getContainerWidthSSType2: function () {
            if (this.data.ParentLeft == this.data.ChildLeft) {
                return 20;
            }
            return (this.data.ChildLeft - this.data.ParentLeft);
        },

        _getPredecessorTop: function () {
            if (this.data.ParentTop > this.data.ChildTop)
                return true;
            else
                return false;
        },

        _getParentPosition: function () {
            if (this.data.ParentTop < this.data.ChildTop) {

                if (this.data.Type == "FF") {
                    if ((this.data.ChildLeft + this.data.ChildWidth) >= (this.data.ParentLeft + this.data.ParentWidth)) {
                        return "FFType2";
                    }
                    else {
                        return "FFType1";
                    }
                }

                else if ((this.data.ParentLeft < this.data.ChildLeft) && (this.data.ChildLeft > (this.data.ParentLeft + this.data.ParentWidth + 25))) {
                    if (this.data.Type == "FS") return "FSType1";
                    if (this.data.Type == "SF") return "SFType1";
                    else if (this.data.Type == "SS") return "SSType2";
                    else if (this.data.Type == "FF") return "FFType2";
                }
                else if ((this.data.ParentLeft < this.data.ChildLeft && (this.data.ChildLeft < (this.data.ParentLeft + this.data.ParentWidth)))
                    || (this.data.ParentLeft == this.data.ChildLeft || this.data.ParentLeft > this.data.ChildLeft)) {

                    if (this.data.ParentLeft > (this.data.ChildLeft + this.data.ChildWidth + 25)) {
                        if (this.data.Type == "SF") return "SFType2";
                    }

                    if (this.data.ParentLeft > this.data.ChildLeft) {
                        if (this.data.Type == "SS") return "SSType1";
                        if (this.data.Type == "SF") return "SFType1";
                        if (this.data.Type == "FF") return "FFType1";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType2";
                    }
                    if (this.data.Type == "FS") return "FSType2";
                    else if (this.data.Type == "SS") return "SSType2";
                    else if (this.data.Type == "FF") return "FFType1";
                    else if (this.data.Type == "SF") return "SFType1";
                }
                else if ((this.data.ParentLeft) < this.data.ChildLeft) {

                    if (this.data.Type == "FS") return "FSType2";
                    else if (this.data.Type == "FF") return "FFType2";
                    else if (this.data.Type == "SS") return "SSType2";
                    else if (this.data.Type == "SF") return "SFType1";
                }

            }
            else if (this.data.ParentTop > this.data.ChildTop) {
                if ((this.data.ParentLeft < this.data.ChildLeft) && (this.data.ChildLeft > (this.data.ParentLeft + this.data.ParentWidth))) {
                    if (this.data.Type == "FS")
                        if (this.data.milestoneParent && this.data.ChildLeft == (this.data.milestoneParent ? (this.data.ParentLeft + this.data.ParentWidth + 4) : (this.data.ParentLeft + this.data.ParentWidth)))
                            return "FSType3";
                        else
                            return "FSType4"
                    if (this.data.ParentLeft < this.data.ChildLeft || ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth))) {
                        if (this.data.Type == "SS") return "SSType4";
                        if (this.data.Type == "FF") return "FFType4";
                        if (this.data.Type == "SF") return "SFType4";
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType4";
                    }
                }
                else if ((this.data.ParentLeft < this.data.ChildLeft && (this.data.ChildLeft < (this.data.ParentLeft + this.data.ParentWidth)))
                    || (this.data.ParentLeft == this.data.ChildLeft || this.data.ParentLeft > this.data.ChildLeft)) {
                    if ((this.data.ChildLeft + this.data.ChildWidth) <= (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType3";
                        if (this.data.Type == "SF") {
                            if ((this.data.ChildLeft + this.data.ChildWidth) < (this.data.ParentLeft)) {
                                return "SFType3";
                            } else {
                                return "SFType4";
                            }
                        }
                        if (this.data.Type == "SS") {
                            if (this.data.ChildLeft <= this.data.ParentLeft)
                                return "SSType3";
                            else
                                 return "SSType4";
                        }
                    }
                    else if ((this.data.ChildLeft + this.data.ChildWidth) > (this.data.ParentLeft + this.data.ParentWidth)) {
                        if (this.data.Type == "FF") return "FFType4";
                        if (this.data.Type == "SF") return "SFType4";
                        if (this.data.Type == "SS")
                            if (this.data.ChildLeft <= this.data.ParentLeft)
                                return "SSType3";
                            else
                                return "SSType4";
                    }
                    if (this.data.Type == "FS") return "FSType3";
                }
                else if (this.data.ParentLeft < this.data.ChildLeft) {

                    if (this.data.Type == "FS") return "FSType3";
                    if (this.data.Type == "SS") return "SSType4";
                    if (this.data.Type == "FF") return "FFType4";
                    if (this.data.Type == "SF") return "SFType4";
                }
            }
            return false;
        },

        _getWidth: function () {
            return ((this.data.ChildLeft + this.data.ChildWidth) < (this.data.ParentLeft + this.data.ParentWidth)) ?
                this.data.ParentWidth : (this.data.ChildWidth + this.data.ParentWidth);
        },


        //renders Weekends in chart
        _renderWeekends: function () {
            var proxy = this,
                left = 0,
                width = (proxy.model.perDayWidth) + 1,
                height = ((proxy.model.updatedRecords.length) * this.model.rowHeight),
                count = proxy.model.scheduleWeeks.length,
                $sunday, $saturday,
                $weekendContainer = ej.buildTag("div.e-secondary_canvas", "", { 'height': height + 'px' }, {});

            for (var i = 0; i < count; i++) {
                $sunday = ej.buildTag("div.e-weekends", "", {
                    'left': left + 'px',
                    'width': width + 'px',
                    'height': height + 'px',
                    'background-color': this.model.weekendBackground
                }, {});
                $saturday = ej.buildTag("div.e-weekends", "", {
                    'left': (left + (proxy.model.perDayWidth * 6)) + 'px',
                    'width': width + 'px',
                    'height': height + 'px',
                    'background-color': this.model.weekendBackground
                }, {});
                $weekendContainer.append($sunday);
                $weekendContainer.append($saturday);
                left = left + this._scheduleWeekWidth;
            }

            proxy._$weekendsContainer.append($weekendContainer);
            proxy._$bodyContent.append(proxy._$weekendsContainer);
        },



        //Renders StripLines for given striplines collection object
        _renderStripLines: function (stripLines) {

            var proxy = this,
                height = ((proxy.model.updatedRecords.length) * this.model.rowHeight),
                stripLineCount = stripLines.length, day, left,
                projectStartDate = proxy.model.projectStartDate,
                dayWidth = this.model.perDayWidth,
                stripLineContainer = ej.buildTag("div.e-stripLines", "", { 'height': height + 'px' }, {});

            for (var i = 0; i < stripLineCount; i++) {

                day = this._getDateFromFormat(stripLines[i].day);
                left = ((day - this._getDateFromFormat(projectStartDate)) / 86400000) * dayWidth;
                stripLines[i].lineWidth = stripLines[i].lineWidth > 30 ? 30 : stripLines[i].lineWidth;

                var stripline = ej.buildTag("div.e-stripline#stripline" + [i] + "", "", {
                    'left': left + 'px',
                    'width': '1px',
                    'height': height + 'px',
                    'border-left-width': stripLines[i].lineWidth + 'px',
                    'border-left-color': stripLines[i].lineColor,
                    'border-left-style': stripLines[i].lineStyle
                }, {});

                var innerspan = ej.buildTag("span.e-striplinespan", stripLines[i].label, {
                    'background-color': stripLines[i].lineColor,
                }, {});

                if (stripLines[i].day && stripLines[i].day.length > 0) {
                    if (stripLines[i].label && stripLines[i].label.length > 0) {
                        stripline.append(innerspan);
                    }
                    stripLineContainer.append(stripline);
                }
            }
            proxy._$stripLineContainer.append(stripLineContainer);
            proxy._$bodyContent.append(proxy._$stripLineContainer);
        },


        //returns progress width for the taskbar
        _getProgressWidth: function (parentwidth, percent) {
            return (parentwidth * percent) / 100;
        },


        //returns progress percentage of the taskbar
        _getProgressPercent: function (parentwidth, progresswidth) {
            return Math.ceil(((progresswidth / parentwidth) * 100).toFixed(2));
        },


        //renders holidays for holiday collection object
        _renderHoliday: function (holiday) {

            var proxy = this, scheduleHeaderType = proxy.model.scheduleHeaderSettings.scheduleHeaderType,
                date = this._getDateFromFormat(holiday.day),
                weekCount = proxy.model.scheduleWeeks.length,
                monthcount = proxy.model.scheduleMonths.length,
                yearcount = proxy.model.scheduleYears.length,
                daycount=proxy.model.scheduleDays.length,
                prevDate = 0,
                left = 0, textLength, text, $holidayContainer, $holidaySpan,
                ganttRecords = proxy.model.updatedRecords.length, height;

            left = ((date - this._getDateFromFormat(proxy.model.projectStartDate)) / 86400000) * this.model.perDayWidth;

            height = (ganttRecords * this.model.rowHeight);
            textLength = holiday.label != null ? holiday.label.length : 0;
            if (holiday.background == null)
                holiday.background = "white";


            var holidayLableSelector = ".e-holiday-label";
            /* for rotate holiday lable in ie8 */
            if (ej.browserInfo().name == "msie" && ej.browserInfo().version < 9) {
                holidayLableSelector = ".e-holiday-label-ie8";
            }
            if (proxy.model.scheduleHeaderSettings.scheduleHeaderType != "day") {
                text = ej.buildTag("div" + holidayLableSelector, holiday.label, {
                    'top': (proxy._getHolidayLabelHeight(textLength)) + 'px',
                }, {});
            }
            else
            {
                text = ej.buildTag("div" + holidayLableSelector, holiday.label, {
                    'bottom': (proxy._getHolidayLabelHeight(textLength)) + 'px',
                }, {});
            }
            $holidayContainer = ej.buildTag("div.e-secondary-canvas-holiday");
            $holidaySpan = ej.buildTag("div.e-holidays", "", {
                'left': left + 'px',
                'width': proxy.model.perDayWidth + 'px',
                'height': height + 'px',
                'background-color': holiday.background
            }, {});

            if (holiday.label != null)
                $holidaySpan.append(text);
            $holidayContainer.append($holidaySpan);

            proxy._$secondaryCanvas.append($holidayContainer);
            proxy._$bodyContent.append(proxy._$secondaryCanvas);
        },

        _getHolidayLabelHeight: function (textlength) {
            var proxy = this, model = proxy.model,
                containerHeight = proxy._getHeight(),
                totalHeight = model.updatedRecords.length * proxy.model.rowHeight;
            if (containerHeight > 0)
                return (containerHeight / 2);
            else if (proxy._viewportHeight < totalHeight)
                return (proxy._viewportHeight / 2);
            else
                return totalHeight / 2;

        },

        // all events bound using this._on will be unbind automatically
        _destroy: function () {

            var proxy = this;
            proxy.element.empty().removeClass("e-ganttchart-core e-ganttchart" + proxy.model.cssClass);

        },
    });

    ej.GanttChart.contextMenuItem = {
        taskDetails: "Task Details....",
        add: "Add new task",
        indent: "Indent",
        outdent: "Outdent",
        Delete: "Delete",
    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Radial Menu elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejRadialMenuBase", "ej.RadialMenuBase", {
        _tags: [{
            tag: "items",
            attr: ["imageName", "imagePath", "imageUrl", "windows.text", "flat.text", "text", "click", "touch", "enabled",
                "badge.enabled", "badge.value", [{
                    tag: "items", attr: ["imageName", "imagePath", "imageUrl", "windows.text", "flat.text", "text", "click",
                     "touch", "enabled"]
                }]]
        }],
        defaults: {
            radius: 150,
            cssClass: "",
            enableAnimation: true,
            open: null,
            close: null,
            items: []
        },
        dataTypes: {
            enableAnimation: "boolean",
            items: "array"
        },
        _initialization: function () {
            this.element.addClass(this.model.cssClass);
            this._svgLink = "http://www.w3.org/2000/svg";
            if (this.model.items.length < 1) {
                this._radialItems = $(this.element.find(">ul").children());
                for (var i = 0; i < this._radialItems.length ; i++) {
                    this.model.items.push(this._itemsObjectCollection($(this._radialItems[i])));
                    if ($(this._radialItems[i]).find("ul").children().length > 0) {
                        this._nestedRadialItems = $(this._radialItems[i]).find("ul").children();
                        this.model.items[i].items = [];
                        for (var j = 0; j < this._nestedRadialItems.length; j++) {
                            this.model.items[i].items.push(this._itemsObjectCollection($(this._nestedRadialItems[j])));
                        }
                    }
                }
            }
            this._itemCount = this.model.items.length;
            $(this.element.children()).remove();
            this._startXY = this.model.radius;
            this._diameter = 2 * this.model.radius;
        },
        _itemsObjectCollection: function (element) {
            var item = {}, innerItem = {};
            $(this._tags[0].attr).each(function (i, e) {
                if (typeof (e) == "string") {
                    var attr = e.split('.');
                    if (attr.length == 1)
                        item[e] = element.attr("data-ej-" + e);
                    else {
                        var prop1 = attr[0], prop2 = attr[1];
                        if (!item[prop1])
                            innerItem = {};
                        innerItem[prop2] = element.attr("data-ej-" + e.replace(".", "-"));
                        item[prop1] = innerItem;
                    }
                }
            });
            return item;
        },
        _createSVGElement: function () {
            var svgObj = document.createElementNS(this._svgLink, 'svg');
            if (this.model.radius != null) {
                svgObj.setAttribute("width", this._diameter);
                svgObj.setAttribute("height", this._diameter);
            }
            return $(svgObj);
        },
        _createGroupElement: function (id) {
            var group = document.createElementNS(this._svgLink, "g");
            if (id)
                group.setAttribute("id", id);
            return $(group);
        },
        _createImageElement: function (width, height, currentEle, x, y) {
            imgLoc = currentEle["imageUrl"] ? currentEle["imageUrl"] : (this.model.renderMode.toLowerCase() != "flat" ? currentEle["imagePath"] + "/"
                + this.model.renderMode.toLowerCase() + "/" + this.model.theme.toLowerCase() + "/" + currentEle["imageName"] : currentEle["imagePath"] + "/"
                + this.model.renderMode.toLowerCase() + "/" + currentEle["imageName"]),
            image = document.createElementNS(this._svgLink, "image");
            image.setAttribute("width", width);
            image.setAttribute("height", height);
            image.setAttributeNS('http://www.w3.org/1999/xlink', 'href', imgLoc);
            image.setAttribute("x", x);
            image.setAttribute("y", y);
            return image;
        },
        _createTextElement: function (x, y, style, content) {
            var text = document.createElementNS(this._svgLink, "text");
            text.setAttribute("x", x);
            text.setAttribute("y", y);
            text.setAttribute("class", style);
            text.textContent = content;
            return text;
        },
        _createPathElement: function (index, direction, style) {
            var path = document.createElementNS(this._svgLink, "path");
            path.setAttribute("index", index);
            path.setAttribute("d", direction);
            path.setAttribute("class", style);
            return path;
        },
        _createPolygonElement: function (index, transform) {
            var polygon = document.createElementNS(this._svgLink, "polygon");
            polygon.setAttribute("points", "10,10 0,10 5,5");
            polygon.setAttribute("index", index);
            polygon.setAttribute("fill", "#FFFFFF");
            polygon.setAttribute("transform", transform);
            return polygon;
        },
        _createCircleElement: function (x, y, r, style) {
            var circle = document.createElementNS(this._svgLink, "circle");
            circle.setAttribute("cx", x);
            circle.setAttribute("cy", y);
            circle.setAttribute("r", r);
            circle.setAttribute("class", style);
            return circle;
        },
        _calculateRadialArc: function (menuCount, size, mRadius, Arcdradius, polyRadius) {
            var itemCurrentPos = 0,
                startAngle = 2 * Math.PI * itemCurrentPos,
                endAngle = 2 * Math.PI * (itemCurrentPos + (10 / (menuCount * 10))),
                radius = 0.5 * size * Math.min(this._diameter, this._diameter),
                menuItemRadius = mRadius * size * Math.min(this._diameter, this._diameter),
                dradius = Arcdradius * radius,
                polygonRadius = polyRadius * radius,
                degPoint = 360 / menuCount,
                degree = degPoint / 2;
            var x1 = [], y1 = [], x2 = [], y2 = [], midx = [], midy = [], dStartX = [], dStartY = [], dEndX = [], dEndY = [], dmidx = [], dmidy = [], deg = [], longArc = [];
            for (var i = 0; i < menuCount; i++) {
                var pathStartAngle = -.5 * Math.PI;
                startAngle = startAngle + pathStartAngle;
                endAngle = endAngle + pathStartAngle - 0.000001;
                longArc[i] = endAngle - startAngle < Math.PI ? 0 : 1;
                var midAngle = (startAngle + endAngle) / 2;
                x1[i] = this._startXY + radius * Math.cos(startAngle);
                y1[i] = this._startXY + radius * Math.sin(startAngle);
                x2[i] = this._startXY + radius * Math.cos(endAngle);
                y2[i] = this._startXY + radius * Math.sin(endAngle);
                midx[i] = this._startXY + menuItemRadius * Math.cos(midAngle);
                midy[i] = this._startXY + menuItemRadius * Math.sin(midAngle);
                dStartX[i] = this._startXY + dradius * Math.cos(startAngle);
                dStartY[i] = this._startXY + dradius * Math.sin(startAngle);
                dEndX[i] = this._startXY + dradius * Math.cos(endAngle);
                dEndY[i] = this._startXY + dradius * Math.sin(endAngle);
                dmidx[i] = this._startXY + polygonRadius * Math.cos(midAngle);
                dmidy[i] = this._startXY + polygonRadius * Math.sin(midAngle);
                deg[i] = degree;
                itemCurrentPos += 10 / (menuCount * 10);
                startAngle = 2 * Math.PI * itemCurrentPos;
                endAngle = 2 * Math.PI * (itemCurrentPos + (10 / (menuCount * 10)));
                degree += degPoint;
            }
            return ({
                x1: x1, y1: y1, x2: x2, y2: y2, midx: midx, midy: midy, dStartX: dStartX, dStartY: dStartY, dEndX: dEndX, dEndY: dEndY, dmidx: dmidx,
                dmidy: dmidy, deg: deg, longArc: longArc, radius: radius, dradius: dradius
            });
        },
        _renderRadialMenu: function () {
            this._radialSVG = this._createSVGElement();
            this._radialChildSVG = this._createSVGElement();
            this._childArcGroup = this._createGroupElement("childarcgroup");
            this._polygonGroup = this._createGroupElement("polygongroup");
            this._polygonGroup.attr("transform", "translate(-10,-10)");
            this._radialArcGroup = this._createGroupElement("arcpathgroup");
            this._menuItemsGroup = this._createGroupElement("menuitemsgroup");
            this._menuItemsGroup.attr("transform", "translate(-10,-5)");
            this._menuItemsArcPaths = this._createGroupElement("menuitemsarcpaths");
            this._menuItemsPaths = this._createGroupElement("itemspathgroup");
            this._radialSVGDiv = ej.buildTag("div#" + this._id + "_svgdiv." + this._prefix + "abs").css("height", this._diameter);
            this._radialCircleSVGDiv = ej.buildTag("div#" + this._id + "_circlesvgdiv." + this._prefix + "abs " + this._prefix + "displaynone");
            this._radialChildSVGDiv = ej.buildTag("div#" + this._id + "_childsvgdiv." + this._prefix + "abs " + this._prefix + "displaynone");
            var pointsVal = this._calculateRadialArc(this._itemCount, 1, 0.28, 0.8, 0.95),
                pointsArcVal = this._calculateRadialArc(this._itemCount, 0.79, null, 0.97, null),
                pointsItemVal = this._calculateRadialArc(this._itemCount, 0.8, null, 0.3, null),
                imgDim = 20;
            for (var i = 0; i < this._itemCount; i++) {
                var direction = this._pathDirection(pointsVal.x1[i], pointsVal.y1[i], pointsVal.radius, pointsVal.longArc[i], pointsVal.x2[i], pointsVal.y2[i],
                    pointsVal.dEndX[i], pointsVal.dEndY[i], pointsVal.dradius, pointsVal.dStartX[i], pointsVal.dStartY[i]),
                    arcDirection = this._pathDirection(pointsArcVal.x1[i], pointsArcVal.y1[i], pointsArcVal.radius, pointsArcVal.longArc[i], pointsArcVal.x2[i],
                    pointsArcVal.y2[i], pointsArcVal.dEndX[i], pointsArcVal.dEndY[i], pointsArcVal.dradius, pointsArcVal.dStartX[i], pointsArcVal.dStartY[i]),
                    itemDirection = this._pathDirection(pointsItemVal.x1[i], pointsItemVal.y1[i], pointsItemVal.radius, pointsItemVal.longArc[i], pointsItemVal.x2[i],
                    pointsItemVal.y2[i], pointsItemVal.dEndX[i], pointsItemVal.dEndY[i], pointsItemVal.dradius, pointsItemVal.dStartX[i], pointsItemVal.dStartY[i]),
                    currentEle = this.model.items[i],
                    imgTextGroup = this._createGroupElement(),
                    itemEnable = this._enableItem(currentEle, imgTextGroup);
                this._menuItemsArcPaths.append(this._createPathElement(i, arcDirection, "" + this._prefix + "arcbgcolor"));
                this._menuItemsPaths.append(this._createPathElement(i, itemDirection, itemEnable.itemPathClass));
                this._radialArcGroup.append(this._createPathElement(i, direction, itemEnable.arcPathClass));
                imgTextGroup.attr({ "cursor": "context-menu", "index": i });
                imgTextGroup.append(this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i], pointsVal.midy[i] - imgDim / 2));
                imgTextGroup.append(this._createTextElement(pointsVal.midx[i] + 10, pointsVal.midy[i] + 25, itemEnable.textClass, itemEnable.dataText));
                if (currentEle.badge && (currentEle.badge.enabled == "true" || currentEle.badge.enabled == true))
                    imgTextGroup.append(this._createCircleElement(pointsVal.midx[i] + 25, pointsVal.midy[i] - 10, 10, "" + this._prefix + "badgecircle"))
                                .append(this._createTextElement(pointsVal.midx[i] + 25, pointsVal.midy[i] - 7, "" + this._prefix + "badgetext", currentEle.badge.value));
                this._menuItemsGroup.append(imgTextGroup);
                if (currentEle["items"]) {
                    var transform = pointsVal.deg[i] > 270 ? "translate(" + (pointsVal.dmidx[i] + 5) + "," + (pointsVal.dmidy[i] + 15) + ")" + "rotate(" + pointsVal.deg[i] + ")"
                        : "translate(" + (pointsVal.dmidx[i] + 10) + "," + (pointsVal.dmidy[i] + 10) + ")" + "rotate(" + pointsVal.deg[i] + ")";
                    this._childArcGroup.append(this._createPathElement(i, direction, "" + this._prefix + "childdefault"));
                    this._polygonGroup.append(this._createPolygonElement(i, transform));
                }
            }
            var circle = this._createCircleElement(this._startXY, this._startXY, this._startXY / 3.5, "" + this._prefix + "circlebgcolor");
            this._menuItemsArcGroup = this._radialArcGroup.clone();
            this._radialChildSVG.append(this._menuItemsPaths).append(this._menuItemsArcGroup).append(this._childArcGroup)
                                .append(this._polygonGroup).append(this._menuItemsArcPaths).append(this._menuItemsGroup).append(circle);
            this._radialContainer.append(this._radialSVGDiv.append(this._radialCircleSVGDiv.append(this._radialSVG.append(this._radialArcGroup)))
                                 .append(this._radialChildSVGDiv.append(this._radialChildSVG)));
        },
        _renderRadialChildMenu: function (index) {
            this._menuChildSVG = this._createSVGElement();
            this._menuChildSVGDiv = ej.buildTag("div#" + this._id + "_menuchildsvgdiv." + this._prefix + "abs");
            if (this.model.enableAnimation) {
                this._radialChildSVGDiv.removeClass("" + this._prefix + "radialshow").addClass("" + this._prefix + "scalehide");
                this._radialChildAnimate();
                this._menuChildSVGDiv.addClass("" + this._prefix + "scaleshow");
            }
            else
                this._radialChildSVGDiv.addClass("" + this._prefix + "displaynone");
            this._radial.removeClass(this.model.imageClass).addClass("" + this._prefix + "backarrow " + this.model.backImageClass + "");
            this._childItemsGroup = this._createGroupElement("childitemsgroup");
            this._childItemsGroup.attr("transform", "translate(-10,-10)");
            this._childItemArcGroup = this._createGroupElement();
            this._childItemsArcPaths = this._createGroupElement("childitemsarcpaths");
            this._childItemsPaths = this._createGroupElement();
            var child = this.model.items[index].items,
                childcount = child.length,
                pointsVal = this._calculateRadialArc(childcount, 1, 0.28, 0.8, null),
                pointsArcVal = this._calculateRadialArc(childcount, 0.79, null, 0.97, null),
                pointsItemVal = this._calculateRadialArc(childcount, 0.8, null, 0.3, null),
                imgDim = 20;
            for (var i = 0; i < childcount; i++) {
                var direction = this._pathDirection(pointsVal.x1[i], pointsVal.y1[i], pointsVal.radius, pointsVal.longArc[i], pointsVal.x2[i], pointsVal.y2[i],
                    pointsVal.dEndX[i], pointsVal.dEndY[i], pointsVal.dradius, pointsVal.dStartX[i], pointsVal.dStartY[i]),
                    arcDirection = this._pathDirection(pointsArcVal.x1[i], pointsArcVal.y1[i], pointsArcVal.radius, pointsArcVal.longArc[i], pointsArcVal.x2[i],
                    pointsArcVal.y2[i], pointsArcVal.dEndX[i], pointsArcVal.dEndY[i], pointsArcVal.dradius, pointsArcVal.dStartX[i], pointsArcVal.dStartY[i]),
                    itemDirection = this._pathDirection(pointsItemVal.x1[i], pointsItemVal.y1[i], pointsItemVal.radius, pointsItemVal.longArc[i], pointsItemVal.x2[i],
                    pointsItemVal.y2[i], pointsItemVal.dEndX[i], pointsItemVal.dEndY[i], pointsItemVal.dradius, pointsItemVal.dStartX[i], pointsItemVal.dStartY[i]),
                    imgTextGroup = this._createGroupElement(),
                    currentEle = this.model.items[index].items[i],
                    itemEnable = this._enableItem(currentEle, imgTextGroup);
                this._childItemArcGroup.append(this._createPathElement(i, direction, itemEnable.arcPathClass));
                this._childItemsArcPaths.append(this._createPathElement(i, arcDirection, "" + this._prefix + "arcbgcolor"));
                this._childItemsPaths.append(this._createPathElement(i, itemDirection, itemEnable.itemPathClass));
                imgTextGroup.attr({ "cursor": "context-menu", "index": i });
                imgTextGroup.append(this._createImageElement(imgDim, imgDim, currentEle, pointsVal.midx[i], pointsVal.midy[i] - imgDim / 2));
                imgTextGroup.append(this._createTextElement(pointsVal.midx[i] + 10, pointsVal.midy[i] + 25, itemEnable.textClass, itemEnable.dataText));
                this._childItemsGroup.append(imgTextGroup);
            }
            var circle = this._createCircleElement(this._startXY, this._startXY, this._startXY / 3.5, "" + this._prefix + "circlebgcolor");
            this._radialSVGDiv.append(this._menuChildSVGDiv.append(this._menuChildSVG.append(this._childItemsPaths)
                              .append(this._childItemArcGroup).append(this._childItemsArcPaths).append(this._childItemsGroup).append(circle)));
        },
        _pathDirection: function (x1, y1, radius, lArc, x2, y2, dEndX, dEndY, dRadius, dStartX, dStartY) {
            return "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + lArc + " " + "1" + " " + x2 + " " + y2 + " " + "L" +
                " " + dEndX + " " + dEndY + " " + "A" + " " + dRadius + " " + dRadius + " " + "1" + " " + lArc + " " + "0" + " " + dStartX + " " + dStartY + " " + "z";
        },
        _enableItem: function (currentObj, ele) {
            var itemPathClass = this._prefix + "itembgcolor", arcPathClass = this._prefix + "default", textClass = "" + this._prefix + "textcolor",
                dataText = this.model.renderMode ? currentObj[this.model.renderMode].text : currentObj.text;
            if (currentObj.enabled == "false" || currentObj.enabled == false) {
                ele.attr("class", this._prefix + "itemdisabled");
                return {
                    itemPathClass: this._prefix + "itembgcolor " + this._prefix + "pathdisabled", arcPathClass: this._prefix + "default " + this._prefix + "pathdisabled",
                    dataText: dataText, textClass: textClass
                };
            }
            else
                return { itemPathClass: itemPathClass, arcPathClass: arcPathClass, dataText: dataText, textClass: textClass };
        },
        _windowsMenuShow: function () {
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "displaynone " + this._prefix + "radialhide").addClass(this._prefix + "radialshow");
                this._radialChildSVGDiv.removeClass(this._prefix + "displaynone " + this._prefix + "radialhide").addClass(this._prefix + "radialshow");
            }
            else {
                this._radialCircleSVGDiv.removeClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.removeClass(this._prefix + "displaynone");
            }
            this._openCloseTrigger("open");
        },
        _windowsMenuHide: function (proxy) {
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "radialshow").addClass(this._prefix + "radialhide");
                this._radialChildSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "scaleshow").addClass(this._prefix + "radialhide");
                setTimeout(function () {
                    proxy._radialCircleSVGDiv.addClass(proxy._prefix + "displaynone");
                    proxy._radialChildSVGDiv.addClass(proxy._prefix + "displaynone");
                }, 160);
            }
            else {
                this._radialCircleSVGDiv.addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.addClass(this._prefix + "displaynone");
            }
            this._openCloseTrigger("close");
        },
        _windowsInnerMenuHide: function (proxy) {
            this._radial.removeClass(this._prefix + "backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
            if (this.model.enableAnimation) {
                this._menuChildSVGDiv.removeClass(this._prefix + "scaleshow").addClass(this._prefix + "scalehide");
                setTimeout(function () { proxy._menuChildSVGDiv.remove(); }, 150);
                setTimeout(function () { proxy._radialChildSVGDiv.removeClass(proxy._prefix + "scalehide " + proxy._prefix + "displaynone").addClass(proxy._prefix + "scaleshow"); }, 200);
            }
            else {
                this._menuChildSVGDiv.remove();
                this._radialChildSVGDiv.removeClass(this._prefix + "displaynone");
            }
        },
        _eventsTrigger: function (data, eventName) {
            var event = this.model[eventName];
            this.model[eventName] = this._childTarget && this.model.items[data.index].items[data.childIndex][eventName] ? this.model.items[data.index].items[data.childIndex][eventName]
                : (!this._childTarget && this.model.items[data.index][eventName] ? this.model.items[data.index][eventName] : this.model[eventName]);
            if (this.model[eventName])
                this._trigger(eventName, data);
            this.model[eventName] = event;
        },
        _openCloseTrigger: function (eventName) {
            if (this.model[eventName])
                this._trigger(eventName, this);
        },
        _menuEnabledDisabled: function (itemIndices, menuClass) {
            var proxy = this,
                itemIndex;
            if ($.isArray(itemIndices)) {
                $.each(itemIndices, function (i, itemIndex) {
                    if (!$.isArray(itemIndex))
                        proxy._menuItems(proxy, itemIndex, menuClass);
                    else {
                        $.each(itemIndex, function (i, index) {
                            if (!$.isArray(index)) {
                                if (!proxy.model.renderMode || (proxy.model.renderMode && (proxy.model.renderMode == "windows" || proxy.model.renderMode == "flat")))
                                    proxy._menuItems(proxy, index, menuClass);
                                itemIndex = index;
                            }
                            else {
                                $.each(index, function (i, childIndex) {
                                    proxy.model.items[itemIndex].items[childIndex].enabled = (menuClass == "disabled") ? false : true;
                                });
                            }
                        });
                    }
                });
            }
        },
        _ejMenuItem: function (proxy, index, menuClass) {
            var currentGroupEle = $(this._menuItemsGroup.find("g")[index]),
                currentItemPathEle = $(this._menuItemsPaths.find("path")[index]),
                currentArcPathEle = $(this._radialChildSVGDiv.find("#arcpathgroup path")[index]);
            if (menuClass == "disabled") {
                currentItemPathEle.attr("class", "" + this._prefix + "itembgcolor " + this._prefix + "pathdisabled");
                currentArcPathEle.attr("class", "" + this._prefix + "default " + this._prefix + "pathdisabled");
                currentGroupEle.attr("class", "" + this._prefix + "itemdisabled");
            }
            else {
                currentItemPathEle.attr("class", "" + this._prefix + "itembgcolor");
                currentArcPathEle.attr("class", "" + this._prefix + "default");
                currentGroupEle.removeAttr("class");
            }
        },
        _ejMenuHide: function (proxy) {
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "radialshow").addClass(this._prefix + "radialhide");
                this._radialChildSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "scaleshow").addClass(this._prefix + "radialhide");
                setTimeout(function () {
                    proxy._radialCircleSVGDiv.addClass(proxy._prefix + "displaynone").removeClass(proxy._prefix + "radialshow " + proxy._prefix + "radialhide " + proxy._prefix + "scaleshow " + proxy._prefix + "scalehide");
                    proxy._radialChildSVGDiv.addClass(proxy._prefix + "displaynone").removeClass(proxy._prefix + "radialshow " + proxy._prefix + "radialhide " + proxy._prefix + "scaleshow " + proxy._prefix + "scalehide");
                }, 160);
            }
            else {
                this._radialCircleSVGDiv.addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.addClass(this._prefix + "displaynone");
            }
            this._openCloseTrigger("close");
        },
        _ejInnerMenuHide: function (proxy) {
            this._radial.removeClass(this._prefix + "backarrow " + this.model.backImageClass + "").addClass(this.model.imageClass);
            if (this.model.enableAnimation) {
                this._radialCircleSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "radialhide " + this._prefix + "scaleshow " + this._prefix + "scalehide").addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.removeClass(this._prefix + "radialshow " + this._prefix + "radialhide " + this._prefix + "scaleshow " + this._prefix + "scalehide").addClass(this._prefix + "displaynone");
                this._menuChildSVGDiv.removeClass(this._prefix + "scaleshow").addClass(this._prefix + "radialhide");
                setTimeout(function () { proxy._menuChildSVGDiv.removeClass(proxy._prefix + "radialhide").remove(); }, 150);
            }
            else {
                this._radialCircleSVGDiv.addClass(this._prefix + "displaynone");
                this._radialChildSVGDiv.addClass(this._prefix + "displaynone");
                this._menuChildSVGDiv.remove();
            }
            this._openCloseTrigger("close");
        },
        _currentBadgeEle: function (index) {
            return this.model.renderMode == "android" ? $(this._imageBadgeGroup.find("g")[index]) : $(this._menuItemsGroup.find("g")[index]);
        },
        _badge: function (index) {
            return this._currentBadgeEle(index).find("." + this._prefix + "badgetext,." + this._prefix + "badgecircle");
        },
        show: function () {
            this.element.removeClass(this._prefix + "displaynone");
        },
        hide: function () {
            this.element.addClass(this._prefix + "displaynone");
        },
        showMenu: function () {
            this._showMenu();
        },
        hideMenu: function () {
            this._hideMenu();
        },
        enableItemByIndex: function (itemIndex) {
            this._menuItems(this, itemIndex, "enabled");
        },
        enableItemsByIndices: function (itemIndices) {
            this._menuEnabledDisabled(itemIndices, "enabled");
        },
        disableItemByIndex: function (itemIndex) {
            this._menuItems(this, itemIndex, "disabled");
        },
        disableItemsByIndices: function (itemIndices) {
            this._menuEnabledDisabled(itemIndices, "disabled");
        },
        updateBadgeValue: function (index, value) {
            this._currentBadgeEle(index).find("." + this._prefix + "badgetext").html(value);
        },
        showBadge: function (index) {
            this._badge(index).show();
        },
        hideBadge: function (index) {
            this._badge(index).hide();
        },
        _setModel: function (options) {
            this._refresh();
        },
        _refresh: function () {
            this._destroy();
            this._init();
        },
        _destroy: function () {
            this.element.removeAttr("class style").children().remove();
        }
    });

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Radial Menu elements
* @copyright Copyright Syncfusion Inc. 2001 - 2015. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {

    ej.widget("ejRadialMenu", "ej.RadialMenu", {
        _rootCSS: "e-radialmenu",
        element: null,
        model: null,
        _setFirst: true,
        defaults: {
            imageClass: "e-radialimage",
            backImageClass: "e-backimage",
            targetElementId: null,
            click: null,
            autoOpen: false,
            position: {
                x: null,
                y: null
            }
        },
        dataTypes: {
            autoOpen: "boolean"
        },
        _init: function () {
            this._id = this.element[0].id;
            if (!(ej.browserInfo().name == "msie" && ej.browserInfo().version < 9)) {
                this._initialization();
                this._prefix = "e-";
                this._renderRadial();
                this._wireEvents();
            }
        },
        _renderRadial: function () {
            this._radial = ej.buildTag("div.e-radial e-abs " + this.model.imageClass + "").css({ "left": this._startXY - 22 + "px", "top": this._startXY - 22 + "px" });
            this._radialContainer = ej.buildTag("div.e-rel").css({ "height": this._diameter, "width": this._diameter });
            this.element.append(this._radialContainer.append(this._radial)).addClass("e-radialmenu e-overlow e-user-select");
            this._radialInitialPosition();
            this._renderRadialMenu();
            var childArc = this._childArcGroup.children(),
                polygonGroup = this._polygonGroup.children();
            this._on(childArc, "mouseenter", this._arcOverHandler);
            this._on(childArc, "mouseleave", this._arcOverHandler);
            this._on(polygonGroup, "mouseenter", this._arcOverHandler);
            this._on(polygonGroup, "mouseleave", this._arcOverHandler);
            this._itemsEvents(this._menuItemsGroup.children(), this._menuItemsPaths.children(), this._menuItemsArcGroup.children());
        },
        _wireEvents: function () {
            var doc = $(document);
            this._radialClick = $.proxy(this._showMenu, this);
            this._targetDown = $.proxy(this._targetDownHandler, this);
            this._targetClick = $.proxy(this._targetClickHandler, this);
            this._itemClick = $.proxy(this._itemClickHandler, this);
            ej.listenEvents([this._radial, this._radialSVGDiv, doc, doc], [ej.endEvent(), ej.endEvent(), ej.endEvent(), ej.startEvent()],
                [this._radialClick, this._itemClick, this._targetClick, this._targetDown], false);
        },
        _itemsEvents: function (group, path, arcPath) {
            this._on(group, "mouseenter", this._itemOverHandler);
            this._on(group, "mouseleave", this._itemOverHandler);
            this._on(path, "mouseenter", this._itemOverHandler);
            this._on(path, "mouseleave", this._itemOverHandler);
            this._on(arcPath, "mouseenter", this._itemOverHandler);
            this._on(arcPath, "mouseleave", this._itemOverHandler);
        },
        _showMenu: function () {
            if (this._radial.hasClass("e-backarrow"))
                this._windowsInnerMenuHide(this);
            else {
                if (this._radialCircleSVGDiv.hasClass("e-displaynone"))
                    this._windowsMenuShow();
                else
                    this._windowsMenuHide(this);
            }
            this._childTarget = false;
        },
        _itemClickHandler: function (e) {
            var targetEle = $(e.target);
            if (targetEle.attr("class") == "e-active" || e.target.tagName == "polygon") {
                this._index = parseInt(targetEle.attr("index"))
                this._renderRadialChildMenu(this._index);
                this._childTarget = true;
                this._itemsEvents(this._childItemsGroup.children(), this._childItemsPaths.children(), this._childItemArcGroup.children());
            }
            this._eventAction(e);
            if (this._event)
                this._eventsTrigger(this._data, "click");
            this._event = false;
        },
        _eventAction: function (e) {
            var targetEle = $(e.target),
                index, text;
            if (targetEle.attr("class") == "e-itembgcolor" || targetEle.attr("class") == "e-default") {
                index = targetEle.index();
                text = !this._childTarget ? $(this._menuItemsGroup.children()[targetEle.index()]).text() : $(this._childItemsGroup.children()[targetEle.index()]).text();
                this._event = true;
                this._data = !this._childTarget ? { index: index, text: text } : { index: this._index, childIndex: index, text: text };
            }
            else if (e.target.tagName == "text" || e.target.tagName == "image") {
                index = targetEle.parent().index();
                text = targetEle.parent().text();
                this._event = true;
                this._data = !this._childTarget ? { index: index, text: text } : { index: this._index, childIndex: index, text: text };
            }
        },
        _radialInitialPosition: function () {
            this._targetElement = $("#" + this.model.targetElementId + "");
            if (this.model.autoOpen) {
                if (this.model.position.x != null && this.model.position.y != null)
                    this.element.css({ "position": "absolute", "top": this.model.position.y + "px", "left": this.model.position.x + "px" });
                else if (this.model.targetElementId)
                    this.element.css({ "position": "absolute", "top": this._targetElement.height() / 2 - this.model.radius + "px", "left": this._targetElement.width() / 2 - this.model.radius + "px" });
                this.show();
            }
            else
                this.hide();
        },
        _radialChildAnimate: function () {
            var proxy = this;
            setTimeout(function () {
                proxy._radialChildSVGDiv.removeClass("e-scalehide e-scaleshow").addClass("e-displaynone");
            }, 150);
        },
        _arcOverHandler: function (e) {
            var targetEle = $(e.target),
                index = parseInt(targetEle.attr("index")),
                targetGroupEle = $(this._childArcGroup.find('path[index=' + index + ']'));
            if (e.type == "mouseenter") {
                if (targetEle.attr("class") == "e-childdefault")
                    targetEle.attr("class", "e-active");
                else
                    targetGroupEle.attr("class", "e-active");
            }
            else {
                if (targetEle.attr("class") == "e-active")
                    targetEle.attr("class", "e-childdefault");
                else
                    targetGroupEle.attr("class", "e-childdefault");
            }
        },
        _itemOverHandler: function (e) {
            var index = $(e.currentTarget).index();
            var targetEle = !this._childTarget ? $(this._menuItemsArcPaths.children()[index]) : $(this._childItemsArcPaths.children()[index]);
            if (e.type == "mouseenter")
                targetEle.attr("class", "e-childdefault");
            else
                targetEle.attr("class", "e-arcbgcolor");
        },
        _targetDownHandler: function (e) {
            var targetEle = $(e.target);
            if (!targetEle.hasClass("e-rel") && $(targetEle.closest(".e-radialmenu")).length) {
                e.preventDefault();
            }
        },
        _targetClickHandler: function (e) {
            var targetEle = $(e.target), x = e.clientX, y = e.clientY, width, height,
                menuTarget = $(targetEle.closest(".e-radialmenu")).length,
                menuState = this._radialCircleSVGDiv.hasClass("e-displaynone");
            if (this.model.targetElementId) {
                width = this._targetElement.width();
                height = this._targetElement.height();
            }
            else {
                width = window.innerWidth;
                height = window.innerHeight;
            }
            if (menuTarget == 0 && $(targetEle.closest(this._targetElement)).length == 0 && this.model.targetElementId) {
                menuState ? this.hide() : this.hideMenu();
            }
            else if ($(targetEle.closest("div.e-radial")).length == 0 && menuState) {
                var left = x > width - this.model.radius ? width - this._diameter : (x > this.model.radius ? x - this.model.radius : 0),
                    top = y > height - this.model.radius ? height - this._diameter : (y > this.model.radius ? y - this.model.radius : 0);
                this.element.css({ "position": "absolute", "top": top + "px", "left": left + "px" });
            }
            else if (menuTarget == 0)
                this.hideMenu();
        },
        _menuItems: function (proxy, index, menuClass) {
            this._ejMenuItem(proxy, index, menuClass);
        },
        _hideMenu: function () {
            if (this._radial.hasClass("e-backarrow"))
                this._ejInnerMenuHide(this);
            else
                this._ejMenuHide(this);
        },
        menuHide: function () {
            this._hideMenu();
        },
        enableItem: function (item) {
            this._ejMenuEnabledDisabled(item, "enabled");
        },
        disableItem: function (item) {
            this._ejMenuEnabledDisabled(item, "disabled");
        },
        enableItems: function (items) {
            this._ejMenuEnabledDisabled(items, "enabled");
        },
        disableItems: function (items) {
            this._ejMenuEnabledDisabled(items, "disabled");
        },
        _ejMenuEnabledDisabled: function (items, menuClass) {
            var proxy = this, ItemObj = $.isArray(items) ? items : $.makeArray(items);
            $.each(ItemObj, function (i, curItem) {
                $.each(proxy.model.items, function (j, baseItem) {
                    if (baseItem.text.toLowerCase() == curItem.toLowerCase())
                        proxy._menuItems(this, j, menuClass);
                    if (baseItem.items) {
                        $.each(baseItem.items, function (k, childItem) {
                            if (childItem.text.toLowerCase() == curItem.toLowerCase())
                                childItem.enabled = (menuClass == "disabled") ? false : true;
                        });
                    }
                });
            });
        }
    });
    $.extend(true, ej.RadialMenu.prototype, ej.RadialMenuBase.prototype);
})(jQuery, Syncfusion);;

ej.Ej3DRender = function () {

   
};

(function ($) {
    
    ej.Ej3DRender.Polygons = [];

    
    ej.Ej3DRender.prototype = {
        matrix3D: function () {
            
        },

        vector3D: function () {
             
        },
       
        BSPTreeBuilder: function () {

        },

        Graphics3D: function () {

        },

        polygon3D: function () {

        }
    }

    var Ej3DRender = new ej.Ej3DRender();
    
    Ej3DRender.vector3D.prototype = {
        //Members
        x: "",
        y: "",
        z: "",
        Epsilon:0.00001,
        //all coordinates indicate 0
        //  empty: new Vector3D(0, 0, 0),

        //isValid: !isNaN(this.x) && !isNaN(this.y) && !isNaN(this.z),
        isValid: function (point) {
            return !isNaN(point.x) && !isNaN(point.y) && !isNaN(point.z)
        },
        //Constructor
        vector3D: function (points, vz) {
            this.x = points.x;
            this.y = points.y;
            this.z = vz;
        },

        vector3D: function (vx, vy, vz) {
            this.x = vx;
            this.y = vy;
            this.z = vz;
            return {x:vx, y:vy, z:vz}
        },

        //operations
        vector3DMinus: function (v1, v2) {
            return this.vector3D(v1.x - v2.x, v1.y - v2.y, v1.z - v2.z);
        },

        vector3DPlus: function (v1, v2) {
            return this.vector3D(v1.x + v2.x, v1.y + v2.y, v1.z + v2.z);
        },

        vector3DMultiply: function (v1, v2) {
            var _x = v1.y * v2.z - v2.y * v1.z;
            var _y = v1.z * v2.x - v2.z * v1.x;
            var _z = v1.x * v2.y - v2.x * v1.y;

            return this.vector3D(_x, _y, _z);
        },

        vector3dAND: function (v1, v2) {
            return (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
        },

        vector3DStarMultiply: function (v1, val) {
            var _x = v1.x * val;
            var _y = v1.y * val;
            var _z = v1.z * val;

            return this.vector3D(_x, _y, _z);
        },

        //Methods
        getLength: function (vector) {
            var sqt = this.vector3dAND(vector, vector);
            return Math.sqrt(sqt);
        },

       

        normalize: function () {
            var l = this.getLength();
            x /= l;
            y /= l;
            z /= l;
        },
        getNormal:function(v1, v2, v3)
        {
            var v4 = this.vector3DMinus(v1, v2);
            var v5 = this.vector3DMinus(v3, v2);
            var n = this.vector3DMultiply(v4, v5);
            var l = this.getLength(n);

           if (l < this.Epsilon)
           {
              l = 0;
            }
           return { x: n.x/ l, y: n.y / l, z: n.z / l }
        }
    }

   Ej3DRender.matrix3D.prototype = {
       //Memebr
        MATRIX_SIZE: 4,
       //Constructor
        matrix3D: function (size) {
            var mData = [];
           for (var i = 0; i < size; i++) {
               mData[i] = this.createArray(size);
           }
           return mData;
        },

       //method
        isAffine: function (mData) {

            return (mData[0][3] == 0) && (mData[1][3] == 0)
                    && (mData[2][3] == 0) && (mData[3][3] == 1);
        },
       createArray: function (initialSize) {
           var a = [];
           for (index = 0; index < initialSize; ++index) {
               a[index] = 0;
           }
           return a;
       },

       getIdentity: function () {

           var mData=this.matrix3D(this.MATRIX_SIZE);

           for (var i = 0; i < this.MATRIX_SIZE; i++) {
               mData[i][i] = 1.0;
           }
          
           return mData;
       },

      getInvertal:function(matrix)
      {
         var m = this.getIdentity();

         for (var i = 0; i < this.MATRIX_SIZE; i++)
         {
           for (var j = 0; j < this.MATRIX_SIZE; j++)
           {
               m[i][j] = this.getMinor(matrix, i, j);
            }
         }

         m = this.transposed(m);
         m = this.getMatrixMultiple((1 / this.getDeterminant(matrix)) , m);

      return m;
      },
      getMatrixMultiple: function (factor, matrix) {
          for (var i = 0; i < matrix.length; i++) {
              for (var j = 0; j < matrix[i].length; j++) {
                  matrix[i][j] = matrix[i][j] * factor;
              }
          }
          return matrix;
      },
      getMatrixVectorMutiple:function(m1, point)
      {
            var x = m1[0][0] * point.x + m1[1][0] * point.y + m1[2][0] * point.z + m1[3][0];
            var y = m1[0][1] * point.x + m1[1][1] * point.y + m1[2][1] * point.z + m1[3][1];
            var z = m1[0][2] * point.x + m1[1][2] * point.y + m1[2][2] * point.z + m1[3][2];

      if (!this.isAffine(m1))
      {
         var c = 1 / (m1[0][3] * point.x + m1[1][3] * point.y + m1[2][3] * point.z + m1[3][3]);
         x *= c;
         y *= c;
         z *= c;
      }

       return {x:x, y:y, z:z};
      },
      getMatrixVectorAnd:function(m1, v1)
      {
            var x = m1[0][0] * v1.x + m1[1][0] * v1.y + m1[2][0] * v1.z;
            var y = m1[0][1] * v1.x + m1[1][1] * v1.y + m1[2][1] * v1.z;
            var z = m1[0][2] * v1.x + m1[1][2] * v1.y + m1[2][2] * v1.z;

            return Ej3DRender.vector3D.prototype.vector3D(x, y, z);
         },
      getMatrixAdd:function(m1, m2) 
      {
          var m = this.matrix3D(4);

       for (var i = 0; i < 4; i++)
        {
          for (var j = 0; j < 4; j++)
          {
             m[i][j] = m1[i][j] + m2[i][j];
          }
        }

       return m;
     },
      getMatrixMultiplication:function(m1, m2)
      {
          var res = this.getIdentity();

         for (var i = 0; i < this.MATRIX_SIZE; i++)
         {
             for (var j = 0; j < this.MATRIX_SIZE; j++)
             {
             var v = 0;

             for (var k = 0; k < this.MATRIX_SIZE; k++)
             {
                 v += m1[k][j] * m2[i][k];
                 var t = v;
             }

             res[i][j] = v;
          }
       }

       return res;
      },
      getMatrixEqual:function(m1,m2)
       {
            var res = true;

           for (var i = 0; i < m1.length; i++)
          {
              for (var j = 0; j < m1.length; j++)
              {
                 if (m1[i][j] != m2[i][j])
                 {
                     res = false;
                 }
             }
         }

      return res;
      },
      getMatrixNotEqual:function(m1, m2)
      {
            var res = true; 

         for (var i = 0; i < m1.length; i++)
        {
             for (var j = 0; j < m1.length; j++)
          {
             if (m1[i][j] != m2[i][j])
             {
                res = false;
             }
          }
       }

        return !res;
    },


      getMinor:function(dd, columnIndex, rowIndex)
      {
          return (((columnIndex + rowIndex) % 2 == 0) ? 1 : -1) * this.getDeterminant(this.getMMtr(dd, columnIndex, rowIndex));

      },

     getMMtr:function(dd, columnIndex, rowIndex)
     {
        var count = dd.length - 1;
        var d = this.createArray(count);

        for (var i = 0; i < count; i++)
        {
            var m = (i >= columnIndex) ? i + 1 : i;
            d[i] = this.createArray(count);

            for (var j = 0; j < count; j++)
            {
                var n = (j >= rowIndex) ? j + 1 : j;

                d[i][j] = dd[m][n];
            }
        }
         return d;
     },
    getDeterminant:function(dd)
    {
        var count = dd.length;
        var res = 0;

        if (count < 2)
        {
            res = dd[0][0];
        }
        else
        {
            var k = 1;

            for (var i = 0; i < count; i++)
            {
                var dm = this.getMMtr(dd, i, 0);

                res += k * dd[i][0] * this.getDeterminant(dm);
                k = (k > 0) ? -1 : 1;
            }
        }

        return res;
    },


       transform: function (x, y, z) {
           var res = this.getIdentity();

           res[3][0] = x;
           res[3][1] = y;
           res[3][2] = z;

           return res;
       },


       turn: function (angle) {
           var res = this.getIdentity();

           res[0][0] = Math.cos(angle);
           res[2][0] = -Math.sin(angle);
           res[0][2] = Math.sin(angle);
           res[2][2] = Math.cos(angle);

           return res;
       },

       tilt: function (angle) {
           var res = this.getIdentity();
          
           res[1][1] = Math.cos(angle);
           res[2][1] = Math.sin(angle);
           res[1][2] = -(Math.sin(angle));
           res[2][2] = Math.cos(angle);
          
           return res;          
       },
       transposed:function(matrix3D)
       {
           var m = this.getIdentity();

           for (var i = 0; i < this.MATRIX_SIZE; i++)
            {
                for (var j = 0; j < this.MATRIX_SIZE; j++)
                {
                    m[i][j] = matrix3D[j][i];
                }
           }
          return m;
       },
      shear:function(xy, xz, yx, yz, zx, zy)
        {
          var res = this.getIdentity();

           res[1, 0] = xy;
           res[2, 0] = xz;
           res[0, 1] = yx;
           res[2, 1] = yz;
           res[0, 2] = zx;
           res[1, 2] = zy;

           return res;
        }


   }
   Ej3DRender.Graphics3D.prototype = {
      addVisual:function(polygon)
         {
          if ((polygon == null) || (polygonobj.test(polygon)))
            {
                return -1;
            }
         polygon.Graphics3D = this;
         return bsptreeobj.add(polygon);
      },

       getVisualCount:function()
           {
           return ej.Ej3DRender.Polygons.length();
       },
       remove:function(polygon)
        {
           ej.Ej3DRender.Polygons.Remove(polygon);
       },
 
     clearVisual:function()
       {
         ej.Ej3DRender.Polygons=null;
       },

      getVisual:function()
          {
                return ej.Ej3DRender.Polygons;
         },
      prepareView:function(perspectiveAngle, depth, rotation, tilt, size)
      {
          if (arguments.length==0) {
              bsptreeobj.build();
          }
          else {
              if (ej.Ej3DRender.transform == null)
                  ej.Ej3DRender.transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
              else
                  ej.Ej3DRender.transform.mViewport = size;
              ej.Ej3DRender.transform.Rotation = rotation;
              ej.Ej3DRender.transform.Tilt = tilt;
              ej.Ej3DRender.transform.Depth = depth;
              ej.Ej3DRender.transform.PerspectiveAngle = perspectiveAngle;
              ej.EjSvgRender.chartTransform3D.transform(ej.Ej3DRender.transform);
              ej.Ej3DRender.tree = bsptreeobj.build();
             
          } 
      },
      view: function (panel, sender, rotation, tilt, size, perspectiveAngle, depth)
      {
          var MaxValue = 32767;
          if (arguments.length == 2) {
              if (panel == null) return;
              var eye = vector.vector3D(0, 0, MaxValue);
              this.drawBspNode3D(ej.Ej3DRender.tree, eye, panel, sender);
          }
          else {
              if (panel == null) return;
            
              if (ej.Ej3DRender.transform == null)
                  ej.Ej3DRender.transform = ej.EjSvgRender.chartTransform3D.transform3D(size);
              else
                  ej.Ej3DRender.transform.mViewport = size;
              ej.Ej3DRender.transform.Rotation = rotation;
              ej.Ej3DRender.transform.Tilt = tilt;
              ej.Ej3DRender.transform.Depth = depth;
              ej.Ej3DRender.transform.PerspectiveAngle = perspectiveAngle;
              ej.EjSvgRender.chartTransform3D.transform(ej.Ej3DRender.transform);
              var eye = vector.vector3D(0, 0, MaxValue);
              this.drawBspNode3D(ej.Ej3DRender.tree, eye, panel, sender);
          }
      },

      draw3DElement: function (tr, sender) {
          if (tr.Plane.element){
              if (tr.Plane.element.tag == "text" || tr.Plane.element.tag == "dataLabel")
                  polygonobj.drawText(tr.Plane, sender);
              else if (tr.Plane.element.tag == "polyline")
                  polygonobj.drawPolyLine(tr.Plane, sender);
              else
                  polygonobj.drawLine(tr.Plane, sender);
            }
              
          else
            polygonobj.draw(tr.Plane, sender);          
      },

    drawBspNode3D:function(tr, eye, panel, sender)
    {
        if (tr == null || ej.Ej3DRender.transform == null) return;
     while (true)
     {
         var r = vector.vector3dAND(polygonobj.getNormal(ej.EjSvgRender.chartTransform3D.result(ej.Ej3DRender.transform), tr.Plane.VectorPoints), eye);
        if (r > tr.Plane.D)
        {
            if (tr.Front != null)
            {
                this.drawBspNode3D(tr.Front, eye, panel, sender);
            }

            this.draw3DElement(tr, sender);

            if (tr.Back != null)
            {
                tr = tr.Back;
                continue;
            }
        }
        else
        {
            if (tr.Back != null)
            {
                this.drawBspNode3D(tr.Back, eye, panel, sender);
            }

            this.draw3DElement(tr, sender);

            if (tr.Front != null)
            {
                tr = tr.Front;
                continue;
            }
        }
        break;
    }

    }

 }


   Ej3DRender.BSPTreeBuilder.prototype = {
       //Members
       EPSILON: 0.0005,
       Polygon:[],
       
       //Methods
       add:function(poly)
       {
           ej.Ej3DRender.Polygons.push(poly);
           return ej.Ej3DRender.Polygons.length - 1;
       },

       remove:function(index)
       {
           ej.Ej3DRender.Polygons.splice(index,1);
       },

       clear:function()
       {
           ej.Ej3DRender.Polygons=[];
       },

       count:function()
       {
           return ej.Ej3DRender.Polygons.length;
       },

       getNext:function(i, count)
        {
            if (i >= count)
            {
             return i - count;
            }
            if (i < 0)
              {
                  return i + count;
              }

             return i;
       },
       getNodeCount:function(el)
        {
            return (el == null) ? 0 : 1 + this.getNodeCount(el.Back) + this.getNodeCount(el.Front);
        },
       vector3DIndexClassification:function(point, ind, res){
           return{
                 index:ind,
                 result: res,
                 vector:point,

                isCuttingBackPoint:false,
                cuttingBackPairIndex:null,
                alreadyCuttedBack:false,

                isCuttingFrontPoint:false,
                cuttingFrontPairIndex:null,
                alreadyCuttedFront:false
         }
       },
       classifyPoint: function(pt, pln)
         {
            var res = "OnPlane";
            var sv = -pln.D - Ej3DRender.vector3D.prototype.vector3dAND(pt , pln.normal);

           if (sv > this.EPSILON)
            {
              res ="OnBack";
            }
          else if (sv < -this.EPSILON)
            {
              res = "OnFront";
            }

            return res;
        },
      
     classifyPolygon:function(pln, plg)
        {
            var res = "Unknown";
            var points = plg.Points;

           if (points == null)
               return res;
               var onBack = 0;
               var onFront = 0;
               var onPlane = 0;
               var normal = pln.normal;// root node normailized value perpendicular direction
               var d = pln.D; // constant of the plan or depth

          for (var i = 0, len = points.length; i < len; i++)
           {
              var r = -d - Ej3DRender.vector3D.prototype.vector3dAND(points[i], normal); // Comparision of Plane point depth with the other nodes
 
              if (r > this.EPSILON)
                {
                  onBack++;
                }
              else if (r < -this.EPSILON)
                {
                   onFront++;
                }
             else
               {
                  onPlane++;
               }

             if ((onBack > 0) && (onFront > 0))
               {
                 break;
               }
          }
         if (onPlane == points.length)
            {
               res = "OnPlane";
            }
          else if (onFront + onPlane == points.length)
           {
              res = "ToRight";
           }
          else if (onBack + onPlane == points.length)
           {
             res = "ToLeft";
           }
          else
          {
                res = "Unknown";
          }
      return res;
     },
   splitPolygon:function(poly, part)
     {
            var backP = [];
            var frontP = [];

    // this code looks for points which lie on the part plane and divide polygon into two parts
    if (poly.Points != null)
    {
        var polyPoints = [];
        var backPartPoints = [];
        var frontPartPoints = [];

        var outpts;
        var inpts;

        var count = poly.Points.length;
        for (var i = 0; i < count;i++)
        {
            var ptB = poly.Points[i];
            var ptC = poly.Points[this.getNext(i + 1, count)];
            var sideB = this.classifyPoint(ptB, part);
            var sideC = this.classifyPoint(ptC, part);

            var vwiwcB = this.vector3DIndexClassification(ptB, polyPoints.length, sideB);
            polyPoints.push(vwiwcB);

            if ((sideB != sideC) && (sideB != "OnPlane") &&
                (sideC !="OnPlane"))
            {
                var v = vector.vector3DMinus(ptB, ptC);
                var dir = vector.vector3DMinus(vector.vector3DStarMultiply(part.normal, (-part.D)), ptC);

                var sv = vector.vector3dAND(dir, part.normal);
                var sect = sv / vector.vector3dAND(part.normal, v);
                var ptP = vector.vector3DPlus(ptC, vector.vector3DStarMultiply(v, sect));
                var vwiwc = this.vector3DIndexClassification(ptP, polyPoints.length, "OnPlane");

                polyPoints.push(vwiwc);
                backPartPoints.push(vwiwc);
                frontPartPoints.push(vwiwc);
            }
            else
                if (sideB == "OnPlane")
                {
                    var ptA = poly.Points[this.getNext(i - 1, count)];
                    var sideA = this.classifyPoint(ptA, part);
                    if ((sideA == sideC)) continue;
                    if ((sideA != "OnPlane") && (sideC != "OnPlane"))
                    {
                        backPartPoints.push(vwiwcB);
                        frontPartPoints.push(vwiwcB);
                    }
                    else
                        if (sideA =="OnPlane")
                        {
                            switch (sideC)
                            {
                                case "OnBack":
                                    backPartPoints.push(vwiwcB);
                                    break;
                                case "OnFront":
                                    frontPartPoints.push(vwiwcB);
                                    break;
                            }
                        }
                        else
                            if (sideC =="OnPlane")
                            {
                                switch (sideA)
                                {
                                    case "OnBack":
                                        backPartPoints.push(vwiwcB);
                                        break;
                                    case "OnFront":
                                        frontPartPoints.push(vwiwcB);
                                        break;
                                }
                            }
                }
        }

        if ((frontPartPoints.length != 0) || (backPartPoints.length != 0))
        {
            for (var i = 0; i < backPartPoints.length - 1; i += 2)
            {
                var vwiwc1 = backPartPoints[i];
                var vwiwc2 = backPartPoints[i + 1];
                vwiwc1.CuttingBackPoint = true;
                vwiwc2.CuttingBackPoint = true;
                vwiwc1.CuttingBackPairIndex = vwiwc2.index;
                vwiwc2.CuttingBackPairIndex = vwiwc1.index;
            }
            for (var i = 0; i < frontPartPoints.length - 1; i += 2)
            {
                var vwiwc1 = frontPartPoints[i];
                var vwiwc2 = frontPartPoints[i + 1];
                vwiwc1.CuttingFrontPoint = true;
                vwiwc2.CuttingFrontPoint = true;
                vwiwc1.CuttingFrontPairIndex = vwiwc2.index;
                vwiwc2.CuttingFrontPairIndex = vwiwc1.index;
            }


            for (var i = 0; i < backPartPoints.length - 1; i++)
            {
                var vwiwc1 = backPartPoints[i];
                if (vwiwc1.alreadyCuttedBack) continue;
                var outpts=this.cutOutBackPolygon(polyPoints, vwiwc1);

                if (outpts.length > 2) {
                    var polygon1 = polygonobj.polygon3D(outpts, poly);
                    backP.push($.extend({}, polygon1));
                }
            }

            for (var i = 0; i < frontPartPoints.length - 1; i++)
            {
                var vwiwc2 = frontPartPoints[i];
                if (vwiwc2.alreadyCuttedFront) continue;
                inpts=this.cutOutFrontPolygon(polyPoints, vwiwc2);
                if (inpts.length > 2) {
                    var polygon2 = polygonobj.polygon3D(inpts, poly);
                    frontP.push($.extend({}, polygon2));
                }
            }
        } 
    }
     else
     {
        backP.push(poly);
        frontP.push(poly); 
     }

    return{BackP:backP, FrontP:frontP}
   },
  cutOutFrontPolygon:function(polyPoints, vwiwc)
   {
      points = [];

    var curVW = vwiwc;

    while (true)
    {
        curVW.alreadyCuttedFront = true;
        points.push(curVW.vector);

        var curVWPair = polyPoints[curVW.CuttingFrontPairIndex];

        if (curVW.CuttingFrontPoint)
        {
            if (!curVWPair.alreadyCuttedFront)
            {
                curVW = curVWPair;
            }
            else
            {
                var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
                var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

                if ((curVWPrev.result =="OnFront") && !curVWPrev.alreadyCuttedFront)
                {
                    curVW = curVWPrev;
                }
                else
                    if ((curVWNext.result == "OnFront") && !curVWNext.alreadyCuttedFront)
                    {
                        curVW = curVWNext;
                    }
                    else
                    {
                        return points;
                    }
            }
        }
        else
        {
            var curPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
            var curNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

            if ((curPrev.result != "OnBack") && !curPrev.alreadyCuttedFront)
            {
                curVW = curPrev;
            }
            else
                if ((curNext.result != "OnBack") && !curNext.alreadyCuttedFront)
                {
                    curVW = curNext;
                }
                else
                {
                    return points;
                }
        }
    }
    return points;
 },

   cutOutBackPolygon:function(polyPoints, vwiwc)
   {
       var points = [];
       var curVW = vwiwc;

    while (true)
    {
        curVW.alreadyCuttedBack = true;
        points.push(curVW.vector);

        var curVWPair = polyPoints[curVW.CuttingBackPairIndex];

        if (curVW.CuttingBackPoint)
        {
            if (!curVWPair.alreadyCuttedBack)
            {
                curVW = curVWPair;
            }
            else
            {
                var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
                var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

                if ((curVWPrev.result == "OnBack") && !curVWPrev.alreadyCuttedBack)
                {
                    curVW = curVWPrev;
                }
                else
                    if ((curVWNext.result == "OnBack") && !curVWNext.alreadyCuttedBack)
                    {
                        curVW = curVWNext;
                    }
                    else
                    {
                        return points;
                    }
            }
        }
        else
        {
            var curVWPrev = polyPoints[this.getNext(curVW.index - 1, polyPoints.length)];
            var curVWNext = polyPoints[this.getNext(curVW.index + 1, polyPoints.length)];

            if ((curVWPrev.result != "OnFront") && !curVWPrev.alreadyCuttedBack)
            {
                curVW = curVWPrev;
            }
            else
                if ((curVWNext.result != "OnFront") && !curVWNext.alreadyCuttedBack)
                {
                    curVW = curVWNext;
                }
                else
                {
                    return points;
                }
        }
    }
     return points
  },
     build:function()
       {
           if(!arguments[0]){
             return (this.build(ej.Ej3DRender.Polygons));               
           }
           else{
               var arlist=arguments[0];
               if (arlist.length < 1) return null;
               var bspNode = {Back:null, Front:null, Plane:null}
               var plane = arlist[0];
               bspNode.Plane = plane;
               var arleft = [];
               var arright = [];

               for (var i = 1, len = arlist.length; i < len; i++)
               {
                   var pln = arlist[i];

                   if (pln == plane) continue;
                   var r = this.classifyPolygon(plane, pln);

                   switch (r)
                   {
                       case "OnPlane":
                       case "ToRight":
                           arright.push(pln);
                           break;

                       case "ToLeft":
                           arleft.push(pln);
                           break;

                       case "Unknown":
                           //if (pln is Line3D || pln is UIElement3D)
                           if(pln.element && (pln.element.tag == "line" || pln.element.tag == "text"))
                           {
                               arleft.push(pln);
                           }
                         else
                         {
                            
                               var result = this.splitPolygon(pln, plane);                              
                               for (var k = 0; k < result.BackP.length; k++) {
                                   result.BackP[k].Name = result.BackP[k].Name + "back";
                                   arleft.push(result.BackP[k]);
                               }
                               for (var j = 0; j < result.FrontP.length; j++) {
                                   result.FrontP[j].Name = result.FrontP[j].Name + "front";
                                   arright.push(result.FrontP[j]);
                               }
                        }
                   break;
               }
           }

           if (arleft.length > 0)
           {
               bspNode.Back = this.build(arleft);
           }

           if (arright.length > 0)
           {
               bspNode.Front = this.build(arright);
           }

           return bspNode;
         }
    }

   }

   Ej3DRender.polygon3D.prototype = {
       // Member    
       epsilon: 0.00001,
       normal: { x: 0, y: 0, z: 0 },
       a: (this.normal) ? this.normal.x : 0,
       b: (this.normal) ? this.normal.y : 0,
       c: (this.normal) ? this.normal.z : 0,
       vector: new (new ej.Ej3DRender()).vector3D(),

       // Constructor
       polygon3D: function (points, tag, index, stroke, strokeThickness, opacity, fill, name) {

           if (arguments.length == 3) {
               this.calcNormal(arguments[0], arguments[1], arguments[2])
           }
           else if (arguments.length == 2) {
               points = arguments[0];
               this.calcNormal(points[0], points[1], points[2]);
               this.vectorPoints = points;
               this.calcNormal(this.vectorPoints);
               var polygon = arguments[1];
               polygon.Normal = this.normal;
               polygon.normal = this.normal;
               polygon.Points = points;
               polygon.VectorPoints = this.vectorPoints;
               polygon.IsSplitted = true;
               polygon.d = this.d;
               polygon.D = this.d;
               return polygon;
                
           }
           else {
               this.calcNormal(points[0], points[1], points[2]);
               this.vectorPoints = points;
               this.calcNormal(this.vectorPoints);
               var element = { Tag: 'path', Parent: arguments[arguments.length - 1] }
               var polygon= {
                   Normal: this.normal,
                   normal: this.normal,
                   Points: points,
                   VectorPoints: this.vectorPoints,
                   Index: index,
                   Tag: tag,
                   Name: (name) ? name : null,
                   StrokeThickness: strokeThickness,
                   Opacity:opacity,
                   Fill: fill,
                   d: this.d,
                   D: this.d
               }
               if (arguments.length != 1) 
                   polygon.Element = element;

               return polygon;
               
           }
       },

       //Methods
       createLine: function (line, x1, y1, x2, y2, depth) {
           var strokeThickness = line.width;
           var vectorColl = [];
           vectorColl[0] = vector.vector3D(x1, y1, depth);
           vectorColl[1] = vector.vector3D(x1 + strokeThickness, y2 + strokeThickness, depth);
           vectorColl[2] = vector.vector3D(x2, y2, depth);
           return this.line3D(line, vectorColl);
       },
       createPolyline: function (points, element) {
           if (points.length == 2) {
               var prePoint = points[1];
               points.push(vector.vector3D(prePoint.x, prePoint.y, prePoint.z));
           }
            return this.polyLine3D(element, points);
       },
       polyLine3D: function (element, points) {
           var plane = this.polygon3D(points);
           plane.element = element;
           return plane;
       },

       line3D:function(element, points)
       {
           var plane = this.polygon3D(points);
           plane.element = element;
           return plane;
       },
      text3D:function(element,points)
      {
          var plane = this.polygon3D(points);
          plane.element = element;
          return plane;
      },

       createBox: function (v1, v2, tag, index, graphics3D, stroke, fill, strokeThickness, opacity, inverse, name, parent) {
           var res = [];

           var p1 = new Array(vector.vector3D(v1.x, v1.y, v1.z),
                vector.vector3D(v2.x, v1.y, v1.z),
                vector.vector3D(v2.x, v2.y, v1.z),
                vector.vector3D(v1.x, v2.y, v1.z))

           var p2 = new Array(
               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v1.x, v2.y, v2.z)
            )

           var p3 = new Array(

               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v1.y, v1.z),
               vector.vector3D(v1.x, v1.y, v1.z)
          )

           var p4 = new Array(
               vector.vector3D(v1.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v1.z),
               vector.vector3D(v1.x, v2.y, v1.z)
           )

           var p5 = new Array(
               vector.vector3D(v1.x, v1.y, v1.z),
               vector.vector3D(v1.x, v1.y, v2.z),
               vector.vector3D(v1.x, v2.y, v2.z),
               vector.vector3D(v1.x, v2.y, v1.z)
           )

           var p6 = new Array(
               vector.vector3D(v2.x, v1.y, v1.z),
               vector.vector3D(v2.x, v1.y, v2.z),
               vector.vector3D(v2.x, v2.y, v2.z),
               vector.vector3D(v2.x, v2.y, v1.z)
           )
           if (arguments[10]) {

               res[0] = this.polygon3D(p1, tag, index, stroke, strokeThickness, opacity, fill, "_0_"+ name, parent);
               res[1] = this.polygon3D(p2, tag, index, stroke, strokeThickness, opacity, fill, "_1_" + name, parent);
               res[2] = this.polygon3D(p3, tag, index, stroke, strokeThickness, opacity, fill, "_2_" + name, parent);
               res[3] = this.polygon3D(p4, tag, index, stroke, strokeThickness, opacity, fill, "_3_" + name, parent);
               res[4] = this.polygon3D(p5, tag, index, stroke, strokeThickness, opacity, fill, "_4_" + name, parent);
               res[5] = this.polygon3D(p6, tag, index, stroke, strokeThickness, opacity, fill, "_5_" + name, parent);
           }
           else {
               var parent= arguments[arguments.length-1];
               res[0] = this.polygon3D(p1, tag, index, stroke, strokeThickness, opacity, fill, "_0_" + index, parent);
               res[1] = this.polygon3D(p2, tag, index, stroke, strokeThickness, opacity, fill, "_1_" + index, parent);
               res[2] = this.polygon3D(p3, tag, index, stroke, strokeThickness, opacity, fill, "_2_" + index, parent);
               res[3] = this.polygon3D(p4, tag, index, stroke, strokeThickness, opacity, fill, "_3_" + index, parent);
               res[4] = this.polygon3D(p5, tag, index, stroke, strokeThickness, opacity, fill, "_4_" + index, parent);
               res[5] = this.polygon3D(p6, tag, index, stroke, strokeThickness, opacity, fill, "_5_" + index, parent);
           }

           if (inverse) {
               graphics.addVisual(res[0]);
               graphics.addVisual(res[1]);
               graphics.addVisual(res[2]);
               graphics.addVisual(res[3]);
               graphics.addVisual(res[4]);
               graphics.addVisual(res[5]);
           }
           else {
               graphics.addVisual(res[5]);
               graphics.addVisual(res[4]);
               graphics.addVisual(res[0]);
               graphics.addVisual(res[1]);
               graphics.addVisual(res[2]);
               graphics.addVisual(res[3]);
              
           }
           return res;
       },

       updateBox: function (plan, v1, v2, sender) {
           if (plan.Length < 6) return;

           this.update(new Array(vector.vector3D(v1.x, v1.y, v1.z),
                                        vector.vector3D(v2.x, v1.y, v1.z),
                                        vector.vector3D(v2.x, v2.y, v1.z),
                                        vector.vector3D(v1.x, v2.y, v1.z)), plan[0], sender);

           this.update(new Array(vector.vector3D(v1.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v2.z),
                                       vector.vector3D(v1.x, v2.y, v2.z)), plan[1], sender);

           this.update(new Array(vector.vector3D(v1.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v1.y, v1.z),
                                       vector.vector3D(v1.x, v1.y, v1.z)), plan[2], sender);

           this.update(new Array(vector.vector3D(v1.x, v2.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v1.z),
                                       vector.vector3D(v1.x, v2.y, v1.z)), plan[3], sender);

           this.update(new Array(vector.vector3D(v1.x, v1.y, v1.z),
                                       vector.vector3D(v1.x, v1.y, v2.z),
                                       vector.vector3D(v1.x, v2.y, v2.z),
                                       vector.vector3D(v1.x, v2.y, v1.z)), plan[4], sender);

           this.update(new Array(vector.vector3D(v2.x, v1.y, v1.z),
                                       vector.vector3D(v2.x, v1.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v2.z),
                                       vector.vector3D(v2.x, v2.y, v1.z)), plan[5], sender);
       },


       calcNormal: function () {
           if (arguments.length >= 3) {
               // Relative information of the points
               var v1 = arguments[0];
               var v2 = arguments[1];
               var v3 = arguments[2];
               var v4 = vector.vector3DMinus(v1, v2);
               var v5 = vector.vector3DMinus(v3, v2);
               var n = vector.vector3DMultiply(v4, v5);

               var l = vector.getLength(n);//Get length of the vector

               if (l < this.epsilon) {
                   l = 1;
               }

               this.normal= vector.vector3D(n.x / l, n.y / l, n.z / l); //Calculate normalization of the vector
               this.d = -(this.normal.x * v1.x + this.normal.y * v1.y + this.normal.z * v1.z);// Normalized values * 1st coordinates Coordinates - Depth of the plan

               if (arguments[3]) {
                   arguments[3].normal = this.normal;
                   arguments.d = this.d;
               }
           }
           else {
               var Points = arguments[0];
               this.calcNormal(Points[0], Points[1], Points[2], arguments[1]);

               for (var i = 3; (i < Points.length) && (this.test()) ; i++) {
                   this.calcNormal(Points[i], Points[0], Points[i / 2]);
               }
           }
       },

       test: function () {
           return !vector.isValid(this.normal);
       },

       transform: function (matrix, plan) {
           if (plan.Points != null) {
               for (var i = 0; i < plan.Points.length; i++) {

                   plan.VectorPoints[i] = plan.Points[i] = matrixobj.getMatrixVectorMutiple(matrix, plan.Points[i]);
               }

               this.calcNormal(plan.VectorPoints, plan);
           }
           else {
               var v = matrix * (plan.normal * -plan.d);
               plan.normal = matrixobj.getMatrixVectorAnd(matrix, plan.normal);
               vector.normalize(normal);
               plan.d = -(plan.normal & v);
           }
       },

       getPoint: function (x, y, normal, d) {
           if (typeof x == double) {
               var z = -(normal.x * x + normal.y * y + d) / normal.z;

               return vector.vector3D(x, y, z);
           }
           else {
               var position = x;
               var ray = y;
               var dir = vector.vector3DMinus(vector.vector3DStarMultiply(normal, (-d)), position);

               var sv = vector.vector3dAND(dir, normal);
               var sect = sv / vector.vector3dAND(normal, ray);

               return vector.vector3DPlus(position, (vector.vector3DStarMultiply(ray * sect)));
           }
       },

       getNormal: function (transform, VectorPoints) {
           var norm;

           if (VectorPoints != null) {
               norm = vector.getNormal(matrixobj.getMatrixVectorMutiple(transform, VectorPoints[0]),
                  matrixobj.getMatrixVectorMutiple(transform, VectorPoints[1]), matrixobj.getMatrixVectorMutiple(transform, VectorPoints[2]))


               for (var i = 3; (i < VectorPoints.length) && !vector.isValid(norm) && VectorPoints[i / 2]; i++) {
                   var v1 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[i]);
                   var v2 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[0]);
                   var v3 = matrixobj.getMatrixVectorMutiple(transform, VectorPoints[i / 2]);

                   norm = vector.getNormal(v1, v2, v3);
               }
           }
           else {
               norm = matrixobj.getMatrixVectorAnd(transform, normal);
               vector.normalize();
           }

           return norm;

       },

     createTextElement:function(position,  element, xLen, yLen)
      {
        
       var vectorColl = [];
       var x = position.x;
       var y = position.y;
 
       var desiredWidth = element.Width;
       var desiredHeight = element.Height;

          vectorColl[0] = this.vector.vector3D(x, y, position.z);
          vectorColl[1] = this.vector.vector3D(x + desiredWidth, y + desiredHeight + yLen, position.z);
          vectorColl[2] = this.vector.vector3D(x + desiredWidth + xLen, y + desiredHeight + yLen, position.z);
         return this.text3D(element, vectorColl);
     },

     redraw: function () { },

     drawPolyLine: function (panel, sender) {
         var transform = ej.Ej3DRender.transform;
         var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
         var StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
         pathDirection.append("M" + " " + (StartPoint.x) + " " + (StartPoint.y) + " ");
         for(var i=0; i< panel.VectorPoints.length; i++)
         {
            lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
            pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");
                     
         }        
          
         direction = pathDirection.toString();

        
         var optionsLine = {
             'id': panel.element.id,
             'stroke-dasharray': panel.element.dashArray,
             'stroke-width': panel.element.width,
             'stroke': panel.element.stroke,
             'd': direction
         };

         sender.svgRenderer.drawPath(optionsLine, panel.element.child);
     },

       drawLine: function (panel, sender) {
           
           var transform = ej.Ej3DRender.transform;
           if (transform == null) return;
           var actual3DPosition1 = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
           var actual3DPosition2 = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[2], transform);

           var optionsLine = {
               'id': panel.element.id,
               x1: actual3DPosition1.x ,
               y1: actual3DPosition1.y,
               x2: actual3DPosition2.x,
               y2:actual3DPosition2.y,
               'stroke-dasharray': panel.element.dashArray,
               'stroke-width': panel.element.width,
               'stroke': panel.element.stroke,
               'opacity': panel.element.opacity
           };

           sender.svgRenderer.drawLine(optionsLine, panel.element.child);
 

       },
       drawText: function (panel, sender) {
           
           var element = panel.element;
           var transform =ej.Ej3DRender.transform;
           if (transform == null) return;
           var actual3DPosition = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
           var x = actual3DPosition.x;
           var y = actual3DPosition.y;
 
           if (element.tag == "text") {
               var options = {
                   'id': element.id,
                   'x': x,
                   'y': y,
                   'fill': element.font.color,
                   'font-size': element.font.size,
                   'font-family': element.font.fontFamily,
                   'font-style': element.font.fontStyle,
                   'font-weight': element.font.fontWeight,
                   'opacity': element.font.opacity,
                   'text-anchor': element.TextAnchor,            
                   "cursor":"default",
                   'transform': element.Angle ? 'rotate(' + element.Angle + ',' + (x) + ',' + y + ')' : null
               };           
              
               sender.svgRenderer.drawText(options, panel.element.Label.Text, panel.element.child, panel.element.Label);			 
           }
           else {
               var series = element.series;           
 
               var labelFormat = (series.yAxis && series.yAxis.labelFormat) ? series.yAxis.labelFormat : "";
               var pointText = (element.point.text) ? element.point.text : element.point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
               var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);
               var margin = series.marker.dataLabel.margin;
               var width = textOffset.width + margin.left + margin.right;
               var height = textOffset.height + margin.top + margin.bottom;
               var location = { X: actual3DPosition.x, Y: actual3DPosition.y };

                  
                      if (series.marker.dataLabel.shape)
                          var elementShape = series.marker.dataLabel.shape;
                      else if (series.marker.dataLabel.shape)
                          elementShape = marker.dataLabel.shape;
                      else
                          elementShape = "None";

                      $.each(sender.model.symbolShape, function (name) {
                          if (elementShape.toLowerCase() == name.toLowerCase())
                              symbolName = name;
                      });

                   var xXalue = location.X - (margin.left) / 2 + (margin.right) / 2;
                   var yValue = location.Y - (margin.top) / 2 - (height / margin.top) + (margin.bottom) / 2;

                   var seriesIndex = $.inArray(series, sender.model.series);
                   ej.EjSeriesRender.prototype.dataLabelSymbol(seriesIndex, series, element.pointIndex, xXalue, yValue, width, height, symbolName, sender);
            
           }

       },

       draw: function (panel,sender) {            
           if (panel.VectorPoints == null || panel.VectorPoints.length <= 0) return;
           var transform = ej.Ej3DRender.transform;
           var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
           var color = panel.Fill;
		   var format = sender.svgRenderer.checkColorFormat(color);
		   if (!format)
		      color=sender.colorNameToHex(color);
           var figure={Segments:null, StartPoint:null};
           if (transform != null)
           {
               figure.StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
               pathDirection.append("M" + " " + (figure.StartPoint.x) + " " + (figure.StartPoint.y) + " ");
               for(var i=0; i< panel.VectorPoints.length; i++)
                {
                       lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
                       pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");
                     
                }
           }
          
           direction = pathDirection.toString();
          
            var name = "Light";
            var lightCoefZ =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(0, 0, 1))) - 1));
            var lightCoefY =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(0, 1, 0))) - 1));
            var lightCoefX =(2 * (Math.abs(vector.vector3dAND(panel.normal, vector.vector3D(1, 0, 0))) - 1));
           if (lightCoefZ == lightCoefX)
           {               
              name = "ZLight";
              color = this.applyZLight(color, sender);  
           }
          else if (((lightCoefY == lightCoefZ) || (lightCoefZ != 0 && lightCoefY < lightCoefZ)))
          {
              name = "XLight";
              color= this.applyXLight(color, sender);
           }
          else if (lightCoefZ < 0)
          {
              name = "ZLight";
              color = this.applyZLight(color, sender);
          }
          else
          {
              name = "Light";
              color = color;
          }
          if (sender.model.AreaType == "none")
               panel.StrokeThickness = 0;
           var optionsMinorGrid = {
               'id': sender.svgObject.id +"_"+ panel.Name,
               'name':name, 
               'fill': color,
               'stroke': panel.Stroke,
               'stroke-width': panel.StrokeThickness,
               'opacity': panel.Opacity,
               'd': direction
           };
           sender.svgRenderer.drawPath(optionsMinorGrid, panel.Element.Parent);

       },

       applyXLight: function (color, sender)
       { 
           var RGB = sender.svgRenderer.hexToRGB(color);
           RGB.R = (parseInt)(RGB.R * 0.7);
           RGB.G = (parseInt)(RGB.G * 0.7);
           RGB.B = (parseInt)(RGB.B * 0.7);
           return sender.svgRenderer.hexFromRGB(RGB);
       },

       applyZLight:function(color, sender)
         {
           var RGB = sender.svgRenderer.hexToRGB(color);
           RGB.R = (parseInt)(RGB.R * 0.9);
           RGB.G = (parseInt)(RGB.G * 0.9);
           RGB.B = (parseInt)(RGB.B * 0.9);
           return sender.svgRenderer.hexFromRGB(RGB);
        },

       update: function (updatedVectors, panel, sender) {
           if (panel.VectorPoints == null || panel.VectorPoints.length <= 0) return;
           panel.VectorPoints = updatedVectors;
           var transform = ej.Ej3DRender.transform;
           var pathDirection = ej.EjSvgRender.utils._getStringBuilder();
           var color = panel.Fill;
           var figure = { Segments: null, StartPoint: null };
           if (transform != null) {
               figure.StartPoint = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[0], transform);
               pathDirection.append("M" + " " + (figure.StartPoint.x) + " " + (figure.StartPoint.y) + " ");
               for (var i = 0; i < panel.VectorPoints.length; i++) {
                   lineSegment = ej.EjSvgRender.chartTransform3D.toScreen(panel.VectorPoints[i], transform);
                   pathDirection.append("L" + " " + (lineSegment.x) + " " + (lineSegment.y) + " ");

               }
           }

           direction = pathDirection.toString();    

           if ($(sender.chartObj.chart3D).find("#" + sender.chartObj.svgObject.id + "_" + panel.Name).length > 0) {
               var element = $(sender.chartObj.chart3D).find("#" + sender.chartObj.svgObject.id + "_" + panel.Name)[0];
               sender.chartObj.svgRenderer._setAttr($(element), { 'd': direction });
              
           }
         
       }


   }
   var Ej3DRender = new ej.Ej3DRender();
   var vector = new Ej3DRender.vector3D();
   var matrixobj = new Ej3DRender.matrix3D();
   var bsptreeobj = new Ej3DRender.BSPTreeBuilder();
   var polygonobj = new Ej3DRender.polygon3D();
   var graphics = new Ej3DRender.Graphics3D();

  
})(jQuery);;
ej.Ej3DAxisRenderer = function () {
};

ej.Ej3DSeriesRender = function () {
};


ej.Ej3DChart = function () {
};

ej.series3DTypes = {};

(function ($) {

    ej.Ej3DAxisRenderer.prototype = {

        _drawAxes: function (index, axis) {
            if (axis.majorGridLines.visible)
                this._drawGridLines3D(axis);
            if (axis.visible && axis.majorTickLines.visible)
                this._renderTicks3D(axis, axis.majorTickLines.size, axis.majorTickLines.width);

            if (axis.visible) {
                this._drawAxisLabel(axis);
                this._drawAxisTitle(axis);
            }
        },

        _drawAxisTitle: function (axis) {
           if(axis.title.visible)
			{
            if (axis.orientation.toLowerCase() == "horizontal") {
                var xtitleLocation = (this.model.elementSpacing) + axis._LableMaxWidth.height;
                var titlesize = (ej.EjSvgRender.utils._measureText(axis.title.text, (axis.width), axis.title.font).height / 2);
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: axis.title.text, location: { x: (axis.width) / 2, y: (xtitleLocation + titlesize + axis.majorTickLines.size + axis.axisLine.width + this.model.wallSize) }, axes: axis };
                this._trigger("axesTitleRendering", commonEventArgs);

                var x1 = commonEventArgs.data.location.x + axis.x;
                var y1 = (axis.opposedPosition) ? (axis.y - commonEventArgs.data.location.y) : (commonEventArgs.data.location.y + axis.y);

                element = { Width: titlesize.width, Height: titlesize.height, Label: commonEventArgs.data, TextAnchor: 'middle', tag: 'text', font: axis.title.font, id: this.svgObject.id + "axisTitle", child: this.chart3D };

                this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
            }
            else {
                var titleSize = ej.EjSvgRender.utils._measureText(axis.title.text, (axis.height), axis.title.font);
                var x1 = (axis.opposedPosition) ? axis.x + ((3 * this.model.elementSpacing) + axis._LableMaxWidth.width + axis.majorTickLines.size + axis.axisLine.width) : axis.x - ((3 * this.model.elementSpacing) + axis._LableMaxWidth.width + axis.majorTickLines.size + axis.axisLine.width + this.model.wallSize);

                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: axis.title.text, location: { x: titleSize.width / 2, y: 0 }, axes: axis };
                this._trigger("axesTitleRendering", commonEventArgs);

                var y1 = commonEventArgs.data.location.y + (axis.y + axis.height) + (((axis.height) / 2) * -1);

                var labelRotation = (axis.opposedPosition) ? 90 : -90;

                element = { Width: titleSize.width, Height: titleSize.height, Angle: -90 , Label: commonEventArgs.data, TextAnchor: 'middle', tag: 'text', font: axis.title.font, id: this.svgObject.id + '_YAxisTitle', child: this.chart3D };

                this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
            }
           }
        },

        _drawAxisLabel: function (axis) {
            var labelsCount = axis.visibleLabels.length;
           
            for (var i = 0; i < labelsCount; i++) {
				if (!ej.util.isNullOrUndefined(axis.visibleLabels[i].Text)) {
                var x1 = 0, y1 = 0, x2 = 0, y2 = 0;
                var textAnchor;
                
                var textSize = ej.EjSvgRender.utils._measureText(axis.visibleLabels[i].Text, this.model.m_AreaBounds.Width, axis.font);
				var value = (axis.visibleLabels[i].Value - axis.visibleRange.min) / (axis.visibleRange.delta);
                value = (axis.isInversed) ? 1 - value : value;
                value = isNaN(value) ? 0 : value;

                if (axis.orientation.toLowerCase() == "horizontal") {
                    x2 = x1 = (Math.round(axis.width * value)) + this.model.m_AreaBounds.X + ((axis.plotOffset));
                    y1 = (axis.opposedPosition) ? (this.model.m_AreaBounds.Y - this.model.wallSize - (axis.majorTickLines.size) - (this.model.elementSpacing/2)) : this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height + this.model.wallSize + (axis.majorTickLines.size) + this.model.elementSpacing;
                    textAnchor="middle"
                }
                else {
                    y1 = y2 = Math.round(axis.plotOffset + axis.y + (textSize.height / 4) + (axis.height * (1 - value)));
                    x1 = (axis.opposedPosition) ? (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width + this.model.wallSize + axis.majorTickLines.size + this.model.elementSpacing / 2) : (this.model.m_AreaBounds.X - this.model.wallSize - axis.majorTickLines.size - this.model.elementSpacing / 2);
                    textAnchor = (axis.opposedPosition) ?"start" : "end"
                }
 
             
                element = { Width: textSize.width, Height: textSize.height, Label: axis.visibleLabels[i], TextAnchor: textAnchor, tag: 'text', font: axis.font, id: this.svgObject.id + axis.orientation + i, child: this.chart3D };

                this.graphics.addVisual(this.polygon.createTextElement(this.vector.vector3D(x1, y1, 0), element, 10, 10));
				}
            }

        },

        _renderTicks3D: function (axis, size, width) {
            var labelsCount = axis.visibleLabels.length;
            var parent;
            var gEle;

            for (var i = 0; i < labelsCount; i++)
            {
                var x1 = 0, x2 = 0, y1=0, y2=0;
            
       
                var value = (axis.visibleLabels[i].Value - axis.visibleRange.min) / (axis.visibleRange.delta);
                value = (axis.isInversed) ? 1 - value : value;
                value = isNaN(value) ? 0 : value;
            
                if (axis.orientation.toLowerCase() == "horizontal")
                {
                    x2 = x1 = (Math.round(axis.width * value)) + this.model.m_AreaBounds.X + ((axis.plotOffset));
                }
                else
                {
                    y1 = y2 = Math.round(axis.plotOffset + ((axis.height) * (1 - value))+axis.y);
                }

                var position = this._calculatePosition3D(axis, axis.tickLinesPosition, size, width, x1, y1, x2, y2);


                var line = { width: axis.majorTickLines.width, stroke: axis.majorTickLines.color, child: this.chart3D,  tag: 'line' };

                line.id = this.svgObject.id + axis.name + "_majorTickLines_" + i;

                this.graphics.addVisual(this.polygon.createLine(line, position.X1, position.Y1, position.X2, position.Y2, 0));

                if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i < labelsCount - 1) {
                    minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                    for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                        value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k + 1)));

                        value = isNaN(value) ? 0 : value;

                        if (axis.orientation.toLowerCase() == "horizontal") {
                            x1 = x2 = Math.round(axis.plotOffset + (this.model.m_AreaBounds.Width * value));
                        }
                        else {
                            y1 = y2 = Math.round(axis.plotOffset + ((this.model.m_AreaBounds.Height) * (1 - value)));
                        }

                        var position = this._calculatePosition3D(axis, axis.tickLinesPosition, size, width, x1, y1, x2, y2);


                        var line = { width: axis.minorTickLines.width, stroke: axis.minorTickLines.color, child: this.chart3D, tag: 'line' };

                        line.id = this.svgObject.id + axis.name + "_minorTickLines_" + i + k;

                        this.graphics.addVisual(this.polygon.createLine(line, position.X1, position.Y1, position.X2, position.Y2, 0));
                    }
                }
            }
        },

        _calculatePosition3D: function (axis, ticksPosition, tickSize, width, x1, y1, x2, y2) {
            var orientation = axis.orientation;
            var isOpposed = axis.opposedPosition;
            ticksPosition = "outside";
            if (axis.orientation.toLowerCase() == "horizontal") {
                switch (ticksPosition) {
                    case "inside":
                        y1 = isOpposed ? width : 0;
                        y2 = isOpposed ? y1 + tickSize : tickSize;
                        break;
                    case "outside":
                        y1 = 0;
                        y2 = isOpposed ? tickSize : y1 + tickSize;
                        break;
                }
                var areabounds = this.model.m_AreaBounds;
                var screenPositionTop = ((isOpposed) ? this.model.m_AreaBounds.Y - this.model.wallSize - (tickSize) : this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height + this.model.wallSize - (tickSize / 2));
                var screenPositionLeft = axis.x;
                y1 += screenPositionTop;
                y2 += screenPositionTop;

                x1 = x2 = x1;
            }
            else {
                switch (ticksPosition) {
                    case "inside":
                        x1 = 0;
                        x2 = isOpposed ? tickSize : x1 + tickSize;
                        break;
                    case "outside":
                        x1 = 0;
                        x2 = isOpposed ? x1 + tickSize : tickSize;
                        break;
                }
                var screenPositionLeft = (isOpposed) ? this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width + this.model.wallSize  : (this.model.m_AreaBounds.X - this.model.wallSize - tickSize);
                var screenPositionTop = this.model.m_AreaBounds.Y;
                x1 += screenPositionLeft;
                x2 += screenPositionLeft;

                y1 = y2 = y1;
            }
            return { X1: x1, Y1: y1, X2: x2, Y2: y2 };
        },

    

        _drawGridLines3D:function(axis)
        {
          
            if (axis == null)
                return;
        
            var labelsCount = axis.visibleLabels.length;
            var minorTicks;
            var orientation = axis.orientation;
            var x1, x2, y1, y2;
            if (orientation.toLowerCase() == "horizontal")
            {
                 
                var i;
                for (i = 0; i < labelsCount; i++)
                {                
                    var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value);
                    value = isNaN(value) ? 0 : value;
					 x2 = x1 = (Math.round(axis.width * value)) + this.model.m_AreaBounds.X+axis.plotOffset;
                    y1 = this.model.m_AreaBounds.Y;
                    y2 = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;
              
                
                    var depth = this.model.depth > 2 ? this.model.depth - 2 : 1;
                    var bottom = axis.opposedPosition ? this.model.m_AreaBounds.Y : this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y;

                    var line = { opacity: axis.majorGridLines.opacity, width: axis.majorGridLines.width, stroke: axis.majorGridLines.color, child: this.chart3D,  tag: 'line' };
                    line.id = this.svgObject.id + axis.name + "_gridlines_" + i;
                    this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));
                    var parallelLine = $.extend({}, line);
                    parallelLine.id = this.svgObject.id + axis.name + "_parallelGridlines_" + i;
                  
                    var line3D =   this.polygon.createLine(parallelLine, x2, 0, x2, -depth, bottom);
                    //To fold the gridline alone the wall(bottom)
                    this.polygon.transform(this.matrixobj.tilt((parseFloat)(Math.PI / 2)), line3D);
                        
                    this.graphics.addVisual(line3D);
                    
                    if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i<labelsCount-1) {
                        minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                        for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                            value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k+1)));
                            value = isNaN(value) ? 0 : value;
                            x2 = x1 = (Math.round(this.model.m_AreaBounds.Width * value) + this.model.m_AreaBounds.X);
                            y1 = this.model.m_AreaBounds.Y;
                            y2 = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;

                            var line = { opacity: axis.minorGridLines.opacity, width: axis.minorGridLines.width, stroke: axis.minorGridLines.color, child: this.chart3D, tag: 'line' };
                            line.id = this.svgObject.id + axis.name + "_minorgridlines_" + i + k;
                            this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));
                            var parallelLine = $.extend({}, line);
                            parallelLine.id = this.svgObject.id + axis.name + "_parallelMinorGridlines_" + i + k;

                            var line3D = this.polygon.createLine(parallelLine, x2, 0, x2, -depth, bottom);
                            //To fold the gridline alone the wall(bottom)
                            this.polygon.transform(this.matrixobj.tilt((parseFloat)(Math.PI / 2)), line3D);

                            this.graphics.addVisual(line3D);
                        }
                    }
                
                    index++;
                }
            }
            else
            {
                var i;
       
              
                for (i = 0; i < labelsCount; i++)
                {
                    //if (i < linesCount)
                    //{
                    var line;
					var value = (axis.visibleLabels[i].Value - axis.visibleRange.min) / (axis.visibleRange.delta); 
                    value = (axis.isInversed) ? 1 - value : value;
                    value = isNaN(value) ? 0 : value;
                    x1 = this.model.m_AreaBounds.X;
                    y1 = Math.round((axis.height) * (1 - value)) + 0.5;
                    y1 += axis.y;
                    x2 = x1 + this.model.m_AreaBounds.Width;
                    y2 = y1;

                    var depth = this.model.depth > 2 ? this.model.depth - 2 : 1;

                    var line = { opacity: axis.majorGridLines.opacity, width: axis.majorGridLines.width, stroke: axis.majorGridLines.color, axisName: axis.name, child: this.chart3D, tag: 'line' };
                 
                    line.id = this.svgObject.id + axis.name + "_gridlines_" + i;

                    this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));

                    var depthD = axis.opposedPosition ? this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X + 1 : this.model.m_AreaBounds.X;

                    var sideLine = $.extend({}, line);
                    sideLine.id = this.svgObject.id + axis.name + "_parallelGridlines_" + i;
              
                    var line3D = this.polygon.createLine(sideLine, -depth, y2, 0, y2, depthD);
                    //To fold the gridline alone the wall(right of vertical)
                    this.polygon.transform(this.matrixobj.turn((parseFloat)(-Math.PI / 2)), line3D);
                    this.graphics.addVisual(line3D);

                    if (axis.minorGridLines.visible && axis.minorTicksPerInterval > 0 && i < labelsCount - 1) {
                        minorTicks = axis.visibleRange.interval / (axis.minorTicksPerInterval + 1);
                        for (var k = 0; k < axis.minorTicksPerInterval; k++) {
                            var value = ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[i].Value + (minorTicks * (k + 1)));
                            value = isNaN(value) ? 0 : value;
                            x1 = this.model.m_AreaBounds.X;
                            y1 = Math.round((axis.height) * (1 - value)) + 0.5;
                            y1 += axis.y;
                            x2 = x1 + this.model.m_AreaBounds.Width;
                            y2 = y1;
                            var line = { opacity: axis.minorGridLines.opacity, width: axis.minorGridLines.width, stroke: axis.minorGridLines.color, axisName: axis.name, child: this.chart3D, tag: 'line' };

                            line.id = this.svgObject.id + axis.name + "_minorgridlines_" + i + k;

                            this.graphics.addVisual(this.polygon.createLine(line, x1, y1, x2, y2, depth));                          

                            var sideLine = $.extend({}, line);
                            sideLine.id = this.svgObject.id + axis.name + "_parallelMinorGridlines_" + i + k;

                            var line3D = this.polygon.createLine(sideLine, -depth, y2, 0, y2, depthD);
                            //To fold the gridline alone the wall(right of vertical)
                            this.polygon.transform(this.matrixobj.turn((parseFloat)(-Math.PI / 2)), line3D);
                            this.graphics.addVisual(line3D);
                        }
                    }

                    ////}
                    index++;
                }

            }
        }

    },

    

    ej.Ej3DSeriesRender.prototype={
        _getSegmentDepth:function(series)
        {
            var actualDepth = this.chartObj.model.depth;
            var start, end;

            if (this.chartObj.model._sideBySideSeriesPlacement)
            {
                var space = actualDepth / 4;
                start = space;
                end = space * 3;
            }
            else
            {
                var index = series.position-1;
                var count = series.all;
                var space = actualDepth / ((count * 2) + count + 1);
                start = space + (space * index * 3);
                end = start + space * 2;
            }
            return {Start:start, End:end, Delta:end-start};
        },

        createSegment: function (center, start, end, height, r, i, y, insideRadius, pointindex) {
         
            return {
                StartValue : start,
                EndValue : end,
                depth : height,
                radius : r,      
                index : i,
                YData : y,
                Center : center,
                inSideRadius : insideRadius,
                ActualEndValue : end,
                ActualStartValue: start,
                pointIndex: pointindex
            }

            return segment;

        },

        calculateSize:function(sender, series){
            var legend = sender.chartObj.model.legend;
   
            var legXSpace = 0;
            var legYSpace = 0;
            if (legend.visible && legend.position.toLowerCase() != "custom") {
                if (legend.position.toLowerCase() == "right" || legend.position.toLowerCase() == "left")
                    legXSpace = ((legend.position.toLowerCase() == "right") ? sender.chartObj.model.margin.right : sender.chartObj.model.margin.left) + sender.chartObj.model.LegendViewerBounds.Width;
                else
                    legYSpace = ((legend.position.toLowerCase() == "top") ? sender.chartObj.model.margin.top : sender.chartObj.model.margin.bottom) + sender.chartObj.model.LegendViewerBounds.Height;

            }

            var yOffset = ((sender.chartObj.model.title.text) ? sender.chartObj.model._titleLocation.Y : 0) + legYSpace;

            series.actualWidth = $(sender.chartObj.svgObject).width() - legXSpace;
            series.actualHeight = $(sender.chartObj.svgObject).height() - yOffset;
			 var centerx = series.actualWidth * 0.5 + ((legend.position.toLowerCase() === "left") ? legXSpace : 0);
            var centery = series.actualHeight * 0.5 + ((legend.position.toLowerCase() === "top") ? yOffset : ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y) : 0));
            return { centerX: centerx, centerY: centery };
        },

        createPoints: function (series, sender) {
            series.segments = [];
            var size = this.calculateSize(sender, series);
            var all = 0;
            var visiblepoints = this._pieVisiblePoints(series);
            for (j = 0; j < visiblepoints.length; j++) {
            all += visiblepoints[j].YValues[0];
            }
            var coef = 360 / all;
            var count = series.points.length;
            var radius =(series.radius)? series.radius : (series.pieCoefficient * Math.min(series.actualWidth, series.actualHeight)) / 2;

            var InsideRadius = (series.InsideRadius) ? series.InsideRadius : 0;
            var YValues = sender.chartObj._getYValues(series);

            var pieHeight = sender.chartObj.model.depth;
            var arcStartAngle = 0, arcEndAngle = 0;
            var current = 0;
            var segindex = 0;
            for (var i = 0; i < count; i++) {
                if (series.points[i].visible) {                    
                    var pointindex = $.inArray(series.points[i], series.points);
                var val = Math.abs(YValues[i]);
                arcEndAngle = Math.abs(val) * ((Math.PI * 2) / all);
                var rect = {};
                rect.x = 0;
                rect.y = 0;

                if (val != 0) {
                    if (series.explodeIndex == i || series.explodeAll) {
                        var offset = {X:0, Y:0};
                        offset.X = Math.cos(2 * Math.PI * (current + val / 2) / all),
                        offset.Y = Math.sin(2 * Math.PI * (current + val / 2) / all);
                        rect.x = 0.01 * radius * offset.X * series.explodeOffset;
                        rect.y = 0.01 * radius * offset.Y * series.explodeOffset;
                    }

                    var center = sender.chartObj.vector.vector3D(rect.x +  size.centerX, rect.y + size.centerY, 0);
                    {
                            var segment = this.createSegment(center, (parseFloat)(coef * current), (parseFloat)(coef * val), pieHeight, radius, i, val, InsideRadius, pointindex, series);
                        series.segments.push(segment);
                    }
                }

                if (series.marker.dataLabel.visible)
                    this._addPieDataLabel(segindex, YValues[i], arcStartAngle, arcStartAngle + arcEndAngle, i, radius, this._isChartRotated(sender) ? sender.chartObj.model.depth + 5 : 0, center, series.points[i]);

                segindex++;
                arcStartAngle += arcEndAngle;
                current += val;

                }
            }
            return series.segments;
        },

        _addPieDataLabel: function (x, y, startAngle, endAngle, i, radius, startDepth, center, point) {
            var angle = (startAngle + endAngle) / 2;
            point.symbolLocation = { x: 0, y: 0, radius: 0, angle: 0, StartDepth: 0 };
            point.symbolLocation.x = x;
            point.symbolLocation.y = y;
            point.symbolLocation.radius = radius;
            point.symbolLocation.angle = angle;
            point.symbolLocation.center = center;
            point.startDepth = startDepth;
        },

        _isChartRotated:function(sender)
           {
            var actualTiltView = Math.abs(sender.chartObj.model.tilt % 360);
            var actualRotateView = Math.abs(sender.chartObj.model.rotattion % 360);
            if ((actualTiltView > 90 && actualTiltView < 270) ^ (actualRotateView > 90 && actualRotateView < 270))
              {
                return true;
              }
                  return false;
          },

        createSector: function (seg, sender, style) {
            var count = (parseInt)(Math.ceil(seg.ActualEndValue / 6));
            var DtoR = Math.PI / 180;
            var inc = 0;
            var Points = [];
            var depth = sender.chartObj.model.depth;
            if (count < 1) return null;
            var res = [];
            var f = seg.ActualEndValue / count;

            var oPts = [];
            var iPts = [];

            for (var i = 0; i < count + 1; i++) {
                var ox = (parseFloat)(seg.Center.x + seg.radius * Math.cos((seg.ActualStartValue + i * f) * DtoR));
                var oy = (parseFloat)(seg.Center.y + seg.radius * Math.sin((seg.ActualStartValue + i * f) * DtoR));

                oPts[i] = { X: ox, Y: oy };

                var ix = (parseFloat)(seg.Center.x + seg.inSideRadius * Math.cos((seg.ActualStartValue + i * f) * DtoR));
                var iy = (parseFloat)(seg.Center.y + seg.inSideRadius * Math.sin((seg.ActualStartValue + i * f) * DtoR));

                iPts[i] = { X: ix, Y: iy };
                Points.push({ X: ox, Y: oy });
            }

            var oPlgs = [];

            for (var i = 0; i < count; i++) {
                vts = new Array(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, 0),
                       sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, depth),
                       sender.chartObj.vector.vector3D(oPts[i + 1].X, oPts[i + 1].Y, depth),
                       sender.chartObj.vector.vector3D(oPts[i + 1].X, oPts[i + 1].Y, 0))


                oPlgs[i] = sender.chartObj.polygon.polygon3D(vts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString()  + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D);
                inc++;
            }

            res[1] = oPlgs;

            if (seg.inSideRadius > 0) {
                var iPlgs = [];

                for (var i = 0; i < count; i++) {
                    vts = new Array(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, 0),
                      sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[i + 1].X, iPts[i + 1].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[i + 1].X, iPts[i + 1].Y, 0))


                    iPlgs[i] = sender.chartObj.polygon.polygon3D(vts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D);
                    inc++;
                }

                res[3] = iPlgs;
            }

            var tVtxs = [];
            var bVtxs = [];

            for (var i = 0; i < count + 1; i++) {
                tVtxs.push(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, 0));
                bVtxs.push(sender.chartObj.vector.vector3D(oPts[i].X, oPts[i].Y, depth));
            }

            if (seg.inSideRadius > 0) {
                for (var i = count; i > -1; i--) {
                    tVtxs.push(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, 0));
                    bVtxs.push(sender.chartObj.vector.vector3D(iPts[i].X, iPts[i].Y, depth));
                }
            }
            else {
                tVtxs.push(seg.Center);
                bVtxs.push(sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth));
            }
            res[0] = [];
            res[0].push(sender.chartObj.polygon.polygon3D(tVtxs, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D));
            inc++;
            res[0].push(sender.chartObj.polygon.polygon3D(bVtxs, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D));
            inc++;



            if (seg.inSideRadius > 0) {
                var rvts = new Array(

                      sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, 0),
                      sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[0].X, iPts[0].Y, depth),
                      sender.chartObj.vector.vector3D(iPts[0].X, iPts[0].Y, 0)
                 )

                var lvts = new Array
                 (
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(iPts[count].X, iPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(iPts[count].X, iPts[count].Y, 0)
                 )
                res[2] = [];

                res[2].push(sender.chartObj.polygon.polygon3D(rvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;
                res[2].push(sender.chartObj.polygon.polygon3D(lvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;             

            }
            else {
                var rvts = new Array(
                     sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[0].X, oPts[0].Y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, 0)
                )

                var lvts = new Array(
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, 0),
                     sender.chartObj.vector.vector3D(oPts[count].X, oPts[count].Y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, depth),
                     sender.chartObj.vector.vector3D(seg.Center.x, seg.Center.y, 0)
                )
                res[2] = [];
                res[2].push(sender.chartObj.polygon.polygon3D(rvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;
                res[2].push(sender.chartObj.polygon.polygon3D(lvts, this, seg.index, style.borderColor, style.borderWidth, style.opacity, style.interior, inc.toString() + "_Region" + "_Series_" + 0 + "_Point_" + seg.index, sender.chartObj.chart3D));
                inc++;


            }


            return res;
        },

        createPolygons: function (series, sender, returnPoly) {
            var segments;
            if (!series.segments || !returnPoly)
                segments = this.createPoints(series, sender);
            else
                segments = series.segments
            var poligons = [];
            var seg;
            var style;

            for (var i = 0; i < segments.length; i++) {
                seg = segments[i];
                var pointindex = segments[i].pointIndex;
                style = sender.chartObj.setStyle(sender, series, 0, pointindex);
                if (typeof (sender.chartObj.model.pointColors[pointindex]) == "object")
                    style.interior = sender.chartObj.model.pointColors[pointindex][0].color;
                var plgs = this.createSector(seg, sender, style);
                if (plgs != null)
                    for (var ai = 0; ai < plgs.length; ai++) {
                        if (!poligons[ai])
                            poligons[ai] = [];
                        if (plgs[ai] != null) {
                            for (var pi = 0; pi < plgs[ai].length; pi++) {
                                poligons[ai].push(plgs[ai][pi]);

                            }
                        }
                    }


            }
            if (returnPoly) {
                return poligons;               
            }

            for (var ai = 0; ai < poligons.length; ai++) {
                for (var k = 0; k < poligons[ai].length; k++) {
                    sender.chartObj.graphics.addVisual(poligons[ai][k]);
                }
            }
        },

        draw3DDataLabel: function (series, pointIndex, point, sender) {

			 var connectorHeight=series.marker.dataLabel.connectorLine.height;
            if (this.chartObj.model.AreaType == "none") {
                location = { x: 0, y: 0 };
                var pointText = (point.text) ? point.text : point.y;

                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: pointText, location: { x: pointX, y: pointY }, series: series, pointIndex: pointIndex };
                this.chartObj._trigger("displayTextRendering", commonEventArgs);

                var size = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);

                if (series.type.toLowerCase() == "pie") {
                    var width = Math.min(series.actualWidth, series.actualHeight) / 2;
                    var center = point.symbolLocation.center;
					 if (ej.util.isNullOrUndefined(connectorHeight))
                         connectorHeight = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, null, series.marker.dataLabel.font).height;
                    if (series != null && series.labelPosition.toLowerCase() != "inside")
                        radius = point.symbolLocation.radius + connectorHeight;
                    else if (series != null)
                        radius = point.symbolLocation.radius / 2;
                   
                    pointX = location.X = center.x + radius * Math.cos(point.symbolLocation.angle);
                    pointY = location.Y = center.y + radius * Math.sin(point.symbolLocation.angle);

                }
                else if (series.type.toLowerCase() == "doughnut")
                {
                   
                    var width = Math.min(series.actualWidth, series.actualHeight) / 2;
                    var center = point.symbolLocation.center;

                    var dradius = point.symbolLocation.radius * series.doughnutCoefficient;
                    if (ej.util.isNullOrUndefined(connectorHeight))
                         connectorHeight = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, null, series.marker.dataLabel.font).height;
                    if (series != null &&series.labelPosition != "inside")
                        radius = point.symbolLocation.radius +  connectorHeight;
                    else if (series != null )
                        radius = dradius + (point.symbolLocation.radius-dradius) / 2;
                    pointX=location.X = center.x + radius * Math.cos(point.symbolLocation.angle);
                    pointY=location.Y = center.y + radius * Math.sin(point.symbolLocation.angle);
                }
            }
            else {
                var labelFormat = series.yAxis.labelFormat ? series.yAxis.labelFormat : "";
                var seriesIndex = $.inArray(series, this.chartObj.model.series);
                var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
                var pointHeight = 0;
                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonEventArgs.data = { Text: pointText, location: { x: pointX, y: pointY }, series: series, pointIndex: pointIndex };
                this.chartObj._trigger("displayTextRendering", commonEventArgs);

                var size = ej.EjSvgRender.utils._measureText(pointText, null, series.marker.dataLabel.font);

                var location = ej.EjSvgRender.utils.Transform3DToVisible(series, point.symbolLocation.x, point.symbolLocation.y, this.chartObj);
                var connectorHeight = (series.marker.dataLabel.connectorLine.height) ? series.marker.dataLabel.connectorLine.height : 0;
                var angle = (6.28 * (1 - (90 / 360.0)));
                var pointY = location.Y;
                var pointX = location.X;
                if (!this.chartObj.model.requireInvertedAxes) {
                    if (point.y > 0)
                        pointY = location.Y + (Math.sin((angle)) * connectorHeight);
                    else
                        pointY = location.Y + (Math.sin((-angle)) * connectorHeight);
                }
                else {
                    if (point.x > 0)
                        pointX = location.X - (Math.sin((angle)) * connectorHeight);
                    else
                        pointX = location.X + (Math.sin((angle)) * connectorHeight);
                      pointHeight = - size.height / 3;
                }
            }

         
          
            var depthInfo = this._getSegmentDepth(series);

            

            if (series.marker.dataLabel.shape.toLowerCase() != "none") {
              
                var element = { tag: 'dataLabel', series: series, point: point, pointIndex: pointIndex, id: this.chartObj.svgObject.id + seriesIndex + '_DataLabel' + pointIndex, child: this.chartObj.chart3D };

                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(pointX, pointY, (point.symbolLocation.z) ? point.symbolLocation.z : 0), element, 0, -size.height));
            }

          
            var element = { Width: size.width, Height: size.height, Label: commonEventArgs.data, TextAnchor: "middle", tag: 'text', font: series.marker.dataLabel.font, Angle: series.marker.dataLabel.angle ,id: this.chartObj.svgObject.id + '_SeriesText' + pointIndex + seriesIndex, child: this.chartObj.chart3D };

            if (this.chartObj.model.AreaType != "none") {
                if (series.marker.dataLabel.connectorLine.height && series.marker.dataLabel.connectorLine.height > 0) {
                    var drawPoints = [];
                    drawPoints.push({ x: location.X, y: location.Y + pointHeight, z: point.symbolLocation.z });

                    drawPoints.push({ x: pointX, y: pointY + pointHeight, z: point.symbolLocation.z });
                    this.drawLineSegment(drawPoints, pointIndex, series);
                }
                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(pointX, pointY, point.symbolLocation.z), element, 0, -size.height));
            }
            else {
                  if (series.labelPosition.toLowerCase() != "inside")
                    this.updateConnectorLine(point, pointIndex, series, location, connectorHeight)
                var finalSize = { x: series.actualWidth, y: series.actualHeight };
                this.chartObj.graphics.addVisual(this.chartObj.polygon.createTextElement(this.chartObj.vector.vector3D(location.X, location.Y, -1), element, 0, -size.height));
            }        
        },
        updateConnectorLine: function (point, pointIndex, series, location, connectorHeight) {
            var drawPoints = [];
            var symbolLocation = point.symbolLocation;
            var x = symbolLocation.center.x + Math.cos(symbolLocation.angle) * symbolLocation.radius;
            var y = symbolLocation.center.y + Math.sin(symbolLocation.angle) * symbolLocation.radius;
            drawPoints.push({ x: x, y: y });
            var labelRadiusFromOrigin = symbolLocation.radius + connectorHeight;
            x = symbolLocation.center.x + (Math.cos((symbolLocation.angle)) * (labelRadiusFromOrigin));
            y = symbolLocation.center.y + (Math.sin((symbolLocation.angle)) * (labelRadiusFromOrigin));
           
            drawPoints.push({ x: x, y: y });
            //var hipen = height / 5;
            //x += (x > symbolLocation.center.x) ? hipen : -hipen;
            //drawPoints.push({ x: x, y: y });
            if (series.marker.dataLabel.connectorLine.type =="bezier")
                drawPoints = this.getBezierApproximation(drawPoints, 256, series);
            this.drawLineSegment(drawPoints, pointIndex, series)
        },
        /// <returns></returns>
        getBezierApproximation: function (controlPoints, outputSegmentCount, series)
        {
            var points = [];
             for (var i = 0; i <= outputSegmentCount; i++)
             {
               var t =i / outputSegmentCount;
               points.push(this.GetBezierPoint(t, controlPoints, 0, controlPoints.length, series));
             }
            return points;
         },
        drawLineSegment: function (drawpoints, pointIndex, series) {
            var vectorPoints = [];
            for (var i = 0; i < drawpoints.length; i++) {
                vectorPoints.push(this.chartObj.vector.vector3D(drawpoints[i].x, drawpoints[i].y, drawpoints[i].z ? drawpoints[i].z:0));
            }
            var seriesIndex=$.inArray(series, this.chartObj.model._visibleSeries);
            var color = (this.chartObj.model.AreaType=="none") ? this.chartObj.model.pointColors[pointIndex] : this.chartObj.model.seriesColors[seriesIndex];
            var stroke = (series.marker.dataLabel.connectorLine.stroke)?series.marker.dataLabel.connectorLine.stroke:color;
            var line = { width: series.marker.dataLabel.connectorLine.width, stroke: stroke, child: this.chartObj.chart3D, tag: 'polyline', id:this.chartObj.svgObject.id + "_" + seriesIndex + "_" + pointIndex };
          
            this.chartObj.graphics.addVisual(this.chartObj.polygon.createPolyline(vectorPoints, line));
        }

    },

     ej.ej3DDoughnutSeries = {

         draw: function (series, sender) {
             var MARGINS_RATIO = 0.03;
             sender._pieVisiblePoints(series);
              sender.calculateSize(sender, series);
              series.radius = (((1 - MARGINS_RATIO) * Math.min(series.actualWidth / 2, series.actualHeight / 2))  * 0.8);
              series.InsideRadius = (series.radius * series.doughnutCoefficient);
              sender.createPolygons(series, sender);
         },
         doAnimation: function (series, sender) {
             var seriesRendering = this;
             series.count = 0;
             $.each(series.segments, function (pointIndex, point) {

                 var radius = point.radius;
                 var insideRadius = point.inSideRadius;

                 series.animate = true;
                 $(sender.chartObj.element).each(function () { point.radius = 0, point.inSideRadius=0 }).animate(
                     { Radius: radius, InsideRadius: insideRadius },

                     {
                         duration: 1000, queue: false, step: function (now, fx) {


                             if (fx.prop.toString() === "Radius") {
                                 point.radius = now;
                             }
                             else {
                                 point.inSideRadius = now;
                             }


                             var poligons = sender.createPolygons(series, sender, true);

                             for (var ai = 0; ai < poligons.length; ai++) {
                                 for (var k = 0; k < poligons[ai].length; k++) {                                     
                                     sender.chartObj.polygon.update(poligons[ai][k].VectorPoints, poligons[ai][k], sender)
                                 }
                             }
                            
                         },
                         complete: function () {
                             sender.chartObj.model.AnimationComplete = true;
                             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonEventArgs.data = { series: series };
                             sender.chartObj._trigger("animationComplete", commonEventArgs);

                         }
                     });
             });
         }
     },

      ej.series3DTypes.doughnut = ej.ej3DDoughnutSeries;

    ej.ej3DPieSeries = {

        draw: function (series, sender) {
            sender._pieVisiblePoints(series);
            series.InsideRadius = null;
            series.radius = null;
            sender.createPolygons(series, sender);
        },
        doAnimation: function (series, sender) {
            var seriesRendering = this;
            series.count = 0;           
            $.each(series.segments, function (pointIndex, point) {
               
                var radius = point.radius;
              
                series.animate = true;
                $(sender.chartObj.element).each(function () { point.radius = 0  }).animate(
                    {  Radius: radius },

                    {
                        duration: 800, queue: false, step: function (now, fx) {

                            
                            if (fx.prop.toString() === "Radius") {
                                point.radius = now;
                            }
                           
                                                    
                            var poligons = sender.createPolygons(series, sender, true);

                            for (var ai = 0; ai < poligons.length; ai++) {
                                for (var k = 0; k < poligons[ai].length; k++) {
                                    //sender.chartObj.graphics.addVisual(poligons[ai][k]);
                                    sender.chartObj.polygon.update(poligons[ai][k].VectorPoints, poligons[ai][k], sender)
                                }
                            }
                            //sender.update()
                            //  var size = { Width: $(sender.chartObj.svgObject).width(), Height: $(sender.chartObj.svgObject).height() };
                            //  sender.chartObj.graphics.prepareView(sender.chartObj.model.perspectiveAngle, sender.chartObj.model.depth, sender.chartObj.model.rotation, sender.chartObj.model.tilt, size);
                            //  sender.chartObj.graphics.view(sender.chartObj.svgObject, sender.chartObj);
                           

                        },
                        complete: function () {                         
                         sender.chartObj.model.AnimationComplete = true;
                         var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                         commonEventArgs.data = { series: series };
                         sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                      }
                    });
            });
        }

    }

    ej.series3DTypes.pie = ej.ej3DPieSeries;

    ej.ej3DStackingColumn = {

        draw: function (series, sender) {
            var visiblePoints = sender._isVisiblePoints(series);
            this.createSegments(series, sender);
            for (var i = 0; i < visiblePoints.length; i++) {
                visiblePoints[i].plans = null;
                ej.ej3DColumnSeries.update(series, visiblePoints[i], i, sender)
            }
        },

        createSegments: function (series, sender) {
            var xValues = sender.chartObj._getXValues(series);
          
            var median;
            if (xValues == null) return;
            
            var sbsInfo = sender.getSideBySideInfo(series);
            var depthInfo = sender._getSegmentDepth(series);           
            median = sbsInfo.Delta / 2;
            var visiblePoints = sender._isVisiblePoints(series);
            for (var i = 0; i < visiblePoints.length; i++) {

                var x1 = xValues[i] + sbsInfo.Start;
                var x2 = xValues[i] + sbsInfo.End;
                var y2 = series.stackedValue.StartValues[i];
                var y1 = series.stackedValue.EndValues[i];

                ej.ej3DColumnSeries._setData(x1, y1, x2, y2, depthInfo.Start, depthInfo.End, sender, visiblePoints[i]);

                if (!series.marker.dataLabel.visible) continue;
                visiblePoints[i].symbolLocation = { x: 0, y: 0, z: 0 };
                switch (series.marker.dataLabel.textPosition) {
                    case "top":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.Delta / 2);
                        break;
                    case "bottom":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.End - depthInfo.Start) / 2;
                        break;
                    default:
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1 + (y2 - y1) / 2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start;
                        break;
                }

            }

        },

        doAnimation: function (series, sender) {            

                var seriesRender = this;

                $.each(series.points, function (pointIndex, point) {

                    var topValue = point.Top;
                    var bottomValue = point.Bottom;

                    $(point).each(function () { point.Top = 0, point.Bottom = 0}).animate(
                    { Top: topValue, Bottom: bottomValue },

                    {
                        duration: 1200, queue: false, step: function (now, fx) {

                            if (fx.prop.toString() === "Top") {
                                point.Top = now;
                            }
                            if (fx.prop.toString() === "Bottom") {
                                point.Bottom = now;
                            }

                            ej.ej3DColumnSeries.update(series, point, pointIndex, sender);

                        },
                        
                         complete: function () {                  
                       
                                sender.chartObj.model.AnimationComplete = true;
                                var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                                commonEventArgs.data = { series: series };
                                sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                            }

                    });

                });

           }

        
        
    },

    ej.series3DTypes.stackingcolumn = ej.ej3DStackingColumn;

    ej.series3DTypes.stackingbar = ej.ej3DStackingColumn;
     
	ej.series3DTypes.stackingbar100 = ej.ej3DStackingColumn;
	 
	ej.series3DTypes.stackingcolumn100 = ej.ej3DStackingColumn;
   
    ej.ej3DColumnSeries = {
        draw: function (series, sender) {
            var visiblePoints = sender._isVisiblePoints(series);
            this.createSegments(series, sender);
            for (var i = 0; i < visiblePoints.length; i++) {
                if (visiblePoints[i].visible) {
                    visiblePoints[i].plans = null;
                    this.update(series, visiblePoints[i], i, sender);
                }
            }
        },

        update: function (series, point, pointIndex, sender) {
            var plans;
            var valueType = series.xAxis._valueType.toLowerCase();
            var seriesIndex = $.inArray(series, sender.chartObj.model._visibleSeries);    
            var xBase = (valueType == "logarithmic") ? null : 1;
            var xIsLogarithmic = (valueType == "logarithmic") ? true :false;
            var left = xIsLogarithmic ? Math.log(point.Left, xBase) : point.Left;
            var right = xIsLogarithmic ? Math.log(point.Right, xBase) : point.Right;
            var bottom = series.yAxis.visibleRange.min;
            var top = series.yAxis.visibleRange.max;
            var xStart = series.xAxis.visibleRange.min;
            var xEnd = series.xAxis.visibleRange.max;
            if ((!(left >= xStart) || !(left <= xEnd)) && (!(right >= xStart) || !(right <= xEnd))) return;
           
            var topValue;
            if(point.Top<0)
                topValue = point.Top > bottom ? point.Top : bottom;
            else
                topValue = (series.yAxis.valueType && series.yAxis.valueType.toLowerCase() == "logarithmic") ?  point.Top : point.Top < top ? point.Top : top;
            var tlpoint = ej.EjSvgRender.utils.Transform3DToVisible(series, point.Left > xStart ? point.Left : xStart, topValue, sender.chartObj);
            var rbpoint = ej.EjSvgRender.utils.Transform3DToVisible(series, xEnd > point.Right ? point.Right : xEnd, bottom > point.Bottom ? bottom : point.Bottom, sender.chartObj);
            
            var tlfVector = sender.chartObj.vector.vector3D(Math.min(tlpoint.X, rbpoint.X), Math.min(tlpoint.Y, rbpoint.Y), point.StartDepth);
            var brbVector = sender.chartObj.vector.vector3D(Math.max(tlpoint.X, rbpoint.X), Math.max(tlpoint.Y, rbpoint.Y), point.EndDepth);
           
            var styleOptions = sender.chartObj.setStyle(sender, series, seriesIndex, pointIndex);
            if (styleOptions.interior.indexOf("url") >= 0)
                styleOptions.interior = sender.chartObj.model.seriesColors[seriesIndex][0].color;
            var name = "Region" + "_Series_" + seriesIndex + "_Point_" + pointIndex

            if (!point.plans)
                point.plans = sender.chartObj.polygon.createBox(tlfVector, brbVector, this, pointIndex, null,
                    styleOptions.borderColor, styleOptions.interior, styleOptions.borderWidth, styleOptions.opacity, sender.chartObj.model.requireInvertedAxes, name, sender.chartObj.chart3D);
            else
                sender.chartObj.polygon.updateBox(point.plans, tlfVector, brbVector, sender);
        },

        _setData:function()
        {  
            var values=arguments;
            point = arguments[arguments.length - 1];
            sender = arguments[arguments.length - 2];
        
            point.Left = values[0];
            point.Bottom = values[3];
            point.Top = values[1];
            point.Right = values[2];
            point.StartDepth = values[4];
            point.EndDepth = values[5];
            point.XRange = sender.getDoubleRange(point.Left, point.Right);
            if (!isNaN(point.Top) && !isNaN(point.Bottom))
                point.YRange = sender.getDoubleRange(point.Top, point.Bottom);
        },

        createSegments:function(series, sender)
        {
            var xValues = sender.chartObj._getXValues(series);
            var YValues = sender.chartObj._getYValues(series);
            var median;
            if (xValues == null) return;
      
            var sbsInfo = sender.getSideBySideInfo(series);
            var depthInfo = sender._getSegmentDepth(series);
         
            median = sbsInfo.Delta / 2;
            var visiblePoints = sender._isVisiblePoints(series);
            for (var i = 0; i < visiblePoints.length; i++)
            {
                 
                var x1 = xValues[i] + sbsInfo.Start;
                var x2 = xValues[i] + sbsInfo.End;
                var y1 = YValues[i];
                var y2 = 0;
        
                this._setData(x1, y1, x2, y2, depthInfo.Start, depthInfo.End, sender, visiblePoints[i]);

                XData = xValues[i];
                YData = YValues[i];
                //Item = ActualData[i];
              
                if (!series.marker.dataLabel.visible) continue;
                visiblePoints[i].symbolLocation = {x:0, y:0, z:0};
                switch (series.marker.dataLabel.textPosition) {
                    case "top":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y1;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.Delta / 2);
                        break;
                    case "bottom":
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = y2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start + (depthInfo.End - depthInfo.Start) / 2;
                        break;
                    default:
                        visiblePoints[i].symbolLocation.x = x1 + median;
                        visiblePoints[i].symbolLocation.y = Math.abs(y2 - y1) / 2;
                        visiblePoints[i].symbolLocation.z = depthInfo.Start;
                        break;
                }
        
            }

    
        },

        doAnimation: function (series, sender) {

            var seriesRender = this;

            $.each(series.points, function (pointIndex, point) {

                var topValue = point.Top;
                $(point).each(function () { point.Top = 0 }).animate(
                { Top: topValue },

                {
                    duration: 1200, queue: false, step: function (now, fx) {

                        if (fx.prop.toString() === "Top") {
                            point.Top = now;
                        }                      

                        seriesRender.update(series, point, pointIndex, sender);

                    },
                    
                     complete: function () {                        
                            sender.chartObj.model.AnimationComplete = true;
                            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                            commonEventArgs.data = { series: series };
                            sender.chartObj._trigger("animationComplete", commonEventArgs);
                  
                        }

                });

            })

        }
     
    }

    ej.series3DTypes.column = ej.ej3DColumnSeries;

    //bar series
    ej.series3DTypes.bar = ej.ej3DColumnSeries;

    ej.Ej3DChart.prototype = {
        renderSeries: function (sender, series) {            

            var series;
            ej.Ej3DSeriesRender.prototype.chartObj = sender;
         
            $.extend(ej.Ej3DSeriesRender.prototype, ej.EjSeriesRender.prototype);
            
            var seriesRendering = new ej.Ej3DSeriesRender();
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = {"series":series};
            seriesRendering.chartObj._trigger("seriesRendering", commonEventArgs);
                var type = series.type.toLowerCase();
                var options = ej.series3DTypes[type];
				 if (ej.util.isNullOrUndefined(options)) return;
                options.draw(series, seriesRendering);
                seriesRendering._pieVisiblePoints(series);
                if (series.marker.dataLabel.visible) {
                    $.each(series.visiblePoints, function (pointIndex, point) {
                        if (point.visible && (ej.util.isNullOrUndefined(point.marker) || (point.marker.dataLabel && point.marker.dataLabel.visible)))
                            seriesRendering.draw3DDataLabel(series, pointIndex, point);
                    });
                }


               
 
        },
        update3DWall: function (sender) {
            if (sender.model.AreaType == "cartesianaxes") {
                $.extend(ej.Ej3DRender.prototype, this);
                this.updateBackWall(sender);
                var axis;
                for (var i = 0; i < sender.model._axes.length; i++) {
                    axis = sender.model._axes[i];
                    if (axis.orientation.toLowerCase() == "vertical") {
                        if (!axis.opposedPosition)
                            this.updateLeftWall(sender);
                        else
                            this.updateRightWall(sender);
                    }
                    else {
                        if (!axis.opposedPosition)
                            this.updateBottomWall(sender);
                        else
                            this.updateTopWall(sender);
                    }
                }
            }                  
        },

      updateTopWall:function(sender)
        {
        
            var offset = 0;
            var  areaBounds= sender.model.m_AreaBounds;
            if (sender.model.wallSize < areaBounds.Y)
                offset = areaBounds.Y - sender.model.wallSize;
           else
                offset = -(sender.model.wallSize - areaBounds.Y);
            var tlfVector = sender.vector.vector3D(areaBounds.X + areaBounds.Width, -sender.model.depth, areaBounds.Y - 0.1);
            var brbVector = sender.vector.vector3D(areaBounds.X, - 0.1, offset);
            topSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, this, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "TopWallBrush", sender.chart3D);
  
          for (var i = 0; i < topSideWallPlans.length; i++)
              sender.polygon.transform(sender.matrixobj.tilt((parseFloat)(Math.PI / 2)), topSideWallPlans[i]);
    
                  
    },

       updateRightWall:function(sender)
        {
           var rightRect = { left: -(sender.model.depth), top: sender.model.m_AreaBounds.Y, bottom: sender.model.m_AreaBounds.Height+sender.model.m_AreaBounds.Y, right: 0 };
           var tlfVector = sender.vector.vector3D(rightRect.left, rightRect.top, sender.model.m_AreaBounds.X+sender.model.m_AreaBounds.Width + 1.5);
           var brbVector = sender.vector.vector3D(rightRect.right, rightRect.bottom, sender.model.m_AreaBounds.X + sender.model.m_AreaBounds.Width + sender.model.wallSize);
           rightSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, sender, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "RightWallBrush", sender.chart3D);
           for (var i = 0; i < rightSideWallPlans.length; i++)
             sender.polygon.transform(sender.matrixobj.turn((parseFloat)(-Math.PI / 2)), rightSideWallPlans[i]);           
        },
        updateBackWall: function (sender) {
            var areaBounds = sender.model.m_AreaBounds;           
            var tlfVector = sender.vector.vector3D(areaBounds.X, areaBounds.Y, sender.model.depth == 0 ? 1.5 : sender.model.depth + sender.model.wallSize);
            var brbVector = sender.vector.vector3D((areaBounds.X + areaBounds.Width), areaBounds.Y + areaBounds.Height, sender.model.depth == 0 ? 1.5 : sender.model.depth);

            sender.polygon.createBox(tlfVector, brbVector, sender, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "BackWallBrush", sender.chart3D);
        },
        updateLeftWall: function (sender) {
            leftRect = { left: -(sender.model.depth), top: sender.model.m_AreaBounds.Y, bottom: sender.model.m_AreaBounds.Height+sender.model.m_AreaBounds.Y, right: 0 };
            offset = sender.model.m_AreaBounds.X;
            var tlfVector = sender.vector.vector3D(leftRect.left, leftRect.top, offset - 0.1);
            var brbVector = sender.vector.vector3D(leftRect.right, leftRect.bottom, offset - sender.model.wallSize);

            leftSideWallPlans = sender.polygon.createBox(tlfVector, brbVector, this, 0, "Graphics3D", "#e2e1e1", "#e2e1e1", 0, 0.15, false, "LeftWallBrush", sender.chart3D);
            for (var i = 0; i < leftSideWallPlans.length; i++)
                sender.polygon.transform(sender.matrixobj.turn((parseFloat)(-Math.PI / 2)), leftSideWallPlans[i]);
        },
        updateBottomWall: function (sender) {
            var areaBounds = sender.model.m_AreaBounds;
            var tlfVector = sender.vector.vector3D((areaBounds.X + areaBounds.Width), -(sender.model.depth), sender.model.wallSize + areaBounds.Y + areaBounds.Height);
            var brbVector = sender.vector.vector3D(areaBounds.X, -0.1, areaBounds.Y+areaBounds.Height + 1);

            bottomSideWallPlans = sender.polygon.createBox(brbVector, tlfVector, this, 0, "Graphics3D", "#D3D3D3", "#D3D3D3", 0, 0.15, false, "BottomWallBrush", sender.chart3D);
            for (var i = 0; i < bottomSideWallPlans.length; i++)
                sender.polygon.transform(sender.matrixobj.tilt((parseFloat)(Math.PI / 2)), bottomSideWallPlans[i]);
        }
    }

})(jQuery);
ej.EjVmlRender = function (element) {
   var doc = document;
    this.vmlNamespace = "urn:schemas-microsoft-com:vml";
    this.vmStyle = "#default#VML";
    doc.namespaces.add('v', 'urn:schemas-microsoft-com:vml');
    this.isdocmode8 = doc.documentMode === 8;
    if (!doc.getElementById("vml_chart")) {
        var cssText
        if (this.isdocmode8) {
            cssText =
                    'v\\:fill, v\\:path, v\\:polyline, v\\:line, v\\:rect,v\\:shape,v\\:oval, v\\:stroke' +
                    '{ behavior:url(#default#VML); display: inline-block; } ';
        }
        else {
            cssText =
               'v\\:*' +
               '{ behavior:url(#default#VML); display: inline-block; } ';
        }
        var vmlStyleSheet = doc.createStyleSheet();
        vmlStyleSheet.owningElement.id = 'vml_chart';
        vmlStyleSheet.cssText = cssText;
    }
    this._rootId = jQuery(element).attr("id");
    this.svgObj = doc.createElement("div");
    this.svgObj.style.position = 'relative';
    this.svgObj.setAttribute('id', this._rootId + '_vml');   
    this.changeOptions = {'id':'id',
        'fill': 'fillcolor', 'stroke': 'strokecolor', 'stroke-width': 'strokeweight', 'd': 'path', 'font-size': 'font-size', 'font-family': 'font-family', 'font-style': 'font-style',
        'font-weight': 'font-weight', 'points':'points'

        };
};
(function($) {
ej.EjVmlRender.prototype = {
  
    drawPath: function (options, element) {
        var darray= options.d.split(" ");
        if($.inArray('A', darray) != -1)
        {
            var w = parseFloat(darray[4]);
            var h = parseFloat(darray[5]);
            options.d=this.drawArc(w, h, options);
        }
        if ($("#" + options.id).length > 0) {
            options.d = this.changePathValue(options.d);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            options.d = this.changePathValue(options.d);
            var shape = document.createElement("v:shape");
            var $shapeEle = $(shape);
            this.applyVMLStyle($shapeEle, options);
            $shapeEle.appendTo(element);
        }
    },
    createLegendSvg:function(element)
    {
        var doc = document;        
        this._rootId = jQuery(element).attr("id");
        this.legendsvgObj = doc.createElement("div");
        this.legendsvgObj.style.position = 'relative';
        this.legendsvgObj.setAttribute('id',"legend_"+ this._rootId + '_vml');       
        return this.legendsvgObj;
    },

    drawPolyline: function (options, element) {
        var darray = options.points.split(" ");
        
        if ($("#" + options.id).length > 0) {
            options.points = this.changePathValue(options.points);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            options.points = this.changePathValue(options.points);
            var shape = document.createElement("v:polyline");
            var $shapeEle = $(shape);
            this.applyVMLStyle($shapeEle, options);
            $shapeEle.appendTo(element);
        }
    },
    setFillAttribute: function (element, options) {
        var fill = document.createElement("v:fill");
        var opacity = options["fill-opacity"] ? options["fill-opacity"] : options["opacity"];
        opacity = (!ej.util.isNullOrUndefined(opacity)) ? opacity : (options["opacity"]) ? options["opacity"] : 1;
        var fillcolor = options["fill"];
        if (fillcolor != "none" && !ej.util.isNullOrUndefined(fillcolor)) {
          
            if (!(fillcolor).match('gradient'))
                $(element).attr("fillcolor", fillcolor);
            else {
                var colorName = ((fillcolor).replace('gradient', ""));
                fill.setAttribute('type', "gradient");
                fill.setAttribute("colors", colorName);
            }
        }
        else 
            opacity = 0.1;
        
        
        fill.setAttribute('opacity', opacity);
        $(fill).appendTo(element);
    },
    setStrokeAttribute: function (element, options) {
        var stroke = options['stroke'];
        if (stroke == "transparent") 
            options['opacity'] = 0.1;
        if (options.name) {
            $(element).attr('name', options.name);
        }
         
         
        var opacity = options["opacity"];
        if (opacity) {
            var fill = document.createElement("v:stroke");

           

            fill.setAttribute('opacity', opacity);
            $(fill).appendTo(element);
        }
    },
    changePathValue: function (options) {
        if (!options["d"]) {
            var pathArray = options.split(" ");
            for (var i = 0; i < pathArray.length; i++) {
                var matches = pathArray[i].match(/\d+/g);
                if (matches != null) {
                    pathArray[i] = Math.round(parseFloat(pathArray[i]));
                }
            }
            options = pathArray.join(' ');
            return options;
        }
    },


    drawArc: function ( w, h, options) {
    var x = 0, y = 0;
    var startAngle = options.start,
    endAngle =options.end,
    endAngle = (Number(Math.PI.toFixed(2)) == Number((endAngle + startAngle).toFixed(2))) ? endAngle - 0.01 : endAngle,
    radius = options.r || w || h,
    cosStart = Math.cos(startAngle),
    sinStart = Math.sin(startAngle),
    cosEnd = Math.cos(endAngle),
    sinEnd = Math.sin(endAngle),
    innerRadius = options.innerR,
    circleCorrection = 0.08 / radius, 
    innerCorrection = (innerRadius && 0.1 / innerRadius) || 0,
    path;

    if (endAngle - startAngle === 0) {
            return '';

    } else if (2 * Math.PI - endAngle + startAngle < circleCorrection) {
            cosEnd = -circleCorrection;
    } else if (endAngle - startAngle < innerCorrection) {
            cosEnd = Math.cos(startAngle + innerCorrection);
        }

        path = [
    'wa', // To draw clockwise arc
    x - radius, 
    y - radius, 
    x + radius, 
    y + radius, 
    x + radius * cosStart, 
    y + radius * sinStart, 
    x + radius * cosEnd, 
    y + radius * sinEnd  
        ];

        if (options.open && !innerRadius) {
            path.push(
        'e',
        M,
        x, 
        y
    );
        }

        path.push(
            // To draw anti clockwise arc
    'at', 
    x - innerRadius, 
    y - innerRadius, 
    x + innerRadius, 
    y + innerRadius, 
    x + innerRadius * cosEnd, 
    y + innerRadius * sinEnd, 
    x + innerRadius * cosStart, 
    y + innerRadius * sinStart, 
    'x', 
    'e' 
);
        path=path.join(" ");
        return path;

  },
    changeVMLStyle: function ($element, options) {
        var chart = this;
		var strokeColor = options.stroke;
        $element.css("width", options.width).css("visibility", options.visibility).css("height", options.height).css("position", "absolute").css('left', options.x).css('top', options.y);
        $.each(options, function (index, val) {
            if (chart.changeOptions[index] != null || chart.changeOptions[index] != undefined) {
                if (index == 'd' || index=='points') {
                    val = chart.changePathValue(options[index]).toLowerCase();
                }
                if (val == 'transparent') {
                    options[index] = val = 'white';
                     if(index=="fill" && (!options.hasOwnProperty('opacity') ||  !options['opacity'])) 
                       options['opacity'] = 0.1;                  
                }
                if ($element[0].tagName === "SPAN") {
                    if (index == "fill") $element.css('color', val);
                    else {
                        if (index == "id") $element.attr(chart.changeOptions[index], val);
                        else
                            $element.css(chart.changeOptions[index], val);
                    }
                }else {

                    if (chart.isdocmode8) { // IE8 setAttribute bug
                        $element.get(0)[chart.changeOptions[index]] = val;
                    } else {
                        $element.get(0).setAttribute(chart.changeOptions[index], val);
                    }      
                }
            }
        });
        if (options.hasOwnProperty('fill-opacity') || options.hasOwnProperty('opacity') || options.hasOwnProperty('fill'))
            if (!($element[0].tagName === "SPAN")) {
                chart.setFillAttribute($element, options);
               if (options.hasOwnProperty('stroke')) {
                    options.stroke = strokeColor;
                    chart.setStrokeAttribute($element, options);
                }
            }
    },
    applyVMLStyle: function ($element, options) {
        
        if ($element[0].tagName !== "SPAN") {
            if (ej.util.isNullOrUndefined(options.width)) options.width = "1000px";
            if (ej.util.isNullOrUndefined(options.height)) options.height = "1000px";
        }
        if (ej.util.isNullOrUndefined(options.x)) options.x = "0px";
        if (ej.util.isNullOrUndefined(options.y)) options.y = "0px";

        this.changeVMLStyle($element, options);
    },
    drawLine: function (options, element) {
        if ($("#" + options.id).length > 0) {
            var from = (options.x1) + ',' + (options.y1);
            var to = (options.x2) + ',' + (options.y2);
            $(options).attr("from", from).attr("to", to);
            this.applyVMLStyle($("#" + options.id), options);
        }
        else {
            var line = document.createElement("v:line");
            var $lineEle = $(line);
            var from = (options.x1) + ',' + (options.y1);
            var to = (options.x2) + ',' + (options.y2);
            $lineEle.attr("from", from).attr("to", to);
            this.applyVMLStyle($lineEle, options);
            $lineEle.appendTo(element);
        }
     },
    //drawPolygon: function (options, element) {
     
    //},
     drawCircle: function (options, element) {
         if ($("#" + options.id).length > 0) {
             options.x = options.cx - options.r;
             options.y = options.cy - options.r;
             options.width = options.height = (2 * options.r);
             this.applyVMLStyle($("#" + options.id), options);
         }
         else {
             var circle = document.createElement("v:oval");
             var $circleEle = $(circle);
             options.x = options.cx - options.r;
             options.y = options.cy - options.r;
             options.width = options.height = (2 * options.r);
             this.applyVMLStyle($circleEle, options);
             if ($("#" + options.id).length == 0)
             $circleEle.appendTo(element);
         }
     },
    
   drawEllipse: function (options, element) {
       if ($("#" + options.id).length > 0) {
           options.x = options.cx - options.rx;
           options.y = options.cy - options.ry;
           options.width = (2 * options.rx);
           options.height = (2 * options.ry);
           this.applyVMLStyle($("#" + options.id), options);
       }
       else {
           var ellipse = document.createElement("v:oval");
           var $ellipseEle = $(ellipse);
           options.x = options.cx - options.rx;
           options.y = options.cy - options.ry;
           options.width = (2 * options.rx);
           options.height = (2 * options.ry);
           this.applyVMLStyle($ellipseEle, options);
           $ellipseEle.appendTo(element);
       }
    },
    drawRect: function (options, element) {
        if ($("#" + options.id).length > 0) {
            this.applyVMLStyle($("#" + options.id), options);
        } else {
            var rect = document.createElement("v:rect");
            var $rectEle = $(rect);
            this.applyVMLStyle($rectEle, options);
            $rectEle.appendTo(element);
        }
    },
    _getAttrVal: function (ele, val, option) {
          val = (val == "x") ? "left" : val;
        val = (val == "y") ? "top" : val;
        var value1 = $(ele).attr(val);
        var value = value1 ? value1 : ($(ele).css(val));
        
        if (value != null)
            return value;
        else
            return option;
    },
    _setAttr: function (element, attribute) { 
        if (attribute.transform) {
		  var attrTrans=attribute.transform;
            var trans = attrTrans.slice(attrTrans.indexOf('(') + 1, attrTrans.indexOf(')')).split(',');
            attribute.x = trans[0];
            attribute.y = trans[1];
        }
        var $ele = $(element);
        if($ele.length>0)
            this.changeVMLStyle($ele, attribute);
    },
    createGradientElement: function (name, colors) {
        var colorName;
       
        if (Object.prototype.toString.call(colors) == '[object Array]') {
            colorName = "gradient" + " ";
            var appendString = "";
            for (var j = 0, i=colors.length-1 ; i>=0, j < colors.length ; j++,i--){
                colorName += appendString + colors[j]["colorStop"] + " " + colors[i]["color"];
                appendString = ",";
            }
        }
        else {
            colorName = colors;
        }
        return colorName; 
    },

    //drawGradient: function (options, gradientEle, element) {

      
    //},
    setSpanAttr: function (options, label, element) {
	    var fontSize = (options["font-size"] > 0) ? (options["font-size"] + "px") : options["font-size"];
        var font = { size: fontSize, fontStyle: options["font-style"], fontFamily: options["font-weight"] };
        var bounds = ej.EjSvgRender.utils._measureText(label, null, font);
        
            $(element).css("white-space", "nowrap");
        
        if (options["text-anchor"] === "middle")
            options.x = Math.round(options.x - bounds.width / 2);
        else if (options["text-anchor"] === "end")
            options.x = Math.round(options.x - bounds.width);
        options.y = Math.round(options.y -( font.size.replace("px","")));
        this.applyVMLStyle(element, options);
        
       
    },
    drawText: function (options, label, groupEle,font) {
        var textDoc = document;
        var $Ele = $("#" + options.id);
        if ($Ele.length > 0) {
            this._textAttrReplace(options, label, font, $Ele);
        }
        else if (options.id.indexOf("YAxisTitle") > 0 && !this.isdocmode8) {
            options.on = true;
            var line = document.createElement("v:line");
            line.style.behavior = "url(#default#VML)";
            line.style.display = "inline-block";
            var bounds = ej.EjSvgRender.utils._measureText(label, null, null);
             
            line.setAttribute("to", (options.x - 1).toString() + "," + (options.y - bounds.width / 2).toString());
            line.setAttribute("from", (options.x).toString() + "," + options.y.toString());
           
            var path =  document.createElement("v:path");
            path.setAttribute("textpathok", true);
            var textpath =  document.createElement("v:textpath");
            textpath.setAttribute("on", true);
            textpath.setAttribute("string", label);
       
          textpath.style["font-style"] = options["font-style"];
          textpath.style["font-size"] = options["font-size"];
          textpath.style["font-family"] = options["font-family"];         
          line.appendChild(path);
          line.appendChild(textpath);           
          $(line).appendTo(groupEle);
        }
        else {
            var span = textDoc.createElement("span");
            var $spanEle = $(span);
            if (jQuery.type(label) == "array") {
                this.setSpanAttr(options, label, $spanEle);
                for (var i = 0; i < label.length; i++) {
                    var textspan = textDoc.createElement("span");
                    $(textspan).html(label[i]);                
                    $(textspan).appendTo(span);
                    var brtag = textDoc.createElement("br");
                    $(brtag).appendTo(span);
                    
                }
            } else {
                this.setSpanAttr(options, label, $spanEle);
                $spanEle.html(label);
            }         
            $spanEle.appendTo(groupEle);          
        }
    },
    //text element attributes values are replaced 
    _textAttrReplace: function (options, label, font, $Ele) {
        if (jQuery.type(label) == "array") {
            var elements = $("#" + options.id).children("span");
            this.setSpanAttr(options, label, $Ele);
            if (elements.length > 0 && elements.length == label.length) {
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    $(element).text(label[i]);
                }
            }
            else {
                $("#" + options.id).remove();
                this.drawText(options, label, this.gTransToolEle, font);

            }
        } else {
            this.setSpanAttr(options, label, $Ele);
            $Ele.text(label);
        }

    },

   drawImage: function (options, element) {
        var imgElement = document.createElement("img");
        var $imgElement = $(imgElement);
          $imgElement.css({ 'position': 'absolute' });
          $.each(options, function (index, val) {
              if (index == "href") index = "src";
                $imgElement.attr(index, val);
        });
        $imgElement.appendTo(element);
       
    },

    createDefs: function () {
        
    },

    //createClipPath: function (options) {
        
    //},
    //createForeignObject: function (options) {
      
    //},
    createGroup: function (options) {
        var group = document.createElement("div");
        if (!ej.util.isNullOrUndefined(options.transform)) {
            var trans = options.transform.slice(options.transform.indexOf('(') + 1, options.transform.indexOf(')')).split(',');
            $(group).css({ 'position': 'absolute', 'left': trans[0] + 'px', 'top': trans[1] + 'px' });
        }
		 if (options.cursor) 
            $(group).css({ 'cursor': options.cursor });
        
        $.each(options, function(index, val) {
            if (index !== 'transform' && index !="clip-path")
                $(group).attr(index, val);
        });
      
        return group;
    },

    createText: function (options, label) {
        var text = document.createElement("span");
        $(text).attr(options);
        if (!ej.util.isNullOrUndefined(label))
            text.textContent = label;
        return text;
        
        
    },   
   
    //Draw clip path for each series to avoid series overlap in multiple axes zooming
    drawAxesBoundsClipPath: function (gSeriesGroupEle, options, invertedAxis) {
        var clipOptions;
        var element = $(gSeriesGroupEle);
        element.removeAttr("clip-path");
        var trans = this._getTransform(options.xAxis, options.yAxis, invertedAxis);
        clipOptions = {
            'id': gSeriesGroupEle.id + '_ClipRect',
            'x': 0,
            'y': 0,
            'width': (trans.width),
            'height': (trans.height),
            'fill': 'white',
            'stroke-width': 1,
            'stroke': 'transparent'
        };
        this.drawClipPath(clipOptions, gSeriesGroupEle);
        element.attr('clip-path', 'url(#' + clipOptions.id + ')');

    },
   
    getAttrVal: function (ele, val, option) {
        var value = $(ele).attr(val);
        if (value != null)
            return value;
        else
            return option;
    },
   
    round: function (value, div, up) {

        return div * (up ? Math.ceil(value / div) : Math.floor(value / div));
    },
    
   
   
    
    hexFromRGB: function (color) {
        var r = color.R;
        var g = color.G;
        var b = color.B;
        var hex = [r.toString(16), g.toString(16), b.toString(16)];
        $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
        return "#" + hex.join("").toUpperCase();
    },
    hexToRGB: function (hex) {
        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
        return result ? {
            R: parseInt(result[1], 16),
            G: parseInt(result[2], 16),
            B: parseInt(result[3], 16)
        } : null;
    },
    checkColorFormat: function (color) {
        return /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.test(color);
    },
    createDelegate: function (context, handler) {
        return function (e) {
            handler.apply(context, [e, this]);
        };
    },

    drawClipPath: function (options, element) {
        if (options.id.indexOf("ChartAreaClipRect") == -1) {
            var left = Math.round(options.x)+'px';
            var top = Math.round(options.y)+'px';
            var right = Math.round(options.width)+'px';
            var bottom = Math.round(options.height) + 'px';
            var cssClip = 'rect(' + top + ' ' + right + ' ' + bottom + ' ' +left + ')';
            $(element).css("position", "absolute").css('clip',cssClip);
        }
    },

    drawCircularClipPath: function (options, element) {
        var radius = Math.round(options.r);
        var x = Math.round(options.cx);
        var y = Math.round(options.cy);
        if (options.id.indexOf("_ClipRect") == -1) {
            var left = (x - radius) + 'px';
            var top = (y - radius) + 'px';
            var right = (x + radius) + 'px';
            var bottom = (y + radius) + 'px';
            var cssClip = 'rect(' + top + ' ' + right + ' ' + bottom + ' ' + left + ')';
            $(element).css("position", "absolute").css('clip', cssClip);
        }
    },
   
    append: function (childEle, parentEle) {
        $(childEle).appendTo(parentEle);
    }
};
})(jQuery);;
ej.EjCanvasRender = function (element) {
    this.svgObj = document.createElement('canvas');
    this._rootId = jQuery(element).attr("id");
    this.svgObj.setAttribute('id', this._rootId + '_canvas');
    this.ctx = this.svgObj.getContext("2d");
    
};
(function ($) {
    ej.EjCanvasRender.prototype = {

        drawPath: function (options, element, canvasTranslate) {

            var path = options.d;
            var dataSplit = path.split(" ");
            var borderWidth = options["stroke-width"];
            if (!options.lgndCtx) {
                var canvasCtx = this.ctx;
                this.ctx = element != undefined ? element : this.ctx;
                this.ctx.save();
                this.ctx.beginPath();
                if (canvasTranslate) this.ctx.translate(canvasTranslate[0], canvasTranslate[1]);
                this.ctx.globalAlpha = options.opacity ? options.opacity : options["fill-opacity"];
                var flag = true;
                this.ctx.lineWidth = borderWidth;
				var dashArray = options["stroke-dasharray"] ? options["stroke-dasharray"].split(",") : false;
                if (dashArray) this.ctx.setLineDash(dashArray);
                this.ctx.strokeStyle = options.stroke;
                for (var i = 0; i < dataSplit.length; i = i + 3) {
                    var x1 = parseFloat(dataSplit[i + 1]);
                    var y1 = parseFloat(dataSplit[i + 2]);
                    switch (dataSplit[i]) {
                        case "M":
                            if (!options.innerR && !options.cx)
                                this.ctx.moveTo(x1, y1);
                            break;
                        case "L":
                            if (!options.innerR)
                                this.ctx.lineTo(x1, y1);
                            break;
                        case "C":
                            this.ctx.bezierCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]), parseFloat(dataSplit[i + 5]), parseFloat(dataSplit[i + 6]));
                            i = i + 4;
                            break;
                        case "A":
                            if (!options.innerR) {
							    if (options.cx) {
                                    this.ctx.arc(options.cx, options.cy, options.radius, 0, 2 * Math.PI, options.counterClockWise);
                               }
								else{
                                    this.ctx.moveTo(options.x, options.y);
                                    this.ctx.arc(options.x, options.y, options.radius, options.start, options.end, options.counterClockWise);
                                    this.ctx.lineTo(options.x, options.y);
								}
                            }
                            else if (flag) {
                                this.ctx.arc(options.x, options.y, options.radius, options.start, options.end, options.counterClockWise);
                                this.ctx.arc(options.x, options.y, options.innerR, options.end, options.start, !options.counterClockWise);
                                flag = false; 
                            }
                            i = i + 5;
                            break;
					    case "z":
                            this.ctx.closePath();
                            break;
                    }
                }
                if (options.fill != "none" && options.fill != undefined) {
                    this.ctx.fillStyle = options.fill;
                    this.ctx.fill();
                }
                if (borderWidth > 0)
                    this.ctx.stroke();
                this.ctx.restore();
                this.ctx = canvasCtx;
            } else {
                var legendCtx = this.lgndCtx;
                this.lgndCtx = element != undefined ? element : this.lgndCtx;
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                if (canvasTranslate) this.lgndCtx.translate(canvasTranslate[0], canvasTranslate[1]);
                this.lgndCtx.globalAlpha = options.opacity ? options.opacity : options["fill-opacity"];

                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                for (var i = 0; i < dataSplit.length; i = i + 3) {
                    var x1 = parseFloat(dataSplit[i + 1]);
                    var y1 = parseFloat(dataSplit[i + 2]);
                    switch (dataSplit[i]) {
                        case "M":
                            this.lgndCtx.moveTo(x1, y1);
                            break;
                        case "L":
                            this.lgndCtx.lineTo(x1, y1);
                            break;
                        case "C":
                            this.lgndCtx.bezierCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]), parseFloat(dataSplit[i + 5]), parseFloat(dataSplit[i + 6]));
                            i = i + 4;
                            break;
                        case "A":
                            this.lgndCtx.arc(parseFloat(dataSplit[i - 5]), parseFloat(dataSplit[i - 4]), x1, 0, 2 * Math.PI, false);
                            i = i + 5;
                            break;
							case "a":
                            this.lgndCtx.beginPath();
                            var centerX = parseFloat(dataSplit[i - 2]) + x1;
                            var centerY = dataSplit[i - 1];
                            this.lgndCtx.arc(centerX, centerY, x1, 0, 2 * Math.PI, false);
                            i = i + 5;
                            break;
                        case "Z":
                            this.lgndCtx.fillStyle = options.fill;
                            this.lgndCtx.fill();
                            i = i - 2;
                            break;
					    case "Q":
                            this.lgndCtx.quadraticCurveTo(x1, y1, parseFloat(dataSplit[i + 3]), parseFloat(dataSplit[i + 4]));
                            i = i + 2;
                            break;
                    }
                }
                if (options.fill != "none" && options.fill != undefined) {
                    this.lgndCtx.fillStyle = options.fill;
                    this.lgndCtx.fill();
                }
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
                this.lgndCtx = legendCtx;
            }
        },
        
       
        drawRect: function (options, element) {
            var canvasCtx = this.ctx;
            this.ctx = element ? (element.canvas ? element : this.ctx ) : this.ctx;
            this.ctx.save();
            this.ctx.beginPath();
            this.ctx.globalAlpha = options.opacity;
            this.ctx.lineWidth = options["stroke-width"];
            this.ctx.strokeStyle = options.stroke;
            this.ctx.rect(options.x, options.y, options.width, options.height);
            if (options.fill == "none") options.fill = "transparent";
            this.ctx.fillStyle = options.fill;
            this.ctx.fillRect(options.x, options.y, options.width, options.height);
            if(options.id.indexOf("Series") >= 0)
                this.ctx.clip();
			this.ctx.stroke();
            this.ctx.restore();
            this.ctx = canvasCtx;
        },
        createGradientElement: function (name, colors) {
            var colorName;
            var ctx = this.ctx;
            if (Object.prototype.toString.call(colors) == '[object Array]') {
                var my_gradient = ctx.createLinearGradient(0, 0, 0, this.svgObj.height);
             
                for (var i = 0; i <= colors.length - 1 ; i++) {
                    var color = colors[i].color;
                    var colorStop = (colors[i].colorStop).slice(0, -1);
                    colorStop = parseInt(colorStop) / 100;
                    my_gradient.addColorStop(colorStop, color);
                    
                }
                colorName = my_gradient;
            }
            else {
                colorName = colors;
            }
            return colorName;
        },
        drawLine: function (options) {
            this.ctx.save();
            this.ctx.beginPath();
			this.ctx.lineWidth = options["stroke-width"];
            this.ctx.strokeStyle = options.stroke;
            this.ctx.moveTo(options.x1, options.y1);
            this.ctx.lineTo(options.x2, options.y2);
            this.ctx.stroke();
            this.ctx.restore();
        },
        drawText: function (options, label) {
            if (!ej.util.isNullOrUndefined(options["font-weight"]) && options["font-weight"].toLowerCase() == "regular")
                options["font-weight"] = "normal";
            var font = (options["font-style"].toLowerCase() + " " + options["font-weight"] + " " + options["font-size"] + " " + options["font-family"]);
            // text-anchor
            var anchor = options["text-anchor"];
            var opacity = options["opacity"] !== undefined ? options["opacity"] : 1;
            if (anchor == "middle")
                anchor = "center";
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.fillStyle = options.fill;                
                this.ctx.font = font;                
                this.ctx.textAlign = anchor;
                this.ctx.globalAlpha = opacity;
                if (options.baseline)
                    this.ctx.textBaseline = options.baseline;
                // dominant-baseline
                //if (options["dominant-baseline"] == "middle")
                    //this.ctx.textBaseline = "center";
                if (!options.labelRotation) {
                    this.ctx.fillText(label, options.x, options.y);
                } else {
                    var txtlngth = 0;
                    if (options.labelRotation == 90 && options.id.indexOf("XLabel") != -1)
                        txtlngth = this.ctx.measureText(label).width;
                    
                    this.ctx.translate(options.x + (txtlngth/2), options.y);
                    this.ctx.rotate(options.labelRotation * Math.PI / 180);
                    this.ctx.fillText(label, 0, 0);
                }
                this.ctx.restore();
            } else {
                this.lgndCtx.save();
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.font = font;
                this.lgndCtx.textAlign = anchor;
                this.lgndCtx.globalAlpha = opacity;
                // dominant-baseline
                if (options["dominant-baseline"] == "middle")
                    this.lgndCtx.textBaseline = "center";
                if (!options.labelRotation) {
                    this.lgndCtx.fillText(label, options.x, options.y);
                } else {
                    this.lgndCtx.translate(options.x, options.y);
                    this.lgndCtx.rotate(options.labelRotation * Math.PI / 180);
                    this.lgndCtx.fillText(label, 0, 0);
                }
                this.lgndCtx.restore();
            }
        },
        drawZoomRect: function(options, chartObj) {

            var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };
            var zoomRect = $("<div id=" + options.id + "></div>");
            var areaBounds = chartObj.model.m_AreaBounds;
			
			// for drawing zoom area within the chartArea
            if (options.x < chartObj.canvasX || options.x + chartPos.left + options.width > chartPos.left + $(this.svgObj).width() - 20) {
                width = this.prevWidth;
                x = this.prevX;
            } else {
                width = this.prevWidth = options.width;
                x = this.prevX = options.x + chartPos.left;
            }
            
            if (options.y < chartObj.canvasY || options.y + chartPos.top + options.height  > chartPos.top + areaBounds.Y + areaBounds.Height) {
                height = this.prevHeight;
                y = this.prevY;
            } else {
                height = this.prevHeight = options.height;
                y = this.prevY = options.y + chartPos.top;
            }
            
            $(zoomRect).css({
                    "width": width,
                    "height": height,
                    "top": y,
                    "left": x,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-color": options.stroke,
                    "border-width": options["stroke-width"]
                });
                document.getElementById(this._rootId).appendChild(zoomRect[0]);
        },
        zoomButton: function (options) {
            var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };
            if ($("#" + options.zoomId).length <= 0) {
                var zoomButton = $("<div id=" + this.svgObj.id + '_ZoomBtn' + "></div>");
                $(zoomButton).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left - 20,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-radius": "0.4em",
                    "border-color": options.fill,
                    "box-sizing": "content-box"
                });                
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.zoomId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);
               
                var attrOptions = {
                    'id': 'zoomPath',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                attrOptions.d = "M26.101,22.893l-6.605-6.174c1.414-2.781,0.897-6.267-1.496-8.504c-2.901-2.711-7.448-2.56-10.161,0.341    c-2.712,2.9-2.56,7.45,0.341,10.163c2.426,2.266,6,2.523,8.694,0.853l6.579,6.151L26.101,22.893z M10.186,16.571    c-1.715-1.604-1.805-4.293-0.203-6.009c1.605-1.715,4.295-1.805,6.009-0.201c1.715,1.603,1.805,4.293,0.202,6.007    C14.59,18.084,11.901,18.175,10.186,16.571";

                var path = document.createElementNS(svgLink, "path");
                $(path).attr(attrOptions).appendTo(svgShape);
                zoomButton[0].appendChild(svgShape);
                document.getElementById(this._rootId).appendChild(zoomButton[0]);
            } else {
                var zoomButton = document.getElementById(this.svgObj.id + '_ZoomBtn');
                $(zoomButton).css({
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left - 20
                });
            }
        },
        panButton: function (options) {
            var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };
            if ($("#" + options.panId).length <= 0) {
                var panButton = $("<div id=" + this.svgObj.id + '_PanBtn' + "></div>");
                $(panButton).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left - 10,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "position": "absolute",
                    "border-radius": "0.4em",
                    "border-color": options.fill,
                    "box-sizing": "content-box"
                });                
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.panId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);

                var attrOptions = {
                    'id': 'panPath',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                attrOptions.points = "26.105,16 21.053,12.211 21.053,14.737 17.263,14.737 17.263,10.947 19.834,10.947 16.044,5.895 12.255,10.947 14.737,10.947 14.737,14.737 10.947,14.737 10.947,12.211 5.895,16 10.947,19.789 10.947,17.263 14.737,17.263 14.737,21.053 12.255,21.053 16.044,26.105 19.834,21.053 17.263,21.053 17.263,17.263 21.053,17.263 21.053,19.789";

                var path = document.createElementNS(svgLink, "polygon");
                $(path).attr(attrOptions).appendTo(svgShape);
                panButton[0].appendChild(svgShape);
                document.getElementById(this._rootId).appendChild(panButton[0]);     
            } else {
                var panButton = document.getElementById(this.svgObj.id + '_PanBtn');
                $(panButton).css({
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left - 10
                });
            }
        },
        resetZoom: function (options) {
            var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };
            if ($("#" + options.resetZoomId).length <= 0) {
                var resetZoom = $("<div id=" + this.svgObj.id + '_ResetZoom' + "></div>");
                $(resetZoom).css({
                    "width": options.width,
                    "height": options.height,
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left,
                    "background-color": options.fill,
                    "border-style": "solid",
                    "border-radius": "0.4em",
                    "position": "absolute",
                    "border-color": options.fill,
                    "box-sizing":"content-box"
                });                
                var svgLink = "http://www.w3.org/2000/svg";
                var svgShape = document.createElementNS(svgLink, "svg");
                svgShape.setAttribute('id', options.resetZoomId);
                svgShape.setAttribute('width', 25);
                svgShape.setAttribute('height', 25);

                var points1 = {
                    'id': 'p1',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points1.points = "11.895,18.398 8.061,22.23 5.796,19.967 5.796,26.283 12.112,26.283 9.848,24.018 13.682,20.186";

                var path1 = document.createElementNS(svgLink, "polygon");
                $(path1).attr(points1).appendTo(svgShape);

                var points2 = {
                    'id': 'p2',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points2.points = "19.691,6.072 21.955,8.337 18.121,12.172 19.908,13.959 23.742,10.123 26.007,12.389 26.007,6.072";

                var path2 = document.createElementNS(svgLink, "polygon");
                $(path2).attr(points2).appendTo(svgShape);

                var points3 = {
                    'id': 'p3',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points3.points = "11.895,13.958 13.682,12.172 9.848,8.337 12.112,6.072 5.796,6.072 5.796,12.389 8.061,10.123";

                var path3 = document.createElementNS(svgLink, "polygon");
                $(path3).attr(points3).appendTo(svgShape);

                var points4 = {
                    'id': 'p4',
                    'fill': '#FFFFFF',
                    'transform': 'translate(-3,-3)'
                };
                points4.points = "19.908,18.396 18.121,20.186 21.955,24.018 19.691,26.283 26.007,26.283 26.007,19.967 23.741,22.23";

                var path4 = document.createElementNS(svgLink, "polygon");
                $(path4).attr(points4).appendTo(svgShape);

                resetZoom[0].appendChild(svgShape);
                document.getElementById(this._rootId).appendChild(resetZoom[0]);
            } else {
                var resetZoom = document.getElementById(this.svgObj.id + '_ResetZoom');
                $(resetZoom).css({
                    "top": options.top + chartPos.top,
                    "left": options.left + chartPos.left
                });
            }
        },
        drawCrosshairLine: function (options, evt) {
            var line = $("<div id=" + options.id + " style='position: absolute; z-index: 13000;'></div>");
            $(line).css({
                "width": options.width,
                "height": options.height,
                "left": options.left,
                "top": options.top,
                "border-style": "solid",
                "opacity": 0.5,
                "visibility": "visible",
                "border-color": options.stroke,
                "border-width": options["stroke-width"],
                "z-index": 100
            });
            document.getElementById(this._rootId).appendChild(line[0]);
        },
        drawCrosshairlabel: function (rectOptions, textOptions, text) {
            var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };
            if ($("#" + rectOptions.id).length > 0) {
                var label = document.getElementById(rectOptions.id);                
                $(label).css({
                    "width": rectOptions.width - 5,
                    "height": rectOptions.height - 5,
                    "left": textOptions.x + chartPos.left - 5,
                    "top": textOptions.y + chartPos.top - 15,
                    "visibility": "visible",
                    "display": rectOptions.display
                });
                document.getElementById(rectOptions.id).textContent = text;
            } else {
                var label = $("<div id=" + rectOptions.id + " style='position: absolute; z-index: 13000;'></div>");
                $(label).css({
                    "color": textOptions.fill,
                    "font-family": textOptions["font-family"],
                    "font-size": textOptions["font-size"],
                    "font-weight": textOptions["font-weight"],
                    "font-style": textOptions["font-style"],
                    "background-color": rectOptions.fill,
                    "border-style": "solid",
                    "border-color": rectOptions.stroke,
                    "border-width": rectOptions["stroke-width"],
                    "opacity": rectOptions["fill-opacity"],
                    "text-align" : "center"
                });
                
                document.getElementById(this._rootId).appendChild(label[0]);
            }
        },
        drawTrackToolTip: function (rectOptions, textOptions, text, padding) {
            var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };
            if ($("#" + rectOptions.id).length > 0) {
                var label = document.getElementById(rectOptions.id);               
                $(label).css({
                    "left": rectOptions.x + chartPos.left,
                    "top": rectOptions.y + chartPos.top,
                    "visibility": "visible",
                    "border-box": "content-box",
					"background-color": rectOptions.fill,
					"color": textOptions.fill, 
					"border-color": rectOptions.stroke,
					"border-width": rectOptions["stroke-width"]
                });
                 if (jQuery.type(text) == "array") {
                    var j = 0;
                    var arrayText = "";
                    for (var i = 0; i < text.length; i++) {
                        arrayText = arrayText + text[i] + '</br>';
                    }
                    $("#" + rectOptions.id).html(arrayText);
                } else
                    $("#" + rectOptions.id).html(text);
            } else {
                var label = $("<div id=" + rectOptions.id + " style='position: absolute; z-index: 13000;'></div>");
                $(label).css({
                    "color": textOptions.fill,
                    "font-family": textOptions["font-family"],
                    "font-size": textOptions["font-size"],
                    "font-weight": textOptions["font-weight"],
                    "font-style": textOptions["font-style"],
                    "width": rectOptions.width,
                    "height": rectOptions.height - padding - 2 * rectOptions["stroke-width"],
                    "background-color": rectOptions.fill,
                    "border-style": "solid",
                    "text-align": "center",
                    "padding" : "0px",
                    "padding-top": (padding/2) + 'px',
                    "padding-bottom": (padding/2) + 'px',
                    "border-color": rectOptions.stroke,
                    "border-width": rectOptions["stroke-width"],
                    "opacity": rectOptions["fill-opacity"]
                });
                document.getElementById(this._rootId).appendChild(label[0]);
            }
        },
        trackSymbol: function (options, evt, symbolName, trackSymbol, chartObj) {
		var chartAreaRect = chartObj.model.m_AreaBounds;
        if (evt.startX >= chartAreaRect.X && evt.startX <= chartAreaRect.X + chartAreaRect.Width && evt.startY >= chartAreaRect.Y && evt.startY <= chartAreaRect.Y + chartAreaRect.Height) {
            var seriesIndex = options.SeriesIndex;
            var pointIndex = options.PointIndex;
            if (chartObj.model.series[seriesIndex].type.toLowerCase() == "bubble") {
                var point = chartObj.model.series[seriesIndex].points[pointIndex];
                var width = (point.radius * 2)+15;//exploding the bubble
                var height = width;
                options.Style.BorderColor = options.Style.Color;
            }
            
            if (chartObj.model.series[seriesIndex].type.toLowerCase() == "scatter" && ej.util.isNullOrUndefined(trackSymbol)) {
                var circleOptions = {
                    "cx": evt.startX -2,
                    "cy": evt.startY -2,
                    "r": options.ShapeSize.width/2,
                    "fill" : options.Style.Color,
                    "opacity": options.Style.Opacity,
                    "stroke" : options.Style.BorderColor
                };
                this.drawCircle(circleOptions, null);
            }
            
        else {
                if ($("#" + "canvas_trackSymbol").length > 0)
                    var parentDiv = $("#" + "canvas_trackSymbol");
                else
                    parentDiv = $("<div id=" + "canvas_trackSymbol" + "></div>");

                var chartPos = { left: this.svgObj.offsetLeft, top: this.svgObj.offsetTop };                
               
                var symbol = $("<div id=" + options.ID + "></div>");
                var left = (evt.startX + chartPos.left - options.ShapeSize.width);
                var top = (evt.startY + chartPos.top - options.ShapeSize.height);
                
                    $(symbol).css({
                       
                        "left":  left ,
                        "top": top,
                        "box-sizing": 'content-box',
                        "position": "absolute",
                        "visibility": "visible",                        
                        "opacity": options.Style.Opacity,
                        "z-index" : 200
                    });
                    var style = options.ShapeSize;
                    options.symbol = symbol;
                    var shapeAttr = {};
                    switch (symbolName.toLowerCase()) {
                        case "circle":
                            $(symbol).css({
                                "width": options.ShapeSize.width + 'px',
                                "height": options.ShapeSize.height + 'px',
                                "left": (left + options.ShapeSize.width / 2 - options.Style.BorderWidth) + 'px',
                                "top": (top + options.ShapeSize.height / 2 - options.Style.BorderWidth) + 'px',
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "border-radius": "100%"
                            });
                            break;
                        case "rectangle":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height,
                                "left": left + options.ShapeSize.width / 2 - options.Style.BorderWidth / 2,
                                "top": top + options.Style.BorderWidth / 2,
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth
                            });
                            break;
                        case "diamond":

                            var direction  = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " 
                                             + "L" + " " + (evt.startX) + " " + (evt.startY + (-options.ShapeSize.height / 2)) + " "
                                             + "L" + " " + (evt.startX + (options.ShapeSize.width / 2)) + " " + (evt.startY) + " "
                                             + "L" + " " + (evt.startX) + " " + (evt.startY + ( options.ShapeSize.height/2)) + " " 
                                             + "L" + " " + (evt.startX + (-options.ShapeSize.width / 2)) + " " + (evt.startY) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);
                          
                            break;
                        case "triangle":
                                                     
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " + (evt.startY + (-style.height / 2))
                                + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);
                           
                            break;
                        case "invertedtriangle":
                                                     
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 2)) + "z";
                            var x = evt.startX;
                            var y = evt.startY;
                            var rotate = 'rotate(180,' + x + ',' + y + ')';

                            shapeAttr.direction = direction;
                            shapeAttr.rotate = rotate;
                            this.drawSvgShape(options, shapeAttr);
                            
                            break;
                       
                        case "cross":
                           
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (style.width / 2))
                                + " " + (evt.startY) + " " + "M" + " " + (evt.startX) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX) + " " +
                                (evt.startY + (-style.height / 2)) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "star":
                           
                            var direction = "M" + " " + (evt.startX + (style.width / 3)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " +
                                (evt.startY + (style.height / 6)) + " " + "L" + " " + (evt.startX + (style.width / 2)) + " " + (evt.startY + (style.height / 6)) + " " + "L" + " " + (evt.startX + (-style.width / 3)) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 3)) + " " + (evt.startY + (-style.height / 2)) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "hexagon":
                           
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (-style.width / 4)) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (style.width / 4)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (style.width / 4)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 4)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                        case "wedge":
                           
                            var direction = "M" + " " + (evt.startX -style.width) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + style.width) + " " +
                                (evt.startY + (-style.height / 2)) + " " + "L" + " " + (evt.startX + (3 * (style.width / 4))) + " " + (evt.startY) + " " + "L" + " " +
                                (evt.startX + (style.width)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX - style.width) + " " + (evt.startY) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "trapezoid":
                                                       
                            var direction = "M" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " +
                                (evt.startY + (-style.height / 4)) + " " + "L" + " " + (evt.startX + (-style.width / 2) + (style.width)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " +
                                (evt.startX + (-style.width / 2) + (style.width)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY + (style.height / 4)) +
                                " " + "L" + " " + (evt.startX + (-style.width / 2)) + " " + (evt.startY) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "uparrow":
                           
                            var direction = "M" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX)) + " " + (evt.startY - (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "downarrow":
                           
                            var direction = "M" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2) - (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX + (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX)) + " " + (evt.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2) + (style.width / 4))) + " " + (evt.startY - (style.height / 2)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "leftarrow":
                           
                            var direction = "M" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " +
                                ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + ((evt.startX -
                                (style.width / 2))) + " " + (evt.startY) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " +
                                (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY + (style.height / 4)) + "z";

                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "rightarrow":

                            var direction = "M" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (-style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + style.width) + " " + (evt.startY) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 2)) + " " + "L" + " " + ((evt.startX - (style.width / 2)) + (style.width / 2)) + " " + (evt.startY + (style.height / 4)) + " " + "L" + " " + ((evt.startX - (style.width / 2))) + " " + (evt.startY + (style.height / 4)) + "z";
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "pentagon":
                           
                            var eq = 72;
                            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
                            var sb = ej.EjSvgRender.utils._getStringBuilder();
                            for (var i = 0; i <= 5; i++) {
                                var deg = i * eq;
                                var rad = (Math.PI / 180) * deg;
                                var x1 = radius * Math.cos(rad);
                                var y1 = radius * Math.sin(rad);
                                if (i == 0)
                                    sb.append("M" + " " + (evt.startX + x1) + " " + (evt.startY + y1) + " ");
                                else
                                    sb.append("L" + " " + (evt.startX + x1) + " " + (evt.startY + y1) + " ");
                            }
                            sb.append("z");
                            var direction = sb.toString();
                            shapeAttr.direction = direction;
                            this.drawSvgShape(options, shapeAttr);

                            break;
                        case "ellipse":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height / 2,
                                "border-style": "solid",
                                "left": left + options.ShapeSize.height / 2 - options.Style.BorderWidth / 2,
                                "top": (evt.startY + chartPos.top - options.ShapeSize.height + 4),
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "-moz-border-radius": "50%",
                                "-webkit-border-radius": "50%",
                                "border-radius": "50%",
                                "border-box": "content-box"
                            });
                            break;
                        case "horizline":
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": 0,
                                "border-style": "solid",
                                "left": left + options.ShapeSize.width / 2,
                                "top": (evt.startY + chartPos.top - 1),
                                "border-color": options.Style.BorderColor,
                                "border-width": '2px',
                                "border-box": "content-box"
                            });
                            break;
                        case "vertline":
                            $(symbol).css({
                                "width": 0,
                                "height": options.ShapeSize.height,
                                "border-style": "solid",
                                "left": (evt.startX + chartPos.left - 1),
                                "top": top + options.ShapeSize.height / 2 - 2,
                                "border-color": options.Style.BorderColor,
                                "border-width": '2px',
                                "border-box": "content-box"
                            });
                            break;
                        default:
                            $(symbol).css({
                                "width": options.ShapeSize.width,
                                "height": options.ShapeSize.height,
                                "border-style": "solid",
                                "background-color": options.Style.Color,
                                "border-color": options.Style.BorderColor,
                                "border-width": options.Style.BorderWidth,
                                "border-radius": "50px"
                            });
                            break;
                    }
                    parentDiv[0].appendChild(symbol[0]);
                document.getElementById(this._rootId).appendChild(parentDiv[0]);
            }
		  }
        },
        //draw SVG Marker shape on mouse over the points
        drawSvgShape: function (options, attr) {
            var chartCanvas = document.getElementById(this._rootId + "_canvas");
            var chartPos = { left: chartCanvas.offsetLeft, top: chartCanvas.offsetTop };
            var svgLink = "http://www.w3.org/2000/svg";
            var svgShape = document.createElementNS(svgLink, "svg");
            svgShape.setAttribute('id', 'mySVG');
            svgShape.setAttribute('width', $(chartCanvas).width());
            svgShape.setAttribute('height', $(chartCanvas).height());
            $(options.symbol).css({
                "left":  chartPos.left + 'px',
                "top": chartPos.top + 'px'
            });
            var attrOptions = {
                'id': 'svgPath',
                'fill-opacity': options.Style.Opacity,
                'stroke-width': options.Style.BorderWidth,
                'fill': options.Style.Color,
                'transform' : attr.rotate,
                'stroke': options.Style.BorderColor
            };
            attrOptions.d = attr.direction;

            var path = document.createElementNS(svgLink, "path");
            $(path).attr(attrOptions).appendTo(svgShape);
            options.symbol[0].appendChild(svgShape);
        },
        drawCircle: function (options, element) {
            if (!options.lgndCtx) {
                var canvasCtx = this.ctx;
                this.ctx = element != undefined ? element : this.ctx;
                this.ctx.save();
                this.ctx.beginPath();
                this.ctx.arc(options.cx, options.cy, options.r, 0, 2 * Math.PI);
                this.ctx.fillStyle = options.fill;
                this.ctx.globalAlpha = options.opacity;
                this.ctx.fill();
                this.ctx.lineWidth = options["stroke-width"];
                this.ctx.strokeStyle = options.stroke;
                this.ctx.stroke();
                this.ctx.restore();
                this.ctx = canvasCtx;
            } else {
                var legendCtx = this.lgndCtx;
                this.lgndCtx = element != undefined ? element : this.lgndCtx;
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                this.lgndCtx.arc(options.cx, options.cy, options.r, 0, 2 * Math.PI);
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.globalAlpha = options.opacity;
                this.lgndCtx.fill();
                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
                this.lgndCtx = legendCtx;
            }
        },    
        
        drawPolyline: function (options, element) {
            
            this.ctx.save();
            this.ctx.beginPath();
            var points = options.points.split(" ");
            for (var i = 0; i < points.length - 1; i++) {
                var point = points[i].split(",");
                var x = point[0];
                var y = point[1];
                if (i == 0) 
                    this.ctx.moveTo(x,y);
                 else
                    this.ctx.lineTo(x,y);
            }
            this.ctx.lineWidth = 0.5;
			 this.ctx.strokeStyle = options.stroke;
            this.ctx.stroke();
            this.ctx.restore();            
        },
        drawPolygon: function (options, element) {
           
        },
        setFillAttribute: function (element, options) {
           
        },
        setStrokeAttribute: function (element, options) {
            
        },
        changePathValue: function (options) {
            
        },


        drawArc: function (w, h, options) {
            var x = 0, y = 0;
  
        },
        changeVMLStyle: function($element, options) {
           
        },
        applyVMLStyle: function ($element, options) {

        },
       
        drawEllipse: function (options, element) {
            var canvasCtx = this.ctx;
            this.ctx = element != undefined ? element : this.ctx;
            var circumference = Math.max(options.rx, options.ry);
            var scaleX = options.rx / circumference;
            var scaleY = options.ry / circumference;
            if (!options.lgndCtx) {
                this.ctx.save();
                this.ctx.beginPath();
                this.ctx.translate(options.cx, options.cy);
                this.ctx.save();
                this.ctx.scale(scaleX, scaleY);
                this.ctx.arc(0, 0, circumference, 0, 2 * Math.PI, false);
                this.ctx.fillStyle = options.fill;
                this.ctx.fill();
                this.ctx.restore();
                this.ctx.lineWidth = options["stroke-width"];
                this.ctx.strokeStyle = options.stroke;
                this.ctx.stroke();
                this.ctx.restore();
                this.ctx = canvasCtx;
            } else {
                this.lgndCtx.save();
                this.lgndCtx.beginPath();
                this.lgndCtx.translate(options.rx, options.ry);
                this.lgndCtx.scale(scaleX, scaleY);
                this.lgndCtx.arc(options.cx - options.rx, options.cy * 2 - options.rx, options.rx, 0, 2 * Math.PI, false);
                this.lgndCtx.fillStyle = options.fill;
                this.lgndCtx.fill();
                this.lgndCtx.restore();
                this.lgndCtx.lineWidth = options["stroke-width"];
                this.lgndCtx.strokeStyle = options.stroke;
                this.lgndCtx.stroke();
                this.lgndCtx.restore();
            }
        },
      
        _getAttrVal: function (ele, val, option) {
           
        },
        _setAttr: function (element, attribute) {
          
        },
       
        setSpanAttr: function (options, label, element) {
           
        },
       
        //text element attributes values are replaced 
        _textAttrReplace: function (options, label, font, $Ele) {
           
        },

        drawImage: function (options, element) {
             this.ctx.save();
             var imageObj = new Image();
             imageObj.src = options.href;
             this.ctx.drawImage(imageObj, options.x, options.y, options.width, options.height);
             this.ctx.restore();
        },

        createDefs: function () {

        },

        createGroup: function (options) {
           
        },

        createText: function (options, label) {
            var text = document.createElement("div");
            var diff= 1.1; // difference between svg and div element rotated text height
            $(text).css({
                "transform": options.rotateAngle,
                "font-family": parseFloat(options["font-family"]),
                "font-size": parseFloat(options["font-size"]) * diff,
                "position": "absolute"}); 
            if (!ej.util.isNullOrUndefined(label))
                text.textContent = label;
            return text;
        },


        //Draw clip path for each series to avoid series overlap in multiple axes zooming
        drawAxesBoundsClipPath: function (gSeriesGroupEle, options, invertedAxis) {
            var clipOptions = options;
           
        },

        getAttrVal: function (ele, val, option) {
       
        },

        hexFromRGB: function (color) {
          
        },

        createDelegate: function (context, handler) {
           
        },

        drawClipPath: function (options, element) {
            
        },

        drawCircularClipPath: function (options, element) {

        },

        append: function (childEle, parentEle) {
            $(childEle).appendTo(parentEle);
			},
			
		createLegendCanvas: function (element) {            
            this.legendsvgObj = document.createElement('canvas');
            this._rootId = jQuery(element).attr("id");
            this.legendsvgObj.setAttribute('id', "legend_" + this._rootId + '_canvas');
            this.lgndCtx = this.legendsvgObj.getContext("2d");
            return this.legendsvgObj;
        }
		
    };
})(jQuery);


;
ej.EjSvgRender = function (element) {

    this.svgLink = "http://www.w3.org/2000/svg";
    this.svgObj = document.createElementNS(this.svgLink, "svg");
    this._rootId = jQuery(element).attr("id");
    var id = this._rootId + '_svg';   
    if ($(document).find("#" + id).length > 0){       
        var count=0
        do {
            count++;
        } while ($(document).find("#" + this._rootId + '_svg' + count).length > 0);
        id = this._rootId + '_svg' + count;
    }    
    this.svgObj.setAttribute('id', id);

};
(function ($) {
ej.EjSvgRender.prototype = {
    drawPath: function (options, element) {
        if ($("#" + options.id).length > 0) { 
            $("#" + options.id).attr(options);
        }
        else {
            var path = document.createElementNS(this.svgLink, "path");
            $(path).attr(options).appendTo(element);
        }

    },
    createLegendSvg:function(element)
    {
        this.svgLink = "http://www.w3.org/2000/svg";
        this.legendsvgObj = document.createElementNS(this.svgLink, "svg");
        this._rootId = jQuery(element).attr("id");
        this.legendsvgObj.setAttribute('id', "legend_" + this._rootId + '_svg');
        return this.legendsvgObj;
    },
    drawPolyline: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var polyline = document.createElementNS(this.svgLink, "polyline");
            $(polyline).attr(options).appendTo(element);
        }

    },

    drawLine: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var path = document.createElementNS(this.svgLink, "line");
            $(path).attr(options);
            $(path).appendTo(element);
        }
    },
    drawPolygon: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var polygon = document.createElementNS(this.svgLink, "polygon");
            $(polygon).attr(options);
            $(polygon).appendTo(element);
        }
    },
    drawCircle: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var circle = document.createElementNS(this.svgLink, "circle");
            $(circle).attr(options).appendTo(element);
        }
    },
    drawEllipse: function (options, element) {
        if ($("#" + options.id).length > 0) {
            $("#" + options.id).attr(options);
        }
        else {
            var ellipse = document.createElementNS(this.svgLink, "ellipse");
            $(ellipse).attr(options).appendTo(element);
        }
    },
    
    drawRect: function (options, element) {
            if ($("#" + options.id).length > 0) {
                $("#" + options.id).attr(options);
            }
            else {
                var rect = document.createElementNS(this.svgLink, "rect");
                $(rect).attr(options).appendTo(element);
            }    
    },

    createGradientElement: function (name, colors, x1, y1, x2, y2, element) {
        var colorName;
        if (Object.prototype.toString.call(colors) == '[object Array]') {
            var options = {
                'id': this.svgObj.id + '_' + name + 'Gradient',
                'x1': x1 + '%',
                'y1': y1 + '%',
                'x2': x2 + '%',
                'y2': y2 + '%'
            };
            var cName = '#' + this.svgObj.id + '_' + name + 'Gradient';
                this.drawGradient(options, colors, element);
            colorName = 'url(#' + this.svgObj.id + '_' + name + 'Gradient)';
        }
        else {
            colorName = colors;
        }
        return colorName;
    },

    drawGradient: function (options, gradientEle, element) {

        var defs = this.createDefs();
        var linerGradient = document.createElementNS(this.svgLink, "linearGradient");
        $(linerGradient).attr(options);
        for (var i = 0; i < gradientEle.length; i++) {
            var stop = document.createElementNS(this.svgLink, "stop");
            $(stop).attr({
                'offset': gradientEle[i].colorStop,
                'stop-color': gradientEle[i].color,
                'stop-opacity': 1
            });
            $(stop).appendTo(linerGradient);
        }

        $(linerGradient).appendTo(defs);
        $(defs).appendTo(element);
    },

    drawText: function (options, label, groupEle,font) {
        
        if ($("#" + options.id).length > 0) 
            this._textAttrReplace(options, label,font);           
       else{
            var text = document.createElementNS(this.svgLink, "text");
            var $text = $(text);


            if (jQuery.type(label) == "array") {
                var j = 0;
                $text.attr(options);
                for (var i = 0; i < label.length; i++) {
                    var textspan = document.createElementNS(this.svgLink, "tspan");
                    textspan.textContent = label[i];
                    $(textspan).attr({ "x": options.x, "dy": j });
                    $(textspan).appendTo(text);
                    var bounds = ej.EjSvgRender.utils._measureText(label[i],null,font);
                    j = bounds.height+2;
                }
            } else {
                text.textContent = label;
                $text.attr(options);
            }
            $text.appendTo(groupEle);

 
        }
    },
    //text element attributes values are replaced 
    _textAttrReplace: function (options, label, font) {
        $("#" + options.id).attr(options);

        if (jQuery.type(label) == "array") {
            var elements = $("#" + options.id).children("tspan");
            var j = 0;
            if (elements.length > 0 && elements.length == label.length) {
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    $(element).attr({ "x": options.x, "dy": j });
                    element.textContent = label[i];
                    var bounds = ej.EjSvgRender.utils._measureText(label[i], null, font);
                    j = bounds.height + 2;
                }
            }
            else {
                $("#" + options.id).remove();
                this.drawText(options, label, this.gTransToolEle,font);

            }
        } else {
            $("#" + options.id).text(label);
        }

    },

    drawImage: function (options, element) {

        var img = document.createElementNS(this.svgLink, 'image');
        img.setAttributeNS(null, 'height', options.height);
        img.setAttributeNS(null, 'width', options.width);
        img.setAttributeNS('http://www.w3.org/1999/xlink', 'href', options.href);
        img.setAttributeNS(null, 'x', options.x);
        img.setAttributeNS(null, 'y', options.y);
        img.setAttributeNS(null, 'id', options.id);
        img.setAttributeNS(null, 'visibility', options.visibility);
        if (!ej.util.isNullOrUndefined(options.clippath) || !ej.util.isNullOrUndefined(options.preserveAspectRatio)) {
            img.setAttributeNS(null, 'clip-path', options.clippath);
            img.setAttributeNS(null, "preserveAspectRatio", options.preserveAspectRatio);
        }
        $(img).appendTo(element);

    },

    createDefs: function () {
        var defs = document.createElementNS(this.svgLink, "defs");
        return defs;
    },

    createClipPath: function (options) {
        var clipPath = document.createElementNS(this.svgLink, "clipPath");
        $(clipPath).attr(options);
        return clipPath;
    },
    createForeignObject: function (options) {
        var object = document.createElementNS(this.svgLink, "foreignObject");
        $(object).attr(options);
        return object;
    },
    createGroup: function (options) {
        var group = document.createElementNS(this.svgLink, "g");
        $(group).attr(options);
        return group;
    },
    createPattern: function (options,element) {
        var pattern = document.createElementNS(this.svgLink, element);
        for (var name in options) {
            if (options.hasOwnProperty(name)) {
                pattern.setAttribute(name, options[name]);
            }
        }
        return pattern;
    },
    createText: function (options, label) {
        var text = document.createElementNS(this.svgLink, "text");
        $(text).attr(options);
        if (!ej.util.isNullOrUndefined(label))
            text.textContent = label;
        return text;
    },

    createPath: function (options) {
        var path = document.createElementNS(this.svgLink, "path");
        $(path).attr(options);
        return path;
    },

    createCircle: function (options) {
        var circle = document.createElementNS(this.svgLink, "circle");
        $(circle).attr(options);
        return circle;
    },

    createLine: function (options) {
        var line = document.createElementNS(this.svgLink, "line");
        $(line).attr(options);
        return line;
    },

   
    _getAttrVal: function (ele, val, option) {
        var value = $(ele).attr(val);
        if (value != null)
            return value;
        else
            return option;
    },
 
    
    hexFromRGB: function (color) {
        var r = color.R;
        var g = color.G;
        var b = color.B;
        var hex = [r.toString(16), g.toString(16), b.toString(16)];
        $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
        return "#" + (hex.join("").toUpperCase());
    },
    
	 checkColorFormat: function (color)
	  {
        return /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.test(color);
      },

    hexToRGB:function(hex) {
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? {
        R: parseInt(result[1], 16),
        G: parseInt(result[2], 16),
        B: parseInt(result[3], 16)
    } : null;
  },


    createDelegate: function (context, handler) {
        return function (e) {
            handler.apply(context, [e, this]);
        };
    },

    drawClipPath: function (options, element) {

        var defs = this.createDefs();
        var clipPath = this.createClipPath({ 'id': options.id });
        this.drawRect(options, clipPath);

        this.append(clipPath, defs);

        this.append(defs, element);

    },

    drawCircularClipPath: function (options, element) {
        var defs = this.createDefs();
        var clipPath = this.createClipPath({ 'id': options.id });
        this.drawCircle(options, clipPath);
        this.append(clipPath, defs);
        this.append(defs, element);
    },
   
   append: function (childEle, parentEle) {
        $(childEle).appendTo(parentEle);
   },
   _setAttr: function (element, attribute) {
       $(element).attr(attribute);
   }
};

ej.EjSvgRender.commonChartEventArgs = {
    cancel: false,
    data: null
};
ej.EjSvgRender.utils = {

   

    _decimalPlaces: function (num) {
        var match = ('' + num).match(/(?:\.(\d+))?(?:[eE]([+-]?\d+))?$/);
        if (!match) { return 0; }
        return Math.max(
       0,
        // Number of digits right of decimal point.
       (match[1] ? match[1].length : 0)
        // Adjust for scientific notation.
       - (match[2] ? +match[2] : 0));
    },
    _getLabelContent: function (pos, axis) {
        switch (axis._categoryValueType) {
            case 'number':
                var customFormat = (ej.util.isNullOrUndefined(axis.labelFormat)) ? null : axis.labelFormat.match('{value}');
                return (ej.util.isNullOrUndefined(axis.labelFormat)) ? axis.labels[Math.floor(pos)] : (customFormat != null) ? axis.labelFormat.replace('{value}', axis.labels[Math.floor(pos)]) : (Globalize.format(axis.labels[Math.floor(pos)], axis.labelFormat));
            case 'date':
                return (Globalize.format(new Date(axis.labels[Math.floor(pos)]), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? "dd/MM/yyyy" : axis.labelFormat)));
            case 'string':
                return axis.labels[Math.floor(pos)];
            default:
                return "";
        }
    },
    
    //Calculation for label size in template
    _getSeriesTemplateSize: function (point, pointIndex, series, isLeft, sender) {
        var point;
        var pointIndex;
        var areaType = sender.model.AreaType;
         
        var chartContainer = sender._id;
        var seriesIndex = $.inArray(series, sender.model._visibleSeries);

            if ($('#template_group_' + chartContainer).length != 0)
                var templateContainer = $('#template_group_' + chartContainer);
            else
                templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

            templateContainer.css('position', 'relative').css('z-index', 1000);
            var cloneNode = $("#" + series.marker.dataLabel.template).clone();
            $(cloneNode).attr("id", series.marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            var $cloneNode = $(cloneNode);
            $cloneNode.css("position", "absolute");
           
            point.count = 1;
            var data = { series: series, point: point };
            $cloneNode.html($cloneNode.html().parseTemplate(data));

            var display = (areaType == "cartesianaxes" || !series.enableAnimation || (series.type.toLowerCase() == "pyramid" || series.type.toLowerCase() == "funnel")) ? "block" : "none";
            $cloneNode.css("display", display).appendTo($(templateContainer));
            $(templateContainer).appendTo('#' + chartContainer);
            point.size = { height: $cloneNode.height(), width: $cloneNode.width() };
         
            if (isLeft) {
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxHeight) || series.LeftLabelMaxHeight < point.size.height) {
                    series.LeftLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.LeftLabelMaxWidth) || series.LeftLabelMaxWidth < point.size.width) {
                    series.LeftLabelMaxWidth = point.size.width;
                }
            }
            else {
                if (ej.util.isNullOrUndefined(series.RightLabelMaxHeight) || series.RightLabelMaxHeight < point.size.height) {
                    series.RightLabelMaxHeight = point.size.height;
                }
                if (ej.util.isNullOrUndefined(series.RightLabelMaxWidth) || series.RightLabelMaxWidth < point.size.width) {
                    series.RightLabelMaxWidth = point.size.width;
                }
            }
        
    },
    getMinPointsDelta: function (axis, chartObj, start) {
        var chart = chartObj;
        var m_minPointsDelta = Number.MAX_VALUE;
        $.each(chart.model._visibleSeries, function (index, series) {
            if (series.visibility.toLowerCase() == 'visible' && axis.name == series._xAxisName) {
                var xValues = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
                if (xValues.length == 1) {
                    var minValue = ej.util.isNullOrUndefined(start) ? series.xAxis.visibleRange.min : start;
                    var delta = xValues[0].xValue - minValue;
                    if (delta != 0)
                        m_minPointsDelta = Math.min(m_minPointsDelta, delta);
                }
                else {
                    $.each(xValues, function (pointIndex, point) {
                        if (pointIndex > 0 && point.xValue) {
                            var deltaValue = point.xValue - xValues[pointIndex - 1].xValue;
                            if (deltaValue != 0) {
                                m_minPointsDelta = Math.min(m_minPointsDelta, deltaValue);
                            }
                        }
                    });
                }
            }
        });

        if (m_minPointsDelta == Number.MAX_VALUE) {
            m_minPointsDelta = 1;
        }
        return m_minPointsDelta;
    },
    //Calculation for label size
    _getSeriesMaxLabel: function (series) {
        var maxTxtDim = { width: 0, height: 0 };
        var width = $(this.svgObject).width();
        var labels=[];
        if (series.labels.length > 0) {
            for (var j = 0; j < series.labels.length; j++) {
                var dim = this._measureText(series.visibleLabels[j], width, series.marker.dataLabel.font);
                if (maxTxtDim.width < dim.width) {
                    maxTxtDim.width = dim.width;
                }
                if (maxTxtDim.height < dim.height) {
                    maxTxtDim.height = dim.height;
                }
            }
            series.LabelMaxHeight = maxTxtDim.height;
            series.LabelMaxWidth = maxTxtDim.width;
        }
        else {
            labels.push(series.rightsidePoints);
            labels.push(series.leftsidePoints);

            for (var k = 0; k < labels.length; k++) {
                for (var j = 0; j < labels[k].length; j++) {
                    var text = (labels[k][j].text) ? labels[k][j].text : labels[k][j].y;
                    var dim = this._measureText(text, width, series.marker.dataLabel.font);
                    if (maxTxtDim.width < dim.width) {
                        maxTxtDim.width = dim.width;
                    }
                    if (maxTxtDim.height < dim.height) {
                        maxTxtDim.height = dim.height;
                    }
                }
                if (k == 0) {
                    series.RightLabelMaxHeight = maxTxtDim.height;
                    series.RightLabelMaxWidth = maxTxtDim.width;
                }
                else {
                    series.LeftLabelMaxHeight = maxTxtDim.height;
                    series.LeftLabelMaxWidth = maxTxtDim.width;
                }
             }
        }
       

    },
    _getMaxLabelWidth: function (axis, sender) {
        this.chartObj = sender;
        var maxTxtDim = { width: 0, height: 0, maxHeight:0, maxWidth:0 };
        var rotateLabel = '';
        var currentRow = 1;
        var vmlrendering = sender.svgRenderer.vmlNamespace;
        var range = axis.visibleRange;
        var intersectAction = axis.labelIntersectAction ? axis.labelIntersectAction.toLowerCase() : "";
        var labelPlacement = axis.labelPlacement;
        var opposedPosition = axis.opposedPosition;
        var orientation = axis.orientation.toLowerCase();
        var isHorizontal = sender.model.requireInvertedAxes ? (orientation == 'vertical') : orientation == 'horizontal';
        var roundingPlaces = axis.roundingPlaces;
        var labelRotation = axis.labelRotation;
        var orientation = axis.orientation;
        var maxHeight = 0;
        var prevLabels = [];
        var visibleLabels = axis.visibleLabels;
        var visibleLabelsLength =visibleLabels.length;
        if (axis.visible) {
                for (var j = 0; j < visibleLabelsLength ; j++) {
                    axis.visibleLabels[j].y = 0;
                    var dim = this._measureText(axis.visibleLabels[j].Text, $(this.svgObject).width(), axis.font, axis.labelRotation);
                    if (maxTxtDim.width < dim.width) {
                        maxTxtDim.width = dim.width;
                        rotateLabel = axis.visibleLabels[j].Text;
                    }
                    if (maxTxtDim.height < dim.height)
                        maxTxtDim.height = dim.height;
                }
			
			if (axis.enableTrim){   // for enable trim
			  var derivedGap = axis.maximumLabelWidth;
			  maxTxtDim.width = maxTxtDim.width > derivedGap ? derivedGap : maxTxtDim.width;
			}
			
            // initialize rows to 1
            maxTxtDim.rows = 1;
            maxTxtDim.maxWidth = maxTxtDim.width;
            maxTxtDim.maxHeight = maxTxtDim.height;
        
            if (!vmlrendering && (labelRotation ||intersectAction)) {
                var intersectRotation = 0;
                if (intersectAction == 'rotate45')
                    intersectRotation = 45;
                else if (intersectAction == 'rotate90')
                    intersectRotation = 90;
                labelRotation = labelRotation!=null ? labelRotation : intersectRotation;
                axis.rotationValue = labelRotation;
                if (labelRotation) {                
                    rotateLabel = (!ej.isNullOrUndefined(rotateLabel)) ? rotateLabel : '';
					labeltextWidth = this._measureText(rotateLabel, $(this.svgObject).width(), axis.font, axis.labelRotation);
					if (axis.enableTrim && labeltextWidth.width > derivedGap && rotateLabel != "") { // to find trimmed text
					  var t, textWidth, text = rotateLabel;
                      for (t = 1; t < text.toString().length; t++) {
                        text = text.toString().substring(0, t) + '... ';
                        textWidth = this._measureText(text, $(sender.svgObject).width(), axis.font);
                        if (textWidth.width >= derivedGap) {
                            text = text.toString().substring(0, t - 1) + '... ';
                            rotateLabel = text;
                            break;
                         }
                      }
                    }
					
                    maxSize = this.rotatedLabel(axis, sender, labelRotation, rotateLabel);
                    if (axis.labelIntersectAction && axis.labelIntersectAction.toLowerCase() == 'multiplerows')
                        maxHeight = maxTxtDim.height + maxSize.height;
                    maxTxtDim.height = maxSize.height;
					maxTxtDim.width = maxSize.width;
                }
            }

                if (axis.labelIntersectAction) {
                    if (intersectAction == 'wrap') {
                        // pointX calculation to find the gap between the ticks
                        labelPlacement = (!(labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : labelPlacement;
                        var ticksbwtLabel = ((axis.labels.length > 1) && (labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
                        var tempInterval = ((axis.labels.length > 1) && (ticksbwtLabel < 0)) ? 1 + ticksbwtLabel : visibleLabels[1].Value;
                        tempInterval = (!(roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(range.interval) == 0 ?
                            1 : ej.EjSvgRender.utils._decimalPlaces(range.interval)))) : parseFloat(tempInterval.toFixed(roundingPlaces));
                        var pointX = Math.ceil(((tempInterval - range.min) / (range.max - range.min)) * (axis.length));
                        var row = Math.round(maxTxtDim.width / pointX);
                        maxTxtDim.rows = labelRotation ? row - currentRow : row + currentRow;
                        maxTxtDim.height = (maxTxtDim.height + (row) * maxTxtDim.height);
                    }
                    else if (intersectAction == 'multiplerows') {
                        var spaceValue = sender._getLegendSpace();
                        var chartBorderWidth = sender.model.border.width;
                      
                        var verticalaxis = sender.model._axes[1];
                        var realWidth = $(sender.svgObject).width() - sender.model.margin.left - sender.model.margin.right;
                        var axisTitleHeight = (axis.title.text == "" || !(axis.visible)) ? 0 : (this._measureText(axis.title.text, realWidth, axis.title.font).height + (2 * sender.model.elementSpacing));
                        var vAxesWidth = sender.model.elementSpacing + axisTitleHeight + axis.majorTickLines.size + axis.axisLine.width;
                        var yLabels = sender._getYValues(sender.model._visibleSeries[0]);
                        var largest = Math.max.apply(Math, yLabels);
                        var vaxis = sender.model._axes[1];
                        var dim = this._measureText(largest, $(this.svgObject).width(), vaxis.font, vaxis.labelRotation);
 
                        var rightSpacing = spaceValue.rightLegendWidth + vAxesWidth + dim.width + sender.model.margin.right + sender.model.margin.left + (2 * chartBorderWidth);
                        var boundsWidth = $(sender.svgObject).width() - (rightSpacing);
                        if (isHorizontal) {
                            var addedMaxHeight = 0;
                            //loop to get current label
                            for (j = 0; j < visibleLabels.length; j++) {
                                //declaration
                                var currentLabel = visibleLabels[j];
                                var flag = false;
                                maxHeight = Math.max(maxHeight, addedMaxHeight)
                                addedMaxHeight = currentLabel.y = maxTxtDim.height;
                                var text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                var textWidth = text.width;
                                var textHeight = text.height;
                                var currentPoint = Math.abs(Math.floor(((currentLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                // loop to get previous labels                         
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var prevLabel = prevLabels[k][l];
                                        var prevPoint = Math.abs(Math.floor(((prevLabel.Value - range.min) / (range.delta)) * (boundsWidth)));
                                        var preTextWidth = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).width;
                                        var value = prevPoint + preTextWidth / 2;
                                        if (value > currentPoint - textWidth / 2 && !ej.util.isNullOrUndefined(currentLabel.y)) {
                                            addedMaxHeight = currentLabel.y + textHeight;
                                            currentLabel.y += textHeight;
                                            if (k + 1 == prevLabels.length)
                                            flag = true;
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }
                                    }
                                  
                                 }
                                currentLabel.y = addedMaxHeight;
                                row = (addedMaxHeight / textHeight) - 1;
                                if( prevLabels[row]==undefined)
                                prevLabels[row] = [];
                                prevLabels[row].push(currentLabel);
                               
                            }
                            maxHeight = (prevLabels.length * textHeight);
                         //   maxHeight = opposedPosition ? maxHeight + textHeight : maxHeight;
                            maxTxtDim.height = maxTxtDim.height > maxHeight ? maxTxtDim.height : maxHeight;
                            //calculating rows
                            if (opposedPosition)
                                maxTxtDim.rows = Math.round((maxHeight + textHeight) / textHeight);
                            else
                                maxTxtDim.rows = Math.round(maxHeight / textHeight);
                            maxTxtDim.rows = labelRotation ? maxTxtDim.rows - 1 : maxTxtDim.rows ;
                            if (maxTxtDim.rows < 1) maxTxtDim.rows = 1;
                        }
                        if (!isHorizontal) {
                            var addedMaxWidth = 0;
                            //loop to get current label
                            for (j = 0; j < axis.visibleLabels.length; j++) {
                                currentLabel = axis.visibleLabels[j];
                                text = this._measureText(currentLabel.Text, $(this.svgObject).width(), axis.font);
                                textWidth = text.width;
                                textHeight = text.height;
                                currentPoint = Math.abs(Math.floor(((currentLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                for (i = 0; i < j; i++) {
                                    // loop to get previous labels
                                    prevLabel = axis.visibleLabels[i];
                                    prevPoint = Math.abs(Math.floor(((prevLabel.Value - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.length)));
                                    var prevTextHeight = this._measureText(prevLabel.Text, $(this.svgObject).width(), axis.font).height;
                                    value = prevPoint + prevTextHeight / 2;
                                    if (value > currentPoint - textHeight / 2 && axis.visibleLabels[i].y == currentLabel.y) {
                                        addedMaxWidth = currentLabel.y + textWidth;
                                        currentLabel.y = addedMaxWidth;
                                    }
                                }
                               var maxheight = addedMaxWidth;
                            }
                            // label width + multiple rows width + gap between the rows
                            maxTxtDim.width = maxTxtDim.width + maxheight +5;
                        }
                    }
               }
        }
        return maxTxtDim;
    },
	
        rotatedLabel: function (axis, sender, value, rotatedLabel) {
            // to get height of rotated labels
            var rotatedOptions = {
                'font-size': axis.font.size,
                'transform': 'rotate(' + value + ',0,0)',
                'font-family': axis.font.fontFamily,
                'font-style': axis.font.fontStyle,
                'rotateAngle' : 'rotate('+ value +'deg)',
                'text-anchor': 'middle'
            };
            var text = sender.svgRenderer.createText(rotatedOptions, rotatedLabel);
            var height = Math.ceil((this._measureBounds(text, sender).height));
			var width = Math.ceil((this._measureBounds(text, sender).width));
            return {height: height, width: width};
        },
		
    _getTransform: function (xAxis, yAxis, invertedAxis) {
        var x , y , width, height;
        if (invertedAxis) {
            x = yAxis.x;
            y = xAxis.y;
            width = yAxis.width;
            height = xAxis.height;
        } else {
            {
                x = xAxis.x;
                y = yAxis.y;
                width = xAxis.width;
                height = yAxis.height;
            }
        }
        return { x: x, y: y, width: width, height: height };
    },
    _measureText: function (text, maxwidth, font) {
        var element = $(document).find("#measureTex");
		$("#measureTex").css('display', 'block'); // fixed for scroll issue in sample browser
        if (element.length==0) {
            var textObj = document.createElement('text');
            $(textObj).attr({ 'id': 'measureTex'});
            document.body.appendChild(textObj);
        }
        else
        {
            var textObj = element[0];
        }
       
        var style=null, size=null, family=null;
        textObj.innerHTML = text;
        if (font != undefined && font.size == undefined) {
            var fontarray = font;
            fontarray=fontarray.split(" ");
            style = fontarray[0];
            size = fontarray[1];
            family = fontarray[2];
        }
       
        if (font != null) {
            textObj.style.fontSize = (font.size > 0) ? (font.size + "px") : font.size? font.size : size ;
            if (textObj.style.fontStyle) 
     			textObj.style.fontStyle = (font.fontStyle)? font.fontStyle : style;
            textObj.style.fontFamily = font.fontFamily ? font.fontFamily : family;
        }
        textObj.style.backgroundColor = 'white';
        textObj.style.position = 'absolute';
        textObj.style.top = -100;
        textObj.style.left = 0;
		textObj.style.visibility = 'hidden';
        if (maxwidth)
            textObj.style.maxwidth = maxwidth + "px";
         
    
        var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
        $("#measureTex").css('display', 'none'); // fixed for scroll issue in sample browser
        return bounds;
    },
    _measureBounds: function (element, sender) {
        if (sender.model.enableCanvasRendering) 
             $(document.body).append(element);
         else {
            sender.svgRenderer.append(element, sender.svgObject);
            sender.svgRenderer.append(sender.svgObject, sender.element);
        }
        var box = element.getBoundingClientRect();
        var bounds = { left: box.left, right: box.right, top: box.top, bottom: box.bottom, width: (box.right - box.left), height: (box.bottom - box.top) };
        $(element).remove();
        return bounds;
    },
    //Draw clip path for each series to avoid series overlap in multiple axes zooming
    _drawAxesBoundsClipPath: function (gSeriesGroupEle, options, sender) {
        var clipOptions;
        var element = $(gSeriesGroupEle);
        var trans = this._getTransform(options.xAxis, options.yAxis, sender.model.requireInvertedAxes);
        var width = (sender.model.AreaType == "polaraxes") ? $(sender.svgObject).width() : trans.width;
        var height = (sender.model.AreaType == "polaraxes") ? $(sender.svgObject).height() : trans.height;

        var hPlotOffset = sender.model.requireInvertedAxes ? options.yAxis.plotOffset : options.xAxis.plotOffset;
        var vPlotOffset = sender.model.requireInvertedAxes ? options.xAxis.plotOffset : options.yAxis.plotOffset;
        if (sender.model.AreaType == 'polaraxes') {
            clipOptions = {
                'id': gSeriesGroupEle ? gSeriesGroupEle.id + '_ClipRect' : '',
                'cx': sender.model.centerX,
                'cy': sender.model.centerY,
                'r': sender.model.Radius,
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'transparent'
            };
            sender.svgRenderer.drawCircularClipPath(clipOptions, gSeriesGroupEle);
        }
        else {
            clipOptions = {
                'id': gSeriesGroupEle ? gSeriesGroupEle.id + '_ClipRect' : '',
                'x': (0 - hPlotOffset),
                'y': (0 - vPlotOffset),
                'width': (width + 2 * hPlotOffset),
                'height': (height + 2 * vPlotOffset),
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'transparent'
            };
            sender.svgRenderer.drawClipPath(clipOptions, gSeriesGroupEle);
        }
                
        element.attr('clip-path', 'url(#' + clipOptions.id + ')');

    },
    _getStringBuilder: function () {

        var data = [];
        var counter = 0;

        return {
            // adds string s to the stringbuilder

            append: function (s) {
                data[counter++] = s;
                return this;
            },

            // removes j elements starting at i, or 1 if j is omitted

            remove: function (i, j) {
                data.splice(i, j || 1);
                return this;
            },

            // inserts string s at i

            insert: function (i, s) {
                data.splice(i, 0, s);
                return this;
            },

            // builds the string

            toString: function (s) { return data.join(s || ""); }
        };


    },
    _addRegion: function (chart, bounds, series, point, pointIndex) {
	    var type = series.type;
        var seriesIndex = $.inArray(series, chart.model._visibleSeries);
        if (seriesIndex >= 0) {
	        var regionItem = { SeriesIndex: seriesIndex, Region: { PointIndex: pointIndex, Bounds: bounds }, type: type };
	        chart.model.chartRegions.push(regionItem);
	    }
    },

    AddRegion: function (chart, bounds, isStripLine) {
       
        if (isStripLine) {
            var regionItem = { isStripLine: isStripLine, Region: { Bounds: bounds } };
            chart.model.chartRegions.push(regionItem);
        }
    },


    _getSvgXY: function (x, y, series, sender) {
        var svgX, svgY;
        if (!(sender.model.requireInvertedAxes)) {
              svgX = x + series.xAxis.x;
              svgY = y + series.yAxis.y;
        } else {
              svgX = x + series.yAxis.x;
              svgY = y + series.xAxis.y;
        }
        return { X: svgX, Y: svgY };
    },
    _getPoint: function (point, series) {
	  
        var x = point.xValue;
        var y = point.YValues[0];

        var xvalue = (series.xAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((x == 0 ? 1 : x), series.xAxis.logBase) : x;
        var yvalue = (series.yAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((y == 0 ? 1 : y), series.xAxis.logBase) : y;
        if (series._isTransposed)
		 {
            xvalue = (1 - this._getPointXY(xvalue, series.xAxis.visibleRange, series.xAxis.isInversed)) * (series.xAxis.height);
            yvalue = this._getPointXY(yvalue, series.yAxis.visibleRange, series.yAxis.isInversed) * (series.yAxis.width);
            point.location = { X: yvalue, Y: xvalue };
            return point.location;
        }
        else
        {
            xvalue =  this._getPointXY(xvalue, series.xAxis.visibleRange, series.xAxis.isInversed) * (series.xAxis.width);
            yvalue = (1-this._getPointXY(yvalue, series.yAxis.visibleRange, series.yAxis.isInversed)) * (series.yAxis.height);
            point.location = { X: xvalue, Y: yvalue };
            return point.location;
        }
    },
    _getPointXY: function (value, Range, isInversed) {
         
        var result = 0;
        result = (value - Range.min) / (Range.delta);
     
        return (isInversed) ? (1 - result) : result;   
        
    },
    
    _dateTimeLabelFormat: function (intervalType) {
        var format = 'd';
        if (intervalType.toLowerCase() == 'years')
            format = 'MMM, yyyy';
        else if (intervalType.toLowerCase() == 'months')
            format = 'dd, MMM';
        else if (intervalType.toLowerCase() == 'days')
            format = 'MM/dd/yyyy';
        else if (intervalType.toLowerCase() == 'hours')
            format = 'dd, hh:mm';
        else if (intervalType.toLowerCase() == 'seconds' || intervalType.toLowerCase() == 'minutes')
            format = 'hh:mm:ss';
        else if (intervalType.toLowerCase() == 'milliseconds')
            format = 'hh:mm:ss:tt';
        return format;
    },
    _getFontString: function (fontObj) {
        if (fontObj == null)
            fontObj = {};
        if (!fontObj.FontFamily)
            fontObj.FontFamily = "Arial";
        if (!fontObj.FontStyle)
            fontObj.FontStyle = "Normal";
        if (!fontObj.Size)
            fontObj.Size = "12px";

        return fontObj.FontStyle + " " + fontObj.Size + " " + fontObj.FontFamily;
    },

    _valueToVector: function (axis, value) {
        return this._coefficientToVector(this._valueToPolarCoefficient(axis, value));
    },

    TransformToVisible: function (currentseries, x, y, sender) {
        x = (currentseries.xAxis._valueType == "logarithmic") && x > 0 ? Math.log(x, currentseries.xAxis.logBase) : x;
        y = (currentseries.xAxis._valueType == "logarithmic") && y > 0 ? Math.log(y, currentseries.yAxis.logBase) : y;
        var radius = sender.model.Radius * this._valueToCoefficient(currentseries.yAxis, y);
        //var radius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToPolarCoefficient(currentseries.yAxis, y);
        var point = this._valueToVector(currentseries.xAxis, x);
        return { X: sender.model.centerX + radius * point.X, Y: sender.model.centerY + radius * point.Y };
    },

    Transform3DToVisible:function(currentseries, x, y, sender)
    {
      if (currentseries.xAxis != null && currentseries.yAxis != null)
       {
          var valueType = currentseries.xAxis._valueType.toLowerCase();
          var xIsLogarithmic = (valueType == "logarithmic") ? true :false;
          x = x = xIsLogarithmic && x > 0 ? math.log(x, xlogarithmicBase) : x;
          y = y;

       if (sender.model.requireInvertedAxes)
        {
           var left = sender.model.m_AreaBounds.X;
           var top = sender.model.m_AreaBounds.Y;
            var pointX = left + currentseries.yAxis.width * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, y, sender);
            var pointY = top + currentseries.xAxis.height * (1 - ej.EjSvgRender.utils._valueToCoefficient(currentseries.xAxis, x, sender))
           return { X: pointX, Y: pointY }          
        }
        else
        {
            var left = currentseries.xAxis.x;
            var top = currentseries.yAxis.y;
            var x = left + Math.round(currentseries.xAxis.width * ej.EjSvgRender.utils._valueToCoefficient(currentseries.xAxis, x, sender));
            var y= top + Math.round(currentseries.yAxis.height * (1 - ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis,y, sender)));
            return {X:x, Y:y}
        }
    }
                
    return new Point(0, 0);
  },

    _valueToPolarCoefficient:function(axis, value)
         {
          var start = axis.visibleRange.min;
          var delta;
          var length;
          if (axis._valueType != "category") {
              delta = (axis.visibleRange.max - axis.visibleRange.interval) - axis.visibleRange.min;
              length = axis.visibleLabels.length - 1;
          }
          else {
              delta = axis.visibleRange.delta;
              length = axis.visibleLabels.length;
          }

            result = (value - start) / delta;
            result *= 1 - 1 / (length);

            return axis.isInversed ?  result : 1 - result;
        },

    _coefficientToVector: function (coefficient)
       {
            var angle = Math.PI * (1.5 - 2 * coefficient);

            return  {X:Math.cos(angle), Y: Math.sin(angle)};
      },


    _valueToCoefficient: function (axis, value, sender) {
        if (sender && sender.model.AreaType=='polaraxes') {
            var yvalue = value;
        }
        else
        var yvalue = (axis._valueType && axis._valueType.toLowerCase() == "logarithmic") ?
          ej.EjSvgRender.utils._logBase((value == 0 ? 1 : value), axis.logBase) : value;

        yvalue = (yvalue - axis.visibleRange.min) / (axis.visibleRange.delta);
        
        return (axis.isInversed) ? 1 - yvalue : yvalue;
    },
    _getBoundingClientRect: function (element, sender,series,invertedAxes) {
        var box = element.getBoundingClientRect();
        var position = $("#" + (sender.svgObject.id))[0].getBoundingClientRect();
        var xSeries, ySeries;
        if (invertedAxes) {
            xSeries = this._getTransform(series.xAxis, series.yAxis, true).x;
            ySeries = this._getTransform(series.xAxis, series.yAxis, true).y;
        } else {
            xSeries = this._getTransform(series.xAxis, series.yAxis, false).x;
            ySeries = this._getTransform(series.xAxis, series.yAxis, false).y;
        }
        var x = box.left - (xSeries + position.left);
        var y = box.top - (ySeries + position.top);
        return { x: x, y: y, width: (box.right - box.left), height: (box.bottom - box.top) };
    },
    _minMax: function (value, min, max) {
        return value > max ? max : (value < min ? min : value);
    },
    _inside: function (value, range) {
        if (value === "")
            return false;
        return (value <= range.max) && (value >= range.min);
    },
    _logBase: function (val, base) {
        return Math.log(val) / Math.log(base);
    },
    _correctRect: function (x1, y1, x2, y2) {
        return { X: Math.min(x1, x2), Y: Math.min(y1, y2), Width: Math.abs(x2 - x1), Height: Math.abs(y2 - y1) };
    },
    _getValuebyPoint: function (x, y, series) {

        var xSize = (this.chartObj.model.requireInvertedAxes) ? series.xAxis.height : series.xAxis.width;
        var ySize = (this.chartObj.model.requireInvertedAxes) ? series.yAxis.width : series.yAxis.height;
      
        var xValue = (series.xAxis.isInversed) ? (1 - (x / xSize)) : (x / xSize);
        var yValue = (series.yAxis.isInversed) ? (1 - (y / ySize)) : (y / ySize);

        xValue = xValue * (series.xAxis.visibleRange.delta) + series.xAxis.visibleRange.min;

        yValue = yValue * (series.yAxis.visibleRange.delta) + series.yAxis.visibleRange.min;

        xValue = (series.xAxis._valueType == "logarithmic") ? Math.pow(series.xAxis.logBase, xValue) : xValue;

        yValue = (series.yAxis._valueType == "logarithmic") ? Math.pow(series.yAxis.logBase, yValue) : yValue;

        return { PointX: xValue, PointY: yValue };

    }
   
};
ej.EjSvgRender.chartTransform3D = {
      ToRadial: Math.PI / 180,

      transform3D: function (size) {
          if (!this.vector){
          this.vector=new (new ej.Ej3DRender()).vector3D();
          this.matrixobj = new (new ej.Ej3DRender()).matrix3D();
          this.bsptreeobj = new (new ej.Ej3DRender()).BSPTreeBuilder();
          this.polygon = new (new ej.Ej3DRender()).polygon3D();
          }
        return {
            mViewport: size,
            Rotation: 0,
            Tilt: 0,
            Depth: 0,
            PerspectiveAngle: 0,
            needUpdate:true,
            centeredMatrix: this.matrixobj.getIdentity(),
            Perspective: this.matrixobj.getIdentity(),
            resultMatrix: this.matrixobj.getIdentity(),
            viewMatrix: this.matrixobj.getIdentity(),
            Depth: 0           
          };
     },

    transform: function (trans) {
        this.setCenter(this.vector.vector3D(trans.mViewport.Width / 2, trans.mViewport.Height / 2, trans.Depth / 2), trans);
        this.setViewMatrix(this.matrixobj.transform(0, 0, trans.Depth), trans)
        this.setViewMatrix(this.matrixobj.getMatrixMultiplication(trans.viewMatrix, this.matrixobj.turn(-this.ToRadial * trans.Rotation)), trans);
        this.setViewMatrix(this.matrixobj.getMatrixMultiplication(trans.viewMatrix, this.matrixobj.tilt(-this.ToRadial * trans.Tilt)), trans);
        this.updatePerspective(trans.PerspectiveAngle, trans);
        trans.needUpdate = true;
        
    },

   updatePerspective:function(angle, trans)
    {
       var width = (((trans.mViewport.Width + trans.mViewport.Height) * Math.tan(this.degreeToRadianConverter((180 - Math.abs(angle % 181)) / 2.0))) + (trans.Depth * 2) / 2);
       trans.Perspective[0][0] = width;
       trans.Perspective[1][1] = width;
       trans.Perspective[2][3] = 1;
       trans.Perspective[3][3] = width;
     
   },

   degreeToRadianConverter:function(degree)
   {
           return degree * Math.PI / 180;
   },
   toScreen:function(vector3D, trans)
      {
       vector3D = this.matrixobj.getMatrixVectorMutiple(this.result(trans), vector3D);
       return { x: vector3D.x, y: vector3D.y };
   },

   setViewMatrix: function (value, trans) {
       if (trans.viewMatrix == value) return;
       trans.viewMatrix = value;
       trans.needUpdate = true;
      
   },

   setCenteredMatrix: function (value, trans) {
       if (trans.viewMatrix == value) return;
       trans.centeredMatrix = value;
       trans.needUpdate = true;
       
   },

   result:function(trans) 
     {
       if (!trans.needUpdate) return trans.resultMatrix;
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(this.matrixobj.getInvertal(trans.centeredMatrix),
                       trans.Perspective);
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(trans.resultMatrix, trans.viewMatrix);
       trans.resultMatrix = this.matrixobj.getMatrixMultiplication(trans.resultMatrix, trans.centeredMatrix);
       trans.needUpdate = false;

       return trans.resultMatrix;
    },


   setCenter:function(center, trans)
    {
        trans.centeredMatrix = this.matrixobj.transform(-center.x, -center.y, -center.z);
        trans.needUpdate = true;
   },
   toPlane:function(point, plane, trans)
     {
     var vec1 = this.vector.vector3D(point.x, point.y, 0);
     var vec2 = this.vector.vector3DPlus(vec1, vector.vector3D(0, 0, 1));

     vec1 = this.vector.vector3DMultiply(trans.centeredMatrix , vec1);
     vec2 = this.vector.vector3DMultiply(trans.centeredMatrix , vec2);

     vec1 = this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.Perspective) , vec1);
     vec2 =  this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.Perspective) , vec2);

     vec1 = this.polygon.getPoint(vec1, this.vector.vector3DMinus(vec2 - vec1));

     vec1 = this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.viewMatrix), vec1);
     vec1 =  this.vector.vector3DMultiply(this.matrixobj.getInvertal(trans.centeredMatrix), vec1);

     return vec1;
    }

};
ej.EjSvgRender.chartSymbol =
{
    _drawSeriesType: function (location, symbolStyle, sender) {
        var seriesType = sender.model.series[symbolStyle.SeriesIndex].type.toLowerCase();
        var trendlinetype = sender.legendItem.drawType;
        seriesType = ej.util.isNullOrUndefined(trendlinetype) ? seriesType : trendlinetype;
        switch (seriesType) {
            case ej.datavisualization.Chart.Type.Line: {
                var line;
                if(ej.util.isNullOrUndefined(trendlinetype))
                     line = this._drawLine(location, symbolStyle, sender, sender.gLegendItemEle);
                else
                    line = this._drawStraightLine(location, symbolStyle, sender, sender.gLegendItemEle);
                return line;
            }
                
            case ej.datavisualization.Chart.Type.StepLine:
                return this._drawStepLine(location, symbolStyle, sender, sender.gLegendItemEle);
                 
            case ej.datavisualization.Chart.Type.StackingArea:
            case ej.datavisualization.Chart.Type.StackingArea100:
            case ej.datavisualization.Chart.Type.Area:
            case ej.datavisualization.Chart.Type.RangeArea:
                return this._drawArea(location, symbolStyle, sender, sender.gLegendItemEle);
                 
            case ej.datavisualization.Chart.Type.StepArea:
                return this._drawStepArea(location, symbolStyle, sender, sender.gLegendItemEle);
                 
            case ej.datavisualization.Chart.Type.Bar:
            case ej.datavisualization.Chart.Type.StackingBar100:
            case ej.datavisualization.Chart.Type.StackingBar:
                return this._drawBar(location, symbolStyle, sender, sender.gLegendItemEle);
                
            case ej.datavisualization.Chart.Type.Pie:
                return this._drawPie(location, symbolStyle, sender, sender.gLegendItemEle);
                
            case ej.datavisualization.Chart.Type.Doughnut:
                return this._drawDoughnut(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Hilo:
                return this._drawHilo(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.HiloOpenClose:
                return this._drawHiloOpenClose(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Candle:
                return this._drawCandle(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Pyramid:
                return this._drawPyramid(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Funnel:
                return this._drawFunnel(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Spline:
                return this._drawSpline(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.SplineArea:
                return this._drawSplineArea(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.RangeColumn:
                return this._drawRangeColumn(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Bubble:
            case ej.datavisualization.Chart.Type.Scatter:
                return this._drawCircle(location, symbolStyle, sender, sender.gLegendItemEle);

            case ej.datavisualization.Chart.Type.Column:
            case ej.datavisualization.Chart.Type.StackingColumn:
            case ej.datavisualization.Chart.Type.StackingColumn100:
                return this._drawColumn(location, symbolStyle, sender, sender.gLegendItemEle);
                
            default:
                return this._drawRectangle(location, symbolStyle, sender, sender.gLegendItemEle);
                 
        }

    },

    _drawCircle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
        			
        var options = {
            'id': symbolStyle.ID, 'cx': location.startX, 'cy': location.startY, 'r': radius,
            'fill': symbolStyle.Style.Color, 'stroke-width': symbolStyle.Style.BorderWidth, 'stroke': symbolStyle.Style.BorderColor, 'opacity': symbolStyle.Style.Opacity, 'visibility': symbolStyle.Style.Visibility, 'lgndCtx': symbolStyle.context
        };

        svgRender.drawCircle(options, element);

        return (location.startX - radius);
    },

    _drawLeftArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY + (style.height / 4)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility':symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },


    _drawRightArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + style.width) + " " + (location.startY) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2)) + (style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY + (style.height / 4)) + " z";       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility':symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);


    },


    _drawUpArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);


    },

    _drawDownArrow: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2) - (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX + (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX)) + " " + (location.startY - (style.height / 2) + (style.height)) + " " + "L" + " " + ((location.startX - (style.width / 2))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2) + (style.height / 2)) + " " + "L" + " " + ((location.startX - (style.width / 2) + (style.width / 4))) + " " + (location.startY - (style.height / 2)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);

    },

    _drawCross: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " "+ (location.startX + (style.width / 2)) + " " + (location.startY) + " " +
        "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawHorizLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY);
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },
    _drawVertLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'opacity': symbolStyle.Style.Opacity,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };


        svgRender.drawPath(options, element);

    },

    _drawTriangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawInvertedTriangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
         path = "M" + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2)) + " "
             + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " "
             + "L" + " " + (location.startX - (style.width / 2)) + " " + (location.startY - (style.height / 2)) + " "
             + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2)) + " z";
        var x = location.startX;
        var y = location.startY;
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawHexagon: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawWedge: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX - style.width) + " " + (location.startY) + " " + "L" + " " + (location.startX + style.width) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (3 * (style.width / 4))) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX - style.width) + " " + (location.startY) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawPentagon: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var eq = 72;
        var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        for (var i = 0; i <= 5; i++) {
            var deg = i * eq;
            var rad = (Math.PI / 180) * deg;
            var x1 = radius * Math.cos(rad);
            var y1 = radius * Math.sin(rad);
            if (i == 0)
                sb.append("M" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");

            else
                sb.append("L" + " " + (location.startX + x1) + " " + (location.startY + y1) + " ");


        }
        sb.append("z");
        var path = sb.toString();
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawStar: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var style = symbolStyle.ShapeSize;
        var svgObj = sender.svgObject;

        var path;
        path = "M" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 6)) + " " + "L" + " " + (location.startX + (-style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 3)) + " " + (location.startY + (-style.height / 2)) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },

    _drawRectangle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2))+ " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);
    },

    _drawTrapezoid: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);

    },

    _drawDiamond: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var path;
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY) + " z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'd': path
        };

        svgRender.drawPath(options, element);


    },
    _drawEllipse: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;

        var x = location.startX;
        var y = location.startY;
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'lgndCtx': symbolStyle.context,
            'cx': x,
            'cy': y,
            'rx': style.width,
            'ry': style.height / 2
        };


        svgRender.drawEllipse(options, element);


    },
    _drawImage: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var x = location.startX + (-style.width / 2);
        var y = location.startY + (-style.width / 2);
        var width = style.width;
        var height = style.height;
        var options = { 'id': svgObj.id + '_image' + symbolStyle.PointIndex, 'height': height, 'width': width, 'href': symbolStyle.Imageurl,
            'x': x, 'y': y, 'visibility': 'visible', 'lgndCtx': symbolStyle.context
        };

        svgRender.drawImage(options, element);


    },
	
	 _drawStraightLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;

        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10)) + " " +
                "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth * 2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };
        svgRender.drawPath(options, element);
        return (location.startX + (-style.width / 2) + (-elementspace / 4));
    },
	
    _drawLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "M" + " " + (location.startX + Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "L" + " " + (location.startX + (style.width/2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth*2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };
        var circlepath = "M" + " " + (location.startX - Math.floor(style.width / 3)) + " " + (location.startY + (style.height / 10)) + " " + "a " + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + 2 * (Math.floor(style.width / 3)) + " " + 0 + " " + "a" + (Math.floor(style.width / 3)) + " " + (Math.floor(style.width / 3)) + " " + 0 + " " + 1 + " " + 0 + " " + (-2 * (Math.floor(style.width / 3))) + " " + 0;
        var circleoptions = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth*2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': circlepath,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);
        svgRender.drawPath(circleoptions, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 4));

    },
    _drawColumn: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
         path = "M" + " " + (location.startX - 3 * (style.width / 5)) + " " + (location.startY - (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (-style.width / 10)) + " " + (location.startY - (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (-style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX - 3 * (style.height / 5)) + " " + (location.startY + (style.height / 2)) + " " + "Z" + " " + "M" + " " + (location.startX + (-style.width / 10) - (style.width / 20)) + " " + (location.startY - (style.height / 4) - (elementspace / 2)) + " " + "L" + " " + (location.startX + (style.width / 10) + (style.width / 20)) + " " + (location.startY - (style.height / 4) - (elementspace / 2)) + " " + "L" + " " + (location.startX + (style.width / 10) + (style.width / 20)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 10) - (style.width / 20)) + " " + (location.startY + (style.height / 2)) + " " + "Z" + " " + "M" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + 3 * (style.width / 5)) + " " + (location.startY) + " " + "L" + " " + (location.startX + 3 * (style.width / 5)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "Z";
      
         var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
             'lgndCtx': true
        };

        svgRender.drawPath(options, element);

         return (location.startX - 3 * (style.width / 5));

    },
    _drawRangeColumn: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
        path = "M" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY - (style.height / 2) - (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "Z";
       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 5));

    },

    _drawBar: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY - 3 * (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY - 3 * (style.height / 5)) + " " + "L" + " " + (location.startX + 3 * (style.width / 10)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "Z" + " " + "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY - (style.height / 5) + (elementspace / 20)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY - (style.height / 5) + (elementspace / 20)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 10) + (elementspace / 20)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 10) + (elementspace / 20)) + " " + "Z" + " " + "M" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height/5) + (elementspace / 10)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height/5) + (elementspace / 10)) + " " + "L" + " " + (location.startX + (-style.width / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/10)) + " " + "L" + " " + (location.startX - (style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/10)) + " " + "Z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 4));

    },
   
    _drawStepLine: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
       
        path = "M" + " " + (location.startX + (-style.width / 2) - (elementspace/4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 10)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (-style.width / 10)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)) + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + "L" + "" + (location.startX + (style.width/2) + (elementspace/4)) + " " + (location.startY + (style.height / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) - (elementspace / 4));

    },
    _drawSpline: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;        
        path = "M" + " " + (location.startX - (style.width / 2)) + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + location.startX + " " + (location.startY - style.height) + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "M" + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2));		
        var options = {
            'id': symbolStyle.ID,
             'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth*2,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX - (style.width / 2));

    },
    _drawSplineArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var path;       
        path = "M" + " " + (location.startX - (style.width / 2)) + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + location.startX + " " + (location.startY - style.height) + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " +  "Z" +" " + "M" + " " + location.startX + " " + (location.startY + (style.height / 5)) + " " + "Q" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)) + " " + (location.startX + (style.width / 2)) + " " + (location.startY - (style.height / 2)) +" " + 0 +" " +" Z";
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX - (style.width / 2));

    },
    _drawArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX - (style.width / 2) - (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 4) + (-elementspace / 8)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4) + (elementspace / 8)) + " " + (location.startY + (-style.height / 2) + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.height / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX - (style.width / 2) - (elementspace / 4));

    },
    _drawStepArea: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;

        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "L" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (-style.height / 2)) + " " + "L" + " " + (location.startX - (style.width / 4)) + " " + (location.startY - (style.height / 2)) + " " + "L" + " " + (location.startX - (style.width / 4)) + " " + (location.startY - (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startY - (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 4)) + " " + (location.startX) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)) + " " + "Z";       
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 4));

    },
    _drawPyramid: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;        
       path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (-elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + (style.height / 2)+(elementspace/8)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 4));


    },
    _drawFunnel: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX + (-style.width / 2) + (-elementspace / 5)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (-style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "L" + " " + (location.startX + (style.width / 5)) + " " + (location.startY + (style.height / 4)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 5)) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "Z";
      
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2) + (-elementspace / 5));

    },
    _drawCandle: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;       
       
        path = "M" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "L" + " " + (location.startX + (-style.width / 2)) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "Z" + " " + "M" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (elementspace / 8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (-style.height / 2) + (-elementspace / 4)) + " " + "M" + " " + (location.startX) + " " + (location.startY + (style.height / 4) + (elementspace / 8)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 4) + (elementspace / 2));
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (-style.width / 2));

    },
    _drawHilo: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var path;
        path = "M" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (style.height / 2)+(elementspace/4)) + " " + "L" + " " + (location.startX + (style.width / 2)) + " " + (location.startY + (-style.height / 2)+(-elementspace/4));        
        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return (location.startX + (style.width / 2));

    },
    _drawHiloOpenClose: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var cartesian = polarToCartesian(18, 12, 4.0710678118654755, 270);
        var elementspace = symbolStyle.ElementSpace;
        var path;        
        
        path = "M" + " " + (location.startX) + " " + (location.startY - 3 * (style.height / 10)) + " " + "L" + " " + (location.startX - (style.width / 2) - (elementspace / 4)) + " " + (location.startY - 3 * (style.height / 10)) + " " + "M" + " " + (location.startX) + " " + (location.startY - (style.height / 2) - (elementspace / 4)) + " " + "L" + " " + (location.startX) + " " + (location.startY + (style.height / 2) + (elementspace / 4)) + " " + "M" + " " + (location.startX) + " " + (location.startY + 3 * (style.height / 10)) + " " + "L" + " " + (location.startX + (style.width / 2) + (elementspace / 4)) + " " + (location.startY + 3 * (style.height / 10));
        var options = {
            'id': symbolStyle.ID,
            'fill': "transparent",
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.Color,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);

        return ((location.startX) - (style.width / 2));

    },

    _drawDoughnut: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var radius = Math.sqrt(style.height * style.width) / 2;
        radius = radius + (elementspace / 5);
        var cartesianlarge = polarToCartesian(location.startX, location.startY, radius, 270);
        var cartesiansmall = polarToCartesian(location.startX + (style.width / 10), location.startY, radius, 270);
        var elementspace = symbolStyle.ElementSpace;
        path = "M" + " " + location.startX + " " + location.startY + " " + "L" + " " + (location.startX + radius) + " " + (location.startY) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 1 + " " + 1 + " " + cartesianlarge[0] + " " + cartesianlarge[1] + " " + "Z" + " " + "M" + " " + (location.startX + (style.width / 10)) + " " + (location.startY - (style.height / 10)) + " " + "L" + (location.startX + (radius)) + " " + (location.startY - style.height / 10) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 0 + " " + 0 + " " + cartesiansmall[0] + " " + cartesiansmall[1] + " " + "Z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };
        var circle = {
            'id': symbolStyle.ID, 'cx': location.startX, 'cy': location.startY, 'r': (radius/2),'lgndCtx': true,
            'fill': "white", 'stroke-width': symbolStyle.Style.BorderWidth, 'stroke': symbolStyle.Style.BorderColor, 'opacity': symbolStyle.Style.Opacity, 'visibility': symbolStyle.Style.Visibility
        };

        svgRender.drawPath(options, element);
        svgRender.drawCircle(circle, element);

        return (location.startX-radius);
    },

    _drawPie: function (location, symbolStyle, sender, element) {
        var svgRender = sender.svgRenderer;
        var svgObj = sender.svgObject;
        var style = symbolStyle.ShapeSize;
        var elementspace = symbolStyle.ElementSpace;
        var radius = Math.sqrt(style.height * style.width) / 2;
        radius = radius + (elementspace / 5);
        var cartesianlarge = polarToCartesian(location.startX, location.startY, radius, 270);
        var cartesiansmall = polarToCartesian(location.startX + (style.width / 10), location.startY, radius, 270);
        var elementspace = symbolStyle.ElementSpace;
        path = "M" + " " + location.startX + " " + location.startY + " " + "L" + " " + (location.startX + radius) + " " + (location.startY) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 1 + " " + 1 + " " + cartesianlarge[0] + " " + cartesianlarge[1] + " " + "Z" + " " + "M" + " " + (location.startX + (style.width / 10)) + " " + (location.startY - (style.height / 10)) + " " + "L" + (location.startX + (radius)) + " " + (location.startY - style.height / 10) + " " + "A" + " " + (radius) + " " + (radius) + " " + 0 + " " + 0 + " " + 0 + " " + cartesiansmall[0] + " " + cartesiansmall[1] + " " + "Z";

        var options = {
            'id': symbolStyle.ID,
            'fill': symbolStyle.Style.Color,
            'stroke-width': symbolStyle.Style.BorderWidth,
            'stroke': symbolStyle.Style.BorderColor,
            'opacity': symbolStyle.Style.Opacity,
            'visibility': symbolStyle.Style.Visibility,
            'd': path,
            'lgndCtx': true
        };

        svgRender.drawPath(options, element);
        return (location.startX - radius);
    }

};
function polarToCartesian(startX, startY, radius, angleInDegrees) {
    var angleInRadians = angleInDegrees * Math.PI / 180.0;
    var x = startX + radius * Math.cos(angleInRadians);
    var y = startY + radius * Math.sin(angleInRadians);
    return [x, y];
}
ej.EjSvgRender.seriesPalette = {
    defaultMetro: ["#E94649", "#F6B53F", "#6FAAB0", "#C4C24A", "#FB954F", "#005277", "#8BC652", "#69D2E7", "#E27F2D", "#6A4B82"],
    defaultGradient:
        {
            borderColors: ["#F34649", "#F6D321", "#6EB9B0", "#CBC26A", "#FBAF4F", "#E2CDB1", "#FFC0B7", "#68E1E6", "#E1A62D", "#9C6EBF"],
            seriesColors: [[{ color: "#F34649", colorStop: "0%" }, { color: "#B74143", colorStop: "100%"}],
     [{ color: "#F6D321", colorStop: "0%" }, { color: "#F6AE26", colorStop: "100%"}],
     [{ color: "#6EB9B0", colorStop: "0%" }, { color: "#3F77BD", colorStop: "100%"}],
     [{ color: "#CBC26A", colorStop: "0%" }, { color: "#9AAD21", colorStop: "100%"}],
     [{ color: "#FBAF4F", colorStop: "0%" }, { color: "#F07542", colorStop: "100%"}],
     [{ color: "#E2CDB1", colorStop: "0%" }, { color: "#AAA089", colorStop: "100%"}],
     [{ color: "#8BC652", colorStop: "0%" }, { color: "#6F9E41", colorStop: "100%"}],
     [{ color: "#68E1E6", colorStop: "0%" }, { color: "#3D9CBE", colorStop: "100%"}],
     [{ color: "#E1A62D", colorStop: "0%" }, { color: "#B66824", colorStop: "100%"}],
     [{ color: "#9C6EBF", colorStop: "0%"}], [{ color: "#593F6D", colorStop: "100%"}]]
        },
    blueMetro: ["#005378", "#006691", "#007EB5", "#0D97D4", "#00AEFF", "#14B9FF", "#54CCFF", "#87DBFF", "#ADE5FF", "#C5EDFF"],
    blueGradient:
     {
         seriesColors: [[{ color: "#005277", colorStop: "0%" }, { color: "#00304F", colorStop: "100%"}],
            [{ color: "#006590", colorStop: "0%" }, { color: "#004068", colorStop: "100%"}],
            [{ color: "#007DB4", colorStop: "0%" }, { color: "#00558B", colorStop: "100%"}],
            [{ color: "#0D97D4", colorStop: "0%" }, { color: "#057FC7", colorStop: "100%"}],
            [{ color: "#00ADFE", colorStop: "0%" }, { color: "#008BE9", colorStop: "100%"}],
            [{ color: "#14B8FE", colorStop: "0%" }, { color: "#0798EB", colorStop: "100%"}],
            [{ color: "#53CBFF", colorStop: "0%" }, { color: "#35AFEB", colorStop: "100%"}],
            [{ color: "#86DAFF", colorStop: "0%" }, { color: "#64C0EC", colorStop: "100%"}],
            [{ color: "#ACE5FF", colorStop: "0%" }, { color: "#8DCEED", colorStop: "100%"}],
            [{ color: "#C4ECFF", colorStop: "0%"}], [{ color: "#A3D1E6", colorStop: "100%"}]],
         borderColors: ["#005277", "#006590", "#007DB4", "#0D97D4", "#00ADFE", "#14B8FE", "#53CBFF", "#86DAFF", "#ACE5FF", "#C4ECFF"]
     },
    greenMetro: ["#496612", "#597B15", "#709A1B", "#87B62A", "#9AD926", "#A6DC37", "#BCE654", "#C8E780", "#D5EFA5", "#E2F3BE"],
    greenGradient:
     {

         seriesColors: [[{ color: "#5C7F16", colorStop: "0%" }, { color: "#384C08", colorStop: "100%"}],
            [{ color: "#6A9319", colorStop: "0%" }, { color: "#486009", colorStop: "100%"}],
            [{ color: "#739D1C", colorStop: "0%" }, { color: "#57760B", colorStop: "100%"}],
            [{ color: "#90B546", colorStop: "0%" }, { color: "#6E9215", colorStop: "100%"}],
            [{ color: "#9AD826", colorStop: "0%" }, { color: "#75A010", colorStop: "100%"}],
            [{ color: "#A5DB36", colorStop: "0%" }, { color: "#8EB91D", colorStop: "100%"}],
            [{ color: "#BBE554", colorStop: "0%" }, { color: "#A4C849", colorStop: "100%"}],
            [{ color: "#C8E780", colorStop: "0%" }, { color: "#B4D072", colorStop: "100%"}],
            [{ color: "#D4EEA5", colorStop: "0%" }, { color: "#BFD593", colorStop: "100%"}],
            [{ color: "#E1F2BD", colorStop: "0%"}], [{ color: "#C8D7A8", colorStop: "100%"}]],
         borderColors: ["#5C7F16", "#6A9319", "#739D1C", "#90B546", "#9AD826", "#A5DB36", "#BBE554", "#C8E780", "#D4EEA5", "#E1F2BD"]
     },

    sandleMetro: ["#6C450C", "#82520D", "#A36812", "#C07F1F", "#E69719", "#E89A2B", "#EEB564", "#F3CB93", "#F7DEB4", "#F9E6CA"],
    sandleGradient:
     {


         seriesColors: [[{ color: "#7F602F", colorStop: "0%" }, { color: "#512D04", colorStop: "100%"}],
            [{ color: "#986827", colorStop: "0%" }, { color: "#673803", colorStop: "100%"}],
            [{ color: "#A16C1F", colorStop: "0%" }, { color: "#8A4B05", colorStop: "100%"}],
            [{ color: "#BF812A", colorStop: "0%" }, { color: "#AD630D", colorStop: "100%"}],
            [{ color: "#E49519", colorStop: "0%" }, { color: "#B86607", colorStop: "100%"}],
            [{ color: "#E7992A", colorStop: "0%" }, { color: "#D7780D", colorStop: "100%"}],
            [{ color: "#EDB463", colorStop: "0%" }, { color: "#D98F31", colorStop: "100%"}],
            [{ color: "#F2CA92", colorStop: "0%" }, { color: "#DAAC6F", colorStop: "100%"}],
            [{ color: "#F6DDB3", colorStop: "0%" }, { color: "#DABE8F", colorStop: "100%"}],
            [{ color: "#F8E5C9", colorStop: "0%"}], [{ color: "#DDBE92", colorStop: "100%"}]],
         borderColors: ["#7F602F", "#986827", "#A16C1F", "#BF812A", "#E49519", "#E7992A", "#EDB463", "#F2CA92", "#F6DDB3", "#F8E5C9"]

     }
};

ej.EjSvgRender.themes = {
    flatlight:
    {
        background: 'transparent',
        legend:
        {
            font: { color: "#282828"},
            title:
            {
               font: { color: "#282828"}
             }
        },
        title:
        {
            font: { color: '#565656'},
            subTitle:
                {
                    font: { color: '#565656'}
                }
        },
        primaryXAxis:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
        secondaryX:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D' ,width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
        },

        primaryYAxis:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', borderWidth: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
        secondaryY:
        {
            majorGridLines:
            {
                color: "#DFDFDF"
            },
            majorTickLines:
            {
                color: "#8E8E8E"
            },
            minorGridLines:
            {
                color: "#DFDFDF"
            },
            minorTickLines:
            {
                color: "#8E8E8E"
            },
            axisLine: { color: '#8E8E8E' },
            font: { color: '#282828' },
            title:
            {
                font: { color: '#282828' }
            },
            crosshairLabel: { rx: 0, ry: 0, border: { color: '#3D3D3D', width: 1 }, fill: '#3D3D3D', font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB' } }
        },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#565656' } }
}
           },
        crosshair:
        {
            line:
            {
                width: 1,
                color: 'Black'
            }
        },
        
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
        colors: ej.EjSvgRender.seriesPalette.defaultMetro

    },

    flatdark:
        {
            background: '#111111',

            legend:
            {
                font: { color: "#C9C9C9"},
                title:
                {
                  font: { color: "#C9C9C9"}
                }
            },
            title:
            {
                font: { color: '#C9C9C9'},
                subTitle:
                {
                    font: { color: '#C9C9C9'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 }, 
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {   rx: 0, ry: 0, fill: '#B5B5B5', border:{color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border:{color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444' } }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: { rx: 0, ry: 0, fill: '#B5B5B5', border: { color: '#B5B5B5', width: 1 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#C9C9C9' } }
}
           },
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'White'
                 }
             },
            
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultMetro,
            colors: ej.EjSvgRender.seriesPalette.defaultMetro

        },


    gradientlight:
        {
            background: 'transparent',
            legend:
            {
                font: { color: "#282828"},
                title:
                {
                    font: { color: "#282828"}
                }
            },
            title:
            {
                font: { color: '#565656'},
                subTitle:
                {
                    font: { color: '#565656'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#DFDFDF"
                 },
                 majorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 minorGridLines:
                  {
                      color: "#DFDFDF"
                  },
                 minorTickLines:
                  {
                      color: "#8E8E8E"
                  },
                 axisLine: { color: '#8E8E8E' },
                 font: { color: '#282828' },
                 title:
                 {

                     font: { color: '#282828' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#3D3D3D',border:{color: '#3D3D3D',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#DBDBDB'} }
             },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#565656' } }
}
           },
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'Black'
                 }
             },
            
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

        },

    gradientdark:
        {
            background: '#111111',
            legend:
            {
                font: { color: "#C9C9C9"},
                title:
               {
                   font: { color: "#C9C9C9"}
               }
            },
            title:
            {
                font: { color: '#C9C9C9'},
                subTitle:
                {
                    font: { color: '#C9C9C9'}
                }
            },
            primaryXAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            secondaryX:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            primaryYAxis:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
            secondaryY:
             {
                 majorGridLines:
                 {
                     color: "#333333"
                 },
                 majorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 minorGridLines:
                  {
                      color: "#333333"
                  },
                 minorTickLines:
                  {
                      color: "#AAAAAA"
                  },
                 axisLine: { color: '#AAAAAA' },
                 font: { color: '#C9C9C9' },
                 title:
                 {

                     font: { color: '#C9C9C9' }
                 },
                 crosshairLabel: {rx: 3, ry: 3, fill: '#B5B5B5',border:{color: '#B5B5B5',  width: 2 }, font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', size: '13px', fontWeight: 'regular', opacity: 1, color: '#444444'} }
             },
     commonSeriesOptions:
           {
marker:{
               dataLabel: { font: { color: '#C9C9C9' } }
}
           },
            crosshair:
             {
                 line:
                 {
                     width: 1,
                     color: 'White'
                 }
             },
             
            seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.defaultGradient.borderColors,
            colors: ej.EjSvgRender.seriesPalette.defaultGradient.seriesColors

        }
};
$.extend(ej.EjSvgRender.themes, {

    "azure":
    {

        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
        colors: ej.EjSvgRender.seriesPalette.blueMetro
    },

    "azuredark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueMetro,
        colors: ej.EjSvgRender.seriesPalette.blueMetro
    },
    "gradient-azure":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
    },

    "gradient-azuredark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.blueGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.blueGradient.seriesColors
    },

    "lime":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
        colors: ej.EjSvgRender.seriesPalette.greenMetro
    },

    "limedark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenMetro,
        colors: ej.EjSvgRender.seriesPalette.greenMetro
    },
    "gradient-lime":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
    },

    "gradient-limedark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.greenGradient.borderColors,
        colors: ej.EjSvgRender.seriesPalette.greenGradient.seriesColors
    },
    "saffron":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
        colors: ej.EjSvgRender.seriesPalette.sandleMetro
    },

    "saffrondark":
    {
        seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleMetro,
        colors: ej.EjSvgRender.seriesPalette.sandleMetro
    },
    "gradient-saffron":
{
    seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
    colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
},

    "gradient-saffrondark":
{
    seriesBorderDefaultColors: ej.EjSvgRender.seriesPalette.sandleGradient.borderColors,
    colors: ej.EjSvgRender.seriesPalette.sandleGradient.seriesColors
}
});
})(jQuery);;
ej.axisTypes = {};

ej.EjAxisRenderer = function () {
};
var _sideBySeriesPadding = function (chartObj, start, end, axis) {
    var visibleSeries = ej.DataManager(chartObj.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
    var isPadding = false;
    var data, startPadding, endPadding;
    var visibleSeriesLength = visibleSeries.length;
	var axisOrientation = axis.orientation.toLowerCase();
    for (var i = 0; i < visibleSeriesLength; i++) {
        var type = visibleSeries[i].type.toLowerCase();
        if (((type.indexOf("column") != -1 || type.indexOf("bar") != -1 || type == "candle" || type.indexOf("hilo") != -1)))
        {
            isPadding = true;
            break;
        }

    }
    if (chartObj.model.AreaType != "polaraxes" && isPadding) {
        axis.m_minPointsDelta = undefined;
        var padding = ej.EjSvgRender.utils.getMinPointsDelta(axis, chartObj, start) * 0.5;
        start = start - padding;
        end = end + padding;
        axis.padding = padding;
    }
    else
		axis.padding=0;
    return data = { startPadding: start, endPadding: end };
};
ej.EjStripline = function (chartobj) {
    this.chart = chartobj;
};
(function ($) {
    ej.EjAxisRenderer.prototype = {

        //Calculate min/max value for series
        _seriesMinMax: function (chartObj, axis, seriesCollection, type) {
            var orientation = axis.orientation.toLowerCase();
            var seriesLength = seriesCollection.length;
            if ((seriesLength > 0 && seriesCollection[0].points.length > 0) || this.chartObj.model.indicators[0]._points.length>0) {

                var maxX, minY, minX, maxY;
                var maxMinXValue = { maxX: null, minX: null };
                var maxMinYValue = { maxY: null, minY: null };
                var xMax = maxMinXValue.maxX;
                var xMin = maxMinXValue.minX;
                var yMax = maxMinYValue.maxY;
                var yMin = maxMinYValue.minY;
                var seriesVisibility,pointsLength,visibleLength,seriesType;

                for (var i = 0; i < seriesLength; i++) {
                    seriesVisibility = seriesCollection[i].visibility.toLowerCase();
                    pointsLength = seriesCollection[i].points.length;
                    if (seriesVisibility === 'visible') {
                        seriesCollection[i]._visiblePoints = [];
                        for (var j = 0; j < pointsLength; j++) {
                            var point = seriesCollection[i].points[j];
                            if (!ej.util.isNullOrUndefined(point.x) && !ej.util.isNullOrUndefined(point.y) || point.isEmpty == false)
                                seriesCollection[i]._visiblePoints.push(point);
                        }
                    }
                }

                for (var i = 0; i < seriesLength; i++) {
                    visibleLength = seriesCollection[i]._visiblePoints.length;
                    if (visibleLength > 0) {
                        seriesVisibility = seriesCollection[i].visibility.toLowerCase();
                        pointsLength = seriesCollection[i].points.length;
                        if (seriesVisibility === 'visible' && pointsLength > 0) {
                            if ((orientation == "horizontal" && !(chartObj.model.requireInvertedAxes)) || (orientation == "vertical" && chartObj.model.requireInvertedAxes) ) {
                                minX = maxX = seriesCollection[i]._visiblePoints[0].xValue;
                                for (var j = 0; j < visibleLength; j++) {
                                    var currentPoint = seriesCollection[i]._visiblePoints[j].xValue;
                                    if (minX > currentPoint)
                                        minX = currentPoint;
                                    if (maxX < currentPoint)
                                        maxX = currentPoint;
                                }
                                seriesCollection[i].minX = minX;
                                seriesCollection[i].maxX = maxX;

                                if (ej.util.isNullOrUndefined(xMin) && visibleLength > 0) {
                                    xMax = seriesCollection[i].maxX;
                                    xMin = seriesCollection[i].minX;
                                }
                                if (xMin > seriesCollection[i].minX) {
                                    xMin = seriesCollection[i].minX;
                                }
                                if (xMax < seriesCollection[i].maxX) {
                                    xMax = seriesCollection[i].maxX;
                                }
                            }
                            else {
                                maxY = minY = seriesCollection[i]._visiblePoints[0].YValues[0];
                                for (var j = 0; j < visibleLength; j++) {
                                    var currentPoint = seriesCollection[i]._visiblePoints[j];
                                    if (seriesCollection[i]._hiloTypes) {
                                        for (var y = 0; y < 2; y++) {
                                            if (minY > currentPoint.YValues[y])
                                                minY = currentPoint.YValues[y];
                                            if (maxY < currentPoint.YValues[y])
                                                maxY = currentPoint.YValues[y];
                                        }
                                    } else {
                                        if (minY > currentPoint.YValues[0])
                                            minY = currentPoint.YValues[0];
                                        if (maxY < currentPoint.YValues[0])
                                            maxY = currentPoint.YValues[0];
                                    }
                                }
                                seriesCollection[i].minY = minY;
                                seriesCollection[i].maxY = maxY;

                                if (ej.util.isNullOrUndefined(yMin) && visibleLength > 0) {
                                    yMax = seriesCollection[i].maxY;
                                    yMin = seriesCollection[i].minY;
                                }
                                if (yMin > seriesCollection[i].minY) {
                                    yMin = seriesCollection[i].minY;
                                }
                                if (yMax < seriesCollection[i].maxY) {
                                    yMax = seriesCollection[i].maxY;
                                }

                                // Finding Min and Max for Column and Bar series
                                seriesType = seriesCollection[i].type
                                if (seriesType == 'column' || seriesType == 'bar')
                                    yMin = (yMin < 0) ? yMin : 0;
                            }
                        }
                    }
                }
                if ((orientation == "horizontal" && !(chartObj.model.requireInvertedAxes)) || (orientation == "vertical" && chartObj.model.requireInvertedAxes)) {

                    // Find min/max for indicator series
                    if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                        var value = this.chartObj.model.indicatorRange[axis.name];
                        if (yMin > value.min || yMin == null)
                            yMin = value.min;
                        if (yMax < value.max || yMax == null)
                            yMax = value.max;                       
                    }

                    if (xMax == xMin)
                        xMax += 1;

                    if (type == "double") {
                        var data = _sideBySeriesPadding(chartObj, xMin, xMax, axis);
                        xMin = data.startPadding;
                        xMax = data.endPadding;
                        deltaX = xMax - xMin;
                        axis.range = this._getDoubleAutoRange(axis, xMin, xMax, deltaX);    
                    }                        
                    else
                        axis.range = this._getLogAutoRange(axis, xMin, xMax, deltaX);

                }
                else {

                    // Find min/max for stacked series
                    if (this.chartObj.model.stackedValue[axis.name]) {
                        var value = this.chartObj.model.stackedValue[axis.name];
                        if (yMin > value.min)
                            yMin = value.min;
                        if (yMax < value.max)
                            yMax = value.max;
                    }

                    // Find min/max for indicator series
                    if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                        value = this.chartObj.model.indicatorRange[axis.name];
                        
                        if (yMin > value.min || yMin == null)
                            yMin = value.min;
                        if (yMax < value.max || yMax == null)
                            yMax = value.max;                        
                    }

                    if (yMax == yMin) {                   // max == min
                        if (yMax < 0)                                  // value less than 0
                            yMax = 0;
                        else if (yMax <= 1 && yMax > 0)   // value between 0 and 1
                            yMin = 0;
                        else {                                                      // value greater than 1
                            yMin = 0;
                            yMax += 1;
                        }
                    }

                    var deltaY = yMax - yMin;

                    if (type == "double")
                        axis.range = this._getDoubleAutoRange(axis, yMin, yMax, deltaY)
                    else
                        axis.range = this._getLogAutoRange(axis, yMin, yMax, deltaY)

                }
            }
            else {
                if (!axis.setRange){
                    axis.range = { min: 0, max: 5, interval: 1, Delta: 4 };
				}
				else {
			        min=(ej.util.isNullOrUndefined(axis.range.min))?0:axis.range.min;
			        max=(ej.util.isNullOrUndefined(axis.range.max))?5:axis.range.max;
			        delta=max-min;
			        axis.range=this._getDoubleAutoRange(axis,min,max,delta)
				}
            }
        },

        //Calculate ranges for Trendline series
        _calculateTrendLineRange: function (chart, axis, seriesCollection) {            
            var series, trendlines, trendline, count, serLength, trendLength, trendlineType;
            var serLength = seriesCollection.length;
            for (var m = 0; m < serLength; m++) {
                series = seriesCollection[m];
                trendlines = series.trendlines;
                if (chart.model.AreaType == "cartesianaxes" && !chart.model.requireInvertedAxes && (axis.name == series._xAxisName || axis.name == series._yAxisName)) {
                    trendLength = trendlines.length;
                    for (var i = 0; i < trendLength; i++) {
                        trendline = trendlines[i];
                        if (trendline.visibility.toLowerCase() === 'visible') {                            
                            
                            if (axis.name == series._xAxisName) {
                                trendlineType = trendline.type.toLowerCase();
                                options = new ej.trendlineTypes[trendlineType]();
                                options.calculateTrendLineSegment(series, trendline, axis, chart);
                            } else
                                if (axis.name == series._yAxisName && !axis.setRange && trendline.points && trendline.points.length > 1) {
                                    count = trendline.points.length - 1;
                                    axis.range.min = axis.range.min > trendline.minY ? trendline.minY : axis.range.min;
                                    axis.range.max = axis.range.max < trendline.maxY ? trendline.maxY : axis.range.max;
                                }
                        }
                    }
                }
            }
            return false;
        },

        AlignRangeStart: function (sDate, intervalSize, intervalType) {
            var sResult = new Date(sDate);
            if (intervalType.toLowerCase() == "days") {
                var day = Math.floor(Math.floor((sDate.getDate()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), day, 0, 0, 0);
            } else if (intervalType.toLowerCase() == "hours") {
                var hour = Math.floor(Math.floor((sDate.getHours()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), hour, 0, 0);
            } else if (intervalType.toLowerCase() == "milliseconds") {
                var milliseconds = Math.floor(Math.floor((sDate.getMilliseconds()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), sDate.getSeconds(), milliseconds);
            } else if (intervalType.toLowerCase() == "seconds") {
                var seconds = Math.floor(Math.floor((sDate.getSeconds()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), sDate.getMinutes(), seconds, 0);
            } else if (intervalType.toLowerCase() == "minutes") {
                var minutes = Math.floor(Math.floor((sDate.getMinutes()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), sDate.getMonth(), sDate.getDate(), sDate.getHours(), minutes, 0, 0);
            } else if (intervalType.toLowerCase() == "months") {
                var month = Math.floor(Math.floor((sDate.getMonth()) / intervalSize) * intervalSize);
                sResult = new Date(sDate.getFullYear(), month, sDate.getDate(), 0, 0, 0);
            } else if (intervalType.toLowerCase() == "years") {
                var year = Math.floor(Math.floor(sDate.getFullYear() / intervalSize) * intervalSize);
                sResult = new Date(year, sDate.getMonth(), sDate.getDate(), 0, 0, 0);
            }
            return sResult;
        },
        _increaseDateTimeInterval: function (date, interval, intervalType) {
            var result = new Date(date);
            if (intervalType.toLowerCase() == "days") {
                result.setDate(date.getDate() + interval);
            } else if (intervalType.toLowerCase() == "hours") {
                result.setHours(date.getHours() + interval);
            } else if (intervalType.toLowerCase() == "milliseconds") {
                result.setMilliseconds(date.getMilliseconds() + interval);
            } else if (intervalType.toLowerCase() == "seconds") {
                result.setSeconds(date.getSeconds() + interval);
            } else if (intervalType.toLowerCase() == "minutes") {
                result.setMinutes(date.getMinutes() + interval);
            } else if (intervalType.toLowerCase() == "months") {
                result.setMonth(date.getMonth() + interval);
            } else if (intervalType.toLowerCase() == "years") {
                result.setYear(date.getFullYear() + interval);
            }

            return result;
        },

        _calculatePadding: function (axis) {

            var start = (typeof axis.range.min == "string" && !isNaN(Date.parse(axis.range.min))) ? Date.parse(axis.range.min) : (axis.range.min).getTime();
            var end = (typeof axis.range.max == "string" && !isNaN(Date.parse(axis.range.max))) ? Date.parse(axis.range.max) : (axis.range.max).getTime();
            var interval = ((this._increaseDateTimeInterval(new Date(start), axis.range.interval, axis._intervalType)).getTime()) - start;
            if (!this.chartObj.zoomed && !axis.setRange) {
                start = new Date(start);
                end = new Date(end);
                var intervalType = axis._intervalType.toLowerCase();
                if (axis.rangePadding.toLowerCase() == 'none') {
                start =  start.getTime();
                end = end.getTime();
            } else if (axis.rangePadding.toLowerCase() == 'additional' || axis.rangePadding.toLowerCase() == 'round') {
                switch (intervalType) {
                    case 'years':
                        var startYear = start.getFullYear();
                        var endYear = end.getFullYear();
                        if (axis.rangePadding.toLowerCase() == 'additional') {
                            start = (new Date(startYear - axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                            end = (new Date(endYear + axis.range.interval, 1, 1, 0, 0, 0)).getTime();
                        } else {
                            start = (new Date(startYear, 1, 1, 0, 0, 0)).getTime();
                            end = (new Date(endYear, 1, 1, 0, 0, 0)).getTime();
                        }
                        break;
                    case 'months':                        
                        var month = start.getMonth();
                        var endMonth = end.getMonth();
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), month, 1, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), endMonth, endMonth == 2 ? 28 : 30, 0, 0, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), month + (-axis.range.interval), 1, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), endMonth + (axis.range.interval), endMonth == 2 ? 28 : 30, 0, 0, 0)).getTime();
                        }
                        break;
                    case 'days':
                        var day = start.getDate();
                        var endDay = end.getDate();
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), day, 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), endDay, 0, 0, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), day + (-axis.range.interval), 0, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), endDay + (axis.range.interval), 0, 0, 0)).getTime();
                        }
                        break;
                    case 'hours':
                        var hour = (start.getHours() / axis.range.interval) * axis.range.interval;
                        var endHour = end.getHours() + (start.getHours() - hour);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour, 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour, 0, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), hour + (-axis.range.interval), 0, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), endHour + (axis.range.interval), 0, 0)).getTime();
                        }
                        break;
                    case 'minutes':
                        var minute = (start.getMinutes() / axis.range.interval) * axis.range.interval;
                        var endMinute = end.getMinutes() + (start.getMinutes() - minute);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHour(), minute, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), endMinute, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), minute + (-axis.range.interval), 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), minute + (axis.range.interval), 0)).getTime();
                        }
                        break;
                    case 'seconds':
                        var second = (start.getSeconds() / axis.range.interval) * axis.range.interval;
                        var endSecond = end.getSeconds() + (start.getSeconds() - second);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second, 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHour(), end.getMinutes(), endSecond, 0)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), second + (-axis.range.interval), 0)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), endSecond + (axis.range.interval), 0)).getTime();
                        }
                        break;
                    case 'milliseconds':
                        var milliSecond = (start.getMilliseconds() / axis.range.interval) * axis.range.interval;
                        var endMilliSecond = end.getMilliseconds() + (start.getMilliseconds() - milliSecond);
                        if (axis.rangePadding.toLowerCase() == 'round') {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond)).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond)).getTime();
                        } else {
                            start = (new Date(start.getFullYear(), start.getMonth(), start.getDay(), start.getHours(), start.getMinutes(), start.getSeconds(), milliSecond + (-axis.range.interval))).getTime();
                            end = (new Date(end.getFullYear(), end.getMonth(), end.getDay(), end.getHours(), end.getMinutes(), end.getSeconds(), endMilliSecond + (axis.range.interval))).getTime();
                        }
                        break;
                }

            }

        }
        axis.actualRange.interval = interval;
        axis.actualRange.min = start;
        axis.actualRange.max = end;
        axis._interval = axis.range.interval;
        axis.actualRange.delta = (axis.actualRange.max - axis.actualRange.min);
        axis.range.min = new Date(start);
        axis.range.max = new Date(end);
        this._calculateVisibleRange(axis);

    },
    _calculateVisibleRange: function (axis, sender) {
        this.chartObj = sender ? sender : this.chartObj;
        axis.visibleRange = $.extend(true, {}, axis.actualRange);
        if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType!="polaraxes" ) {

            axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
            axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
            var baseRange = axis.actualRange;
            if (axis.isInversed) {
                var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }

            var startDate = new Date(start);
            var endDate = new Date(end);
			 if (start == end)
            {
                startDate = new Date(Date.parse(startDate) - 2592000000);
                endDate = new Date(Date.parse(endDate) + 2592000000);
            }
            //Use below code to find min,max and interval for visible range
            var intervalX = this.calculateDateTimeNiceInterval(axis, startDate, endDate);
            axis.visibleRange.min = Math.min(startDate.getTime(), endDate.getTime());
            axis.visibleRange.max = Math.max(startDate.getTime(),endDate.getTime());
            var interval = ((this._increaseDateTimeInterval(startDate, intervalX.interval, intervalX.intervalType)).getTime()) - axis.visibleRange.min;
            axis._intervalType = intervalX.intervalType;	      
            axis.visibleRange.interval = interval;
            axis.visibleRange.delta = Math.abs(axis.visibleRange.max - axis.visibleRange.min);
            axis._interval = intervalX.interval;
        }

    },
    _calculateAxisLabels: function (axis) {
        var position;
        if (this.chartObj.zoomed || axis.zoomed || !axis.setRange)
            position = (this.AlignRangeStart(new Date(axis.visibleRange.min), axis._interval, axis._intervalType)).getTime();

        else
            position = axis.visibleRange.min;
        while (position <= axis.visibleRange.max) {
            if (position >= axis.visibleRange.min && position <= axis.visibleRange.max) {

                axis.visibleLabels.push({ Value: position, Text: (Globalize.format(new Date(position), ((!(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat))) });
            }
            position = (this._increaseDateTimeInterval(new Date(position), axis._interval, axis._intervalType)).getTime();
            var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAxesEventArgs.data = { label: axis.visibleLabels[axis.visibleLabels.length - 1], axis: axis };
            this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
            axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
        }
        axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
    },
    calculateDateTimeNiceInterval: function (axis, startDate, endDate) { 
        var oneDay = 24 * 60 * 60 * 1000;
        //var axisInterval ;
        var totalDays = (Math.abs((startDate.getTime() - endDate.getTime()) / (oneDay)));
        var interval = this.calculateNumericNiceInterval(totalDays / 365, axis, "years");

        if (interval >= 1) {
            return { interval: interval, intervalType: "Years" };
        }

        interval = this.calculateNumericNiceInterval(totalDays / 30, axis, "months");

        if (interval >= 1) {
            return { interval: interval, intervalType: "Months" };
        }

        interval = this.calculateNumericNiceInterval(totalDays, axis, "days");

        if (interval >= 1) {


            return { interval: interval, intervalType: "Days" };
        }

        var totalHours = totalDays * 24;

        interval = this.calculateNumericNiceInterval(totalHours, axis, "hours");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Hours" };
        }

        var totalMinutes = totalDays * 24 * 60;

        interval = this.calculateNumericNiceInterval(totalMinutes, axis, "minutes");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Minutes" };
        }

        var totalSeconds = totalDays * 24 * 60 * 60;

        interval = this.calculateNumericNiceInterval(totalSeconds, axis, "seconds");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Seconds" };
        }

        var totalMilliseconds = totalDays * 24 * 60 * 60 * 1000;

        interval = this.calculateNumericNiceInterval(totalMilliseconds, axis, "milliseconds");

        if (interval >= 1) {

            return { interval: interval, intervalType: "Milliseconds" };
        }

    },
    calculateNumericNiceInterval: function (delta, axis, setInterval) {
        var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
        var niceInterval = delta / desiredIntervalsCount;
        if (axis._valueType=='double'){
            if (axis.desiredIntervals!= null)
                return niceInterval;
        }
        var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
        var intervalDivs = [10, 5, 2, 1];

        for (var i = 0; i < intervalDivs.length; i++) {
            var currentInterval = minInterval * intervalDivs[i];
            if (desiredIntervalsCount < (delta / currentInterval)) {
                if ((axis.intervalType) && axis._valueType == "datetime") {
                    niceInterval=(axis.intervalType.toLowerCase() == setInterval)?Math.ceil(niceInterval):niceInterval;
                }
                return niceInterval;
            }

            niceInterval = currentInterval;
        }

        return niceInterval;
    },
    GetActualDesiredIntervalsCount: function (axis, size) {
        if (ej.util.isNullOrUndefined(axis.desiredIntervals)) {
            var desiredIntervalsCount = (axis.orientation.toLowerCase() == "horizontal" ? 0.533 : 1) * axis.maximumLabels;
            desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
        
                return desiredIntervalsCount;
            } else {
                return axis.desiredIntervals;
            }
        },
        _calculateRange: function (axis, seriesCollection) {
            if (!this.chartObj.zoomed) {
                var defaultDate = new Date();
                if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0 || this.chartObj.model.indicators[0]._points.length > 0) {
                    var maxX, minY;
                    var maxY, minX;
                    var maxMinXValue = { maxX: null, minX: null };
                    var maxMinYValue = { maxY: null, minY: null };
                    for (var i = 0; i < seriesCollection.length; i++) {
                        if (seriesCollection[i].points.length > 0) {
                            if (seriesCollection[i].visibility.toLowerCase() === 'visible' && seriesCollection[i].points.length > 0) {
                                minX = maxX = seriesCollection[i].points[0].xValue;
                                maxY = minY = seriesCollection[i].points[0].YValues[0];
                                for (var j = 0; j < seriesCollection[i].points.length; j++) {
                                    if (minX > seriesCollection[i].points[j].xValue)
                                        minX = seriesCollection[i].points[j].xValue;
                                    if (maxX < seriesCollection[i].points[j].xValue)
                                        maxX = seriesCollection[i].points[j].xValue;
                                    if (minY > seriesCollection[i].points[j].YValues[0])
                                        minY = seriesCollection[i].points[j].YValues[0];
                                    if (maxY < seriesCollection[i].points[j].YValues[0])
                                        maxY = seriesCollection[i].points[j].YValues[0];
                                }
                                seriesCollection[i].minX = minX;
                                seriesCollection[i].maxX = maxX;
                                seriesCollection[i].minY = minY;
                                seriesCollection[i].maxY = maxY;
                            }
                        }
                    
                    if ((maxMinXValue.minX = (maxMinXValue.minX == null  && typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX) > seriesCollection[i].minX) {
                            maxMinXValue.minX = (typeof seriesCollection[i].minX != "number") ? seriesCollection[i].minX : maxMinXValue.minX;
                        }
                        if ((maxMinXValue.maxX = (maxMinXValue.maxX == null && typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX) < seriesCollection[i].maxX) {
                            maxMinXValue.maxX = (typeof seriesCollection[i].maxX != "number") ? seriesCollection[i].maxX : maxMinXValue.maxX;
                        }
                        if (maxMinYValue.minY > seriesCollection[i].minY) {
                            maxMinYValue.minY = seriesCollection[i].minY;
                        }
                        if (maxMinYValue.maxY < seriesCollection[i].maxY) {
                            maxMinYValue.maxY = seriesCollection[i].maxY;
                        }
               
                }
				// Find min/max for indicator series

                  if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                    var value = this.chartObj.model.indicatorRange[axis.name];
                    if (maxMinXValue.minX > new Date(value.min) || maxMinXValue.minX == null)
                        maxMinXValue.minX = new Date(value.min);
                    if (maxMinXValue.maxX < new Date(value.max) || maxMinXValue.maxX == null)
                        maxMinXValue.maxX = new Date(value.max);
                 }
                maxMinXValue.minX = (maxMinXValue.minX == null) ? defaultDate : maxMinXValue.minX;
                maxMinXValue.maxX = (maxMinXValue.maxX == null) ? new Date(new Date().setMonth(defaultDate.getMonth() + 5)) : maxMinXValue.maxX;
                if (maxMinXValue.minX == maxMinXValue.maxX) {
                    maxMinXValue.minX = new Date(Date.parse(maxMinXValue.minX) - 2592000000);
                    maxMinXValue.maxX = new Date(Date.parse(maxMinXValue.maxX) + 2592000000);
                }

                var data = _sideBySeriesPadding(this.chartObj, Date.parse(maxMinXValue.minX), Date.parse(maxMinXValue.maxX), axis);
                maxMinXValue.minX = new Date(data.startPadding);
                maxMinXValue.maxX = new Date(data.endPadding);

                

                var intervalX = this.calculateDateTimeNiceInterval(axis, maxMinXValue.minX, maxMinXValue.maxX);                
                axis._intervalType = (axis.intervalType) ? axis.intervalType : intervalX.intervalType;
               
                    if ((!axis.setRange)) {
                        axis.range = { min: maxMinXValue.minX, max: maxMinXValue.maxX, interval: intervalX.interval };
                    } else {
                        if (ej.util.isNullOrUndefined(axis.actual_Range)) {
                            if (ej.util.isNullOrUndefined(axis.range.interval))
                                axis.range.interval = intervalX.interval;
                            if (!axis.range.max)
                                axis.range.max = maxMinXValue.maxX;
                            if (!axis.range.min)
                                axis.range.min = maxMinXValue.minX;
                            if (typeof axis.range.min == "string" && axis.range.min.indexOf("/Date(") != -1)
                                axis.range.min = new Date(parseInt(axis.range.min.substr(6)));
                            if (typeof axis.range.max == "string" && axis.range.max.indexOf("/Date(") != -1)
                                axis.range.max = new Date(parseInt(axis.range.max.substr(6)));
                            axis.actual_Range = $.extend(true, {}, axis.range);
                            axis.setRange = true;
                        }
                        axis.range = { min: axis.actual_Range.min, max: axis.actual_Range.max, interval: axis.actual_Range.interval };
                    }
                } else {
                    if (!axis.setRange) {
                        axis.range = { min: defaultDate, max: new Date(new Date().setMonth(defaultDate.getMonth() + 5)), interval: 1 };
                        axis._intervalType = 'Months';
                    }
					else {
			            axis.range.min=(ej.util.isNullOrUndefined(axis.range.min))?defaultDate:axis.range.min;
			            axis.range.max=(ej.util.isNullOrUndefined(axis.range.max))?new Date(new Date().setMonth(defaultDate.getMonth() + 5)):axis.range.max;
			            axis.range.interval=(ej.util.isNullOrUndefined(axis.range.interval))?1:axis.range.interval;
			            axis._intervalType='Months';
                    }

                }
            }
        },
        _calculateRanges: function (chartObj, axis, seriesCollection) {

            this.chartObj = chartObj;

            this._calculateRange(axis, seriesCollection);

            if (chartObj.model._drawTrendline)
                this._calculateTrendLineRange(chartObj, axis, seriesCollection);

            this._calculatePadding(axis);

        this._calculateAxisLabels(axis);
    }
};

function ejExtendClass(parent, members) {
    var object = function () {
    };
    object.prototype = new parent();
    $.extend(object.prototype, members);
    return object;
}

//DateTime calculation
var ejDateTimeValue = ejExtendClass(ej.EjAxisRenderer);
ej.axisTypes.datetime = ejDateTimeValue;

var ejDoubleValue = ejExtendClass(ej.EjAxisRenderer, {
    
  
    _calculateRange: function (chartObj, axis, seriesCollection) {
        if (!chartObj.zoomed) {
            this._seriesMinMax(chartObj,axis, seriesCollection, "double");
        }
    },
    
    _getDoubleAutoRange: function (axis, min, max, delta) {
        var interval = this.calculateNumericNiceInterval(delta, axis);
        if (!axis.setRange) {
            return axis.range = { min: min, max: max, interval: interval, Delta: delta };

        } else {
            if ((ej.util.isNullOrUndefined(axis.range.interval))||(axis.range.interval<0))
                      axis.range.interval = interval;
            if (ej.util.isNullOrUndefined(axis.range.max))
                       axis.range.max = max;          
            if (ej.util.isNullOrUndefined(axis.range.min))
                       axis.range.min = min;
     
            axis.actual_Range = $.extend(true, {}, axis.range);
            axis.setRange = true;
           
            return axis.range = { min: axis.actual_Range.min, max: axis.actual_Range.max, interval: axis.actual_Range.interval, delta: (axis.actual_Range.max - axis.actual_Range.min) };
        }
    },
    _calculatePadding: function (chartObj, axis, baseRange) {
        var start = baseRange.min;
        var end = baseRange.max;
        var rangePadding = axis.rangePadding.toLowerCase();
        var interval= baseRange.interval;
        if ((!axis.setRange) && (!chartObj.zoomed)) {
               
            if (rangePadding == 'normal') {
                var minimum = 0, remaining;
                if (start < 0) {
                    start = 0;
                    minimum = baseRange.min + (baseRange.min / 20);

                    remaining =  interval + (minimum %  interval);

                    if ((0.365 *  interval) >= remaining) {
                        minimum -= interval;
                    }

                    if (minimum %  interval < 0) {
                        minimum = (minimum -  interval) - (minimum % interval);
                    }
                } else {
                    minimum = start < ((5.0 / 6.0) * end)
                        ? 0
                        : (start - (end - start) / 2);
                    if (minimum %  interval > 0) {
                        minimum -= (minimum %  interval);
                    }
                }
                var maximum = (end + (end - start) / 20);

                remaining = interval - (maximum % interval);

                if ((0.365 *  interval) >= remaining) {
                    maximum +=  interval;
                }

                if (maximum %  interval > 0) {
                    maximum = (maximum +  interval) - (maximum % interval);
                }
                 if (minimum <= 0) {
                    interval = this.calculateNumericNiceInterval(maximum - minimum, axis);
                    maximum = Math.ceil(maximum / interval) * interval;
                }
                start = (!(axis.roundingPlaces)) ? parseFloat(minimum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(minimum.toFixed(axis.roundingPlaces));
                end = (!(axis.roundingPlaces)) ? parseFloat(maximum.toFixed((ej.EjSvgRender.utils._decimalPlaces(interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(interval)))) : parseFloat(maximum.toFixed(axis.roundingPlaces));

            } else if (rangePadding == 'additional' || rangePadding == 'round') {
                start = Math.floor((start / interval) * interval);
                end = Math.ceil(end / interval) * interval;
                if (rangePadding == 'additional') {
                    start -= interval;
                    end +=  interval;
                }
            }


        }
        axis.actualRange.min = start;
        axis.actualRange.max = end;
        axis.actualRange.interval = interval;
        axis.range.min = start;
        axis.range.max = end;
        axis.range.interval =  interval;
        axis.actualRange.delta = end - start;
        this._calculateVisibleRange(axis);
    },
    _calculateVisibleRange: function (axis, sender) {
        this.chartObj = sender ? sender : this.chartObj;
        axis.visibleRange = $.extend(true, {}, axis.actualRange);
        if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType!="polaraxes" ) {
            var baseRange = axis.actualRange;

            axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
            axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
            if (axis.isInversed) {
                var start=axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }
            var delta =Math.abs(end-start);
            axis.visibleRange.interval = (this.calculateNumericNiceInterval(delta, axis));
            var factor = (ej.util.isNullOrUndefined(axis.roundingPlaces)) ? (ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval)) : axis.roundingPlaces;
            axis.visibleRange.interval = parseFloat(axis.visibleRange.interval.toFixed(factor));
            axis.visibleRange.min = Math.min(start, end);
            axis.visibleRange.max = Math.max(start, end);
            axis.visibleRange.delta = delta;
            axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
        }

    },
    _calculateAxisLabels: function (chartObj, currentAxis) {

        var tempInterval;
        var labelText;
		if (chartObj.zoomed || currentAxis.zoomed || currentAxis.padding)
            tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
		else
            tempInterval = currentAxis.visibleRange.min;
        for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {
            tempInterval = (!(currentAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(currentAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {


                var customFormat = currentAxis.labelFormat;
                labelText = (!(customFormat)) ? (!currentAxis.isStacked100 ? tempInterval : tempInterval + "%") : (customFormat.match('{value}') != null ? customFormat.replace('{value}', tempInterval) : Globalize.format(tempInterval, customFormat));                            
                currentAxis.visibleLabels.push({ Value: tempInterval, Text: labelText });
                // customize label by event
                var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxesEventArgs.data = { label: currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1], axis: currentAxis };
                chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
            }
        }

        currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

    },

    _calculateRanges: function (chartObj, axis, seriesCollection) {
        this.chartObj = chartObj;

        this._calculateRange(chartObj, axis, seriesCollection);

        if (chartObj.model._drawTrendline)
            this._calculateTrendLineRange(chartObj, axis, seriesCollection);

        this._calculatePadding(chartObj, axis, axis.range);

        if(axis.maximumLabels > 0)  // to calculate only if labels have to be displayed
        this._calculateAxisLabels(chartObj, axis);

    }
});
ej.axisTypes.double = ejDoubleValue;
//log axis
var ejLogValue = ejExtendClass(ej.EjAxisRenderer, {
    
    // Get value for log min and max
   _getRoundValue:function (x,div,up)
    {
        return (up ? Math.ceil(x / div) : Math.floor(x / div)) * div;
   },
   
   // Caculate log nice interval
  calculateLogNiceInterval: function (delta, axis) {
       var desiredIntervalsCount = this.GetActualDesiredIntervalsCount(axis, axis.length);
       var niceInterval = delta;
       var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
       var intervalDivs = [10, 5, 2, 1];

       for (var i = 0; i < intervalDivs.length; i++) {
           var currentInterval = minInterval * intervalDivs[i];
           if (desiredIntervalsCount < (delta / currentInterval)) {
               return niceInterval;
           }

           niceInterval = currentInterval;
       }

       return niceInterval;
  },
   
    // Get log value for auto range
    _getLogAutoRange: function (axis, min, max, delta) {
        
        var logStart = ej.EjSvgRender.utils._logBase(min, axis.logBase);
        logStart = $.isNumeric(logStart) ? logStart : min;
        var logEnd = ej.EjSvgRender.utils._logBase(max, axis.logBase);
        logEnd = $.isNumeric(logEnd) ? logEnd : max;
     
        var mulS = this._getRoundValue(logStart, 1, false);
        var mulE = this._getRoundValue(logEnd, 1, true);
        if (mulS == 0 && mulE == 0) {
            mulS = ej.EjSvgRender.utils._logBase((max - max / 2), axis.logBase);
            mulE = ej.EjSvgRender.utils._logBase((max + max / 2), axis.logBase);
        }
        delta = mulE - mulS;        
        
        var interval = this.calculateLogNiceInterval(delta, axis);
        if (!axis.setRange) {
            return axis.range = { min: mulS, max: mulE, interval: interval, Delta: delta };

        } else {
            if (ej.util.isNullOrUndefined(axis.range.interval))
                axis.range.interval = interval;
            if (ej.util.isNullOrUndefined(axis.range.max))
                axis.range.max = mulE;
            if (ej.util.isNullOrUndefined(axis.range.min))
                axis.range.min = mulS;
            axis.actual_Range = $.extend(true, {}, axis.range);
            axis.setRange = true;
          
            return axis.range = {
                min: axis.actual_Range.min,
                max: axis.actual_Range.max,
                interval: axis.actual_Range.interval,
                delta: (axis.actual_Range.max - axis.actual_Range.min)
            };
        }
    },
   
      //Calcualte auto log range from series
        _calculateLogRange: function (chartObj, axis, seriesCollection) {
            if (!chartObj.zoomed) {
                this._seriesMinMax(chartObj,axis, seriesCollection, "logarithmic");
            }
        },
   
       //Assign base range values to actual range
        _calculateLogPadding: function (chartObj, axis, baseRange) {
            // No padding support for log axis
            var start = baseRange.min;
            var end = baseRange.max;
            var interval = baseRange.interval;
            axis.actualRange.min = start;
            axis.actualRange.max = end;
            axis.actualRange.interval = interval;
            axis.range.min = start;
            axis.range.max = end;
            axis.range.interval = interval;
            axis.actualRange.delta = end - start;
            this._calculateVisibleRange(axis);
        },
   
        // Calculate visible range for zooming
        _calculateVisibleRange: function (axis, sender) {
            this.chartObj = sender ? sender : this.chartObj;
            axis.visibleRange = $.extend(true, {}, axis.actualRange);
            if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType!="polaraxes" ) {
			     axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
                axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
                var baseRange = axis.actualRange;
                if (axis.isInversed) {
                    var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                    var end = start - axis.zoomFactor * axis.actualRange.delta;
                }
                else {
                    var start = axis.actualRange.min + axis.zoomPosition * axis.actualRange.delta;
                    var end = start + axis.zoomFactor * axis.actualRange.delta;
                }

                if (start < baseRange.min) {
                    end = end + (baseRange.min - start);
                    start = baseRange.min;
                }

                if (end > baseRange.max) {
                    start = start - (end - baseRange.max);
                    end = baseRange.max;
                }
                var delta = Math.abs(end - start);
                axis.visibleRange.interval = (this.calculateLogNiceInterval(delta, axis));
                var factor = (ej.util.isNullOrUndefined(axis.roundingPlaces)) ?
                    (ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval) == 0 ? 1
                    : ej.EjSvgRender.utils._decimalPlaces(axis.visibleRange.interval)) : axis.roundingPlaces;
                axis.visibleRange.interval = parseFloat(axis.visibleRange.interval.toFixed(factor));
                axis.visibleRange.min = Math.min(start, end);
                axis.visibleRange.max = Math.max(start, end);
                axis.visibleRange.delta = delta;
                axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
            }

        },
   
        //Method implementation for Generate Labels in ChartAxis
        _calculateAxisLabels: function (chartObj, currentAxis) {

            var tempInterval;
            if (chartObj.zoomed || currentAxis.zoomed)
                tempInterval = currentAxis.visibleRange.min - (currentAxis.visibleRange.min % currentAxis.visibleRange.interval);
            else
                tempInterval = currentAxis.visibleRange.min;
            for (; tempInterval <= currentAxis.visibleRange.max; tempInterval += currentAxis.visibleRange.interval) {
                

                if (ej.EjSvgRender.utils._inside(tempInterval, currentAxis.visibleRange)) {      
                    var customFormat = (!(currentAxis.labelFormat)) ? null : currentAxis.labelFormat.match('{value}');
                    var tempIntervaltext = Math.pow(currentAxis.logBase, tempInterval);
                    tempIntervaltext = (!(currentAxis.roundingPlaces)) ?
                        parseFloat(tempIntervaltext.toFixed((ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval) == 0 ? 1
                            : ej.EjSvgRender.utils._decimalPlaces(currentAxis.visibleRange.interval))))
                            : parseFloat(tempIntervaltext.toFixed(currentAxis.roundingPlaces));
                    var labelText = (!(currentAxis.labelFormat)) ? tempIntervaltext : (customFormat != null) ?
                             currentAxis.labelFormat.replace('{value}', tempIntervaltext) :
                              (Globalize.format(tempIntervaltext, currentAxis.labelFormat));
                    currentAxis.visibleLabels.push({ Value:tempInterval, Text: labelText });
                    // customize label by event
                    var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonAxesEventArgs.data = { label: currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1], axis: currentAxis };
                    chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                    currentAxis.visibleLabels[currentAxis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
                }
            }

            currentAxis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(currentAxis, this.chartObj);

        },


        //Category axis calculation
        _calculateRanges: function (chartObj, axis, seriesCollection) {
            this.chartObj = chartObj;
            this._calculateLogRange(chartObj,axis, seriesCollection);
            if (chartObj.model._drawTrendline)
                this._calculateTrendLineRange(chartObj, axis, seriesCollection);
            this._calculateLogPadding(chartObj,axis,axis.range);
            this._calculateAxisLabels(chartObj,axis);
        }

        
    });
    ej.axisTypes.logarithmic = ejLogValue;
    
//Category Axes calculation
    var ejCategoryValue = ejExtendClass(ej.EjAxisRenderer, {
        //axis range calculation and assign indexed value to x value of points
        _calculateCategoryRange: function (axis, seriesCollection) {
            var legendCollapsed = (this.chartObj.model.legendCollapsed == undefined) ? !this.chartObj.zoomed : this.chartObj.model.legendCollapsed;
            if (!this.chartObj.model.excludeDataUpdate || legendCollapsed) {
            var maxXValue = { maxX: null, minX: 0 };
            var labels = [];
            var pointX;
            var intervalX;
            var deltaX;
            if (seriesCollection.length > 0 && seriesCollection[0].points.length > 0  || this.chartObj.model.indicators[0]._points.length>0) {
                for (var m = 0; m < seriesCollection.length; m++) {
                    for (var n = 0; n < seriesCollection[m].points.length; n++) {
                        if (seriesCollection[m]._xAxisValueType == "date")
                            pointX = Globalize.format(new Date(seriesCollection[m].points[n].x), (ej.util.isNullOrUndefined(axis.labelFormat) ? "dd/MM/yyyy" : axis.labelFormat));
                        else
                            pointX = seriesCollection[m].points[n].x;
                        var index = $.inArray(pointX, labels);
                        if (index < 0)
                            labels.push(pointX);
                    }
                }
                for (var l = 0; l < seriesCollection.length; l++) {
                    seriesCollection[l]._pointCollection = [];
                    for (var k = 0; k < seriesCollection[l].points.length; k++) {
                        seriesCollection[l]._pointCollection.push(seriesCollection[l].points[k].x);
                        if (seriesCollection[l]._xAxisValueType == "date")
                            pointX = Globalize.format(new Date(seriesCollection[l].points[k].x), (ej.util.isNullOrUndefined(axis.labelFormat) ? "dd/MM/yyyy" : axis.labelFormat));
                        else
                            pointX = seriesCollection[l].points[k].x;
                        seriesCollection[l].points[k].xValue = $.inArray(pointX, labels);
                       
                    }
                    seriesCollection[l].minX = 0;
                    seriesCollection[l].maxX = (seriesCollection[l]._xAxisValueType == "date") ? seriesCollection[l]._pointCollection.length - 1 : labels.length - 1;
                    // To find maximum x value of axis from series collection
                    if (l == 0) {
                        maxXValue.maxX = seriesCollection[l].maxX;
                        axis.labels = (seriesCollection[l]._xAxisValueType == "date") ? seriesCollection[l]._pointCollection : labels;
                        axis._categoryValueType = seriesCollection[l]._xAxisValueType;
                    }
                    if (maxXValue.maxX < (seriesCollection[l].maxX)) {
                        maxXValue.maxX = seriesCollection[l].maxX;
                        axis.labels = [];
                        axis.labels = labels;
                    }
                }
                if (this.chartObj.model.indicatorRange && this.chartObj.model.indicatorRange[axis.name]) {
                          var value = this.chartObj.model.indicatorRange[axis.name];
                          if (maxXValue.minX > value.min || maxXValue.minX == null)
                              maxXValue.minX = value.min;
                          if (maxXValue.maxX < value.max || maxXValue.maxX == null)
                              maxXValue.maxX = value.max;
                }
                if (maxXValue.minX == maxXValue.maxX && axis.labelPlacement && axis.labelPlacement.toLowerCase() == "onticks") {
                    maxXValue.maxX += 1;
                }
                deltaX = maxXValue.maxX - maxXValue.minX;
                
                if (axis.setRange && axis.range.interval) {
                    intervalX = Math.ceil(axis.range.interval);
                }
                else {
                    intervalX = this._calculateActualInterval(deltaX, axis);
                }

                axis.range = { min: maxXValue.minX, max: maxXValue.maxX, interval: intervalX, Delta: deltaX };
            }
            else {
                var labelLength = axis.labels.length;
                if (labelLength > 0) {
                    deltaX = labelLength - 1;
                    intervalX = this._calculateActualInterval(deltaX, axis); 
                    axis.range = { min: 0, max: labelLength - 1, interval: intervalX, Delta: deltaX };
                }
                else if (!axis.range) // set default range when no series points added to category axes
                    axis.range = { min: 0, max: 6, interval: 1, Delta: 6 };
            }       
            }

        },

    //Calculates actual interval
    _calculateActualInterval: function (delta, axis) {
        if (axis.categoryInterval == null)
            return Math.max(1, Math.floor(delta / this.GetActualDesiredIntervalsCount(axis, axis.length)));
        else
            return axis.categoryInterval;
    },

    //Apply padding based on labelPlacement
    _applyRangePadding: function (axis) {
        if (!this.chartObj.zoomed) {
            // ticks based on labelplacement and polaraxes
            var ticks = 0;
            if(this.chartObj.model.AreaType !== 'polaraxes')
            ticks = ((axis.labelPlacement === null || axis.labelPlacement === undefined)) ? -0.5 : (axis.labelPlacement.toLowerCase() == "betweenticks") ? (-0.5) : 0;
			 if (ticks < 0) {
                axis.range.min = axis.range.min + ticks;
                axis.range.max = axis.range.max - ticks;
                axis.range.delta = axis.range.max - axis.range.min;
                axis.actualRange = axis.range;
            } else {
                var data = _sideBySeriesPadding(this.chartObj, axis.range.min, axis.range.max, axis);
                axis.range.min = data.startPadding;
                axis.range.max = data.endPadding;
                axis.range.delta = axis.range.max - axis.range.min;
                axis.actualRange = axis.range;
			}
        }
        this._calculateVisibleRange(axis);
    },

    //Calculates the visible range  
    _calculateVisibleRange: function (axis, sender) {
        this.chartObj = sender ? sender : this.chartObj;
        axis.visibleRange = $.extend(true, {}, axis.actualRange);
        if ((axis.zoomFactor < 1 || axis.zoomPosition > 0) && this.chartObj.model.AreaType != "polaraxes") {
		     axis.zoomFactor = axis.zoomFactor > 1 ? 1 : (axis.zoomFactor < 0 ? 0 : axis.zoomFactor);
            axis.zoomPosition = axis.zoomPosition < 0 ? 0 : (axis.zoomPosition > 1 ? 1 : axis.zoomPosition);
            var baseRange = axis.actualRange;
            if (axis.isInversed) {
                var start = axis.actualRange.max - axis.zoomPosition * axis.actualRange.delta;
                var end = start - axis.zoomFactor * axis.actualRange.delta;
            }
            else {
                var start = axis.actualRange.min + (axis.zoomPosition * axis.actualRange.delta);
                var end = start + axis.zoomFactor * axis.actualRange.delta;
            }

            if (start < baseRange.min) {
                end = end + (baseRange.min - start);
                start = baseRange.min;
            }

            if (end > baseRange.max) {
                start = start - (end - baseRange.max);
                end = baseRange.max;
            }
            var delta = Math.abs(end - start);
            axis.visibleRange.interval = this._calculateActualInterval(delta, axis);
            axis.visibleRange.min = Math.min(start,end);
            axis.visibleRange.max = Math.max(start, end);
            axis.visibleRange.delta = delta;
            axis.zoomed = (this.chartObj.zoomed === null || this.chartObj.zoomed === undefined) ? true : this.chartObj.zoomed;
        }

    },

    //Generate category label for axis
    _calculateAxisLabels: function (axis) {
        var interval = axis.visibleRange.interval;
        var position = axis.visibleRange.min - (axis.visibleRange.min % axis.range.interval);
        for (; position <= axis.visibleRange.max; position += interval) {
            if (ej.EjSvgRender.utils._inside(position, axis.visibleRange)) {
                var pos = Math.round(position);
                axis.visibleLabels.push({ Value: pos, Text: ej.EjSvgRender.utils._getLabelContent(pos, axis) });

                // customize label by event
                var commonAxesEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                commonAxesEventArgs.data = { label: axis.visibleLabels[axis.visibleLabels.length - 1], axis: axis };
                this.chartObj._trigger("axesLabelRendering", commonAxesEventArgs);
                axis.visibleLabels[axis.visibleLabels.length - 1] = commonAxesEventArgs.data.label;
            }
        }
        axis._LableMaxWidth = ej.EjSvgRender.utils._getMaxLabelWidth(axis, this.chartObj);
    },

    //Category axis calculation
    _calculateRanges: function (chartObj, axis, seriesCollection) {
        this.chartObj = chartObj;

        this._calculateCategoryRange(axis, seriesCollection);
        if (chartObj.model._drawTrendline)
            this._calculateTrendLineRange(chartObj, axis, seriesCollection);
        this._applyRangePadding(axis);

        this._calculateAxisLabels(axis);
    }
});
ej.axisTypes.category = ejCategoryValue;

ej.EjAxisRenderer.prototype = {
    _drawAxes: function (axisIndex, axis) {

        if (this.model.AreaType == "cartesianaxes")
        {
            if (axis.orientation.toLowerCase() == "horizontal")
            {
                this._drawXAxisGridLine(axisIndex, axis);
                if (axis.visible)
                {
                    
                    if (axis.labelPosition!='inside')
                    {
                        this._drawXAxisLabels(axisIndex, axis);
                    }
                    this._drawXTitle(axisIndex, axis);
                }
            }

            if (axis.orientation.toLowerCase() == "vertical") {
                this._drawYAxisGridLine(axisIndex, axis);
                if (axis.visible) {
                    if (axis.labelPosition != 'inside')
                    {
                        this._drawYAxisLabels(axisIndex, axis);
                    }
                    this._drawYTitle(axisIndex, axis);
                }
            }
        }
        else {
            if (axis.orientation.toLowerCase() == "horizontal" && axisIndex == 0) {
                this._drawPolarGridLine(axis);
                this._drawPolarLabels(axis);
                 
            }
             else if(axisIndex == 1)
                this._drawPolarCircle(axis);           
         }
    },
    _getSharpPath:function (width) {
        var value = ((width % 2) == 0) ? 0 : 0.5;
        return value;
    },
    _drawAxisLine: function (axis) {

        // Yaxis MajorGridlines, Ticklines and Labels.
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLine' });
        var sbYMajorTick = ej.EjSvgRender.utils._getStringBuilder();
        var x1 = this.model.centerX;
        var y1 = this.model.centerY;
        var x2 = this.model.centerX;
        var y2 = this.model.centerY - this.model.Radius;
        var axisLine = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2;
        if (axis.visible && axis.axisLine.visible) {
            var options = {
                'id': this.svgObject.id + '_YAxisLines',
                'fill': 'none',
                'stroke-width': axis.axisLine.width,
                'stroke': axis.axisLine.color,
                'opacity': axis.axisLine.opacity,
                'stroke-dasharray': axis.axisLine.dashArray,
                'd': axisLine
            };
            this.svgRenderer.drawPath(options, gEle);

            //Drawing Major Grid Lines 
            $(gEle).appendTo(this.gPolarAxisEle);
        }
       

        if (axis.visible && axis.majorTickLines.visible) {
            gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisMajorTicks' });
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    var label = axis.visibleLabels[j];
                    var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);
                    var line = {};
                    line.X1 = this.model.centerX;
                    line.Y1 = this.model.centerY - radius;
                    line.X2 = line.X1 - axis.majorTickLines.size;
                    line.Y2 = line.Y1;
                    sbYMajorTick.append("M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ");
                }
               
                  yMajorTickDir = sbYMajorTick.toString();
                    var options = {
                        'id': this.svgObject.id + '_YAxisMajorTicks',
                        'fill': 'none',
                        'stroke-width': axis.majorTickLines.width,
                        'stroke': axis.majorTickLines.color,
                        'd': yMajorTickDir
                    };
            //Drawing Major Tickline Lines
                    this.svgRenderer.drawPath(options, gEle);
                    $(gEle).appendTo(this.gPolarAxisEle);
                
        }
        if (axis.visible) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLabels' });
            var font = axis.font;
            for (var j = 0; j < axis.visibleLabels.length; j++) {
                var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, axis.visibleLabels[j].Value, this);
                var labelText = axis.visibleLabels[j].Text;

                var size = ej.EjSvgRender.utils._measureText(labelText, null, axis.font);

                var X = this.model.centerX - axis.majorTickLines.size - (this.model.elementSpacing / 3);
                var Y = this.model.centerY - radius + (size.height / 5);


                var options = {
                    'id': this.svgObject.id + '_' + axis.name + '_YLabel_' + j,
                    'x': X,
                    'y': Y,
                    'fill': font.color,
                    'font-size': font.size,
                    'font-family': font.fontFamily,
                    'font-style': font.fontStyle,
                    'font-weight': font.fontWeight,
                    'opacity': font.opacity,
                    'text-anchor': 'end'
                };

                this.svgRenderer.drawText(options, labelText, gEle);

            }
            //Drawing YAxis Labels
            $(gEle).appendTo(this.gPolarAxisEle);
        }
       

    },
    _drawPolarLabels: function (axis) {
	     if (!axis.visible)
			return 0;
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels'  });
        var aroundRadius = this.model.Radius + axis.majorTickLines.size;
        var font = axis.font;
        var length = axis.visibleLabels.length;
		 var firstLabelX;
        var lastLabelX;
        for (var j = 0; j < length; j++) {
            var coef = ej.EjSvgRender.utils._valueToPolarCoefficient(axis, axis.visibleLabels[j].Value);

            var vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
		 if (!isNaN(vector.X) && !isNaN(vector.Y)) {
            var labelText = axis.visibleLabels[j].Text;
            var x = this.model.centerX + aroundRadius * vector.X;
            var y = this.model.centerY + aroundRadius * vector.Y;
            var textAnchor = (x < this.model.centerX) ? 'end' : ((x > this.model.centerX) ? 'start' : 'middle');

            var size= ej.EjSvgRender.utils._measureText(labelText, this.model.m_AreaBounds.Height, axis.font);
            // calculation for positioning the label along the arc
            if (coef == 0.25)
            {
                x -= this.model.elementSpacing / 2;
                y += (size.height) / 4;
            }
           else if (coef == 0.5)
            {
               y += (size.height) ;
            }
           else if (coef == 0.75)
           {
               x += this.model.elementSpacing / 2;
               y += (size.height) / 4;
           }
           else if (coef == 1 || coef==0) {
               y -= (size.height) / 2;
           }
               
           else if (0.25 < coef && coef < 0.5) {
               x -= this.model.elementSpacing / 2;
               y += (size.height) / 2;
           }
           else if (0.5 < coef && coef <= 0.75) {
               x += this.model.elementSpacing / 2;
               y += (size.height) / 2;
           }
           else if (0 < coef && coef < 0.25)
             {
                   x -= this.model.elementSpacing / 2;
           }
           else
               x += this.model.elementSpacing / 2;

            if (j == 0) {
                firstLabelX = x;
            }
            if (j == length-1) {
                lastLabelX = ej.EjSvgRender.utils._measureText(labelText, null, font).width;
                lastLabelX += x;
                if (lastLabelX > firstLabelX)
                    labelText = ""; //Hide the last overlapping labels
            }
            var options = {
                'id': this.svgObject.id + '_' + axis.name + '_XLabel_' + j,
                'x': x,
                'y': y,
                'fill': font.color,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                'opacity': font.opacity,
                'text-anchor': textAnchor
            };

            this.svgRenderer.drawText(options, labelText, gEle);
          }
        }
		
        $(gEle).appendTo(this.gXaxisEle);

    },
    _drawPolarCircle: function (axis) {
        if (axis.majorGridLines.visible) {
            var sbYMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisGrid' });
            if (this.model.isPolar) {
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    var label = axis.visibleLabels[j];
                    var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);

                    var options = {
                        'id': this.svgObject.id + '_YAxisGridLines_' + j,
                        'cx': this.model.centerX,
                        'cy': this.model.centerY,
                        'r': radius,
                        'fill': 'transparent',
                        'stroke-width': axis.majorGridLines.width,
                        'stroke': axis.majorGridLines.color,
                         'opacity': (this.vmlRendering) ? 0.3: axis.majorGridLines.opacity
                    };

                    //Drawing Major Grid Lines for polar
                    this.svgRenderer.drawCircle(options, gEle);
                   
                }
            }
            else {
                for (var j = 0; j < axis.visibleLabels.length; j++) {
                    
                        var label = axis.visibleLabels[j];
                        var radius = this.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(axis, label.Value, this);
                        // calculation for spider web
                        for (var i = 0; i < this.model._axes[0].visibleLabels.length; i++) {
                            var xAxis = this.model._axes[0];
                            var vector = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[i].Value);
						    if (!isNaN(vector.X) && !isNaN(vector.Y)){
                              var vector2;
                              if ((i + 1) < xAxis.visibleLabels.length) {
                                vector2 = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[i + 1].Value);
                              }
                              else {
                                vector2 = ej.EjSvgRender.utils._valueToVector(xAxis, xAxis.visibleLabels[0].Value);
                              }
                              var connectPoint = { X: this.model.centerX + radius * vector.X, Y: this.model.centerY + radius * vector.Y };
                              var endPoint = { X: this.model.centerX + radius * vector2.X, Y: this.model.centerY + radius * vector2.Y };
                              sbYMajorGrid.append("M" + " " + connectPoint.X + " " + connectPoint.Y + " " + "L" + " " + endPoint.X + " " + endPoint.Y + " ");
						    }
                        }
                }
                if (axis.majorGridLines.visible) {
                    yMajorGridDir = sbYMajorGrid.toString();
                    var options = {
                        'id': this.svgObject.id + '_YAxisGridLines',
                        'fill': 'none',
                        'stroke-width': axis.majorGridLines.width,
                        'stroke': axis.majorGridLines.color,
                        'opacity': axis.majorGridLines.opacity,
                        'stroke-dasharray': axis.majorGridLines.dashArray,
                        'd': yMajorGridDir
                    };
                    //Drawing Major Grid Lines for radar
                    this.svgRenderer.drawPath(options, gEle);
                }
            }
        }
        $(gEle).appendTo(this.gYaxisEle);

    },
    _drawPolarGridLine: function (axis) {

        var chartModel = this.model,
            legend = chartModel.legend,
            legendTitleHeight = ej.EjSvgRender.utils._measureText(legend.title.text, null, legend.title.font).height,
            legendPosition = legend.position.toLowerCase(),
            legXSpace = 0,
            legYSpace = 0,
            sbXMajorGrid = ej.EjSvgRender.utils._getStringBuilder(),
            sbXMajorTick = ej.EjSvgRender.utils._getStringBuilder(),
            borderSize = legend.border.width,
            yOffset, actualWidth, actualHeight,
            isRadar = false, vector, line,
            length = axis.visibleLabels.length,
            chartBorder = chartModel.border.width;
        if (legend.visible && legendPosition != "custom") {
            if (legendPosition == "right" || legendPosition == "left")
                legXSpace = ((legendPosition == "right") ? chartModel.margin.right : chartModel.margin.left) + chartModel.LegendActualBounds.Width + (borderSize + chartBorder) * 2;
            else
                legYSpace = ((legendPosition == "top") ? chartModel.margin.top : chartModel.margin.bottom) + chartModel.LegendActualBounds.Height + legendTitleHeight + (borderSize + chartBorder) * 2;

        }
        // calculating the radius of the chart with avaible size
        yOffset = ((chartModel.title.text) ? chartModel._titleLocation.Y : 0) + legYSpace;

        actualWidth = chartModel.svgWidth - legXSpace;
        actualHeight = chartModel.svgHeight - yOffset;

        chartModel.centerX = actualWidth * 0.5 + ((legendPosition === "left") ? legXSpace : 0);
        chartModel.centerY = actualHeight * 0.5 + ((legendPosition === "top") ? yOffset : ((chartModel.title.text) ? (chartModel._titleLocation.Y) : 0));

        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisGrid' });
        chartModel.finalSize = { width: actualWidth, height: actualHeight };

        chartModel.Radius = Math.min(chartModel.finalSize.width, chartModel.finalSize.height) / 2 - (2 * chartModel.elementSpacing) - axis.majorTickLines.size - axis._LableMaxWidth.maxHeight;
       
        var pos = 0;

        if (axis.majorGridLines.visible) {

            for (var j = 0; j < length; j++) {

                vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
                line = {};
                line.X1 = chartModel.centerX;

                line.Y1 = chartModel.centerY;
                line.X2 = chartModel.centerX + chartModel.Radius * vector.X;
                line.Y2 = chartModel.centerY + chartModel.Radius * vector.Y;
                sbXMajorGrid.append("M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ");
            }

        }
        if (axis.visible && axis.majorTickLines.visible) {
            var radius = chartModel.Radius;
            for (var j = 0; j < length; j++) {
                vector = ej.EjSvgRender.utils._valueToVector(axis, axis.visibleLabels[j].Value);
                line = {};
                line.X1 = chartModel.centerX + radius * vector.X;
                line.Y1 = chartModel.centerY + radius * vector.Y;
                line.X2 = line.X1 + axis.majorTickLines.size * vector.X;
                line.Y2 = line.Y1 + axis.majorTickLines.size * vector.Y;
                var direction = "M" + " " + line.X1 + " " + line.Y1 + " " + "L" + " " + line.X2 + " " + line.Y2 + " ";
                if (direction.indexOf("NaN") == -1)
                    sbXMajorTick.append(direction);
            }
        }
    

   
        if (axis.majorGridLines.visible) {
            xMajorGridDir = sbXMajorGrid.toString();
            var options = {
                'id': this.svgObject.id + '_XAxisGridLines',
                'fill': 'none',
                'stroke-width': axis.majorGridLines.width,
                'stroke': axis.majorGridLines.color,
                'opacity': axis.majorGridLines.opacity,
                'stroke-dasharray': axis.majorGridLines.dashArray,
                'd': xMajorGridDir
            };
            //Drawing XAxis Major Grid Lines
			if (xMajorGridDir.indexOf("NaN")== -1) {
              this.svgRenderer.drawPath(options, gEle);
              $(gEle).appendTo(this.gXaxisEle);
			}
        }

          if (axis.visible && axis.majorTickLines.visible) {
                xMajorTickDir = sbXMajorTick.toString();
                var options = {
                    'id': this.svgObject.id + '_XAxisMajorTicks',
                    'fill': 'none',
                    'stroke-width': axis.majorTickLines.width,
                    'stroke': axis.majorTickLines.color,
                    'd': xMajorTickDir
                };
                //Drawing XAxis Major Ticks Lines
                this.svgRenderer.drawPath(options, gEle);
                $(gEle).appendTo(this.gXaxisEle);
            }       
    },
    _drawXAxisTickLine:function(axisIndex,xAxis,gEle)
    {
        var xMajorTicksDir, xMinorTicksDir;
        var sbXMinorTicks = ej.EjSvgRender.utils._getStringBuilder();       
        var sbXMajorTicks = ej.EjSvgRender.utils._getStringBuilder();       
        var minorPointX;
        var x1 = Math.floor(xAxis.x);
        var x2 = Math.floor(xAxis.x + xAxis.width);
        var labelValue;

        if (axisIndex != 0 && ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') || (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside')) && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
        }

        else
        {
            var y1 = Math.floor(xAxis.y);
            var y2 = Math.floor(xAxis.y);
        }
        if (!gEle)
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTicklines' + '_' + axisIndex });
        xAxis.labelPlacement = (!(xAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : xAxis.labelPlacement;
        var ticksbwtLabel = ((xAxis.labels.length > 0) && (xAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = xAxis.visibleLabels.length;
        //For between ticks and gridlines
        if ((xAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = collectionLength > 0 ? xAxis.visibleLabels.length + 1 : collectionLength;
        }
        for (var i = 0; i < collectionLength; i++) {

            labelValue = (!xAxis.visibleLabels[i]) ? xAxis.visibleLabels[i - 1].Value + xAxis.range.interval : xAxis.visibleLabels[i].Value;
 
            var tempInterval = ((xAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;

            tempInterval = (!(xAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(xAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, xAxis.visibleRange)) {

                if ((xAxis.hidePartialLabels) && ((i == 0) || (i == (collectionLength - 1))))
                    continue;
                if (xAxis.minorGridLines.visible || xAxis.minorTickLines.visible) {
                    var ticksVal = this._getSharpPath(xAxis.minorTickLines.width);
                    var linesVal = this._getSharpPath(xAxis.minorGridLines.width);
                    var yTickPosition;
                    if (xAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = xAxis.visibleRange.max;
                        var logmin = xAxis.visibleRange.min;

                        var logtickstart = Math.pow(xAxis.logBase, minorTicks - xAxis.visibleRange.interval);
                        var logtickend = Math.pow(xAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (xAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                        for (var j = 0; j < xAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointX = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (xAxis.width));

                                //Calculate Minor Ticks
                                if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                    var xTickPosition = Math.floor(minorPointX + x1);
                                    if ((xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition != 'inside') || (xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition == 'inside'))
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                    else
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 + xAxis.minorTickLines.size) : y1 - Math.floor(xAxis.minorTickLines.size);
                                    sbXMinorTicks.append("M" + " " + (xTickPosition + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (xTickPosition + ticksVal) + " " + (y1 + ticksVal) + " ");
                                }
                            }
                        }
                    }
                    else {
                        for (var j = 0, minorTicks = tempInterval ; j < xAxis.minorTicksPerInterval; j++) {
                            minorTicks += xAxis.visibleRange.interval / (xAxis.minorTicksPerInterval + 1);
                            if (minorTicks < xAxis.visibleRange.max && minorTicks > xAxis.visibleRange.min) {
                                minorPointX = Math.ceil(((minorTicks - xAxis.visibleRange.min) / (xAxis.visibleRange.max - xAxis.visibleRange.min)) * (xAxis.width));

                                //Calculate Minor Ticks
                                if ((xAxis.minorTickLines) && xAxis.minorTickLines.visible) {
                                    if ((xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition != 'inside') || (xAxis.tickLinesPosition != 'inside' && xAxis.labelPosition == 'inside'))
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 - xAxis.minorTickLines.size) : Math.floor(xAxis.minorTickLines.size + y1);
                                    else
                                        yTickPosition = (xAxis.opposedPosition) ? Math.floor(y1 + xAxis.minorTickLines.size) : Math.floor(y1 - xAxis.minorTickLines.size);
                                    var points = Math.floor(minorPointX + x1);
                                    sbXMinorTicks.append("M" + " " + (points + ticksVal) + " " + (yTickPosition + ticksVal) + " " + "L" + " " + (points + ticksVal) + " " + (y1 + ticksVal) + " ");
                                }
                            }
                        }
                    }

                }

                //calculate major ticks
                if (xAxis.majorTickLines.visible || xAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(xAxis.majorTickLines.width);
                    var mlVal = this._getSharpPath(xAxis.majorGridLines.width);
                    //Calculate Major Ticks and Grid lines
                    var pointX = this.pointX = Math.ceil(ej.EjSvgRender.utils._getPointXY(tempInterval, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width));
                    //Calculate Major Ticks
                    if (xAxis.majorTickLines.visible) {
                     
                        var lineStart = xAxis.majorTickLines.size + xAxis.axisLine.width / 2;
                        if ((xAxis.tickLinesPosition == 'inside' && xAxis.labelPosition == 'inside') || (xAxis.tickLinesPosition == 'inside' && xAxis.labelPosition != 'inside'))
                            var yPosition = (xAxis.opposedPosition) ? y1 + lineStart : y1 - lineStart;
                        else
                            var yPosition = (xAxis.opposedPosition) ? y1 - lineStart : lineStart + y1;
                        sbXMajorTicks.append("M" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(yPosition) + mtVal) + " " + "L" + " " + (Math.floor(pointX + x1) + mtVal) + " " + (Math.floor(y1 + ((xAxis.opposedPosition) ? (-xAxis.axisLine.width / 2) : (xAxis.axisLine.width / 2))) + mtVal) + " ");
                    }
                }
            }
        }
        //draw minor ticks
        if (xAxis.minorTickLines.visible && xAxis.visible && xAxis.minorTicksPerInterval > 0) {
            xMinorTicksDir = sbXMinorTicks.toString();
            var optionsMinorTicks = {
                'id': this.svgObject.id + '_XAxisMinorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.minorTickLines.width,
                'stroke': xAxis.minorTickLines.color,
                'd': xMinorTicksDir
            };
            if(!gEle)
                this.svgRenderer.drawPath(optionsMinorTicks, gTickEle);
            else
                this.svgRenderer.drawPath(optionsMinorTicks, gEle);

        }
        if (xAxis.majorTickLines.visible && xAxis.visible) {
            xMajorTicksDir = sbXMajorTicks.toString();
            var optionsMajorTicks = {
                'id': this.svgObject.id + '_XAxisMajorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.majorTickLines.width,
                'stroke': xAxis.majorTickLines.color,
                'd': xMajorTicksDir
            };
            if (!gEle) {
                
                this.svgRenderer.drawPath(optionsMajorTicks, gTickEle);
            }
            else
                this.svgRenderer.drawPath(optionsMajorTicks, gEle);
        }
        if(xAxis.tickLinesPosition=='inside')
            $(gTickEle).appendTo(this.gXaxisTickEle);
        else
            $(gEle).appendTo(this.gXaxisEle);

    },
    _drawXAxisGridLine: function (axisIndex, xAxis) {
         var xMajorGridDir, xMinorGridDir;        
        var sbXMinorGrid = ej.EjSvgRender.utils._getStringBuilder();        
        var sbXMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
        var minorPointX;
        var xPoint = [];
        var labelValue;

        var x1 = Math.floor(xAxis.x);
        var x2 = Math.floor(xAxis.x + xAxis.width);
        
        if (axisIndex != 0 && (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y))))
        {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            var y1 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
            var y2 = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis.majorTickLines.size) : Math.floor(xAxis.y - xAxis.majorTickLines.size);
        }
        else
        {
            var y1 = Math.floor(xAxis.y);
            var y2 = Math.floor(xAxis.y);
        }
       
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisGrid' + '_' + axisIndex });
        var gAgb = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisAlternateGridBand' + '_' + axisIndex });

        if (xAxis.visible && xAxis.axisLine.visible) {
            var offset = xAxis.axisLine.offset > 0 ? xAxis.axisLine.offset : 0;
            var val=this._getSharpPath(xAxis.axisLine.width);
            var optionsLine = {
                'id': this.svgObject.id + '_XAxisLine_' + axisIndex,
                x1: (x1 - xAxis.plotOffset)+ val + offset ,
                y1: y1+val,
                x2: (x2 + xAxis.plotOffset) -offset+val,
                y2: y2+val,
                'stroke-dasharray': xAxis.axisLine.dashArray,
                'stroke-width': xAxis.axisLine.width,
                'stroke': xAxis.axisLine.color,
                'opacity': (this.vmlRendering) ? 0.1 : xAxis.axisLine.opacity
            };

            this.svgRenderer.drawLine(optionsLine, gEle);
        }
        if (xAxis.tickLinesPosition != 'inside') {
            this._drawXAxisTickLine(axisIndex, xAxis, gEle);
        }
        xAxis.labelPlacement = (!(xAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : xAxis.labelPlacement;
        var ticksbwtLabel = ((xAxis.labels.length > 0) && (xAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = xAxis.visibleLabels.length;
		//For between ticks and gridlines
        if ((xAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = collectionLength > 0 ? xAxis.visibleLabels.length + 1 : collectionLength;
        }
        for (var i = 0; i < collectionLength; i++) {
			 labelValue = (!xAxis.visibleLabels[i]) ? xAxis.visibleLabels[i - 1].Value + xAxis.range.interval : xAxis.visibleLabels[i].Value;

            var tempInterval = ((xAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;

            tempInterval = (!(xAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(xAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(xAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, xAxis.visibleRange)) {

                if ((xAxis.hidePartialLabels) && ((i == 0) || (i == (collectionLength - 1))))
                    continue;
            
                //Calculate Minor Ticks and Grid lines
                if (xAxis.minorGridLines.visible || xAxis.minorTickLines.visible) {
                    var ticksVal = this._getSharpPath(xAxis.minorTickLines.width);
                    var linesVal = this._getSharpPath(xAxis.minorGridLines.width);
                    var yTickPosition;
                    if (xAxis._valueType == "logarithmic"){
                    minorTicks = tempInterval;
                    var logmax = xAxis.visibleRange.max;
                    var logmin = xAxis.visibleRange.min;

                    var logtickstart = Math.pow(xAxis.logBase, minorTicks - xAxis.visibleRange.interval);
                    var logtickend = Math.pow(xAxis.logBase, minorTicks);
                    var logtickInterval = (logtickend - logtickstart) / (xAxis.minorTicksPerInterval + 1);
                    var logtickPos = logtickstart + logtickInterval;
                    minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                    for (var j = 0; j < xAxis.minorTicksPerInterval; j++) {


                        if (minorTicks < logmax && minorTicks > logmin) {

                            minorPointX = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (xAxis.width));

                           

                            //Calculate Minor Gridlines
                            if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible) {
                                var tickPosition = Math.floor(minorPointX + x1) ;
                                sbXMinorGrid.append("M" + " " + (tickPosition + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (tickPosition + linesVal) + " " + ( Math.floor(xAxis.opposedPosition ?  this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height: this.model.m_AreaBounds.Y) + linesVal) + " ");
                            }

                        }
                        logtickPos += logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, xAxis.logBase);
                    }
                    } else {
                        for (var j = 0, minorTicks = tempInterval ; j < xAxis.minorTicksPerInterval; j++) {
                            minorTicks += xAxis.visibleRange.interval / (xAxis.minorTicksPerInterval + 1);
                            if (minorTicks < xAxis.visibleRange.max && minorTicks > xAxis.visibleRange.min) {
                                minorPointX = Math.ceil(((minorTicks - xAxis.visibleRange.min) / (xAxis.visibleRange.max - xAxis.visibleRange.min)) * (xAxis.width));


                                //Calculate Minor Gridlines
                                if ((xAxis.minorGridLines) && xAxis.minorGridLines.visible)
                                    sbXMinorGrid.append("M" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + (y1 + linesVal) + " " + "L" + " " + (Math.floor(minorPointX + x1) + linesVal) + " " + ( Math.floor(xAxis.opposedPosition ?  this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height: this.model.m_AreaBounds.Y) + linesVal) + " ");
                            }
                        }
                    }
            }

                if (xAxis.majorTickLines.visible || xAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(xAxis.majorTickLines.width);
                    var mlVal = this._getSharpPath(xAxis.majorGridLines.width);
                    var pointX = this.pointX = Math.ceil(ej.EjSvgRender.utils._getPointXY(tempInterval, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width));
                    var mX1 = (Math.floor(pointX + x1));
                    if (xAxis.zoomed)
                        xPoint[0] = xAxis.x;
                    if (i == 0) {
                        if (mX1 == Math.floor(this.model.m_AreaBounds.X)) {
                            xPoint.push(mX1 + mlVal);
                            continue;
                            
                        }
                    }
                    if ((i == (collectionLength - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                         if (mX1 == Math.floor(this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width))
                             continue;
                         if (xAxis.zoomed)
                             xPoint[collectionLength] = mX1 + mlVal;
                       
                    }
                     
                    //Calculate Major Gridlines
                    if (xAxis.majorGridLines.visible)
                        sbXMajorGrid.append("M" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + mlVal) + " " + "L" + " " + (mX1 + mlVal) + " " + (Math.floor(this.model.m_AreaBounds.Y) + mlVal) + " ");
                    if ((xAxis._valueType.toLowerCase() != "category") && i == collectionLength - 1)
                        continue;
                    xPoint.push((mX1 + mlVal));
                }

            }

        }

       
        if (xAxis.alternateGridBand.odd.fill != "transparent" || xAxis.alternateGridBand.even.fill != "transparent") {
            var height = this.model.m_AreaBounds.Height;
            
            for (var i = 1; i < xPoint.length; i += 2) {
                var width = xPoint[i + 1] - xPoint[i];
                width = (width || xAxis.labelPlacement.toLowerCase() == 'onticks') ? width : (xAxis.width+xAxis.x) - xPoint[i];
                if (i == xPoint.length - 1 && xAxis.zoomed) {
                    var zoomWidth = this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width - xPoint[xPoint.length - 1] - xAxis.plotOffset;
                    width = zoomWidth;
                }
                var optionsAlternateodd = {
                    'id': this.svgObject.id + '_XAxisAlternateodd' + [i],
                    'x': xPoint[i],
                    'y': Math.floor(this.model.m_AreaBounds.Y),
                    'width': width,
                    'height': height,
                    'fill': xAxis.alternateGridBand.odd.fill,
                    'opacity': xAxis.alternateGridBand.odd.opacity,
                    'stroke': "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'

                };
                this.svgRenderer.drawRect(optionsAlternateodd, gAgb);
            }

            for (var i = 0; i < xPoint.length; i += 2) {
                var width = xPoint[i + 1] - xPoint[i];
                width = (width || xAxis.labelPlacement.toLowerCase() == 'onticks') ? width : (xAxis.width + xAxis.x) - xPoint[i];
                if (i == 0 && xAxis.zoomed) {
                    var zoomWidth = xPoint[1] - xAxis.x;
                    width = zoomWidth;
                }
                if (i == xPoint.length - 1 && xAxis.zoomed) {
                    var zoomWidth = this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width - xPoint[xPoint.length - 1]-xAxis.plotOffset;
                    width = zoomWidth;
                }
                var optionsAlternateeven = {
                    'id': this.svgObject.id + '_XAxisAlternateeven' + [i],
                    'x': xPoint[i],
                    'y': Math.floor(this.model.m_AreaBounds.Y),
                    'width': width,
                    'height': height,
                    'fill': xAxis.alternateGridBand.even.fill,
                    'opacity': xAxis.alternateGridBand.even.opacity,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'stroke': "transparent"
                };
                this.svgRenderer.drawRect(optionsAlternateeven, gAgb);
            }
            $(gAgb).appendTo(gEle);
        }
        if (xAxis.minorGridLines.visible && xAxis.minorTicksPerInterval > 0) {
            xMinorGridDir = sbXMinorGrid.toString();
            //Drawing Minor Grid Lines
            var optionsMinorGrid = {
                'id': this.svgObject.id + '_XAxisMinorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.minorGridLines.width,
                'stroke': xAxis.minorGridLines.color,
                'stroke-dasharray': xAxis.minorGridLines.dashArray,
                'd': xMinorGridDir
            };
            this.svgRenderer.drawPath(optionsMinorGrid, gEle);
        }       
        xMajorGridDir = sbXMajorGrid.toString();
        if (xAxis.majorGridLines.visible) {
            var options = {
                'id': this.svgObject.id + '_XAxisMajorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': xAxis.majorGridLines.width,
                'stroke': xAxis.majorGridLines.color,
                'opacity': xAxis.majorGridLines.opacity,
                'stroke-dasharray': xAxis.majorGridLines.dashArray,
                'd': xMajorGridDir
            };

            //Drawing Major Grid Lines
            if(options.d != '')
            this.svgRenderer.drawPath(options, gEle);

        }
        $(gEle).appendTo(this.gXaxisEle);      
         
        
    },

    _drawYAxisTickLine:function(axisIndex,yAxis,gEle)
    {
        var yMajorTicksDir = "", yMinorTicksDir = "";
        var pointY = 0;
		 var labelValue;
        if (axisIndex != 1 && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')) && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
            var x2 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
        else if (axisIndex != 1 && (yAxis.labelPosition != 'inside' && yAxis.tickLinesPosition == 'inside') && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = Math.floor(yAxis.x+yAxis.majorTickLines.size);
            var x2 = Math.floor(yAxis.x+yAxis.majorTickLines.size);
        }
        else            
        {
            var x1 = Math.floor(yAxis.x);
            var x2 = Math.floor(yAxis.x);
        }   

        var y1 = Math.floor(yAxis.y + yAxis.height);
        var y2 = Math.floor(yAxis.y);       
    

        var sbYMinorTicks = ej.EjSvgRender.utils._getStringBuilder();   
        var sbYMajorTicks = ej.EjSvgRender.utils._getStringBuilder();      
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTicklines' + '_' + axisIndex });
      
		 yAxis.labelPlacement = (!(yAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : yAxis.labelPlacement;
        var ticksbwtLabel = ((yAxis.labels.length > 0) && (yAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = yAxis.visibleLabels.length;
        //For between ticks and gridlines
        if ((yAxis._valueType.toLowerCase() == ej.datavisualization.Chart.ValueType.Category) && ticksbwtLabel < 0) {
            collectionLength = collectionLength > 0 ? yAxis.visibleLabels.length + 1 : collectionLength;
        }
        for (var i = 0; i < collectionLength; i++)
        {
            labelValue = (!yAxis.visibleLabels[i]) ? yAxis.visibleLabels[i - 1].Value + yAxis.range.interval : yAxis.visibleLabels[i].Value;
            var tempInterval = ((yAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;
            tempInterval = (!(yAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(yAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, yAxis.visibleRange))
            {
 
                if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                    continue;

                // Calculate Minor Ticks and Grid lines      
                if (yAxis.minorGridLines.visible || yAxis.minorTickLines.visible) {
                    var minorPointY;
                    var tickVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var lineVal = this._getSharpPath(yAxis.minorGridLines.width);
                    if (yAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = yAxis.visibleRange.max;
                        var logmin = yAxis.visibleRange.min;

                        var logtickstart = Math.pow(yAxis.logBase, minorTicks - yAxis.visibleRange.interval);
                        var logtickend = Math.pow(yAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (yAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        for (var j = 0; j < yAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointY = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (yAxis.height));

                                //Calculate Minor Ticks
                                if ((yAxis.minorTickLines) && yAxis.minorTickLines.visible) {

                                    if (yAxis.minorTickLines.visible) {
                                        if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                                            ytickPosition = (yAxis.opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                        else
                                            var ytickPosition = (yAxis.opposedPosition) ? -yAxis.minorTickLines.size : yAxis.minorTickLines.size;
                                        sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                    }
                                }                              
                            }
                            logtickPos += logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        }
                    }
                    else {
                        for (var j = 0, minorTicks = tempInterval; j < yAxis.minorTicksPerInterval; j++) {

                            minorTicks += yAxis.visibleRange.interval / (yAxis.minorTicksPerInterval + 1);
                            if (minorTicks < yAxis.visibleRange.max && minorTicks > yAxis.visibleRange.min) {
                                minorPointY = ((minorTicks - yAxis.visibleRange.min) / (yAxis.visibleRange.delta)) * (yAxis.height);
                                // Calculate Minor Ticks   
                                if (yAxis.minorTickLines.visible) {
                                    if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                                        var ytickPosition = (yAxis.opposedPosition) ? yAxis.minorTickLines.size : -yAxis.minorTickLines.size;
                                    else
                                        var ytickPosition = (yAxis.opposedPosition) ? -yAxis.minorTickLines.size : yAxis.minorTickLines.size;
                                    sbYMinorTicks.append("M" + " " + (Math.floor(ytickPosition + x1) + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " " + "L" + " " + (x1 + tickVal) + " " + (Math.floor((minorPointY * -1) + y1) + tickVal) + " ");
                                }
                               
                            }
                        }
                    }
                }
                // Calculate Major Ticks and Grid lines
                if (yAxis.majorTickLines.visible || yAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var mlVal = this._getSharpPath(yAxis.majorGridLines.width);
                    // Calculate Major Ticks
                    if (yAxis.majorTickLines.visible)
                    {
                        pointY = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                        if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                            var xPosition = (yAxis.opposedPosition) ? yAxis.majorTickLines.size + yAxis.axisLine.width / 2 : -(yAxis.majorTickLines.size + yAxis.axisLine.width / 2);
                        else if ((yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside') || (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition == 'inside'))
                            var xPosition = (yAxis.opposedPosition) ? -(yAxis.majorTickLines.size + yAxis.axisLine.width / 2) : (yAxis.majorTickLines.size + yAxis.axisLine.width / 2);

                        sbYMajorTicks.append("M" + " " + (Math.floor(xPosition + x1) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " " + "L" + " " + (Math.floor(x1 + ((yAxis.opposedPosition) ? (yAxis.axisLine.width / 2) : (yAxis.axisLine.width / 2))) + mtVal) + " " + (Math.floor((pointY * -1) + y1) + mtVal) + " ");

                    }


                   
                }
            }
        }

        //Drawning Minor Ticks
        if (yAxis.minorTickLines.visible && yAxis.visible && yAxis.minorTicksPerInterval > 0) {
            yMinorTicksDir = sbYMinorTicks.toString();
            var optionsMinotTick = {
                'id': this.svgObject.id + '_YAxisMinorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.minorTickLines.width,
                'stroke': yAxis.minorTickLines.color,
                'd': yMinorTicksDir
            };
            if(yAxis.tickLinesPosition=='inside')
                this.svgRenderer.drawPath(optionsMinotTick, gTickEle);
            else
                this.svgRenderer.drawPath(optionsMinotTick, gEle);
        }
       

        if (yAxis.majorTickLines.visible && yAxis.visible) {
            yMajorTicksDir = sbYMajorTicks.toString();
            //Drawning Major Ticks
            var optionsMajorTick = {
                'id': this.svgObject.id + '_YAxisMajorTicks_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.majorTickLines.width,
                'stroke': yAxis.majorTickLines.color,
                'd': yMajorTicksDir
            };
            if (yAxis.tickLinesPosition == 'inside')
                this.svgRenderer.drawPath(optionsMajorTick, gTickEle);
            else
                this.svgRenderer.drawPath(optionsMajorTick, gEle);
        }       
       if(!gEle)
           $(gTickEle).appendTo(this.gYaxisTickEle);
        else
           $(gEle).appendTo(this.gYaxisEle);

    },

    _drawYAxisGridLine: function (axisIndex, yAxis) {

        var yMajorGridDir = "", yMajorTicksDir = "", yMinorTicksDir = "", yMinorGridDir = "";
        var pointY = 0;
        var labelValue;
        var y1 = Math.floor(yAxis.y + yAxis.height);
        var y2 = Math.floor(yAxis.y);
        var yPoint = [];
        var axisName = (yAxis.name).replace(/[^a-zA-Z0-9]/g, "");
        if (axisIndex != 1 && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')) && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) {
            var x1 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
            var x2 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
    else if(axisIndex != 1 && (yAxis.labelPosition != 'inside' && yAxis.tickLinesPosition == 'inside') && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) || (yAxis.x == (this.model.m_AreaBounds.X))))) 
        {
            var x1 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
            var x2 = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
        }
        else {
            var x1 = Math.floor(yAxis.x);
            var x2 = Math.floor(yAxis.x);
        }             
        var sbYMinorGrid = ej.EjSvgRender.utils._getStringBuilder();        
        var sbYMajorGrid = ej.EjSvgRender.utils._getStringBuilder();
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisGrid' + '_' + axisIndex });
        var gAgb = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisAlternateGridBand' + '_' + axisIndex });

        if (yAxis.visible && yAxis.axisLine.visible) {
            var offset = yAxis.axisLine.offset > 0 ? yAxis.axisLine.offset : 0;
            var val = this._getSharpPath(yAxis.axisLine.width);
            
            var optionsLine = {
                'id': this.svgObject.id +axisName + '_YAxisLine_' + axisIndex,
                x1: x1+val,
                y1: (y2 - yAxis.plotOffset) + offset+val,
                x2: x2+val,
                y2: (y1 + yAxis.plotOffset) - offset+val,
                'stroke-width': yAxis.axisLine.width,
                'stroke-dasharray': yAxis.axisLine.dashArray,
                'stroke': yAxis.axisLine.color,
                'opacity': (this.vmlRendering) ? 0.1 :yAxis.axisLine.opacity
            };
            this.svgRenderer.drawLine(optionsLine, gEle);
        }
     
        if (yAxis.axisBottomLine) {
            if (yAxis.axisBottomLine.visible) {
                var optionsBottomLine = {
                    'id': this.svgObject.id +axisName+ '_YAxisBottomLine_' + axisIndex,
                    x1: this.model.m_AreaBounds.X,
                    y1: y1,
                    x2: this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width,
                    y2: y1,
                    'stroke-width': yAxis.axisBottomLine.width,
                    'stroke': yAxis.axisBottomLine.color
                };
                this.svgRenderer.drawLine(optionsBottomLine, gEle);
            }
        }
        if (yAxis.tickLinesPosition != 'inside') {
            this._drawYAxisTickLine(axisIndex, yAxis, gEle);
        }

		 yAxis.labelPlacement = (!(yAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.BetweenTicks : yAxis.labelPlacement;
        var ticksbwtLabel = ((yAxis.labels.length > 0) && (yAxis.labelPlacement.toLowerCase() == "betweenticks")) ? -0.5 : 0;
        var collectionLength = yAxis.visibleLabels.length;
        for (var i = 0; i < collectionLength; i++) {
            labelValue = yAxis.visibleLabels[i].Value;
            var tempInterval = ((yAxis.labels.length > 0) && (ticksbwtLabel < 0)) ? labelValue + ticksbwtLabel : labelValue;
            tempInterval = (!(yAxis.roundingPlaces)) ? parseFloat(tempInterval.toFixed((ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval) == 0 ? 1 : ej.EjSvgRender.utils._decimalPlaces(yAxis.visibleRange.interval)))) : parseFloat(tempInterval.toFixed(yAxis.roundingPlaces));

            if (ej.EjSvgRender.utils._inside(tempInterval, yAxis.visibleRange)) {

                if ((yAxis.hidePartialLabels) && ((i == 0) || (i == (yAxis.visibleLabels.length - 1))))
                    continue;

                // Calculate Minor Ticks and Grid lines      
                if (yAxis.minorGridLines.visible || yAxis.minorTickLines.visible) {
                    var minorPointY;
                    var tickVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var lineVal = this._getSharpPath(yAxis.minorGridLines.width);
                    if (yAxis._valueType == "logarithmic") {
                        minorTicks = tempInterval;
                        var logmax = yAxis.visibleRange.max;
                        var logmin = yAxis.visibleRange.min;

                        var logtickstart = Math.pow(yAxis.logBase, minorTicks - yAxis.visibleRange.interval);
                        var logtickend = Math.pow(yAxis.logBase, minorTicks);
                        var logtickInterval = (logtickend - logtickstart) / (yAxis.minorTicksPerInterval + 1);
                        var logtickPos = logtickstart + logtickInterval;
                        minorTicks = ej.EjSvgRender.utils._logBase(logtickPos, yAxis.logBase);
                        for (var j = 0; j < yAxis.minorTicksPerInterval; j++) {


                            if (minorTicks < logmax && minorTicks > logmin) {

                                minorPointY = Math.ceil(((minorTicks - logmin) / (logmax - logmin)) * (yAxis.height));

                            
                              
                                //Calculate Minor Gridlines
                                if (yAxis.minorGridLines.visible)
                                    sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");

                            }
                            logtickPos += logtickInterval;
                            minorTicks = ej.EjSvgRender.utils._logBase(logtickPos,yAxis.logBase);
                        }
                    } else {
                        for (var j = 0, minorTicks = tempInterval; j < yAxis.minorTicksPerInterval; j++) {

                            minorTicks += yAxis.visibleRange.interval / (yAxis.minorTicksPerInterval + 1);
                            if (minorTicks < yAxis.visibleRange.max && minorTicks > yAxis.visibleRange.min) {
                                  minorPointY = ((minorTicks - yAxis.visibleRange.min) / (yAxis.visibleRange.delta)) * (yAxis.height);
                              
                                // Calculate Minor Grid lines   
                                if (yAxis.minorGridLines.visible)
                                    sbYMinorGrid.append("M" + " " + (Math.floor(this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " " + "L" + " " + (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + lineVal) + " " + (Math.floor((minorPointY * -1) + y1) + lineVal) + " ");
                            }
                        }
                    }
                } 
                // Calculate Grid lines
                if (yAxis.majorTickLines.visible || yAxis.majorGridLines.visible) {
                    var mtVal = this._getSharpPath(yAxis.minorTickLines.width);
                    var mlVal = this._getSharpPath(yAxis.majorGridLines.width);
                   
                    // Calculate Major Grid lines
                    if (yAxis.majorGridLines.visible) {
                        pointY = ej.EjSvgRender.utils._getPointXY(tempInterval, yAxis.visibleRange, yAxis.isInversed) * (yAxis.height);
                        var mX1 = (Math.floor(this.model.m_AreaBounds.X) + mlVal);
                        var mY1 = (Math.floor((pointY * -1) + y1) );
                        var mX2 = (Math.floor(this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) + mlVal);
                        if (i == 0) {
                            if (mY1==Math.floor(this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height))
                                continue;
                            if (yAxis.zoomed)
                                yPoint[0] = mY1 + mlVal;
                            
                        }
                        if ((i == (yAxis.visibleLabels.length - 1)) && (this.model.chartArea.border.color != "transparent" && this.model.chartArea.border.width > 0)) {
                           
                            if (mY1 == (Math.floor(this.model.m_AreaBounds.Y))) {
                                yPoint.push(mY1 + mlVal);
                                continue;
                            }

                        }
                       
                        

                        sbYMajorGrid.append("M" + " " + mX1 + " " + (mY1 + mlVal) + " " + "L" + " " + mX2 + " " + (mY1 + mlVal) + " ");
                        if (i == 0) 
                            continue;
                        
                        yPoint.push(mY1 + mlVal);
                        
                    }
                }
            }
        }
        
        if (yAxis.alternateGridBand.odd.fill != "transparent" || yAxis.alternateGridBand.even.fill != "transparent") {
           
            var width = this.model.m_AreaBounds.Width;
            if (yAxis.zoomed) {
                yPoint[yPoint.length] = yAxis.y;
                
            }
            for (var i = 1; i < yPoint.length; i += 2) {
                var height = yPoint[0] - yPoint[1];
                if (i == yPoint.length - 1 && yAxis.zoomed) {
                    var zoomHeight = yPoint[yPoint.length - 2] - yAxis.y;
                    height = zoomHeight;
                }
                var yoptionsAlternateodd = {
                    'id': this.svgObject.id + '_YAxisAlternateodd' + [i],
                    'x': Math.floor(this.model.m_AreaBounds.X),
                    'y': yPoint[i],
                    'width': width,
                    'height': height,
                    'fill': yAxis.alternateGridBand.odd.fill,
                    'opacity': yAxis.alternateGridBand.odd.opacity,
                    'stroke' : "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                };
                this.svgRenderer.drawRect(yoptionsAlternateodd, gAgb);
            }
            
            for (var i = 0; i < yPoint.length; i += 2) {
                var height = yPoint[0] - yPoint[1];
                if (i == yPoint.length - 1 && yAxis.zoomed) {
                    var zoomHeight = yPoint[yPoint.length - 2] - yAxis.y;
                    height = zoomHeight;
                }
                if (i == 0 && yAxis.zoomed) {
                    var zoomHeight = yAxis.y + yAxis.height - yPoint[0] - yAxis.plotOffset;
                    height = zoomHeight;
                }
                var yoptionsAlternateeven = {
                    'id': this.svgObject.id + '_YAxisAlternateeven' + [i],
                    'x': Math.floor(this.model.m_AreaBounds.X),
                    'y': yPoint[i],
                    'width': width,
                    'height': height,
                    'fill': yAxis.alternateGridBand.even.fill,
                    'opacity': yAxis.alternateGridBand.even.opacity,
                    'stroke': "transparent",
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                };
                this.svgRenderer.drawRect(yoptionsAlternateeven, gAgb);
            }
            $(gAgb).appendTo(gEle);
        }

     
        if (yAxis.minorGridLines.visible && yAxis.minorTicksPerInterval > 0) {
            //Drawning Minor Grid Lines
            yMinorGridDir = sbYMinorGrid.toString();
            var optionsMinorGrid = {
                'id': this.svgObject.id + '_YAxisMinorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.minorGridLines.width,
                'stroke': yAxis.minorGridLines.color,
                'stroke-dasharray': yAxis.minorGridLines.dashArray,
                'd': yMinorGridDir
            };
            this.svgRenderer.drawPath(optionsMinorGrid, gEle);

        }

        //Drawing Major Grid Lines
        yMajorGridDir = sbYMajorGrid.toString();
        if (yAxis.majorGridLines.visible) {
            var options = {
                'id': this.svgObject.id + '_YAxisMajorGridLines_' + axisIndex,
                'fill': 'none',
                'stroke-width': yAxis.majorGridLines.width,
                'opacity': yAxis.majorGridLines.opacity,
                'stroke-dasharray': yAxis.majorGridLines.dashArray,
                'stroke': yAxis.majorGridLines.color,
                'd': yMajorGridDir
            };

            if(options.d != '')
                this.svgRenderer.drawPath(options, gEle);

        }
        $(gEle).appendTo(this.gYaxisEle);
    },
	
    _drawXAxisLabels: function (axisIndex, xAxis) {
        var gTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels' + '_' + axisIndex, 'cursor': 'default' });
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisLabels' + '_' + axisIndex, 'cursor': 'default' });
        var prevLabels = [];
        var axisName = (xAxis.name).replace(/[^a-zA-Z0-9]/g, "");
        if (axisIndex != 0 && ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside')||(xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside')) && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {

            xAxis.y = (!xAxis.opposedPosition) ? Math.floor(xAxis.y + xAxis._LableMaxWidth.height) : Math.floor(xAxis.y - xAxis._LableMaxWidth.height);
        }
        else if (axisIndex != 0 && (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && (!(xAxis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y) || (xAxis.y == this.model.m_AreaBounds.Y)))) {
            xAxis.y = (!xAxis.opposedPosition) ? Math.floor(xAxis.y+xAxis.majorTickLines.size) : Math.floor(xAxis.y-xAxis.majorTickLines.size);
        }
        else
            xAxis.y = Math.floor(xAxis.y);  
       
        // loop to draw labels
        for (var i = 0; i < xAxis.visibleLabels.length; i++) {
            var label = xAxis.visibleLabels[i];
            var labelText = label.displayText = ej.util.isNullOrUndefined(label.displayText) ? label.Text : label.displayText;
            if (ej.EjSvgRender.utils._inside(label.Value, xAxis.visibleRange)) {
                //Declaration
                var textanchor = null;
                var textcollection = [];
                    var gap = xAxis.width / xAxis.visibleLabels.length;       //space between ticks
					 var trimGap = xAxis.maximumLabelWidth // this holds maximumLabelWidth value      
                    var range = xAxis.visibleRange;
                    var areaBoundWidth = this.model.m_AreaBounds.Width;
                    var pointX = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(label.Value, range, xAxis.isInversed) * (xAxis.width)));
                    var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font);
                    var textWidth = textSize.width;
                    var textHeight = textSize.height;
                    var labelMaxWidth = xAxis._LableMaxWidth;
                    var labelMaxHeight = labelMaxWidth.height;
                    var labelMaxRow = labelMaxWidth.rows;
                    var vmlrendering = this.svgRenderer.vmlNamespace;
                    var opposedPosition = xAxis.opposedPosition;
                    var lineWidth = xAxis.axisLine.width;
                    var font = xAxis.font;
					var labelIntersectAction = xAxis.labelIntersectAction;
                    var labelPlacement = xAxis.labelPlacement.toLowerCase();
					var labelRotation = xAxis.rotationValue;
                    var count = 0;
                    var flag = false;
                    this.edgeLabel = false; // flag to find the supprot for edge labels
                    var chartAreaWidth = this.model.m_AreaBounds.Width;
                    var chartAreaX = this.model.m_AreaBounds.X;
                    var intersectAction = xAxis.labelIntersectAction.toLowerCase();
                    var svgWidth = $(this.svgObject).width();                   
                    //To Perform trim 
                    if (intersectAction == "trim" || xAxis.enableTrim) {
                        if (labelPlacement == "onticks" && i != xAxis.visibleLabels.length -1) {
                            var nextLabel = xAxis.visibleLabels[i+1];
                            var nextPoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(nextLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                            var nextTextWidth = ej.EjSvgRender.utils._measureText(nextLabel.Text, areaBoundWidth, xAxis.font).width;
                            var value = nextPoint - nextTextWidth / 2;
                            var count = 0, current = false;
                            while (value < pointX + textWidth / 2 &&  count<2) {             // intersect
                                count++; 
                                if (nextTextWidth > textWidth) {                            // getting max label
                                    otherLabel = trimLabel = nextLabel;
                                    current = false;
                                }
                                else {
                                    trimLabel = label;
                                    current = true;
                                }
                                var trimLabelText = trimLabel.displayText = trimLabel.Text.toString();
                                for (var t = 1; t < trimLabelText.length; t++) {
                                    trimLabelText = trimLabel.displayText.substring(0, t) + '... ';
                                    newTextWidth = ej.EjSvgRender.utils._measureText(trimLabelText, areaBoundWidth, font).width;
                                    if (!current) {                                                  // to trim next label
                                        value = nextPoint - newTextWidth / 2;
                                        if (value < pointX + textWidth / 2) {
                                            if (t - 1 != 0) 
                                                trimLabelText = trimLabelText.toString().substring(0, t - 1) + '... ';
                                            nextTextWidth = newTextWidth;
                                            trimLabel.displayText = trimLabelText;
                                            break;
                                        }
                                    }
                                    else {                                                           // to trim current label
                                        value = nextPoint - nextTextWidth / 2;
                                        if (value < pointX + newTextWidth / 2) {
                                            if (t - 1 != 0) 
                                                trimLabelText = trimLabelText.toString().substring(0, t - 1) + '... ';
                                            textWidth = newTextWidth;
                                            labelText = trimLabel.displayText = trimLabelText;
                                            break;
                                        }
                                    }
                                }
                            }
                        } else if (labelPlacement == "betweenticks") {
                            var derivedGap = (xAxis.enableTrim) ? trimGap : gap;
                        if (textWidth > derivedGap && labelText != "") { // check textwidth is greater than gap
                            for (var t = 1; t < labelText.toString().length; t++) {
                                labelText = label.displayText.toString().substring(0, t) + '... ';
                                textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, font).width;                                
                                if (textWidth >= derivedGap) {
                                    labelText = labelText.toString().substring(0, t - 1) + '... ';
                                    label.displayText = labelText;
                                    if (i == xAxis.visibleLabels.length - 1) {
                                        if (((pointX + xAxis.x) + (textWidth / 2) > (svgWidth)) && derivedGap==gap ) {
                                            derivedGap -= (textWidth / 2);
                                            t = 1;
                                            continue;
                                        }
                                    }
                                    break;
                                }
                            }
                            label.displayText = labelText; // trimed text is assigned to displayText
                        }
                      }
                    }
                 var textSize = ej.EjSvgRender.utils._measureText(label.displayText, areaBoundWidth, font);                    
                 var anchor = textSize.width / 2;
                 if (!ej.util.isNullOrUndefined(label.Text)) {
                    var x = pointX + xAxis.x - anchor;
                    if ((xAxis.labelPosition == "inside" && xAxis.tickLinesPosition == 'inside') || ((xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') && xAxis.labelIntersectAction == 'multiplerows'))
                    {                        
                        y = (opposedPosition) ? (xAxis.y + xAxis.majorTickLines.size + textHeight / 2 + (lineWidth / 2)
                       + (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y - xAxis.majorTickLines.size - (lineWidth / 2)
                      - (labelMaxHeight / (2 * labelMaxRow)) + 5);
                    }                        
                    else if (xAxis.labelPosition == 'inside'&& xAxis.tickLinesPosition!='inside')
                    {                        
                          y = (opposedPosition) ? (xAxis.y + textHeight / 2 + (lineWidth / 2)
                       + (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y - (lineWidth / 2)
                      - (labelMaxHeight / (2 * labelMaxRow)) + 5);                    
                    }
                    else if ((xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition != 'inside') || ((xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') && xAxis.labelIntersectAction == 'multiplerows')) {
                        var y = (opposedPosition) ? (xAxis.y - xAxis.majorTickLines.size + textHeight / 2 - (lineWidth / 2)
                            - (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y + xAxis.majorTickLines.size + (lineWidth / 2)
                            + (labelMaxHeight / (2 * labelMaxRow)) + 5);
                    }
                    else if (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside')
                    {                      
                            var y = (opposedPosition) ? (xAxis.y + textHeight / 2 - (lineWidth / 2)
                             - (labelMaxHeight / (2 * labelMaxRow)) - 5) : (xAxis.y + (lineWidth / 2)
                             + (labelMaxHeight / (2 * labelMaxRow)) + 5);                       
                    }
                   
                    
                    if (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition != 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y  : y - xAxis.majorTickLines.size;
                    }
                    else if (xAxis.labelPosition != 'inside' && xAxis.tickLinesPosition == 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y - xAxis.majorTickLines.size : y - xAxis.majorTickLines.size;
                    }
                    else if (xAxis.labelPosition == 'inside' && xAxis.tickLinesPosition == 'inside') {
                        if (xAxis.labelRotation == 90 || xAxis.labelIntersectAction == 'rotate90')
                            y = (opposedPosition) ? y : y - xAxis.majorTickLines.size;
                    }

                        var options = {
                            'id': this.svgObject.id + '_' + axisName + '_XLabel_' + i,
                            'x': x,
                            'y': y,
                            'fill': font.color,
                            'font-size': font.size,
                            'font-family': font.fontFamily,
                            'font-style': font.fontStyle,
                            'font-weight': font.fontWeight,
                            'opacity': font.opacity
                        };
              
                    // To set position of edge labels
                        if (xAxis.edgeLabelPlacement) {
                            var position = xAxis.edgeLabelPlacement.toLowerCase();
                            var lastLabel = xAxis.visibleLabels.length - 1;
                            switch (position) {
                                case "none":
                                    break;
                                case "shift":
                                    {
                                        if (i == 0) {  // Shift first label
                                            if (options.x < xAxis.x) {
                                                var diff = xAxis.x - options.x;
                                                options.x = x = options.x + diff;
                                                this.diff = diff;
                                                this.edgeLabel = true;
                                            }
                                        } else if (i == lastLabel)  // Shift last label
                                            if (options.x + textSize.width > xAxis.x + xAxis.width) {
                                                diff = (options.x + textSize.width) - (xAxis.x + xAxis.width);
                                                options.x = x = options.x - diff;
                                                this.diff = diff;
                                                this.edgeLabel = true;
                                            }
                                        break;
                                    }
                                  case "hide": // to hide the edge labels
                                    if ((i == 0 && options.x < xAxis.x) || (i == lastLabel && options.x + textWidth >  chartAreaWidth + chartAreaX)) {
                                        labelText = "";
                                    }
                            }
                        }
                    //To perform rotation             
                         if (labelRotation != 0 && !vmlrendering)
                             this.labelRotation(xAxis, x + anchor, y, options, label, labelRotation);                                                     

                    //To perform labelIntersect Actions none , wrap, hide ,multiplerows
                         if (xAxis.labelIntersectAction || xAxis.enableTrim) {
                            this.diff = ej.util.isNullOrUndefined(this.diff) ? 0 : this.diff;
                            var intersectAction = xAxis.labelIntersectAction.toLowerCase();
                            if (intersectAction == 'none' && !xAxis.enableTrim) { 
                               
                                this.svgRenderer.drawText(options, labelText, gTickEle);
                                if(xAxis.labelPosition=='inside')
                                    $(gTickEle).appendTo(this.gXaxisTickEle);
                                else
                                    $(gTickEle).appendTo(this.gXaxisEle);
                                continue;
                            } 
                            else if (intersectAction == 'wrap') {
                                
                                var line = 0;
                                var textcoll=this.rowscalculation(labelText.toString(), areaBoundWidth,xAxis,gap,options);
                                var count = textcoll.length;
                                if (xAxis.labelPosition != 'inside')
                                    options.y = opposedPosition ? options.y + textHeight : options.y - textHeight;
                                else
                                    options.y = opposedPosition ? options.y - textHeight : options.y - textHeight * (count);
                                for (k = 0; k < textcoll.length; k++)
                                {
                                    if (xAxis.labelPosition != 'inside')
                                    {
                                        options.y = xAxis.opposedPosition ? options.y - (textHeight) : options.y + (textHeight);
                                        this.svgRenderer.drawText(options, textcoll[k], gTickEle);
                                    }
                                    else
                                    {                                        
                                        options.y = xAxis.opposedPosition ? options.y + (textHeight) : options.y + (textHeight);
                                        this.svgRenderer.drawText(options, textcoll[k], gTickEle);
                                        count -= 1;
                                    }

                                }
                                
                                continue;
                            } 
                            else if (intersectAction == 'hide') {
                                var xAxisLabelLength = xAxis.visibleLabels.length - 1;
                                for (var j = 0; j < i; j++) {
                                    // loop to get previous label
                                    var preLabel = xAxis.visibleLabels[j];
                                    var temp = 0;
                                    opposedPosition = xAxis.opposedPosition;
                                    areaBoundWidth = this.model.m_AreaBounds.Width;
                                   
                                    var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                    var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                    var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                    value = (j == 0) ? value + this.diff : value;
                                    var width = (preTextWidth == 0) ? preTextWidth : textWidth;
                                    if (i != xAxisLabelLength)
                                        width = width / 2;
                                    if (intersectAction == 'hide' && ((xAxis.isInversed) ? (value < pointX + width) : (value > pointX - width)))
                                        label.displayText = labelText = '';
                                }
                            }
                            else if(intersectAction == 'multiplerows'){ // to perform multipleRows on intersect
                                for (var k = 0; k < prevLabels.length && !flag; k++) {
                                    var xAxisLabelLength = xAxis.visibleLabels.length - 1;
                                    if (i != xAxisLabelLength)
                                        textWidth = textWidth / 2;
                                    for (var l = 0; prevLabels[k] && l < prevLabels[k].length; l++) {
                                        var preLabel = prevLabels[k][l];
                                        var temp = 0;
                                        opposedPosition = xAxis.opposedPosition;
                                        areaBoundWidth = this.model.m_AreaBounds.Width;

                                        var prePoint = Math.abs(Math.floor(ej.EjSvgRender.utils._getPointXY(preLabel.Value, range, xAxis.isInversed) * (xAxis.width)));
                                        var preTextWidth = ej.EjSvgRender.utils._measureText(preLabel.displayText, areaBoundWidth, xAxis.font).width;
                                        var value = prePoint + ((xAxis.isInversed) ? -preTextWidth / 2 : preTextWidth / 2);
                                        if (this.edgeLabel || prevLabels.length==1) {
                                            value = value + this.diff;
                                        }
                                        if ((xAxis.isInversed) ? (value < pointX + textWidth) : (value > pointX - textWidth)) {
                                            if (opposedPosition) {
                                                if (xAxis.labelPosition != 'inside')
                                                    temp = options.y - textHeight;
                                                else
                                                    temp = options.y + textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;
                                            } else {
                                                if (xAxis.labelPosition != 'inside')
                                                    temp = options.y + textHeight;
                                                else
                                                    temp = options.y - textHeight;
                                                options.y = temp;
                                                if (k + 1 == prevLabels.length)
                                                    flag = true;
                                                count++;
                                                break;
                                              
                                            }
                                        }
                                        else {
                                            if (l + 1 == prevLabels[k].length) {
                                                flag = true;
                                                break;
                                            }
                                        }                           
                                    }
                                }
                                if (temp == undefined) {
                                    if (prevLabels[0] == undefined)
                                        prevLabels[0] = [];
                                    prevLabels[0].push(label);
                                }
                                else {                                  
                                    if (prevLabels[count] == undefined)
                                        prevLabels[count] = [];
                                    prevLabels[count].push(label);
                                }
                         }         
                        }                  
                    if ((!this.edgeLabel) && !this.zoomed && !xAxis.zoomed && !(labelRotation) && !(labelIntersectAction == "rotate90") && !(labelIntersectAction == "rotate45") && (((pointX + xAxis.x) + (textWidth/2)) > ($(this.svgObject).width()))) {
                        $(options).attr('x', ($(this.svgObject).width() - 2));
                        $(options).attr('text-anchor', 'end');
                    }
					xAxis.visibleLabels[i].y = options.y;
					
					if(this.model.enableCanvasRendering && xAxis.labelPosition == "inside" && (xAxis.labelIntersectAction == "rotate45" || xAxis.labelIntersectAction == "rotate90")){
					options.y = options.y - textWidth;
					}
					
                    this.svgRenderer.drawText(options, labelText, gTickEle);
					 //create region to chart axis labels
                    var labelSize = ej.EjSvgRender.utils._measureText(labelText, null, font);
                    var bounds = { x: options.x, y: options.y, width: labelSize.width, height: labelSize.height };
                    var xAxisLabelRegions = { bounds: bounds, trimText: labelText, labelText: label.Text };
                    this.model.xAxisLabelRegions.push(xAxisLabelRegions)
                }
            }
        }
        if(xAxis.labelPosition=='inside')
            $(gTickEle).appendTo(this.gXaxisTickEle);
        else
            $(gTickEle).appendTo(this.gXaxisEle);

    },
	
    rowscalculation: function (labelText, areaBoundWidth,xAxis,gap,options)
    {
        var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font);
        var textWidth = textSize.width;
        var anchor = textWidth / 2;
        var text = labelText.toString();
        var textLength = text.length;
        var textHeight = textSize.height;
        var textcollection = [];
        i = 0;
        var line = 0;
        
        if (this.edgeLabel) gap = (i == 0) ? gap - this.diff : gap;
        if (textWidth > gap) {
            options.x = options.x + anchor;
            var line = 0;
            if (this.edgeLabel) options.x = (i == 0) ? options.x - this.diff : options.x + this.diff;
            //options.x = options.x + anchor;
            for (var w = 1; w <= text.length; w++) {
                labelText = text.substring(0, w);
                textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font).width;
                if (textWidth > gap) {
                    line = line + 1; // To find the no of rows splitted
                    count = +1;                    
                    labelText = text.substring(0, w - 1);
                    textcollection[i] = labelText;
                    text = text.slice(w - 1, textLength);                    
                    var newTextWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, xAxis.font).width;
                    if (line == 1) // Check for first label
                        options.x = options.x - newTextWidth / 2;
                    i++;
                    w = 0;
                }                
            }           
        }
        textcollection[i] = labelText;
        return textcollection;
    },

	labelRotation: function (axis, x, y, options, label, degree, i) {
	        // To rotate axis labels
	        var labelText = (label.displayText) ? label.displayText : label.Text;
            var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
            var rotate = 'rotate(' + angle + ',' + (x) + ',' + y + ')';
            $(options).attr('transform', rotate);
			$(options).attr('labelRotation', angle);
            var textElement = this.svgRenderer.createText(options, labelText);
            var diffHeight = Math.ceil(ej.EjSvgRender.utils._measureBounds(textElement, this).height - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height);
            diffHeight = axis._LableMaxWidth.height - diffHeight - ej.EjSvgRender.utils._measureText(labelText, null, axis.font).height;
            
        if(axis.labelPosition!='inside')
            var yLocation = (axis.opposedPosition) ? (diffHeight / 2) : (-diffHeight / 2);
        else
            var yLocation = (axis.opposedPosition) ? (-diffHeight / 2) : (diffHeight / 2);

        rotate = 'rotate(' + angle + ',' + (x) + ',' + (y + yLocation) + ')';
        if (this.edgeLabel && degree == 90)               
            y = (i == 0) ? options.y + this.diff : options.y - this.diff;
       
	        $(options).attr({
                'transform' :rotate,
                'y': (y + yLocation)
            });
        },
		
    _drawYAxisLabels: function (axisIndex, yAxis) {
        // method to draw y axis labels
        //if(yAxis.labelPosition=='inside')
        var axisName = (yAxis.name).replace(/[^a-zA-Z0-9]/g, "");
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisLabels' + '_' + axisIndex, 'cursor': 'default' });        
        if (axisIndex != 1 && ((yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition == 'inside') || (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside'))&& !((yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))))
        {
            yAxis.x = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis._LableMaxWidth.width - this.model.elementSpacing) : Math.floor(yAxis.x + yAxis._LableMaxWidth.width + this.model.elementSpacing);
        }
        else if (axisIndex != 1 && (!(yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))) && (yAxis.labelPosition!='inside' && yAxis.tickLinesPosition=='inside'))
        {
            yAxis.x = (!yAxis.opposedPosition) ? Math.floor(yAxis.x - yAxis.majorTickLines.size) : Math.floor(yAxis.x + yAxis.majorTickLines.size);
        }      
        else
        {
            yAxis.x = Math.floor(yAxis.x);
        }   
      
        for (var i = 0; i < yAxis.visibleLabels.length; i++) {
            var label = yAxis.visibleLabels[i];
			var visibleRange = yAxis.visibleRange;
			if (ej.EjSvgRender.utils._inside(label.Value, visibleRange)) {
			    var pointY = Math.abs(ej.EjSvgRender.utils._getPointXY(label.Value, visibleRange, yAxis.isInversed) * (yAxis.height));
			    var text = ej.EjSvgRender.utils._measureText(label.Text, this.model.m_AreaBounds.Height, yAxis.font);
			    var textHeight = text.height;
			    var opposedPosition = yAxis.opposedPosition;
			    var font = yAxis.font;
			    var range = yAxis.visibleRange;
			    var lineWidth = yAxis.axisLine.width;
			    var labelMaxWidth = yAxis._LableMaxWidth;
			    var labelMaxHeight = labelMaxWidth.height;
			    var labelMaxRow = labelMaxWidth.rows;
               var areaBoundWidth = this.model.m_AreaBounds.Width;
               var gap = yAxis.maximumLabelWidth;  // width for 4 digits
               var labelText = label.displayText = label.Text;
			    var textSize = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, yAxis.font);
			    var textWidth = textSize.width;
			    var font = yAxis.font;
				var degree = yAxis.labelRotation;

			    if(yAxis.labelPosition=='inside' && yAxis.tickLinesPosition=='inside')
			    {
			        var x = (opposedPosition) ? (yAxis.x - (yAxis.majorTickLines.size) - (lineWidth / 2) - 5)
               : (yAxis.x + (lineWidth / 2) + 5 + yAxis.majorTickLines.size);
			        var textanchor = (opposedPosition) ? 'end' : 'start';			       
			    }
			    else if (yAxis.labelPosition == 'inside' && yAxis.tickLinesPosition != 'inside')
			    {			        
			        var x = (opposedPosition) ? (yAxis.x - (lineWidth / 2)  - 5)
                : (yAxis.x + (lineWidth / 2) + 5);
			        var textanchor = (opposedPosition) ? 'end' : 'start';			       
			    }
			    else if (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside')
			    {

			        var x = (opposedPosition) ? ((lineWidth / 2) + yAxis.x + 5)
                        : (yAxis.x - (lineWidth / 2) - 5);
			        var textanchor = (opposedPosition) ? 'start' : 'end';
			        
			    }
			    else
			    {
			        var x = (opposedPosition) ? ((yAxis.majorTickLines.size) + (lineWidth / 2) + yAxis.x + 5)
                        : (yAxis.x - (lineWidth / 2) - 5 - yAxis.majorTickLines.size);
			        var textanchor = (opposedPosition) ? 'start' : 'end';			    
			   }                       

                    var y = ((pointY * -1) + (yAxis.y + yAxis.height) + textHeight/4);
                    var options = {
                        'id': this.svgObject.id + '_' + axisName + '_YLabel_' + i,
                        'x': x,
                        'y': y,
                        'fill': font.color,
                        'font-size': font.size,
                        'font-family': font.fontFamily,
                        'font-style': font.fontStyle,
                        'font-weight': font.fontWeight,
                        'opacity': font.opacity,
                        'text-anchor': !ej.util.isNullOrUndefined(degree) && degree != 0 ? "middle" : textanchor
                    };
					
                // To set position of edge labels in yAxis
                    if (yAxis.edgeLabelPlacement) {
                        var position = yAxis.edgeLabelPlacement.toLowerCase();
                        var lastLabel = yAxis.visibleLabels.length - 1;

                        switch (position) {
                            case "none":
                                break;
                            case "shift":
                                {
                                    if (i == 0) {  // Shift first label
                                        if (options.y > yAxis.y + yAxis.height) {
                                            var diff = options.y - (yAxis.y + yAxis.height);
                                            options.y = y = options.y - diff;
                                            this.diff = diff;
                                            this.edgeLabel = true;
                                        }
                                    } else if (i == lastLabel) // Shift last label
                                        if (options.y - textHeight/2 < yAxis.y) {
                                            diff = yAxis.y - (options.y - textHeight / 2);
                                            options.y = y = options.y + diff;
                                            this.diff = diff;
                                            this.edgeLabel = true;
                                        }
                                    break;
                                }
                            case "hide": //to hide the edge labels
                                if ((i == 0 && options.y > yAxis.y + yAxis.height) || (i == lastLabel && options.y - textHeight / 2 < this.model.m_AreaBounds.Y)) {
                                    labelText = label.Text = "";
                                }
                        }
                    }

			      //To perform rotation         
                        if (!ej.util.isNullOrUndefined(degree) && degree != 0 && !this.svgRenderer.vmlNamespace){
						    if (yAxis.labelPosition!='inside')
                               x = (opposedPosition) ? (x + textWidth/2) : (x - textWidth/2);
                            else
                               x = (opposedPosition) ? (x - textWidth/2) : (x + textWidth/2);
							   
						    var labelText = (label.displayText) ? label.displayText : label.Text;
                            var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
                            var rotate = 'rotate(' + angle + ',' + (x) + ',' + y + ')';
                            $(options).attr('transform', rotate);
			                $(options).attr('labelRotation', angle);
                            var textElement = this.svgRenderer.createText(options, labelText);
                            var diffHeight = Math.ceil(ej.EjSvgRender.utils._measureBounds(textElement, this).width - 
							                           ej.EjSvgRender.utils._measureText(labelText, null, yAxis.font).width);
            
                            if (yAxis.labelPosition!='inside')
                                var yLocation = (opposedPosition) ? (diffHeight / 2) : (-diffHeight / 2);
                            else
                                var yLocation = (opposedPosition) ? (-diffHeight / 2) : (diffHeight / 2);

                           rotate = 'rotate(' + angle + ',' + (x + yLocation) + ',' + (y) + ')';
       
	                        $(options).attr({
                                'transform' :rotate,
                                'x': (x + yLocation)
                            });
						}
				 //Label intersection
                    if (yAxis.labelIntersectAction || yAxis.enableTrim) {
                        if (intersectAction == 'none' && !(yAxis.enableTrim)) {
                            this.svgRenderer.drawText(options, label.Text, gEle);
                            continue;
                        }
                        else if (yAxis.enableTrim) {
                            if (textWidth > gap && labelText != "") { // check textwidth is greater than gap
                                for (var t = 1; t < labelText.toString().length; t++) {
                                    labelText = label.displayText.toString().substring(0, t - 1) + '...';
                                    textWidth = ej.EjSvgRender.utils._measureText(labelText, areaBoundWidth, font).width;
                                    if (textWidth >= gap) {
                                        labelText = labelText.toString().substring(0, t - 1) + '...';                                      
                                        break;
                                    }
                                }
                                label.displayText = labelText;
                            }
                        }
                        var intersectAction = yAxis.labelIntersectAction.toLowerCase();
                        for (var j = 0; j < i; j++) {                           
                            var prevLabel = yAxis.visibleLabels[j];
                            var temp = 0;
                            var prevPoint = Math.abs((ej.EjSvgRender.utils._getPointXY(prevLabel.Value, range, yAxis.isInversed) * (yAxis.height)));
                                var prevTextHeight = ej.EjSvgRender.utils._measureText(prevLabel.Text, this.model.m_AreaBounds.Height, font).height;
                                var value = ((prevPoint * -1) + (yAxis.y + yAxis.height) + prevTextHeight/4);
                                value = value + ((yAxis.isInversed) ? prevTextHeight / 2 : -prevTextHeight / 2);
                            if (this.edgeLabel)
                                value = (j == 0) ? value - prevTextHeight / 4 : value;
                                if (intersectAction == 'hide' && prevLabel.Text.toString()!="" && ((yAxis.isInversed) ? value > options.y - textHeight / 4 : value < options.y + textHeight / 4))
                                    labelText=label.Text = '';
                                else if (intersectAction == 'multiplerows' && ((yAxis.isInversed) ? value > options.y - textHeight / 4 : value < options.y + textHeight / 4) && prevLabel.x == options.x) {
                                    if (opposedPosition) {
                                        if(yAxis.labelPosition !="inside")
                                            temp = prevLabel.x + yAxis._LableMaxWidth.maxWidth + 5;  // gap between the rows
                                        else
                                            temp = prevLabel.x - yAxis._LableMaxWidth.maxWidth - 5;                                            
                                        options.x = temp;
                                    }
                                    else {
                                        if (yAxis.labelPosition != "inside")
                                            temp = prevLabel.x - yAxis._LableMaxWidth.maxWidth - 5; // gap between the rows
                                        else
                                            temp = prevLabel.x + yAxis._LableMaxWidth.maxWidth + 5;
                                        options.x = temp;
                                     }
                                }
                            }
                        }

                    yAxis.visibleLabels[i].x = options.x;	
                    this.svgRenderer.drawText(options, labelText, gEle);
                      //create region to chart axis labels
                   var labelSize = ej.EjSvgRender.utils._measureText(labelText, null, font);
                   
                   if (yAxis.opposedPosition)
                       var x = options.x;
                    else
                        x = options.x - labelSize.width;
                   
                    if (yAxis.enableTrim){				   // yAxis labels region stored when enableTrim is true
                       var bounds = { x: x, y: options.y, width: labelSize.width, height: labelSize.height };
                       var yAxisLabelRegions = { bounds: bounds, trimText: labelText, labelText: label.Value };
                       this.model.yAxisLabelRegions.push(yAxisLabelRegions)
					}

            }
        }
        if(yAxis.labelPosition=='inside')
            $(gEle).appendTo(this.gYaxisTickEle);
        else
            $(gEle).appendTo(this.gYaxisEle);
    },
	//To Trim axis title
	 trimText: function (text, maxLength, ellipsis) {
        maxLength--;
        text = text.substring(0, maxLength - ellipsis.length);
        return text + ellipsis;
    },
	
    _drawXTitle: function (axisIndex, axis) {

        if (axis.title.text != "" && axis.title.text && axis.title.visible) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex,'cursor': 'default'});
            var xtitleLocation = (this.model.elementSpacing) + axis._LableMaxWidth.height;
            var axisTitleSize = ej.EjSvgRender.utils._measureText(axis.title.text, (axis.width), axis.title.font);
            var titlesize = axisTitleSize.height / 2;
			var titleText = axis.title.text;
            var isEnableTrim = axis.title.enableTrim;
			var axisTitleWidth=axis.title.maximumTitleWidth;
            if(isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
                var maxTitleWidth = (!axisTitleWidth) ? axis.width - (this.model.elementSpacing * 2) : axisTitleWidth;
			    while (axisTitleSize.width > maxTitleWidth) {
			        titleText = this.trimText(titleText, titleText.length, "...");
			        axisTitleSize = ej.EjSvgRender.utils._measureText(titleText, (axis.width), axis.title.font);
			        if (titleText == "...") {
			            titleText = "";
			            break;
			        }
			    }
			}			  
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);

            var y = this.model.elementSpacing + axis._LableMaxWidth.height + titlesize + axis.majorTickLines.size + axis.axisLine.width;

            if ((axis.tickLinesPosition == 'inside' && axis.labelPosition == 'inside') && (axisIndex == 0 || (axis.y == (this.model.m_AreaBounds.Height + this.model.m_AreaBounds.Y)) || (axis.y == (this.model.m_AreaBounds.Y))))
                var y =this.model.elementSpacing + titlesize + axis.axisLine.width;
            else if ((axisIndex == 0 || (axis.y == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (axis.y == (this.model.m_AreaBounds.X))) && (axis.tickLinesPosition != 'inside' && axis.labelPosition == 'inside'))
                var y = this.model.elementSpacing + titlesize +axis.majorTickLines.size+ axis.axisLine.width;
            else if ((axisIndex == 0 || (axis.y == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (axis.y == (this.model.m_AreaBounds.X))) && (axis.tickLinesPosition == 'inside' && axis.labelPosition != 'inside'))
                var y = this.model.elementSpacing + axis._LableMaxWidth.height+titlesize + axis.axisLine.width;
            else if ((axis.tickLinesPosition != 'inside' && axis.labelPosition != 'inside') || (axisIndex != 1))
                var y = this.model.elementSpacing + axis._LableMaxWidth.height + titlesize + axis.majorTickLines.size + axis.axisLine.width;

            commonEventArgs.data = { title: axis.title.text, location: { x: (axis.width) / 2, y: y }, axes: axis };
            this._trigger("axesTitleRendering", commonEventArgs);
            var options = {
                'id': this.svgObject.id + '_XAxisTitle' + '_' + axisIndex,
                'x': commonEventArgs.data.location.x + axis.x,
                'y': ((axis.opposedPosition) ? (axis.y - commonEventArgs.data.location.y) : (commonEventArgs.data.location.y + axis.y))  + axisTitleSize.height / 3.5,
                'fill': axis.title.font.color,
                'font-size': axis.title.font.size,
                'font-family': axis.title.font.fontFamily,
                'font-style': axis.title.font.fontStyle,
                'font-weight': axis.title.font.fontWeight,
                'opacity': axis.title.font.opacity,
                'text-anchor': 'middle'
            };
            if (this.model.enableCanvasRendering) {
                var matched = jQuery.uaMatch(navigator.userAgent);
                var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
                var browser = matched.browser.toLowerCase();
                 if (browser != "msie" && !isIE11)
                     options.y = !axis.opposedPosition ? options.y + (titlesize / 2) + (browser == "mozilla" ? 1 : -2) : options.y + (titlesize / 2) + 1; // text dominant baseline for canvas
            }
            if ($(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).length > 0)
                $(this.svgObject).find("#" + this.svgObject.id + '_XAxisTitle' + '_' + axisIndex).attr(options);
            else if (!isEnableTrim)
                this.svgRenderer.drawText(options, commonEventArgs.data.title, gEle);
		    else if(isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
                var bounds = { X: options.x - (axisTitleSize.width) / 2, Y: options.y, Width: axisTitleSize.width, Height: axisTitleSize.height };
                var xAxisTitleRegion = { Bounds: bounds, trimText: titleText, labelText: axis.title.text };
                this.model.xAxisTitleRegion.push(xAxisTitleRegion)
                this.svgRenderer.drawText(options, titleText, gEle);
            }
            $(gEle).appendTo(this.gXaxisEle);
			}
        },
    _drawYTitle: function (axisIndex, yAxis) {

        if (yAxis.title.text != "" && yAxis.title.text && yAxis.title.visible) {
            var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTitle' + '_' + axisIndex,'cursor': 'default' });
            var titleSize = ej.EjSvgRender.utils._measureText(yAxis.title.text, (yAxis.height), yAxis.title.font);
            if ((yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition == 'inside') && (axisIndex == 1 || (yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis.axisLine.width-5) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis.axisLine.width);
            else if ((axisIndex == 1 || (yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))) && (yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition == 'inside'))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis.majorTickLines.size + yAxis.axisLine.width) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis.majorTickLines.size + yAxis.axisLine.width);
            else if ((axisIndex == 1 || (yAxis.x == (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) || (yAxis.x == (this.model.m_AreaBounds.X))) && (yAxis.tickLinesPosition == 'inside' && yAxis.labelPosition != 'inside'))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.axisLine.width) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.axisLine.width);
            else if ((yAxis.tickLinesPosition != 'inside' && yAxis.labelPosition != 'inside')||(axisIndex!=1))
                var x = (yAxis.opposedPosition) ? yAxis.x + ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.majorTickLines.size + yAxis.axisLine.width) : yAxis.x - ((3 * this.model.elementSpacing) + yAxis._LableMaxWidth.width + yAxis.majorTickLines.size + yAxis.axisLine.width);
            var titleText = yAxis.title.text;
            var isEnableTrim = yAxis.title.enableTrim;
            var axisTitleWidth = yAxis.title.maximumTitleWidth;
            var axisTitleSize = ej.EjSvgRender.utils._measureText(titleText, (yAxis.height), yAxis.title.font);
            if(isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
                var maxTitleWidth = (!axisTitleWidth)? yAxis.height - (this.model.elementSpacing * 2) : axisTitleWidth;                
                while (axisTitleSize.width > maxTitleWidth) {
                    titleText = this.trimText(titleText, titleText.length, "...");
                    axisTitleSize = ej.EjSvgRender.utils._measureText(titleText, (yAxis.height), yAxis.title.font);
                    if (titleText == "..."){
                        titleText = "";
                        break;
                    }
                        
                }
            }
            if (document.documentMode === 8)
                x = (!yAxis.opposedPosition) ? (x - axisTitleSize.width / 2 + this.model.elementSpacing) : (x + axisTitleSize.width / 2 - this.model.elementSpacing);
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { title: yAxis.title.text, location: { x: titleSize.width / 2, y: 0 }, axes: yAxis };
            this._trigger("axesTitleRendering", commonEventArgs);

            var y = commonEventArgs.data.location.y + (yAxis.y + yAxis.height) + (((yAxis.height) / 2) * -1);

            var labelRotation = (yAxis.opposedPosition) ? 90 : -90;

            var options = {
                'id': this.svgObject.id + '_YAxisTitle' + '_' + axisIndex,
                'x': x,
                'y': y,
                'fill': yAxis.title.font.color,
				'labelRotation': labelRotation,
                'transform': 'rotate(' + labelRotation + ',' + (x) + ',' + y + ')',
                'font-size': yAxis.title.font.size,
                'font-family': yAxis.title.font.fontFamily,
                'font-style': yAxis.title.font.fontStyle,
                'font-weight': yAxis.title.font.fontWeight,
                'opacity': yAxis.title.font.opacity,
                'text-anchor': 'middle',
                'dominant-baseline': 'middle'
            };
             if (this.model.enableCanvasRendering) {
                var matched = jQuery.uaMatch(navigator.userAgent);
                var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
                var browser = matched.browser.toLowerCase();
                 if (browser != "msie" && !isIE11)
                    options.x = !yAxis.opposedPosition ? options.x + (titleSize.height / 4) + (browser == "mozilla" ? 1 : -2) : options.x - (titleSize.height / 4) + 1; // text dominant baseline for canvas
            }
            if ($(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).length > 0)
                $(this.svgObject).find("#" + this.svgObject.id + '_YAxisTitle' + '_' + axisIndex).attr(options);
            else if (!isEnableTrim)
                this.svgRenderer.drawText(options, commonEventArgs.data.title, gEle);
			else if(isEnableTrim &&(ej.util.isNullOrUndefined(axisTitleWidth)||axisTitleWidth>0||isNaN(axisTitleWidth))){
                 if (!this.svgRenderer.vmlNamespace)
                    var bounds = { X: options.x - (axisTitleSize.height / 2), Y: options.y + (axisTitleSize.width / 2), Width: (axisTitleSize.height)/2, Height: axisTitleSize.width };
                 else
                    var bounds = { X: options.x - (axisTitleSize.width)/2, Y: options.y + (axisTitleSize.height / 2), Width: (axisTitleSize.width), Height: axisTitleSize.height };
                var yAxisTitleRegion = { Bounds: bounds, trimText: titleText, labelText: yAxis.title.text };
                this.model.yAxisTitleRegion.push(yAxisTitleRegion)
                this.svgRenderer.drawText(options, titleText, gEle);
            }	
            $(gEle).appendTo(this.gYaxisEle);
			}
		}
    },
ej.EjStripline.prototype = {
    _drawStripline: function (axis, stripLine) {

        var start = (stripLine.start < axis.range.min) ? axis.range.min : stripLine.start;
        var end = (stripLine.end > axis.range.max) ? axis.range.max : stripLine.end;
        var x = 0, height = 0, angle = 0;
        var y = 0, pointYstart = 0, yHeight = 0;
        var width = 0, widthValue = 0, textX = 0, textY = 0;
        if (stripLine.visible) {
		     this.model._stripeline = true;
            var striplineid = (stripLine.zIndex == 'over') ? this.gStriplineOver : this.gStriplineBehind;
            striplineid = ej.util.isNullOrUndefined(striplineid) ? "" : striplineid;
            var isVisible = true;
            if (axis.orientation.toLowerCase() == "horizontal") {
                var pointstart = 0;

                if (stripLine.startFromAxis) {

                    var point = (ej.util.isNullOrUndefined(stripLine.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripLine.offset;
                    pointstart = (((point - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);

                    widthValue = (((point + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.width));

                } else {
                    if (ej.util.isNullOrUndefined(start) && (ej.util.isNullOrUndefined(end) || ej.util.isNullOrUndefined(stripLine.width))) {
                        isVisible = false;
                    } else {
                        start = (jQuery.type(start) == "date") ? (start).getTime() : start;
                        if (!ej.util.isNullOrUndefined(end) && (jQuery.type(end) == "date")) {
                            end = (end).getTime();
                        }
                        pointstart = (ej.EjSvgRender.utils._getPointXY(start, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                        var pointend = (ej.EjSvgRender.utils._getPointXY(end, axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x; //($(this.svgObject).width() - this.model.m_Spacing.Left - this.model.m_Spacing.Right);
                        var widthMax = (ej.EjSvgRender.utils._getPointXY((start + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width)), axis.visibleRange, axis.isInversed) * (axis.width)) + axis.x;

                        widthValue = (axis.isInversed ? (pointend < widthMax) : (pointend > widthMax)) ? Math.abs(pointstart - pointend) : Math.abs(pointstart - widthMax);
                        
                    }
                }
                width = widthValue;
                height = this.chart.model.m_AreaBounds.Height;
                x = (axis.isInversed) ? (pointstart - widthValue) : pointstart;
                y = this.chart.model.m_AreaBounds.Y;
                angle = -90;
                var textWidth = ej.EjSvgRender.utils._measureText(stripLine.text, null, stripLine.font).height;
                if (stripLine.textAlignment == 'middletop') {
                    textX = x+textWidth;
                    textY = Math.abs(y + height / 2);
                } else if (stripLine.textAlignment == 'middlecenter') {
                    textX = (Math.floor(x + width / 2) + (textWidth / 3));
                    textY = Math.abs(y + height / 2);
                } else if (stripLine.textAlignment == 'middlebottom') {
                    textX = x + width - (textWidth);
                    textY = Math.abs(y + height / 2);
                }


            } else {
                if (stripLine.startFromAxis) {
                    var pointY = (ej.util.isNullOrUndefined(stripLine.offset)) ? axis.visibleRange.min : axis.visibleRange.min + stripLine.offset;
                    pointYstart = (axis.y + axis.height) - (((pointY - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                    widthValue = (((pointY + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width) - axis.visibleRange.min) / (axis.visibleRange.delta)) * (axis.height));
                    yHeight = widthValue;
                } else {

                    if (ej.util.isNullOrUndefined(start) && (ej.util.isNullOrUndefined(end) || ej.util.isNullOrUndefined(stripLine.width))) {
                        isVisible = false;
                    } else {
                        start = (jQuery.type(start) == "date") ? (start).getTime() : start;
                        if (!ej.util.isNullOrUndefined(end) && (jQuery.type(end) == "date")) {
                            end = (end).getTime();
                        }
                        pointYstart = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY(start, axis.visibleRange, axis.isInversed) * (axis.height));
                        var pointYend = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((ej.util.isNullOrUndefined(end) ? 0 : end), axis.visibleRange, axis.isInversed) * (axis.height));
                        var widthYValue = (axis.y + axis.height) - (ej.EjSvgRender.utils._getPointXY((start + (ej.util.isNullOrUndefined(stripLine.width) ? 0 : stripLine.width)), axis.visibleRange, axis.isInversed) * (axis.height));
                        yHeight = (axis.isInversed ? (pointYend > widthYValue) : (pointYend < widthYValue)) ? Math.abs(pointYstart - pointYend) : Math.abs(pointYstart - widthYValue);
                    }
                }
                height = yHeight;
                x = this.chart.model.m_AreaBounds.X;
                width = this.chart.model.m_AreaBounds.Width;
                y = pointYstart + ((axis.isInversed) ? 0 : -yHeight);
                var textHeight = ej.EjSvgRender.utils._measureText(stripLine.text, width, stripLine.font).height;

                if (stripLine.textAlignment == 'middletop') {
                    textX = x + width / 2;
                    textY = y + textHeight;
                } else if (stripLine.textAlignment == 'middlecenter') {
                    textX = x + width / 2;
                    textY = (Math.abs(y + (height / 2)) + (textHeight / 3));
                } else if (stripLine.textAlignment == 'middlebottom') {
                    textX = x + width / 2;
                    textY = (y + height) - textHeight;

                }


            }
            if (isVisible) {
                if (!ej.util.isNullOrUndefined(stripLine.imageUrl)) {
                    var imgOptions = {
                        'height': height,
                        'width': width,
                        'href': stripLine.imageUrl,
                        'x': x,
                        'y': y,
                        'id': striplineid.id + '_backImage',
                        'visibility': 'visible',
                        'preserveAspectRatio': 'none'
                    };

                    this.svgRenderer.drawImage(imgOptions, striplineid);
                } else {
                    var options = {
                        'id': striplineid.id + '_striplineRect',
                        'x': x,
                        'y': y,
                        'width': width,
                        'height': height,
                        'fill': stripLine.color,
                        'opacity': stripLine.opacity,
                        'stroke-width': stripLine.borderWidth,
                        'stroke':(stripLine.borderWidth==0)?"transparent": stripLine.borderColor
                    };
                    var textOptions = {
                        'id': striplineid.id + '_striplineRectText',
                        'x': textX,
                        'y': textY,
                        'fill': stripLine.font.color,
                        'transform': 'rotate(' + angle + ',' + (textX) + ',' + textY + ')',
                        'labelRotation': angle,
                        'font-size': stripLine.font.size,
                        'font-family': stripLine.font.fontFamily,
                        'font-style': stripLine.font.fontStyle,
                        'font-weight': stripLine.font.fontWeight,
						'opacity': stripLine.font.opacity,
                        'text-anchor': 'middle'
                    };

                    var bounds = { X: x, Y: y, Width: width, Height: height };
                    var isStripline = stripLine.zIndex == "over" ? true : false;
                    ej.EjSvgRender.utils.AddRegion(this.chart, bounds, isStripline)

                    if (this.chart.model.enableCanvasRendering) {
                        var ctx = this.chart.svgObject.getContext("2d");
                        var clipBounds = axis.orientation.toLowerCase() == 'horizontal' ? { X: axis.x, Y: y, Width: axis.width, Height: this.chart.model.m_AreaBounds.Height }
                        : { X: x, Y: axis.y, Width: this.chart.model.m_AreaBounds.Width, Height: axis.height };
                        ctx.save();
                        ctx.beginPath();
                        ctx.rect(clipBounds.X, clipBounds.Y, clipBounds.Width, clipBounds.Height);
                        ctx.clip();
                        this.svgRenderer.drawRect(options, striplineid);
                        this.svgRenderer.drawText(textOptions, stripLine.text, striplineid);
                        ctx.closePath();
                        ctx.restore();
                    }
                    else {
                        this.svgRenderer.drawRect(options, striplineid);
                        this.svgRenderer.drawText(textOptions, stripLine.text, striplineid);
                    }
                }
            }

        }
    }
};
})(jQuery);
ej.EjLegendRender = function (chartobj) {

    this.chartObj = chartobj;
};


(function ($) {
ej.EjLegendRender.prototype =
{
		sumofArray: function(array) {
            var sum = 0,
            length = array.length;
            for (var i = 0; i < length; i++)
                sum += array[i];
            return sum;
		},

        legendRectSpace: function(svgBounds,divBounds) {
            var width = svgBounds.Width - 10;
            var height = svgBounds.Height - 10;
            var isVisible = false;
            if (Math.ceil(divBounds.Width) >= width && Math.ceil(divBounds.Height) >= height)
                isVisible = true;
            return isVisible;
        },

		drawLegendTitle: function () {
            //draw legend title

            var chart = this.chartObj,
                chartModel = chart.model,
                legend = chartModel.legend,
                legendBounds = chartModel.LegendBounds,
                elementSpacing = chartModel.elementSpacing,
                legendTitle = legend.title,
                measureText = ej.EjSvgRender.utils._measureText,
                font = legendTitle.font,
                legendBoundsWidth = legendBounds.Width,
                titleSize = measureText(legendTitle.text, null, font),
                titleWidth = titleSize.width,
                titleHeight = titleSize.height,
                textAlignment = legendTitle.textAlignment.toLowerCase(),
                startX = 0,
                startY = elementSpacing + (titleHeight / 2);

            switch (textAlignment) {
                case 'far':
                    startX = legendBoundsWidth - titleWidth - startX;
                    break;
                case 'center':
                    startX = legendBoundsWidth / 2 - (titleWidth) / 2;
                    break;
            }

            if (startX < 0) {
                startX = 0;
                legendBoundsWidth = titleWidth;
            }
            if (legendBoundsWidth < titleWidth + startX)
                legendBoundsWidth = titleWidth + startX;

            var options = {
                'id': chart.svgObject.id + '_LegendTitleText',
                'x': startX,
                'y': startY,
                'fill': font.color,
                'font-size': font.size,
                'font-style': font.fontStyle,
                'font-family': font.fontFamily,
                'font-weight': font.fontWeight,
                'text-anchor': 'start',
                'lgndCtx': true
            };

            chart.svgRenderer.drawText(options, legendTitle.text, this.chartObj.gLegendEle);

        },
    
        drawLegendItem: function (legendItem, x, y) {
            var chart = this.chartObj,
                chartModel = chart.model,
                legend = chartModel.legend,
                legendBounds = chartModel.LegendBounds,
                elementSpacing = chartModel.elementSpacing,
                legendBorder = legend.border.width,
                measureText = ej.EjSvgRender.utils._measureText,
                svgRender = ej.EjSvgRender,
                chartSvgRender = chart.svgRenderer,
                legendFont = legend.font,
                symbolShape = chartModel.symbolShape,
                isCanvas = chartModel.enableCanvasRendering,
                shapeSize = legend.itemStyle,
                textHeight = measureText(legendItem.Text, legendBounds.Width, legendFont).height,
                location = {
                    startX: x + shapeSize.width / 2,
                    startY: (y + (textHeight > shapeSize.height ? textHeight : shapeSize.height) / 2)
                },
                pointIndex = legendItem.PointIndex,
                seriesIndex = legendItem.SeriesIndex,
                index = pointIndex ? pointIndex : seriesIndex,
                series = chartModel.series[seriesIndex],
                svgObject = chart.svgObject,
                legendStyle = legendItem.LegendStyle,
                style = {
                    ShapeSize: shapeSize,
                    ElementSpace: elementSpacing,
                    Style: legendStyle,
                    Gradient: legendItem.Gradient,
                    ID: chart.svgObject.id + '_LegendItemShape' + index,
                    SeriesIndex: seriesIndex,
                    context: true
                }, commonEventArgs,
                matched = jQuery.uaMatch(navigator.userAgent),
                isIE11 = !!navigator.userAgent.match(/Trident\/7\./),
                color, heightText,itemInfo, 
                seriesType = series.type.toLowerCase(),
                symbolName, symbol;

            for (name in symbolShape) {
                if (legendItem.Shape.toLowerCase() == name.toLowerCase()) {
                    if (name.toLowerCase() == "seriestype")
                        symbolName = "SeriesType";
                    else
                        symbolName = name;
                    break;
                }
            }

            if (symbolName == "SeriesType") {
                if (seriesType == "radar" || seriesType == "polar") {
                    symbol = series.drawType;
                    symbolName = symbol.capitalizeFirstString();
                }
                if (seriesType == "scatter") {
                    symbol = series.marker.shape;
                    symbolName = symbol.capitalizeFirstString();
                }
                if (legendItem.drawType)// // TrendLine drawType assigned to legend shape
                    symbolName = "SeriesType";
            }

            if (symbolName == "None") {
                if (seriesType == "pie" || seriesType == "doughnut")
                    symbolName = "Circle";
                else
                    symbolName = "Rectangle";
            }

            index = (ej.util.isNullOrUndefined(pointIndex)) ? seriesIndex : pointIndex;
            chart.gLegendItemEle = chartSvgRender.createGroup({ 'id': svgObject.id + '_Legend' + index, 'cursor': 'pointer' });
            commonEventArgs = $.extend({}, svgRender.commonChartEventArgs);
            commonEventArgs.data = {
                legendItem: legendItem,
                location: location,
                legend: true,
                model: chartModel,
                svgRenderer: chartSvgRender,
                svgObject: svgObject,
                symbolShape: symbolName,
                style: style,
                gLegendItemEle: chart.gLegendItemEle
            };

            chart._trigger("legendItemRendering", commonEventArgs);

            if (!commonEventArgs.cancel) {
                startLocation = svgRender.chartSymbol["_draw" + commonEventArgs.data.symbolShape](commonEventArgs.data.location, commonEventArgs.data.style, commonEventArgs.data, chart.gLegendItemEle);
                if (legendStyle.Color && legendStyle.Color.toString().toLowerCase() !== 'gray' && isCanvas) // for canvas gradient color
                    color = legendFont.color;
                else if (legendStyle.Color && legendStyle.Color.toLowerCase() !== 'gray') // for svg
                    color = legendFont.color;
                else
                    color = 'gray';

                itemInfo = commonEventArgs.data.legendItem;
                heightText = svgRender.utils._measureText(itemInfo.Text, legendBounds.Width, legendFont).height;
                if (legend.shape == "seriestype" || legend.shape == "seriesType")
                    x += elementSpacing / 2;
                var options = {
                    'id': this.chartObj.svgObject.id + '_LegendItemText' + index,
                    'x': shapeSize.width + x + elementSpacing / 2,
                    'y': commonEventArgs.data.location.startY,
                    'fill': color,
                    'font-size': legendFont.size,
                    'font-style': legendFont.fontStyle,
                    'font-family': legendFont.fontFamily,
                    'font-weight': legendFont.fontWeight,
                    'text-anchor': 'start',
                    'lgndCtx': true
                };

                options.y = (commonEventArgs.data.location.startY + (heightText / 4)); 
                chartSvgRender.drawText(options, commonEventArgs.data.legendItem.Text, chart.gLegendItemEle);
                chartSvgRender.append(chart.gLegendItemEle, chart.gLegendEle);
                legendbound = {
                    X: legendBounds.X + legendBorder,
                    Y: legendBounds.Y + legendBorder
                };
                x = (startLocation) ? startLocation : x;
                var itembound = { 
                    X: (x), Y: (y), Width: legendItem.Bounds.Width, Height: legendItem.Bounds.Height
                };
                var bounds = { LegendBound: legendbound, ItemBound: itembound };
                var legendRegion = {
                    LegendItem: legendItem,
                    Location: location, SymbolShape: symbolName, Style: style, Bounds: bounds
                };
                chartModel.legendRegion.push(legendRegion);
            }
        },

        drawLegend: function () {
            var chart = this.chartObj,
                chartModel = chart.model,
                legend = chartModel.legend,
                legendViewerBounds = chartModel.LegendViewerBounds,
                legendBounds = chartModel.LegendBounds,
                actualBounds = chartModel.LegendActualBounds,
                AreaType = chartModel.AreaType,
                legendPosition = legend.position.toLowerCase(),
                alignment = legend.alignment.toLowerCase(),
                elementSpacing = chartModel.elementSpacing,
                svgObjectWidth = chartModel.svgWidth,
                svgObjectHeight = chartModel.svgHeight,
                title = chartModel.title,
                subTitle = title.subTitle,
                legendTitle = legend.title,
                legendBorder = legend.border.width,
                measureText = ej.EjSvgRender.utils._measureText,
                margin = chartModel.margin,
                borderSize = chartModel.border.width,
                legendCollection = chartModel.legendCollection,
                collectionLength = legendCollection.length,
                svgWidth = svgObjectWidth - ((borderSize * 2)),
                svgHeight = svgObjectHeight - ((borderSize * 2)),
                itemPadding = legend.itemPadding > 0 ? legend.itemPadding : 0,
                padding = 20,
                hPadding = 10,
                vPadding = 10,
                modelsubTitleHeight = subTitle.text == "" ? 0 : measureText(subTitle.text, svgWidth - margin.left - margin.right, subTitle.font).height + elementSpacing,
                titleLocation = chartModel._titleLocation ? chartModel._titleLocation.Y : 0 + modelsubTitleHeight,
                legendTitleBounds = measureText(legendTitle.text, null, legendTitle.font),
                legendTitleHeight = legendTitleBounds.height,
                legendTitleWidth = legendTitleBounds.width,
                rowDefinitions = chartModel._rowDefinitions,
                maxWidth = 0, startX, startY,
                columnDefinitions = chartModel._columnDefinitions,
                vSizeFar = this.sumofArray(rowDefinitions.farSizes),
                vSizeNear = this.sumofArray(rowDefinitions.nearSizes),
                hSizeFar = this.sumofArray(columnDefinitions.farSizes),
                hSizeNear = this.sumofArray(columnDefinitions.nearSizes),
                isScrolling = false,
                areaBounds = chartModel.m_AreaBounds,
                isScroll = legend.enableScrollbar,
                legendContainer = $(chart.legendContainer),
                legendSvgContainer = $(chart.legendSvgContainer);
            if (legendPosition == 'right' || legendPosition == 'left') {
                hPadding = 10;
            } else {
                vPadding = isScroll ? svgHeight > 200 ? 10 : svgHeight > 100 ? 5 : 0 : 10;
            }
            legendViewerBounds.Height += legendTitleHeight,
            legendBounds.Height += legendTitleHeight;
            if (legend.visible) {
                // draw legend.
                if (legendPosition != 'custom') {
                    if (AreaType == "cartesianaxes") {
                        switch (legendPosition) {
                            case "bottom":
                                legendBounds.Y = areaBounds.Y + areaBounds.Height + hSizeNear - (elementSpacing / 2);
                                break;
                            case "top":
                                legendBounds.Y = titleLocation == 0 ? borderSize + elementSpacing : titleLocation + elementSpacing;
                                break;
                            case "right":
                                legendBounds.X = areaBounds.X + areaBounds.Width + vSizeFar + elementSpacing * 2;
                                break;
                            case "left":
                                legendBounds.X = areaBounds.X - vSizeNear - actualBounds.Width - (elementSpacing / 2) - (legendBorder * 2);
                                break;
                        }
                    } else {
                        switch (legendPosition) {
                            case "bottom":
                                legendBounds.Y = svgHeight - (actualBounds.Height + (legendBorder)) - (elementSpacing * 2);
                                break;
                            case "top":
                                legendBounds.Y = titleLocation == 0 ? borderSize + elementSpacing : titleLocation + elementSpacing;
                                break;
                            case "right":
                                legendBounds.X = svgWidth - actualBounds.Width - (elementSpacing * 2);
                                break;
                            case "left":
                                legendBounds.X = borderSize + (elementSpacing * 2);
                                break;
                        }
                    }
                    if (legendPosition == 'left' || legendPosition == 'right') {
                        if (isScroll) {
                            switch (alignment) {
                                case "center":
                                    legendBounds.Y = areaBounds.Height / 2 - (actualBounds.Height / 2) + areaBounds.Y - legendBorder;
                                    break;
                                case "near":
                                    legendBounds.Y = areaBounds.Y;
                                    break;
                                case "far":
                                    legendBounds.Y = areaBounds.Y + areaBounds.Height - actualBounds.Height - (legendBorder * 2);
                                    break;
                            }
                        } else {
                            switch (alignment) {
                                case "center":
                                    legendBounds.Y = (svgHeight / 2) - ((actualBounds.Height + legendBorder * 2) / 2) + (elementSpacing / 2);
                                    break;
                                case "near":
                                    legendBounds.Y = borderSize + (elementSpacing * 2);
                                    break;
                                case "far":
                                    legendBounds.Y = svgHeight - (actualBounds.Height + (legendBorder)) - (elementSpacing * 2);
                                    break;
                            }
                        }

                    } else {
                        switch (alignment) {
                            case "center":
                                legendBounds.X = (svgWidth / 2) - ((actualBounds.Width + legendBorder * 2) / 2) + (elementSpacing / 2);
                                break;
                            case "near":
                                legendBounds.X = borderSize + (elementSpacing * 2);
                                break;
                            case "far":
                                legendBounds.X = svgWidth - (actualBounds.Width + (legendBorder)) - (elementSpacing * 2);
                                break;
                        }
                    }
                } else {
                    legendBounds.Y = (legend.location.y < svgHeight) ? legend.location.y : 0;
                    legendBounds.X = (legend.location.x < svgWidth) ? legend.location.x : 0;
                }
                if (isScroll) {
                    if (legendPosition == 'right' || legendPosition == 'left') {
                        if (legendBounds.Y < areaBounds.Y || actualBounds.Height + (legendBorder * 2) >= areaBounds.Height) {
                            var legendHeight = (legendBorder * 2);
                            for (var k = 0; k < collectionLength; k++) {
                                legendHeight += legendCollection[k].Bounds.Height + itemPadding;
                                if (legendHeight > areaBounds.Height) {
                                    actualBounds.Height = areaBounds.Height - (legendBorder * 2);
                                    legendBounds.Y = areaBounds.Y;
                                    break;
                                }
                            }
                        }
                    }
                    else {
                        if ((legendBorder * 2 + actualBounds.Width) >= svgWidth) {
                            var legendWidth = (legendBorder * 2);
                            for (var k = 0; k < collectionLength; k++) {
                                legendWidth += legendCollection[k].Bounds.Width + itemPadding;
                                if (legendWidth > svgWidth) {
                                    actualBounds.Width = svgWidth - (legendBorder * 2) - actualBounds.vScrollSize - (borderSize * 2);
                                    legendBounds.X = (svgWidth / 2) - ((actualBounds.Width + legendBorder * 2) / 2) + (elementSpacing / 2);
                                    break;
                                }

                            }
                        }
                    }
                }
                    
                //Set height and width to legend Canvas element
                chart.legendSvgContainer.height = legendBounds.Height;
                chart.legendSvgContainer.width = Math.max(legendTitleWidth, legendBounds.Width);
                currentX = startX = hPadding;
                currentY = startY = vPadding;
                if (legendTitle.text != "" && legendTitle.text) {
                    this.drawLegendTitle();
                    currentY = startY = vPadding + legendTitleHeight;
                }
                for (var k = 0; k < collectionLength; k++) {
                    if (legendPosition == 'top' || legendPosition == 'bottom' || legendPosition == 'custom' || legend.rowCount || legend.columnCount) {
                        if ((currentX + legendCollection[k].Bounds.Width) > legendBounds.Width + startX) {
                            currentX = hPadding;
                            currentY += legendCollection[k].Bounds.Height + itemPadding;
                        }
                        this.drawLegendItem(legendCollection[k], currentX, (currentY));
                        currentX += legendCollection[k].Bounds.Width + itemPadding;
                    }
                    else {
                        if (((currentY + legendCollection[k].Bounds.Height) + legendTitleHeight + borderSize > legendBounds.Height + startY)) {
                            currentY = startY;
                            currentX += maxWidth + (itemPadding);
                        }
                        this.drawLegendItem(legendCollection[k], (currentX), (currentY));
                        currentY += legendCollection[k].Bounds.Height + itemPadding;
                        maxWidth = Math.max(maxWidth, legendCollection[k].Bounds.Width);
                    }
                }
				var offsetVal = $(chart.svgObject).offset().left - $('#' + chart._id).offset().left;
				var legnTx = legendBounds.X + (offsetVal <= 0 ? 0 : offsetVal);
                legendContainer.attr('style','');
                legendContainer.show();
                legendContainer.css({ "position": "absolute", "background": legend.background, "left": legnTx, "top": legendBounds.Y, "width": actualBounds.Width, "height": actualBounds.Height, "border-width": legend.border.width, "border-color": legend.border.color, "border-style": "solid" });
                legendSvgContainer.css({ "height": legendBounds.Height, "width": legendBounds.Width });
                legendBounds.Height = legendBounds.Height - (padding - (vPadding * 2));
                var isVisible = this.legendRectSpace(legendBounds, actualBounds);
                if (isScroll) {
                    if (!isVisible && legendBounds.Width > Math.ceil(actualBounds.Width) && legendBounds.Height > Math.ceil(actualBounds.Height)) {
                        legendContainer.css({ "overflow": "scroll" });
                        isScrolling = true;
                    } else {
                        if (!isVisible && legendBounds.Height > Math.ceil(actualBounds.Height) && actualBounds.vScrollSize >= 0) {
                            legendContainer[0].style.overflowY = "scroll";
                            legendContainer[0].style.overflowX = "hidden";
							if (actualBounds.vScrollSize == 0) {
                                vScrollSize = legendContainer[0].offsetWidth - (legendBorder * 2) - legendContainer[0].clientWidth;
                                legendContainer.css({ "width": actualBounds.Width + vScrollSize });
                            }
                            isScrolling = true;
                        }
                        if (!isVisible && legendBounds.Width > Math.ceil(actualBounds.Width) && actualBounds.hScrollSize >= 0) {
                            legendContainer[0].style.overflowX = "scroll";
                            legendContainer[0].style.overflowY = "hidden";
							if (actualBounds.hScrollSize == 0) {
                                hScrollSize = legendContainer[0].offsetHeight - (legendBorder * 2) - legendContainer[0].clientHeight;
                                legendContainer.css({ "height": actualBounds.Height + hScrollSize });
                            }
                            isScrolling = true;
                        }
                    }
                }
                
                chart.svgRenderer.append(chart.gLegendEle, chart.legendSvgContainer);
                if (!isScrolling) {
                    legendContainer.css({ "width": actualBounds.Width - actualBounds.vScrollSize, "height": actualBounds.Height - actualBounds.hScrollSize });
                }
                if (chart.scrolltop != undefined)
                    legendContainer.scrollTop(chart.scrolltop);
                if (chart.scrollleft != undefined)
                    legendContainer.scrollLeft(chart.scrollleft);

            }
            else
                legendContainer.hide();
        }
    };
})(jQuery);;

ej.seriesTypes = {};

ej.EjSeriesRender = function () {

};
ej.indicatorTypes = {};

ej.EjIndicatorRender = function () {

};
ej.trendlineTypes = {};

ej.EjTrendLineRenderer = function () {

};
(function ($) {
    ej.EjTrendLineRenderer.prototype = {
        draw: function (trendline, series, sender) {
            this.chartObj = sender;
            var trendIndex = $.inArray(trendline, series.trendlines);
            trendline.xAxis = series.xAxis;
            trendline.yAxis = series.yAxis;
            if (trendline.type.toLowerCase() == "linear")
                this.calculateLineDirection(trendline, series, trendIndex);
            else
                this.calculateSplineDirection(trendline, series, trendIndex);

            return false;
        },        
        calculateSplineDirection: function (trendline, series, trendIndex) {
            //var series = trendline;
            var spDirection = "";
            var splinesb = ej.EjSvgRender.utils._getStringBuilder();
            var startingPoint = true;
            var yIndex = 0;
            var visiblePoints = trendline.points;
            var ySpline = ej.EjSeriesRender.prototype.naturalSpline(visiblePoints);
            var firstPoint = null;
            var secondPoint = null;
            var firstIndex = -1;
            var canvasX = 0;
            var canvasY = 0;
            var point, pointIndex;
            var length = visiblePoints.length;
            for (var i = 0; i < length; i++) {
                pointIndex = i;
                secondPoint = visiblePoints[i];
                    if (firstPoint != null) {
                        var controlPoint1 = null;
                        var controlPoint2 = null;
                        var data = ej.EjSeriesRender.prototype.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                        controlPoint1 = data["controlPoint1"];
                        controlPoint2 = data["controlPoint2"];
                        var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, trendline);
                        var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, trendline);
                        var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, trendline);
                        var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, trendline);
                        var chartObj = this.chartObj;
                        if (startingPoint) {
                            splinesb.append("M" + " " + (pt1.X) + " " + (pt1.Y) + " " + "C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                            startingPoint = false;
                        }
                        else
                            splinesb.append("C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                    }
                    firstPoint = secondPoint;
                    firstIndex = pointIndex;
            }
            spDirection = splinesb.toString();
            this._drawLinePath(trendline, series, spDirection, trendIndex);

            return false;
        },
        calculateLineDirection: function (trendline, series, trendIndex) {
            var canvasX = 0;
            var canvasY = 0;
            var point;           
            var lDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();

            var firstPoint = false;
            var secondPoint;
            var length = trendline.points.length;
            for (var i = 0; i < length; i++) {
                secondPoint = trendline.points[i];
                    point = ej.EjSvgRender.utils._getPoint(secondPoint, trendline);
                    if (firstPoint)
                        sb.append("L" + " " + (point.X) + " " + ((point.Y)) + " ");
                    else {
                        sb.append("M" + " " + (point.X) + " " + ((point.Y)) + " ");
                        sb.append("L" + " " + (point.X) + " " + ((point.Y)) + " ");
                        firstPoint = true;
                    }
            }
            lDirection = sb.toString();
            this._drawLinePath(trendline, series, lDirection, trendIndex);
            return false;
        },
        _drawLinePath: function (trendline, series, lDirection, trendIndex) {
            var translate = [];
            translate[0] = trendline.xAxis.x;
            translate[1] = trendline.yAxis.y;

            if (lDirection != "") {
                var options = {
                    'id': this.chartObj.svgObject.id + "_" + trendline.seriesIndex + "_" + trendIndex + "_" + trendline.name.replace(/[^a-zA-Z ]/g, ""),
                    'fill': 'none',
                    'stroke-dasharray': trendline.dashArray,
                    'stroke-width': trendline.width,
                    'stroke': trendline.fill,
                    'opacity': trendline.opacity,
                    'd': lDirection
                };
                trendline.id = options.id;
                this.chartObj.svgRenderer.drawPath(options, series.gTrendGroupEle, translate);
            }
            this.chartObj.svgRenderer.append(series.gTrendGroupEle, this.chartObj.gTrendlinesGroupEle[trendline.seriesIndex]);

            return false;
        },

        calculateTrendLineSegment: function (series, trendline, axis, sender) {
            var chartmodel = sender.model;
            var axis;
            var trendlineType = trendline.type.toLowerCase();
            trendline.backwardForecast = trendline.backwardForecast < 0 ? 0 : trendline.backwardForecast;
            trendline.forwardForecast = trendline.forwardForecast < 0 ? 0 : trendline.forwardForecast;
            
            switch (trendlineType) {
                case "linear":
                    this.updateTrendSource(series, trendline);
                    this.calculateLinearTrendline(series, trendline, axis);
                    break;
                case "exponential":
                    this.updateExponentialTrendSource(series, trendline);
                    this.calculateExponentialTrendline(series, trendline, axis);
                    break;
                case "logarithmic":
                    this.updateLogarithmicTrendSource(series, trendline);
                    this.calculateLogarithmicTrendline(series, trendline, axis);
                    break;
                case "power":
                    this.updatePowerTrendSource(series, trendline);
                    this.calculatePowerTrendline(series, trendline, axis);
                    break;
                case "polynomial":
				    var length = series.points.length;
				    trendline.polynomialOrder = length <= trendline.polynomialOrder ? length : trendline.polynomialOrder;
                    trendline.polynomialOrder = trendline.polynomialOrder < 2 ? 2 : trendline.polynomialOrder;
                    trendline.polynomialOrder = trendline.polynomialOrder > 6 ? 6 : trendline.polynomialOrder;
                    if (length > 1) {
                        this.updateTrendSource(series, trendline);
                        this.calculatePolynomialTrendline(series, trendline, axis);
                    }
                    break;                
            }
            if (series.points.length > 1)
                this.calculateTrendLineRange(trendline, axis);
            return false;
        },
        updateTrendSource: function (series, trendline) {
            var points = series.points;
            var len = points.length;

            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                if (!points[i].isEmpty) {                    
                    trendline.xPoints.push(points[i].xValue);
                    trendline.xValues.push(i + 1);
                    trendline.yValues.push(points[i].y);
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculateLinearTrendline: function (series, trendline, axis) {            
            
            var count = trendline.xValues.length;
            var x1, x2;
            trendline.points = [];

            if (count > 1) {
                if (axis._valueType == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    x2 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                } else {
                     x1 = (trendline.xPoints[0] - trendline.backwardForecast);
                     x2 = (trendline.xPoints[count - 1] + trendline.forwardForecast );
                }

                //trendline linear segment  y = ax + b;
                var y1 = (trendline.slope * (1 - ((count - 1) * (trendline.backwardForecast) / (series.maxX - series.minX)))) + trendline.intercept;
                var y2 = (trendline.slope * (count + ((count - 1) * (trendline.forwardForecast)/(series.maxX - series.minX)))) + trendline.intercept;
                trendline.points.push({ x: x1, xValue: x1, y: y1, YValues: [] });
                trendline.points.push({ x: x2, xValue: x2, y: y2, YValues: [] });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
            }
            return false;
        },
       
        //// Calculate Trendline range
        calculateTrendLineRange: function (trendline, axis) {

            var length = trendline.points.length;
            var y;
            trendline.minY = trendline.points[0].y;
            trendline.maxY = trendline.points[length-1].y;
            if (!axis.setRange) {
                if (trendline.backwardForecast > 0 && axis._valueType != "category" && length > 1) {
                    axis.range.min = axis.range.min > trendline.points[0].x ? trendline.points[0].x : axis.range.min;
                    axis.visibleRange.min = axis.range.min;
                }
                if (trendline.forwardForecast > 0 && length > 1) {
                    axis.range.max = axis.range.max < trendline.points[length - 1].x ? trendline.points[length - 1].x : axis.range.max;
                    axis.visibleRange.max = axis.range.max;
                }
            }
            for (var i = 0; i < length; i++) {
                y = trendline.points[i].y;
                trendline.minY = Math.min(trendline.minY, y);
                trendline.maxY = Math.max(trendline.maxY, y);
            }
            return false;
        },

        /// Calculate Sum of x and y values
        calculateSumXAndYValue: function (trendline) {
            var points = trendline.points;
            var N = trendline.xValues.length;
            var Slope;
            var Intercept;
            var SX = 0;
            var SY = 0;
            var SXX = 0;
            var SXY = 0;
            var SYY = 0;
            var X = trendline.xValues;
            var Y = trendline.yValues;

            for (var i = 0; i < N; i++) {
                SX = SX + X[i];
                SY = SY + Y[i];
                SXY = SXY + X[i] * Y[i];
                SXX = SXX + X[i] * X[i];
                SYY = SYY + Y[i] * Y[i];
            }

            Slope = ((N * SXY) - (SX * SY)) / ((N * SXX) - (SX * SX));
            if (trendline.type.toLowerCase() == "exponential" || trendline.type.toLowerCase() == "power")
                Intercept = Math.exp((SY - (Slope * SX)) / N);
            else
                Intercept = (SY - (Slope * SX)) / N;

            trendline.slope = Slope, trendline.intercept = Intercept;
            return false;
        },

        ////Calculate forward backward Forecast points for dateTime axis
        _increaseDateTimeInterval: function (date, interval, intervalType) {
        var result = new Date(date);
        if (intervalType.toLowerCase() == "days") {
            result.setDate(date.getDate() + interval);
        } else if (intervalType.toLowerCase() == "hours") {
            result.setHours(date.getHours() + interval);
        } else if (intervalType.toLowerCase() == "milliseconds") {
            result.setMilliseconds(date.getMilliseconds() + interval);
        } else if (intervalType.toLowerCase() == "seconds") {
            result.setSeconds(date.getSeconds() + interval);
        } else if (intervalType.toLowerCase() == "minutes") {
            result.setMinutes(date.getMinutes() + interval);
        } else if (intervalType.toLowerCase() == "months") {
            result.setMonth(date.getMonth() + interval);
        } else if (intervalType.toLowerCase() == "years") {
            result.setYear(date.getFullYear() + interval);
        }
        return result;
        },

        ////Animate the trendline Series
        doAnimation: function (series, chartObj, seriesIndex, trendIndex) {         
            var clipRect = $(chartObj.svgObject).find("#" + chartObj.svgObject.id + '_TrendGroup' + '_' + seriesIndex + '_' + trendIndex + "_ClipRect").children();
            $(clipRect).animate(
                { width: parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.svgObject).find("#" + series.gTrendGroupEle.id + "_ClipRect").children(), "width")) },
                {
                    duration: 2000,

                    step: function (now) {
                        chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                    }
                });
            return false;
        }
    },

    ej.ejLinear = ejExtendClass(ej.EjTrendLineRenderer);
    ej.trendlineTypes.linear = ej.ejLinear;

    ej.Exponential = ejExtendClass(ej.EjTrendLineRenderer, {

        updateExponentialTrendSource: function (series, trendline) {
            var points = series.points;
            var len = points.length;

            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                if (!points[i].isEmpty) {
                    trendline.xPoints.push(points[i].xValue);
                    trendline.xValues.push(i + 1);
                    trendline.yValues.push(Math.log(points[i].y));
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculateExponentialTrendline: function (series, trendline, axis) {
            var count = trendline.xValues.length;
            var x1, x2, x3;
            var X1, X2, X3;
            
            var midPoint = Math.round((count / 2));
            trendline.points = [];
            
            if (count > 1) {
                if (axis._valueType == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    X1 = x1.getTime();
                    x2 = (trendline.xPoints[midPoint - 1]);
                    X2 = x2.getTime();
                    x3 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                    X3 = x3.getTime();
                } else {
                    x1 = X1 = (trendline.xPoints[0] - trendline.backwardForecast);
                    x2 = X2 = (trendline.xPoints[midPoint-1]);
                    x3 = X3 =(trendline.xPoints[count - 1] + trendline.forwardForecast);
                }

                //trendline exponential segment  (Intercept * Math.Exp(Slope * xValue));
                var y1 = (trendline.intercept * Math.exp((trendline.slope) * (1 - ((count - 1) * (trendline.backwardForecast) / (series.maxX - series.minX)))));
                var y2 = (trendline.intercept * Math.exp((trendline.slope) * (midPoint)));
                var y3 = (trendline.intercept * Math.exp((trendline.slope) * (count + ((count - 1) * (trendline.forwardForecast) / (series.maxX - series.minX)))));
                trendline.points.push({ x: x1, xValue: X1, y: y1, YValues: [] });
                trendline.points.push({ x: x2, xValue: X2, y: y2, YValues: [] });
                trendline.points.push({ x: x3, xValue: X3, y: y3, YValues: [] });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
                trendline.points[2].YValues[0] = trendline.points[2].y;
            }
            return false;
        }
    });
    ej.trendlineTypes.exponential = ej.Exponential;

    ej.Logarithmic = ejExtendClass(ej.EjTrendLineRenderer, {

        updateLogarithmicTrendSource: function (series, trendline) {
            var points = series.points;
            var len = points.length;

            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                if (!points[i].isEmpty) {
                    trendline.xPoints.push(points[i].xValue);
                    trendline.xValues.push(Math.log(i + 1));
                    trendline.yValues.push(points[i].y);
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculateLogarithmicTrendline: function (series, trendline, axis) {
            var count = trendline.xValues.length;
            var x1, x2, x3;
            var X1, X2, X3;
            var midPoint = Math.round((count / 2));
            trendline.points = [];

            if (count > 1) {
                if (axis._valueType == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    X1 = x1.getTime();
                    x2 = (trendline.xPoints[midPoint - 1]);
                    X2 = x2.getTime();
                    x3 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                    X3 = x3.getTime();
                } else {
                    x1 = X1 = (trendline.xPoints[0] - trendline.backwardForecast);
                    x2 = X2 = (trendline.xPoints[midPoint-1]);
                    x3 = X3 = (trendline.xPoints[count - 1] + trendline.forwardForecast);
                }

                //trendline logarithmic segment  Intercept + Slope * Math.Log(xValue);
                var y1 = (trendline.intercept + (trendline.slope * Math.log(1))) - (1 - ((count - 1) * (trendline.backwardForecast) / (series.maxX - series.minX)));
                var y2 = trendline.intercept + (trendline.slope * Math.log(midPoint));
                var y3 = (trendline.intercept + (trendline.slope * Math.log((count + ((count - 1) * (trendline.forwardForecast) / (series.maxX - series.minX))))));
                trendline.points.push({ x: x1, xValue: X1, y: y1, YValues: [] });
                trendline.points.push({ x: x2, xValue: X2, y: y2, YValues: [] });
                trendline.points.push({ x: x3, xValue: X3, y: y3, YValues: [] });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
                trendline.points[2].YValues[0] = trendline.points[2].y;
            }
            return false;
        }
    });
    ej.trendlineTypes.logarithmic = ej.Logarithmic;

    ej.Power = ejExtendClass(ej.EjTrendLineRenderer, {

        updatePowerTrendSource: function (series, trendline) {
            var points = series.points;
            var len = points.length;

            trendline.xPoints = [];
            trendline.xValues = [];
            trendline.yValues = [];
            for (var i = 0; i < len; i++) {
                if (!points[i].isEmpty) {
                    trendline.xPoints.push(points[i].xValue);
                    trendline.xValues.push(Math.log(i + 1));
                    trendline.yValues.push(Math.log(points[i].y));
                }
            }
            this.calculateSumXAndYValue(trendline);
            return false;
        },
        calculatePowerTrendline: function (series, trendline, axis) {
            var count = trendline.xValues.length;
            var x1, x2, x3;
            var X1, X2, X3;
            var midPoint = Math.round((count / 2));
            trendline.points = [];

            if (count > 1) {
                if (axis._valueType == "datetime") {
                    x1 = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                    X1 = x1.getTime();
                    x2 = (trendline.xPoints[midPoint - 1]);
                    X2 = x2.getTime();
                    x3 = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                    X3 = x3.getTime();
                } else {
                    x1 = X1 = (trendline.xPoints[0] - trendline.backwardForecast);
                    x2 = X2 = (trendline.xPoints[midPoint - 1]);
                    x3 = X3 = (trendline.xPoints[count - 1] + trendline.forwardForecast);
                }

                //trendline power segment  (Intercept * Math.Pow(xValue, Slope))
                var y1 = (trendline.intercept * Math.pow(1, trendline.slope)) - (1 - ((count - 1) * (trendline.backwardForecast) / (series.maxX - series.minX)));
                var y2 = (trendline.intercept * Math.pow(midPoint, trendline.slope));
                var y3 = (trendline.intercept * Math.pow((count + ((count - 1) * (trendline.forwardForecast) / (series.maxX - series.minX))), trendline.slope));
                trendline.points.push({ x: x1, xValue: X1, y: y1, YValues: [] });
                trendline.points.push({ x: x2, xValue: X2, y: y2, YValues: [] });
                trendline.points.push({ x: x3, xValue: X3, y: y3, YValues: [] });

                trendline.points[0].YValues[0] = trendline.points[0].y;
                trendline.points[1].YValues[0] = trendline.points[1].y;
                trendline.points[2].YValues[0] = trendline.points[2].y;
            }
            return false;
        }
    });
    ej.trendlineTypes.power = ej.Power;

    ej.Polynomial = ejExtendClass(ej.EjTrendLineRenderer, {

        //// Calculate Polynomial Trendline with order
        calculatePolynomialTrendline: function (series, trendline, axis) {
            var power = trendline.polynomialOrder;

            // Calculate sum of y datapoints 1 X power matrix
            trendline.polynomialSlopes = new Array(power + 1);
            var xLength = trendline.xValues.length;
            for (var index1 = 0; index1 < xLength; index1++)
            {
                var num2 = trendline.xValues[index1];
                var yval = trendline.yValues[index1];
                
                for (var index2 = 0; index2 <= power; ++index2) {
                    if (!trendline.polynomialSlopes[index2]) trendline.polynomialSlopes[index2] = 0;
                    trendline.polynomialSlopes[index2] += Math.pow(num2, index2) * yval;
                }
            }

            // Calculate sum matrix of x datapoints
            var numArray = new Array(1 + 2 * power);
            var matrixOfA = new Array(power + 1);
            for (var i = 0; i < (power + 1); i++)
                matrixOfA[i] = new Array(3);
            var num1 = 0;
            for (var index1 = 0; index1 < xLength; ++index1) {
                var num2 = 1.0;
                var d = trendline.xValues[index1];
                for (var index2 = 0; index2 < numArray.length; ++index2) {
                    if (!numArray[index2]) numArray[index2] = 0;
                    numArray[index2] += num2;
                    num2 *= d;
                    ++num1;
                }   
            }

            for (var index1 = 0; index1 <= power; ++index1) {
                for (var index2 = 0; index2 <= power; ++index2)
                    matrixOfA[index1][index2] = numArray[index1 + index2];
            }

            //Calculation Gauss jordan eliminiation value of a and b matrix
            if (!this.gaussJordanEliminiation(matrixOfA, trendline.polynomialSlopes))
                trendline.polynomialSlopes = null;

            //Create segments methods
            this.createPolynomialSegments(series, axis, trendline);
            return false;
        },
        createPolynomialSegments: function (series, axis, trendline) {
            var polynomialSlopes = trendline.polynomialSlopes;
            if (polynomialSlopes != null) {
                var count = trendline.xValues.length;
                var midPoint = Math.round((count / 2));
                var x1 = 1;
                var X;
                var length = polynomialSlopes.length;
                trendline.points = [];
                for (var i = 1; i <= length; i++) {                   
                    var x, y;
                    if (i == 1) {
                        if (axis._valueType == "datetime") {
                            x = this._increaseDateTimeInterval(trendline.xPoints[0], -trendline.backwardForecast, axis._intervalType);
                            X = x.getTime();
                        }
                        else
                            x = X = (trendline.xPoints[0] - trendline.backwardForecast);
                        var y = this.getPolynomialYValue(polynomialSlopes, (1 - ((count - 1) * (trendline.backwardForecast) / (series.maxX - series.minX))));
                    }
                    else if (i == polynomialSlopes.length) {
                        if (axis._valueType == "datetime") {
                            x = this._increaseDateTimeInterval(trendline.xPoints[count - 1], trendline.forwardForecast, axis._intervalType);
                            X = x.getTime();
                        }
                        else
                            x = X = (trendline.xPoints[count - 1] + trendline.forwardForecast);

                        var y = this.getPolynomialYValue(polynomialSlopes, (count + ((count - 1) * (trendline.forwardForecast) / (series.maxX - series.minX))));
                    } else {
                        x1 += (count + trendline.forwardForecast) / polynomialSlopes.length;
                        if (axis._valueType == "category") {
                            x = X = trendline.xPoints[0] + (x1 - 1);

                        }
                        else {
                            x = X = trendline.xPoints[parseInt(x1) - 1];
                            if (axis._valueType == "datetime" && count > x1)
                                X = x.getTime();
                        }                        
                        var y = this.getPolynomialYValue(polynomialSlopes, x1);
                    }
                    if (i == 1 || i == polynomialSlopes.length || axis._valueType == "category" || count > x1) {
                        trendline.points.push({ x: x, xValue: X, y: y, YValues: [] });
                        trendline.points[trendline.points.length - 1].YValues[0] = trendline.points[trendline.points.length - 1].y;
                    }
                }
            }
            return false;
        },

        getPolynomialYValue: function(slopes, x){
            var sum=0;
            for (var i = 0; i < slopes.length; i++) {
                sum += slopes[i] * Math.pow(x, i);
            }
            return sum;
        },        

        gaussJordanEliminiation: function (matrixOfA, polynomialSlopes) {
            var length = matrixOfA.length;
            var numArray1 = new Array(length);
            var numArray2 = new Array(length);
            var numArray3 = new Array(length);
            for (var index = 0; index < length; ++index)
                numArray3[index] = 0;
            for (var index1 = 0; index1 < length; ++index1)
            {
                var num1 = 0.0;
                var index2 = 0;
                var index3 = 0;
                for (var index4 = 0; index4 < length; ++index4)
                {
                    if (numArray3[index4] != 1)
                    {
                        for (var index5 = 0; index5 < length; ++index5)
                        {
                            if (numArray3[index5] == 0 && Math.abs(matrixOfA[index4][index5]) >= num1)
                            {
                                num1 = Math.abs(matrixOfA[index4][index5]);
                                index2 = index4;
                                index3 = index5;
                            }
                        }
                    }
                }
                ++numArray3[index3];
                if (index2 != index3)
                {
                    for (var index4 = 0; index4 < length; ++index4)
                    {
                        var num2 = matrixOfA[index2, index4];
                        matrixOfA[index2][index4] = matrixOfA[index3][index4];
                        matrixOfA[index3][index4] = num2;
                    }
                    var num3 = polynomialSlopes[index2];
                    polynomialSlopes[index2] = polynomialSlopes[index3];
                    polynomialSlopes[index3] = num3;
                }
                numArray2[index1] = index2;
                numArray1[index1] = index3;
                if (matrixOfA[index3][index3] == 0.0)
                    return false;
                var num4 = 1.0 / matrixOfA[index3][index3];
                matrixOfA[index3][index3] = 1.0;
               
                for (var index4 = 0; index4 < length; ++index4)
                    matrixOfA[index3][index4] *= num4;
                
                polynomialSlopes[index3] *= num4;
                
                for (var index4 = 0; index4 < length; ++index4)
                {
                    if (index4 != index3)
                    {
                        var num2 = matrixOfA[index4][index3];
                        matrixOfA[index4][index3] = 0.0;
                        for (var index5 = 0; index5 < length; ++index5)
                            matrixOfA[index4][index5] -= matrixOfA[index3][index5] * num2;
                        polynomialSlopes[index4] -= polynomialSlopes[index3] * num2;
                    }
                }
            }
            for (var index1 = length - 1; index1 >= 0; --index1)
            {
                if (numArray2[index1] != numArray1[index1])
                {
                    for (var index2 = 0; index2 < length; ++index2)
                    {
                        var num = matrixOfA[index2, numArray2[index1]];
                        matrixOfA[index2][numArray2[index1]] = matrixOfA[index2][numArray1[index1]];
                        matrixOfA[index2][numArray1[index1]] = num;
                    }
                }
            }
            return true;
        }
    });
    ej.trendlineTypes.polynomial = ej.Polynomial;

    ej.EjIndicatorRender.prototype = {

        draw: function (indicator, sender) {

            this.chartObj = sender;
            var translate = null;

            if (this.chartObj.model.AreaType == "cartesianaxes") {
                var transX = indicator.xAxis.x;
                var transY = indicator.yAxis.y;
                translate = 'translate(' + transX + ',' + transY + ')';
            }

            var indicatorIndex = $.inArray(indicator, this.chartObj.model.indicators);
            var indicatorOptions = { 'id': this.chartObj.svgObject.id + '_indicatorGroup' + '_' + indicatorIndex, 'transform': translate };
            this.gIndicatorGroupEle = this.chartObj.svgRenderer.createGroup(indicatorOptions);

            
            if (indicator._points.length > 0) {
                for (var i = 0; i < indicator.segment.length; i++) {
                    var segment = indicator.segment[i];
                    var style = { fill: segment.fill, width: segment.width, opacity: indicator.opacity };
                        segment.isIndicator = true;
                        segment.xAxis = indicator.xAxis;
                        segment.yAxis = indicator.yAxis;
                         if (segment.type == "line")
                            this.calculateDirection(segment, style, indicatorIndex);
                        else
                            this.drawColumn(segment, style, indicator);
                    }
                }

        },

        _drawLinePath: function (indicator, style, lDirection, indicatorIndex) {
            
          
          
           
            if (lDirection != "") {
                var options = {
				    'name': 'line',
                    'id': this.chartObj.svgObject.id + "_" + indicatorIndex + "_" + indicator.name,
                    'fill': 'none',
                    'stroke-dasharray': indicator.dashArray,
                    'stroke-width': style.width,
                    'stroke': style.fill,
                    'stroke-linecap': indicator.lineCap,
                    'stroke-linejoin': indicator.lineJoin,
                    'opacity': style.opacity,
                    'd': lDirection
                };

                this.chartObj.svgRenderer.drawPath(options, this.gIndicatorGroupEle);
            }

            this.chartObj.svgRenderer.append(this.gIndicatorGroupEle, this.chartObj.gIndicatorEle);
        },

        calculateDirection: function (currentseries, style, indicatorIndex) {
		    var canvasX = 0;
            var canvasY = 0;
		    var point;
            if (this.chartObj.model.enableCanvasRendering) {
                canvasX = currentseries.xAxis.x;
                canvasY = currentseries.yAxis.y;
            }
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();

            var visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(currentseries);
            var lDirection;
            var sb = ej.EjSvgRender.utils._getStringBuilder();

            // Dashed Line issue in Indicator is fixed.
            var firstPoint = false;
            var secondPoint;
            
            for (var i = 0; i < visiblePoints.length; i++) {
                secondPoint = visiblePoints[i];
                if (secondPoint.visible) {
                     point = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    if (firstPoint) {                                             
                        sb.append("L" + " " + (point.X + canvasX) + " " + ((point.Y + canvasY)) + " ");                    
                        }
                    else {                       
                        sb.append("M" + " " + (point.X + canvasX) + " " + ((point.Y + canvasY)) + " ");
                         firstPoint = true;
                       }         

                }
                else {
                    firstPoint = false;
                }

            }

            lDirection = sb.toString();

            this._drawLinePath(currentseries, style, lDirection, indicatorIndex);
        },

        drawColumn: function (option, style, indicator) {
            var series = option;
            var seriesIndex = $.inArray(indicator, this.chartObj.model.indicators);
            series.index = seriesIndex;
            var origin = Math.max(option.yAxis.visibleRange.min, 0);

            var sidebysideinfo = { Start: -0.35, End: 0.35 };
            var visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(series);

            var cSer = this;
			 var chart = cSer.chartObj;
            for (var i = 0; i < visiblePoints.length; i++) {
                var pointIndex = i;
                var point = visiblePoints[i];

                var y1 = point.YValues[0];
                var y2 = origin;
                if (point.visible) {
                    //calculate sides
                    var data = ej.EjSeriesRender.prototype.calculateSides(point, sidebysideinfo);
                    var x1 = data.x1;
                    var x2 = data.x2;
                    var rect = ej.EjSeriesRender.prototype.getRectangle(x1, y1, x2, y2, series, chart);

                    if (rect.X < 0) {
                        rect.Width = rect.Width + rect.X;
                        rect.X = 0;
                    }

                    rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
                    //drawing part
                    var xr = Math.min(0, rect.Width);
                    var yr = Math.min(0, rect.Height);
                    if (rect.Width < 0.0001)// condition checked for IE width issue
                        rect.Width=0.0001;
                    var bounds;
                    if ((xr == 0 || yr == 0) && rect.Width > 0) {
                        options = {
                            'id': this.chartObj.svgObject.id + "_" + seriesIndex + "_" + series.name,
                            'x': rect.X + cSer.chartObj.canvasX,
                            'y': rect.Y + cSer.chartObj.canvasY,
                            'width': rect.Width,
                            'height': rect.Height,
                            'fill': (!visiblePoints[i].fill ) ? style.fill : visiblePoints[i].fill,
                            'stroke-width': series.border.width,
                            'plot': y1 < 0 ? "negative" : "positive",
                            'opacity': series.histogram.opacity,
                            'stroke': (!visiblePoints[i].fill ) ? series.border.color : visiblePoints[i].fill
                        };

                        this.chartObj.svgRenderer.drawRect(options, this.gIndicatorGroupEle);

                        var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                        this._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                    }
                }
            }

            this.chartObj.svgRenderer.append(this.gIndicatorGroupEle, this.chartObj.gIndicatorEle);
        },
        _addRegion: function (chart, bounds, series, point, pointIndex) {
            var type = series.type;
            if (series.index >= 0) {
                var regionItem = { SeriesIndex: series.index, Region: { PointIndex: pointIndex, Bounds: bounds }, type: type, isIndicator: true };
                chart.model.chartRegions.push(regionItem);
            }
        },

        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
              indicator.segment = [];
              indicator.xRange = {};
              indicator.yRange = {};
			  indicator.period = parseInt(indicator.period);   // to convert float value to integer
              if (indicator.period < indicator._points.length && indicator.period > 0) {
              //Upperband 
              var upperSeries = $.extend(true, {}, indicator);
              upperSeries.name = "upperseries";
              upperSeries.fill = indicator.upperLine.fill;
              upperSeries.width = indicator.upperLine.width;
              upperSeries.type = "line";

              for (var k = 0; k < indicator._points.length; k++) {
                  indicator._points[k].YValues.length = 1;
                  indicator._points[k].YValues[0] = indicator._points[k].y = 70;
              }
              
              upperSeries.points = $.extend(true, [], indicator._points);
              var xValues = sender._getXValues(upperSeries);
              var yValues = sender._getYValues(upperSeries);
              
              indicator.xRange.min = Math.min.apply(Math, xValues);
              indicator.xRange.max = Math.max.apply(Math, xValues);
              indicator.yRange.min = Math.min.apply(Math, yValues);
              indicator.yRange.max = Math.max.apply(Math, yValues);

              indicator.segment.push(upperSeries);

              //Lowerband
              var lowerSeries = $.extend(true, {}, indicator);
              lowerSeries.name = "lowerSeries";
              lowerSeries.width = indicator.lowerLine.width;
              lowerSeries.fill = indicator.lowerLine.fill;
              lowerSeries.type = "line";
           

              for (var n = 0; n < indicator._points.length; n++) {
                  indicator._points[n].YValues[0] = indicator._points[n].y = 30;
              }

               lowerSeries.points = $.extend(true, [], indicator._points);

              yValues = sender._getYValues(lowerSeries);

              indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

              indicator.segment.push(lowerSeries);
              
              // signalLine
              var trendSeries = $.extend(true, {}, indicator); 
              trendSeries.name = "signalSeries";
              trendSeries.width = indicator.width;
              trendSeries.fill = indicator.fill;
              trendSeries.type = "line";
              
              var c = 0, c1 = 0;
              var pmf = 0;
              var nmf = 0;
              var len = indicator.period;
              c1 = indicator._points[0].close;

              for (var i = 1; i <=len; ++i) {
                  c = indicator._points[i].close;
                  if (c > c1)
                      pmf += c - c1;
                  else if (c < c1)
                      nmf += c1 - c;
                  c1 = c;
                  indicator._points[i-1].YValues[0] = indicator._points[i-1].y = null;
              }
              pmf = pmf / len;
              nmf = nmf / len;
              
              indicator._points[len].YValues[0] = indicator._points[len].y = 100 - 100 / (1 + pmf / nmf);

              for (var j = len + 1; j < indicator._points.length; j++) {
                  c = indicator._points[j].close;
                  if (c > c1) {
                      pmf = (pmf * (len - 1) + (c - c1)) / len;
                      nmf = (nmf * (len - 1)) / len;
                  }
                  else if (c < c1) {
                      nmf = (nmf * (len - 1) + (c1 - c)) / len;
                      pmf = (pmf * (len - 1)) / len;
                  }
                  c1 = c;
                  indicator._points[j].YValues[0] = indicator._points[j].y = 100 - (100 / (1 + pmf / nmf));
              }
              trendSeries.points = $.extend(true, [], indicator._points);
              var trendPoints = [];
              for (var i = len, j = 0; i < trendSeries.points.length; i++, j++){
                  trendPoints[j] = trendSeries.points[i].y;
				  }

              indicator.yRange.min = Math.min(Math.min.apply(Math, trendPoints), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, trendPoints), indicator.yRange.max);

              indicator.segment.push(trendSeries);
              this.calculateIndicatorRange(sender, indicator);
           }
        },

        calculateIndicatorRange: function(sender, indicator){
            //Indicator X-Axis Calculation
            if (sender.model.indicatorRange[indicator.xAxisName] == undefined) {
                sender.model.indicatorRange[indicator.xAxisName] = {};
                sender.model.indicatorRange[indicator.xAxisName] = { min: indicator.xRange.min, max: indicator.xRange.max };
            } else {
                if (sender.model.indicatorRange[indicator.xAxisName].min > indicator.xRange.min)
                    sender.model.indicatorRange[indicator.xAxisName].min = indicator.xRange.min;
                if (sender.model.indicatorRange[indicator.xAxisName].max < indicator.xRange.max)
                    sender.model.indicatorRange[indicator.xAxisName].max = indicator.xRange.max;
            }
            //Indicator Y-Axis Calculation
            if (sender.model.indicatorRange[indicator.yAxisName] === undefined) {
                sender.model.indicatorRange[indicator.yAxisName] = {};
                sender.model.indicatorRange[indicator.yAxisName] = { min: indicator.yRange.min, max: indicator.yRange.max };
            } else {
                if (sender.model.indicatorRange[indicator.yAxisName].min > indicator.yRange.min)
                    sender.model.indicatorRange[indicator.yAxisName].min = indicator.yRange.min;
                if (sender.model.indicatorRange[indicator.yAxisName].max < indicator.yRange.max)
                    sender.model.indicatorRange[indicator.yAxisName].max = indicator.yRange.max;
            }
        },

        doAnimation: function () {
            var chartObj = this.chartObj;
            var clipRect = $(chartObj.gIndicatorEle).find("#" + this.gIndicatorGroupEle.id + "_ClipRect").children();
            $(clipRect).animate(
                { width: parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gIndicatorEle).find("#" + this.gIndicatorGroupEle.id + "_ClipRect").children(), "width")) },
                {
                    duration: 2000,

                    step: function (now) {
                        chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                    }
                });
        }

    },
    ej.ejRSI = ejExtendClass(ej.EjIndicatorRender);

    ej.indicatorTypes.rsi = ej.ejRSI;
	ej.ejMACD = ejExtendClass(ej.EjIndicatorRender, {


        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};

            var pointsLength = indicator._points.length;
            var longPeriod = indicator.longPeriod;
            var shortPeriod = indicator.shortPeriod;
            var trigger = indicator.trigger;
            var diff = longPeriod-shortPeriod;
            var shortEMA = [];
            var longEMA = [];
            var macdPoints = [];
            var signalEMA = [];
            var histogramPoints = [];
            var length = longPeriod + trigger;
            if (length <= indicator._points.length && ((length-2) >= 0) && (shortPeriod <= longPeriod) && (shortPeriod > 0)) {
                shortEMA = this.calculateEMA(indicator._points, shortPeriod, true);
                longEMA = this.calculateEMA(indicator._points, longPeriod, true);

                //Calculte MACD Line points: MACD Line: (shortPeriod EMA - longPeriod EMA)  
                for (i = 0; i < longEMA.length; i++)
                    macdPoints.push((shortEMA[i + diff] - longEMA[i]));

                //MACD Line 
                var macdSeries = $.extend(true, {}, indicator);
                macdSeries.name = "macdSeries";
                macdSeries.fill = indicator.macdLine.fill;
                macdSeries.width = indicator.macdLine.width;
                macdSeries.type = "line";
                macdSeries.xAxis = indicator.xAxis;
                macdSeries.yAxis = indicator.yAxis;

                var l = longPeriod - 1;
                for (var t = 0, i = 0; t < pointsLength; t++) {
                    if (t < l)
                        indicator._points[t].YValues[0] = indicator._points[t].y = null;
                    else {
                        indicator._points[t].YValues[0] = indicator._points[t].y = macdPoints[i];
                        i += 1;
                    }
                }
                macdSeries.points = $.extend(true, [], indicator._points);
                //////////////////////////////////////////////

                //Signal Line: Trigger value(9-day default) EMA of MACD Line 
                signalEMA = this.calculateEMA(macdPoints, trigger, false);
                var trendSeries = $.extend(true, {}, indicator);
                trendSeries.name = "signalSeries";
                trendSeries.fill = indicator.fill;
                trendSeries.width = indicator.width;
                trendSeries.type = "line";
                trendSeries.xAxis = indicator.xAxis;
                trendSeries.yAxis = indicator.yAxis;

                var l = longPeriod + trigger - 2;
                for (var t = 0, i = 0; t < pointsLength; t++) {
                    if (t < l)
                        indicator._points[t].YValues[0] = indicator._points[t].y = null;
                    else {
                        indicator._points[t].YValues[0] = indicator._points[t].y = signalEMA[i];
                        i += 1;
                    }
                }
                trendSeries.points = $.extend(true, [], indicator._points);
                /////////////////////////////////////////

                //MACD Histogram: MACD Line - Signal Line
                var histogramSeries = $.extend(true, {}, indicator);
                histogramSeries.border = {};
                histogramSeries.name = "histogramSeries";
                histogramSeries.fill = indicator.histogram.fill;
                histogramSeries.opacity = indicator.histogram.opacity;
                histogramSeries.fill = indicator.histogram.fill;
                histogramSeries.border = indicator.histogram.border;
                histogramSeries.type = "column";
                histogramSeries.xAxis = indicator.xAxis;
                histogramSeries.yAxis = indicator.yAxis;

                for (var j = 0, k = l; k < pointsLength; j++, k++)
                    histogramPoints.push((macdPoints[j + (trigger - 1)]) - (signalEMA[j]));
                for (var j = 0, k = l; k < pointsLength; j++, k++) {
                    indicator._points[k].YValues[0] = indicator._points[k].y = histogramPoints[j];
                }
                histogramSeries.points = $.extend(true, [], indicator._points);
                xValues = sender._getXValues(macdSeries);
                indicator.xRange.min = Math.min.apply(Math, xValues);
                indicator.xRange.max = Math.max.apply(Math, xValues);

                switch (indicator.macdType) {
                    case "line":
                        {
                            //Push MACD Line
                            indicator.yRange.min = Math.min.apply(Math, macdPoints);
                            indicator.yRange.max = Math.max.apply(Math, macdPoints);
                            indicator.segment.push(macdSeries);

                            //Push Signal Line
                            var min = Math.min.apply(Math, signalEMA), max = Math.max.apply(Math, signalEMA);
                            indicator.yRange.min = indicator.yRange.min < min ? indicator.yRange.min : min;
                            indicator.yRange.max = indicator.yRange.max > max ? indicator.yRange.max : max;
                            indicator.segment.push(trendSeries);
                        }
                        break;
                    case "histogram":
                        {
                            //Push histogram Series
                            var min1 = Math.min.apply(Math, histogramPoints), max1 = Math.max.apply(Math, histogramPoints);
                            indicator.yRange.min = indicator.yRange.min < min1 ? indicator.yRange.min : min1;
                            indicator.yRange.max = indicator.yRange.max > max1 ? indicator.yRange.max : max1;
                            indicator.segment.push(histogramSeries);
                        }
                        break;
                    case "both":
                        {
                            //Push histogram Series
                            var min1 = Math.min.apply(Math, histogramPoints), max1 = Math.max.apply(Math, histogramPoints);
                            indicator.yRange.min = indicator.yRange.min < min1 ? indicator.yRange.min : min1;
                            indicator.yRange.max = indicator.yRange.max > max1 ? indicator.yRange.max : max1;
                            indicator.segment.push(histogramSeries);

                            //Push MACD Line 
                            indicator.yRange.min = Math.min.apply(Math, macdPoints);
                            indicator.yRange.max = Math.max.apply(Math, macdPoints);
                            indicator.segment.push(macdSeries);

                            //Push Signal Line
                            var min = Math.min.apply(Math, signalEMA), max = Math.max.apply(Math, signalEMA);
                            indicator.yRange.min = indicator.yRange.min < min ? indicator.yRange.min : min;
                            indicator.yRange.max = indicator.yRange.max > max ? indicator.yRange.max : max;
                            indicator.segment.push(trendSeries);
                        }
                        break;
                }
                this.calculateIndicatorRange(sender, indicator);
            }
        },
        calculateEMA: function (points, period, flag) {
            var sma = 0;
            var initialEMA = 0;
            var pointsLength = points.length;
            var getPoints = [];
            var getEma = [];
            var emaPercent = (2 / (period + 1));
            if (flag) { // calculating EMA for Indicators Points
                for (var i = 0; i < period; i++) {
                    getPoints.push(points[i].close);
                    sma = sma + getPoints[i];
                }

                initialEMA = (sma / period);
                getEma.push(initialEMA);
                var emaAvg = initialEMA;
                for (j = period; j < pointsLength; j++) {
                    emaAvg = (points[j].close - emaAvg) * emaPercent + emaAvg;
                    getEma.push(emaAvg);
                }
            } else { //Calculating EMA for Signal Points
                for (var i = 0; i < period; i++) {
                    getPoints.push(points[i]);
                    sma = sma + getPoints[i];
                }

                initialEMA = (sma / period);
                getEma.push(initialEMA);
                var emaAvg = initialEMA;
                for (j = period; j < pointsLength; j++) {
                    emaAvg = (points[j] - emaAvg) * emaPercent + emaAvg;
                    getEma.push(emaAvg);
                }
            }
            return getEma;
        }

    });

    ej.indicatorTypes.macd = ej.ejMACD;
	ej.ejSMA = ejExtendClass(ej.EjIndicatorRender, {

      
        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};


         
            // signalLine
            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

			//variable declaration
            var closevalues = [];
            var smavalues = [];
			var trendPoints = [];
            var sum;
            var length = indicator._points.length;
            var period = indicator.period;
			
			if (period <= 0 || length < period) return;
			
			//Push the Close values
            for (var i = 0; i < length; i++) {
               
                closevalues.push(indicator._points[i].close);

            }

            var close_length =closevalues.length;
			//calculation for SMA values
			
	     
            while (close_length >= period) {
                sum = 0;
                for (var j = 0; j < period; j++) {
                    sum = sum + closevalues[j];

                }
                sum = sum / period;
                smavalues.push(sum);
                closevalues.splice(0, 1);
                close_length = closevalues.length;
            }

	 
           //set the SMA values to Y-Axis
            for (var k = 0; k < length; k++) {
                if (k < period - 1) {
                    indicator._points[k].YValues[0] = indicator._points[k].y = null;

                }
                else {
                    indicator._points[k].YValues[0] = indicator._points[k].y =smavalues[k -(period - 1) ];
                }

            }

            
            trendSeries.points = $.extend(true, [], indicator._points);
            xValues = sender._getXValues(trendSeries);
            
            //Assign the Y-Axis values into trendPoints
            for (var i = period-1 , j = 0; i < trendSeries.points.length; i++, j++)
             {
                trendPoints[j] = trendSeries.points[i].y;
             }
                
			//X-Axis and Y-Axis range calculation
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, trendPoints);
            indicator.yRange.max = Math.max.apply(Math, trendPoints);
            
			//push trendSeries
            indicator.segment.push(trendSeries);
        	this.calculateIndicatorRange(sender, indicator);
            
          
        }

    });

    ej.indicatorTypes.sma = ej.ejSMA;
	
	 ej.ejEMA = ejExtendClass(ej.EjIndicatorRender, {


        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};



            // signalLine
            
            trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.type = "line";
            trendSeries.xAxis = indicator.xAxis;
            trendSeries.yAxis = indicator.yAxis;


            var getCloseValues = [];       
            var ema = [];
            var pointsLength = indicator._points.length;       
            var period = indicator.period;
            var sma = 0;
            if(pointsLength < period || period <=0) return ;
            // K=Smoothing Factor
            var k = (2 / (period + 1));

            //Get a close Values and push the previous Ema 
            for (var i = 0; i < period; i++) {
                getCloseValues.push(indicator._points[i].close);
                sma = sma + getCloseValues[i];            
            }
            ema.push(sma / period);

            //Find Remaining EMA by Adding smoothing factor K to the close values
            for (j = period; j < pointsLength; j++)
                  {              
                   ema.push((indicator._points[j].close - ema[j - period]) * k + ema[j - period]);                  
                   }        
            
            // Set Ema to the Indicator Points
           
            for (var t = 0; t < pointsLength; t++) {
                if (t < (period - 1)) {
                    indicator._points[t].YValues[0] = indicator._points[t].y = null;
                }
                else if (t >= (period - 1))
                {
                    indicator._points[t].YValues[0] = indicator._points[t].y = ema[t - (period - 1)];
                }
            }
             
            // Send the indicator Points for draw series.
            trendSeries.points = $.extend(true, [], indicator._points);
            xValues = sender._getXValues(trendSeries);
            var trendPoints = [];
            for (var i = period, j = 0; i < trendSeries.points.length; i++, j++)
                trendPoints[j] = trendSeries.points[i].y;

            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, trendPoints);
            indicator.yRange.max = Math.max.apply(Math, trendPoints);

            indicator.segment.push(trendSeries);
        	this.calculateIndicatorRange(sender, indicator);
            
          
        }

    });

    ej.indicatorTypes.ema = ej.ejEMA;


    ej.ejSTOCHASTIC = ejExtendClass(ej.EjIndicatorRender, {

        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
			var KPeriod = indicator.kPeriod;
            var DPeriod = indicator.dPeriod;
            var Period = indicator.period;
            if (Period < 1 || DPeriod < 1 || KPeriod < 1) return;
            if (Period > indicator._points.length || DPeriod > indicator._points.length || KPeriod > indicator._points.length) return;
			// Upper Line Calculation
            var upperSeries = $.extend(true, {}, indicator);
            upperSeries.name = "upperseries";
            upperSeries.type = "line";
            upperSeries.fill = indicator.upperLine.fill;
            upperSeries.width = indicator.upperLine.width;

            for (var k = 0; k < indicator._points.length; k++) {
                indicator._points[k].YValues.length = 1;
                indicator._points[k].YValues[0] = indicator._points[k].y = 80;
            }

            upperSeries.points = $.extend(true, [], indicator._points);
            var xValues = sender._getXValues(upperSeries);
            var yValues = sender._getYValues(upperSeries);

            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, yValues);
            indicator.yRange.max = Math.max.apply(Math, yValues);

            indicator.segment.push(upperSeries);
            // End for Upper Line Calculation

            // Lower Line Calculation
            var lowerSeries = $.extend(true, {}, indicator);
            lowerSeries.name = "lowerSeries";
            lowerSeries.width = indicator.lowerLine.width;
            lowerSeries.fill = indicator.lowerLine.fill;
            lowerSeries.type = "line";


            for (var n = 0; n < indicator._points.length; n++) {
                indicator._points[n].YValues[0] = indicator._points[n].y = 20;
            }

            lowerSeries.points = $.extend(true, [], indicator._points);

            xValues = sender._getXValues(lowerSeries);
            yValues = sender._getYValues(lowerSeries);

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

            indicator.segment.push(lowerSeries);
            // End for Lower Line Calculation

            // Signal and Period Line Calculation

            var periodSeries = $.extend(true, {}, indicator);
            periodSeries.name = "periodSeries";
            periodSeries.width = indicator.periodLine.width;
            periodSeries.fill = indicator.periodLine.fill;
            periodSeries.type = "line";

            var HighValues = [];
            var LowValues = [];
            var CloseValues = [];
            var check = [];
            for (var i = 0; i < indicator._points.length; i++) {
                LowValues[i] = indicator._points[i].low;
                HighValues[i] = indicator._points[i].high;
                CloseValues[i] = indicator._points[i].close;
            }


            AddPoints(Period, KPeriod, indicator._points);
            check = indicator._points;
            SMA(Period, KPeriod, indicator._points);
            check = indicator._points;
            periodSeries.points = $.extend(true, [], indicator._points);

            xValues = sender._getXValues(periodSeries);
            yValues = sender._getYValues(periodSeries);

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

            indicator.segment.push(periodSeries);


            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

            SMA((Period+KPeriod-1), DPeriod, indicator._points);
            trendSeries.points = $.extend(true, [], indicator._points);

            indicator.segment.push(trendSeries);

            ///SMA Calculation(Signal Line and Period Line)

            function SMA(Period1, Period2, series) {
                if (indicator._points.length >= Period1 + Period2) {
                    SMACal(Period1, Period2, series);
                }
            }

            function SMACal(period1, period2, series) {
                var count = period1 + (period2-1);
                var temp = [];
                var values = [];
                var sum;
                for (var i = 0; i < indicator._points.length; i++) {
                    var val = check[i].y;
                    temp.push(val);
                }
                var length = temp.length;

                while (length >= count) {
                    sum = 0;
                    for (var j = period1-1; j < (period1 + period2-1); j++) {
                        sum = sum + temp[j];

                    }
                    sum = sum / period2;
                    values.push(sum.toFixed(2));
                    temp.splice(0, 1);
                    length = temp.length;
                }
                var len = count - 1;
                for (var k = 0; k < indicator._points.length; k++) {
                    if (k < len) {
                        series[k].YValues[0] = series[k].y = null;

                    }
                    else {
                        series[k].YValues[0] = series[k].y = Number((values[k - len]));
                    }

                }

            }

            // End for SMA Calculation

            // Period Line Calculation

            function AddPoints(Period1, Period2, series) {
                if (indicator._points.length > Period1) {
                    ComputeStochastics(Period1, Period2, series);
                }
            }
            function ComputeStochastics(len1, len2, series) {
                var len = len1 + len2;
                var mins = [];
                var maxs = [];
                var max;
                var min;
                var top = 0;
                var bottom = 0;
                for (var i = 0; i < len1 - 1; ++i) {
                    maxs.push(0);
                    mins.push(0);
                }

                for (var i = len1 - 1; i < indicator._points.length; ++i) {
                    min = Number.MAX_VALUE;
                    max = Number.MIN_VALUE;
                    for (var j = 0; j < len1; ++j) {
                        min = Math.min(min, LowValues[i - j]);
                        max = Math.max(max, HighValues[i - j]);
                    }
                    maxs.push(max);
                    mins.push(min);
                }

                for (var i = 0 ; i < len - 1; ++i) {
                    series[i].YValues.length = 1;
                    series[i].YValues[0] = series[i].y = null;
                }

                for (var i = len1 - 1; i < indicator._points.length; ++i) {
                    top = 0;
                    bottom = 0;
                    top += CloseValues[i] - mins[i];
                    bottom += maxs[i] - mins[i];
                    series[i].YValues.length = 1;
                    series[i].YValues[0] = series[i].y = (((top / bottom) * 100));
                }
            }

            // End for Period Line Calculation

            // Indicator Axis Range Calculation
            this.calculateIndicatorRange(sender, indicator);
        }


    });
    ej.indicatorTypes.stochastic = ej.ejSTOCHASTIC;
ej.ejbollingerBand = ejExtendClass(ej.EjIndicatorRender, {
        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
            var sum = 0.0;
            var deviationSum = 0.0;
            var limit = indicator._points.length;
            var multiplier =indicator.standardDeviations;
            var movingavg =indicator.period;
            var length = Math.round(movingavg);
            var smaPoints = [];
            var deviations = [];
            var bbiPoints = [];
            //Calculating simple moving average
         if ((length < limit) && (length > 0))
{
            for (var i = 0; i < length; i++) {
                sum += indicator._points[i].close;

            }
            var sma = sum / movingavg;
            //Calculating smapoints using iteration
            for (var i = 0; i < limit; ++i) {
                if (i >= length - 1 && i < limit) {
                    if (i - movingavg >= 0) {
                        var diff = indicator._points[i].close - indicator._points[i - length].close;

                        sum = sum + diff;
                        sma = sum / (movingavg);
                        smaPoints[i] = sma;
                        deviations[i] = Math.pow(indicator._points[i].close - sma, 2);
                        deviationSum += deviations[i] - deviations[i - length];


                    }
                    else {

                        smaPoints[i] = sma;
                        deviations[i] = Math.pow(indicator._points[i].close - sma, 2);
                        deviationSum += deviations[i];
                    }


                    //calculation for standarddeviation for each period

                    var range = Math.sqrt(deviationSum / (movingavg));

                    //lower Band points for bollinger band
                    var lb = smaPoints[i] - (multiplier * range);
                    //upper Band points for bollinger band
                    var ub = smaPoints[i] + (multiplier * range);
                    //Binding the lowerband,upperband and middleband values to bollingerband array if the i value equal to period
                    if (i + 1 == length) {
                        for (var j = 0; j < length - 1; j++)
                            bbiPoints[j] = { "X": indicator._points[j].X, "mb": smaPoints[i], "lb": lb, "ub": ub, visible: true };



                    }
                    //binding the lb,ub,mb values to bollinger band array if i greather than period value.
                    bbiPoints[i] = { "X": indicator._points[i].X, "mb": smaPoints[i], "lb": lb, "ub": ub, visible: true };


                }


                else {
                    if (i < movingavg - 1) {

                        smaPoints[i] = sma;
                        deviations[i] = Math.pow(indicator._points[i].close - sma, 2);
                        deviationSum += deviations[i];
                    }


                }

            }


            var upperSeries = $.extend(true, {}, indicator);
            upperSeries.name = "upperseries";
            upperSeries.fill = indicator.upperLine.fill;
            upperSeries.width = indicator.upperLine.width;
            upperSeries.type = "line";

            //binding bollingerband array values to upper series and assigning null values 
            for (var k = 0; k < indicator._points.length; k++) {
                if (k >= (length - 1)) {
                    indicator._points[k].YValues.length = 1;
                    indicator._points[k].YValues[0] = indicator._points[k].y = bbiPoints[k].ub;
                }
                else {
                    indicator._points[k].YValues[0] = indicator._points[k].y = null;
                }
            }

            upperSeries.points = $.extend(true, [], indicator._points);
            var xValues = sender._getXValues(upperSeries);
            var values = [];
            //storing the y values from the period value
            for (var i = length; i < upperSeries.points.length; i++)
                values.push(upperSeries.points[i].y);
            var yValues = values;
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, yValues);
            indicator.yRange.max = Math.max.apply(Math, yValues);

            indicator.segment.push(upperSeries);


            var lowerSeries = $.extend(true, {}, indicator);
            lowerSeries.name = "lowerSeries";
            lowerSeries.width = indicator.lowerLine.width;
            lowerSeries.fill = indicator.lowerLine.fill;
            lowerSeries.type = "line";


            for (var n = 0; n < indicator._points.length; n++) {
                if (n >= (length - 1)) {
                    indicator._points[n].YValues.length = 1;
                    indicator._points[n].YValues[0] = indicator._points[n].y = bbiPoints[n].lb;
                }
                else {
                    indicator._points[n].YValues[0] = indicator._points[n].y = null;
                }
            }


            lowerSeries.points = $.extend(true, [], indicator._points);

            xValues = sender._getXValues(lowerSeries);

            var lowvalues = [];
            for (var i = length; i < lowerSeries.points.length; i++)
                lowvalues.push(lowerSeries.points[i].y);
            var yValues = lowvalues;

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);

            indicator.segment.push(lowerSeries);



            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

            for (var m = 0; m < indicator._points.length; m++) {
                if (m >= (length - 1)) {
                    indicator._points[m].YValues.length = 1;
                    indicator._points[m].YValues[0] = indicator._points[m].y = bbiPoints[m].mb;
                }
                else {
                    indicator._points[m].YValues[0] = indicator._points[m].y = null;
                }
            }
            trendSeries.points = $.extend(true, [], indicator._points);
            xValues = sender._getXValues(trendSeries);
            var trendvalues = [];
            for (var i = length; i < trendSeries.points.length; i++)
                trendvalues.push(trendSeries.points[i].y);
            var yValues = trendvalues;

            indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
            indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
            indicator.yRange.min = Math.min(Math.min.apply(Math, yValues), indicator.yRange.min);
            indicator.yRange.max = Math.max(Math.max.apply(Math, yValues), indicator.yRange.max);
            indicator.segment.push(trendSeries);
            //calculating indicator range
            this.calculateIndicatorRange(sender, indicator);
           }
        }
     
        });
    ej.indicatorTypes.bollingerband = ej.ejbollingerBand;
  ej.ejATR = ejExtendClass(ej.EjIndicatorRender,{       
        calculateSegment:function(indicator,sender)
        {
            //calculation for atr
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
            var atrPoints = [];
            var high_Low, high_ClosePrevious, low_ClosePrevious;
            high_Low = high_ClosePrevious = low_ClosePrevious = 0.0;
            var trueRange;
            var sumTruRange = 0.0;
            var firstFlag = false;
            var atr;
            var period =  indicator.period;
            if ((period <= 0) || (period > indicator._points.length))
                return;
            for (var i = 0; i < period; i++) {

                high_Low = indicator._points[i].high - indicator._points[i].low;
                if (i > 0) {
                    high_ClosePrevious = Math.abs(indicator._points[i].high - indicator._points[i - 1].close);
                    low_ClosePrevious = Math.abs(indicator._points[i].low - indicator._points[i - 1].close);
                 }
              
                trueRange = Math.max(high_Low, high_ClosePrevious, low_ClosePrevious);
               
                sumTruRange += trueRange;
                atr = (sumTruRange / period);
          
                    atrPoints[i] = { "X": indicator._points[i].X, "YValues": atr };

            }
       
             for (var i = period; i < indicator._points.length; ++i) {
                high_Low = indicator._points[i].high - indicator._points[i].low;
                high_ClosePrevious = Math.abs(indicator._points[i].high - indicator._points[i - 1].close);
                low_ClosePrevious = Math.abs(indicator._points[i].low - indicator._points[i-1].close);
                trueRange = Math.max(high_Low, high_ClosePrevious, low_ClosePrevious);
               
               atr = (atrPoints[i-1].YValues * (period - 1) + trueRange) / period;
             
                atrPoints[i] = {
                    "X": indicator._points[i].X, "YValues": atr
                };

               }
          


             var trendSeries = $.extend(true, {}, indicator);


            trendSeries.name = "trendSeries";
            trendSeries.width = indicator.width;
            trendSeries.fill = indicator.fill;
            trendSeries.type = "line";

         

            for (var k = 0; k < indicator._points.length; k++) {
                if (k >= (period - 1)) {
                    indicator._points[k].YValues.length = 1;
                    indicator._points[k].YValues[0] = indicator._points[k].y = atrPoints[k].YValues;
                }
                else {
                    indicator._points[k].YValues[0] = indicator._points[k].y = null;
                }
            }

           
       
            trendSeries.points = $.extend(true, [], indicator._points);
           
            var xValues = sender._getXValues(trendSeries);
           
            var values = [];
            for (var i = period; i < trendSeries.points.length; i++)
             values.push(trendSeries.points[i].y);
            
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, values);
            indicator.yRange.max = Math.max.apply(Math, values);

           indicator.segment.push(trendSeries);
        

            this.calculateIndicatorRange(sender, indicator);


            
        }

    });
    ej.indicatorTypes.atr = ej.ejATR;

 ej.ejaccumulationDistribution = ejExtendClass(ej.EjIndicatorRender, {
        calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
            indicator.segment = [];
            indicator.xRange = {};
            indicator.yRange = {};
            var adlPoints = [];
            var sum = 0;
          //calculating accumulation distribution line points 
            for (var i = 0; i < indicator._points.length;i++) {
                var close = indicator._points[i].close;
                
                    var value=(((close - indicator._points[i].low) - (indicator._points[i].high - close)) / (indicator._points[i].high - indicator._points[i].low));
                    sum += value * indicator._points[i].volume;
                    adlPoints[i] = sum;
                
            }
            var trendSeries = $.extend(true, {}, indicator);
            trendSeries.name = "trendseries";
            trendSeries.fill = indicator.fill;
            trendSeries.width = indicator.width;
            trendSeries.type = "line";

            //Binding the accumulation distribution array values to indicator y values
            for (var k = 0; k < indicator._points.length; k++) {

                   indicator._points[k].YValues[0] = indicator._points[k].y = adlPoints[k];
                  }

            trendSeries.points = $.extend(true, [], indicator._points);
            var xValues = sender._getXValues(trendSeries);
           var yValues = sender._getYValues(trendSeries);
         
            indicator.xRange.min = Math.min.apply(Math, xValues);
            indicator.xRange.max = Math.max.apply(Math, xValues);
            indicator.yRange.min = Math.min.apply(Math, yValues);
            indicator.yRange.max = Math.max.apply(Math, yValues);

            indicator.segment.push(trendSeries);
            //calculating indicator range
            this.calculateIndicatorRange(sender, indicator);


        }

    });
    ej.indicatorTypes.accumulationdistribution = ej.ejaccumulationDistribution;
ej.ejTMA = ejExtendClass(ej.EjIndicatorRender, {
     calculateSegment: function (indicator, sender) {
         this.chartObj = sender;
         indicator.segment = [];
         indicator.xRange = {};
         indicator.yRange = {};

         // signalLine
         var trendSeries = $.extend(true, {}, indicator);
         trendSeries.name = "trendSeries";
         trendSeries.width = indicator.width;
         trendSeries.fill = indicator.fill;
         trendSeries.type = "line";

         //variable declaration
         var closevalues = [];
         var close_values = [];
         var smavalues = [];
         var tmavalues = [];
         var smaPoints = [];
         var trendPoints = [];
         var sum=0;
         var sum1;
         var length = indicator._points.length;
         var period = indicator.period;
        if (period < 1) return;
        
         if (period < length)
         {
                //Push the values to closevalues array
             for (var i = 0; i < length; i++) {
                 closevalues.push(indicator._points[i].close);
             }
             //find the closevalues length
             var close_length = closevalues.length;
         
             //calculation for SMA values

             while (close_length >= period) {
                 sum = 0;
                 for (var j = 0; j < period; j++) {
                     sum = sum + closevalues[j];

                 }
                 sum = sum / period;
                 smavalues.push( parseFloat(sum.toFixed(2)));
                 closevalues.splice(0, 1);
                 close_length = closevalues.length;
             }
             //calculating sma values depend on period
             for (var k = 0; k < length; k++) {
                 if (k < period - 1) {
                     sum = 0;
                     for (var j = 0; j < k+1; j++) {
                         sum = sum + indicator._points[j].close;

                     }
                     sum = sum / (k+1);
                     smaPoints[k] = parseFloat(sum.toFixed(2));
                 }
                 else {
                     smaPoints[k]= smavalues[k - (period - 1)];
                 }

             }
             //find the smavalues count
             var sma_count = smaPoints.length;
             //calculating TMA values
             while (sma_count >=period) {
                 sum1 = 0;
                 for (var j = 0; j < period; j++) {
                     sum1 = sum1 + smaPoints[j];

                 }
                 sum1 = sum1 / period;
                 tmavalues.push(sum1);
                 smaPoints.splice(0, 1);
                 sma_count = smaPoints.length;
                 }
            //set the TMA values to Y-Axis
             for (var k = 0; k < length; k++) {
                 if (k < period - 1) {
                     indicator._points[k].YValues[0] = indicator._points[k].y = null;

                 }
                 else {
                      indicator._points[k].YValues[0] = indicator._points[k].y = tmavalues[k - (period - 1)];
                    
                 }

             }
             trendSeries.points = $.extend(true, [], indicator._points);
             xValues = sender._getXValues(trendSeries);

             //Assign the Y-Axis values into trendPoints
             for (var i = period - 1, j = 0; i < trendSeries.points.length; i++, j++) {
                 trendPoints[j] = trendSeries.points[i].y;
             }

             //X-Axis and Y-Axis range calculation
             indicator.xRange.min = Math.min.apply(Math, xValues);
             indicator.xRange.max = Math.max.apply(Math, xValues);
             indicator.yRange.min = Math.min.apply(Math, trendPoints);
             indicator.yRange.max = Math.max.apply(Math, trendPoints);

             //push trendSeries
             indicator.segment.push(trendSeries);
             this.calculateIndicatorRange(sender, indicator);

            
         }
     }
 });
 ej.indicatorTypes.tma = ej.ejTMA;
	
	ej.ejMomentum = ejExtendClass(ej.EjIndicatorRender, {
	       calculateSegment: function (indicator, sender) {
            this.chartObj = sender;
              indicator.segment = [];
              indicator.xRange = {};
              indicator.yRange = {};

              //Upperband 
              var upperSeries = $.extend(true, {}, indicator);
              upperSeries.name = "upperseries";
              upperSeries.fill = indicator.upperLine.fill;
              upperSeries.width = indicator.upperLine.width;
              upperSeries.type = "line";

			  var period = indicator.period;
			  var pointsLength = indicator._points.length;
			  
          if(period > 0 && period < pointsLength -1){  // to check period value is greater than 0 and it is less than pointslength
              for (var k = 0; k < pointsLength; k++) {
                  indicator._points[k].YValues.length = 1;
                  indicator._points[k].YValues[0] = indicator._points[k].y = 100;
              }
              
              upperSeries.points = $.extend(true, [], indicator._points);
              var xValues = sender._getXValues(upperSeries);
              var yValues = sender._getYValues(upperSeries);
              
              indicator.xRange.min = Math.min.apply(Math, xValues);
              indicator.xRange.max = Math.max.apply(Math, xValues);
              indicator.yRange.min = Math.min.apply(Math, yValues);
              indicator.yRange.max = Math.max.apply(Math, yValues);

              indicator.segment.push(upperSeries);
              
              // signalLine
              var trendSeries = $.extend(true, {}, indicator); 
              trendSeries.name = "trendSeries";
              trendSeries.width = indicator.width;
              trendSeries.fill = indicator.fill;
              trendSeries.type = "line";
              
               var c = []; 
        var len = indicator.period;      // period value
            
        for (var i = 0; i < indicator._points.length; ++i)
        {
		   var points = indicator._points;
            if (!(i < len))                                       // calculate signal line points
                indicator._points[i].YValues[0] =indicator._points[i].y= (points[i].close / (points[i - len].close) *100);
        }

        for (var k = 0; k < len && k<indicator._points.length; k++) {
            indicator._points[k].YValues[0] = indicator._points[k].y = null;
        }
			  
              trendSeries.points = $.extend(true, [], indicator._points);
              xValues = sender._getXValues(trendSeries);
              var trendPoints = [];
              for (var i = len, j = 0; i < trendSeries.points.length; i++, j++){
                  trendPoints[j] = trendSeries.points[i].y;
			  }

              indicator.xRange.min = Math.min(Math.min.apply(Math, xValues), indicator.xRange.min);
              indicator.xRange.max = Math.max(Math.max.apply(Math, xValues), indicator.xRange.max);
              indicator.yRange.min = Math.min(Math.min.apply(Math, trendPoints), indicator.yRange.min);
              indicator.yRange.max = Math.max(Math.max.apply(Math, trendPoints), indicator.yRange.max);

              indicator.segment.push(trendSeries);
              this.calculateIndicatorRange(sender, indicator);
           }
        }
	});
	
	ej.indicatorTypes.momentum = ej.ejMomentum;
	
    ej.EjSeriesRender.prototype = {

    setLineSeriesStyle: function (currentSeries) {

        var seriesIndex = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
        var serColor = this.chartObj.model.seriesColors[seriesIndex];
       
        
        var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(currentSeries) + seriesIndex, serColor, 0, 0, 0, $(this.chartObj.svgObject).height(), this.chartObj.gSeriesEle);
        var seriesBorder = this.chartObj.model.seriesBorderColors[seriesIndex];
        return { SeriesInterior: seriesInterior, SeriesBorder: seriesBorder };

    },
    getSeriesName: function(series) {
        var serName;
        if (series.name)
            serName = series.name.replace(/\s/g, '');
        else
            serName = "series";
        
        return serName;
    },
    
    drawAreaPath: function (series, style, direction, canvasTranslate) {
	    var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
        var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var translate = null;
        var areaType = this.chartObj.model.AreaType;
        if (areaType == "cartesianaxes")
        {
            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
            translate = 'translate(' + trans.x + ',' + trans.y + ')'
        }
        var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate};

        this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
        if (direction != "") {
            var options = {
                'id': this.chartObj.svgObject.id + "_Series" + seriesIndex,
                'fill': style.SeriesInterior,
                'fill-opacity': series.opacity,
                'stroke-width': series.border.width,
                'stroke': series.border.color,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'd': direction
            };
            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle, canvasTranslate);
        } 
    },

    setAreaSeriesStyle: function (series) {

        var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var maxY = Math.max.apply(0,$.map(series.points, function(v) {return v.YValues[0];
        }));
        var minY = Math.min.apply(0, $.map(series.points, function(v) {
            return v.YValues[0];
        }));
        var minX = Math.min.apply(0, $.map(series.points, function (v) {
            return v.xValue;
        }));

        var point1 = {xValue: minX, YValues : []};
        point1.YValues[0] = minY;
        var point2 = {xValue: minX, YValues : []};
        point2.YValues[0] = maxY;
        var point1Xy = ej.EjSvgRender.utils._getPoint(point1, series);
        var point2Xy = ej.EjSvgRender.utils._getPoint(point2, series);
        var serColor = this.chartObj.model.seriesColors[seriesIndex];

        var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(series) + seriesIndex, serColor, point1Xy.X, point1Xy.Y, point2Xy.X, point2Xy.Y, this.chartObj.gSeriesEle);

        return { SeriesInterior: seriesInterior };

    },





    naturalSpline: function (points) {
        var count = points.length;
        //constant
        var yIndex = 0, a = 6;
        var ySpline = [], ySplineDuplicate = [];

        // assigning the first and last value as zero
        ySpline[0] = ySplineDuplicate[0] = 0;
        ySpline[points.length - 1] = 0;

        for (var i = 1; i < count - 1; i++) {
            var d1 = points[i].xValue - points[i - 1].xValue;
            var d2 = points[i + 1].xValue - points[i - 1].xValue;
            var d3 = points[i + 1].xValue - points[i].xValue;
            var dy1 = points[i + 1].YValues[yIndex] - points[i].YValues[yIndex];
            var dy2 = points[i].YValues[yIndex] - points[i - 1].YValues[yIndex];

            if (d1 == 0 || d2 == 0 || d3 == 0) {
                ySpline[i] = 0;
                ySplineDuplicate[i] = 0;
            }
            else {
                var p = 1 / (d1 * ySpline[i - 1] + 2 * d2);

                ySpline[i] = -p * d3;
                ySplineDuplicate[i] = p * (a * (dy1 / d3 - dy2 / d1) - d1 * ySplineDuplicate[i - 1]);
            }
        }

        for (var k = count - 2; k >= 0; k--)
            ySpline[k] = ySpline[k] * ySpline[k + 1] + ySplineDuplicate[k];


        return ySpline;

    },
    getBezierControlPoints: function (point1, point2, ySpline1, ySpline2, yIndex) {
        var controlPoint1;
        var controlPoint2;
        var one3 = 1 / 3.0;
        var deltaX2 = (point2.xValue - point1.xValue);

        deltaX2 = deltaX2 * deltaX2;

        var dx1 = 2 * point1.xValue + point2.xValue;
        var dx2 = point1.xValue + 2 * point2.xValue;

        var dy1 = 2 * point1.YValues[yIndex] + point2.YValues[yIndex];
        var dy2 = point1.YValues[yIndex] + 2 * point2.YValues[yIndex];

        var y1 = one3 * (dy1 - one3 * deltaX2 * (ySpline1 + 0.5 * ySpline2));
        var y2 = one3 * (dy2 - one3 * deltaX2 * (0.5 * ySpline1 + ySpline2));

        controlPoint1 = { xValue: dx1 * one3, YValues: [y1] };
        controlPoint2 = { xValue: dx2 * one3, YValues: [y2] };

        return { controlPoint1: controlPoint1, controlPoint2: controlPoint2 };


    },

    improveChartPerformance: function(chartSeries) {
        var tempPoints = [];
       
        var xDelta = chartSeries.xAxis.visibleRange.max - chartSeries.xAxis.visibleRange.min;
        var yDelta = chartSeries.yAxis.visibleRange.max - chartSeries.yAxis.visibleRange.min;
       
        var xTolerance = Math.abs((xDelta * 1) / this.chartObj.model.m_AreaBounds.Width);
        var yTolerance = Math.abs((yDelta * 1) / this.chartObj.model.m_AreaBounds.Height);

        var prevXValue = (chartSeries.points[0] && chartSeries.points[0].X > xTolerance) ? 0 : xTolerance;
        var prevYValue = (chartSeries.points[0] && chartSeries.points[0].y > yTolerance) ? 0 : yTolerance;

        var xVal = 0;
        for (var k = 0; k < chartSeries.points.length; k++) {
            var isEmpty = (chartSeries.points[k].isEmpty === null || chartSeries.points[k].isEmpty === undefined) ? false : chartSeries.points[k].isEmpty;
            chartSeries.points[k].visible = (chartSeries.points[k].visible === null || chartSeries.points[k].visible === undefined) ? true : chartSeries.points[k].visible;
            if (chartSeries.points[k].xValue === null || chartSeries.points[k].xValue === undefined) {
                chartSeries.points[k].isEmpty = true;
                chartSeries.points[k].visible = false;
                chartSeries.points[k].xValue = chartSeries.xAxis.visibleRange.min;
            } else
                chartSeries.points[k].isEmpty = isEmpty;
            if ((chartSeries.points[k].YValues[0] == null) || (chartSeries.points[k].YValues[0] == 'undefined') || (chartSeries.points[k].YValues[0] === "")) {
                chartSeries.points[k].isEmpty = true;
                chartSeries.points[k].visible = false;
                chartSeries.points[k].YValues[0] = chartSeries.yAxis.visibleRange.min;
            } else {
                chartSeries.points[k].isEmpty = chartSeries.points[k].isEmpty;
                chartSeries.points[k].visible = (chartSeries.points[k].isEmpty) ? !chartSeries.points[k].isEmpty : chartSeries.points[k].visible;
            }

            var xVal = chartSeries.points[k].xValue;
            var yVal = chartSeries.points[k].YValues[0];
            if (Math.abs(prevXValue - xVal) >= xTolerance || Math.abs(prevYValue - yVal) >= yTolerance) {

                tempPoints.push(chartSeries.points[k]);
                prevXValue = xVal;
                prevYValue = yVal;
            }
        }
        return tempPoints;

    },
    _isVisiblePoints: function(chartSeries) {
        var points = chartSeries.points;
        for (var k = 0; k < points.length; k++) {
            var isEmpty = (ej.util.isNullOrUndefined(points[k].isEmpty)) ? false : points[k].isEmpty;
            points[k].visible = (ej.util.isNullOrUndefined(points[k].visible)) ? true : points[k].visible;
            if (ej.util.isNullOrUndefined(points[k].xValue)) {
                points[k].isEmpty = true;
                points[k].visible = false;
                points[k].xValue = chartSeries.xAxis.visibleRange.min;
            } else
                points[k].isEmpty = isEmpty;

            if ((points[k].YValues[0] == null || points[k].YValues[0] == 'undefined') || (chartSeries._hiloTypes && (points[k].YValues[1] == undefined || points[k].YValues[1] == null))) {
                points[k].isEmpty = true;
                points[k].visible = false;
            } else {
                points[k].isEmpty = points[k].isEmpty;
                points[k].visible = (points[k].isEmpty) ? !points[k].isEmpty : points[k].visible;
            }
            if (jQuery.type(points[k].xValue) == "date")
                points[k].xValue = (points[k].xValue).getTime();
        }
        return points;
    },
   
    _calculatePolarAxesSegment: function (currentseries) {

        var points = currentseries.visiblePoints;
        var segment = ej.EjSvgRender.utils._getStringBuilder();
        var SeriesIndex=$.inArray(currentseries, this.chartObj.model._visibleSeries)
        var size;
        var interval;
        var centerX = this.chartObj.model.centerX;
        var centerY = this.chartObj.model.centerY;
        // calculating side by side info.
        var sideInfo = this._getSeriesPosition(currentseries);

        size = $.extend(true, {}, currentseries.marker.size, size);
        if (size.width <= 10) {
            size.width = size.height = (1.5 * this.chartObj.model.elementSpacing);
        }
        if (currentseries.drawType.toLowerCase() == 'column') {
           
            var innerRadius = 0;
            var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + SeriesIndex };
            this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
            var interval = currentseries.xAxis.visibleRange.interval;
            currentseries.xAxis.labelPlacement = !(currentseries.xAxis.labelPlacement) ? "betweenticks" : currentseries.xAxis.labelPlacement;
            var ticks=0;
            if (currentseries.xAxis._valueType != "category" || (currentseries.xAxis._valueType == "category" && currentseries.xAxis.labelPlacement=="onticks"))
              ticks= interval / 2;
            var inversed = (currentseries.xAxis.isInversed) ? 0 : 1;
            interval = interval / sideInfo.all;
            var nextPoint = interval;
            var visiblePoints = currentseries.visiblePoints;
            var length = (currentseries.xAxis._valueType != "category") ? visiblePoints.length - 1 : visiblePoints.length;
            // calculation for start and end angle for each positions.
            for (var l = 0; l < length; l++) {
                var itemCurrentXPos = temp = (visiblePoints[l].xValue - currentseries.xAxis.visibleRange.min) + ((interval) * sideInfo.pos - ticks);
                itemCurrentXPos = (itemCurrentXPos) / this.chartObj.model.sumofYValues;
                point = visiblePoints[l];
               
                var nextPoint = (currentseries.xAxis.visibleRange.interval) / sideInfo.all;
                point.startAngle = 2 * Math.PI * itemCurrentXPos;
                point.endAngle = 2 * Math.PI * (itemCurrentXPos + ((nextPoint) / this.chartObj.model.sumofYValues));
            }
            if (!visiblePoints[visiblePoints.length - 1].startAngle) {
                visiblePoints[visiblePoints.length - 1].startAngle = visiblePoints[0].startAngle;
                visiblePoints[visiblePoints.length - 1].endAngle = visiblePoints[0].endAngle;
            }
        }
        for (var i = 0; i < points.length; i++) {
            var point1;
            var point2;

            // drawType 'Line' calculation
            if (currentseries.drawType.toLowerCase() == 'line') {
                point1 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i].xValue, points[i].y, this.chartObj);

                if (i < points.length - 1) {
                    point2 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i + 1].xValue, points[i + 1].y, this.chartObj);
                    segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                }
                    // when isClosed is true
                else {
                    if (currentseries.isClosed) {
                        point2 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[0].xValue, points[0].y, this.chartObj);
                        segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (point2.X) + " " + (point2.Y) + " ");
                    }
                }            
            }

            else if (currentseries.drawType.toLowerCase() == 'column') {
                var interval = currentseries.xAxis.visibleRange.interval;
                var inversed= (currentseries.xAxis.isInversed) ? 0 : 1;
                var interval = interval / sideInfo.all;
                // radius for each point based on the y values.
                var radius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, points[i].y);
                
                var chartStartAngle = -.5 * Math.PI;
                var startAngle = points[i].startAngle + chartStartAngle;
                var endAngle = points[i].endAngle + chartStartAngle - 0.000001;

                var x1,y1, x2, y2;
           
                // calculating arc when the Stacking mode is enabled
               
                if (currentseries.isStacking) {
				     var startValue = currentseries.stackedValue.StartValues[i];
                    var endValue = currentseries.stackedValue.EndValues[i];
                    radius = ((startValue==endValue) ? 0 : (this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, endValue)));

                    x1 = centerX + radius * Math.cos(startAngle);
                    y1 = centerY + radius * Math.sin(startAngle);

                    x2 = centerX + radius * Math.cos(endAngle);
                    y2 = centerY + radius * Math.sin(endAngle);

                    innerRadius = this.chartObj.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(currentseries.yAxis, (startValue == 0 && currentseries.yAxis.visibleRange.min != 0) ? currentseries.yAxis.visibleRange.min : startValue);

                    var dStartX = centerX + innerRadius * Math.cos(startAngle);
                    var dStartY = centerY + innerRadius * Math.sin(startAngle);

                    var dEndX = centerX + innerRadius * Math.cos(endAngle);
                    var dEndY = centerY + innerRadius * Math.sin(endAngle);
                     if (this.chartObj.model.isPolar) 
                            direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + "0" + " " + inversed + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " + innerRadius + " " + innerRadius + " " + "1" + " " + "0" + " " + "0" + " " + dStartX + " " + dStartY + " " + "z";
                       else
                            direction = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "L" + " " + dStartX + " " + dStartY + " " + "z";
                    }
                        // calculating arc for normal mode.
                    else {

                        x1 = centerX + radius * Math.cos(startAngle);
                        y1 = centerY + radius * Math.sin(startAngle);

                        x2 = centerX + radius * Math.cos(endAngle);
                        y2 = centerY + radius * Math.sin(endAngle);
                      if (this.chartObj.model.isPolar)
                            direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + radius + " " + radius + " " + "0" + " " + "0" + " " + inversed + " " + x2 + " " + y2 + " " + "L" + " " + centerX + " " + centerY + " " + "z";
                     else
                          direction = "M" + " " + x1 + " " + y1 + " " + "L" + " " + x2 + " " + y2 + " " + "L" + " " + centerX + " " + centerY + " " + "z";
                    }                   
        
                   // gradient creation
                var seriesInterior = this.chartObj.svgRenderer.createGradientElement(this.getSeriesName(currentseries) + "_" + SeriesIndex + i, this.chartObj.model.seriesColors[SeriesIndex], 0, 0, 0, 100, this.chartObj.gSeriesEle);
                options = {
                     'id': this.chartObj.svgObject.id + "_Series" + SeriesIndex + "_Point" + i,
                     'fill': seriesInterior,
                    'stroke': currentseries.border.color,
                    'stroke-width': currentseries.border.width,
                    'seriesIndex': SeriesIndex,
                    'opacity':currentseries.opacity,
                    'pointIndex': i,
                    'd': direction,
                    'start': points[i].startAngle - 1.57,
                    'end': points[i].endAngle - 1.57,
                    'radius': radius,
                    'innerR': innerRadius
                };
                if (this.chartObj.model.isPolar) {
                    options.x = centerX;
                    options.y = centerY;
					bounds = { PointIndex: i, StartAngle: points[i].startAngle, EndAngle: points[i].endAngle, CenterX: centerX, CenterY: centerY, Radius: radius, DRadius: innerRadius };
                }
				if (currentseries.type=="radar") {
                    var line1 = { x: centerX, y: centerY };
                    var line2 = { x: x1, y: y1 };
                    var line3 = { x: x2, y: y2 };
                    bounds = { PointIndex: i, Line1: line1, Line2: line2, Line3: line3 };
                }
                point = currentseries.visiblePoints[i];
                if ((!ej.util.isNullOrUndefined(currentseries.marker) && currentseries.marker.visible) || (!ej.util.isNullOrUndefined(currentseries.marker) && currentseries.marker.dataLabel.visible))
                    point.symbolLocation = { X: ((x1 + x2) / 2), Y: ((y1 + y2) / 2) };
                            
                this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

            }

                // drawType 'Area' calculation
            else {
                point1 = ej.EjSvgRender.utils.TransformToVisible(currentseries, points[i].xValue, points[i].y, this.chartObj);
                if (i == 0) {
                    segment.append("M" + " " + (point1.X) + " " + (point1.Y) + " ");
                }
                    // when isClosed is enabled
                else if (i == points.length - 1) {
                    if(!currentseries.isClosed){
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "L" + " " + (centerX) + " " + (centerY)  );
                    }
                    else
                        segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " " + "z");
                }
                    
                else {
                    segment.append("L" + " " + (point1.X) + " " + (point1.Y) + " ");
                }
            }
            if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() != 'column') {
                var bounds = { X: point1.X - size.width / 2, Y: point1.Y - size.height / 2, Width: size.width, Height: size.height };
                ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, points[i], i);
            }
			if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() == 'column' && this.chartObj.model.enableCanvasRendering)
                ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, null, null); 
        }

        if (!currentseries.regionAdded && currentseries.drawType.toLowerCase() == 'column' && !this.chartObj.model.enableCanvasRendering)
            ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, null, null);
        return segment.toString();
    },
    _pieVisiblePoints: function (currentseries) {
        var visiblePoints = [];
        var visibleLabels = [];
        var point;
        for (var i = 0; i < currentseries.points.length; i++) {
            point = currentseries.points[i];
			point.visible = (point.isEmpty) ? false : point.visible;
            if (ej.util.isNullOrUndefined(point.visible))
                point.visible = true;
            if (point.y < 0) {
                point.YValues[0] = Math.abs(point.YValues[0])
            }
            if (point.visible == true && !ej.util.isNullOrUndefined(point.y)) {
                point.visiblePointIndex = i;
                visiblePoints.push(point);
                visibleLabels.push(point.text ? point.text : point.y);
            }
        }
        currentseries.visiblePoints = visiblePoints;
        currentseries.visibleLabels = visibleLabels;
        return visiblePoints;
    },
    getMinMaxValue: function (point1, point2, degree) {
        var minX = Math.min(point1.x, point2.x);
        var minY = Math.min(point1.y, point2.y);
        var maxX = Math.max(point1.x, point2.x);
        var maxY = Math.max(point1.y, point2.y);
        var pointValue;
        switch (degree) {
            case 0:
            case 360:
                pointValue = maxY;
                break;
            case 90:
                pointValue = minX;
                break;
            case 180:
                pointValue = minY;
                break;
            case 270:
                pointValue = maxX;
                break;
        }
        return pointValue;
    },
    pieDoughnutCenter:function(currentseries)
    {
        var oldCenterX = this.chartObj.model.centerX;
        var oldCenterY = this.chartObj.model.centerY;
        var startAngle = currentseries.startAngle;
        var endAngle = currentseries.endAngle;
        var regions = [-630,-540,-450,-360,-270,-180,-90,0, 90, 180, 270,360,450,540,630];
        var region = [];
        
        if (currentseries.startAngle < currentseries.endAngle) {
            for (i = 0; i < regions.length;i++) {
                if (regions[i] > startAngle && regions[i] < endAngle)
                    region[region.length] = (regions[i] % 360) < 0 ? (regions[i] % 360) + 360 : (regions[i] % 360);
            }
            }
        else {
            for (i = 0; i < regions.length; i++) {
                if (regions[i] < startAngle && regions[i] > endAngle)
                    region[region.length] = (regions[i] % 360) < 0 ? (regions[i] % 360) + 360 : (regions[i] % 360);
            }
        }
        var radius = this.chartObj.model.Radius;
        var startRadian = 2 * Math.PI * (startAngle - 90) / 360;
        var endRadian = 2 * Math.PI * (endAngle - 90) / 360;
        var centerPoint = { x: this.chartObj.model.centerX, y: this.chartObj.model.centerY };
        var startPoint = { x: this.chartObj.model.centerX + radius * Math.cos(startRadian), y: this.chartObj.model.centerY + radius * Math.sin(startRadian) }
        var endPoint = { x: this.chartObj.model.centerX + radius * Math.cos(endRadian), y: this.chartObj.model.centerY + radius * Math.sin(endRadian) }
        switch (region.length) {
            case 0:
                var longX = Math.abs(centerPoint.x - startPoint.x) > Math.abs(centerPoint.x - endPoint.x) ? startPoint.x : endPoint.x;
                var longY = Math.abs(centerPoint.y - startPoint.y) > Math.abs(centerPoint.y - endPoint.y) ? startPoint.y : endPoint.y;
                var midPoint = { x: (centerPoint.x + longX) / 2, y: (centerPoint.y + longY) / 2 }
                this.chartObj.model.centerX = this.chartObj.model.centerX + (this.chartObj.model.centerX - midPoint.x);
                this.chartObj.model.centerY = this.chartObj.model.centerY + (this.chartObj.model.centerY - midPoint.y);
                break;
            case 1:
                var maxRadian = 2 * Math.PI * (region[0] - 90) / 360;
                var maxPoint = { x: this.chartObj.model.centerX + radius * Math.cos(maxRadian), y: this.chartObj.model.centerY + radius * Math.sin(maxRadian) }
                switch (region[0]) {
                    case 0:
                    case 360:
                        point1 = { x: startPoint.x, y: maxPoint.y };
                        point2 = { x: endPoint.x, y: centerPoint.y };
                        break;
                    case 90:
                        point1 = { x: centerPoint.x, y: endPoint.y };
                        point2 = { x: maxPoint.x, y: startPoint.y };
                        break;
                    case 180:
                        point1 = { x: endPoint.x, y: centerPoint.y };
                        point2 = { x: startPoint.x, y: maxPoint.y };
                        break;
                    case 270:
                        point1 = { x: maxPoint.x, y: startPoint.y };
                        point2 = { x: centerPoint.x, y: endPoint.y };
                        break;
                }
                var midPoint = { x: (point1.x + point2.x) / 2, y: (point1.y + point2.y) / 2 }
                this.chartObj.model.centerX = this.chartObj.model.centerX + ((this.chartObj.model.centerX - midPoint.x) >= radius ? 0 : (this.chartObj.model.centerX - midPoint.x));
                this.chartObj.model.centerY = this.chartObj.model.centerY + ((this.chartObj.model.centerY - midPoint.y) >= radius ? 0 : (this.chartObj.model.centerY - midPoint.y));
                break;
            case 2:
                var minRadian = 2 * Math.PI * (region[0] - 90) / 360;
                var maxRadian = 2 * Math.PI * (region[1] - 90) / 360;
                var maxPoint = { x: this.chartObj.model.centerX + radius * Math.cos(maxRadian), y: this.chartObj.model.centerY + radius * Math.sin(maxRadian) }
                var minPoint = { x: this.chartObj.model.centerX + radius * Math.cos(minRadian), y: this.chartObj.model.centerY + radius * Math.sin(minRadian) }
                var point1;
                var point2
                if (region[0] == 90 && region[1] == 180 || region[0] == 270 && region[1] == 0)
                    point1 = { x: minPoint.x, y: maxPoint.y }
                else 
                    point1 = { x: maxPoint.x, y: minPoint.y }
                if (region[0] == 90 || region[0] == 270)
                    point2 = { x: this.getMinMaxValue(startPoint, endPoint, region[0]), y: this.getMinMaxValue(startPoint, endPoint, region[1]) }
                else
                    point2 = { x: this.getMinMaxValue(startPoint, endPoint, region[1]), y: this.getMinMaxValue(startPoint, endPoint, region[0]) }
                var midPoint = { x: Math.abs(point1.x - point2.x) / 2 >= radius ? 0 : (point1.x + point2.x) / 2, y: Math.abs(point1.y - point2.y) / 2 >= radius ? 0 : (point1.y + point2.y) / 2 }
                this.chartObj.model.centerX = this.chartObj.model.centerX + (midPoint.x == 0 ? 0 : (this.chartObj.model.centerX - midPoint.x) >= radius ? 0 : (this.chartObj.model.centerX - midPoint.x));
                this.chartObj.model.centerY = this.chartObj.model.centerY + (midPoint.y == 0 ? 0 : (this.chartObj.model.centerY - midPoint.y) >= radius ? 0 : (this.chartObj.model.centerY - midPoint.y));
                break;
        }
        for (var i = 0; i < currentseries.points.length; i++) {
            this.chartObj.model.startX[i] =this.chartObj.model.startX[i]-( oldCenterX - this.chartObj.model.centerX);
            this.chartObj.model.startY[i] =this.chartObj.model.startY[i]-( oldCenterY - this.chartObj.model.centerY);
        }
    },
    calculatingSliceAngle: function (currentseries) {
        var chart = this.chartObj,
            chartModel = chart.model,
            legend = chartModel.legend,
            legendPosition = legend.position.toLowerCase(),
            legendActualBounds = chartModel.LegendActualBounds,
            visiblePoints = this._pieVisiblePoints(currentseries),
            legXSpace = 0, totalDegree,
            legYSpace = 0,
            legendBorder = legend.border.width,
            border = chartModel.border.width,
            point, margin = chartModel.margin,
            elementSpacing = chartModel.elementSpacing,
            itemCurrentXPos = 0;
        chartModel.arcData = [];
        currentseries.rightsidePoints = [];
        currentseries.leftsidePoints = [];
        currentseries.labels = [];
        chartModel.chartRegions = [];
        chartModel.sumofYValues = 0;
        chartModel.midPoint = 0;
        chartModel.centerX = 0, chartModel.centerY = 0;
        chartModel.startX = [], chartModel.startY = [];
        chartModel.Radius = 0;

        for (j = 0; j < visiblePoints.length; j++) {
            chartModel.sumofYValues += visiblePoints[j].YValues[0];
        }
        if (currentseries.endAngle != null)
            totalDegree = (currentseries.endAngle - currentseries.startAngle);
        else
            totalDegree = 360;
        totalDegree = (totalDegree != 360 && totalDegree != -360) ? totalDegree % 360 : totalDegree;
        if (currentseries.startAngle) {
            currentseries.startAngle = currentseries.startAngle % 360;
            currentseries.endAngle = currentseries.startAngle + totalDegree;
            itemCurrentXPos = (currentseries.startAngle) ? ((chartModel.sumofYValues / 360) * currentseries.startAngle) : 0;
            itemCurrentXPos = itemCurrentXPos / chartModel.sumofYValues;
            chartModel.itemCurrentXPos = itemCurrentXPos;
        }
        else
            currentseries.endAngle = totalDegree;
        if (legend.visible && legendPosition != "custom") {
            if (legendPosition == "right" || legendPosition == "left")
                legXSpace += legendActualBounds.Width + (legendBorder * 2) + elementSpacing;
            else
                legYSpace += legendActualBounds.Height + (legendBorder * 2) + elementSpacing;

        }

        var yOffset = ((chartModel.title.text) ? chartModel._titleLocation.size.height + chartModel.elementSpacing : 0);

        var actualWidth = $(chart.svgObject).width() - (legXSpace + margin.left + margin.right + border * 2);
        var actualHeight = $(chart.svgObject).height() - (legYSpace + yOffset + margin.top + margin.bottom + border * 2);

        chartModel.centerX = (actualWidth * 0.5) + margin.left + ((legend.visible && legendPosition === "left") ? legXSpace : 0);
        chartModel.centerY = (actualHeight * 0.5) + margin.top + yOffset + ((legend.visible && legendPosition === "top") ? legYSpace : 0);
        var endAngle;
        totalDegree = currentseries.endAngle - currentseries.startAngle;
        var pieFactor = totalDegree / 180;
        for (var l = 0; l < currentseries.visiblePoints.length; l++) {
            point = currentseries.visiblePoints[l];
            if (point.visible) {
                var pointIndex = l;
                if (l == 0)
                    point.startAngle = 2 * Math.PI * itemCurrentXPos;
                else
                    point.startAngle = endAngle;
                endAngle = point.endAngle = pieFactor * Math.PI * (point.YValues[0] / chartModel.sumofYValues) + point.startAngle;
                    endAngle = point.endAngle = (isNaN(endAngle)) ? point.startAngle : endAngle;
                    point.start = point.startAngle;
                    point.end = point.endAngle;
            point.midAngle = (point.endAngle + point.startAngle) / 2;
            point.pointIndex = pointIndex;
            var chartStartingAngle = -.5 * Math.PI;

                point.radian = (point.midAngle) % (2 * Math.PI)

                if (currentseries.labelPosition.toLowerCase() == "outsideextended") {

                    if (point.radian < Math.PI)
                        currentseries.rightsidePoints.push(point);
                    else
                        currentseries.leftsidePoints.push(point);
                }
                itemCurrentXPos += point.YValues[0] / chartModel.sumofYValues;

                var midAngle = point.midAngle + chartStartingAngle;

                if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !chart.vmlRendering) {

                    chartModel.startX[pointIndex] = chartModel.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
                    chartModel.startY[pointIndex] = chartModel.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
                }
                else {
                    chartModel.startX[pointIndex] = chartModel.centerX;
                    chartModel.startY[pointIndex] = chartModel.centerY;
                }
            }

        }

        if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
            currentseries.rightsidePoints = ej.DataManager(currentseries.rightsidePoints, ej.Query().sortBy("radian")).executeLocal();
            currentseries.leftsidePoints = ej.DataManager(currentseries.leftsidePoints, ej.Query().sortBy("radian")).executeLocal();
        }
        if (currentseries.marker.dataLabel.template) {
            if (currentseries.labelPosition.toLowerCase() == "outsideextended") {
                for (var i = 0; i < currentseries.rightsidePoints.length; i++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.rightsidePoints[i], currentseries.rightsidePoints[i].pointIndex, currentseries, false, chart);
                }
                for (var k = 0; k < currentseries.leftsidePoints.length; k++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.leftsidePoints[k], currentseries.leftsidePoints[k].pointIndex, currentseries, true, chart);
                }

            }
            else {
                for (var j = 0; j < currentseries.visiblePoints.length; j++) {
                    ej.EjSvgRender.utils._getSeriesTemplateSize(currentseries.visiblePoints[j], currentseries.visiblePoints[j].visiblePointIndex, currentseries, true, chart);
                }
            }
            
        }
        else {
            ej.EjSvgRender.utils._getSeriesMaxLabel(currentseries);
        }

      
        currentseries.finalSize= { width: actualWidth, height: actualHeight };
        return currentseries.finalSize;
    },
	 _drawHiloPath: function (series, style, interior, direction, i, pointbounds) {

	     var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);

        var options = {
            'name': series.type,
            'id': this.chartObj.svgObject.id + "_Series" + seriesIndex + "_Point" + i,
            'fill': "none",
            'stroke-dasharray': series.dashArray,
            'stroke-width': style.borderWidth,
            'stroke': interior,
            'stroke-linecap': series.lineCap,
            'stroke-linejoin': series.lineJoin,
            'opacity': series.opacity,
            'd': direction
        };
        this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
        var valWidth, valHeight;
        var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.chartObj.model.requireInvertedAxes);
        x = (Math.min(pointbounds.point2.X, pointbounds.point1.X) +trans.x) - (style.borderWidth);
        y = (Math.min(pointbounds.point2.Y, pointbounds.point1.Y) +trans.y);
        if (pointbounds.point1.Y != pointbounds.point2.Y) {
            valWidth = style.borderWidth + (style.borderWidth / 2);
              valHeight = Math.abs(pointbounds.point1.Y - pointbounds.point2.Y);
        }
	     else {
            valWidth = Math.abs(pointbounds.point2.X - pointbounds.point1.X);
            valHeight = style.borderWidth + (style.borderWidth / 2); 
        }
        var bounds = { X: x, Y: y, Width: valWidth, Height: valHeight };
        ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);
      
	 },
	 isIntersec: function (other, currBounds) {
	     if (currBounds.Bottom < other.Top || currBounds.Right < other.Left
               || currBounds.Top > other.Bottom || currBounds.Left > other.Right) {
	         return false;
	     }
	      return true;
	 },
	 IntersectWith: function (sender, point, currBounds) {
	     var chartObj = this;
	     var intersect = false;
	     var bounds;
	     if (point.radian > Math.PI && (sender.model.bounds["left"])) {
	         bounds = (sender.model.bounds["left"]);
	         if (bounds.length > 0) {
	             for (var i = point.index + 1; i < bounds.length; i++) {
	                 var other = bounds[i];
	                 if (other && this.isIntersec(other, currBounds)) {
	                     return true;
	                 }
	             }
	         }

	     }
	     else {
	         bounds = (sender.model.bounds["right"]) ? sender.model.bounds["right"] : sender.model.bounds;
	         var chartObj = this;
   	         if (bounds.length > 0) {
   	               for (var i = point.index - 1; i >= 0; i--) {
   	                 var other = bounds[i];
   	                 if (other && this.isIntersec(other, currBounds)) {
   	                     return true;
   	                 }
   	             }
   	         }
	     }
	     return intersect;
	     
	 },
   
   
	 _calculateArcData: function (sender, pointIndex, point, series) {
	     var visiblePoints = series.visiblePoints;
	     var chartObj = sender;
	     var chartStartAngle = -.5 * Math.PI;
	     var currBounds;
	     var startAngle = point.startAngle + chartStartAngle;
	     var endAngle = point.endAngle + chartStartAngle - 0.000001;
	     var totalDegree = series.endAngle - series.startAngle;
	     var longArc = endAngle - startAngle < Math.PI ? 0 : 1;
	     var midAngle = (startAngle + endAngle) / 2;
		  if (ej.util.isNullOrUndefined(point.currentMidAngle)) {
	         point.currentMidAngle = (startAngle + endAngle) / 2;
	     }
	     var direction;
	     var startX;
	     var startY
	     var clockWise=totalDegree > 0?1:0;
	     var longArc = clockWise ? endAngle - startAngle < Math.PI ? 0 : 1 : endAngle - startAngle > -1 * Math.PI ? 0 : 1;
	    
	     if ((pointIndex == series.explodeIndex || series.explodeAll) && !chartObj.vmlRendering) {
           
	         startX = chartObj.model.centerX + Math.cos(midAngle) * series.explodeOffset;
	         startY = chartObj.model.centerY + Math.sin(midAngle) * series.explodeOffset;
	     }
	     else {
	         startX = chartObj.model.centerX;
	         startY = chartObj.model.centerY;
	     }


	     var x1 = startX + chartObj.model.Radius * Math.cos(startAngle);
	     var y1 = startY + chartObj.model.Radius * Math.sin(startAngle);

	     var x2 = startX + chartObj.model.Radius * Math.cos(endAngle);
	     var y2 = startY + chartObj.model.Radius * Math.sin(endAngle);
	    
	     if (series.type.toLowerCase() == "doughnut") {
	         var dStartX = startX + chartObj.model.dRadius * Math.cos(startAngle);
	         var dStartY = startY + chartObj.model.dRadius * Math.sin(startAngle);

	         var dEndX = startX + chartObj.model.dRadius * Math.cos(endAngle);
	         var dEndY = startY + chartObj.model.dRadius * Math.sin(endAngle);
	         var dClockWise = clockWise ? 0 : 1;
	         if (Math.round(point.endAngle - point.startAngle) == 6)
	         {
	             dEndX = dEndX - 0.01;
	             x2 = x2 - 0.01;
	         }
	         direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + chartObj.model.Radius + " " + chartObj.model.Radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2 + " " + "L" + " " + dEndX + " " + dEndY + " " + "A" + " " + chartObj.model.dRadius + " " + chartObj.model.dRadius + " " + "1" + " " + longArc + " " + dClockWise + " " + dStartX + " " + dStartY + " " + "z";
	     }

	     else{
	         if ((point.endAngle-point.startAngle).toFixed(4) == (2*Math.PI).toFixed(4)){
				 var centerx = startX;
	             var centery = startY;
	             direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + chartObj.model.Radius + " " + chartObj.model.Radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2;
            }
			else
	             direction = "M" + " " + x1 + " " + y1 + " " + "A" + " " + chartObj.model.Radius + " " + chartObj.model.Radius + " " + "0" + " " + longArc + " " + clockWise + " " + x2 + " " + y2 + " " + "L" + " " + startX + " " + startY + " " + "z";
	     }

        // display label for pie and doughnut chart
	     

        return { Direction: direction, centerX: centerx, centerY: centery };

    },

    getXCordinate: function (x, radius, angle) {
        var x1 = x + radius * (Math.cos(angle));
        return x1;
    },

    getYCordinate: function (y, radius, angle) {
        var y1 = y + radius * (Math.sin(angle));
        return y1;
    },
 getDoubleRange: function (start, end) {
        var mstart;
        var mend;
        if (start > end) {
            mstart = end;
            mend = start;
        }
        else {
            mstart = start;
            mend = end;
        }

        var mdelta = mend - mstart;
        var mmedian = (mstart + mend) / 2;
        var misEmpty = isNaN(mstart) || isNaN(mend);

        return { Start: mstart, End: mend, Delta: mdelta, Median: mmedian, IsEmpty: misEmpty };

    },
    isDependentSeries: function (series) {
        var type = series.type.toLowerCase();
        var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
       if((!requireInvertedAxes) || (requireInvertedAxes && series.isTransposed))
           return ((type.indexOf("bar") != -1 && !series._isTransposed) || (type.indexOf("column") != -1) || (type.indexOf("hilo") != -1) || (type.indexOf("candle") != -1)) ? true : false;
        else
            return (type.indexOf("bar") != -1) ? true : false;
    },

    getPointXYOrgin: function (x, y, series, sender) {
	     var requireInvertedAxes = sender.model.requireInvertedAxes;
		 var xvalue;
		 var yvalue;
        if ((series.isTransposed && requireInvertedAxes) || series._isTransposed) {
            xvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x)) * (series.xAxis.height);
            yvalue = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.width);
            return { X: yvalue, Y: xvalue };
        }
        else {
             xvalue = ej.EjSvgRender.utils._valueToCoefficient(series.xAxis, x) * (series.xAxis.width);
             yvalue = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, y)) * (series.yAxis.height);
            return { X: xvalue, Y: yvalue };
        }
    },
    getRectangle: function (x1, y1, x2, y2, series, sender) {
        var pt1 = this.getPointXYOrgin(x1, y1, series, sender);
        var pt2 = this.getPointXYOrgin(x2, y2, series, sender);
        return ej.EjSvgRender.utils._correctRect(pt1.X, pt1.Y, pt2.X, pt2.Y);
    },
    calculateSides: function (point, sidebysideinfo) {
        var x1 = point.xValue + sidebysideinfo.Start;
        var x2 = point.xValue + sidebysideinfo.End;
        return { x1: x1, x2: x2 };
    },
    _getSeriesPosition: function (currentSeries) {
     // calculation for position of series in radar and polar.
        if (ej.util.isNullOrUndefined(currentSeries.position) || this.chartObj.model.legendCollapsed) {
            var stacked;
            var stackingposition
            var all=0;
            var seriesCollection=[];
            for (var i = 0; i < this.chartObj.model._visibleSeries.length; i++) {
                var series=this.chartObj.model._visibleSeries[i];
                if (series.visibility.toLowerCase() === 'visible' && series.drawType.toLowerCase() == 'column') {
                    seriesCollection.push(series);
                }
            }
            for (var j = 0; j < seriesCollection.length; j++) {
                var series = seriesCollection[j];
			    if (series._xAxisName == this.chartObj.model._axes[0].name && series._yAxisName == this.chartObj.model._axes[1].name) {
                    if (series.isStacking) {
                        if (!stacked) {
                            stackingposition = all;
                            all++;
                            stacked = true;
                        }
                        series.position = stackingposition;
                    }
                    else {
                        series.position = all;
                        all++;
                      }
				}	  
            }
            for (var k = 0; k < seriesCollection.length; k++) 
                seriesCollection[k].all = all;
            
        }
        return { all: currentSeries.all, pos: currentSeries.position };
    },
     getSideBySidePositions: function (currentSeries) {

         var chart = this;
         var stackingGroup = [];
         var model = chart.chartObj.model;
         var visibleSeriesLength=model._visibleSeries.length;
         if (ej.util.isNullOrUndefined(currentSeries.position) || model.legendCollapsed) {
             var hAxes = (model.requireInvertedAxes) ? model.vAxes : model.hAxes;
             var vAxes = (model.requireInvertedAxes) ? model.hAxes : model.vAxes;

             $.each(hAxes, function (index, hAxis) {
                 var all = 0;
                 var seriesCollection = [];
                 for (j = 0; j <visibleSeriesLength; j++) {
                     var series = model._visibleSeries[j];
                     if (series.visibility.toLowerCase() === 'visible') {
                         if (chart.isDependentSeries(series)) {
                             if (hAxis.name == series._xAxisName) {
                                 seriesCollection.push(series);
                             }
                         }
                     }
                 }
				  var seriesCollectionLength=seriesCollection.length;
                 var vAxeseries = [];
                 for (var j = 0; j < seriesCollectionLength; j++) {
                     var index = (model.requireInvertedAxes) ? seriesCollection[j].yAxis.columnIndex : seriesCollection[j].yAxis.rowIndex;
                     if (!vAxeseries[index]) {
                         vAxeseries[index] = [];
                         seriesCollection[j].yAxis.position = null;
                     }
                     vAxeseries[index].push(seriesCollection[j]);
                 }

                 for (var i = 0; i < vAxeseries.length; i++) {
                     if (!ej.util.isNullOrUndefined(vAxeseries[i])) {
                         vAxeseries[i].all = 0;
                         for (var j = 0; j < vAxeseries[i].length; j++) {
                             series = vAxeseries[i][j];
                             if (series.type.toLowerCase().indexOf("stacking") != -1) {
                                 if (series.stackingGroup) {
                                     if (ej.util.isNullOrUndefined(stackingGroup[series.stackingGroup])) {
                                         vAxeseries[i].all++;
                                         series.position = vAxeseries[i].all;
                                         stackingGroup[series.stackingGroup] = vAxeseries[i].all;
                                     }
                                     else
                                         series.position = stackingGroup[series.stackingGroup];
                                 }
                                 else {
                                     if (ej.util.isNullOrUndefined(series.yAxis.position) || model.legendCollapsed) {
                                         vAxeseries[i].all++;
                                         series.position = vAxeseries[i].all;
                                         series.yAxis.position = vAxeseries[i].all;
										 model.legendCollapsed=false;
                                     }
                                     else
                                         series.position = series.yAxis.position;
                                 }
                             }
                             else {
                                 vAxeseries[i].all++;
                                 series.position = vAxeseries[i].all;
                             }
                         }
                     }
                 }

                 for (var k = 0; k < seriesCollectionLength; k++) {
                     var index = (model.requireInvertedAxes) ? seriesCollection[k].yAxis.columnIndex : seriesCollection[k].yAxis.rowIndex;
                     seriesCollection[k].all = vAxeseries[index].all;
                 }

             });

         }
         return { all: currentSeries.all, pos: currentSeries.position };
     },
	
    getSideBySideInfo: function (series) {
        var pos;
        var all;

        var data = this.getSideBySidePositions(series);
        pos = data.pos;
        all = data.all;
        var sideBySideSeriesPlacement = this.chartObj.model._sideBySideSeriesPlacement;
        var seriesSpacing = 0;
        var pointSpacing = 0.3 - 0;
        if (series.xAxis.m_minPointsDelta == undefined)
            series.xAxis.m_minPointsDelta = ej.EjSvgRender.utils.getMinPointsDelta(series.xAxis, this.chartObj);
        var width = series.xAxis.m_minPointsDelta * (1 - pointSpacing);
        if (!sideBySideSeriesPlacement)
            return this.getDoubleRange(-width / 2, width / 2);
        var div = 1 / all;
        var loc = 0.5 - div * (pos - 1);

        var range = this.getDoubleRange(-loc, -loc + div);

        // multiplying with width  and scaling
        if (!range.IsEmpty) {
            range = this.getDoubleRange(range.Start * width, range.End * width);
            var radius = 0.5 * (1 - seriesSpacing) * range.Delta;
            var median = range.Median;
            range = this.getDoubleRange(median - radius, median + radius);
        }

        return range;

    },



    draw: function (chart, options) {

        this.chartObj = chart;
        
        //DrawLineGraph
        var lDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setLineSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);

        var firstPoint = null;
        var secondPoint;
        var firstIndex = -1;
        for (var i = 0; i < visiblePoints.length; i++) {
            secondPoint = visiblePoints[i];
            if (secondPoint.visible) {
                if (firstPoint != null) {
                    if (visiblePoints.length > firstIndex + 1) {
                        var nextpoint = visiblePoints[firstIndex + 1];
                        var point1 = ej.EjSvgRender.utils._getPoint(firstPoint,currentseries);
                        var point2 = ej.EjSvgRender.utils._getPoint(nextpoint,currentseries);
                        var chartObj = this.chartObj;
                        sb.append("M" + " " + (point1.X + chartObj.canvasX) + " " + ((point1.Y + chartObj.canvasY)) + " " + "L" + " " + (point2.X + chartObj.canvasX) + " " + ((point2.Y + chartObj.canvasY)) + " ");

                    }

                }
                firstPoint = secondPoint;
                firstIndex = i;

            }
            else {
                firstPoint = null;
            }

        }

        lDirection = sb.toString();

        this._drawLinePath(currentseries, style, lDirection);
        
        this.chartObj.svgRenderer.append(this.gSeriesGroupEle,this.chartObj.gSeriesEle);
       
        

    },
    _doLineSymbol: function(element, sec, val, series, invertedAxes) {
        var beginTime = parseInt(val * sec);
        var chartObj=this.chartObj;
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        var centerX = box.x + (box.width / 2);
        var centerY = box.y + (box.height / 2);
        var scaleVal;
        var $ele = $(element);
        var rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
        rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " " ;
        chartObj.svgRenderer._setAttr($ele, { "transform": "scale(0)" });
        $ele.delay(beginTime).animate(
            {
                scale: 1
            },
            {
                duration: 200,

                step: function(now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + rotate });

                }
            }
        );
    },
    animateSymbol: function(element,delayInterval,series,invertedAxes) {
        var chartObj=this.chartObj;
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        var centerX = box.x + (box.width / 2);
        var centerY = box.y + (box.height / 2);
        var scaleVal;
        var $ele = $(element);
        var rotate = chartObj.svgRenderer._getAttrVal($ele, "transform");
        rotate = !ej.util.isNullOrUndefined(rotate) ? rotate : " ";
        $ele.delay(delayInterval).each(function () {this.scale = 0.5; }).animate(
            {
                scale: 1
            },
            {
                duration: 500,

                step: function(now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($ele, { "transform": "translate(" + centerX + " " + centerY + ") scale(" + scaleVal + ") translate(" + (-centerX) + " " + (-centerY) + ")" + (rotate) });
                }
            }
        );
    },
    animateRect: function (element,series,invertedAxes) {
        var chartObj = this.chartObj;        
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        var centerX, centerY,scale1, scale2;
        var $element = $(element);
        if ((series.isTransposed && invertedAxes) || invertedAxes) {
            if (chartObj.svgRenderer._getAttrVal($element, "plot") === "negative") {
                centerX = box.x + box.width;
                centerY = box.y + box.height;
            } else {               
                centerX = box.x;
                centerY = box.y;
			}
            scale1 = ") scale(";
            scale2 = ",1) translate(";

        }
        else {
            if (chartObj.svgRenderer._getAttrVal($element, "plot") === "negative") {
                centerX = box.x;
                centerY = box.y;
			}
			else {
                 centerX = (box.x +box.width);
                 centerY = (box.y + box.height);
             }
            scale1 = ") scale(1,";
            scale2 = ") translate(";

             
        }
        var scaleVal;
        $element.animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now; 
                    chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + scale1 + scaleVal + scale2 + (-centerX) + " " + (-centerY) + ")" });
                }
            });

    },
    animateStackingRect: function (element, series, invertedAxes) {
        var chartObj = this.chartObj;

       var centerX, centerY, scale1, scale2;
        var $element = $(element);
       var boxX,boxY;
        var box = ej.EjSvgRender.utils._getBoundingClientRect(element, chartObj, series, invertedAxes);
        if ((series.isTransposed && invertedAxes) || invertedAxes) {
            boxX = (ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.width);
            centerX = boxX;
            centerY = box.y;
			scale1 = ") scale(";
            scale2 = ",1) translate(";
        }
        else {
            boxY = (1 - ej.EjSvgRender.utils._valueToCoefficient(series.yAxis, 0)) * (series.yAxis.height);
            centerX = box.x;
            centerY = boxY;
            scale1 = ") scale(1,";
            scale2 = ") translate(";
        }
        
        var scaleVal;
        $element.animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    chartObj.svgRenderer._setAttr($element, { "transform": "translate(" + centerX + " " + centerY + scale1 + scaleVal + scale2 + (-centerX) + " " + (-centerY) + ")" });
                }
            });
    },
    _doLineAnimation: function (series) {
        var chartObj=this.chartObj;
		var requireInvertedAxes = chartObj.model.requireInvertedAxes;
		var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
		var gSeriesGroupId = !ej.util.isNullOrUndefined(this.gSeriesGroupEle) ? this.gSeriesGroupEle.id : this.chartObj.gSeriesEle.children[seriesIndex].id;
        var clipRect = $(chartObj.gSeriesEle).find("#" + gSeriesGroupId + "_ClipRect").children();
		if (requireInvertedAxes) {
            var height = parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gSeriesEle).find("#" + gSeriesGroupId + "_ClipRect").children(),"height"));
            chartObj.svgRenderer._setAttr($(clipRect), { "y": height });
            $(clipRect).animate(
                {
                    y: parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gSeriesEle).find("#" + gSeriesGroupId + "_ClipRect").children(),"y")),
                    height:parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gSeriesEle).find("#" + gSeriesGroupId + "_ClipRect").children(),"height")) 


                },
                {
                    duration: 2000,

                    step: function (now) {
                        chartObj.svgRenderer._setAttr($(clipRect), { "y": height - now });
                        chartObj.svgRenderer._setAttr($(clipRect), { "height": now});

                    }
                });
        }
		else
		{
          $(clipRect).animate(
            { 
			  width: parseFloat(chartObj.svgRenderer._getAttrVal($(chartObj.gSeriesEle).find("#" + gSeriesGroupId + "_ClipRect").children(),"width")) 
			},
            {
                duration: 2000,

                step: function (now) {
                    chartObj.svgRenderer._setAttr($(clipRect), { "width": now });
                }
            });
		}
        var elements = $(chartObj.gSymbolGroupEle)[seriesIndex];
		elements = elements.childNodes;
		var dataLabels = $(chartObj.gDataLabelEle)[seriesIndex];
		dataLabels = dataLabels.childNodes;
		var connectorLines = $(chartObj.gConnectorEle)[seriesIndex];
		connectorLines = connectorLines.childNodes;
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
			if (element.tagName != "defs")
              this._doLineSymbol(element, secondsPerPoint, i, series, requireInvertedAxes);
        }  
        
		var j, k, dataLabel, connectorLine;
		for (j = 0; j< dataLabels.length; j++){
		  dataLabel = dataLabels[j];
		  if (dataLabel.tagName != "defs")
		     this._doLineSymbol(dataLabel, secondsPerPoint, i, series, requireInvertedAxes);
		}
        
        for (k = 0; k< connectorLines.length; k++){
		  connectorLine = connectorLines[k];
		  if (connectorLine.tagName != "defs")
		     this._doLineSymbol(connectorLine, secondsPerPoint, i, series, requireInvertedAxes);
		}		
        

    },
    doAnimation: function (series) {

        this._doLineAnimation(series);
    },
    
    _drawLinePath: function (series, style, direction) {
        var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
		var trans;
        var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var translate = null;
        if (this.chartObj.model.AreaType == "cartesianaxes") {
            trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
            translate = 'translate(' + trans.x + ',' + trans.y + ')'
        }
        
        var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate };
        this.chartObj.gSeriesGroupEle =  this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
        if (direction != "" && direction.indexOf("NaN") == -1) {
            var options = {
                'name': series.type,
                'id': this.chartObj.svgObject.id + "_Series" + seriesIndex,
                'fill': 'none',
                'stroke-dasharray': series.dashArray,
                'stroke-width': series.width,
                'stroke': style.SeriesInterior,
                'stroke-linecap': series.lineCap,
                'stroke-linejoin': series.lineJoin,
                'opacity': series.opacity,
                'd': direction
            };

            this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);
        }


    },
   
   
    
    

    drawDataLabel: function (series, index, point, x, y, chartRegionIndex) {
	       // method for data label symbols - cartesianaxes
        point.marker = point.marker ? point.marker : {};
		var labelFormat = series.yAxis.labelFormat,
			pointText = (point.text) ? point.text : (labelFormat) ? labelFormat.replace('{value}', point.y) : point.y,
			chart = this.chartObj,
			chartModel = chart.model,
			seriesIndex = $.inArray(series, chartModel._visibleSeries),
			textAlign, element,
			textBaseline, symbolName,
			marker = $.extend(true, {}, series.marker, point.marker),
			dataLabel = marker.dataLabel,
			dataLabelFont = dataLabel.font,
			textPosition = dataLabel.textPosition.toLowerCase(),
			type = series.type.toLowerCase(),
			size = ej.util.isNullOrUndefined(dataLabelFont.size) ? "11px" : dataLabelFont.size,
			fontStyle = ej.util.isNullOrUndefined(dataLabelFont.fontStyle) ? "Normal" : dataLabelFont.fontStyle,
			fontFamily = ej.util.isNullOrUndefined(dataLabelFont.fontFamily) ? "Segoe UI" : dataLabelFont.fontFamily,
			labelfont = { size: size, fontStyle: fontStyle, fontFamily: fontFamily },
			textOffset = ej.EjSvgRender.utils._measureText(pointText, null, labelfont),
			position = this.textPosition(series, seriesIndex, point, textOffset, type, x, y, chartRegionIndex, index),
			commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { text: pointText, location: { x: position.x + chart.canvasX, y: position.y + chart.canvasY }, seriesIndex: seriesIndex, pointIndex: index };
        chart._trigger("displayTextRendering", commonEventArgs);
        var dataLabelOffset = (marker.dataLabel.offset.y==undefined) ? marker.dataLabel.offset : marker.dataLabel.offset.y;
        var xval = marker.dataLabel.offset.x;
        if (textPosition == 'bottom')
              commonEventArgs.data.location.y = commonEventArgs.data.location.y + dataLabelOffset;
        else
              commonEventArgs.data.location.y = commonEventArgs.data.location.y - dataLabelOffset;
        if (xval)                           
              commonEventArgs.data.location.x = commonEventArgs.data.location.x + xval;
                          
        if (!commonEventArgs.cancel)
        {
            var options = {
                'id': chart.svgObject.id + '_SeriesText' + index + seriesIndex,
                'x': commonEventArgs.data.location.x,
                'y': commonEventArgs.data.location.y,
                'fill': dataLabelFont.color,
                'font-size': dataLabelFont.size,
                'font-family': dataLabelFont.fontFamily,
                'font-style': dataLabelFont.fontStyle,
                'font-weight': dataLabelFont.fontWeight,
                'opacity': dataLabelFont.opacity,
                'text-anchor': 'middle'
            };
            var margin = dataLabel.margin,
                width = textOffset.width + margin.left + margin.right,
                height = textOffset.height + margin.top + margin.bottom,
                location = { X: commonEventArgs.data.location.x, Y: commonEventArgs.data.location.y };

            if (ej.util.isNullOrUndefined(dataLabel.template)) {
				if (dataLabel.shape)
                    element = dataLabel.shape;
                else
                    element = "None";
                $.each(chartModel.symbolShape, function (name) {
                    if (element.toLowerCase() == name.toLowerCase())
                        symbolName = name;
                });
							
				var xXalue = commonEventArgs.data.location.x - (margin.left) / 2 + (margin.right) / 2,
                    yValue = commonEventArgs.data.location.y - (margin.top) / 2 - (height / margin.top) + (margin.bottom) / 2,
                    degree = dataLabel.angle,
                    angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
							
 
							

                //For accurate placement of rotated data labels in canvas
				if (chartModel.enableCanvasRendering && (degree % 360 != 0)) {
				    options.y -= textOffset.height / 4;
				    options.baseline = 'middle';
				}

				if (series._enableSmartLabels) {
				    point.dataLabel = { textX: options.x, textY: options.y, x: xXalue, y: yValue, width: width, height: height }
				    point.margin = {top: margin.top, bottom: margin.bottom}
				}

							    if (point.y == point.low)
							    {
							        point.xPosLow = xXalue;
							        point.yPosLow = yValue;
							        point.widthLow = (symbolName == "None") ? textOffset.width : width;
							        point.heightLow = (symbolName == "None") ? textOffset.height : height;
							        point.textOptionsLow = options;
							        point.textOptionsLow.angle = angle;
							        point.drawTextLow = commonEventArgs.data.text;
							    }
							
							    else {
							        // storing the values in point
							    point.textOptions = options;
							    point.textOptions.angle = angle;
							    point.drawText = commonEventArgs.data.text;
							    point.index = index;
							    point.xPos = xXalue;
							    point.yPos = yValue;
							    point.seriesIndex = seriesIndex;
							    point.width = (symbolName == "None") ? textOffset.width : width;
							    point.height = (symbolName == "None") ? textOffset.height : height;
							    point.symbolName = symbolName;
							   }
						}
            else {
					location.X -= chart.canvasX;
					location.Y -= chart.canvasY;
					this.drawLabelTemplate(series, point, index, location, chart);
                }                             
        }
    },

    //smart labels for pie and doughnut

    updateSmartLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender) {
        var renderingPoints = [],
            chartObj = sender,
            chartModel = chartObj.model,
            legend = chartModel.legend,
            legendActualBounds = chartModel.LegendActualBounds,
            legendPosition = legend.position.toLowerCase(),
            seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()](),
            connectorDirection = "",
			bezierPoints="",
            connectorX = bounds.connector.connectorX,
            connectorY = bounds.connector.connectorY,
            midX = bounds.midPoint.midX,
            midY = bounds.midPoint.midY,
            dMidX = bounds.doughnutMidPoint.dMidX,
            dMidY = bounds.doughnutMidPoint.dMidY,
            startX = startPoint.startX,
            startY = startPoint.startY,
            marker = $.extend(true, {}, currentseries.marker, point.marker),
            textGap = (marker.dataLabel.template) ? 0 : 4,
            outerX,
            radius = chartModel.Radius,
            isIntersectedLabel,
            angle = midAngle,
            smartAngle = (currentseries.startAngle > currentseries.endAngle) ? -0.01 : 0.01,
            connectorType = marker.dataLabel.connectorLine.type.toLowerCase(),
            previousLeft;

        //calculation for inside labels
        if (currentseries.labelPosition.toLowerCase() == 'inside' && ej.util.isNullOrUndefined(point.smartLabelPosition)) {
            positionX = (midX + startX) / 2;
            positionY = (midY + startY) / 2;

            positionX = (midX + positionX) / 2;
            positionY = (midY + positionY) / 2;

            if (point.isIntersected && chartModel.bounds[pointIndex]) {
                currBounds = chartModel.bounds[pointIndex];
            }
            else {
                if (midX > startX)
                    currBounds = { X: positionX, Y: positionY, Left: positionX, Top: positionY, Right: positionX + size.width, Bottom: positionY + size.height };
                else
                    currBounds = { X: positionX, Y: positionY, Left: positionX - size.width, Top: positionY, Right: positionX, Bottom: positionY + size.height };
             }

            do {
                isIntersectedLabel = false;
                if (!this.IntersectWith(chartObj, point, currBounds)) continue;
                previousLeft = currBounds.Left;
                angle += smartAngle;
                point.isIntersected = true;
                currBounds.Angle = angle;
                if (midX > startX) {
                    currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Right = currBounds.X + size.width;
                }
                else {
                    currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Left = currBounds.Right - size.width;
                }
                currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);
                currBounds.Bottom = currBounds.Y + size.height;
                isIntersectedLabel = true;
            } while (isIntersectedLabel)
            if (midX > startX && currBounds.X < startX) {
                currBounds.Right = previousLeft;
                currBounds.Left = previousLeft - size.width;
            }
            chartModel.bounds[pointIndex] = currBounds;

            if (point.isIntersected && chartModel.bounds[pointIndex].Angle) {

                positionX = seriesType.getXCordinate(startX, (radius) + textOffset, chartModel.bounds[pointIndex].Angle);
                positionY = seriesType.getYCordinate(startY, (radius) + textOffset, chartModel.bounds[pointIndex].Angle);

                var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
                var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);
                renderingPoints.push({ X: midX, Y: midY });
                renderingPoints.push({ X: connectorMidX, Y: connectorMidY });
                renderingPoints.push({ X: positionX, Y: positionY });
                if (point.isIntersected && connectorType == 'bezier' && !chartObj.vmlRendering) {
                    bezierPoints = seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
                }
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + " " + "M" + " " + connectorMidX + " " + connectorMidY + " " + "L" + " " + positionX + " " + positionY;

            }
        }

       //calculation for outside labels
        else if (currentseries.labelPosition.toLowerCase() == 'outside' || point.smartLabelPosition =="outside") {
           
            if (midX > startX) 
                currBounds = { X: connectorX, Y: connectorY, Left: connectorX, Top: connectorY, Right: connectorX + size.width, Bottom: connectorY + size.height };
           else
               currBounds = { X: connectorX , Y: connectorY, Left: connectorX - size.width, Top: connectorY, Right: connectorX, Bottom: connectorY + size.height };
            
            do {
                isIntersectedLabel = false;
                if (!this.IntersectWith(chartObj, point, currBounds)) continue;
				 previousLeft = currBounds.Left;
                angle += smartAngle;
                point.isIntersected = true;
                if (midX > startX) {
                    currBounds.X = currBounds.Left = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                    currBounds.Right = currBounds.X + size.width;
                }
                else {
                    currBounds.X = currBounds.Right = seriesType.getXCordinate(startX, (radius) + textOffset, angle);
                   currBounds.Left = currBounds.Right - size.width;
                }

                currBounds.Y = currBounds.Top = seriesType.getYCordinate(startY, (radius) + textOffset, angle);              
                currBounds.Bottom = currBounds.Y + size.height;
                isIntersectedLabel = true;
            } while (isIntersectedLabel)
			 if (midX > startX && currBounds.X < startX) {
                currBounds.Right = previousLeft;
                currBounds.Left = previousLeft - size.width;
            }
            chartModel.bounds[pointIndex] = currBounds;

            var connectorMidX = seriesType.getXCordinate(startX, (radius) + textOffset / 3, midAngle);
            var connectorMidY = seriesType.getYCordinate(startY, (radius) + textOffset / 3, midAngle);

            renderingPoints.push({ X: midX, Y: midY });
            renderingPoints.push({ X: connectorMidX, Y: connectorMidY });
            renderingPoints.push({ X: currBounds.X, Y: currBounds.Y });
            positionX = currBounds.X;
            positionY = currBounds.Y;
            if (textOffset > 0) {
                if (midX < startX) {
                    outerX = currBounds.X - chartModel.elementSpacing;
                    positionX = outerX - textGap;
                }
                else {
                    outerX = currBounds.X + chartModel.elementSpacing;
                    positionX = outerX + textGap;
                }
                renderingPoints.push({ X: outerX, Y: currBounds.Y });
            }
            if (connectorType == 'bezier' && !chartObj.vmlRendering) {
                bezierPoints = seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorMidX + " " + connectorMidY + " "+ "M" + " " + connectorMidX + " " + connectorMidY + " " + "L" + " " + currBounds.X + " " + currBounds.Y + " " + "M" + " " + currBounds.X + " " + currBounds.Y +  " " + "L" + " " + outerX + " " + currBounds.Y;
              }
                      
        }


         //calculation for outsideExtended labels
        else {
            var labelX = midX;
            var labelY = midY;

            midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
            midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));

            var connectorHeight = radius / 4;

            renderingPoints.push({ X: midX, Y: midY });
            midX = midX + (Math.cos((midAngle)) * connectorHeight);
            midY = midY + (Math.sin((midAngle)) * connectorHeight);

            renderingPoints.push({ X: midX, Y: midY });

            var legendWidth = (legend.visible && legendPosition == "left") ? (chartModel.margin.left + legendActualBounds.Width) : 0;
            var centerX = chartModel.centerX;
            var labelEdge;

            if (midX < startX) {

                midX = ((centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                 ? centerX - radius - (connectorHeight * 2) + legendWidth
                                 : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth + legendWidth) - size.width / 2;

                labelEdge = renderingPoints[1].X - currentseries.LeftLabelMaxWidth;

                var height = (centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0) ? connectorHeight
                             : (currentseries.LeftLabelMaxWidth > labelX) ? 0
                             : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;

                connectorLineEdge = +size.width / 2;

            }
            else {
                var legendRight = (legend.visible && legendPosition == "right") ? (chartModel.margin.right + legendActualBounds.Width) : 0;
                midX = ((centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width() - legendRight)
               ? centerX + radius + (connectorHeight * 2) - legendWidth
               : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth - legendRight) + size.width / 2;


                labelEdge = ($(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth) - renderingPoints[1].X;

                var height = (centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width()) ? connectorHeight
                            : ((centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? 0
                            : (labelEdge > 3 * (connectorHeight / 2)) ? connectorHeight : labelEdge / 2;
                connectorLineEdge = -size.width / 2;
            }

            var distanceFromOrigin = (Math.sqrt(Math.pow(labelX - midX, 2) + Math.pow(labelY - midY, 2))) / 10;

            var isLeft = (midX < startX) ? true : false;
            midX = isLeft ? midX + distanceFromOrigin : midX - distanceFromOrigin;
            var index = isLeft ? 1 : -1;

            var bounds = isLeft ? chartModel.bounds["left"] : chartModel.bounds["right"];
            var currBounds = { X: midX, Y: midY, Left: midX, Top: midY, Right: midX + size.width, Bottom: midY + size.height, index: pointIndex };
            if (this.IntersectWith(chartObj, point, currBounds)) {
                renderingPoints.push(midX < startX
                       ? { X: midX + height + connectorLineEdge, Y: midY }
                       : { X: midX - height + connectorLineEdge, Y: midY });
                midY = bounds[point.index + index].Bottom + 2;

                currBounds.Y = midY;
                currBounds.Top = midY;
                currBounds.Bottom = midY + size.height;

            }

            renderingPoints.push({ X: midX + connectorLineEdge, Y: midY });
            if (centerX < renderingPoints[0].X && (renderingPoints[1].X > renderingPoints[2].X))
                renderingPoints[2].X = renderingPoints[1].X;
            if (centerX > renderingPoints[0].X && (renderingPoints[1].X < renderingPoints[2].X))
                renderingPoints[2].X = renderingPoints[1].X;
            if (!isLeft) {
                chartModel.bounds["right"][point.index] = (currBounds);
                if (renderingPoints[renderingPoints.length - 1].X < renderingPoints[renderingPoints.length - 2].X)
                    renderingPoints[renderingPoints.length - 2].X = renderingPoints[renderingPoints.length - 1].X;
               
            }
            else {
                chartModel.bounds["left"][point.index] = (currBounds);
                if (renderingPoints[renderingPoints.length - 1].X > renderingPoints[renderingPoints.length - 2].X)
                    renderingPoints[renderingPoints.length - 2].X = renderingPoints[renderingPoints.length - 1].X;
            }


            connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
            if (connectorType == 'bezier' && !chartObj.vmlRendering) {
                bezierPoints=seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                for (k = 0; k < renderingPoints.length; k++) {
                    if (k == renderingPoints.length - 1)
                        connectorDirection.append(" ");
                    else
                        connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                }
                connectorDirection = connectorDirection.toString();
            }
            positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);
            positionY = renderingPoints[renderingPoints.length - 1].Y;
        }
        return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection, isInterSected:point.isIntersected,bezierPath:bezierPoints };
    },

    updateLabelPosition: function (currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender){

        var renderingPoints=[];
        var chartObj=sender; 
        var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
        var connectorDirection = "";

        var connectorX = bounds.connector.connectorX;
        var connectorY = bounds.connector.connectorY;
        var midX = bounds.midPoint.midX;
        var midY = bounds.midPoint.midY;
        var dMidX = bounds.doughnutMidPoint.dMidX;
        var dMidY = bounds.doughnutMidPoint.dMidY;
        var startX = startPoint.startX;
        var startY = startPoint.startY;
        var marker = $.extend(true, {}, currentseries.marker, point.marker);
        var textGap = (marker.dataLabel.template) ? 0 :4;
       
        var outerX;
        var radius = chartObj.model.Radius;

        //calculation for outside labels

        renderingPoints.push({ X: midX, Y: midY });
        renderingPoints.push({ X: connectorX, Y: connectorY });
        positionX = connectorX;
        positionY = connectorY;

        if (currentseries.labelPosition.toLowerCase() == 'outside') {
            if (textOffset > 0) {
                if (connectorX < startX) {
                    outerX = connectorX - chartObj.model.elementSpacing;
                    positionX = outerX - textGap;
                }
                else {
                    outerX = connectorX + chartObj.model.elementSpacing;
                    positionX = outerX + textGap;
                }
                renderingPoints.push({ X: outerX, Y: connectorY });
            } 
           
            if (marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                connectorDirection = "M" + " " + midX + " " + midY + " " + "L" + " " + connectorX + " " + connectorY + " " + "M" + " " + connectorX + " " + connectorY + "L" + " " + outerX + " " + connectorY;
            }
          
        }


            //calculation for outsideextended labels
        else if (currentseries.labelPosition.toLowerCase() == 'outsideextended') {

            labelX=midX;
            labelY=midY;

            midX = midX + (Math.cos((midAngle)) * (-(radius / 10)));
            midY = midY + (Math.sin((midAngle)) * (-(radius / 10)));


            var renderingPoints = [];

            var connectorHeight = radius / 4;

            renderingPoints.push({ X: midX, Y: midY });
            connectorX = midX + (Math.cos((midAngle)) * connectorHeight);
            connectorY = midY + (Math.sin((midAngle)) * connectorHeight);

            renderingPoints.push({ X: connectorX, Y: connectorY });

            var legendWidth = (chartObj.model.legend.visible && (chartObj.model.legend.position.toLowerCase() == "left" || chartObj.model.legend.position.toLowerCase() == "right")) ? (chartObj.model.elementSpacing) : 0;

            if (midX < startX) {
                connectorX = (chartObj.model.centerX - radius - (connectorHeight * 2) - currentseries.LeftLabelMaxWidth > 0)
                                     ? chartObj.model.centerX - radius - (connectorHeight * 2) + legendWidth
                                     : (currentseries.LeftLabelMaxWidth > labelX) ? labelX : currentseries.LeftLabelMaxWidth;
            }
            else {
                connectorX = (chartObj.model.centerX + radius + (connectorHeight * 2) + currentseries.RightLabelMaxWidth < $(chartObj.svgObject).width())
                     ? chartObj.model.centerX + radius + (connectorHeight * 2) - legendWidth
                     : ((chartObj.model.centerX + radius + currentseries.RightLabelMaxWidth) > $(chartObj.svgObject).width()) ? labelX : $(chartObj.svgObject).width() - currentseries.RightLabelMaxWidth;
                      
            }

            renderingPoints.push({ X: connectorX, Y: connectorY });

            connectorDirection = ej.EjSvgRender.utils._getStringBuilder();
            if (marker.dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering) {
                seriesType.drawBezierSegment(renderingPoints, currentseries, pointIndex, chartObj);
            }
            else {
                for (k = 0; k < renderingPoints.length; k++) {
                    if (k == renderingPoints.length - 1)
                        connectorDirection.append(" ");
                    else
                        connectorDirection.append("M" + " " + (renderingPoints[k].X) + " " + (renderingPoints[k].Y) + " " + "L" + " " + (renderingPoints[k + 1].X) + " " + (renderingPoints[k + 1].Y) + " ");
                }
                connectorDirection = connectorDirection.toString();
            }
            positionX = renderingPoints[renderingPoints.length - 1].X + ((midX < startX) ? -textGap : textGap);;
            positionY = renderingPoints[renderingPoints.length - 1].Y;
        }

       //calculation for inside labels

        else {
            if (currentseries.type.toLowerCase() == "doughnut") {
                positionX = (midX + dMidX) / 2;
                positionY = (midY + dMidY) / 2;
            }
            else {
                positionX = (midX + startX) / 2;
                positionY = (midY + startY) / 2;

                positionX = (midX + positionX) / 2;
                positionY = (midY + positionY) / 2;
            }
        }

        return { positionX: positionX, positionY: positionY, connectorDirection: connectorDirection };
    },
    // method to update the position of the data labels in pyramid chart
        updateSmartLabel: function (chartObj, currentseries, pointIndex, point) {
            // declaration
        var datalabelTemplateVisiblity = true,
            connectorDirection = "",
            labelPosition = currentseries.labelPosition.toLowerCase(),
            marker = $.extend(true, {}, currentseries.marker, point.marker),
            dataLabel = marker.dataLabel,
            areaBounds = chartObj.model.m_AreaBounds,
            type = currentseries.type.toLowerCase(),
            pyrX, pyrY, ele, xPos, yPos;
            point._labelPlacement = "";
            if (labelPosition == "inside" || labelPosition == "outsideextended")
                this.calculateInsideConnectorLines(pointIndex, point, currentseries, chartObj);
            else
                this.compareDataLabels(point, pointIndex, chartObj.model.outsideDataRegionPoints, currentseries, chartObj);

            pyrX = (chartObj.model.enableCanvasRendering && !dataLabel.template) ? chartObj.pyrX : 0;
            pyrY = (chartObj.model.enableCanvasRendering && !dataLabel.template) ? chartObj.pyrY : 0;
            if (type == "pyramid") {
                if ((areaBounds.Height < point.yPos - pyrY) || (point.xPos - pyrX > areaBounds.Width && point._labelPlacement == "insideoverlap") || (chartObj.model.legend.position == "right" && dataLabel.template && (point.xPos + point.width) > areaBounds.Width + chartObj.model.LegendActualBounds.Width)) {
                    datalabelTemplateVisiblity = false;
                    point.drawTextacc = "";
                }
            }
            else {
                if ((point.yPos - pyrY < 0) || (point.xPos - pyrX > areaBounds.Width && point._labelPlacement == "insideoverlap") || (chartObj.model.legend.position == "right" && dataLabel.template && (point.xPos + point.width) > areaBounds.Width + chartObj.model.LegendActualBounds.Width)) {
                    datalabelTemplateVisiblity = false;
                    point.drawTextacc = "";

                }
            }
            if ((labelPosition == "outside" || point._labelPlacement == "insideoverlap") && datalabelTemplateVisiblity)
                connectorDirection = this.drawConnectorLines(pointIndex, point, currentseries, chartObj);

            if (dataLabel.template) {
                ele = $("#" + point.id);
                if (!datalabelTemplateVisiblity) {
                    ele.css("display", "none");
                }
                else {
                    xPos = (point._labelPlacement == "insidenooverlap") ? point.textOptionsacc.x + chartObj.pyrX - point.width / 2 : point.textOptionsacc.x + chartObj.pyrX;
                    yPos = (point._labelPlacement == "insidenooverlap") ? point.textOptionsacc.y + chartObj.pyrY : point.textOptionsacc.y + chartObj.pyrY - chartObj.model.elementSpacing;
                    ele.css("left", xPos).css("top", yPos);
                    ele.css("height", point.height).css("overflow", "hidden");
                }
            }


            return { xPos: point.xPos, yPos: point.yPos, textOptionsacc: point.textOptionsacc, connectorDirection: connectorDirection.connectorDirection,bezierPath:connectorDirection.bezierdir, drawTextacc: point.drawTextacc };
        },
        // method to compare the data labels in chart for intersection
        compareDataLabels: function (point, pointIndex, outsideDataRegionPoints, currentseries, chartObj) {

            var marker = $.extend(true, {}, currentseries.marker, point.marker),
                dataLabel = marker.dataLabel,
                prevLabel, collide, regionPoint, degree, angle, rotate,
                elementSpacing = chartObj.model.elementSpacing,
                margin = marker.dataLabel.margin,
				labelPosition = currentseries.labelPosition.toLowerCase(),
                pyrX = (chartObj.model.enableCanvasRendering) ? chartObj.pyrX : 0,
                pyrY = (chartObj.model.enableCanvasRendering) ? chartObj.pyrY : 0;
            for (var j = 0, length = outsideDataRegionPoints.length; j < length ; j++) {
                prevLabel = outsideDataRegionPoints[j];
                collide = this.isCollide(prevLabel, point, currentseries);
                if (collide.state) {
                    if (labelPosition == "inside" || labelPosition == "outsideextended") {
                        point.textOptionsacc.y = point.yPos = point.textOptionsacc.y + collide.height;
                        positionPoint = (dataLabel.template) ? { x: point.xPos, y: point.yPos } : { x: point.xPos - pyrX, y: point.yPos - pyrY };
                        for (var k = 0, visibleLength = currentseries.visiblePoints.length ; k < visibleLength ; k++) {
                            regionPoint = (chartObj.isPointInPolygon(chartObj.model.chartRegions[0].Region[k].Polygon, positionPoint));
                            if (regionPoint) {
                                point.textOptionsacc.x = point.xPos = point.textOptionsacc.x + point.width;
                                positionPoint = (dataLabel.template) ? { x: point.xPos, y: point.yPos } : { x: point.xPos - pyrX, y: point.yPos - pyrY };
                            }
                        }
                    }
                    else {
                        point._labelPlacement = "outside";
                        point.textOptionsacc.y = point.yPos = (currentseries.type.toLowerCase() == "pyramid") ? point.yPos + collide.height + elementSpacing / 2 : point.yPos + collide.height - elementSpacing / 2;
                        point.textOptionsacc.y = point.textOptionsacc.y + point.height / 4 - elementSpacing / 2 + margin.top / 2 - margin.bottom / 2;
                    }
                }
            }
            degree = dataLabel.angle;
            angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
            rotate = 'rotate(' + angle + ',' + (point.textOptionsacc.x) + ',' + (point.textOptionsacc.y) + ')';
            $(point.textOptionsacc).attr('transform', rotate);
            $(point.textOptionsacc).attr('labelRotation', degree);//for canvas label rotation
            $(point.textOptionsacc).attr({
                'transform': rotate
            });
            if (labelPosition == "inside" || labelPosition == "outsideextended") {
                point.yPos = point.yPos + elementSpacing / 4;
                point.textOptionsacc.y = point.textOptionsacc.y + point.height / 4 + margin.top / 2 - margin.bottom / 2;
            }
            outsideDataRegionPoints[outsideDataRegionPoints.length] = point;
        },

        //Inside region checking and smartlabels
        calculateInsideConnectorLines: function (pointIndex, point, currentseries, chartObj) {

            var polygon = point.Polygon,
                marker = $.extend(true, {}, currentseries.marker, point.marker),
                dataLabel = marker.dataLabel,
                font = dataLabel.font, connectorHeight,
                pyrX = (chartObj.model.enableCanvasRendering) ? chartObj.pyrX : 0,
                pyrY = (chartObj.model.enableCanvasRendering) ? chartObj.pyrY : 0,
                //inside connector line height
                connectorLength = 70,
                datalabelWidth = ej.EjSvgRender.utils._measureText(point.drawTextacc, null, font),
                positionPoint = (dataLabel.template) ? { x: point.xPos, y: point.yPos } : { x: point.xPos + (datalabelWidth.width / 2) - pyrX, y: point.yPos - (point.height / 2) - pyrY },
                regionPoint = chartObj.isPointInPolygon(polygon, positionPoint);
            if (!regionPoint) {
                point._labelPlacement = "insideoverlap";
                point.textOptionsacc['text-anchor'] = 'start';
                if (point.startX + connectorLength < chartObj.model.m_AreaBounds.Width)
                    connectorHeight = connectorLength;
                else
                    connectorHeight = chartObj.model.m_AreaBounds.Width - (point.startX);

                point.textOptionsacc.x = point.xPos = point.startX + connectorHeight;
                point.textOptionsacc.y = point.yPos = point.yLocation;
                if (chartObj.model.enableCanvasRendering && !dataLabel.template) {
                    point.textOptionsacc.x += chartObj.pyrX;
                    point.textOptionsacc.y += chartObj.pyrY;
                    point.xPos += chartObj.pyrX;
                    point.yPos += chartObj.pyrY;
                }
                this.compareDataLabels(point, pointIndex, chartObj.model.outsideDataRegionPoints, currentseries, chartObj);

            }

            else {
                point._labelPlacement = "insidenooverlap";
            }
        },
        //smartLabels connector Lines
        drawConnectorLines: function (index, currentPoint, currentseries, chartObj) {
            var bezierdir = "";
            var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()](),
                startx = currentseries.points[index].startX,
                starty = currentseries.points[index].startY,
                connectorDirection = "",
                marker = $.extend(true, {}, currentseries.marker, currentPoint.marker),
                dataLabel = marker.dataLabel,
                endy = currentPoint.textOptionsacc.y - chartObj.model.elementSpacing / 2,
                pyrX = (chartObj.model.enableCanvasRendering) ? chartObj.pyrX : 0,
                pyrY = (chartObj.model.enableCanvasRendering) ? chartObj.pyrY : 0,
                renderingPoints = [{ X: startx + pyrX, Y: starty + pyrY },//bezier curve control points
                                   { X: currentPoint.textOptionsacc.x, Y: endy }];
            if (dataLabel.connectorLine.type.toLowerCase() == 'bezier' && !chartObj.vmlRendering)
                bezierdir =  seriesType.drawBezierSegment(renderingPoints, currentseries, index, chartObj);
            else {
                connectorDirection = "M" + " " + (startx) + " " + (starty) + " " + "L" + " " + (currentPoint.textOptionsacc.x) + " " + (endy);
                if (chartObj.model.enableCanvasRendering && !dataLabel.template)
                    connectorDirection = "M" + " " + (startx) + " " + (starty) + " " + "L" + " " + (currentPoint.textOptionsacc.x - pyrX) + " " + (endy - pyrY);
            }
            return { connectorDirection: connectorDirection, bezierdir: bezierdir };
        },

        //method to detect collision between data labels
        isCollide: function (prevlabel, currentlabel, currentseries) {
            var width = 0,
                height = 0,
                type = currentseries.type.toLowerCase(),
                state;
            state = !(
           ((prevlabel.yPos + prevlabel.height) < (currentlabel.yPos)) ||
           (prevlabel.yPos > (currentlabel.yPos + currentlabel.height)) ||
           ((prevlabel.xPos + prevlabel.width / 2) < currentlabel.xPos - currentlabel.width / 2) ||
           ((prevlabel.xPos - prevlabel.width) > (currentlabel.xPos + currentlabel.width)));

            if (state && type == "pyramid") {
                height = ((prevlabel.yPos + prevlabel.height) - (currentlabel.yPos));
                width = (prevlabel.xPos + prevlabel.width) - currentlabel.xPos;
            }
            else if (state && type == "funnel") {
                height = ((prevlabel.yPos) - (currentlabel.yPos + currentlabel.height));
                width = (prevlabel.xPos + prevlabel.width) - currentlabel.xPos;
            }
            return { state: state, height: height, width: width };
        },
        // trimming the text when it exceeds chart area
        trimText: function (text, maxLength, ellipsis) {
            maxLength--;
            var length = maxLength - ellipsis.length;
            trimmedText = text.substr(0, length);
            return trimmedText + ellipsis;
        },

        //calculate a length of a text for datalabel trimming
        trimfunction: function (datalabelText, remainingWidth, datalabelLength, elipsis, axisWidth, font) {
            var datalabelText, datalabelLength;
            while (remainingWidth < datalabelLength) {
                datalabelText = this.trimText(datalabelText, datalabelText.length, elipsis);
                datalabelLength = ej.EjSvgRender.utils._measureText(datalabelText, axisWidth, font).width;
                if (datalabelText == elipsis) {
                    datalabelText = "";
                    break;
                }
            }
            return datalabelText;
        },

        //To calculate start and end point of a text inside a pie and doughnut region
        calculatePosition: function (chartObj, textOptions, datalabelLength, startX, startY) {
            var rightEndX, rightEndY, leftEndX, leftEndY;            
            rightEndX = (textOptions.x + (datalabelLength / 2)) - startX;
            rightEndY = (textOptions.y) - (startY);
            leftEndX = (textOptions.x - (datalabelLength / 2)) - startX;
            leftEndY = textOptions.y - startY;
            return { rightEndX: rightEndX, rightEndY: rightEndY, leftEndX: leftEndX, leftEndY: leftEndY };
                        
        },

        //To caluculate textoptions
        textOption:function(currentseries,chartObj,textsize,pyrX,pyrY,pointIndex,textAnchor,positionX,positionY){
            var marker = $.extend(true, {}, currentseries.marker, currentseries.points[pointIndex].marker);
            if (!marker.dataLabel.template) {
                $("#" + chartObj.svgObject.id + '_PointText' + pointIndex).remove();
                var textOptions = {
                    'id': chartObj.svgObject.id + '_PointText' + pointIndex,
                    'x': positionX + pyrX,
                    'y': positionY + textsize.height / 4 + pyrY,
                    'fill': marker.dataLabel.font.color,
                    'font-size': marker.dataLabel.font.size,
                    'font-family': marker.dataLabel.font.fontFamily,
                    'font-style': marker.dataLabel.font.fontStyle,
                    'font-weight': marker.dataLabel.font.fontWeight,
                    'opacity': marker.dataLabel.font.opacity,
                    'text-anchor': textAnchor,
                    'cursor': 'default',
                    'degree':marker.dataLabel.angle
                };
                //To rotate datalabels
                var degree = marker.dataLabel.angle;
                var angle = (degree > 360) ? degree - 360 : (degree < -360) ? degree + 360 : degree;
                var rotate = 'rotate(' + angle + ',' + (positionX) + ',' + (positionY) + ')';
                $(textOptions).attr('transform', rotate);
                $(textOptions).attr('labelRotation', degree);//for canvas label rotation
                $(textOptions).attr({
                    'transform': rotate
                });
            }
            return textOptions;
        },

        //To trim  datalabel when it exceed a particular pie and doughnut region
        labelTrim: function (chartObj, textOptions, datalabelLength, datalabelText, startX, startY, point, font) {
            var chartStartAngle = -.5 * Math.PI;
            var chartEndAngle = .5 * Math.PI;
            var position = this.calculatePosition(chartObj, textOptions, datalabelLength, startX, startY);
            var startAngle = chartObj.model.series[0].startAngle;
            var endAngle = chartObj.model.series[0].endAngle;
            var regionData = chartObj.model.chartRegions[0].Region;
            var pointStartAngle, pointEndAngle;
            var distanceFromCenter;
            var textinRegion;
            var trimPosition, datalabelWidth;
            var ellipsis = "...";
            var firstStartAngle = [];
            var totalDegree = endAngle - startAngle;
            if (totalDegree < 0) {
                endAngle = endAngle / 360;
                var arcAngle = (endAngle) ? 2 * Math.PI * (endAngle < 0 ? 1 + endAngle : endAngle) : 0;
            }
            else
                var arcAngle = startAngle ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;            
            startAngle = (Math.atan2(position.rightEndY, position.rightEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
            if (startAngle < 0) startAngle = 2 * Math.PI + startAngle;
            endAngle = (Math.atan2(position.leftEndY, position.leftEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
            if (endAngle < 0) endAngle = 2 * Math.PI + endAngle;            
            pointStartAngle = parseFloat(point.start.toFixed(14));
            pointEndAngle = parseFloat(point.end.toFixed(14));
            if (totalDegree > 0) {
                if (ej.util.isNullOrUndefined(chartObj.firstStartAngle[0])) {
                    chartObj.firstStartAngle.push(pointStartAngle);
                }
                pointStartAngle = (chartObj.firstStartAngle[0] < 0) ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                pointEndAngle = (chartObj.firstStartAngle[0] < 0) ? 2 * Math.PI + pointEndAngle : pointEndAngle;
            }
            else {
                pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
            }
            pointStartAngle -= arcAngle;
            pointEndAngle -= arcAngle;
            if (startAngle < 0 && (endAngle > 0 || endAngle == null)) {
                pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                pointEndAngle = pointEndAngle <= 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
            }
            if (chartObj.model.centerX < textOptions.x) {
                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position.rightEndX), 2) + Math.pow(Math.abs(position.rightEndY), 2));
            }
            else
                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(position.leftEndX), 2) + Math.pow(Math.abs(position.leftEndY), 2));            
            if (totalDegree < 0) {               
                pointEndAngle = [pointStartAngle, pointStartAngle = pointEndAngle][0];
            }            
            if ((endAngle >= pointStartAngle && endAngle <= pointEndAngle) && (startAngle >= pointStartAngle && startAngle <= pointEndAngle) && (distanceFromCenter <= chartObj.model.Radius && distanceFromCenter > 0)) {
                textinRegion = true;
            }
            else {
                textinRegion = false;
                while (!textinRegion) {
                   datalabelText = this.trimText(datalabelText, datalabelText.length, ellipsis);
                   if (datalabelText == ellipsis) {
                       datalabelText = " ";
                       break;
                   }  
                   datalabeltWidth = ej.EjSvgRender.utils._measureText(datalabelText, null, font);
                   trimPosition = this.calculatePosition(chartObj, textOptions, datalabeltWidth.width, startX, startY);
                   startangle = (Math.atan2(trimPosition.rightEndY, trimPosition.rightEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                   if (startangle < 0) startangle = 2 * Math.PI + startangle;
                   var endAngle = (Math.atan2(trimPosition.leftEndY, trimPosition.leftEndX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                   if (endAngle < 0) endAngle = 2 * Math.PI + endAngle;
                   if (chartObj.model.centerX < textOptions.x) {
                      distanceFromCenter = Math.sqrt(Math.pow(Math.abs(trimPosition.rightEndX), 2) + Math.pow(Math.abs(trimPosition.rightEndY), 2));
                   }
                   else
                      distanceFromCenter = Math.sqrt(Math.pow(Math.abs(trimPosition.leftEndX), 2) + Math.pow(Math.abs(trimPosition.leftEndY), 2));
                   if ((endAngle >= pointStartAngle && endAngle <= pointEndAngle) && (startangle >= pointStartAngle && startangle <= pointEndAngle) && (distanceFromCenter <= chartObj.model.Radius && distanceFromCenter > 0)) {
                      textinRegion = true;
                   }
                 }
            }
                        
         return datalabelText;
        },
    
    
	drawDataLabelAcc: function (sender, currentseries, pointIndex, point) {
	    // method for data label symbols - accumulation series	
	    var pointMarker = point.marker;
	    if (point.visible && ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && currentseries.marker.dataLabel.visible)) {
	        var chartObj = sender;
	        var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	        var outerX;
	        var connectorDirection;
	        var positionX = 0;
	        var positionY = 0;
	        var pointColor;
	        var numberToFixed = ej.util.isNullOrUndefined(chartObj.model.roundingPlaces) ? 2 : chartObj.model.roundingPlaces;
           var type=currentseries.type.toLowerCase();
	        var _labelPosition = currentseries.labelPosition.toLowerCase();
			if ((type == "pyramid" || type == "funnel") && _labelPosition == 'outsideextended')
			{
	            _labelPosition = 'inside';
	        }
	       
	        var text = ej.util.isNullOrUndefined(point.text) ? point.y : point.text;
	        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
	        commonEventArgs.data = { text: text, series: currentseries, pointIndex: pointIndex };
	        chartObj._trigger("displayTextRendering", commonEventArgs);
	       
	        var labelfont = { size: currentseries.marker.dataLabel.font.size, fontStyle: currentseries.marker.dataLabel.font.fontStyle, fontFamily: currentseries.marker.dataLabel.font.fontFamily };
	        var textsize = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, null, labelfont);

	     

	        var margin = currentseries.marker.dataLabel.margin;

	        var textWidth = textsize.width + margin.left + margin.right;
	        var textHeight = textsize.height + margin.top + margin.bottom;

	        if (type == "pyramid" || type == "funnel") {
	            var measureTitle = (chartObj.model.title.text) ? ej.EjSvgRender.utils._measureText(chartObj.model.title.text, $(this.svgObject).width() - chartObj.model.margin.left - chartObj.model.margin.right, chartObj.model.title.font) : 0;
	            var pyrX = chartObj.model.elementSpacing + chartObj.model.margin.left +
                           (chartObj.model.legend.position.toLowerCase() === "left") ?
                           (chartObj.model.LegendActualBounds.Width) : 0;
	            var pyrY = ((chartObj.model.legend.position.toLowerCase() === "top")
                            ? (chartObj.model.LegendActualBounds.Height) : 0) +
                            ((chartObj.model.title.text) ? (chartObj.model._titleLocation.Y + measureTitle.height) : (chartObj.model.margin.top + chartObj.model.elementSpacing));
	            textWidth = textWidth + 10;
	            var Location = { X: point.xLocation + chartObj.pyrX, Y: point.yLocation + chartObj.pyrY };
	            positionX = point.xLocation;
	            positionY = point.yLocation;
	            var marker = point.marker ? point.marker : currentseries.marker;
	            marker = $.extend(true, {}, currentseries.marker, marker);
	            connectorDirection = point.connectorLine;
	            symbolPos = { positionX: positionX, positionY: positionY };

	        }

	        if (type != "pyramid" && type != "funnel") {
	            var textOffset;
	            var radius = chartObj.model.Radius;
	            if (ej.util.isNullOrUndefined(currentseries.marker.dataLabel.connectorLine.height))
	                textOffset = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, null, currentseries.marker.dataLabel.font).height;
               else
	                textOffset = currentseries.marker.dataLabel.connectorLine.height;
	            var chartStartingAngle = -.5 * Math.PI; // to move the arc to 90 degree from the start.
	            var renderingPoints = [];

	            var startX, startY;

	            var midAngle = point.midAngle + chartStartingAngle;
	            if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !chartObj.vmlRendering) {

	                startX = chartObj.model.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
	                startY = chartObj.model.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
	            }
	            else {
	                startX = chartObj.model.centerX;
	                startY = chartObj.model.centerY;
	            }
	            var midX = seriesType.getXCordinate(startX, (radius), midAngle);
	            var midY = seriesType.getYCordinate(startY, (radius), midAngle);

	            var dMidX = seriesType.getXCordinate(startX, (chartObj.model.dRadius), midAngle);
	            var dMidY = seriesType.getYCordinate(startY, (chartObj.model.dRadius), midAngle);

	            var connectorX = this.getXCordinate(startX, (radius) + textOffset, midAngle);
	            var connectorY = this.getYCordinate(startY, (radius) + textOffset, midAngle);

	            if (currentseries.marker.dataLabel.template) {
	                size = point.size;
	            }
	            else {
	                var marker = point.marker ? point.marker : currentseries.marker;
	                marker = $.extend(true, {}, currentseries.marker, marker);
	                var shape = marker.dataLabel.shape;
	                size = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, $(chartObj.svgObject).width(), currentseries.marker.dataLabel.font);
	                if (shape.toLowerCase() != "none") {
	                    var margin = currentseries.marker.dataLabel.margin;
	                    size.height = size.height + margin.top + margin.bottom;
	                    size.width = size.width + margin.left + margin.right;
	                }
	            }

	            var midPoint = { midX: midX, midY: midY };
	            var doughnutMidPoint = { dMidX: dMidX, dMidY: dMidY };
	            var connector = { connectorX: connectorX, connectorY: connectorY };
	            var bounds = { midPoint: midPoint, doughnutMidPoint: doughnutMidPoint, connector: connector };
	            var startPoint = { startX: startX, startY: startY };

	            if (!currentseries._enableSmartLabels) {
	                position = seriesType.updateLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender);
	                positionX = position.positionX;
	                positionY = position.positionY;
	                connectorDirection = position.connectorDirection;
	            }
	                // smart labels

	            else {

	                position = seriesType.updateSmartLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender);
	                positionX = position.positionX;
	                positionY = position.positionY;
                    symbolPos= { positionX: positionX, positionY: positionY, isInterSected:position.isInterSected };
	                connectorDirection = position.connectorDirection;
	            }



	        }
	        var textAnchor = ((_labelPosition != 'inside' && _labelPosition != 'insidenooverlap') || ((type == "pyramid" || type == "funnel") && _labelPosition == 'outside')) ? ((positionX < chartObj.model.centerX) ? 'end' : 'start') : 'middle';
	        if ((type == "pie" || type == "doughnut") && _labelPosition == "inside")
	            textAnchor = (position.connectorDirection != "") ? ((positionX < chartObj.model.centerX) ? 'end' : 'start') : 'middle';
	        if ((type == "pyramid" || type == "funnel") && _labelPosition == "outside")
	            textAnchor = 'start';
        var width = (_labelPosition == 'inside' || _labelPosition == 'insidenooverlap') ? 0 : (positionX < chartObj.model.centerX) ? -textsize.width : textsize.width;
	        var pyrX = 0; var pyrY = 0;
			if ((this.chartObj && this.chartObj.model.enableCanvasRendering) && (type == "pyramid" || type == "funnel")) {
			    pyrX = ((this.chartObj.model.legend.position.toLowerCase() === "left") ? (this.chartObj.model.LegendActualBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;
			    pyrY = ((this.chartObj.model.legend.position.toLowerCase() === "top") ? (this.chartObj.model.LegendActualBounds.Height) : 0) + ((this.chartObj.model.title.text) ? (this.chartObj.model._titleLocation.Y + measureTitle.height) : (this.chartObj.model.margin.top + this.chartObj.model.elementSpacing));
			}
			var textOptions = this.textOption(currentseries, chartObj, textsize, pyrX, pyrY, pointIndex, textAnchor, positionX, positionY);
	        if (currentseries.marker.dataLabel.template) {
	            Location = ej.util.isNullOrUndefined(Location) ? { X: positionX, Y: positionY } : Location;
	            var pointindex = point.pointIndex;	            
	                seriesType.drawLabelTemplate(currentseries, point, pointIndex, Location, sender);
				var linetype = currentseries.marker.dataLabel.connectorLine.type.toLowerCase();
	            if (linetype == "bezier" && (position.bezierPath) != "")
	                sender.svgRenderer.drawPolyline(position.bezierPath, sender.gSeriesConnectorEle);
	            
	        }

	        else {
	            $.each(chartObj.model.symbolShape, function (name) {
	                var marker = point.marker ? point.marker : currentseries.marker;
	                marker = $.extend(true, {}, currentseries.marker, marker);
	                var shape = marker.dataLabel.shape;
	                if (shape.toLowerCase() == name.toLowerCase())
	                    symbolName = name;
	            });

	            positionX = pyrX + positionX - (margin.left) / 2 + (margin.right) / 2 + width / 2;
	            positionY = pyrY + positionY - (margin.top) / 2 + (margin.bottom) / 2;
                //SmartLabels value storing for pyramid and funnel
                if (type == "pyramid" || type == "funnel") {				
                    point.xPos = symbolPos.positionX + pyrX;
                    point.yPos = symbolPos.positionY + pyrY;
                    point.width = (symbolName == "None") ? textsize.width : textWidth;
                    point.height = (symbolName == "None") ? textsize.height : textHeight;
                    point.symbolName = (currentseries.marker.dataLabel.template) ? "none" : symbolName;
                    point.textOptionsacc = textOptions;
                    point.drawTextacc = commonEventArgs.data.text;
				}	
	            //For accurate placement of rotated data labels in canvas
	            if (chartObj.model.enableCanvasRendering && (textOptions.degree % 360 != 0)) {
	                textOptions.y -= (textsize.height / 4);
	                textOptions.baseline = 'middle';
	            }
	            if (currentseries._enableSmartLabels) {
	                var legendBounds;
	                var remainingWidth;
	                var space = 20;
	                var legendWidth = 0; var legendHeight = 0;
	                var axisWidth = chartObj.model.m_AreaBounds.Width + chartObj.model.m_AreaBounds.X + chartObj.model.margin.left;
	                var datalabelText = (typeof commonEventArgs.data.text == "string") ? commonEventArgs.data.text : commonEventArgs.data.text.toString();
	                var font = currentseries.marker.dataLabel.font;
	                var legendPosition = chartObj.model.legend.position.toLowerCase();
	                var labelPosition = currentseries.labelPosition.toLowerCase();
	                var enableCanvasRendering = chartObj.model.enableCanvasRendering;
	                var elementSpacing = chartObj.model.elementSpacing;
					var linetype = currentseries.marker.dataLabel.connectorLine.type.toLowerCase();
	                var datalabelLength = ej.EjSvgRender.utils._measureText(datalabelText, datalabelText.length, font).width;
	                if (chartObj.model.legend.visible) {
	                    if (legendPosition == "left" || legendPosition == "right")
	                        legendBounds = chartObj.model.LegendActualBounds;
	                }
	                else
	                    legendBounds = { Width: 0 };
	                if (type == "pie" || type == "doughnut") {
	                    if (labelPosition == "outside" || labelPosition == "outsideextended" || position.connectorDirection != "") {
	                        if (chartObj.model.centerX < textOptions.x)
	                            remainingWidth = legendPosition == "right" ? (axisWidth - textOptions.x) : (axisWidth - textOptions.x - space);
	                        else
	                            remainingWidth = legendPosition == "left" ? (textOptions.x - space - legendBounds.Width) : (textOptions.x - space);
	                        datalabelText = this.trimfunction(datalabelText, remainingWidth, datalabelLength, "...", axisWidth, font);

	                        if (chartObj.model.legend.visible && (legendPosition == 'top' || legendPosition == 'bottom')) {
	                            datalabelText = this._overlapLegend(textOptions, datalabelText, sender);
	                        }   
	                    }
	                        if (labelPosition == "inside" && (position.connectorDirection == "")) {
	                            datalabelText = this.labelTrim(chartObj, textOptions, datalabelLength, datalabelText, startX, startY, point, font);
	                            if (datalabelText == "") {
	                                point.smartLabelPosition = "outside";
	                                position = seriesType.updateSmartLabelPosition(currentseries, point, pointIndex, bounds, midAngle, startPoint, textOffset, size, sender);
	                                symbolPos = { positionX: position.positionX, positionY: position.positionY, isInterSected: position.isInterSected };
	                                datalabelText = commonEventArgs.data.text;
	                                connectorDirection = position.connectorDirection;
	                                textAnchor = (chartObj.model.centerX < position.positionX) ? "start" : "end";
	                                textOptions = this.textOption(currentseries, chartObj, textsize, pyrX, pyrY, pointIndex, textAnchor, position.positionX, position.positionY);
	                                if (chartObj.model.centerX < position.positionX)
	                                    remainingWidth = legendPosition == "right" ? (axisWidth - textOptions.x - space - legendBounds) : (axisWidth - textOptions.x - space);
	                                else
	                                    remainingWidth = legendPosition == "left" ? (textOptions.x - space - legendBounds) : (textOptions.x - space);
	                                datalabelText = this.trimfunction(datalabelText, remainingWidth, datalabelLength, "...", axisWidth, font);
	                                delete point.smartLabelPosition;
	                            }
	                        }                       
	                    
	                }                

                    else if ((type == "pyramid" || type == "funnel")) {
                            position = seriesType.updateSmartLabel(sender, currentseries, pointIndex, point);
                            positionX = position.xPos;
                            positionY = position.yPos;
                            connectorDirection = position.connectorDirection;
                            textOptions = position.textOptionsacc;
                            datalabelText = point.drawTextacc;
                            
                            if ((connectorDirection != "" || linetype == "bezier") && datalabelText != "") {

                                if (legendPosition == "right") {
                                    remainingWidth = (enableCanvasRendering) ? (axisWidth - textOptions.x) : (axisWidth - textOptions.x - chartObj.pyrX);
                                }
                                else if (legendPosition == "left") {
                                    remainingWidth = enableCanvasRendering ? (axisWidth - textOptions.x) : (axisWidth - textOptions.x - chartObj.pyrX);
                                }
                                else if (legendPosition == "top") {
                                    remainingWidth = (enableCanvasRendering) ? (axisWidth - textOptions.x) : axisWidth - textOptions.x - chartObj.pyrX;
                                }
                                else
                                    remainingWidth = (enableCanvasRendering) ? (axisWidth - textOptions.x) : axisWidth - textOptions.x - chartObj.pyrX;
                                if (remainingWidth < datalabelLength)
                                    datalabelText = this.trimfunction(datalabelText, remainingWidth, datalabelLength, "...", axisWidth, font);
                            }
                        }
                        var datalabelShape = ej.EjSvgRender.utils._measureText(datalabelText, null, font);
                        if (type == "pie" || type == "doughnut")
                            var width = ((_labelPosition == 'inside' || _labelPosition == 'insidenooverlap') && !symbolPos.isInterSected && connectorDirection == "") ? 0 : (positionX < chartObj.model.centerX) ? -datalabelShape.width : datalabelShape.width;
                        else if (type == "pyramid" || type == "funnel")
                            var width = (point._labelPlacement == "insidenooverlap") ? 0 : datalabelShape.width;
                        if (type == "pie" || type == "doughnut") {
                            positionX = pyrX + symbolPos.positionX - (margin.left) / 2 + (margin.right) / 2 + (width / 2);
                            positionY = pyrY + symbolPos.positionY - (margin.top) / 2 + (margin.bottom) / 2;
                        }
                        else {
                            positionX = positionX - (margin.left) / 2 + (margin.right) / 2 + (width / 2);
                            positionY = positionY - (margin.top) / 2 + (margin.bottom) / 2;
                        }
                    if(datalabelText!= ""){
                        seriesType.dataLabelSymbol(0, currentseries, pointIndex, positionX, positionY, datalabelShape.width, textHeight, symbolName, chartObj);
						if (linetype == "bezier" && (position.bezierPath) != "")
                                sender.svgRenderer.drawPolyline(position.bezierPath, sender.gSeriesConnectorEle);
                    }
                    else
                        connectorDirection = "";
	                chartObj.svgRenderer.drawText(textOptions, datalabelText, chartObj.gSeriesTextEle);
	                var datalabelSize = ej.EjSvgRender.utils._measureText(datalabelText, datalabelText.length, font);
	                var untrimmedSize = ej.EjSvgRender.utils._measureText(commonEventArgs.data.text, null, font);
	                var bounds;
	                var minX, minY, maxX, maxY;
	                if (type == "pie" || type == "doughnut") {	                    
	                    if (labelPosition == "inside" && position.connectorDirection=="") {
	                        minX = textOptions.x - datalabelShape.width / 2;
	                        maxX = textOptions.x + datalabelShape.width / 2;
	                        minY = textOptions.y - datalabelShape.height / 2;
	                        maxY = textOptions.y + datalabelShape.height / 2;
	                    }
	                    else {
	                        if (chartObj.model.centerX < textOptions.x) {
	                            minX = textOptions.x;
	                            maxX = textOptions.x + datalabelShape.width;
	                        }
	                        else {
	                            minX = textOptions.x - datalabelShape.width;
	                            maxX = textOptions.x;
	                        }
	                        minY = textOptions.y - datalabelShape.height/2;
	                        maxY = textOptions.y + datalabelShape.height/3;
	                    }
	                    bounds = { centerX: chartObj.model.centerX, minX: minX, minY: minY, maxX: maxX, maxY: maxY, width: datalabelSize.width, height: datalabelSize.height, labelPosition: labelPosition };
	                }

	                  if ((type == "funnel" || type == "pyramid")) {
                            legendWidth = legendPosition == "left" ? legendWidth : 0;
                            var legendHeight = 0;
                            if (connectorDirection != "" || linetype == "bezier") {
                                minX = textOptions.x - legendWidth;
                                maxX = textOptions.x + datalabelShape.width - legendWidth;
                                minY = textOptions.y - datalabelShape.height;
                                maxY = textOptions.y;
                            }
                        if(!enableCanvasRendering)
                            bounds = { minX: minX + chartObj.pyrX + legendWidth, maxX: maxX + chartObj.pyrX + legendWidth, minY: minY + chartObj.pyrY + legendHeight, maxY: maxY + chartObj.pyrY + legendHeight, width: datalabelSize.width, height: datalabelSize.height, labelPosition: labelPosition };
	                    else
                            bounds = { minX: minX + legendWidth, maxX: maxX + legendWidth, minY: minY + legendHeight, maxY: maxY + legendHeight, width: datalabelSize.width, height: datalabelSize.height, labelPosition: labelPosition };
	                }	                
	                accDataLabelRegion = { bounds: bounds, trimmedText: datalabelText, text: commonEventArgs.data.text, type:currentseries.type, font:currentseries.marker.dataLabel.font };
	                chartObj.accDataLabelRegion.type = currentseries.type;
	                chartObj.accDataLabelRegion.connectorDirection = connectorDirection;
	                if (chartObj.accDataLabelRegion.length < currentseries.visiblePoints.length)
	                    chartObj.accDataLabelRegion.push(accDataLabelRegion);	               
	                if (((!ej.util.isNullOrUndefined(pointIndex) && currentseries.explode) && (!enableCanvasRendering)))
	                    chartObj.accDataLabelRegion.splice(pointIndex, 1, accDataLabelRegion);
	                
	            }
	            else {
	                seriesType.dataLabelSymbol(0, currentseries, pointIndex, positionX, positionY, textWidth, textHeight, symbolName, chartObj);
	                chartObj.svgRenderer.drawText(textOptions, commonEventArgs.data.text, chartObj.gSeriesTextEle);
	            }
	        }
			if ((type == "pyramid" || type == "funnel") && currentseries.marker.dataLabel.template && currentseries._enableSmartLabels) {
                    position = seriesType.updateSmartLabel(sender, currentseries, pointIndex, point);
                    positionX = position.xPos;
                    positionY = position.yPos;
                    connectorDirection = position.connectorDirection;

	        }
			
        pointColor = jQuery.type(chartObj.model.pointColors[pointIndex]) == "array" ? chartObj.model.pointColors[pointIndex][0].color : chartObj.model.pointColors[pointIndex];
        var translate = [];
        translate[0] = pyrX;
        translate[1] = pyrY;
        if ((currentseries.marker.dataLabel.connectorLine.type.toLowerCase() != 'bezier' || chartObj.vmlRendering || (type == "pyramid" || type == "funnel")) && (connectorDirection != '')) {
            if (connectorDirection) {
                var connectorOptions = {
                    'id': chartObj.svgObject.id + "_connectorLine" + pointIndex,
                    'stroke': (currentseries.marker.dataLabel.connectorLine.color) ? currentseries.marker.dataLabel.connectorLine.color : pointColor,
                    'stroke-width': currentseries.marker.dataLabel.connectorLine.width,
                    'd': connectorDirection
                };
                chartObj.svgRenderer.drawPath(connectorOptions, chartObj.gSeriesConnectorEle, translate);
            }
        }
    }

	},

	_overlapLegend: function (textOptions, datalabelText, chart) {

	    var svgHeight = chart.svgHeight,
            svgWidth = chart.svgWidth,
	        legendBounds = chart.model.LegendActualBounds,
	        elementSpacing = chart.model.elementSpacing,
	        textHeight = ej.EjSvgRender.utils._measureText((datalabelText, null, textOptions)).height,
	        dataLabelEndPos = textOptions.y + textHeight,
	        dataLabelTopPos = textOptions.y,
	        legendPosition = chart.model.legend.position.toLowerCase(),
            title = chart.model.title,
            subTitle = title.subTitle,
            margin = chart.model.margin,
            borderSize = chart.model.border.width,
            modelsubTitleHeight = subTitle.text == "" ? 0 : ej.EjSvgRender.utils._measureText(subTitle.text, svgWidth - margin.left - margin.right, subTitle.font).height + elementSpacing,
            titleLocation = chart.model._titleLocation ? chart.model._titleLocation.Y : 0 + modelsubTitleHeight,
	        text = datalabelText, legendPosY;
	        
        
	    if (legendPosition == 'bottom') {
	        legendPosY = svgHeight - chart.model.LegendActualBounds.Height - elementSpacing;
            text = dataLabelEndPos > legendPosY ? '' : text;
	    } else {
	        legendPosY = titleLocation == 0 ? borderSize + elementSpacing : titleLocation + elementSpacing;
            text = dataLabelTopPos < (legendPosY + legendBounds.Height) ? '' : text;
        }
	    if (dataLabelEndPos > svgHeight)
	        text = '';
	    return text;

	},

    drawBezierSegment: function (controlpoints, currentseries, pointIndex, sender) {
	    //currentseries.points[pointIndex].marker = currentseries.points[pointIndex].marker ? currentseries.points[pointIndex].marker : {};
	    //var marker = $.extend(true, {}, currentseries.marker, currentseries.points[pointIndex].marker);
	    var polyLine = ej.EjSvgRender.utils._getStringBuilder();
	    var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	    for (var i = 0; i <= 16; i++) {
	        var t = i / 16;
	        var points = seriesType.GetBezierPoint(t, controlpoints, 0, controlpoints.length, currentseries);
	        polyLine.append(points.X + "," + points.Y + " ");
	    }
	    var connectorOptions = {
	        'id': sender.svgObject.id + "_bezierLine" + pointIndex,
	        'stroke': (currentseries.marker.dataLabel.connectorLine.color) ? currentseries.marker.dataLabel.connectorLine.color : sender.model.pointColors[pointIndex],
	        'stroke-width': currentseries.marker.dataLabel.connectorLine.width,
            'fill':'none',
	        'points': polyLine.toString()
	    };
		if (!currentseries._enableSmartLabels)
	       sender.svgRenderer.drawPolyline(connectorOptions, sender.gSeriesConnectorEle);
		else
	       return connectorOptions;
	},
	GetBezierPoint:function(t, controlPoints, index, count, currentseries)
	{
	    var seriesType = new ej.seriesTypes[currentseries.type.toLowerCase()]();
	    if (count == 1)
	        return controlPoints[index];
	    var p0 = seriesType.GetBezierPoint(t, controlPoints, index, count - 1, currentseries);
	    var p1 = seriesType.GetBezierPoint(t, controlPoints, index + 1, count - 1, currentseries);
	    var x = (p0.X) ? p0.X : p0.x;
	    var y = (p0.Y) ? p0.Y : p0.y;
	    var x1 = (p1.X) ? p1.X : p1.x;
	    var y1 = (p1.Y) ? p1.Y : p1.y;
	    var X = (1 - t) * x+ t * x1;
	    var Y = (1 - t) * y + t * y1;
	    if (p0.x)
	        return { x: X, y: Y }
	    else
	        return { X: X, Y: Y };;
	    


       // return new Point((1 - t) * p0.X + t * p1.X, (1 - t) * p0.Y + t * p1.Y);
    },
	
    drawLabelTemplate: function (series, point, pointIndex, location, sender) {
        // method for data label template
        if(sender)
        this.chartObj = sender;
	    var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
        var areaType = this.chartObj.model.AreaType;
        var currentSeries = this.chartObj.model.series[seriesIndex];
		var xPosition = 0, yPosition = 0;
        var chartContainer = this.chartObj._id;
        var marker = $.extend(true, {}, series.marker, point.marker);
        var style = { 'interior': marker.dataLabel.fill, 'opacity': marker.dataLabel.opacity, 'borderColor': marker.dataLabel.border.color, 'borderWidth': marker.dataLabel.border.width };
        var color, width, height;
		var type = this.chartObj.model.chartRegions[pointIndex] ? this.chartObj.model.chartRegions[pointIndex].type : "";
        if (areaType !="none") {

            if ($('#template_group_' + chartContainer).length != 0)
                var templateContainer = $('#template_group_' + chartContainer);
            else
                templateContainer = $("<div></div>").attr('id', "template_group_" + chartContainer);

            templateContainer.css('position', 'relative').css('z-index', 1000);
			if ($("#" + marker.dataLabel.template).length == 0) // To check the specified div is in DOM
                return;
            else
            var cloneNode = $("#" + marker.dataLabel.template).clone();
            $(cloneNode).attr("id", marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            var $cloneNode = $(cloneNode);
            $cloneNode.css("position", "absolute");

            var pointX = point.x; 
            point.dataLabeltemplate = true;	
            point.id = marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer;			
            if (series.xAxis._valueType.toLowerCase() == "datetime")
                point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis._intervalType) : series.xAxis.labelFormat)));
            if (series.xAxis._valueType.toLowerCase() == "category")
                point.x = ej.EjSvgRender.utils._getLabelContent(pointIndex, series.xAxis);

            point.count = 1;
            var data = { series: series, point: point };
            $cloneNode.html($cloneNode.html().parseTemplate(data));

            point.x = pointX;
            var display = (areaType == "cartesianaxes" || !series.enableAnimation || (series.type.toLowerCase() == "pyramid" || series.type.toLowerCase() == "funnel")) ? "block" : "none";
            $cloneNode.css("display", display).appendTo($(templateContainer));
			//template_group_container Div appended in chartcontainer_container Div 
            $(templateContainer).appendTo('#' + 'chartContainer_' + chartContainer);
            var areaBound = this.chartObj.model.m_AreaBounds;
            if (areaType == "cartesianaxes" && !this.chartObj.model.enable3D) {
                xPosition = areaBound.X + (this.chartObj.model.requireInvertedAxes ? currentSeries.yAxis.plotOffset : currentSeries.xAxis.plotOffset);
                yPosition = currentSeries.yAxis.y;
            }
            width = $cloneNode.width() / 2;
            height = $cloneNode.height();
            if (ej.util.isNullOrUndefined(style) && style.interior)
                color = style.interior;
            else
                color= this.chartObj.model.seriesColors[seriesIndex];
        }
        else {
            if (ej.util.isNullOrUndefined(style) && style.interior)
                color = style.interior;
            else
                color = this.chartObj.model.pointColors[point.index];
           width = this.chartObj.model.centerX > location.X ? point.size.width : 0;
            $cloneNode = $('#template_group_' + chartContainer).find("#" + marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer);
            point.id = marker.dataLabel.template + '_' + seriesIndex + '_' + pointIndex + '_' + chartContainer;
		     height = point.size.height / 2;
        }
        
        if (currentSeries.type == 'hilo' || currentSeries.type == 'hiloopenclose' || currentSeries.type == 'candle')
            color = currentSeries.points[pointIndex]._hiloFill;
            color = jQuery.type(color) == "array" ? color[0].color : color;
			
       if (currentSeries.type.indexOf("bar")!=-1) {
            var left = location.X + xPosition - width;
            var top = location.Y + areaBound.Y - height + currentSeries.xAxis.plotOffset;
        } else {
        var left =location.X + xPosition - width;
        var top = yPosition + location.Y - height;
       }
       var backgroundColor = $cloneNode[0].style.backgroundColor;
       $cloneNode.css("left", left).css("top", top).css("background-color", (backgroundColor != '' ? backgroundColor : color)).css("display", "block").css("cursor", "default");
	    
		if(this.chartObj.model.AreaType !="cartesianaxes"){
          point.xPos = left - this.chartObj.pyrX;
          point.yPos = top - this.chartObj.pyrY;
		  point.textOptionsacc = { x: left - this.chartObj.pyrX, y: top - this.chartObj.pyrY };
		}
		else{
		    if (point.y == point.low) {
		        point.xPosLow = left;
		        point.yPosLow = top;
		        point.textOptionsLow = { x: left, y: top };
		        point.dataLabelLow = { textX: left, textY: top, x: left, y: top };
		    }
		    else {
		        point.xPos = left;
		        point.yPos = top;
		        point.seriesIndex = seriesIndex;
		        point.textOptions = { x: left, y: top };
		        point.dataLabel = { textX: left, textY: top, x: left, y: top };
		        point.margin = { top: 0, bottom: 0, left: 0, right: 0 };
		    }
		}
        point.height = $cloneNode.outerHeight();
        point.width = $cloneNode.outerWidth();
        if (areaType == "cartesianaxes") {
            if ((left > xPosition + areaBound.Width || top + height > areaBound.Y + areaBound.Height) || (this.chartObj.zoomed && (left < xPosition || top < yPosition || left > xPosition + areaBound.Width || top > yPosition + areaBound.Height)))
                $cloneNode.remove();
        }
    },
	
	animateLabelTemplate : function(options) {
	// method to animate data label template
        var seriesIndex = $.inArray(options, this.chartObj.model._visibleSeries);
        var length = options.points.length;
        var secondsPerPoint = 1000;
        
        for (var i = 0; i < length; i++) {            
            var ele = options.marker.dataLabel.template + '_' + seriesIndex + '_' + i + '_' + this.chartObj._id;
            ele = $('#' + ele);
            ele.css("display", "none");
            ele.delay(secondsPerPoint).fadeIn(300);
			}
    },
	
	textPosition: function (series, seriesIndex, point, textOffset, type, x, y, chartRegionIndex, index) {
	    var marker = $.extend(true, {}, series.marker, point.marker),
            dataLabel = marker.dataLabel,
            horizontalTextAlignment = dataLabel.horizontalTextAlignment.toLowerCase(),
            verticalTextAlignment = dataLabel.verticalTextAlignment.toLowerCase(),
            textPosition = dataLabel.textPosition.toLowerCase(),
            visiblePointLength = series.points.length,
            seriesPointIndex = chartRegionIndex,
            lineHeight = 10,
            margin = dataLabel.margin;

	    var seriesType = (type.indexOf("bar") != -1 || (type.indexOf("column") != -1 && type!="rangecolumn")) ? (this.chartObj.model.requireInvertedAxes) ? "bar" : "column" : type;
	    if (seriesType == 'column' || seriesType == 'stackingcolumn' || seriesType == 'stackingcolumn100') {
			var bounds = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds,
                pointHeight = bounds.Height,
                pointWidth = bounds.Width;

	        if (textPosition == "bottom") {
	            if ((point.y > 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
                   y = y + pointHeight - margin.bottom;
               else 
                   y = textOffset.height + margin.bottom;
	        } else if (textPosition == "middle") {
               if ((point.y > 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
                   y += pointHeight / 2 + (textOffset.height) / 4;
               else  
                   y -= pointHeight / 2;
           }
           else {
               if ((point.y > 0 && series.yAxis.isInversed) || (point.y < 0 && !series.yAxis.isInversed))
                   y += (lineHeight + (2 * textOffset.height) / 2);
               else
                   y -= (lineHeight + (2 * textOffset.height) / 2);

           }

           if (horizontalTextAlignment && horizontalTextAlignment == "far")
               x += pointWidth / 2;
            else if (horizontalTextAlignment && horizontalTextAlignment == "near")
               x -= pointWidth / 2;

           if (verticalTextAlignment && verticalTextAlignment == "near") {
               if (textPosition == "bottom")
                   y += textOffset.height / 3;
               else
                   y += (textOffset.height + lineHeight);
           }
           else if (verticalTextAlignment && verticalTextAlignment == "far") {
               if (textPosition == "bottom")
                   y -= textOffset.height / 3;
               else
                   y -= (lineHeight + (textOffset.height) / 2);
           }
        }

	    else if (seriesType.indexOf("bar") != -1) {
	        var pointWidth = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds.Width;
	        var pointHeight = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds.Height;
			y = y+ textOffset.height / 4;
		  	if (textPosition == "bottom") {
		  	    if ((point.y > 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
		  	        x = margin.left + textOffset.width / 4;
		  	    else
		  	        x += (pointWidth - textOffset.height - margin.right);
		  	} else if (textPosition == "middle") {
			      if ((point.y > 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
			          x -= pointWidth / 2 + textOffset.width / 2;
			      else
			          x += pointWidth / 2;

		  	} else {
		  	    if ((point.y > 0 && !series.yAxis.isInversed) || (point.y < 0 && series.yAxis.isInversed))
		  	        x += (lineHeight + (2 * textOffset.width) / 2);
                else
		  	        x -= (lineHeight + (2 * textOffset.width) / 2);
           }

		  	if (horizontalTextAlignment && horizontalTextAlignment == "far") {
		  	    if (textPosition == "bottom")
		  	        x += textOffset.width - textOffset.width / 4;
		  	    else
		  	        x += (lineHeight + textOffset.width);
		  	}
		  	else if (horizontalTextAlignment && horizontalTextAlignment == "near") {
		  	    if (textPosition == "bottom")
		  	        x -= textOffset.width / 4;
		  	    else
		  	        x -= (textOffset.width + lineHeight);
		  	}
		  	else // for center
		  	    x += textOffset.width / 4;
				
            if (verticalTextAlignment && verticalTextAlignment == "near")
                y += pointHeight / 2;
            else if (verticalTextAlignment && verticalTextAlignment == "far")
                y -= pointHeight / 2;
	    }
	    else if (seriesType == "rangecolumn" ) {
	         var bounds = this.chartObj.model.chartRegions[seriesPointIndex].Region.Bounds;
	         var height = 0;
	         var width = 0;
	         if (point.high != point.y) {
	            height = bounds.Height;
	     	 	width = bounds.Width;
	          }
	       	 if (this.chartObj.model.requireInvertedAxes) {
	             if (textPosition == "top")
                     x = x - width + (lineHeight + (2 * textOffset.width) / 2);
                 else if (textPosition == "bottom")
                     x = x - width - (2 * textOffset.width + lineHeight);
	             else if (textPosition == "middle")
                     x = x - width;
             }
	         else{
                 if(textPosition == "top")
                     y = y + height - (lineHeight + (2 * textOffset.height) / 2);
	        
                 else if(textPosition == "bottom")
                     y = y + height + (2 * textOffset.height + lineHeight);
	             else if (textPosition == "middle")
                     y = y + height;
	         }
	         if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "far")
                  x += bounds.Width / 2;
	         else if (horizontalTextAlignment && horizontalTextAlignment.toLowerCase() == "near")
                  x -= bounds.Width / 2;	    
            if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "near")
                  y += (textOffset.height + lineHeight);
	         else if (verticalTextAlignment && verticalTextAlignment.toLowerCase() == "far")
                  y -= (lineHeight + (textOffset.height) / 2);
	    
        }
        else if (type == "polar" || type == "radar") {
                y += textOffset.height / 4;
            if (textPosition == "bottom")
                y += (textOffset.height);
            else if (textPosition == "top")
                y -=  (textOffset.height);

            if (horizontalTextAlignment && horizontalTextAlignment == "far")
                x += textOffset.width;
            else if (horizontalTextAlignment && horizontalTextAlignment == "near")
                x -= (textOffset.width);

            if (verticalTextAlignment && verticalTextAlignment == "near")
                y += (textOffset.height)/2;
            else if (verticalTextAlignment && verticalTextAlignment == "far")
                y -= ((textOffset.height)/2);
        }
        else {
            if ((seriesType == "rangearea") && (point.high != point.y)) {
                var length = this.chartObj.model.chartRegions[seriesIndex][index].region.length;
                y = this.chartObj.model.chartRegions[seriesIndex][index].region[length - 1].Y;
				x = this.chartObj.model.chartRegions[seriesIndex][index].region[length - 1].X;
            }    
            if (this.chartObj.model.requireInvertedAxes) {
                if (textPosition == "top")
                    x += (2 * textOffset.width + lineHeight);
                else if (textPosition == "bottom")
                    x -= (lineHeight + (2 * textOffset.width) / 2);
            }
            else {
                if (textPosition == "bottom")
                    y += (2 * textOffset.height + lineHeight);
                else if (textPosition == "top")
                    y -= (lineHeight + (2 * textOffset.height) / 2);
            }
            if (horizontalTextAlignment && horizontalTextAlignment == "far")
                x += textOffset.width + lineHeight;
            else if (horizontalTextAlignment && horizontalTextAlignment == "near")
                x -= ((textOffset.width) + lineHeight);

            if (verticalTextAlignment && verticalTextAlignment == "near")
                y += (textOffset.height + lineHeight);
            else if (verticalTextAlignment && verticalTextAlignment == "far")
                y -= (lineHeight + (textOffset.height) / 2);
        }
        var location = { x: x, y: y };
        return location;
    },
	



    changeCrossHairSymbol: function(element, track, ptIndex, serIndex, series) {
        if (!ej.util.isNullOrUndefined(track)) {
            var trackMarker = {};
            element = $.extend(true, {}, series.marker, element);
            trackMarker = $.extend(true, {}, this.chartObj.model.crosshair.marker, trackMarker);

            if (ej.util.isNullOrUndefined(trackMarker.shape)) {
                trackMarker.shape = element.shape;
            }
            
            if ((!this.chartObj.model.crosshair.visible || ej.util.isNullOrUndefined(trackMarker.fill)) && series.type.toLowerCase() != "bubble" && series.type.toLowerCase() != "scatter") {
                if(trackMarker.shape.toLowerCase()=="cross" ||trackMarker.shape.toLowerCase()=="horizline"||trackMarker.shape.toLowerCase()=="vertline" )   
                    trackMarker.fill = jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex];         
                else
                    trackMarker.fill = element.border.color;
            }
            if (series.type.toLowerCase() == "bubble" && !this.chartObj.model.enableCanvasRendering) {
                var box = $(this.chartObj.gSeriesEle).find("#" + this.chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex)[0].getBoundingClientRect();
                trackMarker.size.width = (box.right - box.left) / 2 + 15;
                trackMarker.size.height = (box.bottom - box.top) / 2 + 15;
            } else {
                if (trackMarker.size.width <= element.size.width)
                    trackMarker.size.width = element.size.width + 2;
                if (trackMarker.size.height <= element.size.height)
                    trackMarker.size.height = element.size.height + 2;
               
                if (ej.util.isNullOrUndefined(trackMarker.border.color) || !this.chartObj.model.crosshair.visible){              
                 if (!ej.util.isNullOrUndefined(element.fill))
                    trackMarker.border.color = jQuery.type(element.fill) == "array" ? element.fill._gradientStop[0].color : element.fill;
                else
                  trackMarker.border.color = jQuery.type(this.chartObj.model.seriesColors[serIndex]) == "array" ? this.chartObj.model.seriesColors[serIndex][0].color : this.chartObj.model.seriesColors[serIndex];
                }
            }
        }
        series._trackMarker = trackMarker;
        return trackMarker;


    },
	
    dataLabelSymbol: function (seriesIndex, series, pointIndex, x, y, width, height, symbolName, sender, group) {
        var chartObj = sender;
        if (symbolName == 'None')
            return;
            var point = series.points[pointIndex];
            var location = { startX: x, startY: y };
            if(!sender.model.enable3D)
                var symbolEle = group ? group : chartObj.gDataLabelEle;
				symbolEle = chartObj.model.enableCanvasRendering ? null : symbolEle;
            var element = $.extend(true, {}, series.marker, point.marker);
            var gradientName = "symbol";
            var style = { 'interior': element.dataLabel.fill, 'opacity': element.dataLabel.opacity, 'borderColor': element.dataLabel.border.color, 'borderWidth': element.dataLabel.border.width };
            var options = {
                'style': style,
                'gradientName': gradientName,
                'symbolEle': symbolEle,
                'width': width,
                'height': height,
                'point': point,
                'location': location,
                'seriesIndex': seriesIndex,
                'pointIndex': pointIndex,
                'element': element,
                'symbolName': symbolName,
                'id': chartObj.svgObject.id + '_dataLabel_series'+ seriesIndex + "_" + pointIndex
            };
            this.drawSymbolStyle(options, chartObj);
    },

    drawSymbolStyle: function (options, sender) {
        var chartObj = sender;
        var borderColor, borderWidth, opacity;
        var style = options.style, gradientName = options.gradientName, symbolEle = options.symbolEle, width = options.width;
        var height = options.height, point = options.point, location = options.location, seriesIndex = options.seriesIndex, visibility=options.visibility;
        var pointIndex = options.pointIndex, element = options.element, symbolName = options.symbolName;
        var trackSymbol = options.trackSymbol;
        var isCanvas = chartObj.model.enableCanvasRendering;
        var isPolar = chartObj.model.AreaType == 'polaraxes';
        if (style != null) {
            if (style.borderColor != "")
                borderColor = style.borderColor ? style.borderColor: "transparent";

            opacity = style.opacity;
            borderWidth = style.borderWidth < 0 ? 0 : style.borderWidth;
        }
        var colors = null;
        if (style.interior) {
            colors = ej.util.isNullOrUndefined(style.interior._gradientStop) ? style.interior : style.interior._gradientStop;
        }
        else {       
            if (point.fill)
                colors = jQuery.type(point.fill) == "array" ? point.fill[0].color : point.fill;
            else {
                var pointColors = chartObj.model.pointColors;
                colors = (chartObj.model.AreaType != "none") ? chartObj.model.seriesColors[seriesIndex] : jQuery.type(pointColors[pointIndex]) == "array" ? pointColors[pointIndex][0].color : pointColors[pointIndex];
            }
        }
       chartObj.symbolColorName = chartObj.svgRenderer.createGradientElement(gradientName + seriesIndex, colors, 0, 0, 0, $(chartObj.svgObject).height(), symbolEle);

        var symbolStyle = { ShapeSize: {width:width, height:height}, Style: { BorderColor: borderColor, BorderWidth: borderWidth, Opacity: opacity, Visibility:visibility, Color: chartObj.symbolColorName}, PointIndex: pointIndex, SeriesIndex: seriesIndex, Imageurl: element.imageUrl, Image: element.image, ID:options.id };

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { location: location, style: symbolStyle };
        chartObj._trigger("symbolRendering", commonEventArgs);
        if (!commonEventArgs.cancel) {
            if (isCanvas){
				var series = chartObj.model.series[seriesIndex];
				//Added for Marker location while setting plotoffset value to axis
                  location.startY = (!chartObj.model.requireInvertedAxes) ? (location.startY + (series.yAxis.y ? (series.yAxis.y - chartObj.canvasY) : 0)) : location.startY;
              
                commonEventArgs.data.location.startY = location.startY;
                if (ej.util.isNullOrUndefined(trackSymbol)) { // to draw marker/datalabel
				    if (chartObj.model.AreaType == "cartesianaxes") {
					    symbolEle = chartObj.svgRenderer.ctx;
					    chartObj.svgRenderer.ctx.save();
				        chartObj.svgRenderer.ctx.beginPath();
                        if (!chartObj.model.requireInvertedAxes)
                            chartObj.svgRenderer.ctx.rect(series.xAxis.x, series.yAxis.y, series.xAxis.width, series.yAxis.height);
					    else
                            chartObj.svgRenderer.ctx.rect(series.yAxis.x, series.xAxis.y, series.yAxis.width, series.xAxis.height);
                        chartObj.svgRenderer.ctx.clip();
					    ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, symbolEle);
					    chartObj.svgRenderer.ctx.restore();
				    }
				    else{
				        if (isCanvas && isPolar) {
				            chartObj.svgRenderer.ctx.save();
				            chartObj.svgRenderer.ctx.beginPath();
				            chartObj.svgRenderer.ctx.arc(chartObj.model.centerX, chartObj.model.centerY, chartObj.model.Radius, 0, 2 * Math.PI, false);
				            chartObj.svgRenderer.ctx.clip();
				        }
				        ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, symbolEle);
				        if (isCanvas && isPolar)
				            chartObj.svgRenderer.ctx.restore();
                    }
                   
                }
                else {
                        var ptIndex = commonEventArgs.data.style.PointIndex;
                        var serIndex = commonEventArgs.data.style.SeriesIndex;
                        if (!chartObj.model.series[serIndex].highlightSettings.enable) {
                            if ($('#' +chartObj._id+'_selection_'+serIndex+'_'+ptIndex+'_canvas').length == 0)
                                chartObj.svgRenderer.trackSymbol(commonEventArgs.data.style, commonEventArgs.data.location, symbolName, trackSymbol, chartObj);
                        }
                    }
                }
                else {
                    var ptIndex = commonEventArgs.data.style.PointIndex;
                    var serIndex = commonEventArgs.data.style.SeriesIndex;
                    var series = chartObj.model.series[serIndex];
                    var type = series.type;
                    var className;
                    if (series.highlightSettings.enable && trackSymbol && !chartObj.model.crosshair.visible) {
                        var style = $.extend(true, commonEventArgs.data.style);
                        var highlight =series.highlightSettings;
                        style.Style.Color = (highlight.pattern.toLowerCase() == "none" || highlight.pattern == "") ? (highlight.color != "" ? highlight.color : style.Style.Color) : "url(#" + highlight.pattern.toLowerCase() + '_Highlight_2D'+")";
                        style.Style.Opacity = highlight.opacity;
                        style.Style.BorderColor = highlight.border.color;
                        style.Style.BorderWidth = highlight.border.width;
                        // checked condition for is already selected or not
                        if(type == 'bubble')
                            className = $('#' + this.chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex).attr('class');
                        else
                            className = $('#' + this.chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex+'_symbol').attr('class');
                        if (className != "SelectionStyleseries" + serIndex)
                            ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, style, chartObj, (chartObj.model.enable3D) ? sender.chart3D : symbolEle);
                    }
                    else {
                        if (type == 'scatter')
                            className = $('#' + chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex + '_symbol').attr('class');
                        else
                            className = $('#' + chartObj.svgObject.id + '_Series' + serIndex + '_Point' + ptIndex).attr('class');
                        if (className != "SelectionStyleseries" + serIndex)
                            ej.EjSvgRender.chartSymbol["_draw" + symbolName](commonEventArgs.data.location, commonEventArgs.data.style, chartObj, (chartObj.model.enable3D) ? sender.chart3D : symbolEle);
                    }
                }
            }

        if (!sender.chart3D && chartObj.model.AreaType == "none" && $(symbolEle).children().not("defs").length > chartObj.model._visibleSeries[seriesIndex].visiblePoints.length) {
            $(symbolEle.childNodes[options.pointIndex]).replaceWith($(symbolEle.childNodes[$(symbolEle.childNodes).length - 1]));
        }

    },
	
    drawSymbol: function (seriesIndex,series, pointIndex, x, y,chart,trackSymbol) {

        if (ej.util.isNullOrUndefined(this.chartObj)) this.chartObj = chart;


        var point = series.points[pointIndex];
       

             var symbolName = "None";
             var tracker = this.changeCrossHairSymbol(point.marker ? point.marker : series.marker, trackSymbol, pointIndex, seriesIndex, series);
             var element = tracker ? tracker : $.extend(true, {}, series.marker, point.marker);

             $.each(this.chartObj.model.symbolShape, function(name) {
                 if (element.shape.toLowerCase() == name.toLowerCase())
                     symbolName = name;
             });
                if (symbolName == "None")
                    return;
                var location = this.chartObj.model.AreaType == "cartesianaxes" ? { startX: x + this.chartObj.canvasX, startY: y + this.chartObj.canvasY } : { startX: x, startY: y };
                var style = {'interior' : element.fill, 'opacity': element.opacity, 'borderColor': element.border.color, 'borderWidth' :element.border.width };

                var symbolEle = (trackSymbol) ? this.chartObj.gTrackerEle : this.chartObj.gSymbolGroupEle[seriesIndex];
                var id = (trackSymbol) ? (this.chartObj.svgObject.id + '_trackSymbol_' + seriesIndex + "_" + pointIndex) : (this.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex + '_symbol');
                var gradientName = trackSymbol ? "TrackSymbol" : "symbol";
                var width = element.size.width;
                var height = element.size.height;
                var options = {
                    'style': style,
                    'gradientName': gradientName,
                    'symbolEle': symbolEle,
                    'width': width,
                    'height': height,
                    'point': point,
                    'location': location,
                    'seriesIndex': seriesIndex,
                    'pointIndex': pointIndex,
                    'element': element,
                    'visibility': (point.visible) ? 'visible' : 'hidden',
                    'symbolName': symbolName,
                    'trackSymbol' : trackSymbol,
                    'id': id
                };
                this.drawSymbolStyle(options, this.chartObj, trackSymbol);

                if (series.type.toLowerCase() == "scatter") {
                    var areaBoundsX = series.xAxis.x;
                    var areaBoundsY = series.yAxis.y;
                    if (this.chartObj.model.enableCanvasRendering) {
                        areaBoundsX = 0;
                        areaBoundsY = 0;
                    }
                    var bounds = { X: areaBoundsX + location.startX - (height / 2), Y: areaBoundsY + location.startY - (width / 2), Height: height, Width: width };
                    ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, pointIndex);
                }
    },

    chartAreaType: "cartesianAxes",
    requireInvertedAxes: false,
    stackingSeries: false,
    hiloTypes:false
    
};


function ejExtendClass(parent, members) {
    var object = function () { };
    object.prototype = new parent();
    $.extend(object.prototype, members);
    return object;
}

ej.ejLineSeries = ejExtendClass(ej.EjSeriesRender);

ej.seriesTypes.line = ej.ejLineSeries;


ej.ejStepLineSeries = ejExtendClass(ej.EjSeriesRender, {

    draw: function (chart, options) {
        var chartObj = this.chartObj = chart;
        var lDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setLineSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);

        var firstPoint = this.getFirstVisiblePoint(visiblePoints);
        var secondPoint;
        var point1 = ej.EjSvgRender.utils._getPoint(firstPoint, currentseries);
        sb.append("M " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY));
        var point2;
        for (var i = 0; i < visiblePoints.length - 1; i++) {
            if (visiblePoints[i + 1].visible) {
                secondPoint = { xValue: visiblePoints[i + 1].xValue, YValues: [visiblePoints[i + 1].y] };
                point2 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                var step = { xValue: visiblePoints[i + 1].xValue, YValues: [firstPoint.YValues[0]] };
                var stepPoint = ej.EjSvgRender.utils._getPoint(step, currentseries);
                sb.append(" L " + (stepPoint.X + chartObj.canvasX) + " " + (stepPoint.Y + chartObj.canvasY) + " L " + (point2.X + chartObj.canvasX) + " " + (point2.Y + chartObj.canvasY));

                firstPoint = secondPoint;
                point1 = point2;
            }
        }
        lDirection = sb.toString();

        this._drawLinePath(currentseries, style, lDirection);

        this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

      
       
       
    },
    getFirstVisiblePoint: function (points) {
        for (var i = 0; i < points.length; i++)
            if (points[i].visible)
                return points[i];
    },
    doAnimation: function (series) {

        this._doLineAnimation(series);
    }

});

ej.ejStepAreaSeries = ejExtendClass(ej.EjSeriesRender, {

    draw: function (chart, options) {
        this.chartObj = chart;
		var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
        var aDirection;
        var sb = ej.EjSvgRender.utils._getStringBuilder();
        var currentseries = options;
        var style = this.setAreaSeriesStyle(currentseries);
        if (currentseries.sorting)
            currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
        var visiblePoints = this.improveChartPerformance(currentseries);
        var xOffset = 0;
        var firstPoint = null;
        var secondPoint;
        if (currentseries.xAxis._valueType.toLowerCase() == "category" && currentseries.xAxis.labelPlacement.toLowerCase() != "onticks")
            xOffset = 0.5;
        var origin = Math.max(options.yAxis.visibleRange.min, 0);
        var startPoint = null;
        var start = true;
        var chartObj = this.chartObj;
        for (var i = 0; i <= visiblePoints.length; i++) {

            if (i < visiblePoints.length) {
                if (visiblePoints[i].visible) {

                    secondPoint = { xValue: visiblePoints[i].xValue - xOffset, YValues: [visiblePoints[i].y] };
                    point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                    if (!startPoint) {
                        startPoint = { xValue: visiblePoints[i].xValue - xOffset, YValues: [origin] }
                        var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentseries);
                        sb.append("M" + " " + (startLoc.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY));
                    }
                    if (firstPoint != null) {
                        var step = { xValue: secondPoint.xValue, YValues: [firstPoint.YValues[0]] };
                        var stepPoint = ej.EjSvgRender.utils._getPoint(step, currentseries);
                        if (start) {
                            sb.append(" " + "L" + " " + (point2.X + chartObj.canvasX) + " " + (point2.Y + chartObj.canvasY) + " ");
                            start = false;
                        }
                        sb.append("L" + " " + (stepPoint.X + chartObj.canvasX) + " " + ((stepPoint.Y + chartObj.canvasY)) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " ");
                        if ((xOffset == 0 && i == visiblePoints.length - 1) || (i < (visiblePoints.length - 1) && !visiblePoints[i + 1].visible)) {
                            if (xOffset > 0) {
                                secondPoint = { xValue: visiblePoints[i].xValue + xOffset, YValues: [visiblePoints[i].y] };
                                var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                                 if (requireInvertedAxes)
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.Y + chartObj.canvasX) + " " + (startLoc.X + chartObj.canvasY) + " ");
                                 else
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY) + " ");
                            }
                            else
                               if (requireInvertedAxes)
                                    sb.append("L" + " " + (point1.Y + chartObj.canvasX)+ " " + (startLoc.X + chartObj.canvasY) + " ");
                                else
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX)+ " " + (startLoc.Y + chartObj.canvasY) + " ");
                            startPoint = null;
                            firstPoint = null;
                            start = true;
                        }
                    }
                    if (startPoint) {
                        point2 = point1;
                        firstPoint = secondPoint;
                    }
                }
                else {
                    startPoint = null;
                    firstPoint = null;
                    start = true;
                }

              }
             else {
                    if  (visiblePoints[i-1].visible && xOffset > 0) {
                                secondPoint = { xValue: visiblePoints[i - 1].xValue + xOffset, YValues: [visiblePoints[i - 1].y] };
                                var point1 = ej.EjSvgRender.utils._getPoint(secondPoint, currentseries);
                               if (requireInvertedAxes)
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.Y + chartObj.canvasX) + " " + (startLoc.X + chartObj.canvasY) + " ");
                                else
                                    sb.append("L" + " " + (point1.X + chartObj.canvasX) + " " + (point1.Y + chartObj.canvasY) + " " + "L" + " " + (point1.X + chartObj.canvasX) + " " + (startLoc.Y + chartObj.canvasY) + " ");
                       }                           
                }
            
         }
        aDirection = sb.toString();

        this.drawAreaPath(currentseries, style, aDirection);

        this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

       
     
    },

    doAnimation: function (series) {

        this._doLineAnimation(series);
    }

});
ej.seriesTypes.steparea = ej.ejStepAreaSeries;


ej.seriesTypes.stepline = ej.ejStepLineSeries;


 ej.ejColumnSeries = ejExtendClass(ej.EjSeriesRender, {
   
  
      
     draw: function (chart,options)
      { 
         this.chartObj=chart;
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);    
         var origin = Math.max(options.yAxis.visibleRange.min, 0);
        
         var sidebysideinfo = this.getSideBySideInfo(series);    
         var visiblePoints = this._isVisiblePoints(series);
              
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var pointMarker;
         
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex=i;
             var point = visiblePoints[i];
             pointMarker = visiblePoints[i].marker;
           
             var y1 = point.YValues[0];
             var y2 = origin;
             if (point.visible) {
             //calculate sides
             var data = cSer.calculateSides(point, sidebysideinfo);
             var x1 = data.x1;
             var x2 = data.x2;
             
			 var styleOptions= this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

			 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);

             if (rect.X < 0) {
                 rect.Width = rect.Width + rect.X;
                 rect.X = 0;
             }
			 
             rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
             //drawing part
             var xr = Math.min(0, rect.Width);
             var yr = Math.min(0, rect.Height);

             var  bounds;        
             if ((xr == 0 || yr == 0) && rect.Width>0) {                      
                         options = {
                             'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + pointIndex,
                             'x': rect.X + cSer.chartObj.canvasX,
                             'y': rect.Y + cSer.chartObj.canvasY,
                             'width': rect.Width,
                             'height': rect.Height,
                             'fill': styleOptions.interior,
                             'stroke-width': styleOptions.borderWidth,
                             'plot': y1 < 0 ? "negative" : "positive",
                              'opacity': styleOptions.opacity,
                             'stroke': styleOptions.borderColor
                         };

                         cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                         var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
            
                  if (requireInvertedAxes)
                     point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                  else
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        
     },
   
     doAnimation: function (series) {
	   var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
	   var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
	   var gSeriesGroupId = !ej.util.isNullOrUndefined(this.gSeriesGroupEle) ? this.gSeriesGroupEle.id : this.chartObj.gSeriesEle.children[seriesIndex].id;
       var elements = $(this.chartObj.gSeriesEle).find("#" + gSeriesGroupId).children("rect");
	   var dataLabels = $(this.chartObj.gDataLabelEle)[seriesIndex];
	   dataLabels = dataLabels.childNodes;
	   var element, i, j, symbol, dataLabel;
       for (i = 0; i < elements.length; i++) {
           element = elements[i];
           this.animateRect(element, series, requireInvertedAxes);
       }
        var symbolElements = $(this.chartObj.gSymbolGroupEle)[seriesIndex];
		symbolElements = symbolElements.childNodes;
        for (j = 0; j < symbolElements.length; j++) {
            symbol = symbolElements[j];
			if (symbol.tagName != "defs")
              this._doLineSymbol(symbol, 500, 2, series, requireInvertedAxes);
        }

        for (j = 0; j< dataLabels.length; j++) {
		  dataLabel = dataLabels[j];
		  if (dataLabel.tagName != "defs")
		     this._doLineSymbol(dataLabel, 500, 2, series, requireInvertedAxes);
		}		
   },
   isRegion:true

 });
 ej.seriesTypes.column = ej.ejColumnSeries;

 ej.ejStackingColumnSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {
         this.chartObj = chart;
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         //var origin = Math.max(options.yAxis.visibleRange.min, 0);
        
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);

         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];

             var y2 = series.stackedValue.StartValues[i];
             var y1= series.stackedValue.EndValues[i];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions= this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
                 if (rect.X < 0) {
                     rect.Width = rect.Width + rect.X;
                     rect.X = 0;
                 }

                 rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
                 //drawing part
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 var bounds;
                 if ((xr == 0 || yr == 0) && rect.Width > 0) {
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
                         'plot': y1 < 0 ? "negative" : "positive",
                         'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor
                     };

                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                     var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
                
                 if (requireInvertedAxes)
                     point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                 else
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        

     },
     doAnimation: function (series) {
	     var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
	     var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
		 var dataLabels = $(this.chartObj.gDataLabelEle).children().not("defs");
		 var element, symbol, dataLabel;
         for (var i = 0; i < elements.length; i++) {
             element = elements[i];
             this.animateStackingRect(element, series, requireInvertedAxes);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle)[seriesIndex];
		 symbolElements = symbolElements.childNodes;
         for (var j = 0; j < symbolElements.length; j++) {
             symbol = symbolElements[j];
			 if (symbol.tagName != "defs")
               this._doLineSymbol(symbol, 500, 2, series, requireInvertedAxes);
         }
		 for(var k = 0; k< dataLabels.length; k++) {
		    dataLabel = dataLabels[k];
		    if (dataLabel.id.indexOf("series" + seriesIndex) != -1)
		     this._doLineSymbol(dataLabel, 500, 2, series, requireInvertedAxes);
		}
     },
     
     stackingSeries: true,
	  isRegion:true

 });
 ej.seriesTypes.stackingcolumn = ej.ejStackingColumnSeries;
 
 ej.seriesTypes.stackingcolumn100 = ej.ejStackingColumnSeries;

 ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;

 ej.ejRangeColumnSeries = ejExtendClass(ej.EjSeriesRender, {
   
     draw: function (chart, options) {
         this.chartObj = chart;
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
      
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);

         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];

             var y1 = point.YValues[0];
             var y2 = point.YValues[1];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                  
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);
				  
                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
                 if (rect.X < 0) {
                     rect.Width = rect.Width + rect.X;
                     rect.X = 0;
                 }
                 var width=cSer.chartObj.model.m_AreaBounds.Width;
                 rect.Width = ((rect.X + rect.Width) > width) ? width - rect.X : rect.Width;
                 //drawing part
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 var bounds;
                 if (xr == 0 || yr == 0) {
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
                         'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor
                     };

                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle, trans.y);

                     //Add region for each rect
                     var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }
                 //Add rect location details for symbol                
                 if (requireInvertedAxes)
                     point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                 else
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

      

     },

     doAnimation: function (series) {
	     var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
         for (var i = 0; i < elements.length; i++) {
             var element = elements[i];
             this.animateRect(element,series, requireInvertedAxes);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
         for (var j = 0; j < symbolElements.length; j++) {
             var symbol = symbolElements[j];
             this._doLineSymbol(symbol, 500, 2, series, requireInvertedAxes);
         }
     },
     hiloTypes: true,
	  isRegion:true

 });
 ej.seriesTypes.rangecolumn = ej.ejRangeColumnSeries;

 ej.ejStackingBarSeries = ejExtendClass(ej.EjSeriesRender, {
     
     draw: function (chart, options) {
         this.chartObj = chart;
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         //var origin = Math.max(options.yAxis.visibleRange.min, 0);
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);

         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];
             var y1 = series.stackedValue.EndValues[i];
             var y2 =series.stackedValue.StartValues[i];
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
				 if (!requireInvertedAxes) {
				     if (rect.X < 0) {
				         rect.Width = rect.Width + rect.X;
				         rect.X = 0;
				     }
				     rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
				 }
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                 if (xr == 0 || yr == 0) {
                     var bounds;
                     options = {
                         'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                         'x': rect.X + cSer.chartObj.canvasX,
                         'y': rect.Y + cSer.chartObj.canvasY,
                         'width': rect.Width,
                         'height': rect.Height,
                         'fill': styleOptions.interior,
                         'stroke-width': styleOptions.borderWidth,
						 'opacity': styleOptions.opacity,
                         'stroke': styleOptions.borderColor
                     };

                     cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                     var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                     bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                     ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }

               
                 if (!requireInvertedAxes)
                     point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                 else
                     point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
               
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
 

     },
     doAnimation: function (series) {
	     var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
	     var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
		 var dataLabels = $(this.chartObj.gDataLabelEle)[seriesIndex];
		 dataLabels = dataLabels.childNodes;
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
		 var symbol, element, dataLabel;
         for (var i = 0; i < elements.length; i++) {
             element = elements[i];
             this.animateStackingRect(element, series, requireInvertedAxes);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle)[seriesIndex];
		 symbolElements = symbolElements.childNodes;
         for (var j = 0; j < symbolElements.length; j++) {
             symbol = symbolElements[j];
			 if (symbol.tagName != "defs")
               this._doLineSymbol(symbol, 500, 2, series, requireInvertedAxes);
         }
		 for(var k = 0; k< dataLabels.length; k++) {
		    dataLabel = dataLabels[k];
		    if (dataLabel.tagName != "defs")
		     this._doLineSymbol(dataLabel, 500, 2, series, requireInvertedAxes);
		}
     },

     stackingSeries: true,
     requireInvertedAxes: true,
	  isRegion:true

 });
 ej.seriesTypes.stackingbar = ej.ejStackingBarSeries;

 ej.seriesTypes.stackingbar100 = ej.ejStackingBarSeries;

 ej.ejBarSeries = ejExtendClass(ej.EjSeriesRender, {
    
     draw: function (chart, options) {
         this.chartObj = chart;
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
         var origin = Math.max(options.yAxis.visibleRange.min, 0);       
         var style = $.extend(true, {}, this.chartObj.model.seriesStyle, series.style);
         var sidebysideinfo = this.getSideBySideInfo(series);
         var visiblePoints = this._isVisiblePoints(series);
         var colors;
         var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
         
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var cSer = this;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             var point = visiblePoints[i];           
             var y1 = point.YValues[0];
             var y2 = origin;
             if (point.visible) {
                 //calculate sides
                 var data = cSer.calculateSides(point, sidebysideinfo);
                 var x1 = data.x1;
                 var x2 = data.x2;
                 
				 var styleOptions = this.chartObj.setStyle(cSer, series, seriesIndex, pointIndex);

                 var rect = cSer.getRectangle(x1, y1, x2, y2, series, chart);
				 if (!requireInvertedAxes) {
				     if (rect.X < 0)
				     {
				         rect.Width = rect.Width + rect.X;
				         rect.X = 0;
				     }
				     rect.Width = ((rect.X + rect.Width) > cSer.chartObj.model.m_AreaBounds.Width) ? cSer.chartObj.model.m_AreaBounds.Width - rect.X : rect.Width;
				 }
                
                 var xr = Math.min(0, rect.Width);
                 var yr = Math.min(0, rect.Height);

                if (xr == 0 || yr == 0) {
                     var bounds;          
                         options = {
                             'id': cSer.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + pointIndex,
                             'x': rect.X + cSer.chartObj.canvasX,
                             'y': rect.Y + cSer.chartObj.canvasY,
                             'width': rect.Width,
                             'height': rect.Height,
                             'fill': styleOptions.interior,
                             'stroke-width': styleOptions.borderWidth,
                             'plot': y1 < 0 ? "negative" : "positive",
                             'opacity': styleOptions.opacity,
                             'stroke': styleOptions.borderColor
                         };

                         cSer.chartObj.svgRenderer.drawRect(options, cSer.gSeriesGroupEle);

                         var svgXy = ej.EjSvgRender.utils._getSvgXY((rect.X), (rect.Y), series, cSer.chartObj);
                        bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };
                     
                        ej.EjSvgRender.utils._addRegion(cSer.chartObj, bounds, series, point, pointIndex);
                 }


                 
                  if (!requireInvertedAxes)
                    point.symbolLocation = { X: rect.X + (rect.Width / 2), Y: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? ((rect.Y) + (rect.Height)) : (rect.Y) };
                else
                    point.symbolLocation = { X: ((y1 < 0 && !series.yAxis.isInversed) || (series.yAxis.isInversed && y1 > 0)) ? (rect.X) : (rect.X + rect.Width), Y: ((rect.Y) + (rect.Height / 2)) };
                
             }
         }

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

        
     },
     doAnimation: function(series) {
	     var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var elements = $(this.chartObj.gSeriesEle).find("#" + this.gSeriesGroupEle.id).children("rect");
		 var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries);
		 var dataLabels = $(this.chartObj.gDataLabelEle)[seriesIndex];
		 dataLabels= dataLabels.childNodes;
         for (var i = 0; i < elements.length; i++) {
             var element = elements[i];
              this.animateRect(element, series,requireInvertedAxes);
         }
         var symbolElements = $(this.chartObj.gSymbolGroupEle)[seriesIndex];
		 symbolElements = symbolElements.childNodes;
         for (var j = 0; j < symbolElements.length; j++) {
             var symbol = symbolElements[j];
			 if(symbol.tagName != "defs")
               this._doLineSymbol(symbol, 500, 2, series, requireInvertedAxes);
         }   
        for(var j = 0; j< dataLabels.length; j++){
		  var dataLabel = dataLabels[j];
		  if(dataLabel.tagName != "defs")
		     this._doLineSymbol(dataLabel, 500, 2, series, requireInvertedAxes);
		}		
     },
     requireInvertedAxes: true,
	  isRegion:true
 });
 ej.seriesTypes.bar = ej.ejBarSeries;


 ej.ejStackingAreaSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {
         this.chartObj = chart;
         var aDirection;
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);

         currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("xValue")).executeLocal();



         var visiblePoints = this._isVisiblePoints(currentSeries);
     
         var origin = Math.max(options.yAxis.visibleRange.min, currentSeries.stackedValue.StartValues[0]);
         var canvasX = this.chartObj.canvasX;
         var canvasY = this.chartObj.canvasY;

         var startPoint = { xValue: visiblePoints[0].xValue, YValues: [origin] };
         var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
         areasb.append("M" + " " + (startLoc.X + canvasX) + " " + (startLoc.Y + canvasY) + " ");
         //var index = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
         
         for (j = 0; j < currentSeries.stackedValue.EndValues.length; j++) {
             if (visiblePoints[j].visible) {
                    if (visiblePoints.length > j + 1) {
                        currentPoint = { xValue: visiblePoints[j].xValue, YValues: [currentSeries.stackedValue.EndValues[j]] };
                        point1 = visiblePoints[j].location = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);
                        if (visiblePoints[j + 1].visible) {
                            nextpoint = { xValue: visiblePoints[j + 1].xValue, YValues: [currentSeries.stackedValue.EndValues[j + 1]] };
                            point2 = visiblePoints[j + 1].location = ej.EjSvgRender.utils._getPoint(nextpoint, currentSeries);
                        }
                        else {
                            nextpoint = { xValue: visiblePoints[j].xValue, YValues: [origin] };
                            point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentSeries);
                        }

                     if (j > 0 && !visiblePoints[j - 1].visible) {
                         previouspoint = ej.EjSvgRender.utils._getPoint({ xValue: visiblePoints[j - 1].xValue, YValues: [origin] }, currentSeries);
                         areasb.append("L" + " " + (previouspoint.X + canvasX) + " " + ((previouspoint.Y + canvasY)) + " " + " " + "L" + " " + (point1.X + canvasX) + " " + ((previouspoint.Y + canvasY)) + " ");
                     }

                     areasb.append("L" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ");

                 }
             }
         }
                
        
          
         for (var j = currentSeries.stackedValue.StartValues.length - 1; j > 0; j--) {
                 if (visiblePoints[j].visible) {

                     var currentPoint = { xValue: visiblePoints[j].xValue, YValues: [currentSeries.stackedValue.StartValues[j]] };
                     var nextpoint = { xValue: visiblePoints[j - 1].xValue, YValues: [currentSeries.stackedValue.StartValues[j - 1]] };

                     var point1 = ej.EjSvgRender.utils._getPoint(currentPoint, currentSeries);
                     var point2 = ej.EjSvgRender.utils._getPoint(nextpoint, currentSeries);
                 if (this.chartObj.model.enableCanvasRendering)
                     areasb.append("L" + " " + (point1.X + canvasX) + " " + (point1.Y + canvasY) + " " + "L" + " " + (point2.X + canvasX) + " " + (point2.Y + canvasY) + " ");
                 else
                     areasb.append((point1.X) + " " + ((point1.Y)) + " " + (point2.X) + " " + ((point2.Y)) + " ");
                     }
                 }
                

         aDirection = areasb.toString();

         this.drawAreaPath(currentSeries, style, aDirection);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

          


     },

     doAnimation: function (series) {

         this._doLineAnimation(series);
     },
     stackingSeries:true

 });
 ej.seriesTypes.stackingarea = ej.ejStackingAreaSeries;

 ej.seriesTypes.stackingarea100 = ej.ejStackingAreaSeries;

 ej.ejAreaSeries = ejExtendClass(ej.EjSeriesRender, {

  
     draw: function (chart,options) {
         this.chartObj = chart;
         var aDirection;
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);
         var chartObj = this.chartObj;
         currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("xValue")).executeLocal();
      
         var visiblePoints = this._isVisiblePoints(currentSeries);
         var origin =  Math.max(options.yAxis.visibleRange.min, 0);
         
         var startPoint;
         var translate = [];
         translate[0] = chartObj.canvasX;
         translate[1] = chartObj.canvasY;
        
         for (var i = 0; i < visiblePoints.length; i++) {
             if (visiblePoints[i].visible) {
                 if (visiblePoints.length > i + 1) {
                     if (!startPoint) {
                         startPoint = { xValue: visiblePoints[i].xValue, YValues: [origin] }
                         var point1 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append("M" + " " + (point1.X) + " " + ((point1.Y)));
                     }


                     var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);

                     areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));

                     if (!visiblePoints[i + 1].visible) {
                         point = { xValue: visiblePoints[i].xValue, YValues: [origin] }
                         var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                         var point3 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)) + " " + "L" + " " + (point3.X) + " " + ((point3.Y)));
                         startPoint = null;
                     }
                 }
                 else {
                     if (visiblePoints[i - 1] && visiblePoints[i - 1].visible) {
                         var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                         areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));
                     }
                 }
             }         
         }

         var endPoint = { xValue: visiblePoints[visiblePoints.length - 1].xValue, YValues: [origin] };
         var endLoc = ej.EjSvgRender.utils._getPoint(endPoint, currentSeries);
         if (visiblePoints.length > 1)
             areasb.append(" " + "L" + " " + (endLoc.X) + " " + ((endLoc.Y)) + " ");
         
         aDirection = areasb.toString();

         this.drawAreaPath(currentSeries, style, aDirection, translate);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     
       

     },

       doAnimation: function(series) {

           this._doLineAnimation(series);
       }

 });

 ej.seriesTypes.area = ej.ejAreaSeries;

 ej.ejRangeAreaSeries = ejExtendClass(ej.EjSeriesRender, {


     draw: function (chart, options) {
         this.chartObj = chart;
         var aDirection;
         var areasb = ej.EjSvgRender.utils._getStringBuilder();
         var currentSeries = options;
         var style = this.setAreaSeriesStyle(currentSeries);
         var seriesIndex = $.inArray(currentSeries, this.chartObj.model._visibleSeries);
         currentSeries.points = ej.DataManager(currentSeries.points, ej.Query().sortBy("xValue")).executeLocal();

         var visiblePoints = this._isVisiblePoints(currentSeries);        
         var translate = [];
         translate[0] = this.chartObj.canvasX;
         translate[1] = this.chartObj.canvasY;
         var startPoint;
         var internalRegion=[]
         for (var i = 0; i < visiblePoints.length; i++) {
             internalRegion[i] = {Region:{PointIndex:i}, SeriesIndex:seriesIndex, region:[]};
             if (visiblePoints[i].visible) {     
                     if (!startPoint) {
                         var origin = visiblePoints[i].low;
                         startPoint = { xValue: visiblePoints[i].xValue, YValues: [origin] }
                         var point1 = ej.EjSvgRender.utils._getPoint(startPoint, currentSeries);
                         areasb.append("M" + " " + (point1.X) + " " + ((point1.Y)));
                     }
                 var point1 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                 areasb.append(" " + "L" + " " + (point1.X) + " " + ((point1.Y)));
                 internalRegion[i].region.push({ X: point1.X, Y: point1.Y });
                 if (i != 0)
                     internalRegion[i - 1].region.push({ X: point1.X, Y: point1.Y });
                 
                 if( (i +1 ) < visiblePoints.length &&  !visiblePoints[i+1].visible)
                 {
                     for (var j = i; j >=0; j--)
                     {
                         if (visiblePoints[j].visible) {
                             origin = visiblePoints[j].low;
                             var point = { xValue: visiblePoints[j].xValue, YValues: [origin] };
                             var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                             areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)));
                             internalRegion[j].region.push({ X: point2.X, Y: point2.Y });
                             if(j!=0)
                                 internalRegion[j - 1].region.push({ X: point2.X, Y: point2.Y });
                         }
                         else
                             break;
                     }
                     startPoint = null;
                 }

              
             }
         }
         for (var i = visiblePoints.length-1; i >= 0; i--)
         {
             if (visiblePoints[i].visible)
             {
                 origin = visiblePoints[i].low;
                 var point = { xValue: visiblePoints[i].xValue, YValues: [origin] };
                 var point2 = ej.EjSvgRender.utils._getPoint(point, currentSeries);
                 areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)));
                 internalRegion[i].region.push({ X: point2.X, Y: point2.Y });
                 if(i!=0)
                     internalRegion[i - 1].region.push({ X: point2.X, Y: point2.Y });

             }
             else if (visiblePoints.length>i && visiblePoints[i].visible)
             {
                 var point2 = ej.EjSvgRender.utils._getPoint(visiblePoints[i], currentSeries);
                 areasb.append(" " + "L" + " " + (point2.X) + " " + ((point2.Y)));
                 break;
             }                       
             else if (i != (visiblePoints.length - 1))
			  {
                 var count=0;
                 for(k=i-1;k>=0;k--)
                 {
                     if (visiblePoints[k].visible)
                        count++;
                     else if (!visiblePoints[k].visible)
                        break;
                 }
                 i = i - count;
             }
         }     
         aDirection = areasb.toString();
         this.chartObj.model.chartRegions.push(internalRegion);
         this.drawAreaPath(currentSeries, style, aDirection, translate);
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     },

     doAnimation: function (series) {

         this._doLineAnimation(series);
     },
     hiloTypes:true

 });

 ej.seriesTypes.rangearea = ej.ejRangeAreaSeries;

 ej.ejSplineSeries = ejExtendClass(ej.EjSeriesRender, {
    
     draw: function (chart,options) {
         this.chartObj = chart;
         var series = options;
         var spDirection = "";
         var splinesb = ej.EjSvgRender.utils._getStringBuilder();
         var style = this.setLineSeriesStyle(series);
         var yIndex = 0;
         var visiblePoints = this._isVisiblePoints(series);
         var ySpline = this.naturalSpline(visiblePoints);
         var firstPoint = null;
         var secondPoint = null;
         var firstIndex = -1;
		 //Removed spline sorting behavior based on the 'X' points.
         //series.points = ej.DataManager(series.points, ej.Query().sortBy("X")).executeLocal();
         
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             secondPoint = visiblePoints[i];             
             if (secondPoint.visible) {
                 if (firstPoint != null) {
                     var controlPoint1 = null;
                     var controlPoint2 = null;
                     var data = this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                     controlPoint1 = data["controlPoint1"];
                     controlPoint2 = data["controlPoint2"];
                     var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                     var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                     var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                     var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                     var chartObj = this.chartObj;
                     splinesb.append("M" + " " + (pt1.X + chartObj.canvasX) + " " + (pt1.Y + chartObj.canvasY) + " " + "C" + " " + (bpt1.X + chartObj.canvasX) + " " + (bpt1.Y + chartObj.canvasY) + " " + (bpt2.X + chartObj.canvasX) + " " + (bpt2.Y + chartObj.canvasY) + " " + (pt2.X + chartObj.canvasX) + " " + (pt2.Y + chartObj.canvasY) + " ");

                 }
                 firstPoint = secondPoint;
                 firstIndex = pointIndex;
             } else {
                 firstPoint = null;
             }
         }
         
        spDirection = splinesb.toString();
        this._drawLinePath(series, style, spDirection);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
 
       

     },

     doAnimation: function (series) {

          this._doLineAnimation(series);
       }

 });

 ej.seriesTypes.spline = ej.ejSplineSeries;

 ej.ejSplineAreaSeries = ejExtendClass(ej.EjSeriesRender, {


     draw: function (chart, options) {
         this.chartObj = chart;
		 var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         var series = options;
         var spDirection = "";
         var splinesb = ej.EjSvgRender.utils._getStringBuilder();
         var style = this.setAreaSeriesStyle(series);
         var yIndex = 0;
         //Removed splineArea sorting behavior based on the 'X' points.
         series.points = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this._isVisiblePoints(series);
         var ySpline = this.naturalSpline(visiblePoints);
         var firstPoint = null;
         var secondPoint = null;
         var firstIndex = -1;
		
         var origin = Math.max(options.yAxis.visibleRange.min, 0);
         var startPoint = null;
         var start = true;
         var chartObj = this.chartObj;
         var translate = [];
         translate[0] = chartObj.canvasX;
         translate[1] = chartObj.canvasY;
         var count = 0;
         for (var i = 0; i < visiblePoints.length; i++) {
             var pointIndex = i;
             secondPoint = visiblePoints[i];
             if (secondPoint.visible) {
                 if (!startPoint) {
                     startPoint = { xValue: visiblePoints[i].xValue, YValues: [origin] };
                     var startLoc = ej.EjSvgRender.utils._getPoint(startPoint, series);
                    }
                 if (firstPoint != null) {
                     var controlPoint1 = null;
                     var controlPoint2 = null;
                     var data = this.getBezierControlPoints(firstPoint, secondPoint, ySpline[firstIndex], ySpline[pointIndex], yIndex);
                     controlPoint1 = data["controlPoint1"];
                     controlPoint2 = data["controlPoint2"];
                     var pt1 = ej.EjSvgRender.utils._getPoint(firstPoint, series);
                     var pt2 = ej.EjSvgRender.utils._getPoint(secondPoint, series);
                     var bpt1 = ej.EjSvgRender.utils._getPoint(controlPoint1, series);
                     var bpt2 = ej.EjSvgRender.utils._getPoint(controlPoint2, series);
                     if (start) {
                         if (requireInvertedAxes)
                             splinesb.append("M" + " " + (startLoc.X) + " " + (pt1.Y) + " " + "L" + " " + (pt1.X) + " " + ((pt1.Y)) + " ");
                         else
                             splinesb.append("M" + " " + (pt1.X) + " " + (startLoc.Y) + " " + "L" + " " + (pt1.X) + " " + ((pt1.Y)) + " ");
                         start = false;
                     }
                     splinesb.append("C" + " " + (bpt1.X) + " " + (bpt1.Y) + " " + (bpt2.X) + " " + (bpt2.Y) + " " + (pt2.X) + " " + (pt2.Y) + " ");
                     if (pointIndex == visiblePoints.length - 1 || (pointIndex < visiblePoints.length - 1 && !visiblePoints[i + 1].visible)) {
                        if (requireInvertedAxes)
                             splinesb.append("L" + " " + (startLoc.X) + " " +  (pt2.Y));
                         else
                             splinesb.append("L" + " " + (pt2.X) + " " + (startLoc.Y));
                         startPoint = null;
                         start = true;
                     }
                 }
                 firstPoint = secondPoint;
                 firstIndex = pointIndex;
             } else {
                 firstPoint = null;
             }
         }

         spDirection = splinesb.toString();

         this.drawAreaPath(series, style, spDirection, translate);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

    
        

     },
     doAnimation: function (series) {

         this._doLineAnimation(series);
     }

 });

 ej.seriesTypes.splinearea = ej.ejSplineAreaSeries;



 ej.ejScatterSeries = ejExtendClass(ej.EjSeriesRender, {

        
    
     draw: function (chart,options) {
         this.chartObj = chart;
         var series = options;
		 var seriesIndex = $.inArray(options, this.chartObj.model._visibleSeries);
        var visiblePoints = this._isVisiblePoints(series);
		 var visiblePointsLength=visiblePoints.length;
		 var point,point1;
        for (var i = 0; i < visiblePointsLength; i++)
        {
            point = visiblePoints[i];
            var pointIndex = i;
            point1 = ej.EjSvgRender.utils._getPoint(point, options);
            this.drawSymbol(seriesIndex, options, pointIndex, point1.X, point1.Y);
        }

     },

       doAnimation: function(series) {

           var elements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
           for (var i = 0; i < elements.length; i++) {
               var ubound = 20;
               var lbound = 0;
               var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
               var delayInterval = parseInt(randomValue * 50);
               this.animateSymbol(elements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
           }


       }

 });

 ej.seriesTypes.scatter = ej.ejScatterSeries;

 
 ej.ejBubbleSeries = ejExtendClass(ej.EjSeriesRender, {

     createBubbleGroup: function(series) {

         var seriesIndex = $.inArray(series, this.chartObj.model.series);
         var transX = series.xAxis.x;

         var transY = series.yAxis.y;
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + transX + ',' + transY + ')' };

         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

     },
     draw: function (chart,options) {
         this.chartObj = chart;
         var series = options;    
         series.points = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
         var seriesIndex = $.inArray(series, this.chartObj.model._visibleSeries); 
         var visiblePoints = this._isVisiblePoints(series);
         var sizeValueIndex = 1;
		 //bubble responsive
         var bubbleOptions = options.bubbleOptions;
         var temp = 100;
         var areaBounds = chart.model.m_AreaBounds;
         var value = Math.max(areaBounds.Height, areaBounds.Width);
         var minRadius = (bubbleOptions && bubbleOptions.minRadius) ? bubbleOptions.minRadius : 1;
         var maxRadius = (bubbleOptions && bubbleOptions.maxRadius) ? bubbleOptions.maxRadius : 3;
         var percentChange=value/temp;
         var minRadius = minRadius * percentChange;
         var maxRadius = maxRadius * percentChange;
         var radius = maxRadius - minRadius;
         var segmentRadius;
         var maximumSize = Math.max.apply(0,$.map(visiblePoints,function (v) { return v.YValues[sizeValueIndex]; }));
         this.createBubbleGroup(series);
         for (var i = 0; i < visiblePoints.length; i++) {
             var point = visiblePoints[i];
             var pointIndex= i;
             if (point.visible) {
                 segmentRadius = minRadius + radius * Math.abs(point.YValues[sizeValueIndex] / maximumSize);
                 var location = ej.EjSvgRender.utils._getPoint(point, series);
                 
				 var styleOptions = this.chartObj.setStyle(this, series, seriesIndex, pointIndex);
				 
                 var options = {
                     'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + '_Point' + pointIndex,
                     'cx': location.X + this.chartObj.canvasX,
                     'cy': (location.Y) + this.chartObj.canvasY,
                     'r': segmentRadius,
                     'fill': styleOptions.interior,
                     'stroke-width': styleOptions.borderWidth,
                     'opacity': styleOptions.opacity,
                     'stroke': styleOptions.borderColor
                 };

                 point.radius = segmentRadius; // radius is stored to draw tracksymbol for bubble in canvas
                 
                 this.chartObj.svgRenderer.drawCircle(options, this.gSeriesGroupEle);
				 
			 

                 var cx = location.X;
                 var cy = location.Y;
                 var r = segmentRadius;


                 var valwidth, x, y, valheight;
                 x = ((cx - r) + series.xAxis.x);
                 y = ((cy - r) + series.yAxis.y);

                 valheight = 2 * r;
                 valwidth = 2 * r;

                 var bounds = { X: x, Y: y, Width: valwidth, Height: valheight };
                 ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, series, null, i);
             }
         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

         
         
     },
    
     
     doAnimation: function(series) {
         var elements = $(this.gSeriesGroupEle).children().not("defs");
         var symbolElements = $(this.chartObj.gSymbolGroupEle).children().not("defs");
		 var dataLableElements = $(this.chartObj.gDataLabelEle).children().not("defs");
		 var connectorElements = $(this.chartObj.gConnectorEle).children().not("defs")
		 var ubound = 20;
         var lbound = 0;
         var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
         var delayInterval = parseInt(randomValue * 50);
         for (var i = 0; i < elements.length; i++) {
             this.animateSymbol(elements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
         }
		 for (var i = 0; i < symbolElements.length; i++) {
             this.animateSymbol(symbolElements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
         }
		 for (var i = 0; i < dataLableElements.length; i++) {
             this.animateSymbol(dataLableElements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
         }
		 for (var i = 0; i < connectorElements.length; i++) {
             this.animateSymbol(connectorElements[i], delayInterval, series, this.chartObj.model.requireInvertedAxes);
         }
     }
 });

ej.seriesTypes.bubble = ej.ejBubbleSeries;
 ej.ejhiloSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         //DrawHiloGraph      
        
         var currentseries = options;
          currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries);
         var point = null;
         
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, requireInvertedAxes);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
       

         for (var i = 0; i < visiblePoints.length; i++) {
             point = visiblePoints[i];
			 var pointIndex= i;
             if (point.visible) {
                
                 var lDirection = "";
                    if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]))
                     continue;
					 
                    var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex);

                    var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex, seriesIndex);
              
				 
                 var lowvalue = {}, highvalue = {};
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                 } else {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                 }
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue   , currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 lDirection = "M" + " " + (point1.X + this.chartObj.canvasX) + " " + ((point1.Y + this.chartObj.canvasY)) + " " + "L" + " " + (point2.X + this.chartObj.canvasX) + " " + ((point2.Y + this.chartObj.canvasY)) + " ";
                 var pointbounds = { point1:point1, point2:point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
             }
         }
        
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
       
         

      
     },
     
     doAnimation: function (series) {

         var elements = $(this.gSeriesGroupEle).children().not("defs");
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
        }     
     },
     hiloTypes: true
 });

 ej.seriesTypes.hilo = ej.ejhiloSeries;
 
 //Draw HiloOpenClose
 ej.ejhiloopencloseSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         //DrawHiloGraph      

         var currentseries = options;
         currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries);
         var point = null;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, requireInvertedAxes);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         
         var interior;

     


         for (var i = 0; i < visiblePoints.length; i++) {
		    var pointIndex = i;
             point = visiblePoints[i];
             if (point.visible) {

                 var lDirection = "";
                 var drawOpenWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "open") ? true : false);
                 var drawCloseWing = (ej.util.isNullOrUndefined(currentseries.drawMode) ? true : (currentseries.drawMode.toLowerCase() == "both" || currentseries.drawMode.toLowerCase() == "close") ? true : false);

                
                 var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                 if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]) || ej.util.isNullOrUndefined(point.YValues[2]) || ej.util.isNullOrUndefined(point.YValues[3]))
                     continue;
                  
                    
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     
                 } else {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                    
                 }
				 currentseries.fill = currentseries.isFill ? currentseries.fill : null;
                 if (point.YValues[2] < point.YValues[3]) 
                     currentseries.bearFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? ((currentseries.fill) ? currentseries.fill : "#339933") : currentseries.bearFillColor;
                 
                 else 
                     currentseries.bullFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? ((currentseries.fill) ? currentseries.fill : "#E51400") : currentseries.bullFillColor;
                    
				 
                 var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                 var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex,seriesIndex, interior);
                 

				 if (point.fill) {
                     interior = jQuery.type(point.interior) == "array" ? point.fill[0] : point.fill;
                 }
                 else {
                      point._hiloFill = interior;
                 }
				 var canvasX = this.chartObj.canvasX;
				 var canvasY = this.chartObj.canvasY;
                 
                 //Draw open points
                 if (drawOpenWing) {

                     var startOpenValue = { xValue: openvalue.xValue + sidebysideinfo.Median, YValues: openvalue.YValues };
                     var pto1 = ej.EjSvgRender.utils._getPoint(startOpenValue, currentseries);
                     var endOpenValue = { xValue: openvalue.xValue + sidebysideinfo.Start, YValues: openvalue.YValues };
                     var pto2 = ej.EjSvgRender.utils._getPoint(endOpenValue, currentseries);
                      lDirection = "M" + " " + (pto1.X + canvasX) + " " + ((pto1.Y + canvasY)) + " " + "L" + " " + (pto2.X + canvasX) + " " + ((pto2.Y + canvasY)) + " ";
                     var openbounds = { point1: pto1, point2: pto2 };
                     this._drawHiloPath(currentseries, styleOptions,pathInterior,  lDirection, i, openbounds);
                     lDirection = "";

                 }
                 //Draw close points
                 if (drawCloseWing) {
                     var startCloseValue = { xValue: closevalue.xValue + sidebysideinfo.Median, YValues: closevalue.YValues };
                     var ptc1 = ej.EjSvgRender.utils._getPoint(startCloseValue, currentseries);
                     var endCloseValue = { xValue: closevalue.xValue + sidebysideinfo.End, YValues: closevalue.YValues };
                     var ptc2 = ej.EjSvgRender.utils._getPoint(endCloseValue, currentseries);
                     lDirection = "M" + " " + (ptc1.X + canvasX) + " " + ((ptc1.Y + canvasY)) + " " + "L" + " " + (ptc2.X + canvasX) + " " + ((ptc2.Y + canvasY)) + " ";
                     var pointbounds = { point1: ptc1, point2: ptc2 };
                     this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                     lDirection = "";

                 }
                 // Draw High low points
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 lDirection = "M" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ";
                 var bounds = { point1: point1, point2: point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, bounds);
             }

         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     
     },
      
     doAnimation: function (series) {

        var elements = $(this.gSeriesGroupEle).children().not("defs");
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            this._doLineSymbol(element, secondsPerPoint, i, series, this.chartObj.model.requireInvertedAxes);
        }
     },
     hiloTypes: true

 });

 ej.seriesTypes.hiloopenclose = ej.ejhiloopencloseSeries;
 
 //Draw candle
ej.ejCandleSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         var requireInvertedAxes = this.chartObj.model.requireInvertedAxes;
         //DrawCandleGraph      

         var currentseries = options;
         currentseries.points = ej.DataManager(currentseries.points, ej.Query().sortBy("xValue")).executeLocal();
         var visiblePoints = this._isVisiblePoints(currentseries);
         var sidebysideinfo = this.getSideBySideInfo(currentseries);
         var point = null;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var trans = ej.EjSvgRender.utils._getTransform(currentseries.xAxis, currentseries.yAxis, requireInvertedAxes);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);
         var interior;

         for (var i = 0; i < visiblePoints.length; i++) {
             point = visiblePoints[i];
			  var pointIndex= i;
             if (point.visible) {

                 var lDirection = "";
               
                 var lowvalue = {}, highvalue = {}, openvalue = {}, closevalue = {};
                 if (ej.util.isNullOrUndefined(point.YValues[0]) || ej.util.isNullOrUndefined(point.YValues[1]) || ej.util.isNullOrUndefined(point.YValues[2]) ||ej.util.isNullOrUndefined(point.YValues[3]))
                     continue;
                
                 if (point.YValues[0] < point.YValues[1]) {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     
                 } else {
                     lowvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[1]] };
                     highvalue = { xValue: point.xValue + sidebysideinfo.Median, YValues: [point.YValues[0]] };
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     
                 }      
                 if (point.YValues[2] < point.YValues[3]) {
                     
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     currentseries.bearFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bearFillColor)) ? "#339933" : currentseries.bearFillColor;
                     
                 } else {
                    
                     openvalue = { xValue: point.xValue, YValues: [point.YValues[3]] };
                     closevalue = { xValue: point.xValue, YValues: [point.YValues[2]] };
                     currentseries.bullFillColor = interior = (ej.util.isNullOrUndefined(currentseries.bullFillColor)) ? "#E51400" : currentseries.bullFillColor;
                    
                 }
                 
                 var styleOptions = this.chartObj.setStyle(this, currentseries, seriesIndex, pointIndex, interior);

                 var pathInterior = this.chartObj.setHiloStyle(currentseries, pointIndex,seriesIndex, interior);
				 
				 if (point.style && point.style.interior) {
                     interior = jQuery.type(point.style.interior) == "array" ? point.style.interior[0] : point.style.interior;
                 }
               
                 else {
                     point._hiloFill = pathInterior;
                 }

                 //Draw High low points
                 var point1 = ej.EjSvgRender.utils._getPoint(lowvalue, currentseries);
                 var point2 = ej.EjSvgRender.utils._getPoint(highvalue, currentseries);
                 var canvasX = this.chartObj.canvasX;
                 var canvasY = this.chartObj.canvasY;
                 lDirection = "M" + " " + (point1.X + canvasX) + " " + ((point1.Y + canvasY)) + " " + "L" + " " + (point2.X + canvasX) + " " + ((point2.Y + canvasY)) + " ";
                 var pointbounds = { point1: point1, point2: point2 };
                 this._drawHiloPath(currentseries, styleOptions, pathInterior, lDirection, i, pointbounds);
                 //Draw open and close points
              
                 var startRect = { xValue: closevalue.xValue + sidebysideinfo.Start, YValues: closevalue.YValues };
                     var orginRect = ej.EjSvgRender.utils._getPoint(startRect, currentseries);

                     var rectSize = { xValue: openvalue.xValue + sidebysideinfo.End, YValues: openvalue.YValues };
                     var sizeRect = ej.EjSvgRender.utils._getPoint(rectSize, currentseries);
                     var rect = ej.EjSvgRender.utils._correctRect(orginRect.X, orginRect.Y, sizeRect.X, sizeRect.Y);
                  
                     var candleOptions = {
                        'id': this.chartObj.svgObject.id + '_Series' + seriesIndex+ '_Point' + i,
                        'x': rect.X + this.chartObj.canvasX,
                        'y': rect.Y + this.chartObj.canvasY,
                        'width': rect.Width,
                        'height': rect.Height,
                        'fill': styleOptions.interior,
                        'stroke-width': styleOptions.borderWidth,
                         'stroke': styleOptions.borderColor
                     };

                    this.chartObj.svgRenderer.drawRect(candleOptions, this.gSeriesGroupEle);

                    var svgXy = ej.EjSvgRender.utils._getSvgXY(rect.X, rect.Y, currentseries, this.chartObj);
                    var bounds = { X: svgXy.X, Y: svgXy.Y, Width: rect.Width, Height: rect.Height };

                    ej.EjSvgRender.utils._addRegion(this.chartObj, bounds, currentseries, point, i);
                                    
             }
             
         }
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
        

     },
     
     doAnimation: function (series) {
      var elements = $(this.gSeriesGroupEle).children().not("defs");
        var secondsPerPoint = 2000 / elements.length;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i];
            this._doLineSymbol(element, secondsPerPoint, i,series, this.chartObj.model.requireInvertedAxes);
        }
        
     },
     hiloTypes: true
 });
 ej.seriesTypes.candle = ej.ejCandleSeries;

 ej.ejPieSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var currentseries = options;
         var internalRegion = [];
         var coefficient = currentseries.pieCoefficient >= 0 ? (currentseries.pieCoefficient <= 1 ? currentseries.pieCoefficient : 1) : 0;
         this.chartObj.model.bounds = [];
        
         var size = this.calculatingSliceAngle(currentseries);                  

        this.chartObj.model.Radius = 0.5 * coefficient * Math.min(size.width, size.height);
        var min= Math.min(size.width, size.height);
        if (min < 0)
        {
            return min;
        }
        var totalDegree=Math.abs(currentseries.endAngle-currentseries.startAngle);
        if (totalDegree < 270) 
            this.pieDoughnutCenter(currentseries);
         var text;
         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         var visibility = (currentseries.enableAnimation && !currentseries._animatedSeries) ? 'hidden' : 'visible';

        
         var txtOptions = { 'id': this.chartObj.svgObject.id + '_textGroup' + '_' + seriesIndex, 'visibility': visibility };
         this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);
             
         var dataLabelOptions = { 'id': this.chartObj.svgObject.id + '_DataLabel_' + seriesIndex , 'visibility': visibility };
		 this.chartObj.gDataLabelEle = this.chartObj.svgRenderer.createGroup(dataLabelOptions);
		 
         var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
         this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

         var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
         this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);
             
                 
         for (var j = 0; j < currentseries.points.length; j++) {
             pointIndex = j;
			  if (isNaN(currentseries.points[j].startAngle)) continue;
             var point = currentseries.points[j];
             if (point.visible) {
                 var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries);
                 var sliceXY = result.Direction.split(" "); 
             var styleOptions = this.chartObj.setStyle(this, currentseries, 0,  pointIndex);
			 if (currentseries.startAngle < currentseries.endAngle) // for canvas rendering with modified start and end angle
                     var counterClockWise = false;
                 else
                     counterClockWise = true;
             options = {
                 'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'd': result.Direction,
                     'opacity': styleOptions.opacity,
                     'radius':this.chartObj.model.Radius,
                     'start': point.startAngle - 1.57,
                     'end': point.endAngle - 1.57,
                     'pointIndex': pointIndex,
					 'counterClockWise': counterClockWise,
                     'innerR': 0,
					  'cx': result.centerX,
                     'cy':result.centerY,
                     'x': sliceXY[12],
                     'y': sliceXY[13]
                 };
                 this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);

                 region = { PointIndex: pointIndex, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] };
                 internalRegion.push(region);

             }
         }

         var seriesData = { Radius: this.chartObj.model.Radius, CenterX: this.chartObj.model.centerX, CenterY: this.chartObj.model.centerY };
         var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);
     },
     doAnimation: function (series, seriesType) {
         var chartObj = this.chartObj;
         var seriesRender = this;
         var angle = (series.points[0]) ? series.points[0].startAngle : 0;
         $.each(series.visiblePoints, function (pointIndex, point) {
             var startAngle = point.startAngle;
             var endAngle = point.endAngle;
             chartObj.model._radius = chartObj.model.Radius;
             
             point.startAngle = 0;
             point.endAngle =point.startAngle;
             chartObj.model.Radius = 0;
             var piePath = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[pointIndex];
             $(piePath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(
                 { StartAngle: startAngle, EndAngle: endAngle, pieRadius: chartObj.model._radius },

                 {
                     duration: 1200, queue: false, step: function (now, fx) {

                         if (fx.prop.toString() === "StartAngle") {
                             point.startAngle = now;
                             point.endAngle = point.startAngle;
                         }
                         else if (fx.prop.toString() == "pieRadius" && chartObj.model) {
                             chartObj.model.Radius = now;
                         } 
                         else { 
                             point.endAngle = now;
                         }
                         if (chartObj.model) {
                             var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series);
                             chartObj.svgRenderer._setAttr($(piePath), { "d": result.Direction });
                             chartObj.model.Radius = chartObj.model._radius;
                         }
                     },
                     complete: function () {
                         if (pointIndex == series.visiblePoints.length - 1) {                             
                                 chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle.id), { "visibility": "visible" });
                                 if (chartObj.gSymbolGroupEle)
                                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle.id), { "visibility": "visible" });
								 if (chartObj.gDataLabelEle)
                                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gDataLabelEle.id), { "visibility": "visible" });
                                 if (series.marker.dataLabel.template) {
                                     var elements = $(chartObj.element[0].childNodes[0].childNodes);
                                     for (var i = 0; i < elements.length; i++)
                                         $(elements[i]).css('display', 'block');
                                 }
                                 if (chartObj.gSeriesConnectorEle)
                                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesConnectorEle.id), { "visibility": "visible" });
                             chartObj.model.AnimationComplete = true;
                             series.AnimationComplete = true;
                             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonEventArgs.data = { series: series };
                             chartObj._trigger("animationComplete", commonEventArgs);
                         }
                     }
                 });
           });
     },
     chartAreaType : "None"

 });

 ej.seriesTypes.pie = ej.ejPieSeries;

 ej.ejDoughnutSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {
         this.chartObj = chart;
         var currentseries = options;
         var internalRegion = [];

         this.chartObj.model.bounds = [];
         var coefficient = currentseries.doughnutCoefficient >= 0 ? (currentseries.doughnutCoefficient <= 1 ? currentseries.doughnutCoefficient : 1) : 0;
         var doughnutSize = currentseries.doughnutSize >= 0 ? (currentseries.doughnutSize <= 1 ? currentseries.doughnutSize : 1) : 0;
         var size = this.calculatingSliceAngle(currentseries);
        
         this.chartObj.model.Radius = 0.5 * doughnutSize * Math.min(size.width, size.height);
         var min = Math.min(size.width, size.height);
         if (min < 0) {
             return min;
         }
         var totalDegree = Math.abs(currentseries.endAngle - currentseries.startAngle);
         if (totalDegree < 270)//for centering the chart
            this.pieDoughnutCenter(currentseries);
         this.chartObj.model.dRadius = coefficient * this.chartObj.model.Radius;


         var numberToFixed = ej.util.isNullOrUndefined(this.chartObj.model.roundingPlaces) ? 2 : this.chartObj.model.roundingPlaces;

         var seriesIndex = $.inArray(currentseries, this.chartObj.model._visibleSeries);
         var serOptions = { 'id': this.chartObj.svgObject.id + '_SeriesGroup' + '_' + seriesIndex };
         this.gSeriesGroupEle = this.chartObj.svgRenderer.createGroup(serOptions);

         var visibility = (currentseries.enableAnimation && !currentseries._animatedSeries) ? 'hidden' : 'visible';

          
         var txtOptions = { 'id': this.chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex, 'visibility': visibility };
         this.chartObj.gSeriesTextEle = this.chartObj.svgRenderer.createGroup(txtOptions);

         var symbolOptions = { 'id': this.chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'visibility': visibility };
         this.chartObj.gSymbolGroupEle = this.chartObj.svgRenderer.createGroup(symbolOptions);

         var connectorOptions = { 'id': this.chartObj.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'visibility': visibility };
         this.chartObj.gSeriesConnectorEle = this.chartObj.svgRenderer.createGroup(connectorOptions);

         var dataLabelOptions = { 'id': this.chartObj.svgObject.id + '_DataLabel_' + seriesIndex, 'visibility': visibility };
		 this.chartObj.gDataLabelEle = this.chartObj.svgRenderer.createGroup(dataLabelOptions);
		 
         if ((doughnutSize > 0) && (coefficient >= 0) && (doughnutSize <= 1) && (coefficient <= 1)) {
           
         for (var j = 0; j < currentseries.points.length; j++) {
             var point = currentseries.points[j];
             var pointIndex = j;
             if (point.visible) {
                 var explodeX = 0;
                 var explodeY = 0;
                 var result = this._calculateArcData(this.chartObj, pointIndex, point, currentseries);
				 //for canvas slice explode
                 if ((pointIndex == currentseries.explodeIndex || currentseries.explodeAll) && !this.chartObj.vmlRendering) {
                     var chartStartAngle = -.5 * Math.PI; 
                     var startAngle = point.startAngle + chartStartAngle;
                     var endAngle = point.endAngle + chartStartAngle - 0.000001;
                     var midAngle = (startAngle + endAngle) / 2;
                     explodeX = this.chartObj.model.centerX + Math.cos(midAngle) * currentseries.explodeOffset;
                     explodeY = this.chartObj.model.centerY + Math.sin(midAngle) * currentseries.explodeOffset;
                 }

             var styleOptions = this.chartObj.setStyle(this, currentseries, 0, pointIndex);

                 if (currentseries.startAngle < currentseries.endAngle) // for canvas rendering with modified start and end angle
                     var counterClockWise = false;
                 else
                     counterClockWise = true;
                 
             options = {
                 'id': this.chartObj.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'opacity': styleOptions.opacity,
                     'stroke-linejoin': currentseries.lineJoin,
                     'd': result.Direction,
                     'start': point.startAngle - 1.57,
                     'end': point.endAngle - 1.57,
                     'pointIndex': pointIndex,
                     'radius': this.chartObj.model.Radius,
                     'innerR': this.chartObj.model.dRadius,
                     'counterClockWise': counterClockWise,
                     'x': explodeX !=0 ? explodeX : this.chartObj.model.centerX,
                     'y': explodeY !=0 ? explodeY : this.chartObj.model.centerY
                 };
                 this.chartObj.svgRenderer.drawPath(options, this.gSeriesGroupEle);


                 region = { PointIndex: j, StartAngle: point.startAngle, EndAngle: point.endAngle, StartX: this.chartObj.model.startX[pointIndex], StartY: this.chartObj.model.startY[pointIndex] };
                 internalRegion.push(region);

             }
         }
}
         var seriesData = { Radius: this.chartObj.model.Radius, DRadius: this.chartObj.model.dRadius, CenterX: this.chartObj.model.centerX, CenterY: this.chartObj.model.centerY };
         var region = { Series: currentseries, SeriesData: seriesData, Region: internalRegion, SeriesIndex: $.inArray(currentseries, this.chartObj.model._visibleSeries) };
         this.chartObj.model.chartRegions.push(region);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

     },

    
     doAnimation: function (series, seriesType) {
         var chartObj = this.chartObj;
         var seriesRender = this;
         var angle = (series.points[0]) ? series.points[0].startAngle : 0;
         $.each(series.visiblePoints, function (pointIndex, point) {
             var startAngle = point.startAngle;
             var endAngle = point.endAngle;

             point.startAngle = 0;
             point.endAngle = point.startAngle;           
         var doughPath = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[pointIndex];
             $(doughPath).each(function () { this.StartAngle = angle, this.EndAngle = angle; }).animate(

             { StartAngle: startAngle, EndAngle: endAngle },

             {
             duration: 1200, queue: false, step: function (now, fx) {
                 if (fx.prop.toString() === "StartAngle") {
                     point.startAngle = now;
                     point.endAngle = point.startAngle;
                 }
                 else                     
                     point.endAngle = now;
                 if (chartObj.model) {
                     var result = seriesRender._calculateArcData(chartObj, pointIndex, point, series);
                     chartObj.svgRenderer._setAttr($(doughPath), { "d": result.Direction });
                 }
             },
             complete: function () {
                 if (pointIndex == series.visiblePoints.length - 1 && chartObj.model) {
                         chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle.id), { "visibility": "visible" });
                         if (chartObj.gSymbolGroupEle)
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle.id), { "visibility": "visible" });
						 if (chartObj.gDataLabelEle)
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gDataLabelEle.id), { "visibility": "visible" });
                         if (series.marker.dataLabel.template) {
                             var elements = $(chartObj.element[0].childNodes[0].childNodes);
                             for (var i = 0; i < elements.length; i++)
                                 $(elements[i]).css('display', 'block');
                         }
                         if (chartObj.gSeriesConnectorEle)
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesConnectorEle.id), { "visibility": "visible" });
             
                     chartObj.model.AnimationComplete = true;
                     series.AnimationComplete = true;
                     var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                     commonEventArgs.data = { series: series };
                     chartObj._trigger("animationComplete", commonEventArgs);

                 }
             }
         });             
         });
     },

     chartAreaType:"None"
 });

 ej.seriesTypes.doughnut = ej.ejDoughnutSeries;

 ej.ejPyramidSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {

         this.chartObj = chart;
         var chartModel = chart.model,
             point,
             seriesRender = this,
             legendActualBounds = chartModel.LegendActualBounds,
             currentseries = options,
             visiblePoints = this._pieVisiblePoints(currentseries),
             internalRegion = [],
             translate = [],
             legend = chartModel.legend,
             legendPosition = legend.position.toLowerCase(),
             legendWidth = legend.border.width,
             textSize = 0,
             text,
             borderSize = chartModel.border.width,
             measureTitle = (chartModel.title.text) ? ej.EjSvgRender.utils._measureText(chartModel.title.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.font) : 0,
             xOffset = (chartModel.margin.left + chartModel.elementSpacing + (borderSize * 2) + chartModel.elementSpacing + ((legendPosition === "right" || legendPosition === "left") ? legendActualBounds.Width + (2 * legendWidth) : 0)),
             yOffset = (((chartModel.title.text) ? (chartModel._titleLocation.Y + measureTitle.height) : chartModel.elementSpacing) + (borderSize * 2) + chartModel.margin.top + chartModel.elementSpacing + chartModel.elementSpacing + ((legendPosition === "top" || legendPosition === "bottom") ? legendActualBounds.Height + (2 * legendWidth) + chartModel.elementSpacing : 0));

         chartModel.chartRegions = [];
         chartModel.sumofYValues = 0;

         if (currentseries.labelPosition.toLowerCase() === 'outside') {
             for (var i = 0; i < visiblePoints.length; i++) {
                 var textWidth = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(visiblePoints[i].text) ? visiblePoints[i].y : visiblePoints[i].text), null, currentseries.marker.dataLabel.font).width;
                 if (textSize < textWidth)
                     textSize = textWidth;
             }
             chartModel.textSize = textSize;
         }
         if (currentseries._enableSmartLabels) {
             chartModel.actualWidth = ($(chart.svgObject).width() - xOffset) * 0.8; //calculate a pyramid width when smartlabel is enabled
             chartModel.resWidth = ($(chart.svgObject).width() - xOffset - chartModel.actualWidth) / 2;
         }
         else {
             chartModel.actualWidth = $(chart.svgObject).width() - xOffset - textSize;
             chartModel.resWidth = 0;
         }
         chartModel.actualHeight = $(chart.svgObject).height() - yOffset;
         var pyrX = chart.pyrX = ((legendPosition === "left") ? (legendActualBounds.Width + (2 * legendWidth)) : 0) + chartModel.elementSpacing + chartModel.margin.left;
         var pyrY = chart.pyrY = ((legendPosition === "top") ? (legendActualBounds.Height + (2 * legendWidth)) : 0) + ((chartModel.title.text) ? (chartModel._titleLocation.Y + measureTitle.height) : (chartModel.margin.top + chartModel.elementSpacing));
         translate[0] = pyrX;
         translate[1] = pyrY;
         for (var j = 0; j < visiblePoints.length; j++) {
             chartModel.sumofYValues += visiblePoints[j].YValues[0];
         }
         var dataLabelOptions = { 'id': chart.svgObject.id + '_DataLabel_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gDataLabelEle = chart.svgRenderer.createGroup(dataLabelOptions);


         var numberToFixed = ej.util.isNullOrUndefined(chartModel.roundingPlaces) ? 2 : chartModel.roundingPlaces;

         var seriesIndex = $.inArray(currentseries, chartModel._visibleSeries);
         var serOptions = { 'id': chart.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         this.gSeriesGroupEle = chart.svgRenderer.createGroup(serOptions);


         var txtOptions = { 'id': chart.svgObject.id + '_textGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gSeriesTextEle = chart.svgRenderer.createGroup(txtOptions);

         var symbolOptions = { 'id': chart.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gSymbolGroupEle = chart.svgRenderer.createGroup(symbolOptions);

         var connectorOptions = { 'id': chart.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gSeriesConnectorEle = chart.svgRenderer.createGroup(connectorOptions);


         if (currentseries.pyramidMode.toLowerCase() == "linear")
             this.calculateLinearSegments(currentseries);
         else
             this.calculateSurfaceSegments(currentseries);

         for (j = 0; j < currentseries.points.length; j++) {
             var pointIndex = j;
             point = currentseries.points[j];
             if (point.visible) {
                 var result = this._getPyramidData(currentseries, pointIndex);

                 var styleOptions = chart.setStyle(this, currentseries, 0, pointIndex);

             options = {
                     'id': chart.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                 'fill': styleOptions.interior,
                 'stroke-width': styleOptions.borderWidth,
                 'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'opacity': styleOptions.opacity,
                     'pointIndex': pointIndex,
                     'd': result.Direction
                 };
                 chart.svgRenderer.drawPath(options, this.gSeriesGroupEle, translate);

                 if (currentseries.marker.dataLabel.template) {
                     ej.EjSvgRender.utils._getSeriesTemplateSize(point, pointIndex, currentseries, true, chart);
                 }

                 point.xLocation = result.PositionX;
                 point.yLocation = result.PositionY;
                 point.connectorLine = result.Connector;
                 point.startX = result.startX;
                 point.startY = result.startY;
                 point.Polygon = result.Polygon;
                 region = { PointIndex: pointIndex, Line1: result.Line1, Line2: result.Line2, Line3: result.Line3, Line4: result.Line4, Polygon: result.Polygon };
                 internalRegion.push(region);
             }
         }
         var region = { Series: currentseries, Region: internalRegion, SeriesIndex: $.inArray(currentseries, chartModel._visibleSeries) };
         chartModel.chartRegions.push(region);

         if (this.gSeriesGroupEle) {
             $(this.gSeriesGroupEle.childNodes[pointIndex]).bind('mousemove', function (evt) {
                 evt.target = seriesRender.gSeriesGroupEle.childNodes[pointIndex];
                 seriesRender.chartObj.chartInteractiveBehavior(evt);
             });

             chart.svgRenderer.append(this.gSeriesGroupEle, chart.gSeriesEle);
         }

        

     },



     doAnimation: function (series) {
      
     },
     _getPyramidData: function (series, index) {
        
         var model = this.chartObj.model;
         var pointMarker = series.points[index].marker;
         var offset = ((series.explodeIndex == index) || (series.explodeAll) ? series.explodeOffset : 0);
         var resWidth = model.resWidth;
         var top = model.pyramidData[index].CurrY;
         var bottom = model.pyramidData[index].CurrY + model.pyramidData[index].Height;
         var topRadius = 0.5 * (1 - model.pyramidData[index].CurrY);
         var bottomRadius = 0.5 * (1 - bottom);
         var point = series.points[index];

         var line1 = { x: resWidth + offset + topRadius * model.actualWidth, y: top * model.actualHeight };
         var line2 = { x: resWidth + offset + (1 - topRadius) * model.actualWidth, y: top * model.actualHeight };
         var line3 = { x: resWidth + offset + (1 - bottomRadius) * model.actualWidth, y: bottom * model.actualHeight };
         var line4 = { x: resWidth + offset + bottomRadius * model.actualWidth, y: bottom * model.actualHeight };

         var direction = "M" + " " + (line1.x) + " " + (line1.y) + " " + "L" + " " + (line2.x) + " " + (line2.y) + " " + "L" + " " + (line3.x) + " " + (line3.y) + " " + "L" + " " + (line4.x) + " " + (line4.y) + " " + "z";
         var polygon = [
             { x: line1.x, y: line1.y },
             { x: line2.x, y: line2.y },
             { x: line3.x, y: line3.y },
             { x: line4.x, y: line4.y }
            ];
         if ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && series.marker.dataLabel.visible) {
             var positionX;
             var positionY;
             var startX = (line2.x + line3.x) / 2;
             var startY = (line2.y + line3.y) / 2;
             if (series.labelPosition.toLowerCase() === 'outside' ) {

                 positionX = (line2.x + line3.x) / 2;
                 positionY = (line2.y + line3.y) / 2;

                 var pyrX = ((this.chartObj.model.legend.position.toLowerCase() == "left" || this.chartObj.model.legend.position.toLowerCase() == "right") ? (this.chartObj.model.LegendActualBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;

                 if (ej.util.isNullOrUndefined(series.marker.dataLabel.connectorLine.height)) {
                     if (series._enableSmartLabels)
                         var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + resWidth + (2 * this.chartObj.model.elementSpacing));
                     else
                         var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + this.chartObj.model.textSize + this.chartObj.model.elementSpacing);
                 }
                 else
                     textOffset = series.marker.dataLabel.connectorLine.height;
                                                
                 var connectorX = this.getXCordinate(positionX, textOffset, 0);
                 var connectorY = this.getYCordinate(positionY, textOffset, 0);
                
                 var connectorDirection = "M" + " " + positionX + " " + positionY + " " + "L" + " " + connectorX + " " + connectorY;

                 positionX = connectorX;
                 positionY = connectorY ;

                 
             }
             else {
                 positionX = (line1.x + line2.x) / 2;
                 positionY = (line3.y + line1.y) / 2;
             }
         }
         return { Direction: direction, PositionX: positionX, PositionY: positionY, Connector: connectorDirection, Line1: line1, Line2: line2, Line3: line3, Line4: line4, startX: startX, startY: startY, Polygon: polygon };
     },

     calculateLinearSegments: function (series) {
         var currY = 0;
         var model = this.chartObj.model;
         model.pyramidData = [];
         var gapRatio = series.gapRatio >= 0 ? (series.gapRatio <= 1 ? series.gapRatio : 1) : 0;
         var coef = 1 / (model.sumofYValues * (1 + gapRatio / (1 - gapRatio)));
         var count = series.points.length;
         for (var i = 0; i < series.points.length; i++) {
             if (series.points[i].visible) {
                 var height = coef * series.points[i].YValues[0];
                 model.pyramidData[i] = { CurrY: currY, Height: height };
                 currY += (gapRatio / (count - 1)) + height;
             }
         }
     },

     calculateSurfaceSegments: function (series) {
         var count = series.points.length;
         var model = this.chartObj.model;
         model.pyramidData = [];
         var currY = 0;
         var y = [];
         var height = [];
         var gapRatio = series.gapRatio >= 0 ? (series.gapRatio <= 1 ? series.gapRatio : 1) : 0;
         var gapHeight = gapRatio / (count - 1);
         var preSum = this.getSurfaceHeight(0, model.sumofYValues);
         for (i = 0; i < count; i++) {
             if (series.points[i].visible) {
                 y[i] = currY;
                 height[i] = this.getSurfaceHeight(currY, Math.abs(series.points[i].YValues[0]));
                 currY += height[i] + gapHeight * preSum;
             }
         }
         var coef = 1 / (currY - gapHeight * preSum);
         for (var i = 0; i < count; i++) {
             if (series.points[i].visible) {
                 var currHeight = coef * y[i];
                 model.pyramidData[i] = { CurrY: currHeight, Height: coef * height[i] };
             }
         }
     },

     getSurfaceHeight: function (y, surface) {
         var result = this.solveQuadraticEquation(1, 2 * y, -surface);
         if (result) {
             return result;
         }
         return 0;
     },
     solveQuadraticEquation: function (a, b, c) {
         var root1;
         var root2;

         if (a != 0) {
             var d = b * b - 4 * a * c;

             if (d >= 0) {
                 var sd = Math.sqrt(d);

                 root1 = (-b - sd) / (2 * a);
                 root2 = (-b + sd) / (2 * a);
                 return Math.max(root1, root2);

             }
         }
         else if (b != 0) {
             root1 = -c / b;
             root2 = -c / b;
             return Math.max(root1, root2);
         }

         return false;
     },

     chartAreaType: "None"
 });

 ej.seriesTypes.pyramid = ej.ejPyramidSeries;
   
 ej.ejFunnelSeries = ejExtendClass(ej.EjSeriesRender, {
     draw: function (chart, options) {        
         this.chartObj = chart;
         var chartModel = chart.model,
             legend = chartModel.legend,
             legendPosition = legend.position.toLowerCase(),
             legendActualBounds = chartModel.LegendActualBounds,
             point,
             seriesRender = this,
             currentseries = options,
             visiblePoints = this._pieVisiblePoints(currentseries),
             internalRegion = [],
             legendWidth = legend.border.width,
             translate = [],
             textSize = 0,
             text,
             borderSize = chartModel.border.width,
             measureTitle = (chartModel.title.text) ? ej.EjSvgRender.utils._measureText(chartModel.title.text, $(this.svgObject).width() - chartModel.margin.left - chartModel.margin.right, chartModel.title.font) : 0,
             xOffset = (chartModel.margin.left + chartModel.elementSpacing +  chartModel.elementSpacing + (borderSize * 2) + ((legendPosition === "right" || legendPosition === "left") ? legendActualBounds.Width + (2 * legendWidth) : 0)),
             yOffset = (((chartModel.title.text) ? (chartModel._titleLocation.Y + measureTitle.height) : chartModel.elementSpacing) + chartModel.margin.top + (borderSize * 2) + chartModel.elementSpacing + chartModel.elementSpacing + ((legendPosition === "top" || legendPosition === "bottom") ? legendActualBounds.Height + (2 * legendWidth) + chartModel.elementSpacing : 0));

         chartModel.chartRegions = [];
         chartModel.sumofYValues = 0;
         if (currentseries.labelPosition.toLowerCase() === 'outside') {
             for (var i = 0; i < visiblePoints.length; i++) {
                 var textWidth = ej.EjSvgRender.utils._measureText((ej.util.isNullOrUndefined(visiblePoints[i].text) ? visiblePoints[i].y : visiblePoints[i].text), null, currentseries.marker.dataLabel.font).width;
                 if (textSize < textWidth)
                     textSize = textWidth;
             }
             chartModel.textSize = textSize;
         }
         if (currentseries._enableSmartLabels) {
             chartModel.actualWidth = ($(chart.svgObject).width() - xOffset) * 0.8;  //calculate a width of a funnel chart when smartlabel is enabled
             chartModel.resWidth = ($(chart.svgObject).width() - xOffset - chartModel.actualWidth) / 2;
         }
         else {
             chartModel.actualWidth = $(chart.svgObject).width() - xOffset - textSize;
             chartModel.resWidth = 0;
         }
         chartModel.actualHeight = $(chart.svgObject).height() - yOffset;
         var pyrX = chart.pyrX = ((legendPosition === "left") ? (legendActualBounds.Width + (2 * legendWidth)) : 0) + chartModel.elementSpacing + chartModel.margin.left;
         var pyrY = chart.pyrY = ((legendPosition === "top") ? (legendActualBounds.Height + (2 * legendWidth)) : 0) + ((chartModel.title.text) ? (chartModel._titleLocation.Y + measureTitle.height) : (chartModel.margin.top + chartModel.elementSpacing));
         translate[0] = pyrX;
         translate[1] = pyrY;
         for (var j = 0; j < visiblePoints.length; j++) {
             chartModel.sumofYValues += visiblePoints[j].YValues[0];
         }


         var numberToFixed = ej.util.isNullOrUndefined(chartModel.roundingPlaces) ? 2 : chartModel.roundingPlaces;

         var seriesIndex = $.inArray(currentseries, chartModel._visibleSeries);
         var serOptions = { 'id': chart.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         this.gSeriesGroupEle = chart.svgRenderer.createGroup(serOptions);
         var dataLabelOptions = { 'id': chart.svgObject.id + '_DataLabel_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gDataLabelEle = chart.svgRenderer.createGroup(dataLabelOptions);


         var txtOptions = { 'id': chart.svgObject.id + '_textGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gSeriesTextEle = chart.svgRenderer.createGroup(txtOptions);

         var symbolOptions = { 'id': chart.svgObject.id + '_symbolGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gSymbolGroupEle = chart.svgRenderer.createGroup(symbolOptions);

         var connectorOptions = { 'id': chart.svgObject.id + '_connectorGroup' + '_' + seriesIndex, 'transform': 'translate(' + pyrX + ',' + pyrY + ')' };
         chart.gSeriesConnectorEle = chart.svgRenderer.createGroup(connectorOptions);

         var currY = 0;
         var model = chartModel;
         model.funnelData = [];
         var gapRatio = currentseries.gapRatio >= 0 ? (currentseries.gapRatio <= 1 ? currentseries.gapRatio : 1) : 0;
         var coef = 1 / (model.sumofYValues * (1 + gapRatio / (1 - gapRatio)));
         var count = currentseries.points.length;
         var spacing = gapRatio / count;
         for (var i = count - 1; i >= 0; i--) {
             if (currentseries.points[i].visible) {
                 var height = coef * currentseries.points[i].YValues[0];
                 model.funnelData[i] = { CurrY: currY, Height: height };
                 currY += height + spacing;
             }
         }
         for (j = 0; j < count  ; j++) {
             var pointIndex = j;
             point = currentseries.points[j];
             if (point.visible) {
                 var result = this._getFunnelData(currentseries, pointIndex, chart);

                 var styleOptions = chart.setStyle(this, currentseries, 0, pointIndex);

                 options = {
                     'id': chart.svgObject.id + '_Series' + seriesIndex + "_Point" + pointIndex,
                     'fill': styleOptions.interior,
                     'stroke-width': styleOptions.borderWidth,
                     'stroke': styleOptions.borderColor,
                     'stroke-linecap': currentseries.lineCap,
                     'stroke-linejoin': currentseries.lineJoin,
                     'opacity': styleOptions.opacity,
                     'pointIndex': pointIndex,
                     'd': result.Direction
                 };
                 chart.svgRenderer.drawPath(options, this.gSeriesGroupEle, translate);

                 if (currentseries.marker.dataLabel.template) {
                     ej.EjSvgRender.utils._getSeriesTemplateSize(point, pointIndex, currentseries, true, chart);
                 }

                 point.xLocation = result.PositionX;
                 point.yLocation = result.PositionY;
                 point.connectorLine = result.Connector;
                 point.startX = result.startX;
                 point.startY = result.startY;
                 point.Polygon = result.Polygon;
                 region = { PointIndex: pointIndex, Line1: result.Line1, Line2: result.Line2, Line3: result.Line3, Line4: result.Line4, Line5: result.Line5, Line6: result.Line6, Polygon: result.Polygon };
                 internalRegion.push(region);
             }
         }
         var region = { Series: currentseries, Region: internalRegion, SeriesIndex: $.inArray(currentseries, chartModel._visibleSeries) };
         chartModel.chartRegions.push(region);

         if(this.gSeriesGroupEle)
         $(this.gSeriesGroupEle.childNodes[pointIndex]).bind('mousemove', function (evt) {
             evt.target = seriesRender.gSeriesGroupEle.childNodes[pointIndex];
             seriesRender.chartObj.chartInteractiveBehavior(evt);
         });

         chart.svgRenderer.append(this.gSeriesGroupEle, chart.gSeriesEle);


     },



     doAnimation: function (series) {

     },
     _getFunnelData: function (series, index, chart) {

         var model = this.chartObj.model;
         var pointMarker = series.points[index].marker;
		 var point = series.points[index];
         var resWidth = model.resWidth;
         var offset = ((series.explodeIndex == index) || (series.explodeAll) ? series.explodeOffset : 0);
         var topRadius, minRadius, bottomRadius, endTop, endMin, endBottom, top, bottomY, bottom;
         var line1, line2, line3, line4, line5, line6;
         var lineWidth;
         var minimumHeight;
         if (series.funnelHeight.indexOf("%") != -1)
             minimumHeight = model.actualHeight * (parseInt(series.funnelHeight) / 100);
         else
             minimumHeight = parseInt(series.funnelHeight);
         var minimumWidth;
         if (series.funnelWidth.indexOf("%") != -1)
             minimumWidth = model.actualWidth * (parseInt(series.funnelWidth) / 100);
         else
             minimumWidth = parseInt(series.funnelWidth);

         top =  model.funnelData[index].CurrY * model.actualHeight;
         bottom = top + model.funnelData[index].Height*model.actualHeight;
         lineWidth = minimumWidth + (model.actualWidth - minimumWidth) * ((model.actualHeight - minimumHeight - top) / (model.actualHeight - minimumHeight));
         topRadius = (model.actualWidth / 2) - lineWidth / 2;
         endTop = topRadius + lineWidth;
         if (bottom > model.actualHeight - minimumHeight || model.actualHeight == minimumHeight)
             lineWidth= minimumWidth;
         else
             lineWidth= minimumWidth + (model.actualWidth - minimumWidth) * ((model.actualHeight - minimumHeight - bottom) / (model.actualHeight - minimumHeight));
         bottomRadius = (model.actualWidth / 2) - (lineWidth / 2);
         endBottom = bottomRadius + lineWidth;
         if (top >= model.actualHeight - minimumHeight) {
             topRadius = bottomRadius = minRadius = (model.actualWidth / 2) - minimumWidth / 2;
             endTop = endBottom = endMin = (model.actualWidth / 2) + minimumWidth / 2;

         } else if (bottom > (model.actualHeight - minimumHeight)) {
             minRadius = bottomRadius = (model.actualWidth / 2) - lineWidth / 2;
             endMin = endBottom = minRadius + lineWidth;

             bottomY = model.actualHeight - minimumHeight;
         }

         line1 = { x: resWidth + offset + topRadius, y: top };
         line2 = { x: resWidth + offset + endTop, y: top };
         line4 = { x: resWidth + offset + endBottom, y: bottom };
         line5 = { x: resWidth + offset + bottomRadius, y: bottom };
         line3 = { x: resWidth + offset + endBottom, y: bottom };
         line6 = { x: resWidth + offset + bottomRadius, y: bottom };
         if (bottomY) {
             line3 = { x: resWidth + offset + endMin, y: bottomY };
             line6 = { x: resWidth + offset + minRadius, y: bottomY };
         }
         var direction = "M" + " " + (line1.x) + " " + (line1.y) + " " + "L" + " " + (line2.x) + " " + (line2.y) + " " + "L" + " " + (line3.x) + " " + (line3.y) + " " + "L" + " " + (line4.x) + " " + (line4.y) + " " + "L" + " " + (line5.x) + " " + (line5.y) + " " + "L" + " " + (line6.x) + " " + (line6.y) + " " + "z";
         var polygon = [
           { x: line1.x, y: line1.y },
           { x: line2.x, y: line2.y },
           { x: line3.x, y: line3.y },
           { x: line4.x, y: line4.y },
           { x: line5.x, y: line5.y },
           { x: line6.x, y: line6.y }
            ];
		 if ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && series.marker.dataLabel.visible) {
             var positionX;
             var positionY;
             var startX = (line2.x + line3.x) / 2;
             var startY = (line2.y + line3.y) / 2;   
             if (series.labelPosition.toLowerCase() === 'outside' ) {

                 positionX = (line2.x + line3.x) / 2;
                 positionY = (line2.y + line3.y) / 2;

                 var pyrX = ((this.chartObj.model.legend.position.toLowerCase() == "left" || this.chartObj.model.legend.position.toLowerCase() == "right") ? (this.chartObj.model.LegendActualBounds.Width) : 0) + this.chartObj.model.elementSpacing + this.chartObj.model.margin.left;

                 if (ej.util.isNullOrUndefined(series.marker.dataLabel.connectorLine.height)) {
                     if (series._enableSmartLabels)
                         var textOffset = $(this.chartObj.svgObject).width() - (pyrX + positionX + resWidth + (2 * this.chartObj.model.elementSpacing));
                     else
                         var textOffset = $(this.chartObj.svgObject).width() - (positionX + this.chartObj.model.textSize + this.chartObj.model.elementSpacing);
                 }
                 else
                     textOffset = series.marker.dataLabel.connectorLine.height;

                 var connectorX = this.getXCordinate(positionX, textOffset, 0);
                 var connectorY = this.getYCordinate(positionY, textOffset, 0);

                 var connectorDirection = "M" + " " + positionX + " " + positionY + " " + "L" + " " + connectorX + " " + connectorY;

                 positionX = connectorX;
                 positionY = connectorY;


             }
             else {
                 positionX = (line1.x + line2.x) / 2;
                 positionY = (line1.y + line4.y) / 2;
             }
         }
         return { Direction: direction, PositionX: positionX, PositionY: positionY, Connector: connectorDirection, Line1: line1, Line2: line2, Line3: line3, Line4: line4, Line5: line5, Line6: line6, startX: startX, startY: startY, Polygon: polygon };
     },

     
     chartAreaType: "None"
 });

 ej.seriesTypes.funnel = ej.ejFunnelSeries;
    /*Funnel series*/

 ej.ejPolarSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._pieVisiblePoints(currentseries);
         var internalRegion = [];
        

         var style = this.setLineSeriesStyle(currentseries);
        
         if (currentseries.drawType.toLowerCase() == "column") {
             var range=currentseries.xAxis.visibleRange;
			 range.interval = 1;
             var length = (currentseries.xAxis._valueType == "category") ? range.max : range.max - 1;
             this.chartObj.model.sumofYValues = 0;
             var count = range.interval;
             var min = range.min;
             do{
                 this.chartObj.model.sumofYValues += range.interval;
                 min += range.interval;
             } while (min <= length)
           //  this.chartObj.model.sumofYValues = visiblePoints.length;
         }

         var direction = this._calculatePolarAxesSegment(currentseries);

         if (currentseries.drawType.toLowerCase() == "area")
             this.drawAreaPath(currentseries, style, direction);
         else if (currentseries.drawType.toLowerCase() == "line")
             this._drawLinePath(currentseries, style, direction);
         
         
         
         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

       
     },
     doAnimation: function (series) {
             var chartObj = this.chartObj;
             series.regionAdded = true;
             chartObj.model._radius = chartObj.model.Radius;
             chartObj.model.Radius = 0;
             var seriesRender = this;
             var seriesIndex = $.inArray(series, chartObj.model._visibleSeries);
             var Path = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[0];
             $(Path).animate(
                 {  Radius: chartObj.model._radius },

                 {
                     duration: 1200, queue: false, step: function (now, fx) {  
                          
                       chartObj.model.Radius = now;
                                                 
                       var direction = seriesRender._calculatePolarAxesSegment(series);
                       if(series.drawType.toLowerCase()!='column')
                       chartObj.svgRenderer._setAttr($(Path), { "d": direction });
                       chartObj.model.Radius = chartObj.model._radius;
                     },
                     complete: function () {
                         series.regionAdded = false;                         
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex), { "visibility": "visible" });
                         
                             chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex), { "visibility": "visible" });

							 chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_DataLabel' + '_' + seriesIndex), { "visibility": "visible" });
                             chartObj.model.AnimationComplete = true;
                             var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                             commonEventArgs.data = { series: series };
                             chartObj._trigger("animationComplete", commonEventArgs);
                         }
                     
                 });       
     },
     chartAreaType: "PolarAxes"

 });

 ej.seriesTypes.polar = ej.ejPolarSeries;
 ej.ejRadarSeries = ejExtendClass(ej.EjSeriesRender, {

     draw: function (chart, options) {

         this.chartObj = chart;
         var point;
         var seriesRender = this;
         var currentseries = options;
         var visiblePoints = this._pieVisiblePoints(currentseries);
         var internalRegion = [];

         var style = this.setLineSeriesStyle(currentseries);

         if (currentseries.drawType.toLowerCase() == "column") {
             var range = currentseries.xAxis.visibleRange;
			 range.interval = 1;
             var length = (currentseries.xAxis._valueType == "category") ? range.max : range.max - 1;
             this.chartObj.model.sumofYValues = 0;
             var count = range.interval;
             var min = range.min;
             do {
                 this.chartObj.model.sumofYValues += range.interval;
                 min += range.interval;
             } while (min <= length)
             //  this.chartObj.model.sumofYValues = visiblePoints.length;
         }

         var direction = this._calculatePolarAxesSegment(currentseries);

          if (currentseries.drawType.toLowerCase() == "area")
             this.drawAreaPath(currentseries, style, direction);
         else if (currentseries.drawType.toLowerCase() == "line")
             this._drawLinePath(currentseries, style, direction);

         this.chartObj.svgRenderer.append(this.gSeriesGroupEle, this.chartObj.gSeriesEle);

         

     },
     doAnimation: function (series) {
         var chartObj = this.chartObj;
         series.regionAdded = true;
         chartObj.model._radius = chartObj.model.Radius;
         chartObj.model.Radius = 0;
         var seriesRender = this;
         var seriesIndex = $.inArray(series, chartObj.model._visibleSeries);
         var Path = $(chartObj.gSeriesEle).find("#" + seriesRender.gSeriesGroupEle.id)[0].childNodes[0];
         $(Path).animate(
             { Radius: chartObj.model._radius },

             {
                 duration: 1200, queue: false, step: function (now, fx) {

                     chartObj.model.Radius = now;

                     var direction = seriesRender._calculatePolarAxesSegment(series);
                     if (series.drawType.toLowerCase() != 'column')
                         chartObj.svgRenderer._setAttr($(Path), { "d": direction });
                     chartObj.model.Radius = chartObj.model._radius;
                 },
                 complete: function () {
                     series.regionAdded = false;                    
                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_TextGroup' + '_' + seriesIndex), { "visibility": "visible" });
                   
                     chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_symbolGroup' + '_' + seriesIndex), { "visibility": "visible" });
					chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.svgObject.id + '_DataLabel' + '_' + seriesIndex), { "visibility": "visible" });
                     chartObj.model.AnimationComplete = true;
                     var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                     commonEventArgs.data = { series: series };
                     chartObj._trigger("animationComplete", commonEventArgs);
                 }

             });
     },
     chartAreaType: "PolarAxes"

 });

 ej.seriesTypes.radar = ej.ejRadarSeries;
})(jQuery);;
ej.ejChart = {};
(function($) {
String.prototype.parseTemplate = function () {
    var str = this;
    $.each(arguments[0], function (dataIndex, dataValue) {
        if (dataValue.count > 0) {
            $.each(dataValue, function (datachildIndex, datachildValue) {
                str = str.replace(new RegExp('\\#' + dataIndex + '.' + datachildIndex + '\\#', 'gm'), datachildValue);
            });
        } else {
            str = str.replace(new RegExp('\\#' + dataIndex + '\\#', 'gm'), dataValue);
        }

    });
    return str;
};

String.prototype.capitalizeFirstString = (function () {
    return this.charAt(0).toUpperCase() + this.slice(1);
});

var Gradient = function (colors) {
    this._gradientStop = [];
    for (var j = 0; j < colors.length; j++) {
        this._gradientStop.push(colors[j]);
    }
};



 ej.ejChart = {

    _renderSfChart: function () {

        var svgSupport = (window.SVGSVGElement) ? true : false;
        $("#chartContainer_" + this._id).remove();
        if (svgSupport) {
            if (this.model.enableCanvasRendering) {
                this.svgRenderer = new ej.EjCanvasRender(this.element);
                this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
                this.chartContainer = $("<div></div>").css("position", "relative").attr('id', "chartContainer_" + this._id);
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
                $(this.legendContainer).appendTo(this.chartContainer);
                $(this.chartContainer).appendTo(this.element);
            } else {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height","0px");
                this.chartContainer = $("<div></div>").css("position", "relative").attr('id', "chartContainer_" + this._id);
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
                $(this.legendContainer).appendTo(this.chartContainer);
                $(this.chartContainer).appendTo(this.element);
				this.legendContainer.find("svg").attr("class", "e-designerhide");
            }

                this._createChart();
                $(this.svgObject).appendTo(this.element);
        }
        else {
            this.model.enableCanvasRendering = false;
            this.svgRenderer = new ej.EjVmlRender(this.element);
            this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new div element for legend
            this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
            this.chartContainer = $("<div></div>").css("position", "relative").attr('id', "chartContainer_" + this._id);
            this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            $(this.legendContainer).appendTo(this.chartContainer);
            $(this.chartContainer).appendTo(this.element);
            this._createChart();
            $(this.svgObject).appendTo(this.chartContainer);
        }
        return true;
    },

 setSvgSize: function (sender) {
            var chartObj = sender;
            var containerHeight = $(chartObj.element).height();
            var height = 450; //set default height if size not specified to chart or container
            var width =(ej.isTouchDevice()) ? 250: 600;
            var containerWidth = $(chartObj.element).width();
            var legendcontainerHeight = $(chartObj.chartContainer).height();
            if (chartObj.model.size.width) {
                var chartWidth = chartObj.model.size.width;
                if (typeof (chartWidth)== "string" && chartWidth.indexOf("%") != -1)
                    width = (containerWidth / 100) * parseInt(chartWidth);
                else
                    width = parseInt(chartWidth);
            }
            else if (containerWidth > 0)
                width = containerWidth;

            $(chartObj.svgObject).width(width);

            if (chartObj.model.size.height) {
                var chartHeight = chartObj.model.size.height;
                if (typeof (chartHeight) == "string" && chartHeight.indexOf("%") != -1)
                    height = (!this.vmlRendering) ? (containerHeight / 100) * parseInt(chartHeight) : height;
                else
                    height = parseInt(chartHeight);
            }
            else if (containerHeight > 0)
                height = containerHeight;
            else
                $(chartObj.svgObject).css("display", "block"); //Added css to chart due to 5px difference for container and svg (http://jsfiddle.net/Me5Zd/)       
            $(chartObj.svgObject).height(height);
			
			chartObj.svgObject.width = width;   // assigning width for canvas
			chartObj.svgObject.height = height;  // assigning height for 
},
    setModelProperties: function (excludeDataUpdate) {

        //set size for the svgObject
        this.setSvgSize(this);

        this.svgWidth = $(this.svgObject).width();
        this.svgHeight = $(this.svgObject).height();

        this.chartCross = {
            visible: (excludeDataUpdate) ? this.chartCross.visible : this.model.crosshair.visible,
            mArea: this.model.crosshair.type
        };

        // extend series with commonSeriesOptions
        this.serAnimation = [];
        if (this.model.series) {
            var trendlines, len;
            this.model._drawTrendline = false;
            for (var j = 0; j < this.model.series.length; j++) {
                this.model.series[j] = ej.copyObject({}, this.model.commonSeriesOptions, this.model.series[j]);
                this.serAnimation.push((this.model.series[j].enableAnimation === null || this.model.series[j].enableAnimation === undefined) ? this.model.commonSeriesOptions.enableAnimation : this.model.series[j].enableAnimation);
                    trendlines = this.model.series[j].trendlines;
                    len = trendlines.length;
                    for (var i = 0; i < len; i++) {
                        trendlines[i] = ej.copyObject({}, this.model.trendlineDefaults, trendlines[i]);
                        if(trendlines[i].visibility.toLowerCase() == "visible")
                            this.model._drawTrendline = true;
            }
        }
        }
        if (this.model.indicators) {
            for (var k = 0; k < this.model.indicators.length; k++) {
                this.model.indicators[k] = ej.copyObject({}, this.model.indicatorDefaults, this.model.indicators[k]);
            }
        }

    },

    _createChart: function () {
        this.svgObject = this.svgRenderer.svgObj;
        var commonlaodEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        this._trigger("load", commonlaodEventArgs);
        this.setTheme(ej.EjSvgRender.themes, this.model.theme);
        var model = this.model;
  
        if (model.series[0] && model.series[0]._isdesigntime) {
            model.series[0] = {};
            model.series[0].points = [];
            var randomPoints = [];
            for (var i = 1; i <= 10; i++) {
                randomPoints.push({ x: i, y: (Math.floor(Math.random() * (10 - 40)) + 40) });

            }
            model.series[0].points = randomPoints;
        }
		this.vmlRendering = (this.svgRenderer.vmlNamespace) ? true : false;
        if ($(this.element).is(":visible") || $(this.element).css("visibility") != "hidden" || $(this.element).css('display') != 'none') {
            this.bindEvents();
            this.bindTo();
        }
    },
    setTheme: function (jsonObj, themeName) {
        var name = themeName.toLowerCase();
        if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") == -1) && (name.indexOf("dark") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["flatlight"], jsonObj[name]);
        else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark")) >= 0 && (name.indexOf("gradient") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["flatdark"], jsonObj[name]);
        else if ((name.indexOf("azure") >= 0 || name.indexOf("lime") >= 0 || name.indexOf("saffron") >= 0) && (name.indexOf("gradient") >= 0) && (name.indexOf("dark") == -1))
            jsonObj[name] = $.extend(true, {}, jsonObj["gradientlight"], jsonObj[name]);
        else if ((name.indexOf("azuredark") >= 0 || name.indexOf("limedark") >= 0 || name.indexOf("saffrondark") >= 0) && (name.indexOf("gradient") >= 0))
            jsonObj[name] = $.extend(true, {}, jsonObj["gradientdark"], jsonObj[name]);
        else if (!(jsonObj[name])) name = "flatlight";

        if (this.model._themeChanged)
            this.model = ej.copyObject({}, this.model, jsonObj[name]);
        else
            this.model = ej.copyObject({}, jsonObj[name], this.model);


    },
    _drawBackInterior: function () {
        if (this.model.backInterior && this.model.backInterior._gradientStop) {
            var options = {
                'id': this.svgObject.id + '_backGradient',
                'x1': '0%',
                'y1': '0%',
                'x2': '0%',
                'y2': $(this.svgObject).height() + '%'
            };
            this.svgRenderer.drawGradient(options, this.model.backInterior._gradientStop);
        }
    },
    


    _drawBackImage: function () {
        var options = {
            'height': $(this.svgObject).height(),
            'width': $(this.svgObject).width(),
            'href': this.model.backGroundImageUrl,
            'x': 0,
            'y': 0,
            'id': this.svgObject.id + '_backImage',
            'visibility': 'visible',
            'preserveAspectRatio': 'none'
        };

        this.svgRenderer.drawImage(options, this.svgObject);

    },
    _getDataType: function (val) {
        var type = typeof (val);
        if (type === "number" || type === "string")
            return type;
        else if (jQuery.type(val) == "date")
            return "date";
        else
            return false;

    },
    addedXYValues: function (excludeDataUpdate) {
        excludeDataUpdate = (excludeDataUpdate) ? excludeDataUpdate : false;
        var type = this.model.series[0].type.toLowerCase();
        if (ej.seriesTypes[type].prototype.chartAreaType == "cartesianAxes") {
            this.model.AreaType = "cartesianaxes";
        }
        else if (ej.seriesTypes[type].prototype.chartAreaType == "None")
            this.model.AreaType = "none";
        else {
            if (type == 'polar')
                this.model.isPolar = true;
            else
                this.model.isPolar = false;

            this.model.AreaType = "polaraxes";
        }
        if (!excludeDataUpdate) {
            for (var i = 0; i < this.model.series.length; i++) {
                if(this.model.series[i].visibility == "visible") {
                var setXValueType = false, isDatetime = false, pointX, currentSeries;

                if (!this.model.series[i].dataSource) {
                    this.model.series[i] = $.extend(true, {}, this.model.commonSeriesOptions, this.model.series[i]);
                    currentSeries = this.model.series[i];
                    if (this.model.primaryXAxis.valueType == "datetime")
                        isDatetime = true;
                    for (k = 0; k < currentSeries.points.length; k++) {
                        currentSeries.points[k].YValues = [];
                        if ((parseFloat(currentSeries.points[k].x)) && (/^[.0-9- ]*$/.test(currentSeries.points[k].x) == true) && (this.model.primaryXAxis.valueType != "category") && (this.model.AreaType != "none")) {
                            if (!isDatetime) {
                                currentSeries.points[k].xValue = parseFloat(currentSeries.points[k].x);
                                currentSeries.points[k].x = parseFloat(currentSeries.points[k].x);
                            }
                            else {
                                currentSeries.points[k].xValue = new Date(currentSeries.points[k].x);
                                currentSeries.points[k].x = new Date(currentSeries.points[k].x);
                                }
                       }
                      else
                          currentSeries.points[k].xValue = currentSeries.points[k].x;
                        if (!setXValueType) {
                            pointX = currentSeries.points[k].x;
                            if (!ej.util.isNullOrUndefined(pointX)) {
                                currentSeries._xAxisValueType = this._getDataType(pointX);
                                setXValueType = true;
                            }
                        }
                        if (!(currentSeries._hiloTypes)) {
                            currentSeries.points[k].YValues[0] = currentSeries.points[k].y;

                            if (!ej.util.isNullOrUndefined(currentSeries.points[k].size))
                                currentSeries.points[k].YValues[1] = currentSeries.points[k].size;

                        }
                        else {
                            if (!ej.util.isNullOrUndefined(currentSeries.points[k].high)) {
                                currentSeries.points[k].YValues[0] = currentSeries.points[k].high;
                                currentSeries.points[k].y = currentSeries.points[k].high;
                            }
                            if (!ej.util.isNullOrUndefined(currentSeries.points[k].low))
                                currentSeries.points[k].YValues[1] = currentSeries.points[k].low;
                            if (!ej.util.isNullOrUndefined(currentSeries.points[k].open))
                                currentSeries.points[k].YValues[2] = currentSeries.points[k].open;
                            if (!ej.util.isNullOrUndefined(currentSeries.points[k].close))
                                currentSeries.points[k].YValues[3] = currentSeries.points[k].close;
                        }
                    }
                }
                else {
                    currentSeries = this.model.series[i];
                    currentSeries._xAxisValueType = null;
                    for (var j = 0, length=currentSeries.points.length; j <length; j++) {
                        pointX = currentSeries.points[j].x;                  
                        if (!ej.util.isNullOrUndefined(pointX) && ej.util.isNullOrUndefined(currentSeries._xAxisValueType)) {
                            currentSeries._xAxisValueType = this._getDataType(pointX);                            
                        }
                        if (currentSeries._xAxisValueType != "string")//skip the category series to avoid series removing on zooming
                            currentSeries.points[j].xValue= currentSeries.points[j].x;
                    }
                }
			 }
            }
           this._setInternalValues();
         }
    },
    _setInternalValues: function () {
        
        for (var k = 0; k < this.model.series.length; k++) {
            var series = this.model.series[k];
            if (this.model.AreaType == "cartesianaxes" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "cartesianAxes") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }
            else if (this.model.AreaType == "none" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "None") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }
            else if (this.model.AreaType == "polaraxes" && ej.seriesTypes[series.type.toLowerCase()].prototype.chartAreaType == "PolarAxes") {
                series._zOrder = (series.zOrder) ? series.zOrder : 0;
                this.model._visibleSeries.push(series);
            }

            if (series._yAxisName || series._xAxisName) {
                if (this.model._axes) {
                    for (var t = 0; t < this.model._axes.length; t++) {
                        if (series._yAxisName && (series._yAxisName == this.model._axes[t].name)) {
                            this.model._axes[t].orientation = (this.model._axes[t].orientation) ? this.model._axes[t].orientation : (this.model.requireInvertedAxes) ? "horizontal" : "vertical";
                        }
                        if (series._xAxisName && (series._xAxisName == this.model._axes[t].name)) {
                            this.model._axes[t].orientation = (this.model._axes[t].orientation) ? this.model._axes[t].orientation : (this.model.requireInvertedAxes) ? "vertical" : "horizontal";
                        }
                    }
                }
            }
        }
    },
    setHiloStyle: function(series, pointIndex, seriesIndex, interiorColor){
        var point = series.points[pointIndex];
                  
        if (point.fill)
              interior = point.fill;
         else
              interior = series.fill ? series.fill : interiorColor;
				
            // setting default series color		
            var colors = interior ? interior : this.model.seriesColors[seriesIndex];
          
            // applying gradient color		  
         
            seriesInterior = jQuery.type(colors) == "array" ? colors[0].color : colors;

            return seriesInterior;
                       
    },
	setStyle: function (chart, series, seriesIndex, pointIndex, interiorColor) {
    
        var point = series.points[pointIndex];
       // var pointStyle = point.style ? point.style : " ";
        var interior, borderColor, borderWidth, opacity, seriesFill;
		if (series._hiloTypes)
		   seriesFill = series.isFill ? series.fill : null; 
		   
        // setting interior
        if (chart.chartObj.model.AreaType == "cartesianaxes") {
            if (point.fill)
                interior = point.fill;
            else
                interior = seriesFill ? seriesFill : interiorColor;
				
		// setting default series color		
            var colors = interior ? interior : this.model.seriesColors[seriesIndex];
          
        // applying gradient color		  
			if (chart.chartObj.model.requireInvertedAxes) 
                var seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 100, 0, chart.gSeriesGroupEle);
            else                                       
                seriesInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + "_" + seriesIndex + pointIndex, colors, 0, 0, 0, 100, chart.gSeriesGroupEle);
         } 
		 
		// interior for pie type series
        else {
            var pointColors = this.model.pointColors;
            interior = point.fill ? point.fill : jQuery.type(pointColors[pointIndex]) == "array" ? pointColors[pointIndex][0].color : pointColors[pointIndex];
            var pointInterior = this.svgRenderer.createGradientElement(chart.getSeriesName(series) + pointIndex, interior, 0, 0, 0, ($(chart.svgObject).height() || parseFloat(this.model.size.height)), chart.chartObj.gSeriesEle);
        }
        
        // setting borderColor
            borderColor =((point.border) && point.border.color)? point.border.color : series.border.color;
            borderWidth =((point.border) && point.border.width) ? point.border.width : series.border.width;
            opacity = point.opacity ? point.opacity : series.opacity;
            

        var options = {
            'interior': seriesInterior ? seriesInterior: pointInterior,
            'borderColor': borderColor,
            'borderWidth': borderWidth,
            'opacity': opacity
        };
        return options;
    },
	
    colorNameToHex: function (colour) {
        var color = colour;
        var colours = {
            "aliceblue": "#f0f8ff", "antiquewhite": "#faebd7", "aqua": "#00ffff", "aquamarine": "#7fffd4", "azure": "#f0ffff",
            "beige": "#f5f5dc", "bisque": "#ffe4c4", "black": "#000000", "blanchedalmond": "#ffebcd", "blue": "#0000ff", "blueviolet": "#8a2be2", "brown": "#a52a2a", "burlywood": "#deb887",
            "cadetblue": "#5f9ea0", "chartreuse": "#7fff00", "chocolate": "#d2691e", "coral": "#ff7f50", "cornflowerblue": "#6495ed", "cornsilk": "#fff8dc", "crimson": "#dc143c", "cyan": "#00ffff",
            "darkblue": "#00008b", "darkcyan": "#008b8b", "darkgoldenrod": "#b8860b", "darkgray": "#a9a9a9", "darkgreen": "#006400", "darkkhaki": "#bdb76b", "darkmagenta": "#8b008b", "darkolivegreen": "#556b2f",
            "darkorange": "#ff8c00", "darkorchid": "#9932cc", "darkred": "#8b0000", "darksalmon": "#e9967a", "darkseagreen": "#8fbc8f", "darkslateblue": "#483d8b", "darkslategray": "#2f4f4f", "darkturquoise": "#00ced1",
            "darkviolet": "#9400d3", "deeppink": "#ff1493", "deepskyblue": "#00bfff", "dimgray": "#696969", "dodgerblue": "#1e90ff",
            "firebrick": "#b22222", "floralwhite": "#fffaf0", "forestgreen": "#228b22", "fuchsia": "#ff00ff",
            "gainsboro": "#dcdcdc", "ghostwhite": "#f8f8ff", "gold": "#ffd700", "goldenrod": "#daa520", "gray": "#808080", "green": "#008000", "greenyellow": "#adff2f",
            "honeydew": "#f0fff0", "hotpink": "#ff69b4",
            "indianred ": "#cd5c5c", "indigo ": "#4b0082", "ivory": "#fffff0", "khaki": "#f0e68c",
            "lavender": "#e6e6fa", "lavenderblush": "#fff0f5", "lawngreen": "#7cfc00", "lemonchiffon": "#fffacd", "lightblue": "#add8e6", "lightcoral": "#f08080", "lightcyan": "#e0ffff", "lightgoldenrodyellow": "#fafad2",
            "lightgrey": "#d3d3d3", "lightgreen": "#90ee90", "lightpink": "#ffb6c1", "lightsalmon": "#ffa07a", "lightseagreen": "#20b2aa", "lightskyblue": "#87cefa", "lightslategray": "#778899", "lightsteelblue": "#b0c4de",
            "lightyellow": "#ffffe0", "lime": "#00ff00", "limegreen": "#32cd32", "linen": "#faf0e6",
            "magenta": "#ff00ff", "maroon": "#800000", "mediumaquamarine": "#66cdaa", "mediumblue": "#0000cd", "mediumorchid": "#ba55d3", "mediumpurple": "#9370d8", "mediumseagreen": "#3cb371", "mediumslateblue": "#7b68ee",
            "mediumspringgreen": "#00fa9a", "mediumturquoise": "#48d1cc", "mediumvioletred": "#c71585", "midnightblue": "#191970", "mintcream": "#f5fffa", "mistyrose": "#ffe4e1", "moccasin": "#ffe4b5",
            "navajowhite": "#ffdead", "navy": "#000080",
            "oldlace": "#fdf5e6", "olive": "#808000", "olivedrab": "#6b8e23", "orange": "#ffa500", "orangered": "#ff4500", "orchid": "#da70d6",
            "palegoldenrod": "#eee8aa", "palegreen": "#98fb98", "paleturquoise": "#afeeee", "palevioletred": "#d87093", "papayawhip": "#ffefd5", "peachpuff": "#ffdab9", "peru": "#cd853f", "pink": "#ffc0cb", "plum": "#dda0dd", "powderblue": "#b0e0e6", "purple": "#800080",
            "red": "#ff0000", "rosybrown": "#bc8f8f", "royalblue": "#4169e1",
            "saddlebrown": "#8b4513", "salmon": "#fa8072", "sandybrown": "#f4a460", "seagreen": "#2e8b57", "seashell": "#fff5ee", "sienna": "#a0522d", "silver": "#c0c0c0", "skyblue": "#87ceeb", "slateblue": "#6a5acd", "slategray": "#708090", "snow": "#fffafa", "springgreen": "#00ff7f", "steelblue": "#4682b4",
            "tan": "#d2b48c", "teal": "#008080", "thistle": "#d8bfd8", "tomato": "#ff6347", "turquoise": "#40e0d0",
            "violet": "#ee82ee",
            "wheat": "#f5deb3", "white": "#ffffff", "whitesmoke": "#f5f5f5",
            "yellow": "#ffff00", "yellowgreen": "#9acd32"
        };

        if (Object.prototype.toString.call(color) == '[object Array]')
            return color;

        if (typeof colours[color.toLowerCase()] != 'undefined')
            return colours[color.toLowerCase()];

        return color;
    },
	
	// set culture for chart
     _setCulture: function (culture) {
        this.culture = Globalize.culture(culture);
    },
	
	
     draw: function (excludeDataUpdate) {

        this._drawBackInterior();
        this.model.chartRegions = [];         
		this.model.outsideDataRegionPoints = [];
        this.model.regionCount = null;
        this.model.excludeDataUpdate = excludeDataUpdate;
		this.model.yAxisLabelRegions = [];
        this.model.xAxisLabelRegions = [];
        if (!excludeDataUpdate)
            this.model._visibleSeries = [];
        this.model.stackedValue = {};
		this._setCulture(this.model.locale);
		 var seriesLength = this.model.series.length;
        var series;
        var seriesType;
        if (ej.util.isNullOrUndefined(this.model.sideBySideSeriesPlacement))  // for setting sideBySideSeriesPlacement property values
            this.model._sideBySideSeriesPlacement = (this.model.enable3D) ? false : true;
        else
            this.model._sideBySideSeriesPlacement = this.model.sideBySideSeriesPlacement;
        for (var i = 0; i < seriesLength; i++)
        {
            series = this.model.series[i];
            seriesType = series.type.toLowerCase();
            series._isTransposed = (seriesType.indexOf("bar") == -1) ? series.isTransposed : !series.isTransposed;
        }
        if (seriesLength > 0)
            this.addedXYValues(excludeDataUpdate);
        this.model.AreaType = this.model.AreaType || "cartesianaxes";
        

        var commonpreRenderEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonpreRenderEventArgs.data = {};
        this._trigger("preRender", commonpreRenderEventArgs);

        this.gLegendEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_Legend' });

        this.gTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ChartTitle' });
        this.gSubTitleEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ChartsubTitle' });
        this._createAxisLabelAndRange();

        this._initializeSeriesColors();

        this._calculateLegendBounds();

        this.legendRender = new ej.EjLegendRender(this);

        var commonLegendEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonLegendEventArgs.data = { legendBounds: this.model.LegendBounds };
        this._trigger("legendBoundsCalculate", commonLegendEventArgs);

         // Assigning indicator points
        for (var j = 0; j < this.model.indicators.length; j++) {
            var indicator = this.model.indicators[j];
			indicator._points = [];
            indicator.xAxisName = (!(indicator.xAxisName)) ? this.model._axes[0].name : indicator.xAxisName;
            indicator.yAxisName = (!(indicator.yAxisName)) ? this.model._axes[1].name : indicator.yAxisName;
            indicator.isIndicator = true;
            if (indicator.dataSource) {
                indicator._hiloTypes = true;
                this._processJsonData(indicator.dataSource, indicator);
            } else if (indicator.points.length > 0) {
                for (var i = 0; i < indicator.points.length; i++) {
				     indicator._points = indicator.points;
                    indicator._points[i].xValue = indicator.points[i].x;
                    indicator._points[i].YValues = [];
                    indicator._points[i].YValues[0] = indicator.points[i].close;
                }
            }
            else if (indicator.seriesName != "") {
                for (var index = 0; index < this.model._visibleSeries.length; index++) {
                    var series = this.model._visibleSeries[index];
                    if (indicator.seriesName == series.name)
                        indicator._points = $.extend(true, [], series.points);
                }
            }
        }

         // calculate category points
        for (k = 0; k < this.model._axes.length; k++) {
            if (this.model._axes[k].valueType && this.model._axes[k].valueType.toLowerCase() == "category") {
                for (i = 0; i < this.model.indicators.length; i++) {
                    indicator = this.model.indicators[i];
                    if (this.model._axes[k].name == indicator.xAxisName) {
                        for (j = 0; j < indicator._points.length; j++)
                            indicator._points[j].xValue = j;
                    }
                }
            }
        }

         // calculate ranges for indicator
        this.model.indicatorRange = [];
        for (index = 0; index < this.model.indicators.length; index++) {
            indicator = this.model.indicators[index];
            if (indicator._points.length > 0) {
                type = indicator.type.toLowerCase();
              options = new ej.indicatorTypes[type]();
              options.calculateSegment(indicator, this);
         }
     }


         for (var m = 0; m < this.model._visibleSeries.length; m++) {
            var options = this.model._visibleSeries[m];
            for (i = 0; i < options.points.length; i++) {
                if (typeof options.points[i].xValue == "string" && options.points[i].xValue.indexOf("/Date(") != -1)
                    options.points[i].xValue = new Date(parseInt(options.points[i].xValue.substr(6)));
            }
        }
        this.model.hAxes = [];
        this.model.vAxes = [];
        var chartobj = this;
        var isStriplineOver = false, isStriplineBehind = false;
        this._arrangeAxis();
        for (var k = 0; k < this.model._axes.length; k++) {
            var axis = this.model._axes[k];
            var seriesCollection = [];
            var seriesLength = this.model.series.length;
            var orientation = this.model._axes[k].orientation.toLowerCase();
            var padding = this.model._axes[k].rangePadding.toLowerCase();
            if (padding == "auto") {
                if (orientation == "vertical") {
                    axis.rangePadding = (!this.model.requireInvertedAxes) ? "normal" : "none";
                }
                if (orientation == "horizontal") {
                    axis.rangePadding = (this.model.requireInvertedAxes) ? "normal" : "none";
                }
                for (var j = 0; j < seriesLength; j++) {
                    var type = this.model.series[j].type.toLowerCase();
                    if (type.indexOf("100") != -1) {
                        if (axis.name == this.model.series[j]._yAxisName) {
                            axis.rangePadding = "round"; break;
                        }
                    }
                }
            }          
            for (var slCount = 0; slCount < axis.stripLine.length; slCount++) {
                axis.stripLine[slCount] = $.extend(true, {}, this.model.stripLineDefault, axis.stripLine[slCount]);
                if (axis.stripLine[slCount].zIndex.toLowerCase() == 'over') {
                    isStriplineOver = true;
                } else {
                    isStriplineBehind = true;
                }
            }
            var axisOrientation = axis.orientation.toLowerCase();
            var zOrder = this.model._visibleSeries.length - 1;
            for (j = 0; j < this.model._visibleSeries.length; ++j) {
                zOrder -= 1;
                                      
                    var series = this.model._visibleSeries[j];
                    if((!series.xAxisName)){
                    series._xAxisName = chartobj.model._axes[0].name;
					 }
					if(!series.yAxisName){
                    series._yAxisName = chartobj.model._axes[1].name;
                    }
					 series._xAxisName=(!(series._xAxisName))?(!(series.xAxisName)?chartobj.model._axes[0].name:series.xAxisName):series._xAxisName;
                    series._yAxisName = (!(series._yAxisName)) ? (!(series.yAxisName)?chartobj.model._axes[1].name :series.yAxisName):series._yAxisName;
                    var xAxisName = series._xAxisName.toLowerCase();
                    var yAxisName = series._yAxisName.toLowerCase();
                    var name = axis.name.toLowerCase();
                    if (xAxisName || yAxisName) {
                        if (xAxisName == name || yAxisName == name) {
                            if (axisOrientation == "horizontal" && axis.valueType && axis.valueType.toString() == "datetime") {
                                for (var m = 0; m < series.points.length; m++) {
                                    if (typeof series.points[m].xValue == "string" && !isNaN(Date.parse(series.points[m].xValue)))
                                        series.points[m].xValue = new Date(Date.parse(series.points[m].xValue));
                                }
                            }
                            else {
                                if (series._xAxisValueType == "string" && !(axis.valueType))
                                    axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? 'category' : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'category' : 'double';

                                if (series._xAxisValueType == "date" && !(axis.valueType))
                                    axis._valueType = (axisOrientation == 'horizontal' && !this.model.requireInvertedAxes) ? "datetime" : (axisOrientation == 'vertical' && this.model.requireInvertedAxes) ? 'datetime' : 'double';
                            }
							if (this.model._visibleSeries[j].visibility.toLowerCase() == 'visible')
                            seriesCollection.push(series);
                        }
                    
                }
            }
            if (axisOrientation == ((!this.model.requireInvertedAxes) ? "vertical" : "horizontal")) {
                var isStacked100 = false;
                var isStacked = false;               
                var seriesLength = seriesCollection.length;
                axis.isStacked100 = (seriesCollection.length>0) ? true :false;
                for (var i = 0; i < seriesLength; i++) {
                    var type = seriesCollection[i].type.toLowerCase();
                    if (ej.seriesTypes[type].prototype.stackingSeries || (this.model.AreaType == 'polaraxes' && seriesCollection[i].drawType.toLowerCase() == 'column' && seriesCollection[i].isStacking)) {
                        if (type.indexOf("100") != -1 && (!isStacked100)) {
                            this._calculateStackingCumulativeValues(seriesCollection, axis);
                            isStacked100 = true;
                        } else if (type.indexOf("100") == -1 && (!isStacked)) {
                            this._calculateStackingValues(seriesCollection, axis);
                            isStacked = true;
                            axis.isStacked100 = false;
                        }                     
                    }
                    else {
                        axis.isStacked100 = false;
                    }
                }
            }
            axis._valueType = (!(axis._valueType)) ? (!(axis.valueType) ? "double" : axis.valueType) : axis._valueType;
            var axisRange = new ej.axisTypes[axis._valueType.toLowerCase()]();
            axisRange._calculateRanges(chartobj, axis, seriesCollection);
            var commonAxisEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAxisEventArgs.data = { range: axis.range };
            this._trigger("axesRangeCalculate", commonAxisEventArgs);
        }

        var commonAxesInitEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonAxesInitEventArgs.data = { axes: this.model._axes };
        this._trigger("axesLabelsInitialize", commonAxesInitEventArgs);

        this._calculateAreaBounds();

        if (this.model.enable3D) {

            var Ej3DRender = new ej.Ej3DRender();
            this.vector = new Ej3DRender.vector3D();
            this.matrixobj = new Ej3DRender.matrix3D();
            this.bsptreeobj = new Ej3DRender.BSPTreeBuilder();
            this.polygon = new Ej3DRender.polygon3D();
            this.graphics = new Ej3DRender.Graphics3D();

            this.chart3D = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_Chart3D' });
            ej.Ej3DRender.Polygons = [];
        }


        if (this.model.AreaType != "none") {        

            var commonAreaEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonAreaEventArgs.data = { areaBounds: this.model.m_AreaBounds };
            this._trigger("chartAreaBoundsCalculate", commonAreaEventArgs);

            this._drawChartAreaRect();

            if (this.model.title.text != "" && this.model.title.text)
                this._drawTitle();

            this._calculateAxisSize();

            this.gXaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxis' });

            this.gYaxisEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxis' });
     
            this.gPolarAxisEle = this.svgRenderer.createGroup( { 'id': this.svgObject.id + '_AxisLine' });
            if (this.model.enable3D) {

                $.extend(ej.Ej3DAxisRenderer.prototype, this);
                this.chartaxis = new ej.Ej3DAxisRenderer(this);
            }
            else {
                $.extend(ej.EjAxisRenderer.prototype, this);
                this.chartaxis = new ej.EjAxisRenderer(this);
            }

            var chartaxis = this.chartaxis;
			this.model.xAxisTitleRegion = [];
            this.model.yAxisTitleRegion = [];
            for (var l = 0; l < this.model._axes.length; l++) {
                chartaxis._drawAxes(l, this.model._axes[l]);
            }
          
            if (this.model.enable3D) {
                ej.Ej3DChart.prototype.update3DWall(this);
            }
          

            //Based on the column/bar series, modified primaryAxis append to the SVG Object
            if (!this.model.requireInvertedAxes) { 
                this.svgRenderer.append(this.gXaxisEle, this.svgObject);
                this.svgRenderer.append(this.gYaxisEle, this.svgObject);
            } else {
                this.svgRenderer.append(this.gYaxisEle, this.svgObject);
                this.svgRenderer.append(this.gXaxisEle, this.svgObject);
            }

            var x = (this.model.AreaType == 'polaraxes') ? 0: this.model.m_AreaBounds.X;
            var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;

            var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
            var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

            var clipRectOptions = {
                'id': this.svgObject.id + '_ChartAreaClipRect',
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'Gray'
            };

            this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);
			
			this._setZoomProperties();
        }

        else {
            this._drawChartAreaRect();

            clipRectOptions = {
                'id': this.svgObject.id + '_ChartAreaClipRect',
                'x': 0,
                'y': 0,
                'width': $(this.svgObject).width(),
                'height': $(this.svgObject).height(),
                'fill': 'white',
                'stroke-width': 1,
                'stroke': 'Gray'
            };

            this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);

            if (this.model.title.text != "" && this.model.title.text)
                this._drawTitle();
        }

        if (isStriplineBehind && this.model.AreaType == "cartesianaxes" && !this.model.enable3D) {
            this.gStriplineBehind = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineBehind', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
            this._renderStripline('behind');
        }

        if (this.model.initSeriesRender)
            this.seriesRender();

        if (this.model.initSeriesRender)
            this.indicatorRender();
         
		var annotationsLength = this.model.annotations.length; 
		for (var a = 0; a < annotationsLength; a++) {
            var currentAnnotation = this.model.annotations[a];
            if (currentAnnotation.visible) {
                this.annotationRender(currentAnnotation, a);
              }
        }
		
		if (this.model.AreaType == "polaraxes" && this.model._axes[1].visible && !this.model.enable3D) {
            this.chartaxis._drawAxisLine(this.model._axes[1]);
        }

        this.gXaxisTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_XAxisTick' });

        this.gYaxisTickEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_YAxisTick' });
      

        $.extend(ej.EjAxisRenderer.prototype, this);
        this.chartaxis = new ej.EjAxisRenderer(this);
        var chartaxis = this.chartaxis;
      
        for (var l = 0; l < this.model._axes.length; l++)
        {   
             	
            if ((this.model._axes[l].tickLinesPosition == 'inside' || this.model._axes[l].labelPosition=='inside') && (this.model.AreaType.toLowerCase() == "cartesianaxes")){
                if (this.model._axes[l].orientation.toLowerCase() == "horizontal")
                {
                    
                    chartaxis._drawXAxisTickLine(l, this.model._axes[l]);
                    chartaxis._drawXAxisLabels(l, this.model._axes[l]);

                    this.svgRenderer.append(this.gXaxisTickEle, this.svgObject);
                }
                else if (this.model._axes[l].orientation.toLowerCase() == "vertical")
                {
                    chartaxis._drawYAxisTickLine(l, this.model._axes[l]);
                    chartaxis._drawYAxisLabels(l, this.model._axes[l]);

                    this.svgRenderer.append(this.gYaxisTickEle, this.svgObject);
                }               
              
            }                     
        }         
       
             
       

        if (isStriplineOver && this.model.AreaType == "cartesianaxes" && !this.model.enable3D) {
            this.gStriplineOver = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_StriplineOver', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' });
            this._renderStripline('over');
        }

         if (!this.resetZooming && (this.zoomed || this.panning))
             this._enableZoomingButtons();
         this.resetZooming = false;
        var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonLoadedEventArgs.data = { model: this.model };
        this._trigger("loaded", commonLoadedEventArgs);


    },
	
	// set value to private variable for two way binding
	_setZoomProperties:function(){
	    for (var l = 0; l < this.model._axes.length; l++) {
			  var axis=this.model._axes[l];
               
		var _zf = axis.zoomFactor.toFixed(3);
        var _zp = axis.zoomPosition.toFixed(3);
        if (axis.name == this.model.primaryXAxis.name && (_zf!= this._xZoomFactor() ||_zp!= this._xZoomPosition() )) {
            this._xZoomFactor(axis.zoomFactor.toFixed(3));
            this._xZoomPosition(axis.zoomPosition.toFixed(3));
        }
        if (axis.name == this.model.primaryYAxis.name && (_zf != this._yZoomFactor() || _zp != this._yZoomPosition())) {
            this._yZoomFactor(axis.zoomFactor.toFixed(3));
            this._yZoomPosition(axis.zoomPosition.toFixed(3));
        }
				
        }
	},

	  annotationRender: function (currentAnnotation, a) {                           // method to render annotation
            //declaration and initialization
            var chartContainer = this._id;
            var flag = false;
            var currentSeries = this.model.series[0];
            var seriesType = currentSeries.type.toLowerCase();
            var areaBoundsX = this.model.m_AreaBounds.X;
            var areaBoundsY = this.model.m_AreaBounds.Y;
            var areaBoundsWidth = this.model.m_AreaBounds.Width;
            var areaBoundsHeight = this.model.m_AreaBounds.Height;
            var marginLeft = this.model.margin.left;
            var marginTop = this.model.margin.top;
            var centerX = this.model.centerX;
            var centerY = this.model.centerY;
            var radius = this.model.Radius;

            currentAnnotation = $.extend(true, {}, this.model.annotationsDefault, currentAnnotation);   // extending defaults

            // creating annotation group
            if ($('#annotation_group_' + chartContainer).length != 0)
                var annotationContainer = $('#annotation_group_' + chartContainer);
            else
                annotationContainer = $("<div></div>").attr('id', "annotation_group_" + chartContainer);
            annotationContainer.css({ "position": "absolute", "left": "0px", "top": "0px" });

            var content = currentAnnotation.content;
            element = $("#" + content);
            if (element.get(0).tagName == "SCRIPT")                // for jsrender
                var element = $($("#" + content)[0].innerHTML);
            var cloneNode = element.clone();                        // clone
            var id = 'annotation_' + chartContainer + "_" + content + "_" + a;
            $(cloneNode).attr("id", id);
            var $cloneNode = $(cloneNode);
            $cloneNode.css({
                'position': "absolute",
                'z-index': 2000
            });

            // initialization
            var region = currentAnnotation.region.toLowerCase();
            var coordinateUnit = currentAnnotation.coordinateUnit.toLowerCase();
            var annotationX = currentAnnotation.x;
            var annotationY = currentAnnotation.y;
            var horizontalAlignment = currentAnnotation.horizontalAlignment.toLowerCase();
            var verticalAlignment = currentAnnotation.verticalAlignment.toLowerCase();

            if (coordinateUnit == "pixels") {                          //pixels
                if (region == "series") {                             // chart area / series
                    if ((areaBoundsWidth + areaBoundsX + marginLeft > annotationX + areaBoundsX) &&
                        areaBoundsHeight + areaBoundsY + marginTop > annotationY + areaBoundsY) {
                        var left = annotationX + areaBoundsX;
                        var top = annotationY + areaBoundsY;
                    }
                }
                else if (region == "chart") {                         // chart
                    if (($(this.svgObject).width() > annotationX) &&
                         $(this.svgObject).height() > annotationY) {
                        left = annotationX;
                        top = annotationY;
                    }
                }

                $cloneNode.css({
                    "left": left,
                    "top": top
                });
                flag = true;                                // to denote that the annotation should be appended in DOM
            } else if (coordinateUnit == "points") {        //points
                point = {
                    'x': annotationX,
                    'y': annotationY
                };

                if (this.model.AreaType == "polaraxes") {   // polar
                    xAxis = this.model._axes[0];
                    yAxis = this.model._axes[1];
                    if ((annotationX >= xAxis.visibleRange.min && annotationX <= xAxis.visibleRange.max)
                        && (annotationY >= yAxis.visibleRange.min && annotationY <= yAxis.visibleRange.max)) {
                        var location = this.TransformToVisibleAnnotation(xAxis, yAxis, annotationX, annotationY, this);
                        $cloneNode.css({
                            "left": location.X,
                            "top": location.Y
                        });
                        flag = true;
                    }
                } else if (seriesType == "pyramid" || seriesType == "funnel") { // pyramid and funnel
                    var currentPoint = currentSeries.points[annotationX];
                    $cloneNode.css({
                        "left": currentPoint.xLocation + this.pyrX,
                        "top": currentPoint.yLocation + this.pyrY
                    });
                    flag = true;
                } else if (seriesType == "pie" || seriesType == "doughnut") { // pie and doughnut
                    currentPoint = currentSeries.visiblePoints[annotationX];
                    var pointIndex = annotationX;
                    var startX, startY;
                    var series = new ej.seriesTypes[seriesType]();
                    var midAngle = currentPoint ? currentPoint.currentMidAngle : 0;
                    if ((pointIndex == currentSeries.explodeIndex || currentSeries.explodeAll) && !chartObj.vmlRendering) {
                        startX = centerX + Math.cos(midAngle) * currentSeries.explodeOffset;
                        startY = centerY + Math.sin(midAngle) * currentSeries.explodeOffset;
                    } else {
                        startX = centerX;
                        startY = centerY;
                    }
                    var midX = series.getXCordinate(startX, (radius), midAngle);
                    var midY = series.getYCordinate(startY, (radius), midAngle);

                    positionX = (midX + startX) / 2;
                    positionY = (midY + startY) / 2;

                    positionX = (midX + positionX) / 2;
                    positionY = (midY + positionY) / 2;

                    $cloneNode.css({
                        "left": positionX,
                        "top": positionY
                    });
                    flag = true;
                } else {                                               // cartesianaxes
                    var xAxisName = currentAnnotation.xAxisName;
                    var yAxisName = currentAnnotation.yAxisName;
                    var axesLength = this.model._axes.length;
                    for (var i = 0; i < axesLength; i++) {
                        if (this.model._axes[i].name == xAxisName)
                            var xAxis = this.model._axes[i];
                        if (this.model._axes[i].name == yAxisName)
                            var yAxis = this.model._axes[i];
                    }
                    if (!xAxis) xAxis = this.model._axes[0];           // setting primary axis by default
                    if (!yAxis) yAxis = this.model._axes[1];

                    if ((annotationX >= xAxis.visibleRange.min && annotationX <= xAxis.visibleRange.max)
                        && (annotationY >= yAxis.visibleRange.min && annotationY <= yAxis.visibleRange.max)) {
                        if (!this.model.requireInvertedAxes) {
						    location = this._getAnnotationPoint(point, xAxis, yAxis);
                            $cloneNode.css({
                                "left": location.X + areaBoundsX,
                                "top": location.Y + yAxis.y
                            });
                        }
                        else {
                            point.x = point.y + (point.y = point.x, 0);// swapping x, y => a = b + (b=a, 0)
                            location = this._getAnnotationPoint(point, yAxis, xAxis);
                            $cloneNode.css({
                                 "left": location.X + areaBoundsX,
                                 "top": location.Y + xAxis.y
                            });
						}
                        flag = true;
                    }
                }
            } else {                                                                      //other alignment
                $cloneNode.css("display", "block").appendTo($(annotationContainer));       //append
                $(annotationContainer).appendTo('#' + 'chartContainer_' + chartContainer);
                width = $cloneNode.outerWidth();
                height = $cloneNode.outerHeight();
                if (region == "chart") {
                    switch (horizontalAlignment) {
                        case "middle":
                            left = $(this.svgObject).width() / 2 - width / 2;
                            break;
                        case "left":
                            left = 0;
                            break;
                        case "right":
                            left = $(this.svgObject).width() - width;
                    }
                    switch (verticalAlignment) {
                        case "middle":
                            top = $(this.svgObject).height() / 2 - height / 2;
                            break;
                        case "top":
                            top = 0;
                            break;
                        case "bottom":
                            top = $(this.svgObject).height() - height;
                    }
                } else {
                    if (seriesType == "pie" || seriesType == "doughnut") {             // for pie and doughnut
                        left = centerX;
                        top = centerY;
                        switch (horizontalAlignment) {
                            case "middle":
                                left = centerX - width / 2;
                                break;
                            case "left":
                                left = centerX - radius;
                                break;
                            case "right":
                                left = centerX + radius - width;
                        }

                        switch (verticalAlignment) {
                            case "middle":
                                top = centerY - height / 2;
                                break;
                            case "top":
                                top = centerY - radius;
                                break;
                            case "bottom":
                                top = centerY + radius;
                        }
                    } else {
                        switch (horizontalAlignment) {
                            case "middle":
                                left = areaBoundsX + (areaBoundsWidth / 2) - width / 2;
                                break;
                            case "left":
                                left = areaBoundsX;
                                break;
                            case "right":
                                left = areaBoundsX + areaBoundsWidth - width;
                        }

                        switch (verticalAlignment) {
                            case "middle":
                                top = areaBoundsY + (areaBoundsHeight / 2) - height / 2;
                                break;
                            case "top":
                                top = areaBoundsY;
                                break;
                            case "bottom":
                                top = areaBoundsY + areaBoundsHeight - height;
                        }
                    }
                }
                if (seriesType == "pyramid" || seriesType == "funnel") {         // to remove margin value
                    left -= marginLeft;
                }
                $cloneNode.css({
                    "left": left,
                    "top": top
                });
            }

            left = parseFloat($cloneNode.css("left"));
            top = parseFloat($cloneNode.css("top"));                              // get left and top value
            if (coordinateUnit != "none" && flag) {
                $cloneNode.css("display", "block").appendTo($(annotationContainer));   //append
                $(annotationContainer).appendTo('#' + 'chartContainer_' + chartContainer);
				width = $cloneNode.outerWidth();
                height = $cloneNode.outerHeight();
                switch (horizontalAlignment) {
                    case "middle":
                        left -= width / 2;
                        break;
                    case "left":
                        left -= width;
                        break;
                }
                switch (verticalAlignment) {
                    case "middle":
                        top -= height / 2;
                        break;
                    case "top":
                        top -= height;
                        break;
                }
            }
            var margin = currentAnnotation.margin;                      // margin values
            left = left + margin.left - margin.right;
            top = top + margin.top - margin.bottom;

            commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);          // trigger event
            commonEventArgs.data = {
                content: $cloneNode.html(), location: { x: left, y: top },
                opacity: currentAnnotation.opacity, angle: currentAnnotation.angle
            };
            this._trigger("annotationRendering", commonEventArgs);

            $cloneNode.html(commonEventArgs.data.content);
            $cloneNode.css({
                "left": commonEventArgs.data.location.x,
                "top": commonEventArgs.data.location.y,
                "opacity": commonEventArgs.data.opacity                  //opacity
            });

            var angle = commonEventArgs.data.angle;                     // angle
            if (angle) {
                $cloneNode.css({
                    '-ms-transform': 'rotate(' + angle + 'deg)',        //IE 10 and 9
                    '-webkit-transform': 'rotate(' + angle + 'deg)',    // chrome
                    '-moz-transform': 'rotate(' + angle + 'deg)',       // firefox
                    '-o-transform': 'rotate(' + angle + 'deg)',         // opera
                    '-transform': 'rotate(' + angle + 'deg)'            // general
                });
            }

            var eleId = $cloneNode.attr("id");                          // id of annotation
            var topHeight = bottomHeight = leftWidth = rightWidth = 0;
            var chartWidth = $("#" + this._id).width();
            var chartHeight = $("#" + this._id).height();
            if (flag && (angle == 0) && (region == "series" || coordinateUnit == "points")) {
                if (areaBoundsWidth + areaBoundsX < left + width) {                       // clip right width
                    var excessWidth = (left + width) - (areaBoundsWidth + areaBoundsX);
                    rightWidth = width - excessWidth;
                }
                if (left < areaBoundsX) {                                                  // clip left width
                    leftWidth = areaBoundsX - left;
                }
                if (areaBoundsHeight + areaBoundsY < top + height) {                      // clip bottom height
                    excessHeight = (top + height) - (areaBoundsHeight + areaBoundsY);
                    bottomHeight = height - excessHeight;
                }
                if (top < areaBoundsY) {                                                 // clip top height
                    topHeight = areaBoundsY - top;
                }
                rightWidth = (rightWidth == 0) ? width : rightWidth;
                bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
                document.getElementById(eleId).style.clip = "rect(" + topHeight.toString() + "px," +  // clipping annotation
                     rightWidth.toString() + "px," +
                     bottomHeight.toString() + "px," +
                     leftWidth + "px)";
            }
            else if (flag && (angle == 0) && region == "chart") {
                if (chartWidth < left + width) {                              // clip right width
                    excessWidth = (left + width) - chartWidth;
                    var rightWidth = width - excessWidth;
                }
                if (left < 0) {                                              // clip left width
                    var leftWidth = width + left;
                    leftWidth = (leftWidth == 0) ? width : leftWidth;
                }
                if (chartHeight < top + height) {                            // clip bottom height
                    var excessHeight = (top + height) - chartHeight;
                    var bottomHeight = height - excessHeight;
                }
                if (top < 0) {                                               // clip top height
                    topHeight = height + top;
                    topHeight = (topHeight == 0) ? height : topHeight;
                }
                rightWidth = (rightWidth == 0) ? width : rightWidth;
                bottomHeight = (bottomHeight == 0) ? height : bottomHeight;
                document.getElementById(eleId).style.clip = "rect(" +
                    topHeight.toString() + "px," +
                    rightWidth.toString() + "px," +
                    bottomHeight.toString() + "px," +
                    leftWidth + "px)";
            }
        },                                                                 // ---------------------------- end of annotation 

        TransformToVisibleAnnotation: function (xAxis, yAxis, x, y, sender) {       // method to get the position of points in polar/ radar
            x = (xAxis._valueType == "logarithmic") && x > 0 ? Math.log(x, xAxis.logBase) : x;
            y = (xAxis._valueType == "logarithmic") && y > 0 ? Math.log(y, yAxis.logBase) : y;
            var radius = sender.model.Radius * ej.EjSvgRender.utils._valueToCoefficient(yAxis, y, this);
            var point = ej.EjSvgRender.utils._valueToVector(xAxis, x);
            return { X: sender.model.centerX + radius * point.X, Y: sender.model.centerY + radius * point.Y };
        },

        _getAnnotationPoint: function (point, xAxis, yAxis) {                     // method to get the position of points (cartesianaxes)
            var requireInvertedAxes = this.model.requireInvertedAxes;
			 var x = point.x;
            var y = point.y;

            var xvalue = (xAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((x == 0 ? 1 : x), xAxis.logBase) : x;
            var yvalue = (yAxis._valueType == "logarithmic") ? ej.EjSvgRender.utils._logBase((y == 0 ? 1 : y), xAxis.logBase) : y;
            xvalue = ej.EjSvgRender.utils._getPointXY(xvalue, xAxis.visibleRange, xAxis.isInversed) * (xAxis.width);
            yvalue = (1 - ej.EjSvgRender.utils._getPointXY(yvalue, yAxis.visibleRange, yAxis.isInversed)) * (yAxis.height);

            point.location = { X: xvalue, Y: yvalue };
            return point.location;
        },
		
     indicatorRender: function() {
         if ($(this.svgObject).find("#" + this.svgObject.id + "_IndicatorCollection").length > 0) {
             $(this.svgObject).find("#" + this.svgObject.id + "_IndicatorCollection").remove();
         }

         var indOptions = { 'id': this.svgObject.id + '_IndicatorCollection', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' };

         this.gIndicatorEle = this.svgRenderer.createGroup(indOptions);

         for (var i = 0; i < this.model.indicators.length && this.model.indicators[i].visible && this.model.indicators[i]._points.length>0; i++) {
             var indicator = this.model.indicators[i];
             var options;
             for (var j = 0; j < this.model._axes.length; j++) {
                 var axis = this.model._axes[j];

                 if (axis.name && axis.orientation.toLowerCase() == 'horizontal' && axis.name.toLowerCase() == indicator.xAxisName.toLowerCase()) {
                     indicator.xAxis = axis;
                 } else if (axis.name && axis.orientation.toLowerCase() == 'vertical' && axis.name.toLowerCase() == indicator.yAxisName.toLowerCase()) {
                     indicator.yAxis = axis;
                 }
             }
             var type = indicator.type.toLowerCase();
             options = new ej.indicatorTypes[type]();
             //Clip the canvas Chart indicator series out of chartArea Bounds
             if (this.model.enableCanvasRendering) {
                 this.svgRenderer.ctx.save();
                 this.svgRenderer.ctx.beginPath();
                 this.svgRenderer.ctx.rect(this.model.m_AreaBounds.X, indicator.yAxis.y, indicator.xAxis.width, indicator.yAxis.height);
                 this.svgRenderer.ctx.clip();
                 options.draw(indicator, this);
                 this.svgRenderer.ctx.restore();
             } else
                 options.draw(indicator, this);
             var element = options.gIndicatorGroupEle;
             if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, indicator, this);

             if (!this.vmlRendering && !this.model.enableCanvasRendering) {
                 if (indicator.enableAnimation && (!indicator._animatedSeries || indicator.type != indicator._previousType)) {
                     this.svgRenderer.append(this.gIndicatorEle, this.svgObject);
                     options.doAnimation(indicator);
                     this.model.AnimationComplete = true;
                     indicator._animatedSeries = true;
                     indicator._previousType = indicator.type;

                 }
             }
	   	  this.svgRenderer.append(this.gIndicatorEle, this.svgObject); 
         }
		 
     },
    seriesRender: function () {
	     this.model.allPoints = [];
        this.model.markerRegion = [];
        var chartObj = this;
        if ($(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").length > 0) {
            $(this.svgObject).find("#" + this.svgObject.id + "_SeriesCollection").remove();
        }
        var serOptions = { 'id': this.svgObject.id + '_SeriesCollection', 'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)' };
        var enable3D = this.model.enable3D;
        var areaType = this.model.AreaType;
        var enableCanvasRendering = this.model.enableCanvasRendering;
		 var axesLength = this.model._axes.length;
        this.gSeriesEle = this.svgRenderer.createGroup(serOptions);
        var visibleSeries = ej.DataManager(this.model._visibleSeries, ej.Query().sortBy("_zOrder")).executeLocal();
		 var visibleSeriesLength=visibleSeries.length;
        for (var i = 0; i < visibleSeriesLength; i++) {
            if (visibleSeries[i].visibility == "visible") {
			    if(ej.util.isNullOrUndefined(this.model.series[i].enableSmartLabels))  // for setting enablesmartlabels property values
                     this.model.series[i]._enableSmartLabels = (this.model.AreaType) == "none" ? true : false;
			     else 
			         this.model.series[i]._enableSmartLabels = this.model.series[i].enableSmartLabels; 
            if (!ej.util.isNullOrUndefined(visibleSeries[i].xAxis)) {
                visibleSeries[i].xAxis = null;
                visibleSeries[i].yAxis = null;
            }
            for (var j = 0; j < axesLength; j++) {
                var axis = this.model._axes[j];
                var index = i;
				 var axisOrientation = axis.orientation.toLowerCase();
                if (axis.name && axisOrientation == 'horizontal' && axis.name.toLowerCase() == visibleSeries[i]._xAxisName.toLowerCase()) {
                    visibleSeries[i].xAxis = axis;
                }
                else if (axis.name && axisOrientation == 'vertical' && axis.name.toLowerCase() == visibleSeries[i]._yAxisName.toLowerCase()) {
                    visibleSeries[i].yAxis = axis;
                }
                if (this.model.requireInvertedAxes) {
                    if (axis.name && axisOrientation == 'vertical' && axis.name.toLowerCase() == visibleSeries[i]._xAxisName.toLowerCase()) {
                        visibleSeries[i].xAxis = axis;
                    }
                    else if (axis.name && axisOrientation == 'horizontal' && axis.name.toLowerCase() == visibleSeries[i]._yAxisName.toLowerCase()) {
                        visibleSeries[i].yAxis = axis;
                    }
                }

            }
        }
        }
        for (var m = 0; m < visibleSeriesLength; m++) {
            var index = m;
            var seriesOptions = visibleSeries[m];
			 var  seriesOptionsVisibility=seriesOptions.visibility.toLowerCase();
            if (areaType == "none" && seriesOptionsVisibility === 'visible' && !enable3D) {
                if (index === 0) {

                    var type = seriesOptions.type.toLowerCase();
                    var seriesIndex = $.inArray(visibleSeries[0], this.model._visibleSeries);
                    this.model._visibleSeries[seriesIndex] = visibleSeries[0] = $.extend(true, seriesOptions, visibleSeries[0]);
                    var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonEventArgs.data = { series: seriesOptions };
                    this._trigger("seriesRendering", commonEventArgs);
                    var seriesType = new ej.seriesTypes[type.toLowerCase()]();
                    var minhightwidth =seriesType.draw(chartObj, seriesOptions); 
                    if (!minhightwidth) {
                        if (seriesOptions.enableAnimation && !this.vmlRendering && !enableCanvasRendering && (!seriesOptions._animatedSeries || seriesOptions.type != seriesOptions._previousType)) {
                            seriesType.doAnimation(seriesOptions, seriesType);
                            seriesOptions._animatedSeries = true;
                            seriesOptions._previousType = seriesOptions.type;
                            }
                            this.drawAccDisplayText(seriesType, seriesOptions);

							//Show the datalabel text element on redraw when enable animation of series
                            if (seriesOptions.enableAnimation && seriesOptions.visiblePoints.length > 0 && seriesOptions.AnimationComplete  && !this.vmlRendering && !enableCanvasRendering) {
                            chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesTextEle.id), { "visibility": "visible" });
                            if (chartObj.gSymbolGroupEle)
                                chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSymbolGroupEle.id), { "visibility": "visible" });
                            var elements;
                            if (seriesOptions.marker.dataLabel.template) {
                                elements = $(chartObj.element[0].childNodes[0].childNodes);
                                for (var i = 0; i < elements.length; i++)
                                    $(elements[i]).css('display', 'block');
                    }
                            if (chartObj.gSeriesConnectorEle)
                                chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gSeriesConnectorEle.id), { "visibility": "visible" });
							
							 if (chartObj.gDataLabelEle)
                                chartObj.svgRenderer._setAttr($(chartObj.gSeriesEle).find("#" + chartObj.gDataLabelEle.id), { "visibility": "visible" });
                }
            }
                }
            }
            else {			 
                if (seriesOptionsVisibility === 'visible' && !enable3D) {
                    this.renderSeries(seriesOptions);
                    if (areaType == "cartesianaxes" && !this.model.requireInvertedAxes)
                        this._renderTrendline(chartObj, seriesOptions);
                }
                else {
                    if (!enable3D)
                        this.svgRenderer.append(this.svgObject, this.element);
                    else {
                        if (((areaType == "none" && index == 0) || areaType == "cartesianaxes") && seriesOptionsVisibility === 'visible' && !enableCanvasRendering)
                       ej.Ej3DChart.prototype.renderSeries(this, seriesOptions);
					 }
			      }
			  }
        }
		
		if (this.model.AreaType != "none"){
		 // smart labels for cartesian axis
		  var i, currentSeries, currentPoint, points, currentSeriesType;
		  var areaBounds = this.model.m_AreaBounds;
		  for (i = 0; i < visibleSeriesLength; i++) {
	        currentSeries = this.model.series[i];
	        points = currentSeries._visiblePoints;
	        if (currentSeries._enableSmartLabels && this.model.AreaType == "cartesianaxes") {
			   currentSeriesType = currentSeries.type.toLowerCase();
	            if (currentSeriesType == "column" || currentSeriesType == "stackingcolumn" ||
                    currentSeriesType == "bar" || currentSeriesType == "stackingbar"
					|| currentSeriesType == "stackingbar100" || currentSeriesType == "stackingcolumn100" || currentSeriesType == "rangecolumn")
	                this.cartesianColumnSmartLabels(currentSeries, points, i)
	            else {
	                this.cartesianSmartLabels(currentSeries, points, i);
				}
				    for (var j=0; j< points.length; j++){
					    currentPoint = points[j]; // condition to hide partially visible labels
						currentPoint.hide = false;
					    if ((currentPoint.xPos - currentPoint.width/2 < 0) || (currentPoint.xPos + currentPoint.width/2 > areaBounds.Width)||
						     (currentPoint.yPos + currentPoint.width/2 > areaBounds.Height)||(currentPoint.yPos - currentPoint.height/2 < 0))
							 currentPoint.hide = true;
		            }
			}
		 }
		
        // to draw connector lines for data label
		  var h, currentPoint;
		  var areaBoundsX = chartObj.model.m_AreaBounds.X;
		  for (i = 0; i < visibleSeriesLength; i++) {
		   currentSeries = visibleSeries[i];
		   if (currentSeries.visibility == "visible" && currentSeries.marker.dataLabel.visible) {
		       for (h = 0; h < currentSeries.points.length; h++) {
		           currentPoint = currentSeries.points[h];
		           if ((currentPoint.connectorFlag || currentPoint.newConnectorFlag) && !currentPoint.hide && currentPoint.drawText != "" && currentSeries.visibility == "visible" && !ej.util.isNullOrUndefined(currentPoint.textOptions)) {
                    if (chartObj.model.enableCanvasRendering) {
                        if (!chartObj.model.requireInvertedAxes)
                            chartObj.svgRenderer.ctx.rect(areaBoundsX, currentSeries.yAxis.y, currentSeries.xAxis.width, currentSeries.yAxis.height);
                        else
                            chartObj.svgRenderer.ctx.rect(areaBoundsX + currentSeries.yAxis.plotOffset, currentSeries.xAxis.y, currentSeries.yAxis.width, currentSeries.xAxis.height);
                        chartObj.svgRenderer.ctx.clip();
                        this.drawConnectorLines(i, h, currentPoint);
                    } else if (currentSeries._enableSmartLabels)
                        this.drawConnectorLines(i, h, currentPoint);
                }
            }
		  }
        }

        // to draw data label symbol and text
		  var currentPointLabel;
		  var areaBoundsX = chartObj.model.m_AreaBounds.X;
		  for (i = 0; i < visibleSeriesLength; i++) {
		      currentSeries = visibleSeries[i];
		      var type=currentSeries.type.toLowerCase();
		      if (currentSeries.visibility == "visible" && !this.model.enable3D) {
		          for (h = 0; h < currentSeries.points.length; h++) {
		              currentPoint = currentSeries.points[h];
					  currentPointLabel = currentPoint.marker && currentPoint.marker.dataLabel && currentPoint.marker.dataLabel.visible;
		              if ((currentSeries.marker.dataLabel.visible || currentPointLabel) && !currentPoint.hide && !currentPoint.dataLabeltemplate && currentSeries.visibility == "visible"  && !ej.util.isNullOrUndefined(currentPoint.textOptions) && !currentPoint.hide) {
                        currentPoint.textOptions.transform = 'rotate(' + currentPoint.textOptions.angle + ',' + (currentPoint.xPos) + ',' + (currentPoint.yPos) + ')';
                        currentPoint.textOptions.labelRotation = currentPoint.textOptions.angle;  //for canvas label rotation
                    if (chartObj.model.enableCanvasRendering) {    // for canvas
                        if (!chartObj.model.requireInvertedAxes)
                            chartObj.svgRenderer.ctx.rect(areaBoundsX, currentSeries.yAxis.y, currentSeries.xAxis.width, currentSeries.yAxis.height);
                        else
                            chartObj.svgRenderer.ctx.rect(areaBoundsX + currentSeries.yAxis.plotOffset, currentSeries.xAxis.y, currentSeries.yAxis.width, currentSeries.xAxis.height);
                        chartObj.svgRenderer.ctx.clip();
                        ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPos, currentPoint.yPos, currentPoint.width, currentPoint.height, currentPoint.symbolName, this)
                        this.svgRenderer.drawText(currentPoint.textOptions, currentPoint.drawText, this.gSeriesTextEle);
                    } else {
                        ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPos, currentPoint.yPos, currentPoint.width, currentPoint.height, currentPoint.symbolName, this, this.gDataLabelEle[i])
                        this.svgRenderer.drawText(currentPoint.textOptions, currentPoint.drawText, this.gSeriesTextEle[i]);
                    }
		                  if (type == "rangecolumn" || type == "rangearea") {
		                      
		                      currentPoint.textOptionsLow.transform = 'rotate(' + currentPoint.textOptionsLow.angle + ',' + (currentPoint.xPosLow) + ',' + (currentPoint.yPosLow) + ')';
		                      currentPoint.textOptionsLow.labelRotation = currentPoint.textOptionsLow.angle;  //for canvas label rotation
		                      if (chartObj.model.enableCanvasRendering) {    // for canvas
		                          if (!chartObj.model.requireInvertedAxes)
		                              chartObj.svgRenderer.ctx.rect(areaBoundsX, currentSeries.yAxis.y, currentSeries.xAxis.width, currentSeries.yAxis.height);
		                          else
		                              chartObj.svgRenderer.ctx.rect(areaBoundsX + currentSeries.yAxis.plotOffset, currentSeries.xAxis.y, currentSeries.yAxis.width, currentSeries.xAxis.height);
		                          chartObj.svgRenderer.ctx.clip();
		                          
		                          ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPosLow, currentPoint.yPosLow, currentPoint.widthLow, currentPoint.heightLow, currentPoint.symbolName, this)
		                          
		                          this.svgRenderer.drawText(currentPoint.textOptionsLow, currentPoint.drawTextLow, this.gSeriesTextEle);
		                      }
		                      else {
		                          
		                          ej.EjSeriesRender.prototype.dataLabelSymbol(i, currentSeries, h, currentPoint.xPosLow, currentPoint.yPosLow, currentPoint.widthLow, currentPoint.heightLow, currentPoint.symbolName, this, this.gDataLabelEle[i])
		                          this.svgRenderer.drawText(currentPoint.textOptionsLow, currentPoint.drawTextLow, this.gSeriesTextEle[i]);

		                      }
		                  }
                }
            }
		  }
        }

        // to append connector lines
		var i, currentSeries;
        for (i = 0; i < visibleSeriesLength; i++) {
		    currentSeries = visibleSeries[i];
            if (currentSeries.visibility == "visible" && currentSeries.marker.dataLabel.visible && currentSeries.connectorLines) {
                this.svgRenderer.append(this.gConnectorEle[i], this.gSeriesEle);
                currentSeries.connectorLines = false;
            }
		}
		
        // to append marker, data label symbol and text
		var index;
		var length = this.model._visibleSeries.length;
        for (i = 0; i < length; i++) {
		    currentSeries = this.model._visibleSeries[i];
		    index = !ej.util.isNullOrUndefined(currentSeries.zOrder) ? currentSeries.zOrder: index;
            if (currentSeries.visibility == "visible" && !this.model.enable3D) {
                this.svgRenderer.append(this.gSymbolGroupEle[i], this.gSeriesEle);
                if (!this.model.enable3D && !this.model.enableCanvasRendering) {
				    if(this.gDataLabelEle[i].childNodes.length > 0)
                        this.svgRenderer.append(this.gDataLabelEle[i], this.gSeriesEle);
					if(this.gSeriesTextEle[i].childNodes.length > 0)
                        this.svgRenderer.append(this.gSeriesTextEle[i], this.gSeriesEle);
                }
            }
          }
        }
		var length = (this.model.AreaType=="none") ? 1 : this.model._visibleSeries.length;
		var s, series, templateContainer, dataLabel;
		for (s = 0; s < length; s++){
		 options = this.model._visibleSeries[s];
		 if (!this.vmlRendering && options.visibility =="visible" && !this.model.enableCanvasRendering && !this.model.enable3D) {
            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
            this.svgRenderer.append(this.svgObject, this.element);
			 series = new ej.seriesTypes[options.type.toLowerCase()]();
			 series.chartObj = chartObj;
			 series.gSeriesGroupEle = $("#" + this.svgObject.id + "_SeriesGroup_" + s)[0];
            if (options.enableAnimation && (!options._animatedSeries || options._previousType != options.type)) {
                options._animatedSeries = true;
                options._previousType = options.type;
			    templateContainer = $('#template_group_' + this._id); 
                dataLabel=options.marker.dataLabel;				
               if (templateContainer.children().length != 0 && (dataLabel.visible && dataLabel.template))	 	
                   series.animateLabelTemplate(options);
                   series.doAnimation(options);   // to animate series
            }
         }
		}
		
		// to append trend line to DOM and perform animation
		  var visibleSeries = this.model._visibleSeries;
		  var length = visibleSeries.length;
		  if (!this.model.enableCanvasRendering && !this.model.enable3D){
		    for (var k =0; k <length; k++){
		          if (visibleSeries[k].visibility.toLowerCase() == "visible" && this.gTrendlinesGroupEle && 
				      this.gTrendlinesGroupEle[k] && this.gTrendlinesGroupEle[k].childElementCount != 0)
		         this.svgRenderer.append(this.gTrendlinesGroupEle[k], this.svgObject);
			}
		  }
		 
		  var trendlines, trendline, trendlineType, options, i, j, trendLineLength;
		  for (i = 0; i < length; i++){
		    currentSeries = this.model._visibleSeries[i];
			trendlines = currentSeries.trendlines;
			trendLineLength = currentSeries.trendlines.length;
			for (j = 0; j < trendLineLength; j++) {
              trendline = trendlines[j];
			  trendlineType = trendline.type.toLowerCase();
			  options = new ej.trendlineTypes[trendlineType]();
			  if (trendline.visibility.toLowerCase() === 'visible' && trendline.points && trendline.points.length > 1 &&
			   currentSeries.enableAnimation && !this.vmlRendering && (!currentSeries._animatedTrendline || trendline._previousType != trendline.type)) {
                        trendline._previousType = trendline.type;
                        options.doAnimation(currentSeries, this, i, j);
                        currentSeries._animatedTrendline = (trendLineLength - 1 == i) ? true : false;
                }
			}
		  }
	
	      var seriesEle, symbolEle;
		  var length = this.model._visibleSeries.length;
		  var svgObjectId = this.svgObject.id;
		  for (var i =0; i <length; i++){
		     var seriesEle = $("#" + svgObjectId + "_SeriesGroup_" + i);
			 var symbolEle = $("#" + svgObjectId + "_symbolGroup_" + i);
			 if (seriesEle.length >0 && symbolEle.length >0)
			    symbolEle.insertAfter(seriesEle);    
		  }
		  
	
		  this.model.AnimationComplete = true;
          var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
          commonEventArgs.data = {series : options};
          this._trigger("animationComplete", commonEventArgs);
		
		if (enableCanvasRendering) {
            this.svgRenderer.append(this.svgObject, this.element);
            if (this.panning) $("#" + this._id+ "_canvas").css({ "cursor": "pointer" });
        }       
		if (enable3D && !enableCanvasRendering) {
          this.svgRenderer.append(this.chart3D, this.svgObject);
           
          var size = { Width: $(this.svgObject).width(), Height: $(this.svgObject).height() };
          this.graphics.prepareView(this.model.perspectiveAngle, this.model.depth, this.model.rotation, this.model.tilt, size);
          this.graphics.view(this.svgObject, this);

          //for (var i = 0; i < visibleSeries.length; i++) {
          //    if (visibleSeries[i].visibility.toLowerCase() == "visible" && visibleSeries[i].enableAnimation && !(this.model.animated)) {
          //        var seriesRendering = new ej.Ej3DSeriesRender();
          //        var type = visibleSeries[i].type.toLowerCase();
          //        var options = ej.series3DTypes[type];
          //        if (this.model.AreaType == "none" && i > 0)
          //            break;
          //        options.doAnimation(visibleSeries[i], seriesRendering);
          //    }

          //}

        }
        if (this.svgRenderer.vmlNamespace && (areaType != "none"))
            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
        if (areaType == 'polaraxes')
            this.svgRenderer.append(this.gPolarAxisEle, this.svgObject);
        if ($(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").length > 0) {
            var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_StriplineOver").detach();
            detachEle.appendTo(this.svgObject);
        }
		
		this.legendRender.drawLegend();
		
        chartObj.model.initSeriesRender = true;
    },

     //Draw the trendline series
    _renderTrendline: function (chartObj, seriesOptions) {
        var seriesIndex = $.inArray(seriesOptions, chartObj.model.series);
        var translate = null;

        var transX = seriesOptions.xAxis.x;
        var transY = seriesOptions.yAxis.y;
        translate = 'translate(' + transX + ',' + transY + ')';
        var trendlinesOptions = { 'id': chartObj.svgObject.id + '_TrendlinesGroup' + '_' + seriesIndex, 'clip-path': 'url(#' + chartObj.svgObject.id + '_TrendlinesGroup' + '_' + seriesIndex + '_ClipRect)' };
        if (!chartObj.gTrendlinesGroupEle) chartObj.gTrendlinesGroupEle = [];
        chartObj.gTrendlinesGroupEle[seriesIndex] = chartObj.svgRenderer.createGroup(trendlinesOptions);
        var trendlines = seriesOptions.trendlines;
        var options, trendline;
        var length = trendlines.length, trendOptions, trendlineType, commonEventArgs, backwardPoints, forwardPoints;
        for (var i = 0; i < length; i++) {
            trendline = trendlines[i];
            trendline.seriesIndex = seriesIndex;
            if (trendline.visibility.toLowerCase() === 'visible' && trendline.points && trendline.points.length > 1) {
                trendOptions = { 'id': chartObj.svgObject.id + '_TrendGroup' + '_' + seriesIndex + '_' + i, 'transform': translate, 'clip-path': 'url(#' + chartObj.svgObject.id + '_TrendGroup' + '_' + seriesIndex + '_' + i + '_ClipRect)' };
                seriesOptions.gTrendGroupEle = null;
                seriesOptions.gTrendGroupEle = chartObj.svgRenderer.createGroup(trendOptions);
                trendlineType = trendline.type.toLowerCase();
                options = new ej.trendlineTypes[trendlineType]();

                ////Trigger Trendline rendering events
                commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                backwardPoints = trendline.points[0];
                forwardPoints = trendline.points[trendline.points.length - 1];
                commonEventArgs.data = { series: seriesOptions, trendline: trendline, forwardForecastPoints: forwardPoints, backwardForecastPoints: backwardPoints };
                this._trigger("trendlineRendering", commonEventArgs);

                if (chartObj.model.enableCanvasRendering) {
                    this.svgRenderer.ctx.save();
                    this.svgRenderer.ctx.beginPath();
                    this.svgRenderer.ctx.rect(chartObj.model.m_AreaBounds.X, seriesOptions.yAxis.y, seriesOptions.xAxis.width, seriesOptions.yAxis.height);
                    this.svgRenderer.ctx.clip();
                    this.svgRenderer.ctx.beginPath();
                    options.draw(trendline, seriesOptions, chartObj);
                    this.svgRenderer.ctx.restore();
                } else {
                    options.draw(trendline, seriesOptions, chartObj);
                    //Add axes bounds clip path for series
                    var element = seriesOptions.gTrendGroupEle;
                    if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, seriesOptions, this);
                }
            }
        }

    },
    _renderStripline: function (zIndex) {
        $.extend(ej.EjStripline.prototype, this);
        this.chartstripline = new ej.EjStripline(this);
        var stripline = this.chartstripline;
		 this.model._stripeline = false;
        for (var sl = 0; sl < this.model._axes.length; sl++) {
            var slAxis = this.model._axes[sl];
            for (var slCount = 0; slCount < slAxis.stripLine.length; slCount++) {
                if (slAxis.stripLine[slCount].zIndex.toLowerCase() == zIndex)
                    stripline._drawStripline(slAxis, slAxis.stripLine[slCount]);
            }
        }
        if (zIndex == 'over' && this.model._stripeline) {

            this.svgRenderer.append(this.gStriplineOver, this.svgObject);
        } else if(this.model._stripeline){
            this.svgRenderer.append(this.gStriplineBehind, this.svgObject);
        }
    },

    renderSeries: function (options) {

        var chart = this;
        var axis;
        var index;
        var seriesIndex = $.inArray(options, this.model._visibleSeries);

        for (var i = 0; i < options.points.length; i++) {
            if (typeof options.points[i].x == "string" && options.points[i].x.indexOf("/Date(") != -1)
                options.points[i].x = new Date(parseInt(options.points[i].x.substr(6)));
        }
        options.xAxis = (options.xAxis === null || options.xAxis === undefined) ? this.model._axes[0] : options.xAxis;
        options.yAxis = (options.yAxis === null || options.yAxis === undefined) ? this.model._axes[1] : options.yAxis;
        var type = options.type.toLowerCase();
		var drawtype = options.drawType.toLowerCase();
        var symbolOptions;
        var translate = null;
        var visibility;

        if (this.model.AreaType == "cartesianaxes") {
            var trans = ej.EjSvgRender.utils._getTransform(options.xAxis, options.yAxis, this.model.requireInvertedAxes);
            var translate = 'translate(' + trans.x + ',' + trans.y + ')';
            visibility = 'visible';
        }
        else {
            visibility = (options.enableAnimation) ? 'hidden' : 'visible'
        }
        var txtOptions = { 'id': this.svgObject.id + '_TextGroup_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        var dataLabelOptions = { 'id': this.svgObject.id + '_DataLabel_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        if (type != "scatter")
            symbolOptions = { 'id': this.svgObject.id + '_symbolGroup_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        else
            symbolOptions = { 'id': this.svgObject.id + '_SeriesGroup' + '_' + seriesIndex, 'transform': translate };
        var connectorOptions = { 'id': this.svgObject.id + '_ConnectorGroup_' + seriesIndex , 'transform': translate, 'visibility': visibility };
        if (!this.gSymbolGroupEle) this.gSymbolGroupEle = [];
        this.gSymbolGroupEle[seriesIndex] = this.svgRenderer.createGroup(symbolOptions);
	    if (!this.gSeriesTextEle) this.gSeriesTextEle = [];
        this.gSeriesTextEle[seriesIndex] = this.svgRenderer.createGroup(txtOptions);
		if (!this.gConnectorEle) this.gConnectorEle = [];
        this.gConnectorEle[seriesIndex] = this.svgRenderer.createGroup(connectorOptions);
		if (!this.gDataLabelEle) this.gDataLabelEle = [];
        this.gDataLabelEle[seriesIndex] = this.svgRenderer.createGroup(dataLabelOptions);

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = { series: options };
        this._trigger("seriesRendering", commonEventArgs);

        var series = new ej.seriesTypes[type]();
        if (this.model.enableCanvasRendering && this.model.AreaType != "none") {
            this.svgRenderer.ctx.save();
            this.svgRenderer.ctx.beginPath();
            if (!this.model.requireInvertedAxes) { //checked for bar and stackingbar series
                this.canvasX = options.xAxis.x;
                this.canvasY = options.yAxis.y;
                this.svgRenderer.ctx.rect(options.xAxis.x, options.yAxis.y, options.xAxis.width, options.yAxis.height);
            } else {
                var translate = ej.EjSvgRender.utils._getTransform(options.xAxis, options.yAxis, this.model.requireInvertedAxes);
                this.canvasX = translate.x;
                this.canvasY = translate.y;
                this.svgRenderer.ctx.rect(options.yAxis.x, options.xAxis.y, options.yAxis.width, options.xAxis.height);
            }
            this.svgRenderer.ctx.clip();
            series.draw(this, options);
			this.svgRenderer.ctx.restore();
        }
        else if (this.model.enableCanvasRendering && this.model.AreaType == "polaraxes") {
            this.svgRenderer.ctx.save();
            this.svgRenderer.ctx.beginPath();
            this.svgRenderer.ctx.arc(this.model.centerX, this.model.centerY, this.model.Radius, 0, 2 * Math.PI, false);
            this.svgRenderer.ctx.clip();
            this.canvasX = this.canvasY = 0;
            series.draw(this, options);
            this.svgRenderer.ctx.restore();
        }
        else {
            this.canvasX = this.canvasY = 0;
            if (this.model.AreaType == "cartesianaxes")
                series.draw(this, options);
            else if (this.model.AreaType == "polaraxes" && (options._xAxisName == this.model._axes[0].name && options._yAxisName == this.model._axes[1].name))
                series.draw(this, options);
        }
        
        //Add axes bounds clip path for series
        var element = series.gSeriesGroupEle;
        if (element) ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);

        // Draw Symbols       
            if (!(type == "candle" || type == "hilo" || type == "hiloopenclose")) {
                var chartObj = this;
				var point1;
				var areatype = chartObj.model.AreaType.toLowerCase();
				var regiontype = ej.seriesTypes[type].prototype.isRegion;
				if (!(type == "scatter")){
				    $.each(options.points, function (pointIndex, point) {
				        if ((point.visible) && (point.x !== "") && ((point.marker && point.marker.visible) || (options.marker.visible))) {
				            //this condition provide for better Scatter performance
				            if ((areatype == "polaraxes" && drawtype == "column") || regiontype)
				                point1 = point.symbolLocation;
				            else
				                point1 = (areatype != "polaraxes")
                                ? ej.EjSvgRender.utils._getPoint(point, options) : ej.EjSvgRender.utils.TransformToVisible(options, point.xValue, point.y, chartObj)
				            if (areatype == "polaraxes") {
				                var labelFormat = options.yAxis.labelFormat ? options.yAxis.labelFormat : "";
				                var pointText = (point.text) ? point.text : point.y + labelFormat.substring(labelFormat.indexOf('}') + 1);
				                var textOffset = ej.EjSvgRender.utils._measureText(pointText, null, options.marker.dataLabel.font);
				            }

				            series.drawSymbol(seriesIndex, options, pointIndex, point1.X, point1.Y);
				        }
                });
				}
                if (type != "scatter") {
                    //Add axes bounds clip path for marker
                    if (options.marker.visible) {
                        element = this.gSymbolGroupEle[seriesIndex];
                       ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                    }
                    // Add axes bounds clip path for marker text
                    if (options.marker.dataLabel.visible) {
                        element = this.gSeriesTextEle[seriesIndex];
                        ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
						 // for data label shape
						 element = this.gDataLabelEle[seriesIndex];
						 ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                      }
					// Add axes bounds clip path for connector line
					if (!ej.util.isNullOrUndefined(this.gConnectorEle) && this.gConnectorEle.length >0) {
					    element = this.gConnectorEle[seriesIndex];
                        ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
					}
                } else {
                    element = this.gSymbolGroupEle[seriesIndex];
                    ej.EjSvgRender.utils._drawAxesBoundsClipPath(element, options, this);
                    this.svgRenderer.append(this.gSymbolGroupEle, this.gSeriesEle);

                    if (options.tooltip.visible || type == "scatter") {
                        if (!this.model.enableCanvasRendering)
                            var elements = $(this.gSymbolGroupEle).children().not("defs");
                        else
                            elements = this.model.series[seriesIndex].points;
                        if (this.vmlRendering) {
                            this.cloneSeriesEle = $(this.gSeriesEle).clone();
                            this.cloneobj = $(this.element).clone();
                            this.svgclone = $(this.svgObject).clone();
                            $(document.body).append(this.cloneobj);
                            this.svgRenderer.append(this.cloneSeriesEle, this.svgclone);
                            this.svgRenderer.append(this.svgclone, this.cloneobj);
                        }
                        else {
                            this.svgRenderer.append(this.gSeriesEle, this.svgObject);
                            this.svgRenderer.append(this.svgObject, this.element);
                        }

                        if (this.vmlRendering) {
                            $(this.cloneobj).remove();
                            $(this.svgclone).remove();
                        }
                    }
                }
            }

        

        //Draw Series Text

        this.drawDisplayText(series, options);

    },
	
	drawDisplayText: function (series, options){
	    // method to draw text 
	   
	        var chartObj = this;
			var point1;
			var type = options.type.toLowerCase();
	        var areatype = chartObj.model.AreaType.toLowerCase();
	        var regiontype = ej.seriesTypes[type].prototype.isRegion;
	        var drawtype = options.drawType.toLowerCase();
	        var pointMarker;
	        var seriesIndex = $.inArray(options, chartObj.model._visibleSeries);
	        if (ej.util.isNullOrUndefined(chartObj.model.regionCount))
                chartObj.model.regionCount = -1;	        
	         
            var isCanvas = chartObj.model.enableCanvasRendering;
            var isPolar = chartObj.model.AreaType.toLowerCase() == 'polaraxes';
            $.each(options.points, function (pointIndex, point) {
                pointMarker = point.marker;
	            if ((areatype == "polaraxes" && drawtype == "column") || regiontype)
	                 point1 = point.symbolLocation;
	            else
	                 point1 = (areatype != "polaraxes")
                     ? ej.EjSvgRender.utils._getPoint(point, options) : ej.EjSvgRender.utils.TransformToVisible(options, point.xValue, point.y, chartObj)
	            if (point.visible && ((pointMarker && pointMarker.dataLabel && pointMarker.dataLabel.visible) || (!pointMarker || !pointMarker.dataLabel) && options.marker.dataLabel.visible)) {
	                if (regiontype)
	                 chartObj.model.regionCount++;
	                 
                     var type = chartObj.model.series[seriesIndex].type.toLowerCase();
					 //we have reset the chartRegions array based on the zooming points for column and stackingColumn series only
                     if (chartObj.zoomed && (type == "column" || type == "stackingcolumn" || type == "stackingcolumn100")) {
	                    for (var i = 0; i < chartObj.model.chartRegions.length; i++) {
	                        if (seriesIndex == chartObj.model.chartRegions[i].SeriesIndex && pointIndex == chartObj.model.chartRegions[i].Region.PointIndex) {
	                            if (isCanvas && chartObj.model.AreaType == "cartesianaxes") {
	                                chartObj.svgRenderer.ctx.save();
	                                chartObj.svgRenderer.ctx.beginPath();
	                                if (!chartObj.model.requireInvertedAxes) // checked for bar and stackingbar
	                                    chartObj.svgRenderer.ctx.rect(options.xAxis.x, options.yAxis.y, options.xAxis.width, options.yAxis.height);
	                                else
	                                    chartObj.svgRenderer.ctx.rect(options.yAxis.x, options.xAxis.y, options.yAxis.width, options.xAxis.height);
	                                chartObj.svgRenderer.ctx.clip();
	                                series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, i);
	                                chartObj.svgRenderer.ctx.restore();
	                            }
                                else
	                              series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, i);
	                        }
	                    }
	                }
	                else {
                         if (isCanvas && chartObj.model.AreaType == "cartesianaxes") {
	                        chartObj.svgRenderer.ctx.save();
                             chartObj.svgRenderer.ctx.beginPath();
	                        if (!chartObj.model.requireInvertedAxes) // checked for bar and stackingbar
	                            chartObj.svgRenderer.ctx.rect(options.xAxis.x, options.yAxis.y, options.xAxis.width, options.yAxis.height);
	                        else
	                            chartObj.svgRenderer.ctx.rect(options.yAxis.x, options.xAxis.y, options.yAxis.width, options.xAxis.height);
	                        chartObj.svgRenderer.ctx.clip();
	                        series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount);
	                        if (type == "rangecolumn" || type == "rangearea") {	                           
	                            point.y = point.low;
	                            series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount);
	                            point.y = point.high;
	                        }
	                        chartObj.svgRenderer.ctx.restore();
                         }
                         else {
                             if(isCanvas && isPolar){
                                 chartObj.svgRenderer.ctx.save();
                                 chartObj.svgRenderer.ctx.beginPath();
                                 chartObj.svgRenderer.ctx.arc(chartObj.model.centerX, chartObj.model.centerY, chartObj.model.Radius, 0, 2 * Math.PI, false);
                                 chartObj.svgRenderer.ctx.clip();
								 point1.X -= chartObj.canvasX;
								 point1.Y -= chartObj.canvasY;
                             }
                             series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount);
                                 if (type == "rangecolumn" || type == "rangearea") {
                                     point.y = point.low;
                                     series.drawDataLabel(options, pointIndex, point, point1.X, point1.Y, chartObj.model.regionCount);
                                     point.y = point.high;
                                 }
                             if (isCanvas && isPolar)
                                 chartObj.svgRenderer.ctx.restore();
                         }
	                }
	            }
	        });
	},
	
	// method to avoid collision between data labels 
	cartesianSmartLabels: function (currentSeries, points, seriesIndex) {
        // declaration
	    var length = points.length;
	    var count = 0;
	    connectorFlag = false;
	    this.processCount = 1;
	    top = [], rightTop = [],
	    left = [], leftTop = [],
	    right = [], rightTop = [],
	    rightBottom = [], leftBottom = [],
	    bottom = [];
		var markerWidth, markerHeight, i, d, c, currentPoint, location,
		ele, leftValue, topValue;
	    // loop to store marker region
	    for (d = 0; d < this.model.series.length; d++) {
		    markerWidth = this.model.series[d].marker.size.width;
	        markerHeight = this.model.series[d].marker.size.height;
	        if (this.model.series[d].marker.visible) {
	            for (c = 0; c < this.model.series[d].points.length; c++) {
	                currentPoint = this.model.series[d].points[c];
					location = currentPoint.location;
	                this.model.markerRegion[this.model.markerRegion.length] = { seriesIndex: d, xPos: location.X + this.canvasX, yPos: location.Y + this.canvasY, width: markerWidth, height: markerHeight };
	            }
	        }
	    }
		
		var currentPoint, j, padding = 10, diff;
		var areaBounds = this.model.m_AreaBounds;
		var width, height, xPos, yPos;
		for (j=0; j< length; j++){   // to reposition the partially visible data labels
		    currentPoint = points[j];
			currentPoint.hide = false;
			xPos = currentPoint.xPos; yPos = currentPoint.yPos;
			width = currentPoint.width; height = currentPoint.height;
		    if (xPos - width/2 < 0 && xPos > -width/2){
		         diff = xPos - width/2;
		         currentPoint.textOptions.x = currentPoint.xPos = xPos + Math.abs(diff) + 10;
				 currentPoint.newConnectorFlag  = true;
		    }
			if (xPos + width/2 > areaBounds.Width && xPos + width/2 < areaBounds.Width + width/2){
			     diff = xPos - width/2;
		         currentPoint.textOptions.x = currentPoint.xPos = areaBounds.Width - width/2 -10;
				 currentPoint.newConnectorFlag  = true;
			}
		    if (yPos + width/2 > areaBounds.Height && yPos + height/2 < areaBounds.Height + height/2){
			     diff = yPos + height/2 - areaBounds.Height;
		         currentPoint.yPos = yPos - diff - 10 ;
				 currentPoint.textOptions.y = currentPoint.yPos + 5;
				 currentPoint.newConnectorFlag  = true;
			}
			 if (yPos - height/2 < 0 && yPos > -height/2){
		         diff = yPos - height/2;
		         currentPoint.yPos = currentPoint.yPos + Math.abs(diff) + 10;
				 currentPoint.textOptions.y = currentPoint.yPos +5; 
				 currentPoint.newConnectorFlag  = true;
		    }
		}
		
	    // loop to compare the points
	    for (i = 0; i < length; i++) {
	        if (!ej.util.isNullOrUndefined(points[i].yPos))
	          this.model.allPoints[this.model.allPoints.length] = points[i];
		    else
		      continue;
	        connectorFlag = false;
	        this.compareDataLabels(i, points, count, this.processCount); 
	        currentPoint = points[i];
	        currentPoint.connectorFlag = connectorFlag; // flag to draw connector lines
	        if (points[i].dataLabeltemplate) {    // for repositioning data label template
	            ele = $("#" + points[i].id);
	            leftValue = points[i].xPos;
	            topValue = !this.model.enableCanvasRendering ? points[i].yPos : points[i].yPos + this.model.m_AreaBounds.Y;
	            ele.css("left", leftValue).css("top", topValue);
	            if (this.model.AreaType == "cartesianaxes") {   // to avoid template rendering outside area bounds
	                areaBounds = this.model.m_AreaBounds;
	                xPosition = areaBounds.X + (this.model.requireInvertedAxes ? currentSeries.yAxis.plotOffset : currentSeries.xAxis.plotOffset);
	                yPosition = currentSeries.yAxis.y;
	                width = points[i].width;
	                height = points[i].height;
	                if ((leftValue > xPosition + areaBounds.Width || leftValue + width < xPosition || topValue + height < yPosition || topValue + height > areaBounds.Y + areaBounds.Height) ||
                        (this.zoomed && (leftValue < xPosition || topValue < yPosition || leftValue > xPosition + areaBounds.Width || topValue > yPosition + areaBounds.Height)))
	                    ele.remove();
	            }
	        }
	        else {
	            // to store data label position for furture processing
				this.storeDataLabelPositions (currentPoint);
	        }
	    }
	},
	
	storeDataLabelPositions: function (currentPoint){
	  var position = currentPoint.position;
	  switch(position){
				case "top":
				    top[this.processCount -1] = currentPoint;
					this.processCount = 1;
					break;
				case "rightTop":
				    rightTop[this.processCount -1] = currentPoint;
					this.processCount = 1;
					break;
				case "right":
				    right[this.processCount -1] = currentPoint;
					this.processCount = 1;
					break;
				case "rightBottom":
				    rightBottom[this.processCount -1] = currentPoint;
					this.processCount = 1;
					break
				case "bottom":
				    bottom[this.processCount -1] = currentPoint;
					this.processCount = 1;
					break;
				case "left":
				    left[this.processCount -1] = currentPoint;
					this.processCount = 1;
					break;
				case "leftBottom":
				    leftBottom[this.processCount -1] = currentPoint;
					this.processCount = 1;
					break;
				case "leftTop":
				    leftTop[this.processCount -1] = currentPoint;
					break;
				}
	},
	
	// method to avoid collision between data labels  - column series type
	cartesianColumnSmartLabels: function (currentSeries, points, seriesIndex) {
	    var length = points.length;
	    var count = 0;
	    connectorFlag = false;
	    this.processCount = 1;
		var i, currentPoint, ele;
	    // loop to compare the points
	    for (i = 0; i < length; i++) {
	        if(!ej.util.isNullOrUndefined(points[i].yPos))
	            this.model.allPoints[this.model.allPoints.length] = points[i];
	        else
	            continue;
	        if (currentSeries._enableSmartLabels) {
	            connectorFlag = false;
	            this.compareColumnDataLabels(currentSeries, i, points, count, this.processCount);
	        }
	        currentPoint = currentSeries.points[i];
	        if (points[i].dataLabeltemplate) {
	            ele = $("#" + points[i].id);
	            ele.css("left", points[i].xPos).css("top", points[i].yPos);
	        }
	    }
	},

	compareColumnDataLabels: function (currentSeries, i, points, count, processCount) {
	    var length = this.model.allPoints.length - 1;
		var prevLabel, currentLabel, padding;
		var type = currentSeries.type.toLowerCase();
	    for (var j = 0; j < length; j++) {
	        prevLabel = this.model.allPoints[j];
	        currentLabel = this.model.allPoints[this.model.allPoints.length - 1];
	        this.currentCollideLabel = prevLabel;
	        padding = 5;
	        collide = this.isCollide(prevLabel, currentLabel);
	        textPosition = currentSeries.marker.dataLabel.textPosition;
	        if (collide.state) {  
			    // for vertical orientation series types
	            if (type == "column" || type == "stackingcolumn" || type == "rangecolumn") {
	                if (textPosition == "top" || textPosition == "middle")
	                    currentLabel.textOptions.y = currentLabel.yPos = currentLabel.yPos + collide.height + padding;
	                else
	                    currentLabel.textOptions.y = currentLabel.yPos = currentLabel.yPos - collide.height - padding;
	                currentLabel.textOptions.y = currentLabel.textOptions.y + padding + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	            } else {
				    // for horizontal orientation series types
	                if (textPosition == "top" || textPosition == "middle")
	                    currentLabel.textOptions.x = currentLabel.xPos = prevLabel.xPos - prevLabel.width/2 -currentLabel.width/2- padding;
	                else
	                    currentLabel.textOptions.x= currentLabel.xPos = prevLabel.xPos + prevLabel.width / 2 + currentLabel.width / 2 + padding;
	            }
	            this.compareColumnDataLabels(currentSeries, i, points, count, this.processCount);  // to call recursive
	        }
	    }
	},

    // method to comapare data labels and place without collision
	compareDataLabels: function (i, points, count, processCount) {
	    var length = this.model.allPoints.length;
		var j, padding, prevLabel, currentLabel;
	    for (j = 0; j < length; j++) {
	        prevLabel = this.model.allPoints[j];
	        currentLabel = this.model.allPoints[length - 1];
	        this.currentCollideLabel = prevLabel;
	        padding = 10;
	        collide = this.isCollide(prevLabel, currentLabel, j);
	        if (collide.state || collide.marker) {
	            connectorFlag = true;
	            switch (count) {
	                case 0:                        // top
	                    if (this.processCount > 1) this.resetValues(currentLabel);
	                    if (!collide.marker) {
	                        if (!ej.util.isNullOrUndefined(this.prevLabel)) {
	                            checkCollide = this.isCollide(this.prevLabel, currentLabel);
	                            if (!checkCollide.state) { this.prevLabel = prevLabel; this.processCount = 1; };
	                        } else {
	                            this.prevLabel = prevLabel;
	                            this.processCount = 1;
	                        }
	                    }
	                    else {
	                        this.prevLabel = collide.markerRegion;
	                        this.processCount = 1;
	                    }
	                                            // right
	                    this.resetValues(currentLabel);
	                    if (processCount > 1 && right[this.processCount - 2]) {
	                         prevLabel = right[this.processCount - 2];
	                         currentLabel.textOptions.x = currentLabel.xPos = prevLabel.xPos +  (prevLabel.width / 2 + currentLabel.width / 2 + padding);
	                    }
	                    else 
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos + (this.prevLabel.width / 2 + currentLabel.width / 2 + padding);
	                    count += 1;
	                    currentLabel.position = "right";
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;

	                case 1:                       
                              // right bottom
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos + this.prevLabel.width / 2 + currentLabel.width / 2 + padding;
	                    if (processCount > 1 && rightBottom[this.processCount - 2]) {
	                      prevLabel = rightBottom[this.processCount - 2];
	                      currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos + this.canvasY + prevLabel.height + padding;
	                    }
	                    else 
	                        currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "rightBottom";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
					

	                case 2:                       

					// bottom
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    if (processCount > 1 && bottom[this.processCount - 2]) {
	                        prevLabel = bottom[this.processCount - 2];
	                       currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos + currentLabel.height + padding;
	                    }
	                    else 
	                        currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "bottom";
	                    count += 1;
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 3:                         
 // left bottom
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos - this.prevLabel.width / 2 - currentLabel.width / 2 - padding;
	                    if (processCount > 1 && leftBottom[this.processCount - 2]) {
	                        prevLabel = leftBottom[this.processCount - 2];
	                        currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos + this.canvasY + prevLabel.height + padding;
	                    }
	                       else 
	                        currentLabel.textOptions.y = currentLabel.yPos = currentLabel.location.Y + this.canvasY + currentLabel.height + padding / 2;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "leftBottom";
	                    count += 1;
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 4:                         
// left
	                   this.resetValues(currentLabel);
	                    if (processCount > 1 && left[this.processCount - 2]) {
	                        prevLabel = left[this.processCount - 2];
	                        currentLabel.textOptions.x = currentLabel.xPos = (prevLabel.xPos - prevLabel.width / 2 - currentLabel.width / 2 - padding);
	                    }
	                    else
	                        currentLabel.textOptions.x = currentLabel.xPos = (this.prevLabel.xPos - this.prevLabel.width / 2 - currentLabel.width / 2 - padding);
	                    currentLabel.position = "left";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 5:                        
 // left top
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos - this.prevLabel.width / 2 - currentLabel.width / 2 - padding;
	                    if (processCount > 1 && leftTop[this.processCount - 2]) {
	                        prevLabel = leftTop[this.processCount - 2];
	                        currentLabel.textOptions.y = currentLabel.yPos = (prevLabel.yPos) - (currentLabel.height + padding);
	                    }
	                    else 
	                    currentLabel.textOptions.y = currentLabel.yPos = this.prevLabel.yPos - currentLabel.height - padding;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "leftTop";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 6:                         
//top

                        this.resetValues(currentLabel);
                        if (processCount > 1 && top[processCount - 2]) {
	                        prevLabel = top[processCount -2];
	                        currentLabel.textOptions.y = currentLabel.yPos = (prevLabel.yPos) - (currentLabel.height + padding);
	                    }
	                    else 
	                    currentLabel.textOptions.y = currentLabel.yPos = (this.prevLabel.yPos) - (this.processCount * (currentLabel.height + padding));
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    count += 1;
	                    currentLabel.position = "top";
	                    this.compareDataLabels(i, points, count, this.processCount);
						break;
	                case 7:                        
// right top
	                    this.resetValues(currentLabel);
	                    currentLabel.textOptions.x = currentLabel.xPos = this.prevLabel.xPos + this.prevLabel.width / 2 + currentLabel.width / 2 + padding;
	                    if (processCount > 1 && rightTop[this.processCount - 2]) {
	                        prevLabel = rightTop[this.processCount - 2];
	                        currentLabel.textOptions.y = currentLabel.yPos = prevLabel.yPos - currentLabel.height - padding;     
	                    }
	                    else 
	                        currentLabel.textOptions.y = currentLabel.yPos = this.prevLabel.yPos - currentLabel.height - padding;
	                    currentLabel.textOptions.y = currentLabel.textOptions.y + padding/2 + currentLabel.margin.top / 2 - currentLabel.margin.bottom / 2;
	                    currentLabel.position = "rightTop";
	                    count += 1; 
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	                case 8:
	                    count = 0; 
	                    this.processCount += 1;
	                    this.compareDataLabels(i, points, count, this.processCount);
	                    break;
	            }
	        }
	    }
	},

     // method to draw connector lines for data labels in cartesian axis
	drawConnectorLines: function (seriesIndex, index, currentPoint) {
	    var bottom = false;
		var drawConnectorLine = true;
		var xPos = currentPoint.xPos;
		var yPos = currentPoint.yPos;
		var areaBounds = this.model.m_AreaBounds;
	    var location = currentPoint.location ? currentPoint.location : currentPoint.symbolLocation;
	    if (location.Y > currentPoint.textOptions.y)
	        var endY = (currentPoint.textOptions.y);                         // top
	    else {
            bottom = true
	        endY = (currentPoint.textOptions.y - currentPoint.height / 2);  // bottom
	    }
	    if (currentPoint.dataLabeltemplate) {                               // for template
	        endY = currentPoint.yPos - this.model.m_AreaBounds.Y + currentPoint.height ;
	        currentPoint.textOptions.x = currentPoint.textOptions.x - this.model.m_AreaBounds.X + currentPoint.width/4 + this.canvasX;
	    } else if (xPos > areaBounds.Width || (xPos + currentPoint.width) < 0 ||
             		yPos - currentPoint.height > areaBounds.Height || yPos+ currentPoint.height < 0)
		    drawConnectorLine = false;
		if (drawConnectorLine){
	      var connectorDirection = "M" + " " + (location.X + this.canvasX) + " " + (location.Y + this.canvasY) + " " + "L" + " " + (currentPoint.textOptions.x) + " " + (endY);
	      var connectorLineValues = this.model.series[seriesIndex].marker.dataLabel.connectorLine;
	      var stroke = !ej.util.isNullOrUndefined(connectorLineValues.color) ? connectorLineValues.color : this.model.series[seriesIndex].fill;
	      var connectorOptions = {
	        'id': this.svgObject.id +  "_dataLabelConnectorLine_series" + seriesIndex + "_" + index,
	        'stroke': stroke,
	        'stroke-width': connectorLineValues.width,
	        'd': connectorDirection
	      };
	      this.model.series[seriesIndex].connectorLines = true;
	      this.svgRenderer.drawPath(connectorOptions, this.gConnectorEle[seriesIndex]);
		}
	},

    // to reset the data label values back 
	resetValues: function (currentPoint) {
	    currentPoint.textOptions.y = currentPoint.dataLabel.textY;
	    currentPoint.yPos = currentPoint.dataLabel.y;
	    currentPoint.textOptions.x = currentPoint.dataLabel.textX;
	    currentPoint.xPos = currentPoint.dataLabel.x
	},

	// to detect collision
	isCollide: function (a, b, j) {
	    var width = 0;
	    var height = 0;
		var state = false, markerState = false;
		var borderWidth = a.seriesIndex ? this.model.series[a.seriesIndex].marker.dataLabel.border.width : 0;
		borderWidth += b.seriesIndex ? this.model.series[b.seriesIndex].marker.dataLabel.border.width : 0;
		var series = this.model.series;
		var firstSeries = series[a.seriesIndex];
		var secondSeries = series[b.seriesIndex];
		var aXPos = a.xPos , aYPos= a.yPos, bXPos = b.xPos, bYPos = b.yPos;
		a.xPos += firstSeries.xAxis.x, a.yPos += firstSeries.yAxis.y,
        b.xPos += secondSeries.xAxis.x, b.yPos += secondSeries.yAxis.y;		
		
	    if (a != b) {
	        state = !(                    // to compare data labels
            ((a.yPos + a.height) < (b.yPos)) ||
            (a.yPos > (b.yPos + b.height)) ||
            ((a.xPos + a.width / 2 + borderWidth) < b.xPos - b.width / 2) ||
            (a.xPos - a.width/2 > (b.xPos + b.width/2)));
	        if (state) {
	            height = (a.yPos + a.height) - b.yPos;
	            width = (a.xPos + a.width) - b.xPos;
	        }
			a.xPos = aXPos, a.yPos = aYPos;
	    }
	    if (!state) {
	        for (var f = 0; f < this.model.markerRegion.length; f++) {
	            a = this.model.markerRegion[f];
				firstSeries = series[a.seriesIndex];
				aXPos = a.xPos , aYPos = a.yPos;
				a.xPos+= firstSeries.xAxis.x, b.yPos+= firstSeries.yAxis.y;
	            markerState= !(                // to compare data label and marker
                ((a.yPos + a.height) < (b.yPos - b.height/2)) ||
                (a.yPos > (b.yPos + b.height)) ||
                ((a.xPos + a.width) < b.xPos) ||
                (a.xPos > (b.xPos + b.width)));
	            if (markerState)
	                break;
	        }
	    }
		
		a.xPos = aXPos, a.yPos = aYPos;
		b.xPos = bXPos, b.yPos = bYPos;
		// to hide the partially visible data labels
		
	    return { state: state, marker: markerState, markerRegion: a, width: width, height: height };
	},

	drawAccDisplayText: function (series, options) {
	    // method to draw text 
	    var chartObj = this;
		var pointsLength = options.points.length;
	    chartObj.accDataLabelRegion = [];
		chartObj.firstStartAngle = [];
	    
	        if ((options.type.toLowerCase() != "pyramid" && options.type.toLowerCase() != "funnel") && (options.leftsidePoints.length > 0 || options.rightsidePoints.length > 0)) {
	            chartObj.model.bounds["left"] = [];
	            chartObj.model.bounds["right"] = [];
				var rightsidePointsLength = options.rightsidePoints.length;
	            var leftsidePointsLength = options.leftsidePoints.length;

	            for (var i = 0; i < rightsidePointsLength; i++) {
	                options.rightsidePoints[i].index = i;
	                series.drawDataLabelAcc(chartObj, options, options.rightsidePoints[i].visiblePointIndex, options.rightsidePoints[i])
	            }
	            for (var j = leftsidePointsLength - 1; j >= 0; j--) {
	                options.leftsidePoints[j].index = j;
	                series.drawDataLabelAcc(chartObj, options, options.leftsidePoints[j].visiblePointIndex, options.leftsidePoints[j])
	            }
	        }
	        else {
	             for (var i = 0; i < pointsLength; i++) {
	                options.points[i].index = i;
					if (isNaN(options.points[i].startAngle)&& options.type.toLowerCase()=="pie") continue;
	                series.drawDataLabelAcc(chartObj, options, i, options.points[i])
	             }
	        }
		    this.svgRenderer.append(this.gSeriesConnectorEle,  this.gSeriesEle);
			
	        this.svgRenderer.append(this.gSymbolGroupEle,  this.gSeriesEle);
			
			this.svgRenderer.append(this.gDataLabelEle,  this.gSeriesEle);
			
	        this.svgRenderer.append(this.gSeriesTextEle,  this.gSeriesEle);
	    

	    this.svgRenderer.append(this.gSeriesEle,  this.svgObject);

	    if (!this.vmlRendering) 
	    this.svgRenderer.append(this.svgObject, this.element);
	},

    _processOData: function (series) {
        var chart = this;
        var queryPromise = series.dataSource.executeQuery(series.query);
        queryPromise.done(function (e) {
            chart._processJsonData(e.result, series);
            series.visibility = 'visible';
            $(chart.svgObject).empty();
            $(chart.legendSvgContainer).empty();
            if (chart.model.enableCanvasRendering) {
                chart.svgRenderer.ctx.clearRect(0, 0, chart.svgObject.width, chart.svgObject.height);
                $("#canvas_trackSymbol").remove();
            }

            var redrawChart=true;
            for (var i = 0; i < chart.model.series.length; i++)
                if (chart.model.series[i].dataSource)
                {
                    if (chart.model.series[i].visibility == 'hidden') {
                        redrawChart = false;
                        break;
                    }
                }
            if (redrawChart) {
                chart.draw();
            }
        });
        queryPromise.fail(function (e) {
            series.visibility = 'visible';
            $(chart.svgObject).empty();
            $(chart.legendSvgContainer).empty();
            var redrawChart=true;
            for (var i = 0; i < chart.model.series.length; i++)
                if (chart.model.series[i].dataSource)
                {
                    if (chart.model.series[i].visibility == 'hidden') {
                        redrawChart = false;
                        break;
                    }
                }
            if (redrawChart) {
                chart.draw();
            }
        });
    },
    _processJsonData: function (jsonObj, series) {
	    if(series.points.length > 0)
           series.points.length = 0;
        var xdata = [];
        var dataSource = series.dataSource;
        var xName = series.xName;
        var yNames = series.yName;
        var pointColorMappingName = series.pointColorMappingName;
        var textMappingName = series.marker.dataLabel.textMappingName;
        var pointColorMappingNameValue = [];
        var textMappingNameValue = [];

        for (var j = 0; j < jsonObj.length; j++) {
            if (typeof jsonObj[j][xName] == "string" && jsonObj[j][xName].indexOf("/Date(") != -1) {
                jsonObj[j][xName] = new Date(parseInt(jsonObj[j][xName].substr(6)));
                xdata.push((jsonObj[j][xName]) ? jsonObj[j][xName] : j);
            }
            else
                xdata.push(!ej.util.isNullOrUndefined(jsonObj[j][xName]) ? jsonObj[j][xName] : j);
            pointColorMappingNameValue.push(!ej.util.isNullOrUndefined(jsonObj[j][pointColorMappingName]) ? jsonObj[j][pointColorMappingName] : null);
            textMappingNameValue.push(!ej.util.isNullOrUndefined(jsonObj[j][textMappingName]) ? jsonObj[j][textMappingName] : null);
        }
        var yValues = [];
        var index = 0;
            if (series._hiloTypes) {
                var yVal = [];
                //For high
                var high = series.high;
                for (var s = 0; s < jsonObj.length; s++) {
                    yVal.push(jsonObj[s][high] === null || jsonObj[s][high] === undefined ? jsonObj[s][high] : parseFloat(jsonObj[s][high]));

                }
                yValues[index] = yVal;
                index = index + 1;
                //For low
                yVal = [];
                var low = series.low;
                for (var l = 0; l < jsonObj.length; l++) {
                    yVal.push(jsonObj[l][low] === null || jsonObj[l][low] === undefined ? jsonObj[l][low] : parseFloat(jsonObj[l][low]));
                }
                yValues[index] = yVal;
                index = index + 1;
                if (series.type.toLowerCase() !== 'hilo') {
                    //For open
                    yVal = [];
                    var open = series.open;
                    for (var h = 0; h < jsonObj.length; h++) {
                        yVal.push(jsonObj[h][open] === null || jsonObj[h][open] === undefined ? jsonObj[h][open] : parseFloat(jsonObj[h][open]));
                    }
                    yValues[index] = yVal;
                    index = index + 1;

                    //For close
                    yVal = [];
                    var close = series.close;
                    for (var mI = 0; mI < jsonObj.length; mI++) {
                        yVal.push((jsonObj[mI][close] === null || jsonObj[mI][close] === undefined) ? jsonObj[mI][close] : parseFloat(jsonObj[mI][close]));
                    }
                    yValues[index] = yVal;
 index = index + 1;
                    //For volume
                    if (series.volume) {
                        yVal = [];
                        var volume = series.volume;
                        for (var vI = 0; vI < jsonObj.length; vI++) {
                            yVal.push((jsonObj[vI][volume] === null || jsonObj[vI][volume] === undefined) ? jsonObj[vI][volume] : parseFloat(jsonObj[vI][volume]));
                        }
                        yValues[index] = yVal;
                    }
                }
            }
        if (yNames) {
           
                var yNVal = [];
                for (var yI = 0; yI < jsonObj.length; yI++) {
                    yNVal.push((jsonObj[yI][yNames]) ? parseFloat(jsonObj[yI][yNames]) : jsonObj[yI][yNames]);
                }
                yValues[index] = yNVal;
                index = index + 1;
            

            if (series.type.toLowerCase() == 'bubble') {
                yVal = [];
                var size = series.size;
                for (var b = 0; b < jsonObj.length; b++) {
                    yVal.push(jsonObj[b][size] === null || jsonObj[b][size] === undefined ? jsonObj[b][size] : parseFloat(jsonObj[b][size]));
                }
                yValues[index] = yVal;
            }
        }


        for (var m = 0; m < xdata.length; m++) {
            series.points[m] = { x: xdata[m], xValue: xdata[m], y: yValues[0][m], YValues: [], visible: true, fill: pointColorMappingNameValue[m], text: textMappingNameValue[m] };
            if (series.type.toLowerCase() == 'bubble') {
                series.points[m].YValues.push(yValues[0][m]);
                series.points[m].size = series.points[m].YValues[1] = yValues[1][m];
            }
            else if (series._hiloTypes && yValues.length > 1) {
                series.points[m].high = series.points[m].YValues[0] = yValues[0][m];
                series.points[m].low = series.points[m].YValues[1] = yValues[1][m];
                if (series.type.toLowerCase() != 'hilo' && series.type.toLowerCase() != 'rangecolumn'&& yValues.length > 3) {
                    series.points[m].open = series.points[m].YValues[2] = yValues[2][m];
                    series.points[m].close = series.points[m].YValues[3] = yValues[3][m];
                if (series.volume)
                    series.points[m].volume = series.points[m].YValues[4] = yValues[4][m];
                }
            }
            else {
                series.points[m].YValues.push(yValues[0][m]);
            }
        }
    },
    bindTo: function (excludeDataUpdate) {
         // remove the entire selection style
          this._removeSelection();
		  $(document).find('[id*="_TrackToolTip"]').remove();// fixed for mobile issue
          $('#template_group_' + this._id).remove();
		  this.model.primaryXAxis._valueType = null;
		  this.model.primaryXAxis.position = null;
		  this.model.primaryYAxis._valueType = null;
		  this.model.primaryYAxis.position = null;
		  this.model.requireInvertedAxes = false;
		  var seriesLength = this.model.series.length;
         var axesLength = this.model.axes.length;
         for (var i = 0; i < axesLength; i++)
		  {
                this.model.axes[i]._valueType = null;
                this.model.axes[i].position = null;
         }
		  if (this.model._axes)
         {
		       axesLength = this.model._axes.length;
              for (var j = 0; j < axesLength; j++)
              {
                  this.model._axes[j]._valueType = null;
                  this.model._axes[j].position = null;
				  
              }
         }
		  for (var i = 0; i < seriesLength; i++) {
		      var series = this.model.series[i];
		      series.position = null;
		      if (series.fill && this.model.seriesColors && series.fill != this.model.seriesColors[i])
		          this.model.series[i].isFill = true;      
		  }
		  if (this.model.AreaType && this.model.AreaType == "none") {
		      var series = this.model.series[0];
			  var point;
		      for (var i = 0; series.points && i < series.points.length; i++) {
		          point = series.points[i]
		          if (point.fill && this.model.pointColors && point.fill != this.model.pointColors[i])
		              point.isFill = true;
		          if (!point.isFill)  
		              point.fill = null;		           
		      }
		  }
          if(this.model.enable3D)
            ej.Ej3DRender.transform = null;
          if (this.model.series) {
            var series = this.model.series;
            this.setModelProperties(excludeDataUpdate);
            if (!excludeDataUpdate) {
                for (var i = 0; i < series.length; i++) {
                    if (series[i].visibility == null || series[i].visibility == undefined) {
                        series[i].visibility = 'visible';
                    }
					 if (series[i].type) series[i]._hiloTypes = ej.seriesTypes[series[i].type.toLowerCase()].prototype.hiloTypes;
                    if (!(series[i].points))
                        series[i].points = [];
                    if (series[i].dataSource) {
                        if (series[i].dataSource instanceof ej.DataManager) {
                            series[i].visibility = 'hidden';
						    if (ej.isNullOrUndefined(series[i].query) || !(series[i].query instanceof ej.Query))
                                series[i].query = ej.Query();
                            this._processOData(series[i]);
                        } else if (series[i].dataSource.length > 0) {

                            this._processJsonData(series[i].dataSource, series[i]);

                        }
                    }
                }
            }
            if($(this.svgObject).width()>0)
            this.draw(excludeDataUpdate);
        }



    },

	calculateHeight : function(chart){              // height calculation
	     var $svgObj = $(chart.svgObject);
	     var chartHeight = chart.model.size.height;
         var containerHeight= $(chart.element).height();
         var height = (containerHeight / 100) * parseInt(chartHeight);
		 $svgObj.height(height);
		 return height;
	},
	
	calculateWidth : function(chart){               // width calculation
	       var $svgObj = $(chart.svgObject);
	       var chartWidth = chart.model.size.width;
           var containerWidth = $(chart.element).width();
           var width = (containerWidth / 100) * parseInt(chartWidth);
		   $svgObj.width(width);
		   return width;
	},

     // store the element class logic perform here
	selectedStyle: function (chart) {
	    var selected= $('#' + chart._id).find('[class*="Selection"]');
	    var selection = [];
	    for (var i = 0; i < selected.length; i++) {
	        selection[i] = [];
	        selection[i].id = selected[i].id;
	        selection[i].className = $('#' + selection[i].id).attr('class');
	    }
	    if ($('[id $= Def ]').length > 0) {
	        selection.pattern = $('[id $= Def ]');
	    }

	    return selection;
	},
	
	chartResize: function () {
        var chart = this;
        var $svgObj = $(chart.svgObject);
        var selection = this.selectedStyle(chart);
        if (this.model.enableCanvasRendering) {
            $('[id*=' + this._id + '_Selection_' + ']').remove();
            $("#canvas_trackSymbol").remove();
            var ctx = chart.svgObject.getContext("2d");
            ctx.clearRect(0, 0, this.svgRenderer.svgObj.width, this.svgRenderer.svgObj.height);
			
			if (typeof chart.model.size.width=="string" && chart.model.size.width.indexOf("%") != -1) {           // to set width 
                var width = chart.calculateWidth(chart);
				this.svgRenderer.svgObj.width = width;
            }
			else
              this.svgRenderer.svgObj.width = $("#" + this._id).width();
            
			if (typeof chart.model.size.height=="string" && chart.model.size.height.indexOf("%") != -1) {          // to set height
                 var height = chart.calculateHeight(chart);
				  this.svgRenderer.svgObj.height = height;
            }
            else
			 this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : parseInt($(chart.element).height());
			

        }
        if (this.resizeTO) clearTimeout(this.resizeTO);
        this.resizeTO = setTimeout(function () {
          if (!ej.util.isNullOrUndefined(chart.model)) {
            if (typeof chart.model.size.width == "string" && chart.model.size.width.indexOf("%") != -1) {
               chart.calculateWidth(chart);
            }
            else
                $svgObj.width($(chart.element).width());
      
            if (typeof chart.model.size.height == "string" && chart.model.size.height.indexOf("%") != -1) {
                 chart.calculateHeight(chart);
            }
            else
                $svgObj.height($(chart.element).height());
            if ($(chart.svgObject).width() > 0) {
                $(chart.svgObject).empty();
	        $(chart.legendSvgContainer).empty();
                $('#template_group_' + chart._id).remove();
				$("#annotation_group_" + chart._id).remove();       //to remove annotation from DOM on resize
                chart.disableAnimation();
                for (var j = 0; j < chart.model.series.length; j++)
                    chart.model.series[j].regionAdded = false;
                chart.draw();
                for (var i = 0; i < selection.length; i++) {
                    $('#' + selection[i].id).attr('class', selection[i].className);
                }
                if(selection.pattern)
                    chart.svgRenderer.append(selection.pattern, chart.svgObject);
                if (chart.zoomed)
                    chart._enableZoomingButtons();
                chart.enableAnimation();
            }
		  }
        }, 500);

    },
    isDevice: function () {
        return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        // comment above line temporary. Due to the below code event wont bind for tablet device
        //  return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
    },
    bindResizeEvents: function () {
        if (this.model._resizeEventRegistered)
            return 0;
        if (ej.isTouchDevice() && this._isOrientationSupported())
            this._on($(window), "orientationchange", this.chartResize);
        else
            this._on($(window), "resize", this.chartResize);
        this.model._resizeEventRegistered = true;
    },
    removeResizeEvents: function () {
        if (this.model._resizeEventRegistered) {
            if (ej.isTouchDevice() && this._isOrientationSupported())
                this._off($(window), "orientationchange", this.chartResize);
            else
				this._off($(window), "resize", this.chartResize);
            this.model._resizeEventRegistered = false;
        }
    },
	 _isOrientationSupported: function () {
            return ("orientation" in window && "onorientationchange" in window);
    },
    bindEvents: function () {
	    var matched = jQuery.uaMatch(navigator.userAgent);
		
	    this._on($(this.svgObject), "touchstart", this.chartMouseDown);
        this._on(this.element, "touchmove", this.chartMouseMove);
        this._on($(document), "touchend", this.chartMouseUp);
		this._on(this.element, 'touchleave', this.chartMouseLeave);
		
		//this._on($(this.svgObject), "mousedown", this.chartMouseDown);
		this._on($(this.element), "mousedown", this.chartMouseDown);
        this._on(this.element, 'mouseleave', this.chartMouseLeave);
        this._on($(document), "mouseup", this.chartMouseUp);
        var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
        if (ej.isTouchDevice() && !isIE11 && !this.vmlRendering) {
            if (window.navigator.msPointerEnabled) {
                this._on(this.element, 'MSPointerMove', this.chartMouseMove);
                this._on(this.element, 'MSPointerDown', this.chartMouseDown);
			    this._on(this.element, 'MSPointerOut', this.chartMouseLeave);
                this._on($(document), "MSPointerUp", this.chartMouseUp);
			    this._on($(this.svgObject), 'click', this.chartMouseClick);
                this._on($(this.legendSvgContainer), 'click', this.chartMouseClick);
                if (this.model.zooming.enable || this.model.crosshair.visible)
				$(this.svgObject).css('-ms-touch-action', 'none');
            }
            else {
                this._on(this.element, 'touchmove', this.chartTouchMove);
                this._on($(this.svgObject), 'touchstart', this.chartTouchClick);
                    this._on($(this.legendSvgContainer), 'touchstart', this.chartTouchClick);
            }
        }
        else {
            this._on(this.element, 'mousemove', this.chartMouseMove);
            this._on($(this.element), "click", this.chartMouseClick);
           this._on($(this.legendSvgContainer), 'click', this.chartMouseClick);
        }
        if (this.model.canResize) {
            this.bindResizeEvents();
        }
        
        var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
        var eventName = matched.browser.toLowerCase() == "mozilla" ? (isIE11 ? "mousewheel": "DOMMouseScroll") : "mousewheel";
        this._on(this.element, eventName, this.chartMouseWheel);

    },
    calZoomFactors: function (cumulativeScale, origin, currentZoomFactor, currentZoomPos) {
        var calcZoomFactorVal, calcZoomPosition;
        if (cumulativeScale == 1) {
            calcZoomFactorVal = 1;
            calcZoomPosition = 0;
        } else {
            calcZoomFactorVal = ej.EjSvgRender.utils._minMax(1 / cumulativeScale, 0, 1);
            calcZoomPosition = currentZoomPos + ((currentZoomFactor - calcZoomFactorVal) * origin);
        }
        return { zoomMFactor: calcZoomFactorVal, zoomMPosition: calcZoomPosition };
    },
    doMouseWheelZoom: function (cumulativeScale, origin, axis) {
        if (cumulativeScale >= 1) {
            var calZoomVal = this.calZoomFactors(cumulativeScale, origin, axis.zoomFactor, axis.zoomPosition);
            //calcZoomPos = Math.Round(calcZoomPos, 2);
            if (axis.zoomPosition != calZoomVal.zoomMPosition) {
                this.zoomed = true;
                axis.zoomPosition = calZoomVal.zoomMPosition;
                axis.zoomFactor = (calZoomVal.zoomMPosition + calZoomVal.zoomMFactor) > 1 ? (1 - calZoomVal.zoomMPosition) : calZoomVal.zoomMFactor;
                return true;
            }
        }

        return false;

    },
    enableAnimation: function () {
        for (var m = 0; m < this.model.series.length; m++) {
            var series = this.model.series[m];
            series.enableAnimation = this.serAnimation[m];
        }
    },
    disableAnimation: function () {
        var chart = this;
        for (var m = 0; m < chart.model.series.length; m++) 
            chart.model.series[m].enableAnimation = false;
        for (var k = 0; k < chart.model.indicators.length; k++) 
            chart.model.indicators[k].enableAnimation = false;
    },
    _removeTrackBall: function () {
        var id = "#" + this.svgObject.id;
        $("#" + this.svgObject.id + "_trackSymbol").remove();
		 $(document).find("#measureTex").remove();
		 if (this.chartCross.visible) {
		     // Visibility is set to hidden
		     var element = $(this.svgObject).find(id + "_AxisCrossToolTip");
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
		     element = $(this.svgObject).find(id + "_CrosshairVertical");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0' });
		     element = $(this.svgObject).find(id + "_CrosshairHorizontal");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0' });
		     
        element = $(this.svgObject).find(id + "_TrackAxisToolTip");
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden'});
        element = $(this.svgObject).find(id + "_Tracker");
        this.svgRenderer._setAttr($(element), { "d": 'M 0 0'});
        element = $(this.svgObject).find('[id*="_TrackToolTip"]');
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        element = $(this.svgObject).find('[id*="_TrackToolTip"]').children();
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        $(element).css('visibility', 'hidden');
        element = $(this.svgObject).find('[id*="_trackSymbol_"]');
        this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
        $(document).find('[id*="_TrackToolTipTemplate_"]').attr("visibility", "hidden");
        $(document).find('[id*="_TrackToolTipTemplate_"]').css("display", "none");
        $(id + "_CrosshairVertical").css("visibility", "hidden");
        $(id + "_CrosshairHorizontal").css("visibility", "hidden");
        $("#" + this._id).find('[id*="canvas_AxisToolTipRect"]').css("visibility", "hidden");
        $("#" + this._id).find('[id*="canvas_ToolTipRect"]').css("visibility", "hidden");
		$("#" + this._id).find('[id*="canvas_Tracker"]').remove();
        $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
		}
    },
    enableTrackBall: function () {
        this.model.crosshair.visible = this.chartCross.visible;
        this.model.crosshair.type = this.chartCross.mArea;
    },
    disableTrackBall: function () {

        this.model.crosshair.visible = false;
        
        this._removeTrackBall();
    },
    _enableZoomingButtons: function () {
	    if (this.model.AreaType != 'cartesianaxes') {
                $("#" + this._id + "_canvas_ResetZoom").remove();
                $("#" + this._id + "_canvas_PanBtn").remove();
                $("#" + this._id + "_canvas_ZoomBtn").remove();
        }
        if (this.model.AreaType == 'cartesianaxes' && ($("#" + this._id + '_ResetZoom').length == 0 || this.model.enableCanvasRendering)) {
            //Remove zoom buttons before adding them
	        this._removeZoomkit();
            this.resetZoomButton();
            this.panButton();
            this.zoomButton();
        }
    },
    chartMouseWheel: function (e) {
        $("#" + this.svgObject.id + "_TrackToolTip").hide(); // fixed tooltip mouse wheel issue
		if (this.model.zooming.enableMouseWheel && this.model.AreaType == "cartesianaxes" && !this.model.enable3D && !this.vmlRendering) {
            var chart = this;       
            var chartZoomed;
            this.zoomed = false;
            var canUpdate = false;
            var id = "#" + chart.svgObject.id;
            var matched = jQuery.uaMatch(navigator.userAgent);
			var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            var wheelDelta = e.originalEvent.wheelDelta;
			var direction = matched.browser.toLowerCase() == "mozilla" ? ((isIE11 ? ((wheelDelta / 120) > 0 ? 1 : -1) : -(e.originalEvent.detail) / 3 > 0 ? 1 : -1)) : ((wheelDelta / 120) > 0 ? 1 : -1);
            for (var i = 0; i < this.model._axes.length; i++) {
                var axis = this.model._axes[i];
                if ((axis.orientation.toLowerCase() == "vertical" && (chart.model.zooming.type.toLowerCase() == "y" || chart.model.zooming.type.trim().toLowerCase() == "x,y")) || (axis.orientation.toLowerCase() == "horizontal" && (chart.model.zooming.type.toLowerCase() == "x" || chart.model.zooming.type.trim().toLowerCase() == "x,y"))) {

                    var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                    var cumulativeScale = Math.max(currentScale + (0.25 * direction), 1);
                    canUpdate = canUpdate | chart.doMouseWheelZoom(cumulativeScale, 0.5, axis);
                }
                if (axis.zoomFactor != 1 && axis.zoomPosition != 0)
                    chartZoomed = true;
            }

            this.disableAnimation();

            this.disableTrackBall();
            if (canUpdate) {
                this.model.legendCollapsed = false;
                this.zoomed = true;
                chart.redraw(true);
              }
            
            this._enableZoomingButtons();
            this.mouseWheelCoords = { x: e.originalEvent.pageX, y: e.originalEvent.pageY };
            if (!chartZoomed) {
                $(chart.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').remove();
				$("#" + chart._id + '_ResetZoom').remove();
                $("#" + chart._id + '_PanBtn').remove();
                $("#" + chart._id + '_ZoomBtn').remove();
                this.enableAnimation();
                this.panning = false;
            }
			var matched = jQuery.uaMatch(navigator.userAgent);
            if (e.preventDefault && matched.browser == 'chrome' && this.model.zooming.enable)
                e.preventDefault();
            e.returnValue = false;
        }

        $(".tooltipDiv").remove();
    },
    getLegendData: function (evt) {
        var mouseMoveCords = this.calMousePosition(evt);
        var scrolltop = $("#legend_" + this._id).scrollTop();
        var scrollleft = $("#legend_" + this._id).scrollLeft();
        this.scrolltop = scrolltop;
        this.scrollleft = scrollleft;
        this.mousemoveX = mouseMoveCords.X+scrollleft;
        this.mousemoveY = mouseMoveCords.Y+scrolltop;

        {
            var currentX = this.mousemoveX;
            var currentY = this.mousemoveY;
            var chartId = this._id;
            var region;
            if ((!ej.util.isNullOrUndefined(evt.target.id) && this.svgRenderer._getAttrVal($(evt.target).parents(':eq(1)'), 'id') == this.svgObject.id + '_Legend') ||(this.model.enableCanvasRendering) || (this.vmlRendering)) {
                $.each(this.model.legendRegion, function (index, regionItem) {

                    if ((currentX >= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X) && (currentX <= regionItem.Bounds.LegendBound.X + regionItem.Bounds.ItemBound.X + regionItem.Bounds.ItemBound.Width)) {
                        if ((currentY >= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y - (regionItem.Bounds.ItemBound.Height / 4)) && (currentY <= regionItem.Bounds.LegendBound.Y + regionItem.Bounds.ItemBound.Y + regionItem.Bounds.ItemBound.Height)) {
                           region = regionItem;
                           $("#legend_" +chartId + "_canvas").css("cursor", "pointer");
                        }else
                            $("#legend_" + chartId + "_canvas").css("cursor", "default");
                    } 
                });
                if (!ej.util.isNullOrUndefined(region)) {
                    return { legendItem: region, series: this.model._visibleSeries[region.LegendItem.SeriesIndex] };
                }
            }

        }
        return false;
    },

    get3DSeriesPoint: function (evt) {
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var x = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.X;
        var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;
        var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
        var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

        if (currentX > x && currentX < (x + width) && currentY > y && currentY < (y + height)) {
            var nodeName = $(evt.target)[0].nodeName;
            if ((nodeName == "path" || nodeName == "shape") && $(evt.target)[0].id.indexOf("Region")>1) {
                index = $(evt.target)[0].id.match(/(\d+)/g);
                pointIndex = parseInt(index[index.length - 1]);
                seriesIndex = parseInt(index[index.length - 2]);
                var regionValue = {};
                regionValue.Region = {};
                regionValue.Region.PointIndex = pointIndex;
                regionValue.SeriesIndex = seriesIndex;
                var pointData = {};
                pointData.pointIndex = pointIndex;
                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                return region;
            }
        }
    },

    GetSeriesPoint: function (evt) {
        var mouseX;
        var mouseY;
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var region;
        var chartObj = this;
        if (chartObj.model.enable3D) {
            return this.get3DSeriesPoint(evt);           
        }
        var x = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.X;
        var y = (this.model.AreaType == 'polaraxes') ? 0 : this.model.m_AreaBounds.Y;
        var width = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).width() : this.model.m_AreaBounds.Width;
        var height = (this.model.AreaType == 'polaraxes') ? $(this.svgObject).height() : this.model.m_AreaBounds.Height;

        if ((this.model.AreaType == 'cartesianaxes' || this.model.AreaType == 'polaraxes')) {
          
            if (currentX > x && currentX < (x + width) && currentY > y && currentY < (y + height)) {
                $.each(this.model.chartRegions, function (index, regionItem) {
                    if (!ej.util.isNullOrUndefined(regionItem.SeriesIndex)) {
                    var currentSer = chartObj.model._visibleSeries[regionItem.SeriesIndex];
                    if (currentSer.drawType.toLowerCase() == 'column' && evt.target) {
                        if (chartObj.model.enableCanvasRendering) {
                                    if (currentSer.type.toLowerCase() == "polar") {
                                        var bounds = regionItem.Region.Bounds;
                                        var chartStartAngle = -.5 * Math.PI;
                                        var innerRadius = bounds.DRadius;
                                        var fromCenterX;
                                        var fromCenterY;
                                        fromCenterX = (currentX) - (bounds.CenterX);
                                        fromCenterY = (currentY) - (bounds.CenterY);
                                        var series = chartObj.model._visibleSeries[regionItem.SeriesIndex];
                                        var startAngle = series.startAngle;
                                        var endAngle = series.endAngle;

                                        var arcAngle = (startAngle) ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;
                                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle) % (2 * Math.PI);
                                        if (clickAngle < 0 && bounds.PointIndex != 0) clickAngle = 2 * Math.PI + clickAngle;

                                        var pointStartAngle = parseFloat(bounds.StartAngle.toFixed(14));
                                        var pointEndAngle = parseFloat(bounds.EndAngle.toFixed(14));
                                        pointStartAngle -= arcAngle;
                                        pointEndAngle -= arcAngle;

                                        if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                                            index = bounds.PointIndex;
                                            distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                                            if (distanceFromCenter <= bounds.Radius && distanceFromCenter > innerRadius) {
                                                var pointRegion = { Region: { PointIndex: index }, SeriesIndex: regionItem.SeriesIndex, type: regionItem.type };
                                                region = pointRegion;
                                            }
                                        }
                                    } else if (currentSer.type.toLowerCase() == "radar") {
                                        var bounds = regionItem.Region.Bounds;
                                        if (bounds.Line1) {
                                            line1 = bounds.Line1;
                                            line2 = bounds.Line2;
                                            line3 = bounds.Line3;
                                            var polygon = [
                                                            { x: line1.x, y: line1.y },
                                                            { x: line2.x, y: line2.y },
                                                            { x: line3.x, y: line3.y },
                                            ];
                                            var currentPoint = { x: currentX, y: currentY };
                                            var point = false;
                                            for (var p = false, i = -1, l = polygon.length, j = l - 1; ++i < l; j = i)
                                                ((polygon[i].y <= currentPoint.y && currentPoint.y < polygon[j].y) || (polygon[j].y <= currentPoint.y && currentPoint.y < polygon[i].y))
                                                && (currentPoint.x < (polygon[j].x - polygon[i].x) * (currentPoint.y - polygon[i].y) / (polygon[j].y - polygon[i].y) + polygon[i].x)
                                                && (p = !p);
                                            point = p;
                                            if (point) {
                                                index = bounds.PointIndex;
                                                var pointRegion = { Region: { PointIndex: index }, SeriesIndex: regionItem.SeriesIndex, type: regionItem.type };
                                                region = pointRegion;
                                            }
                                        }
                                    }
                                }
                                else {
                                    var nodeName = $(evt.target)[0].nodeName;
                                    if ((nodeName == "path" || nodeName == "shape") && $($(evt.target).parent())[0].id == chartObj.svgObject.id + '_SeriesGroup' + '_' + regionItem.SeriesIndex) {
                                        index = $(evt.target)[0].id.match(/(\d+)/g);
                                        pointIndex = parseInt(index[index.length - 1]);
                                        regionItem = { Region: { PointIndex: pointIndex }, SeriesIndex: regionItem.SeriesIndex, type: regionItem.type };
                                        region = regionItem;
                                        return false;
                                    }
                                }
                    }
                    else {
                        if ((((currentX >= regionItem.Region.Bounds.X) && (currentX <= (regionItem.Region.Bounds.X + (regionItem.Region.Bounds.Width))))) && ((currentY > regionItem.Region.Bounds.Y) && (currentY < (regionItem.Region.Bounds.Y) + (regionItem.Region.Bounds.Height)))) {
                            region = regionItem;
                        }
                    }
                    }
					else {
                        regionValue = chartObj.model.chartRegions;
                        var pointData = {};
						 var areaValue;
					if (!regionItem.isStripLine) {
                        var regionLength = regionItem.length;
						 var requireInvertedAxes = chartObj.model.requireInvertedAxes;
                        var axisName = (!requireInvertedAxes) ? chartObj.model.series[regionItem[regionLength - 1].SeriesIndex]._yAxisName : chartObj.model.series[regionItem[regionLength - 1].SeriesIndex]._xAxisName;
                        for (var k = 0; k < chartObj.model._axes.length; k++)
                        {
                            if (axisName == chartObj.model._axes[k].name)
                            {
                               areaValue = chartObj.model._axes[k];
                               break;
                            }
                        }
                        for (var i = 0; i < regionValue.length; i++) {                          
                            for (var j = 0; j < regionValue[i].length; j++) {
                                var polygon = [];
                                line1 = regionValue[i][j].region;
                                if (line1[0]) polygon.push({ x: line1[0].X + areaValue.x, y: line1[0].Y + areaValue.y });
                                if (line1[1]) polygon.push({ x: line1[1].X + areaValue.x, y: line1[1].Y + areaValue.y });
                                if (line1[2]) polygon.push({ x: line1[2].X + areaValue.x, y: line1[2].Y + areaValue.y });
                                if (line1[3]) polygon.push({ x: line1[3].X + areaValue.x, y: line1[3].Y + areaValue.y });

                                var currentPoint = { x: currentX, y: currentY };
                                var point = chartObj.isPointInPolygon(polygon, currentPoint);
                                if (point)
                                    region = regionValue[i][j];
                            }                                               
                        }
                    }
                    }
                });
            }
            if (region) {
                return { region: region, location: { x: currentX, y: currentY} };
            }
        }

        else {
            var chartObj = this;
            var distanceFromCenter;
            var regionValue;
            var chartRegions = chartObj.model.chartRegions;
            if (chartRegions.length > 0 && (chartRegions[0].Series.type.toLowerCase() == "pyramid"|| chartRegions[0].Series.type.toLowerCase() == "funnel")) {
                var nodeName = $(evt.target)[0].nodeName;
                if ((nodeName == "path" || nodeName == "shape") && $($(evt.target).parent())[0].id == chartObj.svgObject.id + '_SeriesGroup' + '_' + chartRegions[0].SeriesIndex) {
                    regionValue = chartObj.model.chartRegions[0];
                    index = $(evt.target)[0].id.match(/(\d+)/g);
                    index = parseInt(index[index.length - 1]);
                    regionValue.Region.PointIndex = index;
                    var pointData = {};
                    pointData.pointIndex = index;
                    region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                    return region;
                }
                else {                    
                    regionValue = chartRegions[0];
                    var pyrX = this.pyrX;
                    var pyrY = this.pyrY;
                    var pointData = {};
                    var polygon;
                        for (var j = 0; j < regionValue.Region.length; j++) {
                            line1 = regionValue.Region[j].Line1;
                            line2 = regionValue.Region[j].Line2;
                            line3 = regionValue.Region[j].Line3;
                            line4 = regionValue.Region[j].Line4;
                            if (chartRegions[0].Series.type.toLowerCase() == "pyramid")
                                polygon = [
                                            { x: line1.x + pyrX, y: line1.y + pyrY },
                                            { x: line2.x + pyrX, y: line2.y + pyrY },
                                            { x: line3.x + pyrX, y: line3.y + pyrY },
                                            { x: line4.x + pyrX, y: line4.y + pyrY }
                                ];
                            else {
                                line5 = regionValue.Region[j].Line5;
                                line6 = regionValue.Region[j].Line6;
                                polygon = [
                                            { x: line1.x + pyrX, y: line1.y + pyrY },
                                            { x: line2.x + pyrX, y: line2.y + pyrY },
                                            { x: line3.x + pyrX, y: line3.y + pyrY },
                                            { x: line4.x + pyrX, y: line4.y + pyrY },
                                            { x: line5.x + pyrX, y: line5.y + pyrY },
                                            { x: line6.x + pyrX, y: line6.y + pyrY }
                                ];
                            }

                            var currentPoint = { x: currentX, y: currentY };
                            var point = this.isPointInPolygon(polygon, currentPoint);
                            if (point) {
                                pointData.pointIndex = regionValue.Region[j].PointIndex;
                                regionValue.Region.PointIndex = pointData.pointIndex;
                                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY } };                                                                
                                return region;
                            }

                        }
                    }                
                }
            
            else {
                // ReSharper disable DuplicatingLocalDeclaration
                $.each(chartObj.model.chartRegions, function (regionIndex, regionValue) {
                    // ReSharper restore DuplicatingLocalDeclaration
                    var chartStartAngle = -.5 * Math.PI;
                    var seriesData = regionValue.SeriesData; var regionData = regionValue.Region;
                    var innerRadius = (chartObj.model._visibleSeries[regionValue.SeriesIndex].type.toLowerCase() == 'doughnut') ? seriesData.DRadius : 0;
                    var fromCenterX;
                    var fromCenterY;
                    fromCenterX = (currentX) - (seriesData.CenterX);
                    fromCenterY = (currentY) - (seriesData.CenterY);
                    var series = chartObj.model._visibleSeries[regionValue.SeriesIndex];
                    var startAngle = series.startAngle;
                    var endAngle = series.endAngle;
                    startAngle = startAngle < 0 ? startAngle + 360 : startAngle;
                    endAngle = endAngle < 0 ? endAngle + 360 : endAngle;
                    var totalDegree = (series.endAngle - series.startAngle);
                    //Anticlockwise pie and doughnut
                    if (totalDegree < 0) {
                        endAngle = endAngle / 360;
                        var arcAngle = (endAngle) ? 2 * Math.PI * (endAngle < 0 ? 1 + endAngle : endAngle) : 0;
                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle) % (2 * Math.PI);;
                        if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;

                        pointData = [];
                        for (var i = 0; i < regionData.length; i++) {
                            var pointStartAngle = parseFloat(regionData[i].StartAngle.toFixed(14));
                            var pointEndAngle = parseFloat(regionData[i].EndAngle.toFixed(14));
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            pointStartAngle -= arcAngle;
                            pointEndAngle -= arcAngle;
                            pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                            pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                            if (series.startAngle > 0 && series.endAngle < 0) {
                                pointStartAngle = pointStartAngle <= 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                                pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            }
                            if (clickAngle <= pointStartAngle && clickAngle >= pointEndAngle) {
                                pointData.push(regionData[i]);
                                break;
                            }
                        }
                    }  //Clockwise pie and doughnut
                    else {
                        var arcAngle = (startAngle) ? 2 * Math.PI * (chartObj.model.itemCurrentXPos < 0 ? 1 + chartObj.model.itemCurrentXPos : chartObj.model.itemCurrentXPos) : 0;
                        var clickAngle = (Math.atan2(fromCenterY, fromCenterX) - chartStartAngle - arcAngle)%(2*Math.PI);
                        if (clickAngle < 0) clickAngle = 2 * Math.PI + clickAngle;
                        pointData = [];
                        for (var i = 0; i < regionData.length; i++) {
                            var pointStartAngle = parseFloat(regionData[i].StartAngle.toFixed(14));
                            var pointEndAngle = parseFloat(regionData[i].EndAngle.toFixed(14));
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle < 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            pointStartAngle -= arcAngle;
                            pointEndAngle -= arcAngle;
                            pointStartAngle = parseFloat(pointStartAngle.toFixed(14));
                            pointEndAngle = parseFloat(pointEndAngle.toFixed(14));
                            if (series.startAngle < 0 && (series.endAngle > 0||series.endAngle==null)) {
                            pointStartAngle = pointStartAngle < 0 ? 2 * Math.PI + pointStartAngle : pointStartAngle;
                            pointEndAngle = pointEndAngle <= 0 ? 2 * Math.PI + pointEndAngle : pointEndAngle;
                            }
                            if (clickAngle >= pointStartAngle && clickAngle <= pointEndAngle) {
                                pointData.push(regionData[i]);
                                break;
                            }
                        }
                    }
                    if (pointData.length > 0) {
                        var isExploded = (series.explodeAll || (series.explodeIndex == pointData[0].PointIndex));
                        if (isExploded) {

                            var fromCenterX1 = currentX - pointData[0].StartX;
                            var fromCenterY1 = currentY - pointData[0].StartY;

                            if (pointData.length > 0) {

                                distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX1), 2) + Math.pow(Math.abs(fromCenterY1), 2));
                                if (distanceFromCenter <= seriesData.Radius && distanceFromCenter > innerRadius) {
                                    regionData.PointIndex = pointData[0].PointIndex;
                                    region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                                }
                            }
                        }
                        else {
                            distanceFromCenter = Math.sqrt(Math.pow(Math.abs(fromCenterX), 2) + Math.pow(Math.abs(fromCenterY), 2));
                            if (distanceFromCenter <= seriesData.Radius && distanceFromCenter > innerRadius) {
                                regionData.PointIndex = pointData[0].PointIndex;
                                region = { region: regionValue, pointData: pointData, location: { x: currentX, y: currentY} };
                            }
                        }
                    }

                });

                return region;
            }
        }
    },
    isPointInPolygon: function (polygon, point) {
        for (var p = false, i = -1, l = polygon.length, j = l - 1; ++i < l; j = i)
            ((polygon[i].y <= point.y && point.y < polygon[j].y) || (polygon[j].y <= point.y && point.y < polygon[i].y))
            && (point.x < (polygon[j].x - polygon[i].x) * (point.y - polygon[i].y) / (polygon[j].y - polygon[i].y) + polygon[i].x)
            && (p = !p);
        return p;
    },
    mousePosition: function (evt) {
         if (!ej.util.isNullOrUndefined(evt.pageX) && evt.pageX>0)
            return { x: evt.pageX, y: evt.pageY };
        else if (!ej.util.isNullOrUndefined(evt.originalEvent.pageX) && evt.originalEvent.pageX>0)
            return { x: evt.originalEvent.pageX, y: evt.originalEvent.pageY };
         else if (evt.originalEvent.changedTouches != undefined)
         {
             if (!ej.util.isNullOrUndefined(evt.originalEvent.changedTouches[0].pageX) && evt.originalEvent.changedTouches[0].pageX > 0)
                 return { x: evt.originalEvent.changedTouches[0].pageX, y: evt.originalEvent.changedTouches[0].pageY };
         }
         else
             return { x: 0, y: 0 };
         
    },


    calMousePosition: function (e) {
        var matched = jQuery.uaMatch(navigator.userAgent);
        var mouseposition = this.mousePosition(e);
        e.pageX = mouseposition.x;
        e.pageY = mouseposition.y;
        var mouseX, mouseY;
        var browser = matched.browser.toLowerCase();
        mouseX = (e.pageX) - $(this.svgObject).offset().left;
        mouseY = (e.pageY) - $(this.svgObject).offset().top;
       

        return { X: mouseX, Y: mouseY };

    },

    cancelEvent: function (e) {
        e.stopPropagation();
        var matched = jQuery.uaMatch(navigator.userAgent);
        if(matched.browser == 'chrome' && (this.model.zooming.enable||this.model.crosshair.visible) ){
            e.preventDefault();
        }
        if (this.model.enable3D) {
            e.preventDefault();
            e.stopPropagation();
        }
        e.returnValue = false;
        e.cancelBubble = true;
        return false;
    },
    resetZoom: function () {
        var chartobj = this;
        var id = "#" + this.svgObject.id;
		$(".tooltipDiv").remove(); //To remove tooltip template
        if (chartobj.zoomed) {
            $.each(this.model._axes, function (index, axis) {
                axis.zoomFactor = 1;
                axis.zoomPosition = 0;
            });
			//Reset primary axis zoomFactor and zoomPosition
			chartobj.model.primaryXAxis.zoomFactor = 1;
            chartobj.model.primaryYAxis.zoomFactor = 1;
            chartobj.model.primaryXAxis.zoomPosition = 0;
            chartobj.model.primaryYAxis.zoomPosition = 0;
			$("#"+chartobj._id + "_canvas").css({ "cursor": "default" });
			$("#" + chartobj._id + "_ResetZoom").remove();
			$("#" + chartobj._id + "_PanBtn").remove();
			$("#" + chartobj._id + "_ZoomBtn").remove();
			    
            $(id + "_ResetZoom").remove();
            $(id + "_PanBtn").remove();
            $(id + "_ZoomBtn").remove();
            chartobj.zoomed = false;
            chartobj.zooming = false;
            chartobj.panning = false;
            chartobj.resetZooming = true;
            chartobj.enableTrackBall();
            chartobj.redraw();
            chartobj.enableAnimation();          
            chartobj.svgRenderer._setAttr($(chartobj.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection"), { "cursor": "default" });
        }
    },
    startZoom: function () {
        var chartObj = this;
        var id = "#" + this.svgObject.id;
		$(".tooltipDiv").remove(); //To remove tooltip template
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'true' });
        chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#14B9FF" });
        chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": '#FFFFFF' });
        chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#B7B7B7' });
        chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": iconColor });
        $(id + "_ZoomBtn").css({ "border-color": "#14B9FF", "background-color": "#14B9FF" });
        $(id + "_PanBtn").css({ "border-color": "#B7B7B7", "background-color": "#B7B7B7" });
		$("#"+chartObj._id + "_canvas").css({ "cursor": "default" });
        this._cursorToDefault();
        chartObj.panning = false;
        chartObj.enableTrackBall();
    },
    startPan: function () {
        var chartObj = this;
        var id = "#" + this.svgObject.id;
		$(".tooltipDiv").remove(); //To remove tooltip template
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        chartObj.svgRenderer._setAttr($(id + '_ZoomBtn'), { 'selected': 'false' });
        chartObj.svgRenderer._setAttr($(id + '_PanBtn'), { 'selected': 'true' });
        chartObj.svgRenderer._setAttr($(id + '_ZoomPath'), { "fill": "#B7B7B7" });
        chartObj.svgRenderer._setAttr($(id + '_ZoomIcon'), { "fill": iconColor });
        chartObj.svgRenderer._setAttr($(id + '_PanPath'), { "fill": '#14B9FF' });
        chartObj.svgRenderer._setAttr($(id + '_panIcon'), { "fill": '#FFFFFF' });
        $(id + "_ZoomBtn").css({ "border-color": "#B7B7B7", "background-color": "#B7B7B7" });
        $(id + "_PanBtn").css({ "border-color": "#14B9FF", "background-color": "#14B9FF" });
		$("#"+chartObj._id + "_canvas").css({ "cursor": "pointer" });
        this._cursorToPointer();
        chartObj.panning = true;
        chartObj.disableTrackBall();

    },
    zoomButton: function () {
        var padding = 20;
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (3 * 32) - padding); //32 path bouding box for pan,reset, zoom button 
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ZoomBtn', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'true' });

        var pathOptions = {
            'id': this.svgObject.id + '_ZoomPath',
            'd': "M32,27c0,2.75-2.25,5-5,5H5c-2.75,0-5-2.25-5-5V5c0-2.75,2.25-5,5-5h22c2.75,0,5,2.25,5,5V27z",
            'fill': this.panning ? '#b7b7b7' : '#14B9FF',
            'zoomId': this._id + '_ZoomBtn',
            'width': 27,
            'height': 27,
            'left': transX + 20,
            'top': transY
        };
        if (this.model.enableCanvasRendering) {
            this.svgRenderer.zoomButton(pathOptions);
            this._on($("#" + this.svgObject.id + '_ZoomBtn'), "touchstart click", this.startZoom);
        } else {
            this.svgRenderer.drawPath(pathOptions, gEle);

            var options = {
                'id': this._id + '_ZoomIcon',
                'd': "M26.101,22.893l-6.605-6.174c1.414-2.781,0.897-6.267-1.496-8.504c-2.901-2.711-7.448-2.56-10.161,0.341    c-2.712,2.9-2.56,7.45,0.341,10.163c2.426,2.266,6,2.523,8.694,0.853l6.579,6.151L26.101,22.893z M10.186,16.571    c-1.715-1.604-1.805-4.293-0.203-6.009c1.605-1.715,4.295-1.805,6.009-0.201c1.715,1.603,1.805,4.293,0.202,6.007    C14.59,18.084,11.901,18.175,10.186,16.571",
                'fill': this.panning ? iconColor : '#FFFFFF'
            };
            this.svgRenderer.drawPath(options, gEle);
            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.startZoom);
        }
    },
    panButton: function () {

        var padding = 15;
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (2 * 32) - padding); //32 path bounding box for pan and resetzoom button
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_PanBtn', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'false' });

        var pathOptions = {
            'id': this.svgObject.id + '_PanPath',
            'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
            'fill': !this.panning ? '#b7b7b7' : '#14B9FF',
            'panId': this._id + '_PanBtn',
            'width': 27,
            'height': 27,
            'left': transX + 10,
            'top': transY
        };
        if (this.model.enableCanvasRendering) {
            this.svgRenderer.panButton(pathOptions);
            this._on($("#" + this.svgObject.id + '_PanBtn'), "touchstart click", this.startPan);
        } else {
            this.svgRenderer.drawPath(pathOptions, gEle);

            var options = {
                'id': this._id + '_panIcon',
                'points': "26.105,16 21.053,12.211 21.053,14.737 17.263,14.737 17.263,10.947 19.834,10.947 16.044,5.895 12.255,10.947 14.737,10.947 14.737,14.737 10.947,14.737 10.947,12.211 5.895,16 10.947,19.789 10.947,17.263 14.737,17.263 14.737,21.053 12.255,21.053 16.044,26.105 19.834,21.053 17.263,21.053 17.263,17.263 21.053,17.263 21.053,19.789",
                'fill': !this.panning ? iconColor : '#FFFFFF'
            };
            this.svgRenderer.drawPolygon(options, gEle);
            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.startPan);

        }
    },
    resetZoomButton: function () {

        var padding = 10;
        var transX = this.model.m_AreaBounds.X + (this.model.m_AreaBounds.Width - (32 + padding)); //32 path bouding box
        var transY = (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) + (this.model.elementSpacing - (this.model.m_AreaBounds.Height));
        var gEle = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_ResetZoom', 'transform': 'translate(' + transX + ',' + transY + ')', 'selected': 'false' });
        var iconColor = this.model.theme.toLowerCase().indexOf("dark") >= 0 ? "#000000" : " #FFFFFF";
        var options = {
            'id': this.svgObject.id + '_ResetZoomPath',
            'd': "M 32 27 c 0 2.75 -2.25 5 -5 5 H 5 c -2.75 0 -5 -2.25 -5 -5 V 5 c 0 -2.75 2.25 -5 5 -5 h 22 c 2.75 0 5 2.25 5 5 V 27 Z",
            'fill': '#b7b7b7',
            'resetZoomId': this._id + '_ResetZoom',
            'width': 27,
            'height': 27,
            'left': transX,
            'top': transY
        };
        if (this.model.enableCanvasRendering) {
            this.svgRenderer.resetZoom(options);
            this._on($("#" + this.svgObject.id + '_ResetZoom'), "touchstart click", this.resetZoom);
        } else {
           
            this.svgRenderer.drawPath(options, gEle);

            var ptOptions = {
                'id': this._id + '_ptIcon',
                'points': "11.895,18.398 8.061,22.23 5.796,19.967 5.796,26.283 12.112,26.283 9.848,24.018 13.682,20.186",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(ptOptions, gEle);
            var pt1Options = {
                'id': this._id + '_ptIcon1',
                'points': "19.691,6.072 21.955,8.337 18.121,12.172 19.908,13.959 23.742,10.123 26.007,12.389 26.007,6.072",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt1Options, gEle);

            var pt2Options = {
                'id': this._id + '_ptIcon2',
                'points': "11.895,13.958 13.682,12.172 9.848,8.337 12.112,6.072 5.796,6.072 5.796,12.389 8.061,10.123",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt2Options, gEle);

            var pt3Options = {
                'id': this._id + '_ptIcon3',
                'points': "19.908,18.396 18.121,20.186 21.955,24.018 19.691,26.283 26.007,26.283 26.007,19.967 23.741,22.23",
                'fill': iconColor
            };

            this.svgRenderer.drawPolygon(pt3Options, gEle);

            this.svgRenderer.append(gEle, this.svgObject);

            this._on($(gEle), "touchstart click", this.resetZoom);
        }

    },

    doZoom: function (zoomRect, zoomRectWidth, zoomRectHeight) {
        var chart = this;
		$(".tooltipDiv").remove(); //To remove tooltip template
        if (chart.model.enableCanvasRendering) {
            var zoomArea = $("#" + this.svgObject.id + "_ZoomArea")[0].getClientRects()[0];
            var chartArea = $("#" + this.svgObject.id)[0].getClientRects()[0];
            var zoomX = zoomArea.left - chartArea.left - $(document).scrollLeft(); 
            var zoomY = zoomArea.top - chartArea.top - $(document).scrollTop(); 
        } else {
            var zoomX = parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "x"));
            var zoomY = parseFloat(chart.svgRenderer._getAttrVal($(zoomRect), "y"));
        }
        $("#" + this.svgObject.id + "_ZoomArea").remove();
        this.drag = false;
        this.zoomed = true;
        for (var k = 0; k < this.model._axes.length; k++) {
            var axis = this.model._axes[k];
            var previousZoomFactor = axis.zoomFactor;
            var previousZoomPosition = axis.zoomPosition;
            if (axis.orientation.toLowerCase() == "horizontal") {
                axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "y"
                    ? previousZoomFactor * (zoomRectWidth / (chart.model.m_AreaBounds.Width)) : axis.zoomFactor;
                axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "y"
                    ? previousZoomPosition + Math.abs((zoomX - chart.model.m_AreaBounds.X) / (chart.model.m_AreaBounds.Width)) * previousZoomFactor : axis.zoomPosition;
                if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                    axis.zoomFactor = previousZoomFactor;
                    axis.zoomPosition = previousZoomPosition;

                }
            } else {
                axis.zoomFactor = chart.model.zooming.type.toLowerCase() != "x"
                    ? previousZoomFactor * zoomRectHeight / chart.model.m_AreaBounds.Height : axis.zoomFactor;
                axis.zoomPosition = chart.model.zooming.type.toLowerCase() != "x"
                    ? previousZoomPosition + (1 - Math.abs((zoomRectHeight + (zoomY - chart.model.m_AreaBounds.Y)) / (chart.model.m_AreaBounds.Height))) * previousZoomFactor : axis.zoomPosition;
                if (parseFloat(axis.zoomFactor.toFixed(3)) <= 0.001) {
                    axis.zoomFactor = previousZoomFactor;
                    axis.zoomPosition = previousZoomPosition;

                }
            }
        }

        //Getting zoomed region data when zoomedData event fired                    
        var series = this.model._visibleSeries;
        var seriesLength = series.length;
        var zoomedData = [];
        var selectedData = [];
        var points = [];
        var type;
        var axisValueType;
        var xVisibleRange;
        var yVisibleRange;
        var pointsLength;
        for (var j = 0; j < seriesLength; j++) {
            if (series[j].visibility == "visible") {
                points = series[j].points;
                pointsLength = points.length;
                type = series[j].type.toLowerCase();
                axisValueType = new ej.axisTypes[series[j].xAxis._valueType.toLowerCase()]();
                axisValueType._calculateVisibleRange(series[j].xAxis, chart);
                axisValueType = new ej.axisTypes[series[j].yAxis._valueType.toLowerCase()]();
                axisValueType._calculateVisibleRange(series[j].yAxis, chart);
                xVisibleRange = series[j].xAxis.visibleRange;
                yVisibleRange = series[j].yAxis.visibleRange;
                for (var k = 0; k < pointsLength; k++) {
                    if ((xVisibleRange.min <= points[k].xValue && xVisibleRange.max >= points[k].xValue) && (yVisibleRange.min <= points[k].y && yVisibleRange.max >= points[k].y)) {
                        if (series[j]._hiloTypes) {
                            if (type == "rangearea" || type == "rangecolumn" || type == "hilo")
                                zoomedData.push({ XValue: points[k].x, High: points[k].high, Low: points[k].low });
                            else
                                zoomedData.push({ XValue: points[k].x, High: points[k].high, Low: points[k].low, Open: points[k].open, Close: points[k].close });
                        }
                        else
                            zoomedData.push({ XValue: points[k].x, YValue: points[k].y });
                    }
                }
            }
            if (zoomedData.length != 0)
                selectedData.push({ selectedData: zoomedData });
            zoomedData = [];
        }

        var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
        commonEventArgs.data = {};
        commonEventArgs.data.series = selectedData;
        this._trigger("zoomed", commonEventArgs);

        this.disableAnimation();
        this.model.legendCollapsed = false;
        this.redraw(true);

        this._enableZoomingButtons();
    },
    _cursorToDefault: function () {
        var id = "#" + this.svgObject.id;
        this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "default" });
    },
    _cursorToPointer: function () {
        var id = "#" + this.svgObject.id;
        this.svgRenderer._setAttr($(this.svgObject).find(id + "_XAxis," + id + "_ChartArea," + id + "_YAxis," + id + "_SeriesCollection," + id + "_StriplineBehind," + id + '_StriplineOver'), { "cursor": "pointer" });
    },
    chartMouseUp: function (evt) {
	   // this.cancelEvent(evt);
	   if (!this.currentPageX && this.panning) {
            var mouseDownCords = this.calMousePosition(evt);
            this.currentPageX = mouseDownCords.X;
            this.currentPageY = mouseDownCords.Y;
        }
        var chart = this;
        if (this.model.zooming.enable && this.model.AreaType == "cartesianaxes") {
            var zoomRect = $("#" + this.svgObject.id + "_ZoomArea");
            if (zoomRect[0]) {
                if (this.model.enableCanvasRendering) {
                    var zoomArea = zoomRect[0].getClientRects()[0];
                    var zoomRectWidth = zoomArea.width;
                    var zoomRectHeight = zoomArea.height;
                } else {
                    var $zoomRect = $(zoomRect);
                    var zoomRectWidth = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "width"));
                    var zoomRectHeight = parseFloat(this.svgRenderer._getAttrVal($zoomRect, "height"));
                }
            }
            if (zoomRectWidth > 0 && zoomRectHeight > 0) {
                chart.doZoom(zoomRect, zoomRectWidth, zoomRectHeight);
            }
            else
                this.drag = false;
            if (!chart.panning)
                this.enableTrackBall();
        }
       // Panning is done on mouse up for other devices and when deferredZoom is set to true
        var id = this.svgObject.id;
        var parentId = ej.util.isNullOrUndefined(evt.target.parentNode) ? "" : evt.target.parentNode.id;

        //condition to find the buttons
        if (parentId == id + "_ResetZoom" || parentId == id + '_ZoomBtn' || evt.target.id == id + "_ResetZoom" || evt.target.id == id + '_ZoomBtn')
            this.panning = false;

        //Panning for other devices
        if ((ej.isTouchDevice() || this.model.zooming.enableDeferredZoom) && this.panning && this.doPan && this.model.AreaType == "cartesianaxes") {
            var oDelta;
            oDelta = {
                'x': this.oPreviousCoords.x - this.currentPageX,
                'y': this.oPreviousCoords.y - this.currentPageY
            };

            this.oPreviousCoords = {
                'x': this.currentPageX,
                'y': this.currentPageY
            };
            $.each(this.model._axes, function (index, axis) {
                var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                chart.translate(axis, (oDelta.x), (oDelta.y), currentScale);
            });
            this.model.legendCollapsed = false;
            this.redraw(true);
            this._cursorToPointer();
            this._enableZoomingButtons();

        }
        
        if (this.doPan)
            this.doPan = false;
        this.rotateActivate = false;
    },

    chartMouseDown: function (e) {
	     this.cancelEvent(e);
        if (!this.svgRenderer.vmlNamespace) {
            this.oPreviousCoords = {};
            if (this.svgRenderer._getAttrVal($(e.target).parent(), "id") != this.svgObject.id + "_ResetZoom" && this.svgRenderer._getAttrVal($(e.target).parent(), "id") != this.svgObject.id + "_PanBtn" && this.model.zooming.enable && !(this.panning) && this.model.AreaType == "cartesianaxes") {
                var mouseDownCords = this.calMousePosition(e);
                this.mouseDownX = mouseDownCords.X;
                this.mouseDownY = mouseDownCords.Y;
                if (this.mouseDownX >= this.model.m_AreaBounds.X && this.mouseDownX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mouseDownY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mouseDownY >= this.model.m_AreaBounds.Y) {
                    this.drag = true;
                    this.disableTrackBall();
                }


            }
            else if (this.panning) {
                var mousePanCords = this.calMousePosition(e);
                this.mousePanX = mousePanCords.X;
                this.mousePanY = mousePanCords.Y;
                if (this.mousePanX >= this.model.m_AreaBounds.X && this.mousePanX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mousePanY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mousePanY >= this.model.m_AreaBounds.Y) {
                    this.doPan = true;
                    this.oPreviousCoords = {
                        'x': e.pageX,
                        'y': e.pageY
                    };
                }
            }


        }
        if (this.model.enable3D && this.model.enableRotation)
            this.rotateActivate = true;
    },
    chartMouseLeave: function (evt) {
        this.cancelEvent(evt);
        // Changes the visibility to hidden
        $("#" + this.svgObject.id + "_TrackToolTip").hide();
        if (($(".tooltipDiv").hasClass(this.svgObject.id)))
            $(".tooltipDiv").remove();
        this._removeTrackBall();
        this._removeHighlight();
        
    },
    chartTouchClick: function (evt) {
	    this.cancelEvent(evt);
         evt = evt.originalEvent.touches[0];
        this._doClick(evt);
    },
    _doClick: function (evt) {
        this._removeHighlight();
        if (!this.model._selectedData)
            this.model._selectedData = [];
        var seriesIndex;
        var chartSeries;
        for (var i = 0; i < this.model._visibleSeries.length; i++) {
            chartSeries = this.model._visibleSeries[i];
            var type = chartSeries.type.toLowerCase();
            if (this.model.AreaType == "cartesianaxes" && chartSeries.visibility.toLowerCase() == 'visible' && type !== "scatter" && type !== "bubble" && type !== "column" && type.indexOf("bar")==-1 && type !== "stackingcolumn" && type !== "stackingcolumn100" && !chartSeries._hiloTypes) {
                var serY = [];
                var serX = [];
                var location = null;
                var mouseMoveCords = this.calMousePosition(evt);
                this.mousemoveX = mouseMoveCords.X;
                this.mousemoveY = mouseMoveCords.Y;
                var closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousemoveX, this.mousemoveY);
                if (!ej.util.isNullOrUndefined(closestXyPoint.point)) {
				    seriesIndex = i;
                    var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonPointEventArgs.data = { location: {x: this.mousemoveX, y: this.mousemoveY}, region: { SeriesIndex: i ,Region: { PointIndex: closestXyPoint.index } } };
                    this._trigger("pointRegionClick", commonPointEventArgs);
                }
            }
        }
        var data = this.GetSeriesPoint(evt);
        if (data) {
           seriesIndex = data.region.SeriesIndex;	        
           var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
           commonEventArgs.data = data;
           this._trigger("pointRegionClick", commonEventArgs); 
        }
        if (this.model.AreaType == "none" && this.model.enable3D && $(evt.target)[0].nodeName == "path") {
            index = $(evt.target)[0].id.match(/(\d+)/g);
            index = parseInt(index[index.length - 1]);
            var currentexplodeindex = this.model._visibleSeries[0].explodeIndex;
            if (currentexplodeindex == index)
                this.model._visibleSeries[0].explodeIndex = null;           
            else
                this.model._visibleSeries[0].explodeIndex = index;                                                                 
            if (!this.vmlRendering && !this.model.enableCanvasRendering)
                var selection = this.selectedStyle(this);
            $(this.svgObject).empty();
            
            this.draw();			 
            $('[id*=' + this.svgObject.id + '_LegendItemShape],[id*=_Region_Series_]').each(function () {
                $(this).attr('class', '')
            })
            }
        var legenddata = this.getLegendData(evt);
        if (legenddata) {
            $('#template_group_' + this._id).remove();
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = legenddata;
            this._trigger("legendItemClick", commonEventArgs);
            var seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
            // legend selection logic perform here
            if (!this.model.legend.toggleSeriesVisibility) {                 
                this.model._selectedData[seriesIndex] = { event: evt, legendData: legenddata, data: data, isLegend: true };
                this.legendSelection(this, legenddata, evt, data);
            }
            if (!commonEventArgs.cancel && this.model.legend.toggleSeriesVisibility) {
				$('[id*=' + this._id + '_Selection_' + ']').remove();
                if (this.model.AreaType != "none") {
                    var seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
                    var trendlineIndex = legenddata.legendItem.LegendItem.TrendLineIndex;
                    if (ej.util.isNullOrUndefined(trendlineIndex)) {
                    legenddata.series.visibility = legenddata.series.visibility.toLowerCase() === 'visible' ? 'hidden' : 'visible';
                        this.model.series[seriesIndex].visibility = legenddata.series.visibility;
                    if (this._notifyArrayChange)
                            this._notifyArrayChange("series[" + seriesIndex + "]visibility", legenddata.series.visibility);
                    } else {
                        var trendlineVisible = legenddata.series.trendlines[trendlineIndex].visibility.toLowerCase();
                        trendlineVisible = trendlineVisible === 'visible' ? 'hidden' : 'visible';
                        this.model.series[seriesIndex].trendlines[trendlineIndex].visibility = trendlineVisible;
                        if (this._notifyArrayChange)
                            this._notifyArrayChange("series[" + seriesIndex + "].trendlines[" + trendlineIndex + "].visibility", trendlineVisible);
                }
                }
                else {
                    var point = legenddata.series.points[legenddata.legendItem.LegendItem.PointIndex].visible;
                    legenddata.series.points[legenddata.legendItem.LegendItem.PointIndex].visible = point == true ? false : true;
                }
                if (this.model.enableCanvasRendering) {
                    /** Canvas Chart image is cleared when click the legend item to show/hide the series**/
                    var chartRect = document.getElementById(this._id).getClientRects()[0];
                    this.svgRenderer.ctx.clearRect(0, 0, chartRect.width, chartRect.height);
                    //Canvas series marker element removed
					$("#" + this._id).find('[id*="canvas_symbol"]').remove();
                }
                $(this.svgObject).empty();
            	$(this.legendSvgContainer).empty();
                for (var j = 0; j < this.model.series.length; j++)
                    this.model.series[j].regionAdded = false;
                this.model.legendCollapsed = true;                
                this.disableAnimation();
                this.draw();
                this.enableAnimation();
                if (this.model.AreaType != "none" && this.zoomed)
                    this._enableZoomingButtons();
            }
        }

        // SVG selection logic started here
        if (!ej.isNullOrUndefined(seriesIndex) && !legenddata && this.model._visibleSeries[seriesIndex].selectionSettings.enable) {
            this.model._selectedData[seriesIndex] = { event: evt, legendData: legenddata, commonArgs: commonEventArgs, pointArgs: commonPointEventArgs, index: seriesIndex, data:data };
            this.segmentSelection(evt, legenddata, commonEventArgs, commonPointEventArgs, seriesIndex, data);
        }

    },

     //segment selection logic
    segmentSelection: function (evt, legenddata, commonEventArgs, commonPointEventArgs, seriesIndex, data) {
        var selectedClass = '';
        if (evt.target.parentNode != null) {
            var parentNodeId = evt.target.parentNode.id;
            var targetId = evt.target.id;
            var isTrackball = false;
			if (parentNodeId) {
                var isElement = (parentNodeId.indexOf(this.svgObject.id + "_TextGroup_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_SeriesGroup_") >= 0 || (parentNodeId.indexOf(this.svgObject.id + "_Chart3D") >= 0 && targetId.indexOf("_Region_Series_") >= 0) || parentNodeId.indexOf(this.svgObject.id + "_symbolGroup") >= 0) ? true : false;
				if (parentNodeId.indexOf(this.svgObject.id + "_TrackSymbol_") >= 0) {
					isElement = true;
					isTrackball = true;
				}
			}
        }

        var selectionPoint = commonEventArgs ? commonEventArgs : commonPointEventArgs;
        var chart = this;
        if (isElement && !this.model.enableCanvasRendering) {
            if (this.model.enable3D && selectionPoint) {
                var data = selectionPoint.data;
                var series = this.model.series[data.region.SeriesIndex];
                var clusterPoints = [];
                if (series.selectionSettings.enable) {
                    var className = $('#' + evt.target.id).attr('class');
                    if (className) {
                        var dElement = (className.indexOf('SelectionStyle') >= 0) ? true : false;
                        var dClusterElement = (className.indexOf('Selectionseries') >= 0) ? true : false;
                    }
                    if (dElement || dClusterElement) {
                        if (dClusterElement) {
                            var name, pathId, gElement;
                            var xvalue = chart.model._visibleSeries[data.region.SeriesIndex]._visiblePoints[data.region.Region.PointIndex].x;
                            for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                                for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                                    if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].x)
                                        clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
                                }
                            }
                            for (k = 0; k < clusterPoints.length; k++) {
                                pathId = "_Region_Series_" + clusterPoints[k].seriesIndex + "_Point_" + clusterPoints[k].pointIndex;
                                gElement = $(this.svgObject).find('[id$=' + pathId + ']');
                                for (var j = 0, len = gElement.length; j < len; j++) {
                                    $("[id=" + gElement[j].id + "]").attr('class', '');
                                }
                            }
                            $("[id *= '_LegendItemShape']").attr('class', '');
                        }
                        else {
                            var pathId = "_Region_Series_" + data.region.SeriesIndex + "_Point_";
                            var threedElement = (evt.target.id.indexOf(pathId) >= 0) ? true : false;
                            if (threedElement) {
                                // for selection in 3D toggle
                                var gElement = $(this.svgObject).find('[id*=' + pathId + ']');
                                for (var i = 0; i < gElement.length; i++) {
                                    $("[id=" + gElement[i].id + "]").attr('class', '');
                                }
                            }
                            this.model.AreaType == 'none' ? $('#' + this.svgObject.id + '_LegendItemShape' + data.region.Region.PointIndex).attr('class', '') : $('#' + this.svgObject.id + '_LegendItemShape' + data.region.SeriesIndex).attr('class', '');
                        }
                    } else {
                        if (this.model.AreaType == 'none')
                            $('[id*=' + this.svgObject.id + '_LegendItemShape],[id*=_Region_Series_]').each(function () {
                                $(this).attr('class', '')
                            })
                        this.selection(this, evt, series, isTrackball, null, data);
                    }
                }
            } else {
                if (!selectionPoint) {
                    var matchStr = this.svgObject.id + '_SeriesGroup_';
                    var selectionIndex = evt.target.parentNode.id.substr(matchStr.length);
                    var seriesIndex = selectionIndex;
                }
                if (this.model.series[seriesIndex] && this.model.series[seriesIndex].selectionSettings.enable) {
                    var matchStr = this.svgObject.id + '_SeriesGroup_';
                    var index = seriesIndex;
                    var mode = this.model.series[seriesIndex].selectionSettings.mode;
                    var seriesIndex = legenddata ? legenddata.legendItem.LegendItem.SeriesIndex : index;
                    selectedClass = $('#' + evt.target.id).attr('class') ? $('#' + evt.target.id).attr('class') : '';
                    var series = this.model.series[seriesIndex];
                    var isSelected = false;
                    var selection, matchString, indexVal, text, pointClass;
                    matchString = this.svgObject.id + "_Series";
                    var clusterPoints = [];
                    indexVal = targetId.match(/(\d+)/g);
                    pointIndex = parseInt(indexVal[indexVal.length - 1]);
                    if (targetId.indexOf('SeriesText') >= 0) {
                        var datas = this.GetSeriesPoint(evt);
                        pointIndex = datas.region.Region.PointIndex;
                    }
                    var xvalue = this.model.series[seriesIndex]._visiblePoints[pointIndex].x;
                    for (var i = 0, len = this.model._visibleSeries.length; i < len ; i++) {
                        for (var j = 0; j < this.model._visibleSeries[i]._visiblePoints.length; j++) {
                            if (xvalue == this.model._visibleSeries[i]._visiblePoints[j].x)
                                clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
                        }
                    }
                    for (k = 0; k < clusterPoints.length; k++) {
                        selection = this.model._visibleSeries[k].selectionSettings.mode;
                        if (selection == 'cluster' && selectionPoint) {
                            data = selectionPoint.data;
                            pointIndex = clusterPoints[k].pointIndex;

                            selectedClass = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + pointIndex).attr('class') ? $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + pointIndex).attr('class') : '';
                            $("[id*=" + this.svgObject.id + '_Series' + k + "]").each(function () {
                                pointClass = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + pointIndex).attr('class') ? $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + pointIndex).attr('class') : '';
                                if (pointClass.indexOf('Selection') >= 0)
                                    isSelected = true
                            })
                            if (selectedClass.indexOf('Selection') >= 0 || isSelected) {
                                $("[id *= " + pointIndex + "]").each(function () {
                                    if (this.parentNode.id == chart.svgObject.id + '_SeriesGroup_' + k || this.parentNode.id == chart.svgObject.id + '_symbolGroup_' + k) {
                                        $(this).attr('class', '');
                                    }
                                })
                                $("[id *= '_LegendItemShape']").attr('class', '');
                            }
                        }
                    }
                }
                    // checked whether parent node child has Selection style
                    $("[id*=" + this.svgObject.id + '_Series' + index + "]").each(function () {
                        var pointClass = $(this).attr('class') ? $(this).attr('class') : '';
                        if (pointClass.indexOf('Selection') >= 0)
                            isSelected = true
                    })

                    if (selectedClass.indexOf('Selection') >= 0 || isSelected) {
                        $('#' + evt.target.id).attr('class', '');
                        $("[id*=" + this.svgObject.id + '_Series' + index + "]").each(function () {
                            if (this.parentNode.id == chart.svgObject.id + '_SeriesGroup_' + seriesIndex || this.parentNode.id == chart.svgObject.id + '_symbolGroup_' + seriesIndex) {
                                $(this).attr('class', '');
                                if (mode == "series" || (mode == "point" && !isSelected))
                                    chart.model._selectedData[index]=null;
                            }
                        })
                        index = chart.model.AreaType == 'none' ? targetId.substr((this.svgObject.id + '_Series' + index + '_Point').length) : index;
                        $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', '');
                        selectedClass.indexOf('Selection') < 0 ? this.selection(this, evt, series, isTrackball, null, data) : 0;
                    }
                    else if (series)
                        this.selection(this, evt, series, isTrackball, null, data);
               
            }
        }
        else if (this.model.enableCanvasRendering) {
            var data = commonPointEventArgs ? commonPointEventArgs.data : data;
            if (data && this.model.series[data.region.SeriesIndex].selectionSettings.enable) {
                var series = $.extend({}, this.model.series[data.region.SeriesIndex]);
                series.pointIndex = data.region.Region.PointIndex;
                series.seriesIndex = data.region.SeriesIndex;
                series.data = data;
                var pointSelect = $('#' + this._id + '_Selection_' + series.seriesIndex + '_point_' + series.pointIndex + '_canvas').length == 0 ? true : false;
                var index = this.model.AreaType == 'none' ? series.pointIndex : series.seriesIndex;
                if (series.selectionSettings.mode == 'cluster') {
                    if ($('#' + this._id + '_Selection_Cluster' + '_point_' + series.pointIndex + '_canvas').length != 0) {
                        $('[id*=' + this._id + '_Selection_').remove();
                        pointSelect = false;
                    } else
                        this.canvasSelection(this, evt, series);
                }

                if ($('#' + this._id + '_Selection_series' + index + '_canvas').length == 0 && pointSelect) {
                    if (series.selectionSettings.mode == 'point') {
                        $('[id*=' + this._id + '_Selection_' + series.seriesIndex + '_point_]').remove();
                        if (this.model.AreaType == 'none')
                            $('[id*=' + this._id + '_Selection_Legend]').remove();
                    }
                    this.canvasSelection(this, evt, series);
                }
                else {
                    $('#' + this._id + '_Selection_' + series.seriesIndex + '_point_' + series.pointIndex + '_canvas').remove();
                    $('#' + this._id + '_Selection_Legend' + index + '_canvas').remove();
                    $('#' + this._id + '_Selection_series' + index + '_canvas').remove();
                }
            }
        }
    },


     //legend Selection logic perform here
    legendSelection: function (chart, legenddata, evt, data) {
        var seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
        if (chart.model.enableCanvasRendering) {
            var series = $.extend({}, legenddata.series);
            series.seriesIndex = seriesIndex;
            series.pointIndex = legenddata.legendItem.LegendItem.PointIndex;
            var index = chart.model.AreaType == 'none' ? series.pointIndex : series.seriesIndex;
            if ($('#' + chart._id + '_Selection_Legend' + index + '_canvas').length == 0)
                chart.canvasSelection(chart, evt, series, legenddata);
            else {
                $('#' + chart._id + '_Selection_Legend' + index + '_canvas').remove();
                $('[id*=' + chart._id + '_Selection_series' + index + ']').remove();
                $('[id*=' + chart._id + '_Selection_' + index + '_point_]').remove();
            }
        } else {
            var series = chart.model.series[seriesIndex];
            var index = chart.model.AreaType == 'none' ? legenddata.legendItem.LegendItem.PointIndex : legenddata.legendItem.LegendItem.SeriesIndex;
            var legendItem = chart.svgObject.id + '_LegendItemShape' + index
            var legendClass = $('#' + legendItem).attr('class') ? $('#' + legendItem).attr('class') : '';
            if (legendClass.indexOf('Selection') < 0)
                chart.selection(chart, evt, series, false, legenddata, data);
            else {
                $('[id*=' + chart.svgObject.id + '_Series' + seriesIndex + '],[id*=_Region_Series_' + seriesIndex + '_Point_]').each(function () {
                    $(this).attr('class', '');
                })
                $('#' + chart.svgObject.id + '_LegendItemShape' + index).attr('class', '');
                this.model._selectedData[seriesIndex] = null;
            }
        }
    },

    chartMouseClick: function (evt) {
	
	this.cancelEvent(evt);
        if (window.navigator.msPointerEnabled) {
            evt = evt.originalEvent;
        }
        this._doClick(evt);
    },

    drawTrackerSymbol: function (series, seriesIndex, ptIndex, tracker, point, id) {

        var type = series.type.toLowerCase();
        var chartSeriesObj = new ej.seriesTypes[type]();

        var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
        if (this.model.AreaType != "none") {
            if (this.model.AreaType == 'cartesianaxes' && ((point.X + trans.x) <= (trans.x + trans.width)) && ((point.X + trans.x) >= trans.x || point.X == 0) &&
                 ((point.Y + trans.y) >= (trans.y) || point.Y == 0)) {
                if (this.model.crosshair.visible || type == "bubble" || type == "scatter") {
                    if ($(this.svgObject).find("#" + this.svgObject.id + '_TrackSymbolClipRect' + '_' + seriesIndex).length == 0 && !this.vmlRendering) {
                        var clipRectOptions = {
                            'id': this.svgObject.id + '_TrackSymbolClipRect' + '_' + seriesIndex,
                            'x': 0,
                            'y': 0,
                            'width': trans.width,
                            'height': trans.height,
                            'fill': 'white',
                            'stroke-width': 1,
                            'stroke': 'Gray'
                    };

                    this.svgRenderer.drawClipPath(clipRectOptions, this.svgObject);
                    }
                    $('#' + this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
                    var trackId = "canvas_trackSymbol_" +seriesIndex;
                    $("#" + this._id).find('[id*=' +trackId + ']').remove();
                    var transSymbolOptions = { 'clip-path': 'url(#' + this.svgObject.id + '_TrackSymbolClipRect' + '_' + seriesIndex +')', 'id': this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex, 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                    this.gTrackerEle = this.svgRenderer.createGroup(transSymbolOptions);
                } else
				    this.gTrackerEle = null; // empty the previously created tracker group if no tracker is drawn now
                }
            else {
			    $('#' + this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
                var transSymbolOptions = { 'id': this.svgObject.id + '_TrackSymbol' + '_' + seriesIndex };
                this.gTrackerEle = this.svgRenderer.createGroup(transSymbolOptions);
                if(!this.model.enableCanvasRendering) {
                    if (this.model.AreaType == 'polaraxes')
                        this.gTrackerEle.setAttribute('clip-path', 'url(#' + this.svgObject.id + '_SeriesGroup_' + seriesIndex + '_ClipRect)');
                    else {
                        this.gTrackerEle.setAttribute('clip-path', 'url(#' + this.svgObject.id + '_TrackSymbolClipRect' + '_' + seriesIndex + ')');
                        this.gTrackerEle.setAttribute('transform', 'translate(' + trans.x + ',' + trans.y + ')');
                    }
               }
            }
           
            if (this.model.crosshair.marker.visible && !series.isIndicator&& series.points[ptIndex].visible)
                chartSeriesObj.drawSymbol(seriesIndex, series, ptIndex, (point.X), (point.Y), this, tracker);
            if (this.gTrackerEle && $("#" + this.gTrackerEle.id).length < 1)
                this.svgRenderer.append(this.gTrackerEle, this.svgObject);
        }
            
        
    },
    getClosesPointXY: function (serX, serY, series, x, y) {
        var closestPoint;
        var valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
        var ptIndex;
        var chartPoint;
        var location;
        var pointIndex;
        var closestX;
        var closestY;
        var size = series.marker ? series.marker.size : { height: 6, width: 6 };
        if ((y <= (valAxis.y + valAxis.height) && valAxis.y <= y) && (valAxis.x <= x && x <= (valAxis.x + valAxis.width))) {
            for (var i = 0; i < series.points.length; i++) {
                chartPoint = series.points[i];
                location = chartPoint.location;
                pointIndex = i;
                closestX = null;
                closestY = null;
                if (location) {
                if (x > location.X + valAxis.x - (size.width / 2) && x < location.X + valAxis.x + (size.width / 2)) {
                    closestX = chartPoint.x;
                }
                if (y > location.Y + valAxis.y - (size.height / 2) && y < location.Y + valAxis.y + (size.height / 2)) {
                    closestY = chartPoint.YValues[0];
                }
                if (!ej.util.isNullOrUndefined(closestX) && !ej.util.isNullOrUndefined(closestY)) {
                    closestPoint = chartPoint;
                    ptIndex = i;
                }
                }
            }         
        }
        return { point: closestPoint, index: ptIndex };
    },
    getClosestPointX: function (serX, series, x, y) {
        var closestPoint = [];
        var valAxis = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
        if (this.model.requireInvertedAxes) {
            x = Math.abs(y - (valAxis.y + valAxis.height));
            y = Math.abs(x - valAxis.x);
        }
        else {
            x = Math.abs(x - valAxis.x);
            y = Math.abs(y - (valAxis.y + valAxis.height));
        }
        var mousePoint = ej.EjSvgRender.utils._getValuebyPoint(x, y, series);
        var pointX = (this.model.requireInvertedAxes) ? mousePoint.PointX : mousePoint.PointX;
        var ptIndex = [];
        $.each(series.points, function (pointIndex, chartPoint) {
            serX.push(chartPoint.xValue);
        });
        var closest = this.getClosest(serX, pointX);
        if (!ej.util.isNullOrUndefined(closest)) {
            $.each(series.points, function (pointIndex, cPoint) {
                if (cPoint.xValue == closest) {
                    closestPoint.push(cPoint);
                    ptIndex.push(pointIndex);

                }
            });

        }
        return { point: closestPoint, index: ptIndex };
    },
    getClosest: function (obj, val) {
        var closest = null;

        // Work out min and max
        var min = Math.min.apply(null, obj);
        var max = Math.max.apply(null, obj);

        // Only calculate closest if point is within array
        if (val >= min - 1 && val <= max + 1) {
            var i;
            for (i = 0; i < obj.length; i++) {
                if (closest == null || Math.abs(obj[i] - val) < Math.abs(closest - val)) {
                    closest = obj[i];
                }
            }
        }
        return closest;
    },
    createTooltip: function (region, evt, series) {
        if(!series)
          series = this.model._visibleSeries[region.SeriesIndex];
         var seriesPoint = series.points[region.Region.PointIndex];
        var point = $.extend(true, {}, seriesPoint);
		var tooltipMargin=10;
        if (point.visible) {
		     var format = series.tooltip.format;
		     var numberToFixed = ej.util.isNullOrUndefined(series.yAxis.roundingPlaces) ? 2 : series.yAxis.roundingPlaces;
		     var dataPoints = this.getTooltipFormat(point, series, region.SeriesIndex, region.Region.PointIndex, format, numberToFixed);;
		     point.x = dataPoints.data.x;
		     point.y = dataPoints.data.y;
		     var tooltip = null;
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { currentText: tooltip, seriesIndex: region.SeriesIndex, pointIndex: region.Region.PointIndex, series: series };
            this._trigger("toolTipInitialize", commonEventArgs);
            var tooltipdiv = $('.tooltipDiv');
            if ($('.tooltipDiv').length == 0) {
                this.tooltipFirst = true;
                tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 13000; display: block;'></div>");
                $(document.body).append(tooltipdiv);
                $('.tooltipDiv').addClass(this.svgObject.id);
            } else {
                $(".tooltipDiv").css("display", "block");
                this.tooltipFirst = false;
            }
            if (series.tooltip.template != null) {
                var cloneNode = $("#" + series.tooltip.template).clone();
                $('.tooltipDiv')[0].innerHTML = "";
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                series.count = 1;
                point.count = 1;
                var seriesColor = this.getSeriesColor(point, region.SeriesIndex, series);
				if (seriesColor)
                    $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
				else
                   $(tooltipdiv).css("background-color", this.model.pointColors[region.Region.PointIndex]);
				var data = { series: series, point: point };
				if (!ej.util.isNullOrUndefined(commonEventArgs.data.currentText))
				    $(tooltipdiv).html(commonEventArgs.data.currentText);
                else
                $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
            } else {
                $(tooltipdiv).html(commonEventArgs.data.ToolTip);
            }
			var areaBounds = this.model.m_AreaBounds;
            var xPos = evt.pageX  + tooltipMargin;
            var yPos = evt.pageY  + tooltipMargin;
            var tooltipWidth = $(tooltipdiv).width();
            var tooltipHeight = $(tooltipdiv).height();
            var position = document.getElementById(this.svgObject.id).getClientRects()[0];
            if (xPos === undefined || xPos === null)
                xPos = evt.pageX  + tooltipMargin;
            if (yPos === undefined || yPos === null)
                yPos = evt.pageY  + tooltipMargin;
            $(tooltipdiv).css("left", xPos);
            $(tooltipdiv).css("top", yPos);
            //Checking top position whether tooltip display outside of the area bounds
            var templateRect = $(tooltipdiv)[0].getBoundingClientRect();
            if (templateRect.right > (areaBounds.X + areaBounds.Width + position.left)) {
                var diff = (xPos - tooltipWidth);
                $(tooltipdiv).css('left', diff + $(document).scrollLeft());
            }
            if (templateRect.bottom > areaBounds.Y + areaBounds.Height + position.top) {
                var diff = templateRect.bottom - (areaBounds.Y + areaBounds.Height + position.top)
                $(tooltipdiv).css('top', templateRect.top - diff + $(document).scrollTop());
            }
			var tooltipOptions = series.tooltip;
            if (tooltipOptions.enableAnimation) {
                $(tooltipdiv).css({
                    'transition-property': 'left,top',
                    '-moz-transition-property': 'left,top', /* Firefox 4 */
                    '-webkit-transition-property': 'left,top', /* Safari and Chrome */
                    '-o-transition-property': 'left,top',
                    'transition-duration': tooltipOptions.duration,
                    '-moz-transition-duration': tooltipOptions.duration, /* Firefox 4 */

                    '-webkit-transition-duration': this.tooltipFirst ? '0s' : tooltipOptions.duration, /* Safari and Chrome */
                    '-o-transition-duration': tooltipOptions.duration /* Opera */
                });
            }
            templateRect = $(tooltipdiv)[0].getBoundingClientRect();
            if(templateRect.top < areaBounds.Y + position.top)
                $(tooltipdiv).css('top', areaBounds.Y + position.top + $(document).scrollTop());
        }
    },

    translate: function (axis, translateX, translateY, currentScale) {
        var offset = axis.orientation.toLowerCase() == "horizontal"
                                              ? translateX / this.model.m_AreaBounds.Width / currentScale
                                              : translateY / this.model.m_AreaBounds.Height / currentScale;

        axis.zoomPosition = axis.orientation.toLowerCase() == "horizontal"
            ? ej.EjSvgRender.utils._minMax(axis.zoomPosition + offset, 0, (1 - axis.zoomFactor))
            : ej.EjSvgRender.utils._minMax(axis.zoomPosition - offset, 0, (1 - axis.zoomFactor));
    },
    highlightFill: function (highlight, chart, seriesIndex, pointIndex, index, legendData) {
        var seriesColors = chart.model.seriesColors;
        var pointColors = chart.model.pointColors;
        var color = highlight.color;
		if (legendData && chart.model.AreaType == 'none')
            pointIndex = legendData.legendItem.LegendItem.PointIndex;
        var pointColor = legendData ? seriesColors[seriesIndex] : chart.model.series[seriesIndex].points[pointIndex].fill;
        var fill = (color != "" ? color : chart.model.AreaType == 'none' ? pointColors[pointIndex] : legendData ? seriesColors[seriesIndex] : pointColor ? pointColor : seriesColors[seriesIndex]);
        if (Object.prototype.toString.call(fill) === '[object Array]')
            fill = fill[1].color;
        return fill;
    },
	
	
   
     // Canvas highlight and selection started here
     // Canvas highlight started here
    canvasHighlight: function (chart, evt, series, legendData) {

        var highlight = series.highlightSettings;
        var data = series.data;
        var color = highlight.color;
        var opacity = highlight.opacity;
        var borderColor = highlight.border.color;
        var borderWidth = highlight.border.width;
        var patternName = highlight.pattern.toLowerCase();
        var legendVisible = chart.model.legend.visible;
        var mode = highlight.mode;
        var seriesIndex = series.seriesIndex;
        var pointIndex = series.pointIndex;
        var seriesType = series.type.toLowerCase();
        var containerStyle = document.getElementById(chart._id + '_canvas').getBoundingClientRect();
        var regions = [];
        var points = [];
        var clusterPoints = [];
        var fill = this.highlightFill(highlight, chart, seriesIndex, pointIndex, index, legendData);
        if (chart.model.AreaType != 'none') {
            for (var i = 0, len = chart.model.chartRegions.length; i < len ; i++) {
                    if ((mode == 'series' || legendData) && (seriesIndex == chart.model.chartRegions[i].SeriesIndex && !(this.selectedPoint == chart.model.chartRegions[i].Region.PointIndex))) 
                        regions.push(chart.model.chartRegions[i].Region);
                    }
                   
               
        } else {
            for (var i = 0; i < chart.model.chartRegions[0].Region.length; i++) {
                if (pointIndex == chart.model.chartRegions[0].Region[i].PointIndex)
                    regions.push(chart.model.chartRegions[0].Region[i]);
            }
        }
        if (mode == 'cluster' && chart.model.Areatype != 'none') {
            var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].x;
            for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                    if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].x)
                        clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };

                }
            }
        }
        for (var k = 0; k < clusterPoints.length; k++) {
            clusterseriesIndex = clusterPoints[k].seriesIndex;
            clusterpointIndex = clusterPoints[k].pointIndex;
            for (var l = 0; l < chart.model.chartRegions.length; l++) {
                if (clusterseriesIndex == chart.model.chartRegions[l].SeriesIndex && clusterpointIndex == chart.model.chartRegions[l].Region.PointIndex)
                    regions.push(chart.model.chartRegions[l].Region);
            }
        }
        var index = chart.model.AreaType == 'none' ? pointIndex : seriesIndex;
        if ((mode == 'series' || legendData) && ($('#' + chart._id + '_Selection_series' + index + '_canvas').length == 0) && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0) {
             for (var i = 0; i < series.points.length; i++) {
                    //condition for highlight of marker on selection checked here
                    if (!(this.selectedPoint == series.points[i].xValue))
                        points.push(series.points[i]);
                }
            ctx = this.createCanvasElement(chart._id + '_Highlight_series' + seriesIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
            this.createRect(chart, ctx);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("bar") != -1) {
                for (i = 0; i < regions.length; i++) {
                    var isUnSelected = $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + i + '_canvas').length == 0 ? true : false;
                    this.canvasSeriesRect(regions[i], opacity, borderColor, borderWidth, color, isUnSelected)
                }
            }
            else if (seriesType == 'pie' || seriesType == 'doughnut')
                this.canvasHighlightCircle(chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions[0], seriesType, ctx);
            else if (seriesType == 'pyramid' || seriesType == 'funnel')
                this.canvasHighlightPyramid(chart, regions[0], highlight, ctx, opacity, borderColor, borderWidth, color, seriesType);
            else if (seriesType == 'bubble')
                this.canvasBubbleHighlight(highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor);
            else if (seriesType == 'scatter')
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor);
        }
		else if (mode == 'cluster' && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0 && this.model.AreaType != 'none' && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_series' + seriesIndex + '_canvas').length == 0) {
                for (var i = 0, len = this.model._visibleSeries.length; i < len; i++) {
                    var series = this.model._visibleSeries[i];
                    series.seriesIndex = i;
                    points.push(series.points[pointIndex]);

                }
                ctx = this.createCanvasElement(chart._id + '_Highlight_' + '_Cluster' + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
                this.createRect(chart, ctx);
                for (var i = 0, len = this.model._visibleSeries.length; i < len; i++) {
                    highlight = chart.model._visibleSeries[i].highlightSettings;
                    color = highlight.color;
                    opacity = highlight.opacity;
                    borderColor = highlight.border.color;
                    borderWidth = highlight.border.width;
                    patternName = highlight.pattern.toLowerCase();
                    fill = this.highlightFill(highlight, chart, i, pointIndex, index, legendData);
                    color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);

                    if (seriesType.indexOf("column") != -1 || seriesType.indexOf("bar") != -1) {
                        var isUnSelected = $('#' + chart._id + '_Selection_' + i + '_point_' + pointIndex + '_canvas').length == 0 ? true : false;
                        this.canvasSeriesRect(regions[i], opacity, borderColor, borderWidth, color, isUnSelected);
                    }
                    else if (seriesType == 'bubble') {
                        this.canvasBubbleHighlight(highlight, regions, chart, i, color, borderWidth, opacity, borderColor)
                    }
                    else if (seriesType == 'scatter') {
                        this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor)
                    }

                }
            }
         else if (mode == 'point' && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_series' + seriesIndex + '_canvas').length == 0 && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0) {
            points[0] = series.points[pointIndex];
            ctx = this.createCanvasElement(chart._id + '_Highlight_' + seriesIndex + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            this.createRect(chart, ctx);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("bar") != -1) {
                regions = []; regions[0] = data.region.Region;
                this.canvasSeriesRect(regions[0], opacity, borderColor, borderWidth, color, true)
            }
            else if (seriesType == 'pie' || seriesType == 'doughnut')
                this.canvasHighlightCircle(chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions[0], seriesType, ctx);
            else if (seriesType == 'pyramid' || seriesType == 'funnel') {
                this.canvasHighlightPyramid(chart, regions[0], highlight, ctx, opacity, borderColor, borderWidth, color, seriesType);
            }
            else if (seriesType == 'bubble') {
                regions = []; regions[0] = data.region.Region;
                this.canvasBubbleHighlight(highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor)                
            }
            else if (seriesType == 'scatter') {
                regions = []; regions[0] = data.region.Region;
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor)
            }
        }
        // legend highlight (legend symbol)
        if (legendVisible && $('#' + chart._id + '_Selection_Legend' + index + '_canvas').length == 0) {
            if (mode == 'cluster' && !legendData && this.model.AreaType != 'none') {
                    for (var i = 0, len = this.model._visibleSeries.length; i < len; i++) {
                        var series = this.model._visibleSeries[i];
                        series.seriesIndex = i;
                        this.canvasHighlightLegend(chart, series, highlight, 'Highlight', containerStyle, color)
                    }
                }
            else
                this.canvasHighlightLegend(chart, series, highlight, 'Highlight', containerStyle, color)
            
        }
        if (series.marker.visible == true && this.model.AreaType != 'none')
            this.canvasHighlightMarker(chart, series, points, highlight, ctx, evt)
    },

     // canvas rect operations perform here
    canvasSeriesRect: function (regions, opacity, borderColor, borderWidth, color, isUnSelected) {
        var options = {
            'opacity': opacity,
            'stroke': borderColor,
            'stroke-width': borderWidth,
            'fill': color
        }

      
            options.x = regions.Bounds.X;
            options.y = regions.Bounds.Y;
            options.width = regions.Bounds.Width;
            options.height = regions.Bounds.Height;
            if (isUnSelected)
                this.drawRect(options, ctx);
        
    },

     // // canvas selection logic started here
    canvasSelection: function (chart, evt, series, legendData) {

        var highlight = series.selectionSettings;
        var data = series.data;
        var color = highlight.color;
        var opacity = highlight.opacity;
        var borderColor = highlight.border.color;
        var borderWidth = highlight.border.width;
        var patternName = highlight.pattern.toLowerCase();
        var legendVisible = chart.model.legend.visible;
        var mode = highlight.mode;
        var seriesIndex = series.seriesIndex;
        var pointIndex = series.pointIndex;
        var seriesType = series.type.toLowerCase();
        var containerStyle = document.getElementById(chart._id + '_canvas').getBoundingClientRect();
        var regions = [];
        var points = [];
        var clusterPoints = [];
        var seriesData = { seriesIndex: seriesIndex, series: series };
        var index = chart.model.AreaType == 'none' ? pointIndex : seriesIndex;
        var fill = this.highlightFill(highlight, chart, seriesIndex, pointIndex, index, legendData);
        if (chart.model.AreaType != 'none') {
             for (var i = 0, len = chart.model.chartRegions.length; i < len; i++) {
                    if ((mode == 'series' || legendData) && seriesIndex == chart.model.chartRegions[i].SeriesIndex)
                        regions.push(chart.model.chartRegions[i].Region);
                  

                }
        } else {
            for (var i = 0; i < chart.model.chartRegions[0].Region.length; i++) {
                if (pointIndex == chart.model.chartRegions[0].Region[i].PointIndex)
                    regions.push(chart.model.chartRegions[0].Region[i]);
            }
        }
        if (mode == 'cluster' && chart.model.Areatype != 'none') {
            var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].x;
            for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                    if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].x)
                        clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };

                }
            }
        }
        for (var k = 0; k < clusterPoints.length; k++) {
            clusterseriesIndex = clusterPoints[k].seriesIndex;
            clusterpointIndex = clusterPoints[k].pointIndex;
            for (var l = 0; l < chart.model.chartRegions.length; l++) {
                if (clusterseriesIndex == chart.model.chartRegions[l].SeriesIndex && clusterpointIndex == chart.model.chartRegions[l].Region.PointIndex)
                    regions.push(chart.model.chartRegions[l].Region);
            }


        }
        if (mode == 'series' || legendData) {
            // removed selection here
            if (this.model.AreaType == 'none') $('[id*=' + chart._id + '_Selection_' + ']').remove();
            var length = this.model.AreaType == 'none' ?series.points.length:this.model.series.length; 
            for (var i = 0; i < length; i++) {
                $('#' + chart._id + '_Selection_series' + i + '_canvas').remove();
                if (i != seriesIndex)
                    this.model._selectedData[i] = null;
            }
            for (var i = 0; i < length; i++) {
                if (this.model.AreaType == 'none')
                    $('#' + chart._id + '_Selection_Legend' + i + '_canvas').remove();
                else {
                    if(this.model.series[i].selectionSettings.mode == 'series')
                        $('#' + chart._id + '_Selection_Legend' + i + '_canvas').remove();
                    else if($('[id*=' + chart._id + '_Selection_' + i + '_point_' + ']').length == 0)
                            $('#' + chart._id + '_Selection_Legend' + i + '_canvas').remove();
				    else if (this.model.series[i].selectionSettings.mode == 'cluster')
                            $('#' + chart._id + '_Selection_Legend' + i + '_canvas').remove();		
                }
            }

            points = series.points;
            ctx = this.createCanvasElement(chart._id + '_Selection_series' + index + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
            this.createRect(chart, ctx);
            this._trigger("seriesRegionClick", seriesData);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("bar") != -1) {
                for (i = 0; i < regions.length; i++)
                    this.canvasSeriesRect(regions[i], opacity, borderColor, borderWidth, color, true)
            }
            else if (seriesType == 'pie' || seriesType == 'doughnut')
                this.canvasHighlightCircle(chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions[0], seriesType, ctx);
            else if (seriesType == 'pyramid' || seriesType == 'funnel') 
				this.canvasHighlightPyramid(chart, regions[0], highlight, ctx, opacity, borderColor, borderWidth, color, seriesType);
            else if (seriesType == 'bubble')
                this.canvasBubbleHighlight(highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor);
            else if (seriesType == 'scatter')
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor);
        }
	    else if (mode == 'cluster' && this.model.AreaType != 'none' && $('#' + chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas').length == 0) {
                this.selectedPoint = pointIndex;
                var series;
                $('[id*=' + this._id + '_Selection_' + 'Cluster').remove();

                for (var i = 0, len = this.model._visibleSeries.length ; i < len ; i++) {
                    series = this.model._visibleSeries[i];
                    series.seriesIndex = i;
                    points.push(series.points[pointIndex]);
                }
                ctx = this.createCanvasElement(chart._id + '_Selection_' + 'Cluster' + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
                this.createRect(chart, ctx);
                for (i = 0, len = this.model._visibleSeries.length; i < len; i++) {
                    highlight = chart.model._visibleSeries[i].selectionSettings;
                    color = highlight.color;
                    opacity = highlight.opacity;
                    borderColor = highlight.border.color;
                    borderWidth = highlight.border.width;
                    patternName = highlight.pattern.toLowerCase();
                    fill = this.highlightFill(highlight, chart, i, pointIndex, index, legendData);
                    color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
                    if (seriesType.indexOf("column") != -1 || seriesType.indexOf("bar") != -1) {
                        this.canvasSeriesRect(regions[i], opacity, borderColor, borderWidth, color, true)
                    }
                    else if (seriesType == 'bubble') {
                        this.canvasBubbleHighlight(highlight, regions, chart, i, color, borderWidth, opacity, borderColor)
                    }
                    else if (seriesType == 'scatter') {
                        this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor)
                    }
                }
            }
        else if (mode == 'point' && $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length == 0) {
            this.selectedPoint = pointIndex;
			if (this.model.AreaType == 'none') $('[id*=' + chart._id + '_Selection_' + ']').remove();
            points[0] = series.points[pointIndex];
            ctx = this.createCanvasElement(chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            this.createRect(chart,ctx);
            color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
            if (seriesType.indexOf("column") != -1 || seriesType.indexOf("bar") != -1) {
                regions = []; regions[0] = data.region.Region;
                this.canvasSeriesRect(regions[0], opacity, borderColor, borderWidth, color, true)
            }
            else if (seriesType == 'pie' || seriesType == 'doughnut')
                this.canvasHighlightCircle(chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions[0], seriesType, ctx);
            else if (seriesType == 'pyramid' || seriesType == 'funnel') {
                this.canvasHighlightPyramid(chart, regions[0], highlight, ctx, opacity, borderColor, borderWidth, color, seriesType);
            }
            else if (seriesType == 'bubble') {
                regions = []; regions[0] = data.region.Region;
                this.canvasBubbleHighlight(highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor);
            }
            else if (seriesType == 'scatter') {
                regions = []; regions[0] = data.region.Region;
                this.canvasScatterHighlight(regions, color, borderWidth, opacity, borderColor)
            }
        }
        else if ($('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').length > 0) {
            $('#' + chart._id + '_Selection_' + seriesIndex + '_point_' + pointIndex + '_canvas').remove();
            $('#' + chart._id + '_Selection_Legend' + index + '_canvas').remove();
        }   
        // legend highlight (legend symbol)
        if (legendVisible){
            if (mode == 'cluster' && !legendData && this.model.AreaType != 'none') {
                    var series;
                    for (var i = 0, len = this.model._visibleSeries.length; i < this.model._visibleSeries.length; i++) {
                        series = this.model._visibleSeries[i];
                        series.seriesIndex = i;
                        this.canvasHighlightLegend(chart, series, highlight, 'Selection', containerStyle, color)
                    }
                }
            else
                    this.canvasHighlightLegend(chart, series, highlight, 'Selection', containerStyle, color)
            }

        if (series.marker.visible == true && this.model.AreaType != 'none')
            this.canvasHighlightMarker(chart, series, points, highlight, ctx, evt)
    },

     // clip rect logic calculated here
    createRect: function (chart,ctx) {
        if (chart.model.AreaType != 'none') {
            ctx.lineWidth = 0;
            ctx.strokeStyle = 'transparent';
            ctx.rect(chart.model.m_AreaBounds.X, chart.model.m_AreaBounds.Y, chart.model.m_AreaBounds.Width, chart.model.m_AreaBounds.Height);
            ctx.clip();
            ctx.stroke();
            ctx.save();
        }
    },

     //canvas pyramid and funnel draw
    canvasHighlightPyramid: function (chart, regions, highlight, ctx, opacity, borderColor, borderWidth, color, seriesType) {
        var fill = (highlight.color != "" ? highlight.color : chart.model.pointColors[pointIndex]);
        var measureTitle = (chart.model.title.text) ? ej.EjSvgRender.utils._measureText(chart.model.title.text, $(this.svgObject).width() - chart.model.margin.left - chart.model.margin.right, chart.model.title.font) : 0;
        var pyrX = chart.pyrX = ((chart.model.legend.position.toLowerCase() === "left") ? (chart.model.LegendViewerBounds.Width) : 0) + chart.model.elementSpacing + chart.model.margin.left;
        var pyrY = chart.pyrY = ((chart.model.legend.position.toLowerCase() === "top") ? (chart.model.LegendViewerBounds.Height) : 0) + ((chart.model.title.text) ? (chart.model._titleLocation.Y + measureTitle.height) : (chart.model.margin.top + chart.model.elementSpacing));
        var options = {
            'opacity': opacity,
            'stroke': borderColor,
            'stroke-width': borderWidth,
            'fill': color,
            'type': seriesType,
            'd': "M" + " " + (pyrX + regions.Line1.x) + " " + (pyrY + regions.Line1.y) + " " + "L" + " " + (pyrX + regions.Line2.x) + " " + (pyrY + regions.Line2.y) + " " + "L" + " " + (pyrX + regions.Line3.x) + " " + (pyrY + regions.Line3.y) + " " + "L" + " " + (pyrX + regions.Line4.x) + " " + (pyrY + regions.Line4.y) + " " + "z"
        }
        if (seriesType == 'funnel')
            options.d = "M" + " " + (pyrX + regions.Line1.x) + " " + (pyrY + regions.Line1.y) + " " + "L" + " " + (pyrX + regions.Line2.x) + " " + (pyrY + regions.Line2.y) + " " + "L" + " " + (pyrX + regions.Line3.x) + " " + (pyrY + regions.Line3.y) + " " + "L" + " " + (pyrX + regions.Line4.x) + " " + (pyrY + regions.Line4.y) + " " + "L" + " " + (pyrX + regions.Line5.x) + " " + (pyrY + regions.Line5.y) + " " + "L" + " " + (pyrX + regions.Line6.x) + " " + (pyrY + regions.Line6.y) + " " + "z";
        ej.EjCanvasRender.prototype.drawPath(options, ctx);
    },

     // canvas scatter logic
    canvasScatterHighlight: function (regions, color, borderWidth, opacity, borderColor) {
        var location, options = {}, size = {};
        for (var i = 0; i < regions.length; i++) {
            location = { startX: regions[i].Bounds.X + regions[i].Bounds.Height, startY: regions[i].Bounds.Y + regions[i].Bounds.Width };
            size.height = regions[i].Bounds.Height;
            size.width = regions[i].Bounds.Width;
            options = {};
            options.ShapeSize = size;
            options.cx = location.startX;
            options.cy = location.startY;
            options.Style = {};
            options.Style.BorderColor = borderColor;
            options.Style.Color = color;
            options.Style.Opacity = opacity;
            options.Style.BorderWidth = borderWidth;
            ej.EjSvgRender.chartSymbol['_drawCircle'](location, options, this, ctx);
        }
    },

     // canvas bubble logic perform here
    canvasBubbleHighlight: function (highlight, regions, chart, seriesIndex, color, borderWidth, opacity, borderColor) {
        var size = 10, radius, fill, bubbleOptions = {};
        for (var i = 0; i < regions.length; i++) {
            radius = regions[i].Bounds.Height / 2 + size;
            fill = (highlight.color != "" ? highlight.color : chart.model.seriesColors[seriesIndex]);
            bubbleOptions = {
                'cx': regions[i].Bounds.X - this.model.series[seriesIndex].xAxis.x + radius - size + this.canvasX,
                'cy': regions[i].Bounds.Y - this.model.series[seriesIndex].yAxis.y + radius - size + this.canvasY,
                'r': radius,
                'fill': color,
                'stroke-width': borderWidth,
                'opacity': opacity,
                'stroke': borderColor
            };
            ej.EjCanvasRender.prototype.drawCircle(bubbleOptions, ctx);
        }
    },

     // canvas circle draw
    canvasHighlightCircle: function (chart, highlight, pointIndex, opacity, borderColor, borderWidth, color, regions, seriesType, ctx) {
        var fill = (highlight.color != "" ? highlight.color : chart.model.pointColors[pointIndex]);
        var options = {
            'opacity': opacity,
            'stroke': borderColor,
            'lineWidth': borderWidth,
            'color': color,
            'x': regions.StartX,
            'y': regions.StartY,
            'radius': chart.model.Radius,
            'innerRadius': chart.model.dRadius,
            'startAngle': regions.StartAngle,
            'endAngle': regions.EndAngle,
            'type': seriesType
        }
        this.highlightSegment(ctx, options);
    },

     // canvas marker logic
    canvasHighlightMarker: function (chart, series, points, settings, ctx, evt) {
        var highlight = settings;
        var patternName = highlight.pattern.toLowerCase();
        var seriesType = series.type.toLowerCase();
        var options = {};
        var fill = (highlight.color != "" ? highlight.color : chart.model.seriesColors[series.seriesIndex]);
        color = (patternName == "none" || patternName == "") ? fill : this.canvasPattern(ctx, chart, evt, fill, highlight);
        options.Style = {};
        options.Style.BorderColor = highlight.border.color;
        options.Style.Color = color;
        options.Style.Opacity = highlight.opacity;
        options.Style.BorderWidth = highlight.border.width;
        var symbolLocation, location, markerOptions, symbolName;
        for (var i = 0; i < points.length; i++) {
            symbolLocation = points[i].symbolLocation;
            symbolLocation = (seriesType.indexOf("line") != -1 || seriesType.indexOf("area") != -1) ? points[i].location : points[i].symbolLocation;
            if (symbolLocation) {
                location = this.model.AreaType == "cartesianaxes" ? { startX: symbolLocation.X + this.canvasX, startY: symbolLocation.Y + this.canvasY } : { startX: symbolLocation.X, startY: symbolLocation.Y };
                markerOptions = series.marker;
                symbolName;
                $.each(chart.model.symbolShape, function (name) {
                    if (markerOptions.shape.toLowerCase() == name.toLowerCase())
                        symbolName = name;
                });
                options.ShapeSize = markerOptions.size;
                options.r = Math.sqrt(markerOptions.size.height * markerOptions.size.height + markerOptions.size.width * markerOptions.size.width) / 2;
                options.cx = location.startX;
                options.cy = location.startY;
                ej.EjSvgRender.chartSymbol['_draw' + symbolName](location, options, chart, ctx);
            }
        }

    },

     // canvas selction legend logic perform here
    canvasHighlightLegend: function (chart, series, settings, name, containerStyle, color) {
        var index = chart.model.AreaType == 'none' ? series.pointIndex : series.seriesIndex;
        var id = chart._id + '_' + name + '_Legend' + index + '_canvas';
        if ($('#' + id).length == 0) {
            legendCtx = this.createCanvasElement(chart._id + '_' + name + '_Legend' + index + '_canvas', chart.svgWidth, chart.svgHeight, containerStyle);
            if (chart.model.AreaType != 'none')
                legendRegion = chart.model.legendRegion[series.seriesIndex];
            else
                legendRegion = chart.model.legendRegion[series.pointIndex];
            var legendBounds = { startX: legendRegion.Bounds.LegendBound.X + legendRegion.Location.startX, startY: legendRegion.Bounds.LegendBound.Y + legendRegion.Location.startY };
            var svgData = { svgRenderer: chart.svgRenderer, svgObject: chart.svgObject };
            var symbolStyle = { 'Style': { 'Color': color, 'Opacity': settings.opacity, 'Visibility': true,'BorderWidth':0,BorderColor:color }, 'context': true, 'ShapeSize': legendRegion.Style.ShapeSize }
            ej.EjSvgRender.chartSymbol["_draw" + legendRegion.SymbolShape](legendBounds, symbolStyle, svgData, legendCtx);
        }
    },

     // canvas create element 
    createCanvasElement: function (id, width, height, style) {
        svgObj = document.createElement('canvas');
        _rootId = jQuery(this.element).attr("id");
        svgObj.setAttribute('id', id);
        svgObj.height = height;
        svgObj.width = width;
        this.svgRenderer.append(svgObj, this.element);
        var main = document.getElementById(id);
        main.style.left = style.left + $(document).scrollLeft() + 'px';
        main.style.top = style.top + $(document).scrollTop() + 'px';
        main.style.position = "absolute";

        return svgObj.getContext("2d");
    },

     // canvas pie doughnut logic
    highlightSegment: function (ctx, options) {
        ctx.save();
        ctx.beginPath();
        if (options.type == 'pie') {
            ctx.moveTo(options.x, options.y);
            ctx.arc(options.x, options.y, options.radius, options.startAngle - 1.57, options.endAngle - 1.57, false);
        } else {
            ctx.arc(options.x, options.y, options.radius, options.startAngle - 1.57, options.endAngle - 1.57, false);
            ctx.arc(options.x, options.y, options.innerRadius, options.endAngle - 1.57, options.startAngle - 1.57, true);
        }
        ctx.fillStyle = options.color;
        ctx.globalAlpha = options.opacity;
        ctx.fill();
        ctx.lineWidth = options.lineWidth;
        ctx.strokeStyle = options.stroke;
        ctx.clip();
        ctx.stroke();
        ctx.closePath();
        ctx.restore();        
    },

    // Canvas pattern started here
    canvasPattern: function (ctx, chart, evt, fill, settings) {

        var hoverStyle = settings;
        var name = hoverStyle.name;
        var style = hoverStyle.pattern.toLowerCase();
        var color = fill;
        var opacity = hoverStyle.opacity;
        var backgroundColor = "#ffffff";
        var borderColor = hoverStyle.border.color;
        var borderWidth = hoverStyle.border.width;
        var patternStyle = document.createElement('canvas');
        var patternContext = patternStyle.getContext('2d');
        var pathOptions = [], translate = [];
        switch (style) {
            case "chessboard":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'fill': 'white', 'stroke': 'white', 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = { 'x': 0, 'y': 0, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                pathOptions[2] = { 'x': 5, 'y': 5, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                patternStyle.width = 10;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "pacman":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 18, 'height': 18, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 9.081 9.194 L 14.887 6.114 C 14.075000000000001 4.618 12.484 3.0620000000000007 10.596 3.0620000000000007 L 8.835 3.0620000000000007 C 6.138 3.063 3 6.151 3 8.723 L 3 10.402000000000001 C 3 12.974 6.138 16.063000000000002 8.835 16.063000000000002 L 10.596 16.063000000000002 C 12.681000000000001 16.063000000000002 14.431000000000001 14.303000000000003 15.131 12.549000000000003 L 9.081 9.194 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 18;
                patternStyle.height = 18;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "crosshatch":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 8, 'height': 8, 'fill': 'white', 'stroke': fill, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 0 0 L 8 8 Z',
                    'stroke-width': 1,
                    'stroke': color,
                    'name': 'path'
                };
                pathOptions[2] = {
                    'd': 'M 8 0 L 0 8 Z',
                    'stroke-width': 1,
                    'stroke': color,
                    'name': 'path'
                };
                patternStyle.width = 8;
                patternStyle.height = 8;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
             case "dots":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 7, 'height': 7, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'cx': 3,
                    'cy': 3,
                    'r': 2,
                    'stroke-width': 1,
                    'fill': color,
                    'name': 'circle'
                };
                patternStyle.width = 7;
                patternStyle.height = 7;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "diagonalforward":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'fill': 'white', 'stroke': fill, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 3 -3 L 9 3 M 6 6 L 0 0 M 3 9 L -3 3',
                    'stroke-width': 2,
                    'stroke': color,
                    'name': 'path'
                };
                patternStyle.width = 6;
                patternStyle.height = 6;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "diagonalbackward":
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'stroke-width': 2, 'fill': 'white', 'stroke': fill, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 3 -3 L -3 3 M 0 6 L 6 0 M 9 3 L 3 9',
                    'stroke-width': 2,
                    'stroke': color,
                    'name': 'path'
                };
                patternStyle.width = 6;
                patternStyle.height = 6;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "grid":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M 1 3.5 L 11 3.5 M 0 3.5 L 11 3.5 M 0 7.5 L 11 7.5 M 0 11.5 L 11 11.5 M 5.5 0 L 5.5 12 M 11.5 0 L 11.5 12 Z',
                    'stroke-width': 1,
                    'stroke': color
                };
                patternStyle.width = 6;
                patternStyle.height = 6;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "turquoise":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 17, 'height': 17, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 2.697 0.5319999999999996 C 3.8695005238907747 0.5319999999999996 4.82 1.4771268931071944 4.82 2.643 C 4.82 3.808873106892805 3.8695005238907747 4.754 2.697 4.754 C 1.5244994761092252 4.754 0.5739999999999998 3.808873106892805 0.5739999999999998 2.643 C 0.5739999999999998 1.4771268931071944 1.5244994761092252 0.5319999999999996 2.697 0.5319999999999996 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M 13.928 0.5319999999999996 C 15.100500523890776 0.5319999999999996 16.051000000000002 1.4771268931071944 16.051000000000002 2.643 C 16.051000000000002 3.808873106892805 15.100500523890776 4.754 13.928 4.754 C 12.755499476109225 4.754 11.805 3.808873106892805 11.805 2.643 C 11.805 1.4771268931071944 12.755499476109225 0.5319999999999996 13.928 0.5319999999999996 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M 8.313 0.5319999999999996 C 9.485500523890776 0.5319999999999996 10.436 1.4771268931071944 10.436 2.643 C 10.436 3.808873106892805 9.485500523890776 4.754 8.313 4.754 C 7.140499476109226 4.754 6.19 3.808873106892805 6.19 2.643 C 6.19 1.4771268931071944 7.140499476109226 0.5319999999999996 8.313 0.5319999999999996 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M 13.928 6.16 C 15.100500523890776 6.16 16.051000000000002 7.105126893107196 16.051000000000002 8.271 C 16.051000000000002 9.436873106892806 15.100500523890776 10.382000000000001 13.928 10.382000000000001 C 12.755499476109225 10.382000000000001 11.805 9.436873106892806 11.805 8.271 C 11.805 7.105126893107196 12.755499476109225 6.16 13.928 6.16 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[5] = { 'name': 'path', 'd': 'M 8.313 6.16 C 9.485500523890776 6.16 10.436 7.105126893107196 10.436 8.271 C 10.436 9.436873106892806 9.485500523890776 10.382000000000001 8.313 10.382000000000001 C 7.140499476109226 10.382000000000001 6.19 9.436873106892806 6.19 8.271 C 6.19 7.105126893107196 7.140499476109226 6.16 8.313 6.16 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[6] = { 'name': 'path', 'd': 'M 13.928 11.787999999999998 C 15.100500523890776 11.787999999999998 16.051000000000002 12.733126893107194 16.051000000000002 13.899 C 16.051000000000002 15.064873106892804 15.100500523890776 16.009999999999998 13.928 16.009999999999998 C 12.755499476109225 16.009999999999998 11.805 15.064873106892804 11.805 13.899 C 11.805 12.733126893107194 12.755499476109225 11.787999999999998 13.928 11.787999999999998 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[7] = { 'name': 'path', 'd': 'M 8.313 11.787999999999998 C 9.485500523890776 11.787999999999998 10.436 12.733126893107194 10.436 13.899 C 10.436 15.064873106892804 9.485500523890776 16.009999999999998 8.313 16.009999999999998 C 7.140499476109226 16.009999999999998 6.19 15.064873106892804 6.19 13.899 C 6.19 12.733126893107194 7.140499476109226 11.787999999999998 8.313 11.787999999999998 Z', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                patternStyle.width = 17;
                patternStyle.height = 17;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "star":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 21, 'height': 21, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M 15.913 18.59 L 10.762 12.842 L 5.613 18.75 L 8.291 11.422 L 0.325 9.91 L 8.154 8.33 L 5.337 0.91 L 10.488 6.658 L 15.637 0.75 L 12.959 8.078 L 20.925 9.59 L 13.096 11.17 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                patternStyle.width = 21;
                patternStyle.height = 21;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "triangle":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M 4.987 0 L 7.48 4.847 L 9.974 9.694 L 4.987 9.694 L 0 9.694 L 2.493 4.847 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                patternStyle.width = 10;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "circle":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 9, 'height': 9, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'circle',
                    'cx': 5.125,
                    'cy': 3.875,
                    'r': 3.625,
                    'stroke-width': 1,
                    'fill': color
                };
                patternStyle.width = 9;
                patternStyle.height = 9;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "tile":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 18, 'height': 18, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0 9 L 0 0 L 9 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M 9 9 L 9 0 L 18 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M 0 18 L 0 9 L 9 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M 9 18 L 9 9 L 18 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 18;
                patternStyle.height = 18;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "horizontaldash":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0 1.5 L 10 1.5 M 0 5.5 L 10 5.5 M 0 9.5 L 10 9.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 12;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "verticaldash":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 1.5 0 L 1.5 10 M 5.5 0 L 5.5 10 M 9.5 0 L 9.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 12;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "rectangle":
                pathOptions[0] = { 'x': 0, 'y': 0, 'name': 'rect', 'width': 12, 'height': 12, 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                pathOptions[2] = { 'name': 'rect', 'x': 7, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                patternStyle.width = 12;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "box":
                pathOptions[0] = { 'x': 0, 'y': 0, 'name': 'rect', 'width': 13, 'height': 13, 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1.5, 'y': 1.5, 'width': 10, 'height': 9, 'fill': color, 'opacity': opacity };
                patternStyle.width = 10;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "horizontalstripe":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 12, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0 0.5 L 10 0.5 M 0 4.5 L 10 4.5 M 0 8.5 L 10 8.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 10;
                patternStyle.height = 12;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "verticalstripe":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 10, 'transform': "translate(0,0)", 'fill': 'white', 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M 0.5 0 L 0.5 10 M 4.5 0 L 4.5 10 M 8.5 0 L 8.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                patternStyle.width = 12;
                patternStyle.height = 10;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
            case "bubble":
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 20, 'height': 20, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'circle', 'cx': 5.217, 'cy': 11.325, 'r': 3.429, 'stroke-width': 1, 'fill': '#D0A6D1' };
                pathOptions[2] = { 'name': 'circle', 'cx': 13.328, 'cy': 6.24, 'r': 4.884, 'stroke-width': 1, 'fill': color };
                pathOptions[3] = { 'name': 'circle', 'cx': 13.277, 'cy': 14.66, 'r': 3.018, 'stroke-width': 1, 'fill': '#D0A6D1' };
                patternStyle.width = 20;
                patternStyle.height = 20;
                this.loadPattern(chart, pathOptions, '', patternContext);
                break;
              
           
           
            case "custom":
                patternStyle = document.getElementById(hoverStyle.customPattern);
                break;
        }
        var pattern = ctx.createPattern(patternStyle, 'repeat');
        return pattern;
    },
     // Canvas pattern end
     // Canvas highlight and selection end
     // SVG highlight and selection started here
    selection: function (chart, evt, series,tracker,legendData, data) {
       // var data = this.GetSeriesPoint(evt);
        var seriesIndex = $.inArray(series, chart.model._visibleSeries);
        var index = seriesIndex;
        var highlight = series.selectionSettings;
        var name = 'series' + seriesIndex;//seriesName.replace(/ /g, ''); // removed space for series name
        var found_class = false;
        var targetId = evt.target.id;
        var parentNodeId = evt.target.parentNode.id;
        var isElement = (parentNodeId.indexOf(this.svgObject.id + "_SeriesGroup_") >= 0 ||parentNodeId.indexOf(this.svgObject.id + "_Chart3D") >= 0) ? true : false;
        var pathName = $('#' + targetId).attr('name');
        var patternName = highlight.pattern.toLowerCase();
        var mode = highlight.mode;
        var seriesData = { seriesIndex: seriesIndex, series: series };
        $('style').each(function () {
            if ($(this).html().indexOf('.SelectionStyle' + name) > -1) {
                found_class = true;
            }
        });
        if (mode == 'series' || legendData || this.model.AreaType == 'none') {
            for (i = 0; i < this.model.series.length; i++) {
                if (chart.model.series[i].selectionSettings.mode != 'point' || this.model.AreaType == 'none') {
                    $(this.svgObject).find('[id*=' + this.svgObject.id + '_SeriesGroup_' + i + '],[id*=' + this.svgObject.id + '_symbolGroup_' + i + '],[id*=_Region_Series_' + i + '_]').each(function () {
                        $(this).find("*").attr('class', '');
                        $(this).attr('class', '');
                        if (i != index && chart.model._selectedData)
                            chart.model._selectedData[i] = null;
                    })
                }                
            }
            for (i = 0; i < this.model.legendCollection.length; i++) {
                if (chart.model.series[this.model.legendCollection[i].SeriesIndex].selectionSettings.mode != 'point' || this.model.AreaType == 'none') {
                    $('#' + this.svgObject.id + '_LegendItemShape' + i).attr('class', '');
                }
            }
        }
        if (mode == 'cluster' && this.model.AreaType != 'none' && !chart.model.enable3D) {
            $("[class*=" + "SelectionStyle]").attr('class', '');
            for (i = 0; i < chart.model._visibleSeries.length; i++) {
                $('#' + this.svgObject.id + '_LegendItemShape' + i).attr('class', '');
            }
        }
        if (chart.model.AreaType == 'none') {
            if (!chart.model.enable3D) {
                var matchStr = legendData ? this.svgObject.id + '_LegendItemShape' : this.svgObject.id + "_Series" + seriesIndex + "_Point";
                var id = legendData ? this.svgObject.id + '_LegendItemShape' + legendData.legendItem.LegendItem.PointIndex : targetId;
                pointIndex = id.substr(matchStr.length);
                index = pointIndex;
            } else {
                index = legendData ? legendData.legendItem.LegendItem.PointIndex : data.region.Region.PointIndex;
            }
        }



        if (chart.model.enable3D && !chart.vmlRendering) {
            // point previous selection removed here
            if (mode != 'cluster') {
                var selectedId = "_Region_Series_" + seriesIndex + "_Point_";
                var isSeriesSelected = $(this.svgObject).find('[id*=' + selectedId + ']');
                if (isSeriesSelected.length > 0) {
                    for (var i = 0; i < isSeriesSelected.length; i++) {
                        $('#' + isSeriesSelected[i].id).attr('class', '');
                    }
                    $("[id*=" + "SelectionSegment" + name + "]").remove();
                    $("[id*=" + "SelectionPath" + name + "]").remove();
                    $("[id*=" + "SelectionDef" + name + "]").remove();
                }
            }
            else {
                for (var i = 0, length = chart.model._visibleSeries.length; i < length ; i++) {
                    $("[class*=" + "Selectionseries]").attr('class', '');
                    $("[id *= '_LegendItemShape']").attr('class', '');

                }

            }
            if (mode == 'series' || legendData) {
                this.highlightSeries(chart, seriesIndex, index, 'Selection', name, highlight, evt, legendData)
                this._trigger("seriesRegionClick", seriesData);
                $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'SelectionStyle' + name + '1')
            }
            else if (mode == 'cluster' && chart.model.AreaType != 'none') {
                this.highlightCluster(chart, data.pointData.pointIndex, 'Selection', name, highlight, evt)
            }
            else {
                this.highlightPoint(chart, seriesIndex, data.pointData.pointIndex, 'Selection', name, highlight, evt)
                $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'SelectionStyle' + name + '1')
            }
        } else {
            if (mode != 'cluster') {
                var colorStyle = {};
                colorStyle.name = "2D";
                colorStyle.index = seriesIndex; colorStyle.api = "Selection";
                colorStyle.color = highlight.color != "" ? highlight.color : legendData ? (this.model.AreaType == 'none' ? this.model.pointColors[index] : this.model.seriesColors[index]) : $('#' + targetId).attr('fill');
                if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                    colorStyle.color = colorStyle.color[1].color;
                var fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? highlight.color : this.pattern(chart, evt, colorStyle, highlight);
                var opacity = highlight.opacity;
                var strokeColor = highlight.border.color;
                var strokeWidth = highlight.border.width;
                if (!found_class) {
                    this.createStyle(chart, 'Selection', name, opacity, fill, strokeColor, strokeWidth);
                    this.createStyle(chart, 'SelectionLegend', name, opacity, fill, 'transparent', 0);
                }
            }
        
            if (highlight.mode == 'series' || legendData) {
                if (!isElement) {
                    var otherElements = $('#' + evt.target.parentNode.id)[0].childNodes;
                    for (var i = 0; i < otherElements.length; i++)
                        $('#' + otherElements[i].id).attr('class', 'SelectionStyle');
                }
                if (chart.model.AreaType != 'none')
                    $("[id*=" + this.svgObject.id + '_Series' + index + "]").each(function () {
                        if (this.parentNode.id == chart.svgObject.id + '_SeriesGroup_' + index || this.parentNode.id == chart.svgObject.id + '_symbolGroup_' + index) {
                            var className = $(this).attr("name") ? 'SelectionPathStyle' + name : 'SelectionStyle' + name;
                            $(this).attr('class', className);
                        }
                    });
                else
                    legendData ? $('#' + this.svgObject.id + '_Series' + seriesIndex + '_Point' + index).attr('class', 'SelectionStyle' + name) : $('#' + targetId).attr('class', 'SelectionStyle' + name);

                //for legend highlight sets here
                $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'SelectionLegendStyle' + name);

                // removed tracker symbol
                if (tracker) $('#' + parentNodeId).remove();

                this._trigger("seriesRegionClick", seriesData);
            }

            else if (highlight.mode == "cluster" && chart.model.AreaType != 'none' && data) {
                var matchString, clustername, indexVal, text, colorStyle, opacity, strokeColor, fill, strokeWidth;
                var matchString;
                matchString = this.svgObject.id + "_Series";
                var clusterPoints = [];
                index = targetId.match(/(\d+)/g);
                pointIndex = parseInt(index[index.length - 1]);
                if (targetId.indexOf('SeriesText') >= 0) {
                    var datas = this.GetSeriesPoint(evt);
                    pointIndex = datas.region.Region.PointIndex;
                }
                var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].x;
                for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                    for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                        if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].x)
                            clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
                    }
                }
                for (k = 0; k < clusterPoints.length; k++) {
                    name = 'series' + k;
                    //creating individual styles for the points in different series
                    colorStyle = {};
                    colorStyle.name = "2D"; colorStyle.api = "Selection" + name;
                    highlight = chart.model._visibleSeries[k].selectionSettings;
                    colorStyle.color = highlight.color != "" ? highlight.color : this.model.seriesColors[k];
                    if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                        colorStyle.color = colorStyle.color[1].color;
                    opacity = highlight.opacity;
                    var strokeColor = highlight.border.color;
                    var strokeWidth = highlight.border.width;
                    var fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? colorStyle.color : this.pattern(chart, evt, colorStyle, highlight);
                    this.createStyle(chart, 'Selection', name, opacity, fill, strokeColor, strokeWidth);
                    this.createStyle(chart, 'SelectionLegend', name, opacity, fill, 'transparent', 0);
                    //applying the created style to the selected element 
                    if (!isElement) {
                        text = 'Point' + data.region.Region.PointIndex;
                        $('#' + targetId).attr('class', 'SelectionStyle');
                    }
                    var className = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr("name") ? 'SelectionPathStyle' + name : 'SelectionStyle' + name;
                    $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr('class', className);
                    $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex + '_symbol').attr('class', className);
                    if (chart.model._visibleSeries[k].visibility != "hidden")
                        $('#' + this.svgObject.id + '_LegendItemShape' + k).not("[class*='Highlight']").attr('class', 'SelectionLegendStyle' + name);
                }
                if (tracker) {
                    $('#' + this.svgObject.id + '_Series' + k + '_Point' + data.region.Region.PointIndex).attr('class', 'SelectionStyle' + name);
                    $('#' + this.svgObject.id + '_Series' + k + '_Point' + data.region.Region.PointIndex + '_symbol').attr('class', 'SelectionStyle' + name);
                    $('#' + parentNodeId).remove();
                }
            }


            else {
                // checked condition for whether target id is area related or not
                if (!$('#' + targetId).attr("name") && targetId != chart.svgObject.id + '_Series' + index) {
                    $("[id*=" + this.svgObject.id + "_LegendItemShape]").each(function () {
                        $(this).not("[class*='Selection']").attr('class', '');
                    });
                    if (isElement) {
                        $('#' + targetId).attr('class', 'SelectionStyle' + name);
                        $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'SelectionLegendStyle' + name);
                    } else
                        $('#' + targetId).attr('class', 'SelectionStyle');
                    $('#' + targetId + '_symbol').attr('class', 'SelectionStyle' + name);
                    if (data)
                        $('#' + this.svgObject.id + '_Series' + seriesIndex + '_Point' + data.region.Region.PointIndex).attr('class', 'SelectionStyle' + name);
                    if (tracker) {
                        $('#' + this.svgObject.id + '_Series' + seriesIndex + '_Point' + data.region.Region.PointIndex + '_symbol').attr('class', 'SelectionStyle' + name);
                        $('#' + parentNodeId).remove();
                    }
                }

                // checked condition for financial series
                var finName = $('#' + targetId).attr("name");
                if (finName != undefined && (finName == 'candle' || finName.indexOf('hilo') >= 0)) {
                    $('[id*=' + targetId + ']').attr('class', 'SelectionStyle' + name);
                    $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class', 'SelectionLegendStyle' + name);
                }
            }
        }
    },

     // style append operation perform here
    createStyle: function (chart, api,name, opacity, fill, strokeColor, strokeWidth) {
        if (chart.vmlRendering) {
            var style = document.createElement('style');
            style.id = api + "Segment";
            opacity = opacity * 100;
            var text = "." +api+"Style" + name + " {filter: alpha(opacity=" + opacity + ");}."+api+"PathStyle" + name + "  {filter: alpha(opacity=" + opacity + ");}";
            style.setAttribute("type", "text/css");
            if (style.styleSheet) {   // for IE
                style.styleSheet.cssText = text;
            } else {                // others
                var textnode = document.createTextNode(text);
                style.appendChild(textnode);
            }
            var header = document.getElementsByTagName('head')[0];
            header.appendChild(style);
        } else {
            $("<style id="+api+"Segment" + name + " type='text/css'> ."+api+"Style" + name + "{ fill:" + fill + ";opacity:" + opacity + ";stroke:" + strokeColor + ";stroke-width:" + strokeWidth + "}</style>").appendTo("body");
            $("<style id=" + api + "Path" + name + " type='text/css'> ." + api + "PathStyle" + name + "{ opacity:" + opacity + ";stroke:" + strokeColor + ";stroke-width:" + strokeWidth + "} </style>").appendTo("body");
        }
    },

     // point highlight 3D perform here
    highlightPoint: function (chart, seriesIndex, pointIndex,api,name,highlight,evt ) {
        var pathId = "_Region_Series_" + seriesIndex + "_Point_" + pointIndex;
        var gElement = $(this.svgObject).find('[id$=' + pathId + '],[id*=' + pathId + 'back],[id*=' + pathId + 'front]');
        for (var i = 0; i < gElement.length; i++) {
            var style3D = [];
            style3D.api = api; style3D.seriesName = name;
            style3D.name = $("[id=" + gElement[i].id + "]").attr('name');
            style3D.index = i;
            var color = this.highlightFill(highlight, chart, seriesIndex, pointIndex);
            var className = ($('#' + gElement[i].id).attr('class'));
            className = className ? className : '';
            if (className.indexOf('SelectionStyle') < 0 && (className.indexOf('Selection' + name + 'Style') < 0)) {
                $("[id=" + gElement[i].id + "]").attr('class', this.dStyle(style3D, color, chart, evt, highlight));
            }
        }
    },
       //cluster highlight of 3D is perfomed here
    highlightCluster: function (chart, pointIndex, api, name, highlight, evt) {
        if (chart.model.AreaType != 'none') {
            var name, pathId, gElement, style3D, color, classname, clusterPoints = [];
            var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].x;
            for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                    if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].x)
                        clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
                }
            }
            for (k = 0; k < clusterPoints.length; k++) {
                name = 'series' + k;
                pathId = "_Region_Series_" + clusterPoints[k].seriesIndex + "_Point_" + clusterPoints[k].pointIndex;
                 gElement = $(this.svgObject).find('[id$=' + pathId + '],[id*=' + pathId + 'back],[id*=' + pathId + 'front]');
                 for (var j = 0; j < gElement.length; j++) {
                        style3D = [];
                        style3D.api = api + name; style3D.seriesName = 'series' + k;
                        style3D.name = $("[id=" + gElement[j].id + "]").attr('name');
                        style3D.index = j;
                        seriesIndex = k;
                        highlight = (api == 'Highlight') ? chart.model._visibleSeries[k].highlightSettings : chart.model._visibleSeries[k].selectionSettings;
                        color = this.highlightFill(highlight, chart, seriesIndex, pointIndex);
                        className = ($('#' + gElement[j].id).attr('class'));
                        className = className ? className : '';
                        if ((className.indexOf('Selection' + name + 'Style') < 0) && (className.indexOf('SelectionStyle') < 0)) {
                            $("[id=" + gElement[j].id + "]").attr('class', this.dStyle(style3D, color, chart, evt, highlight));
                        }

                    }
                    if (api == 'Highlight')
                        $('#' + this.svgObject.id + '_LegendItemShape' + k).not("[class*='Selection']").attr('class', 'Highlight' + name + 'Style' + name + '1');
                    else
                        $('#' + this.svgObject.id + '_LegendItemShape' +k).attr('class', 'Selection' + name + 'Style' + name + '1');
                }

       }
    },  
     // point series 3D perform here
    highlightSeries: function (chart, seriesIndex, index, api, name, highlight, evt, legendData){
        var data = this.GetSeriesPoint(evt);
        if (data) {
            var seriesIndex = data.region.SeriesIndex;
            var pointIndex = data.region.Region.PointIndex;
        }
        var pathId = chart.model.AreaType == 'none' ? "_Region_Series_" + seriesIndex + "_Point_" + index : "_Region_Series_" + seriesIndex + "_Point_";
        var gElement = $(this.svgObject).find('[id*=' + pathId + ']');
        var className = legendData ? $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class') : ($('#' + evt.target.id).attr('class'));
        className = className ? className : '';
        if (className.indexOf('SelectionStyle') < 0 || legendData) {
            for (var i = 0; i < gElement.length; i++) {
                var style3D = [];
                style3D.api = api; style3D.seriesName = name;
                style3D.name = $("[id=" + gElement[i].id + "]").attr('name');
                style3D.index = i;
                var color = this.highlightFill(highlight, chart, seriesIndex, pointIndex, index, legendData);
				var className = $('#' + gElement[i].id).attr('class');
                className = className ? className : '';
                if (className.indexOf('SelectionStyle') < 0&& (className.indexOf('Selection' + name + 'Style') < 0)) {
                    $("[id=" + gElement[i].id + "]").attr('class', this.dStyle(style3D, color, chart, evt, highlight));
                }
            }
        }
    },

     //SVG highlight logic started here
    highlight: function (chart, evt, series, legendData) {
        var data = this.GetSeriesPoint(evt);
        var highlight = series.highlightSettings;
        var seriesIndex = $.inArray(series, chart.model._visibleSeries);
        var name = 'series' + seriesIndex;//series.name.replace(/ /g, ''); // removed space for series name
        var found_class = false;
        var index = seriesIndex;
        var targetId = evt.target.id;
        var parentNodeId = evt.target.parentNode.id;
        var isElement = (parentNodeId.indexOf(this.svgObject.id + "_SeriesGroup_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_Chart3D") >= 0) ? true : false;
        var pathName = $('#' + targetId).attr('name');
        var patternName = highlight.pattern.toLowerCase();
        if (chart.model.AreaType == 'none')
        {
            if (!chart.model.enable3D) {
                var matchStr = legendData ? this.svgObject.id + '_LegendItemShape' : this.svgObject.id + "_Series" + seriesIndex + "_Point";
                var id = legendData ? this.svgObject.id + '_LegendItemShape' + legendData.legendItem.LegendItem.PointIndex : targetId;
                pointIndex = id.substr(matchStr.length);
                index = pointIndex;
            } else {
                index = legendData ? legendData.legendItem.LegendItem.PointIndex:data.region.Region.PointIndex;
            }
        }
        $('style').each(function () {
            if ($(this).html().indexOf('.HighlightStyle'+ name) > -1) {
                found_class = true;
            }
        });
        if (chart.model.enable3D && !chart.vmlRendering) {
            if (highlight.mode == 'series' || legendData) {
                this.highlightSeries(chart, seriesIndex, index, 'Highlight', name, highlight, evt,legendData)
                $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightStyle' + name + '1');
            } else if (highlight.mode == 'cluster' && chart.model.AreaType != 'none') {
                this.highlightCluster(chart, data.pointData.pointIndex, 'Highlight', name, highlight, evt)
                
            } else {
                this.highlightPoint(chart, seriesIndex, data.pointData.pointIndex, 'Highlight', name, highlight, evt)
                $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightStyle' + name + '1');
            }
        } else {
            if (highlight.mode != "cluster") {
                var colorStyle = {};
                colorStyle.name = "2D";
                colorStyle.index = seriesIndex; colorStyle.api = "Highlight"; colorStyle.color = highlight.color != "" ? highlight.color : legendData ? (this.model.AreaType == 'none' ? this.model.pointColors[index] : this.model.seriesColors[index]) : $('#' + targetId).attr('fill');
                if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                    colorStyle.color = colorStyle.color[1].color;
                var fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? highlight.color : this.pattern(chart, evt, colorStyle, highlight);
                var opacity = highlight.opacity;
                var strokeColor = highlight.border.color;
                var strokeWidth = highlight.border.width;
                if (!found_class) {
                    this.createStyle(chart, 'Highlight', name, opacity, fill, strokeColor, strokeWidth);
                    this.createStyle(chart, 'HighlightLegend', name, opacity, fill, 'transparent', 0);
                }
            }

            // sets the style for the segment or path started here
            var className = legendData ? $('#' + this.svgObject.id + '_LegendItemShape' + index).attr('class') : $('#' + targetId).attr('class');
            if (className != "SelectionStyle" + name && className != "SelectionPathStyle" + name) {
                // checked mode of the highlight series
                if (highlight.mode == 'series' || legendData) {
                    if (chart.model.AreaType != 'none')
                        $("[id*=" + this.svgObject.id + '_Series' + index + "]").each(function () {
                            if (this.parentNode.id == chart.svgObject.id + '_SeriesGroup_' + index || this.parentNode.id == chart.svgObject.id + '_symbolGroup_' + index) {
                                var selectClass = $(this).attr('class');
                                var className = $(this).attr("name") ? 'HighlightPathStyle' + name : 'HighlightStyle' + name;
                                // this condition is checked for whether already selected and it's symbol
                                if (selectClass != 'SelectionStyle' + name && selectClass != 'SelectionPathStyle' + name && parentNodeId.indexOf(chart.svgObject.id + '_symbolGroup_' + index) <= 0) {
                                    $(this).attr('class', className);
                                }
                            }
                        });
                    else
                        legendData ? $('#' + this.svgObject.id + '_Series' + seriesIndex + '_Point' + index).not("[class*='Selection']").attr('class', 'HighlightStyle' + name) : $('#' + targetId).attr('class', 'HighlightStyle' + name);

                    //for legend highlight sets here
                    $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
                }
                 else if (highlight.mode == "cluster" && chart.model.AreaType != 'none' && data) {
                    var matchString;
                    matchString = this.svgObject.id + "_Series";
                    var clusterPoints = [];
                    index = targetId.match(/(\d+)/g);
                    pointIndex = parseInt(index[index.length - 1]);
                    var xvalue = chart.model._visibleSeries[seriesIndex]._visiblePoints[pointIndex].x;
                    for (var i = 0, len = chart.model._visibleSeries.length; i < len ; i++) {
                        for (var j = 0; j < chart.model._visibleSeries[i]._visiblePoints.length; j++) {
                            if (xvalue == chart.model._visibleSeries[i]._visiblePoints[j].x)
                                clusterPoints[clusterPoints.length] = { seriesIndex: i, pointIndex: j };
                        }
                    }
                    for (k = 0; k < clusterPoints.length; k++) {
                        name = 'series' + k;
                        //creating individual styles for the points in different series
                        colorStyle = {};
                        colorStyle.name = "2D"; colorStyle.api = "Highlight" + name;
                        highlight = chart.model._visibleSeries[k].highlightSettings;
                        colorStyle.color = highlight.color != "" ? highlight.color : this.model.seriesColors[k];
                        if (Object.prototype.toString.call(colorStyle.color) === '[object Array]')
                            colorStyle.color = colorStyle.color[1].color;
                        opacity = highlight.opacity;
                        strokeColor = highlight.border.color;
                        strokeWidth = highlight.border.width;
                        fill = ((patternName == "none" || chart.vmlRendering) || patternName == "") ? colorStyle.color : this.pattern(chart, evt, colorStyle, highlight);
                        if (!found_class) {
                            this.createStyle(chart, 'Highlight', name, opacity, fill, strokeColor, strokeWidth);
                            this.createStyle(chart, 'HighlightLegend', name, opacity, fill, 'transparent', 0);
                        }
                        if (!isElement) {
                            text = 'Point' + data.region.Region.PointIndex;
                        }
                        //applying the created style to the selected element 
                        var selectClass = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr('class');
                        var symbol = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex + '_symbol').attr('class');
                        var className = $('#' + matchString + clusterPoints[k].seriesIndex + '_Point' + clusterPoints[k].pointIndex).attr("name") ? 'HighlightPathStyle' + name : 'HighlightStyle' + name;
                        // this condition is checked for whether already selected and it's symbol
                        if (selectClass != 'SelectionStyle' + 'series' + k && selectClass != 'SelectionPathStyle' + name && parentNodeId.indexOf(chart.svgObject.id + '_symbolGroup_' + k) <= 0) {
                            $('#' + matchString + clusterPoints[k].seriesIndex + '_' + 'Point' + clusterPoints[k].pointIndex).attr('class', className);
                            $('#' + matchString + clusterPoints[k].seriesIndex + '_' + 'Point' + clusterPoints[k].pointIndex + '_symbol').attr('class', className);
                        }
                        if (chart.model._visibleSeries[k].visibility != "hidden")
                            $('#' + this.svgObject.id + '_LegendItemShape' + k).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
                    }

                }

				else {
					// checked condition for whether target id is area related or not
					if (!$('#' + targetId).attr("name") && targetId != chart.svgObject.id + '_Series' + index) {
						selectClass = $('#' + targetId).attr('class') ? $('#' + targetId).attr('class') : '';
						if ((selectClass.indexOf('Selection') != 0 && parentNodeId.indexOf(chart.svgObject.id + '_symbolGroup_' + index) <= 0 && data)) {
                            if(isElement)
                                $('#' + targetId).attr('class', 'HighlightStyle' + name);
                            $('#' + targetId + '_symbol').attr('class', 'HighlightStyle' + name);
                            $('#' + this.svgObject.id + '_Series' + seriesIndex + '_Point' + data.region.Region.PointIndex).not("[class*='Selection']").attr('class', 'HighlightStyle' + name);
                            $('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
                        }
                    }
					// checked condition for financial series
                    var finName = $('#' + targetId).attr("name");
                    if (finName != undefined && (finName == 'candle' || finName.indexOf('hilo') >= 0)) {
                        $('[id*=' + targetId + ']').attr('class', 'HighlightStyle' + name);
						$('#' + this.svgObject.id + '_LegendItemShape' + index).not("[class*='Selection']").attr('class', 'HighlightLegendStyle' + name);
					}

				}
			}
        }

    },

     // 3D style class created here
    dStyle: function (style3D, color, chart, evt, settings) {
        var vector = ej.Ej3DRender.prototype.polygon3D.prototype;
        var highlight = settings;
        var name = style3D.api;
        var seriesName = style3D.seriesName;
        var opacity = highlight.opacity;
        color = chart.colorNameToHex(color);
        if (style3D.name == "XLight")
            color = vector.applyXLight(color, chart);
        else if (style3D.name == "ZLight")
            color = vector.applyZLight(color, chart);
        else
            color = color;
        var colorStyle = {};
        colorStyle.name = style3D.name; colorStyle.color = color; colorStyle.api = name;
        color = (highlight.pattern == "none") ? color : this.pattern(chart, evt, colorStyle, highlight);
        if (document.getElementById(name + "Segment" + seriesName)) {
            var style = document.getElementById(name + "Segment" + seriesName);
            style.innerHTML = style.innerHTML + '.' + name + 'Style' + seriesName+ style3D.index + ' { fill:' + color + ";opacity:" + opacity + ' }';
        } else {
            var style = document.createElement('style');
            style.id = name + "Segment" + seriesName;
            style.type = 'text/css';
            style.innerHTML = '.' + name + 'Style' + seriesName + style3D.index + ' { fill:' + color + ";opacity:" + opacity + ' }';
            document.getElementsByTagName('head')[0].appendChild(style);
        }
        return name + "Style" + seriesName + style3D.index;

    },

     // 3D style class end

     // pattern of the SVG started here
    pattern: function (chart, evt, colorStyle, settings) {
        var hoverStyle = settings;
        var name = colorStyle.api;
        var style = hoverStyle.pattern.toLowerCase();
        var color = colorStyle.color;
        var opacity = hoverStyle.opacity != "" ? hoverStyle.opacity : $('#' + evt.target.id).attr('opacity');
        var backgroundColor = "#ffffff";
        if (document.getElementById(name + "Def")) {
            var svg = $("#" + name + "Def")[0];
        } else {
            var svgid = { 'id': name + "Def" }
            var svg = chart.svgRenderer.createPattern(svgid, 'svg');
            chart.svgRenderer.append(svg, chart.svgObject);
        }
        if (chart.model.enable3D) {
            if (colorStyle.name == "XLight")
                backgroundColor = "#595959";
            else if (colorStyle.name == "ZLight")
                backgroundColor = "#737373";
            else
                backgroundColor = "#808080";
        }
        if ($("#" + style + '_' + name + '_' + colorStyle.name)) {
            $("#" + style + '_' + name + '_' + colorStyle.name).remove();
        }
        var pathOptions = [];
        switch (style) {
            case "chessboard":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'x': 0, 'y': 0, 'width': 10, 'height': 10 }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = { 'x': 0, 'y': 0, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                pathOptions[2] = { 'x': 5, 'y': 5, 'width': 5, 'height': 5, 'fill': color, 'opacity': opacity, 'name': 'rect' };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "pacman":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '17.917', 'height': '18.384' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 17.917, 'height': 18.384, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M9.081,9.194l5.806-3.08c-0.812-1.496-2.403-3.052-4.291-3.052H8.835C6.138,3.063,3,6.151,3,8.723v1.679   c0,2.572,3.138,5.661,5.835,5.661h1.761c2.085,0,3.835-1.76,4.535-3.514L9.081,9.194z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "crosshatch":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '8', 'height': '8' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 8, 'height': 8, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M0 0L8 8ZM8 0L0 8Z',
                    'stroke-width': 1,
                    'stroke': color,
                    'name': 'path'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "dots":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 7, 'height': 7, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'cx': 3,
                    'cy': 3,
                    'r': 2,
                    'stroke-width': 1,
                    'fill': color,
                    'name': 'circle'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "diagonalforward":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name': 'rect' };
                pathOptions[1] = {
                    'd': 'M 3 -3 L 9 3 M 6 6 L 0 0 M 3 9 L -3 3',
                    'stroke-width': 2,
                    'stroke': color,
                    'name': 'path'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "diagonalbackward":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity, 'name' : 'rect' };
                pathOptions[1]= {
                    'd': 'M 3 -3 L -3 3 M 0 6 L 6 0 M 9 3 L 3 9',
                    'stroke-width': 2,
                    'stroke': color,
                    'name':'path'
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "grid":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '6', 'height': '6' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 6, 'height': 6, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M1 3.5L11 3.5 M0 3.5L11 3.5 M0 7.5L11 7.5 M0 11.5L11 11.5 M5.5 0L5.5 12 M11.5 0L11.5 12Z',
                    'stroke-width': 1,
                    'stroke': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "turquoise":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '17', 'height': '17' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 17, 'height': 17, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0.5739999999999998,2.643a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M11.805,2.643a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M6.19,2.643a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M11.805,8.217a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[5] = { 'name': 'path', 'd': 'M6.19,8.217a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[6] = { 'name': 'path', 'd': 'M11.805,13.899a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                pathOptions[7] = { 'name': 'path', 'd': 'M6.19,13.899a2.123,2.111 0 1,0 4.246,0a2.123,2.111 0 1,0 -4.246,0', 'stroke-width': 1, 'stroke-miterlimit': 10, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "star":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '21', 'height': '21' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 21, 'height': 21, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M15.913,18.59L10.762 12.842 5.613 18.75 8.291 11.422 0.325 9.91 8.154 8.33 5.337 0.91 10.488 6.658 15.637 0.75 12.959 8.078 20.925 9.59 13.096 11.17 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "triangle":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '10', 'height': '10' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 10, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'path',
                    'd': 'M4.987,0L7.48 4.847 9.974 9.694 4.987 9.694 0 9.694 2.493 4.847 z',
                    'stroke-width': 1,
                    'stroke': color,
                    'fill': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "circle":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '9', 'height': '9' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 9, 'height': 9, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = {
                    'name': 'circle',
                    'cx': 5.125,
                    'cy': 3.875,
                    'r': 3.625,
                    'stroke-width': 1,
                    'fill': color
                };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "tile":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '18', 'height': '18' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 18, 'height': 18, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0,9L0 0 9 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[2] = { 'name': 'path', 'd': 'M9,9L9 0 18 0 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[3] = { 'name': 'path', 'd': 'M0,18L0 9 9 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                pathOptions[4] = { 'name': 'path', 'd': 'M9,18L9 9 18 9 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "horizontaldash":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '12', 'height': '12' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0,1.5 L10 1.5 M0,5.5 L10 5.5 M0,9.5 L10 9.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "verticaldash":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '12', 'height': '12' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 12, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M1.5,0 L1.5 10 M5.5,0 L5.5 10 M9.5,0 L9.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "rectangle":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'x': 0, 'y': 0, 'width': 12, 'height': 12 }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'width': 12, 'height': 12, 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                pathOptions[2] = { 'name': 'rect', 'x': 7, 'y': 2, 'width': 4, 'height': 9, 'fill': color, 'opacity': opacity };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "box":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'x': 0, 'y': 0, 'width': 10, 'height': 10 }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'width': 13, 'height': 13, 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'rect', 'x': 1.5, 'y': 1.5, 'width': 10, 'height': 9, 'fill': color, 'opacity': opacity };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "horizontalstripe":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '10', 'height': '12' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 10, 'height': 12, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0,0.5 L10 0.5 M0,4.5 L10 4.5 M0,8.5 L10 8.5 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "verticalstripe":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '12', 'height': '10' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = { 'name': 'rect', 'x': 0, 'y': 0, 'width': 12, 'height': 10, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'path', 'd': 'M0.5,0 L0.5 10 M4.5,0 L4.5 10 M8.5,0 L8.5 10 z', 'stroke-width': 1, 'stroke': color, 'fill': color };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
            case "bubble":
                var patternGroup = { 'id': style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index, 'patternUnits': 'userSpaceOnUse', 'width': '20', 'height': '20' }
                var pattern = chart.svgRenderer.createPattern(patternGroup, 'pattern');
                pathOptions[0] = {'name':'rect', 'x': 0, 'y': 0, 'width': 20, 'height': 20, 'transform': "translate(0,0)", 'fill': backgroundColor, 'opacity': opacity };
                pathOptions[1] = { 'name': 'circle', 'cx': 5.217, 'cy': 11.325, 'r': 3.429, 'stroke-width': 1, 'fill': '#D0A6D1' };
                pathOptions[2] = { 'name': 'circle', 'cx': 13.328, 'cy': 6.24, 'r': 4.884, 'stroke-width': 1, 'fill': color };
                pathOptions[3] = { 'name': 'circle', 'cx': 13.277, 'cy': 14.66, 'r': 3.018, 'stroke-width': 1, 'fill': '#D0A6D1' };
                this.loadPattern(chart, pathOptions, pattern);
                svg.appendChild(pattern);
                return "url(#" + style + "_" + name + "_" + colorStyle.name + "_" + colorStyle.index + ")";
                break;
             case "custom":
                return "url(#" + hoverStyle.customPattern + ")"
                break;
        }

    },
	loadPattern: function (chart, options, pattern, ctx) {
        var path;
        for (var i = 0; i < options.length; i++) {
            if (!ctx) {
                path = chart.svgRenderer.createPattern(options[i], options[i].name);
                pattern.appendChild(path);
            } else {
                if (options[i].name == 'rect')
                    this.drawRect(options[i], ctx);
                else if (options[i].name == 'path')
                    ej.EjCanvasRender.prototype.drawPath(options[i], ctx);
                else if(options[i].name == 'circle')
                    ej.EjCanvasRender.prototype.drawCircle(options[i], ctx);
                }
        }
    },
	drawRect: function (options, element) {
        element.save();
        element.beginPath();
        element.globalAlpha = options.opacity;
        element.lineWidth = options["stroke-width"];
        element.strokeStyle = options.stroke;
	    element.rect(options.x, options.y, options.width, options.height);
        if (options.fill == "none") options.fill = "transparent";
        element.fillStyle = options.fill;
        if (options.transform) element.translate(options.transform[0], options.transform[1]);
        element.rotate(options.rotate * Math.PI / 180);        
        element.fillRect(options.x, options.y, options.width, options.height);
        element.clip();
        element.stroke();
        element.restore();
    },
     // pattern of the SVG end
     // SVG highlight and selection end

    tooltip: function (chart, evt) {
        var tooltipShowing = false;
        var chartSeries;
        var seriesIndex;
        var data = this.GetSeriesPoint(evt);
        $("#" + "canvas" + "_trackSymbol").remove();
		if (ej.util.isNullOrUndefined(data)) { // To hide the tooltip when it is moved on chartArea other than series
             $("#" + chart.svgObject.id + "_TrackToolTip").remove();
            //Removing tooltip Template when mouse is moved on chart area other than series
            $(".tooltipDiv").remove();
		}	
		if (!this.model.crosshair.visible && this.model.AreaType == "cartesianaxes") {
		    var indicators = this.model.indicators;
		    var seriesCollection = this.model._visibleSeries;
			var currentIndicator;
		    for (var j = 0; j < indicators.length && indicators[j].segment; j++) {
			    currentIndicator = indicators[j]; 
			    if (currentIndicator.visible)
		            seriesCollection = seriesCollection.concat(currentIndicator.segment);
		    }
		    var seriesCollectionLength = seriesCollection.length;
		    for (var i = 0; i < seriesCollectionLength ; i++) {
		        chartSeries = seriesCollection[i];
                seriesIndex = i;
				
				var type = chartSeries.type.toLowerCase(); // to remove track symbol on mouse move out of the point
				if ((ej.util.isNullOrUndefined(data)) && (type == "bubble" || type == "scatter"))
				    $('#' + chart.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
				
				if (chartSeries.visibility.toLowerCase() == 'visible' && type !== "scatter" && type !== "bubble" && type !== "column" && (type.indexOf("bar")==-1) && type !== "stackingcolumn" && type !== "stackingcolumn100" && !chartSeries._hiloTypes) {
                    $("#" + chart.svgObject.id + "_TrackToolTipTemplate_" + seriesIndex).remove();
                    if (!($(".tooltipDiv").hasClass(chart.svgObject.id)))
                        $(".tooltipDiv").remove();
                    $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackSymbol" + '_' + seriesIndex).remove();
                    $('#' + chart.svgObject.id + '_TrackSymbol' + '_' + seriesIndex).remove();
                    var serY = [];
                    var serX = [];
                    location = null;
                    var closestXyPoint = chart.getClosesPointXY(serX, serY, chartSeries, chart.mousemoveX, chart.mousemoveY);
                    if (closestXyPoint.point) {
                        location = ej.EjSvgRender.utils._getPoint(closestXyPoint.point, chartSeries);
                        var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                        commonPointEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveX }, region: { SeriesIndex: i, Region: { PointIndex: closestXyPoint.index } } };
                        chart._trigger("pointRegionMouseMove", commonPointEventArgs);
                    }
                    var pointData = this.model.prevPoint;
                    if (pointData && pointData.seriesIndex == i && !closestXyPoint.point) {
                        var pointLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                        chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, pointLocation);
                        this.model.prevPoint = null;
                    }
                    
                    if (!chartSeries.isIndicator && chartSeries.marker.visible && closestXyPoint.point && !commonPointEventArgs.cancel) {
                        if (ej.util.isNullOrUndefined(closestXyPoint.point.marker) || ej.util.isNullOrUndefined(closestXyPoint.point.marker.visible) || (closestXyPoint.point.marker && closestXyPoint.point.marker.visible))
                        chart.drawTrackerSymbol(chartSeries, seriesIndex, closestXyPoint.index, false, location);

                        if (pointData && closestXyPoint.point != pointData.point){
                            var prevLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                            chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, prevLocation, id);
                        }
                        if (ej.util.isNullOrUndefined(closestXyPoint.point.marker) || ej.util.isNullOrUndefined(closestXyPoint.point.marker.visible) || (closestXyPoint.point.marker && closestXyPoint.point.marker.visible))
                        this.model.prevPoint = { point: closestXyPoint.point, pointIndex: closestXyPoint.index, series: chartSeries, seriesIndex: seriesIndex };
                    }


                    if (chartSeries.tooltip.visible && closestXyPoint.point && (!data || (i > data.region.SeriesIndex && !(chartSeries._zOrder < this.model._visibleSeries[data.region.SeriesIndex]._zOrder)))) {
                        trans = ej.EjSvgRender.utils._getTransform(chartSeries.xAxis, chartSeries.yAxis, chart.model.requireInvertedAxes);
                        if ((location.X + trans.x) <= (trans.x + trans.width) && ((location.X + trans.x) >= trans.x || location.X == 0) && (Math.abs(location.Y - (trans.y + trans.height)) <= (trans.y + trans.height))
                                     && (Math.abs(location.Y - (trans.y + trans.height)) >= trans.y || location.Y == 0)) {                         
                            if (!commonPointEventArgs.cancel) {
                                if ($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").length == 0) {
                                    var transOptions = { 'id': chart.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                    chart.gTransToolEle = chart.svgRenderer.createGroup(transOptions);
                                }
                                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')', 'visibility': 'visible' });
                                if (ej.util.isNullOrUndefined(chartSeries.tooltip.template)) {
                                    tooltipShowing = true;
                                    $(".tooltipDiv").remove();
                                    chart.displayShowTooltip(location, closestXyPoint.point, chartSeries, closestXyPoint.index);
                                }
                                else {
                                    tooltipShowing = true;
                                    $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
                                    var region = { SeriesIndex: seriesIndex, Region: { PointIndex: closestXyPoint.index } };
                                    chart.createTooltip(region, evt, chartSeries);
                                }

                                //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                                // var detachEle = $(chart.svgObject).find("#" + chart.svgObject.id + "_TrackToolTip").detach();
                                //detachEle.appendTo(chart.svgObject);
                            }
                           
                        }
                    }
                }
            }
        }
	   else {
            $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").hide();
        }
       
        if (!($(".tooltipDiv").hasClass(this.svgObject.id)))
            $(".tooltipDiv").remove();
        if (data && ej.util.isNullOrUndefined(data.region.isStripLine)) {
            if (this.model.AreaType == "none" && this.model._visibleSeries[data.region.SeriesIndex].explode) {
                if (this.model._visibleSeries[data.region.SeriesIndex].enableAnimation) {
                    if (this.model.AnimationComplete || this.model.enableCanvasRendering)
                        this.pieExplosion(data, evt);
                }
                else if(!this.model.enable3D)
                    this.pieExplosion(data, evt);
            }
            if (data.region.isIndicator) {
                series = this.model.indicators[data.region.SeriesIndex];
                var seriesPoint = series.segment[0].points[data.region.Region.PointIndex];
                series.fill = series.segment[0].fill;
            } else {
                series = this.model._visibleSeries[data.region.SeriesIndex];
                var seriesPoint = this.model._visibleSeries[data.region.SeriesIndex].points[data.region.Region.PointIndex];
            }

            if (seriesPoint && (series.type.toLowerCase() == "bubble" || series.type.toLowerCase() == "scatter" || this.model.AreaType=='polaraxes')) {
                for (var i = 0; i < this.model._visibleSeries.length; i++) {
                    $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackSymbol" + '_' + i).remove();
                    var trackId = "canvas_trackSymbol_" + i;
                    $("#" + this._id).find('[id*=' + trackId + ']').remove();
                }
                var pointlocation = (this.model.AreaType != "polaraxes")
                    ? ej.EjSvgRender.utils._getPoint(seriesPoint, series)
                    : (ej.EjSvgRender.utils.TransformToVisible(series, seriesPoint.xValue, seriesPoint.y, this))
               
                if (this.model.AreaType == "cartesianaxes")
                    this.drawTrackerSymbol(series, data.region.SeriesIndex, data.region.Region.PointIndex, true, pointlocation);
                else {
                    var prevPoint = this.model.prevPoint;
                    if (!ej.util.isNullOrUndefined(prevPoint) && prevPoint!= data && series.marker.visible)
                    {
                        series = this.model._visibleSeries[prevPoint.region.SeriesIndex];
                        seriesPoint = this.model._visibleSeries[prevPoint.region.SeriesIndex].points[prevPoint.region.Region.PointIndex];
                        pointlocation = ej.EjSvgRender.utils.TransformToVisible(series, seriesPoint.xValue, seriesPoint.y, this);
                        this.drawTrackerSymbol(series, prevPoint.region.SeriesIndex, prevPoint.region.Region.PointIndex, null, pointlocation);
                        this.model.prevPoint = null;
                    }
                    if (series.marker.visible && series.drawType != "column") {
                        this.drawTrackerSymbol(series, data.region.SeriesIndex, data.region.Region.PointIndex, true, pointlocation);
                        this.model.prevPoint = data;                        
                    }
                }
            }

            if (series.tooltip.visible && !tooltipShowing) {

                if (series.visibility.toLowerCase() == 'visible') {
                    if (ej.util.isNullOrUndefined(series.tooltip.template)) {
                        var location;
                        if (this.model.AreaType == "cartesianaxes") {

                            var trans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
                            if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                var transToolOptions = { 'id': this.svgObject.id + '_TrackToolTip', 'transform': 'translate(' + trans.x + ',' + trans.y + ')' };
                                this.gTransToolEle = this.svgRenderer.createGroup(transToolOptions);
                            }
                            this.svgRenderer._setAttr($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip"), { 'transform': 'translate(' + trans.x + ',' + trans.y + ')' });
                            $('#' + this.svgObject.id + "_TrackToolTip").attr("visibility", "visible");
                            location = { X: (data.location.x - this.model.m_AreaBounds.X), Y: Math.abs(data.location.y - (this.model.m_AreaBounds.Y)) };
                        } else {
                            if ($(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").length == 0) {
                                var transToolTipOptions = { 'id': this.svgObject.id + '_TrackToolTip' };
                                this.gTransToolEle = this.svgRenderer.createGroup(transToolTipOptions);
                            }
                            location = { X: (data.location.x), Y: Math.abs(data.location.y) };
                        }


                         $(".tooltipDiv").remove();
                        this.displayShowTooltip(location, seriesPoint, series, data.region.Region.PointIndex);
                            
                        //Comment the below code for VML -The line series tooltip color not displayed due to attach and detach the element to DOM
                        // if (series.type.toLowerCase() == "bubble" || series.type.toLowerCase() == "scatter") {
                        //     var detachEle = $(this.svgObject).find("#" + this.svgObject.id + "_TrackToolTip").detach();
                        //    detachEle.appendTo(this.svgObject);
                        // }
                    } else {
                        $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden');
                        this.createTooltip(data.region, evt);
                    }
                }
            }
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = data;
            this._trigger("pointRegionMouseMove", commonEventArgs);
        }
		else if (!ej.util.isNullOrUndefined(this.model.prevPoint)&& this.model.AreaType == "polaraxes") {
            var prevPoint = this.model.prevPoint;
            series = this.model._visibleSeries[prevPoint.region.SeriesIndex];
            seriesPoint = this.model._visibleSeries[prevPoint.region.SeriesIndex].points[prevPoint.region.Region.PointIndex];
            pointlocation = ej.EjSvgRender.utils.TransformToVisible(series, seriesPoint.xValue, seriesPoint.y, this);
            this.drawTrackerSymbol(series, prevPoint.region.SeriesIndex, prevPoint.region.Region.PointIndex, null, pointlocation);
            this.model.prevPoint = null;
        }
        else {
            if (!ej.util.isNullOrUndefined(this.model.explodeValue)) {
                var type = this.model.series[0].type.toLowerCase();
                var series = this.model._visibleSeries[0];
                var seriesType = new ej.seriesTypes[type]();
                var targetElement;
                var id;
                var _labelPosition = series.labelPosition.toLowerCase();
                var result = seriesType._calculateArcData(this, this.model.explodeValue, series.points[this.model.explodeValue], series);
                if (this.model.enableCanvasRendering) {
                    this.model.series[0].explodeIndex = null;
                    var chartRect = document.getElementById(this.svgObject.id).getClientRects()[0];
                    this.svgRenderer.ctx.clearRect(chartRect.left, chartRect.top, chartRect.width, chartRect.height);
                    $("#" + this._id).ejChart("redraw");
                }
                //var explodeId = this.model.series[0].name + "group" + index;
                id = this.svgObject.id + '_SeriesGroup' + '_0';
                targetElement = $(this.gSeriesEle).children('#' + id)[0];
                var elements = $(targetElement).children();
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                    if (parseInt(index) == this.model.explodeValue) {
                        this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                        break;
                    }
                }
                
                
               seriesType.drawDataLabelAcc(this, series, this.model.explodeValue, series.points[this.model.explodeValue]);
               
                this.model.explodeValue = null;
            }
        }


    },

    chartCrossHair: function (chart, mouseLocation) {
	     var requireInvertedAxes = chart.model.requireInvertedAxes;
        if ((!this.model.enable3D) && this.model.crosshair.visible && this.model.crosshair.type.toLowerCase()=="trackball") {
            //Hiding previous trackball and tooltip when moving from one column to another
            if (this.model.enableCanvasRendering) {
                $("#" + this._id).find('[id*="canvas_ToolTipRect"]').css("visibility", "hidden");
                $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
            }
            else {                
                element = $(this.svgObject).find('[id*="_TrackToolTip"]');
                this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
                element = $(this.svgObject).find('[id*="_TrackToolTip"]').children();
                this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
                $(element).css({ 'visibility': 'hidden' });
                element = $(this.svgObject).find('[id*="_trackSymbol_"]');
                this.svgRenderer._setAttr($(element), { "visibility": 'hidden' });
            }
            var point;
            var seriesIndex;
            var series;
            var dir;
            var indicators = this.model.indicators;
            var seriesCollection = this.model._visibleSeries;
			 var showTrackLine = false;
            for (var j = 0; j < indicators.length && indicators[j].segment; j++) {
                seriesCollection = seriesCollection.concat(indicators[j].segment);
            }
            
            for (var i = 0; i < seriesCollection.length; i++) {
			 //   if(seriesCollection[0].xAxis.name == seriesCollection[i].xAxis.name)
                seriesIndex = i;
                series = seriesCollection[i];
                if (series.visibility.toLowerCase() == 'visible') {
                    var serX = [];
                    var insideBounds;
                    if (!requireInvertedAxes)
                        insideBounds = chart.mousemoveX > series.xAxis.x && chart.mousemoveX < series.xAxis.x + series.xAxis.width;                    
                    else
                        insideBounds = chart.mousemoveY > series.xAxis.y && chart.mousemoveY < series.xAxis.y + series.xAxis.height;
                    if(this.model.enableCanvasRendering)
                        this.canvasX = series.xAxis.x;
                    var closestPoint = chart.getClosestPointX(serX, series, chart.mousemoveX, chart.mousemoveY);
				    var visibleRange = series.xAxis.visibleRange;   // condition to check the closestPoint is inside the visible range
				    if (closestPoint.point != "" && (closestPoint.point[0].xValue >= visibleRange.min) && (closestPoint.point[0].xValue <= visibleRange.max)) {                        
                        if (insideBounds) {
                          for (var t = 0; t < closestPoint.point.length; t++) {
                              if (closestPoint.point[t].visible) {
                                  if (requireInvertedAxes) {
                                      point = ej.EjSvgRender.utils._getPoint(closestPoint.point[t], series);
                                      dir = "M" + " " + (series.yAxis.x) + " " + (point.Y + series.xAxis.y) + " " + "L" + " " + (series.yAxis.x + series.yAxis.width) + " " + (point.Y + series.xAxis.y);
                                  }
                                  else {
                                      point = ej.EjSvgRender.utils._getPoint(closestPoint.point[t], series);
                                      var dir = "M" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y) + " " + "L" + " " + (point.X + series.xAxis.x) + " " + (chart.model.m_AreaBounds.Y + chart.model.m_AreaBounds.Height);
                                  }
                                
                                  var chartPos = { left: chart.svgObject.offsetLeft, top: chart.svgObject.offsetTop };

                                var pathOptions = {
                                    'id': chart.svgObject.id + "_Tracker",
                                    'fill': 'none',
                                    'stroke-width': chart.model.crosshair.line.width,
                                    'clip-path': 'url(#' + chart.svgObject.id + '_ChartAreaClipRect)',
                                    'stroke': chart.model.crosshair.line.color,
                                    'd': dir
                                };
                                if ($("#" + chart.svgObject.id + "_Tracker").length == 0) {
                                    if (!chart.model.enableCanvasRendering) {
                                        chart.svgRenderer.drawPath(pathOptions, chart.svgObject);
                                        showTrackLine = true;
                                    }
                                    else {
                                        pathOptions.width = (requireInvertedAxes) ? this.model.m_AreaBounds.Width : 0,
                                        pathOptions.height = (requireInvertedAxes) ? 0 : this.model.m_AreaBounds.Height,
                                        pathOptions.top = this.model.m_AreaBounds.Y + chartPos.top,
                                        pathOptions.left = this.model.m_AreaBounds.X + chartPos.left,
                                        chart.svgRenderer.drawCrosshairLine(pathOptions, chart.svgObject);
                                    }
                                }
                                else {
								    showTrackLine = true;
                                    chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_Tracker"), { "d": dir });
                                    if (chart.model.enableCanvasRendering) {
                                        var leftPos;
										 var topPos;
										 if (requireInvertedAxes) {
                                              var chartAreaStartPos = chartPos.top + series.xAxis.y + $(document).scrollTop();
                                              var chartAreaEndPos = series.xAxis.height + chartPos.top + series.xAxis.y + $(document).scrollTop();
                                              var pointPos = point.Y + series.xAxis.y + chartPos.top + $(document).scrollTop();
                                              //Condition checked whether trackball line position is within the chartArea
                                              topPos = (pointPos < chartAreaEndPos && pointPos > chartAreaStartPos) ? pointPos : pointPos >= chartAreaEndPos ? chartAreaEndPos : chartAreaStartPos;//check trackball line within the chartArea
                                              +$("#" + chart.svgObject.id + "_Tracker").css("top", topPos);
                                         }
                                         else {
                                             var chartAreaStartPos = chartPos.left + series.xAxis.x;
                                             var chartAreaEndPos = series.xAxis.width + chartPos.left + series.xAxis.x;
                                             var pointPos = point.X + series.xAxis.x + chartPos.left;
                                             //Condition checked whether trackball line position is within the chartArea
                                             leftPos = (pointPos < chartAreaEndPos && pointPos > chartAreaStartPos) ? pointPos : pointPos > chartAreaEndPos ? chartAreaEndPos : chartAreaStartPos;//check trackball line within the chartArea
                                             $("#" + chart.svgObject.id + "_Tracker").css("left", leftPos);
										  }
                                    }
                                    $("#" + this._id).find('[id*="canvas_Tracker"]').css("visibility", "visible");
                                }

                                var crossHairTrans = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, requireInvertedAxes);
                                var tToolOptions = { 'transform': 'translate(' + crossHairTrans.x + ',' + crossHairTrans.y + ')', 'id': chart.svgObject.id + '_TrackToolTip' + '_' + seriesIndex };

                                chart.gTransToolEle = chart.svgRenderer.createGroup(tToolOptions);

                                var prePoint = this.model.prePoint;

                                //To check the current closestPoint and previous point is same
                                if (ej.util.isNullOrUndefined(prePoint) || prePoint.pointIndex[0] != closestPoint.index[0] || !$('#' + chart.svgObject.id + "_trackSymbol_" + seriesIndex + '_' + closestPoint.index)) {

                                    //Remove the track elements only once on point change
                                    if (ej.util.isNullOrUndefined(initialPoint)) {
                                        if (this.model.crosshair.tooltipTemplate != null) {
                                            $(document).find('[id*="_TrackToolTipTemplate_"]').attr("visibility", "hidden");
                                        }
                                        if (chart.gTransToolEle && chart.gTrackerEle) {
                                            $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]').attr("visibility", "hidden");
                                            $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
                                        }
                                        var initialPoint = closestPoint;
                                    }


                                    chart.drawTrackerSymbol(series, seriesIndex, closestPoint.index[t], true, point);
                                     
                                    // Condition to find the crosshair is specified with tooltip template
                                    if (!chart.model.crosshair.tooltipTemplate) {
                                        chart.displayTooltip(point, closestPoint.point[t], series, closestPoint.index[t]);
                                        $('#' + this.svgObject.id + '_TrackToolTip_' + seriesIndex).css("visibility", "visible");
                                        $('#' + this.svgObject.id + '_TrackToolTip_' + seriesIndex).attr("visibility", "visible");
                                    }
                                    else {
                                        var tX, tY;
                                        var chartLocation = document.getElementById(this.svgRenderer.svgObj.id).getClientRects()[0];
                                        chart.displayTemplateTooltip(tX, tY, point, closestPoint.point[t], series, seriesIndex, closestPoint.index[t], chartLocation);
                                    }
                                } else {
                                    // If both are same points, then viibility is set to visible
                                    var element = $(this.svgObject).find('[id*="_TrackSymbol_"],[id*="_TrackToolTip"]');
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "visible");
                                    element = $(this.svgObject).find('[id*="_TrackToolTip"]').children();
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    
                                    element = $(this.svgObject).find('[id*="_trackSymbol_"]');
                                    this.svgRenderer._setAttr($(element), { "visibility": 'visible' });

                                    $(chart.svgObject).find('[id*="_TrackAxisToolTip"]').attr("visibility", "visible");
                                    
                                    // Sets the tooltip template visibility to visible
                                    if (this.model.crosshair.tooltipTemplate != null) {
                                        element = $(document).find('[id*="_TrackToolTipTemplate_"]');
                                        this.svgRenderer._setAttr($(element), { "visibility": 'visible' });
                                    }
                                }
                            } else {
                                $('#' + this.svgObject.id + '_TrackToolTip_' + seriesIndex).remove();
                                $('#' + this.svgObject.id + '_TrackSymbol_' + seriesIndex).remove();
                            }
                        }
						}
						else{
						$("#" + chart.svgObject.id + "_TrackToolTip_" + i).remove();
						$("#" + chart.svgObject.id + "_TrackSymbol_" + i).remove();
						$("#" + chart.svgObject.id + "_TrackAxisToolTip").remove();
						}
                    } else {
                        if (!showTrackLine) // //condition checked to display Track line for single series when chart have more than one series. 
                            $(chart.svgObject).find('[id*="_Tracker"]').remove();
						$("#" + chart.svgObject.id + "_TrackToolTip_" + i).remove();
						$("#" + chart.svgObject.id + "_TrackSymbol_" + i).remove();
						$("#" + chart._id).find('[id*=' + "_canvas_ToolTipRect_" + i + ']').css("visibility", "hidden");
						$("#" + chart._id).find('[id*=' + "_canvas_trackSymbol_" + i + ']').css("visibility", "hidden");
                    }
                }
            }
           // To get the previous point values
            if (this.model.closestPoint != null)
            {
                this.model.prePoint = { point: closestPoint.point, pointIndex: closestPoint.index, series: series, seriesIndex: seriesIndex };
            }
             if ($("#" +this.svgObject.id + "_TrackAxisToolTip").length == 0) {     // condition to check track axis tooltip is existing in DOM
                var gTrackAxisOptions = { 'id': chart.svgObject.id + '_TrackAxisToolTip', 'visibility': 'visible' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gTrackAxisOptions);
                chart.svgRenderer.append(chart.gTrackAxisEle, chart.svgObject);
            }
           
             if (point) {
                 var pointValue = closestPoint.point[0];
                $.each(chart.model._axes, function (axisIndex, axis) {
                    $(chart.svgObject).find('[id*="_TrackAxisToolTip"]').attr("visibility", "visible");
                    var areaX = chart.model.m_AreaBounds.X;
                    if (!requireInvertedAxes) {
                        if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                            if ((point.X + areaX) <= (axis.x + axis.width) && axis.x <= (point.X + areaX)) {
                                var valueType = axis._valueType.toLowerCase();
                                var xVal = ((point.X / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                                if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal);
                                if (valueType == "datetime") xVal = (Globalize.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat)));
                                if (valueType == "category") xVal = ej.EjSvgRender.utils._getLabelContent(Math.floor(xVal), axis);
                                var pointLocation = { X: (point.X + areaX), Y: (axis.opposedPosition) ? (axis.y + axis.height - axis.majorTickLines.size) : (axis.y + axis.height + axis.majorTickLines.size + 10) };
                                chart.displayAxisTooltip(pointLocation, xVal, axis, axisIndex, mouseLocation);
                            }
                        }
                    }
                    else {
                        if (axis.orientation.toLowerCase() == "vertical" && axis.crosshairLabel.visible) {
                            if ((chart.mousemoveY) <= (axis.y + axis.height) && axis.y <= (chart.mousemoveY)) {
                                var valueType = axis._valueType.toLowerCase();
                                var yVal = pointValue.xValue;
                                if (valueType == "logarithmic") yVal = Math.pow(axis.logBase, yVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                                if (valueType == "datetime") yVal = (Globalize.format(new Date(Math.floor(yVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat)));
                                if (valueType == "category") yVal = ej.EjSvgRender.utils._getLabelContent(yVal, axis);
                                var ypointLocation = { X: axis.x, Y: (point.Y + axis.y) };
                                chart.displayAxisTooltip(ypointLocation, yVal, axis, axisIndex, mouseLocation);                                
                            }
                        }
                    }

                });
            }


        }
    },
    chartAreaCrossHair: function (chart, mouseLocation) {
        var round = ((ej.util.isNullOrUndefined(this.model.primaryYAxis.roundingPlaces)) ? 2 : this.model.primaryYAxis.roundingPlaces);
        if ((!this.model.enable3D) && this.model.crosshair.visible && this.model.crosshair.type.toLowerCase() == "crosshair") {
            var d = "M" + " " + (this.mousemoveX) + " " + (this.model.m_AreaBounds.Y) + " " + "L" + " " + (this.mousemoveX) + " " + (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height);
            var dhor = "M" + " " + (this.model.m_AreaBounds.X) + " " + (this.mousemoveY) + " " + "L" + " " + (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) + " " + (this.mousemoveY);
            
           
            var chartPos = { left: chart.svgObject.offsetLeft, top: chart.svgObject.offsetTop };
            if ($("#" + chart.svgObject.id + "_CrosshairVertical").length == 0) {
                var options = {
                    'id': chart.svgObject.id + "_CrosshairVertical",
                    'fill': 'none',
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': d
                };
                if (chart.model.enableCanvasRendering) {
                    options.width = 0,
                    options.height = this.model.m_AreaBounds.Height,
                    options.top = this.model.m_AreaBounds.Y + chartPos.top,
                    options.left = this.model.m_AreaBounds.X + chartPos.left,
                    chart.svgRenderer.drawCrosshairLine(options, chart.svgObject);
                }
                else
                    chart.svgRenderer.drawPath(options, chart.svgObject);                    
            } else {
                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairVertical"), { "d": d });
                if (chart.model.enableCanvasRendering)
                    $("#" + chart.svgObject.id + "_CrosshairVertical").css("left", this.mousemoveX + chartPos.left);
                $("#" + chart.svgObject.id + "_CrosshairVertical").css("visibility", "visible");
            }
            
            if ($("#" + chart.svgObject.id + "_CrosshairHorizontal").length == 0) {
                var optionsHor = {
                    'id': chart.svgObject.id + "_CrosshairHorizontal",
                    'fill': 'none',
                    'stroke-width': chart.model.crosshair.line.width,
                    'stroke': chart.model.crosshair.line.color,
                    'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)',
                    'd': dhor
                };
                if (chart.model.enableCanvasRendering) {
                    optionsHor.top = this.model.m_AreaBounds.Y + chartPos.top,
                    optionsHor.left = this.model.m_AreaBounds.X + chartPos.left,
                    optionsHor.width = this.model.m_AreaBounds.Width,
                    optionsHor.height = 0,
                    chart.svgRenderer.drawCrosshairLine(optionsHor, chart.svgObject);
                }
                else
                    chart.svgRenderer.drawPath(optionsHor, chart.svgObject);
            } else {
                chart.svgRenderer._setAttr($(chart.svgObject).find("#" + chart.svgObject.id + "_CrosshairHorizontal"), { "d": dhor });
                $("#" + chart.svgObject.id + "_CrosshairHorizontal").css("visibility", "visible");
                if (chart.model.enableCanvasRendering) {
                    $("#" + chart.svgObject.id + "_CrosshairHorizontal").css("top", this.mousemoveY + chartPos.top);
                    $("#" + chart.svgObject.id + "_CrosshairHorizontal").css("left", this.model.m_AreaBounds.X + chartPos.left);                    
                }
            }
            if ( $('#' + chart.svgObject.id + '_AxisCrossToolTip').length ==0) {
                var gAxisCrossOptions = { 'id': chart.svgObject.id + '_AxisCrossToolTip', 'visibility': 'visible' };
                chart.gTrackAxisEle = chart.svgRenderer.createGroup(gAxisCrossOptions);
                chart.svgRenderer.append(chart.gTrackAxisEle, chart.svgObject);
            }
            else {
                this.svgRenderer._setAttr($(chart.gTrackAxisEle), { "visibility": 'visible' });
            }

            $.each(chart.model._axes, function (axisIndex, axis) {
                var valueType = axis._valueType.toLowerCase();
                if (axis.orientation.toLowerCase() == "horizontal" && axis.crosshairLabel.visible) {
                    if ((chart.mousemoveX) <= (axis.x + axis.width) && axis.x <= (chart.mousemoveX)) {
                        var labelplacement = (axis._valueType == "category" && axis.labelPlacement.toLowerCase()=='betweenticks') ? 0.5 : 0;
                        var xVal = (((chart.mousemoveX - axis.x) / (axis.width)) * (axis.visibleRange.delta) + axis.visibleRange.min + labelplacement).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "logarithmic") xVal = Math.pow(axis.logBase, xVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "datetime") xVal = (Globalize.format(new Date(Math.floor(xVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat)));
                        if (valueType == "category") xVal = ej.EjSvgRender.utils._getLabelContent(xVal, axis);
                        var xPointLocation = { X: (chart.mousemoveX), Y: (axis.y) };
                        chart.displayAxisTooltip(xPointLocation, xVal, axis, axisIndex, mouseLocation);
						 $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).show();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).show();
                    }
					 else
					 {
                        $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).hide();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).hide();
                    }
                } else if (axis.orientation.toLowerCase() == "vertical" && axis.crosshairLabel.visible) {
                    if ((chart.mousemoveY) <= (axis.y + axis.height) && axis.y <= (chart.mousemoveY)) {
                        var yVal = ((Math.abs(1 - (chart.mousemoveY - (axis.y)) / (axis.height)) * (axis.visibleRange.delta) + axis.visibleRange.min).toFixed(round).replace(new RegExp("\\.0{" + round + "}"), ""));
                        if (valueType == "logarithmic") yVal = Math.pow(axis.logBase, yVal).toFixed(2).replace(new RegExp("\\.0{" + 2 + "}"), "");
                        if (valueType == "datetime") yVal = (Globalize.format(new Date(Math.floor(yVal)), ((ej.util.isNullOrUndefined(axis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(axis._intervalType) : axis.labelFormat)));
                        if (valueType == "category") yVal = ej.EjSvgRender.utils._getLabelContent(yVal, axis);
                        var ypointLocation = { X: axis.x, Y: (chart.mousemoveY) };
                        chart.displayAxisTooltip(ypointLocation, yVal, axis, axisIndex, mouseLocation);
						 $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).show();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).show();
                    }
					 else
				     {
                        $("#" + chart.svgObject.id + '_AxisToolTipText' + '_' + axisIndex).hide();
                        $("#" + chart.svgObject.id + '_AxisToolTipRect' + '_' + axisIndex).hide();
                    }
                }
            });

        }
    },
    chartTouchMove: function (evt) {
	   this.cancelEvent(evt);
        evt = evt.originalEvent.touches[0];
        this.chartInteractiveBehavior(evt);

    },

    isZoomButtonHovered: function (target) {
        if (target.parentNode != null) {
            var parentId = target.parentNode.id;
            var id = target.id;
            if (id.indexOf('_ZoomBtn') == -1 && id.indexOf('_ResetZoom') == -1 && id.indexOf('_PanBtn') == -1 && parentId.indexOf('_ZoomBtn') == -1 && parentId.indexOf('_ResetZoom') == -1 && parentId.indexOf('_PanBtn') == -1)
                return false;
        }
        return true;
    },

    chartInteractiveBehavior: function (evt) {
        var mouseMoveCords = this.calMousePosition(evt);
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;

        var id = "#" + this.svgObject.id;
        if (this.mouseWheelCoords) {
            var changeX = Math.abs(this.mouseWheelCoords.x - evt.pageX);
            var changeY = Math.abs(this.mouseWheelCoords.y - evt.pageY);
            if ((changeX > 0 || changeY > 0) && !(this.panning)) {
                this.enableTrackBall();
            } else {
                this.disableTrackBall();
            }
        }

        var chart = this;
        if (evt.target.parentNode != null) {
            var parentNodeId = evt.target.parentNode.id;
            var targetId = evt.target.id;
            var isElement = (parentNodeId.indexOf(this.svgObject.id + "_SeriesGroup_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_symbolGroup_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_TrackSymbol_") >= 0 || parentNodeId.indexOf(this.svgObject.id + "_Chart3D") >= 0) ? true : false;
        }
        if (chart.model.AreaType == "cartesianaxes" && this.mousemoveX >= this.model.m_AreaBounds.X && this.mousemoveX < (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width) && this.mousemoveY < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height) && this.mousemoveY >= this.model.m_AreaBounds.Y && this.model.initSeriesRender) {
            var mouseLocation = { x: chart.mousemoveX, y: chart.mousemoveY };
            var targetId = chart.svgRenderer._getAttrVal($(evt.target).parent(), "id");
            if ((targetId != undefined) && (targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ZoomBtn'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_ResetZoom'), "id") || targetId == chart.svgRenderer._getAttrVal($(this.svgObject).find(id + '_PanBtn'), "id"))) {
                this.disableTrackBall();
            }
            else {
                this.enableTrackBall();
            }

            //show tooltip and track ball
            if (!this.model.crosshair.visible && !this.isZoomButtonHovered(evt.target))
                this.tooltip(chart, evt);
            else {
                //Removing marker symbol when placing mouse outside chart area
                var pointData = this.model.prevPoint;
                if (pointData) {
                    if (this.model.enableCanvasRendering)
                        $("#canvas_trackSymbol").remove();
                    else {
                        var pointLocation = ej.EjSvgRender.utils._getPoint(pointData.point, pointData.series);
                        chart.drawTrackerSymbol(pointData.series, pointData.seriesIndex, pointData.pointIndex, null, pointLocation);
                    }
                    this.model.prevPoint = null;
                }

                $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
                $("#" + chart.svgObject.id + "_TrackToolTip").hide();
                $(".tooltipDiv").css("display", "none");
            }

            if (!this.model.enable3D) {
			    //crossHair of chart
			    this.chartCrossHair(chart, mouseLocation);

			    //crossHair for chart area
			    this.chartAreaCrossHair(chart, mouseLocation);
			}

        } else {
            if ((chart.model.AreaType == "none" || chart.model.AreaType == 'polaraxes') && !this.isZoomButtonHovered(evt.target))
                this.tooltip(chart, evt);
            else {
                $("#" + chart.svgObject.id).find("#" + chart.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
                $("#" + chart.svgObject.id + "_TrackToolTip").hide();
                $(".tooltipDiv").css("display","none");
                this._removeTrackBall();
            }
        }
        // Checked condition for series highlighting
        if (!this.model.crosshair.visible) {
            if (this.model.enableCanvasRendering)
                this.highlightCanvas(this, evt);
            else
                this.highlightSvg(chart, parentNodeId,isElement, evt)
        }
    },

    // highlight SVG logic started here
    highlightSvg: function (chart, parentNodeId, isElement, evt) {
        if (parentNodeId != undefined) {
            var matchStr = this.svgObject.id + "_SeriesGroup_";
            var serIndex = parentNodeId.substr(matchStr.length);
            if (parentNodeId.indexOf(this.svgObject.id + "_symbolGroup_") >= 0) {
                matchStr = this.svgObject.id + "_symbolGroup_";
                serIndex = parentNodeId.substr(matchStr.length);
            }
        }
        var data = this.GetSeriesPoint(evt);
        serIndex = data != undefined ? data.region.SeriesIndex : serIndex
        var series = this.model.series[serIndex];
        if (isElement && series) {
            if (series.highlightSettings.enable)
                this.highlight(chart, evt, series);
        }

    },

     // canvas highlight logic started here
    highlightCanvas: function (chart, evt) {
        for (var i = 0; i < this.model._visibleSeries.length; i++) {
            var seriesIndex;
            var chartSeries;
            seriesIndex = i;
            chartSeries = this.model._visibleSeries[i];
            var type = chartSeries.type.toLowerCase();
            if (this.model.AreaType == "cartesianaxes" && chartSeries.visibility.toLowerCase() == 'visible' && type !== "scatter" && type !== "bubble" && type !== "column" && type.indexOf("bar") == -1 && type !== "stackingcolumn" && type !== "stackingcolumn100" && !chartSeries._hiloTypes) {
                var serY = [];
                var serX = [];
                var location = null;
                var mouseMoveCords = this.calMousePosition(evt);
                this.mousemoveX = mouseMoveCords.X;
                this.mousemoveY = mouseMoveCords.Y;
                var closestXyPoint = this.getClosesPointXY(serX, serY, chartSeries, this.mousemoveX, this.mousemoveY);
                if (!ej.util.isNullOrUndefined(closestXyPoint.point)) {
                    var commonPointEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
                    commonPointEventArgs.data = { location: { x: this.mousemoveX, y: this.mousemoveY }, region: { SeriesIndex: i, Region: { PointIndex: closestXyPoint.index } } };
                }
            }
        }
        var data = commonPointEventArgs ? commonPointEventArgs.data : this.GetSeriesPoint(evt);
        if (data && this.model.series[data.region.SeriesIndex].highlightSettings.enable) {
            var series = $.extend({}, this.model.series[data.region.SeriesIndex]);
            series.pointIndex = data.region.Region.PointIndex;
            series.seriesIndex = data.region.SeriesIndex;
            series.data = data;
            if (this.model.AreaType != 'none' || ($('#' + this._id + '_Selection_series' + series.pointIndex + '_canvas').length == 0))
                this.canvasHighlight(this, evt, series);
        }
    },
    chartMouseMove: function (evt) {
        // removed highlight style
        $("[class*=" + "HighlightStyle" + "]").attr('class', '');
        $("[class*=" + "HighlightLegendStyle" + "]").attr('class', '');
	     $("[class*=" + "HighlightLegendPathStyle" + "]").attr('class', '');
        $("[class*=" + "HighlightPathStyle" + "]").attr('class', '');
	     $("[class*=" + "Highlightseries" + "]").attr('class', '');
        $("[id*=" + this._id + "_Highlight_" + "]").remove();// removed canvas highlight
        // removed highlight style
	   this.cancelEvent(evt);
	   
	     if (this.rotateActivate && this.oPreviousCoords) {
            var difY = (this.oPreviousCoords.Y - this.mousemoveY);
            var difX = (this.oPreviousCoords.X - this.mousemoveX);          
            if (difX||difY) {
                this.model.tilt -= difY;
                this.model.rotation += difX;     
                // store the class name 
                var selection = this.selectedStyle(this);
                $(this.chart3D).empty();              
                var size = { Width: $(this.svgObject).width(), Height: $(this.svgObject).height() };
                this.graphics.view(this.svgObject, this, this.model.rotation, this.model.tilt, size, this.model.perspectiveAngle, this.model.depth);
                this.svgRenderer.append(this.chart3D, this.svgObject);
                this.svgRenderer.append(this.gTransToolEle, this.svgObject);
                // Apply the class
                for (var i = 0; i < selection.length; i++) {
                    $('#' + selection[i].id).attr('class', selection[i].className);
                }
            }
            this.oPreviousCoords.Y = this.mousemoveY;
            this.oPreviousCoords.X = this.mousemoveX;
        }
		 
        var chart = this;
        this.calMousePosition(evt)
		$("#" + this._id + "_tooltip").remove();
        if (!this.panning) {
            if (!this.drag)
                this.chartInteractiveBehavior(evt);
            var id = "#" + this.svgObject.id;
            if (this.model.zooming.enable && this.model.AreaType == "cartesianaxes" && !this.model.enable3D) {
                if (this.drag) {
                    this.disableTrackBall();
                    $('#' + this.svgObject.id + '_ZoomArea').remove();
                    var width, height, x, y;
                    var currentX = this.mousemoveX;
                    var currentY = this.mousemoveY;
                    var areaBounds = this.model.m_AreaBounds;
                    if (this.mousemoveX < areaBounds.X) {
                        currentX = areaBounds.X;
                    } else if (this.mousemoveX > (areaBounds.X + areaBounds.Width)) {
                        currentX = areaBounds.X + areaBounds.Width;
                    } 
                    if (this.mousemoveY < areaBounds.Y) {                     
                        currentY = areaBounds.Y;
                    } else if (this.mousemoveY > (areaBounds.Y + areaBounds.Height)) {
                        currentY = areaBounds.Y + areaBounds.Height;
                    }
                    else {
                        currentY = this.mousemoveY;
                    }
                    if (this.model.zooming.type.toLowerCase() == "x") {                  
                        width = Math.abs(currentX - this.mouseDownX);
                        height = areaBounds.Height;
                        x = currentX > this.mouseDownX ? this.mouseDownX : currentX;
                        y = areaBounds.Y;
                    } else if (this.model.zooming.type.toLowerCase() == "y") {
                        width = areaBounds.Width;                      
                        height = Math.abs(currentY - this.mouseDownY);
                        x = areaBounds.X;
                        y = currentY > this.mouseDownY ? this.mouseDownY : currentY;

                    } else {                        
                        width = Math.abs(currentX - this.mouseDownX);
                        height = Math.abs(currentY - this.mouseDownY);
                        x = currentX > this.mouseDownX ? this.mouseDownX : currentX;
                        y = currentY > this.mouseDownY ? this.mouseDownY : currentY;

                    }                 
                    $(this.svgObject).css({
                        '-moz-user-select': '-moz-none',
                        '-khtml-user-select': 'none',
                        '-webkit-user-select': 'none',
                        '-ms-user-select': 'none',
                        'user-select': 'none'
                    });

                    this.zooming = true;

                    var rectOptions = {
                        'id': this.svgObject.id + '_ZoomArea',
                        'x': x,
                        'y': y,
                        'width': width,
                        'height': height,
                        'fill': 'rgba(69,114,167,0.25)',
                        'stroke-width': 1,
                        'stroke': 'rgba(69,114,167,0.25)',
                        'clip-path': 'url(#' + this.svgObject.id + '_ChartAreaClipRect)'
                    };
                    if (this.model.enableCanvasRendering)
                        this.svgRenderer.drawZoomRect(rectOptions, this);
                    else
                        this.svgRenderer.drawRect(rectOptions, this.svgObject);
                }

            }
        }
		var deferredZoom = this.model.zooming.enableDeferredZoom;
        // Panning
        if (this.panning && this.doPan && this.model.AreaType == "cartesianaxes") {
            this.currentPageX = evt.pageX;
            this.currentPageY = evt.pageY;
            $("#" + this._id + "_canvas").css({ "cursor": "pointer" });
            if (!ej.isTouchDevice() && !deferredZoom) {
                var oDelta;
                oDelta = {
                    'x': this.oPreviousCoords.x - evt.pageX,
                    'y': this.oPreviousCoords.y - evt.pageY
                };

                this.oPreviousCoords = {
                    'x': evt.pageX,
                    'y': evt.pageY
                };
                $.each(this.model._axes, function(index, axis) {
                    var currentScale = Math.max(1 / ej.EjSvgRender.utils._minMax(axis.zoomFactor, 0, 1), 1);
                    chart.translate(axis, (oDelta.x), (oDelta.y), currentScale);
                });

                this.redraw(true);
                this._cursorToPointer();
                this._enableZoomingButtons();
            }
            // Translate the series for other devices
            else if (ej.isTouchDevice() || deferredZoom) {
                for (var k = 0; k < this.model._visibleSeries.length; k++) {
                    var series = this.model.series[k];
                    var transform = ej.EjSvgRender.utils._getTransform(series.xAxis, series.yAxis, this.model.requireInvertedAxes);
                    var transX = (evt.pageX - this.oPreviousCoords.x) + transform.x;
                    var transY = (evt.pageY - this.oPreviousCoords.y) + transform.y;
                    var translate = 'translate(' + transX + ',' + transY + ')';
                    var svgObjectId = this.svgObject.id;
                    $("#" + svgObjectId + "_SeriesGroup_" + k).attr("transform", translate);
                    $("#" + svgObjectId + "_symbolGroup_" + k).attr("transform", translate);
                    $("#" + svgObjectId + "_TextGroup_" + k).attr("transform", translate);
                    
                }
                for (var index = 0; index < this.model.indicators.length && this.model.indicators[index]._points.length>0; index++) {
                    var indicator = this.model.indicators[index];
                    transform = ej.EjSvgRender.utils._getTransform(indicator.xAxis, indicator.yAxis, this.model.requireInvertedAxes);
                    transX = (evt.pageX - this.oPreviousCoords.x) + transform.x;
                    transY = (evt.pageY - this.oPreviousCoords.y) + transform.y;
                    translate = 'translate(' + transX + ',' + transY + ')';
                    $("#" + this.svgObject.id + "_indicatorGroup_" + index).attr("transform", translate);
                }
            }
        }
		
		//Declaration
            var targetid = evt.target.id;
            var id = this._id;
            var parentId = ej.util.isNullOrUndefined(evt.target.parentNode) ? "" : evt.target.parentNode.id;
			
	//condition to find the buttons and display tooltip on hover
            if (parentId == id + '_svg_ResetZoom')
                createBtnTooltip("Reset");
            else if (parentId == id + '_svg_PanBtn')
                createBtnTooltip("Pan");
            else if (parentId == id + '_svg_ZoomBtn')
                createBtnTooltip("Zoom");

	// method to create tooltip for zooming and panning buttons
        function createBtnTooltip(text) {
            var id = chart._id;
            if ($("#" + id + "_tooltip").length <= 0) {
                var tooltipdiv = $("<div></div>").attr('id', id + "_tooltip");
                $(tooltipdiv).html("&nbsp" + text + "&nbsp");
                $(document.body).append(tooltipdiv);
                // adding css prop to the div
                $(tooltipdiv).css({
                    "left": evt.pageX + 10,
                    "top": evt.pageY + 10,
                    "display": "block",
                    "position": "absolute",
                    "z-index": "13000",
                    "cursor": "default",
                    "font-family": "Segoe UI",
                    "color": "#707070",
                    "font-size": "12px",
                    "background-color": "#FFFFFF",
                    "border": "1px solid #707070"
                });
            } else {
                $("#" + id + "_tooltip").css({
                    "left": evt.pageX + 10,
                    "top": evt.pageY + 10
                });
            }

        }
           var svgId = this.svgObject.id;
           var hAxes = this.model.hAxes;
           var vAxes = this.model.vAxes;
  
            if (targetid.indexOf("_YLabel_") >= 0) {
                var axis;
                // Loop to find the axis
                for (var j = 0; j < vAxes.length; j++) {
                    if (targetid.indexOf(svgId + '_' + vAxes[j].name + '_YLabel_') >= 0) {
                        axis = vAxes[j];
                        break;
                    }
                }
                if (axis && axis.enableTrim) {
                    // Loop to find the labels in the axis
                    visibleLabelsLength = axis.visibleLabels.length;
                    for (var i = 0; i < visibleLabelsLength && this.model.AreaType == 'cartesianaxes'; i++) {
                        if (targetid == svgId  + '_' + vAxes[j].name + '_YLabel_' + i) {
                            var ele = axis.visibleLabels[i];                             
                            var str = ele.Text == ele.displayText ? "" : ((axis.labelFormat) ? axis.labelFormat.replace("{value}", ele.Text) : ele.Text);
                            if (str == "") break;
                            if (axis.labelPosition.toLowerCase() == 'inside') {
                                this._hideTooltip();
                                this._removeTrackBall();
                            }
                            this.showAxisTooltip(evt.pageX, evt.pageY, str);
                        }
                    }
                }
            }
    
            if (targetid.indexOf("_XLabel_") >= 0) {
                var axis;
                // Loop to find the axis
                for (var j = 0; j < hAxes.length; j++) {
                    if (targetid.indexOf(svgId + '_' + hAxes[j].name + '_XLabel_') >= 0) {
                        axis = hAxes[j];
                        break;
                    }
                }
                if (axis && (axis.enableTrim || axis.labelIntersectAction.toLowerCase() == "trim")) {
                    // Loop to find the labels in the axis
                    visibleLabelsLength = axis.visibleLabels.length;
                    for (var i = 0; i < visibleLabelsLength && this.model.AreaType == 'cartesianaxes'; i++) {
                        if (targetid == svgId + '_' + hAxes[j].name + '_XLabel_' + i) {
                            var ele = axis.visibleLabels[i];
                            var str = ele.Text == ele.displayText ? "" : ele.Text;
                            if (str == "") break;
                            if (axis.labelPosition.toLowerCase() == 'inside') {
                                this._hideTooltip();
                                this._removeTrackBall();
                            }
                            this.showAxisTooltip(evt.pageX, evt.pageY, str);
                }
            }
                }
            }
            for (i = 0; i < this.model.series.length; i++) {
                if (this.model.series[i]._enableSmartLabels && !this.model.enable3D) {
                    var template = this.model.series[i].marker.dataLabel.template;
                    var font = this.model.series[i].marker.dataLabel.font;
                    if (this.model.AreaType == "none" && ej.util.isNullOrUndefined(template)) {
                        this.datalabelTooltip(evt, i, font);
                    }                    
                }
            }           
        if ($(this.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').length > 0) {
            $(this.svgObject).find(id + '_ResetZoom,' + id + '_PanBtn,' + id + '_ZoomBtn').appendTo(this.svgObject);
        }

        var legenddata = this.getLegendData(evt);
        if (legenddata) {
            var commonLegendMoveEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLegendMoveEventArgs.data = legenddata;
            if (legenddata.series.highlightSettings.enable) {
                if (this.model.enableCanvasRendering) {
                    var series = $.extend({}, legenddata.series);
                    series.seriesIndex = legenddata.legendItem.LegendItem.SeriesIndex;
                    series.pointIndex = legenddata.legendItem.LegendItem.PointIndex;
                    var index = this.model.AreaType == 'none' ? series.pointIndex : series.seriesIndex;
                    if ($('#' + chart._id + '_Selection_Legend' + index + '_canvas').length == 0)
                        this.canvasHighlight(chart, evt, series, legenddata);
                }
                else
                    this.highlight(chart, evt, legenddata.series, legenddata);
            }
            this._trigger("legendItemMouseMove", commonLegendMoveEventArgs);
            if (this.model.enableCanvasRendering)
                $(this.legendContainer).children().css({ "cursor": "pointer" });
        }
        else if (this.model.enableCanvasRendering)
            $(this.legendContainer).children().css('cursor', 'default');
        if (this.model.enableCanvasRendering) {

            //condition to find the buttons and display tooltip on hover for Canvas
            if (targetid.indexOf("_ResetZoom") != -1 || parentId.indexOf("_ResetZoom") != -1)
                createBtnTooltip("Reset");
            else if (targetid.indexOf("_PanBtn") != -1 || parentId.indexOf("_PanBtn") != -1)
                createBtnTooltip("Pan");
            else if (targetid.indexOf("_ZoomBtn") != -1 || parentId.indexOf("_ZoomBtn") != -1)
                createBtnTooltip("Zoom");
            this.canvasAxisLabelTooltip(evt);
        }
		var proxy = this;
        if (this.model.xAxisTitleRegion) {
            var currentX = this.mousemoveX;
            var currentY = this.mousemoveY;
            $.each(this.model.xAxisTitleRegion, function (index, regionItem) {
                if ((currentX >= regionItem.Bounds.X) && (currentX <= regionItem.Bounds.X + regionItem.Bounds.Width)) {
                    if ((currentY >= regionItem.Bounds.Y - (regionItem.Bounds.Height)) && (currentY <= regionItem.Bounds.Y)) {
					    if (regionItem.trimText != regionItem.labelText) {
                        proxy.showAxisTooltip(evt.pageX, evt.pageY, regionItem.labelText);
					   }
                    }
                }
            });
        }
        if (this.model.yAxisTitleRegion) {
            var currentX = this.mousemoveX;
            var currentY = this.mousemoveY;
            $.each(this.model.yAxisTitleRegion, function (index, regionItem) {
                 if ((currentX >= regionItem.Bounds.X) && (currentX <= regionItem.Bounds.X + regionItem.Bounds.Width)) {
                    if ((currentY >= regionItem.Bounds.Y - (regionItem.Bounds.Height)) && (currentY <= regionItem.Bounds.Y)) {
					    if (regionItem.trimText != regionItem.labelText) {
                        proxy.showAxisTooltip(evt.pageX, evt.pageY, regionItem.labelText);
						}
                    }
                }
            });
        }
    },
    _hideTooltip: function () {
        $("#" + this.svgObject.id).find("#" + this.svgObject.id + "_TrackToolTip").attr('visibility','hidden');
        $("#" + this.svgObject.id + "_TrackToolTip").attr('visibility', 'hidden').hide();
        $('.tooltipDiv').css("display", "none");
    },
    _removeHighlight: function () {
        $("[id*=" + "HighlightSegment" + "]").remove();
        $("[id*=" + "HighlightPath" + "]").remove();
        $("[id*=" + "HighlightLegendSegment" + "]").remove();
        $("[id*=" + "HighlightLegendPath" + "]").remove();
        $("[id*=" + "HighlightDef" + "]").remove();
	     $("[id*=" + "Highlightseries" + "]").remove();
        $("[id*=" + this._id + "_Highlight_" + "]").remove();
    },
    _removeSelection: function () {
        $("[id*=" + "SelectionSegment" + "]").remove();
        $("[id*=" + "SelectionPath" + "]").remove();
        $("[id*=" + "SelectionLegendSegment" + "]").remove();
        $("[id*=" + "SelectionLegendPath" + "]").remove();
        $("[id*=" + "SelectionDef" + "]").remove();
	     $("[id*=" + "Selectionseries" + "]").remove();
        $('[id*=' + this._id + '_Selection_' + ']').remove();
    },

     
     //tooltip for trimmed datalabel
    datalabelTooltip:function(evt, i, font){
        var mouseMoveCords = this.calMousePosition(evt);
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;
		var containerId = this._id;
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var vmlRendering = this.vmlRendering;
        var space = this.model.elementSpacing;
        var chart = this;
        var minX, minY, maxX, maxY;
        var tooltipdiv, tooltipLength;
        var remaining
        $.each(this.accDataLabelRegion, function (index, regionItem) {
            minX = regionItem.bounds.minX;
            minY = regionItem.bounds.minY;
            maxX = regionItem.bounds.maxX;
            maxY = regionItem.bounds.maxY;            
            if (vmlRendering) {
                minX = regionItem.bounds.minX + (regionItem.bounds.width / 2);
                maxX = regionItem.bounds.maxX + (regionItem.bounds.width / 2);
                minY = regionItem.bounds.minY;
                maxY = regionItem.bounds.maxY;
            }
            if (currentX >= minX && currentX <= maxX) {
                if (currentY >= minY && currentY <= maxY) {
                    tooltipdiv = $("<div></div>").attr('id', containerId + "_tooltip");
                    if (regionItem.trimmedText != regionItem.text) {
                        $("#container_svg_TrackToolTip").hide();
                        $("#container_canvas_TrackToolTip").hide();
                        $(tooltipdiv).html(regionItem.text);
                        $(document.body).append(tooltipdiv);
                        tooltipLength = ej.EjSvgRender.utils._measureText(regionItem.text, regionItem.text.length, regionItem.font).width;
                        $(tooltipdiv).css({
                            "top": evt.pageY + space,
                            "display": "block",
                            "position": "absolute",
                            "z-index": "13000",
                            "cursor": "default",
                            "color": font.color,
                            "font-size": font.size,
                            "background-color": "#FFFFFF",
                            "border": "1px solid #707070",
                            "white-space": "nowrap",
                            "padding": "5px"
                        });
                        if (regionItem.type == "pyramid" || regionItem.type == "funnel") {
                            remaining = chart.model.m_AreaBounds.Width + chart.model.m_AreaBounds.X - evt.pageX;
                            if (remaining > tooltipLength)
                                $(tooltipdiv).css({ "left": evt.pageX + space });
                            else {
                                left = evt.pageX - tooltipLength+(2*space);
                                $(tooltipdiv).css({ "left": left  });

                            }
                        }
                        if (regionItem.type == "pie" || regionItem.type == "doughnut") {
                            if (minX > regionItem.bounds.centerX) {
                                remaining = chart.svgWidth - chart.model.margin.left - chart.model.margin.right - currentX;
                                if (remaining < tooltipLength) {
                                    left = evt.pageX - tooltipLength;
                                    $(tooltipdiv).css({ "left": left + space });
                                }
                                else
                                    $(tooltipdiv).css({ "left": evt.pageX + space });
                            }
                            else {
                                remaining = currentX;
                                if (remaining < tooltipLength) {
                                    $(tooltipdiv).css({ "left": evt.pageX + space });
                                }
                                else {
                                    left = currentX - tooltipLength;
                                    $(tooltipdiv).css({ "left": left + space });
                                }
                            }
                        }                       

                    }
                }
            }
        });
    },
    // Tooltip in svg 
      showAxisTooltip: function (pageX, pageY, str) {
	        var id = this._id;
            var tooltipdiv = $("<div></div>").attr('id', id + "_tooltip");
            $(tooltipdiv).html("&nbsp" + str + "&nbsp");
            $(document.body).append(tooltipdiv);
            // adding css prop to the div
            $(tooltipdiv).css({
                "left": pageX + 10,
                "top": pageY + 10,
                "display": "block",
                "position": "absolute",
                "z-index": "13000",
                "cursor": "default",
                "font-family": "Segoe UI",
                "color": "#707070",
                "font-size": "12px",
                "background-color": "#FFFFFF",
                "border": "1px solid #707070"
            });
        },
    canvasAxisLabelTooltip: function(evt){
        var mouseMoveCords = this.calMousePosition(evt);
        this.mousemoveX = mouseMoveCords.X;
        this.mousemoveY = mouseMoveCords.Y;
        var currentX = this.mousemoveX;
        var currentY = this.mousemoveY;
        var containerId = this._id;
        if (this.model.xAxisLabelRegions) {
            $.each(this.model.xAxisLabelRegions, function (index, regionItem) {
                var axisLabelRegions = regionItem;

                if ((currentX >= regionItem.bounds.x) && (currentX <= regionItem.bounds.x + regionItem.bounds.width)) {
                    if ((currentY >= regionItem.bounds.y - (regionItem.bounds.height / 3)) && (currentY <= regionItem.bounds.y)) {

                        var tooltipdiv = $("<div></div>").attr('id', containerId + "_tooltip");
                        if (regionItem.trimText != regionItem.labelText) {
                            $(tooltipdiv).html("&nbsp" + regionItem.labelText + "&nbsp");
                            $(document.body).append(tooltipdiv);
                            // adding css prop to the div
                            $(tooltipdiv).css({
                                "left": evt.pageX + 10,
                                "top": evt.pageY + 10,
                                "display": "block",
                                "position": "absolute",
                                "z-index": "13000",
                                "cursor": "default",
                                "font-family": "Segoe UI",
                                "color": "#707070",
                                "font-size": "12px",
                                "background-color": "#FFFFFF",
                                "border": "1px solid #707070"
                            });
                        }
                    }
                }
            });
        }
 // tooltip in canvas for y axis labels
 if (this.model.yAxisLabelRegions) {
                    $.each(this.model.yAxisLabelRegions, function (index, regionItem) {
                        var yaxisLabelRegions = regionItem;



                if ((currentX >= regionItem.bounds.x) && (currentX <= regionItem.bounds.x + regionItem.bounds.width)) {
                    if ((currentY >= regionItem.bounds.y - (regionItem.bounds.height / 3)) && (currentY <= regionItem.bounds.y)) {

                        var tooltipdiv = $("<div></div>").attr('id', containerId + "_tooltip");
                        if (regionItem.trimText != regionItem.labelText) {
                            $(tooltipdiv).html("&nbsp" + regionItem.labelText + "&nbsp");
                            $(document.body).append(tooltipdiv);
                                    // adding css prop to the div
                                    $(tooltipdiv).css({
                                        "left": evt.pageX + 10,
                                        "top": evt.pageY + 10,
                                        "display": "block",
                                        "position": "absolute",
                                        "z-index": "13000",
                                        "cursor": "default",
                                        "font-family": "Segoe UI",
                                        "color": "#707070",
                                        "font-size": "12px",
                                        "background-color": "#FFFFFF",
                                        "border": "1px solid #707070"
                                    });
                                }
                            }
                        }
                    });
                }
    },
    pieExplosion: function (data) {
        var region = data;
        var id;
        var targetElement;
        var symbolName;
        var series = region.region.Series;
        var seriesType = new ej.seriesTypes[series.type.toLowerCase()]();
        var explodePoint = series.explodeIndex;
        var _labelPosition = series.labelPosition.toLowerCase();
        if (explodePoint != region.pointData[0].PointIndex && !region.region.Series.explodeAll) {
            series.explodeIndex = region.pointData[0].PointIndex;

            if (!ej.util.isNullOrUndefined(this.model.explodeValue)) {
                var result = seriesType._calculateArcData(this, this.model.explodeValue, series.points[this.model.explodeValue], region.region.Series);
                id = this.svgObject.id + '_SeriesGroup' + '_' + region.region.SeriesIndex;
                
                targetElement = $(this.gSeriesEle).children('#' + id)[0];
                var elements = $(targetElement).children();
                for (var i = 0; i < elements.length; i++) {
                    var element = elements[i];
                    var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                    if (parseInt(index) == this.model.explodeValue) {
                        this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                        break;
                    }
                }


                seriesType.drawDataLabelAcc(this, series, this.model.explodeValue, series.points[this.model.explodeValue])
                
            }

            result = seriesType._calculateArcData(this, region.pointData[0].PointIndex, this.model.chartRegions[0].Series.points[region.pointData[0].PointIndex], region.region.Series);
            id = this.svgObject.id + '_SeriesGroup' + '_' + region.region.SeriesIndex;
            targetElement = $(this.gSeriesEle).children('#' + id)[0];
            var elements = $(targetElement).children();
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                var index = this.svgRenderer._getAttrVal($(element)[0], "pointIndex");
                if (parseInt(index) == region.pointData[0].PointIndex) {
                    this.svgRenderer._setAttr($(element), { 'd': result.Direction });
                    break;
                }
            }
            if (this.model.enableCanvasRendering) {
                var chartRect = document.getElementById(this.svgObject.id).getClientRects()[0];
                this.svgRenderer.ctx.clearRect(chartRect.left, chartRect.top, chartRect.width, chartRect.height);
                $("#" + this._id).ejChart("redraw");
            }
            
             seriesType.drawDataLabelAcc(this, series, region.pointData[0].PointIndex, series.points[region.pointData[0].PointIndex])
          
            this.model.explodeValue = region.pointData[0].PointIndex;

        }
        region.region.Series.explodeIndex = explodePoint;
    },
    displayTemplateTooltip: function (x, y, location, seriesPoint, series, serIndex, ptIndex, chartLoc) {
        var point = $.extend(true, {}, seriesPoint);
        var padding = 5;
        point.x = point.X;
        if (series.xAxis._valueType.toLowerCase() == "datetime")
            point.x = (Globalize.format(new Date(point.X), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis.intervalType) : series.xAxis.labelFormat)));
        if (series.xAxis._valueType.toLowerCase() == "category") point.x = ej.EjSvgRender.utils._getLabelContent(ptIndex, series.xAxis);
        // x and y is set to 0, if undefined
        x = ej.util.isNullOrUndefined(x) ? 0 : x;
        y = ej.util.isNullOrUndefined(y) ? 0 : y;
        var templateLocX = this.model.m_AreaBounds.X + chartLoc.left;
        var xLoc = location.X + x + templateLocX + this.model.crosshair.marker.size.width + padding;
        var yLoc = location.Y + y + this.model.m_AreaBounds.Y + chartLoc.top;
        var tooltipdiv;
        if ($("#" + this.svgObject.id + "_TrackToolTipTemplate_" + serIndex).length == 0) {
            tooltipdiv = $("<div class='TrackToolTipTemplate' style='position: absolute; z-index: 13000; display: block;'></div>");            
            $(document.body).append(tooltipdiv);
        }
        else {
            $(document).find('[id*="_TrackToolTipTemplate_"]').css("display", "block");
            tooltipdiv = $("#" + this.svgObject.id + "_TrackToolTipTemplate_" + serIndex)
            tooltipdiv.empty();
        }
        
        this.svgRenderer._setAttr($(tooltipdiv), { "id": this.svgObject.id + "_TrackToolTipTemplate_" + serIndex });
        if (this.model.crosshair.tooltipTemplate != null) {            
            var cloneNode = $("#" + this.model.crosshair.tooltipTemplate).clone();
            $(cloneNode).css("display", "block").appendTo(tooltipdiv);
            series.count = 1;
            point.count = 1;
            var seriesColor = this.getSeriesColor(point, serIndex, series);
            $(tooltipdiv).css("background-color", jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor);
            var data = { series: series, point: point };
            $(tooltipdiv).html($(tooltipdiv).html().parseTemplate(data));
        }
        yLoc -= parseFloat($(tooltipdiv).css('height')) / 2;
        var templateWidth = parseFloat($(tooltipdiv).css('width'));
        if (xLoc + templateWidth > templateLocX + this.model.m_AreaBounds.Width)
            xLoc = location.X + x + templateLocX - this.model.crosshair.marker.size.width - padding - templateWidth;
        $(tooltipdiv).css("left", xLoc);
        $(tooltipdiv).css("top", yLoc);
    },

    displayAxisTooltip: function (location, text, axis, index, mouseLoc) {

        if (axis._valueType.toLowerCase() == "double") {
            var customFormat = (!(axis.labelFormat)) ? null : axis.labelFormat.match('{value}');
            text = (!(axis.labelFormat)) ? text : (customFormat != null) ? (axis.labelFormat == "${value}") ? axis.labelFormat.replace('{value}', '$' + Number(text)) : axis.labelFormat.replace('{value}', Number(text)) : (Globalize.format(Number(text), axis.labelFormat));
        }
        var maxTickSize = 0;
        for (var i = 0; i < this.model._axes.length; i++) {
            if (this.model._axes[i].majorTickLines.size > maxTickSize) {
                maxTickSize = this.model._axes[i].majorTickLines.size;
            }
        }
        var maxLocation = (axis.opposedPosition) ? ((maxTickSize + 10) + (axis.x + axis.width)) : (axis.x + axis.width);
        if (location.X >= 0 && location.X <= maxLocation) {
            var x = (location.X);
            var y = location.Y;
            var padding = 5;
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { axisIndex: index, chartAxis: axis, currentTrackText: text, location: mouseLoc };
            this._trigger("trackAxisToolTip", commonTrackTextArgs);

            var trackAxisText = commonTrackTextArgs.data.currentTrackText;

            var textOffset = ej.EjSvgRender.utils._measureText(trackAxisText, null, axis.crosshairLabel.font);
            if (axis.orientation.toLowerCase() == 'horizontal') {
                x = x - textOffset.width / 2;
                if (axis.labelPosition == 'inside') {
                    if (axis.opposedPosition == false) {
                        y = axis.Location.Y1 - textOffset.height + padding + maxTickSize;
                    }
                    if (axis.opposedPosition == true) {
                        y = axis.Location.Y1 + textOffset.height + padding - maxTickSize;
                    }
                }
                else{                   
                    if (axis.opposedPosition == true) {
                      y = axis.Location.Y1 - textOffset.height/2 + padding - maxTickSize;  
                    }
                    if (axis.opposedPosition == false) {
                        y = axis.Location.Y1 + textOffset.height - padding + maxTickSize;
                    }
                }

            }
            if (!(axis.opposedPosition) && (x + textOffset.width) > (this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X)) {
                x = ((this.model.m_AreaBounds.Width + this.model.m_AreaBounds.X) - textOffset.width) + padding + this.model.elementSpacing;
            }
            if (axis.orientation.toLowerCase() == 'vertical') {
			     y = location.Y + textOffset.height / 4;
                if (axis.labelPosition == 'inside') {               
                    if (axis.opposedPosition == true) {
                        x = axis.x - textOffset.width - maxTickSize;
                    }
                    if (axis.opposedPosition == false) {
                        x = axis.x + padding;
                    }
                } else {
                    if (axis.opposedPosition == true) {
                         x = axis.x + padding;
                    }
                    if (axis.opposedPosition == false) {
                        x = axis.x - textOffset.width - maxTickSize;
                    }
                }                
            }

            var textAxisOptions = {
                'id': this.svgObject.id + '_AxisToolTipText' + '_' + index,
                'x': x,
                'y': y,
                'fill': axis.crosshairLabel.font.color,
                'font-size': axis.crosshairLabel.font.size,
                'font-family': axis.crosshairLabel.font.fontFamily,
                'font-style': axis.crosshairLabel.font.fontStyle,
                'font-weight': axis.crosshairLabel.font.fontWeight,
                'text-anchor': 'start',
                'opacity': axis.crosshairLabel.font.opacity
              
            };

            var fontSize = ej.EjSvgRender.utils._measureText(commonTrackTextArgs.data.currentTrackText, this.model.m_AreaBounds.Width, axis.crosshairLabel.font);
            var crosshairLabel = axis.crosshairLabel;
             var toolAxisRectOptions = {
                'id': this.svgObject.id + '_AxisToolTipRect' + '_' + index,
                'x': (x - padding),
                'y': (y - (fontSize.height)),
                'width': (fontSize.width) + (2* padding),
                'height': (2*fontSize.height)-(2*padding),
                'rx': crosshairLabel.rx,
                'ry': crosshairLabel.ry,
                'fill': crosshairLabel.fill,
                'stroke-width': crosshairLabel.border.width,
                'stroke': crosshairLabel.border.color
            };
             var axes = this.model._axes[index];
            if ((mouseLoc.y < (axes.y +axes.height) && mouseLoc.y > axes.y)||(mouseLoc.x < (axes.x + axes.width) && mouseLoc.x > axes.x)) {
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipRect' + '_' + index), { "display": 'block' });
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipText' + '_' + index), { "display": 'block' });
                     toolAxisRectOptions.display = 'block';
                 }
            else {
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipRect' + '_' + index), { "display": 'none' });
                     this.svgRenderer._setAttr($('#' + this.svgObject.id + '_AxisToolTipText' + '_' + index), { "display": 'none' });
                     toolAxisRectOptions.display = 'none';
                 }
             if (!this.model.enableCanvasRendering) {
             this.svgRenderer.drawRect(toolAxisRectOptions, this.gTrackAxisEle);
             this.svgRenderer.drawText(textAxisOptions, commonTrackTextArgs.data.currentTrackText, this.gTrackAxisEle);
             }else
                 this.svgRenderer.drawCrosshairlabel(toolAxisRectOptions, textAxisOptions, commonTrackTextArgs.data.currentTrackText);
        }
    },
    displayTooltip: function (location, point, series, pointIndex) {
        var seriesIndex = $.inArray(series, this.model._visibleSeries);
		 var requireInvertedAxes = this.model.requireInvertedAxes;
        if ((location.X + this.model.m_AreaBounds.X > this.model.m_AreaBounds.X || location.X == 0) && ((location.X + this.model.m_AreaBounds.X) <= (this.model.m_AreaBounds.X + this.model.m_AreaBounds.Width)) &&
     (location.Y + this.model.m_AreaBounds.Y) >= this.model.m_AreaBounds.Y && (location.Y + this.model.m_AreaBounds.Y) < (this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height)) {
            var padding = 5;
            var x = location.X + (!series.isIndicator ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) :0) + padding + 1;
            var y = (location.Y);
            var format = series.tooltip.format;
            var numberToFixed = ((ej.util.isNullOrUndefined(series.yAxis.roundingPlaces)) ? 2 : series.yAxis.roundingPlaces);
            var trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);
            var seriesName = (series.name) ? (series.name).replace(' ', '') : "series" + seriesIndex;
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { Location : {X: x, Y: y}, Series:series, serIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText.text };
            var tooltip = series.tooltip;
            this._trigger("trackToolTip", commonTrackTextArgs);
            if (!commonTrackTextArgs.cancel) {
                x = commonTrackTextArgs.data.Location.X;
                y = commonTrackTextArgs.data.Location.Y;
                var font = tooltip.font ? $.extend(false, series.font, {}, tooltip.font) : series.font;
                var textOffset = ej.EjSvgRender.utils._measureText(commonTrackTextArgs.data.currentText, series.xAxis.width, font);
                var seriesColor = this.getSeriesColor(point, seriesIndex, series);
                var toolTipOptions = this.getTooltipOptions(seriesColor, series);
                var rectBorderColor = toolTipOptions.rectBColor;
                var rectFillColor = toolTipOptions.rectFColor;
                var textColor = toolTipOptions.rectTextColor;
                var rX = toolTipOptions.rectX;
                var rY = toolTipOptions.rectY;
                var rectOptions = series.tooltip;
                var rectWidth = textOffset.width + (2 * padding);
                var rectHeight = textOffset.height + (2 * padding);               
                axisWidth = (requireInvertedAxes) ? series.yAxis.width : series.xAxis.width;
                if (x + rectWidth - padding >= axisWidth)
                    x = x - rectWidth - ((!series.isIndicator) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) : 0) - (2 * padding) - 2;
                var toolRectOptions = {
                    'id': this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + seriesName,
                    'x': x - padding + this.canvasX,
                    'y': y - textOffset.height / 2 - padding + (this.model.enableCanvasRendering ?(requireInvertedAxes)?series.xAxis.y: series.yAxis.y : 0),
                    'width': rectWidth,
                    'height': rectHeight,
                    'fill': rectOptions.fill ? rectOptions.fill : rectFillColor,
                    'stroke-width': rectOptions.border.width ? rectOptions.border.width : 1,
                    'stroke': rectOptions.border.color ? rectOptions.border.color : rectBorderColor,
                    'fill-opacity': rectOptions.opacity ? rectOptions.opacity : 0.85,
                    'rx': rectOptions.rx ? rectOptions.rx : rX,
                    'ry': rectOptions.ry ? rectOptions.ry : rY
                };

                //this.svgRenderer.drawRect(toolRectOptions, this.gTransToolEle);
                var text = commonTrackTextArgs.data.currentText;
                var fontText = text;
                if (commonTrackTextArgs.data.currentText.indexOf("<br/>") >= 0) {
                    text = commonTrackTextArgs.data.currentText.split("<br/>");
                    fontText = text[0];
                }
                var fontSize = ej.EjSvgRender.utils._measureText(text, series.xAxis.width, font).height;
                var textOptions = {
                    'id': this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + seriesName,
                    'x': x + this.canvasX,
                    'y': y - textOffset.height / 2 - padding + fontSize + (this.model.enableCanvasRendering ? series.yAxis.y : 0),
                    'fill': (tooltip.font && tooltip.font.color) ? font.color : textColor,
                    'font-size': font.size,
                    'font-family': font.fontFamily,
                    'font-style': font.fontStyle,
                    'font-weight': font.fontWeight,
                    'text-anchor': 'start'
                };

                if (this.model.enableCanvasRendering)
                    this.svgRenderer.drawTrackToolTip(toolRectOptions, textOptions, text, padding);
                else {
                    this.svgRenderer.drawRect(toolRectOptions, this.gTransToolEle);
                    this.svgRenderer.drawText(textOptions, text, this.gTransToolEle, font);
                }              
                $('#' + this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + seriesName).attr("visibility", "visible");
                $('#' + this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + seriesName).css("visibility", "visible");
                $('#' + this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + seriesName).attr("visibility", "visible");
                $('#' + this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + seriesName).css("visibility", "visible");
            }
            else {              
                $('#' + this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + seriesName).attr("visibility", "hidden");
                $('#' + this.svgObject.id + '_ToolTipRect' + '_' + seriesIndex + '_' + seriesName).css("visibility", "hidden");
                $('#' + this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + seriesName).attr("visibility", "hidden");
                $('#' + this.svgObject.id + '_ToolTipText' + '_' + seriesIndex + '_' + seriesName).css("visibility", "hidden");
            }
            if ($('#' + this.svgObject.id + "_TrackToolTip_" + seriesIndex).length == 0)
                this.svgRenderer.append(this.gTransToolEle, this.svgObject);
        } else {
            $('#' + this.svgObject.id + '_TrackToolTip_' + seriesIndex).remove();
			//Set canvas Trackball tooltip and symbol visibility hidden when tooltip display out of the chartArea
            $("#" + this._id).find('[id*="canvas_ToolTipRect"]').css("visibility", "hidden");
            $("#" + this._id).find('[id*="canvas_trackSymbol"]').css("visibility", "hidden");
        }

    },
    getTooltipOptions: function (seriesColor, series) {
        var rectBorderColor, rectFillColor, rX, rY, textColor;
        if (this.model.theme.indexOf("gradient") >= 0) {
            rectBorderColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
            rectFillColor = 'white';
            textColor = "#333333";
            rX = 5;
            rY = 5;

        } else {
            var type = series.type.toLowerCase();
            if (type.indexOf("column")==-1 && type.indexOf("bar")==-1 && type!="rangearea" && this.model.AreaType != "none") {
                rectFillColor = jQuery.type(seriesColor) == "array" ? seriesColor[0].color : seriesColor;
                rectBorderColor = 'transparent';
                textColor = "white";
            } else {
                rectBorderColor = "#333333";
                rectFillColor = 'white';
                textColor = "#333333";
            }
            rX = 0;
            rY = 0;
        }
        return { rectBColor: rectBorderColor, rectFColor: rectFillColor, rectTextColor: textColor, rectX: rX, rectY: rY };
    },
    getSeriesColor: function (point, seriesIndex, series) {
        var seriesColor;
        if (series && !series.isIndicator) {
            if (point._hiloFill) {
                if (!point._hiloFill._gradientStop)
                    seriesColor = point._hiloFill;
                else {
                    seriesColor = point._hiloFill._gradientStop[0].color;
                }
            }
            else if (point.fill) {
                if (!point.fill._gradientStop)
                    seriesColor = point.fill;
                else {
                    seriesColor = point.fill._gradientStop[0].color;
                }
            }
            else {
                seriesColor = this.model.seriesColors[seriesIndex];
            }
        } else {
            seriesColor = series.fill;
        }
        return seriesColor;
    },
    getTooltipFormat: function (seriesPoint, series, serIndex, ptIndex, format, numberToFixed) {
        var point = $.extend(true, {}, seriesPoint);
        series.count = 1;
        point.count = 1;
        if (this.model.AreaType == "cartesianaxes") {
            if (series.xAxis._valueType.toLowerCase() == "datetime")
                point.x = (Globalize.format(new Date(point.xValue), ((ej.util.isNullOrUndefined(series.xAxis.labelFormat)) ? ej.EjSvgRender.utils._dateTimeLabelFormat(series.xAxis._intervalType) : series.xAxis.labelFormat)));
            else if (series.xAxis._valueType.toLowerCase() == "category")
                point.x = ej.EjSvgRender.utils._getLabelContent(point.xValue, series.xAxis);
		    else
            {               
                point.x = (ej.EjSvgRender.utils._decimalPlaces(point.x) == 0) ? point.x : point.x.toFixed(numberToFixed);
                var customFormat = (!(series.xAxis.labelFormat)) ? null : series.xAxis.labelFormat.match('{value}');
                point.x = (!(series.xAxis.labelFormat)) ? point.x : (customFormat != null) ? (series.xAxis.labelFormat == "${value}") ? series.xAxis.labelFormat.replace('{value}', '$' + Number(point.x)) : series.xAxis.labelFormat.replace('{value}', Number(point.x)) : (Globalize.format(Number(point.x), series.xAxis.labelFormat));
            }		
        }
        else {
            if (series._xAxisValueType == "date")
                point.x = (Globalize.format(new Date(point.xValue), (ej.EjSvgRender.utils._dateTimeLabelFormat('days'))));
        }
        point.y = (ej.EjSvgRender.utils._decimalPlaces(point.y) == 0) ? point.y : point.y.toFixed(numberToFixed);
        var customFormat = (!(series.yAxis.labelFormat)) ? null : series.yAxis.labelFormat.match('{value}');
        point.y = (!(series.yAxis.labelFormat)) ? point.y : (customFormat != null) ? (series.yAxis.labelFormat == "${value}") ? series.yAxis.labelFormat.replace('{value}', '$' + Number(point.y)) : series.yAxis.labelFormat.replace('{value}',  Number(point.y)) : (Globalize.format(Number(point.y), series.yAxis.labelFormat));
       
        var data = { series: series, point: point };
        format = format.parseTemplate(data);
        return {data: point,text:format};
    },
    displayShowTooltip: function (location, point, series, pointIndex) {
	    var requireInvertedAxes = this.model.requireInvertedAxes;
       if (point.visible) {
            // fixed multiple tooltip issue in mobile
            $(".ejTooltip").not("#" + this.svgObject.id + "_TrackToolTip").remove();
            var measureText;
			var parentZindex;
            var seriesIndex = (!series.isIndicator) ? $.inArray(series, this.model._visibleSeries) : 0,
            seriesColor,
            textOffset,
            padding = 7,
            x,
            y,
            trackTooltipText,
            format = series.tooltip.format;
            var position = document.getElementById(this.svgObject.id).getClientRects()[0];
            var chartPos = this.model.m_AreaBounds;
            var type = series.type.toLowerCase();
            //Set tooltip position ,text and color
            switch (this.model.AreaType) {

                case "cartesianaxes":
                    var numberToFixed = (series.yAxis.roundingPlaces === null || series.yAxis.roundingPlaces === undefined) ? 2 : series.yAxis.roundingPlaces;
                     x = location.X + ((!series.isIndicator) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width)) : 0) + (series.type == "column" || series.type == "bar" || series.type == "stackingbar" || series.type == "stackingcolumn" || series.type == "rangecolumn" ? chartPos.X : (requireInvertedAxes) ? series.yAxis.x : series.xAxis.x);
                    //condition checked for multipleaxis line tooltip position changed
                    y = (location.Y) + (type.indexOf("column") != -1 || type.indexOf("bar") != -1 || type.indexOf("rangearea") != -1 || type == "scatter" || type == "bubble" ? chartPos.Y : (requireInvertedAxes) ? series.xAxis.y : series.yAxis.y);
                    seriesColor = this.getSeriesColor(point, seriesIndex, series);
                    break;

                case "polaraxes":
                    var numberToFixed = (series.yAxis.roundingPlaces === null || series.yAxis.roundingPlaces === undefined) ? 2 : series.yAxis.roundingPlaces;
                    x = location.X + padding + ((ej.util.isNullOrUndefined(series._trackMarker)) ? series.marker.size.width : (series._trackMarker.size.width));
                    y = location.Y; 
                    seriesColor = this.getSeriesColor(point, seriesIndex, series);
                    break;

                case "none":
                    x = location.X + series.marker.size.width + padding;
                    y = (location.Y);
                    numberToFixed = ej.util.isNullOrUndefined(this.model.roundingPlaces) ? 2 : chartObj.model.roundingPlaces;
                    seriesColor = this.model.pointColors[pointIndex];
                    break;
            }
            trackTooltipText = this.getTooltipFormat(point, series, seriesIndex, pointIndex, format, numberToFixed);;
            textOffset = ej.EjSvgRender.utils._measureText(trackTooltipText.text);
            var toolTipOptions = this.getTooltipOptions(seriesColor, series);
            var rectBorderColor = toolTipOptions.rectBColor;
            var rectFillColor = toolTipOptions.rectFColor;
            var textColor = toolTipOptions.rectTextColor;
            var rX = toolTipOptions.rectX;
            var rY = toolTipOptions.rectY;
            var rectOptions = series.tooltip;
            //draw tooltip rectangle
            var tooltipdivRect;
            var padding = 5;

            if (document.getElementById(this.svgObject.id + "_TrackToolTip") == null)
                tooltipdivRect = $("<div id=" + this.svgObject.id + "_TrackToolTip class = " + 'ejTooltip' + "></div>");
            else
                tooltipdivRect = $("#" + this.svgObject.id + "_TrackToolTip");
			var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (ej.isTouchDevice() && !isIE11 && !this.vmlRendering) {
                if (window.navigator.msPointerEnabled)
                    this._on(tooltipdivRect, 'MSPointerMove', this.chartTooltipHover);
                else
                    this._on(tooltipdivRect, 'touchmove', this.chartTooltipHover);
            }
            else {
                this._on(tooltipdivRect, 'mousemove', this.chartTooltipHover);
            }
            $("#" + this.svgObject.id + "_TrackToolTip").show();
			if (!parentZindex)
                parentZindex = $('#' + this._id)[0].offsetParent.style.zIndex;
            var rectOptions = {
                'top': y + $(document).scrollTop(),
                'left': x + $(document).scrollLeft(),
                "background-color": (rectOptions.fill) ? rectOptions.fill : rectFillColor,
                "border-style": "solid",
                "position": "absolute",
                "border-color": (rectOptions.border.color) ? rectOptions.border.color : rectBorderColor,
                "border-width": (rectOptions.border.width || rectOptions.border.width==0) ? rectOptions.border.width : 1,
                "opacity": rectOptions.opacity,
				 'z-index': parentZindex + 1000,
                'border-radius': ((rectOptions.rx) ? rectOptions.rx : rX).toString() + "px " + ((rectOptions.ry) ? rectOptions.ry : rY).toString() + "px",               
                "padding-left": '5px',
				 "padding-right": '5px',
                "padding-top": '2px',
                "padding-bottom": '2px'
            };
            $(tooltipdivRect).css(rectOptions);

            //event for tooltip text
            var commonTrackTextArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonTrackTextArgs.data = { seriesIndex: seriesIndex, pointIndex: pointIndex, currentText: trackTooltipText.text };
            this._trigger("toolTipInitialize", commonTrackTextArgs);
 	        if (!commonTrackTextArgs.cancel) {
            //draw tooltip text
            var fontSize = 0;
            var tooltip = series.tooltip;
            var font = (tooltip.font) ? $.extend(false, series.font, {}, tooltip.font) : series.font;
            
            // draw tooltip text styles
            var textOptions = {
                'top': y + $(document).scrollTop(),
                'left': x + $(document).scrollLeft(),
                "color": (tooltip.font && tooltip.font.color) ? font.color : textColor,
                'font-size': font.size,
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'font-weight': font.fontWeight,
                "align-self": "baseline"
            };
            $(tooltipdivRect).css(textOptions);

            // append the tooltip Div (condition for performance without remove div)
            if (document.getElementById(this.svgObject.id + "_TrackToolTip") == null)
                $(document.body).append(tooltipdivRect[0]);

            // Add string in text array for tooltip format
            var text = measureText = commonTrackTextArgs.data.currentText;
            $("#" + this.svgObject.id + "_TrackToolTip").html(text);

            // div size calculation perform here
            var textHeight = 0;
            var textWidth = 0;
            text = commonTrackTextArgs.data.currentText.split("<br/>");
            for (var i = 0; i < text.length; i++) {
                text[i] = text[i];
                textHeight += ej.EjSvgRender.utils._measureText(text[i], null, font).height;
                var width = ej.EjSvgRender.utils._measureText(text[i], null, font).width;
                if (textWidth < width)
                    textWidth = width;
            }

            fontSize = (this.model.AreaType == "cartesianaxes") ? ej.EjSvgRender.utils._measureText(measureText, series.xAxis.width, font).height : ej.EjSvgRender.utils._measureText(text, null, font).height;
            var maxWidth = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.yAxis.width : series.xAxis.width : $(this.svgObject).width() - (this.model.legend.position.toLowerCase() == "right" ? (this.model.LegendViewerBounds.Width + 2 * this.model.elementSpacing) : 0);
            var maxHeight = (this.model.AreaType == "cartesianaxes") ? (this.model.requireInvertedAxes) ? series.xAxis.height : series.yAxis.height : $(this.svgObject).height() - (this.model.legend.position.toLowerCase() == "bottom" ? (this.model.LegendViewerBounds.Height + this.model.elementSpacing) : 0);

            //Adjust x-position to display tooltip within chart area 
            var box = $("#" + this.svgObject.id + "_TrackToolTip")[0].getBoundingClientRect();

            $("#" + this.svgObject.id + "_TrackToolTip").css("top", ($(document).scrollTop() + box.top + position.top - ((textHeight + 4) / 2)));
			$("#" + this.svgObject.id + "_TrackToolTip").css("left",($(document).scrollLeft() + box.left + position.left + (box.left - x))); 
            if ((x + (textWidth + padding)) >= maxWidth +  series.xAxis.x ) {
                var areaPos = document.getElementById(this.svgObject.id).getClientRects()[0];
                var diff = x - ((textWidth + padding * 2) + ((!series.isIndicator) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.width : 0) : series._trackMarker.size.width) : 0) + (2 * padding));
                $("#" + this.svgObject.id + "_TrackToolTip").css("left", diff + areaPos.left - (padding) + $(document).scrollLeft());
            }
            box = $("#" + this.svgObject.id + "_TrackToolTip")[0].getBoundingClientRect();
            if (box.bottom >= (series.yAxis.y + maxHeight + position.top)) {
                var diffY = y - (((box.bottom - box.top) + ((!series.isIndicator) ? ((ej.util.isNullOrUndefined(series._trackMarker)) ? ((series.marker.visible) ? series.marker.size.height : 0) : series._trackMarker.size.height) : 0))) + (4 * padding)+ position.top ;
                $("#" + this.svgObject.id + "_TrackToolTip").css("top", diffY + $(document).scrollTop());
             }
            //adjust rectangle size based on text length  
            box = $("#" + this.svgObject.id + "_TrackToolTip")[0].getBoundingClientRect();
			if (box.left < 0) {
			// checked condition for whether left position display outsude of the chart area
                $("#" + this.svgObject.id + "_TrackToolTip").css("left", x);
            }
            if (box.top < chartPos.Y)
                $("#" + this.svgObject.id + "_TrackToolTip").css("top", chartPos.Y);
            $("#" + this.svgObject.id + "_TrackToolTip").show();
        }
        }
    },
	chartTooltipHover: function (evt) {
        var box = $("#" + evt.target.id)[0].getBoundingClientRect();
		var diff = evt.clientX - box.left;
		var padding = 5;
		$("#" + evt.target.id).css("left", box.left + diff + padding);
		box = $("#" + evt.target.id)[0].getBoundingClientRect();
		var areaPos = document.getElementById(this.svgObject.id).getClientRects()[0];
		if (box.right > areaPos.right + padding)
				$("#" + evt.target.id).css("left", evt.clientX - (box.right - box.left) - padding);
    },
    _initializeSeriesColors: function () {
        var chartObj = this;
        var modelColor = (chartObj.model.palette) ? chartObj.model.palette : chartObj.model.colors;
        var count = modelColor.length;
        var areaType = chartObj.model.AreaType;
        chartObj.model.seriesColors = [];
        chartObj.model.seriesBorderColors = [];
        chartObj.model.pointColors = [];
        chartObj.model.pointBorderColors = [];
        var elementCollection, color, borderColor, element, tlines;
        if (areaType != "none") {
            elementCollection = chartObj.model._visibleSeries;
            color = chartObj.model.seriesColors;
            borderColor = chartObj.model.seriesBorderColors;
        }
        else {
            elementCollection = chartObj.model._visibleSeries[0].points;
            color = chartObj.model.pointColors;
            borderColor = chartObj.model.pointBorderColors;
        }       
        for (var i = 0; i < elementCollection.length; i++) {
            element = elementCollection[i];
            tlines = element.trendlines;
                element.isFill = (element.isFill == undefined) ? ((element.fill && element.fill != "") ? true : false) : element.isFill;
                if (element.isFill) {
                    color[i] = element.fill;
                } else {
                    color[i] = modelColor[i % count];
                    if (!element._hiloTypes)
                        element.fill = color[i];
                }
                if (areaType != "none" && tlines.length > 0)
                    this._initializeTrendlinesColors(tlines, color[i]);
                if (!ej.util.isNullOrUndefined(element.border) && !ej.util.isNullOrUndefined(element.border.color) && element.border.color != "") {
                    borderColor[i] = element.border.color;
                } else {
                    borderColor[i] = chartObj.model.seriesBorderDefaultColors[i % 10];
                    if (areaType != "none")
                        element.border.color = borderColor[i];
                }
                element.dataPoint = element.points;
            } 
    },

    _initializeTrendlinesColors: function(tlines, color) {
        for (var k in tlines) {
            if (ej.util.isNullOrUndefined(tlines[k].fill) || tlines[k].fill == "")
                tlines[k].fill = color;
        }
    },

    _createAxisLabelAndRange: function () {
	     var count = 0;
        var seriesLength = this.model.series.length;
		 var series;
        for (var i = 0; i < seriesLength; i++) {
            series = this.model.series[i];
            if (series._isTransposed)
                count++;
        }
        if (count == seriesLength)
            this.model.requireInvertedAxes = true;
        else
        {
            series = this.model.series[0];
            if (series._isTransposed)
                this.model.requireInvertedAxes = true;
        }
        if (this.model.requireInvertedAxes) {
            this.model.primaryYAxis.orientation = "horizontal";
            this.model.primaryXAxis.orientation = "vertical";
            this.model.primaryYAxis.labelPlacement = (!(this.model.primaryYAxis.labelPlacement)) ? ej.datavisualization.Chart.LabelPlacement.OnTicks : this.model.primaryYAxis.labelPlacement;
        }
        else {
            this.model.primaryYAxis.orientation = "vertical";
            this.model.primaryXAxis.orientation = "horizontal";
        }
		 
        this.model.primaryYAxis.name = ej.util.isNullOrUndefined(this.model.primaryYAxis.name) ? "SecondaryAxis" : this.model.primaryYAxis.name;
        this.model.primaryXAxis.name = ej.util.isNullOrUndefined(this.model.primaryXAxis.name) ? "PrimaryAxis" : this.model.primaryXAxis.name;

        this.model._axes = [];
        this.model._axes[0] = this.model.primaryXAxis;
        this.model._axes[1] = this.model.primaryYAxis;

        var len = this.model.axes.length;
		 var series = this.model.series;
		 var indicatorSeries = this.model.indicators;
        for (var i = 0; i < len; i++) {
            var axis = this.model.axes[i];
			if (axis.name) {
			//Sets orientation for axes collections
            for (var j = 0; j < series.length; j++) {
                var type = series[j].type.toLowerCase();
                    if (series[j].xAxisName == axis.name) {
                        axis.orientation = (type.indexOf("bar") != -1) ? series[j].isTransposed ? "horizontal" : "vertical" : series[j].isTransposed ? "vertical" : "horizontal";
                        break;
                    }
                    else if (series[j].yAxisName == axis.name) {
                        axis.orientation = (type.indexOf("bar") != -1) ? series[j].isTransposed ? "vertical" : "horizontal" : series[j].isTransposed ? "horizontal" : "vertical";
                            break;
                        }
                 }
				  for (var k = 0; k < indicatorSeries.length; k++) {
                    if (indicatorSeries[k].xAxisName == axis.name) {
                        axis.orientation ="horizontal";
                        break;
                    }
                    else if (indicatorSeries[k].yAxisName == axis.name) {
                        axis.orientation ="vertical";
                        break;
                    }
                 }
		     if (!axis.orientation) continue;
            if (axis.orientation.toLowerCase() == "horizontal")
                this.model.axes[i] = $.extend(true, {}, this.model.secondaryX, axis);
            else
                this.model.axes[i] = $.extend(true, {}, this.model.secondaryY, axis);
            this.model._axes[i + 2] = this.model.axes[i];
		  }
        }
        for (var k = 0; k < this.model._axes.length; k++) {
            var axis = this.model._axes[k];

            axis.visibleLabels = [];
            axis.range = (axis.range.min == null && axis.range.max == null && axis.range.interval == null) ? null : axis.range;            
            axis.setRange = (!(axis.range)) ? false : (ej.util.isNullOrUndefined(axis.setRange)) ? true : axis.setRange;
            axis.setRange = ((axis.actual_Range) || (axis.setRange != false)) ? true : false;
            axis.actualRange = (!(axis.actualRange)) ? (axis.range == null) ? {} : $.extend(true, {}, axis.range) : axis.actualRange;
            axis.visibleRange = (!(axis.visibleRange)) ? (axis.range == null) ? {} : axis.range : axis.visibleRange;
            axis.name = !(axis.name) ? k.toString() : axis.name;
        }
    },






    _drawTitle: function () {
        var measureTitle = ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font);
        var modelTitleHeight = (this.model.title.text == "") ? 0 : (measureTitle.height + this.model.elementSpacing);
        var svgWidth = $(this.svgObject).width();
        var margin = this.model.margin;

        // Drawing the chart title
        var legendSpace = (this.model.legend.position.toLowerCase() == "left") ? this.model.LegendViewerBounds.Width / 2 : ((this.model.legend.position.toLowerCase() == "right" ? (-this.model.LegendViewerBounds.Width / 2) : 0))
        var x = (this.model.AreaType != "cartesianaxes") ?  (margin.left + legendSpace) : this.model.m_AreaBounds.X;
        if (this.model.title.text != "" && this.model.title.text != null) {
            var titleLocation = this.model.margin.top + (modelTitleHeight / 2) + (this.model.elementSpacing);
            var locX = ((this.model.AreaType != "cartesianaxes") ? (svgWidth - margin.left-margin.right)/2 : (this.model.m_AreaBounds.Width / 2)) - measureTitle.width / 2;
           
            if (this.model.title.textAlignment.toLowerCase() == "near") {
                locX = 0;
            }
            else if (this.model.title.textAlignment.toLowerCase() == "far") {
                locX = $(this.svgObject).width() - (this.model.margin.right + this.model.elementSpacing + this.model.m_AreaBounds.X) - measureTitle.width;
            }


            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { title: this.model.title.text, location: { x: locX, y: titleLocation }, size: measureTitle };
            this._trigger("titleRendering", commonEventArgs);
            this.model._titleLocation = { X: commonEventArgs.data.location.x + x, Y: commonEventArgs.data.location.y, size: measureTitle };
            var options = {
                'id': this.svgObject.id + '_ChartTitle',
                'x': commonEventArgs.data.location.x + x,
                'y': commonEventArgs.data.location.y,
                'fill': this.model.title.font.color,
                'font-size': this.model.title.font.size,
                'font-family': this.model.title.font.fontFamily,
                'font-style': this.model.title.font.fontStyle,
                'font-weight': this.model.title.font.fontWeight,
				'opacity': this.model.title.font.opacity,
                'text-anchor': 'start'
            };

            this.svgRenderer.drawText(options, commonEventArgs.data.title, this.gTitleEle);

            this.svgRenderer.append(this.gTitleEle, this.svgObject);
     if (this.model.title.subTitle.text != "" && this.model.title.subTitle.text)
       this._drawSubTitle();
        }
    },



    _getLegendSize: function (series) {

        var legend = this.model.legend,
            symbolSize = legend.itemStyle,
            textSize = ej.EjSvgRender.utils._measureText(series.name, null, legend.font),
            padding = 10,
            width = symbolSize.width + padding + textSize.width,
            height = Math.max(symbolSize.height, textSize.height);

        return { Width: width, Height: height };

    },

    _getYValues: function (series) {
        var values = [];
        for (var i = 0; i < series.points.length; i++)
            values.push(series.points[i].y);
        return values;
    },
    _getXValues: function (series) {
        var values = [];
        for (var i = 0; i < series.points.length; i++)
            values.push(series.points[i].xValue);
        return values;
    },




    _drawSubTitle: function () {


        var measureTitle = ej.EjSvgRender.utils._measureText(this.model.title.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.font);
        var measuresubTitle = ej.EjSvgRender.utils._measureText(this.model.title.subTitle.text, $(this.svgObject).width() - this.model.margin.left - this.model.margin.right, this.model.title.subTitle.font);

        var textanchor = "middle";

        var modelsubTitleHeight = (this.model.title.subTitle.text == "") ? 0 : (measuresubTitle.height + this.model.elementSpacing);

        // Drawing the chart subtitle

        if (this.model.title.subTitle.text != "" && this.model.title.subTitle.text != null) {
            var subTitleLocation =  (modelsubTitleHeight) / 2 + this.model.elementSpacing + this.model._titleLocation.Y ;

            var locX = this.model._titleLocation.X + (measureTitle.width / 2);

            if (this.model.title.subTitle.textAlignment.toLowerCase() == "near") {

                locX = this.model._titleLocation.X;
                textanchor = "start";
            }
            else if (this.model.title.subTitle.textAlignment.toLowerCase() == "far") {

                locX = this.model._titleLocation.X + measureTitle.width;

                textanchor = "end";
            }
            var commonEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonEventArgs.data = { subTitle: this.model.title.subTitle.text, location: { x: locX, y: subTitleLocation }, size: measuresubTitle };
            this._trigger("subTitleRendering", commonEventArgs);
            this.model._subTitleLocation = { X: commonEventArgs.data.location.x, Y: commonEventArgs.data.location.y };
            var options = {
                'id': this.svgObject.id + '_ChartsubTitle',

                'x': commonEventArgs.data.location.x,
                'y': commonEventArgs.data.location.y,
                'fill': this.model.title.subTitle.font.color,
                'font-size': this.model.title.subTitle.font.size,
                'font-family': this.model.title.subTitle.font.fontFamily,
                'font-style': this.model.title.subTitle.font.fontStyle,
                'font-weight': this.model.title.subTitle.font.fontWeight,
				'opacity': this.model.title.subTitle.font.opacity,
                'text-anchor': textanchor
            };

            this.svgRenderer.drawText(options, commonEventArgs.data.subTitle, this.gSubTitleEle);

            this.svgRenderer.append(this.gSubTitleEle, this.gTitleEle);
        }
    },


    _calculateStackingValues: function (seriesCollection, axis) {

        this.model.stackedValue[axis.name] = [];
        this.model.stackedValue[axis.name].min = 0;
        this.model.stackedValue[axis.name].max = 0;
        var stackAxes = (!this.model.requireInvertedAxes) ? this.model.hAxes : this.model.vAxes;

        for (var k = 0; k < stackAxes.length; k++) {
            j = 0;
            var lastPosValue = [];
            var lastNegValue = [];
            for (var i = 0; i < seriesCollection.length; i++) {
                if (seriesCollection[i]._xAxisName == stackAxes[k].name) {
                    if (seriesCollection[i].type.toLowerCase().indexOf("stacking") != -1 || seriesCollection[i].isStacking) {
                        var values = {};
                        values.StartValues = [];
                        values.EndValues = [];
                        if (seriesCollection[i].stackingGroup && seriesCollection[i].type.toLowerCase() != "stackingarea") {
                            if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup])) {
                                lastPosValue[seriesCollection[i].stackingGroup] = [];
                                lastNegValue[seriesCollection[i].stackingGroup] = [];
                            }
                        }

                        else {
                            seriesCollection[i].stackingGroup = "";
                            if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup])) {
                                lastPosValue[seriesCollection[i].stackingGroup] = [];
                                lastNegValue[seriesCollection[i].stackingGroup] = [];
                            }
                        }
                        series = seriesCollection[i];
                        series.points = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
                        var yValues = this._getYValues(series);
                        for (var j = 0; j < series.points.length; j++) {
                            var lastValue = 0;
                            var currentValue = yValues[j];
                            if (ej.util.isNullOrUndefined(lastPosValue[seriesCollection[i].stackingGroup][series.points[j].xValue])) {
                                lastPosValue[seriesCollection[i].stackingGroup][series.points[j].xValue] = 0;
                            }
                            if (ej.util.isNullOrUndefined(lastNegValue[seriesCollection[i].stackingGroup][series.points[j].xValue])) {
                                lastNegValue[seriesCollection[i].stackingGroup][series.points[j].xValue] = 0;
                            }
                            if (values.StartValues.length <= j) {
                                values.StartValues.push(0);
                                values.EndValues.push(0);
                            }
                            if (currentValue > 0) {
                                lastValue = lastPosValue[seriesCollection[i].stackingGroup][series.points[j].xValue];
                                lastPosValue[seriesCollection[i].stackingGroup][series.points[j].xValue] += currentValue;
                            }
                            else {
                                lastValue = lastNegValue[seriesCollection[i].stackingGroup][series.points[j].xValue];
                                lastNegValue[seriesCollection[i].stackingGroup][series.points[j].xValue] += currentValue;
                            }

                            values.StartValues[j] = lastValue;
                            values.EndValues[j] = currentValue + lastValue;
                            series.points[j].YValues = [values.EndValues[j]];
                        }
                        values.stackedSeries = false;
                        this.model.stackedValue[axis.name].push(values);
                        if (this.model.stackedValue[axis.name].min > Math.min.apply(0, values.StartValues))
                            this.model.stackedValue[axis.name].min = Math.min.apply(0, values.StartValues);
                        if (this.model.stackedValue[axis.name].max < Math.max.apply(0, values.EndValues))
                            this.model.stackedValue[axis.name].max = Math.max.apply(0, values.EndValues);
                        series.stackedValue = values;
                    }
                }
            }
        }
    },
    _calculateStackingCumulativeValues: function (seriesCollection, axis) {
        this.model.stackedValue[axis.name] = [];
        this.model.stackedValue[axis.name].min = 0;
        this.model.stackedValue[axis.name].max = 0;
        var stackAxes = (!this.model.requireInvertedAxes) ? this.model.hAxes : this.model.vAxes;
        var seriesLength = seriesCollection.length;
        var axesLength = stackAxes.length;
        var percent;
        for (var k = 0; k < axesLength ; k++) {
            var posValues = [];
            var negValues = [];
            var frequencies = [];
            var stackingName = " ";

            //calculate the cumulative frequencies from the collection of series  
            for (var i = 0; i < seriesLength; i++) {
                stackingName = seriesCollection[i].stackingGroup;
                if (seriesCollection[i].type.toLowerCase().indexOf("100") != -1) {
                    if (seriesCollection[i]._xAxisName == stackAxes[k].name) {
                        if (seriesCollection[i].stackingGroup && seriesCollection[i].type.toLowerCase() != " stackingarea100 ") {
                            if (!posValues[stackingName]) {
                                frequencies[stackingName] = [];
                                posValues[stackingName] = [];
                                negValues[stackingName] = [];
                            }
                        }
                        else {
                            seriesCollection[i].stackingGroup = "";
                            stackingName = seriesCollection[i].stackingGroup;
                            if (!posValues[stackingName]) {
                                frequencies[stackingName] = [];
                                posValues[stackingName] = [];
                                negValues[stackingName] = [];
                            }
                        }
                        var yValues = this._getYValues(seriesCollection[i]);
                        var yValuesLength = yValues.length;
                        for (var j = 0; j < yValuesLength; j++) {
                            var getYValue = yValues[j];
                            if (!frequencies[stackingName][seriesCollection[i].points[j].xValue]) {
                                frequencies[stackingName][seriesCollection[i].points[j].xValue] = 0;
                            }
                            frequencies[stackingName][seriesCollection[i].points[j].xValue] += Math.abs(getYValue);
                        }
                    }
                }
              
            }

            //calculate the cumulative percentage for each series points
            for (var i = 0; i < seriesLength; i++) {
                stackingName = seriesCollection[i].stackingGroup;
                if (seriesCollection[i].type.toLowerCase().indexOf("100") != -1) {
                    if (seriesCollection[i]._xAxisName == stackAxes[k].name) {
                        var values = {};
                        values.StartValues = [];
                        values.EndValues = [];
                        series = seriesCollection[i];
                        series.points = ej.DataManager(series.points, ej.Query().sortBy("xValue")).executeLocal();
                        var yValues = this._getYValues(series);
                        var yValuesLength = yValues.length;
                        for (var j = 0; j < yValuesLength; j++) {
                            var lastValue = 0;
                            var currentValue = yValues[j];

                            //Pecentage calculation for point 
                            percent = (currentValue / frequencies[stackingName][series.points[j].xValue]) * 100
                            currentValue = !isNaN(percent) ? percent :0;
                            series.points[j].percentage = currentValue.toFixed(2);
                            if (!posValues[stackingName][series.points[j].xValue]) {
                                posValues[stackingName][series.points[j].xValue] = 0;
                            }
                            if (!negValues[stackingName][series.points[j].xValue]) {
                                negValues[stackingName][series.points[j].xValue] = 0;
                            }
                            if (values.StartValues.length <= j) {
                                values.StartValues.push(0);
                                values.EndValues.push(0);
                            }
                            if (currentValue > 0) {
                                lastValue = posValues[stackingName][series.points[j].xValue];
                                posValues[stackingName][series.points[j].xValue] += currentValue;
                            }
                            else {
                                lastValue = negValues[stackingName][series.points[j].xValue];
                                negValues[stackingName][series.points[j].xValue] += currentValue;
                            }
                            values.StartValues[j] = lastValue;
                            values.EndValues[j] = lastValue + currentValue;
                            if (values.EndValues[j] > 100) values.EndValues[j] = 100;
                            series.points[j].YValues = [values.EndValues[j]];
                        }
                        values.stackedSeries = false;
                        this.model.stackedValue[axis.name].push(values);
                        if (this.model.stackedValue[axis.name].min > Math.min.apply(0, values.StartValues))
                            this.model.stackedValue[axis.name].min = Math.min.apply(0, values.StartValues);
                        if (this.model.stackedValue[axis.name].max < Math.max.apply(0, values.EndValues))
                            this.model.stackedValue[axis.name].max = Math.max.apply(0, values.EndValues);
                        if (this.model.stackedValue[axis.name].min > Math.min.apply(0, values.EndValues))
                            this.model.stackedValue[axis.name].min = -100;

                        series.stackedValue = values;
                    }
                }
            }
        }
    },
    _legendItemBounds: function (itemCount, legendItemWidth, legendItemHeight) {
        var legend = this.model.legend,
            position = legend.position.toLowerCase(),
            itemPadding = legend.itemPadding,
            legnedHeightIncr, column, legendWidth = 0, legendHeight = 0;
        if (ej.util.isNullOrUndefined(legend.columnCount) && legend.rowCount) {
            legnedHeightIncr = legend.rowCount;
            column = Math.ceil(itemCount / legnedHeightIncr);
            legendWidth = legendItemWidth * column;
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        else if (ej.util.isNullOrUndefined(legend.rowCount) && legend.columnCount) {
            legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
            legendWidth = legendItemWidth * this.model.legend.columnCount;
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        else if ((legend.rowCount) && (legend.columnCount)) {
            if (legend.columnCount < legend.rowCount) {
                legnedHeightIncr = legend.rowCount;
                column = Math.ceil(itemCount / legnedHeightIncr);
                legendWidth = legendItemWidth * column;
            }
            else {
                if (position === 'top' || position === 'bottom' || position === 'custom') {
                    legnedHeightIncr = Math.ceil(itemCount / legend.columnCount);
                    legendWidth = legendItemWidth * legend.columnCount;
                } else {
                    legnedHeightIncr = legend.rowCount;
                    column = Math.ceil(itemCount / legnedHeightIncr);
                    legendWidth = legendItemWidth * column;
                }
                
            }
            legendHeight = legendItemHeight * legnedHeightIncr;
        }
        legendHeight += this.model.elementSpacing;
        return { LegendWidth: legendWidth, LegendHeight: legendHeight };
    },

    _calculateLegendBounds: function () {
        // intialize
        this.model.legendCollection = [];
        this.model.legendRegion = [];
        this.model.svgHeight = $(this.svgObject).height();
        this.model.svgWidth = $(this.svgObject).width();

        var chart = this,
            chartModel = chart.model,
            math = Math,
            max = math.max,
            min = math.min,
            abs = math.abs,
            legend = chartModel.legend,
            padding = 10,
            legendSizeHeight = legend.size.height,
            legendSizeWidth = legend.size.width,
            itemPadding = legend.itemPadding > 0 ? legend.itemPadding : 0,
            position = legend.position.toLowerCase(),
            width = 0, height = 0,
            svgHeight = chartModel.svgHeight,
            svgWidth = chartModel.svgWidth,
            legendItemWidth = 0,
            legendItemHeight = 0,
            legendWidth = 0,
            legendHeight = 0,
            legnedHeightIncr = 1,
            currentLegend,
            index,
            bounds,
            legendItem,
            legendviewerHeight = 0, legendviewerWidth = 0,
            legendHeightTemp, legendWidthTemp,
            tempSeries = {},
            legendSeries = [], trendLineLength, trendLines,
            trendLine,
            areaType = chartModel.AreaType,
            visibleSeries = chartModel._visibleSeries,
            rowCount = legend.rowCount, point,
            columnCount = legend.columnCount,
            elementSpacing = chartModel.elementSpacing,
            visiblePoints, series,
            length, legendColor,
            titleSize = ej.EjSvgRender.utils._measureText(legend.title.text, null, legend.title.font),
            legendsize, colorGradName,
            vScrollSize = 0, hScrollSize = 0,
            borderSize = chartModel.border.width,
            legendBorder = legend.border.width,
            svgObjectHeight = svgHeight - ((elementSpacing * 4) + (borderSize * 2) + (legend.border.width * 2)),
            svgObjectWidth = svgWidth - ((elementSpacing * 4) + (borderSize * 2) + (legendBorder * 2));
        if (visibleSeries && legend.visible) {
            //calculating legend Position
            // Legend collections added here
            if (areaType == "none") {
                series = visibleSeries[0];
                visiblePoints = ej.EjSeriesRender.prototype._isVisiblePoints(series);
                length = visiblePoints.length;
                for (var j = 0; j < length; j++) {
                    legendColor = legend.fill ? (!(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop) : chartModel.pointColors[j];
                    point = visiblePoints[j];
                    if (!point.isEmpty) {
                        tempSeries = {
                            name: point.x,
                            Font: series.font ? series.font : legend.font,
                            seriesIndex: 0,
                            pointIndex: j,
                            fill: legendColor,
                            visibility: point.visible ? 'visible' : 'hidden'
                        };
                        legendSeries.push(tempSeries);
                    }
                }
            }
            else {
                length = visibleSeries.length;
                for (var j = 0; j < length; j++) {
                    legendColor = legend.fill ? (!(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop) : chartModel.seriesColors[j];
                    series = visibleSeries[j];
                    tempSeries = {
                        name: series.name ? series.name : 'series' + j,
                        Font: series.font ? series.font : legend.font,
                        seriesIndex: j,
                        fill: legendColor,
                        visibility: series.visibility
                    };
                    legendSeries.push(tempSeries);
                    trendLines = series.trendlines;
                    trendLineLength = trendLines.length;
                    for (var i = 0; i < trendLineLength; i++) {
                        trendLine = trendLines[i];
                        legendColor = legend.fill ? (!(legend.fill._gradientStop) ? legend.fill : legend.fill._gradientStop) : trendLine.fill;
                        if (trendLine.visibility != '') {
                            tempSeries = {
                                name: trendLine.name,
                                Font: series.font ? series.font : legend.font,
                                seriesIndex: j,
                                trendLineIndex: i,
                                fill: legendColor,
                                isTrendLine: true,
                                visibility: series.visibility.toLowerCase() == "visible" ? trendLine.visibility : "hidden"
                            };
                            legendSeries.push(tempSeries);
                        }
                    }
                }
            }
            legendSeriesLength = legendSeries.length;
            for (var j = 0; j < legendSeriesLength; j++) {
                currentLegend = legendSeries[j];
                legendsize = chart._getLegendSize(currentLegend);
                legendItemWidth = max(legendsize.Width, legendItemWidth);
                legendItemHeight = max(legendsize.Height, legendItemHeight);
            }

            legendHeight = legendItemHeight + elementSpacing * 2;
            legendWidth = legendItemWidth;

            if (columnCount || rowCount) {
                legendBounds = chart._legendItemBounds(legendSeries.length, legendItemWidth + (itemPadding), legendItemHeight + (itemPadding));
                legendWidth = legendBounds.LegendWidth;
                legendHeight = legendBounds.LegendHeight;
                if (position === 'top' || position === 'bottom' || position === 'custom')
                    legendHeight = legendHeight - itemPadding + elementSpacing;
                else
                    legendWidth = legendWidth - itemPadding;
            }

            for (var k = 0; k < legendSeriesLength; k++) {
                currentLegend = legendSeries[k];
                index = currentLegend.seriesIndex;
                series = chartModel.series[index];
                legendsize = chart._getLegendSize(currentLegend);
                if (!(rowCount) && !(columnCount)) {
                    if ((position == 'top' || position == 'bottom' || position == 'custom')) {
                        width += legendsize.Width + itemPadding;
                        if (width > svgObjectWidth && k !=0) {
                            width -= legendsize.Width + itemPadding;
                            legendWidth = max(legendWidth, width);
                            width = legendsize.Width + itemPadding;
                            legnedHeightIncr++;
                            legendHeight += legendItemHeight + itemPadding;
                        }
                        else
                            legendWidth = max(legendWidth, width);

                        height = max(height, legendItemHeight);
                    }
                    else {
                        height += legendsize.Height + itemPadding;
                        if (height > svgObjectHeight) {
                            height -= legendsize.Height + itemPadding;
                            legendHeight = max(legendHeight, height);
                            height = legendsize.Height + itemPadding;
                            legendWidth += legendItemWidth + itemPadding;
                        }
                        else
                            legendHeight = max(legendHeight, height);

                        width = max(width, legendItemWidth);
                    }
                }
                if (currentLegend.visibility.toLowerCase() == "visible")
                    colorGradName = chart.svgRenderer.createGradientElement('legend' + k, legendSeries[k].fill, 0, 0, 0, svgObjectHeight, chart.gLegendEle);
                else
                    colorGradName = 'gray';

                bounds = (rowCount || columnCount) ? { Width: legendItemWidth, Height: legendItemHeight } : legendsize;
                legendItem = {
                    Text: currentLegend.name ? currentLegend.name : 'series' + index,
                    Shape: legend.shape,
                    LegendStyle: {
                        BorderColor: legend.itemStyle.border.color,
                        BorderWidth: legend.itemStyle.border.width,
                        Opacity: legend.opacity,
                        Color: colorGradName,
                        Font: currentLegend.Font
                    },
                    Bounds: bounds,
                    SeriesIndex: index,
                };
                if (currentLegend.isTrendLine) {
                    legendItem.TrendLineIndex = currentLegend.trendLineIndex;
                    legendItem.drawType = series.trendlines[currentLegend.trendLineIndex].type.toLowerCase() == 'linear' ? 'line' : 'spline';
                }
                if (areaType == "none")
                    legendItem.PointIndex = currentLegend.pointIndex;

                chartModel.legendCollection.push(legendItem);
            }

            //LegendBounds calculation Perform here..
            if (position === 'top' || position === 'bottom' || position === 'custom') {
                legendWidth = titleSize.width > legendWidth - itemPadding ? (titleSize.width + padding * 2 + itemPadding) : legendWidth + padding * 2;
                width += padding ;
                height += padding * 2;
                chartModel.LegendBounds = { Width: max(legendWidth, width) - itemPadding, Height: max(legendHeight, height), Rows: legnedHeightIncr };
            }
            else {
                legendWidth = titleSize.width > legendWidth ? (titleSize.width + padding * 2) : legendWidth + padding * 2;
                width += padding;
                height += padding;
                chartModel.LegendBounds = { Width: max(legendWidth, width), Height: max(legendHeight, height) + padding - itemPadding, Columns: legnedHeightIncr };
            }
            //Calculating legend viewer bounds or calculate user specified legend bounds
            chartModel.LegendViewerBounds = { Width: "0", Height: "0" };

            if (legendSizeHeight == "" || legendSizeHeight == null) {
                if (position == 'left' || position == 'right' || position == 'custom')
                    legendHeightTemp = abs(svgHeight);
                else
                    legendHeightTemp = abs((svgHeight / 100) * parseInt('20%'));
                chartModel.LegendViewerBounds.Height = min(legendHeightTemp, chartModel.LegendBounds.Height);
            }
            else{
				if (legendSizeHeight.indexOf("%") != -1)
                    chartModel.LegendViewerBounds.Height = abs((svgHeight / 100) * parseInt(legendSizeHeight));
                else
                    chartModel.LegendViewerBounds.Height = parseInt(legendSizeHeight);
			}                

            if (legendSizeWidth == "" || legendSizeWidth == null) {
                if (position == 'top' || position == 'bottom' || position == 'custom')
                    legendWidthTemp = abs(svgWidth);
                else
                    legendWidthTemp = abs((svgWidth / 100) * parseInt('20%'));
                chartModel.LegendViewerBounds.Width = min(legendWidthTemp, chartModel.LegendBounds.Width);
            }
            else{
				 if (legendSizeWidth.indexOf("%") != -1)
                    chartModel.LegendViewerBounds.Width = abs((svgWidth / 100) * parseInt(legendSizeWidth));
                else
                    chartModel.LegendViewerBounds.Width = parseInt(legendSizeWidth);
			}                
        }
        else {
            chartModel.LegendBounds = { Width: 0, Height: 0 };
            chartModel.LegendViewerBounds = { Width: 0, Height: 0 };
        }
        legendContainer = $(chart.legendContainer);
        legendSvgContainer = $(chart.legendSvgContainer);
        if (legend.enableScrollbar) {
            chartModel.LegendActualBounds = chartModel.LegendViewerBounds;
            legendContainer.removeAttr("style");
            legendContainer.css({ "overflow": "scroll", "visibility": "hidden", "width": chartModel.LegendViewerBounds.Width, "height": chartModel.LegendViewerBounds.Height });
            legendSvgContainer.css({ "height": chartModel.LegendBounds.Height, "width": chartModel.LegendBounds.Width });
            if (chartModel.LegendBounds.Width > chartModel.LegendViewerBounds.Width && chartModel.LegendBounds.Height > chartModel.LegendViewerBounds.Height) {
                legendContainer.css({ "overflow": "scroll" });
                hScrollSize = legendContainer[0].offsetHeight - (legendBorder * 2) - legendContainer[0].clientHeight;
                vScrollSize = legendContainer[0].offsetWidth - (legendBorder * 2) - legendContainer[0].clientWidth;
            } else if (chartModel.LegendBounds.Width > chartModel.LegendViewerBounds.Width) {
                legendContainer[0].style.overflowX = "scroll";
                legendContainer[0].style.overflowY = "hidden";
                hScrollSize = legendContainer[0].offsetHeight - (legendBorder * 2) - legendContainer[0].clientHeight;
                vScrollSize = 0;
            } else if (chartModel.LegendBounds.Height > chartModel.LegendViewerBounds.Height) {
                legendContainer[0].style.overflowY = "scroll";
                legendContainer[0].style.overflowX = "hidden";
                hScrollSize = 0;
                vScrollSize = legendContainer[0].offsetWidth - (legendBorder * 2) - legendContainer[0].clientWidth;
            }
            
        }
        else {
            chartModel.LegendActualBounds = chartModel.LegendBounds;
            if (legendSizeWidth != "" && legendSizeWidth != null)
                chartModel.LegendActualBounds.Width = parseInt(legendSizeWidth);
            if (legendSizeHeight != "" && legendSizeHeight != null)
                chartModel.LegendActualBounds.Height = parseInt(legendSizeHeight);
        }
        chartModel.LegendActualBounds.Height += Math.abs(hScrollSize);
        chartModel.LegendActualBounds.Width += Math.abs(vScrollSize);
        chartModel.LegendActualBounds.hScrollSize = hScrollSize;
        chartModel.LegendActualBounds.vScrollSize = vScrollSize;
    },

    GetPointXYOrgin: function (x, y, orginX, orginY) {

        var xvalue = ((x - orginX) / (this.model.axes.PrimaryXaxis.visibleRange.max - orginX)) * (this.model.m_AreaBounds.Width);
        var yvalue = ((y - orginY) / (this.model.axes.PrimaryYaxis.visibleRange.max - orginY)) * (this.model.m_AreaBounds.Height - this.GetPointXY(this.model.axes.PrimaryXaxis.visibleRange.min, Math.max(this.model.axes.PrimaryYaxis.visibleRange.min, 0)).Y);

        return { X: xvalue, Y: yvalue };
    },






  


    _drawChartAreaRect: function () {

        var fillColor = !(this.model.background && this.model.background._gradientStop) ? this.model.background : 'url(#' + this.svgObject.id + '_backGradient)';

        if (this.model.backGroundImageUrl)
            fillColor = 'transparent';
        
        var chartBorder = this.model.border;
        var options = {
            'id': this.svgObject.id + '_SvgRect',
            'x': chartBorder.width / 2,
            'y': chartBorder.width / 2,
            'width': $(this.svgObject).width() - (2 * chartBorder.width),
            'height': $(this.svgObject).height() - (2 * chartBorder.width),
            'fill': fillColor,
            'opacity': chartBorder.opacity,
            'stroke-width': chartBorder.width,
            'stroke': chartBorder.color
        };
        this.svgRenderer.drawRect(options, this.svgObject);
        


        if (this.model.backGroundImageUrl)
            this._drawBackImage();

        if (this.model.AreaType == 'cartesianaxes' && !this.model.enable3D) {
            var borderOptions = {
                'id': this.svgObject.id + '_ChartArea',
                'x': this.model.m_AreaBounds.X,
                'y': this.model.m_AreaBounds.Y,
                'width': this.model.m_AreaBounds.Width,
                'height': this.model.m_AreaBounds.Height,
                'fill': this.model.chartArea.background,
                'stroke-width': this.model.chartArea.border.width,
                'opacity': this.model.chartArea.border.opacity,
                'stroke': this.model.chartArea.border.color
            };
            this.svgRenderer.drawRect(borderOptions, this.svgObject);


        }

    },
    axesIndexCount: function (axis,index) {
        var vRowcount = [];
        for (var k = 0; k < axis.length; k++) {
            var currentAxis = axis[k];
            if (currentAxis.orientation.toLowerCase() == "vertical") {
                if (currentAxis.rowIndex == index)
                    vRowcount.push(currentAxis);
            }
            else {
                if (currentAxis.columnIndex == index)
                    vRowcount.push(currentAxis);
            }
        }
        return vRowcount ;
    },
    axesCount: function (axis) {
        var vRowcount = [], indexValue=[], definitionsLength=0;
        var chartModel = this.model;
        $.each(axis, function (index, currentAxis) {
            if (currentAxis.orientation.toLowerCase() == "vertical") {
                definitionsLength =(chartModel.rowDefinitions)?chartModel.rowDefinitions.length:0;
                if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex });
                else if (currentAxis.rowIndex != vRowcount[vRowcount.length - 1].axis.rowIndex)
                    vRowcount.push({ axis: currentAxis, index: currentAxis.rowIndex});
            }
            else {
                definitionsLength =(chartModel.columnDefinitions)? chartModel.columnDefinitions.length:0;
                if (index == 0) vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
                else if (currentAxis.columnIndex != vRowcount[vRowcount.length - 1].axis.columnIndex)
                    vRowcount.push({ axis: currentAxis, index: currentAxis.columnIndex });
            }
        });
        var length = vRowcount.length;
        if (definitionsLength > length) {
            length = definitionsLength;
            for (var l = 0; l < definitionsLength; l++)
                indexValue.push(l);
        } else {
            for (var i = 0; i < length; i++)
                indexValue.push(vRowcount[i].index);
        }
        
        return { length: length, indexValue: indexValue };

    },
   
     //Calculate position for each vertical axis
    _calRowSize: function () {
        var start = 0;
        var yaxisOrign = [];
        var totalRealLength = 0;
        var orginY = this.model.m_AreaBounds.Y + this.model.m_AreaBounds.Height;
        //Caluculate size for rowdefinitions values
        if (this.model._rowDefinitions[0].rowDefinitions != null) {
            for (var i = 0, rowMax = this.model._rowDefinitions.length; i < rowMax; i++) {
                var rowDef = this.model._rowDefinitions[i].rowDefinitions;
                rowDef.rowHeight = Math.abs(rowDef.rowHeight);
                var realLength = 0;
                if (rowDef.unit.toLowerCase() == "percentage"){
                    var length = this._axisRowDefinitionSize(this.model.rowDefinitions);
                    realLength = Math.floor((this.model.m_AreaBounds.Height) * (rowDef.rowHeight / length));
                }  
                else
                    realLength = rowDef.rowHeight;
                if (i == 0) yaxisOrign.push(orginY);
                   
                      yaxisOrign.push( (orginY - realLength - totalRealLength ));
                    totalRealLength += realLength;
                

            }
            
        }
        //Calculate size based on chart area size
        else {
            var height = Math.floor((this.model.m_AreaBounds.Height) / this.model._rowDefinitions.length);

            for (start = (orginY) ; start >= this.model.m_AreaBounds.Y; start = start - height) {
                yaxisOrign.push(start);
            }
        }
        return yaxisOrign;
    },
    
     //Calculate position for each horizontal axis
    _calColumnSize: function () {
        var xaxisOrign = [];
        var orginX = this.model.m_AreaBounds.X;
        var areaWidth = this.model.m_AreaBounds.Width;
        var totalRealLength = 0;
        var columnDef = this.model._columnDefinitions;
        //Caluculate size for columndefinitions values
        if (columnDef[0].columnDefinitions != null) {
            for (var i = 0, max = columnDef.length; i < max; i++) {
                var rowDef = columnDef[i].columnDefinitions;
				rowDef.columnWidth = Math.abs(rowDef.columnWidth);
                var realLength;
                if (rowDef.unit.toLowerCase() == "percentage"){
				    var length = this._axisColumnDefinitionSize(this.model.columnDefinitions);
				    realLength = Math.floor((areaWidth) * (rowDef.columnWidth / length));
                } 
                else
                    realLength = rowDef.columnWidth;
                if (i == 0) xaxisOrign.push(orginX);

                xaxisOrign.push((orginX + realLength + totalRealLength));
                totalRealLength += realLength;
            }
        }
        //Caluculate size based on chart area size
        else {
            var width = Math.floor((areaWidth) / columnDef.length);
            for (var start = orginX, maxStart = orginX + areaWidth; start <= maxStart; start = start + width) {
                xaxisOrign.push(start);

            }
        }
        return xaxisOrign;
    },
    
     //Calculate position and size for each axis
    _calculateAxisSize: function () {

        //For Horizontal
        var isInitAxes = true, isOppInitAxes = true;
        var xaxisOrign = this._calColumnSize();
        var columnDefinition = this.model._columnDefinitions;
        for (var i = 0, maxLength = columnDefinition.length; i < maxLength; i++) {
            var x1 = xaxisOrign[i];
            var x2 = xaxisOrign[i + 1];
            var nearIndexVal = 0;
            var farIndexVal = 0;;
            var isColFirstAxes = true, isColOppFirstAxes = true;
            var oppY = this.model.m_AreaBounds.Y;
            var norY = oppY + this.model.m_AreaBounds.Height;
            for (var j = 0, maxCol = columnDefinition[i].axis.length; j < maxCol; j++) {
                var currentColAxis = columnDefinition[i].axis[j];
                var realY = 0, axisColumnSpace=0;
                if (!currentColAxis.opposedPosition) {
                    if (!(isColFirstAxes && (isInitAxes || nearIndexVal==0))) {
                        axisColumnSpace = columnDefinition.nearSizes[nearIndexVal];
                        realY = norY = (norY + axisColumnSpace);
                        nearIndexVal++;
                    }
                    else {
                        realY = norY;
                        isColFirstAxes = false;
                    }

                } else {
                    if (!(isColOppFirstAxes && (isOppInitAxes || farIndexVal==0))) {
                        axisColumnSpace = columnDefinition.farSizes[farIndexVal];
                        realY = oppY = (oppY - axisColumnSpace);
                        farIndexVal++;
                    }
                    else {
                        realY = oppY;
                        isColOppFirstAxes = false;
                    }
                    }
                if (currentColAxis.columnSpan) {
                    //Skip the axis size calculation for duplicate span axis
                    if (!(columnDefinition[i].index == currentColAxis.columnIndex))
                        continue;
                    var axisSize = (maxLength >= (i + currentColAxis.columnSpan)) ? i + currentColAxis.columnSpan : i+1;
                    x2 = xaxisOrign[axisSize];
                    if (!currentColAxis.opposedPosition && nearIndexVal==0)isInitAxes = isColFirstAxes;
                    if (currentColAxis.opposedPosition && farIndexVal == 0) isOppInitAxes = isColOppFirstAxes;
                } else {
                    x2 = xaxisOrign[i + 1];
                    isInitAxes = true;
                    isOppInitAxes = true;
                }
                
               currentColAxis.Location = { };
                currentColAxis.Location.X1 = x1 + currentColAxis.plotOffset;
                currentColAxis.Location.Y1 = realY;
                currentColAxis.Location.X2 = x2 - currentColAxis.plotOffset;
                currentColAxis.Location.Y2 = realY;
                currentColAxis.x = currentColAxis.Location.X1;
                currentColAxis.y = currentColAxis.Location.Y1;
                currentColAxis.width = (currentColAxis.Location.X2 - currentColAxis.Location.X1);
                currentColAxis.height = currentColAxis.Location.Y2 - currentColAxis.Location.Y1;

            }
        }
        
        //For Vertical
        isInitAxes = true, isOppInitAxes = true;
        var yaxisOrign = this._calRowSize();
        var rowDefinition = this.model._rowDefinitions;
        for (var i = 0, rowLength = rowDefinition.length; i < rowLength; i++) {
            var y1 = yaxisOrign[i];
            var y2 = yaxisOrign[i + 1];
            var nearIndex = 0;
            var farIndex = 0;
            var isFirstAxes = true, isOppFirstAxes = true;
           var x = this.model.m_AreaBounds.X;
           var norX = x, oppX = x + this.model.m_AreaBounds.Width;
           for (var j = 0, maxRow = rowDefinition[i].axis.length; j < maxRow; j++) {
                var axisSpace = 0;
                var realX = 0;
               
                var currentAxis = rowDefinition[i].axis[j];
               
               if (!currentAxis.opposedPosition) {
                   if (!(isFirstAxes && (isInitAxes || nearIndex==0))) {
                        axisSpace = rowDefinition.nearSizes[nearIndex];
                        realX = norX = (norX - axisSpace);
                        nearIndex++;
                    } else {
                        realX = x;
                        isFirstAxes = false;
                    }

                } else {
                   if (!(isOppFirstAxes && (isOppInitAxes|| isOppInitAxes==0))) {
                        axisSpace = rowDefinition.farSizes[farIndex];
                        realX = oppX = (oppX + axisSpace);
                        farIndex++;
                    } else {
                        realX = oppX;
                        isOppFirstAxes = false;
                    }
                }
                if (currentAxis.rowSpan) {
                    //Skip the axis size calculation for duplicate span axis
                    if (!(rowDefinition[i].index == currentAxis.rowIndex))
                        continue;
                    var axisRowSize = (rowLength >= (i + currentAxis.rowSpan)) ? i + currentAxis.rowSpan : i + 1;
                    y2 = yaxisOrign[axisRowSize];
                    if (!currentAxis.opposedPosition && nearIndex==0) isInitAxes = isFirstAxes;
                    if (currentAxis.opposedPosition && farIndex==0) isOppInitAxes = isOppFirstAxes;
                } else {
                    y2 = yaxisOrign[i + 1];
                    isInitAxes = true;
                    isOppInitAxes = true;
                }
                currentAxis.Location = { };
                currentAxis.Location.X1 = realX;
                currentAxis.Location.Y1 = y1 - currentAxis.plotOffset;
                currentAxis.Location.X2 = realX;
                currentAxis.Location.Y2 = y2 + currentAxis.plotOffset;
                currentAxis.x = currentAxis.Location.X1;
                currentAxis.y = currentAxis.Location.Y2;
                currentAxis.height = (currentAxis.Location.Y1 - currentAxis.Location.Y2);
                currentAxis.width = currentAxis.axisLine.width;

            }
        }
    },
    _arrangeAxis: function () {
        var chartobj = this;
        var axis;
        var customRow;
        //Generate columnIndex/RowIndex value if it is not specify in sample  
        for (var i = 0; i < this.model._axes.length; i++) {
            axis = this.model._axes[i];
            if (axis.orientation.toLowerCase() == "horizontal") {
                chartobj.model.hAxes.push(axis);
                axis.columnIndex = (ej.util.isNullOrUndefined(axis.columnIndex)) ? 0 : (axis.columnIndex);
                chartobj.model.hAxes[chartobj.model.hAxes.length - 1].columnIndex = axis.columnIndex;
            } else if (axis.orientation.toLowerCase() == "vertical") {

                chartobj.model.vAxes.push(axis);
                axis.rowIndex = (ej.util.isNullOrUndefined(axis.rowIndex)) ? 0 : (axis.rowIndex);
                chartobj.model.vAxes[chartobj.model.vAxes.length - 1].rowIndex = axis.rowIndex;
                customRow = (!ej.util.isNullOrUndefined(chartobj.model.rowDefinitions)) ? chartobj.model.rowDefinitions[axis.rowIndex] : null;
                if (customRow) {
                    axis.axisBottomLine = {};
                    axis.axisBottomLine.visible = true;
                    axis.axisBottomLine.lineWidth = customRow.lineWidth;
                    axis.axisBottomLine.color = customRow.lineColor;
                }
            }
        }

        this.model.hAxes = ej.DataManager(this.model.hAxes, ej.Query().sortBy("columnIndex")).executeLocal();

        this.model.vAxes = ej.DataManager(this.model.vAxes, ej.Query().sortBy("rowIndex")).executeLocal();
        
        //column axis information push into _rowDefinitions
        var rowCount = this.axesCount(chartobj.model.vAxes);
         chartobj.model._rowDefinitions = [];
        if (chartobj.model.vAxes.length>0) {
            for (var j = 0; j < rowCount.length; j++) {
                var rowCollection = chartobj.model.rowDefinitions;
                var rowDefinitions = ((rowCollection)) ? (rowCollection[j] ? rowCollection[j] : rowCollection[0]) : null;
                chartobj.model._rowDefinitions.push({axis: this.axesIndexCount(chartobj.model.vAxes, rowCount.indexValue[j])
                                , index: rowCount.indexValue[j], rowDefinitions: rowDefinitions });
            }
        }

        var rowDefinition = chartobj.model._rowDefinitions;
        //Using filter to get spanning row axis collection
        var spanningRow = this._axisFilter(rowDefinition, "rowSpan");
        //Arrange the entire axis for row span
        this._SpanningAxes(rowDefinition, spanningRow, true);

        //column axis information push into _columnDefinitions
        var columnCount = this.axesCount(chartobj.model.hAxes);
        chartobj.model._columnDefinitions = [];
        if (chartobj.model.hAxes.length > 0) {
            for (var k = 0; k < columnCount.length; k++) {
                var columnCollection = chartobj.model.columnDefinitions;
                var columnDefinitions = (columnCollection) ? (columnCollection[k] ? columnCollection[k] : columnCollection[0]) : null;
                chartobj.model._columnDefinitions.push({
                    axis: this.axesIndexCount(chartobj.model.hAxes, columnCount.indexValue[k])
                    , index: columnCount.indexValue[k], columnDefinitions: columnDefinitions });
            }
        }
        
        var columnDefinition = chartobj.model._columnDefinitions;
        //Using filter to get spanning column axis collection
        var spanningColumn = this._axisFilter(columnDefinition, "columnSpan");
        //Arrange the entire axis for column span
        this._SpanningAxes(columnDefinition, spanningColumn, false);
       
        this._axisSize();
        
        
    },
    
     //Arrange the entire axis for spanning
    _SpanningAxes: function (definition,spanningCollection, isRow) {
        for (var spanIndex = 0, spanMax = spanningCollection.length; spanIndex < spanMax; spanIndex++) {
            var currentAxis = spanningCollection[spanIndex];
             if (currentAxis == null) break;
             var span = isRow ? currentAxis.rowSpan : currentAxis.columnSpan;
             var index = isRow ? currentAxis.rowIndex : currentAxis.columnIndex;
            if(definition[index]!=undefined){
             var axisIndex = definition[index].axis.indexOf(currentAxis);
             for (var k = 1, m = index + 1; k < span && m < definition.length; k++, m++) {
                 if (definition[m].axis.length >= axisIndex) {
                     //Insert duplicate span row to appropriate rowdefinition
                     definition[m].axis.splice(axisIndex, 0, currentAxis);
                 }
             }
             }
         }
     },
    
     //Filter span axis from axes
    _axisFilter: function (definition, fieldName) {
        var axisResult = [];
        definition.filter(function (axes) {
             //Execute query to get spanning axis from axis array
              var axisCollection = ej.DataManager(axes["axis"], ej.Query().where(fieldName, ">", 1)).executeLocal();
              for (var i = 0, max = axisCollection.length; i < max; i++)
                  axisResult.push(axisCollection[i]);
        });
        return axisResult;
    },
    
    _axisSize: function () {
        //The below calcultion to get approximate length of axis
         var spaceValue = this._getLegendSpace();
         var hSpace = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right - (this.model.elementSpacing * 2) - spaceValue.leftLegendWidth - spaceValue.rightLegendWidth;
        var hWidth = Math.floor(hSpace / this.axesCount(this.model.hAxes).length);

        var vSpace = $(this.svgObject).height() - this.model.margin.top - this.model.margin.bottom - (this.model.elementSpacing * 2) - spaceValue.topLegendHeight - spaceValue.bottomLegendHeight - spaceValue.modelTitleHeight-spaceValue.modelsubTitleHeight;
        var vWidth = Math.floor(vSpace / this.axesCount(this.model.vAxes).length);

       

        for (var j = 0; j < this.model._axes.length; j++) {
            axis = this.model._axes[j];
            if (axis.orientation.toLowerCase() == "horizontal") {
                if (this.model.AreaType != "polaraxes") {
                    var customColumn = (!ej.util.isNullOrUndefined(this.model.columnDefinitions)) ? this.model.columnDefinitions[axis.columnIndex] : [];
                    var columnLength = hWidth;
                    if (customColumn) {
                        customColumn.unit = (customColumn.unit == undefined) ? "percentage" : customColumn.unit;
                        customColumn.columnWidth = (customColumn.columnWidth == undefined) ? 100 : customColumn.columnWidth;
                        if (customColumn.unit.toLowerCase() == "percentage") {
                            var length = this._axisColumnDefinitionSize(this.model.columnDefinitions);
                            columnLength = Math.floor((hWidth) * (customColumn.columnWidth / length));
                        }
                        else
                            columnLength = customColumn.columnWidth;

                    }
                    axis.length = axis.columnSpan ? axis.columnSpan * columnLength : columnLength;
                }
                else {
                    if (hSpace > vSpace*2)
                        axis.length = hSpace - vSpace;
                    else
                        axis.length = hSpace;
                }
            } else if (axis.orientation.toLowerCase() == "vertical") {
                if (this.model.AreaType != "polaraxes") {
                    var customRow = (!ej.util.isNullOrUndefined(this.model.rowDefinitions)) ? this.model.rowDefinitions[axis.rowIndex] : [];
                    var realLength = vWidth;
                    if (customRow) {
                        customRow.unit = (customRow.unit == undefined) ? "percentage" : customRow.unit;
                        customRow.rowHeight = (customRow.rowHeight == undefined) ? 100 : customRow.rowHeight;
                        if (customRow.unit.toLowerCase() == "percentage") {
                            var length = this._axisRowDefinitionSize(this.model.rowDefinitions);
                            realLength = Math.floor((vWidth) * (customRow.rowHeight / length));
                        }
                        else
                            realLength = customRow.rowHeight;

                    }
                    axis.length = axis.rowSpan ? (axis.rowSpan * realLength) : realLength;
                }
                else {
                    axis.length = vSpace / 2;
                }
            }
        }
    },
	_axisRowDefinitionSize: function (array) {
        var length = 0;
        if (array) {
            for (var i = 0; i < array.length; i++) {
                length = length + array[i].rowHeight;
            }
        } else {
            length = 100;
        }
        return length;
    },
    _axisColumnDefinitionSize: function (array) {
        var length = 0;
        if (array) {
            for (var i = 0; i < array.length; i++) {
                length = length + array[i].columnWidth;
            }
        } else {
            length = 100;
        }
        return length;
    },
   _axisMeasure: function (axis, realWidth, orientation,j) {
        var axisTitleHeight =0; 
        var labelSize = 0;
        axis.axisLine.width = axis.axisLine.width < 0 ? 0 : axis.axisLine.width;
		axis.majorTickLines.size = axis.majorTickLines.size < 0 ? 0 : axis.majorTickLines.size;
		var axisLineWidth = !this.model.enable3D && axis.axisLine.visible && axis.axisLine.width;
		var size = ej.EjSvgRender.utils._measureText(axis.title.text, realWidth, axis.title.font);
		if(orientation == "vertical")
		{
		 labelSize= axis._LableMaxWidth.width;
		 axisTitleHeight = (axis.title.text == "" || !(axis.title.visible) || !(axis.visible)) ? 0 : ((document.documentMode === 8) ? size.width : size.height + (2 * this.model.elementSpacing));
		}
		else
		{
		labelSize=(axis._LableMaxWidth.height);
		axisTitleHeight = (axis.title.text == "" || !(axis.title.visible) || !(axis.visible)) ? 0 : (size.height + (this.model.elementSpacing));
		}
		var width;		
		if ((j == 0) && (axis.tickLinesPosition == 'inside' && axis.labelPosition == 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + axisLineWidth + (this.model.elementSpacing);
		else if ((j == 0) && (axis.tickLinesPosition == 'inside' && axis.labelPosition != 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + axisLineWidth + (this.model.elementSpacing) + labelSize;
		else if ((j == 0) && (axis.tickLinesPosition != 'inside' && axis.labelPosition == 'inside'))
		    axis.AxisMaxWidth = axisTitleHeight + axisLineWidth + (this.model.elementSpacing) + axis.majorTickLines.size;
		else
		    axis.AxisMaxWidth = axisTitleHeight + axisLineWidth + this.model.elementSpacing + axis.majorTickLines.size + labelSize;
        return axis.AxisMaxWidth;
    },
     
    _arraySum: function (val) {
        var total = 0;
        for (var i = 0; i < val.length; i++) {
            total +=  parseFloat(val[i]) || 0;
        }
        return total;
    },
    _calSpace: function (realWidth, definitions,orientation) {
        var nearSizes = [];
        var farSizes = [];
        for (var i = 0; i < definitions.length; i++) {
            var nearIndex = 0, farIndex = 0;
            for (var j = 0; j < definitions[i].axis.length; j++) {
                var currentaxis = definitions[i].axis[j];
                var measureValue = this._axisMeasure(currentaxis, realWidth, orientation,j);
                if (!currentaxis.opposedPosition) {
                    if (nearSizes.length <= nearIndex) {
                        nearSizes.push(measureValue);
                    } else if (nearSizes[nearIndex] < (measureValue)) {
                        nearSizes[nearIndex] = measureValue;
                    }
                    nearIndex++;
                }
                else {
                    if (farSizes.length <= farIndex) {
                        farSizes.push(measureValue);
                    } else if (farSizes[farIndex] < (measureValue)) {
                        farSizes[farIndex] = measureValue;
                    }
                    farIndex++;
                }

            }
        }
        for (var i = 0; i < definitions.length; i++) {
            var nearIndex = 0;
            var labelsize = 0;
            for (var j = 0; j < definitions[i].axis.length; j++) {
                var currentoppaxis = definitions[i].axis[j];
                if (!currentoppaxis.opposedPosition) {
                    if (j != 0) {
                        if (currentoppaxis.orientation == "Vertical") {
                            labelSize = currentoppaxis._LableMaxWidth.width;
                            
                        }
                        else {
                            labelSize = currentoppaxis._LableMaxWidth.height;                            
                        }

                        if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            nearSizes[nearIndex] -= (labelSize + currentoppaxis.majorTickLines.size);
                            break;
                        }
                        else if (currentoppaxis.labelPosition != 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            nearSizes[nearIndex] -= currentoppaxis.majorTickLines.size;
                            break;
                        }
                        else if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition != 'inside') {
                            nearSizes[nearIndex] -= labelSize;
                            break;
                        }
                        else {
                            nearSizes[nearIndex] = nearSizes[nearIndex];
                            break;
                        }
                    }
                    else
                        break;
                }

            }
            break;
        }
        for (var i = 0; i < definitions.length; i++) {
            var farIndex = 0;
            for (var j = 0; j < definitions[i].axis.length; j++) {                
                var currentoppaxis = definitions[i].axis[j];                
                if (currentoppaxis.opposedPosition) {
                    if (j != 0) {
                        if (currentoppaxis.orientation == "Vertical") {
                            labelSize = currentoppaxis._LableMaxWidth.width;

                        }
                        else {
                            labelSize = currentoppaxis._LableMaxWidth.height;
                        }
                        if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            farSizes[farIndex] -= (labelSize + currentoppaxis.majorTickLines.size+5);
                            break;
                        }
                        else if (currentoppaxis.labelPosition != 'inside' && currentoppaxis.tickLinesPosition == 'inside') {
                            farSizes[farIndex] -= currentoppaxis.majorTickLines.size;
                            break;
                        }
                        else if (currentoppaxis.labelPosition == 'inside' && currentoppaxis.tickLinesPosition != 'inside') {
                            farSizes[farIndex] -= labelSize;
                            break;
                        }
                        else {
                            farSizes[farIndex] = farSizes[farIndex];
                            break;
                        }
                    }
                    else
                        break;
                }              
                
             }
            break;
        }
        return { nearSizes: nearSizes, farSizes: farSizes };
    },
    
    _getLegendSpace: function () {
        var chart = this.model,
            legend = chart.legend,
            position = legend.position.toLowerCase(),
            bounds = chart.LegendActualBounds,
            measureText = ej.EjSvgRender.utils._measureText,
            title = chart.title,
            subTitle = title.subTitle,
            elementSpacing = chart.elementSpacing,
            margin = chart.margin,
            svgWidth = chart.svgWidth,
            legendBorder = legend.border.width,
            ltheight = 0, space = 0,
            itemPadding = 10,
            leftLegendWidth = 0, rightLegendWidth = 0,
            topLegendHeight = 0, bottomLegendHeight = 0,
            modelTitleHeight = 0, modelsubTitleHeight = 0;
        if (legend.visible) {
            space = (bounds.Width + (itemPadding / 2) + elementSpacing + (2 * legendBorder));
            leftLegendWidth = position == 'left' ? space : 0;
            rightLegendWidth = position == 'right' ? space : 0;
            if (legend.title.text)
                ltheight = measureText(legend.title, null, legend.title.font).height;
            topLegendHeight = position == 'top' ? (bounds.Height + ltheight + (2 * legendBorder)) : 0;
            bottomLegendHeight = position == 'bottom' ? (bounds.Height + ltheight + (2 * legendBorder)) : 0;
        }            
        modelTitleHeight = (title.text == "") ? 0 : (measureText(title.text, svgWidth - margin.left - margin.right, title.font).height + elementSpacing);
        modelsubTitleHeight = (subTitle.text == "") ? 0 : (measureText(subTitle.text, svgWidth - margin.left - margin.right, subTitle.font).height + elementSpacing);

        return {
            leftLegendWidth: leftLegendWidth, rightLegendWidth: rightLegendWidth, topLegendHeight: topLegendHeight, bottomLegendHeight: bottomLegendHeight, modelTitleHeight: modelTitleHeight, modelsubTitleHeight: modelsubTitleHeight
        };
    },

    _calculateAreaBounds: function () {
        var chartobj = this;
        var realWidth = $(this.svgObject).width() - this.model.margin.left - this.model.margin.right;
        // Calculate area bounds X and width
        var chartBorderWidth = chartobj.model.border.width;

        //Calcultion for multiple axes(vertical)
        var sizes = this._calSpace(realWidth, chartobj.model._rowDefinitions,"vertical");
        chartobj.model._rowDefinitions.nearSizes = sizes.nearSizes;
        chartobj.model._rowDefinitions.farSizes = sizes.farSizes;
        var vAxesWidth = this._arraySum(sizes.nearSizes);
        var vAxesOppWidth = this._arraySum(sizes.farSizes);

        var spaceValue=this._getLegendSpace();
       
        var x = vAxesWidth + spaceValue.leftLegendWidth + chartobj.model.margin.left + chartBorderWidth;     
        var rightSpacing = vAxesOppWidth + spaceValue.rightLegendWidth + chartobj.model.margin.right + chartobj.model.margin.left + (2 * chartBorderWidth);
        var width = $(this.svgObject).width();
        var boundsWidth =  width - (x + rightSpacing);
        if (boundsWidth < 0 && this.model.AreaType == "cartesianaxes") {
            $(this.svgObject).width(width - boundsWidth + 1);
            this._calculateAreaBounds();
        }
        else {
        // Calculate area bounds Y and Height   
        
        //Calcultion for multiple axes(Horizontal)
        var columnSizes = this._calSpace(realWidth, chartobj.model._columnDefinitions,"horizontal");
        chartobj.model._columnDefinitions.nearSizes = columnSizes.nearSizes;
            chartobj.model._columnDefinitions.farSizes = columnSizes.farSizes;
            var hAxesWidth = this._arraySum(columnSizes.nearSizes);
            var hAxesOppWidth = this._arraySum(columnSizes.farSizes);
         
        var y = hAxesOppWidth  + this.model.margin.top+ (this.model.elementSpacing) + spaceValue.modelTitleHeight + (spaceValue.modelsubTitleHeight)  + spaceValue.topLegendHeight + chartBorderWidth;
            var bottomSpacing = hAxesWidth + this.model.margin.bottom + spaceValue.bottomLegendHeight + (2 * chartBorderWidth);
            var boundsHeight = Math.abs($(this.svgObject).height() - (y + bottomSpacing));

            this.model.m_AreaBounds = { X: x, Y: y, Width: boundsWidth, Height: boundsHeight };
            this.model.m_Spacing = { Left: x, Top: y, Right: rightSpacing, Bottom: bottomSpacing };

            if (this.model.enableCanvasRendering) {  // for adding bounds in canvas rendering
                this.canvasX = x;
                this.canvasY = y;
                this.canvasWidth = boundsWidth;
                this.canvasHeight = boundsHeight;
            } else 
                this.canvasX = this.canvasY = this.canvasHeight = this.canvasWidth = 0;
        }
      }
};

jQuery.uaMatch = function (ua) {
    ua = ua.toLowerCase();

    var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
        /(webkit)[ \/]([\w.]+)/.exec(ua) ||
        /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
        /(msie) ([\w.]+)/.exec(ua) ||
        ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
        [];

    return {
        browser: match[1] || "",
        version: match[2] || "0"
    };
};


 
})(jQuery);;
/**
* @fileOverview Plugin to style the Html Chart elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej) {

    ej.widget("ejChart", "ej.datavisualization.Chart", {
 
        element: null,
 
        model: null,

        validTags: ["div"],
 
        defaults: {

            border: {

                color: 'transparent',

                width: 0,

                opacity: 0.3
            },

            chartArea:
            {

                border: {

                    color: 'Gray',

                    width: 0.5,

                    opacity: 0.3
                },

                background: 'transparent'

            },

            primaryXAxis:
            {

                range: {

                    min: null,

                    max: null,

                    interval: null
                },

                labelPosition: 'outside',

                tickLinesPosition: 'outside',

                alternateGridBand:
                        {

                            odd: {

                                fill: "transparent",

                                opacity: 1
                            },

                            even: {

                                fill: "transparent",

                                opacity: 1
                            }
                        },

                majorGridLines:
                {

                    width: 1,

                    dashArray: "",

                    visible: true,

                    opacity: 1

                },

                majorTickLines:
               {

                   width: 1,

                   size: 5,

                   visible: true
               },

                minorGridLines:
               {

                   width: 1,

                   dashArray: "",

                   visible: true

               },

                minorTickLines:
                {

                    width: 1,

                    size: 5,

                    visible: true
                },

                minorTicksPerInterval: null,

                columnIndex: null,

                columnSpan: null,

                labelRotation: null,

                valueType: null,

                name: null,

                labelFormat: null,

                desiredIntervals: null,

                intervalType: null,

                roundingPlaces: null,

                logBase: 10,

                plotOffset: 0,
                labels: [],

                stripLine: [
                       {

                           visible: false,

                           startFromAxis: false,

                           text: "",

                           width: 0,

                           textAlignment: "middlecenter",

                           font: {

                               fontFamily: 'Segoe UI',

                               fontStyle: 'Normal',

                               size: '12px',

                               fontWeight: 'Regular',

                               color: 'black',

                               opacity: 1
                           },

                           start: null,

                           end: null,

                           color: 'gray',

                           borderColor: 'black',

                           zIndex: 'over'
                       }
                ],

                title:
               {


                   text: "",

				    visible:true,

                   enableTrim: false,

                   maximumTitleWidth: null,

                   font:
                   {

                       fontFamily: 'Segoe UI',

                       fontStyle: 'Normal',

                       size: '16px',

                       opacity: 1,

                       fontWeight: 'regular'
                   }
               },
                rangePadding: 'Auto',
                additionalPadding: [1, 1],
                orientation: 'Horizontal',

                maximumLabels: 3,

                opposedPosition: false,

                axisLine:
                {

                    visible: true,

                    width: 1,

                    dashArray: "",

                    offset: 0
                },

                labelIntersectAction: "none",

                maximumLabelWidth: 34,

                enableTrim: false,


                edgeLabelPlacement: "none",



                isInversed: false,



                font:
                {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    size: '13px',

                    fontWeight: 'regular',

                    opacity: 1
					
                },

                visible: true,

                crosshairLabel:
                {

                    visible: false
                },

                zoomFactor: 1,

                zoomPosition: 0
            },


            primaryYAxis:
              {

                  range: {

                      min: null,

                      max: null,

                      interval: null
                  },

                  labelPosition: 'outside',

                  tickLinesPosition: 'outside',

                  alternateGridBand:
                             {

                                 odd: {

                                     fill: "transparent",

                                     opacity: 1
                                 },

                                 even: {


                                     fill: "transparent",

                                     opacity: 1
                                 }
                             },

                  majorGridLines:
                 {

                     width: 1,

                     dashArray: "",

                     visible: true,

                     opacity: 1

                 },

                  majorTickLines:
                {

                    width: 1,

                    size: 5,

                    visible: true
                },

                  minorGridLines:
                 {

                     width: 1,

                     dashArray: "",

                     visible: true

                 },

                  minorTickLines:
                {

                    width: 1,

                    size: 5,

                    visible: true
                },

                  minorTicksPerInterval: null,

                  rowIndex: null,

                  rowSpan: null,

                  valueType: null,

                  name: null,

                  labelFormat: null,

                  desiredIntervals: null,

                  intervalType: null,

                  roundingPlaces: null,
                  labels: [],

                  title:
                  {

                      text: "",

					   visible:true,

                      enableTrim: false,

                      maximumTitleWidth: null,

                      font:
                     {

                         fontFamily: 'Segoe UI',

                         fontStyle: 'Normal',

                         opacity: 1,

                         size: '16px',

                         fontWeight: 'regular'
                     }
                  },

                  rangePadding: 'Auto',


                  stripLine: [
                       {

                           visible: false,

                           startFromAxis: false,

                           width: 0,

                           text: "",

                           textAlignment: "middlecenter",

                           font: {

                               fontFamily: 'Segoe UI',

                               fontWeight: 'Regular',

                               fontStyle: 'Normal',

                               size: '12px',

                               color: 'black',

                               opacity: 1
                           },

                           start: null,

                           end: null,

                           color: 'gray',

                           borderColor: 'black',

                           zIndex: 'over'
                       }
                  ],

                  logBase: 10,

                  plotOffset: 0,

                  orientation: 'Vertical',

                  maximumLabels: 3,

                  labelIntersectAction: "none",


                  maximumLabelWidth: 34,

                  enableTrim: false,

                  edgeLabelPlacement: "none",


                  isInversed: false,


                  crosshairLabel:
                 {

                     visible: false
                 },

                  visible: true,

                  opposedPosition: false,

                  font:
               {

                   fontFamily: 'Segoe UI',

                   fontStyle: 'Normal',

                   size: '13px',

                   opacity: 1,

                   fontWeight: 'regular'
				   
               },

                  axisLine:
              {

                  visible: true,

                  width: 1,

                  dashArray: "",

                  offset: 0
              },


                  zoomFactor: 1,

                  zoomPosition: 0
              },
            axes: [],

            secondaryX:
             {
                 range: {
                     min: null,
                     max: null,
                     interval: null
                 },
                 tickLinesPosition: "outside",
                 labelPosition: "outside",

                 majorGridLines:
        {

            width: 1,

            dashArray: "",

            visible: true
        },
                 alternateGridBand:
           {
               odd: {
                   fill: "transparent",
                   opacity: 1
               },
               even: {
                   fill: "transparent",
                   opacity: 1
               }
           },

                 majorTickLines:
         {

             width: 1,

             size: 5,

             visible: true
         },

                 minorGridLines:
         {

             width: 1,

             dashArray: "",

             visible: true

         },

                 minorTickLines:
        {

            width: 1,

            size: 5,

            visible: true
        },

                 minorTicksPerInterval: null,

                 columnIndex: null,

                 columnSpan: null,

                 labelRotation: null,

                 valueType: null,

                 name: null,

                 labelFormat: null,

                 desiredIntervals: null,

                 intervalType: null,

                 roundingPlaces: null,

                 logBase: 10,

                 plotOffset: 0,
                 labels: [],
                 stripLine: [
              {
                  visible: false,
                  startFromAxis: false,
                  text: "",
                  textAlignment: "middlecenter",
                  font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'regular', size: '12px', color: 'black', opacity: 1 },
                  start: null,
                  end: null,
                  color: 'gray',
                  borderColor: 'black',
                  zIndex: 'over'
              }
                 ],

                 title:
         {


             text: "",
             visible:true,
             enableTrim: false,
             maximumTitleWidth: null,
             font:
        {

            fontFamily: 'Segoe UI',

            fontStyle: 'Normal',

            size: '16px',

            opacity: 1,

            fontWeight: 'regular'
			
        }

         },

                 rangePadding: 'Auto',
                 additionalPadding: [1, 1],

                 orientation: 'Horizontal',

                 maximumLabels: 3,

                 opposedPosition: false,

                 axisLine:
          {

              visible: true,

              width: 1,

              dashArray: "",

              offset: 0
          },

                 labelIntersectAction: "none",

                 edgeLabelPlacement: "none",

                 font:
          {

              fontFamily: 'Segoe UI',

              fontStyle: 'Normal',

              size: '13px',

              fontWeight: 'regular',

              opacity: 1
          },

                 visible: true,

                 crosshairLabel:
           {

               visible: false
           },

                 zoomFactor: 1,

                 zoomPosition: 0
             },

            secondaryY:
                {
                    range: {
                        min: null,
                        max: null,
                        interval: null
                    },
                    tickLinesPosition: "outside",
                    labelPosition: "outside",

                    majorGridLines:
       {

           width: 1,

           dashArray: "",

           visible: true

       },
                    alternateGridBand:
           {
               odd: {
                   fill: "transparent",
                   opacity: 1
               },
               even: {
                   fill: "transparent",
                   opacity: 1
               }
           },

                    majorTickLines:
                        {

                            width: 1,

                            size: 5,

                            visible: true
                        },

                    minorGridLines:
                        {

                            width: 1,

                            dashArray: "",

                            visible: true

                        },

                    minorTickLines:
                        {

                            width: 1,

                            size: 5,

                            visible: true
                        },

                    minorTicksPerInterval: null,

                    rowIndex: null,

                    rowSpan: null,

                    valueType: null,

                    name: null,

                    labelFormat: null,

                    desiredIntervals: null,

                    intervalType: null,

                    roundingPlaces: null,
                    labels: [],

                    title:
                    {

                        text: "",
                        visible:true,
                        enableTrim: false,
                        maximumTitleWidth: null,
                        font:
                        {
                            fontFamily: 'Segoe UI',

                            fontStyle: 'Normal',

                            opacity: 1,

                            size: '16px',

                            fontWeight: 'regular'
                        }
                    },

                    rangePadding: 'Auto',

                    stripLine: [
                         {
                             visible: false,
                             startFromAxis: false,
                             text: "",
                             textAlignment: "middlecenter",
                             font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'regular', size: '12px', color: 'black', opacity: 1 },
                             start: null,
                             end: null,
                             color: 'gray',
                             borderColor: 'black',
                             zIndex: 'over'
                         }
                    ],

                    logBase: 10,

                    plotOffset: 0,

                    orientation: 'Vertical',

                    maximumLabels: 3,

                    labelIntersectAction: "none",

                    edgeLabelPlacement: "none",

                    crosshairLabel:
                    {

                        visible: false
                    },

                    visible: true,

                    opposedPosition: false,

                    font:
                    {
                        fontFamily: 'Segoe UI',

                        fontStyle: 'Normal',

                        size: '13px',

                        opacity: 1,

                        fontWeight: 'regular'
						
                    },

                    axisLine:
                        {

                            visible: true,

                            width: 1,

                            dashArray: "",

                            offset: 0
                        },


                    zoomFactor: 1,

                    zoomPosition: 0
                },
            trendlineDefaults: {
                type: "linear",
                visibility: "",
                name: "Trendline",
                fill: "",
                width: 1,
                opacity: 1,
                dashArray: "",
                forwardForecast: 0,
                backwardForecast: 0,
                polynomialOrder: 2
            },
            indicatorDefaults: {
                visible: true,
                points: [],
                tooltip: { visible: false, format: "#point.x# : #point.y#", fill: null, border: { width: 1, color: null }, duration: '500ms', enableAnimation: true, opacity: 0.95 },
                seriesName: "",
                font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 },
                type: "SMA",
                period: 14,
                standardDeviations: 2,
                kPeriod: 3,
                dPeriod: 3,
                periodLine: { fill: "blue", width: 2 },
                lowerLine: { fill: "#008000", width: 2 },
                upperLine: { fill: "#ff0000", width: 2 },
                macdLine: { fill: "#ff9933", width: 2 },
                histogram: { fill: "#ccccff", opacity: 1, border: { color: "#9999ff", width: 1 } },
                fill: "#00008B",
                width: 2,
                xAxisName: "",
                yAxisName: "",
                visibility: "visible",
                macdType: "line",
                shortPeriod: 12,
                longPeriod: 26,
                trigger: 9,
                enableAnimation: false
            },
            annotationsDefault: {
                visible: false,
                content: "",
                coordinateUnit: "none",
                verticalAlignment: "middle",
                horizontalAlignment: "middle",
                region: "chart",
                x: 0,
                y: 0,
                opacity: 1,
                angle: 0,
                xAxisName: "",
                yAxisName: "",
                margin: {
                    left: 0,
                    right: 0,
                    bottom: 0,
                    top: 0
                }
            },
            stripLineDefault: {

                visible: false,

                startFromAxis: false,

                width: 0,

                text: "",

                textAlignment: 'middlecenter',

                font:
                {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    fontWeight: 'Regular',

                    size: '12px',

                    color: 'black',

                    opacity: 1
                },

                start: null,

                end: null,

                color: 'gray',

                borderColor: 'black',

                zIndex: 'over',

                borderWidth: 1
            },

            rowDefinitions: null,

            columnDefinitions: null,

            title:
				{

				    text: "",

				    textAlignment: "center",

				    font: {

				        fontFamily: 'Segoe UI',

				        fontStyle: 'Normal',

				        size: '20px',

				        opacity: 1,

				        fontWeight: 'regular'
				    },


				    subTitle:
                           {

                               text: "",

                               textAlignment: 'far',

                               font:
                               {

                                   fontFamily: 'Segoe UI',

                                   fontStyle: 'Normal',

                                   size: '12px',

                                   opacity: 1,

                                   fontWeight: 'regular'
                               }
                           }

				},





            lineCap: { butt: 'butt', round: 'round', square: 'square' },
            lineJoin: { round: 'round', bevel: 'bevel', miter: 'miter' },
            legendAlignment: { near: 'near', center: 'center', far: 'far' },
            legendPosition: { top: 'top', bottom: 'bottom', right: 'right', left: 'left', custom: 'custom' },

            enableAnimation: true,

            legend:
            {

                title:
                      {
                          text: "",

                          textAlignment: 'center',

                          font:
                              {

                                  size: '12px',

                                  fontFamily: "Segoe UI",

                                  fontStyle: 'normal',

                                  fontWeight: 'regular'
                              }
                      },

                border:
                {

                    color: 'transparent',


                    width: 1
                },

                visible: true,

                toggleSeriesVisibility: true,
				
				enableScrollbar: true,

                itemPadding: 10,

                shape: 'None',

                alignment: 'Center',

                position: 'Bottom',

                location:
                {

                    x: 0,

                    y: 0
                },

                itemStyle:
                {
                    height: 10,

                    width: 10,

                    border:
                    {

                        color: 'transparent',

                        width: 1
                    }
                },

                rowCount: null,

                columnCount: null,

                opacity: 1,

                fill: null,

                background: null,

                font:
                {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    fontWeight: 'Regular',

                    size: '12px'
                },

                size:
                    {

                        height: null,

                        width: null
                    }
            },

            indicators: [
              {
                  points: [],

                  tooltip:
				  {

				      visible: false,

				      format: "#point.x# : #point.y#",

				      fill: null,

				      border: {

				          width: 1,

				          color: null
				      },

				      duration: '500ms',

				      enableAnimation: true,

				      opacity: 0.95
				  },

                  seriesName: "",

                  type: "SMA",

                  period: 14,

                  standardDeviations: 2,

                  kPeriod: 3,

                  dPeriod: 3,

                  periodLine: {

                      fill: "blue",

                      width: 2
                  },

                  lowerLine: {


                      fill: "#008000",

                      width: 2
                  },

                  upperLine: {

                      fill: "#ff0000",

                      width: 2
                  },

                  macdLine: {

                      fill: "#ff9933",

                      width: 2
                  },

                  histogram: {

                      fill: "#ccccff",

                      opacity: 1,

                      border: {

                          color: "#9999ff",

                          width: 1
                      }
                  },

                  fill: "#00008B",

                  visible: true,

                  font: {

                      fontFamily: 'Segoe UI',

                      fontStyle: 'Normal',

                      fontWeight: 'Regular',

                      size: '12px',

                      color: '#707070',

                      opacity: 1
                  },

                  width: 2,

                  xAxisName: "",

                  yAxisName: "",

                  macdType: "line",

                  shortPeriod: 12,

                  longPeriod: 26,

                  trigger: 9,

                  visibility: "visible",

                  enableAnimation: false
              }
            ],


            backGroundImageUrl: null,

            annotations: [{

                visible: false,

                content: "",

                coordinateUnit: "none",

                verticalAlignment: "middle",

                horizontalAlignment: "middle",

                region: "chart",

                x: 0,

                y: 0,

                opacity: 1,

                angle: 0,

                xAxisName: "",

                yAxisName: "",

                margin: {

                    left: 0,

                    right: 0,

                    bottom: 0,

                    top: 0
                }
            }],

            series: [
                {

                    type: 'column',

                    enableAnimation: false,

                    isClosed: true,

					 isTransposed:false,

                    isStacking: true,

                    drawType: 'line',

                    doughnutCoefficient: 0.4,

                    explodeOffset: 25,

                    pyramidMode: 'linear',

                    funnelWidth: '11.6%',

                    funnelHeight: '32.7%',

                    highlightSettings: {

                        enable: false,

                        mode: 'series',

                        color: "",

                        opacity: 0.6,

                        border: {

                            color: "",

                            width: 2
                        },

                        pattern: 'none',

                        customPattern: ""
                    },

                    selectionSettings: {

                        enable: false,

                        mode: 'series',

                        color: "",

                        opacity: 0.6,

                        border: {

                            color: "",

                            width: 2
                        },

                        pattern: 'none',

                        customPattern: ""
                    },

                    labelPosition: 'inside',

                    gapRatio: 0,

                    points: null,

                    pieCoefficient: 0.8,

                    doughnutSize: 0.8,

                    dataSource: null,

                    high: '',

                    low: '',

                    open: '',

                    close: '',

                    bullFillColor: null,

                    bearFillColor: null,

                    query: null,

                    xName: '',

                    yName: '',

                    pointColorMappingName: '',



                    visibility: "visible",

                    startAngle: null,

                    endAngle: null,

                    xAxisName: null,

                    yAxisName: null,

                    explodeAll: null,

                    explode: false,

                    explodeIndex: null,

                    enableSmartLabels: null,

                    tooltip: {

                        visible: false,

                        format: "#point.x# : #point.y#",

                        template: null,


                        fill: null,

                        border: {

                            width: 1,

                            color: null
                        },

                        enableAnimation: true,

                        duration: '500ms',

                        opacity: 0.95
                    },

                    fill: null,

                    opacity: 1,

                    lineCap: 'butt',

                    lineJoin: 'round',

                    dashArray: "",

                    border: {

                        width: 1,

                        color: 'transparent'
                    },
                    width: 2,

                    marker: {

                        shape: 'circle',

                        imageUrl: "",

                        size: {

                            width: 6,

                            height: 6
                        },

                        visible: false,

                        opacity: 1,

                        fill: null,

                        border: {

                            color: 'white',

                            width: 3
                        },

                        dataLabel: {

                            visible: false,

                            angle: 0,

                            textMappingName:'',

                            verticalTextAlignment: 'center',

                            horizontalTextAlignment: 'center',

                            textPosition: 'top',

                            shape: 'none',

                            opacity: 1,

                            fill: null,

                            border: {

                                color: 'white',

                                width: 0.1
                            },

                            connectorLine: {

                                width: 0.5,

                                type: 'line',

                                color: null
                            },

                            offset: {

                                x: 0,

                                y:0
                            },

                            font: {

                                fontFamily: 'Segoe UI',

                                fontStyle: 'Normal',

                                fontWeight: 'Regular',

                                size: '12px',

                                opacity: 1
                            },

                            margin: {

                                left: 5,

                                top: 5,

                                bottom: 5,

                                right: 5
                            }

                        }
                    },


                    trendlines: [{

                        visibility: "",

                        type: "linear",

                        name: "Trendline",

                        fill: "",

                        width: 1,

                        opacity: 1,

                        dashArray: "",

                        forwardForecast: 0,

                        backwardForecast: 0,

                        polynomialOrder: 2
                    }],


                    font: {

                        fontFamily: 'Segoe UI',

                        fontStyle: 'Normal',

                        fontWeight: 'Regular',

                        size: '12px',

                        color: '#707070',

                        opacity: 1
                    },
                    _isdesigntime: true

                }
            ],

            commonSeriesOptions: {

                type: 'column',

                enableAnimation: false,

                isClosed: true,

				 isTransposed:false,

                isStacking: false,

                drawType: 'line',

                doughnutCoefficient: 0.4,

                explodeOffset: 25,

                pyramidMode: 'linear',

                funnelWidth: '11.6%',

                funnelHeight: '32.7%',

                highlightSettings: {

                    enable: false,

                    mode: 'series',

                    color: "",

                    opacity: 0.6,

                    border: {

                        color: "",

                        width: 2
                    },

                    pattern: 'none',

                    customPattern: ""
                },

                selectionSettings: {

                    enable: false,

                    mode: 'series',

                    color: "",

                    opacity: 0.6,

                    border: {

                        color: "",

                        width: 2
                    },

                    pattern: 'none',

                    customPattern: ""
                },

                labelPosition: 'inside',

                gapRatio: 0,

                pieCoefficient: 0.8,

                doughnutSize: 0.8,

                dataSource: null,

                xName: '',

                yName: '',

                pointColorMappingName:'',

                high: '',

                low: '',

                open: '',

                close: '',

                startAngle: null,

                endAngle: null,

                xAxisName: null,

                yAxisName: null,

                explodeAll: false,

                explode: false,

                explodeIndex: null,

                enableSmartLabels: null,

                tooltip: {

                    visible: false,

                    format: "#point.x# : #point.y#",

                    template: null,

                    fill: null,

                    border: {

                        color: null,

                        width: 1
                    },

                    enableAnimation: true,

                    duration: '500ms',

                    opacity: 0.95
                },

                fill: null,

                opacity: 1,

                lineCap: 'butt',

                lineJoin: 'round',

                dashArray: "",

                border: {

                    color: 'transparent',

                    width: 1
                },
                width: 2,

                marker:
                {

                    shape: 'circle',

                    imageUrl: "",

                    size: {

                        width: 6,

                        height: 6
                    },

                    visible: false,

                    opacity: 1,

                    fill: null,

                    border: {

                        color: 'white',

                        width: 3
                    },

                    dataLabel:
                    {

                        visible: false,

                        angle: 0,

                        textMappingName:'',

                        verticalTextAlignment: 'center',

                        horizontalTextAlignment: 'center',

                        textPosition: 'top',

                        shape: 'none',


                        opacity: 1,

                        fill: null,

                        border: {

                            color: 'white',

                            width: 0.1
                        },

                        offset: {

                            x: 0,

                            y: 0
                        },

                        connectorLine:
                        {

                            width: 0.5,

                            type: 'line'
                        },

                        font:
                        {

                            fontFamily: 'Segoe UI',

                            fontStyle: 'Normal',

                            fontWeight: 'Regular',

                            size: '12px',

                            opacity: 1
                        },

                        margin:
                        {

                            left: 5,

                            top: 5,

                            bottom: 5,

                            right: 5
                        }
                    }
                },

                trendlines: [{

                    visibility: "",

                    type: "linear",

                    name: "Trendline",

                    fill: "",

                    width: 1,

                    opacity: 1,

                    dashArray: "",

                    forwardForecast: 0,

                    backwardForecast: 0,

                    polynomialOrder: 2
                }],

                font: {

                    fontFamily: 'Segoe UI',

                    fontStyle: 'Normal',

                    fontWeight: 'Regular',

                    size: '12px',

                    color: '#707070',

                    opacity: 1
                }
            },

            crosshair:
            {

                marker:
                {

                    visible: true,

                    size:
                    {

                        width: 10,

                        height: 10
                    },

                    opacity: 1,

                    border:
                    {

                        width: 3
                    }
                },

                visible: false,

                type: "crosshair"
            },


            pointStyle:
            {
                lineCap: 'butt', lineJoin: 'round', opacity: 1, interior: null, borderColor: null, borderWidth: 1
            },
            textStyle:
            {
                marker: { textAlignment: 'center', textPosition: 'top' },
                font: { fontFamily: 'Segoe UI', fontStyle: 'Normal', fontWeight: 'Regular', size: '12px', color: '#707070', opacity: 1 }
            },
            symbolShape: {
                None: 0,
                LeftArrow: 1,
                RightArrow: 2,
                Circle: 3,
                Cross: 4,
                HorizLine: 5,
                VertLine: 6,
                Diamond: 7,
                Rectangle: 8,
                Triangle: 9,
                InvertedTriangle: 10,
                Hexagon: 11,
                Pentagon: 12,
                Star: 13,
                Ellipse: 14,
                Wedge: 15,
                Trapezoid: 16,
                UpArrow: 17,
                DownArrow: 18,
                Image: 19,
                SeriesType: 20

            },
            initSeriesRender: true,

            theme: "flatlight",

            canResize: false,

            enable3D: false,

            enableRotation: false,

            sideBySideSeriesPlacement: null,

            perspectiveAngle: 90,

            rotation: 0,

            tilt: 0,

            wallSize: 2,

            depth: 100,

            enableCanvasRendering: false,

            zooming:
            {

                enable: false,

                type: 'x,y',

                enableMouseWheel: false,

                enableDeferredZoom: false
            },
            type: 'x,y',

            locale: "en-US",

            xZoomFactor: 1,

            yZoomFactor: 1,

            xZoomPosition: 0,

            yZoomPosition: 0,

            load: "",

            axesLabelRendering: "",

            axesRangeCalculate: "",

            axesTitleRendering: "",

            chartAreaBoundsCalculate: "",

            legendItemRendering: "",

            legendBoundsCalculate: "",

            preRender: "",

            seriesRendering: "",


            trendlineRendering: "",

            symbolRendering: "",

            titleRendering: "",
            subtitleRendering: "",

            axesLabelsInitialize: "",

            pointRegionClick: "",

            seriesRegionClick: "",

            pointRegionMouseMove: "",

            legendItemClick: "",

            legendItemMouseMove: "",

            displayTextRendering: "",

            toolTipInitialize: "",

            trackAxisToolTip: "",

            trackToolTip: "",

            animationComplete: "",

            zoomed:"",

            destroy: "",

            create: "",

            margin: { left: 10, right: 10, top: 10, bottom: 10 },
            size: { width: null, height: null },
            elementSpacing: 10,

            dateStart: 25568.791666666668

        },

        dataTypes: {
            commonSeriesOptions: { dataSource: "data" },
            axes: "array",
            series: "array",
            annotations: "array",
            indicators: "array",
            seriesColors: "array",
            palette: "array",
            rowDefinitions: "array",
            columnDefinitions: "array",
            seriesBorderColors: "array",
            pointColors: "array",
            pointBorderColors: "array",
            initSeriesRender: "boolean",
            theme: "enum",
            canResize: "boolean",
            elementSpacing: "number",
            primaryXAxis: {
                labels: "array",
                stripLine: "array",
                orientation: "enum",
                rangePadding: "enum",
                tickLinesPosition: "enum",
                labelPosition: "enum",
                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number"
            },
            primaryYAxis: {
                labels: "array",
                stripLine: "array",
                orientation: "enum",
                rangePadding: "enum",

                opposedPosition: "boolean",
                zoomFactor: "number",
                zoomPosition: "number"
            },
            legend: {

                shape: "enum",
                alignment: "enum",
                position: "enum",
                itemPadding: "number"
            },
            zooming: {
                enable: "boolean",
                type: "string",
                enableMouseWheel: "boolean"
            },
            size: {
                width: "string",
                height: "string"
            }

        },

        observables: ["xZoomFactor", "yZoomFactor", "xZoomPosition", "yZoomPosition"],

        _tags: [{
            tag: "series",
            attr: ["xAxisName", "yAxisName", "endAngle", "startAngle", "explodeIndex", "labelPosition", "xName", "yName","pointColorMappingName", "pyramidMode", 
                   "pieCoefficient", "explodeAll", "explodeOffset", "funnelWidth", "funnelHeight", "gapRatio", "isClosed","isTransposed", 
                   "isStacking", "bearFillColor", "bullFillColor", "dataSource", "enableAnimation", "doughnutCoefficient", "doughnutSize", 
                   "enableSmartLabels", "drawType",
			       [
				     {
				         tag: "points", attr: ["x", "y", "text", "textMappingName", "isEmpty", "fill", "visible"]
				     }
			       ],
            ],
            singular: "series"
        }, {
            tag: "axes",
            attr: ["columnIndex", "rowIndex", "desiredIntervals", "edgeLabelPlacement", "intervalType", "labelFormat", "labelPosition", "labelRotation", "logBase", "labelIntersectAction", "opposedPosition", "plotOffset", "rangePadding", "roundingPlaces", "valueType", "zoomFactor", "zoomPosition", "orientation",
              [
                {
                    tag: "stripLine", attr: ["borderColor", "startFromAxis", "zIndex", "textAlignment"],
                    singular: "stripLine"
                }
              ],
            ],
            singular: "axis"
        },
              {
                  tag: "primaryXAxis.stripLine",
                  attr: ["borderColor", "startFromAxis", "zIndex", "textAlignment"],
                  singular: "primaryXAxis.stripLine"
              },
            {
                tag: "primaryYAxis.stripLine",
                attr: ["borderColor", "startFromAxis", "zIndex", "textAlignment"],
                singular: "primaryYAxis.stripLine"
            },
           {
               tag: "indicators",
               attr: ["seriesName", "type", "xName", "period", "xAxisName", "visibility", "macdType", "shortPeriod", "longPeriod", "trigger", "enableAnimation", "yAxisName", "standardDeviations", "kPeriod", "dPeriod", "periodLine", "lowerLine", "upperLine", "macdLine", "histogram",
                  [
                    {
                        tag: "points", attr: []
                    }
                  ]
               ],
               singular: "indicator"
           },
          {
              tag: "annotations",
              attr: ["visible", "content", "coordinateUnit", "opacity", "verticalAlignment", "horizontalAlignment", "region", "x", "y", "angle", "xAxisName", "yAxisName", "margin"],
              singular: "annotation"
          }
        ],
        _xZoomFactor: ej.util.valueFunction("xZoomFactor"),
        _yZoomFactor: ej.util.valueFunction("yZoomFactor"),
        _xZoomPosition: ej.util.valueFunction("xZoomPosition"),
        _yZoomPosition: ej.util.valueFunction("yZoomPosition"),


        _init: function () {

            this._renderSfChart();

        },
        _destroy: function () {
            $('#template_group_' + this._id).remove();
            $("#annotation_group_" + this._id).remove();
            $(this.element).removeClass("e-chart e-js").find("#" + this.svgObject.id).remove();
            $(this.element).removeClass("e-chart e-js").find("#legend_" + this.svgObject.id).remove();
        },

        _series: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _series_points: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _primaryXAxis_stripLine: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _primaryYAxis_stripLine: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _axes: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },
        _axes_stripLine: function (index, property, value, old) {
            this.redraw();
            this._trigger("refresh");
        },

        _ignoreOnExport: ["dataSource"],


        _removeZoomkit: function () {
            if (this.model.zooming.enable) {
                $('#' + this.svgObject.id + '_ResetZoom').remove();
                $('#' + this.svgObject.id + '_PanBtn').remove();
                $('#' + this.svgObject.id + '_ZoomBtn').remove();
            }
        },


        redraw: function (excludeDataUpdate) {
            if (this.model.enableCanvasRendering) {
                if (this.model.zooming.enable && this.svgRenderer.ctx)
                    this.svgRenderer.ctx.clearRect(0, 0, ej.util.isNullOrUndefined(this.model.size.width) ? $("#" + this._id).width() : this.model.size.width, ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height);
                else {
                    $(this.svgObject).remove();
                    var width = this.svgWidth;
                    $("#canvas_trackSymbol").remove();
                    this.svgRenderer = new ej.EjCanvasRender(this.element); // to create canvas container
                    this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
                    this.svgRenderer.svgObj.width = ej.util.isNullOrUndefined(this.model.size.width) ? width : this.model.size.width;
                    this.svgObject = this.svgRenderer.svgObj;
                    var ctx = this.svgRenderer.svgObj.getContext("2d");

                    $(this.legendSvgContainer).remove();
                    $(this.legendContainer).remove();
                    this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
                    this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                    this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
                    $(this.legendContainer).appendTo(this.chartContainer);
                }
            }
            else if ((this.svgObject.id).indexOf("_canvas") != -1) {
                this._removeZoomkit();
                $(this.svgObject).remove();
                this.svgRenderer = new ej.EjSvgRender(this.element);  // to create svg container
                this.svgObject = this.svgRenderer.svgObj;
                this.canvasX = this.canvasY = 0;

                $(this.legendSvgContainer).remove();
                $(this.legendContainer).remove();
                this.legendContainer = $("<div></div>").attr('id', "legend_" + this._id).css("height", "0px");
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
                $(this.legendContainer).appendTo(this.chartContainer);
            } else {
                $(this.svgObject).empty();              // using existing svg container			   
                $(this.legendSvgContainer).empty();
            }
            $("#annotation_group_" + this._id).remove();   // to remove annotation on redraw
            if (!excludeDataUpdate) {

                if (this.model.canResize)
                    this.bindResizeEvents();
                else
                    this.removeResizeEvents();
            }
            this.bindTo(excludeDataUpdate);
            if (this.model._selectedData) {
                var selectedData = this.model._selectedData;
                var data;
                var length = selectedData.length;
                for (var i = 0; i < length; i++) {
                    data = selectedData[i];
                    if (data) {
                        if (!data.isLegend)
                            this.segmentSelection(data.event, data.legendData, data.commonArgs, data.pointArgs, data.index, data.data);
                        else
                            this.legendSelection(this, data.legendData, data.event, data.data);
                    }
                }                
            }
        },


        "export": function (type, action, multipleExport) {
            if (type && type.toLowerCase() == "excel") {
                var modelClone = $.extend(true, {}, this.model);
                var attr = { action: action, method: 'post' };
                var form = ej.buildTag('form', "", null, attr);
                var series;
                var chartobj = this;
                if (this._ignoreOnExport) {
                    series = modelClone.series;
                    for (var j = 0; j < series.length; j++) {
                        delete series[j].dataSource;
                        delete series[j].query;
                        series[j].fill = jQuery.type(series[j].fill) == "array" ? series[j].fill[0].color : series[j].fill;
                    }
                }
                if (ej.raiseWebFormsServerEvents) {
                    var args = { model: modelClone, originalEventType: "excelExporting" };
                    var clientArgs = { model: JSON.stringify(modelClone) };
                    ej.raiseWebFormsServerEvents("excelExporting", args, clientArgs);
                }
                else {
                    if (multipleExport) {
                        var chartObjectArray = {};
                        $('body').find('.e-datavisualization-chart').each(function (index, object) {
                            var chartObject = $(object).data('ejChart');
                            if (!ej.isNullOrUndefined(chartObject)) {
                                var modelClone = JSON.parse(JSON.stringify(chartObject.model));
                                if (chartobj._ignoreOnExport) {
                                    series = modelClone.series;
                                    for (var j = 0; j < series.length; j++) {
                                        delete series[j].dataSource;
                                        delete series[j].query;
                                        series[j].fill = jQuery.type(series[j].fill) == "array" ? series[j].fill[0].color : series[j].fill;
                                    }

                                    chartObjectArray[index] = JSON.stringify(modelClone);
                                    var inputAttr = { name: 'ChartModel', type: 'hidden', value: JSON.stringify(modelClone) }
                                    var input = ej.buildTag('input', "", null, inputAttr);
                                    form.append(input);
                                }
                            }
                        });
                    }
                    else {
                        var inputAttr = { name: 'ChartModel', type: 'hidden', value: JSON.stringify(modelClone) }
                        var input = ej.buildTag('input', "", null, inputAttr);
                        form.append(input);
                        form.append(this);
                    }
                    $('body').append(form);
                    form.submit();
                }
                return true;
            }
            else {
                var exportChart = this.exportChart();
                return exportChart;
            }
        },

        exportChart: function () {
            var Id = this._id;
            if ($("#Export_" + Id).length <= 0) {

                var Id = this._id;
                var chart = document.getElementById(Id + '_canvas');
                var legend = document.getElementById('legend_' + Id + '_canvas');
                var selectionChart = $('[id*=' + Id + '_Selection_' + ']');

                var exportChart = document.createElement('canvas');
                exportChart.setAttribute('id', 'Export_' + Id);
                exportChart.setAttribute('width', this.svgWidth);
                exportChart.setAttribute('height', this.svgHeight);

                //$("body").append(exportChart);
                var exprtCtx = exportChart.getContext("2d");

                var legendPos = this.model.LegendBounds;
                exprtCtx.drawImage(chart, 0, 0);

                if (legendPos)
                    exprtCtx.drawImage(legend, legendPos.X, legendPos.Y);
                // export for selection
                for (var i = 0; i < selectionChart.length; i++) {
                    var containerStyle = document.getElementById(selectionChart[i].id).getBoundingClientRect();
                    exprtCtx.drawImage(selectionChart[i], 0, 0);
                }
                //$("#Export_" +Id).css('display', 'none');
            }
            return exportChart;
        },

        animate: function (options) {
            if (!options) { // Animate the entire series and indicators "previous behavior"
                var series = this.model.series;
                for (var i = 0; i < series.length; i++) {
                    series[i]._animatedSeries = false;
                    series[i]._animatedTrendline = false;
                    series[i].AnimationComplete = false; //pie doughnut animation with datalabel
                }
                var indicators = this.model.indicators;
                for (var j = 0; j < indicators.length; j++) {
                    indicators[j]._animatedSeries = false;
                    indicators[j]._animatedTrendline = false;
                }
            } else if (options.constructor === Array) { // Animate array of series
                for (var i = 0; i < options.length; i++) {
                    options[i]._animatedSeries = false;
                    options[i]._animatedTrendline = false;
                    options[i].AnimationComplete = false; //pie doughnut animation with datalabel
                }
            } else {   // Animate the specific series
                options._animatedSeries = false;
                options._animatedTrendline = false;
                options.AnimationComplete = false;  //pie doughnut animation with datalabel
            }
            this.redraw();
        },


        _setModel: function (options) {
            var series;
            for (var prop in options) {
                this.disableAnimation();
                switch (prop) {
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this.setTheme(ej.EjSvgRender.themes, this.model.theme);
                        $.extend(true, this.model.primaryXAxis, ej.EjSvgRender.themes[this.model.theme].primaryXAxis);
                        $.extend(true, this.model.primaryYAxis, ej.EjSvgRender.themes[this.model.theme].primaryYAxis);
                        for (var k = 0; k < this.model.axes.length; k++) {
                            if (this.model.axes[k].orientation && this.model.axes[k].orientation.toLowerCase() == "horizontal")
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryX);
                            else
                                $.extend(true, this.model.axes[k], ej.EjSvgRender.themes[this.model.theme].secondaryY);
                        }
                        for (var l = 0; l < this.model.series.length; l++) {
                            $.extend(true, this.model.series[l], ej.EjSvgRender.themes[this.model.theme].commonSeriesOptions);
                        }
                        break;
                    case "commonSeriesOptions":
                        for (var i = 0; i < this.model.series.length; i++)
                            $.extend(true, this.model.series[i], {}, options[prop]);
                        break;
                    case "series":
                        series = this.model.series;
                        $.extend(true, series, {}, options[prop]);
                        for (var i = 0, len = series.length; i < len; i++) {
                            if (series[i].enableAnimation)
                                series[i]._animatedSeries = false;
                        }
                        break;
                    case "legend":
                        $.extend(true, this.model.legend, {}, options[prop]);
                        break;
                    case "axes":
                        $.extend(true, this.model.axes, {}, options[prop]);
                        break;
                    case "primaryXAxis":
                        this.model.primaryXAxis.setRange = (options[prop].range) ? true : false;
                        this.model.primaryXAxis.actual_Range = (this.model._axes[0].setRange) ? null : this.model._axes[0].actual_Range;
                        $.extend(true, this.model.primaryXAxis, {}, options[prop]);
                        break;
                    case "primaryYAxis":
                        this.model.primaryYAxis.setRange = (options[prop].range) ? true : false;
                        this.model.primaryYAxis.actual_Range = (this.model._axes[1].setRange) ? null : this.model._axes[1].actual_Range;
                        $.extend(true, this.model.primaryYAxis, {}, options[prop]);
                        break;
                    case "xZoomFactor":
                        this.model.primaryXAxis.zoomFactor = this._xZoomFactor();
                        break;
                    case "yZoomFactor":
                        this.model.primaryYAxis.zoomFactor = this._yZoomFactor();
                        break;
                    case "xZoomPosition":
                        this.model.primaryXAxis.zoomPosition = this._xZoomPosition();
                        break;
                    case "yZoomPosition":
                        this.model.primaryYAxis.zoomPosition = this._yZoomPosition();
                        break;
                    case "drilldown":
                        this.model.series = [];
                        this.model.explodeValue = null;
                        $.extend(true, this.model, {}, options[prop]);
                        break;
                    case "datasource":
                        for (var i = 0; i < this.model.series.length; i++) {
                            this.model.series[i].dataSource = null;
                            this.model.series[i].points = null;
                        }
                        $.extend(true, this.model, {}, options[prop]);
                        break;
                    case "chartArea":
                        $.extend(true, this.model.chartArea, {}, options[prop]);
                        break;

                    default:
                        $.extend(true, this.model, {}, options[prop]);
                        series = this.model.series;
                        for (var i = 0, len = series.length; i < len; i++) {
                            if (series[i].enableAnimation)
                                series[i]._animatedSeries = false;
                        }
                }
            }
            if (this.model.canResize)
                this.bindResizeEvents();
            else
                this.removeResizeEvents();
            $("#annotation_group_" + this._id).remove(); // to remove annotation on set model
            if (this.model.enableCanvasRendering) {
                $(this.svgObject).remove();
                $("#canvas_trackSymbol").remove();
                this.svgRenderer = new ej.EjCanvasRender(this.element); // to create canvas container
                this.svgRenderer.svgObj.height = ej.util.isNullOrUndefined(this.model.size.height) ? "450" : this.model.size.height;
                this.svgRenderer.svgObj.width = ej.util.isNullOrUndefined(this.model.size.width) ? $("#" + this._id).width() : this.model.size.width;
                this.svgObject = this.svgRenderer.svgObj;

                $(this.legendSvgContainer).remove();
                this.legendSvgContainer = this.svgRenderer.createLegendCanvas(this.element); //creating new CANVAS element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            }
            else if ((this.svgObject.id).indexOf("_canvas") != -1) {
                this._removeZoomkit();
                $(this.svgObject).remove();
                this.svgRenderer = new ej.EjSvgRender(this.element);  // to create svg container
                this.svgObject = this.svgRenderer.svgObj;
                this.canvasX = this.canvasY = 0;

                $(this.legendSvgContainer).remove();
                this.legendSvgContainer = this.svgRenderer.createLegendSvg(this.element); //creating new SVG element for legend
                this.svgRenderer.append(this.legendSvgContainer, this.legendContainer);
            }
            else {
                $(this.svgObject).empty();              // using existing svg container
                $(this.legendSvgContainer).empty();
            }

            this.bindTo();
        }
    });


    ej.datavisualization.Chart.CrosshairType = {

        Crosshair: 'crosshair',

        TrackBall: 'trackBall'
    };

    ej.datavisualization.Chart.ExportType = {

        Image: 'image',

        Excel: 'excel'
    };

    ej.datavisualization.Chart.ValueType = {

        Double: 'double',

        DateTime: 'datetime',

        Category: 'category',

        Logarithmic: 'logarithmic'
    };

    ej.datavisualization.Chart.Type = {

        Line: 'line',

        Spline: 'spline',

        Column: 'column',

        Doughnut: 'doughnut',

        Area: 'area',

        SplineArea: 'splinearea',

        StepLine: 'stepline',

        StepArea: 'steparea',

        Pie: 'pie',

        Hilo: 'hilo',

        HiloOpenClose: 'hiloopenclose',

        Candle: 'candle',

        Bubble: 'bubble',

        Scatter: 'scatter',

        Bar: 'bar',

        StackingArea: 'stackingarea',

        StackingArea100: 'stackingarea100',

        RangeColumn: 'rangecolumn',

        StackingColumn: 'stackingcolumn',

        StackingColumn100: 'stackingcolumn100',

        StackingBar: 'stackingbar',

        StackingBar100: 'stackingbar100',

        Pyramid: 'pyramid',

        Funnel: 'funnel',

        Polar: 'polar',

        Radar: 'radar',

        RangeArea: 'rangearea'

    };

    ej.datavisualization.Chart.LabelPlacement = {

        BetweenTicks: "betweenTicks",

        OnTicks: "onTicks"
    };

    ej.datavisualization.Chart.TrendlinesType = {

        Linear: "linear",

        Exponential: "exponential",

        Logarithmic: "logarithmic",

        Power: "power",

        Polynomial: "polynomial"
    };

    ej.datavisualization.Chart.LabelIntersectAction = {

        None: "none",

        Rotate90: "rotate90",

        Rotate45: "rotate45",

        Wrap: "wrap",

        Trim: "trim",

        Hide: "hide",

        MultipleRows: "multipleRows"
    };

    ej.datavisualization.Chart.EdgeLabelPlacement = {

        None: "none",

        Shift: "shift",

        Hide: "hide"
    };

    ej.datavisualization.Chart.Theme = {

        Azure: 'azure',

        FlatLight: 'flatlight',

        Azuredark: 'azuredark',

        Lime: 'lime',

        LimeDark: 'limedark',

        Saffron: 'saffron',

        SaffronDark: 'saffrondark',

        GradientLight: 'gradientlight',

        GradientDark: 'gradientdark'
    };

    ej.datavisualization.Chart.FontStyle = {

        Normal: 'normal',

        Italic: 'italic'
    };

    ej.datavisualization.Chart.FontWeight = {

        Regular: 'regular',

        Bold: 'bold',

        Lighter: 'lighter'
    };

    ej.datavisualization.Chart.IntervalType = {

        Days: 'days',

        Hours: 'hours',

        Seconds: 'seconds',

        Milliseconds: 'milliseconds',

        Minutes: 'minutes',

        Months: 'months',

        Years: 'years'
    };

    ej.datavisualization.Chart.RangePadding = {

        Additional: 'additional',

        Normal: 'normal',

        None: 'none',

        Round: 'round',

        Auto: 'auto'
    };

    ej.datavisualization.Chart.TextAlignment = {

        MiddleTop: 'middletop',

        MiddleCenter: 'middlecenter',

        MiddleBottom: 'middlebottom'
    };

    ej.datavisualization.Chart.LabelPosition = {

        Inside: 'inside',

        Outside: 'outside'
    };

    ej.datavisualization.Chart.ZIndex = {

        Over: 'over',

        Behind: 'behind'
    };

    ej.datavisualization.Chart.Unit = {

        percentage: 'percentage',

        pixel: 'pixel'
    };

    ej.datavisualization.Chart.PyramidMode = {

        Linear: 'linear',

        Surface: 'Surface'
    };

    ej.datavisualization.Chart.DrawType = {

        Line: 'line',

        Column: 'column',

        Area: 'area'
    };

    ej.datavisualization.Chart.Pattern = {

        None: 'none',

        Chessboard: 'chessboard',

        Crosshatch: 'crosshatch',

        Dots: 'dots',

        Pacman: 'pacman',

        DiagonalBackward: 'diagonalBackward',

        DiagonalForward: 'diagonalForward',

        Grid: 'grid',

        Turquoise: 'turquoise',

        Star: 'star',

        Triangle: 'triangle',

        Circle: 'circle',

        Tile: 'tile',

        HorizontalDash: 'horizontalDash',

        VerticalDash: 'verticalDash',

        Rectangle: 'rectangle',

        Box: 'box',

        VerticalStripe: 'verticalStripe',

        HorizontalStripe: 'horizontalStripe',

        Bubble: 'bubble',

        Custom: 'custom'
    };

    ej.datavisualization.Chart.Mode = {

        Series: 'series',

        Point: 'point',
		
		Cluster: 'cluster'
    };

    ej.datavisualization.Chart.Shape = {

        None: 'none',

        LeftArrow: 'leftarrow',

        RightArrow: 'rightarrow',

        Circle: 'circle',

        Cross: 'cross',

        HorizLine: 'horizline',

        VertLine: 'vertLine',

        Diamond: 'diamond',

        Rectangle: 'rectangle',

        Triangle: 'triangle',

        InvertedTriangle: 'invertedtriangle',

        Hexagon: 'hexagon',

        Pentagon: 'pentagon',

        Star: 'star',

        Ellipse: 'ellipse',

        Wedge: 'wedge',

        Trapezoid: 'trapezoid',

        UpArrow: 'uparrow',

        DownArrow: 'downarrow',

        Image: 'image'
    };

    ej.datavisualization.Chart.DrawMode = {

        Both: 'both',

        Open: 'open',

        Close: 'close'
    };

    ej.datavisualization.Chart.LineCap = {

        Butt: 'butt',

        Round: 'round',

        Square: 'square'
    };

    ej.datavisualization.Chart.LineJoin = {

        Round: 'round',

        Bevel: 'bevel',

        Miter: 'miter'
    };

    ej.datavisualization.Chart.Position = {

        Top: 'top',

        Middle: 'middle',

        Bottom: 'bottom'
    };

    ej.datavisualization.Chart.Alignment = {

        Center: 'center',

        Near: 'near',

        Far: 'far'
    };
	
	ej.datavisualization.Chart.TickLinesPosition = {

        Inside: 'inside',

        Outside: 'outside'
    };
	
	ej.datavisualization.Chart.CoordinateUnit = {

        None: 'none',

        Pixels: 'pixels',
		
		Points: 'points'
    };
	
	ej.datavisualization.Chart.HorizontalAlignment = {

        Left: 'left',

        Right: 'right',
		
		Middle: 'middle'
    };
	
	ej.datavisualization.Chart.VerticalAlignment = {

        Top: 'top',

        Bottom: 'bottom',
		
		Middle: 'middle'
    };
	
	ej.datavisualization.Chart.Region = {

        Chart: 'chart',

        Series: 'series'
    };
	
	ej.datavisualization.Chart.ConnectorLineType = {

        Line: 'line',

        Bezier: 'bezier'
    };
	
	ej.datavisualization.Chart.HorizontalTextAlignment = {

        Near: 'near',

        Far: 'far',
		
		Center: 'center'
    };
	
	ej.datavisualization.Chart.VerticalTextAlignment = {

        Near: 'near',

        Far: 'far',
		
		Center: 'center'
    };
	
	ej.datavisualization.Chart.MACDType = {

        Line: 'line',

        Histogram: 'histogram',
		
		Both: 'both'
    };
	
	ej.datavisualization.Chart.IndicatorsType = {

        RSI: 'rsi',

        Momentum: 'momentum',
		
		Bollingerband: 'bollingerband',
		
		Accumulationdistribution : 'accumulationdistribution',
		
		EMA: 'ema',
		
		SMA: 'sma',
		
		Stochastic: 'stochastic',
		
		ATR: 'atr',
		
		MACD: 'macd',
		
		TMA: 'tma'
    };
	
    $.extend(ej.datavisualization.Chart.prototype, ej.ejChart);
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin for Circular Gauge
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 11.2 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget({ "ejCircularGauge": "ej.datavisualization.CircularGauge"    }, {
        _rootCSS: "e-circulargauge",
        validTags: ["div", "span"],
        _savedPoints: [],
        _labelRadius:0,
        _customLblMaxSize: 0,
      
        defaults:   {
              
            value: 0,
              
            minimum: 0,
              
            maximum: 100,
              
            radius: 180,
              
            width: 360,   
              
            height: 360,   
              
            frame: {
                  
                frameType: "fullcircle",   
                  
                backgroundImageUrl: null,
                  
                halfCircleFrameStartAngle: 180,
                  
                halfCircleFrameEndAngle: 360
            },
              
            backgroundColor: null,
              
            interiorGradient: null,   
              
            readOnly: true,
              
            enableAnimation: true,
              
            animationSpeed: 500,
              
            theme: "flatlight",
              
            isRadialGradient: false,
              
            enableResize: false,
              
            tooltip: {
                  
                showLabelTooltip: false,
                  
                showCustomLabelTooltip: false,
                  
                templateID: null,
            },
              
            outerCustomLabelPosition: "bottom",
              
            gaugePosition: "center",  
              
            distanceFromCorner: 20,  
              
            rangeZOrder :"rear",            
               
            drawTicks: null,
              
            drawLabels: null,
              
            drawPointers: null,
              
            drawRange: null,
              
            drawCustomLabel: null,
              
            drawIndicators: null,
              
            drawPointerCap: null,
              
            load: null,
              
            renderComplete: null,
              
            mouseClick: null,
              
            mouseClickMove: null,
              
            mouseClickUp: null,

            scales: null
        },
          

        _defaultScaleValues: function () {
            return {
                  
                size: 6,
                  
                pointerCap: {
                      
                    radius: 7,
                      
                    borderWidth: 3,
                      
                    interiorGradient: null,   
                      
                    borderColor: null,
                      
                    backgroundColor:null
                },
                  
                showScaleBar: false,
                  
                sweepAngle: 310,
                  
                radius: 170,
                  
                startAngle: 115,
                  
                majorIntervalValue: 10,
                  
                minorIntervalValue: 2,
                  
                maximum: null,
                  
                minimum: null,
                  
                border: {
                      
                    color: null,
                      
                    width:1.5
                },
                  
                backgroundColor: null,
                  
                direction: "clockwise",   
                  
                showPointers: true,
                  
                showRanges: false,
                  
                showTicks: true,
                  
                showLabels: true,
                  
                showIndicators: false,
				  
                opacity:1,
				  
				shadowOffset:0,                  
                  
                pointers: [{
                      
                    distanceFromScale: 0,
                      
                    showBackNeedle: false,
                      
                    backNeedleLength: 10,
                      
                    length: 150,
                      
                    placement: "near",   
                      
                    width: 7,
                      
                    opacity: 1,
                    
                      
                    value: null,
                      
                    border: {
                          
                        color: null,
                          
                        width: 1.5
                    },
                      
                    backgroundColor: null,
                      
                    gradients: null,   
                      
                    type: "needle",   
                      
                    needleType: "triangle",   
                      
                    markerType: "rectangle",  
                      
                    imageUrl: "",
                      
                    pointerValueText: {
                          
                        showValue: false,
                          
                        distance: 20,
                          
                        font: {
                              
                            size: "11px",
                              
                            fontFamily: "Arial",
                              
                            fontStyle: "Bold"
                        },
                          
                        color: "#8c8c8c",
                          
                        opacity: 1,
                          
                        autoAngle: false,
                          
                        angle: 0,
                    }
                }],
                  
                ranges: [{
                      
                    distanceFromScale: 25,
                      
                    size: 5,
                      
                    placement: "near",   
                      
                    startValue: null,
                      
                    endValue: null,
                      
                    startWidth: null,
                      
                    endWidth: null,
                      
                    gradients: null,   
                    
                      
                    opacity: null,
                      
                    backgroundColor: "#32b3c6",
                      
                    border: {
                          
                        color: "#32b3c6",
                          
                        width:1.5
                    }
                }],
                  
                ticks: [{
                      
                    angle: 0,
                      
                    distanceFromScale: 0,
                      
                    color:null,
                      
                    type: "major",   
                      
                    placement: "near",   
                      
                    height: 16,
                      
                    width: 3
                }, {
                    angle: 0,
                    distanceFromScale: 0,
                    color:null,
                    type: "minor",   
                    placement: "near",   
                    height: 7,
                    width: 1
                }],
                  
                labels: [{
                      
                    angle: 0,
                      
                    autoAngle:false,
                      
                    opacity: null,
                      
                    font: {
                     
                        size: "11px",
                    
                        fontFamily: "Arial",
                    
                        fontStyle: "Bold"
                    },
                      
                    color: null,
                      
                    distanceFromScale: 0,
                      
                    includeFirstValue: true,
                      
                    placement: "near",   
                      
                    type: "major",  
                      
                    unitText: "",
                      
                    unitTextPosition: "back",  

                }],
                  
                indicators: [{
                      
                    height: 15,
                      
                    width: 15,
                      
                    type: "circle",   
                      
                    imageUrl: null,
                      
                    position: {
                          
                        x: 0,
                          
                        y: 0
                    },
                      
                    stateRanges: [{
                          
                        endValue: 0,
                          
                        startValue: 0,
                          
                        backgroundColor: null,
                          
                        borderColor: null,
                          
                        text: "",
                          
                        textColor: null,
                          
                        font: null
                    }]
                }],                  
                  
                subGauges: [{
                      
                    height: 150,
                      
                    width: 150,
                      
                    position: {
                          
                        x: 0,
                          
                        y: 0
                    }
                }],
                //textAngle ,value:"CustomText", position:{X:5,Y:10}, color, font: { size: "11px", fontFamily: "Arial", fontStyle: "Bold" },
                customLabels: []
            };
        },
        dataTypes: {
            scales: "data"
        },
        observables: ["value", "minimum", "maximum"],
        _tags: [{
            tag: "scales",
            attr: ["showScaleBar", "pointerCap.radius", "pointerCap.borderWidth", "pointerCap.interiorGradient", "pointerCap.borderColor", "pointerCap.backgroundColor", "sweepAngle", "startAngle", "showPointers", "showTicks", "backgroundColor", "scaleRadius", "majorIntervalValue", "minorIntervalValue", "shadowOffset", "showRanges", "showLabels", "showCustomLabels", "showIndicators", "border.width", "border.color",
                [{
                    tag: "pointers", attr: ["distanceFromScale", "showBackNeedle", "backNeedleLength", "backgroundColor", "needleType", "markerType", "border.width", "border.color", "imageUrl", "pointerValueText.showValue", "pointerValueText.distance", "pointerValueText.font.size", "pointerValueText.font.fontFamily", "pointerValueText.font.fontStyle", "pointerValueText.angle", "pointerValueText.autoAngle", "pointerValueText.color", "pointerValueText.opacity"]
                }, {
                    tag: "labels", attr: ["autoAngle", "distanceFromScale", "includeFirstValue", "unitText", "unitTextPosition", "font.size", "font.fontFamily", "font.fontStyle"]
                }, {
                    tag: "ticks", attr: ["distanceFromScale"]
                }, {
                    tag: "ranges", attr: ["distanceFromScale", "startValue", "endValue", "startWidth", "endWidth", "backgroundColor", "border.color", "border.width"]
                }, {
                    tag: "indicators", attr: ["imageUrl", "position.x", "position.y",
                        [{
                            tag: "stateRanges", attr: ["endValue", "startValue", "backgroundColor", "borderColor", "textColor"]
                        }]
                    ]
                }, {
                    tag: "subGauges", attr: ["controlID", "position.x", "position.y"]
                }, {
                    tag: "customLabels", attr: ["positionType", "textAngle", "position.x", "position.y", "font.size", "font.fontFamily", "font.fontStyle"]
                }]
            ]
        }],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),
 
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "radius": this.model.radius = options[option]; break;
                    case "frame": $.extend(this.model.frame, options[option]); break;
                    case "tooltip": $.extend(this.model.tooltip, options[option]); break;
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "interiorGradient": this.model.interiorGradient = options[option]; break;
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "theme": this.model.theme = options[option]; break;
                    case "isRadialGradient": this.model.isRadialGradient = options[option]; break;
                    case "outerCustomLabelPosition": this.model.outerCustomLabelPosition = options[option]; break;
                    case "gaugePosition": this.model.gaugePosition = options[option]; break;
                    case "distanceFromCorner": this.model.distanceFromCorner = options[option]; break;
                    case "rangeZOrder ": this.model.rangeZOrder  = options[option]; break;
                    case "value":
                        (this.value() == "") && this.value(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (var j = 0; this.model.scales[i].pointers[j] != null; j++) {
                                this.model.scales[i].pointers[j].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        (this.minimum() == "") && this.minimum(0);
                        for (var k = 0; this.model.scales[k] != null; k++) {
                            this.model.scales[k].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        (this.maximum() == "") && this.maximum(100);
                        for (var l = 0; this.model.scales[l] != null; l++) {
                            this.model.scales[l].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        $.extend(true, this.model.scales, options[option]);
                        break;
                }
            }
            this._init();
        },

      
        _destroy: function () {
            this._unWireEvents();
            this.element.empty().removeClass("e-widget");
        },


        // constructor function
		  
        _init: function () {
            _gaugeCount = $(".e-circulargauge").length;
            initialGaugeCount = _gaugeCount;
            //this._isHalfCircle = false;
            this._scalesInitialize();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            this._wireEvents();
            this._onWindowResize();
        },
	 
        _onWindowResize: function () {
            if (this.model.enableResize) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this.resizeCanvas);
                else
                    this._on($(window), "orientationchange", this.resizeCanvas);
            }
        },
		 
        _initialize: function () {
            this._initObject(this);
            this._drawOuterLayer();
            this._drawScales();
            if (this.model.renderComplete)
                this._onRenderComplete();
        },
		 
        _scalesInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
		 
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },
	 	
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._LoadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
	 	
        _initObject: function (element) {
            this._savedPoints=[];
            this.element.addClass("e-widget");
            element.GaugeEl = this.element;
            if (element.canvasEl) {
                element.canvasEl.remove();
                element.GaugeEl.empty();
            }
            else
                element.canvasEl = $("<canvas></canvas>");
                var outerLabelCount=0;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (var j = 0; this.model.scales[i].pointers[j] != null; j++) {
                    if (this.model.scales[i].pointers[j].value == null)
                        this.model.scales[i].pointers[j].value = this.value();
                }
                for (var k = 0; this.model.scales[i].customLabels[k] != null; k++) {
                    outerLabelCount++;
                    if (this.model.scales[i].customLabels[k].value != null && element.GaugeEl.find('div').length == 0) {
                        if (this.model.scales[i].customLabels[k] != null && this.model.scales[i].customLabels[k].positionType !=null && this.model.scales[i].customLabels[k].positionType == "outer") {
                            element.outerDiv = ej.buildTag("div");
                            if (element.model.outerCustomLabelPosition == "bottom") {
                                element.GaugeEl.append(element.canvasEl);
                                element.GaugeEl.append(element.outerDiv);
                                element.outerDiv.css('text-align', 'center');
                            }
                            else {
                                if (element.model.outerCustomLabelPosition != "top") {
                                    var table = ej.buildTag("TABLE");
                                    table.css('width', '100%');
                                    var tr = ej.buildTag("TR");
                                    var td1 = ej.buildTag("TD");
                                    var td2 = ej.buildTag("td");
                                    if (element.model.outerCustomLabelPosition == "left") {
                                        td1.append(element.outerDiv);
                                        td2.append(element.canvasEl);
                                    }
                                    else {
                                        td1.append(element.canvasEl);
                                        td2.append(element.outerDiv);
                                    }
                                    tr.append(td1);
                                    tr.append(td2);
                                    table.append(tr);
                                    element.GaugeEl.append(table);
                                    element.outerDiv.css({
                                        'width': this.element.width() - element.model.width
                                    });
                                }
                                else {
                                    element.GaugeEl.append(element.outerDiv);
                                    element.GaugeEl.append(element.canvasEl);
                                    element.outerDiv.css('text-align', 'center');
                                }
                            }
                        }
                        else
                            element.GaugeEl.append(element.canvasEl);
                    }
                }
                if (outerLabelCount==0)
                    element.GaugeEl.append(element.canvasEl);
            }
            element.canvasEl.attr("role", "presentation");
            if (_gaugeCount == initialGaugeCount)
            {
                initialDivWidth = window.innerWidth;
            }
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            if (this._isHalfCircle && this._isHalfCircle()) {
                if ((element.model.frame.halfCircleFrameEndAngle - element.model.frame.halfCircleFrameStartAngle) >= 180) {
                    if (element.model.frame.halfCircleFrameStartAngle == 0) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.width / 2;
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width / 2;
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.width / 2;
                                break;
                        }
                    }
                    else if(element.model.frame.halfCircleFrameStartAngle == 90){
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = element.model.height / 2;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = element.model.height / 2;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "middleright":
                                element.centerY = element.model.height / 2;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 180) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.width / 2;
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width / 2;
                                break;
                            case "middleleft":
                                element.centerY = element.model.height / 2 + element.model.radius / 2;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "middleright":
                                element.centerY = element.model.height / 2 + element.model.radius / 2;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.distanceFromCorner + element.model.radius;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width / 2;
                                break;
                        }
                    }
                }
                else if (element.model.frame.halfCircleFrameStartAngle == 270 && element.model.frame.halfCircleFrameEndAngle == 90) {
                    switch (element.model.gaugePosition) {
                        case "center":
                            element.centerY = element.model.height / 2;
                            element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                            break;
                        case "topleft":
                            element.centerY = element.model.radius + element.model.distanceFromCorner;
                            element.centerX = element.model.distanceFromCorner;
                            break;
                        case "topright":
                            element.centerY = element.model.radius + element.model.distanceFromCorner;
                            element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                            break;
                        case "topcenter":
                            element.centerY = element.model.radius + element.model.distanceFromCorner;
                            element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                            break;
                        case "middleleft":
                            element.centerY = element.model.height / 2;
                            element.centerX = element.model.distanceFromCorner;
                            break;
                        case "middleright":
                            element.centerY = element.model.height / 2;
                            element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                            break;
                        case "bottomleft":
                            element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                            element.centerX = element.model.distanceFromCorner;
                            break;
                        case "bottomright":
                            element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                            element.centerX = element.model.width - (element.model.radius + element.model.distanceFromCorner);
                            break;
                        case "bottomcenter":
                            element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                            element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                            break;
                    }
                }
                else if ((element.model.frame.halfCircleFrameEndAngle - element.model.frame.halfCircleFrameStartAngle) <= 90) {
                    if (element.model.frame.halfCircleFrameStartAngle == 0) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.distanceFromCorner + element.model.radius);
                                element.centerX = (element.model.width / 2) - (element.model.radius/2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 90) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = (element.model.width/2) + (element.model.radius/2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) - (element.model.radius / 2);
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - (element.model.radius + element.model.distanceFromCorner);
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 180) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomleft":
                                element.centerY=element.model.height-element.model.distanceFromCorner;
                                element.centerX = element.model.radius + element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width - element.model.distanceFromCorner;
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) + (element.model.radius / 2);
                                break;
                        }
                    }
                    else if (element.model.frame.halfCircleFrameStartAngle == 270) {
                        switch (element.model.gaugePosition) {
                            case "center":
                                element.centerY = (element.model.radius / 2) + (element.model.height / 2);
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "topleft":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "topright":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "topcenter":
                                element.centerY = element.model.distanceFromCorner + element.model.radius;
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                            case "middleleft":
                                element.centerY = (element.model.height / 2) + (element.model.radius / 2);
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "middleright":
                                element.centerY = (element.model.radius / 2) + (element.model.height / 2);;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomleft":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.distanceFromCorner;
                                break;
                            case "bottomright":
                                element.centerY = element.model.height - element.model.distanceFromCorner;
                                element.centerX = element.model.width - (element.model.distanceFromCorner + element.model.radius);
                                break;
                            case "bottomcenter":
                                element.centerY = element.model.height-element.model.distanceFromCorner;
                                element.centerX = (element.model.width / 2) - (element.model.radius / 2);
                                break;
                        }
                    }
                }
            }
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
		 	
        _wireEvents: function () {
            this.startEv = 'mousedown';
            this.moveEv = 'mousemove';
            this.endEv = 'mouseup';
            this.scrollEv = 'wheel';
            this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
            this.onHoverOCustomLabel = $.proxy(this._onHoverOCustomLabel, this);
            this.onLeaveOCustomLabel = $.proxy(this._onLeaveOCustomLabel, this);
            if (this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip) {
                $(this.canvasEl).bind(this.moveEv, this.onMouseMoveHandler);
                $(this.canvasEl).bind(this.scrollEv, this.onMouseMoveHandler);
            }
            if(!this.model.readOnly)
                $(this.canvasEl).bind(this.startEv, this.onMouseDownHandler);
            if (this.model.tooltip.showCustomLabelTooltip){
                $('.'+this._id + 'outercustomlbl').bind("mouseenter", this.onHoverOCustomLabel);
                $('.'+this._id + 'outercustomlbl').bind("mouseleave", this.onLeaveOCustomLabel);
            }
        },
		 
        _unWireEvents: function () {
            $(this.canvasEl).unbind(this.startEv, this.onMouseDownHandler);
        },
       
        _onHoverOCustomLabel: function(e) {
            if (e.currentTarget.innerHTML!=null || e.currentTarget.innerHTML!="")
                this._showTooltip(e, e.currentTarget.innerHTML);
        },
      
        _onLeaveOCustomLabel:function(){
            this._hideTooltip();
        },
     
        _showTooltip: function (event,val) {
            var tooltipText = val + "";
            var tooltipdiv = $('.tooltipDiv');
            if (tooltipdiv.length == 0) {
                tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 105; display: block;'></div>");
                $(document.body).append(tooltipdiv);
            }
            if (this.model.tooltip.templateID != "" && this.model.tooltip.templateID != null) {
                var cloneNode = $("#" + this.model.tooltip.templateID).clone();
                $('.tooltipDiv')[0].innerHTML = "";
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                $(tooltipdiv).css({
                    'background-color': this.model.backgroundColor,
                    'border': '1px solid #bbbcbb',
                    'border-radius': '3px',
                    'color': '#565656'
                });
                tooltipdiv.html(tooltipdiv.html().replace('#label#', tooltipText));
            } else {
                $(tooltipdiv).html(tooltipText);
                $(tooltipdiv).css({
                'background-color': 'white',
                'border': '2px solid #bbbcbb',
                'position': 'absolute',
                'padding': '10px 20px',
                'margin-top': '5px',
                'text-align': 'left',
                'font': '12px Segoe UI',
                'font-stretch': 'condensed',
                'display': 'inline-block',
                'border-radius': '3px',
                'color': '#565656',
                'width': 'auto'
                });
            }
            var tooltipMargin = 10;
            var xPos = event.pageX + tooltipMargin;
            var yPos = event.pageY + tooltipMargin;
            xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
            yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());
            $(tooltipdiv).css("left", xPos);
            $(tooltipdiv).css("top", yPos);
            $('.tooltipDiv').show();
        },
       
        _hideTooltip: function () {
            $('.tooltipDiv').remove();//.fadeOut(0, "linear");
        },
	 
        _onMouseDown: function (e) {
            this._blockDefaultActions(e);
            this._mouseDown=true;
            var startPoint = { "x": this.centerX, "y": this.centerY };
            var endPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
            var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
            var scaleRadius,majorTickHeight,self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.pointers != null) {
                    self.pointerEl = element.pointers;
                    scaleRadius = element.radius;
                    majorTickHeight = element.ticks[0].height;
                    $.each(element.pointers, function (pointerIndex, pointerElement) {
                        if (self._isHalfCircle())
                            element.showBackNeedle = false;
                        var angle = self._getAngle(pointerElement.value);
                        if (angle > 360)
                            angle = angle - 360;
                        var greatervalue = angle + pointerElement.width;
                        var lesservalue = angle - pointerElement.width;
                        var radius = Math.sqrt((endPoint.x - startPoint.x) * (endPoint.x - startPoint.x) + (endPoint.y - startPoint.y) * (endPoint.y - startPoint.y));
                        //condition added for the markerpointer whether the user mousedown is made on marker pointer location or not.
                        var cond = radius < ((pointerElement.placement == "far") ? scaleRadius + pointerElement.width + pointerElement.distanceFromScale : (pointerElement.placement == "center") ? scaleRadius - pointerElement.distanceFromScale : scaleRadius - 15 - majorTickHeight - pointerElement.distanceFromScale) && radius > ((pointerElement.placement == "far") ? scaleRadius + pointerElement.distanceFromScale : (pointerElement.placement == "center") ? scaleRadius - pointerElement.width - pointerElement.distanceFromScale : scaleRadius - pointerElement.width - 15 - majorTickHeight - pointerElement.distanceFromScale);
                        var chkPointerType = (pointerElement.type == "needle") ? radius <= pointerElement.length : cond;
                        if (self._isBetween(lesservalue, greatervalue, indegree) && chkPointerType) {
                                self._onMouseClick(angle, element.value);
                            self.activeElement = pointerElement;
                            $(document).bind(self.moveEv, self.onMouseMoveHandler);
                            $(document).bind(self.endEv, self.onMouseUpHandler);
                        }
                    });
                }
            });
        },
	 
        _onMouseUp: function () {
            this._mouseDown=false;
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
            this._onMouseClickUp(this._getAngle(this.activeElement.value), this.activeElement.value);
            this.activeElement = null;
        },
		 	
        _onMouseMove: function (e) {
            if (this._mouseDown && !ej.isNullOrUndefined(this.activeElement)) {
                this._blockDefaultActions(e);
                var startPoint = { "x": this.centerX, "y": this.centerY };
                var endPoint = { "x": e.pageX - $(this.canvasEl).offset().left, "y": e.pageY - $(this.canvasEl).offset().top };
                var indegree = 180 * this._getCirucumferenceAngle(startPoint, endPoint) / Math.PI;
                if (indegree < this.scaleEl[this.scaleIndex].startAngle && !this._isHalfCircle())
                    indegree = indegree + 360;
                this._onMouseClickMove(this._getAngle(this.activeElement.value), this.activeElement.value);
                if (this._getValue(indegree) <= this.scaleEl[this.scaleIndex].maximum) {
                    if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                        this.activeElement.value = this._getValue(indegree);
                    else
                        this.activeElement.value = this.scaleEl[this.scaleIndex].maximum - this._getValue(indegree);
                }
                if (this.contextEl.putImageData)
                    this._reDrawPointer();
                else
                    this._init();
            }
            else
            {
                if (this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip)
                {
                    var hit = false;
                    for (var i = 0; this._savedPoints[i] != null; i++)
                    {
                        var current = { "X": e.pageX - $(this.canvasEl).offset().left, "Y": e.pageY - $(this.canvasEl).offset().top };
                        if (current.X > this._savedPoints[i].startX && current.X < (this._savedPoints[i].startX + this._savedPoints[i].width) && current.Y > this._savedPoints[i].startY && current.Y < (this._savedPoints[i].startY + this._savedPoints[i].height)) {
                            this._showTooltip(e, this._savedPoints[i].value);
                            hit = true;
                        }
                        else if (hit == false)
                            this._hideTooltip();
                    }
                }
            }
        },

		 	
        _isHalfCircle: function () {
            if (this.model.frame.frameType == "halfcircle")
                return true;
            else
                return false;
        },
        
        _calFontLength: function (font) {
            var minText = this.model.scales[0].minimum + this.model.scales[0].labels[0].unitText;
            var maxText = this.model.scales[0].maximum + this.model.scales[0].labels[0].unitText;
            var text = maxText.length > minText.length ? maxText : minText;
            var length = $('<span id="test"></span>').css({ font: font, display: 'none', whiteSpace: 'nowrap' }).appendTo($('body')).text(text).width();
            $("#test").remove();
            return length/2;
        },
		 
        _getHalfCircleYPosition: function () {
            return this._getYCordinate(this.centerY, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2);
        },
		 
        _getHalfCircleXPosition: function () {
            return this._getXCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2);
        },
		 
        _getXCordinate: function (x, radius, angle) {
            var x1 = x + radius * (Math.cos(Math.PI * (angle / 180)));
            return x1;
        },
		 	
        _getYCordinate: function (y, radius, angle) {
            var y1 = y + radius * (Math.sin(Math.PI * (angle / 180)));
            return y1;
        },
		 
        _getAngle: function (value) {
            var tempValue, angle;
            if (value >= this.scaleEl[this.scaleIndex].minimum && value <= this.scaleEl[this.scaleIndex].maximum)
                tempValue = this.scaleEl[this.scaleIndex].direction == "clockwise" ? value - this.scaleEl[this.scaleIndex].minimum : this.scaleEl[this.scaleIndex].maximum - value;
            else {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise") {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                }
                else {
                    if (value <= this.scaleEl[this.scaleIndex].minimum)
                        tempValue = this.scaleEl[this.scaleIndex].maximum;
                    else
                        tempValue = this.scaleEl[this.scaleIndex].minimum;
                }
            }
            angle = (tempValue * ((this.scaleEl[this.scaleIndex].sweepAngle) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum))) + (this.scaleEl[this.scaleIndex].startAngle);
            return angle;
        },
		 
        _subtractDecimal: function (value, interval) {
            var strValue = value.toString();
            var strInterval = interval.toString();
            var vDecimal;
            var iDecimal;
            if (strValue.indexOf('.') > -1)
                vDecimal = strValue.length - strValue.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (strInterval.indexOf('.') > -1)
                iDecimal = strInterval.length - strInterval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var decimal = vDecimal > iDecimal ? vDecimal : iDecimal;
            var correctValue = (value * Math.pow(10, decimal) - interval * Math.pow(10, decimal)) / Math.pow(10, decimal);
            return correctValue;
        },
		 	
        _getCirucumferenceAngle: function (startPoint, endPoint) {
            if (endPoint.x > startPoint.x) {
                if (endPoint.y > startPoint.y)
                    return this._tangent(startPoint, endPoint); //quadrant2
                else {
                    if (endPoint.y == startPoint.y)
                        return 0;
                    else
                        return 2 * Math.PI + this._tangent(startPoint, endPoint); //quadrant1
                }
            }
            else {
                if (endPoint.x == startPoint.x) {
                    if (endPoint.y == startPoint.y)
                        return 0;   //0 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI / 2;  // 90 degree
                        else
                            return 1.5 * Math.PI; // 270 degree
                    }
                }
                else {
                    if (endPoint.y == startPoint.y)
                        return Math.PI; // 180 degree
                    else {
                        if (endPoint.y > startPoint.y)
                            return Math.PI + this._tangent(startPoint, endPoint);  // quadrant3
                        else
                            return Math.PI + this._tangent(startPoint, endPoint); //quadrant4
                    }
                }
            }
        },
       
        _calcDistanceFactor: function (angle, placement, labelLength) {
            var distanceFactor;
            if ((angle > 240 && angle <= 300) || (angle > 60 && angle <= 120))
                distanceFactor = 0;
            else if ((angle > 330 && angle <= 360) || (angle >= 0 && angle <= 30))
                distanceFactor = -labelLength * 0.5;
            else if ((angle > 30 && angle <= 60) || (angle > 300 && angle <= 330))
                distanceFactor = -labelLength / 2;
            else if ((angle > 150 && angle <= 210))
                distanceFactor = labelLength * 0.5;
            else
                distanceFactor = labelLength / 2;
            return placement == "far" ? -distanceFactor : distanceFactor;
        },
		 
        _tangent: function (startPoint, endPoint) {
            var distance = (endPoint.y - startPoint.y) / (endPoint.x - startPoint.x);
            var inradians = Math.atan(distance);
            //indegrees=180*inradians/Math.PI
            return inradians;
        },
		 
        _getValue: function (angle) {
            if (this.scaleEl[this.scaleIndex].direction == "counterclockwise")
                return ((angle - this.scaleEl[this.scaleIndex].startAngle) / this.scaleEl[this.scaleIndex].sweepAngle) * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum);
            else
                return ((angle - this.scaleEl[this.scaleIndex].startAngle) / this.scaleEl[this.scaleIndex].sweepAngle) * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) + this.scaleEl[this.scaleIndex].minimum;
        },
		 
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.model.scales;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.showScaleBar)
                    self._setScaleCordinates(element);
            });
            if (this.model.rangeZOrder  == "rear") {
                this._setRanges();
                this._setTicks();
            }
            else {
                this._setTicks();
                this._setRanges();
            }
            this._setLabels();
            this._subGauge();
            this._setCustomLabel();
            this._setPointers();
            if (this.model.scales.length == 1){
            $.each(this.model.scales, function (index, element) {
                if (element.showPointers) {
                    if (element.pointers.length > 1)
                        self.model.enableAnimation = false;
                }
            });
            }
            if (!this.contextEl.putImageData)
                this.model.enableAnimation = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    if (this.model.enableAnimation) {
                        this._onAnimate(delay);
                    }
                }
            }
            this._setIndicators();
        },
		 
        _setTicks: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (tickIndex, tickElement) {
                            self.tickIndex = tickIndex;
                            self._setTicksCordinates(tickIndex, tickElement);
                        });
                    }
                }
            });
        },
	 
        _setLabels: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (labelIndex, labelElement) {
                            self.labelIndex = labelIndex;
                            self._setLabelCoridnates(labelIndex, labelElement);
                        });
                    }
                }
            });
        },
		 
        _setIndicators: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showIndicators) {
                    self.scaleIndex = index;
                    if (element.indicators != null) {
                        self.indicatorEl = element.indicators;
                        $.each(element.indicators, function (indIndex, indElement) {
                            self.indicatorIndex = indIndex;
                            self._drawIndicator(indIndex, indElement);
                        });
                    }
                }
            });
        },
		 
        _setPointers: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showPointers) {
                    self.scaleIndex = index;
                    if (element.pointers != null) {
                        self.pointerEl = element.pointers;
                        $.each(element.pointers, function (pointerIndex, pointerElement) {
                            if (self._isHalfCircle())
                                element.showBackNeedle = false;
                            self.pointerIndex = pointerIndex;
                            self._drawPointers(pointerIndex, pointerElement);
                            //  setInterval( function(){ self._drawPointers(pointerIndex,pointerElement);}, 30 ); 
                        });
                    }
                }

            });
        },
		 
        _onAnimate: function (delay) {
            var self = this;
			self.pointerValue = ej.util.isNullOrUndefined(self.pointerValue) ? self.model.scales[0].pointers[0].value : self.pointerValue;
            var currentValue = self.model.scales[0].minimum;
            var pointerValue = self.pointerValue;
            if (this.timer != null)
                window.clearInterval(this.timer);
            this.timer = setInterval(function pointerchan() {
                if (pointerValue > currentValue) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    pointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, pointerValue);
                }
                else if (currentValue >= pointerValue) {
                    window.clearInterval(self.timer);
                }
            }, delay);
        },
		 
        _setRanges: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                if (element.showRanges) {
                    self.scaleIndex = index;
                    if (element.ranges != null) {
                        self.rangeEl = element.ranges;
                        $.each(element.ranges, function (rangeIndex, rangeElement) {
                            self.rangeIndex = rangeIndex;
                            self._setRangeCordinates(rangeIndex, rangeElement);
                        });
                    }
                }
            });
        },
		 
        _setCustomLabel: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabels !=null) {
                    self.customLabelEl = element.customLabels;
                    $.each(element.customLabels, function (labelIndex, labelElement) {
                        self.customLabelIndex = labelIndex;
                        if (self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex] != null && self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex].positionType !=null && self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex].positionType == "outer")
                            self._setOuterCustomLabels(labelIndex, labelElement);
                        else
                            self._setCustomLabelCordinates(labelIndex, labelElement);
                    });
                }
            });
        },
	 
        _subGauge: function () {
            var self = this;
            $.each(this.model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.subGauges != null) {
                    self.subGaugeEl = element.subGauges;
                    $.each(element.subGauges, function (subGaugeIndex, subGaugeElement) {
                        self.subGaugeIndex = index;
                        self._setSubGauge(subGaugeIndex, subGaugeElement);
                    });
                }
            });
        },
        
        _setOuterCustomLabels: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            var div = ej.buildTag("div."+this._id+"outercustomlbl");
            div.text(this.model.scales[this.scaleIndex].customLabels[index].value);
            var position=(this.model.outerCustomLabelPosition=="right" || this.model.outerCustomLabelPosition=="left")?"left":"center";
            this.outerDiv.append(div);
            this.outerDiv.append('</br>');
            if (position == "center")
                div.css({ 'display': 'inline-block', 'margin': '0 auto', 'max-width': this.model.width });
            else
                div.css({ 'display': 'inline-block', 'max-width': this.element.width() - this.model.width > 10 ? this.element.width() - this.model.width : 10 });
            div.css({
                "color": element.color,
                'overflow': 'hidden',
                'text-overflow': 'ellipsis',
                'white-space': 'nowrap',
                'font-size': (element.font != null && element.font.size != null) ? element.font.size : "12px",
                'font-family': (element.font != null && element.font.fontFamily != null) ? element.font.fontFamily : "Arial",
                'font-weight': (element.font != null && element.font.fontStyle != null) ? element.font.fontStyle : "Normal",
                'text-align': position
            });

        },
	 	
        _setScaleCordinates: function (element) {
            this.location = {
                "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY,
                "startAngle": element.startAngle, "endAngle": (element.startAngle + element.sweepAngle),
                "startRadius": this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2, "endRadius": this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2
            };
            this.style = {
                "radius": element.radius - element.size / 2,
                "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color), // this._getColor(element, element.borderColor),
                "lineWidth": element.border.width,
                "size": element.size,
                "isFill": true,
                "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "isStroke": true,
                "shadowOffset": element.shadowOffset,
                "fillStyle": (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor), //this._getColor(element, element.backgroundColor),
                "counterClockwise": element.direction == "clockwise" ? false : true
            };
            if (element.maximum < element.minimum) {
                var tempValue = element.maximum;
                element.maximum = element.minimum;
                element.minimum=tempValue;
            }
            this.maximum(element.maximum);
            this.minimum(element.minimum);
            if (this._notifyArrayChange) {
                this._notifyArrayChange("scales[" + this.scaleIndex + "]maximum", element.maximum);
                this._notifyArrayChange("scales[" + this.scaleIndex + "]minimum", element.minimum);
            }
            this._drawScaleBar(this.location, this.style);
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 	
        _drawOuterLayer: function () {
            var interiorGradient, startPoint, endPoint, hcFrameInnerAngle;
            if (this._isHalfCircle()) {
                startPoint = { "x": this.centerX, "y": this.centerY };
                endPoint = { "x": this.centerX + this.model.radius, "y": this.centerY / 2 };
                hcFrameInnerAngle = this._getCirucumferenceAngle(startPoint, endPoint);
            }
            if (!this.model.isRadialGradient) {
                interiorGradient = this.contextEl.createLinearGradient(0, 0, 0, this.canvasEl[0].height);
            }
            else {
                interiorGradient = this.contextEl.createRadialGradient(this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, 0, this.canvasEl[0].width / 2, this.canvasEl[0].height / 2, this.model.radius);
            }
            if (!ej.isNullOrUndefined(this.model.interiorGradient))
                this._setGradientColor(this, interiorGradient, this.model.interiorGradient.colorInfo);

            this.frameOuterLocation = { "centerX": this.centerX, "hcCenterX": this.centerX, "hcCenterY": this.centerY, "centerY": this.centerY, "startAngle": this._isHalfCircle() ? Math.PI * ((this.model.frame.halfCircleFrameStartAngle) / 180) : 0, "endAngle": this._isHalfCircle() ? Math.PI * ((this.model.frame.halfCircleFrameEndAngle) / 180) : 2 * Math.PI };
            if (this._isHalfCircle())
                this.frameInnerLocation = { "centerX": this.centerX, "hcCenterX": this._getXCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2), "hcCenterY": this._getYCordinate(this.centerX, 0, (this.model.frame.halfCircleFrameStartAngle + this.model.frame.halfCircleFrameEndAngle) / 2), "centerY": this.centerY, "startAngle": Math.PI * ((this.model.frame.halfCircleFrameStartAngle) / 180) + hcFrameInnerAngle, "endAngle": Math.PI * ((this.model.frame.halfCircleFrameEndAngle) / 180) - hcFrameInnerAngle };
            this.frameInnerStyle = {
                "radius": this.model.radius,
                "isStroke": false, "isFill": true,
                "fillStyle": (this.model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(null, this.model.backgroundColor),
                "counterClockwise": false
            };
            if (this.model.frame.frameType == "fullcircle") {
                this._drawCircleFrame(this.frameOuterLocation, this.frameInnerStyle);
            }
            else if (this.model.frame.frameType == "halfcircle") {
                this._drawHalfCircle(this.frameInnerLocation, this.frameInnerStyle);
            }
            if (this.contextEl.getImageData)
                this.outerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 
        _setTicksCordinates: function (index, element) {
            var startX, startY, lineHeight, minimumMeet, interval, radius, angle;
            if (element.type == "major") {
                interval = this.scaleEl[this.scaleIndex].majorIntervalValue;
                this.majorTickHeight = element.height;
            }
            else {
                interval = this.scaleEl[this.scaleIndex].minorIntervalValue;
            }
            if (element.placement == "far")
                radius = this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
            if (element.placement == "center")
                radius = this.scaleEl[this.scaleIndex].radius - element.height / 2 - this.scaleEl[this.scaleIndex].border.width / 2 - element.distanceFromScale;
            if (element.placement == "near")
                radius = this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - this.scaleEl[this.scaleIndex].border.width / 2 - element.distanceFromScale;
            lineHeight = element.placement == "near" ? -element.height : element.height;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum)
                        minimumMeet = true;
                    angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.location = { "startX": startX, "startY": startY };
                    this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.width, "strokeStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                    if (this.model.drawTicks)
                        this._onDrawTicks(angle, value);
                    this._drawTickMark(this.location, this.style);
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.location = { "startX": startX, "startY": startY };
                this.style = { "angle": element.angle + angle, "isStroke": true, "isFill": false, "lineHeight": lineHeight, "lineWidth": element.width, "strokeStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                //if (this.model.drawTicks)
                //    this._onDrawTicks(angle, value);
                //this._drawTickMark(this.location, this.style);
            }

            if (this.contextEl.getImageData)
                this.tickImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 
        _setLabelCoridnates: function (index, element) {
            var minimumMeet;
            var startX, startY, lineHeight, incrementAngle, interval, radius;
            interval = element.type == "major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
            incrementAngle = element.type == "major" ? (this.majorIntervalAngle = this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval)) : this.scaleEl[this.scaleIndex].sweepAngle / ((this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) / interval);
            if (element.placement == "far")
                radius = this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + this.majorTickHeight + element.distanceFromScale;
            if (element.placement == "center")
                radius = this.scaleEl[this.scaleIndex].radius - 10 - element.distanceFromScale;
            if (element.placement == "near")
                radius = this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - 10 - this.majorTickHeight - element.distanceFromScale;
            this._labelRadius = radius;
            for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value = this._subtractDecimal(value, interval)) {
                if (value == this.scaleEl[this.scaleIndex].minimum)
                    minimumMeet = true;
                if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                    if (value == this.scaleEl[this.scaleIndex].minimum && !element.includeFirstValue) {
                        continue;
                    }
                    var angle = this._getAngle(value);
                    angle = angle > 360 ? angle - 360 : angle;
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                    this.location = { "startX": startX, "startY": startY };
                    this.style = { "placement": element.placement, "textPositionAngle": angle, "angle": this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": value, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                    if (this.model.drawLabels)
                        this._onDrawLabels(this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, value);
                    this._drawLabel(this.location, this.style, false);
                    if (this.model.tooltip.showLabelTooltip)
                        this._savedPoints.push({ "startX": startX - 10, "startY": startY-5, "width": 20, "height": parseInt(element.font.size.replace(/\D/g, '')), "value": value });
                }
            }
            if (!minimumMeet) {
                angle = this._getAngle(this.scaleEl[this.scaleIndex].minimum);
                angle = angle > 360 ? angle - 360 : angle;
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, angle);
                this.location = { "startX": startX, "startY": startY };
                this.style = { "angle": this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? angle + element.angle : element.angle, "isStroke": false, "isFill": true, "textValue": this.scaleEl[this.scaleIndex].minimum, "opacity": !(element.opacity) ? 1 : element.opacity, "font": this._getFontString(element, element.font), "lineHeight": lineHeight, "lineWidth": element.width, "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color) };
                if (this.model.drawLabels)
                    this._onDrawLabels(angle, value);
                this._drawLabel(this.location, this.style, false);
                if (this.model.tooltip.showLabelTooltip)
                     this._savedPoints.push({ "startX": startX - 10, "startY": startY-5, "width": 20, "height": parseInt(element.font.size.replace(/\D/g, '')), "value": value });
                angle = this._getAngle(value);
                angle = angle > 360 ? angle - 360 : angle;
            }
            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
		 
        _setRangeCordinates: function (index, element) {
                if (element.startValue < this.scaleEl[this.scaleIndex].maximum) {
                    var tempRangeStart = element.startValue >= this.scaleEl[this.scaleIndex].minimum ? element.startValue : this.scaleEl[this.scaleIndex].minimum;
                    var tempRangeEnd = element.endValue > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.endValue;
                var startX, startY, endX, endY, startAngle, radius, endAngle, startRadius, endRadius, grd, controlAngle;
                controlAngle = this._getAngle((tempRangeStart + tempRangeEnd) / 2);
                startAngle = this._getAngle(tempRangeStart);
                endAngle = this._getAngle(tempRangeEnd);
                radius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - this.scaleEl[this.scaleIndex].size / 2 - element.size - (ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].ticks[0]) ? 16 : this.scaleEl[this.scaleIndex].ticks[0].height);
                startRadius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - this.scaleEl[this.scaleIndex].size / 2 - (ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].ticks[0]) ? 16 : this.scaleEl[this.scaleIndex].ticks[0].height);
                endRadius = this.scaleEl[this.scaleIndex].radius - element.distanceFromScale - element.size - this.scaleEl[this.scaleIndex].size / 2 - (ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].ticks[0]) ? 16 : this.scaleEl[this.scaleIndex].ticks[0].height);
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, startAngle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, startAngle);
                endX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, radius, endAngle);
                endY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, radius, endAngle);
                startAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": startX, "y": startY })) / Math.PI;
                endAngle = (180 * this._getCirucumferenceAngle({ "x": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "y": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY }, { "x": endX, "y": endY })) / Math.PI;

                if (!ej.isNullOrUndefined(element.gradients)) {
                    grd = this.contextEl.createRadialGradient(this.centerX, this.centerY, endRadius, this.centerX, this.centerY, startRadius);
                    this._setGradientColor(this, grd, element.gradients.colorInfo);
                } else
                    grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor); // this._getColor(element, element.backgroundColor);
                this.location = { "startX": startX, "startY": startY, "endX": endX, "endY": endY, "startAngle": startAngle, "endAngle": endAngle };
                this.style = {
                    "placement":element.placement,
                    "radius": element.placement=="near"?this.scaleEl[this.scaleIndex].radius - element.distanceFromScale:this.scaleEl[this.scaleIndex].radius + element.distanceFromScale,
                    "rangeStart": tempRangeStart,
                    "rangeEnd": tempRangeEnd,
                    "startWidth": element.startWidth,
                    "isFill": true,
                    "fillStyle": grd,
                    "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : element.border.color, // element.borderColor,
                    "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                    "counterClockwise": this.scaleEl[this.scaleIndex].direction == "clockwise" ? false : true,
                    "startRadius": startRadius, "endRadius": endRadius, "endWidth": element.endWidth, "lineWidth": element.border.width, "isStroke": true
                };
                if (this.model.drawRange)
                    this._onDrawRange(this.location, this.style);
                this._drawRange(this.location, this.style);
                if (this.contextEl.getImageData)
                this.rangeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
                }
        },
		 
        _setSubGauge: function (index, element) {
            var subGaugeDiv = $('div[id=' + element.controlID + ']');
            if (subGaugeDiv.length > 0 && subGaugeDiv.find('canvas').length) {
                var sourceCanvas = subGaugeDiv.find('canvas')[0].getContext('2d');
                this.contextEl.drawImage(sourceCanvas.canvas, element.position.x, element.position.y, element.width, element.height);
                subGaugeDiv.css('display', 'none');
                if (this.contextEl.getImageData)
                    this.subGaugeImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            }
        },
		 
        _setCustomLabelCordinates: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            element.color = element.color ? (element.color) : "#282828";
            this.location = element.position ? ({ "startX": element.position.x, "startY": element.position.y }) : { "startX": 0, "startY": 0 };
            this.style = { "angle": element.textAngle, "textValue": element.value, "fillStyle": (element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color), "font": this._getFontString(element, element.font) };
            if (this.model.drawCustomLabel)
                this._onDrawCustomLabel(this.location, this.style);
            this._drawLabel(this.location, this.style, true);
            if (this.model.tooltip.showCustomLabelTooltip)
                this._savedPoints.push({ "startX": this.location.startX-30, "startY": this.location.startY-5, "width": 60, "height": !ej.isNullOrUndefined(element.font) ? parseInt(element.font.size.replace(/\D/g, '')) : 10, "value": this.style.textValue });
            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },
	 
        _drawIndicator: function (index, element) {
            var self = this, isInStateRange = false;
            this.location = { "centerX": element.position.x - element.width / 2, "textLocation": element.position, "centerY": element.position.y - element.height / 2, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = {
                "radius": (element.height + element.width) / 2,
                "strokeStyle": "#2BA104",
                "cornerRadius": element.type == "roundedrectangle" ? 2 : 0,
                "height": element.height,
                "width": element.width,
                "lineWidth": element.indicatorBorderWidth,
                "fillStyle": "#2BA104",
                "isStroke": true,
                "isFill": true,
                "indicatorText": element.indicatorText,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.model.drawIndicators)
                this._onDrawIndicators(this.style, this.location);
            if (element.type == ej.datavisualization.CircularGauge.IndicatorType.Image) {
                var image = new Image();
                image.onload = function () {
                    self.contextEl.drawImage(this, element.position.x, element.position.y);
                };
                image.src = element.imageUrl;
            }
            else if (element.stateRanges != null) {
                $.each(element.stateRanges, function (stateRangeIndex, srEl) {//srEl = StateRange Element
                    if (self.pointerEl[self.pointerIndex].value >= srEl.startValue && self.pointerEl[self.pointerIndex].value <= srEl.endValue) {
                        isInStateRange = true;
                        if (!ej.isNullOrUndefined(srEl.text) && srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = (srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : srEl.textColor; // srEl.textColor;
                            self.style.font = element.font;
                        }
                        self.style.strokeStyle = (srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : srEl.borderColor;
                        self.style.fillStyle = (srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : srEl.backgroundColor;
                        self._drawIndicatorFrame(element.type, self.location, self.style);
                    }
                });
            }
            if (!isInStateRange && element.type != ej.datavisualization.CircularGauge.IndicatorType.Image) {
                self._drawIndicatorFrame(element.type, self.location, self.style);
            }
            if (this.contextEl.getImageData && element.type != ej.datavisualization.CircularGauge.IndicatorType.Image)
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
        },

        _drawIndicatorFrame: function(type, location,style) {
            switch (type) {
                case "circle":
                    this._drawCircleFrame(location, style);
                    break;
                case "roundedrectangle":
                case "rectangle":
                    this._drawRectangleFrame(location, style);
                    break;
                case "text":
                    this._drawText(location, style);
                    break;
            }
        },
		 	
        _drawScaleBar: function (location, style) {
            this.contextEl.shadowColor = (style.strokeStyle == "transparent") ? "rgba(0,0,0,0)" : style.strokeStyle;
            if (style.shadowOffset)
                this.contextEl.shadowBlur = style.shadowOffset;
            this._contextOpenPath(style, this);
            if ((location.endAngle - location.startAngle) == 0)
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (0 / 180), false);
            else if ((location.endAngle - location.startAngle) % 360 == 0) {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (0 / 180), Math.PI * (360 / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (360 / 180), Math.PI * (0 / 180), true);
            }
            else {
                this.contextEl.arc(location.centerX, location.centerY, location.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), false);
                this.contextEl.arc(location.centerX, location.centerY, location.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), true);
            }
            this._contextClosePath(style, this);
        },
		 
        _drawTickMark: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.moveTo(0, 0);
            this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this._contextClosePath(style, this);
        },
	 
        _drawLabel: function (location, style, isCustomLabel) {
            var distanceFactor = 0;
            if (!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel) {
                var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPosition;
                if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "back")
                    style.textValue = ((style.textValue % 1 != 0) ? +style.textValue.toFixed(3) : style.textValue) + this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
                else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "front")
                    style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
                distanceFactor = this._calcDistanceFactor(style.textPositionAngle, style.placement, this._calFontLength(style.font));
            }
            this._contextOpenPath(style, this);
            this.contextEl.textAlign = "center";
            this.contextEl.textBaseline = "middle";
            this.contextEl.font = style.font;
            this.contextEl.translate(location.startX + distanceFactor, location.startY);
            this.contextEl.moveTo(0, 0);
            this.scaleEl[this.scaleIndex].labels[this.labelIndex].autoAngle ? this.contextEl.rotate((Math.PI * ((style.angle - 270) / 180))) : this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.fillText(style.textValue, 0, 0);
            this._contextClosePath(style, this);
        },
		 
        _drawCircleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, style.counterClockwise);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
		 
        _drawHalfCircle: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.lineJoin = "round";
            this.contextEl.arc(location.centerX, location.centerY, style.radius, location.startAngle, location.endAngle, false);
            this.contextEl.lineTo(location.hcCenterX, location.hcCenterY);
            this._contextClosePath(style, this);
        },
		 
        _drawRectangleFrame: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.centerX, location.centerY - style.height / 2);
            this.contextEl.moveTo(style.cornerRadius, 0);
            this.contextEl.lineTo(style.width - style.cornerRadius, 0);
            this.contextEl.quadraticCurveTo(style.width, 0, style.width, style.cornerRadius);
            this.contextEl.lineTo(style.width, style.height - style.cornerRadius);
            this.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.cornerRadius, style.height);
            this.contextEl.lineTo(style.cornerRadius, style.height);
            this.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.cornerRadius);
            this.contextEl.lineTo(0, style.cornerRadius);
            this.contextEl.quadraticCurveTo(0, 0, style.cornerRadius, 0);
            this._contextClosePath(style, this);
            if (style.indicatorText)
                this._drawText(location, style);
        },
	 
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = (style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor;
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textLocation.x, location.textLocation.y);
            this.contextEl.closePath();
        },
		 	
        _drawRange: function (location, style) {
            if(style.startWidth == null && style.endWidth == null){
            this._contextOpenPath(style, this);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.startRadius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
            this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.endRadius, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
            this._contextClosePath(style, this);
            }
            else {
                var widthDifference = style.startWidth > style.endWidth ? style.startWidth - style.endWidth : style.endWidth - style.startWidth;
                var angleDifference = this.scaleEl[this.scaleIndex].direction == "clockwise" ? location.startAngle < location.endAngle ? location.endAngle - location.startAngle : 360 - (location.startAngle - location.endAngle) : location.endAngle < location.startAngle ? location.startAngle - location.endAngle : location.startAngle + (360 - location.endAngle);
                var degreeDifference = angleDifference / (widthDifference * 2);
                if (style.startWidth < style.endWidth) {
                    if (style.startWidth != 0) {
                        this._contextOpenPath(style, this);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.placement == "near" ? style.radius - style.startWidth : style.radius + style.startWidth, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
                        this._contextClosePath(style, this);
                    }
                    style.radius = style.placement == "near" ? style.radius - style.startWidth : style.radius + style.startWidth;
                    style.endWidth -= style.startWidth;
                    style.startWidth = 0;
                    for (var i = style.startWidth; i < style.endWidth; i += 0.5) {
                        this.contextEl.beginPath();
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius, Math.PI * (location.endAngle / 180) , Math.PI * (location.startAngle / 180) , !style.counterClockwise);
                        this.contextEl.lineWidth = 2;
                        this.contextEl.strokeStyle = style.fillStyle;
                        this.contextEl.stroke();
                        this.contextEl.restore();
                        location.startAngle = this.scaleEl[this.scaleIndex].direction == "clockwise" ? location.startAngle + degreeDifference : location.startAngle - degreeDifference;;
                        style.radius = style.placement == "near" ? style.radius - 0.5 : style.radius + 0.5;
                    }
                }
                else {
                    if (style.endWidth != 0) {
                        this._contextOpenPath(style, this);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius, Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180), style.counterClockwise);
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.placement == "near" ? style.radius - style.endWidth : style.radius + style.endWidth, Math.PI * (location.endAngle / 180), Math.PI * (location.startAngle / 180), !style.counterClockwise);
                        this._contextClosePath(style, this);
                    }
                    style.radius = style.placement == "near" ? style.radius - style.endWidth : style.radius + style.endWidth;
                    style.startWidth -= style.endWidth;
                    style.endWidth = 0;
                    for (i = style.endWidth; i < style.startWidth; i += 0.5) {
                        this.contextEl.beginPath();
                        this.contextEl.arc(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, style.radius,Math.PI * (location.startAngle / 180), Math.PI * (location.endAngle / 180),  style.counterClockwise);
                        this.contextEl.lineWidth = 2;
                        this.contextEl.strokeStyle = style.fillStyle;
                        this.contextEl.stroke();
                        this.contextEl.restore();
                        location.endAngle = this.scaleEl[this.scaleIndex].direction == "clockwise" ? location.endAngle - degreeDifference : location.endAngle + degreeDifference;
                        style.radius = style.placement == "near" ? style.radius - 0.5 : style.radius + 0.5;
                    }
                }
            }
        },
		 
        _drawPointers: function (index, element) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var startX, startY, angle, height, grd;
            angle = this._getAngle(element.value);
            angle = angle > 360 ? angle - 360 : angle;
            if (element.type == "needle") {
                if (element.placement == "far")
                    height = element.length + this.scaleEl[this.scaleIndex].size / 2;
                if (element.placement == "center")
                    height = element.length;
                if (element.placement == "near")
                    height = element.length - this.scaleEl[this.scaleIndex].size / 2;
            }
            else {
                height = element.length;
                if (element.placement == "far") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius + this.scaleEl[this.scaleIndex].size / 2 + element.distanceFromScale, angle);
                }
                if (element.placement == "center") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale, angle);
                }
                if (element.placement == "near") {
                    startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                    startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerX, this.scaleEl[this.scaleIndex].radius - this.scaleEl[this.scaleIndex].size / 2 - element.distanceFromScale - this.majorTickHeight - 15, angle);
                }
                if (!(this._isHalfCircle && this._isHalfCircle())) {
                    if (this.model.height > this.model.width)
                        startY += (this.model.height - this.model.width) / 2;
                    else if (this.model.height < this.model.width)
                        startY -= (this.model.width - this.model.height) / 2;
                }
            }
            if (height > this.model.radius)
                height = this.model.radius;

            this.location = { "startX": element.type == "needle" ? this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX : startX, "startY": element.type == "needle" ? this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY : startY };
            if (!ej.isNullOrUndefined(element.gradients)) {
                grd = this.contextEl.createLinearGradient(0, 0, 0, element.width);
                this._setGradientColor(this, grd, element.gradients.colorInfo);
            } else
                grd = (element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor);
            this.style = {
                "width": element.width,
                "isFill": true,
                "isStroke": true,
                "radius": 0,
                "showBackNeedle": element.showBackNeedle,
                "backNeedleLength": element.backNeedleLength,
                "angle": element.type == "needle" ? angle : element.placement == "far" ? angle : angle + 180,
                "height": height, "lineWidth": element.border.width, "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                "strokeStyle": (element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color),
                "fillStyle": grd,
                "imageUrl": element.imageUrl,
                "type":element.type,
            };
            if (this.model.drawPointers)
                this._onDrawPointers(angle, element.value);
            if (element.type == "needle") {
                this._drawNeedlePointer(this.location, this.style, element);
                this._setPointerCap(element);
            }
            else {
                if (element.markerType == "roundedrectangle")
                    this.style.radius = 5;
                this._drawMarkerType(element.markerType, this.location, this.style);
            }
            if (element.pointerValueText.showValue) {
                startX = this._getXCordinate(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._labelRadius + element.pointerValueText.distance, angle);
                startY = this._getYCordinate(this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, this._labelRadius + element.pointerValueText.distance, angle);
                this.location = { "startX": startX, "startY": startY };
                this.style = { "angle": element.pointerValueText.autoAngle ? element.pointerValueText.angle + angle : element.pointerValueText.angle, "textValue": element.value, "fillStyle": (element.pointerValueText.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.pointerValueText.color), "font": this._getFontString(element, element.pointerValueText.font), "opacity": element.pointerValueText.opacity };
                this._drawPointerValueText(this.location, this.style);
            }
            if (this.contextEl.getImageData)
                this.pointerImage = this.contextEl.getImageData(0, 0, this.model.width, this.model.height);
            if (this._notifyArrayChange)
                this._notifyArrayChange("scales[" + this.scaleIndex + "]pointers[" + index + "]value", element.value);
            this.value(element.value);
        },
        _drawPointerValueText: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.textAlign = "center";
            this.contextEl.textBaseline = "middle";
            this.contextEl.font = style.font;
            this.contextEl.translate(location.startX , location.startY);
            this.contextEl.moveTo(0, 0);
            this.scaleEl[this.scaleIndex].pointers[this.pointerIndex].pointerValueText.autoAngle ? this.contextEl.rotate((Math.PI * ((style.angle - 270) / 180))) : this.contextEl.rotate((Math.PI * (style.angle / 180)));
            this.contextEl.fillText(style.textValue, 0, 0);
            this._contextClosePath(style, this);
        },
        
        _drawMarkerType: function (type, location, style) {
            switch (type) {
                case "rectangle":
                    this._drawRectangle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "triangle":
                    this._drawTriangle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "ellipse":
                    this._drawEllipse(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "diamond":
                    this._drawDiamond(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "pentagon":
                    this._drawPentagon(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "circle":
                    this._drawCircle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "slider":
                    this._drawSlider(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "pointer":
                    this._drawPointer(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "wedge":
                    this._drawWedge(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "trapezoid":
                    this._drawTrapezoid(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawRoundedRectangle(location, style, this);
                    if ((!ej.isNullOrUndefined(style.imageUrl)) || style.imageUrl != "")
                        this._drawImagePointer(location, style, this);
                    break;
                case "image":
                    this._drawImagePointer(location, style, this);
                    break;
            }
        },

	 
        _drawNeedlePointer: function (location, style, element) {
            if (this.pointerEl[this.pointerIndex].needleType == "image") {
                this._drawImagePointer(location, style, element);
            }
            else{
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * ((style.angle) / 180));
            this.contextEl.moveTo(0, -style.width / 2);
            if (this.pointerEl[this.pointerIndex].needleType == "triangle") {
                this.contextEl.lineTo(style.height, 0);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "rectangle") {
                this.contextEl.lineTo(style.height, -style.width / 2);
                this.contextEl.lineTo(style.height, style.width / 2);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "trapezoid") {
                this.contextEl.lineTo(style.height, -style.width / 4);
                this.contextEl.lineTo(style.height, style.width / 4);
            }
            else if (this.pointerEl[this.pointerIndex].needleType == "arrow") {
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 6);
                this.contextEl.lineTo(style.height - style.height / 4, -style.width / 2);
                this.contextEl.lineTo(style.height, 0);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 2);
                this.contextEl.lineTo(style.height - style.height / 4, style.width / 6);
            }
            this.contextEl.lineTo(0, style.width / 2);
            if (style.showBackNeedle) {
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCap.radius / 2), style.width / 2);
                this.contextEl.lineTo(-(style.backNeedleLength + this.scaleEl[this.scaleIndex].pointerCap.radius / 2), -style.width / 2);
            }
            this._contextClosePath(style, this);
            }
            this.canvasEl.attr("aria-label", this.model.scales[this.scaleIndex].pointers[this.pointerIndex].value);
        },
        
        _drawImagePointer: function (location, style, element) {
            var self = this;
            var image = new Image();
            var angle = style.angle;
            var xPos = location.startX;
            var yPos = location.startY;
            var elemWidth = style.width;
            var elemHeight = style.height;
            var type = style.type;
            image.onload = function () {
                self.contextEl.save();
                self.contextEl.translate(xPos, yPos);
                self.contextEl.rotate(Math.PI * (angle / 180));
                self.contextEl.drawImage(this, 0, -elemHeight / 2, elemWidth, elemHeight);
                self.contextEl.restore();
            };
            image.src = style.imageUrl;
        },
	 
        _setPointerCap: function (element) {
            var grd;
            //var gradients = [{ "ColorStop": 0.19, "Color": "#F2DFDF" }, { "ColorStop": 0.71, "Color": "#FFFFFF" }, { "ColorStop": 0.77, "Color": "#836B33" }];
            grd = this.contextEl.createRadialGradient(this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, 0, this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, this._isHalfCircle() ? this._getHalfCircleYPosition() : this.canvasEl[0].height / 2, this.scaleEl[this.scaleIndex].pointerCap.radius);
            if (!ej.isNullOrUndefined(this.scaleEl[this.scaleIndex].pointerCap.interiorGradient))
                this._setGradientColor(this, grd, this.scaleEl[this.scaleIndex].pointerCap.interiorGradient.colorInfo);
            else
                grd = (this.scaleEl[this.scaleIndex].pointerCap.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.backgroundColor)); // this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.backgroundColor));
            this.location = { "centerX": this._isHalfCircle() ? this._getHalfCircleXPosition() : this.centerX, "centerY": this._isHalfCircle() ? this._getHalfCircleYPosition() : this.centerY, "startAngle": 0, "endAngle": 2 * Math.PI };
            this.style = { "isStroke": true, "isFill": true, "strokeStyle": (this.scaleEl[this.scaleIndex].pointerCap.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, this._getColor(element, this.scaleEl[this.scaleIndex].pointerCap.borderColor)), "radius": this.scaleEl[this.scaleIndex].pointerCap.radius, "lineWidth": this.scaleEl[this.scaleIndex].pointerCap.borderWidth, "fillStyle": grd };
            if (this.model.drawPointerCap)
                this._onDrawPointerCap();
            this._drawCircleFrame(this.location, this.style);
        },

        
        isAndroid: function () {
            return (/android/i.test(navigator.userAgent.toLowerCase()));
        },
        
        redraw: function (value) {
            switch (value) {
                case "scale":
                    this._reDrawScale(); break;
                case "pointer":
                    this._reDrawPointer(); break;
                case "range":
                    this._reDrawRange(); break;
                case "label":
                    this._reDrawLabel(); break;
                case "tickMark":
                    this._reDrawTickMark();
                    break;
                case "subGauges":
                    this._reDrawSubGauge();
                    break;
                case "CustomLabel":
                    this._reDrawCustomLabel();
                    break;
            }
        },
       
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                return this._getPointerImage();
        },
      
        _getPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
      
        _getSubGaugeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
      
        _getCustomLabelImage: function () {
            if (this.subGaugeImage)
                return this.subGaugeImage;
            else
                return this._getSubGaugeImage();
        },
       
        _getRangeImage: function () {
            if (this.model.rangeZOrder  == "rear") {
                if (this.scaleImage)
                    return this.scaleImage;
                else
                    return this.outerImage;
            }
            else {
                if (this.tickImage)
                    return this.tickImage;
                else
                    return this._getTickImage();
            }
        },
       
        _getLabelImage: function () {
            if (this.model.rangeZOrder  == "rear") {
                if (this.tickImage)
                    return this.tickImage;
                else
                    return this._getTickImage();
            }
            else {
                if (this.tickImage)
                    return this.rangeImage;
                else
                    return this._getRangeImage();
            }
        },
      
        _getTickImage: function () {
            if (this.model.rangeZOrder  == "rear") {
                if (this.rangeImage)
                    return this.rangeImage;
                else
                    return this._getRangeImage();
            }
            else {
                if (this.scaleImage)
                    return this.scaleImage;
                else
                    return this.outerImage;
            }
        },
     
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].value;
            else
                return null;
        },
       
        setMarkerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
        
        getMarkerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].distanceFromScale;
            else
                return null;
        },
        
        setPointerLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].length = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
       
        getPointerLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].length;
            else
                return null;
        },
        
        setPointerWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
       
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].width;
            else
                return null;
        },

        
        setBackNeedleLength: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        
        getBackNeedleLength: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].backNeedleLength;
            else
                return null;
        },
        
        setNeedleStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].needleType = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
        
        getNeedleStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].needleType;
            else
                return null;
        },
       
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else this._initialize();
            }
        },
		 
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].placement;
            else
                return null;
        },
      
        setPointerNeedleType: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                this.scaleEl[scaleIndex].pointers[pointerIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else this._initialize();
            }
        },
          
        getPointerNeedleType: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].type;
            else
                return null;
        },
          
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
                if (value == "roundedrectangle" || !ej.isNullOrUndefined(ej.datavisualization.CircularGauge.MarkerType[value.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); })])) {
                    this.scaleEl[scaleIndex].pointers[pointerIndex].markerType = value;
                    if (this.contextEl.putImageData && !this.isAndroid())
                        this._reDrawCustomLabel();
                    else this._initialize();
                }
            }
        },
          
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length)
                return this.scaleEl[scaleIndex].pointers[pointerIndex].markerType;
            else
                return null;
        },
          
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
          
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].value;
            else
                return null;
        },

          
        setSubGaugeLocation: function (scaleIndex, gaugeIndex, value) {
            if (scaleIndex < this.model.scales.length && gaugeIndex < this.model.scales[scaleIndex].subGauges.length && value != null) {
                this.scaleEl[scaleIndex].subGauges[gaugeIndex].position.x = value.x;
                this.scaleEl[scaleIndex].subGauges[gaugeIndex].position.y = value.y;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawSubGauge();
                else
                    this._initialize();
            }
        },
          
        getSubGaugeLocation: function (scaleIndex, gaugeIndex) {
            if (scaleIndex < this.model.scales.length && gaugeIndex < this.model.scales[scaleIndex].subGauges.length)
                return this.scaleEl[scaleIndex].subGauges[gaugeIndex].position;
            else
                return null;
        },
          
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawCustomLabel();
                else
                    this._initialize();
            }
        },
          
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.model.scales.length && customLabelIndex < this.model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle;
            else
                return null;
        },
          
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
            else
                return null;
        },
          
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
            else
                return null;
        },
          
        setRangeSize: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].size = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeSize: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].size;
            else
                return null;
        },
          
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
            else
                return null;
        },
          
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].placement;
            else
                return null;
        },
          
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].border.width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawRange();
                else
                    this._initialize();
            }
        },
          
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.model.scales.length && rangeIndex < this.model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].border.width;
            else
                return null;
        },
          
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.model.scales.length && pointerIndex < this.model.scales[scaleIndex].pointers.length && value != null) {
            if(value >= this.scaleEl[scaleIndex].minimum && value <= this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].pointers[pointerIndex].value = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawPointer();
                else
                    this._initialize();
            }
        },
          
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
            else
                return null;
        },
          
        setLabelDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale;
            else
                return null;
        },
          
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].type;
            else
                return null;
        },
          
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawLabel();
                else this._initialize();
            }
        },
          
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].placement;
            else
                return null;
        },
          
        setTickAngle: function (scaleIndex, tickIndex, angle) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].angle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else
                    this._initialize();
            }
        },
          
        getTickAngle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].angle;
            else
                return null;
        },
          
        setTickStyle: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].type = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickStyle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].type;
            else
                return null;
        },
          
        setTickPlacement: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].placement = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickPlacement: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].placement;
            else
                return null;
        },
          
        setTickWidth: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickWidth: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].width;
            else
                return null;
        },
          
        setTickHeight: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].height = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickHeight: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].height;
            else
                return null;
        },
          
        setTickDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawTickMark();
                else this._initialize();
            }
        },
          
        getTickDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.model.scales.length && tickIndex < this.model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale;
            else
                return null;
        },
          
        setStartAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].startAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getStartAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].startAngle;
            else
                return null;
        },
          
        setSweepAngle: function (scaleIndex, angle) {
            if (scaleIndex < this.model.scales.length && angle != null) {
                this.scaleEl[scaleIndex].sweepAngle = angle;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getSweepAngle: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].sweepAngle;
            else
                return null;
        },
          
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                if(value < this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].minimum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minimum;
            else
                return null;
        },
          
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                if(value > this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].maximum = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].maximum;
            else
                return null;
        },
          
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].size = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].size;
            else
                return null;
        },
          
        setScaleRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].radius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].radius;
            else
                return null;
        },
          
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
            else
                return null;
        },
          
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
            else
                return null;
        },
          
        setPointerCapRadius: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].pointerCap.radius = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getPointerCapRadius: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].pointerCap.radius;
            else
                return null;
        },
          
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].border.width = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].border.width;
            else
                return null;
        },
          
        setPointerCapBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].pointerCap.borderWidth = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getPointerCapBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].pointerCap.borderWidth;
            else
                return null;
        },
          
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.model.scales.length && value != null) {
                this.scaleEl[scaleIndex].direction = value;
                if (this.contextEl.putImageData && !this.isAndroid())
                    this._reDrawScale();
                else this._initialize();
            }
        },
          
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.model.scales.length)
                return this.scaleEl[scaleIndex].direction;
            else
                return null;
        },
          
        includeFirstValue: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.model.scales.length && labelIndex < this.model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].includeFirstValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else this.initialize();
            }
        },
          
        _reDrawScale: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this.outerImage, 0, 0);
                this._drawScales();

            }
        },
          
        _reDrawPointer: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getPointerImage(), 0, 0);
                this._setPointers();
                this._setIndicators();
            }
        },
          
        _reDrawCustomLabel: function () {
            if (this.model.frame.backgroundImageUrl)
                this.initialize();
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawRange: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                if (this.model.rangeZOrder  == "rear") {
                    this._setRanges();
                    this._setTicks();
                }
                else {
                    this._setRanges();
                }
                this._setLabels();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawLabel: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawTickMark: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                if (this.model.rangeZOrder  == "rear")
                    this._setTicks();
                else {
                    this._setTicks();
                    this._setRanges();
                }
                this._setLabels();
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        _reDrawSubGauge: function () {
            if (this.model.frame.backgroundImageUrl)
                this._initialize();
            else {
                this.contextEl.putImageData(this._getSubGaugeImage(), 0, 0);
                this._subGauge();
                this._setCustomLabel();
                this._setPointers();
                this._setIndicators();

            }
        },
          
        refreshSubGauge: function () {
            if (this.contextEl.putImageData)
                this._reDrawSubGauge();
            else
                this._initialize();
        },
		  
        refresh: function () {
            this._initialize();
        },
          
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10) {
                return false;
            }
            else {
                var image = this.canvasEl[0].toDataURL();
                image = image.replace(/^data:[a-z]*;,/, '');
                var image1 = image.split(',');
                var byteString = atob(image1[1]);
                var buffer = new ArrayBuffer(byteString.length);
                var intArray = new Uint8Array(buffer);
                for (var i = 0; i < byteString.length; i++) {
                    intArray[i] = byteString.charCodeAt(i);
                }
                var blob = new Blob([buffer], { type: "image/png" });
                if (ej.browserInfo().name === "msie")
                    window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
                else {
                    var pom = document.createElement('a');
                    var url = URL.createObjectURL(blob);
                    pom.href = url;
                    pom.setAttribute('download', fileName + '.' + fileType);
                    if (document.createEvent) {
                        var e = document.createEvent("MouseEvents");
                        e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                        pom.dispatchEvent(e);
                    }
                    else if (pom.fireEvent) {
                        pom.fireEvent("onclick");
                    }
                }
                return true;
            }
        },
		  
        resizeCanvas: function () {
            if (_gaugeCount!=0)
                _gaugeCount = _gaugeCount - 1;
            else
                _gaugeCount = $(".e-circulargauge").length - 1;
            var state, chk=true;
            if (!ej.isNullOrUndefined(this.GaugeEl.parent().attr("style")))
                state = this.GaugeEl.parent().attr("style").split(";");
            if (!ej.isNullOrUndefined(state)){
            $.each(state, function (key, val) {
                while (val.indexOf("width")!=-1){
                    chk = val.indexOf("px") == -1 ? true : false;
                    break;
                }
            });
            }
            if (chk) {
            var ratio = window.innerWidth / initialDivWidth;
                this.model.width *= ratio;
                this.model.height *= ratio;
                this.model.radius *= ratio;
                for (var i = 0; this.model.scales[i] != null; i++) {
                    this.model.scales[i].radius *= ratio;
                    this.model.scales[i].pointerCap.radius *= ratio;
                    for (var j = 0; this.model.scales[i].labels[j] != null || this.model.scales[i].pointers[j] != null || this.model.scales[i].ranges[j] != null || this.model.scales[i].indicators[j] != null || this.model.scales[i].subGauges[j] != null || this.model.scales[i].ticks[j] != null; j++) {
                        if (!ej.isNullOrUndefined(this.model.scales[i].customLabels[j])) {
                            if (this.model.scales[i].customLabels[j].positionType != "outer")
                                this.model.scales[i].customLabels[j].position.x *= ratio;
                            this.model.scales[i].customLabels[j].font.size = (parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0]) * ratio < 10) ? "10px" : ((parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0]) * ratio) > this._customLblMaxSize) ? this._customLblMaxSize.toString() + 'px' : ((parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0])) * ratio).toString() + 'px';
                        }
                        if (!ej.isNullOrUndefined(this.model.scales[i].ticks[j])) this.model.scales[i].ticks[j].height *= ratio;
                        if (!ej.isNullOrUndefined(this.model.scales[i].pointers[j])) {
                            this.model.scales[i].pointers[j].length *= ratio;
                            this.model.scales[i].pointers[j].width *= ratio;
                            this.model.scales[i].pointers[j].backNeedleLength *= ratio;
                        }
                        if (!ej.isNullOrUndefined(this.model.scales[i].ranges[j])) {
                            this.model.scales[i].ranges[j].distanceFromScale *= ratio;
                            this.model.scales[i].ranges[j].size *= ratio;
                        }
                        if (!ej.isNullOrUndefined(this.model.scales[i].indicators[j]) && this.model.scales[i].showIndicators) {
                            this.model.scales[i].indicators[j].height *= ratio;
                            this.model.scales[i].indicators[j].width *= ratio;
                            this.model.scales[i].indicators[j].location.x *= ratio;
                        }
                        if (!(ej.isNullOrUndefined(this.model.scales[i].subGauges[j])) && (ej.isNullOrUndefined(this.model.scales[i].subGauges[j].controlID))) {
                            this.model.scales[i].subGauges[j].height *= ratio;
                            this.model.scales[i].subGauges[j].width *= ratio;
                            this.model.scales[i].subGauges[j].position.x *= ratio;
                        }
                    }
                }
                this.refresh();
                if (_gaugeCount == 0) {
                    initialDivWidth = window.innerWidth;
                }
            }
        },

          
		  
        _onDrawTicks: function (tickAngle, pointerValue) {
            var tick = { index: this.tickIndex, element: this.tickEl[this.tickIndex], angle: parseInt(tickAngle) };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, pointerValue: pointerValue, style: this.style, position: this.location,tick:tick };
            this._trigger("drawTicks", data);
        },
		  
        _onDrawLabels: function (labelAngle, pointerValue) {
            var label = { index: this.labelIndex, element: this.labelEl[this.labelIndex], angle: parseInt(labelAngle) };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, pointerValue: pointerValue, style: this.style, position: this.location,label:label };
            this._trigger("drawLabels", data);
        },
		  
        _onDrawPointers: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], angle: parseInt(pointerAngle), pointerValue: pointerValue};
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, pointer: pointer };
            this._trigger("drawPointers", data);
        },
		  
        _onDrawRange: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeEl], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawRange", data);
        },
		  
        _onDrawCustomLabel: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawCustomLabel", data);
        },
		  
        _onDrawIndicators: function () {
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawIndicators", data);
        },
		  
        _onDrawPointerCap: function () {
            var data = { object: this, scaleElement: this.model.scales, position: this.location, style: this.style, context: this.contextEl };
            this._trigger("drawPointerCap", data);
        },
		  
        _onRenderComplete: function () {
            var data = { object: this, scaleElement: this.model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },
		  
        _onMouseClick: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.position, pointers: pointer };
            this._trigger("mouseClick", data);
        },
		  
        _onMouseClickMove: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, pointers: pointer };
            this._trigger("mouseClickMove", data);
        },
		  
        _onMouseClickUp: function (pointerAngle, pointerValue) {
            var pointer = { index: this.pointerIndex, element: this.pointerEl[this.pointerIndex], value: pointerValue, angle: parseInt(pointerAngle) }
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, pointers: pointer };
            this._trigger("mouseClickUp", data);
        },

          
		  
        _setTheme: function () {
            var selectedTheme = ej.datavisualization.CircularGauge.Themes[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
		  
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = ej.datavisualization.CircularGauge.Themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].backgroundColor) ? selectedTheme.backgroundColor : this.model.backgroundColor);
                for (var i = 0; i < this.model.scales.length; i++) {
                    this.model.scales[i].backgroundColor = (!this.model.scales[i].backgroundColor || this.model.scales[i].backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.scales[i].backgroundColor;
                    this.model.scales[i].border.color = (!this.model.scales[i].border.color || this.model.scales[i].border.color == jsonObj[result[th]].scales.border.color) ? selectedTheme.scales.border.color : this.model.scales[i].border.color;
                    for (var j = 0; j < this.model.scales[i].pointers.length; j++) {
                        this.model.scales[i].pointers[j].backgroundColor = (!this.model.scales[i].pointers[j].backgroundColor || this.model.scales[i].pointers[j].backgroundColor == jsonObj[result[th]].scales.pointers.backgroundColor) ? selectedTheme.scales.pointers.backgroundColor : this.model.scales[i].pointers[j].backgroundColor;
                        this.model.scales[i].pointers[j].border.color = (!this.model.scales[i].pointers[j].border.color || this.model.scales[i].pointers[j].border.color == jsonObj[result[th]].scales.pointers.border.color) ? selectedTheme.scales.pointers.border.color : this.model.scales[i].pointers[j].border.color;
                        this.model.scales[i].pointerCap.backgroundColor = (!this.model.scales[i].pointerCap.backgroundColor || this.model.scales[i].pointerCap.backgroundColor == jsonObj[result[th]].scales.pointerCap.backgroundColor) ? selectedTheme.scales.pointerCap.backgroundColor : this.model.scales[i].pointerCap.backgroundColor;
                        this.model.scales[i].pointerCap.borderColor = (!this.model.scales[i].pointerCap.borderColor || this.model.scales[i].pointerCap.borderColor == jsonObj[result[th]].scales.pointerCap.borderColor) ? selectedTheme.scales.pointerCap.borderColor : this.model.scales[i].pointerCap.borderColor;
                    }
                    for (var k = 0; k < this.model.scales[i].ticks.length; k++) {
                        this.model.scales[i].ticks[k].color = (!this.model.scales[i].ticks[k].color || this.model.scales[i].ticks[k].color == jsonObj[result[th]].scales.ticks.color) ? selectedTheme.scales.ticks.color : this.model.scales[i].ticks[k].color;
                    }
                    for (var l = 0; l < this.model.scales[i].labels.length; l++) {
                        this.model.scales[i].labels[l].color = (!this.model.scales[i].labels[l].color || this.model.scales[i].labels[l].color == jsonObj[result[th]].scales.labels.color) ? selectedTheme.scales.labels.color : this.model.scales[i].labels[l].color;
                    }
                }
            }
        },
		  
        _getFontString: function (element, font) {
            return (font ? (font.fontStyle ? font.fontStyle : "") + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily : "");
        },
		  
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.model.Orientation == "Horizontal" && element.markerPlacement == "near") {
                element.contextEl.moveTo(location.startX + style.width - (style.width / 6), location.startY);   
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.moveTo(location.startX + (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY);   
            }
            this._contextClosePath(style, element);
        },
		  
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
		  
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		  
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.radius, -style.height / 2);
            element.contextEl.lineTo(style.width - style.radius, -style.height / 2);
            element.contextEl.quadraticCurveTo(style.width, -style.height / 2, style.width, (-style.height / 2) + style.radius);
            element.contextEl.lineTo(style.width, (style.height / 2) - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height / 2, style.width - style.radius, style.height / 2);
            element.contextEl.lineTo(style.radius, style.height / 2);
            element.contextEl.quadraticCurveTo(0, style.height / 2, 0, style.height / 2 - style.radius);
            element.contextEl.lineTo(0, -style.height / 2 + style.radius);
            element.contextEl.quadraticCurveTo(0, -style.height / 2, style.radius, -style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.model.Width, element.model.Height);
                if (element.model.scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
		  
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		  
        _setPointerDimension: function (style, element) {
            if (!ej.isNullOrUndefined(element.model.Orientation)) {
                if (element.model.Orientation == "Horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
		  
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
		  
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
		  
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
		  
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
		  
        _isBetween: function (first, last, number) {
            return (first < last ? number >= first && number <= last : number >= last && number <= first);
        },
		  
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
		  
        _setGradientColor: function (element, gradients, options) {
            var self = element;
            if (options.Name || typeof (options) === "string") {
                gradients.addColorStop(0, this._getColor(element, options));
                gradients.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradients.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : self._getColor(element, colorElement.color));
                });
        }
    });
      
    ej.datavisualization.CircularGauge.Frame = {
          
        FullCircle: "fullcircle",
          
        HalfCircle: "halfcircle"
    };
      
    ej.datavisualization.CircularGauge.Directions = {
          
        Clockwise: "clockwise",
          
        CounterClockwise: "counterclockwise"
    };
      
    ej.datavisualization.CircularGauge.PointerPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.PointerType = {
          
        Needle: "needle",
          
        Marker: "marker"
    };
      
    ej.datavisualization.CircularGauge.NeedleType = {
          
        Triangle: "triangle",
          
        Rectangle: "rectangle",
          
        Trapezoid: "trapezoid",
          
        Arrow: "arrow",
          
        Image: "image",
    };
      
    ej.datavisualization.CircularGauge.MarkerType = {
          
        Rectangle: "rectangle",
          
        Triangle: "triangle",
          
        Ellipse: "ellipse",
          
        Diamond: "diamond",
          
        Pentagon: "pentagon",
          
        Circle: "circle",
          
        Slider: "slider",
          
        Pointer: "pointer",
          
        Wedge: "wedge",
          
        Trapezoid: "trapezoid",
          
        RoundedRectangle: "roundedrectangle",
          
        Image: "image"
    };
      
    ej.datavisualization.CircularGauge.RangePlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.TickType = {
          
        Major: "major",
          
        Minor: "minor"
    };
      
    ej.datavisualization.CircularGauge.gaugePosition = {
          
        TopLeft: "topleft",
          
        TopRight: "topright",
          
        TopCenter: "topcenter",
          
        MiddleLeft: "middleleft",
          
        MiddleRight: "middleright",
          
        Center: "center",
          
        BottomLeft: "bottomleft",
          
        BottomRight: "bottomright",
          
        BottomCenter: "bottomcenter",
    };
      
    ej.datavisualization.CircularGauge.TickPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.CustomLabelPositionType = {
          
        Inner: "inner",
          
        Outer: "outer"
    };
      
    ej.datavisualization.CircularGauge.OuterCustomLabelPosition = {
          
        Left: "left",
          
        Right: "right",
          
        Top: "top",
          
        Bottom: "bottom",
    };
      
    ej.datavisualization.CircularGauge.LabelPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.CircularGauge.LabelType = {
          
        Major: "major",
          
        Minor: "minor"
    };
      
    ej.datavisualization.CircularGauge.UnitTextPlacement = {
          
        Back: "back",
          
        Front: "front"
    };
      
    ej.datavisualization.CircularGauge.RangeZOrderPosition = {
          
        Rear: "rear",
          
        Front: "front"
    };
      
    ej.datavisualization.CircularGauge.IndicatorType = {
          
        Rectangle: "rectangle",
          
        Circle: "circle",
          
        RoundedRectangle: "roundedrectangle",
          
        Text: "text",
          
        Image:"image"
    };
	  
    ej.datavisualization.CircularGauge.Themes = {
	  
        flatlight: {
            backgroundColor: "#FFFFFF",
            scales: {
                pointerCap: {
                    borderColor: "#424242",
                    backgroundColor: "#424242"
                },
                backgroundColor: "#777777",
                border: {
                    color: "#777777"
                },
                pointers: {
                    backgroundColor: "#424242",
                    border: {
                        color:"#424242"
                    },
                },
                ticks: {
                    color: "#777777"
                },
                labels: {
                    color: "#282828"
                }
            }
        },
	  
        flatdark: {
            backgroundColor: "#1F1F1F",
            scales: {
                pointerCap: {
                    borderColor: "#686868",
                    backgroundColor: "#686868"
                },
                backgroundColor: "#7a7a7a",
                border: {
                    color:"#7a7a7a"
                },
                pointers: {
                    backgroundColor: "#686868",
                    border: {
                        color:"#686868"
                    },
                },
                ticks: {
                    color: "#7a7a7a"
                },
                labels: {
                    color: "#d3d3d3"
                }
            }
        }
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
	 

    ej.widget({ "ejLinearGauge": "ej.datavisualization.LinearGauge"}, {
       
        element: null,
        _rootCSS: "e-lineargauge",
        _animationFlag: true,
        
        model: null,
        _customLblMaxSize: 0,
        _savedPoints: [],
        validTags: ["div", "span"],

       
        defaults:  {
			  	
            value: 0,
			  
            minimum: 0,
			  
            maximum: 100,
			  
            width: 150,
			  
            height: 400,
			  
            theme: "flatlight",
			  
            orientation: "Vertical",
			  
            pointerGradient1: null,
			  
            pointerGradient2: null,
			  
            backgroundColor: null,
			  
            borderColor: null,
			  
            labelColor: null,
			  
            tickColor: null,
			  
            readOnly: true,
			  
            enableResize: false,
              
            tooltip: {
                  
                showLabelTooltip: false,
                  
                showCustomLabelTooltip: false,
                  
                    templateID: null,
            },
              
            outerCustomLabelPosition: "bottom",
              
			frame: {
			      
			    backgroundImageUrl: null,
			      
			    outerWidth: 12,
			      
                innerWidth:8
			},
			  
            scales: null,
			  
            enableAnimation: true,
              
            enableMarkerPointerAnimation: true,
			  
            animationSpeed: 500,
			  
            drawTicks: null,
			  
            drawLabels: null,
			  
            drawBarPointers: null,
			  
            drawMarkerPointers: null,
			  
            drawRange: null,
			  
            drawCustomLabel: null,
			  
            drawIndicators: null,
			  
            load: null,
			  
            init: null,
			  
            renderComplete: null,
              
            mouseClick: null,
              
            mouseClickMove: null,
              
            mouseClickUp: null,
        },
				  	
        _defaultScaleValues: function () {
            return {
				  	
                minimum: null,
				  	
                maximum: null,
				  	
                majorIntervalValue: 10,
				  	
                minorIntervalValue: 2,
				  	
                direction: "counterclockwise",
				  	
                backgroundColor: null,
				  	
                border: {
                      
                    color: null,
                      
                    width:1.5
                },
				  	
                opacity: NaN,
				  	
                width: 30,
				  	
                shadowOffset: 0,
				  	
                length: 290,
				  	
                type: "rectangle",
				  	
                position: { 
				  	
				x: 50, 
				  	
				y: 50 },
				  	
                showRanges: false,
				  	
                showIndicators: false,
				  	
                showCustomLabels: false,
				  	
                showLabels: true,
				  	
                showTicks: true,
				  	
                showBarPointers: true,
				  	
                showMarkerPointers: true,
				  	
                ticks: [{
                                          
                    distanceFromScale: {
                          
                        x: 0,
                          
                        y:0
                    },
					  	
                    angle: 0,
					  	
                    color: null,
					  	
                    type: "majorinterval",
					  	
                    placement: "near",
					  	
                    opacity: 0,
					  	
                    height: 10,
					  	
                    width: 3
                },
                {              
                    distanceFromScale: {
                        x: 0,
                        y:0
                    },
                    angle: 0,
                    color: null,
                    type: "minorinterval",
                    placement: "near",
                    opacity: 0,
                    height: 5,
                    width: 2
                }],
                  
                ranges: [{
                      
                    endWidth: 10,
                      
                    placement: "center",
					  
                    startWidth: 10,
					  
                    distanceFromScale: 0,
					  
                    endValue: 60,
					  
                    startValue: 20,
					  
                    gradients: null,
					  
                    backgroundColor: null,
					  
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: null
                }],
					  
                labels: [{
                      
                    distanceFromScale: {
                            
                        x: -10,
                            
                        y:0
                    },
                      
                    angle: 0,
					  
                    font: { 
					  
					size: "11px", 
					  
					fontFamily: "Arial", 
					  
					fontStyle: "bold" },
					  
                    textColor: null,
					  
                    opacity: 0,
					  
                    type: "major",
					  
                    placement: "near",
					  
                    includeFirstValue: true,
					  
					unitText: "",
					  
					unitTextPlacement: "back"  
                }],
				      
                markerPointers: [{
					  
                    type: "triangle",
					  
                    length: 30,
					  
                    placement: "far",
					  
                    gradients: null,
					  
                    distanceFromScale: 0,
					  
                    width: 30,
					  
                    value: null,
					  
                    backgroundColor: null,
					  
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: 1
                }],
					  
                barPointers: [{
				      
                    gradients: null,
					  
                    distanceFromScale: 0,
					  
                    width: 30,
					  
                    value: null,
					  
                    backgroundColor: null,
                      
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: 1
                }],
					  	
                indicators: [{
					  
                    font: { 
					  
					size: "11px", 
					  
					fontFamily: "Arial",
					  					
					fontStyle: "bold" },
					  					
                    height: 30,
					  	
                    type: "rectangle",
					  	
                    width: 30,
						  	
                    position: { 
					  	
					x: 0, 
					  	
					y: 0 },
					  	
                    textLocation: { 
					  	
					x: 0, 
					  	
					y: 0 },
					  
                    stateRanges: [{
					  
                    endValue: 60,
					  
                    startValue: 50,
					  
                    backgroundColor: null,
					  
                    borderColor: null,
					  
                    text: "",
					  
                        textColor: null
                    }],
					  
                    backgroundColor: null,
                                          
                    border: {
                          
                        color: null,
                          
                        width:1.5
                    },
					  
                    opacity: NaN
                }],
					  
                customLabels: [{
					  
                    font: { 
					  
					size: "11px", 
					  
					fontFamily: "Arial", 
					  
					fontStyle: "bold" },
					  
                    color: null,
					  
                    opacity: 0,
					  
                    value: "",
					  
                    textAngle: 0,
					  
                    position: { 
					  
					x: 0, 
					  
					y: 0 },
                      
                    positionType:"inner"
                }]
            };
        },

        dataTypes: {
            scales: "data"
        },
        observables: ["value", "minimum", "maximum"],
        _tags: [{
            tag: "scales",
            attr: ["majorIntervalValue", "minorIntervalValue", "backgroundColor", "shadowOffset", "showRanges", "showIndicators", "showCustomLabels", "showLabels", "showTicks", "showBarPointers", "showMarkerPointers", "border.color", "border.width", "position.x", "position.y",
                [{
                    tag: "markerPointers", attr: ["distanceFromScale", "backgroundColor", "border.width", "border.color"]
                }, {
                    tag: "barPointers", attr: ["distanceFromScale", "backgroundColor", "border.width", "border.color"]
                }, {
                    tag: "ranges", attr: ["distanceFromScale", "startValue", "endValue", "startWidth", "endWidth", "backgroundColor", "border.color", "border.width"]
                }, {
                    tag: "ticks", attr: ["distanceFromScale.x", "distanceFromScale.y"]
                }, {
                    tag: "indicators", attr: ["backgroundColor", "textLocation", "font.size", "font.fontFamily", "font.fontStyle", "position.x", "position.y", "textLocation.x", "textLocation.y", "borderColor", "textColor",
                        [{
                            tag: "stateRanges", attr: ["endValue", "startValue", "backgroundColor", "borderColor", "textColor"]
                        }]
                    ]
                }, {
                    tag: "labels", attr: ["distanceFromScale.x", "distanceFromScale.y", "textColor", "includeFirstValue", "unitText", "unitTextPlacement", "font.size", "font.fontFamily", "font.fontStyle"]
                }, {
                    tag: "customLabels", attr: ["textAngle", "font.size", "font.fontFamily", "font.fontStyle", "position.x", "position.y"]
                }]
            ]
        }],
        value: ej.util.valueFunction("value"),
        minimum: ej.util.valueFunction("minimum"),
        maximum: ej.util.valueFunction("maximum"),

          
		  	 
        _init: function () {
            _linearGaugeCount = $(".e-lineargauge").length;
            initialGaugeCount = _linearGaugeCount;
            this._initialize();
            this._trigger("load");
            this._setTheme();
            this._render();
            this.wireEvents();
			this._onWindowResize();
        },
		  	
		_onWindowResize:function()
        {
            if (this.model.enableResize) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this.resizeCanvas);
                else
                    this._on($(window), "orientationchange", this.resizeCanvas);
            }
        },
		  	 
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "theme": this.model.theme = options[option]; this._init(); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "orientation": this.model.orientation = options[option]; break;
                    case "pointerGradient1": this.model.pointerGradient1 = options[option]; break;
                    case "pointerGradient2": this.model.pointerGradient2 = options[option]; break;
                    case "labelColor": this.model.labelColor = options[option]; break;
                    case "tick": $.extend(this.model.tick,options[option]); break;
                    case "backgroundColor": this.model.backgroundColor = options[option]; break;
                    case "borderColor": this.model.borderColor = options[option]; break;
                    case "frame": $.extend(this.model.frame, options[option]);
                    case "outerCustomLabelPosition": this.model.outerCustomLabelPosition = options[option]; break;
                    case "tooltip": $.extend(this.model.tooltip, options[option]); break;
                    case "readOnly": this.model.readOnly = options[option]; break;
                    case "value":
                        (this.value() == "") && this.value(0);
                        for (var i = 0; this.model.scales[i] != null; i++) {
                            for (var j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                                this.model.scales[i].markerPointers[j].value = parseFloat(this.value());
                            }
                        }
                        for (var k = 0; this.model.scales[k] != null; k++) {
                            for (var l = 0; this.model.scales[k].barPointers[l] != null; l++) {
                                this.model.scales[k].barPointers[l].value = parseFloat(this.value());
                            }
                        }
                        break;
                    case "minimum":
                        (this.minimum() == "") && this.minimum(0);
                        for (var m = 0; this.model.scales[m] != null; m++) {
                            this.model.scales[m].minimum = parseInt(this.minimum());
                        }
                        break;
                    case "maximum":
                        (this.maximum() == "") && this.maximum(0);
                        for (var n = 0; this.model.scales[n] != null; n++) {
                            this.model.scales[n].maximum = parseInt(this.maximum());
                        }
                        break;
                    case "scales":
                        this.model.scales = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._render();
            this.wireEvents();
        },
          	       	 
        _destroy: function () {
            this.activeElement = null;
            this.canvasEl = null;
            this.contextEl = null;
            this.unWireEvents();
            this.element.empty().removeClass("e-widget");
        },
		  	
        _initialize: function () {
            this.GaugeEl = this.element;
            this.scaleStartX = [];
            this.scaleStartY = [];
            this.isScaleModified = false;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },
		  
        _render: function () {
            this.initialize();
        },
		  
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },
		  	
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    innerElement = proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    innerElement = proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },
		  	
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    ele = proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    ele = proxy._LoadIndividualDefaultValues(ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
		  
        initialize: function () {
            this._initObject(this);

            if (this.Model.frame.backgroundImageUrl)
                this._drawCustomImage(this, this.Model.frame.backgroundImageUrl);
            else {
                if (this.Model.scales != null)
                    this._drawScales();
            }
        },
		  
        _initObject: function (element) {
            this._savedPoints = [];
            this.element.addClass("e-widget");
            element.GaugeEl = element.element;
            if (element.canvasEl) {
                element.canvasEl.parent().empty();
                element.GaugeEl.empty();
            }
            element.canvasEl = $("<canvas></canvas>");
            var outerLabelCount = 0;
            for (var i = 0; this.model.scales[i] != null; i++) {
                if (this.model.scales[i].minimum == null)
                    this.model.scales[i].minimum = this.minimum();
                if (this.model.scales[i].maximum == null)
                    this.model.scales[i].maximum = this.maximum();
                for (var j = 0; this.model.scales[i].markerPointers[j] != null; j++) {
                    if (this.model.scales[i].markerPointers[j].value == null)
                        this.model.scales[i].markerPointers[j].value = this.value();
                }
                for (var l = 0; this.model.scales[i].barPointers[l] != null; l++) {
                    if (this.model.scales[i].barPointers[l].value == null)
                        this.model.scales[i].barPointers[l].value = this.value();
                }
                for (var k = 0; this.model.scales[i].customLabels[k] != null && this.model.scales[i].showCustomLabels == true; k++) {
                    outerLabelCount++;
                    if (this.model.scales[i].customLabels[k].value != null && element.GaugeEl.find('div').length == 0) {
                        if (this.model.scales[i].customLabels[k] != null && this.model.scales[i].customLabels[k].positionType != null && this.model.scales[i].customLabels[k].positionType == "outer") {
                            element.outerDiv = ej.buildTag("div");
                            if (element.model.outerCustomLabelPosition == "bottom") {
                                element.GaugeEl.append(element.canvasEl);
                                element.GaugeEl.append(element.outerDiv);
                                element.outerDiv.css('text-align', 'center');
                                element.GaugeEl.css({ 'width': element.model.width });
                            }
                            else {
                                if (element.model.outerCustomLabelPosition != "top") {
                                    var table = ej.buildTag("TABLE");
                                    table.css('width', '100%');
                                    var tr = ej.buildTag("TR");
                                    var td1 = ej.buildTag("TD");
                                    var td2 = ej.buildTag("td");
                                    if (element.model.outerCustomLabelPosition == "left") {
                                        td1.append(element.outerDiv);
                                        td2.append(element.canvasEl);
                                    }
                                    else {
                                        td1.append(element.canvasEl);
                                        td2.append(element.outerDiv);
                                    }
                                    tr.append(td1);
                                    tr.append(td2);
                                    table.append(tr);
                                    element.GaugeEl.append(table);
                                    element.outerDiv.css({
                                        'width': this.element.width() - element.model.width
                                    });
                                }
                                else {
                                    element.GaugeEl.append(element.outerDiv);
                                    element.GaugeEl.append(element.canvasEl);
                                    element.GaugeEl.css({ 'width': element.model.width });
                                    element.outerDiv.css('text-align', 'center');
                                }
                            }
                        }
                        else
                            element.GaugeEl.append(element.canvasEl);
                    }
                }
                if (outerLabelCount == 0)
                    element.GaugeEl.append(element.canvasEl);
            }
            element.canvasEl.attr("role", "presentation");
            if (_linearGaugeCount == initialGaugeCount) {
                initialLinearDivWidth = window.innerWidth;
            }
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
		  
        _drawFrameCircle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.arc(location.startX, location.startY, style.circleRadius, 0, 2 * Math.PI, true);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
		  
        _drawFrameRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.moveTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                element._drawText(location, style);
        },
		  
        _drawFrameThermometer: function (location, style, element) {
            var radius = element.Model.orientation == "Vertical" ? Math.sqrt(style.width * style.width + style.width * style.width) / 2 : Math.sqrt(style.height * style.height + style.height * style.height) / 2;   
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "Vertical") {
                if (element.scaleEl[element.scaleIndex].direction == "counterclockwise") {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + style.height - Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (-45 / 180), Math.PI * (225 / 180), false);
                    element.contextEl.lineTo(location.startX, location.startY + style.calDistance + style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.width / 2, style.width / 2, -Math.PI, 0, false);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.calDistance + style.width / 2);
                }
                else {
                    element.contextEl.arc(location.startX + Math.cos(Math.PI * (45 / 180)) * radius,
                                       location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                                       radius, Math.PI * (45 / 180), Math.PI * (-225 / 180), true);
                    element.contextEl.lineTo(location.startX, location.startY + style.height - style.width / 2);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width / 2, location.startY + style.height - style.width / 2, style.width / 2, -Math.PI, 0, true);
                    else
                        element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.width / 2);
                }
            }
            else {
                if (element.scaleEl[element.scaleIndex].direction == "counterclockwise") {
                    element.contextEl.arc(location.startX + style.width - radius / 4 - Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (135 / 180), Math.PI * (225 / 180), true);
                    element.contextEl.lineTo(location.startX + style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), true);
                    else
                        element.contextEl.lineTo(location.startX + style.height / 2, location.startY + style.height);

                }
                else {
                    element.contextEl.arc(location.startX + radius / 4 + Math.cos(Math.PI * (45 / 180)) * radius,
                               location.startY + Math.sin(Math.PI * (45 / 180)) * radius,
                               radius, Math.PI * (45 / 180), Math.PI * (315 / 180), false);
                    element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY);
                    if (style.topRounded)
                        element.contextEl.arc(location.startX + style.width - style.height / 2, location.startY + style.height / 2, style.height / 2, Math.PI * (270 / 180), Math.PI * (90 / 180), false);
                    else
                        element.contextEl.lineTo(location.startX + style.width - style.height / 2, location.startY + style.height);
                }
            }
            this._contextClosePath(style, element);
        },
		  
        _drawFrameRoundedRectangle: function (location, style, element) {
            this._contextOpenPath(style, element);
            element.contextEl.moveTo(location.startX + style.radius, location.startY);
            element.contextEl.lineTo(location.startX + style.width - style.radius, location.startY);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY, location.startX + style.width, location.startY + style.radius);
            element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.radius);
            element.contextEl.quadraticCurveTo(location.startX + style.width, location.startY + style.height, location.startX + style.width - style.radius, location.startY + style.height);
            element.contextEl.lineTo(location.startX + style.radius, location.startY + style.height);
            element.contextEl.quadraticCurveTo(location.startX, location.startY + style.height, location.startX, location.startY + style.height - style.radius);
            element.contextEl.lineTo(location.startX, location.startY + style.radius);
            element.contextEl.quadraticCurveTo(location.startX, location.startY, location.startX + style.radius, location.startY);
            this._contextClosePath(style, element);
            if (style.indicatorText)
                this._drawText(location, style);
        },
		  
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },
		  
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },
		  
        _drawScales: function () {
            var self = this;
            this.scaleEl = this.Model.scales;
            this.contextEl.save();
            this.contextEl.translate(this.Model.frame.outerWidth + this.Model.frame.innerWidth, this.Model.frame.outerWidth + this.Model.frame.innerWidth);
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                self._setScaleCordinates(element, element.type);
            });
            this._setTicks();
            this._setLabels();
            this._setRange();
            this._setCustomLabel();
            this._flagPointer = false;
            this._tempOpacity=this.model.scales[0].barPointers[0].opacity;
            this._setBarPointers();
            this._setMarkerPointers();
            this._setIndicators();
            $.each(this.Model.scales, function (ind, elemt) {
                if (elemt.showBarPointers) {
                    if (elemt.barPointers.length > 1)
                        self.model.enableAnimation = false;
                }
                if (elemt.showMarkerPointers) {
                    if (elemt.markerPointers.length > 1)
                        self.model.enableAnimation = false;
                }
            });
            if (!this.contextEl.putImageData)
                this.model.enableAnimation = false;
            if (this.model.animationSpeed != null && this.model.animationSpeed > 0) {
                var delay = this.model.animationSpeed / 25;
                if (delay >= 0) {
                    if (this.model.enableAnimation && this._animationFlag) {
                        this._onAnimate(delay);
                    }
                }
            }
        },
		  
        _setTicks: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showTicks) {
                    self.scaleIndex = index;
                    if (element.ticks != null) {
                        self.tickEl = element.ticks;
                        $.each(element.ticks, function (tickIndex, tickElement) {
                            self.tickIndex = tickIndex;
                            self._setTicksCordinates(tickElement, tickIndex);
                        });
                    }
                }
            });
        },
		  
        _setLabels: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showLabels) {
                    self.scaleIndex = index;
                    if (element.labels != null) {
                        self.labelEl = element.labels;
                        $.each(element.labels, function (labelIndex, labelElement) {
                            self.labelIndex = labelIndex;
                            self._setLabelCordinates(labelElement, labelIndex);
                        });
                    }
                }
            });
        },
		  
        _setIndicators: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.indicators != null && element.showIndicators) {
                    self.indicatorEl = element.indicators;
                    $.each(element.indicators, function (indicatorIndex, indicatorElement) {
                        self.indicatorIndex = indicatorIndex;
                        self._drawIndicator(indicatorIndex, indicatorElement);
                    });
                }
            });
        },
		  
        _setBarPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showBarPointers) {
                    self.scaleIndex = index;
                    if (element.barPointers != null) {
                        self.barPointerEl = element.barPointers;
                        $.each(element.barPointers, function (barPointerIndex, barPointerElement) {
                            self.barPointerIndex = barPointerIndex;
                            element.opacity = (self.scaleIndex == 0 && self.barPointerIndex == 0 && self.model.enableAnimation == true && (self._flagPointer == false && self._animationFlag == true) && self.model.scales[0].type == "thermometer") ? 0 : self._tempOpacity;
                            self._drawScaleBarPointer(barPointerElement, barPointerIndex);
                            self._flagPointer=true;
                        });
                    }
                }
            });
        },
		  
        _setMarkerPointers: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                if (element.showMarkerPointers) {
                    self.scaleIndex = index;
                    if (element.markerPointers != null) {
                        self.markerPointerEl = element.markerPointers;
                        $.each(element.markerPointers, function (markerPointerIndex, markerPointerElement) {
                            self.markerPointerIndex = markerPointerIndex;
                            self._drawMarkerPointer(markerPointerElement, markerPointerIndex);
							self.canvasEl.attr("aria-label", self.model.scales[self.scaleIndex].markerPointers[self.markerPointerIndex].value);
                        });
                    }
                }
            });
        },
		  
        _onAnimate: function (delay) {
            var self = this, timer, timer1;
            var currentValue = self.model.scales[0].minimum;
            var barPointerValue = self.model.scales[0].barPointers[0].value;
            var markerPointerValue = self.model.scales[0].markerPointers[0].value;
            timer = setInterval(function pointerchan() {
                if (barPointerValue > currentValue || currentValue == self.model.scales[0].minimum) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    if (self.scaleEl[0].type == "thermometer") {
                        self.model.scales[0].barPointers[0].value = (barPointerValue > currentValue) ? currentValue: barPointerValue;
                        if (self.contextEl.putImageData != "undefined") {
                            self._setBarPointers();
                        }
                        else {
                            barPointerValue > currentValue ? self.setBarPointerValue(0, 0, currentValue) : self.setBarPointerValue(0, 0, barPointerValue);
                        }
                    }
                    else
                        barPointerValue > currentValue ? self.setBarPointerValue(0, 0, currentValue) : self.setBarPointerValue(0, 0, barPointerValue);
                }
                else {
                    self._animationFlag = false;
                    self.setBarPointerValue(0, 0, barPointerValue);
                    window.clearInterval(timer);
                }
            }, delay);
            timer1 = setInterval(function pointerchan() {
            if(self.model.enableMarkerPointerAnimation){
                if (markerPointerValue > currentValue || currentValue == self.model.scales[0].minimum) {
                    currentValue = currentValue + ((self.model.scales[0].maximum-self.model.scales[0].minimum)/100);
                    if (self.scaleEl[0].type == "thermometer") {
                        self.model.scales[0].markerPointers[0].value = (markerPointerValue > currentValue) ? currentValue : markerPointerValue;
                        if (self.contextEl.putImageData != "undefined") {
                            self._setMarkerPointers();
                            }
                            else {
                                markerPointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, markerPointerValue);
                            }
                        }
                        else
                            markerPointerValue > currentValue ? self.setPointerValue(0, 0, currentValue) : self.setPointerValue(0, 0, markerPointerValue);
                    }
                    else {
                        self._animationFlag = false;
                        self.setPointerValue(0, 0, markerPointerValue);
                        window.clearInterval(timer1);
                }}
            }, delay);
        },
		  
        _setRange: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.ranges != null && element.showRanges) {
                    self.rangeEl = element.ranges;
                    $.each(element.ranges, function (rangeIndex, rangeElement) {
                        self.rangeIndex = rangeIndex;
                        self._drawRange(rangeElement);
                    });
                }
            });
        },
		  
        _setCustomLabel: function () {
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.customLabels != null && element.showCustomLabels) {
                    self.customLabelEl = element.customLabels;
                    $.each(element.customLabels, function (cusLblIndex, cusLblElement) {
                        self.customLabelIndex = cusLblIndex;
                        if (self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex] != null && self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex].positionType != null && self.model.scales[self.scaleIndex].customLabels[self.customLabelIndex].positionType == "outer")
                            self._setOuterCustomLabelCordinates(cusLblIndex, cusLblElement);
                        else
                            self._setCustomLabelCordinates(cusLblIndex, cusLblElement);
                    });
                }
            });
        },
          
        _setOuterCustomLabelCordinates: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            var div = ej.buildTag("div." + this._id + "outercustomlbl");
            div.text(this.model.scales[this.scaleIndex].customLabels[index].value);
            var position = (this.model.outerCustomLabelPosition == "right" || this.model.outerCustomLabelPosition == "left") ? "left" : "center";
            this.outerDiv.append(div);
            this.outerDiv.append('</br>');
            if (position == "center")
                div.css({ 'display': 'inline-block', 'margin': '0 auto', 'max-width': this.model.width });
            else
                div.css({ 'display': 'inline-block', 'max-width': this.element.width() - this.model.width > 10 ? this.element.width() - this.model.width : 10 });
            div.css({
                "color": element.color,
                'overflow': 'hidden',
                'text-overflow': 'ellipsis',
                'white-space': 'nowrap',
                'font-size': (element.font != null && element.font.size != null) ? element.font.size : "12px",
                'font-family': (element.font != null && element.font.fontFamily != null) ? element.font.fontFamily : "Arial",
                'font-weight': (element.font != null && element.font.fontStyle != null) ? element.font.fontStyle : "Normal",
                'text-align': position
            });

        },
		  
        _setScaleCordinates: function (element, scaleStyle) {
            var location, style, radius;
            this.opacity = 1;
            this.bottomRadius = Math.sqrt(element.width * element.width + element.width * element.width) / 2;

            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.Model.frame.outerWidth + this.Model.frame.innerWidth),
                width: this.canvasEl[0].width - 2 * (this.Model.frame.outerWidth + this.Model.frame.innerWidth)
            };
            if (this.Model.orientation == "Vertical") {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.width) * ((element.position.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.length) * ((element.position.y) / 100);
            }
            else {
                this.scaleStartX[this.scaleIndex] = (this.bounds.width - element.length) * ((element.position.x) / 100);
                this.scaleStartY[this.scaleIndex] = (this.bounds.height - element.width) * ((element.position.y) / 100);
            }
            radius = scaleStyle == "roundedrectangle" ? 5 : 0;
            location = { "startX": this.scaleStartX[this.scaleIndex], "startY": this.scaleStartY[this.scaleIndex] };
            style = {
                "width": this.Model.orientation == "Vertical" ? element.width : element.length, "isStroke": true,
                "topRounded": true,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((this.Model.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.backgroundColor)),
                "lineWidth": element.border.width,
                "radius": radius,
                "height": this.Model.orientation == "Vertical" ? element.length : element.width,
                "isFill": true,
                "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
            };
	    if (element.maximum < element.minimum) {
                var tempValue = element.maximum;
                element.maximum = element.minimum;
                element.minimum=tempValue;
            }
            this.minimum(element.minimum);
            this.maximum(element.maximum);
            if (this._notifyArrayChange) {
                this._notifyArrayChange("scales[" + this.scaleIndex + "]maximum", element.maximum);
                this._notifyArrayChange("scales[" + this.scaleIndex + "]minimum", element.minimum);
            }
            if (element.shadowOffset) {
                this.contextEl.shadowBlur = element.shadowOffset;
                this.contextEl.shadowColor = (style.fillStyle == "transparent") ? "rgba(0,0,0,0)" : style.fillStyle;
            }
            this._drawFrame(scaleStyle, location, style);
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && !this.isScaleModified) {
                this._modifyWidth();
                this.isScaleModified = true;
            }
            if (this.contextEl.getImageData)
                this.scaleImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		  
        _setTicksCordinates: function (element, index) {
            var staticPosition, height, lineChangePosition, interval;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {
                interval = element.type == "majorinterval" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.placement == "near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] : this.scaleStartY[this.scaleIndex];
                }
                else if (element.placement == "far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width;
                }
                else if (element.placement == "center") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
                }
                height = element.placement == "near" ? -element.height : element.height;
                for (var value = this.scaleEl[this.scaleIndex].maximum; value >= this.scaleEl[this.scaleIndex].minimum; value -= interval) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this._getClockwiseLinePosition(value);
                        this.location = {
                            "lineChangePosition": lineChangePosition + (this.Model.orientation == "horizontal" ? element.distanceFromScale.x : (element.distanceFromScale.y)),
                            "lineStaticPosition": staticPosition + (this.Model.orientation == "horizontal" ? element.distanceFromScale.y : (element.distanceFromScale.x))
                        };
                        this.style = {
                            "lineHeight": height,
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "tickShape": element.TickShape,
                            "strokeStyle": element.color ? ((element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color)) : ((this.Model.tickColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.tickColor)),
                            "lineWidth": element.width
                        }
                        if (this.Model.drawTicks)
                            this._onDrawTicks(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270, value);
                        this._drawTickMark(this.location, this.style);
                    }
                }

                if (this.contextEl.getImageData)
                    this.tickImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
            }
        },
		  
        _drawTickMark: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.lineWidth = style.lineWidth;
            this.contextEl.strokeStyle = style.strokeStyle;
            if (this.Model.orientation == "Vertical")
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
            else
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
            this.contextEl.moveTo(0, 0);
            if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
            this.contextEl.lineTo(style.lineHeight, 0);
            this.contextEl.stroke();
            this.contextEl.restore();
            this.contextEl.closePath();
        },
		  
        _addDecimal: function (lblValue, lblInterval) {
            var value = lblValue.toString();
            var interval = lblInterval.toString();
            var vDecimal;
            var iDecimal;
            if (value.indexOf('.') > -1)
                vDecimal = value.length - value.indexOf('.') - 1;
            else
                vDecimal = 0;
            if (interval.indexOf('.') > -1)
                iDecimal = interval.length - interval.indexOf('.') - 1;
            else
                iDecimal = 0;
            var decimal = vDecimal > iDecimal ? vDecimal : iDecimal;
            var correctValue = (lblValue * Math.pow(10, decimal) + lblInterval * Math.pow(10, decimal)) / Math.pow(10, decimal);
            return correctValue;
        },
		  
        _setLabelCordinates: function (element, index) {

            var xDistanceFromScale, yDistanceFromScale, staticPosition, lineChangePosition, interval;
            if (this.scaleEl[this.scaleIndex].majorIntervalValue > this.scaleEl[this.scaleIndex].minorIntervalValue) {

                if (this.Model.orientation == "Vertical") {
                    xDistanceFromScale = element.distanceFromScale.x;
                    yDistanceFromScale = element.distanceFromScale.y;
                }
                else {
                    xDistanceFromScale = element.distanceFromScale.y;
                    yDistanceFromScale = element.distanceFromScale.x;
                }
                interval = element.type == "major" ? this.scaleEl[this.scaleIndex].majorIntervalValue : this.scaleEl[this.scaleIndex].minorIntervalValue;
                if (element.placement == "near") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width - 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "right" : "center";
                }
                else if (element.placement == "far") {
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width + 5;
                    this.contextEl.textAlign = this.Model.orientation == "Vertical" ? "left" : "center";
                }
                else {
                    this.contextEl.textAlign = "center";
                    staticPosition = this.Model.orientation == "Vertical" ? this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + this.scaleEl[this.scaleIndex].border.width / 2 : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + this.scaleEl[this.scaleIndex].border.width / 2;
                }
                for (var value = this.scaleEl[this.scaleIndex].minimum; value <= this.scaleEl[this.scaleIndex].maximum; value = this._addDecimal(value, interval)) {
                    if (interval == this.scaleEl[this.scaleIndex].minorIntervalValue && value % this.scaleEl[this.scaleIndex].majorIntervalValue != 0 || interval == this.scaleEl[this.scaleIndex].majorIntervalValue) {
                        lineChangePosition = this.scaleEl[this.scaleIndex].direction == "counterclockwise" ? this._getCounterClockwiseLinePosition(value) : this._getClockwiseLinePosition(value);
                        this.labelValue = value;
                        this.location = {
                            "lineChangePosition": lineChangePosition + (yDistanceFromScale),
                            "lineStaticPosition": staticPosition + (xDistanceFromScale)
                        };
                        this.style = {
                            "angle": this.Model.orientation == "Vertical" ? element.angle : element.angle + 270,
                            "fillStyle": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.textColor)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "opacity": isNaN(element.opacity) ? 1 : element.opacity,
                            "font": this._getFontString(this, element.font),
                            "textValue": this.labelValue
                        };

                        if (this.Model.drawLabels)
                            this._onDrawLabels(this.Model.orientation == "Vertical" ? element.angle : element.angle + 270);
                        this._drawLabel(this.location, this.style,false);
                    }
                }
            }

            if (this.contextEl.getImageData)
                this.labelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);

        },
		  
        _drawLabel: function (location, style, isCustomLabel) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.textBaseline = "middle";
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.font = style.font;
            if (style.opacity)
                this.contextEl.globalAlpha = style.opacity;
            if (this.Model.orientation == "Vertical") {
                this.contextEl.translate(location.lineStaticPosition, location.lineChangePosition);
                if (this.model.tooltip.showLabelTooltip && !isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineStaticPosition + 5, "startY": location.lineChangePosition + 10, "width": 15, "height": 15, "value": style.textValue });
                if (this.model.tooltip.showCustomLabelTooltip && isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineStaticPosition - 35, "startY": location.lineChangePosition + 10, "width": 110, "height": 15, "value": style.textValue });
            }
            else {
                this.contextEl.translate(location.lineChangePosition, location.lineStaticPosition);
                if (this.model.tooltip.showLabelTooltip && !isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineChangePosition + 10, "startY": location.lineStaticPosition + 10, "width": 15, "height": 15, "value": style.textValue });
                if (this.model.tooltip.showCustomLabelTooltip && isCustomLabel)
                    this._savedPoints.push({ "startX": location.lineChangePosition - 35, "startY": location.lineStaticPosition + 10, "width": 110, "height": 15, "value": style.textValue });
            }
            this.contextEl.moveTo(0, 0);
            if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                this.contextEl.rotate((Math.PI * (style.angle / 180)));
            else
                this.contextEl.rotate(-(Math.PI * (style.angle / 180)));
			if(!ej.isNullOrUndefined(isCustomLabel) && !isCustomLabel){
			    var textPostion = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitTextPlacement;
            if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "back")
                style.textValue = style.textValue + this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText;
            else if (!ej.isNullOrUndefined(textPostion) && textPostion.toString() == "front")
                style.textValue = this.model.scales[this.scaleIndex].labels[this.labelIndex].unitText + style.textValue;
            }
            this.contextEl.fillText(style.textValue, 0, 0);
            this.contextEl.fill();
            this.contextEl.restore();

        },
		  
        _drawScaleBarPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var grd, lineYPosition, radius, height, width, startY, backgroundColor, startX, gradients;
            gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2}];
            radius = this.scaleEl[this.scaleIndex].type == "roundedrectangle" ? 5 : 0;
            if (this.scaleEl[this.scaleIndex].direction == "clockwise") {
                lineYPosition = this._getClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.width, this.scaleStartY[this.scaleIndex]);
                }
                else {
                    startX = this.scaleStartX[this.scaleIndex];
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + element.width / 2);
                }
            }
            else {
                lineYPosition = this._getCounterClockwiseLinePosition(element.value);
                if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isScaleModified) {
                    this._restoreWidth();
                    this.isModify = true;
                }
                if (this.Model.orientation == "Vertical") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition, startX + element.width, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition);
                }
                else {
                    startX = this.scaleEl[this.scaleIndex].type == "thermometer" ? lineYPosition - this.scaleEl[this.scaleIndex].width / 2 - this.scaleEl[this.scaleIndex].border.width / 2 : lineYPosition - this.scaleEl[this.scaleIndex].border.width / 2;
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2, startX, this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 + element.width / 2);
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.gradients)
                this._setGradientColor(this, grd, element.gradients.colorInfo);
            else if (this.Model.ScaleInterior)
                this._setGradientColor(this, grd, this.Model.ScaleInterior.colorInfo);
            else
                this._setGradientColor(this, grd, gradients);
            if (this.Model.orientation == "Vertical") {
                startY = this.scaleEl[this.scaleIndex].direction == "clockwise" ? this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].border.width / 2 : lineYPosition;
                if (this.scaleEl[this.scaleIndex].direction == "counterclockwise" && this.scaleEl[this.scaleIndex].type == "thermometer")
                    startY = startY - this.scaleEl[this.scaleIndex].width / 2;
                height = this.scaleEl[this.scaleIndex].direction == "clockwise" ? lineYPosition - this.scaleStartY[this.scaleIndex] : this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition - this.scaleEl[this.scaleIndex].border.width / 2;
                width = element.width;
            }
            else {
                startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                height = element.width;
                width = this.scaleEl[this.scaleIndex].direction == "clockwise" ? lineYPosition - this.scaleStartX : this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].length - lineYPosition;
            }
            this.location = { "startX": startX + this.scaleEl[this.scaleIndex].border.width / 2, "startY": startY };
            this.style = {
                "width": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "horizontal") ? width + height / 2 - this.scaleEl[this.scaleIndex].border.width / 2 : width,
                "lineWidth": element.border.width,
                "radius": radius,
                "topRounded": false,
                "isStroke": false,
                "isFill": true,
                "height": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "Vertical") ? height + width / 2 : height,
                "strokeStyle": element.border.color == null ? ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor)) : ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)),
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "calDistance": (this.scaleEl[this.scaleIndex].type == "thermometer" && this.Model.orientation == "Vertical") ? (this.scaleEl[this.scaleIndex].width - this.barPointerEl[this.barPointerIndex].width) / 2 : 0
            };
            this.value(element.value);
            if (this._notifyArrayChange)
                this._notifyArrayChange("scales[" + this.scaleIndex + "]barpointers[" + index + "]value", element.value);
            if (this.Model.drawBarPointers)
                this._onDrawBarPointers(element.value);
            this._drawFrame(this.scaleEl[this.scaleIndex].type, this.location, this.style);
            if (this.contextEl.getImageData)
                this.barPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		  
        _drawMarkerPointer: function (element, index) {
            element.value = element.value > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.value;
            element.value = element.value < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.value;
            var startX, startY, radius, linePosition, angle, grd, backgroundColor;
            var gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2}];
            this.markerPlacement = element.placement;
            radius = Math.sqrt(element.width * element.width + element.length * element.length) / 2;
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isModify)
                this._modifyWidth();
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "near") {
                    if (element.type == "star")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale - element.width;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale + (element.border.width);
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "near") {
                    if (element.type == "star")
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale - element.length;
                    else
                        startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale + (element.border.width);
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            linePosition = this.scaleEl[this.scaleIndex].direction == "clockwise" ? this._getClockwiseLinePosition(element.value) : this._getCounterClockwiseLinePosition(element.value);
            if (element.type == "star") {
                if (this.Model.orientation == "Vertical") {
                    grd = this.contextEl.createLinearGradient(startX, this.scaleStartY[this.scaleIndex], startX + element.width, this.scaleStartY[this.scaleIndex]);
                    startY = linePosition - element.length / 3;
                }
                else {
                    grd = this.contextEl.createLinearGradient(linePosition, startX, linePosition, startX + element.length);
                    startY = linePosition - element.width / 2;
                }
            }
            else {
                grd = this.contextEl.createLinearGradient(0, 0, element.width, 0);
                startY = linePosition;
            }
            if (element.type == "roundedrectangle") {
                if (this.Model.orientation == "Vertical" && this.markerPlacement == "near")
                    startY += element.length;
                else if (this.Model.orientation == "horizontal") {
                    if (this.markerPlacement == "near")
                        startY -= (element.width);
                    startX += element.width / 2;
                }
            }
            if (element.backgroundColor)
                backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
            else if (element.gradients)
                this._setGradientColor(this, grd, element.gradients.colorInfo);
            else if (this.Model.PointerInterior)
                this._setGradientColor(this, grd, this.Model.PointerInterior.colorInfo);
            else
                this._setGradientColor(this, grd, gradients);

            this.location = { "startX": this.Model.orientation == "Vertical" ? startX : startY, "startY": this.Model.orientation == "Vertical" ? startY : startX };
            this.style = {
                "width": element.width,
                "radius": element.type == "rectangle" ? 0 : radius,
                "height": element.length,
                "lineWidth": element.border.width,
                "isFill": true,
                "isStroke": true,
                "angle": angle,
                "strokeStyle": ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)),
                "markerPlacement": this.markerPlacement,
                "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd))
            };
            this.value(element.value);
            if (this._notifyArrayChange)
                this._notifyArrayChange("scales[" + this.scaleIndex + "]markerpointers[" + index + "]value", element.value);
            if (this.Model.drawMarkerPointers)
                this._onDrawMarkerPointers(angle, element.value);
            if (element.type == "roundedrectangle")
                this.style.radius = 5;
            this._drawMarkerType(element.type, this.location, this.style);
            if (this.scaleEl[this.scaleIndex].type == "thermometer" && this.isModify) {
                this._restoreWidth();
                this.isScaleModified = false;
            }
            if (this.contextEl.getImageData)
                this.markerPointerImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },

        _drawMarkerType: function (type, location, style) {
            switch (type) {
                case "rectangle":
                    this._drawRectangle(location, style, this);
                    break;
                case "triangle":
                    this._drawTriangle(location, style, this);
                    break;
                case "ellipse":
                    this._drawEllipse(location, style, this);
                    break;
                case "diamond":
                    this._drawDiamond(location, style, this);
                    break;
                case "pentagon":
                    this._drawPentagon(location, style, this);
                    break;
                case "circle":
                    this._drawCircle(location, style, this);
                    break;
                case "slider":
                    this._drawSlider(location, style, this);
                    break;
                case "star":
                    this._drawStar(location, style, this);
                    break;
                case "pointer":
                    this._drawPointer(location, style, this);
                    break;
                case "wedge":
                    this._drawWedge(location, style, this);
                    break;
                case "trapezoid":
                    this._drawTrapezoid(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawRoundedRectangle(location, style, this);
                    break;
            }
        },

		  
        _drawRange: function (element) {
            if (element.startValue < this.scaleEl[this.scaleIndex].maximum && element.endValue > this.scaleEl[this.scaleIndex].minimum && this.scaleEl[this.scaleIndex].minimum < this.scaleEl[this.scaleIndex].maximum) {
                var tempStartValue = element.startValue < this.scaleEl[this.scaleIndex].minimum ? this.scaleEl[this.scaleIndex].minimum : element.startValue;
                var tempEndValue = element.endValue > this.scaleEl[this.scaleIndex].maximum ? this.scaleEl[this.scaleIndex].maximum : element.endValue;
                this.rangePosition = element.placement;
                var startLinePosition, endLinePosition, startX, startY, grd, backgroundColor;
                var gradients = [{ "ColorStop": 0, "Color": (this.Model.pointerGradient1 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient1 }, { "ColorStop": 1, "Color": (this.Model.pointerGradient2 == "transparent") ? "rgba(0,0,0,0)" : this.Model.pointerGradient2 }];
                startLinePosition = (this.scaleEl[this.scaleIndex].direction == "clockwise") ? this._getClockwiseLinePosition(tempStartValue) : this._getCounterClockwiseLinePosition(tempStartValue);
                endLinePosition = (this.scaleEl[this.scaleIndex].direction == "clockwise") ? this._getClockwiseLinePosition(tempEndValue) : this._getCounterClockwiseLinePosition(tempEndValue);
                if (this.Model.orientation == "Vertical") {
                    if (element.placement == "far")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width;
                    if (element.placement == "near")
                        startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    if (element.placement == "center")
                        startX = (element.startWidth > element.endWidth) ? this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.startWidth / 2 : this.scaleStartX[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.endWidth / 2;
                    grd = this.contextEl.createLinearGradient(endLinePosition, endLinePosition, endLinePosition, startLinePosition);
                    this.location = { "startX": startX, "startY": startLinePosition, "endY": endLinePosition };
                }
                else {
                    if (element.placement == "far")
                        startY = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + element.distanceFromScale + this.scaleEl[this.scaleIndex].border.width;
                    if (element.placement == "near")
                        startY = this.scaleStartY[this.scaleIndex] + element.distanceFromScale;
                    if (element.placement == "center")
                        startY = (element.startWidth > element.endWidth) ? this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.startWidth / 2 : this.scaleStartY[this.scaleIndex] + element.distanceFromScale + this.scaleEl[this.scaleIndex].width / 2 - element.endWidth / 2;
                    grd = this.contextEl.createLinearGradient(endLinePosition, startY, startLinePosition, startY);
                    this.location = { "startX": startLinePosition, "startY": startY, "endX": endLinePosition };
                }

                if (element.backgroundColor)
                    backgroundColor = ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor));
                else if (element.gradients)
                    this._setGradientColor(this, grd, element.gradients.colorInfo);
                else if (this.Model.RangeInterior)
                    this._setGradientColor(this, grd, this.Model.RangeInterior.colorInfo);
                else
                    this._setGradientColor(this, grd, gradients);

                this.style = {
                    "startWidth": element.startWidth, "lineWidth": element.border.width,
                    "isStroke": true, "isFill": true, "opacity": isNaN(element.opacity) ? 0.4 : element.opacity,
                    "endWidth": element.endWidth,
                    "fillStyle": element.backgroundColor ? ((backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, backgroundColor)) : ((grd == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, grd)),
                    "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : ((this.Model.borderColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.borderColor))
                };
                if (this.Model.drawRange)
                    this._onDrawRange();
                this._drawRangeBar(this.location, this.style);
                if (this.contextEl.getImageData)
                    this.rangeImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
            }
        },
		  
        _drawRangeBar: function (location, style) {
            this._contextOpenPath(style, this);
            if (this.Model.orientation == "Vertical") {
                this.contextEl.moveTo(location.startX, location.startY);
                this.contextEl.lineTo(location.startX, location.endY);
                if (this.rangePosition == "near") {
                    this.contextEl.lineTo(location.startX - style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX - style.startWidth, location.startY);
                }
                else {
                    this.contextEl.lineTo(location.startX + style.endWidth, location.endY);
                    this.contextEl.lineTo(location.startX + style.startWidth, location.startY);
                }
            }
            else {
                this.contextEl.moveTo(location.startX, location.startY);
                this.contextEl.lineTo(location.endX, location.startY);
                if (this.rangePosition == "near") {
                    this.contextEl.lineTo(location.endX, location.startY - style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY - style.startWidth);
                }
                else {
                    this.contextEl.lineTo(location.endX, location.startY + style.endWidth);
                    this.contextEl.lineTo(location.startX, location.startY + style.startWidth);
                }
            }
            this._contextClosePath(style, this);
        },
		  
        _setCustomLabelCordinates: function (index, element) {
            this._customLblMaxSize = this._customLblMaxSize < parseFloat(element.font.size.match(/\d+/)[0]) ? parseFloat(element.font.size.match(/\d+/)[0]) : this._customLblMaxSize;
            var startX, startY;
            this.contextEl.textAlign = "center";
            if (this.Model.orientation == "Vertical") {
                startX = (this.bounds.width) * ((element.position.x) / 100);
                startY = (this.bounds.height) * ((element.position.y) / 100);
            }
            else {
                startX = (this.bounds.width) * ((element.position.x) / 100);
                startY = (this.bounds.height) * ((element.position.y) / 100);
            }
            this.location = { "lineStaticPosition": this.Model.orientation == "Vertical" ? startX : startY, "lineChangePosition": this.Model.orientation == "Vertical" ? startY : startX }
            this.style = { "angle": element.textAngle, "textValue": element.value, "fillStyle": element.color ? ((element.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.color)) : ((this.Model.labelColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, this.Model.labelColor)), "font": this._getFontString(this, element.font) };
            if (this.Model.drawCustomLabel)
                this._onDrawCustomLabel();
            this._drawLabel(this.location, this.style,true);

            if (this.contextEl.getImageData)
                this.customLabelImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },
		  
        _drawIndicator: function (index, element) {
            var self = this, xlocation, ylocation, txtLocation, isInStateRange = false;
            xlocation = (this.bounds.width - 2 * element.width) * ((element.position.x) / 100);
            ylocation = (this.bounds.height - 2 * element.height) * ((element.position.y) / 100);
            txtLocation = { "x": (this.bounds.width) * ((element.textLocation.x) / 100), "y": (this.bounds.height) * ((element.textLocation.y) / 100) };
            self.location = { "startX": element.type == "circle" ? xlocation + element.width : xlocation, "textLocation": txtLocation, "startY": element.type == "circle" ? ylocation + element.height : ylocation, "startAngle": 0, "endAngle": 2 * Math.PI };
            self.style = {
                "radius": element.type == "roundedrectangle" ? 2 : 0,
                "strokeStyle": element.border.color ? ((element.border.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.border.color)) : this._getColor(element, "#FFFFFF"),
                "angle": 0,
                "circleRadius": (element.height + element.width) / 2,
                "height": element.height,
                "width": element.width,
                "lineWidth": element.border.width,
                "fillStyle": element.backgroundColor ? ((element.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(element, element.backgroundColor)) : this._getColor(element, "#FFFFFF"),
                "isStroke": true,
                "isFill": true,
                "indicatorText": null,
                "textColor": null,
                "font": null,
                "counterClockwise": false
            };
            if (this.Model.drawIndicators)
                this._onDrawIndicators(self.style, self.location);
            if (element.stateRanges != null) {
                $.each(element.stateRanges, function (sRIndex, srEl) {  
                    if (self.markerPointerEl[self.markerPointerIndex].value >= srEl.startValue && self.markerPointerEl[self.markerPointerIndex].value <= srEl.endValue) {
                        isInStateRange = true;
                        if (!ej.isNullOrUndefined(srEl.text) && srEl.text.length > 0) {
                            self.style.indicatorText = srEl.text;
                            self.style.textColor = ((srEl.textColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.textColor));
                            self.style.font = self._getFontString(self, element.font);
                        }
                        if (element.type != "text") {
                            self.style.strokeStyle = ((srEl.borderColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.borderColor));
                            self.style.fillStyle = ((srEl.backgroundColor == "transparent") ? "rgba(0,0,0,0)" : self._getColor(element, srEl.backgroundColor));
                            self._drawFrame(element.type, self.location, self.style, self);
                        }
                        else if (element.type == "text")
                            self._drawText(self.location, self.style);
                    }
                });
                }
            if (!isInStateRange && element.type != "text")
                this._drawFrame(element.type, self.location, self.style, self);
            if (this.contextEl.getImageData)
                this.indicatorImage = this.contextEl.getImageData(0, 0, this.Model.width, this.Model.height);
        },

        _drawFrame: function (type, location, style) {
            switch (type) {
                case "circle":
                    this._drawFrameCircle(location, style, this);
                    break;
                case "rectangle":
                    this._drawFrameRectangle(location, style, this);
                    break;
                case "roundedrectangle":
                    this._drawFrameRoundedRectangle(location, style, this);
                    break;
                case "thermometer":
                    this._drawFrameThermometer(location,style,this) ;
                    break;
            }
        },
		  
        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.textAlign = "center";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.font = style.font;
            this.contextEl.fillText(style.indicatorText, location.textLocation.x, location.textLocation.y);
            this.contextEl.closePath();
        },
		  
        _drawTriangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawPointer: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 4);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(0, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(style.width / 2, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawWedge: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(3 * style.width / 4, 0);
            element.contextEl.lineTo(style.width, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawSlider: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 4, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(style.width / 4, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawStar: function (location, style, element) {
            this._contextOpenPath(style, element);
            if (element.Model.orientation == "horizontal" && element.markerPlacement == "near") {
                element.contextEl.moveTo(location.startX + style.width - (style.width / 6), location.startY);   
                element.contextEl.lineTo(location.startX, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width, location.startY + style.height - style.height / 3);
                element.contextEl.lineTo(location.startX + style.width / 6, location.startY);
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY + style.height);
            }
            else {
                element.contextEl.moveTo(location.startX + (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX, location.startY + (style.height / 3));   
                element.contextEl.lineTo(location.startX + style.width - (style.width / 6), location.startY + style.height);   
                element.contextEl.lineTo(location.startX + style.width / 2, location.startY);   
            }
            this._contextClosePath(style, element);
        },
		  
        _drawPentagon: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 3, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 4);
            element.contextEl.lineTo(style.width, style.height / 4);
            element.contextEl.lineTo(style.width / 3, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawDiamond: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(style.width / 2, -style.height / 2);
            element.contextEl.lineTo(style.width, 0);
            element.contextEl.lineTo(style.width / 2, style.height / 2);
            element.contextEl.lineTo(0, 0);
            this._contextClosePath(style, element);
        },
		  
        _drawCircle: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
		  
        _drawTrapezoid: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 4);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 4);
            this._contextClosePath(style, element);
        },
		  
        _drawRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(0, 0);
            element.contextEl.lineTo(0, -style.height / 2);
            element.contextEl.lineTo(style.width, -style.height / 2);
            element.contextEl.lineTo(style.width, style.height / 2);
            element.contextEl.lineTo(0, style.height / 2);
            this._contextClosePath(style, element);
        },
		  
        _drawRoundedRectangle: function (location, style, element) {
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY - style.height / 2);
            this._setContextRotation(style, element);
            element.contextEl.moveTo(style.radius, 0);
            element.contextEl.lineTo(style.width - style.radius, 0);
            element.contextEl.quadraticCurveTo(style.width, 0, style.width, style.radius);
            element.contextEl.lineTo(style.width, style.height - style.radius);
            element.contextEl.quadraticCurveTo(style.width, style.height, style.width - style.radius, style.height);
            element.contextEl.lineTo(style.radius, style.height);
            element.contextEl.quadraticCurveTo(0, style.height, 0, style.height - style.radius);
            element.contextEl.lineTo(0, style.radius);
            element.contextEl.quadraticCurveTo(0, 0, style.radius, 0);
            this._contextClosePath(style, element);
        },
		  
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.Model.width, element.Model.height);
                if (element.Model.scales != null)
                    element._drawScales();
                if (element.Model.Items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
		  
        _drawEllipse: function (location, style, element) {
            var radius = Math.sqrt(style.height * style.height + style.width * style.width) / 2;
            style = this._setPointerDimension(style, element);
            this._contextOpenPath(style, element);
            element.contextEl.translate(location.startX, location.startY);
            this._setContextRotation(style, element);
            element.contextEl.scale(2, 1);
            element.contextEl.arc(radius / 2, 0, radius / 2, 0, Math.PI * 2, true);
            this._contextClosePath(style, element);
        },
          
		  	
        _getIndicatorImage: function () {
            if (this.pointerImage)
                return this.pointerImage;
            else
                return this._getMarkerPointerImage();
        },
		  	
        _getBarPointerImage: function () {
            if (this.customLabelImage)
                return this.customLabelImage;
            else
                return this._getCustomLabelImage();
        },
		  	
        _getMarkerPointerImage: function () {
            if (this.barPointerImage)
                return this.barPointerImage;
            else
                return this._getCustomLabelImage();
        },
		  	
        _getCustomLabelImage: function () {
            if (this.rangeImage)
                return this.rangeImage;
            else
                return this._getRangeImage();
        },
		  	
        _getRangeImage: function () {
            if (this.labelImage)
                return this.labelImage;
            else
                return this._getLabelImage();
        },
		  	
        _getLabelImage: function () {

            if (this.tickImage)
                return this.tickImage;
            else
                return this._getTickImage();
        },
		  	
        _getTickImage: function () {
            if (this.scaleImage)
                return this.scaleImage;
            else
                return this.outerImage;
        },
		  
        setPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
            if(value <= this.scaleEl[scaleIndex].maximum && value >= this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                    this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                    if (!ej.isNullOrUndefined(this.outerDiv) && this.model.scale[scaleIndex].showCustomLabels)
                        this.outerDiv.empty();
                        this._setCustomLabel();
                        this._setMarkerPointers();
                        this._setIndicators();
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].value;
            else
                return null;
        },
		  
        setPointerWidth: function (scaleIndex, pointerIndex, width) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && width != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].width = width;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].width;
            else
                return null;
        },
		  
        setPointerHeight: function (scaleIndex, pointerIndex, height) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && height != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].length = height;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerHeight: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].length;
            else
                return null;
        },
		  
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.r + ", " + option.g + "," + option.b + ", " + option.a / 255 + ")");
            }
        },
		  
        setPointerDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].distanceFromScale;
            else
                return null;
        },
		  
        setPointerPlacement: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].placement = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getPointerPlacement: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].placement;
            else
                return null;
        },
		  
        setMarkerStyle: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length && value != null) {
                this.scaleEl[scaleIndex].markerPointers[pointerIndex].type = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else {
                        this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                        this._setMarkerPointers();
                    }
                }
                else
                    this.initialize();
            }
        },
		  
        getMarkerStyle: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].markerPointers.length)
                return this.scaleEl[scaleIndex].markerPointers[pointerIndex].type;
            else
                return null;
        },
		  
        setBarPointerValue: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
            if(value <= this.scaleEl[scaleIndex].maximum && value >= this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].barPointers[pointerIndex].value = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		  
        getBarPointerValue: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].value;
            else
                return null;
        },
		  
        setBarWidth: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].width = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		  
        getBarWidth: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].width;
            else
                return null;
        },
		  
        setBarDistanceFromScale: function (scaleIndex, pointerIndex, value) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length && value != null) {
                this.scaleEl[scaleIndex].barPointers[pointerIndex].distanceFromScale = value;
                if (this.contextEl.putImageData) {
                    if (this.scaleEl[this.scaleIndex].type == "thermometer")
                        this.initialize();
                    else
                        this._reDrawBarPointer();
                }
                else
                    this.initialize();
            }
        },
		  
        getBarDistanceFromScale: function (scaleIndex, pointerIndex) {
            if (scaleIndex < this.Model.scales.length && pointerIndex < this.Model.scales[scaleIndex].barPointers.length)
                return this.scaleEl[scaleIndex].barPointers[pointerIndex].distanceFromScale;
            else
                return null;
        },
		  
        setCustomLabelValue: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].value = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
		  
        getCustomLabelValue: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].value;
            else
                return null;
        },
		  
        setCustomLabelAngle: function (scaleIndex, customLabelIndex, value) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length && value != null) {
                this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle = value;
                if (this.contextEl.putImageData)
                    this._reDrawCustomLabel();
                else
                    this.initialize();
            }
        },
		  
        getCustomLabelAngle: function (scaleIndex, customLabelIndex) {
            if (scaleIndex < this.Model.scales.length && customLabelIndex < this.Model.scales[scaleIndex].customLabels.length)
                return this.scaleEl[scaleIndex].customLabels[customLabelIndex].textAngle;
            else
                return null;
        },
		  
        setRangeStartValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeStartValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startValue;
            else
                return null;
        },
		  
        setRangeEndValue: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endValue = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeEndValue: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endValue;
            else
                return null;
        },
		  
        setRangeStartWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeStartWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].startWidth;
            else
                return null;
        },
		  
        setRangeEndWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeEndWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].endWidth;
            else
                return null;
        },
		  
        setRangeDistanceFromScale: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeDistanceFromScale: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].distanceFromScale;
            else
                return null;
        },
		  
        setRangePosition: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].placement = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangePosition: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].placement;
            else
                return null;
        },
		  
        setRangeBorderWidth: function (scaleIndex, rangeIndex, value) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length && value != null) {
                this.scaleEl[scaleIndex].ranges[rangeIndex].border.width = value;
                if (this.contextEl.putImageData)
                    this._reDrawRange();
                else
                    this.initialize();
            }
        },
		  
        getRangeBorderWidth: function (scaleIndex, rangeIndex) {
            if (scaleIndex < this.Model.scales.length && rangeIndex < this.Model.scales[scaleIndex].ranges.length)
                return this.scaleEl[scaleIndex].ranges[rangeIndex].border.width;
            else
                return null;
        },
		  
        setLabelAngle: function (scaleIndex, labelIndex, angle) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && angle != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].angle = angle;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelAngle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].angle;
            else
                return null;
        },
		  
        setLabelStyle: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].type = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelStyle: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].type;
            else
                return null;
        },
		  
        setLabelPlacement: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].placement = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelPlacement: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].placement;
            else
                return null;
        },
		  
        setLabelXDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.x = value;
                if (this.contextEl.putImageData) this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelXDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.x;
            else
                return null;
        },
		  
        setLabelYDistanceFromScale: function (scaleIndex, labelIndex, value) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length && value != null) {
                this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.y = value;
                if (this.contextEl.putImageData)
                    this._reDrawLabel();
                else
                    this.initialize();
            }
        },
		  
        getLabelYDistanceFromScale: function (scaleIndex, labelIndex) {
            if (scaleIndex < this.Model.scales.length && labelIndex < this.Model.scales[scaleIndex].labels.length)
                return this.scaleEl[scaleIndex].labels[labelIndex].distanceFromScale.y;
            else
                return null;
        },
		  
        setTickAngle: function (scaleIndex, tickIndex, angle) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && angle != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].angle = angle;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickAngle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].angle;
            else
                return null;
        },
		  
        setTickWidth: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].width = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickWidth: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].width;
            else
                return null;
        },
		  
        setTickHeight: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].height = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickHeight: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].height;
            else
                return null;
        },
		  
        setTickStyle: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].type = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickStyle: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].type;
            else
                return null;
        },
		  
        setTickPlacement: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].placement = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickPlacement: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].placement;
            else
                return null;
        },
		  
        setTickXDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.x = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickXDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.x;
            else
                return null;
        },
		  
        setTickYDistanceFromScale: function (scaleIndex, tickIndex, value) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length && value != null) {
                this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.y = value;
                if (this.contextEl.putImageData) {
                    this._reDrawTickMark();
                }
                else
                    this.initialize();
            }
        },
		  
        getTickYDistanceFromScale: function (scaleIndex, tickIndex) {
            if (scaleIndex < this.Model.scales.length && tickIndex < this.Model.scales[scaleIndex].ticks.length)
                return this.scaleEl[scaleIndex].ticks[tickIndex].distanceFromScale.y;
            else
                return null;
        },
		  
        setScaleLocation: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].position.x = value.x;
                this.scaleEl[scaleIndex].position.y = value.y;
                this.initialize();
            }
        },
		  
        getScaleLocation: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return { "x": this.scaleEl[scaleIndex].position.x, "y": this.scaleEl[scaleIndex].position.y };
            else
                return null;
        },
		  
        setMaximumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
            if(value > this.scaleEl[scaleIndex].minimum)
                this.scaleEl[scaleIndex].maximum = value;
                this.initialize();
            }
        },
		  
        getMaximumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].maximum;
            else
                return null;
        },
		  
        setMinimumValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
            if(value < this.scaleEl[scaleIndex].maximum)
                this.scaleEl[scaleIndex].minimum = value;
                this.initialize();
            }
        },
		  
        getMinimumValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[this.scaleIndex].minimum;
            else
                return null;
        },
		  
        setScaleBarSize: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].width = value;
                this.initialize();
            }
        },
		  
        getScaleBarSize: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].width;
            else
                return null;
        },
		  
        setScaleBarLength: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].length = value;
                this.initialize();
            }
        },
		  
        setScaleStyle: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].type = value;
                this.initialize();
            }
        },
		  
        getScaleStyle: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].type;
            else
                return null;
        },
		  
        getScaleBarLength: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].length;
            else
                return null;
        },
		  
        setScaleBorderWidth: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].border.width = value;
                this.initialize();
            }
        },
		  
        setScaleDirection: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].direction = value;
                this.initialize();
            }
        },
		  
        getScaleDirection: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].direction;
            else
                return null;
        },
		  
        getScaleBorderWidth: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].border.width;
            else
                return null;
        },
		  
        setMajorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].majorIntervalValue = value;
                this.initialize();
            }
        },
		  
        getMajorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].majorIntervalValue;
            else
                return null;
        },
		  
        setMinorIntervalValue: function (scaleIndex, value) {
            if (scaleIndex < this.Model.scales.length && value != null) {
                this.scaleEl[scaleIndex].minorIntervalValue = value;
                this.initialize();
            }
        },
		  
        getMinorIntervalValue: function (scaleIndex) {
            if (scaleIndex < this.Model.scales.length)
                return this.scaleEl[scaleIndex].minorIntervalValue;
            else
                return null;
        },
		  
        _reDrawBarPointer: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.customLabelImage) ? this.customLabelImage() : !ej.isNullOrUndefined(this.rangeImage) ? this.rangeImage : !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setBarPointers();
                this._setMarkerPointers();
                this._setIndicators();
            }
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getBarPointerImage(), 0, 0);
                    this._setBarPointers();
                    this._setMarkerPointers();
                    this._setIndicators();
                }

            }
        },
		  
        _reDrawMarkerPointer: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.customLabelImage) ? this.customLabelImage() : !ej.isNullOrUndefined(this.rangeImage) ? this.rangeImage : !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setMarkerPointers();
            }
            else {
                if (this.contextEl.putImageData != "undefined") {
                    this.contextEl.putImageData(this._getMarkerPointerImage(), 0, 0);
                    this._setMarkerPointers();
                }

            }
        },
		  
        _reDrawCustomLabel: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.rangeImage) ? this.rangeImage : !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getCustomLabelImage(), 0, 0);
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        _reDrawRange: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.labelImage) ? this.labelImage : !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getRangeImage(), 0, 0);
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        _reDrawLabel: function () {
            if (this.Model.frame.backgroundImageUrl){
                var tmpData = !ej.isNullOrUndefined(this.tickImage) ? this.tickImage : !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getLabelImage(), 0, 0);
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        _reDrawTickMark: function () {
            if (this.Model.frame.backgroundImageUrl) {
                var tmpData = !ej.isNullOrUndefined(this.scaleImage) ? this.scaleImage : null;
                this.contextEl.putImageData(tmpData, 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();
            }
            else {
                this.contextEl.putImageData(this._getTickImage(), 0, 0);
                this._setTicks();
                this._setLabels();
                this._setRange();
                this._setCustomLabel();
                this._setIndicators();
                this._setBarPointers();
                this._setMarkerPointers();

            }
        },
		  
        refresh: function () {
            this._init();
        },
		  
		exportImage: function(fileName, fileType) {
			/// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
		    /// <param name="fileType" type="String">fileType to which the image has been saved</param>
		    if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10) {
		        return false;
		    }
		    else {
		        var image = this.canvasEl[0].toDataURL();
		        image = image.replace(/^data:[a-z]*;,/, '');
		        var image1 = image.split(',');
		        var byteString = atob(image1[1]);
		        var buffer = new ArrayBuffer(byteString.length);
		        var intArray = new Uint8Array(buffer);
		        for (var i = 0; i < byteString.length; i++) {
		            intArray[i] = byteString.charCodeAt(i);
		        }
		        var blob = new Blob([buffer], { type: "image/png" });
		        if (ej.browserInfo().name === "msie")
		            window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
		        else {
		            var pom = document.createElement('a');
		            var url = URL.createObjectURL(blob);
		            pom.href = url;
		            pom.setAttribute('download', fileName + '.' + fileType);
		            if (document.createEvent) {
		                var e = document.createEvent("MouseEvents");
		                e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
		                pom.dispatchEvent(e);
		            }
		            else if (pom.fireEvent) {
		                pom.fireEvent("onclick");
		            }
		        }
		        return true;
		    }
        },
		 
		resizeCanvas: function () {
		    if (_linearGaugeCount != 0)
		        _linearGaugeCount = _linearGaugeCount - 1;
		    else
		        _linearGaugeCount = $(".e-lineargauge").length - 1;
		    var state, chk = true;
		    if (!ej.isNullOrUndefined(this.GaugeEl.parent().attr("style")))
		        state = this.GaugeEl.parent().attr("style").split(";");
		    if (!ej.isNullOrUndefined(state)) {
		        $.each(state, function (key, val) {
		            while (val.indexOf("width") != -1) {
		                chk = val.indexOf("px") == -1 ? true : false;
		                break;
		            }
		        });
		    }
            if (chk) {
                var ratio = window.innerWidth / initialLinearDivWidth;
                this.model.width *=ratio;
				for(var i=0;this.model.scales[i]!=null;i++){
                this.model.scales[i].length *= ratio;
                for (var j = 0; this.model.scales[i].markerPointers[j] != null || this.model.scales[i].barPointers[j] != null || this.model.scales[i].indicators[j] != null || this.model.scales[i].customLabels[j] != null || this.model.scales[i].ranges[j] != null || this.model.scales[i].labels[j] != null || this.model.scales[i].ticks[j] != null; j++) {
                    if (!ej.isNullOrUndefined(this.model.scales[i].markerPointers[j])) {
                        this.model.scales[i].markerPointers[j].length *= ratio;
                        this.model.scales[i].markerPointers[j].width *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].barPointers[j])) {
                        this.model.scales[i].barPointers[j].distanceFromScale *= ratio;
                        this.model.scales[i].barPointers[j].width *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].indicators[j]) && this.model.scales[i].showIndicators) {
                        this.model.scales[i].indicators[j].height *= ratio;
                        this.model.scales[i].indicators[j].width *= ratio;
                        this.model.scales[i].indicators[j].position.x *= ratio;
                        this.model.scales[i].indicators[j].textLocation.x *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].ticks[j])) {
                        this.model.scales[i].ticks[j].length *= ratio;
                        this.model.scales[i].ticks[j].width *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].ranges[j])) {
                        this.model.scales[i].ranges[j].startWidth *= ratio;
                        this.model.scales[i].ranges[j].endWidth *= ratio;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[i].customLabels[j])) {
                        if (this.model.scales[i].customLabels[j].positionType != "outer")
                            this.model.scales[i].customLabels[j].position.x *= ratio;
                        this.model.scales[i].customLabels[j].font.size = (parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0]) * ratio < 10) ? "10px" : ((parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0]) * ratio) > this._customLblMaxSize) ? this._customLblMaxSize.toString() + 'px' : ((parseFloat(this.model.scales[i].customLabels[j].font.size.match(/\d+/)[0])) * ratio).toString() + 'px';
                    }
				}
				}
				//this._initialize();
				this._render();
				if (_linearGaugeCount == 0) {
				    initialLinearDivWidth = window.innerWidth;
				}
            }
        },
        //Client-Side Events
	 
		_onDrawTicks: function (tickAngle, tickValue) {
		    var tick = { index: this.tickIndex, element: this.tickEl[this.tickIndex], angle: parseInt(tickAngle), value: tickValue }
            var data = { Object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawTicks", data);
        },
		 
		_onDrawLabels: function (labelAngle) {
		    var label = { index: this.labelIndex, element: this.labelEl[this.labelIndex], angle: parseInt(labelAngle), value: this.labelValue }
		    var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, label: label };
		    this._trigger("drawLabels", data);
		},
		 
        _onDrawBarPointers: function (pointerValue) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, barPointerIndex: this.barPointerIndex, barElement: this.barPointerEl[this.barPointerIndex], context: this.contextEl, style: this.style, position: this.location, pointerValue: pointerValue };
            this._trigger("drawBarPointers", data);
        },
		 
        _onDrawMarkerPointers: function (pointerAngle, pointerValue) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, markerPointerIndex: this.markerPointerIndex, markerElement: this.markerPointerEl[this.markerPointerIndex], context: this.contextEl, style: this.style, position: this.location, pointerValue: pointerValue, pointerAngle: parseInt(pointerAngle) };
            this._trigger("drawMarkerPointers", data);
        },
		 
        _onDrawRange: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, rangeIndex: this.rangeIndex, rangeElement: this.rangeEl[this.rangeIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawRange", data);
        },
	 
        _onDrawCustomLabel: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, customLabelIndex: this.customLabelIndex, customLabelElement: this.customLabelEl[this.customLabelIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawCustomLabel", data);
        },
		 
        _onDrawIndicators: function (location, style) {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, indicatorIndex: this.indicatorIndex, indicatorEl: this.indicatorEl[this.indicatorIndex], context: this.contextEl, style: this.style, position: this.location };
            this._trigger("drawIndicators", data);
        },
		 
        onLoad: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("load", data);
        },
	 
        _onInit: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("init", data);
        },
		 
        _onRenderComplete: function () {
            var data = { object: this, Model: this.Model, scaleElement: this.Model.scales, context: this.contextEl };
            this._trigger("renderComplete", data);
        },
        
        _onMouseClick: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, markerPointer: markerpointer };
            this._trigger("mouseClick", data);
        },
        
        _onMouseClickMove: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, markerPointer: markerpointer };
            this._trigger("mouseClickMove", data);
        },
        
        _onMouseClickUp: function (pointerValue) {
            var markerpointer = { index: this.markerPointerIndex, element: this.markerPointerEl[this.markerPointerIndex], value: pointerValue };
            var data = { object: this, scaleElement: this.model.scales[this.scaleIndex], scaleIndex: this.scaleIndex, context: this.contextEl, style: this.style, position: this.location, markerPointer: markerpointer };
            this._trigger("mouseClickUp", data);
        },
        //_trigger: function (type, event, data) {
        //    var fn = this.Model[type];
        //    event = $.Event(event);
        //    event.type = type;
        //    $(this.GaugeEl).trigger(event, data);
        //    return !($.isFunction(fn) &&
        //        fn(this, data) === false ||
        //        event.isDefaultPrevented());
        //},

        //trigger: function (element, eventName, e) {
        //    e = $.extend(e || {}, new $.Event(eventName));
        //    e.stopPropagation();
        //    $(element).trigger(e);
        //    return e.isDefaultPrevented();
        //},
		 
        _restoreWidth: function () {
            this.scaleEl[this.scaleIndex].length = this.scaleEl[this.scaleIndex].length + this.bottomRadius + this.scaleEl[this.scaleIndex].width;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].width / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.bottomRadius - this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] - this.scaleEl[this.scaleIndex].width / 2;
            }
        },
		 
        _modifyWidth: function () {
            this.scaleEl[this.scaleIndex].length = this.scaleEl[this.scaleIndex].length - this.bottomRadius - this.scaleEl[this.scaleIndex].width;
            if (this.Model.orientation == "Vertical") {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartY[this.scaleIndex] = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
            }
            else {
                if (this.scaleEl[this.scaleIndex].direction == "clockwise")
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.bottomRadius + this.scaleEl[this.scaleIndex].width / 2;
                else
                    this.scaleStartX[this.scaleIndex] = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2;
            }
        },
        // return y positions for vertical gauge
	 
        _getClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = (value - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100);
            return linePosition;
        },
		 
        _getCounterClockwiseLinePosition: function (value) {
            var tempVal, linePosition;
            tempVal = this.scaleEl[this.scaleIndex].maximum - value + this.scaleEl[this.scaleIndex].minimum;
            tempVal = (tempVal - this.scaleEl[this.scaleIndex].minimum) / (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum) * 100;
            linePosition = this.Model.orientation == "Vertical" ? this.scaleStartY[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100) : this.scaleStartX[this.scaleIndex] + parseInt((tempVal * this.scaleEl[this.scaleIndex].length) / 100);
            return linePosition;
        },
		 
        _getValue: function (position) {
            var tempVal;
            if (this.Model.orientation == "Vertical")
                tempVal = ((position.y - this.scaleStartY[this.scaleIndex]) / this.scaleEl[this.scaleIndex].length) * 100;
            else
                tempVal = (((position.x - this.scaleStartX[this.scaleIndex])) / this.scaleEl[this.scaleIndex].length) * 100;
            var value = ((tempVal * (this.scaleEl[this.scaleIndex].maximum - this.scaleEl[this.scaleIndex].minimum)) + this.scaleEl[this.scaleIndex].minimum) / 100;
            if (this.scaleEl[this.scaleIndex].direction == "counterclockwise")
                value = this.scaleEl[this.scaleIndex].maximum - value;
            else
                value = this.scaleEl[this.scaleIndex].minimum + value;
            return value;
        },
		 
        _getPointerXPosition: function (element) {
            var startX, angle;
            if (this.Model.orientation == "Vertical") {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
                if (this.markerPlacement == "near") {
                    startX = this.scaleStartX[this.scaleIndex] + element.distanceFromScale;
                    angle = 180;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - (Math.sqrt(element.length * element.length + element.width * element.width) / 2) + element.distanceFromScale;
                    else
                        startX = this.scaleStartX[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.width / 2 + element.distanceFromScale;
                    angle = 0;
                }
            }
            else {
                if (this.markerPlacement == "far") {
                    startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width + this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 90;
                }
                if (this.markerPlacement == "near") {
                    startX = this.scaleStartY[this.scaleIndex] - this.scaleEl[this.scaleIndex].border.width / 2 + element.distanceFromScale;
                    angle = 270;
                }
                if (this.markerPlacement == "center") {
                    if (element.type == "circle")
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - (Math.sqrt(element.length * element.length + element.width * element.width) / 2) + element.distanceFromScale;
                    else
                        startX = this.scaleStartY[this.scaleIndex] + this.scaleEl[this.scaleIndex].width / 2 - element.length / 2 + element.distanceFromScale;
                    angle = 90;
                }
            }
            return { "startx": startX, "angle": angle };
        },
		 
        _hexFromRGB: function (r, g, b) {
            var hex = [r.toString(16), g.toString(16), b.toString(16)];
            $.each(hex, function (nr, val) { if (val.length === 1) { hex[nr] = "0" + val; } });
            return hex.join("").toUpperCase();
        },
	 
        _setGradientColor: function (element, gradients, options) {
            if (options.Name || typeof (options) === "string") {
                gradients.addColorStop(0, options);
                gradients.addColorStop(1, options);
            }
            else
                $.each(options, function (index, colorElement) {
                    gradients.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : colorElement.color);
                });
        },
		 
        _getFontString: function (element, font) {
            return font.fontStyle + " " + ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily;
        },
	 
        _setPointerDimension: function (style, element) {
            if (element.Model.orientation) {
                if (element.Model.orientation == "horizontal") {
                    var tempWidth = style.width;
                    var tempHeight = style.height;
                    style.height = tempWidth;
                    style.width = tempHeight;
                }
            }
            return style;
        },
		 
        _setContextRotation: function (style, element) {
            element.contextEl.rotate(Math.PI * (style.angle / 180));
        },
	 
        wireEvents: function () {
            this.startEv = 'mousedown';
            this.moveEv = 'mousemove';
            this.endEv = 'mouseup';
            this.scrollEv = 'wheel';
            this.onMouseMoveHandler = $.proxy(this._onMouseMove, this);
            //this.onMouseUpHandler = $.proxy(this._onMouseUp, this);
            this.onHoverOCustomLabel = $.proxy(this._onHoverOCustomLabel, this);
            this.onLeaveOCustomLabel = $.proxy(this._onLeaveOCustomLabel, this);
            if(!this.model.readOnly)
            this.onMouseDownHandler = $.proxy(this._onMouseDown, this);
            if (this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip) {
                $(this.canvasEl).bind(this.moveEv, this.onMouseMoveHandler);
                $(this.canvasEl).bind(this.scrollEv, this.onMouseMoveHandler);
            }
            this.element.bind(this.startEv, this.onMouseDownHandler);
            if (this.model.tooltip.showCustomLabelTooltip) {
                $('.' + this._id + 'outercustomlbl').bind("mouseenter", this.onHoverOCustomLabel);
                $('.' + this._id + 'outercustomlbl').bind("mouseleave", this.onLeaveOCustomLabel);
            }
        },
		 
        unWireEvents: function () {
            this.element.unbind(this.startEv, this.onMouseDownHandler);
        },
       
        _onHoverOCustomLabel: function (e) {
            if (e.currentTarget.innerHTML != null || e.currentTarget.innerHTML != "")
                this._showTooltip(e, e.currentTarget.innerHTML);
        },
        
        _onLeaveOCustomLabel: function () {
            this._hideTooltip();
        },
		 
        _blockDefaultActions: function (e) {
            e.cancelBubble = true;
            e.returnValue = false;
            if (e.preventDefault) e.preventDefault();
            if (e.stopPropagation) e.stopPropagation();
        },
		 
        _onMouseDown: function (e) {
            var endPoint, position, greaterValue, lesserValue, startX;
            this._blockDefaultActions(e);
            this._mouseDown = true;
            endPoint = { "x": e.pageX - $(this.canvasEl).offset().left - (this.Model.frame.outerWidth + this.Model.frame.innerWidth), "y": e.pageY - $(this.canvasEl).offset().top - (this.Model.frame.outerWidth + this.Model.frame.innerWidth) };
            var self = this;
            $.each(this.Model.scales, function (index, element) {
                self.scaleIndex = index;
                if (element.markerPointers != null) {
                    self.markerPointerEl = element.markerPointers;
                    $.each(element.markerPointers, function (mpIndex, mpElement) {
                        if (self.scaleEl[self.scaleIndex].direction == "clockwise")
                            position = self._getClockwiseLinePosition(mpElement.value);
                        else
                            position = self._getCounterClockwiseLinePosition(mpElement.value);
                        greaterValue = position + mpElement.width;
                        lesserValue = position - mpElement.width;
                        startX = self._getPointerXPosition(mpElement).startx;
                        var isOffset = self._isBetween(((self.Model.orientation == "horizontal" ? endPoint.y : endPoint.x) - mpElement.width), ((self.Model.orientation == "horizontal" ? endPoint.y : endPoint.x) + mpElement.width), startX);
                        if ((self.Model.orientation == "horizontal" ? self._isBetween(lesserValue, greaterValue, endPoint.x) : self._isBetween(lesserValue, greaterValue, endPoint.y)) && isOffset)
                            self.activeElement = mpElement;
                        if (self.Model.scales[self.scaleIndex].barPointers[mpIndex] != null)
                            self.activeBarElement = self.Model.scales[self.scaleIndex].barPointers[mpIndex];
                        if (self.model.mouseClick)
                            self._onMouseClick(mpElement.value);
                        self.onMouseMoveHandler = $.proxy(self._onMouseMove, self);
                        self.onMouseUpHandler = $.proxy(self._onMouseUp, self);
                        $(document).bind(self.moveEv, self.onMouseMoveHandler);
                        $(document).bind(self.endEv, self.onMouseUpHandler);
                    });
                }
            });
        },
		 
        _isBetween: function (first, last, number) {
            return (first < last ? number >= first && number <= last : number >= last && number <= first);
        },
		 
        _onMouseUp: function () {
            this._mouseDown = false;
            this.mouseMove = false;
            $(document).unbind(self.moveEv, self.onMouseMoveHandler);
            $(document).unbind(self.endEv, self.onMouseUpHandler);
            if (this.model.mouseClickUp)
                this._onMouseClickUp(this.activeElement.value);
            this.activeElement = null;
        },
		 
        _onMouseMove: function (e) {
            if (this._mouseDown && !ej.isNullOrUndefined(this.activeElement)) {
                this._blockDefaultActions(e);
                var endPoint = { "x": e.pageX - $(this.canvasEl).offset().left - (this.Model.frame.outerWidth + this.Model.frame.innerWidth), "y": e.pageY - $(this.canvasEl).offset().top - (this.Model.frame.outerWidth + this.Model.frame.innerWidth) };
                this.activeElement.value = this._getValue(endPoint);
                this.value(this.activeElement.value);
                if (this.model.mouseClickMove)
                    this._onMouseClickMove(this.activeElement.value);
                if (this.activeBarElement)
                    this.activeBarElement.value = this._getValue(endPoint);
                if (this.contextEl.putImageData)
                    this._reDrawBarPointer();
                else
                    this._init();
            }
            else {
                if (this.model.tooltip.showCustomLabelTooltip || this.model.tooltip.showLabelTooltip) {
                    var hit = false;
                    for (var i = 0; this._savedPoints[i] != null; i++) {
                        var current = { "X": e.pageX - $(this.canvasEl).offset().left, "Y": e.pageY - $(this.canvasEl).offset().top };
                     
                        if (current.X > this._savedPoints[i].startX && current.X < (this._savedPoints[i].startX + this._savedPoints[i].width) && current.Y > this._savedPoints[i].startY && current.Y < (this._savedPoints[i].startY + this._savedPoints[i].height)) {
                            this._showTooltip(e, this._savedPoints[i].value);
                            hit = true;
                        }
                        else if (hit == false)
                            this._hideTooltip();
                    }
                }
            }
        },
     
        _showTooltip: function (event, val) {
            var tooltipText = val + "";
            var tooltipdiv = $('.tooltipDiv');
            if (tooltipdiv.length == 0) {
                tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 105; display: block;'></div>");
                $(document.body).append(tooltipdiv);
            }
            if (this.model.tooltip.templateID != "" && this.model.tooltip.templateID != null) {
                var cloneNode = $("#" + this.model.tooltip.templateID).clone();
                $('.tooltipDiv')[0].innerHTML = "";
                $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                $(tooltipdiv).css({
                    'background-color': this.model.backgroundColor,
                    'border': '1px solid #bbbcbb',
                    'border-radius': '3px',
                    'color': '#565656'
                });
                tooltipdiv.html(tooltipdiv.html().replace('#label#', tooltipText));
            } else {
                $(tooltipdiv).html(tooltipText);
                $(tooltipdiv).css({
                    'background-color': 'white',
                    'border': '2px solid #bbbcbb',
                    'position': 'absolute',
                    'padding': '10px 20px',
                    'margin-top': '5px',
                    'text-align': 'left',
                    'font': '12px Segoe UI',
                    'font-stretch': 'condensed',
                    'display': 'inline-block',
                    'border-radius': '3px',
                    'color': '#565656',
                    'width': 'auto'
                });
            }
            var tooltipMargin = 10;
            var xPos = event.pageX + tooltipMargin;
            var yPos = event.pageY + tooltipMargin;
            xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
            yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());
            $(tooltipdiv).css("left", xPos);
            $(tooltipdiv).css("top", yPos);
            $('.tooltipDiv').show();
        },
        
        _hideTooltip: function () {
            $('.tooltipDiv').remove();//.fadeOut(0, "linear");
        },
       
        _setTheme: function () {
            var selectedTheme = ej.datavisualization.LinearGauge.Themes[this.model.theme];
            this._setThemeColors(selectedTheme);
        },
		 
        _setThemeColors: function (selectedTheme) {
            var result = [], jsonObj = ej.datavisualization.LinearGauge.Themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                this.model.backgroundColor = ((!this.model.backgroundColor || this.model.backgroundColor == jsonObj[result[th]].scales.backgroundColor) ? selectedTheme.scales.backgroundColor : this.model.backgroundColor);
                this.model.borderColor = ((!this.model.borderColor || this.model.borderColor == jsonObj[result[th]].scales.border.color) ? selectedTheme.scales.border.color : this.model.borderColor);
                this.model.labelColor = ((!this.model.labelColor || this.model.labelColor == jsonObj[result[th]].scales.labels.labelColor) ? selectedTheme.scales.labels.labelColor : this.model.labelColor);
                this.model.tickColor = ((!this.model.tickColor || this.model.tickColor == jsonObj[result[th]].scales.ticks.color) ? selectedTheme.scales.ticks.color : this.model.tickColor);

                for (var i = 0; i < this.model.scales.length; i++) {
                    for (var m = 0; m < this.model.scales[i].markerPointers.length; m++) {
                        this.model.scales[i].markerPointers[m].backgroundColor = (!this.model.scales[i].markerPointers[m].backgroundColor || this.model.scales[i].markerPointers[m].backgroundColor == jsonObj[result[th]].scales.markerPointers.backgroundColor) ? selectedTheme.scales.markerPointers.backgroundColor : this.model.scales[i].markerPointers[m].backgroundColor;
                        this.model.scales[i].markerPointers[m].border.color = (!this.model.scales[i].markerPointers[m].border.color || this.model.scales[i].markerPointers[m].border.color == jsonObj[result[th]].scales.markerPointers.border.color) ? selectedTheme.scales.markerPointers.border.color : this.model.scales[i].markerPointers[m].border.color;
                    }
                    for (var b = 0; b < this.model.scales[i].barPointers.length; b++) {
                        this.model.scales[i].barPointers[b].backgroundColor = (!this.model.scales[i].barPointers[b].backgroundColor || this.model.scales[i].barPointers[b].backgroundColor == jsonObj[result[th]].scales.barPointers.backgroundColor) ? selectedTheme.scales.barPointers.backgroundColor : this.model.scales[i].barPointers[b].backgroundColor;
                        this.model.scales[i].barPointers[b].border.color = (!this.model.scales[i].barPointers[b].border.color || this.model.scales[i].barPointers[b].border.color == jsonObj[result[th]].scales.barPointers.border.color) ? selectedTheme.scales.barPointers.border.color : this.model.scales[i].barPointers[b].border.color;
                    }
                }
            }
        }
    });
    
    ej.datavisualization.LinearGauge.TickType = {
          
        MajorInterval: "majorinterval",
          
        MinorInterval: "minorinterval"
    };
      
    ej.datavisualization.LinearGauge.LabelType = {
          
        Major: "major",
          
        Minor: "minor"
    };
      
    ej.datavisualization.LinearGauge.FontStyle = {
          
        Bold: "bold",
          
        Italic: "italic",
          
        Regular: "regular",
          
        Strikeout: "strikeout",
          
        Underline: "underline"
    };


      
    ej.datavisualization.LinearGauge.PointerPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.LinearGauge.TickPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.LinearGauge.LabelPlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      
    ej.datavisualization.LinearGauge.RangePlacement = {
          
        Near: "near",
          
        Far: "far",
          
        Center: "center"
    };
      

    ej.datavisualization.LinearGauge.UnitTextPlacement = {
          
        Front: "front",
          
        Back: "back"
    };
      

    ej.datavisualization.LinearGauge.Directions = {
          
        Clockwise: "clockwise",
          
        CounterClockwise: "counterclockwise"
    };
      
    ej.datavisualization.LinearGauge.ScaleType = {
          
        Rectangle: "rectangle",
          
        RoundedRectangle: "roundedrectangle",
          
        Thermometer: "thermometer"
    };
      
    ej.datavisualization.LinearGauge.IndicatorType = {
          
        Rectangle: "rectangle",
          
        Circle: "circle",
          
        RoundedRectangle: "roundedrectangle",
          
        Text: "text"
    };
      
    ej.datavisualization.LinearGauge.MarkerType = {
          
        Rectangle: "rectangle",
          
        Triangle: "triangle",
          
        Ellipse: "ellipse",
          
        Diamond: "diamond",
          
        Pentagon: "pentagon",
          
        Circle: "circle",
          
        Star: "star",
          
        Slider: "slider",
          
        Pointer: "pointer",
          
        Wedge: "wedge",
          
        Trapezoid: "trapezoid",
          
        RoundedRectangle: "roundedrectangle"
    };
      
    ej.datavisualization.LinearGauge.CustomLabelPositionType = {
          
        Inner: "inner",
          
        Outer: "outer"
    };
      
    ej.datavisualization.LinearGauge.OuterCustomLabelPosition = {
          
        Left: "left",
          
        Right: "right",
          
        Top: "top",
          
        Bottom: "bottom",
    };
	  
    ej.datavisualization.LinearGauge.Themes = {
	  
        flatlight: {
            scales: {
                backgroundColor: "#FFFFFF",
                border: { color: "#1d1e1e" },
                barPointers: {
                    backgroundColor: "#8abc3b",
                    border: { color: "#8abc3b" }
                },
                markerPointers: {
                    backgroundColor: "#212121",
                    border: { color: "#212121" }
                },
                ticks: {
                    color: "#1d1e1e"
                },
                labels: {
                    labelColor: "#222222"
                }
            }
        },
		  
        flatdark: {
            scales: {
                backgroundColor: "#808080",
                border: { color: "#808080" },
                barPointers: {
                    backgroundColor: "#8abc3b",
                    border: { color: "#8abc3b" }
                },
                markerPointers: {
                    backgroundColor: "#CCCCCC",
                    border: { color: "#CCCCCC" }
                },
                ticks: {
                    color: "#808080"
                },
                labels: {
                    labelColor: "#CCCCCC"
                }
            }
        }
    };

})(jQuery, Syncfusion);
;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/


(function ($, ej, undefined) {

    ej.widget({ "ejDigitalGauge": "ej.datavisualization.DigitalGauge" }, {
     
        element: null,
     
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-digitalgauge",
        
        defaults:   {
              

            segmentData: {
                "0": [0, 1, 2, 3, 4, 5, 14, 15], "1": [1, 2], "2": [0, 14, 1, 6, 8, 4, 3, 15], "3": [0, 1, 2, 3, 6, 8, 14, 15], "4": [1, 2, 5, 6, 8], "5": [0, 2, 3, 5, 6, 8, 14, 15], "6": [0, 2, 3, 4, 5, 6, 8, 14, 15],
                "7": [0, 1, 2, 14], "8": [0, 1, 2, 3, 4, 5, 6, 8, 14, 15], "9": [0, 1, 2, 3, 5, 6, 8, 14, 15], "A": [0, 1, 2, 4, 5, 6, 8, 14], "B": [0, 1, 2, 3, 7, 9, 8, 14, 15], "C": [0, 3, 4, 5, 14, 15],
                "D": [0, 1, 2, 3, 7, 9, 14, 15], "E": [0, 3, 4, 5, 6, 8, 14, 15], "F": [0, 4, 5, 6, 8, 14], "G": [0, 2, 3, 4, 5, 8, 14, 15], "H": [1, 2, 4, 5, 6, 8], "I": [0, 3, 7, 9, 14, 15],
                "J": [1, 2, 3, 4, 15], "K": [4, 5, 6, 10, 11], "L": [3, 4, 5, 15], "M": [1, 2, 4, 5, 10, 13], "N": [1, 2, 4, 5, 11, 13], "O": [0, 1, 2, 3, 4, 5, 14, 15], "P": [0, 1, 4, 5, 6, 8, 14],
                "Q": [0, 1, 2, 3, 4, 5, 11, 14, 15], "R": [0, 1, 4, 5, 6, 8, 11, 14], "S": [0, 2, 3, 5, 6, 8, 14, 15], "T": [0, 7, 9, 14], "U": [1, 2, 3, 4, 5, 15], "V": [4, 5, 10, 12], "W": [1, 2, 4, 5, 11, 12],
                "X": [10, 11, 12, 13], "Y": [1, 5, 6, 7, 8], "Z": [0, 3, 10, 12, 14, 15]
            },
              
            matrixSegmentData: {
                "1": [0, 3, 0, 4, 1, 1, 1, 2, 1, 3, 1, 4, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "2": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 2, 5, 2, 5, 1, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "3": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 1, 5, 1, 6, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5],
                "4": [0, 3, 0, 4, 0, 5, 1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 4, 2, 5, 3, 0, 3, 1, 3, 4, 3, 5, 4, 0, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4, 6, 5],
                "5": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "6": [0, 3, 0, 4, 0, 5, 0, 6, 1, 2, 1, 3, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 2, 5, 3, 5, 6, 5, 7, 6, 3, 6, 4, 6, 5, 6, 6],
                "7": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "8": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "9": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 4, 6, 5],
                "0": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "a": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "b": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "c": [1, 3, 1, 4, 1, 5, 1, 6, 2, 2, 2, 3, 3, 1, 3, 2, 4, 1, 4, 2, 5, 2, 5, 3, 6, 3, 6, 4, 6, 5, 6, 6],
                "d": [0, 6, 0, 7, 1, 6, 1, 7, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 5, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "e": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "f": [0, 4, 0, 5, 0, 6, 0, 7, 1, 3, 1, 4, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "g": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "h": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "i": [0, 3, 0, 4, 2, 1, 2, 2, 2, 3, 2, 4, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "j": [1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "k": [0, 1, 0, 2, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "l": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 5, 4, 6, 5, 5, 5, 6, 6, 5, 6, 6],
                "m": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 0, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 3, 5, 4, 5, 6, 5, 7, 6, 0, 6, 1, 6, 3, 6, 4, 6, 6, 6, 7],
                "n": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 0, 2, 1, 2, 2, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 0, 5, 1, 5, 6, 5, 7, 6, 0, 6, 1, 6, 6, 6, 7],
                "o": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 2, 5, 2, 6, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "p": [1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 3, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 6, 1, 6, 2],
                "q": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7],
                "r": [0, 1, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 2, 1, 2, 2, 2, 6, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "s": [1, 2, 1, 3, 1, 4, 1, 5, 2, 1, 2, 2, 3, 1, 3, 2, 4, 3, 4, 4, 5, 4, 5, 5, 6, 1, 6, 2, 6, 3, 6, 4],
                "t": [0, 3, 0, 4, 1, 3, 1, 4, 2, 1, 2, 2, 2, 3, 2, 4, 2, 5, 2, 6, 3, 3, 3, 4, 4, 3, 4, 4, 5, 3, 5, 4, 6, 4, 6, 5, 6, 6, 6, 7],
                "u": [1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "v": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 5, 4, 5, 5, 6, 4],
                "w": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 0, 2, 1, 2, 3, 2, 4, 2, 6, 2, 7, 3, 0, 3, 1, 3, 3, 3, 4, 3, 6, 3, 7, 4, 0, 4, 1, 4, 3, 4, 4, 4, 6, 4, 7, 5, 0, 5, 1, 5, 2, 5, 3, 5, 4, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "x": [1, 1, 1, 2, 1, 6, 1, 7, 2, 2, 2, 3, 2, 5, 2, 6, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 2, 5, 3, 5, 5, 5, 6, 6, 1, 6, 2, 6, 6, 6, 7],
                "y": [1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 2, 5, 3, 6, 1, 6, 2],
                "z": [1, 2, 1, 3, 1, 4, 1, 5, 1, 6, 1, 7, 2, 6, 2, 7, 3, 5, 3, 6, 4, 4, 4, 5, 5, 3, 5, 4, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "A": [0, 3, 0, 4, 1, 2, 1, 3, 1, 4, 1, 5, 2, 2, 2, 3, 2, 4, 2, 5, 3, 1, 3, 2, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "B": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 3, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "C": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 4, 0, 4, 1, 5, 1, 5, 2, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "D": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "E": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "F": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "G": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 0, 2, 1, 3, 0, 3, 1, 3, 4, 3, 5, 3, 6, 4, 0, 4, 1, 4, 6, 5, 1, 5, 2, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "H": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "I": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "J": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 5, 1, 6, 2, 5, 2, 6, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 4, 6, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "K": [0, 1, 0, 2, 0, 5, 0, 6, 1, 1, 1, 2, 1, 4, 1, 5, 2, 1, 2, 2, 2, 3, 2, 4, 3, 1, 3, 2, 3, 3, 4, 1, 4, 2, 4, 3, 4, 4, 5, 1, 5, 2, 5, 4, 5, 5, 6, 1, 6, 2, 6, 5, 6, 6],
                "L": [0, 1, 0, 2, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "M": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 5, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "N": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 4, 2, 6, 2, 7, 3, 1, 3, 2, 3, 5, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "O": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5],
                "P": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2],
                "Q": [0, 2, 0, 3, 0, 4, 0, 5, 1, 1, 1, 2, 1, 5, 1, 6, 2, 0, 2, 1, 2, 6, 2, 7, 3, 0, 3, 1, 3, 6, 3, 7, 4, 0, 4, 1, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 5, 5, 6, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                "R": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 1, 3, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 3, 3, 4, 3, 5, 3, 6, 4, 1, 4, 2, 4, 5, 5, 1, 5, 5, 6, 2, 6, 1, 6, 2, 6, 6],
                "S": [0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 1, 1, 1, 2, 2, 1, 2, 2, 3, 2, 3, 3, 3, 4, 3, 5, 4, 5, 4, 6, 5, 5, 5, 6, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "T": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 4, 2, 4, 3, 4, 4, 4, 5, 4],
                "U": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 6, 4, 7, 5, 1, 5, 2, 5, 6, 5, 7, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6],
                "V": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 2, 3, 3, 3, 5, 3, 6, 4, 3, 4, 4, 4, 5, 4, 6, 5, 4, 5, 5, 6, 4],
                "W": [0, 1, 0, 2, 0, 6, 0, 7, 1, 1, 1, 2, 1, 6, 1, 7, 2, 1, 2, 2, 2, 6, 2, 7, 3, 1, 3, 2, 3, 6, 3, 7, 4, 1, 4, 2, 4, 4, 4, 6, 4, 7, 5, 1, 5, 2, 5, 3, 5, 5, 5, 6, 5, 7, 6, 1, 6, 2, 6, 6, 6, 7],
                "X": [0, 0, 0, 1, 0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 2, 4, 3, 4, 4, 4, 5, 5, 1, 5, 2, 5, 5, 5, 6, 6, 0, 6, 1, 6, 6, 6, 7],
                "Y": [0, 0, 0, 1, 0, 6, 0, 7, 1, 0, 1, 1, 1, 6, 1, 7, 2, 1, 2, 2, 2, 5, 2, 6, 3, 2, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 5, 3, 5, 4, 6, 3, 6, 4],
                "Z": [0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 1, 6, 1, 7, 2, 5, 2, 6, 3, 4, 3, 5, 4, 2, 4, 3, 5, 1, 5, 2, 6, 1, 6, 2, 6, 3, 6, 4, 6, 5, 6, 6, 6, 7],
                ",": [5, 3, 5, 4, 5, 5, 6, 4, 6, 5, 7, 3, 7, 4],
                ":": [1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5],
                "%": [0, 6, 0, 7, 1, 1, 1, 2, 1, 5, 1, 6, 2, 1, 2, 2, 2, 4, 2, 5, 3, 3, 3, 4, 4, 2, 4, 3, 5, 1, 5, 2, 5, 4, 5, 5, 6, 0, 6, 1, 6, 4, 6, 5],
                "!": [0, 3, 0, 4, 0, 5, 1, 3, 1, 4, 1, 5, 2, 3, 2, 4, 2, 5, 3, 3, 3, 4, 3, 5, 4, 3, 4, 4, 4, 5, 5, 3, 5, 4, 5, 5, 7, 4],
                "(": [0, 2, 0, 3, 1, 1, 1, 2, 2, 1, 2, 2, 3, 1, 3, 2, 4, 1, 4, 2, 5, 1, 5, 2, 6, 1, 6, 2, 7, 2, 7, 3],
                ")": [0, 5, 0, 6, 1, 6, 1, 7, 2, 6, 2, 7, 3, 6, 3, 7, 4, 6, 4, 7, 5, 6, 5, 7, 6, 6, 6, 7, 7, 5, 7, 6],
                ".": [5, 3, 5, 4, 5, 5, 6, 3, 6, 4, 6, 5, 7, 3, 7, 4, 7, 5]
            },
              
            frame: {
                  
                backgroundImageUrl: null,
                  
                innerWidth: 6,
                  
                outerWidth: 10,
            },
              
            height: 150,
              
            width: 400,
              
            enableResize: false,
              
            themes: "flatlight",
              
            items: null,
              
            init: null,
              
            load: null,
              
            renderComplete: null,
              
            itemRendering: null,
              
            value: "text"
        },

          
        dataTypes: {
            segmentData: "data",
            matrixSegmentData: "data",
            items: "array"
        },

          
        _setValues: function () {
            this.gaugeEl = this.element;
            this.segmentCount = null;
            this.contextEl = null;
            this.style = null;
            this._value = null;
            this.location = null;
            this.canvasEl = null;
            this.segement16X = null;
            this.segment16Y = null;
            this.segmentHeight = null;
            this.segmentAngle = null;
            this.startX = 5;
            this.startY = 5;
            this.gradient = null;
            this.itemIndex = null;
            this.characterSpace = null;
            this.outerImage = null;
            this.radius = null;
            this.frameOuterLocation = null;
            this.frameInnerLocation = null;
            this.glassFrameLocation = null;
            this.glassFrameStyle = null;
            this.frameOuterStyle = null;
            this.character = null;
            this.frameInnerStyle = null;
            this._itemInitialize();
        },
        observables: ["value"],
        _tags: [{
            tag: "items",
            attr: ["characterSettings.count", "characterSettings.opacity", "characterSettings.spacing", "characterSettings.type", "enableCustomFont", "segmentSettings.color", "segmentSettings.gradient", "segmentSettings.length", "segmentSettings.opacity", "segmentSettings.spacing", "segmentSettings.width", "shadowBlur", "shadowOffsetX", "shadowOffsetY", "textAlign", "shadowColor", "textColor", "font.size", "font.fontFamily", "font.fontStyle", "position.x", "position.y"]
        }],
        value: ej.util.valueFunction("value"),


          
        _destroy: function () {
            this.element.empty().removeClass("e-widget");
        },
          
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "height":
                        this.model.height = options[option];
                        break;
                    case "width":
                        this.model.width = options[option];
                        break;
                    case "items":
                        this.model.items = options[option];
                        this._itemInitialize();
                        break;
                    case "frame":
                        $.extend(this.model.frame, options[option]); break;
                    case "themes": this.model.themes = options[option]; break;
                    case "value":
                        for (var i = 0; this.model.items[i] != null; i++)
                            this.model.items[i].value = this.value();
                        break;
                }
            }
            this.refresh();
        },
          
        _init: function () {
            _digitalGaugeCount = $(".e-digitalgauge").length;
            initialGaugeCount = _digitalGaugeCount;
            this._setValues();
            this._trigger("load");
            this._setTheme();
            this._initialize();
            this._onWindowResize();
        },
          
        _onWindowResize: function () {
            if (this.model.enableResize) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this.resizeCanvas);
                else
                    this._on($(window), "orientationchange", this.resizeCanvas);
            }
        },
          
        _setTheme: function () {
            var selectedThem = ej.datavisualization.DigitalGauge.Themes[this.model.themes];
            this._setThemeColors(selectedThem);
        },
          
        _setThemeColors: function (selectedThem) {
            var result = [], jsonObj = ej.datavisualization.DigitalGauge.Themes;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var th = 0; th < result.length; th++) {
                for (var i = 0; i < this.model.items.length; i++) {
                    this.model.items[i].segmentSettings.color = (ej.isNullOrUndefined(this.model.items[i].segmentSettings.color) || this.model.items[i].segmentSettings.color == jsonObj[result[th]].items.segmentSettings.color) ? selectedThem.items.segmentSettings.color : this.model.items[i].segmentSettings.color;
                    this.model.items[i].shadowColor = (!this.model.items[i].shadowColor || this.model.items[i].shadowColor == jsonObj[result[th]].items.shadowColor) ? selectedThem.items.shadowColor : this.model.items[i].shadowColor;
                    this.model.items[i].textColor = (!this.model.items[i].textColor || this.model.items[i].textColor == jsonObj[result[th]].items.textColor) ? selectedThem.items.textColor : this.model.items[i].textColor;
                }
            }
        },
          
        _initialize: function () {
            if (this.model.init)
                this._clientSideOnInit();
            this._initObject(this);
            if (this.model.load)
                this._clientSideOnLoad();
            if (this.model.frame.backgroundImageUrl != null)
                this._drawCustomImage(this, this.model.frame.backgroundImageUrl);
            else
                this._renderItems();
            if (this.model.renderComplete)
                this._clientSideOnRenderComplete();
        },
          
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.items != null) {
                $.each(this.model.items, function (index, element) {
                    var obj = proxy._sendDefaultItem();
                    $.extend(true, obj, element);
                    $.extend(true, element, obj);
                });
            }
            else {
                this.model.items = [this._sendDefaultItem()];
            }
        },
          
        _sendDefaultItem: function () {
            var defaultItems;
            return defaultItems = {
                  
                characterSettings: {
                      
                    count: 4,
                       
                    opacity: 1,
                      
                    spacing: 2,
                      
                    type:ej.datavisualization.DigitalGauge.CharacterType.EightCrossEightDotMatrix,
                },
                  
                enableCustomFont: false,
                  
                segmentSettings: {
                      
                    color: null,
                      
                    gradient:null,
                       
                    length: 2,
                      
                    opacity: 0,
                      
                    spacing: 1,
                      
                    width: 1,
                },
                  
                shadowBlur: 0,
                  
                shadowOffsetX: 1,
                  
                shadowOffsetY: 1,
                  
                textAlign: "left",
                  
                font:   {
                      
                    size: "11px",
                      
                    fontFamily: "Arial",
                      
                    fontStyle: "italic"
                },
                  
                position: {
                      
                    x: 0,
                      
                    y: 0
                },
                  
                shadowColor: null,
                  
                textColor: null,
                  
                value: null
            };
        },
          
        _initObject: function (element) {
            this.element.addClass("e-widget");
            element.gaugeEl = this.element;
            for (var i = 0; this.model.items[i] != null; i++) {
                if (this.model.items[i].value == null)
                    this.model.items[i].value = this.value();
            }
            if (element.canvasEl)
                element.canvasEl.parent().empty();
            element.canvasEl = $("<canvas></canvas>");
            element.gaugeEl.append(element.canvasEl);
            element.canvasEl.attr("role", "presentation");
            if (_digitalGaugeCount == initialGaugeCount) {
                initialDigitalDivWidth = window.innerWidth;
            }
            element.canvasEl[0].setAttribute("width", element.model.width);
            element.canvasEl[0].setAttribute("height", element.model.height);
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");
        },
          
        _drawCustomImage: function (element, imageUrl) {
            var image = new Image();
            $(image).load(function () {
                element.contextEl.drawImage(this, 0, 0, element.model.width, element.model.height);
                if (element.model.Scales != null)
                    element._drawScales();
                if (element.model.items != null)
                    element._renderItems();
            }).attr('src', imageUrl);
        },
          
        _setSegmentCount: function (element) {
            switch (element) {
                case "sevensegment": this._SegmentCount = 7; break;
                case "fourteensegment": this._SegmentCount = 14; break;
                case "sixteensegment": this._SegmentCount = 16; break;
                case "eightcrosseightdotmatrix": this._SegmentCount = [8, 8]; break;
                case "eightcrosseightsquarematrix": this._SegmentCount = [8, 8]; break;
                default:
                    this._SegmentCount = 7;
            }

        },
          
        _setInnerPosition: function () {
            this.contextEl.save();
            this.contextEl.translate(this.model.frame.outerWidth + this.model.frame.innerWidth, this.model.frame.outerWidth + this.model.frame.innerWidth);
            this.bounds = {
                height: this.canvasEl[0].height - 2 * (this.model.frame.outerWidth + this.model.frame.innerWidth),
                width: this.canvasEl[0].width - 2 * (this.model.frame.outerWidth + this.model.frame.innerWidth)
            };
        },
          
        _setWidth: function () {
            var characterCount = [];
            if (this.model.items != null) {
                $.each(this.model.items, function (index, element) {
                    characterCount.push(element.characterSettings.count);
                });
            }
        },

          
        _renderItems: function () {
            if (this.model.items != null) {
                this._setInnerPosition();
                var self = this;
                $.each(this.model.items, function (index, element) {
                    self._setSegmentCount(element.characterSettings.type);
                    self.itemIndex = index;
                    self.canvasEl.attr("aria-label", element.value);
                    self._setShadow(index, element);
                    if (element.enableCustomFont)
                        self._setCustomFont(index, element);
                    else if (element.characterSettings.type.indexOf("matrix") != -1)
                        self._drawMatrixSegments(index, element);
                    else
                        self._drawSegments(index, element);
                });
            }
        },

          
        _setGradientColor: function (element, gradient, options) {
            if (options.Name || typeof (options) === "string") {
                gradient.addColorStop(0, this._getColor(element, options));
                gradient.addColorStop(1, this._getColor(element, options));
            }
            else
                $.each(options, function (index, colorElement) {
                    gradient.addColorStop(colorElement.colorStop != NaN ? colorElement.colorStop : 0, typeof (colorElement.color) === "string" ? colorElement.color : this._getColor(element, colorElement.color));
                });
        },

          
        _getColor: function (element, option) {
            if (typeof (option) === "string") {
                return option;
            }
            else {
                return ("rgba(" + option.R + ", " + option.G + "," + option.B + ", " + option.A / 255 + ")");
            }
        },

          
        _drawMatrixSegments: function (index, element) {
            var segmentXCollection = [], segmentCollection = [];
            if (element.value) {
                this._value = element.value.toString().split('');
                  
                element.characterSettings.count = (this._value.length > 4)?this._value.length:4;
            }
            else
                this._value = "";
            this.radius = (element.characterSettings.type.indexOf("dot") != -1) ? (element.segmentSettings.length + element.segmentSettings.width) / 2 : element.segmentSettings.width / 2;
            var controlStartX = this.startX = (this.bounds.width - element.characterSettings.count * (this._SegmentCount[0] * (2 * this.radius) + element.characterSettings.spacing + this._SegmentCount[0] * element.segmentSettings.spacing)) * (element.position.x / 100);
            var controlStartY = this.startY = (this.bounds.height - (this._SegmentCount[1] * ((element.characterSettings.type.indexOf("dot") != -1) ? 2 * this.radius : element.segmentSettings.length) + this._SegmentCount[1] * element.segmentSettings.spacing)) * (element.position.y / 100);
            for (var character = 0; character < element.characterSettings.count; character++) {
                if (this._value) {
                    this.character = element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character];
                    segmentCollection = this.model.matrixSegmentData[this.character];
                }
                if (character != 0) {
                    controlStartX = this.startX = this.startX + element.characterSettings.spacing + element.segmentSettings.spacing + (2 * this.radius);
                    this.startY = controlStartY;
                }
                for (var dotY = 0; dotY < this._SegmentCount[1]; dotY++) {
                    if (dotY != 0) {
                        this.startY = ((element.characterSettings.type.indexOf("dot") != -1) ? (2 * this.radius) : element.segmentSettings.length) + this.startY + element.segmentSettings.spacing;
                        this.startX = controlStartX;
                    }
                    if (segmentCollection) {
                        $.each(segmentCollection, function (segIndex) {
                            if (segIndex % 2 == 0) {
                                if (segmentCollection[segIndex] > dotY)
                                    return false;
                                if (segmentCollection[segIndex] == dotY)
                                    segmentXCollection.push(parseInt(segmentCollection[segIndex + 1]));
                            }
                        });
                    }
                    for (var dotX = 0; dotX < this._SegmentCount[0]; dotX++) {
                        if (dotX != 0)
                            this.startX = this.startX + 2 * this.radius + element.segmentSettings.spacing;
                        if (element.characterSettings.type.indexOf("dot") != -1)
                            this.gradient = this.contextEl.createRadialGradient(0, 0, 0, 0, 0, this.radius);
                        else
                            this.gradient = this.contextEl.createLinearGradient(0, 0, element.segmentSettings.width, 0);
                        if (element.segmentSettings.gradient)
                            this._setGradientColor(this, this.gradient, element.segmentSettings.gradient.colorInfo);
                        this.location = { "startX": this.startX, "startY": this.startY };
                        this.style = {
                            "opacity": (segmentXCollection && ($.inArray(dotX, segmentXCollection)) != -1) ? element.characterSettings.opacity : element.segmentSettings.opacity,
                            "height": element.segmentSettings.length,
                            "width": element.segmentSettings.width,
                            "fillStyle": ((element.segmentSettings.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.segmentSettings.color)),
                            "skewX": element.SkewAngleX,
                            "skewY": element.SkewAngleX
                        };
                        if (this.model.itemRendering)
                            this._clientSideOnItemRendering(true, dotX, dotY);
                        if (element.characterSettings.type.indexOf("dot") != -1)
                            this._drawDot(this.location, this.style);
                        else
                            this._drawSquare(this.location, this.style);
                    }
                    segmentXCollection = [];
                }
            }
        },

          
        _drawSegments: function (index, element) {
            var segmentCollection = [];
            if (element.value) {
                this._value = element.value.toUpperCase().toString().split('');
                //beyond the four character it align in center. else it align left
                element.characterSettings.count = (this._value.length > 4) ? this._value.length : 4;
            }
            this.characterSpace = element.characterSettings.type == "sevensegment" ? 2 * element.segmentSettings.width : 4 * element.segmentSettings.width;
            this._renderSegmentCalculation(element);
            this.gradient = this.contextEl.createLinearGradient(0, 0, 0, element.segmentSettings.width);
            if (element.segmentSettings.color)
                this._setGradientColor(this, this.gradient, element.segmentSettings.color);
            else if (element.segmentSettings.gradient)
                this._setGradientColor(this, this.gradient, element.segmentSettings.gradient.colorInfo);
            //else
            //    this._setGradientColor(this, this.gradient, this.model.segmentColor);
            for (var character = 0; character < element.characterSettings.count; character++) {
                if (element.value)
                    segmentCollection = this.model.segmentData[element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character]];
                for (var segment = 0; segment < this._SegmentCount; segment++) {
                    if (character != 0)
                        this.segment16X[segment] = this.segment16X[segment] + element.segmentSettings.length + this.characterSpace + element.characterSettings.spacing;
                    if (this._value)
                        this.character = element.textAlign == "right" ? this._value[this._value.length - element.characterSettings.count + character] : this._value[character];
                    this.location = { "startX": this.segment16X[segment], "startY": this.segment16Y[segment] };
                    this.style = {
                        "angle": this.angle[segment],
                        "fillStyle": this.gradient,
                        "isStroke": false,
                        "isFill": true,
                        "characterHeight": element.characterSettings.type == "sevensegment" ? element.segmentSettings.length : this.segmentHeight[segment],
                        "segmentWidth": element.segmentSettings.width,
                        "opacity": (segmentCollection && ($.inArray(segment, segmentCollection) != -1)) ? element.characterSettings.opacity : element.segmentSettings.opacity
                    };
                    if (this.model.itemRendering)
                        this._clientSideOnItemRendering(false, segment);
                    this._drawSegmentLayers(this.location, this.style);
                }
                if (this._notifyArrayChange)
                    this._notifyArrayChange("items[" + index + "]value", element.value);
                this.value(element.value);
                segmentCollection = [];
            }
        },

        
        _setCustomFont: function (index, element) {
            this.startX = (this.bounds.width - this._measureText(element.value, 0, this._getFontString(this, element.font)).width) * (element.position.x / 100);
            this.startY = (this.bounds.height - this._measureText(element.value, 0, this._getFontString(this, element.font)).height) * (element.position.y / 100);
            this.location = { "startX": this.startX, "startY": this.startY };
            this.style = { "font": this._getFontString(this, element.font), "text": element.value, "textColor": element.textColor ? ((element.textColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.textColor)) : ((element.segmentSettings.color == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.segmentSettings.color)) };
            if (this.model.itemRendering)
                this._clientSideOnItemRendering(false, segment);
            this._drawText(this.location, this.style);
        },

          
        _getFontString: function (element, font) {
            return ((font.size == null) ? "11px" : font.size) + " " + font.fontFamily + " " + (font.fontStyle ? font.fontStyle : "");
        },

          
        _renderSegmentCalculation: function (element) {
            var length = element.segmentSettings.length, width = element.segmentSettings.width;
            this.startX = (this.bounds.width - element.characterSettings.count * (length + this.characterSpace + element.characterSettings.spacing)) * (element.position.x / 100);
            this.startY = (this.bounds.height - 2 * length - width) * (element.position.y / 100);
            var tempLength = element.characterSettings.type == "sevensegment" ? length : length / 2;
            this.segment16X = [
                  this.startX + width / 2,
                  this.startX + length + 4 * width,
                  this.startX + length + 4 * width,
                  this.startX + width / 2,
                  this.startX,
                  this.startX,
                  this.startX + width / 2,
                  this.startX + tempLength + 2 * width,
                  this.startX + 2.5 * width + tempLength,
                  this.startX + tempLength + 2 * width,
                  this.startX + length + 2.5 * width,
                  this.startX + tempLength + 2.5 * width,
                  this.startX + tempLength + 1.5 * width,
                  this.startX + 1.5 * width,
                  this.startX + 5 * width / 2 + tempLength,
                  this.startX + 2.5 * width + tempLength
            ];
            this.segment16Y = [
                  this.startY,
                  this.startY,
                  this.startY + length + width,
                  this.startY + 2 * length + 2 * width,
                  this.startY + length + width,
                  this.startY,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY,
                  this.startY + width,
                  this.startY + length + width,
                  this.startY + length + width,
                  this.startY + width,
                  this.startY,
                  this.startY + 2 * length + 2 * width
            ];
            this.segmentHeight = [
                  length / 2,
                  length,
                  length,
                  length / 2,
                  length,
                  length,
                  length / 2,
                  length,
                  length / 2,
                  length,
                  length,
                  length,
                  length,
                  length,
                  length / 2,
                  length / 2
            ];
            this.angle = [-90, 0, 0, -90, 0, 0, -90, 0, -90, 0, 27, -27, 27, -27, -90, -90];
            if (element.characterSettings.type == "sevensegment")
                this.segment16X[2] = this.segment16X[1] = this.startX + length + 2 * width;
            if (element.characterSettings.type == "fourteensegment")
                this.segmentHeight[3] = this.segmentHeight[0] = length + 2 * width;
        },

          
        _drawSegmentLayers: function (location, style) {
            this._contextOpenPath(style, this);
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.rotate(Math.PI * (style.angle / 180));
            this.contextEl.moveTo(0, 0);
            this.contextEl.lineTo(-style.segmentWidth, style.segmentWidth);
            this.contextEl.lineTo(-style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(0, style.characterHeight + style.segmentWidth);
            this.contextEl.lineTo(style.segmentWidth, style.characterHeight);
            this.contextEl.lineTo(style.segmentWidth, style.segmentWidth);
            this._contextClosePath(style, this);
        },

          
        _drawDot: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.arc(0, 0, this.radius, 0, 2 * Math.PI, true);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

          
        _setShadow: function (index, element) {
            this.contextEl.save();
            this.contextEl.shadowColor = ((element.shadowColor == "transparent") ? "rgba(0,0,0,0)" : this._getColor(this, element.shadowColor));
            this.contextEl.shadowOffsetY = element.shadowOffsetY;
            this.contextEl.shadowOffsetX = element.shadowOffsetX;
            this.contextEl.shadowBlur = element.shadowBlur;
        },

          
        _drawSquare: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.translate(location.startX, location.startY);
              
            this.contextEl.fillStyle = style.fillStyle;
            this.contextEl.globalAlpha = style.opacity;
            this.contextEl.rect(0, 0, style.width, style.height);
            this.contextEl.fill();
            this.contextEl.closePath();
            this.contextEl.restore();
        },

          

        _drawText: function (location, style) {
            this.contextEl.beginPath();
            this.contextEl.save();
            this.contextEl.font = style.font;
            this.contextEl.textBaseline = "hanging";
            this.contextEl.fillStyle = ((style.textColor == "transparent") ? "rgba(0,0,0,0)" : style.textColor);
            this.contextEl.fillText(style.text, location.startX, location.startY);
            this.contextEl.closePath();
            this.contextEl.restore();
        },

          

        setValue: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].value = value;
                this._initialize();
            }
        },

          
        getValue: function (itemIndex) {
            return this.model.items[itemIndex].value;
        },

          

        setPosition: function (itemIndex, value) {
            if (itemIndex < this.model.items.length && value != null) {
                this.model.items[itemIndex].position.x = value.x;
                this.model.items[itemIndex].position.y = value.y;
                this._initialize();
            }
        },


          

        getPosition: function (itemIndex) {
            if (itemIndex < this.model.items.length)
                return { "x": this.model.items[itemIndex].position.x, "y": this.model.items[itemIndex].position.y };
            else
                return null;
        },

          
        refresh: function () {
            this._setTheme();
            this._initialize();
        },

          
        exportImage: function (fileName, fileType) {
            /// <summary>This function save the rendered canvas image</summary>
            /// <param name="fileName" type="String">fileName to which the image has been saved</param>
            /// <param name="fileType" type="String">fileType to which the image has been saved</param>
            if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10) {
                return false;
            }
            else {
                var image = this.canvasEl[0].toDataURL();
                image = image.replace(/^data:[a-z]*;,/, '');
                var image1 = image.split(',');
                var byteString = atob(image1[1]);
                var buffer = new ArrayBuffer(byteString.length);
                var intArray = new Uint8Array(buffer);
                for (var i = 0; i < byteString.length; i++) {
                    intArray[i] = byteString.charCodeAt(i);
                }
                var blob = new Blob([buffer], { type: "image/png" });
                if (ej.browserInfo().name === "msie")
                    window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
                else {
                    var pom = document.createElement('a');
                    var url = URL.createObjectURL(blob);
                    pom.href = url;
                    pom.setAttribute('download', fileName + '.' + fileType);
                    if (document.createEvent) {
                        var e = document.createEvent("MouseEvents");
                        e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                        pom.dispatchEvent(e);
                    }
                    else if (pom.fireEvent) {
                        pom.fireEvent("onclick");
                    }
                }
                return true;
            }
        },

      
        resizeCanvas: function () {
            if (_digitalGaugeCount != 0)
                _digitalGaugeCount = _digitalGaugeCount - 1;
            else
                _digitalGaugeCount = $(".e-digitalgauge").length - 1; var state, chk = true;
            if (!ej.isNullOrUndefined(this.canvasEl.parent().attr("style")))
                state = this.GaugeEl.parent().attr("style").split(";");
            if (!ej.isNullOrUndefined(state)) {
                $.each(state, function (key, val) {
                    while (val.indexOf("width") != -1) {
                        chk = val.indexOf("px") == -1 ? true : false;
                        break;
                    }
                });
            }
            if (chk) {
                var ratio = window.innerWidth / initialDigitalDivWidth;
                this.model.width *= ratio;
                for (var i = 0; this.model.items[i] != null; i++) {
                    this.model.items[i].segmentSettings.width *= ratio;
                    this.model.items[i].segmentSettings.spacing *= ratio;
                    this.model.items[i].characterSettings.spacing *= ratio;
                }
                this.refresh();
                if (_digitalGaugeCount == 0) {
                    initialDigitalDivWidth = window.innerWidth;
                }
            }
        },

          
          
        _clientSideOnLoad: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("load", data);
        },

          
        _clientSideOnItemRendering: function (isMatrix, row, column) {
            var data;
            if (isMatrix)
                data = { object: this, items: this.model.items, character: this.character, context: this.contextEl, position: this.location, style: this.style, row: row, column: column };
            else
                data = { object: this, model: this.model, id: this.model.ClientId, items: this.model.items, character: this.character, context: this.contextEl, position: this.location, style: this.style, segment: row };
            this._trigger("itemRendering", data);
        },

          
        _clientSideOnInit: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("init", data);
        },

          
        _clientSideOnRenderComplete: function () {
            var data = { object: this, items: this.model.items, context: this.contextEl };
            this._trigger("renderComplete", data);
        },


          
          
        _contextOpenPath: function (style, element) {
            element.contextEl.save();
            element.contextEl.beginPath();
            if (style.strokeStyle)
                element.contextEl.strokeStyle = style.strokeStyle;
            if (style.opacity != undefined)
                element.contextEl.globalAlpha = style.opacity;
            if (style.lineWidth)
                element.contextEl.lineWidth = style.lineWidth;
            if (style.fillStyle)
                element.contextEl.fillStyle = style.fillStyle;
        },

          
        _contextClosePath: function (style, element) {
            element.contextEl.closePath();
            if (style.isFill)
                element.contextEl.fill();
            if (style.isStroke)
                element.contextEl.stroke();
            element.contextEl.restore();
        },

          
        _measureText: function (text, maxwidth, font) {
            var textObj = document.createElement('DIV');
            textObj.innerHTML = text;
            if (font != null)
                textObj.style.font = font;
            textObj.style.backgroundColor = 'white';
            textObj.style.position = 'absolute';
            textObj.style.top = -100;
            textObj.style.left = 0;
            if (maxwidth)
                textObj.style.maxwidth = maxwidth + "px";
            document.body.appendChild(textObj);
            var bounds = { width: textObj.offsetWidth, height: textObj.offsetHeight };
            $(textObj).remove();
            return bounds;
        }

    });

      
    ej.datavisualization.DigitalGauge.CharacterType = {
          
        SevenSegment: "sevensegment",
          
        FourteenSegment: "fourteensegment",
          
        SixteenSegment: "sixteensegment",
          
        EightCrossEightDotMatrix: "eightcrosseightdotmatrix",
          
        EightCrossEightSquareMatrix: "eightcrosseightsquarematrix"
    };
      
    ej.datavisualization.DigitalGauge.TextAlign = {
          
        Left: "left",
          
        Right: "right"
    };

      
    ej.datavisualization.DigitalGauge.FontStyle = {
          
        Normal: "normal",
          
        Bold: "bold",
          
        Italic: "italic",
          
        Underline: "underline",
          
        Strikeout: "strikeout"
    };

      
    ej.datavisualization.DigitalGauge.Themes = {
          
        flatlight: {
            items: {
                segmentSettings: {
                    color: "#232323",
                },
                shadowColor: "#232323",
                textColor: "#232323"
            }
        },
          
        flatdark: {
            items: {
                segmentSettings: {
                    color: "#b1b0b0",
                },
                shadowColor: "#b1b0b0",
                textColor: "#b1b0b0"
            }
        }
    };

})(jQuery, Syncfusion);;;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.s
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
  
    ej.widget("ejBulletGraph", "ej.datavisualization.BulletGraph",    {
    
        element: null,

        model: null,
        validTags: ["div"],
    
        _tags: [{
            tag: 'qualitativeRanges',
            attr: ['rangeEnd', 'rangeStroke', 'rangeOpacity']
        },
        {
            tag: 'quantitativeScaleSettings.featureMeasures',
            attr: ['value', 'comparativeMeasureValue']
        }],
    
        defaults:   {
              
            value: 0,
              
            comparativeMeasureValue: 0,
              
            width: 595,
              
            height: 90,
              
            theme: "flatlight",
              
            orientation: "horizontal",
              
            flowDirection: "forward",
              
            qualitativeRangeSize: 32,
              
            quantitativeScaleLength: 475,
              
            tooltipSettings:   {
                  
                  
                enableCaptionTooltip: false,
                  
                captionTemplate: null,
                  
                visible: true,
                  
                template: null
            },
              
            quantitativeScaleSettings:    {
                  
                location: {
                      
                    x: 10,
                      
                    y: 10
                },
                  
                minimum: 0,
                  
                maximum: 10,
                  
                interval: 1,
                  
                minorTicksPerInterval: 4,
                  
                majorTickSettings: {
                      
                    size: 13,
                      
                    stroke: null,
                      
                    width: 2
                },
                  
                minorTickSettings: {
                      
					size: 7,
                 
                    stroke: null,
                      
                    width: 2
                },
                  
                tickPosition: "far",
                  
                tickPlacement: "outside",
                  
                labelSettings: {
                      
                    labelPlacement: "outside",
                      
                    labelPrefix: "",
                      
                    labelSuffix: "",
                      
                    stroke: null,
                      
                    size: 12,
                      
                    offset: 15,
                      
                    font: {
                          
                        fontFamily: 'Segoe UI',
                          
                        fontStyle: 'Normal ',
                          
                        fontWeight: 'regular',
                          
                        opacity: 1
                    },
                      
                    position: "below"
                },
                  
                featuredMeasureSettings: {
                      

                    stroke: null,
                      
                    width: 6
                },
                  
                comparativeMeasureSettings: {
                      
                    stroke: null,
                      
                    width: 5
                },
                  
                featureMeasures: [{
                      
                    value: null,
                      
                    comparativeMeasureValue: null,
                      
                    category: null
                }]
            },
              
            fields: {
                  
                dataSource: null,
                  
                query: null,
                  
                tableName: null,
                  
                category: null,
                  
                featureMeasures: null,
                  
                comparativeMeasure: null
            },
              
            enableAnimation: true,
              
            enableResizing: true,
              
            applyRangeStrokeToTicks: false,
              
            applyRangeStrokeToLabels: false,
              
            qualitativeRanges: [{
                  
                rangeEnd: 4.3,
                  
                rangeStroke: null,
                  
                rangeOpacity: 1
            }, {
                  
                rangeEnd: 7.3,
                  
                rangeStroke: null,
                  
                rangeOpacity: 1
            }, {
                  
                rangeEnd: 10,
                  
                rangeStroke: null,
                  
                rangeOpacity: 1
            }],
              
            captionSettings: {
                  
                enableTrim: true,
                  
                textPosition: 'float',
                  
                textAlignment: 'Near',
                  
                textAnchor: 'start',
                  
                padding: 5,
                  
                textAngle: 0,
                  
                location: {
                      
                    x: 17,
                      
                    y: 30
                },
                  
                text: "",
                  
                font: {
                      
                    color: null,
                      
                    fontFamily: 'Segoe UI',
                      
                    fontStyle: 'Normal',
                      
                    size: '12px',
                      
                    fontWeight: 'regular',
                      
                    opacity: 1
                },
                  
                subTitle: {
                      
               textPosition: 'float',
                 
               textAlignment: 'Near',
                 
               textAnchor: 'start',
                 
                    padding: 5,
                      
                    textAngle: 0,
                      
                    text: "",
                      
                    location: {
                          
                        x: 10,
                          
                        y: 45
                    },
                      
                    font: {
                          
                        color: null,                          
                        
                        fontFamily: 'Segoe UI',
                          
                        
                        fontStyle: 'Normal ',                          
                        
                        size: '12px',
                          
                        fontWeight: 'regular',
                          
                        opacity: 1
                    }
                },

                  
                indicator: {
                      
                    textPosition: 'float',
                      
                    textAlignment: 'Near',
                      
                    textAnchor: 'start',
                      
                    padding:5,
                      
                    visible: false,
                      
                    textAngle: 0,
                      
                    textSpacing: 3,
                      
                    text: "",                    
                      
                    symbol: {
                          
                        border: {
                              
                            color: null,
                              
                            width: 1,
                        },
                          
                        color: null,
                          
                        shape: "",
                          
                        imageURL: "",
                          
                        size: {
                              
                            width: 10,
                              
                            height: 10
                        },
                          
                        opacity: 1
                    },
                      
                    location: {
                          
                        x: 10,
                          
                        y: 60
                    },
                      
                    font: {
                          
                        color: null,
                          

                        fontFamily: 'Segoe UI',
                          

                        fontStyle: 'Normal ',
                          

                        size: '12px',
                          
                        fontWeight: 'regular',
                          
                        opacity: 1
                    }
                }
            },

              
              
            load: '',
              
            drawTicks: null,
              
            drawLabels: null,
              
            drawCaption: null,
              
            drawIndicator: null,
              
            drawQualitativeRanges: null,
              
            drawFeatureMeasureBar: null,
              
            drawCategory: null,
              
            drawComparativeMeasureSymbol: null
        },

        observables: ["value", "comparativeMeasureValue"],
        value: ej.util.valueFunction("value"),
        comparativeMeasureValue: ej.util.valueFunction("comparativeMeasureValue"),
          
        dataTypes: {
            quantitativeScaleSettings: {
                labelSettings: "data",
                featureMeasures: "array"
            },
            fields: { dataSource: "data", query: "data" },
            qualitativeRanges: "array",
            captionSettings: "data"
        },

        // constructor function
          
        _init: function () {
            this._renderBulletGraph();
        },
          
        _isSVG: function () {
            if (window.SVGSVGElement)
                return true;
            else
                return false;
        },
          
        _value: ej.util.valueFunction("value"),
        _comparativeMeasureValue: ej.util.valueFunction("comparativeMeasureValue"),
          
        _qualitativeRanges: function (index, property, value, old) {
            this.redraw();
            this._trigger('refresh');
        },
          
        _quantitativeScaleSettings_featureMeasures: function (index, property, value, old) {
            this.redraw();
            this._trigger('refresh');
        },
          
        _renderBulletGraph: function () {
            if (this._isSVG()) {
                this.svgRenderer = new ej.EjSvgRender(this.element);
                this.svgObject = this.svgRenderer.svgObj;
                this._trigger("load");
                this._setTheme(ej.datavisualization.BulletGraph.Themes, this.model.theme);
                this._renderBulletElements();
                this.bindEvents();
                if (this.model.enableAnimation)
                    this._animateMeasures();
            }
        },
          
        _animateMeasures: function () {
            this._doAnimation();
            this._doLineAnimation();
        },
          
        _renderBulletElements: function () {
            this.svgObject.setAttribute('width', this.model.width);
            this.svgObject.setAttribute('height', this.model.height);
            this.svgObject.setAttribute('viewBox', "0 0 " + this.model.width + " " + this.model.height);
            this.svgObject.setAttribute('preserveAspectRatio', "xMinYMin");
            this.svgWidth = this.model.width;
            this.svgHeight = this.model.height;
            var captionGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_captionGroup' });
            var scaleGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_scaleGroup' });
            var bulletGroup = this.svgRenderer.createGroup({ 'id': this.svgObject.id + '_outerWrap' });

            this._initializeValues();
            var intervalValue = ((this.model.quantitativeScaleLength) / ((this._scale.maximum - this._scale.minimum) / this._scale.interval));

            //render captionSettings
            this._drawCaption();

            //render indicator
            this._drawIndicator();

            //render caption group
            this._drawCaptionGroup(captionGroup);

            //render scale
            this._scaleGroup = scaleGroup;
            this._drawScale(captionGroup, scaleGroup);

            //draw ranges
            this._drawQualitativeRanges(scaleGroup);

            //render ticks
            this._drawMajorTicks(intervalValue, scaleGroup);
            this._drawMinorTicks(intervalValue, scaleGroup);

            //render labels
            this._drawLabels(intervalValue, scaleGroup);

            // data binding
            this._bindData();
                        
            $(scaleGroup).appendTo(bulletGroup);
            $(captionGroup).appendTo(bulletGroup);

            this._changeOrientation(scaleGroup);

            $(bulletGroup).appendTo(this.svgObject);
            $(this.svgObject).appendTo(this.element);
        },
      
        _drawCaptionGroup: function (captionGroup) {
            var caption = this.model.captionSettings;
            var subTitle = caption.subTitle;
            var indicator = (this._indicator ? this._indicator.settings : caption.indicator);
            var location;

            caption._location = subTitle._location = indicator._location = location;

            var smartCaptionPlacement = (caption.textPosition.toLowerCase()!='float' && caption.text!="");
            var smartSubTitlePlacement = (subTitle.textPosition.toLowerCase() != 'float' && subTitle.text != "");
            var smartIndicatorPlacement = indicator.visible && (indicator.textPosition.toLowerCase() != 'float');

            if (smartCaptionPlacement || smartSubTitlePlacement || smartIndicatorPlacement) {                

                var captionSize = ej.EjSvgRender.utils._measureText(caption.text, null, caption.font);
                var subTitleSize = ej.EjSvgRender.utils._measureText(subTitle.text, null, subTitle.font);
                var indicatorTextSize;
                var indicatorRegion;
                if (indicator.visible) {                    
                    indicatorTextSize = this._indicator.bounds;
                    var indicatorSize = {
                        width: indicator.symbol.size.width + indicatorTextSize.width + indicator.textSpacing,
                        height: Math.max(indicator.symbol.size.height, indicatorTextSize.height)
                    }
                    indicatorRegion = {
                        x: 0,
                        y: 0,
                        width: indicatorSize.width,
                        height: indicatorSize.height,
                        padding: indicator.padding,
                        anchor: indicator.textAnchor.toLowerCase(),
                        alignment: indicator.textAlignment.toLowerCase()
                    };
                }

                var captionRegion = { 
                    x: 0, 
                    y: 0, 
                    width: captionSize.width,                    
                    height: parseFloat(caption.font.size),
                    padding: caption.padding,
                    anchor: caption.textAnchor.toLowerCase(),
                    alignment: caption.textAlignment.toLowerCase()
                };
                var subTitleRegion = {
                    x: 0,
                    y: 0,
                    width: subTitleSize.width,
                    height: parseFloat(subTitle.font.size),
                    padding: subTitle.padding,
                    anchor: subTitle.textAnchor.toLowerCase(),
                    alignment: subTitle.textAlignment.toLowerCase()
                };
                
                var scaleRegion = { x: this._scale.location.x, y: this._scale.location.y, width: this.model.quantitativeScaleLength, height: this.model.qualitativeRangeSize };
                var elements= [ smartCaptionPlacement ?caption : null, smartSubTitlePlacement ? subTitle : null, smartIndicatorPlacement ? indicator : null];

                this._positionTextGroup(elements, [captionRegion, subTitleRegion, indicatorRegion], scaleRegion);
                this._locateTextGroup(elements, [captionRegion, subTitleRegion, indicatorRegion]);
            }
            var captionLocation = caption._location == undefined ? caption.location : caption._location;
            var subTitleLocation = subTitle._location == undefined ? subTitle.location : subTitle._location;
            var indicatorLocation = indicator._location == undefined ? indicator.location : indicator._location;

            caption.displayText = caption.text;
            if (caption.enableTrim == true) {
                captionLocation.x = captionLocation.x < 0 ? 0 : captionLocation.x;
                captionLocation.y = captionLocation.y < 0 ? 0 : captionLocation.y;
                caption.displayText = this._displayText(caption, captionLocation);
            }
            if (subTitle.text != "") {
                subTitle.displayText = subTitle.text;
                if (caption.enableTrim == true) {
                    subTitleLocation.x = subTitleLocation.x < 0 ? 0 : subTitleLocation.x;
                    subTitleLocation.y = subTitleLocation.y < 0 ? 0 : subTitleLocation.y;
                    subTitle.displayText = this._displayText(subTitle, subTitleLocation);
                }
            }
            if (indicator.visible) {
                indicator.displayText = indicator.text;
                if (this.model.captionSettings.enableTrim == true) {
                    indicatorLocation.x = indicatorLocation.x < 0 ? 0 : indicatorLocation.x;
                    indicatorLocation.y = indicatorLocation.y < 0 ? 0 : indicatorLocation.y;
                }
                var location = (indicator._location ? indicator._location : indicator.location);
                var symbolRect = {
                    'x': location.x + indicator.symbol.size.width / 2,
                    'y': location.y - indicator.symbol.size.height / 2,
                    'width': indicator.symbol.size.width,
                    'height': indicator.symbol.size.height,
                };
                var textOptions = this._indicatorTextOptions(indicator, symbolRect, this._indicator.bounds, this.svgObject.id + '_Indicator');
                var symbolOptions = this._indicatorSymbolOptions(captionGroup, indicator);
                this._drawBulletSymbol(this._indicator.settings.symbol.shape, symbolRect, symbolOptions, captionGroup);
                if (this.model.captionSettings.enableTrim == true)
                    indicator.displayText = this._displayText(indicator, symbolRect);
            }
            caption.displayText = caption.textPosition.toLowerCase() != "float" ? this._captionOverlap(this.model.captionSettings) : caption.displayText;
            this.svgRenderer.drawText(this._textOptions(caption, this.svgObject.id + '_Caption'), caption.displayText, captionGroup);
            if (subTitle.text != ""){
                subTitle.displayText = subTitle.textPosition.toLowerCase() != "float" ? this._subOverlap(this.model.captionSettings.subTitle) : subTitle.displayText;
                this.svgRenderer.drawText(this._textOptions(subTitle, this.svgObject.id + '_SubTitle'), subTitle.displayText, captionGroup);
            }   
            if (indicator.visible) {
                indicator.displayText = indicator.textPosition.toLowerCase() != "float" ? this._indOverlap(indicator,symbolRect) : indicator.displayText;
                this.svgRenderer.drawText(textOptions, indicator.displayText, captionGroup);
            }
        },
        // text overlap method is perform herer for caption , subtitle and indicator when position is except float
       _captionOverlap: function (settings) {
                var gap;
                if ((settings.textPosition == settings.subTitle.textPosition) && (settings.textAlignment == settings.subTitle.textAlignment) && settings.textAngle > 0 && settings.textAngle < 120 ) {
                    gap = settings.subTitle._location.y - settings._location.y;
                }
                else if ((settings.textPosition == settings.indicator.textPosition) && (settings.textAlignment == settings.indicator.textAlignment) && settings.textAngle > 0 && settings.textAngle < 120 ){
                    gap = settings.indicator._location.y - settings._location.y;
                }
                else{
                   if ((settings.textPosition == settings.subTitle.textPosition) && (settings._location.y < settings.subTitle._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = settings.subTitle._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == settings.subTitle.textPosition) && (settings._location.y > settings.subTitle._location.y) && settings.textAngle > 190 && settings.textAngle < 360     ){
                        gap = settings._location.y - settings.subTitle._location.y;
                   }
                   else if ((settings.textPosition == settings.indicator.textPosition) && (settings._location.y < settings.indicator._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = settings.indicator._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == settings.indicator.textPosition) && (settings._location.y > settings.indicator._location.y) && settings.textAngle > 190 && settings.textAngle < 360    ){
                        gap = settings._location.y - settings.indicator._location.y;
                   }
                }
                 var text = this._trim(settings.displayText,settings,gap,isRotate = true);           
            return text;             
        },
        _indOverlap: function (settings,loc) {
            var gap;
            var subTitle = this.model.captionSettings.subTitle;
            var title = this.model.captionSettings;
             if ((settings.textPosition == subTitle.textPosition) && (settings.textAlignment == subTitle.textAlignment) && settings.textAngle > 190 && settings.textAngle < 360 ) {
                    gap = loc.y - subTitle._location.y;
                }
                else if ((settings.textPosition == title.textPosition) && (settings.textAlignment ==title.textAlignment) && settings.textAngle > 190 && settings.textAngle < 360 ){
                    gap = loc.y - title._location.y;
                }
                else{
                   if ((settings.textPosition == subTitle.textPosition) && (settings._location.y < subTitle._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = subTitle._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == subTitle.textPosition) && (settings._location.y > subTitle._location.y) && settings.textAngle > 190 && settings.textAngle < 360     ){
                        gap = settings._location.y - subTitle._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y < title._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = title._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y > title._location.y) && settings.textAngle > 190 && settings.textAngle < 360    ){
                        gap = settings._location.y - title._location.y;
                   }
                }
                 var text = this._trim(settings.displayText,settings,gap,isRotate = true);           
            return text;             
        },
        _subOverlap: function (settings) {
            var gap;
            var indicator = this.model.captionSettings.indicator;
            var title = this.model.captionSettings;
             if ((settings.textPosition == indicator.textPosition) && (settings.textAlignment == indicator.textAlignment) && settings.textAngle > 0 && settings.textAngle < 180 ) {
                    gap = indicator._location.y - settings._location.y;
                }
                else if ((settings.textPosition == title.textPosition) && (settings.textAlignment ==title.textAlignment) && settings.textAngle > 190 && settings.textAngle < 360 ){
                    gap = settings._location.y - title._location.y;
                }
                else{
                   if ((settings.textPosition == indicator.textPosition) && (settings._location.y < indicator._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = indicator._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == indicator.textPosition) && (settings._location.y > indicator._location.y) && settings.textAngle > 190 && settings.textAngle < 360     ){
                        gap = settings._location.y - indicator._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y < title._location.y) && settings.textAngle > 0 && settings.textAngle < 180 ){
                       gap = title._location.y - settings._location.y;
                   }
                   else if ((settings.textPosition == title.textPosition) && (settings._location.y > title._location.y) && settings.textAngle > 190 && settings.textAngle < 360    ){
                        gap = settings._location.y - title._location.y;
                   }
                }
                 var text = this._trim(settings.displayText,settings,gap,isRotate = true);           
            return text;             
        },
        // rotated label width calculated here
        rotatedLabel: function (font, sender, value, rotatedLabel) {
            var rotatedOptions = {
                'font-size': font.size,
                'transform': 'rotate(' + value + ',0,0)',
                'font-family': font.fontFamily,
                'font-style': font.fontStyle,
                'rotateAngle': 'rotate(' + value + 'deg)',
                'text-anchor': 'middle'
            };
            var text = sender.svgRenderer.createText(rotatedOptions, rotatedLabel);
            var width = Math.ceil((ej.EjSvgRender.utils._measureBounds(text, sender).width));
            return width;
        },
        // Trim size calculated here
        calcGap: function (angle, location, maxWidth, maxHeight, isRotate, Width, Height) {
            var trimSize, space, rotate;
            if (angle >= 0 && angle <= 90) {

                if (((maxHeight + location.y) >= Height)&& angle > 0) {
                    space = Height - location.y;
                    isRotate = true;
                }
                else if (((maxHeight + location.y) <= Height) && ((maxWidth + location.x) >= Width) && angle > 0) {
                    space = Width - location.x;
                }
                else {
                    if (((maxHeight + location.y) <= Height) && (angle > 0)) {
                        space = Height - location.y;
                        isRotate = true;
                    }
                    else
                        space = Width - location.x;
                }
            }
            else if (angle > 90 && angle < 180) {

                if (((maxHeight + location.y) >= Height)) {
                    space = Height - location.y;
                    isRotate = true;
                }
                else if (((maxHeight + location.y) <= Height) && ((maxWidth + location.x) >= Width)) {
                    space = location.x;
                }
                else {
                    if ((maxWidth + location.x) <= Width) {
                        space = location.x;
                    }
                    else {
                        space = Height - location.y;
                        isRotate = true;
                    }

                }
            }
            else if (angle >= 180 && angle < 270) {

                if ((location.x - maxWidth) >= 0) {
                    space = location.y;
                    isRotate = true;
                }
                else if (((location.y - maxHeight) <= 0) && ((location.x - maxWidth) <= 0)) {
                    space = location.x;
                }
                else {
                    if ((location.y - maxHeight) <= 0) {
                        space = Height - location.y;
                        isRotate = true;
                    }
                    else {
                        space = location.x;
                    }
                }

            }
            else {
                if ((maxWidth + location.x) <= Width) {
                    space = location.y;
                    isRotate = true;
                }
                else if (((location.y - maxHeight) >= 0) && ((maxWidth + location.x) >= Width)) {
                    space = Width - location.x;
                }
                else {
                    if ((maxWidth + location.x) >= Width) {
                        space = Width - location.x;
                    }
                    else {
                        space = location.y;
                        isRotate = true;
                    }
                }
            }

            return { trimSize: space, rotate: isRotate };
        },
        // Scale Location calculated here
        _scaleLoc: function () {
            var scale = this._scale;
            var offset = scale.labelSettings.offset;
            var tickSize = scale.majorTickSettings.size;
            var pointY, pointHeight;
            var label = this._scale.labelSettings;
            var labelBounds = ej.EjSvgRender.utils._measureText(label.labelPrefix + scale.maximum + label.labelSuffix, null, label);
            var labelSize = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? labelBounds.height : labelBounds.width;            
            pointY = scale.location.y;
            var qualitativeRangeSize = this.model.qualitativeRangeSize;
            var placement = this._tickPosition + this._tickPlacement + this._labelPosition + this._labelPlacement;
            if (this._tickPosition == "center")
                tickSize = (tickSize > qualitativeRangeSize) && ((tickSize - qualitativeRangeSize) / 2);
            switch (placement) {
                case 'faroutsidebelowoutside':
                    pointHeight = pointY + labelSize + qualitativeRangeSize + offset + ((tickSize > offset) && (tickSize - offset));
                    break;
                case 'faroutsidebelowinside':
                    pointHeight = pointY + tickSize + qualitativeRangeSize 
                    pointY = pointY - (((offset + labelSize) > qualitativeRangeSize) && ((offset + labelSize) - qualitativeRangeSize));
                    break;
                case 'faroutsideaboveoutside':
                    pointHeight = pointY + tickSize + qualitativeRangeSize;
                    pointY = pointY - (offset + labelSize)
                    break;
                case 'faroutsideaboveinside':
                    pointHeight = pointY + tickSize + qualitativeRangeSize + (((offset + labelSize) > (qualitativeRangeSize + tickSize)) ? ((offset + labelSize) - (qualitativeRangeSize + tickSize)) : 0 );
                    break;
                case 'farinsidebelowoutside':
                    pointHeight = pointY + labelSize + qualitativeRangeSize + offset;
                    pointY = pointY - ((tickSize > qualitativeRangeSize) && (tickSize - qualitativeRangeSize));
                    break;
                case 'farinsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - ((tickSize > qualitativeRangeSize) && (tickSize -qualitativeRangeSize)) - (((offset + labelSize) > (qualitativeRangeSize + tickSize)) && (offset + labelSize - tickSize)) 
                    break;
                case 'farinsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - (offset + labelSize) - ((tickSize > (offset + labelSize + qualitativeRangeSize)) && (tickSize - (offset + labelSize + qualitativeRangeSize)));
                    break;
                case 'farinsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + (((offset + labelSize) > qualitativeRangeSize) && (labelSize + offset - qualitativeRangeSize));
                    pointY = pointY - ((tickSize > qualitativeRangeSize) && (tickSize - qualitativeRangeSize));
                    break;
                case 'nearoutsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + offset + labelSize;
                    pointY = pointY - tickSize;
                    break;
                case 'nearoutsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - tickSize - (((offset + labelSize) > (qualitativeRangeSize + tickSize)) && ((offset + labelSize) - (qualitativeRangeSize + tickSize)));
                    break;
                case 'nearoutsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize;
                    pointY = pointY - tickSize - ((tickSize < (offset + labelSize)) && (offset + labelSize - tickSize));
                    break;
                case 'nearoutsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + (((offset + labelSize) > qualitativeRangeSize) && (offset + labelSize - qualitativeRangeSize));
                    pointY = pointY - tickSize;
                    break;
                case 'nearinsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (offset + labelSize + qualitativeRangeSize)) ? (tickSize - (qualitativeRangeSize)) : (offset + labelSize));
                    break;
                case 'nearinsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > qualitativeRangeSize) && (tickSize - qualitativeRangeSize));
                    pointY = pointY - (((offset + labelSize) > qualitativeRangeSize) && (offset + labelSize - qualitativeRangeSize));
                    break;
                case 'nearinsideaboveoutside':
                    pointHeight = pointY + ((tickSize > qualitativeRangeSize) ? tickSize : qualitativeRangeSize);
                    pointY = pointY - offset - labelSize;
                    break;
                case 'nearinsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (labelSize + offset)) ? (tickSize - qualitativeRangeSize) : (labelSize + offset - qualitativeRangeSize));
                    break;
                case 'centeroutsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > offset + labelSize) ? tickSize : offset + labelSize);
                    pointY = pointY - tickSize;
                    break;
                case 'centeroutsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY - tickSize - (((offset + labelSize) > (qualitativeRangeSize + tickSize)) && ((offset + labelSize) - (qualitativeRangeSize + tickSize)));
                    break;
                case 'centeroutsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY - tickSize - ((tickSize < (offset + labelSize)) && (offset + labelSize - tickSize));
                    break;
                case 'centeroutsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + (((offset + labelSize) > (qualitativeRangeSize + tickSize)) ? (offset + labelSize - qualitativeRangeSize - tickSize) : tickSize);
                    pointY = pointY - tickSize;
                    break;
                case 'centerinsidebelowoutside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (offset + labelSize)) ? tickSize : (offset + labelSize));
                    pointY = pointY - tickSize;
                    break;
                case 'centerinsidebelowinside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY -  (((offset + labelSize - qualitativeRangeSize) > tickSize ) ? (offset + labelSize - qualitativeRangeSize) : tickSize);
                    break;
                case 'centerinsideaboveoutside':
                    pointHeight = pointY + qualitativeRangeSize + tickSize;
                    pointY = pointY - tickSize - ((tickSize < (offset + labelSize)) && (offset + labelSize - tickSize));
                    break;
                case 'centerinsideaboveinside':
                    pointHeight = pointY + qualitativeRangeSize + ((tickSize > (labelSize + offset)) ? tickSize : (labelSize + offset - qualitativeRangeSize));
                    pointY = pointY - tickSize;
                    break;
            }
            return {x: scale.location.x, y: pointY, height:pointHeight };

        },
        // display text method for trimming
        _displayText: function (settings, loc) {
             var location = $.extend(true, loc);
            var angle = settings.textAngle % 360;
            var text = settings.text;
            var font = settings.font;
            var scale = this._scale;
            var label = this._scale.labelSettings;
            var labelWidth = ej.EjSvgRender.utils._measureText(label.labelPrefix + scale.minimum + label.labelSuffix, null, label).width;
            var textWidth = ej.EjSvgRender.utils._measureText(text, null, font).width;
            var maxHeight = ej.EjSvgRender.utils.rotatedLabel(settings, this, angle, settings.text);
            var maxWidth = this.rotatedLabel(font, this, angle, settings.text);
            var gap, padding = 5;
            var height = this.model.height;
            var width = this.model.width;
            var isRotate = false;
            var scaleLocation = this._scaleLoc();
            if (settings.textSpacing) {
                textWidth = textWidth + settings.textSpacing;
                maxWidth = maxWidth + settings.textSpacing;
            }

            if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) {

               if ((location.x <= scaleLocation.x) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height)) {
                    width = (scale.location.x - labelWidth / 2);
               }
               else if ((location.x <= scaleLocation.x) && (location.y <= scaleLocation.y) && (angle < 90)) {
                    height = scaleLocation.y;
               }
               else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + this.model.quantitativeScaleLength)) && (location.y <= scaleLocation.y)) {
                    height = scaleLocation.y;
               }
               else if ((location.y <= scaleLocation.y) && (location.x >= (scaleLocation.x + this.model.quantitativeScaleLength)) && (angle >= 90)) {
                    height = scaleLocation.y;
               }

               else if ((location.x >= (scaleLocation.x + this.model.quantitativeScaleLength)) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height) && (angle < 270)) {
                    width = width - (scaleLocation.x + this.model.quantitativeScaleLength);
                    location.x = location.x - (scaleLocation.x + this.model.quantitativeScaleLength);
               }
               else if ((location.x <= scaleLocation.x) && (location.y >= scaleLocation.height) && (angle > 270)) {
                    location.y = location.y - scaleLocation.height;
               }
               else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + this.model.quantitativeScaleLength)) && (location.y >= scaleLocation.height)) {
                    location.y = location.y - scaleLocation.height;
               }
               else if ((location.x >= (scaleLocation.x + this.model.quantitativeScaleLength)) && (location.y >= scaleLocation.height) && (angle >= 180) && (angle < 270)) {
                    location.y = location.y - scaleLocation.height;
               }

               gap = this.calcGap(angle, location, maxWidth, maxHeight, isRotate, width, height);
               isRotate = gap.rotate;
               gap = gap.trimSize;
            }
            else {
                var scaleWidth = scaleLocation.height - scaleLocation.y;
                scaleLocation.y = this.model.height - scale.location.y - this.model.quantitativeScaleLength;
                scaleLocation.height = scaleLocation.y + this.model.quantitativeScaleLength;

                if ((location.x <= scaleLocation.x) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height)) {
                    width = (scale.location.x - labelWidth / 2);
                }
                else if ((location.x <= scaleLocation.x) && (location.y <= scaleLocation.y) && (angle < 90)) {
                    height = scaleLocation.y;
                }
                else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + scaleWidth)) && (location.y <= scaleLocation.y)) {
                    height = scaleLocation.y;
                }
                else if ((location.y <= scaleLocation.y) && (location.x >= (scaleLocation.x + scaleWidth)) && (angle >= 90)) {
                    height = scaleLocation.y;
                }

                else if ((location.x >= (scaleLocation.x + scaleWidth)) && (scaleLocation.y <= location.y) && (location.y <= scaleLocation.height) && (angle <= 260)) {
                    width = width - (scaleLocation.x + scaleWidth);
                    location.x = location.x - (scaleLocation.x + scaleWidth);
                }
                else if ((location.x <= scaleLocation.x) && (location.y >= scaleLocation.height) && (angle >= 280)) {
                    location.y = location.y - scaleLocation.height;
                }
                else if ((location.x >= scaleLocation.x) && (location.x <= (scaleLocation.x + scaleWidth)) && (location.y >= scaleLocation.height)) {
                    location.y = location.y - scaleLocation.height;
                }
                else if ((location.x >= (scaleLocation.x + scaleWidth)) && (location.y >= scaleLocation.height) && (angle >= 180) && (angle <= 260)) {
                    location.y = location.y - scaleLocation.height;
                }

                gap = this.calcGap(angle, location, maxWidth, maxHeight, isRotate, width, height);
                isRotate = gap.rotate;
                gap = gap.trimSize;
            }
            // Trimming opetation called here
            var displayText = text;
            displayText = this._trim(text,settings,gap,isRotate);
            return displayText;
        },

        // trim calculation perform here
        _trim: function (text, settings, gap, isRotate) {
            var displayText = text;
            var maxWidth = this.rotatedLabel(settings.font, this, settings.textAngle, text);
            var textWidth = ej.EjSvgRender.utils._measureText(text, null, settings.font).width;
            if (settings.textSpacing) {
                textWidth = textWidth + settings.textSpacing;
                maxWidth = maxWidth + settings.textSpacing;
            }
            if (textWidth > gap && text != "") {
                var space;
                for (var t = 1; t <= text.toString().length; t++) {
                    text = displayText.toString().substring(0, t) + '... ';
                    if (isRotate == true)
                        space = ej.EjSvgRender.utils.rotatedLabel(settings, this, settings.textAngle, text);
                    else
                        space = ej.EjSvgRender.utils._measureText(text, null, settings.font).width;

                    if (settings.textSpacing) {
                        space = space + settings.textSpacing;
                    }
                    if (space >= gap) {
                        text = text.toString().substring(0, t - 1) + '... ';
                        displayText = text;
                        break;
                    }
                }
                var temp = displayText.toString(0, text.toString.lenght - 4);
                if (temp != displayText)
                    displayText = text;
            }
            return displayText;
        },
       
        _getVerticalScaleLocation: function(scaleWidth){
            return { x: this._scale.location.y, y: this.model.height - this._scale.location.x - scaleWidth };

        },
      
        _horizontalTextPositioning: function (element, region, scaleLocation, scaleRegion, regions) {
            var position = element.textPosition.toLowerCase();
            var alignment = region.alignment;
            var scaleLocation = this._scaleLoc();
            var label = this._scale.labelSettings;
            var maxLabelWidth = ej.EjSvgRender.utils._measureText(label.labelPrefix + this._scale.maximum + label.labelSuffix, null, label).width;            
            switch (position) {
                case 'left':
                    region.x = scaleLocation.x - region.width - region.padding - maxLabelWidth/2;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height/3: (alignment == 'far' ? scaleRegion.height : region.height));
                    regions[0].push(region);
                    break;
                case 'right':
                    region.x = scaleLocation.x + scaleRegion.width + region.padding + maxLabelWidth/2;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height / 3 : (alignment == 'far' ? scaleRegion.height : region.height));
                    regions[1].push(region);
                    break;
                case 'top':
                    region.x = (alignment == 'center') ? scaleRegion.x + scaleRegion.width / 2 - region.width / 2 : (alignment == 'near' ? scaleRegion.x : scaleRegion.x + scaleRegion.width - region.width);
                    region.y = scaleLocation.y - region.padding;
                    regions[2].push(region);
                    break;
                case 'bottom':                    
                    region.x = (alignment == 'center') ? scaleRegion.x + scaleRegion.width / 2 - region.width / 2 : (alignment == 'near' ? scaleRegion.x : scaleRegion.x + scaleRegion.width - region.width);
                    region.y = scaleLocation.height + region.height / 2 + region.padding;
                    regions[3].push(region);
                    break;
            }
        },
       
        _VerticalTextPositioning: function (element, region, scaleLocation, scaleRegion, regions) {
            var position = element.textPosition.toLowerCase();
            var alignment = region.alignment;
            var scaleBounds = this._scaleLoc();
            switch (position) {
                case 'left':
                    region.x = scaleBounds.y - region.width - region.padding;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height / 3 : (alignment == 'far' ? scaleRegion.height : region.height));
                    if (this._tickPosition == 'near' && this._tickPlacement == 'inside' && (this._labelPosition=='above' && this._labelPlacement=='inside'))
                        region.x += this._scale.majorTickSettings.size;
                    regions[0].push(region);
                    break;
                case 'right':
                    region.x = scaleBounds.height + region.padding;
                    region.y = scaleLocation.y + ((alignment == 'center') ? scaleRegion.height / 2 + region.height / 3 : (alignment == 'far' ? scaleRegion.height : region.height));
                    regions[1].push(region);
                    break;
                case 'top':
                    region.x = scaleRegion.x - region.width / 2 + (alignment == 'center' ? scaleRegion.width / 2 : (alignment == 'far' && scaleRegion.width));
                    region.y = scaleRegion.y - region.padding - this._scale.labelSettings.size/2;                   
                    regions[2].push(region);
                    break;
                case 'bottom':
                    region.x = scaleRegion.x - region.width / 2 + (alignment=='center' ? scaleRegion.width / 2 : (alignment == 'far' && scaleRegion.width));
                    region.y = scaleRegion.y + scaleRegion.height + region.height + region.padding - region.height / 3 + this._scale.labelSettings.size / 2;
                    regions[3].push(region);
                    break;
            }
        },
    
        _positionTextGroup: function (elements, regions, horScaleRegion) {
            var elementRegions=[[], [], [], []];
            var isHorizontal = (this._orientation == 'horizontal');            
            var scaleLocation = isHorizontal ? this._scaleLocation : this._getVerticalScaleLocation(horScaleRegion.width);
            var scaleRegion = isHorizontal ? horScaleRegion : { x: scaleLocation.x, y: scaleLocation.y, width: horScaleRegion.height, height: horScaleRegion.width };
            for (var i = 0; i < elements.length; i++) {
                if (elements[i]) {
                    if (isHorizontal)
                        this._horizontalTextPositioning(elements[i], regions[i], scaleLocation, scaleRegion, elementRegions);
                    else
                        this._VerticalTextPositioning(elements[i], regions[i], scaleLocation, scaleRegion, elementRegions);
                }                
            }
            this._avoidElementsOverlapping(elementRegions);
        },
       
        _avoidElementsOverlapping: function (regions) {
            for (var j = 0; j < regions.length; j++) {
                var region = regions[j];
                var isOverlap = false;
                do {
                    for (var i = 0; i < region.length; i++) {
                        for (var k = i - 1; k >= 0; k--) {
                            if (this._isOverlapping(region[i], region[k])) {
                                switch (j) {
                                    //Placing overlapped text for left and right positioned text elements
                                    case 0:
                                    case 1:
                                        if (region[k].y > region[i].y)
                                            region[k].y = region[i].y + region[k].height + (region[k].padding > 1 ? region[k].padding : 1);
                                        else
                                            region[i].y = region[k].y + region[i].height + (region[i].padding > 1 ? region[i].padding : 1);
                                        break;
                                        //Placing overlapped text at top position
                                    case 2:
                                        region[k].y = region[i].y - region[i].height - (region[k].padding > 1 ? region[k].padding : 1);
                                        break;
                                        //Placing overlapped text at bottom position
                                    default:
                                        region[i].y = region[k].y + region[k].height + (region[k].padding > 1 ? region[k].padding : 1);
                                        break;
                                }
                                isOverlap = true;
                                break;
                            }
                            else
                                isOverlap = false;
                        }
                    }
                } while (isOverlap);
            }
            this._applyLeftRightAnchor([regions[0], regions[1]]);
            this._applyTopBottomAnchor([regions[2], regions[3]]);
        },
      
        _locateTextGroup: function (elements, regions) {            
            for (var i = 0; i < elements.length; i++) {
                if (elements[i]) {
                    elements[i]._location = { x: 0, y: 0 };
                    elements[i]._location.x = regions[i].x;
                    elements[i]._location.y = regions[i].y;
                }
            }
        },
     
        _applyLeftRightAnchor: function (regions) {
            var anchor;
            for (var i = 0; i < regions.length; i++) {
                var region = regions[i];
                var maxSize = 0;
                for (var j = 0; j < region.length; j++)
                    maxSize = Math.max(maxSize, region[j].width);
                for (var j = 0; j < region.length; j++) {
                    anchor = region[j].anchor;
                    if (anchor == 'start')
                        region[j].x += (i == 0) && (region[j].width - maxSize);
                    else if (anchor == 'middle')
                        region[j].x += (i == 0) ? (region[j].width - maxSize) / 2 : (maxSize - region[j].width) / 2;
                    else
                        region[j].x += (i != 0) && (maxSize - region[j].width);
                }
            }
        },
     
        _applyTopBottomAnchor: function (regions) {            
            for (var i = 0; i < regions.length; i++) {
                var nearAlignment = [], farAlignment = [], centerAlignment = [];
                var region = regions[i];
                for (var j = 0; j < region.length; j++)
                    if (region[j].alignment == 'near')
                        nearAlignment.push(region[j])
                    else if (region[j].alignment == 'far')
                        farAlignment.push(region[j]);
                    else
                        centerAlignment.push(region[j]);

                this._applyNearAnchor(nearAlignment);
                this._applyCenterAnchor(centerAlignment);
                this._applyFarAnchor(farAlignment);
            }
        },
       
        _applyNearAnchor: function (regions) {
            if (regions.length > 1) {
                var maxSize = 0;
                for (var i = 0; i < regions.length; i++)
                    maxSize = Math.max(maxSize, regions[i].width);
                for (var i = 0; i < regions.length; i++)
                    if (regions[i].anchor == 'start')
                        regions[i].x += (this._orientation == 'vertical') && (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'middle')
                        regions[i].x += (this._orientation == 'horizontal') && (maxSize - regions[i].width) / 2;
                    else
                        regions[i].x += (this._orientation == 'horizontal') ? (maxSize - regions[i].width) : (maxSize - regions[i].width) / 2;
            }
        },
      
        _applyCenterAnchor: function (regions) {
            if (regions.length > 1) {
                var maxSize = 0;
                for (var i = 0; i < regions.length; i++)
                    maxSize = Math.max(maxSize, regions[i].width);
                for (var i = 0; i < regions.length; i++)
                    if (regions[i].anchor == 'start')
                        regions[i].x += (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'end')
                        regions[i].x += (maxSize - regions[i].width) / 2;
            }
        },
     
        _applyFarAnchor: function (regions) {
            if (regions.length > 1) {
                var maxSize = 0;
                for (var i = 0; i < regions.length; i++)
                    maxSize = Math.max(maxSize, regions[i].width);
                for (var i = 0; i < regions.length; i++)
                    if (regions[i].anchor == 'start')
                        regions[i].x += (this._orientation == 'horizontal') ? regions[i].width - maxSize : (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'middle')
                        regions[i].x += (this._orientation == 'horizontal') && (regions[i].width - maxSize) / 2;
                    else if (regions[i].anchor == 'end')
                        regions[i].x += (this._orientation == 'vertical') && (maxSize - regions[i].width) / 2;
            }
        },
       
        _isOverlapping: function (reg1, reg2) {
            return !(reg1.x + reg1.width < reg2.x || reg1.x > reg2.x + reg2.width || reg1.y - reg1.height > reg2.y || reg1.y < reg2.y - reg2.height);
        },
        
        _initializeValues: function () {
            this._scale = this.model.quantitativeScaleSettings;
            this._labelPosition = this._scale.labelSettings.position.toLowerCase();
            this._tickPosition = this._scale.tickPosition.toLowerCase();
            this._flowDirection = this.model.flowDirection.toLowerCase();
            this._orientation = this.model.orientation.toLowerCase();
            this._tickPlacement = this._scale.tickPlacement.toLowerCase();
            this._labelPlacement = this._scale.labelSettings.labelPlacement.toLowerCase();
        },
      
        _changeOrientation: function (bulletGroup) {
            if (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical) {               
                bulletGroup.setAttribute("transform", "translate(0," + this.model.height +")rotate(-90)");
            }
        },
       
        _setModel: function (options) {
            var isFlag = true;
            for (var key in options) {
                this.model.enableAnimation = false;
                switch (key) {
                    case "height":
                        this.model.height = options[key];
                        break;
                    case "width":
                        this.model.width = options[key];
                        break;
                    case "theme":
                        this.model.theme = options[key];
                        this._setTheme(ej.datavisualization.BulletGraph.Themes, this.model.theme);
                        break;
                    case "orientation":
                        this.model.orientation = options[key];
                        break;
                    case "flowDirection":
                        this.model.flowDirection = options[key];
                        break;
                    case "qualitativeRangeSize":
                        this.model.qualitativeRangeSize = options[key];
                        break;
                    case "quantitativeScaleLength":
                        this.model.quantitativeScaleLength = options[key];
                        break;
                    case "quantitativeScaleSettings":
                        $.extend(true, this.model.quantitativeScaleSettings, {}, options[key]);
                        break;
                    case "applyRangeStrokeToTicks":
                        this.model.applyRangeStrokeToTicks = options[key];
                        break;
                    case "applyRangeStrokeToLabels":
                        this.model.applyRangeStrokeToLabels = options[key];
                        break;
                    case "qualitativeRanges":
                        $.extend(true, this.model.qualitativeRanges, {}, options[key]);
                        break;
                    case "captionSettings":
                        $.extend(true, this.model.captionSettings, {}, options[key]);
                        break;
                    case "dataSource":
                        $.extend(true, this.model.fields, {}, options[key]);
                        break;
                    case "value":
                        for (var i = 0; this.model.quantitativeScaleSettings.featureMeasures[i] != null; i++) {
                            this.model.quantitativeScaleSettings.featureMeasures[i].value = parseFloat(this.value());
                        }
                        break;
                    case "comparativeMeasureValue":
                        for (var i = 0; this.model.quantitativeScaleSettings.featureMeasures[i] != null; i++) {
                            this.model.quantitativeScaleSettings.featureMeasures[i].comparativeMeasureValue = parseFloat(this.comparativeMeasureValue());
                        }
                        break;
                    case "enableAnimation":
                        this.model.enableAnimation = options[key];
                        if (this.model.enableAnimation) {
                            $(this.svgObject).empty();
                            this._renderBulletElements();
                            this._animateMeasures();
                        }
                        isFlag = false;
                        break;
                }
            }
            if (isFlag) {
                $(this.svgObject).empty();
                this._renderBulletElements();
            }
        },
      
        _bindData: function () {

            if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) {
                if (this.model.fields["dataSource"] instanceof ej.DataManager) {
                    this._initDataSource();
                }
                else {
                    this._dataCount = this.model.fields["dataSource"].length;
                    this._drawMeasures();
                }
            }
            else {      //this.model.quantitativeScaleSettings.featureMeasures != null
                this._dataCount = this._scale.featureMeasures.length;
                this._drawMeasures();
            }
        },
      
        _drawMeasures: function () {
            this._drawFeatureMeasureBar();
            this._drawComparativeMeasureSymbol();
        },
      
        _initDataSource: function () {
            var query = this._columnToSelect(this.model.fields);
            var proxy = this;
            var queryPromise = this.model.fields["dataSource"].executeQuery(query);
            queryPromise.done(function (e) {
                proxy.model.fields["dataSource"] = e.result;
                proxy._dataCount = e.result.length;
                proxy._drawFeatureMeasureBar();
                proxy._drawComparativeMeasureSymbol();
            });
        },
      
        _columnToSelect: function (mapper) {
            var column = [], queryManager = ej.Query();
            if (ej.isNullOrUndefined(mapper.query)) {
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "query" && col !== "dataSource")
                        column.push(mapper[col]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!this.model.fields["dataSource"].dataSource.url.match(mapper.tableName + "$"))
                    !ej.isNullOrUndefined(mapper.tableName) && queryManager.from(mapper.tableName);
            }
            else
                queryManager = mapper.query;
            return queryManager;
        },
             
        _drawCaption: function () {
            if (this.model.drawCaption) {
                var data = { font: this.model.captionSettings.font, location: this.model.captionSettings.location, subTitle: this.model.captionSettings.subTitle, text: this.model.captionSettings.text, textAngle: this.model.captionSettings.textAngle };
                this._trigger("drawCaption", data);
                this.model.captionSettings = $.extend(this.model.captionSettings, data);
            }
        },
      
        _drawBulletSymbol: function(shape, symbolRect, symbolOptions, captionGroup){
            var arrowPath = "M" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2))) + " " + (symbolRect.y) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                    + "L" + " " + ((symbolRect.x - (symbolRect.width / 2)) + symbolRect.width) + " " + (symbolRect.y + (symbolRect.height / 4));

            switch (shape.toLowerCase()) {
                case 'circle':
                    var radius = Math.min(symbolRect.height, symbolRect.width) / 2;
                    symbolOptions.cx = symbolRect.x;
                    symbolOptions.cy = symbolRect.y,
                    symbolOptions.r = radius,
                    this.svgRenderer.drawCircle(symbolOptions, captionGroup);
                    break;

                case 'leftarrow':                        
                    symbolOptions.d = arrowPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'rightarrow':
                    symbolOptions.d = arrowPath;
                    symbolOptions.transform = 'rotate(180,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'uparrow':
                    symbolOptions.d = arrowPath;
                    symbolOptions.transform = 'rotate(90,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'downarrow':
                    symbolOptions.d = arrowPath;
                    symbolOptions.transform = 'rotate(-90,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'cross':
                    var crossPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " " +
                        "M" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = crossPath;
                    symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'horizontalline':
                    var horizPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " " + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = horizPath;
                    symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'verticalline':
                    var verPath = "M" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = verPath;
                    symbolOptions.stroke = (symbolOptions.stroke) ? symbolOptions.stroke : symbolOptions.fill;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'triangle':
                    var triPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " Z";
                    symbolOptions.d = triPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'invertedtriangle':
                    var triPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " Z";
                    symbolOptions.d = triPath;
                    symbolOptions.transform = 'rotate(180,' + (symbolRect.x) + ',' + symbolRect.y + ')';
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'hexagon':
                    var hexPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 4)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 4)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = hexPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'wedge':
                    var wedgePath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + symbolRect.width / 2) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 4)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = wedgePath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'pentagon':
                    var eq = 72;
                    var radius = Math.sqrt(symbolRect.height * symbolRect.height + symbolRect.width * symbolRect.width) / 2;
                    var sb = ej.EjSvgRender.utils._getStringBuilder();
                    for (var i = 0; i <= 5; i++) {
                        var deg = i * eq;
                        var rad = (Math.PI / 180) * deg;
                        var x1 = radius * Math.cos(rad);
                        var y1 = radius * Math.sin(rad);
                        if (i == 0)
                            sb.append("M" + " " + (symbolRect.x + x1) + " " + (symbolRect.y + y1) + " ");

                        else
                            sb.append("L" + " " + (symbolRect.x + x1) + " " + (symbolRect.y + y1) + " ");

                    }                        
                    symbolOptions.d = sb.toString();
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'star':
                    var starPath = "M" + " " + (symbolRect.x + (symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 6)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 6)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 3)) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = starPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'rectangle':
                    var rectPath = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 2));
                    symbolOptions.d = rectPath;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'trapezoid':
                    var path = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (-symbolRect.height / 4)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2) + (symbolRect.width)) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2) + (symbolRect.width)) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y + (symbolRect.height / 4)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = path;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'diamond':
                    var path = "M" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (-symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (symbolRect.width / 2)) + " " + (symbolRect.y) + " "
                        + "L" + " " + (symbolRect.x) + " " + (symbolRect.y + (symbolRect.height / 2)) + " "
                        + "L" + " " + (symbolRect.x + (-symbolRect.width / 2)) + " " + (symbolRect.y);
                    symbolOptions.d = path;
                    this.svgRenderer.drawPath(symbolOptions, captionGroup);
                    break;
                case 'ellipse':                        
                    symbolOptions.cx = symbolRect.x;
                    symbolOptions.cy = symbolRect.y;
                    symbolOptions.rx = symbolRect.width/2;
                    symbolOptions.ry = symbolRect.height/2;
                    this.svgRenderer.drawEllipse(symbolOptions, captionGroup);
                    break;
                case 'image':
                    symbolOptions.x = symbolRect.x - symbolRect.width / 2;
                    symbolOptions.y = symbolRect.y - symbolRect.height / 2;
                    symbolOptions.width = symbolRect.width;
                    symbolOptions.height = symbolRect.height;
                    symbolOptions.href = this.model.captionSettings.indicator.symbol.imageURL;
                    symbolOptions.visibility = 'visible';
                    this.svgRenderer.drawImage(symbolOptions, captionGroup);
                    break;
                default:
                    break;
            }                
        },
       
        _drawIndicator: function () {
            if (this.model.captionSettings.indicator.visible) {
                var indicatorSettings = this.model.captionSettings.indicator;
                var indicatorEventArgs = $.extend({}, { indicatorSettings: this.model.captionSettings.indicator });
                if (this.model.drawIndicator) {
                    this._trigger("drawIndicator", indicatorEventArgs);
                    if (indicatorEventArgs.cancel == false) {
                        indicatorSettings = indicatorEventArgs.indicatorSettings;
                        this.model.captionSettings.indicator = indicatorSettings;
                    }
                }
                var textBounds = ej.EjSvgRender.utils._measureText(indicatorSettings.text, null, indicatorSettings.font);
                this._indicator = { 'bounds': textBounds, 'settings': indicatorSettings };
            }
        },
      
        _indicatorTextOptions: function (settings, rect, bounds, id) {
            var location = settings._location ? settings._location : settings.location;
            return {
                'class' : id,
                'x': location.x + rect.width + settings.textSpacing,
                'y': location.y - rect.height / 2 + parseFloat(settings.font.size) / 3,
                'fill': settings.font.color,
                'font-size': settings.font.size,
                'font-family': settings.font.fontFamily,
                'font-style': settings.font.fontStyle,
                'font-weight': settings.font.fontWeight,
                'text-anchor': 'start',
                'opacity': settings.font.opacity,
                'transform': 'rotate(' + settings.textAngle + ',' + rect.x  + ',' + rect.y + ')'
            };
        },
      
        _indicatorSymbolOptions: function (group, settings) {
            return {
                'id': group.id + '_indicatorSymbol',
                'stroke': settings.symbol.border.color,
                'fill': settings.symbol.color,
                'stroke-width': settings.symbol.border.width,
                'opacity': settings.symbol.opacity
            };
        },
       
        _textOptions: function (text, id) {
            var location = (text._location ? text._location : text.location);
            var textOptions = {
                'class': id,
                'x': location.x,
                'y': location.y,
                'fill': text.font.color,
                'font-size': text.font.size,
                'font-family': text.font.fontFamily,
                'font-style': text.font.fontStyle,
                'font-weight': text.font.fontWeight,
                'text-anchor': 'start',
                'opacity': text.font.opacity,
				'cursor':'default',
                'transform': 'rotate(' + text.textAngle + ',' + (location.x) + ',' + (location.y) + ')'
            };
            return textOptions;
        },
       
        _drawScale: function (captionGroup, scaleGroup) {
            var scaleLocation = this._scale._location ? this._scale._location : this._scale.location;
            var pointY, pointX;
            if (this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Far || this._tickPosition == ej.datavisualization.BulletGraph.TickPosition.Center)
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? scaleLocation.y : scaleLocation.y + this._scale.labelSettings.offset + this._scale.labelSettings.size;
            else
                pointY = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below) ? scaleLocation.y + this._scale.majorTickSettings.size : scaleLocation.y + this._scale.labelSettings.offset + this._scale.labelSettings.size + this._scale.majorTickSettings.size;

            if (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward)
                pointX = scaleLocation.x;
            else
                pointX = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? scaleLocation.x - this._scale.labelSettings.offset : scaleLocation.x;

            var scaleOptions = {
                'id': this.svgObject.id + '_SvgScale',
                'x': pointX,
                'y': pointY,
                'width': this.model.quantitativeScaleLength,
                'height': this.model.qualitativeRangeSize,
                'fill': "transparent",
                'stroke-width': 0
            };
            this.svgRenderer.drawRect(scaleOptions, scaleGroup);
        },
     
        _drawMajorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, y1, tickIndex = 0;
            var scaleLocation = this._scale.location;
            var isTicksInside = (this._tickPlacement == 'inside');
            var locX = this._scale.location.x;
            var scaleLength = this.model.quantitativeScaleLength;
            var labelOffset = this._scale.labelSettings.offset;

            var rangeSize = this.model.qualitativeRangeSize;
            var majorTickWidth = this._scale.majorTickSettings.width;
            var majorTickSize = this._scale.majorTickSettings.size;

            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);
            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isBelow = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below);

            if (isForward)
                majorPointX = isHorizontal ? locX + (majorTickWidth / 2) : locX + scaleLength + (majorTickWidth / 2);
            else
                majorPointX = isHorizontal ? locX + scaleLength - (majorTickWidth / 2) : locX + (majorTickWidth / 2);
            
            if(this._tickPosition == 'far')
                y1 = scaleLocation.y + rangeSize + (isTicksInside ? -majorTickSize : 0);
            else if(this._tickPosition == 'near')
                y1 = scaleLocation.y + (isTicksInside ? 0 : -majorTickSize);
            else
                y1 = scaleLocation.y + (rangeSize / 2) - (majorTickSize / 2);
            
            var min = this._scale.minimum;
            var max = this._scale.maximum;
            var interval = this._scale.interval;
            
            for (var i = min; i <= max; i += interval) {
                tickIndex++;
                if (i >= max)
                    if (isHorizontal && !isForward)
                        majorPointX += majorTickWidth;
                    else
                        majorPointX -= majorTickWidth;
                if (!isHorizontal && isForward)
                    majorPointX = (i >= max) ? majorPointX + (majorTickWidth) : (i == min) ? majorPointX - (majorTickWidth) : majorPointX;

                if (this.model.applyRangeStrokeToTicks)
                    strokeColor = this._bindingRangeStrokes(majorPointX - (majorTickWidth / 2));

                var options = this._majorTickLines(this._scale, majorPointX, y1);

                if (!isHorizontal && isForward && i == min)
                    majorPointX = majorPointX + majorTickWidth;
                
                if (this.model.drawTicks) {
                    data = { majorTickSettings: this._scale.majorTickSettings, minorTickSettings: this._scale.minorTickSettings, minorTicksPerInterval: this._scale.minorTicksPerInterval, maximum: max, minimum: min, interval: interval };
                    this._trigger("drawTicks", data);                    
                    options = this._majorTickLines(data, majorPointX, y1);
                }
                var majorTicks = this.svgRenderer.createLine(options);
                majorPointX = ((!isForward && isHorizontal) || (isForward && !isHorizontal)) ? majorPointX - intervalValue : majorPointX + intervalValue;
                scaleGroup.appendChild(majorTicks);
            }
        },
        _majorTickLines: function (data, majorPointX, y1) {
            var options = {
                'x1': majorPointX,
                'y1': y1,
                'x2': majorPointX,
                'y2': y1 + data.majorTickSettings.size,
                'stroke-width': data.majorTickSettings.width,
                'stroke': (this.model.applyRangeStrokeToTicks && strokeColor) ? strokeColor : data.majorTickSettings.stroke
            };
            return options;
        },
       
        _forwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x)) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (var k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= (this._rangeCollection[k] + this._scale.location.x) && majorPointX <= (this._rangeCollection[k + 1] + this._scale.location.x))
                    return this.model.qualitativeRanges[k + 1].rangeStroke;
            }
        },
       
        _backwardStrokeBinding: function (majorPointX) {
            if (majorPointX >= this._rangeCollection[this._rangeCollection.length - 1]) {
                return this.model.qualitativeRanges[0].rangeStroke;
            }
            for (k = 0; k <= this._rangeCollection.length - 1; k++) {
                if (majorPointX >= this._rangeCollection[k] && majorPointX < this._rangeCollection[k + 1]) {
                    var index = $.inArray(this._rangeCollection[k], this._rangeCollection)
                    return this.model.qualitativeRanges[(this._rangeCollection.length - 1) - index].rangeStroke;
                }
            }
        },
      
        _bindingRangeStrokes: function (majorPointX) {
            if (this._rangeCollection.length == 1) {
                if (majorPointX >= this._scale.location.x && majorPointX <= (this._rangeCollection[0] + this._scale.location.x))
                    return this.model.qualitativeRanges[0].rangeStroke;
            }
            else {
                if ((this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward) || (this.model.orientation == ej.datavisualization.BulletGraph.Orientation.Vertical && this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)) {
                    return this._forwardStrokeBinding(majorPointX);
                }
                else {
                    return this._backwardStrokeBinding(majorPointX);
                }
            }
        },
       
        _drawMinorTicks: function (intervalValue, scaleGroup) {
            var majorPointX, minorPointX, y1, y2, x, tickIndex = 0;

            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);

            var isTicksInside = (this._tickPlacement == 'inside');
            var locX = this._scale.location.x;
            var rangeSize = this.model.qualitativeRangeSize;
            var labelOffset = this._scale.labelSettings.offset;

            var majorTickSize = this._scale.majorTickSettings.size;
            var minorTickSize = this._scale.minorTickSettings.size;
            var minorTickWidth = this._scale.minorTickSettings.width;
            var scaleLength = this.model.quantitativeScaleLength;

            if (isForward)
                majorPointX = isHorizontal ? locX : locX + scaleLength;
            else
                majorPointX = isHorizontal ? locX + scaleLength - (minorTickWidth / 2) : locX;

            switch (this._tickPosition) {
                case 'far':
                    y1 = this._scale.location.y + rangeSize;
                    y2 = y1 + (isTicksInside ? -minorTickSize : minorTickSize);
                    break;
                case 'near':
                    y1 = this._scale.location.y + (isTicksInside ? minorTickSize : -minorTickSize);
                    y2 = this._scale.location.y;
                    break;                
                default:
                    y1 = this._scale.location.y + (rangeSize / 2) - (minorTickSize / 2);
                    y2 = y1 + minorTickSize;
                    break;
            }
            var maxLimit = this._scale.maximum;
            var min = this._scale.minimum;
            var interval = this._scale.interval;
            var minorTicksPerInterval = this._scale.minorTicksPerInterval;
           
            for (var i = min; i <= maxLimit; i += interval) {
                minorPointX = intervalValue / minorTicksPerInterval;

                for (var j = 1; j <= minorTicksPerInterval; j++) {
                    tickIndex++;
                    x = ((!isHorizontal && isForward) || (isHorizontal && !isForward))
                        ? majorPointX - minorPointX + (minorPointX / (minorTicksPerInterval + 1))
                        : majorPointX + minorPointX - (minorPointX / (minorTicksPerInterval + 1));

                    if (!isForward && isHorizontal) {
                        if (tickIndex >= ((maxLimit - min) * minorTicksPerInterval))
                            x += (minorTickWidth / 2);
                    }

                    if (this.model.applyRangeStrokeToTicks)
                        strokeColor = this._bindingRangeStrokes(x);
                   
                    var options = this._minorTickLines(this._scale, x, y1, y2);
                    if (this.model.drawTicks) {
                        data = { majorTickSettings: this._scale.majorTickSettings, minorTickSettings: this._scale.minorTickSettings, minorTicksPerInterval: this._scale.minorTicksPerInterval, maximum: this._scale.maximum, minimum: this._scale.minimum, interval: this._scale.interval };
                        this._trigger("drawTicks", data);
                        this._scale = $.extend(this._scale, data);
                        options = this._minorTickLines(data, x, y1, y2);
                    }
                   
                    var chkMax;
                    if (!isForward && isHorizontal) {
                        chkMax = (scaleLength + locX + 1);
                        if (x <= chkMax && x >= locX + 1) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    else if (isForward && !isHorizontal) {
                        chkMax = (locX + 1);
                        if (x >= chkMax) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    else {
                        chkMax = scaleLength + locX + 1;
                        if (x <= chkMax) {
                            var minorTicks = this.svgRenderer.createLine(options);
                            scaleGroup.appendChild(minorTicks);
                        }
                    }
                    
                    minorPointX = (intervalValue / minorTicksPerInterval) * (j + 1);
                }
                majorPointX = ((!isForward && isHorizontal) || (isForward && !isHorizontal)) ? majorPointX - intervalValue : majorPointX + intervalValue;
            }
        },
        _minorTickLines: function (data, x, y1, y2) {
            var options = {
                'x1': x,
                'y1': y1,
                'x2': x,
                'y2': y2,
                'stroke-width': data.minorTickSettings.width,
                'stroke': (this.model.applyRangeStrokeToTicks && strokeColor) ? strokeColor : data.minorTickSettings.stroke
            };
            return options;
        },
       
        _drawLabels: function (intervalValue, scaleGroup) {
            var labelX, pointY;
            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);

            var locX = this._scale.location.x;
            var offset = this._scale.labelSettings.offset;
            var scaleLength = this.model.quantitativeScaleLength;

            var majorTickWidth = this._scale.majorTickSettings.width;

            if (isForward)
                labelX = isHorizontal ? locX : locX + scaleLength;
            else
                labelX = isHorizontal ? locX + scaleLength : locX;
            
            var placement = this._tickPosition + this._labelPlacement;
            var isLabelsBelow = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below);

            var locY = this._scale.location.y;
            var rangeSize = this.model.qualitativeRangeSize;
            var labelSize = this._scale.labelSettings.size;
            var majorTickSize = this._scale.majorTickSettings.size;

            switch (placement) {
                case 'faroutside':
                case 'centeroutside':                    
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize + offset + labelSize : this._scale.location.y - offset;
                    break;
                case 'farinside':
                case 'centerinside':
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize - offset : this._scale.location.y + offset + (isHorizontal ? labelSize : 0);
                    break;
                case 'nearoutside':
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize + offset + labelSize : this._scale.location.y - offset;
                    break;
                case 'nearinside':
                    pointY = isLabelsBelow ? this._scale.location.y + rangeSize - offset : this._scale.location.y + offset + labelSize;
                    break;
            }
            var min = this._scale.minimum;
            var max = this._scale.maximum;
            var interval = this._scale.interval;
            var isLabelsInside = (this._labelPlacement == "inside");

            for (var i = min; i <= max; i += interval) {               
                var y = pointY;
                var x = labelX;
                var font = this._scale.labelSettings.font;
                font.size = labelSize;
                var labelWidth = {width: 0, height: 0};
                if (!isHorizontal) {
                    var label = this._scale.labelSettings.labelPrefix + i + this._scale.labelSettings.labelSuffix;
                    switch (placement) {
                        case 'faroutside':
                        case 'centeroutside':
                            if (isLabelsBelow)
                                y -= labelSize;
                            else {
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            break;
                        case 'farinside':
                        case 'centerinside':
                            if (isLabelsBelow) {                                
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            break;
                        case 'nearoutside':
                            if (!isLabelsBelow) {
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            else
                                y -= labelSize;
                            break;
                        case 'nearinside':
                            if (isLabelsBelow) {
                                labelWidth = ej.EjSvgRender.utils._measureText(label, null, font);
                                y -= labelWidth.width;
                            }
                            else
                                y -= labelSize;
                            break;
                    }                    
                    x -= labelSize/3;
                    transformText = 'rotate(' + 90 + ',' + (x) + ',' + (y) + ')';
                }
                else
                    transformText = 'rotate(' + 0 + ',' + (x) + ',' + (y) + ')';

                if (this.model.applyRangeStrokeToLabels)
                    strokeColor = this._bindingRangeStrokes(labelX);
                var labelOptions = this._labelOptions(this.model.quantitativeScaleSettings.labelSettings, x, y);
                if (this.model.drawLabels) {
                    var data = { font: this._scale.labelSettings.font, labelPrefix: this._scale.labelSettings.labelPrefix, labelSuffix: this._scale.labelSettings.labelSuffix, offset: offset, size: labelSize, stroke: this._scale.labelSettings.stroke };
                    this._trigger("drawLabels", data);
                    labelOptions = this._labelOptions(data, x, y);
                }

                if (isForward)
                    labelX = isHorizontal ? labelX + intervalValue : labelX - intervalValue;
                else
                    labelX = isHorizontal ? labelX - intervalValue : labelX + intervalValue;
                                                              
                this.svgRenderer.drawText(labelOptions, labelOptions.labelPrefix + i + labelOptions.labelSuffix, scaleGroup);
            }
        },

        _labelOptions: function (data, labelX, pointY) {
            var labelOptions = {
                'x': labelX,
                'y': pointY,
                'text-anchor': (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) ? 'middle' : 'start',
                'fill': (this.model.applyRangeStrokeToLabels && strokeColor) ? strokeColor : data.stroke,
                'font-size': data.size + "px",
                'font-family': data.font.fontFamily,
                'font-style': data.font.fontStyle,
                'font-weight': data.font.fontWeight,
                'opacity': data.font.opacity,
                'transform': transformText,
                'labelPrefix': data.labelPrefix,
                'labelSuffix': data.labelSuffix,
            };
            return labelOptions;
        },
      
        _drawQualitativeRanges: function (scaleGroup) {
            var pointX = this._scale.location.x, pointY = this._scale.location.y, width;
            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);
            var labelOffset = this._scale.labelSettings.offset;

            this._rangeCollection = [];

            var placement = this._tickPosition + this._labelPlacement;
            var isLabelBelow = (this._labelPosition == ej.datavisualization.BulletGraph.LabelPosition.Below);

            var scaleHeight = this.model.qualitativeRangeSize;
            
            var locX = this._scale.location.x;
            var min = this._scale.minimum;
            var max = this._scale.maximum;
            var scaleLength = this.model.quantitativeScaleLength;            

            for (var i = this.model.qualitativeRanges.length - 1; i >= 0; i--) {
                var data = { object: this, rangeIndex: i, rangeOptions: this.model.qualitativeRanges[i], rangeEndValue: this.model.qualitativeRanges[i].rangeEnd };
                if (this.model.drawQualitativeRanges)
                    this._trigger("drawQualitativeRanges", data);
                var rangeEnd = data.rangeEndValue;
                rangeEnd = (rangeEnd > max) ? max : rangeEnd;
                if (isForward) {
                    pointX = isHorizontal ? locX : locX + (scaleLength - scaleLength / ((max - min) / ((max - min) - (max - rangeEnd))));
                    width = isHorizontal ? scaleLength / ((max - min) / ((max - min) - (max - rangeEnd))) : scaleLength / ((max - min) / ((max - min) - (max - rangeEnd)));
                }
                else {
                    pointX = isHorizontal ? locX + (scaleLength - scaleLength / ((max - min) / ((max - min) - (max - rangeEnd)))) : locX;
                    width = isHorizontal ? scaleLength / ((max - min) / ((max - min) - (max - rangeEnd))) : scaleLength / ((max - min) / ((max - min) - (max - rangeEnd)));
                }
                var rangeOptions = {
                    'x': pointX,
                    'y': pointY,
                    'height': this.model.qualitativeRangeSize,
                    'width': (width > 0) ? (width < scaleLength ? width : scaleLength) : 0,
                    'fill': data.rangeOptions.rangeStroke,
                    'opacity': data.rangeOptions.rangeOpacity
                };
                ((isHorizontal && isForward) || (!isHorizontal && !isForward)) ? this._rangeCollection.push(width) : this._rangeCollection.push(pointX);                
                this.svgRenderer.drawRect(rangeOptions, scaleGroup);
            }
            this._rangeCollection.sort(this._sortRangeCollection);
        },
      
        _sortRangeCollection: function (a, b) {
            return (a - b);
        },
      
        _calculateFeatureMeasureBounds: function (value, categoryValue) {
            var min = this._scale.minimum;
            value = (value < min && min < 0) ? min : value;                
            if (value >= min) {
                var x, lx, width;
                var locX = this._scale.location.x;
                var scaleLength = this.model.quantitativeScaleLength;
                var diff = this._scale.maximum - this._scale.minimum;
                var valueDiff = this._scale.maximum - value;
                var orientation = this._flowDirection.toLowerCase() + this._orientation.toLowerCase();
                categoryValue = (categoryValue == null) ? "" : categoryValue;
                var font = this._scale.labelSettings.font;
                font.size = this._scale.labelSettings.size;
                var stringLength = ej.EjSvgRender.utils._measureText(categoryValue.toString(), null, font).width;

                switch (orientation) {
                    case 'forwardhorizontal':
                    case 'backwardvertical':                        
                            x = locX + ((min > 0) ? 0 : scaleLength / diff * Math.abs(min));
                            width = scaleLength / (diff / ( (min > 0) ? diff - valueDiff : value));                            
                            if (value < 0) {
                                width = Math.abs(width);
                                x -= width;
                            }
                            width = (x + width < locX + scaleLength) ? width : locX + scaleLength - x;
                            lx = locX - ((orientation == 'forwardhorizontal') ? (stringLength / 2 + this._scale.labelSettings.offset) : this._scale.labelSettings.offset);
                            break;
                    default:
                        x = locX + (scaleLength - scaleLength / (diff / (diff - valueDiff)));
                        width = (min > 0) ? scaleLength / (diff / (diff - valueDiff)) : scaleLength / (diff / (value));
                        if (value < 0) {
                            width = Math.abs(width);
                            x -= width;
                        }
                        if (x < locX ) {
                            width = x + width - locX;
                            x = locX;
                        }                        
                        lx = locX + scaleLength + ((orientation == 'backwardhorizontal') ? (stringLength / 2 + this._scale.labelSettings.offset) : this._scale.labelSettings.offset);
                        break;
                }
                return { pointX: x, Width: width, lPointX: lx };
            }
            return false;
        },
      
        _drawFeatureMeasureBar: function () {
            var j = 1, count, value, categoryValue;
            var pointY, lPointY;
            var isDataFields = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null);
            var isVertical = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical);
            var bulletValue = (typeof (this.model.value) == 'function') ? this._value() : this.model.value;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = isDataFields ? this.model.fields["dataSource"][i][this.model.fields.featureMeasures] : (ej.isNullOrUndefined(this._scale.featureMeasures[i].value) ? bulletValue : this._scale.featureMeasures[i].value);
                    categoryValue = isDataFields ? this.model.fields["dataSource"][i][this.model.fields.category] : this._scale.featureMeasures[i].category;

                    pointY = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2);
                    lPointY = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) + (this._scale.featuredMeasureSettings.width / 2) - 1;

                    var bounds = this._calculateFeatureMeasureBounds(value, categoryValue);
                    if (bounds) {

                        var featureBarOptions = this._featureBar(this._scale, bounds.pointX, pointY, bounds.Width, i);
                        if (this.model.drawFeatureMeasureBar) {
                            var data = { featuredMeasureSettings: this._scale.featuredMeasureSettings };
                            this._trigger("drawFeatureMeasureBar", data);
                            featureBarOptions = this._featureBar(data, bounds.pointX, pointY, bounds.Width, i);
                        }
                        this.svgRenderer.drawRect(featureBarOptions, this._scaleGroup);

                        if (isVertical)
                            transformText = 'rotate(' + 90 + ',' + bounds.lPointX + ',' + (lPointY - 4) + ')';
                        else
                            transformText = 'rotate(' + 0 + ',' + bounds.lPointX + ',' + lPointY + ')';


                        var categoryOptions = this._drawcategory(this.model.quantitativeScaleSettings.labelSettings, bounds.lPointX, lPointY);
                        if (this.model.drawCategory) {
                            var data = { size: this.model.quantitativeScaleSettings.labelSettings.size, stroke: this.model.quantitativeScaleSettings.labelSettings.stroke, font: this.model.quantitativeScaleSettings.labelSettings.font, categoryValue: categoryValue };
                            this._trigger("drawCategory", data);
                            categoryOptions = this._drawcategory(data, bounds.lPointX, lPointY);
                            categoryValue = data.categoryValue;
                        }
                        if (!ej.isNullOrUndefined(categoryValue))
                            this.svgRenderer.drawText(categoryOptions, categoryValue, this._scaleGroup);
                        j += 2;
                        this.value(value);
                    }
                }

            }
        },
        _drawcategory: function (category, lPointX, lPointY) {
            var categoryOptions = {
                'x': lPointX,
                'y': lPointY + (this._scale.featuredMeasureSettings.width / 2),
                'text-anchor': 'middle',
                'fill': category.stroke,
                'font-size': category.size + "px",
                'font-family': category.font.fontFamily,
                'font-style': category.font.fontStyle,
                'font-weight': category.font.fontWeight,
                'opacity': category.font.opacity,
                'transform': transformText
            };
            return categoryOptions;
        },
        _featureBar: function (data, pointX, pointY, width, i) {
            var featureBarOptions = {
                'class': this.svgObject.id + "_FeatureMeasure",
                'id': this.svgObject.id + "_FeatureMeasure_" + i,
                'x': pointX,
                'y': pointY,
                'height': data.featuredMeasureSettings.width,
                'width': width,
                'fill': data.featuredMeasureSettings.stroke,
                'onmouseover': "evt.target.setAttribute('opacity', '0.7')",
                'onmouseout': "evt.target.setAttribute('opacity', '1')"
            };
            return featureBarOptions;
        },
       
        _drawComparativeMeasureSymbol: function () {
            var j = 1, value;
            var y1, y2, x1;

            var isForward = (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward);
            var isHorizontal = (this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal);
            var bulletCMValue = (typeof (this.model.comparativeMeasureValue) == 'function') ? this._comparativeMeasureValue() : this.model.comparativeMeasureValue;
            if (this._dataCount > 0) {
                for (var i = this._dataCount - 1; i >= 0; i--) {
                    value = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][i][this.model.fields.comparativeMeasure] : (ej.isNullOrUndefined(this._scale.featureMeasures[i].comparativeMeasureValue) ? bulletCMValue : this._scale.featureMeasures[i].comparativeMeasureValue);
                    if (value >= this._scale.minimum && value <= this._scale.maximum) {
                        y1 = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) - this._scale.featuredMeasureSettings.width;
                        y2 = this._scale.location.y + (((this.model.qualitativeRangeSize / this._dataCount) * j) / 2) - (this._scale.featuredMeasureSettings.width / 2) + (2 * this._scale.featuredMeasureSettings.width);
                        
                        if (isForward)
                            x1 = isHorizontal ? this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))) : this._scale.location.x + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value)))));
                        else
                            x1 = isHorizontal ? this._scale.location.x + (this.model.quantitativeScaleLength - (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))))) : this._scale.location.x + (this.model.quantitativeScaleLength / ((this._scale.maximum - this._scale.minimum) / ((this._scale.maximum - this._scale.minimum) - (this._scale.maximum - value))));

                        var compareMeasureOptions = this._compareMeasure(this._scale, x1, y1, y2, i, value);
                        if (this.model.drawComparativeMeasureSymbol) {
                            var data = { comparativeMeasureSettings: this._scale.comparativeMeasureSettings };
                            this._trigger("drawComparativeMeasureSymbol", data);
                            compareMeasureOptions = this._compareMeasure(data, x1, y1, y2, i, value);
                        }
                       
                        this.svgRenderer.drawLine(compareMeasureOptions, this._scaleGroup);
                        j += 2;
                        this.comparativeMeasureValue(value);
                    }
                }
            }
        },
        _compareMeasure: function(data, x1, y1, y2, i, value){
            var compareMeasureOptions = {
                'class': this.svgObject.id + "_ComparativeMeasure",
                'id': this.svgObject.id + "_ComparativeMeasure_" + i,
                'x1': (value == this._scale.maximum) ? x1 - (data.comparativeMeasureSettings.width / 2) : (value == this._scale.minimum) ? x1 + (data.comparativeMeasureSettings.width / 2) : x1,
                'y1': y1,
                'x2': (value == this._scale.maximum) ? x1 - (data.comparativeMeasureSettings.width / 2) : (value == this._scale.minimum) ? x1 + (data.comparativeMeasureSettings.width / 2) : x1,
                'y2': y2,
                'stroke-width': data.comparativeMeasureSettings.width,
                'stroke': data.comparativeMeasureSettings.stroke
            };
            return compareMeasureOptions;
        },
       
        _setTheme: function (jsonObj, themeName) {
            var result = [];
            this._scale = this.model.quantitativeScaleSettings;
            for (var name in jsonObj) {
                result.push(name);
            }
            for (var i = 0; i < result.length; i++) {
                this._scale.majorTickSettings.stroke = ((!this._scale.majorTickSettings.stroke || this._scale.majorTickSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.majorTickSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.majorTickSettings.stroke : this._scale.majorTickSettings.stroke);
                this._scale.minorTickSettings.stroke = ((!this._scale.minorTickSettings.stroke || this._scale.minorTickSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.minorTickSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.minorTickSettings.stroke : this._scale.minorTickSettings.stroke);
                this._scale.labelSettings.stroke = ((!this._scale.labelSettings.stroke || this._scale.labelSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.labelSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.labelSettings.stroke : this._scale.labelSettings.stroke);
                this._scale.featuredMeasureSettings.stroke = ((!this._scale.featuredMeasureSettings.stroke || this._scale.featuredMeasureSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.featuredMeasureSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.featuredMeasureSettings.stroke : this._scale.featuredMeasureSettings.stroke);
                this._scale.comparativeMeasureSettings.stroke = ((!this._scale.comparativeMeasureSettings.stroke || this._scale.comparativeMeasureSettings.stroke == jsonObj[result[i]].quantitativeScaleSettings.comparativeMeasureSettings.stroke) ? jsonObj[themeName].quantitativeScaleSettings.comparativeMeasureSettings.stroke : this._scale.comparativeMeasureSettings.stroke);
                this.model.captionSettings.font.color = ((!this.model.captionSettings.font.color || this.model.captionSettings.font.color == jsonObj[result[i]].captionSettings.font.color) ? jsonObj[themeName].captionSettings.font.color : this.model.captionSettings.font.color);
                this.model.captionSettings.subTitle.font.color = ((!this.model.captionSettings.subTitle.font.color || this.model.captionSettings.subTitle.font.color == jsonObj[result[i]].captionSettings.subTitle.font.color) ? jsonObj[themeName].captionSettings.subTitle.font.color : this.model.captionSettings.subTitle.font.color);
                this.model.captionSettings.indicator.font.color = ((!this.model.captionSettings.indicator.font.color || this.model.captionSettings.indicator.font.color == jsonObj[result[i]].captionSettings.indicator.font.color) ? jsonObj[themeName].captionSettings.indicator.font.color : this.model.captionSettings.indicator.font.color);
                this.model.captionSettings.indicator.symbol.color = ((!this.model.captionSettings.indicator.symbol.color || this.model.captionSettings.indicator.symbol.color == jsonObj[result[i]].captionSettings.indicator.symbol.color) ? jsonObj[themeName].captionSettings.indicator.symbol.color : this.model.captionSettings.indicator.symbol.color);
                for (var j = 0; j < this.model.qualitativeRanges.length; j++)
                    this.model.qualitativeRanges[j].rangeStroke = ((!this.model.qualitativeRanges[j].rangeStroke) ? (ej.isNullOrUndefined(jsonObj[themeName].qualitativeRanges[j]) ? jsonObj[themeName].qualitativeRanges[0].rangeStroke : jsonObj[themeName].qualitativeRanges[j].rangeStroke) : this.model.qualitativeRanges[j].rangeStroke);
            }
            //            this.model = $.extend(true, this.model, jsonObj[themeName], this.model);
        },

      
       
        _onDrawQualitativeRanges: function (rangeOptions, rangeEnd, rangeIndex) {
            var data = { object: this, scaleElement: this.model.quantitativeScaleSettings, rangeIndex: rangeIndex, rangeElement: rangeOptions, rangeEndValue: rangeEnd };
            this._trigger("drawQualitativeRanges", data);
        },
        
       
        _doAnimation: function () {
            var elements = $("." + this.svgObject.id + "_FeatureMeasure");
            for (var i = elements.length - 1; i >= 0; i--) {
                var element = elements[i];
                this._animateFeatureBar(element);
            }
        },
      
        _doLineAnimation: function () {
            var compareElements = $("." + this.svgObject.id + "_ComparativeMeasure");
            var secondsPerPoint = 2000 / compareElements.length;
            for (var i = compareElements.length - 1; i >= 0; i--) {
                var element = compareElements[i];
                $(element).attr("transform", "scale(0)");
                this._doLineSymbol(element, secondsPerPoint, i);
            }
        },
       
        _animateFeatureBar: function (element) {
            var box = element.getBBox();
            var left, top;
            if (((this._orientation == ej.datavisualization.BulletGraph.Orientation.Horizontal) && (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Backward)) || ((this._orientation == ej.datavisualization.BulletGraph.Orientation.Vertical) && (this._flowDirection == ej.datavisualization.BulletGraph.FlowDirection.Forward))) {
                left = box.x + box.width;
                top = box.y + box.height;
            }
            else {
                left = box.x;
                top = box.y;
            }
            $(element).animate(
            {
                scale: 1
            },
            {
                duration: 1000,

                step: function (now) {
                    scaleVal = now;
                    $(element).attr("transform", "translate(" + left + " " + top + ") scale(" + now + ",1) translate(" + (-left) + " " + (-top) + ")");
                }
            });
        },
      
        _doLineSymbol: function (element, sec, val) {
            var beginTime = parseInt(val * sec);
            var box = element.getBBox();
            var centerX = box.x + (box.width / 2);
            var centerY = box.y + (box.height / 2);
            $(element).delay(beginTime).animate(
                {
                    scale: 1
                },
                {
                    duration: 200,
                    step: function (now) {
                        $(element).attr("transform", "translate(" + centerX + " " + centerY + ") scale(" + now + ") translate(" + (-centerX) + " " + (-centerY) + ")");
                    }
                });
        },
      
        bindEvents: function () {
            if (this.model.tooltipSettings.visible)
                this._on($(this.svgObject), "mousemove", this._bulletMouseMove);
            if (this.model.enableResizing) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this._bulletResize);
                else
                    this._on($(window), "orientationchange", this._bulletResize);
            }
        },
       
        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
       
        _bulletResize: function () {
            var bullet = this;
            var $svgObj = $(bullet.svgObject);
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                if (!bullet.isDevice()) {
                    if (bullet.svgWidth > $(this).width()) {
                        $svgObj.width($(this).width());
                    } else {
                        $svgObj.width(bullet.svgWidth);
                    }
                    if (bullet.svgHeight > $(this).height()) {
                        $svgObj.height($(this).height());
                    } else {
                        $svgObj.height(bullet.svgHeight);
                    }

                }
                else {
                    $svgObj.width($(bullet.element).width());
                    $svgObj.height($(bullet.element).height());
                }
                bullet.redraw();
            }, 500);
        },
       
        _bulletMouseMove: function (evt) {
            var currentVal, targetVal, categoryVal, measureId, targetId, targetClass, tooltipdiv = $('.tooltipDiv');
            $('#tooltip').remove();
            $(".tooltipDiv").remove();
            targetId = $(evt.target).attr("id");
            targetClass = $(evt.target).attr("class");
            var captionSettings = this.model.captionSettings;
            var tooltipdiv = $("<div id = 'tooltip'></div>");
            var str = "";
            // adding css prop to the div
            $(tooltipdiv).css({
                "left": evt.pageX + 10,
                "top": evt.pageY + 10,
                "display": "block",
                "position": "absolute",
                "z-index": "13000",
                "cursor": "default",
                "font-family": "Segoe UI",
                "color": "#707070",
                "font-size": "12px",
                "background-color": "#FFFFFF",
                "border": "1px solid #707070"
            });
            if (targetClass == this.svgObject.id + "_Caption")
                str = evt.target.textContent == captionSettings.text ? "" : captionSettings.text;
            else if (targetClass == this.svgObject.id + "_SubTitle")
                str = evt.target.textContent == captionSettings.subTitle.text ? "" : captionSettings.subTitle.text;
            else if (targetClass == this.svgObject.id + "_Indicator")
                str = evt.target.textContent == captionSettings.indicator.text ? "" : captionSettings.indicator.text;
            if (str != "") {
                $(tooltipdiv).html("&nbsp" + str + "&nbsp");
                $(document.body).append(tooltipdiv);
            }
            else if (this.model.tooltipSettings.enableCaptionTooltip && (targetClass == this.svgObject.id + '_Caption' || targetClass == this.svgObject.id + '_SubTitle' || targetClass == this.svgObject.id + '_Indicator')) {
                var text = $('.' + targetClass).text();
                var data = { Text: text };
                var template = this.model.tooltipSettings.captionTemplate;
                if ($('.tooltipDiv').length == 0) {
                    tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 105; display: block;'></div>");
                    $(document.body).append(tooltipdiv);
                    $('.tooltipDiv').addClass(this.svgObject.id);
                }
                if (template != "" && template != null) {
                    var cloneNode = document.getElementById(template);
                    cloneNode = (cloneNode ? $(cloneNode).clone() : $(template));
                    $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                    $(tooltipdiv).html($(cloneNode).render(data));
                } else {
                    $(tooltipdiv).html(text);
                }
                $(tooltipdiv).css('font-size', '12px');
                var xPos = evt.clientX + $(document).scrollLeft() + 10;
                var yPos = evt.clientY + $(document).scrollTop() + 10;
                xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
                yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

                if (xPos === undefined || xPos === null)
                    xPos = evt.clientX + $(document).scrollLeft() + 10;
                if (yPos === undefined || yPos === null)
                    yPos = evt.clientY + $(document).scrollTop() + 10;
                $(tooltipdiv).css({
                    'left': xPos,
                    'top': yPos,
                    '-webkit-border-radius': "5px 5px 5px 5px",
                    '-moz-border-radius': "5px 5px 5px 5px",
                    '-o-border-radius': "5px 5px 5px 5px",
                    'border-radius': "5px 5px 5px 5px",
                    'background-color': "White",
                    'border': "1px Solid Black",
                    'padding-bottom': "5px",
                    'padding-left': "5px",
                    'padding-right': "5px",
                    'padding-top': "5px"
                });

            }
            if (targetClass != "undefined" && (targetClass == this.svgObject.id + '_FeatureMeasure' || targetClass == this.svgObject.id + '_ComparativeMeasure')) {
                var bulletValue = (typeof (this.model.value) == 'function') ? this._value() : this.model.value;
                var bulletCMValue = (typeof (this.model.comparativeMeasureValue) == 'function') ? this._comparativeMeasureValue() : this.model.comparativeMeasureValue;
                measureId = targetId.substring(targetId.lastIndexOf("_") + 1);
                currentVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.featureMeasures] : (ej.isNullOrUndefined(this._scale.featureMeasures[measureId].value) ? bulletValue : this._scale.featureMeasures[measureId].value);
                targetVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"][measureId][this.model.fields.comparativeMeasure] : (ej.isNullOrUndefined(this._scale.featureMeasures[measureId].comparativeMeasureValue) ? bulletCMValue : this._scale.featureMeasures[measureId].comparativeMeasureValue);
                categoryVal = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? (ej.isNullOrUndefined(this.model.fields["dataSource"][measureId][this.model.fields.category])) ? null : this.model.fields["dataSource"][measureId][this.model.fields.category] : (ej.isNullOrUndefined(this._scale.featureMeasures[measureId].category)) ? null : this._scale.featureMeasures[measureId].category;
                var data = { currentValue: currentVal, targetValue: targetVal, category: categoryVal };
                if ($('.tooltipDiv').length == 0) {
                    tooltipdiv = $("<div class='tooltipDiv' style='position: absolute; z-index: 105; display: block;'></div>");
                    $(document.body).append(tooltipdiv);
                    $('.tooltipDiv').addClass(this.svgObject.id);
                }
                if (this.model.tooltipSettings.template != "" && this.model.tooltipSettings.template != null) {
                    var cloneNode = $("#" + this.model.tooltipSettings.template).clone();
                    $(cloneNode).css("display", "block").appendTo(tooltipdiv);
                    $(tooltipdiv).html($(cloneNode).render(data));
                } else {
                    var category = (!ej.isNullOrUndefined(data.category)) ? ("<br/> Category : " + data.category) : "";
                    $(tooltipdiv).html("Current : " + data.currentValue + "<br/> Target : " + data.targetValue + category);
                }
                $(tooltipdiv).css("font-size", "12px");
                var xPos = evt.clientX + $(document).scrollLeft() + 10;
                var yPos = evt.clientY + $(document).scrollTop() + 10;
                xPos = ((xPos + $(tooltipdiv).width()) < $(window).width()) ? (xPos) : (xPos - $(tooltipdiv).width());
                yPos = ((yPos + $(tooltipdiv).height()) < $(window).height()) ? (yPos) : (yPos - $(tooltipdiv).height());

                if (xPos === undefined || xPos === null)
                    xPos = evt.clientX + $(document).scrollLeft() + 10;
                if (yPos === undefined || yPos === null)
                    yPos = evt.clientY + $(document).scrollTop() + 10;
                $(tooltipdiv).css({
                    'left': xPos,
                    'top': yPos,
                    '-webkit-border-radius': "5px 5px 5px 5px",
                    '-moz-border-radius': "5px 5px 5px 5px",
                    '-o-border-radius': "5px 5px 5px 5px",
                    'border-radius': "5px 5px 5px 5px",
                    'background-color': "White",
					'color':"black",
                    'border': "1px Solid Black",
                    'padding-bottom': "5px",
                    'padding-left': "5px",
                    'padding-right': "5px",
                    'padding-top': "5px"
                });
            }
        },

       
        redraw: function () {
            $(this.svgObject).empty();
            this._renderBulletElements();
        },
        
        destroy: function () {
            $(this.element).removeClass("e-bulletgraph e-js").find("#" + this.svgObject.id).remove();
        },
          
        setFeatureMeasureBarValue: function (index, measureValue) {
            this._scale = this.model.quantitativeScaleSettings;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasures.length;   
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.featureMeasures] = measureValue;
                else
                    this._scale.featureMeasures[index].value = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doAnimation();
            }
        },
          
        setComparativeMeasureSymbol: function (index, measureValue) {
            this._scale = this.model.quantitativeScaleSettings;
            var length = (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null) ? this.model.fields["dataSource"].length : this._scale.featureMeasures.length;   
            if (index < length && measureValue != null) {
                if (!ej.isNullOrUndefined(this.model.fields) && this.model.fields["dataSource"] != null)
                    this.model.fields["dataSource"][index][this.model.fields.comparativeMeasure] = measureValue;
                else
                    this._scale.featureMeasures[index].comparativeMeasureValue = measureValue;
                this.redraw();
                if (this.model.enableAnimation)
                    this._doLineAnimation();
            }
        }

    });
      
    ej.datavisualization.BulletGraph.Orientation = {
          
        Horizontal: "horizontal",
          
        Vertical: "vertical"
    };
      
    ej.datavisualization.BulletGraph.TickPlacement = {
          
        Inside: "inside",
          
        Outside: "outside"
    };
      
    ej.datavisualization.BulletGraph.LabelPlacement = {
          
        Inside: "inside",
          
        Outside: "outside"
    };
      
    ej.datavisualization.BulletGraph.Shape = {
          
        Circle: "circle",
          
        Cross: "cross",
          
        Diamond: "diamond",
          
        DownArrow: "downarrow",
          
        Ellipse: "ellipse",
          
        HorizontalLine: "horizontalLine",
          
        Image: "image",
          
        InvertedTriangle: "invertedtriangle",
          
        LeftArrow: "leftarrow",
          
        Pentagon: "pentagon",
          
        Rectangle: "Rectangle",
          
        RightArrow: "rightarrow",
          
        Star: "star",
          
        Trapezoid: "trapezoid",
          
        Triangle: "triangle",
          
        UpArrow: "uparrow",
          
        VerticalLine: "verticalline",
          
        Wedge: "wedge",
    };
      
    ej.datavisualization.BulletGraph.TickPosition = {
          
        Far: "far",
          
        Near: "near",
          
        Center: "center"
    };
      
    ej.datavisualization.BulletGraph.LabelPosition = {
          
        Below: "below",
          
        Above: "above"
    };
      
    ej.datavisualization.BulletGraph.FlowDirection = {
          
        Forward: "forward",
          
        Backward: "backward"
    };
      
    ej.datavisualization.BulletGraph.FontStyle = {
          
        Normal: "Normal",
          
        Italic: "Italic",
          
        Oblique: "Oblique"
    };
      
    ej.datavisualization.BulletGraph.FontWeight = {
          
        Normal: "Normal",
          
        Bold: "Bold",
          
        Bolder: "Bolder",
          
        Lighter: "Lighter"
    };
      
    ej.datavisualization.BulletGraph.Themes = {
          
        flatlight: {
            quantitativeScaleSettings: {
                majorTickSettings: { stroke: "#191919" },
                minorTickSettings: { stroke: "#191919" },
                labelSettings: {
                    stroke: "#191919"
                },
                featuredMeasureSettings: { stroke: "#191919" },
                comparativeMeasureSettings: { stroke: "#191919" }
            },
            qualitativeRanges: [{
                rangeStroke: "#ebebeb"
            }, {
                rangeStroke: "#d8d8d8"
            }, {
                rangeStroke: "#7f7f7f"
            }],
            captionSettings: {
                font: { color: "#191919" },
                subTitle: {
                    font: { color: "#191919" }
                },
                indicator: {
                    font: { color: "#191919" },
                    symbol: { color: "#191919" }
                }
            }
        },
          
        flatdark: {
              
            quantitativeScaleSettings: {
                majorTickSettings: { stroke: "#ffffff" },
                minorTickSettings: { stroke: "#ffffff" },
                labelSettings: {
                    stroke: "#ffffff"
                },
                featuredMeasureSettings: { stroke: "#ffffff" },
                comparativeMeasureSettings: { stroke: "#ffffff" }
            },
            qualitativeRanges: [{
                rangeStroke: "#b3b3b3"
            }, {
                rangeStroke: "#999999"
            }, {
                rangeStroke: "#4d4d4d"
            }],
            captionSettings: {
                font: { color: "#ffffff" },
                subTitle: {
                    font: { color: "#ffffff" }
                },
                indicator: {
                    font: { color: "#ffffff" },
                    symbol: { color: "#ffffff" }
                }
            }
        }
    };

})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html RangeNavigator elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    
    ej.widget("ejRangeNavigator", "ej.datavisualization.RangeNavigator", {

        ejChart: "",

        validTags: ["div"],
       
        defaults: {

            theme: "",

            padding: "0",

            enableAutoResizing: false,

            allowSnapping: false,


            sizeSettings: {

                width: "",

                height: ""
            },
            _size: {

            },

            locale: "en-US",

            valueType: "datetime",

            valueAxisSettings: {

                rangePadding: 'none',

                axisLine:
                    {

                        visible: false
                    },

                font: {

                    size: '0px',
                },

                majorTickLines:
                {

                    width: 0,

                    size: 0,

                    visible: true
                },

                majorGridLines: {

                    visible: false
                },

                visible: false
            },

            rangePadding: "none",

            enableRTL: false,

            dataSource: "",

            xName: "x",

            yName: "y",

            tooltipSettings: {

                visible: true,

                labelFormat: "MM/dd/yyyy",

                tooltipDisplayMode: "always",

                backgroundColor: "#303030",

                font:
                {

                    color: '#FFFFFF',

                    family: 'Segoe UI',

                    fontStyle: 'Normal',

                    size: '10px',
                    opacity: 1,

                    weight: 'regular'
                }
            },

            zoomPosition: "0",

            zoomFactor: "1",

            selectedRangeSettings: {

                start: "",

                end: ""
            },

            selectedData: "",

            rangeSettings: {

                start: "",

                end: ""
            },

            enableDeferredUpdate: true,

            series: "",

            seriesSettings: "",

            labelSettings: {

                style: {

                    font: {

                        color: '#333333',

                        family: 'Segoe UI',

                        style: 'Normal',

                        size: '10px',

                        opacity: 1,

                        weight: 'regular'
                    },

                    horizontalAlignment: "middle"
                },


                higherLevel: {

                    intervalType: 'years',

                    style: {

                        font: {

                            color: "black",

                            fontFamily: "Segoe UI",

                            fontStyle: "Normal",

                            size: "12px",

                            opacity: 1,

                            fontWeight: "regular"
                        },

                        horizontalAlignment: "middle"
                    },

                    gridLineStyle: {

                        color: "#B5B5B5",

                        width: 1,

                        dashArray: "20 5 0"
                    },


                    border:
                    {

                        color: "transparent",

                        width: 0.5
                    },

                    fill: "transparent",

                    position: "top",

                    visible: true,

                    labelPlacement: "outside"
                },

                lowerLevel: {

                    intervalType: 'quarters',

                    style: {

                        font: {

                            color: "black",

                            fontFamily: "Segoe UI",

                            fontStyle: "Normal",

                            size: "12px",

                            opacity: 1,

                            fontWeight: "regular"
                        },

                        horizontalAlignment: "middle"
                    },

                    gridLineStyle:
                    {
                        color: "#B5B5B5",

                        width: 1,

                        dashArray: ""
                    },

                    border:
                    {

                        color: "transparent",

                        width: 0.5
                    },

                    fill: "transparent",

                    position: "bottom",

                    visible: true,

                    labelPlacement: "outside"
                }
            },

            navigatorStyleSettings: {

                selectedRegionColor: "#EFEFEF",

                unselectedRegionColor: "#5EABDE",

                thumbColor: "#2382C3",

                thumbRadius: 10,

                leftThumbTemplate: null,

                rightThumbTemplate: null,

                thumbStroke: "#303030",

                background: "#dddddd",

                border:
                {

                    color: "transparent",

                    width: 0.5,

                    dashArray: ""
                },

                opacity: 1,

                selectedRegionOpacity: 0,

                unselectedRegionOpacity: 0.3,

                majorGridLineStyle:
                {

                    color: "#B5B5B5",

                    visible: true
                },

                minorGridLineStyle:
                {

                    color: "#B5B5B5",

                    visible: true
                },
            },


            loaded: "",

            load: "",

            rangeChanged: "",

            selectedRangeStart: "",
            selectedRangeEnd: ""
        },

        dataTypes: {
        
            enableAutoResizing:"boolean",
            allowSnapping:"boolean",
         
            dataSource: "data",
            tooltipSettings: {
                visible:"boolean",
                labelFormat:"string",
                tooltipDisplayMode:"string",
            },
            
            zoomPosition: "string",
            zoomFactor: "string",
           
            selectedData: "string",
            enableDeferredUpdate: "boolean",
            series: "array",
           
            labelSettings: {
                higherLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement: "string"
                },
                lowerLevel: {
                    intervalType: 'string',
                    position: "string",
                    visible: "boolean",
                    labelPlacement:"string"
                }
            },
            navigatorStyleSettings: {
                thumbRadius:"number",
            },
       
        },

        observables: ["selectedRangeStart", "selectedRangeEnd"],

         _tags: [{
                tag: "series", 
                attr: ["xName", "yName", "dataSource", "enableAnimation",
				[
				     {
				       tag: "points", attr:["x", "y", "text", "isEmpty", "fill", "visible"]
				     }
				 ]				
				],
				 singular:"series"
				                
            } 
         ],

          _selectedRangeStart: ej.util.valueFunction("selectedRangeStart"),
          _selectedRangeEnd: ej.util.valueFunction("selectedRangeEnd"),

		 _series: function (index, property, value, old) {		   
		  this.renderNavigator();
		  this._trigger("refresh");           
        },
		_series_points:function(index, property, value, old){		   
		  this.renderNavigator();
		  this._trigger("refresh");    
		},


        _themes: {
            flatlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1 ,weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {                        
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0},
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width:1, dashArray:"15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width:0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity:1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity:0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            azurelight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1,dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            limelight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#A9CA44 ",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            saffronlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#e5e5e5",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlight: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientazure: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientlime: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI',style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            gradientsaffron: {
                tooltipSettings: {
                    backgroundColor: "#303030",
                    font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#B5B5B5", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#bbbbbb", colorStop: "0%" }, { color: "#efefef", colorStop: "15%" }, { color: "#bbbbbb", colorStop: "85%" }, { color: "#efefef", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#303030",
                    background: "#EFEFEF",
                    border: { color: "#606262", width: 0 },
                    opacity: 1,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    majorGridLineStyle: { color: "#8c8c8c", visible: true },
                    minorGridLineStyle: { color: "#8c8c8c", visible: true },
                },
            },

            flatdark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {                        
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width:0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            azuredark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#5EABDE",
                    thumbColor: "#2382C3",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            limedark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#A9CA44",
                    thumbColor: "#AECF49",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },
            
            saffrondark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI',style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: "#353635",
                    unselectedRegionColor: "#FAA113",
                    thumbColor: "#F9920B",
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientdark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI',style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientazuredark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbColor: [{ color: "#67C1DC", colorStop: "50%" }, { color: "#3D93AA", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientlimedark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI', style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', fontStyle: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1,weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbColor: [{ color: "#A5C14A", colorStop: "50%" }, { color: "#738B1F", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

            gradientsaffrondark: {
                tooltipSettings: {
                    backgroundColor: "#FFFFFF",
                    font: { color: '#1E1E1E', family: 'Segoe UI',style: 'Normal', size: '10px', opacity: 1, weight: 'regular' },
                },
                labelSettings: {
                    style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                    higherLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' }, horizontalAlignment: "left" },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "#FFFFFF", width: 1, dashArray: "15 5 0" },
                    },
                    lowerLevel: {
                        style: { font: { color: '#FFFFFF', family: 'Segoe UI', style: 'Regular', size: '13px', opacity: 1, weight: 'regular' } },
                        border: { color: "transparent", width: 0 },
                        fill: "transparent",
                        gridLineStyle: { color: "transparent", width: 0 },
                    },
                },
                navigatorStyleSettings: {
                    selectedRegionColor: [{ color: "#0a0a0a", colorStop: "0%" }, { color: "#282828", colorStop: "15%" }, { color: "#282828", colorStop: "85%" }, { color: "#0a0a0a", colorStop: "100%" }],
                    unselectedRegionColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbColor: [{ color: "#FEB75B", colorStop: "50%" }, { color: "#ED7E16", colorStop: "100%" }],
                    thumbRadius: 10,
                    thumbStroke: "#FFFFFF",
                    background: "#FFFFFF",
                    opacity: 0.08,
                    unselectedRegionOpacity: 0.3,
                    selectedRegionOpacity: 0,
                    border: { color: "#FFFFFF", width: 0 },
                    majorGridLineStyle: { color: "#FFFFFF", visible: true },
                    minorGridLineStyle: { color: "#FFFFFF", visible: true },
                },
            },

        },
        /**
                * To configure the properties at runtime using SetModel		
                * @private
                */
        _setModel: function (options) {
            for (var prop in options) {
              //  this.disableAnimation();
                switch (prop) {                   
                    case "theme":
                        this.model._themeChanged = true;
                        this.model.theme = options[prop];
                        this._setTheme(this._themes, this.model.theme);
                    case "selectedRangeStart":
                        this.model.selectedRangeSettings.start = this._selectedRangeStart();
                        break;
                    case "selectedRangeEnd":
                        this.model.selectedRangeSettings.end = this._selectedRangeEnd();
                        break;
                    default:
                        $.extend(true, this.model, {}, options[prop]);
                }
            }
          //  $(this.svgDocument).empty();		  
            this.bindTo();
			this.renderNavigator(this);
			this._bindevents();
        },
     
        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },


        _createSvg: function (element) {

            this.svgSupport = (window.SVGSVGElement) ? true : false;
            var bwr = jQuery.uaMatch(navigator.userAgent);
            this.brow = bwr.browser.toLowerCase();
            if (this.svgSupport) {
                //rendering object created
                this.renderer = new ej.EjSvgRender(this.element);
                //Creating svg document
                this.svgns = this.renderer.svgLink;
                this.svgDocument = this.renderer.svgObj;
            }
            else {
                //rendering object created
                this.renderer = new ej.EjVmlRender(this.element);
                //Creating VML document
                this.svgDocument = this.renderer.svgObj;
            }

        },

        _setTheme: function (jsonObj, themeName) {
            var name = themeName.toLowerCase();
            this.model = ej.copyObject({}, this.model, jsonObj[name]);
        },

        _setPositions: function () {

            var higherposition = this.model.labelSettings.higherLevel.position;
            var lowerposition = this.model.labelSettings.lowerLevel.position;
            var highervisible = this.model.labelSettings.higherLevel.visible;
            var lowervisible = this.model.labelSettings.lowerLevel.visible;
            var higherlabelplace = this.model.labelSettings.higherLevel.labelPlacement;
            var lowerlabelplace = this.model.labelSettings.lowerLevel.labelPlacement;
            //to initialize the positions of the sliders, labelbars and chart
            if (higherposition === "top" && lowerposition === "bottom") {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;                    
            }
            else if ((higherposition === "bottom" && lowerposition === "bottom")) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;

            }
            else if (higherposition === "bottom" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + 1;
                topPosition = 0;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.minLowHeight + this.centerHeight + 1;
            }
            else if (higherposition === "top" && lowerposition === "top") {
                this.centerPosition = this.minLowHeight + this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }

            if (highervisible === false && lowervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight;
                this.bottomPostion = this.newHeight;
            }
            else if (highervisible === false) {
                this.centerPosition = 0;
                this.centerHeight = this.newHeight - this.minLowHeight;
                this.bottomPosition = this.newHeight;
            }
            else if (lowervisible === false) {
                this.centerPosition = this.minHighHeight + 1;
                this.centerHeight = this.newHeight - this.minHighHeight;
                this.bottomPosition = this.minHighHeight + this.centerHeight + 1;
            }

            if (higherlabelplace === "outside" && lowerlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.centerHeight;
            }
            else if (higherlabelplace === "outside") {
                this.sliderPosition = this.centerPosition;
                this.sliderHeight = this.newHeight;
            }
            else if (lowerlabelplace === "outside") {
                this.sliderPosition = 0;
                this.sliderHeight = this.centerHeight + this.minHighHeight;
            }
            else {
                this.sliderPosition = 0;
                this.sliderHeight = this.newHeight;
            }
            
        },

        //adding redraw method

        redraw: function (evt) {
            this.renderNavigator(this);
        },

        renderNavigator: function (evt) {
            $(this.svgDocument).empty();
		    this.culture = Globalize.culture(this.model.locale);
            this._initializeVariables.call(this, this.model, this.model.series, this.model.seriesSettings);

            this._rootId = $(this.element).attr("id");
            this.leftPadding =parseFloat($("#"+this._rootId).css('padding-left'));

            //initializing minimum height and width for the control
            var fontsize = parseFloat(this.model.labelSettings.higherLevel.style.font.size);
            var type = this.model.valueType;
            if (type == "datetime") {
                this.minHighHeight = fontsize < 20 ?
                    20 :
                    fontsize + 2;
                fontsize = parseFloat(this.model.labelSettings.lowerLevel.style.font.size);
            }
            else
                this.minHighHeight = 0;
            this.minLowHeight = fontsize < 20 ?
                20 :
                fontsize + 2;
            if (type == "datetime") {
                var ht = ($(this.element).height() && this.newHeight==undefined) ? $(this.element).height() :this.newHeight==undefined?120:this.newHeight;
                this.minCenterHeight = parseFloat(ht) - parseFloat(this.minLowHeight) - parseFloat(this.minHighHeight);
            } else {
                var ht = ($(this.element).height()) && this.newHeight==undefined ? $(this.element).height() -this.minLowHeight : this.newHeight==undefined?100:this.newHeight;
                this.minCenterHeight = parseFloat(ht) - parseFloat(this.minLowHeight) - parseFloat(this.minHighHeight);
            }
            this.minHeight = this.minHighHeight + this.minCenterHeight + this.minLowHeight;
            this.padding = parseFloat(this.model.padding);
            
            
            //calculating the height and width of the control
            this.newWidth = this.model.sizeSettings.width ? this.model.sizeSettings.width : $(this.element).width();
            this.newHeight = this.model.sizeSettings.height ? this.model.sizeSettings.height : ($(this.element).height() && this.newHeight==undefined) > this.minHeight ? $(this.element).height() : this.minHeight;
            
            
            //initializing the higherlevel and lowerlevel collections
            this._higherTextNode = [];
            this._higherTotalValues = [];
            this._higherLineLeft = [];
            this._higherTextLeft = [];
            this._lowerTextNode = [];
            this._lowerTotalValues = [];
            this._lowerLineLeft = [];
            this._lowerTextLeft = [];

            //calculating positions
            this.centerPosition=0;
            this.bottomPosition = 0;
            this.centerHeight = 0;
            this.sliderPosition = 0;
            this.sliderHeight = 0;
            
            //set positions
            this._setPositions();

            //set initial width and height for the svg document
            $(this.svgDocument).empty();
            this.svgDocument.setAttribute('style', 'overflow:visible;position:relative');
            if (this.svgSupport) {
                this.svgDocument.setAttribute("height", this.newHeight);
                this.svgDocument.setAttribute("width", this.newWidth);
            }
            else {
                $(this.svgDocument).width(this.newWidth);
                $(this.svgDocument).height(this.newHeight);
            }
            this.model._size.width = this.newWidth;
            this.model._size.height = this.newHeight;
            var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.data = { model: this.model };
            this._trigger("loaded", commonLoadedEventArgs);
            if (this.model.theme != "")
                this._setTheme(this._themes, this.model.theme);
            //create gradient
            var navigatorStyleSettings = this.model.navigatorStyleSettings;
            var uncolor = navigatorStyleSettings.unselectedRegionColor;
            this.unselectgrad = this.renderer.createGradientElement('unselected', uncolor, 150, 0, 150, 100, this.svgDocument);
            var navibackcolor = navigatorStyleSettings.background;
            this.naviback = this.renderer.createGradientElement('naviback', navibackcolor, 150, 0, 150, 100, this.svgDocument);
            var selcolor = navigatorStyleSettings.selectedRegionColor;
            this.selectedgrad = this.renderer.createGradientElement('selected', selcolor, 150, 0, 150, 100, this.svgDocument);
            var tbColor = navigatorStyleSettings.thumbColor;
            this.tbClr = this.renderer.createGradientElement('thumbClr', tbColor, 150, 0, 150, 100, this.svgDocument);
            
            var areaHeight;
            if (type == "numeric")
                areaHeight = this.newHeight - navigatorStyleSettings.border.width - this.minLowHeight;
            else
                areaHeight = this.newHeight - navigatorStyleSettings.border.width - this.minLowHeight - this.minHighHeight;
            //Creating border for navigator
            var naviBorderOptions = {
                'id':this._rootId+'_Border',
                'x':this.padding,
                'y': this.sliderPosition,
                'width': this.newWidth >0 ? this.newWidth-this.padding * 2: 0,
                'height': this.sliderHeight,
                'fill': this.naviback,
                'stroke-width': navigatorStyleSettings.border.width,
                'stroke': navigatorStyleSettings.border.color,
                'opacity': navigatorStyleSettings.opacity,
                'stroke-dasharray': navigatorStyleSettings.border.dashArray,
                
            }

            this.renderer.drawRect(naviBorderOptions, this.svgDocument);

            //creating g tags to group higherlevel and lowerlevel elements(txt,line).
            if ($("#" + this._rootId + 'higherLevel').length == 0)
                this.higherLevel = this.renderer.createGroup({ 'id': this._rootId + 'higherLevel' });
            if ($("#" + this._rootId + 'lowerLevel').length == 0)
                this.lowerLevel = this.renderer.createGroup({ 'id': this._rootId + 'lowerLevel' });
            this.centerLevel = this.renderer.createGroup({ 'id': this._rootId + 'centerLelvel' });
            this.higherLevel.height = this.minHighHeight;
            this.lowerLevel.height = this.minLowHeight;

            //creating g tags to group tooltipSettings
            if (this.model.tooltipSettings.visible && this.svgSupport) {
                this.leftTooltip = this.renderer.createGroup({ 'id': this._rootId + 'leftTooltip' });
                this.rightTooltip = this.renderer.createGroup({ 'id': this._rootId + 'rightTooltip' });

                var leftToolOptions = {
                    'fill': this.model.tooltipSettings.backgroundColor,
                    'd': 'M 0 0 L 0 21 L 64 21 L 64 6 L 70 0 Z'
                };
                this.renderer.drawPath(leftToolOptions, this.leftTooltip);
                var tooltipoptions = this.model.tooltipSettings.font;
                

                var textOptions = {
                    'id': this._rootId + '_LeftToolText',
                    'x': 5,
                    'fill': tooltipoptions.color,
                    'font-size': tooltipoptions.size,
                    'font-family': tooltipoptions.family,
                    'font-style': tooltipoptions.style,
                    'font-weight': tooltipoptions.weight,
                    'text-anchor': 'start',
                    'opacity': tooltipoptions.opacity,
                    'dominant-baseline': 'middle',
                    
                    
                };
                this.renderer.drawText(textOptions, Globalize.format(new Date(1 / 1 / 2000), "MMMM, yyyy"), this.leftTooltip);
                this.leftTxt = this.leftTooltip.childNodes[1];
                var rightToolOptions = {
                    'fill': this.model.tooltipSettings.backgroundColor,
                    'd': 'M 0 0 L 70 0 L 70 21 L 6 21 L 6 6 Z',
                    'horizontal-alignment': 'stretch'
                };
                this.renderer.drawPath(rightToolOptions, this.rightTooltip);
                this.renderer.drawText(textOptions, Globalize.format(new Date(1 / 1 / 2000), "MMMM, yyyy"), this.rightTooltip);
                this.rightTxt = this.rightTooltip.childNodes[1];

             
            }
            //creating g tag for holding chart
            this.chartGView = this.renderer.createGroup({ 'id': this._rootId + 'charView' });
            
            this.renderer.append(this.chartGView, this.svgDocument);
            if(this.model.valueType=="datetime")
            this.renderer.append(this.higherLevel, this.svgDocument);
            this.renderer.append(this.lowerLevel, this.svgDocument);
            this.renderer.append(this.centerLevel, this.svgDocument);
            
            //creating g tag for sliders
            this.leftUnSelected=this.renderer.createGroup({'id':'unselectleft'});
            this.gLeftSlider = this.renderer.createGroup({ 'id': 'leftslider' });
            this.gRightSlider = this.renderer.createGroup({ 'id': 'rightslider' });
            this.gCenterSlider = this.renderer.createGroup({ 'id': 'centerslider' });
            this.rightUnSelected = this.renderer.createGroup({ 'id': 'unselectright' });

            var unLeftSliderOptions = {
                'width': this.newWidth > 0 ? this.newWidth - 4 - this.padding * 2 : 0,
                'height': this.sliderHeight,
                'fill':this.unselectgrad,
                'opacity': navigatorStyleSettings.unselectedRegionOpacity,
                'transform': 'translate('+this.padding+',' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unLeftSliderOptions, this.leftUnSelected);
            this.leftUnArea = this.leftUnSelected.firstChild;
            this.renderer.append(this.leftUnSelected, this.svgDocument);

            var centerSliderOptions = {
                'width': this.newWidth > 0 ? this.newWidth - 4 - this.padding * 2 : 0,
                'height': this.sliderHeight,
                'fill': this.selectedgrad,
                'opacity': navigatorStyleSettings.selectedRegionOpacity,
                'transform': 'translate('+this.padding+','+this.sliderPosition+')'
            };

            this.renderer.drawRect(centerSliderOptions, this.gCenterSlider);
            this.centerSlider = this.gCenterSlider.firstChild;
            $(this.centerSlider).css('cursor', 'pointer');

            this.renderer.append(this.gCenterSlider, this.svgDocument);
            var leftSliderOptions = {                 
                'width': '2',
                'height': this.sliderHeight,
                'fill': navigatorStyleSettings.thumbStroke,
                'opacity': '1',
                'y': this.sliderPosition,
                'transform': 'translate(' + this.padding + ')'
               
                
            };
            var lcircleOptions = {
                'id': 'leftCircle' + this._id,
                'cx': this.padding,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': navigatorStyleSettings.thumbRadius,
                'stroke-width': 2,
                'stroke': navigatorStyleSettings.thumbStroke,
                'fill': this.tbClr
            };

                
            this.renderer.drawRect(leftSliderOptions, this.gLeftSlider);
			// checking condition for whether using thumb
            if (this.model.navigatorStyleSettings.leftThumbTemplate == null) {
                this.renderer.drawCircle(lcircleOptions, this.gLeftSlider);
                this.leftCircle = this.gLeftSlider.lastChild;
                $(this.leftCircle).css('cursor', 'pointer');
            }

            // Adding left Thumb  

            else {
                var isID = document.getElementById(this.model.navigatorStyleSettings.leftThumbTemplate) ? true : false;
                if ($('#' + this._id + "thumbleft").length > 0)
                    $('#' + this._id + "thumbleft").remove();
                var container = this._id;
				if (($("#" + container).css('transform')) == 'none')
                    $("#" + container).css('transform', 'translate(0, 0)');
                if ($("#" + container+"template_group_").length != 0)
                    var templateContainer = $("#" + container + "template_group_");
				else
				{
                   var templateContainer = $("<div></div>").attr('id',container+"template_group_");
				    $(templateContainer).css("position", "relative");
				}
                var temp = $("<div></div>").attr("id", this._id + "thumbleft");
                var cloneNode;
                if (isID == true)
                    cloneNode = $("#" + this.model.navigatorStyleSettings.leftThumbTemplate)[0].innerHTML;
                else
                    cloneNode = this.model.navigatorStyleSettings.leftThumbTemplate;
                var $cloneNode = $(cloneNode);
                $(temp).css("position", "absolute");
                $cloneNode.appendTo($(temp));
                $(temp).appendTo(templateContainer);
                if (this.svgSupport) 
					$(templateContainer).appendTo('#' + container);
                else
                    $(templateContainer).appendTo(this.svgDocument);
            	var top = this.sliderPosition + (this.sliderHeight / 2) - (($cloneNode.height()) / 2);
				$(temp).css("top", top).css('cursor', 'pointer').css('z-index','1');
			}
            this.leftSlider = this.gLeftSlider.firstChild;
            $(this.leftSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gLeftSlider, this.svgDocument);

			var width = parseFloat(this.newWidth - 4 - this.padding*2); // To check width on initialize the range navigator
			width = width < 0 ? 0 : width;
			
            var unRightSliderOptions = {
                'width': width,
                'height': this.sliderHeight,
                'fill': this.unselectgrad,
                'opacity': navigatorStyleSettings.unselectedRegionOpacity,
                'transform': 'translate('+this.padding+',' + this.sliderPosition + ')'
            };
            this.renderer.drawRect(unRightSliderOptions, this.rightUnSelected);
            this.rightUnArea = this.rightUnSelected.firstChild;
            this.renderer.append(this.rightUnSelected, this.svgDocument);
            var rightSliderOptions = {               
                'width': '2',
                'height': this.sliderHeight,
                'fill': this.model.navigatorStyleSettings.thumbStroke,
                'opacity': '1',
                'transform': 'translate(' + parseFloat(this.newWidth - 4-this.padding) + ', ' + this.sliderPosition + ' )'
            };
            var rcircleOptions = {
                'id': "rightCircle" + this._id,
                'cx': this.newWidth - 4 - this.padding,
                'cy': this.sliderPosition + (this.sliderHeight / 2),
                'r': this.model.navigatorStyleSettings.thumbRadius,
                'stroke-width': 2,
                'stroke': this.model.navigatorStyleSettings.thumbStroke,
                'fill': this.tbClr
            };
            this.renderer.drawRect(rightSliderOptions, this.gRightSlider);
            if (this.model.navigatorStyleSettings.rightThumbTemplate == null)
            {
                this.renderer.drawCircle(rcircleOptions, this.gRightSlider);
                this.rightCircle = this.gRightSlider.lastChild;
                $(this.rightCircle).css('cursor', 'pointer');
            }

            // append right thumb   

            else {
                var isID = document.getElementById(this.model.navigatorStyleSettings.rightThumbTemplate) ? true : false;
                if ($('#' + this._id + "thumbright").length > 0)
                    $('#' + this._id + "thumbright").remove();
                var container = this._id;
                if ($("#" + container + "template_group_").length != 0)
                    var templateContainer = $("#" + container + "template_group_");
                else
				 {
                    var templateContainer = $("<div></div>").attr('id',container+ "template_group_");
					 $(templateContainer).css("position", "absolute");
				 }
                var temp = $("<div></div>").attr("id", this._id + "thumbright");
                var cloneNode;
                if (isID == true)
                    cloneNode = $("#" + this.model.navigatorStyleSettings.rightThumbTemplate)[0].innerHTML;
                else
                    cloneNode = this.model.navigatorStyleSettings.rightThumbTemplate;
                var $cloneNode = $(cloneNode);
                $(temp).css("position", "absolute");
                $cloneNode.appendTo($(temp));
                $(temp).appendTo(templateContainer);
                if (this.svgSupport) 
					$(templateContainer).appendTo('#' + container);
                else
                    $(templateContainer).appendTo(this.svgDocument);
                var top = this.sliderPosition + (this.sliderHeight / 2) - (($cloneNode.height()) / 2);
                $(temp).css("top", top).css('cursor', 'pointer').css('z-index', '1');
             }

            this.rightSlider = this.gRightSlider.firstChild;
            $(this.rightSlider).css('cursor', 'w-resize');

            this.renderer.append(this.gRightSlider, this.svgDocument);
            if (this.model.tooltipSettings.visible && this.svgSupport) {
                this.renderer.append(this.leftTooltip, this.svgDocument);
                this.renderer.append(this.rightTooltip, this.svgDocument);
            }
            
            this.element.append(this.svgDocument);
            if (this.svgSupport) {
                this.trueCoords = this.svgDocument.createSVGPoint();

                this.grabPoint = this.svgDocument.createSVGPoint();
            }

            this.backDrop = this.element;
            this.dragTarget = null;            
            if(this.sliderHeight > 0)
                 this._renderChart.call(this, this.model);
            if (evt != undefined && evt.target != undefined)
                this.resize = true;
            
            if ((this.startDateTime != undefined && this.endDateTime != undefined) || (this.startValue != undefined && this.endValue != undefined)) {
                this.calculateInterval.call(this, this.model.labelSettings.higherLevel, this.model.labelSettings.lowerLevel);

                this.setSliders();
            } else {
                if (!this.svgSupport)
                    this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                else
                    this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + this.padding + ',' + 0 + ')' });

                this.leftTooltip.setAttribute('opacity', 0);
                this.rightTooltip.setAttribute('opacity', 0);
            }
            if (this.model.tooltipSettings.visible && this.svgSupport && this.model.tooltipSettings.tooltipDisplayMode == "ondemand") {
                this.rightTooltip.setAttribute('opacity', 0);
                this.leftTooltip.setAttribute('opacity', 0);
            }
            //this._trigger("loaded", this.model);
        },

        eachInterval:"",

        //Rendering chart
        _renderChart: function (model) {
            if (model.dataSource != "" && model.series == "" && model.seriesSettings == "") {
                $(this.chartGView).ejChart(
                {
				    locale : this.model.locale,
                    border: { width: 0, color: "transparent" },
                    margin: { left: 0, right: 0, top: 0, bottom: 0 },
                    elementSpacing: 0,
                    chartArea:
                    {
                        background: "transparent",
                        border: { width: 0 }
                    },
                    primaryXAxis: {
                        axisLine: { visible: false },
                        font: { size: '0px', },
                        majorTickLines:
                        {
                            lineWidth: 0,
                            size: 0,
                            visible: true
                        },
                        majorGridLines: { visible: false },
                        rangePadding: this.model.rangePadding,
                        visible: false,
                        isInversed: this.model.enableRTL

                    },
                    primaryYAxis: this.model.valueAxisSettings,
                 series: [{
                     dataSource: model.dataSource, xName: model.xName, yName: model.yName,
                     type: 'line',
                     width:1.5,
                     enableAnimation:false

                 },
                 ],
                 size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                 legend: {
                     visible: false,
                     itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                 }
             });
            }
            else if (model.dataSource != "" && model.series != "" && model.series[0].dataSource == undefined || model.seriesSettings != "" && model.seriesSettings.dataSource == undefined && model.dataSource!="") {
                model.seriesSettings.dataSource = model.dataSource;
                model.seriesSettings.xName = model.seriesSettings.xName ? model.seriesSettings.xName : model.xName;
                model.seriesSettings.yName = model.seriesSettings.yName ? model.seriesSettings.yName : model.yName;
                var series = model.series;
                if (series == "") {
                    series = [{
                        dataSource: model.dataSource, xName: model.seriesSettings.xName, yName: model.seriesSettings.yName,
                        type: 'line',
                        width: 1.5,
                        enableAnimation: false

                    }];
                }
               
                $(this.chartGView).ejChart(
            {
			    locale : this.model.locale,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    font: { size: '0px', },
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.enableRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }
            else if (model.series != "" && model.series[0].dataSource != undefined || model.seriesSettings != "" && model.seriesSettings.dataSource != undefined) {
                if (model.seriesSettings == "") {
                    model.seriesSettings = {
                        type: 'line'
                    };
                }
                var series = model.series;
                var length = series.length;
                if (series == "") {
                    series = [{
                        dataSource: model.seriesSettings.dataSource, xName: model.seriesSettings.xName, yName: model.seriesSettings.yName,
                        type: 'line',
                        width: 1.5,
                        enableAnimation: false

                    }];
                }
                
                $(this.chartGView).ejChart(
            {
			    locale : this.model.locale,
                border: { width: 0, color: "transparent" },
                margin: { left: 0, right: 0, top: 0, bottom: 0 },
                elementSpacing: 0,
                chartArea:
                {
                    background: "transparent",
                    border: { width: 0 }
                },
                primaryXAxis: {

                    axisLine: { visible: false },
                    font: { size: '0px', },
                    majorTickLines:
                    {
                        lineWidth: 0, size: 0, visible: true
                    },
                    majorGridLines: { visible: false },
                    rangePadding: this.model.rangePadding,
                    visible: false,
                    isInversed: this.model.enableRTL

                },
                primaryYAxis: this.model.valueAxisSettings,
                series: series,
                commonSeriesOptions: model.seriesSettings,
                size: { height: parseFloat(this.sliderHeight).toString(), width: parseFloat(this.newWidth - (2 * this.padding)).toString(), padding: -10 },
                legend: {
                    visible: false,
                    itemSize: { height: 0, width: 0, borderColor: 'transparent', borderWidth: 0 },
                }
            });
            }

            this.ejChart = $(this.chartGView).data("ejChart");
            this.renderer._setAttr($(this.chartGView), { 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')','height': this.sliderHeight });
           
        },

        _init: function () {
            this._createSvg(this);
			var commonLoadedEventArgs = $.extend({}, ej.EjSvgRender.commonChartEventArgs);
            commonLoadedEventArgs.data = { model: this.model };
            this._trigger("load", commonLoadedEventArgs);
            this.renderNavigator(this);
			if (this.model.enableAutoResizing)
                this._on($(window), "resize", this.rangeChange);           
             
            this._bindevents();
        },
		
		 rangeChange:function(evt){
             var id;
			 this.rangeResize = true;
             clearTimeout(id);       
             if(!ej.isNullOrUndefined(window.orientation))                   
                id = setTimeout(this.renderNavigator(), 500);
		     else
			    this.renderNavigator();
				
          },

        isDevice: function () {
            // return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
            // comment above line temporary. Due to the below code event wont bind for tablet device
            return (/mobile|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },

		//To check whether the device is Windows
        isWindows: function () {
          if (!ej.getBooleanVal($('head'), 'data-ej-android') && !ej.getBooleanVal($('head'), 'data-ej-ios') && !ej.getBooleanVal($('head'), 'data-ej-ios7') && !ej.getBooleanVal($('head'), 'data-ej-flat'))
            return this._windows();
        },
		
		 _windows: function () {
          return (/trident|windows phone/i.test(navigator.userAgent.toLowerCase())) || (ej.getBooleanVal($('head'), 'data-ej-windows', false) === true);
        },
		
        _bindevents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (window.navigator.msPointerEnabled && !isIE11) {
                this._on($("#" +this._id + "template_group_"), "MSPointerDown", this._grab);
                this._on($("#" +this._id + "template_group_"), "MSPointerMove", this._drag);
                this._on($("#" +this._id + "template_group_"), "MSPointerUp", this._drop);
                this._on($(this.svgDocument), "MSPointerDown", this._grab);
                this._on($(this.svgDocument), "MSPointerMove", this._drag);
                this._on($(this.svgDocument), "MSPointerUp", this._drop);
                this._on($(window), "MSPointerUp", this.mouseup);
                $(this.svgDocument).css('-ms-touch-action', 'none');
            }
            else if (this.isDevice()) {
                 var isSafari = (navigator.userAgent.indexOf('Safari') != -1 && navigator.userAgent.indexOf('Chrome') == -1); 
				  var isInIframe = (window.location != window.parent.location) ? true : false;
                 var userAgent = isInIframe? window.parent.navigator.userAgent.toLowerCase():window.navigator.userAgent.toLowerCase();
                 var device = /mobile|tablet|android|kindle/i.test(userAgent);
                 if((ej.isMobile() && this.isWindows()) || (!device && ej.isMobile() && isSafari)){ // for safari mobile browser and windows phone
                    this._on($("#" + this._id + "template_group_"), "mousedown", this._grab);
                    this._on($("#" + this._id + "template_group_"), "mousemove", this._drag);
                    this._on($("#" + this._id + "template_group_"), "mouseup", this._drop);
                    this._on($(this.svgDocument), "mousedown", this._grab);
                    this._on($(this.svgDocument), "mousemove", this._drag);
                    this._on($(this.svgDocument), "mouseup", this._drop);
                    $(this.svgDocument).css('-ms-touch-action', 'none');
                } else {
                    this._on($("#" + this._id + "template_group_"), "touchstart", this._grab);
                    this._on($("#" + this._id + "template_group_"), "touchmove", this._drag);
                    this._on($("#" + this._id + "template_group_"), "touchend", this._drop);
                    this._on($(this.svgDocument), "touchstart", this._grab);
                    this._on($(this.svgDocument), "touchmove", this._drag);
                    this._on($(this.svgDocument), "touchend", this._drop);
                }
            }
            else if (matched.browser.toLowerCase() == "chrome") {
                this._on($("#" + this._id + "template_group_"), "touchstart", this._grab);
                this._on($("#" + this._id + "template_group_"), "touchmove", this._drag);
                this._on($("#" + this._id + "template_group_"), "touchend", this._drop);
                this._on($("#" + this._id + "template_group_"), "mousedown", this._grab);
                this._on($("#" + this._id + "template_group_"), "mousemove", this._drag);
                this._on($("#" + this._id + "template_group_"), "mouseup", this._drop);
                this._on($(this.svgDocument), "touchstart", this._grab);
                this._on($(this.svgDocument), "touchmove", this._drag);
                this._on($(this.svgDocument), "touchend", this._drop);
                this._on($(this.svgDocument), "mousedown", this._grab);
                this._on($(this.svgDocument), "mousemove", this._drag);
                this._on($(this.svgDocument), "mouseup", this._drop);
            }
            else {
                this._on($("#" + this._id + "template_group_"), "mousedown", this._grab);
                this._on($("#" + this._id + "template_group_"), "mousemove", this._drag);
                this._on($("#" + this._id + "template_group_"), "mouseup", this._drop);
                this._on($(this.svgDocument), "mousedown", this._grab);
                this._on($(this.svgDocument), "mousemove", this._drag);
                this._on($(this.svgDocument), "mouseup", this._drop);
            }
            if (isIE11)
                $(this.svgDocument).css('touch-action', 'none');

            this._on($(window), "mouseup", this.mouseup);
        },

        _initializeVariables: function (model, series, sSettings) {
            //intializing ismouseup flag
            this.ismouseup = false;
            var labelStyle = this.model.labelSettings.style;
            var lowLabelStyle = this.model.labelSettings.lowerLevel.style;
            var highLabelStyle = this.model.labelSettings.higherLevel.style;
            //checking labelstyle
            if (highLabelStyle == "")
                this.model.labelSettings.higherLevel.style = labelStyle;
            if (lowLabelStyle == "") {
                this.model.labelSettings.lowerLevel.style = labelStyle;
            }
            else if (lowLabelStyle != undefined && lowLabelStyle.horizontalAlignment == undefined) {
                this.model.labelSettings.lowerLevel.style.horizontalAlignment = "middle";
            }
            if (this.model.valueType == "numeric" && this.model.tooltipSettings.labelFormat=="MM/dd/yyyy") {
                this.model.tooltipSettings.labelFormat = "d";
            }
            //checking for datasource and finding start and end values
            if (model.dataSource != "" && model.xName != "" && model.yName != "") {
                var length = series.length;
                if (length > 0) {
                    for (var i = 0; i < length; i++) {
                        series[i].dataSource = model.dataSource;
                        series[i].type = series[i].type ? series[i].type : (sSettings.type ? model.type : "line");
                        series[i].xName = series[i].xName ? series[i].xName : model.xName;
                        series[i].yName = series[i].yName ? series[i].yName : model.yName;
                        this._processJsonData(series[i].dataSource, series[i]);
                    }
                } else
                    this._processJsonData(model.dataSource, this.model);
            }
            else if (series!="" && series != undefined && series[0].dataSource != undefined && series[0].xName != undefined && series[0].yName != undefined) {
                this._processJsonData(series[0].dataSource, series[0]);
            }
            else if (sSettings.dataSource != undefined) {
                var length = series.length;
                if (length > 0) {
                    for (var i = 0; i < length; i++) {
                        series[i].dataSource = sSettings.dataSource;
                        series[i].type = series[i].type ? series[i].type : (sSettings.type ? sSettings.type : "line");
                        series[i].xName = series[i].xName ? series[i].xName : sSettings.xName;
                        series[i].yName = series[i].yName ? series[i].yName : sSettings.yName;
                        this._processJsonData(series[i].dataSource, series[i]);
                    }
                } else
                    this._processJsonData(sSettings.dataSource, sSettings);
            }
            else if (this.model.rangeSettings.start != "" && this.model.rangeSettings.end != "") {
                if (this.model.valueType == "datetime") {
                    this.startDateTime = new Date(this.model.rangeSettings.start);
                    this.endDateTime = new Date(this.model.rangeSettings.end);
                }
                else {
                    this.startValue = this.model.rangeSettings.start;
                    this.endValue = this.model.rangeSettings.end;
                }
            }
            // initializing start year and end year
            if (this.startDateTime != undefined && this.endDateTime != undefined) {
                if (this.model.valueType == "datetime" && this.model.rangePadding == "round") {
                    var start = new Date(this.startValue);
                    var end = new Date(this.endValue);
                    this.startDateTime = new Date(start.getFullYear(), start.getMonth(), start.getDate(), 0, 0, 0);
                    this.endDateTime = new Date(end.getFullYear(), end.getMonth(), end.getDate(), 23, 59, 59);
                }
                this.startYear = this.startDateTime.getFullYear();
                this.startMonth = this.startDateTime.getMonth(); 
                this.endYear = this.endDateTime.getFullYear();
                this.endMonth = this.endDateTime.getMonth();                
            }
        },

        setSliders: function () {
            var wholeSize = this.newWidth - this.padding * 2;
            var type = this.model.valueType;
            var selectedRangeStart = this.model.selectedRangeSettings.start;
            var selectedRangeEnd = this.model.selectedRangeSettings.end;
           // var leftSliderPosition, rightSliderPosition;
            if (type == "datetime")
                this.eachInterval = wholeSize / parseFloat(this.endDateTime.getTime() - this.startDateTime.getTime());
            else
                this.eachInterval = wholeSize / parseFloat(this.endValue - this.startValue);
           
            if (type == "datetime") {
                if (selectedRangeStart=="" || new Date(selectedRangeStart).getTime() < this.startDateTime.getTime())
                    selectedRangeStart = this.startDateTime;
                if (selectedRangeEnd == "" || new Date(selectedRangeEnd).getTime() > this.endDateTime.getTime())
                    selectedRangeEnd = this.endDateTime;
                }
            else {
                if (selectedRangeStart=="" || selectedRangeStart<this.startValue)
                    selectedRangeStart = this.startValue;
                if (selectedRangeEnd == "" || selectedRangeEnd > this.endValue)
                    selectedRangeEnd = this.endValue;
                }         

            if (type == "datetime") {
                this.leftSliderPosition = (new Date(selectedRangeStart).getTime() - this.startDateTime.getTime()) * this.eachInterval + this.padding;
                this.rightSliderPosition = (new Date(selectedRangeEnd).getTime() - this.startDateTime.getTime()) * this.eachInterval + this.padding;
                selectedRangeStart = new Date(selectedRangeStart);
                selectedRangeEnd = new Date(selectedRangeEnd)
            }
            else {
                this.leftSliderPosition = (selectedRangeStart - this.startValue) * this.eachInterval + this.padding;
                this.rightSliderPosition = (selectedRangeEnd - this.startValue) * this.eachInterval + this.padding;
            }
            if (this.model.tooltipSettings.tooltipDisplayMode == "always")
                this.setopacity = 1;
            else
                this.setopacity = 0;
                this.setSliderPositions(this.leftSliderPosition, null, null);
                this.setSliderPositions(null, null, this.rightSliderPosition);
            this.setopacity = 1;
           if (this.model.enableDeferredUpdate) {
               this._calculateSelectedData();
               // checking condition for range event fire
                if (type == "datetime") {
                    if (this.startDateTime.getTime() !== selectedRangeStart.getTime() && this.endDateTime.getTime() !== selectedRangeEnd.getTime() && !this.rangeResize) {
                        this._trigger("rangeChanged", this.model);
                    }
                }
                else {
                    if (this.startValue !== selectedRangeStart && this.endValue !== selectedRangeEnd && !this.rangeResize) {
                        this._trigger("rangeChanged", this.model);
                    }
                }
				this.rangeResize=false;
            }
        },

        centerSliderWidth: "",
        zoomp: 0,
        zoomf:1,

        setSliderPositions: function (leftValue, centerValue, rightValue) {
            var doubledate;
            var type = this.model.valueType;
            var cPosition;
            var labelPlacement = this.model.labelSettings.higherLevel.labelPlacement;
            cPosition = labelPlacement == "inside" ? 0 : this.centerPosition;
            var rtlValue;
            if (leftValue != null && leftValue >= this.padding && leftValue < this.newWidth - this.padding) {
                this.leftSliderPosition = leftValue;
                rtlValue = this.model.enableRTL ? (this.newWidth - leftValue) : leftValue;
                if(type=="datetime")
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                if (this.model.tooltipSettings.visible && this.svgSupport && this.leftTxt) {
                    this.leftTooltip.setAttribute('opacity', this.setopacity);
                    if(type=="datetime")
                        this.leftTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                    else
                        this.leftTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                    var txtwidth = this.leftTooltip.lastChild.getBBox().width + 15;
                    var txtheight = this.leftTooltip.lastChild.getBBox().height;
                    var txty = this.brow!="msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.leftTxt.setAttribute('y', txty);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";

                    if (leftValue - txtwidth >= 0) {
                        this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue - txtwidth) + ',' + cPosition + ')' });
                        this.leftTxt.setAttribute('x', 5);
                    }
                    else if (leftValue + txtwidth < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftValue) + ',' + cPosition + ')' });
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                        this.leftTxt.setAttribute('x', 10);
                    }
                    else if (this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue) + ',' + (cPosition + 30) + ')'});
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    else {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue) + ',' + (cPosition) + ')'});
                        this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                    }
                    if (leftValue > this.newWidth - this.rightTooltip.lastChild.getBBox().width - 15 && this.rightTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.leftTooltip), {'transform': 'translate(' + (leftValue - txtwidth) + ',' + (cPosition + 30) + ')'});
                    }
                    if (this.rightTooltip.getCTM().f >= 30 && leftValue < this.rightTooltip.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + this.rightTooltip.getCTM().e + ',' + (cPosition) + ')'});
                    }
                }
                if (this.ismouseup == false) {
                    // this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding});
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if(this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue-10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                    }// adding the left position for thumbleft
                    else {
                        var left = leftValue - ($("#" + this._id + "thumbleft").width() / 2);
                        $("#" + this._id + "thumbleft").css("left", left);
                    }
                    this.renderer._setAttr($(this.centerSlider), {'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')'});                   
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                    this.renderer._setAttr($(this.centerSlider), { 'width': width});
                }
                else {
                    this.ismouseup = false;
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftValue - this.padding, 'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')' });
                    if (!this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + this.sliderPosition + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftValue + ',' + 0 + ')' });
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftValue });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftValue - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                    }   // adding the left position for thumbleft
                    else {
                        var left = leftValue - ($("#" + this._id + "thumbleft").width() / 2);
                        $("#" + this._id + "thumbleft").css("left", left);
                    }
                    this.renderer._setAttr($(this.centerSlider), {'transform': 'translate(' + parseFloat(leftValue) + ',' + this.sliderPosition + ')'});
                    var width = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                    this.renderer._setAttr($(this.centerSlider), { 'width': width});
                }
                this.model.zoomPosition = (leftValue-this.padding) / (this.newWidth-this.padding*2);
                this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);

                if (type == "datetime") 
                    this.model.selectedRangeSettings.start = new Date(doubledate);                                   
                else 
                        this.model.selectedRangeSettings.start = doubledate;

            }
            else if (rightValue != null && rightValue > this.padding) {
                if (rightValue > this.newWidth - this.padding + 1)
                    rightValue = this.newWidth - this.padding;
                this.rightSliderPosition = rightValue;
                var startDateTime = new Date(this.model.rangeSettings.start);
                rtlValue = this.model.enableRTL ? (this.newWidth - rightValue) : rightValue;
                doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                if(type=="datetime")
                    doubledate = new Date(doubledate);
                    //fixed issue for end value change
                else
                    doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                    this.model.selectedRangeSettings.end = doubledate;
                if (this.model.tooltipSettings.visible && this.svgSupport && this.rightTxt) {
                    this.rightTooltip.setAttribute('opacity', this.setopacity);
                    if(type=="datetime")
                        this.rightTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                    else
                        this.rightTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                    var txtwidth = this.rightTooltip.lastChild.getBBox().width + 15;
                    var txtheight = this.rightTooltip.lastChild.getBBox().height;
                    var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                    this.rightTxt.setAttribute('y', txty);
                    this.rightTxt.setAttribute('x', 10);
                    this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                    this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                    if (rightValue + txtwidth <= this.newWidth) {
                        this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                        this.renderer._setAttr($(this.rightTooltip), {'transform': 'translate(' + (rightValue) + ',' + cPosition + ')'});
                    }
                    else if (rightValue + txtwidth >= this.newWidth && rightValue - txtwidth > this.centerSlider.getCTM().e) {
                        this.renderer._setAttr($(this.rightTooltip), {'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                        this.rightTxt.setAttribute('x', 5);
                    }
                    else if (this.leftTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + (cPosition + 30) + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    else {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue - txtwidth) + ',' + cPosition + ')'});
                        this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                    }
                    if (rightValue < this.leftTooltip.getCTM().e + txtwidth && this.leftTooltip.getCTM().f < 30) {
                        this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightValue) + ',' + (cPosition + 30) + ')'});
                    }
                    if (this.leftTooltip.getCTM().f >= 30 && rightValue > this.leftTooltip.getCTM().e + txtwidth && this.rightTooltip.getCTM().e > this.leftTooltip.getCTM().e + txtwidth) {
                        this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + this.leftTooltip.getCTM().e + ',' + (cPosition) + ')' });
                        this.rightTxt.setAttribute('x', 5);
                    }
                }
                    var rightUnAreawidth =  this.newWidth - rightValue-this.padding-2; // To check width on slider change
					rightUnAreawidth = rightUnAreawidth < 0 ? 0 : rightUnAreawidth;
					
                this.renderer._setAttr($(this.rightUnArea), {'width': rightUnAreawidth, 'transform': 'translate(' + parseFloat(rightValue + 2) + ',' + this.sliderPosition + ')' });
                this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightValue + ',' + this.sliderPosition + ')'});
                if (this.rightCircle != undefined) {
                    if (this.svgSupport)
                        this.renderer._setAttr($(this.rightCircle), { 'cx': rightValue });
                    else
                        this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightValue-10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' }); //this.sliderPosition + (this.sliderHeight / 2)
                }// adding the left position for thumbright
                else {
                    var left = rightValue - ($("#" + this._id + "thumbright").width() / 2);
                    $("#" + this._id + "thumbright").css("left", left);
                }
                this.centerSliderWidth = parseFloat(this.rightSlider.getBoundingClientRect().left) - parseFloat(this.leftSlider.getBoundingClientRect().left);
                this.renderer._setAttr($(this.centerSlider), { 'width': Math.abs(this.centerSliderWidth)});

              //  this.model.zoomFactor = (rightValue - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
            }
            else if (centerValue != null) {
                var rectwidth = this.renderer._getAttrVal($(this.leftSlider),'width');
                var leftPosX = centerValue;
               
                var rightPosX = centerValue + parseFloat(this.renderer._getAttrVal($(this.centerSlider),'width'));
                var startDateTime = new Date(this.model.rangeSettings.start);
                rtlValue = this.model.enableRTL ? (this.newWidth - leftPosX) : leftPosX;
                if (leftPosX >= this.padding && rightPosX <= this.newWidth - this.padding - 2) {
                    doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                if(type=="datetime")
                    doubledate = new Date(doubledate);
                    this.model.selectedRangeSettings.start = doubledate;
                    if(this.svgSupport)
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + 0 + ')' });
                    else
                        this.renderer._setAttr($(this.leftSlider), { 'transform': 'translate(' + leftPosX + ',' + parseFloat(this.sliderPosition) + ')' });
                    this.leftSliderPosition = leftPosX;
                    if (this.leftCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.leftCircle), { 'cx': leftPosX });
                        else
                            this.renderer._setAttr($(this.leftCircle), { 'transform': 'translate(' + parseFloat(leftPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });                        
                    }
                    else {
                        var left = leftPosX - ($("#" + this._id + "thumbleft").width() / 2);
                        $("#" + this._id + "thumbleft").css("left", left);
                    }
                    if (this.model.tooltipSettings.visible && this.svgSupport && this.leftTxt) {
                        this.leftTooltip.setAttribute('opacity', this.setopacity);
                        if(type=="datetime")
                            this.leftTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                        else
                            this.leftTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                        var txtwidth = this.leftTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.leftTooltip.lastChild.getBBox().height;
                        var txty = this.brow != "msie" ? (txtheight + 6) / 2 : (txtheight / 2) + 6;
                        this.leftTxt.setAttribute('y', txty);
                        this.rightTxt.setAttribute('y', txty);
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (leftPosX - txtwidth >= 0) {
                            this.leftTooltip.firstChild.setAttribute('d', this.leftd);
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX - txtwidth) + ',' + cPosition + ')' });
                            this.leftTxt.setAttribute('x', 5);
                        }
                        else {
                            this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + cPosition + ')'});
                            this.leftTooltip.firstChild.setAttribute('d', this.rightd);
                            this.leftTxt.setAttribute('x', 10);
                            if (this.leftSliderPosition + txtwidth > this.rightSlider.getCTM().e) {
                                this.renderer._setAttr($(this.leftTooltip), { 'transform': 'translate(' + (leftPosX) + ',' + (cPosition + 30) + ')'});
                            }

                        }
                    }
                    rtlValue = this.model.enableRTL ? (this.newWidth - rightPosX) : rightPosX;
                    var doubledate = (rtlValue / this.eachInterval) + this.startDateTime.getTime() - (this.padding / this.eachInterval);
                    if (type == "datetime")
                        doubledate = new Date(doubledate);
					// fixing issue for end value when using numeric valutype
                    else
                        doubledate = (rtlValue / this.eachInterval) + this.startValue - (this.padding / this.eachInterval);
                        this.model.selectedRangeSettings.end = doubledate;
                    this.renderer._setAttr($(this.rightSlider), { 'transform': 'translate(' + rightPosX + ',' + this.sliderPosition + ')' });
                    this.rightSliderPosition = rightPosX;
                    if (this.rightCircle != undefined) {
                        if (this.svgSupport)
                            this.renderer._setAttr($(this.rightCircle), { 'cx': rightPosX });
                        else
                            this.renderer._setAttr($(this.rightCircle), { 'transform': 'translate(' + parseFloat(rightPosX - 10) + ',' + parseFloat(this.sliderPosition + (this.sliderHeight / 2)) + ')' });                        
                    }
                    else {
                        var left = rightPosX - ($("#" + this._id + "thumbright").width() / 2);
                        $("#" + this._id + "thumbright").css("left", left);
                    }
                    if (this.model.tooltipSettings.visible && this.svgSupport) {
                        this.rightTooltip.setAttribute('opacity', this.setopacity);
                        if(type=="datetime")
                            this.rightTxt.textContent = Globalize.format(new Date(doubledate), this.model.tooltipSettings.labelFormat);
                        else
                            this.rightTxt.textContent = Globalize.format(doubledate, this.model.tooltipSettings.labelFormat);
                        var txtwidth = this.rightTooltip.lastChild.getBBox().width + 15;
                        var txtheight = this.rightTooltip.lastChild.getBBox().height;
                        this.leftd = "M 0 0 L 0 " + (txtheight + 6) + " L " + (txtwidth - 6) + " " + (txtheight + 6) + " L " + (txtwidth - 6) + " 6 L " + txtwidth + " 0 Z";
                        this.rightd = "M 0 0 L " + txtwidth + " 0 L " + txtwidth + " " + (txtheight + 6) + " L 6 " + (txtheight + 6) + " L 6 6 Z";
                        if (rightPosX + txtwidth <= this.newWidth) {
                            this.rightTooltip.firstChild.setAttribute('d', this.rightd);
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX) + ',' + cPosition + ')' });
                            this.rightTxt.setAttribute('x', 10);
                        }
                        else {
                            this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + cPosition + ')'});
                            this.rightTooltip.firstChild.setAttribute('d', this.leftd);
                            this.rightTxt.setAttribute('x', 5);
                            if (this.rightTooltip.getBoundingClientRect().left < this.leftSlider.getBoundingClientRect().left) {
                                this.renderer._setAttr($(this.rightTooltip), { 'transform': 'translate(' + (rightPosX - txtwidth) + ',' + (cPosition + 30) + ')' });
                            }
                        }
                    }
                    this.renderer._setAttr($(this.leftUnArea), { 'width': leftPosX - this.padding,'transform': 'translate(' + this.padding + ',' + this.sliderPosition + ')'});
                    this.renderer._setAttr($(this.rightUnArea), { 'transform': 'translate(' + parseFloat(rightPosX + 2) + ',' + this.sliderPosition + ')','width': this.newWidth - rightPosX-this.padding-2});

                    this.model.zoomPosition = (this.leftSlider.getBoundingClientRect().left - this.padding) / (this.newWidth - this.padding * 2);

                    this.renderer._setAttr($(this.centerSlider), { 'transform': 'translate(' + centerValue + ',' + this.sliderPosition + ')' });
                    this.model.zoomFactor = (this.rightSlider.getBoundingClientRect().left + parseFloat(this.renderer._getAttrVal($(this.rightSlider), 'width')) - this.leftSlider.getBoundingClientRect().left) / (this.newWidth - this.padding * 2);
                }
               
            }

            if (!this.model.enableDeferredUpdate && this.leftSlider.getBoundingClientRect().left<this.rightSlider.getBoundingClientRect().left) {
                this._calculateSelectedData();
                if (this.zoomp != this.model.zoomPosition || this.zoomf != this.model.zoomFactor) {
                    this._trigger("rangeChanged", this.model);
                    this.zoomp = this.model.zoomPosition;
                    this.zoomf = this.model.zoomFactor;
                }
            }
          
        },

        _calculateSelectedData: function () {
            var j = [];
            var count = 0;
            if(this.model.dataSource!="")
                for (var i = 0; i < this.model.dataSource.length; i++) {
                    if (Date.parse(this.model.selectedRangeSettings.start) <= Date.parse(this.model.dataSource[i][this.model.xName]) && Date.parse(this.model.selectedRangeSettings.end) >= Date.parse(this.model.dataSource[i][this.model.xName])) {
                        j[count] = this.model.dataSource[i];
                        count++;
                    }
                }
            else if (this.model.series!="" && this.model.series[0].dataSource!="" && this.model.series[0].dataSource != "") {
                for (var i = 0; i < this.model.series[0].dataSource.length; i++) {
                    if (Date.parse(this.model.selectedRangeSettings.start) <= Date.parse(this.model.series[0].dataSource[i][this.model.series[0].xName]) && Date.parse(this.model.selectedRangeSettings.end) >= Date.parse(this.model.series[0].dataSource[i][this.model.series[0].xName])) {
                        j[count] = this.model.series[0].dataSource[i];
                        count++;
                    }
                }
            }
            this.model.selectedData = j;
        },

        calculateInterval: function (higherLevel, lowerLevel) {
            if (higherLevel.visible === true && this.model.valueType=="datetime") {
                if (higherLevel.intervalType === "years")
                    this.setYearInterval(this, "higher");
                else if (higherLevel.intervalType === "quarters")
                    this.setQuarterInterval(this, "higher", 0);
                else if (higherLevel.intervalType === "months")
                    this.setMonthInterval(this, "higher", 0);
                else if (higherLevel.intervalType == "weeks")
                    this.setWeekInterval(this, "higher", 0);
                else if (higherLevel.intervalType === "days")
                    this.setDayInterval(this, "higher",0);
                else if (higherLevel.intervalType === "hours")
                    this.setHourInterval(this, "higher",0);
            }
            if (lowerLevel.visible === true && this.model.valueType=="datetime") {

                if (lowerLevel.intervalType === "years")
                    this.setYearInterval(this, "lower");
                else if (lowerLevel.intervalType === "quarters")
                    this.setQuarterInterval(this, "lower", 0);
                else if (lowerLevel.intervalType === "months")
                    this.setMonthInterval(this, "lower", 0);
                else if(lowerLevel.intervalType=="weeks")
                    this.setWeekInterval(this, "lower", 0);
                else if (lowerLevel.intervalType === "days")
                    this.setDayInterval(this, "lower",0);
                else if (lowerLevel.intervalType === "hours")
                    this.setHourInterval(this, "lower",0);
            }

            if (lowerLevel.visible === true && this.model.valueType == "numeric") {
                this.setInterval(this);                
            }


        },

        setInterval: function (naviobj) {
            var currentValue, lastValue;
            var delta = this.endValue - this.startValue;
            var rangePadding = naviobj.model.rangePadding;
            var start = this.startValue;
            var numericInterval = this.calculateNumericInterval(naviobj, delta);
            if (rangePadding == "additional") {
                this.endValue = this.endValue + numericInterval;
                this.startValue = this.startValue - numericInterval;
                delta = this.endValue - this.startValue;
              //  numericInterval = this.calculateNumericInterval(naviobj, delta);
            } else if (rangePadding == "normal") {
                var minimum = 0, remaining;
                if (start < 0) {
                    start = 0;
                    minimum = this.startValue + (this.startValue / 20);

                    remaining = numericInterval + (minimum % numericInterval);

                    if ((0.365 * numericInterval) >= remaining) {
                        minimum -= numericInterval;
                    }

                    if (minimum % numericInterval < 0) {
                        minimum = (minimum - numericInterval) - (minimum % numericInterval);
                    }
                } else {
                    minimum = start < ((5.0 / 6.0) * this.endValue)
                        ? 0
                        : (start - (this.endValue - start) / 2);
                    if (minimum % numericInterval > 0) {
                        minimum -= (minimum % numericInterval);
                    }
                }
                var maximum = (this.endValue + (this.endValue - start) / 20);

                remaining = numericInterval - (maximum % numericInterval);

                if ((0.365 * numericInterval) >= remaining) {
                    maximum += numericInterval;
                }

                if (maximum % numericInterval > 0) {
                    maximum = (maximum + numericInterval) - (maximum % numericInterval);
                }
                numericInterval = this.calculateNumericInterval(naviobj, maximum - minimum);
                this.startValue = minimum;
                this.endValue = maximum;

            } else if (rangePadding == "round") {
                this.startValue = Math.floor((this.startValue / numericInterval) * numericInterval);
                this.endValue = Math.ceil((this.endValue / numericInterval) * numericInterval);
            }

            currentValue = naviobj.startValue;
            count = 0;
            while (currentValue <= naviobj.endValue) {
                this._insertNumericText(naviobj,currentValue);
                count++;
                currentValue = currentValue + numericInterval;
            }
            this.insertLabels(naviobj, "lower");

        },

        calculateNumericInterval: function (naviobj,delta) {
            var desiredIntervalsCount = this.GetDesiredIntervalsCount(naviobj.newWidth);
            var niceInterval = delta / desiredIntervalsCount;
            var minInterval = Math.pow(10, Math.floor(ej.EjSvgRender.utils._logBase(niceInterval, 10)));
            var intervalDivs = [10, 5, 2, 1];

            for (var i = 0; i < intervalDivs.length; i++) {
                var currentInterval = minInterval * intervalDivs[i];
                if (desiredIntervalsCount < (delta / currentInterval)) {
                    return niceInterval;
                }

                niceInterval = currentInterval;
            }

            return niceInterval;
        },

        GetDesiredIntervalsCount: function (size) {
           // if (ej.util.isNullOrUndefined(axis.desiredIntervals)) {
                var desiredIntervalsCount = 0.8* 2;
                desiredIntervalsCount = Math.max((size * (desiredIntervalsCount / 100)), 1);
                return desiredIntervalsCount;
            //} else {
            //    return axis.desiredIntervals;
          //  }
        },

        setHourInterval: function (naviobj, position,level) {

            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addHours = function (hours) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0) {
                    dat.setMilliseconds(dat.getMilliseconds() + (1000 - dat.getMilliseconds()));
                }
                if (dat.getSeconds() !== 0) {
                    dat.setSeconds(dat.getSeconds() + (60 - dat.getSeconds()));
                }
                if (dat.getMinutes() !== 0) {                    
                    dat.setMinutes(dat.getMinutes() + (60 - dat.getMinutes()));
                }
                else
                    dat.setHours(dat.getHours() + hours);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hhtt", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addHours(1);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setHourInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "hht", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;                        
                        currentDate = currentDate.addHours(1);
                    }
                    this.insertLabels(naviobj, position);
                    break;
            }
        },

        // Empty the labelbars
        _emptyLabelBars: function (naviobj, position) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
        },

        //to insert week text as there is no globalized format for week
        _insertWeekText: function (naviobj, position, textValue, currentDate, count, bwidth) {
            var hiLabelFont = naviobj.model.labelSettings.higherLevel.style.font;
            var loLabelFont = naviobj.model.labelSettings.lowerLevel.style.font;
            if (position === "higher") {
                if (count == 0)
                    naviobj._higherTotalValues[count] = 0;
                else
                    naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": hiLabelFont.size,
                    "font-family": hiLabelFont.family,
                    "font-style": hiLabelFont.style,
                    "font-weight": hiLabelFont.weight,
                    "fill": hiLabelFont.color
                };
                naviobj.renderer.drawText(textOptions, textValue, naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                if (count == 0)
                    naviobj._lowerTotalValues[count] = 0;
                else
                    naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": loLabelFont.size,
                    "font-family": loLabelFont.family,
                    "font-style": loLabelFont.style,
                    "font-weight": loLabelFont.weight,
                    "fill": loLabelFont.color
                };

                naviobj.renderer.drawText(textOptions, textValue, naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                if (this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }

            return bwidth;
        },

        //To calculate Week Interval
        setWeekInterval: function (naviobj, position, level) {

            this._emptyLabelBars(naviobj, position);
            //get week number of a year
            Date.prototype.getWeek = function () {
                var date = new Date(this.getTime());
                date.setHours(0, 0, 0, 0);
                // Thursday in current week decides the year.
                date.setDate(date.getDate() + 3 - (date.getDay() + 6) % 7);
                // January 4 is always in week 1.
                var week1 = new Date(date.getFullYear(), 0, 4);
                // Adjust to Thursday in week 1 and count number of weeks from date to week1.
                return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000
                                      - 3 + (week1.getDay() + 6) % 7) / 7);
            }

            Date.prototype.dayOfWeek = function () {
                var days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                return days[this.getDay()];
            }

            Date.prototype.addDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + noOfDays);
                return dat;
            }

            Date.prototype.subDays = function (noOfDays) {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() - noOfDays);
                return dat;
            }

            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
			var culture = this.model.locale;
			var locale = ej.datavisualization.RangeNavigator.locale[culture] ? ej.datavisualization.RangeNavigator.locale[culture] : ej.datavisualization.RangeNavigator.locale["en-US"];

			
			    while (currentDate.dayOfWeek() != "Monday") {
			        currentDate = currentDate.subDays(1);
			    }
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + currentDate.getWeek() + Globalize.format(currentDate, " MMMM, yyyy"), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + currentDate.getWeek() + Globalize.format(currentDate, " MMM, yy"), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].longWeeks + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setWeekInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertWeekText(naviobj, position, locale.intervals['week'].shortWeeks + currentDate.getWeek(), currentDate, count, bwidth);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays(7);
                    }
                        this.insertLabels(naviobj, position);
                    break;
            }
        },
       
        setDayInterval: function (naviobj, position,level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = []; 
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();

            Date.prototype.addDays = function () {
                var dat = new Date(this.valueOf());
                dat = new Date(dat.getFullYear(), dat.getMonth(), dat.getDate() + 1);
                return dat;
            }

            var hourArray = [];
            var currentDate = naviobj.startDateTime;
            var count = 0;
            var txtBlockWidth = 0;
            var bwidth = 0;
            var bBox;
            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth= this._insertText(naviobj, position, "dddd, MMMM d, yyyy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;                       
                        currentDate = currentDate.addDays();
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "ddd, MMM d, yy", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                        
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dddd, d", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                        
                    }
                    if (txtBlockWidth >= naviobj.newWidth)
                        naviobj.setDayInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "dd", currentDate, count, bwidth, false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addDays();                       
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        setMonthInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();           

            var currentDate = naviobj.startDateTime;            
			var textPadding =5;
            var count = 0;
            var date = new Date(naviobj.startDateTime);
            var txtBlockWidth = 0;
            var bwidth = 0;
            var bBox;

            switch (level) {
                case 0:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM, yyyy", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth + textPadding >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMMM", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth + textPadding >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth,false);
                        txtBlockWidth += bwidth;                        
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    if (txtBlockWidth + textPadding >= naviobj.newWidth)
                        naviobj.setMonthInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    while (currentDate <= naviobj.endDateTime) {
                        bwidth = this._insertText(naviobj, position, "MMM", currentDate, count, bwidth, true);
                        txtBlockWidth += bwidth;
                        count++;
                        currentDate = currentDate.addMonths();
                    }
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;

            }

        },

        _quarter: function (naviobj, position, quarterValue, yearString, date, txtBlockWidth, count) {
            
            for (var i = naviobj.startYear; i <= naviobj.endYear; i++) {
                var stringDate = yearString != null ? Globalize.format(date, yearString) : "";
                if (i === naviobj.startYear) {
                    for (var j = naviobj.startMonth; j <= 11; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight,
                                "fill":naviobj.model.labelSettings.higherLevel.style.font.color
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight,
                                "fill":naviobj.model.labelSettings.lowerLevel.style.font.color
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if(position=="higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            if (j === naviobj.startMonth)
                                naviobj._lowerTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._lowerTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            if (j === naviobj.startMonth)
                                naviobj._higherTotalValues[count] = naviobj.startDateTime.getTime() - naviobj.startDateTime.getTime();
                            else
                                naviobj._higherTotalValues[count] = new Date(naviobj.startYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else if (i === naviobj.endYear) {
                    for (var j = 0; j <= naviobj.endMonth; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();                            
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(naviobj.endYear + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }
                else {
                    for (var j = 0; j <= 11; j++) {
                        if (position == "higher") {
                            var textOptions = {
                                'id': this._rootId + '_higLabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight
                            };
                        }
                        else {
                            var textOptions = {
                                'id': this._rootId + '_LabelBarText' + '_' + count,
                                'x': 0,
                                'y': 0,
                                'text-anchor': 'start',
                                'dominant-baseline': 'middle',
                                "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                                "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                                "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                                "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight
                            };
                        }
                        if (j >= 0 && j <= 2) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "1 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(3));
                            k = 2;
                        }
                        else if (j >= 3 && j <= 5) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "2 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(6));
                            k = 5;

                        }
                        else if (j >= 6 && j <= 8) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "3 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(9));
                            k = 8;

                        }
                        else if (j >= 9 && j <= 11) {
                            if (position == "higher")
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.higherLevel);
                            else
                                naviobj.renderer.drawText(textOptions, quarterValue + "4 " + stringDate, naviobj.lowerLevel);
                            date = new Date(date.setMonth(12));
                            k = 11;

                        }


                        if (position === "lower") {
                            naviobj._lowerTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        else {
                            naviobj._higherTotalValues[count] = new Date(i + "/" + parseInt(j + 1) + "/1").getTime() - naviobj.startDateTime.getTime();
                            j = k;
                            naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                            var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                            var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                            txtBlockWidth += textOffset.width;
                        }
                        count++;
                    }
                }

            }
            return txtBlockWidth;
        },

        setQuarterInterval: function (naviobj, position, level) {
            naviobj._higherTextNode = [];
            naviobj._higherTotalValues = [];
            naviobj._lowerTextNode = [];
            naviobj._lowerTotalValues = [];
            if (position === "higher")
                $("#higherLevel").children().empty();
            else
                $("#lowerLevel").children().empty();
            var date = new Date(naviobj.startDateTime);
            var count = 0;
            var k = 0;
            var txtBlockWidth = 0;
			var culture = this.model.locale;
			var locale = ej.datavisualization.RangeNavigator.locale[culture] ? ej.datavisualization.RangeNavigator.locale[culture]:
                                                                               ej.datavisualization.RangeNavigator.locale["en-US"];
            switch (level) {
                case 0:
                    txtBlockWidth=this._quarter(naviobj, position, locale.intervals['quarter'].longQuarters, "yyyy", date, txtBlockWidth, count);                    
                    if (txtBlockWidth >= (naviobj.newWidth - this.padding))
                        naviobj.setQuarterInterval(naviobj, position, 1);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 1:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].longQuarters, "yy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= (naviobj.newWidth - this.padding))
                        naviobj.setQuarterInterval(naviobj, position, 2);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 2:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].shortQuarters, "yyyy", date, txtBlockWidth, count);
                    if (txtBlockWidth >= (naviobj.newWidth - this.padding))
                        naviobj.setQuarterInterval(naviobj, position, 3);
                    else
                        this.insertLabels(naviobj, position);
                    break;
                case 3:
                    txtBlockWidth = this._quarter(naviobj, position, locale.intervals['quarter'].shortQuarters, null, date, txtBlockWidth, count);
                    this.insertLabels(naviobj, position);
                    break;
                default:
                    break;
            }
        },

        _insertText: function (naviobj, position, txtValue, currentDate, count, bwidth, setsubstring) {

            Date.prototype.addYears = function (years) {
                var dat = new Date(this.valueOf())
                if (dat.getMilliseconds() !== 0 || dat.getSeconds() !== 0
                 || dat.getMinutes() !== 0 || dat.getHours() !== 0 || dat.getDate() !== 0
                 || dat.getMonth() !== 0) {
                    dat = new Date(dat.getFullYear() + 1, 0, 1);
                }
                else
                    dat.setYear(dat.getFullYear() + years);
                return dat;
            }

            Date.prototype.addMonths = function () {
                var dat = new Date(this.valueOf())
                dat = new Date(dat.getFullYear(), dat.getMonth() + 1, 1);
                return dat;
            }

            if (position === "higher") {
                naviobj._higherTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_higLabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.higherLevel.style.font.size,
                    "font-family": naviobj.model.labelSettings.higherLevel.style.font.fontFamily,
                    "font-style": naviobj.model.labelSettings.higherLevel.style.font.fontStyle,
                    "font-weight": naviobj.model.labelSettings.higherLevel.style.font.fontWeight,
                    "fill":naviobj.model.labelSettings.higherLevel.style.font.color
                };
                if(setsubstring==false)
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue), naviobj.higherLevel);
                else
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue).toString().substring(0, 1), naviobj.higherLevel);
                naviobj._higherTextNode[count] = naviobj.higherLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                if(this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            else {
                naviobj._lowerTotalValues[count] = currentDate.getTime() - naviobj.startDateTime.getTime();
                var textOptions = {
                    'id': this._rootId + '_LabelBarText' + '_' + count,
                    'x': 0,
                    'y': 0,
                    'text-anchor': 'start',
                    'dominant-baseline': 'middle',
                    "font-size": naviobj.model.labelSettings.lowerLevel.style.font.size,
                    "font-family": naviobj.model.labelSettings.lowerLevel.style.font.fontFamily,
                    "font-style": naviobj.model.labelSettings.lowerLevel.style.font.fontStyle,
                    "font-weight": naviobj.model.labelSettings.lowerLevel.style.font.fontWeight,
                    "fill":naviobj.model.labelSettings.lowerLevel.style.font.color
                };
                if(setsubstring==false)
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue), naviobj.lowerLevel);
                else
                    naviobj.renderer.drawText(textOptions, Globalize.format(currentDate, txtValue).substring(0, 1), naviobj.lowerLevel);
                naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
                var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                if(this.svgSupport)
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
                else
                    var textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
                bwidth = Math.max(bwidth, textOffset.width);
            }
            return bwidth;
        },

        _insertNumericText: function (naviobj,currentValue) {
            naviobj._lowerTotalValues[count] = currentValue - naviobj.startValue;
            var font = naviobj.model.labelSettings.style.font;
            var textOptions = {
                'id': this._rootId + '_LabelBarText' + '_' + count,
                'x': 0,
                'y': 0,
                'text-anchor': 'start',
                'dominant-baseline': 'middle',
                "font-size": font.size,
                "font-family": font.family,
                "font-style": font.style,
                "font-weight": font.weight,
                "fill": font.color
            };
                naviobj.renderer.drawText(textOptions, currentValue, naviobj.lowerLevel);
            naviobj._lowerTextNode[count] = naviobj.lowerLevel.childNodes[count];
            var labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
            var textOffset;
            if(this.svgSupport) {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont);
            } else {
                textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].innerHTML, null, labelfont);
            } 
        },

        setYearInterval: function (naviobj, position) {
            var currentDate = naviobj.startDateTime;
            count=0;
            while (currentDate <= naviobj.endDateTime) {
                this._insertText(naviobj, position, "yyyy", currentDate,count,0,false);                   
                count++;
                currentDate = currentDate.addYears(1);
            }            
            this.insertLabels(naviobj, position);
        },

        _applyPadding:function(){
            var i;
            for (i = 0; i < this._higherTotalValues.length; i++) {
                this._higherTotalValues[i] += this.padding;
            }
            for (i = 0; i < this._lowerTotalValues.length; i++) {
                this._lowerTotalValues[i] += this.padding;
            }
        },

        insertLabels: function (naviobj, level) {
            // this._applyPadding();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var lowgridColor = this.model.labelSettings.lowerLevel.gridLineStyle.color;
            this.lgClr = this.renderer.createGradientElement('lgColor', lowgridColor, 150, 0, 150, 100, this.svgDocument);
            var type = this.model.valueType;
            var wholeSize = parseFloat(this.renderer._getAttrVal($(naviobj.svgDocument), 'width'));
            if (type == "datetime")
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endDateTime.getTime() - this.startDateTime.getTime());
            else
                this.eachInterval = (wholeSize - this.padding * 2) / parseFloat(this.endValue - this.startValue);
            var count = 0;
            var labelcount = 0;
            var txtwidth = 0;
            var rectoptions;
            var lineleft;
            var newleft;
            var textOffset;
            var labelfont;
            var i;
            if (level === "higher" && naviobj.model.labelSettings.higherLevel.visible===true) {

                if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth > 0 ? (naviobj.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                    rectoptions = {
                        x: this.padding,
                        y: 1,
                        width: naviobj.newWidth - naviobj.padding * 2,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }
                else {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.bottomPosition,
                        width: naviobj.newWidth > 0 ? (naviobj.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minHighHeight,
                        'stroke': naviobj.model.labelSettings.higherLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.higherLevel.border.width,
                        fill: 'transparent'

                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.higherLevel);
                }

                naviobj._higherLineLeft = [];

                if (!naviobj.model.enableRTL) {
                    lineleft = (naviobj._higherTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._higherTotalValues[naviobj._higherTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var isrtl;
                if (naviobj.model.enableRTL) {
                    i = naviobj._higherTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }
                var setLeft = 0;

                for (; isrtl ? i >= 0 : i < naviobj._higherTextNode.length; isrtl?i--:i++) {
                    naviobj._higherLineLeft[i] = lineleft;
                    //  var txt =  naviobj._higherTextNode[i].getBBox();
                    labelfont = { size: naviobj.model.labelSettings.higherLevel.style.font.size, fontStyle: naviobj.model.labelSettings.higherLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.higherLevel.style.font.fontFamily };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.higherLevel.childNodes[count].textContent, null, labelfont);
                    txtwidth = textOffset.width;
                    newleft = naviobj._higherTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl?naviobj.newWidth-((naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding)):
                        (naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle") {
                        setleft = isrtl ? newleft - (newleft - lineleft) / 2 - txtwidth / 2: (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left") {
                        setleft = isrtl ? newleft + 10 : lineleft + 10;
                    }
                    else
                        setleft = isrtl ? lineleft - txtwidth - 2 : newleft - txtwidth - 2;
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'fill': naviobj.model.labelSettings.higherLevel.style.font.color });
                    naviobj._higherTextLeft[i] = setleft;
                    
                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.minHighHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position == "top") {
                        if (this.svgSupport) {
                            if(matched.browser.toLowerCase() == "msie")
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                            else
                                naviobj._higherTextNode[i].setAttribute("y", naviobj.centerPosition - 9);
                        }
                        else
                            naviobj.renderer._setAttr($(naviobj._higherTextNode[i]), { 'y': (naviobj.centerPosition / 2) - 5 });
                        rectoptions = {
                            x1: lineleft,
                            y1: '0',
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray':naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    else {
                        naviobj._higherTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: this.bottomPosition,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': naviobj.model.labelSettings.higherLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.higherLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.higherLevel.gridLineStyle.dashArray,

                        };
                        if (i != 0)
                        naviobj.renderer.drawLine(rectoptions, naviobj.higherLevel);
                    }
                    if ((setleft < this.padding || naviobj.newWidth - lineleft - this.padding < txtwidth) && type == "datetime") {
                        if (isrtl) {
                            if (setleft < this.padding && lineleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle")
                                    naviobj._higherTextNode[i].setAttribute("x", this.padding + (lineleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                                    naviobj._higherTextNode[i].setAttribute("x", this.padding + 10);
                                else
                                    naviobj._higherTextNode[i].setAttribute("x", lineleft - txtwidth - 2);
                            }
                            else if (naviobj.newWidth - lineleft < txtwidth && naviobj.newWidth - newleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle")
                                    naviobj._higherTextNode[i].setAttribute("x", newleft + (naviobj.newWidth - newleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                                    naviobj._higherTextNode[i].setAttribute("x", newleft + 10);
                                else
                                    naviobj._higherTextNode[i].setAttribute("x", naviobj.newWidth - txtwidth - 2 - this.padding);
                            }
                            else
                                naviobj.higherLevel.removeChild(naviobj._higherTextNode[i]);
                        }
                        else {
                            naviobj.higherLevel.removeChild(naviobj._higherTextNode[i]);
                            continue;
                        }
                    }
                    else if (setleft + txtwidth > naviobj.newWidth - this.padding && type == "datetime") {
                        if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "middle")
                            setleft = lineleft + (naviobj.newWidth - lineleft - this.padding) / 2 - txtwidth / 2;
                        else if (naviobj.model.labelSettings.higherLevel.style.horizontalAlignment === "left")
                            setleft = lineleft;
                        else
                            setleft = naviobj.newWidth - txtwidth * 3/2;
                        naviobj._higherTextNode[i].setAttribute("x", setleft);
                        continue;
                    }
                    if (!this.model.enableRTL) {
                        lineleft = naviobj._higherTotalValues[i + 1] * this.eachInterval + this.padding;
                    } else {
                        lineleft= naviobj.newWidth - (naviobj._higherTotalValues[i -1] * this.eachInterval + this.padding);
                    }
                    count++;
                }

                

            }
            else if (naviobj.model.labelSettings.lowerLevel.visible === true) {

                naviobj._lowerLineLeft = [];
                if (!naviobj.model.enableRTL) {
                    lineleft = (naviobj._lowerTotalValues[count] * this.eachInterval) + this.padding;
                } else {
                    lineleft = naviobj.newWidth - ((naviobj._lowerTotalValues[naviobj._lowerTextNode.length - 1] * this.eachInterval) + this.padding);
                }
                var hy = naviobj.model.labelSettings.higherLevel.visible ? naviobj.bottomPosition : naviobj.centerHeight;

                if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: hy,
                        width: this.newWidth - naviobj.padding * 2,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    if (type != "numeric")
                        naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.centerHeight,
                        width: this.newWidth > 0 ? (this.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                    rectoptions = {
                        x: this.padding,
                        y: 0,
                        width: this.newWidth > 0 ? (this.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minLowHeight - 1,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }
                else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                    rectoptions = {
                        x: this.padding,
                        y: naviobj.minHighHeight - 1,
                        width: this.newWidth > 0 ? (this.newWidth - naviobj.padding * 2) : 0,
                        height: naviobj.minLowHeight,
                        'stroke': naviobj.model.labelSettings.lowerLevel.border.color,
                        'stroke-width': naviobj.model.labelSettings.lowerLevel.border.width,
                        fill: 'transparent'
                    };
                    naviobj.renderer.drawRect(rectoptions, naviobj.lowerLevel);
                }

                if (naviobj.model.enableRTL) {
                    i = naviobj._lowerTextNode.length - 1;
                    isrtl = true;
                }
                else {
                    i = 0;
                    isrtl = false;
                }


                for (; isrtl ? i >= 0 : i < naviobj._lowerTextNode.length; isrtl?i--:i++) {
                    naviobj._lowerLineLeft[i] = lineleft;
                    labelfont = { size: naviobj.model.labelSettings.lowerLevel.style.font.size, fontStyle: naviobj.model.labelSettings.lowerLevel.style.font.fontStyle, fontFamily: naviobj.model.labelSettings.lowerLevel.style.font.fontFamily };
                    textOffset = ej.EjSvgRender.utils._measureText(naviobj.lowerLevel.childNodes[count].textContent, null, labelfont); // var txt = naviobj._lowerTextNode[i].getBBox();
                    txtwidth = textOffset.width;// txt.width;
                    var setleft = 0;
                    newleft = naviobj._lowerTotalValues[i + 1] === undefined
                        ? isrtl ? 0 : naviobj.newWidth :
                        isrtl ? naviobj.newWidth - ((naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding)) :
                        (naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding);
                    if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle") {
                        setleft = isrtl ? (lineleft - newleft) / 2 + newleft - txtwidth / 2 : (newleft - lineleft) / 2 + lineleft - txtwidth / 2;
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left") {
                        setleft = isrtl? newleft + 2 : lineleft + 2;
                    }
                    else
                        setleft = isrtl ? lineleft - txtwidth - 2 : newleft - txtwidth - 2;
                    if (type == "numeric")
                        setleft = lineleft - txtwidth / 2;
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'x': setleft });
                    naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'fill': naviobj.model.labelSettings.lowerLevel.style.font.color });

                    naviobj._lowerTextLeft[i] = setleft;

                    if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "top") {
                        if (!this.svgSupport)
                            naviobj.renderer._setAttr($(naviobj._lowerTextNode[i]), { 'y': naviobj.newHeight - textOffset.height });
                        else {
                            if (matched.browser.toLowerCase() == "msie")
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 5);
                            else
                                naviobj._lowerTextNode[i].setAttribute("y", naviobj.newHeight - 9);

                        }
                        var ht = naviobj.model.labelSettings.higherLevel.visible ? parseFloat(naviobj.minHighHeight + naviobj.centerHeight) : naviobj.minHighHeight;
                        rectoptions = {
                            x1: lineleft,
                            y1: ht,
                            x2: lineleft,
                            y2: naviobj.newHeight,
                            'stroke': this.lgClr,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0 && type != "numeric")
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "bottom" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.bottomPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.centerHeight,
                            x2: lineleft,
                            y2: naviobj.bottomPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray':naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.lowerLevel.position === "top" && naviobj.model.labelSettings.higherLevel.position === "bottom") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: 0,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    else if (naviobj.model.labelSettings.higherLevel.position === "top" && naviobj.model.labelSettings.lowerLevel.position === "top") {
                        naviobj._lowerTextNode[i].setAttribute("y", naviobj.centerPosition - 5);
                        rectoptions = {
                            x1: lineleft,
                            y1: naviobj.minHighHeight,
                            x2: lineleft,
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.labelSettings.lowerLevel.gridLineStyle.color,
                            'stroke-width': naviobj.model.labelSettings.lowerLevel.gridLineStyle.width,
                            'stroke-dasharray': naviobj.model.labelSettings.lowerLevel.gridLineStyle.dashArray,
                        };
                        if (i != 0)
                            naviobj.renderer.drawLine(rectoptions, naviobj.lowerLevel);
                    }
                    if ((setleft < this.padding || naviobj.newWidth - lineleft - this.padding < txtwidth) && type == "datetime") {
                        if (isrtl) {
                            if (setleft < this.padding && lineleft - this.padding > txtwidth) {
                                if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle")
                                    naviobj._lowerTextNode[i].setAttribute("x", this.padding + (lineleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                                    naviobj._lowerTextNode[i].setAttribute("x", this.padding + 10);
                                else
                                    naviobj._lowerTextNode[i].setAttribute("x", lineleft - txtwidth - 2);
                            }
                            else if (naviobj.newWidth - lineleft < txtwidth && naviobj.newWidth -this.padding - newleft > txtwidth) {
                                if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle")
                                    naviobj._lowerTextNode[i].setAttribute("x", newleft + (naviobj.newWidth - newleft - this.padding) / 2 - txtwidth / 2);
                                else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                                    naviobj._lowerTextNode[i].setAttribute("x", newleft + 10);
                                else
                                    naviobj._lowerTextNode[i].setAttribute("x", naviobj.newWidth - txtwidth - 2 - this.padding);
                            }
                            else
                                naviobj.lowerLevel.removeChild(naviobj._lowerTextNode[i]);
                            count--;
                        }
                        else {
                            naviobj.lowerLevel.removeChild(naviobj._lowerTextNode[i]);
                            count--;
                        }
                    }
                    else if (setleft + txtwidth > naviobj.newWidth - this.padding && type == "datetime") {
                        if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "middle")
                            setleft = lineleft + (naviobj.newWidth - lineleft - this.padding) / 2 - txtwidth / 2;
                        else if (naviobj.model.labelSettings.lowerLevel.style.horizontalAlignment === "left")
                            setleft = lineleft;
                        else
                            setleft = naviobj.newWidth - txtwidth * 3 / 2;
                        naviobj._lowerTextNode[i].setAttribute("x", setleft);                        
                    }
                    if (!this.model.enableRTL)
                        lineleft = naviobj._lowerTotalValues[i + 1] * this.eachInterval + this.padding;
                    else
                        lineleft = naviobj.newWidth - (naviobj._lowerTotalValues[i - 1] * this.eachInterval + this.padding);
                    count++;
                }



            }
            naviobj._setGridlines(naviobj);
            $(this.higherLevel).css('cursor', 'pointer');
            $(this.lowerLevel).css('cursor', 'pointer');
            //this.higherLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');
            //this.lowerLevel.setAttributeNS(null, 'transform', 'translate(' + 20 + ',' + 0 + ')');

        },

        _setGridlines: function (naviobj) {

            if (naviobj.model.navigatorStyleSettings.minorGridLineStyle.visible === true) {

                for (var i = 1; i < naviobj._lowerTextNode.length; i++) {
                    if (naviobj.model.labelSettings.higherLevel.position == "top" && naviobj.model.labelSettings.lowerLevel.position == "top") {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerHeight + naviobj.minLowHeight+naviobj.minHighHeight,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition,
                            'stroke': naviobj.model.navigatorStyleSettings.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }
                    else {
                        var rectoptions = {
                            x1: naviobj._lowerLineLeft[i],
                            y1: naviobj.centerPosition,
                            x2: naviobj._lowerLineLeft[i],
                            y2: naviobj.centerPosition+naviobj.centerHeight,
                            'stroke': naviobj.model.navigatorStyleSettings.minorGridLineStyle.color,
                            'stroke-width': '1'
                        };
                        naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                    }
                    
                }
            }
            else {
                for (var i = 0; i < naviobj._higherTextNode.length; i++) {
                    // var lineLeft = naviobj._lowerLineLeft[i];
                    var rectoptions = {
                        x1: naviobj._higherLineLeft[i],
                        y1: naviobj.centerHeight + naviobj.minLowHeight,
                        x2: naviobj._higherLineLeft[i],
                        y2: naviobj.centerPosition,
                        'stroke': naviobj.model.navigatorStyleSettings.majorGridLineStyle.color,
                        'stroke-width': '1'
                    };
                    naviobj.renderer.drawLine(rectoptions, naviobj.centerLevel);
                }
            }

        },

        bindTo: function () {
            if (!ej.util.isNullOrUndefined(this.model.dataSource)) {
                if (this.model.dataSource != null && this.model.dataSource.length > 0) {

                    this._processJsonData(this.model.dataSource, this.model);
                }
            }

        },

        _processJsonData: function (jsonObj, newmodel) {

            var xdata = [];
            var yValues = [];
            var index = 0;

            for (var j = 0; j < jsonObj.length; j++) {
                if (typeof jsonObj[j][newmodel.xName] == "string" && jsonObj[j][newmodel.xName].indexOf("/Date(") != -1)
                    jsonObj[j][newmodel.xName] = new Date(parseInt(jsonObj[j][newmodel.xName].substr(6)));
                if (jQuery.type(jsonObj[j][newmodel.xName]) != "string" && jQuery.type(jsonObj[j][newmodel.xName]) != "date")
                    xdata.push(parseFloat(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.xName]) ? jsonObj[j][newmodel.xName] : j));
                else {
                    xdata.push(!ej.util.isNullOrUndefined(jsonObj[j][newmodel.xName]) ? jsonObj[j][newmodel.xName] : j);
                }
            }

            if (!ej.util.isNullOrUndefined(newmodel.yName)) {
                for (var k = 0; k < newmodel.yName.length; k++) {
                    var yNVal = [];
                    for (var yI = 0; yI < jsonObj.length; yI++) {
                        yNVal.push(jsonObj[yI][newmodel.yName[k]] === null || jsonObj[yI][newmodel.yName[k]] === undefined ? jsonObj[yI][newmodel.yName[k]] : parseFloat(jsonObj[yI][newmodel.yName[k]]));
                    }
                    yValues[k] = yNVal;
                    index = k + 1;
                }
            }

            this.endDateTime = new Date(Math.max.apply(null, xdata));
            this.startDateTime = new Date(Math.min.apply(null, xdata));
            if (this.endDateTime.getTime() == this.startDateTime.getTime()) {
                this.startDateTime.setHours(0);
                this.endDateTime.setHours(23);
            }
            this.endValue = this.endDateTime.getTime();
            this.startValue = this.startDateTime.getTime();
        },

        calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;
            
            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else if (matched.browser.toLowerCase() == "msie") {

                mouseX = (e.originalEvent.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.originalEvent.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }
            else {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        calMousePosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;
            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                mouseX = (e.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (e.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = 0;//$(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else {
                mouseX = (e.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        calTouchPosition: function (e) {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var mouseX, mouseY;

            if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).parent().offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).parent().offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
                this.grabPadding = 0;
            } else if (matched.browser.toLowerCase() == "msie") {

                mouseX = (e.originalEvent.pageX) - $(this.svgDocument).offset().left;
                mouseY = (e.originalEvent.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }
            else {
                var touch = e.originalEvent.touches[0];
                mouseX = (touch.pageX) - $(this.svgDocument).offset().left;
                mouseY = (touch.pageY) - $(this.svgDocument).offset().top;
                this.leftPadding = $(this.svgDocument).offset().left;
            }

            return { X: mouseX, Y: mouseY };

        },

        _grab: function (evt) {
            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);
            var naviobj = this;// evt.data.Param1;
            var mousedownCords;
            if ((this.svgSupport && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") {
                mousedownCords = naviobj.calMousePosition(evt);
            } else if (!this.svgSupport) {
                mousedownCords = naviobj.calMousePosition(evt);
            } else {
                mousedownCords = naviobj.calTouchPosition(evt);
            } // naviobj.calMousePosition(evt);
            naviobj.mouseDownX = mousedownCords.X;
            naviobj.mouseDownY = mousedownCords.Y;
            // find out which element we moused down on
            
            var targetElement;
            if (naviobj.edge == true) {
                targetElement = naviobj.target;
            }
            else {
                targetElement = evt.target;
            }
            // used to find a thumb id
            var current = targetElement;
            var targetId;
            while (current.parentNode) {
				current = current.parentNode
				if (current.id == this._id + "thumbleft")
                {
                    targetId = this._id + "thumbleft";
                }
                else if (current.id == this._id + "thumbright")
                {
                    targetId = this._id + "thumbright";
                }
            }
            //$(targetElement).css('cursor', 'w-resize');
            // you cannot drag the background itself, so ignore any attempts to mouse down on it

            if (targetElement.parentNode.id === "leftslider" || targetElement.parentNode.id === "rightslider" || targetId == this._id + "thumbleft" || targetId == this._id + "thumbright" /*|| targetElement.parentNode.parentNode.id === "item_left" */|| targetElement.parentNode.id === "centerslider" || targetElement.parentNode.id === this._rootId + "higherLevel" || targetElement.parentNode.id === this._rootId + "lowerLevel")
            {
                //set the item moused down on as the element to be dragged
                naviobj.dragTarget = targetElement;
                naviobj.grabbed = true;
                naviobj.edge = false;
                if (this.svgSupport) {
                    if (targetId == this._id + "thumbleft" || targetId == this._id + "thumbright")
                        var transMatrix = {e:evt.clientX,f:evt.clientY};
                    else
                        var transMatrix = naviobj.dragTarget.getCTM();
                    if (matched.browser.toLowerCase() == "mozilla" || matched.browser.toLowerCase() == "webkit") {
                        naviobj.grabPoint.x = naviobj.mouseDownX + this.grabPadding - Number(transMatrix.e);
                    }
                    else
                        naviobj.grabPoint.x = naviobj.mouseDownX + this.leftPadding - Number(transMatrix.e);
                    naviobj.grabPoint.y = naviobj.trueCoords.y - Number(transMatrix.f);
                }
                var txtPosition;
                var i;
                var flag = false;
                var isrtl = naviobj.model.enableRTL;
                if (isrtl) {
                    naviobj._higherLineLeft.sort(function (a, b) { return a - b });
                    naviobj._lowerLineLeft.sort(function (a, b) { return a - b });
                }
                if (naviobj.dragTarget.parentNode.id === this._rootId + "higherLevel" && naviobj.model.valueType == "datetime") {
                    naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i=0;i< naviobj._higherLineLeft.length;i++) {
                        if (i + 1 < naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i] && txtPosition < naviobj._higherLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._higherLineLeft.length && txtPosition > naviobj._higherLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._higherLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._higherLineLeft[0]);
                        flag = false;
                    }
                }
                else if (naviobj.dragTarget.parentNode.id === this._rootId + "lowerLevel" && naviobj.model.valueType == "datetime") {
                    naviobj.grabbed = false;
                    txtPosition = naviobj.mouseDownX;
                    for (i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                            flag = true;
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i]) {
                            naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                            naviobj.setSliderPositions(null, null, naviobj.newWidth - naviobj.padding);
                            flag = true;
                        }
                    }
                    if (isrtl && !flag) {
                        naviobj.setSliderPositions(0 + naviobj.padding, null, null);
                        naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[0]);
                        flag = false;
                    }
                }

            }
        },

        _drag: function (evt) {

            evt.preventDefault();
            var matched = jQuery.uaMatch(navigator.userAgent);

            var naviobj = this;//evt.data.Param1;
            if ((this.svgSupport && evt.originalEvent.toString() !== "[object TouchEvent]") && matched.browser.toLowerCase() != "msie") {
                var mousedownCords = naviobj.calMousePosition(evt);
            }
            else if (!this.svgSupport)
                var mousedownCords = naviobj.calMousePosition(evt);
            else
                var mousedownCords = naviobj.calTouchPosition(evt);//naviobj.calMousePosition(evt);
            naviobj.mouseDownX = mousedownCords.X;
            naviobj.mouseDownY = mousedownCords.Y;
            if(this.svgSupport)
            naviobj.GetTrueCoords(evt);
            // if we don't currently have an element in tow, don't do anything
            if (naviobj.dragTarget) {
                // account for the offset between the element's origin and the
                //    exact place we grabbed it... this way, the drag will look more natural
                if (!naviobj.edge && naviobj.dragTarget === naviobj.leftSlider || naviobj.dragTarget === naviobj.rightSlider || naviobj.dragTarget === naviobj.centerSlider) {
                    if (this.svgSupport) {
					if(matched.browser.toLowerCase() == "webkit"||matched.browser.toLowerCase() == "mozilla")
					{
                        var newX = naviobj.mouseDownX - naviobj.grabPoint.x;
                        var newY = naviobj.mouseDownY - naviobj.grabPoint.y;
						}
						else
						{
						var newX = naviobj.trueCoords.x - naviobj.grabPoint.x;
                        var newY = naviobj.trueCoords.y - naviobj.grabPoint.y;
						}
                    }
                    else {
                        var newX = naviobj.mouseDownX;
                        var newY = naviobj.mouseDownY;
                    }
                }
                else {
                    var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    //  var newX = naviobj.mouseDownX;// naviobj.trueCoords.x - naviobj.leftPadding;
                    var newY = naviobj.mouseDownY;// naviobj.trueCoords.y;
                } // used to find a id of the thumb
                var current = naviobj.dragTarget;
                var targetId;
                while (current.parentNode) {
                    current = current.parentNode
                    if (current.id == this._id + "thumbleft") {
                        targetId = this._id + "thumbleft";
                    }
                    else if (current.id == this._id + "thumbright") {
                        targetId = this._id + "thumbright";
                    }
                }
                naviobj.diff = naviobj.leftSlider.getBoundingClientRect().left - naviobj.rightSlider.getBoundingClientRect().left;
                naviobj.leftdiff = naviobj.rightSlider.getBoundingClientRect().left - this.padding;
                naviobj.rightdiff = naviobj.leftSlider.getBoundingClientRect().left - this.padding;
                if (naviobj.dragTarget.parentNode.id === "leftslider" || targetId == this._id + "thumbleft") {
                    naviobj.grabbed = false;
                    if (naviobj.diff >= -1 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.rightdiff<naviobj.newWidth-(2*this.padding)) {
                        naviobj.target = naviobj.rightSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        if (newX <= this.rightSliderPosition) {
                            naviobj.setSliderPositions(newX, null, null);
                            this.leftSliderPosition = newX;
                            this.leftset = false;
                        }
                        else {
                            naviobj.setSliderPositions(this.rightSliderPosition, null, null);
                            this.leftset = true;
                            this.leftSliderPosition = this.rightSliderPosition;
                        }
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.left = newX;
                }
                else if (naviobj.dragTarget.parentNode.id === "rightslider" || targetId == this._id + "thumbright") {
                    naviobj.grabbed = false;
                    // if (((this.svgSupport) && naviobj.leftSlider.getCTM().e == naviobj.rightSlider.getCTM().e && newX < naviobj.newWidth-15 && naviobj.rightSlider.getCTM().e == (naviobj.newWidth - naviobj.padding)) || (naviobj.leftSlider.getBoundingClientRect().left == naviobj.rightSlider.getBoundingClientRect().left && newX < naviobj.newWidth - 15 && naviobj.rightSlider.getBoundingClientRect().left == (naviobj.newWidth - naviobj.padding))) {                                                                
                    if (naviobj.diff >= 0 && naviobj.diff <= 1 && naviobj.switched == false && naviobj.leftdiff >0) {
                        naviobj.target = naviobj.leftSlider;
                        naviobj.switched = true;
                        naviobj.edge = true;
                        naviobj._grab(evt);
                        naviobj._drag(evt);
                    }
                    else {
                        //if (newX >= naviobj.leftSlider.getCTM().e)
                        if (newX >= this.leftSliderPosition) { // naviobj.leftSlider.getBoundingClientRect().left - this.leftPadding)
                            naviobj.setSliderPositions(null, null, newX);
                            this.rightSliderPosition = newX;
                        } else {
                            naviobj.setSliderPositions(null, null, this.leftSliderPosition);
                            this.rightSliderPosition = this.leftSliderPosition;
                        }
                        //naviobj.setSliderPositions(null, null, naviobj.leftSlider.getCTM().e);
                        if (naviobj.diff >= 1 || naviobj.diff <= -1)
                            naviobj.switched = false;
                    }
                    naviobj.right = newX;
                }
                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                    naviobj.grabbed = false;
                    naviobj.setSliderPositions(null, parseFloat(newX), null);
                    if (newX > newX + this.renderer._getAttrVal($(naviobj.dragTarget), 'width'))
                        naviobj.setSliderPositions(null, newX, null);
                    naviobj.center = newX;
                } 
            }
        },


        _drop: function (evt) {
            // if we aren't currently dragging an element, don't do anything
            if (this.dragTarget) {
                // since the element currently being dragged has its pointer-events turned off,
                //    we are afforded the opportunity to find out the element it's being dropped on
                var targetElement = evt.target;

                var naviobj = this;
                // set the global variable to null, so nothing will be dragged until we
                //    grab the next element
                if (naviobj.model.allowSnapping && parseInt(naviobj.leftSlider.getBoundingClientRect().left+2) < parseInt( naviobj.rightSlider.getBoundingClientRect().left)) {
                    naviobj.ismouseup = true;
                    if (naviobj.dragTarget.parentNode.id === "leftslider")
                        var txtPosition = naviobj.left;
                    else if (naviobj.dragTarget.parentNode.id === "rightslider")
                        var txtPosition = naviobj.right;
                    else if (naviobj.dragTarget.parentNode.id === "centerslider")
                        var txtPosition = naviobj.center;
                    var pos = 0;

                    for (var i = 0; i < naviobj._lowerLineLeft.length; i++) {
                        if (i + 1 < naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i] && txtPosition < naviobj._lowerLineLeft[i + 1]) {
                            naviobj.snapValue = naviobj._lowerLineLeft[i];
                            pos = (naviobj._lowerLineLeft[i + 1] - naviobj.snapValue);
                            if (txtPosition >= (naviobj.snapValue + (pos / 2))) {
                                naviobj.snapValue = naviobj._lowerLineLeft[i + 1];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i + 1]);
                                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i+1] + parseFloat(naviobj.centerSlider.getAttribute("width")));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i + 1], null, null);
                                }

                            }
                            else {
                                naviobj.snapValue = naviobj._lowerLineLeft[i];
                                if (naviobj.dragTarget.parentNode.id === "leftslider")
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                else if (naviobj.dragTarget.parentNode.id === "rightslider")
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i]);
                                else if (naviobj.dragTarget.parentNode.id === "centerslider") {
                                    naviobj.setSliderPositions(null, null, naviobj._lowerLineLeft[i] + parseFloat(naviobj.centerSlider.getAttribute("width")));
                                    naviobj.setSliderPositions(naviobj._lowerLineLeft[i], null, null);
                                }

                            }
                        }
                        else if (i + 1 === naviobj._lowerLineLeft.length && txtPosition > naviobj._lowerLineLeft[i]) {
                            naviobj.setSliderPositions(null, null,naviobj.newWidth-naviobj.padding);
                        }
                    }
                }
                this.dragTarget = null;
               
                if (this.leftTooltip != undefined && this.model.tooltipSettings.tooltipDisplayMode=="always") {
                    this.leftTooltip.setAttribute('opacity', 1);
                    this.rightTooltip.setAttribute('opacity', 1);
                }
                else if (this.leftTooltip != undefined && this.model.tooltipSettings.tooltipDisplayMode == "ondemand") {
                    this.leftTooltip.setAttribute('opacity', 0);
                    this.rightTooltip.setAttribute('opacity', 0);
                }
                if (this.model.enableDeferredUpdate) {
                    this._calculateSelectedData();
                    if (!naviobj.grabbed) {
                        if (this.model.valueType == "datetime") {
                            if (this.startDateTime.getTime() !== this.model.selectedRangeSettings.start.getTime() || this.endDateTime.getTime() !== this.model.selectedRangeSettings.end.getTime()) 
                                this._trigger("rangeChanged", this.model);
                        }
                        else {
                            if (this.startValue !== this.model.selectedRangeSettings.start || this.endValue !== this.model.selectedRangeSettings.end)
                                this._trigger("rangeChanged", this.model);
                        }                       
                        
                    }
                }
            }
            this._setSelectedRange();
        },

        mouseup: function (evt) {
            if (this.dragTarget != null) {
                this._drop(evt);
                if (this.dragTarget != null) {
                    this.dragTarget.setAttributeNS(null, 'pointer-events', 'all');
                    this.dragTarget = null;
                }
                if (this.model.enableDeferredUpdate) {
                    this._calculateSelectedData();
                    this._trigger("rangeChanged", this.model);
                  
                }
            }
        },

        _setSelectedRange: function () {           
             
           this._selectedRangeStart(this.model.selectedRangeSettings.start);
           this._selectedRangeEnd(this.model.selectedRangeSettings.end);
          
        },

        GetTrueCoords: function (evt) {
            // find the current zoom level and pan setting, and adjust the reported
            //    mouse position accordingly
            var newScale = this.svgDocument.currentScale;
            var translation = this.svgDocument.currentTranslate;
            var matched = jQuery.uaMatch(navigator.userAgent);
            if (evt.originalEvent.toString() !== "[object TouchEvent]" && matched.browser.toLowerCase() != "msie") {
                this.trueCoords.x = (evt.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.clientY - translation.y) / newScale;
            }
            else if (matched.browser.toLowerCase() == "msie") {
                this.trueCoords.x = (evt.originalEvent.clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.clientY - translation.y) / newScale;
            }
            else {
                this.trueCoords.x = (evt.originalEvent.touches[0].clientX - translation.x) / newScale;
                this.trueCoords.y = (evt.originalEvent.touches[0].clientY - translation.y) / newScale;
            }
        }
    });

	 ej.datavisualization.RangeNavigator.locale = {};
     ej.datavisualization.RangeNavigator.locale["en-US"] = {

        intervals: {
            quarter: {longQuarters: "Quarter ", shortQuarters: "Q"},
            week: {longWeeks: "Week ", shortWeeks: "W"}
        }
     };
     ej.datavisualization.RangeNavigator.IntervalType = {

         Years: 'years',

         Quarters: 'quarters',

         Months: 'months',

         Weeks: 'weeks',

         Days: 'days',

         Hours: 'hours'
     };

     ej.datavisualization.RangeNavigator.Position = {

         Top: 'top',        

         Bottom: 'bottom'      

         
     };

     ej.datavisualization.RangeNavigator.FontStyle = {

         Normal: 'normal',

         Italic: 'italic',

         Bold: 'bold'
     };

     ej.datavisualization.RangeNavigator.FontWeight = {

         Regular: 'regular',

         Bold: 'bold',

         Lighter: 'lighter'
     };

     ej.datavisualization.RangeNavigator.HorizontalAlignment = {

         Left: 'left',

         Right: 'right',

         Middle: 'middle'
     };
     ej.datavisualization.RangeNavigator.LabelPlacement = {

         Inside: 'inside',

         Outside: 'outside'
      
     };    

     ej.datavisualization.RangeNavigator.ValueType = {

         Numeric: 'numeric',

         Datetime: 'datetime'

     };

     ej.datavisualization.RangeNavigator.RangePadding = {

         Additional: 'additional',

         Normal: 'normal',

         None: 'none',

         Round: 'round'        

     };
	 
	  ej.datavisualization.RangeNavigator.Theme = {

        Azure: 'azure',

        FlatLight: 'flatlight',

        Azuredark: 'azuredark',

        Lime: 'lime',

        LimeDark: 'limedark',

        Saffron: 'saffron',

        SaffronDark: 'saffrondark',

        GradientLight: 'gradientlight',

        GradientDark: 'gradientdark'
    };



})(jQuery, Syncfusion);

jQuery.uaMatch = function (ua) {
    ua = ua.toLowerCase();

    var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
        /(webkit)[ \/]([\w.]+)/.exec(ua) ||
        /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
        /(msie) ([\w.]+)/.exec(ua) ||
        ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
        [];

    return {
        browser: match[1] || "",
        version: match[2] || "0"
    };
};;
/**
* @fileOverview Plugin to style the Html Map elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/

(function ($, ej, undefined) {
    
    ej.widget({ "ejMap": "ej.datavisualization.Map" }, {

        validTags: ["div"],
        _initPrivateProperties: function () {
            this._rootgroup = null;
            this._bubblegroup = null;           
            this._scale = 1;
            this._prevDistance = 0;
            this._tileTranslatePoint = { x: 0, y: 0 };
            this._translatePoint = { x: 0, y: 0 };
            this._prevPoint = null;
            this._Tiles = [];
            this._prevScale = 0;
            this._tileDiv = null;
            this._containerWidth = 500;
            this._containerHeight = 400;
            this._baseTranslatePoint = { x: 0, y: 0 };
            this._isDragging = false;           
            this._prevLevel = 1;
            this._startPoint = { x: 0, y: 0 };
            this._stopPoint = { x: 0, y: 0 };
            this.mouseClickable = false;
            this._browser = null;
            this._baseScale = 0;
            this._mapBounds = null;
            this._svgns = "http://www.w3.org/2000/svg";
            this._ispanning = false;
            this._dragStartX = 0;
            this._isNavigationPressed = false;
			this._iskeyboardKeysPressed = false;
            this._isPolygonSelected = false;
            this._dragStartY = 0;
            this._width = 350;
            this._height = 350;
            this._margintop = 0;
            this._marginleft = 0;
            this._svgDocument = null;
            this._tooltipElement = null;
            this._templateDiv = null;
            this._legendDiv = null;
            this._mapContainer = null;
            this._isSVG = true;
            this._VMLPathFractionCount = 0;
            this._sliderControl = null; 
            this._isTileMap=false;
            this._isRendered = false;
            this._urlTemplate = null;
            this._polylineCount = 0;
            this._pointscount = 0;
            this._isPinching = false;
            this._groupSize = null;			
			this._groupBorder = { x: 0, y: 0 };
        },


        defaults: {
            background: "white",			
            zoomSettings:{              
                minValue: 1,
                maxValue: 100,
                factor: 1,
                level: 1,
                enableZoomOnSelection: false,
                enableZoom: true,
            },          
            centerPosition: null,           
            enableResize: true,
            enableAnimation: false,           
            draggingOnSelection: false,
            navigationControl:{			
				enableNavigation:false,						
				content:null,
				orientation:'vertical',			
				absolutePosition:[0,0],								
				dockPosition:'centerleft',
			},           
            enableLayerChangeAnimation: false,
            enablePan: true,           
            baseMapIndex: 0,
            shapeSelected: "",
            markerSelected: "",
            zoomedIn: "",
            onRenderComplete: "",
            panned: "",            
            zoomedOut: "",           
            mouseover : "",
            mouseleave : "",                                  
            layers: []
        },

        observables: [            
            "baseMapIndex",
            "enablePan",
            "enableResize",
            "enableAnimation",
            "zoomSettings.level",
			"zoomSettings.minValue",
            "zoomSettings.maxValue",
            "zoomSettings.factor",
            "zoomSettings.enableZoom",
			"zoomSettings.enableZoomOnSelection",
            "navigationControl.enableNavigation",
            "navigationControl.orientation",
            "navigationControl.absolutePosition",
            "navigationControl.dockPosition",            
        ],

		_tags: [{
            tag: "layers",
            attr: ["legendSettings.showLegend", "legendSettings.positionX", "legendSettings.positionY", "legendSettings.type", "legendSettings.labelOrientation",
                    "legendSettings.title", "legendSettings.mode", "legendSettings.position", "legendSettings.dockOnMap", "legendSettings.dockPosition",
                    "legendSettings.leftLabel", "legendSettings.rightLabel", "bubbleSettings.showBubble", "bubbleSettings.valuePath", "bubbleSettings.minValue",
                    "bubbleSettings.maxValue", "bubbleSettings.color", "enableSelection", "enableMouseHover", "showTooltip", "showMapItems", "mapItemsTemplate", "shapeData",
                    "dataSource", "shapePropertyPath", "shapeDataPath", "layerType", "bingMapType", "urltemplate", "shapeSettings.highlightColor",
                    "shapeSettings.highlightBorderWidth", "shapeSettings.selectionColor", "shapeSettings.fill", "shapeSettings.strokeThickness", "shapeSettings.selectionStrokeWidth",
                    "shapeSettings.stroke", "shapeSettings.selectionStroke", "shapeSettings.highlightStroke", "shapeSettings.colorValuePath", "shapeSettings.valuePath",
                    "shapeSettings.autoFill", "shapeSettings.enableGradient",
                   
			        [{
	                    tag: "markers",
	                    attr: ["label", "latitude", "longitude"],
	                    singular: "marker",
			        },
                    {
                        tag: "bubbleSettings.colorMappings.rangeColorMapping",
                        attr: ["from", "to", "color"],
                        singular: "bubblerangeColorMap",
                    },
                    {
                        tag: "bubbleSettings.colorMappings.equalColorMapping",
                        attr: ["value", "color"],
                        singular: "bubbleequalColorMap",
                    },
                    {
                        tag: "shapeSettings.colorMappings.rangeColorMapping",
                        attr: ["from", "to", "color"],
                        singular: "shaperangeColorMap",
                    },
                    {
                        tag: "shapeSettings.colorMappings.equalColorMapping",
                        attr: ["value", "color"],
                        singular: "shapeequalColorMap",
                    },

			        {
	                    tag: "subLayers",
	                    attr: ["legendSettings.showLegend", "legendSettings.positionX", "legendSettings.positionY", "legendSettings.type", "legendSettings.labelOrientation",
                                "legendSettings.title", "legendSettings.mode", "legendSettings.position", "legendSettings.dockOnMap", "legendSettings.dockPosition",
                                "legendSettings.leftLabel", "legendSettings.rightLabel", "bubbleSettings.showBubble", "bubbleSettings.valuePath", "bubbleSettings.minValue",								
                                "bubbleSettings.maxValue", "bubbleSettings.color", "enableSelection", "enableMouseHover", "showTooltip", "showMapItems", "mapItemsTemplate", "shapeData",
								"dataSource", "shapePropertyPath", "shapeDataPath","layerType", "bingMapType", "key", "urltemplate", "shapeSettings.highlightColor",
								"shapeSettings.highlightBorderWidth", "shapeSettings.selectionColor","shapeSettings.fill", "shapeSettings.strokeThickness", "shapeSettings.selectionStrokeWidth",
								"shapeSettings.stroke", "shapeSettings.selectionStroke","shapeSettings.highlightStroke", "shapeSettings.colorValuePath", "shapeSettings.valuePath", 
								"shapeSettings.autoFill", "shapeSettings.enableGradient",
                                [{
                                    tag: "markers",
                                	attr: ["label", "latitude", "longitude"],
                                	    singular: "marker",
                                },
                                {
                                    tag: "bubbleSettings.colorMappings.rangeColorMapping",
                                    attr: ["from", "to", "color"],
                                    singular: "bubblerangeColorMap",
                                },
                                {
                                    tag: "bubbleSettings.colorMappings.equalColorMapping",
                                    attr: ["value", "color"],
                                    singular: "bubbleequalColorMap",
                                },
                                {
                                    tag: "shapeSettings.colorMappings.rangeColorMapping",
                                    attr: ["from", "to", "color"],
                                    singular: "shaperangeColorMap",
                                },
                                {
                                    tag: "shapeSettings.colorMappings.equalColorMapping",
                                    attr: ["value", "color"],
                                    singular: "shapeequalColorMap",
                                }
                               ],
	                    ],
	                    singular: "subLayer",
			        }]
                ],
                singular: "layer",
        }],
		
        enableZoom: ej.util.valueFunction("zoomSettings.enableZoom"),
        enableZoomOnSelection: ej.util.valueFunction("zoomSettings.enableZoomOnSelection"),
        zoomLevel: ej.util.valueFunction("zoomSettings.level"),
        minZoom: ej.util.valueFunction("zoomSettings.minValue"),
        zoomFactor: ej.util.valueFunction("zoomSettings.factor"),
        maxZoom: ej.util.valueFunction("zoomSettings.maxValue"),
        baseMapIndex: ej.util.valueFunction("baseMapIndex"),
        enablePan: ej.util.valueFunction("enablePan"),
        enableResize: ej.util.valueFunction("enableResize"),
        enableAnimation: ej.util.valueFunction("enableAnimation"),
        enableNavigation: ej.util.valueFunction("navigationControl.enableNavigation"),
        orientation: ej.util.valueFunction("navigationControl.orientation"),
        absolutePosition: ej.util.valueFunction("navigationControl.absolutePosition"),
        dockPosition: ej.util.valueFunction("navigationControl.dockPosition"),
       
        dataTypes: {
            layers: "array"
        },       

        _destroy: function () {            
            this._unWireEvents();
            if (_isSVG) {
                $(this.element).removeClass("e-datavisualization-map e-js").find("#svgDocument").remove();
            }
            else {
                $(this.element).removeClass("e-datavisualization-map e-js").find("#rootGroup").remove();
            }
        },
       
        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "zoomSettings":
                        if (this.enableZoom()) {                            
                            this.zoom(this.zoomLevel());                            
                        }
                        break;
                    case "baseMapIndex":
                        this.baseMapIndex(options[prop]);
                        var map = this;
                        if (this.enableAnimation()) {
                            this.model.enableLayerChangeAnimation = true;
                            $(this._mapContainer).animate({ opacity: 0 }, 500, function () {
                                map.refresh();
                            });

                        } else {
                            this.refresh();
                        }

                        break;                    
                    case "background":                        
                        $(this._mapContainer).css('background-image', this.model.background);
                }
            }
        },
		
	_layers: function(index, property, value, old){
	if (property == "shapeSettings.fill" || property == "shapeSettings.strokeThickness") {
                this.clearShapeSelection();
            }
		else if(property=="showMapItems" || property=="dataSource")
		 {
			this.refreshLayers();
		 }
		 else 
		 {
		   this.refresh();
		 }
    },

    _layers_markers: function(index, property, value, old){
    	   this.refreshMarkers();
    },

    _layers_subLayers: function(index, property, value, old){
    	  if(property=="showMapItems" || property=="dataSource")
		 {
			this.refreshLayers();
		 }
		 else 
		 {
		   this.refresh();
		 }
    },
       
        _init: function () {
            var proxy = this;
			this._navigationStyle = null;
            this.navigationControlData = null;
            this._initPrivateProperties();
            $.each(this.model.layers, function (index) {
                proxy._layerInitialize(index);
            });
            this._mapContainer = this.element;
            $(this._mapContainer).css({ "overflow": "hidden" });
			  if (document.documentMode == 8) {
                $(this._mapContainer).css({ "overflow": "hidden" , "position" :"relative" });
            }
            this.refresh();
            this._trigger("onRenderComplete");
            this._isRendered = true;
            if (_isSVG) {
                $(this._svgDocument).pinchZoom(this._rootgroup, this);
            }
        },

        _layerInitialize: function (layerindex) {
            var proxy = this;
            if (this.model.layers[layerindex] != null) {
                $.each(this.model.layers, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = new shapeLayer();
                    $.extend(obj, element);
                    $.extend(element, obj);
                    $.each(element.subLayers, function (subindex, subelement) {
                        subelement = proxy._checkArrayObject(subelement, subindex);
                        var subobj = new shapeLayer();
                        $.extend(subobj, subelement);
                        $.extend(subelement, subobj);
                    });
                });
            }
            else {

                this.layers[0] = new shapeLayer();

            }
        },
       
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = new shapeLayer();
                    proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },
       
        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },
     
        _refreshWithAnimation: function () {
            this.model.layers[this.baseMapIndex()]._setMapItemsPositionWithAnimation(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._setMapItemsPositionWithAnimation(this);
            }
        },
       
        _resizeShape: function () {
            this.model.layers[this.baseMapIndex()]._resizeShapes(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._resizeShapes(this);
            }

        },
        
        _refrshLayers: function () {
            this.model.layers[this.baseMapIndex()]._setMapItemsPosition(this);
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                this.model.layers[this.baseMapIndex()].subLayers[key]._setMapItemsPosition(this);
            }

        },
       
        _panTileMap: function (factorX, factorY, centerPosition) {
            
            var totalSize = Math.pow(2, this.zoomLevel()) * 256;
            this._tileTranslatePoint.x = (factorX / 2) - (totalSize / 2);
            this._tileTranslatePoint.y = (factorY / 2) - (totalSize / 2);
            var position = this._convertTileLatLongtoPoint(centerPosition[0],centerPosition[1]);
            this._tileTranslatePoint.x -= position.x - (factorX / 2);
            this._tileTranslatePoint.y -= position.y - (factorY / 2);

        },
       
        _generateTiles: function (zoomLevel) {
            var userLang = navigator.language || navigator.userLanguage;
            this.Tiles = [];
            var xcount, ycount;
            xcount = ycount = Math.pow(2, zoomLevel);
            var baseLayer = this.model.layers[this.baseMapIndex()];
            var endY = Math.min(ycount, ((-this._tileTranslatePoint.y + this._height) / 256) + 1);
            var endX = Math.min(xcount, ((-this._tileTranslatePoint.x + this._width) / 256) + 1);
            var startX = (-(this._tileTranslatePoint.x + 256) / 256);
            var startY = (-(this._tileTranslatePoint.y + 256) / 256);
            for (var i = Math.round(startX) ; i < Math.round(endX) ; i++) {
                for (var j = Math.round(startY) ; j < Math.round(endY) ; j++) {
                    var x = 256 * i + this._tileTranslatePoint.x;
                    var y = 256 * j + this._tileTranslatePoint.y;
                    if (x > -256 && x <= this._width && y > -256 && y < this._height) {
                        if (i >= 0 && j >= 0) {
                            var tile = new Tile(i, j);
                            tile.left = x;
                            tile.top = y;
                            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                                tile.src = this._getBingMap(tile, baseLayer.key, baseLayer.bingMapType, userLang);
                            } else {
                                tile.src = this._urlTemplate.replace("level", this.zoomLevel()).replace('tileX', tile.X).replace('tileY', tile.Y);
                            }
                            this.Tiles.push(tile);
                        }
                    }
                }
            }
            var proxTiles = $.extend(true, [], this.Tiles);
            for (var layerIndex = 0; layerIndex < this.model.layers[this.baseMapIndex()].subLayers.length;layerIndex++) {
                var layer = this.model.layers[this.baseMapIndex()].subLayers[layerIndex];
                if (layer.layerType == ej.datavisualization.Map.LayerType.OSM || layer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                    for (var tileIndex = 0; tileIndex < proxTiles.length; tileIndex++) {
                        var baseTile = proxTiles[tileIndex];
                        var subtile = $.extend(true, {}, baseTile);
                        if (layer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                            subtile.src = this._getBingMap(subtile, layer.key, layer.bingMapType, userLang);
                        } else {
                            subtile.src = layer.urlTemplate.replace("level", this.zoomLevel()).replace('tileX', baseTile.X).replace('tileY', baseTile.Y);
                        }
                        this.Tiles.push(subtile);
                    }

                }
            }
            this._arrangeTiles();
        },
       
        _getBingMap:function(tile,key,type,lang) {
            var quadKey = "";
            for (var i = this.zoomLevel(); i > 0; i--)
            {
                var digit = 0;
                var mask = 1 << (i - 1);
                if ((tile.X & mask) != 0)
                {
                    digit++;
                }
                if ((tile.Y & mask) != 0)
                {
                    digit+=2;
                }
                quadKey = quadKey+""+digit;
            }
            var layerType;
            if (type == ej.datavisualization.Map.BingMapType.Aerial) {
                layerType = "A,G";
            }
            else if (type == ej.datavisualization.Map.BingMapType.AerialWithLabel) {
                layerType = "A,G,L";
            } else {
                layerType = "G,VE,BX,L,LA";
            }
            var imageUrl = "http://ak.dynamic.t2.tiles.virtualearth.net/comp/ch/" + quadKey + "?mkt=" + lang + "&ur=IN&it=" + layerType + "&shading=hill&og=45&n=z&Key=" + key;
            return imageUrl;
        },
               
        _arrangeTiles: function (){        
            var pathTemplate = "<div><div style='position:absolute;left:{{:left}}px;top:{{:top}}px;height:{{:height}}px;width:{{:width}}px;'><img src={{:src}}></img></div></div>";
            var tmpl2 = $.templates(pathTemplate);
            var htmlString = tmpl2.render(this.Tiles);
            var first = this._mapContainer[0].children[0];            
            this._tileDiv.html(htmlString);            
        },

        _generatePath: function () {
           
            var baseLayer = this.model.layers[this.baseMapIndex()];            
            var pathCollection = '';
            this._polylineCount = 0;
            this._pointscount = 0;
            _isSVG = (window.SVGSVGElement) ? true : false;

            
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Geometry) {
                if (baseLayer != undefined && baseLayer.shapeData != null) {
                    baseLayer._isBaseLayer = true;
                    this._isTileMap = false;
					this._scale = this.zoomLevel();
                    pathCollection = this._readShapeData(baseLayer);
                }
            }
            else {                
                this._isTileMap = true;
                this._scale = Math.pow(2, this.zoomLevel() - map.zoomFactor());
            }
            for (var key = 0; key < baseLayer.subLayers.length; key++) {
                if (baseLayer.subLayers[key].layerType == ej.datavisualization.Map.LayerType.Geometry && baseLayer.subLayers[key].shapeData != null) {
                    baseLayer.subLayers[key]._isBaseLayer = false;
                    pathCollection += this._readShapeData(baseLayer.subLayers[key]);
                }
            }
            if (_isSVG) {
                var htmlstring = '<div style="position:relative;" id="tileDiv"/><svg xmlns="http://www.w3.org/2000/svg" style= "overflow:hidden;z-index:0;float:left;left:0,top:0"' +
                             'id="svgDocument"> <g id="rootGroup">' + pathCollection + '</g></svg>';
                this._mapContainer.html(htmlstring);               
                this._svgDocument = this.element.find("#svgDocument")[0];
                this._tileDiv = this.element.find("#tileDiv");
                
            }
            else {
                document.createStyleSheet().addRule(".vml", "behavior:url(#default#VML);display:inline-block");
                document.namespaces.add('v', 'urn:schemas-microsoft-com:vml', "#default#VML");
                var vmlGroup = '<div id="tileDiv"/><v:group   id = "rootGroup" style="position:absolute; width:' + this._width + 'px;height:' + this._height + 'px;" coordorigin = "0 0" coordsize="' + this._width + ' ' + this._height + '">' + pathCollection + '</v:group>';
                this._mapContainer.html(vmlGroup);                
                this._tileDiv = this.element.find("#tileDiv");
                this._bubblegroup = this._createGroup(false,"bubbleGroup");               
                this._bubblegroup.style.position = "absolute";
                this.element.append(this._bubblegroup);
            }
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.OSM || baseLayer.layerType == ej.datavisualization.Map.LayerType.Bing) {
                this._urlTemplate = baseLayer.urlTemplate;
                if (this.model.centerPosition != null) {
                    this._panTileMap(this._width, this._height, this.model.centerPosition);
                }
                else {
                    this._panTileMap(this._width, this._height, [0, 0]);
                }
                this._generateTiles(this.zoomLevel());
				if(_isSVG)
				{
				  $(this._svgDocument).css("position","relative");
				}
            }
            if(_isSVG)
			{			
                $(this._mapContainer).css("position", "relative");
			}			
            this._rootgroup = this.element.find("#rootGroup")[0];

            this._on($(this.element), ej.eventType.mouseDown, { map: this }, this.dragDown);
            this._on($(this.element), ej.eventType.mouseMove, { map: this }, this.dragMove);
            this._on($(this.element), ej.eventType.mouseUp, { map: this }, this.dragUp);

        },

        _calculateBBox: function (polygonDatas, polylineDatas,pointData)
        {
            var minLatitude = maxLatitude = minLongitude = maxLongitude = 0;
            var isEntered = false;
            if (polygonDatas != null && polygonDatas.length > 0) {
                for (var i = 0; i < polygonDatas.length; i++) {
                    var currentData = polygonDatas[i];
                    if (currentData.geometry != null) {
                        coords = currentData.geometry.coordinates;
                    } else {
                        coords = currentData.coordinates;
                    }
                    var currentDataLength = coords.length;
                    for (var j = 0; j < currentDataLength; j++) {
                        var subData = '';
                        if (currentDataLength > 1) {
                            subData = coords[j][0];
                            if (subData.length <= 2) {
                                subData = coords[j];
                            }
                        } else {
                            subData = coords[j];
                        }
                        if (subData.length > 2) {
                            for (var k = 0; k < subData.length; k++) {
                              if(subData[k][1]!=null && subData[k][0]!=null){
                                if (!isEntered) {
                                    minLatitude = maxLatitude = subData[k][1];
                                    minLongitude = maxLongitude = subData[k][0];
                                    isEntered = true;

                                } else {
                                    minLatitude = Math.min(minLatitude, subData[k][1]);
                                    maxLatitude = Math.max(maxLatitude, subData[k][1]);
                                    minLongitude = Math.min(minLongitude, subData[k][0]);
                                    maxLongitude = Math.max(maxLongitude, subData[k][0]);
                                }
                                }
                            }
                        } else {
                          if(subData[k][1]!=null && subData[k][0]!=null){
                            if (!isEntered) {
                                minLatitude = maxLatitude = subData[1];
                                minLongitude = maxLongitude = subData[0];
                                isEntered = true;
                            } else {
                                minLatitude = Math.min(minLatitude, subData[1]);
                                maxLatitude = Math.max(maxLatitude, subData[1]);
                                minLongitude = Math.min(minLongitude, subData[0]);
                                maxLongitude = Math.max(maxLongitude, subData[0]);
                            }
                           }
                        }
                    }
                }
            }
            if (pointData!=null && pointData.length > 0) {
                for (var i = 0; i < pointData.length; i++) {
                    var currentData = pointData[i];
                    subData = currentData.geometry.coordinates;
                    if (!isEntered) {
                        minLatitude = maxLatitude = subData[1]; // (φ)
                        minLongitude = maxLongitude = subData[0];
                        isEntered = true;
                    } else {
                        minLatitude = Math.min(minLatitude, subData[1]);
                        maxLatitude = Math.max(maxLatitude, subData[1]);
                        minLongitude = Math.min(minLongitude, subData[0]);
                        maxLongitude = Math.max(maxLongitude, subData[0]);
                    }
                }
            }
            if (polylineDatas!=null && polylineDatas.length > 0) {
                for (var i = 0; i < polylineDatas.length; i++) {
                    var coordinates = polylineDatas[i].geometry.coordinates;
                    for (var k = 0; k < coordinates.length; k++) {
                        if (!isEntered) {
                            minLatitude = maxLatitude = coordinates[k][1]; // (φ)
                            minLongitude = maxLongitude = coordinates[k][0];
                            isEntered = true;
                        } else {
                            minLatitude = Math.min(minLatitude, coordinates[k][1]);
                            maxLatitude = Math.max(maxLatitude, coordinates[k][1]);
                            minLongitude = Math.min(minLongitude, coordinates[k][0]);
                            maxLongitude = Math.max(maxLongitude, coordinates[k][0]);
                        }
                    }
                }
            }
            return [[minLongitude, minLatitude], [maxLongitude, maxLatitude]];        
        },
        		
        _readShapeData: function (layer) {
            var map = this;
            if (layer.shapeData != null) {
                if (typeof layer.shapeData == 'string') {
                    var dataManger =new ej.DataManager(layer.shapeData);
                    var query = ej.Query().from(layer);
                    dataManger.executeQuery(query).done(function (e) {
                        if (e.result != null) {
                            return map._getPathCollection(e.result, layer);
                        }
                    });

                } else {
                 return this._getPathCollection(layer.shapeData, layer);
                }
                
            }
        },
       
        _getFactor:function()
        {
            var contHeight;
            var contWidth;
            var heightfactor;
            var widthfactor
            if (this._mapBounds != null) {
                var startpoint = this._convertTileLatLongtoPointForShapes(this._mapBounds[0][1], this._mapBounds[0][0], this._mapBounds);
                var endpoint = this._convertTileLatLongtoPointForShapes(this._mapBounds[1][1], this._mapBounds[1][0], this._mapBounds);
                var minDiff = 0;
                var widthfactor = heightfactor = 1;
                contHeight = endpoint.y - startpoint.y;
                contWidth = endpoint.x - startpoint.x;
            }
            else {
                contHeight = 500;
                contWidth = 500;
            }
            if (contHeight < this._height) {
                heightfactor = parseFloat(Math.abs(this._height / contHeight + "e+1").toString().split('.')[0] / 10);
            }
            if (contWidth < this._width) {
                widthfactor = parseFloat(Math.abs(this._width / contWidth + "e+1").toString().split('.')[0] / 10);
            }

            return Math.min(widthfactor, heightfactor);
        },

        _getPathCollection:function(data,layer)
        {
            var shapePath = "";
            var type = "";
            var dataManager="";
            if (data.features != null) {
                dataManager = new ej.DataManager(data.features);
                type = "geometry.type";
                if (layer._isBaseLayer) {
                    this._mapBounds = data.bbox;
                }
            }
            else if (data.geometries != null) {
                dataManager = new ej.DataManager(data.geometries);
                type = "type";
                if (layer._isBaseLayer) {
                    if (data.bbox != null)
                        this._mapBounds = [[data.bbox[0], data.bbox[1]], [data.bbox[2], data.bbox[3]]];
                    }                
            }
			if(data.geometries!=null || data.features != null){
            var queryPolyLine = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'LineString');
            var queryPolygon = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'Polygon');
            var queryMulitiPolygon = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'MultiPolygon');
            var polylineDatas = dataManager.executeLocal(queryPolyLine);           
            var polygonDatas = dataManager.executeLocal(queryPolygon);
            var queryPoint = ej.Query().from(layer).where(type, ej.FilterOperators.equal, 'Point');
            var pointData = dataManager.executeLocal(queryPoint);
            var multiPolygonDatas = dataManager.executeLocal(queryMulitiPolygon);
            this._polylineCount = this._polylineCount + polylineDatas.length;
            this._pointscount = this._pointscount + pointData.length;
            $.merge(polygonDatas, multiPolygonDatas);
            if (this._mapBounds == null && layer._isBaseLayer) {
                this._mapBounds = this._calculateBBox(polygonDatas, polylineDatas, pointData);
            }
			else if (this._mapBounds == null)
                   this._mapBounds = [[-180, -85], [180, 85]] ;

            var factor = this._getFactor();

			layer._newBounds = [];                 
            var pointHtml = "";
            if (pointData.length > 0) {
                for (var i = 0; i < pointData.length; i++) {
                    var currentData = pointData[i];                   
                    var check = '';
                    subData = currentData.geometry.coordinates;
                    var data = [];
                    var data_clean = [];                   
                    data.push({
                        lat: parseFloat(subData[1]),
                        lng: parseFloat(subData[0])                        
                    });                    
                    var l = 0;

                    var point = data[0];
                    latitude = point.lat; // (φ)
                    longitude = point.lng;   // (λ)                   
                    var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds,factor);
                    if (_isSVG) {
                        data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                    }
                    else {
                        data_clean.push({ x: p.x - ((10 / this._scale)/2), y: p.y-((10 / this._scale) / 2), lat: point.lat, lng: point.lng });
                    }

                    if (_isSVG) {
                        pointHtml += '<circle cx="' + data_clean[0].x + '" cy="' + data_clean[0].y + '" r="' + 5 / this._scale + '" stroke="' + layer.shapeSettings.stroke + '" stroke-width="' + layer.shapeSettings.strokeThickness + '0" fill="' + layer.shapeSettings.fill + '" />';
                    }
                    else {
                        var pointHtmlString = '<v:oval display="block" fill="' + layer.shapeSettings.fill + '" style="position:absolute;top: ' + data_clean[0].y + 'px;left:' + data_clean[0].x + 'px;width:' + 10 / this._scale + 'px;height:' + 10 / this._scale + 'px;stroke-width:' + layer.shapeSettings.strokeThickness + 'px;"></v:oval>';
                        pointHtml += pointHtmlString;
                    }
                }

            }            
            if (polygonDatas.length > 0)
            {
                var minX, maxX, minY, maxY,flag=true;
                for (var i = 0; i < polygonDatas.length; i++) {
                    var currentData = polygonDatas[i];
                    var midCoordinate = 0;
                    var midCoordinateLength;
                    var coords;
                    if (currentData.geometry != null) {
                        coords = currentData.geometry.coordinates;
                    }
                    else {
                        coords = currentData.coordinates;
                    }                    
                    midCoordinateLength = coords[0][0].length;
                    var currentDataLength = coords.length;
                    if (currentDataLength > 1) {
                        var lst = coords;
                        for (var index = 0; index < lst.length; index++) {
                            if (midCoordinateLength < lst[index][0].length) {
                                midCoordinateLength = lst[index][0].length;
                                midCoordinate = index;
                            }
                        }
                    }
                    var path_code = '';
                    for (var j = 0; j < currentDataLength; j++) {
                        var subData = '';                       
                        if (currentDataLength > 1) {
                            subData = coords[j][0];
                            if (subData.length <=2) {
                                subData = coords[j];
                            }
                        }
                        else {
                            subData = coords[j];
                        }
                        var data = [];
                        var data_clean = [];
                        if (subData.length > 2) {
                            for (var k = 0; k < subData.length; k++) {
                                data.push({
                                    lat: parseFloat(subData[k][1]),
                                    lng: parseFloat(subData[k][0])
                                });
                            }
                        }
                        else {
                            data.push({
                                lat: parseFloat(subData[1]),
                                lng: parseFloat(subData[0])
                            });
                        }                       
                        for (var l = 0; l < data.length - 1|| (data.length==1 && l==0); l++) {
                            var point = data[l];
                            latitude = point.lat; // (φ)
                            longitude = point.lng;   // (λ                           
                            var boundingBox = this._mapBounds;
                            if (this._isTileMap||(longitude >= boundingBox[0][0] && longitude <= boundingBox[1][0] && latitude >= boundingBox[0][1] && latitude <= boundingBox[1][1])) {
                               
                                var p = this._convertTileLatLongtoPointForShapes(latitude, longitude, boundingBox, factor);
                                if (flag) {
                                    minX = maxX = p.x;
                                    minY = maxY = p.y;
                                    flag = false;
                                }
                                else {
                                    minX = Math.min(minX, p.x);
                                    maxX = Math.max(maxX, p.x);
                                    minY = Math.min(minY, p.y);
                                    maxY = Math.max(maxY, p.y);
                                }                                
                                if (_isSVG) {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                                else {
                                    data_clean.push({ x: p.x, y: p.y, lat: point.lat, lng: point.lng });
                                }
                            }
                            
                        }
                        if (j == midCoordinate) {
                            layer._newBounds.push(this._findMidPointofPoylgon(data_clean));
                        }
                        
                        if (data_clean.length > 0) {
                            if (_isSVG) {
                                path_code += "M" + (data_clean[0].x) + "," + (data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + (data_clean[m].x) + "," + (data_clean[m].y);
                                }
                                path_code += "Z";
                            }
                            else {
                                path_code += "m" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                path_code += " l" + parseInt(data_clean[0].x) + "," + parseInt(data_clean[0].y);
                                for (m = 1; m < data_clean.length; m++) {
                                    path_code += "," + parseInt(data_clean[m].x) + "," + parseInt(data_clean[m].y);
                                }
                                path_code += " e";
                            }
                        }

                    }
                    if (path_code != "") {
                        if (_isSVG) {
                            shapePath += '<path class="mapShape" stroke=' + layer.shapeSettings.stroke +
                                        ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) +
                                         ' fill= ' + layer.shapeSettings.fill + ' d="' + path_code + '"  stroke-linejoin= round stroke-linecap= square></path>';
                        }
                        else {
                            shapePath += '<v:shape  style="width:' + this._width + 'px;height:' + this._height + 'px;"  coordsize="'
                                + this._width + ' ' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSettings.stroke +
                                           ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) + 'px"' +
                                            ' fillcolor= ' + layer.shapeSettings.fill + ' path="' + path_code + '"></v:shape>';
                        }
                    }
                    else {
                        polygonDatas.splice(i, 1);
                        i--;
                    }
                    
                }
                
                this._groupSize = { minX: minX, maxX: maxX, minY: minY, maxY: maxY };
                
            }
            
            if (polylineDatas.length > 0) {

                for (var i = 0; i < polylineDatas.length; i++) {
                    var coordinates = polylineDatas[i].geometry.coordinates;
                    var linedata = '';
                    for (var k = 0; k < coordinates.length; k++) {
                        var factor = this._getFactor();
                        var points = this._convertTileLatLongtoPointForShapes(coordinates[k][1], coordinates[k][0], this._mapBounds,factor);
                        linedata += points.x + "," + points.y;
                        if (k != coordinates.length - 1)
                            linedata += ",";
                    }

                    if (_isSVG) {
                        shapePath += '<polyline stroke=' + layer.shapeSettings.stroke +
                                    ' stroke-width=' + (layer.shapeSettings.strokeThickness / this._scale) + ' fill="transparent" stroke-width="1" points="' + linedata + '" ></polyline>';

                    }
                    else {
                        shapePath += '<v:polyline coordsize="'+ this._width + ',' + this._height + '" coordorigin="0 0" strokecolor=' + layer.shapeSettings.stroke +
                                                               ' strokeweight=' + (layer.shapeSettings.strokeThickness / this._scale) + 'px"' +
                                                                ' fillcolor=' + layer.shapeSettings.fill + '  points="' + linedata + '"/>';


                    }
                }

            }           
            var lines = [].concat(polygonDatas, polylineDatas);
            layer._polygonData = $.merge(lines, pointData);
            return shapePath + pointHtml;
			}
			else{
			  layer._polygonData=[];
			    return null;
		   }
        },
             
        _createGroup: function (isRoot,name) {
            
            var vmlGroup;
            vmlGroup = document.createElement('<v:group class='+name+'>');
            vmlGroup.style.width = this._width + 'px';
            vmlGroup.style.height = this._height + 'px';            
            vmlGroup.coordorigin = "0 0";
            vmlGroup.coordsize = this._width + ' ' + this._height;
            if (isRoot) {
                this._rootgroup = vmlGroup;
                vmlGroup.style.left = '20px';
                vmlGroup.style.top = '20px';
            }
            return vmlGroup;
        },

        _generatePaletteColorsForShape: function (shape, shapelayer, palette, palettesettings) {

            if (palettesettings != null) {
                shapelayer.shapeSettings.highlightColor = palettesettings.highlightColor;
                shapelayer.shapeSettings.highlightStroke = palettesettings.highlightStroke;
                shapelayer.shapeSettings.highlightBorderWidth = palettesettings.highlightBorderWidth;
                shapelayer.shapeSettings.selectionColor = palettesettings.SelectionColor;
                shapelayer.shapeSettings.selectionStroke = palettesettings.SelectionStroke;
                shapelayer.shapeSettings.selectionStrokeWidth = palettesettings.SelectionStrokeWidth;
            }

            if (_isSVG) {
                shape.setAttribute("fill", palette[shapelayer._prevPaletteIndex]);
            }
            else {
                shape.fillcolor = palette[shapelayer._prevPaletteIndex];
            }
            shapelayer._prevPaletteIndex = shapelayer._prevPaletteIndex + 1;
            if (shapelayer._prevPaletteIndex > palette.length - 1)
                shapelayer._prevPaletteIndex = 0;
        },

        _renderLayers: function (layer, pathCount) {
            layer._initializeLocalValues();   
            if (layer.selectedItems == null) {
                layer.selectedItems = [];
            }       
            var proxyrootGroup = this._rootgroup;
            var proxySVG = this._svgDocument;
            var proxyControl = this;
            var shapelayer = layer;
            if (shapelayer.legendSettings != null && shapelayer.shapeSettings.colorMappings != null && (shapelayer.legendSettings.type == undefined || shapelayer.legendSettings.type == ej.datavisualization.Map.LegendType.Layers))
                 shapelayer._generateLegends(this);
            if (shapelayer.legendSettings != null && shapelayer.bubbleSettings.colorMappings != null && (shapelayer.legendSettings.type == ej.datavisualization.Map.LegendType.Bubbles))
				 shapelayer._generateBubbleLegends(this);
            if (shapelayer.shapeSettings.colorMappings != null && shapelayer.shapeSettings.colorMappings.rangeColorMapping != null) {

                shapelayer.shapeSettings.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            if (shapelayer.bubbleSettings.colorMappings != null && shapelayer.bubbleSettings.colorMappings.rangeColorMapping != null) {
                shapelayer.bubbleSettings.colorMappings.rangeColorMapping.sort(this._orderByNameAscending);
            }
            var minValue = 0;
            var maxValue = 0;
            var sum = 0;
            if (shapelayer.dataSource != null && shapelayer.bubbleSettings != null) {
                if (shapelayer.bubbleSettings.colorValuePath  == null) {
                    shapelayer.bubbleSettings.colorValuePath  = shapelayer.bubbleSettings.valuePath;
                }
                for (var i = 0; i < shapelayer.dataSource.length; i++) {
                    var bubbledata = parseFloat(this._reflection(shapelayer.dataSource[i], shapelayer.bubbleSettings.valuePath));
                    if (i != 0) {
                        if (bubbledata > maxValue) {
                            maxValue = bubbledata;
                        } else if (bubbledata < minValue) {
                            minValue = bubbledata;
                        }
                    } else {
                        minValue = maxValue = bubbledata;
                    }
                }
            }

            this._generateTooltipForLayers(shapelayer);
            if (shapelayer.shapeData != null) {
                var shapesource = [];
                for (var key in shapelayer.dataSource) {
                    var shapeitem = shapelayer.dataSource[key];
                    var shapeItemValue = proxyControl._reflection(shapeitem, shapelayer.shapeDataPath);
                    shapesource.push(shapeItemValue);
                }
                for (var dataIndex = 0; dataIndex < shapelayer._polygonData.length; dataIndex++) {
                    var ValueObject = shapelayer._polygonData[dataIndex].properties;
                    var DBFValue = proxyControl._reflection(ValueObject, shapelayer.shapePropertyPath);
                    var shape = this._rootgroup.childNodes[dataIndex + pathCount];
                    var obj = {};
                    var bounds = shapelayer._newBounds[dataIndex];

                    if (shapelayer.shapeSettings.autoFill) {
                        switch (shapelayer.shapeSettings.colorPalette) {
                            case ej.datavisualization.Map.ColorPalette.Palette1:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette1, shapelayer._colorPaletteSettings.Palette1);
                                break;
                            case ej.datavisualization.Map.ColorPalette.Palette2:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette2, shapelayer._colorPaletteSettings.Palette2);
                                break;
                            case ej.datavisualization.Map.ColorPalette.Palette3:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.colorPalettes.Palette3, shapelayer._colorPaletteSettings.Palette3);
                                break;
                            case ej.datavisualization.Map.ColorPalette.CustomPalette:
                                this._generatePaletteColorsForShape(shape, shapelayer, shapelayer.shapeSettings.customPalette, null);
                                break;
                        }
                    }

                    if (shapesource.indexOf(DBFValue) != -1) {
                        var item = shapelayer.dataSource[shapesource.indexOf(DBFValue)];
                        var ItemValue = DBFValue;
                        if (item != null) {
                            if (shape != null) {                              
                               
                                var id = ItemValue;
                            }
                                layer._bounds.push(bounds);
                                if (shapelayer.showMapItems) {
                                    var itemTemplateDiv = $("<div class='mapItems' style='display:block;position:absolute;pointer-events: none;'></div>");
                                    itemTemplateDiv[0].className = ItemValue;
                                    proxyControl._templateDiv.append(itemTemplateDiv);
                                    $(itemTemplateDiv).css({ "left": bounds.x, "top": bounds.y });
                                    var htmlString;
                                    if (shapelayer.mapItemsTemplate == null) {
                                        htmlString= $(' <div><label>{{:' + shapelayer.shapeSettings.valuePath + '}}</label></div>').render(item);;
                                    }
                                    else {
                                        htmlString = $("#" + shapelayer.mapItemsTemplate).render(item);
                                    }
                                    $(itemTemplateDiv).html(htmlString);
                                    layer._mapItems.push(itemTemplateDiv);
                                }                                   
                                                             
                                if (shapelayer.shapeSettings.colorMappings != null) {
                                    if (shapelayer.shapeSettings.colorValuePath  == null) {
                                        shapelayer.shapeSettings.colorValuePath  = shapelayer.shapeSettings.valuePath;
                                    }
                                    var shapeValue = proxyControl._reflection(item, shapelayer.shapeSettings.colorValuePath );
                                    if (shapeValue != null && !shapelayer.shapeSettings.autoFill) {
                                        if (((shapelayer.legendSettings != null && shapelayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) || shapelayer.shapeSettings.enableGradient) && shapelayer.shapeSettings.colorMappings.rangeColorMapping != null) {
                                            obj["legendrect"] = shapelayer._updateLegendRange(shapeValue, shapelayer, shape);
                                            var shapeColor = shapelayer.shapeSettings.fill;
                                            if (obj["legendrect"] != undefined) {
                                                shapeColor = obj["legendrect"].color;
                                            }
                                            if (_isSVG) {
                                                shape.setAttribute("fill", shapeColor);
                                            }
                                            else {
                                                shape.fillcolor = shapeColor;
                                            }
                                        } else {
                                            shapelayer._fillColors(shapeValue, shapelayer.shapeSettings.colorMappings, shape);
                                        }
                                    }
                                }
                                else if (!shapelayer.shapeSettings.autoFill) {
                                    if (_isSVG) {
                                        shape.setAttribute("fill", shapelayer.shapeSettings.fill);
                                    }
                                    else {
                                        shape.fillcolor = shapelayer.shapeSettings.fill;
                                        shape.strokecolor = shapelayer.shapeSettings.stroke;
                                    }
                                }
                                if (shapelayer.bubbleSettings.showBubble) {
                                    if (shapelayer.bubbleSettings != null && shapelayer.bubbleSettings.valuePath != null) {
                                        if (_isSVG) {
                                            var circle = document.createElementNS(proxyControl._svgns, "circle");
                                        }
                                        else {
                                            var bubbleID = "bubble_" + id;
                                            var bubbleHtmlString = '<v:oval class="mapBubble" id=' + bubbleID + ' display="block" style="position:absolute;top: ' + 0 + 'px;left:' + 0 + 'px;width:100px;height:100px;stroke-width:' + 0 + 'px;"><v:fill opacity="0.9"/></v:oval>';
                                            this._bubblegroup.innerHTML = this._bubblegroup.innerHTML + bubbleHtmlString;
                                            var circle = document.getElementById("bubble_" + id);
                                        }

                                        var bubbleValue = proxyControl._reflection(item, shapelayer.bubbleSettings.valuePath);
                                        var bubblecolorValue = proxyControl._reflection(item, shapelayer.bubbleSettings.colorValuePath);
                                        if (shapelayer.bubbleSettings.colorMappings != null) {
                                            shapelayer._fillColors(bubblecolorValue, shapelayer.bubbleSettings.colorMappings, circle);
                                        }
                                        else {
                                            if (_isSVG) {
                                                circle.setAttribute("fill", shapelayer.bubbleSettings.color);
                                            }
                                            else {
                                                circle.strokecolor = shapelayer.bubbleSettings.color;
                                                circle.fillcolor = shapelayer.bubbleSettings.color;
                                            }
                                        }
                                        var radius = proxyControl._getRatioOfBubble(shapelayer.bubbleSettings.minValue, shapelayer.bubbleSettings.maxValue, bubbleValue, minValue, maxValue);
                                        if (_isSVG) {
                                            $(circle).attr({
                                                "cx": bounds.x, "cy": bounds.y,
                                                "fill-opacity": shapelayer.bubbleSettings.bubbleOpacity, "r": radius, "class": "mapBubble",
                                            });
                                            if (proxyControl.enableAnimation()) {
                                                $(circle).css("display", "none");
                                            }
                                        }
                                        else {
                                            $(circle).css({
                                                "height": +2 * radius + "px",
                                                "width": +2 * radius + "px"
                                            });
                                        }
                                        if (shapelayer.bubbleSettings.showTooltip == true) {
                                            var tooltipTemplateDiv = $("<div class='bubbleToolTip'  style='position:absolute;display:none;z-index:9999'/>");
                                            var template_ID = null;
                                            if (shapelayer.bubbleSettings.tooltipTemplate != null) {
                                                template_ID = shapelayer.bubbleSettings.tooltipTemplate;
                                            } else {
                                                $(tooltipTemplateDiv).append('<div class="bubbleToolTip"  style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">' + bubbleValue + '</label></p></div></div>');
                                            }
                                            $(circle).mouseenter({ htmlobj: tooltipTemplateDiv, templateID: template_ID, itemsrc: item }, proxyControl._bubbleEnterFunction);
                                            $(circle).mousemove({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleOverFunction);
                                            $(circle).mouseleave({ htmlobj: tooltipTemplateDiv }, proxyControl._bubbleleaveFunction);
                                        } else {
                                            $(circle).css("pointer-events", "none");
                                        }
                                        if (_isSVG) {
                                            proxySVG.appendChild(circle);
                                            layer._bubbles.push(circle);
                                        }
                                        else {
                                            layer._bubbles.push(circle);
                                        }
                                    }
                                }
                                obj["data"] = item;
                        }
                    }
                   var labelElement;
                    if (layer.labelSettings != null && layer.labelSettings.showLabels && layer.labelSettings.labelPath != null) {
                        var labelValue = proxyControl._reflection(ValueObject, layer.labelSettings.labelPath);
                        if (labelValue != null) {
                            if (bounds == null) {
                               //bounds = (shape);
                            }
                            labelElement = $("<div class='labelStyle'></div>");                            
                            $(labelElement).css({ "pointer-events": "none", "position": "absolute" });
                            labelElement[0].innerHTML = labelValue;
                            this._templateDiv.append(labelElement);
                            $(labelElement).css({ "left": bounds.x, "top": bounds.y });
                            proxyControl._off($(labelElement), ej.eventType.mouseUp, proxyControl._polyClickFunction);
                            proxyControl._on($(labelElement), ej.eventType.mouseUp, { Param1: null, Param2: layer, Param3: shape }, proxyControl._polyClickFunction);
							proxyControl._off($(labelElement), ej.eventType.mouseLeave, proxyControl._polyLeaveFunction);
                            proxyControl._on($(labelElement), ej.eventType.mouseLeave, { Param1: layer,map:this }, proxyControl._polyLeaveFunction);
							labelElement[0].setAttribute("nodeValue", shape.getAttribute("fill"));
                            layer._labelCollection.push(labelElement);
                            layer._labelBounds.push(bounds);
                            layer._labelData.push(labelValue);

                        }
                    }
					obj["shapeIndex"]=dataIndex;
                    obj["shape"] = shape;
                    obj["shapeData"] = layer._polygonData[dataIndex];
                    proxyControl._off($(labelElement), ej.eventType.mouseMove, proxyControl._polyMoveFunction);
                    proxyControl._on($(labelElement), ej.eventType.mouseMove, { Param1: shapelayer, Param2: obj }, proxyControl._polyMoveFunction);
                    if (shape != null) {
                        var shapefill = null; 
                        if (_isSVG) {
                            shapefill = shape.getAttribute("fill");
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSettings.fill;
                            shape.setAttribute("nodeValue", shapefill);
                        }
                        else {
                            shapefill = shape.fillcolor.value;
                            if (shapefill == "undefined")
                                shapefill = layer.shapeSettings.fill;
                            shape.style.behavior = shapefill;
                        }
                    }
					if (proxyControl.enableZoomOnSelection()) {
                        $(shape).hover(function() {
                            $(this).css('cursor', 'pointer');
                        }, function() {
                            $(this).css('cursor', 'auto');
                        });
					}
					layer._mapShapes.push(obj);
					$(shape).mouseenter({ Param1: shapelayer, Param2: obj, map: proxyControl }, proxyControl._polyEnterFunction);
					this._off($(shape), ej.eventType.mouseDown ,proxyControl._polyMouseDown);
					this._on($(shape), ej.eventType.mouseDown, { Param1: obj, Param2: shapelayer }, proxyControl._polyMouseDown);
					this._off($(shape), ej.eventType.mouseUp, proxyControl._polyClickFunction);
                    this._on($(shape), ej.eventType.mouseUp, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyClickFunction);
					this._off($(shape), ej.eventType.mouseMove, proxyControl._polyMoveFunction);
                    this._on($(shape), ej.eventType.mouseMove, { Param1: shapelayer, Param2: obj }, proxyControl._polyMoveFunction);
                    this._off($(shape), ej.eventType.mouseLeave, proxyControl._polyLeaveFunction);
					this._on($(shape), ej.eventType.mouseLeave, { Param1: shapelayer,Param2: obj,map:this }, proxyControl._polyLeaveFunction);
                    this._off($(shape), ej.eventType.mouseUp, proxyControl._polyUpFunction);
					this._on($(shape), ej.eventType.mouseUp, { Param1: obj, Param2: shapelayer, Param3: shape }, proxyControl._polyUpFunction);
                }
            }
        },

        validateBubblePosition: function (polygon, midpoint, bubblerad) {
            var leftPoints = [], rightPoints = [];
            if (!this.isPointInPolygon(polygon, midpoint, true)) {
                var previouspoint = 0;
                for (var j = 0; j < polygon.length; j++) {
                    var point = { x: (polygon[j].x + this._translatePoint.x) * this._scale, y: (polygon[j].y + this._translatePoint.y) * this._scale };
                    if ((point.y < midpoint.y && previouspoint.y >= midpoint.y) || (previouspoint.y < midpoint.y && point.y >= midpoint.y)) {
                        if (point.x < midpoint.x)
                            leftPoints.push(point.x);
                        else
                            rightPoints.push(point.x);
                    }
                    previouspoint = point;
                }

                if (!leftPoints.length > 0 || !rightPoints.length > 0) {
                    if (leftPoints.length > 0) {
                        var j = midpoint.x + bubblerad;
                        while (!this.isPointInPolygon(polygon, { x: j - 2, y: midpoint.y }, true)) {
                            j = j - 2;
                        }
                        midpoint.x = parseFloat(j - 2 - bubblerad);
                    }
                    if (rightPoints.length > 0) {
                        var j = midpoint.x - bubblerad;
                        while (!this.isPointInPolygon(polygon, { x: j + 2, y: midpoint.y }, true)) {
                            j = j + 2;
                        }
                        midpoint.x = parseFloat(j + 2 + bubblerad);
                    }
                }
            }
            return midpoint;
        },

        isPointInPolygon: function (polygon, point, needTranslate) {
            var insidePolygon = false;
            var j = 0;
            for (var i = 0; i < polygon.length; i++) {
                var currentPoint = { x: polygon[i].x, y: polygon[i].y };
                var previousPoint = { x: polygon[j].x, y: polygon[j].y };

                if (needTranslate) {
                    currentPoint = { x: (polygon[i].x + this._translatePoint.x) * this._scale, y: (polygon[i].y + this._translatePoint.y) * this._scale };
                    previousPoint = { x: (polygon[j].x + this._translatePoint.x) * this._scale, y: (polygon[j].y + this._translatePoint.y) * this._scale };
                }

                var pointYDiff = point.y - currentPoint.y;
                var previousPointYDiff = previousPoint.y - currentPoint.y;
                var previousPointXDiff = previousPoint.x - currentPoint.x;

                if ((currentPoint.y < point.y && previousPoint.y >= point.y) || (previousPoint.y < point.y && currentPoint.y >= point.y)) {
                    if ((currentPoint.x + (pointYDiff / previousPointYDiff) * previousPointXDiff) < point.x) {
                        insidePolygon = true;
                    }
                }
                j = i;
            }
            return insidePolygon;
        },

        dragDown: function (event) {
                                  
            var map = event.data.map;
            var layer = map.model.layers[0];

            var mapArea = this._mapContainer;

            if (event.type == "mousedown")
            {
                map._startPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
                map._stopPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
            }
            else if (event.type == "touchstart") {
                map._startPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
                map._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
            }
            else if (event.type == "MSPointerDown") {
                map._startPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
                map._stopPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
            }
            map.mouseClickable = true;

            if (map.model.draggingOnSelection) {

                if (layer._prevSelectedShapes.length > 0) {

                    for (var i = 0; i < layer._prevSelectedShapes.length; i++) {                                               
                        layer._prevSelectedShapes[i].setAttribute("class", "mapShape");
                        if (_isSVG) {                            
                            layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                            layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                            layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                        }
                        else {
                            layer.fillcolor = layer._prevSelectedShapes[i].getAttribute("nodeValue");
                            layer.strokecolor = layer.shapeSettings.stroke
                            layer.strokeweight = layer.shapeSettings.strokeThickness / this._scale;
                        }
                    }
                    layer._prevSelectedShapes = [];
                }
            }
        },

        dragMove: function (event) {
            
            var map = event.data.map;
            var layer = map.model.layers[0];
            var width; var height;

            var mapArea = this._mapContainer;

            if (map.model.draggingOnSelection) {

                if (map.mouseClickable) {                    
                    map.ispanning = false;


                    if (event.type == "mousemove")
                        map._stopPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
                    else if (event.type == "touchmove")
                        map._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
                    else if (event.type == "MSPointerMove")
                        map._stopPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
                    
                    $('.e-mapDragSelection').remove();
                    var div = $('<div class = "e-mapDragSelection"></div>');
                    
                    width = Math.abs(map._stopPoint.x - map._startPoint.x),
                    height = Math.abs(map._stopPoint.y - map._startPoint.y);

                    $(div).css({ "top": Math.min(map._startPoint.y, map._stopPoint.y), "left": Math.min(map._startPoint.x, map._stopPoint.x), width: width, height: height,  position: "absolute", "z-index": 100 });
                    $(div).appendTo("#maps");
                }
            }
        },

        dragUp: function (event) {        
            var map = event.data.map;
            var width = Math.abs(map._stopPoint.x - map._startPoint.x);
            var height = Math.abs(map._stopPoint.y - map._startPoint.y);
            var layer = map.model.layers[0];
            var mapArea = this._mapContainer;
        
            if (map.model.draggingOnSelection) {
        
                $('.e-mapDragSelection').remove();
                $(".e-mapDragSelection").css({
                    "display": "none"
                });
                map.mouseClickable = false;

                for (var i = 0; i < map.model.layers[0]._mapShapes.length; i++) {
                    var shape = map.model.layers[0]._mapShapes[i].shape;
                    var rect = shape.getBoundingClientRect();
                    if (!(rect.left - mapArea[0].offsetLeft + rect.width < Math.min(map._startPoint.x, map._stopPoint.x) || Math.min(map._startPoint.x, map._stopPoint.x) + width < rect.left - mapArea[0].offsetLeft ||
                           rect.top - mapArea[0].offsetTop + rect.height < Math.min(map._startPoint.y, map._stopPoint.y) || Math.min(map._startPoint.y, map._stopPoint.y) + height < rect.top - mapArea[0].offsetTop)) {

                        if (!map.model.layers[0]._contains(layer._prevSelectedShapes, shape))

                            layer._prevSelectedShapes.push(shape);
                    }
                }


                for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                    var shape = layer._prevSelectedShapes[i];
                    var rect = shape.getBoundingClientRect();
                    shape.setAttribute("class", "e-mapSelectedShape mapShape");
                    if (_isSVG) {
                        shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                        shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                        shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));
                    }
                }                
            }           
        },

        isIntersectBubbles: function () {
            var listofBounds = [];
            for (var k = 0; k < this.model.layers.length; k++) {
                var layer = this.model.layers[k];
                Array.prototype.push.apply(listofBounds, layer._bubbleCollection);
                for (var i = 0; i < layer.subLayers.length; i++) {
                    Array.prototype.push.apply(listofBounds, layer.subLayers[i]._bubbleCollection);
                }
            }

            for (var i = 0; i < listofBounds.length; i++) {
                var x = listofBounds[i].getAttribute('cx');
                var y = listofBounds[i].getAttribute('cy');
                var r = listofBounds[i].getAttribute('r');
                for (var j = i + 1; j < listofBounds.length; j++) {

                    var x1 = listofBounds[j].getAttribute('cx');
                    var y1 = listofBounds[j].getAttribute('cy');
                    var r1 = listofBounds[j].getAttribute('r');
                    var distBubble = Math.sqrt(((x - x1) * (x - x1)) + ((y - y1) * (y - y1)));
                    var rad = parseFloat(r) + parseFloat(r1);
                   
                    if (distBubble <= rad) {
                        return true;
                    }
                }
            }
            return false;
        },
       
        _zooming: function (delta, e) {
            var position = this._getCurrentPoint(e);
            if (delta >= 120) {
                this._zoomingIn(position.x, position.y, e);
            }
            else {
                this._zoomingOut(position.x, position.y, e);
            }
        },

        _zoomingIn: function (x, y, event, isAnimate) {
            var map = this;
            if (map.enableZoom() && map.zoomLevel() >= map.minZoom() && map.zoomLevel() <= map.maxZoom()) {
                var scal = map._scale;
                var position = { x: x, y: y };
                map._prevScale = scal;
                if (!this._isTileMap) {
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map.zoomFactor())) / (map._width / x);
                    map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map.zoomFactor())) / (map._height / y);
                    map._scale = scal + map.zoomFactor();
                    map.zoomLevel(map.zoomLevel() + map.zoomFactor());
                }
                else {
                    this._tileZoom(map.zoomLevel() - map.zoomFactor(), map.zoomLevel(), position);
                    var prevLevel = map.zoomLevel();
                    map.zoomLevel(map.zoomLevel() + map.zoomFactor());
                    this._generateTiles(this.zoomLevel());
                    map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._scale = (Math.pow(2, prevLevel));
                }
                
                map._trigger("zoomedIn", { originalEvent: event, zoomLevel: map.zoomLevel() });
                map._applyTransform(map._scale, map._translatePoint);
                map._refrshLayers();
                map._resizeShape();               
            
                map._updateSliderValue();
            }
        },

        _zoomingOut: function (x, y, event, isAnimate) {
            var map = this;
            if (map.enableZoom()) {              
                if (map.zoomLevel() > this.minZoom()) {

                    var scal = map._scale;
                    var position = { x: x, y: y };
                    map._prevScale = scal;
                    if (!map._isTileMap) {                       
                       if (map._scale > 1) {		
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale - map.zoomFactor())) / (map._width / x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale - map.zoomFactor())) / (map._height / y);
                        map._scale = map._scale - map.zoomFactor();
                        map.zoomLevel(map.zoomLevel() - map.zoomFactor());
                        map._scale = map.zoomLevel();
                        }
				    }
                    else {
                        map._tileZoom(map.zoomLevel() + map.zoomFactor(), map.zoomLevel(), position);
                        var prevLevel = map.zoomLevel();
                        map.zoomLevel(map.zoomLevel() - map.zoomFactor());
                       
                        map._generateTiles(map.zoomLevel());
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, map.zoomLevel() - map.zoomFactor()))) / (Math.pow(2, map.zoomLevel() - map.zoomFactor()));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, map.zoomLevel() - map.zoomFactor()))) / (Math.pow(2, map.zoomLevel() - map.zoomFactor()));
                        map._scale = (Math.pow(2, map.zoomLevel() - map.zoomFactor()));
                    }                    
                    if (map.zoomLevel() < map.minZoom()) {
                        map.zoomLevel(map.minZoom());
                        map.zoom(map.zoomLevel());
                    }
                   
                    map._applyTransform(map._scale, map._translatePoint);
                    map._refrshLayers();
                    map._resizeShape();                                  

                    map._updateSliderValue();
                }
                else {                   
                    map.zoom(map.minZoom(),false);
                }
                map._trigger("zoomedOut", { originalEvent: event, zoomLevel: map.zoomLevel() });
            }
        },

        _getRatioOfBubble: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var bubbleRadius = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
			if(maxValue == minValue)
				bubbleRadius = (((parseFloat(max) - parseFloat(min)) / 100)) + parseFloat(min);
			
            return bubbleRadius;
        },

        _reflection: function (Source, path) {
            var ShapeValues = Source;
            if (path != null && Source != null) {
                var parts = path.split(".");
                var i = 0;
                parts.push(path);
                for (; i < parts.length; i++) {                    
                    if (Source != undefined && ShapeValues != null) {
                        var hasProperty = ShapeValues.hasOwnProperty(parts[i]);
                        if (hasProperty) {
                            $.each(ShapeValues, function (prop, propval) {
                                if (prop == parts[i]) {
                                    ShapeValues = propval;
                                    return false;
                                }

                            });
                        }
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        },
		
		clearShapeSelection:function(){		
		 var index = this.baseMapIndex();
            this.model.layers[index]._clearShapeWidth(this._scale);
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._clearShapeWidth(this._scale);
            }
		},
        shapeSelectionOnResize: function () {
        var index = this.baseMapIndex();
          this.model.layers[index]._shapeSelection();
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._shapeSelection();
            }
        },

        refreshMarkers: function () {
            var index = this.baseMapIndex();
            this.model.layers[index]._generateMarkerForLayers(this);
            for (var key = 0; key < this.model.layers[index].subLayers.length; key++) {
                (this.model.layers[index].subLayers[key])._generateMarkerForLayers(this);
            }
        },

        _generateTooltipForLayers: function (layer) {
             if (layer.showTooltip) {
			   if (document.documentMode == 8){
					var tooltip = document.querySelectorAll("shapeToolTip");
				}
				else{
					var tooltip = document.getElementsByClassName("shapeToolTip");
				}
                if (tooltip != null && tooltip.length == 0) {
                    var tooltip_templateDiv = $("<div class='shapeToolTip' style='display:none;position:absolute;z-index:1000;pointer-events:none;'></div>");
                    $(document.body).append(tooltip_templateDiv);
                    layer._tooltipElement = tooltip_templateDiv;
                }
                else
                {
                    layer._tooltipElement = tooltip[0];
                }
                if (layer.tooltipTemplate == null) {
                    var path = layer.shapeSettings.valuePath;
                    if (path != null) {
                        layer.tooltipTemplate = 'defaultTooltip';                      
                        this._mapContainer.append($('<div  id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:#data["' + path + '"]}}</label></p></div></div></div>'));
                    }
                }
            }                
        },
       
        _orderByNameAscending: function (a, b) {
            if (a.Range == b.Range) {
                return 0;
            } else if (a.Range > b.Range) {
                return 1;
            }
            return -1;
        },

        _resizingContainer: function () {            
            this._scale = this._isTileMap ? Math.pow(2, this.zoomLevel() - 1) : 1;
            if (this._isTileMap) {
                this._translatePoint.x = this._tileTranslatePoint.x / this._scale;
                this._translatePoint.y = this._tileTranslatePoint.y / this._scale;
            }
            else {                     
                this._translatePoint.x = this._tileTranslatePoint.x / this._scale;
                this._translatePoint.y = this._tileTranslatePoint.y / this._scale;
            }           
            this._applyTransform(this._scale, this._translatePoint);
            if (this.zoomLevel() > 1) {
                this._scale = this._isTileMap ? Math.pow(2, this.zoomLevel() - map.zoomFactor()) : this.zoomLevel();
                this._applyTransform(this._scale, this._translatePoint);
            }
            this._baseScale = 1;
            this._baseTranslatePoint = { x: (this._translatePoint.x * 2), y: (this._translatePoint.y * 2) };
           
        },

        _calculateMarginConditions: function () {
            var trans = this._calculateMinMax();
            if (this._translatePoint.y> trans.maxY) {
                this._translatePoint.y = trans.maxY;
            }
            else if (this._translatePoint.y < trans.minY) {
                this._translatePoint.y = trans.minY;
            }
            if (this._translatePoint.x > trans.maxX) {
                this._translatePoint.x = trans.maxX;
            }
            else if (this._translatePoint.x < trans.minX) {
                this._translatePoint.x = trans.minX;
            }
        },

        _calculateMinMax: function () {           
            var bounds= this._groupBorder;            
            var maximumTransX, maximumTransY, minimumTransX, minimumTransY;
            if (this._containerHeight * this._scale <= this._height) {
                maximumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale) - bounds.y;
                minimumTransY = (this._height - this._containerHeight * this._scale) / (2 * this._scale) - bounds.y;
            } else {
                maximumTransY = -bounds.y + 1;
                minimumTransY = (this._height - this._containerHeight * this._scale) / (this._scale) - bounds.y - 1;
            }
            if (this._containerWidth * this._scale <= this._width) {
                maximumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale)- bounds.x;
                minimumTransX = (this._width - this._containerWidth * this._scale) / (2 * this._scale) - bounds.x;
            } else {
                maximumTransX = -bounds.x + 1;
                minimumTransX = (this._width - this._containerWidth * this._scale) / (this._scale) - bounds.x - 1;
            }
            return { minX: minimumTransX, maxX: maximumTransX, minY: minimumTransY, maxY: maximumTransY };
        },
      
        _findMidPointofPoylgon: function (points) {
          if(points.length>0){
            var min = 0;           
            var max = points.length;
            var initialPoint = points[0];
            var minX=initialPoint.x,maxX = initialPoint.x;
            var minY = initialPoint.y, maxY = initialPoint.y;
            var startX,startY,startX1,startY1,sum=0,xSum=0,ySum=0;
           
            for (var i = min; i <= max - 1; i++) {
                var prevPoint = points[i];
                startX = prevPoint.x;
                startY = prevPoint.y;
                if (i == max - 1) {
                    startX1 = initialPoint.x;
                    startY1 = initialPoint.y;
                }
                else {
                    var nextPoint = points[i + 1];
                    startX1 = nextPoint.x;
                    startY1 = nextPoint.y;
                }
                sum = sum + Math.abs(((startX * startY1)) - (startX1 * startY));
                xSum = xSum + Math.abs(((startX + startX1) * (((startX * startY1) - (startX1 * startY)))));
                ySum = ySum + Math.abs(((startY + startY1) * (((startX * startY1) - (startX1 * startY)))));

            }
            sum = 0.5 * sum;
            xSum = (1 / (4 * sum)) * xSum;
            ySum = (1 / (4 * sum)) * ySum;

            /* Code for finding nearest points in polygon related to midPoint*/
            var _rightMinPoint = { x: 0, y: 0 };
            var _rightMaxPoint = { x: 0, y: 0 };
            var _leftMinPoint = { x: 0, y: 0 };
            var _leftMaxPoint = { x: 0, y: 0 };

            for (var i = min; i <= max - 1; i++) {
                var point = points[i];
                if (point.x > xSum) {
                    if (point.y < ySum && ySum - point.y < ySum - _rightMinPoint.y)
                        _rightMinPoint = { x: point.x, y: point.y };
                    else if (point.y > ySum && (_rightMaxPoint.y == 0 || point.y - ySum < _rightMaxPoint.y - ySum))
                        _rightMaxPoint = { x: point.x, y: point.y };
                }
                else {
                    if (point.y < ySum && ySum - point.y < ySum - _leftMinPoint.y)
                        _leftMinPoint = { x: point.x, y: point.y };
                    else if (point.y > ySum && (_leftMaxPoint.y == 0 || point.y - ySum < _leftMaxPoint.y - ySum))
                        _leftMaxPoint = { x: point.x, y: point.y };
                }
            }

            return { x: xSum, y: ySum, rightMin: _rightMinPoint, rightMax: _rightMaxPoint, leftMin: _leftMinPoint, leftMax: _leftMaxPoint, points:points };             
}        
        },
        _convertPointToLatLong: function (pixelX, pixelY, bounds, factor){
        
            var Size;
            if (bounds == null) { bounds=[[-180,-85],[180,85]] }
            if (this._isTileMap) {
                Size = Math.pow(2, 1) * 256;
            }
            else {
                if(factor == null)
                    Size = Math.min(this._height, this._width);
                else
                    Size = Math.min(this._height, this._width)*factor;
            }

            var x=(Math.min(pixelX,0,Size-1)/Size)-0.5;
            var y=0.5-(Math.min(pixelY,0,Size-1)/Size);
            latitude = 90 - 360 * Math.atan(Math.exp(-y * 2 * Math.PI)) / Math.PI;
            longitude = 360 * x;
            return {x:latitude,y:longitude};
                        

        },
        _convertTileLatLongtoPointForShapes: function (lat, lng, bounds,factor) {
            var Size;
            if (bounds == null) { bounds=[[-180,-85],[180,85]] }
            if (this._isTileMap) {
                Size = Math.pow(2, 1) * 256;
            }
            else {
                if(factor == null)
                    Size = Math.min(this._height, this._width);
                else
                    Size = Math.min(this._height, this._width)*factor;
            }
           
            var x = (lng + 180) / 360;
            var sinLatitude = Math.sin(lat * Math.PI / 179.5);
            if (sinLatitude == -1)
                sinLatitude = -0.5;
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
            
            var pixelX = (Math.min(Math.max(x * Size + 0.5, 0), Size - 1));
            var pixelY = (Math.min(Math.max(y * Size + 0.5, 0), Size - 1));
            return { x: pixelX, y: pixelY };
           
            
        },
     
        _convertTileLatLongtoPoint: function (lat, lng) {
            var Size = Math.pow(2, this.zoomLevel()) * 256;          
            var x = (lng + 180) / 360; 
            var sinLatitude = Math.sin(lat * Math.PI / 180);
            var y = 0.5 - Math.log((1 + sinLatitude) / (1 - sinLatitude)) / (4 * Math.PI);
            var pixelX = (x * Size + 0.5) + this._tileTranslatePoint.x;
            var pixelY = (y * Size + 0.5) + this._tileTranslatePoint.y;            
            return { x: pixelX, y: pixelY };
        },
      
        _convertLatLongtoPointforMarker: function (lat, lng) {
            var factor = this._getFactor();
            var point = this._convertTileLatLongtoPointForShapes(lat, lng, this._mapBounds,factor);
            return { x: (point.x + this._translatePoint.x) * this._scale, y: (point.y + this._translatePoint.y) * this._scale };

        },
     
        _animate: function (animateduration) {
            this._calculateMarginConditions();
            $(this._rootgroup).stop(true, false);
            if (this._sliderControl != null) {
                $(this._sliderControl).stop(true, false);
            }
            var map = this;
            var currentTranformation = { x: this._translatePoint.x, y: this._translatePoint .y};
            var startPoint = [this._prevPoint.x, this._prevPoint.y];
            var endPoint = [this._translatePoint.x, this._translatePoint.y];

            function slope(horizontal, vertical) {
                
                if (horizontal[0] == vertical[0]) {
                    return null;
                }
                return (vertical[1] - horizontal[1]) / (vertical[0] - horizontal[0]);
            }

            function intercept(point, slopeValue) {
                if (slopeValue === null) {
                    return point[0];
                }
                return point[1] - slopeValue * point[0];
            }

            var slopeFactor = slope(startPoint, endPoint);
            var slopeIntersection = intercept(startPoint, slopeFactor);
            var horizontalDifference= endPoint[0] - startPoint[0];
            var verticalDifference=endPoint[1] - startPoint[1];
            var scaleDifference= this._scale-this._prevScale;
            var currentScale = this._scale;
            this._updateSliderValue();
            $(this._rootgroup).animate(
                {
                    count: 10
                },
                {
                    duration: animateduration,
                    step: function(now, fx) {
                        var scaleX = map._prevScale + ((now * (scaleDifference / fx.end)));
                        var transX = startPoint[0] + (now * (horizontalDifference / fx.end)) / (scaleX / currentScale);
                        var transY;
                        if (slopeFactor == null) {
                            transY = startPoint[1] + (now * (verticalDifference / fx.end));
                        } else {
                            transY = ((slopeFactor * transX) + slopeIntersection);
                        }
                        if (_isSVG) {
                            map._rootgroup.setAttribute('transform', 'scale(' + scaleX + ') translate(' + transX + ', ' + transY + ')');
                        }
                        else {
                            map._rootgroup.coordorigin = -transX + ',' + -transY;
                            map._rootgroup.coordsize = (map._width / scaleX) + ',' + (map._height / scaleX);
                        }
                        map._translatePoint.x = transX;
                        map._translatePoint.y = transY;
                        map._scale = scaleX;
                        map._refrshLayers();
                        map._resizeShape();
                    },
                    complete: function() {
                        map._translatePoint.x = currentTranformation.x;
                        map._translatePoint.y = currentTranformation.y;
                        map._scale = currentScale;
                        this.count = 0;
                    }
                }
            );

        },

        _applyTransform: function (scale, translatePoint) {
            this._scale = scale;
            this._translatePoint = translatePoint;
           if(!this._isTileMap)
            this._calculateMarginConditions();  
             if((this._translatePoint.x !=Number.POSITIVE_INFINITY) &&(this._translatePoint.y !=Number.POSITIVE_INFINITY))
            {
            if (_isSVG) {
                this._rootgroup.setAttribute('transform', 'scale(' + this._scale + ') translate(' + this._translatePoint.x + ', ' + this._translatePoint.y + ')');
            }
            else {
                this._rootgroup.coordorigin = (-this._translatePoint.x) + ','  + (-this._translatePoint.y);
                this._rootgroup.coordsize = (this._width / scale)  + ',' + (this._height / scale);
            }
            }
        },
       
        _mouseMove: function (event) {
            if (event.type == "touchmove" && event.originalEvent.touches.length > 1) {
                this._isPinching = true;
            }
            else {
                var map = this;
                if (map.ispanning && map.enablePan()) {
                    event.preventDefault();
                    var position;
                    if (event.type == "mousemove")
                        position = { x: event.pageX, y: event.pageY };
                    else if (event.type == "touchmove")
                        position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                    else if (event.type == "MSPointerMove")
                        position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - (this._dragStartX - position.x);
                        var curtileY = this._tileTranslatePoint.y - (this._dragStartY - position.y);
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;                        
                        this._generateTiles(this.zoomLevel());
                    }                    
                    var curX = map._translatePoint.x - (map._dragStartX - position.x) / map._scale;
                    var curY = map._translatePoint.y - (map._dragStartY - position.y) / map._scale;
                    if (curX != map._translatePoint.x || curY != map._translatePoint.y) {
                        map._isDragging = true;
                    }
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    map._applyTransform(map._scale, map._translatePoint);
                    map._dragStartX = position.x;
                    map._dragStartY = position.y;
                    map._refrshLayers();
                }
            }
        },
       
        _svgMouseLeave: function (event) {
            var map = this;
            map.ispanning = false;
        },
        
        _mouseWheel: function (event) {
            if (this.enableZoom()) {
		    if(this.enableAnimation()){
				$(this._rootgroup).stop(true, false);
			}
            var e = event.originalEvent;
            var original = event;
            if (!_isSVG) {
                e = event;
            }
            e.preventDefault ? e.preventDefault() : original.preventDefault();
	    	if(_isSVG)
	    	{
	    		this._zooming(event.originalEvent.wheelDelta, event.originalEvent);
	    	}
	    	else
	    	{
            	this._zooming(event.originalEvent.wheelDelta, e);
	    	}
            }
        },

        _mouseButtonUp: function (event) {

            var map = this;
            if (map.ispanning) {
                map._trigger("panned", { originalEvent: event });
            }
            map.ispanning = false;
        },
        
        _mouseUp: function (event) {
            this.ispanning = false;
            this._isDragging = false;
        },

        _mouseButtonDown: function (event) {
            this._isPinching = false;
           if (event.target.className != "e-vhandle") {         
            this._isNavigationPressed = false;
            if(this.enableAnimation()){
                $(this._rootgroup).stop(true, false);
                if (this._sliderControl != null) {
                    $(this._sliderControl).stop(true, false);
                }
			}
            if (event.type == "touchstart" && event.originalEvent.touches.length > 1)
            { }
            else {
                event.preventDefault();
                var position;
                if (event.type == "mousedown")
                    position = { x: event.pageX, y: event.pageY };
                else if (event.type == "touchstart")
                    position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                else if (event.type == "MSPointerDown")
                    position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                var map = this;
                map.ispanning = true;
                map._dragStartX = position.x;
                map._dragStartY = position.y;
            }
		}
        },

        _getCurrentPoint: function (event) {
            var map = this._mapContainer;
            var xPos = event.pageX - map[0].offsetLeft;
            var yPos = event.pageY - map[0].offsetTop;
            return { x: xPos, y: yPos };
        },
        
        _doubleClick: function (event) {            
            var map = this;
			var prevLevel1 = map.zoomLevel();
            if (!this._isNavigationPressed && event.target.className.toString().indexOf("e-icon1")==-1) {
                var position = this._getCurrentPoint(event);               
                if (map.enableZoom() && map.zoomLevel() + map.zoomFactor() >= map.minZoom() && map.zoomLevel() + map.zoomFactor() <= map.maxZoom()) {
                    var scal = map._scale;
                    map._prevScale = scal;
                    if (!this._isTileMap) {
                        map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                        map._translatePoint.x -= (map._width / map._scale - map._width / (map._scale + map.zoomFactor())) / (map._width / position.x);
                        map._translatePoint.y -= (map._height / map._scale - map._height / (map._scale + map.zoomFactor())) / (map._height / position.y);
                        map._scale = scal + map.zoomFactor();                       
                        map.zoomLevel(map.zoomLevel() + map.zoomFactor());
                    }
                    else {
                        this._tileZoom(map.zoomLevel() - map.zoomFactor(), map.zoomLevel(), position);
                        var prevLevel = map.zoomLevel();
                        map.zoomLevel(map.zoomLevel() + map.zoomFactor());
                        this._generateTiles(this.zoomLevel());                       
                        map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                        map._scale = (Math.pow(2, prevLevel));                        
                    }
					 if (prevLevel1 < map.zoomLevel() )
                    {
                        map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map.zoomLevel() });
                    }
                    if (map.enableAnimation() && !this._isTileMap) {
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                        map._refrshLayers();
                        map._resizeShape();
                    }                 
                    map._updateSliderValue();
                }
            }
        },
       
        _tileZoom: function (prevlevel,level, position) {
            
            var map = this;
            if (level > 0 && level < 20) {
                this._tileDiv.html("");
                var prevSize = Math.pow(2, prevlevel) * 256;
                if (position == undefined) {
                    position = { x: this._width / 2, y: this._height / 2 };
                }
                var totalSize = Math.pow(2, level) * 256;
                var percentX = ((position.x - this._tileTranslatePoint.x) / prevSize) * 100;
                var percentY = ((position.y - this._tileTranslatePoint.y) / prevSize) * 100;
                var pointX = (percentX * totalSize) / 100;
                var pointY = (percentY * totalSize) / 100;
                this._tileTranslatePoint.x = position.x - pointX;
                this._tileTranslatePoint.y = position.y - pointY;
                
            }
        },          
        
        _bubbleEnterFunction: function (event) {
            if (event.data.templateID != null) {
                var tooltiphtmlString = $("#" + event.data.templateID).render(event.data.itemsrc);
                $(event.data.htmlobj).html(tooltiphtmlString);
            }
            $(document.body).append(event.data.htmlobj);
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },
        
        _bubbleleaveFunction: function (event) {
            $(event.data.htmlobj).remove();
            return $(event.data.htmlobj).css("display", "none");
        },
        
        _bubbleOverFunction: function (event) {
            return $(event.data.htmlobj).css("display", "block").css({ "left": (event.pageX + 8) + "px", "top": (event.pageY + 6) + "px" });
        },
       
        _polyEnterFunction: function (event) {
            var layer = event.data.Param1;
            var shape = event.data.Param2["shape"];
            var map = event.data.map;
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            if (layer.enableMouseHover) {
                layer._clearShapeWidth(map._scale);
                if (layer._prevHoverdLegend != null && !layer._contains(layer._prevSelectedLegends, layer._prevHoverdLegend)) {
                    layer._prevHoverdLegend.css("background-color", _isSVG ? layer._prevHoveredShape.getAttribute("fill") : layer._prevHoveredShape.fillcolor.value);
                }
                if (_isSVG) {
                    shape.setAttribute("stroke-width", (layer.shapeSettings.highlightBorderWidth/map._scale));
                    shape.setAttribute('stroke', layer.shapeSettings.highlightStroke);
                } else {
                    shape.strokeweight = layer.shapeSettings.highlightBorderWidth;
                    shape.strokecolor = layer.shapeSettings.highlightStroke/map._scale;
                }

                if (layer.shapeSettings.highlightColor != null) {
                    if (layer.shapeSettings.highlightColor != "transparent" && !layer._contains(layer._prevSelectedShapes,shape)) {
                        shape.setAttribute("class", "e-mapHighlightedShape mapShape");
                        if (_isSVG) {
							if(layer.shapeSettings.highlightColor!='none'){
								shape.setAttribute('fill', layer.shapeSettings.highlightColor);								
							}
							shape.setAttribute("stroke-width", (layer.shapeSettings.highlightBorderWidth/map._scale));
							shape.setAttribute('stroke', layer.shapeSettings.highlightStroke);
                        } else {
                            shape.fillcolor = layer.shapeSettings.highlightColor;
                        }
                    }
                    if (legend.length > 0 && !layer._contains(layer._prevSelectedLegends,legend[0].legend)) {
                        legend[0].legend.setAttribute("class", "e-mapHighlightedShape mapShape");
                        legend[0].legend.css("background-color", layer.shapeSettings.highlightColor);
                        layer._prevHoverdLegend = legend[0].legend;
                    }

                    layer._prevHoveredShape = shape;
                }

                if (layer.legendSettings!= null && layer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && layer.shapeSettings.colorMappings!=null && layer.shapeSettings.colorMappings.rangeColorMapping != null
                     && !layer.shapeSettings.autoFill && (layer.legendSettings.showLegend == undefined || layer.legendSettings.showLegend)) {
                    for (var i = 0; i < layer._mapShapes.length; i++) {
                        var mapshape = layer._mapShapes[i].shape;
                        if (mapshape == shape) {
                            var mappings = null;
                            if (layer.shapeSettings.colorMappings.rangeColorMapping != undefined) {
                                mappings = layer.shapeSettings.colorMappings.rangeColorMapping;
                            }
                            var _legendwidth = layer.legendSettings.width;
                            if (layer.legendSettings.width == undefined)
                                _legendwidth = 150;

                            var rectwidth = (_legendwidth / mappings.length) / 10;
                            if (layer._mapShapes[i].legendrect != null) {
                                var rectPosition = layer._mapShapes[i].legendrect.marginLeft;
                                $(layer._interactiveArrow).css({ "margin-left": rectPosition + Math.ceil(rectwidth) - layer._interactiveArrow.width() / 2, "display": "block" });
                                break;
                            }
                        }
                    }
                }
                map._trigger("mouseover", { originalEvent: event.data.Param2 });
            }
                                    
        },

        _updateShapeRect:function(layer) {

            for (var index = 0; index < layer._mapShapes.length; index++) {
                var obj = layer._mapShapes[index];
                if (obj.left == null) {
                    obj["left"] = layer._mapShapes[index].shape.getBoundingClientRect().left;
                    obj["right"] = layer._mapShapes[index].shape.getBoundingClientRect().right;
                    obj["top"] = layer._mapShapes[index].shape.getBoundingClientRect().top;
                    obj["bottom"] = layer._mapShapes[index].shape.getBoundingClientRect().bottom;
                    layer._mapShapes[index] = obj;
                }
            }
        },
        
        _polyUpFunction: function (event) {
            var layer = event.data.Param2;
            var shape = event.data.Param3;
            if (this.enableZoomOnSelection() && !this._isDragging && !this._isTileMap && !this._isPinching) {
                    if (_isSVG) {
                        var bounds = shape.getBBox();
                        this._zoomOnSelection(bounds);
                    } else {                        
                            this._updateShapeRect(layer);

                        for (var index = 0; index < layer._mapShapes.length; index++) {
                            if (shape == layer._mapShapes[index].shape) {
                                var left = layer._mapShapes[index].left;
                                var top = layer._mapShapes[index].top;
                                var right = layer._mapShapes[index].right;
                                var bottom = layer._mapShapes[index].bottom;
                                var bound = { x: left, y: top, width: right - left, height: bottom - top };
                                this._zoomOnSelection(bound);
                                break;
                            }
                        }
                    }
                    this.ispanning = false;
            }
            var element = layer._tooltipElement;
            if (element != null) {
                $(element).delay(200).queue( function(next){ 
						$(this).css('display','none'); 
						next();
						
				});
            }                        
        },

        _markerPressed: function (event) {
            this._trigger("markerSelected", { originalEvent: event.data });
        },
			
        _polyMouseDown:function(event)
        {
           
            var layer = event.data.Param2;
            var position;
            if (event.type == "mousedown")
                position = { x: event.pageX, y: event.pageY };
            else if (event.type == "touchstart")
                position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
            else if (event.type == "MSPointerDown")
                position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };

            if (event.data.Param1.hasOwnProperty("data") && layer.showTooltip) {
                var element = layer._tooltipElement;
                var template = layer.tooltipTemplate;
                var parentSize = $(this._mapContainer).offset();
                if (parentSize == null)
                    parentSize = { left: 0, width: 0, height: 0 };
                var tooltipSize = layer._tooltipSize;
                var tooltipdisplayheight = this._height;
                var tooltipdisplaywidth = this._width;
				
                if (element != null && template != null) {
                    $(element).css({ "display": "block" });
                    var htmlString = $("#" + template).render(event.data.Param1["data"]);
                    $(element).html(htmlString);
                     var height= element[0]!=null ? element[0].clientHeight : element.clientHeight;
					 var width= element[0]!=null ? element[0].clientWidth : element.clientWidth;
					 tooltipSize = { height: height, width: width };

					

					var xPos = position.x - $(this._mapContainer).offset().left;
			        var yPos = position.y - $(this._mapContainer).offset().top;

                    var xnewPos = position.x;
                    var ynewPos = position.y;

			       if (tooltipSize.width + xPos >= tooltipdisplaywidth) {
                         if(xnewPos - tooltipSize.width > parentSize.left)
			                xnewPos = ((parentSize.left + tooltipdisplaywidth) - (tooltipdisplaywidth - xPos)) - tooltipSize.width;
                         else
                            xnewPos -= (tooltipSize.width + xPos - tooltipdisplaywidth);  
			            }
			            if (tooltipSize.height > tooltipdisplayheight) {
			                if (tooltipdisplayheight + tooltipSize.height > this._height + yPos) 
			                    ynewPos -= parentSize.top + 10;
			            }
                        else if (tooltipSize.height + yPos >= tooltipdisplayheight) {
			                ynewPos -= (tooltipSize.height + yPos - tooltipdisplayheight);
			            }   

                    $(element).css({ "left": xnewPos + 10, "top": ynewPos + 10 });
                }
            }
        },
      
       _polyClickFunction: function (event) {

            var ctrlkey = event.ctrlKey;
            var layer = event.data.Param2;
            var map = this;
            var shape = event.data.Param3;
            var legend = jQuery.grep(layer._smartLabels, function (obj) { return obj.shape == shape });
            map._isPolygonSelected = true;

            if (layer.enableSelection) {

                if (ctrlkey && layer.selectionMode == ej.datavisualization.Map.SelectionMode.Multiple) {
				 if (legend != null && legend.length > 0) {
                        if ($.inArray(legend[0].legend, layer._prevSelectedLegends) == -1) {
				            legend[0].legend.setAttribute("class", "e-mapSelectedShape mapShape");
                            legend[0].legend.css("background-color", layer.shapeSettings.selectionColor);
                            layer._prevSelectedLegends.push(legend[0].legend);
                        }
                        else {
                            var index = layer._prevSelectedLegends.indexOf(legend[0].legend);
				            legend[0].legend[0].setAttribute("class", "mapShape");
                            legend[0].legend.css("background-color", legend[0].legend[0].getAttribute("nodeValue"));
                            layer._prevSelectedLegends.splice(index, 1);
                        }
                    }
                    if ($.inArray(shape, layer._prevSelectedShapes) == -1) {
				        shape.setAttribute("class", "e-mapSelectedShape mapShape");
                        if (_isSVG) {
                            shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                            shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                            shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));                            
                        }
                        else {
                            shape.fillcolor = layer.shapeSettings.selectionColor;
                            shape.strokecolor = layer.shapeSettings.selectionStroke
                            shape.strokeweight = layer.shapeSettings.selectionStrokeWidth / map._scale;
                        }
                        layer._prevSelectedShapes.push(shape);
                        layer.selectedItems.push(event.data.Param1);
                    }
                    else {
				        shape.setAttribute("class", "mapShape");
                        shape.setAttribute("fill", shape.getAttribute("nodeValue"));
                        shape.setAttribute("stroke", layer.shapeSettings.stroke);
                        shape.setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                        var index = layer._prevSelectedShapes.indexOf(shape);
                        layer._prevSelectedShapes.splice(index, 1);
                        layer.selectedItems.splice(index, 1);
                    }
                }

                else {
                     for (var i = 0; i < layer._prevSelectedLegends.length; i++) {
                        layer._prevSelectedLegends[i][0].setAttribute("class", "mapShape");
                        layer._prevSelectedLegends[i].css("background-color", layer._prevSelectedLegends[i][0].getAttribute("nodeValue"));
                    }
                    if ((legend != null && legend.length > 0)) {
                         legend[0].legend.setAttribute("class", "e-mapSelectedShape mapShape");
                         legend[0].legend.css("background-color", layer.shapeSettings.selectionColor);
                         layer._prevSelectedLegends.push(legend[0].legend);
                     }
                   
                    for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                        layer._prevSelectedShapes[i].setAttribute("class", "mapShape");                        
                            if (_isSVG) {
                                layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                                layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                                layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                            }
                            else {
                                layer.fillcolor = layer._prevSelectedShapes[i].getAttribute("nodeValue");
                                layer.strokecolor = layer.shapeSettings.stroke
                                layer.strokeweight = layer.shapeSettings.strokeThickness / this._scale;
                            }                            
                        }
                        if (!this._isPinching && ($.inArray(shape, layer._prevSelectedShapes) == -1)) {
                        shape.setAttribute("class", "e-mapSelectedShape mapShape");                      
                            layer._prevSelectedShapes = [];
                            layer.selectedItems = [];
                            if (_isSVG) {
                                shape.setAttribute("fill", layer.shapeSettings.selectionColor);
                                shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                                shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / map._scale));
                            }
                            else {
                                shape.fillcolor = layer.shapeSettings.selectionColor;
                                shape.strokecolor = layer.shapeSettings.selectionStroke;
                                shape.strokeweight = layer.shapeSettings.selectionStrokeWidth / map._scale;
                            }
                            
                            layer._prevSelectedShapes.push(shape);
                            layer.selectedItems.push(event.data.Param1);
                        }
                        else {
                            layer._prevSelectedShapes=[];
                            layer.selectedItems = [];
                        }
                        
                    

                    
                }
            }
            map._trigger("shapeSelected", { originalEvent: layer.selectedItems });
        },

        _updateSelection: function (layer, shape, data) {
            if (layer.enableSelection) {
                if (layer.selectionMode != ej.datavisualization.Map.SelectionMode.Multiple) {
                    layer._prevSelectedShapes.pop();
                    layer.selectedItems.pop();
                }
                layer._prevSelectedShapes.push(shape);
                layer.selectedItems.push(data);
            }
        },

	   _zoomOnSelection: function (bounds) {            
	       var layerScale;
	       if (!_isSVG) {
	           bounds.x = (bounds.x - (this._baseTranslatePoint.x/2)) - (bounds.width/2);
	           bounds.y = (bounds.y - (this._baseTranslatePoint.y/2)) - (bounds.height / 2);
	       }
	       var boundwidth = bounds.width;
	       var boundHeight =bounds.height;
	       if ((this._width - 100) / (this._height - 100) > boundwidth / boundHeight) {
	           layerScale = (this._height - 100) / boundHeight;
	       } else {
	           layerScale = (this._width - 100) / boundwidth;
	       }
	       this._prevScale = this._scale;
	       this._scale = layerScale;
	       this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
	       this.zoomLevel(this._scale - this._baseScale + 1);
	       var level = this.zoomLevel();
	       if (!(level > this.minZoom() && level < this.maxZoom())) {
	           this.zoomLevel(level > (((this.maxZoom() - this.minZoom()) / 2) + this.minZoom()) ? this.maxZoom() : this.minZoom());
	           this._scale = this.zoomLevel() + this._baseScale - 1;
	       }
	       var leftMargin = (this._width / 2) - ((boundwidth * this._scale) / 2);
	       var leftPos = leftMargin / this._scale;
	       var topMargin = (this._height / 2) - ((boundHeight * this._scale) / 2);
	       var topPos = topMargin / this._scale;
	       this._translatePoint.x = -bounds.x + leftPos;
	       this._translatePoint.y = -bounds.y + topPos;
	       if (this.enableAnimation() && !this._isTileMap) {
	           this._animate(1200);
	       }
	       else {
	           this._applyTransform(this._scale, this._translatePoint);
	           this._updateSliderValue();
	       }	                  
        },
       
          _polyMoveFunction: function (event) {
	       var map = event.data.Param1;
		   if(this.model!=null){
            var baseLayer = this.model.layers[this.baseMapIndex()];
	        var element = map._tooltipElement;
            var template = map.tooltipTemplate;
            var tooltipObject = event.data.Param2;
            var tooltipSize = map._tooltipSize;
			var parentSize=$(this._mapContainer).offset();
			 if (parentSize == null)
                parentSize = { left: 0, top:0, width: 0, height: 0 };

			if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == "right") {
			    width = this._width + baseLayer.legendSettings.width + 20 + (baseLayer.legendSettings.leftLabel.length * 10) + (baseLayer.legendSettings.rightLabel.length * 10);
			}
			else {
                width = this._width;
			}
			var tooltipdisplayheight = this._height;
			var tooltipdisplaywidth = width;

			var xPos = event.pageX - parentSize.left;
			var yPos = event.pageY - parentSize.top;

            var xnewPos = event.pageX;
            var ynewPos = event.pageY;

             if (element != null && template != null && tooltipObject.hasOwnProperty("data") && event.data.Param1.showTooltip) {
                    $(element).css({ "display": "block" });
                    var htmlString = $("#" + template).render(tooltipObject["data"]);
                    $(element).html(htmlString);
                     var height= element[0]!=null ? element[0].clientHeight : element.clientHeight;
					 var width= element[0]!=null ? element[0].clientWidth : element.clientWidth;
					 tooltipSize = { height: height, width: width };
                    
            if (tooltipSize.width + xPos >= tooltipdisplaywidth) {
             if(xnewPos - tooltipSize.width > parentSize.left)
			    xnewPos = ((parentSize.left + tooltipdisplaywidth) - (tooltipdisplaywidth - xPos)) - tooltipSize.width;
             else
                xnewPos -= (tooltipSize.width + xPos - tooltipdisplaywidth);  
			}
			if (tooltipSize.height > tooltipdisplayheight) {
			    if (tooltipdisplayheight + tooltipSize.height > this._height + yPos) 
			        ynewPos -= parentSize.top + 10;
			}
            else if (tooltipSize.height + yPos >= tooltipdisplayheight) {
			    ynewPos -= (tooltipSize.height + yPos - tooltipdisplayheight);
			}  
                $(element).css({ "left": xnewPos + 10, "top": ynewPos + 10});
			}
            }
          },

        _polyLeaveFunction: function (event) {
            var layer = event.data.Param1;
            var map = event.data.map;
            var shape = event.data.Param2.shape;
            var element = layer._tooltipElement;
            var selectedshape = layer._prevSelectedShapes;            
            if (selectedshape.length > 0)
            {
                shape.setAttribute("class", "mapShape");
                for (i = 0; i < selectedshape.length; i++)
                {
                    selectedshape[i].setAttribute("class", "e-mapSelectedShape mapShape");                    
                }                
            }
            else
            {
                shape.setAttribute("class", "mapShape");
            }            
            if (element != null) {
                $(element).css("display", "none");
            }
            if (layer.legendSettings!=null && layer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && layer._interactiveArrow != null)
                $(layer._interactiveArrow).css("display", "none");
            layer._clearShapeWidth(map._scale);
            map._trigger("mouseleave", { originalEvent: event.data.Param2 });
        },

        _wireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._on($(this._mapContainer), ej.eventType.mouseDown, this._mouseButtonDown);
			$(document).keydown({ className: "home", map: this }, this._keyboardKeysPressed);
            this._on($(this._mapContainer), ej.eventType.mouseMove, this._mouseMove);
            this._on($(document), ej.eventType.mouseUp, this._mouseUp);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (isIE11)
                browser = "msie";
            if (window.navigator.msPointerEnabled) {
                this._on($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
                $(this._mapContainer).css('-ms-touch-action', 'none');
            }
            else {
                this._on($(this._mapContainer), "mouseup", this._mouseButtonUp);
            }
            var map = this;
            this._browser = browser;
            if (browser != "mozilla") {
                this._on($(this._mapContainer), "mousewheel", this._mouseWheel);
            }
            else {
                var elem = this._svgDocument;
                if (elem.addEventListener) {
                    // Mouse Scrolling event for firefox
                    elem.addEventListener("DOMMouseScroll", MouseWheel, false);
                }

            }
            function MouseWheel(event) {
			if(map.enableAnimation()){
				$(map._rootgroup).stop(true, false);
			}
			  event.preventDefault(event);
                map._zooming(-40 * event.detail, event);
            }
            this._on($(this._mapContainer), "dblclick", this._doubleClick);
            if (this.model.enableResize) {
                if (!ej.isTouchDevice())
                    this._on($(window), "resize", this._mapResize);
                else
                    this._on($(window), "orientationchange", this._mapResize);
            }
        },

        _mapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var oldSize = { width: this._width, height: this._height };
            var oldTranslatePoint = this._translatePoint;
            var map = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                map.refresh();
                if (!map._isTileMap) {
                    map._translatePoint.x = (oldTranslatePoint.x / oldSize.width) * map._width;
                    map._translatePoint.y = (oldTranslatePoint.y / oldSize.height) * map._height;
                    map._applyTransform(map._scale, map._translatePoint);
                    map._refrshLayers();
                }
            }, 500);
        },

        _isDevice: function () {
            return (/mobile|tablet|android|kindle/i.test(navigator.userAgent.toLowerCase()));
        },
        
        _unWireEvents: function () {
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            this._on($(document), ej.eventType.mouseUp, this._mouseUp);
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            if (isIE11)
                browser = "msie";
			if (browser != "mozilla") {
                this._off($(this._mapContainer), "mousewheel", this._mouseWheel);
            }
            if (window.navigator.msPointerEnabled) {
                this._off($(this._mapContainer), "MSPointerDown", this._mouseButtonDown);
                this._off($(this._mapContainer), "MSPointerMove", this._mouseMove);
                this._off($(this._mapContainer), "MSPointerUp", this._mouseButtonUp);
                $(this._mapContainer).css('-ms-touch-action', 'none');
            }
            else if (browser == "webkit" || browser == "chrome" || browser == "mozilla") {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            else {
                this._off($(this._mapContainer), "mousedown", this._mouseButtonDown);
                this._off($(this._mapContainer), "mousemove", this._mouseMove);
                this._off($(this._mapContainer), "mouseup", this._mouseButtonUp);

            }
            this._off($(this._mapContainer), "dblclick", this._doubleClick);

        },

        navigateTo: function (latitude, longitude, level,isAnimate) {
		     if (isAnimate == undefined) {
                isAnimate = this.enableAnimation();
            }
		    level=parseFloat(level);
            if (level == undefined) level = this.zoomLevel();
            if (level > this.minZoom() && level < this.maxZoom()) {
                this.zoomLevel(level);
            }
            else {
                this.zoomLevel(level > (((this.maxZoom() - this.minZoom()) / 2) + this.minZoom()) ? this.maxZoom() : this.minZoom());

            }
            var factor = this._getFactor();
            var translatePoint = this._convertTileLatLongtoPointForShapes(latitude, longitude, this._mapBounds,factor);
            this._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
            this._prevScale = this._scale;
            this._scale = this._baseScale + ((level - 1) * this.zoomFactor());           
            var leftPosition = ((this._containerWidth + this._baseTranslatePoint.x) / 2) / this._scale;
            var topPosition = ((this._containerHeight + this._baseTranslatePoint.y) / 2) / this._scale;
            this._translatePoint.x = -translatePoint.x+ leftPosition;
            this._translatePoint.y = -translatePoint.y+ topPosition;
            if (isAnimate && !this._isTileMap ){
                this._animate(1200);
            }
            else {
                this._applyTransform(this._scale, this._translatePoint);
            }
            this._updateSliderValue();
            this._refrshLayers();
        },

        selectShape: function (obj, layer,isZoom) {
            if (obj != null) {
                if (layer == null) {
                    layer = this.model.layers[this.baseMapIndex()];
                }
                for (var i = 0; i < layer._mapShapes.length; i++) {
                    var data = layer._mapShapes[i].data;
                    var shape = layer._mapShapes[i].shape;
                    if (data != null && obj == this._reflection(data, layer.shapeSettings.valuePath)) {

                        if (layer._prevSelectedShapes.length != 0 && layer.selectionMode != ej.datavisualization.Map.SelectionMode.Multiple) {
                            for (var i = 0; i < layer._prevSelectedShapes.length; i++) {
                                if (_isSVG) {
                                    layer._prevSelectedShapes[i].setAttribute("fill", layer._prevSelectedShapes[i].getAttribute("nodeValue"));
                                    layer._prevSelectedShapes[i].setAttribute("stroke", layer.shapeSettings.stroke);
                                    layer._prevSelectedShapes[i].setAttribute("stroke-width", layer.shapeSettings.strokeThickness / this._scale);
                                } else {
                                    layer._prevSelectedShapes[i].fillcolor = layer._prevSelectedShapes[i].style.behavior;
                                    layer._prevSelectedShapes[i].strokecolor = layer.shapeSettings.stroke;
                                    layer._prevSelectedShapes[i].strokeweight = layer.shapeSettings.strokeThickness / this._scale;
                                }
                            }
                        }
                        if (layer.enableSelection) {
                            if (_isSVG) {
							    if(layer.shapeSettings.selectionColor!="none"){
                                    shape.setAttribute("fill", layer.shapeSettings.selectionColor);
							    }
                                shape.setAttribute("stroke", layer.shapeSettings.selectionStroke);
                                shape.setAttribute("stroke-width", (layer.shapeSettings.selectionStrokeWidth / this._scale));
                            } else {
							     if(layer.shapeSettings.selectionColor!="none"){
                                      shape.fillcolor = layer.shapeSettings.selectionColor;
								}
                                shape.strokecolor = layer.shapeSettings.selectionStroke;
                                shape.strokeweight = layer.shapeSettings.selectionStrokeWidth / this._scale;
                            }
                            this._updateSelection(layer, shape, obj);
						if (!layer._contains(layer.selectedItems, layer._mapShapes[i]))
                            layer.selectedItems.push(layer._mapShapes[i]);
                            this._trigger("shapeSelected", { originalEvent: layer.selectedItems });
                        }
                        if (isZoom && this.enableZoomOnSelection()) {
                            this._zoomOnSelection(shape.getBBox());
                        }
                        i = layer._mapShapes.length;
                    }
                }
            }

        },

        _getIntersectedElements: function (evt, shapes) {
            evt.width += 5;
            evt.height += 5;
            if (_isSVG && this._browser!='mozilla' && this._browser!='webkit') {
                var rpos = this._svgDocument.createSVGRect();
                rpos.x = evt.left;
                rpos.y = evt.top;
                rpos.width = evt.width;
                rpos.height = evt.height;
                return this._svgDocument.getIntersectionList(rpos, null);
            }
            else {
                var elements = []; 
				var parentSize=$(this._mapContainer).offset();
                var  parentLeft=parentSize.left;
                var parentTop= parentSize.top;			
                for (var i = 0; i < shapes.length; i++) {
                    var shape = shapes[i].shape;
                    var bounds = shape.getBoundingClientRect();					
					var leftPos= $(shape).offset().left- parentLeft;
					var topPos= $(shape).offset().top- parentTop;					
                    if (this._isIntersect(evt, { left: leftPos, top: topPos, height: (bounds.bottom  - bounds.top ), width: (bounds.right -bounds.left) })) {
                        elements.push(shape);
                        return elements;
                    }
                }
                return elements;
            }
                
                
        },

        _isIntersect: function (rect1, rect2) {           
            
            if (rect1.left >= (rect2.left + rect2.width) || rect1.left + rect1.width <= rect2.left ||
                rect1.top >= rect2.top + rect2.height || rect1.top + rect1.height <= rect2.top) {
                    return false;
                }
                return true;
            
        },

        pan: function (direction) {
            var map = this;
            var Xdiff = 0;
            var Ydiff = 0;
            if (this.zoomLevel() != 1) {
                switch (direction) {

                    case 'right':
                        {
                            Xdiff = this._width / 7;
                            Ydiff = 0;
                            break;
                        }
                    case 'top':
                        {
                            Xdiff = 0;
                            Ydiff = -(this._height / 7);
                            break;
                        }n
                    case 'left':
                        {
                            Xdiff = -(this._width / 7);
                            Ydiff = 0;
                            break;
                        }
                    case 'bottom':
                        {
                            Xdiff = 0;
                            Ydiff = this._height / 7;
                            break;
                        }
                }
                if (map.enablePan()) {
                    if (this._isTileMap) {
                        var curtileX = this._tileTranslatePoint.x - Xdiff / map._scale;
                        var curtileY = this._tileTranslatePoint.y - Ydiff / map._scale;
                        this._tileTranslatePoint.x = curtileX;
                        this._tileTranslatePoint.y = curtileY;
                        this._generateTiles(this.zoomLevel());
                    }                    
                    var curX = map._translatePoint.x - Xdiff / map._scale;
                    map._prevScale = map._scale;
                    var curY = map._translatePoint.y - Ydiff / map._scale;
                    map._prevPoint = { x: this._translatePoint.x, y: this._translatePoint.y };
                    this._translatePoint.x = curX;
                    this._translatePoint.y = curY;
                    if (map.enableAnimation() && !this._isTileMap) {                       
                        map._animate(600);
                    }
                    else {
                        map._applyTransform(map._scale, map._translatePoint);
                    }
                    map._refrshLayers();
                }
            }
			
        },      

        zoom: function (level,isAnimate) {
           var map = this;
		    var prevLevel = map.zoomLevel();
            if (level <= this.maxZoom() && level >= this.minZoom()) {
                if (this._isTileMap) {
                    this._tileZoom(map.zoomLevel(), level, {x:this._width/2,y:this._height/2});
                    var prevLevel = map.zoomLevel();
                    map.zoomLevel(level);
                    this._generateTiles(this.zoomLevel());
                    map._translatePoint.x = (map._tileTranslatePoint.x - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._translatePoint.y = (map._tileTranslatePoint.y - (0.5 * Math.pow(2, prevLevel))) / (Math.pow(2, prevLevel));
                    map._scale = (Math.pow(2, prevLevel));					
                }
                else {                    
                    var fac = (map._baseScale + ((level - 1) * map.zoomFactor()));
                    map._prevPoint = { x: map._translatePoint.x, y: map._translatePoint.y };
                    map._prevScale = map._scale;
                    map._translatePoint.x -= (map._width / map._scale - map._width / fac) / 2;
                    map._translatePoint.y -= (map._height / map._scale - map._height / fac) / 2;
                    map._scale = fac;
                    map.zoomLevel(level);

                    if (map.enableAnimation() && isAnimate || isAnimate == undefined) {
                        map._animate(600);
                    }                  
                }
				if(!map.enableAnimation() || !isAnimate)
				{
				     map._applyTransform(map._scale, map._translatePoint);
                     map._refrshLayers();
                     map._resizeShape();       
				}
				 map._updateSliderValue();                  
                
            }
            else if (level <= this.minZoom()) {
                this.zoomLevel(this.minZoom());
                this.zoom(this.zoomLevel());
            }
            else if (level >= this.maxZoom()) {
                this.zoomLevel(this.maxZoom());
                this.zoom(this.zoomLevel());
            }
			  if (prevLevel < level) {
                map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map.zoomLevel() });
            }
            else  if (prevLevel > level) {
                map._trigger("zoomedOut", { originalEvent: null, zoomLevel: map.zoomLevel() });
            }
			
        },

       refresh: function () {
            
            this._unWireEvents();
            $(this._svgDocument).empty();
            if (this._svgDocument != null) {
                this._svgDocument = null;
            }
            $(this._mapContainer).empty();
            this._scale = 1;
            this._margintop = 0;
            this._marginleft = 0;
            this._translatePoint = { x: 0, y: 0 };            
            this._height = this._mapContainer.height();
            this._width = this._mapContainer.width();
             if (this._height == 0) {
                 this._height = this._mapContainer[0].parentElement.clientHeight != 0 ? this._mapContainer[0].parentElement.clientHeight : $(document).height();
            }
            if (this._width == 0) {
                this._width = this._mapContainer[0].parentElement.clientWidth != 0 ? this._mapContainer[0].parentElement.clientWidth : $(document).width();
            }
            if (this.baseMapIndex() >= this.model.layers.length) {
                this.baseMapIndex(0);
            }
			 var baseLayer = this.model.layers[this.baseMapIndex()];
			 if (baseLayer.legendSettings.showLegend && baseLayer.shapeSettings.colorMappings != null) {
                baseLayer._sizeCalculation(this);
            }
            this._generatePath();           
            if (this._groupSize != null) {
                this._containerHeight = this._groupSize.maxY - this._groupSize.minY;
                this._containerWidth = this._groupSize.maxX - this._groupSize.minX;
                this._groupBorder = { x: this._groupSize.minX, y: this._groupSize.minY };
            }
            this._resizingContainer();
            this._renderMapElements();
            
            if ((baseLayer.legendSettings != null && baseLayer.legendSettings.showLegend) &&(baseLayer.shapeSettings.colorMappings!=null)) {
                
                    this._renderLegend();
                
            } 
           if (_isSVG) {
               $(this._svgDocument).css({ height: this._height, width: this._width, "margin-top": this._margintop, "margin-left": this._marginleft });
               
            }			
            if (this.model.enableLayerChangeAnimation) {
                $(this._mapContainer).animate({ opacity: 1 }, 500);
            }
            if(this.model.centerPosition!=null && !this._isTileMap){
            this.navigateTo(this.model.centerPosition[0],this.model.centerPosition[1],this.zoomLevel(),false);
            }
            
        },

        _updateSliderValue: function (isAnimate) {
            var slider = this._sliderControl;
            if (isAnimate == undefined) isAnimate = this.enableAnimation();
            if (slider != null) {
                var obj = slider.data("ejSlider");               
                obj.option("value", this.zoomLevel());
            }
        },

        _createDivElement: function (parent, child,base,classname) {
            var parentElement = parent;
            var childElement = child;           
            childElement.appendTo(parentElement);           
            parentElement.appendTo(base);            
            this._on(childElement,ej.eventType.mouseDown,{ className: classname, map: this }, this._navigationControlPressed);
            
        },

        _navigationControlPressed: function (event) {
            
            event.stopPropagation();
            this._isNavigationPressed = true;
            var map = event.data.map;
            var isAnimate = map.enableAnimation();            
            if (event.data.className == "zoomIn") {               
                map.zoom(map.zoomLevel() + 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
				 map._refrshLayers();
            }
            else if (event.data.className == "zoomOut") {               
                map.zoom(map.zoomLevel() - 1, true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }
            else if (event.data.className == "panLeft") {
                map.pan("left");                
            }
            else if (event.data.className == "panRight") {
                map.pan("right");
            }
            else if (event.data.className == "panTop") {
                map.pan("top");
            }
            else if (event.data.className == "panBottom") {
                map.pan("bottom");
            }
            else if (event.data.className == "home") {                
                map.zoom(1,true);
                if (!isAnimate) {
                    map._updateSliderValue();
                }
            }            
           
        },
	
		 _keyboardKeysPressed: function (event) {
            this._iskeyboardKeysPressed = true;
            var map = event.data.map;
            var isAnimate = map.enableAnimation();
            if (event.ctrlKey && event.keyCode==38) {    			
                map._zoomingIn(map._width / 2, map._height / 2, event, isAnimate);
            }
            else if (event.ctrlKey && event.keyCode==40) {    
                map._zoomingOut(map._width / 2, map._height / 2, event, isAnimate);
            }
            else if (event.keyCode==37) {
                map.pan("left");                
            }
            else if (event.keyCode==39) {
                map.pan("right");
            }
            else if (event.keyCode==38) {
                map.pan("top");
            }
            else if (event.keyCode==40) {
                map.pan("bottom");
            }
		},
				
        refreshNavigationControl: function (navigation) {
            
            var baseLayer = this.model.layers[this.baseMapIndex()];
            var prevNav = this._mapContainer.find("#ejNavigation");
            if (prevNav.length > 0) {
                this._mapContainer[0].removeChild(prevNav[0]);
            }
            if (this.model.navigationControl != null && this.enableNavigation()) {
                if (navigation == undefined) {
                    navigation = this.model.navigationControl;
                }
                var navigationOrientation = ej.Orientation.Vertical;
                var controlSize;
                var navigationHeight = 120;
                var navigationWidth = 12;
                controlSize = { width: 90, height: 320 };
                var navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-vert'/>");
                if (this.model.navigationControl.content == null) {
                    var sliderDiv = $("<div style='height:120px;width:10px;margin-top:-197px;margin-left: 34px;' />");
                }
                if (navigation.orientation == 'horizontal') {
                    navigationOrientation = ej.Orientation.Horizontal;
                    navigationHeight = 12;
                    navigationWidth = 120;
                    controlSize = { width: 320, height: 90 };
                    navigationControl = $("<div id='ejNavigation' class='e-map-navigation e-orientation-horz' />");
                    if (this.model.navigationControl.content == null) {
                        sliderDiv = $("<div style='height:10px;width:120px;margin-top:-18px' />");
                    }
                }
                if (this.model.navigationControl.content == null) {
                    var baseDiv = $("<div class='e-panContainer'/>");
                    var isHor = navigationOrientation == 'horizontal' ? 'Horz' : 'Vert';
                    if (navigation.orientation == 'horizontal') {
                        var zoominOutDiv = $("<div style='margin-left: 94px;' />");
                        var slidercontrol = $("<div style='margin-left: 137px;'/>");
                    }
                    else {
                        var zoominOutDiv = $("<div />");
                        var slidercontrol = $("<div style='margin-top: 34px;'/>");
                    }
                    baseDiv.appendTo(navigationControl);
                    zoominOutDiv.appendTo(navigationControl);
                    sliderDiv.appendTo(slidercontrol);
                    slidercontrol.appendTo(navigationControl);
                    this._sliderControl = sliderDiv;
                    var navPos = { x: 0, y: 0 };
                    if (navigation.dockPosition == null || navigation.dockPosition == ej.datavisualization.Map.Position.None) {
                        navPos.x = (this._width * navigation.absolutePosition.x) / 100;
                        navPos.y = (this._height * navigation.absolutePosition.y) / 100;
                    } else {
                        navPos = this._getPosition(navigation.dockPosition, controlSize);
                    }
                    navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                }
                var map = this;
				var changeZoom = this._height * 0.0025;
                if (this.model.navigationControl.content == null) {
                    if (this._browser != 'chrome' && this._browser != 'msie') {
                        navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 320px;position:absolute;z-index:2;zoom:' + changeZoom + ';}</style>');
                    }
                    else {
                        navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 320px;position:absolute;z-index:2;-moz-transform: scale(' + changeZoom + ');}</style>');
                    }
                }
                else {
                    if (this._browser != 'mozilla' && this._browser != 'chrome') {
                        navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 320px;position:absolute;z-index:2;}</style>');
                    }
                    else {
                        navigationStyle = $('<style> .e-map-navigation {width: 90px;height: 320px;position:absolute;z-index:2;}</style>');
                    }
                }
				if (this._browser!='mozilla' && this._browser!='chrome'){
				navigationStyle.remove();
				$('html > head').append(navigationStyle);
				}
				else{
				$('html > head').append(navigationStyle);
				}
				if (this.model.navigationControl.content == null) {
				    $(sliderDiv).ejSlider({
				        orientation: navigationOrientation,
				        sliderType: ej.SliderType.MinRange,
				        value: 1,
				        animationSpeed: 1200,
				        minValue: this.minZoom(),
				        showTooltip: true,
				        enableAnimation: false,
				        maxValue: this.maxZoom(),
				        incrementStep: this.zoomFactor(),
				        slide: onslide,
				        change: onchange,
				        height: navigationHeight,
				        width: navigationWidth
				    });
				    navigationControl.appendTo(this._mapContainer);
				    if (navigation.orientation == 'horizontal') {
				        this._createDivElement($("<div title='ZoomIn' class='e-icon1 e-incrementButton  icon_margin1' />"), $("<div class='e-icon1 nav-inc-" + isHor + "  e-innerIncrement'/>"), zoominOutDiv, "zoomIn");
				        this._createDivElement($("<div title='ZoomOut'class='e-icon1 e-incrementButton icon_margin2'/>"), $("<div class='e-icon1 nav-dec-" + isHor + " e-innerDecrement'/>"), zoominOutDiv, "zoomOut");
				    }
				    else {
				        this._createDivElement($("<div title='ZoomIn' class='e-icon1 e-incrementButton  icon_margin1' />"), $("<div class='e-icon1 nav-inc-" + isHor + "  e-innerIncrement'/>"), zoominOutDiv, "zoomIn");
				        this._createDivElement($("<div title='ZoomOut' class='e-icon1 e-incrementButton icon_margin2'/>"), $("<div class='e-icon1 nav-dec-" + isHor + " e-innerDecrement'/>"), zoominOutDiv, "zoomOut");
				    }
				    this._createDivElement($("<div title='PanTop' class='e-icon1 e-radialTop'/>"), $("<div class='e-icon1 e-arrowUp'/>"), baseDiv, "panTop");
                    this._createDivElement($("<div title='PanLeft' class='e-icon1 e-radialLeft'/>"), $("<div class='e-icon1 e-arrowLeft'/>"), baseDiv, "panLeft");
				    this._createDivElement($("<div title='PanRight' class='e-icon1 e-radialRight'/>"), $("<div class='e-icon1 e-arrowRight'/>"), baseDiv, "panRight");
                    this._createDivElement($("<div title='PanBottom' class='e-icon1 e-radialBottom'/>"), $("<div class='e-icon1 e-arrowDown'/>"), baseDiv, "panBottom");
                    var homeDiv = $("<div title='Home' class='e-icon1 e-home-bg'><div class='e-icon1 e-map-home'></div>");
				    homeDiv.appendTo(baseDiv);
				    homeDiv.mousedown({ className: "home", map: this }, this._navigationControlPressed);
				}
				else {
                    if (this.navigationControlData == null) {
                        this.navigationControlData = $("#" + this.model.navigationControl.content);
                        this.navigationControlData.css({ 'display': 'block' });                            
                        this.navigationControlData.appendTo(navigationControl);
                    }
                    else {                            
                        this.navigationControlData.appendTo(navigationControl);
                    }                                                
                    navigationControl.appendTo(this._mapContainer);
                                       
				    controlSize = { width: this.navigationControlData[0].getBoundingClientRect().right - this.navigationControlData[0].getBoundingClientRect().left, height: this.navigationControlData[0].getBoundingClientRect().bottom - this.navigationControlData[0].getBoundingClientRect().top };                    
                    var navPos = { x: 0, y: 0 };
                    if (navigation.dockPosition == null || navigation.dockPosition == ej.datavisualization.Map.Position.None) {
                        navPos.x = (this._width * navigation.absolutePosition.x) / 100;
                        navPos.y = (this._height * navigation.absolutePosition.y) / 100;
                    } else {
                        navPos = this._getPosition(navigation.dockPosition, controlSize);
                    }

                    if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
                        navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + baseLayer.legendSettings.height  + "px" });
                    }
                    else if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
                            navigationControl.css({ "margin-left": navPos.x + baseLayer.legendSettings.width + 20 + (baseLayer.legendSettings.leftLabel.length * 10) + (baseLayer.legendSettings.rightLabel.length * 10)+ "px", "margin-top": navPos.y + "px" });
                    }
                    else if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default) {
                        navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + baseLayer.legendSettings.height + "px" });
                    }
                    else if (baseLayer.legendSettings.dockOnMap && baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left && baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default) {
                        navigationControl.css({ "margin-left": navPos.x + baseLayer.legendSettings.width + "px", "margin-top": navPos.y + "px" });
                    }
                    else {
                        navigationControl.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                    }
                }
            }                     

			function onchange(args) {
                if (map != null && map._isRendered && map.zoomLevel() != args.value) {
                    map.zoom(args.value, false);
                }
            
            }

            function onslide(args) {
                if (map != null && map._isRendered && map.zoomLevel() != args.value) {
                    map.zoom(args.value, false);
                }
            }
        },

        _getPosition: function (position,size) {
            var Position = { x: 0, y: 0 };
            switch (position) {
                case ej.datavisualization.Map.Position.TopLeft:                   
                    break;
                case ej.datavisualization.Map.Position.TopCenter:
                    Position.x = (this._width / 2) - (size.width / 2);
                    break;
                case ej.datavisualization.Map.Position.TopRight:
                    Position.x = this._width - size.width;
                    break;
                case ej.datavisualization.Map.Position.CenterLeft:
                    Position.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.Position.Center:
                    Position.x = (this._width / 2) - (size.width / 2);
                    Position.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.Position.CenterRight:
                    Position.x = this._width - size.width;
                    Position.y = (this._height / 2) - (size.height / 2);
                    break;
                case ej.datavisualization.Map.Position.BottomLeft:
                    Position.y = this._height - size.height;
                    break;
                case ej.datavisualization.Map.Position.BottomCenter:
                    Position.x = (this._width / 2) - (size.width / 2);
                    Position.y = this._height - size.height;
                    break;
                case ej.datavisualization.Map.Position.BottomRight:
                    Position.x = this._width - size.width;
                    Position.y = this._height - size.height;
                    break;

            }
            return Position;
        },

        _renderMapElements: function () {           
            this._templateDiv = $("<div class='TemplateDiv'/>");
            this._templateDiv.appendTo(this._mapContainer);
            this._templateDiv.css({
                'pointer-events': 'none', 'overflow': 'hidden', "position":"absolute",
                'z-index': '1', 'height': this._height, 'width': this._width
            });			
            this._wireEvents();
            this.refreshLayers();       
            this.refreshNavigationControl(this.model.navigationControl);
        },
		
        _renderLegend: function () {

            var baseLayer = this.model.layers[this.baseMapIndex()];
            this._legendDiv = $("<div class='LegendDiv'/>");
            this._legendDiv.appendTo(this._mapContainer);

            if (baseLayer.legendSettings.labelOrientation == undefined)
                baseLayer.legendSettings.labelOrientation == ej.datavisualization.Map.LabelOrientation.Vertical;
            if (!baseLayer.legendSettings.dockOnMap) {
                if (baseLayer.legendSettings.labelOrientation == ej.datavisualization.Map.LabelOrientation.Horizontal || baseLayer.legendSettings.labelOrientation == ej.datavisualization.Map.LabelOrientation.Vertical) {

                    var position = (baseLayer.legendSettings.position == undefined) ? "topleft" : baseLayer.legendSettings.position;
                    var legendPos = this._getPosition(position, this._legendSize);
                    this._legendDiv.css({
                         "position": "absolute",
                         'z-index': '2', 'height': this._legendSize.height, 'width': this._legendSize.width,
                         "margin-left": legendPos.x + "px", "margin-top": legendPos.y
                    });
                }
            }
            else {

                if (baseLayer.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
                    pos = "none";
                    height = baseLayer.legendSettings.height + 50;
                    width = baseLayer.legendSettings.width + 20 + (baseLayer.legendSettings.leftLabel.length * 10) + (baseLayer.legendSettings.rightLabel.length * 10);
                }
                else {
                    pos = "auto";
                    height = baseLayer.legendSettings.height;
                    width = baseLayer.legendSettings.width;
                }
                this._legendDiv.css({
                    "position": "absolute",
                    'z-index': '2', 'height': this._legendSize.height, 'width': this._legendSize.width
                });

                if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Center) {
                    this._legendDiv.css({
                        "margin-left": (this._width / 2) - (this._legendSize.width / 2)
                    });
                }
                else if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Far) {
                    this._legendDiv.css({
                        "margin-left": this._width - this._legendSize.width
                    });
                }
                else if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Center) {
                    this._legendDiv.css({
                        "margin-top": (this._height / 2) - (this._legendSize.height / 2)
                    });
                }
                else if ((baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left || baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) && baseLayer.legendSettings.alignment == ej.datavisualization.Map.Alignment.Far) {
                    this._legendDiv.css({
                        "margin-top": this._height - this._legendSize.height
                    });
                }

                if (baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) {
                    this._legendDiv.css({
                        "margin-top": this._height + 5
                    });
                }
                else if (baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) {

                    this._legendDiv.css({

                        "margin-left": this._width + 5

                    });
                }
            }

           
            this.refreshLayers();
            this.refreshNavigationControl(this.model.navigationControl);
        },
       
		_renderMapLayers:function(){
		
		 $(this._templateDiv).empty();
            var prevPathCount = 0;
            var baseLayer = this.model.layers[this.baseMapIndex()];
            if (baseLayer.layerType == ej.datavisualization.Map.LayerType.Geometry) {
                this._renderLayers(baseLayer, prevPathCount);
                if (baseLayer.shapeData != null)
                    prevPathCount = baseLayer._polygonData.length;
            }
            else {
                baseLayer._initializeLocalValues();
                var contribution = $("<div class='map-contribution'>");
                contribution[0].innerHTML = baseLayer.contribution;
                this._mapContainer.append(contribution);
            }
            for (var key = 0; key < this.model.layers[this.baseMapIndex()].subLayers.length; key++) {
                var sublayer = this.model.layers[this.baseMapIndex()].subLayers[key];
                sublayer._isBaseLayer = false;
                if (sublayer.layerType == ej.datavisualization.Map.LayerType.Geometry && sublayer.shapeData!=null) {
                    this._renderLayers(sublayer, prevPathCount);
                    prevPathCount += sublayer._polygonData.length;
                }
                else {
                    sublayer._initializeLocalValues();
                }
            }
            this.refreshMarkers();
            if (_isSVG && this.enableAnimation() && !this._isTileMap) {
                this._refreshWithAnimation();
            }
            else {
                this._refrshLayers();
            }

		},

        refreshLayers: function () {          
		   var baseLayer = this.model.layers[this.baseMapIndex()];
		   this._processOData(baseLayer,this);          
		   this.shapeSelectionOnResize();
        },
		
		_processOData:function(layer,map){		    
		    if(layer.dataSource!=null){
                      if (layer.dataSource instanceof ej.DataManager) {
				          var queryPromise = layer.dataSource.executeQuery(layer.query);
			             queryPromise.done(function (e) {
			             if(e.result!=null)
				         {
                            layer.dataSource=e.result;
						    map._renderMapLayers();
				         }
			           });        
				  }
				 else
				 {
				    map._renderMapLayers();
				 }
            }
			else
			{
			  map._renderMapLayers();
			}
		}
		
		

    });

 

    var Tile = function (x, y) {
        this.X = x;
        this.Y = y;
        this.left = 0;
        this.top = 0;       
        this.height = 256;
        this.width = 256;
        this.src = null;
    };
   
    var shapeLayer = function () {
                
        this.enableSelection = true;
        this.selectionMode = "default";		
		this.bingMapType = "aerial";	
        this.key = "";   
        this.selectedItems = [];        
        this.enableMouseHover = false;
        this.shapeData = null;
        this.markers = [];
        this.dataSource = null;
        this.urlTemplate = 'http://a.tile.openstreetmap.org/level/tileX/tileY.png';
        this.showTooltip = false;
        this.tooltipTemplate = null;
        this.mapItemsTemplate = null;
        this.enableAnimation = false;

        this.legendSettings = {		
			showLegend:false,		
			showLabels:false,	
            legendPadding:15,			
			position: "topleft",			
			positionX: 0,		
			positionY:0,				
			height:0, 		
			width:0,
			iconHeight: 20,
			iconWidth: 20,		
			type:"layers", 		
			mode:"default", 					
			title: "", 			
			leftLabel:"", 						
			rightLabel:"",			
			icon:"rectangle",						
			dockOnMap:false,					
			dockPosition: "top",					
            labelOrientation: "vertical",
			alignment: "bottom",
            columnCount:0
        };

        this.labelSettings = {			
			showLabels: false, 						
			labelPath: "",			
			enableSmartLabel: false,			
			smartLabelSize:"fixed",			
			labelLength:2
		};        
        this.markerTemplate = null;       
		this.showMapItems = false;      
        this.layerType = 'geometry';
        
        this._colorPaletteSettings = {
            'Palette1':
            {
                'highlightColor': '#F7CD1C',
                'highlightStroke': 'white',
                'highlightBorderWidth': '1',
                'SelectionColor': '#F96C0D',
                'SelectionStroke': 'white',
                'SelectionStrokeWidth': '2.5'
            },
            'Palette2':
            {
                'highlightColor': '#68A3EA',
                'highlightStroke': 'White',
                'highlightBorderWidth': '1',
                'SelectionColor': '#116EF4',
                'SelectionStroke': 'White',
                'SelectionStrokeWidth': '2.5'
            },
            'Palette3':
            {
                'highlightColor': '#CCCCCC',
                'highlightStroke': 'white',
                'highlightBorderWidth': '1',
                'SelectionColor': '#4D4D4D',
                'SelectionStroke': 'white',
                'SelectionStrokeWidth': '2.5'
            }
        };
        this.colorPalettes = {
            'Palette1': ['#4A3825', '#736F3D','#F2DABD','#BF9D7E','#7F6039','#7F715F','#70845D','#CC995C','#736F3D','#89541B'],
            'Palette2': ['#E51400', '#730202','#EF6535','#C63477','#BF004D','#F0A30B','#CE1B1B','#97993D','#D6BF38','#835A2C'],
            'Palette3': ['#A4C400', '#008B00','#1BA0E2','#0050EF','#AA00FF','#D90073','#E51400','#F96800','#E3C800','#A20026']
        };
        
        this._prevPaletteIndex = 0;        
        this._newBounds = [];      
        this.subLayers = [];

        this.shapeSettings = {           
            highlightColor: "gray",          
            highlightBorderWidth: 1,            
            selectionColor: "gray",
            fill: "#E5E5E5",          
            strokeThickness: "0.2",            
            stroke: "#C1C1C1",
            colorPalette: 'palette1',
            highlightStroke: "#C1C1C1",
            selectionStroke: "#C1C1C1",           
            selectionStrokeWidth: 1,
            colorValuePath : null,
            valuePath: null,
            enableGradient: false,        
            colorMappings: null,
            autoFill: false
        };

       
        this.bubbleSettings = {            
            showBubble: true,          
            bubbleOpacity: "0.9",         
            minValue: 10,
            maxValue: 20,
            color: "gray",
            colorValuePath : null,
            valuePath: null,
            colorMappings: null,
            showTooltip: false,
            tooltipTemplate: null
        };
    };

    shapeLayer.prototype =
    {        
        dataTypes: {
            dataSource: "data",
            markers: "array",
            subLayers: "array",
            shapeSettings: {
                colorMappings: "array"
            },
            bubbleSettings: {
                colorMappings: "array"
            }

        },
       
        _initializeLocalValues:function()
        {
            this._svgns = "http://www.w3.org/2000/svg";
            this._bounds = [];
            this._bubbleCollection = [];
            this._prevSelectedShapes = [];
            this._prevSelectedLegends = [];
            this._isBaseLayer = true;
            this._prevHoverdLegend = null;
            this._prevHoveredShape = null;
            this._labelCollection = [];
            this._scrollBar = null;
            this._mapShapes = [];
            this._bubbles = [];
            this._labelBounds = [];            
            this._bubbleCount = 0;
            this._mapItems = [];           
            this._mapMarkers = [];
            this.selectedItems;
			this._tooltipSize = { height: 0, width: 0 };
            this._smartLabels = [];
            this._labelData = [];
            this._interactiveArrow = null;
            this._legendRects = [];

        },
     
        _generateMarkerForLayers: function (map) {
            this._mapMarkers = [];
            var rootTop = map._rootgroup.getBoundingClientRect().top;
            var rootLeft = map._rootgroup.getBoundingClientRect().left;
            for (var key = 0; key < this.markers.length; key++) {
                var markerObeject = this.markers[key];               
                if (this.markerTemplate != null) {
                    var markerTemplateDiv = $('.markerTemplateDiv');
                    markerTemplateDiv = $("<div class='mapMarker' style='display:block;position:absolute;pointer-events: stroke;'></div>");                   
                    map._templateDiv.append(markerTemplateDiv);
                    $(markerTemplateDiv).css({ height: this._height, width: this._width, "margin-top": this._margintop, "margin-left": this._marginleft });
                    var htmlString = $("#" + this.markerTemplate).render(markerObeject);
                    $(markerTemplateDiv).html(htmlString);
                    map._on($(markerTemplateDiv), ej.eventType.mouseDown, { marker: $(markerTemplateDiv), data: markerObeject }, map._markerPressed);
                    this._mapMarkers.push(markerTemplateDiv);
                }
                else {                    
                    var markerItem = $(' <div class="mapMarker" style="display:block;position:absolute;pointer-events: stroke;"><label>' + markerObeject.label + '</label></div>');// document.createElementNS(this._svgns, "text");
                    if (_isSVG) {
                        map._templateDiv.append(markerItem);
                    }
                    else {
                        markerItem.appendTo(map._templateDiv);
                    }
                    map._on($(markerItem), ej.eventType.mouseDown, { marker: $(markerItem), data: markerObeject }, map._markerPressed);
                    this._mapMarkers.push(markerItem);

                }
                  var baseLayer = this;
                if (baseLayer.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left) {
                    this._height = this._height;
                    this._width = this._width - parseFloat(width);
                    this._marginleft = parseFloat(width);
             }            
            }            
        },

        _contains : function(array,actualobj) {
            var length = array.length;
            if (length > 0) {
                while (length--) {
                    if (array[length] === actualobj) {
                        return true;
                    }
                }
            }
            return false;
        },
       
          _shapeContains: function (array, actualobj) {
            var length = array.length;
            if (length > 0) {
                while (length--) {
                    if (array[length].shapeIndex === actualobj) {
                        return { isContains: true, index: array.length - length };
                    }
                }
            }
            return { isContains: false };

        },  
		
		 _sizeCalculation: function (map) {

            var colorMappings = this.shapeSettings.colorMappings;
            var isRange = false;
            if (this.shapeSettings.colorMappings != null) {

                if (colorMappings.rangeColorMapping != null)
                {
                    colorMappings = colorMappings.rangeColorMapping;
                    isRange = true;
                }
                if (colorMappings.equalColorMapping != null)
                {
                    colorMappings = colorMappings.equalColorMapping;
                }
                        
                var width = 0;
                var height = 0;
				var legendPadding = this.legendSettings.legendPadding;
                var iconWidth = this.legendSettings.iconWidth + 5;
                var iconHeight = this.legendSettings.iconHeight + 5;
                var legendHeight = iconHeight + legendPadding;
                var totalHeight = this.legendSettings.iconHeight + legendPadding;
                var totalWidth = 0;
                var yPos = 10;
                var xPos = 10;
                var legendSettings = this.legendSettings;
                var columnCount = this.legendSettings.columnCount;
                if (legendSettings.height == 0 && legendSettings.width == 0  && columnCount == 0) {
                    for (var i = 0; i < colorMappings.length; i++) {
                        var label = colorMappings[i].legendLabel != null ? colorMappings[i].legendLabel : (isRange ? colorMappings[i].from : colorMappings[i].value);
                        var labelwidth = this._calcWidth(label);
                        var legendWidth = iconWidth + labelwidth + legendPadding;
                        
                        if ((legendSettings.dockOnMap && (legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom))||!legendSettings.dockOnMap) {
                            if (map._width < (width + legendWidth)) {
                                totalHeight = ((iconHeight) + legendPadding) + totalHeight;
                                totalWidth = Math.max(totalWidth, width);
                                width = legendWidth;
                            }
                            else {
                                width += (legendWidth + 5);
                            }
                        }
                        else {

                            if (map._height < (height + iconHeight + legendPadding)) {
                                totalWidth += width;
                                totalHeight = Math.max(totalHeight, height);
                                width = legendWidth;
                                height = 0;
                            }
                            else {
                                height += (iconHeight + legendPadding);
                                width = Math.max(legendWidth, width);
                            }
                        }
                        

                    }
                    if ((legendSettings.dockOnMap && (legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom)) || !legendSettings.dockOnMap) {
                        totalWidth = Math.max(totalWidth, width);
                        totalHeight += 5;
                    }
                    else {
                        totalWidth += (width + legendPadding);
                        totalHeight = Math.max(totalHeight, height);
                    }

                }
                else if (columnCount == 0) {
                    if ((legendSettings.height.toString().indexOf("%")) != -1) {

                        totalHeight = (map._height / 100) * parseInt(legendSettings.height.toString().replace('%', ''))
                    }
                    else {

                        totalHeight = legendSettings.height;
                    }
                    if ((legendSettings.width.toString().indexOf("%")) != -1) {

                        totalWidth = (map._width / 100) * parseInt(legendSettings.width.replace('%', ''))
                    }
                    else {

                        totalWidth = legendSettings.width;
                    }

                }

                if (legendSettings.height == 0 && legendSettings.width == 0 && columnCount != 0) {

                    for (var i = 0; i < colorMappings.length; i++) {
                        var label = colorMappings[i].legendLabel != null ? colorMappings[i].legendLabel : (isRange ? colorMappings[i].from : colorMappings[i].value);
                        var labelwidth = this._calcWidth(label);
                       
                        var legendWidth = iconWidth + labelwidth + legendPadding-5;

                        if (i % columnCount != 0) {
                            width += legendWidth;
                            if (i == columnCount - 1) {
                                totalWidth = Math.max(totalWidth, width);
                            }

                        }
                        else {
                            if (i != 0)
                                totalHeight = (iconWidth + legendPadding) + totalHeight;
                            totalWidth = Math.max(totalWidth, width);
                            width = legendWidth;
                        }
                    }
                }
            }
        
        
            if (legendSettings.height != 0 && legendSettings.width != 0)
			{
				 totalHeight = legendSettings.height;
				 totalWidth = legendSettings.width;
            }

			map._legendSize = { height: totalHeight, width: totalWidth };
			if (legendSettings.dockOnMap) {
			    if (legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive) {
			        totalHeight = 55;
			    }
			    if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) {
			        map._height = map._height - parseFloat(totalHeight);
			        this.legendSettings.tempWidth = width;
			    }

			    else if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top) {
			        map._height = map._height - parseFloat(totalHeight);
			        map._margintop = parseFloat(totalHeight);
			    }
			    else if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Left) {

			        map._width = map._width - totalWidth;
			        map._marginleft = width;
			    }
			    else if (this.legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Right) {
			        map._height = map._height;
			        map._width = map._width - totalWidth;
			    }
			}
                
              
            
        },

        
        _clearShapeWidth: function (scale) {
            if (scale == null) scale = 1;
            for (var index = 0; index < this._mapShapes.length; index++) {
                var mapshape = this._mapShapes[index];
                if (!this._contains(this._prevSelectedShapes,mapshape.shape)) {
                    if (_isSVG) {
                        mapshape.shape.setAttribute("fill", mapshape.shape.getAttribute("nodeValue"));
                        mapshape.shape.setAttribute("stroke-width", (this.shapeSettings.strokeThickness / scale));
                        mapshape.shape.setAttribute("stroke", this.shapeSettings.stroke);
                    }
                    else {
                        mapshape.shape.fillcolor = mapshape.shape.style.behavior;
                        mapshape.shape.strokeweight = this.shapeSettings.strokeThickness;
                        mapshape.shape.strokecolor = this.shapeSettings.stroke;
                    }
                }
            }
            


        },
			
       _shapeSelection: function () {
            for (var index = 0; index < this._mapShapes.length; index++) {
                var mapshape = this._mapShapes[index];
                var check = this._shapeContains(this.selectedItems, mapshape.shapeIndex);
                if (check.isContains) {
                     if (!this._contains(this._prevSelectedShapes, mapshape.shape))
                     this._prevSelectedShapes.push(mapshape.shape);
                     if (_isSVG) {
                        mapshape.shape.setAttribute("fill", this.shapeSettings.selectionColor);
                        mapshape.shape.setAttribute("stroke-width", this.shapeSettings.selectionStrokeWidth);
                        mapshape.shape.setAttribute("stroke", this.shapeSettings.selectionStroke);
                    }
                    else {
                        mapshape.shape.fillcolor = this.shapeSettings.selectionColor;
                        mapshape.shape.strokeweight = this.shapeSettings.selectionStrokeWidth;
                        mapshape.shape.strokecolor = this.shapeSettings.selectionStroke;
                    }
                }
            }
        },
                
        _rgbToHex: function(R, G, B) {
             return "#" + this._toHex(R) + this._toHex(G) + this._toHex(B);
        },
                
        _toHex: function (n) {
            var charstr = "0123456789ABCDEF";
            n = parseInt(n, 10);
            if (isNaN(n)) return "00";
            n = Math.max(0, Math.min(n, 255));
            return charstr.charAt((n - n % 16) / 16)
                + charstr.charAt(n % 16);
        },
                
        _hexToR: function(h) {
             return parseInt((this._cropHex(h)).substring(0, 2), 16);
        },
               
        _hexToG: function(h) {
             return parseInt((this._cropHex(h)).substring(2, 4), 16);
        },
       
        _hexToB: function(h) {
             return parseInt((this._cropHex(h)).substring(4, 6), 16);
        },
      
        _cropHex: function(h) {
             return (h.charAt(0) == "#") ? h.substring(1, 7) : h;
        },
                
        _getRangeColorValues: function(range, start, end) {
            var rangeColorValues = [];
            rangeColorValues.push(start);
            if (start > end) {
                var rangeValue = (start - end) / (range-1);
                for (var i = range; i > 2; i--) {
                    start = start - rangeValue;
                    rangeColorValues.push(start);
                }
            }
            else {
                var rangeValue = (end - start) / (range-1);
                for (var i = 2; i < range; i++) {
                    start = start + rangeValue;
                    rangeColorValues.push(start + rangeValue);
                }
            }
            rangeColorValues.push(end);
            return rangeColorValues;
        },
       
        _generateGradientCollection: function(gradientColors) {
            var gradientCollection = [];
            var startRangeColor = gradientColors[0];
            var endRangeColor = gradientColors[gradientColors.length - 1];
            var startR = this._hexToR(startRangeColor);
            var startG = this._hexToG(startRangeColor);
            var startB = this._hexToB(startRangeColor);
            var endR = this._hexToR(endRangeColor);
            var endG = this._hexToG(endRangeColor);
            var endB = this._hexToB(endRangeColor);
            var range = 10;
            var rRange = this._getRangeColorValues(range, startR, endR);
            var gRange = this._getRangeColorValues(range, startG, endG);
            var bRange = this._getRangeColorValues(range, startB, endB);
            for (var i = 0; i < range; i++) {
                gradientCollection.push(this._rgbToHex(rRange[i], gRange[i], bRange[i]));
            }
            return gradientCollection;
        },
		
        _calculateTextWidth: function (text) {
            var span = $('<span>' + text + '</span>');
            $('body').append(span);
            var width = span.width();
            span.remove();    
            return width;
        },
        
        _trimFunction: function (str, width) {
      
            var span = $("#spantag").text(str);
            var text = str;
            while (span.width() > width)
            {    
                text = text.slice(0, -2);
                span.text(text + "...");   
            }

            return text;
    
        },
               
        _createLabel:function(content, xpos, ypos,className) {
            var label = $("<div class="+className+"></div>"); // document.createElementNS(this._svgns, "text");
			label[0].innerHTML = content;
			label.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute"
			});
            return label;
        },
                
        _createInteractiveArrow: function (xpos, ypos) {
            
            var interactiveElement = $("<div class='e-icon1 e-interactiveArrow'></div>");            
            interactiveElement[0].innerHTML = "&#9650";
            
            interactiveElement.css({
                "margin-left": xpos + "px",
                "margin-top": ypos + "px",
                "position": "absolute",                
                
            });
            return interactiveElement;
        },
         _getEllipseLegend: function (xPos, yPos) {

            var rect = $("<div class='e-mapLegend'/>");
            rect.css({
                "height": this.legendSettings.iconHeight + "px",
                "width": this.legendSettings.iconWidth + "px",
                "border-radius": legendSettings.iconHeight / 2 + "px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;

        },

        _getRectLegend: function (xPos, yPos) {
            var rect = $("<div />");
            rect.css({
                "height": this.legendSettings.iconHeight + "px",
                "width": this.legendSettings.iconWidth + "px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;
        },
		
		_generateLegend: function (map) {
           
            var isRange = true;
            var colormapping = null;
            if (this.shapeSettings.colorMappings.rangeColorMapping != undefined) {
                colormapping = this.shapeSettings.colorMappings.rangeColorMapping;
            } else if (this.shapeSettings.colorMappings.equalColorMapping != undefined) {
                colormapping = this.shapeSettings.colorMappings.equalColorMapping;
                isRange = false;
            }
                var xPos = 0;
                var yPos = 0;
                var legendSettings = this.legendSettings;
                var columnWidth = 0;
                var width = 0;
                var height = 0;
				var legendPadding = legendSettings.legendPadding;
                var iconwidth = legendSettings.iconWidth + 5;
                var totalHeight = legendSettings.iconHeight + legendPadding;
                var totalWidth = 0;
                var columnCount = legendSettings.columnCount;
                for (var i = 0; i < colormapping.length; i++) {
                    var label = colormapping[i].legendLabel = colormapping[i].legendLabel != null ? colormapping[i].legendLabel : (isRange ? colormapping[i].from : colormapping[i].value);
                    var labelwidth = this._calcWidth(label);
                    
                    var legendWidth = legendSettings.iconWidth + legendPadding-5 + labelwidth;


                    if (columnCount != 0) {
                        if (i % columnCount != 0) {

                            this._drawLegend(colormapping[i], xPos, yPos,map);
                            xPos += (legendWidth + 5);

                        }
                        else {
                            if (i != 0)
                                yPos += (legendSettings.iconHeight + legendPadding);
                            xPos = 0;
                            this._drawLegend(colormapping[i], xPos, yPos,map);
                            xPos += (legendWidth + 5);
                        }
                    }
                    else {
                        if (legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Top || legendSettings.dockPosition == ej.datavisualization.Map.DockPosition.Bottom) {

                            if (map._legendSize.width < xPos + legendWidth) {

                                yPos += (legendSettings.iconHeight + legendPadding);
                                xPos = 0;
                                this._drawLegend(colormapping[i], xPos, yPos,map);
                                xPos += legendWidth;
                            }
                            else {
                                this._drawLegend(colormapping[i], xPos, yPos,map);
                                xPos += legendWidth;
                            }
                        }
                        else {

                            if (map._legendSize.height < yPos + legendSettings.iconHeight) {

                                yPos = 0;
                                xPos += (columnWidth + (legendPadding-5));
                                this._drawLegend(colormapping[i], xPos, yPos,map);
                                columnWidth = 0;
                                yPos += (legendSettings.iconHeight + legendPadding);
                            }
                            else {
                                this._drawLegend(colormapping[i], xPos, yPos,map);
                                columnWidth = Math.max(columnWidth, legendWidth);
                                yPos += (legendSettings.iconHeight + legendPadding);
                            }
                        }

                    }
                }


                
            
        },
        _drawLegend: function (colormapping, xPos, yPos,map) {
            var legendItem;
            if (this.legendSettings.icon == ej.datavisualization.Map.LegendIcons.Circle) {
                legendItem = this._getEllipseLegend(xPos, yPos);
            }
            else {
                legendItem = this._getRectLegend( xPos, yPos);
            }
            xPos += (this.legendSettings.iconWidth + 5);
            $(legendItem).css("background-color", colormapping.color);
            legendItem.appendTo(map._legendDiv);
            var legendText = $("<div class='e-defaultLegendLabel'/>");
            legendText.css({
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            
            legendText[0].innerHTML = colormapping.legendLabel;
            legendText.appendTo(map._legendDiv);
        },



       _generateLegends: function (map) {
            var colorMappings = this.shapeSettings.colorMappings;
            if( (colorMappings.rangeColorMapping != null) || (colorMappings.equalColorMapping != null)) {
                var xPos = 0;
                var yPos = 0;
                var legendSettings = this.legendSettings;
                var columnWidth = 0;
                var width = 0;
                var height = 0;
                var iconWidth = this.legendSettings.iconWidth + 5;
                var iconHeight = this.legendSettings.iconHeight + 5;
                var legendHeight = this.legendSettings.iconHeight + this.legendSettings.legendPadding;
                var totalHeight = iconHeight + this.legendSettings.legendPadding;
                var totalWidth = 0;
                if ((this.legendSettings.showLegend == undefined || this.legendSettings.showLegend) && !this.shapeSettings.autoFill) {
                    $(map._mapContainer).append($('<div  id="labelTooltip" style="display:none;background-color:grey;padding-left:5px; padding-right:5px;position:absolute;z-index:1000;pointer-events:none;"/>'));
                    var baseLayer = this;
                    var xpos = 10;
                    var ypos = 0;
                
                    var totalHeight = this.iconHeigh + this.legendSettings.legendPadding;
                    var _legendheight = this.legendSettings.height;
                    var _legendwidth = this.legendSettings.width;
                    var legenddiv = $("<div/>");
                    if (baseLayer.legendSettings.dockOnMap) {
                        legenddiv = $(map._legendDiv);
                    }
                    else if (!baseLayer.legendSettings.dockOnMap && (baseLayer.legendSettings.mode != ej.datavisualization.Map.LegendMode.Interactive)) {
                        legenddiv = $(map._legendDiv);
                    }
                    else {
                        if (_isSVG) {
                            legenddiv.appendTo(map._templateDiv);
                        }
                        else {
                            map._templateDiv.append(legenddiv);
                        }
                    }


                    if ((this.legendSettings.type == undefined || this.legendSettings.type == ej.datavisualization.Map.LegendType.Layers) && this.shapeSettings.colorMappings != null
                       && (this.legendSettings.mode == undefined || this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default || (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive
                       && this.shapeSettings.colorMappings.equalColorMapping != null))) {

                        this._generateLegend(map);


                    } else if ((this.legendSettings.type == undefined || this.legendSettings.type == ej.datavisualization.Map.LegendType.Layers) && this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive && this.shapeSettings.colorMappings != null) {
                        var textcon = '';

                        if (this.legendSettings.height == 0)
                            _legendheight = 18;
                        if (this.legendSettings.width == 0)
                            _legendwidth = 150;

                        if (this.legendSettings.leftLabel == null)
                            this.legendSettings.leftLabel = '';
                        if (this.legendSettings.title != null) {
                            var newxpos = xpos;
                            if (!this.legendSettings.showLabels)
                                newxpos = xpos + this.legendSettings.leftLabel.length * 10;

                            var _legendtitlewidth = this.legendSettings.title.length * 10;
                            var text = titleText = this.legendSettings.title;;

                            if (_legendtitlewidth > _legendwidth) {
                                for (var i = 1; i < titleText.toString().length; i++) {
                                    text = titleText.toString().substring(0, i - 1) + '...';
                                }
                            }
                            var textcon = this._createLabel(text, newxpos, ypos, 'e-interactivelegend-title');
                            textcon[0].title = titleText;
                            textcon.css({
                                "width": _legendwidth + "px"
                            });
                            if (_isSVG)
                                textcon.appendTo(legenddiv);
                            else
                                legenddiv.append(textcon);
                            ypos += 25;
                        }

                        if (this.legendSettings.showLabels)
                            ypos += 25;

                        if (this.legendSettings.leftLabel != null && !this.legendSettings.showLabels) {
                            var textcon = this._createLabel(this.legendSettings.leftLabel, xpos, ypos - 3, 'e-interactivelegend-leftlabel');

                            if (_isSVG)
                                textcon.appendTo(legenddiv);
                            else
                                legenddiv.append(textcon);
                            xpos = xpos + this.legendSettings.leftLabel.length * 10;
                        }

                        var interactiveElement = this._createInteractiveArrow(xpos, ypos + _legendheight);
                        interactiveElement.appendTo(legenddiv);
                        this._interactiveArrow = interactiveElement;

                        var _legendgroup = null;
                        if (!_isSVG && this.shapeSettings.enableGradient) {
                            _legendgroup = map._createGroup(false, "legendGroup");
                            _legendgroup.style.left = 0 + 'px';
                            _legendgroup.style.top = 0 + 'px';
                            _legendgroup.style.position = "relative";
                            legenddiv.append(_legendgroup);
                        }
						
                        var mappings= colorMappings.rangeColorMapping;
                        for (var key = 0; key < mappings.length; key++) {
                            var colorMapping = mappings[key];
                            if (!colorMapping.hideLegend) {
                                var gradientCollection = [];
                                if (this.shapeSettings.enableGradient) {
                                    gradientCollection = this._generateGradientCollection(colorMapping.gradientColors);
                                }

                                var obj = {};
                                if (this.shapeSettings.enableGradient) {
                                    if (_isSVG) {
                                        var canvas = $("<canvas/>");
                                        var ctx = canvas[0].getContext("2d");
                                        var grd = ctx.createLinearGradient(0, 0, 300, 0);

                                        var temp = 0;
                                        for (var i = 0; i < 10; i++) {
                                            temp = temp + (1 / 10);
                                            if (i == 0) {
                                                grd.addColorStop(0, colorMapping.gradientColors[0]);
                                                grd.addColorStop(temp, gradientCollection[i]);
                                            } else if (i == gradientCollection.length - 1) {
                                                grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                                grd.addColorStop(temp, colorMapping.gradientColors[1]);
                                            } else {
                                                grd.addColorStop(temp - (1 / 10), gradientCollection[i]);
                                                grd.addColorStop(temp, gradientCollection[i + 1]);
                                            }
                                        }
                                        ctx.fillStyle = grd;
                                        ctx.fillRect(0, 0, 300, 300);
                                        canvas.css({
                                            "height": _legendheight + "px",
                                            "width": (_legendwidth / mappings.length) + "px",
                                            "margin-left": xpos + "px",
                                            "margin-top": ypos + "px",
                                            "opacity": "0.9",
                                            "filter": "alpha(opacity=90)", /* For IE8 and earlier */
                                            "position": "absolute"
                                        });
                                        canvas.appendTo(legenddiv);
                                    } else {
                                        var legendID = "legend" + key;
                                        var legendHtmlString = '<v:rect id=' + legendID + ' display="block" style="position:absolute;top: ' + (ypos - 2) + 'px;left:' + xpos + 'px;width:' + (_legendwidth / mappings.length) + 'px;height:' + _legendheight + 'px;"><v:fill opacity="0.9px" type="gradient" method="linear sigma" angle="270"/><v:stroke opacity="0px"/></v:rect>';
                                        _legendgroup.innerHTML = _legendgroup.innerHTML + legendHtmlString;
                                        var legendelement = document.getElementById(legendID);
                                        legendelement.fillcolor = colorMapping.gradientColors[0];
                                        legendelement.fill.color2 = colorMapping.gradientColors[1];
                                    }
                                }
                                else {
                                    var rect = $("<div/>");
                                    rect.css({
                                        "height": _legendheight + "px",
                                        "width": _legendwidth / this.shapeSettings.colorMappings.rangeColorMapping.length + "px",
                                        "background-color": colorMapping.color,
                                        "margin-left": xpos + "px",
                                        "margin-top": ypos + "px",
                                        "opacity": "0.9",
                                        "filter": "alpha(opacity=90)",/* For IE8 and earlier */
                                        "position": "absolute"
                                    });

                                    if (_isSVG)
                                        rect.appendTo(legenddiv);
                                    else
                                        legenddiv.append(rect);
                                }

                                for (var i = 0; i < 10; i++) {
                                    obj = {};
                                    obj["marginLeft"] = xpos;
                                    this._legendRects.push(obj);
                                    xpos = xpos + (_legendwidth / mappings.length) / 10;
                                }

                                if (this.legendSettings.showLabels) {
                                    var labelxpos = xpos - (_legendwidth / mappings.length);
                                    var labelypos = ypos - 25;
                                    var startlabel = this._createLabel((colorMapping.from), labelxpos, labelypos, 'e-legend-rangestartlabel');
                                    labelxpos = xpos;
                                    var endlabel = this._createLabel((colorMapping.to), labelxpos, labelypos);
                                    if (colorMapping.legendLabel != undefined)
                                        endlabel = this._createLabel((colorMapping.legendLabel), labelxpos - (colorMapping.legendLabel.length * 10) / 2, labelypos, 'e-legend-rangeendlabel');
                                    if (_isSVG) {
                                        if (colorMapping == mappings[0])
                                            startlabel.appendTo(legenddiv);
                                        endlabel.appendTo(legenddiv);
                                    }
                                    else {
                                        if (colorMapping == mappings[0])
                                            legenddiv.append(startlabel);
                                        legenddiv.append(endlabel);
                                    }
                                }
                            }
                        }

                        if (this.legendSettings.rightLabel != null && !this.legendSettings.showLabels) {
                            var textcon = this._createLabel(this.legendSettings.rightLabel, xpos + 10, ypos - 3, 'e-interactivelegend-rightlabel');
                            if (_isSVG)
                                textcon.appendTo(legenddiv);
                            else
                                legenddiv.append(textcon);
                            xpos = xpos + this.legendSettings.rightLabel.length * 10;
                        }

                        totalwidth = xpos + 10;
                        totalheight = ypos + _legendheight + this._interactiveArrow.height() + 10;
						map._legendSize = { width: totalwidth, height: totalheight };
						if (baseLayer.legendSettings.dockOnMap) {
						    if (this.legendSettings.dockPosition == 'left')
						        map._marginleft = totalwidth;
						    else if (this.legendSettings.dockPosition == 'top')
						        map._margintop = totalheight;
						    
						}
                    }

                    if (!baseLayer.legendSettings.dockOnMap && this.legendSettings.position == 'none') {
                        var posx = (map._width * this.legendSettings.positionX) / 100;
                        var posy = (map._height * this.legendSettings.positionY) / 100;
                        legenddiv.css({ "margin-left": posx + "px", "margin-top": posy + "px" });
                        
                    }
                    else if (!baseLayer.legendSettings.dockOnMap && (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive)) {
                        var position = (this.legendSettings.position == undefined) ? "topleft" : this.legendSettings.position;
                        var controlSize = { width: totalwidth, height: totalheight };
                        var navPos = map._getPosition(position, controlSize);
                        legenddiv.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y });

                    }
                }
                    

                   
                
            }
        },

        _calcWidth: function (text) {
            var span = $('<span  class="e-defaultLegendLabel">' + text + '</span>');
            $('body').append(span);
            var width = span.width()+5;
            span.remove();
            return width;
        },
       

		_generateBubbleLegends: function (map) {
		    if (this.legendSettings.showLegend == undefined || this.legendSettings.showLegend) {
			    var xpos = 10;
                var ypos = 10;
                var _bubblelegendheight = this.legendSettings.height;
                var _bubblelegendwidth = this.legendSettings.width;
                var bubblelegenddiv = $("<div/>");
                if (_isSVG) {
                    bubblelegenddiv.appendTo(map._templateDiv);
                }
                else {
                    map._templateDiv.append(bubblelegenddiv);
                }
				var totalwidth = 0;
                var totalheight = 0;
                var isRange = true;
                var mappings = null;
                if (this.bubbleSettings.colorMappings.rangeColorMapping != undefined) {
                    mappings = this.bubbleSettings.colorMappings.rangeColorMapping;
                } else if (this.bubbleSettings.colorMappings.equalColorMapping != undefined) {
                    mappings = this.bubbleSettings.colorMappings.equalColorMapping;
                    isRange = false;
                }
                if ((this.legendSettings.type == ej.datavisualization.Map.LegendType.Bubbles) && this.bubbleSettings.colorMappings != null
                   && (this.legendSettings.mode == undefined || this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Default || (this.legendSettings.mode == ej.datavisualization.Map.LegendMode.Interactive
                   && this.bubbleSettings.colorMappings.equalColorMapping != null))) {
                    
				    if (this.legendSettings.height == undefined)
                        _bubblelegendheight = 20;
				    if (this.legendSettings.width == undefined)
                        _bubblelegendwidth = 20;

                    for (var key = 0; key < mappings.length; key++) {
                        var colorMapping = mappings[key];
                        if (!colorMapping.hideLegend) {
                            var rect = $("<div class='e-mapBubbleLegend'/>");

                            if (this.legendSettings.icon == ej.datavisualization.Map.LegendIcons.Circle) {
                                rect.css({
                                    "height": _bubblelegendheight + "px",
                                    "width": _bubblelegendwidth + "px",
                                    "border-radius": this.legendSettings.iconHeight / 2 + "px",
                                    "background-color": colorMapping.color,
                                    "margin-left": xpos + "px",
                                    "margin-top": ypos + "px",
                                    "position": "absolute"
                                });
                            }
                            else {
                                rect.css({
                                    "height": _bubblelegendheight + "px",
                                    "width": _bubblelegendwidth + "px",
                                    "background-color": colorMapping.color,
                                    "margin-left": xpos + "px",
                                    "margin-top": ypos + "px",
                                    "position": "absolute"
                                });
                            }
							var textcon = this._createLabel(colorMapping.legendLabel, xpos + _bubblelegendwidth + 5, ypos, 'e-legendlabeltext');
							if (colorMapping.legendLabel != null) {
                                textcon[0].innerText= colorMapping.legendLabel;

                            } else {
                                colorMapping.legendLabel = textcon[0].innerText = isRange ? colorMapping.from : colorMapping.value;
                            }
							if (_isSVG) {
                                rect.appendTo(bubblelegenddiv);
                                textcon.appendTo(bubblelegenddiv);
                             }
                             else {
                                 bubblelegenddiv.append(rect);
                                 bubblelegenddiv.append(textcon);
							}
							if (totalwidth < (textcon[0].innerText.length * 10) + _bubblelegendwidth)
							    totalwidth = (textcon[0].innerText.length * 10) + _bubblelegendwidth;
                            var textHeight = textcon.height();
                            var rectHeight = rect.height();
                            if (textHeight > rectHeight) {
                                ypos += (textHeight + 5);
                            } else {
                                ypos += (rectHeight + 5);
                            }
						}
                    }
                    totalheight = ypos;
                }                
				if (this.legendSettings.position == 'none') {
				    var posx = (map._width * this.legendSettings.positionX) / 100;
				    var posy = (map._height * this.legendSettings.positionY) / 100;
                    bubblelegenddiv.css({ "margin-left": posx + "px", "margin-top": posy + "px" });
                } else {
				    var position = (this.legendSettings.position == undefined) ? "topleft" : this.legendSettings.position;
                    var controlSize = { width: totalwidth, height: totalheight };
                    var navPos = map._getPosition(position, controlSize);
                    bubblelegenddiv.css({ "margin-left": navPos.x + "px", "margin-top": navPos.y + "px" });
                }
            }
				
		},
		
        _animateBubble: function (element, delayInterval) {
            var radius = element.getAttribute("r");
            var scaleVal;
            var $ele = $(element);
            var layer = this;
            $ele.delay(delayInterval).each(function () { }).animate(
                {
                    r: radius
                },
                {
                    duration: 700,

                    step: function (now) {
                        scaleVal = now;
                        if (_isSVG) {
                            $ele.attr("style", "display:block;pointer-events:stroke;");
                            $ele[0].setAttribute("r", scaleVal);
                        }
                        else {
                        }
                    },
                    complete: function () {
                        layer._bubbleCount++;
                        if (layer._bubbleCount == layer._bubbles.length) {
                            layer._setMapElements();
                        }
                    }
                }

            );
        },
        
        _setMapElements: function () {
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];                
                $(item).css({ "display": "block" });
            }
        },

        _setMapItemsPositionWithAnimation: function (map) {
            this._bubbleCollection = [];
            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var bubblerad = this._bubbles[key].getAttribute('r');
                var position = this._bounds[key];               
                var xpos = ((position.x + map._translatePoint.x) * map._scale);
                var ypos = ((position.y + map._translatePoint.y) * map._scale);

                var finalPosition = map.validateBubblePosition(position.points, { x: xpos, y: ypos }, bubblerad);

                var ubound = 20;
                var lbound = 0;
                var randomValue = Math.floor(Math.random() * (ubound - lbound) + lbound);
                var delayInterval = parseInt(randomValue * 50);
                if (_isSVG) {
                    $(bubble).attr({ "cx": finalPosition.x, "cy": finalPosition.y });
                }
                this._animateBubble(bubble, delayInterval);
                this._bubbleCollection.push(bubble);
            }
            for (var key = 0; key < this._mapMarkers.length; key++) {
                var item = this._mapMarkers[key];
                var marker;
                if (this.markers.length > 0) {
                    marker = this.markers[key];
                }                
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(marker.latitude, marker.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforMarker(marker.latitude, marker.longitude);
                }

                if (_isSVG) {
                    var xpos = position.x;
                }
                else {
                    var xpos = ((position.x + map._transformX) * map._scale);
                }
                var ypos = position.y;
                $(item).css({ "display":"block","left": xpos, "top": ypos - 100 });
                $(item).delay(500).each(function () { }).animate({ "top": ypos }, 500);
               
            }
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                var box = item[0].getBoundingClientRect();               
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (box.width / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (box.height / 4);
                if(this._bubbles.length > 0)
				    $(item).css({ "left": xpos, "top": ypos, "display": "none" });
				else
                   $(item).css({ "left": xpos, "top": ypos, "display": "block" });
               
            }
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth =_isSVG?box.width: box.right - box.left;
                var boxheight=_isSVG?box.height: box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "smartLabelStyle") {                    
                    item[0].className = "labelStyle"; 
                     $(item[0]).css({ "pointer-events": "none", "position": "absolute" });                   
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos });

            }
            if (this.labelSettings!=null && this.labelSettings.showLabels) {
                this._validateSmartLabel(map);
            }
        },

        _resizeShapes: function (map) {
            var thickness = this.shapeSettings.strokeThickness / map._scale;
            if (this._mapShapes != undefined) {
            for (var i = 0; i < this._mapShapes.length; i++) {
                var element = this._mapShapes[i].shape;
                if (_isSVG) {
                    if (element.localName == 'circle') {
                        element.setAttribute("r", 5 / map._scale);
                    } else {
                        if (this._contains(this._prevSelectedShapes,element)) {
                            element.setAttribute("stroke-width", this.shapeSettings.selectionStrokeWidth / map._scale);
                        } else {
                            element.setAttribute("stroke-width", thickness);
                        }
                    }
                } else {
                    if (element.nodeName == 'oval') {

                    } else {
                        if (this._contains(this._prevSelectedShapes,element)) {
                            element.strokeweight = this.shapeSettings.selectionStrokeWidth / map._scale;
                        } else {
                            element.strokeweight = thickness;
                        }
                    }
                }
            }
          }
        },
       
       _setMapItemsPosition: function(map) {
            this._bubbleCollection = [];
			if(this._mapItems!=undefined)
			{
            for (var key = 0; key < this._mapItems.length; key++) {
                var item = this._mapItems[key];
                var position = this._bounds[key];
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 4);
                $(item).css({ "left": xpos, "top": ypos });               
            }
			}
			if(this._bubbles!=undefined)
			{
            for (var key = 0; key < this._bubbles.length; key++) {
                var bubble = this._bubbles[key];
                var bubblerad = this._bubbles[key].getAttribute('r');
                var position = this._bounds[key];
                if (_isSVG) {
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);

                    var finalPosition = map.validateBubblePosition(position.points, { x: xpos, y: ypos }, bubblerad);

                    $(bubble).attr({ "cx": finalPosition.x, "cy": finalPosition.y });
                }
                else {
                    bubble = document.getElementById(bubble.id);
                    var xpos = ((position.x + map._translatePoint.x) * map._scale);
                    var ypos = ((position.y + map._translatePoint.y) * map._scale);

                    var finalPosition = map.validateBubblePosition(position.points, { x: xpos, y: ypos }, bubblerad);

                    var bubbleTop = finalPosition.y - (bubble.getBoundingClientRect().bottom - bubble.getBoundingClientRect().top) / 2;
                    var bubbleLeft = finalPosition.x - (bubble.getBoundingClientRect().right - bubble.getBoundingClientRect().left) / 2;
                    
                    $(bubble).css({
                        "left": bubbleLeft,
                        "top": bubbleTop
                    });
                }
                this._bubbleCollection.push(bubble);
            }
			}
            if(this._mapMarkers!=undefined){
            for (var key = 0; key < this._mapMarkers.length; key++) {
                var item = this._mapMarkers[key];

                var marker;
                if (this.markers.length > 0) {
                    marker = this.markers[key];
                }               
                var position;
                if (map._isTileMap) {
                    position = map._convertTileLatLongtoPoint(marker.latitude, marker.longitude);
                }
                else {
                    position = map._convertLatLongtoPointforMarker(marker.latitude, marker.longitude);
                }            
                var xpos = position.x;               
                var ypos = position.y;            
                $(item).css({ "left": xpos, "top": ypos });                
            }
			}
			if(this._labelCollection!=undefined){
            for (var key = 0; key < this._labelCollection.length; key++) {
                var item = this._labelCollection[key];
                var position = this._labelBounds[key];
                $(item).css("display", "block");
                var box = item[0].getBoundingClientRect();
                var boxwidth = _isSVG ? box.width : box.right - box.left;
                var boxheight = _isSVG ? box.height : box.bottom - box.top;
                var xpos = ((position.x + map._translatePoint.x) * map._scale) - (boxwidth / 2);
                var ypos = ((position.y + map._translatePoint.y) * map._scale) - (boxheight / 2);
                if (item[0].className = "smartLabelStyle") {                    
                    item[0].className = "labelStyle";
                    $(item[0]).css("background-color", "transparent");
                    $(item[0]).css({ "pointer-events": "none", "position": "absolute" });
                }
                item[0].innerHTML = this._labelData[key];
                $(item).css({ "left": xpos, "top": ypos});
               
            }
			}
            if (this.labelSettings != null && this.labelSettings.showLabels) {
                this._validateSmartLabel(map);
            }
        },
       
        _validateSmartLabel: function (map) {
            this._smartLabels = [];
            var filledRects = [];
            if (this._labelCollection.length > 0) {
                for (var index = 0; index < this._mapShapes.length; index++) {
                    var shapemidPoint = this._labelBounds[index];
                    var midPoint = { x: (shapemidPoint.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.y + map._translatePoint.y) * map._scale };

                    var rightMinPoint = { x: (shapemidPoint.rightMin.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.rightMin.y + map._translatePoint.y) * map._scale };
                    var rightMaxPoint = { x: (shapemidPoint.rightMax.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.rightMax.y + map._translatePoint.y) * map._scale };
                    var leftMinPoint = { x: (shapemidPoint.leftMin.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.leftMin.y + map._translatePoint.y) * map._scale };
                    var leftMaxPoint = { x: (shapemidPoint.leftMax.x + map._translatePoint.x) * map._scale, y: (shapemidPoint.leftMax.y + map._translatePoint.y) * map._scale };

                    var labelElement = this._labelCollection[index];
                    if (midPoint.x > 0 && midPoint.x < map._width && midPoint.y > 0 && midPoint.y < map._height) {
                        var shape = this._mapShapes[index].shape;
                        var bounds = shape.getBoundingClientRect();                        
                        var flag = false;
                        var labelSize = labelElement[0].getBoundingClientRect();
                        var labelHeight = labelSize.height;
                        var labelWidth = labelSize.width;
                        if (!_isSVG) {
                            bounds = { width: (bounds.right*map._scale) - (bounds.left*map._scale), height: (bounds.bottom *map._scale)- (bounds.top*map._scale) };
                            labelHeight = labelSize.bottom - labelSize.top;
                            labelWidth = labelSize.right - labelSize.left;
                        }

                        var rightIntersect = false, leftIntersect = false;
                        if ((labelWidth / 2 > rightMinPoint.x - midPoint.x || labelWidth / 2 > rightMaxPoint.x - midPoint.x)
                            && (labelHeight / 2 > midPoint.y - rightMinPoint.y || labelHeight / 2 > rightMaxPoint.y - midPoint.y)) {
                            rightIntersect = true;
                        }
                        else if ((labelWidth / 2 > midPoint.x - leftMinPoint.x || labelWidth / 2 > midPoint.x - leftMaxPoint.x)
                            && (labelHeight / 2 > midPoint.y - leftMinPoint.y || labelHeight / 2 > leftMaxPoint.y - midPoint.y)) {
                            leftIntersect = true;
                        }
                        
                        var leftPosition = 0;
                        var topPosition = 0;
                        var factor = 20;
                        var xArray = [];
                        var yArray = [];
                        if (labelHeight > bounds.height || labelWidth > bounds.width || (rightIntersect || leftIntersect)) {
                            
                            if (this.labelSettings.enableSmartLabel) {
                                if (this.labelSettings.smartLabelSize == ej.datavisualization.Map.LabelSize.Fixed) {
                                    labelHeight = 25;
                                    labelWidth = 15 * this.labelSettings.labelLength;
                                }
                                else {                                    
                                    labelWidth *= 1.3;
                                    labelHeight = 25;
                                }
                                while (!flag) {
                                    if (factor > 400) {
                                        flag = true;
                                    }
                                    xArray = pushToArray(factor);
                                    yArray = xArray;
                                    for (var xIndex = 0; xIndex < xArray.length; xIndex++) {
                                        for (var yIndex = 0; yIndex < xArray.length; yIndex++) {
                                            leftPosition = yArray[yIndex];
                                            topPosition = xArray[xIndex];                                           
                                            if (midPoint.x + leftPosition + labelWidth > map._width && midPoint.x < map._width) {
                                                leftPosition -=(midPoint.x + leftPosition + labelWidth)- map._width;
                                            }
                                            if (midPoint.x + leftPosition <0) {
                                                leftPosition =0;
                                            }
                                            if (midPoint.y + topPosition  <0) {
                                                topPosition=0;
                                            }
                                            if (midPoint.y + topPosition + labelHeight > map._height && midPoint.y < map._height) {
                                                topPosition -= (midPoint.y + topPosition + labelHeight) - map._height;
                                            }
                                            var rect = { left: midPoint.x + leftPosition, top: midPoint.y + topPosition, height: labelHeight, width: labelWidth };
                                            if (isEmpty(rect)) {
                                                var ele = map._getIntersectedElements(rect, this._mapShapes);
                                                if (ele != null && ele.length == 0) {
                                                    flag = true;                                                   
                                                    xIndex = xArray.length;
                                                    yIndex = xArray.length;
                                                    labelElement[0].className = 'smartLabelStyle';
                                                    $(labelElement).css({ "pointer-events": "stroke", "position": "absolute" });

                                                    if (this.labelSettings.smartLabelSize == ej.datavisualization.Map.LabelSize.Fixed) {
                                                        labelElement[0].innerHTML = labelElement[0].innerHTML.substring(0, this.labelSettings.labelLength);
                                                    }
                                                    labelElement.mouseenter({ Param1: this, Param2: this._mapShapes[index],map:map }, map._polyEnterFunction);
                                                    labelElement.mousemove({ Param1: this, Param2: this._mapShapes[index] }, map._polyMoveFunction);
                                                    $(labelElement).css({ left: midPoint.x + leftPosition, top: midPoint.y + topPosition, "background-color": _isSVG ? shape.getAttribute("fill") : shape.fillcolor.value});
                                                    filledRects.push(rect);
                                                    var labelObj = {};
                                                    labelObj["shape"] = shape;
                                                    labelObj["legend"] = labelElement;
                                                    this._smartLabels.push(labelObj);
                                                }
                                            }
                                        }
                                    }
                                    factor += 10;
                                }
                            } else {
                                var canExecute = true;
                                var minY = 0;
                                var maxY = 0;
                                var flag = true;

                                // Finding Minimum and Maximum Y position point calculation
                                for (var j = 0; j < shapemidPoint.points.length; j++) {
                                    var point = { x: (shapemidPoint.points[j].x + map._translatePoint.x) * map._scale, y: (shapemidPoint.points[j].y + map._translatePoint.y) * map._scale };
                                    if (flag) {
                                        minY = point.y;
                                        maxY = point.y;
                                        flag = false;
                                    }
                                    else {
                                        minY = Math.min(minY, point.y);
                                        maxY = Math.max(maxY, point.y);
                                    }
                                }

                                var heightfactor = Math.floor((maxY - minY) / labelHeight);
                                var shapePoints = [];

                                // Grouping points based on label height
                                for (var k = 0; k < heightfactor; k++) {
                                    var groupPoints = [];
                                    shapePoints.push(groupPoints);
                                }

                                for (var j = 0; j < shapemidPoint.points.length; j++) {
                                    var point = { x: (shapemidPoint.points[j].x + map._translatePoint.x) * map._scale, y: (shapemidPoint.points[j].y + map._translatePoint.y) * map._scale };
                                    var positionfactor = Math.floor((point.y - minY) / labelHeight);
                                    if (positionfactor > 0)
                                        positionfactor -= 1;
                                    var groupPoints = shapePoints[positionfactor];
                                    if (groupPoints == undefined)
                                        groupPoints = [];
                                    groupPoints.push({ x: point.x, y: point.y });
                                    shapePoints[positionfactor] = groupPoints;
                                }

                                // Finding minimum and maximum x point calculation 
                                for (var j = 0; j < shapePoints.length; j++) {
                                    var groupPoints = shapePoints[j];
                                    flag = true;
                                    var minX =0, maxX =0, top = 0;
                                    var leftPoints = [], rightPoints = [];
                                    
                                    for (var k = 0; k < groupPoints.length; k++) {
                                        if (flag) {
                                            minX = groupPoints[k].x;
                                            maxX = groupPoints[k].x;
                                            top = groupPoints[k].y;
                                            flag = false;
                                        }
                                        else {
                                            minX = Math.min(minX, groupPoints[k].x);
                                            maxX = Math.max(maxX, groupPoints[k].x);
                                            if (groupPoints[k].x == maxX)
                                                top = groupPoints[k].y;
                                        }
                                        if (groupPoints[k].x < midPoint.x)
                                            leftPoints.push(groupPoints[k].x);
                                        if (groupPoints[k].x > midPoint.x)
                                            rightPoints.push(groupPoints[k].x);
                                    }

                                    if (bounds.left < map._mapContainer[0].getBoundingClientRect().left + 1) {
                                        minX = map._mapContainer[0].getBoundingClientRect().left + 1 - bounds.left;
                                        leftPoints.push(minX);
                                    }

                                    //  Checking intersection of points with label.
                                    var intersect = false;
                                    for (var k = 0; k < leftPoints.length; k++) {
                                        if (maxX - leftPoints[k] < labelWidth + 1) {
                                            intersect = true;
                                            break;
                                        }
                                    }
                                    if (!intersect) {
                                        for (var k = 0; k < rightPoints.length; k++) {
                                            if (rightPoints[k] - minX < labelWidth + 1) {
                                                intersect = true;
                                                break;
                                            }
                                        }
                                    }
                                    if (maxX - minX > labelWidth + 1 && !intersect) {
                                        labelElement.css("left", bounds.left * map._scale + (minX - bounds.left * map._scale) + (maxX - minX) / 2 - labelWidth / 2 + "px");
                                        labelElement.css("top", top + "px");
                                        if (maxY - top < labelHeight)
                                            labelElement.css("top", top - labelHeight + "px");
                                        canExecute = false;
                                        break;
                                    }
                                }
                                if(canExecute)
                                    labelElement.css("display", "none");
                            }
                        }
                    } 
                }
            }

            function pushToArray(value) {
                var array = [];
                array.push(0);
                for (var j = 10; j <= value; j += 20) {
                    array.push(-j);
                    array.push(j);
                }

                return array;
            }

            function isEmpty(rect) {
                for (var i = 0; i < filledRects.length; i++) {
                    if (isIntersect(rect, filledRects[i]))
                        return false;
                }
                return true;
            }
           
            function isIntersect(rect1, rect2) {
                
                if (rect1.left - 2 >= (rect2.left + rect2.width) || rect1.left - 2 + rect1.width <= rect2.left - 2 ||
                    rect1.top - 2 >= rect2.top + rect2.height || rect1.top+rect1.height <= rect2.top-2) {
                    return false;
                }
                return true;
            }


        },
              
        _fillColors: function (value, colorMapping, shape) {

            if (colorMapping.rangeColorMapping != null) {
               
                this._fillRangeColors(value, colorMapping.rangeColorMapping, shape);
            }
            else if (colorMapping.equalColorMapping != null) {
                this._fillEqualColors(value, colorMapping, shape);
            }
        },

        _fillEqualColors: function (value, colorMapping, shape) {
            $.each(colorMapping.equalColorMapping, function (index, gValue) {
                if (gValue.value == value) {
                    if (_isSVG) {
                        shape.setAttribute("fill", gValue.color);
                    }
                    else {
                        shape.fillcolor = gValue.color;
                    }
                }
            });
        },
       
        _updateLegendRange: function (value, shapelayer, shape) {
            var colormapping = shapelayer.shapeSettings.colorMappings.rangeColorMapping;
            for (var index = 0; index < colormapping.length; index++) {
                var gradientCollection = null;
                var mapping = colormapping[index];
                if (shapelayer.shapeSettings.enableGradient) {
                    gradientCollection = this._generateGradientCollection(mapping.gradientColors);
                }
                if (value >= mapping.from && value <= mapping.to) {
                    var minValue = index;
                    if (index != 0)
                        minValue = index * 10;
                    var minRange = mapping.from;
                    var maxRange = mapping.from + ((mapping.to - mapping.from) / 10);
                    for (var i = minValue; i < minValue + 10; i++) {
                        if (value >= minRange && value <= maxRange) {
                            var obj = {};
                            var shapeOpacity = this._getColorRatio(0.7, 1, value, mapping.from, mapping.to);
                            if (shapelayer._legendRects[i] != undefined)
                                obj = shapelayer._legendRects[i];
                            if (shapelayer.shapeSettings.enableGradient) {
                                if (index != 0) {
                                    obj["color"] = gradientCollection[i - index * 10];
                                } else {
                                    obj["color"] = gradientCollection[i];
                                }
                            } else
                                obj["color"] = mapping.color;
                                $(shape).css({ "opacity": shapeOpacity });
                            return obj;
                        }
                        minRange = maxRange;
                        maxRange = maxRange + ((mapping.to - mapping.from) / 10);
                    }
                }
            }
        },

        _fillRangeColors: function (value, colormapping, shape) {
            
            for (var index = 0; index < colormapping.length; index++) {
                var mapping = colormapping[index];
                if (value >= mapping.from && value <= mapping.to) {
                    var shapeOpacity = this._getColorRatio(0.7, 1, value, mapping.from, mapping.to);
                    if (_isSVG) {
                        shape.setAttribute("fill", mapping.color);
                    }
                    else {
                        shape.fillcolor = mapping.color;
                    }
                    $(shape).css({ "opacity": shapeOpacity });
                }
            }
        },
       
        _getColorRatio: function (min, max, value, minValue, maxValue) {
            var percent = (100 / (maxValue - minValue)) * (value - minValue);
            var colorCode = (((parseFloat(max) - parseFloat(min)) / 100) * percent) + parseFloat(min);
            return colorCode;
        }

    };

    $.fn.pinchZoom = function (rootgroup, map) {

        var stateOrigin,
                    stateTf,
                    root = this[0],
                    startTouches = [],

           getTouchEventPoint = function (evt) {

               var p = root.createSVGPoint(),
                   targetTouches = evt.targetTouches,
                   offsetX,
                   offsetY;

               if (targetTouches.length) {
                   var touch = targetTouches[0];
                   offsetX = touch.pageX;
                   offsetY = touch.pageY;
               }

               p.x = offsetX;
               p.y = offsetY;

               return p;
           };

        handleTouchStart = function (evt) {
            var g = rootgroup;
            stateTf = g.getCTM().inverse();
            stateOrigin = getTouchEventPoint(evt).matrixTransform(stateTf);
            var touches = evt.touches;
            if (touches.length >= 2) {
                for (var i = 0; i < touches.length; i++) {
                    var touch = touches[i]
                      , found = false;
                    for (var j = 0; j < startTouches.length; j++) {
                        var startTouch = startTouches[j];
                        if (touch.identifier === startTouch.identifier) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        var touchCopy = $.extend({}, touch);
                        startTouches.push(touchCopy);
                    }
                }
            }

            evt.preventDefault();
        };

        getDistance = function (touch1, touch2) {
            var x = touch2.pageX - touch1.pageX,
                y = touch2.pageY - touch1.pageY;
            return Math.sqrt((x * x) + (y * y));
        };

        setCTM = function (element, matrix) {
		    var isZoomIn=null;
            var s = "matrix(" + matrix.a + "," + matrix.b + "," + matrix.c + "," + matrix.d + "," + matrix.e + "," + matrix.f + ")";
            if (matrix.a > map._baseScale && map.enableZoom() && map.enablePan()) {
			if(matrix.a > map._scale)
			{
			     isZoomIn=true;
			}
			else if(matrix.a < map._scale)
			{
			isZoomIn=false;
			}
			    map._scale = matrix.a;
			    map._translatePoint.x = matrix.e / matrix.a;
			    map._translatePoint.y = matrix.f / matrix.a;
			    map._applyTransform(map._scale, map._translatePoint);
  			    map.zoomLevel(map._scale - map._baseScale + 1);
	    		if (isZoomIn != null && isZoomIn) {
				    map._trigger("zoomedIn", { originalEvent: null, zoomLevel: map.zoomLevel() });
				    map._updateSliderValue(false);
					map._resizeShape();
				}
				else if(isZoomIn!=null)
				{
				    map._trigger("zoomedOut", { originalEvent: null, zoomLevel: map.zoomLevel() });
				    map._updateSliderValue(false);
					map._resizeShape();
				}
	    		map._isDragging = true;
                map._refrshLayers();
            }

        };

        handleTouchEnd = function (evt) {
            var changedTouches = evt.changedTouches;
            for (var i = 0; i < changedTouches.length; i++) {
                var changedTouch = changedTouches[i];
                for (var j = 0; j < startTouches.length; j++) {
                    var startTouch = startTouches[j];
                    if (startTouch.identifier === changedTouch.identifier) {
                        var idx = startTouches.indexOf(startTouch);
                        startTouches.splice(idx, 1);
                    }
                }
            }
            evt.preventDefault();
        };

        updateTouch = function (dstTouch, srcTouch) {
            dstTouch.pageX = srcTouch.pageX;
            dstTouch.pageY = srcTouch.pageY;
        };

        handleTouchMove = function (evt) {
            evt.preventDefault();

            var g = rootgroup,
                touches = evt.touches,
                z,
                p,
                k;

            if (touches.length >= 2) {
                var touch0 = touches[0]
                  , touch1 = touches[1]
                  , startTouch0 = startTouches[0]
                  , startTouch1 = startTouches[1];
                z = getDistance(touch0, touch1) / getDistance(startTouch0, startTouch1);
                p = root.createSVGPoint();
                p.x = (touch0.pageX + touch1.pageX) / 2;
                p.y = (touch0.pageY + touch1.pageY) / 2;
                p = p.matrixTransform(g.getCTM().inverse());
                k = root.createSVGMatrix().translate(p.x, p.y).scale(z).translate(-p.x, -p.y);
                setCTM(g, g.getCTM().multiply(k));
                if (typeof stateTf === "undefined") {
                    stateTf = g.getCTM().inverse();
                }
                stateTf = stateTf.multiply(k.inverse());
                updateTouch(startTouch0, touch0);
                updateTouch(startTouch1, touch1);
            } else if (!startTouches.length) {
                p = getTouchEventPoint(evt).matrixTransform(stateTf);
                setCTM(g, stateTf.inverse().translate(p.x - stateOrigin.x, p.y - stateOrigin.y));
            }
        };
        this[0].addEventListener('touchstart', handleTouchStart, false);
        this[0].addEventListener('touchend', handleTouchEnd, false);
        this[0].addEventListener('touchmove', handleTouchMove, false);


    };

    jQuery.uaMatch = function (ua) {
        ua = ua.toLowerCase();

        var match = /(chrome)[ \/]([\w.]+)/.exec(ua) ||
            /(webkit)[ \/]([\w.]+)/.exec(ua) ||
            /(opera)(?:.*version|)[ \/]([\w.]+)/.exec(ua) ||
            /(msie) ([\w.]+)/.exec(ua) ||
            ua.indexOf("compatible") < 0 && /(mozilla)(?:.*? rv:([\w.]+)|)/.exec(ua) ||
            [];

        return {
            browser: match[1] || "",
            version: match[2] || "0"
        };
    };
    
    ej.datavisualization.Map.LayerType = {
        Geometry: "geometry",
        OSM: "osm",
        Bing: "bing"
    };
   
    ej.datavisualization.Map.LegendIcons = {       
        Rectangle: "rectangle",      
        Circle: "circle"
    };
   
    ej.datavisualization.Map.LabelSize = {       
        Fixed: "fixed",        
        Default: "default"
    };

    ej.datavisualization.Map.LegendMode = {       
        Default: "default",       
        Interactive: "interactive"		
    };

    ej.datavisualization.Map.BingMapType = {    
        Aerial: "aerial",    
        AerialWithLabel: "aerialwithlabel",    
        Road: "road",
    };

    ej.datavisualization.Map.SelectionMode = {       
        Default: "default",       
        Multiple: "multiple"
    };

    ej.datavisualization.Map.ColorPalette = {        
        Palette1: "palette1", 
        Palette2: "palette2", 
        Palette3: "palette3", 
        CustomPalette: "custompalette"
    };
   
    ej.datavisualization.Map.LegendType = {
        Layers: "layers",
        Bubbles: "bubbles"
    }
 
    ej.datavisualization.Map.Position = {
        None: "none",
        TopLeft: "topleft",
        TopCenter: "topcenter",
        TopRight: "topright",
        CenterLeft: "centerleft",
        Center: "center",
        CenterRight: "centerright",
        BottomLeft: "bottomleft",
        BottomCenter: "bottomcenter",
        BottomRight: "bottomright"
    }
    
    ej.datavisualization.Map.DockPosition = {    
        Top: "top",	
        Bottom: "bottom",    
        Right: "right",    
        Left: "left"
    };
	 ej.datavisualization.Map.Alignment = {
        
        Near: "near",       
        Center: "center",       
        Far: "far"
    };
    
    ej.datavisualization.Map.LabelOrientation = {	
        Horizontal: "horizontal",	
        Vertical: "vertical"
    };
    
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Html TreeMap elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author &lt;a href="mailto:licensing@syncfusion.com"&gt;Syncfusion Inc&lt;/a&gt;
*/
(function ($, ej, undefined) {
   
    ej.widget({ "ejTreeMap": "ej.datavisualization.TreeMap" }, {

        validTags: ["div"],

        defaults: {           
			          
            leafItemSettings: {               
                borderThickness: 1,               
                borderBrush: "white",
                showLabels: false,
                labelPath: null,
                itemTemplate: null,
                labelPosition: "topleft",
                labelVisibilityMode: "visible"
            },
            dataSource: null,
            groupColorMapping:[],			
            enableDrillDown: false,
            drillDownHeaderColor: null,
            drillDownSelectionColor: '#000000',          			
            colorValuePath: null,
            weightValuePath: null,			
            treeMapItems: [],
            showLegend: false,
            borderBrush: "white",
            borderThickness: 1,
            enableResize: true,
            itemsLayoutMode: "squarified",						
            levels: [],
                       
            legendSettings: {			
                template:"",				
                iconHeight:15,			
                iconWidth:15,				
                height:0,						
                width:0,			
                dockPosition:"top",				 
                alignment: "near",               
                columnCount :0				
            },            			         
            rangeColorMapping: [],

            uniColorMapping: { 			
                color: null 
            },
			            
            desaturationColorMapping: { 				            
                from:0, 				               
                to:0,				              
                color: "",								
                rangeMinimum:0,				
                rangeMaximum:0
            },

            paletteColorMapping: {                
                colors:[], 
            },            
            highlightOnSelection: false,			           
			selectionMode: "default",		
            highlightGroupOnSelection: false,
			groupSelectionMode: "default",
			draggingOnSelection: false,
			draggingGroupOnSelection: false,
            showTooltip: false,
            tooltipTemplate: null,
            highlightBorderThickness: 5,
            highlightGroupBorderThickness: 4,
            highlightBorderBrush: "gray",
            highlightGroupBorderBrush: "gray"

        },
        
        dataTypes: {
            dataSource: "data",
            treeMapItems: "array",
            levels: "array",
            rangeColorMapping: "array",
            paletteColorMapping: "object",
            groupColorMapping: "array"
        },
		
        observables: ["dataSource",
              "colorValuePath",
              "weightValuePath",
              "showLegend",
              "enableResize",
              "highlightOnSelection",
			  "selectionMode",
			  "groupSelectionMode",
             "highlightGroupOnSelection",
             "enableDrillDown",
              "drillDownHeaderColor",
             "drillDownSelectionColor",
             "showTooltip",
             "highlightBorderThickness",
             "highlightBorderBrush",
               "itemsLayoutMode",
              "leafItemSettings.borderThickness",
              "leafItemSettings.borderBrush",
              "leafItemSettings.showLabels",
              "leafItemSettings.labelPath",
              "legendSettings.iconWidth",
              "legendSettings.iconHeight",
              "legendSettings.dockPosition",
              "legendSettings.height",
              "legendSettings.width",
              "uniColorMapping.color",
              "desaturationColorMapping.from",
              "desaturationColorMapping.to",
              "desaturationColorMapping.color",
              "desaturationColorMapping.rangeMinimum",
              "desaturationColorMapping.rangeMaximum"              
        ],
        _tags: [
            {
                tag: "levels",
                attr: ["groupPath", "groupGap", "headerHeight", "showHeader", "groupPadding", "groupBackground", "groupBorderColor", "groupBorderThickness"],
                singular: "level"
            },
            {
                tag: "rangeColorMapping",
                attr: ["color", "legendLabel", "from", "to"],
                singular: "rangeColor"
            },           
            {
                tag: "groupColorMapping",
                attr: ["groupID",
            {
                tag: "groupColorMapping.rangeColorMapping",
                attr: ["color", "legendLabel", "from", "to"],
                singular: "groupRangeColor"
            },            

              "uniColorMapping.color",
              "desaturationColorMapping.from",
              "desaturationColorMapping.to",
              "desaturationColorMapping.color",
              "desaturationColorMapping.rangeMinimum",
              "desaturationColorMapping.rangeMaximum"
                ],
                singular: "groupColor"
            }

        ],

        dataSource: ej.util.valueFunction("dataSource"),
        colorValuePath: ej.util.valueFunction("colorValuePath"),
        weightValuePath: ej.util.valueFunction("weightValuePath"),
        showLegend: ej.util.valueFunction("showLegend"),
        enableResize: ej.util.valueFunction("enableResize"),
        highlightOnSelection: ej.util.valueFunction("highlightOnSelection"),
		selectionMode:ej.util.valueFunction("selectionMode"),
        highlightGroupOnSelection: ej.util.valueFunction("highlightGroupOnSelection"),
		groupSelectionMode:ej.util.valueFunction("groupSelectionMode"),
        enableDrillDown: ej.util.valueFunction("enableDrillDown"),
        drillDownHeaderColor: ej.util.valueFunction("drillDownHeaderColor"),
        drillDownSelectionColor: ej.util.valueFunction("drillDownSelectionColor"),
        showTooltip: ej.util.valueFunction("showTooltip"),
        highlightBorderThickness: ej.util.valueFunction("highlightBorderThickness"),
        itemsLayoutMode: ej.util.valueFunction("itemsLayoutMode"),
        highlightBorderBrush: ej.util.valueFunction("highlightBorderBrush"),
        borderThickness: ej.util.valueFunction("leafItemSettings.borderThickness"),
        borderBrush: ej.util.valueFunction("leafItemSettings.borderBrush"),
        showLabels: ej.util.valueFunction("leafItemSettings.showLabels"),
        labelPath: ej.util.valueFunction("leafItemSettings.labelPath"),
        iconWidth: ej.util.valueFunction("legendSettings.iconWidth"),
        iconHeight: ej.util.valueFunction("legendSettings.iconHeight"),
        dockPosition: ej.util.valueFunction("legendSettings.dockPosition"),
        legendheight: ej.util.valueFunction("legendSettings.height"),
        legendwidth: ej.util.valueFunction("legendSettings.width"),
        itemTemplate: ej.util.valueFunction("leafItemSettings.itemTemplate"),
        uniColor: ej.util.valueFunction("uniColorMapping.color"),
        _color: ej.util.valueFunction("desaturationColorMapping.color"),
        _from: ej.util.valueFunction("desaturationColorMapping.from"),
        _to: ej.util.valueFunction("desaturationColorMapping.to"),
        _rangeMinimum: ej.util.valueFunction("desaturationColorMapping.rangeMinimum"),
        _rangeMaximum: ej.util.valueFunction("desaturationColorMapping.rangeMaximum"),

        _initPrivateProperties: function () {
            this._svgDocument = null;
            this._templateDiv = null;
            this._legenddiv = null;
            this._drillHeaderDiv = null;
            this._drillHoverDiv = null;
            this._legendHeight = 0;
            this._backgroundTile = null;
            this._tooltipSize = { height: 0, width: 0 };
            this._height = 500;
            this._margintop = null;
            this._marginleft = null;
            this._startPoint = { x: 0, y: 0 };
            this._stopPoint = { x: 0, y: 0 };
            this.mouseClickable = false;
            this.dragDiv = null;
            this._initDiv = null;
            this._width = 500;
            this._svgns = "http://www.w3.org/2000/svg";
            this._prevSelectedItems=[];
            this._prevSelectedHeaders = [];
            this._isLevelColors;
			 if (this.selectedItems == null)
                this.selectedItems = [];
            this.treeMapItemSelected = null;
            this.treeMapGroupSelected = null;
            this._browser = null;
            this._toolTipElement = null;
            this._rootTreeMapItems = [];
            this.treemapgroups = [];
            this._drillHeader = null;
            this._drilldownItem = null;
            this._drilldownItems = [];
            this._treeMapHeaders = [];
            this._drilldownHeaders = [];
            this._drillDownItemSelected = null;
            this._treeMapLabels = [];
            this._labelTemplateElements = [];
            this._itemGroups = [];
            this._prevSelectedGroupDatas = [];

        },

        _setModel: function (options) {
            for (var prop in options) {
                switch (prop) {
                    case "itemsLayoutMode":
                        this.itemsLayoutMode(options[prop]);
                        this.refresh();
                        break;
                }
            }
        },
		
		
		
        _levels: function(index, property, value, old){
            this.refresh();
        },
        _rangeColorMapping: function(index, property, value, old){
            this._setColorMappings(this._rootTreeMapItems, this.model);
        },
        _groupColorMapping: function(index, property, value, old){
            this._setColorMappings(this._rootTreeMapItems, this.model);
        },
        _groupColorMapping_rangeColorMapping: function(index, property, value, old){
            this._setColorMappings(this._rootTreeMapItems, this.model);
        },
      
        _init: function () {

            this._initPrivateProperties();
            var style = $('<style>.drillGroupRect:hover{ background-color:' + this.drillDownSelectionColor() + ';}</style>');
            $('html > head').append(style);
            this._levelInitialize();

            this.refresh();           

        },

        _destroy: function() {
            this._unWireEvents();
            $(this.element).removeClass("e-treemap e-js").find().remove(".tooltipelement");
            $(this.element).empty();
        },
       
        _levelInitialize: function () {
            var proxy = this;
            if (this.model.levels != null) {
                $.each(this.model.levels, function (index, element) {                    
                    element = proxy._checkArrayObject(element, index);
                    var obj = new treeMapLevel();                    
                    $.extend(obj, element);
                    $.extend(element, obj);                    
                });
            }
            else {
                this.levels.treeMapLevel = new treeMapLevel();

            }
			
            $(document).click({treeMap: this}, this._docClickFunction);
        },

        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = new treeMapLevel();
                    proxy._loadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        _loadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, ele) {
                if (ele instanceof Array) {
                    proxy._checkArrayObject(ele, name);
                }
                else if (ele != null && typeof ele == "object") {
                    proxy._loadIndividualDefaultValues(ele, ele, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        refresh: function () {

            this._unWireEvents();
            if (this.model.enableResize) {
                if (this._drilldownItems.length == 0) {
                    this._initPrivateProperties();
                }
            }
            else {
                this._initPrivateProperties();
            }

            $(this.element).empty();
            if (this._svgDocument != null) {
                $(this._svgDocument).empty();
            }
            this._height = this.element.height();
            this._width = this.element.width();
            if (this._height == 0) {
                this._height = this.element[0].parentElement.clientHeight != 0 ? this.element[0].parentElement.clientHeight : $(document).height();
            }
            if (this._width == 0) {
                this._width = this.element[0].parentElement.clientWidth != 0 ? this.element[0].parentElement.clientWidth : $(document).width();
            }
            if (this.showLegend()) {
                this._sizeCalculation();
            }            
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();
            var isIE11 = !!navigator.userAgent.match(/Trident\/7\./);
            this._browser = browser;
            $(this.element).css({ 'position': 'relative' });			
           
            if (this._height != 0 && this._width != 0) {                
                this._templateDiv = $("<div class='_templateDiv'></div>");
                this._templateDiv.css({
                    'pointer-events': 'none', 'overflow': 'hidden', 'float': 'left',
                    'z-index': '2', 'height': this._height, 'width': this._width, "margin-top": this._margintop,  "margin-left": this._marginleft, "position": "absolute", "left": "0", "top": "0"
                });
                if (this.enableDrillDown() && this._drilldownItems.length == 0) {
                    this._drillHeaderDiv = $("<div class='_drillHeaderDiv'></div>");
                    this._drillHeaderDiv.css({
                        'overflow': 'hidden', 'float': 'left',
                        'z-index': '3', "position": "absolute", "left": "0", "top": "0"
                    });
                    this._drillHoverDiv = $("<div class='_drillHoverDiv'></div>");
                    this._drillHoverDiv.css({
                        'overflow': 'hidden', 'float': 'left',
                        'z-index': '3', 'height': this._height - 30, 'width': this._width, "position": "absolute", "left": "0", "top": "30px"
                    });
                }
                this._backgroundTile = $('<div id="backgroundTile" style="overflow:hidden;z-index:0;"></div>');
                if(this.showLegend()) {
                    if (this.model.legendSettings != null) {
                        this._renderLegend();                       
                    }
                }
                if(this.dataSource()!=null){
                    if (this._drilldownItems.length > 0) {
                        if (this.enableDrillDown() && this.model.levels.length >= 1 && ((this._drilldownItem != null) || this._drilldownItem == null)) {
                            for (var i = 0; i < this._drillHeaderDiv[0].children[0].children.length; i++) {
                                if (this._drillHeaderDiv[0].children[0].children[i].className == "e-drilldownlabel")
                                    header = this._drillHeaderDiv[0].children[0].children[i].innerHTML;
                            }
                            this._backgroundTile[0].innerHTML = "";
                            this._drillHeaderDiv[0].innerHTML = "";
                            this._templateDiv[0].innerHTML = "";
                            this._drillHoverDiv[0].innerHTML = "";
                            this._drillDownHeader(header, true);
                            this._groupdrillDownLevels(this._drilldownItems[this._drilldownItems.length - 1], this._drilldownItems.length);
                        }
                        if (this.showLegend()) {
                            this._generateLegend();
                        }
                        this._generateToolTip();
                    }
                    else {
                        if (this.model.dataSource instanceof ej.DataManager) {
                            this._processOData(this);
                        }
                        else {
                            this._renderTreeMap(this.dataSource());
                        }
                    }
                }
                if (this.enableResize()) {
                    this._on($(window), "resize", this._treeMapResize);
                }
            }
			this._selectItemResize();
        },
		
        _renderTreeMap:function(data)
        {
            this._groupLevels(data);
            if (this.showLegend()) {
                this._generateLegend();
            }
            this._generateToolTip();
            this._renderLabels();
        },
		
   _renderLegend: function () {
            this._legenddiv = $("<div class='LegendDiv'/>");
            this._legenddiv.appendTo(this.element);

            this._legenddiv.css({
                'pointer-events': 'none', 'overflow': 'hidden', "position": "absolute",
                'z-index': '2', 'height': this._legendSize.height, 'width': this._legendSize.width
            });
        },

        _processOData: function (treemap) {  
            var treeMap= this;		 
            var queryPromise = treemap.model.dataSource.executeQuery(treemap.model.query);
            queryPromise.done(function (e) {
                if(e.result!=null)
                {
                    treeMap._renderTreeMap(e.result);
                }
            });        
        },

        _treeMapResize: function (event) {
            event.preventDefault();
            event.stopPropagation();
            var treemap = this;
            if (this.resizeTO) clearTimeout(this.resizeTO);
            this.resizeTO = setTimeout(function () {
                treemap.refresh();
            }, 500);
        },
              
        _unWireEvents: function(){
            var matched = jQuery.uaMatch(navigator.userAgent);
            var browser = matched.browser.toLowerCase();                      
            

        },

        _groupLevels: function (data) {
           
            var subItems = null;
            if (this.enableDrillDown()) {                
                this._drilldownHeaders.push(this.model.levels[0].groupPath);
                this._drillDownHeader(false);
            }
            var rootItems = this._getTopGroupitem(this.labelPath(), data, 0);
            subItems = rootItems;
            if (this.model.levels.length == 0) {
                var levelItem = this.model.leafItemSettings;
                this._rootTreeMapItems = rootItems;
                this._getTopLevels(rootItems[0].innerItems, levelItem);
                this._generateTreeMapItems(rootItems[0].innerItems, "gray", this.colorValuePath());
                if(this.showLabels())
                    this._generateLabels(rootItems[0].innerItems, levelItem);
            }
            if(this.model.levels.length>0){
                for (var i = 0; i <= this.model.levels.length; i++) {
                    var levelItem = this.model.levels[i];
                  if (levelItem != null) {
                        if (!$.isNumeric(levelItem.groupGap)) levelItem.groupGap = 0;
                        if (!$.isNumeric(levelItem.groupPadding)) levelItem.groupPadding = 0;
                        if (!$.isNumeric(levelItem.groupBorderThickness)) levelItem.groupBorderThickness = 0;
                    }
                    if (i != 0) {
                        var cloneItems = subItems;
                        var PrevItem = this.model.levels[i - 1];
                        subItems = [];
                        if (levelItem == null) levelItem = this.model.leafItemSettings;
                        this._generateSubItems(cloneItems, levelItem, subItems, PrevItem);
                        
                        this._rootTreeMapItems = subItems;                        
                        if (this.enableDrillDown())
                        {
                            if (i == 1) {
                                this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);                                
                                return false;                          
                            }
                        }
                        else if (i == this.model.levels.length) {
                            this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);
                        }                                                  
                        if (this.showLabels() && i == this.model.levels.length) {
                            this._generateLabels(subItems, levelItem);
                        }
                      
                    }
                    else {
                        var rootItems1 = this._getGroupitem(levelItem.groupPath, rootItems[0].Data, levelItem.headerHeight);
                        this._rootTreeMapItems = rootItems1;
                        this._getTopLevels(rootItems1, levelItem);
                        subItems = rootItems1;                        
                    }
                }
                
            }
        },
        
        _groupdrillDownLevels: function (data, actualindex) {       

            var subItems = null;
            if (actualindex == this.model.levels.length-1)
            {
                this._drillHoverDiv.css("pointer-events", "none");
            }
            else
            {
                this._drillHoverDiv.css("pointer-events", "auto");
            }
            if (this.model.levels.length > 0) {
                var canExecute = false;
                for (var i = 0; i < this.model.levels.length; i++) {                   
                    var levelItem = this.model.levels[i];
                    if (i != 0) {
                        var cloneItems = subItems;
                        var PrevItem = this.model.levels[i - 1];
                        subItems = [];
                        if (levelItem == null) {
                            levelItem = this.model.leafItemSettings;
                        }
                        if (i == actualindex + 1) {
                            this._generateSubItems(cloneItems, levelItem, subItems, PrevItem, null, (actualindex + 1 != i));

                        }
                        else {
                            this._generateSubItems(cloneItems, levelItem, subItems, PrevItem, true, (actualindex + 1 != i));

                        }
                        this._rootTreeMapItems = subItems;
                        if ((i == actualindex + 1 || i == this.model.levels.length - 1) && !canExecute) {
                            this._generateTreeMapItems(subItems, "gray", PrevItem.DisplayPath);                                                       
                            canExecute = true;
                        }                        
                        
                        if (actualindex == this.model.levels.length - 1 && i == actualindex && levelItem.showLabels) {
                            this._generateLabels(subItems, levelItem);
                        }
                           
                    }
                    else {

                        var rootItems1 = this._getGroupitem(levelItem.groupPath, data.Data, levelItem.headerHeight);
                        this._rootTreeMapItems = rootItems1;
                        this._getTopLevels(rootItems1, levelItem);
                        subItems = rootItems1;
                    }
                }
            }
            
            if (this.showLegend()) {
                this._generateLegend();
            }
            this._renderLabels();
            this._trigger("drillDownItemSelected", { level: actualindex, header: this.getDrillDownHeader(this._drilldownHeaders), prevLevel: actualindex - 1 });
        },

        _getTopLevels: function (rootItems, level) {
            var gap = (level.groupGap != null) ? level.groupGap : 0;
            var layout = this.itemsLayoutMode();
            var legendHeight = 0;
            var defaultspacing = 0;
            if (this.enableDrillDown()) {
                defaultspacing = 30;
            }
            if (this.showLegend() && this.model.legendSettings != null) {
                legendHeight = legendHeight;                    
            }
                
            this._legendHeight = legendHeight;
            if (this.enableDrillDown()) {
                this._drillHeaderDiv.css({
                    "top": legendHeight
                });

                this._drillHoverDiv.css({
                    "top": 30 + legendHeight
                })
            }               
                
            if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceHorizontal) {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, true, legendHeight);
            }
            else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceVertical) {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, false, legendHeight);
            }
            else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceAuto) {
                this._calculateSliceAndDiceItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, null, legendHeight);
            }
            else {
                this._calculateSquarifiedItemSize(rootItems, 0, legendHeight, this._width, this._height, gap, defaultspacing, legendHeight);
            }
            
        },
      
        _generateSubItems: function (cloneItems, levelItem, subItems, PrevItem, disablelabels, disablebackground) {
            var grouppading = PrevItem.groupPadding;
            if (grouppading == "") { grouppading = 0; }
            var layout = this.itemsLayoutMode();
            var cloneItemsCount = cloneItems.length;
            var path = levelItem.groupPath;
            if (path == null) path = this.labelPath();
            if (path == null) path = this.weightValuePath();
            var gap = (levelItem.groupGap != null) ? levelItem.groupGap : (levelItem.gap != null) ? levelItem.gap:0;
            var headerHeight = ((PrevItem.showHeader || PrevItem.showHeader == null) && !this.enableDrillDown()) ? PrevItem.headerHeight : 0;
            var totalItemsCount = 0;
            for (var j = 0; j < cloneItemsCount; j++) {
                var treeItems = this._getGroupitem(path, cloneItems[j].Data, levelItem.headerHeight);
                if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceHorizontal) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, true, 0);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceVertical) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, false, 0);
                }
                else if (layout == ej.datavisualization.TreeMap.ItemsLayoutMode.SliceAndDiceAuto) {
                    this._calculateSliceAndDiceItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, null, 0);
                }
                else {
                    this._calculateSquarifiedItemSize(treeItems, cloneItems[j].LeftPosition + parseFloat(grouppading), cloneItems[j].TopPosition + parseFloat(grouppading), cloneItems[j].ItemWidth - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), cloneItems[j].ItemHeight - (PrevItem.groupBorderThickness + (2 * parseFloat(grouppading))), parseFloat(gap), headerHeight, 0);
                }                
                var headerItem = this._rootTreeMapItems[j];                
                headerItem.headerHeight = PrevItem.headerHeight;
                headerItem.showHeader = PrevItem.showHeader;
                headerItem.headerWidth = cloneItems[j].ItemWidth;
                headerItem.headerTemplate = PrevItem.headerTemplate;
                headerItem.headerLeftPosition = cloneItems[j].LeftPosition;
                headerItem.headerTopPosition = cloneItems[j].TopPosition;

                if (this.enableDrillDown())
                    this._createBackGround(headerItem, PrevItem, cloneItems[j], disablebackground);                

                if (PrevItem.showHeader || PrevItem.showHeader == null) {
                   
                    if(!this.enableDrillDown()){
                        this._createBackGround(headerItem, PrevItem, cloneItems[j]);
                        this._generateHeaders(headerItem, PrevItem);
                    }
                    
                }
               
                cloneItems[j].ChildtreeMapItems = treeItems;
                cloneItems[j].GroupingLevel = this.model.levels.indexOf(PrevItem) + 1;
				
                if (this.model.levels.indexOf(PrevItem) == 0){
                    if (this.model.groupColorMapping != null && this.model.groupColorMapping.length > 0){
                        this._isLevelColors= true;
                        for (var a = 0; a < this.model.groupColorMapping.length; a++){
                            var colormapping = this.model.groupColorMapping[a];
                            if (colormapping.groupID == headerItem.header){
                                this._setColorMappings(treeItems, colormapping);
                            }
                        }
                    }
                }
                else if( this.model.levels.indexOf(PrevItem) > 0 )
                {				
                    if (this.model.groupColorMapping != null  && this.model.groupColorMapping.length > 0){
                        for (var a = 0; a < this.model.groupColorMapping.length; a++){				
                            var colormapping = this.model.groupColorMapping[a];
                            if (colormapping.groupID == cloneItems[j].ParentHeader){
                                this._setColorMappings(treeItems, colormapping);
                            }
                        }
                    }
                }
				
                for (var k = 0; k < treeItems.length; k++) {
                    subItems[totalItemsCount] = treeItems[k];
                    if (treeItems[k].backgroundColor == null)
                        treeItems[k].backgroundColor = cloneItems[j].backgroundColor;
                    if (treeItems[k].backgroundOpacity == null)
                        treeItems[k].backgroundOpacity = cloneItems[j].backgroundOpacity;                   
                    if ((this.enableDrillDown()) || (subItems[totalItemsCount].ItemHeight < headerItem.ItemHeight - headerItem.headerHeight)) {
                        subItems[totalItemsCount].ParentHeader = headerItem.header;
                        subItems[totalItemsCount].ItemHeight -= grouppading;
                        subItems[totalItemsCount].ItemWidth -= grouppading;
                        subItems[totalItemsCount].LeftPosition += PrevItem.groupBorderThickness;
                        subItems[totalItemsCount].TopPosition += PrevItem.groupBorderThickness;
                       
                    }
                    totalItemsCount++;
                }              
            }
            if (disablelabels == undefined &&  PrevItem.showLabels) {
                this._generateLabels(cloneItems, PrevItem);
            }
        },
       
        _createBackGround: function (header, level,parentitem, disablebackground) {            
            var rect = $("<div />");
            var height = (header.showHeader || header.showHeader == null || this.enableDrillDown()) ? header.ItemHeight - (2 * level.groupBorderThickness) : 0;
            var top = (header.showHeader || header.showHeader == null || this.enableDrillDown()) ? header.TopPosition : 0;
            rect.css({
                "height": height + "px",
                "width": header.ItemWidth - (2 * level.groupBorderThickness) + "px",
                "left": header.LeftPosition + "px",
                "top": top + "px",
                "border-style": "solid",
                "border-width": level.groupBorderThickness+"px",
                "position": "absolute"               
            });

            if(level.groupBorderColor!=null)
            {
                rect.css("border-color",level.groupBorderColor);
            }
            if(level.groupBackground!=null)
            {
                rect.css("background-color",level.groupBackground);
            }
            this.treemapgroups.push(rect);
            rect.appendTo(this._backgroundTile);
            var height = parseFloat(rect[0].style.top) - 30 - this._legendHeight + "px";            
            if (this.enableDrillDown() && !disablebackground) {
                var hoverrect = $("<div class='drillGroupRect'/>");
                hoverrect.css({
                    "height": rect[0].style.height,
                    "width": rect[0].style.width,
                    "left": rect[0].style.left,
                    "top": height,
                    "position": "absolute",
                    "opacity": 0.2,
                    "display": "block"
                });
                this._drillHoverDiv.append(hoverrect);
                $(hoverrect).mousedown({ treemap: this, level: level, param1: parentitem }, this._headerClickFunction);
                $(hoverrect).mouseleave({ treemap: this, param1: header.header }, this._mouseLeaveFunction);
                var hoverData = { data: header.Data, label:header.header, header: header.header };
                $(hoverrect).mousemove({ treemap: this, hoverItem: hoverData }, this._mouseRectHoverFunction);
            }
            $(rect).mousedown({ treemap: this, level: level, param1: parentitem }, this._headerClickFunction);
            $(rect).mousemove({ treemap: this, hoverItem: header.Data }, this._mouseRectHoverFunction);
        },

        _mouseRectHoverFunction: function (event) {
          
            var treeMap = event.data.treemap;
            var item = event.data.hoverItem;
            if (treeMap.showTooltip()) {
                var element = treeMap._toolTipElement;
                var template = treeMap.model.tooltipTemplate;
                if (element != null && template != null) {
                    $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
                    var htmlString = $("#" + template).render(item);
                    $(element).html(htmlString);
                    var height = element[0] != null ? element[0].clientHeight : element.clientHeight;
                    var width = element[0] != null ? element[0].clientWidth : element.clientWidth;
                    treeMap._tooltipSize = { height: height, width: width };
                }

                var tooltipSize = treeMap._tooltipSize;
                if (tooltipSize.width + event.pageX >= treeMap._width) {
                    event.pageX -= tooltipSize.width;
                }
                if (tooltipSize.height + event.pageY >= treeMap._height) {
                    event.pageY -= tooltipSize.height;
                }               
                if (element != null) {

                    if (treeMap.enableDrillDown()) {
                        $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });

                    }
                    else {
                        $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "none" });

                    }

                }

            }
        },		        
               
        _mouseLeaveFunction: function (event) {
            var treeMap = event.data.treemap;
            if (treeMap.showTooltip())
            {
                if (treeMap._toolTipElement != null) {
                    $(treeMap._toolTipElement).css("display", "none");
                }
            }
        },
		       
        _headerClickFunction: function (event) {
            var level = event.data.level;
            var data = event.data.param1;
            var ctrlkey = event.ctrlKey;
            var treeMap = event.data.treemap;
            if (treeMap.highlightGroupOnSelection()) {
                if (treeMap._browser != "msie") {
                    var lastItem = treeMap._backgroundTile[0].children[treeMap._backgroundTile[0].children.length - 0];
                    treeMap._backgroundTile[0].insertBefore(this, lastItem);
                }
                if (treeMap.groupSelectionMode() == "multiple" &&  ctrlkey) {

                    if ($.inArray(this,treeMap._prevSelectedHeaders)==-1) {
                        $(this).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush() });
                        treeMap._prevSelectedHeaders.push(this);
                        treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                    }
                    else {
                        
                        $(this).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
                        var index = treeMap._prevSelectedHeaders.indexOf(this);
                        treeMap._prevSelectedHeaders.splice(index, 1);
                        treeMap._prevSelectedGroupDatas.splice(index, 1);
                    }
                }
                else {
                    for (var k = 0; k < treeMap._prevSelectedHeaders.length; k++) {
		               
                        $(treeMap._prevSelectedHeaders[k]).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
                    }

                    if ($.inArray(this, treeMap._prevSelectedHeaders) == -1) {
                        $(this).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush() });
                        treeMap._prevSelectedHeaders = [];
                        treeMap._prevSelectedHeaders.push(this);
                    }
                    else {
                        treeMap._prevSelectedHeaders = [];
                    }
		            
                    treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                }
            }
			if (treeMap.enableDrillDown() && treeMap.model.levels.length > 1 && ((treeMap._drilldownItem != null && treeMap._drilldownItem.header != data.header) || treeMap._drilldownItem == null)) {
                    var header = "";
                    for(var i=0;i< treeMap._drillHeaderDiv[0].children[0].children.length;i++)
                    {
                        if (treeMap._drillHeaderDiv[0].children[0].children[i].className == "e-drilldownlabel")
                            header = treeMap._drillHeaderDiv[0].children[0].children[i].innerHTML;
                    }
                    treeMap._backgroundTile[0].innerHTML = "";
                    treeMap._labelTemplateElements = [];
                    treeMap._drillHeaderDiv[0].innerHTML = "";
                    treeMap._templateDiv[0].innerHTML = "";
                    treeMap._drillHoverDiv[0].innerHTML = "";
                    treeMap._drilldownHeaders.push(data.header);
                    treeMap._drillDownHeader(true);
                     treeMap._drilldownItem = data;
                    treeMap._drilldownItems.push(data);
                    treeMap._groupdrillDownLevels(data, treeMap._drilldownItems.length);
                    return false;
		       		        
                }
            },
            rectClick:function(event)
            {
                var treeMap = event.data.treemap;
                var itemCollection = event.data.Param1;
                var level = treeMap.model.levels[0];
                var tooltipObject;
				 var isContain;
                var ctrlkey = event.ctrlKey;
                if (treeMap.highlightGroupOnSelection()) {
                    for (var i = 0; i < treeMap.treemapgroups.length; i++) {
                        var item = treeMap.treemapgroups[i];
                        var rect = item[0].getBoundingClientRect();
                   
                        if (rect.left < event.clientX && rect.left + rect.width > event.clientX
                        && rect.top < event.clientY && rect.top + rect.height > event.clientY) {
                            $(item).css({
                                "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush(),
                                "box-sizing":"border-box",
                                "-moz-box-sizing":"border-box",
                                "-webkit-box-sizing":"border-box",
                            });
							isContain = true;
                            break;
                        }
                    }
					if(isContain)
					{
                        if (treeMap.groupSelectionMode() == "multiple" && ctrlkey) {

                            if ($.inArray(item, treeMap._prevSelectedHeaders) == -1) {
                                $(item).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush()});
                                treeMap._prevSelectedHeaders.push(item);
                                treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                            }
                            else {

                                $(item).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor,});
                                var index = treeMap._prevSelectedHeaders.indexOf(item);
                                treeMap._prevSelectedHeaders.splice(index, 1);
                                treeMap._prevSelectedGroupDatas.splice(index, 1);
                            }
                        }
                        else {
                            for (var k = 0; k < treeMap._prevSelectedHeaders.length; k++) {

                                $(treeMap._prevSelectedHeaders[k]).css({ "border-width": level.groupBorderThickness, "border-color": level.groupBorderColor });
                            }

                            if ($.inArray(item, treeMap._prevSelectedHeaders) == -1) {
                                $(item).css({ "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.highlightBorderBrush() });
                                treeMap._prevSelectedHeaders = [];
                           
                                treeMap._prevSelectedHeaders.push(item);
                            }
                            else {
                                treeMap._prevSelectedHeaders = [];                                              
                            }

                            treeMap._prevSelectedGroupDatas.push(event.data.Param1);
                        }								
					}
					treeMap._trigger("treeMapGroupSelected", { selectedGroups: treeMap._prevSelectedHeaders, orginalEvent: event });
                }

                if (treeMap.highlightOnSelection()) {

                    for (var i = 0; i < treeMap._rootTreeMapItems.length; i++) {
                        var item = treeMap._rootTreeMapItems[i];
                        var rect = item.Rectangle.getBoundingClientRect();
                        if (rect.left < event.clientX && rect.left + rect.width > event.clientX
                            && rect.top < event.clientY && rect.top + rect.height > event.clientY) {
                            $(item.Rectangle).css({
                                "border-width": treeMap.highlightBorderThickness(), "border-color": treeMap.highlightBorderBrush(),
                                "box-sizing": "border-box",
                                "-moz-box-sizing": "border-box",
                                "-webkit-box-sizing": "border-box",
                            });
                            isContain = true;
                            break;
                        }
                    }

                    if (isContain) {
                        if (treeMap.selectionMode() == "multiple" && ctrlkey) {
                            if ($.inArray(item.Rectangle, treeMap._prevSelectedItems) == -1) {
                                $(item.Rectangle).css({
                                    "border-width": treeMap.highlightBorderThickness(), "border-color": treeMap.highlightBorderBrush(),
                                    "box-sizing": "border-box",
                                    "-moz-box-sizing": "border-box",
                                    "-webkit-box-sizing": "border-box",
                                });
                                treeMap._prevSelectedItems.push(item.Rectangle);
                                treeMap.selectedItems.push(item.Data);
                            }
                            else {
                                $(item.Rectangle).css({ "border-width": treeMap.borderThickness(), "border-color": treeMap.borderBrush() });
                                var index = treeMap._prevSelectedItems.indexOf(item.Rectangle);
                                treeMap._prevSelectedItems.splice(index, 1);
                                treeMap.selectedItems.splice(index, 1);
                            }
                        }
                        else {
                            for (var k = 0; k < treeMap._prevSelectedItems.length; k++) {
                                $(treeMap._prevSelectedItems[k]).css({ "border-width": treeMap.borderThickness(), "border-color": treeMap.borderBrush() });
                            }

                            if ($.inArray(item.Rectangle, treeMap._prevSelectedItems) == -1) {
                                $(item.Rectangle).css({ "border-width": treeMap.highlightBorderThickness(), "border-color": treeMap.highlightBorderBrush() });
                                treeMap._prevSelectedItems = [];
                                treeMap.selectedItems = [];
                                treeMap._prevSelectedItems.push(item.Rectangle);
                                treeMap.selectedItems.push(item.Data);
                            }
                            else {
                                treeMap._prevSelectedItems = [];
                                treeMap.selectedItems = [];
                            }
                        }
                        treeMap._trigger("treeMapItemSelected", { selectedItems: treeMap.selectedItems, originalEvent: event });
                    }
                    
                }
                if (treeMap.model.showTooltip && treeMap._toolTipElement != null) {

                    var element = treeMap._toolTipElement;
                    var position;
                    if (event.type == "mousedown")
                        position = { x: event.pageX, y: event.pageY };
                    else if (event.type == "touchstart")
                        position = { x: event.originalEvent.changedTouches[0].pageX, y: event.originalEvent.changedTouches[0].pageY };
                    else if (event.type == "MSPointerDown")
                        position = { x: event.originalEvent.pageX, y: event.originalEvent.pageY };
                    var tooltipSize = treeMap._tooltipSize;
                    if (tooltipSize.width + position.x >= treeMap._width) {
                        position.x -= tooltipSize.width;
                    }
                    if (tooltipSize.height + position.y >= treeMap._height) {
                        position.y -= tooltipSize.height;
                    }

                    if (element != null && event.data.treemap != null) {
                        var template = treeMap.model.tooltipTemplate;
                        $(element).css({ "left": position.x + 10, "top": position.y + 10, "display": "block" });


                        for (var i = 0; i < itemCollection.length; i++) {
                            var item = itemCollection[i];
                            var rect = item.Rectangle.getBoundingClientRect();
                            if (rect.left < event.clientX && rect.left + rect.width > event.clientX
                                && rect.top < event.clientY && rect.top + rect.height > event.clientY) {
                                item = item.Data
                                break;
                            }
                        }
                        var htmlString = $("#" + template).render(item);
                        $(element).html(htmlString);
                    }

                }
            
        },
        
            dragDown: function (event) {

                var treeMap = event.data.treemap;
                var level = treeMap.model.levels[0];

                var mapArea = treeMap.element;
                if (event.type == "mousedown") {
                    treeMap._startPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
                    treeMap._stopPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
                }
                else if (event.type == "touchstart") {
                    treeMap._startPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
                    treeMap._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
                }
                else if (event.type == "MSPointerDown") {
                    treeMap._startPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
                    treeMap._stopPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };
                }

                treeMap.mouseClickable = true;

                if (treeMap.model.draggingGroupOnSelection) {

                    if (treeMap._prevSelectedGroupDatas.length > 0) {
                        for (var i = 0; i < treeMap._prevSelectedGroupDatas.length; i++) {
                            var item = treeMap._prevSelectedGroupDatas[i][0];
                            var rect = item.getBoundingClientRect();
                            if ($.inArray(item, treeMap._prevSelectedGroupDatas[i][0]) == -1) {
                                $(treeMap._prevSelectedGroupDatas[i]).css({ "border-width": level.groupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush });
                            }
                        }
                        treeMap._prevSelectedGroupDatas = [];
                    }
                }

                if (treeMap.model.draggingOnSelection) {

                    if (treeMap._prevSelectedItems.length > 0) {
                        for (var i = 0; i < treeMap._prevSelectedItems.length; i++) {
                            var item = treeMap._prevSelectedItems[i].rectangle;
                            var rect = item.getBoundingClientRect();
                            if ($.inArray(item, treeMap._prevSelectedItems[i].rectangle) == -1) {
                                $(treeMap._prevSelectedItems[i].rectangle).css({ "border-width": level.groupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush });
                            }
                        }
                        treeMap._prevSelectedItems = [];
                    }
                }
            },

            dragMove: function (event) {

                var width; var height;
                var treeMap = event.data.treemap;
                var mapArea = treeMap.element;

                if (treeMap.model.draggingGroupOnSelection || treeMap.model.draggingOnSelection) {

                    if (treeMap.mouseClickable) {

                        if (event.type == "mousemove")
                            treeMap._stopPoint = { x: event.pageX - mapArea[0].offsetLeft, y: event.pageY - mapArea[0].offsetTop };
                        else if (event.type == "touchmove")
                            treeMap._stopPoint = { x: event.originalEvent.changedTouches[0].pageX - mapArea[0].offsetLeft, y: event.originalEvent.changedTouches[0].pageY - mapArea[0].offsetTop };
                        else if (event.type == "MSPointerMove")
                            treeMap._stopPoint = { x: event.originalEvent.pageX - mapArea[0].offsetLeft, y: event.originalEvent.pageY - mapArea[0].offsetTop };


                        $('#dragDiv').remove();
                        var div = $('<div id = "dragDiv"></div>');

                        width = Math.abs(treeMap._stopPoint.x - treeMap._startPoint.x),
                        height = Math.abs(treeMap._stopPoint.y - treeMap._startPoint.y);
                        $(div).css({ "top": Math.min(treeMap._startPoint.y, treeMap._stopPoint.y), "left": Math.min(treeMap._startPoint.x, treeMap._stopPoint.x), width: width, height: height, border: "1px solid green", position: "absolute", "z-index": 100 });
                        $(div).appendTo("#svgDocument");
                    }
                }
            },

            dragUp: function (event) {

                var treeMap = event.data.treemap;
                var width = Math.abs(treeMap._stopPoint.x - treeMap._startPoint.x);
                var height = Math.abs(treeMap._stopPoint.y - treeMap._startPoint.y);
                var mapArea = treeMap.element;

                if (treeMap.model.draggingGroupOnSelection || treeMap.model.draggingOnSelection) {

                    $('#dragDiv').remove();
                    $("#dragDiv").css({
                        "display": "none"
                    });
                    treeMap.mouseClickable = false;
                }

                if (treeMap.model.draggingGroupOnSelection) {

                    for (var i = 0; i < treeMap.treemapgroups.length; i++) {
                        var item = treeMap.treemapgroups[i];
                        var rect = item[0].getBoundingClientRect();

                        if (!(rect.left - mapArea[0].offsetLeft + rect.width < Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) || Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) + width < rect.left - mapArea[0].offsetLeft ||
                            rect.top - mapArea[0].offsetTop + rect.height < Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) || Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) + height < rect.top - mapArea[0].offsetTop)) {
                            if (!treeMap._contains(treeMap._prevSelectedGroupDatas, item))
                                treeMap._prevSelectedGroupDatas.push(item);
                        }
                    }

                    for (var i = 0; i < treeMap._prevSelectedGroupDatas.length; i++) {
                        var item = treeMap._prevSelectedGroupDatas[i][0];
                        var rect = item.getBoundingClientRect();

                        $(item).css({
                            "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush,
                            "box-sizing": "border-box",
                            "-moz-box-sizing": "border-box",
                            "-webkit-box-sizing": "border-box",
                        });
                    }
                    treeMap.selectedItems.push(treeMap._prevSelectedGroupDatas);
                }

                if (treeMap.model.draggingOnSelection) {

                    for (var i = 0; i < treeMap._rootTreeMapItems.length; i++) {
                        var item = treeMap._rootTreeMapItems[i];
                        var rect = item.rectangle.getBoundingClientRect();

                        if (!(rect.left - mapArea[0].offsetLeft + rect.width < Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) || Math.min(treeMap._startPoint.x, treeMap._stopPoint.x) + width < rect.left - mapArea[0].offsetLeft ||
                             rect.top - mapArea[0].offsetTop + rect.height < Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) || Math.min(treeMap._startPoint.y, treeMap._stopPoint.y) + height < rect.top - mapArea[0].offsetTop)) {
                            if (!treeMap._contains(treeMap._prevSelectedItems, item))
                                treeMap._prevSelectedItems.push(item);
                        }
                    }

                    for (var i = 0; i < treeMap._prevSelectedItems.length; i++) {
                        var item = treeMap._prevSelectedItems[i].rectangle;
                        var rect = item.getBoundingClientRect();
                        $(item).css({
                            "border-width": treeMap.model.highlightGroupBorderThickness, "border-color": treeMap.model.highlightGroupBorderBrush,
                            "box-sizing": "border-box",
                            "-moz-box-sizing": "border-box",
                            "-webkit-box-sizing": "border-box",
                        });
                    }
                    treeMap.selectedItems.push(treeMap._prevSelectedItems);
                }
            },

        _generateLabels: function (Items,level) {            
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                if (level.labelTemplate == null) {                    
                    var leftpaddingsize = 8;
                    var toppaddingsize = 5;
                    var textcon = $('<label style="font-size:14px;font-weight:normal;font-family:Segoe UI;">' + item.label + '</label>');
                    if (this.enableDrillDown())
                    {
                        $(textcon).css({
                            "position": "absolute","color":"black",
                            "overflow": "hidden",  "left": item.LeftPosition + leftpaddingsize + "px",
                            "top": item.TopPosition + toppaddingsize + "px", "font-weight":"normal","pointer-events":"none"
                        });
                    }
                    else
                    {
                        $(textcon).css({
                            "position": "absolute", "color": "white",
                            "overflow": "hidden", "left": item.LeftPosition + leftpaddingsize + "px",
                            "top": item.TopPosition + toppaddingsize + "px", "font-weight": "normal", "pointer-events": "none"
                        });
                    }
                    this._treeMapLabels.push(textcon);
                    this._templateDiv.append(textcon);
                    var padding = 0, borderthickness = 0;
                    if (level.groupPadding != undefined)
                        padding = level.groupPadding;
                    if (level.groupBorderThickness != undefined)
                        borderthickness = level.groupBorderThickness;
                    var labelWidth = (textcon[0].getBoundingClientRect().width > 0) ? textcon[0].getBoundingClientRect().width + padding + borderthickness + 2 : item.label.length * 8;
                    var labelWidth = (labelWidth > item.ItemWidth - leftpaddingsize/2) ? item.ItemWidth - leftpaddingsize/2 : labelWidth;
                    var labelHeight = textcon[0].getBoundingClientRect().height;
                    if (labelHeight == 0 && item.ItemHeight > 18)
                        labelHeight = 18;
                    else if (labelHeight > 0 && labelHeight > item.ItemHeight)
                        labelHeight = item.ItemHeight - toppaddingsize;
                    $(textcon).css({ "width": labelWidth + "px", "height": labelHeight + "px" });
                    item.labelPosition = level.labelPosition;
                    item.labelVisibilityMode = borderthickness;
                    item.groupPadding = level.groupPadding;
                    item.groupBorderThickness = level.groupBorderThickness;
                    textcon[0].data = item;
                    this._labelTemplateElements.push(textcon);
                    var textMode = level.labelVisibilityMode;
                    if (textMode == ej.datavisualization.TreeMap.VisibilityMode.HideOnExceededLength){						
                        var textelement = $('<span>'+textcon[0].innerHTML+'</span>');
                        $(document.body).append(textelement);
                        var width = textelement.outerWidth();
                        var height = textelement.outerHeight();
                        textelement.remove();
                        var widthfactor;
                        if (width > item.ItemWidth)
                        {							
                            widthfactor = width / item.ItemWidth;
                        }
                        else
                        {
                            widthfactor = item.ItemWidth / width;
                        }			
                        var neededWidth = widthfactor * height + (2 * leftpaddingsize * widthfactor);	
                        var availableheight = item.ItemHeight - (2 * toppaddingsize);						
                        var hidelabel = false;						
                        if (width > (item.ItemWidth - leftpaddingsize) && height < (item.ItemHeight - toppaddingsize)){
                            if ( neededWidth < availableheight){
                                $(textcon).css({
                                    "display": "block",
                                    "word-wrap": "break-word"
                                });
                            }
                            else{
                                hidelabel = true;
                            }
                        }
                        else if (height > (item.ItemHeight - 2*toppaddingsize)){
                            hidelabel = true;
                        }
                        else if (width > (item.ItemWidth - leftpaddingsize)){
                            hidelabel = true;
                        }
						
                        if (hidelabel){
                            $(textcon).css({
                                "display": "none"
                            });
                        }
                    }
                }            
                if(level.labelTemplate != null) {
                    var OriginalNode = $("#" + level.labelTemplate);
                    var item_templateDiv = $("<div style='overflow:hidden;display:block;position:absolute;pointer-events: none;'></div>");
                    item_templateDiv[0].data = item;
                    item.labelPosition = level.labelPosition;
                    item.labelVisibilityMode = level.labelVisibilityMode;
                    var tmpl2 = $.templates(OriginalNode.html());
                    var htmlString = "";
                    if (item.Data instanceof Array)
                        htmlString = tmpl2.render(item.Data[0]);
                        
                    else
                        htmlString = tmpl2.render(item.Data);
                    $(item_templateDiv).html(htmlString);
                    this._templateDiv.append(item_templateDiv);
                    this._labelTemplateElements.push(item_templateDiv);
                }
            }
        },

        _generateToolTip: function () {

            if (this.showTooltip()) {
                if (document.documentMode == 8){
                    var tooltip = document.querySelectorAll("tooltipelement");
                }
                else{
                    var tooltip = document.getElementsByClassName("tooltipelement");
                }
                if (tooltip != null && tooltip.length == 0) {
                    var tooltip_templateDiv = $("<div class='tooltipelement' style='display:none;position:absolute;z-index:1000;pointer-events:none;'></div>");
                    $(document.body).append(tooltip_templateDiv);
                    this._toolTipElement = tooltip_templateDiv;
                }
                else
                {
                    this._toolTipElement = tooltip[0];
                }
                if (this.model.tooltipTemplate == null) {

                    var path;
                    if (this.labelPath()!=null)
                        path = this.labelPath();
                    else 
                        path = this.weightValuePath();
                    if (path != null) {
                        this.model.tooltipTemplate = 'defaultTooltip';

                        this.element.append($('<div id="defaultTooltip" style="display:none;"><div style="margin-left:10px;margin-top:-25px;"><div class="defaultToolTip"><p style="margin-top:-4px"><label  style="color:rgb(27, 20, 20);font-size:14px;font-weight:normal;font-family:Segoe UI">{{:#data["' + path + '"]}}</label></p></div></div></div>'));
                    }
                }
            }           
        },

        _generateLegend: function () {            
            if (this.model.rangeColorMapping != null) {
                var colormapping = this.model.rangeColorMapping;
                var xPos = 0;
                var yPos = 0;
                var legendSettings = this.model.legendSettings;
                var columnWidth = 0;
                var width = 0;
                var height = 0;
                var iconwidth = this.iconWidth() + 5;
                var totalHeight = this.iconHeight() + 15;
                var totalWidth = 0;
                var columnCount = this.model.legendSettings.columnCount;
                for (var i = 0; i < colormapping.length; i++) {

                    var labelwidth = this._calcWidth(colormapping[i].legendLabel);
                    var legendWidth = this.iconWidth() + 10 + labelwidth;


                    if (columnCount != 0) {
                        if (i % columnCount != 0) {
                           
                            this._drawLegend(colormapping[i], xPos, yPos);
                            xPos += (legendWidth+5);

                        }
                        else {
                            if (i != 0)
                            yPos += (this.iconHeight() + 18);
                            xPos = 0;
                            this._drawLegend(colormapping[i], xPos, yPos);
                            xPos += (legendWidth + 5);
                        }
                    }
                    else {
                        if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {

                            if (this._legendSize.width < xPos + legendWidth) {

                                yPos += (this.iconHeight() + 18);
                                xPos = 0;
                                this._drawLegend(colormapping[i], xPos, yPos);
                                xPos += legendWidth;
                            }
                            else {
                                this._drawLegend(colormapping[i], xPos, yPos);
                                xPos += legendWidth;
                            }
                        }
                        else {

                            if (this._legendSize.height < yPos + this.iconHeight()) {

                                yPos = 0;
                                xPos += (columnWidth + 10);
                                this._drawLegend(colormapping[i], xPos, yPos);
                                columnWidth = 0;
                                yPos += (this.iconHeight() + 18);
                            }
                            else {
                                this._drawLegend(colormapping[i], xPos, yPos);
                                columnWidth = Math.max(columnWidth, legendWidth);
                                yPos += (this.iconHeight() + 18);
                            }
                        }

                    }
                }


                if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Center) {
                    this._legenddiv.css({
                        "margin-left": (this._width / 2) - (this._legendSize.width / 2)
                    });
                }
                else if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Far) {
                    this._legenddiv.css({
                        "margin-left": this._width - this._legendSize.width
                    });
                }
                else if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Left || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Center) {
                    this._legenddiv.css({
                        "margin-top": (this._height / 2) - (this._legendSize.height / 2)
                    });
                }
                else if ((this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Left || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) && legendSettings.alignment == ej.datavisualization.TreeMap.Alignment.Far) {
                    this._legenddiv.css({
                        "margin-top": this._height - this._legendSize.height
                    });
                }

                if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                    this._legenddiv.css({
                        "margin-top": this._height + 5
                    });
                }
                else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) {

                    this._legenddiv.css({

                        "margin-left": this._width + 5

                    });
                }
            }
        },
		 _drawLegend: function (colormapping, xPos, yPos) {
            var legendItem;
            if (this.model.legendSettings.template == "Ellipse") {
                legendItem = this._getEllipseLegend(this.model.legendSettings, xPos, yPos);
            }
            else {
                legendItem = this._getRectLegend(this.model.legendSettings, xPos, yPos);
            }
            xPos += (this.iconWidth() + 5);
            $(legendItem).css("background-color", colormapping.color);
            legendItem.appendTo(this._legenddiv);
            var legendText = $("<div/>");
            legendText.css({
                "left": xPos + "px",
                "top": yPos + "px",
                        "position": "absolute"
                    });
                    $(legendText).css({"font-size":"15px"});
            legendText[0].innerHTML = colormapping.legendLabel;
            legendText.appendTo(this._legenddiv);
        },
       
        _calcWidth: function (text) {
            var span = $('<span>' + text + '</span>');
            $('body').append(span);
            var width = span.width();
            span.remove();
            return width;
        },
        
        _getEllipseLegend: function (legengItem, xPos, yPos) {

            var rect = $("<div class='e-mapLegend'/>");
            rect.css({
                "height": this.iconHeight()+ "px",
                "width": this.iconWidth() + "px",
                "border-radius":this.iconHeight()/2+"px",
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;

        },
       
        _getRectLegend: function (legengItem, xPos, yPos) {
            var rect = $("<div />");
            rect.css({
                "height": this.iconHeight()+ "px",
                "width": this.iconWidth() + "px",                
                "left": xPos + "px",
                "top": yPos + "px",
                "position": "absolute"
            });
            return rect;
        },
        
        _getPositionFromParent: function (element, parentelement) {
            var child = element.getBoundingClientRect();
            var parent = parentelement.getBoundingClientRect();
            var width = window.SVGSVGElement ? child.width: child.right-child.left;
            var height = window.SVGSVGElement ? child.height : child.bottom - child.top;            
            return { left: child.left - parent.left, top: child.top - parent.top, height: height, width: width };
        },
        
        _renderLabels: function () {
           
            for (var j = 0; j < this._labelTemplateElements.length; j++) {
                var label = this._labelTemplateElements[j];
                var bounds = this._getPositionFromParent(label[0], this._templateDiv[0]);
                var item = label[0].data;
                var pos = this._getDockPositionPoint({left:item.LeftPosition,top:item.TopPosition, width: item.ItemWidth, height: item.ItemHeight }, bounds, item);
                
                $(label).css({
                    "left": pos.x,
                    "top": pos.y,
                    "pointer-events": "none",
                    "overflow":"hidden"
                });
                if (item.labelVisibilityMode == ej.datavisualization.TreeMap.VisibilityMode.HideOnExceededLength) {
                    if (bounds.height > item.ItemHeight || bounds.width > item.ItemWidth) {
                        $(label).css("display", "none");
                    }
                }
            }
        },
       
        _generateHeaders: function (headerValue,level) {           
            var textMode = level.headerVisibilityMode;
            var generateHeader = false;
            var leftpadding = 2;
            var toppadding = 2;
            if (textMode == ej.datavisualization.TreeMap.VisibilityMode.HideOnExceededLength) {
                var textelement = $('<span>'+headerValue.header+'</span>');
                $(document.body).append(textelement);
                var width = textelement.outerWidth();
                textelement.remove();
                if (headerValue.headerHeight < (headerValue.ItemHeight- (2 * leftpadding)) && width < (headerValue.ItemWidth-(2 * toppadding))){
                    generateHeader = true;
                }					
            }
            else {
                if (headerValue.headerHeight < headerValue.ItemHeight){
                    generateHeader = true;
                }
            }
            if(generateHeader) {
                if (headerValue.headerTemplate == null) {
                    var headerItem = $('<div style="display:block;position:absolute;pointer-events: stroke;overflow: hidden;"><label style="font-size:15px;font-weight:normal;font-family:Segoe UI;">' + headerValue.header + '</label></div>');
                    $(headerItem).css({
                        "left": headerValue.headerLeftPosition+leftpadding,
                        "top": headerValue.headerTopPosition+toppadding,
                        "width": headerValue.headerWidth - level.groupPadding,
                        "height": headerValue.headerHeight,
                        "margin-left": level.groupPadding                    
                    });
                    this._templateDiv.append(headerItem);
                }
                else {
                    var item_templateDiv = $("<div style='display:block;position:absolute;pointer-events: none;overflow: hidden;'></div>");
                    this._templateDiv.append(item_templateDiv);
                    $(item_templateDiv).css({
                        "left": headerValue.headerLeftPosition,
                        "width": headerValue.headerWidth, "top": headerValue.headerTopPosition,
                        "height": headerValue.headerHeight
                    });
                    var arguments = { header: headerValue.header, data: headerValue.Data };
                    var htmlString = $("#" + headerValue.headerTemplate).render(arguments);
                    $(item_templateDiv).html(htmlString);
                }
            }
        },

        getDrillDownHeader:function(header)
        {
            var headerString = "";
            if(header.length==1)
            {
                return header[0];
            }
            else{
                for (var index = 0; index < header.length; index++)
                {
                    headerString += header[index];
                    if(index!=header.length-1)
                    {
                        headerString += ".";
                    }
                }
            }
            return headerString;
            
        },
	
        _drillDownHeader: function (enableNavigation) {

            var drilledHeader = this.getDrillDownHeader(this._drilldownHeaders);
            var item_templateDiv = $("<div class='e-drilldownHeader' style='display:block;overflow: hidden;'><div class='e-drilldownlabel' style='height:30px;position:absolute;margin-top: 5px;top: 0px;'>" + drilledHeader + "</div></div>");
            if (enableNavigation)
                item_templateDiv = $("<div class='e-drilldownHeader' style='display:block;overflow: hidden;'><svg class='e-drilldownarrow' style='width:15px;height:15px;margin-top: 10px;'><polyline points='8,0 8,10 0,5 8,0'/></svg><div class='e-drilldownlabel'  style='left:17px;position:absolute;height:30px;position:absolute;margin-top: 5px;top: 0px;'>" + drilledHeader + "</div></div>");
            
            $(item_templateDiv).css({
                "left": "0px",
                "width": this._width+"px","top": 0,
                "height": "30px","cursor":"pointer"
            });    
            if(this.drillDownHeaderColor()!=null)
            {
                $(item_templateDiv).css("background-color",this.drillDownHeaderColor());
            }			
            this._drillHeaderDiv.append(item_templateDiv);
            $(item_templateDiv).mousedown({ treemap: this }, this._drilldownfunction);
            
        },
		
        _drilldownfunction: function (event) {
            var treeMap = event.data.treemap;                       
            if (treeMap._drilldownItems.length > 0) {
                if (treeMap._drilldownItems.length == 1) {
                    treeMap._drilldownItems = [];
                    treeMap.refresh();
                }
                else {
                    var header = "";
                    for (var i = 0; i < treeMap._drillHeaderDiv[0].children[0].children.length; i++) {
                        if (treeMap._drillHeaderDiv[0].children[0].children[i].className == "e-drilldownlabel")
                            header = treeMap._drillHeaderDiv[0].children[0].children[i].innerHTML;
                    }
                    var headers = header.split(".");
                    treeMap._drillHeaderDiv[0].innerHTML = "";
                    treeMap._backgroundTile[0].innerHTML = "";
                    var drillHeader = "";
                    if (treeMap._drilldownHeaders.length > 1)
                    {
                        treeMap._drilldownHeaders.pop(treeMap._drilldownHeaders.length - 1);
                    }                    
                    treeMap._labelTemplateElements = [];
                    treeMap._drillDownHeader(true);
                    treeMap._drilldownItem = treeMap._drilldownItems[treeMap._drilldownItems.length - 2];
                    treeMap._drilldownItems.pop(treeMap._drilldownItems[treeMap._drilldownItems.length - 1]);
                    treeMap._templateDiv[0].innerHTML = "";
                    treeMap._groupdrillDownLevels(treeMap._drilldownItem, treeMap._drilldownItems.length);
                }
                treeMap._trigger("drillDownItemSelected", { level: treeMap._drilldownItems.length, header: treeMap.getDrillDownHeader(treeMap._drilldownHeaders), prevLevel: treeMap._drilldownItems.length - 1 });
            }            
        },
       
      _sizeCalculation: function () {

            if (this.model.rangeColorMapping != null) {
                var colormapping = this.model.rangeColorMapping;
                var width = 0;
                var height = 0;
                var iconwidth = this.iconWidth() + 5;
                var totalHeight = this.iconHeight() + 15;
                var totalWidth = 0;
                var columnCount = this.model.legendSettings.columnCount;

                var legendSettings = this.model.legendSettings;
                if ((this.legendheight() == 0 && this.legendwidth() == 0) && (columnCount == 0)) {
                    for (var i = 0; i < colormapping.length; i++) {
                        var labelwidth = this._calcWidth(colormapping[i].legendLabel);
                        var legendWidth = iconwidth + labelwidth + 10;
                        if (this.legendheight() == 0 && this.legendwidth() == 0) {
                            if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                                if (this._width < (width + legendWidth)) {
                                    totalHeight = (this.iconHeight() + 15) + totalHeight;
                                    totalWidth = Math.max(totalWidth, width);
                                    width = legendWidth;
                                }
                                else {
                                    width += legendWidth;
                                }
                            }
                            else {

                                if (this._height < (height + this.iconHeight() + 15)) {
                                    totalWidth += width;
                                    totalHeight = Math.max(totalHeight, height);
                                    width = legendWidth;
                                    height = 0;
                                }
                                else {
                                    height += (this.iconHeight() + 18);
                                    width = Math.max(legendWidth, width);
                                }
                            }
                        }
                        else {

                            xPos += _legendlabelwidth + 18;
                            if (xPos + 150 >= this.legendwidth()) {
                                xPos = 10;
                                yPos += 40;
                            }
                        }
                    }

                    if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top || this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                        totalWidth = Math.max(totalWidth, width);
                        totalHeight += 5;
                    }
                    else {
                        totalWidth += (width + 15);
                        totalHeight = Math.max(totalHeight, height);
                    }
                }
                else if (columnCount == 0) {
                    if ((this.legendheight().toString().indexOf("%")) != -1) {

                        totalHeight = (this._height / 100) * parseInt(this.legendheight().replace('%', ''))
                    }
                    else {

                        totalHeight = this.legendheight();
                    }
                    if ((this.legendwidth().toString().indexOf("%")) != -1) {

                        totalWidth = (this._width / 100) * parseInt(this.legendwidth().replace('%', ''))
                    }
                    else {

                        totalWidth = this.legendwidth();
                    }

                }

                if (columnCount != 0) {

                    for (var i = 0; i < colormapping.length; i++) {
                        var labelwidth = this._calcWidth(colormapping[i].legendLabel);
                        var legendWidth = iconwidth + labelwidth + 10;

                        if (i % columnCount != 0) {
                            width += legendWidth;
                            if (i == columnCount - 1) {
                                totalWidth = Math.max(totalWidth, width);
                            }

                        }
                        else {
                            if (i != 0)
                                totalHeight = (this.iconHeight() + 15) + totalHeight;
                            totalWidth = Math.max(totalWidth, width);
                            width = legendWidth;
                        }
                    }
                }

                this._legendSize = { height: totalHeight, width: totalWidth };
                if (this.model.legendSettings != null) {

                    if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Bottom) {
                        this._height = this._height - parseFloat(totalHeight);
                        this.model.legendSettings.tempWidth = width;
                    }

                    else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Top) {
                        this._height = this._height - parseFloat(totalHeight);

                        this._margintop = parseFloat(totalHeight);
                    }
                    else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Left) {
                        this._width = this._width - totalWidth;
                        this._marginleft = totalWidth;

                    }
                    else if (this.dockPosition() == ej.datavisualization.TreeMap.DockPosition.Right) {
                        this._height = this._height;
                        this._width = this._width - totalWidth;
                    }
                }
            }
        },

        _getDockPositionPoint:function(size,bounds,item)
        {
            var position = item.labelPosition;
            var x = 2, y = 2;
            if (item.groupPadding != undefined && item.groupBorderThickness != undefined) {
                x = item.groupPadding + item.groupBorderThickness + x;
                y = item.groupPadding + item.groupBorderThickness + y;
            }
            if (item.showHeader && !this.enableDrillDown())
                y = y + item.headerHeight;
            if (position == ej.datavisualization.TreeMap.Position.TopCenter) {
                x = (size.width/2)-(bounds.width/2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.TopRight) {
                x = size.width - bounds.width - x;
            }
            else if (position == ej.datavisualization.TreeMap.Position.CenterLeft) {
                y = (size.height / 2) - (bounds.height / 2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.Center || position==null) {
                x = (size.width / 2) - (bounds.width / 2);
                y = (size.height / 2) - (bounds.height / 2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.CenterRight) {
                x = size.width - bounds.width -x;
                y = (size.height / 2) - (bounds.height / 2);
            }
            else if (position == ej.datavisualization.TreeMap.Position.BottomLeft) {
                y = size.height - bounds.height - y;
            }
            else if (position == ej.datavisualization.TreeMap.Position.BottomCenter) {
                x = (size.width / 2) - (bounds.width / 2);
                y = size.height - bounds.height-y;
            }
            else if (position == ej.datavisualization.TreeMap.Position.BottomRight) {
                x = size.width - bounds.width-x;
                y = size.height - bounds.height-y;
            }
            return {x:size.left+x,y:size.top+y};
        },
       
        _generateTreeMapItems: function (Items, fill, path) {
            this._rootTreeMapItems= Items;
            this._getTreeMapItemTemplate(Items);
            if (Items != null) {
                if (!this._isLevelColors){
                    this._setColorMappings(Items, this.model);
                }
                for (var i = 0; i < Items.length; i++) {
                    var item = Items[i];                    
                                      
                    var rect = item.rectangle;
                    if (item.Data != null && item.Data.length>0) {
                        item.Data = item.Data[0];
                    }
                    item.Rectangle = rect;
                    if(item.backgroundColor!=null)
                        $(rect).css("background-color",item.backgroundColor);
                    else
                        $(rect).css("background-color","#E5E5E5");
                    if(item.backgroundOpacity !=null)
                    {
                        $(rect).css("opacity",item.backgroundOpacity);
                    }
                    this._wireEventForTreeMapItem(item, rect);
                 
                }
                if (this.model.rangeColorMapping.length == 0 && this.uniColor() == null && this._color() != "") {
                    this._setDesaturationColor(Items, this.model.desaturationColorMapping);
                }
                else if (this.model.PaletteColorMapping != null) {
                    this._setPaletteColor(Items, this.model.PaletteColorMapping);
                }
            }
        },

        _wireEventForTreeMapItem: function (item,rect) {
            $(rect).mouseenter({ Param1: item, Param2: this }, this._rectMouseEnter);
            $(rect).mousemove({ Param1: item, Param2: this }, this._rectMouseMove);
            $(rect).mouseleave({ treeMap: this }, this._rectMouseLeave);
        },

        _getTreeMapItemTemplate: function (Items) {            
            
            var rects ="";
            for (var i = 0; i < Items.length; i++) {
                var item = Items[i];
                if (this.model.leafItemSettings.itemTemplate == null)
                    rects += '<div style="border-style:solid;position:absolute;left:' + item.LeftPosition + 'px;top:' + item.TopPosition + 'px;height:' + item.ItemHeight + 'px;width:' + item.ItemWidth + 'px;border-width:' + this.borderThickness() + 'px;border-color:' + this.borderBrush() + '"></div>';
                else {
                    var OriginalNode = $("#" + this.model.leafItemSettings.itemTemplate);
                    if (item.Data != null && item.Data.length == 1) {
                        item.Data = item.Data[0];
                    }
                    var tmpl2 = $.templates(OriginalNode.html());
                    var htmlString = "<div style='overflow:hidden;position:absolute;left:" + item.LeftPosition + "px;top:" + item.TopPosition + "px;height:" + item.ItemHeight + "px;width:" + item.ItemWidth + "px'>" + tmpl2.render(item) + "</div>";
                    rects += htmlString;
                }
            }
            if (this._svgDocument != null && this.enableDrillDown()) {
                this._svgDocument[0].innerHTML = "";
            }
            this._svgDocument = $('<div style= "overflow:hidden;' +
                              'z-index:1;"' +
                             'id="svgDocument">' + rects + '</div>');
            this._backgroundTile.appendTo(this.element);
            this._svgDocument.appendTo(this.element);           
            this._svgDocument.css({
                "height": this._height + "px", "width": this._width + "px", "margin-top": this._margintop + "px", "margin-left": this._marginleft, "overflow": "hidden",
                "z-index": 1, "position": "absolute", "left": "0", "top": "0"
            });
            $(this._backgroundTile).css({
                "height": this._height + "px", "width": this._width + "px", "margin-top": this._margintop + "px", "margin-left": this._marginleft, "overflow": "hidden",
                "z-index": 0, "position": "absolute", "left": "0", "top": "0"
            });
            this._templateDiv.appendTo(this.element);
            /* Below event hook added for IE10 tooltip issue*/
            $(this._templateDiv).mousemove({ Param2: this }, this._rectMouseMove);            
            this._off($(this.element), ej.eventType.mouseDown, this.rectClick);
            this._on($(this.element), ej.eventType.mouseDown, { treemap: this, Param1: Items }, this.rectClick);
            $(this.element).mouseup({ treeMap: this }, this._docClickFunction);
            this._on($(this.element), ej.eventType.mouseDown, { treemap: this }, this.dragDown);
            this._on($(this.element), ej.eventType.mouseMove, { treemap: this }, this.dragMove);
            this._on($(this.element), ej.eventType.mouseUp, { treemap: this }, this.dragUp);

            if (this.enableDrillDown()) {
                this._drillHeaderDiv.appendTo(this.element);
                this._drillHoverDiv.appendTo(this.element);
            }
            for (var i = 0; i<Items.length; i++) {
                Items[i].rectangle = this._svgDocument[0].childNodes[i];
            }
        },	       

        selectItem: function (obj) {
            if (this._rootTreeMapItems != null) {
                for (var i = 0; i < this._rootTreeMapItems.length; i++) {
                    var item = this._rootTreeMapItems[i];
                    var rect = item.rectangle;
                    if (item.Data != null && item.Data.length > 0) {
                        item.Data = item.Data[0];
                    }
                   
                    if (obj == item.Data) {
                        if (this.highlightOnSelection()) {

                            $(rect).css({ "border-width": this.highlightBorderThickness(), "border-color": this.highlightBorderBrush() });
                            if (this._browser != "msie") {
                                var lastItem = this._svgDocument[0].children[this._svgDocument[0].children.length - 1];
                                this._svgDocument[0].insertBefore(rect, lastItem);
                            }
                           
                                this._prevSelectedItems.push(rect);
                               
                        }
                        }
                    
                }
            }

        },
      
        _selectTreemapItem: function (obj) {

            if (this._rootTreeMapItems != null) {
                for (var i = 0; i < this._rootTreeMapItems.length; i++) {
                    var item = this._rootTreeMapItems[i];
                    var rect = item.rectangle;
                    if (item.Data != null && item.Data.length > 0) {
                        item.Data = item.Data[0];
                    }
                    if (obj == item.Data && !this._contains(this.selectedItems, obj)) {

                        if (this.highlightOnSelection()) {
                            $(rect).css({ "border-width": this.highlightBorderThickness(), "border-color": this.highlightBorderBrush() });
                            if (this._browser != "msie") {
                                var lastItem = this._svgDocument[0].children[this._svgDocument[0].children.length - 1];
                                this._svgDocument[0].insertBefore(rect, lastItem);
                            }
                            this._prevSelectedItems = [];
                            this.selectedItems = [];
                            this._prevSelectedItems.push(rect);
                            this.selectedItems.push(obj);
                        }
                    }

                }
            }
        },
        
        _selectItemResize: function () {

            for (var i = 0; i < this._rootTreeMapItems.length; i++) {
                var item = this._rootTreeMapItems[i];
                var rect = item.rectangle;
                if (item.Data != null && item.Data.length > 0) {
                    item.Data = item.Data[0];
                }
                if (this._contains(this.selectedItems, item.Data)) {
                    this.selectItem(item.Data);
                }
                
            }
        },
		
        selectItemByPath: function (path, value, seperator) {
            var propertyPath = path.split(seperator);
            var propertyValue = value.split(seperator);
            var path, value;
            var treemapItems = this._rootTreeMapItems;
            for (var i = 0; i < propertyPath.length; i++)
            {
                var path = propertyPath[i];
                var value = propertyValue[i];
                var itemsCollection = [];
                for (var k = 0; k < treemapItems.length; k++) {
                    var item = treemapItems[k];
                    var dataObj = item.Data;
                    var objValue;
                    for (var j = 0; j < propertyPath.length; j++) {
                        objValue = this._reflection(dataObj, path);
                    }

                    if (value == objValue) {
                        itemsCollection.push(item);
                    }
                }
                treemapItems = itemsCollection;
            }
            for (var i = 0; i < treemapItems.length; i++)
            {
                var item = treemapItems[i];
                this.selectItem(item.Data);
            }
        },

        _contains: function (array, actualobj) {
            var length = array.length;
            if (length > 0) {
                while (length--) {
                    if (array[length] === actualobj) {
                        return true;
                    }
                }
            }
            return false;
        },
       
        _rectMouseEnter: function(event) {
            var tooltipObject = event.data.Param1.Data;
            if (tooltipObject != null) {
                var element = event.data.Param2._toolTipElement;
                var template = event.data.Param2.model.tooltipTemplate;
                if (element != null && template != null) {
                    $(element).css({ "left": event.pageX+10, "top": event.pageY+10, "display": "block" });
                    var htmlString = $("#" + template).render(tooltipObject);
                    $(element).html(htmlString);
                    var height= element[0]!=null ? element[0].clientHeight : element.clientHeight;
                    var width= element[0]!=null ? element[0].clientWidth : element.clientWidth;
                    event.data.Param2._tooltipSize = { height: height, width: width };
                    
                }
            }
        },
       
        _rectMouseMove: function (event) {
            var treeMap = event.data.Param2;
            var element = treeMap._toolTipElement;
            var canExecute = true;
            /* Below If condition code was added for IE10 Tooltip issue*/
            if (event.data.Param1 == undefined) {
                if (element != null) {
                    var tooltipObject;
                    for (var i = 0; i < treeMap._rootTreeMapItems.length; i++) {
                        var item = treeMap._rootTreeMapItems[i];
                        if (item.rectangle.offsetLeft < event.offsetX && item.rectangle.offsetLeft + item.rectangle.offsetWidth > event.offsetX
                            && item.rectangle.offsetTop < event.offsetY && item.rectangle.offsetTop + item.rectangle.offsetHeight > event.offsetY) {
                            tooltipObject = item.Data;
                            break;
                        }
                    }
                    if (tooltipObject != null) {
                        var template = event.data.Param2.model.tooltipTemplate;
                        if (element != null && template != null) {
                            $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
                            var htmlString = $("#" + template).render(tooltipObject);
                            $(element).html(htmlString);
                            var height = element[0] != null ? element[0].clientHeight : element.clientHeight;
                            var width = element[0] != null ? element[0].clientWidth : element.clientWidth;
                            treeMap._tooltipSize = { height: height, width: width };

                        }
                    }
                    else {
                        if (element != null) {
                            canExecute = false;
                            $(element).css("display", "none");
                        }
                    }
                }
            }

            if (canExecute) {
                var tooltipSize = treeMap._tooltipSize;
                if (tooltipSize.width + event.pageX >= treeMap._width) {
                    event.pageX -= tooltipSize.width;
                }
                if (tooltipSize.height + event.pageY >= treeMap._height) {
                    event.pageY -= tooltipSize.height;
                }
                if (treeMap.enableDrillDown() && treeMap._browser != 'msie') {
                    for (var i = 0; i < treeMap._drillHoverDiv[0].children.length; i++) {
                        var child = treeMap._drillHoverDiv[0].children[i];
                        if (child.className == event.data.Param1.ParentHeader) {
                            $(child).css({ "display": "block" });
                        }
                    }
                }
                if (element != null) {
                    $(element).css({ "left": event.pageX + 10, "top": event.pageY + 10, "display": "block" });
                }
            }
        },

        _rectMouseLeave: function (event) {            
            var element = event.data.treeMap._toolTipElement;
            if (element != null) {
                $(element).css("display", "none");
            }
        },
		
        _docClickFunction: function(event) {
            var element = event.data.treeMap._toolTipElement;
            if (element != null) {
                $(element).delay(200).queue( function(next){ 
                    $(this).css('display','none'); 
                    next();
                });
            }
        },
       
        _setColorMappings: function(Items, colorMappings){
            for (var i = 0; i < Items.length; i++) {			
                if ((colorMappings.rangeColorMapping != null && colorMappings.rangeColorMapping.length > 0) || (colorMappings.uniColorMapping != null &&(colorMappings.uniColorMapping.color != null || this.uniColor() !=null ))) {
                    var ColorValue = Items[i]._generateColorMappings(colorMappings,this);
                    Items[i].backgroundColor = ColorValue;
                }
                else if (colorMappings.desaturationColorMapping != null && (colorMappings.desaturationColorMapping.color != "" || this._color() != "")){
                    this._setDesaturationColor(Items, colorMappings.desaturationColorMapping);
                }
                else if (colorMappings.paletteColorMapping != null){
                    this._setPaletteColor(Items, colorMappings.paletteColorMapping);
                }
            }
        },
     
        _setDesaturationColor: function (Items, desaturationColorMapping) {
            Items = Items.sort(this._orderBycolorWeight);

            var from = (typeof desaturationColorMapping.from == "number") ? desaturationColorMapping.from : this._from();
            var to = (typeof desaturationColorMapping.to == "number") ? desaturationColorMapping.to : this._to();
            var rangeMin = (typeof desaturationColorMapping.rangeMinimum == "number") ? desaturationColorMapping.rangeMinimum : this._rangeMinimum();
            var rangeMax = (typeof desaturationColorMapping.rangeMaximum == "number") ? desaturationColorMapping.rangeMaximum : this._rangeMaximum();
            if (from < 0 || from > 1)
                from = 1;
            if (to > 1 || to < 0)
                to = 0;

            var interval = (from - to) / Items.length;
            for (var i = 0; i < Items.length; i++) {
                Items[i].backgroundColor = (typeof desaturationColorMapping.color == "string") ? desaturationColorMapping.color : this._color();              
                if (Items[i].colorWeight >= rangeMin && Items[i].colorWeight <= rangeMax) {
                    Items[i].backgroundOpacity= from;
                    from = from - interval;
                }
            }
        },

        _setPaletteColor: function (Items, PaletteColors) {
            Items = Items.sort(this._orderBycolorWeight);
            for (var i = 0; i < Items.length; i++) {
                if (PaletteColors.colors != null && PaletteColors.colors.length > 0) {
                    Items[i].backgroundColor= PaletteColors.colors[i % (PaletteColors.colors.length)];
                } 
            }
        },

        _orderBycolorWeight: function (a, b) {
            if (a.colorWeight == b.colorWeight) {
                return 0;
            } else if (a.colorWeight < b.colorWeight) {
                return 1;
            }
            return -1;
        },
       
        _orderByAreaWight: function (a, b) {
            if (a.AreaByWeight == b.AreaByWeight) {
                return 0;
            } else if (a.AreaByWeight < b.AreaByWeight) {
                return 1;
            }
            return -1;
        },

        _calculateSliceAndDiceItemSize: function (Items, xPos, yPos, width, height, gap, headerHeight, isHorizontal,legendHeight) {

            if (gap == "") { gap = 0;}
            var Gap = gap;
            var totalWeight = this._getTotalWeight(Items);
            if (isHorizontal == undefined) {
                isHorizontal = width > height ? true : false;
            }
            var AvailableSize = { "Width": width, "Height": height-legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var parentArea = AvailableSize.Height * AvailableSize.Width;
            var itemsCount = Items.length;
            if (isHorizontal) {
                var parentHeight = AvailableSize.Height;
                var allottedWidth = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childWidth = childarea / parentHeight;
                    gap = (childWidth > Gap) ? Gap : 0;

                    if (allottedWidth <= AvailableSize.Width) {
                        Items[i].ItemWidth = (i != itemsCount - 1) ? childWidth - parseFloat(gap) : childWidth;
                        Items[i].ItemHeight = parentHeight;
                        Items[i].LeftPosition = allottedWidth + xPos;
                        Items[i].TopPosition = yPos + headerHeight;
                        allottedWidth += childWidth;
                    }
                }
            }
            else {
                var parentWidth = AvailableSize.Width;
                var allottedHeight = 0;
                for (var i = 0; i < itemsCount; i++) {
                    var childarea = (parentArea / totalWeight) * Items[i].weight;
                    var childHeight = childarea / parentWidth;
                    gap = (childHeight > Gap) ? Gap : 0;
                    if (allottedHeight <= AvailableSize.Height) {
                        Items[i].ItemWidth = parentWidth;
                        Items[i].ItemHeight = (i != itemsCount - 1) ? childHeight - parseFloat(gap) : childHeight;
                        Items[i].LeftPosition = xPos;
                        Items[i].TopPosition = allottedHeight + yPos + headerSize;
                        allottedHeight += childHeight;
                    }
                }
            }
        },

        _calculateSquarifiedItemSize: function (Items, xPos, yPos, width, height, Gap, headerHeight, legendHeight) {

            if (Gap == "") { Gap = 0; }            
            var totalweight = this._getTotalWeight(Items);
            var AvailableSize = { "Width": width, "Height": height-legendHeight };
            var headerSize = headerHeight < AvailableSize.Height ? headerHeight : 0;
            AvailableSize.Height = AvailableSize.Height - headerSize;
            for (var i = Items.length - 1; i >= 0; i--) {
                var item = Items[i];
                item["AreaByWeight"] = (AvailableSize.Height * AvailableSize.Width) * item.weight / totalweight;
                item.headerTopPosition = yPos;
            }
            var AvailableArea = { "X": xPos, "Y": yPos + headerSize, "Width": AvailableSize.Width, "Height": AvailableSize.Height };
            var OrderedItems = Items.sort(this._orderByAreaWight);
            var curX = 0, curY = 0;
            var j = 0;
            for (var i = 0; i < Items.length; i = j) {
                var firstTreemapItem = OrderedItems[i];
                weightObject = this._getGroupTotalWeight(firstTreemapItem, OrderedItems, AvailableArea, i);
                var GroupTotalWeight = weightObject.totalWeight;
                j = weightObject.index;
                var currentRect = {};
                var isHorizontal = (AvailableArea.Width > AvailableArea.Height) ? true : false;
                for (var k = i; k < j; k++) {
                    var item = OrderedItems[k];
                    var areaSum = GroupTotalWeight;
                    if (k == i) {
                        currentRect.X = AvailableArea.X;
                        currentRect.Y = AvailableArea.Y;
                        if (isHorizontal) {
                            currentRect.Width = areaSum / AvailableArea.Height;
                            currentRect.Height = AvailableArea.Height;
                            AvailableArea.X += currentRect.Width;
                            AvailableArea.Width = Math.max(0, AvailableArea.Width - currentRect.Width);
                        }
                        else {
                            currentRect.Width = AvailableArea.Width;
                            currentRect.Height = areaSum / AvailableArea.Width;
                            AvailableArea.Y += currentRect.Height;
                            AvailableArea.Height = Math.max(0, AvailableArea.Height - currentRect.Height);
                        }
                        curX = currentRect.X;
                        curY = currentRect.Y;
                    }
                    var rect = {};
                    if (OrderedItems.indexOf(item) != OrderedItems.length - 1) {
                        rect.X = 0;
                        rect.Y = 0;
                        rect.Width = (isHorizontal) ? currentRect.Width - parseFloat(Gap) : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height - parseFloat(Gap);
                        if (j - k != 1) {
                            if (isHorizontal) {
                                rect.Height -= parseFloat(Gap);
                            }
                            else {
                                rect.Width -= parseFloat(Gap);
                            }
                        }
                    }
                    else {
                        rect.Width = (isHorizontal) ? currentRect.Width : item.AreaByWeight / currentRect.Height;
                        rect.Height = (isHorizontal) ? item.AreaByWeight / currentRect.Width : currentRect.Height;
                    }
                    item.ItemWidth = rect.Width;
                    item.ItemHeight = rect.Height;
                    item.LeftPosition = curX;
                    item.TopPosition = curY;
                    if (isHorizontal) {
                        curY += (j - k != 1) ? rect.Height + parseFloat(Gap) : rect.Height;
                    }
                    else {
                        curX += (j - k != 1) ? rect.Width + parseFloat(Gap) : rect.Width;
                    }
                }
            }
        },
        
        _getGroupTotalWeight: function (firstTreemapItem, OrderedItems, AvailableArea, j) {
            var GroupTotalWeight = 0;
            var GroupMaxAspectRatio = 0;
            for (; j < OrderedItems.length; j++) {
                var ShorterSideLength = this._getShortersideLength(AvailableArea.Width, AvailableArea.Height);
                var lastTreemapItem = OrderedItems[j];
                GroupTotalWeight += lastTreemapItem.AreaByWeight;
                var GroupWidth = GroupTotalWeight / ShorterSideLength;
                var firstitemheight = firstTreemapItem.AreaByWeight / GroupWidth;
                var lastitemheight = lastTreemapItem.AreaByWeight / GroupWidth;
                if (j == 0)
                    GroupMaxAspectRatio = this._aspectRatio(GroupWidth, ShorterSideLength);
                var TempAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                if (GroupTotalWeight == lastTreemapItem.AreaByWeight || TempAspectRatio < GroupMaxAspectRatio) {
                    GroupMaxAspectRatio = TempAspectRatio;
                }
                else {
                    GroupTotalWeight -= lastTreemapItem.AreaByWeight;
                    GroupWidth = GroupTotalWeight / ShorterSideLength;
                    GroupMaxAspectRatio = Math.max(this._aspectRatio(firstitemheight, GroupWidth), this._aspectRatio(lastitemheight, GroupWidth));
                    break;
                }
            }
            return { totalWeight: GroupTotalWeight, index: j };
        },
       
        _aspectRatio: function (x, y) {
            return (x > y) ? (x / y) : (y / x);
        },

        _getShortersideLength: function (width, height) {
            return width > height ? height : width;
        },

        _getTotalWeight: function (Items) {
            var total = 0;
            for (var i = 0; i < Items.length; i++) {
                total += parseFloat(Items[i].weight);
            }
            return total;

        },

        _getGroupitem: function (path, dataSource, headerHeight) {
            var obj = [];            
            if (dataSource != null && this.weightValuePath() != null) {                
                for (var i in dataSource) {
                    var item = dataSource[i];
                    if (item != null && item.hasOwnProperty(path) ? true : this._containsProperty(item,path)) {
                        var itemValue = this._reflection(item, path);
                        if ($.inArray(itemValue, obj) == -1) {
                            obj.push(itemValue);
                        }
                    }
                }                
                var objCollection = [];
                for (var i = 0; i < obj.length; i++) {
                    var Coll = {};
                    var groupObj = [];
                    var groupId = obj[i];
                    var colorWeight = 0;
                    var weight = 0;
                    var count = 0;
                    for (var j = 0; j < dataSource.length;j++) {
                        var item = dataSource[j];
                        var itemValue = this._reflection(item, path);
                        if (itemValue == groupId) {
                            var weightValue = this._reflection(item, this.weightValuePath());
                            if (weightValue < 0) {
                                weightValue = -1 * weightValue;
                            }
                            var colorWeightValue = this._reflection(item, this.colorValuePath());
                            groupObj.push(item);
                            count++;
                            if (weightValue != null) {
                                weight += parseFloat(weightValue);
                            }
                            if (colorWeightValue != null) {
                                colorWeight += parseFloat(colorWeightValue);
                            }
                        }
                    }
                    
                    Coll["header"] = groupId;
                    Coll["data"] = groupObj;
                    Coll["weight"] = weight;
                    Coll["colorWeight"] = colorWeight;
                    Coll["headerHeight"] = headerHeight;
                    if (weight > 0) {
                        var newTreemapItem = new treeMapItem(Coll);
                        objCollection.push(newTreemapItem);
                    }
                }
                return objCollection;
            }
            else { return null; }
        },
      
        _getTopGroupitem: function (path, dataSource) {            
            if (path == null) path = this.labelPath();
            if (path == null) path = this.weightValuePath();
            var obj = [];
            var Coll = {};
            var weight = 0, colorWeight = 0;
            var groupId="";
            var innerobjCollection = [];
            var itemColl = [];
            for (var j = 0; j < dataSource.length; j++) {
                var item = dataSource[j];
                var itemValue = this._reflection(item, path);
                var innerColl={};
                var weightValue = this._reflection(item, this.weightValuePath());
                if (weightValue < 0) {
                    weightValue = -1 * weightValue;
                }
                var colorWeightValue = this._reflection(item, this.colorValuePath());      
                if (weightValue != null) {
                    weight += parseFloat(weightValue);
                }
                if (colorWeightValue != null) {
                    colorWeight += parseFloat(colorWeightValue);
                }
                itemColl.push(item);
                innerColl["weight"] = weightValue;
                innerColl["data"] = item;                
                innerColl["header"] = itemValue;
                innerColl["colorWeight"] = colorWeightValue;                                       
                var newinnerTreemapItem = new treeMapItem(innerColl);
                innerobjCollection.push(newinnerTreemapItem);
            }        
            Coll["header"] = groupId;
            Coll["data"] = itemColl;
            Coll["weight"] = weight;
            Coll["innerItem"] = innerobjCollection;
            Coll["colorWeight"] = colorWeight;                                   
            var treemapColl = [];
            var treeitem = new treeMapItem(Coll);
            treemapColl.push(treeitem);
            return  treemapColl;
        },         
       
        _containsProperty: function (object, propertyName) {
            for (var key in object)
            {
                if(key == propertyName)
                {
                    return true;
                }
            }
            return false;
        },

        _reflection: function (Source, Path) {
            var ShapeValues = Source;
            if (Path != null && Source != null) {
                var parts = Path.split(".");
                parts.push(Path);
                var i = 0;
                for (; i < parts.length; i++) {
                    if (ShapeValues != null) {
                        var hasProperty = ShapeValues.hasOwnProperty(parts[i]) ? true : this._containsProperty(ShapeValues, parts[i]);
                        if (hasProperty) {
                            $.each(ShapeValues, function (prop, propval) {
                                if (prop == parts[i]) {
                                    ShapeValues = propval;
                                    return false;
                                }
                            });
                        }
                    }
                }
                return ShapeValues;
            }
            else {
                return null;
            }

        }

    });

    var treeMapLevel = function () {        
        this.groupBackground = null;       
        this.groupBorderColor = null;
        this.groupBorderThickness = 1;       
        this.groupPadding = 4;
        this.groupPath = null;
        this.groupGap = 1;       
        this.headerHeight = 20;
        this.showHeader = true;       
        this.showLabels = false;
        this.headerTemplate = null;       
        this.labelTemplate = null;
        this.labelPosition = "center";
        labelVisibilityMode = "visible";      
        headerVisibilityMode = "visible";

        this.treeMapItems = [];
    };

    treeMapLevel.prototype = {

    };
	
    var TreeMapGroupColorMapping = function () {	
        this.groupID = null;		       
        this.rangeColorMapping = [];

        this.desaturationColorMapping = { 				            
            from:0, 				            
            to:0,			
            color: "",								            
            rangeMinimum:0,		
            rangeMaximum:0
        };  
       
        this.uniColorMapping = {             
            color: null 
        };
     
        this.paletteColorMapping = {           
            colors: [],
        };
    };		
	
    var TreeMapRangeColorMapping = function () {
        
        this.from = -1;		
        this.to = -1;		    
        this.legendlabel = null;		      
        this.color = null;
    };
	
    var treeMapItem = function (params) {
        this.weight = params.weight;
        this.colorWeight = params.colorWeight;
        this.Data = params.data;
        this.headerHeight = params.headerHeight;
        this.header = params.header;
        this.label = params.header;
        this.headerLeftPosition = 0;
        this.headerTopPosition = 0;
        this.innerItems = params.innerItem;
        this.headerWidth = 0;
        this.headerTemplate = null;
    };

    treeMapItem.prototype = {

        _generateColorMappings: function (colorMappings,treeMap) {
            if (colorMappings.rangeColorMapping != null && colorMappings.rangeColorMapping.length > 0) {
                return this._getRangeBrushColor(colorMappings.rangeColorMapping);
            }
            if (treeMap.uniColor() != null && colorMappings.uniColorMapping!=null) {
                return treeMap.uniColor();
            }
            else if (colorMappings.uniColorMapping.color != null && colorMappings.uniColorMapping != null) {
                return this._getUniColor(colorMappings.uniColorMapping);
            }
            
        },

        _getUniColor: function (uniColorMapping) {
            return uniColorMapping.color;
        },
      
        _getRangeBrushColor: function (rangeColorMapping) {
            for (var j = 0; j < rangeColorMapping.length; j++) {
                var rangeBrush = rangeColorMapping[j];
                if (this.colorWeight >= rangeBrush.from && this.colorWeight <= rangeBrush.to) {
                    return rangeBrush.color;
                }
            }
        }
    };
	
	ej.datavisualization.TreeMap.selectionMode = {
		Default: "default",
		Multiple: "multiple"
	};
	
	ej.datavisualization.TreeMap.groupSelectionMode = {
		Default: "default",
		Multiple: "multiple"
	};
	
    ej.datavisualization.TreeMap.ItemsLayoutMode = {        
        Squarified: "squarified",        
        SliceAndDiceHorizontal: "sliceanddicehorizontal",        
        SliceAndDiceVertical: "sliceanddicevertical",        
        SliceAndDiceAuto: "sliceanddiceauto"
    };  

    ej.datavisualization.TreeMap.DockPosition = {        
        Top: "top",        
        Bottom: "bottom",        
        Right: "right",        
        Left: "left"
    };

    ej.datavisualization.TreeMap.Position = {        
        TopLeft: "topleft",        
        TopCenter: "topcenter",        
        TopRight: "topright",        
        CenterLeft: "centerleft",        
        Center: "center",        
        CenterRight: "centerright",        
        BottomLeft: "bottomleft",        
        BottomCenter: "bottomcenter",        
        BottomRight:"bottomright"
    };
    ej.datavisualization.TreeMap.Alignment = {       
        Near: "near",        
        Center: "center",        
        Far: "far"
    }
    ej.datavisualization.TreeMap.VisibilityMode = {        
        Visible:"visible",        
        HideOnExceededLength:"hideonexceededlength"
    };
})(jQuery, Syncfusion);
;
/**
* @fileOverview Plugin to style the Barcode elements
* @copyright Copyright Syncfusion Inc. 2001 - 2014. All rights reserved.
* Use of this code is subject to the terms of our license.
* A copy of the current license can be obtained at any time by e-mailing
* licensing@syncfusion.com. Any infringement will be prosecuted under
* applicable laws.
* @version 12.1
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {
    /** ejBarcode is the plugin name.*/
    /** "ej.datavisualization.Barcode" is "namespace.className" will hold functions and properties.*/
   
    ej.widget("ejBarcode", "ej.datavisualization.Barcode", {
        /** Widget element will be automatically set in this*/
        element: {
            target: null
        },

        /** User defined model will be automatically set in this*/
        model: null,
        validTags: ["div", "span"],
        _rootCSS: "e-barcode",

        defaults: {
            displayText: true,
            text: "",
            symbologyType: "qrbarcode",
            textColor: 'black',
            lightBarColor: 'white',
            darkBarColor: 'black',
            quietZone: {
                left: 1,
                top: 1,
                right: 1,
                bottom: 1,
                all: 1
            },
            narrowBarWidth: 1,
            wideBarWidth: 3,
            barHeight: 150,
            barcodeToTextGapHeight: 10,
            xDimension: 4,
            encodeStartStopSymbol: true,
            load: null,
            enabled: true,
        },

        /**
         * Specify the data types for default properties 
         * @private
         */
        dataTypes: {
            displayText: "boolean",
            text: "string",
            enabled: "boolean",
            symbologyType: "enum",
            narrowBarWidth: "number",
            wideBarWidth: "number",
            barHeight: "number",
            encodeStartStopSymbol: "boolean"
        },

        /**
         * @private
         */
        _defaultScaleValues: function () {
            return {
                enabled: true,
                displayText: true,
                text: "",
                symbologyType: "qrbarcode",
                textColor: 'black',
                lightBarColor: 'white',
                darkBarColor: 'black',
                quietZone: { left: 1, top: 1, right: 1, bottom: 1 },
                narrowBarWidth: 1,
                wideBarWidth: 3,
                barHeight: 150,
                barcodeToTextGapHeight: 10,
                xDimension: 4,
                encodeStartStopSymbol: true
            };
        },

        /**
         * @constructor
         * @private
         */
        _init: function () {
            this._initialize();
            this._renderBarcode();
        },

        /**
         * @private
         */
        _destroy: function () {
            this.element.empty();
        },

        /**
        * To configure the properties at runtime using SetModel		
        * @private
        */
        _setModel: function (options) {
            var option;
            for (option in options) {
                switch (option) {
                    case "enabled": this._disabled(!options[option]); break;
                    case "height": this.model.height = options[option]; break;
                    case "width": this.model.width = options[option]; break;
                    case "lightBarColor": this.model.lightBarColor = options[option]; break;
                    case "darkBarColor": this.model.darkBarColor = options[option]; break;
                    case "textColor": this.model.textColor = options[option]; break;
                    case "displayText": this.model.displayText = options[option]; break;
                    case "quietZone": this.model.quietZone = options[option]; break;
                    case "narrowBarWidth": this.model.narrowBarWidth = options[option]; break;
                    case "wideBarWidth": this.model.wideBarWidth = options[option]; break;
                    case "barHeight": this.model.barHeight = options[option]; break;
                    case "xDimension": this.model.xDimension = options[option]; break;
                    case "encodeStartStopSymbol": this.model.encodeStartStopSymbol = options[option]; break;
                    case "symbologyType":
                        this.model.symbologyType = options[option];
                        this._itemInitialize();
                        break;
                    case "text":
                        this.model.text = options[option];
                        this._itemInitialize();
                        break;
                }
            }
            this._renderBarcode();
        },

        /**
         * Initializes control.
         * @private
         */
        _initialize: function () {
            this.BarcodeEl = this.element;
            this.target = this.element[0];
            this._itemInitialize();
            this.Model = this.model;
        },

        /**
         * Renders barcode.
         * @private
         */
        _renderBarcode: function () {
            this.initialize();
        },

        /**
         * @private
         */
        _itemInitialize: function () {
            var proxy = this;
            if (this.model.scales != null) {
                $.each(this.model.scales, function (index, element) {
                    element = proxy._checkArrayObject(element, index);
                    var obj = proxy._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                });
            }
            else {
                this.model.scales = [this._defaultScaleValues()];
            }
        },

        /**
         * @private
         */
        _checkArrayObject: function (element, initialName) {
            var proxy = this;
            $.each(element, function (name, innerElement) {
                if (innerElement instanceof Array) {
                    innerElement = proxy._checkArrayObject(innerElement, name);
                }
                else if (innerElement != null && typeof innerElement == "object") {
                    var allObjects = proxy._defaultScaleValues();
                    innerElement = proxy._LoadIndividualDefaultValues(innerElement, allObjects, (typeof name === "number") ? initialName : name);
                }
            });
            return element;
        },

        /**
         * @private
         */
        _LoadIndividualDefaultValues: function (obj, allObjects, name) {
            var defaultObj = null;
            var proxy = this;
            $.each(allObjects, function (n, element) {
                if (name == n) {
                    defaultObj = element;
                    return;
                }
            });
            if (defaultObj instanceof Array) defaultObj = defaultObj[0];

            $.each(obj, function (objName, Ele) {
                if (Ele instanceof Array) {
                    Ele = proxy._checkArrayObject(Ele, name);
                }
                else if (Ele != null && typeof Ele == "object") {
                    Ele = proxy._LoadIndividualDefaultValues(Ele, defaultObj, (typeof objName === "number") ? name : objName);
                }
            });

            $.extend(defaultObj, obj);
            $.extend(obj, defaultObj);
            return obj;
        },

        /**
         * Initializes the control.
         * @private
         */
        initialize: function () {
            if (!this.model.enabled)
                return;

            this._initObject(this);

            if (this.Model.text != null)
                this._findBarcodeType();
        },

        /**
        * To enable the barcode 
        */
        enable: function () {
            if (!this.model.enabled) {
                this.element.removeClass("e-disable");
                this.model.enabled = true;
            }
        },

        /**
        * To disable the barcode  
        */
        disable: function () {
            if (this.model.enabled) {
                this.element.addClass("e-disable");
                this.model.enabled = false;
            }
        },

        /**
         * @private
         */
        _disabled: function (boolean) {
            if (boolean) this.disable();
            else this.enable();
        },

        /**
         * Initializes rendering canvas.
         * @private
         */
        _initObject: function (element) {
            element.BarcodeEl = $("#" + element.target.id);
            if (element.canvasEl)
                element.canvasEl.remove();
            else
                element.canvasEl = $("<canvas></canvas>");
            element.BarcodeEl.append(element.canvasEl);
            element.BarcodeEl.css("width", element.model.width);
            element.BarcodeEl.css("height", element.model.height);
            element.canvasEl[0].setAttribute("width", element.BarcodeEl.width());
            element.canvasEl[0].setAttribute("height", element.BarcodeEl.height());
            element.centerX = element.canvasEl[0].width / 2;
            element.centerY = element.canvasEl[0].height / 2;
            var elem = element.canvasEl[0];
            if (typeof window.G_vmlCanvasManager != "undefined") {
                elem = window.G_vmlCanvasManager.initElement(elem);
            }
            if (!elem || !elem.getContext) {
                return;
            }
            element.contextEl = element.canvasEl[0].getContext("2d");

            if (element.contextEl == undefined) //IE 8
                return;
        },

        /**
         * Finds the barcode type and encodes text.
         * @private
         */
        _findBarcodeType: function () {
            if (this.contextEl == undefined) //IE 8
                return;

            if (this.Model.text == "")
                return;

            this.textFont = { size: "12px", fontFamily: "Segoe UI", fontStyle: "Regular" };

            if (this.model.quietZone.all > 1)
                this.model.quietZone.left = this.model.quietZone.right = this.model.quietZone.top = this.model.quietZone.bottom = this.model.quietZone.all;

            intercharacterGap = 1;
            extendedText = "";
            continuous = false;
            validatorExp = "";
            encodedText = this.Model.text;
            var startSymbol, stopSymbol, symbolTable, extendedCodes;

            var type = this.Model.symbologyType;
            var oneD = true;

            /** Apply symbology setting.*/
            if (ej.datavisualization.Barcode.SymbologyType.dataMatrix == type || ej.datavisualization.Barcode.SymbologyType.qrBarcode == type) {
                oneD = false;
            }
            else {
                startSymbol = ej.datavisualization.Barcode.symbologySettings[type].startSymbol;
                stopSymbol = ej.datavisualization.Barcode.symbologySettings[type].stopSymbol;
                validatorExp = ej.datavisualization.Barcode.symbologySettings[type].validatorExp;
                symbolTable = ej.datavisualization.Barcode.symbologySettings[type].symbolTable;
                extendedCodes = ej.datavisualization.Barcode.symbologySettings[type].extendedCodes;

                if (ej.datavisualization.Barcode.SymbologyType.code39Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
                else if (ej.datavisualization.Barcode.SymbologyType.code93Extended == type) {
                    var cArray = encodedText.split("");
                    var builder = "";
                    for (var j = 0; j < cArray.length; j++) {
                        var item = cArray[j];
                        for (var k = 0; k < extendedCodes.length; k++) {
                            if (extendedCodes[k][0] == item) {
                                var chArray = extendedCodes[k][1];
                                if (!(chArray == undefined)) {
                                    for (var i = 0; i < chArray.length; i++) {
                                        builder += chArray[i];
                                    }
                                }
                            }
                        }
                    }
                    extendedText = builder;
                }
            }

            /** Process one dimensional barcode.*/
            if (oneD) {
                if (!this._validateText(this.Model.text, validatorExp)) {
                    console.log("Barcode Text contains characters that are not accepted by this barcode specification.");
                    return;
                }

                if (this.model.symbologyType == "code32" && this.model.text.length != 8) {
                    console.log("Barcode Text Length that are not accepted by this barcode specification.");
                    return;
                }

                if (extendedText.length > 0)
                    encodedText = extendedText;

                if (this.Model.encodeStartStopSymbol)
                    encodedText = startSymbol.toString() + encodedText + stopSymbol.toString();

                var w = this._getCharWidth(encodedText, symbolTable);

                if (continuous == false)
                    w -= (intercharacterGap * encodedText.length);
                else {
                    w -= (extendedText.length > 0) ?
                        intercharacterGap * (extendedText.length - this.Model.text.length) :
                        intercharacterGap;
                }

                var textHeight = parseInt(this.Model.displayText ? this.textFont.size.replace('px', '') : 0);
                var h = this.Model.quietZone.top + this.Model.quietZone.bottom + this.Model.barHeight + intercharacterGap + (this.Model.displayText ? (textHeight + this.Model.barcodeToTextGapHeight) : 0);
                w += this.Model.quietZone.left + this.Model.quietZone.right;

                /** Prepare the canvas.*/
                this.Model.width = w;
                this.canvasEl[0].setAttribute("width", this.Model.width);
                this.BarcodeEl.css("width", this.Model.width);

                /** Draw one dimensional barcode.*/
                this._draw1DBarcode(encodedText, symbolTable);
            }
                /** Process two dimensional barcode.*/
            else {
                if (ej.datavisualization.Barcode.SymbologyType.dataMatrix == type)
                    this._buildDataMatrix();
                else if (ej.datavisualization.Barcode.SymbologyType.qrBarcode == type)
                    this._buildQRBarcode();
            }

            /** Raise load event*/
            this._raiseEvent("load");
        },

        /**
         * Load event.
         * @private
         * @event
         */
        _raiseEvent: function (event) {
            if (this.model[event])
                this._trigger(event);
        },

        /**
         * Generates QR barcode.
         * @private
         */
        _buildQRBarcode: function () {
            var txt = this.Model.text;
            this.m_EciAssignmentNumber = 3;
            this.mode = "numeric";
            this.isEci = false;
            this.m_Version = 1;
            var dataCapacityOfVersions = new Array(14, 26, 42, 62, 84, 106, 122, 152, 180, 213, 251, 287, 331, 362, 412, 450, 504, 560, 624, 666, 711, 779, 857, 911, 997, 1059, 1125, 1190, 1264, 1370, 1452, 1538, 1628, 1722, 1809, 1911, 1989, 2099, 2213, 2331);
            for (var i = 0; i < txt.length; i++) {
                var cCode = txt.charCodeAt(i);
                if (cCode < 58 && cCode > 47) {
                    /** numeric only*/
                }
                else if ((cCode < 91 && cCode > 64) || txt[i] == '$' || txt[i] == '%' || txt[i] == '*' || txt[i] == '+' || txt[i] == '-' || txt[i] == '.' || txt[i] == '/' || txt[i] == ':' || txt[i] == ' ') {
                    this.mode = "alphanumeric";
                }
                else if ((cCode >= 65377 && cCode <= 65439) || (cCode >= 97 && cCode <= 122)) {
                    this.mode = "binary";
                    break;
                }
                else {
                    this.mode = "binary";
                    this.isEci = true;
                    break;
                }
            }
            if (this.mode == "numeric")
                dataCapacityOfVersions = new Array(34, 63, 101, 149, 202, 255, 293, 365, 432, 513, 604, 691, 796, 871, 991, 1082, 1212, 1346, 1500, 1600, 1708, 1872, 2059, 2188, 2395, 2544, 2701, 2857, 3035, 3289, 3486, 3693, 3909, 4134, 4343, 4588, 4775, 5039, 5313, 5596);
            else if (this.mode == "alphanumeric")
                dataCapacityOfVersions = new Array(20, 38, 61, 90, 122, 154, 178, 221, 262, 311, 366, 419, 483, 528, 600, 656, 734, 816, 909, 970, 1035, 1134, 1248, 1326, 1451, 1542, 1637, 1732, 1839, 1994, 2113, 2238, 2369, 2506, 2632, 2780, 2894, 3054, 3220, 3391);

            if (this.isEci == true) {
                this.m_EciAssignmentNumber = this._findECINumber(txt);
            }
            for (var i = 0; i < dataCapacityOfVersions.length; i++) {
                if (dataCapacityOfVersions[i] > txt.length) {
                    this.m_Version = i + 1;
                    break;
                }
            }

            this.qrbarcodeValues = { Version: this.m_Version, ErrorCorrectionLevel: 1, NumberOfDataCodeWord: this._getNumberOfDataCodeWord(), NumberOfErrorCorrectingCodeWords: this._getNumberOfErrorCorrectingCodeWords(), NumberOfErrorCorrectionBlocks: this._getNumberOfErrorCorrectionBlocks(), End: this._getEnd(), DataCapacity: this._getDataCapacity(), FormatInformation: this._getFormatInformation(), VersionInformation: this._getVersionInformation() };
            this.noOfModules = (this.m_Version - 1) * 4 + 21;

            this.moduleValue = this._create2DArray(this.noOfModules, this.noOfModules);

            for (var i = 0; i < this.noOfModules; i++) {
                for (var j = 0; j < this.noOfModules; j++) {
                    this.moduleValue[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            this._drawPDP(0, 0);
            this._drawPDP(this.noOfModules - 7, 0);
            this._drawPDP(0, this.noOfModules - 7);
            this._drawTimingPattern();

            if (this.m_Version != 1) {
                var allignCoOrdinates = this._getAlignmentPatternCoOrdinates();

                for (var i = 0; i < allignCoOrdinates.length; i++) {
                    for (var j = 0; j < allignCoOrdinates.length; j++) {
                        var x = allignCoOrdinates[i];
                        var y = allignCoOrdinates[j];
                        if (this.moduleValue[x][y].IsPDP != true)
                            this._drawAlignmentPattern(x, y);
                    }
                }
            }

            this._allocateFormatVersionInfo();
            this._encodeData();
            this._drawFormatInformation();
            this._addQuietZone();

            var w = this.moduleValue.length * this.Model.xDimension;
            var h = this.moduleValue[0].length * this.Model.xDimension;

            /** Prepare canvas.*/
            this.Model.width = w;
            this.canvasEl[0].setAttribute("width", this.Model.width);
            this.BarcodeEl.css("width", this.Model.width);

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw QR barcode in canvas.*/
            this._drawQRBarcode(h);
        },

        /** 
         * Allocate format and version information.
         * @private
         */
        _allocateFormatVersionInfo: function () {
            for (var i = 0; i < 9; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            for (var i = this.noOfModules - 8; i < this.noOfModules; i++) {
                this.moduleValue[8][i].IsFilled = true;
                this.moduleValue[i][8].IsFilled = true;
            }
            if (this.m_Version > 6) {
                var versionInformation = this.qrbarcodeValues.VersionInformation;
                var count = 0;
                for (var i = 0; i < 6; i++) {
                    for (var j = 2; j >= 0; j--) {
                        this.moduleValue[i][this.noOfModules - 9 - j].IsBlack = versionInformation[count] == 1 ? true : false;
                        this.moduleValue[i][this.noOfModules - 9 - j].IsFilled = true;

                        this.moduleValue[this.noOfModules - 9 - j][i].IsBlack = versionInformation[count++] == 1 ? true : false;
                        this.moduleValue[this.noOfModules - 9 - j][i].IsFilled = true;
                    }
                }
            }
        },

        /**
         * @private
         */
        _drawTimingPattern: function () {
            for (var i = 8; i < this.noOfModules - 8; i += 2) {
                this.moduleValue[i][6].IsBlack = true;
                this.moduleValue[i][6].IsFilled = true;

                this.moduleValue[i + 1][6].IsBlack = false;
                this.moduleValue[i + 1][6].IsFilled = true;

                this.moduleValue[6][i].IsBlack = true;
                this.moduleValue[6][i].IsFilled = true;

                this.moduleValue[6][i + 1].IsBlack = false;
                this.moduleValue[6][i + 1].IsFilled = true;
            }
            this.moduleValue[this.noOfModules - 8][8].IsBlack = true;
            this.moduleValue[this.noOfModules - 8][8].IsFilled = true;
        },

        /** 
         * Draws QR barcode.
         * @private
         */
        _drawQRBarcode: function (ht) {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;

            var whiteBrush = 'white';
            var blackBrush = 'black';

            this.Model.height = ht;
            this.canvasEl[0].setAttribute("height", this.Model.height);
            this.BarcodeEl.css("height", this.Model.height);
            var dim = parseInt(this.Model.xDimension);

            var y = 0;

            var w = this.noOfModules + 2 * quietZone, h = this.noOfModules + 2 * quietZone;

            this.contextEl.beginPath();

            for (var i = 0; i < w; i++) {
                var x = 0;

                for (var j = 0; j < h; j++) {
                    var colorBrush = null;
                    if (this.moduleValue[i][j].IsBlack)
                        colorBrush = 'black';
                    else
                        colorBrush = 'white';

                    if (this.dataAllocationValues[j][i].IsFilled)
                        if (this.dataAllocationValues[j][i].IsBlack)
                            colorBrush = 'black';

                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)

                    x += dim;
                }

                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },

        /** 
         * Adds quiet zone to the barcode.
         * @private
         */
        _addQuietZone: function () {
            var quietZone = this.Model.quietZone.all;
            if (quietZone < 2)
                quietZone = 2;
            var w = this.noOfModules + 2 * quietZone;
            var h = this.noOfModules + 2 * quietZone;
            var tempValue1 = this._create2DArray(w, h);
            var tempValue2 = this._create2DArray(w, h);

            /** Initialize array.*/
            for (var i = 0; i < w; i++) {
                for (var j = 0; j < h; j++) {
                    tempValue1[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                    tempValue2[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };
                }
            }

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    tempValue1[i][j] = this.moduleValue[i - quietZone][j - quietZone];
                    tempValue2[i][j] = this.dataAllocationValues[i - quietZone][j - quietZone];
                }
            }

            this.moduleValue = tempValue1;
            this.dataAllocationValues = tempValue2;
        },

        /** 
         * Adds format information to the barcode.
         * @private
         */
        _drawFormatInformation: function () {
            var formatInformation = this.qrbarcodeValues.FormatInformation;

            var count = 0;
            for (var i = 0; i < 7; i++) {
                /** Draw from 14 to 8.*/
                if (i == 6)
                    this.moduleValue[i + 1][8].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[i][8].IsBlack = formatInformation[count] == 1 ? true : false;

                this.moduleValue[8][this.noOfModules - i - 1].IsBlack = formatInformation[count++] == 1 ? true : false;
            }
            count = 14;
            for (var i = 0; i < 7; i++) {
                /** Draw from 0 to 6.*/
                if (i == 6)
                    this.moduleValue[8][i + 1].IsBlack = formatInformation[count] == 1 ? true : false;
                else
                    this.moduleValue[8][i].IsBlack = formatInformation[count] == 1 ? true : false;
                this.moduleValue[this.noOfModules - i - 1][8].IsBlack = formatInformation[count--] == 1 ? true : false;
            }

            /** Draw 7.*/
            this.moduleValue[8][8].IsBlack = formatInformation[7] == 1 ? true : false;
            this.moduleValue[8][this.noOfModules - 8].IsBlack = formatInformation[7] == 1 ? true : false;
        },

        /** 
         * @private
         */
        _dataAllocationAndMasking: function (Data) {
            var num = this.noOfModules;
            this.dataAllocationValues = this._create2DArray(num, num);
            var point = 0;

            for (var i = 0; i < num; i++) 
                for (var j = 0; j < num; j++) 
                    this.dataAllocationValues[i][j] = { IsBlack: false, IsFilled: false, IsPDP: false };

            for (var i = num - 1; i >= 0; i -= 2) {
                for (var j = num - 1; j >= 0; j--)
                    point = this._allocateValues(Data, i, j, point);

                i -= 2;
                if (i == 6)
                    i--;

                for (var j = 0; j < num; j++)
                    point = this._allocateValues(Data, i, j, point);
            }
            for (var i = 0; i < num; i++)
                for (var j = 0; j < num; j++)
                    if (!this.moduleValue[i][j].IsFilled) {
                        var flag = this.dataAllocationValues[i][j].IsBlack;
                        if (flag)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }


        },

        /** 
         * @private
         */
        _allocateValues: function (Data, i, j, point) {
            if (!(this.moduleValue[i][j].IsFilled && this.moduleValue[i - 1][j].IsFilled)) {
                if (!this.moduleValue[i][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i][j].IsBlack = Data[point++];
                    if ((i + j) % 3 == 0) {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i][j].IsBlack)
                            this.dataAllocationValues[i][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i][j].IsBlack = true;
                    }

                    this.dataAllocationValues[i][j].IsFilled = true;
                }

                if (!this.moduleValue[i - 1][j].IsFilled) {
                    if (point + 1 < Data.length)
                        this.dataAllocationValues[i - 1][j].IsBlack = Data[point++];
                    if ((i - 1 + j) % 3 == 0) {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                    }
                    else {
                        if (this.dataAllocationValues[i - 1][j].IsBlack)
                            this.dataAllocationValues[i - 1][j].IsBlack = false;
                        else
                            this.dataAllocationValues[i - 1][j].IsBlack = true;
                    }
                    this.dataAllocationValues[i - 1][j].IsFilled = true;
                }
            }

            return point;
        },

        /** 
         * Encodes data.
         * @private
         */
        _encodeData: function () {
            var eData = new Array();
            var c = -1;

            switch (this.mode) {
                case "numeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    break;

                case "alphanumeric":
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    break;

                case "binary":
                    if (this.isEci) {
                        /** Add ECI Mode Indicator.*/
                        eData[++c] = false;
                        eData[++c] = true;
                        eData[++c] = true;
                        eData[++c] = true;

                        /** Add ECI assignment number.*/
                        var numberInBool = this._stringToBoolArray(this.m_EciAssignmentNumber.toString(), 8);
                        for (var e = 0; e < numberInBool.length; e++)
                            eData[++c] = numberInBool[e];
                    }
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    break;
            }

            var numberOfBitsInCharacterCountIndicator = 0;
            if (this.m_Version < 10) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 10;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 9;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 8;
                        break;
                }
            }
            else if (this.m_Version < 27) {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 12;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 11;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }
            else {
                switch (this.mode) {
                    case "numeric":
                        numberOfBitsInCharacterCountIndicator = 14;
                        break;
                    case "alphanumeric":
                        numberOfBitsInCharacterCountIndicator = 13;
                        break;
                    case "binary":
                        numberOfBitsInCharacterCountIndicator = 16;
                        break;
                }
            }

            var dataStringArray = this.Model.text;
            var numberOfBitsInCharacterCountIndicatorInBool = this._intToBoolArray(dataStringArray.length, numberOfBitsInCharacterCountIndicator);

            for (var i = 0; i < numberOfBitsInCharacterCountIndicator; i++)
                eData[++c] = numberOfBitsInCharacterCountIndicatorInBool[i];

            if (this.mode == "numeric") {
                var number = "";
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    number += dataStringArray[i];

                    if (i % 3 == 2 && i != 0 || i == dataStringArray.length - 1) {
                        if (number.length == 3)
                            numberInBool = this._stringToBoolArray(number, 10);
                        else if (number.length == 2)
                            numberInBool = this._stringToBoolArray(number, 7);
                        else
                            numberInBool = this._stringToBoolArray(number, 4);
                        number = "";
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }
            else if (this.mode == "alphanumeric") {
                var numberInString = "";
                var number = 0;
                for (var i = 0; i < dataStringArray.length; i++) {
                    var numberInBool = new Array();
                    numberInString += dataStringArray[i];

                    if (i % 2 == 0 && i + 1 != dataStringArray.length)
                        number = 45 * this._getAlphanumericvalues(dataStringArray[i]);

                    if (i % 2 == 1 && i != 0) {
                        number += this._getAlphanumericvalues(dataStringArray[i]);
                        numberInBool = this._intToBoolArray(number, 11);
                        number = 0;
                        for (var n = 0; n < numberInBool.length; n++)
                            eData[++c] = numberInBool[n];
                        numberInString = "";
                    }
                    if (i != 1 && numberInString != null)
                        if (i + 1 == dataStringArray.length && numberInString.length == 1) {
                            number = this._getAlphanumericvalues(dataStringArray[i]);
                            numberInBool = this._intToBoolArray(number, 6);
                            number = 0;
                            for (var j = 0; j < numberInBool.length; j++)
                                eData[++c] = numberInBool[j];
                        }
                }
            }
            else if (this.mode = "binary") {
                {
                    for (var i = 0; i < dataStringArray.length; i++) {
                        var number = 0;
                        if ((dataStringArray.charCodeAt(i) >= 32 && dataStringArray.charCodeAt(i) <= 126) || (dataStringArray.charCodeAt(i) >= 161 && dataStringArray.charCodeAt(i) <= 255))
                            number = dataStringArray.charCodeAt(i);
                        else if (dataStringArray.charCodeAt(i) >= 65377 && dataStringArray.charCodeAt(i) <= 65439)
                            number = dataStringArray.charCodeAt(i) - 65216;
                        else {
                            console.log("Input text contains non-convertable characters");
                            return;
                        }
                        var numberInBool = this._intToBoolArray(number, 8);
                        for (var j = 0; j < numberInBool.length; j++)
                            eData[++c] = numberInBool[j];
                    }
                }
            }

            /** Terminator.*/
            for (var i = 0; i < 4; i++)
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else
                    eData[++c] = false;

            /** Encode to code words.*/
            for (; ;)              /** Add Padding Bits.*/
            {
                if (eData.length % 8 == 0)
                    break;
                else
                    eData[++c] = false;
            }

            for (; ;) {
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = true;       //11101100
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                }
                if (eData.length / 8 == this.qrbarcodeValues.NumberOfDataCodeWord)
                    break;
                else {
                    eData[++c] = false;       //00010001
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = false;
                    eData[++c] = true;
                }
            }

            var dataBits = this.qrbarcodeValues.NumberOfDataCodeWord;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            var totalBlockSize = blocks[0];
            if (blocks.length == 6)
                totalBlockSize = blocks[0] + blocks[3];

            var ds1 = new Array(totalBlockSize);

            var testEncodeData = eData;
            var n = -1;
            if (blocks.length == 6)     /** If seperated into two seperate blocks.*/
            {
                var dataCodeWordLength = blocks[0] * blocks[2] * 8;
                testEncodeData = new Array(dataCodeWordLength);
                for (var i = 0; i < dataCodeWordLength; i++)
                    testEncodeData[++n] = eData[i];
            }
            var dsOne = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[0]);
            dsOne = this._createBlocks(testEncodeData, blocks[0]);

            for (var i = 0; i < blocks[0]; i++) {
                ds1[i] = this._splitCodeWord(dsOne, i, testEncodeData.length / 8 / blocks[0]);
            }

            n = -1;
            if (blocks.length == 6) {
                testEncodeData = new Array();
                for (var i = parseInt(blocks[0] * blocks[2] * 8) ; i < eData.length; i++)
                    testEncodeData[++n] = eData[i];

                var dsTwo = this._create2DArray(blocks[0], testEncodeData.length / 8 / blocks[3]);
                dsTwo = this._createBlocks(testEncodeData, blocks[3]);

                for (var i = blocks[0], count = 0; i < totalBlockSize; i++)
                    ds1[i] = this._splitCodeWord(dsTwo, count++, testEncodeData.length / 8 / blocks[3]);
            }
            eData = new Array();
            c = -1;
            for (var i = 0; i < 125; i++) {
                for (var k = 0; k < totalBlockSize; k++)
                    for (var j = 0; j < 8; j++)
                        if (i < ds1[k].length) {
                            var temp = ds1[k][i][j];
                            eData[++c] = (temp == '1' ? true : false);
                        }
            }

            this.corCodeWords = { DataBits: this.qrbarcodeValues.NumberOfDataCodeWord, ECCW: this.qrbarcodeValues.NumberOfErrorCorrectingCodeWords, DC: 0 };

            var dataBits = this.corCodeWords.DataBits;
            var eccw = this.corCodeWords.ECCW;
            blocks = this.qrbarcodeValues.NumberOfErrorCorrectionBlocks;

            if (blocks.length == 6)
                this.corCodeWords.DataBits = (dataBits - blocks[3] * blocks[5]) / blocks[0];
            else
                this.corCodeWords.DataBits = dataBits / blocks[0];

            this.corCodeWords.ECCW = eccw / totalBlockSize;
            var polynomial = new Array(totalBlockSize);
            var count = 0;

            for (var i = 0; i < blocks[0]; i++) {
                this.corCodeWords.DC = ds1[count];
                polynomial[count++] = this._getERCW();
            }
            if (blocks.length == 6) {
                this.corCodeWords.DataBits = (dataBits - blocks[0] * blocks[2]) / blocks[3];

                for (var i = 0; i < blocks[3]; i++) {
                    this.corCodeWords.DC = ds1[count];
                    polynomial[count++] = this._getERCW();
                }
            }
            if (blocks.length != 6) {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < blocks[0]; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }
            else {
                for (var i = 0; i < polynomial[0].length; i++) {
                    for (var k = 0; k < totalBlockSize; k++)
                        for (var j = 0; j < 8; j++)
                            if (i < polynomial[k].length)
                                eData[++c] = (polynomial[k][i][j] == '1' ? true : false);
                }
            }


            this._dataAllocationAndMasking(eData);
        },

        /** 
         * @private
         */
        _getERCW: function () {
            var decimalRepresentation;
            var ecw;
            var decimalValue = new Array(this.corCodeWords.DataBits);
            var gx = new Array();

            switch (this.corCodeWords.ECCW) {
                case 7:
                    gx = new Array(0, 87, 229, 146, 149, 238, 102, 21);
                    gx = this._getElement(gx);
                    break;
                case 10:
                    gx = new Array(0, 251, 67, 46, 61, 118, 70, 64, 94, 32, 45);
                    gx = this._getElement(gx);
                    break;
                case 13:
                    gx = new Array(0, 74, 152, 176, 100, 86, 100, 106, 104, 130, 218, 206, 140, 78);
                    gx = this._getElement(gx);
                    break;
                case 15:
                    gx = new Array(0, 8, 183, 61, 91, 202, 37, 51, 58, 58, 237, 140, 124, 5, 99, 105);
                    gx = this._getElement(gx);
                    break;
                case 16:
                    gx = new Array(0, 120, 104, 107, 109, 102, 161, 76, 3, 91, 191, 147, 169, 182, 194, 225, 120);
                    gx = this._getElement(gx);
                    break;
                case 17:
                    gx = new Array(0, 43, 139, 206, 78, 43, 239, 123, 206, 214, 147, 24, 99, 150, 39, 243, 163, 136);
                    gx = this._getElement(gx);
                    break;
                case 18:
                    gx = new Array(0, 215, 234, 158, 94, 184, 97, 118, 170, 79, 187, 152, 148, 252, 179, 5, 98, 96, 153);
                    gx = this._getElement(gx);
                    break;
                case 20:
                    gx = new Array(0, 17, 60, 79, 50, 61, 163, 26, 187, 202, 180, 221, 225, 83, 239, 156, 164, 212, 212, 188, 190);
                    gx = this._getElement(gx);
                    break;
                case 22:
                    gx = new Array(0, 210, 171, 247, 242, 93, 230, 14, 109, 221, 53, 200, 74, 8, 172, 98, 80, 219, 134, 160, 105, 165, 231);
                    gx = this._getElement(gx);
                    break;
                case 24:
                    gx = new Array(0, 229, 121, 135, 48, 211, 117, 251, 126, 159, 180, 169, 152, 192, 226, 228, 218, 111, 0, 117, 232, 87, 96, 227, 21);
                    gx = this._getElement(gx);
                    break;
                case 26:
                    gx = new Array(0, 173, 125, 158, 2, 103, 182, 118, 17, 145, 201, 111, 28, 165, 53, 161, 21, 245, 142, 13, 102, 48, 227, 153, 145, 218, 70);
                    gx = this._getElement(gx);
                    break;
                case 28:
                    gx = new Array(0, 168, 223, 200, 104, 224, 234, 108, 180, 110, 190, 195, 147, 205, 27, 232, 201, 21, 43, 245, 87, 42, 195, 212, 119, 242, 37, 9, 123);
                    gx = this._getElement(gx);
                    break;
                case 30:
                    gx = new Array(0, 41, 173, 145, 152, 216, 31, 179, 182, 50, 48, 110, 86, 239, 96, 222, 125, 42, 173, 226, 193, 224, 130, 156, 37, 251, 216, 238, 40, 192, 180);
                    gx = this._getElement(gx);
                    break;
                case 32:
                    gx = new Array(0, 10, 6, 106, 190, 249, 167, 4, 67, 209, 138, 138, 32, 242, 123, 89, 27, 120, 185, 80, 156, 38, 69, 171, 60, 28, 222, 80, 52, 254, 185, 220, 241);
                    gx = this._getElement(gx);
                    break;
                case 34:
                    gx = new Array(0, 111, 77, 146, 94, 26, 21, 108, 19, 105, 94, 113, 193, 86, 140, 163, 125, 58, 158, 229, 239, 218, 103, 56, 70, 114, 61, 183, 129, 167, 13, 98, 62, 129, 51);
                    gx = this._getElement(gx);
                    break;
                case 36:
                    gx = new Array(0, 200, 183, 98, 16, 172, 31, 246, 234, 60, 152, 115, 0, 167, 152, 113, 248, 238, 107, 18, 63, 218, 37, 87, 210, 105, 177, 120, 74, 121, 196, 117, 251, 113, 233, 30, 120);
                    gx = this._getElement(gx);
                    break;
                case 40:
                    gx = new Array(0, 59, 116, 79, 161, 252, 98, 128, 205, 128, 161, 247, 57, 163, 56, 235, 106, 53, 26, 187, 174, 226, 104, 170, 7, 175, 35, 181, 114, 88, 41, 47, 163, 125, 134, 72, 20, 232, 53, 35, 15);
                    gx = this._getElement(gx);
                    break;
                case 42:
                    gx = new Array(0, 250, 103, 221, 230, 25, 18, 137, 231, 0, 3, 58, 242, 221, 191, 110, 84, 230, 8, 188, 106, 96, 147, 15, 131, 139, 34, 101, 223, 39, 101, 213, 199, 237, 254, 201, 123, 171, 162, 194, 117, 50, 96);
                    gx = this._getElement(gx);
                    break;
                case 44:
                    gx = new Array(0, 190, 7, 61, 121, 71, 246, 69, 55, 168, 188, 89, 243, 191, 25, 72, 123, 9, 145, 14, 247, 1, 238, 44, 78, 143, 62, 224, 126, 118, 114, 68, 163, 52, 194, 217, 147, 204, 169, 37, 130, 113, 102, 73, 181);
                    gx = this._getElement(gx);
                    break;
                case 46:
                    gx = new Array(0, 112, 94, 88, 112, 253, 224, 202, 115, 187, 99, 89, 5, 54, 113, 129, 44, 58, 16, 135, 216, 169, 211, 36, 1, 4, 96, 60, 241, 73, 104, 234, 8, 249, 245, 119, 174, 52, 25, 157, 224, 43, 202, 223, 19, 82, 15);
                    gx = this._getElement(gx);
                    break;
                case 48:
                    gx = new Array(0, 228, 25, 196, 130, 211, 146, 60, 24, 251, 90, 39, 102, 240, 61, 178, 63, 46, 123, 115, 18, 221, 111, 135, 160, 182, 205, 107, 206, 95, 150, 120, 184, 91, 21, 247, 156, 140, 238, 191, 11, 94, 227, 84, 50, 163, 39, 34, 108);
                    gx = this._getElement(gx);
                    break;
                case 50:
                    gx = new Array(0, 232, 125, 157, 161, 164, 9, 118, 46, 209, 99, 203, 193, 35, 3, 209, 111, 195, 242, 203, 225, 46, 13, 32, 160, 126, 209, 130, 160, 242, 215, 242, 75, 77, 42, 189, 32, 113, 65, 124, 69, 228, 114, 235, 175, 124, 170, 215, 232, 133, 205);
                    gx = this._getElement(gx);
                    break;
                case 52:
                    gx = new Array(0, 116, 50, 86, 186, 50, 220, 251, 89, 192, 46, 86, 127, 124, 19, 184, 233, 151, 215, 22, 14, 59, 145, 37, 242, 203, 134, 254, 89, 190, 94, 59, 65, 124, 113, 100, 233, 235, 121, 22, 76, 86, 97, 39, 242, 200, 220, 101, 33, 239, 254, 116, 51);
                    gx = this._getElement(gx);
                    break;
                case 54:
                    gx = new Array(0, 183, 26, 201, 87, 210, 221, 113, 21, 46, 65, 45, 50, 238, 184, 249, 225, 102, 58, 209, 218, 109, 165, 26, 95, 184, 192, 52, 245, 35, 254, 238, 175, 172, 79, 123, 25, 122, 43, 120, 108, 215, 80, 128, 201, 235, 8, 153, 59, 101, 31, 198, 76, 31, 156);
                    gx = this._getElement(gx);
                    break;
                case 56:
                    gx = new Array(0, 106, 120, 107, 157, 164, 216, 112, 116, 2, 91, 248, 163, 36, 201, 202, 229, 6, 144, 254, 155, 135, 208, 170, 209, 12, 139, 127, 142, 182, 249, 177, 174, 190, 28, 10, 85, 239, 184, 101, 124, 152, 206, 96, 23, 163, 61, 27, 196, 247, 151, 154, 202, 207, 20, 61, 10);
                    gx = this._getElement(gx);
                    break;
                case 58:
                    gx = new Array(0, 82, 116, 26, 247, 66, 27, 62, 107, 252, 182, 200, 185, 235, 55, 251, 242, 210, 144, 154, 237, 176, 141, 192, 248, 152, 249, 206, 85, 253, 142, 65, 165, 125, 23, 24, 30, 122, 240, 214, 6, 129, 218, 29, 145, 127, 134, 206, 245, 117, 29, 41, 63, 159, 142, 233, 125, 148, 123);
                    gx = this._getElement(gx);
                    break;
                case 60:
                    gx = new Array(0, 107, 140, 26, 12, 9, 141, 243, 197, 226, 197, 219, 45, 211, 101, 219, 120, 28, 181, 127, 6, 100, 247, 2, 205, 198, 57, 115, 219, 101, 109, 160, 82, 37, 38, 238, 49, 160, 209, 121, 86, 11, 124, 30, 181, 84, 25, 194, 87, 65, 102, 190, 220, 70, 27, 209, 16, 89, 7, 33, 240);
                    gx = this._getElement(gx);
                    break;
                case 62:
                    gx = new Array(0, 65, 202, 113, 98, 71, 223, 248, 118, 214, 94, 0, 122, 37, 23, 2, 228, 58, 121, 7, 105, 135, 78, 243, 118, 70, 76, 223, 89, 72, 50, 70, 111, 194, 17, 212, 126, 181, 35, 221, 117, 235, 11, 229, 149, 147, 123, 213, 40, 115, 6, 200, 100, 26, 246, 182, 218, 127, 215, 36, 186, 110, 106);
                    gx = this._getElement(gx);
                    break;
                case 64:
                    gx = new Array(0, 45, 51, 175, 9, 7, 158, 159, 49, 68, 119, 92, 123, 177, 204, 187, 254, 200, 78, 141, 149, 119, 26, 127, 53, 160, 93, 199, 212, 29, 24, 145, 156, 208, 150, 218, 209, 4, 216, 91, 47, 184, 146, 47, 140, 195, 195, 125, 242, 238, 63, 99, 108, 140, 230, 242, 31, 204, 11, 178, 243, 217, 156, 213, 231);
                    gx = this._getElement(gx);
                    break;
                case 66:
                    gx = new Array(0, 5, 118, 222, 180, 136, 136, 162, 51, 46, 117, 13, 215, 81, 17, 139, 247, 197, 171, 95, 173, 65, 137, 178, 68, 111, 95, 101, 41, 72, 214, 169, 197, 95, 7, 44, 154, 77, 111, 236, 40, 121, 143, 63, 87, 80, 253, 240, 126, 217, 77, 34, 232, 106, 50, 168, 82, 76, 146, 67, 106, 171, 25, 132, 93, 45, 105);
                    gx = this._getElement(gx);
                    break;
                case 68:
                    gx = new Array(0, 247, 159, 223, 33, 224, 93, 77, 70, 90, 160, 32, 254, 43, 150, 84, 101, 190, 205, 133, 52, 60, 202, 165, 220, 203, 151, 93, 84, 15, 84, 253, 173, 160, 89, 227, 52, 199, 97, 95, 231, 52, 177, 41, 125, 137, 241, 166, 225, 118, 2, 54, 32, 82, 215, 175, 198, 43, 238, 235, 27, 101, 184, 127, 3, 5, 8, 163, 238);
                    gx = this._getElement(gx);
                    break;
            }

            decimalValue = this._toDecimal(this.corCodeWords.DC, decimalValue);
            decimalRepresentation = this._divide(gx, decimalValue);
            ecw = this._toBinary(decimalRepresentation);
            return ecw;
        },

        /** 
         * Convert to binary.
         * @private
         */
        _toBinary: function (decimalRepresentation) {
            var toBinary = new Array(this.corCodeWords.ECCW);
            for (var i = 0; i < decimalRepresentation.length; i++) {
                var temp = decimalRepresentation[i].toString(2);
                var str = new String();
                for (var j = 0; j < (8 - temp.length) ; j++)
                    str += "0";
                toBinary[i] = str + temp;
            }
            return toBinary;
        },

        /** 
         * @private
         */
        _divide: function (gx, decimalValue) {
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            var eccWords = this.corCodeWords.ECCW;
            var messagePolynom = [];
            for (var i = 0; i < decimalValue.length; i++) {
                messagePolynom.push({
                Exponent: decimalValue.length - 1 - i,
                Coefficient: decimalValue[i]
            })
            }

            var generatorPolynom = [];
            for (var i = 0; i < gx.length; i++)
            {
                generatorPolynom.push({
                Exponent: gx.length - 1 - i,
                Coefficient: this._findElement(gx[i], alpha)
            })
            }

            for (var i = 0; i < messagePolynom.length; i++)
            {
                messagePolynom[i].Exponent = messagePolynom[i].Exponent + eccWords;
            }

            var genLeadtermFactor = messagePolynom[0].Exponent - generatorPolynom[0].Exponent;
            for (var i = 0; i < generatorPolynom.length; i++) {
                generatorPolynom[i].Coefficient = generatorPolynom[i].Coefficient;
                generatorPolynom[i].Exponent = generatorPolynom[i].Exponent + genLeadtermFactor;
            }

            var leadTermSource = messagePolynom;
            for (var i = 0; i < messagePolynom.length; i++)
            {
                if (leadTermSource[0].Coefficient == 0) {
                    leadTermSource.splice(0, 1);
                }
                else {
                    var alphaNotation = this._convertToAlphaNotation(leadTermSource, alpha);
                    var resPoly = this._multiplyGeneratorPolynomByLeadterm(generatorPolynom, alphaNotation[0], i);
                    resPoly = this._convertToDecNotation(resPoly,alpha);
                    resPoly = this._XORPolynoms(leadTermSource, resPoly);
                    leadTermSource = resPoly;
                }

            }
            var returnArray = [];
            for (var i = 0; i < leadTermSource.length; i++) {
                returnArray.push(leadTermSource[i].Coefficient);
            }
            return returnArray;
        },

        /** 
         * @private
         */
         _convertToAlphaNotation: function(poly, alpha) {
             var newPoly = [];
             for (var i = 0; i < poly.length; i++) {
                 if (poly[i].Coefficient != 0) {
                     newPoly.push({
                         Exponent: poly[i].Exponent,
                         Coefficient: this._findElement(poly[i].Coefficient, alpha)
                     })
                 }
             }
             return newPoly;
         },

         /** 
         * @private
         */
         _multiplyGeneratorPolynomByLeadterm: function(genPolynom, leadTerm, lowerExponentBy) {
             var tempPolynom = [];
             for (var i = 0; i < genPolynom.length; i++) {
                 tempPolynom.push({
                     Exponent: genPolynom[i].Exponent - lowerExponentBy,
                     Coefficient: (genPolynom[i].Coefficient + leadTerm.Coefficient) % 255
             })
             }
             return tempPolynom;
         },

         /** 
         * @private
         */
         _convertToDecNotation: function(poly, alpha) {
             var tempPolynom = [];
             for (var i = 0; i < poly.length; i++) {
             tempPolynom.push({
                     Exponent: poly[i].Exponent,
                     Coefficient:  this._getIntValFromAlphaExp(poly[i].Coefficient, alpha)
             })
             }
             return tempPolynom;
         },

         /** 
         * @private
         */
         _getIntValFromAlphaExp: function(element, alpha) {
             if (element > 255)
            {
                element = element - 255;
            }
            return alpha[element];
         },
         
         
         /** 
         * @private
         */
         _XORPolynoms: function(messagePolynom, resPolynom) {
              var resultPolynom = [];
              var longPoly = [];
              var shortPoly = [];
            if (messagePolynom.length >= resPolynom.length)
            {
                longPoly = messagePolynom;
                shortPoly = resPolynom;
            }
            else
            {
                longPoly = resPolynom;
                shortPoly = messagePolynom;
            }
            
            for (var i = 0; i < longPoly.length; i++)
            {
            resultPolynom.push({
                     Exponent: messagePolynom[0].Exponent - i,
                     Coefficient:  longPoly[i].Coefficient ^ (shortPoly.length > i ? shortPoly[i].Coefficient : 0)
             })
            }
            resultPolynom.splice(0, 1);
            return resultPolynom;
         },

        /** 
         * @private
         */
        _findElement: function (element, alpha) {
            var j = 0;
            for (; j < alpha.length; j++) {
                if (element == alpha[j])
                    break;
            }
            return j;
        },

        /** 
         * @private
         */
        _getElement: function (element) {
            var gx = new Array(element.length);
            var alpha = new Array(1, 2, 4, 8, 16, 32, 64, 128, 29, 58, 116, 232, 205, 135, 19, 38, 76, 152, 45, 90, 180, 117, 234, 201, 143, 3, 6, 12, 24, 48, 96, 192, 157, 39, 78, 156, 37, 74, 148, 53, 106, 212, 181, 119, 238, 193, 159, 35, 70, 140, 5, 10, 20, 40, 80, 160, 93, 186, 105, 210, 185, 111, 222, 161, 95, 190, 97, 194, 153, 47, 94, 188, 101, 202, 137, 15, 30, 60, 120, 240, 253, 231, 211, 187, 107, 214, 177, 127, 254, 225, 223, 163, 91, 182, 113, 226, 217, 175, 67, 134, 17, 34, 68, 136, 13, 26, 52, 104, 208, 189, 103, 206, 129, 31, 62, 124, 248, 237, 199, 147, 59, 118, 236, 197, 151, 51, 102, 204, 133, 23, 46, 92, 184, 109, 218, 169, 79, 158, 33, 66, 132, 21, 42, 84, 168, 77, 154, 41, 82, 164, 85, 170, 73, 146, 57, 114, 228, 213, 183, 115, 230, 209, 191, 99, 198, 145, 63, 126, 252, 229, 215, 179, 123, 246, 241, 255, 227, 219, 171, 75, 150, 49, 98, 196, 149, 55, 110, 220, 165, 87, 174, 65, 130, 25, 50, 100, 200, 141, 7, 14, 28, 56, 112, 224, 221, 167, 83, 166, 81, 162, 89, 178, 121, 242, 249, 239, 195, 155, 43, 86, 172, 69, 138, 9, 18, 36, 72, 144, 61, 122, 244, 245, 247, 243, 251, 235, 203, 139, 11, 22, 44, 88, 176, 125, 250, 233, 207, 131, 27, 54, 108, 216, 173, 71, 142);
            for (var i = 0; i < element.length; i++) {
                if (element[i] > 255)
                    element[i] = element[i] - 255;
                gx[i] = alpha[element[i]];
            }
            return gx;
        },

        /** 
         * @private
         */
        _toDecimal: function (inString) {
            var decimalValue = new Array(inString.length);
            for (var i = 0; i < inString.length; i++)
                decimalValue[i] = parseInt(inString[i], 2);

            return decimalValue;
        },

        /** 
         * @private
         */
        _splitCodeWord: function (ds, blk, count) {
            count = parseInt(count);
            var ds1 = new Array(count);
            for (var i = 0; i < count; i++)
                ds1[i] = ds[blk][i];
            return ds1;
        },

        /** 
         * Create blocks for datamatrix barcode.
         * @private
         */
        _createBlocks: function (data, noOfBlocks) {
            var ret = this._create2DArray(noOfBlocks, data.length / 8 / noOfBlocks);
            var stringValue = "";
            var j = 0, i = 0, blockNumber = 0;

            for (; j < data.length; j++) {
                if (j % 8 == 0 && j != 0) {
                    ret[blockNumber][i] = stringValue;
                    stringValue = "";
                    i++;

                    if (i == (data.length / noOfBlocks / 8)) {
                        blockNumber++;
                        i = 0;
                    }
                }

                stringValue += data[j] ? 1 : 0;
            }
            ret[blockNumber][i] = stringValue;
            return ret;
        },

        /** 
         * Returns alphanumeric values for the character.
         * @private
         */
        _getAlphanumericvalues: function (charValue) {
            var valueInInt = -1;
            var alphaNumArray = new Array('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', ' ', '$', '%', '*', '+', '-', '.', '/', ':');
            valueInInt = alphaNumArray.indexOf(charValue);
            return valueInInt;
        },

        /** 
         * @private
         */
        _stringToBoolArray: function (numberInString, noOfBits) {
            var numberInBool = new Array(noOfBits);

            var number = 0;
            for (var i = 0; i < numberInString.length; i++)
                number = number * 10 + numberInString.charCodeAt(i) - 48;

            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;

            return numberInBool;
        },

        /** 
         * @private
         */
        _intToBoolArray: function (number, noOfBits) {
            var numberInBool = new Array(noOfBits);
            for (var i = 0; i < noOfBits; i++)
                numberInBool[noOfBits - i - 1] = ((number >> i) & 1) == 1;
            return numberInBool;
        },

        /** 
         * @private
         */
        _drawPDP: function (x, y) {
            var i, j;
            for (i = x, j = y; i < x + 7; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 6].IsBlack = true;
                this.moduleValue[i][y + 6].IsFilled = true;
                this.moduleValue[i][y + 6].IsPDP = true;

                if (y + 7 < this.noOfModules) {
                    this.moduleValue[i][y + 7].IsBlack = false;
                    this.moduleValue[i][y + 7].IsFilled = true;
                    this.moduleValue[i][y + 7].IsPDP = true;
                }
                else if (y - 1 >= 0) {
                    this.moduleValue[i][y - 1].IsBlack = false;
                    this.moduleValue[i][y - 1].IsFilled = true;
                    this.moduleValue[i][y - 1].IsPDP = true;
                }

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 6][j].IsBlack = true;
                this.moduleValue[x + 6][j].IsFilled = true;
                this.moduleValue[x + 6][j].IsPDP = true;

                if (x + 7 < this.noOfModules) {
                    this.moduleValue[x + 7][j].IsBlack = false;
                    this.moduleValue[x + 7][j].IsFilled = true;
                    this.moduleValue[x + 7][j].IsPDP = true;
                }
                else if (x - 1 >= 0) {
                    this.moduleValue[x - 1][j].IsBlack = false;
                    this.moduleValue[x - 1][j].IsFilled = true;
                    this.moduleValue[x - 1][j].IsPDP = true;
                }
            }

            if (x + 7 < this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x + 7][y + 7].IsBlack = false;
                this.moduleValue[x + 7][y + 7].IsFilled = true;
                this.moduleValue[x + 7][y + 7].IsPDP = true;
            }
            else if (x + 7 < this.noOfModules && y + 7 >= this.noOfModules) {
                this.moduleValue[x + 7][y - 1].IsBlack = false;
                this.moduleValue[x + 7][y - 1].IsFilled = true;
                this.moduleValue[x + 7][y - 1].IsPDP = true;
            }
            else if (x + 7 >= this.noOfModules && y + 7 < this.noOfModules) {
                this.moduleValue[x - 1][y + 7].IsBlack = false;
                this.moduleValue[x - 1][y + 7].IsFilled = true;
                this.moduleValue[x - 1][y + 7].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 5; i++, j++) {
                this.moduleValue[i][y].IsBlack = false;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 4].IsBlack = false;
                this.moduleValue[i][y + 4].IsFilled = true;
                this.moduleValue[i][y + 4].IsPDP = true;

                this.moduleValue[x][j].IsBlack = false;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 4][j].IsBlack = false;
                this.moduleValue[x + 4][j].IsFilled = true;
                this.moduleValue[x + 4][j].IsPDP = true;
            }

            x++;
            y++;
            for (i = x, j = y; i < x + 3; i++, j++) {
                this.moduleValue[i][y].IsBlack = true;
                this.moduleValue[i][y].IsFilled = true;
                this.moduleValue[i][y].IsPDP = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;
                this.moduleValue[i][y + 2].IsPDP = true;

                this.moduleValue[x][j].IsBlack = true;
                this.moduleValue[x][j].IsFilled = true;
                this.moduleValue[x][j].IsPDP = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
                this.moduleValue[x + 2][j].IsPDP = true;
            }
            this.moduleValue[x + 1][y + 1].IsBlack = true;
            this.moduleValue[x + 1][y + 1].IsFilled = true;
            this.moduleValue[x + 1][y + 1].IsPDP = true;
        },

        /** 
         * @private
         */
        _drawAlignmentPattern: function (x, y) {
            var i, j;
            for (i = x - 2, j = y - 2; i < x + 3; i++, j++) {
                this.moduleValue[i][y - 2].IsBlack = true;
                this.moduleValue[i][y - 2].IsFilled = true;

                this.moduleValue[i][y + 2].IsBlack = true;
                this.moduleValue[i][y + 2].IsFilled = true;

                this.moduleValue[x - 2][j].IsBlack = true;
                this.moduleValue[x - 2][j].IsFilled = true;

                this.moduleValue[x + 2][j].IsBlack = true;
                this.moduleValue[x + 2][j].IsFilled = true;
            }

            for (i = x - 1, j = y - 1; i < x + 2; i++, j++) {
                this.moduleValue[i][y - 1].IsBlack = false;
                this.moduleValue[i][y - 1].IsFilled = true;

                this.moduleValue[i][y + 1].IsBlack = false;
                this.moduleValue[i][y + 1].IsFilled = true;

                this.moduleValue[x - 1][j].IsBlack = false;
                this.moduleValue[x - 1][j].IsFilled = true;

                this.moduleValue[x + 1][j].IsBlack = false;
                this.moduleValue[x + 1][j].IsFilled = true;
            }
            this.moduleValue[x][y].IsBlack = true;
            this.moduleValue[x][y].IsFilled = true;
        },

        /** 
         * @private
         */
        _getAlignmentPatternCoOrdinates: function () {
            var allign = new Array();
            switch (this.m_Version) {
                case 02:
                    allign = new Array(6, 18);
                    break;
                case 03:
                    allign = new Array(6, 22);
                    break;
                case 04:
                    allign = new Array(6, 26);
                    break;
                case 05:
                    allign = new Array(6, 30);
                    break;
                case 06:
                    allign = new Array(6, 34);
                    break;
                case 07:
                    allign = new Array(6, 22, 38);
                    break;
                case 08:
                    allign = new Array(6, 24, 42);
                    break;
                case 09:
                    allign = new Array(6, 26, 46);
                    break;
                case 10:
                    allign = new Array(6, 28, 50);
                    break;
                case 11:
                    allign = new Array(6, 30, 54);
                    break;
                case 12:
                    allign = new Array(6, 32, 58);
                    break;
                case 13:
                    allign = new Array(6, 34, 62);
                    break;
                case 14:
                    allign = new Array(6, 26, 46, 66);
                    break;
                case 15:
                    allign = new Array(6, 26, 48, 70);
                    break;
                case 16:
                    allign = new Array(6, 26, 50, 74);
                    break;
                case 17:
                    allign = new Array(6, 30, 54, 78);
                    break;
                case 18:
                    allign = new Array(6, 30, 56, 82);
                    break;
                case 19:
                    allign = new Array(6, 30, 58, 86);
                    break;
                case 20:
                    allign = new Array(6, 34, 62, 90);
                    break;
                case 21:
                    allign = new Array(6, 28, 50, 72, 94);
                    break;
                case 22:
                    allign = new Array(6, 26, 50, 74, 98);
                    break;
                case 23:
                    allign = new Array(6, 30, 54, 78, 102);
                    break;
                case 24:
                    allign = new Array(6, 28, 54, 80, 106);
                    break;
                case 25:
                    allign = new Array(6, 32, 58, 84, 110);
                    break;
                case 26:
                    allign = new Array(6, 30, 58, 86, 114);
                    break;
                case 27:
                    allign = new Array(6, 34, 62, 90, 118);
                    break;
                case 28:
                    allign = new Array(6, 26, 50, 74, 98, 122);
                    break;
                case 29:
                    allign = new Array(6, 30, 54, 78, 102, 126);
                    break;
                case 30:
                    allign = new Array(6, 26, 52, 78, 104, 130);
                    break;
                case 31:
                    allign = new Array(6, 30, 56, 82, 108, 134);
                    break;
                case 32:
                    allign = new Array(6, 34, 60, 86, 112, 138);
                    break;
                case 33:
                    allign = new Array(6, 30, 58, 86, 114, 142);
                    break;
                case 34:
                    allign = new Array(6, 34, 62, 90, 118, 146);
                    break;
                case 35:
                    allign = new Array(6, 30, 54, 78, 102, 126, 150);
                    break;
                case 36:
                    allign = new Array(6, 24, 50, 76, 102, 128, 154);
                    break;
                case 37:
                    allign = new Array(6, 28, 54, 80, 106, 132, 158);
                    break;
                case 38:
                    allign = new Array(6, 32, 58, 84, 110, 136, 162);
                    break;
                case 39:
                    allign = new Array(6, 26, 54, 82, 110, 138, 166);
                    break;
                case 40:
                    allign = new Array(6, 30, 58, 86, 114, 142, 170);
                    break;
            }
            return allign;
        },

        /** 
         * @private
         */
        _findECINumber: function (text) {
            var num = 3;
            for (var i = 0; i < text.length; i++) {
                var cCode = text.charCodeAt(i);
                if (cCode >= 32 && cCode <= 255) {
                    continue;
                }

                var check = cCode.toString(16);
                var CP437CharSet = new Array("2591", "2592", "2593", "2502", "2524", "2561", "2562", "2556", "2555", "2563", "2551", "2557", "255D", "255C", "255B", "2510", "2514", "2534", "252C", "251C", "2500", "253C", "255E", "255F", "255A", "2554", "2569", "2566", "2560", "2550", "256C", "2567", "2568", "2564", "2565", "2559", "2558", "2552", "2553", "256B", "256A", "2518", "250C", "2588", "2584", "258C", "2590", "2580", "25A0");
                var latin2CharSet = new Array("104", "2D8", "141", "13D", "15A", "160", "15E", "164", "179", "17D", "17B", "105", "2DB", "142", "13E", "15B", "2C7", "161", "15F", "165", "17A", "2DD", "17E", "17C", "154", "102", "139", "106", "10C", "118", "11A", "10E", "110", "143", "147", "150", "158", "16E", "170", "162", "155", "103", "13A", "107", "10D", "119", "11B", "10F", "111", "144", "148", "151", "159", "16F", "171", "163", "2D9");
                var latin3CharSet = new Array("126", "124", "130", "15E", "11E", "134", "17B", "127", "125", "131", "15F", "11F", "135", "17C", "10A", "108", "120", "11C", "16C", "15C", "10B", "109", "121", "11D", "16D", "15D");
                var latin4CharSet = new Array("104", "138", "156", "128", "13B", "160", "112", "122", "166", "17D", "105", "2DB", "157", "129", "13C", "2C7", "161", "113", "123", "167", "14A", "17E", "14B", "100", "12E", "10C", "118", "116", "12A", "110", "145", "14C", "136", "172", "168", "16A", "101", "12F", "10D", "119", "117", "12B", "111", "146", "14D", "137", "173", "169", "16B");
                var windows1250CharSet = new Array("141", "104", "15E", "17B", "142", "105", "15F", "13D", "13E", "17C");
                var windows1251CharSet = new Array("402", "403", "453", "409", "40A", "40C", "40B", "40F", "452", "459", "45A", "45C", "45B", "45F", "40E", "45E", "408", "490", "401", "404", "407", "406", "456", "491", "451", "454", "458", "405", "455", "457");
                var windows1252CharSet = new Array("20AC", "201A", "192", "201E", "2026", "2020", "2021", "2C6", "2030", "160", "2039", "152", "17D", "2018", "2019", "201C", "201D", "2022", "2013", "2014", "2DC", "2122", "161", "203A", "153", "17E", "178");
                var windows1256CharSet = new Array("67E", "679", "152", "686", "698", "688", "6AF", "6A9", "691", "153", "6BA", "6BE", "6C1");

                /** Check for CP437*/
                if (CP437CharSet.indexOf(check) != -1) {
                    num = 2;
                    break;
                }
                    /** Check for ISO/IEC 8859-2*/
                else if (latin2CharSet.indexOf(check) != -1) {
                    num = 4;
                    break;
                }
                    /** Check for ISO/IEC 8859-3*/
                else if (latin3CharSet.indexOf(check) != -1) {
                    num = 5;
                    break;
                }
                    /** Check for ISO/IEC 8859-4*/
                else if (latin4CharSet.indexOf(check) != -1) {
                    num = 6;
                    break;
                }
                    /** Check for ISO/IEC 8859-5*/
                else if (check >= 1025 && check <= 1119 && check != 1037 && check != 1104 && check != 1117) {
                    num = 7;
                    break;
                }
                    /** Check for ISO/IEC 8859-6*/
                else if ((check >= 1569 && check <= 1594) || (check >= 1600 && check <= 1618) || check == 1567 || check == 1563 || check == 1548) {
                    num = 8;
                    break;
                }
                    /** Check for ISO/IEC 8859-7*/
                else if ((check >= 900 && check <= 974) || check == 890) {
                    num = 9;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 1488 && check <= 1514) {
                    num = 10;
                    break;
                }
                    /** Check for ISO/IEC 8859-8*/
                else if (check >= 3585 && check <= 3675) {
                    num = 13;
                    break;
                }
                    /** Check for Windows1250*/
                else if ((windows1250CharSet.indexOf(check) != 1) || (check >= 1569 && check <= 1610)) {
                    num = 21;
                    break;
                }
                    /** Check for Windows1251*/
                else if ((windows1251CharSet.indexOf(check) != 1) || (check >= 1040 && check <= 1103)) {
                    num = 22;
                    break;
                }
                    /** Check for Windows1252*/
                else if (windows1252CharSet.indexOf(check) != -1) {
                    num = 23;
                    break;
                }
                    /** Check for Windows1256*/
                else if (windows1256CharSet.indexOf(check) != -1) {
                    num = 24;
                    break;
                }
            }
            return num;
        },

        /** 
         * @private
         */
        _getNumberOfDataCodeWord: function () {
            var countOfDataCodeWord = 0;

            switch (this.m_Version) {
                case 01:
                    countOfDataCodeWord = 16;
                    break;
                case 02:
                    countOfDataCodeWord = 28;
                    break;
                case 03:
                    countOfDataCodeWord = 44;
                    break;
                case 04:
                    countOfDataCodeWord = 64;
                    break;
                case 05:
                    countOfDataCodeWord = 86;
                    break;
                case 06:
                    countOfDataCodeWord = 108;
                    break;
                case 07:
                    countOfDataCodeWord = 124;
                    break;
                case 08:
                    countOfDataCodeWord = 154;
                    break;
                case 09:
                    countOfDataCodeWord = 182;
                    break;
                case 10:
                    countOfDataCodeWord = 216;
                    break;
                case 11:
                    countOfDataCodeWord = 254;
                    break;
                case 12:
                    countOfDataCodeWord = 290;
                    break;
                case 13:
                    countOfDataCodeWord = 334;
                    break;
                case 14:
                    countOfDataCodeWord = 365;
                    break;
                case 15:
                    countOfDataCodeWord = 415;
                    break;
                case 16:
                    countOfDataCodeWord = 453;
                    break;
                case 17:
                    countOfDataCodeWord = 507;
                    break;
                case 18:
                    countOfDataCodeWord = 563;
                    break;
                case 19:
                    countOfDataCodeWord = 627;
                    break;
                case 20:
                    countOfDataCodeWord = 669;
                    break;
                case 21:
                    countOfDataCodeWord = 714;
                    break;
                case 22:
                    countOfDataCodeWord = 782;
                    break;
                case 23:
                    countOfDataCodeWord = 860;
                    break;
                case 24:
                    countOfDataCodeWord = 914;
                    break;
                case 25:
                    countOfDataCodeWord = 1000;
                    break;
                case 26:
                    countOfDataCodeWord = 1062;
                    break;
                case 27:
                    countOfDataCodeWord = 1128;
                    break;
                case 28:
                    countOfDataCodeWord = 1193;
                    break;
                case 29:
                    countOfDataCodeWord = 1267;
                    break;
                case 30:
                    countOfDataCodeWord = 1373;
                    break;
                case 31:
                    countOfDataCodeWord = 1455;
                    break;
                case 32:
                    countOfDataCodeWord = 1541;
                    break;
                case 33:
                    countOfDataCodeWord = 1631;
                    break;
                case 34:
                    countOfDataCodeWord = 1725;
                    break;
                case 35:
                    countOfDataCodeWord = 1812;
                    break;
                case 36:
                    countOfDataCodeWord = 1914;
                    break;
                case 37:
                    countOfDataCodeWord = 1992;
                    break;
                case 38:
                    countOfDataCodeWord = 2102;
                    break;
                case 39:
                    countOfDataCodeWord = 2216;
                    break;
                case 40:
                    countOfDataCodeWord = 2334;
                    break;
            }

            return countOfDataCodeWord;
        },

        /** 
         * Returns the number of error correcting codewords based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectingCodeWords: function () {
            var index = (this.m_Version - 1) * 4;
            index += 1;
            var numberOfErrorCorrectingCodeWords = new Array(7, 10, 13, 17, 10, 16, 22, 28, 15, 26, 36, 44, 20, 36, 52, 64, 26, 48, 72, 88, 36, 64, 96, 112, 40, 72, 108, 130, 48, 88, 132, 156, 60, 110, 160, 192, 72, 130, 192, 224, 80, 150, 224, 264, 96, 176, 260, 308, 104, 198, 288, 352, 120, 216, 320, 384, 132, 240, 360, 432, 144, 280, 408, 480, 168, 308, 448, 532, 180, 338, 504, 588, 196, 364, 546, 650, 224, 416, 600, 700, 224, 442, 644, 750, 252, 476, 690, 816, 270, 504, 750, 900, 300, 560, 810, 960, 312, 588, 870, 1050, 336, 644, 952, 1110, 360, 700, 1020, 1200, 390, 728, 1050, 1260, 420, 784, 1140, 1350, 450, 812, 1200, 1440, 480, 868, 1290, 1530, 510, 924, 1350, 1620, 540, 980, 1440, 1710, 570, 1036, 1530, 1800, 570, 1064, 1590, 1890, 600, 1120, 1680, 1980, 630, 1204, 1770, 2100, 660, 1260, 1860, 2220, 720, 1316, 1950, 2310, 750, 1372, 2040, 2430);
            return numberOfErrorCorrectingCodeWords[index];
        },

        /** 
         * Returns the number of error correction blocks based on QR version.
         * @private
         */
        _getNumberOfErrorCorrectionBlocks: function () {
            var numberOfErrorCorrectionBlocks = new Array();
            switch (this.m_Version) {
                case 01:
                case 02:
                case 03:
                    numberOfErrorCorrectionBlocks[0] = 1;
                    break;
                case 04:
                case 05:
                    numberOfErrorCorrectionBlocks[0] = 2;
                    break;
                case 06:
                case 07:
                    numberOfErrorCorrectionBlocks[0] = 4;
                    break;
                case 08:
                    numberOfErrorCorrectionBlocks = new Array(2, 60, 38, 2, 61, 39);
                    break;
                case 09:
                    numberOfErrorCorrectionBlocks = new Array(3, 58, 36, 2, 59, 37);
                    break;
                case 10:
                    numberOfErrorCorrectionBlocks = new Array(4, 69, 43, 1, 70, 44);
                    break;
                case 11:
                    numberOfErrorCorrectionBlocks = new Array(1, 80, 50, 4, 81, 51);
                    break;
                case 12:
                    numberOfErrorCorrectionBlocks = new Array(6, 58, 36, 2, 59, 37);
                    break;
                case 13:
                    numberOfErrorCorrectionBlocks = new Array(8, 59, 37, 1, 60, 38);
                    break;
                case 14:
                    numberOfErrorCorrectionBlocks = new Array(4, 64, 40, 5, 65, 41);
                    break;
                case 15:
                    numberOfErrorCorrectionBlocks = new Array(5, 65, 41, 5, 66, 42);
                    break;
                case 16:
                    numberOfErrorCorrectionBlocks = new Array(7, 73, 45, 3, 74, 46);
                    break;
                case 17:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 1, 75, 47);
                    break;
                case 18:
                    numberOfErrorCorrectionBlocks = new Array(9, 69, 43, 4, 70, 44);
                    break;
                case 19:
                    numberOfErrorCorrectionBlocks = new Array(3, 70, 44, 11, 71, 45);
                    break;
                case 20:
                    numberOfErrorCorrectionBlocks = new Array(3, 67, 41, 13, 68, 42);
                    break;
                case 21:
                case 22:
                    numberOfErrorCorrectionBlocks[0] = 17;
                    break;
                case 23:
                    numberOfErrorCorrectionBlocks = new Array(4, 75, 47, 14, 76, 48);
                    break;
                case 24:
                    numberOfErrorCorrectionBlocks = new Array(6, 73, 45, 14, 74, 46);
                    break;
                case 25:
                    numberOfErrorCorrectionBlocks = new Array(8, 75, 47, 13, 76, 48);
                    break;
                case 26:
                    numberOfErrorCorrectionBlocks = new Array(19, 74, 46, 4, 75, 47);
                    break;
                case 27:
                    numberOfErrorCorrectionBlocks = new Array(22, 73, 45, 3, 74, 46);
                    break;
                case 28:
                    numberOfErrorCorrectionBlocks = new Array(3, 73, 45, 23, 74, 46);
                    break;
                case 29:
                    numberOfErrorCorrectionBlocks = new Array(21, 73, 45, 7, 74, 46);
                    break;
                case 30:
                    numberOfErrorCorrectionBlocks = new Array(19, 75, 47, 10, 76, 48);
                    break;
                case 31:
                    numberOfErrorCorrectionBlocks = new Array(2, 74, 46, 29, 75, 47);
                    break;
                case 32:
                    numberOfErrorCorrectionBlocks = new Array(10, 74, 46, 23, 75, 47);
                    break;
                case 33:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 21, 75, 47);
                    break;
                case 34:
                    numberOfErrorCorrectionBlocks = new Array(14, 74, 46, 23, 75, 47);
                    break;
                case 35:
                    numberOfErrorCorrectionBlocks = new Array(12, 75, 47, 26, 76, 48);
                    break;
                case 36:
                    numberOfErrorCorrectionBlocks = new Array(6, 75, 47, 34, 76, 48);
                    break;
                case 37:
                    numberOfErrorCorrectionBlocks = new Array(29, 74, 46, 14, 75, 47);
                    break;
                case 38:
                    numberOfErrorCorrectionBlocks = new Array(13, 74, 46, 32, 75, 47);
                    break;
                case 39:
                    numberOfErrorCorrectionBlocks = new Array(40, 75, 47, 7, 76, 48);
                    break;
                case 40:
                    numberOfErrorCorrectionBlocks = new Array(18, 75, 47, 31, 76, 48);
                    break;
            }

            return numberOfErrorCorrectionBlocks;
        },

        /** 
         * Returns the end values based on qr version
         * @private
         */
        _getEnd: function () {
            var endValues = new Array(208, 359, 567, 807, 1079, 1383, 1568, 1936, 2336, 2768, 3232, 3728, 4256, 4651, 5243, 5867, 6523, 7211, 7931, 8683, 9252, 10068, 10916, 11796, 12708, 13652, 14628, 15371, 16411, 17483, 18587, 19723, 20891, 22091, 23008, 24272, 25568, 26896, 28256, 29648);
            return endValues[this.m_Version - 1];
        },

        /** 
         * Returns the data capacity based on QR version
         * @private
         */
        _getDataCapacity: function () {
            var dataCapacityValues = new Array(26, 44, 70, 100, 134, 172, 196, 242, 292, 346, 404, 466, 532, 581, 655, 733, 815, 901, 991, 1085, 1156, 1258, 1364, 1474, 1588, 1706, 1828, 1921, 2051, 2185, 2323, 2465, 2611, 2761, 2876, 3034, 3196, 3362, 3532, 3706);
            return dataCapacityValues[this.m_Version - 1];
        },

        /** 
         * @private
         */
        _getFormatInformation: function () {
            var formatInformation = new Array(1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1);
            return formatInformation;
        },

        /** 
         * Returns the QR version
         * @private
         */
        _getVersionInformation: function () {
            var versionInformation = new Array();
            switch (this.m_Version) {
                case 07:
                    versionInformation = new Array(0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0);
                    break;
                case 08:
                    versionInformation = new Array(0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0);
                    break;
                case 09:
                    versionInformation = new Array(1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0);
                    break;
                case 10:
                    versionInformation = new Array(1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0);
                    break;
                case 11:
                    versionInformation = new Array(0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0);
                    break;
                case 12:
                    versionInformation = new Array(0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0);
                    break;
                case 13:
                    versionInformation = new Array(1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0);
                    break;
                case 14:
                    versionInformation = new Array(1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0);
                    break;
                case 15:
                    versionInformation = new Array(0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0);
                    break;
                case 16:
                    versionInformation = new Array(0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0);
                    break;
                case 17:
                    versionInformation = new Array(1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0);
                    break;
                case 18:
                    versionInformation = new Array(1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0);
                    break;
                case 19:
                    versionInformation = new Array(0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1, 0);
                    break;
                case 20:
                    versionInformation = new Array(0, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0);
                    break;
                case 21:
                    versionInformation = new Array(1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0);
                    break;
                case 22:
                    versionInformation = new Array(1, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0);
                    break;
                case 23:
                    versionInformation = new Array(0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0);
                    break;
                case 24:
                    versionInformation = new Array(0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0);
                    break;
                case 25:
                    versionInformation = new Array(1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0);
                    break;
                case 26:
                    versionInformation = new Array(1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0);
                    break;
                case 27:
                    versionInformation = new Array(0, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0);
                    break;
                case 28:
                    versionInformation = new Array(0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0);
                    break;
                case 29:
                    versionInformation = new Array(1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0);
                    break;
                case 30:
                    versionInformation = new Array(1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0);
                    break;
                case 31:
                    versionInformation = new Array(0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0);
                    break;
                case 32:
                    versionInformation = new Array(1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1);
                    break;
                case 33:
                    versionInformation = new Array(0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1);
                    break;
                case 34:
                    versionInformation = new Array(0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1);
                    break;
                case 35:
                    versionInformation = new Array(1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1);
                    break;
                case 36:
                    versionInformation = new Array(1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 1);
                    break;
                case 37:
                    versionInformation = new Array(0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1);
                    break;
                case 38:
                    versionInformation = new Array(0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 1);
                    break;
                case 39:
                    versionInformation = new Array(1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1);
                    break;
                case 40:
                    versionInformation = new Array(1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1);
                    break;
            }

            return versionInformation;
        },

        /** 
         * Returns the code points of the text
         * @private
         */
        _getStringCodePoints: (function () {
            function surrogatePairToCodePoint(charCode1, charCode2) {
                return ((charCode1 & 0x3FF) << 10) + (charCode2 & 0x3FF) + 0x10000;
            }

            /** Read string in character by character and create an array of code points.*/
            return function (str) {
                var codePoints = [], i = 0, charCode;
                while (i < str.length) {
                    charCode = str.charCodeAt(i);
                    if ((charCode & 0xF800) == 0xD800)
                        codePoints.push(surrogatePairToCodePoint(charCode, str.charCodeAt(++i)));
                    else
                        codePoints.push(charCode);
                    ++i;
                }
                return codePoints;
            }
        })(),

        /** 
         * Creates 2 dimensional array.
         * @private
         */
        _create2DArray: function (rows, columns) {
            rows = parseInt(rows);
            columns = parseInt(columns);
            var twoDArray = new Array(rows);

            for (var i = 0; i < rows; i++)
                twoDArray[i] = new Array(columns);

            return twoDArray;
        },

        /** 
         * Builds datamatrix barcode.
         * @private
         */
        _buildDataMatrix: function () {
            var symbolAttributes = [[10, 10, 1, 1, 3, 5, 1, 3], [12, 12, 1, 1, 5, 7, 1, 5], [14, 14, 1, 1, 8, 10, 1, 8], [16, 16, 1, 1, 12, 12, 1, 12], [18, 18, 1, 1, 18, 14, 1, 18], [20, 20, 1, 1, 22, 18, 1, 22], [22, 22, 1, 1, 30, 20, 1, 30], [24, 24, 1, 1, 36, 24, 1, 36], [26, 26, 1, 1, 44, 28, 1, 44], [32, 32, 2, 2, 62, 36, 1, 62], [36, 36, 2, 2, 86, 42, 1, 86], [40, 40, 2, 2, 114, 48, 1, 114], [44, 44, 2, 2, 144, 56, 1, 144], [48, 48, 2, 2, 174, 68, 1, 174], [52, 52, 2, 2, 204, 84, 2, 102], [64, 64, 4, 4, 280, 112, 2, 140], [72, 72, 4, 4, 368, 144, 4, 92], [80, 80, 4, 4, 456, 192, 4, 114], [88, 88, 4, 4, 576, 224, 4, 144], [96, 96, 4, 4, 696, 272, 4, 174], [104, 104, 4, 4, 816, 336, 6, 136], [120, 120, 6, 6, 1050, 408, 6, 175], [132, 132, 6, 6, 1304, 496, 8, 163], [144, 144, 6, 6, 1558, 620, 10, 156], [8, 18, 1, 1, 5, 7, 1, 5], [8, 32, 2, 1, 10, 11, 1, 10], [12, 26, 1, 1, 16, 14, 1, 16], [12, 36, 2, 1, 22, 18, 1, 22], [16, 36, 2, 1, 32, 24, 1, 32], [16, 48, 2, 1, 49, 28, 1, 49]];
            var codeWords = this._getStringCodePoints(this.Model.text);
            var correctionCodewords;
            var symAttr;

            /** ASCII Encoder.*/
            var encodedPoints = new Array(); var n = 0;
            for (var e = 0; e < codeWords.length; e++) {
                if (codeWords[e] < 127)
                    encodedPoints[n] = codeWords[e] + 1;
                else {
                    encodedPoints[n++] = 235;
                    encodedPoints[n] = codeWords[e] - 127;
                }
                n++;
            }

            /** Find symbol attribute.*/
            for (var e = 0; e < symbolAttributes.length; e++) {
                if (symbolAttributes[e][4] >= codeWords.length) {
                    symAttr = symbolAttributes[e];
                    break;
                }
            }

            if (symAttr[4] > encodedPoints.length) { //pad codewords
                var l = encodedPoints.length;
                var temp = new Array(symAttr[4]);
                var t = 0;

                for (var i = 0; i < l; i++)
                    temp[i] = encodedPoints[i];

                if (l < (symAttr[4] + 1))
                    temp[l] = 129; // denotes padding.

                l++;
                while (l < symAttr[4]) {	// more padding
                    var v = 129 + (((l + 1) * 149) % 253) + 1;
                    if (v > 254)
                        v -= 254;
                    temp[l++] = v;
                }

                encodedPoints = temp;
            }

            correctionCodewords = symAttr[5];
            errCorrArray = this._errorCorrection(correctionCodewords, encodedPoints);

            var finalCodeWord = encodedPoints.concat(errCorrArray);
            var dataMatrixArray = this._createMatrix(finalCodeWord, symAttr);

            var w = dataMatrixArray.length * this.Model.xDimension;
            var h = dataMatrixArray[0].length * this.Model.xDimension;

            /** Prepare the canvas.*/
            this.Model.width = w;
            this.canvasEl[0].setAttribute("width", this.Model.width);
            this.BarcodeEl.css("width", this.Model.width);

            this.contextEl.beginPath();
            this.contextEl.fillStyle = 'white';
            this.contextEl.fillRect(0, 0, w, h);
            this.contextEl.closePath();

            /** Draw DataMatrix.*/
            this._drawDataMatrix(dataMatrixArray, w, h);
        },

        /** 
         * Draws datamatrix barcode.
         * @private
         */
        _drawDataMatrix: function (dmArray, w, h) {
            this.Model.height = h;
            this.canvasEl[0].setAttribute("height", this.Model.height);
            this.BarcodeEl.css("height", this.Model.height);
            var dim = parseInt(this.Model.xDimension);
            var y = 0;

            this.contextEl.beginPath();

            for (var i = 0; i < dmArray.length; i++) {
                var x = 0;
                for (var j = 0; j < dmArray[i].length; j++) {
                    var colorBrush = 'white';
                    if (dmArray[i][j] == 1)
                        colorBrush = 'black';
                    this.contextEl.fillStyle = colorBrush;
                    this.contextEl.fillRect(x, y, dim, dim)
                    x += dim;
                }
                y += dim;
            }

            this.contextEl.closePath();
            this.contextEl.stroke();
        },

        /** 
         * Creates matrix.
         * @private
         */
        _createMatrix: function (codeWord, symAttr) {
            var x, y, NC, NR;
            var places = new Array();

            var W = symAttr[1], H = symAttr[0];
            var FW = W / symAttr[2], FH = H / symAttr[3];

            NC = W - 2 * (W / FW);
            NR = H - 2 * (H / FH);

            var places = new Array(NC * NR);
            this._ecc200placement(places, NR, NC);
            var matrix = new Array(W * H);

            for (var i = 0; i < matrix.length; i++)
                matrix[i] = 0;

            for (y = 0; y < H; y += FH) {
                for (x = 0; x < W; x++)
                    matrix[parseInt(y * W) + x] = 1;
                for (x = 0; x < W; x += 2)
                    matrix[parseInt((y + FH - 1) * W) + x] = 1;
            }
            for (x = 0; x < W; x += FW) {
                for (y = 0; y < H; y++)
                    matrix[parseInt(y * W) + x] = 1;
                for (y = 0; y < H; y += 2)
                    matrix[parseInt(y * W) + x + FW - 1] = 1;
            }
            for (y = 0; y < NR; y++) {
                for (x = 0; x < NC; x++) {
                    var v = places[((NR - y - 1) * NC) + x];
                    if (v == 1 || v > 7 && (codeWord[(v >> 3) - 1] & (1 << (v & 7))) != 0) {
                        var inde = parseInt((parseInt(1 + y + (2 * parseInt(y / (FH - 2)))) * W) + 1 + x + +(2 * parseInt(x / (FW - 2))));
                        matrix[inde] = 1;
                    }
                }
            }

            var _w = symAttr[1], _h = symAttr[0];

            var tempArray = this._create2DArray(_w, _h);

            for (var x1 = 0; x1 < _w; x1++) {
                for (var y1 = 0; y1 < _h; y1++) {
                    tempArray[x1][y1] = matrix[(_w * y1) + x1];
                }
            }

            /** Rotate the array left.*/
            var tempArray2 = this._create2DArray(_h, _w);

            for (var i = 0; i < _h; i++) {
                for (var j = 0; j < _w; j++) {
                    tempArray2[_h - 1 - i][j] = tempArray[j][i];
                }
            }

            var w = symAttr[0] + (this.Model.quietZone.top + this.Model.quietZone.bottom);
            var h = symAttr[1] + (this.Model.quietZone.left + this.Model.quietZone.right);
            var quietZone = this.Model.quietZone.all;

            var dataMatrixArray = this._create2DArray(w, h);

            for (var i = 0; i < w; i++)
                for (var j = 0; j < h; j++)
                    dataMatrixArray[i][j] = 0;

            for (var i = quietZone; i < w - quietZone; i++) {
                for (var j = quietZone; j < h - quietZone; j++) {
                    dataMatrixArray[i][j] = tempArray2[i - quietZone][j - quietZone];
                }
            }

            return dataMatrixArray;
        },

        /** 
         * @private
         */
        _ecc200placement: function (array, NR, NC) {
            var r, c, p;
            // invalidate
            for (r = 0; r < NR; r++)
                for (c = 0; c < NC; c++)
                    array[r * NC + c] = 0;
            // start
            p = 1;
            r = 4;
            c = 0;
            do {
                // check corner
                if (r == NR && !(c != 0))
                    this._ecc200placementcornerA(array, NR, NC, p++);
                if ((r == NR - 2) && !(c != 0) && ((NC % 4) != 0))
                    this._ecc200placementcornerB(array, NR, NC, p++);
                if (r == NR - 2 && !(c != 0) && (NC % 8) == 4)
                    this._ecc200placementcornerC(array, NR, NC, p++);
                if (r == NR + 4 && c == 2 && !((NC % 8) != 0))
                    this._ecc200placementcornerD(array, NR, NC, p++);
                // up/right
                do {
                    if (r < NR && c >= 0 && !(array[r * NC + c] != 0)) {
                        this._ecc200placementblock(array, NR, NC, r, c, p++);
                    }
                    r -= 2;
                    c += 2;
                }
                while (r >= 0 && c < NC);
                r++;
                c += 3;
                // down/left
                do {
                    if (r > -1) {
                        if (c < NC && !(array[r * NC + c] != 0)) {
                            this._ecc200placementblock(array, NR, NC, r, c, p++);
                        }
                    }
                    r += 2;
                    c -= 2;
                }
                while (r < NR && c >= 0);
                r += 3;
                c++;
            }
            while (r < NR || c < NC);
            // unfilled corner
            if (!(array[NR * NC - 1] != 0))
                array[NR * NC - 1] = array[NR * NC - NC - 2] = 1;
        },

        /** 
         * @private
         */
        _ecc200placementcornerA: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, 1, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 2, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerB: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 4, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 3);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerC: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 3, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 2, 0, p, 6);
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 2);
            this._ecc200placementbit(array, NR, NC, 2, NC - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, 3, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementcornerD: function (array, NR, NC, p) {
            this._ecc200placementbit(array, NR, NC, NR - 1, 0, p, 7);
            this._ecc200placementbit(array, NR, NC, NR - 1, NC - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, 0, NC - 3, p, 5);
            this._ecc200placementbit(array, NR, NC, 0, NC - 2, p, 4);
            this._ecc200placementbit(array, NR, NC, 0, NC - 1, p, 3);
            this._ecc200placementbit(array, NR, NC, 1, NC - 3, p, 2);
            this._ecc200placementbit(array, NR, NC, 1, NC - 2, p, 1);
            this._ecc200placementbit(array, NR, NC, 1, NC - 1, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementblock: function (array, NR, NC, r, c, p) {
            this._ecc200placementbit(array, NR, NC, r - 2, c - 2, p, 7);
            this._ecc200placementbit(array, NR, NC, r - 2, c - 1, p, 6);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 2, p, 5);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 1, p, 4);
            this._ecc200placementbit(array, NR, NC, r - 1, c - 0, p, 3);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 2, p, 2);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 1, p, 1);
            this._ecc200placementbit(array, NR, NC, r - 0, c - 0, p, 0);
        },

        /** 
         * @private
         */
        _ecc200placementbit: function (array, NR, NC, r, c, p, b) {
            if (r < 0) {
                r += NR;
                c += 4 - ((NR + 4) % 8);
            }
            if (c < 0) {
                c += NC;
                r += 4 - ((NC + 4) % 8);
            }
            array[r * NC + c] = (p << 3) + b;
        },

        /** 
         * Adds error correction codeword.
         * @private
         */
        _errorCorrection: function (num, codeWords) {
            var i, k, index = 1;
            var rsPolynomial;
            var temp = new Array(num + 1);

            /** Create log array.*/
            {
                var m_log = new Array(256);
                var m_aLog = new Array(256);

                m_log[0] = -255;
                m_aLog[0] = 1;

                for (var i = 1; i <= 255; i++) {
                    m_aLog[i] = m_aLog[i - 1] * 2;

                    if (m_aLog[i] >= 256)
                        m_aLog[i] = m_aLog[i] ^ 301;

                    m_log[m_aLog[i]] = i;
                }
            }

            /** CreateRSPolynomial.*/
            temp[0] = 1;
            for (i = 1; i <= num; i++) {
                temp[i] = 1;
                for (k = i - 1; k > 0; k--) {
                    if (temp[k] != 0)
                        temp[k] = m_aLog[(m_log[temp[k]] + index) % 255];
                    temp[k] ^= temp[k - 1];
                }
                temp[0] = m_aLog[(m_log[temp[0]] + index) % 255];
                index++;
            }

            rsPolynomial = new Array(num);
            for (var e = 0; e < rsPolynomial.length; e++)
                rsPolynomial[e] = temp[e];

            var t = 0;

            /** Create error correction array.*/
            var errCorrArray = new Array(num);

            for (var i = 0; i < num; i++)
                errCorrArray[i] = 0;

            for (var i = 0; i < codeWords.length; i++) {
                t = errCorrArray[num - 1] ^ codeWords[i];
                for (var j = num - 1; j > 0; j--) {
                    if (t != 0 && rsPolynomial[j] != 0)
                        errCorrArray[j] = (errCorrArray[j - 1] ^ m_aLog[(m_log[t] + m_log[rsPolynomial[j]]) % 255]);
                    else
                        errCorrArray[j] = errCorrArray[j - 1];
                }
                if (t != 0 && rsPolynomial[0] != 0)
                    errCorrArray[0] = (m_aLog[(m_log[t] + m_log[rsPolynomial[0]]) % 255]);
                else
                    errCorrArray[0] = 0;
            }

            errCorrArray = errCorrArray.reverse();
            return errCorrArray;
        },

        /** 
         * Validates the text against the barcode type.
         * @private
         */
        _validateText: function (str, exp) {
            var patt = new RegExp(exp);
            return patt.test(str);
        },

        /** 
         * @private
         */
        _getCharWidth: function (encodedText, symbolTable) {
            var charW = 0;
            var charArray = encodedText.split("");
            for (var e = 0; e < charArray.length; e++)
                charW = charW + intercharacterGap + this._getWidth(symbolTable, charArray[e]);

            return charW;
        },

        /** 
         * @private
         */
        _getWidth: function (symbolTable, c) {
            var check = false;
            var wid = 0;
            for (var e = 0; e < symbolTable.length; e++) {
                if (symbolTable[e][0] == c) {
                    var bars = symbolTable[e][2];
                    for (var i = 0; i < bars.length; i++) {
                        if (check == false) {
                            if (bars.length % 2 != 0)
                                continuous = true;
                        }

                        wid = wid + (bars[i] * this.Model.narrowBarWidth);
                        check = true;
                    }
                    return wid;
                }
            }
            return wid;
        },

        /** 
         * Draws 1 dimensional barcode.
         * @private
         */
        _draw1DBarcode: function (encodedText, symbolTable) {
            if (symbolTable.length > 0) {
                var charArray = encodedText.split("");
                var left = this.Model.quietZone.left;
                var top = this.Model.quietZone.top;
                this.Model.height = top + this.Model.barHeight + this.Model.quietZone.bottom;

                var textSize = parseInt(this.textFont.size.replace('px', ''));
                if (this.Model.displayText) {
                    this.Model.height += (textSize + this.Model.barcodeToTextGapHeight);
                }
                this.canvasEl[0].setAttribute("height", this.Model.height);
                this.BarcodeEl.css("height", this.Model.height);
                var wid = this.Model.width;
                var hei = this.Model.height;
              

                this.contextEl.fillStyle = 'white';
                this.contextEl.fillRect(0, 0, wid, hei);

                for (var e = 0; e < charArray.length; e++) {
                    for (var h = 0; h < symbolTable.length; h++) {
                        if (symbolTable[h][0] == charArray[e]) {
                            var bars = symbolTable[h][2];
                            for (var i = 0; i < bars.length; i++) {
                                var barWidth = bars[i] * this.Model.narrowBarWidth;
                                if (i % 2 == 0) {
                                    this.contextEl.beginPath();
                                    this.contextEl.fillStyle = this.Model.darkBarColor;
                                    this.contextEl.fillRect(left, top, barWidth, this.Model.barHeight)
                                    this.contextEl.closePath();                                
                                }
                                else {
                                    this.contextEl.beginPath();
                                    this.contextEl.fillStyle = this.model.lightBarColor;
                                    this.contextEl.fillRect(left, top, barWidth, this.model.barHeight);
                                    this.contextEl.closePath();                                
                                }
                                left += barWidth;
                            }
                            if (bars.length % 2 != 0)
                                left += intercharacterGap;
                        }
                    }
                }

                

                this.Model.width = left += (this.Model.quietZone.right);

                /** Draws text for 1D barcodes.*/
                if (this.Model.displayText) {
                    var txtF = (this.textFont.fontStyle.toLowerCase() == "regular") ? "" : +this.textFont.fontStyle.toLowerCase() + " ";
                    txtF += this.textFont.size + " " + this.textFont.fontFamily.toLowerCase() + ",sans-serif";
                    this.contextEl.font = txtF;

                    var textWidth = 0;
                    textWidth = this.contextEl.measureText(this.Model.text).width;
                    if (this.Model.width < textWidth) {
                        this.Model.width = textWidth;
                        this.canvasEl[0].setAttribute("width", this.Model.width);
                        this.BarcodeEl.css("width", this.Model.width);
                    }

                    this.contextEl.beginPath();
                    this.contextEl.fillStyle = this.Model.textColor;
                    this.contextEl.textAlign = "center";
                    hei = hei - textSize + textSize / 2;
                    this.contextEl.fillText(this.Model.text, (this.Model.width / 2), hei);
                    this.contextEl.closePath();
                    this.contextEl.stroke();
                }
            }
        }
    });

    /**
     * Enum for Symbol type.
	 * @enum {string}
     * @global
     */
    ej.datavisualization.Barcode.SymbologyType = {
        /** Specifies Code 39 Barcode. */
        code39: "code39",
        /** Specifies Code 39 Extended Barcode. */
        code39Extended: "code39extended",
        /** Specifies Code 11 Barcode. */
        code11: "code11",
        /** Specifies Codabar Barcode. */
        codabar: "codabar",
        /** Specifies Code 32 Barcode. */
        code32: "code32",
        /** Specifies Code 93 Barcode. */
        code93: "code93",
        /** Specifies Code 93 Extended Barcode. */
        code93Extended: "code93extended",
        /** Specifies Code 128 A Barcode. */
        code128A: "code128a",
        /** Specifies Code 128 B Barcode. */
        code128B: "code128b",
        /** Specifies Code 128 C Barcode. */
        code128C: "code128c",
        /** Specifies DataMatrix Barcode. */
        dataMatrix: "datamatrix",
        /** Specifies QR Barcode. */
        qrBarcode: "qrbarcode"
    };

    /**
     * Enum for Symbol type.
     * @readonly
     * @private
     */
    ej.datavisualization.Barcode.symbologySettings = {
        /** Specifies symbology settinngs for the Code 39 Barcode. */
        code39: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 39 Extended Barcode. */
        code39extended: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x00-\x7F]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["A", 10, [3, 1, 1, 1, 1, 3, 1, 1, 3]], ["B", 11, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 12, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 13, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["E", 14, [3, 1, 1, 1, 3, 3, 1, 1, 1]], ["F", 15, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 16, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 17, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["I", 18, [1, 1, 3, 1, 1, 3, 3, 1, 1]], ["J", 19, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 20, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 21, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 22, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 23, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["O", 24, [3, 1, 1, 1, 3, 1, 1, 3, 1]], ["P", 25, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 26, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 27, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 28, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 29, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 30, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 31, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 32, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 33, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 34, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 35, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["-", 36, [1, 3, 1, 1, 1, 1, 3, 1, 3]], [".", 37, [3, 3, 1, 1, 1, 1, 3, 1, 1]], [" ", 38, [1, 3, 3, 1, 1, 1, 3, 1, 1]], ["$", 39, [1, 3, 1, 3, 1, 3, 1, 1, 1]], ["/", 40, [1, 3, 1, 3, 1, 1, 1, 3, 1]], ["+", 41, [1, 3, 1, 1, 1, 3, 1, 3, 1]], ["%", 42, [1, 1, 1, 3, 1, 3, 1, 3, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]],
            extendedCodes: [['\0', ['%', 'U']], [String.fromCharCode(parseInt('0001', 16)), ['$', 'A']], [String.fromCharCode(parseInt('0002', 16)), ['$', 'B']], [String.fromCharCode(parseInt('0003', 16)), ['$', 'C']], [String.fromCharCode(parseInt('0004', 16)), ['$', 'D']], [String.fromCharCode(parseInt('0005', 16)), ['$', 'E']], [String.fromCharCode(parseInt('0006', 16)), ['$', 'F']], ['\a', ['$', 'G']], ['\b', ['$', 'H']], ['\t', ['$', 'I']], ['\n', ['$', 'J']], ['\v', ['$', 'K']], ['\f', ['$', 'L']], ['\r', ['$', 'M']], [String.fromCharCode(parseInt('000e', 16)), ['$', 'N']], [String.fromCharCode(parseInt('000f', 16)), ['$', 'O']], [String.fromCharCode(parseInt('0010', 16)), ['$', 'P']], [String.fromCharCode(parseInt('0011', 16)), ['$', 'Q']], [String.fromCharCode(parseInt('0012', 16)), ['$', 'R']], [String.fromCharCode(parseInt('0013', 16)), ['$', 'S']], [String.fromCharCode(parseInt('0014', 16)), ['$', 'T']], [String.fromCharCode(parseInt('0015', 16)), ['$', 'U']], [String.fromCharCode(parseInt('0016', 16)), ['$', 'V']], [String.fromCharCode(parseInt('0017', 16)), ['$', 'W']], [String.fromCharCode(parseInt('0018', 16)), ['$', 'X']], [String.fromCharCode(parseInt('0019', 16)), ['$', 'Y']], [String.fromCharCode(parseInt('001a', 16)), ['$', 'Z']], [String.fromCharCode(parseInt('001b', 16)), ['%', 'A']], [String.fromCharCode(parseInt('001c', 16)), ['%', 'B']], [String.fromCharCode(parseInt('001d', 16)), ['%', 'C']], [String.fromCharCode(parseInt('001e', 16)), ['%', 'D']], [String.fromCharCode(parseInt('001f', 16)), ['%', 'E']], [' ', [' ']], ['!', ['/', 'A']], ['"', ['/', 'B']], ['#', ['/', 'C']], ['$', ['/', 'D']], ['%', ['/', 'E']], ['&', ['/', 'F']], ['\'', ['/', 'G']], ['(', ['/', 'H']], [')', ['/', 'I']], ['*', ['/', 'J']], ['+', ['/', 'K']], [',', ['/', 'L']], ['-', ['/', 'M']], ['.', ['/', 'N']], ['/', ['/', 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', ['/', 'Z']], [';', ['%', 'F']], ['<', ['%', 'G']], ['=', ['%', 'H']], ['>', ['%', 'I']], ['?', ['%', 'J']], ['@', ['%', 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', ['%', 'K']], ['\\', ['%', 'L']], [']', ['%', 'M']], ['^', ['%', 'N']], ['_', ['%', 'O']], ['`', ['%', 'W']], ['a', ['+', 'A']], ['b', ['+', 'B']], ['c', ['+', 'C']], ['d', ['+', 'D']], ['e', ['+', 'E']], ['f', ['+', 'F']], ['g', ['+', 'G']], ['h', ['+', 'H']], ['i', ['+', 'I']], ['j', ['+', 'J']], ['k', ['+', 'K']], ['l', ['+', 'L']], ['m', ['+', 'M']], ['n', ['+', 'N']], ['o', ['+', 'O']], ['p', ['+', 'P']], ['q', ['+', 'Q']], ['r', ['+', 'R']], ['s', ['+', 'S']], ['t', ['+', 'T']], ['u', ['+', 'U']], ['v', ['+', 'V']], ['w', ['+', 'W']], ['x', ['+', 'X']], ['y', ['+', 'Y']], ['z', ['+', 'Z']], ['{', ['%', 'P']], ['|', ['%', 'Q']], ['}', ['%', 'R']], ['~', ['%', 'S']], [String.fromCharCode(parseInt('007f', 16)), ['%', 'T']]]
        },
        /** Specifies symbology settings for the Code 11 Barcode. */
        code11: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[0-9\-]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 2]], ["1", 1, [2, 1, 1, 1, 2]], ["2", 2, [1, 2, 1, 1, 2]], ["3", 3, [2, 2, 1, 1, 1]], ["4", 4, [1, 1, 2, 1, 2]], ["5", 5, [2, 1, 2, 1, 1]], ["6", 6, [1, 2, 2, 1, 1]], ["7", 7, [1, 1, 1, 2, 2]], ["8", 8, [2, 1, 1, 2, 1]], ["9", 9, [2, 1, 1, 1, 1]], ["-", 10, [1, 1, 2, 1, 1]], ["*", 0, [1, 1, 2, 2, 1]]]
        },
        /** Specifies symbology settings for the Codabar Barcode. */
        codabar: {
            startSymbol: 'A',
            stopSymbol: 'B',
            validatorExp: "^[0-9\-\.\$\/\:\+]*$",
            symbolTable: [["0", 0, [1, 1, 1, 1, 1, 2, 2]], ["1", 0, [1, 1, 1, 1, 2, 2, 1]], ["2", 0, [1, 1, 1, 2, 1, 1, 2]], ["3", 0, [2, 2, 1, 1, 1, 1, 1]], ["4", 0, [1, 1, 2, 1, 1, 2, 1]], ["5", 0, [2, 1, 1, 1, 1, 2, 1]], ["6", 0, [1, 2, 1, 1, 1, 1, 2]], ["7", 0, [1, 2, 1, 1, 2, 1, 1]], ["8", 0, [1, 2, 2, 1, 1, 1, 1]], ["9", 0, [2, 1, 1, 2, 1, 1, 1]], ["-", 0, [1, 1, 1, 2, 2, 1, 1]], ["$", 0, [1, 1, 2, 2, 1, 1, 1]], [":", 0, [2, 1, 1, 1, 2, 1, 2]], ["/", 0, [2, 1, 2, 1, 1, 1, 2]], [".", 0, [2, 1, 2, 1, 2, 1, 1]], ["+", 0, [1, 1, 2, 1, 2, 1, 2]], ["A", 0, [1, 1, 2, 2, 1, 2, 1]], ["B", 0, [1, 1, 1, 2, 1, 2, 2]]]
        },
        /** Specifies symbology settings for the Code 32 Barcode. */
        code32: {
            startSymbol: '*',
            stopSymbol: '*',
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\*\.\$\/\+\%]+$",
            symbolTable: [["0", 0, [1, 1, 1, 3, 3, 1, 3, 1, 1]], ["1", 1, [3, 1, 1, 3, 1, 1, 1, 1, 3]], ["2", 2, [1, 1, 3, 3, 1, 1, 1, 1, 3]], ["3", 3, [3, 1, 3, 3, 1, 1, 1, 1, 1]], ["4", 4, [1, 1, 1, 3, 3, 1, 1, 1, 3]], ["5", 5, [3, 1, 1, 3, 3, 1, 1, 1, 1]], ["6", 6, [1, 1, 3, 3, 3, 1, 1, 1, 1]], ["7", 7, [1, 1, 1, 3, 1, 1, 3, 1, 3]], ["8", 8, [3, 1, 1, 3, 1, 1, 3, 1, 1]], ["9", 9, [1, 1, 3, 3, 1, 1, 3, 1, 1]], ["B", 10, [1, 1, 3, 1, 1, 3, 1, 1, 3]], ["C", 11, [3, 1, 3, 1, 1, 3, 1, 1, 1]], ["D", 12, [1, 1, 1, 1, 3, 3, 1, 1, 3]], ["F", 13, [1, 1, 3, 1, 3, 3, 1, 1, 1]], ["G", 14, [1, 1, 1, 1, 1, 3, 3, 1, 3]], ["H", 15, [3, 1, 1, 1, 1, 3, 3, 1, 1]], ["J", 0x10, [1, 1, 1, 1, 3, 3, 3, 1, 1]], ["K", 0x11, [3, 1, 1, 1, 1, 1, 1, 3, 3]], ["L", 0x12, [1, 1, 3, 1, 1, 1, 1, 3, 3]], ["M", 0x13, [3, 1, 3, 1, 1, 1, 1, 3, 1]], ["N", 20, [1, 1, 1, 1, 3, 1, 1, 3, 3]], ["P", 0x15, [1, 1, 3, 1, 3, 1, 1, 3, 1]], ["Q", 0x16, [1, 1, 1, 1, 1, 1, 3, 3, 3]], ["R", 0x17, [3, 1, 1, 1, 1, 1, 3, 3, 1]], ["S", 0x18, [1, 1, 3, 1, 1, 1, 3, 3, 1]], ["T", 0x19, [1, 1, 1, 1, 3, 1, 3, 3, 1]], ["U", 0x1a, [3, 3, 1, 1, 1, 1, 1, 1, 3]], ["V", 0x1b, [1, 3, 3, 1, 1, 1, 1, 1, 3]], ["W", 0x1c, [3, 3, 3, 1, 1, 1, 1, 1, 1]], ["X", 0x1d, [1, 3, 1, 1, 3, 1, 1, 1, 3]], ["Y", 30, [3, 3, 1, 1, 3, 1, 1, 1, 1]], ["Z", 0x1f, [1, 3, 3, 1, 3, 1, 1, 1, 1]], ["*", 0, [1, 3, 1, 1, 3, 1, 3, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Barcode. */
        code93: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x41-\x5A\x30-\x39\x20\-\.\$\/\+\%\ ]+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]]
        },
        /** Specifies symbology settings for the Code 93 Extended Barcode. */
        code93extended: {
            startSymbol: '*',
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F\x00fb\x00fd\x00fe\'�'\'�'\'�']+$",
            symbolTable: [["0", 0, [1, 3, 1, 1, 1, 2]], ["1", 1, [1, 1, 1, 2, 1, 3]], ["2", 2, [1, 1, 1, 3, 1, 2]], ["3", 3, [1, 1, 1, 4, 1, 1]], ["4", 4, [1, 2, 1, 1, 1, 2]], ["5", 5, [1, 2, 1, 2, 1, 2]], ["6", 6, [1, 2, 1, 3, 1, 1]], ["7", 7, [1, 1, 1, 1, 1, 4]], ["8", 8, [1, 3, 1, 2, 1, 1]], ["9", 9, [1, 4, 1, 1, 1, 1]], ["A", 10, [2, 1, 1, 1, 1, 3]], ["B", 11, [2, 1, 1, 2, 1, 2]], ["C", 12, [2, 1, 1, 3, 1, 1]], ["D", 13, [2, 2, 1, 1, 1, 2]], ["E", 14, [2, 2, 1, 2, 1, 1]], ["F", 15, [2, 3, 1, 1, 1, 1]], ["G", 0x10, [1, 1, 2, 1, 1, 3]], ["H", 0x11, [1, 1, 2, 2, 1, 2]], ["I", 0x12, [1, 1, 2, 3, 1, 1]], ["J", 0x13, [1, 2, 2, 1, 1, 2]], ["K", 20, [1, 3, 2, 1, 1, 1]], ["L", 0x15, [1, 1, 1, 1, 2, 3]], ["M", 0x16, [1, 1, 1, 2, 2, 2]], ["N", 0x17, [1, 1, 1, 3, 2, 1]], ["O", 0x18, [1, 2, 1, 1, 2, 2]], ["P", 0x19, [1, 3, 1, 1, 2, 1]], ["Q", 0x1a, [2, 1, 2, 1, 1, 2]], ["R", 0x1b, [2, 1, 2, 2, 1, 1]], ["S", 0x1c, [2, 1, 1, 1, 2, 2]], ["T", 0x1d, [2, 1, 1, 2, 2, 1]], ["U", 30, [2, 2, 1, 1, 2, 1]], ["V", 0x1f, [2, 2, 2, 1, 1, 1]], ["W", 0x20, [1, 1, 2, 1, 2, 2]], ["X", 0x21, [1, 1, 2, 2, 2, 1]], ["Y", 0x22, [1, 2, 2, 1, 2, 1]], ["Z", 0x23, [1, 2, 3, 1, 1, 1]], ["-", 0x24, [1, 2, 1, 1, 3, 1]], [".", 0x25, [3, 1, 1, 1, 1, 2]], [" ", 0x26, [3, 1, 1, 2, 1, 1]], ["$", 0x27, [3, 2, 1, 1, 1, 1]], ["/", 40, [1, 1, 2, 1, 3, 1]], ["+", 0x29, [1, 1, 3, 1, 2, 1]], ["%", 0x2a, [2, 1, 1, 1, 3, 1]], ["*", 0, [1, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00ff', 16)), 0, [1, 1, 1, 1, 4, 1, 1]], [String.fromCharCode(parseInt('00fb', 16)), 0x2b, [1, 2, 1, 2, 2, 0]], [String.fromCharCode(parseInt('00fc', 16)), 0x2c, [3, 1, 2, 1, 1, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x2d, [3, 1, 1, 1, 2, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x2e, [1, 2, 2, 2, 1, 1]]],
            extendedCodes: [['\0', [String.fromCharCode(parseInt('00fc', 16)), 'U']], [String.fromCharCode(parseInt('0001', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'A']], [String.fromCharCode(parseInt('0002', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'B']], [String.fromCharCode(parseInt('0003', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'C']], [String.fromCharCode(parseInt('0004', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'D']], [String.fromCharCode(parseInt('0005', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'E']], [String.fromCharCode(parseInt('0006', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'F']], ['\a', [String.fromCharCode(parseInt('00fb', 16)), 'G']], ['\b', [String.fromCharCode(parseInt('00fb', 16)), 'H']], ['\t', [String.fromCharCode(parseInt('00fb', 16)), 'I']], ['\n', [String.fromCharCode(parseInt('00fb', 16)), 'J']], ['\v', [String.fromCharCode(parseInt('00fb', 16)), 'K']], ['\f', [String.fromCharCode(parseInt('00fb', 16)), 'L']], ['\r', [String.fromCharCode(parseInt('00fb', 16)), 'M']], [String.fromCharCode(parseInt('000e', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'N']], [String.fromCharCode(parseInt('000f', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'O']], [String.fromCharCode(parseInt('0010', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'P']], [String.fromCharCode(parseInt('0011', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Q']], [String.fromCharCode(parseInt('0012', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'R']], [String.fromCharCode(parseInt('0013', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'S']], [String.fromCharCode(parseInt('0014', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'T']], [String.fromCharCode(parseInt('0015', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'U']], [String.fromCharCode(parseInt('0016', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'V']], [String.fromCharCode(parseInt('0017', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'W']], [String.fromCharCode(parseInt('0018', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'X']], [String.fromCharCode(parseInt('0019', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Y']], [String.fromCharCode(parseInt('001a', 16)), [String.fromCharCode(parseInt('00fb', 16)), 'Z']], [String.fromCharCode(parseInt('001b', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'A']], [String.fromCharCode(parseInt('001c', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'B']], [String.fromCharCode(parseInt('001d', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'C']], [String.fromCharCode(parseInt('001e', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'D']], [String.fromCharCode(parseInt('001f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'E']], [' ', [' ']], ['!', [String.fromCharCode(parseInt('00fd', 16)), 'A']], ['"', [String.fromCharCode(parseInt('00fd', 16)), 'B']], ['#', [String.fromCharCode(parseInt('00fd', 16)), 'C']], ['$', [String.fromCharCode(parseInt('00fd', 16)), 'D']], ['%', [String.fromCharCode(parseInt('00fd', 16)), 'E']], ['&', [String.fromCharCode(parseInt('00fd', 16)), 'F']], ['\'', [String.fromCharCode(parseInt('00fd', 16)), 'G']], ['(', [String.fromCharCode(parseInt('00fd', 16)), 'H']], [')', [String.fromCharCode(parseInt('00fd', 16)), 'I']], ['*', [String.fromCharCode(parseInt('00fd', 16)), 'J']], ['+', [String.fromCharCode(parseInt('00fd', 16)), 'K']], [',', [String.fromCharCode(parseInt('00fd', 16)), 'L']], ['-', [String.fromCharCode(parseInt('00fd', 16)), 'M']], ['.', [String.fromCharCode(parseInt('00fd', 16)), 'N']], ['/', [String.fromCharCode(parseInt('00fd', 16)), 'O']], ['0', ['0']], ['1', ['1']], ['2', ['2']], ['3', ['3']], ['4', ['4']], ['5', ['5']], ['6', ['6']], ['7', ['7']], ['8', ['8']], ['9', ['9']], [':', [String.fromCharCode(parseInt('00fd', 16)), 'Z']], [';', [String.fromCharCode(parseInt('00fc', 16)), 'F']], ['<', [String.fromCharCode(parseInt('00fc', 16)), 'G']], ['=', [String.fromCharCode(parseInt('00fc', 16)), 'H']], ['>', [String.fromCharCode(parseInt('00fc', 16)), 'I']], ['?', [String.fromCharCode(parseInt('00fc', 16)), 'J']], ['@', [String.fromCharCode(parseInt('00fc', 16)), 'V']], ['A', ['A']], ['B', ['B']], ['C', ['C']], ['D', ['D']], ['E', ['E']], ['F', ['F']], ['G', ['G']], ['H', ['H']], ['I', ['I']], ['J', ['J']], ['K', ['K']], ['L', ['L']], ['M', ['M']], ['N', ['N']], ['O', ['O']], ['P', ['P']], ['Q', ['Q']], ['R', ['R']], ['S', ['S']], ['T', ['T']], ['U', ['U']], ['V', ['V']], ['W', ['W']], ['X', ['X']], ['Y', ['Y']], ['Z', ['Z']], ['[', [String.fromCharCode(parseInt('00fc', 16)), 'K']], ['\\', [String.fromCharCode(parseInt('00fc', 16)), 'L']], [']', [String.fromCharCode(parseInt('00fc', 16)), 'M']], ['^', [String.fromCharCode(parseInt('00fc', 16)), 'N']], ['_', [String.fromCharCode(parseInt('00fc', 16)), 'O']], ['`', [String.fromCharCode(parseInt('00fc', 16)), 'W']], ['a', [String.fromCharCode(parseInt('00fe', 16)), 'A']], ['b', [String.fromCharCode(parseInt('00fe', 16)), 'B']], ['c', [String.fromCharCode(parseInt('00fe', 16)), 'C']], ['d', [String.fromCharCode(parseInt('00fe', 16)), 'D']], ['e', [String.fromCharCode(parseInt('00fe', 16)), 'E']], ['f', [String.fromCharCode(parseInt('00fe', 16)), 'F']], ['g', [String.fromCharCode(parseInt('00fe', 16)), 'G']], ['h', [String.fromCharCode(parseInt('00fe', 16)), 'H']], ['i', [String.fromCharCode(parseInt('00fe', 16)), 'I']], ['j', [String.fromCharCode(parseInt('00fe', 16)), 'J']], ['k', [String.fromCharCode(parseInt('00fe', 16)), 'K']], ['l', [String.fromCharCode(parseInt('00fe', 16)), 'L']], ['m', [String.fromCharCode(parseInt('00fe', 16)), 'M']], ['n', [String.fromCharCode(parseInt('00fe', 16)), 'N']], ['o', [String.fromCharCode(parseInt('00fe', 16)), 'O']], ['p', [String.fromCharCode(parseInt('00fe', 16)), 'P']], ['q', [String.fromCharCode(parseInt('00fe', 16)), 'Q']], ['r', [String.fromCharCode(parseInt('00fe', 16)), 'R']], ['s', [String.fromCharCode(parseInt('00fe', 16)), 'S']], ['t', [String.fromCharCode(parseInt('00fe', 16)), 'T']], ['u', [String.fromCharCode(parseInt('00fe', 16)), 'U']], ['v', [String.fromCharCode(parseInt('00fe', 16)), 'V']], ['w', [String.fromCharCode(parseInt('00fe', 16)), 'W']], ['x', [String.fromCharCode(parseInt('00fe', 16)), 'X']], ['y', [String.fromCharCode(parseInt('00fe', 16)), 'Y']], ['z', [String.fromCharCode(parseInt('00fe', 16)), 'Z']], ['{', [String.fromCharCode(parseInt('00fc', 16)), 'P']], ['|', [String.fromCharCode(parseInt('00fc', 16)), 'Q']], ['}', [String.fromCharCode(parseInt('00fc', 16)), 'R']], ['~', [String.fromCharCode(parseInt('00fc', 16)), 'S']], [String.fromCharCode(parseInt('007f', 16)), [String.fromCharCode(parseInt('00fc', 16)), 'T']]],
        },
        /** Specifies symbology settings for the Code 128 A Barcode. */
        code128a: {
            startSymbol: String.fromCharCode(parseInt('00f9', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[\0\x0001\x0002\x0003\x0004\x0005\x0006\a\b\t\n\v\f\r\x000e\x000f\x0010\x0011\x0012\x0013\x0014\x0015\x0016\x0017\x0018\x0019\x001a\x001b\x001c\x001d\x001e\x001f !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_\x00f0\x00f1\x00f2\x00f3\x00f4]",
            symbolTable: [["\0", 0x40, [1, 1, 1, 4, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 0x41, [1, 2, 1, 1, 2, 4]], [String.fromCharCode(parseInt('0002', 16)), 0x42, [1, 2, 1, 4, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 0x43, [1, 4, 1, 1, 2, 2]], [String.fromCharCode(parseInt('0004', 16)), 0x44, [1, 4, 1, 2, 2, 1]], [String.fromCharCode(parseInt('0005', 16)), 0x45, [1, 1, 2, 2, 1, 4]], [String.fromCharCode(parseInt('0006', 16)), 70, [1, 1, 2, 4, 1, 2]], ["\a", 0x47, [1, 2, 2, 1, 1, 4]], ["\b", 0x48, [1, 2, 2, 4, 1, 1]], ["\t", 0x49, [1, 4, 2, 1, 1, 2]], ["\n", 0x4a, [1, 4, 2, 2, 1, 1]], ["\v", 0x4b, [2, 4, 1, 2, 1, 1]], ["\f", 0x4c, [2, 2, 1, 1, 1, 4]], ["\r", 0x4d, [4, 1, 3, 1, 1, 1]], [String.fromCharCode(parseInt('000e', 16)), 0x4e, [2, 4, 1, 1, 1, 2]], [String.fromCharCode(parseInt('000f', 16)), 0x4f, [1, 3, 4, 1, 1, 1]], [String.fromCharCode(parseInt('0010', 16)), 80, [1, 1, 1, 2, 4, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x51, [1, 2, 1, 1, 4, 2]], [String.fromCharCode(parseInt('0012', 16)), 0x52, [1, 2, 1, 2, 4, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x53, [1, 1, 4, 2, 1, 2]], [String.fromCharCode(parseInt('0014', 16)), 0x54, [1, 2, 4, 1, 1, 2]], [String.fromCharCode(parseInt('0015', 16)), 0x55, [1, 2, 4, 2, 1, 1]], [String.fromCharCode(parseInt('0016', 16)), 0x56, [4, 1, 1, 2, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x57, [4, 2, 1, 1, 1, 2]], [String.fromCharCode(parseInt('0018', 16)), 0x58, [4, 2, 1, 2, 1, 1]], [String.fromCharCode(parseInt('0019', 16)), 0x59, [2, 1, 2, 1, 4, 1]], [String.fromCharCode(parseInt('001a', 16)), 90, [2, 1, 4, 1, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x5b, [4, 1, 2, 1, 2, 1]], [String.fromCharCode(parseInt('001c', 16)), 0x5c, [1, 1, 1, 1, 4, 3]], [String.fromCharCode(parseInt('001d', 16)), 0x5d, [1, 1, 1, 3, 4, 1]], [String.fromCharCode(parseInt('001e', 16)), 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('001f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f9', 16)), 0x67, [2, 1, 1, 4, 1, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 B Barcode. */
        code128b: {
            startSymbol: String.fromCharCode(parseInt('00fd', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "^[\x00-\x7F]",
            symbolTable: [[" ", 0, [2, 1, 2, 2, 2, 2]], ["!", 1, [2, 2, 2, 1, 2, 2]], ["\"", 2, [2, 2, 2, 2, 2, 1]], ["#", 3, [1, 2, 1, 2, 2, 3]], ["$", 4, [1, 2, 1, 3, 2, 2]], ["%", 5, [1, 3, 1, 2, 2, 2]], ["&", 6, [1, 2, 2, 2, 1, 3]], ["\'", 7, [1, 2, 2, 3, 1, 2]], ["(", 8, [1, 3, 2, 2, 1, 2]], [")", 9, [2, 2, 1, 2, 1, 3]], ["*", 10, [2, 2, 1, 3, 1, 2]], ["+", 11, [2, 3, 1, 2, 1, 2]], [",", 12, [1, 1, 2, 2, 3, 2]], ["-", 13, [1, 2, 2, 1, 3, 2]], [".", 14, [1, 2, 2, 2, 3, 1]], ["/", 15, [1, 1, 3, 2, 2, 2]], ["0", 0x10, [1, 2, 3, 1, 2, 2]], ["1", 0x11, [1, 2, 3, 2, 2, 1]], ["2", 0x12, [2, 2, 3, 2, 1, 1]], ["3", 0x13, [2, 2, 1, 1, 3, 2]], ["4", 20, [2, 2, 1, 2, 3, 1]], ["5", 0x15, [2, 1, 3, 2, 1, 2]], ["6", 0x16, [2, 2, 3, 1, 1, 2]], ["7", 0x17, [3, 1, 2, 1, 3, 1]], ["8", 0x18, [3, 1, 1, 2, 2, 2]], ["9", 0x19, [3, 2, 1, 1, 2, 2]], [":", 0x1a, [3, 2, 1, 2, 2, 1]], [";", 0x1b, [3, 1, 2, 2, 1, 2]], ["<", 0x1c, [3, 2, 2, 1, 1, 2]], ["=", 0x1d, [3, 2, 2, 2, 1, 1]], [">", 30, [2, 1, 2, 1, 2, 3]], ["?", 0x1f, [2, 1, 2, 3, 2, 1]], ["@", 0x20, [2, 3, 2, 1, 2, 1]], ["A", 0x21, [1, 1, 1, 3, 2, 3]], ["B", 0x22, [1, 3, 1, 1, 2, 3]], ["C", 0x23, [1, 3, 1, 3, 2, 1]], ["D", 0x24, [1, 1, 2, 3, 1, 3]], ["E", 0x25, [1, 3, 2, 1, 1, 3]], ["F", 0x26, [1, 3, 2, 3, 1, 1]], ["G", 0x27, [2, 1, 1, 3, 1, 3]], ["H", 40, [2, 3, 1, 1, 1, 3]], ["I", 0x29, [2, 3, 1, 3, 1, 1]], ["J", 0x2a, [1, 1, 2, 1, 3, 3]], ["K", 0x2b, [1, 1, 2, 3, 3, 1]], ["L", 0x2c, [1, 3, 2, 1, 3, 1]], ["M", 0x2d, [1, 1, 3, 1, 2, 3]], ["N", 0x2e, [1, 1, 3, 3, 2, 1]], ["O", 0x2f, [1, 3, 3, 1, 2, 1]], ["P", 0x30, [3, 1, 3, 1, 2, 1]], ["Q", 0x31, [2, 1, 1, 3, 3, 1]], ["R", 50, [2, 3, 1, 1, 3, 1]], ["S", 0x33, [2, 1, 3, 1, 1, 3]], ["T", 0x34, [2, 1, 3, 3, 1, 1]], ["U", 0x35, [2, 1, 3, 1, 3, 1]], ["V", 0x36, [3, 1, 1, 1, 2, 3]], ["W", 0x37, [3, 1, 1, 3, 2, 1]], ["X", 0x38, [3, 3, 1, 1, 2, 1]], ["Y", 0x39, [3, 1, 2, 1, 1, 3]], ["Z", 0x3a, [3, 1, 2, 3, 1, 1]], ["[", 0x3b, [3, 3, 2, 1, 1, 1]], ["\\", 60, [3, 1, 4, 1, 1, 1]], ["]", 0x3d, [2, 2, 1, 4, 1, 1]], ["^", 0x3e, [4, 3, 1, 1, 1, 1]], ["_", 0x3f, [1, 1, 1, 2, 2, 4]], ["`", 0x40, [1, 1, 1, 4, 2, 2]], ["a", 0x41, [1, 2, 1, 1, 2, 4]], ["b", 0x42, [1, 2, 1, 4, 2, 1]], ["c", 0x43, [1, 4, 1, 1, 2, 2]], ["d", 0x44, [1, 4, 1, 2, 2, 1]], ["e", 0x45, [1, 1, 2, 2, 1, 4]], ["f", 70, [1, 1, 2, 4, 1, 2]], ["g", 0x47, [1, 2, 2, 1, 1, 4]], ["h", 0x48, [1, 2, 2, 4, 1, 1]], ["i", 0x49, [1, 4, 2, 1, 1, 2]], ["j", 0x4a, [1, 4, 2, 2, 1, 1]], ["k", 0x4b, [2, 4, 1, 2, 1, 1]], ["l", 0x4c, [2, 2, 1, 1, 1, 4]], ["m", 0x4d, [4, 1, 3, 1, 1, 1]], ["n", 0x4e, [2, 4, 1, 1, 1, 2]], ["o", 0x4f, [1, 3, 4, 1, 1, 1]], ["p", 80, [1, 1, 1, 2, 4, 2]], ["q", 0x51, [1, 2, 1, 1, 4, 2]], ["r", 0x52, [1, 2, 1, 2, 4, 1]], ["s", 0x53, [1, 1, 4, 2, 1, 2]], ["t", 0x54, [1, 2, 4, 1, 1, 2]], ["u", 0x55, [1, 2, 4, 2, 1, 1]], ["v", 0x56, [4, 1, 1, 2, 1, 2]], ["w", 0x57, [4, 2, 1, 1, 1, 2]], ["x", 0x58, [4, 2, 1, 2, 1, 1]], ["y", 0x59, [2, 1, 2, 1, 4, 1]], ["z", 90, [2, 1, 4, 1, 2, 1]], ["{", 0x5b, [4, 1, 2, 1, 2, 1]], ["|", 0x5c, [1, 1, 1, 1, 4, 3]], ["}", 0x5d, [1, 1, 1, 3, 4, 1]], ["~", 0x5e, [1, 3, 1, 1, 4, 1]], [String.fromCharCode(parseInt('007f', 16)), 0x5f, [1, 1, 4, 1, 1, 3]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00f1', 16)), 0x61, [4, 1, 1, 1, 1, 3]], [String.fromCharCode(parseInt('00f2', 16)), 0x60, [1, 1, 4, 3, 1, 1]], [String.fromCharCode(parseInt('00f3', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00f4', 16)), 0x62, [4, 1, 1, 3, 1, 1]], [String.fromCharCode(parseInt('00fc', 16)), 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fd', 16)), 0x68, [2, 1, 1, 2, 1, 4]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
        /** Specifies symbology settings for the Code 128 C Barcode. */
        code128c: {
            startSymbol: String.fromCharCode(parseInt('00fe', 16)),
            stopSymbol: String.fromCharCode(parseInt('00ff', 16)),
            validatorExp: "[0-9]",
            symbolTable: [["\0", 0, [2, 1, 2, 2, 2, 2]], [String.fromCharCode(parseInt('0001', 16)), 1, [2, 2, 2, 1, 2, 2]], [String.fromCharCode(parseInt('0002', 16)), 2, [2, 2, 2, 2, 2, 1]], [String.fromCharCode(parseInt('0003', 16)), 3, [1, 2, 1, 2, 2, 3]], [String.fromCharCode(parseInt('0004', 16)), 4, [1, 2, 1, 3, 2, 2]], [String.fromCharCode(parseInt('0005', 16)), 5, [1, 3, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0006', 16)), 6, [1, 2, 2, 2, 1, 3]], ["\a", 7, [1, 2, 2, 3, 1, 2]], ["\b", 8, [1, 3, 2, 2, 1, 2]], ["\t", 9, [2, 2, 1, 2, 1, 3]], ["\n", 10, [2, 2, 1, 3, 1, 2]], ["\v", 11, [2, 3, 1, 2, 1, 2]], ["\f", 12, [1, 1, 2, 2, 3, 2]], ["\r", 13, [1, 2, 2, 1, 3, 2]], [String.fromCharCode(parseInt('000e', 16)), 14, [1, 2, 2, 2, 3, 1]], [String.fromCharCode(parseInt('000f', 16)), 15, [1, 1, 3, 2, 2, 2]], [String.fromCharCode(parseInt('0010', 16)), 0x10, [1, 2, 3, 1, 2, 2]], [String.fromCharCode(parseInt('0011', 16)), 0x11, [1, 2, 3, 2, 2, 1]], [String.fromCharCode(parseInt('0012', 16)), 0x12, [2, 2, 3, 2, 1, 1]], [String.fromCharCode(parseInt('0013', 16)), 0x13, [2, 2, 1, 1, 3, 2]], [String.fromCharCode(parseInt('0014', 16)), 20, [2, 2, 1, 2, 3, 1]], [String.fromCharCode(parseInt('0015', 16)), 0x15, [2, 1, 3, 2, 1, 2]], [String.fromCharCode(parseInt('0016', 16)), 0x16, [2, 2, 3, 1, 1, 2]], [String.fromCharCode(parseInt('0017', 16)), 0x17, [3, 1, 2, 1, 3, 1]], [String.fromCharCode(parseInt('0018', 16)), 0x18, [3, 1, 1, 2, 2, 2]], [String.fromCharCode(parseInt('0019', 16)), 0x19, [3, 2, 1, 1, 2, 2]], [String.fromCharCode(parseInt('001a', 16)), 0x1a, [3, 2, 1, 2, 2, 1]], [String.fromCharCode(parseInt('001b', 16)), 0x1b, [3, 1, 2, 2, 1, 2]], [String.fromCharCode(parseInt('001c', 16)), 0x1c, [3, 2, 2, 1, 1, 2]], [String.fromCharCode(parseInt('001d', 16)), 0x1d, [3, 2, 2, 2, 1, 1]], [String.fromCharCode(parseInt('001e', 16)), 30, [2, 1, 2, 1, 2, 3]], [String.fromCharCode(parseInt('001f', 16)), 0x1f, [2, 1, 2, 3, 2, 1]], [" ", 0x20, [2, 3, 2, 1, 2, 1]], ["!", 0x21, [1, 1, 1, 3, 2, 3]], ["\"", 0x22, [1, 3, 1, 1, 2, 3]], ["#", 0x23, [1, 3, 1, 3, 2, 1]], ["$", 0x24, [1, 1, 2, 3, 1, 3]], ["%", 0x25, [1, 3, 2, 1, 1, 3]], ["&", 0x26, [1, 3, 2, 3, 1, 1]], ["\'", 0x27, [2, 1, 1, 3, 1, 3]], ["[", 40, [2, 3, 1, 1, 1, 3]], [")", 0x29, [2, 3, 1, 3, 1, 1]], ["*", 0x2a, [1, 1, 2, 1, 3, 3]], ["+", 0x2b, [1, 1, 2, 3, 3, 1]], [",", 0x2c, [1, 3, 2, 1, 3, 1]], ["-", 0x2d, [1, 1, 3, 1, 2, 3]], [".", 0x2e, [1, 1, 3, 3, 2, 1]], ["/", 0x2f, [1, 3, 3, 1, 2, 1]], ["0", 0x30, [3, 1, 3, 1, 2, 1]], ["1", 0x31, [2, 1, 1, 3, 3, 1]], ["2", 50, [2, 3, 1, 1, 3, 1]], ["3", 0x33, [2, 1, 3, 1, 1, 3]], ["4", 0x34, [2, 1, 3, 3, 1, 1]], ["5", 0x35, [2, 1, 3, 1, 3, 1]], ["6", 0x36, [3, 1, 1, 1, 2, 3]], ["7", 0x37, [3, 1, 1, 3, 2, 1]], ["8", 0x38, [3, 3, 1, 1, 2, 1]], ["9", 0x39, [3, 1, 2, 1, 1, 3]], [":", 0x3a, [3, 1, 2, 3, 1, 1]], [";", 0x3b, [3, 3, 2, 1, 1, 1]], ["<", 60, [3, 1, 4, 1, 1, 1]], ["=", 0x3d, [2, 2, 1, 4, 1, 1]], [">", 0x3e, [4, 3, 1, 1, 1, 1]], ["?", 0x3f, [1, 1, 1, 2, 2, 4]], ["@", 0x40, [1, 1, 1, 4, 2, 2]], ["A", 0x41, [1, 2, 1, 1, 2, 4]], ["B", 0x42, [1, 2, 1, 4, 2, 1]], ["C", 0x43, [1, 4, 1, 1, 2, 2]], ["D", 0x44, [1, 4, 1, 2, 2, 1]], ["E", 0x45, [1, 1, 2, 2, 1, 4]], ["F", 70, [1, 1, 2, 4, 1, 2]], ["G", 0x47, [1, 2, 2, 1, 1, 4]], ["H", 0x48, [1, 2, 2, 4, 1, 1]], ["I", 0x49, [1, 4, 2, 1, 1, 2]], ["J", 0x4a, [1, 4, 2, 2, 1, 1]], ["K", 0x4b, [2, 4, 1, 2, 1, 1]], ["L", 0x4c, [2, 2, 1, 1, 1, 4]], ["M", 0x4d, [4, 1, 3, 1, 1, 1]], ["N", 0x4e, [2, 4, 1, 1, 1, 2]], ["O", 0x4f, [1, 3, 4, 1, 1, 1]], ["P", 80, [1, 1, 1, 2, 4, 2]], ["Q", 0x51, [1, 2, 1, 1, 4, 2]], ["R", 0x52, [1, 2, 1, 2, 4, 1]], ["S", 0x53, [1, 1, 4, 2, 1, 2]], ["T", 0x54, [1, 2, 4, 1, 1, 2]], ["U", 0x55, [1, 2, 4, 2, 1, 1]], ["V", 0x56, [4, 1, 1, 2, 1, 2]], ["W", 0x57, [4, 2, 1, 1, 1, 2]], ["X", 0x58, [4, 2, 1, 2, 1, 1]], ["Y", 0x59, [2, 1, 2, 1, 4, 1]], ["Z", 90, [2, 1, 4, 1, 2, 1]], ["[", 0x5b, [4, 1, 2, 1, 2, 1]], ["\\", 0x5c, [1, 1, 1, 1, 4, 3]], ["]", 0x5d, [1, 1, 1, 3, 4, 1]], ["^", 0x5e, [1, 3, 1, 1, 4, 1]], ["_", 0x5f, [1, 1, 4, 1, 1, 3]], ["`", 0x60, [1, 1, 4, 3, 1, 1]], ["a", 0x61, [4, 1, 1, 1, 1, 3]], ["b", 0x62, [4, 1, 1, 3, 1, 1]], ["c", 0x63, [1, 1, 3, 1, 4, 1]], [String.fromCharCode(parseInt('00f0', 16)), 0x66, [4, 1, 1, 1, 3, 1]], [String.fromCharCode(parseInt('00fa', 16)), 0x65, [3, 1, 1, 1, 4, 1]], [String.fromCharCode(parseInt('00fb', 16)), 100, [1, 1, 4, 1, 3, 1]], [String.fromCharCode(parseInt('00fe', 16)), 0x69, [2, 1, 1, 2, 3, 2]], [String.fromCharCode(parseInt('00ff', 16)), -1, [2, 3, 3, 1, 1, 1, 2]]],
        },
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Diagram elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
* Use of this code is subject to the terms of our license.
* A copy of the current license can be obtained at any time by e-mailing
* licensing@syncfusion.com. Any infringement will be prosecuted under
* applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    "use strict";
    //#region ej.Diagram widget
    ej.widget("ejDiagram", "ej.datavisualization.Diagram", {
        element: null,
        model: null,
        validTags: ["div"],
        //#region defaults
        defaults: {
            width: null,
            height: null,
            nodes: [],
            connectors: [],
            defaultSettings: {
                connector: null,
                node: null
            },
            nodeTemplate: null,
            connectorTemplate: null,
            dataSourceSettings: {
                dataSource: null,
                query: null,
                tableName: null,
                id: "",
                parent: "",
                nodes: null,
                connectors: null,
                //nodeBinding: null,
                //connectorBinding: null,
                root: ""
            },
            snapSettings: {
                horizontalGridLines: {
                    linesInterval: [1.25, 18.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75],
                    snapInterval: [20],
                    lineDashArray: "",
                    lineColor: "lightgray"
                },
                verticalGridLines: {
                    linesInterval: [1.25, 18.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75, 0.25, 19.75],
                    snapInterval: [20],
                    lineDashArray: "",
                    lineColor: "lightgray"
                },
                snapConstraints: 1 | 2 | 4 | 8,
                enableSnapToObject: true,
                snapAngle: 5,
                snapObjectDistance: 5
            },
            scrollSettings: {
                horizontalOffset: 0,
                verticalOffset: 0,
                currentZoom: 1,
                viewPortHeight: 0,
                viewPortWidth: 0,
                zoomFactor: 0.2,
                padding: 0
            },
            pageSettings: {
                pageWidth: null,
                pageHeight: null,
                multiplePage: false,
                pageBorderWidth: 0,
                pageBackgroundColor: "#ffffff",
                pageBorderColor: "#565656",
                pageMargin: 24,
                showPageBreak: false,
                pageOrientation: "portrait",
                scrollLimit: "infinity",
                scrollableArea: { x: Number.POSITIVE_INFINITY, y: Number.POSITIVE_INFINITY, width: Number.POSITIVE_INFINITY, height: Number.POSITIVE_INFINITY },
                autoScrollBorder: { left: 15, top: 15, right: 15, bottom: 15 },
            },
            locale: "en-US",
            contextMenu: {
                items: [],
                showCustomMenuItemsOnly: false
            },
            enableContextMenu: true,
            enableAutoScroll: true,
            tooltip: null,
            showTooltip: true,
            layout: {
                type: "none",
                orientation: "toptobottom",
                horizontalSpacing: 30,
                verticalSpacing: 30,
                marginX: 0,
                marginY: 0,
                fixedNode: "",
                getLayoutInfo: null
            },
            drawingTools: {},
            backgroundImage: null,
            backgroundColor: "transparent",
            bridgeDirection: "top",
            version: "13.1",
            constraints: 1 << 1 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6,
            tool: 1 << 1 | 1 << 2,
            drawType: {},
            selectedItems: {
                offsetX: 0,
                offsetY: 0,
                width: 0,
                height: 0,
                rotateAngle: 0,
                children: [],
                constraints: 1 << 1 | 1 << 2 | 1 << 3,
                userHandles: [],
                tooltip: {
                    templateId: "",
                    alignment: { horizontal: "center", vertical: "bottom" },
                    margin: { top: 10 }
                }
            },
            commandManager: {
                commands: {}
            },
            historyManager: {
                push: null,
                pop: null,
                canPop: null,
                undo: null,
                redo: null,
                startGroupAction: null,
                closeGroupAction: null
            },
            nodeCollectionChange: null,
            autoScrollChange: null,
            itemClick: null,
            connectorCollectionChange: null,
            selectionChange: null,
            mouseLeave: null,
            mouseEnter: null,
            mouseOver: null,
            click: null,
            doubleClick: null,
            dragEnter: null,
            dragOver: null,
            dragLeave: null,
            drop: null,
            drag: null,
            textChange: null,
            sizeChange: null,
            connectionChange: null,
            rotationChange: null,
            contextMenuClick: null,
            contextMenuBeforeOpen: null,
            connectorSourceChange: null,
            connectorTargetChange: null,
            scrollChange: null,
            segmentChange: null,
            propertyChange: null,
            groupChange: null,
            create: null,
            destroy: null
        },
        observables: [
            "nodes",
            "connectors",
            "locale",
            "enableContextMenu",
            "enableAutoScroll",
            "showTooltip",
            "bridgeDirection",
            "tool",
            "pageSettings.pageHeight",
            "pageSettings.pageWidth",
            "pageSettings.multiplePage",
            "pageSettings.pageBorderWidth",
            "pageSettings.pageBackgroundColor",
            "pageSettings.pageBorderColor",
            "pageSettings.pageMargin",
            "pageSettings.showPageBreak",
            "pageSettings.pageOrientation",
            "pageSettings.scrollLimit",
            "snapSettings.enableSnapToObject",
            "snapSettings.snapAngle",
            "snapSettings.snapObjectDistance",
            "selector.offsetX",
            "selector.offsetY",
            "selector.width",
            "selector.height",
            "selector.rotateAngle",
            "scrollSettings.horizontalOffset",
            "scrollSettings.verticalOffset",
            "scrollSettings.zoomFactor"
        ],
        nodes: ej.util.valueFunction("nodes"),
        connectors: ej.util.valueFunction("connectors"),
        locale: ej.util.valueFunction("locale"),
        enableContextMenu: ej.util.valueFunction("enableContextMenu"),
        enableAutoScroll: ej.util.valueFunction("enableAutoScroll"),
        showTooltip: ej.util.valueFunction("showTooltip"),
        bridgeDirection: ej.util.valueFunction("bridgeDirection"),
        tool: ej.util.valueFunction("tool"),
        _pageHeight: ej.util.valueFunction("pageSettings.pageHeight"),
        _pageWidth: ej.util.valueFunction("pageSettings.pageWidth"),
        _multiplePage: ej.util.valueFunction("pageSettings.multiplePage"),
        _pageBorderWidth: ej.util.valueFunction("pageSettings.pageBorderWidth"),
        _pageBackgroundColor: ej.util.valueFunction("pageSettings.pageBackgroundColor"),
        _pageBorderColor: ej.util.valueFunction("pageSettings.pageBorderColor"),
        _pageMargin: ej.util.valueFunction("pageSettings.pageMargin"),
        _showPageBreak: ej.util.valueFunction("pageSettings.showPageBreak"),
        _pageOrientation: ej.util.valueFunction("pageSettings.pageOrientation"),
        _scrollLimit: ej.util.valueFunction("pageSettings.scrollLimit"),
        _enableSnapToObject: ej.util.valueFunction("snapSettings.enableSnapToObject"),
        _snapAngle: ej.util.valueFunction("snapSettings.snapAngle"),
        _snapObjectDistance: ej.util.valueFunction("snapSettings.snapObjectDistance"),
        _selectorOffsetX: ej.util.valueFunction("selector.offsetX"),
        _selectorOffsetY: ej.util.valueFunction("selector.offsetY"),
        _selectorWidth: ej.util.valueFunction("selector.width"),
        _selectorHeight: ej.util.valueFunction("selector.height"),
        _selectorRotateAngle: ej.util.valueFunction("selector.rotateAngle"),
        _horizontalOffset: ej.util.valueFunction("scrollSettings.horizontalOffset"),
        _verticalOffset: ej.util.valueFunction("scrollSettings.verticalOffset"),
        zoomFactor: ej.util.valueFunction("scrollSettings.zoomFactor"),
        dataTypes: {
            nodes: "data",
            connectors: "data",
            contextMenu: {
                items: "array",
                showCustomMenuItemsOnly: "boolean"
            },
            layout: {
                type: "enum",
                orientation: "enum",
                horizontalSpacing: "number",
                verticalSpacing: "number",
                marginX: "number",
                marginY: "number"
            },
        },
        //#endregion
        //#region Local members
        _nodes: [],
        _connectors: [],
        _canvas: null,
        _svg: null,
        _toolToActivate: null,
        _inAction: false,
        _isPinching: false,
        _isEditing: false,
        _nodeToHit: null,
        _currentCursor: null,
        _pasteIndex: null,
        _page: null,
        _view: null,
        _currentLabel: null,
        _zOrder: 0,
        _clipboardData: null,
        _currZoom: 1,
        _mouseEventTriggered: false,
        _selectedItem: "",
        _historyList: {
            currentEntry: {
                next: null,
                previous: null
            },
            canUndo: false,
            canRedo: false
        },
        _defaultContextMenuItems: [
            { name: "cut", text: "Cut" },
            { name: "copy", text: "Copy" },
            { name: "paste", text: "Paste" },
            { name: "undo", text: "Undo" },
            { name: "redo", text: "Redo" },
            { name: "selectAll", text: "Select All" },
            {
                name: "grouping",
                text: "Grouping",
                subItems: [
                    { name: "group", text: "Group" },
                    { name: "ungroup", text: "Ungroup" }
                ]
            },
            {
                name: "order",
                text: "Order",
                subItems: [
                    { name: "bringToFront", text: "Bring To Front" },
                    { name: "moveForward", text: "Move Forward" },
                    { name: "sendToBack", text: "Send To Back" },
                    { name: "sendBackward", text: "Send Backward" }
                ]
            }
        ],
        _hScrollOffset: 0,
        _vScrollOffset: 0,
        _hScrollbar: null,
        _vScrollbar: null,
        _scrollPixel: 30,
        _previousSelectedItems: [],
        //#endregion
        //#region public members
        activeTool: null,
        selectionList: [],
        tools: null,
        nameTable: {},
        //#endregion
        //#region Initialization
        _init: function () {
            var svgSupport = (window.SVGSVGElement) ? true : false;
            if (svgSupport) {
                this._cloneGlobalVariables();
                this._spatialSearch = ej.datavisualization.Diagram.SpatialSearch();
                this._initDefaults();
                this._initViews();
                this._initData();
                this._doLayout();
                this._initCanvas();
                this._initDiagramTool();
                this._initVisualGuide();
                this._wireEvents();
                this._initHandles();
                this._initContextMenu();
                this._drawingTool = false;
                this._isDropped = false;
                this._initCommands();
                this._initHistoryManager();
                if (this.model.zoomFactor && this.model.zoomFactor !== .2)
                    this.model.scrollSettings.zoomFactor = this.model.zoomFactor;
                this._updateScrollSettings(this.model.scrollSettings);
                delete this._viewPort;
            }
        },

        refresh: function () {
            this._unWireEvents();
            this._destroy();           
            this._historyList = null;
            $(this.element).addClass("e-datavisualization-diagram");
            var svgSupport = (window.SVGSVGElement) ? true : false;
            if (svgSupport) {
                this._cloneGlobalVariables();
                this._spatialSearch = ej.datavisualization.Diagram.SpatialSearch();
                this._initDefaults();
                this._initViews();
                this._initData();
                this._doLayout();
                this._initCanvas();
                this._initDiagramTool();
                this._initVisualGuide();
                this._wireEvents();
                this._initHandles();
                this._initContextMenu();
                this._drawingTool = false;
                this._isDropped = false;
                this._initCommands();
                this._initHistoryManager();
                if (this.model.zoomFactor && this.model.zoomFactor !== .2)
                    this.model.scrollSettings.zoomFactor = this.model.zoomFactor;
                this._updateScrollSettings(this.model.scrollSettings);
            }
            this._vScrollOffset = 0;
            this._hScrollOffset = 0;
            this._updateScrollOffset(this._hScrollOffset, this._vScrollOffset);
       },
        _initCommands: function (newCommands) {
            var modifiers = ej.datavisualization.Diagram.KeyModifiers;
            var keys = ej.datavisualization.Diagram.Keys;
            var commands;
            if (newCommands) {
                commands = this.model.commandManager.commands;
            } else {
                newCommands = this.model.commandManager.commands;
                commands = {
                    "copy":
                        {
                            execute: $.proxy(this.copy, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.C, keyModifiers: modifiers.Control }
                        },
                    "paste":
                        {
                            execute: $.proxy(this._commandPaste, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.V, keyModifiers: modifiers.Control }
                        },
                    "cut":
                        {
                            execute: $.proxy(this.cut, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.X, keyModifiers: modifiers.Control }
                        },
                    "delete":
                        {
                            execute: $.proxy(this.remove, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.Delete }
                        },
                    "undo":
                        {
                            execute: $.proxy(this.undo, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.Z, keyModifiers: modifiers.Control }
                        },
                    "redo":
                        {
                            execute: $.proxy(this.redo, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.Y, keyModifiers: modifiers.Control }
                        },
                    "selectAll":
                        {
                            execute: $.proxy(this._selectCommand, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.A, keyModifiers: modifiers.Control }
                        },
                    "nudgeUp":
                        {
                            execute: $.proxy(this._nudgeCommand, this),
                            canExecute: $.proxy(this._canExecute, this),
                            parameter: "up",
                            gesture: { key: keys.Up }
                        },
                    "nudgeRight":
                        {
                            execute: $.proxy(this._nudgeCommand, this),
                            canExecute: $.proxy(this._canExecute, this),
                            parameter: "right",
                            gesture: { key: keys.Right }
                        },
                    "nudgeDown":
                        {
                            execute: $.proxy(this._nudgeCommand, this),
                            canExecute: $.proxy(this._canExecute, this),
                            parameter: "down",
                            gesture: { key: keys.Down }
                        },
                    "nudgeLeft":
                        {
                            execute: $.proxy(this._nudgeCommand, this),
                            canExecute: $.proxy(this._canExecute, this),
                            parameter: "left",
                            gesture: { key: keys.Left }
                        },
                    "startEdit":
                        {
                            execute: $.proxy(this._startEditCommand, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: 113 }
                        },
                    "endEdit":
                        {
                            execute: $.proxy(this._endEdit, this),
                            canExecute: $.proxy(this._canExecute, this),
                            gesture: { key: keys.Escape }
                        }
                };
            }
            var i;
            for (i in newCommands) {
                if (commands[i]) {
                    if (newCommands[i]) {
                        if (newCommands[i] && commands[i].gesture && newCommands[i].gesture) newCommands[i].gesture = ej.datavisualization.Diagram.Gesture($.extend({}, true, commands[i].gesture, newCommands[i].gesture));
                        $.extend(commands[i], true, newCommands[i]);
                    } else commands[i] = null;
                } else {
                    if (newCommands[i]) {
                        var cmd = ej.datavisualization.Diagram.Command(newCommands[i]);
                        commands[i] = cmd;
                    }
                }
            }
            this.model.commandManager.commands = commands;
        },

        _initViews: function (isLoad) {
            if (!isLoad) {
                this._views = [];
                this._views.push(this._id);
            }
            this._views[this._id] = {
                "type": "mainview",
                "context": ej.datavisualization.Diagram.SvgContext,
                "style": "position:relative; height:" + this.model.height + "; width:" + this.model.width + ";"
            };
        },
        _initDefaults: function () {
            this._initTools();
        },
        _initCanvas: function (isload) {
            this._renderCanvas(isload);
        },
        _initVisualGuide: function () {
            if (this.model.tooltip) {
                this.model.tooltip = ej.datavisualization.Diagram.Tooltip(this.model.tooltip);
            }
            if (this.model.selectedItems) {
                if (this.model.selectorConstraints) {
                    this.model.selectedItems.constraints = this.model.selectorConstraints;
                }

                if (this.model.selectedItems.tooltip) {
                    this.model.selectedItems.tooltip = ej.datavisualization.Diagram.Tooltip(this.model.selectedItems.tooltip);
                }
                if (this.showTooltip() !== undefined) {
                    if (this.showTooltip())
                        this.model.selectedItems.tooltip = ej.datavisualization.Diagram.Tooltip({ margin: { top: 10 } });
                    else this.model.selectedItems.tooltip = null;
                }
                if (this.model.selectedItems.tooltip && this.model.tooltipTemplateId) {
                    this.model.selectedItems.tooltip.templateId = this.model.selectedItems.tooltip.templateId || this.model.tooltipTemplateId;
                }
            }
        },
        _initTools: function () {
            this.tools = {
                "select": new ej.datavisualization.Diagram.SelectTool(this),
                "move": new ej.datavisualization.Diagram.MoveTool(this),
                "resize": new ej.datavisualization.Diagram.ResizeTool(this),
                "rotate": new ej.datavisualization.Diagram.RotateTool(this),
                //"pivot": new ej.datavisualization.Diagram.PivotTool(this),
                "endPoint": new ej.datavisualization.Diagram.ConnectionEditTool(this),
                "panTool": new ej.datavisualization.Diagram.PanTool(this),
                "straightLine": new ej.datavisualization.Diagram.StraightLineTool(this),
                "orthogonalLine": new ej.datavisualization.Diagram.OrthogonalLineTool(this),
                "phase": new ej.datavisualization.Diagram.PhaseTool(this),
                "shapeTool": new ej.datavisualization.Diagram.ShapeTool(this),
                "text": new ej.datavisualization.Diagram.TextTool(this),
            };
            this._toolToActivate = "select";
            this.activeTool = this.tools[this._toolToActivate];
            this._registerDrawingTools();
        },
        _initDiagramTool: function () {
            var tool = this.tool();
            var shape;
            if (tool & ej.datavisualization.Diagram.Tool.ContinuesDraw) {
                if (this.model.drawType) {
                    shape = this.model.drawType.shape ? this.model.drawType.shape : this.model.drawType.type;
                    this._drawToolShape(shape, false);
                }
            } else if (tool & ej.datavisualization.Diagram.Tool.DrawOnce) {
                if (this.model.drawType) {
                    shape = this.model.drawType.shape ? this.model.drawType.shape : this.model.drawType.type;
                    this._drawToolShape(shape, true);
                }
            } else if (tool & ej.datavisualization.Diagram.Tool.ZoomPan) {
                if (ej.datavisualization.Diagram.Util.canPanning(this)) {
                    this._toolToActivate = "panTool";
                    this.activeTool = this.tools[this._toolToActivate];
                    this.activeTool._isMouseDown = false;
                    this._currentCursor = "pointer";
                }
            } else if (tool & ej.datavisualization.Diagram.Tool.MultipleSelect) {
                this.activateTool("select");
                this.activeTool.singleAction = false;
            }
        },
        _initHandles: function () {
            var userHandles = this.model.selectedItems.userHandles;
            if (userHandles) {
                for (var i = 0; i < userHandles.length; i++) {
                    userHandles[i] = ej.datavisualization.Diagram.UserHandle(userHandles[i]);
                    if (userHandles[i].tool) {
                        userHandles[i].tool.diagram = this;
                        this.tools[userHandles[i].name] = userHandles[i].tool;
                    }
                }
            }
        },
        _initHistoryManager: function () {
            this.model.historyManager.push = $.proxy(this.addHistoryEntry, this);
            this.model.historyManager.pop = $.proxy(this.removeHistoryEntry, this);
            this.model.historyManager.canPop = $.proxy(this.canRemoveHistoryEntry, this);
            this.model.historyManager.startGroupAction = $.proxy(this._startGroupAction, this);
            this.model.historyManager.closeGroupAction = $.proxy(this._closeGroupAction, this);
        },

        _cloneGlobalVariables: function () {
            this._historyList = $.extend(true, {}, this._historyList);
        },
        _setNodesConnectors: function (options, isLoad) {
            var nodes = [], sNodes = false, sConnectors = false;
            var nLength = options.nodes ? (typeof options.nodes === 'function' ? options.nodes().length : options.nodes.length) : 0;
            var cLength = options.connectors ? (options.connectors && typeof options.connectors === 'function' ? options.connectors().length : options.connectors.length) : 0;
            if (nLength != this._nodes.length) {
                this.nodes(typeof options.nodes === 'function' ? options.nodes() : options['nodes']);
                this._initNodeCollection(isLoad);
                nodes = this.nodes(); sNodes = true;

            }
            if (cLength != this._connectors.length) {
                this.connectors(typeof options.connectors === 'function' ? options.connectors() : options['connectors']);
                this._initConnectorCollection(); sConnectors = true;
                nodes = nodes.length > 0 ? this.nodes().concat(this.connectors()) : this.connectors();
            }
            if (nodes.length > 0 || options.nodes.length == 0 || options.connectors.length == 0) {
                this._resetObjectCollection(nodes, isLoad, sNodes, sConnectors);
            }
            return true;
        },
        _setModel: function (options) {
            var initObject = false;
            var diagram, object;
            for (var option in options) {
                switch (option) {
                    case "height":
                    case "width":
                        this._initViews(true);
                        this._initCanvas(true);
                        break;
                    case "nodes":
                    case "connectors":
                        if (!initObject)
                            initObject = this._setNodesConnectors(options, this._isLoad);
                        break;
                    case "phases":
                        this.model.phases = [];
                        this._initPhaseCollection(options.phases);
                        this.element.remove(".ej-d-node");
                        this.element.remove(".ej-d-group");
                        diagram = this;
                        this._views.forEach(function (viewid) {
                            var view = diagram._views[viewid];
                            diagram._renderphases(view);
                        });
                        break;
                    case "enableContextMenu":
                        this.model.enableContextMenu = options[option];
                        this._initContextMenu(this._isLoad);
                        break;
                    case "contextMenu":
                        this.model.contextMenu = options[option];
                        this._initContextMenu(this._isLoad);
                        break;
                    case "snapSettings":
                        object = options[option];
                        if (object && Object.keys(object).length > 0) {
                            this._updateSnapSettings(options[option]);
                            ej.datavisualization.Diagram.SvgContext._renderGrid(this._canvas, this._svg, this._page, this);
                        }
                        break;
                    case "selectorConstraints":
                        this._setSelectorConstraints(options[option]);
                        break;
                    case "constraints":
                        this.model.constraints = options[option];
                        if (this._hasSelection())
                            this._clearSelection();
                        break;
                    case "drawingTools":
                        this.model.drawingTools = options[option];
                        this._registerDrawingTools();
                        break;
                    case "backgroundImage":
                        this.model.backgroundImage = options[option];
                        ej.datavisualization.Diagram.SvgContext._renderBackground(this._canvas, this._svg, this._page, this.model);
                        break;
                    case "enableAutoScroll":
                        this.enableAutoScroll(options[option]);
                        break;
                    case "autoScrollMargin":
                        this.model.autoScrollMargin = options[option];
                        break;
                    case "layout":
                        this.model.layout = $.extend(true, {}, this.model.layout, options[option]);
                        this._setLayout(options[option]);
                        break;
                    case "selectedItems":
                        this.updateSelector(options[option]);
                        break;
                    case "pageSettings":
                        this.updatePageSettings(options[option]);
                        break;
                    case "scrollSettings":
                        this._updateScrollSettings(options[option]);
                        break;
                    case "locale":
                        this.locale(options[option]);
                        var contextmenu = document.getElementById(this.element[0].id + "_contextMenu");
                        contextmenu.parentNode.removeChild(contextmenu);
                        if (this.model.contextMenu && this.model.contextMenu.items && this.model.contextMenu.items.length > 0)
                            this.model.contextMenu.items = [];
                        this._renderContextMenu();
                        break;
                    case "nodeTemplate":
                        this.clear();
                        this._initData();
                        diagram = this;
                        this._views.forEach(function (viewid) {
                            var view = diagram._views[viewid];
                            var nodes = diagram._setNodeZOrder(view);
                            diagram._renderDiagramObjects(nodes, view);
                        });
                        this.layout();
                        break;
                    case "bridgeDirection":
                    case "tool":
                        this.update(options);
                        break;
                    case "connectorTemplate":
                        this._updateConnectors();
                        break;
                    case "commandManager":
                        var commands = options[option] && options[option]["commands"] ? options[option]["commands"] : undefined;
                        this._initCommands(commands);
                        break;
                    case "historyManager":
                        this.model.historyManager = $.extend(true, this.model.historyManager, {}, options[option]);
                        this._initHistoryManager();
                        break;
                    case "showTooltip":
                        this.model.showTooltip = options[option];
                        if (this.showTooltip())
                            this.model.selectedItems.tooltip = ej.datavisualization.Diagram.Tooltip({ margin: { top: 10 } });
                        else
                            this.model.selectedItems.tooltip = null;
                        break;
                    case "toolTipTemplateId":
                        if (!this.model.selectedItems.tooltip)
                            this.model.selectedItems.tooltip = ej.datavisualization.Diagram.Tooltip({ margin: { top: 10 } });
                        this.model.selectedItems.tooltip.templateId = options[option];
                        break;
                    case "tooltip":
                        if (options[option] === null) this.model.tooltip = null;
                        else
                            this.model.tooltip = ej.datavisualization.Diagram.Tooltip($.extend(true, this.model.tooltip, {}, options[option]));
                        break;
                    case "dataSourceSettings":
                        object = options[option];
                        if (object && Object.keys(object).length > 0) {
                            this.clear();
                            $.extend(true, this.model.dataSourceSettings, {}, options[option]);
                            this._initData();
                            diagram = this;
                            this._views.forEach(function (viewid) {
                                var view = diagram._views[viewid];
                                var nodes = diagram._setNodeZOrder(view);
                                diagram._renderDiagramObjects(nodes, view);
                            });
                            this.layout();
                        }
                        break;
                }
            }
        },
        _resetObjectCollection: function (nodes, isLoad, sNode, sConnectors) {
            if (sNode) {
                $(this.element).find(".ej-d-node").remove();
                $(this.element).find(".ej-d-group").remove();
            }
            if (sConnectors) {
                var children = $(this.element).find(".ej-d-connector");
                for (var i = children.length - 1; i >= 0; i--) {
                    var child = this.nameTable[children[i].id];
                    if (child && child.parent == "")
                        $(children[i]).remove();
                }
            }
            if (this._hasSelection())
                this._clearSelection();
            var diagram = this;
            this._views.forEach(function (viewid) {
                var view = diagram._views[viewid];
                var nodes = diagram._setNodeZOrder(view);
                diagram._renderDiagramObjects(nodes, view);
            });

        },
        _getConnectors: function () {
            var connectors = [];
            for (var prop in this.nameTable) {
                var connector = this.nameTable[prop];
                if (connector._type === "connector" || connector.segments)
                    connectors.push(connector);
            }
            return connectors;
        },
        _initData: function () {
            if (this.model.dataSourceSettings)
                var dataSource = this.model.dataSourceSettings.dataSource || this.model.dataSourceSettings.nodes || this.model.dataSourceSettings.connectors;
            if (dataSource) {
                var nodes = [];
                var connectors = [];
                var dataSourceApplied;
                if (this.model.dataSourceSettings.nodes || this.model.dataSourceSettings.connectors) {
                    if (this.model.dataSourceSettings.nodes) {
                        dataSource = this.model.dataSourceSettings.nodes.dataSource || this.model.dataSourceSettings.dataSource;
                        if (dataSource) {
                            if (dataSource instanceof ej.DataManager) {
                                $.ajaxSetup({
                                    async: false
                                });
                                var query = this._findQuery(this.model.dataSourceSettings.nodes, this.model.dataSourceSettings.nodeBinding);
                                var queryPromise = dataSource.executeQuery(eval(query));
                                var diagram = this;
                                queryPromise.done(function (e) {
                                    diagram.retriveData = e.result;
                                    diagram._applyTemplate(diagram.model.dataSourceSettings, e.result, nodes, "nodes");
                                });
                            } else
                                this._applyTemplate(this.model.dataSourceSettings, dataSource, nodes, "nodes");
                        }
                        if (this.model.dataSourceSettings.connectors) {
                            dataSource = this.model.dataSourceSettings.connectors.dataSource || this.model.dataSourceSettings.dataSource;
                            if (dataSource) {
                                if (dataSource instanceof ej.DataManager) {
                                    $.ajaxSetup({
                                        async: false
                                    });
                                    query = this._findQuery(this.model.dataSourceSettings.connectors, this.model.dataSourceSettings.connectorBinding);
                                    queryPromise = dataSource.executeQuery(eval(query));
                                    queryPromise.done(function (e) {
                                        diagram.retriveData = e.result;
                                        diagram._applyTemplate(diagram.model.dataSourceSettings, e.result, connectors, "connectors");
                                    });
                                } else {
                                    this._applyTemplate(this.model.dataSourceSettings, dataSource, connectors, "connectors");
                                }
                            }
                        }
                    }
                } else if (dataSource instanceof ej.DataManager) {
                    this._initDataSource(dataSource, nodes, connectors);
                    if (this.model.dataSourceSettings.parent)
                        dataSourceApplied = true;
                }
                else if (dataSource.length) {
                    this._applyDataSource(this.model.dataSourceSettings, dataSource, nodes, connectors);
                    if (this.model.dataSourceSettings.parent)
                        dataSourceApplied = true;
                }
                var tempNodes = [], tempconnectors = [];
                if (this.nodes().length > 0)
                    tempNodes = this.nodes();
                if (this.connectors().length > 0)
                    tempconnectors = this.connectors();
                this.nodes(nodes);
                this.connectors(connectors);
                var i;
                for (i = 0; i < tempNodes.length; i++) {
                    this.nodes().push(tempNodes[i]);
                    this._nodes = $.extend(true, [], this.nodes());
                }
                for (i = 0; i < tempconnectors.length; i++) {
                    this.connectors().push(tempconnectors[i]);
                    this._connectors = $.extend(true, [], this.connectors());
                }
            }
            var collapsedNodes = this._initNodeCollection(false, dataSourceApplied);
            this._initConnectorCollection(dataSourceApplied);
            if (collapsedNodes.length > 0) {
                this._collapseNodes(collapsedNodes);
            }
            this._initPhaseCollection();
        },
        _initNodeCollection: function (isLoad, dataSourceApplied) {
            var collapsedNodes = [];
            var nodes = this.nodes();
            var length = this.nodes().length;
            var nodeTemplate;
            if (this.model.nodeTemplate && !this.model.dataSourceSettings.dataSource) {
                if (typeof this.model.nodeTemplate === "string") {
                    this.model.nodeTemplate = ej.util.getObject(this.model.nodeTemplate, window);
                }
                if ($.isFunction(this.model.nodeTemplate)) {
                    nodeTemplate = this.model.nodeTemplate;
                }
            }
            for (var i = 0; i < length; i++) {
                if (typeof nodes[i].shape !== "object") {
                    if (nodes[i].type == "bpmn" && this._isLoad) nodes[i].children = undefined;
                    nodes[i] = ej.datavisualization.Diagram.NodeType(nodes[i], this);
                }
                if (nodes[i].name == "") {
                    nodes[i].name = "node_" + ej.datavisualization.Diagram.Util.randomId();
                }
                if (nodes[i].type === "swimlane" || nodes[i].isSwimlane) {
                    nodes[i] = ej.datavisualization.Diagram.SwimLaneHelper._initSwimLane(nodes[i], this);
                    this.nameTable[nodes[i].name] = ej.datavisualization.Diagram.Group(nodes[i]);

                } else if (nodes[i]._type === "group" || (nodes[i].children && nodes[i].children.length > 0)) {
                    nodes[i] = this._getNewGroup(nodes[i]);
                    nodes[i]._type = "group";
                    this.nameTable[nodes[i].name] = nodes[i];
                    this._initGroupNode(nodes[i]);
                } else {
                    if (!dataSourceApplied) {
                        nodes[i] = this._getNewNode(nodes[i]); //ej.datavisualization.Diagram.Node(nodes[i]);
                        this.nameTable[nodes[i].name] = nodes[i];
                    } else if (nodes[i].shape && typeof nodes[i].shape === "object")
                        ej.datavisualization.Diagram.Util._updateShapeProperties(nodes[i]);
                }
                if (!dataSourceApplied && nodeTemplate) nodeTemplate(this, nodes[i]);
                if (nodes[i]._type === "node" && nodes[i].labels.length && (nodes[i].width == 0 || nodes[i].height == 0))
                    this._getNodeDimension(nodes[i]);
                this.nameTable[nodes[i].name] = nodes[i];
                this._updateQuad(nodes[i]);
                if (!nodes[i].isExpanded) {
                    collapsedNodes.push(nodes[i]);
                }
            }
            this._nodes = $.extend(true, [], nodes);
            return collapsedNodes;
        },
        _initConnectorCollection: function (dataSourceApplied) {
            var connectors = this.connectors();
            var connector, connectorTemplate;
            if (this.model.connectorTemplate) {
                if (typeof this.model.connectorTemplate === "string") {
                    this.model.connectorTemplate = ej.util.getObject(this.model.connectorTemplate, window);
                }
                if ($.isFunction(this.model.connectorTemplate)) {
                    connectorTemplate = this.model.connectorTemplate;
                }
            }
            for (var i = 0, len = connectors.length; i < len; i++) {
                connector = dataSourceApplied ? connectors[i] : this._getNewConnector(connectors[i]);
                this.nameTable[connector.name] = connector;
                if (!dataSourceApplied) {
                    this._dock(connector, this.nameTable);
                }
                this._updateEdges(connector);
                if (connectorTemplate && !dataSourceApplied) connectorTemplate(this, connector);
                connectors[i] = connector;
            }
            this._connectors = $.extend(true, [], connectors);
        },
        _initPhaseCollection: function (phases) {
            var phasesColl = phases ? (phases) : this.model.phases;
            if (phasesColl) {
                for (var i = 0; i < phasesColl.length; i++) {
                    var cphase = phasesColl[i];
                    if (cphase) {
                        var phase = this.nameTable[cphase.name];
                        if (phase)
                            delete this.nameTable[phase.name];
                        this.nameTable[cphase.name] = cphase;
                        if (phases)
                            this.model.phases.push(cphase);
                    }
                }
            }
        },
        //#region Data Source
        _initDataSource: function (dataSource, nodes, connectors) {
            var diagram = this;
            if (dataSource.dataSource.json && dataSource.dataSource.json.length > 0) {
                diagram._applyDataSource(diagram.model.dataSourceSettings, dataSource.dataSource.json, nodes, connectors);
            }
            else if (!dataSource.dataSource.table) {
                var mapper = this.model.dataSourceSettings;
                if (dataSource instanceof ej.DataManager) {
                    $.ajaxSetup({
                        async: false
                    });
                    var query = this._findQuery(mapper, this.model.dataSourceSettings.nodeBinding);
                    var queryPromise = this.model.dataSourceSettings["dataSource"].executeQuery(eval(query));
                    queryPromise.done(function (e) {
                        diagram.retriveData = e.result;
                        diagram._applyDataSource(diagram.model.dataSourceSettings, e.result, nodes, connectors);
                    });
                }
            }
        },
        _findQuery: function (mapper, binding) {
            var column = [], queryManager = ej.Query();
            var query = mapper.query || this.model.dataSourceSettings.query;

            if (ej.isNullOrUndefined(query)) {
                var dataSource = mapper.dataSource || this.model.dataSourceSettings.dataSource;
                var tableName = mapper.tableName || this.model.dataSourceSettings.tableName;;
                for (var col in mapper) {
                    if (col !== "tableName" && col !== "dataSource" && col !== "designTemplate" && !col.match("Binding"))
                        if (mapper[col]) column.push(mapper[col]);
                }
                for (var tempcol in binding) {
                    if (tempcol === "labels") {
                        var labels = binding[tempcol];
                        for (var i = 0; i < labels.length; i++) {
                            if (labels[i].text) column.push(labels[i].text);
                        }
                    } else if (tempcol == "shape" || tempcol == "line") {
                        if (binding[tempcol][type]) col.push(binding[tempcol][type]);
                    } else
                        if (binding[tempcol]) column.push(binding[tempcol]);
                }
                if (column.length > 0)
                    queryManager.select(column);
                if (!dataSource["dataSource"].url.match(tableName + "$"))
                    !ej.isNullOrUndefined(tableName) && queryManager.from(tableName);
            } else
                queryManager = query;
            return queryManager;
        },
        _applyDataSource: function (mapper, data, nodes, connectors) {
            var rootNodes = [];
            var firstNode, r, i, j, item, node, n, obj, firstLvel, nextLevel, nodeTemplate, connectorTemplate;
            for (r = 0; r < data.length; r++) {
                obj = data[r];
                if (rootNodes[obj[mapper.parent]])
                    rootNodes[obj[mapper.parent]].items.push(obj);
                else
                    rootNodes[obj[mapper.parent]] = { items: [obj] };
                if (mapper.root === obj[mapper.id])
                    firstNode = { items: [obj] };
            }
            firstLvel = [];
            if (mapper.parent) {
                if (firstNode)
                    firstLvel.push(firstNode);
                else {
                    for (n in rootNodes) {
                        if ((!n && n !== 0) || n == "undefined" || n == "\"\"" || n == "null") {
                            firstLvel.push(rootNodes[n]);
                        }
                    }
                }
            }
            else firstLvel.push(rootNodes[undefined]);
            if (this.model.nodeTemplate) {
                if (typeof this.model.nodeTemplate === "string") {
                    this.model.nodeTemplate = ej.util.getObject(this.model.nodeTemplate, window);
                }
                if ($.isFunction(this.model.nodeTemplate)) {
                    nodeTemplate = this.model.nodeTemplate;
                }

            }
            if (this.model.connectorTemplate) {
                if (typeof this.model.connectorTemplate === "string") {
                    this.model.connectorTemplate = ej.util.getObject(this.model.connectorTemplate, window);
                }
                if ($.isFunction(this.model.connectorTemplate)) {
                    connectorTemplate = this.model.connectorTemplate;
                }
            }
            for (i = 0; i < firstLvel.length; i++) {
                for (j = 0; j < firstLvel[i].items.length; j++) {
                    item = firstLvel[i].items[j];
                    node = this._applyNodeTemplate(mapper, item, nodeTemplate);
                    nodes.push(node);
                    if (mapper.parent) {
                        nextLevel = rootNodes[node[mapper.id]];
                        if (nextLevel.items)
                            this._renderChildNodes(mapper, nextLevel, node.name, nodes, connectors, rootNodes, nodeTemplate, connectorTemplate);
                    }
                }
            }
        },
        _renderChildNodes: function (mapper, parent, ancestor, nodes, connectors, rootNodes, nodeTemplate, connectorTemplate) {
            var j, child, node, nextLevel;
            for (j = 0; j < parent.items.length; j++) {
                child = parent.items[j];
                node = this._applyNodeTemplate(mapper, child, nodeTemplate);
                nodes.push(node);
                connectors.push(this._applyConnectorTemplate(mapper, null, ancestor, node.name, connectorTemplate));
                nextLevel = rootNodes[child[mapper.id]]
                if (nextLevel)
                    this._renderChildNodes(mapper, nextLevel, node.name, nodes, connectors, rootNodes, nodeTemplate, connectorTemplate);
            }
        },
        _applyTemplate: function (mapper, data, collection, collname) {
            var nodeTemplate, i, node, conn, connectorTemplate;
            if (this.model.nodeTemplate) {
                if (typeof this.model.nodeTemplate === "string") {
                    this.model.nodeTemplate = ej.util.getObject(this.model.nodeTemplate, window);
                }
                if ($.isFunction(this.model.nodeTemplate)) {
                    nodeTemplate = this.model.nodeTemplate;
                }
            }
            if (this.model.connectorTemplate) {
                if (typeof this.model.connectorTemplate === "string") {
                    this.model.connectorTemplate = ej.util.getObject(this.model.connectorTemplate, window);
                }
                if ($.isFunction(this.model.connectorTemplate)) {
                    connectorTemplate = this.model.connectorTemplate;
                }
            }
            if (collname == "nodes") {
                for (i = 0; i < data.length; i++) {
                    node = this._applyNodeTemplate(mapper, data[i], nodeTemplate);
                    collection.push(node);
                }
            }
            if (collname == "connectors") {
                for (i = 0; i < data.length; i++) {
                    conn = this._applyConnectorTemplate(mapper, data[i], null, null, connectorTemplate);
                    collection.push(conn);
                }
            }
        },

        _applyNodeTemplate: function (mapper, item, nodeTemplate) {
            var root = item;
            if (root.name === "" || !root.name)
                root.name = ej.datavisualization.Diagram.Util.randomId();
            if (root)
                root = this._getNewNode(root);
            if (nodeTemplate) {
                nodeTemplate(this, root);
            }
            if (root._type === "group") {
                root = this._getNewGroup(root);
                this._initGroupNode(root);
            }
            return root;
        },
        _applyConnectorTemplate: function (mapper, item, sourceNode, targetNode, connectorTemplate) {
            var connector = {};
            connector.name = ej.datavisualization.Diagram.Util.randomId();
            connector.sourceNode = sourceNode;
            connector.targetNode = targetNode;
            connector = this._getNewConnector(connector);
            if (connectorTemplate) {
                connectorTemplate(this, connector);
            }
            return connector;
        },
        _collapseNodes: function (nodes) {
            var node;
            for (var i = 0; i < nodes.length; i++) {
                node = nodes[i];
                this._collapseChildren(node);
            }
        },
        //#endregion
        //#region Get elements
        _getNewNode: function (options) {
            if (options.shape && typeof options.shape === "object")
                ej.datavisualization.Diagram.Util._updateShapeProperties(options);
            return ej.datavisualization.Diagram.Node($.extend(true, {}, this.model.defaultSettings.node, options));
        },
        _getNewGroup: function (options) {
            var child = null, index = 0;
            if (options.children && options.children.length > 0) {
                for (var i in options.children) {
                    child = this.nameTable[this._getChild(options.children[i])];
                    if (child) {
                        ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[child.name], child);
                        ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, child);
                        if (child.segments) {
                            index = this.connectors().indexOf(child);
                            delete this.nameTable[child.name];
                            this.nameTable[child.name] = this._getNewConnector(child);
                            if (index >= 0) this.connectors()[index] = this.nameTable[child.name];
                            this._dock(this.nameTable[child.name], this.nameTable);
                            ej.datavisualization.Diagram.Util.updateBridging(this.nameTable[child.name], this);
                        } else if (child._type === "node") {
                            index = this.nodes().indexOf(child);
                            delete this.nameTable[child.name];
                            this.nameTable[child.name] = this._getNewNode(child);
                            if (index >= 0) this.nodes()[index] = this.nameTable[child.name];
                        } else if (child._type === "group") {
                            index = this.nodes().indexOf(child);
                            delete this.nameTable[child.name];
                            this.nameTable[child.name] = this._getNewGroup(child);
                            if (index >= 0) this.nodes()[index] = this.nameTable[child.name];
                        }
                    }
                }
            }
            return ej.datavisualization.Diagram.Group($.extend(true, {}, this.model.defaultSettings.node, options));
        },
        _getNewConnector: function (options) {
            var obj = $.extend(true, {}, this.model.defaultSettings.connector, options);
            if (this.model.defaultSettings.connector && (this.model.defaultSettings.connector.segments && this.model.defaultSettings.connector.segments[0])) {
                obj.defaultType = this.model.defaultSettings.connector.segments[0].type;
            }
            ej.datavisualization.Diagram.Util._initConnectionEnds(obj, this);
            return ej.datavisualization.Diagram.Connector(obj);
        },
        //#endregion
        _setOverview: function (overview, id) {
            if (overview) {
                this._overview = overview;
                this._viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent, true);
                if (!this._views[overview._id]) {
                    this._views.push(overview._id);
                    this._views[overview._id] = { "context": ej.datavisualization.Diagram.SvgContext, "type": "overview" };
                    overview._renderDocument(this._views[overview._id]);
                    var view = this._views[overview._id];
                    var nodes = this._setNodeZOrder(view);
                    this._renderDiagramObjects(nodes, view, null, true);
                    overview._updateOverview(view);
                }
                delete this._viewPort;
            } else {
                overview = $("#" + id).ejOverview("instance");
                overview._removeDocument(this._views[id]);
                this._views[id] = undefined;
                var index = this._views.indexOf(id);
                this._views.splice(index, 1);
                //this._overview = null;
            }
        },
        _updateScrollOffset: function (hScrollOffset, vScrollOffset, canScale) {
            this._hScrollOffset = Number(hScrollOffset);
            this._vScrollOffset = Number(vScrollOffset);
            this._verticalOffset(Number(vScrollOffset));
            this._horizontalOffset(Number(hScrollOffset));
            this._viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            ej.datavisualization.Diagram.ScrollUtil._transform(this, hScrollOffset, vScrollOffset, canScale);
            var diagram = this;
            this._views.forEach(function (viewid) {
                var view = diagram._views[viewid];
                if (view.type == "overview") {
                    var ovw = $("#" + viewid).ejOverview("instance");
                    if (ovw)
                        ovw._scrollOverviewRect(hScrollOffset, vScrollOffset, diagram._currZoom);
                }
            });
            this.model.scrollSettings.viewPortHeight = this._viewPort.height;
            this.model.scrollSettings.viewPortWidth = this._viewPort.width;
            this.model.scrollSettings.currentZoom = this._currZoom;
        },
        _getChild: function (child) {
            return ej.datavisualization.Diagram.Util.getChild(child);
        },
        _getChildren: function (children) {
            if (children) {
                var children1 = [];
                for (var i = 0; i < children.length; i++) {
                    var child = children[i];
                    if (child) {
                        if (typeof (child) == "object") {
                            children1.push(child.name);
                        } else {
                            children1.push(child);
                        }
                    }
                }
                return children1;
            }
        },
        _initGroupNode: function (group) {
            var child = null;
            for (var i = 0; i < group.children.length; i++) {
                child = group.children[i];
                if (child && typeof (child) == "object") {
                    if (typeof child.shape !== "object")
                        child = ej.datavisualization.Diagram.NodeType(child, this);
                    if (child._type != "group" && !child.children && !child.segments && child.type != "connector") {
                        if (group.type != "bpmn") child = this._getNewNode(child);
                        if (child.name == "")
                            child.name = ej.datavisualization.Diagram.Util.randomId();
                        if (child._type == "node" && child.labels.length && (child.width == 0 || child.height == 0))
                            this._getNodeDimension(child);
                    }
                    else if (child.segments || child.type == "connector") {
                        child = this._getNewConnector(child);
                        if (child.name == "") {
                            child.name = ej.datavisualization.Diagram.Util.randomId();
                        }
                    }
                    else {
                        child = this._getNewGroup(child);
                        if (child.name == "") {
                            child.name = ej.datavisualization.Diagram.Util.randomId();
                        }
                        this._initGroupNode(child);
                    }
                    group.children[i] = child.name;
                    child.parent = group.name;
                    this.nameTable[child.name] = child;
                    if (group.type != "bpmn") this.nodes()[this.nodes().length] = this.nameTable[child.name];
                }
                this._updateQuad(this.nameTable[this._getChild(group.children[i])]);
            }
        },
        _getNodeDimension: function (node) {
            var prevWidth = node._width, prevHeight = node._height;
            var width = node.width ? node.width : (node.maxWidth ? node.maxWidth : node.minWidth);
            var text = document.createElement("span");
            var w = 0, h = 0, label, i;
            if (node.labels.length) {
                ej.datavisualization.Diagram.Util.attr(text, { "id": node.name + "_label", "class": "ej-d-label", "style": "display: inline-block; position: absolute; pointer-events: all; line-height: normal;" });
                this.element[0].appendChild(text);
                for (i = 0; i < node.labels.length; i++) {
                    label = node.labels[i];
                    if (label.bold) text.style.fontWeight = "bold";
                    if (label.italic) text.style.fontStyle = "italic";
                    text.style.textDecoration = label.textDecoration;
                    text.style.fontFamily = label.fontFamily;
                    text.style.fontSize = label.fontSize + "px";
                    text.style.color = label.fontColor;
                    text.style.backgroundColor = label.fillColor;
                    text.style.borderColor = label.borderColor;
                    text.style.borderWidth = label.borderWidth + "px";
                    text.textContent = label.text;
                    if (label.wrapping == "nowrap") {
                        text.style.whiteSpace = "nowrap";
                        text.style.wordWrap = "normal";
                    } else {
                        if (label.wrapping == "wrap") {
                            text.style.wordBreak = "break-all";
                        }
                        text.style.wordWrap = "break-word";
                    }
                    if (node.maxWidth) text.style.maxWidth = width - (label.margin.left + label.margin.right) - label.fontSize + "px";
                    if (node.minWidth) text.style.minWidth = width - (label.margin.left + label.margin.right) - label.fontSize + "px";
                    w += text.offsetWidth + label.margin.left + label.margin.right;
                    h += text.offsetHeight + label.margin.top + label.margin.bottom;
                }
                node._width = Math.max(node.minWidth || 0, w || 0);
                if (node.maxWidth) node._width = Math.min(node.maxWidth, node._width);
                node._height = Math.max(node.minHeight || 0, h || 0);
                if (node.maxHeight) node._height = Math.min(node.maxHeight, node._height);
                if (!node.width && node.labels[0].horizontalAlignment == "left") node.offsetX += (node._width - prevWidth) / 2;
                if (!node.width && node.labels[0].horizontalAlignment == "right") node.offsetX -= (node._width - prevWidth) / 2;
                if (!node.height && node.labels[0].verticalAlignment == "top") node.offsetY += (node._height - prevHeight) / 2;
                if (!node.height && node.labels[0].verticalAlignment == "bottom") node.offsetY -= (node._height - prevHeight) / 2;
                if (node.parent) var parent = this.nameTable[node.parent];
                if (parent && this._svg) {
                    var resizeTool = this.tools["resize"];
                    resizeTool._updateContainer(node, parent, true);
                }
                this.element[0].removeChild(text);
            }
        },
        _getTightChildrenBounds: function (node) {
            var children = this._getChildren(node.children), rect, bounds = null, child;
            for (var i = 0, len = children.length; i < len; i++) {
                child = this.nameTable[children[i]];
                if (child) {
                    var rect1 = ej.datavisualization.Diagram.Util.bounds(child);
                    rect = ej.datavisualization.Diagram.Geometry.rect([rect1.topLeft, rect1.topRight, rect1.bottomRight, rect1.bottomLeft]);
                    if (child.verticalAlign == "stretch") {
                        rect.height = 0;
                    } else if (child.horizontalAlign == "stretch") {
                        rect.width = 0;
                    }
                }
                if (bounds)
                    bounds = this._union(rect, bounds);
                else
                    bounds = rect;
            }
            return bounds;
        },
        _getBoundsWithoutStretch: function (node, vertical, horizontal) {
            var children = this._getChildren(node.children), rect, rect1, bounds = null, child;
            for (var i = 0, len = children.length; i < len; i++) {
                child = this.nameTable[children[i]];
                rect1 = ej.datavisualization.Diagram.Util.bounds(child);
                rect = ej.datavisualization.Diagram.Geometry.rect([rect1.topLeft, rect1.topRight, rect1.bottomRight, rect1.bottomLeft]);
                if (vertical) {
                    if (child.verticalAlign == "stretch" && !(child.verticalAlign == "stretch" && child.horizontalAlign == "stretch")) {
                        rect.height = 0;
                    }
                } else if (horizontal) {
                    if (child.horizontalAlign == "stretch" && !(child.verticalAlign == "stretch" && child.horizontalAlign == "stretch")) {
                        rect.width = 0;
                    }
                }

                if (bounds)
                    bounds = this._union(rect, bounds);
                else
                    bounds = rect;
            }
            bounds.width += node.paddingLeft + node.paddingRight;
            bounds.height += node.paddingTop + node.paddingBottom;
            return bounds;
        },
        _getPhaseStack: function (group) {
            var stack;
            if (group && group.isSwimlane) {
                if (group.children && group.children.length > 0) {
                    stack = this.nameTable[this._getChild(group.children[1])];
                }
            }
            return stack;
        },
        _getMinMaxSize: function (node, width, height, stretch, isExeed) {
            var nw = null, nh = null;
            if (width >= node.minWidth && width <= node.maxWidth) {
                nw = width;
            } else {
                if (node.horizontalAlign == "stretch" && node.parent && !stretch && !isExeed) {
                    var group = this.nameTable[node.parent];
                    if (group) {
                        var rSize = ej.datavisualization.Diagram.Util.bounds(group); // ej.datavisualization.Diagram.SvgContext._measureCanvasSize(group, this);
                        if (rSize.width == 0 && rSize.height == 0) {
                            rSize.width = width;
                            rSize.height = height;
                        }
                        var newSize = this._getMinMaxSize(group, rSize.width, rSize.height);
                        if (group.isSwimlane && group.orientation == "vertical") {
                            var head = this.nameTable[group.children[0]];
                            if (head)
                                newSize.height -= head.height;
                        }
                        newSize.width -= (group.paddingLeft + group.paddingRight + group.marginLeft + group.marginRight);
                        newSize.height -= (group.paddingTop + group.paddingBottom + group.marginTop + group.marginBottom);
                        nw = newSize.width ? newSize.width : width;
                    }
                }
                else if (isExeed) {
                    var group = this.nameTable[node.parent];
                    if (group) {
                        var rSize = ej.datavisualization.Diagram.Util.bounds(group); // ej.datavisualization.Diagram.SvgContext._measureCanvasSize(group, this);
                        var newSize = this._getMinMaxSize(group, rSize.width, rSize.height);
                        nw = newSize.width ? newSize.width : width;
                    }
                }
                else if (width < node.minWidth && node.minWidth != 0)
                    nw = node.minWidth;
                else if (width > node.maxWidth && node.maxWidth != 0 && !node.isSwimlane)
                    nw = node.maxWidth;
                else
                    nw = width;
            }
            if (height >= node.minHeight && height <= node.maxHeight) {
                nh = height;
            } else {
                var group = this.nameTable[node.parent];
                if (node.verticalAlign == "stretch" && node.parent && !stretch && !node.isLane) {
                    var rSize = ej.datavisualization.Diagram.Util.bounds(group);
                    if (rSize.width == 0 && rSize.height == 0) {
                        //rSize.width = width;
                        //rSize.height = height;
                        var status = null;
                        var state = false;
                        rSize.width = node.minWidth;
                        rSize.height = node.minHeight;
                        if (node.minWidth == 0 || node.minHeight == 0) {
                            rSize.width = width;
                            rSize.height = height;
                            status = false;
                            state = true;
                        }
                        if (!state)
                            status = true;
                    }
                    var newSize = this._getMinMaxSize(group, rSize.width, rSize.height);
                    newSize.width -= (group.paddingLeft + group.paddingRight + group.marginLeft + group.marginRight);
                    newSize.height -= (group.paddingTop + group.paddingBottom + group.marginTop + group.marginBottom);
                    if (status) {
                        newSize.height -= node.marginTop;

                    }
                    nh = newSize.height ? newSize.height : height;
                } else if (height < node.minHeight && node.minHeight != 0)
                    nh = node.minHeight;
                else if (height > node.maxHeight && node.maxHeight != 0)
                    nh = node.maxHeight;
                else
                    nh = height;
            }
            return { width: nw, height: nh };
        },
        _updateConnectorBridging: function (baseConnector) {
            var quads = this._getConnectors();
            for (var q = 0; q < quads.length; q++) {
                var connector = quads[q];
                if (connector.visible) {
                    if (connector.segments) {
                        if (connector != baseConnector && ej.datavisualization.Diagram.Util.canBridge(connector, this)) {
                            ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                            ej.datavisualization.Diagram.DiagramContext.update(connector, this);
                        }
                    }
                }
            }
        },
        _drawToolShape: function (type, args) {
            if (type == "rectangle" || type == "ellipse" || type == "image" || type == "path" || type == "polygon" || type == "native" || type == "html" || type === "text") {
                this.activateTool("shapeTool", args);
            }
            else if (type == "straightLine") {
                this.activateTool("straightLine", args);
            }
            else if (type == "orthogonalLine") {
                this.activateTool("orthogonalLine", args);
            }
        },
        _destroy: function () {
            this.element.empty().removeClass("e-datavisualization-diagram");
        },
        //#endregion       
        _updateScrollSettings: function (option) {
            var isLimited = this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Limited ? true : false;
            var scrollableArea = this.model.pageSettings.scrollableArea;
            if ((option.horizontalOffset || option.horizontalOffset === 0)) {
                var hValue = Number((typeof option.horizontalOffset === 'function' ? option.horizontalOffset() : option.horizontalOffset));
                if (this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity || ((hValue >= this._hScrollbar.model.minimum && hValue <= this._hScrollbar.model.maximum) || (isLimited && hValue >= scrollableArea.x && hValue <= scrollableArea.x + scrollableArea.width)))
                    this._updateScrollOffset(hValue, this._verticalOffset());
            }
            if ((option.verticalOffset || option.verticalOffset === 0)) {
                var vValue = Number((typeof option.verticalOffset === 'function' ? option.verticalOffset() : option.verticalOffset));
                if (this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity || ((vValue >= this._vScrollbar.model.minimum && vValue <= this._vScrollbar.model.maximum) || (isLimited && vValue >= scrollableArea.y && vValue <= scrollableArea.y + scrollableArea.height)))
                    this._updateScrollOffset(this._horizontalOffset(), vValue);
            }
            if (option.zoomFactor)
                this.zoomFactor((typeof option.zoomFactor === 'function' ? option.zoomFactor() : option.zoomFactor));
            if (option.padding) {
                this.model.scrollSettings.padding = option.padding;
                this._updateScrollOffset(this._horizontalOffset(), this._verticalOffset());
            }

        },
        updatePageSettings: function (option) {
            if (option) {
                if (option.pageHeight) {
                    this._pageHeight(Number(typeof option.pageHeight === 'function' ? option.pageHeight() : option.pageHeight));
                }
                if (option.pageWidth) {
                    this._pageWidth(Number(typeof option.pageWidth === 'function' ? option.pageWidth() : option.pageWidth));
                }
                if (option.pageBorderWidth) {
                    this._pageBorderWidth(Number(typeof option.pageBorderWidth === 'function' ? option.pageBorderWidth() : option.pageBorderWidth));
                }
                if (option.pageMargin) {
                    this._pageMargin(Number(typeof option.pageMargin === 'function' ? option.pageMargin() : option.pageMargin));
                }
                if (option.pageBackgroundColor) {
                    this._pageBackgroundColor(typeof option.pageBackgroundColor === 'function' ? option.pageBackgroundColor() : option.pageBackgroundColor);
                }
                if (option.scrollLimit) {
                    this._scrollLimit(typeof option.scrollLimit === 'function' ? option.scrollLimit() : option.scrollLimit);
                }
                if (option.scrollableArea) {
                    this.model.pageSettings.scrollableArea = option.scrollableArea;
                }
                if (option.pageBorderColor) {
                    this._pageBorderColor(typeof option.pageBorderColor === 'function' ? option.pageBorderColor() : option.pageBorderColor);
                }
                ej.datavisualization.Diagram.PageUtil._updatePageSize(this);
                ej.datavisualization.Diagram.SvgContext._updateBackground(this._hScrollOffset, this._vScrollOffset, this._currZoom, this);
                ej.datavisualization.Diagram.SvgContext._updateGrid(this._hScrollOffset, this._vScrollOffset, this._currZoom, this);
            }
        },
        _updateSnapSettings: function (option) {
            if (option) {
                if (option.snapAngle) {
                    this._snapAngle(Number(typeof option.snapAngle === 'function' ? option.snapAngle() : option.snapAngle));
                }
                if (option.snapObjectDistance) {
                    this._snapObjectDistance(Number(typeof option.snapObjectDistance === 'function' ? option.snapObjectDistance() : option.snapObjectDistance));
                }
            }
        },
        //#region Public methods
        _cloneObject: function (node) {
            if ((this.getObjectType(node) === "node" || this.getObjectType(node) === "connector") && node.type != "bpmn") {
                node = $.extend(true, {}, node);
                return node;
            }
            else if ((this.getObjectType(node) === "group" || node.type === "bpmn") && !node.isSwimlane) {
                var group = ($.extend(true, {}, node));
                for (var i = 0; group.children && group.children.length > 0 && i < group.children.length; i++) {
                    group.children[i] = this._cloneObject(this.nameTable[this._getChild(group.children[i])]);
                    group.children[i].parent = group.name;
                }
                return group;
            }
            else if (this.getObjectType(node) === "group" && node.isSwimlane) {
                var swmlnObj = this.getNode(node);
                return swmlnObj;
            }
        },
        _cloneSelectionList: function () {
            var obj = [];
            for (var i = 0 && this.selectionList && this.selectionList.length > 0; i < this.selectionList.length; i++) {
                if (this.getObjectType(this.selectionList[i]) === "pseudoGroup") {
                    var pGroup = this.selectionList[0];
                    for (var j = 0 ; pGroup.children && pGroup.children.length > 0 && j < pGroup.children.length; j++) {
                        obj.push(this._cloneObject(this.nameTable[this._getChild(pGroup.children[j])]));
                    }
                }
                else {
                    obj.push(this._cloneObject(this.selectionList[i]));
                }
            }
            return obj;
        },
        getNode: function (name, childTable) {
            if (name) {
                return this._getNode(name, childTable);
            }
        },
        addPhase: function (name, options) {
            this._addPhase(name, options);
        },
        stopEvents: function (evt) {
            if (this.activeTool.inAction) {
                if (!evt.originalEvent.target && evt.target) {
                    evt.originalEvent.target = evt.target;
                    this._mouseup(evt);
                }
            }
        },
        _updateSwimlanePhase: function (phase, options, selupdate, updateChildren) {
            if (phase && options) {
                if (options.label) {
                    var psStkNode = this.nameTable["phaseStack" + phase.name];
                    if (psStkNode) {
                        options.label = $.extend(true, {}, options.label);
                        options.label.rotateAngle = (options.orientation === "vertical") ? 270 : 0;
                        options.label.name = psStkNode.labels[0].name;
                        options.label.mode = ej.datavisualization.Diagram.LabelEditMode.View
                        phase.label = options.label;
                        this.updateLabel(psStkNode.name, psStkNode.labels[0], phase.label);
                    }
                }
                if (options.lineColor || options.lineDashArray || options.lineWidth) {
                    var attr = {}
                    if (options.lineColor) {
                        phase.lineColor = options.lineColor;
                        attr["stroke"] = options.lineColor;
                    }
                    if (options.lineDashArray) {
                        phase.lineDashArray = options.lineDashArray;
                        attr["stroke-dasharray"] = options.lineDashArray;
                    }
                    if (options.lineWidth) {
                        phase.lineWidth = options.lineWidth;
                        attr["stroke-width"] = options.lineWidth;
                    }
                    ej.datavisualization.Diagram.SvgContext._updatePhaseStyle(phase, attr, this);
                }
                if (options.offset != phase.offset) {
                    var dx = .0001, dy = .0001;
                    this.tools["phase"].selectedSeperetor = phase;
                    if (phase.orientation === "horizontal")
                        dx = options.offset - phase.offset;
                    else if (phase.orientation === "vertical")
                        dy = options.offset - phase.offset;
                    this.tools["phase"]._updateSwimlane(phase, dx, dy, (phase.orientation === "vertical") ? true : false, true, updateChildren);
                    if (!selupdate) {
                        this._clearSelection(true);
                        this.addSelection(phase, true);
                    }
                    this.tools["phase"].selectedSeperetor = null;
                }
            }
        },
        _updateNodeVisibility: function (node, options) {
            if (node) {
                if (node.type === "group") {
                    for (var i = 0; i < node.children.length; i++) {
                        var child = this.nameTable[node.children[i]];
                        if (child)
                            this._updateNodeVisibility(child, options);
                    }
                }
                node.visible = options.visible;
            }
        },
        updateNode: function (name, options) {
            if (name) {
                var node = this._findNode(name);
                //  if (!(options.isSwimlane || options.isLane || options.isPhase || options.type === "phase")) {

                var cont = (node && (node.isSwimlane || node.isLane || node.isPhase || node.type === "phase"));
                if ((cont) || !cont) { // if ((cont && this._needUpdate) || !cont) {
                    this._needUpdate = false;
                    var resource = null;
                    if (node) {
                        if (!this._isUndo)
                            this._recordPropertiesChanged(node, options, "node");
                        if (options.pivot) {
                            if (options.pivot.x !== undefined) {
                                var difx = options.pivot.x - node.pivot.x;
                                node.pivot.x = options.pivot.x;
                                node.offsetX += difx * (node.width || node._width || 0);
                            }
                            if (options.pivot.y !== undefined) {
                                var dify = options.pivot.y - node.pivot.y;
                                node.pivot.y = options.pivot.y;
                                node.offsetY += dify * (node.height || node._height || 0);
                            }

                        }
                        if (options.offsetX | options.offsetX === 0) {
                            var dx = options.offsetX - node.offsetX;
                            resource = { element: node, cause: ej.datavisualization.Diagram.ActionType.Unknown, propertyName: "offsetX", oldValue: node.offsetX, newValue: node.offsetX + dx }
                            this._raisePropertyChange(resource);
                            this._translate(node, dx, 0, this.nameTable);
                            if (node.marginLeft != 0) {
                                node.marginLeft += dx;
                            }
                        }
                        if (options.offsetY | options.offsetY === 0) {
                            var dy = options.offsetY - node.offsetY;
                            resource = { element: node, cause: ej.datavisualization.Diagram.ActionType.Unknown, propertyName: "offsetY", oldValue: node.offsetY, newValue: node.offsetY + dy }
                            this._raisePropertyChange(resource);
                            this._translate(node, 0, dy, this.nameTable);
                            if (node.marginTop != 0) {
                                node.marginTop += dy;
                            }
                        }
                        if (options.rotateAngle | options.rotateAngle === 0) {
                            this._rotate(node, options.rotateAngle - node.rotateAngle, this.nameTable);
                        }
                        if (options.width) {
                            node.width = node.width ? node.width : 1;
                            resource = { element: node, cause: ej.datavisualization.Diagram.ActionType.Unknown, propertyName: "width", oldValue: node.width, newValue: options.width }
                            this._raisePropertyChange(resource);
                            this.scale(node, options.width / node.width, 1, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                            if (node.type === "bpmn" && node._type === "group") {
                                ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                            }
                            if (node.isSwimlane) {
                                node.minWidth = options.width;
                                this._updateLastPhase(node, node.width, true);
                            }
                        }
                        if (options.height) {
                            node.height = node.height ? node.height : 1;
                            resource = { element: node, cause: ej.datavisualization.Diagram.ActionType.Unknown, propertyName: "height", oldValue: node.height, newValue: options.height }
                            this._raisePropertyChange(resource);
                            this.scale(node, 1, options.height / node.height, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                            if (node.type === "bpmn" && node._type === "group") {
                                ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                            }
                            if (node.isSwimlane) {
                                node.minHeight = options.height;
                                this._updateLastPhase(node, node.height, true);
                            }
                        }
                        if (options.connectorPadding != undefined) {
                            node.connectorPadding = options.connectorPadding;
                            this.scale(node, 1, 1, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                        }
                        if (options.visible != undefined) {
                            this._updateNodeVisibility(node, options);
                            if (!options.visible)
                                this._clearSelection();
                        }
                        if (options.constraints) {
                            if (node._type != "group") {
                                if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow || options.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                                    if (!(node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)) {
                                        node.constraints = options.constraints;
                                        ej.datavisualization.Diagram.DiagramContext.renderShadow(node, this);
                                    }
                                    else {
                                        node.constraints = options.constraints;
                                        ej.datavisualization.Diagram.DiagramContext.removeShadow(node, this);
                                    }
                                }
                            }
                            node.constraints = options.constraints;
                            if (this.selectionList[0] && this.selectionList[0].name === node.name) {
                                ej.datavisualization.Diagram.SvgContext.clearSelector(this._adornerSvg, this._adornerLayer);
                                ej.datavisualization.Diagram.SvgContext.renderSelector(this.selectionList[0], this._adornerSvg, this._adornerLayer, this._currZoom, this.model.selectedItems.constraints);
                                if (this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                                    ej.datavisualization.Diagram.SvgContext.renderUserHandles(this.model.selectedItems.userHandles, this.selectionList[0], this._adornerSvg, this.selectionList[0].type === "pseudoGroup",
                                        this._currZoom, this._adornerLayer, this);
                            }
                        }
                        //if (node.type != "group") {
                        if (options.fillColor) {
                            node.fillColor = options.fillColor;
                        }
                        if (options.cssClass || options.cssClass === "") {
                            node.cssClass = options.cssClass;
                        }
                        if (options.opacity || options.opacity === 0) {
                            node.opacity = options.opacity;
                        }
                        if (options.borderColor) {
                            node.borderColor = options.borderColor;
                        }
                        if (options.borderWidth || options.borderWidth === 0) {
                            node.borderWidth = options.borderWidth;
                        }
                        if (options.borderDashArray) {
                            node.borderDashArray = options.borderDashArray;
                        }
                        if (options.gradient !== undefined)
                            node.gradient = options.gradient;
                        if (options.tooltip !== undefined)
                            node.tooltip = ej.datavisualization.Diagram.Tooltip($.extend(true, node.tooltip, {}, options.tooltip));
                        node = this._findNode(name);
                        if (options.shape && typeof options.shape == "object") {
                            ej.datavisualization.Diagram.Util._updateShapeProperties(options);
                            if (ej.browserInfo().name === "msie") {
                                if (node.type == "html") {
                                    var htmlelement = document.getElementById(node.name + "_parentdiv");
                                    htmlelement.parentNode.removeChild(htmlelement);
                                }
                            }
                            ej.datavisualization.Diagram.DiagramContext.setNodeShape(node, this);
                        }
                        else if (this.getObjectType(node) == "node")
                            node = this._updateShape(node, options);
                        if (node && node.type != "phase") {
                            ej.datavisualization.Diagram.DiagramContext.update(node, this);
                            if (node.type == "bpmn")
                                ej.datavisualization.Diagram.DiagramContext.updateBPMNNodeStyle(node, this);
                            else
                                ej.datavisualization.Diagram.DiagramContext.updateNodeStyle(node, this);
                        }
                        if (this.selectionList[0] && this.selectionList[0].name === node.name)
                            ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);

                        if (options.isExpanded !== undefined) {
                            //if (this.model.layout.type == "hierarchicaltree")
                            {
                                if (options.isExpanded) {
                                    node.isExpanded = true;
                                    this._expandChildren(node);
                                }
                                else {
                                    node.isExpanded = false;
                                    this._collapseChildren(node);
                                }
                                this.layout();
                                this._addSelection(node);
                            }
                        }
                    }
                }
                if (name) {
                    var node = this._findNode(name);
                    if (node && node._type === "node" && options && options.type === "group" && options.children && options.children.length > 0) {
                        var isRendered = false;
                        if (this._svg.document.getElementById(node.name))
                            isRendered = true;
                        this.remove(node);
                        node.borderColor = "transparent";
                        node.borderDashArray = "";
                        node.borderWidth = 1;
                        node.fillColor = "#1BA0E2";
                        node.labels = [];
                        node.fillColor = "";
                        node.type = options.type;
                        node.children = options.children;
                        node = ej.datavisualization.Diagram.Group(node);
                        this._initGroupNode(node);
                        if (isRendered) {
                            ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                            this.add(node);
                            //this._clearSelection(true);
                            //this._addSelection(node);
                        }
                    } else {
                    }
                }
                if (node && (node.isLane || node.isSwimlane || node.type === "phase" || node.isPhase)) {
                    this._needUpdate = true;
                    if (node.isLane)
                        this._updateLane(node, options);
                    else if (node.isSwimlane) {
                        this._needUpdate = false;
                        this._updateSwimlane(node, options, true);
                    }
                    else if (node.type === "phase" || node.phase) {
                        this._needUpdate = false;
                        this._updateSwimlanePhase(node, options);
                    }
                }
                else {
                    if (node && node.parent && this.nameTable[node.parent]) {
                        var parent = this.nameTable[node.parent];
                        this._updateParentBounds(parent.name);
                        this.tools.resize._updateContainer(node, parent, false);
                    }
                }
            }
        },
        _requirePhaseUpdate: function (node) {
            if (node && node.parent) {
                var parent = this.nameTable[this._getChild(node.parent)];
                if (parent && parent.isLane) {
                    var canvasStack = this.nameTable[this._getChild(parent.parent)];
                    if (canvasStack) {
                        var swimlane = this.nameTable[this._getChild(canvasStack.parent)];
                        this.tools.move._updateMargin(node, parent);
                        this.tools.move.selectedObject = node;
                        this.tools.move._updateContainer(node);
                        this.tools.move.selectedObject = null;
                    }
                }
            }
        },

        _updateShape: function (node, options) {
            var isShapeModified = false;
            if (options.type) {
                node.type = options.type;
                node._shape = options.shape ? options.shape : options.type;
                isShapeModified = true;
            }
            if (options.shape) {
                node.shape = options.shape;
                isShapeModified = true;
            }
            if (node.type != "bpmn") {
                if (options.cornerRadius) {
                    node.cornerRadius = options.cornerRadius;
                    isShapeModified = true;
                }
                if (options.pathData) {
                    node.pathData = options.pathData;
                    isShapeModified = true;
                }
                if (options.templateId) {
                    node.templateId = options.templateId;
                    isShapeModified = true;
                }
                if (options.html) {
                    node.html = options.html;
                    isShapeModified = true;
                }
                if (options.contentId) {
                    node.contentId = options.contentId;
                    isShapeModified = true;
                }
                if (options.source) {
                    node.source = options.source;
                    isShapeModified = true;
                }
                if (options.textBlock) {
                    node.textBlock = options.textBlock;
                    isShapeModified = true;
                }
                if (options.points) {
                    node.points = options.points;
                    isShapeModified = true;
                }
                if (isShapeModified) {
                    if (node.children) delete node.children;
                    node = ej.datavisualization.Diagram.NodeType(node, this);
                    if (node.type == "html" && ej.browserInfo().name === "msie") {
                        var htmlelement = document.getElementById(node.name + "_parentdiv");
                        htmlelement.parentNode.removeChild(htmlelement);
                    }
                    ej.datavisualization.Diagram.DiagramContext.setNodeShape(node, this);
                    this._clearSelection(true);
                    this._addSelection(node, true);
                }
            }
            else {
                var shape = $.extend(true, {}, this.nameTable[node.name]);
                if (options.event) {
                    shape.event = options.event;
                    isShapeModified = true;
                }
                if (options.trigger) {
                    shape.trigger = options.trigger;
                    isShapeModified = true;
                }
                if (options.gateway) {
                    shape.gateway = options.gateway;
                    isShapeModified = true;
                }
                if (options.collection != undefined) {
                    shape.collection = options.collection;
                    isShapeModified = true;
                }
                if (options.activity) {
                    shape.activity = options.activity;
                    isShapeModified = true;
                }
                if (options.task) {
                    shape.task = $.extend(true, {}, shape.task, options.task);
                    isShapeModified = true;
                }
                if (options.subProcess) {
                    shape.subProcess = $.extend(true, {}, shape.subProcess, options.subProcess);
                    isShapeModified = true;
                }
                if (options.direction) {
                    shape.direction = options.direction;
                    isShapeModified = true;
                }
                if (isShapeModified) {
                    var nextNode = this._svg.getElementById(node.name).nextSibling;
                    this._removeElement(node);
                    ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[node.name], node);
                    ej.datavisualization.Diagram.Util.removeItem(this.nodes(), this.nameTable[node.name]);
                    this._nodes = $.extend(true, [], this.nodes());
                    node = ej.datavisualization.Diagram.NodeType(shape, this);
                    node = ej.datavisualization.Diagram.Group(node);
                    this._initGroupNode(node);
                    this.nameTable[node.name] = node;
                    this.nodes().push(node);
                    this._nodes = $.extend(true, [], this.nodes());
                    this._updateQuad(node);
                    ej.datavisualization.Diagram.DiagramContext.renderGroup(node, this);
                    if (nextNode) this._svg.getElementById(node.name).parentNode.insertBefore(this._svg.getElementById(node.name), this._svg.getElementById(nextNode.id));
                    this._clearSelection(true);
                    this._addSelection(node, true);
                }
            }
            return node;
        },
        _updateObject: function (node) {
            if (node) {
                if (node.type == "pseudoGroup") {
                    var child;
                    var children = this._getChildren(node.children);
                    for (var i = 0, len = children.length; i < len; i++) {
                        child = this.nameTable[children[i]];
                        ej.datavisualization.Diagram.DiagramContext.update(child, this);
                    }
                }
                else {
                    if (node.parent) {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(this.nameTable[node.parent], this);
                    }
                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                }
            }
        },
        _udatePhaseSize: function (node, options) {
            var phsStack = this.nameTable[this._getChild(node.children[1])];
            if (node.orientation === "vertical") {
                if (phsStack.minWidth != options.phaseSize) {
                    phsStack.width = phsStack.maxWidth;
                    phsStack.minWidth = options.phaseSize;
                    var xdif = phsStack.maxWidth - options.phaseSize;
                    var xVar = (options.phaseSize / phsStack.maxWidth)
                    phsStack.maxWidth = options.phaseSize;
                    this.scale(phsStack, xVar, .0001, ej.datavisualization.Diagram.Point(phsStack.offsetX, phsStack.offsetY), this.nameTable);
                    if (phsStack.children && phsStack.children.length > 0) {
                        for (var i = 0; i < phsStack.children.length; i++) {
                            var child = this.nameTable[this._getChild(phsStack.children[i])];
                            if (child) {
                                this.scale(child, xVar, .0001, ej.datavisualization.Diagram.Point(child.offsetX, child.offsetY), this.nameTable);
                            }
                        }
                    }
                    var laneStack = this.nameTable[this._getChild(node.children[2])];
                    laneStack.marginLeft -= xdif;
                }
            }
            else if (node.orientation === "horizontal") {
                if (phsStack.minHeight != options.phaseSize) {
                    phsStack.height = phsStack.maxHeight;
                    phsStack.minHeight = options.phaseSize;
                    var ydif = phsStack.maxHeight - options.phaseSize;
                    var yVar = (options.phaseSize / phsStack.maxHeight)
                    yVar = (yVar === Infinity) ? 0 : yVar;
                    phsStack.maxHeight = options.phaseSize;
                    this.scale(phsStack, .0001, yVar, ej.datavisualization.Diagram.Point(phsStack.offsetX, phsStack.offsetY), this.nameTable);
                    if (phsStack.children && phsStack.children.length > 0) {
                        for (var i = 0; i < phsStack.children.length; i++) {
                            var child = this.nameTable[this._getChild(phsStack.children[i])];
                            if (child) {
                                this.scale(child, .0001, yVar, ej.datavisualization.Diagram.Point(child.offsetX, child.offsetY), this.nameTable);
                            }
                        }
                    }
                    var laneStack = this.nameTable[this._getChild(node.children[2])];
                    laneStack.marginTop -= ydif;
                }
            }
        },

        _updateLane: function (node, options) {
            if (node && options) {
                //#region updateHeader 
                if (options.header) {
                    if (node.children && node.children.length > 0) {
                        var hdname, head;
                        hdname = this._getChild(node.children[0]);
                        if (hdname) {
                            head = this.nameTable[hdname];
                        }
                        if (head) {
                            this.updateLabel(head.name, head.labels[0], options.header);
                            if (options.header.fillColor && options.header.fillColor != head.fillColor) {
                                this.updateNode(head.name, { fillColor: options.header.fillColor })
                            }
                        }
                    }
                }
                //#endregion
                if (options.fillColor && options.fillColor != node.fillColor) {
                    this.updateNode(node.name, { fillColor: options.fillColor })
                }
                if (options.minHeight && options.minHeight != node.minHeight) {
                    node.minHeight = options.minHeight;
                }
                if (options.minWidth && options.minWidth != node.minWidth) {
                    node.minWidth = options.minWidth;
                }
                if (options.constraints && options.constraints != node.constraints) {
                    node.constraints = options.constraints;
                }
            }
        },
        _updateLastPhase: function (node, options, needUpdate) {
            if (node.isSwimlane) {
                if (node.phases && node.phases.length == 1) {
                    var df = 0;
                    var phase = this.nameTable[node.phases[0]];
                    if (node.orientation === "horizontal") {
                        phase.offset = node.minWidth - df;
                        this.updateNode(this.nameTable["phaseStack" + node.phases[0]].name, { width: node.minWidth - df });
                        if (needUpdate) {
                            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                            this.updateNode(this.nameTable["phaseStack" + node.phases[0]].name, { offsetX: bounds.x + node.width / 2 });
                        }
                    } else if (node.orientation === "vertical") {
                        phase.offset = node.minHeight - df
                        var height = phase.offset - df;
                        height = height - (options.header ? options.header.height : 50);
                        this.updateNode(this.nameTable["phaseStack" + node.phases[0]].name, { height: height });

                        if (needUpdate) {
                            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                            this.updateNode(this.nameTable["phaseStack" + node.phases[0]].name, { offsetY: (options.header ? options.header.height : 50) / 2 + bounds.y + node.height / 2 });
                        }
                    }
                }
            }
        },
        _updateSwimlane: function (node, options, update) {
            //#region updateHeader 
            if (options.header) {
                if (node.children && node.children.length > 0) {
                    var hdname, head;
                    hdname = this._getChild(node.children[0]);
                    if (hdname) {
                        head = this.nameTable[hdname];
                    }
                    if (head) {
                        this.updateLabel(head.name, head.labels[0], options.header);
                        if (options.header.fillColor && options.header.fillColor != head.fillColor) {
                            this.updateNode(head.name, { fillColor: options.header.fillColor })
                        }
                    }
                }
            }
            //#endregion
            //#region offset
            if (options.offsetX && options.offsetX != node.offsetX) {
                var dx = options.offsetX - node.offsetX;
                this._translate(node, dx, 0, this.nameTable);
            }
            if (options.offsetY && options.offsetY != node.offsetY) {
                var dy = options.offsetY - node.offsetY;
                this._translate(node, 0, dy, this.nameTable);
            }
            //#endregion  
            //#region phases
            if (options.phases && options.phases.length > 0) {
                var phase;
                for (var i = 0; i < options.phases.length; i++) {
                    phase = this.nameTable[this._getChild(options.phases[i])];
                    if (phase)
                        this._updateSwimlanePhase(phase, options.phases[i], true);
                }
            }
            if (update) {
                if (options.phaseSize || options.phaseSize === 0) {
                    if (options.phaseSize != node.phaseSize)
                        var lastPhase = true;
                    if (node.orientation === "horizontal") {
                        var dy = options.phaseSize - node.phaseSize;
                        var laneStack = this.nameTable[this._getChild(node.children[2])];
                        this._translate(laneStack, 0, dy, this.nameTable);
                        laneStack.marginTop += dy;
                        node.phaseSize += dy;
                        ///updatePhaseStack
                        var phaseStack = this.nameTable[this._getChild(node.children[1])];
                        this._updateChildPhaseNode(phaseStack, 0, node.phaseSize, false);
                        ///update swimlane
                        node.height += dy;
                        node.offsetY += dy / 2;
                    }
                    else {

                        var dx = options.phaseSize - node.phaseSize;
                        var laneStack = this.nameTable[this._getChild(node.children[2])];
                        this._translate(laneStack, dx, 0, this.nameTable);
                        laneStack.marginLeft += dx;
                        node.phaseSize += dx;
                        ///updatePhaseStack
                        var phaseStack = this.nameTable[this._getChild(node.children[1])];
                        this._updateChildPhaseNode(phaseStack, node.phaseSize, 0, true);
                        ///update swimlane Header
                        var header = this.nameTable[this._getChild(node.children[0])];
                        if (header) {
                            var pWidth = header.width;
                            this.scale(header, ((pWidth + dx) / header.width), 0, ej.datavisualization.Diagram.Point(header.offsetX, header.offsetY), this.nameTable);
                            this._translate(header, (header.width - (pWidth)) / 2, 0, this.nameTable);
                        }
                        ///update swimlane
                        node.width += dx;
                        node.offsetX += dx / 2;
                    }
                }
            }
            //#endregion
            //#region lanes
            if (options.lanes && options.lanes.length > 0) {
                var lane;
                for (var i = 0; i < options.lanes.length; i++) {
                    lane = this.nameTable[this._getChild(options.lanes[i])];
                    if (lane) {
                        this._updateLane(lane, options.lanes[i]);
                    }
                }
            }
            //#endregion
            if (!update) {
                if (options && options.phaseSize)
                    var isPhaseUpdate = false;

                if (options.minHeight && options.minHeight != node.minHeight) {
                    node.minHeight = options.minHeight;
                    isPhaseUpdate = true;
                }
                if (options.minWidth && options.minWidth != node.minWidth) {
                    node.minWidth = options.minWidth;
                    isPhaseUpdate = true;
                }
                if (isPhaseUpdate) {
                    this._updateLastPhase(node, options);
                };
                if (options.phaseSize || (options.phaseSize === 0)) {
                    this._udatePhaseSize(node, options);
                    if (options.phaseSize === 0)
                        node.phaseSize = 0;
                }
            }
            ej.datavisualization.Diagram.DiagramContext.update(node, this);
            ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
        },
        _updateChildPhaseNode: function (phaseStack, x, y, isVertical) {
            var dx, dy, i, pHeight, pWidth;
            for (i = 0; i < phaseStack.children.length; i++) {
                var child = this.nameTable[this._getChild(phaseStack.children[i])];
                if (!isVertical) {
                    pHeight = child.height;
                    this.scale(child, (x / child.width), (y / child.height), ej.datavisualization.Diagram.Point(child.offsetX, child.offsetY), this.nameTable);
                    this._translate(child, x, (child.height - pHeight) / 2, this.nameTable);
                }
                else {
                    pWidth = child.width;
                    this.scale(child, (x / child.width), (y / child.height), ej.datavisualization.Diagram.Point(child.offsetX, child.offsetY), this.nameTable);
                    this._translate(child, (child.width - pWidth) / 2, y, this.nameTable);
                }
            }
        },
        updateConnector: function (name, options) {
            if (name) {
                var connector = this._findConnector(name);
                if (connector) {
                    if (!this._isUndo)
                        this._recordPropertiesChanged(connector, options, "connector");
                    if (options.constraints !== undefined) {
                        connector.constraints = options.constraints;
                    }
                    if (options.lineColor) {
                        connector.lineColor = options.lineColor;
                    }
                    if (options.cssClass || options.cssClass === "") {
                        connector.cssClass = options.cssClass;
                    }
                    if (options.lineWidth || options.lineWidth === 0) {
                        connector.lineWidth = options.lineWidth;
                    }
                    if (options.lineDashArray !== undefined) {
                        connector.lineDashArray = options.lineDashArray;
                    }
                    if (options.opacity !== undefined) {
                        connector.opacity = options.opacity;
                    }
                    if (options.cornerRadius !== undefined) {
                        connector.cornerRadius = options.cornerRadius;
                    }
                    if (options.sourcePadding != undefined) {
                        connector.sourcePadding = options.sourcePadding;
                        this._dock(connector, this.nameTable);
                    }
                    if (options.targetPadding != undefined) {
                        connector.targetPadding = options.targetPadding;
                        this._dock(connector, this.nameTable);
                    }
                    if (options.visible != undefined) {
                        connector.visible = options.visible;
                        if (!options.visible)
                            this._clearSelection();
                    }
                    if (options.tooltip !== undefined)
                        connector.tooltip = ej.datavisualization.Diagram.Tooltip($.extend(true, connector.tooltip, {}, options.tooltip));
                    if (options.segments) {
                        connector.segments = options.segments;
                        ej.datavisualization.Diagram.Util._initConnectionEnds(connector, this);
                        if (this.model.defaultSettings.connector && this.model.defaultSettings.connector.segments[0]) {
                            var defaultType = this.model.defaultSettings.connector.segments[0].type;
                        }
                        ej.datavisualization.Diagram.Util._initializeSegments(connector, defaultType);
                        this._dock(connector, this.nameTable);
                        ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                        //this._updateQuad(this, connector);
                        if (connector == this.selectionList[0]) {
                            this._clearSelection(true);
                            this._addSelection(connector);
                        }
                    }
                    var decorator;
                    if (options.targetDecorator) {
                        if (connector.targetDecorator) {
                            decorator = $.extend(true, {}, connector.targetDecorator, options.targetDecorator);
                        } else {
                            decorator = ej.datavisualization.Diagram.Decorator(options.targetDecorator);
                        }
                        connector.targetDecorator = decorator;
                    }
                    if (options.sourceDecorator) {
                        if (connector.sourceDecorator) {
                            decorator = $.extend(true, {}, connector.sourceDecorator, options.sourceDecorator);
                        } else {
                            decorator = ej.datavisualization.Diagram.Decorator(options.sourceDecorator);
                        }
                        connector.sourceDecorator = decorator;
                    }
                    if (options.sourceDecorator || options.targetDecorator) {
                        ej.datavisualization.Diagram.DiagramContext.clearDecorators(connector, this);
                        ej.datavisualization.Diagram.DiagramContext.renderDecorators(connector, this);
                    }
                    if (options.hasOwnProperty("sourceNode") || options.hasOwnProperty("targetNode") || options.hasOwnProperty("targetPort") || options.hasOwnProperty("sourcePort") || options.hasOwnProperty("targetPoint") || options.hasOwnProperty("sourcePoint")) {
                        if (options.sourceNode !== undefined) {
                            var node = this.nameTable[connector.sourceNode];
                            if (node && node.outEdges.indexOf(connector.name) != -1)
                                ej.datavisualization.Diagram.Util.removeItem(node.outEdges, connector.name);
                            connector.sourceNode = options.sourceNode;
                        }
                        if (options.targetNode !== undefined) {
                            var node = this.nameTable[connector.targetNode];
                            if (node && node.inEdges.indexOf(connector.name) != -1)
                                ej.datavisualization.Diagram.Util.removeItem(node.inEdges, connector.name);
                            connector.targetNode = options.targetNode;
                        }
                        if (options.targetPort !== undefined) {
                            connector.targetPort = options.targetPort;
                        }
                        if (options.sourcePort !== undefined) {
                            connector.sourcePort = options.sourcePort;
                        }
                        if (options.targetPoint !== undefined) {
                            ej.datavisualization.Diagram.Util._setLineEndPoint(connector, options.targetPoint, true);
                        }
                        if (options.sourcePoint !== undefined) {
                            ej.datavisualization.Diagram.Util._setLineEndPoint(connector, options.sourcePoint, false);
                        }
                        this._updateEdges(connector);
                        this._dock(connector, this.nameTable);
                    }
                    ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                    if (this._svg) {
                        ej.datavisualization.Diagram.DiagramContext.update(connector, this);
                        ej.datavisualization.Diagram.DiagramContext.updateConnectorStyle(connector, this);
                    }
                    if (this.selectionList[0] && this.selectionList[0].name === connector.name)
                        ej.datavisualization.Diagram.SvgContext.updateSelector(connector, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
                }
            }
        },

        _updateDrawType: function (option) {
            var shape, singleAction = false;
            var tool = option.tool;
            if (tool & ej.datavisualization.Diagram.Tool.ContinuesDraw)
                singleAction = false;
            else if (tool & ej.datavisualization.Diagram.Tool.DrawOnce)
                singleAction = true;
            var drawType = option.drawType ? option.drawType : this.model.drawType;
            if (drawType) {
                if (drawType.type === "connector") {
                    if (drawType.segments && drawType.segments[0].type === "straight")
                        this.activateTool("straightLine", singleAction);
                    else
                        this.activateTool("orthogonalLine", singleAction);
                }
                else if (drawType.type === "text")
                    this.activateTool("text", singleAction);
                else if (drawType.type || drawType.shape) {
                    shape = drawType.shape ? drawType.shape : drawType.type;
                    if (shape)
                        this._drawToolShape(shape, singleAction);
                }
            }
        },

        update: function (option) {
            if (option.scrollSettings) {
                this._updateScrollSettings(option.scrollSettings);
            }
            if (option.tool) {
                this.activeTool._showAllPorts(true);
                var tool = option.tool;
                this.tool(tool);
                if (tool & ej.datavisualization.Diagram.Tool.ContinuesDraw || tool & ej.datavisualization.Diagram.Tool.DrawOnce) {
                    this._updateDrawType(option);
                }
                else if (tool & ej.datavisualization.Diagram.Tool.ZoomPan) {
                    if (ej.datavisualization.Diagram.Util.canPanning(this)) {
                        this._toolToActivate = "panTool";
                        this.activeTool = this.tools[this._toolToActivate];
                        this.activeTool._isMouseDown = false;
                        this._currentCursor = "pointer";
                    }
                }
                else if (tool & ej.datavisualization.Diagram.Tool.MultipleSelect) {
                    this.activateTool("select", false);
                }
            }
            if (option.bridgeDirection) {
                this.bridgeDirection(option.bridgeDirection);
                var connectors = this.connectors();
                for (var i = 0, len = connectors.length; i < len; i++) {
                    var connector = connectors[i];
                    if (connector != baseConnector && ej.datavisualization.Diagram.Util.canBridge(connector, this)) {
                        ej.datavisualization.Diagram.Util.updateBridging(connector, this);
                        ej.datavisualization.Diagram.DiagramContext.update(connector, this);
                    }
                }
            }
        },
        bringToCenter: function (rect) {
            var scale = this._currZoom;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            var actualbounds = { x: rect.x * scale, y: rect.y * scale, width: rect.width * scale, height: rect.height * scale };
            var hoffset = actualbounds.x + actualbounds.width / 2 - viewPort.width / 2;
            var voffset = actualbounds.y + actualbounds.height / 2 - viewPort.height / 2;
            ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, 1, this._hScrollOffset - hoffset, this._vScrollOffset - voffset, null, false);
        },
        bringIntoView: function (bounds) {
            var x = 0, y = 0;
            var scale = this._currZoom;
            bounds.right = bounds.right ? bounds.right : bounds.x + bounds.width;
            bounds.bottom = bounds.bottom ? bounds.bottom : bounds.y + bounds.height;
            bounds = { x: bounds.x * scale, y: bounds.y * scale, right: bounds.right * scale, bottom: bounds.bottom * scale };
            var hoffset = this._hScrollOffset;
            var voffset = this._vScrollOffset;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            if (bounds.right > (-hoffset + viewPort.width)) {
                x = bounds.right - viewPort.width;
            }
            if (bounds.x < -hoffset) {
                x = bounds.x;
            }
            if (bounds.bottom > (-voffset + viewPort.height)) {
                y = bounds.bottom - viewPort.height;
            }
            if (bounds.y < -voffset) {
                y = bounds.y;
            }
            ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, 1, this._hScrollOffset - x, this._vScrollOffset - y, null, false);

        },
        getBridgeSegment: function (startPt, endPt, angle, bridgeSpace, sweep) {
            var path = "A " + bridgeSpace / 2 + " " + bridgeSpace / 2 + "  " + angle + " , 1 " + sweep + " " + endPt.x + "," + endPt.y;
            return path;
        },
        fitToPage: function (mode, region, margin) {
            var zoomFactor, factor, scale = {}, bounds;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            //margin
            margin = margin || {};
            margin = {
                top: !isNaN(margin.top) ? margin.top : 25,
                bottom: !isNaN(margin.bottom) ? margin.bottom : 25,
                left: !isNaN(margin.left) ? margin.left : 25,
                right: !isNaN(margin.right) ? margin.right : 25
            };
            //region
            region = region ? region : "pageSettings";
            //fit mode
            if ((region == "pageSettings" && this.model.pageSettings.pageWidth && this.model.pageSettings.pageHeight) || (this.nodes().length > 0 || this.connectors().length > 0)) {
                mode = mode ? mode : "page";
                if (region != "custom") {
                    bounds = this._getDigramBounds(region);
                }
                var width = bounds.width;
                var height = bounds.height;
                scale.x = (viewPort.width - (margin.left + margin.right)) / width;
                scale.y = (viewPort.height - (margin.top + margin.bottom)) / height;
                var deltaX = this._hScrollOffset;
                var deltaY = this._vScrollOffset;
                var centerX, centerY;
                switch (mode) {
                    case "width":
                        zoomFactor = scale.x;
                        factor = zoomFactor / this._getCurrentZoom();
                        centerX = (viewPort.width - (region != "content" ? bounds.width : width) * zoomFactor) / 2 - bounds.x * zoomFactor;
                        deltaX += centerX + (margin.left - margin.right) / 2 * zoomFactor;
                        //deltaX -= bounds.x * zoomFactor;
                        deltaY -= this._vScrollOffset * factor;
                        deltaY = region != "custom" ? deltaY : deltaY + this._vScrollOffset * factor;
                        break;
                    case "height":
                        zoomFactor = scale.y;
                        factor = zoomFactor / this._getCurrentZoom();
                        //var centerX = (viewPort.width - (bounds.x > 0 ? bounds.width : width) * zoomFactor - bounds.x * zoomFactor) / 2;
                        //deltaX += centerX - (bounds.x * zoomFactor) / 4 + margin.left;
                        //deltaY -= bounds.y * zoomFactor - margin.top;
                        centerX = (viewPort.width - (region != "content" ? bounds.width : width) * zoomFactor) / 2 - bounds.x * zoomFactor;
                        centerY = (viewPort.height - (region != "content" ? bounds.height : height) * zoomFactor) / 2 - bounds.y * zoomFactor;
                        deltaX += centerX + (margin.left - margin.right) / 2 * zoomFactor;
                        deltaY += centerY + (margin.top - margin.bottom) / 2 * zoomFactor;
                        break;
                    case "page":
                        zoomFactor = Math.min(scale.x, scale.y);
                        factor = zoomFactor / this._getCurrentZoom();
                        centerX = (viewPort.width - (region != "content" ? bounds.width : width) * zoomFactor) / 2 - bounds.x * zoomFactor;
                        centerY = (viewPort.height - (region != "content" ? bounds.height : height) * zoomFactor) / 2 - bounds.y * zoomFactor;
                        deltaX += centerX + (margin.left - margin.right) / 2 * zoomFactor;
                        deltaY += centerY + (margin.top - margin.bottom) / 2 * zoomFactor;
                        break;
                }
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, factor, deltaX, deltaY, new ej.datavisualization.Diagram.Point(0, 0), true);
            } else {
                factor = 1 / this._getCurrentZoom();
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, factor, this._hScrollOffset, this._vScrollOffset, new ej.datavisualization.Diagram.Point(0, 0), true);
            }
        },
        exportDiagram: function (options) {
            var fileType, customBounds, content;
            if (!options) options = {};
            //region
            var region = options && options.region ? options.region : "content";
            //mode
            var mode = options && options.mode ? options.mode : "download";
            //margin
            var margin = options.margin || {};
            margin = {
                top: !isNaN(margin.top) ? margin.top : 25,
                bottom: !isNaN(margin.bottom) ? margin.bottom : 25,
                left: !isNaN(margin.left) ? margin.left : 25,
                right: !isNaN(margin.right) ? margin.right : 25
            };
            //region
            var bounds = this._getDigramBounds(region);
            if (options.bounds) {
                customBounds = true;
                bounds = {
                    x: (!isNaN(options.bounds.x) ? options.bounds.x : bounds.x),
                    y: (!isNaN(options.bounds.y) ? options.bounds.y : bounds.y),
                    width: (options.bounds.width || bounds.width),
                    height: (options.bounds.height || bounds.height)
                };
            }
            bounds.x -= margin.left;
            bounds.y -= margin.top;
            bounds.width += margin.left + margin.right;
            bounds.height += margin.top + margin.bottom;
            //Target file name
            var fileName = options.fileName || "diagram";
            if (options.format != "svg") {
                var canvas = this._diagramAsCanvas({ bounds: bounds, margin: margin, region: region }, customBounds);
                fileType = options.format || "jpg";
                var image = content = canvas.toDataURL();
                if (mode == "data") return content;
                if (!(ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10)) {
                    image = image.replace(/^data:[a-z]*;,/, '');
                    var image1 = image.split(',');
                    var byteString = atob(image1[1]);
                    var buffer = new ArrayBuffer(byteString.length);
                    var intArray = new Uint8Array(buffer);
                    for (var i = 0; i < byteString.length; i++) {
                        intArray[i] = byteString.charCodeAt(i);
                    }
                }

            } else {
                fileType = options.format || "svg";
                var svg = content = this._diagramAsSvg({ bounds: bounds, margin: margin });
                if (mode == "data") return content;
                if (!(ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10))
                    buffer = new XMLSerializer().serializeToString(svg);
            }
            if (mode == "download") {
                if (ej.browserInfo().name === "msie" && parseFloat(ej.browserInfo().version) < 10) {
                    alert("Downloading option is not supported in IE-9, Please use the returned data");
                    return content;
                } else {
                    var blob = new Blob([buffer], { type: "text/plain;charset=utf-8" });
                    if (ej.browserInfo().name === "msie")
                        window.navigator.msSaveOrOpenBlob(blob, fileName + '.' + fileType);
                    else {
                        var pom = document.createElement('a');
                        var url = URL.createObjectURL(blob);
                        pom.href = url;
                        pom.setAttribute('download', fileName + '.' + fileType);
                        if (document.createEvent) {
                            var e = document.createEvent("MouseEvents");
                            e.initMouseEvent("click", true, true, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
                            pom.dispatchEvent(e);
                        } else if (pom.fireEvent) {
                            pom.fireEvent("onclick");
                        }
                    }
                }
            }
        },
        _diagramAsSvg: function (options) {
            var svg = new ej.datavisualization.Diagram.Svg({ "id": "diagram_svg", "width": options.bounds.width, "height": options.bounds.height });
            document.body.appendChild(svg.document);

            //var g = this._svg.document.getElementById(this._canvas.id + "_backgroundLayer");
            //g = g.cloneNode(true);
            //svg.appendChild(g);
            //g.setAttribute("transform", "translate(" + (-options.bounds.x) + ", " + (-options.bounds.y) + ")");

            var g = this._svg.document.getElementById(this._canvas.id + "_pageBackground");
            g = g.cloneNode(true);
            svg.appendChild(g);
            g.setAttribute("transform", "translate(" + (-options.bounds.x) + ", " + (-options.bounds.y) + ")");

            g = this._svg.document.getElementById(this._canvas.id + "_diagramLayer");
            g = g.cloneNode(true);
            svg.appendChild(g);
            g.setAttribute("transform", "translate(" + (-options.bounds.x) + ", " + (-options.bounds.y) + ")");

            var nodes = this.nodes();
            //renderLabels
            for (var i = 0; i < nodes.length; i++) {
                var node = nodes[i];
                ej.datavisualization.Diagram.SvgContext._renderLabelsAsSvg(node, svg, this);
            }
            var connectors = this.connectors();
            for (var i = 0; i < connectors.length; i++) {
                var connector = connectors[i];
                ej.datavisualization.Diagram.SvgContext._renderLabelsAsSvg(connector, svg, this);
            }
            document.body.removeChild(svg.document);
            svg.appendChild(this._svg.document.getElementById(this._canvas.id + "patterndefinition").cloneNode(true));
            return svg.document;
        },
        _diagramAsCanvas: function (options, customBounds) {
            var elements, element;
            var region = options.bounds;
            var margin = options.margin;
            var pageBounds = this._getDigramBounds();
            var bgColor = this.model.pageSettings.pageBackgroundColor;
            var canvas = new ej.datavisualization.Diagram.Canvas({ "id": "mycanvas", "width": region.width, "height": region.height });
            canvas._diagram = this;
            var context = canvas.document.getContext("2d");
            context.fillStyle = "white";
            context.translate(-region.x, -region.y);
            context.fillRect(region.x, region.y, region.width, region.height);
            context.fillStyle = bgColor;
            context.fillRect(pageBounds.x, pageBounds.y, pageBounds.width, pageBounds.height);
            if (this.model.pageSettings.pageBorderWidth) {
                context.strokeStyle = this.model.pageSettings.pageBorderColor == "none" ? "transparent" : this.model.pageSettings.pageBorderColor;
                context.lineWidth = this.model.pageSettings.pageBorderWidth;
                context.strokeRect(pageBounds.x, pageBounds.y, pageBounds.width, pageBounds.height);
            }
            elements = this.nodes().concat(this.connectors());
            elements = this._sortByZIndex(elements, true);
            this._sortByGroupChildrenZIndex(true);
            for (i = 0; i < elements.length; i++) {
                element = elements[i];
                if (element.segments)
                    ej.datavisualization.Diagram.CanvasContext.renderConnector(element, canvas);
                else if (element._type === "group" && !element.parent)
                    ej.datavisualization.Diagram.CanvasContext.renderGroup(element, canvas, this);
                else if (element._type === "node" && !element.isPhase)
                    ej.datavisualization.Diagram.CanvasContext.renderNode(element, canvas);
            }
            var tx = 0, ty = 0;
            if (this.model.phases) {
                for (var i = 0; i < this.model.phases.length; i++) {
                    var parNode = this.nameTable[this.model.phases[i].parent];
                    if (parNode) {
                        var children = this._getChildren(parNode.children);
                        if (children.length > 1)
                            var header = this.nameTable[children[0]];
                        var bounds = ej.datavisualization.Diagram.Util.bounds(parNode);
                        var points = [], nPoint;
                        var top = bounds.top + 50;
                        var left = bounds.left + 50;
                        if (this.model.phases[i].orientation == "vertical") {
                            points.push({ x: 0, y: 0 });
                            points.push({ x: bounds.width, y: 0 });
                            nPoint = ej.datavisualization.Diagram.SvgContext._convertToSVGPoints(points);
                            tx = bounds.x;
                            ty = bounds.y + this.model.phases[i].offset;
                            if (header)
                                top += header.height;
                            for (var j = 0, len = points.length; j < len; ++j) {
                                var point = ej.datavisualization.Diagram.Geometry.translate(points[j], tx, ty);
                                points[j] = point;
                            }
                        } else {
                            points.push({ x: 0, y: header.height ? header.height : 0 });
                            points.push({ x: 0, y: bounds.height });
                            tx = bounds.x + this.model.phases[i].offset;
                            ty = bounds.y;
                            for (var j = 0, len = points.length; j < len; ++j) {
                                var point = ej.datavisualization.Diagram.Geometry.translate(points[j], tx, ty);
                                points[j] = point;
                            }
                        }
                    }
                    ej.datavisualization.Diagram.CanvasContext.renderPhases(this.model.phases[i], canvas, points);
                }
            }
            /*canvas.line({ x: region.x + margin.left / 2, y: region.y }, { x: region.x + margin.left / 2, y: region.y + region.height }, { lineWidth: margin.left, stroke: bgColor });
        var y = region.y + region.height - margin.bottom / 2;
        canvas.line({ x: region.x, y: y }, { x: region.x + region.width, y: y }, { lineWidth: margin.bottom, stroke: bgColor });
        var x = region.x + region.width - margin.right / 2;
        canvas.line({ x: x, y: region.y + region.height }, { x: x, y: region.y }, { lineWidth: margin.right, stroke: bgColor });
        y = region.y + margin.top / 2;
        canvas.line({ x: region.x + region.width, y: y }, { x: region.x, y: y }, { lineWidth: margin.top, stroke: bgColor });*/
            return canvas.document;
        },
        print: function () {
            var url = this.exportDiagram({ mode: "data", region: "pageSettings" });
            var img = document.createElement('img');
            var attr = {
                "id": this.element[0].id + "_printImage",
                "src": url,
            };
            ej.datavisualization.Diagram.Util.attr(img, attr);
            var div = document.createElement("div");
            div.appendChild(img);

            // specify window parameters
            var PrintWind = window.open('');
            if (PrintWind != null) {
                PrintWind.document.write('<html><head><title></title></head>');
                PrintWind.document.write('<BODY onload="window.print()">');
                PrintWind.document.write("<center>" + div.innerHTML + "</center>");
                PrintWind.document.close();
            }
        },
        save: function () {
            var drawingTools = this.model.drawingTools;
            var userHandles = this.model.selectedItems.userHandles;
            this.model.drawingTools = null;
            this.model.selectedItems.userHandles = null;
            if (this.nameTable["multipleSelection"]) {
                ej.datavisualization.Diagram.Util.removeItem(this.nodes(), this.nameTable["multipleSelection"]);
                this._nodes = $.extend(true, [], this.nodes());
            }
            this._updateTableNodes();
            var model = jQuery.extend(true, {}, this.model);
            this._updateGroupChildren(model);
            this.model.drawingTools = drawingTools;
            this.model.selectedItems.userHandles = userHandles;
            this._checkForNullValues(model);
            return model;
        },
        _checkForNullValues: function (model) {
            this._checkTargetNull(model.pageSettings);

        },
        _checkTargetNull: function (target) {
            for (var member in target) {
                if (target[member] == null || target[member] == Infinity) {
                    target[member] = 0;
                }
                else if (typeof (target[member]) === "object" && target[member])
                    this._checkTargetNull(target[member]);
            }
        },
        _updateGroupChildren: function (model) {
            var nodes = [], swimlane;
            if (model && model.nodes && model.nodes.length > 0) {
                for (var i = 0; i < model.nodes.length; i++) {
                    if (model.nodes[i].parent === "" && !model.nodes[i].isSwimlane) {
                        this._updateChildren(model.nodes[i]);
                        nodes.push(jQuery.extend(true, {}, model.nodes[i]))
                    }
                    else if (model.nodes[i].isSwimlane) {
                        swimlane = this._getNode(model.nodes[i].name)
                        if (swimlane)
                            nodes.push(jQuery.extend(true, {}, swimlane))
                    }
                }
            }
            model.nodes = nodes;
        },
        _updateChildren: function (node) {
            var children = [];
            if (node && node.children && node.children.length > 0) {
                var child, childName;
                for (var i = 0; i < node.children.length; i++) {
                    childName = this._getChild(node.children[i]);
                    if (childName)
                        child = this.nameTable[childName];
                    if (child)
                        children.push(jQuery.extend(true, {}, child));
                }
            }
            node.children = children;
        },
        _containsNode: function (node) {
            node = this._getChild(node);
            var chnode = null;
            for (var i = 0; i < this.nodes().length; i++) {
                chnode = this.nodes()[i];
                if (chnode) {
                    if (chnode.name === node)
                        return true;
                }
            }
            return null;
        },
        _containsPhase: function (phaseName) {
            var cphase = null;
            for (var i = 0; i < this.model.phases.length; i++) {
                cphase = this.model.phases[i];
                if (cphase) {
                    if (cphase.name === phaseName)
                        return true;
                }
            }
            return null;
        },
        _updateToModel: function (node) {
            if (node) {
                var cnode = null;
                if (node.children && node.children.length > 0) {
                    for (var i = 0; i < node.children.length; i++) {
                        cnode = this.nameTable[this._getChild(node.children[i])];
                        this._updateToModel(cnode);
                    }
                }
                if (!this._containsNode(node)) {
                    this.nodes().push(node);
                    this._nodes = $.extend(true, [], this.nodes());
                }
            }
        },
        _updateTableNodes: function () {
            var i = 0, node, connector, cphase;
            for (i; i < this.nodes().length; i++) {
                node = this.nameTable[this.nodes()[i].name];
                if (node) {
                    if (node.children && node.children.length > 0) {
                        this._updateToModel(node);
                    }
                    this.nodes()[i] = node;
                    if (node._type === "group") {
                        this._updateTableGroup(node);
                    }
                    if (node.phases && node.phases.length > 0) {
                        for (var n = 0; n < node.phases.length; n++) {
                            cphase = this.nameTable[node.phases[n]];
                            if (cphase) {
                                if (!this._containsPhase(cphase.name)) {
                                    this.model.phases.push(cphase);
                                }
                                else
                                    cphase = cphase;
                            }
                        }
                    }
                }
            }
            for (i; i < this.connectors().length; i++) {
                connector = this.nameTable[this.connectors()[i].name];
                if (connector) {
                    this.connectors()[i] = connector;
                }
            }
        },
        _updateTableGroup: function (group) {
            var children = group.children;
            for (var i = 0; i < group.children.length; i++) {
                group.children[i] = this.nameTable[this._getChild(group.children[i])];
                if (group.children[i] && group.children[i]._type === "group") {
                    this._updateTableGroup(group.children[i]);
                }
            }
        },
        upgrade: function (data) {
            //Support to upgrade diagram JSON data from 12.1
            if (!data.version) {
                //CommonAPIChanges
                this.commonAPIChanges(data);
                //Node and Connector constraints set to default
                if (data.nodes) {
                    for (var i = 0 ; i < data.nodes.length; i++) {
                        if (!data.nodes[i].inEdges) {
                            data.nodes[i].inEdges = [];
                            data.nodes[i].outEdges = [];
                        }
                        data.nodes[i].constraints = ej.datavisualization.Diagram.NodeConstraints.Default;
                    }
                }
            }
            else if (data.version == "13.1") {
                //No Code Changes
            }
        },
        commonAPIChanges: function (data) {
            // defaultSettings API Added
            data.defaultSettings = { node: data.nodeDefaults, connectors: data.connectorDefaults };
            // line is changed to segments; Inserting segments
            if (data.connectors) {
                for (var i = 0 ; i < data.connectors.length; i++) {
                    //Setting the Connector's constraints to Default
                    data.connectors[i].constraints = ej.datavisualization.Diagram.ConnectorConstraints.Default;
                    if (data.connectors[i].line) {
                        if (data.connectors[i].line.type == "straight") {
                            var segments = [{
                                type: "straight",
                                startPoint: data.connectors[i].line.startPoint,
                                endPoint: data.connectors[i].line.endPoint
                            }];
                            this.commonSourceTargetChanges(data.connectors[i]);
                        }
                        else if (data.connectors[i].line.type == "orthogonal") {
                            var segments = [{
                                type: "orthogonal",
                                startPoint: data.connectors[i].line.startPoint,
                                endPoint: data.connectors[i].line.endPoint,
                            }];
                            this.commonSourceTargetChanges(data.connectors[i]);
                        }
                        else if (data.connectors[i].line.type == "bezier") {
                            var segments = [{
                                type: "bezier",
                                startPoint: data.connectors[i].line.startPoint,
                                endPoint: data.connectors[i].line.endPoint,
                                point1: data.connectors[i].line.point1,
                                point2: data.connectors[i].line.point2
                            }];
                            this.commonSourceTargetChanges(data.connectors[i]);
                        }
                        data.connectors[i].sourcePoint = data.connectors[i].line.startPoint;
                        data.connectors[i].targetPoint = data.connectors[i].line.endPoint;
                        data.connectors[i].segments = segments;
                    }
                }
            }
        },
        commonSourceTargetChanges: function (data) {
            if (data.sourceNodeName)
                data.sourceNode = data.sourceNodeName;
            if (data.sourceNodePort)
                data.sourcePort = data.sourceNodePort;
            if (data.targetNodeName)
                data.targetNode = data.targetNodeName;
            if (data.targetPortName)
                data.targetPort = data.targetPortName;
        },
        load: function (data) {
            this._isLoad = true;
            this._spatialSearch = ej.datavisualization.Diagram.SpatialSearch();
            this.clear();
            var contextmenu = document.getElementById(this.element[0].id + "_contextMenu");
            if (contextmenu)
                contextmenu.parentNode.removeChild(contextmenu);
            var load = {
                nodes: data.nodes ? data.nodes : [],
                connectors: data.connectors ? data.connectors : [],
                commandManager: data.commandManager ? data.commandManager : {},
                contextMenu: data.contextMenu ? data.contextMenu : {},
                snapSettings: data.snapSettings ? data.snapSettings : {},
                historyManager: data.historyManager ? data.historyManager : this._initHistoryManager(),
                backgroundImage: data.backgroundImage ? data.backgroundImage : null,
                enableAutoScroll: data.enableAutoScroll ? data.enableAutoScroll : true,
                autoScrollMargin: data.autoScrollMargin ? data.autoScrollMargin : 20,
                layout: data.layout ? data.layout : {},
                pageSettings: data.pageSettings ? data.pageSettings : {},
                click: data.click ? data.click : null,
                connectionChange: data.connectionChange ? data.connectionChange : null,
                defaultSettings: data.defaultSettings ? data.defaultSettings : { node: null, connector: null },
                dataSource: data.dataSource ? data.dataSource : null,
                dataSourceSchema: data.dataSourceSettings ? data.dataSourceSettings : {},
                doubleClick: data.doubleClick ? data.doubleClick : null,
                enableContextMenu: data.enableContextMenu ? data.enableContextMenu : true,
                tooltip: data.tooltip ? data.tooltip : null,
                height: data.height ? data.height : null,
                mouseEnter: data.mouseEnter ? data.mouseEnter : null,
                mouseOver: data.mouseOver ? data.mouseOver : null,
                mouseLeave: data.mouseLeave ? data.mouseLeave : null,
                nodeTemplate: data.nodeTemplate ? data.nodeTemplate : null,
                rotationChange: data.rotationChange ? data.rotationChange : null,
                selectorConstraints: data.selectorConstraints ? data.selectorConstraints : ej.datavisualization.Diagram.SelectorConstraints.All,
                width: data.width ? data.width : null,
                constraints: data.constraints,
                phases: data.phases ? data.phases : [],
                scrollSettings: data.scrollSettings ? data.scrollSettings : {
                    horizontalOffset: 0,
                    verticalOffset: 0,
                    currentZoom: 1,
                    viewPortHeight: 0,
                    viewPortWidth: 0,
                    zoomFactor: data.zoomFactor ? data.zoomFactor : 0.2
                },
            };
            if (data.showTooltip !== undefined) {
                load.showTooltip = data.showTooltip;
            }
            if (data.tooltipTemplateId !== undefined) {
                load.tooltipTemplateId = data.tooltipTemplateId;
            }
            $("#" + this._id).ejDiagram(load);
            this._isLoad = false;
        },
        layout: function () {
            this._setLayout();
        },
        _setLayout: function () {
            //this.model.layout = new ej.datavisualization.Diagram.HierarchicalLayout(this.model.layout);
            this._doLayout();
            ej.datavisualization.Diagram.PageUtil._updatePageSize(this);
            this._clearSelection();
            //this._updateNodes();
            //this._updateConnectors();
        },
        addSelection: function (node, clearSelection) {
            var selectedItem, pseudoGroup;
            if (node && !this._selectionContains(node)) {
                if (this.model.selectedItems.children.length > 0 && !clearSelection) {
                    pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                    if (this.model.selectedItems.children.length > 1)
                        pseudoGroup.children = this.selectionList[0].children;
                    else
                        pseudoGroup.children.push(this.selectionList[0].name)
                    pseudoGroup.children.push(node.name);
                    this.nodes().push(pseudoGroup);
                    this.nameTable[pseudoGroup.name] = pseudoGroup;
                    ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this, null, true);
                    selectedItem = pseudoGroup;
                }
                else {
                    selectedItem = node;
                }
                if (this._hasSelection())
                    this._clearSelection(true);
                this._addSelection(selectedItem);
            }
        },

        clearSelection: function () {
            this._clearSelection();
        },
        removeSelection: function (node) {
            if (node) {
                var node = this.nameTable[node.name];
                if (this.selectionList[0] == node || this.selectionList[0].type == "pseudoGroup" && this.selectionList[0].children.length == 1) { this._clearSelection(); }
                else if (this.selectionList[0].type == "pseudoGroup" && this._collectionContains(node.name, this.selectionList[0].children)) {
                    var index = this.selectionList[0].children.indexOf(node.name);
                    if (index != -1)
                        ej.datavisualization.Diagram.Util.removeItem(this.selectionList[0].children, node.name);
                    else
                        ej.datavisualization.Diagram.Util.removeItem(this.selectionList[0].children, node);
                    ej.datavisualization.Diagram.Util.removeItem(this.model.selectedItems.children, node);
                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this);
                    this._updateSelectionHandle();
                }
            }
        },
        updateSelectedObject: function (nodeName, parent) {
            var node = this.nameTable[nodeName];
            if (node) {
                this.activeTool.selectedObject = node;
            }
            if (parent && parent.isSwimlane)
                this.activeTool.selectedObject = parent
        },
        updateSelection: function (isDragging) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                this._updateSelectionHandle(isDragging);
            }
        },
        activateTool: function (toolName, singleAction) {
            if (this.tools[toolName]) {
                if (this.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                    this.activeTool._showAllPorts(true);
                }
                this._toolToActivate = toolName;
                this.activeTool = this.tools[this._toolToActivate];
                this.activeTool.singleAction = true;
                if (singleAction === false || toolName === "panTool")
                    this.activeTool.singleAction = false;
            }
            if (this.activeTool instanceof ej.datavisualization.Diagram.PanTool) {
                this._currentCursor = "pointer";
                this.activeTool.startPoint = ej.datavisualization.Diagram.Point(0, 0);
                this.activeTool._isMouseDown = false;
            }
            if (this.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                if (this.activeTool.singleAction)
                    this.tool(this.tool() | ej.datavisualization.Diagram.Tool.DrawOnce);
                else
                    this.tool(this.tool() | ej.datavisualization.Diagram.Tool.ContinuesDraw);
                this.activeTool._showAllPorts();
            }
            this._drawingTool = true;
        },
        deactivateTool: function () {
            if (this.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                this.activeTool._showAllPorts(true);
            }
            if (this.tool() & ej.datavisualization.Diagram.Tool.DrawOnce)
                this.tool(this.tool() ^ ej.datavisualization.Diagram.Tool.DrawOnce);
            if (this.tool() & ej.datavisualization.Diagram.Tool.ContinuesDraw)
                this.tool(this.tool() ^ ej.datavisualization.Diagram.Tool.ContinuesDraw);
            if (this.tool() & ej.datavisualization.Diagram.Tool.ZoomPan) {
                this._toolToActivate = "panTool";
                this.activeTool = this.tools[this._toolToActivate];
                this.activeTool._mousedown = false;
                this._currentCursor = "pointer";
            } else {
                this._toolToActivate = "select";
                this.activeTool = this.tools[this._toolToActivate];
                this.tool(ej.datavisualization.Diagram.Tool.SingleSelect | ej.datavisualization.Diagram.Tool.MultipleSelect);
            }
        },
        updateLabel: function (nodeName, label, obj) {
            if (nodeName && ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var mNode = this._findNode(nodeName);
                if (!mNode)
                    mNode = this._findConnector(nodeName);
                if (obj && label && !obj.name || label.name == obj.name) {
                    if (mNode) {
                        if (!this._isUndo)
                            this._recordPropertiesChanged(mNode, obj, "label", label);
                        for (var prop in obj)
                            if (label.hasOwnProperty(prop)) {
                                label[prop] = obj[prop];
                            }
                        if (obj["wrapText"] !== undefined) label["wrapping"] = obj["wrapText"] ? "wrapwithoverflow" : "nowrap";
                        if (mNode.segments || mNode.type !== "text") {
                            ej.datavisualization.Diagram.DiagramContext.updateLabelStyle(mNode, label, this);
                            ej.datavisualization.Diagram.DiagramContext.updateLabel(mNode, label, this);
                        } else if (mNode.type === "text") {
                            ej.datavisualization.Diagram.DiagramContext.updateTextBlock(mNode, label, this);
                        }
                    }
                    if (obj.mode === ej.datavisualization.Diagram.LabelEditMode.Edit) {
                        this.startLabelEdit(mNode, label);
                    }
                }
            }
        },
        startLabelEdit: function (node, label) {
            if (node) {
                //Disables the edit mode of labels in the selectionList
                for (var i = 0; i < this.selectionList.length; i++) {
                    var mNode = this.selectionList[i];
                    if (mNode !== node && typeof mNode.labels.length != "undefined") {
                        for (var j = 0; j < mNode.labels.length; j++)
                            mNode.labels[j].mode = ej.datavisualization.Diagram.LabelEditMode.View;
                    }
                }
                if (!label && node.type === "text")
                    label = node.textBlock;
                if (this._setLabelEditing(label))
                    this._startEdit(node);
                else {
                    this._endEdit();
                }
            }
        },
        _setSelectorConstraints: function (constraints) {
            this.model.selectedItems.constraints = constraints;
            if (this.selectionList[0]) {
                ej.datavisualization.Diagram.SvgContext.clearSelector(this._adornerSvg, this._adornerLayer);
                ej.datavisualization.Diagram.SvgContext.renderSelector(this.selectionList[0], this._adornerSvg, this._adornerLayer, this._currZoom, constraints);
                if (this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                    ej.datavisualization.Diagram.SvgContext.renderUserHandles(this.model.selectedItems.userHandles, this.selectionList[0], this._adornerSvg, this.selectionList[0].type == "pseudoGroup",
                        this._currZoom, this._adornerLayer, this);
            }
        },
        _raiseGroupChangeEvent: function (element, oldParent, newParent, cause) {
            var args = {};
            args.element = element ? element : null;
            args.oldParent = oldParent ? oldParent : null;
            args.newParent = newParent ? newParent : null;
            args.cause = cause ? cause : null;
            this._raiseEvent("groupChange", args);
        },
        group: function () {
            if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup") {
                var selectionList = [];
                var i;
                for (i = 0; i < this.selectionList[0].children.length; i++)
                    selectionList.push(this.nameTable[this.selectionList[0].children[i]]);
                this._sortByZIndex(selectionList, true);
                var element;
                var groupName = ej.datavisualization.Diagram.Util.randomId();
                var group = new ej.datavisualization.Diagram.Group({ name: groupName, });
                var oldParent, newParent;
                this._clearSelection(true);
                for (i = 0; i < selectionList.length; i++) {
                    this._isUndo = true;
                    group.children.push(selectionList[i].name);
                    if (selectionList[i].parent)
                        oldParent = this.nameTable[selectionList[i].parent];
                    selectionList[i].parent = groupName;
                    ej.datavisualization.Diagram.Util.removeItem(this.selectionList, selectionList[i].name);
                    if (!selectionList[i].segments) {
                        ej.datavisualization.Diagram.Util.removeItem(this.nodes(), this.nameTable[selectionList[i].name]);
                        this._nodes = $.extend(true, [], this.nodes());
                    }
                    else {
                        ej.datavisualization.Diagram.Util.removeItem(this.connectors(), this.nameTable[selectionList[i].name]);
                        this._connectors = $.extend(true, [], this.connectors());
                    }
                    ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[selectionList[i].name], selectionList[i]);
                    if (this._browserInfo.name === "msie") {
                        if (selectionList[i].type == "html") {
                            element = document.getElementById(selectionList[i].name + "_parentdiv");
                            element.parentNode.removeChild(element);
                        } else {
                            if (selectionList[i]._type === "group")
                                this._checkForHtmlNode(selectionList[i]);
                        }
                    }
                    // $("#" + selectionList[i].name)[0].parentNode.removeChild($("#" + selectionList[i].name)[0]);
                    element = this._svg.document.getElementById(selectionList[i].name);
                    if (element)
                        element.parentNode.removeChild(element);
                    this._removeElement(selectionList[i]);
                    this._raiseGroupChangeEvent(selectionList[i], oldParent, group, "group");
                }
                ej.datavisualization.Diagram.Util.clear(this.selectionList);
                group = this._getNewGroup(group);
                this.add(group);
                //this._addSelection(group);
                var entry = { type: "groupchanged", object: group, actionType: "group", category: "internal" };
                this.addHistoryEntry(entry);
                //var entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.GroupHandleCmd(group, "group"));
                //this.historyManager.addHistoryEntry(entry);
                this._isUndo = false;
            }
        },
        _checkForHtmlNode: function (group) {
            var children = this._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                var child = this.nameTable[children[i]];
                if (child) {
                    if (child._type === "group") {
                        this._checkForHtmlNode(child);
                    } else {
                        if (child.type == "html") {
                            var element = document.getElementById(child.name + "_parentdiv");
                            element.parentNode.removeChild(element);
                        }
                    }
                }
            }
        },
        _removeChildren: function (group, args) {
            var children = this._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                var child = this.nameTable[children[i]];
                if (child) {
                    if (group.isLane) {
                        this._disConnect(child, args);
                        this._removeConnector(child, args);
                    }
                    else {
                        if (child.segments) {
                            this._removeEdges(child);
                            child.sourceNode = child.sourcePort = child.targetNode = child.targetPort = null;
                        }
                        else
                            this._disConnect(child, args, true);
                    }
                    ej.datavisualization.Diagram.Util.removeItem(this.nodes(), child);
                    this._nodes = $.extend(true, [], this.nodes());
                    if (child._type === "group") {
                        this._removeChildren(child, args);
                        ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[child.name], child);
                        ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, child);
                        delete this.nameTable[child.name];
                    } else {
                        ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[child.name], child);
                        ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, child);
                        delete this.nameTable[child.name];
                    }
                    this._removeElement(child);
                }
            }
        },
        ungroup: function () {
            if (this.selectionList[0] && this.selectionList[0]._type === "group" && this.selectionList[0].canUngroup && !(this.selectionList[0].isSwimlane || this.selectionList[0].isLane)) {
                this._isUndo = true;
                var selectionList = this.selectionList[0].children;
                var i, node;
                var htmlLayer = this._svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                node = this.selectionList[0];
                $("#" + this.selectionList[0].name)[0].parentNode.removeChild($("#" + this.selectionList[0].name)[0]);
                if ($("#" + this.selectionList[0].name)[0])
                    $("#" + this.selectionList[0].name)[0].parentNode.removeChild($("#" + this.selectionList[0].name)[0]);
                if (this._browserInfo.name === "msie")
                    this._checkForHtmlNode(this.selectionList[0]);
                this._clearSelection();
                var object;
                ej.datavisualization.Diagram.Util.clear(this.selectionList);
                var newParent, oldParent;
                for (i = 0; i < selectionList.length; i++) {
                    var item = this.nameTable[selectionList[i]];
                    if (item.parent)
                        oldParent = this.nameTable[item.parent];
                    if ($(htmlLayer).children("#" + item.name)[0])
                        htmlLayer.removeChild($(htmlLayer).children("#" + item.name)[0]);
                    if (item) {
                        if (item.parent)
                            item.parent = null;
                        if (item.segments) {
                            object = item;
                            object.parent = "";
                            this._updateEdges(object);
                            this._dock(object, this.nameTable);
                            ej.datavisualization.Diagram.DiagramContext.renderConnector(object, this);
                            if (this.connectors().indexOf(object) === -1) {
                                this.connectors().push(object);
                                this._connectors = $.extend(true, [], this.connectors());
                            }
                        }
                        else {
                            object = item;
                            object.parent = "";
                            if (object._type === "group") {
                                this._updateChildrenEdges(object);
                                if (object.isLane || object.isSwimlane) {
                                    object.width = 0;
                                    object.height = 0;
                                }
                                ej.datavisualization.Diagram.DiagramContext.renderGroup(object, this);
                            }
                            else
                                ej.datavisualization.Diagram.DiagramContext.renderNode(object, this);
                            if (this.nodes().indexOf(object) === -1) {
                                this.nodes().push(object);
                                this._nodes = $.extend(true, [], this.nodes());
                            }
                            if (item.parent)
                                newParent = this.nameTable[item.parent];
                            this._raiseGroupChangeEvent(item, oldParent, newParent, "unGroup");
                        }
                    }
                }
                //this._clearSelection();
                ej.datavisualization.Diagram.Util.removeItem(this.nodes(), this._findNode(node.name));
                ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[node.name], node);
                ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, node);
                var entry = { type: "groupchanged", object: node, actionType: "ungroup", category: "internal" };
                this.addHistoryEntry(entry);

                //var entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.GroupHandleCmd(node, "ungroup"));
                //this.historyManager.addHistoryEntry(entry);
                this._isUndo = false;
            }
        },
        nudge: function (direction, delta) {
            delta = !delta ? 1 : delta;
            var dir = { up: { x: 0, y: -delta }, down: { x: 0, y: delta }, left: { x: -delta, y: 0 }, right: { x: delta, y: 0 } };
            var nudge = dir[direction];
            if (nudge && this.selectionList[0]) {
                if (!this._currentState) {
                    this.addHistoryEntry({
                        type: "nudge", object: this.selectionList[0], values: { x: this.selectionList[0].offsetX, y: this.selectionList[0].offsetY },
                        category: "internal"
                    });
                }
                this._nudge(this.selectionList[0], nudge.x, nudge.y, direction);
            }
        },
        selectAll: function () {
            var collection, nodes, i, len, item = null;
            this._clearSelection(true);
            collection = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", name: "multipleSelection" });
            nodes = this.nodes();
            for (i = 0, len = nodes.length; i < len; i++) {
                if (ej.datavisualization.Diagram.Util.canSelect(nodes[i])) {
                    item = this.nameTable[nodes[i].name];
                    if (item && item.parent == "" && item.type != "pseudoGroup" && item.visible) {
                        collection.children.push(item.name);
                    }
                }
            }
            var connectors = this.connectors();
            for (i = 0, len = connectors.length; i < len; i++) {
                if (ej.datavisualization.Diagram.Util.canSelect(connectors[i])) {
                    if (connectors[i].parent == "" && connectors[i].visible) {
                        collection.children.push(connectors[i].name);
                    }
                }
            }
            ej.datavisualization.Diagram.Util._updateGroupBounds(collection, this);
            //ej.datavisualization.Diagram.Util.clear(this.selectionList);
            this.nodes().push(collection);
            this._nodes = $.extend(true, [], this.nodes());
            this.nameTable[collection.name] = collection;
            if (collection.children.length > 0)
                this._addSelection(collection);
        },
        align: function (direction) {
            if (this.selectionList[0] && (this.selectionList[0].type === "group" || this.selectionList[0].type == "pseudoGroup") && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var history = [];
                var cNode;
                var cornerNodes = this._findCornerNodes();
                if (direction === "left")
                    cNode = cornerNodes.left;
                else if (direction === "right")
                    cNode = cornerNodes.right;
                else if (direction === "top")
                    cNode = cornerNodes.top;
                else if (direction === "bottom")
                    cNode = cornerNodes.bottom;
                var cNodeBounds;
                if (cNode)
                    cNodeBounds = ej.datavisualization.Diagram.Util.bounds(cNode);
                else if (direction === "center" || direction === "middle") {
                    cNodeBounds = ej.datavisualization.Diagram.Util.bounds(this.selectionList[0]);
                }
                if (cNodeBounds) {
                    var i, bounds, node, x = 0, y = 0;
                    for (i = 0; i < list.length; i++) {
                        node = this.nameTable[list[i]];
                        bounds = ej.datavisualization.Diagram.Util.bounds(node);
                        if (direction === "left")
                            x = -(bounds.x - cNodeBounds.x);
                        else if (direction === "right")
                            x = -((bounds.x + bounds.width) - (cNodeBounds.x + cNodeBounds.width));
                        else if (direction === "top")
                            y = -(bounds.y - cNodeBounds.y);
                        else if (direction === "bottom")
                            y = -((bounds.y + bounds.height) - (cNodeBounds.y + cNodeBounds.height));
                        else if (direction === "center")
                            x = -((bounds.x + bounds.width / 2) - (cNodeBounds.x + cNodeBounds.width / 2));
                        else if (direction === "middle")
                            y = -((bounds.y + bounds.height / 2) - (cNodeBounds.y + cNodeBounds.height / 2));
                        this._moveNode(x, y, node);
                        history.push({ object: node, delta: { x: x, y: y } });
                    }
                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this);
                    ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
                }
                this.addHistoryEntry({ type: "alignCommand", values: history, category: "internal", object: this.selectionList[0] });
            }
        },
        sameSize: function () {
            if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup" && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var fNode = this.nameTable[this.selectionList[0].children[0]];
                var fNodeBounds = ej.datavisualization.Diagram.Util.bounds(fNode);
                var history = [];
                for (var i = 0; i < list.length; i++) {
                    var element = this.nameTable[list[i]];
                    if (!(element.segments)) {
                        //if ((element._shape === "path") && ((element.height !== fNodeBounds.height) || (element.width !== fNodeBounds.width)))
                        //    element._scaled = true;
                        history.push({ object: element, values: { width: element.width || element._width || 0, height: element.height || element._height || 0 } });
                        this.updateNode(element.name, { width: fNodeBounds.width, height: fNodeBounds.height });
                    }
                }
                this.addHistoryEntry({ type: "sizeCommand", category: "internal", values: history });
                ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this);
                ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
            }
        },
        sameHeight: function () {
            if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup" && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var fNode = this.nameTable[this.selectionList[0].children[0]];
                var history = [];
                var fNodeBounds = ej.datavisualization.Diagram.Util.bounds(fNode);
                for (var i = 0; i < list.length; i++) {
                    var element = this.nameTable[list[i]];
                    if (!(element.segments)) {
                        //if ((element._shape === "path") && ((element.height !== fNodeBounds.height)))
                        //    element._scaled = true;
                        history.push({ object: element, values: { width: element.width || element._width || 0, height: element.height || element._height || 0 } });
                        this.updateNode(element.name, { height: fNodeBounds.height });
                    }
                }
                this.addHistoryEntry({ type: "sizeCommand", category: "internal", values: history });
                ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this);
                ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
            }
        },
        sameWidth: function () {
            if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup" && this.selectionList[0].children && this.selectionList[0].children.length > 0) {
                var list = this.nameTable[this.selectionList[0].name].children;
                var fNode = this.nameTable[this.selectionList[0].children[0]];
                var fNodeBounds = ej.datavisualization.Diagram.Util.bounds(fNode);
                var history = [];
                for (var i = 0; i < list.length; i++) {
                    var element = this.nameTable[list[i]];
                    if (!(element.segments)) {
                        //if ((element._shape === "path") && (element.width !== fNodeBounds.width))
                        //    element._scaled = true;
                        history.push({ object: element, values: { width: element.width || element._width || 0, height: element.height || element._height || 0 } });
                        this.updateNode(element.name, { width: fNodeBounds.width });
                    }
                }
                this.addHistoryEntry({ type: "sizeCommand", category: "internal", values: history });
                ej.datavisualization.Diagram.Util._updateGroupBounds(this.selectionList[0], this);
                ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectionList[0], this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
            }
        },
        spaceDown: function () {
            if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup") {
                var lstSelections = this.selectionList[0].children;
                this._sliceDockLines(lstSelections);
                this._sortByYvalue(this.selectionList[0].children);
                var selectionCount = lstSelections.length, fSpacing;
                var history = [];
                if (lstSelections && selectionCount > 2) {
                    var cornerNodes = this._findCornerNodes();
                    var firstNode = cornerNodes.top;
                    var dHei = 0;
                    var pre = null, nxt = null;
                    for (var i in lstSelections) {
                        var pre = null, nxt = null;
                        pre = this.nameTable[this._getChild(lstSelections[i])];
                        nxt = this.nameTable[this._getChild(lstSelections[Number(i) + 1])];
                        if (pre && nxt) {
                            var preBounds = ej.datavisualization.Diagram.Util.bounds(pre);
                            var nxtBounds = ej.datavisualization.Diagram.Util.bounds(nxt);
                            dHei += nxtBounds.y - (preBounds.y + preBounds.height);
                        }
                    }
                    var space = dHei / (selectionCount - 1);
                    for (var i in lstSelections) {
                        pre = this.nameTable[this._getChild(lstSelections[i])];
                        nxt = this.nameTable[this._getChild(lstSelections[Number(i) + 1])];
                        if (pre && nxt) {
                            var preBounds = ej.datavisualization.Diagram.Util.bounds(pre);
                            var nxtBounds = ej.datavisualization.Diagram.Util.bounds(nxt);
                            this._translate(nxt, 0, (preBounds.bottom - (nxtBounds.top)) + space, this.nameTable);
                            history.push({ object: nxt, delta: { x: 0, y: (preBounds.bottom - (nxtBounds.top)) + space } });
                            this._updateQuad(this, nxt);
                            this._updateNodeMargin(nxt);
                            ej.datavisualization.Diagram.DiagramContext.update(nxt, this);
                        }
                    }
                    this.addHistoryEntry({ type: "spacingCommand", values: history, category: "internal", object: this.selectionList[0] });
                }
            }
        },
        spaceAcross: function () {
            if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup") {
                var lstSelections = this.selectionList[0].children;
                this._sliceDockLines(lstSelections);
                this._sortByXvalue(this.selectionList[0].children);
                var selectionCount = lstSelections.length, fSpacing;
                var history = [];
                if (lstSelections && selectionCount > 2) {
                    var cornerNodes = this._findCornerNodes();
                    var firstNode = cornerNodes.left;
                    var dWid = 0;
                    var pre = null, nxt = null;
                    for (var i in lstSelections) {

                        var pre = null, nxt = null;
                        pre = this.nameTable[this._getChild(lstSelections[i])];
                        nxt = this.nameTable[this._getChild(lstSelections[Number(i) + 1])];
                        if (pre && nxt) {
                            var preBounds = ej.datavisualization.Diagram.Util.bounds(pre);
                            var nxtBounds = ej.datavisualization.Diagram.Util.bounds(nxt);
                            dWid += nxtBounds.x - (preBounds.x + preBounds.width);
                        }
                    }
                    var space = dWid / (selectionCount - 1);
                    var sdx = 0;
                    for (var i in lstSelections) {
                        pre = this.nameTable[this._getChild(lstSelections[i])];
                        nxt = this.nameTable[this._getChild(lstSelections[Number(i) + 1])];
                        if (pre && nxt) {
                            var preBounds = ej.datavisualization.Diagram.Util.bounds(pre);
                            var nxtBounds = ej.datavisualization.Diagram.Util.bounds(nxt);
                            this._translate(nxt, (preBounds.right - (nxtBounds.left)) + space, 0, this.nameTable);
                            history.push({ object: nxt, delta: { x: (preBounds.right - (nxtBounds.left)) + space, y: 0 } });
                            this._updateQuad(this, nxt);
                            this._updateNodeMargin(nxt);
                            ej.datavisualization.Diagram.DiagramContext.update(nxt, this);
                        }
                    }
                    this.addHistoryEntry({ type: "spacingCommand", values: history, category: "internal", object: this.selectionList[0] });
                }
            }
        },
        sendBackward: function () {
            var ovarLapNodes, i;
            var prevNode;
            if (this.selectionList[0] && this.selectionList[0].type != "pseudoGroup") {
                var node = this.selectionList[0];
                if (node) {
                    ovarLapNodes = this._findOverlapNode(node);
                    this._sortByZIndex(ovarLapNodes);
                    for (var k = 0; k < ovarLapNodes.length; k++)
                        if (node.zOrder > ovarLapNodes[k].zOrder) {
                            prevNode = ovarLapNodes[k];
                            break;
                        }
                    var start = node.zOrder;
                    if (prevNode) {
                        var end = prevNode.zOrder;
                        this._sendElementsToBack(start, end, node, prevNode);
                        this.addHistoryEntry({ type: "zOrder", command: "back", start: end, end: start, nearestNode: prevNode, object: node, category: "internal" });
                    }
                }
            }
        },
        _sendElementsToBack: function (start, end, node, prevNode) {
            var i;
            for (i = end; i < start; i++) {
                if (node.parent != "")
                    this._findObjectByIndex(i, true).zOrder += 1;
                else
                    this._findObjectByIndex(i).zOrder += 1;
            }
            if (node.parent != "") {
                if (node.parent) {
                    var parent = this.nameTable[node.parent];
                    this._findChildren(parent, node.name).zOrder = end;
                }
            } else
                this._findObjectByName(node.name).zOrder = end;
            this._svg.getElementById(prevNode.name).parentNode.insertBefore(this._svg.getElementById(node.name), this._svg.getElementById(prevNode.name));
            this._updateSelectionHandle();
        },
        _bringElementsToFront: function (start, end, node, prevNode) {
            var i;
            for (i = start + 1; i <= end; i++) {
                if (node.parent != "")
                    this._findObjectByIndex(i, true).zOrder -= 1;
                else
                    this._findObjectByIndex(i).zOrder -= 1;
            }
            if (node.parent) {
                var parent = this.nameTable[node.parent];
                this._findChildren(parent, node.name).zOrder = end;
            } else
                this._findObjectByName(node.name).zOrder = end;

            this._svg.getElementById(prevNode.name).parentNode.insertBefore(this._svg.getElementById(node.name), this._svg.getElementById(prevNode.name).nextSibling);
            this._updateSelectionHandle();
        },
        moveForward: function () {
            var ovarLapNodes, i;
            var prevNode;
            if (this.selectionList[0] && this.selectionList[0].type != "pseudoGroup") {
                var node = this.selectionList[0];
                if (node) {
                    ovarLapNodes = this._findOverlapNode(node);
                    this._sortByZIndex(ovarLapNodes);
                    for (var k = ovarLapNodes.length - 1; k >= 0; k--)
                        if (node.zOrder < ovarLapNodes[k].zOrder) {
                            prevNode = ovarLapNodes[k];
                            break;
                        }

                    var start = node.zOrder;
                    if (prevNode) {
                        var end = prevNode.zOrder;
                        this._bringElementsToFront(start, end, node, prevNode);
                        this.addHistoryEntry({ type: "zOrder", command: "front", start: end, end: start, nearestNode: prevNode, object: node, category: "internal" });
                    }
                }
            }
        },
        sendToBack: function () {
            var node = this.selectionList[0];
            var prevNode;
            if (node && node.type != "pseudoGroup") {
                var ovarLapNodes = this._findOverlapNode(node);
                var nodes = this._sortByZIndex(ovarLapNodes);
                var start = node.zOrder;
                var end = node.zOrder;
                var firstNode;
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i].zOrder < end) {
                        if (i == 0) firstNode = nodes[i];
                        end = nodes[i].zOrder;
                        prevNode = nodes[i];
                    }
                }
                if (prevNode) {
                    this._sendElementsToBack(start, end, node, prevNode);
                    this.addHistoryEntry({ type: "zOrder", command: "back", start: end, end: start, nearestNode: firstNode, object: node, category: "internal" });
                }
            }
        },
        bringToFront: function () {
            var node = this.selectionList[0];
            var prevNode;
            if (node && node.type != "pseudoGroup") {
                var ovarLapNodes = this._findOverlapNode(node);
                var nodes = this._sortByZIndex(ovarLapNodes);
                var start = node.zOrder;
                var end = node.zOrder;
                for (var i = 0; i < nodes.length; i++) {
                    if (nodes[i].zOrder > end) {
                        end = nodes[i].zOrder;
                        prevNode = nodes[i];
                    }
                }
                if (prevNode) {
                    this._bringElementsToFront(start, end, node, prevNode);
                    this.addHistoryEntry({ type: "zOrder", command: "front", start: end, end: start, nearestNode: nodes[i - 1], object: node, category: "internal" });
                }
            }
        },
        _getChildFromCollection: function (collection, name) {
            if (collection && collection.length > 0 && name) {
                for (var i = 0; i < collection.length; i++) {
                    if (collection[i].name === name)
                        return collection[i];
                }
            }
        },
        _removeFromContainer: function (object, undochild) {
            if (object, undochild) {
                this._remove(object, null, true);
                if (object.parent) {
                    var parent = this.nameTable[object.parent];
                    if (parent && parent.children.length > 0) {
                        ej.datavisualization.Diagram.Util.removeItem(parent.children, object.name);
                        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this);
                    }
                }
            }
        },
        _updateNameTable: function (node) {
            if (node.children && node.children.length > 0) {
                for (var i = 0; i < node.children.length; i++) {
                    var child = typeof node.children[i] == "object" ? node.children[i] : null;
                    if (child) {
                        this.nameTable[child.name] = child;
                        if (child.children && child.children.length > 0)
                            this._updateNameTable(child);
                    }
                }
            }
        },
        _updateContainer: function (object, undochild, resize, rotate) {

            var parent = this._svg.getElementById(undochild.parent);
            if (!parent)
                parent = this._diagramLayer;
            this._updateNameTable(undochild);
            this.nodes().push(undochild);
            this._nodes = $.extend(true, [], this.nodes());
            this.nameTable[undochild.name] = undochild;
            if (undochild._type === "group") {
                ej.datavisualization.Diagram.SvgContext.renderGroup(undochild, this._svg, parent, this.nameTable, this);
            } else if (undochild.segments) {
                ej.datavisualization.Diagram.SvgContext.renderConnector(undochild, this._svg, parent);
            } else {
                ej.datavisualization.Diagram.SvgContext.renderNode(undochild, this._svg, parent);
            }
            var parentObj = this.nameTable[undochild.parent];
            if (parentObj) {
                parentObj.children.push(undochild.name);
                if (resize) {
                    var resizeTool = this.tools["resize"];
                    resizeTool._updateMargin(undochild, parentObj);
                    resizeTool._updateContainer(undochild, parentObj, true);
                }
                else if (rotate) {
                    var rotateTool = this.tools["rotate"];
                    rotateTool._updateContainer(undochild, undochild, true);
                }
                else {
                    var moveTool = this.tools["move"];
                    moveTool._updateContainer(undochild, parentObj, true);
                    //ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parentObj, this);
                }
            }
            this.deactivateTool();
            ej.datavisualization.Diagram.DiagramContext.update(undochild, this);
        },
        _recordMultiRotationChanged: function (args) {

            var collection, undochild, object, i, lines;
            if (args.undoObject && args.redoObject) {
                collection = args.redoObject.collection
                if (args.undo)
                    collection = args.undoObject.collection

                for (i = 0; i < collection.length; i++) {
                    undochild = jQuery.extend(true, {}, collection[i]);
                    object = this.nameTable[undochild.name]
                    //var redoChild = this._getChildFromCollection(args.redoObject.collection, undochild.name)
                    //if (redoChild)
                    //    jQuery.extend(true, {}, redoChild);
                    if (undochild && object) {
                        this._removeFromContainer(object, undochild);
                        this._updateContainer(object, undochild, false, true);
                    }
                }
                lines = args.redoObject.adjustLines.lines
                if (args.undo)
                    lines = args.undoObject.adjustLines.lines
                for (var i = 0; lines && lines.length > 0 && i < lines.length ; i++) {
                    this.add(lines[i]);
                }
                var pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                pseudoGroup.children = args.undoObject.collection;
                ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this);
                this._clearSelection(true);
                this.nodes().push(pseudoGroup);
                this._nodes = $.extend(true, [], this.nodes());
                this.nameTable[pseudoGroup.name] = pseudoGroup;
                this._addSelection(pseudoGroup, true);
                this.activeTool.selectedObject = pseudoGroup;
            }
        },
        _recordMultiPinPointChanged: function (args) {
            var collection, undochild, object, i, lines;
            if (args.undoObject && args.redoObject) {
                collection = args.redoObject.collection
                if (args.undo)
                    collection = args.undoObject.collection

                for (i = 0; i < collection.length; i++) {
                    undochild = jQuery.extend(true, {}, collection[i]);
                    object = this.nameTable[undochild.name]
                    //var redoChild = this._getChildFromCollection(args.redoObject.collection, undochild.name)
                    //if (redoChild)
                    //    jQuery.extend(true, {}, redoChild);
                    if (undochild && object) {
                        this._removeFromContainer(object, undochild);
                        this._updateContainer(object, undochild);
                    }
                }
                lines = args.redoObject.adjustLines.lines
                if (args.undo)
                    lines = args.undoObject.adjustLines.lines
                for (var i = 0; lines && lines.length > 0 && i < lines.length ; i++) {
                    this.add(lines[i]);
                }
                var pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                pseudoGroup.children = args.undoObject.collection;
                ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this);
                this._clearSelection(true);
                this.nodes().push(pseudoGroup);
                this._nodes = $.extend(true, [], this.nodes());
                this.nameTable[pseudoGroup.name] = pseudoGroup;
                this._addSelection(pseudoGroup, true);
                this.activeTool.selectedObject = pseudoGroup;
            }

        },
        _recordMultiSizeChanged: function (args) {

            var collection, undochild, object, i, lines;
            if (args.undoObject && args.redoObject) {
                collection = args.redoObject.collection
                if (args.undo)
                    collection = args.undoObject.collection

                for (i = 0; i < collection.length; i++) {
                    undochild = jQuery.extend(true, {}, collection[i]);
                    object = this.nameTable[undochild.name]
                    //var redoChild = this._getChildFromCollection(args.redoObject.collection, undochild.name)
                    //if (redoChild)
                    //    jQuery.extend(true, {}, redoChild);
                    if (undochild && object) {
                        this._removeFromContainer(object, undochild, true);
                        this._updateContainer(object, undochild, true);
                    }
                }
                lines = args.redoObject.adjustLines.lines
                if (args.undo)
                    lines = args.undoObject.adjustLines.lines
                for (var i = 0; lines && lines.length > 0 && i < lines.length ; i++) {
                    this.add(lines[i]);
                }
                var pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                pseudoGroup.children = args.undoObject.collection;
                ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this);
                this._clearSelection(true);
                this._addSelection(pseudoGroup, true);
            }
        },
        undo: function () {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this) && ej.datavisualization.Diagram.Util.canUndo(this)) {
                var args;
                this._isUndo = true;
                var entryItem = this.getUndoEntry();
                if (entryItem) {
                    if (entryItem.type == "closeGroup") {
                        this._groupUndo = true;
                    }
                    else this._undo(entryItem);
                    if (this._groupUndo) {
                        entryItem = this.getUndoEntry();
                        while (entryItem.type != "startGroup") {
                            this._undo(entryItem);
                            entryItem = this.getUndoEntry();
                        }
                        delete this._groupUndo;
                    }
                }
                this._isUndo = false;
            }
        },
        redo: function () {
            this._isUndo = true;
            //this.historyManager.redo();
            var entryItem = this.getRedoEntry();
            if (entryItem) {
                if (entryItem.type == "startGroup") {
                    this._groupRedo = true;
                }
                else this._redo(entryItem);
                if (this._groupRedo) {
                    while (entryItem.type != "closeGroup") {
                        entryItem = this.getRedoEntry();
                        this._redo(entryItem);
                    }
                    delete this._groupRedo;
                }
            }
            this._isUndo = false;
        },

        _undo: function (entryItem) {
            if (entryItem) {
                if (entryItem.category == "internal") {
                    var object = entryItem.object || (entryItem.undoObject && entryItem.undoObject.node) || entryItem.shape;
                    if (object && object.parent)
                        var parent = typeof object.parent == "object" ? object.parent.name : object.parent;
                    if (parent && !object.isLane) {
                        if (!this.nameTable[parent]) {
                            this._isUndo = false;
                            this.undo();
                            return;
                        }
                    }
                    var args;
                    switch (entryItem.type) {
                        case "positionchanged":
                            if (entryItem.swimlaneMultiSelection) {
                                entryItem.undo = true;
                                this._recordMultiPinPointChanged(entryItem);
                            }
                            else {
                                args = { object: entryItem.undoObject, isMultipleNode: entryItem.isMultipleNode };
                                this._recordPinPointChanged(args);
                            }
                            break;
                        case "rotationchanged":
                            if (entryItem.swimlaneMultiSelection) {
                                entryItem.undo = true;
                                this._recordMultiRotationChanged(entryItem);
                            }
                            else {
                                args = { object: entryItem.undoObject, isMultipleNode: entryItem.isMultipleNode };
                                this._recordRotationChanged(args);
                            }
                            break;
                        case "sizechanged":
                            if (entryItem.swimlaneMultiSelection) {
                                entryItem.undo = true;
                                this._recordMultiSizeChanged(entryItem);
                            }
                            else {
                                args = { object: entryItem.undoObject, isMultipleNode: entryItem.isMultipleNode };
                                this._recordSizeChanged(args);
                            }
                            break;
                        case "endpointchanged":
                            args = { undoObject: entryItem.undoObject, redoObject: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode, _isUndo: true };
                            this._recordEndPointChanged(args);
                            break;
                        case "collectionchanged":
                            if (entryItem.changeType == "insert") {
                                if (!this.nameTable[entryItem.object.name]) {
                                    this._isUndo = false;
                                    this.undo();
                                }
                            }
                            args = { changeType: entryItem.changeType, childTable: entryItem.childTable, edgeTable: entryItem.edgeTable, isUndo: true, sourcePorts: entryItem.sourcePorts, targetPorts: entryItem.targetPorts, index: entryItem.index };
                            args.object = jQuery.extend(true, {}, entryItem.object);
                            var update = this._recordCollectionChanged(args, entryItem);
                            if (update) {
                                entryItem.sourcePorts = args.sourcePorts;
                                entryItem.targetPorts = args.targetPorts;
                                entryItem.edgeTable = args.edgeTable;
                                entryItem.childTable = args.childTable;
                            }
                            break;
                        case "labelchanged":
                            args = { object: entryItem.shape, label: entryItem.previousLabelValue, index: entryItem.index ? entryItem.index : 0 };
                            this._recordLabelChanged(args);
                            break;
                        case "groupchanged":
                            args = { object: entryItem.object, isUndo: true, actionType: entryItem.actionType };
                            this._recordGroupChanged(args);
                            break;
                        case "phasesizechanged":
                            args = entryItem;
                            this._recordPhasesizeChanged(args);
                            break;
                        case "phasecollectionchanged":
                            var snapConstraints = this.model.snapSettings.snapConstraints;
                            this.model.snapSettings.snapConstraints = ej.datavisualization.Diagram.SnapConstraints.None;
                            if (entryItem.isAdded && entryItem.phase) {
                                this._removePhase(entryItem.phase.name)
                                if (entryItem.prevPhase)
                                    this._updateSwimlanePhase(this.nameTable[entryItem.prevPhase.name], entryItem.prevPhase, true, true);
                                if (entryItem.nextPhase)
                                    this._updateSwimlanePhase(this.nameTable[entryItem.nextPhase.name], entryItem.nextPhase, true, true);
                                this._clearSelection();
                            }
                            else {
                                if (entryItem.prevPhase)
                                    this._updateSwimlanePhase(this.nameTable[entryItem.prevPhase.name], entryItem.prevPhase, true, true);
                                if (entryItem.nextPhase)
                                    this._updateSwimlanePhase(this.nameTable[entryItem.nextPhase.name], entryItem.nextPhase, true, true);
                                this._addPhase(entryItem.phase.parent, entryItem.phase);
                            }
                            this.model.snapSettings.snapConstraints = snapConstraints;
                            break;
                        case "propertiesChanged":
                            var node = this.nameTable[entryItem.object.name];
                            var currentState = this._getModifiedProperties(node, entryItem.values, entryItem[entryItem.elementType]);
                            if (entryItem.elementType == "node")
                                this.updateNode(node.name, entryItem.values);
                            else if (entryItem.elementType == "connector")
                                this.updateConnector(node.name, entryItem.values);
                            else if (entryItem.elementType == "label")
                                this.updateLabel(node.name, entryItem.label, entryItem.values);
                            else if (entryItem.elementType == "port")
                                this.updatePort(node.name, entryItem.port, entryItem.values);
                            entryItem.values = currentState;
                            break;
                        case "nudge":
                            this._recordNudgingChanges(object, entryItem);
                            break;
                        case "spacingCommand":
                        case "alignCommand":
                            this._recordAlignCommandChanges(entryItem);
                            break;
                        case "sizeCommand":
                            this._recordSizeCommandChanges(entryItem);
                            break;
                        case "zOrder":
                            var end = entryItem.start;
                            if (entryItem.command == "back") {
                                this._bringElementsToFront(entryItem.start, entryItem.end, entryItem.object, entryItem.nearestNode);
                                entryItem.command = "front";
                            }
                            else {
                                this._sendElementsToBack(entryItem.start, entryItem.end, entryItem.object, entryItem.nearestNode);
                                entryItem.command = "back";
                            }
                            entryItem.nearestNode = this._findObjectByIndex(end);
                            var temp = entryItem.start;
                            entryItem.start = entryItem.end;
                            entryItem.end = temp;
                            break;
                    }
                } else {
                    if (this.model.historyManager.undo) {
                        var undo = this.model.historyManager.undo;
                        if (typeof undo == "string") undo = ej.util.getObject(undo, window);
                        if ($.isFunction(undo)) {
                            var data = undo(entryItem.data);
                            if (data) entryItem.data = data;
                        }
                    }
                }
            }
        },

        _redo: function (entryItem) {
            if (entryItem) {
                if (entryItem.category == "internal") {
                    var object = entryItem.object || (entryItem.undoObject && entryItem.undoObject.node) || entryItem.shape;
                    if (object && object.parent)
                        var parent = typeof object.parent == "object" ? object.parent.name : object.parent;
                    if (parent) {
                        if (!this.nameTable[parent]) {
                            this._isUndo = false;
                            this.redo();
                            return;
                        }
                    }
                    var args;
                    switch (entryItem.type) {
                        case "positionchanged":
                            if (entryItem.swimlaneMultiSelection) {
                                entryItem.undo = false;
                                this._recordMultiPinPointChanged(entryItem);
                            }
                            else {
                                var args = { object: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode };
                                this._recordPinPointChanged(args);
                            }
                            break;
                        case "rotationchanged":
                            if (entryItem.swimlaneMultiSelection) {
                                entryItem.undo = false;
                                this._recordMultiRotationChanged(entryItem);
                            }
                            else {
                                args = { object: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode };
                                this._recordRotationChanged(args);
                            }
                            break;
                        case "sizechanged":
                            if (entryItem.swimlaneMultiSelection) {
                                entryItem.undo = false;
                                this._recordMultiSizeChanged(entryItem);
                            }
                            else {
                                args = { object: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode };
                                this._recordSizeChanged(args);
                            }
                            break;
                        case "endpointchanged":
                            args = { undoObject: entryItem.undoObject, redoObject: entryItem.redoObject, isMultipleNode: entryItem.isMultipleNode, _isUndo: false };
                            this._recordEndPointChanged(args);
                        case "collectionchanged":
                            args = {
                                changeType: entryItem.changeType, childTable: entryItem.childTable, edgeTable: entryItem.edgeTable,
                                isUndo: false, sourcePorts: entryItem.sourcePorts, targetPorts: entryItem.targetPorts, index: entryItem.index
                            };
                            args.object = jQuery.extend(true, {}, entryItem.object);
                            this._recordCollectionChanged(args);
                            break;
                        case "labelchanged":
                            args = { object: entryItem.shape, label: entryItem.updatedLabelValue, index: entryItem.index ? entryItem.index : 0 };
                            this._recordLabelChanged(args);
                            break;
                        case "groupchanged":
                            args = { object: entryItem.object, actionType: entryItem.actionType, isUndo: false };
                            this._recordGroupChanged(args);
                            break;
                        case "phasesizechanged":
                            args = entryItem;
                            var temp = entryItem.redoObject;
                            args.redoObject = entryItem.undoObject;
                            args.undoObject = temp;
                            if (args.modifiedValues) args.modifiedValues.dx = -args.modifiedValues.dx
                            if (args.modifiedValues) args.modifiedValues.dy = -args.modifiedValues.dy
                            this._recordPhasesizeChanged(args);
                            if (args.modifiedValues) args.modifiedValues.dx = -args.modifiedValues.dx
                            if (args.modifiedValues) args.modifiedValues.dy = -args.modifiedValues.dy
                            break;
                        case "phasecollectionchanged":
                            var snapConstraints = this.model.snapSettings.snapConstraints;
                            this.model.snapSettings.snapConstraints = ej.datavisualization.Diagram.SnapConstraints.None;
                            if (entryItem.isAdded && entryItem.phase) {
                                this._addPhase(entryItem.phase.parent, entryItem.phase);
                                this._clearSelection();
                            }
                            else {
                                this._removePhase(entryItem.phase.name);
                            }
                            this.model.snapSettings.snapConstraints = snapConstraints;
                            break;
                        case "propertiesChanged":
                            var node = this.nameTable[entryItem.object.name];
                            var currentState = this._getModifiedProperties(node, entryItem.values, entryItem[entryItem.elementType]);
                            if (entryItem.elementType == "node")
                                this.updateNode(node.name, entryItem.values);
                            else if (entryItem.elementType == "connector")
                                this.updateConnector(node.name, entryItem.values);
                            else if (entryItem.elementType == "label")
                                this.updateLabel(node.name, entryItem.label, entryItem.values);
                            else if (entryItem.elementType == "port")
                                this.updatePort(node.name, entryItem.port, entryItem.values);
                            entryItem.values = currentState;
                            break;
                        case "nudge":
                            this._recordNudgingChanges(object, entryItem);
                            break;
                        case "spacingCommand":
                        case "alignCommand":
                            this._recordAlignCommandChanges(entryItem);
                            break;
                        case "sizeCommand":
                            this._recordSizeCommandChanges(entryItem);
                            break;
                        case "zOrder":
                            var end = entryItem.start;
                            if (entryItem.command == "back") {
                                this._bringElementsToFront(entryItem.start, entryItem.end, entryItem.object, entryItem.nearestNode);
                                entryItem.command = "front";
                            }
                            else {
                                this._sendElementsToBack(entryItem.start, entryItem.end, entryItem.object, entryItem.nearestNode);
                                entryItem.command = "back";
                            }
                            entryItem.nearestNode = this._findObjectByIndex(end);
                            var temp = entryItem.start;
                            entryItem.start = entryItem.end;
                            entryItem.end = temp;
                            break;
                    }
                } else if (this.model.historyManager.redo) {
                    var redo = this.model.historyManager.redo;
                    if (typeof redo == "string") redo = ej.util.getObject(redo, window);
                    if ($.isFunction(redo)) {
                        var data = redo(entryItem.data);
                        if (data) entryItem.data = data;
                    }
                }
            }
        },
        _startGroupAction: function () {
            this.addHistoryEntry({ type: "startGroup", category: "internal" });
        },
        _closeGroupAction: function () {
            this.addHistoryEntry({ type: "closeGroup", category: "internal" });
        },

        addHistoryEntry: function (entry) {
            if (!this.model.historyManager.suspend) {
                if (entry.category != "internal") {
                    entry = { data: entry };
                }
                if (this._historyList.currentEntry && (this._historyList.currentEntry.data || this._historyList.currentEntry.category == "internal")) {
                    var cEntry = this._historyList.currentEntry;
                    if (cEntry.next) {
                        if (cEntry.previous) {
                            var entry1 = cEntry.next;
                            entry1.previous = null;
                            cEntry.next = entry;
                            entry.previous = cEntry;
                        }
                    } else {
                        cEntry.next = entry;
                        entry.previous = cEntry;
                    }
                }
                this._historyList.currentEntry = entry;
                this._historyList.canUndo = true;
                this._historyList.canRedo = false;
            }
        },
        removeHistoryEntry: function (entry) {
            if (this._historyList.currentEntry != null) {
                var cEntry = this._historyList.currentEntry;
                //previous and next entries of target entry
                var prev, nxt;
                if (cEntry) {
                    if (cEntry == entry) {
                        prev = cEntry.previous;
                        nxt = cEntry.next;
                        this._historyList.currentEntry = nxt || prev;
                    } else {
                        var nextEntry = cEntry.next;
                        while (nextEntry) {
                            if (nextEntry == entry) {
                                prev = nextEntry.previous;
                                nxt = nextEntry.next;
                                break;
                            } else nextEntry = nextEntry.next;
                        }
                        var prevEntry = cEntry.previous;
                        while (prevEntry) {
                            if (prevEntry == entry) {
                                prev = prevEntry.previous;
                                nxt = prevEntry.next;
                                break;
                            } else prevEntry = prevEntry.previous;
                        }
                    }
                }
                if (prev && nxt) {
                    prev.next = nxt;
                    nxt.previous = prev;
                } else if (prev) {
                    prev.next = null;
                } else if (nxt) {
                    nxt.previous = null;
                }
            }
        },
        canRemoveHistoryEntry: function (entry) {
            return entry.category != "internal";
        },
        clearHistory: function () {
            var hList = this._historyList;
            hList.currentEntry = null;
            hList.canUndo = false;
            hList.canRedo = false;
            this._hScrollbar._scrollData = null;
            this._vScrollbar._scrollData = null;
        },
        getUndoEntry: function () {
            var undoEntry = null;
            var currentEntry;
            var hList = this._historyList;
            if (hList.canUndo) {
                undoEntry = hList.currentEntry;
                currentEntry = hList.currentEntry.previous;
                if (currentEntry) {
                    hList.currentEntry = currentEntry;
                    if (!hList.canRedo)
                        hList.canRedo = true;
                } else {
                    hList.canRedo = true;
                    hList.canUndo = false;
                }
            }
            return undoEntry;
        },
        getRedoEntry: function () {
            var redoEntry = null;
            var entryCurrent;
            var hList = this._historyList;
            if (hList.canRedo) {
                if (!hList.currentEntry.previous && !hList.canUndo) {
                    entryCurrent = hList.currentEntry;
                } else
                    entryCurrent = hList.currentEntry.next;
                if (entryCurrent) {
                    hList.currentEntry = entryCurrent;
                    if (!hList.canUndo)
                        hList.canUndo = true;
                    if (!entryCurrent.next) {
                        hList.canRedo = false;
                        hList.canUndo = true;
                    }
                }
                redoEntry = hList.currentEntry;
            }
            return redoEntry;
        },
        cut: function () {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                this._pasteIndex = 0;
                this._setClipboard();
                this._delete();
            }
        },
        addPorts: function (name, ports) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var node = this.nameTable[name];
                if (node && ports) {
                    var port;
                    for (var i = 0; i < ports.length; i++) {
                        port = ej.datavisualization.Diagram.Port(ports[i]);
                        node.ports.push(port);
                        ej.datavisualization.Diagram.DiagramContext.renderPort(node, port, this);
                    }
                }
            }
        },
        updatePort: function (nodeName, port, options) {
            if (nodeName && ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var mNode = this._findNode(nodeName);
                if (options && port && (!options.name || port.name == options.name)) {
                    if (mNode) {
                        if (!this._isUndo)
                            this._recordPropertiesChanged(mNode, options, "port", port);
                        for (var prop in options)
                            if (port.hasOwnProperty(prop)) {
                                port[prop] = options[prop];
                            }
                        ej.datavisualization.Diagram.DiagramContext.updatePort(mNode, port, this);
                    }
                }
            }
        },
        copy: function () {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                this._pasteIndex = 1;
                if (this.selectionList && this.selectionList.length > 0) {
                    this._setClipboard();
                }
            }
            return this._cloneSelectionList();
        },
        paste: function (object, rename) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                //this._isUndo = true; 
                this._paste(object, rename !== undefined ? rename : true);
                //this._isUndo = false;
            }
        },
        _commandPaste: function () {
            this.paste();
        },
        _setClipboard: function () {
            if (this.selectionList.length > 0) {
                var data, node = this.selectionList[0];
                var childTable = {};
                if (node._type === "group" || node.type === "pseudoGroup") {
                    var temp, i, j;
                    var children = this._getChildren(node.children);
                    for (i = 0; i < children.length; i++) {
                        for (j = i + 1; j < children.length; j++) {
                            var nodei = this.nameTable[children[i]];
                            var nodej = this.nameTable[children[j]];
                            if (nodej && nodei) {
                                if (nodej.zOrder < nodei.zOrder) {
                                    temp = children[i];
                                    children[i] = children[j];
                                    children[j] = temp;
                                }
                            }
                        }
                    }
                    childTable = this._getChildTable(node, childTable);
                }
                childTable[node.name] = node;
                data = $.extend(true, {}, { "childTable": childTable, "node": node });
                if (data.node.segments) {
                    data.node.targetNode = null;
                    data.node.targetPort = null;
                    data.node.sourceNode = null;
                    data.node.sourcePort = null;
                }
                this._clipboardData = data;
            }
        },
        _getChildTable: function (group, childTable) {
            var child, names, name;
            names = this._getChildren(group.children);
            if (names) {
                for (var i = 0; i < names.length; i++) {
                    name = names[i];
                    child = this.nameTable[name];
                    if (child && (child.parent === group.name || group.type == "pseudoGroup")) {
                        var node = childTable[child.name] = $.extend(true, {}, child);
                        if (node._type === "group" || node.type === "pseudoGroup")
                            childTable = this._getChildTable(node, childTable);
                        if (node.inEdges && node.inEdges.length > 0) {
                            for (var e in node.inEdges) {
                                var cCon = this.nameTable[node.inEdges[e]];
                                if (cCon) {
                                    if (childTable[cCon.name])
                                        delete childTable[cCon.name];
                                    childTable[cCon.name] = cCon;
                                }
                            }
                        }
                        if (node.outEdges && node.outEdges.length > 0) {
                            for (var f in node.outEdges) {
                                var cCon = this.nameTable[node.outEdges[f]];
                                if (cCon) {
                                    if (childTable[cCon.name])
                                        delete childTable[cCon.name];
                                    childTable[cCon.name] = cCon;
                                }
                            }
                        }
                    }
                }
            }
            return childTable;
        },
        _paste: function (object, rename) {
            if (!object) {
                if (this._clipboardData) {
                    var data = this._clipboardData;
                    var dx, child;
                    data = $.extend(true, {}, { "childTable": data.childTable, "node": data.node });
                    var node = data.node;
                    node.zOrder = -1;
                    if (node && node.type != "phase") {
                        if (!node.container) {
                            if (node.type == "pseudoGroup") {
                                var names = node.children;
                                var pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                                this._isUndo = true;
                                for (var i = 0; i < names.length; i++) {
                                    var name = names[i];
                                    child = data.childTable[name];
                                    child.zOrder = -1;
                                    if (child && (child.isLane || child.isSwimlane)) {
                                        var swimChild = this._pasteSwimlane(child, data, true, this._pasteIndex * 10);
                                        if (swimChild)
                                            pseudoGroup.children.push(swimChild.name);
                                    } else if (child) {
                                        name = name + ej.datavisualization.Diagram.Util.randomId();
                                        delete (data.childTable[child.name]);
                                        child.name = name;
                                        child.parent = "";
                                        data.childTable[child.name] = child;
                                        this._preserveConnection(data.childTable, child);
                                        pseudoGroup.children.push(name);
                                        if (child._type === "group" && child.type != "bpmn") {
                                            this._pasteChildren(data.childTable, child);
                                        }
                                        else
                                            child.children = [];
                                        dx = this._pasteIndex * 10;
                                        this._translate(child, dx, dx, this.nameTable);
                                        if (child.segments && child.segments.length == 1 && child.segments[0].type == "orthogonal" && !child.sourceNode && !child.targetNode) {
                                            points = child.segments[0].points;
                                        }
                                        var success = false;
                                        success = this.add(child);
                                        if (points && success) {
                                            child = this.nameTable[child.name];
                                            child.segments[0].points = points;
                                            ej.datavisualization.Diagram.DiagramContext.update(child, this);
                                            ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.selectionList[0], this._adornerSvg, this._currZoom);
                                        }
                                    }
                                }
                                this._clearSelection();
                                this.nodes().push(pseudoGroup);
                                this._nodes = $.extend(true, [], this.nodes());
                                this.nameTable[pseudoGroup.name] = pseudoGroup;
                                ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this);
                                this._addSelection(pseudoGroup);
                                var childTable = {};
                                childTable = this._getChildTable(this.selectionList[0], childTable);
                                var entry = { type: "collectionchanged", object: this._updatePsedoGroupChildren(this.selectionList[0]), childTable: jQuery.extend(true, {}, childTable), changeType: "insert", category: "internal" };
                                this.addHistoryEntry(entry);
                                this._isUndo = false;
                            } else if (node.type == "bpmn") {
                                var randomId = ej.datavisualization.Diagram.Util.randomId();
                                node.parent = "";
                                node.name = node.name + randomId;
                                node.inEdges = [];
                                node.outEdges = [];
                                node.children = [];
                                dx = this._pasteIndex * 10;
                                this._translate(node, dx, dx, this.nameTable);
                                this.add(node);
                            } else {
                                node.name = node.name + ej.datavisualization.Diagram.Util.randomId();
                                node.parent = "";
                                if (node._type === "group") {
                                    this._pasteChildren(data.childTable, node);
                                } else if (node.segments) {
                                    //this._disConnect(node);
                                } else if (!node.segments) {
                                    //@Fix
                                    //this._disConnect(node);
                                    node.inEdges = [];
                                    node.outEdges = [];
                                }
                                dx = this._pasteIndex * 10;
                                this._translate(node, dx, dx, this.nameTable);
                                if (node.segments && node.segments.length == 1 && node.segments[0].type == "orthogonal" && !node.sourceNode && !node.targetNode) {
                                    var points = node.segments[0].points;
                                }
                                var success = this.add(node);
                                if (success && points) {
                                    node = this.nameTable[node.name];
                                    node.segments[0].points = points;
                                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                                    ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.selectionList[0], this._adornerSvg, this._currZoom);
                                }
                                this._updateSelectionHandle();
                                //this._clearSelection();
                                //this._addSelection(node);
                            }
                        } else {
                            node = this._pasteSwimlane(node, data);
                            dx = this._pasteIndex * 10;
                            this._translate(node, dx, dx, this.nameTable);
                            ej.datavisualization.Diagram.DiagramContext.update(node, this);
                            if (this._selectionContains(node)) {
                                this._clearSelection(true);
                                this._addSelection(node);
                            }
                        }
                        this._pasteIndex++;
                    }
                }
            }
            else {
                var name = "";
                var connectors = [];
                var pseudoGroup = ej.datavisualization.Diagram.Group({ "name": "multipleSelection", type: "pseudoGroup" });
                if (rename)
                    name = ej.datavisualization.Diagram.Util.randomId();;
                for (var i = 0; i < object.length; i++) {
                    object[i].name += name;
                    if (object[i].type != "connector") {
                        if ((object[i].type === "group" || object[i].type === "bpmn") && !object[i].isSwimlane) {
                            this._clonePasteGroup(object[i], name);
                        }
                        if (object[i].isSwimlane) {
                            this._clonePasteSwimlaneObj(object[i], name);
                        }
                        if (object[i].outEdges) object[i].outEdges = [];
                        if (object[i].inEdges) object[i].inEdges = [];
                        if (object.length > 1) pseudoGroup.children.push(object[i].name);
                        this.add($.extend({}, object[i]));
                    }
                    else
                        connectors.push(object[i]);
                }
                for (var i = 0; i < connectors.length; i++) {
                    if (connectors[i].sourceNode && this.nameTable[connectors[i].sourceNode + name])
                        connectors[i].sourceNode += name;
                    else {
                        connectors[i].sourceNode = "";
                        connectors[i].sourcePort = "";
                    }
                    if (connectors[i].targetNode && this.nameTable[connectors[i].targetNode + name])
                        connectors[i].targetNode += name;
                    else {
                        connectors[i].targetNode = "";
                        connectors[i].targetPort = "";
                    }
                    if (object.length > 1) pseudoGroup.children.push(connectors[i].name);
                    this.add($.extend({}, connectors[i]));
                }
                if (object.length > 1) {
                    this.nodes().push(pseudoGroup);
                    this._nodes = $.extend(true, [], this.nodes());
                    this.nameTable[pseudoGroup.name] = pseudoGroup;
                    this._clearSelection(true);
                    ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this);
                    this.addSelection(pseudoGroup, true);
                }
            }
        },
        _clonePasteGroup: function (group, name) {
            for (var i = 0; i < group.children.length; i++) {
                group.children[i].name += name;
                group.children[i].parent = group.name;
            }
        },
        _clonePasteSwimlaneObj: function (obj, name) {
            obj.name += name;
            //update lanes
            if (obj.lanes && obj.lanes.length > 0) {
                for (var i = 0; i < obj.lanes.length; i++) {
                    if (obj.lanes[i]) {
                        obj.lanes[i].name += name;
                        if (obj.lanes[i].children && obj.lanes[i].children.length > 0) {
                            var children = obj.lanes[i].children;
                            for (var m = 0; m < children.length; m++) {
                                var child = jQuery.extend(true, {}, children[m]);
                                child.name += name;
                                child.parent = obj.lanes[i].name;
                                obj.lanes[i].children[m] = child;
                            }
                        }
                    }
                }
            }
            //update phases
            if (obj.phases && obj.phases.length > 0) {
                for (var i = 0; i < obj.phases.length; i++) {
                    if (obj.phases[i]) {
                        obj.phases[i].name += name;
                        obj.phases[i].parent = obj.name;
                    }
                }
            }
        },
        _cloneSwimlaneObj: function (obj, name) {
            obj = jQuery.extend(true, {}, obj);
            obj.name += name;
            //update lanes
            if (obj.lanes && obj.lanes.length > 0) {
                for (var i = 0; i < obj.lanes.length; i++) {
                    if (obj.lanes[i]) {
                        obj.lanes[i].name += name;
                        if (obj.lanes[i].children && obj.lanes[i].children.length > 0) {
                            var children = obj.lanes[i].children;
                            for (var m = 0; m < children.length; m++) {
                                var child = jQuery.extend(true, {}, children[m]);
                                child.name += name;
                                child.parent = obj.lanes[i].name;
                                obj.lanes[i].children[m] = child;
                            }
                        }
                    }
                }
            }
            //update phases
            if (obj.phases && obj.phases.length > 0) {
                for (var i = 0; i < obj.phases.length; i++) {
                    if (obj.phases[i]) {
                        obj.phases[i].name += name;
                        obj.phases[i].parent = obj.name;
                    }
                }
            }
            return obj;
        },
        _pasteSwimlane: function (node, data, mSelection, x) {
            x = x ? x : 0;
            var orgName = node.name;
            var name = ej.datavisualization.Diagram.Util.randomId();
            this._cloneSwimlane(node, data, name);
            if (node.isSwimlane) {
                var nSwimLane = this.getNode(node.name);
                if (!nSwimLane)
                    nSwimLane = this.getNode(data.childTable[orgName], data.childTable);
                node.offsetX += x;
                node.offsetY += x;
                node = ej.datavisualization.Diagram.SwimLaneHelper._initSwimLane(ej.datavisualization.Diagram.SwimLane(nSwimLane), this);
                this.add(node);
                ej.datavisualization.Diagram.DiagramContext.update(node, this);
                return node;
            }
            else if (node.isLane) {
                var swimlane = ej.datavisualization.Diagram.SwimLaneHelper._createSwimlane(node, this, data);
                if (swimlane) {
                    swimlane.width = 0;
                    swimlane.height = 0;
                    this.add(swimlane);
                    return swimlane;
                }
            }
        },
        _cloneNode: function (node, data, name, pnode) {
            var nameState = false;
            var parentName = null;
            if (node) {
                node = (typeof (node) === "string") ? node : node.name;
                var node = jQuery.extend(true, {}, data.childTable[node]);
                if (node.isPhaseStack) {
                    node.name = node.name.split("phaseStack")[0] + name + "phaseStack";
                    nameState = true;
                    parentName = node.name;
                }
                if (node.children && node.children.length > 0) {
                    for (var i in node.children) {
                        this._cloneNode(node.children[i], data, name, parentName);
                        if (node.children[i]) (typeof (node.children[i]) === "string") ? node.children[i] += name : node.children[i].name += name;
                    }
                }
                if (pnode) {
                    node.parent = (typeof (pnode) === "string") ? pnode : pnode.name;
                }
                else if (node.parent != "")
                    node.parent += name;
                if (!nameState)
                    node.name += name;
                this._preserveConnection(data.childTable, node);
                this.nameTable[node.name] = node;
            }
        },
        _cloneSwimlane: function (node, data, name) {
            var tempName = node.name;
            node.name += name;
            if (node.children && node.children.length > 0) {
                for (var i in node.children) {
                    this._cloneNode(node.children[i], data, name, node);
                    if (typeof (node.children[i]) === "string") {
                        (node.children[i] += name)
                    }
                    else {
                        if (node.children[i].isPhaseStack)
                            node.children[i] = tempName + name + "phaseStack";
                        else
                            node.children[i] = node.children[i].name + name;
                    }
                }
            }
            if (node.phases && node.phases.length > 0) {
                var phase;
                for (var i in node.phases) {
                    phase = jQuery.extend(true, {}, this.nameTable[node.phases[i]])
                    phase.name += name;
                    phase.parent += name;
                    this.nameTable[phase.name] = phase;
                    node.phases[i] = phase.name;
                }
            }
            this.nameTable[node.name] = node;
        },
        _updateChildName: function (node, name) {
            if (node && name) {
                var children = node.children;
                for (var i = 0; i < children.length; i++) {
                    children[i] = children[i] + name;
                }
            }
        },
        _pasteChild: function (child, childTable, group, childNames) {
            var name = child.name + ej.datavisualization.Diagram.Util.randomId();
            delete (childTable[child.name]);
            child.name = name;
            childTable[child.name] = child;
            this._preserveConnection(childTable, child);
            child.parent = group.name;
            childNames.push(name);
            if (child._type === "group") {
                this._pasteChildren(childTable, child);
            }
            this.add(child);
        },
        _pasteChildren: function (childTable, group) {
            var child, names, name;
            var childNames = [];
            names = group.children;
            for (var i = 0; i < names.length; i++) {
                name = this._getChild(names[i]);
                child = childTable[name];
                child.zOrder = -1;
                if (child) {
                    this._pasteChild(child, childTable, group, childNames);
                }
            }
            group.children = childNames;
            this._preserveConnection(childTable, group);
        },
        _preserveConnections: function (childTable, group) {
            var child, names;
            names = this._getChildren(group.children);
            for (var i = 0; i < names.length; i++) {
                child = childTable[names[i]];
                if (child) {
                    this._preserveConnection(childTable, child);
                    if (child._type === "group")
                        this._preserveConnections(childTable, group);
                }
            }
        },
        _preserveConnection: function (childTable, child) {
            var i, connection, len;
            if (!child.segments) {
                len = child.inEdges.length;
                if (len > 0) {
                    for (i = 0; i < len; i++) {
                        connection = childTable[child.inEdges[i]];
                        if (connection) {
                            connection.targetNode = child.name;
                        }
                    }
                    child.inEdges = [];
                }
                len = child.outEdges.length;
                if (len > 0) {
                    for (i = 0; i < child.outEdges.length; i++) {
                        connection = childTable[child.outEdges[i]];
                        if (connection) {
                            connection.sourceNode = child.name;
                        }
                    }
                    child.outEdges = [];
                }
            } else {
                var node;
                if (child.targetNode) {
                    node = childTable[child.targetNode];
                    if (node)
                        node.inEdges.push(child.name);
                    //else
                    //    child.targetNode = null;
                }
                if (child.sourceNode) {
                    node = childTable[child.sourceNode];
                    if (node)
                        node.outEdges.push(child.name);
                    //else
                    //    child.sourceNode = null;
                }
            }
        },
        _expandChildren: function (node) {
            for (var i = 0; i < node.outEdges.length; i++) {
                var conn = this.nameTable[node.outEdges[i]];
                conn.visible = true;
                ej.datavisualization.Diagram.DiagramContext.update(conn, this);
                this._updateQuad(conn);
                var target = this.nameTable[conn.targetNode];
                if (target.isExpanded)
                    this._expandChildren(target);
                target.visible = true;
                target.excludeFromLayout = false;
                ej.datavisualization.Diagram.DiagramContext.update(target, this);
                this._updateQuad(target);
            }
        },
        _collapseChildren: function (node) {
            for (var i = 0; i < node.outEdges.length; i++) {
                var conn = this.nameTable[node.outEdges[i]];
                conn.visible = false;
                ej.datavisualization.Diagram.DiagramContext.update(conn, this);
                ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[conn.name], conn);
                ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, conn);
                var target = this.nameTable[conn.targetNode];
                if (target.isExpanded)
                    this._collapseChildren(target);
                target.visible = false;
                target.excludeFromLayout = true;
                ej.datavisualization.Diagram.DiagramContext.update(target, this);
                ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[target.name], target);
                ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, target);
            }
        },
        zoomTo: function (zoom) {
            if (!zoom) {
                zoom = new ej.datavisualization.Diagram.Zoom();
            }
            if (zoom.zoomCommand & ej.datavisualization.Diagram.ZoomCommand.ZoomIn) {
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 + zoom.zoomFactor), 0, 0, zoom.focusPoint, true);
            } else {
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 / (1 + zoom.zoomFactor)), 0, 0, zoom.focusPoint, true);
            }
        },
        remove: function (node) {
            if (node && this.nameTable[node.name] && ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                this._delete(node);
            }
            else
                this._delete();
        },
        clear: function () {
            this._clearSelection();
            var i;
            for (i = this.connectors().length - 1; i >= 0; i--) {
                this._remove(this.connectors()[i], null, true);
                this._connectors = $.extend(true, [], this.connectors());
            }
            for (i = this.nodes().length - 1; i >= 0; i--) {
                this._remove(this.nodes()[i], null, true);
                this._nodes = $.extend(true, [], this.nodes());
            }
            this._nodes = [];
            this._connectors = [];
            this.clearHistory();
            $("#" + this._canvas.id + "_hScrollbar").ejScrollBar({ width: 0 });
            $("#" + this._canvas.id + "_vScrollbar").ejScrollBar({ height: 0 });
        },
        _updateChildOnGroup: function (node) {
            var parent = null;
            if (node && node.parent) {
                parent = this._svg.document.getElementById(node.parent);
                var parentObj = this.nameTable[node.parent];
                if (parentObj)
                    parentObj.children.push(node.name);
                if (node._type == "group") this._initGroupNode(node);
                ej.datavisualization.Diagram.Util._refreshParentGroup(node, this);
            }
            return parent;
        },
        _add: function (node, render, arg, restrictSelection) {
            var success = false;
            if (node.isSwimlane && (node.lanes && node.lanes.length > 0)) {
                node = ej.datavisualization.Diagram.SwimLaneHelper._initSwimLane(ej.datavisualization.Diagram.SwimLane(node), this);
            }
            if (node.length > 0) {
                for (var i = 0; i < node.length; i++) {
                    this.add(node[i]);
                }
            } else {
                node = ej.datavisualization.Diagram.NodeType(node, this);
                //#region for temporary
                if (node.addInfo) {
                    if (node.addInfo.height)
                        node.height = node.addInfo.height;
                    if (node.addInfo.width)
                        node.width = node.addInfo.width;
                }
                //#endregion
                if (node.type == "connector") var type = "connector"; else type = "node";
                var args = { changeType: "insert", element: this.getNode(node), cancel: false };
                if (!node.isPhase && (!this._selectedSymbol || this._endEditing)) {
                    this._raiseEvent(type + "CollectionChange", args);
                    node._absolutePath = null;
                }
                if (node.isLane)
                    this._updateLane(node, args.element);
                else if (node.isSwimlane)
                    this._updateSwimlane(node, args.element);
                if (!args.cancel) {
                    if (!(node.isLane || node.isSwimlane)) {
                        if (node.type != "connector") {
                            if (node._type === "group" || (node.children && node.children.length > 0)) {
                                node = ej.datavisualization.Diagram.Group(node);
                                node._type = "group";
                                this._initGroupNode(node);
                            }
                            else if (!node.isPhase) {
                                //if (!(this.activeTool && this.activeTool.name === "textTool"))
                                node = this._getNewNode(node);
                                this._getNodeDimension(node);
                            }
                            success = this.nodes().push(node);
                            this._nodes = $.extend(true, [], this.nodes());
                            this.nameTable[node.name] = node;
                            if (success) {
                                if (node._type === "group" || (node.children && node.children.length > 0)) {
                                    this._updateParent(node);
                                    if (node.offsetX == 0 && node.offsetY == 0)
                                        ej.datavisualization.Diagram.Util._updateGroupBounds(node, this, true);
                                    this._updateChildrenEdges(node);
                                    var parent = null;
                                    if (node.parent) {
                                        var parent = this._svg.document.getElementById(node.parent);
                                        var parentObj = this.nameTable[node.parent];
                                        if (parentObj)
                                            parentObj.children.push(node.name);
                                        this._initGroupNode(node);
                                        ej.datavisualization.Diagram.Util._refreshParentGroup(node, this);

                                    }
                                    if (render && !node.parent) {
                                        ej.datavisualization.Diagram.Util._updateGroupBounds(node, this, true);
                                        ej.datavisualization.Diagram.DiagramContext.renderGroup(node, this);
                                    }
                                } else {
                                    if (render) {
                                        var parent = null;
                                        parent = this._updateChildOnGroup(node);
                                        if ((node.parent === "" || node.isPhase) || this._isInsert)
                                            ej.datavisualization.Diagram.DiagramContext.renderNode(node, this, parent);
                                    }
                                }
                                this._updateQuad(node);
                                this.nameTable[node.name] = node;
                                if (render && !restrictSelection) {
                                    this._clearSelection(true);
                                    this._addSelection(node);
                                }
                            }
                        }
                        else {
                            node = this._getNewConnector(node);
                            success = this.model.connectors.push(node);
                            if (success) {
                                this._updateEdges(node);
                                this._dock(node, this.nameTable);
                                ej.datavisualization.Diagram.Util.updateBridging(node, this);
                                if (!node.parent) {
                                    if (render)
                                        ej.datavisualization.Diagram.DiagramContext.renderConnector(node, this);
                                }
                                this.nameTable[node.name] = node;
                                this._updateQuad(node);
                                this._updateChildOnGroup(node);
                                if (render && !restrictSelection) {
                                    this._clearSelection(true);
                                    this._addSelection(node);
                                }
                            }
                        }
                    }
                    else {
                        success = this.nodes().push(node);
                        this._nodes = $.extend(true, [], this.nodes());
                        this.nameTable[node.name] = node;
                        if (success) {
                            if (node._type === "group") {
                                if (render) {
                                    if (this._paletteItemClick) {
                                        node.offsetX = node.minWidth / 2 + 50;
                                        node.offsetY = node.minHeight / 2 + 50;
                                    }
                                    ej.datavisualization.Diagram.DiagramContext.renderGroup(node, this);
                                    if (node.phases && node.phases.length > 0) {
                                        var cphase = null;
                                        for (var i = 0, len = node.phases.length; i < len; i++) {
                                            cphase = this.nameTable[node.phases[i]];
                                            if (cphase.type == "phase") {
                                                ej.datavisualization.Diagram.SvgContext.renderphase(cphase, this._svg, this._diagramLayer, this);
                                            }
                                        }
                                    }
                                    this._clearSelection(true);
                                    this._addSelection(node);
                                }
                            }
                            this._updateQuad(node);
                        }
                    }
                    this._setZorder(node);
                }
            }
            if (!this._isUndo && (!this._selectedSymbol || this._endEditing) && success) {
                var childTable = {};
                if (this.selectionList && this.selectionList.length > 0)
                    childTable = this._getChildTable(this.selectionList[0], childTable);
                if (node.isSwimlane) {
                    node = this._getNode(node.name, this.nameTable);
                }
                if ((!arg || (arg && !arg.entryHistory)) && !node.isPhase) {
                    var entry = { type: "collectionchanged", object: jQuery.extend(true, {}, node), childTable: jQuery.extend(true, {}, childTable), changeType: "insert", category: "internal" };
                    this.addHistoryEntry(entry);
                }
            }
            this._trigger("refresh");
            return success;
        },
        add: function (node, args) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                return this._add(node, true, args);
            }
        },
        updateUserHandles: function (node) {
            node = this.nameTable[node.name];
            ej.datavisualization.Diagram.SvgContext.updateUserHandles(this.model.selectedItems.userHandles, node, this._adornerSvg, false, false, this._currZoom, this);
        },
        scrollToNode: function (node) {
            var nodeX, nodeY;
            var curZoomfactor = this._currZoom;
            var elementBounds = this.element[0].getBoundingClientRect();
            var nodeBounds = ej.datavisualization.Diagram.Util.bounds(node);
            nodeX = (nodeBounds.x + nodeBounds.width / 2) * curZoomfactor;
            nodeY = (nodeBounds.y + nodeBounds.height / 2) * curZoomfactor;
            var hScrollOffset = this._hScrollOffset, vScrollOffset = this._vScrollOffset;
            if (elementBounds.width + this._hScrollOffset < nodeX + nodeBounds.width / 2) {
                hScrollOffset = (nodeX + ((nodeBounds.width / 2 + 20) * curZoomfactor) - elementBounds.width); // * 20 as scroll bar width 
            }
            if (elementBounds.height + this._vScrollOffset < nodeY + nodeBounds.height / 2) {
                vScrollOffset = (nodeY + ((nodeBounds.height / 2 + 20) * curZoomfactor) - elementBounds.height); // * 20 as scroll bar height 
            }
            this._updateScrollOffset(hScrollOffset, vScrollOffset);
        },
        findNode: function (id) {
            return this._findNode(id);
        },
        getObjectType: function (element) {
            return ej.datavisualization.Diagram.Util.getObjectType(element);
        },
        //#endregion
        //#region support methods       
        _registerDrawingTools: function () {
            for (var toolname in this.model.drawingTools) {
                var tool = this.model.drawingTools[toolname];
                tool.diagram = this;
                this.tools[tool.name] = tool;
            }
        },
        _sortByXvalue: function (nodes) {
            var temp, i, j;
            var fNode, lNode;
            for (i = 0; i < nodes.length; i++) {
                for (j = i + 1; j < nodes.length; j++) {
                    var nodei = this.nameTable[nodes[i]];
                    var nodej = this.nameTable[nodes[j]];
                    fNode = ej.datavisualization.Diagram.Util.bounds(nodei);
                    lNode = ej.datavisualization.Diagram.Util.bounds(nodej);
                    if ((fNode.x > lNode.x) || (fNode.right > lNode.right)) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
            }
            return nodes;
        },
        _sortByYvalue: function (nodes) {
            var temp, i, j;
            var fNode, lNode;
            for (i = 0; i < nodes.length; i++) {
                for (j = i + 1; j < nodes.length; j++) {
                    var nodei = this.nameTable[nodes[i]];
                    var nodej = this.nameTable[nodes[j]];
                    fNode = ej.datavisualization.Diagram.Util.bounds(nodei);
                    lNode = ej.datavisualization.Diagram.Util.bounds(nodej);
                    if ((fNode.y > lNode.y) || (fNode.bottom > lNode.bottom)) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
            }
            return nodes;
        },
        _sortByZIndex: function (nodes, asc) {
            var temp, i, j;
            if (!asc) {
                for (i = 0; i < nodes.length; i++) {
                    for (j = i + 1; j < nodes.length; j++) {
                        if (nodes[i].zOrder < nodes[j].zOrder) {
                            temp = nodes[i];
                            nodes[i] = nodes[j];
                            nodes[j] = temp;
                        }
                    }

                }
            }
            else {
                for (i = 0; i < nodes.length; i++) {
                    for (j = i + 1; j < nodes.length; j++) {
                        if (nodes[i].zOrder > nodes[j].zOrder) {
                            temp = nodes[i];
                            nodes[i] = nodes[j];
                            nodes[j] = temp;
                        }
                    }

                }
            }
            return nodes;
        },
        _sortByGroupChildrenZIndex: function (asc) {
            var nodes = this.nodes();
            var temp, i, j, k;
            for (k = 0; k < nodes.length; k++) {
                var node = nodes[k];
                if (node._type === "group" && node.type !== "bpmn" && !node.isSwimlane) {
                    var children = this._getChildren(node.children);
                    for (i = 0; i < children.length; i++) {
                        for (j = i + 1; j < children.length; j++) {
                            var childeNode1 = this.nameTable[children[i]];
                            var childeNode2 = this.nameTable[children[j]];
                            if ((!asc ? childeNode1.zOrder < childeNode2.zOrder : childeNode1.zOrder > childeNode2.zOrder) &&
                                !childeNode1.isPhase && !childeNode2.isPhase && !childeNode1.isLane && !childeNode2.isLane) {
                                temp = children[i];
                                children[i] = children[j];
                                children[j] = temp;
                            }
                        }
                    }
                    node.children = children;
                }
            }
        },
        _findCornerNodes: function () {
            var lNode, rNode, tNode, bNode, i, nBounds;
            var list = this.selectionList[0].children;
            var fnode = this.nameTable[this.selectionList[0].children[0]];
            lNode = fnode, rNode = fnode, tNode = fnode, bNode = fnode;
            if (list.length > 1) {
                for (i = 0; i < list.length; i++) {
                    var node = this.nameTable[list[i]];
                    nBounds = ej.datavisualization.Diagram.Util.bounds(node);

                    var rNodeBounds = ej.datavisualization.Diagram.Util.bounds(rNode);
                    if ((nBounds.x + nBounds.width) > (rNodeBounds.x + rNodeBounds.width))
                        rNode = node;

                    var bNodeBounds = ej.datavisualization.Diagram.Util.bounds(bNode);
                    if ((nBounds.y + nBounds.height) > (bNodeBounds.y + bNodeBounds.height))
                        bNode = node;

                    var lNodeBounds = ej.datavisualization.Diagram.Util.bounds(lNode);
                    if (nBounds.x < lNodeBounds.x)
                        lNode = node;

                    var tNodeBounds = ej.datavisualization.Diagram.Util.bounds(tNode);
                    if (nBounds.y < tNodeBounds.y)
                        tNode = node;
                }
            }
            return { left: lNode, right: rNode, top: tNode, bottom: bNode };
        },
        _moveNode: function (diffX, diffY, node) {
            this._translate(node, diffX, diffY, this.nameTable);
            this._updateNodeMargin(node);
            ej.datavisualization.Diagram.DiagramContext.update(node, this);
        },
        _nudge: function (node, x, y, direction, isChild) {
            //this.activateTool("move");
            //this.activeTool.inAction = true; 
            var proceedY = true, proceedX = true, move = {}, child;
            var pNode = this.nameTable[node.parent];
            if (pNode) {
                if (pNode.isLane)
                    move = this._moveOnLane(node, pNode, x, y);
                if (move && !move.proceedY) {
                    proceedY = false;
                }
                if (move && !move.proceedX) {
                    proceedX = false;
                }
            }
            var resource;
            if (!isChild) {
                if (!node.segments) {
                    var oldValue = (direction === "right" || direction === "left") ? node.offsetX : node.offsetY;
                    var newValue = (direction === "right" || direction === "left") ? node.offsetX + x : node.offsetY + y;                    
                    var propertyName = (direction === "right" || direction === "left") ? "offsetX" : "offsetY";
                    resource = { element: node, cause: ej.datavisualization.Diagram.ActionType.Nudge, keyCode: direction, propertyName: propertyName, oldValue: oldValue, newValue: newValue };
                    this._raisePropertyChange(resource);
                }
                else {
                    resource = {
                        element: node, cause: ej.datavisualization.Diagram.ActionType.Nudge, keyCode: direction, propertyName: "sourcePoint", oldValue: { sourcePoint: node.sourcePoint },
                        newValue: { sourcePoint: { x: node.sourcePoint.x + x, y: node.sourcePoint.y + y } }
                    };
                    this._raisePropertyChange(resource);
                    resource = {
                        element: node, cause: ej.datavisualization.Diagram.ActionType.Nudge, keyCode: direction, propertyName: "targetPoint", oldValue: { targetPoint: node.targetPoint },
                        newValue: { targetPoint: { x: node.targetPoint.x + x, y: node.targetPoint.y + y } }
                    };
                    this._raisePropertyChange(resource);
                }
            }
            if (!proceedY)
                y = 0;
            if (!proceedX)
                x = 0;
            if (node.type == "pseudoGroup" && this.activeTool._isParentAsLane(node)) {
                for (var i = 0; i < node.children.length; i++) {
                    child = this.nameTable[this._getChild(node.children[i])];
                    if (child) {
                        this._nudge(child, x, y, direction, true);
                    }
                }
                ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
            }
            else
                this._translate(node, x, y, this.nameTable);
            this._updateNodeMargin(node);
            if (node.parent) {
                var pNode = this.nameTable[node.parent];
                if (pNode && pNode.isLane) {
                    var moveTool = this.tools["move"];
                    moveTool._updateContainer(node, pNode, true);
                }
            }
            else
                ej.datavisualization.Diagram.DiagramContext.update(node, this);
            this._updateSelectionHandle(true);
            //ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._svg, this._currZoom, this, this.model.selectedItems.constraints);
        },
        _moveOnLane: function (node, parent, x, y) {
            var proceedY = false, dx = 0, proceedX = false, dy = 0;
            x = x ? x : 0; y = y ? y : 0;
            if (parent.children && parent.children.length > 0) {
                var header = this.nameTable[this._getChild(parent.children[0])];
                if (header && header.name.indexOf("_Headerr") != -1) {
                    if (parent.orientation === "horizontal") {
                        dx = header.width;
                    }
                    else if (parent.orientation === "vertical") {
                        dy = header.height;
                    }
                }
            }
            if (node && parent) {
                var nBBox = ej.datavisualization.Diagram.Util.bounds(node);
                var gBBox = ej.datavisualization.Diagram.Util.bounds(parent);
                if (nBBox.y + y >= gBBox.y + parent.paddingTop + dy) {
                    proceedY = true;
                }
                if (nBBox.x + x >= gBBox.x + parent.paddingLeft + dx) {
                    proceedX = true;
                }
                return {
                    proceedY: proceedY, proceedX: proceedX,
                    x: (gBBox.x + parent.paddingLeft + dx) - nBBox.x + x,
                    y: (gBBox.y + parent.paddingTop + dy) - nBBox.y + y,
                };
            }
        },
        _selectCommand: function (args) {
            if (this._isEditing) {
                this._endEdit();
                document.getElementById(this._id).focus();
            }
            args.keyDownEventArgs.preventDefault();
            this.selectAll();
        },
        _nudgeCommand: function (args) {
            if (!this._currentState && this.selectionList[0]) { this._currentState = { x: this.selectionList[0].offsetX, y: this.selectionList[0].offsetY }; }
            this.nudge(args.parameter, args.keyDownEventArgs.shiftKey ? 5 : 1);
            args.keyDownEventArgs.preventDefault();
        },
        _startEditCommand: function () {
            if (this.selectionList.length > 0) {
                var shape = this.selectionList[0];
                if (!(shape.type == "pseudoGroup")) {
                    if (shape.type === "text" && this._setLabelEditing(shape.textBlock))
                        this._startEdit(shape);
                    if (shape.isSwimlane) {
                        var header = this.nameTable[shape.name + "_header_swimlane"];
                        if (header)
                            this._startEdit(header);
                    }
                    else if (shape.type && shape.type === "phase") {
                        var phase = this.nameTable["phaseStack" + shape.name]
                        if (phase)
                            this._startEdit(phase);
                    }
                    else if (shape.labels.length > 0 && this._setLabelEditing(shape.labels[0]))
                        this._startEdit(shape);
                }
            }
        },
        _findObjectByName: function (name) {
            var node = null;
            if (this._findNode(name))
                node = this._findNode(name);
            if (this._findConnector(name))
                node = this._findConnector(name);
            return node;
        },
        _findObjectByIndex: function (index, group) {
            var nodes, findNode = null, node, m, k;
            if (group) {
                var parent = this.nameTable[this.selectionList[0].parent];
                if (parent) {
                    nodes = this._getChildren(parent.children);
                    for (k = 0; k < nodes.length; k++) {
                        node = this.nameTable[nodes[k]];
                        if (node.zOrder === index)
                            findNode = nodes[k];
                    }
                }
            } else {
                for (m = 0; m < this.nodes().length; m++) {
                    node = this.nodes()[m];
                    if (node.zOrder === index)
                        findNode = node;
                }
                for (m = 0; m < this.connectors().length; m++) {
                    node = this.connectors()[m];
                    if (node.zOrder === index)
                        findNode = node;
                }
            }
            if (typeof (findNode) === "string")
                return this.nameTable[findNode];
            return findNode;
        },
        _findOverlapNode: function (node) {
            var oNodes = [], m, i, rNodes = [];
            if (!node.parent) {
                for (m = 0; m < this.nodes().length; m++) {
                    oNodes.push(this.nodes()[m]);
                }
                for (m = 0; m < this.connectors().length; m++) {
                    oNodes.push(this.connectors()[m]);
                }
                var nBound = ej.datavisualization.Diagram.Util.bounds(node);
                if (node) {
                    for (i = 0; i < oNodes.length; i++) {
                        var chBound = ej.datavisualization.Diagram.Util.bounds(oNodes[i]);
                        if (node.name !== oNodes[i].name && !(node.children && this._collectionContains(oNodes[i].name, node.children))) {
                            if (this._intersect(nBound, chBound))
                                rNodes.push(oNodes[i]);
                        }
                    }
                }
            } else {
                var nBound = ej.datavisualization.Diagram.Util.bounds(node);
                var parent = this.nameTable[node.parent], child;
                if (parent) {
                    var children = this._getChildren(parent.children);
                    for (m = 0; m < children.length; m++) {
                        if (children[m] !== node.name) {
                            child = this.nameTable[children[m]];
                            var chBound = ej.datavisualization.Diagram.Util.bounds(child);
                            if (child && this._intersect(nBound, chBound))
                                rNodes.push(child);
                        }
                    }
                }
            }
            return rNodes;
        },
        _intersect: function (a, b) {
            return (a.x <= (b.x + b.width) &&
                b.x <= (a.x + a.width) &&
                a.y <= (b.y + b.height) &&
                b.y <= (a.y + a.height));
        },
        //#endregion 
        //#region Rendering
        _createDefaultTooltip: function () {
            var div = document.createElement("div");
            var attr = {
                "id": this.element[0].id + "_DefaulttooltipDiv",
                "class": "e-diagram-tooltip-default",
                "style": "padding-top:3px; height: 24px;pointer-events:none;",
            };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            this._tooltipLayer.appendChild(div);
            if (!this.model.tooltipTemplateId) {
                var span = document.createElement("span");
                attr = { "class": "e-diagram-tooltipLabel-default", "id": "e-diagram-tooltipLabel-default" };
                ej.datavisualization.Diagram.Util.attr(span, attr);
                div.appendChild(span);
            }
            return div;
        },
        _createMouseOverTooltip: function () {
            var div = document.createElement("div");
            var attr = {
                "id": this.element[0].id + "_mouseovertooltipDiv",
                "class": "e-diagram-tooltip-mouseover",
                "style": "pointer-events:none;",
            };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            this._tooltipLayer.appendChild(div);
            return div;
        },
        _renderTooltip: function (object, start) {
            if (!(object.parent && this.nameTable[object.parent] && this.nameTable[object.parent].container)) {
                if (this.model.selectedItems.tooltip) {
                    var tooltipDiv = document.getElementById(this.element[0].id + "_DefaulttooltipDiv");
                    if (!tooltipDiv) {
                        tooltipDiv = this._createDefaultTooltip();
                    }
                    var template;
                    var objectValues;
                    var html;
                    var width;
                    var curZoomfactor = this._currZoom;
                    if (!this.model.selectedItems.tooltip.templateId) {
                        if (start) tooltipDiv.style.width = "150px";
                        width = 150;
                        var span = tooltipDiv.childNodes[0];
                        if (this.activeTool instanceof ej.datavisualization.Diagram.MoveTool) {
                            if (!object.segments)
                                objectValues = "X : " + Math.round((object.offsetX - object.width * object.pivot.x)) + "px  " + "Y : " + Math.round((object.offsetY - object.height * object.pivot.y)) + "px";
                        } else if (this.activeTool instanceof ej.datavisualization.Diagram.ResizeTool) {
                            objectValues = "W : " + Math.round(object.width) + "px  H : " + Math.round(object.height) + "px";
                        } else if (this.activeTool instanceof ej.datavisualization.Diagram.RotateTool) {
                            if (start) tooltipDiv.style.width = "50px";
                            width = 50;
                            objectValues = Math.round(object.rotateAngle) + "\xB0";
                        }
                        if (objectValues) {
                            var obj = this._getTooltipPosition(object, this.model.selectedItems.tooltip);
                            obj.x = -this._hScrollOffset + obj.x * curZoomfactor;
                            obj.y = -this._vScrollOffset + obj.y * curZoomfactor;
                            tooltipDiv.style.top = obj.y + "px";
                            tooltipDiv.style.left = obj.x + "px";
                            tooltipDiv.style.transform = "translate(" + obj.tx + " ," + obj.ty + ")";
                            span.textContent = objectValues;
                        } else {
                            this._removeTooltip();
                        }
                    } else if ($.templates && this.model.selectedItems.tooltip) {
                        if (!object.segments) {
                            tooltipDiv.style.top = -this._vScrollOffset + ((object.offsetY * curZoomfactor - (object.height / 2) * curZoomfactor) + (object.height * curZoomfactor) + 15) + "px";
                            tooltipDiv.style.left = -this._hScrollOffset + ((object.offsetX * curZoomfactor - (object.width / 2) * curZoomfactor)) + (object.width / 2 * curZoomfactor - $(tooltipDiv).width() / 2) + "px";
                            template = $.templates("#" + this.model.selectedItems.tooltip.templateId);
                            objectValues = [{ name: this.activeTool.selectedObject.name, x: (object.offsetX - object.width / 2) + "px", y: (object.offsetY - object.height / 2) + "px", width: object.width + "px", height: object.height + "px", rotationAngle: object.rotateAngle }];
                            html = template.render(objectValues);
                            $(tooltipDiv).html(html);
                        }
                    }
                }
            }
        },
        _removeTooltip: function () {
            var tooltipDiv = document.getElementById(this.element[0].id + "_DefaulttooltipDiv");
            if (tooltipDiv) {
                tooltipDiv.parentNode.removeChild(tooltipDiv);
            }
        },
        _removeMouseOverTooltip: function () {
            var tooltipDiv = document.getElementById(this.element[0].id + "_mouseovertooltipDiv");
            if (tooltipDiv) {
                tooltipDiv.parentNode.removeChild(tooltipDiv);
            }
        },
        _renderCanvas: function (isLoad) {
            this._browserInfo = ej.browserInfo();
            this._canvas = document.getElementById(this.element[0].id + "_canvas");
            if (!this._canvas) {
                this._canvas = document.createElement("div");
                this.element.append(this._canvas);
            }
            this._canvas.setAttribute("id", this.element[0].id + "_canvas");
            this._canvas.setAttribute("class", "drawing");
            this._canvas.setAttribute("style", "position:relative; height:" + this.model.height + "; width:" + this.model.width + ";");
            this.element.css({
                overflow: "hidden", outline: "none", display: "block", height: this.model.height, width: this.model.width
            });
            this._views[this._id].canvas = this._canvas;
            this.element[0].setAttribute("tabindex", 0);

            this._renderDiagram(isLoad);
        },
        _getDigramBounds: function (mode) {
            var left, top, right, bottom;
            left = this._spatialSearch.pageLeft || 0;
            top = this._spatialSearch.pageTop || 0;
            right = this._spatialSearch.pageRight - this._spatialSearch.pageLeft || 0;
            bottom = this._spatialSearch.pageBottom - this._spatialSearch.pageTop || 0;
            if (mode != "content") {
                if (this.model.pageSettings && this._multiplePage()) {
                    left = this._spatialSearch.pageLeft;
                    top = this._spatialSearch.pageTop;
                    if (this._pageWidth()) {
                        left = Math.floor(left / this._pageWidth()) * this._pageWidth();
                        right = Math.ceil(this._spatialSearch.pageRight / this._pageWidth()) * this._pageWidth() - left;
                    }
                    if (this._pageHeight()) {
                        top = Math.floor(top / this._pageHeight()) * this._pageHeight();
                        bottom = Math.ceil(this._spatialSearch.pageBottom / this._pageHeight()) * this._pageHeight() - top;
                    }
                    if (this._spatialSearch.pageRight === null) {
                        if (this._pageWidth() !== 0) {
                            right = this._pageWidth();
                        }
                    }
                    if (this._spatialSearch.pageBottom === null) {
                        if (this._pageHeight() !== 0) {
                            bottom = this._pageHeight();
                        }
                    }
                }
                else {
                    if (this._pageWidth()) {
                        left = 0;
                        right = this._pageWidth();
                    }
                    if (this._pageHeight()) {
                        top = 0;
                        bottom = this._pageHeight();
                    }
                }
            }
            var svgBounds = new ej.datavisualization.Diagram.Rectangle();
            svgBounds.x = left;
            svgBounds.y = top;
            svgBounds.width = right;
            svgBounds.height = bottom;
            return svgBounds;
        },
        _initContextMenu: function (isLoad) {
            if (this.enableContextMenu()) {
                this._renderContextMenu(isLoad);
            }
            else {
                $("#" + this.element[0].id + "_contextMenu").remove();
            }
        },
        _renderContextMenuItem: function (item) {
            var li = document.createElement("li");
            var a = document.createElement("a");
            var menu = ej.datavisualization.Diagram.Locale[this.locale()];
            a.innerHTML = menu[item.name] || item.text;
            li.setAttribute("id", this.element[0].id + "_contextMenu_" + item.name);
            var div = document.createElement("span");
            div.setAttribute("id", item.text + "_image");
            div.setAttribute("style", "display:table-cell; vertical-align:middle; text-align:center");
            div.setAttribute("class", "sf-d-menuitem " + item.name);
            a.appendChild(div);
            if (item.subItems && item.subItems.length > 0) {
                var subUl = document.createElement("ul");
                var i = 0, subli;
                for (i; i < item.subItems.length; i++) {
                    subli = this._renderContextMenuItem(item.subItems[i], subUl);
                    subli.setAttribute("style", "display:block;");
                    subUl.appendChild(subli);
                }
                li.appendChild(subUl);
            }
            li.appendChild(a);
            return li;
        },
        _renderContextMenu: function (isLoad) {
            if (!isLoad && !this.model.contextMenu.showCustomMenuItemsOnly) {
                var additionalItems = [];
                for (var i = 0; i < this._defaultContextMenuItems.length; i++) {
                    var isExist = false;
                    if (this.model.contextMenu.items.length) {
                        for (var j = 0; j < this.model.contextMenu.items.length; j++) {
                            if (this.model.contextMenu.items[j].name === this._defaultContextMenuItems[i].name) {
                                isExist = true;
                                break;
                            }
                            if (j === this.model.contextMenu.items.length - 1 && !isExist) {
                                additionalItems.push(this._defaultContextMenuItems[i])
                            }
                        }
                    }
                    else
                        additionalItems.push(this._defaultContextMenuItems[i])
                }
                for (var i = 0; i < additionalItems.length; i++) {
                    this.model.contextMenu.items.push(additionalItems[i]);
                }
            }
            var menuCollection = this.model.contextMenu.items;
            var i = 0, item, mItem;
            var ul = document.createElement("ul");
            ul.setAttribute("id", this.element[0].id + "_contextMenu");
            for (i = 0; i < menuCollection.length; i++) {
                item = menuCollection[i];
                if (item && item.name) {
                    mItem = this._renderContextMenuItem(item, ul);
                    ul.appendChild(mItem);
                }
            }
            this._canvas.appendChild(ul);
            $("#" + this.element[0].id + "_contextMenu").ejMenu({
                menuType: ej.MenuType.ContextMenu,
                contextMenuTarget: "#" + this.element[0].id + "_canvas",
                click: $.proxy(this._onMenuItemClick, this),
                beforeOpen: $.proxy(this._onContextMenuOpen, this),
                mouseover: $.proxy(this._onMenuItemMouseOver, this),
                mouseout: $.proxy(this._onMenuItemMouseOut, this),
                close: $.proxy(this._onContextMenuClose, this),
            });
            $("#" + this.element[0].id + "_contextMenu")[0].style.display = "none";
        },
        _onContextMenuClose: function () {
            this.element[0].focus();
        },
        _onContextMenuOpen: function (args) {
            var _id = this.element[0].id;
            var node = this._findNodeUnderMouse(args);
            var isMulSel = false;
            if ($(args.target).parents(".e-scrollbar").length) {
                args.cancel = true;
                $("#" + this.element[0].id + "_contextMenu").ejMenu("instance").hideContextMenu();
            }
            if (this.selectionList[0] && (this.selectionList[0].type === "pseudoGroup" || this.selectionList[0]._type === "group"))
                isMulSel = true;
            if (node && (this.selectionList.length <= 0 || (this.selectionList[0] != node)) && !isMulSel) {
                args.cancel = true;
                $("#" + this.element[0].id + "_contextMenu").ejMenu("instance").hideContextMenu();
            }
            if (!node && this.selectionList.length > 0 && !isMulSel) {
                this._clearSelection();
            }
            if (this.enableContextMenu()) {
                if ($("#" + _id + "_contextMenu_cut")[0])
                    $("#" + _id + "_contextMenu_cut")[0].style.display = this.selectionList.length ? "block" : "none";
                if ($("#" + _id + "_contextMenu_copy")[0])
                    $("#" + _id + "_contextMenu_copy")[0].style.display = this.selectionList.length ? "block" : "none";
                if ($("#" + _id + "_contextMenu_paste")[0])
                    $("#" + _id + "_contextMenu_paste")[0].style.display = this._clipboardData ? "block" : "none";
                if ($("#" + _id + "_contextMenu_undo")[0])
                    $("#" + _id + "_contextMenu_undo")[0].style.display = this._historyList && this._historyList.canUndo ? "block" : "none";
                if ($("#" + _id + "_contextMenu_redo")[0])
                    $("#" + _id + "_contextMenu_redo")[0].style.display = this._historyList && this._historyList.canRedo ? "block" : "none";
                if ($("#" + _id + "_contextMenu_selectAll")[0])
                    $("#" + _id + "_contextMenu_selectAll")[0].style.display = this.nodes().length + this.connectors().length ? "block" : "none";
                if ($("#" + _id + "_contextMenu_grouping")[0])
                    $("#" + _id + "_contextMenu_grouping")[0].style.display = this.selectionList[0] && (this.selectionList[0].type == "group" ||
                        this.selectionList[0].type == "pseudoGroup") ? "block" : "none";
                if ($("#" + _id + "_contextMenu_order")[0])
                    $("#" + _id + "_contextMenu_order")[0].style.display = this.selectionList.length ? "block" : "none";
            }
            var obj = ({ diagram: this, contextmenu: args });
            this._trigger("contextMenuBeforeOpen", obj);
        },
        _onMenuItemClick: function (args) {
            var menuId = args.events.ID.replace(this._id + "_" + "contextMenu_", "");
            switch (menuId) {
                case "cut":
                    this.cut();
                    break;
                case "copy":
                    this.copy();
                    break;
                case "paste":
                    this.paste();
                    break;
                case "undo":
                    this.undo();
                    break;
                case "redo":
                    this.redo();
                    break;
                case "selectAll":
                    this.selectAll();
                    break;
                case "group":
                    this.group();
                    break;
                case "ungroup":
                    this.ungroup();
                    break;
                case "bringToFront":
                    this.bringToFront();
                    break;
                case "sendToBack":
                    this.sendToBack();
                    break;
                case "moveForward":
                    this.moveForward();
                    break;
                case "sendBackward":
                    this.sendBackward();
                    break;
                default:
                    break;
            }
            this._trigger("contextMenuClick", args);
        },
        _onMenuItemMouseOver: function (args) {
            var t = args.events.element.lastChild.childNodes[1];
            $(t).addClass("hover");
        },
        _onMenuItemMouseOut: function (args) {
            var t = args.events.element.lastChild.childNodes[1];
            $(t).removeClass("hover");
        },
        _renderDiagram: function (isload) {
            var diagram = this;
            this._viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            this._views.forEach(function (viewid) {
                var view = diagram._views[viewid];
                if (view.type == "mainview" && view.context == ej.datavisualization.Diagram.SvgContext) {
                    view.context._renderDocument(view, diagram, isload);
                    diagram._createScrollbar(diagram._canvas);
                    ej.datavisualization.Diagram.ScrollUtil._initScrollbar(diagram);
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
                }
                //if (diagram.model.layout && diagram.model.layout.type !== ej.datavisualization.Diagram.LayoutTypes.None) {
                //    diagram._doLayout();
                //};
                if (!isload) {
                    var nodes = diagram._setNodeZOrder(view);
                    diagram._renderDiagramObjects(nodes, view);
                    diagram._renderphases(view);
                }
            });
            $(this._svg.document).pinchDiagram(this._view, this);
        },
        _createScrollbar: function (canvas) {
            var hScrollbar = document.createElement("div");
            hScrollbar.setAttribute("id", canvas.id + "_hScrollbar");
            if (!document.getElementById(canvas.id + "_hScrollbar"))
                canvas.appendChild(hScrollbar);
            var vScrollbar = document.createElement("div");
            vScrollbar.setAttribute("id", canvas.id + "_vScrollbar");
            if (!document.getElementById(canvas.id + "_vScrollbar"))
                canvas.appendChild(vScrollbar);
        },
        _doLayout: function () {
            if (this.model.layout && this.model.layout.type !== ej.datavisualization.Diagram.LayoutTypes.None) {
                //this.model.layout = new ej.datavisualization.Diagram.HierarchicalLayout(this.model.layout);
                //this.model.layout._setModel(this);
                this._updateEdgeCollection();
                ej.datavisualization.Diagram.Layout.doLayout(this);
                //this.model.layout.doLayout(this);
            }
        },
        _updateNodes: function () {
            var nodes = this.nodes();
            for (var i = 0; i < nodes.length; i++) {
                ej.datavisualization.Diagram.DiagramContext.update(nodes[i], this);
            }
        },
        _updateConnectors: function () {
            var connections = this.connectors();
            for (var i = 0; i < connections.length; i++) {
                ej.datavisualization.Diagram.DiagramContext.update(connections[i], this);
            }
        },
        _updateEdgeCollection: function () {
            for (var i = 0; i < this.connectors().length; i++) {
                this._updateEdges(this.connectors()[i]);
            }
        },
        updateViewPort: function () {
            this._viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            ej.datavisualization.Diagram.DiagramContext.updateViewPort(this);
            this.model.scrollSettings.viewPortHeight = this._viewPort.height;
            this.model.scrollSettings.viewPortWidth = this._viewPort.width;
        },
        _containsSameIndex: function (index, parent) {
            if (!parent) {
                var nodes = this.nodes(), cNode;
                for (var i = 0; i < nodes.length > 0; i++) {
                    cNode = nodes[i];
                    if (cNode && cNode.zOrder === index)
                        return true;
                }
            }
            else {
                var children = parent.children, child;
                for (var i = 0; i < children.length; i++) {
                    child = this.nameTable[this._getChild(children[i])];
                    if (child && child.zOrder) {
                        if (child.zOrder === index)
                            return true;
                    }
                }
            }
            return false;
        },
        _setZorder: function (node) {
            if (node && node.zOrder) {
                if (node.zOrder == -1) {
                    if (this._containsSameIndex(this._zOrder)) {
                        do {
                            this._zOrder++;
                        } while (this._containsSameIndex(this._zOrder));
                        node.zOrder = this._zOrder;
                    }
                    else {
                        node.zOrder = this._zOrder;
                        this._zOrder++;
                    }
                }
                else {
                }
                if (node._type === "group") {
                    var children = node.children, child;
                    for (var i = 0; i < children.length; i++) {
                        child = this.nameTable[this._getChild(children[i])];
                        if (child && child.zOrder && child.zOrder === -1) {
                            var inx = i;
                            if (this._containsSameIndex(i, node)) {
                                do {
                                    inx++;
                                } while (this._containsSameIndex(inx));
                                child.zOrder = inx;
                            }
                            else {
                                child.zOrder = i;
                            }
                        }
                    }
                }
            }
        },
        _union: function (rect1, rect2) {
            var x = Math.min(rect1.x, rect2.x);
            var y = Math.min(rect1.y, rect2.y);
            var overWidth = Math.max(rect1.x + rect1.width, rect2.x + rect2.width);
            var overheight = Math.max(rect1.y + rect1.height, rect2.y + rect2.height);
            return new ej.datavisualization.Diagram.Rectangle(x, y, overWidth - x, overheight - y);
        },
        _getSeperetor: function (id) {
            return this.nameTable[id];
        },
        _updatePhase: function (group) {
            if (group && group.phases && group.phases.length > 0) {
                var lIndex = group.phases.length;
                var defPhase = this.nameTable[group.phases[lIndex - 1]];
                if (defPhase) {
                    if (group.orientation == "vertical") {
                        var dy = group.height - defPhase.offset;
                        defPhase.offset = group.height;
                        var phaseNode = this.nameTable["phaseStack" + defPhase.name];
                        if (phaseNode) {
                            this.updateNode(phaseNode.name, { height: phaseNode.height + dy, offsetY: phaseNode.offsetY + (dy / 2) });
                        }
                    }
                    else {
                        defPhase.offset = group.width;
                    }
                }
            }
        },
        _renderphases: function () {
            var phases = this.model.phases;
            if (phases) {
                for (var i = 0, len = phases.length; i < len; i++) {
                    if (phases[i].type == "phase") {
                        ej.datavisualization.Diagram.SvgContext.renderphase(phases[i], this._svg, this._diagramLayer, this);
                    }
                }
            }
        },
        _setNodeZOrder: function (view) {
            var temp = null, jindex, iindex;
            var nodes = this.nodes();
            nodes = nodes.concat(this.connectors());
            for (var f = 0; f < nodes.length; f++) {
                if (nodes[f].parent === "")
                    this._setZorder(nodes[f]);
            }
            for (var i = 0; i < nodes.length; i++) {
                for (var j = i + 1; j < nodes.length; j++) {
                    if (nodes[i].zOrder > nodes[j].zOrder && (nodes[i].parent === "" && nodes[j].parent === "")) {
                        temp = nodes[i];
                        nodes[i] = nodes[j];
                        nodes[j] = temp;
                    }
                }
                if (nodes[i]._type === "group") {
                    var children = nodes[i].children;
                    if (!nodes[i].isSwimlane) {
                        for (var m = 0; m < children.length; m++) {
                            for (var n = m + 1; n < children.length; n++) {
                                var child1 = typeof (children[m]) === "object" ? children[m] : this.nameTable[children[m]];
                                var child2 = typeof (children[n]) === "object" ? children[n] : this.nameTable[children[n]];
                                if (child1.zOrder > child2.zOrder && !child1.isPhase && !child2.isPhase && !child1.isLane && !child2.isLane) {
                                    temp = children[m];
                                    children[m] = children[n];
                                    children[n] = temp;
                                }
                            }
                        }
                    }
                }
            }
            return nodes;
        },
        _renderDiagramObjects: function (nodes, view, isLoad, isoverView) {
            for (var i = 0, len = nodes.length; i < len; ++i) {
                if (this.getObjectType(nodes[i]) === "node" || this.getObjectType(nodes[i]) === "group") {
                    var node = this.nameTable[nodes[i].name];
                    ej.datavisualization.Diagram.DiagramContext._renderNodeObject(node, view, isLoad, isoverView, this)
                }
                else {
                    ej.datavisualization.Diagram.DiagramContext._renderConnectorObject(nodes[i], view, this);
                }
            }
        },

        _unWireEvents: function () {           
            var svgDocument = $(this._canvas);
            this._off(svgDocument, ej.eventType.mouseDown, this._mousedown);
            this._off(svgDocument, ej.eventType.mouseMove, this._mousemove);
            this._off(svgDocument, ej.eventType.mouseUp, this._mouseup);
            this._off(svgDocument, ej.eventType.mouseLeave, this._documentmouseup);
            this._off(svgDocument, "dblclick", this._doubleclick);
            this._off(svgDocument, "doubletap", this._doubleclick);
            this._off(this.element, "keydown", this._keydown);
            this._off(this.element, "keyup", this._keyup);
            //this._on(this.element, "contextmenu", this._contextmenu);
            this._off(svgDocument, "pinchin", this._pinchin);
            this._off(svgDocument, "pinchout", this._pinchout);
            this._off(this.element, "touchstart", this._handleTouchStart);
            if (ej.browserInfo().name === "mozilla") {
                this._off(this.element, "DOMMouseScroll", this._handleMouseWheel);
            }
            else
                this._off(this.element, "mousewheel", this._handleMouseWheel);
            //window document event
            this._off($(document), "mouseup", this._documentmouseup);

            if (window) {
                this._off($(window), "resize", this.updateViewPort);
            }
        },

        //#endregion
        //#region Events
        _wireEvents: function () {
            //svg document events
            var svgDocument = $(this._canvas);
            this._on(svgDocument, ej.eventType.mouseDown, this._mousedown);
            this._on(svgDocument, ej.eventType.mouseMove, this._mousemove);
            this._on(svgDocument, ej.eventType.mouseUp, this._mouseup);
            this._on(svgDocument, ej.eventType.mouseLeave, this._documentmouseup);
            this._on(svgDocument, "dblclick", this._doubleclick);
            this._on(svgDocument, "doubletap", this._doubleclick);
            this._on(this.element, "keydown", this._keydown);
            this._on(this.element, "keyup", this._keyup);
            //this._on(this.element, "contextmenu", this._contextmenu);
            this._on(svgDocument, "pinchin", this._pinchin);
            this._on(svgDocument, "pinchout", this._pinchout);
            this._on(this.element, "touchstart", this._handleTouchStart);
            if (ej.browserInfo().name === "mozilla") {
                this._on(this.element, "DOMMouseScroll", this._handleMouseWheel);
            }
            else
                this._on(this.element, "mousewheel", this._handleMouseWheel);
            //window document event
            this._on($(document), "mouseup", this._documentmouseup);

            if (window) {
                this._on($(window), "resize", this.updateViewPort);
            }
            var obj = this;
            $(svgDocument).ejDroppable({
                drop: function (event, ui) {
                    obj._svgdrop(event, ui);
                    obj._isDropped = true;
                },
                mouseover: function (event, ui) {
                    if ((obj.activeTool.inAction === false) && obj._selectedSymbol) {
                        obj._addSymbolToDiagram(event);
                    }
                },
            });
        },
        _mousePosition: function (evt, exOffset) {
            var e = this._isTouchEvent(evt);
            var scrollLeft = this._scrollLeft !== undefined ? this._scrollLeft : this._canvas.scrollLeft;
            var scrollTop = this._scrollTop !== undefined ? this._scrollTop : this._canvas.scrollTop;
            var controlBBox = this._controlBBox ? this._controlBBox : this._canvas.getBoundingClientRect();
            var layerx = (e.clientX + scrollLeft) - controlBBox.left;
            var layery = (scrollTop + e.clientY) - controlBBox.top;
            if (!exOffset) {
                layerx = (layerx + this._hScrollOffset) / this._currZoom;
                layery = (layery + this._vScrollOffset) / this._currZoom;
            }
            return new ej.datavisualization.Diagram.Point(Math.round(layerx * 100) / 100, Math.round(layery * 100) / 100);
        },
        _contextmenu: function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
        },
        _pinchin: function (evt) {
            evt.preventDefault();
        },
        _pinchout: function (evt) {
            evt.preventDefault();
        },
        _handleTouchStart: function (evt) {
            evt.preventDefault();
        },
        _mousedown: function (evt) {
            this._isPinching = false;
            this._viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            if (this._invoke(evt)) {
                this._scrollLeft = this._canvas.scrollLeft;
                this._scrollTop = this._canvas.scrollTop;
                this._controlBBox = this._canvas.getBoundingClientRect();
                var position = this._mousePosition(evt);
                if (!this._isForeignObject(evt.originalEvent.target))
                    evt.preventDefault();
                if (this.model.contextMenu && $("#" + this.element[0].id + "_contextMenu").css("display") !== "none") {
                    evt.cancel = true;
                }
                if (this._isEditing) {
                    this._endEdit();
                    if (this.activeTool.name === "textTool") {
                        if (this.activeTool.singleAction) {
                            this.activeTool.diagram.deactivateTool();
                        }
                    }
                }
                if ((evt.which === 1 || evt.which === 3) || (evt.which === 0 && evt.originalEvent.changedTouches)) {
                    this._removeMouseOverTooltip();
                    if (evt.which === 0 && evt.originalEvent.changedTouches) {
                        var node = null;
                        var isUserHandle = this._isUserHandle(evt);
                        if ((this._isInternalTool(this.activeTool) && !this.activeTool.inAction)) {
                            if (!isUserHandle) {
                                node = this._findNodeUnderMouse(evt);
                                this._raiseMouseEvents(node, position);
                            }
                            if (!node && this.selectionList[0]) {
                                node = this.selectionList[0];
                            }
                            this._checkToolToActivate(evt, node);
                        }
                    }
                    if ((navigator.platform.match("Mac") ? evt.metaKey : evt.ctrlKey) && evt.shiftKey) {
                        node = this._findNodeUnderMouse(evt);
                        if (!node && this.selectionList[0]) {
                            node = this.selectionList[0];
                        }
                        this._checkToolToActivate(evt, node);
                    }
                    if (this._toolToActivate === "panTool") {
                        this.activeTool = this.tools[this._toolToActivate];
                    }
                    else {
                        if (this._toolToActivate === "move") {
                            node = this._findNodeUnderMouse(evt);
                            if (node && node.isLane && ej.datavisualization.Diagram.Util.canMultiSelectOnLane(node)) {
                                this._toolToActivate = "select";
                                this._currentCursor = "default";
                                this._updateCursor();
                            }
                        }
                        var tool = this.tools[this._toolToActivate] || this.activeTool;
                        this.activateTool(this._toolToActivate, tool ? tool.singleAction : null);
                    }
                    this.activeTool.mousedown(evt);
                    this._focusedElement = this.activeTool._findNodeUnderMouse(evt, true);
                }
            }
        },
        _mousemove: function (evt) {
            this._mouseEventTriggered = true;
            if (!this._viewPort) this._viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this, true);
            if (evt.type == "touchmove" && evt.originalEvent.touches.length > 1) {
                this._isPinching = true;
            } else {
                this._isPinching = false;
                var node = null;
                var isUserHandle = this._isUserHandle(evt);
                if ((this.activeTool.inAction === false) && this._selectedSymbol) {
                    this._addSymbolToDiagram(evt);
                } else {
                    if ((this._isInternalTool(this.activeTool) && !this.activeTool.inAction)) {
                        if (!isUserHandle) {
                            var node = this._findNodeUnderMouse(evt);
                            if (node) {
                                if (this._nodeUnderMouse && this._nodeUnderMouse != node && !this._nodeUnderMouse.segments) {
                                    this.activeTool._showPort(this._nodeUnderMouse, true);
                                }
                                if (!node.segments) {
                                    this.activeTool._showPort(node);
                                }
                            }
                            //this._raiseMouseEvents(node);
                        }
                        if (!node && this.selectionList[0]) {
                            node = this.selectionList[0];
                        }
                        if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                            this._checkToolToActivate(evt, node);
                        }
                        if (evt.which == 1 && evt.buttons == 1)
                            this._focusedElement = this.activeTool._findNodeUnderMouse(evt, true);
                    } else {
                        if (!this.activeTool.inAction) {
                            this._toolToActivate = this.activeTool.name;
                            this._currentCursor = this.activeTool.cursor;
                        }
                        nodeUnderMouse = this._findNodeUnderMouse(evt);
                        if (node) {
                            //this._raiseMouseEvents(node);
                        }
                    }
                    this._updateCursor();
                    var nodeUnderMouse = nodeUnderMouse || this._findNodeUnderMouse(evt);
                    this.activeTool.mousemove(evt);
                    var position = this._mousePosition(evt);
                    this._raiseMouseEvents(nodeUnderMouse, position);
                    var canAvoidConnection = false;
                    if (this.activeTool.inAction) {
                        do {
                            if (nodeUnderMouse && !ej.datavisualization.Diagram.Util.canAllowDrop(nodeUnderMouse)) {
                                if (nodeUnderMouse.parent) nodeUnderMouse = this.nameTable[nodeUnderMouse.parent];
                                else if (nodeUnderMouse.segments) { canAvoidConnection = true; break; } else nodeUnderMouse = null;
                            }
                            else break;
                        } while (nodeUnderMouse);
                        if (this._nodeUnderMouse && this._nodeUnderMouse.segments && this._nodeUnderMouse != nodeUnderMouse)
                            ej.datavisualization.Diagram.SvgContext._removeConnectorHighlighter(this._adornerLayer, this._adornerSvg);
                        if (nodeUnderMouse && ((nodeUnderMouse != this._focusedElement && nodeUnderMouse != this.nodeUnderMouse) &&
                            (canAvoidConnection || ej.datavisualization.Diagram.Util.canAllowDrop(nodeUnderMouse)))) {
                            var type = this.getObjectType(nodeUnderMouse);
                            if (!((this._focusedElement && (this._focusedElement.isLane || this._focusedElement.isSwimlane || this._focusedElement.isPhase)) && type == "node" && type == "connector")) {
                                if (!((this._nodeUnderMouse && (this._nodeUnderMouse.isLane || this._nodeUnderMouse.isSwimlane || this._nodeUnderMouse.isPhase))
                                    && nodeUnderMouse.segments)) {
                                    this._nodeUnderMouse = nodeUnderMouse;
                                }
                            } else this._nodeUnderMouse = null;
                        } else this._nodeUnderMouse = null;
                    } else if (!this._focusedElement) this._nodeUnderMouse = nodeUnderMouse;
                    this._initiateAutoScroll(evt);
                }
            }
        },
        _mouseup: function (evt) {
            if (!this._isPinching && evt.originalEvent && evt.originalEvent.target) {
                if (this._invoke(evt)) {
                    var foreignObject = this._isForeignObject(evt.originalEvent.target);
                    if (!foreignObject)
                        evt.preventDefault();
                    if (this.activeTool instanceof ej.datavisualization.Diagram.PanTool) {
                        document.onmousemove = null;
                        document.onmouseup = null;
                        if ((!ej.datavisualization.Diagram.Util.isPageEditable(this) || this.activeTool instanceof ej.datavisualization.Diagram.PanTool) && !this.activeTool.inAction) {
                            var obj = this.activeTool._findNodeUnderMouse(evt);
                            if (obj)
                                this._raiseEvent("itemClick", { actualObject: obj, selectedObject: obj, model: this.model, cancel: false, event: evt });
                            this._raiseEvent("click", { element: obj, event: evt });
                            if (this.activeTool instanceof ej.datavisualization.Diagram.PanTool) {
                                document.onmousemove = null;
                                document.onmouseup = null;
                            }
                        }
                    } else if (evt.target.id === this._id + "_canvas_svg" && (!this.activeTool.name == "panTool" || !this.activeTool.inAction))
                        this._raiseEvent("click", { element: this, count: evt.originalEvent.detail, event: evt });

                    if (this._selectedSymbol && !this._selectedSymbol.isLane) {
                        this._dropSymbol();
                        this._isDragg = true;
                    }
                    this.activeTool.mouseup(evt);
                    if ((!(this.activeTool instanceof ej.datavisualization.Diagram.TextTool)) && document.activeElement && document.activeElement.id !== this.element[0].id)
                        if (!this._isEditing && !foreignObject)
                            this.element[0].focus();
                    this._currentCursor = this.activeTool.cursor;
                    this._updateCursor();
                }
            }
            this._focusedElement = null;
            this._nodeUnderMouse = this._findNodeUnderMouse(evt);
            delete this._scrollLeft;
            delete this._scrollTop;
            delete this._controlBBox;
            this._mouseEventTriggered = false;
            this._isDragg = false;
            delete this._viewPort;
        },
        _doubleclick: function (evt) {
            if (this.activeTool.name != "panTool") {
                var args;
                args = { element: this.getNode(this._nodeUnderMouse), cancel: false, actualObject: this.getNode(this.activeTool.actualObject) }
                if (!this._isEditing)
                    this._raiseEvent("doubleClick", args);
                if (this._nodeUnderMouse)
                    this.activeTool._doubleClick(this._nodeUnderMouse);
                if (!this.activeTool.inAction && ((this.selectionList.length > 0) || this.activeTool.selectedObject) && !args.cancel) {

                    if (this.activeTool.selectedObject)
                        var obj = this.activeTool.selectedObject;
                    else
                        var obj = this.selectionList[0];
                    if (!this._isEditing && !(obj.type == "pseudoGroup") && !args.cancel) {
                        this.activeTool._findLabelUnderMouse(evt, false, this._mousePosition(evt.originalEvent));
                        if (obj.labels && obj.labels.length > 0) {
                            var label = this.activeTool.getLabelUnderMouse();
                            if (!label) label = obj.labels[0];
                        }
                        if (!this._isEditing && obj.type === "text" && this._setLabelEditing(obj.textBlock)) {
                            this.scrollToNode(obj);
                            this._startEdit(obj);
                        }
                        else if (!this._isEditing && !obj.isLane && !obj.isSwimlane && (obj.labels && (obj.labels.length === 0 || typeof obj.labels.length === "undefined") || (obj.labels && obj.labels.length > 0 && this._setLabelEditing(label)))) {
                            this.scrollToNode(obj);
                            this._startEdit(obj);
                        }
                        else if (obj.isLane && (obj.labels && obj.labels.length > 0 && this._setLabelEditing(label))) {
                            if (obj.labels && obj.labels.length > 0) {
                                this.scrollToNode(obj);
                                this._startEdit(obj);
                            }
                            else
                                this.activeTool.selectedObject = null;
                        }
                    }
                }
            }
            this.activeTool._isMouseDown = false;
        },
        _documentmouseup: function (evt) {
            var selectedSymbol = this._selectedSymbol;
            this._removeMouseOverTooltip();
            if (this.activeTool && this.activeTool.inAction && !(this.activeTool instanceof ej.datavisualization.Diagram.TextTool)) {
                this.activeTool.mouseup(evt, true);
            }
            if (selectedSymbol) {
                if (evt.keyCode != 27) {
                    var preview = document.getElementsByClassName("e-paletteItem dragClone")[0];
                    if (preview != null) {
                        preview.style.display = "block";
                    }
                    this._selectedSymbol = selectedSymbol;
                }
                this._raiseEvent("dragLeave", { element: this.selectionList[0] });
                if (this._mouseEventTriggered)
                    this._remove(this.selectionList[0]);
            }
            this._mouseEventTriggered = false;
            this._isDragg = false;
        },
        _invoke: function (evt) {
            if (evt.originalEvent.target) {
                var className = evt.originalEvent.target.className;
                if (className && typeof className === "string" && (className == "edit" || className == "editBox" || className.indexOf("e-hhandle") > -1 || className.indexOf("e-hhandlespace") > -1 || className.indexOf("e-vhandle") > -1 || className.indexOf("e-vhandlespace") > -1)) {
                    return false;
                }
            }
            return true;
        },
        _handleMouseWheel: function (evt) {
            if (evt.originalEvent.preventDefault)
                evt.originalEvent.preventDefault();
            var position = new ej.datavisualization.Diagram.Point(0, 0);
            //this.deactivateTool();
            if ((navigator.platform.match("Mac") ? evt.metaKey : evt.ctrlKey)) {
                position = this._mousePosition(evt.originalEvent);
                if (evt.originalEvent.wheelDelta > 0 || -40 * evt.originalEvent.detail > 0) {
                    ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 + Number(this.zoomFactor())), 0, 0, position, true);
                } else {
                    ej.datavisualization.Diagram.ZoomUtil.zoomPan(this, (1 / (1 + Number(this.zoomFactor()))), 0, 0, position, true);
                }
            } else {
                var offsetY = 0;
                var oVal = {
                    zoom: this._getCurrentZoom(), horizontalOffset: this._hScrollOffset,
                    verticalOffset: this._vScrollOffset, viewPort: this._viewPort
                };
                var scrolled;
                if (evt.originalEvent.wheelDelta > 0 || -40 * evt.originalEvent.detail > 0) {
                    var top = this._vScrollbar.model.minimum;

                    var limited = this._scrollLimit() === "limited" && (this._vScrollOffset - this._scrollPixel) >= this.model.pageSettings.scrollableArea.y * this._currZoom;
                    if ((this._vScrollOffset - this._scrollPixel) >= top || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity || limited) {
                        this._updateScrollOffset(this._hScrollOffset, (this._vScrollOffset - this._scrollPixel));
                        scrolled = true;
                        offsetY = -this._scrollPixel;
                    } else {
                        this._updateScrollOffset(this._hScrollOffset, top);
                        scrolled = true;
                        offsetY = 0;
                    }
                } else {
                    var cHeight = this._vScrollbar.model.maximum;
                    var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
                    limited = this._scrollLimit() === "limited" &&
                       (this.model.pageSettings.scrollableArea.y + this.model.pageSettings.scrollableArea.height) * this._currZoom >=
                       (this._vScrollOffset + this._scrollPixel + viewPort.height);
                    if ((cHeight) >= (this._vScrollOffset + this._scrollPixel) || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity || limited) {
                        this._updateScrollOffset(this._hScrollOffset, (this._vScrollOffset + this._scrollPixel));
                        scrolled = true;
                        offsetY = this._scrollPixel;
                    } else {
                        this._updateScrollOffset(this._hScrollOffset, cHeight);
                        scrolled = true;
                        offsetY = 0;
                    }
                }
                if (scrolled) {
                    var nVal = {
                        zoom: this._getCurrentZoom(), horizontalOffset: this._hScrollOffset,
                        verticalOffset: this._vScrollOffset, viewPort: this._viewPort
                    };
                    this._raiseEvent("scrollChange", { newValues: nVal, oldValues: oVal });
                }
                this._updateEditBox(0, offsetY);
            }
        },
        _setActiveTool: function (evt) {
            var point = this._mousePosition(evt);
            if (!this.activeTool.inAction && (this._isUserHandleActive(this.activeTool.name) || !this.model.selectedItems.userHandles || !this.model.selectedItems.userHandles.length)) {
                if (this._isInternalTool(this.activeTool) || this.activeTool.singleAction) {
                    var node;
                    var success = false;
                    //var handle = this._isHandle(evt);
                    if (this.selectionList[0]) {
                        node = this.selectionList[0];
                        success = this._checkToolToActivate(evt, node);
                    }
                    if (!success) {
                        node = this._findNodeUnderMouse(evt);
                        if (this._isUserHandle(evt)) {
                            var handleName = $(evt.target).parents("g").first().context.id.split("_")[0];
                            this._checkToolToActivate(evt, node);
                            this.activeTool = this.tools[this._toolToActivate];
                        }
                        else {
                            this._checkToolToActivate(evt, node);
                            this._raiseMouseEvents(node);
                            if (this._isUserHandleActive(this.activateTool.name) && !this._drawingTool)
                                this.activeTool = this.tools[this._toolToActivate];
                        }
                    }
                }
            }
        },
        _beginAutoScroll: function (option, evt) {
            var delay = 100;
            var callBack = this;
            var left = 0, top = 0;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this);
            var cWidth = this._hScrollbar.model.maximum + viewPort.width;
            var cHeight = this._vScrollbar.model.maximum + viewPort.height;
            if (this._canAutoScroll) {
                switch (option) {
                    case "right":
                        if ((this._hScrollOffset + viewPort.width + 10) < cWidth || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            left = 10;
                        }
                        break;
                    case "left":
                        if (this._hScrollOffset - 10 > this._hScrollbar.model.minimum || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            left = -10;
                        }
                        break;
                    case "bottom":
                        if (this._vScrollOffset + viewPort.height + 10 < cHeight || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            top = 10;
                        }
                        break;
                    case "top":
                        if (this._vScrollOffset - 10 > this._vScrollbar.model.minimum || this._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                            top = -10;
                        }
                        break;
                }
                setTimeout(function () {
                    callBack._raiseEvent("autoScrollChange", { delay: delay })
                    callBack._updateScrollBar(left, top, option, evt);
                }, delay);
            }
        },
        _updateScrollBar: function (x, y, option, evt) {
            if (x === 0 && y === 0) {
                this._canAutoScroll = false;
            }
            else if (this.activeTool.inAction) {
                var hScrollOffset = this._hScrollOffset + (x * this._currZoom);
                var vScrollOffset = this._vScrollOffset + (y * this._currZoom);
                var isUpdated = this._updateSelectionOnScroll(x, y);
                if (isUpdated) {
                    this._updateScrollOffset(hScrollOffset, vScrollOffset);
                    this._beginAutoScroll(option, evt);
                }
                else {
                    this._canAutoScroll = false;
                }
            }
        },
        _updateSelectionOnScroll: function (x, y) {
            var tool = this.activeTool;
            if (tool.name === "select") {
                tool.currentPoint = new ej.datavisualization.Diagram.Point(tool.currentPoint.x + x, tool.currentPoint.y + y);
                tool._updateHelper();
            } else if (tool.name === "orthogonalLine" || tool.name === "straightLine") {
                tool.currentPoint = new ej.datavisualization.Diagram.Point(tool.currentPoint.x + x, tool.currentPoint.y + y);
                tool._targetPossibleConnection = tool._currentPossibleConnection;
                tool._targetPort = tool._possibleConnectionPort;
                tool._updateHelper();
            } else if (this.selectionList.length > 0) {
                var node = this.selectionList[0];
                if (!node.isLane)
                    tool.currentPoint = ej.datavisualization.Diagram.Point(tool.currentPoint.x + x, tool.currentPoint.y + y);
                if (tool.name === "move") {
                    if (!node.isLane) {
                        if (tool._canMoveLabel)
                            this._translateLabel(node, tool.activeLabel, x, y);
                        else
                            this._translate(node, x, y, this.nameTable);
                    }
                    else
                        return false;
                } else if (tool.name === "resize") {
                    ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this);
                    tool._updateSize(node, tool.previousPoint, tool.currentPoint);
                } else if (tool.name === "endPoint") {
                    if (tool.selectedSegment && tool._endPoint == "segmentEnd") {
                        var difx = tool.currentPoint.x - tool.previousPoint.x;
                        var dify = tool.currentPoint.y - tool.previousPoint.y;
                        tool._updateAdjacentSegments(difx, dify);

                    } else {
                        var cancelEvent = tool._updatePoints(node);
                        if (!cancelEvent.cancel) {
                            tool._disconnect(node);
                            tool._updateConnection(node, cancelEvent.updateSelection);
                        }
                    }
                }
                tool.previousPoint = ej.datavisualization.Diagram.Point(tool.currentPoint.x, tool.currentPoint.y);
                if (!tool._canMoveLabel && node.type != "phase") {
                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    this._updateSelectionHandle(true);
                    this._renderTooltip(node);
                }
            }
            return true;
        },
        _isUserHandle: function (evt) {
            if (evt.target.parentNode.className.animVal === "userHandle") {
                return true;
            } else {
                return false;
            }
        },
        _isUserHandleActive: function (name) {
            var userHandleActive = false;
            if (this.model.selectedItems.userHandles && this.model.selectedItems.userHandles.length > 0) {
                var i = 0;
                var length = this.model.selectedItems.userHandles.length;
                for (i; i < length; i++) {
                    if (this.model.selectedItems.userHandles[i].name === name) {
                        userHandleActive = true;
                        break;
                    } else
                        userHandleActive = true;
                }
            }
            return userHandleActive;
        },
        _raiseMouseEvents: function (node, position) {
            if (!this._isEditing) {
                if (this._nodeUnderMouse && (!node || this._nodeUnderMouse != node)) {
                    var tooltipDiv = document.getElementById(this.element[0].id + "_mouseovertooltipDiv");
                    if (tooltipDiv) {
                        tooltipDiv.parentNode.removeChild(tooltipDiv);
                    }
                    this._raiseEvent("mouseLeave", { element: this.getNode(this._nodeUnderMouse) });
                    if (this._nodeUnderMouse != null && (!this._nodeUnderMouse.segments) && !(this.activeTool instanceof ej.datavisualization.Diagram.LineTool)) {
                        this.activeTool._showPort(this._nodeUnderMouse, true);
                    }
                }
                if (node) {
                    if (this._nodeUnderMouse !== node) {
                        this._raiseEvent("mouseEnter", { element: this.getNode(node) });
                    }
                }
                if (node) {
                    var args = { element: this.getNode(node) };
                    this._raiseEvent("mouseOver", args);
                    if (args.element && !this.activeTool.inAction) {
                        this._renderMouseOverTooltip(node, position);
                    }
                    else {
                        this._removeMouseOverTooltip();
                    }
                }
            }
        },
        _renderMouseOverTooltip: function (node, position) {
            if (ej.datavisualization.Diagram.Util.canShowTooltip(node, this)) {
                var type = this.getObjectType(node);
                var tooltip;
                if (type == "node" || type == "group")
                    var Constraints = ej.datavisualization.Diagram.NodeConstraints;
                else Constraints = ej.datavisualization.Diagram.ConnectorConstraints;
                if (node.constraints & Constraints.InheritTooltip)
                    tooltip = this.model.tooltip; else tooltip = node.tooltip;
                if (tooltip) {
                    var x, y, tx, ty;
                    var obj = this._getTooltipPosition(node, tooltip, position);
                    x = obj.x; y = obj.y; tx = obj.tx; ty = obj.ty;
                    x = -this._hScrollOffset + x * this._currZoom;
                    y = -this._vScrollOffset + y * this._currZoom;
                    var style = "left:" + x + "px; top:" + y + "px; display: block; position: absolute; pointer-events: none; transform:" + "translate(" + tx + "," + ty + ")";
                    var tooltipDiv = document.getElementById(this.element[0].id + "_mouseovertooltipDiv");
                    if (!tooltipDiv) {
                        tooltipDiv = this._createMouseOverTooltip();
                    }
                    tooltipDiv.style.left = x + "px";
                    tooltipDiv.style.top = y + "px";
                    tooltipDiv.style.transform = "translate(" + tx + " ," + ty + ")";
                    tooltipDiv.style.display = "block";
                    tooltipDiv.style.position = "absolute";
                    if ($.templates) {
                        var template = $.templates("#" + tooltip.templateId).render(node);
                    }
                    $(tooltipDiv).html(template);
                }
            }
        },
        _getTooltipPosition: function (node, tooltip, position) {
            var bounds = ej.datavisualization.Diagram.Util.bounds(node, this.activeTool.name == "rotatetool" ? true : false);
            var x, y, tx, ty;
            if (tooltip.relativeMode == "object") {
                switch (tooltip.alignment.horizontal) {
                    case "left":
                        x = bounds.x;
                        x += tooltip.margin.left - tooltip.margin.right;
                        tx = "-100%";
                        break;
                    case "right":
                        x += tooltip.margin.left;
                        x = bounds.right;
                        tx = "0%";
                        break;
                    case "center":
                        x = bounds.center.x;
                        tx = "-50%";
                        break;
                }
                switch (tooltip.alignment.vertical) {
                    case "top":
                        y = bounds.y;
                        y += tooltip.margin.top - tooltip.margin.bottom;
                        ty = "-100%";
                        break;
                    case "bottom":
                        y = bounds.bottom;
                        y += tooltip.margin.top;
                        ty = "0%";
                        break;
                    case "center":
                        y = bounds.center.y;
                        ty = "-50%";
                        break;
                }
            }
            else if (position) { x = position.x + tooltip.margin.left; y = position.y + tooltip.margin.top; }
            return { x: x, y: y, tx: tx, ty: ty };
        },
        _isHandle: function (evt) {
            var handle = $(evt.target).parent("g.handle");
            if (handle.length > 0) {
                return true;
            }
            return false;
        },
        _findNodeUnderMouse: function (evt) {
            var node = null;
            var type;
            if (evt.target) {
                var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector,.ej-d-group");
                var id = evt.target.id.split('_');
                if (id[id.length - 1] == "lblbg") return;
                if (parent) {
                    type = parent.attr("class");
                    if (type === "ej-d-node" || type === "ej-d-group") {
                        node = this._findNode(parent.attr("id"));
                        if (!node) node = this._findNode(parent.attr("id").split('_parentdiv')[0]);
                        if (node && node.parent && this.nameTable[node.parent] && this.nameTable[node.parent].type == "bpmn")
                            node = this.nameTable[node.parent];
                    } else if (type === "ej-d-connector") {
                        node = this._findConnector(parent.attr("id"));
                    }
                }
            }
            return node;
        },
        _findConnectableNodeUnderMouse: function (evt) {
            var node = null;
            var type;
            var parents = $(evt.target).parents(".ej-d-node,.ej-d-group");
            var parent = parents[0];
            var i = 1;
            while (parent) {
                type = parent.getAttribute("class");
                node = this._findNode(parent.getAttribute("id"));
                if (ej.datavisualization.Diagram.Util.canConnect(node) || !node.parent) {
                    parent = null;
                } else {
                    parent = parents[i];
                    i++;
                }
            }
            return node;
        },
        _isForeignObject: function (target) {
            var foreignObj = target;
            if (foreignObj) {
                while (foreignObj.parentNode != null) {
                    if (foreignObj.tagName === "foreignObject" || (typeof foreignObj.className == "string" && foreignObj.className.split(' ').indexOf("foreignObject") !== -1)) {
                        return foreignObj;
                    } else {
                        foreignObj = foreignObj.parentNode;
                    }
                }
            }
            return null;
        },
        _altKeyPressed: function (keyModifier) {
            return keyModifier & ej.datavisualization.Diagram.KeyModifiers.Alt;
        },
        _ctrlKeyPressed: function (keyModifier) {
            return keyModifier & ej.datavisualization.Diagram.KeyModifiers.Control;
        },
        _shiftKeyPressed: function (keyModifier) {
            return keyModifier & ej.datavisualization.Diagram.KeyModifiers.Shift;
        },
        _keydown: function (evt) {
            this._removeMouseOverTooltip();
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            var ctrlKey = navigator.platform.match("Mac") ? evt.metaKey : evt.ctrlKey;
            var key = evt.key;
            for (var i in this.model.commandManager.commands) {
                var command = this.model.commandManager.commands[i];
                if (command && command.gesture) {
                    if (command.gesture.keyModifiers || command.gesture.key) {
                        if ((!command.gesture.key || (keycode == command.gesture.key || key == command.gesture.key))
                            && (!command.gesture.keyModifiers || ((ctrlKey || evt.altKey || evt.shiftKey) &&
                            (!this._altKeyPressed(command.gesture.keyModifiers) || evt.altKey) &&
                            (!this._shiftKeyPressed(command.gesture.keyModifiers) || evt.shiftKey) &&
                            (!this._ctrlKeyPressed(command.gesture.keyModifiers) || ctrlKey)))) {
                            this._currentCommand = i;
                            if (command.execute && command.canExecute) {
                                var canExecute = command.canExecute;
                                if (typeof command.canExecute == "string") canExecute = ej.util.getObject(command.canExecute, window);
                                if ($.isFunction(canExecute) && canExecute({ model: this.model })) {
                                    var execute = command.execute;
                                    if (typeof command.execute === "string") {
                                        execute = ej.util.getObject(command.execute, window);
                                    }
                                    if ($.isFunction(execute))
                                        execute({ keyDownEventArgs: evt, parameter: command.parameter, model: this.model });
                                }
                            }
                        }
                    }
                }
            }
            this.activeTool.keydown(evt);
        },
        _keyup: function (evt) {
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            if (!this._isEditing && (keycode === 37 || keycode === 38 || keycode === 39 || keycode === 40)) {
                if (this._currentState) {
                    this.addHistoryEntry({ type: "nudge", category: "internal", values: this._currentState, object: this.selectionList[0] });
                }
                delete this._currentState;
                evt.preventDefault();
                this.activeTool.inAction = false;
                this._updateSelectionHandle();
            }
        },
        _registerHistoryEvents: function () {
            this.historyManager.addEventHandler("collectionchanged", $.proxy(this._recordCollectionChanged, this));
            this.historyManager.addEventHandler("positionchanged", $.proxy(this._recordPinPointChanged, this));
            this.historyManager.addEventHandler("sizechanged", $.proxy(this._recordResizeChanged, this));
            this.historyManager.addEventHandler("rotationchanged", $.proxy(this._recordRotationChanged, this));
            this.historyManager.addEventHandler("endpointchanged", $.proxy(this._recordEndPointChanged, this));
            this.historyManager.addEventHandler("labelchanged", $.proxy(this._recordLabelChanged, this));
            this.historyManager.addEventHandler("groupingchanged", $.proxy(this._recordGroupChanged, this));
        },
        _svgdrop: function (evt, ui) {
            if (this._selectedSymbol && ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var args = {};
                var e = evt.originalEvent;
                if (e.preventDefault)
                    e.preventDefault();
                else
                    e.returnValue = false;
                var scale = this._currZoom;
                var offset = $("#" + this.element[0].id).offset();
                var coor = this._isTouchEvent(evt);
                var x = (coor ? coor.pageX : evt.pageX) + this._hScrollOffset - offset.left;
                var y = (coor ? coor.pageY : evt.pageY) + this._vScrollOffset - offset.top;
                var paletteItem = $.extend(true, {}, this._selectedSymbol);
                var symbolPaletteId = ui.helper[0].getAttribute("paletteId");
                var symbolPalette = $("#" + symbolPaletteId).ejSymbolPalette("instance");
                if (symbolPaletteId) {
                    this._selectedSymbol = null;
                    paletteItem.name += ej.datavisualization.Diagram.Util.randomId();
                    if (!paletteItem.isPhase && paletteItem.segments) {
                        if (paletteItem.segments) {
                            ej.datavisualization.Diagram.Util._translateLine(paletteItem, x, y);
                        }
                        paletteItem = this.getNode(paletteItem);
                    } else if (!paletteItem.isPhase) {
                        var palNameTable = $.extend(true, {}, symbolPalette.nameTable);
                        paletteItem.offsetX = x / scale;
                        paletteItem.offsetY = y / scale;
                        if (paletteItem._type === "group") {
                            var children = this._getChildren(paletteItem.children);
                            for (var j = 0; j < children.length; j++) {
                                var newObj = $.extend(true, {}, palNameTable[children[j]]);
                                newObj.parent = paletteItem.name;
                                newObj.name = paletteItem.name + newObj.name;
                                children[j] = newObj.name;
                                this.nameTable[newObj.name] = newObj;
                                this.nodes().push(newObj);
                                this._nodes = $.extend(true, [], this.nodes());
                                this._updateQuad(this.nameTable[newObj.name]);
                            }
                            paletteItem.height = paletteItem.width = 0;
                            this._updateChildBounds(paletteItem, this.nameTable);
                        }
                        if (paletteItem._type === "group")
                            paletteItem = this._getNewGroup(paletteItem);
                        else if (paletteItem.segments)
                            paletteItem = this._getNewConnector(paletteItem);
                        else if (!(paletteItem.isPhase || paletteItem.type === "phase"))
                            paletteItem = this._getNewNode(paletteItem);
                    }
                    if (paletteItem && paletteItem.isLane) {
                        this._cloneGroupNode(paletteItem, ej.datavisualization.Diagram.Util.randomId());
                        paletteItem = ej.datavisualization.Diagram.SwimLaneHelper._createDiagramLane(paletteItem, this.nameTable)
                    }
                    args = { element: paletteItem, cancel: false };
                    this._raiseDropEvent(args);

                    if (paletteItem && paletteItem.isLane) {
                        var swimlane = ej.datavisualization.Diagram.SwimLaneHelper._createSwimlane(paletteItem, this);
                        if (swimlane && swimlane.isSwimlane)
                            paletteItem = swimlane;
                    }
                    if (!paletteItem.isPhase) {
                        if (!args.cancel) {
                            this.add(paletteItem);
                        }
                        paletteItem = this.selectionList[0];
                        this.scrollToNode(paletteItem);
                        if (paletteItem.labels.length > 0 && paletteItem.labels[0].mode === "edit") {
                            this._isEditing = true;
                            this.startLabelEdit(paletteItem, paletteItem.labels[0]);
                        } else
                            this.element[0].focus();
                    }
                }
            } else {
                var node = null;
                node = this._findNodeUnderMouse(evt);
                args = { element: ui.helper, e: evt, draggable: ui.draggable, targetNode: node };
                if (node)
                    this._raiseDropEvent(args);
            }
        },
        _isTouchEvent: function (evt) {
            if (evt.type == "touchmove" || evt.type == "touchstart" || evt.type == "touchend") {
                return evt.originalEvent ? evt.originalEvent.changedTouches[0] : evt.changedTouches[0];
            }
            return evt;
        },
        _raiseDropEvent: function (args) {
            args.source = this._symbolPalette ? this._symbolPalette : null;
            args.objectType = args.source ? ej.datavisualization.Diagram.ObjectTypes.Palette : ej.datavisualization.Diagram.ObjectTypes.Diagram;
            this._raiseEvent("drop", args);
            this._symbolPalette = null;
        },
        //#endregion
        //#region Undo Redo
        _recordCollectionChanged: function (args, orgObj) {
            this._isUndo = true;
            this._isInsert = true;
            var i;
            var changeType = args.changeType;
            if (args.isUndo) {
                if (args.changeType == "insert")
                    changeType = "remove";
                else
                    changeType = "insert";
            }
            this._clearSelection(changeType == "insert" ? false : true);
            if (changeType == "insert") {
                if (!args.object.container) {
                    if (args.object.type != "pseudoGroup") {
                        if (!args.object.segments) {
                            var port = null;
                            if (args.object.inEdges && args.object.inEdges.length > 0) {
                                for (i = 0; i < args.object.inEdges.length; i++) {
                                    if (this.nameTable[args.object.inEdges[i]])
                                        this.nameTable[args.object.inEdges[i]].targetNode = args.object.name;
                                    if (args.targetPorts)
                                        port = args.targetPorts[args.object.inEdges[i]];
                                    if (port)
                                        if (this.nameTable[args.object.inEdges[i]])
                                            this.nameTable[args.object.inEdges[i]].targetPort = port;
                                }
                            }
                            if (args.object.outEdges && args.object.outEdges.length > 0) {
                                for (i = 0; i < args.object.outEdges.length; i++) {
                                    if (this.nameTable[args.object.outEdges[i]])
                                        this.nameTable[args.object.outEdges[i]].sourceNode = args.object.name;
                                    if (args.sourcePorts)
                                        port = args.sourcePorts[args.object.outEdges[i]];
                                    if (port)
                                        if (this.nameTable[args.object.outEdges[i]])
                                            this.nameTable[args.object.outEdges[i]].sourcePort = port;
                                }
                            }
                        }
                        if (args.object._type === "group")
                            this._addChildren(args.object, args.childTable);
                        if (args.object.isSwimlane)
                            args.object = ej.datavisualization.Diagram.SwimLaneHelper._initSwimLane(args.object, this)
                        this.add(args.object, args);
                        if (args.object.parent) {
                            var pNode = this.nameTable[args.object.parent];
                            if (pNode && pNode.isLane) {
                                var moveTool = this.tools["move"];
                                moveTool._updateContainer(args.object, pNode, true, true);
                            }
                        }
                        if (args.object._type === "group") {
                            this._addConnector(args.object, args);
                            this._addGroupConnector(args.object, args);
                        }
                        else {
                            this._addConnector(args.object, args);
                        }
                    } else {
                        delete this.nameTable["multipleSelection"];
                        var node, j;
                        var pseudoGroup = ej.datavisualization.Diagram.Group({ "name": "multipleSelection", type: "pseudoGroup" });

                        for (i = 0; i < args.object.children.length; i++) {
                            child = typeof args.object.children[i] === "object" ? args.object.children[i] : args.childTable[args.object.children[i]];
                            if (this.getObjectType(child) === "group" && !child.isSwimlane)
                                this._addChildren(child, args.childTable);
                            this.add(child, args);
                            pseudoGroup.children.push(child.name);
                        }
                        this._addGroupConnector(args.object, args);
                        for (i = 0; i < pseudoGroup.children.length; i++) {
                            node = this.nameTable[pseudoGroup.children[i]];
                            if (!node.segments) {
                                if (node.inEdges.length > 0) {
                                    for (j = 0; j < node.inEdges.length; j++)
                                        this.nameTable[node.inEdges[j]].targetNode = node.name;
                                }
                                if (node.outEdges.length > 0) {
                                    for (j = 0; j < node.outEdges.length; j++)
                                        this.nameTable[node.outEdges[j]].sourceNode = node.name;
                                }
                            }
                        }
                        this._clearSelection();
                        this.nodes().push(pseudoGroup);
                        this._nodes = $.extend(true, [], this.nodes());
                        pseudoGroup.offsetX = args.object.offsetX;
                        pseudoGroup.offsetY = args.object.offsetY;
                        pseudoGroup.height = args.object.height;
                        pseudoGroup.width = args.object.width;
                        pseudoGroup.rotateAngle = args.object.rotateAngle;
                        this.nameTable[pseudoGroup.name] = pseudoGroup;
                        this.addSelection(pseudoGroup, true);
                    }
                } else {
                    var child;
                    for (var i in args.childTable) {
                        child = args.childTable[i];
                        this.nameTable[i] = child;
                    }
                    node = args.object;
                    if (node.isSwimlane) {
                        node.width = 0;
                        node.height = 0;
                        this.add(node);
                    } else if (node.isLane) {
                        var moveTool = this.tools["move"];
                        if (args.object && args.object.parent)
                            var parentStack = this.nameTable[args.object.parent];
                        if (parentStack) {
                            if (parentStack)
                                var swimlane = this.nameTable[parentStack.parent];
                            if (swimlane) {
                                moveTool.selectedObject = args.object;
                                moveTool._addNewLane(args.object, swimlane, true, args.index);
                                this._clearSelection();
                                moveTool.selectedObject = null;
                            }
                        }
                    }
                }
            }
            else if (changeType == "remove") {
                if (args.object.type != "pseudoGroup" && !args.object.isLane) {
                    this._removeAdjacentConnections(args.object, args);
                    if (args.object.isSwimlane)
                        this._remove(this.nameTable[args.object.name]);
                    else
                        this._remove(args.object);
                    return true;
                }
                else if (args.object.isLane) {
                    var obj = args.object;
                    if (!args.index) {
                        var index = null
                        if (args.object.isLane && orgObj) {
                            orgObj.index = this._getRemoveIndex(args.object, true);
                        }
                    }
                    this._remove(obj);
                    if (obj.container && obj.parent) {
                        var parent = this.nameTable[obj.parent];
                        if (parent && parent.container) {
                            if (parent.container.type == "stack") {
                                if (this._getChild(parent.children[0])) {
                                    this.tools.move._updateContainer(this.nameTable[this._getChild(parent.children[0])], false, false, false);
                                    ej.datavisualization.Diagram.DiagramContext.update(parent, this);
                                } else {
                                    var prePar = this.nameTable[parent.parent];
                                    if (prePar) {
                                        this._remove(prePar);
                                    }
                                }
                            }
                        }
                    }
                }
                else {
                    for (i = 0; i < args.object.children.length; i++) {
                        this._remove(this.nameTable[this._getChild(args.object.children[i])]);
                    }
                }
            }
            this._isUndo = false;
            this._isInsert = false;
        },
        _updateRecordPinPoint: function (node, childTable, isMultipleSelection) {
            var object, i, item, objNode;
            // this._clearSelection();
            if (node.type == "pseudoGroup") {
                var undoObj = node;
                var children = this._getChildren(node.children);
                for (i = 0; i < children.length; i++) {
                    item = this.nameTable[children[i]];
                    objNode = childTable[children[i]];
                    if (objNode) {
                        if (!item.segments) {
                            this._translate(item, objNode.offsetX - item.offsetX, objNode.offsetY - item.offsetY, this.nameTable);
                        } else {
                            var objLine = childTable[item.name];
                            item.sourcePoint = objLine.sourcePoint;
                            item.targetPoint = objLine.targetPoint;
                            item.segments = jQuery.extend(true, item.segments, objLine.segments);
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(item, this);
                    }
                }
                if (this.selectionList[0]) {
                    if (this.selectionList[0].name === undoObj.name) {
                        this.selectionList[0].offsetX = undoObj.offsetX;
                        this.selectionList[0].offsetY = undoObj.offsetY;
                    }
                }
                ej.datavisualization.Diagram.SvgContext.updateSelector(undoObj, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
            } else {
                if (node.segments) {
                    if (isMultipleSelection) {
                        object = childTable[node.name];
                        node.sourcePoint = object.sourcePoint;
                        node.targetPoint = object.targetPoint;
                        node.segments = object.segments;
                        ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    } else {
                        this._clearSelection(true);
                        object = this._findConnector(node.name);
                        var obj;

                        if (node.targetNode) {
                            obj = this.nameTable[node.targetNode];
                            ej.datavisualization.Diagram.Util.removeItem(obj.inEdges, node);
                        }
                        if (node.sourceNode) {
                            obj = this.nameTable[node.sourceNode];
                            ej.datavisualization.Diagram.Util.removeItem(obj.outEdges, node);
                        }
                        if (node.targetNode) {
                            obj = this.nameTable[node.targetNode];
                            obj.inEdges.push(node.name);
                        }
                        if (node.sourceNode) {
                            obj = this.nameTable[node.sourceNode];
                            obj.outEdges.push(node.name);
                        }
                        object.targetNode = node.targetNode;
                        object.sourceNode = node.sourceNode;
                        object.targetPort = node.targetPort;
                        object.sourcePort = node.sourcePort;

                        object.sourcePoint = node.sourcePoint;
                        object.targetPoint = node.targetPoint;
                        object.segments = node.segments;
                        this._updateLabels(object, node.labels);
                        ej.datavisualization.Diagram.DiagramContext.update(object, this);
                        ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
                        this.addSelection(node, true);
                    }
                } else {
                    if (isMultipleSelection) {
                        object = childTable[node.name];
                        this._translate(node, object.offsetX - node.offsetX, object.offsetY - node.offsetY, childTable);
                        ej.datavisualization.Diagram.DiagramContext.update(object, this);
                    } else {
                        this._clearSelection(true);
                        object = this._findNode(node.name);
                        this._translate(object, node.offsetX - object.offsetX, node.offsetY - object.offsetY, this.nameTable);
                        this._updateLabels(object, node.labels);
                        if (node.parent) {
                            ej.datavisualization.Diagram.Util._updateGroupBounds(this.nameTable[object.parent], this);
                        }
                        if (object.parent) {
                            var parent = this.nameTable[object.parent];
                            if (parent && object.parent && node.parent == "") {
                                ej.datavisualization.Diagram.Util.removeItem(parent.children, object.name);
                                this._delete(object);
                                this.add(node);
                            }
                            if (parent && parent.container) {
                                if (parent.container.type == "canvas") {
                                    this._updateNodeMargin(object);
                                    ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this);
                                }
                                else if (parent.container.type == "stack") {

                                    ej.datavisualization.Diagram.SvgContext._alignOnStack(parent, this);
                                }
                            }
                        }
                        if (node.parent) {
                            var parent = this.nameTable[node.parent];
                            if (node.parent && object.parent == "") {
                                this._delete(object);
                                node = $.extend(true, {}, node);
                                this.nodes().push(node);
                                this._nodes = $.extend(true, [], this.nodes());
                                parent.children.push(node.name);
                                this.nameTable[node.name] = node;
                                ej.datavisualization.Diagram.DiagramContext.renderNode(node, this, parent);
                            }
                            if (parent && parent.isLane) {
                                var moveTool = this.tools["move"];
                                this._updateUndoObject(node, object)
                                moveTool._updateContainer(object, parent, true);
                                this._renderNodeOnLane(object, this, null, true);
                                ej.datavisualization.Diagram.DiagramContext.update(object, this);
                            }
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(object, this);
                        this.addSelection(object, true);
                    }
                }
            }
        },

        _renderNodeOnLane: function (group, parent) {
            var view, panel;
            var diagram = this;
            diagram._views.forEach(function (viewid) {
                var isOverView
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                parent = view.svg.getElementById(group.parent);
                if (view.type == "overview") {
                    isOverView = true;
                }
                if (group._type === "group")
                    view.context.renderGroup(group, panel, parent || view.diagramLayer, diagram.nameTable, diagram, null, true);
                else if (diagram.getObjectType(group) === "node")
                    view.context.renderNode(group, panel, parent || view.diagramLayer);
            });
        },
        _updateUndoObject: function (node, object) {
            if (object.parent)
                var parent = this.nameTable[object.parent];
            if (parent && this._collectionContains(object.name, parent.children)) {
                ej.datavisualization.Diagram.Util.removeItem(parent.children, object.name);
                if (node.parent)
                    var newParent = this.nameTable[node.parent];
                if (newParent && newParent.children && !this._collectionContains(object.name, newParent.children)) {
                    newParent.children.push(node.name);
                    this.nameTable[node.name].parent = newParent.name;
                    if (newParent.isLane) {
                        var moveTool = this.tools["move"];
                        moveTool._updateMargin(object, newParent);
                    }
                }
            }
            else {
                if (node.parent) {
                    var parent = this.nameTable[node.parent];
                }
            }
        },
        _collectionContains: function (name, coll) {
            for (var i = 0; i < coll.length; i++) {
                if (typeof (coll[i]) === "string" && coll[i] === name) {
                    return true;
                }
                else if (coll[i].name === name) {
                    return true;
                }
            }
        },
        _recordPinPointChanged: function (args) {
            var node = args.object.node;
            var childTable = args.object.childTable;
            if (!node.container || node.isSwimlane)
                this._updateRecordPinPoint(node, childTable);
        },

        _updateRecordRotation: function (object, childTable) {
            var node, i, item, objNodeChild;
            if (object.type == "pseudoGroup") {
                this._clearSelection(true);
                node = this.nameTable[object.name];
                var children = object.children;
                for (i = 0; i < children.length; i++) {
                    item = this.nameTable[children[i]];
                    objNodeChild = childTable[children[i]];
                    if (item) {
                        if (!item.segments) {
                            if (objNodeChild) {
                                this._translate(item, objNodeChild.offsetX - item.offsetX, objNodeChild.offsetY - item.offsetY, this.nameTable);
                                this._rotate(item, objNodeChild.rotateAngle - item.rotateAngle, this.nameTable);
                            }
                        } else {
                            var objLine = childTable[item.name];
                            item.sourcePoint = objLine.sourcePoint;
                            item.targetPoint = objLine.targetPoint;
                            item.segments = objLine.segments;
                        }
                        ej.datavisualization.Diagram.DiagramContext.update(item, this);
                    }
                }
                if (node) {
                    var newangle = object.rotateAngle - node.rotateAngle;
                    node.rotateAngle += newangle;
                    if (this.selectionList[0])
                        this.selectionList[0].rotateAngle = node.rotateAngle;
                }
                this.addSelection(object, true);
            }
            else {
                var parent = null;
                if (object.parent) {
                    parent = this.nameTable[object.parent];
                }
                if (parent && parent.isLane) {
                    var rotateTool = this.tools["rotate"];
                    rotateTool._updateContainer(object, parent, true);
                }
                if (parent && parent.container) {
                    if (parent.container.type == "canvas") {
                        this._clearSelection(true);
                        var rnode = this._findNode(object.name);
                        this._rotate(rnode, object.rotateAngle - rnode.rotateAngle, this.nameTable);
                        ej.datavisualization.Diagram.DiagramContext.update(rnode, this);
                        this.addSelection(rnode, true);
                        this._updateNodeMargin(rnode);
                        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this);
                    }
                    else if (parent.container.type == "stack") {

                        ej.datavisualization.Diagram.SvgContext._alignOnStack(parent, this);
                    }

                }
                else if (object) {
                    this._clearSelection(true);
                    node = this._findNode(object.name);
                    this._rotate(node, object.rotateAngle - node.rotateAngle, this.nameTable);
                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    this.addSelection(node, true);
                }
            }
        },

        _recordSizeChanged: function (args) {
            var node = args.object.node;
            var childTable = args.object.childTable;
            //  if (!node.container)
            this._updateRecordSize(node, childTable);
        },

        _updateRecordSize: function (object, childTable) {
            var node, i, childObj;
            this._clearSelection(true);
            if ((object._type === "group" || object.type === "pseudoGroup") && !(object.isLane || object.isSwimlane)) {
                if (object.type == "pseudoGroup")
                    this.nameTable[object.name] = object;
                node = this._findNode(object.name);
                var nodeChildren = this._getChildren(node.children);
                var objChildren = this._getChildren(object.children);
                for (i = 0; i < nodeChildren.length; i++) {
                    var item = this.nameTable[objChildren[i]];
                    childObj = childTable[objChildren[i]];
                    if (childObj) {
                        if (!item.segments) {
                            this._translate(item, childObj.offsetX - item.offsetX, childObj.offsetY - item.offsetY, this.nameTable);
                            this.scale(item, (childObj.width / item.width), (childObj.height / item.height), ej.datavisualization.Diagram.Point(item.offsetX, item.offsetY), this.nameTable);
                            //ej.datavisualization.Diagram.DiagramContext.update(item, this);
                        } else {
                            var kk = childTable[item.name];
                            item.sourcePoint = kk.sourcePoint;
                            item.targetPoint = kk.targetPoint;
                            item.segments = kk.segments;
                            //ej.datavisualization.Diagram.DiagramContext.update(item, this);
                        }
                    }
                }
                //if (node.name === object.name) {
                node.offsetX = object.offsetX;
                node.offsetY = object.offsetY;
                node.width = object.width;
                node.height = object.height;
                if (!(this.activeTool.name == "resize" && this.activeTool.inAction && this.selectionList[0].type != "pseudoGroup" && this.selectionList[0].type != "group"))
                    this._updateAssociatedConnectorEnds(node, this.nameTable);

                ej.datavisualization.Diagram.DiagramContext.update(node, this);
                this.addSelection(node, true);
            }
            else if (object.isLane || object.isSwimlane) {
                node = this._findNode(object.name);
                var resizeTool = this.tools["resize"];
                resizeTool.selectedObject = node;
                resizeTool.initialBounds = { width: resizeTool.selectedObject.width, height: resizeTool.selectedObject.height, y: resizeTool.selectedObject.offsetY, x: resizeTool.selectedObject.offsetX };
                resizeTool.endPoint = resizeTool.startPoint;
                ej.datavisualization.Diagram.SvgContext._removeContainerHelper(resizeTool.selectedObject, resizeTool.diagram._svg, resizeTool.diagram._adornerLayer);
                var object1 = resizeTool.diagram.nameTable[resizeTool.selectedObject.name];
                if (resizeTool.selectedObject.container) {
                    object1.minWidth = object.minHeight;
                    object1.minHeight = object.minWidth;
                }
                resizeTool._updateSize(resizeTool.selectedObject, null, null, Math.floor(object.width - node.width), Math.floor(object.height - node.height), true);
                if (resizeTool.selectedObject.parent) {//canvas panel
                    var parent = resizeTool.diagram.nameTable[resizeTool.selectedObject.parent];
                    if (parent._type === "group" && parent.container && parent.container.type == "stack") {
                        ej.datavisualization.Diagram.SvgContext._removeContainerHelper(resizeTool.selectedObject, resizeTool.diagram._svg, resizeTool.diagram._adornerLayer);
                        object1 = resizeTool.diagram.nameTable[resizeTool.selectedObject.name];
                        var newsize = resizeTool.diagram._getMinMaxSize(resizeTool.selectedObject, object.width, object.height);
                        object1.height = newsize.height;
                        object1.width = newsize.width;
                        if (resizeTool.selectedObject.container) {
                            object1.minWidth = object1.width;
                            object1.minHeight = object1.height;
                        }
                        resizeTool._updateContainer(resizeTool.diagram.nameTable[resizeTool.selectedObject.name]);
                    }
                }
                else if (resizeTool.selectedObject) {//stack panel
                    resizeTool.selectedObject.minWidth = resizeTool.selectedObject.width;
                    resizeTool.selectedObject.minHeight = resizeTool.selectedObject.height;
                }
                ej.datavisualization.Diagram.DiagramContext.update(resizeTool.selectedObject, resizeTool.diagram);
                resizeTool.diagram._updateSelectionHandle();
                this.deactivateTool();
            }
            else {
                var parent = null;
                if (object.parent) {
                    parent = this.nameTable[object.parent];
                }
                if (parent && parent.container) {
                    if (parent.container.type == "canvas") {

                        node = this._findNode(object.name);
                        this._translate(node, object.offsetX - node.offsetX, object.offsetY - node.offsetY, this.nameTable);
                        this.scale(node, object.width / node.width, object.height / node.height, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                        ej.datavisualization.Diagram.DiagramContext.update(node, this);
                        if (this.selectionList && this.selectionList.length > 0)
                            ej.datavisualization.Diagram.SvgContext.updateSelector(node, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
                        else
                            this.addSelection(node, true);
                        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this);
                    }
                    else if (parent.container.type == "stack") {

                        ej.datavisualization.Diagram.SvgContext._alignOnStack(parent, this);
                    }

                }
                else if (!object.segments) {

                    node = this._findNode(object.name);
                    this._translate(node, object.offsetX - node.offsetX, object.offsetY - node.offsetY, this.nameTable);
                    this.scale(node, object.width / node.width, object.height / node.height, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), this.nameTable);
                    ej.datavisualization.Diagram.DiagramContext.update(node, this);
                    this.addSelection(node, true);
                }
                else { }
                if (parent && parent.isLane) {
                    var resizeTool = this.tools["resize"];
                    resizeTool._updateContainer(object, parent, true);
                }
            }

        },

        _recordRotationChanged: function (args) {
            var node = args.object.node;
            var childTable = args.object.childTable;
            //  if (!node.container)
            this._updateRecordRotation(node, childTable);
        },

        _recordEndPointChanged: function (args) {
            var connector, currentObject, previousObject, obj;
            if (args._isUndo) {
                connector = currentObject = args.undoObject;
                previousObject = args.redoObject;
            }
            else {
                connector = currentObject = args.redoObject;
                previousObject = args.undoObject;
            }
            if (connector.parent) {
                connector = this._findChildren(connector.parent, connector.name);
            }
            else
                connector = this._findConnector(connector.name);

            if (previousObject.targetNode) {
                obj = this.nameTable[previousObject.targetNode];
                ej.datavisualization.Diagram.Util.removeItem(obj.inEdges, previousObject);
            }
            if (previousObject.sourceNode) {
                obj = this.nameTable[previousObject.sourceNode];
                ej.datavisualization.Diagram.Util.removeItem(obj.outEdges, previousObject);
            }
            if (currentObject.targetNode) {
                obj = this.nameTable[currentObject.targetNode];
                obj.inEdges.push(currentObject.name);
            }
            if (currentObject.sourceNode) {
                obj = this.nameTable[currentObject.sourceNode];
                obj.outEdges.push(currentObject.name);
            }
            connector.targetNode = currentObject.targetNode;
            connector.sourceNode = currentObject.sourceNode;
            connector.targetPort = currentObject.targetPort;
            connector.sourcePort = currentObject.sourcePort;
            connector.sourcePoint = currentObject.sourcePoint;
            connector.targetPoint = currentObject.targetPoint;
            connector.segments = currentObject.segments;
            this._updateEdges(connector);
            this._dock(connector, this.nameTable);
            ej.datavisualization.Diagram.Util._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort, this);
            ej.datavisualization.Diagram.DiagramContext._refreshSegments(connector, this);
            ej.datavisualization.Diagram.Util.updateBridging(connector, this);
            ej.datavisualization.Diagram.DiagramContext.update(connector, this);
            this._updateConnectorBridging(connector);
            this._updateSelectionHandle();
            if (this.selectionList[0])
                ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.selectionList[0], this._adornerSvg, this._currZoom);
            ej.datavisualization.Diagram.SvgContext.updateSelector(connector, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
        },
        _recordLabelChanged: function (args) {
            var node = this.nameTable[args.object.name];
            if (args.object.type === "text") {
                node.textBlock.text = args.label;
                ej.datavisualization.Diagram.DiagramContext.updateTextBlock(node, node.textBlock, this);
            } else {
                node.labels[args.index].text = args.label;
                ej.datavisualization.Diagram.DiagramContext.updateLabel(node, node.labels[args.index], this);
            }
        },
        _recordGroupChanged: function (args) {
            var actionType = args.actionType;
            if (args.isUndo) {
                if (args.actionType === "group")
                    actionType = "ungroup";
                if (args.actionType === "ungroup")
                    actionType = "group";
            }
            this._clearSelection(actionType == "group" ? true : false);
            var selectionList = this._getChildren(args.object.children);
            if (actionType === "group") {
                var group = new ej.datavisualization.Diagram.Group({ name: args.object.name });
                for (i = 0; i < selectionList.length; i++) {
                    group.children.push(selectionList[i]);
                    var childNode = this.nameTable[selectionList[i]];
                    if (childNode.parent)
                        oldParent = this.nameTable[childNode.parent];
                    childNode.parent = group.name;
                    var groupElement = this._svg.document.getElementById(selectionList[i]);
                    if (groupElement)
                        groupElement.parentNode.removeChild(groupElement);
                    if (childNode.parent)
                        newParent = this.nameTable[childNode.parent];
                    this._raiseGroupChangeEvent(childNode, oldParent, newParent, "group");
                }
                //group.name = args.object.name;
                this.add(group);
                this._addSelection(group);
            }
            else {
                var i, node, oldParent, newParent;
                node = args.object;
                var groupElement = this._svg.document.getElementById(node.name);
                if (groupElement)
                    groupElement.parentNode.removeChild(groupElement);
                var nodeChildren = this._getChildren(node.children);
                for (i = 0; i < nodeChildren.length; i++) {
                    var childNode = this.nameTable[nodeChildren[i]];
                    if (childNode.parent)
                        oldParent = this.nameTable[childNode.parent];
                    childNode.parent = "";
                    this._isUndo = true;
                    if (childNode.segments) {
                        ej.datavisualization.Diagram.DiagramContext.renderConnector(childNode, this);
                    }
                    else if (childNode._type != "group") {
                        ej.datavisualization.Diagram.DiagramContext.renderNode(childNode, this);
                    }
                    else {
                        ej.datavisualization.Diagram.DiagramContext.renderGroup(childNode, this);
                    }
                    this._isUndo = false;
                    this.nodes().push(childNode);
                    this._nodes = $.extend(true, [], this.nodes());
                    if (childNode.parent)
                        newParent = this.nameTable[childNode.parent];
                    this._raiseGroupChangeEvent(childNode, oldParent, newParent, "ungroup");
                }
                ej.datavisualization.Diagram.Util.removeItem(this.nodes(), this._findNode(node.name));
                this._nodes = $.extend(true, [], this.nodes());
            }
        },

        _recordNudgingChanges: function (obj, entryItem) {
            var obj = entryItem.object;
            var values = { x: obj.offsetX, y: obj.offsetY };
            this._translate(obj, entryItem.values.x - obj.offsetX, entryItem.values.y - obj.offsetY, this.nameTable);
            entryItem.values = values;
            this._updateObject(obj);
            this._updateSelectionHandle();
        },

        _recordPropertiesChanged: function (node, options, type, object) {
            var currentState = this._getModifiedProperties((type == "port" || type == "label") ? object : node, options);
            if (currentState && Object.keys(currentState).length) {
                var entry = { type: "propertiesChanged", values: currentState, category: "internal", object: node, elementType: type };
                entry[type] = object;
                this.addHistoryEntry(entry);
            }
        },
        _recordAlignCommandChanges: function (entryItem) {
            if (entryItem && entryItem.values.length) {
                for (var i = 0; i < entryItem.values.length; i++) {
                    var obj = entryItem.values[i];
                    if (obj.object && obj.delta) {
                        var current = { x: -obj.delta.x, y: -obj.delta.y };
                        this._translate(obj.object, -obj.delta.x, -obj.delta.y, this.nameTable);
                        obj.delta = current;
                        this._updateObject(obj.object);
                    }
                }
                ej.datavisualization.Diagram.Util._updateGroupBounds(entryItem.object, this);
                if (this._hasSelection())
                    ej.datavisualization.Diagram.SvgContext.updateSelector(entryItem.object, this._adornerSvg, this._currZoom, this, this.model.selectedItems.constraints);
                else
                    this.addSelection(entryItem.object, true);
            }
        },
        _recordSizeCommandChanges: function (entryItem) {
            if (entryItem && entryItem.values.length) {
                for (var i = 0; i < entryItem.values.length; i++) {
                    var obj = entryItem.values[i];
                    if (obj.object && obj.values) {
                        var current = { width: obj.object.width || obj.object._width || 0, height: obj.object.height || obj.object._height || 0 };
                        this.scale(obj.object, obj.values.width / (obj.object.width || obj.object._width || 0),
                            obj.values.height / (obj.object.height || obj.object.height || 0),
                          { x: obj.object.offsetX, y: obj.object.offsetY }, this.nameTable);
                        obj.values = current;
                        this._updateObject(obj.object);
                    }
                }
            }
        },
        _getModifiedProperties: function (node, options, object) {
            object = object || node;
            if (options && Object.keys(options).length && object) {
                var current = {};
                for (var prop in options) {
                    if (options[prop] != object[prop] || typeof object[prop] == "object") {
                        if (object[prop] && object[prop] instanceof Array)
                            current[prop] = object[prop];
                        else if (object[prop] && typeof object[prop] == "object")
                            current[prop] = $.extend(true, {}, object[prop]);
                        else
                            current[prop] = object[prop];
                    }
                }
                return current;
            }
        },
        _addCollectionChangeToHistory: function (node) {
            if (node.parent == "") {
                var childTable = {};
                if (this.selectionList && this.selectionList.length > 0)
                    childTable = this._getChildTable(this.selectionList[0], childTable);
                var entry = { type: "collectionchanged", object: jQuery.extend(true, {}, node), childTable: jQuery.extend(true, {}, childTable), changeType: "insert", category: "internal" };
                this.addHistoryEntry(entry);
            }
        },
        _recordPhasesizeChanged: function (args) {
            var phaseTool = this.tools["phase"];
            phaseTool.selectedSeperetor = this.nameTable[args.redoObject.name];

            var ptCur = this._isUndo ? jQuery.extend(true, {}, args.startPoint) : jQuery.extend(true, {}, args.currentPoint);
            var ptSrt = this._isUndo ? jQuery.extend(true, {}, args.currentPoint) : jQuery.extend(true, {}, args.startPoint);
            phaseTool.startPoint = ptSrt;
            phaseTool.currentPoint = ptCur;
            var horizontalsnap = { snapped: false, offset: 0, left: false, right: false };
            var verticalsnap = { snapped: false, offset: 0, top: false, bottom: false };
            phaseTool.selectedObject = this.nameTable[phaseTool.selectedSeperetor.parent];
            phaseTool.initialBounds = { width: phaseTool.selectedObject.width, height: phaseTool.selectedObject.height, y: phaseTool.selectedObject.offsetY, x: phaseTool.selectedObject.offsetX };
            if (phaseTool.selectedSeperetor.orientation === "horizontal") {
                var dx = ptSrt.x - ptCur.x;
                dx = phaseTool._snapRight(horizontalsnap, verticalsnap, args.modifiedValues.dx, 0, phaseTool.selectedObject, false);
                dy = 0;
            }
            else {
                var dy = ptCur.y - ptSrt.y;
                dy = phaseTool._snapBottom(horizontalsnap, verticalsnap, 0, args.modifiedValues.dy, phaseTool.selectedObject, false);
                dx = 0;
            }

            phaseTool._updateSwimlane(phaseTool.selectedSeperetor, dx, dy, phaseTool.selectedSeperetor.orientation == "vertical" ? true : false, true);
            this._clearSelection(true);
            phaseTool._removePhasehelper();
        },
        //#endregion
        //#region Helper methods
        _addSymbolToDiagram: function (evt) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var scale = this._currZoom;
                var offset = $("#" + this.element[0].id).offset();
                var coor = this._isTouchEvent(evt);
                var x = (coor ? coor.pageX : evt.pageX) + this._hScrollOffset - offset.left;
                var y = (coor ? coor.pageY : evt.pageY) + this._vScrollOffset - offset.top;
                var paletteItem = $.extend(true, {}, this._selectedSymbol);
                var element;
                if (this._selectedSymbol) {
                    paletteItem.name += ej.datavisualization.Diagram.Util.randomId();
                    element = paletteItem;
                    if (paletteItem.segments) {
                        ej.datavisualization.Diagram.Util._translateLine(paletteItem, x / scale, y / scale);
                    }
                    else {
                        this._cloneGroupNode(paletteItem, ej.datavisualization.Diagram.Util.randomId());
                        if (paletteItem._type === "group" && paletteItem.type != "bpmn") {
                            var cBound = ej.datavisualization.Diagram.Util._getChildrenBounds(paletteItem, this);
                            ej.datavisualization.Diagram.Util._translate(paletteItem, (x / scale - cBound.x - cBound.height / 2), (y / scale - cBound.y - cBound.width / 2), this.nameTable, this);
                        } else {
                            paletteItem.offsetX = x / scale;
                            paletteItem.offsetY = y / scale;
                        }
                        if (paletteItem.isLane) {
                            var obj = ej.datavisualization.Diagram.SwimLaneHelper._createDiagramLane(paletteItem, this.nameTable);
                            obj.isLane = paletteItem.isLane;
                            obj.orientation = paletteItem.orientation;
                            element = obj;
                        }
                    }
                    if (element) {
                        var args = { element: element, cancel: false };
                        this._raiseEvent("dragEnter", args);
                        if (!args.cancel) {
                            if (this._isEditing) {
                                this._endEditing = true;
                                this._endEdit();
                                delete this._endEditing;
                            }
                            if (element && element.labels && element.labels.length > 0) {
                                for (var j = 0; j < element.labels.length; j++)
                                    element.labels[j].visible = false;
                            }
                            this.add(element);
                            this._focusedElement = element;
                            if (element.isLane)
                                ej.datavisualization.Diagram.SvgContext._hideNode(element, this._svg);
                            this.element[0].focus();
                            this._toolToActivate = "move";
                            if (document.getElementsByClassName("e-paletteItem dragClone")[0]) {
                                $(".e-paletteItem.dragClone").css("display", "none");
                            }
                            this.activateTool("move", false);
                            this.activeTool.mousedown(evt);
                            this.activeTool.mousemove(evt);
                        }
                    }
                }
            }
        },
        _removeSymbolFromDiagram: function () {
            if (this._selectedSymbol) {
                var displays = document.getElementsByClassName("e-paletteItem dragClone")[0];
                if (displays != null) {
                    displays.style.display = "block";
                }
                this._remove(this.selectionList[0]);
                this._selectedSymbol = null;
            }
        },
        _updateGroupChild: function (group) {
            var children = group.children;
            for (var i = 0; i < group.children.length; i++) {
                group.children[i] = this.nameTable[this._getChild(group.children[i])];
                if (group.children[i] && group.children[i]._type === "group") {
                    this.updateGroupChild(group.children[i]);
                }
            }
        },
        _dropSymbol: function (object) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                var actualObject = object || this.selectionList[0] || this.activeTool.selectedObject;
                var args = { element: this.getNode(actualObject) };
                if (this._nodeUnderMouse && (this._nodeUnderMouse.isLane || this._nodeUnderMouse.isSwimlane ||
                    this._nodeUnderMouse.segments)) {
                    args.target = this._nodeUnderMouse;
                }
                var added = false;
                if (!args.target) args.target = this.model;
                if (actualObject._type === "group")
                    this._updateGroupChild(actualObject);
                this._remove(actualObject);
                this.nameTable[args.element.name] = args.element;
                this._selectedSymbol = null;
                this._raiseDropEvent(args);
                if (!args.cancel) {
                    this._isInsert = true;
                    var newElement = object || args.element;
                    if (newElement && newElement.labels && newElement.labels.length > 0) {
                        for (var j = 0; j < newElement.labels.length; j++)
                            newElement.labels[j].visible = true;
                    }
                    added = this.add(newElement);
                    this.activeTool.selectedObject = this.selectionList[0];
                    this._isInsert = false;
                } else delete this.nameTable[args.element.name];
                if (added) {
                    var node = this.nameTable[args.element.name];
                    this._updateDroppedSymbol(node);
                }
                if (!ej.datavisualization.Diagram.Util.canDoSingleSelection(this)) {
                    this._clearSelection(true);
                    if (this.tool() & ej.datavisualization.Diagram.Tool.ZoomPan) {
                        this.activateTool("panTool");
                    }
                }
            }
        },
        _updateDroppedSymbol: function (node) {
            if (node && this.selectionList.length > 0) {
                this.scrollToNode(node);
                if ((node.labels.length > 0 && node.labels[0].mode === "edit") && !node.labels[0].readOnly) {
                    if (!node.isPhase && !(node.isSwimlane || node.isLane)) {
                        this._isEditing = true;
                        this.startLabelEdit(node, node.labels[0]);
                    }
                } else {
                    this.element[0].focus();
                }
                if (this._nodeUnderMouse && this._nodeUnderMouse.segments) {
                    ej.datavisualization.Diagram.SvgContext._removeConnectorHighlighter(this._adornerLayer, this._adornerSvg);
                }
            }
        },
        _initiateAutoScroll: function (evt) {
            if (this.activeTool.inAction && this.enableAutoScroll()) {
                var viewPort = this._viewPort;
                var pt = this._mousePosition(evt, true);
                var autoScrollBorder = this.model.pageSettings.autoScrollBorder;
                if (pt.x + autoScrollBorder.right >= viewPort.width - 18) {
                    this._beginAutoScroll("right", evt);
                    this._canAutoScroll = true;
                }
                else if (pt.x <= autoScrollBorder.left) {
                    this._beginAutoScroll("left", evt);
                    this._canAutoScroll = true;
                }
                else if (pt.y + autoScrollBorder.bottom >= viewPort.height - 18) {
                    this._beginAutoScroll("bottom", evt);
                    this._canAutoScroll = true;
                }
                else if (pt.y <= autoScrollBorder.top) {
                    this._beginAutoScroll("top", evt);
                    this._canAutoScroll = true;
                }
                else {
                    this._canAutoScroll = false;
                }
            }
        },
        _updatePoints: function (helper, startPoint, endPoint, isTarget) {
            var offset = new ej.datavisualization.Diagram.Point(endPoint.x - startPoint.x, endPoint.y - startPoint.y);
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset)) {
                helper._updateEndPoint(offset.x, offset.y, isTarget);
            }
        },
        _cloneGroupNode: function (node, id, nameTable) {
            var child = null, clnObj;
            var children = node.children = this._getChildren(node.children);
            if (node._type === "group") {
                for (var i = 0; i < children.length; i++) {
                    if (nameTable)
                        child = nameTable[children[i]];
                    else {
                        if (this._paletteTable)
                            child = this._paletteTable[children[i]];
                    }
                    if (child) {
                        child = $.extend(true, {}, child);
                        if (child._type === "group")
                            this._cloneGroupNode(child, id);
                        clnObj = $.extend(true, {}, child);
                        clnObj.name += id;
                        clnObj.parent = node.name;
                        children[i] = clnObj.name;
                        this.nameTable[clnObj.name] = clnObj;
                    }
                }
            }
        },
        _addGroupConnector: function (group, arg) {
            var edges;
            var connector;
            var k, j, len, l;
            if (!group.segments) {
                var children = this._getChildren(group.children);
                for (k = 0; k < children.length; k++) {
                    var child = this.nameTable[children[k]];
                    if (child.inEdges && child.inEdges.length > 0) {
                        edges = child.inEdges;
                        for (j = 0, len = edges.length; j < len; j++) {
                            if (arg.edgeTable)
                                connector = arg.edgeTable[edges[j]];
                            else
                                connector = arg.childTable[edges[j]];
                            if (connector)
                                this.add(connector);
                        }
                    }
                    if (child.outEdges && child.outEdges.length > 0) {
                        edges = child.outEdges;
                        for (j = 0, len = edges.length; j < len; j++) {
                            if (arg.edgeTable)
                                connector = arg.edgeTable[edges[j]];
                            else
                                connector = arg.childTable[edges[j]];
                            if (connector)
                                this.add(connector);
                        }
                    }
                }
            }
        },
        _addConnector: function (node, arg) {
            var edges;
            var connector;
            var k, j, len, l;
            if (!node.segments) {
                if (node.inEdges && node.inEdges.length > 0) {
                    edges = node.inEdges;
                    for (j = 0, len = edges.length; j < len; j++) {
                        connector = arg.edgeTable[edges[j]];
                        this.add(connector);
                    }
                }
                if (node.outEdges && node.outEdges.length > 0) {
                    edges = node.outEdges;
                    for (j = 0, len = edges.length; j < len; j++) {
                        connector = arg.edgeTable[edges[j]];
                        this.add(connector);
                    }
                }
            }
        },
        _addChildren: function (group, childTable) {
            var children = this._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                var node = childTable[children[i]];
                if (node._type === "group")
                    this._addChildren(node, childTable);
                this.nameTable[children[i]] = node;
            }
        },
        _updateLabels: function (node, labels) {
            node.labels = labels;
            for (var i = 0; i < node.labels.length; i++)
                ej.datavisualization.Diagram.DiagramContext.updateLabel(node, node.labels[i], this);
        },
        _getNode: function (name, childTable) {
            if (!childTable)
                var node = typeof (name) === "object" ? this.nameTable[name.name] : this.nameTable[name];
            else var node = typeof (name) === "object" ? childTable[name.name] : childTable[name];
            if (!node && typeof (name) === "object") {
                node = name;
            }
            else if (node && typeof (name) === "string") {
                node = this.nameTable[name];
            }
            if (node) {
                if (node.isLane) {
                    return ej.datavisualization.Diagram.SwimLaneHelper._mapObject(this, node, "lane")
                }
                else if (node.isPhase || node.type === "phase") {
                    return ej.datavisualization.Diagram.SwimLaneHelper._mapObject(this, node, "phase")
                }
                else if (node.isSwimlane) {
                    return ej.datavisualization.Diagram.SwimLaneHelper._mapObject(this, node, "swimlane", childTable)
                }
                else if (node._type === "node" || node._type === "group" || node._type === "pseudoGroup" || this.getObjectType(node) == "node") {
                    return node;
                }
                    //else if (node.shape) {
                    //    return this._getNewNode(node);
                    //}
                    //else if (node.segments) {
                    //    return this._getNewConnector(node);
                    //}
                else
                    return node;
            }
            return null;
        },
        _updateEdges: function (connector) {
            var node;
            if (ej.datavisualization.Diagram.Util.isTargetConnected(connector)) {
                node = this.nameTable[connector.targetNode];
                if (node && node.inEdges.indexOf(connector.name) == -1)
                    node.inEdges.push(connector.name);
            }
            if (ej.datavisualization.Diagram.Util.isSourceConnected(connector)) {
                node = this.nameTable[connector.sourceNode];
                if (node && node.outEdges.indexOf(connector.name) == -1)
                    node.outEdges.push(connector.name);
            }
        },
        _updateChildrenEdges: function (object) {
            var children = this._getChildren(object.children);
            for (var i = 0; i < children.length; i++) {
                var child = children[i];
                if (child.segments) {
                    this._updateEdges(child);
                    this._dock(child, this.nameTable);
                }
                else if (child._type === "group") {
                    this._updateChildrenEdges(child);
                }
            }
        },
        _updateParent: function (object) {
            var children = this._getChildren(object.children);
            for (var i = 0; i < children.length; i++) {
                var index = this.nodes().indexOf(this.nameTable[children[i]]);
                var child = (object.type != "bpmn") ? this._getNewNode(this.nameTable[children[i]]) : this.nameTable[children[i]];
                if (child && child._type === "group") {
                    this._updateParent(child);
                }
                else if (child) {
                    child.parent = object.name;
                    ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[child.name], this.nameTable[child.name]);
                    ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, this.nameTable[child.name]);
                    this.nameTable[child.name] = child;
                    if (index != -1) {
                        this.nodes()[index] = child;
                    }
                }
                this._updateQuad(this.nameTable[child.name]);
            }
        },
        _findPortAtPoint: function (point, node) {
            var ports;
            var port;
            var location;
            var size, pt, x, y;
            var bounds;
            if (node) {
                bounds = ej.datavisualization.Diagram.Util.bounds(node, true);
                ports = node.ports;
                var matrix;
                for (var i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    size = ej.datavisualization.Diagram.Size(port.size, port.size);
                    pt = ej.datavisualization.Diagram.Util._getPortPosition(port, bounds);
                    matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                    pt = ej.Matrix.transform(matrix, pt);
                    x = pt.x - port.size / 2;
                    y = pt.y - port.size / 2;
                    location = ej.datavisualization.Diagram.Rectangle(x, y, size.width, size.height);
                    if (ej.datavisualization.Diagram.Geometry.containsPoint(location, point)) {
                        return ej.datavisualization.Diagram.Util.canConnect(port, true) ? port : null;
                    }
                }
            }
            return null;
        },
        _findLabelAtPoint: function (point, node) {
            var labels, label = null;
            var location;
            var size, pt, x, y, width, height, tempPt;
            var bounds, textElement, matrix;
            var htmlLayer = this._svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
            var intersectingLabels = [];
            if (node) {
                bounds = ej.datavisualization.Diagram.Util.bounds(node, true);
                labels = node.labels;
                if (labels.length > 0) {
                    label = labels[0];
                    for (var i = 0, len = labels.length; i < len; i++) {
                        label = labels[i];
                        textElement = $(htmlLayer).find("#" + node.name + "_" + label.name)[0];

                        width = textElement ? textElement.offsetWidth : 0;
                        height = textElement ? textElement.offsetHeight : 0;
                        if (!width) width = bounds.width;
                        if (!height) height = label.fontSize;
                        size = ej.datavisualization.Diagram.Size(width, height);
                        pt = ej.datavisualization.Diagram.Util._getLabelPosition(label, bounds);
                        pt.x = bounds.x + pt.x;
                        pt.y = bounds.y + pt.y;
                        matrix = ej.Matrix.identity();
                        if (!node.segments) ej.Matrix.rotate(matrix, -node.rotateAngle, node.offsetX, node.offsetY);
                        else ej.Matrix.rotate(matrix, 0, bounds.x + bounds.width / 2, bounds.y + bounds.height / 2);
                        tempPt = ej.Matrix.transform(matrix, point);
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, -label.rotateAngle, pt.x, pt.y);
                        tempPt = ej.Matrix.transform(matrix, tempPt);

                        if (label.verticalAlignment == ej.datavisualization.Diagram.VerticalAlignment.Top) pt.y += size.height / 2;
                        else if (label.verticalAlignment == ej.datavisualization.Diagram.VerticalAlignment.Bottom) pt.y -= size.height / 2;

                        if (label.horizontalAlignment == ej.datavisualization.Diagram.HorizontalAlignment.Left) pt.x += size.width / 2;
                        else if (label.horizontalAlignment == ej.datavisualization.Diagram.HorizontalAlignment.Right) pt.x -= size.width / 2;

                        x = pt.x - size.width / 2;
                        y = pt.y - size.height / 2;
                        location = ej.datavisualization.Diagram.Rectangle(x, y, size.width, size.height);
                        if (ej.datavisualization.Diagram.Geometry.containsPoint(location, tempPt))
                            intersectingLabels.push(label);
                        if (i == labels.length - 1)
                            return (intersectingLabels.length ? intersectingLabels[intersectingLabels.length - 1] : null);
                    }
                }
            }
            return label;
        },
        _getCurrentZoom: function () {
            return this._currZoom;
        },
        _findNode: function (id) {
            return this.nameTable[id];
        },
        _findPort: function (node, portId) {
            var ports = node.ports;
            for (var i = 0, len = ports.length; i < len; i++) {
                if (ports[i].name === portId) {
                    return ports[i];
                }
            }
            return null;
        },
        _findConnector: function (id) {
            return this.nameTable[id];
        },
        _findChildren: function (group, name) {
            var child, innerchild;
            var children = this._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                child = this.nameTable[children[i]];
                if (child) {
                    if (name == child.name) {
                        return child;
                    }
                    if (child._type === "group") {
                        innerchild = this._findChildren(child, name);
                        if (innerchild) return innerchild;
                    }
                }
            }
            return null;
        },
        _removeLaneParent: function (node) {
            if (node.isLane && node.parent) {
                var stack = this.nameTable[node.parent];
                if (stack.children && stack.children.length === 1) {
                    if (stack && stack.parent) {
                        var swimlane = this.nameTable[stack.parent];
                        if (swimlane) {
                            return { remove: true, node: swimlane };
                        }
                    }
                }
            }
            return { remove: false }
        },
        _delete: function (node) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                node = node || this.selectionList[0];
                if (node) {
                    if ((node.type && node.type != "phase") || node.segments || node._type == "node" || node._type == "pseudoGroup") {
                        var removeParent = this._removeLaneParent(node)
                        if (removeParent.remove) {
                            node = removeParent.node;
                        }
                        this._removeAdjacentConnections(node);
                        var obj = node;
                        if (obj.type == "pseudoGroup") {
                            var children = this._getChildren(obj.children);
                            for (var i = children.length - 1; i >= 0; i--) {
                                this._remove(this.nameTable[children[i]]);
                                this._isUndo = true;
                            }
                            this._isUndo = false;
                        }
                        else {
                            if (this._nodeUnderMouse === obj)
                                this._nodeUnderMouse = null;
                            this._remove(obj);
                            if (obj.container && obj.parent) {
                                var parent = this.nameTable[obj.parent];
                                if (parent && parent.container) {
                                    if (parent.container.type == "stack") {
                                        if (this._getChild(parent.children[0])) {
                                            this.tools.move._updateContainer(this.nameTable[this._getChild(parent.children[0])]);
                                            ej.datavisualization.Diagram.DiagramContext.update(parent, this);
                                        } else {
                                            var prePar = this.nameTable[parent.parent];
                                            if (prePar) {
                                                this._remove(prePar);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        this._clearSelection();
                    }
                    else if (node.type && node.type == "phase") {
                        this._removePhase(node.name);
                        this._clearSelection();
                    }
                }
            }
        },
        _removeAdjacentConnections: function (node, entry) {
            var childTable = {};
            var edgeTable = {};
            childTable = this._getChildTable(node, childTable);
            edgeTable = this._getEdgeTable(node, edgeTable);
            if (!this._isUndo || entry) {
                var sourcePorts = {}, targetPorts = {}, connector, count, i;
                if (!node.segments) {
                    if (node.outEdges && node.outEdges.length > 0) {
                        count = node.outEdges.length;
                        for (i = 0; i < count; i++) {
                            connector = this.nameTable[node.outEdges[i]];
                            if (connector && connector.sourcePort)
                                sourcePorts[connector.name] = connector.sourcePort;
                        }
                    }
                    if (node.inEdges && node.inEdges.length > 0) {
                        count = node.inEdges.length;
                        for (i = 0; i < count; i++) {
                            connector = this.nameTable[node.inEdges[i]];
                            if (connector && connector.targetPort)
                                targetPorts[connector.name] = connector.targetPort;
                        }
                    }
                }
                if (!this._isUndo) {
                    var index = null;
                    if (node.isLane) {
                        index = this._getRemoveIndex(node);
                    }
                    var entry = { type: "collectionchanged", object: this._updatePsedoGroupChildren(node), index: index, childTable: jQuery.extend(true, {}, childTable), edgeTable: jQuery.extend(true, {}, edgeTable), sourcePorts: sourcePorts, targetPorts: targetPorts, changeType: "remove", category: "internal" };
                    if (!node.isPhase)
                        this.addHistoryEntry(entry);
                }
                else {
                    entry.targetPorts = targetPorts;
                    entry.sourcePorts = sourcePorts;
                    entry.edgeTable = edgeTable;
                    entry.childTable = childTable;
                }
            }
        },
        _updatePsedoGroupChildren: function (node) {
            var obj = jQuery.extend(true, {}, node), child;
            if (node && node.type == "pseudoGroup") {
                for (var i = 0; i < obj.children.length; i++) {
                    child = this.nameTable[this._getChild(obj.children[i])];
                    if (child && child.isSwimlane) {
                        obj.children[i] = this.getNode(child);
                    }
                }
            }
            return obj;
        },
        _getRemoveIndex: function (lane, isReverse) {
            if (lane.parent) {
                var parent = this.nameTable[lane.parent], child;
                if (parent && parent.children && parent.children.length > 0) {
                    for (var i in parent.children) {
                        child = this._getChild(parent.children[i])
                        if (child === this._getChild(lane)) {
                            return Number(i);
                        }
                    }
                }
            }
            return 0;
        },
        _getSwimlaneHeader: function (group) {
            var header;
            if (group && group.isSwimlane) {
                if (group.children && group.children.length > 0) {
                    header = this.nameTable[group.children[0]];
                }
            }
            return header;
        },
        _addPhase: function (name, options) {
            var group = this.nameTable[name];
            var excess = false;
            if (group && group.isSwimlane && options) {
                if (group && group.orientation && options.offset) {
                    if (group.orientation == "horizontal") {
                        var stoffset = group.offsetX - group.width / 2;
                        var laneHdr = this._getFirstLane(group);
                        if (laneHdr)
                            stoffset += laneHdr.width;
                        excess = ((options.offset > group.width) || (((group.offsetX - group.width / 2) + options.offset) < stoffset)) ? true : false;
                    }
                    else if (group.orientation == "vertical") {
                        stoffset = group.offsetY - group.height / 2;
                        var swlnhdr = this._getSwimlaneHeader(group);
                        if (swlnhdr)
                            stoffset += swlnhdr.height;
                        var laneHdr = this._getFirstLane(group);
                        if (laneHdr)
                            stoffset += laneHdr.height;
                        excess = (options.offset > group.height || (((group.offsetY - group.height / 2) + options.offset) < stoffset)) ? true : false;
                    }
                }
                if (options.offset && !excess) {
                    var phase = ej.datavisualization.Diagram.phase(options);
                    phase.parent = group.name;
                    if (this.nameTable[phase.name]) {
                        ej.datavisualization.Diagram.Util.removeItem(this.model.phases, this.nameTable[phase.name]);
                        delete this.nameTable[phase.name];
                    }
                    var index = this._getInsertIndex(phase) + 1;
                    var prevPhase = $.extend(true, {}, this.nameTable[group.phases[index - 1]]);
                    this._updateInsertPhase(phase, group.phases[index], group, index, phase);
                    group.phases.splice(index, 0, phase.name);
                    this.model.phases.push(phase);
                    this.nameTable[phase.name] = phase;
                    ej.datavisualization.Diagram.SvgContext.renderphase(phase, this._svg, this._diagramLayer, this);
                    if (!this._isUndo) {
                        var nextPhase = $.extend(true, {}, this.nameTable[group.phases[index + 1]]);
                        var entry = { type: "phasecollectionchanged", phase: phase, prevPhase: prevPhase, nextPhase: nextPhase, category: "internal", isAdded: true };
                        this.addHistoryEntry(entry);
                    }
                }
            }
        },
        _removePhase: function (name, needDelete) {
            var phase = this.nameTable[name];
            if (phase) {
                var parent = this.nameTable[phase.parent];
                if (parent && parent.phases && parent.phases.length > 1) {
                    var diagram = this;
                    this._views.forEach(function (viewid) {
                        var view = diagram._views[viewid];
                        var phaseEle = view.svg.getElementById(name);
                        if (phaseEle)
                            phaseEle.parentNode.removeChild(phaseEle);
                    });
                    //var phaseEle = this._svg.getElementById(name);
                    //if (phaseEle)
                    //    phaseEle.parentNode.removeChild(phaseEle);
                    if (!needDelete) {
                        var prevPhase = this._getPrevPhase(phase);
                        var prePhase = $.extend(true, {}, prevPhase);
                        var extPhase = prevPhase;
                        if (prevPhase == phase)
                            extPhase = this._getNextPhase(phase)
                        var extOffset = 0;
                        if (extPhase != phase) {
                            //#region remove from phase stack and add the offset of extentphase
                            if (this._getPhaseIndex(phase.name) != 0) {
                                extOffset = (phase.offset - extPhase.offset);
                                extPhase.offset += (phase.offset - extPhase.offset);
                            }
                            else {
                                extOffset = (extPhase.offset);
                            }
                            var delNode = this.nameTable["phaseStack" + name];
                            ej.datavisualization.Diagram.Util.removeItem(this.nodes(), delNode);
                            this._nodes = $.extend(true, [], this.nodes());
                            var stack = this.nameTable[delNode.parent];
                            if (stack) {
                                var extNode = this.nameTable["phaseStack" + extPhase.name];
                                if (extNode) {
                                    if (this._getPhaseIndex(phase.name) === 0) {
                                        if (phase.orientation === "horizontal")
                                            extNode.width = extOffset;
                                        else {
                                            var header = this._getSwimlaneHeader(this.nameTable[phase.parent]);
                                            extNode.height = extOffset;
                                            if (header)
                                                extNode.height -= header.height;
                                        }
                                    }
                                    else if (this._getPhaseIndex(phase.name) != (parent.phases.length - 1)) {
                                        if (phase.orientation === "horizontal")
                                            extNode.width += (extOffset);
                                        else
                                            extNode.height += (extOffset);
                                    }
                                    else {
                                        if (phase.orientation === "horizontal")
                                            extNode.width += extOffset;
                                        else
                                            extNode.height += extOffset;
                                    }
                                }
                                if (phase) {
                                    this._deletePhase(name, parent);
                                    delete this.nameTable[name];
                                }
                                ej.datavisualization.Diagram.SvgContext._updatephase(extPhase, this);
                                ej.datavisualization.Diagram.Util.removeItem(parent.phases, phase.name);
                                ej.datavisualization.Diagram.Util.removeItem(stack.children, delNode.name);
                                ej.datavisualization.Diagram.SvgContext._alignOnStackContainer(stack, this);
                                if (this._getPhaseIndex(prePhase.name) > -1)
                                    var nxPhase = this.nameTable[parent.phases[this._getPhaseIndex(prePhase.name) + 1]];
                                if (!this._isUndo) {
                                    var entry = { type: "phasecollectionchanged", phase: phase, prevPhase: prePhase, nextPhase: $.extend(true, {}, nxPhase), category: "internal", isAdded: false };
                                    this.addHistoryEntry(entry);
                                }
                                var diagram = this;
                                this._views.forEach(function (viewid) {
                                    var view = diagram._views[viewid];
                                    var element = view.svg.getElementById(delNode.name);
                                    var htmlLayer = view.svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                                    var label = $(htmlLayer).find("#" + delNode.name)[0];
                                    if (element && element.parentNode)
                                        element.parentNode.removeChild(element);
                                    if (label)
                                        htmlLayer.removeChild(label);
                                });
                            }
                            ej.datavisualization.Diagram.DiagramContext.update(stack, this);
                            //#endregion
                        }
                    }
                }
            }
        },
        _deletePhase: function (child, parent) {
            for (var i = 0; i < this.model.phases.length ; i++) {
                if (this.model.phases[i] && this.model.phases[i].name === child)
                    this.model.phases.splice(i, 1);
                else if (this.model.phases[i] && this.model.phases[i].name === "phaseStack" + child)
                    this.model.phases.splice(i, 1);
            }
            if (parent.phases && parent.phases.length > 0) {
                for (var i in parent.phases) {
                    if (parent.phases[i] && parent.phases[i] === child) {
                        parent.phases.splice(i, 1);
                    }
                }
            }
        },
        _getEdgeTable: function (group, edgeTable) {
            var edges, len, i, j, depend;
            if (!group.segments) {
                if (group._type === "group") {
                    var children = this._getChildren(group.children);
                    for (i = 0; i < children.length; i++) {
                        var node = this.nameTable[children[i]];
                        if (node && node.inEdges && node.inEdges.length > 0) {
                            edges = node.inEdges;
                            for (j = 0, len = edges.length; j < len; j++) {
                                depend = this.nameTable[edges[j]];
                                if (depend)
                                    var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                            }
                        }
                        if (node && node.outEdges && node.outEdges.length > 0) {
                            edges = node.outEdges;
                            for (j = 0, len = edges.length; j < len; j++) {
                                depend = this.nameTable[edges[j]];
                                if (depend)
                                    var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                            }
                        }
                    }
                }
                if (group.inEdges && group.inEdges.length > 0) {
                    edges = group.inEdges;
                    for (j = 0, len = edges.length; j < len; j++) {
                        depend = this.nameTable[edges[j]];
                        if (depend)
                            var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                    }
                }
                if (group.outEdges && group.outEdges.length > 0) {
                    edges = group.outEdges;
                    for (j = 0, len = edges.length; j < len; j++) {
                        depend = this.nameTable[edges[j]];
                        if (depend)
                            var table = edgeTable[depend.name] = $.extend(true, {}, depend);
                    }
                }
            }
            return edgeTable;
        },
        _remove: function (obj, dependent, isClear) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this)) {
                if (obj) {
                    var type;
                    if (obj.segments) type = "connector";
                    else type = "node";
                    var dependent = dependent ? ej.datavisualization.Diagram.dependentconnector.dependent : ej.datavisualization.Diagram.dependentconnector.independent;
                    var args = { changeType: "remove", element: this.getNode(obj), cancel: false, dependent: dependent };
                    if (!obj.segments) { args.adjustDependent = true; args.deleteDependent = true; }
                    if (!obj.isPhase && !this._selectedSymbol)
                        this._raiseEvent(type + "CollectionChange", args);
                    if (!args.cancel) {
                        if (ej.datavisualization.Diagram.Util.canDelete(obj) || isClear) {
                            var name = obj.name;
                            if (obj.type == "group" && obj.type != "pseudoGroup") {
                                var children = this._getChildren(obj.children);
                                for (var i = 0; i < children.length; i++) {
                                    var child = this.nameTable[children[i]];
                                    if (child && !ej.datavisualization.Diagram.Util.canDelete(child)) return;
                                }
                            }
                            if (obj.isSwimlane) {
                                if (obj.phases && obj.phases.length > 0) {
                                    for (i = obj.phases.length - 1 ; i >= 0; i--) {
                                        this._removePhase(obj.phases[i], true);
                                    }
                                }
                            }
                            if (!obj.segments) {
                                this._disConnect(obj, args);
                                //if (obj.type != "group")
                                this._removeConnector(obj, args);
                                ej.datavisualization.Diagram.Util.removeItem(this.nodes(), this.nameTable[obj.name]);
                                this._nodes = $.extend(true, [], this.nodes());
                            } else {
                                this._removeEdges(obj);
                                obj.sourceNode = obj.sourcePort = obj.targetNode = obj.targetPort = null;
                                ej.datavisualization.Diagram.Util.removeItem(this.connectors(), this.nameTable[obj.name]);
                                this._connectors = $.extend(true, [], this.connectors());
                            }
                            type = this.getObjectType(obj);
                            ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(this._spatialSearch, this._spatialSearch.quadTable[obj.name], obj);
                            ej.datavisualization.Diagram.SpatialUtil._updateBounds(this, this._spatialSearch, obj);
                            if (obj.parent) {
                                var parent = this.nameTable[obj.parent];
                                if (parent && parent.type == "group") {
                                    ej.datavisualization.Diagram.Util.removeItem(parent.children, obj.name);
                                    ej.datavisualization.Diagram.Util._refreshParentGroup(obj, this);
                                    this._updateQuad(parent);
                                }
                            }
                            delete this.nameTable[name];
                            if (obj.segments)
                                this._updateConnectorBridging(obj);
                            var element = this._svg.getElementById(obj.name);
                            if (obj._type == "group" && obj.type != "pseudoGroup") {
                                this._removeChildren(obj, args);
                            }
                            this._removeElement(obj);
                            if (obj._type == "group" && obj.container) {
                                ej.datavisualization.Diagram.SvgContext._removephases(obj, this);
                            }
                            if (element && element.parentNode)
                                element.parentNode.removeChild(element);
                            ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.activeTool.selectedObject, this._adornerSvg, this._adornerLayer);
                            this._clearSelection(this._selectedSymbol ? true : false);
                        }
                    }
                }
            }
        },
        _removeConnector: function (node, args) {
            var edges;
            var connector;
            var i, len;
            if (!node.segments) {
                //If adjustDependent is false all connector associated with Nodes get deleted.
                if (args.deleteDependent && !(args.adjustDependent)) {
                    if (node.inEdges && node.inEdges.length > 0) {
                        edges = node.inEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    if (node.outEdges && node.outEdges.length > 0) {
                        edges = node.outEdges;
                        for (i = 0, len = edges.length; i < len; i++) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                }
                //If both deleteDependent & adjustDependent are true changes to Visio behaviour
                if (args.deleteDependent && args.adjustDependent) {
                    if (node.inEdges && node.inEdges.length > 0 && node.outEdges && node.outEdges.length > 0) {
                        edges = node.outEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                        edges = node.inEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            if (this.nameTable[edges[i]])
                                this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    else if (node.inEdges && node.inEdges.length == 0 && node.outEdges.length > 0) {
                        edges = node.outEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    else if (node.outEdges && node.outEdges.length == 0 && node.inEdges.length > 0) {
                        edges = node.inEdges;
                        for (i = edges.length - 1 ; i >= 0; i--) {
                            //Remove the Connector
                            if (this.nameTable[edges[i]])
                                this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    else if ((node.inEdges && node.inEdges.length == 1 || node.inEdges.length > 1) && (node.outEdges.length == 1)) {
                        edges = node.outEdges;
                        for (i = 0, len = edges.length; i < len; i++) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                    else if (!(node.inEdges && node.inEdges.length > 1 && node.outEdges.length > 1)) {
                        edges = node.inEdges;
                        for (i = 0, len = edges.length; i < len; i++) {
                            //Remove the Connector
                            this._updateConnection(this.nameTable[edges[i]], true);
                        }
                    }
                }
            }
        },
        _updateConnection: function (connector, args) {
            if (connector && ej.datavisualization.Diagram.Util.canDelete(connector)) {
                this._remove(connector, args);
                this.updateConnector(connector.name);
            }
        },
        _removeElement: function (element) {
            for (var i = 0; i < this._views.length; i++) {
                var view = this._views[i];
                this._views[view].context.removeChild(element, this._views[view]);
            }
        },
        _disConnect: function (node, args, isChild) {
            var edgesIn, edgesOut, connector, parentNode, targetNode, data;
            var i, j, leni, lenj, canDelete;
            if (node.inEdges && node.inEdges.length > 0) {
                edgesIn = node.inEdges;
                for (i = 0, leni = edgesIn.length; i < leni; i++) {
                    if (ej.datavisualization.Diagram.Util.canDelete(this.nameTable[edgesIn[i]]))
                        canDelete = true;
                }
            }
            if (args.deleteDependent && args.adjustDependent && !isChild && canDelete) {
                if (node.inEdges && node.inEdges.length > 0 && node.outEdges.length > 0) {
                    edgesIn = node.inEdges;
                    edgesOut = node.outEdges;
                    if ((node.inEdges.length > 1 || node.inEdges.length == 1) && node.outEdges.length == 1) {
                        for (i = 0, leni = edgesIn.length; i < leni; i++) {
                            if (this.nameTable[edgesIn[i]] && this.nameTable[edgesOut[0]]) {
                                if ((this.nameTable[edgesIn[i]].targetNode == this.nameTable[edgesOut[0]].sourceNode)) {
                                    targetNode = this.nameTable[this.nameTable[edgesOut[0]].targetNode];
                                    if (targetNode) {
                                        data = $.extend(true, {}, targetNode);
                                        if (this.nameTable[edgesIn[i]].sourceNode != this.nameTable[edgesOut[0]].targetNode) {
                                            this.nameTable[edgesIn[i]].targetNode = this.nameTable[edgesOut[0]].targetNode;
                                            this.nameTable[edgesIn[i]].targetPort = this.nameTable[edgesOut[0]].targetPort;
                                            targetNode.inEdges[i] = this.nameTable[edgesIn[i]].name;
                                        }
                                    }
                                    //Updating the Connector Dock and Connector
                                    ej.datavisualization.Diagram.Util.dock(this.nameTable[edgesIn[i]], this.nameTable);
                                    ej.datavisualization.Diagram.DiagramContext.update(this.nameTable[edgesIn[i]], this);
                                }
                            }
                        }
                        if (this.nameTable[edgesOut[0]]) {
                            this.nameTable[edgesOut[0]].sourceNode = null;
                            this.nameTable[edgesOut[0]].sourcePort = null;
                        }
                    }
                    else if (!(node.inEdges.length > 1 && node.outEdges.length > 1)) {
                        // Need to code when many in and only one out 
                        for (i = 0, leni = edgesIn.length; i < leni; i++) {
                            if (node.outEdges.length > 0) {
                                if (this.nameTable[edgesIn[i]]) {
                                    parentNode = this.nameTable[this.nameTable[edgesIn[i]].sourceNode];
                                    data = $.extend(true, {}, parentNode);
                                    for (j = 0, lenj = edgesOut.length; j < lenj; j++) {
                                        if ((this.nameTable[edgesIn[i]].targetNode == this.nameTable[edgesOut[j]].sourceNode)) {
                                            this.nameTable[edgesOut[j]].sourceNode = this.nameTable[edgesIn[i]].sourceNode;
                                            this.nameTable[edgesOut[j]].sourcePort = this.nameTable[edgesIn[i]].sourcePort;
                                            parentNode.outEdges[data.outEdges.length + j] = this.nameTable[edgesOut[j]].name;
                                            //Updating the Connector Dock and Connector
                                            ej.datavisualization.Diagram.Util.dock(this.nameTable[edgesOut[j]], this.nameTable);
                                            ej.datavisualization.Diagram.DiagramContext.update(this.nameTable[edgesOut[j]], this);
                                        }
                                    }
                                    this.nameTable[edgesIn[i]].targetNode = null;
                                    this.nameTable[edgesIn[i]].targetPort = null;
                                }
                            }
                        }
                    }
                } return;
            }
            if (node.inEdges && node.inEdges.length > 0) {
                edgesIn = node.inEdges;
                for (i = 0, leni = edgesIn.length; i < leni; i++) {
                    connector = this.nameTable[edgesIn[i]];
                    if (connector) {
                        connector.targetNode = null;
                        connector.targetPort = null;
                    }
                }
            }
            if (node.outEdges && node.outEdges.length > 0) {
                edgesOut = node.outEdges;
                for (i = 0, leni = edgesOut.length; i < leni; i++) {
                    connector = this.nameTable[edgesOut[i]];
                    if (connector) {
                        connector.sourceNode = null;
                        connector.sourcePort = null;
                    }
                }
            }
        },
        _removeEdges: function (connector) {
            if (connector.targetNode) {
                if (this.nameTable[connector.targetNode] && this.nameTable[connector.targetNode].inEdges)
                    ej.datavisualization.Diagram.Util.removeItem(this.nameTable[connector.targetNode].inEdges, connector.name);
            }
            if (connector.sourceNode) {
                if (this.nameTable[connector.sourceNode] && this.nameTable[connector.sourceNode].outEdges)
                    ej.datavisualization.Diagram.Util.removeItem(this.nameTable[connector.sourceNode].outEdges, connector.name);
            }
        },
        _selectionContains: function (node) {
            if (this._hasSelection() && node)
                for (var i = 0; i < this.selectionList.length; i++) {
                    if (node.name === this.selectionList[i].name)
                        return true;
                }
            return false;
        },
        _getPhaseIndex: function (name) {
            var phase, group, index = -1;
            phase = this.nameTable[name];
            if (phase && phase.parent) {
                group = this.nameTable[phase.parent];
                if (group.phases && group.phases.length > 0) {
                    index = group.phases.indexOf(name);
                }
            }
            return index;
        },
        _getNextPhase: function (phase) {
            var nxtPhase = null, index;
            var group = this.nameTable[phase.parent]
            if (group && group.phases && group.phases.length > 0) {
                index = this._getPhaseIndex(phase.name);
                if (index === group.phases.length - 1)
                    nxtPhase = this.nameTable[group.phases[index]];
                else
                    nxtPhase = this.nameTable[group.phases[index + 1]];
            }
            return nxtPhase;
        },
        _getPrevPhase: function (phase) {

            var prevPhase = null, index;
            var group = this.nameTable[phase.parent]
            if (group && group.phases && group.phases.length > 0) {
                index = this._getPhaseIndex(phase.name);
                if (index === 0)
                    prevPhase = this.nameTable[group.phases[0]];
                else
                    prevPhase = this.nameTable[group.phases[index - 1]];
            }
            return prevPhase;
        },
        _getPrevOffset: function (phase, prevPhase, nextPhase, parent, parentBounds) {
            var stckHeadWidth = 50, stack, firstCanvas, dleft = 0, dtop = 0;
            if (parent.children[0]) {
                var head = this.nameTable[this._getChild(parent.children[0])];
                if (head) {
                    dtop = head.height;
                }
            }
            if (parent.orientation == "horizontal") {
                if (parent.children[2])
                    stack = this.nameTable[this._getChild(parent.children[2])];
                if (stack && stack.children && stack.children.length > 0) {
                    firstCanvas = this.nameTable[this._getChild(stack.children[0])];
                    if (firstCanvas && firstCanvas.children && firstCanvas.children.length > 0) {
                        var _firstchild = this.nameTable[this._getChild(firstCanvas.children[0])]
                        if (_firstchild && _firstchild.name.indexOf("_Headerr") != -1) {
                            dleft = _firstchild.width;
                        }
                    }
                }
            }
            return { left: dleft, top: dtop };
        },
        _getPhaseBounds: function (phase) {
            var bounds = null, group, phaseBounds, prevOffset, left = 0, right = 0, top = 0, bottom = 0;
            if (phase && phase.type == "phase" && phase.parent) {
                var prevPhase = this._getPrevPhase(phase);
                var nextPhase = this._getNextPhase(phase)
                group = this.nameTable[phase.parent];
                if (group && group.orientation) {
                    bounds = ej.datavisualization.Diagram.Util.bounds(group)
                    if (group.orientation == "horizontal") {
                        prevOffset = this._getPrevOffset(phase, prevPhase, nextPhase, group, bounds);
                        if (prevPhase && prevPhase.name === phase.name) {
                            left = prevOffset.left + bounds.left;
                            right = left + (phase.offset - left - prevOffset.left);
                        }
                        else {
                            if (prevPhase)
                                left = bounds.left + prevPhase.offset;
                            right = (phase.offset - left + bounds.left);
                        }
                        top = prevOffset.top + bounds.top;
                        bottom = bounds.bottom - top;
                        phaseBounds = ej.datavisualization.Diagram.Rectangle(left, top, right, bottom);
                    }
                    else {
                        prevOffset = this._getPrevOffset(phase, prevPhase, nextPhase, group, bounds);
                        if (prevPhase === phase) {
                            var head = (typeof (group.children[0]) === "string") ? this.nameTable[group.children[0]] : this.nameTable[group.children[0].name];
                            left = prevOffset.left + bounds.left;
                            bottom = phase.offset + top;
                            if (head)
                                bottom -= head.height;
                            top = prevOffset.top + bounds.top;
                        }
                        else {
                            left = prevOffset.left + bounds.left;
                            if (prevPhase)
                                top = (prevPhase.offset + bounds.top);
                            bottom = bounds.top + phase.offset - top;
                        }

                        right = bounds.right - left;
                        phaseBounds = ej.datavisualization.Diagram.Rectangle(left, top, right, bottom);
                    }
                }
            }
            return phaseBounds;
        },
        _addSelection: function (selectedShape, preventEvent) {
            if (ej.datavisualization.Diagram.Util.isPageEditable(this) && (ej.datavisualization.Diagram.Util.canDoSingleSelection(this) || (selectedShape && selectedShape._type === "pseudoGroup" || this._selectedSymbol))) {
                if (selectedShape && (ej.datavisualization.Diagram.Util.canSelect(selectedShape) || selectedShape.type == "phase")) {
                    if (!this._selectionContains(selectedShape)) {
                        var newItems = [], oldItems = [], selectedItems = [];
                        if (this._previousSelectedItems) oldItems = this._previousSelectedItems;
                        if (selectedShape.type == "pseudoGroup") {
                            var children = this._getChildren(selectedShape.children);
                            for (var i = 0; i < children.length; i++) {
                                var child = this.nameTable[children[i]];
                                if (oldItems.indexOf(child) == -1) newItems.push(child);
                                else ej.datavisualization.Diagram.Util.removeItem(oldItems, child);
                                selectedItems.push(child);
                            }
                        }
                        else {
                            newItems.push(selectedShape);
                            selectedItems.push(selectedShape);
                        }
                        var args = { changeType: "insert", element: this.getNode(selectedShape), selectedItems: selectedItems, oldItems: oldItems, newItems: newItems, cancel: false };
                        if ((!this._selectedSymbol || this._endEditing) && !preventEvent) {
                            this._raiseEvent("selectionChange", args);
                            this._previousSelectedItems = args.newItems;
                        }
                        if (!args.cancel) {
                            this.selectionList.push(selectedShape);
                            this._updateSelectorObject(selectedShape);
                            var bounds = this._getPhaseBounds(selectedShape);
                            ej.datavisualization.Diagram.SvgContext.renderSelector(selectedShape, this._adornerSvg, this._adornerLayer, this._currZoom,
                                this.model.selectedItems.constraints, bounds);
                            if ((this.model.selectedItems.userHandles != null && this.model.selectedItems.userHandles.length > 0)) {
                                var isMultipleSelection = false;
                                if (selectedShape.type == "pseudoGroup")
                                    isMultipleSelection = true;
                                if (this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                                    ej.datavisualization.Diagram.SvgContext.renderUserHandles(this.model.selectedItems.userHandles, selectedShape, this._adornerSvg, isMultipleSelection, this._currZoom, this._adornerLayer, this);
                            }
                        }
                    }
                }
            }
        },
        _getInsertIndex: function (phase) {
            var index = -1, offset, cphase;
            var group = this.nameTable[phase.parent];
            if (group && group.phases && group.phases.length > 0) {
                if (this.nameTable[group.phases[0]]) {
                    offset = this.nameTable[group.phases[0]].offset;
                    for (var i = 0; i < group.phases.length; i++) {
                        cphase = this.nameTable[group.phases[i]];
                        if (cphase && cphase.offset <= phase.offset && cphase.offset >= offset) {
                            offset = cphase.offset;
                            index = i;
                        }
                    }
                }
            }
            return index;
        },
        _updateInsertPhase: function (nphase, ophase, group, index) {
            var oldPhase = this.nameTable[ophase];
            var phaseStack = this.nameTable[this._getChild(group.children[1])];
            if (phaseStack && phaseStack.children && phaseStack.children.length > 0) {
                var oldChild = this.nameTable["phaseStack" + ophase];
                if (group.isSwimlane && group.orientation && group.orientation == "vertical") {
                    var h = oldChild.height;
                    oldChild.height = oldPhase.offset - nphase.offset;
                    var dH = h - oldChild.height;
                    var newStackChild = ej.datavisualization.Diagram.Node({ parent: phaseStack.name, name: "phaseStack" + nphase.name, height: dH, width: oldChild.width });
                }
                else {
                    var w = oldChild.width;
                    oldChild.width = oldPhase.offset - nphase.offset;
                    var dW = w - oldChild.width;
                    var newStackChild = ej.datavisualization.Diagram.Node({ parent: phaseStack.name, name: "phaseStack" + nphase.name, height: oldChild.height, width: dW, isPhase: true });
                }
                newStackChild.labels = [(nphase.label ? nphase.label : ej.datavisualization.Diagram.Label())];
                newStackChild.labels[0].rotateAngle = (nphase.orientation == "horizontal") ? 0 : 270;
                newStackChild.constraints = ej.datavisualization.Diagram.NodeConstraints.Default & ~(ej.datavisualization.Diagram.NodeConstraints.Select | ej.datavisualization.Diagram.NodeConstraints.Connect);
                newStackChild.isPhase = true;
                this.add(newStackChild, { entryHistory: true });
                //#region swapping the element 
                phaseStack.children.pop(newStackChild.name);
                phaseStack.children.splice(index, 0, newStackChild.name);
                //#endregion 
                this.tools["move"]._updateContainer(newStackChild);
            }
        },
        _getFirstLane: function (group) {
            if (group.children[2])
                var stack = this.nameTable[this._getChild(group.children[2])];
            if (stack && stack.children && stack.children.length > 0) {
                var lane = this.nameTable[this._getChild(stack.children[0])]
                if (lane && lane.children && lane.children.length > 0) {
                    var laneHdr = this.nameTable[this._getChild(lane.children[0])];
                    if (laneHdr)
                        return laneHdr;
                }
            }
            return null;
        },
        updateSelector: function (option) {
            if (this._selectedItem != "") {
                var options = {};
                if (option.offsetX)
                    options.offsetX = Number(typeof option.offsetX === 'function' ? option.offsetX() : option.offsetX);
                if (option.offsetY)
                    options.offsetY = Number(typeof option.offsetY === 'function' ? option.offsetY() : option.offsetY);
                if (option.width)
                    options.width = Number(typeof option.width === 'function' ? option.width() : option.width);
                if (option.height)
                    options.height = Number(typeof option.height === 'function' ? option.height() : option.height);
                if (option.rotateAngle)
                    options.rotateAngle = Number(typeof option.rotateAngle === 'function' ? option.rotateAngle() : option.rotateAngle);
                this.updateNode(this._selectedItem, options);
            }
            if (option.userHandles) {
                this.model.selectedItems.userHandles = option.userHandles;
                this._initHandles();
                if (this.selectionList[0] && this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                    ej.datavisualization.Diagram.SvgContext.updateUserHandles(this.model.selectedItems.userHandles, this.selectionList[0], this._adornerSvg, this.selectionList[0].type == "pseudoGroup", false, this._currZoom, this);
            }
            if (option.tooltip) {
                this.model.selectedItems.tooltip = ej.datavisualization.Diagram.Tooltip($.extend(true, this.model.selectedItems.tooltip, {}, option.tooltip));
            }
        },
        _updateSelectorObject: function (selectedShape) {
            if (!selectedShape.segments) {
                this._selectorOffsetX(selectedShape.offsetX);
                this._selectorOffsetY(selectedShape.offsetY);
                this._selectorWidth(selectedShape.width);
                this._selectorHeight(selectedShape.height);
                this._selectorRotateAngle(selectedShape.rotateAngle);
                this._selectedItem = selectedShape.name;
            }
            if (selectedShape.type == "pseudoGroup") {
                for (var i = 0; i < selectedShape.children.length; i++) {
                    var child = selectedShape.children[i];
                    if (typeof (child) === "object") {
                        if (this.model.selectedItems.children.indexOf(child) < 0)
                            this.model.selectedItems.children.push(child);
                    }
                    else if (typeof (child) === "string") {
                        if (this.model.selectedItems.children.indexOf(this.nameTable[child]) < 0)
                            this.model.selectedItems.children.push(this.nameTable[child]);
                    }
                }
            }
            else {
                if (this.model.selectedItems.children.indexOf(selectedShape) < 0)
                    this.model.selectedItems.children.push(selectedShape);
            }
        },
        _clearSelectorObject: function () {
            this._selectorOffsetX(0);
            this._selectorOffsetY(0);
            this._selectorWidth(0);
            this._selectorHeight(0);
            this._selectorRotateAngle(0);
            this._selectedItem = "";
            this.model.selectedItems.children = [];
        },
        _raiseEvent: function (type, args) {
            if (this.model[type]) {
                args.elementType = this.getObjectType(args.element);
                return this._trigger(type, args);
            }
        },
        _clearSelection: function (preventEvent) {
            if (this._hasSelection()) {
                if (this._isEditing && this.selectionList.length > 0 && !this._isDragg) {
                    this._endEdit();
                }
                var args = { changeType: "remove", element: null, selectedItems: [], oldItems: this.model.selectedItems.children, newItems: [], cancel: false };
                if (!this._selectedSymbol && !preventEvent) {
                    this._raiseEvent("selectionChange", args);
                    this._previousSelectedItems = [];
                }
                if (!args.cancel) {
                    this._clearSelectorObject();
                    if (this.selectionList[0] && this.selectionList[0].type == "pseudoGroup") {
                        ej.datavisualization.Diagram.Util.removeItem(this.nodes(), this.selectionList[0]);
                        this._nodes = $.extend(true, [], this.nodes());
                        delete this.nameTable[this.selectionList[0].name];
                    }
                    ej.datavisualization.Diagram.Util.clear(this.selectionList);
                    ej.datavisualization.Diagram.SvgContext.clearSelector(this._adornerSvg, this._adornerLayer);
                }
            }
        },
        _updateSelectionHandle: function (isDragging) {
            if (this.selectionList.length > 0) {
                var shape = this.nameTable[this.selectionList[0].name];
                var constraints = this.model.selectedItems.constraints;
                if (isDragging && (this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles))
                    constraints = constraints & ~ej.datavisualization.Diagram.SelectorConstraints.UserHandles;
                this._updateSelectorObject(shape);
                var updated = ej.datavisualization.Diagram.SvgContext.updateSelector(shape, this._adornerSvg, this._currZoom, this, constraints);
                if (!updated) {
                    if ((this.model.selectedItems.userHandles != null && this.model.selectedItems.userHandles.length > 0)) {
                        var isMultipleSelection = false;
                        if (shape.type == "pseudoGroup")
                            isMultipleSelection = true;
                        if (this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                            ej.datavisualization.Diagram.SvgContext.updateUserHandles(this.model.selectedItems.userHandles, shape, this._adornerSvg, isMultipleSelection, isDragging, this._currZoom, this);
                    }
                }
            }
        },
        _hasSelection: function () {
            return this.selectionList.length > 0;
        },
        _checkToolToActivate: function (evt, node) {
            var success = false;
            var pseudoGroupConstraints;
            pseudoGroupConstraints = ej.datavisualization.Diagram.Util._getPseudoGroupConstraints(this, node, this.selectionList[0]);
            if (node) {
                if (node.segments && ((navigator.platform.match("Mac") ? evt.metaKey : evt.ctrlKey) && evt.shiftKey) || ((navigator.platform.match("Mac") ? evt.metaKey : evt.ctrlKey) && evt.altKey)) {
                    this._toolToActivate = "endPoint";
                    this._currentCursor = "move";
                    success = true;
                }
                if (!success) {
                    if (this._selectionContains(node)) {
                        if (node.segments) {
                            success = this._canActivateEndPointTool(evt, node);
                        }
                        else {
                            if (!pseudoGroupConstraints || pseudoGroupConstraints & ej.datavisualization.Diagram.NodeConstraints.Resize)
                                success = this._canActivateResizeTool(evt, node);
                            if (!success) {
                                if ((!pseudoGroupConstraints || (pseudoGroupConstraints & ej.datavisualization.Diagram.NodeConstraints.Rotate)))
                                    success = this._canActivateRotateTool(evt, node);
                            }
                        }
                        if (!success) {
                            success = this._canActivateUserHandle(evt, node);
                        }
                        if (!success) {
                            if (!pseudoGroupConstraints || pseudoGroupConstraints & ej.datavisualization.Diagram.NodeConstraints.Drag)
                                if (node && node.isLane && ej.datavisualization.Diagram.Util.canMultiSelectOnLane(node)) {
                                    this._toolToActivate = "select";
                                    this._currentCursor = "default";
                                }
                                else
                                    success = this._canActivateMoveTool(evt, node);
                        }
                        if (!success) {
                            success = this._canActivatephaseTool(evt);
                        }
                    }
                    else {
                        if (!pseudoGroupConstraints || pseudoGroupConstraints & ej.datavisualization.Diagram.NodeConstraints.Drag)
                            success = this._canActivateMoveTool(evt, node);
                    }
                }
            }
            else {
                success = this._canActivatephaseTool(evt);
            }
            if (!success) {
                if (this.tool() & ej.datavisualization.Diagram.Tool.ZoomPan) {
                    this._toolToActivate = "panTool";
                    this._currentCursor = "pointer";
                }
                else {
                    this._toolToActivate = "select";
                    this._currentCursor = "default";
                }
            }
            if (this._toolToActivate != "rotate") {
                this._adornerSvg.document.removeAttribute("class");
            }
            return success;
        },
        _canActivatephaseTool: function (evt) {
            if ($(evt.target).parents(".ej-d-seperator").first()[0]) {
                var id = $(evt.target).parents("g").first()[0].id;
                var phase = this._getSeperetor(id);
                if (phase) {
                    if (phase.orientation == "horizontal")
                        this._currentCursor = "e-resize";
                    else
                        this._currentCursor = "n-resize";
                }
                else
                    this._currentCursor = "default";
                this._toolToActivate = "phase";
                return true;
            }
        },
        _canActivateUserHandle: function (evt) {
            if (this._isUserHandle(evt)) {
                var handleName = $(evt.target).parents("g").first().context.id.split("_")[0];
                this._toolToActivate = handleName;
                this._currentCursor = this.tools[this._toolToActivate].cursor;
                return true;
            }
            return false;
        },
        _canActivateEndPointTool: function (evt, connector) {
            if (connector) {
                var endPointHandle = evt.target.getAttribute("class");
                if (endPointHandle && ((endPointHandle === "targetEndPoint" && ej.datavisualization.Diagram.Util.canDragTargetEnd(connector)) ||
                    (endPointHandle === "sourceEndPoint" && ej.datavisualization.Diagram.Util.canDragSourceEnd(connector)) ||
                (endPointHandle.match("bezierpoint") || endPointHandle == "segmentEnd") && ej.datavisualization.Diagram.Util.canDragSegmentThumbs(connector))
                    ) {
                    this._toolToActivate = "endPoint";
                    this._currentCursor = "move";
                    return true;
                }
            }
            return false;
        },
        _canActivateResizeTool: function (evt, node) {
            var dir = evt.target.getAttribute("class");
            if (ej.datavisualization.Diagram.Util.canResize(node, dir) && (this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.Resizer)) {
                if (dir === "nw-resize" || dir === "n-resize" || dir === "ne-resize" || dir === "w-resize" || dir === "e-resize" || dir === "sw-resize" || dir === "s-resize" || dir === "se-resize") {
                    this._toolToActivate = "resize";
                    if ((node.rotateAngle >= 0 && node.rotateAngle < 25) || (node.rotateAngle >= 160 && node.rotateAngle <= 200) || (node.rotateAngle >= 340 && node.rotateAngle <= 360)) {
                        this._currentCursor = dir;
                    }
                    else if ((node.rotateAngle >= 25 && node.rotateAngle <= 65) || (node.rotateAngle >= 205 && node.rotateAngle <= 240)) {

                        if (dir === "n-resize" || dir === "s-resize") {
                            this._currentCursor = "ne-resize";
                        }
                        else if (dir === "nw-resize" || dir === "se-resize") {
                            this._currentCursor = "n-resize";
                        }
                        else if (dir === "e-resize" || dir === "w-resize") {
                            this._currentCursor = "nw-resize";
                        }
                        else {
                            this._currentCursor = "e-resize";
                        }
                    }
                    else if ((node.rotateAngle >= 70 && node.rotateAngle <= 110) || (node.rotateAngle >= 250 && node.rotateAngle <= 290)) {
                        if (dir === "n-resize" || dir === "s-resize") {
                            this._currentCursor = "e-resize";
                        }
                        else if (dir === "nw-resize" || dir === "se-resize") {
                            this._currentCursor = "ne-resize";
                        }
                        else if (dir === "e-resize" || dir === "w-resize") {
                            this._currentCursor = "n-resize";
                        }
                        else {
                            this._currentCursor = "nw-resize";
                        }
                    }
                    else if ((node.rotateAngle >= 115 && node.rotateAngle <= 155) || (node.rotateAngle >= 295 && node.rotateAngle <= 335)) {
                        if (dir === "n-resize" || dir === "s-resize") {
                            this._currentCursor = "nw-resize";
                        }
                        else if (dir === "nw-resize" || dir === "se-resize") {
                            this._currentCursor = "e-resize";
                        }
                        else if (dir === "e-resize" || dir === "w-resize") {
                            this._currentCursor = "ne-resize";
                        }
                        else {
                            this._currentCursor = "n-resize";
                        }
                    }
                    return true;
                }
            }
            return false;
        },
        _canActivateMoveTool: function (evt, node) {
            var findNode = this._findNodeUnderMouse(evt);
            if (findNode && findNode.name === node.name) {
                var canMove = true;
                if (node.parent) {
                    if (this.selectionList.length > 0 && this._selectionContains(node))
                        canMove = true;
                    else
                        canMove = false;
                }
                if (canMove && ej.datavisualization.Diagram.Util.canMoveLabel(node)) {
                    var id = evt.target.parentNode.id.split('_');
                    if (id[id.length - 1] == "lblbg") {
                        this._toolToActivate = "move";
                        this._currentCursor = "pointer";
                        return true;
                    }
                }
                if (ej.datavisualization.Diagram.Util.canMove(node)) {
                    this._toolToActivate = "move";
                    if (this.selectionList.length > 0 && (this._selectionContains(node) || (this.selectionList[0].children && this._findChildren(this.selectionList[0], node.name)))) {
                        this._currentCursor = "move";
                    }
                    else {
                        this._currentCursor = "default";
                    }
                    return true;
                }
            }
            return false;
        },
        _canActivateRotateTool: function (evt, node) {
            if (ej.datavisualization.Diagram.Util.canRotate(node) && (this.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.Rotator)) {
                var rotateHandle = evt.target.getAttribute("class");
                if (rotateHandle === "rotate") {
                    this._toolToActivate = "rotate";
                    if (ej.browserInfo().name === "msie") {
                        this._currentcursor = "default";
                        this._adornerSvg.document.setAttribute("class", "svg-rotate-ie");
                        this._currentCursor = "";
                    }
                    else {
                        this._adornerSvg.document.setAttribute("class", "svg-rotate");
                        this._currentCursor = "";
                    }
                    return true;
                }
            }
            return false;
        },
        _canActivatePivotTool: function (evt) {
            var pivotHandle = evt.target.getAttribute("class");
            if (pivotHandle === "pivot") {
                this._toolToActivate = "pivot";
                this._currentCursor = "default";
                return true;
            }
            return false;
        },
        _updateCursor: function () {
            this.element[0].style.cursor = this._currentCursor;
            this._svg.document.parentNode.getElementsByClassName("htmlLayer")[0].style.cursor = this._currentCursor;
        },
        _getResizeCursor: function (resizeDirection) {
            var cursor = "default";
            switch (resizeDirection) {
                case "topleft": cursor = "nw-resize";
                    break;
                case "topcenter": cursor = "n-resize";
                    break;
                case "topright": cursor = "ne-resize";
                    break;
                case "middleleft": cursor = "w-resize";
                    break;
                case "middleright": cursor = "e-resize";
                    break;
                case "bottomleft": cursor = "sw-resize";
                    break;
                case "bottomcenter": cursor = "s-resize";
                    break;
                case "bottomright": cursor = "se-resize";
                    break;
            }
            return cursor;
        },
        _startEdit: function (shape) {
            var label;
            var centerX, centerY;
            var bbox;
            var nodeBounds;
            var bounds;
            var curZoomfactor;
            var matrix = ej.Matrix.identity();
            var width = shape.width ? shape.width : shape._width;
            var height = shape.height ? shape.height : shape._height;
            ej.Matrix.rotate(matrix, shape.rotateAngle, width / 2, height / 2);
            if (shape.type === "text") {
                if (this._svg.getElementById(shape.name + "_shape") !== null)
                    bbox = this._svg.getElementById(shape.name + "_shape").getBBox();
                else
                    bbox = { x: shape.offsetX - width * shape.pivot.x, y: shape.offsetY - height * shape.pivot.y, width: width, height: height };
                label = shape.textBlock;
                if (label && !label.readOnly) {
                    nodeBounds = ej.datavisualization.Diagram.Util.bounds(shape);
                    bounds = new ej.datavisualization.Diagram.Rectangle(bbox.x - label.fontSize, bbox.y - label.fontSize, bbox.width + label.fontSize / 2, bbox.height + label.fontSize / 2);
                    curZoomfactor = this._currZoom;
                    var diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds));
                    centerX = shape.offsetX - width * shape.pivot.x + diff.x;
                    centerY = shape.offsetY - height * shape.pivot.y + diff.y;
                    if (label.horizontalAlignment == "left")
                        centerX += bbox.width / 2;
                    else if (label.horizontalAlignment == "right")
                        centerX -= bbox.width / 2;
                    if (label.verticalAlignment == "top")
                        centerY += bbox.height / 2;
                    else if (label.verticalAlignment == "bottom")
                        centerY -= bbox.height / 2;
                    if (bbox.width === 0 || bbox.height === 0) {
                        bbox = ej.datavisualization.Diagram.Util.bounds(shape);
                        bounds.width = bbox.width == 0 ? 50 : nodeBounds.width * curZoomfactor;
                        bounds.x = (centerX * curZoomfactor - bounds.width / 2 * curZoomfactor) - this._hScrollOffset;

                        bounds.height = label.fontSize;
                        bounds.y = (centerY * curZoomfactor - bounds.height / 2 * curZoomfactor) - this._vScrollOffset;
                    }
                    else {
                        bounds.width = 50;
                        bounds.height = label.fontSize;
                        bounds.x = (centerX * curZoomfactor - (bounds.width / 2 * curZoomfactor)) - this._hScrollOffset;
                        bounds.y = (centerY * curZoomfactor - (bounds.height / 2 * curZoomfactor)) - this._vScrollOffset;
                    }
                    this._boundingBox = bounds;
                    this._createEditBox(label, bounds.x, bounds.y, bounds.width, bounds.height, shape);
                    this._isEditing = true;
                }
            }
            else {
                if (shape.labels.length > 0) {
                    label = this.activeTool.getLabelUnderMouse();
                    if (!label) label = shape.labels[0];
                }
                else {
                    var mNode = this._findNode(shape.name);
                    label = new ej.datavisualization.Diagram.Label({ "name": "label" + ej.datavisualization.Diagram.Util.randomId() });
                    shape.labels.push(label);
                    var parent = (mNode !== shape) ? mNode : null;
                    ej.datavisualization.Diagram.DiagramContext.addNodeLabel(shape, label, parent, this);
                    if (this.activeTool.name === "textTool")
                        this.activateTool.activeLabel = label;
                }
                if (label && !label.readOnly) {
                    var element = document.getElementById(shape.name + "_" + label.name);
                    if (element) {
                        nodeBounds = ej.datavisualization.Diagram.Util.bounds(shape);
                        bounds = new ej.datavisualization.Diagram.Rectangle(element.offsetLeft, element.offsetTop, element.offsetWidth, element.offsetHeight);
                        bbox = bounds;
                        curZoomfactor = this._currZoom;
                        var position = ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds);
                        if (shape.segments && typeof shape.offset != "object") {
                            bounds = element.getBoundingClientRect();
                            bounds = { x: bounds.left, y: bounds.top, width: bounds.width, height: bounds.height };
                            var controlBBox = this._controlBBox ? this._controlBBox : this._canvas.getBoundingClientRect();
                            centerX = (bounds.x + bounds.width / 2 - controlBBox.left + this._hScrollOffset) / this._currZoom;
                            centerY = (bounds.y + bounds.height / 2 - controlBBox.top + this._vScrollOffset) / this._currZoom;
                        }
                        else {
                            if (label.horizontalAlignment == "left")
                                position.x += bbox.width / 2;
                            else if (label.horizontalAlignment == "right")
                                position.x -= bbox.width / 2;
                            if (label.verticalAlignment == "top")
                                position.y += bbox.height / 2;
                            else if (label.verticalAlignment == "bottom")
                                position.y -= bbox.height / 2;
                            var diff = ej.Matrix.transform(matrix, position);
                            centerX = shape.offsetX - width * shape.pivot.x + diff.x;
                            centerY = shape.offsetY - height * shape.pivot.y + diff.y;
                        }
                        if (bbox.width === 0 || bbox.height === 0) {
                            bbox = ej.datavisualization.Diagram.Util.bounds(shape);
                            if (shape.segments) {
                                bounds.width = bbox.width < 50 ? 50 : nodeBounds.width * curZoomfactor;
                                bounds.x = (centerX * curZoomfactor - bounds.width / 2 * curZoomfactor) - this._hScrollOffset;
                            }
                            else {
                                bounds.width = bbox.width < 50 ? 50 : (nodeBounds.width * curZoomfactor);
                                bounds.x = (centerX * curZoomfactor - bounds.width / 2) - this._hScrollOffset;
                            }
                            bounds.height = label.fontSize;
                            bounds.y = (centerY * curZoomfactor - bounds.height / 2 * curZoomfactor) - this._vScrollOffset;
                        }
                        else {
                            bounds.x = (centerX * curZoomfactor - (bounds.width / 2 * curZoomfactor)) - this._hScrollOffset;
                            bounds.y = (centerY * curZoomfactor - (bounds.height / 2 * curZoomfactor)) - this._vScrollOffset;
                        }
                        this._boundingBox = bounds;
                        this._createEditBox(label, bounds.x, bounds.y, bounds.width, bounds.height, shape);
                        this._isEditing = true;
                    }
                    else {
                        this.activeTool.selectedObject = null;
                    }
                }
            }
        },
        _createEditBox: function (label, x, y, width, height, shape) {
            if (this._svg) {
                var htmlLayer = this._svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                if (htmlLayer) {
                    if (shape.type === "text")
                        var lblEle = $(htmlLayer).find("#" + shape.name + "_shape_lblbg")[0];
                    else
                        var lblEle = $(htmlLayer).find("#" + shape.name + "_" + label.name + "_lblbg")[0];
                    if (lblEle) {
                        lblEle.style.display = "none";
                    }
                }
            }
            var nodeBounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var div = document.createElement("div");

            var attr = {
                "id": this.element[0].id + "_editBoxDiv",
                "class": "edit",
                "style": "top:" + y + "px;left:" + x + "px;width:" + width + "px;height:" + height + "px;" +
                    "font-family:" + label.fontFamily + ";font-size:" + label.fontSize + "px;min-width: 50px;min-height:" + label.fontSize + "px;",
            };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            var textBox = document.createElement("textarea");
            var fontColor = label.fontColor ? label.fontColor.toLowerCase() : "black";
            fontColor = fontColor != "white" && fontColor != "ffffff" && fontColor != "rgb(255,255,255)" ? fontColor : "black";
            attr = {
                "id": this.element[0].id + "_editBox",
                "class": "editBox",
                "style": "font-size:" + label.fontSize + "px;font-family:" + label.fontFamily + ";min-height:"
                    + label.fontSize + "px;line-height:" + label.fontSize + "px;color:" + fontColor +
                    ";font-weight:" + (label.bold ? "bold" : "normal") + ";font-style:" + (label.italic ? "italic" : "normal") + ";text-decoration:" + (label.textDecoration),
                "nodeWidth": nodeBounds.width, "nodeHeight": nodeBounds.height, "fontSize": label.fontSize, "nodeOffsetX": shape.offsetX, "nodeOffsetY": shape.offsetY, "containerName": shape.name,
            };
            ej.datavisualization.Diagram.Util.attr(textBox, attr);
            textBox.value = label.text;
            div.appendChild(textBox);
            $(this._canvas).append(div);
            if (label.text !== "") {
                this._updateEditor(null, div, textBox, nodeBounds.width, label.fontSize);
            }
            this._on($(textBox), "focusout", this._editboxfocusout);
            this._on($(textBox), "keyup", this._editBoxKeyUp);
            this._on($(textBox), "input", this._editBoxTextChange);
            textBox.focus();
            textBox.select();
        },
        _editBoxKeyUp: function (evt) {
            this._updateLabelPosition(evt);
        },
        _editBoxTextChange: function (evt) {
            this._updateLabelPosition(evt);
        },
        _updateEditBox: function (x, y) {
            var editBox = document.getElementById(this.element[0].id + "_editBoxDiv");
            if (editBox) {
                editBox.style.left = editBox.offsetLeft - x + "px";
                editBox.style.top = editBox.offsetTop - y + "px";
            }
        },
        _editboxfocusout: function () {
            this._endEditing = true;
            this._endEdit();
            delete this._endEditing;
        },
        _updateLabelPosition: function (evt) {
            var minWidth = 50;
            var curZoomfactor = this._currZoom;
            var parentBoundary = this._boundingBox;
            var fontSize = Number(evt.target.getAttribute("fontSize"));
            var nodeWidth = Number(evt.target.getAttribute("nodeWidth"));
            var conCenterX = parentBoundary.x + (parentBoundary.width / 2);
            var conCenterY = parentBoundary.y + (parentBoundary.height / 2);
            var bounds = this._textLength(evt);
            var width;
            if (bounds.width === 0) {
                width = parentBoundary.width < minWidth ? minWidth : parentBoundary.width;
                evt.target.parentNode.style.width = width + "px";
                evt.target.parentNode.style.left = conCenterX - width / 2 + "px";
            }
            else {
                if (bounds.width <= (nodeWidth * curZoomfactor))
                    width = bounds.width < minWidth ? minWidth : bounds.width;
                else
                    width = nodeWidth < minWidth ? minWidth : (nodeWidth * curZoomfactor);
                evt.target.parentNode.style.width = width + "px";
                evt.target.parentNode.style.left = conCenterX - (width / 2) + "px";
            }
            if (bounds.height === 0) {
                evt.target.style.height = fontSize + "px";
                evt.target.parentNode.style.top = conCenterY - fontSize / 2 + 'px';
            }
            else {
                if (this._selectedItem) {
                    var node = this.nameTable[this._selectedItem];
                    if (node.type === "text") {
                        var label = this._currentLabel;
                    }
                }
                this._updateEditor(evt, evt.target.parentNode, evt.target, nodeWidth, fontSize, label);
            }
        },
        _updateEditor: function (evt, div, textBox, nodeWidth, fontSize, label) {
            var offsetX, offsetY;
            var elHeight;
            var bounds = this._textLength(evt, label);
            var curZoomfactor = this._currZoom;
            var minWidth = 50;
            var x = this._boundingBox.x;
            var y = this._boundingBox.y;
            var width = this._boundingBox.width;
            var height = this._boundingBox.height;
            var str = textBox.value;
            var line = str.split('\n');
            var temp = (nodeWidth * curZoomfactor) < bounds.width ? (nodeWidth * curZoomfactor) : bounds.width;
            var elWidth = temp < minWidth ? minWidth : temp;
            div.style.width = elWidth + "px";
            if (evt)
                offsetX = (elWidth - width) / 2;
            else
                offsetX = (elWidth - width * curZoomfactor) / 2;
            div.style.left = x - offsetX - fontSize / 4 + "px";
            if (line.length > 1 || bounds.width != nodeWidth) {
                if (this._browserInfo.name === "mozilla")
                    elHeight = textBox.scrollHeight - 5;
                else
                    elHeight = textBox.scrollHeight - 4;
                textBox.style.height = 'auto';
            }
            else
                elHeight = fontSize;
            var offset = $(textBox).offset();
            var canvasOffset = $(this._canvas).offset();
            var canvasHeight = $(this._canvas).height();
            if (!canvasHeight)
                canvasHeight = this._getCanvasHeight(this._canvas, canvasHeight);
            if (offset.top - canvasOffset.top + Number(elHeight) > canvasHeight) {
                elHeight = elHeight - ((offset.top - canvasOffset.top + Number(elHeight)) - (canvasHeight));
                if (this._browserInfo.name === "mozilla")
                    elHeight = elHeight - 7;
                else
                    elHeight = elHeight - 6;
            }
            textBox.style.height = elHeight + "px";
            if (evt)
                offsetY = (elHeight - height) / 2;
            else
                offsetY = (elHeight - height * curZoomfactor) / 2;
            this._boundingBox = new ej.datavisualization.Diagram.Rectangle(x - offsetX, y - offsetY, elWidth, elHeight);
            div.style.top = y - offsetY - fontSize / 4 + "px";
        },
        _getCanvasHeight: function (canvas, canvasHeight) {
            var height = canvasHeight, element = canvas;
            while (height <= 0) {
                element = element.parentNode;
                if (element) {
                    height = $(element).height();
                }
            }
            return height;
        },
        _textLength: function (evt, label) {
            var temp = document.createElement("div");
            var textElement = document.getElementById(this.element[0].id + "_editBox");
            var attr = {
                "id": this.element[0].id + "_editBoxHiddenDiv",
                "style": "position : absolute; width : auto; height : auto;" +
                         "font-size:" + textElement.style.fontSize + ";font-family:" + textElement.style.fontFamily + ";word-wrap: break-word;white-space:pre-wrap;",
            };
            if (label) {
                attr.style = "position : absolute; width : auto; height : auto;" + "font-weight:" + (label.bold ? "bold" : "normal") + ";font-style:" + (label.italic ? "italic" : "normal") + ";text-decoration:" + (label.textDecoration) + ";font-size:" + label.fontSize + "px;font-size:" + textElement.style.fontSize + "px;font-size:" + label.fontSize + "px;font-family:" + textElement.style.fontFamily + ";word-wrap: break-word;white-space:pre-wrap;";
            }
            ej.datavisualization.Diagram.Util.attr(temp, attr);
            if (evt) {
                temp.innerHTML = evt.target.value;
            } else {
                temp.innerHTML = textElement.value;
            }
            document.body.appendChild(temp);
            var bounds = temp.getBoundingClientRect();
            document.body.removeChild(temp);
            return bounds;
        },
        _endEdit: function (keyCode) {
            var args, entry, label, prevWidth, prevHeight;
            var editBox = document.getElementById(this.element[0].id + "_editBox");
            if (editBox) {
                var shape = this._findNode(editBox.getAttribute("containerName"));;
                if (this.activeTool.name === "textTool")
                    shape = this.activeTool.selectedObject;
                else if (!shape) {
                    for (var i = 0; i < this.selectionList.length; i++) {
                        var mNode = this.selectionList[i];
                        if (typeof mNode.labels.length != "undefined" && mNode.labels.length > 0 && mNode.labels[0].mode === ej.datavisualization.Diagram.LabelEditMode.Edit)
                            shape = this.selectionList[i];
                    }
                }
                if (shape) {
                    this._isEditing = false;
                    var label;
                    if (shape.type == "text") label = shape.textBlock;
                    else {
                        label = this.activeTool.getLabelUnderMouse();
                        if (!label) label = shape.labels[0];
                    }
                    if (label) var prevText = label.text;
                    if (shape._type == "node" && (!shape.width || !shape.height) && this.nameTable[shape.name]) {
                        prevWidth = shape._width;
                        prevHeight = shape._height;
                        shape.labels[0].text = editBox.value;
                        this._getNodeDimension(shape);
                    }
                    var tempKeyCode;
                    if (keyCode) {
                        tempKeyCode = keyCode.keyDownEventArgs.keyCode === 27 ? "ESC" : String.fromCharCode(keyCode.keyDownEventArgs.keyCode);
                    }
                    if (editBox.value != prevText)
                        args = this._raiseEvent("textChange", { element: this.getNode(shape), value: editBox.value, keyCode: tempKeyCode });
                    if (!args) {
                        if (shape && shape.type !== undefined && shape.type === "text") {
                            var oldText = prevText;
                            shape.textBlock.text = editBox.value;
                            shape.textBlock.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                            if (!this._findNode(shape.name)) {
                                if (editBox.value !== "") {
                                    var height = Number(editBox.style.height.substring(0, editBox.style.height.length - 2));
                                    if (shape.height < height) {
                                        shape.offsetY = (shape.offsetY - (shape.height / 2)) + (height / 2);
                                        shape.height = height;
                                    }
                                    if (this.selectionList.length > 0)
                                        this._clearSelection(true);
                                    this.add(shape);
                                    var parent = this.nameTable[shape.parent];
                                    if (parent && parent.isLane) {
                                        var move = this._moveOnLane(shape, parent);
                                        if (!move.proceedX)
                                            this.updateNode(shape.name, { offsetX: shape.offsetX + move.x });
                                        if (!move.proceedY)
                                            this.updateNode(shape.name, { offsetY: shape.offsetY + move.y });
                                        this.tools["resize"]._updateContainer(parent);
                                    }
                                }
                                this.activeTool.selectedObject = null;
                            } else {
                                ej.datavisualization.Diagram.DiagramContext.update(shape, this);
                                entry = { type: "labelchanged", shape: shape, previousLabelValue: oldText, updatedLabelValue: editBox.value, category: "internal" };
                                this.addHistoryEntry(entry);
                                //entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.TextChangeCmd(shape, shape.textBlock.text, editBox.value));
                                //this.historyManager.addHistoryEntry(entry);
                            }
                        } else if (shape != undefined && !(shape.type == "pseudoGroup")) {
                            if (label != undefined) {
                                prevText = prevText ? prevText : label.text;
                                if (prevText != editBox.value) {
                                    entry = { type: "labelchanged", shape: shape, previousLabelValue: label.text, updatedLabelValue: editBox.value, index: shape.labels.indexOf(label), category: "internal" };
                                    this.addHistoryEntry(entry);
                                    //entry = new ej.datavisualization.Diagram.HistoryEntry(new ej.datavisualization.Diagram.LabelChangeCmd(shape, label.text, editBox.value));
                                    //this.historyManager.addHistoryEntry(entry);
                                }
                                label.text = editBox.value;
                                label.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                                ej.datavisualization.Diagram.DiagramContext.updateLabel(shape, label, this);
                                if (shape.isPhase) {
                                    if ((shape.name).split("phaseStack")[1])
                                        var phase = this.nameTable[(shape.name).split("phaseStack")[1]];
                                    if (phase && phase.label)
                                        phase.label.text = editBox.value;
                                }
                                this.activeTool.selectedObject = null;
                            }
                            else if (shape.labels.length == 0) {
                                label = ej.datavisualization.Diagram.Label();
                                label.text = editBox.value;
                                label.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                                shape.labels.push(label);
                                ej.datavisualization.Diagram.DiagramContext.addNodeLabel(shape, label, null, this);
                            }
                            if (this.activeTool.activeLabel) this.activeTool.activeLabel = null;
                        }
                        if (shape._type == "node" && (!shape.width || !shape.height) && this.nameTable[shape.name]) {
                            ej.datavisualization.Diagram.DiagramContext.update(shape, this);
                            if (shape.parent) ej.datavisualization.Diagram.DiagramContext.update(this.nameTable[shape.parent], this);
                        }
                    }
                    else {
                        if (shape._width && prevWidth) shape._width = prevWidth;
                        if (shape._height && prevHeight) shape._height = prevHeight;
                        if (shape && shape.type === "text") {
                            ej.datavisualization.Diagram.DiagramContext.update(shape, this);
                        }
                        else {
                            label = this.activeTool.activeLabel ? this.activeTool.activeLabel : (shape.labels.length > 0 ? shape.labels[0] : null);
                            if (label != undefined) {
                                label.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                                ej.datavisualization.Diagram.DiagramContext.updateLabel(shape, label, this);
                            }
                        }
                        if (this.activeTool) {
                            this.activeTool.activeLabel = null;
                            this.activeTool.selectedObject = null;
                        }
                    }
                }
                this._off($(editBox), "focusout", this._editboxfocusout);
                this._off($(editBox), "keyup", this._editBoxKeyUp);
                this._off($(editBox), "input", this._editBoxTextChange);
                var element = $("#" + this.element[0].id + "_editBoxDiv")[0];
                if (element)
                    element.parentNode.removeChild(element);
                if (this._svg) {
                    var htmlLayer = this._svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                    if (htmlLayer) {
                        if (shape.type === "text")
                            var lblEle = $(htmlLayer).find("#" + shape.name + "_shape_lblbg")[0];
                        else {
                            if (label)
                                var lblEle = $(htmlLayer).find("#" + shape.name + "_" + label.name + "_lblbg")[0];
                        }
                        if (lblEle) {
                            lblEle.style.display = "block";
                        }
                    }
                }
            }
        },
        _setLabelEditing: function (label) {
            var success;
            if (label) {
                if (label.readOnly) {
                    label.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                    success = false;
                }
                else {
                    if (this._currentLabel)
                        this._currentLabel.mode = ej.datavisualization.Diagram.LabelEditMode.View;
                    label.mode = ej.datavisualization.Diagram.LabelEditMode.Edit;
                    this._currentLabel = label;
                    success = true;
                }
                return success;
            }
        },
        _isInternalTool: function (tool) {
            if (tool instanceof ej.datavisualization.Diagram.SelectTool || tool instanceof ej.datavisualization.Diagram.ConnectionEditTool || tool instanceof ej.datavisualization.Diagram.MoveTool || tool instanceof ej.datavisualization.Diagram.ResizeTool
                  || tool instanceof ej.datavisualization.Diagram.PhaseTool || tool instanceof ej.datavisualization.Diagram.RotateTool || (tool instanceof ej.datavisualization.Diagram.PanTool && (this.tool() & ej.datavisualization.Diagram.Tool.SingleSelect) && (this.tool() & ej.datavisualization.Diagram.Tool.ZoomPan)))
                return true;
            else
                return false;
        },
        _updateNodeMargin: function (node) {
            if (node.parent != "") {
                var group = this.nameTable[node.parent];
                if (group && group.container && group.container.type == "canvas") {
                    var groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                    var bounds1 = ej.datavisualization.Diagram.Util.bounds(node);
                    var bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
                    node.marginLeft = bounds.x - (groupBounds.x + group.paddingLeft);
                    node.marginRight = (groupBounds.x + group.width) - (bounds.x + bounds.width) - group.paddingRight;
                    node.marginTop = bounds.y - (groupBounds.y + group.paddingTop);
                    node.marginBottom = groupBounds.bottom - (bounds.y + bounds.height) - group.paddingBottom;
                }
            }
            if ((node.type == "pseudoGroup" || node.name == "multipleSelection") && node.children.length > 0) {
                var child;
                for (var i = 0; i < node.children.length; i++) {
                    child = this.nameTable[this._getChild(node.children[i])];
                    if (child) {
                        this._updateNodeMargin(child);
                    }
                }
            }
        },
        _raisePropertyChange: function (resource) {
            if (resource) {
                this._raiseEvent("propertyChange", resource);
            }
        },
        _translate: function (node, dx, dy, nameTable, isContainer) {
            if (dx || dy) {
                if (!node.segments) {
                    node.offsetX += dx;
                    node.offsetY += dy;
                    if ((node._type === "group" || node.type === "pseudoGroup") && !isContainer) {
                        var nodes = this._getChildren(node.children);
                        var child;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            if (child) {
                                this._translate(child, dx, dy, nameTable);
                                if (child.parent && (child.parent != node.name && node.type != "pseudoGroup"))
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], this);
                            }
                        }
                        var childTable = this._getChildTable(node, {});
                        for (var child in childTable) {
                            if (node.children.indexOf(child) == -1) {
                                var childNode = nameTable[child];
                                if (childNode && childNode.sourceNode && childNode.targetNode && node.children.indexOf(childNode.sourceNode) > -1 && node.children.indexOf(childNode.sourceNode) > -1)
                                    this._translate(childNode, dx, dy, nameTable);
                            }
                        }
                        if (node.type != "group" && node.container)
                            ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                    }
                    if (node != this.activeTool.helper) {
                        this._updateAssociatedConnectorEnds(node, nameTable);
                        if (this.nameTable[node.name]) this._updateQuad(node);
                    }
                } else {
                    ej.datavisualization.Diagram.Util._translateLine(node, dx, dy, node);
                    this._dock(node, nameTable);
                    ej.datavisualization.Diagram.Util.updateBridging(node, this);
                    this._updateConnectorBridging(node);
                }
            }
        },
        _translateLabel: function (node, label, dx, dy) {
            label.margin["left"] += dx;
            label.margin["top"] += dy;
            label.margin["right"] -= dx;
            label.margin["bottom"] -= dy;
            if (label.horizontalAlignment == "center") label.margin["right"] = 0;
            if (label.verticalAlignment == "center") label.margin["bottom"] = 0;
            ej.datavisualization.Diagram.DiagramContext.updateLabel(node, label, this);
        },
        _rotate: function (node, angle, nameTable) {
            if (!node.segments && angle) {
                node.rotateAngle += angle;
                node.rotateAngle %= 360;
                if (node.rotateAngle < 0) node.rotateAngle += 360;
                if ((node._type === "group" || node.type === "pseudoGroup")) {
                    var matrix = ej.Matrix.identity();
                    var child, pinx, piny;
                    ej.Matrix.rotate(matrix, angle, node.offsetX, node.offsetY);
                    var nodes = this._getChildren(node.children);
                    var child;
                    for (var i = 0; i < nodes.length; i++) {
                        child = nameTable[nodes[i]];
                        if (child) {
                            if (!(child.segments)) {
                                this._rotate(child, angle, nameTable);
                                pinx = child.offsetX;
                                piny = child.offsetY;
                                var actualpt = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(pinx, piny));
                                this._translate(child, actualpt.x - pinx, actualpt.y - piny, nameTable);
                                this._updateAssociatedConnectorEnds(child, nameTable);
                            } else {
                                if (!child.sourceNode)
                                    this._setEndPoint(child, ej.Matrix.transform(matrix, child.sourcePoint), false);
                                if (!child.targetNode)
                                    this._setEndPoint(child, ej.Matrix.transform(matrix, child.targetPoint), true);
                            }
                            if (child.parent && (child.parent != node.name && node.type != "pseudoGroup"))
                                ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], this);
                        }
                    }
                    ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                }
                if (node.parent) {
                    this._updateParentBounds(node.parent);
                }
                if (node != this.activeTool.helper)
                    this._updateQuad(node);
                this._updateAssociatedConnectorEnds(node, nameTable);
            }
        },
        scale: function (node, sw, sh, pivot, nameTable, skipScalOnChild, updateMinMax, isHelper) {
            if ((sw && sw != 1) || (sh && sh != 1) || node.isPhase) {
                if (!node.container) {
                    var matrix = ej.Matrix.identity();
                    if (!node.segments) {
                        if ((node._type === "group" || node.type === "pseudoGroup")) {
                            if (node.type != "bpmn" || (node.width * sw >= 20 && node.height * sh >= 20)) {
                                var nodes = this._getChildren(node.children);
                                var child;
                                for (var i = 0; i < nodes.length; i++) {
                                    child = nameTable[nodes[i]];
                                    if (child) {
                                        this.scale(child, sw, sh, pivot, nameTable);
                                        if (child.parent && (child.parent != node.name && node.type != "pseudoGroup"))
                                            ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], this);
                                    }
                                }
                                if (node.children.length <= 0) {
                                    ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                                    ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                                    ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                                    var newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                                    var width = node.width ? node.width * sw : node._width * sw;
                                    var height = node.height ? node.height * sh : node._height * sh;
                                    if (width > 1) {
                                        if (node.width) {
                                            if (width != node.width) node._scaled = true;
                                            node.width = width;
                                            node.offsetX = newPosition.x;
                                        } else {
                                            if (width != node._width) node._scaled = true;
                                            node.maxWidth = (node.parent && nameTable[node.parent].width != node.maxWidth) ? nameTable[node.parent].width : width;
                                            node.offsetX = newPosition.x - (node._width * sw - node._width) / 2;
                                        }
                                    }
                                    if (height > 1) {
                                        if (node.height) {
                                            if (height != node.height) node._scaled = true;
                                            node.height = height;
                                            node.offsetY = newPosition.y;
                                        } else {
                                            if (height != node._height) node._scaled = true;
                                            node.maxHeight = (node.parent && nameTable[node.parent].height > height) ? nameTable[node.parent].height : height;
                                            node.offsetY = newPosition.y - (node._height * sh - node._height) / 2;
                                        }
                                    }
                                }
                                if (node.type != "bpmn")
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                            }
                        } else {
                            ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                            ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                            ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                            var newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                            var width = node.width * sw;
                            var height = node.height * sh;
                            if (width > 1) {
                                if (width != node.width) {
                                    node._scaled = true;
                                }
                                node.width = width;
                                node.offsetX = newPosition.x;
                            }
                            if (height > 1) {
                                if (height != node.height) {
                                    node._scaled = true;
                                }
                                node.height = height;
                                node.offsetY = newPosition.y;
                            }
                        }
                        if (node.parent) {
                            this._updateParentBounds(node.parent);
                        }
                        if (!(this.activeTool.name == "resize" && this.activeTool.inAction && this.selectionList[0] &&
                            this.selectionList[0].type != "pseudoGroup" && this.selectionList[0].type != "group"))
                            this._updateAssociatedConnectorEnds(node, nameTable);
                        if (node != this.activeTool.helper)
                            this._updateQuad(node);
                    } else {
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        for (var i = 0; i < node.segments.length; i++) {
                            var segment = node.segments[i];
                            for (var j = 0; j < segment.points.length; j++) {
                                segment.points[j] = ej.Matrix.transform(matrix, segment.points[j]);
                                if (i == 0 && j == 0 && !node.sourceNode) {
                                    segment._startPoint = segment.points[0];
                                    segment._endPoint = segment.points[segment.points.length - 1];
                                    ej.datavisualization.Diagram.Util._setLineEndPoint(node, ej.Matrix.transform(matrix, node.sourcePoint), false);
                                }
                                if (i == node.segments.length - 1 && j == segment.points.length - 1 && !node.targetNode) {
                                    segment._startPoint = segment.points[0];
                                    segment._endPoint = segment.points[segment.points.length - 1];
                                    node.targetPoint = segment._endPoint;
                                }
                            }
                            segment._startPoint = segment.points[0];
                            segment._endPoint = segment.points[segment.points.length - 1];
                            if (segment.type == "orthogonal") {
                                if (segment.length || segment.length === 0)
                                    segment.length = segment._length = ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                                else
                                    ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, node.segments[i - 1], node.segments[i + 1], node.sourcePoint, node.targetPoint);
                            } else {
                                segment.point = segment._endPoint;
                            }

                        }
                        if (node.sourceNode || node.targetNode)
                            this._dock(node, nameTable);
                    }
                } else {
                    switch (node.container.type) {
                        case "canvas":
                            var newSize = {};
                            var matrix = ej.Matrix.identity();
                            ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                            ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                            ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                            newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                            width = node.width * sw;
                            height = node.height * sh;

                            if (updateMinMax) {
                                node.minHeight = height;
                                node.minWidth = width;
                            }
                            if (isHelper != true)
                                newSize = this._getMinMaxSize(node, width, height);
                            else {
                                newSize.width = width;
                                newSize.height = height;

                            }
                            if (newSize && newSize.width > 1) {
                                var x = node.offsetX - node.width / 2;
                                if (node.width != newSize.width) node._scaled = true;
                                node.width = newSize.width;
                                node.offsetX = x + node.width / 2;
                            }
                            if (newSize && newSize.height > 1) {
                                var y = node.offsetY - node.height / 2;
                                if (node.height != newSize.height) node._scaled = true;
                                node.height = newSize.height;
                                node.offsetY = y + node.height / 2;
                            }
                            if (!skipScalOnChild)
                                ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(node, this, true);
                            break;
                        case "stack":
                            var matrix = ej.Matrix.identity();
                            ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                            ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                            ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                            newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                            var marginLeft = 0;
                            var marginRight = 0;
                            if (sw != 0) {
                                marginLeft = node.marginLeft;
                                marginRight = node.marginRight;
                            }
                            width = (node.width * sw) - (marginLeft + marginRight);
                            height = (node.height * sh) - (node.marginTop + node.marginBottom);
                            newSize = this._getMinMaxSize(node, width, height);
                            if (newSize.width > 1) {
                                x = node.offsetX - node.width / 2;
                                if (node.width != newSize.width) node._scaled = true;
                                node.width = newSize.width - node.marginLeft;
                                node.offsetX = x + node.width / 2 + (node.marginLeft);
                            }
                            if (newSize.height > 1) {
                                y = node.offsetY - node.height / 2;
                                if (node.height != newSize.height) node._scaled = true;
                                node.height = newSize.height - node.marginTop;
                                node.offsetY = y + node.height / 2 + (node.marginTop);
                            }
                            ej.datavisualization.Diagram.SvgContext._alignOnStack(node, this);
                            break;
                    }
                }
            }
        },
        _updateParentBounds: function (node) {
            node = this.nameTable[this._getChild(node)];
            if (node && node.type != "bpmn" && !node.container) {
                ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                ej.datavisualization.Diagram.DiagramContext.update(node, this);
                if (node.parent) {
                    this._updateParentBounds(node.parent);
                }
            }
        },
        _dock: function (connector, nameTable, updateSelection) {
            var length = connector.segments.length;
            var srcPointslength = connector.segments[0].points.length;
            var targetPointslength = connector.segments[connector.segments.length - 1].points.length;
            ej.datavisualization.Diagram.Util.dock(connector, nameTable);
            if (length != connector.segments.length && this._svg) {
                ej.datavisualization.Diagram.Util._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort, this);
                ej.datavisualization.Diagram.DiagramContext._refreshSegments(connector, this);
            }
            if (this.selectionList[0] == connector) {
                if (updateSelection || srcPointslength != connector.segments[0].points.length || targetPointslength != connector.segments[connector.segments.length - 1].points.length) {
                    ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.selectionList[0], this._adornerSvg, this._currZoom);
                }
            }
            this._updateQuad(connector);
        },
        _setEndPoint: function (connector, point, isTarget) {
            ej.datavisualization.Diagram.Util._setLineEndPoint(connector, point, isTarget);
            this._updateQuad(connector);
        },
        _updateAssociatedConnectorEnds: function (node, nameTable) {
            var i, len;
            var connector;
            if (node.inEdges.length || node.outEdges.length) {
                //Finding the segment points to avoid calculating path points multiple times to dock every edge
                var nodeToNode = this._isNodeToNodeConnection(node);
                var nodeToDock = node;
                if (node.type == "bpmn" && node._type == "group") {
                    nodeToDock = typeof node.children[0] == "object" ? node.children[0] : this.nameTable[node.children[0]];
                }
                if (nodeToNode) {
                    var segmentPoints = ej.datavisualization.Diagram.Util._findSegmentPoints(nodeToDock);
                }
                for (i = 0, len = node.inEdges.length; i < len; i++) {
                    connector = nameTable[node.inEdges[i]];
                    if (connector) {
                        if (!connector.targetPadding) {
                            nodeToDock._segmentPoints = segmentPoints;
                        }
                        else delete nodeToDock._segmentPoints;
                        this._dock(connector, nameTable);
                    }
                }
                for (i = 0, len = node.outEdges.length; i < len; i++) {
                    connector = nameTable[node.outEdges[i]];
                    if (connector) {
                        if (!connector.sourcePadding) { nodeToDock._segmentPoints = segmentPoints; }
                        else delete nodeToDock._segmentPoints;
                        this._dock(connector, nameTable);
                    }
                }
                delete nodeToDock._segmentPoints;
            }
        },
        _isNodeToNodeConnection: function (node) {
            var i, len;
            if (node.inEdges.length) {
                for (i = 0, len = node.inEdges.length; i < len; i++) {
                    if (this.nameTable[node.inEdges[i]] && !this.nameTable[node.inEdges[i]].targetPort && (node.type != "bpmn" || this.nameTable[node.inEdges[i]].segments[0].type != "orthogonal")) return true;
                }
            }
            if (node.outEdges.length) {
                for (i = 0, len = node.outEdges.length; i < len; i++) {
                    if (this.nameTable[node.outEdges[i]] && !this.nameTable[node.outEdges[i]].sourcePort && (node.type != "bpmn" || this.nameTable[node.outEdges[i]].segments[this.nameTable[node.outEdges[i]].segments.length - 1].type != "orthogonal")) return true;
                }
            }
        },
        _updateChildBounds: function (node, nameTable) {
            var offX, offY;
            if (node.width == 0 && node.height == 0) {
                offX = node.offsetX;
                offY = node.offsetY;
                ej.datavisualization.Diagram.Util._updateGroupBounds(node, this);
                this._translate(node, offX - node.offsetX, offY - node.offsetY, nameTable);
            }
            else {
                offX = node.offsetX;
                offY = node.offsetY;
                var childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, this);
                node.offsetX = (childBounds.x + childBounds.width * node.pivot.x);
                node.offsetY = (childBounds.y + childBounds.height * node.pivot.y);
                var dx = offX - node.offsetX;
                var dy = offX - node.offsetY;
                this._translate(node, dx, dy, nameTable);
                var deltaWidth = node.width / childBounds.width;
                var deltaHeight = node.height / childBounds.height;
                this._scale(node, deltaWidth, deltaHeight, node.pivot, nameTable);;
                this._translate(node, offX - node.offsetX, offY - node.offsetY, nameTable);
            }
        },
        _updateQuad: function (object) {
            if (this._spatialSearch && this.nameTable[object.name]) {
                var type = this.getObjectType(object), skip = false;
                if (object.isPhase || object.isLane || object._isHeader || object.isPhaseStack || object.type == "pseudoGroup") skip = true;
                if (object.parent && this.nameTable[object.parent]) {
                    var parent = this.nameTable[object.parent];
                    if (parent.type == "bpmn" || parent.isPhaseStack || parent.isSwimlane) skip = true;
                    if (parent.type != "bpmn") this._updateQuad(parent);
                }
                if (!skip && !object._connectorType)
                    ej.datavisualization.Diagram.SpatialUtil._updateQuad(this, this._spatialSearch, object);
            }
        },
        _sliceDockLines: function (list) {
            var list1 = $.extend({}, list);
            var child = null;
            for (var i in list1) {
                child = this.nameTable[this._getChild(list1[i])];
                if (child && child.segments) {
                    if (child.sourceNode || child.targetNode) {
                        list.pop(child.name);
                    }
                }
            }
        },
        _canExecute: function () {
            if (this._isEditing && (this._currentCommand == "delete" || this._currentCommand == "cut" ||
                this._currentCommand == "copy" || this._currentCommand == "paste" || this._currentCommand.match("nudge")))
                return false;
            if (this._currentCommand == "endEdit" && !this._isEditing) return false;
            return true;
        }
        //#endregion
    });
    ej.datavisualization.Diagram.Locale = {};
    ej.datavisualization.Diagram.Locale["en-US"] = {
        cut: "Cut",
        copy: "Copy",
        paste: "Paste",
        undo: "Undo",
        redo: "Redo",
        selectAll: "SelectAll",
        grouping: "Grouping",
        group: "Group",
        ungroup: "Ungroup",
        order: "Order",
        bringToFront: "BringToFront",
        moveForward: "MoveForward",
        sendBackward: "SendBackward",
        sendToBack: "SendToBack"
    }
    $.fn.pinchDiagram = function (rootgroup, map) {
        var startTouches = [],
        handleTouchStart = function (evt) {
            var touches = evt.touches;
            if (touches.length >= 2) {
                for (var i = 0; i < touches.length; i++) {
                    var touch = touches[i]
                      , found = false;
                    for (var j = 0; j < startTouches.length; j++) {
                        var startTouch = startTouches[j];
                        if (touch.identifier === startTouch.identifier) {
                            found = true;
                            break;
                        }
                    }
                    if (!found) {
                        var touchCopy = $.extend({}, touch);
                        startTouches.push(touchCopy);
                    }
                }
            }
            evt.preventDefault();
        },
        getDistance = function (touch1, touch2) {
            var x = touch2.pageX - touch1.pageX,
                y = touch2.pageY - touch1.pageY;
            return Math.sqrt((x * x) + (y * y));
        },
        handleTouchEnd = function (evt) {
            var changedTouches = evt.changedTouches;
            for (var i = 0; i < changedTouches.length; i++) {
                var changedTouch = changedTouches[i];
                for (var j = 0; j < startTouches.length; j++) {
                    var startTouch = startTouches[j];
                    if (startTouch.identifier === changedTouch.identifier) {
                        var idx = startTouches.indexOf(startTouch);
                        startTouches.splice(idx, 1);
                    }
                }
            }
            evt.preventDefault();
        },
        updateTouch = function (dstTouch, srcTouch) {
            dstTouch.pageX = srcTouch.pageX;
            dstTouch.pageY = srcTouch.pageY;
        },
        handleTouchMove = function (evt) {
            evt.preventDefault();
            var touches = evt.touches, z;
            if (touches.length >= 2) {
                var touch0 = touches[0]
                  , touch1 = touches[1]
                  , startTouch0 = startTouches[0]
                  , startTouch1 = startTouches[1];
                z = getDistance(touch0, touch1) / getDistance(startTouch0, startTouch1);
                var focusPoint = map._mousePosition(evt, false);
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(map, z, 0, 0, focusPoint, true, true);
                updateTouch(startTouch0, touch0);
                updateTouch(startTouch1, touch1);
            }
        };
        this[0].addEventListener('touchstart', handleTouchStart, false);
        this[0].addEventListener('touchend', handleTouchEnd, false);
        this[0].addEventListener('touchmove', handleTouchMove, false);
    };
    //#endregion
})(jQuery, Syncfusion);;
(function ($, ej) {
    "use strict";
    ej.MatrixTypes = { Identity: 0, Translation: 1, Scaling: 2, Unknown: 4 };
    ej.MatrixDefaults = { m11: 1, m12: 0, m21: 0, m22: 1, offsetX: 0, offsetY: 0, type: ej.MatrixTypes.Identity };
    ej.Matrix = {
        identity: function () {
            return $.extend(true, {}, ej.MatrixDefaults);
        },
        multiply: function (matrix1, matrix2) {
            var type = matrix1.type;
            var type2 = matrix2.type;
            if (type2 == ej.MatrixTypes.Identity) {
                return;
            }
            if (type == ej.MatrixTypes.Identity) {
                matrix1.m11 = matrix2.m11;
                matrix1.m12 = matrix2.m12;
                matrix1.m21 = matrix2.m21;
                matrix1.m22 = matrix2.m22;
                matrix1.offsetX = matrix2.offsetX;
                matrix1.offsetY = matrix2.offsetY;
                matrix1.type = matrix2.type;
                return;
            }
            if (type2 == ej.MatrixTypes.Translation) {
                matrix1.offsetX += matrix2.offsetX;
                matrix1.offsetY += matrix2.offsetY;
                if (type != ej.MatrixTypes.Unknown) {
                    matrix1.type |= ej.MatrixTypes.Translation;
                }
                return;
            }
            if (type != ej.MatrixTypes.Translation) {
                var num = type << 4 | type2;
                switch (num) {
                    case 34:
                        matrix1.m11 *= matrix2.m11;
                        matrix1.m22 *= matrix2.m22;
                        return;
                    case 35:
                        matrix1.m11 *= matrix2.m11;
                        matrix1.m22 *= matrix2.m22;
                        matrix1.offsetX = matrix2.offsetX;
                        matrix1.offsetY = matrix2.offsetY;
                        matrix1.type = (ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling);
                        return;
                    case 36: break;
                    default:
                        {
                            switch (num) {
                                case 50:
                                    matrix1.m11 *= matrix2.m11;
                                    matrix1.m22 *= matrix2.m22;
                                    matrix1.offsetX *= matrix2.m11;
                                    matrix1.offsetY *= matrix2.m22;
                                    return;
                                case 51:
                                    matrix1.m11 *= matrix2.m11;
                                    matrix1.m22 *= matrix2.m22;
                                    matrix1.offsetX = matrix2.m11 * matrix1.offsetX + matrix2.offsetX;
                                    matrix1.offsetY = matrix2.m22 * matrix1.offsetY + matrix2.offsetY;
                                    return;
                                case 52: break;
                                default:
                                    switch (num) {
                                        case 66:
                                        case 67:
                                        case 68: break;
                                        default: return;
                                    }
                                    break;
                            }
                            break;
                        }
                }
                var result = this.identity();
                this._setMatrix(result, matrix1.m11 * matrix2.m11 + matrix1.m12 * matrix2.m21,
                    matrix1.m11 * matrix2.m12 + matrix1.m12 * matrix2.m22, matrix1.m21 * matrix2.m11 + matrix1.m22 * matrix2.m21,
                    matrix1.m21 * matrix2.m12 + matrix1.m22 * matrix2.m22,
                    matrix1.offsetX * matrix2.m11 + matrix1.offsetY * matrix2.m21 + matrix2.offsetX, matrix1.offsetX * matrix2.m12 + matrix1.offsetY * matrix2.m22 + matrix2.offsetY);
                if (result.m21 || result.m12) {
                    result.type = ej.MatrixTypes.Unknown;
                }
                else {
                    if (result.m11 && result.m11 != 1.0 || result.m22 && result.m22 != 1.0) {
                        result.type = ej.MatrixTypes.Scaling;
                    }
                    if (result.offsetX || result.offsetY) {
                        result.type |= ej.MatrixTypes.Translation;
                    }
                    if ((result.type & (ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling)) == ej.MatrixTypes.Identity) {
                        result.type = ej.MatrixTypes.Identity;
                    }
                }
                matrix1.m11 = result.m11;
                matrix1.m12 = result.m12;
                matrix1.m21 = result.m21;
                matrix1.m22 = result.m22;
                matrix1.offsetX = result.offsetX;
                matrix1.offsetY = result.offsetY;
                matrix1.type = result.type;
                return;
            }
            var offsetX = matrix1.offsetX;
            var offsetY = matrix1.offsetY;
            matrix1.m11 = matrix2.m11;
            matrix1.m12 = matrix2.m12;
            matrix1.m21 = matrix2.m21;
            matrix1.m22 = matrix2.m22;
            matrix1.offsetX = matrix2.offsetX;
            matrix1.offsetY = matrix2.offsetY;
            matrix1.type = matrix2.type;
            matrix1.offsetX = offsetX * matrix2.m11 + offsetY * matrix2.m21 + matrix2.offsetX;
            matrix1.offsetY = offsetX * matrix2.m12 + offsetY * matrix2.m22 + matrix2.offsetY;
            if (type2 == ej.MatrixTypes.Unknown) {
                matrix1.type = ej.MatrixTypes.Unknown;
                return;
            }
            matrix1.type = (ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling);
        },
        transform: function (matrix, point) {
            var pt = this._multiplyPoint(matrix, point.x, point.y);
            return { x: Math.round(pt.x * 100) / 100, y: Math.round(pt.y * 100) / 100 };
        },
        rotate: function (matrix, angle, centerX, centerY) {
            angle %= 360.0;
            this.multiply(matrix, this._createRotationRadians(angle * 0.017453292519943295, centerX ? centerX : 0, centerY ? centerY : 0));
        },
        scale: function (matrix, scaleX, scaleY, centerX, centerY) {
            this.multiply(matrix, this._createScaling(scaleX, scaleY, centerX ? centerX : 0, centerY ? centerY : 0));
        },
        translate: function (matrix, offsetX, offsetY) {
            if (matrix.type & ej.MatrixTypes.Identity) {
                this._setMatrix(matrix, 1.0, 0.0, 0.0, 1.0, offsetX, offsetY, ej.MatrixTypes.Translation);
                return;
            }
            if (matrix.type & ej.MatrixTypes.Unknown) {
                matrix.offsetX += offsetX;
                matrix.offsetY += offsetY;
                return;
            }
            matrix.offsetX += offsetX;
            matrix.offsetY += offsetY;
            matrix.type |= ej.MatrixTypes.Translation;
        },
        _createScaling: function (scaleX, scaleY, centerX, centerY) {
            var result = this.identity();
            this._setMatrix(result, scaleX, 0.0, 0.0, scaleY, centerX - scaleX * centerX, centerY - scaleY * centerY,
                !(centerX || centerY) ? ej.MatrixTypes.Scaling : ej.MatrixTypes.Scaling | ej.MatrixTypes.Translation);
            return result;
        },
        _createRotationRadians: function (angle, centerX, centerY) {
            var result = this.identity();
            var num = Math.sin(angle);
            var num2 = Math.cos(angle);
            var offsetX = centerX * (1.0 - num2) + centerY * num;
            var offsetY = centerY * (1.0 - num2) - centerX * num;
            this._setMatrix(result, num2, num, -num, num2, offsetX, offsetY, ej.MatrixTypes.Unknown);
            return result;
        },
        _multiplyPoint: function (matrix, x, y) {
            switch (matrix.type) {
                case ej.MatrixTypes.Identity: break;
                case ej.MatrixTypes.Translation:
                    x += matrix.offsetX;
                    y += matrix.offsetY;
                    break;
                case ej.MatrixTypes.Scaling:
                    x *= matrix.m11;
                    y *= matrix.m22;
                    break;
                case ej.MatrixTypes.Translation | ej.MatrixTypes.Scaling:
                    x *= matrix.m11;
                    x += matrix.offsetX;
                    y *= matrix.m22;
                    y += matrix.offsetY;
                    break;
                default:
                    var num = y * matrix.m21 + matrix.offsetX;
                    var num2 = x * matrix.m12 + matrix.offsetY;
                    x *= matrix.m11;
                    x += num;
                    y *= matrix.m22;
                    y += num2;
                    break;
            }
            return { x: x, y: y };
        },
        _setMatrix: function (matrix, m11, m12, m21, m22, offsetX, offsetY, type) {
            matrix.m11 = m11;
            matrix.m12 = m12;
            matrix.m21 = m21;
            matrix.m22 = m22;
            matrix.offsetX = offsetX;
            matrix.offsetY = offsetY;
            matrix.type = type || ej.MatrixTypes.Identity;
        }
    };
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej) {
    "use strict";
    //#region HitTesting
    ej.datavisualization.Diagram.HitTesting = {
        _getResizeHandleCenter: function (shape, position, scale) {
            if (!scale) scale = 1;
            var corners = this.bounds(shape);
            var center = ej.datavisualization.Diagram.Point(0, 0);
            switch (position) {
                case "topleft":
                    center = corners.topLeft;
                    break;
                case "topcenter":
                    center = corners.top;
                    break;
                case "topright":
                    center = corners.topRight;
                    break;
                case "middleleft":
                    center = corners.left;
                    break;
                case "middleright":
                    center = corners.right;
                    break;
                case "bottomleft":
                    center = corners.bottomLeft;
                    break;
                case "bottomcenter":
                    center = corners.bottom;
                    break;
                case "bottomright":
                    center = corners.bottomRight;
                    break;
                case "pivot":
                    center = ej.datavisualization.Diagram.Point(shape.offsetX, shape.offsetY);
                    break;
                case "rotate":
                    var thumbHeight = 20;
                    var dist = 20 / scale; //distance between element top and rotator thumb
                    var pt = ej.datavisualization.Diagram.Point(shape.offsetX, (shape.offsetY - shape.height * shape.pivot.y) - thumbHeight / 2 - dist);
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, shape.rotateAngle, shape.offsetX, shape.offsetY);
                    center = ej.Matrix.transform(matrix, pt);
                    break;
            }
            return center;
        },
        findResizeHandleAtPoint: function (node, point) {
            var resizeDirection = null;
            if (node) {
                var center;
                var success;
                var directions = ["topleft", "topcenter", "topright", "middleleft", "middleright", "bottomleft", "bottomcenter", "bottomright"];
                for (var i = 0, len = directions.length; i < len; i++) {
                    center = this._getResizeHandleCenter(node, directions[i]);
                    if (center !== ej.datavisualization.Diagram.Point(0, 0)) {
                        success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(center.x, center.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                        if (success) {
                            resizeDirection = directions[i];
                            break;
                        }
                    }
                }
            }
            return resizeDirection;
        },
        findRotationHandleAtPoint: function (node, point, scale) {
            var center = this._getResizeHandleCenter(node, "rotate", scale);
            var success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(center.x, center.y), 10); // right now 7 is default handle radius size, need to give as constant variable
            if (success) {
                return true;
            }
            return false;
        },
        findPivotHandleAtPoint: function (node, point) {
            var center = this._getResizeHandleCenter(node, "pivot");
            var success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(center.x, center.y), 7); // right now 7 is default handle radius size, need to give as constant variable
            if (success) {
                return true;
            }
            return false;
        },
        findEndPointHandleAtPoint: function (connector, point) {
            var endPointHandle = null;
            if (connector) {
                var success;
                var points = connector.line.points;
                var pt = points[points.length - 1];
                success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                if (success) {
                    endPointHandle = "targetEndPoint";
                }
                if (!success) {
                    pt = points[0];
                    success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                    if (success) {
                        endPointHandle = "sourceEndPoint";
                    }
                }
                if (!success && connector.line.type === "bezier") {
                    pt = connector.line._point1;
                    success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt.x, pt.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                    if (success) {
                        endPointHandle = "bezierpoint1";
                    }
                    if (!success) {
                        var pt1 = connector.line._point2;
                        success = ej.datavisualization.Diagram.Geometry.checkPointOnCircle(point, ej.datavisualization.Diagram.Point(pt1.x, pt1.y), 7); // right now 7 is default handle radius size, need to give as constant variable
                        if (success) {
                            endPointHandle = "bezierpoint2";
                        }
                    }
                }
            }
            return endPointHandle;
        }
    };
    //#endregion
    //#region Util
    ej.datavisualization.Diagram.Util = {
        findPortByName: function (node, portName) {
            var port;
            if (node.ports)
                for (var i = 0, len = node.ports.length; i < len; i++) {
                    port = node.ports[i];
                    if (port.name === portName) {
                        return port;
                    }
                }
            return null;
        },
        attr: function (element, attributes) {
            for (var atr in attributes) {
                element.setAttribute(atr.toString(), attributes[atr]);
            }
        },
        removeItem: function (array, item) {
            var index = array.indexOf(item);
            if (index >= 0)
                array.splice(index, 1);
        },
        clear: function (array) {
            while (array.length > 0) {
                array.pop();
            }
        },
        getChild: function (child) {
            if (child) {
                if (typeof (child) == "object") {
                    return child.name;
                } else {
                    return child;
                }
            }
        },
        randomId: function () {
            var chars = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz";
            var id = "";
            //get 4 random chars
            for (var i = 0; i < 4; i++) {
                var num = Math.floor(Math.random() * chars.length);
                id += chars.substring(num, num + 1);
            }
            return id;
        },
        htmlEncode: function (html) {
            var str = html.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&apos;/g, '\'');
            return str;
        },
        canMove: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Drag;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Drag;
        },
        canMoveLabel: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragLabel;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.DragLabel;
        },
        canRotate: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Rotate;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Rotate;
        },
        canShowTooltip: function (node, diagram) {
            if (node.segments) {
                var inherit = node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.InheritTooltip;
            } else
                var inherit = node.constraints & ej.datavisualization.Diagram.NodeConstraints.InheritTooltip;
            if (inherit) {
                return diagram.model.tooltip;
            } return node.tooltip;
        },
        canSelect: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Select;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Select;
        },
        canBridge: function (connector, diagram) {
            if (connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Bridging) {
                return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Bridging;
            }
            else if (connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.InheritBridging) {
                return diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.Bridging;
            }
        },
        canAllowPan: function (node) {
            return node.constraints & ej.datavisualization.Diagram.NodeConstraints.AllowPan;
        },
        canAllowDrop: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.AllowDrop;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.AllowDrop;
        },
        canDelete: function (node) {
            if (node.segments) {
                return node.constraints & ej.datavisualization.Diagram.ConnectorConstraints.Delete;
            } else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Delete;
        },
        isPageEditable: function (diagram) {
            if (diagram) {
                return diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PageEditable;
            }
        },
        canDoMultipleSelection: function (diagram) {
            if (diagram && this.isPageEditable(diagram)) {
                return diagram.tool() & ej.datavisualization.Diagram.Tool.MultipleSelect;
            }
        },
        canDoSingleSelection: function (diagram) {
            if (diagram && this.isPageEditable(diagram)) {
                return diagram.tool() & ej.datavisualization.Diagram.Tool.SingleSelect;
            }
        },
        _getPseudoGroupConstraints: function (diagram, node, selectionlist) {
            var pseudoGroupConstraints;
            if (selectionlist && selectionlist.type === "pseudoGroup") {
                if (selectionlist.children.indexOf(node.name) >= 0 || node.name === selectionlist.name)
                    if (diagram.model.selectedItems.getConstraints) {
                        pseudoGroupConstraints = diagram.model.selectedItems.getConstraints();
                    }
                    else {
                        if (selectionlist.children) {
                            pseudoGroupConstraints = ej.datavisualization.Diagram.NodeConstraints.Default;
                            for (var i = 0; i < selectionlist.children.length ; i++) {
                                var nodes = diagram.nameTable[diagram._getChild(selectionlist.children[i])];
                                var nodeConstraints = nodes.constraints;
                                if (!(nodeConstraints & ej.datavisualization.Diagram.NodeConstraints.Resize))
                                    pseudoGroupConstraints = pseudoGroupConstraints & ~ej.datavisualization.Diagram.NodeConstraints.Resize;
                                if (!(nodeConstraints & ej.datavisualization.Diagram.NodeConstraints.Rotate))
                                    pseudoGroupConstraints = pseudoGroupConstraints & ~ej.datavisualization.Diagram.NodeConstraints.Rotate;
                                if (!(nodeConstraints & ej.datavisualization.Diagram.NodeConstraints.Drag))
                                    pseudoGroupConstraints = pseudoGroupConstraints & ~ej.datavisualization.Diagram.NodeConstraints.Drag;
                            }
                        }
                    }
            }
            return pseudoGroupConstraints;
        },
        canPanning: function (diagram) {
            if (diagram) {
                return (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableX) || (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableY);
            }
        },
        canZooming: function (diagram) {
            if (diagram) {
                return (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.Zoomable);
            }
        },
        canUndo: function (diagram) {
            if (diagram) {
                return (diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.Undoable);
            }
        },
        canMultiSelectOnLane: function (node) {
            return node.constraints & ej.datavisualization.Diagram.NodeConstraints.MultiSelect;
        },
        canDragSourceEnd: function (connector) {
            return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragSourceEnd;
        },
        canDragTargetEnd: function (connector) {
            return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragTargetEnd;
        },
        canDragSegmentThumbs: function (connector) {
            return connector.constraints & ej.datavisualization.Diagram.ConnectorConstraints.DragSegmentThumb;
        },
        isTargetConnected: function (connector) {
            return connector.targetNode ? true : false;
        },
        isSourceConnected: function (connector) {
            return connector.sourceNode ? true : false;
        },
        bounds: function (object, excludeRotation) {
            var rect;
            if (object.segments) {
                for (var i = 0; i < object.segments.length; i++) {
                    var segment = object.segments[i];
                    if (i == 0) {
                        if (segment.points)
                            rect = ej.datavisualization.Diagram.Geometry.rect(segment.points);
                    }
                    else
                        rect = ej.datavisualization.Diagram.Geometry.union(rect, ej.datavisualization.Diagram.Geometry.rect(segment.points));
                    rect.width = rect.width == 0 ? object.lineWidth : rect.width;
                    rect.height = rect.height == 0 ? object.lineWidth : rect.height;
                }
            }
            else if (object._type == "node" || object._type === "group" || object._type == "pseudoGroup") {
                var width = object.width ? object.width : object._width || 0;
                var height = object.height ? object.height : object._height || 0;
                rect = { x: object.offsetX - width * object.pivot.x, y: object.offsetY - height * object.pivot.y, width: width, height: height };
            }
            else
                rect = object;
            if (rect) {
                var bounds = {};
                rect.x = Math.round(rect.x * 100) / 100;
                rect.y = Math.round(rect.y * 100) / 100;
                rect.width = Math.round(rect.width * 100) / 100;
                rect.height = Math.round(rect.height * 100) / 100;
                bounds["width"] = rect.width;
                bounds["height"] = rect.height;
                bounds["x"] = bounds["left"] = rect.x;
                bounds["right"] = rect.x + rect.width;
                bounds["y"] = bounds["top"] = rect.y;
                bounds["bottom"] = rect.y + rect.height;
                bounds["center"] = { x: rect.x + rect.width / 2, y: rect.y + rect.height / 2 };
                bounds["topLeft"] = { x: rect.x, y: rect.y };
                bounds["topCenter"] = { x: rect.x + rect.width / 2, y: rect.y };
                bounds["topRight"] = { x: rect.x + rect.width, y: rect.y };
                bounds["middleLeft"] = { x: rect.x, y: rect.y + rect.height / 2 };
                bounds["middleRight"] = { x: rect.x + rect.width, y: rect.y + rect.height / 2 };
                bounds["bottomLeft"] = { x: rect.x, y: rect.y + rect.height };
                bounds["bottomCenter"] = { x: rect.x + rect.width / 2, y: rect.y + rect.height };
                bounds["bottomRight"] = { x: rect.x + rect.width, y: rect.y + rect.height };
                if (object.rotateAngle && !excludeRotation) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    bounds.topLeft = ej.Matrix.transform(matrix, bounds.topLeft);
                    bounds.topCenter = ej.Matrix.transform(matrix, bounds.topCenter);
                    bounds.topRight = ej.Matrix.transform(matrix, bounds.topRight);
                    bounds.middleLeft = ej.Matrix.transform(matrix, bounds.middleLeft);
                    bounds.middleRight = ej.Matrix.transform(matrix, bounds.middleRight);
                    bounds.bottomLeft = ej.Matrix.transform(matrix, bounds.bottomLeft);
                    bounds.bottomCenter = ej.Matrix.transform(matrix, bounds.bottomCenter);
                    bounds.bottomRight = ej.Matrix.transform(matrix, bounds.bottomRight);
                    var rbounds = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                    bounds.x = bounds.left = rbounds.x;
                    bounds.y = bounds.top = rbounds.y;
                    bounds.right = rbounds.x + rbounds.width;
                    bounds.bottom = rbounds.y + rbounds.height;
                    bounds.center = ej.datavisualization.Diagram.Point(rbounds.x + rbounds.width / 2, rbounds.y + rbounds.height / 2);
                }
                bounds["points"] = [bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft];
                return bounds;
            }
            return rect;
        },
        _swapBounds: function (object, bounds) {
            var object, bounds;
            bounds = bounds;
            if (object.rotateAngle) {
                if (object.rotateAngle < 45) { }
                else if (object.rotateAngle <= 135) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    var temp;
                    temp = bounds["topCenter"];
                    bounds["topCenter"] = bounds["middleLeft"];
                    bounds["middleLeft"] = bounds["bottomCenter"];
                    bounds["bottomCenter"] = bounds["middleRight"];
                    bounds["middleRight"] = temp;
                    temp = bounds["topLeft"];
                    bounds["topLeft"] = bounds["bottomLeft"];
                    bounds["bottomLeft"] = bounds["bottomRight"];
                    bounds["bottomRight"] = bounds["topRight"];
                    bounds["topRight"] = temp;
                }
                else if (object.rotateAngle <= 225) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    var temp;
                    temp = bounds["topCenter"];
                    bounds["topCenter"] = bounds["bottomCenter"];
                    bounds["bottomCenter"] = temp;
                    temp = bounds["middleLeft"];
                    bounds["middleLeft"] = bounds["middleRight"];
                    bounds["middleRight"] = temp;
                    temp = bounds["bottomLeft"];
                    bounds["bottomLeft"] = bounds["topLeft"];
                    bounds["topLeft"] = temp;
                    temp = bounds["bottomRight"];
                    bounds["bottomRight"] = bounds["topRight"];
                    bounds["topRight"] = temp;
                }
                else if (object.rotateAngle <= 315) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, object.rotateAngle, object.offsetX, object.offsetY);
                    var temp;
                    temp = bounds["topCenter"];
                    bounds["topCenter"] = bounds["middleRight"];
                    bounds["middleRight"] = bounds["bottomCenter"];
                    bounds["bottomCenter"] = bounds["middleLeft"];
                    bounds["middleLeft"] = temp;
                    temp = bounds["topRight"];
                    bounds["topRight"] = bounds["bottomRight"];
                    bounds["bottomRight"] = bounds["bottomLeft"];
                    bounds["bottomLeft"] = bounds["topLeft"];
                    bounds["topLeft"] = temp;
                }
                bounds["points"] = [bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft];
                return bounds;
            }
        },
        canResize: function (node, dir) {
            if (dir) {
                switch (dir) {
                    case "n-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorth;
                    case "s-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouth;
                    case "e-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeEast;
                    case "w-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeWest;
                    case "se-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouthEast;
                    case "ne-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast;
                    case "sw-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest;
                    case "nw-resize":
                        return node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest;
                }
            }
            else
                return node.constraints & ej.datavisualization.Diagram.NodeConstraints.Resize;
        },
        canConnect: function (node, isPort) {
            var connect = null;
            if (isPort) {
                connect = node.constraints & ej.datavisualization.Diagram.PortConstraints.Connect;
            }
            else if (!node.segments)
                connect = node.constraints & ej.datavisualization.Diagram.NodeConstraints.Connect;
            return connect;
        },
        _initializeSegments: function (connector, defaultType) {
            if (connector.segments && connector.segments.length <= 1) {
                if (!connector.segments.length) {
                    connector.segments.push(ej.datavisualization.Diagram.Segment({ type: defaultType || "straight" }));
                }
                var first = connector.segments[0];
                switch (first.type) {
                    case "straight":
                        if (!first.point) { first._point = connector.targetPoint; }
                        else if (first.point.x != connector.targetPoint.x || first.point.y != connector.targetPoint.y) {
                            //Need to add a method
                            connector.segments.push({ type: first.type || "straight" });
                        }
                        break;
                    case "orthogonal":
                        if (first.length && first.direction) {
                            ej.datavisualization.Diagram.Util._addOrthogonalPoints(first, null, null, connector.sourcePoint, connector.targetPoint);
                            if (first._endPoint.x != connector.targetPoint.x || first._endPoint.y != connector.targetPoint.y) {
                                connector.segments.push({ type: first.type || "straight" });
                            }
                        }
                        else {
                            first._length = first.length || 20;
                            first._direction = first.direction || "auto";
                        }
                        break;
                    case "bezier":
                        if (!first.point) first._point = connector.targetPoint;
                        else if (first.point.x != connector.targetPoint.x || first.point.y != connector.targetPoint.y) {
                            //Need to add a method
                            connector.segments.push({ type: first.type || "straight" });
                        }
                        break;
                }
            }
            var startPoint = connector.sourcePoint;
            var prev = null;
            var next = null;
            var runAngle;
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                segment._startPoint = startPoint;
                next = connector.segments[i + 1];
                segment = connector.segments[i] = ej.datavisualization.Diagram.Segment(connector.segments[i]);
                if (segment.type == "orthogonal") {
                    segment._direction = segment.direction || "auto";
                    segment._length = segment.length || 20;
                    runAngle = ej.datavisualization.Diagram.Util._addOrthogonalPoints(connector.segments[i], prev, next, connector.sourcePoint, connector.targetPoint, runAngle);
                    startPoint = connector.segments[i]._endPoint;
                    if (!segment.length || !segment.direction) {
                        break;
                    }
                }
                else if (segment.type == "straight") {
                    segment.points.push(startPoint);
                    segment.points.push(connector.segments[i].point || connector.targetPoint);
                    segment._point = segment._endPoint = connector.segments[i].point || connector.targetPoint;
                    runAngle = ej.datavisualization.Diagram.Geometry.findAngle(startPoint, segment._endPoint);
                    startPoint = connector.segments[i]._endPoint;
                    if (!segment.point) {
                        break;
                    }
                }
                else if (segment.type == "bezier") {
                    segment._point = segment._endPoint = connector.segments[i].point || connector.targetPoint;
                    segment.points.push(startPoint);
                    segment.points.push(connector.segments[i].point || connector.targetPoint);
                    runAngle = ej.datavisualization.Diagram.Geometry.findAngle(startPoint, segment._endPoint);
                    ej.datavisualization.Diagram.Util._updateBezierPoints(segment);
                    startPoint = connector.segments[i]._endPoint;
                    if (!segment.point) {
                        break;
                    }
                }
                else {
                    var segment = ej.datavisualization.Diagram.Segment({ type: "orthogonal" });
                    segment._direction = "auto";
                    segment._length = 20;
                    segment._startPoint = startPoint;
                    segment._endPoint = connector.targetPoint;
                    connector.segments.push(segment);
                    ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, prev, null, connector.sourcePoint, connector.targetPoint, runAngle);
                    break;
                }
                prev = connector.segments[i];
            }
            if (startPoint.x != connector.targetPoint.x || startPoint.y != connector.targetPoint.y) {
                var segment = ej.datavisualization.Diagram.Segment({ type: (prev ? prev.type : defaultType) || "straight", });
                segment._startPoint = startPoint;
                segment._endPoint = connector.targetPoint;
                if (segment.type == "orthogonal") {
                    ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, prev, null, connector.sourcePoint, connector.targetPoint, runAngle);
                }
                else {
                    segment._point = connector.targetPoint;
                    segment.points.push(startPoint);
                    segment.points.push(connector.targetPoint);
                    if (segment.type == "bezier") { this._updateBezierPoints(segment); }
                }
                connector.segments.push(segment);
            }
        },
        _getDockableBPMNNode: function (node, connector, nameTable, isTarget) {
            var segment = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            if (node && node.type == "bpmn")
                if (segment.type == "orthogonal") {
                    return node;
                } else if (node._type == "group")
                    node = (typeof node.children[0] == "object") ? node.children[0] : nameTable[node.children[0]];
            return node;
        },
        dock: function (connector, nameTable) {
            var targetNode, targetPort = null, sourceNode, sourcePort = null;
            var srcDecoratorSize = connector._srcDecoratorSize, tarDecoratorSize = connector._tarDecoratorSize;
            if (connector.sourcePadding) connector._srcDecoratorSize += connector.sourcePadding;
            if (connector.targetPadding) connector._tarDecoratorSize += connector.targetPadding
            targetNode = nameTable[connector.targetNode];
            if (targetNode)
                targetPort = this.findPortByName(targetNode, connector.targetPort);
            sourceNode = nameTable[connector.sourceNode];
            if (sourceNode)
                sourcePort = this.findPortByName(sourceNode, connector.sourcePort);
            var sourceConnected = this.isSourceConnected(connector);
            var targetConnected = this.isTargetConnected(connector);
            if (!sourcePort) {
                sourceNode = this._getDockableBPMNNode(sourceNode, connector, nameTable, false);
                if (sourceNode && sourceNode.connectorPadding) connector._srcDecoratorSize += sourceNode.connectorPadding;
            } else if (sourcePort.connectorPadding) connector._srcDecoratorSize += sourcePort.connectorPadding;
            if (!targetPort) {
                targetNode = this._getDockableBPMNNode(targetNode, connector, nameTable, true);
                if (targetNode && targetNode.connectorPadding) connector._tarDecoratorSize += targetNode.connectorPadding;
            } else if (targetPort.connectorPadding) connector._tarDecoratorSize += targetPort.connectorPadding;
            if (targetConnected && sourceConnected && targetPort && sourcePort) {
                this._dockPortToPort(connector, targetNode, targetPort, sourceNode, sourcePort);
            }
            else if (targetConnected && targetPort) {
                if (sourceConnected) {
                    if (connector.segments.length == 1) {
                        this._dockNode(connector, sourceNode, null, targetConnected, sourceConnected);
                        this._dockPortToNode(connector, targetNode, targetPort, sourceNode, sourcePort, true);
                    }
                    else {
                        this._dockNode(connector, sourceNode, null, targetConnected, sourceConnected);
                        this._updateIntermediateSegments(connector);
                        this._dockPortToPoint(connector, targetNode, targetPort, true);
                        this._updatePreviousSegment(connector);
                    }
                }
                else {
                    this._dockPortToPoint(connector, targetNode, targetPort, true);
                    this._updatePreviousSegment(connector);
                }
            }
            else if (sourceConnected && sourcePort) {
                if (targetConnected) {
                    if (connector.segments.length == 1) {
                        this._dockNode(connector, targetNode, null, targetConnected, sourceConnected);
                        this._dockPortToNode(connector, sourceNode, sourcePort, targetNode, targetPort, false);
                    }
                    else {
                        this._dockPortToPoint(connector, sourceNode, sourcePort, false);
                        this._updateIntermediateSegments(connector);
                        this._dockNode(connector, targetNode, null, targetConnected, sourceConnected);
                        this._updatePreviousSegment(connector);
                    }
                }
                else {
                    this._dockPortToPoint(connector, sourceNode, sourcePort, false);
                    this._updateIntermediateSegments(connector);
                }
            }
            else if (targetConnected && sourceConnected) {
                if (targetNode && sourceNode)
                    this._dockNode(connector, targetNode, sourceNode, targetConnected, sourceConnected);
            }
            else if (targetConnected) {
                this._dockNode(connector, targetNode, null, targetConnected, sourceConnected);

            }
            else if (sourceConnected) {
                this._dockNode(connector, sourceNode, null, targetConnected, sourceConnected);
            }
            this._adjustEndPoint(connector, sourceNode, sourcePort, targetNode, targetPort);
            connector._srcDecoratorSize = srcDecoratorSize;
            connector._tarDecoratorSize = tarDecoratorSize;
        },
        _dockPortToPort: function (connector, targetNode, targetPort, sourceNode, sourcePort) {
            if (connector.segments.length == 1) {
                var portLocation;
                if (targetNode)
                    var targetBounds = this.bounds(targetNode, true);
                if (sourceNode)
                    var sourceBounds = this.bounds(sourceNode, true);
                var matrix;
                var first = connector.segments[0];
                var last = connector.segments[connector.segments.length - 1];
                var segment = connector.segments[0];
                if (segment.type !== "orthogonal") {
                    if (targetPort) {
                        portLocation = this._getPortPosition(targetPort, targetBounds);
                        if (targetNode.rotateAngle) {
                            matrix = ej.Matrix.identity();
                            ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                            portLocation = ej.Matrix.transform(matrix, portLocation);
                        }
                        this._setLineEndPoint(connector, portLocation, true);
                    }
                    if (sourcePort) {
                        portLocation = this._getPortPosition(sourcePort, sourceBounds);
                        if (sourceNode.rotateAngle) {
                            matrix = ej.Matrix.identity();
                            ej.Matrix.rotate(matrix, sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                            portLocation = ej.Matrix.transform(matrix, portLocation);
                        }
                        this._setLineEndPoint(connector, portLocation, false);
                    }
                    if (segment.type === "bezier")
                        this._updateBezierPoints(segment, targetNode, targetPort, targetBounds, sourceNode, sourcePort, sourceBounds);
                } else {
                    var targetPortLocation = this._getPortPosition(targetPort, targetBounds);
                    var sourcePortLocation = this._getPortPosition(sourcePort, sourceBounds);
                    var targetDirection = this._swapDirection(targetNode.rotateAngle, this._getDirection(targetBounds, targetPortLocation));
                    var sourceDirection = this._swapDirection(sourceNode.rotateAngle, this._getDirection(sourceBounds, sourcePortLocation));
                    if (sourceNode.rotateAngle) {
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                        sourcePortLocation = ej.Matrix.transform(matrix, sourcePortLocation);
                    }
                    if (targetNode.rotateAngle) {
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                        targetPortLocation = ej.Matrix.transform(matrix, targetPortLocation);
                    }
                    this._constructSegements(connector, first, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
                    this._setLineEndPoint(connector, first.points[0], false);
                    this._setLineEndPoint(connector, first.points[first.points.length - 1], true);
                }
            }
            else {
                this._dockPortToPoint(connector, sourceNode, sourcePort, false);
                this._updateIntermediateSegments(connector);
                this._dockPortToPoint(connector, targetNode, targetPort, true);
                this._updatePreviousSegment(connector);
            }
        },
        _dockPortToNode: function (connector, targetNode, targetPort, sourceNode, sourcePort, isTarget) {
            var first = connector.segments[0];
            var last = connector.segments[connector.segments.length - 1];
            if (targetNode)
                var targetBounds = this.bounds(targetNode, true);
            if (sourceNode)
                var sourceBounds = this.bounds(sourceNode, true);
            var matrix;
            var segment = isTarget ? last : first;
            if (segment.type !== "orthogonal") {
                var portLocation = this._getPortPosition(targetPort, targetBounds);
                matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                portLocation = ej.Matrix.transform(matrix, portLocation);
                //segment._endPoint = portLocation;
                this._setLineEndPoint(connector, portLocation, isTarget);
            }
            else {
                var targetPortLocation = this._getPortPosition(targetPort, targetBounds);
                var targetDirection = this._swapDirection(targetNode.rotateAngle, this._getDirection(targetBounds, targetPortLocation));
                matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                targetPortLocation = ej.Matrix.transform(matrix, targetPortLocation);
                var sourcePortLocation;
                if (!isTarget) {
                    sourcePortLocation = connector.targetPoint;
                }
                else {
                    sourcePortLocation = connector.sourcePoint;
                }
                var sourceDirection;
                if (sourceNode) {
                    sourceDirection = this._getDirection(sourceBounds, sourcePortLocation);
                }
                else sourceDirection = "top";
                if (sourceDirection) {
                    targetBounds = this.bounds(targetNode);
                    if (sourceNode)
                        sourceBounds = this.bounds(sourceNode);
                    else sourceBounds = { left: sourcePortLocation.x, right: sourcePortLocation.x, top: sourcePortLocation.y, bottom: sourcePortLocation.y };
                    if (isTarget) {
                        this._setLineEndPoint(connector, targetPortLocation, true);
                        this._constructSegements(connector, last, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
                    }
                    else {
                        this._setLineEndPoint(connector, targetPortLocation, false);
                        this._constructSegements(connector, first, sourceBounds, sourcePortLocation, sourceDirection, targetBounds, targetPortLocation, targetDirection);
                    }
                }
            }
        },
        _dockPortToPoint: function (connector, targetNode, targetPort, isTarget) {
            var first = connector.segments[0];
            var last = connector.segments[connector.segments.length - 1];
            var segment = isTarget ? last : first;
            if (segment.type !== "orthogonal") {
                this._dockPortToNode(connector, targetNode, targetPort, null, null, isTarget);
            }
            else {
                var targetBounds = this.bounds(targetNode, true);
                var targetPortLocation = this._getPortPosition(targetPort, targetBounds);
                var targetDirection = this._swapDirection(targetNode.rotateAngle, this._getDirection(targetBounds, targetPortLocation));
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                targetPortLocation = ej.Matrix.transform(matrix, targetPortLocation);
                var sourcePortLocation;
                var sourcePortLocation;
                if (!isTarget) {
                    sourcePortLocation = first._endPoint;
                }
                else {
                    sourcePortLocation = last._startPoint;
                }
                var sourceBounds = this.bounds(ej.datavisualization.Diagram.Rectangle(sourcePortLocation.x, sourcePortLocation.y, 0, 0));
                targetBounds = this.bounds(targetNode);
                var sourceDirection = "top";
                if (isTarget) {
                    this._setLineEndPoint(connector, targetPortLocation, true);
                    this._pointsFromNodeToPoint(connector, last, targetDirection, targetBounds, targetNode.rotateAngle, targetPortLocation, sourcePortLocation, true); //{(connector, last, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection);
                }
                else {
                    this._setLineEndPoint(connector, targetPortLocation, false);
                    this._pointsFromNodeToPoint(connector, first, targetDirection, targetBounds, targetNode.rotateAngle, targetPortLocation, sourcePortLocation, false);
                }
            }
        },
        _dockNode: function (connector, targetNode, sourceNode, targetConnected, sourceConnected) {
            var startsegment, endsegment;
            var minSpace = connector._srcDecoratorSize + connector._tarDecoratorSize;
            if (connector.segments.length == 1) {
                startsegment = endsegment = connector.segments[0];
            }
            else {
                startsegment = connector.segments[0];
                endsegment = connector.segments[connector.segments.length - 1];
            }
            if (targetConnected && sourceConnected && !connector.targetPort && !connector.sourcePort) {
                if (connector.segments.length == 1) {
                    if (connector.segments[0].type == "orthogonal") {
                        var sourceBounds = this.bounds(sourceNode);
                        var targetBounds = this.bounds(targetNode);
                        this._swapBounds(sourceNode, sourceBounds);
                        this._swapBounds(targetNode, targetBounds);
                        var srcPoint, tarPoint, srcDirection, tarDirection;
                        ////Test top side.
                        if (sourceBounds.y - minSpace >= targetBounds.bottom) {
                            srcPoint = sourceBounds.topCenter;
                            srcDirection = "top";
                            tarPoint = targetBounds.bottomCenter;
                            tarDirection = "bottom";
                        }
                            ////Test bottom side.
                        else if (sourceBounds.bottom + minSpace <= targetBounds.y) {
                            tarPoint = targetBounds.topCenter;
                            tarDirection = "top";
                            srcPoint = sourceBounds.bottomCenter;
                            srcDirection = "bottom";
                        }
                        if ((sourceBounds.bottom + minSpace > targetBounds.y) && (sourceBounds.y < targetBounds.bottom + minSpace)) {
                            if (sourceBounds.right + minSpace < targetBounds.x || (targetBounds.right >= sourceBounds.x - minSpace && sourceBounds.x > targetBounds.x)) {
                                ////rightleft
                                srcPoint = sourceBounds.middleRight;
                                srcDirection = "right";
                                tarPoint = targetBounds.middleLeft;
                                tarDirection = "left";
                            }
                            else {
                                if (targetBounds.right + minSpace < sourceBounds.x || (sourceBounds.right >= targetBounds.x - minSpace && sourceBounds.x < targetBounds.x)) {
                                    srcPoint = sourceBounds.middleLeft;
                                    srcDirection = "left";
                                    tarPoint = targetBounds.middleRight;
                                    tarDirection = "right";
                                }
                                else if (sourceBounds.x <= targetBounds.x) {
                                    srcPoint = sourceBounds.middleLeft;
                                    srcDirection = "left";
                                    tarPoint = targetBounds.middleLeft;
                                    tarDirection = "left";
                                    ////left left
                                }
                                else if (sourceBounds.x <= targetBounds.right) {
                                    ////right right
                                    srcPoint = sourceBounds.middleRight;
                                    srcDirection = "right";
                                    tarPoint = targetBounds.middleRight;
                                    tarDirection = "right";
                                }
                                else {
                                    ////left right
                                    srcPoint = sourceBounds.middleLeft;
                                    srcDirection = "left";
                                    tarPoint = targetBounds.middleRight;
                                    tarDirection = "right";
                                }
                            }
                        }
                        var targetPoint;
                        targetPoint = this._findEndPoint(srcDirection, srcPoint, sourceNode) || sourceBounds.center;
                        srcPoint = this._findIntersection(sourceNode, sourceBounds, connector, srcPoint, targetPoint, false) || srcPoint;
                        targetPoint = this._findEndPoint(tarDirection, tarPoint, targetNode) || targetBounds.center;
                        tarPoint = this._findIntersection(targetNode, targetBounds, connector, tarPoint, targetPoint, true) || tarPoint;
                        this._setLineEndPoint(connector, srcPoint, false);
                        this._setLineEndPoint(connector, tarPoint, true);
                        if (connector.segments[0].type == "orthogonal") {
                            this._constructSegements(connector, connector.segments[0], targetBounds, connector.targetPoint, tarDirection, sourceBounds, connector.sourcePoint, srcDirection);
                            connector.segments[0]._direction = srcDirection;
                        }
                    }
                    else {
                        if (targetNode) {
                            targetBounds = this.bounds(targetNode);
                            this._swapBounds(targetNode, targetBounds);
                            if (sourceNode) {
                                sourceBounds = this.bounds(sourceNode);
                                this._swapBounds(sourceNode, sourceBounds);
                                this._setLineEndPoint(connector, sourceBounds.center, false);
                                this._setLineEndPoint(connector, targetBounds.center, true);
                                this._dockBounds(connector, sourceNode, sourceBounds, false);
                            }
                            this._setLineEndPoint(connector, targetBounds.center, true);
                            this._dockBounds(connector, targetNode, targetBounds, true);
                        }
                        if (connector.segments[0].type == "bezier") {
                            this._updateBezierPoints(connector.segments[0]);
                        }
                    }
                }
                else {
                    this._dock(connector, startsegment, sourceNode, null, false);
                    this._updateIntermediateSegments(connector);
                    if (connector.segments.length == 1) {
                        this._dock(connector, startsegment, sourceNode, targetNode, false);
                        this._dock(connector, endsegment, targetNode, sourceNode, true);
                    }
                    else {
                        this._dock(connector, endsegment, targetNode, null, true);
                        this._updatePreviousSegment(connector);
                    }
                }
            }
            else if (targetConnected && !connector.targetPort) {
                this._dock(connector, endsegment, targetNode, sourceNode, true);
                this._updatePreviousSegment(connector);
            }
            else if (sourceConnected && !connector.sourcePort) {
                this._dock(connector, startsegment, targetNode, sourceNode, false);
                this._updateIntermediateSegments(connector);
            }
        },
        _adjustEndPoint: function (connector, sourceNode, sourcePort, targetNode, targetPort) {
            if (connector.sourcePadding || sourceNode && sourceNode.connectorPadding || targetNode && targetNode.connectorPadding) {
                var length = 0;
                var targetBounds = targetNode ? this.bounds(targetNode) : null;
                var sourceBounds = sourceNode ? this.bounds(sourceNode) : null;
                if (sourceNode) {
                    var endPoint;
                    var segment = connector.segments[0];
                    if (segment.type !== "orthogonal") endPoint = segment.type !== "bezier" ? segment._endPoint : segment._point1;
                    else endPoint = segment.points[1];
                    if (sourcePort) length = connector.sourcePadding + sourcePort.connectorPadding;
                    var sourcePortLocation = this._adjustPoint(segment._startPoint, endPoint, true, length);
                    this._setLineEndPoint(connector, sourcePortLocation, false);
                    if (segment.type == "bezier") this._updateBezierPoints(segment, targetNode, targetPort, targetBounds, sourceNode, sourcePort, sourceBounds);
                }
                if (targetNode) {
                    var startPoint;
                    var segment = connector.segments[connector.segments.length - 1];
                    if (segment.type !== "orthogonal") startPoint = segment.type !== "bezier" ? segment._startPoint : segment._point2;
                    else startPoint = segment.points[segment.points.length - 2];
                    if (targetPort) length = connector.targetPadding + targetPort.connectorPadding;
                    var targetPortLocation = this._adjustPoint(startPoint, segment._endPoint, false, length);
                    this._setLineEndPoint(connector, targetPortLocation, true);
                    if (segment.type == "bezier") this._updateBezierPoints(segment, targetNode, targetPort, targetBounds, sourceNode, sourcePort, sourceBounds);
                }
            }
        },
        _findEndPoint: function (direction, point, node) {
            var targetPoint;
            switch (direction) {
                case "top":
                    targetPoint = { x: point.x, y: point.y + Math.max(node.width, node.height) };
                    break;
                case "bottom":
                    targetPoint = { x: point.x, y: point.y - Math.max(node.width, node.height) };
                    break;
                case "left":
                    targetPoint = { y: point.y, x: point.x + Math.max(node.width, node.height) };
                    break;
                case "right":
                    targetPoint = { y: point.y, x: point.x - Math.max(node.width, node.height) };
                    break;
            }
            return targetPoint;
        },
        _dock: function (connector, line, targetNode, sourceNode, isTarget) {
            var sourceBounds;
            if (targetNode) {
                var node = targetNode;
                var point = isTarget ? line._startPoint : line._endPoint;
                sourceBounds = this.bounds(targetNode);
                this._swapBounds(targetNode, sourceBounds);
                if (!isTarget) {
                    if (line.type == "orthogonal" && connector.segments[1] && connector.segments[1].type == "orthogonal") {
                        if (connector.segments[0]._isInternal && ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line._endPoint)) {
                            this._insertSegmentAtSourceEnd(connector, line, sourceBounds, targetNode, isTarget);
                        }
                        else if (ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line._endPoint)) {
                            this._removeFirstSegment(connector, line, sourceBounds, targetNode, isTarget);
                        }
                        else {
                            var dockPoint, dockDirection;;
                            switch (line.direction) {
                                case "left":
                                    dockDirection = "left";
                                    dockPoint = sourceBounds.middleLeft;
                                    break;
                                case "right": dockDirection = "right";
                                    dockPoint = sourceBounds.middleRight;
                                    break;
                                case "top": dockDirection = "top";
                                    dockPoint = sourceBounds.topCenter;
                                    break;
                                case "bottom": dockDirection = "bottom";
                                    dockPoint = sourceBounds.bottomCenter;
                                    break;
                            }
                            line.points = [];
                            var targetPoint = this._findEndPoint(dockDirection, dockPoint, targetNode) || sourceBounds.center;
                            dockPoint = this._findIntersection(targetNode, sourceBounds, connector, dockPoint, targetPoint, isTarget) || dockPoint;
                            line.points.push(dockPoint);
                            line.points.push(line._endPoint);
                            this._setLineEndPoint(connector, dockPoint, isTarget);
                        }
                    }
                    else {
                        this._dockNodeToPoint(connector, line._endPoint, targetNode, isTarget);
                    }
                }
                else {
                    if (line.type == "orthogonal") {
                        var second = connector.segments[connector.segments.length - 2];
                        if (second && second.type == "orthogonal") {
                            if (line.points.length > 2 &&
                                ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line.points[line.points.length - 2])) {
                                var removePreSegment = false;
                                if (second.direction == "top" || second.direction == "bottom") {
                                    if (second._endPoint.x >= sourceBounds.left && second._endPoint.x <= sourceBounds.right) {
                                        this._removeLastSegment(connector, line, second, targetNode, sourceNode, isTarget); return;
                                    }
                                    else {
                                        second._length = second.length = Math.abs(second._startPoint.y - sourceBounds.middleLeft.y);;
                                        if (second.length == 0) {
                                            removePreSegment = true;
                                        }
                                        else {
                                            if (second.length < 0) {
                                                second._length = second.length *= -1;
                                                second._direction = second.direction = (second.direction == "top") ? "bottom" : "top";
                                            }
                                            connector.targetPoint = second._endPoint.x > sourceBounds.right ? sourceBounds.middleRight : sourceBounds.middleLeft;
                                        }
                                    }
                                }
                                else {
                                    if (second._endPoint.y >= sourceBounds.top && second._endPoint.y <= sourceBounds.bottom) {
                                        this._removeLastSegment(connector, line, second, targetNode, sourceNode, isTarget); return;
                                    }
                                    else {
                                        second._length = second.length = Math.abs(second._startPoint.x - sourceBounds.topCenter.x);;
                                        if (second.length == 0) removePreSegment = true;
                                        else {
                                            if (second.length < 0) {
                                                second._length = second.length *= -1;
                                                second._direction = second.direction = (second.direction == "left") ? "right" : "left";
                                            }
                                            connector.targetPoint = second._endPoint.y > sourceBounds.bottom ? sourceBounds.bottomCenter : sourceBounds.topCenter;
                                        }
                                    }
                                }
                                if (removePreSegment) {
                                    connector.segments.splice(connector.segments.length - 2, 1);
                                    this._dock(connector, line, targetNode, sourceNode, isTarget);
                                    return;
                                }
                                connector.targetPoint = this._findIntersection(targetNode, sourceBounds, connector, connector.targetPoint, sourceBounds.center, isTarget) || connector.targetPoint;
                                this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
                                line._direction = this._getOrthoDirection(line._startPoint, line._endPoint, second);
                                this._setLineEndPoint(connector, connector.targetPoint, isTarget);
                            }
                            else if (ej.datavisualization.Diagram.Geometry.containsPoint(sourceBounds, line._startPoint)) {
                                var diflength = second.direction == "left" || second.direction == "right" ? node.width / 2 : node.height / 2;
                                second._length = second.length -= diflength + 30;
                                if (second.length == 0) {
                                    connector.segments.splice(connector.segments.length - 2, 1);
                                    this._dock(connector, line, targetNode, sourceNode, isTarget);
                                    return;
                                }
                                if (second.direction == "left" || second.direction == "right")
                                    connector.targetPoint = second.direction == "right" ? sourceBounds.middleLeft : sourceBounds.middleRight;
                                else
                                    connector.targetPoint = second.direction == "bottom" ? sourceBounds.topCenter : sourceBounds.bottomCenter;
                                connector.targetPoint = this._findIntersection(targetNode, sourceBounds, connector, connector.targetPoint, sourceBounds.center, isTarget) || connector.targetPoint;
                                this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
                                line._direction = this._getOrthoDirection(line._startPoint, line._endPoint, second);
                                this._setLineEndPoint(connector, connector.targetPoint, isTarget);
                            }
                            else {
                                this._dockNodeToPoint(connector, point, targetNode, isTarget);
                            }
                        } else {
                            this._dockNodeToPoint(connector, point, targetNode, isTarget);
                        }
                    }
                    else { this._dockNodeToPoint(connector, point, targetNode, isTarget); }
                }
            }
        },
        _removeLastSegment: function (connector, line, second, targetNode, sourceNode, isTarget) {
            connector.segments.splice(connector.segments.length - 1, 1);
            second.length = second._length = null;
            second.direction = null;
            second._direction = line._direction;
            this._dock(connector, second, targetNode, sourceNode, isTarget);
        },
        _insertSegmentAtSourceEnd: function (connector, line, sourceBounds, sourceNode, isTarget) {
            var second = connector.segments[1];
            var third = connector.segments[2];
            if (connector.segments[0].direction == "left" || connector.segments[0].direction == "right") {
                var orientation = "horizontal";
                connector.sourcePoint = second.direction == "top" ? sourceBounds.topCenter : sourceBounds.bottomCenter;
                if (second && (second.length || second.length === 0))
                    second._length = second.length = Math.max(Math.abs(connector.sourcePoint.y - second._endPoint.y), 25);
                if (third && (third.length || third.length === 0)) {
                    third.length = third._length = Math.abs(sourceBounds.topCenter.x - third._endPoint.x);
                }
            }
            else {
                var orientation = "vertical";
                connector.sourcePoint = second.direction == "left" ? sourceBounds.middleLeft : sourceBounds.middleRight;
                if (second && (second.length || second.length === 0))
                    second._length = second.length = Math.max(Math.abs(connector.sourcePoint.x - second._endPoint.x), 25);
                if (third && (third.length || third.length === 0)) {
                    third.length = third._length = Math.abs(sourceBounds.middleRight.y - third._endPoint.y);
                }
            }
            //Updating adjacent segments
            if (connector.segments[3] && (connector.segments[3].length || connector.segments[3].length == 0)) {
                var value = second.direction == "bottom" || second.direction == "right" ? second.length : -second.length;
                connector.segments[3].length = connector.segments[3]._length = orientation == "vertical" ?
                    Math.abs(connector.sourcePoint.x + value - connector.segments[3]._endPoint.x) :
                    Math.abs(connector.sourcePoint.y + value - connector.segments[3]._endPoint.y);
                if (connector.segments[3].length < 0) {
                    var value = second.direction == "bottom" || second.direction == "right" ? second.length : -second.length;
                    connector.segments[3].length = connector.segments[3]._length = orientation == "vertical" ?
                        Math.abs(connector.sourcePoint.x + value - connector.segments[3]._endPoint.x) :
                    Math.abs(connector.sourcePoint.y + value - connector.segments[3]._endPoint.y);
                }
            }
            connector.segments.splice(0, 1);
            connector.sourcePoint = this._findIntersection(sourceNode, sourceBounds, connector, connector.sourcePoint, sourceBounds.center, isTarget) || connector.sourcePoint;
            this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
            this._setLineEndPoint(connector, connector.sourcePoint, isTarget);
        },
        _removeFirstSegment: function (connector, line, sourceBounds, sourceNode, isTarget) {
            var newLength, firstSegLength;
            var direction = connector.segments[1].direction || this._getBezierDirection(connector.segments[1].points[0], connector.segments[1].points[1]);
            if (direction == "left" || direction == "right") {
                var length = line._endPoint.y - sourceBounds.middleRight.y;
                connector.segments[0].direction = connector.segments[0]._direction = length >= 0 ? "bottom" : "top";
                connector.segments[0].length = connector.segments[0]._length = Math.abs(length);
                var actualdist = direction == "right" ? connector.segments[1]._endPoint.x - sourceBounds.right :
                    sourceBounds.left - connector.segments[1]._endPoint.x;
                connector.sourcePoint = direction == "left" ? sourceBounds.middleLeft : sourceBounds.middleRight;
                newLength = Math.abs(connector.sourcePoint.x - connector.segments[0]._startPoint.x);
                firstSegLength = Math.abs(connector.sourcePoint.y - connector.segments[0]._endPoint.y);
            }
            else {
                if (connector.segments[0].length || connector.segments[0].length == 0) {
                    var length = line._endPoint.x - sourceBounds.bottomCenter.x;
                    connector.segments[0].direction = connector.segments[0]._direction = length >= 0 ? "right" : "left";
                    connector.segments[0].length = connector.segments[0]._length = Math.abs(length);
                }
                var actualdist = direction == "bottom" ? connector.segments[1]._endPoint.y - sourceBounds.bottom :
                    sourceBounds.top - connector.segments[1]._endPoint.y;
                connector.sourcePoint = direction == "top" ? sourceBounds.topCenter : sourceBounds.bottomCenter;
                newLength = Math.abs(connector.sourcePoint.y - connector.segments[0]._startPoint.y);
                firstSegLength = Math.abs(connector.sourcePoint.x - connector.segments[0]._endPoint.x);
            }
            if (connector.segments[1] && (connector.segments[1].length || connector.segments[1].length === 0)) {
                connector.segments[1]._length = connector.segments[1].length = actualdist - 25;
                if (connector.segments[1]._length < 0) {
                    connector.segments.splice(0, 2);
                    if (connector.segments[0].length || connector.segments[0].length === 0)
                        connector.segments[0].length = connector.segments[0]._length = newLength;
                    var newSegment = ej.datavisualization.Diagram.Segment({ type: "orthogonal", length: firstSegLength, direction: direction });
                }
            }
            if (!newSegment) {
                newSegment = ej.datavisualization.Diagram.Segment({ type: "orthogonal", length: 25, direction: direction });
            }
            var segments = [newSegment];
            newSegment._isInternal = true;
            connector.segments = segments.concat(connector.segments);
            connector.sourcePoint = this._findIntersection(sourceNode, sourceBounds, connector, connector.sourcePoint, sourceBounds.center, isTarget) || connector.sourcePoint;
            this._updateConnectorSegments(connector, connector.sourcePort, connector.targetPort);
            this._setLineEndPoint(connector, connector.sourcePoint, isTarget);
        },
        _dockNodeToPoint: function (connector, fixedPoint, node, isTarget) {
            var line, sourceBounds, segment;
            line = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            if (line.type != "orthogonal") {
                sourceBounds = this.bounds(node);
                this._swapBounds(node, sourceBounds);
                this._setLineEndPoint(connector, sourceBounds.center, isTarget);
                this._dockBounds(connector, node, sourceBounds, isTarget);
            }
            else {
                sourceBounds = this.bounds(node);
                this._swapBounds(node, sourceBounds);
                var dockPoint, dockDirection, targetDirection;
                var point = fixedPoint;
                var prev = connector.segments[connector.segments.length - 2];
                var left = Math.abs(point.x - sourceBounds.left);
                var right = Math.abs(point.x - sourceBounds.right);
                var top = Math.abs(point.y - sourceBounds.top);
                var bottom = Math.abs(point.y - sourceBounds.bottom);
                if (prev && isTarget) {
                    var horizontal = prev.direction == "left" || prev.direction == "right";
                    var vertical = prev.direction == "top" || prev.direction == "bottom";
                }
                line = !isTarget ? connector.segments[0] : connector.segments[connector.segments.length - 1];
                if ((line.points.length == 2 || !prev) && line.type == "orthogonal") {
                    horizontal = vertical = false;
                }
                var targetPoint;
                if (sourceBounds.bottom < point.y && !horizontal) {
                    dockDirection = "bottom";
                    targetDirection = "top";
                    dockPoint = sourceBounds.bottomCenter;
                }
                else if (sourceBounds.y > point.y && !horizontal) {
                    dockDirection = "top";
                    targetDirection = "bottom";
                    dockPoint = sourceBounds.topCenter;
                }
                else if (sourceBounds.right < point.x && !vertical) {
                    dockDirection = "right";
                    targetDirection = "left";
                    dockPoint = sourceBounds.middleRight;
                }
                else if (sourceBounds.x > point.x && !vertical) {
                    dockDirection = "left";
                    targetDirection = "right";
                    dockPoint = sourceBounds.middleLeft;
                }
                else {
                    var min = left;
                    dockPoint = sourceBounds.middleLeft;
                    dockDirection = "left";
                    targetDirection = "right";
                    if (min >= right && right != 0) {
                        dockPoint = sourceBounds.middleRight;
                        dockDirection = "right";
                        targetDirection = "left";
                        min = right;
                    }
                    if (min > top && top != 0) {
                        dockPoint = sourceBounds.topCenter;
                        dockDirection = "top";
                        targetDirection = "bottom";
                        min = top;
                    }
                    if (min > bottom && bottom != 0) {
                        dockPoint = sourceBounds.bottomCenter;
                        dockDirection = "bottom";
                        targetDirection = "top";
                        min = bottom;
                    }
                }
                if (dockPoint) {
                    if (isTarget || connector.segments.length == 1) {
                        segment = connector.segments[connector.segments.length - 1];
                        if (segment.type == "orthogonal") {
                            segment._direction = isTarget ? targetDirection : dockDirection;
                        }
                    }
                    else { segment = connector.segments[0]; }
                    targetPoint = this._findEndPoint(dockDirection, dockPoint, node) || sourceBounds.center;
                    dockPoint = this._findIntersection(node, sourceBounds, connector, dockPoint, targetPoint, isTarget) || dockPoint;
                    isTarget ? connector.targetPoint = dockPoint : connector.sourcePoint = dockPoint;
                    if (connector.segments.length != 1)
                        this._addOrthogonalPoints(segment, !isTarget ? null : connector.segments[connector.segments.length - 2], !isTarget ? connector.segments[1] : null, connector.sourcePoint, connector.targetPoint);
                    this._setLineEndPoint(connector, dockPoint, isTarget);
                }
            }
        },
        _getDockSegment: function (connector, line, rotateAngle, bounds, isTarget, direction) {
            var segment; var segments = this._getEdges(rotateAngle, bounds);
            {
                var direction = "", segment, intersecSeg;
                //var line = connector.line;
                var length = ej.datavisualization.Diagram.Geometry.distance(line._startPoint, line._endPoint);
                if (isTarget)
                    intersecSeg = ej.datavisualization.Diagram.LineSegment(line.points[line.points.length - 2], line.points[line.points.length - 1]);
                else {
                    if (length > 5) {
                        intersecSeg = ej.datavisualization.Diagram.LineSegment(line._startPoint, line._endPoint);
                    }
                    else {
                        intersecSeg = ej.datavisualization.Diagram.LineSegment(line._startPoint, connector.segments[1] ? connector.segments[1]._endPoint : connector.targetPoint);
                    }
                }
                if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.left)) {
                    segment = segments.left;
                    direction = "left";
                }
                else if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.top)) {
                    segment = segments.top;
                    direction = "top";
                }
                else if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.right)) {
                    segment = segments.right;
                    direction = "right";
                }
                else if (ej.datavisualization.Diagram.Geometry.intersectSegment(intersecSeg, segments.bottom)) {
                    segment = segments.bottom;
                    direction = "bottom";
                }
            }
            direction = this._swapDirection(rotateAngle, direction);
            if (segment) {
                return { "dockPoint": ej.datavisualization.Diagram.Geometry.midPoint(segment), "direction": direction };
            }
            return null;
        },
        _dockBounds: function (connector, node, bounds, isTarget) {
            var line;
            line = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            var strPt = line._startPoint;
            var endPt = line._endPoint || line.point;
            if (line.type == "bezier" && (line.point1 || line.point2)) {
                if (isTarget) strPt = line._point2;
                else endPt = line._point1;
            }
            var intersection = this._findIntersection(node, bounds, connector, strPt, endPt, isTarget);
            if (intersection) {
                this._setLineEndPoint(connector, intersection, isTarget);
                if (line.type === "bezier") {
                    this._updateBezierPoints(line);
                }
            }
        },
        _findIntersection: function (node, bounds, connector, strPt, endPt, isTarget) {
            var segmentPoints, length;
            var line = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            var pad = node.connectorPadding;
            pad += isTarget ? connector.targetPadding : connector.sourcePadding;
            if (line.type == "orthogonal") {
                strPt = { x: strPt.x, y: strPt.y };
                if (strPt.x == endPt.x) if (strPt.y < endPt.y) strPt.y -= pad + 5; else strPt.y += pad + 5;
                if (strPt.y == endPt.y) if (strPt.x < endPt.x) strPt.x -= pad + 5; else strPt.x += pad + 5;
            }
            var angle;
            var point = isTarget || line.type == "orthogonal" ? strPt : endPt;
            if (line.type != "orthogonal") {
                if (isTarget) {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(strPt, endPt);
                    endPt = ej.datavisualization.Diagram.Geometry.transform({ x: endPt.x, y: endPt.y }, angle, Math.max(node.width / 2, node.height / 2));
                }
                else {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(endPt, strPt);
                    strPt = ej.datavisualization.Diagram.Geometry.transform({ x: strPt.x, y: strPt.y }, angle, Math.max(node.width / 2, node.height / 2));
                }
            }
            if (node._shape === "ellipse") {
                var thisSegment = ej.datavisualization.Diagram.LineSegment(strPt, endPt);
                intersection = this._getEllipseIntersectCoords(node, thisSegment, connector, point, isTarget);
                return intersection;
            }
            else if (node._shape === "polygon" && !(node.type == "bpmn" && line.type == "orthogonal")) {
                segmentPoints = [];
                if (!node._segmentPoints) {
                    segmentPoints = this._findSegmentPoints(node, connector, isTarget);
                } else segmentPoints = node._segmentPoints;

                if (node.rotateAngle) {
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                }
                length = segmentPoints.length;
                var pts = [];
                for (var i = 0; i < length; i++) {
                    var pt1 = ej.datavisualization.Diagram.Point(segmentPoints[i].x + (node.offsetX - node.width * node.pivot.x), segmentPoints[i].y + (node.offsetY - node.height * node.pivot.y));
                    if (node.rotateAngle != 0) {
                        pt1 = ej.Matrix.transform(matrix, pt1);
                    }
                    if (node.connectorPadding || connector.sourcePadding || connector.targetPadding) {
                        var x = node.offsetX == pt1.x ? 0 : (node.offsetX < pt1.x ? 1 : -1);
                        var y = node.offsetY == pt1.y ? 0 : (node.offsetY < pt1.y ? 1 : -1);
                        pt1 = pad ? ej.datavisualization.Diagram.Geometry.translate(pt1, x * pad, y * pad) : pt1;
                    }
                    pts[i] = pt1;
                }
                segmentPoints = pts;
            }
            else if (node._shape === "path" && !(node.type == "bpmn" && line.type == "orthogonal")) {
                if (!node._segmentPoints) {
                    var pts = this._findSegmentPoints(node, connector, isTarget);
                }
                segmentPoints = pts || node._segmentPoints;
                if (((node.pathData.split("m").length - 1) + (node.pathData.split("M").length - 1)) == 1) segmentPoints[segmentPoints.length] = segmentPoints[0];
                length = segmentPoints.length;
            }
            else {
                if (node.connectorPadding || connector.sourcePadding || connector.targetPadding) {
                    var rect = ej.datavisualization.Diagram.Rectangle(bounds.x - pad, bounds.y - pad, node.width + 2 * pad, node.height + 2 * pad);
                    bounds = this.bounds(rect);
                }
                segmentPoints = bounds.points;
                segmentPoints[segmentPoints.length] = segmentPoints[0];
                length = bounds.points.length;
            }
            var thisSegment = ej.datavisualization.Diagram.LineSegment(strPt, endPt);
            var intersection = this._getIntersectionPoints(thisSegment, segmentPoints, true, point);
            return intersection;
        },
        _getIntersectionPoints: function (thisSegment, segmentPoints, minimal, point) {
            var i;
            var length = segmentPoints.length;
            var segment = ej.datavisualization.Diagram.LineSegment(segmentPoints[0], segmentPoints[1]);
            var intersection = ej.datavisualization.Diagram.Geometry.intersectSegment(thisSegment, segment);
            if (intersection) {
                if (!minimal) return intersection;
                var min = ej.datavisualization.Diagram.Geometry.distance(intersection, point);
            }
            //if (intersection)
            if (isNaN(min) || min > 0) {
                for (i = 1; i < length - 1 ; i++) {
                    segment = ej.datavisualization.Diagram.LineSegment(segmentPoints[i], segmentPoints[i + 1]);
                    var intersect = ej.datavisualization.Diagram.Geometry.intersectSegment(thisSegment, segment);
                    if (intersect) {
                        if (!minimal) return intersect;
                        var dist = ej.datavisualization.Diagram.Geometry.distance(intersect, point);
                        if (isNaN(min) || min > dist) { min = dist; intersection = intersect; }
                        if (min >= 0 && min <= 1)
                            break;
                    }
                }
            }
            return intersection;
        },

        _findSegmentPoints: function (node, connector, isTarget) {
            var pts = [];
            if (node._shape == "path") {
                var svg = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
                svg.setAttribute('xlink', 'http://www.w3.org/1999/xlink');
                document.body.appendChild(svg);
                var pathNode = document.createElementNS('http://www.w3.org/2000/svg', "path");
                pathNode.setAttributeNS(null, "d", node._absolutePath || node.pathData);
                svg.appendChild(pathNode);
                var pathBounds = (!node._absoluteBounds || node._scaled || node._absolutePath) ? pathNode.getBBox() : node._absoluteBounds;
                var pad = node.connectorPadding;
                if (connector)
                    pad += isTarget ? connector.targetPadding : connector.sourcePadding;
                if (pad || !node._absolutePath || node._scaled) {
                    var pathData = ej.datavisualization.Diagram.Geometry.updatePath(-pad, -pad, node.width + 2 * pad, node.height + 2 * pad, node._absolutePath || node.pathData, null, pathBounds, pad ? null : node);
                    pathNode.setAttributeNS(null, "d", pathData);
                    if ((node._scaled && !pad) || !node._absolutePath) {
                        node._absolutePath = pathData;
                        node._scaled = false;
                    }
                } else pathNode.setAttributeNS(null, "d", node._absolutePath);
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                var pathLength = pathNode.getTotalLength();
                for (var sample, sampleLength = 0; sampleLength <= pathLength; sampleLength += 10) {
                    sample = pathNode.getPointAtLength(sampleLength);
                    var pt1 = { x: node.offsetX - node.width * node.pivot.x + sample.x, y: node.offsetY - node.height * node.pivot.y + sample.y };
                    if (node.rotateAngle != 0) {
                        pt1 = ej.Matrix.transform(matrix, pt1);
                    }
                    pts.push(pt1);
                }
                document.body.removeChild(svg);
            }
            else if (node._shape == "polygon") {
                var length = node.points.length;
                for (var i = 0; i < length; i++) pts[i] = node.points[i];
                pts[pts.length] = pts[0];
                ej.datavisualization.Diagram.Geometry.updatePolygonPoints(node, pts);
            } else return null;
            return pts;
        },
        _getEdges: function (rotateAngle, bounds) {
            var points = bounds.points;
            var segments = {};
            var angle = rotateAngle % 360;
            var temp;
            segments.left = ej.datavisualization.Diagram.LineSegment(points[3], points[0]);
            segments.top = ej.datavisualization.Diagram.LineSegment(points[0], points[1]);
            segments.right = ej.datavisualization.Diagram.LineSegment(points[1], points[2]);
            segments.bottom = ej.datavisualization.Diagram.LineSegment(points[2], points[3]);
            if (angle < 0) {
                angle += 360;
            }
            if (angle > 315 || angle < 45) {
            }
            else if (angle < 135) {
                temp = segments.top;
                segments.top = segments.left;
                segments.left = segments.bottom;
                segments.bottom = segments.right;
                segments.right = temp;
            }
            else if (angle < 225) {
                temp = segments.top;
                segments.top = segments.bottom;
                segments.bottom = temp;
                temp = segments.left;
                segments.left = segments.right;
                segments.right = temp;
            }
            else {
                temp = segments.bottom;
                segments.bottom = segments.left;
                segments.left = segments.top;
                segments.top = segments.right;
                segments.right = temp;
            }
            return segments;
        },
        _pointsFromNodeToPoint: function (connector, segment, direction, bounds, angle, point, endPoint, isTarget) {
            var dir, num;
            var minSpace = connector.segments.length > 1 ? 0 : 15;
            var points = [];
            points.push(point);
            var straight;
            straight = (point.y == endPoint.y && (direction == "left" && endPoint.x < point.x || direction == "right" && endPoint.x > point.x)) ||
               (point.x == endPoint.x && (direction == "top" && endPoint.y < point.y || direction == "bottom" && endPoint.y > point.y));
            //if (angle == 0)
            //if (!straight && !isTarget) straight = point.x > bounds.left && point.x < bounds.right && point.y > bounds.top && point.y < bounds.bottom;
            if (!straight) {
                if (direction == "top" || direction == "bottom") {
                    if (direction == "top" && endPoint.y < point.y && endPoint.y > point.y - minSpace || direction == "bottom" && endPoint.y > point.y && endPoint.y < point.y + minSpace) {
                        var y = direction == "top" ? bounds.top - minSpace : bounds.bottom + minSpace;
                        points.push({ x: point.x, y: y });
                        points.push({ x: point.x + (endPoint.x - point.x) / 2, y: y });
                        points.push({ x: point.x + (endPoint.x - point.x) / 2, y: endPoint.y });
                    }
                    else if (Math.abs(point.x - endPoint.x) > minSpace && (direction == "top" && endPoint.y < point.y || direction == "bottom" && endPoint.y > point.y)) {
                        //twosegments
                        points.push({ x: point.x, y: endPoint.y });
                    }
                    else if (Math.abs(point.x - endPoint.x) <= minSpace || ((endPoint.x < bounds.left || endPoint.x > bounds.right) && endPoint.y != point.y)) {
                        //three segments
                        var y = direction == "top" ? bounds.top - minSpace : bounds.bottom + minSpace;
                        points.push({ x: point.x, y: y });
                        points.push({ x: endPoint.x, y: y });
                    }
                    else {
                        //four segments
                        var y = direction == "top" ? bounds.top - minSpace : bounds.bottom + minSpace;
                        var x = (endPoint.x < point.x) ? bounds.left - minSpace : bounds.right + minSpace;
                        points.push({ x: point.x, y: y });
                        points.push({ x: x, y: y });
                        points.push({ x: x, y: endPoint.y });
                    }
                }
                else {
                    if (direction == "left" && endPoint.x < point.x && endPoint.x > point.x - minSpace || direction == "right" && endPoint.x > point.x && endPoint.x < point.x + minSpace) {
                        var x = direction == "left" ? bounds.left - minSpace : bounds.right + minSpace;
                        points.push({ x: x, y: point.y });
                        points.push({ x: x, y: point.y + (endPoint.y - point.y) / 2 });
                        points.push({ x: endPoint.x, y: point.y + (endPoint.y - point.y) / 2 });
                    }
                    else if (Math.abs(point.y - endPoint.y) > minSpace && (direction == "left" && endPoint.x < point.x || direction == "right" && endPoint.x > point.x)) {
                        points.push({ x: endPoint.x, y: point.y });
                        //twosegments
                    }
                    else if (Math.abs(point.y - endPoint.y) <= minSpace || ((endPoint.y < bounds.top || endPoint.y > bounds.bottom) && endPoint.x != point.x)) {
                        //three segments
                        var x = direction == "left" ? bounds.left - minSpace : bounds.right + minSpace;
                        points.push({ x: x, y: point.y });
                        points.push({ x: x, y: endPoint.y });
                    }
                    else {
                        //four segments
                        var y = (endPoint.y < point.y) ? bounds.top - minSpace : bounds.bottom + minSpace;
                        var x = direction == "left" ? bounds.left - minSpace : bounds.right + minSpace;
                        points.push({ x: x, y: point.y });
                        points.push({ x: x, y: y });
                        points.push({ x: endPoint.x, y: y });
                    }
                }
            }
            points.push(endPoint);
            if (isTarget) points.reverse();
            segment._startPoint = points[0];
            segment._endPoint = points[points.length - 1];
            segment.points = points;
            if (segment.direction) {
                if (segment.direction == "left" || segment.direction == "right") {
                    segment.length = segment._length = Math.abs(segment._startPoint.x - segment._endPoint.x);
                    if (segment._startPoint.x < segment._endPoint.x) segment._direction = segment.direction = "right"; else segment._direction = segment.direction = "left";
                }
                else {
                    segment.length = segment._length = Math.abs(segment._startPoint.y - segment._endPoint.y);
                    if (segment._startPoint.y < segment._endPoint.y) segment._direction = segment.direction = "bottom"; else segment._direction = segment.direction = "top";
                }
            }
        },
        _constructSegements: function (connector, segment, targetBounds, targetPortLocation, targetDirection, sourceBounds, sourcePortLocation, sourceDirection) {
            var points, reverse;
            if (sourceDirection === "right") {
                if (targetDirection === "left") {
                    points = this._rightToLeft(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, false);
                }
                else if (targetDirection === "right") {
                    points = this._rightToRight(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                else if (targetDirection === "top") {
                    points = this._rightToTop(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, false);
                }
                else if (targetDirection === "bottom") {
                    points = this._rightToBottom(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, false);
                }
            }
            if (sourceDirection === "left") {
                if (targetDirection === "right") {
                    points = this._rightToLeft(connector, sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    reverse = true;
                }
                if (targetDirection === "left") {
                    points = this._leftToLeft(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                if (targetDirection === "top") {
                    points = this._leftToTop(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, false);
                }
                if (targetDirection === "bottom") {
                    points = this._leftToBottom(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, false);
                }
            }
            if (sourceDirection === "top") {
                if (targetDirection === "left") {
                    points = this._leftToTop(connector, sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    reverse = true;
                }
                if (targetDirection === "right") {
                    points = this._rightToTop(connector, sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    reverse = true;
                }
                if (targetDirection === "top") {
                    points = this._topToTop(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
                if (targetDirection === "bottom") {
                    points = this._topToBottom(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, false);
                }
            }
            if (sourceDirection === "bottom") {
                if (targetDirection === "left") {
                    points = this._leftToBottom(connector, sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    reverse = true;
                }
                if (targetDirection === "right") {
                    points = this._rightToBottom(connector, sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    reverse = true;
                }
                if (targetDirection === "top") {
                    points = this._topToBottom(connector, sourceBounds, sourcePortLocation, targetBounds, targetPortLocation, true);
                    reverse = true;
                }
                if (targetDirection === "bottom") {
                    points = this._bottomToBottom(connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation);
                }
            }
            if (reverse) {
                points.reverse();
            }
            var point1 = points[points.length - 1];
            var point2 = points[points.length - 2];
            while (points.length > 2 && point1.x == point2.x && point1.y == point2.y) {
                points.splice(points.length - 1, 1);
                point1 = points[points.length - 1];
                point2 = points[points.length - 2];
            }
            point1 = points[0];
            point2 = points[1];
            while (points.length > 2 && point1.x == point2.x && point1.y == point2.y) {
                points.splice(0, 1);
                point1 = points[0];
                point2 = points[1];
            }
            segment._startPoint = points[0];
            segment._endPoint = points[points.length - 1];
            segment.points = points;
            //this._setPoints(connector.line, points);
        },
        _updateIntermediateSegments: function (connector) {
            var startPoint = connector.segments[0]._endPoint;
            var prev = connector.segments[0];
            var length = prev.points.length;
            var next = null;
            var runAngle;
            if (connector.segments[1]) {
                var segment = connector.segments[1];
                segment._startPoint = startPoint;
                if (segment.type == "orthogonal" && (segment.length || segment.length === 0)) {
                    if (segment.direction == "left" || segment.direction == "right") {
                        if (connector.segments[2]) {
                            if (connector.segments[2].length || connector.segments[2].length === 0) {
                                if (connector.segments[2].direction == "top")
                                    connector.segments[2].length += segment._startPoint.y - segment._endPoint.y;
                                else
                                    connector.segments[2].length -= segment._startPoint.y - segment._endPoint.y;
                                if (connector.segments[2].length < 0) {
                                    connector.segments[2].length *= -1;
                                    connector.segments[2].direction = connector.segments[2]._direction = segment.direction == "right" ? "left" : "right";
                                }
                                connector.segments[2]._length = connector.segments[2].length;
                            }
                        }
                        segment._endPoint.y = segment._startPoint.y;
                    }
                    else {
                        if (connector.segments[2]) {
                            if (connector.segments[2].length || connector.segments[2].length === 0) {
                                if (connector.segments[2].direction == "left")
                                    connector.segments[2].length += segment._startPoint.x - segment._endPoint.x;
                                else
                                    connector.segments[2].length -= segment._startPoint.x - segment._endPoint.x;
                                if (connector.segments[2].length < 0) {
                                    connector.segments[2].length *= -1;
                                    connector.segments[2].direction = connector.segments[2]._direction = segment.direction == "bottom" ? "top" : "bottom";
                                }
                                connector.segments[2]._length = connector.segments[2].length;
                            }
                        }
                        segment._endPoint.x = segment._startPoint.x;
                    }
                    if (connector.sourcePort && prev.points.length > 2) {
                        if (segment.direction == "left" || segment.direction == "right") {
                            if (prev.points[prev.points.length - 1].y == prev.points[prev.points.length - 2].y) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                                if (segment.points.length == 2) {
                                    segment._endPoint.y = segment._startPoint.y;
                                }
                            }
                        }
                        else {
                            if (prev.points[prev.points.length - 1].x == prev.points[prev.points.length - 2].x) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                                if (segment.points.length == 2) {
                                    segment._endPoint.x = segment._startPoint.x;
                                }
                            }
                        }
                    }
                    if (segment.length || segment.length === 0)
                        segment.length = segment._length = ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment.points[1]);
                    if (segment.length > 0)
                        segment.direction = segment._direction = this._getBezierDirection(segment._startPoint, segment._endPoint) || segment._direction;
                    this._addOrthogonalPoints(segment, prev, connector.segments[2], connector.sourcePoint, connector.targetPoint);
                    if (connector.segments[2])
                        this._addOrthogonalPoints(connector.segments[2], connector.segments[1], connector.segments[3], connector.sourcePoint, connector.targetPoint);
                }
                else if (segment.type == "orthogonal" && segment.length === null) {
                    if (connector.sourcePort && prev.points.length > 2) {
                        if (segment._direction == "left" || segment._direction == "right") {
                            if (prev.points[prev.points.length - 1].y == prev.points[prev.points.length - 2].y) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                            }
                        }
                        else {
                            if (prev.points[prev.points.length - 1].x == prev.points[prev.points.length - 2].x) {
                                segment._startPoint = segment.points[0] = prev.points[prev.points.length - 2];
                                prev.points.splice(prev.points.length - 1, 1);
                                prev._endPoint = prev.points[prev.points.length - 1];
                            }
                        }
                    }
                    segment._startPoint = segment.points[0] = connector.segments[0]._endPoint;
                    this._addOrthogonalPoints(connector.segments[1], connector.segments[0], null, connector.sourcePoint, connector.targetPoint);
                }
                else {
                    segment._startPoint = segment.points[0] = connector.segments[0]._endPoint;
                    if (segment.type == "bezier") {
                        this._updateBezierPoints(segment);
                    }
                }
            }
        },
        _updatePreviousSegment: function (connector) {
            var previousSegment = connector.segments[connector.segments.length - 2];
            var current = connector.segments[connector.segments.length - 1];
            if (previousSegment) {
                if (previousSegment.type == "orthogonal" && current.points.length > 2) {
                    var update = false;
                    var direction = previousSegment.points.length == 2 ? previousSegment.direction :
                        ej.datavisualization.Diagram.Util._getBezierDirection(previousSegment.points[previousSegment.points.length - 2],
                        previousSegment.points[previousSegment.points.length - 1]);
                    if (direction == "left" || direction == "right") {
                        if (current.points[0].y == current.points[1].y) {
                            update = true;
                            var dif = current.points[1].x - current.points[0].x;
                            current.points.splice(0, 1);
                            current._startPoint = current.points[0];
                            previousSegment._endPoint = current.points[0];
                        }
                    }
                    else {
                        if (current.points[0].x == current.points[1].x) {
                            update = true;
                            current.points.splice(0, 1);
                            current._startPoint = current.points[0];
                            previousSegment._endPoint = current.points[0];
                        }
                    }
                }
                previousSegment._endPoint = current._startPoint;
                previousSegment.points[previousSegment.points.length - 1] = current._startPoint;
                if (previousSegment.type == "orthogonal" && update) {
                    previousSegment._length = previousSegment.length = ej.datavisualization.Diagram.Geometry.distance(previousSegment._startPoint, previousSegment._endPoint);
                    if (previousSegment._length > 0)
                        previousSegment._direction = previousSegment.direction = this._getBezierDirection(previousSegment._startPoint, previousSegment._endPoint);
                    if (connector.segments.length > 2 || !connector.sourcePort)
                        this._addOrthogonalPoints(previousSegment, connector.segments[connector.segments.length - 3], current, connector.sourcePoint, connector.targetPoint);
                }
                else {
                    previousSegment.point = current._startPoint;
                }
            }
        },
        _initConnectionEnds: function (connector, diagram) {
            if (connector.segments && connector.segments[0] && connector.segments[0].type == "orthogonal" && connector.segments[0].length) {
                var port, bounds;
                if (connector.sourceNode) {
                    bounds = ej.datavisualization.Diagram.Util.bounds(diagram.nameTable[connector.sourceNode], true);
                    if (connector.sourcePort) {
                        port = ej.datavisualization.Diagram.Util.findPortByName(diagram.nameTable[connector.sourceNode], connector.sourcePort);
                        connector.sourcePoint = ej.datavisualization.Diagram.Util._getPortPosition(port, bounds);
                    }
                    else {
                        var line = connector.segments[0];
                        switch (line.direction) {
                            case "left":
                                connector.sourcePoint = bounds.middleLeft;
                                break;
                            case "right":
                                connector.sourcePoint = bounds.middleRight;
                                break;
                            case "top":
                                connector.sourcePoint = bounds.topCenter;
                                break;
                            case "bottom":
                                connector.sourcePoint = bounds.bottomCenter;
                                break;
                        }
                    }
                }
                if (connector.targetNode) {
                    bounds = ej.datavisualization.Diagram.Util.bounds(diagram.nameTable[connector.targetNode], true);
                    if (connector.targetPort) {
                        port = ej.datavisualization.Diagram.Util.findPortByName(diagram.nameTable[connector.targetNode], connector.targetPort);
                        connector.targetPoint = ej.datavisualization.Diagram.Util._getPortPosition(port, bounds);
                    }
                    else {
                        var line = connector.segments[0];
                        switch (line.direction) {
                            case "left":
                                connector.targetPoint = bounds.middleLeft;
                                break;
                            case "right":
                                connector.targetPoint = bounds.middleRight;
                                break;
                            case "top":
                                connector.targetPoint = bounds.topCenter;
                                break;
                            case "bottom":
                                connector.targetPoint = bounds.bottomCenter;
                                break;
                        }
                    }
                }
            }
        },
        _swapDirection: function (rotateAngle, direction) {
            var dir;
            var angle = rotateAngle % 360;
            if (angle < 0) {
                angle += 360;
            }
            var dirs;
            if (angle > 315 || angle < 45) {
                dir = direction;
            }
            else if (angle < 135) {
                dirs = { left: "top", top: "right", right: "bottom", bottom: "left" };
                dir = dirs[direction];
            }
            else if (angle < 225) {
                dirs = { left: "right", top: "bottom", right: "left", bottom: "top" };
                dir = dirs[direction];
            }
            else {
                dirs = { left: "bottom", top: "left", right: "top", bottom: "right" };
                dir = dirs[direction];
            }
            return dir;
        },
        _getEllipseIntersectCoords: function (node, connectorSeg, connector, point, isTarget) {
            var segmentPoints = [], intersection;
            var pad = node.connectorPadding;
            if (connector) pad += isTarget ? connector.targetPadding : connector.sourcePadding;
            if (!pad) pad = 0;
            var rx = node.width / 2;
            var ry = node.height / 2;
            //var cx = node.offsetX - node.width * node.pivot.x - pad;
            //var cy = node.offsetY - node.height * node.pivot.y - pad;
            var matrix = ej.Matrix.identity();
            ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
            //Convert Top Round Side to Path
            var topArchPath = "M " + (node.offsetX - node.width * node.pivot.x - pad) + ", " + (node.offsetY) + " A " + rx + " " + ry + " 0 0 " + 1 + " " + (node.offsetX + node.width * (1 - node.pivot.x) + pad) + ", " + (node.offsetY) +
                 " A " + rx + " " + ry + " 0 0 " + 1 + " " + (node.offsetX - node.width * node.pivot.x - pad) + ", " + (node.offsetY) + "z";
            var path = document.createElementNS('http://www.w3.org/2000/svg', "path");
            path.setAttributeNS(null, "d", topArchPath);
            var len = path.getTotalLength(), pts = [], pt1, pt2, coor;
            for (var i = 0; i < len; i = i + 3) {
                var pathSeg = path.getPointAtLength(i);
                if (node.rotateAngle != 0) {
                    pathSeg = ej.Matrix.transform(matrix, pathSeg);
                }
                pts[pts.length] = pathSeg;
            }
            if (pts[pts.length - 1] != path.getPointAtLength(len)) pts[pts.length] = path.getPointAtLength(len);
            segmentPoints = pts;
            intersection = this._getIntersectionPoints(connectorSeg, segmentPoints, true, point);
            return intersection;
        },
        _rightToLeft: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, swap) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (swap) {
                tarExtension = connector._srcDecoratorSize;
                srcExtension = connector._tarDecoratorSize;
            }
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var min = srcExtension + tarExtension;
            if (targetPortLocation.x - tarExtension > sourcePortLocation.x + srcExtension) {
                if (sourcePortLocation.y != targetPortLocation.y) {
                    if ((sourcePortLocation.x + srcExtension) >= targetBounds.left) {
                        x = sourceBounds.right + srcExtension;
                        y = sourcePortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetPortLocation.y > sourceBounds.top) {
                            y = targetPortLocation.y;
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetPortLocation.y <= sourceBounds.top) {
                            x = targetBounds.right + tarExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            y = targetBounds.top - tarExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            x = targetBounds.left - tarExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            y = targetPortLocation.y;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                        }
                    }
                    else {
                        x = sourceBounds.right + srcExtension;
                        if ((targetBounds.left - sourceBounds.right) / 2 > (srcExtension + tarExtension)) x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        y = sourcePortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetPortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                //5 segment
                if (Math.abs(targetBounds.top - sourceBounds.bottom) < min || Math.abs(targetBounds.bottom - sourceBounds.top) < min) {
                    if ((sourcePortLocation.x + srcExtension) > targetBounds.right) {
                        x = sourcePortLocation.x + srcExtension;
                        y = sourcePortLocation.y;
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                        y = sourcePortLocation.y;
                    }
                }
                else {
                    x = sourceBounds.right + srcExtension;
                    y = sourcePortLocation.y;
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if ((targetBounds.top - sourceBounds.bottom) >= min) {
                    y = targetBounds.top - tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else if (sourcePortLocation.y > targetPortLocation.y) {
                    if ((sourceBounds.top - targetBounds.bottom) < min) {
                        if (targetBounds.left > sourceBounds.right) y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2
                        else y = Math.min((targetBounds.top - tarExtension), (sourceBounds.top - srcExtension));
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    if (targetBounds.left > sourceBounds.right) y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                    else y = Math.max((targetBounds.bottom + tarExtension), (sourceBounds.bottom + srcExtension));
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                x = targetBounds.left - tarExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
                y = targetPortLocation.y;
                points.push(ej.datavisualization.Diagram.Point(x, y));
            }
            points.push(targetPortLocation);
            return points;
        },
        _rightToRight: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (targetBounds.right >= sourceBounds.left) {
                if (sourcePortLocation.y >= targetBounds.top - tarExtension && sourcePortLocation.y <= targetBounds.bottom + tarExtension) {
                    x = sourcePortLocation.x + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourcePortLocation.y >= targetBounds.center.y) {
                        if (sourcePortLocation.x > targetBounds.left) {
                            if (sourcePortLocation.y > targetPortLocation.y) {
                                y = targetBounds.bottom + tarExtension;
                            }
                            else {
                                y = targetBounds.top - tarExtension;
                            }
                        }
                        else {
                            y = targetBounds.top - tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else if (targetBounds.right < sourceBounds.right) {
                    x = sourceBounds.right + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    //points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
                } else {
                    x = targetPortLocation.x + tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    //points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
                }
            }
            else {
                x = sourcePortLocation.x + srcExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (targetPortLocation.y >= sourceBounds.top - srcExtension && targetPortLocation.y <= sourceBounds.bottom + srcExtension) {
                    if (targetPortLocation.y <= sourceBounds.center.y) {
                        y = sourceBounds.top - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = sourceBounds.bottom + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
            points.push(targetPortLocation);
            return points;
        },
        _rightToTop: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, swap) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (swap) {
                tarExtension = connector._srcDecoratorSize;
                srcExtension = connector._tarDecoratorSize;
            }
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var min = srcExtension + tarExtension;
            if (targetPortLocation.x >= sourcePortLocation.x + srcExtension) {
                if ((sourcePortLocation.y + srcExtension) >= targetPortLocation.y && targetBounds.height !== 0) {
                    if (targetBounds.left <= sourceBounds.right) {
                        x = targetBounds.right + tarExtension;
                    }
                    else if ((targetBounds.left - sourceBounds.right) > min) {
                        x = sourceBounds.right + srcExtension;
                    }
                    else {
                        x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.top - tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            else {
                if (sourceBounds.bottom + srcExtension > targetBounds.top - tarExtension) {
                    if ((sourcePortLocation.x + srcExtension) <= targetBounds.right) {
                        x = targetBounds.right + tarExtension;
                    }
                    else {
                        x = sourceBounds.right + srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourceBounds.top >= targetBounds.top) {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                    }
                    else if (sourceBounds.top >= targetBounds.top - tarExtension)
                        y = targetBounds.top - tarExtension - srcExtension;
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    x = sourcePortLocation.x + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if ((targetBounds.top - sourceBounds.bottom) < min && targetBounds.height !== 0) {
                        if ((sourceBounds.left - targetBounds.right) >= min) {
                            y = sourceBounds.bottom + srcExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            x = sourceBounds.left - srcExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                            y = targetBounds.top - tarExtension;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                        }
                        else {
                            y = targetBounds.top - (targetBounds.top - sourceBounds.bottom) / 2;
                            points.push(ej.datavisualization.Diagram.Point(x, y));
                        }
                    }
                    else {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _rightToBottom: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, swap) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (swap) {
                tarExtension = connector._srcDecoratorSize;
                srcExtension = connector._tarDecoratorSize;
            }
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var min = srcExtension + tarExtension;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                if (sourcePortLocation.y <= targetBounds.bottom + tarExtension) {
                    if (targetBounds.left > (sourceBounds.right + srcExtension)) {
                        x = sourcePortLocation.x + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if ((sourceBounds.right + srcExtension) > targetPortLocation.x) {
                        x = sourcePortLocation.x + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((sourceBounds.top - targetBounds.bottom) > min) {
                            y = targetBounds.bottom + tarExtension;
                        }
                        else {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = sourcePortLocation.y;
                    }
                }
            }
            else {
                if (sourcePortLocation.y <= targetBounds.bottom) {
                    if ((sourceBounds.right + srcExtension) > targetBounds.right) {
                        x = sourceBounds.right + srcExtension;
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.bottom + tarExtension;
                    if (y <= (sourceBounds.bottom + srcExtension)) {
                        y = sourceBounds.bottom + srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    if ((targetBounds.bottom + tarExtension) < sourceBounds.top - srcExtension) {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((sourceBounds.top - targetBounds.bottom) < min) {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = sourceBounds.bottom + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _leftToLeft: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var min = srcExtension + tarExtension;
            if (targetPortLocation.x - tarExtension >= sourcePortLocation.x - srcExtension) {
                x = sourceBounds.left - srcExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (sourcePortLocation.y > targetPortLocation.y) {
                    if ((targetPortLocation.y + tarExtension) >= sourceBounds.top) {
                        y = sourceBounds.top - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((targetBounds.left - sourceBounds.right) > min) {
                            x = targetBounds.left - tarExtension;
                        }
                        else {
                            x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetPortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetPortLocation.x;
                    }
                }
                else {
                    if ((sourceBounds.bottom + srcExtension) < targetPortLocation.y) {
                        y = targetPortLocation.y;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetPortLocation.x;
                    }
                    else {
                        y = sourceBounds.bottom + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((targetBounds.left - sourceBounds.right) > min) {
                            x = targetBounds.left - tarExtension;
                        }
                        else {
                            x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                if (targetPortLocation.y > sourcePortLocation.y) {
                    if (sourcePortLocation.y > (targetBounds.top - tarExtension)) {
                        if ((sourceBounds.left - targetBounds.right) < min) {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                        else {
                            x = sourceBounds.left - srcExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.top - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if ((targetBounds.bottom + tarExtension) >= sourcePortLocation.y) {
                        if ((sourceBounds.left - targetBounds.right) < min) {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                        else {
                            x = sourceBounds.left - srcExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.left - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                x = targetBounds.left - tarExtension;
                points.push(ej.datavisualization.Diagram.Point(x, y));
            }
            points.push(ej.datavisualization.Diagram.Point(x, targetPortLocation.y));
            points.push(targetPortLocation);
            return points;
        },
        _leftToTop: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, swap) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (swap) {
                tarExtension = connector._srcDecoratorSize;
                srcExtension = connector._tarDecoratorSize;
            }
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            if (sourcePortLocation.x <= targetPortLocation.x) {
                if (sourcePortLocation.y > targetPortLocation.y && targetBounds.left <= sourceBounds.left) {
                    x = targetBounds.left - tarExtension;
                }
                else {
                    x = sourceBounds.left - srcExtension;
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (sourceBounds.bottom <= targetBounds.top) {
                    if ((sourceBounds.bottom + srcExtension) < targetBounds.top - tarExtension) {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                    }
                    else if (targetPortLocation.x < sourceBounds.right) {
                        y = sourceBounds.bottom + (targetBounds.top - sourceBounds.bottom) / 2;
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                }
                else {
                    if (sourceBounds.top - srcExtension >= targetBounds.top - tarExtension) {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
            }
            else {
                if (targetPortLocation.y <= (sourcePortLocation.y + srcExtension)) {
                    if (sourceBounds.left - srcExtension > targetBounds.right + tarExtension) {
                        x = sourceBounds.left - srcExtension;
                    }
                    else {
                        if (targetBounds.right + tarExtension >= sourceBounds.left - srcExtension) {
                            x = targetBounds.left - tarExtension;
                        }
                        else {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.top - tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _leftToBottom: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, swap) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            y = sourcePortLocation.y;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (swap) {
                tarExtension = connector._srcDecoratorSize;
                srcExtension = connector._tarDecoratorSize;
            }
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var min = srcExtension + tarExtension;
            if (sourcePortLocation.x - srcExtension <= targetPortLocation.x) {
                if (targetBounds.left <= sourcePortLocation.x && sourcePortLocation.y <= targetBounds.top) {
                    x = targetBounds.left - tarExtension;
                }
                else {
                    x = sourceBounds.left - srcExtension;
                }
                points.push(ej.datavisualization.Diagram.Point(x, y));
                if (targetBounds.bottom + tarExtension < sourceBounds.top - srcExtension) {
                    if ((sourceBounds.top - targetBounds.bottom) < min) {
                        y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                else {
                    if (targetBounds.bottom < sourceBounds.bottom) {
                        y = sourceBounds.bottom + srcExtension;
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            else {
                if (!(sourcePortLocation.y - srcExtension > targetPortLocation.y)) {
                    if (targetBounds.right >= sourceBounds.left) {
                        x = targetBounds.left - tarExtension;
                    }
                    else if ((sourceBounds.left - targetBounds.right) < min) {
                        x = sourceBounds.left - (sourceBounds.left - targetBounds.right) / 2;
                    }
                    else {
                        x = sourceBounds.left - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.bottom + tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _topToTop: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            x = sourcePortLocation.x;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var min = srcExtension + tarExtension;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                if (targetPortLocation.y - tarExtension >= sourcePortLocation.y - srcExtension) {
                    y = sourceBounds.top - srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetPortLocation.x <= sourceBounds.right) {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetBounds.height !== 0) {
                            if ((targetBounds.top - sourceBounds.bottom) > min) {
                                y = targetBounds.top - tarExtension;
                            }
                            else {
                                y = sourceBounds.bottom + (targetBounds.top - sourceBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (targetBounds.left <= sourcePortLocation.x) {
                        if ((sourceBounds.top - targetBounds.bottom) > min) {
                            y = sourceBounds.top - srcExtension;
                        }
                        else {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.left - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.top - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                if (targetPortLocation.y - tarExtension >= sourcePortLocation.y - srcExtension) {
                    y = sourceBounds.top - srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetPortLocation.x >= sourceBounds.left) {
                        x = sourceBounds.left - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (targetBounds.height !== 0) {
                            if ((targetBounds.top - sourceBounds.bottom) > min) {
                                y = targetBounds.top - tarExtension;
                            }
                            else {
                                y = sourceBounds.bottom + (targetBounds.top - sourceBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (targetBounds.right >= sourcePortLocation.x) {
                        if ((sourceBounds.top - targetBounds.bottom) > min) {
                            y = sourceBounds.top - srcExtension;
                        }
                        else {
                            y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.top - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        if (targetBounds.height !== 0) {
                            y = targetBounds.top - tarExtension;
                        }
                        else {
                            y = targetPortLocation.y;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            x = targetPortLocation.x;
            points.push(ej.datavisualization.Diagram.Point(x, y));
            points.push(targetPortLocation);
            return points;
        },
        _topToBottom: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation, swap) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            x = sourcePortLocation.x;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (swap) {
                tarExtension = connector._srcDecoratorSize;
                srcExtension = connector._tarDecoratorSize;
            }
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var minSpace = srcExtension + tarExtension;
            if (targetPortLocation.x > sourcePortLocation.x) {
                if (sourceBounds.top >= targetBounds.bottom) {
                    if ((sourceBounds.top - targetBounds.bottom) >= minSpace) {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((targetBounds.left - sourceBounds.right) >= minSpace) {
                            x = sourceBounds.right + srcExtension;
                        }
                        else {
                            x = sourceBounds.right + (targetBounds.left - sourceBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (targetBounds.top < sourceBounds.top) {
                        if (targetBounds.left >= (sourceBounds.right + srcExtension)) {
                            y = sourceBounds.top - srcExtension;
                        }
                        else {
                            y = targetBounds.top - tarExtension;
                        }
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetBounds.left >= (sourceBounds.right + srcExtension)) {
                        x = sourceBounds.right + srcExtension;
                    }
                    else {
                        x = targetBounds.right + tarExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    y = targetBounds.bottom + tarExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            }
            else if (targetPortLocation.x < sourcePortLocation.x) {
                if (sourceBounds.top >= targetBounds.bottom) {
                    if ((sourceBounds.top - targetBounds.bottom) > minSpace) {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if ((sourceBounds.left - targetBounds.right) >= minSpace) {
                            x = sourceBounds.left - srcExtension;
                        }
                        else {
                            x = targetBounds.right + (sourceBounds.left - targetBounds.right) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (sourceBounds.left - srcExtension >= (targetBounds.right + tarExtension)) {
                        y = sourceBounds.top - srcExtension;
                    }
                    else {
                        y = sourceBounds.top - srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourceBounds.left >= (targetBounds.right + tarExtension) || ((sourceBounds.right + srcExtension) < (targetBounds.right + tarExtension))) {
                        x = targetBounds.right + tarExtension;
                    }
                    else {
                        x = sourceBounds.right + srcExtension;
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (sourceBounds.left >= (targetBounds.right + tarExtension)) {
                        y = targetBounds.bottom + tarExtension;
                    }
                    else {
                        if (targetBounds.bottom <= sourceBounds.bottom) {
                            y = sourceBounds.bottom + srcExtension;
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                    }
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                }
                points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            }
            points.push(targetPortLocation);
            return points;
        },
        _bottomToBottom: function (connector, targetBounds, targetPortLocation, sourceBounds, sourcePortLocation) {
            var points = [];
            var x, y;
            points.push(sourcePortLocation);
            x = sourcePortLocation.x;
            var srcExtension = connector._srcDecoratorSize;
            var tarExtension = connector._tarDecoratorSize;
            if (sourceBounds.width == 0) srcExtension = 0;
            if (targetBounds.width == 0) tarExtension = 0;
            var minSpace = srcExtension + tarExtension;
            if (targetPortLocation.x >= sourcePortLocation.x) {
                if (sourceBounds.bottom + srcExtension >= targetBounds.bottom + srcExtension) {
                    y = sourceBounds.bottom + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if ((targetPortLocation.x) < sourceBounds.right) {
                        x = sourceBounds.right + srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (sourceBounds.top - srcExtension >= targetBounds.bottom + tarExtension) {
                            if ((sourceBounds.top - targetBounds.bottom) >= minSpace) {
                                y = targetBounds.bottom + tarExtension;
                            }
                            else {
                                y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (sourcePortLocation.x > targetBounds.left) {
                        if ((targetBounds.bottom - sourceBounds.bottom) >= minSpace) {
                            y = sourceBounds.bottom + srcExtension;
                        }
                        else {
                            y = sourceBounds.bottom + (targetBounds.bottom - sourceBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.left - tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            else {
                if (sourceBounds.bottom + srcExtension >= targetBounds.bottom + srcExtension) {
                    y = sourceBounds.bottom + srcExtension;
                    points.push(ej.datavisualization.Diagram.Point(x, y));
                    if (targetPortLocation.x > sourceBounds.left) {
                        x = sourceBounds.left - srcExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        if (sourceBounds.top - srcExtension >= targetBounds.bottom + tarExtension) {
                            if ((sourceBounds.top - targetBounds.bottom) >= minSpace) {
                                y = targetBounds.bottom + tarExtension;
                            }
                            else {
                                y = targetBounds.bottom + (sourceBounds.top - targetBounds.bottom) / 2;
                            }
                        }
                        else {
                            y = targetBounds.bottom + tarExtension;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
                else {
                    if (sourcePortLocation.x < targetBounds.right) {
                        if ((targetBounds.bottom - sourceBounds.bottom) >= minSpace) {
                            y = sourceBounds.bottom + srcExtension;
                        }
                        else {
                            y = sourceBounds.bottom + (targetBounds.bottom - sourceBounds.bottom) / 2;
                        }
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        x = targetBounds.right + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                    else {
                        y = targetBounds.bottom + tarExtension;
                        points.push(ej.datavisualization.Diagram.Point(x, y));
                    }
                }
            }
            points.push(ej.datavisualization.Diagram.Point(targetPortLocation.x, y));
            points.push(targetPortLocation);
            return points;
        },
        _addOrthogonalPoints: function (line, prev, next, sourcePt, endPt, runAngle) {
            var points = [];
            var strt = prev ? prev._endPoint : sourcePt;
            points.push(strt);
            if (!line._direction) line._direction = this._getOrthoDirection(strt, endPt, prev);
            var angle = (!runAngle && runAngle != 0) ? this._directionToAngle(line._direction || this._getBezierDirection(strt, endPt)) : runAngle;
            var direction = line._direction;
            var angles = { "right": 0, "left": 180, "top": 270, "bottom": 90, "auto": 90 };
            if (direction == "left" || direction == "right" || direction == "top" || direction == "bottom") {
                angle = angles[direction];
            }
            else {
                switch (direction) {
                    case "straight":
                        angle = 1 - 1;
                        break;
                    case "clockWise90":
                        angle += 91 - 1;
                        break;
                    case "opposite":
                        angle += 181 - 1;
                        break;
                    case "antiClockWise90":
                        angle += 271 - 1;
                        break;
                }
            }
            if ((line.length || line.length == 0) && line.direction) {
                var ctrlPoint1 = ej.datavisualization.Diagram.Geometry.transform(strt, angle || 0, line._length);
                ctrlPoint1.x = Math.round(ctrlPoint1.x * 100) / 100;
                ctrlPoint1.y = Math.round(ctrlPoint1.y * 100) / 100;
                points.push(ctrlPoint1);
                line._endPoint = ctrlPoint1;
                if (next) {
                    next._startPoint = line._endPoint;
                    if (next.points) {
                        next.points[0] = line._endPoint;
                        if (next.type == "bezier") {
                            this._updateBezierPoints(next);
                        }
                    }
                }
            }
            else {
                if (strt.x != endPt.x && strt.y != endPt.y || (prev && prev._startPoint.x == endPt.x)) {
                    var extra = 20;
                    var tempx = endPt.x - (ctrlPoint1 ? ctrlPoint1.x : strt.x);
                    var tempy = endPt.y - (ctrlPoint1 ? ctrlPoint1.y : strt.y);
                    tempx = Math.abs(tempx) <= 0.0001 ? 0 : tempx;
                    tempy = Math.abs(tempy) <= 0.0001 ? 0 : tempy;
                    var ptsCount;
                    var twoPt;
                    switch (angle) {
                        case 0: if (endPt.x < strt.x) break;
                            twoPt = Math.abs(strt.x - endPt.x) < 30;
                            break;
                        case 180: if (endPt.x > strt.x) break;
                            twoPt = Math.abs(strt.x - endPt.x) < 30;
                            break;
                        case 90: if (endPt.y < strt.y) break; twoPt = Math.abs(strt.y - endPt.y) < 30; break;
                        case 270: if (endPt.y > strt.y) break;
                            twoPt = Math.abs(strt.y - endPt.y) < 30;
                            break;
                    }
                    if (!prev) {
                        if (twoPt) {
                            extra = (angle == 90 || angle == 270) ? Math.abs(strt.y - endPt.y) : Math.abs(strt.x - endPt.x);
                            ptsCount = 2;
                            if (extra < 10) { extra += 20; ptsCount = 3 }
                        }
                        else ptsCount = 3;
                        if (ptsCount != 2 || (Math.abs(tempy) > 1 && Math.abs(tempx) > 1))
                            var ctrlPoint1 = ej.datavisualization.Diagram.Geometry.transform(strt, angle, extra);
                    }
                    else ptsCount = (prev && prev._startPoint.x == endPt.x) ? 3 : (tempx == 0 || tempy == 0) ? 2 : 3;
                    if (ptsCount == 3) {
                        var direction = line._direction == "auto" ? prev ? prev._direction == "left" || prev.direction == "right" ? "left" : "top" : "bottom" : line._direction;
                        if ((direction == "left" || direction == "right") || tempy == 0) {
                            tempy = tempy > 0 ? tempy : -tempy;
                            var ctrlPoint2 = ej.datavisualization.Diagram.Geometry.transform(ctrlPoint1 || strt, 90, endPt.y - (ctrlPoint1 ? ctrlPoint1.y : strt.y));
                        }
                        else {
                            tempx = tempx > 0 ? tempx : -tempx;
                            var ctrlPoint2 = ej.datavisualization.Diagram.Geometry.transform(ctrlPoint1 || strt, 0, endPt.x - (ctrlPoint1 ? ctrlPoint1.x : strt.x));
                        }
                    }
                    if (ctrlPoint1) {
                        ctrlPoint1.x = Math.round(ctrlPoint1.x * 100) / 100;
                        ctrlPoint1.y = Math.round(ctrlPoint1.y * 100) / 100;
                        points.push(ctrlPoint1);
                    }
                    if (ctrlPoint2) {
                        ctrlPoint2.x = Math.round(ctrlPoint2.x * 100) / 100;
                        ctrlPoint2.y = Math.round(ctrlPoint2.y * 100) / 100;
                        points.push(ctrlPoint2);
                    }
                }
                line._direction = this._getBezierDirection(points[points.length - 1], endPt);
                points.push(endPt);
                line._endPoint = endPt;
                if (next) {
                    next._startPoint = line._endPoint;
                    if (!next.points) next.points = [];
                    next.points[0] = line._endPoint;
                    if (next.type == "bezier") {
                        this._updateBezierPoints(next);
                    }
                }
            }
            line.points = points;
            return angle;
        },
        _getOrthoDirection: function (src, point, prev) {
            if (!prev) {
                if (src.y < point.y) {
                    return "bottom";
                }
                else if (src.y > point.y) {
                    return "top";
                }
                else if (src.x < point.x) {
                    return "right";
                }
                else if (src.x > point.x) {
                    return "left";
                }
            }
            else {
                var tar = point;
                if (Math.abs(tar.x - src.x) > Math.abs(tar.y - src.y) && (prev && (prev._direction == "left" || prev._direction == "right"))) {
                    return src.x < tar.x ? "right" : "left";
                }
                else {
                    return src.y < tar.y ? "bottom" : "top";
                }
            }
        },
        _directionToAngle: function (direction) {
            var angles = { "right": 0, "left": 180, "top": 270, "bottom": 90, "auto": 90 };
            return angles[direction];
        },
        _resetOrthogonalPoints: function (line) {
            var points = [];
            points.push(line.sourcePoint);
            points.push(line.targetPoint);
            line.points = [];
            line.sourcePoint = points[0];
            line.points.push(points[0]);
            var ctrlPoints = this._constructOrthogonalPoints(points[0], points[1]);
            line.points.push(ctrlPoints[0]);
            line.points.push(ctrlPoints[1]);
            line.targetPoint = points[1];
            line.points.push(points[1]);
        },
        _updateBezierPoints: function (line, targetNode, targetPort, targetBounds, sourceNode, sourcePort, sourceBounds) {
            var bounds;
            var angles = { "right": 0, "left": 180, "top": 270, "bottom": 90 };
            if (line.vector1 || line.vector2) {
                if (line.vector1) {
                    line._point1 = ej.datavisualization.Diagram.Geometry.transform(line._startPoint, line.vector1.angle, line.vector1.distance);
                }
                if (line.vector2) {
                    line._point2 = ej.datavisualization.Diagram.Geometry.transform(line._endPoint, line.vector2.angle, line.vector2.distance);
                }
            }
            if ((line.point1 == null && !line.vector1) || (line.point2 == null && !line.vector2)) {
                var target = line._endPoint;
                var source = line._startPoint;
                var dir;
                var matrix;
                if (!sourcePort)
                    dir = this._getBezierDirection(source, target);
                else {
                    bounds = sourceBounds;
                    source = this._getPortPosition(sourcePort, bounds);
                    if (sourceNode.rotateAngle !== 0) {
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, sourceNode.rotateAngle, sourceNode.offsetX, sourceNode.offsetY);
                        source = ej.Matrix.transform(matrix, source);
                    }
                    dir = this._getDirection(bounds, source, true);
                }
                if (!line.vector1)
                    line._point1 = line.point1 || this._getBezierAdjPoint(angles[dir], line._startPoint, line._endPoint);
                if (!targetPort) {
                    dir = this._getBezierDirection(target, source);
                }
                else {
                    bounds = targetBounds;
                    target = this._getPortPosition(targetPort, bounds);
                    if (targetNode.rotateAngle !== 0) {
                        matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, targetNode.rotateAngle, targetNode.offsetX, targetNode.offsetY);
                        target = ej.Matrix.transform(matrix, target);
                    }
                    dir = this._getDirection(bounds, target, true);
                }
                if (!line.vector2)
                    line._point2 = line.point2 || this._getBezierAdjPoint(angles[dir], line._endPoint, line._startPoint);
            }
        },
        _getBezierDirection: function (src, tar) {
            if (Math.abs(tar.x - src.x) > Math.abs(tar.y - src.y)) {
                return src.x < tar.x ? "right" : "left";
            }
            else {
                return src.y < tar.y ? "bottom" : "top";
            }
        },
        _getDirection: function (bounds, point, excludeBounds) {
            var points = bounds.points;
            var center = bounds.center;
            var part = excludeBounds ? 45 : (180 / (2 + 2 / (bounds.height / bounds.width)));
            var fourty5 = part;
            var one35 = (180 - part);
            var two25 = one35 + (2 * part);
            var three15 = 360 - part;
            var dir;
            var angle = ej.datavisualization.Diagram.Geometry.findAngle(point, center);
            if (angle > fourty5 && angle < one35) {
                dir = "top";
            }
            else if (angle > one35 && angle < two25) {
                dir = "right";
            }
            else if (angle > two25 && angle < three15) {
                dir = "bottom";
            }
            else {
                dir = "left";
            }
            return dir;
        },
        _getBezierAdjPoint: function (angle, srcEnd, tarEnd) {
            var distance = 60;
            var endAdj = ej.datavisualization.Diagram.Point(0, 0);
            var dir;
            if (angle > 45 && angle < 135) {
                dir = "Bottom";
            }
            else if (angle > 135 && angle < 225) {
                dir = "Left";
            }
            else if (angle > 225 && angle < 315) {
                dir = "Top";
            }
            else {
                dir = "Right";
            }
            switch (dir) {
                case "Right":
                    distance = Math.min(Math.abs(srcEnd.x - tarEnd.x) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x + distance, srcEnd.y);
                    break;
                case "Bottom":
                    distance = Math.min(Math.abs(srcEnd.y - tarEnd.y) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x, srcEnd.y + distance);
                    break;
                case "Left":
                    distance = Math.min(Math.abs(srcEnd.x - tarEnd.x) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x - distance, srcEnd.y);
                    break;
                case "Top":
                    distance = Math.min(Math.abs(srcEnd.y - tarEnd.y) * 0.45, distance);
                    endAdj = ej.datavisualization.Diagram.Point(srcEnd.x, srcEnd.y - distance);
                    break;
            }
            return endAdj;
        },
        _setLineEndPoint: function (connector, point, isTarget, dock) {
            var line = connector;
            point.x = Math.round(point.x * 100) / 100;
            point.y = Math.round(point.y * 100) / 100;
            if (isTarget) {
                var lastsegment = connector.segments[connector.segments.length - 1];
                line.targetPoint = point;
                if (lastsegment.type != "orthogonal") {
                    lastsegment.point = lastsegment._point = point;
                }
                lastsegment.points[lastsegment.points.length - 1] = point;
                lastsegment._endPoint = point;
            }
            else {
                line.sourcePoint = point;
                var first = connector.segments[0];
                first.points[0] = point;
                first._startPoint = point;
                if (first.type == "orthogonal" && !connector.sourcePort) {
                    if (first.length || first.length === 0) {
                        if (first.points.length == 2) {
                            if (first._direction == "left" || first._direction == "right") {
                                first._endPoint.y = first._startPoint.y;
                                var length = Math.abs(first._startPoint.x - first._endPoint.x);
                            }
                            else {
                                first._endPoint.x = first._startPoint.x;
                                var length = Math.abs(first._startPoint.y - first._endPoint.y);
                            }
                            connector.segments[0]._length = length;
                            connector.segments[0].length = connector.segments[0]._length;
                            connector.segments[0].direction = connector.segments[0]._direction;
                        }
                    }
                }
            }
            var segment = isTarget ? connector.segments[connector.segments.length - 1] : connector.segments[0];
            if (segment.type === "orthogonal" && ((!isTarget && !connector.sourcePort) || isTarget && !connector.targetPort) && (!(connector.sourceNode && connector.targetNode) || connector.segments.length > 1)) {
                segment.points = [];
                this._addOrthogonalPoints(segment, isTarget ? connector.segments[connector.segments.length - 2] : null,
                    isTarget ? null : connector.segments[1], connector.sourcePoint, connector.targetPoint);
            }
            else if (segment.type === "bezier") {
                this._updateBezierPoints(segment);
            }
        },
        _setBezierPoint: function (segment, point, whichpoint) {
            if (whichpoint === "bezierpoint1") {
                if (segment.vector1) {
                    segment.vector1 = {
                        distance: ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, point),
                        angle: ej.datavisualization.Diagram.Geometry.findAngle(segment._startPoint, point),
                    };
                    segment._point1 = point;
                }
                else
                    segment.point1 = segment._point1 = point;
            }
            else {
                if (segment.vector2) {
                    segment.vector2 = {
                        distance: ej.datavisualization.Diagram.Geometry.distance(segment._endPoint, point),
                        angle: ej.datavisualization.Diagram.Geometry.findAngle(segment._endPoint, point),
                    };
                    segment._point2 = point;
                }
                else
                    segment.point2 = segment._point2 = point;
            }
        },
        _setPoints: function (line, points) {
            line.sourcePoint = line.points[0] = points[0];
            line.targetPoint = line.points[line.points.length - 1] = points[points.length - 1];
            line.points = points;
            if (line.type === "bezier") {
                line.point1 = points[2] || line.point1;
                line.point2 = points[3] || line.point2;
                if (!line.point1)
                    line._point1 = line.point1;
                if (!line.point2)
                    line._point2 = line.point2;
                this._updateBezierPoints(line);
            }
        },
        _updateConnectorSegments: function (connector, ignoreSource, ignoreTarget, diagram) {
            var runAngle;
            var startPoint = connector.sourcePoint;
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                var prev = connector.segments[i - 1];
                var next = connector.segments[i + 1];
                segment._startPoint = prev ? prev._endPoint : startPoint;
                if (!(i == 0 && ignoreSource) && !(i == connector.segments.length - 1 && ignoreTarget)) {
                    if (segment.type == "orthogonal")
                        runAngle = ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, prev, next, connector.sourcePoint, connector.targetPoint, runAngle);
                    else {
                        runAngle = ej.datavisualization.Diagram.Geometry.findAngle(startPoint, segment._endPoint);
                        if (segment.type == "bezier") {
                            this._updateBezierPoints(segment);
                        }
                    }
                }
                startPoint = segment._endPoint;
            }
            if (diagram) {
                diagram._dock(connector, diagram.nameTable);
            }
        },
        _translateLine: function (connector, dx, dy) {
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                for (var j = 0; j < segment.points.length; j++) {
                    if (j == 0) {
                        segment._startPoint = segment.points[j] = ej.datavisualization.Diagram.Geometry.translate(segment.points[j], dx, dy);
                        if (i == 0) {
                            connector.sourcePoint = segment._startPoint;
                        }
                    }
                    else if (j == segment.points.length - 1) {
                        segment._endPoint = segment.points[j] = ej.datavisualization.Diagram.Geometry.translate(segment.points[j], dx, dy);
                        if (i == connector.segments.length - 1)
                            connector.targetPoint = segment._endPoint;
                    }
                    else segment.points[j] = ej.datavisualization.Diagram.Geometry.translate(segment.points[j], dx, dy);
                }
                if (segment.type === ej.datavisualization.Diagram.Segments.Bezier) {
                    segment._point1 = ej.datavisualization.Diagram.Geometry.translate(segment._point1, dx, dy);
                    segment._point2 = ej.datavisualization.Diagram.Geometry.translate(segment._point2, dx, dy);
                    if (segment.point1) { segment.point1 = segment._point1; }
                    if (segment.point2) { segment.point2 = segment._point2; }
                    segment.point = segment._endPoint;
                }
                else if (segment.type == "straight") {
                    segment.point = segment._point = segment._endPoint;
                }
            }
        },
        _adjustPoint: function (source, target, isStart, length) {
            var pt = isStart ? { x: source.x, y: source.y } : { x: target.x, y: target.y };
            if (source.x == target.x) {
                if (source.y < target.y && isStart || source.y > target.y && !isStart)
                    pt.y += length;
                else pt.y -= length;
            }
            else if (source.y == target.y) {
                if (source.x < target.x && isStart || source.x > target.x && !isStart)
                    pt.x += length;
                else pt.x -= length;
            }
            else {
                var angle;
                if (isStart) {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(source, target);
                    pt = ej.datavisualization.Diagram.Geometry.transform(source, angle, length);
                }
                else {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(target, source);
                    pt = ej.datavisualization.Diagram.Geometry.transform(target, angle, length);
                }
            }
            return pt;
        },
        _getLabelPosition: function (label, bounds, offset) {
            var x = bounds.width * (offset ? offset.x : label.offset.x);
            var y = bounds.height * (offset ? offset.y : label.offset.y);
            if (label.horizontalAlignment == "left")
                x += label.margin.left;
            else if (label.horizontalAlignment == "right")
                x -= label.margin.right;
            else
                x += label.margin.left - label.margin.right;
            if (label.verticalAlignment == "top")
                y += label.margin.top;
            else if (label.verticalAlignment == "bottom")
                y -= label.margin.bottom;
            else
                y += label.margin.top - label.margin.bottom;
            return ej.datavisualization.Diagram.Point(x, y);
        },
        _getPortPosition: function (port, bounds, bExcludeBoundsLocation) {
            if (bExcludeBoundsLocation) {
                return ej.datavisualization.Diagram.Point(bounds.width * port.offset.x, bounds.height * port.offset.y);
            }
            else {
                var x = bounds.center.x - bounds.width / 2 + (bounds.width * port.offset.x);
                var y = bounds.center.y - bounds.height / 2 + (bounds.height * port.offset.y);
                return ej.datavisualization.Diagram.Point(x, y);
            }
        },
        _translate: function (node, dx, dy, nameTable, isContainer, diagram) {
            if (ej.datavisualization.Diagram.Util.canMove(node)) {
                if (!node.segments) {
                    node.offsetX += dx;
                    node.offsetY += dy;
                    if (node._type === "group" && !isContainer) {
                        var nodes = diagram._getChildren(node.children);
                        var child;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            this._translate(child, dx, dy, nameTable, null, diagram);
                            if (child.parent && (child.parent != node.name && node.type != "pseudoGroup"))
                                ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], diagram);
                        }
                        if (node._type != "group" && node.container)
                            ej.datavisualization.Diagram.Util._updateGroupBounds(node, diagram);
                    }
                } else {
                    ej.datavisualization.Diagram.Util._translateLine(node, dx, dy, node);
                }
            }
        },
        scale: function (node, sw, sh, pivot, nameTable, skipScalOnChild, updateMinMax, isHelper, diagram) {
            if (!node.container) {
                var matrix = ej.Matrix.identity();
                if (!node.segments) {
                    if (node._type === "group") {
                        var nodes = diagram._getChildren(node.children);
                        var child;
                        for (var i = 0; i < nodes.length; i++) {
                            child = nameTable[nodes[i]];
                            if (child) {
                                this.scale(child, sw, sh, pivot, nameTable);
                                if (child.parent && (child.parent != node.name && node.type != "pseudoGroup"))
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(nameTable[child.parent], diagram);
                            }
                        }
                        ej.datavisualization.Diagram.Util._updateGroupBounds(node, diagram);
                    } else {
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        var newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        var width = node.width * sw;
                        var height = node.height * sh;
                        if (width > 1) {
                            node.width = width;
                            node.offsetX = newPosition.x;
                        }
                        if (height > 1) {
                            node.height = height;
                            node.offsetY = newPosition.y;
                        }
                    }
                } else {
                    ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                    for (var i = 0; i < node.segments.length; i++) {
                        var segment = node.segments[i];
                        for (var j = 0; j < segment.points.length; j++) {
                            segment.points[j] = ej.Matrix.transform(matrix, segment.points[j]);
                            if (i == 0 && j == 0 && !node.sourceNode) {
                                segment._startPoint = segment.points[0];
                                segment._endPoint = segment.points[segment.points.length - 1];
                                ej.datavisualization.Diagram.Util._setLineEndPoint(node, ej.Matrix.transform(matrix, node.sourcePoint), false);
                            }
                            if (i == node.segments.length - 1 && j == segment.points.length - 1 && !node.targetNode) {
                                segment._startPoint = segment.points[0];
                                segment._endPoint = segment.points[segment.points.length - 1];
                                node.targetPoint = segment._endPoint;
                            }
                        }
                        segment._startPoint = segment.points[0];
                        segment._endPoint = segment.points[segment.points.length - 1];
                        if (segment.type == "orthogonal") {
                            if (segment.length || segment.length === 0)
                                segment.length = segment._length = ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                            else
                                ej.datavisualization.Diagram.Util._addOrthogonalPoints(segment, node.segments[i - 1], node.segments[i + 1], node.sourcePoint, node.targetPoint);
                        } else {
                            segment.point = segment._endPoint;
                        }
                    }
                }
            }
            else {
                switch (node.container.type) {
                    case "canvas":
                        var newSize = {};
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        width = node.width * sw;
                        height = node.height * sh;
                        if (updateMinMax) {
                            node.minHeight = height;
                            node.minWidth = width;
                        }
                        if (isHelper != true)
                            newSize = diagram._getMinMaxSize(node, width, height);
                        else {
                            newSize.width = width;
                            newSize.height = height;
                        }
                        if (newSize && newSize.width > 1) {
                            var x = node.offsetX - node.width / 2;
                            node.width = newSize.width;
                            node.offsetX = x + node.width / 2;
                        }
                        if (newSize && newSize.height > 1) {
                            var y = node.offsetY - node.height / 2;
                            node.height = newSize.height;
                            node.offsetY = y + node.height / 2;
                        }
                        if (!skipScalOnChild)
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(node, this, true);
                        break;
                    case "stack":
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, -node.rotateAngle, pivot.x, pivot.y);
                        ej.Matrix.scale(matrix, sw, sh, pivot.x, pivot.y);
                        ej.Matrix.rotate(matrix, node.rotateAngle, pivot.x, pivot.y);
                        newPosition = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY));
                        width = (node.width * sw) - (node.marginLeft + node.marginRight);
                        height = (node.height * sh) - (node.marginTop + node.marginBottom);
                        newSize = diagram._getMinMaxSize(node, width, height);
                        if (newSize.width > 1) {
                            x = node.offsetX - node.width / 2;
                            node.width = newSize.width - node.marginLeft;
                            node.offsetX = x + node.width / 2 + (node.marginLeft);
                        }
                        if (newSize.height > 1) {
                            y = node.offsetY - node.height / 2;
                            node.height = newSize.height - node.marginTop;
                            node.offsetY = y + node.height / 2 + (node.marginTop);
                        }
                        ej.datavisualization.Diagram.SvgContext._alignOnStack(node, diagram);
                        break;
                }
            }
        },
        _getIntersectingElement: function (diagram, bounds) {
            var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(diagram._spatialSearch, bounds);
            var elements = [];
            for (var i = 0; i < quads.length; i++) {
                var quad = quads[i];
                if (quad.objects.length > 0) {
                    for (var j = 0; j < quad.objects.length; j++) {
                        var quadNode = quad.objects[j];
                        if (quadNode.visible) {
                            var nodebounds = ej.datavisualization.Diagram.Util.bounds(quadNode);
                            if (ej.datavisualization.Diagram.Geometry.intersectsRect(bounds, nodebounds)) {
                                quadNode = diagram._sortByZIndex(quadNode, true);
                                elements.push(quadNode);
                            }
                        }
                    }
                }
            }
            return elements;
        },
        _updateChildBounds: function (node, diagram) {
            var offX, offY, dx, dy;
            dx = node.width;
            dy = node.height;
            offX = node.offsetX;
            offY = node.offsetY;
            var childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, diagram);
            node.offsetX = childBounds.x + childBounds.width / 2;
            node.offsetY = childBounds.y + childBounds.height / 2;
            node.height = childBounds.height;
            node.width = childBounds.width;
            var deltaWidth = dx / node.width;
            var deltaHeight = dy / node.height;
            this.scale(node, deltaWidth, deltaHeight, node.pivot, diagram.nameTable, null, null, null, diagram);
            this._translate(node, offX - node.offsetX, offY - node.offsetY, diagram.nameTable, null, diagram);
        },
        _updateGroupBounds: function (node, diagram, needUpdate, angleChange) {
            var exWidth, exHeight, exOffX, exOffY;
            if (node && !node.container) {
                if (node && ((node.type === "pseudoGroup") || (node.children && node.children.length > 0))) {
                    exWidth = node.width;
                    exHeight = node.height;
                    exOffX = node.offsetX;
                    exOffY = node.offsetY;
                    if (angleChange)
                        this._updateRotateAngle(node, diagram.nameTable);
                    var bounds = this._getChildrenBounds(node, diagram, needUpdate);
                    var x = bounds.x + bounds.width * node.pivot.x;
                    var y = bounds.y + bounds.height * node.pivot.y;
                    var newposition = { x: x, y: y };
                    if (node.rotateAngle) {
                        var matrix = ej.Matrix.identity();
                        ej.Matrix.rotate(matrix, node.rotateAngle);
                        newposition = ej.Matrix.transform(matrix, newposition);
                    }
                    if (bounds) {
                        node.offsetX = newposition.x;
                        node.offsetY = newposition.y;
                        node.width = bounds.width;
                        node.height = bounds.height;
                    }
                    if (needUpdate) {
                        var child = null;
                        if (node.children && node.children.length > 0) {
                            for (var k = 0; k < node.children.length; k++) {
                                if (diagram._getChild(node.children[k]))
                                    child = diagram.nameTable[diagram._getChild(node.children[k])];
                                if (child) {
                                    if (child._type === "group" || (child.children && child.children.length > 0)) {
                                        this._updateGroupBounds(child, diagram, needUpdate);
                                    }
                                }
                            }
                        }
                        //update the initial size of the group
                        if (exWidth && exWidth != node.width)
                            diagram.scale(node, exWidth / node.width, 1, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), diagram.nameTable);
                        if (exHeight && exHeight != node.height)
                            diagram.scale(node, 1, exHeight / node.height, new ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY), diagram.nameTable);
                        //update the initial offset of the group
                        if (exOffX && exOffX != node.offsetX)
                            diagram._translate(node, exOffX - node.offsetX, 1, diagram.nameTable);
                        if (exOffX && exOffY != node.offsetY)
                            diagram._translate(node, 1, exOffY - node.offsetY, diagram.nameTable);
                    }
                }
            }
        },
        _getChildrenBounds: function (group, diagram, needUpdate) {
            var children = diagram._getChildren(group.children), rect,
                bounds = ej.datavisualization.Diagram.Rectangle(), child;
            if (children.length > 0) {
                child = diagram.nameTable[children[0]];
                if (child)
                    bounds = this._rotateChildBounds(child, group);
            }
            for (var i = 0, len = children.length; i < len; i++) {
                child = diagram.nameTable[children[i]];
                if (child) {
                    if (child._type === "group") {
                        this._updateGroupBounds(child, diagram, needUpdate);
                        rect = this._rotateChildBounds(child, group);
                    } else
                        rect = this._rotateChildBounds(child, group);
                }
                if (rect)
                    bounds = ej.datavisualization.Diagram.Geometry.union(bounds, rect);
            }
            return bounds;
        },
        _updateRotateAngle: function (node, nameTable) {
            if (node.offsetX == 0 && node.offsetY == 0 && node.width == 0 && node.height == 0) {
                for (var i = 0; i < node.children.length; i++) {
                    var child = nameTable[node.children[i]];
                    if (child && !child.segments && !node.rotateAngle) {
                        node.rotateAngle = child.rotateAngle;
                        return;
                    }
                }
            }
        },
        _rotateChildBounds: function (child, group) {
            if (child.segments || (group.rotateAngle || child.rotateAngle)) {
                var bounds = this.bounds(child);
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, -group.rotateAngle);
                var topleft = ej.Matrix.transform(matrix, bounds.topLeft);
                var topright = ej.Matrix.transform(matrix, bounds.topRight);
                var bottomLeft = ej.Matrix.transform(matrix, bounds.bottomLeft);
                var bottomRight = ej.Matrix.transform(matrix, bounds.bottomRight);
                var x = Math.min(topleft.x, topright.x, bottomLeft.x, bottomRight.x);
                var y = Math.min(topleft.y, topright.y, bottomLeft.y, bottomRight.y);
                return {
                    x: x,
                    width: Math.max(topleft.x, topright.x, bottomLeft.x, bottomRight.x) - x,
                    y: y,
                    height: Math.max(topleft.y, topright.y, bottomLeft.y, bottomRight.y) - y
                };
            }
            return {
                x: child.offsetX - (child.width || child._width) * child.pivot.x, y: child.offsetY - (child.height || child._height) * child.pivot.y,
                width: child.width || child._width, height: child.height || child._height
            };
        },
        updateBridging: function (connector, diagram) {
            if (connector.segments.length > 0) {
                for (var m = 0; m < connector.segments.length; m++) {
                    connector.segments[m]._bridges = [];
                }
            }
            if (ej.datavisualization.Diagram.Util.canBridge(connector, diagram)) {
                var points1 = this.getPoints(connector);
                var bridgeSpacing = connector.bridgeSpace;
                var lstBridge = [];
                var count = -1;
                var childBounds = ej.datavisualization.Diagram.Util.bounds(connector);
                var quads = diagram._getConnectors();
                if (quads.length > 0) {
                    for (var q = 0; q < quads.length; q++) {
                        var connector1 = quads[q];
                        if ((connector1.segments && connector1.visible) && connector.name !== connector1.name) {
                            var bounds = ej.datavisualization.Diagram.Util.bounds(connector);
                            var bounds1 = ej.datavisualization.Diagram.Util.bounds(connector1);
                            if (ej.datavisualization.Diagram.Geometry.intersectsRect(bounds, bounds1)) {
                                var points2 = this.getPoints(connector1);
                                var intersectPts = this.interSect(points1, points2, false, diagram, true);
                                /* Start For Loop */
                                if (intersectPts.length > 0) {
                                    for (var i = 0; i < intersectPts.length; i++) {
                                        var fullLength = 0;
                                        var segmentIndex = 0;
                                        var length = 0;
                                        var pointIndex = 0;
                                        var obj = this.getLengthAtFractionPoint(connector, intersectPts[i]);
                                        length = obj.lengthAtFractionPt, fullLength = obj.fullLength, segmentIndex = obj.segmentIndex, pointIndex = obj.pointIndex;
                                        if (segmentIndex < 0) {
                                            continue;
                                        }
                                        if (connector.segments[0]) {
                                            var startBridge, endBridge;
                                            var fractLength = (length - (bridgeSpacing / 2)) / fullLength;
                                            startBridge = this.getPointAtLength((length - (bridgeSpacing / 2)), points1);
                                            fractLength = (length + (bridgeSpacing / 2)) / fullLength;
                                            endBridge = this.getPointAtLength((length + (bridgeSpacing / 2)), points1);
                                            if (ej.datavisualization.Diagram.Geometry.isEmptyPoint(endBridge))
                                                endBridge = startBridge;
                                            var start, end;
                                            if (segmentIndex === 0) {
                                                start = connector.sourcePoint;
                                            }
                                            else {
                                                start = connector.segments[segmentIndex - 1]._endPoint;
                                            }
                                            if (connector.segments[segmentIndex].type === "straight")
                                                end = connector.segments[segmentIndex]._point;
                                            else
                                                end = connector.segments[segmentIndex].points[pointIndex];
                                            var angle = this._findAngle(start, end);
                                            if (lstBridge.length > segmentIndex && lstBridge[segmentIndex] != null) {
                                                var fixedPoint;
                                                if (segmentIndex === 0) {
                                                    fixedPoint = connector.sourcePoint;
                                                }
                                                else {
                                                    fixedPoint = connector.segments[segmentIndex - 1]._endPoint;
                                                }
                                                var fix = Math.abs(this.findLength(fixedPoint, endBridge));
                                                var var1 = 0;
                                                var insertAt = -1;
                                                count = -1;
                                                for (var k = 0; k < lstBridge[segmentIndex].bridges.length; k++) {
                                                    count++;
                                                    var arcSeg = lstBridge[segmentIndex].bridges[k];
                                                    var1 = Math.abs(this.findLength(fixedPoint, arcSeg.endPoint));
                                                    if (fix < var1) {
                                                        insertAt = count;
                                                        break;
                                                    }
                                                }
                                                if (insertAt >= 0) {
                                                    var paths;
                                                    paths = this.createSegment(startBridge, endBridge, angle, pointIndex, connector, diagram);
                                                    lstBridge[segmentIndex].bridges.splice(insertAt, 0, paths);
                                                    lstBridge[segmentIndex].bridges.join();
                                                    lstBridge[segmentIndex].bridgeStartPoint.splice(insertAt, 0, startBridge);
                                                    lstBridge[segmentIndex].bridgeStartPoint.join();
                                                    lstBridge[segmentIndex].segmentIndex = segmentIndex;
                                                }
                                                else {
                                                    var paths = this.createSegment(startBridge, endBridge, angle, pointIndex, connector, diagram);
                                                    lstBridge[segmentIndex].bridges.push(paths);
                                                    lstBridge[segmentIndex].bridgeStartPoint.push(startBridge);
                                                    lstBridge[segmentIndex].segmentIndex = segmentIndex;
                                                }
                                            }
                                            else {
                                                if (!isNaN(startBridge.x) && !isNaN(startBridge.y) && !ej.datavisualization.Diagram.Geometry.isEmptyPoint(endBridge)) {
                                                    var arcs = this.createSegment(startBridge, endBridge, angle, pointIndex, connector, diagram);
                                                    var stPoints = [];
                                                    var edPoints = [];
                                                    stPoints.push(startBridge);
                                                    edPoints.push(endBridge);
                                                    if (lstBridge.length < segmentIndex + 1) {
                                                    }
                                                    lstBridge[segmentIndex] = $.extend(true, {}, ej.datavisualization.Diagram.BridgeSegments, {});
                                                    lstBridge[segmentIndex].bridges.push(arcs);
                                                    lstBridge[segmentIndex].bridgeStartPoint = stPoints;
                                                    lstBridge[segmentIndex].segmentIndex = segmentIndex;
                                                }
                                            }
                                        }
                                    }
                                }
                                /* End For Loop */
                            }
                        }
                    }
                }
                if (lstBridge.length != 0) {
                    for (var i = 0; i < lstBridge.length; i++) {
                        var bridge = lstBridge[i];
                        if (!bridge)
                            continue;
                        for (var k = 1; k < bridge.bridges.length; k++) {
                            if (this.findLength(bridge.bridges[k].endPoint,
                                bridge.bridges[k - 1].endPoint) < bridgeSpacing) {
                                bridge.bridges[k - 1].endPoint = bridge.bridges[k].endPoint;
                                var subBridge = bridge.bridges[k - 1];
                                var arc = diagram.getBridgeSegment(subBridge.startPoint, subBridge.endPoint, subBridge.angle, bridgeSpacing, subBridge.sweep);
                                bridge.bridges[k - 1].path = arc;
                                bridge.bridges.splice(k, 1);
                                bridge.bridgeStartPoint.splice(k, 1);
                                k--;
                            }
                        }
                        var pre = connector.sourcePoint;
                        for (var j = 0; j < bridge.bridges.length; j++) {
                            var subBridge = bridge.bridges[j];
                            var end;
                            if (connector.segments[bridge.segmentIndex].type === "straight")
                                end = connector.segments[bridge.segmentIndex]._endPoint;
                            else if (connector.segments[bridge.segmentIndex].type === "orthogonal")
                                end = connector.segments[bridge.segmentIndex].points[subBridge.segmentPointIndex];
                            if (segmentIndex != 0)
                                pre = connector.segments[bridge.segmentIndex]._endPoint;
                            var rectPoints = [];
                            rectPoints.push(ej.datavisualization.Diagram.Point(end.x - bridgeSpacing, end.y - bridgeSpacing));
                            rectPoints.push(ej.datavisualization.Diagram.Point(end.x + bridgeSpacing, end.y + bridgeSpacing));
                            var rect = ej.datavisualization.Diagram.Geometry.rect(rectPoints);
                            rectPoints = [];
                            rectPoints.push(ej.datavisualization.Diagram.Point(pre.x - bridgeSpacing, pre.y - bridgeSpacing));
                            rectPoints.push(ej.datavisualization.Diagram.Point(pre.x + bridgeSpacing, pre.y + bridgeSpacing));
                            var rect1 = ej.datavisualization.Diagram.Geometry.rect(rectPoints);
                            if (!ej.datavisualization.Diagram.Geometry.containsPoint(rect, bridge.bridgeStartPoint[j]) &&
                                !ej.datavisualization.Diagram.Geometry.containsPoint(rect1, subBridge.endPoint)) {
                                connector.segments[bridge.segmentIndex]._bridges.push(subBridge);
                                pre = subBridge.endPoint;
                            }
                        }
                    }
                }
            }
        },
        createSegment: function (start, end, angle, segmentPointIndex, connector, diagram) {
            var pathSegment = $.extend(true, {}, ej.datavisualization.Diagram.PathSegment, {});
            var arc, sweep;
            sweep = this.sweepDirection(angle, connector, diagram);
            arc = diagram.getBridgeSegment(start, end, angle, connector.bridgeSpace, sweep);
            pathSegment.path = arc;
            pathSegment.startPoint = start;
            pathSegment.endPoint = end;
            pathSegment.angle = angle;
            pathSegment.segmentPointIndex = segmentPointIndex;
            pathSegment.sweep = sweep;
            return pathSegment;
        },
        sweepDirection: function (angle, connector, diagram) {
            var angle1 = Math.abs(angle);
            if (diagram.bridgeDirection() === "top" || diagram.bridgeDirection() === "bottom") {
                var sweep = 1;
                if (angle1 >= 0 && angle1 <= 90) {
                    sweep = 0;
                }
            }
            else if (diagram.bridgeDirection() === "right" || diagram.bridgeDirection() === "left") {
                var sweep = 1;
                if (angle < 0 && angle >= -180) {
                    sweep = 0;
                }
            }
            if (diagram.bridgeDirection() === "right" || diagram.bridgeDirection() === "bottom") {
                if (sweep === 0)
                    sweep = 1;
                else
                    sweep = 0;
            }
            return sweep;
        },
        getPointAtLength: function (length, pts) {
            var run = 0;
            var pre = null;
            var found = ej.datavisualization.Diagram.Point(0, 0);
            for (var i = 0; i < pts.length; i++) {
                var pt = pts[i];
                if (!pre) {
                    pre = pt;
                    continue;
                }
                else {
                    var l = this.findLength(pre, pt);
                    if (run + l > length) {
                        var r = length - run;
                        var deg = this.findAngle(pre, pt);
                        var x = r * Math.cos(deg * Math.PI / 180);
                        var y = r * Math.sin(deg * Math.PI / 180);
                        found = ej.datavisualization.Diagram.Point(pre.x + x, pre.y + y);
                        break;
                    }
                    else {
                        run += l;
                    }
                }
                pre = pt;
            }
            return found;
        },
        getLengthAtFractionPoint: function (connector, pointAt) {
            var confirm = 100, fullLength = 0, segmentIndex = -1, count = 0, lengthAtFractionPt = 0, pointIndex = -1;
            if (connector.segments === null)
                return 0;
            var segments = connector.segments;
            var pt1 = connector.sourcePoint;
            var previouspt2 = pt1;
            for (var i = 0; i < connector.segments.length; i++) {
                var points = [];
                var segment1 = connector.segments[i];
                for (var j = 0; j < segment1.points.length; j++) {
                    var point2 = segment1.points[j];
                    points.push(point2);
                }
                for (var j = 0; j < points.length; j++) {
                    var pt2 = points[j];
                    var suspect = this.getSlope(pt2, pt1, pointAt, connector);
                    if (suspect < confirm) {
                        confirm = suspect;
                        lengthAtFractionPt = fullLength + this.findLength(pointAt, previouspt2);
                        segmentIndex = count;
                        pointIndex = j;
                    }
                    fullLength += this.findLength(pt2, pt1);
                    pt1 = pt2;
                    previouspt2 = pt2;
                }
                count++;
            }
            return { "lengthAtFractionPt": lengthAtFractionPt, "fullLength": fullLength, "segmentIndex": segmentIndex, "pointIndex": pointIndex };
        },
        getSlope: function (startPt, endPt, point, connector) {
            var three = 3.0;
            var delX = Math.abs(startPt.x - endPt.x);
            var delY = Math.abs(startPt.y - endPt.y);
            var lhs = ((point.y - startPt.y) / (endPt.y - startPt.y));
            var rhs = ((point.x - startPt.x) / (endPt.x - startPt.x));
            if (!isFinite(lhs) || !isFinite(rhs) || isNaN(lhs) || isNaN(rhs)) {
                if (startPt.x === endPt.x) {
                    if (startPt.y === endPt.y) {
                        return 10000;
                    }
                    else if (((startPt.y > point.y) && (point.y > endPt.y)) || ((startPt.y < point.y) && (point.y < endPt.y))) {
                        return Math.abs(startPt.x - point.x);
                    }
                    else {
                        return 10000;
                    }
                }
                else if (startPt.y == endPt.y) {
                    if (((startPt.x > point.x) && (point.x > endPt.x)) || ((startPt.x < point.x) && (point.x < endPt.x))) {
                        return Math.abs(startPt.y - point.y);
                    }
                    else {
                        return 10000;
                    }
                }
                else {
                    return 10000;
                }
            }
            else if (connector.segments.length > 0) {
                if ((startPt.x >= point.x && point.x >= endPt.x) || (startPt.x <= point.x && point.x <= endPt.x) || delX < three) {
                    if ((startPt.y >= point.y && point.y >= endPt.y) || (startPt.y <= point.y && point.y <= endPt.y) || delY < three) {
                        return Math.abs(lhs - rhs);
                    }
                    else {
                        return 10000;
                    }
                }
                else {
                    return 10000;
                }
            }
            else {
                return 10000;
            }
        },
        getPoints: function (connector) {
            var points = [];
            // points.push(connector.sourcePoint);
            if (connector.segments.length > 0) {
                for (var i = 0; i < connector.segments.length; i++) {
                    var segment = connector.segments[i];
                    if (segment.type === "straight" || segment.type === "bezier") {
                        if (i === 0)
                            points.push(connector.sourcePoint);
                        points.push(segment._point);
                    }
                    else if (segment.type === "orthogonal") {
                        for (var j = 0; j < connector.segments[i].points.length; j++) {
                            points.push(connector.segments[i].points[j]);
                        }
                    }
                }
            }
            return points;
        },
        interSect: function (points1, points2, self, diagram, zOrder) {
            if (self && points2.length >= 2) {
                points2.splice(0, 1);
                points2.splice(0, 1);
            }
            var points = [];
            for (var i = 0; i < points1.length - 1; i++) {
                var pt = this.interSect1(points1[i], points1[i + 1], points2, diagram, zOrder);
                if (pt.length > 0) {
                    for (var k = 0; k < pt.length; k++) {
                        points.push(pt[k]);
                    }
                }
                if (self && points2.length >= 1) {
                    points2.splice(0, 1);
                }
            }
            return points;
        },
        checkforHorizontalLine: function (angle) {
            var temp = 0;
            var ang = Math.abs(angle);
            if (ang > 90) {
                temp = 180 - ang;
            }
            else {
                temp = ang;
            }
            return temp;
        },
        interSect1: function (startPoint, endPoint, points, diagram, zOrder) {
            var points1 = [];
            for (var i = 0; i < points.length - 1; i++) {
                var point = this.interSect2(startPoint, endPoint, points[i], points[i + 1]);
                if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(point)) {
                    var angle = this._findAngle(startPoint, endPoint);
                    var angle1 = this._findAngle(points[i], points[i + 1]);
                    angle = this.checkforHorizontalLine(angle);
                    angle1 = this.checkforHorizontalLine(angle1);
                    if (diagram.bridgeDirection() === "left" || diagram.bridgeDirection() === "right") {
                        if (angle > angle1) {
                            points1.push(point);
                        }
                    }
                    else if (diagram.bridgeDirection() === "top" || diagram.bridgeDirection() === "bottom") {
                        if (angle < angle1) {
                            points1.push(point);
                        }
                    }
                    if (angle === angle1 && zOrder) {
                        points1.push(point);
                    }
                }
            }
            return points1;
        },
        interSect2: function (start1, end1, start2, end2) {
            var lineUtil1 = ej.datavisualization.Diagram.LineUtil(start1.x, start1.y, end1.x, end1.y);
            var lineUtil2 = ej.datavisualization.Diagram.LineUtil(start2.x, start2.y, end2.x, end2.y);
            var line3 = this.interSect3(lineUtil1, lineUtil2);
            if (line3.enabled) {
                return line3.intersectPt;
            }
            else {
                return ej.datavisualization.Diagram.Point(0, 0);
            }
        },
        interSect3: function (lineUtil1, lineUtil2) {
            var point = ej.datavisualization.Diagram.Point(0, 0);
            var l1 = lineUtil1;
            var l2 = lineUtil2;
            var d = (l2.y2 - l2.y1) * (l1.x2 - l1.x1) - (l2.x2 - l2.x1) * (l1.y2 - l1.y1);
            var n_a = (l2.x2 - l2.x1) * (l1.y1 - l2.y1) - (l2.y2 - l2.y1) * (l1.x1 - l2.x1);
            var n_b = (l1.x2 - l1.x1) * (l1.y1 - l2.y1) - (l1.y2 - l1.y1) * (l1.x1 - l2.x1);
            if (d == 0)
                return { enabled: false, intersectPt: point };
            var ua = n_a / d;
            var ub = n_b / d;
            if (ua >= 0 && ua <= 1 && ub >= 0 && ub <= 1) {
                point.x = l1.x1 + (ua * (l1.x2 - l1.x1));
                point.y = l1.y1 + (ua * (l1.y2 - l1.y1));
                return { enabled: true, intersectPt: point };
            }
            return { enabled: false, intersectPt: point };
        },
        findLength: function (startPt, endPt) {
            try {
                var length = Math.sqrt(Math.pow((startPt.x - endPt.x), 2) + Math.pow((startPt.y - endPt.y), 2));
                return length;
            }
            catch (e) {
                throw e;
            }
        },
        _findAngle: function (startPt, endPt) {
            var xDiff = startPt.x - endPt.x; var yDiff = startPt.y - endPt.y;
            return Math.atan2(yDiff, xDiff) * (180 / Math.PI);
        },
        findAngle: function (startPt, endPt) {
            if (ej.datavisualization.Diagram.Geometry.isEqualPoint(startPt, endPt))
                return 0;
            var r = ej.datavisualization.Diagram.Point(endPt.x, startPt.y);
            var sr = this.findLength(startPt, r);
            var re = this.findLength(r, endPt);
            var es = this.findLength(endPt, startPt);
            var ang = Math.asin(re / es);
            ang = ang * 180 / Math.PI;
            if (startPt.x < endPt.x) {
                if (startPt.y < endPt.y) {
                }
                else {
                    ang = 360 - ang;
                }
            }
            else {
                if (startPt.y < endPt.y) {
                    ang = 180 - ang;
                }
                else {
                    ang = 180 + ang;
                }
            }
            return ang;
        },
        _mapPalettePhase: function (item) {
            var sPoint, ePoint;
            if (item.orientation && item.orientation == "horizontal") {
                sPoint = { x: 0, y: 0 };
                ePoint = { x: 40, y: 0 };
            }
            else {
                sPoint = { x: 0, y: 0 };
                ePoint = { x: 0, y: 40 };
            }
            var phase = ej.datavisualization.Diagram.Connector({
                name: item ? item.name : ej.datavisualization.Diagram.Util.randomId(),
                segments: [{ type: "straight" }], sourcePoint: { x: sPoint.x, y: sPoint.y }, targetPoint: { x: ePoint.x, y: ePoint.y },
                lineWidth: item.lineWidth ? item.lineWidth : 1,
                lineDashArray: item.lineDashArray ? item.lineDashArray : null,
                lineColor: item.lineColor ? item.lineColor : "#606060",
                targetDecorator: { shape: "none" },
                isPhase: true,
                orientation: item.orientation ? item.orientation : "horizontal",
                labels: item.label ? [item.label] : [],
                paletteItem: item.paletteItem ? item.paletteItem : null,
            });
            return phase;
        },
        _updateShapeProperties: function (options) {
            options._type = "node";
            if (!options.shape.type) options.shape.type = "rectangle";
            switch (options.shape.type) {
                case "rectangle":
                    options.type = "basic";
                    options._shape = options.shape.type;
                    if (options.shape.cornerRadius)
                        options.cornerRadius = options.shape.cornerRadius;
                    break;
                case "ellipse":
                    options.type = "basic";
                    options._shape = options.shape.type;
                    break;
                case "path":
                    options.type = "basic";
                    options._shape = options.shape.type;
                    options.pathData = options.shape.pathData;
                    break;
                case "polygon":
                    options.type = "basic";
                    options._shape = options.shape.type;
                    options.points = options.shape.points;
                    break;
                case "image":
                    options.type = "image";
                    options._shape = options.shape.type;
                    options.source = options.shape.src;
                    break;
                case "native":
                    options.type = "native";
                    options._shape = options.shape.type;
                    options.templateId = options.shape.templateId;
                    break;
                case "text":
                    options.type = "text";
                    options._shape = options.shape.type;
                    options.templateId = options.shape.textBlock;
                    break;
                case "html":
                    options.type = "html";
                    options._shape = options.shape.type;
                    options.templateId = options.shape.templateId;
                    break;
                default:
                    break;
            }
        },
        _refreshParentGroup: function (node, diagram) {
            var parentObj = diagram.nameTable[node.parent];
            if (parentObj) {
                this._updateGroupBounds(parentObj, diagram);
                var parent = parentObj.parent ? document.getElementById(parentObj.parent) : diagram._diagramLayer;
                ej.datavisualization.Diagram.DiagramContext.renderGroup(parentObj, diagram);
            }
        },
        intersectsWith: function (rect, point1, point2) {
            var point = point1;
            var success = ((((rect.x < point.x) && (point.x < (rect.x + rect.width)))
           && (rect.y < point.y)) && (point.y < (rect.y + rect.height)));
            return success;
        },
        _findOffsetOnConnector: function (connector, handle, offset, diagram) {
            var length = 0;
            var lengths = [];
            //find connector length
            for (var i = 0; i < connector.segments.length; i++) {
                lengths[i] = [];
                for (var j = 0; j < connector.segments[i].points.length - 1; j++) {
                    length += ej.datavisualization.Diagram.Geometry.distance(connector.segments[i].points[j], connector.segments[i].points[j + 1]);
                    lengths[i][j] = length;
                }
            }
            var targetNode = connector.sourceNode && handle.offset == 0 && diagram.nameTable[connector.sourceNode] ||
                connector.targetNode && handle.offset == 1 && diagram.nameTable[connector.targetNode];
            var bnds = targetNode ? this.bounds(targetNode) : null;
            var offset = handle.offset || 0;
            var prevLength, targetSegment, angle, pt;
            //offset length
            var offLength = length * offset;
            for (var i = 0; i < lengths.length; i++) {
                for (var j = 0; j < lengths[i].length; j++) {
                    //segment where offset lies 
                    if (lengths[i][j] >= offLength) {
                        targetSegment = connector.segments[i];
                        angle = ej.datavisualization.Diagram.Geometry.findAngle(targetSegment.points[j], connector.segments[i].points[j + 1]);
                        pt = ej.datavisualization.Diagram.Geometry.transform(targetSegment.points[j], angle, offLength - (prevLength || 0));
                        var endPt;
                        if (targetNode) {
                            if (this.intersectsWith(bnds, pt) && !ej.datavisualization.Diagram.Geometry.containsRect(bnds, this.bounds(connector))) {
                                var direction = handle.offset == 0 ? this._getBezierDirection(targetSegment.points[j], connector.segments[i].points[j + 1])
                                    : this._getBezierDirection(targetSegment.points[j + 1], connector.segments[i].points[j]);
                                endPt = handle.offset == 0 ? targetSegment.points[j + 1] : targetSegment.points[j];
                                var skip = false;
                                switch (direction) {
                                    case "top":
                                        if (bnds.y > endPt.y)
                                            pt.y = bnds.top;
                                        else {
                                            skip = true;
                                        }
                                        break;
                                    case "bottom":
                                        if (bnds.bottom < endPt.y)
                                            pt.y = bnds.bottom;
                                        else {
                                            skip = true;
                                        }
                                        break;
                                    case "right":
                                        if (bnds.right < endPt.x)
                                            pt.x = bnds.right;
                                        else {
                                            skip = true;
                                        }
                                        break;
                                    case "left":
                                        if (bnds.left > endPt.x)
                                            pt.x = bnds.left;
                                        else {
                                            skip = true;
                                        }
                                        break;
                                }
                                if (skip) {
                                    offLength = handle.offset == 1 ? (prevLength || 0) : lengths[i][j];
                                    prevLength = handle.offset == 0 ? lengths[i][j] : (lengths[i][j - 2] || (
                                        lengths[i - 1] && lengths[i - 1].length > 1 ? lengths[i - 1][lengths[i - 1].length - 1] : lengths[i - 2] && lengths[i - 2][lengths[i - 2].length - 1]) || 0);
                                    handle.offset == 0 ? i = i : !j ? i -= 2 : j -= 2;
                                    continue;
                                }
                            } return { segment: i, point: j, offset: { x: pt.x, y: pt.y } };
                        }
                        else return { segment: i, point: j, offset: { x: pt.x, y: pt.y } };
                    }
                    prevLength = lengths[i][j];
                }
            }
        },
        _alignLabelOnSegments: function (node, label, diagram, obj) {
            var angle = obj.angle % 360;
            obj.angle %= 360;
            var fourty5 = 45;
            var one35 = 135;
            var two25 = 225;
            var three15 = 315;
            var bnds;
            if (label.offset == 0 && node.sourceNode || label.offset == 1 && node.targetNode) {
                var source = label.offset == 0 && node.sourceNode ? diagram.nameTable[node.sourceNode] : diagram.nameTable[node.targetNode];
                if (source) bnds = this.bounds(source);
                if (bnds) {
                    var part = 180 / (2 + 2 / (bnds.height / bnds.width));
                    fourty5 = part;
                    one35 = (180 - part);
                    two25 = one35 + (2 * part);
                    three15 = 360 - part;
                }
            }
            var hAlign, vAlign;
            switch (label.alignment) {
                case "auto":
                    vAlign = hAlign = "center";
                    if (obj.direction) {
                        if (obj.angle >= fourty5 && obj.angle <= one35 || obj.angle >= two25 && obj.angle <= three15) { hAlign = obj.direction; }
                        else { vAlign = obj.direction; }
                    }
                    if (label.offset != 0.5) {
                        if (obj.angle >= fourty5 && obj.angle <= one35) { vAlign = "top"; if (!obj.direction) (obj.angle < 90) ? hAlign = "right" : hAlign = "left"; }
                        else if (obj.angle >= two25 && obj.angle <= three15) { vAlign = "bottom"; if (!obj.direction) (obj.angle > 270) ? hAlign = "right" : hAlign = "left"; }
                        else if (obj.angle > one35 && obj.angle < two25) {
                            hAlign = "right"; if (!obj.direction) (obj.angle > 180) ? vAlign = "top" : vAlign = "bottom";
                        }
                        else { hAlign = "left"; if (!obj.direction) (obj.angle > 0) ? vAlign = "bottom" : vAlign = "top"; }
                    }
                    break;
                case "left":
                    if (obj.angle >= fourty5 && obj.angle <= one35) { hAlign = "right"; vAlign = label.offset == 0.5 ? "center" : "top" }
                    else if (obj.angle >= two25 && obj.angle <= three15) { hAlign = "left"; vAlign = label.offset == 0.5 ? "center" : "bottom" }
                    else if (obj.angle > fourty5 && obj.angle < two25) { vAlign = "top"; hAlign = label.offset == 0.5 ? "center" : "right" }
                    else { vAlign = "bottom"; hAlign = (label.offset == 0.5) ? "center" : "left" }
                    break;
                case "right":
                    if (obj.angle >= fourty5 && obj.angle <= one35) { hAlign = "left"; vAlign = label.offset == 0.5 ? "center" : "top" }
                    else if (obj.angle >= two25 && obj.angle <= three15) { hAlign = "right"; vAlign = label.offset == 0.5 ? "center" : "bottom" }
                    else if (obj.angle > fourty5 && obj.angle < two25) { vAlign = "bottom"; hAlign = label.offset == 0.5 ? "center" : "right" }
                    else { vAlign = "top"; hAlign = label.offset == 0.5 ? "center" : "left" }
                    break;
                case "center":
                    hAlign = "center";
                    vAlign = "center";
                    break;
            }
            return { hAlign: hAlign, vAlign: vAlign };
        },
        getObjectType: function (element) {
            if (element) {
                if (element.segments) return "connector";
                else if ((element._type === "node" || element.type == "bpmn" || element.type == "basic" || element.type == "flow" || element.type == "arrow")) return "node";
                else if (element.type === "text" || element.type === "image" || element.type === "native" || element.type === "html") return "node";
                else if (element._type != "group" && element.type == "pseudoGroup") return "pseudoGroup";
                else if (element._type === "group" || element.isLane || element.isSwimlane) return "group";
                else if (element.isSwimlane) return "swimlane";
                else return null;
            }
            return null;
        },
    };
    ej.datavisualization.Diagram.BridgeSegments = {
        bridges: [],
        bridgeStartPoint: [],
        segmentIndex: -1
    };
    ej.datavisualization.Diagram.PathSegment = {
        path: "",
        startPoint: { "x": 0, "y": 0 },
        endPoint: { "x": 0, "y": 0 },
        angle: 0,
        segmentPointIndex: -1,
        sweep: 1
    };
    ej.datavisualization.Diagram.LineUtil = function (x1, y1, x2, y2) {
        return { "x1": Number(x1) || 0, "y1": Number(y1) || 0, "x2": Number(x2) || 0, "y2": Number(y2) || 0 };
    };
    //#endregion
    //#region Geometry
    ej.datavisualization.Diagram.Geometry = {
        checkPointOnCircle: function (point, center, radius) {
            var r2 = Math.pow(radius, 2);
            var x2 = Math.pow((point.x - center.x), 2);
            var y2 = Math.pow((point.y - center.y), 2);
            if ((x2 + y2) <= r2) {
                return true;
            }
            return false;
        },
        rect: function (points) {
            var rectangle = ej.datavisualization.Diagram.Rectangle();
            if (arguments.length === 1) {
                var left = Number.MAX_VALUE;
                var top = Number.MAX_VALUE;
                var right = -Number.MAX_VALUE;
                var bottom = -Number.MAX_VALUE;
                var len = points.length;
                var x, y;
                for (var i = 0; i < len; i++) {
                    x = points[i].x;
                    y = points[i].y;
                    if (x < left) {
                        left = x;
                    }
                    if (x > right) {
                        right = x;
                    }
                    if (y < top) {
                        top = y;
                    }
                    if (y > bottom) {
                        bottom = y;
                    }
                }
                rectangle = ej.datavisualization.Diagram.Rectangle(left, top, right - left, bottom - top);
            }
            else if (arguments.length === 2) {
                var ptStart = ej.datavisualization.Diagram.Point(arguments[0]);
                var ptEnd = ej.datavisualization.Diagram.Point(arguments[1]);
                var swap;
                if (ptStart.x > ptEnd.x) {
                    swap = ptStart.x;
                    ptStart.x = ptEnd.x;
                    ptEnd.x = swap;
                }
                if (ptStart.y > ptEnd.y) {
                    swap = ptStart.y;
                    ptStart.y = ptEnd.y;
                    ptEnd.y = swap;
                }
                rectangle = ej.datavisualization.Diagram.Rectangle(ptStart.x, ptStart.y, ptEnd.x - ptStart.x, ptEnd.y - ptStart.y);
            }
            return rectangle;
        },
        union: function (rect1, rect2) {
            var x = Math.min(rect1.x, rect2.x);
            var y = Math.min(rect1.y, rect2.y);
            var width = Math.max(rect1.x + rect1.width, rect2.x + rect2.width);
            var height = Math.max(rect1.y + rect1.height, rect2.y + rect2.height);
            return ej.datavisualization.Diagram.Rectangle(x, y, width - x, height - y);
        },
        distance: function (pt1, pt2) {
            return Math.sqrt(Math.pow(pt2.x - pt1.x, 2) + Math.pow(pt2.y - pt1.y, 2));
        },
        translate: function (pt1, dx, dy) {
            return ej.datavisualization.Diagram.Point(pt1.x + dx, pt1.y + dy);
        },
        intersectsRect: function (rect1, rect2) {
            return ((((rect2.x < (rect1.x + rect1.width)) && (rect1.x < (rect2.x + rect2.width)))
                && (rect2.y < (rect1.y + rect1.height))) && (rect1.y < (rect2.y + rect2.height)));
        },
        findAngle: function (point1, point2) {
            var angle = Math.atan2(point2.y - point1.y, point2.x - point1.x);
            angle = (180 * angle / Math.PI);
            angle %= 360;
            if (angle < 0) {
                angle += 360;
            }
            return angle;
        },
        intersectSegment: function (segment1, segment2) {
            var x1 = segment1.point1.x;
            var y1 = segment1.point1.y;
            var x2 = segment1.point2.x;
            var y2 = segment1.point2.y;
            var x3 = segment2.point1.x;
            var y3 = segment2.point1.y;
            var x4 = segment2.point2.x;
            var y4 = segment2.point2.y;
            var a1, a2, b1, b2, c1, c2, x, y;
            var r1, r2, r3, r4;
            var denom, offset, num;
            a1 = y2 - y1;
            b1 = x1 - x2;
            c1 = (x2 * y1) - (x1 * y2);
            r3 = ((a1 * x3) + (b1 * y3) + c1);
            r4 = ((a1 * x4) + (b1 * y4) + c1);
            if ((r3 !== 0) && (r4 !== 0) && this.sameSign(r3, r4)) {
                return null;
            }
            a2 = y4 - y3;
            b2 = x3 - x4;
            c2 = (x4 * y3) - (x3 * y4);
            r1 = (a2 * x1) + (b2 * y1) + c2;
            r2 = (a2 * x2) + (b2 * y2) + c2;
            if ((r1 !== 0) && (r2 !== 0) && (this.sameSign(r1, r2))) {
                return null;
            }
            denom = (a1 * b2) - (a2 * b1);

            if (denom === 0) {
                return null;
            }
            if (denom < 0) {
                offset = -denom / 2;
            }
            else {
                offset = denom / 2;
            }
            offset = 0;
            num = (b1 * c2) - (b2 * c1);
            if (num < 0) {
                x = (num - offset) / denom;
            }
            else {
                x = (num + offset) / denom;
            }
            num = (a2 * c1) - (a1 * c2);
            if (num < 0) {
                y = (num - offset) / denom;
            }
            else {
                y = (num + offset) / denom;
            }
            return ej.datavisualization.Diagram.Point(x, y);
        },
        updatePath: function (x, y, width, height, pathData, svg, bounds, appendBoundsTo) {
            var bBox, isInit;
            if (svg && svg.pathBounds) {
                isInit = true;
                bBox = bounds || svg.pathBounds(pathData);
            }
            else {
                if (appendBoundsTo && !appendBoundsTo._absolutePath) isInit = true; else isInit = false;
                bBox = bounds;
            }
            if (appendBoundsTo) appendBoundsTo._absoluteBounds = bBox;
            var newX = 0, newY = 0;
            var isResize = false;
            if (x.toString() !== bBox.x || y.toString() !== bBox.y) {
                newX = x - Number(bBox.x);
                newY = y - Number(bBox.y);
            }
            if (width !== bBox.width || height !== bBox.height) {
                newX = width / Number(bBox.width);
                newY = height / Number(bBox.height);
                isResize = true;
            }
            return ej.datavisualization.Diagram.Geometry.processPathData(pathData, newX, newY, isResize, bBox.x, bBox.y, x, y, isInit, svg);
        },
        processPathData: function (path, newX, newY, isResizing, oldX, oldY, offsetX, offsetY, isInit, svg) {
            var shape = document.createElementNS('http://www.w3.org/2000/svg', "path");
            shape.setAttribute("d", path);
            if (isInit) {
                svg = svg || new ej.datavisualization.Diagram.Svg({});
                path = svg.absolutePath(shape).getAttribute('d');
                shape.setAttribute("d", path);
            }
            var x0, y0, x1, y1, x2, y2, segs = shape.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;
                if (isResizing) {
                    if (x != undefined) {
                        x = ej.datavisualization.Diagram.Geometry.scalePathData(x, newX, oldX, offsetX);
                    }
                    if (y != undefined) {
                        y = ej.datavisualization.Diagram.Geometry.scalePathData(y, newY, oldY, offsetY);
                    }
                    if (x1 != undefined) {
                        x1 = ej.datavisualization.Diagram.Geometry.scalePathData(x1, newX, oldX, offsetX);
                    }
                    if (y1 != undefined) {
                        y1 = ej.datavisualization.Diagram.Geometry.scalePathData(y1, newY, oldY, offsetY);
                    }
                    if (x2 != undefined) {
                        x2 = ej.datavisualization.Diagram.Geometry.scalePathData(x2, newX, oldX, offsetX);
                    }
                    if (y2 != undefined) {
                        y2 = ej.datavisualization.Diagram.Geometry.scalePathData(y2, newY, oldY, offsetY);
                    }
                }
                else {
                    if (x != undefined) {
                        x = (x + newX);
                    }
                    if (y != undefined) {
                        y = (y + newY);
                    }
                    if (x1 != undefined) {
                        x1 = (x1 + newX);
                    }
                    if (y1 != undefined) {
                        y1 = (y1 + newY);
                    }
                    if (x2 != undefined) {
                        x2 = (x2 + newX);
                    }
                    if (y2 != undefined) {
                        y2 = (y2 + newY);
                    }
                }
                switch (char) {
                    case 'M':
                        segs.replaceItem(shape.createSVGPathSegMovetoAbs(x, y), i);
                        break;
                    case 'L':
                        segs.replaceItem(shape.createSVGPathSegLinetoAbs(x, y), i);
                        break;
                    case 'H':
                        segs.replaceItem(shape.createSVGPathSegLinetoHorizontalAbs(x), i);
                        break;
                    case 'V':
                        segs.replaceItem(shape.createSVGPathSegLinetoVerticalAbs(y), i);
                        break;
                    case 'C':
                        segs.replaceItem(shape.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i);
                        break;
                    case 'S':
                        segs.replaceItem(shape.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i);
                        break;
                    case 'Q':
                        segs.replaceItem(shape.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i);
                        break;
                    case 'T':
                        segs.replaceItem(shape.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i);
                        break;
                    case 'A':
                        var r1 = seg.r1, r2 = seg.r2;
                        if (isResizing) {
                            r1 = (r1 * newX);
                            r2 = (r2 * newY);
                        }
                        segs.replaceItem(shape.createSVGPathSegArcAbs(x, y, r1, r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i);
                        break;
                    case 'Z':
                        x = x0; y = y0;
                        break;
                }
                // Record the start of a subpath
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }
            var pathData = shape.getAttribute("d");
            return pathData;
        },
        scalePathData: function (val, scaleFactor, oldOffset, newOffset) {
            if (val !== oldOffset) {
                if (newOffset !== oldOffset) {
                    val = (((val * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)))
                        + (newOffset - Number(oldOffset)));
                }
                else
                    val = ((Number(val) * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)));
            }
            else {
                if (newOffset !== oldOffset) {
                    val = newOffset;
                }
            }
            return val;
        },
        updatePolygonPoints: function (node, points) {
            if (node.points) {
                if (!points)
                    points = node.points;
                ej.datavisualization.Diagram.Geometry.scalePoints(points, node);
                var bounds = ej.datavisualization.Diagram.Geometry.rect(node.points);
                for (var i = 0, len = node.points.length; i < len; ++i) {
                    var point = ej.datavisualization.Diagram.Geometry.translate(points[i], -bounds.x, -bounds.y);
                    point = ej.datavisualization.Diagram.Geometry.translate(point, 0, 0);
                    points[i] = point;
                }
            }
        },
        scalePoints: function (points, node) {
            var bounds = ej.datavisualization.Diagram.Geometry.rect(node.points);
            if (bounds.width > 0 && bounds.height > 0) {
                var scaleX = node.width / bounds.width;
                var scaleY = node.height / bounds.height;
                for (var i = 0, len = node.points.length; i < len; ++i) {
                    points[i].x = points[i].x * scaleX;
                    points[i].y = points[i].y * scaleY;
                }
            }
        },
        sameSign: function (a, b) {
            return ((a * b) >= 0);
        },
        containsPoint: function (rect, point) {
            return ((((rect.x <= point.x) && (point.x <= (rect.x + rect.width)))
            && (rect.y <= point.y)) && (point.y <= (rect.y + rect.height)));
        },
        containsRect: function (rect, value) {
            return ((((rect.x <= value.x) && ((value.x + value.width) <= (rect.x + rect.width)))
               && (rect.y <= value.y)) && ((value.y + value.height) <= (rect.y + rect.height)));
        },
        inflate: function (rect, x, y) {
            rect.x -= x;
            rect.y -= y;
            rect.width += 2 * x;
            rect.height += 2 * y;
        },
        isEmptyRect: function (rect) {
            return rect.x === 0 && rect.y === 0 && rect.width === 0 && rect.height === 0;
        },
        isEmptyPoint: function (point) {
            return point.x === 0 && point.y === 0;
        },
        isEqualPoint: function (point1, point2) {
            return point1.x === point2.x && point1.y === point2.y;
        },
        isEmptySize: function (size) {
            return size.width === 0 && size.height === 0;
        },
        length: function (segment) {
            return ej.datavisualization.Diagram.Geometry.distance(segment.point1, segment.point2);
        },
        midPoint: function (segment) {
            return ej.datavisualization.Diagram.Point((segment.point1.x + segment.point2.x) / 2, (segment.point1.y + segment.point2.y) / 2);
        },
        transform: function (point, angle, length) {
            var pt = {};
            pt.x = Math.round((point.x + length * Math.cos(angle * Math.PI / 180)) * 100) / 100;
            pt.y = Math.round((point.y + length * Math.sin(angle * Math.PI / 180)) * 100) / 100;
            return pt;
        }
    };
    ej.datavisualization.Diagram.LineSegment = function (point1, point2) {
        return { "point1": point1, "point2": point2 };
    };
    ej.datavisualization.Diagram.Point = function (x, y) {
        return { "x": Number(x) || 0, "y": Number(y) || 0 };
    };
    ej.datavisualization.Diagram.Size = function (width, height) {
        return { "width": Number(width) || 0, "height": Number(height) || 0 };
    };
    ej.datavisualization.Diagram.Rectangle = function (x, y, width, height) {
        return { "x": Number(x) || 0, "y": Number(y) || 0, "width": Number(width) || 0, "height": Number(height) || 0 };
    };
    //#endregion
    //#region Enums
    ej.datavisualization.Diagram.KeyModifiers = {
        None: 0,
        Control: 1 << 0,
        Meta: 1 << 0,
        Alt: 1 << 1,
        Shift: 1 << 2
    };
    ej.datavisualization.Diagram.Keys = {
        None: "",
        Number0: 0,
        Number1: 1,
        Number2: 2,
        Number3: 3,
        Number4: 4,
        Number5: 5,
        Number6: 6,
        Number7: 7,
        Number8: 8,
        Number9: 9,
        A: 65,
        B: 66,
        C: 67,
        D: 68,
        E: 69,
        F: 70,
        G: 71,
        H: 72,
        I: 73,
        J: 74,
        K: 75,
        L: 76,
        M: 77,
        N: 78,
        O: 79,
        P: 80,
        Q: 81,
        R: 82,
        S: 83,
        T: 84,
        U: 85,
        V: 86,
        W: 87,
        X: 88,
        Y: 89,
        Z: 90,
        Left: 37,
        Up: 38,
        Right: 39,
        Down: 40,
        Escape: 27,
        Delete: 46
    };
    ej.datavisualization.Diagram.DecoratorShapes = {
        None: "none",
        Arrow: "arrow",
        OpenArrow: "openarrow",
        Circle: "circle",
        Diamond: "diamond",
        Path: "path"
    };
    ej.datavisualization.Diagram.ActionType = {
        Unknown: "unknown",
        Nudge: "nudge"
    };
    ej.datavisualization.Diagram.dependentconnector = {
        dependent: 1,
        independent: 2
    }
    ej.datavisualization.Diagram.ContainerType = {
        Canvas: "canvas",
        Stack: "stack",
    };
    ej.datavisualization.Diagram.PortShapes = {
        X: "x",
        Circle: "circle",
        Square: "square",
        Path: "path"
    };
    ej.datavisualization.Diagram.PortVisibility = {
        Visible: 1 << 0,
        Hidden: 1 << 1,
        Hover: 1 << 2,
        Connect: 1 << 3,
        Default: 1 << 3
    };
    ej.datavisualization.Diagram.TextAlign = {
        Left: "left",
        Center: "center",
        Right: "right",
        Justify: "justify"
    };
    ej.datavisualization.Diagram.HorizontalAlignment = {
        Left: "left",
        Center: "center",
        Right: "right"
    };
    ej.datavisualization.Diagram.LabelEditMode = {
        Edit: "edit",
        View: "view"
    };
    ej.datavisualization.Diagram.RelativeMode = {
        Object: "object",
        Mouse: "mouse"
    };
    ej.datavisualization.Diagram.TextWrapping = {
        NoWrap: "nowrap",
        Wrap: "wrap",
        WrapWithOverflow: "wrapwithoverflow",
    };
    ej.datavisualization.Diagram.VerticalAlignment = {
        Top: "top",
        Center: "center",
        Bottom: "bottom"
    };
    ej.datavisualization.Diagram.TextDecorations = {
        Underline: "underline",
        Overline: "overline",
        LineThrough: "line-through",
        None: "none"
    };
    ej.datavisualization.Diagram.NodeConstraints = {
        None: 1 << 0,
        Select: 1 << 1,
        Delete: 1 << 2,
        Drag: 1 << 3,
        Rotate: 1 << 4,
        Connect: 1 << 5,
        ResizeNorthEast: 1 << 6,
        ResizeEast: 1 << 7,
        ResizeSouthEast: 1 << 8,
        ResizeSouth: 1 << 9,
        ResizeSouthWest: 1 << 10,
        ResizeWest: 1 << 11,
        ResizeNorthWest: 1 << 12,
        ResizeNorth: 1 << 13,
        Resize: 1 << 6 | 1 << 7 | 1 << 8 | 1 << 9 | 1 << 10 | 1 << 11 | 1 << 12 | 1 << 13,
        Shadow: 1 << 14,
        DragLabel: 1 << 15,
        AllowPan: 1 << 16,
        AspectRatio: 1 << 17,
        AllowDrop: 1 << 18,
        InheritTooltip: 1 << 19,
        MultiSelect: 1 << 20,
        Default: 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6 | 1 << 7 | 1 << 8 | 1 << 9 | 1 << 10 | 1 << 11 | 1 << 12 | 1 << 13 | 1 << 19
    };
    ej.datavisualization.Diagram.ConnectorConstraints = {
        None: 1 << 0,
        Select: 1 << 1,
        Delete: 1 << 2,
        Drag: 1 << 3,
        DragSourceEnd: 1 << 4,
        DragTargetEnd: 1 << 5,
        DragSegmentThumb: 1 << 6,
        Bridging: 1 << 7,
        DragLabel: 1 << 8,
        InheritBridging: 1 << 9,
        AllowDrop: 1 << 10,
        InheritTooltip: 1 << 11,
        Default: 1 << 1 | 1 << 2 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6 | 1 << 9 | 1 << 11
    };
    ej.datavisualization.Diagram.PortConstraints = {
        None: 1 << 0,
        Connect: 1 << 1
    };
    ej.datavisualization.Diagram.SelectorConstraints = {
        None: 1 << 0,
        Rotator: 1 << 1,
        Resizer: 1 << 2,
        UserHandles: 1 << 3,
        All: 1 << 1 | 1 << 2 | 1 << 3
    };
    ej.datavisualization.Diagram.UserHandlePositions = {
        TopLeft: "topleft",
        TopCenter: "topcenter",
        TopRight: "topright",
        MiddleLeft: "middleleft",
        MiddleRight: "middleright",
        BottomLeft: "bottomleft",
        BottomCenter: "bottomcenter",
        BottomRight: "bottomright"
    };
    ej.datavisualization.Diagram.SnapConstraints = {
        None: 0,
        SnapToHorizontalLines: 1,
        SnapToVerticalLines: 2,
        SnapToLines: 1 | 2,
        ShowHorizontalLines: 4,
        ShowVerticalLines: 8,
        ShowLines: 4 | 8,
        All: 1 | 2 | 4 | 8
    };
    ej.datavisualization.Diagram.PageOrientations = {
        Landscape: "landscape",
        Portrait: "portrait"
    };
    ej.datavisualization.Diagram.ScrollLimit = {
        Infinity: "infinity",
        Diagram: "diagram",
        Limited: "limited"
    };
    ej.datavisualization.Diagram.LayoutOrientations = {
        TopToBottom: "toptobottom",
        BottomToTop: "bottomtotop",
        LeftToRight: "lefttoright",
        RightToLeft: "righttoleft"
    };
    ej.datavisualization.Diagram.ObjectTypes = {
        Diagram: "diagram",
        Palette: "palette",
        Lane: "lane",
        Group: "group"
    };
    ej.datavisualization.Diagram.PaletteDefaults = {
        name: "",
        expanded: false,
        subControlId: null,
        items: []
    };
    ej.datavisualization.Diagram.Palette = function (options, palette) {
        var nodeDefault = (palette.model.defaultSettings && palette.model.defaultSettings.node) ? palette.model.defaultSettings.node : null;
        var connectorDefault = (palette.model.defaultSettings && palette.model.defaultSettings.connector) ? palette.model.defaultSettings.connector : null;
        if (options.items)
            for (var i = 0; i < options.items.length; i++) {
                if (options.items[i].shape && typeof options.items[i].shape == "object")
                    ej.datavisualization.Diagram.Util._updateShapeProperties(options.items[i]);
                else options.items[i] = ej.datavisualization.Diagram.NodeType(options.items[i], palette);
                if (options.items[i].isLane) {
                    options.items[i] = ej.datavisualization.Diagram.SwimLaneHelper._createPaletteLane(options.items[i], palette.nameTable);
                }
                else if (options.items[i].segments)
                    options.items[i] = ej.datavisualization.Diagram.Connector($.extend(true, {}, connectorDefault, options.items[i]));
                else if (options.items[i]._type === "group" || (options.items[i].children && options.items[i].children.length > 0)) {
                    options.items[i] = ej.datavisualization.Diagram.Group($.extend(true, {}, nodeDefault, options.items[i]));
                    palette._initGroupNode(options.items[i], palette);
                }
                else if (options.items[i].type == "phase") {
                    options.items[i] = ej.datavisualization.Diagram.Util._mapPalettePhase(options.items[i]);
                }
                else
                    options.items[i] = ej.datavisualization.Diagram.Node($.extend(true, {}, nodeDefault, options.items[i]));
            }
        return $.extend(true, {}, ej.datavisualization.Diagram.PaletteDefaults, options);
    };
    ej.datavisualization.Diagram.Shapes = {
        Image: "image",
        Text: "text",
        Html: "html",
        Native: "native",
        Basic: "basic",
        Flow: "flow",
        Arrow: "arrow",
        BPMN: "bpmn",
    };
    ej.datavisualization.Diagram.ShapeDefaults = {
        shape: "",
        source: "",
        pathData: "",
        textBlock: null,
        points: [],
        templateId: null,
    };
    ej.datavisualization.Diagram.Shape = function (options) {
        if (options) {
            if (options.type === "html") {
                if (typeof (options.html) === "string")
                    options.html = ej.datavisualization.Diagram.Util.htmlEncode(options.html);
            }
            if (options.type === "text")
                options.textBlock = ej.datavisualization.Diagram.TextBlock(options.textBlock ? options.textBlock : {});
            if (options.type == "image" || options.type == "native" || options.type === "text" || options.type === "html")
                options._shape = options.type;
            if (!options._type)
                options._type = options.type ? options.type : "node";
            if (!options._shape)
                options._shape = options.shape ? options.shape : "rectangle";
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.ShapeDefaults, options);
    };
    ej.datavisualization.Diagram.Segments = {
        Straight: "straight",
        Orthogonal: "orthogonal",
        Bezier: "bezier"
    };
    ej.datavisualization.Diagram.SegmentDefaults = {
        type: ej.datavisualization.Diagram.Segments.Straight,
        point: null,
        point1: null,
        point2: null,
        vector1: null,
        vector2: null,
        _point1: ej.datavisualization.Diagram.Point(),
        _point2: ej.datavisualization.Diagram.Point(),
        length: null, _length: null,
        _bridges: [],
        direction: null, _direction: null,
    };
    ej.datavisualization.Diagram.Segment = function (options) {
        var line = $.extend(true, {}, ej.datavisualization.Diagram.SegmentDefaults, options);
        var points = [];
        line["points"] = points;
        if (line.type == "orthogonal") {
            line._length = line.length;
            line._direction = line.direction;
        }
        else {
            if (line.type == "bezier") {
                line._point1 = line.point1;
                line._point2 = line.point2;
            }
            line._point = line.point;
        }
        return line;
    };
    //#endregion
    //#region Margin
    ej.datavisualization.Diagram.MarginDefaults = {
        top: 0,
        left: 0,
        right: 0,
        bottom: 0
    };
    ej.datavisualization.Diagram.Margin = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.MarginDefaults, options);
    };
    ej.datavisualization.Diagram.GestureDefaults = {
        key: ej.datavisualization.Diagram.Keys.None,
        keyModifiers: ej.datavisualization.Diagram.KeyModifiers.None
    };
    ej.datavisualization.Diagram.Gesture = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.GestureDefaults, options);
    };
    //#region Command
    ej.datavisualization.Diagram.CommandDefaults = {
        gesture: ej.datavisualization.Diagram.Gesture(),
        execute: null,
        canExecute: null,
        parameter: null
    };
    ej.datavisualization.Diagram.Command = function (options) {
        if (options.gesture) options.gesture = ej.datavisualization.Diagram.Gesture(options.gesture);
        if (!options.canExecute) options.canExecute = function (args) {
            return true;
        }
        if (!options.execute) options.execute = function (args) {
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.CommandDefaults, options);
    };
    //#endregion
    //#region Tooltip
    ej.datavisualization.Diagram.TooltipDefaults = {
        templateId: "",
        relativeMode: "object",
        alignment: {
            horizontal: "center",
            vertical: "bottom",
        },
        margin: { left: 5, right: 5, top: 5, bottom: 5 },
        offset: null,
        delay: 0
    };
    ej.datavisualization.Diagram.Tooltip = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.TooltipDefaults, options);
    };
    //#endregion
    //#region Path Util
    ej.datavisualization.Diagram.Path = function () {
        this._path = "";
    };
    ej.datavisualization.Diagram.Path.prototype = {
        moveTo: function (x, y) {
            this._path += "M" + x + "," + y; return this;
        },
        lineTo: function (x, y) {
            this._path += "L" + x + "," + y; return this;
        },
        cubicBezierTo: function (points, isSmooth) {
            if (isSmooth)
                this._path += "S";
            else
                this._path += "C";
            this._addPoints(points);
            return this;
        },
        quadraticBezierTo: function (points, isSmooth) {
            if (isSmooth)
                this._path += "T";
            else
                this._path += "Q";
            this._addPoints(points);
            return this;
        },
        ellipticalArcTo: function (rx, ry, xAngle, arcFlag, sweepFlag, x, y) {
            this._path += "A" + rx + "," + ry + " " + xAngle + " " + arcFlag + "," + sweepFlag + " " + x + "," + y;
            return this;
        },
        close: function () {
            this._path += "Z"; return this;
        },
        toString: function () {
            return this._path;
        },
        _addPoints: function (points) {
            var point;
            for (var i = 0; i < points.length; i++) {
                point = points[i];
                this._path += point.x + ", " + point.y;
                if (i !== (points.length - 1))
                    this._path += " ";
            }
        }
    };
    //#endregion
    ej.datavisualization.Diagram.DecoratorDefaults = {
        shape: ej.datavisualization.Diagram.DecoratorShapes.Arrow,
        width: 8,
        height: 8,
        borderColor: "black",
        borderWidth: 1,
        fillColor: "black",
        pathData: "",
        cssClass: ""
    };
    ej.datavisualization.Diagram.Decorator = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.DecoratorDefaults, options);
    };
    //#endregion
    ej.datavisualization.Diagram.TextBlockDefaults = {
        readOnly: false,
        bold: false,
        italic: false,
        text: "",
        textDecoration: ej.datavisualization.Diagram.TextDecorations.None,
        fontSize: 12,
        fontFamily: "Arial",
        fontColor: "black",
        offset: ej.datavisualization.Diagram.Point(0.5, 0.5),
        textAlign: ej.datavisualization.Diagram.TextAlign.Center,
        horizontalAlignment: ej.datavisualization.Diagram.HorizontalAlignment.Center,
        verticalAlignment: ej.datavisualization.Diagram.VerticalAlignment.Center,
        wrapping: ej.datavisualization.Diagram.TextWrapping.WrapWithOverflow,
        margin: ej.datavisualization.Diagram.Margin(),
        mode: ej.datavisualization.Diagram.LabelEditMode.Edit,
        width: 50,
        rotateAngle: 0
    };
    ej.datavisualization.Diagram.TextBlock = function (options) {
        if (options.wrapText !== undefined) {
            if (options.wrapText) options.wrapping = "wrapwithoverflow";
            else options.wrapping = "nowrap";
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.TextBlockDefaults, options);
    };
    ej.datavisualization.Diagram.LabelDefaults = $.extend(true, {}, ej.datavisualization.Diagram.TextBlockDefaults, {
        name: "",
        visible: true,
        borderColor: "transparent",
        borderWidth: 0,
        fillColor: "transparent",
        cssClass: ""
    });
    ej.datavisualization.Diagram.Label = function (options) {
        var label = $.extend(true, {}, ej.datavisualization.Diagram.LabelDefaults, options);
        if (!label.name)
            label.name = "label_" + ej.datavisualization.Diagram.Util.randomId();
        return label;
    };
    //#endregion
    //#region Port
    ej.datavisualization.Diagram.PortDefaults = {
        name: "",
        visibility: ej.datavisualization.Diagram.PortVisibility.Default,
        constraints: ej.datavisualization.Diagram.PortConstraints.Connect,
        size: 8,
        offset: ej.datavisualization.Diagram.Point(0, 0),
        borderColor: "#1a1a1a",
        borderWidth: 1,
        fillColor: "white",
        shape: ej.datavisualization.Diagram.PortShapes.Square,
        pathData: "",
        connectorPadding: 0,
        cssClass: ""
    };
    //#region Shadow
    ej.datavisualization.Diagram.ShadowDefaults = {
        distance: 5,
        angle: 45,
        opacity: 0.7
    };
    ej.datavisualization.Diagram.Shadow = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.ShadowDefaults, options);
    };
    ej.datavisualization.Diagram.Port = function (options) {
        var port = $.extend(true, {}, ej.datavisualization.Diagram.PortDefaults, options);
        if (!port.name)
            port.name = "port_" + ej.datavisualization.Diagram.Util.randomId();
        return port;
    };
    //#endregion
    //#region Gradient
    ej.datavisualization.Diagram.StopDefaults = {
        color: "",
        offset: 0,
        opacity: 1
    };
    ej.datavisualization.Diagram.Stop = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.StopDefaults, options);
    };
    ej.datavisualization.Diagram.LinearGradientDefaults = {
        type: "linear",
        x1: 0,
        x2: 0,
        y1: 0,
        y2: 0,
        stops: []
    };
    ej.datavisualization.Diagram.LinearGradient = function (options) {
        for (var i = 0; i < options.stops.length; i++) {
            options.stops[i] = ej.datavisualization.Diagram.Stop(options.stops[i]);
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.LinearGradientDefaults, options);
    };
    ej.datavisualization.Diagram.RadialGradientDefaults = {
        type: "radial",
        cx: 0,
        cy: 0,
        fx: 0,
        fy: 0,
        r: 50,
        stops: []
    };
    ej.datavisualization.Diagram.RadialGradient = function (options) {
        for (var i = 0; i < options.stops.length; i++) {
            options.stops[i] = ej.datavisualization.Diagram.Stop(options.stops[i]);
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.RadialGradientDefaults, options);
    };
    //#region Node
    ej.datavisualization.Diagram.NodeBaseDefaults = {
        name: "",
        width: 0,
        height: 0,
        offsetX: 0,
        offsetY: 0,
        visible: true,
        zOrder: -1,
        excludeFromLayout: false,
        constraints: ej.datavisualization.Diagram.NodeConstraints.Default,
        parent: "",
        labels: [],
        ports: [],
        inEdges: [],
        outEdges: [],
        rotateAngle: 0,
        pivot: ej.datavisualization.Diagram.Point(0.5, 0.5),
        addInfo: {},
        marginLeft: 0,
        marginTop: 0,
        marginRight: 0,
        marginBottom: 0,
        horizontalAlign: ej.datavisualization.Diagram.HorizontalAlignment.Left,
        verticalAlign: ej.datavisualization.Diagram.VerticalAlignment.Top,
        minWidth: 0,
        maxWidth: 0,
        minHeight: 0,
        maxHeight: 0,
        connectorPadding: 0,
        cornerRadius: 0,
        paletteItem: null,
    };
    ej.datavisualization.Diagram.NodeBase = function (options) {
        var i;
        if (options.labels)
            for (i = 0; i < options.labels.length; i++) {
                options.labels[i] = ej.datavisualization.Diagram.Label(options.labels[i]);
            }
        if (options.ports)
            for (i = 0; i < options.ports.length; i++) {
                options.ports[i] = ej.datavisualization.Diagram.Port(options.ports[i]);
            }
        options.paletteItem = ej.datavisualization.Diagram.PaletteItem(options.paletteItem ? options.paletteItem : {});
        return $.extend(true, {}, ej.datavisualization.Diagram.NodeBaseDefaults, options);
    };
    ej.datavisualization.Diagram.NodeDefaults = {
        fillColor: "white",
        borderColor: "black",
        borderWidth: 1,
        borderDashArray: "",
        opacity: 1,
        gradient: null,
        type: ej.datavisualization.Diagram.Shapes.Basic,
        isExpanded: true,
        shadow: ej.datavisualization.Diagram.Shadow(),
        cssClass: ""
    };
    ej.datavisualization.Diagram.Node = function (options) {
        options = options || {};
        if (options.gradient) {
            if (options.gradient.type === "linear") {
                options.gradient = ej.datavisualization.Diagram.LinearGradient(options.gradient);
            }
            else if (options.gradient.type === "radial") {
                options.gradient = ej.datavisualization.Diagram.RadialGradient(options.gradient);
            }
        }
        if (options.shadow) {
            options.shadow = ej.datavisualization.Diagram.Shadow(options.shadow);
        }
        if (options.tooltip) {
            options.tooltip = ej.datavisualization.Diagram.Tooltip(options.tooltip);
        }
        if (options && !options.name) {
            options.name = "node_" + ej.datavisualization.Diagram.Util.randomId();
        }
        options.minWidth = options.minWidth ? options.minWidth : 0;
        options.minHeight = options.minHeight ? options.minHeight : 0;
        options.maxWidth = options.maxWidth ? options.maxWidth : 0;
        options.maxHeight = options.maxHeight ? options.maxHeight : 0;
        if (!options.width && !options._width) options._width = options.minWidth ? options.minWidth : options.maxWidth;
        if (!options.height && !options._height) options._height = options.minHeight ? options.minHeight : options.maxHeight;
        return $.extend(true, {}, ej.datavisualization.Diagram.NodeDefaults, ej.datavisualization.Diagram.NodeBase(options), ej.datavisualization.Diagram.Shape(options));
    };
    //#endregion
    //#region Connector
    ej.datavisualization.Diagram.ConnectorDefaults = {
        name: "",
        visible: true,
        lineDashArray: "",
        targetDecorator:
            {
                shape: ej.datavisualization.Diagram.DecoratorShapes.Arrow,
                width: 8,
                height: 8,
                borderColor: "black",
                fillColor: "black",
                pathData: ""
            },
        sourceDecorator:
            {
                shape: ej.datavisualization.Diagram.DecoratorShapes.Arrow,
                width: 8,
                height: 8,
                borderColor: "black",
                fillColor: "black",
                pathData: ""
            },
        segments: [ej.datavisualization.Diagram.Segment({ type: "straight" })],
        sourcePoint: ej.datavisualization.Diagram.Point(),
        targetPoint: ej.datavisualization.Diagram.Point(),
        lineColor: "black",
        lineWidth: 1,
        constraints: ej.datavisualization.Diagram.ConnectorConstraints.Default,
        opacity: 1,
        parent: "",
        labels: [],
        zOrder: -1,
        lineHitPadding: 10,
        addInfo: {},
        targetNode: null,
        targetPort: null,
        sourceNode: null,
        sourcePort: null,
        marginLeft: 0,
        marginTop: 0,
        marginRight: 0,
        marginBottom: 0,
        horizontalAlign: ej.datavisualization.Diagram.HorizontalAlignment.Left,
        verticalAlign: ej.datavisualization.Diagram.VerticalAlignment.Top,
        cornerRadius: 0,
        bridgeSpace: 10,
        sourcePadding: 0,
        targetPadding: 0,
        type: "connector",
        cssClass: ""
    };
    ej.datavisualization.Diagram.Connector = function (options) {
        options = options || {};
        options.name = options.name ? options.name : ej.datavisualization.Diagram.Util.randomId();
        options.targetDecorator = ej.datavisualization.Diagram.Decorator(options.targetDecorator || {});
        options.sourceDecorator = ej.datavisualization.Diagram.Decorator(options.sourceDecorator || { shape: ej.datavisualization.Diagram.DecoratorShapes.None });
        options.segments = options.segments || [ej.datavisualization.Diagram.Segment({ type: "straight" })];
        options._srcDecoratorSize = Math.max(parseInt(options.sourceDecorator.height), parseInt(options.sourceDecorator.width)) + 5 * options.sourceDecorator.borderWidth;
        options._tarDecoratorSize = Math.max(parseInt(options.targetDecorator.height), parseInt(options.targetDecorator.width)) + 5 * options.targetDecorator.borderWidth;
        if (options.segments.length) {
            options.sourcePoint = options.sourcePoint || ej.datavisualization.Diagram.Point();
            options.targetPoint = options.targetPoint || ej.datavisualization.Diagram.Point();
            ej.datavisualization.Diagram.Util._initializeSegments(options, options.defaultType);
        }
        if (options.labels)
            for (var i = 0; i < options.labels.length; i++) {
                options.labels[i] = ej.datavisualization.Diagram.Label(options.labels[i]);
            }
        if (options.tooltip) {
            options.tooltip = ej.datavisualization.Diagram.Tooltip(options.tooltip);
        }
        options.paletteItem = ej.datavisualization.Diagram.PaletteItem(options.paletteItem ? options.paletteItem : {});
        return $.extend(true, {}, ej.datavisualization.Diagram.ConnectorDefaults, options);
    };
    //#endregion
    //#region Container
    ej.datavisualization.Diagram.ContainerDefaults = {
        type: ej.datavisualization.Diagram.ContainerType.Canvas,
        orientation: "vertical",
    };
    ej.datavisualization.Diagram.Container = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.ContainerDefaults, options);
    };
    // #endregion
    //#region Group
    ej.datavisualization.Diagram.GroupDefaults = {
        children: [],
        canUngroup: true,
        allowDrop: true,
        container: null,
        paddingLeft: 0,
        paddingTop: 0,
        paddingRight: 0,
        paddingBottom: 0,
        minWidth: 0,
        maxWidth: 0,
        minHeight: 0,
        maxHeight: 0,
        type: "group"
    };
    ej.datavisualization.Diagram.Group = function (options) {
        options = options ? options : {};
        options.minWidth = options.minWidth ? options.minWidth : 0;
        options.minHeight = options.minHeight ? options.minHeight : 0;
        options.maxWidth = options.maxWidth ? options.maxWidth : 0;
        options.maxHeight = options.maxHeight ? options.maxHeight : 0;
        options.container = options.container ? ej.datavisualization.Diagram.Container(options.container) : null;
        options.fillColor = options.fillColor ? options.fillColor : "transparent";
        options.borderColor = options.borderColor ? options.borderColor : "transparent";
        if (options && !options.name) {
            options.name = "group_" + ej.datavisualization.Diagram.Util.randomId();
        }
        if (options.tooltip) {
            options.tooltip = ej.datavisualization.Diagram.Tooltip(options.tooltip);
        }
        if (!options._type) options._type = options.type ? options.type : "group";
        return $.extend(true, {}, ej.datavisualization.Diagram.NodeDefaults, ej.datavisualization.Diagram.GroupDefaults, ej.datavisualization.Diagram.NodeBase(options));
    };
    //#endregion
    //#region ZoomCommand
    ej.datavisualization.Diagram.ZoomCommand = {
        ZoomIn: 1,
        ZoomOut: 2
    };
    ej.datavisualization.Diagram.Zoom = function (options) {
        return $.extend(true, {}, { zoomFactor: 0.2, focusPoint: null, zoomCommand: ej.datavisualization.Diagram.ZoomCommand.ZoomIn }, options);
    };
    //#endregion
    ej.datavisualization.Diagram.DragState = {
        Starting: "starting",
        Dragging: "dragging",
        Completed: "completed"
    };
    ej.datavisualization.Diagram.ResizeState = {
        Starting: "starting",
        Resizing: "resizing",
        Completed: "completed"
    };
    ej.datavisualization.Diagram.DiagramConstraints = {
        None: 1 << 0,
        PageEditable: 1 << 1,
        Bridging: 1 << 2,
        Zoomable: 1 << 3,
        PannableX: 1 << 4,
        PannableY: 1 << 5,
        Pannable: 1 << 4 | 1 << 5,
        Undoable: 1 << 6,
        Default: 1 << 1 | 1 << 3 | 1 << 4 | 1 << 5 | 1 << 6
    };
    ej.datavisualization.Diagram.Tool = {
        None: 1 << 0,
        SingleSelect: 1 << 1,
        MultipleSelect: 1 << 2,
        ZoomPan: 1 << 3,
        DrawOnce: 1 << 4,
        ContinuesDraw: 1 << 5
    };
    ej.datavisualization.Diagram.Region = {
        Content: "content",
        PageSettings: "pageSettings"
    };
    ej.datavisualization.Diagram.FileFormats = {
        JPG: "jpg",
        PNG: "png",
        BMP: "bmp",
        SVG: "svg"
    };
    ej.datavisualization.Diagram.ExportModes = {
        Download: "download",
        Data: "data"
    };
    ej.datavisualization.Diagram.FitMode = {
        Page: "page",
        Width: "width",
        Height: "height"
    };
    //#endregion
    //#region UserHandle
    ej.datavisualization.Diagram.UserHandleDefaults = {
        name: "",
        pathData: "",
        borderColor: "",
        backgroundColor: "#2382c3",
        position: ej.datavisualization.Diagram.UserHandlePositions.BottomCenter,
        pathColor: "white",
        tool: "",
        size: 20,
        visible: true,
        enableMultiSelection: false
    };
    ej.datavisualization.Diagram.UserHandle = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.UserHandleDefaults, options);
    };
    //#endregion
    //#region Swimlane
    ej.datavisualization.Diagram.SwimLaneDefaults = {
        header: {
            width: 30,
            text: "", fontColor: "black", fontSize: 11, fontFamily: "Arial", bold: false, italic: false, textDecoration: "none"
        },
        fillColor: "transparent",
        orientation: "vertical",
        type: "group",
        isSwimlane: true,
        offsetX: 100,
        offsetY: 100,
        minHeight: 100,
        maxHeight: 100,
        minWidth: 100,
        maxWidth: 100,
        lanes: [],
        phases: [],
        phaseSize: 20,
        constraints: ej.datavisualization.Diagram.NodeConstraints.Default | ej.datavisualization.Diagram.NodeConstraints.AllowDrop & ~(ej.datavisualization.Diagram.NodeConstraints.ResizeNorth |
                      ej.datavisualization.Diagram.NodeConstraints.ResizeWest | ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest |
                  ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast | ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest),
        addInfo: {},
        cssClass: ""
    };
    ej.datavisualization.Diagram.SwimLane = function (options) {
        options = options || {};
        if (options && !options.name) {
            options.name = "swimlane_" + ej.datavisualization.Diagram.Util.randomId();
        }
        if (options.tooltip) {
            options.tooltip = ej.datavisualization.Diagram.Tooltip(options.tooltip);
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.SwimLaneDefaults, options);
    };
    //#endregion  
    //#region Lane
    ej.datavisualization.Diagram.LaneDefaults = {
        header: {
            width: 30,
            text: "",
            fontColor: "black",
            fontSize: 11,
            fontFamily: "Arial",
            bold: false,
            italic: false,
            textDecoration: "none"
        },
        name: "",
        children: [],
        isLane: true,
        orientation: "horizontal",
        fillColor: "white",
        labels: [],
        addInfo: {},
        constraints: ej.datavisualization.Diagram.NodeConstraints.Default | ej.datavisualization.Diagram.NodeConstraints.AllowDrop & ~ej.datavisualization.Diagram.NodeConstraints.Connect,
        cssClass: ""
    };
    ej.datavisualization.Diagram.Lane = function (options) {
        options = options || {};
        if (options && !options.name) {
            options.name = "lane_" + ej.datavisualization.Diagram.Util.randomId();
        }
        if (options.tooltip) {
            options.tooltip = ej.datavisualization.Diagram.Tooltip(options.tooltip);
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.LaneDefaults, options);
    };
    //#endregion  
    //#region phase
    ej.datavisualization.Diagram.phaseDefaults = {
        name: "",
        type: "phase",
        offset: 0,
        lineWidth: 1,
        lineDashArray: "3,3",
        lineColor: "#606060",
        parent: "",
        orientation: "horizontal",
        label: null
    };
    ej.datavisualization.Diagram.phase = function (options) {
        if (options.label) {
            options.label = ej.datavisualization.Diagram.Label(options.label);
            options.label.text = options.label.text ? options.label.text : "Phase"
        }
        return $.extend(true, {}, ej.datavisualization.Diagram.phaseDefaults, options);
    };
    //#endregion 
    //#region paletteItem
    ej.datavisualization.Diagram.PaletteItemDefaults = {
        height: undefined,
        enableScale: true,
        previewHeight: undefined,
        previewWidth: undefined,
        margin: ej.datavisualization.Diagram.Margin({ left: 4, right: 4, top: 4, bottom: 4 })
    };
    ej.datavisualization.Diagram.PaletteItem = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.PaletteItemDefaults, options);
    };
    //#endregion 
    ej.datavisualization.Diagram.SwimLaneHelper = {
        //#region diagram
        _initSwimLane: function (obj, diagram, headAdded) {
            var canvas = null;
            var lanes = null;
            obj.orientation = obj.orientation ? obj.orientation : "vertical";
            var phases = this._initphases(obj, diagram);
            lanes = this._initLanes(obj, diagram, headAdded);
            var childern = [];
            var header = null;
            if (!obj.header) {
                obj.header = { text: "", height: 0, fillColor: "white" };
            }
            if (obj.orientation === "horizontal") {
                var height = obj.header.height;
            } else {
                var width = obj.header.width;
            }
            if (obj.header) {
                header = ej.datavisualization.Diagram.Node({
                    _type: "node",
                    _isHeader: true,
                    name: obj.name + "_header_swimlane",
                    labels: [
                        {
                            text: obj.header.text ? obj.header.text : "",
                            offset: { x: 0.5, y: .5 },
                            fontColor: obj.header.fontColor ? obj.header.fontColor : "black",
                            fontSize: obj.header.fontSize ? obj.header.fontSize : 11,
                            fontFamily: obj.header.fontFamily ? obj.header.fontFamily : "Arial",
                            bold: obj.header.bold ? obj.header.bold : false,
                            italic: obj.header.italic ? obj.header.italic : false,
                            textDecoration: obj.header.textDecoration ? obj.header.textDecoration : "none",
                        },
                    ],
                    height: height ? height : 50,
                    width: width ? width : 50,
                    fillColor: obj.header.fillColor ? obj.header.fillColor : "white",
                    rotateAngle: 0,
                    constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select ^ ej.datavisualization.Diagram.NodeConstraints.Connect,
                    horizontalAlign: "stretch",
                    parent: obj.name,
                });
            }
            if (header) {
                diagram.nodes().push(header);
                diagram._nodes = $.extend(true, [], diagram.nodes());
                diagram.nameTable[header.name] = header;
                childern.push(header.name);
            }
            //var phaseNode = this._initPhaseNode(obj, diagram, childern);
            var seprtrValue = null;
            if (obj.phaseSize == 0)
                var seprtrValue = 0.001;
            else
                var seprtrValue = obj.phaseSize ? obj.phaseSize : 20;
            var PhaseStack = this._initPhaseStack(obj, diagram, childern, seprtrValue);
            var hAlign = null, vAlign = null;
            if (obj.orientation == "vertical")
                vAlign = "stretch";
            else
                hAlign = "stretch";
            var orientation, marginLeft = 0, marginTop = 0, minHeight = 0;
            if (obj.orientation == "horizontal") {
                orientation = "vertical";
                marginTop = (obj.header ? obj.header.height : 50) + seprtrValue;
                minHeight = 0;
            }
            else {
                orientation = "horizontal";
                marginTop = (obj.header ? obj.header.height : 50);
                marginLeft = (obj.phaseSize ? obj.phaseSize : seprtrValue);
                minHeight = (obj.header ? obj.header.height : 50) + seprtrValue;
            }
            var stack = ej.datavisualization.Diagram.Group({
                name: obj.name + "_stack",
                children: lanes,
                parent: obj.name,
                container: { type: ej.datavisualization.Diagram.ContainerType.Stack, orientation: orientation },
                horizontalAlign: "stretch",
                verticalAlign: "stretch",
                marginTop: marginTop,
                width: 10,
                height: 10,
                marginLeft: marginLeft,
                minHeight: minHeight,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default & ~
                    (ej.datavisualization.Diagram.NodeConstraints.Select | ej.datavisualization.Diagram.NodeConstraints.Connect),
            });
            var cW = 0, cH = 0;
            if (obj.orientation == "horizontal") {
                cW = obj.width;
                cH = obj.height;
            }
            else if (obj.orientation == "vertical") {
                cW = obj.width;
                cH = obj.height - header.height;
            }
            diagram.nodes().push(stack);
            diagram._nodes = $.extend(true, [], diagram.nodes());
            diagram.nameTable[stack.name] = stack;
            childern.push(stack.name);
            canvas = ej.datavisualization.Diagram.Group({
                name: obj.name ? obj.name : "",
                type: "group",
                addInfo: obj.addInfo ? obj.addInfo : {},
                parent: "",
                offsetX: obj.offsetX ? obj.offsetX : 100,
                offsetY: obj.offsetY ? obj.offsetY : 100,
                children: childern,
                zOrder: (obj.zOrder || obj.zOrder === 0) ? obj.zOrder : -1,
                fillColor: "transparent",
                labels: [{ mode: ej.datavisualization.Diagram.LabelEditMode.View }],
                container: { type: ej.datavisualization.Diagram.ContainerType.Canvas, },
                minHeight: cH ? cH : 100,
                maxHeight: obj.maxHeight ? obj.maxHeight : Number.POSITIVE_INFINITY,
                minWidth: cW ? cW : 100,
                maxWidth: obj.maxWidth ? obj.maxWidth : Number.POSITIVE_INFINITY,
                constraints: obj.constraints ? obj.constraints | ej.datavisualization.Diagram.NodeConstraints.AllowDrop : (ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.AllowDrop ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorth ^
                        ej.datavisualization.Diagram.NodeConstraints.ResizeWest ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest ^
                    ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest),
                phaseSize: seprtrValue,
                cssClass: obj.cssClass
            });
            //diagram._setZorder(canvas);
            canvas.isSwimlane = true;
            //this.nodes().push(canvas);
            canvas.phases = phases;
            canvas.orientation = obj.orientation;
            if (canvas.orientation == "horizontal")
                canvas.constraints = canvas.constraints ^ ej.datavisualization.Diagram.NodeConstraints.ResizeEast ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthEast;
            else
                canvas.constraints = canvas.constraints ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouth ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthEast;
            return canvas;
        },
        _initPhaseStack: function (obj, diagram, children, seprtrValue) {
            var orientation = "horizontal", horizontalAlign = "left", verticalAlign = "top", isHoizon;
            if (orientation) {
                if (obj.orientation == "horizontal") {
                    orientation = "horizontal";
                    horizontalAlign = "stretch";
                    isHoizon = true;
                }
                else {
                    orientation = "vertical";
                    verticalAlign = "stretch";
                    isHoizon = false;
                }
            }
            var phaseStack = ej.datavisualization.Diagram.Group({
                name: obj.name + "phaseStack",
                container: { type: "stack", orientation: orientation },
                type: "group",
                parent: obj.name,
                offsetX: 0,
                offsetY: 0,
                minHeight: seprtrValue,
                maxHeight: seprtrValue,
                minWidth: seprtrValue,
                maxWidth: seprtrValue,
                marginTop: obj.header ? obj.header.height : 50,
                isPhaseStack: true,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select ^ ej.datavisualization.Diagram.NodeConstraints.Connect
                //horizontalAlign: horizontalAlign,
                //verticalAlign: verticalAlign,
            });
            var df = 0;
            var rotateAngle = 0;
            if (obj.phaseSize) {
                obj.phaseSize = obj.phaseSize < 1 ? 20 : obj.phaseSize;
            }
            var height, width;
            for (var i = 0; i < obj.phases.length; i++) {
                if (isHoizon) {
                    height = obj.phaseSize ? obj.phaseSize : 20;
                    width = obj.phases[i].offset - df;
                }
                else {
                    height = obj.phases[i].offset - df;
                    if (i == 0)
                        height = height - (obj.header ? obj.header.height : 50);//added swimlane header height
                    width = obj.phaseSize ? obj.phaseSize : 20;
                    if (obj.phases[i].label)
                        obj.phases[i].label.rotateAngle = 270;
                }
                var node = ej.datavisualization.Diagram.Node({
                    parent: obj.name + "phaseStack",
                    name: "phaseStack" + obj.phases[i].name,
                    height: height, width: width,
                    labels: [obj.phases[i].label],
                    constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select ^ ej.datavisualization.Diagram.NodeConstraints.Connect
                });
                node.labels[0].name = "phaseStack" + obj.phases[i].name + "label",
                node.labels[0].mode = ej.datavisualization.Diagram.LabelEditMode.View,
                node.isPhase = true;
                df = obj.phases[i].offset;
                diagram.nameTable[node.name] = node;
                phaseStack.children.push(node.name);
                diagram.nodes().push(node);
                diagram._nodes = $.extend(true, [], diagram.nodes());
            }
            diagram.nodes().push(phaseStack);
            diagram._nodes = $.extend(true, [], diagram.nodes());
            children.push(phaseStack);
            diagram.nameTable[phaseStack.name] = phaseStack;
        },
        _initLanes: function (obj, diagram, headAdded) {
            var lanes = [], lane = null;
            if (obj.lanes) {
                for (var i = 0; i < obj.lanes.length; i++) {
                    obj.lanes[i]._laneHeader = obj.lanes[i]._laneHeader ? obj.lanes[i]._laneHeader : obj.lanes[i].header
                    lane = this._initLane(obj.lanes[i], obj, diagram, headAdded);
                    lanes.push(lane.name);
                    diagram.nodes().push(lane);
                    diagram._nodes = $.extend(true, [], diagram.nodes());
                    diagram.nameTable[lane.name] = lane;
                }
            }
            return lanes;
        },
        _initLane: function (laneobj, obj, diagram, headAdded) {
            var hAlign = null, vAlign = null;
            if (obj.orientation == "horizontal")
                hAlign = "stretch";
            else vAlign = "stretch";
            var success = false;
            var lane = ej.datavisualization.Diagram.Group({
                name: laneobj.name,
                addInfo: laneobj.addInfo ? laneobj.addInfo : {},
                type: "group",
                isLane: true,
                parent: obj.name + "_stack",
                offsetX: 0,
                offsetY: 0,
                borderColor: laneobj.borderColor ? laneobj.borderColor : "black",
                orientation: obj.orientation ? obj.orientation : "horizontal",
                fillColor: laneobj.fillColor,
                container: { type: ej.datavisualization.Diagram.ContainerType.Canvas, },
                horizontalAlign: hAlign ? hAlign : "left",
                verticalAlign: vAlign ? vAlign : "top",
                minHeight: laneobj.height ? laneobj.height : 100,
                maxHeight: laneobj.maxHeight ? laneobj.maxHeight : Number.POSITIVE_INFINITY,
                minWidth: laneobj.width ? laneobj.width : 100,
                maxWidth: laneobj.maxWidth ? laneobj.maxWidth : Number.POSITIVE_INFINITY,
                paddingTop: 20,
                paddingRight: 20,
                paddingBottom: 20,
                paddingLeft: 20,
                constraints: laneobj.constraints ? laneobj.constraints | ej.datavisualization.Diagram.NodeConstraints.AllowDrop : (ej.datavisualization.Diagram.NodeConstraints.Default | ej.datavisualization.Diagram.NodeConstraints.AllowDrop & ~ej.datavisualization.Diagram.NodeConstraints.Connect),
                cssClass: laneobj.cssClass
            });
            if (laneobj.labels && laneobj.labels.length > 0) {
                for (var h = 0; h < laneobj.labels.length; h++) {
                    laneobj.labels[h] = ej.datavisualization.Diagram.Label(laneobj.labels[h]);
                }
                lane.labels = laneobj.labels;
            }
            if (!headAdded) {
                var laneHeader = laneobj._laneHeader ? laneobj._laneHeader : null;
                var header = this._iniLaneHeader(laneobj, lane.name, obj, diagram, laneHeader);
                if (header) {
                    diagram.nodes().push(header);
                    diagram._nodes = $.extend(true, [], diagram.nodes());
                    diagram.nameTable[header.name] = header;
                    lane.children.push(header.name);
                }
            }
            var laneChildren;
            laneChildren = this._initChildren(laneobj, lane.name, diagram);
            for (var i = 0; i < laneChildren.length; i++) {
                lane.children.push(laneChildren[i]);
            }

            return lane;
        },
        _iniLaneHeader: function (headObj, parentName, obj, diagram, laneHeader) {
            var node = null;
            if (headObj.header) {
                node = ej.datavisualization.Diagram.Node({
                    _isHeader: true,
                    name: parentName + "_Headerr_",
                    type: "node",
                    labels: [
                    {
                        text: headObj.header.text ? headObj.header.text : "", offset: { x: 0.5, y: .5 }, width: 100, rotateAngle: 270, marginLeft: -10,
                        fontColor: headObj.header.fontColor ? headObj.header.fontColor : "black",
                        fontSize: headObj.header.fontSize ? headObj.header.fontSize : 11,
                        fontFamily: headObj.header.fontFamily ? headObj.header.fontFamily : "Arial",
                        bold: headObj.header.bold ? headObj.header.bold : false,
                        italic: headObj.header.italic ? headObj.header.italic : false,
                        textDecoration: headObj.header.textDecoration ? headObj.header.textDecoration : "none",
                    }],
                    height: laneHeader.height ? laneHeader.height : 50,
                    width: laneHeader.width ? laneHeader.width : 50,
                    fillColor: headObj.header.fillColor ? headObj.header.fillColor : "white",
                    rotateAngle: 0,
                    constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select ^ ej.datavisualization.Diagram.NodeConstraints.Connect,
                    marginLeft: -20,
                    marginTop: -20,
                    marginRight: -20,
                    marginBottom: -20,
                    parent: parentName,
                });
                if (obj.orientation && obj.orientation == "horizontal") {
                    node.verticalAlign = "stretch";
                    node.height = 1;
                    node.width = laneHeader.width ? laneHeader.width : 50;
                }
                else if (obj.orientation && obj.orientation == "vertical") {
                    node.horizontalAlign = "stretch";
                    node.labels[0].rotateAngle = 0;
                    node.height = laneHeader.width ? laneHeader.width : 50;
                    node.width = 1;
                }
            }
            return node;
        },
        _initChildren: function (laneObj, parentName, diagram) {
            var children = [], child = null;
            if (laneObj.children) {
                for (var i = 0; i < laneObj.children.length; i++) {
                    if (typeof laneObj.children[i] == "object") {
                        if (laneObj.type && laneObj.type === "bpmn")
                            child = laneObj.children[i]
                        else
                            child = this._initChild(laneObj.children[i], parentName, diagram);
                    }
                    else
                        child = diagram.nameTable[laneObj.children[i]];
                    if (child) {
                        if (!child.segments) {
                            if (child.children)
                                var childNodes = this._initChildren(child, child.name, diagram);
                            if (childNodes) child.children = childNodes;
                            diagram.nodes().push(child);
                            diagram._nodes = $.extend(true, [], diagram.nodes());
                        }
                        else {
                            diagram.connectors().push(child);
                            diagram._connectors = $.extend(true, [], diagram.connectors());
                        }
                        children.push(child.name);
                        diagram.nameTable[child.name] = child;
                    }
                }
            }
            return children;
        },
        _initChild: function (child, parentName, diagram) {
            var node = null;
            if (child && child.children && child.children.length > 0) {
                node = diagram._getNewGroup(ej.datavisualization.Diagram.NodeType(child, diagram))
            }
            else
                node = diagram._getNewNode(ej.datavisualization.Diagram.NodeType(child, diagram));
            node.parent = parentName;
            return node;
        },
        _swapPhaseSize: function (obj) {
            if (obj.phases.length > 0) {
                var phases = obj.phases, i, j, temp;
                for (i = 0; i < phases.length; i++) {
                    for (j = 0; j < (phases.length - i - 1) ; j++) {
                        if (phases[j].offset > phases[j + 1].offset) {
                            temp = phases[j];
                            phases[j] = phases[j + 1];
                            phases[j + 1] = temp;
                        }
                    }
                }
            }
        },
        _setSwimlaneSize: function (obj) {
            if (obj.phases.length > 0) {
                var phase;
                for (var i in obj.phases) {
                    phase = obj.phases[i];
                    if (obj.orientation === "horizontal") {
                        if (phase.offset > obj.width)
                            obj.width = phase.offset;
                    }
                    else {
                        if (phase.offset > obj.height)
                            obj.height = phase.offset;
                    }
                }
            }
        },
        _setLastPhaseSize: function (obj) {
            if (obj.phases.length > 0) {
                var lastPhase = obj.phases[obj.phases.length - 1];
                if (obj.orientation === "horizontal")
                    lastPhase.offset = obj.width;
                else
                    lastPhase.offset = obj.height;
            }
        },
        _NeedDefaultPhase: function (obj, offset) {
            if (obj.phases.length > 0)
                return false;
            return true;
        },
        _checkForDefPhase: function (obj) {
            if (obj.phases.length > 0)
                return false;
            return true;
        },
        _initphases: function (obj, diagram) {
            var phases = [];
            var phase = null, angle = 0;
            if (!diagram.model.phases)
                diagram.model.phases = [];
            var offset = 0;

            if (obj.orientation === "vertical") {
                offset = obj.height;
                angle = 270;
            }
            else {
                offset = obj.width;
            }
            var defPhase = ej.datavisualization.Diagram.phase({ name: obj.name + "defPhase", offset: offset, label: obj._lastPhase ? obj._lastPhase.labels[0] : { text: "Phase", rotateAngle: angle } });
            if (this._NeedDefaultPhase(obj))
                obj.phases.push(defPhase);
            else {
                this._swapPhaseSize(obj);
                this._setSwimlaneSize(obj);
                this._setLastPhaseSize(obj);
            }
            if (obj.phases && obj.phases.length > 0) {
                for (var i = 0; i < obj.phases.length; i++) {
                    phase = ej.datavisualization.Diagram.phase(obj.phases[i]);
                    phase.parent = obj.name;
                    phase.orientation = obj.orientation;
                    diagram.model.phases.push(phase);
                    diagram.nameTable[phase.name] = phase;
                    phases.push(phase.name);
                }
            }
            return phases;
        },
        //#endregion
        //#region palette
        _createSwimlane: function (obj, diagram, data) {
            var children = [], child = null, headAdded = false;
            for (var i = 0; i < obj.children.length; i++) {
                child = diagram.nameTable[obj.children[i]];
                if (child && ((child.name).split("_Headerr").length) <= 1)
                    children.push(child);
            }
            var nameTable = diagram.nameTable;
            var height = 500, width = 500;
            var headNode = null;
            if (obj.children[0]) {
                var node = nameTable[obj.children[0]];
                headNode = { text: "Title", width: 50, fillColor: node.fillColor, height: 50 };

            } else headNode = { text: "Title", width: 50, fillColor: "white", height: 50 };

            if (!obj.orientation) {
                if (obj.container)
                    obj.orientation = obj.container.orientation;
            }
            if (obj.orientation == "horizontal" && diagram._selectedSymbol) {
                height = obj.height ? obj.height : 100;
                width = obj.width ? obj.width : 650;
            } else {
                height = obj.height ? obj.height : 650;
                width = obj.width ? obj.width : 100;
            }
            var nSwimLane = {
                name: obj.name,
                header: headNode,
                fillColor: "white",
                minWidth: width,
                minHeight: height,
                height: height + 1,
                width: width + 1,
                maxHeight: height + Number.POSITIVE_INFINITY,
                maxWidth: width + Number.POSITIVE_INFINITY,
                offsetX: obj.offsetX,
                offsetY: obj.offsetY,
                orientation: obj.orientation ? obj.orientation : "horizontal",
                lanes: [
                    {
                        name: obj.name + "_canvas",
                        addInfo: obj.addInfo ? obj.addInfo : {},
                        header: {
                            text: node.labels[0].text ? node.labels[0].text : "HEADER",
                            fontSize: node.labels[0].fontSize ? node.labels[0].fontSize : 11,
                            width: 50,
                            fillColor: node.fillColor,
                            bold: node.labels[0].bold ? true : false,
                            italic: node.labels[0].italic ? true : false,
                            fontFamily: node.labels[0].fontFamily ? node.labels[0].fontFamily : "Arial",
                            fontColor: node.labels[0].fontColor ? node.labels[0].fontColor : "black",
                        },
                        fillColor: obj.fillColor ? obj.fillColor : "white",
                        height: height,
                        width: width,
                        children: children ? children : [],
                        _laneHeader: obj._laneHeader ? obj._laneHeader : null
                    },
                ]
            };
            var item = this._initSwimLane(ej.datavisualization.Diagram.SwimLane(nSwimLane), diagram, headAdded);
            return item;
        },
        _createDiagramLane: function (obj, nameTable) {
            var height = 0, width = 0, hAlign, vAlign, headerWidth = 0;
            if (obj.orientation == "horizontal") {
                height = obj.height ? obj.height : 1;
                width = obj.width ? obj.width : 150;
                hAlign = "stretch";
                vAlign = "top";
                headerWidth = 50;
            }
            else if (obj.orientation == "vertical") {
                height = obj.height ? obj.height : 1;
                width = obj.width ? obj.width : 120;
                hAlign = "left";
                vAlign = "stretch";
                headerWidth = 50;
            }
            var content = nameTable[obj.children[1]];
            var canvas = ej.datavisualization.Diagram.Group({
                name: obj.name,
                type: "group",
                isLane: true,
                addInfo: obj.addInfo ? obj.addInfo : {},
                borderColor: "black",
                offsetX: obj.offsetX,
                offsetY: obj.offsetY,
                fillColor: content.fillColor,
                container: { type: ej.datavisualization.Diagram.ContainerType.Canvas, },
                horizontalAlign: hAlign ? hAlign : "left",
                verticalAlign: vAlign ? vAlign : "top",
                minHeight: height ? height : 100,
                maxHeight: Number.POSITIVE_INFINITY,
                minWidth: width ? width : 100,
                orientation: obj.orientation ? obj.orientation : "horizontal",
                maxWidth: Number.POSITIVE_INFINITY,
                paddingTop: 20,
                paddingRight: 20,
                paddingBottom: 20,
                paddingLeft: 20,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorth ^
                        ej.datavisualization.Diagram.NodeConstraints.ResizeWest ^ ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest ^
                    ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest ^
                ej.datavisualization.Diagram.NodeConstraints.Connect,
            });
            if (obj.orientation == "horizontal")
                canvas.constraints = canvas.constraints ^ ej.datavisualization.Diagram.NodeConstraints.ResizeEast ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthEast;
            else
                canvas.constraints = canvas.constraints ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouth ^ ej.datavisualization.Diagram.NodeConstraints.ResizeSouthEast;
            canvas._laneHeader = content._laneHeader ? content._laneHeader : null;
            var head = nameTable[obj.children[0]];
            var headObj = { header: { text: head.labels[0].text ? head.labels[0].text : "Function", fontSize: head.labels[0].fontSize ? head.labels[0].fontSize : 11, width: headerWidth, fillColor: head.fillColor } };
            var header = this._iniLaneHeader(headObj, canvas.name, { orientation: obj.orientation }, null, content._laneHeader);
            header.name = canvas.name + "_Headerr_";
            nameTable[header.name] = header;
            canvas.children.push(header.name);
            return canvas;
        },
        _createPaletteLane: function (obj, nameTable) {
            var height = 0, width = 0, offsetX = 0, offsetY = 0, rotateAngle = 0, mleft = 0, mTop = 0, align = "left";
            var offx = 0, offY = 0;
            if (obj.paletteItem && !obj.paletteItem.enableScale && !obj.paletteItem.width) {
                if (obj.orientation === "horizontal") {
                    offx += 2;
                }
            }
            if (obj.orientation == "horizontal") {
                height = obj.height ? obj.height : 50;
                width = 25;
                offsetX = 14 + offx;
                offsetY = height / 2 + 1;
                rotateAngle = 270;
                align = "left";
            }
            else if (obj.orientation == "vertical") {
                height = 25;
                width = obj.width ? obj.width : 50;
                offsetX = width / 2;
                offsetY = 15;
                align = "left";
            }
            var name = obj.header.text ? obj.header.text : "";
            var header = ej.datavisualization.Diagram.Node({
                name: obj.name + "_" + name,
                _type: "node",
                labels: [
                    { text: obj.header.text ? obj.header.text : "Function", fontSize: obj.header.fontSize ? obj.header.fontSize : 11, offset: { x: 0.5, y: .5 }, width: 100, rotateAngle: rotateAngle, }
                ],
                height: height,
                width: width,
                fillColor: obj.header.fillColor ? obj.header.fillColor : "white",
                //constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select,
                parent: obj.name,
                offsetX: offsetX,
                offsetY: offsetY,
                constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Connect,
            });
            nameTable[header.name] = header;
            if (obj.orientation == "horizontal") {
                height = obj.height;
                width = obj.width - 25;
                offsetX = 25 + width / 2 + offx;
                offsetY = height / 2 + 1;
            }
            else if (obj.orientation == "vertical") {
                height = obj.height - 25;
                width = obj.width;
                offsetX = width / 2;
                offsetY = 25 + height / 2;
            }
            var content = ej.datavisualization.Diagram.Node({
                name: obj.name + "content",
                _type: "node",
                height: height,
                width: width,
                fillColor: obj.fillColor ? obj.fillColor : "white",
                //constraints: ej.datavisualization.Diagram.NodeConstraints.Default ^ ej.datavisualization.Diagram.NodeConstraints.Select,
                parent: obj.name,
                offsetX: offsetX,
                offsetY: offsetY,
            });
            content._laneHeader = obj.header;
            nameTable[content.name] = content;
            var group = ej.datavisualization.Diagram.Group({
                name: obj.name + "_group",
                addInfo: obj.addInfo ? obj.addInfo : {},
                height: obj.height,
                width: obj.width,
                offsetX: obj.offsetX,
                offsetY: obj.offsetY,
                children: [header.name, content.name, ],
                paletteItem: obj.paletteItem ? obj.paletteItem : null,
            });
            group.isLane = true;
            group.orientation = obj.orientation;
            return group;
        },
        //#endregion
        //#region Map swimlane
        _mapObject: function (diagram, node, type, childTable) {
            var obj;
            switch (type) {
                case "lane":
                    obj = this._mapLane(diagram, node);
                    break;
                case "phase":
                    obj = this._mapPhase(diagram, node);
                    break;
                case "swimlane":
                    obj = this._mapSwimlane(diagram, node, childTable);
                    break;
            }
            return obj;
        },
        //#region for mapping the Lane
        _mapLane: function (diagram, lane, childTable) {
            var nLane = ej.datavisualization.Diagram.Lane();
            this._mapLaneHeader(diagram, lane, nLane, childTable);
            this._mapChildren(diagram, lane, nLane, childTable);
            nLane.name = lane.name;
            nLane.labels = lane.labels;
            nLane.fillColor = lane.fillColor;
            nLane.orientation = lane.orientation;
            nLane.addInfo = lane.addInfo ? lane.addInfo : {};
            if (lane.orientation === "horizontal") {
                nLane.height = lane.height;
                nLane.width = null;
            }
            else {
                nLane.height = null;
                nLane.width = lane.width;
            }
            return nLane;
        },
        _mapLaneHeader: function (diagram, orgobj, clObj, childTable) {
            if (orgobj.children[0])
                var head = diagram.nameTable[diagram._getChild(orgobj.children[0])];
            if (!head && childTable)
                var head = childTable[diagram._getChild(orgobj.children[0])];
            if (head) {
                if (head.labels && head.labels[0]) {
                    clObj.header.text = head.labels[0].text
                    clObj.header.fontColor = head.labels[0].fontColor
                    clObj.header.fontSize = head.labels[0].fontSize
                    clObj.header.fontFamily = head.labels[0].fontFamily
                    clObj.header.bold = head.labels[0].bold
                    clObj.header.italic = head.labels[0].italic
                    clObj.header.textDecoration = head.labels[0].textDecoration
                }
                clObj.header.fillColor = head.fillColor;
                if (orgobj.orientation === "vertical") {
                    clObj.header.height = head.height;
                    clObj.header.width = null;
                }
                else {
                    clObj.header.height = null;
                    clObj.header.width = head.width;
                }
            }
        },
        _mapChildren: function (diagram, orgobj, clObj, childTable) {
            var child;
            if (orgobj.children && orgobj.children.length >= 2) {
                for (var i = 1; i < orgobj.children.length; i++) {
                    if (diagram._getChild(orgobj.children[i]))
                        child = diagram.nameTable[diagram._getChild(orgobj.children[i])];
                    if (childTable && !child)
                        child = childTable[diagram._getChild(orgobj.children[i])];
                    if (child) {
                        clObj.children.push(child);
                    }
                }
            }
        },
        //#endregion 
        //#region for mapping the swimlane
        _mapSwimlane: function (diagram, swimlane, childTable) {
            var nSwimlane = ej.datavisualization.Diagram.SwimLane();
            this._mapSwimlaneHeader(diagram, swimlane, nSwimlane, childTable);
            this._mapLanes(diagram, swimlane, nSwimlane, childTable);
            this._mapPhases(diagram, swimlane, nSwimlane, childTable);
            nSwimlane.name = swimlane.name;
            nSwimlane.orientation = swimlane.orientation;
            nSwimlane.offsetX = swimlane.offsetX;
            nSwimlane.offsetY = swimlane.offsetY;
            nSwimlane.maxHeight = swimlane.maxHeight;
            nSwimlane.maxWidth = swimlane.maxWidth;
            nSwimlane.minHeight = swimlane.minHeight;
            if (nSwimlane.orientation === "horizontal")
                nSwimlane.minWidth = (swimlane.width && swimlane.minWidth && swimlane.width > swimlane.minWidth) ? swimlane.width : swimlane.minWidth;
            else
                nSwimlane.minWidth = swimlane.minWidth;
            nSwimlane.phaseSize = swimlane.phaseSize;
            nSwimlane.constraints = swimlane.constraints;
            nSwimlane.zOrder = swimlane.zOrder;
            nSwimlane.addInfo = swimlane.addInfo ? swimlane.addInfo : {};
            nSwimlane.height = this._mapSwimlaneSize(swimlane, nSwimlane).height;
            nSwimlane.width = this._mapSwimlaneSize(swimlane, nSwimlane).width;
            return nSwimlane;
        },
        _mapSwimlaneSize: function (orgObj, nSwimlane) {
            var height = 0, width = 0, lane;
            if (orgObj.orientation == "horizontal") {
                height += nSwimlane.header ? nSwimlane.header.height : 0;
                //add phase height 
                height += orgObj.phaseSize ? orgObj.phaseSize : 20;
                height = orgObj.height - height;
                width = orgObj.width;
                return { height: height, width: width };
            }
            else {
                // height += nSwimlane.header ? nSwimlane.header.height : 0;
                height = orgObj.height - height
                width = orgObj.width;
                width -= orgObj.phaseSize ? orgObj.phaseSize : 20;
                return { height: height, width: width };
            }
        },
        _mapSwimlaneHeader: function (diagram, orgobj, clObj, childTable) {
            if (orgobj.children[0])
                var head = diagram.nameTable[diagram._getChild(orgobj.children[0])];
            if (!head && childTable)
                head = childTable[diagram._getChild(orgobj.children[0])];
            if (head) {
                if (head.labels && head.labels[0]) {
                    clObj.header.text = head.labels[0].text
                    clObj.header.fontColor = head.labels[0].fontColor
                    clObj.header.fontSize = head.labels[0].fontSize
                    clObj.header.fontFamily = head.labels[0].fontFamily
                    clObj.header.bold = head.labels[0].bold
                    clObj.header.italic = head.labels[0].italic
                    clObj.header.textDecoration = head.labels[0].textDecoration
                }
                clObj.header.fillColor = head.fillColor;
                clObj.header.height = head.height;
                clObj.header.width = null;
            }
        },
        _mapLanes: function (diagram, orgobj, clObj, childTable) {
            var laneStack, lane, nLane;
            if (orgobj.children[2] && diagram._getChild(orgobj.children[2]))
                laneStack = diagram.nameTable[diagram._getChild(orgobj.children[2])];
            if (laneStack && laneStack.children && laneStack.children.length > 0) {
                for (var i = 0; i < laneStack.children.length; i++) {
                    if (diagram._getChild(laneStack.children[i]))
                        lane = diagram.nameTable[diagram._getChild(laneStack.children[i])];
                    if (lane) {
                        nLane = this._mapLane(diagram, lane, childTable);
                        if (nLane) clObj.lanes.push(nLane);
                    }
                }

            }
        },
        _mapPhases: function (diagram, orgobj, clObj) {
            if (orgobj.phases && orgobj.phases.length > 1) {
                var length = orgobj.phases.length;
                if (orgobj.phases[orgobj.phases.length - 1] && diagram.nameTable[orgobj.phases[orgobj.phases.length - 1]]) {
                    var lstPhse = diagram.nameTable[orgobj.phases[orgobj.phases.length - 1]];
                    if (!(lstPhse.name.split("defPhase").length > 1) && (lstPhse.offset != orgobj.width))
                        length++;
                }
                for (var i = 0; i < length; i++) {
                    var cphase = this._mapPhase(diagram, diagram.nameTable[orgobj.phases[i]]);
                    if (cphase) {
                        clObj.phases.push(cphase);
                    }
                }
                if (diagram.nameTable["phaseStack" + orgobj.phases[orgobj.phases.length - 1]])
                    clObj._lastPhase = this._mapPhase(diagram, diagram.nameTable["phaseStack" + orgobj.phases[orgobj.phases.length - 1]]);
            }
            else if (orgobj.phases && orgobj.phases.length == 1) {
                var cphase = this._mapPhase(diagram, diagram.nameTable[orgobj.phases[0]]);
                if (cphase) {
                    clObj.phases.push(cphase);
                }
            }
        },
        //#endregion 
        //#region for mapping the Phase
        _mapPhase: function (diagram, phase) {
            var cphase;
            if (phase)
                cphase = diagram.nameTable[phase.name];
            if (cphase)
                return cphase;
            else return null;
        },
        _getSwimLaneNode: function (node, selNode, isPhase, diagram) {
            if (node.isSwimlane) {
                if (isPhase) {
                    if (diagram.nameTable[diagram._getChild(node.children[2])].container.orientation == selNode.orientation)
                        return node;
                }
                else if (diagram.nameTable[diagram._getChild(node.children[2])].container.orientation != selNode.orientation)
                    return node;
            }
            if (node.parent) {
                var parent = diagram.nameTable[node.parent];
                if (parent) {
                    return this._getSwimLaneNode(parent, selNode, undefined, diagram);
                }
            }
            return null;
        }
        //#endregion 
        //#endregion 
    };
})(jQuery, Syncfusion);

;
/**
* @fileOverview Plugin to style the Html Button elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej) {
    "use strict";
    ej.datavisualization.Diagram.BasicShapes = {
        Rectangle: "rectangle",
        Ellipse: "ellipse",
        Path: "path",
        Polygon: "polygon",
        Triangle: "triangle",
        Plus: "plus",
        Star: "star",
        Pentagon: "pentagon",
        Heptagon: "heptagon",
        Octagon: "octagon",
        Trapezoid: "trapezoid",
        Decagon: "decagon",
        RightTriangle: "righttriangle",
        Cylinder: "cylinder"
    };
    ej.datavisualization.Diagram.FlowShapes = {
        Process: "process",
        Decision: "decision",
        Document: "document",
        PreDefinedProcess: "predefinedprocess",
        Terminator: "terminator",
        PaperTap: "papertap",
        DirectData: "directdata",
        SequentialData: "sequentialdata",
        Sort: "sort",
        MultiDocument: "multidocument",
        Collate: "collate",
        SummingJunction: "summingjunction",
        Or: "or",
        InternalStorage: "internalstorage",
        Extract: "extract",
        ManualOperation: "manualoperation",
        Merge: "merge",
        OffPageReference: "offpagereference",
        SequentialAccessStorage: "sequentialaccessstorage",
        Annotation1: "annotation1",
        Annotation2: "annotation2"
    };
    ej.datavisualization.Diagram.ArrowShapes = {
        None: "none",
        CircularArrow: "circulararrow",
        CurvedRightArrow: "curvedrightarrow",
        CurvedUpArrow: "curveduparrow",
        CurvedLeftArrow: "curvedleftarrow",
        CurvedDownArrow: "curveddownarrow",
        JumpingRightArrow: "jumpingrightarrow",
        JumpingLeftArrow: "jumpingleftarrow"
    };
    ej.datavisualization.Diagram.BPMNShapes = {
        Event: "event",
        Gateway: "gateway",
        SequentialFlow: "sequentialflow",
        AssociationFlow: "associationflow",
        MessageFlow: "messageflow",
        Message: "message",
        DataObject: "dataobject",
        DataSource: "datasource",
        Activity: "activity"
    };
    ej.datavisualization.Diagram.BPMNEvents = {
        Start: "start",
        Intermediate: "intermediate",
        End: "end",
        NonInterruptingStart: "noninterruptingstart",
        NonInterruptingIntermediate: "noninterruptingintermediate",
    };
    ej.datavisualization.Diagram.BPMNActivity = {
        None: "none",
        Task: "task",
        SubProcess: "subprocess",
    };
    ej.datavisualization.Diagram.BPMNGateways = {
        None: "none",
        Exclusive: "exclusive",
        Inclusive: "inclusive",
        Parallel: "parallel",
        Complex: "complex",
        EventBased: "eventbased",
    };
    ej.datavisualization.Diagram.BPMNTriggers = {
        None: "none",
        Message: "message",
        Timer: "timer",
        Escalation: "escalation",
        Link: "link",
        Error: "error",
        Compensation: "compensation",
        Signal: "signal",
        Multiple: "multiple",
        Parallel: "parallel",
    };
    ej.datavisualization.Diagram.BPMNLoops = {
        None: "none",
        Standard: "standard",
        ParallelMultiInstance: "parallelmultiinstance",
        SequenceMultiInstance: "sequencemultiinstance"
    };
    ej.datavisualization.Diagram.BPMNTasks = {
        None: "none",
        Service: "service",
        Receive: "receive",
        Send: "send",
        InstantiatingReceive: "instantiatingreceive",
        Manual: "manual",
        BusinessRule: "businessrule",
        User: "user",
        Script: "script",
        Parallel: "parallel",
    };
    ej.datavisualization.Diagram.BPMNBoundary = {
        Default: "default",
        Call: "call",
        Event: "event",
    };
    ej.datavisualization.Diagram.BPMNDirections = {
        None: "none",
        UniDirectional: "unidirectional",
        BiDirectional: "bidirectional",
    };
    ej.datavisualization.Diagram.BPMNTask = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.BPMNTaskDefaults, options);
    };
    ej.datavisualization.Diagram.BPMNSubProcess = function (options) {
        return $.extend(true, {}, ej.datavisualization.Diagram.BPMNSubProcessDefaults, options);
    };
    ej.datavisualization.Diagram.BPMNTaskDefaults = {
        loop: ej.datavisualization.Diagram.BPMNLoops.None,
        type: ej.datavisualization.Diagram.BPMNTasks.None,
        call: false,
        compensation: false
    };
    ej.datavisualization.Diagram.BPMNSubProcessDefaults = {
        loop: ej.datavisualization.Diagram.BPMNLoops.None,
        boundary: ej.datavisualization.Diagram.BPMNBoundary.Default,
        adhoc: false,
        compensation: false,
    };
    ej.datavisualization.Diagram.BPMNEventDefaults = {
        event: ej.datavisualization.Diagram.BPMNEvents.Start,
        trigger: ej.datavisualization.Diagram.BPMNTriggers.None,
    };
    ej.datavisualization.Diagram.BPMNActivityDefaults = {
        activity: ej.datavisualization.Diagram.BPMNActivity.Task,
        task: ej.datavisualization.Diagram.BPMNTask(),
        subProcess: ej.datavisualization.Diagram.BPMNSubProcess(),
    };
    ej.datavisualization.Diagram.BPMNGatewayDefaults = {
        gateway: ej.datavisualization.Diagram.BPMNGateways.None,
    };
    ej.datavisualization.Diagram.BPMNFlowDefaults = {
        direction: ej.datavisualization.Diagram.BPMNDirections.None
    };
    ej.datavisualization.Diagram.BPMNDataObjectDefaults = {
        collection: false,
    };
    ej.datavisualization.Diagram.NodeType = function (options, diagram) {
        var defaultType = diagram.model.defaultSettings && diagram.model.defaultSettings.node && diagram.model.defaultSettings.node.type ? diagram.model.defaultSettings.node.type : "";
        var nodeDefault = !options.isPhase && diagram.model.defaultSettings && diagram.model.defaultSettings.node ? diagram.model.defaultSettings.node : {};
        if (!options.type && !options.children && !options.isLane && !options.isSwimlane && !options.isPhase && !options.isPhaseStack) {
            if (options.segments || options.sourceNode || options.sourcePort || options.sourcePoint || options.targetNode || options.targetPort || options.targetPoint)
                options.type = "connector";
            else 
                options.type = defaultType ? defaultType : "basic";
        }
        if (options.type == "basic" || options.type == "flow" || options.type == "arrow" || options.type == "bpmn" || options.type == "image" || options.type == "native" || options.type === "html" || options.type === "text") {
            options._type = "node";
            if (options.type != "bpmn") options = $.extend(true, {}, nodeDefault, options);
        }
        if (options.type == "basic")
            options = ej.datavisualization.Diagram.BasicShape(options);
        else if (options.type == "flow")
            options = ej.datavisualization.Diagram.FlowShape(options);
        else if (options.type == "arrow")
            options = ej.datavisualization.Diagram.ArrowShape(options);
        else if (options.type == "bpmn")
            options = ej.datavisualization.Diagram.BPMNShape(options, diagram);
        return options;
    };
    ej.datavisualization.Diagram.BasicShape = function (options) {
        options._shape = "path";
        if (!options.shape) options.shape = "rectangle";
        switch (options.shape) {
            case "rectangle":
                options._shape = "rectangle";
                break;
            case "ellipse":
                options._shape = "ellipse";
                break;
            case "path":
                options._shape = "path";
                break;
            case "polygon":
                options._shape = "polygon";
                break;
            case "triangle":
                options.pathData = "M81.1582,85.8677L111.1582,33.9067L141.1582,85.8677L81.1582,85.8677z";
                break;
            case "plus":
                options.pathData = "M696.6084,158.2656L674.8074,158.2656L674.8074,136.4656L658.4084,136.4656L658.4084,158.2656L636.6084,158.2656L636.6084,174.6646L658.4084,174.6646L658.4084,196.4656L674.8074,196.4656L674.8074,174.6646L696.6084,174.6646L696.6084,158.2656z";
                break;
            case "star":
                options.pathData = "M540.3643,137.9336L546.7973,159.7016L570.3633,159.7296L550.7723,171.9366L558.9053,194.9966L540.3643,179.4996L521.8223,194.9966L529.9553,171.9366L510.3633,159.7296L533.9313,159.7016L540.3643,137.9336z";
                break;
            case "pentagon":
                options.pathData = "M370.9702,194.9961L359.5112,159.7291L389.5112,137.9341L419.5112,159.7291L408.0522,194.9961L370.9702,194.9961z";
                break;
            case "heptagon":
                options.pathData = "M223.7783,195.7134L207.1303,174.8364L213.0713,148.8034L237.1303,137.2174L261.1883,148.8034L267.1303,174.8364L250.4813,195.7134L223.7783,195.7134z";
                break;
            case "octagon":
                options.pathData = "M98.7319,196.4653L81.1579,178.8923L81.1579,154.0393L98.7319,136.4653L123.5849,136.4653L141.1579,154.0393L141.1579,178.8923L123.5849,196.4653L98.7319,196.4653z";
                break;
            case "trapezoid":
                options.pathData = "M127.2842,291.4492L95.0322,291.4492L81.1582,256.3152L141.1582,256.3152L127.2842,291.4492z";
                break;
            case "decagon":
                options.pathData = "M657.3379,302.4141L642.3369,291.5161L636.6089,273.8821L642.3369,256.2481L657.3379,245.3511L675.8789,245.3511L690.8789,256.2481L696.6089,273.8821L690.8789,291.5161L675.8789,302.4141L657.3379,302.4141z";
                break;
            case "righttriangle":
                options.pathData = "M836.293,292.9238L776.293,292.9238L776.293,254.8408L836.293,292.9238z";
                break;
            case "cylinder":
                options.pathData = "M 542.802,362.009C 542.802,368.452 525.341,373.676 503.802,373.676C 482.263,373.676 464.802,368.452 464.802,362.009L 464.802,466.484C 464.802,472.928 482.263,478.151 503.802,478.151C 525.341,478.151 542.802,472.928 542.802,466.484L 542.802,362.016C 542.802,368.459 525.341,373.534 503.802,373.534C 482.263,373.534 464.802,368.31 464.802,361.867L 464.802,362.016C 464.802,355.572 482.263,350.349 503.802,350.349C 525.341,350.349 542.802,355.572 542.802,362.016";
                break;
                //case "diamond": options.pathData = "M 397.784,287.875L 369.5,316.159L 341.216,287.875L 369.5,259.591L 397.784,287.875 Z"; break;
        }
        return options;
    };
    ej.datavisualization.Diagram.FlowShape = function (options) {
        options._shape = "path";
        if (!options.shape) options.shape = "process";
        var constraints = options.constraints ? options.constraints : ej.datavisualization.Diagram.NodeConstraints.Default;
        if (!(options.constraints & ej.datavisualization.Diagram.NodeConstraints.AspectRatio))
            constraints = constraints | ej.datavisualization.Diagram.NodeConstraints.AspectRatio;
        switch (options.shape) {
            case "process":
                options.pathData = "M419.511,76.687L359.511,76.687L359.511,43.086L419.511,43.086z";
                break;
            case "decision":
                options.pathData = "M 253.005,115.687L 200.567,146.071L 148.097,115.687L 200.534,85.304L 253.005,115.687 Z";
                break;
            case "document":
                options.pathData = "M 60 31.9 c 0 0 -11 -7.7 -30 0 s -30 0 -30 0 V 0 h 60 V 31.9 Z";
                break;
            case "predefinedprocess":
                options.pathData = "M 0,0 L 50,0 L 50,50 L 0,50 Z  M 8.333333333333334,0 L 8.333333333333334,50 M 41.66666666666667,0 L 41.66666666666667,50";
                break;
            case "terminator":
                options.pathData = "M 269.711,29.3333C 269.711,44.061 257.772,56 243.044,56L 158.058,56C 143.33,56 131.391,44.061 131.391,29.3333L 131.391,29.3333C 131.391,14.6057 143.33,2.66669 158.058,2.66669L 243.044,2.66669C 257.772,2.66669 269.711,14.6057 269.711,29.3333 Z";
                break;
            case "papertap":
                options.pathData = "M0.000976562,17.2042 L0.000976562,47.1654 C0.000976562,47.1654 14.403,53.5455 25.001,47.1654 C35.599,40.7852 44.403,43.5087 50.001,47.1654 L50.001,17.2042 M50.001,32.7987 L50.001,2.84052 C50.001,2.84052 35.599,-3.54271 25.001,2.84052 C14.403,9.22376 5.599,6.49418 0.000976562,2.84052 L0.000976562,32.7987";
                break;
            case "directdata":
                options.pathData = "M 1477 613 L 1619 613 C 1627 613 1634 636 1634 665 C 1634 694 1627 717 1619 717 L 1477 717 C 1469 717 1461 694 1461 665 C 1461 636 1469 613 1477 613 ZM 1619 613 C 1610 613 1603 636 1603 665 C 1603 694 1610 717 1619 717M 1619 613 C 1610 613 1603 636 1603 665 C 1603 694 1610 717 1619 717";
                break;
            case "sequentialdata":
                options.pathData = "M0.00297546,24.999 C0.00297546,11.1922 10.433,0.00216177 23.295,0.00216177 C36.159,0.00216177 46.585,11.1922 46.585,24.999 C46.585,38.8057 36.159,49.9979 23.295,49.9979 C10.433,49.9979 0.00297546,38.8057 0.00297546,24.999 z M23.294,49.999 L50.002,49.999";
                break;
            case "sort":
                options.pathData = "M50.001,24.9971 L25.001,49.9971 L0.000976562,24.9971 L25.001,-0.00286865 L50.001,24.9971 z M0.000976562,24.9971 L50.001,24.9971";
                break;
            case "multidocument":
                options.pathData = "M43.6826,40 C44.8746,40.6183 45.8586,41.3502 46.8366,42.1122 L46.8366,4.74487 L3.09857,4.74487 L3.09857,10.9544 M46.837,35.1437 C48.027,35.7653 49.025,36.6042 50.003,37.3695 L50.003,0.0021928 L6.26497,0.0021928 L6.26497,4.74451 M43.6826,47.1132 L43.6826,10.7652 L0.00257874,10.7652 L0.00257874,47.1132 C0.00257874,47.1132 12.5846,53.6101 21.8426,47.1132 C31.1006,40.6163 38.7926,43.3935 43.6826,47.1132 z";
                break;
            case "collate":
                options.pathData = "M50.001,0.00286865 L25.001,25.0029 L0.000976562,0.00286865 L50.001,0.00286865 z M0.000976562,50.0029 L25.001,25.0029 L50.001,50.0029 L0.000976562,50.0029 z";
                break;
            case "summingjunction":
                options.constraints = constraints;
                options.pathData = "M7.3252,42.6768 L42.6772,7.32477 M42.6768,42.6768 L7.3248,7.32477 M0.000976562,25.001 C0.000976562,11.193 11.197,0.000976562 25.001,0.000976562 C38.809,0.000976562 50.001,11.193 50.001,25.001 C50.001,38.809 38.809,50.001 25.001,50.001 C11.197,50.001 0.000976562,38.809 0.000976562,25.001 z";
                break;
            case "or":
                options.pathData = "M 0 50 L 100 50 M 50 100 L 50 0.0 M 0 50 C 0 22.384 22.392 0 50 0 C 77.616 0 100 22.384 100 50 C 100 77.616 77.616 100 50 100 C 22.392 100 0 77.616 0 50 Z";
                break;
            case "internalstorage":
                options.pathData = "M 0 3.8194444444444446A 2.5,3.8194444444444446 0 0,1 2.5,0L 47.5 0A 2.5,3.8194444444444446 0 0,1 50,3.8194444444444446L 50 45.833333333333336A 2.5,3.8194444444444446 0 0,1 47.5,49.65277777777778L 2.5 49.65277777777778A 2.5,3.8194444444444446 0 0,1 0,45.833333333333336L 0 3.8194444444444446ZM 0 11.458333333333334L 50 11.458333333333334M 12.5 0L 12.5 49.65277777777778";
                break;
            case "extract":
                options._shape = "polygon";
                options.constraints = constraints;
                options.points = [{ x: 0, y: 35 }, { x: 30, y: 0 }, { x: 60, y: 35 }];

                break;
            case "manualoperation":
                options._shape = "polygon";
                options.points = [{ x: 46.4, y: 28.8 }, { x: 14.8, y: 28.8 }, { x: 0, y: 0 }, { x: 60, y: 0 }];
                break;
            case "merge":
                options._shape = "polygon";
                options.constraints = constraints;
                options.points = [{ x: 60, y: 0 }, { x: 30, y: 35 }, { x: 0, y: 0 }];
                break;
            case "offpagereference":
                options._shape = "polygon";
                options.points = [{ x: 60, y: 33.3 }, { x: 30.1, y: 39 }, { x: 0, y: 33.3 }, { x: 0, y: 0 }, { x: 60, y: 0 }];
                break;
            case "sequentialaccessstorage":
                options.constraints = constraints;
                options.pathData = "M 60 30 C 60 13.4 46.6 0 30 0 S 0 13.4 0 30 s 13.4 30 30 30 h 28.6 v -6.5 h -9.9 C 55.5 48 60 39.5 60 30 Z";
                break;
            case "annotation1":
                options.pathData = "M49.9984,50.0029 L-0.00271199,50.0029 L-0.00271199,0.00286865 L49.9984,0.00286865";
                break;
            case "annotation2":
                options.pathData = "M49.9977,50.0029 L25.416,50.0029 L25.416,0.00286865 L49.9977,0.00286865 M25.4166,25.0029 L-0.00227869,25.0029";
                break;
                //case "display": options.pathData = "M47.8809,19.2914 L32.7968,-0.00594145 L11.3902,-0.00594145 C7.93166,-0.00594145 0.00124586,11.187 0.00124586,24.9968 C0.00124586,38.8032 7.93166,49.9962 11.3902,49.9962 L32.7968,49.9962 L47.615,31.0388 C47.615,31.0388 52.7986,24.9968 47.8809,19.2914 z"; break;
                //case "delay": options.pathData = "M50.0044,-0.00482496 L18.9098,-0.00482496 L3.46654,18.958 C3.46654,18.958 -4.06481,25.0015 3.07999,30.7062 L18.9098,50.0013 L50.0044,50.0013"; break;
                //case "storeddata": options.pathData = "M 5.555555555555555 0L 50 0A 1.5,30 0 0,1 50,0A 5.555555555555555,25 0 0,0 50,50A 1.5,30 0 0,1 50,50L 5.555555555555555 50A 5.555555555555555,25 0 0,1 5.555555555555555,0Z"; break;
                //case "card": options.points = [{ x: 0, y: 12.5 }, { x: 0, y: 50 }, { x: 50, y: 50 }, { x: 50, y: 0 }, { x: 12.5, y: 0 }, { x: 0, y: 12.5 }]; break;
                //case "data": options.points = [{ x: 0, y: 50 }, { x: 37.5, y: 50 }, { x: 50, y: 0 }, { x: 12.5, y: 0 }, { x: 0, y: 50 }]; break;
        }
        return options;
    };
    ej.datavisualization.Diagram.ArrowShape = function (options) {
        options._shape = "path";
        if (!options.shape) options.shape = "circulararrow";
        switch (options.shape) {
            case "circulararrow":
                options.pathData = "M433.4624,503.8848C429.4244,493.2388,419.1354,485.6678,407.0734,485.6678C391.4884,485.6678,378.8544,498.3018,378.8544,513.8868L384.4984,513.8868C384.4984,501.4178,394.6054,491.3108,407.0734,491.3108C415.9494,491.3108,423.6264,496.4338,427.3144,503.8848L422.9114,503.8848L426.8974,508.8848L430.8824,513.8868L434.8684,508.8848L438.8544,503.8848L433.4624,503.8848z";
                break;
            case "curvedrightarrow":
                options.pathData = "M0.83006144,28.226993 C2.4030598,37.092016 9.5750604,44.49601 19.365042,48.044014 L19.365042,43.129004 25.376032,47.706001 31.386044,52.282999 25.376032,56.858988 19.365042,61.436991 19.365042,55.399027 C8.24608,51.372995 0.50004381,42.368995 0.50004357,31.905003 0.50004381,30.65601 0.6160717,29.429996 0.83006144,28.226993 z M31.402985,0.5 L31.402985,7.4419994 C15.834992,7.4419994 2.9619988,16.497999 0.81800008,28.275999 0.61199981,27.139999 0.49999988,25.981999 0.5,24.803999 0.49999988,11.379999 14.334993,0.5 31.402985,0.5 z";
                break;
            case "curveduparrow":
                options.pathData = "M52.283021,0.51798058 L56.859025,6.5279841 61.436024,12.537989 55.399022,12.537989 C51.373021,23.656996 42.369015,31.403002 31.905011,31.403002 30.65601,31.403002 29.430011,31.288002 28.22701,31.073002 37.092015,29.500999 44.496018,22.328995 48.044018,12.537989 L43.129018,12.537989 47.706021,6.5279841 z M0.5,0.5 L7.4420163,0.5 C7.4420163,16.067989 16.497985,28.940971 28.276,31.083977 27.140013,31.290977 25.981993,31.402 24.804015,31.402 11.380005,31.402 0.5,17.567989 0.5,0.5 z";
                break;
            case "curvedleftarrow":
                options.pathData = "M31.073116,28.227982 C31.287107,29.429977 31.403074,30.65599 31.403074,31.905989 31.403074,42.368966 23.657099,51.372959 12.538074,55.399935 L12.538074,61.436979 6.5270846,56.85898 0.51707193,52.282993 6.5270846,47.706976 12.538074,43.129983 12.538074,48.044958 C22.328056,44.496956 29.500118,37.091961 31.073116,28.227982 z M0.5,0.5 C17.568025,0.5 31.403045,11.380997 31.403045,24.803993 31.403045,25.981992 31.291046,27.139992 31.085045,28.276992 28.941042,16.497995 16.068023,7.442997 0.5,7.442997 z";
                break;
            case "curveddownarrow":
                options.pathData = "M24.804015,0.50000453 C25.981993,0.50000453 27.140013,0.61301103 28.276,0.8180277 16.498045,2.9630154 7.4420161,15.836023 7.4420156,31.404005 L0.5,31.404005 C0.5,14.336024 11.380004,0.50000453 24.804015,0.50000453 z M31.90502,0.5 C42.369025,0.49999976 51.37303,8.2460016 55.399031,19.365004 L61.436034,19.365004 56.859034,25.377005 52.28303,31.387007 47.70603,25.377005 43.129027,19.365004 48.044028,19.365004 C44.496027,9.5760015 37.092024,2.4040001 28.227019,0.82999995 29.43002,0.61699978 30.65602,0.49999976 31.90502,0.5 z";
                break;
            case "jumpingrightarrow":
                options.pathData = "M571.7207,727.7451L563.4507,716.8841L572.3297,716.8841C565.9217,699.1961,552.5427,686.2371,536.5277,683.3951C538.6997,683.0091,540.9157,682.8001,543.1717,682.8001C562.0777,682.8001,578.3457,696.7961,585.6187,716.8841L596.5277,716.8841L588.2577,727.7451L579.9907,738.6041L571.7207,727.7451z";
                break;
            case "jumpingleftarrow":
                options.pathData = "M688.4209,727.7441L696.6889,716.8851L687.8119,716.8851C694.2179,699.1971,707.5989,686.2381,723.6119,683.3941C721.4409,683.0101,719.2259,682.8011,716.9679,682.8011C698.0639,682.8011,681.7959,696.7951,674.5209,716.8851L663.6119,716.8851L671.8839,727.7441L680.1509,738.6031L688.4209,727.7441z";
                break;
                //case "uTurnArrow": options.pathData = "M1093.3154,165.6641L1093.3154,157.0071C1093.3154,145.6621,1084.1184,136.4651,1072.7724,136.4651C1061.4274,136.4651,1052.2304,145.6621,1052.2304,157.0071L1052.2304,196.4651L1061.5834,196.4511L1061.5834,157.5951C1061.5834,151.4141,1066.5934,146.4041,1072.7724,146.4041C1078.9534,146.4041,1083.9624,151.4141,1083.9624,157.5951L1083.9624,165.6641L1074.1574,165.6641L1081.1214,174.9601L1088.0864,184.2561L1095.0514,174.9601L1102.0164,165.6641L1093.3154,165.6641z"; break;
        }
        return options;
    };
    ej.datavisualization.Diagram.BPMNShape = function (options, diagram) {
        if (!options.shape) options.shape = "event";
        if (diagram) {
            var nodeDefault = diagram.model.defaultSettings && diagram.model.defaultSettings.node ? diagram.model.defaultSettings.node : {};
            if (options.shape != "sequentialflow" || options.shape != "associationflow" || options.shape != "messageflow")
            {
                options = $.extend(true, {}, nodeDefault, options);
                if (!(options.labels && options.labels.length))
                    options.labels = [{}];
                if (!options.fillColor)
                    options.fillColor = "white";
                if (!options.borderColor)
                    options.borderColor = "black";
                if (!options.borderWidth)
                    options.borderWidth = 1;
                if (!options.opacity)
                    options.opacity = 1;
                if (!options.gradient)
                    options.gradient = null;
                if (!(options.labels.length && options.labels[0].offset))
                    options.labels[0] = $.extend(true, {}, options.labels[0], { offset: { x: 0.5, y: 1 }, verticalAlignment: "top", margin: { top: 2 } });
            }
            switch (options.shape) {
                case "event":
                    options = ej.datavisualization.Diagram.DefautShapes._getBPMNEventShape(options, diagram);
                    break;
                case "gateway":
                    options = ej.datavisualization.Diagram.DefautShapes._getBPMNGatewayShape(options, diagram);
                    break;
                case "dataobject":
                    options = ej.datavisualization.Diagram.DefautShapes._getBPMNDataShape(options, diagram);
                    break;
                case "activity":
                    options = ej.datavisualization.Diagram.DefautShapes._getBPMNActivityShape(options, diagram);
                    break;
                case "message":
                    options = $.extend(true, {}, options, { _shape: "polygon", points: [{ x: 0, y: 0 }, { x: 19.8, y: 12.8 }, { x: 40, y: 0 }, { x: 0, y: 0 }, { x: 0, y: 25.5 }, { x: 40, y: 25.5 }, { x: 40, y: 0 }] });
                    break;
                case "datasource":
                    options = $.extend(true, {}, options, { _shape: "path", pathData: "M 0 10.6 c 0 5.9 16.8 10.6 37.5 10.6 S 75 16.4 75 10.6 v 0 v 68.9 v -0.1 C 75 85.3 58.2 90 37.5 90 S 0 85.3 0 79.4 l 0 0.1 V 56 V 40.6 L 0 10.6 C 0 4.7 16.8 0 37.5 0 S 75 4.7 75 10.6 S 58.2 21.2 37.5 21.2 S 0 16.5 0 10.6 l 0 6.7 v -0.2 c 0 5.9 16.8 10.6 37.5 10.6 S 75 22.9 75 17.1 v 6.8 v -0.1 c 0 5.9 -16.8 10.6 -37.5 10.6 S 0 29.6 0 23.8" });
                    break;
                case "sequentialflow":
                    options = $.extend(true, {}, options, { segments: [{ type: "straight" }], targetDecorator: { shape: "arrow" }, lineWidth: 2 });
                    break;
                case "associationflow":
                    options = ej.datavisualization.Diagram.DefautShapes._getBPMNAssociationShape(options, diagram);
                    break;
                case "messageflow":
                    options = $.extend(true, {}, options, { segments: [{ type: "straight" }], sourceDecorator: { shape: "circle", fillColor: "white" }, targetDecorator: { shape: "arrow", fillColor: "white" }, lineWidth: 2, lineDashArray: "4 4" });
                    break;
            }
        }
        return options;
    };
    ej.datavisualization.Diagram.DefautShapes = {
        //#region BPMN
        _getBPMNEventShape: function (options, diagram) {
            var childNodes = [], i;
            if (options.children == undefined) options.children = [];
            options._type = "group";
            options.canUngroup = false;
            options = $.extend(true, {}, ej.datavisualization.Diagram.BPMNEventDefaults, options);
            for (i = 0; i < options.children.length; i++) {
                if (diagram.nameTable[options.children[i].name || options.children[i]])
                    childNodes.push(diagram.nameTable[options.children[i].name || options.children[i]]);
            }
            var pivot = options.pivot || { x: 0.5, y: 0.5 };
            var centerX = options.offsetX - options.width * pivot.x + options.width / 2;
            var centerY = options.offsetY - options.height * pivot.y + options.height / 2;
            if (!childNodes.length) {
                var constraints = ej.datavisualization.Diagram.NodeConstraints.Default & ~(ej.datavisualization.Diagram.NodeConstraints.Select | ej.datavisualization.Diagram.NodeConstraints.Connect);
                var defaultProperty = { parent: options.name, type: "node", shape: "path", offsetX: centerX, offsetY: centerY, ports: [] };
                childNodes.push({ name: options.name + "_0", width: options.width, height: options.height, pathData: "M164.1884,84.6909000000001C156.2414,84.6909000000001,149.7764,78.2259000000001,149.7764,70.2769000000001C149.7764,62.3279000000001,156.2414,55.8629000000001,164.1884,55.8629000000001C172.1354,55.8629000000001,178.6024,62.3279000000001,178.6024,70.2769000000001C178.6024,78.2259000000001,172.1354,84.6909000000001,164.1884,84.6909000000001", constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_1", width: options.width * .85, height: options.height * .85, pathData: "M164.1884,84.6909000000001C156.2414,84.6909000000001,149.7764,78.2259000000001,149.7764,70.2769000000001C149.7764,62.3279000000001,156.2414,55.8629000000001,164.1884,55.8629000000001C172.1354,55.8629000000001,178.6024,62.3279000000001,178.6024,70.2769000000001C178.6024,78.2259000000001,172.1354,84.6909000000001,164.1884,84.6909000000001", constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_2", width: options.width * .5, height: options.height * .5, pathData: "M164.1884,84.6909000000001C156.2414,84.6909000000001,149.7764,78.2259000000001,149.7764,70.2769000000001C149.7764,62.3279000000001,156.2414,55.8629000000001,164.1884,55.8629000000001C172.1354,55.8629000000001,178.6024,62.3279000000001,178.6024,70.2769000000001C178.6024,78.2259000000001,172.1354,84.6909000000001,164.1884,84.6909000000001", constraints: constraints, _isDisabled: true });
                for (i = 0; i < childNodes.length; i++)
                    childNodes[i] = ej.datavisualization.Diagram.Node($.extend(true, {}, defaultProperty, childNodes[i]));
                options.children = childNodes;
                options.constraints = options.constraints ? options.constraints : ej.datavisualization.Diagram.NodeConstraints.Default;
            }
            childNodes[0].fillColor = "white";
            childNodes[1].fillColor = options.fillColor;
            childNodes[2].fillColor = "white";
            childNodes[0].borderColor = options.borderColor;
            childNodes[1].borderColor = options.borderColor;
            childNodes[2].borderColor = options.borderColor;
            childNodes[0].borderDashArray = "1 0";
            childNodes[1].borderDashArray = "1 0";
            childNodes[1].visible = true;
            childNodes[2].visible = true;
            childNodes[0].borderWidth = options.borderWidth;
            childNodes[0].opacity = options.opacity;
            childNodes[1].opacity = options.opacity;
            childNodes[2].opacity = options.opacity;
            childNodes[0].gradient = null;
            childNodes[1].gradient = options.gradient;
            childNodes[2].height = options.height * .5;
            switch (options.event) {
                case "start":
                    childNodes[0].fillColor = childNodes[1].fillColor;
                    childNodes[0].gradient = options.gradient;
                    childNodes[1].visible = false;
                    break;
                case "noninterruptingstart":
                    childNodes[0].fillColor = childNodes[1].fillColor;
                    childNodes[0].gradient = options.gradient;
                    childNodes[0].borderDashArray = "2 3";
                    childNodes[1].visible = false;
                    break;
                case "noninterruptingintermediate":
                    childNodes[0].borderDashArray = "2 3";
                    childNodes[1].borderDashArray = "2 3";
                    break;
                case "end":
                    childNodes[0].fillColor = "black";
                    childNodes[2].fillColor = "black";
                    childNodes[2].borderColor = options.fillColor;
                    break;
            }
            switch (options.trigger) {
                case ej.datavisualization.Diagram.BPMNTriggers.None:
                    childNodes[2].visible = false;
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Message:
                    childNodes[2].pathData = "M0,0  L19.8,12.8 L40,0 L0, 0 L0, 25.5  L40, 25.5 L 40, 0";
                    childNodes[2].height = 3 * childNodes[2].width / 4;
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Timer:
                    childNodes[2].pathData = "M40,20c0,8.654-5.496,16.024-13.189,18.81" +
		"C24.685,39.58,22.392,40,20,40C8.954,40,0,31.046,0,20S8.954,0,20,0S40,8.954,40,20z M20,0 L20,2.583 L20,5.283 M10.027,2.681 L11.659,5.507 L12.669,7.257 M2.731,9.989 L6.014,11.885 L7.307,12.631 M0.067,19.967 L2.667,19.967 L5.35,19.967" +
    "M2.748,29.939 L5.731,28.217 L7.323,27.298 M10.056,37.236 L11.292,35.095 L12.698,32.66 M20.033,39.9 L20.033,36.417 L20.033,34.617 M30.006,37.219 L28.893,35.292 L27.364,32.643 M37.302,29.911 L34.608,28.355 L32.727,27.269" +
     "M39.967,19.933 L37.417,19.933 L34.683,19.933 M37.286,9.961 L34.583,11.521 L32.71,12.602 M29.977,2.664 L28.653,4.957 L27.336,7.24 M22.104,8.5 L19.688,20 L24.75,20 L31.604,20 L24.75,20 L19.688,20z";
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Escalation:
                    childNodes[2].pathData = "M 30.001 8.098 L 11.842 43.543 l 18.159 -18.882 l 18.162 18.882 L 30.001 8.098 Z ";
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Link:
                    childNodes[2].pathData = "M 32.014 19.258 v 5.992 H 9.373 v 9.504 h 22.641 v 5.988 L 50.622 30 L 32.014 19.258 Z";
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Error:
                    childNodes[2].pathData = "M 23.77 18.527 l -7.107 27.396 l 8.507 -17.247 L 36.94 40.073 l 6.394 -25.997 l -8.497 15.754 L 23.77 18.527 Z";
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Compensation:
                    childNodes[2].pathData = "M 25.7086 0 L 0 25 L 25.7086 50 V 26.3752 L 50 50 V 0 L 25.7086 23.6248 V 0 Z ";
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Signal:
                    childNodes[2].pathData = "M 50 50 H 0 L 25.0025 0 L 50 50 Z";
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Multiple:
                    childNodes[2].pathData = "M 17.784 48.889 H 42.21 l 7.548 -23.23 L 29.997 11.303 L 10.236 25.658 L 17.784 48.889 Z";
                    break;
                case ej.datavisualization.Diagram.BPMNTriggers.Parallel:
                    childNodes[2].pathData = "M 27.276 49.986 h 5.58 v -17.15 h 17.146 V 27.17 h -17.15 l 0.004 -17.15 h -5.58 l -0.004 17.15 H 9.994 v 5.666 h 17.278 L 27.276 49.986 Z";
                    break;
            }
            return options;
        },
        _getBPMNGatewayShape: function (options, diagram) {
            var childNodes = [], i;
            if (options.children == undefined) options.children = [];
            options._type = "group";
            options.canUngroup = false;
            options = $.extend(true, {}, ej.datavisualization.Diagram.BPMNGatewayDefaults, options);
            for (i = 0; i < options.children.length; i++) {
                if (diagram.nameTable[options.children[i].name || options.children[i]])
                    childNodes.push(diagram.nameTable[options.children[i].name || options.children[i]]);
            }
            var pivot = options.pivot || { x: 0.5, y: 0.5 };
            var centerX = options.offsetX - options.width * pivot.x + options.width / 2;
            var centerY = options.offsetY - options.height * pivot.y + options.height / 2;
            if (!childNodes.length) {
                var constraints = ej.datavisualization.Diagram.NodeConstraints.Default ^ (ej.datavisualization.Diagram.NodeConstraints.Select | ej.datavisualization.Diagram.NodeConstraints.Connect);
                var defaultProperty = { parent: options.name, type: "node", shape: "path", offsetX: centerX, offsetY: centerY, ports: [] };
                childNodes.push({ name: options.name + "_0", width: options.width, height: options.height, pathData: "M 40 20 L 20 40 L 0 20 L 20 0 L 40 20 Z", constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_1", width: options.width * .45, height: options.height * .45, borderColor: "white", fillColor: "black", constraints: constraints, _isDisabled: true });
                for (i = 0; i < childNodes.length; i++)
                    childNodes[i] = ej.datavisualization.Diagram.Node($.extend(true, {}, defaultProperty, childNodes[i]));
                options.children = childNodes;
                options.constraints = options.constraints ? options.constraints : ej.datavisualization.Diagram.NodeConstraints.Default;
            }
            childNodes[0].fillColor = options.fillColor;
            childNodes[0].borderColor = options.borderColor;
            childNodes[0].borderWidth = options.borderWidth;
            childNodes[0].opacity = options.opacity;
            childNodes[1].opacity = options.opacity;
            childNodes[0].gradient = options.gradient;
            childNodes[1].visible = true;
            childNodes[1].shape = childNodes[1]._shape = "path";
            switch (options.gateway) {
                case "none": childNodes[1].visible = false; break;
                case "exclusive": childNodes[1].pathData = "M 11.196 29.009 l 6.36 -9.712 l -5.764 -8.899 h 4.393 l 3.732 5.979 l 3.656 -5.979 h 4.354 l -5.789 9.039 l 6.36 9.572 h -4.532 l -4.126 -6.437 l -4.139 6.437 H 11.196 Z"; break;
                case "inclusive": childNodes[1].pathData = "M 20.323 31.333 c -6.625 0 -12.015 -5.39 -12.015 -12.015 s 5.39 -12.015 12.015 -12.015 s 12.016 5.39 12.016 12.015 S 26.948 31.333 20.323 31.333 Z M 20.323 9.303 c -5.522 0 -10.015 4.493 -10.015 10.015 s 4.492 10.015 10.015 10.015 s 10.016 -4.493 10.016 -10.015 S 25.846 9.303 20.323 9.303 Z"; break;
                case "parallel": childNodes[1].pathData = "M 18.394 29.542 v -8.833 H 9.626 v -3.691 h 8.768 V 8.251 h 3.734 v 8.767 h 8.768 v 3.691 h -8.768 v 8.833 H 18.394 Z"; break;
                case "eventbased": childNodes[1].pathData = "M 20.322 29.874 c -5.444 0 -9.873 -4.43 -9.873 -9.874 s 4.429 -9.874 9.873 -9.874 s 9.874 4.429 9.874 9.874 S 25.767 29.874 20.322 29.874 Z M 20.322 10.876 c -5.03 0 -9.123 4.093 -9.123 9.124 s 4.093 9.124 9.123 9.124 c 5.031 0 9.124 -4.093 9.124 -9.124 S 25.354 10.876 20.322 10.876 ZM 20.322 32.891 c -7.107 0 -12.89 -5.783 -12.89 -12.891 c 0 -7.107 5.782 -12.89 12.89 -12.89 c 7.108 0 12.891 5.783 12.891 12.89 C 33.213 27.108 27.431 32.891 20.322 32.891 Z M 20.322 7.86 c -6.693 0 -12.14 5.446 -12.14 12.14 s 5.446 12.141 12.14 12.141 c 6.694 0 12.141 -5.446 12.141 -12.141 S 27.017 7.86 20.322 7.86 ZM 24.191 25.386 h -7.984 l -2.469 -7.595 l 6.461 -4.693 l 6.461 4.693 L 24.191 25.386 Z M 16.934 24.386 h 6.531 l 2.02 -6.213 l -5.285 -3.839 l -5.285 3.839 L 16.934 24.386 Z"; break;
                case "complex":
                    childNodes[1].shape = childNodes[1]._shape = "polygon";
                    childNodes[1].points = [{ x: 29.198, y: 19.063 }, { x: 23.089, y: 19.063 }, { x: 27.794, y: 14.358 }, { x: 26.38, y: 12.944 }, { x: 21.223, y: 18.101 }, { x: 21.223, y: 10.443 }, { x: 19.223, y: 10.443 }, { x: 19.223, y: 17.976 }, { x: 14.022, y: 12.776 }, { x: 12.608, y: 14.19 }, { x: 17.48, y: 19.063 }, { x: 10.365, y: 19.063 }, { x: 10.365, y: 21.063 }, { x: 18.261, y: 21.063 }, { x: 12.392, y: 26.932 }, { x: 13.806, y: 28.346 }, { x: 19.223, y: 22.929 }, { x: 19.223, y: 30.225 }, { x: 21.223, y: 30.225 }, { x: 21.223, y: 22.805 }, { x: 25.925, y: 27.507 }, { x: 27.339, y: 26.093 }, { x: 22.309, y: 21.063 }, { x: 29.198, y: 21.063 }];
                    break;
            }
            return options;
        },
        _getBPMNDataShape: function (options, diagram) {
            var childNodes = [], i;
            if (options.children == undefined) options.children = [];
            options._type = "group";
            var pivot = options.pivot || { x: 0.5, y: 0.5 };
            options.canUngroup = false;
            options = $.extend(true, {}, ej.datavisualization.Diagram.BPMNDataObjectDefaults, options);
            var y = options.offsetY ? options.offsetY - options.height * pivot.y : -options.height * pivot.y;
            for (i = 0; i < options.children.length; i++) {
                if (diagram.nameTable[options.children[i].name || options.children[i]])
                    childNodes.push(diagram.nameTable[options.children[i].name || options.children[i]]);
            }
            var centerX = options.offsetX - options.width * pivot.x + options.width / 2;
            if (!childNodes.length) {
                var constraints = ej.datavisualization.Diagram.NodeConstraints.Default ^ (ej.datavisualization.Diagram.NodeConstraints.Select | ej.datavisualization.Diagram.NodeConstraints.Connect);
                var defaultProperty = { parent: options.name, type: "node", shape: "path", offsetX: centerX, ports: [] };
                childNodes.push({ name: options.name + "_0", width: options.width, height: options.height, offsetY: y + options.height / 2, shape: "polygon", points: [{ x: 29.904, y: 5 }, { x: 7.853, y: 5 }, { x: 7.853, y: 45 }, { x: 42.147, y: 45 }, { x: 42.147, y: 17.242 }, { x: 29.932, y: 5 }, { x: 29.932, y: 17.242 }, { x: 42.147, y: 17.242 }], constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_1", width: options.width * .075, height: options.height * .15, offsetY: options.height * .85 + y, fillColor: "black", pathData: "M 0 0 L 0.1 0 L 0.1 2 L 0 2 Z M 0.4 0 L 0.6 0 L 0.6 2 L0.4 2 Z M 0.9 0 L 1 0 L 1 2 L 0.9 2 Z ", constraints: constraints, _isDisabled: true });
                for (i = 0; i < childNodes.length; i++)
                    childNodes[i] = ej.datavisualization.Diagram.Node($.extend(true, {}, defaultProperty, childNodes[i]));
                options.children = childNodes;
                options.constraints = options.constraints ? options.constraints : ej.datavisualization.Diagram.NodeConstraints.Default;
            }
            childNodes[0].fillColor = options.fillColor;
            childNodes[0].borderColor = options.borderColor;
            childNodes[0].borderWidth = options.borderWidth;
            childNodes[0].opacity = options.opacity;
            childNodes[1].opacity = options.opacity;
            childNodes[0].gradient = options.gradient;
            childNodes[1].visible = options.collection ? true : false;
            return options;
        },
        _getBPMNActivityShape: function (options, diagram) {
            var childNodes = [], i;
            if (options.children == undefined) options.children = [];
            options._type = "group";
            options.canUngroup = false;
            options = $.extend(true, {}, ej.datavisualization.Diagram.BPMNActivityDefaults, options);
            var pivot = options.pivot || { x: 0.5, y: 0.5 };
            var x = options.offsetX ? options.offsetX - options.width * pivot.x : -options.width * pivot.x;
            var y = options.offsetY ? options.offsetY - options.height * pivot.y : -options.height * pivot.y;
            var centerX = options.offsetX - options.width * pivot.x + options.width / 2;
            for (i = 0; i < options.children.length; i++) {
                if (diagram.nameTable[options.children[i].name || options.children[i]])
                    childNodes.push(diagram.nameTable[options.children[i].name || options.children[i]]);
            }
            if (!childNodes.length) {
                var constraints = ej.datavisualization.Diagram.NodeConstraints.Default & ~(ej.datavisualization.Diagram.NodeConstraints.Select | ej.datavisualization.Diagram.NodeConstraints.Connect);
                var defaultProperty = { parent: options.name, type: "node", shape: "path", ports: [] };
                childNodes.push({ name: options.name + "_0", width: options.width, height: options.height, offsetX: x + options.width * .5, offsetY: y + options.height * .5, pathData: "M 90 82.895 C 90 86.819 86.776 90 82.8 90 H 7.2 C 3.224 90 0 86.819 0 82.895 V 7.105 C 0 3.181 3.224 0 7.2 0 h 75.6 C 86.776 0 90 3.181 90 7.105 V 82.895 Z", constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_1", width: options.width * .075, height: options.height * .075, offsetX: x + options.width * .9, offsetY: y + options.height * .9, borderColor: "black", fillColor: "black", constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_2", width: options.width * .14, height: options.height * .14, offsetX: x + options.width * .14, offsetY: y + options.height * .14, borderColor: "black", constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_3", width: options.width * .125, height: options.height * .125, offsetX: x + options.width * 0.6, offsetY: y + options.height * .9, borderColor: "black", fillColor: "black", constraints: constraints, _isDisabled: true });
                childNodes.push({ name: options.name + "_4", width: options.width * .15, height: options.height * .15, offsetX: x + options.width * 0.7, offsetY: y + options.height * .9, borderColor: "black", fillColor: "black", constraints: constraints, _isDisabled: true });
                for (i = 0; i < childNodes.length; i++)
                    childNodes[i] = ej.datavisualization.Diagram.Node($.extend(true, {}, defaultProperty, childNodes[i]));
                options.children = childNodes;
                options.constraints = options.constraints ? options.constraints : ej.datavisualization.Diagram.NodeConstraints.Default;
            }
            childNodes[0].fillColor = options.fillColor;
            childNodes[0].borderColor = options.borderColor;
            childNodes[0].borderWidth = options.borderWidth;
            childNodes[1].visible = true;
            childNodes[2].visible = true;
            childNodes[3].visible = true;
            childNodes[4].visible = false;
            childNodes[0].borderWidth = 1;
            childNodes[0].borderDashArray = "1 0";
            childNodes[2].width = options.width * .14;
            childNodes[2].height = options.height * .14;
            childNodes[0].opacity = childNodes[1].opacity = childNodes[2].opacity = childNodes[3].opacity = childNodes[4].opacity = options.opacity;
            childNodes[0].gradient = options.gradient;
            var loop = options.activity == "task" ? options.task.loop : options.subProcess.loop;
            var compensation = options.activity == "task" ? options.task.compensation : options.subProcess.compensation;
            switch (loop) {
                case "none":
                    childNodes[1].visible = false;
                    break;
                case "standard":
                    childNodes[1].pathData = "M 52.002 73.379 c -2.494 -2.536 -6.55 -2.534 -9.043 0 c -1.208 1.228 -1.874 2.861 -1.874 4.598 c 0 1.225 0.337 2.395 0.957 3.411 l -1.167 1.186 l 2.071 0.458 l 2.071 0.458 l -0.45 -2.106 l -0.45 -2.106 l -1.292 1.314 c -1.119 -2.065 -0.842 -4.709 0.877 -6.458 c 2.084 -2.119 5.475 -2.117 7.557 0 c 2.083 2.119 2.083 5.565 0 7.685 c -0.976 0.992 -2.272 1.557 -3.65 1.59 l 0.025 1.068 c 1.65 -0.041 3.2 -0.716 4.368 -1.903 c 1.208 -1.228 1.874 -2.861 1.874 -4.597 C 53.875 76.24 53.209 74.607 52.002 73.379 Z";
                    break;
                case "parallelmultiinstance":
                    childNodes[1].pathData = "M 51.5,69.5 L52.5,69.5 L52.5,84.5 L51.5 84.5 Z M 46.5,69.5 L47.5,69.5 L47.5,84.5 L46.5 84.5 Z M 41.5,69.5 L42.5,69.5 L42.5,84.5 L41.5 84.5 Z  ";
                    break;
                case "sequencemultiinstance":
                    childNodes[1].pathData = "M 40.375,71.5 L 55.375,71.5 L 55.375,72.5 L 40.375,72.5 Z M 40.375,76.5 L 55.375,76.5 L 55.375,77.5 L 40.375,77.5 Z M 40.375,76.5 L 55.375,76.5 L 55.375,77.5 L 40.375,77.5 Z M 40.375,81.5 L 55.375,81.5 L 55.375,82.5 L 40.375,82.5 Z";;
                    break;
            }
            if (compensation)
                childNodes[3].pathData = "M 22.462 18.754 l -6.79 3.92 l 6.79 3.92 V 22.89 l 6.415 3.705 v -7.841 l -6.415 3.705 V 18.754 Z M 28.331 19.701 v 5.947 l -5.149 -2.973 L 28.331 19.701 Z M 21.916 25.647 l -5.15 -2.973 l 5.15 -2.973 V 25.647 Z M 22.275 12.674 c -5.513 0 -9.999 4.486 -9.999 9.999 c 0 5.514 4.486 10.001 9.999 10.001 c 5.514 0 9.999 -4.486 9.999 -10.001 C 32.274 17.16 27.789 12.674 22.275 12.674 Z M 22.275 32.127 c -5.212 0 -9.453 -4.241 -9.453 -9.454 c 0 -5.212 4.241 -9.453 9.453 -9.453 c 5.212 0 9.453 4.241 9.453 9.453 C 31.728 27.887 27.487 32.127 22.275 32.127 Z";
            else
                childNodes[3].visible = false;

            if (options.activity == ej.datavisualization.Diagram.BPMNActivity.Task) {
                switch (options.task.type) {
                    case "none":
                        childNodes[2].visible = false;
                        break;
                    case "service":
                        childNodes[2].pathData = "M 32.699 20.187 v -4.005 h -3.32 c -0.125 -0.43 -0.292 -0.83 -0.488 -1.21 l 2.373 -2.375 l -2.833 -2.83 l -2.333 2.333 c -0.44 -0.253 -0.9 -0.448 -1.387 -0.595 v -3.32 h -4.003 v 3.32 c -0.46 0.137 -0.89 0.322 -1.3 0.537 l -2.285 -2.275 l -2.833 2.83 l 2.285 2.278 c -0.235 0.42 -0.41 0.847 -0.547 1.307 h -3.33 v 4.005 h 3.33 c 0.148 0.488 0.343 0.955 0.588 1.395 l -2.325 2.325 l 2.822 2.832 l 2.373 -2.382 c 0.392 0.205 0.792 0.37 1.212 0.497 v 3.33 h 4.003 v -3.33 c 0.46 -0.138 0.89 -0.323 1.3 -0.547 l 2.43 2.432 l 2.822 -2.832 l -2.42 -2.422 c 0.222 -0.41 0.4 -0.85 0.535 -1.297 H 32.699 Z M 22.699 21.987 c -2.1 0 -3.803 -1.703 -3.803 -3.803 c 0 -2.1 1.703 -3.803 3.803 -3.803 c 2.1 0 3.803 1.703 3.803 3.803 C 26.502 20.285 24.8 21.987 22.699 21.987 Z";
                        break;
                    case "receive":
                        childNodes[2].pathData = "M 12.217 12.134 v 13.334 h 20 V 12.134 H 12.217 Z M 30.44 13.007 l -8.223 5.35 l -8.223 -5.35 H 30.44 Z M 13.09 24.594 V 13.459 l 9.127 5.94 l 9.127 -5.94 v 11.135 H 13.09 Z";
                        childNodes[2].height = options.height * .12;
                        break;
                    case "send":
                        childNodes[2].pathData = "M 45.7256 3.16055 L 25 23.4017 L 4.27442 3.16055 H 45.7256 Z M 47.8963 46.8413 H 2.10375 V 4.80813 L 25 27.1709 L 47.8963 4.80813 V 46.8413 Z";
                        childNodes[2].height = options.height * .12;
                        break;
                    case "instantiatingreceive":
                        childNodes[2].pathData = "M 16.306 17.39 v 8.79 h 13.198 v -8.79 H 16.306 Z M 28.375 17.946 l -5.47 3.558 l -5.47 -3.558 H 28.375 Z M 28.948 25.625 H 16.861 v -7.389 l 6.043 3.931 l 6.043 -3.931 V 25.625 Z M 22.905 11.785 c -5.514 0 -9.999 4.486 -9.999 10 c 0 5.514 4.485 10 9.999 10 s 9.999 -4.486 9.999 -10 C 32.904 16.272 28.419 11.785 22.905 11.785 Z M 22.905 31.239 c -5.212 0 -9.453 -4.241 -9.453 -9.454 c 0 -5.212 4.241 -9.453 9.453 -9.453 s 9.452 4.241 9.452 9.453 C 32.357 26.998 28.117 31.239 22.905 31.239 Z";
                        childNodes[2].width = options.width * .2;
                        childNodes[2].height = options.height * .2;
                        break;
                    case "manual":
                        childNodes[2].pathData = "M 13.183 15.325 h 2.911 c 0.105 0 0.207 -0.043 0.281 -0.117 c 0.078 -0.074 0.117 -0.176 0.117 -0.281 c 0 -0.753 0.718 -1.362 1.596 -1.362 h 2.579 c -0.117 0.227 -0.191 0.48 -0.195 0.757 c 0 0.433 0.168 0.851 0.46 1.144 c 0.008 0.004 0.015 0.011 0.019 0.015 c -0.289 0.285 -0.475 0.691 -0.479 1.148 c 0 0.433 0.168 0.846 0.46 1.139 c 0.011 0.012 0.023 0.02 0.035 0.032 c -0.301 0.281 -0.491 0.694 -0.495 1.155 c 0 0.432 0.168 0.847 0.46 1.143 c 0.265 0.266 0.612 0.414 0.975 0.414 h 0.839 c 0.027 0.004 0.051 0.012 0.074 0.012 h 8.443 c 0.352 0 0.636 0.344 0.636 0.761 c 0 0.414 -0.285 0.753 -0.636 0.753 h -6.687 c -0.019 0 -0.035 -0.008 -0.051 -0.008 h -2.27 c -0.121 -0.835 -0.667 -1.187 -1.795 -1.187 h -2.158 c -0.223 0 -0.402 0.18 -0.402 0.403 c 0 0.219 0.179 0.398 0.402 0.398 h 2.158 c 0.972 0 1.019 0.203 1.019 0.784 c 0 0.219 0.179 0.399 0.402 0.399 c 0.008 0 0.016 -0.004 0.027 -0.004 c 0.028 0.004 0.055 0.016 0.082 0.016 h 2.56 c 0.34 0.015 0.616 0.343 0.616 0.752 c 0 0.418 -0.285 0.757 -0.636 0.761 h -0.004 h -6.442 c -0.878 0 -1.595 -0.639 -1.595 -1.427 v -0.683 c 0 -0.109 -0.043 -0.211 -0.114 -0.285 c -0.078 -0.074 -0.179 -0.117 -0.285 -0.117 h -0.004 l -2.989 0.027 c -0.223 0 -0.398 0.184 -0.398 0.402 c 0 0.219 0.179 0.395 0.398 0.395 h 0.004 l 2.591 -0.02 v 0.282 c 0 1.229 1.073 2.223 2.391 2.223 h 3.895 c 0.004 0 0.007 0.004 0.011 0.004 h 2.536 c 0.792 0 1.436 -0.698 1.436 -1.561 c 0 -0.273 -0.07 -0.53 -0.188 -0.752 h 5.49 c 0.792 0 1.436 -0.695 1.436 -1.553 c 0 -0.858 -0.644 -1.557 -1.436 -1.557 h -3.566 c 0.121 -0.226 0.199 -0.487 0.199 -0.768 c 0 -0.468 -0.195 -0.882 -0.495 -1.167 c 0.301 -0.285 0.495 -0.698 0.495 -1.163 c 0 -0.456 -0.191 -0.866 -0.483 -1.152 c 0.293 -0.285 0.483 -0.694 0.483 -1.151 c 0 -0.858 -0.647 -1.557 -1.439 -1.557 h -8.373 c -1.167 0 -2.142 0.757 -2.352 1.76 l -2.548 -0.004 c -0.219 0 -0.399 0.18 -0.399 0.403 C 12.784 15.145 12.964 15.325 13.183 15.325 L 13.183 15.325 Z M 21.907 19.707 c -0.191 0 -0.328 -0.094 -0.41 -0.176 c -0.144 -0.145 -0.226 -0.355 -0.226 -0.577 c 0.003 -0.418 0.289 -0.753 0.643 -0.753 h 4.468 c 0.008 0 0.015 -0.004 0.027 -0.008 h 0.051 c 0.351 0 0.636 0.344 0.636 0.761 c 0 0.414 -0.286 0.753 -0.636 0.753 H 21.907 Z M 27.097 16.629 c 0 0.414 -0.286 0.753 -0.64 0.753 h -4.464 c -0.004 0 -0.004 0 -0.004 0 h -0.082 c -0.191 0 -0.328 -0.098 -0.414 -0.18 c -0.14 -0.145 -0.222 -0.352 -0.222 -0.573 c 0 -0.413 0.285 -0.749 0.631 -0.753 h 3.434 c 0 0 0 0 0.004 0 h 1.116 c 0.008 0 0.012 -0.004 0.02 -0.004 C 26.819 15.887 27.097 16.215 27.097 16.629 L 27.097 16.629 Z M 27.097 14.322 c 0 0.41 -0.278 0.737 -0.62 0.749 c -0.008 0 -0.012 0 -0.016 0 h -3.637 c -0.008 0 -0.015 0.004 -0.023 0.004 h -0.886 c -0.004 0 -0.008 0 -0.012 0 c -0.187 0 -0.324 -0.094 -0.406 -0.176 c -0.144 -0.144 -0.226 -0.355 -0.226 -0.577 c 0.003 -0.414 0.293 -0.753 0.643 -0.753 h 4.468 c 0.008 0 0.015 -0.004 0.027 -0.004 h 0.051 C 26.811 13.565 27.097 13.905 27.097 14.322 L 27.097 14.322 Z M 27.097 14.322";
                        childNodes[2].width = options.width * .2;
                        break;
                    case "businessrule":
                        childNodes[2].pathData = "M 32.844 13.245 h -0.089 v 0 H 13.764 v -0.015 h -1.009 v 16.989 h 0.095 v 0.011 h 19.716 v -0.011 h 0.278 V 13.245 Z M 31.844 14.229 v 4.185 h -18.08 v -4.185 H 31.844 Z M 18.168 25.306 v 3.938 h -4.404 v -3.938 H 18.168 Z M 13.764 24.322 v -4.923 h 4.404 v 4.923 H 13.764 Z M 19.177 25.306 h 12.667 v 3.938 H 19.177 V 25.306 Z M 19.177 24.322 v -4.923 h 12.667 v 4.923 H 19.177 Z";
                        break;
                    case "user":
                        childNodes[2].pathData = "M 21.762 21.935 c 2.584 0 4.687 -2.561 4.687 -5.703 c 0 -3.147 -2.103 -5.703 -4.687 -5.703 c -1.279 0 -2.475 0.61 -3.363 1.721 c -0.855 1.071 -1.327 2.484 -1.324 3.983 C 17.075 19.374 19.178 21.935 21.762 21.935 L 21.762 21.935 Z M 21.762 11.779 c 1.894 0 3.436 1.995 3.436 4.452 c 0 2.453 -1.541 4.452 -3.436 4.452 c -1.895 0 -3.44 -1.999 -3.44 -4.452 C 18.323 13.774 19.864 11.779 21.762 11.779 L 21.762 11.779 Z M 25.699 21.309 c -0.348 0 -0.626 0.277 -0.626 0.626 c 0 0.344 0.277 0.622 0.626 0.622 c 2.136 0 3.875 1.74 3.875 3.879 c 0 0.272 -0.227 0.498 -0.501 0.498 H 14.447 c -0.274 0 -0.497 -0.223 -0.497 -0.498 c 0 -2.139 1.736 -3.879 3.872 -3.879 c 0.344 0 0.625 -0.277 0.625 -0.622 c 0 -0.348 -0.28 -0.626 -0.625 -0.626 c -2.826 0 -5.124 2.297 -5.124 5.126 c 0 0.965 0.784 1.749 1.748 1.749 h 14.626 c 0.964 0 1.748 -0.784 1.748 -1.749 C 30.822 23.606 28.524 21.309 25.699 21.309 L 25.699 21.309 Z M 22.217 9.832 c 0.448 -0.263 0.924 -0.396 1.419 -0.396 c 1.895 0 3.436 1.995 3.436 4.452 c 0 0.439 -0.048 0.873 -0.143 1.284 c -0.08 0.336 0.128 0.672 0.464 0.751 c 0.048 0.012 0.098 0.019 0.143 0.019 c 0.284 0 0.541 -0.195 0.608 -0.483 c 0.119 -0.506 0.18 -1.034 0.18 -1.571 c 0 -3.147 -2.102 -5.703 -4.687 -5.703 c -0.711 0 -1.419 0.198 -2.054 0.573 c -0.296 0.174 -0.397 0.559 -0.219 0.855 C 21.536 9.911 21.921 10.009 22.217 9.832 L 22.217 9.832 Z M 27.697 18.81 c -0.345 0 -0.626 0.277 -0.626 0.622 c 0 0.348 0.281 0.626 0.626 0.626 c 2.137 0 3.75 1.782 3.75 3.918 c 0 0.07 -0.013 0.141 -0.043 0.205 c -0.14 0.314 0.003 0.684 0.318 0.823 c 0.082 0.037 0.167 0.055 0.253 0.055 c 0.241 0 0.466 -0.141 0.57 -0.373 c 0.101 -0.226 0.153 -0.464 0.153 -0.714 C 32.699 21.15 30.523 18.81 27.697 18.81 L 27.697 18.81 Z M 27.697 18.81";
                        break;
                    case "script":
                        childNodes[2].pathData = "M 22.453 15.04 c 0 0 -1.194 -3.741 2.548 -3.774 c 0 0 2.497 0.126 1.766 4.321 c -0.008 0.043 -0.015 0.086 -0.024 0.13 c -0.806 4.323 -2.516 8.42 -3.193 10.581 h 3.904 c 0 0 0.983 4.581 -2.549 4.968 H 13.292 c 0 0 -3.097 -1.42 -1.517 -5.323 l 3 -10.839 H 11.84 c 0 0 -1.129 -2.902 1.709 -3.806 l 11.425 -0.032 l -0.73 0.355 l -1.193 1.726 L 22.453 15.04 Z M 22.409 12.597 c 0 0 -0.242 0.483 -0.278 0.98 h -9.098 c 0 0 -0.06 -0.871 0.714 -1.041 L 22.409 12.597 Z M 26.341 27.734 c 0 0 -0.13 2.678 -2.226 1.871 c 0 0 -0.823 -0.565 -0.758 -1.855 L 26.341 27.734 Z M 22.905 15.008 c 0 0 0.653 -0.258 0.709 -1.501 c 0 0 0.145 -1.144 1.483 -0.693 c 0 0 0.808 0.355 0.259 2.404 c 0 0 -2.226 8.5 -3.032 10.339 c 0 0 -1.064 2.646 0.096 4.226 h -8.581 c 0 0 -1.806 -0.452 -0.741 -3.613 c 0 0 2.935 -9.549 3.193 -11.162 L 22.905 15.008 Z";
                        break;
                }
                if (options.task.call) childNodes[0].borderWidth = 4;

                if (options.task.compensation && options.task.loop != "none") {
                    childNodes[1].offsetX = centerX - childNodes[1].width / 2 - 2;
                    childNodes[1].visible = true;
                    childNodes[3].visible = true;
                    childNodes[3].offsetX = centerX + childNodes[1].width / 2 + 2;
                    childNodes[4].visible = false;
                }
                else if (options.task.loop != "none") {
                    childNodes[1].offsetX = centerX;
                    childNodes[1].visible = true;
                }
                else if (options.task.compensation) {
                    childNodes[3].offsetX = centerX;
                    childNodes[3].visible = true;
                }
            }
            else if (options.activity == ej.datavisualization.Diagram.BPMNActivity.SubProcess) {
                childNodes[2].visible = false;
                switch (options.subProcess.boundary) {
                    case "default":
                        childNodes[0].borderWidth = 1;
                        childNodes[0].borderDashArray = "1 0";
                        break;
                    case "call":
                        childNodes[0].borderWidth = 4;
                        childNodes[0].borderDashArray = "1 0";
                        break;
                    case "event":
                        childNodes[0].borderWidth = 1;
                        childNodes[0].borderDashArray = "2 2";
                        break;
                }
                if (options.subProcess.adhoc)
                    childNodes[4].pathData = "M 47.04 81.045 h -13 v -12 h 13 V 81.045 Z M 35.04 80.045 h 11 v -10 h -11 V 80.045 Z M 37.524 74.67 L 43.555 74.67 L43.555 75.02 L37.524 75.02 Z M 40.165 72.029 L 40.915 72.029 L40.915 78.060 L40.165 78.060 Z     M 49.832 76.811 v -2.906 c 0 0 0.466 -1.469 1.931 -1.5 c 1.465 -0.031 2.331 1.219 2.897 1.688 s 1.06 0.75 1.526 0.75 c 0.466 0 1.548 -0.521 1.682 -1.208 s 0.083 3.083 0.083 3.083 s -0.76 0.969 -1.859 0.969 c -1.066 0 -1.865 -0.625 -2.464 -1.438 s -1.359 -0.998 -2.064 -0.906 C 50.598 75.467 49.832 76.811 49.832 76.811 Z";
                else
                    childNodes[4].pathData = "M 8.13789 15 H 0 V 0 H 8.13789 V 15 Z M 0.625991 13.75 H 7.51189 V 1.25 H 0.625991 V 13.75 Z M 2.18095 7.03125 L 5.95631 7.03125 L 5.95631 7.46875 L 2.18095 7.46875 Z M 3.8342 3.73 L 4.30369 3.73 L 4.30369 11.2687 L 3.8342 11.2687 Z";

                if (options.subProcess.compensation && options.subProcess.loop != "none") {
                    childNodes[1].visible = true;
                    childNodes[3].visible = true;
                    childNodes[4].visible = true;
                    childNodes[1].offsetX = centerX - childNodes[3].width / 2 - childNodes[1].width / 2 - 2;
                    childNodes[3].offsetX = centerX;
                    childNodes[4].offsetX = centerX + childNodes[3].width / 2 + childNodes[4].width / 2 + 2;
                }
                else if (options.subProcess.loop != "none") {
                    childNodes[3].visible = false;
                    childNodes[1].visible = true;
                    childNodes[4].visible = true;
                    childNodes[1].offsetX = centerX - childNodes[1].width / 2 - 2;
                    childNodes[4].offsetX = centerX + childNodes[4].width / 2 + 2;
                }
                else if (options.subProcess.compensation) {
                    childNodes[1].visible = false;
                    childNodes[3].visible = true;
                    childNodes[4].visible = true;
                    childNodes[3].offsetX = centerX - childNodes[3].width / 2 - 2;
                    childNodes[4].offsetX = centerX + childNodes[4].width / 2 + 2;
                }
                else {
                    if (childNodes[3].visible == true) childNodes[3].visible = false;
                    childNodes[4].offsetX = centerX;
                    childNodes[4].visible = true;
                }
            }
            return options;
        },
        _getBPMNAssociationShape: function (options, diagram) {
            var node = { segments: [{ type: "straight" }], lineWidth: 2, lineDashArray: "2 2", targetDecorator: {}, sourceDecorator: {} };
            options = $.extend(true, {}, ej.datavisualization.Diagram.BPMNFlowDefaults, options);
            options.constraints = ej.datavisualization.Diagram.ConnectorConstraints.Default;
            options._type = "connector";
            switch (options.direction) {
                case "none":
                    node.targetDecorator.shape = "none";
                    node.sourceDecorator.shape = "none";
                    break;
                case "unidirectional":
                    node.targetDecorator.shape = "arrow";
                    node.sourceDecorator.shape = "none";
                    break;
                case "bidirectional":
                    node.targetDecorator.shape = "arrow";
                    node.sourceDecorator.shape = "arrow";
                    break;
            }
            return $.extend(true, {}, node, options);
        },
        //#endregion
    };
})(jQuery, Syncfusion);;

(function ($, ej) {
    "use strict";

    //#region SVG
    var Svg = (function () {
        function Svg(attr) {
            this.xmlns = "http://www.w3.org/2000/svg";
            this.document = document.createElementNS(this.xmlns, "svg");
            this.id = attr.id;
            if (attr) {
                ej.datavisualization.Diagram.Util.attr(this.document, attr);
            }
            return this;
        }
        Svg.prototype.element = function (attr, tag) {
            var element;
            if (attr && attr.id) {
                element = this.document.getElementById(attr.id);
            }
            if (!element) {
                element = document.createElementNS(this.xmlns, tag);
            }
            if (attr)
                ej.datavisualization.Diagram.Util.attr(element, attr);
            return element;
        };
        Svg.prototype.g = function (attr) {
            return this.element(attr, "g");
        };
        Svg.prototype.defs = function (attr) {
            return this.element(attr, "defs");
        };
        Svg.prototype.image = function (attr) {
            return this.element(attr, "image");
        };
        Svg.prototype.path = function (attr) {
            return this.element(attr, "path");
        };
        Svg.prototype.text = function (attr) {
            return this.element(attr, "text");
        };
        Svg.prototype.textPath = function (attr) {
            return this.element(attr, "textPath");
        };
        Svg.prototype.tspan = function (attr) {
            return this.element(attr, "tspan");
        };
        Svg.prototype.rect = function (attr) {
            return this.element(attr, "rect");
        };
        Svg.prototype.ellipse = function (attr) {
            return this.element(attr, "ellipse");
        };
        Svg.prototype.circle = function (attr) {
            return this.element(attr, "circle");
        };
        Svg.prototype.line = function (attr) {
            return this.element(attr, "line");
        };
        Svg.prototype.polyline = function (attr) {
            return this.element(attr, "polyline");
        };
        Svg.prototype.polygon = function (attr) {
            return this.element(attr, "polygon");
        };
        Svg.prototype.foreignObject = function (attr) {
            return this.element(attr, "foreignObject");
        };
        Svg.prototype.linearGradient = function (attr) {
            return this.element(attr, "linearGradient");
        };
        Svg.prototype.radialGradient = function (attr) {
            return this.element(attr, "radialGradient");
        };
        Svg.prototype.stop = function (attr) {
            return this.element(attr, "stop");
        };
        Svg.prototype.pattern = function (attr) {
            return this.element(attr, "pattern");
        };
        Svg.prototype.appendChild = function (element) {
            this.document.appendChild(element);
        };
        Svg.prototype.removeChild = function (element, parent) {
            if (parent) {
                parent.removeChild(element);
            } else {
                this.document.removeChild(element);
            }
        };
        Svg.prototype.getElementById = function (id) {
            return this.document.getElementById(id);
        };
        Svg.prototype.getElementsByClassName = function (name) {
            return this.document.getElementsByClassName(name);
        };
        Svg.prototype.getElementsByTagName = function (name) {
            return this.document.getElementsByTagName(name);
        };
        Svg.prototype.pathBounds = function (data) {
            var attributes = { d: data };
            var path = this.path(attributes);
            var svg = new ej.datavisualization.Diagram.Svg({ "id": "TempSvg" });
            document.body.appendChild(svg.document);
            svg.document.appendChild(path);
            var bounds = path.getBBox();
            document.body.removeChild(svg.document);
            return bounds;
        };
        Svg.prototype.textBounds = function (text, attr, textElement) {
            var bounds = null;
            if (text && attr) {
                var svgText = this.text(attr);
                svgText.appendChild(document.createTextNode(text));
                this.document.appendChild(svgText);
                bounds = svgText.getBBox();
                this.document.removeChild(svgText);
            }
            else if (textElement) {
                this.document.appendChild(textElement);
                bounds = textElement.getBBox();
                this.document.removeChild(textElement);
            }
            return bounds;
        };
        Svg.prototype.absolutePath = function (path) {
            var x0, y0, x1, y1, x2, y2, segs = path.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if (/[MLHVCSQTA]/.test(char)) {
                    if ('x' in seg) x = seg.x;
                    if ('y' in seg) y = seg.y;
                } else {
                    if ('x1' in seg) x1 = x + seg.x1;
                    if ('x2' in seg) x2 = x + seg.x2;
                    if ('y1' in seg) y1 = y + seg.y1;
                    if ('y2' in seg) y2 = y + seg.y2;
                    if ('x' in seg) x += seg.x;
                    if ('y' in seg) y += seg.y;
                    switch (char) {
                        case 'm': segs.replaceItem(path.createSVGPathSegMovetoAbs(x, y), i); break;
                        case 'l': segs.replaceItem(path.createSVGPathSegLinetoAbs(x, y), i); break;
                        case 'h': segs.replaceItem(path.createSVGPathSegLinetoHorizontalAbs(x), i); break;
                        case 'v': segs.replaceItem(path.createSVGPathSegLinetoVerticalAbs(y), i); break;
                        case 'c': segs.replaceItem(path.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i); break;
                        case 's': segs.replaceItem(path.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i); break;
                        case 'q': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i); break;
                        case 't': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i); break;
                        case 'a': segs.replaceItem(path.createSVGPathSegArcAbs(x, y, seg.r1, seg.r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i); break;
                        case 'z': case 'Z': x = x0; y = y0; break;
                    }
                }
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }
            return path;
        };
        return Svg;
    })();

    ej.datavisualization.Diagram.Svg = Svg;
    //#endregion

    //#region SVG Renderer
    ej.datavisualization.Diagram.SvgContext = {

        _renderDocument: function (view, diagram, isload) {
            if (window.SVGSVGElement) {
                var attr = {
                    id: view.canvas.id + "_svg",
                    version: "1.1",
                    xlink: "http://www.w3.org/1999/xlink",
                    "style": "position:absolute",
                    "class": "mainview_svg"
                };
                if (!isload)
                    var svg = new ej.datavisualization.Diagram.Svg(attr);
                else
                    svg = diagram._svg
                view.svg = svg;
                if (diagram) {
                    diagram._svg = svg;
                    if (!isload && !document.getElementById(svg.document.id))
                        view.canvas.appendChild(svg.document);
                    diagram._view = this._renderView(view.canvas, view.svg, isload);
                    diagram._page = this._renderPage(view.canvas, view.svg, diagram._view, isload);
                    this._renderBackground(view.canvas, view.svg, diagram._page, diagram.model, isload);
                    diagram._pageBackgroundLayer = this._renderBackgroundLayer(view.canvas, view.svg, diagram._page, isload);
                    this._renderGrid(view.canvas, view.svg, diagram._page, diagram);
                    diagram._diagramLayer = view.diagramLayer = this._renderDiagramLayer(view.canvas, view.svg, diagram._page);
                    diagram._htmlLayer = this._renderHtmlLayer(view.canvas, isload);
                    this._renderAdornerLayer(diagram, view.canvas, isload);
                }
                else {


                }
            }
        },

        _renderView: function (canvas, svg, isload) {
            var view = svg.document.getElementById(canvas.id + "_view");
            if (!view) {
                var attr = { id: canvas.id + "_view" };
                view = svg.g(attr);
            }
            if (!isload)
                svg.appendChild(view);
            return view;
        },

        _renderHtmlLayer: function (canvas, isload) {
            var div = document.getElementById(canvas.id + "_htmlLayer");
            if (!div)
                div = document.createElement("div");
            var attr = { "id": canvas.id + "_htmlLayer", "class": "htmlLayer" };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            var htmlLayer = div;
            div.style.pointerEvents = "none";
            if (!isload)
                canvas.appendChild(htmlLayer);
            return htmlLayer;
        },

        _renderAdornerLayer: function (diagram, canvas, isload) {
            var div = document.getElementById(canvas.id + "_adorner");
            if (!div)
                div = document.createElement("div");
            var attr = { "id": canvas.id + "_adorner", "class": "adornerLayer", "style": "position:absolute;left:0px;top:0px" };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            diagram._adornerLayer = this._renderAdornerSvg(diagram, canvas, div, isload);
            diagram._tooltipLayer = this._renderTooltipLayer(canvas, div, isload);
            if (!isload)
                canvas.appendChild(div);
        },

        _renderTooltipLayer: function (canvas, parent, isload) {
            var div = document.getElementById(canvas.id + "_tooltip");
            if (!div)
                div = document.createElement("div");
            var attr = { "id": canvas.id + "_tooltip", "class": "diagram_tooltip" };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            var tooltipdiv = div;
            div.style.pointerEvents = "none";
            if (!isload)
                parent.appendChild(tooltipdiv);
            return tooltipdiv;
        },

        _renderPage: function (canvas, svg, parent, isload) {
            var page = svg.document.getElementById(canvas.id + "_page");
            if (!page) {
                var attr = { "id": canvas.id + "_page", "class": "page" };
                page = svg.g(attr);
            }
            if (!isload)
                parent.appendChild(page);
            return page;
        },

        _renderDiagramLayer: function (canvas, svg, parent) {
            var diagramLayer = svg.document.getElementById(canvas.id + "_diagramLayer");
            if (!diagramLayer) {
                var attr = { "id": canvas.id + "_diagramLayer", "class": "DiagramLayer" };
                diagramLayer = svg.g(attr);
                parent.appendChild(diagramLayer);
            }
            return diagramLayer;
        },

        _renderAdornerSvg: function (diagram, canvas, parent, isload) {
            if (window.SVGSVGElement) {
                var attr = {
                    id: canvas.id + "_svg_adorner",
                    version: "1.1",
                    xlink: "http://www.w3.org/1999/xlink",
                    "style": "position:absolute",
                    "fill": "none",
                    "pointer-events": "none"
                };
                var svg;
                if (!isload)
                    svg = new ej.datavisualization.Diagram.Svg(attr);
                else
                    svg = diagram._adornerSvg;
                if (!isload && !document.getElementById(svg.document.id))
                    parent.appendChild(svg.document);
                var adornerLayer = svg.document.getElementById(canvas.id + "_adornerLayer");
                if (!adornerLayer) {
                    var attr = { "id": canvas.id + "_adornerLayer", "class": "AdornerLayer", "pointer-events": "none" };
                    adornerLayer = svg.g(attr);
                    svg.document.appendChild(adornerLayer);
                }
                diagram._adornerSvg = svg;
                return adornerLayer;
            }
        },

        _renderBackgroundLayer: function (canvas, svg, parent, isload) {
            var pageBackgroundLayer = svg.document.getElementById(canvas.id + "_pageBackground");
            if (!pageBackgroundLayer) {
                var attr = {
                    "id": canvas.id + "_pageBackground",
                    "class": "PageBackgroundLayer",
                    "style": "pointer-events:none;",
                };
                pageBackgroundLayer = svg.g(attr);
            }
            if (!isload)
                parent.appendChild(pageBackgroundLayer);
            return pageBackgroundLayer;
        },

        _renderBackground: function (canvas, svg, parent, model, isload) {
            svg.document.style.msTouchAction = "none";
            svg.document.style.display = "block";
            svg.document.style.backgroundColor = "white";
            var attr = { id: canvas.id + "_backgroundLayer" };
            var g = svg.g(attr);
            attr = { "id": canvas.id + "_background", "x": "0", "y": "0", "width": "100%", "height": "100%", "preserveAspectRatio": "none", "pointer-events": "none" };
            var image = svg.image(attr);
            if (model.backgroundImage) {
                image.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", model.backgroundImage);
                if (!svg.getElementById(canvas.id + "_background")) {
                    g.appendChild(image);
                }
            }
            else {
                if (svg.getElementById(image.id)) {
                    g.removeChild(svg.getElementById(image.id));
                }
            }
            var rect = svg.rect({
                "id": canvas.id + "_backgroundColor", "x": "0", "y": "0", "width": "100%", "height": "100%", "fill": model.backgroundColor, "preserveAspectRatio": "none", "pointer-events": "none"
            });
            if (model.backgroundColor) {
                if (!svg.getElementById(canvas.id + "_backgroundColor")) {
                    g.appendChild(rect);
                }
            }
            else {
                if (svg.getElementById(rect.id)) {
                    g.removeChild(svg.getElementById(rect.id));
                }
            }

            if (!svg.getElementById(g.id))
                parent.appendChild(g);

        },

        _renderGrid: function (canvas, svg, parent, diagram) {
            var defs = diagram._svg.document.getElementById(canvas.id + "patterndefinition")
            if (!defs) {
                defs = svg.defs({ "id": canvas.id + "patterndefinition" });
                svg.appendChild(defs);
            }

            var g;
            if (diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowHorizontalLines ||
                diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowVerticalLines) {
                var grid = document.getElementById(canvas.id + "_grid");
                var rect = document.getElementById(canvas.id + "pattern_gridline");
                var pattern = document.getElementById(canvas.id + "pattern_grid");
                if (pattern && pattern.parentNode)
                    pattern.parentNode.removeChild(pattern);
                var attr = { id: canvas.id + "_grid", "pointer-events": "none" };
                g = svg.g(attr);
                var height = 0;
                if (diagram.model.snapSettings.horizontalGridLines.linesInterval) {
                    diagram.model.snapSettings.horizontalGridLines.linesInterval.forEach(function (entry) {
                        height += entry;
                    });
                }
                var width = 0;
                if (diagram.model.snapSettings.verticalGridLines.linesInterval) {
                    diagram.model.snapSettings.verticalGridLines.linesInterval.forEach(function (entry) {
                        width += entry;
                    });
                }
                var scale = diagram._currZoom;

                attr = { id: canvas.id + "pattern_grid", x: 0, y: 0, width: width * scale, height: height * scale, patternUnits: "userSpaceOnUse" };
                var pattern = svg.pattern(attr);
                //pattern.setAttribute("patternUnits", "userSpaceOnUse");
                scale = this._scaleSnapInterval(diagram.model, scale);
                this._renderHorizontalGridlines(scale, pattern, width, diagram.model, svg);
                this._renderVerticalGridlines(scale, pattern, height, diagram.model, svg);
                defs.appendChild(pattern);
                var rect = svg.rect({ id: canvas.id + "pattern_gridline", "x": diagram._hScrollOffset, "y": diagram._vScrollOffset, "width": "100%", "height": "100%", "fill": "url(#" + canvas.id + "pattern_grid)" });
                g.appendChild(rect);
                if (!grid)
                    parent.appendChild(g);
            }
            else {
                g = document.getElementById(canvas.id + "_grid");
                if (g) {
                    var grid = document.getElementById(canvas.id + "pattern_gridline");
                    if (grid && grid.parentNode)
                        grid.parentNode.removeChild(grid);
                }
            }
        },

        _updateBackground: function (hoffset, voffset, scale, diagram) {
            var attr = {
                x: hoffset,
                y: voffset
            };
            var bgClr = document.getElementById(diagram._id + "_canvas_backgroundColor");
            if (bgClr)
                ej.datavisualization.Diagram.Util.attr(bgClr, attr);
            var bgImg = document.getElementById(diagram._id + "_canvas_background");
            if (bgImg)
                ej.datavisualization.Diagram.Util.attr(bgImg, { "transform": "scale(" + scale + ")" });
        },

        _updateGrid: function (hoffset, voffset, scale, diagram) {
            var grid = document.getElementById(diagram._canvas.id + "pattern_gridline");
            var i;
            if (grid) {
                var pattern = document.getElementById(diagram._canvas.id + "pattern_grid");
                var height = 0;
                if (diagram.model.snapSettings.horizontalGridLines.linesInterval) {
                    for (i = 0; i < diagram.model.snapSettings.horizontalGridLines.linesInterval.length; i++) {
                        height += diagram.model.snapSettings.horizontalGridLines.linesInterval[i];
                    }
                }
                var width = 0;
                if (diagram.model.snapSettings.verticalGridLines.linesInterval) {
                    for (i = 0; i < diagram.model.snapSettings.verticalGridLines.linesInterval.length; i++) {
                        width += diagram.model.snapSettings.verticalGridLines.linesInterval[i];
                    }
                }

                var attr = {
                    x: hoffset,
                    y: voffset
                };
                ej.datavisualization.Diagram.Util.attr(grid, attr);

                scale = this._scaleSnapInterval(diagram.model, scale);
                var svg = diagram._svg;
                var defs = svg.defs({ "id": diagram._canvas.id + "patterndefinition" });
                if (pattern)
                    defs.removeChild(pattern);
                var attr = { id: diagram._canvas.id + "pattern_grid", x: 0, y: 0, width: width * scale, height: height * scale, patternUnits: "userSpaceOnUse" };
                pattern = diagram._svg.pattern(attr);
                this._renderHorizontalGridlines(scale, pattern, width, diagram.model, diagram._svg);
                this._renderVerticalGridlines(scale, pattern, height, diagram.model, diagram._svg);
                defs.appendChild(pattern);
            }
        },

        _scaleSnapInterval: function (model, scale) {
            if (scale >= 2) {
                while (scale >= 2) {
                    scale /= 2;
                }
            }
            else if (scale <= 0.5) {
                while (scale <= 0.5) {
                    scale *= 2;
                }
            }
            var i;
            if (scale !== 1) {
                model.snapSettings.horizontalGridLines._snapinterval = [];
                for (i = 0; i < model.snapSettings.horizontalGridLines.snapInterval.length; i++) {
                    model.snapSettings.horizontalGridLines._snapinterval[i] =
                        model.snapSettings.horizontalGridLines.snapInterval[i] * scale;
                }
                model.snapSettings.verticalGridLines._snapinterval = [];
                for (i = 0; i < model.snapSettings.verticalGridLines.snapInterval.length; i++) {
                    model.snapSettings.verticalGridLines._snapinterval[i] =
                        model.snapSettings.verticalGridLines.snapInterval[i] * scale;
                }
            }
            return scale;
        },

        _renderHorizontalGridlines: function (scale, pattern, width, model, svg) {
            var snapSettings = model.snapSettings;
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowHorizontalLines) {
                var space = 0;
                var cnt = 0;
                for (var i = 0; i < snapSettings.horizontalGridLines.linesInterval.length; i = i + 2) {
                    var thickness = snapSettings.horizontalGridLines.linesInterval[i];
                    var line = svg.line({
                        "x1": 0, "y1": space * scale, "x2": width * scale, "y2": space * scale, "stroke": snapSettings.horizontalGridLines.lineColor,
                        "stroke-width": thickness, "stroke-dasharray": snapSettings.verticalGridLines.lineDashArray
                    });
                    //svg.appendChild(line);
                    space += snapSettings.horizontalGridLines.linesInterval[i + 1] + thickness;
                    pattern.appendChild(line);
                    cnt++;
                }
            }
        },

        _renderVerticalGridlines: function (scale, pattern, height, model, svg) {
            var snapSettings = model.snapSettings;
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.ShowVerticalLines) {
                var space = 0;
                var cnt = 0;
                for (var i = 0; i < snapSettings.verticalGridLines.linesInterval.length; i = i + 2) {
                    var thickness = model.snapSettings.verticalGridLines.linesInterval[i];
                    var line = svg.line({
                        "x1": space * scale, "y1": 0, "x2": space * scale, "y2": height * scale, "stroke": snapSettings.verticalGridLines.lineColor,
                        "stroke-width": thickness, "stroke-dasharray": snapSettings.verticalGridLines.lineDashArray
                    });
                    space += snapSettings.verticalGridLines.linesInterval[i + 1] + thickness;
                    pattern.appendChild(line);
                    cnt++;
                }
            }
        },
        //#region Transformation
        transformView: function (diagram, hOffset, vOffset) {
            var view = diagram._view;
            var attr = {
                "transform": "translate(" + hOffset + "," + vOffset + ")"
            };
            ej.datavisualization.Diagram.Util.attr(view, attr);
            ej.datavisualization.Diagram.Util.attr(diagram._adornerLayer, attr);
            if (diagram._htmlLayer) {
                diagram._htmlLayer.style.left = hOffset + "px";
                diagram._htmlLayer.style.top = vOffset + "px";
                diagram._htmlLayer.style.position = "absolute";
            }
        },
        scaleContent: function (diagram, scale) {
            var diagramLayer = diagram._diagramLayer;
            ej.datavisualization.Diagram.Util.attr(diagramLayer, { "transform": "scale(" + scale + ")" });
            //ej.datavisualization.Diagram.Util.attr(diagram._adornerLayer, { "transform": "scale(" + scale + ")" });
            if (diagram._htmlLayer) {
                diagram._htmlLayer.style.webkitTransform = "scale(" + scale + ")";
                diagram._htmlLayer.style.MozTransform = "scale(" + scale + ")";
                diagram._htmlLayer.style.OTransform = "scale(" + scale + ")";
                diagram._htmlLayer.style.msTransform = "scale(" + scale + ")";
                diagram._htmlLayer.style.transform = "scale(" + scale + ")";
            }
            if (diagram.selectionList.length > 0)
                this.updateSelector(diagram.selectionList[0], diagram._adornerSvg, scale, diagram, diagram.model.selectedItems.constraints);
            if (diagram.selectionList[0]) {
                if ((diagram.model.selectedItems.userHandles && diagram.model.selectedItems.userHandles.length > 0)) {
                    var isMultipleSelection = false;
                    if (diagram.selectionList[0].type == "pseudoGroup")
                        isMultipleSelection = true;
                    if (diagram.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                        ej.datavisualization.Diagram.SvgContext.updateUserHandles(diagram.model.selectedItems.userHandles, diagram.selectionList[0], diagram._adornerSvg, isMultipleSelection, false, scale, diagram);
                }
            }
        },
        setSize: function (diagram, width, height) {
            ej.datavisualization.Diagram.Util.attr(diagram._svg.document, { width: width + "px", height: height + "px" });
            ej.datavisualization.Diagram.Util.attr(diagram._adornerSvg.document, { width: width + "px", height: height + "px" });
        },
        //#endregion

        _findIndex: function (diagram, node) {
            var lastNode, newIndex = 1, index, elements;
            if (node.parent)
                elements = $("#" + node.parent.name).find(">.node,>.connector,>.group");
            else
                elements = $("#DiagramContent_canvas_svg").find(">.node,>.connector,>.group");
            if (node.name != "helper") {
                lastNode = elements[elements.length - 1];
                if (lastNode && lastNode.id == "helper")
                    lastNode = elements[elements.length - 2];
                if (lastNode) {
                    index = 0;
                    if (lastNode.className.animVal == "ej-d-node" || lastNode.className.animVal == "ej-d-group") {
                        newIndex = index + 1;
                    }
                    else if (lastNode.className.animVal == "ej-d-connector") {
                        newIndex = index + 1;
                    }
                }
            }
            return newIndex;
        },
        //#region Render Node

        //#region render
        renderNode: function (node, svg, parent, palette, diagram) {
            var g;
            // if (node.visible) 
            {
                var g;

                var width = node.width ? node.width : node._width || 0;
                var height = node.height ? node.height : node._height || 0;
                var name = node.name;
                var offX = 0, offY = 0, x = 0, y = 0, rAngle = 0;
                var visible = node.visible ? "visible" : "hidden";
                if (!palette) {
                    x = node.offsetX - width * node.pivot.x;
                    y = node.offsetY - height * node.pivot.y;
                    offX = node.offsetX;
                    offY = node.offsetY;
                    rAngle = node.rotateAngle;
                }
                g = svg.g({
                    "id": name, "class": "ej-d-node", "transform": "rotate(" + rAngle + "," + offX + "," + offY +
                        "),translate(" + x + "," + y + ")", "visibility": visible
                });
                if (parent) {
                    parent.appendChild(g);
                }
                else {
                    svg.appendChild(g);
                }
                if (node._isDisabled)
                    ej.datavisualization.Diagram.Util.attr(g, { "style": "pointer-events:none;" });
                this._renderNode(node, svg, g, diagram, palette);
            }
            return g;
        },

        _renderNode: function (node, svg, g, diagram, palette) {
            this._renderShape(node, svg, g, diagram);
            this._renderLabels(node, svg, null, palette);
            this._renderPorts(node, svg, g);
        },

        _renderShadow: function (node, svg) {
            if (!node.segments) {

                var g = svg.document.getElementById(node.name);
                if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                    var fill = node.fillColor;
                    if (node.type != "text" && node.type != "image") {
                        var shape = svg.document.getElementById(node.name + "_shape");
                    }
                    else if (node.type === "image") {
                        var shape = svg.document.getElementById(node.name + "_backRect");
                    }
                    var shadow = shape.cloneNode(true);
                    var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                    var attr = { id: node.name + "_shadow", transform: "translate(" + point.x + "," + point.y + ")", "stroke": "lightgrey", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none", opacity: node.shadow.opacity };
                    g.insertBefore(shadow, shape);
                    ej.datavisualization.Diagram.Util.attr(shadow, attr);
                }
            }
        },

        _updateShadow: function (node, svg) {
            var g = svg.document.getElementById(node.name);
            var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
            var attr = { id: node.name + "_shadow", transform: "translate(" + point.x + "," + point.y + ")", opacity: node.shadow.opacity };
            svg.g(attr);

        },

        _removeShadow: function (node, svg) {
            if (node.type != "text") {
                var g = svg.document.getElementById(node.name);
                var shadow = svg.document.getElementById(node.name + "_shadow");
                if (shadow)
                    g.removeChild(shadow);
            }
        },

        _renderphase: function (node, svg, parent, diagram) {
            var g;
            g = svg.g({ "id": node.name, "class": "ej-d-seperator" });
            if (node.parent) {
                //parent = svg.getElementById(node.parent);
                //if (parent)
                parent.appendChild(g);
            } else {
                svg.appendChild(g);
            }
            var visibility = "visible";
            var tx = 0, ty = 0;
            var parNode = diagram.nameTable[node.parent];
            if (parNode) {
                var children = diagram._getChildren(parNode.children);
                if (children.length > 1)
                    var header = diagram.nameTable[children[0]];
                var bounds = ej.datavisualization.Diagram.Util.bounds(parNode);
                var points = [], nPoint;
                var top = bounds.top + 50;
                var left = bounds.left + 50;
                if (node.orientation == "vertical") {
                    points.push({ x: 0, y: 0 });
                    points.push({ x: bounds.width, y: 0 });
                    nPoint = this._convertToSVGPoints(points);
                    tx = bounds.x;
                    ty = bounds.y + node.offset;
                    if (header)
                        top += header.height;
                    if (!(ty < bounds.bottom && ty > top)) {
                        visibility = "hidden";
                    }
                } else {
                    points.push({ x: 0, y: header.height ? header.height : 0 });
                    points.push({ x: 0, y: bounds.height });
                    nPoint = this._convertToSVGPoints(points);
                    tx = bounds.x + node.offset;
                    ty = bounds.y;

                    if (!(tx < bounds.right && tx > left)) {
                        visibility = "hidden";
                    }
                }


                var attr = {
                    "id": node.name + "_phase",
                    "stroke": node.lineColor,
                    "stroke-width": node.lineWidth,
                    "stroke-dasharray": node.lineDashArray,
                    "points": nPoint,
                    //"transform": "translate(" + tx + "," + ty + ")",
                };
                var line = svg.polyline(attr);
                g.appendChild(line);

                attr = {
                    "id": node.name + "_phase_hitTest",
                    "class": "hitTest",
                    "stroke-width": 10,
                    "points": nPoint,
                    //"transform": "translate(" + tx + "," + ty + ")",
                    "stroke": "transparent",

                };
                line = svg.polyline(attr);
                g.appendChild(line);
                g.setAttribute("transform", "translate(" + tx + "," + ty + ")");
            }

        },

        renderphase: function (node, svg, parent, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                if (parent)
                    parent = panel.document.getElementById(parent.id)
                view.context._renderphase(node, panel, parent || view.diagramLayer, diagram);
            });
        },

        _renderGroupBackground: function (group, g, svg) {
            if (group.fillColor != "none") {
                var x = group.offsetX - group.width * group.pivot.x;
                var y = group.offsetY - group.height * group.pivot.y;
                var fillColor = group.fillColor, borderColor = group.borderColor;
                if (group.type == "bpmn") {
                    fillColor = "transparent";
                    borderColor = "transparent";
                }
                var angle = group.rotateAngle;
                var pt = new ej.datavisualization.Diagram.Point(x + group.width * group.pivot.x, y + group.height * group.pivot.y);
                var attr = {
                    "id": group.name + "_shape",
                    "rx": group.cornerRadius,
                    "ry": group.cornerRadius,
                    "width": Math.round((group.width < 0) ? 0 : group.width),
                    "height": Math.round((group.height < 0) ? 0 : group.height),
                    "fill": fillColor,
                    "stroke": borderColor,
                    "stroke-width": group.borderWidth,
                    "opacity": group.opacity,
                    "stroke-dasharray": group.borderDashArray,
                    "transform": "rotate(" + angle + "," + pt.x + "," + pt.y + "),translate(" + x + "," + y + ")",
                };
                if (group.gradient && group.type != "bpmn")
                    attr["fill"] = this._renderGradient(group.name, group.gradient, svg);
                this._addCssClass(group, attr);
                g.appendChild(svg.rect(attr));
            }
        },

        renderGroup: function (group, svg, parent, nameTable, diagram, isLoad, overView) {
            var g;
            var visible = group.visible ? "visible" : "hidden";
            //if (group.visible) 
            {
                g = svg.g({ "id": group.name, "class": "ej-d-group", "visibility": visible });
                if (parent) {
                    parent.appendChild(g);
                } else {
                    svg.appendChild(g);
                }
                if (group.parent == "" && group.container) {
                    if (isLoad) {
                        group.height = 0;
                        group.width = 0;
                    }
                    if (!overView)
                        this._alignOnContainer(group, diagram);
                }
                this._renderGroupBackground(group, g, svg);

                var children = group.children;
                if (children && children.length > 0) {
                    for (var i = 0, len = children.length; i < len; i++) {
                        var child = nameTable[ej.datavisualization.Diagram.Util.getChild(children[i])];
                        if (child) {
                            if (child._type === "group") {
                                this.renderGroup(child, svg, g, nameTable, diagram, isLoad, overView);
                                ej.datavisualization.Diagram.Util._updateGroupBounds(child, diagram, true);
                                ej.datavisualization.Diagram.DiagramContext.update(child, diagram);
                            }
                            else if (child.segments) {
                                ej.datavisualization.Diagram.Util.updateBridging(child, diagram);
                                this.renderConnector(child, svg, g, diagram);
                            }
                            else
                                this.renderNode(child, svg, g, undefined, diagram);
                        }
                    }
                }
                this._renderLabels(group, svg);
                this._renderPorts(group, svg, g);
            }
            if (!overView) {
                if (group.container && group.phases && group.phases.length > 0) {
                    diagram._needUpdate === undefined ? (diagram._needUpdate = true) : "";
                    diagram._updatePhase(group);
                }
            }
            else {
                if (group.container && group.phases && group.phases.length > 0) {
                    var overViewId = diagram._overview._id;
                    var view, panel, parent;
                    view = diagram._views[overViewId];
                    panel = view.svg || view._canvas;
                    parent = panel.document.getElementById(parent.id)
                    for (var i = 0; i < group.phases.length; i++) {
                        var node = diagram.nameTable[group.phases[i]];
                        if (node)
                            view.context._renderphase(node, panel, parent || view.diagramLayer, diagram);
                    }
                }

            }
            return g;
        },
        _alignOnContainer: function (group, diagram, stretch, isExeed, updateLane) {
            switch (group.container && group.container.type) {
                case "canvas":
                    var angle = group.rotateAngle;
                    //group.rotateAngle = 0;
                    if (!(group.rotateAngle == 0 && angle == 0)) {
                        if (group.parent == "")
                            group.rotateAngle = 0;
                        else
                            diagram._rotate(group, -angle, diagram.nameTable);
                    }

                    this._alignOnCanvasContainer1(group, diagram, stretch, isExeed, updateLane);
                    if (!((group.rotateAngle == 0) && (angle == 0)))
                        diagram._rotate(group, angle, diagram.nameTable);
                    break;
                case "stack":
                    this._alignOnStackContainer(group, diagram);
                    break;
            }
        },

        _removeFromContainer: function (node, svg) {
            var element = svg.getElementById(node.name);
            var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
            if (element)
                element.parentNode.removeChild(element);
            if (node._type == "group") {
                for (var i = 0; i < node.children.length; i++) {
                    var child = typeof node.children[i] == "object" ? node.children[i].name : node.children[i];
                    this._removeHtmlChildren(child, htmlLayer);
                }
            } else {
                this._removeHtmlChildren(node, htmlLayer);
            }
        },

        _removeHtmlChildren: function (node, htmlLayer) {
            if (htmlLayer) {
                var htmlElement = htmlLayer.children[node.name];
                if (htmlElement)
                    htmlLayer.removeChild(htmlElement);
                var htmlNode = htmlLayer.children[node.name + "_parentdiv"];
                if (htmlNode)
                    htmlLayer.removeChild(htmlNode);
            }
        },

        _childrenBounds: function (group, diagram) {
            var children = diagram._getChildren(group.children);
            var bounds = new ej.datavisualization.Diagram.Rectangle();
            bounds = ej.datavisualization.Diagram.Util.bounds(diagram.nameTable[children[0]]);
            for (var i = 1, len = children.length; i < len; i++) {
                var rect = ej.datavisualization.Diagram.Util.bounds(diagram.nameTable[children[i]], diagram);
                if (rect)
                    bounds = diagram._union(bounds, rect);
            }
            return bounds;
        },

        _alignOnCanvasContainer1: function (group, diagram, stretch, isExeed, updateLane) {
            var rSize = this._measureCanvasSize1(group, diagram, updateLane);
            var newSize = diagram._getMinMaxSize(group, rSize.width, rSize.height, stretch, isExeed);
            group.width = newSize.width;
            group.height = newSize.height;
            this._alignOnCanvas1(group, diagram);
        },
        _alignOnCanvas1: function (group, diagram, resize) {
            var child = null;
            var children = diagram._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                child = diagram.nameTable[children[i]];
                if (child) {
                    var pos = this._getCanvasPosition1(group, child, diagram, resize);
                    diagram._translate(child, pos.x, pos.y, diagram.nameTable);
                    diagram._updateQuad(child);
                }
            }
        },
        _getCanvasPosition1: function (group, node, diagram, resize) {
            var dx = null, dy = null;
            var groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            switch (node.horizontalAlign) {
                case "left":
                    dx = groupBounds.x - bounds.x + node.marginLeft + group.paddingLeft;
                    if (diagram.isPhaseResize)
                        dx = 0;
                    break;
                case "center":
                    dx = (groupBounds.x + groupBounds.width / 2) - (bounds.x + bounds.width / 2);
                    if (node.marginLeft)
                        dy = groupBounds.x - bounds.x + node.marginLeft + group.paddingLeft;
                    break;
                case "right":
                    dx = (groupBounds.x + groupBounds.width) - (bounds.x + bounds.width + group.paddingRight);
                    break;
                case "stretch":
                    var angle = node.rotateAngle;
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, -node.rotateAngle, diagram.nameTable);

                    var deltaWidth = (group.width - group.paddingLeft - group.paddingRight - node.marginLeft - node.marginRight) / node.width;
                    diagram.scale(node, deltaWidth, 0, group.pivot, diagram.nameTable);
                    groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                    bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                    dx = (groupBounds.x + group.paddingLeft) - (bounds.x) + node.marginLeft;
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, angle, diagram.nameTable);

                    if (node.container && node.container.type == "stack") {
                        var childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, diagram);
                        var stackWidth = diagram.nameTable[node.name].width;
                        //if (childBounds.width < stackWidth) {
                        var diffX = stackWidth - childBounds.width - group.paddingLeft - group.paddingRight;
                        var children = diagram._getChildren(node.children);
                        for (var i = 0; i < children.length; i++) {
                            var lastChild = diagram.nameTable[children[i]];
                            if (lastChild) {
                                var wid = lastChild.width;
                                lastChild.width += diffX;
                                lastChild.offsetX += diffX / 2;
                                lastChild.minWidth = lastChild.width;
                                // if (wid != lastChild.width) {
                                this._updateCanvasChildren(lastChild, diagram);
                                //}
                                if (resize && lastChild.container && lastChild.container.type == "canvas") {
                                    this._alignOnCanvas1(lastChild, diagram);
                                }
                            }
                        }
                        //}
                    }
                    break;
            }
            switch (node.verticalAlign) {
                case "top":
                    dy = groupBounds.y - bounds.y + node.marginTop + group.paddingTop;
                    if (diagram.isPhaseResize)
                        dy = 0;
                    break;
                case "center":
                    dy = (groupBounds.y + groupBounds.height / 2) - (bounds.y + bounds.height / 2) + node.marginTop;
                    if (node.marginTop)
                        dy = groupBounds.y - bounds.y + node.marginTop + group.paddingTop;
                    break;
                case "bottom":
                    dy = (groupBounds.y + groupBounds.height) - (bounds.y + bounds.height + group.paddingBottom);
                    break;
                case "stretch":
                    var angle = node.rotateAngle; var height
                    var nodeBounds123 = ej.datavisualization.Diagram.Util.bounds(node);
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, -node.rotateAngle, diagram.nameTable);
                    if (node.height == 0)
                        height = 1;
                    else height = node.height;
                    var deltaHeight = (group.height - group.paddingTop - group.paddingBottom - node.marginTop - node.marginBottom) / height;
                    diagram.scale(node, 0, deltaHeight, group.pivot, diagram.nameTable);
                    groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                    bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                    dy = (groupBounds.y + group.paddingTop) - (bounds.y) + node.marginTop;
                    if (!(node.rotateAngle == 0) && (angle == 0))
                        diagram._rotate(node, angle, diagram.nameTable);
                    if (node.container && node.container.type == "stack") {
                        childBounds = ej.datavisualization.Diagram.Util._getChildrenBounds(node, diagram);
                        var stackHeight = diagram.nameTable[node.name].height;
                        // if (childBounds.height < stackHeight) {
                        var diffY = stackHeight - childBounds.height - group.paddingTop - group.paddingBottom;
                        var children = diagram._getChildren(node.children);
                        lastChild = diagram.nameTable[children[children.length - 1]];
                        if (lastChild) {
                            var hei = lastChild.height;
                            lastChild.height += diffY;
                            lastChild.offsetY += diffY / 2;
                            lastChild.minHeight = lastChild.height;
                            if (hei != lastChild.height) {
                                this._updateCanvasChildren(lastChild, diagram);
                            }

                            if (resize && lastChild.container && lastChild.container.type == "canvas") {
                                this._alignOnCanvas1(lastChild, diagram);
                            }
                        }
                        // }
                    }
                    break;
            }
            return { x: dx, y: dy };
        },

        _updateCanvasChildren: function (group, diagram) {
            var children = diagram._getChildren(group.children), child;
            if (children && children.length > 0) {
                for (var i = 0; i < children.length; i++) {
                    child = diagram.nameTable[children[i]];
                    if (child) {
                        if (child.horizontalAlign == "stretch") {
                            var dx = group.width - child.width;
                            child.width += dx;
                            child.offsetX += dx / 2;
                        }
                        else if (child.verticalAlign == "stretch") {
                            var dy = group.height - child.height;
                            child.height += dy;
                            child.offsetY += dy / 2;
                        }
                    }
                }
            }
        },
        _measureCanvasSize1: function (group, diagram, updateLane) {
            var child = null;
            var bounds = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0), transX = 0, transY = 0;
            var rect = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0);
            var children = diagram._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                child = diagram.nameTable[children[i]];
                if (child && !child._isHeader) {
                    if (child._type != "group") {
                        var bounds1 = ej.datavisualization.Diagram.Util.bounds(child);
                        if (!bounds1.left < 0)
                            bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
                        else
                            bounds = bounds1;
                        bounds.y = 0;
                        bounds.x = 0;
                    } else {
                        this._alignOnContainer(child, diagram);
                        bounds1 = ej.datavisualization.Diagram.Util.bounds(child);
                        bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
                        bounds.y = 0;
                        bounds.x = 0;
                        if (child.rotateAngle == 0) {
                            bounds.width = bounds1.width;
                            bounds.height = bounds1.height;
                        }
                    }
                }
                if (child && !child._isHeader) {
                    if (child.container && child.container.type == "stack") {
                        bounds.width += child.marginLeft;
                        bounds.height += child.marginTop;
                    } else {
                        if (child && child.marginRight < 0) {
                            if (child.parent) {
                                var parent = diagram.nameTable[diagram._getChild(child.parent)];
                                if (parent && parent.isLane && parent.orientation != "vertical") {
                                    child.marginRight = 0;
                                }
                            }
                        }
                        bounds.width += child.marginLeft;
                        bounds.height += child.marginTop;
                        if (!updateLane || (updateLane && updateLane != child.parent)) {
                            bounds.width += child.marginRight;
                            bounds.height += child.marginBottom;
                        }
                    }
                }
                if (bounds) {
                    rect = diagram._union(bounds, rect);
                }
            }
            rect.width += group.paddingLeft + group.paddingRight;
            rect.height += group.paddingTop + group.paddingBottom;
            return rect;
        },

        _updateMargin: function (node, group) {
            if (group && group.container && group.container.type == "canvas") {
                var groupBounds = ej.datavisualization.Diagram.Util.bounds(group); //ej.datavisualization.Diagram.SvgContext.getCanvasBoundingBox(group, this.diagram, node, true);
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                node.marginLeft = bounds.x - (groupBounds.x + group.paddingLeft);
                node.marginRight = (groupBounds.x + group.width) - (bounds.x + bounds.width) - group.paddingRight;
                node.marginTop = bounds.y - (groupBounds.y + group.paddingTop);
                node.marginBottom = (groupBounds.y + group.height) - (bounds.y + bounds.height) - group.paddingBottom;
            }
        },
        _alignOnStackContainer: function (group, diagram) {

            var angle = group.rotateAngle;
            if (group.parent == "")
                group.rotateAngle = 0;
            else
                diagram._rotate(group, -angle, diagram.nameTable);
            var rSize = this._measureStackSize(group, diagram);
            var newSize = diagram._getMinMaxSize(group, rSize.width, rSize.height);
            group.width = newSize.width;
            group.height = newSize.height;// - group.marginTop;
            this._alignOnStack(group, diagram);
            diagram._rotate(group, angle, diagram.nameTable);
            diagram._updateAssociatedConnectorEnds(group, diagram.nameTable);
        },

        _alignOnStack: function (group, diagram) {
            var child = null, bounds, transX = 0, transY = 0;
            var children = diagram._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                child = diagram.nameTable[children[i]];
                if (child) {
                    var grAngle = child.rotateAngle;
                    if (!((child.rotateAngle == 0) && (grAngle == 0)))
                        diagram._rotate(child, -child.rotateAngle, diagram.nameTable);
                    var pos = this._getStackPosition(group, child, transX, transY, diagram);

                    diagram._translate(child, pos.x, pos.y, diagram.nameTable);
                    if (group.container.orientation == "vertical")
                        transY += (child.height ? child.height : child._height) + child.marginTop + child.marginBottom;
                    else
                        transX += (child.width ? child.width : child._width) + child.marginLeft + child.marginRight;
                    if (!((child.rotateAngle == 0) && (grAngle == 0)))
                        diagram._rotate(child, grAngle, diagram.nameTable);
                    diagram._updateQuad(child);
                }
            }
        },
        _getStackPosition: function (group, node, transX, transY, diagram) {
            var groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            var dx = null, dy = null;
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            if (group.container.orientation == "vertical") {
                dy = groupBounds.y - bounds.y + transY + node.marginTop + group.paddingTop;
                switch (node.horizontalAlign) {
                    case "left":
                        dx = groupBounds.x - bounds.x + node.marginLeft + group.paddingLeft;
                        break;
                    case "center":
                        dx = groupBounds.center.x - bounds.center.x;
                        break;
                    case "right":
                        dx = (groupBounds.x + groupBounds.width) - (bounds.x + bounds.width + group.paddingRight);
                        break;
                    case "stretch":
                        var deltaWidth = (group.width - group.paddingLeft - group.paddingRight) / width;
                        diagram.scale(node, deltaWidth, 1, group.pivot, diagram.nameTable);
                        groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                        bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                        dx = (groupBounds.x + group.paddingLeft) - (bounds.x);
                        break;
                }
            }
            else {
                dx = groupBounds.x - bounds.x + transX + node.marginLeft + group.paddingLeft;
                switch (node.verticalAlign) {
                    case "top":
                        dy = groupBounds.y - bounds.y + node.marginTop + group.paddingTop;
                        break;
                    case "center":
                        dy = groupBounds.center.y - bounds.center.y;
                        break;
                    case "bottom":
                        dy = (groupBounds.y + groupBounds.height) - (bounds.y + bounds.height + group.paddingBottom);
                        break;
                    case "stretch":
                        var deltaHeight = (group.height - group.paddingTop - group.paddingBottom) / height;
                        diagram.scale(node, 1, deltaHeight, group.pivot, diagram.nameTable);
                        groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                        bounds = ej.datavisualization.Diagram.Util.bounds(node, diagram);
                        dy = (groupBounds.y + group.paddingTop) - (bounds.y);
                        break;
                }
            }
            return { x: dx, y: dy }
        },
        _measureStackSize: function (group, diagram, stretch, isExeed, updateLane) {
            var child = null;
            var bounds = new ej.datavisualization.Diagram.Rectangle(), transX = 0, transY = 0;
            var rect = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0);
            var children = diagram._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                child = diagram.nameTable[children[i]];
                if (child) {
                    if (child._type != "group") {
                        bounds = ej.datavisualization.Diagram.Util.bounds(child);
                        bounds.y = 0;
                        bounds.x = 0;
                    } else {
                        bounds = this._measureContainerSize(child, diagram, stretch, isExeed, updateLane);
                        bounds.y = 0;
                        bounds.x = 0;
                    }

                    bounds.width += transX + child.marginLeft + child.marginRight;
                    bounds.height += transY + child.marginTop + child.marginBottom;
                }
                if (bounds) {
                    rect = diagram._union(bounds, rect);
                }
                if (group.container.orientation == "horizontal") {
                    transX = bounds.width;
                }
                else {
                    transY = bounds.height;
                }
            }
            rect.width += group.paddingLeft + group.paddingRight;
            rect.height += group.paddingTop + group.paddingBottom;
            return rect;
        },

        _measureContainerSize: function (node, diagram, stretch, isExeed, updateLane) {
            var bounds = null;
            if (node._type === "group" && node.container) {
                switch (node.container.type) {
                    case "canvas":
                        this._alignOnContainer(node, diagram, stretch, isExeed, updateLane);
                        return ej.datavisualization.Diagram.Util.bounds(node);
                        break;
                    case "stack":
                        return this._measureStackSize(node, diagram);
                        break;
                }
            }
        },

        _renderGradient: function (name, gradient, svg) {
            var fill;
            var id;
            if (gradient.type === "linear") {
                id = this._renderLinearGradient(name, gradient, svg);
                fill = "url(#" + id + ")";
            }
            else if (gradient.type === "radial") {
                id = this._renderRadialGradient(name, gradient, svg);
                fill = "url(#" + id + ")";
            }
            return fill;
        },

        _renderLinearGradient: function (name, gradient, svg) {
            var defs = svg.getElementsByTagName("defs")[0];
            if (defs) {
                var attr = { "id": name + "_gradient", "x1": gradient.x1 + "%", "y1": gradient.y1 + "%", "x2": gradient.x2 + "%", "y2": gradient.y2 + "%" };
                var linear = svg.linearGradient(attr);
                this._renderStops(gradient, svg, linear);
                defs.appendChild(linear);
                return linear.id;
            }
            return null;
        },

        _renderRadialGradient: function (name, gradient, svg) {
            var defs = svg.getElementsByTagName("defs")[0];
            if (defs) {
                var attr = {
                    "id": name + "_gradient", "cx": gradient.cx + "%", "cy": gradient.cy + "%",
                    "fx": gradient.fx + "%", "fy": gradient.fy + "%", "r": gradient.r + "%"
                };
                var radial = svg.radialGradient(attr);
                this._renderStops(gradient, svg, radial);
                defs.appendChild(radial);
                return radial.id;
            }
            return null;
        },

        _renderStops: function (gradient, svg, element) {
            if (svg.getElementById(element.getAttribute("id")) === element) {
                while (element.firstChild) {
                    element.removeChild(element.firstChild);
                }
            }
            var stops = gradient.stops;
            var stop;
            for (var i = 0, len = stops.length; i < len; i++) {
                stop = stops[i];
                element.appendChild(svg.stop({ "offset": stop.offset + "%", "stop-color": stop.color, "stop-opacity": stop.opacity }));
            }
        },

        _renderShape: function (node, svg, g, diagram) {
            var type = node._shape ? node._shape : node.type;
            switch (type) {
                case "rectangle":
                    this._renderRect(node, svg, g);
                    break;
                case "ellipse":
                    this._renderEllipse(node, svg, g);
                    break;
                case "image":
                    this._renderImage(node, svg, g);
                    break;
                case "path":
                    this._renderPath(node, svg, g);
                    break;
                case "polygon":
                    this._renderPolygon(node, svg, g);
                    break;
                case "text":
                    this._renderTextElement(node, svg, g);
                    break;
                case "html":
                    this._renderHtmlElement(node, svg, g, diagram);
                    break;
                case "native":
                    this._rendercontent(node, svg, g, diagram);
                    break;
            }
        },

        _renderHTMLTemplate: function (node, diagram) {
            if (node.templateId && $.templates) {
                return diagram._renderEjTemplate("#" + node.templateId, node);
            }
        },

        _renderSvgTemplate: function (node, svg, g, html) {
            var div = document.createElement('div');
            div.innerHTML = '<svg xmlns="http://www.w3.org/2000/svg"><g id="tempNative">' + html + '</g></svg>';
            document.body.appendChild(div);
            var content = document.getElementById("tempNative").cloneNode(true);
            content.id = node.name + "_shape";
            g.appendChild(content);
            var attr = {
                "id": node.name + "_shape",
                "fill": node.fillColor, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "opacity": node.opacity, "stroke-dasharray": node.borderDashArray
            };
            this._addCssClass(node, attr);
            if (node.gradient)
                attr["fill"] = this._renderGradient(node.name, node.gradient, svg);
            svg.g(attr);
            var bounds = content.getBBox();
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var delwidth = bounds.width ? width / bounds.width : 1;
            var delheight = bounds.height ? height / bounds.height : 1;
            content.setAttribute("transform", "scale(" + delwidth + "," + delheight + ")");
            content.setAttribute("id", node.name + "_shape");
            div.parentNode.removeChild(div);
        },



        _rendercontent: function (node, svg, g, diagram) {
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                var fill = node.fillColor;
                var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                var shadow = {
                    "id": node.name + "_shadow", "width": node.width, "height": node.height, "stroke": node.borderColor != "none" || node.borderColor != "transparent" ? "lightgrey" : "none", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
                    "transform": "translate(" + point.x + "," + point.y + ")", "opacity": node.shadow.opacity
                };
                g.appendChild(svg.rect(shadow));
            }

            if (!node.segments && node.templateId && $.templates) {
                var width = node.width;
                var height = node.height;
                if (node._type === "node" && !node.height && !node.width) {
                    node.width = node._width;
                    node.height = node._height;
                }
                var html = diagram._renderEjTemplate("#" + node.templateId, node);
                node.width = width;
                node.height = height;
                this._renderSvgTemplate(node, svg, g, html);
            }
        },

        _fill: function (node, svg) {
            var fill;
            if (node.gradient) {
                fill = this._renderGradient(node.name, node.gradient, svg);
            }
            if (!fill) {
                fill = node.fillColor;
            }
            return fill;
        },

        _addCssClass: function (object, attr) {
            if (object.cssClass) {
                if (attr) {
                    if (attr["class"])
                        attr["class"] = attr["class"] + " " + object.cssClass;
                    else
                        attr["class"] = object.cssClass;
                }
            }
            object._cssClass = object.cssClass;
            return attr;
        },

        _updateCssClass: function (object, attr) {
            if ((object._cssClass != "" || (object._cssClass === "" && object.cssClass !== "")) && object._cssClass !== undefined && object.cssClass != object._cssClass) {
                var classTmp = attr["class"];
                if (classTmp)
                    attr["class"] = classTmp.replace(object._cssClass, "").trim();
                return this._addCssClass(object, attr);
            }
            return attr;
        },

        _renderRect: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var attr = {
                "id": node.name + "_shape", "rx": node.cornerRadius, "ry": node.cornerRadius,
                "width": width, "height": height, "fill": fill, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "opacity": node.opacity, "stroke-dasharray": node.borderDashArray
            };
            this._addCssClass(node, attr);
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                var shadow = {
                    "id": node.name + "_shadow", "rx": node.cornerRadius, "ry": node.cornerRadius,
                    "width": node.width, "height": node.height, "stroke": node.borderColor != "none" || node.borderColor != "transparent" ? "lightgrey" : "none", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
                    "transform": "translate(" + point.x + "," + point.y + ")", "opacity": node.shadow.opacity
                };
                g.appendChild(svg.rect(shadow));
            }
            g.appendChild(svg.rect(attr));
        },

        _renderEllipse: function (node, svg, g) {
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.name + "_shape", "rx": width / 2, "ry": height / 2,
                "cx": width / 2, "cy": height / 2,
                "fill": fill, "stroke": node.borderColor, "stroke-width": node.borderWidth,
                "opacity": node.opacity, "stroke-dasharray": node.borderDashArray

            };
            this._addCssClass(node, attr);
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                var shadow = {
                    "id": node.name + "_shadow", "rx": node.width / 2, "ry": node.height / 2,
                    "cx": node.width / 2, "cy": node.height / 2, "stroke": node.borderColor != "none" || node.borderColor != "transparent" ? "lightgrey" : "none", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
                    "transform": "translate(" + point.x + "," + point.y + ")", "opacity": node.shadow.opacity
                };
                g.appendChild(svg.ellipse(shadow));
            }
            g.appendChild(svg.ellipse(attr));
        },

        _renderImage: function (node, svg, g) {
            var fill = node.fillColor;
            var backRect = this._renderBackgroundRect(node, svg, g);
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var attr = {
                "id": node.name + "_shape", "width": width, "height": height,
                "opacity": node.opacity, "preserveAspectRatio": "none",
            };
            var image = new Image();
            image.src = node.source;
            var shape = svg.image(attr);
            shape.setAttributeNS("http://www.w3.org/1999/xlink", "xlink:href", image.src);
            g.appendChild(shape);
        },

        _renderPolygon: function (node, svg, g) {
            var fill = this._fill(node, svg);
            ej.datavisualization.Diagram.Geometry.updatePolygonPoints(node);
            var points = this._convertToSVGPoints(node.points);
            var attr = {
                "id": node.name + "_shape", "fill": fill, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray, "points": points
            };
            this._addCssClass(node, attr);
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                var shadow = {
                    "id": node.name + "_shadow", "points": points, "stroke": node.borderColor != "none" || node.borderColor != "transparent" ? "lightgrey" : "none", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
                    "transform": "translate(" + point.x + "," + point.y + ")", "opacity": node.shadow.opacity
                };
                g.appendChild(svg.polygon(shadow));
            }
            g.appendChild(svg.polygon(attr));
        },

        _renderPath: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var d = ej.datavisualization.Diagram.Geometry.updatePath(0, 0, width, height, node.pathData, svg, null, node);
            node._absolutePath = d;
            var attr = {
                "id": node.name + "_shape", "width": width,
                "height": height, "d": d, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray, "stroke": node.borderColor,
                "stroke-width": node.borderWidth, "fill": fill

            };
            this._addCssClass(node, attr);
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                var shadow = {
                    "id": node.name + "_shadow", "width": node.width,
                    "height": node.height, "d": d, "stroke": node.borderColor != "none" || node.borderColor != "transparent" ? "lightgrey" : "none", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
                    "transform": "translate(" + point.x + "," + point.y + ")", "opacity": node.shadow.opacity
                };
                g.appendChild(svg.path(shadow));
            }
            g.appendChild(svg.path(attr));
        },

        _renderHtmlElement: function (node, svg, g, diagram) {
            var backRect = this._renderBackgroundRect(node, svg, g);
            if (!node.segments && node.templateId) {
                var width = node.width ? node.width : node._width;
                var height = node.height ? node.height : node._height;
                var x = node.offsetX - width * node.pivot.x;
                var y = node.offsetY - height * node.pivot.y;
                var div = document.createElement("div");
                var style = "width:" + width + "px;height:" + height + "px;padding:1px; left:" + x + "px; top:" + y + "px; ";
                if (ej.browserInfo().name === "msie")
                    style += "display: block; position: absolute; border:0px; pointer-events: all; opacity:" + node.opacity + ";";
                var attr = { "id": node.name + "_html", "class": "foreignObject", "style": style };
                this._addCssClass(node, attr);
                ej.datavisualization.Diagram.Util.attr(div, attr);
                if (!node.segments && node.templateId) {
                    var tmplString = this._renderHTMLTemplate(node, diagram)
                    div.innerHTML = tmplString;
                }
                if (ej.browserInfo().name === "msie") {
                    var parentdiv = document.createElement("div");
                    var attr1 = { "id": node.name + "_parentdiv", "class": "ej-d-node" };
                    ej.datavisualization.Diagram.Util.attr(parentdiv, attr1);
                    parentdiv.appendChild(div);
                    var visibility = node.visible ? "visible" : "hidden";
                    parentdiv.style.visibility = visibility;
                    var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                    if (htmlLayer)
                        htmlLayer.appendChild(parentdiv);
                    else
                        svg.document.parentNode.appendChild(div);
                }
                else {
                    attr = { "id": node.name + "_shape", "width": width, "height": height, "style": "opacity:" + node.opacity + ";" };
                    var shape = svg.foreignObject(attr);
                    shape.appendChild(div);
                    g.appendChild(shape);
                }
            }
        },

        _initializeGuidelines: function (svg, attr, parent) {
            var g = svg.g(attr);
            parent.appendChild(g);
            return g;
        },

        _renderSideAlignmentLines: function (start, end, g, svg, scale) {
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 1
            });
            g.appendChild(line);
        },

        _renderCenterAlignmentLines: function (start, end, g, svg, scale) {
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 1,
            });
            g.appendChild(line);
        },

        _renderSpacingLines: function (start, end, g, svg, scale) {
            var d, d1;
            var div1 = svg.g({});
            if (start.x == end.x) {
                d = "M" + (start.x * scale - 5) + " " + (start.y * scale + 5) + " L" + start.x * scale + " " + start.y * scale + " L" + (start.x * scale + 5) + " " + (start.y * scale + 5) + "z";
                d1 = "M" + (end.x * scale - 5) + " " + (end.y * scale - 5) + " L" + end.x * scale + " " + end.y * scale + " L" + (end.x * scale + 5) + " " + (end.y * scale - 5) + "z";
                div1.appendChild(svg.line({ "x1": start.x * scale - 8, "x2": start.x * scale + 8, "y1": start.y * scale - 1, "y2": start.y * scale - 1, "stroke": "#07EDE1", "stroke-width": 1 }));
                div1.appendChild(svg.line({ "x1": end.x * scale - 8, "x2": end.x * scale + 8, "y1": end.y * scale + 1, "y2": end.y * scale + 1, "stroke": "#07EDE1", "stroke-width": 1 }));
            }
            else {
                d = "M" + (start.x * scale + 5) + " " + (start.y * scale + 5) + " L" + start.x * scale + " " + start.y * scale + " L" + (start.x * scale + 5) + " " + (start.y * scale - 5) + "z";
                d1 = "M" + (end.x * scale - 5) + " " + (end.y * scale - 5) + " L" + end.x * scale + " " + end.y * scale + " L" + (end.x * scale - 5) + " " + (end.y * scale + 5) + "z";
                div1.appendChild(svg.line({ "x1": start.x * scale - 1, "x2": start.x * scale - 1, "y1": start.y * scale - 8, "y2": start.y * scale + 8, "stroke": "#07EDE1", "stroke-width": 1 }));
                div1.appendChild(svg.line({ "x1": end.x * scale + 1, "x2": end.x * scale + 1, "y1": end.y * scale - 8, "y2": end.y * scale + 8, "stroke": "#07EDE1", "stroke-width": 1 }));
            }
            div1.appendChild(svg.path({ "d": d, "fill": "#07EDE1" }));
            var line = svg.line({
                "x1": start.x * scale, "y1": start.y * scale,
                "x2": end.x * scale, "y2": end.y * scale, "stroke": "#07EDE1", "stroke-width": 0.6, "fill": "#07EDE1"
            });
            div1.appendChild(line);
            div1.appendChild(svg.path({ "d": d1, "fill": "#07EDE1" }));
            g.appendChild(div1);
        },

        _removeGuidelines: function (parent, g) {
            if (g != null) {
                parent.removeChild(g);
            }
        },

        //#endregion

        //#region update
        update: function (data, diagram) {
            var svg = diagram._svg;
            if (data._type === "group")
                this.updateGroup(data, svg, diagram);
            else if (data.segments)
                this.updateConnector(data, svg, diagram);
            else
                this.updateNode(data, svg, diagram);
        },

        _updateGoupBackground: function (group, svg) {
            var x = group.offsetX - group.width * group.pivot.x;
            var y = group.offsetY - group.height * group.pivot.y;
            var angle = group.rotateAngle;
            var pt = new ej.datavisualization.Diagram.Point(x + group.width * group.pivot.x, y + group.height * group.pivot.y);
            var fillColor = group.fillColor, borderColor = group.borderColor;
            if (group.type == "bpmn") {
                fillColor = "transparent";
                borderColor = "transparent";
            }
            var attr = {
                "id": group.name + "_shape",
                "rx": group.cornerRadius,
                "ry": group.cornerRadius,
                "width": Math.round((group.width < 0) ? 0 : group.width),
                "height": Math.round((group.height < 0) ? 0 : group.height),
                "fill": fillColor,
                "stroke": borderColor,
                "stroke-width": group.borderWidth,
                "opacity": group.opacity,
                "stroke-dasharray": group.borderDashArray,
                "transform": "rotate(" + angle + "," + pt.x + "," + pt.y + "),translate(" + x + "," + y + ")",
            };
            if (group.gradient && group.type != "bpmn")
                attr["fill"] = this._renderGradient(group.name, group.gradient, svg);
            var element = svg.getElementById(group.name + "_shape");
            if (element) {
                attr["class"] = element.getAttribute("class");
                this._updateCssClass(group, attr);
                ej.datavisualization.Diagram.Util.attr(element, attr);
            }
        },

        updateGroup: function (group, svg, diagram, layout) {
            var children = diagram._getChildren(group.children);
            var visible = group.visible ? "visible" : "hidden";
            var style = "display:block;"
            if (!(group.visible))
                style = "display:none;"


            svg.g({ "id": group.name, "visibility": "visible", "style": "display:block" });
            for (var i = 0, len = children.length; i < len; i++) {
                var child = diagram.nameTable[children[i]];
                if (child) {
                    if (child._type === "group")
                        this.updateGroup(child, svg, diagram, layout);
                    else if (child.segments) {
                        ej.datavisualization.Diagram.Util.updateBridging(child, diagram);
                        this.updateConnector(child, svg, diagram);
                    }
                    else
                        this.updateNode(child, svg, diagram, layout);
                }
            }
            svg.g({ "id": group.name, "visibility": visible, "style": style });
            this._updateGoupBackground(group, svg);

            this._updateAssociatedConnector(group, svg, diagram);
            this._updateLabels(group, svg, diagram);
            this._updatePorts(group, svg);
            if (group.isSwimlane && group.phases && group.phases.length > 0) {
                var des = null;
                for (var j = 0; j < group.phases.length; j++) {
                    des = diagram.nameTable[group.phases[j]];
                    if (des && des.parent == group.name)
                        this._updatephase(des, diagram);
                }
            }
        },

        _removephases: function (group, diagram) {
            //if (group.isSwimlane && diagram.model.phases && diagram.model.phases.length > 0) {
            //    var des = null;
            //    for (var j = diagram.model.phases.length - 1; j >= 0; j--) {
            //        des = diagram.model.phases[j];
            //        if (des) {
            //            var element = diagram._svg.getElementById(des.name + "_phase");
            //            if (element)
            //                element.parentNode.removeChild(element);
            //            ej.datavisualization.Diagram.Util.removeItem(diagram.model.phases, des);
            //        }
            //    }
            //}
            if (group.isSwimlane && group.phases) {
                for (var j = group.phases.length ; j >= 0; j--) {
                    var des = diagram.nameTable[group.phases[j]];
                    if (des) {
                        var element = diagram._svg.getElementById(des.name + "_phase");
                        if (element)
                            element.parentNode.removeChild(element);
                        ej.datavisualization.Diagram.Util.removeItem(diagram.model.phases, des);
                    }
                }
            }
        },
        _updatePhaseStyle: function (phase, options, diagram) {
            if (phase && options) {
                options.id = phase.name + "_phase";
                diagram._svg.g(options);
            }
        },

        _updatephase: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._updatephase1(node, diagram, panel);
            });
        },

        _updatephase1: function (node, diagram, svg) {
            svg = svg ? svg : diagram._svg;
            var parNode = diagram.nameTable[node.parent];
            if (parNode) {
                var tx = 0, ty = 0, transform = null;
                var points = [], nPoint;
                var bounds = ej.datavisualization.Diagram.Util.bounds(parNode);
                var children = diagram._getChildren(parNode.children);
                if (children.length > 1)
                    var header = diagram.nameTable[children[0]];
                var top = bounds.top + 50;
                var left = bounds.left + 50;
                var visibility = "visible";
                if (node.orientation == "vertical") {
                    tx = bounds.x;
                    ty = bounds.y + node.offset;
                    if (header)
                        top += header.height;
                    if (ty - .1 <= bounds.bottom && ty - .1 >= top) {
                        transform = "translate(" + tx + "," + ty + ")";
                    }
                    points.push({ x: 0, y: 0 });
                    points.push({ x: bounds.width, y: 0 });
                    nPoint = this._convertToSVGPoints(points);
                }
                else {
                    tx = bounds.x + node.offset;
                    ty = bounds.y;
                    if (tx - .1 <= bounds.right && tx - .1 > left) {
                        transform = "translate(" + tx + "," + ty + ")";
                    }
                    if (header) {
                        points.push({ x: 0, y: header.height });
                        points.push({ x: 0, y: bounds.height });
                    }
                    nPoint = this._convertToSVGPoints(points);
                }

                var attr = {
                    "points": nPoint,
                    "id": node.name + "_phase",
                };
                svg.polyline(attr);
                attr = {
                    "points": nPoint,
                    "id": node.name + "_phase_hitTest",
                };
                svg.polyline(attr);
                if (transform)
                    attr = {
                        "id": node.name,
                        "transform": transform,
                        //"visibility": visibility
                    }
                if (node.label) {
                    this._updatePhaseLabel(node, node.label, diagram)
                }
                svg.g(attr);
                if (svg.getElementById(attr.id))
                    $('#' + attr.id).insertAfter($('#' + attr.id)[0].parentNode.lastChild);
            }
        },
        _updatePhaseLabel: function (node, label, diagram) {
            var svg = diagram._svg;
            var bounds = diagram._getPhaseBounds(node);
            var text = svg.getElementById(node.name + "_" + label.name);
            if (text) {
                text.setAttribute("fill", label.fontColor);
                if (!label.text)
                    return;
                text.textContent = label.text;
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                this._wrapText(node, bounds, text, label, svg);
                this._alignTextOnPhase(node, bounds, text, label, svg);
            }
        },
        updateLabelStyle: function (node, label, svg) {
            var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
            var text = $(htmlLayer).find("#" + node.name + "_" + label.name)[0];
            if (node && text && label) {
                if (label.bold) text.style.fontWeight = "bold"; else text.style.fontWeight = "";
                if (label.italic) text.style.fontStyle = "italic"; else text.style.fontStyle = "";
                if (label.visible) text.style.visibility = ""; else text.style.visibility = "hidden";
                text.style.textDecoration = label.textDecoration;
                text.style.fontFamily = label.fontFamily;
                text.style.fontSize = label.fontSize + "px";
                text.style.color = label.fontColor;
                text.style.backgroundColor = label.fillColor;
                text.style.borderColor = label.borderColor;
                text.style.borderWidth = label.borderWidth + "px";
                text.style.borderStyle = "solid";
                var attr = this._updateCssClass(label, { "class": text.className });
                if (attr)
                    text.className = attr["class"];
            }
        },

        updatePort: function (node, port, svg) {
            var portshape = svg.getElementById(node.name + "_" + port.name);
            if (portshape) {
                if (portshape && portshape.parentNode)
                    portshape.parentNode.removeChild(portshape);
                var g = document.getElementById(node.name);
                portshape = ej.datavisualization.Diagram.SvgContext._renderPort(node, port, svg);
                g.appendChild(portshape);
            }

        },

        _updateNodeStyle: function (node, svg) {
            if (node && svg) {
                var attr = {};
                attr["opacity"] = node.opacity;
                attr["fill"] = node.fillColor;
                attr["stroke"] = node.borderColor;
                attr["stroke-width"] = node.borderWidth;
                attr["stroke-dasharray"] = node.borderDashArray;
                if (node.gradient)
                    attr["fill"] = this._renderGradient(node.name, node.gradient, svg);
                if (node._type == "node") {
                    var type = node.type == "text" ? "_backRect" : "_shape";
                    if (ej.browserInfo().name === "msie" && node.type == "html") {
                        //ej.datavisualization.Diagram.Util.attr(svg.getElementById(node.name + "_html"), attr);
                    } else {
                        var element = svg.getElementById(node.name + type);
                        if (element) {
                            attr["class"] = element.getAttribute("class");
                            this._updateCssClass(node, attr);
                            ej.datavisualization.Diagram.Util.attr(element, attr);
                        }
                    }
                    if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                        var shadow = svg.document.getElementById(node.name + "_shadow");
                        if (shadow)
                            ej.datavisualization.Diagram.Util.attr(shadow, { "fill": node.fillColor != "none" || node.fillColor != "transparent" ? "lightgrey" : "none" });
                    }
                }
            }
        },

        _updateConnectorStyle: function (connector, svg) {
            if (connector && svg) {
                var attr = {};
                attr["opacity"] = connector.opacity;
                attr["stroke"] = connector.lineColor;
                attr["stroke-width"] = connector.lineWidth;
                attr["stroke-dasharray"] = connector.lineDashArray;
                var element = svg.getElementById(connector.name + "_segments");
                if (element) {
                    attr["class"] = element.getAttribute("class");
                    this._updateCssClass(connector, attr);
                    ej.datavisualization.Diagram.Util.attr(element, attr);
                }

            }
        },

        updateTargetDecoratorStyle: function (connector, svg) {
            if (svg) {
                var attr = {};
                attr["stroke"] = connector.targetDecorator.borderColor;
                attr["fill"] = connector.targetDecorator.fillColor;
                var element = svg.getElementById(connector.name + "_targetDecorator");
                if (element) {
                    attr["class"] = element.getAttribute("class");
                    this._updateCssClass(connector.targetDecorator, attr);
                    ej.datavisualization.Diagram.Util.attr(element, attr);
                }
            }
        },

        updateSourceDecoratorStyle: function (connector, svg) {
            if (svg) {
                var attr = {};
                attr["stroke"] = connector.sourceDecorator.borderColor;
                attr["fill"] = connector.sourceDecorator.fillColor;
                var element = svg.getElementById(connector.name + "_sourceDecorator");
                if (element) {
                    attr["class"] = element.getAttribute("class");
                    this._updateCssClass(connector.sourceDecorator, attr);
                    ej.datavisualization.Diagram.Util.attr(element, attr);
                }
            }
        },

        updateNode: function (node, svg, diagram, layout) {
            if (svg) {
                this._updateLabels(node, svg, diagram);
                this._updateNode(node, svg);
                if (diagram._layoutInAction) return;
                this._updateAssociatedConnector(node, svg, diagram);
                if (!diagram.activeTool.inAction || diagram.activeTool.name !== "move" && diagram.activeTool.name !== "rotate") {
                    this._updatePorts(node, svg);
                }
            }
        },

        updateTextBlock: function (node, label, svg) {
            var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
            var parent = $(htmlLayer).children("#" + node.name)[0];
            var textContainter = $(parent).children("#" + node.name + "_shape_lblbg")[0];
            var text = textContainter.childNodes[0];
            if (node && text && label) {
                if (label.bold) text.style.fontWeight = "bold"; else text.style.fontWeight = "";
                if (label.italic) text.style.fontStyle = "italic"; else text.style.fontStyle = "";
                text.style.textDecoration = label.textDecoration;
                text.style.fontFamily = label.fontFamily;
                text.style.fontSize = label.fontSize + "px";
                text.style.color = label.fontColor;
            }
            this._updateTextElement(node, svg);
        },

        _updateAssociatedConnector: function (node, svg, diagram) {
            var i, len;
            for (i = 0, len = node.inEdges.length; i < len; i++) {
                if (diagram.nameTable[node.inEdges[i]]) {
                    if (diagram.nameTable[node.inEdges[i]] && ej.datavisualization.Diagram.Util.canBridge(diagram.nameTable[node.inEdges[i]], diagram)) {
                        ej.datavisualization.Diagram.Util.updateBridging(diagram.nameTable[node.inEdges[i]], diagram);
                        diagram._updateConnectorBridging(diagram.nameTable[node.inEdges[i]]);
                    }
                    ej.datavisualization.Diagram.DiagramContext.update(diagram.nameTable[node.inEdges[i]], diagram);
                }
            }
            for (i = 0, len = node.outEdges.length; i < len; i++) {
                if (diagram.nameTable[node.outEdges[i]]) {
                    if (diagram.nameTable[node.outEdges[i]] && ej.datavisualization.Diagram.Util.canBridge(diagram.nameTable[node.outEdges[i]], diagram)) {
                        ej.datavisualization.Diagram.Util.updateBridging(diagram.nameTable[node.outEdges[i]], diagram);
                        diagram._updateConnectorBridging(diagram.nameTable[node.outEdges[i]]);
                    }
                    ej.datavisualization.Diagram.DiagramContext.update(diagram.nameTable[node.outEdges[i]], diagram);
                }
            }
        },

        _updateNode: function (node, svg, layout) {
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var x = node.offsetX - width * node.pivot.x;
            var y = node.offsetY - height * node.pivot.y;
            var angle = node.rotateAngle;
            var pt = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
            var visible = node.visible ? "visible" : "hidden";
            var attr = ({
                "id": node.name, "transform": "rotate(" + angle + "," + pt.x + "," + pt.y + "),translate(" + x + "," + y + ")",
                "visibility": visible
            });
            svg.g(attr);
            var type = node._shape ? node._shape : node.type;
            switch (type) {
                case "rectangle":
                    svg.rect({
                        "id": node.name + "_shape", "width": (width < 0) ? 0 : width, "height": (height < 0) ? 0 : height,
                    });
                    if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)
                        svg.rect({
                            "id": node.name + "_shadow", "width": node.width, "height": node.height,
                        });
                    break;
                case "ellipse":
                    svg.ellipse({
                        "id": node.name + "_shape", "rx": width / 2, "ry": height / 2,
                        "cx": width / 2, "cy": height / 2
                    });
                    if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)
                        svg.ellipse({
                            "id": node.name + "_shadow", "rx": node.width / 2, "ry": node.height / 2,
                            "cx": node.width / 2, "cy": node.height / 2
                        });
                    break;
                case "image":
                    this._updateBackgroundRect(node, svg);
                    svg.image({
                        "id": node.name + "_shape", "width": width, "height": height,
                    });
                    if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)
                        svg.rect({
                            "id": node.name + "_shadow", "width": node.width, "height": node.height,
                        });
                    break;
                case "path":
                    var d;
                    if (node._scaled || !node._absolutePath) {
                        d = ej.datavisualization.Diagram.Geometry.updatePath(0, 0, width, height, node.pathData, svg, node._absoluteBounds);
                        node._absolutePath = d;
                        node._scaled = false;
                    }
                    else d = node._absolutePath;
                    svg.path({
                        "id": node.name + "_shape", "d": d,
                    });
                    if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)
                        svg.path({
                            "id": node.name + "_shadow", "d": d,
                        });
                    break;
                case "polygon":
                    this._updatePolygon(node, svg);
                    break;
                case "text":
                    this._updateTextElement(node, svg);
                    break;
                case "html":
                    this._updateHtmlElement(node, svg);
                    if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)
                        svg.rect({
                            "id": node.name + "_shadow", "width": node.width, "height": node.height
                        });
                    break;
                case "native":
                    if (!$("#" + node.name + "_shape")[0]) {
                        var bounds = { width: width, height: height };
                    } else {
                        var bounds = $("#" + node.name + "_shape")[0].getBBox();
                    }
                    var delwidth = bounds.width ? width / bounds.width : 1;
                    var delheight = bounds.height ? height / bounds.height : 1;
                    svg.g({
                        "id": node.name + "_shape", "width": width, "height": height,
                        "transform": "scale(" + delwidth + "," + delheight + ")"
                    });
                    if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)
                        svg.rect({
                            "id": node.name + "_shadow", "width": node.width, "height": node.height
                        });
                    break;
            }
        },

        _updateHtmlElement: function (node, svg) {
            this._updateBackgroundRect(node, svg);
            var visibility = node.visible ? "visible" : "hidden";
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var x = node.offsetX - width * node.pivot.x;
            var y = node.offsetY - height * node.pivot.y;
            var div;
            if (ej.browserInfo().name === "msie") {
                var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                var parentDiv = htmlLayer.children[node.name + "_parentdiv"];
                if (parentDiv) {
                    div = parentDiv.children[node.name + "_html"];
                    if (div) {
                        div.style.left = x + "px";
                        div.style.top = y + "px";
                        div.style.webkitTransform = "rotate(" + node.rotateAngle + "deg)";
                        div.style.MozTransform = "rotate(" + node.rotateAngle + "deg)";
                        div.style.OTransform = "rotate(" + node.rotateAngle + "deg)";
                        div.style.msTransform = "rotate(" + node.rotateAngle + "deg)";
                        div.style.transform = "rotate(" + node.rotateAngle + "deg)";

                        div.style.visibility = visibility;
                    }
                }
            }
            else {
                div = svg.getElementById(node.name + "_html");
                if (div) {
                    var style = "width:" + width + "px;height:" + height + "px;padding:1px; left:" + x + "px; top:" + y + "px; ";
                    ej.datavisualization.Diagram.Util.attr(div, { "style": style });
                    svg.foreignObject({
                        "id": node.name + "_shape", "width": width, "height": height
                    });
                }
            }
            if (div) {
                div.style.width = node.width.toString() + "px";
                div.style.height = node.height.toString() + "px";
            }
        },

        _updatePolygon: function (node, svg) {
            ej.datavisualization.Diagram.Geometry.updatePolygonPoints(node);
            svg.polygon({
                "id": node.name + "_shape", "points": this._convertToSVGPoints(node.points)
            });
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow)
                svg.polygon({
                    "id": node.name + "_shadow", "points": this._convertToSVGPoints(node.points)
                });
        },

        _convertToSVGPoints: function (points) {
            var pts = "";
            for (var i = 0, len = points.length; i < len; i++) {
                pts += points[i].x + "," + points[i].y + " ";
            }
            return pts.trim();
        },

        _renderTextElement: function (node, svg, g) {
            var textElement = node.textBlock;
            if (textElement) {
                var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                this._renderBackgroundRect(node, svg, g);
                var container = document.createElement("div");
                var text = document.createElement("span");
                ej.datavisualization.Diagram.Util.attr(text, { "id": node.name, "class": "ej-d-label", "style": "display: inline-block; position: absolute; pointer-events: all; line-height: normal; alignment-baseline:middle" });
                if (textElement.bold) text.style.fontWeight = "bold";
                if (textElement.italic) text.style.fontStyle = "italic";
                text.style.textDecoration = textElement.textDecoration;
                text.style.fontFamily = textElement.fontFamily;
                text.style.fontSize = textElement.fontSize + "px";
                text.style.color = textElement.fontColor;
                text.textContent = textElement.text;
                var style = "display: inline-block; position: absolute; width: inherit; height: inherit; pointer-events: none; ";
                ej.datavisualization.Diagram.Util.attr(container, { "id": node.name + "_shape_lblbg", "style": style });
                container.appendChild(text);

                var element = $(htmlLayer).children("#" + node.name)[0];
                if (element) htmlLayer.removeChild(element);
                element = document.createElement("div");
                var width = node.width ? node.width : node._width;
                var height = node.height ? node.height : node._height;
                var x = node.offsetX - width * node.pivot.x;
                var y = node.offsetY - height * node.pivot.y;
                var style = "width:" + width + "px; height:" + height + "px; left:" + x + "px; top:" + y + "px; display: block; position: absolute; pointer-events: none; ";
                var class1 = "ej-d-node";
                ej.datavisualization.Diagram.Util.attr(element, { "id": node.name, "class": class1, "style": style });
                element.style.webkitTransform = "rotate(" + node.rotateAngle + "deg) ";
                element.style.MozTransform = "rotate(" + node.rotateAngle + "deg) ";
                element.style.OTransform = "rotate(" + node.rotateAngle + "deg) ";
                element.style.msTransform = "rotate(" + node.rotateAngle + "deg)";
                element.style.transform = "rotate(" + node.rotateAngle + "deg) ";
                element.appendChild(container);

                if (htmlLayer)
                    htmlLayer.appendChild(element);
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                this._alignTextOnLabel(node, bounds, text, container, node.textBlock);
                //this._renderShadow(node, svg, g);
            }
        },

        _updateTextElement: function (node, svg) {
            this._updateBackgroundRect(node, svg);
            var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
            var parent = this._findChild(htmlLayer.childNodes, node);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            var width, height, x, y;
            if (parent) {
                width = bounds.width; height = bounds.height;
                x = node.offsetX - node.width * node.pivot.x; y = node.offsetY - node.height * node.pivot.y;
                parent.style.width = width + "px";
                parent.style.height = height + "px";
                parent.style.left = x + "px";
                parent.style.top = y + "px";
                parent.style.webkitTransform = "rotate(" + node.rotateAngle + "deg)";
                parent.style.MozTransform = "rotate(" + node.rotateAngle + "deg)";
                parent.style.OTransform = "rotate(" + node.rotateAngle + "deg)";
                parent.style.msTransform = "rotate(" + node.rotateAngle + "deg)";
                parent.style.transform = "rotate(" + node.rotateAngle + "deg)";

            }
            var textContainer = $(parent).children("#" + node.name + "_shape_lblbg")[0];
            if (textContainer) {
                var text = textContainer.childNodes[0];
                text.textContent = node.textBlock.text;
                this._alignTextOnLabel(node, bounds, text, textContainer, node.textBlock);
            }
        },

        _renderBackgroundRect: function (node, svg, g) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.name + "_backRect", "class": "backrect", "x": 0, "y": 0,
                "width": node.width, "height": node.height, "fill": fill,
                "stroke": node.borderColor, "stroke-width": node.borderWidth, "opacity": node.opacity,
                "stroke-dasharray": node.borderDashArray
            };
            this._addCssClass(node, attr);
            if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                var point = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                var shadow = {
                    "id": node.name + "_shadow", "class": "backrect", "width": node.width, "height": node.height, "stroke": "lightgrey", "fill": fill != "none" || fill != "transparent" ? "lightgrey" : "none",
                    "transform": "translate(" + point.x + "," + point.y + ")", "opacity": node.shadow.opacity
                };
                g.appendChild(svg.rect(shadow));
            }
            g.appendChild(svg.rect(attr));

        },

        _updateBackgroundRect: function (node, svg) {
            var fill = this._fill(node, svg);
            var attr = {
                "id": node.name + "_backRect", "x": 0, "y": 0,
                "width": node.width, "height": node.height,
                "fill": fill, "stroke": node.borderColor, "stroke-width": node.borderWidth,
                "opacity": node.opacity, "stroke-dasharray": node.borderDashArray
            };
            svg.rect(attr);
        },

        _hideNode: function (node, svg) {
            var attr = { "id": node.name, "style": "pointer-events:none;" };
            svg.rect(attr);
        },

        _showNode: function (node, svg) {
            var attr = { "id": node.name, "style": "pointer-events:auto;" };
            svg.rect(attr);
        },
        _removePhasehelper: function (diagram) {
            if (diagram._adornerSvg.getElementById("phase_helper"))
                diagram._adornerSvg.removeChild(diagram._adornerSvg.getElementById("phase_helper"), diagram._adornerLayer);
        },
        _updatePhaseHelper: function (diagram, phase, bounds) {
            if (phase) {
                var svg = diagram._adornerSvg, scale = diagram._currZoom, attr;
                bounds.width = bounds.width < 0 ? 1 : bounds.width;
                bounds.height = bounds.height < 0 ? 1 : bounds.height;
                attr = {
                    "id": "phase_helper", "class": "helper", "pointer-events": "none", "width": bounds.width * scale, "height": bounds.height * scale
                };
                diagram._adornerSvg.rect(attr);
            }
        },
        _phasehelper: function (diagram, phase, bounds) {
            if (phase) {
                var svg = diagram._adornerSvg, scale = diagram._currZoom;
                var offsetX = (bounds.x + bounds.width / 2);
                var offsetY = (bounds.y + bounds.height / 2);
                var rect = svg.rect({
                    "id": "phase_helper", "class": "helper",
                    "stroke-width": 2, "stroke-dasharray": "3,3", "pointer-events": "none",
                    "width": bounds.width * scale, "height": bounds.height * scale, "fill": "transparent", "stroke": "green",
                    "transform": "translate(" + (offsetX - bounds.width / 2) * scale + "," + (offsetY - bounds.height / 2) * scale + "),rotate(" + 0 + "," + (bounds.width / 2) * scale + "," + (bounds.height / 2) * scale + ")"
                });
                diagram._adornerLayer.appendChild(rect);
            }
        },
        _drawContainerHelper: function (diagram) {
            var shape = diagram.activeTool.helper;
            var svg = diagram._adornerSvg;
            var scale = diagram._currZoom;
            if (shape)
                var bounds = ej.datavisualization.Diagram.Util.bounds(shape, true);
            var height = bounds.height;
            var width = bounds.width;
            var pivot;
            if (shape.pivot) pivot = { x: bounds.width * shape.pivot.x, y: bounds.height * shape.pivot.y };
            else pivot = { x: width / 2, y: height / 2 };
            var rotateAngle = shape.rotateAngle ? shape.rotateAngle : 0;
            var rect = svg.rect({
                "id": "helper", "class": "helper",
                "stroke-width": 1, "stroke-dasharray": "3,3", "pointer-events": "none",
                "width": width * scale, "height": height * scale, "fill": "transparent", "stroke": "red",
                "transform": "translate(" + (bounds.x) * scale + "," + (bounds.y) * scale + "),rotate(" + rotateAngle + "," + (pivot.x) * scale + "," + (pivot.y) * scale + ")"
            });
            diagram._adornerLayer.appendChild(rect);
        },

        _updateContainerHelper: function (diagram) {
            var attr;
            var shape = diagram.activeTool.helper;
            var scale = diagram._currZoom;
            if (shape)
                var bounds = ej.datavisualization.Diagram.Util.bounds(shape, true);
            var height = bounds.height;
            var width = bounds.width;
            var pivot;
            if (shape.pivot) pivot = { x: bounds.width * shape.pivot.x, y: bounds.height * shape.pivot.y };
            else pivot = { x: width / 2, y: height / 2 };
            var rotateAngle = shape.rotateAngle ? shape.rotateAngle : 0;
            attr = {
                "id": "helper", "class": "helper",
                "width": width * scale, "height": height * scale, "fill": "transparent", "stroke": "red",
                "transform": "translate(" + (bounds.x) * scale + "," + (bounds.y) * scale + "),rotate(" + rotateAngle + "," + (pivot.x) * scale + "," + (pivot.y) * scale + ")"
            };
            diagram._adornerSvg.rect(attr);
        },

        _removeContainerHelper: function (node, svg, parent) {
            if (svg.getElementById("helper"))
                svg.removeChild(svg.getElementById("helper"), parent);
        },
        //#endregion

        setNodeShape: function (node, svg, parent, diagram) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.name);
                g = $(parentg).find("#" + node.name)[0];
            }
            else {
                g = svg.getElementById(node.name);
            }
            if (g) {
                $(g).empty();
                this._renderNode(node, svg, g, diagram);
            }
        },

        setLine: function (connector, svg, parent, diagram) {
            var g;
            if (parent) {
                var parentg = svg.getElementById(parent.name);
                g = $(parentg).find("#" + connector.name)[0];
            }
            else {
                g = svg.getElementById(connector.name);
            }
            $(g).empty();

            this.renderConnector(connector, svg, g, diagram);
        },

        addNodeLabel: function (node, label, svg, parent, diagram) {
            this._renderLabels(node, svg);
        },
        //#endregion

        //#region Render Connector
        renderConnector: function (connector, svg, parent, diagram) {
            var g;
            var visible = connector.visible ? "visible" : "hidden";
            g = svg.g({ "id": connector.name, "class": "ej-d-connector", "visibility": visible });
            if (parent) {
                if (parent.id !== g.id) {
                    parent.appendChild(g);
                }
            }
            else {
                svg.appendChild(g);
            }
            var seg = svg.g({ "id": connector.name + "segments" });
            this._renderSegments(connector, svg, seg, diagram);
            g.appendChild(seg);
            this._renderLabels(connector, svg, diagram);
            this._renderDecorators(connector, svg, g, diagram);
            return g;
        },

        _renderSegments: function (connector, svg, g, diagram) {
            var visibility = connector.visible && ((diagram ? diagram._browserInfo.name : ej.browserInfo().name) === "msie") ? "collapse" : "hidden";
            var path = this._findPath(connector, diagram);
            var attr = {
                "id": connector.name + "_hitTest", "class": "hitTest", "stroke-width": connector.lineHitPadding, "d": path,
                "stroke": "transparent", "visibility": "hidden", "pointer-events": connector.visible ? "stroke" : "none"
            };
            var line = svg.path(attr);
            g.appendChild(line);
            attr = {
                "id": connector.name + "_segments",
                "d": path, "fill": "none", "stroke": connector.lineColor,
                "stroke-width": connector.lineWidth, "stroke-dasharray": connector.lineDashArray, "opacity": connector.opacity
            };
            this._addCssClass(connector, attr);
            line = svg.path(attr);
            g.appendChild(line);
        },

        _findPath: function (connector, diagram) {
            var st, end, path;
            for (var i = 0; i < connector.segments.length; i++) {
                var seg = connector.segments[i];
                if (seg._bridges.length > 0) {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        bridge._rendered = false;
                    }
                }
                var points = seg.points;
                if (i == 0) {
                    if (diagram && connector.sourceNode) var node = diagram._findNode(connector.sourceNode);
                    points = this._clipDecorators(connector, seg, true, node);
                    if (seg.type == "bezier" && connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                        points[0] = ej.datavisualization.Diagram.Util._adjustPoint(points[0], seg._point1, true, connector.lineWidth);
                    }
                    path = "M" + points[0].x + " " + points[0].y;
                }
                if (i == connector.segments.length - 1) {
                    if (diagram && connector.targetNode) var node = diagram._findNode(connector.targetNode);
                    points = this._clipDecorators(connector, seg, false, node);
                }
                if (seg.type != "bezier") {
                    if (connector.cornerRadius > 0) {
                        for (var j = 0; j < points.length - 1; j++) {
                            var segLength = ej.datavisualization.Diagram.Geometry.distance(points[j], points[j + 1]);
                            if (segLength > 0) {
                                if (i < connector.segments.length - 1 || j < points.length - 2) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        end = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], false, segLength / 2);
                                    }
                                    else end = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], false, connector.cornerRadius);
                                }
                                else end = points[j + 1];

                                if (i > 0 || j > 0) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        st = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], true, segLength / 2);
                                        if (i < connector.segments.length - 1 || j < points.length - 2)
                                            end = null;
                                    }
                                    else st = ej.datavisualization.Diagram.Util._adjustPoint(points[j], points[j + 1], true, connector.cornerRadius);
                                }

                                if (st) {
                                    path += " Q" + points[j].x + " " + points[j].y + " " + st.x + " " + st.y;
                                }
                                if (end) {
                                    if (seg._bridges.length > 0) {
                                        path = this._updateBridging(seg, path, j);
                                        if (seg.type === "orthogonal")
                                            path = this._updateBridging(seg, path, j + 1);
                                    }
                                    path += " L" + end.x + " " + end.y;
                                }
                            }
                        }
                    }
                    else {
                        for (var j = 0; j < points.length; j++) {
                            if (j > 0) {
                                path = this._updateBridging(seg, path, j);
                                path += " L" + points[j].x + " " + points[j].y;
                            }
                        }
                    }
                }
                else {
                    var endPoint = seg._endPoint;
                    if (connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                        endPoint = ej.datavisualization.Diagram.Util._adjustPoint(seg._endPoint, seg._point2, true, connector.lineWidth);
                    }
                    if (connector.targetNode && diagram) {
                        var targetNode = diagram._findNode(connector.targetNode);
                        if (node.borderColor != "none")
                            endPoint = ej.datavisualization.Diagram.Util._adjustPoint(endPoint, seg._point2, true, targetNode.borderWidth / 2);
                    }
                    path += " C" + seg._point1.x + " " + seg._point1.y + " " + seg._point2.x + " " + seg._point2.y + " " + endPoint.x + " " + endPoint.y;
                }
            }
            return path;
        },

        _updateBridging: function (seg, path, pointIndex) {
            var pathData = path;
            if (seg._bridges.length > 0) {
                if (seg.type === "straight") {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (!bridge._rendered) {
                            pathData += " L" + bridge.startPoint.x + " " + bridge.startPoint.y;
                            pathData += bridge.path;
                        }
                    }
                }
                else if (seg.type === "orthogonal") {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (bridge.segmentPointIndex === pointIndex && !bridge._rendered) {
                            pathData += " L" + bridge.startPoint.x + " " + bridge.startPoint.y;
                            pathData += bridge.path;
                            bridge._rendered = true;
                        }
                    }
                }
            }
            return pathData;
        },

        _refreshSegments: function (connector, svg, diagram) {
            this._updateConnector(connector, svg, diagram);
            this._updateDecorators(connector, svg, diagram);
        },

        _clipDecorators: function (connector, segment, source, node) {
            var points = [];
            if (!source && connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var start, end;
                for (var i = 0; i < segment.points.length; i++) {
                    points[i] = segment.points[i];
                }
                start = points[points.length - 1];
                end = points[points.length - 2];
                var len = ej.datavisualization.Diagram.Geometry.distance(start, end);
                len = (len == 0) ? 1 : len;
                var width = 2 * connector.lineWidth;
                var newPoint = ej.datavisualization.Diagram.Point();
                newPoint.x = start.x + width * (end.x - start.x) / len;
                newPoint.y = start.y + width * (end.y - start.y) / len;
                if (node && node.borderColor != "none")
                    newPoint = ej.datavisualization.Diagram.Util._adjustPoint(newPoint, end, true, node.borderWidth / 2);
                points[points.length - 1] = newPoint;
            }
            else if (source && connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                for (var i = 0; i < segment.points.length; i++) {
                    points[i] = segment.points[i];
                }
                start = points[0];
                end = points[1];
                var len = ej.datavisualization.Diagram.Geometry.distance(start, end);
                len = (len == 0) ? 1 : len;
                var width = 2 * connector.lineWidth;
                var newPoint = ej.datavisualization.Diagram.Point();
                newPoint.x = start.x + width * (end.x - start.x) / len;
                newPoint.y = start.y + width * (end.y - start.y) / len;
                if (segment.type == "bezier") {
                    newPoint.x = start.x + (segment._point1.x - start.x) / len;
                    newPoint.y = start.y + (segment._point1.y - start.y) / len;
                }
                if (node && node.borderColor != "none")
                    newPoint = ej.datavisualization.Diagram.Util._adjustPoint(newPoint, segment.type != "bezier" ? end : segment._point1, true, node.borderWidth / 2);
                points[0] = newPoint;
            }
            else if (source) {
                for (var i = 0; i < segment.points.length; i++) {
                    points[i] = segment.points[i];
                }
                if (node && node.borderColor != "none")
                    points[0] = ej.datavisualization.Diagram.Util._adjustPoint(points[0], points[1], true, node.borderWidth / 2);
            }
            else {
                for (var i = 0; i < segment.points.length; i++) {
                    points[i] = segment.points[i];
                }
                if (node && node.borderColor != "none")
                    points[points.length - 1] = ej.datavisualization.Diagram.Util._adjustPoint(points[points.length - 1], points[points.length - 2], true, node.borderWidth / 2);
            }
            return points;
        },

        _updateConnector: function (connector, svg, diagram) {
            var path = this._findPath(connector, diagram);
            svg.path({ "id": connector.name + "_segments", "d": path });
            svg.path({
                "id": connector.name + "_hitTest", "d": path, "pointer-events": (connector.isPhase ||
                    (diagram._selectedSymbol && diagram.selectionList[0] && diagram.selectionList[0].name == connector.name))
                    ? "none" : (connector.visible ? "stroke" : "none")
            });
        },

        updateConnector: function (connector, svg, diagram) {
            //var svg = diagram._svg;
            if (svg) {
                var visible = connector.visible ? "visible" : "hidden";
                svg.g({ "id": connector.name, "visibility": visible });
                this._updateConnector(connector, svg, diagram);
                this._updateLabels(connector, svg, diagram);
                this._updateDecorators(connector, svg, diagram);
            }
        },

        renderDecorators: function (connector, svg, diagram) {
            var g = svg.getElementById(connector.name);
            this._renderDecorators(connector, svg, g, diagram);
        },

        clearDecorators: function (connector, svg, diagram) {
            this._updateConnector(connector, svg, diagram);
            var g = svg.getElementById(connector.name);
            var dec = svg.getElementById(connector.name + "_targetDecorator");
            if (dec)
                svg.removeChild(dec, g);
            dec = svg.getElementById(connector.name + "_sourceDecorator");
            if (dec)
                svg.removeChild(dec, g);
        },
        //#endregion

        //#region Render Decorator
        _renderDecorators: function (connector, svg, g, diagram) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var segment = connector.segments[connector.segments.length - 1];
                startPoint = segment.points[segment.points.length - 2];
                endPoint = connector.targetPoint;
                if (segment.type == "bezier") {
                    startPoint = segment._point2;
                }
                if (connector.targetNode && diagram) {
                    var targetNode = diagram._findNode(connector.targetNode);
                    if (targetNode && targetNode.borderColor != "none")
                        endPoint = ej.datavisualization.Diagram.Util._adjustPoint(endPoint, startPoint, true, targetNode.borderWidth / 2);
                }
                this._renderDecorator(connector.name + "_targetDecorator", endPoint,
                    startPoint, connector, connector.targetDecorator, svg, g);
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var segment = connector.segments[0];
                startPoint = connector.sourcePoint;
                endPoint = segment.points[1];
                if (segment.type == "bezier") {
                    endPoint = segment._point1;
                }
                if (connector.sourceNode && diagram) {
                    var sourceNode = diagram._findNode(connector.sourceNode);
                    if (sourceNode && sourceNode.borderColor != "none")
                        startPoint = ej.datavisualization.Diagram.Util._adjustPoint(startPoint, endPoint, true, sourceNode.borderWidth / 2);
                }
                this._renderDecorator(connector.name + "_sourceDecorator", startPoint,
                    endPoint, connector, connector.sourceDecorator, svg, g);
            }
        },

        _renderDecorator: function (name, point1, point2, connector, decorator, svg, g) {
            var shape;
            var attr;
            var d, x;
            var size = ej.datavisualization.Diagram.Size(decorator.width, decorator.height);
            switch (decorator.shape) {
                case "arrow":
                    d = this._constructArrow(point1, size);
                    attr = { "id": name, "class": "decorator", "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth };
                    this._addCssClass(decorator, attr);
                    shape = svg.path(attr);
                    x = 1.1 * decorator.borderWidth;
                    break;
                case "openarrow":
                    d = this._constructArrow(point1, size, true);
                    attr = { "id": name, "class": "decorator", "d": d, "fill": "transparent", "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth };
                    this._addCssClass(decorator, attr);
                    shape = svg.path(attr);
                    x = 1.1 * decorator.borderWidth;
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = {
                        "id": name, "class": "decorator", "rx": rx, "ry": ry, "cx": (point1.x + size.width / 2), "cy": (point1.y),
                        "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth
                    };
                    this._addCssClass(decorator, attr);
                    shape = svg.ellipse(attr);
                    x = 0.5 * decorator.borderWidth;
                    break;
                case "diamond":
                    d = this._constructDiamond(point1, size, svg);
                    attr = {
                        "id": name, "class": "decorator", "x": point1.x, "y": point1.y, "width": size.width,
                        "height": size.height, "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth
                    };
                    this._addCssClass(decorator, attr);
                    shape = svg.path(attr);
                    x = 0.7 * decorator.borderWidth;
                    break;
                case "path":
                    d = ej.datavisualization.Diagram.Geometry.updatePath(point1.x, point1.y - size.height / 2, size.width, size.height, decorator.pathData, svg);
                    decorator._absolutePath = d;
                    attr = {
                        "id": name, "class": "decorator", "x": point1.x, "y": point1.y, "width": size.width,
                        "height": size.height, "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth
                    };
                    this._addCssClass(decorator, attr);
                    shape = svg.path(attr);
                    x = 0.5 * decorator.borderWidth;
                    break;
            }
            //apply translate matrix and rotate matrix here
            if (shape) {
                if (decorator.borderColor != "none") {
                    var angle = ej.datavisualization.Diagram.Geometry.findAngle(point1, point2);
                    var extra = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, angle, x);
                    shape.setAttribute("transform", "translate(" + extra.x + "," + extra.y + "),rotate(" + angle + " " + point1.x + " " + point1.y + ")");
                }
                else
                    shape.setAttribute("transform", "rotate(" + ej.datavisualization.Diagram.Geometry.findAngle(point1, point2) + " " + point1.x + " " + point1.y + ")");
                g.appendChild(shape);
            }
        },

        _updateDecorators: function (connector, svg, diagram) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var segment = connector.segments[connector.segments.length - 1];
                var points = segment.points;
                startPoint = points[points.length - 2];
                endPoint = connector.targetPoint;
                if (segment.type == "bezier") {
                    startPoint = segment._point2;
                }
                if (connector.targetNode && diagram) {
                    var targetNode = diagram._findNode(connector.targetNode);
                    if (targetNode && targetNode.borderColor != "none")
                        endPoint = ej.datavisualization.Diagram.Util._adjustPoint(connector.targetPoint, startPoint, true, targetNode.borderWidth / 2);
                }
                this._updateDecorator(connector.name + "_targetDecorator", endPoint, startPoint, connector, connector.targetDecorator, svg);
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                var segment = connector.segments[0];
                var points = segment.points;
                startPoint = connector.sourcePoint;
                endPoint = points[1];
                if (segment.type == "bezier") {
                    endPoint = segment._point1;
                }
                if (connector.sourceNode && diagram) {
                    var sourceNode = diagram._findNode(connector.sourceNode);
                    if (sourceNode.borderColor != "none")
                        startPoint = ej.datavisualization.Diagram.Util._adjustPoint(connector.sourcePoint, endPoint, true, sourceNode.borderWidth / 2);
                }
                this._updateDecorator(connector.name + "_sourceDecorator", startPoint, endPoint, connector, connector.sourceDecorator, svg);
            }
        },

        _updateDecorator: function (name, point1, point2, connector, decorator, svg) {
            var shape;
            var attr;
            var d, x;
            var size = ej.datavisualization.Diagram.Size(decorator.width, decorator.height);

            switch (decorator.shape) {
                case "arrow":
                    d = this._constructArrow(point1, size);
                    attr = { "id": name, "d": d };
                    shape = svg.path(attr);
                    x = 1.1 * decorator.borderWidth;
                    break;
                case "openarrow":
                    d = this._constructArrow(point1, size, true);
                    attr = { "id": name, "d": d, "fill": "transparent", "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth };
                    shape = svg.path(attr);
                    x = 1.1 * decorator.borderWidth;
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = { "id": name, "rx": rx, "ry": ry, "cx": (point1.x + size.width / 2), "cy": (point1.y), "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth };
                    shape = svg.ellipse(attr);
                    x = 0.5 * decorator.borderWidth;
                    break;
                case "diamond":
                    d = this._constructDiamond(point1, size, svg);
                    attr = { "id": name, "x": point1.x, "y": point1.y, "width": size.width, "height": size.height, "d": d, "fill": decorator.fillColor, "stroke": decorator.borderColor, "stroke-width": decorator.borderWidth };
                    shape = svg.path(attr);
                    x = 0.7 * decorator.borderWidth;
                    break;
                case "path":
                    d = ej.datavisualization.Diagram.Geometry.updatePath(point1.x, point1.y - size.height / 2, size.width, size.height, decorator.pathData, svg);
                    decorator._absolutePath = d;
                    attr = {
                        "id": name, "d": d
                    };
                    shape = svg.path(attr);
                    x = 0.5 * decorator.borderWidth;
                    break;
            }
            //apply translate matrix and rotate matrix here
            if (shape) {
                if (decorator.borderColor != "none") {
                    var angle = ej.datavisualization.Diagram.Geometry.findAngle(point1, point2);
                    var extra = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, angle, x);
                    shape.setAttribute("transform", "translate(" + extra.x + "," + extra.y + "),rotate(" + angle + " " + point1.x + " " + point1.y + ")");
                }
                else
                    shape.setAttribute("transform", "rotate(" + ej.datavisualization.Diagram.Geometry.findAngle(point1, point2) + " " + point1.x + " " + point1.y + ")");
            }
        },

        _constructArrow: function (point, size, isOpen) {
            var path = new ej.datavisualization.Diagram.Path();
            path.moveTo(point.x + size.width, point.y + size.height / 2);
            path.lineTo(point.x, point.y);
            path.lineTo(point.x + size.width, point.y - size.height / 2);
            if (!isOpen) {
                path.close();
            }
            return path.toString();
        },

        _constructDiamond: function (point, size, svg) {
            var path = new ej.datavisualization.Diagram.Path();
            path.moveTo(point.x + size.width, point.y);
            path.lineTo(point.x + size.width / 2, point.y + size.height / 2);
            path.lineTo(point.x, point.y);
            path.lineTo(point.x + size.width / 2, point.y - size.height / 2);
            path.lineTo(point.x + size.width, point.y);
            var d = path.toString();
            return d;
        },
        //#endregion

        //#region Render Label
        _renderLabels: function (node, svg, diagram, palette) {
            if (node.labels.length && node.type != "text") {
                var labels = node.labels;
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                var width, height, x, y;
                var element;
                var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                element = $(htmlLayer).children("#" + node.name)[0];
                if (element) htmlLayer.removeChild(element);
                element = document.createElement("div");
                if (!node.segments) {
                    width = node.width ? node.width : node._width;
                    height = node.height ? node.height : node._height;
                    x = node.offsetX - width * node.pivot.x;
                    y = node.offsetY - height * node.pivot.y;
                }
                else {
                    width = bounds.width;
                    height = bounds.height;
                    x = bounds.x; y = bounds.y;
                }
                if (palette) {
                    x = 0;
                    y = 0;
                }
                var visibility = node.visible ? "visible" : "hidden";
                var style = "width:" + width + "px; height:" + height + "px; left:" + x + "px; top:" + y + "px; display: block; position: absolute; pointer-events: none; " + "visibility:" + visibility + "; padding: inherit;";
                var class1 = node._type === "node" ? "ej-d-node" : (node._type === "group" ? "ej-d-group" : "ej-d-connector");
                ej.datavisualization.Diagram.Util.attr(element, { "id": node.name, "class": class1, "style": style });
                if (node.rotateAngle) {
                    element.style.webkitTransform = "rotate(" + node.rotateAngle + "deg) ";
                    element.style.MozTransform = "rotate(" + node.rotateAngle + "deg) ";
                    element.style.OTransform = "rotate(" + node.rotateAngle + "deg) ";
                    element.style.msTransform = "rotate(" + node.rotateAngle + "deg)";
                    element.style.transform = "rotate(" + node.rotateAngle + "deg) ";
                    element.style["transform-origin"] = node.pivot.x * 100 + "% " + node.pivot.y * 100 + "%";

                }
                if (htmlLayer) htmlLayer.appendChild(element);
                for (var i = 0; i < labels.length; i++)
                    this._renderLabel(node, bounds, labels[i], element, htmlLayer, svg, width, height, diagram);
            }
        },
        _isParentVisible: function (name, svg) {
            var nodeEle = svg.document.getElementById(name);
            while (nodeEle && (nodeEle.getAttribute("class") != "DiagramLayer" || nodeEle.localName && nodeEle.localName !== "svg")) {
                if (nodeEle.localName && nodeEle.localName !== "svg")
                    break;
                if (nodeEle && (nodeEle.getAttribute("visibility") !== "visible"))
                    return true;
                nodeEle = nodeEle.parentNode;
            }
        },
        _renderLabel: function (node, nodeBounds, label, element, htmlLayer, svg, width, height, diagram) {
            var container = document.createElement("div");
            var text = document.createElement("span");
            if (svg.document && svg.document.getAttribute("class") === "overview_svg")
                ej.datavisualization.Diagram.Util.attr(text, this._addCssClass(label, { "id": node.name + "_" + label.name, "class": "ej-d-label", "style": "display: inline-block; position: absolute; line-height: normal; pointer-events: none" }));
            else
                ej.datavisualization.Diagram.Util.attr(text, this._addCssClass(label, { "id": node.name + "_" + label.name, "class": "ej-d-label", "style": "display: inline-block; position: absolute; line-height: normal; pointer-events: all" }));
            if (label.bold) text.style.fontWeight = "bold";
            if (label.italic) text.style.fontStyle = "italic";
            if (!label.visible || this._isParentVisible(node.name, svg)) text.style.visibility = "hidden";
            text.style.textDecoration = label.textDecoration;
            text.style.fontFamily = label.fontFamily;
            text.style.fontSize = label.fontSize + "px";
            text.style.color = label.fontColor;
            text.style.backgroundColor = label.fillColor;
            text.style.borderColor = label.borderColor;
            text.style.borderWidth = label.borderWidth + "px";
            text.style.borderStyle = "solid";
            text.textContent = label.text;
            width = width > label.width ? width : label.width;
            //height = height > label.height ? height : label.height;
            width -= (label.margin.left || 0) + (label.margin.right || 0);
            height -= (label.margin.top || 0) + (label.margin.bottom || 0);
            var style = "display: inline-block; position: absolute; pointer-events: none; " + "width:" + width + "px;height:" + height + "px;";
            ej.datavisualization.Diagram.Util.attr(container, { "id": node.name + "_" + label.name + "_lblbg", "style": style });
            container.appendChild(text);

            element.appendChild(container);
            this._alignTextOnLabel(node, nodeBounds, text, container, label, diagram);
        },

        _findChild: function (childNodes, node) {
            var length = childNodes.length;
            for (var i = 0; i < length; i++) {
                if (childNodes[i].id == node.name) {
                    return childNodes[i];
                }
            }
        },
        _updateLabels: function (node, svg, diagram) {
            if (node.labels && node.labels.length && node.type != "text") {
                var labels = node.labels;
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                var width, height, x, y;
                var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                if (htmlLayer) {
                    var parent = htmlLayer.childNodes[node.zOrder];
                    if (!parent || parent.id != node.name) {
                        var childNodes = htmlLayer.childNodes;
                        parent = this._findChild(childNodes, node, svg);
                    }
                }
                if (parent) {
                    if (!node.segments) {
                        width = node.width ? node.width : node._width;
                        height = node.height ? node.height : node._height;
                        x = node.offsetX - width * node.pivot.x;
                        y = node.offsetY - height * node.pivot.y;
                    }
                    else {
                        width = bounds.width; height = bounds.height;
                        x = bounds.x; y = bounds.y;
                    }
                    var visibility = node.visible ? "visible" : "hidden";
                    parent.style.width = width + "px";
                    parent.style.height = height + "px";
                    parent.style.left = x + "px";
                    parent.style.top = y + "px";
                    var transform = "rotate(" + node.rotateAngle + "deg) ";
                    var origin;
                    if (node.pivot)
                        origin = node.pivot.x * 100 + "% " + node.pivot.y * 100 + "%";
                    else
                        origin = 50 + "% " + 50 + "%";
                    parent.style.webkitTransform = transform;
                    parent.style.MozTransform = transform;
                    parent.style.OTransform = transform;
                    parent.style.msTransform = transform;
                    parent.style.transform = transform;
                    parent.style["transform-origin"] = parent.style["ms-transform-origin"] = parent.style["webkit-transform-origin"] =
                         parent.style["o-transform-origin"] = parent.style["moz-transform-origin"] = origin;
                    parent.style.visibility = visibility;
                    var textContainter, labelheight, labelwidth;
                    for (var i = 0; i < labels.length; i++) {
                        var label = labels[i];
                        textContainter = parent.childNodes[i];
                        var text = textContainter.childNodes[0];
                        if (label.text || text.textContent) {
                            labelwidth = width > label.width ? width : label.width;
                            labelheight = height > label.height ? height : label.height;
                            labelwidth -= (label.margin.left || 0) + (label.margin.right || 0);
                            labelheight -= (label.margin.top || 0) + (label.margin.bottom || 0);
                            textContainter.style.height = labelheight + "px";
                            textContainter.style.width = labelwidth + "px";
                            text.textContent = label.text;
                            this._alignTextOnLabel(node, bounds, text, textContainter, label, diagram);
                        }
                    }
                }

            }
        },

        _alignTextOnPhase: function (node, bounds, text, label, svg) {
            if (svg.getElementById(node.name))
                var phaseBounds = svg.getElementById(node.name).getBoundingClientRect();
            if (phaseBounds) {
                var textbounds = text.getBBox();
                var background = svg.getElementById(node.name + "_" + label.name + "_lblbg");
                text.setAttribute("transform", "translate(" + -textbounds.width / 2 + "," + 50 + "), rotate(" + 0 + "," + 0 + "," + 0 + ")");
                text.setAttribute("pointer-events", "auto");
                if (background) {
                    var attr = {
                        "width": textbounds.width, "height": textbounds.height, "fill": label.fillColor,
                        "stroke": label.borderColor, "stroke-width": label.borderWidth,
                        "transform": "translate(" + -textbounds.width + "," + phaseBounds.top + "), rotate(" + 0 + "," + 0 + "," + 0 + ")"
                    };
                    ej.datavisualization.Diagram.Util.attr(background, attr);
                }
            }
        },

        updateLabel: function (node, label, svg, diagram) {
            if (typeof label.text === "string") {
                var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                var htmlLayer = svg.document.parentNode.getElementsByClassName("htmlLayer")[0] || diagram._htmlLayer;
                var parent = $(htmlLayer).children("#" + node.name)[0];
                var textContainter = $(parent).children("#" + node.name + "_" + label.name + "_lblbg")[0];
                if (textContainter) {
                    var text = textContainter.childNodes[0];
                    text.textContent = label.text;
                    if (node.type == "node" && !node.width && !node.height)
                        this.updateNode(node, svg, diagram);
                    this._alignTextOnLabel(node, bounds, text, textContainter, label, diagram);
                }
            }
        },

        _alignTextOnLabel: function (node, nodeBounds, text, textContainter, label, diagram) {
            var actualDimensions, hAlign, vAlign;
            if (node.segments && typeof label.offset != "object") {
                var obj = this._getConnectorHandlePosition(label, node, 1, diagram);
                var pt = obj.position;
                var offset = {
                    x: (pt.x - nodeBounds.x) / nodeBounds.width,
                    y: (pt.y - nodeBounds.y) / nodeBounds.height
                };
                var alignment = ej.datavisualization.Diagram.Util._alignLabelOnSegments(node, label, diagram, obj);
                vAlign = alignment.vAlign; hAlign = alignment.hAlign;
            }
            else {
                var offset = { x: label.offset.x, y: label.offset.y };
                hAlign = label.horizontalAlignment;
                vAlign = label.verticalAlignment;
            }
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            width -= label.margin.left + label.margin.right;
            height -= label.margin.top + label.margin.bottom;
            actualDimensions = { x: width, y: height };
            if (label.rotateAngle) {
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, label.rotateAngle, offset.x, offset.y);
                actualDimensions = ej.Matrix.transform(matrix, { x: width, y: height });
            }
            var x = 0, y = 0, marginX, marginY;
            var canMoveLabel = ej.datavisualization.Diagram.Util.canMoveLabel(node);
            if (label.wrapping == ej.datavisualization.Diagram.TextWrapping.NoWrap) {
                text.style.whiteSpace = "nowrap";
                text.style.wordWrap = "normal";
                text.style.maxWidth = "";
                text.style.maxHeight = "";
            }
            else {
                if (label.textAlign != "justify")
                    text.style.whiteSpace = "pre-wrap";
                else text.style.whiteSpace = "pre-line";
                if (label.wrapping == ej.datavisualization.Diagram.TextWrapping.Wrap) {
                    text.style.wordBreak = "break-all";
                    text.style.wordWrap = "break-word";
                    if (!node.segments) {
                        if (Math.abs(actualDimensions.x) < label.width) actualDimensions.x = label.width;
                        text.style.maxWidth = Math.abs(actualDimensions.x) + "px";
                        text.style.maxHeight = Math.abs(actualDimensions.y) + "px";
                    }
                    else text.style.maxWidth = Math.abs(label.width) + "px";
                }
                else {
                    text.style.wordWrap = "break-word";
                    text.style.maxWidth = "";
                    text.style.maxHeight = "";
                }
            }
            switch (hAlign) {
                case ej.datavisualization.Diagram.HorizontalAlignment.Left:
                    x = "0%";
                    marginX = label.margin.left + "px";
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Center:
                    x = "-50%";
                    marginX = label.margin.left - label.margin.right + "px";
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Right:
                    x = "-100%";
                    marginX = -label.margin.right + "px";
                    break;
            }
            switch (vAlign) {
                case ej.datavisualization.Diagram.VerticalAlignment.Top:
                    y = "0%";
                    marginY = label.margin.top + "px";
                    break;
                case ej.datavisualization.Diagram.VerticalAlignment.Center:
                    y = "-50%";
                    marginY = label.margin.top - label.margin.bottom + "px";
                    break;
                case ej.datavisualization.Diagram.VerticalAlignment.Bottom:
                    y = "-100%";
                    marginY = -label.margin.bottom + "px";
                    break;
            }
            if (canMoveLabel) {
                textContainter.style.marginLeft = marginX;
                textContainter.style.marginTop = marginY;
            }
            else {
                text.style.marginLeft = marginX;
                text.style.marginTop = marginY;
            }
            offset.x *= 100;
            offset.y *= 100;
            text.style.textAlign = label.textAlign;
            text.style.webkitTransform = "translate(" + x + ", " + y + ") rotate(" + label.rotateAngle + "deg)";
            text.style.MozTransform = "translate(" + x + ", " + y + ") rotate(" + label.rotateAngle + "deg)";
            text.style.OTransform = "translate(" + x + ", " + y + ") rotate(" + label.rotateAngle + "deg)";
            text.style.msTransform = "translate(" + x + ", " + y + ") rotate(" + label.rotateAngle + "deg)";
            text.style.transform = "translate(" + x + ", " + y + ") rotate(" + label.rotateAngle + "deg)";
            text.style.left = "0px";
            text.style.top = "0px";
            if (textContainter) {
                textContainter.style.left = offset.x + "%";
                textContainter.style.top = offset.y + "%";
            }
        },
        //#region Render labels as svg
        _renderLabelsAsSvg: function (node, svg, diagram) {
            var g = svg.document.getElementById(node.name);
            if (node.type == "text") {
                this._renderTextElementAsSvg(node, node.textBlock, g, svg, diagram);
            } else {
                for (var i = 0; i < node.labels.length; i++) {
                    this._renderTextElementAsSvg(node, node.labels[i], g, svg, diagram);
                }
            }
        },
        _renderTextElementAsSvg: function (node, label, g, svg, diagram) {
            var attr = {
                "id": node.name + "_" + label.name, "class": "ej-d-label", "font-family": label.fontFamily,
                "font-size": label.fontSize, "fill": label.fontColor, "text-decoration": label.textDecoration,
                "dominant-baseline": "middle", "pointer-events": "none"
            };
            if (label.bold)
                attr["font-weight"] = "bold";
            if (label.italic)
                attr["font-style"] = "italic";
            var text = svg.text(attr);
            text.textContent = label.text;
            attr = {
                "id": node.name + "_" + label.name + "_lblbg",
                "fill": label.fillColor,
                "stroke": label.borderColor, "stroke-width": label.borderWidth, "transform": "translate(" + 0 + "," + 0 + ")"
            };
            var bg = svg.rect(attr);
            if (!label.text)
                return;
            g.appendChild(text);
            g.insertBefore(bg, text);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            this._wrapText(node, bounds, text, label, svg);
            this._alignTextOnSvgLabel(node, bounds, text, label, svg, diagram);
        },
        _alignTextOnSvgLabel: function (node, nodeBounds, text, label, svg, diagram) {
            var bounds = text.getBBox();
            var offset = ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds);
            var point = this._adjustOffsetWithTextBounds(node, label, text, offset, bounds, nodeBounds, diagram);
            var bWidth = label.borderWidth || 0;
            text.setAttribute("transform", "translate(" + (point.x + bWidth) + "," + (point.y + bWidth) + "), rotate(" + label.rotateAngle + "," + (offset.x - point.x) + "," + (offset.y - point.y) + ")");
            //Updates the label background
            var background = svg.getElementById(node.name + "_" + label.name + "_lblbg");
            if (background) {
                var x = point.x + bounds.x;
                var y = point.y;
                if (ej.browserInfo().name === "mozilla")
                    y = point.y + label.fontSize / 2;
                x = x + bWidth / 2;
                y = y + bWidth / 2;
                var attr = {
                    "width": bounds.width + bWidth, "height": bounds.height + bWidth, "transform": "translate(" + x + "," + y + "), rotate(" + label.rotateAngle + "," + (offset.x - x) + "," + (offset.y - y) + ")", "fill": label.fillColor ? label.fillColor : "transparent",
                    "stroke": label.borderColor, "stroke-width": label.borderWidth
                };
                ej.datavisualization.Diagram.Util.attr(background, attr);
            }
        },
        _adjustOffsetWithTextBounds: function (node, label, text, offset, bounds, nodeBounds, diagram) {
            var point = { x: 0, y: 0 };
            var y = 0;
            var vAlign, hAlign;
            if (node.segments && typeof label.offset != "object") {
                var obj = this._getConnectorHandlePosition(label, node, 1, diagram);
                var pt = obj.position;
                var ptt = {
                    x: (pt.x - nodeBounds.x) / nodeBounds.width,
                    y: (pt.y - nodeBounds.y) / nodeBounds.height
                };
                ptt = ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds, ptt);
                offset.x = ptt.x;
                offset.y = ptt.y;
                var alignment = ej.datavisualization.Diagram.Util._alignLabelOnSegments(node, label, diagram, obj);
                hAlign = alignment.hAlign;
                vAlign = alignment.vAlign;
            }
            else {
                hAlign = label.horizontalAlignment;
                vAlign = label.verticalAlignment;
            }
            if (vAlign == ej.datavisualization.Diagram.VerticalAlignment.Top)
                y = offset.y;
            else if (vAlign == ej.datavisualization.Diagram.VerticalAlignment.Center)
                y = offset.y - bounds.height / 2;
            else
                y = offset.y - bounds.height;
            point.y = y;
            if (label.textAlign == "justify") {
                if (text.childNodes.length > 1) {
                    { point.x = offset.x - node.width / 2; }
                }
                else { point.x = offset.x; }
            }
            else {
                switch (hAlign) {
                    case ej.datavisualization.Diagram.HorizontalAlignment.Left:
                        switch (label.textAlign) {
                            case ej.datavisualization.Diagram.TextAlign.Left:
                                point.x = offset.x;
                                break;
                            case ej.datavisualization.Diagram.TextAlign.Center:
                                point.x = offset.x + bounds.width / 2;
                                break;
                            case ej.datavisualization.Diagram.TextAlign.Right:
                                point.x = offset.x + bounds.width;
                                break;
                        }
                        break;
                    case ej.datavisualization.Diagram.HorizontalAlignment.Center:
                        switch (label.textAlign) {
                            case ej.datavisualization.Diagram.TextAlign.Left:
                                point.x = offset.x - bounds.width / 2;
                                break;
                            case ej.datavisualization.Diagram.TextAlign.Center:
                                point.x = offset.x;
                                break;
                            case ej.datavisualization.Diagram.TextAlign.Right:
                                point.x = offset.x + bounds.width / 2;
                                break;
                        }
                        break;
                    case ej.datavisualization.Diagram.HorizontalAlignment.Right:
                        switch (label.textAlign) {
                            case ej.datavisualization.Diagram.TextAlign.Left:
                                point.x = offset.x - bounds.width;
                                break;
                            case ej.datavisualization.Diagram.TextAlign.Center:
                                point.x = offset.x - bounds.width / 2;
                                break;
                            case ej.datavisualization.Diagram.TextAlign.Right:
                                point.x = offset.x;
                                break;
                        }
                        break;
                }
            }
            if (node.segments || node.type == "group") {
                point.x = point.x + nodeBounds.x;
                point.y = point.y + nodeBounds.y;
                offset.x += nodeBounds.x;
                offset.y += nodeBounds.y;
            }
            if (ej.browserInfo().name === "mozilla")
                point.y = point.y - label.fontSize / 2;
            return point;
        },

        _wrapText: function (node, textBBox, text, label, svg) {
            var str = label.text; var attr = null;
            while (text.hasChildNodes()) {
                text.removeChild(text.lastChild);
            }
            var bbWidth, bbHeight;
            if (node.segments) {
                bbWidth = textBBox.width;
                bbHeight = textBBox.height;
            }
            else {
                bbWidth = textBBox.width - label.margin.left - label.margin.right;
                bbHeight = textBBox.height - label.margin.top - label.margin.bottom;
            }
            bbWidth = bbWidth < label.width ? label.width : bbWidth;
            var eachLine = str.split('\n');
            var x, y, tspan, j, string = "", childNodes, bounds, i;
            if (svg.getElementById(text.id))
                childNodes = svg.getElementById(text.id).childNodes;
            if (childNodes) {
                var wrap = label.wrapping == "wrapwithoverflow" ? true : false;
                for (j = 0; j < eachLine.length; j++) {
                    tspan = svg.tspan();
                    tspan.style.fontSize = label.fontSize;
                    text.appendChild(tspan);
                    if (label.wrapping != "nowrap") {
                        var words = label.wrapping == "wrapwithoverflow" ? eachLine[j].split(" ") : eachLine[j];
                        for (i = 0; i < words.length; i++) {
                            tspan.textContent += ((txt && wrap) ? " " : "") + words[i];
                            bounds = childNodes[childNodes.length - 1].getComputedTextLength();
                            if (bounds >= bbWidth) {
                                if (!(i == words.length - 1)) {
                                    tspan = svg.tspan();
                                    text.appendChild(tspan);
                                }
                            }
                            else {
                                if (wrap) {
                                    var txt = tspan.textContent;
                                    tspan.textContent += (words[i + 1] || "");
                                    var length = childNodes[childNodes.length - 1].getComputedTextLength();
                                    tspan.textContent = txt;
                                    if (length > bbWidth) {
                                        tspan = svg.tspan();
                                        text.appendChild(tspan);
                                    }
                                }
                            }
                        }

                    }
                    else tspan.textContent += eachLine[j];
                }
                this._wrapTextAlign(text, childNodes, label.fontSize, label.textAlign, node);
            }
        },

        _wrapTextAlign: function (text, childNodes, height, textAlign, node) {
            var x, difspace, wordSpacing, txt;
            for (var i = 0; i < childNodes.length; i++) {
                if (textAlign == "justify" && i != childNodes.length - 1) {
                    txt = childNodes[i].textContent;
                    if (txt[txt.length - 1] == " ") { txt = txt.slice(0, txt.length - 1); } if (txt[0] == " ") { txt = txt.slice(1, txt.length); }
                    childNodes[i].textContent = txt;
                    x = childNodes[i].getComputedTextLength();
                    difspace = node.width - x;
                    wordSpacing = difspace / (txt.split(" ").length - 1);
                    childNodes[i].setAttribute("word-spacing", wordSpacing + "px");
                    x = childNodes[i].getComputedTextLength();
                }
                else x = childNodes[i].getComputedTextLength();
                if (textAlign == "justify") { childNodes.length > 1 ? textAlign = "left" : textAlign = "center"; }
                switch (textAlign) {
                    case "left":
                        x = 0;
                        break;
                    case "center":
                        x = -x / 2;
                        break;
                    case "right":
                        x = -x;
                        break;
                }
                var attr = { "x": Number(x), "dy": height };
                var tspan = childNodes[i];
                ej.datavisualization.Diagram.Util.attr(tspan, attr);
            }
        },
        //#endregion

        //#region Render Port
        _renderPorts: function (node, svg, g) {
            var ports = node.ports;
            var port;
            for (var i = 0, len = ports.length; i < len; ++i) {
                port = ports[i];
                var shape = this._renderPort(node, port, svg);
                g.appendChild(shape);
            }
        },

        _renderPort: function (node, port, svg) {
            var shape;
            var attr, visibility;
            var size = ej.datavisualization.Diagram.Size(port.size, port.size);
            var point;
            if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hidden || port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover || port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect)
                visibility = "hidden";
            else
                visibility = "visible";
            if (node._type === "group")
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), false);
            else
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), true);
            switch (port.shape) {
                case "x":
                    var d = this._constructX(point, port.size);
                    attr = {
                        "id": node.name + "_" + port.name, "class": "ej-d-port", "fill": port.fillColor, "visibility": visibility,
                        "stroke": port.borderColor, "stroke-width": port.borderWidth, "d": d
                    };
                    this._addCssClass(port, attr);
                    shape = svg.path(attr);
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    attr = {
                        "id": node.name + "_" + port.name, "class": "ej-d-port", "rx": rx, "ry": ry, "visibility": visibility,
                        "cx": point.x, "cy": point.y, "fill": port.fillColor, "stroke": port.borderColor,
                        "stroke-width": port.borderWidth
                    };
                    this._addCssClass(port, attr);
                    shape = svg.ellipse(attr);
                    break;
                case "square":
                    attr = {
                        "id": node.name + "_" + port.name, "class": "ej-d-port", "x": point.x - port.size / 2, "visibility": visibility,
                        "y": point.y - port.size / 2, "width": size.width, "height": size.height,
                        "fill": port.fillColor, "stroke": port.borderColor, "stroke-width": port.borderWidth
                    };
                    this._addCssClass(port, attr);
                    shape = svg.rect(attr);
                    break;
                case "path":
                    var width = size.width;
                    var height = size.height;
                    var d = port._absolutePath = ej.datavisualization.Diagram.Geometry.updatePath(point.x - port.size / 2, point.y - port.size / 2, width, height, port.pathData, svg);
                    var attr = {
                        "id": node.name + "_" + port.name, "width": width,
                        "height": height, "d": d, "stroke": port.borderColor,
                        "stroke-width": port.borderWidth, "fill": port.fillColor, "visibility": visibility

                    };
                    this._addCssClass(port, attr);
                    shape = svg.path(attr);
                    break;
            }
            return shape;
        },
        _insertPort: function (node, port, svg) {
            var g;
            g = document.getElementById(node.name);
            var portshape = ej.datavisualization.Diagram.SvgContext._renderPort(node, port, svg);
            g.appendChild(portshape);
            if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hidden || port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover || port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect) {
                portshape.setAttribute("visibility", "hidden");
            }

        },
        _updatePort: function (node, port, svg) {
            var size = ej.datavisualization.Diagram.Size(port.size, port.size);
            var point = null;
            if (node._type === "group")
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), false);
            else
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true), true);
            var id = node.name + "_" + port.name;
            if (node._type === "group" && node.rotateAngle) {
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                point = ej.Matrix.transform(matrix, point);
            }
            switch (port.shape) {
                case "x":
                    var d = this._constructX(point, port.size);
                    svg.path({ "id": id, "d": d });
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    if (node.type === "group") {
                        svg.ellipse({ "id": id, "rx": rx, "ry": ry, "cx": 0, "cy": 0, "transform": "rotate(" + node.rotateAngle + "," + (point.x) + "," + (point.y) + "),translate(" + (point.x) + "," + (point.y) + ")" });
                    }
                    else
                        svg.ellipse({ "id": id, "rx": rx, "ry": ry, "cx": point.x, "cy": point.y });
                    break;
                case "square":
                    if (node.type === "group") {
                        svg.rect({ "id": id, "x": 0, "y": 0, "transform": "rotate(" + node.rotateAngle + "," + (point.x) + "," + (point.y) + "),translate(" + (point.x - port.size / 2) + "," + (point.y - port.size / 2) + ")" });
                    }
                    else
                        svg.rect({ "id": id, "x": point.x - port.size / 2, "y": point.y - port.size / 2 });
                    break;
                case "path":
                    var width = size.width;
                    var height = size.height;
                    var d = port._absolutePath = ej.datavisualization.Diagram.Geometry.updatePath(point.x - port.size / 2, point.y - port.size / 2, width, height, port.pathData, svg);
                    var attr = {
                        "id": id, "d": d
                    };
                    svg.path(attr);
                    break;
            }
        },

        _updatePorts: function (node, svg) {
            var ports = node.ports;
            for (var i = 0, len = ports.length; i < len; ++i) {
                var port = ports[i];
                this._updatePort(node, port, svg);
            }
        },

        _constructX: function (point, size) {
            var path = new ej.datavisualization.Diagram.Path();
            path.moveTo((point.x - size / 2), (point.y - size / 2));
            path.lineTo((point.x + size / 2), (point.y + size / 2));
            path.moveTo((point.x + size / 2), (point.y - size / 2));
            path.lineTo((point.x - size / 2), (point.y + size / 2));
            return path.toString();
        },
        _enableSelectedNode: function (node, svg, diagram) {
            if (node) {
                if (node.type == "pseudoGroup") {
                    var children = node.children;
                    for (var i = 0, len = children.length; i < len; i++) {
                        var child = diagram.nameTable[diagram._getChild(children[i])];
                        if (child)
                            this._enableSelectedNode(child, svg, diagram);
                    }
                } else {
                    var attr = {
                        "id": node.name,
                        "pointer-events": "auto",
                        "style": "pointer-events:block",
                    }
                    if (node.segments) {
                        var element = svg.document.getElementById(node.name + "_hitTest");
                        element.setAttribute("pointer-events", node.visible ? "stroke" : "none");
                    }
                    svg.g(attr);
                }
                if (diagram._adornerSvg) {
                    var selector = diagram._adornerSvg.document.getElementById(diagram._adornerSvg.document.id + "handle_g");
                    if (selector) selector.setAttribute("pointer-events", "visible");
                }
                $(".ej-d-seperator").css("pointer-events", "stroke");
                $(".e-resize").css("pointer-events", "visible");
            }
        },
        _disableSelectedNode: function (node, svg, diagram) {
            if (node) {
                if (node.type == "pseudoGroup") {
                    var children = node.children;
                    for (var i = 0, len = children.length; i < len; i++) {
                        var child = diagram.nameTable[diagram._getChild(children[i])];
                        if (child)
                            this._disableSelectedNode(child, svg, diagram);
                    }
                } else {
                    var attr = {
                        "id": node.name,
                        "style": "pointer-events:none",
                        "pointer-events": "none",
                    }
                    if (node.segments) {
                        var element = svg.document.getElementById(node.name + "_hitTest");
                        element.setAttribute("pointer-events", "none");
                    }
                    svg.g(attr);
                }
                if (diagram._adornerSvg) {
                    var selector = diagram._adornerSvg.document.getElementById(diagram._adornerSvg.document.id + "handle_g");
                    if (selector) selector.setAttribute("pointer-events", "none");
                }
                $(".e-resize").css("pointer-events", "none");
                $(".ej-d-seperator").css("pointer-events", "none");
            }
        },

        _drawStackHighlighter: function (node, svg, parent, scale, currentPoint, isVertcal) {
            var nPoint, points = [];
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            if (isVertcal) {
                if (currentPoint.y >= bounds.y && currentPoint.y < bounds.center.y) {
                    points.push({ x: bounds.topLeft.x * scale - 5, y: bounds.topLeft.y * scale - 5 });
                    points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
                    points.push({ x: bounds.topLeft.x * scale - 5, y: bounds.topLeft.y * scale + 5 });
                    points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
                    points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
                    points.push({ x: bounds.topRight.x * scale + 5, y: bounds.topRight.y * scale - 5 });
                    points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
                    points.push({ x: bounds.topRight.x * scale + 5, y: bounds.topRight.y * scale + 5 });
                    points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
                }
                else {
                    points.push({ x: bounds.bottomLeft.x * scale - 5, y: bounds.bottomLeft.y * scale - 5 });
                    points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
                    points.push({ x: bounds.bottomLeft.x * scale - 5, y: bounds.bottomLeft.y * scale + 5 });
                    points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
                    points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
                    points.push({ x: bounds.bottomRight.x * scale + 5, y: bounds.bottomRight.y * scale - 5 });
                    points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
                    points.push({ x: bounds.bottomRight.x * scale + 5, y: bounds.bottomRight.y * scale + 5 });
                    points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
                }
            }
            else {
                if (currentPoint.x >= bounds.x && currentPoint.x < bounds.center.x) {
                    points.push({ x: bounds.topLeft.x * scale - 5, y: (bounds.topLeft.y) * scale - 5 });
                    points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
                    points.push({ x: bounds.topLeft.x * scale + 5, y: (bounds.topLeft.y) * scale - 5 });
                    points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
                    points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
                    points.push({ x: bounds.bottomLeft.x * scale - 5, y: bounds.bottomLeft.y * scale + 5 });
                    points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
                    points.push({ x: bounds.bottomLeft.x * scale + 5, y: bounds.bottomLeft.y * scale + 5 });
                    points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
                    points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
                }
                else {
                    points.push({ x: bounds.topRight.x * scale - 5, y: bounds.topRight.y * scale - 5 });
                    points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
                    points.push({ x: bounds.topRight.x * scale + 5, y: (bounds.topRight.y) * scale - 5 });
                    points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
                    points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
                    points.push({ x: bounds.bottomRight.x * scale - 5, y: (bounds.bottomRight.y) * scale + 5 });
                    points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
                    points.push({ x: bounds.bottomRight.x * scale + 5, y: (bounds.bottomRight.y) * scale + 5 });
                    points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
                    points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
                }

            }
            var nPoint = this._convertToSVGPoints(points);
            var attr = {
                "id": "nodeHighlighter",
                "points": nPoint,
                "fill": "transparent",
                "stroke": "red",
                "stroke-width": 2,
                "pointer-events": "none",
                "style": "pointer-events: none",
            };
            var line = svg.polyline(attr);
            parent.appendChild(line);
            return line;
        },

        _drawNodeHighlighter: function (node, svg, parent, scale) {
            var shape;
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var x = node.offsetX - width * node.pivot.x;
            var y = node.offsetY - height * node.pivot.y;
            var center = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
            var transform = "rotate(" + node.rotateAngle + "," + center.x * scale + "," + center.y * scale + ")";
            var stroke = node.isLane || node.isSwimlane ? "red" : "#0dc923";
            var attr = {
                "id": "nodeHighlighter",
                "class": "ej-d-node",
                "x": x * scale,
                "y": y * scale,
                "width": width * scale,
                "height": height * scale,
                "fill": "transparent",
                "stroke": stroke,
                "stroke-width": 2,
                "pointer-events": "none",
                "transform": transform,
                "style": "pointer-events: none",
            };
            shape = svg.rect(attr);
            parent.appendChild(shape);
            return shape;
        },

        _removeNodeHighlighter: function (svg, parent) {
            var highlighter = svg.getElementById("nodeHighlighter");
            if (highlighter) {
                svg.removeChild(highlighter, parent);
            }
        },
        _drawPortHighlighter: function (port, node, svg, parent, scale) {
            var shape, size, point, x, y, transform;
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var bounds;
            var fill = "#8CC63F";
            var border = "#1A1A1A";
            var borderWidth = 0.5;
            if (!port) {
                bounds = ej.datavisualization.Diagram.Util.bounds(node, true);
                size = ej.datavisualization.Diagram.Size(bounds.width, bounds.height);
                point = ej.datavisualization.Diagram.Point(bounds.x, bounds.y);
                x = point.x * scale;
                y = point.y * scale;
                border = fill;
                fill = "transparent";
                borderWidth = 2;
                var center = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
                transform = "rotate(" + node.rotateAngle + "," + center.x * scale + "," + center.y * scale + ")";
            }
            else {
                size = ej.datavisualization.Diagram.Size(port.size * 2, port.size * 2);
                point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true));
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, node.rotateAngle, node.offsetX, node.offsetY);
                point = ej.Matrix.transform(matrix, point);
                x = (point.x - port.size) * scale;
                y = (point.y - port.size) * scale;
                transform = "rotate(" + 0 + "," + (x + size.width / 2) + "," + (y + size.height / 2) + ")";
            }
            var attr = {
                "id": "portHighlighter", "class": "ej-d-port", "x": x,
                "y": y, "width": size.width * scale, "height": size.height * scale,
                "fill": fill, "stroke": border, "stroke-width": borderWidth, "pointer-events": "none", "transform": transform
            };
            shape = svg.rect(attr);
            parent.appendChild(shape);
            return shape;
        },

        _drawConnectorHighlighter: function (connector, diagram, scale) {
            var svg = diagram._adornerSvg;
            var g = svg.g({ "id": "connector_highlighter", "visibility": "visible", transform: "scale(" + scale + ")" });
            diagram._adornerLayer.appendChild(g);
            var path = this._findPath(connector, diagram);
            var attr = {
                "id": "highlighter_segments",
                "d": path, "fill": "none", "stroke": "#0dc923",
                "stroke-width": connector.lineWidth, "pointer-events": "none"
            };
            var line = svg.path(attr);
            g.appendChild(line);
            return g;
        },

        _removeConnectorHighlighter: function (parent, svg) {
            var highlighter = svg.getElementById("connector_highlighter");
            if (highlighter) {
                parent.removeChild(highlighter);
            }
        },
        _removePortHighlighter: function (svg, parent) {
            var highlighter = svg.getElementById("portHighlighter");
            if (highlighter) {
                svg.removeChild(highlighter, parent);
            }
        },

        _renderPhaseResize: function (shape, svg, scale, constraints, parent) {
            var nPoint, points = [];
            var clName = "", isHorizon = false, isVerti = false;
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            //#region left
            nPoint, points = [];
            points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
            points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
            nPoint = this._convertToSVGPoints(points);
            var attr = { "id": "w-resize", "points": nPoint, "stroke-dasharray": "3,3", "fill": "transparent", "stroke": "red", "stroke-width": 2, "pointer-events": "none" };
            line = svg.polyline(attr);
            parent.appendChild(line);
            //#endregion  

            //#region Top
            nPoint, points = [];
            points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
            points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
            nPoint = this._convertToSVGPoints(points);
            var attr = { "id": "n-resize", "points": nPoint, "stroke-dasharray": "3,3", "fill": "transparent", "stroke": "red", "stroke-width": 2, "pointer-events": "none" };
            line = svg.polyline(attr);
            parent.appendChild(line);
            //#endregion 

            //#region right
            clName = "";
            if (shape.orientation == "vertical") {
                clName = "e-resize"
                isVerti = true;
            }
            nPoint, points = [];
            points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
            points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
            nPoint = this._convertToSVGPoints(points);
            if (isVerti) {
                attr = { "id": "e-resize-hitTest", "class": clName, "points": nPoint, "fill": "transparent", "stroke": "transparent", "stroke-width": 10, "pointer-events": isVerti ? "stroke" : "none" };
                var line = svg.polyline(attr);
                parent.appendChild(line);
            }
            var attr = { "id": "e-resize", "class": clName, "points": nPoint, "stroke-dasharray": "3,3", "fill": "transparent", "stroke": "red", "stroke-width": 2, "pointer-events": isVerti ? "stroke" : "none" };
            line = svg.polyline(attr);
            parent.appendChild(line);
            //#endregion

            //#region bottom
            clName = "";
            if (shape.orientation == "horizontal") {
                clName = "s-resize"
                isHorizon = true;
            }
            nPoint, points = [];
            points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
            points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
            nPoint = this._convertToSVGPoints(points);
            if (isHorizon) {
                attr = { "id": "s-resize-hitTest", "class": clName, "points": nPoint, "fill": "transparent", "stroke": "transparent", "stroke-width": 10, "pointer-events": isHorizon ? "stroke" : "none" };
                var line = svg.polyline(attr);
                parent.appendChild(line);
            }
            var attr = { "id": "s-resize", "class": clName, "points": nPoint, "stroke-dasharray": "3,3", "fill": "transparent", "stroke": "red", "stroke-width": 2, "pointer-events": isHorizon ? "stroke" : "none" };
            line = svg.polyline(attr);
            parent.appendChild(line);
            //#endregion
        },
        //#endregion

        //#region Render Selector

        renderSelector: function (shape, svg, parent, scale, constraints, bounds) {
            $(".phaseSelector").remove();
            var width = shape.width ? shape.width : shape._width;
            var height = shape.height ? shape.height : shape._height;
            if (shape.segments) {
                var g = svg.g({
                    "id": svg.document.id + "handle_g", "class": "handle", "pointer-events": "visible"
                });
                parent.appendChild(g);
                this._renderEndPointHandle(shape, svg, scale);
            }
            else if (shape.type != "phase") {
                if (!(shape.isLane || shape.isSwimlane)) {
                    var g = svg.g({
                        "id": svg.document.id + "handle_g", "class": "handle", "pointer-events": "visible",
                        "transform": "translate(" + (shape.offsetX - width * shape.pivot.x) * scale + "," + (shape.offsetY - height * shape.pivot.y) * scale + "),rotate(" + shape.rotateAngle + "," + (width * shape.pivot.x) * scale + "," + (height * shape.pivot.y) * scale + ")"
                    });
                    parent.appendChild(g);
                    this._renderResizeHandle(shape, svg, scale, constraints);
                }
                else {
                    var g = svg.g({
                        "id": svg.document.id + "handle_g", "class": "handle", "pointer-events": "visible"
                    });
                    parent.appendChild(g);
                    this._renderPhaseResize(shape, svg, scale, constraints, g, bounds);
                }
            }
            else if (shape.type == "phase" && bounds) {
                var g = svg.g({
                    "id": svg.document.id + "handle_g", "class": "handle", "pointer-events": "visible",
                    "transform": "translate(" + (bounds.x) * scale + "," + (bounds.y) * scale + "),rotate(0," + (bounds.width / 2) * scale + "," + (bounds.height / 2) * scale + ")"
                });
                if (!svg.document.getElementById(svg.document.id + "handle_g"))
                    parent.appendChild(g);
                var rect = svg.rect({
                    "id": "phaseSelector", "class": "phaseSelector",
                    "stroke-width": 2, "stroke-dasharray": "3,3", "pointer-events": "none",
                    "width": (bounds.width < 0) ? 1 : bounds.width * scale, "height": (bounds.height < 0) ? 1 : bounds.height * scale, "fill": "transparent", "stroke": "red",
                });
                g.appendChild(rect);
            }
        },

        _renderResizeHandle: function (node, svg, scale, constraints) {
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var pivot = node.pivot;
            var rotate = ej.datavisualization.Diagram.Point(width * pivot.x, -20 / scale);
            if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Rotator) {
                this._renderPivotLine(node, svg, scale);
                this._renderRotateThumb("rotate", node, rotate.x, rotate.y, scale, svg);
            }
            if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Resizer) {
                this._renderResizeBorder("resizeBorder", node, svg, scale);
                this._renderResizeCorner("nw-resize", node, 0, 0, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorthWest));
                this._renderResizeCorner("n-resize", node, (width / 2) * scale, 0, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorth));
                this._renderResizeCorner("ne-resize", node, width * scale, 0, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeNorthEast));
                this._renderResizeCorner("w-resize", node, 0, (height / 2) * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeWest));
                this._renderResizeCorner("e-resize", node, width * scale, (height / 2) * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeEast));
                this._renderResizeCorner("sw-resize", node, 0, height * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouthWest));
                this._renderResizeCorner("s-resize", node, (width / 2) * scale, height * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouth));
                this._renderResizeCorner("se-resize", node, width * scale, height * scale, svg, !(node.constraints & ej.datavisualization.Diagram.NodeConstraints.ResizeSouthEast));
            }
        },

        renderUserHandles: function (handles, shape, svg, isMultipleSelection, scale, parent, diagram) {
            if (shape) {
                if (shape.type === "phase" && diagram)
                    var bounds = diagram._getPhaseBounds(shape);
                if (handles.length) {
                    var g = svg.g({ "id": svg.document.id + "userHandle_g", "class": "userHandle", "pointer-events": "fill" });
                    parent.appendChild(g);
                    for (var handleIndex = 0; handleIndex < handles.length; handleIndex++) {
                        if ((isMultipleSelection && handles[handleIndex].enableMultiSelection) || !isMultipleSelection) {
                            if (handles[handleIndex].visible)
                                this._renderUserHandle(handles[handleIndex], shape, svg, scale, g, bounds, diagram);
                        }
                    }
                }
            }
        },

        _renderUserHandle: function (handle, node, svg, scale, g, bounds, diagram) {
            var position = this._getHandlePosition(handle, node, scale, bounds, diagram);
            var d = handle._absolutePath = ej.datavisualization.Diagram.Geometry.updatePath(0, 0, handle.size / 2, handle.size / 2, handle.pathData, svg);
            var x = position.x * scale;// - (node.segments ? 0 : handle.size / 4);
            var y = position.y * scale;// - (node.segments ? 0 : handle.size / 4)
            var shape = this._getHandleShape(handle, x, y);
            var icon = this._getHandleIcon(handle, d, x - handle.size / 4, y - handle.size / 4);
            g.appendChild(svg.circle(shape));
            g.appendChild(svg.path(icon));
        },

        _getHandleIcon: function (handle, d, x, y) {
            var icon = {
                "id": handle.name + "_icon", "class": "userHandle-icon", "d": d, "fill": handle.pathColor, "transform": "translate(" + x + "," + y + ")"
            };
            return icon;
        },

        _getHandlePosition: function (handle, node, scale, phsBounds, diagram) {
            if (node) {
                if (!node.segments || !handle.offset) {
                    var positionPoints = ej.datavisualization.Diagram.Point();
                    if (node.type === "phase" && phsBounds)
                        var bounds = ej.datavisualization.Diagram.Util.bounds(phsBounds);
                    else
                        var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                    var position = handle.position;
                    var distance = 10;
                    var matrix = ej.Matrix.identity();
                    if (node.rotateAngle)
                        ej.Matrix.rotate(matrix, node.rotateAngle);
                    var point = new ej.datavisualization.Diagram.Point();
                    switch (position) {
                        case "topleft":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), distance + (handle.size / 2)));
                            positionPoints.x = bounds.topLeft.x - (point.x / scale);
                            positionPoints.y = bounds.topLeft.y - (point.y / scale);
                            break;
                        case "topcenter":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(0, distance + (handle.size / 2)));
                            positionPoints.x = bounds.topCenter.x - (point.x / scale);
                            positionPoints.y = bounds.topCenter.y - point.y / scale;
                            break;
                        case "topright":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), (distance / 2) - (handle.size)));
                            positionPoints.x = bounds.topRight.x + (point.x / scale);
                            positionPoints.y = bounds.topRight.y + (point.y / scale);
                            break;
                        case "middleleft":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(-(distance + (handle.size / 2)), 0));
                            positionPoints.x = bounds.middleLeft.x + (point.x / scale);
                            positionPoints.y = bounds.middleLeft.y + (point.y / scale);
                            break;
                        case "middleright":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), 0));
                            positionPoints.x = bounds.middleRight.x + (point.x / scale);
                            positionPoints.y = bounds.middleRight.y + (point.y / scale);
                            break;
                        case "bottomleft":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(-(distance + (handle.size / 2)), distance + (handle.size / 2)));
                            positionPoints.x = bounds.bottomLeft.x + (point.x / scale);
                            positionPoints.y = bounds.bottomLeft.y + (point.y / scale);
                            break;
                        case "bottomcenter":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(0, ((distance / 2) - (handle.size))));
                            positionPoints.x = bounds.bottomCenter.x - (point.x / scale);
                            positionPoints.y = bounds.bottomCenter.y - (point.y / scale);
                            break;
                        case "bottomright":
                            point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(distance + (handle.size / 2), distance + (handle.size / 2)));
                            positionPoints.x = bounds.bottomRight.x + (point.x / scale);
                            positionPoints.y = bounds.bottomRight.y + (point.y / scale);
                            break;
                    }
                    return positionPoints;
                }
                else {
                    return this._getConnectorHandlePosition(handle, node, scale, diagram).position;
                }
            }
        },
        _getConnectorHandlePosition: function (handle, node, scale, diagram) {
            var length = 0;
            var position = ej.datavisualization.Diagram.Util._findOffsetOnConnector(node, handle, handle.offset, diagram);
            var i = position.segment;
            var j = position.point;
            var pt = position.offset;
            var targetSegment = node.segments[i]; var targetPoint = node.segments[i].points[j];
            var angle = ej.datavisualization.Diagram.Geometry.findAngle(targetPoint, node.segments[i].points[j + 1]);
            if (handle.alignment && handle.alignment != "auto") {
                if (handle.offset == 1) angle += 180;
                return { position: ej.datavisualization.Diagram.Geometry.transform(position.offset, handle.offset != 0 && handle.offset != 1 ? angle + 45 : angle, this._getAlignedPosition(node, handle)), angle: angle };
            }
            else {
                //avoid overlapping
                var obj = this._avoidOverlapWithSourceAndTarget(node, handle.offset, handle.size, position.offset, angle, i, j, diagram);
                pt = { position: obj.position, angle: handle.offset == 1 ? (180 + angle) % 360 : angle, direction: obj.direction };
            }
            return pt;
        },
        _getAlignedPosition: function (connector, handle) {
            var cnst = handle.text === undefined ? 10 : 0;
            switch (handle.alignment) {
                case "auto":
                    if (handle.offset == 0) { return -((handle.size || 0) / 2 + cnst); }
                    else { return (handle.size || 0) / 2 + cnst; }
                case "center":
                    return 0;
                case "left":
                    return -((handle.size || 0) / 2 + cnst);
                case "right":
                    return (handle.size || 0) / 2 + cnst;
            }
            return 0;
        },
        _avoidOverlapWithSourceAndTarget: function (connector, offset, size, position, angle, i, j, diagram) {
            var next, ang;
            var pt = position; var targetSegment = connector.segments[i];
            var cnst = 90;
            if (offset == 0) {
                next = connector.segments[i].points[j + 2] ? connector.segments[i].points[j + 2] : connector.segments[i + 1] ? connector.segments[i + 1].points[1] : null;
                if (next) {
                    ang = ej.datavisualization.Diagram.Geometry.findAngle(connector.segments[i].points[j], next);
                }
                if (connector.sourceNode) cnst = 45;
            }
            else if (offset == 1) {
                next = connector.segments[i].points[j - 1] ? connector.segments[i].points[j - 1] : connector.segments[i - 1] ? connector.segments[i - 1][connector.segments[i].points.length - 2] : null;
                if (next) {
                    ang = ej.datavisualization.Diagram.Geometry.findAngle(connector.segments[i].points[j + 1], next);
                }
                angle += 180;
                angle %= 360;
                if (connector.targetNode) cnst = 45;
            }
            if (next) {
                if (angle % 90 == 0) {
                    var direction;
                    if (ang > 270 && angle == 0) angle = 360;
                    if (angle == 90) {
                        if (ang < angle) { angle += cnst; direction = "right"; }
                        else { angle -= cnst; direction = "left"; }
                    }
                    else if (angle == 270) {
                        if (ang < angle) { angle += cnst; direction = "left"; }
                        else { angle -= cnst; direction = "right"; }
                    }
                    else if (angle == 180) {
                        if (ang < angle) { angle += cnst; direction = "bottom"; }
                        else { angle -= cnst; direction = "top"; }
                    }
                    else {
                        if (ang < angle) { angle += cnst; direction = "top"; }
                        else { angle -= cnst; direction = "bottom"; }
                    }
                    pt = ej.datavisualization.Diagram.Geometry.transform(pt, angle, size != undefined ? Math.max(5, size / 2 + 10) : 0);
                } else {
                    var fourty5 = 45;
                    var one35 = 135;
                    var two25 = 225;
                    var three15 = 315;
                    var bounds;
                    if (offset == 0 && connector.sourceNode || offset == 1 && connector.targetNode) {
                        var source = offset == 0 && connector.sourceNode ? diagram.nameTable[connector.sourceNode] : diagram.nameTable[connector.targetNode];
                        if (source) bounds = ej.datavisualization.Diagram.Util.bounds(source);
                        if (bounds) {
                            var part = 180 / (2 + 2 / (bounds.height / bounds.width));
                            fourty5 = part;
                            one35 = (180 - part);
                            two25 = one35 + (2 * part);
                            three15 = 360 - part;
                        }
                    }
                    if (angle >= fourty5 && angle <= one35 || angle >= two25 && angle <= three15) cnst = angle % 180 > 90 ? -cnst : cnst;
                    else if (angle > one35 && angle < two25) angle > 180 ? cnst = -45 : cnst = 45;
                    else angle > 0 ? cnst = -45 : cnst = 45;
                    pt = ej.datavisualization.Diagram.Geometry.transform(pt, angle + cnst, size != undefined ? Math.max(5, size / 2 + 10) : 0);
                }
            } else {
                var fourty5 = 45;
                var one35 = 135;
                var two25 = 225;
                var three15 = 315;
                var bounds;
                if (offset == 0 && connector.sourceNode || offset == 1 && connector.targetNode) {
                    var source = offset == 0 && connector.sourceNode ? diagram.nameTable[connector.sourceNode] : diagram.nameTable[connector.targetNode];
                    if (source) bounds = ej.datavisualization.Diagram.Util.bounds(source);
                    if (bounds) {
                        var part = 180 / (2 + 2 / (bounds.height / bounds.width));
                        fourty5 = part;
                        one35 = (180 - part);
                        two25 = one35 + (2 * part);
                        three15 = 360 - part;
                    }
                }
                if (angle >= fourty5 && angle <= one35 || angle >= two25 && angle <= three15) cnst = angle % 180 > 90 ? -cnst : cnst;
                else if (angle > one35 && angle < two25) angle > 180 ? cnst = -45 : cnst = 45;
                else angle <= fourty5 ? cnst = -45 : cnst = 45;
                angle += 360;
                angle %= 360;
                pt = ej.datavisualization.Diagram.Geometry.transform(pt, angle + cnst, size != undefined ? Math.max(5, size / 2 + (Math.abs(cnst % 45) > 0 ? 13 : 10)) : 0);
            }
            return { position: pt, direction: direction };
        },
        _getHandleShape: function (handle, cx, cy) {
            var shape = {
                "id": handle.name + "_shape", "class": "userHandle", "cx": cx, "cy": cy, "r": handle.size / 2, "fill": handle.backgroundColor,
                "stroke": handle.borderColor, "stroke-width": handle.borderWidth || 0, "title": handle.name
            };
            return shape;
        },

        _updateHandleShape: function (handle, cx, cy, svg) {
            var attr = { "id": handle.name + "_shape", "cx": cx, "cy": cy };
            svg.circle(attr);
        },

        _updateHanleIcon: function (handle, d, svg, x, y) {
            var attr = { "id": handle.name + "_icon", "d": d, "transform": "translate(" + x + "," + y + ")" };
            svg.path(attr);
        },

        _getHanleIcon: function (handle, d, cx, cy) {
            var icon = {
                "id": handle.name, "d": d, "stroke": handle.borderColor,
                "stroke-width": handle.borderWidth, "fill": handle.fillColor
            };
            return icon;
        },

        renderPivotPoint: function (node, scale, svg) {
            var x = node.width * node.pivot.x;
            var y = node.height * node.pivot.y;
            if (!svg.getElementById("pivot")) {
                ej.datavisualization.Diagram.SvgContext._renderResizeCorner("pivot", node,
                      x * scale, y * scale, svg, undefined, 4);
            }
            else ej.datavisualization.Diagram.SvgContext._updateResizeCorner("pivot",
                      x * scale, y * scale, svg);
            this._updatePivotLine(node, svg, scale, true);
        },

        _removePivotPoint: function (node, svg, scale) {
            var pivot = svg.getElementById("pivot");
            if (pivot) {
                pivot.parentNode.removeChild(pivot);
                this._updatePivotLine(node, svg, scale, false);
            }
        },

        _renderPivotLine: function (node, svg, scale) {
            if (!node.container) {
                var width = node.width ? node.width : node._width;
                var height = node.height ? node.height : node._height;
                var rotateThumb = ej.datavisualization.Diagram.Point(width * node.pivot.x, -20 / scale);
                var pivot = ej.datavisualization.Diagram.Point(width * node.pivot.x, 0);
                var handle = svg.getElementById(svg.document.id + "handle_g");
                var line = svg.line({
                    "id": "pivotLine", "x1": rotateThumb.x * scale, "y1": rotateThumb.y * scale, "x2": pivot.x * scale, "y2": pivot.y * scale,
                    "stroke": "black", "stroke-width": 1, "stroke-dasharray": "2,3", "fill": "none"
                });
                handle.appendChild(line);
            }
        },

        _renderResizeBorder: function (id, node, svg, scale) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var rect = svg.rect({
                "id": id, "width": width * scale, "height": height * scale, "stroke": "#097F7F", "stroke-width": 0.6, "stroke-dasharray": "6,3", "fill": "none", "pointer-events": "none"
            });
            handle.appendChild(rect);
        },

        _renderResizeCorner: function (corner, node, cx, cy, svg, state, r) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var pEvnts;
            var fill = "white";
            if (!(ej.datavisualization.Diagram.Util.canResize(node))) {
                fill = "darkgray";
            }
            if (state) {
                fill = "darkgray";
                pEvnts = "none";
            }
            var attr = { "id": corner, "fill": fill, "stroke": "black", "stroke-width": 1, "cx": cx, "cy": cy, "r": r || 7 };
            handle.appendChild(svg.circle(attr));
            attr = { "id": corner + "_transparent", "class": corner, "fill": "transparent", "cx": cx, "cy": cy, "r": r || 10 };
            if (pEvnts) attr["pointer-events"] = pEvnts;
            handle.appendChild(svg.circle(attr));
        },

        _renderRotateThumb: function (corner, node, cx, cy, scale, svg) {
            if (!node.container) {
                var handle = svg.getElementById(svg.document.id + "handle_g");
                var d = "M 16.856 10.239 L 18 3.438 l -2.189 0.817 c -0.974 -1.694 -2.482 -2.995 -4.326 -3.696 C 9.375 -0.245 7.071" +
                    "-0.18 5.01 0.744 C 2.942 1.668 1.365 3.342 0.558 5.453 c -0.803 2.115 -0.738 4.414 0.185 6.478 c 0.925 2.064 " +
                "2.6 3.645 4.714 4.45 c 0.969 0.371 1.993 0.554 3.013 0.554 c 1.345 0 2.685 -0.317 3.897 -0.948 l -1.016 -1.962 c -1.584 0.822" +
                "-3.445 0.929 -5.114 0.293 c -1.56 -0.596 -2.793 -1.762 -3.479 -3.289 C 2.078 9.502 2.031 7.803 2.622 6.244 C 3.216 4.68 4.387" +
                " 3.443 5.914 2.761 C 7.437 2.079 9.137 2.03 10.7 2.626 c 1.246 0.475 2.271 1.328 2.986 2.424 L 11.54 5.851 L 16.856 10.239 Z";
                var fill = "#231f20";
                if (!(ej.datavisualization.Diagram.Util.canRotate(node)))
                    fill = "darkgray";
                var width = 20;
                var height = 20;
                var dist = 20;
                var attr = {
                    "id": "rotatehandle", "class": corner, "fill": fill, "stroke": "black", "stroke-width": 0.5, "width": width, "height": height, "d": d,
                    "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
                };
                var path = svg.path(attr);
                var circle = svg.circle({
                    "id": "rotate", "class": corner, "fill": "transparent", "stroke": "none", "cx": width / 2, "cy": height / 2, "r": 13,
                    "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
                });
                handle.appendChild(path);
                handle.appendChild(circle);
            }
        },

        _refreshEndPointHandles: function (shape, svg, scale) {
            if (shape.segments) {
                var handle = svg.getElementById(svg.document.id + "handle_g");
                $(handle).empty();
                this._renderEndPointHandle(shape, svg, scale);
            }
        },
        _renderEndPointHandle: function (connector, svg, scale) {
            if (!connector.isPhase) {
                var enabled = ej.datavisualization.Diagram.Util.canDragSegmentThumbs(connector);
                for (var i = 0; i < connector.segments.length; i++) {
                    var segment = connector.segments[i];
                    if (segment.type == "bezier") {
                        this._renderBezierLine("bezierline1_" + i, segment._startPoint, segment._point1, scale, svg, enabled);
                        this._renderBezierLine("bezierline2_" + i, segment._endPoint, segment._point2, scale, svg, enabled);
                        this._renderEndPointCorner("bezierpoint1_" + i, segment._point1.x * scale, segment._point1.y * scale, false, svg, enabled);
                        this._renderEndPointCorner("bezierpoint2_" + i, segment._point2.x * scale, segment._point2.y * scale, false, svg, enabled);
                    }
                    var points = segment.points;
                    if (i == 0) {
                        this._renderEndPointCorner("sourceEndPoint", segment._startPoint.x * scale, segment._startPoint.y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                              ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                    }
                    if (i == connector.segments.length - 1) {
                        this._renderEndPointCorner("targetEndPoint", segment._endPoint.x * scale, segment._endPoint.y * scale, ej.datavisualization.Diagram.Util.isTargetConnected(connector), svg,
                            ej.datavisualization.Diagram.Util.canDragTargetEnd(connector));
                        if (segment.type == "orthogonal") {
                            this._renderTerminalOrthoThumbs(segment, svg, scale, i, enabled);
                        }
                    }
                    else {
                        if (segment.type != 'orthogonal') {
                            this._renderSegmentEndThumb("segmentEnd_" + i, segment._endPoint.x * scale, segment._endPoint.y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                                   ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                        } else {
                            this._renderTerminalOrthoThumbs(segment, svg, scale, i, enabled);
                        }

                    }
                }
            }
        },

        _renderTerminalOrthoThumbs: function (segment, svg, scale, index, isEnabled) {
            var orientation;
            if (segment.points.length > 2)
                for (var i = 0; i < segment.points.length - 1; i++) {
                    var length = ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]);
                    if (segment.points[i].y == segment.points[i + 1].y) {
                        orientation = "horizontal";
                    }
                    else orientation = "vertical";
                    var visible;
                    if (length >= 50) visible = "visible"; else visible = "hidden";
                    this._renderOrthogonalThumb(("OrthoThumb_" + i) + "_" + index, ((segment.points[i].x + segment.points[i + 1].x) / 2) * scale, ((segment.points[i].y + segment.points[i + 1].y) / 2) * scale, svg, visible, orientation, isEnabled);
                }
            else {
                var length = segment.length || ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                if (length >= 50) visible = "visible"; else visible = "hidden";
                if (segment._startPoint.y == segment._endPoint.y) {
                    orientation = "horizontal";
                }
                else orientation = "vertical";
                this._renderOrthogonalThumb("OrthoThumb_" + index, ((segment._endPoint.x + segment._startPoint.x) / 2) * scale, ((segment._endPoint.y + segment._startPoint.y) / 2) * scale, svg, visible, orientation, isEnabled);
            }
        },

        _renderEndPointCorner: function (corner, x, y, isConnected, svg, isenabled) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = isConnected ? "#8CC63F" : "white";
            if (!isenabled) fill = "darkgray";
            var attr = { "id": corner, "class": corner, opacity: 0.75, "fill": fill, "stroke": "black", "stroke-width": 2, "cx": x, "cy": y, "r": 7 };
            handle.appendChild(svg.circle(attr));
        },

        _renderSegmentEndThumb: function (corner, x, y, isConnected, svg, isenabled) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = "#e2e2e2";
            if (!isenabled) fill = "darkgray";
            var attr = { "id": corner, "class": "segmentEnd", "fill": fill, "stroke": "black", "stroke-width": 2, "cx": x, "cy": y, "r": 5 };
            handle.appendChild(svg.circle(attr));
        },
        _renderOrthogonalThumb: function (corner, x, y, svg, visibility, orientation, isEnabled) {
            var path, h, v; if (orientation == "horizontal") {
                path = "M0,7 L15,0 L30,7 L15,14 z"; h = -15; v = -7;
            } else { path = "M7,0 L0,15 L7,30 L14,15 z"; h = -7; v = -15; };
            var handle = svg.getElementById(svg.document.id + "handle_g");
            var fill = "#e2e2e2";
            if (!isEnabled) fill = "darkgray";
            var attr = {
                "id": corner, "class": "segmentEnd", "fill": fill, "stroke": "black", "stroke-width": 1, d: path, transform: "translate(" + (x + h) + "," + (y + v) + ")",
                "visibility": visibility
            };
            handle.appendChild(svg.path(attr));
        },
        _renderBezierLine: function (id, start, end, scale, svg) {
            var line = svg.line({
                "id": id, "x1": start.x * scale, "y1": start.y * scale, "x2": end.x * scale, "y2": end.y * scale,
                "stroke": "black", "stroke-width": 1, "stroke-dasharray": "3,3", "fill": "none"
            });
            var handle = svg.getElementById(svg.document.id + "handle_g");
            handle.appendChild(line);
        },
        _renderControlPointCorner: function (corner, point, index, svg) {
            var g = svg.g({ "class": corner });
            svg.getElementById(svg.document.id + "handle_g").appendChild(g);
            var attr = {
                "id": corner + "_" + index, "fill": "white", "stroke": "black", "stroke-width": 2,
                "cx": point.x, "cy": point.y, "r": 5
            };
            g.appendChild(svg.circle(attr));
        },
        updateSelector: function (shape, svg, scale, diagram, constraints) {
            if (shape) {
                var width = shape.width ? shape.width : shape._width;
                var height = shape.height ? shape.height : shape._height;
                if (shape.segments) {
                    this._updateEndPointHandle(shape, svg, scale);
                    if (constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles)
                        this.updateUserHandles(diagram.model.selectedItems.userHandles, shape, svg, false, false, scale, diagram);
                } else if (shape.type === "phase") {
                    var bounds = diagram._getPhaseBounds(shape);
                    svg.g({
                        "id": svg.document.id + "handle_g", "class": "handle", "pointer-events": "visible",
                        "transform": "translate(" + (bounds.x) * scale + "," + (bounds.y) * scale + "),rotate(0," + (bounds.width / 2) * scale + "," + (bounds.height / 2) * scale + ")"
                    });
                    svg.rect({
                        "id": "phaseSelector", "class": "phaseSelector",
                        "stroke-width": 2, "stroke-dasharray": "3,3", "pointer-events": "none",
                        "width": (bounds.width < 0) ? 1 : bounds.width * scale, "height": (bounds.height < 0) ? 1 : bounds.height * scale, "fill": "transparent", "stroke": "red",
                    });
                } else {
                    if (!(shape.isSwimlane || shape.isLane)) {
                        var g = svg.g({
                            "id": svg.document.id + "handle_g",
                            "transform": "translate(" + (shape.offsetX - width * shape.pivot.x) * scale + "," + (shape.offsetY - height * shape.pivot.y) * scale + "),rotate(" + shape.rotateAngle + "," + (width * shape.pivot.x) * scale + "," + (height * shape.pivot.y) * scale + ")"
                        });
                    }
                    var type = diagram.getObjectType(shape);
                    if (type === "group" || shape.type === "pseudoGroup" || !((diagram.activeTool.name == "move" || diagram.activeTool.name == "rotate") && diagram.activeTool.inAction)) {
                        this._updateResizeHandle(shape, svg, scale, constraints);
                    }
                    if (diagram.activeTool.name == "rotatetool" && diagram.activeTool.inAction) {
                        this.renderPivotPoint(shape, scale, svg);
                    }
                    if (diagram.model.selectedItems.userHandles && diagram.model.selectedItems.userHandles.length > 0) {
                        var isMultipleSelection = false;
                        if (diagram.selectionList[0] && diagram.selectionList[0].name == "multipleSelection")
                            isMultipleSelection = true;
                        if (constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles) {
                            this.updateUserHandles(diagram.model.selectedItems.userHandles, shape, svg, isMultipleSelection, false, scale, diagram);
                            return true;
                        }
                    }
                }
            }
        },

        _updatePhaseResize: function (node, svg, scale, constraints) {

            var nPoint, points = [], attr;
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);

            //#region left
            nPoint, points = [];
            points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
            points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
            nPoint = this._convertToSVGPoints(points);
            attr = { "id": "w-resize", "points": nPoint, };
            svg.polyline(attr);
            //#endregion  

            //#region top
            nPoint, points = [];
            points.push({ x: bounds.topLeft.x * scale, y: bounds.topLeft.y * scale });
            points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
            nPoint = this._convertToSVGPoints(points);
            attr = { "id": "n-resize", "points": nPoint };
            svg.polyline(attr);
            //#endregion 

            //#region right 
            nPoint, points = [];
            points.push({ x: bounds.topRight.x * scale, y: bounds.topRight.y * scale });
            points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
            nPoint = this._convertToSVGPoints(points);
            attr = { "id": "e-resize-hitTest", "points": nPoint };
            svg.polyline(attr);
            attr = { "id": "e-resize", "points": nPoint };
            svg.polyline(attr);
            //#endregion


            //#region bottom
            nPoint, points = [];
            points.push({ x: bounds.bottomLeft.x * scale, y: bounds.bottomLeft.y * scale });
            points.push({ x: bounds.bottomRight.x * scale, y: bounds.bottomRight.y * scale });
            nPoint = this._convertToSVGPoints(points);
            attr = { "id": "s-resize-hitTest", "points": nPoint, };
            svg.polyline(attr);
            attr = { "id": "s-resize", "points": nPoint, };
            svg.polyline(attr);
            //#endregion

        },
        _updateResizeHandle: function (node, svg, scale, constraints) {
            if (!(node.isSwimlane || node.isLane)) {
                var width = node.width ? node.width : node._width;
                var height = node.height ? node.height : node._height;
                var corner = node._corners;
                var rotate = ej.datavisualization.Diagram.Point(width * node.pivot.x, -20 / scale);
                if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Rotator) {
                    this._updateRotateHandle("rotate", rotate.x, rotate.y, scale, svg, node);
                    this._updatePivotLine(node, svg, scale);
                }
                //this._updateResizeCorner("pivot", node.offsetX * scale, node.offsetY * scale, svg);
                if (constraints & ej.datavisualization.Diagram.SelectorConstraints.Resizer) {
                    this._updateResizeCorner("nw-resize", 0, 0, svg);
                    this._updateResizeCorner("n-resize", (width / 2) * scale, 0, svg);
                    this._updateResizeCorner("ne-resize", width * scale, 0, svg);
                    this._updateResizeCorner("w-resize", 0, (height / 2) * scale, svg);
                    this._updateResizeCorner("e-resize", width * scale, (height / 2) * scale, svg);
                    this._updateResizeCorner("sw-resize", 0, height * scale, svg);
                    this._updateResizeCorner("s-resize", (width / 2) * scale, height * scale, svg);
                    this._updateResizeCorner("se-resize", width * scale, height * scale, svg);
                    this._updateResizeBorder("resizeBorder", node, svg, scale);
                }
            }
            else this._updatePhaseResize(node, svg, scale, constraints);
        },
        _updateRotateHandle: function (thumb, cx, cy, scale, svg, node) {
            var width = 20;
            var height = 20;
            var dist = 20;
            var attr = {
                "id": "rotatehandle",
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            };
            svg.path(attr);
            svg.circle({
                "id": "rotate",
                "transform": "translate(" + (cx * scale - width / 2) + "," + (cy * scale - height) + ")"
            });
        },

        _updatePivotLine: function (node, svg, scale, considerpivot) {
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var rotateThumb = ej.datavisualization.Diagram.Point(width * node.pivot.x, -20 / scale);
            var pivot;
            if (considerpivot) {
                pivot = ej.datavisualization.Diagram.Point(width * node.pivot.x, height * node.pivot.y);
            } else pivot = ej.datavisualization.Diagram.Point(width * node.pivot.x, 0);
            var line = svg.line({
                "id": "pivotLine", "x1": rotateThumb.x * scale, "y1": rotateThumb.y * scale, "x2": pivot.x * scale, "y2": pivot.y * scale
            });
        },
        _updateResizeBorder: function (id, node, svg, scale) {
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            svg.rect({
                "id": id, "width": width * scale, "height": height * scale,
            });
        },

        _updateResizeCorner: function (corner, cx, cy, svg) {
            var attr = { "id": corner, "cx": cx, "cy": cy };
            svg.circle(attr);
            var attr = { "id": corner + "_transparent", "cx": cx, "cy": cy };
            svg.circle(attr);
        },

        _updateEndPointHandle: function (connector, svg, scale) {
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                var points = segment.points;
                if (i == connector.segments.length - 1) {
                    this._updateEndPointCorner("targetEndPoint", points[points.length - 1].x * scale, points[points.length - 1].y * scale, ej.datavisualization.Diagram.Util.isTargetConnected(connector), svg,
                ej.datavisualization.Diagram.Util.canDragTargetEnd(connector));
                    if (segment.type == "orthogonal") {
                        this._updateTerminalOrthoThumbs(segment, svg, scale, i);
                    }
                }
                else {
                    if (segment.type != "orthogonal") {
                        this._updateSegmentEndThumb("segmentEnd_" + i, segment._endPoint.x * scale, segment._endPoint.y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                            ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                    }
                    else {
                        this._updateTerminalOrthoThumbs(segment, svg, scale, i);
                    }
                }
                if (i == 0)
                    this._updateEndPointCorner("sourceEndPoint", points[0].x * scale, points[0].y * scale, ej.datavisualization.Diagram.Util.isSourceConnected(connector), svg,
                        ej.datavisualization.Diagram.Util.canDragSourceEnd(connector));
                if (segment.type == "bezier") {
                    var enabled = ej.datavisualization.Diagram.Util.canDragSegmentThumbs(connector);
                    this._updateEndPointCorner("bezierpoint1_" + i, segment._point1.x * scale, segment._point1.y * scale, false, svg, enabled);
                    this._updateEndPointCorner("bezierpoint2_" + i, segment._point2.x * scale, segment._point2.y * scale, false, svg, enabled);
                    this._updateBezierLine("bezierline1_" + i, segment._startPoint, segment._point1, scale, svg);
                    this._updateBezierLine("bezierline2_" + i, segment._endPoint, segment._point2, scale, svg);
                }

            }
            //if (points.length > 2) {
            //    for (var i = 1, len = points.length; i < len - 1; ++i) {
            //        this._updateControlPointCorner("controlPoint", points[i], i, svg);
            //    }
            //}
        },

        _updateEndPointCorner: function (corner, x, y, isConnected, svg, isenabled) {
            var fill = isConnected ? "#8CC63F" : "white";
            if (!isenabled) fill = "darkgray";
            var attr = { "id": corner, "fill": fill, "cx": x, "cy": y };
            svg.circle(attr);
        },
        _updateSegmentEndThumb: function (corner, x, y, isConnected, svg, isenabled) {
            var attr = { "id": corner, "cx": x, "cy": y };
            svg.circle(attr);
        },
        _updateOrthoThumb: function (corner, x, y, svg, visibility, orientation) {
            var path, h, v; if (orientation == "horizontal") {
                path = "M0,7 L15,0 L30,7 L15,14 z"; h = -15; v = -7;
            } else { path = "M7,0 L0,15 L7,30 L14,15 z"; h = -7; v = -15; };
            var attr = { "id": corner, visibility: visibility, transform: "translate(" + (x + h) + "," + (y + v) + ")", d: path };
            svg.path(attr);
        },
        _updateTerminalOrthoThumbs: function (segment, svg, scale, index) {
            var orientation;
            if (segment.points.length > 2)
                for (var i = 0; i < segment.points.length - 1; i++) {
                    var length = ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]);
                    if (segment.points[i].y == segment.points[i + 1].y) {
                        orientation = "horizontal";
                    }
                    else orientation = "vertical";
                    var visible;
                    if (length >= 50) visible = "visible"; else visible = "hidden";
                    this._updateOrthoThumb(("OrthoThumb_" + i) + "_" + index, ((segment.points[i].x + segment.points[i + 1].x) / 2) * scale, ((segment.points[i].y + segment.points[i + 1].y) / 2) * scale, svg, visible, orientation);

                }
            else {
                var length = segment.length || ej.datavisualization.Diagram.Geometry.distance(segment._startPoint, segment._endPoint);
                if (segment._startPoint.y == segment._endPoint.y) {
                    orientation = "horizontal";
                }
                else orientation = "vertical";
                if (length >= 50) visible = "visible"; else visible = "hidden";
                this._updateOrthoThumb("OrthoThumb_" + index, ((segment._endPoint.x + segment._startPoint.x) / 2) * scale, ((segment._endPoint.y + segment._startPoint.y) / 2) * scale, svg, visible, orientation);
            }
        },
        _updateBezierLine: function (id, start, end, scale, svg) {
            var line = svg.line({
                "id": id, "x1": start.x * scale, "y1": start.y * scale, "x2": end.x * scale, "y2": end.y * scale
            });
        },
        _updateControlPointCorner: function (corner, point, index, svg) {
            svg.circle({ "id": corner + "_" + index, "cx": point.x, "cy": point.y });
        },

        updateUserHandles: function (handles, node, svg, isMultipleSelection, isDragging, scale, diagram) {
            var userHandle = svg.document.getElementById(svg.document.id + "userHandle_g");
            if (diagram.model.selectedItems.constraints & ej.datavisualization.Diagram.SelectorConstraints.UserHandles) {
                if (!userHandle) {
                    this.renderUserHandles(handles, node, svg, isMultipleSelection, scale, diagram._adornerLayer, diagram);
                } else {
                    if (handles && handles.length > 0) {
                        for (var handleIndex = 0; handleIndex < handles.length; handleIndex++) {
                            if ((isMultipleSelection && handles[handleIndex].enableMultiSelection) || !isMultipleSelection)
                                this._updateHandle(handles[handleIndex], node, svg, isDragging, scale, diagram);
                        }
                    }
                }
            }
        },

        _updateHandle: function (handle, node, svg, isDragging, scale, diagram) {
            if (!isDragging && handle.visible) {
                var position = this._getHandlePosition(handle, node, scale, null, diagram);
                var d = handle._absolutePath;
                if (!d) d = handle._absolutePath = ej.datavisualization.Diagram.Geometry.updatePath(0, 0, handle.size / 2, handle.size / 2, handle.pathData, svg);
                var x = position.x * scale;// - (node.segments ? 0 : handle.size / 4);
                var y = position.y * scale;// - (node.segments ? 0 : handle.size / 4)
                this._updateHandleShape(handle, x, y, svg);
                this._updateHanleIcon(handle, d, svg, x - handle.size / 4, y - handle.size / 4);
                if (svg.getElementById(handle.name + "_shape") && handle.visible) {
                    svg.getElementById(handle.name + "_shape").style.display = "block";
                    svg.getElementById(handle.name + "_icon").style.display = "block";
                }
            }
            else {
                if (svg.getElementById(handle.name + "_shape")) {
                    svg.getElementById(handle.name + "_shape").style.display = "none";
                    svg.getElementById(handle.name + "_icon").style.display = "none";
                }
            }
        },

        clearSelector: function (svg, parent) {
            var handle = svg.getElementById(svg.document.id + "handle_g");
            if (handle)
                parent.removeChild(handle);
            handle = svg.getElementById(svg.document.id + "userHandle_g");
            if (handle)
                parent.removeChild(handle);
        },

        _initializePageBreaks: function (svg, attr, g) {
            var g1 = svg.g(attr);
            g.appendChild(g1);
            return g1;
        },

        _renderVPageBreakLine: function (start, end, svg, g) {
            var line = svg.line({
                "x1": start.x, "y1": start.y,
                "x2": end.x, "y2": end.y,
                "style": "stroke:#aaaaaa;stroke-width:1;stroke-dasharray:10,10 "
            });
            g.appendChild(line);
        },

        _renderPageSettingsRect: function (start, end, svg, g, _stroke, _fill, _strokewidth) {
            var rect = svg.rect({
                "id": "pageback",
                "x": start.x, "y": start.y, "width": end.x, "height": end.y,
                "style": "fill:" + _fill + ";stroke:" + _stroke + ";stroke-width:" + _strokewidth + ""
            });
            g.appendChild(rect);
        },

        _updatePageBakground: function (start, end, svg, g, _stroke, _fill, _strokewidth) {
            var rect = svg.document.getElementById("pageback");
            var attr = {
                "x": start.x, "y": start.y, "width": (end.x < 0) ? 0 : end.x, "height": (end.y < 0) ? 0 : end.y,
                "style": "fill:" + _fill + ";stroke:" + _stroke + ";stroke-width:" + _strokewidth + ""
            };
            if (rect)
                ej.datavisualization.Diagram.Util.attr(rect, attr);
        },

        _renderHPageBreakLine: function (start, end, svg, g) {
            var line = svg.line({
                "x1": start.x, "y1": start.y,
                "x2": end.x, "y2": end.y,
                "style": "stroke:#aaaaaa;stroke-width:1;stroke-dasharray:10,10 "
            });
            g.appendChild(line);
        },

        _removePageBreaks: function (svg, g, layer) {
            if (g != null) {
                if (layer != null) {
                    layer.removeChild(g);
                }
            }
        },
        removeChild: function (element, view) {
            var elementTar = view.svg.document.getElementById(element.name);
            var htmlLayer = view.svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
            var label = this._findChild(htmlLayer.childNodes, element);
            if (elementTar)
                elementTar.parentNode.removeChild(elementTar);
            if (label)
                htmlLayer.removeChild(label);
            if (element._type === "node" && element.type === "html" && ej.browserInfo().name === "msie") {
                var htmlelement = htmlLayer.children[element.name + "_parentdiv"];
                if (htmlelement)
                    htmlelement.parentNode.removeChild(htmlelement);
            }
        }
        //#endregion
    };
    //#endregion

    //#region Common Renderer
    ej.datavisualization.Diagram.DiagramContext = {

        _renderNodes: function (view, isLoad, isoverView, diagram) {
            if (diagram.nodes()) {
                var nodes = diagram.nodes();
                for (var i = 0, len = nodes.length; i < len; ++i) {
                    var node = diagram.nameTable[nodes[i].name];
                    var panel = view.svg || view._canvas;
                    this._renderNodeObject(node, view, isLoad, isoverView, diagram);
                }
            }
        },

        _renderNodeObject: function (node, view, isLoad, isoverView, diagram) {
            var panel = view.svg || view._canvas;
            if (node._type === "group" && !node.parent) {
                if (!node.container) {
                    ej.datavisualization.Diagram.Util._updateGroupBounds(node, diagram, true);
                }
                view.context.renderGroup(node, panel, view.diagramLayer, diagram.nameTable, diagram, isLoad, isoverView);
                ej.datavisualization.Diagram.Util._updateGroupBounds(node, diagram);
                ej.datavisualization.Diagram.DiagramContext.update(node, diagram);
            }
            else {
                if (!node.parent)
                    view.context.renderNode(node, panel, view.diagramLayer, null, diagram);
            }
        },

        _renderConnectors: function (view, diagram) {
            if (diagram.connectors()) {
                var connectors = diagram.connectors();
                var panel = view.svg || view._canvas;
                var connector;
                for (var i = 0, len = connectors.length; i < len; ++i) {
                    connector = connectors[i];
                    diagram._setZorder(connector);
                }
                for (var i = 0, len = connectors.length; i < len; ++i) {
                    connector = connectors[i];
                    this._renderConnectorObject(connector, view, diagram);
                }
            }
        },

        _renderConnectorObject: function (connector, view, diagram) {
            var panel = view.svg || view._canvas;
            ej.datavisualization.Diagram.Util.updateBridging(connector, diagram);
            if (connector.parent && connector.parent != "") {
                var parent = diagram._svg.document.getElementById(connector.parent)
                if (parent)
                    view.context.renderConnector(connector, panel, parent, diagram);
            }
            else
                view.context.renderConnector(connector, panel, view.diagramLayer, diagram);
        },

        updateViewPort: function (diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                if (view.context == ej.datavisualization.Diagram.SvgContext && view.type == "mainview") {
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
                }
            });
        },
        renderNode: function (node, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                if (parent)
                    parent = panel.document.getElementById(parent.id)
                view.context.renderNode(node, panel, parent || view.diagramLayer, undefined, diagram);
            });
        },

        renderPort: function (node, port, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._insertPort(node, port, panel);
            });
        },
        renderConnector: function (connector, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context.renderConnector(connector, panel, parent || view.diagramLayer, diagram);
            });
        },

        renderGroup: function (group, diagram, parent) {
            var view, panel;

            diagram._views.forEach(function (viewid) {
                var isOverView
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                parent = view.svg.getElementById(group.parent);
                if (view.type == "overview") {
                    isOverView = true;
                }
                view.context.renderGroup(group, panel, parent || view.diagramLayer, diagram.nameTable, diagram, null, isOverView);
            });
        },
        update: function (data, diagram, layout) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                if (data._type === "group" || data.type === "pseudoGroup")
                    view.context.updateGroup(data, panel, diagram, layout);
                else if (data.segments)
                    view.context.updateConnector(data, panel, diagram);
                else
                    view.context.updateNode(data, panel, diagram, layout);
            });
        },
        _refreshSegments: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                if (connector.segments)
                    view.context._refreshSegments(connector, panel, diagram);
            });
        },
        renderShadow: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._renderShadow(node, panel);
            });
        },
        updateShadow: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._updateShadow(node, panel);
            });
        },
        removeShadow: function (node, diagram) {
            var view, panel;

            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view._canvas;
                view.context._removeShadow(node, panel);
            });
        },
        addNodeLabel: function (shape, label, parent, diagram) {
            var view, panel;
            if (diagram._views) {
                diagram._views.forEach(function (viewid) {
                    view = diagram._views[viewid];
                    panel = view.svg || view.canvas;
                    view.context.addNodeLabel(shape, label, panel, parent, diagram);
                });
            }
        },

        updateLabel: function (node, label, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateLabel(node, label, panel, diagram);
            });
        },

        updatePort: function (node, port, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updatePort(node, port, panel, diagram);
            });
        },

        setNodeShape: function (node, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.setNodeShape(node, panel, parent, diagram);
            });
        },

        setLine: function (connector, diagram, parent) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.setLine(connector, panel, parent, diagram);
            });
        },

        renderDecorators: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.renderDecorators(connector, panel, diagram);
            });
        },

        clearDecorators: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.clearDecorators(connector, panel, diagram);
            });
        },

        updateTargetDecoratorStyle: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateTargetDecoratorStyle(node, panel);
            });
        },

        updateSourceDecoratorStyle: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateSourceDecoratorStyle(node, panel);
            });
        },

        updateNodeStyle: function (node, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context._updateNodeStyle(node, panel);
            });
        },

        updateLabelStyle: function (node, label, diagram) {
            diagram._views.forEach(function (viewid) {
                var view = diagram._views[viewid];
                var panel = view.svg || view.canvas;
                view.context.updateLabelStyle(node, label, panel);
            });
        },

        updateConnectorStyle: function (connector, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context._updateConnectorStyle(connector, panel);
            });
        },

        updateTextBlock: function (node, label, diagram) {
            var view, panel;
            diagram._views.forEach(function (viewid) {
                view = diagram._views[viewid];
                panel = view.svg || view.canvas;
                view.context.updateTextBlock(node, label, panel);
            });

        },

        updateBPMNNodeStyle: function (node, diagram) {
            var child, children, i = 0;
            node = ej.datavisualization.Diagram.BPMNShape(node, diagram);
            if (node.children) {
                children = diagram._getChildren(node.children);
                for (i = 0; i < children.length; i++) {
                    child = diagram.nameTable[children[i]];
                    ej.datavisualization.Diagram.DiagramContext.updateNodeStyle(child, diagram, node);
                }
            } else ej.datavisualization.Diagram.DiagramContext.updateNodeStyle(node, diagram);
        },

        updateBPMNNodeShape: function (node, diagram) {
            var child, children, i = 0;
            node = ej.datavisualization.Diagram.BPMNShape(node, diagram);
            children = diagram._getChildren(node.children);
            if (children.length > 0 && diagram.nameTable[children[0]]) {
                for (i = 0; i < children.length; i++) {
                    child = diagram.nameTable[children[i]];
                    if (child)
                        ej.datavisualization.Diagram.DiagramContext.setNodeShape(child, diagram, node);
                }
            }
            else {
                var element = diagram._svg.getElementById(node.name);
                element.parentNode.removeChild(element);
                diagram.add(node);
            }
        }
    };
    //#endregion
})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html SymbolPalette elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    "use strict";
    //#region SymbolPalette widget
    ej.widget("ejSymbolPalette", "ej.datavisualization.SymbolPalette", {
        element: null,
        model: null,
        nameTable: {},
        //#region Initialization
        defaults: {
            width: 250,
            height: 400,
            paletteItemWidth: undefined,
            paletteItemHeight: undefined,
            previewWidth: undefined,
            previewHeight: undefined,
            previewOffset: { x: 110, y: 110 },
            diagramId: null,
            headerHeight: 30,
            selectedPaletteName: 0,
            cssClass: "e-datavisualization-symbolpalette",
            showPaletteItemText: true,
            allowDrag: true,
            palettes: [],
            defaultSettings: {
                connector: null,
                node: null
            },
            selectionChange: null,
        },
        _oldItem: null,
        dataTypes: {
            paletteItemWidth: "number",
            paletteItemHeight: "number",
            previewWidth: "number",
            previewHeight: "number",
            headerHeight: "number",
            showPaletteItemText: "boolean",
            allowDrag: "boolean",
            palettes: "array",
        },
        _canvas: null,
        _init: function () {
            if (!(ej.browserInfo().name === "msie" && Number(ej.browserInfo().version) < 9)) {
                this.selectedItem = null;
                this.activePalette = null;
                this._selectedElement = null;
                this._renderSymbolPalette();
                this._wireEvents();
                this.updateScrollerViewport();
            }
        },
        _setModel: function (options) {
            for (var option in options) {
                switch (option) {
                    case "width":
                        this._setWidth(options[option]);
                        break;
                    case "height":
                        this._setHeight(options[option]);
                        break;
                    case "paletteItemWidth":
                        this.model.paletteItemWidth = options[option];
                        this._redrawContent();
                        break;
                    case "paletteItemHeight":
                        this.model.paletteItemHeight = options[option];
                        this._redrawContent();
                        break;
                    case "headerHeight":
                        this._setHeaderHeight(options[option]);
                        break;
                    case "cssClass":
                        this._setCssClass(options[option]);
                        break;
                    case "showPaletteItemText":
                        this._showItemText(options[option]);
                        break;
                    case "allowDrag":
                        this._setAllowDrag(options[option]);
                        break;
                }
            }
        },
        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },
        //#endregion
        //#region Rendering
        _renderSymbolPalette: function () {
            this._canvas = document.createElement("div");
            this._canvas.setAttribute("class", "e-scanvas");
            this._canvas.setAttribute("id", this.element[0].id + "_canvas");
            this.element[0].setAttribute("style", "height:" + this.model.height + "; width:" + this.model.width + ";");
            this.element.append(this._canvas);
            this.element.ejScroller({ width: this._setViewPortWidth(), height: this._setViewPortHeight(), enableTouchScroll: false });
            var palettes = this.model.palettes;
            if (palettes) {
                for (var i = 0, len = palettes.length; i < len; i++) {
                    palettes[i] = ej.datavisualization.Diagram.Palette(palettes[i], this);
                    this._updateNodeWidth(palettes[i]);
                    this._renderPalette(palettes[i]);
                }
            }
        },
        _updateNodeWidth: function (palette) {
            if (palette && palette.items && palette.items.length > 0) {
                var item = null;
                for (var i = 0; palette.items && i < palette.items.length ; i++) {
                    item = palette.items[i];
                    this._updatePaletteItemWidth(item);
                }
            }
        },
        _updatePaletteItemWidth: function (item) {
            var bounds = ej.datavisualization.Diagram.Util.bounds(item);
            if (!item.segments) {
                if (item.width > (this.element.width() - 40)) {
                    item.width = this.element.width() - 40;
                }
                else if (item.paletteItem && item.paletteItem.width && item.paletteItem.width > (this.element.width() - 40)) {
                    item.paletteItem.width = this.element.width() - 40;
                }
            }
            else {
                if (bounds.width > (this.element.width() - 40) && !item.paletteItem) {
                    ej.datavisualization.Diagram.Util.scale(item, (this.element.width() - 45) / bounds.width, 1, bounds.center);
                }
            }
        },
        _renderHtmlLayer: function (canvas, parent) {
            var div = document.createElement("div");
            var attr = { "id": canvas.id + "_htmlLayer", "class": "htmlLayer" };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            var htmlLayer = div;
            div.style.pointerEvents = "none";
            div.style.position = "absolute";
            div.style.left = parent ? parent.style.paddingLeft : "0px";
            div.style.top = parent ? parent.style.paddingTop : "0px";
            canvas.appendChild(htmlLayer);
            return htmlLayer;
        },
        _initGroupNode: function (group) {
            var child = null;
            var nodeDefault = this.model.defaultSettings && this.model.defaultSettings.node ? this.model.defaultSettings.node : null;
            var connectorDefault = this.model.defaultSettings && this.model.defaultSettings.connector ? this.model.defaultSettings.connector : null;
            for (var i = 0; i < group.children.length; i++) {
                child = group.children[i];
                if (typeof (child) == "object") {
                    if (child._type == "group") {
                        child = ej.datavisualization.Diagram.Group($.extend(true, {}, nodeDefault, child));
                        palette._initGroupNode(child);
                    }
                    else if (child.segments)
                        child = ej.datavisualization.Diagram.Connector($.extend(true, {}, connectorDefault, child));
                    else
                        child = ej.datavisualization.Diagram.Node($.extend(true, {}, nodeDefault, child));
                    child = ej.datavisualization.Diagram.NodeType(child, this);
                    child.parent = group.name;
                    group.children[i] = child.name;
                    this.nameTable[child.name] = child;
                }
            }
        },
        _setViewPortWidth: function () {
            this.element[0].setAttribute("style", "width:" + this.model.width + ";");
            var w = this.element.width();
            // var left = this.element[0].offsetLeft;
            return Math.abs(w);
        },
        _setViewPortHeight: function () {
            this.element[0].setAttribute("style", "height:" + this.model.height + ";");
            var h = this.element.height();
            //var top = this.element[0].offsetTop;
            return Math.abs(h);
        },
        _renderPalette: function (palette) {
            this._renderHeader(palette);
            this._renderContent(palette);
            this._collapse(palette);
        },
        _renderHeader: function (palette) {
            var header = document.createElement("div");
            var attr = { "id": palette.name + "_header", "class": "e-header", "style": "height:" + this.model.headerHeight + "px;" };
            ej.datavisualization.Diagram.Util.attr(header, attr);
            this._renderHeaderArrow(header, palette.expanded, palette.name);
            this._renderHeaderText(header, palette.name);
            this._addExpandedClass(header, palette.expanded);
            if (this.model.selectedPaletteName === palette.name) {
                $(header).addClass("selected");
                this.activePalette = palette;
            }
            this._canvas.appendChild(header);
        },
        _renderHeaderText: function (header, text) {
            var span = document.createElement("span");
            span.setAttribute("class", "e-header-text");
            span.innerHTML = text;
            header.appendChild(span);
        },
        _renderHeaderArrow: function (header, expanded, index) {
            var arrow = document.createElement("span");
            arrow.setAttribute("class", "e-header-arrow e-icon");
            arrow.setAttribute("style", "top:" + (this.model.headerHeight - 16) / 2 + "px;");
            this._addExpandedClass(arrow, expanded);
            if (this.model.selectedPaletteName === name)
                $(arrow).addClass("selected");
            header.appendChild(arrow);
        },
        _renderContent: function (palette) {
            var content = document.createElement("div");
            content.setAttribute("id", palette.name + "_content");
            content.setAttribute("class", "e-scontent");
            if ((this.model.palettes.length - 1) === this.model.palettes.indexOf(palette))
                content.setAttribute("style", "border-bottom-width: 0px;");
            this._canvas.appendChild(content);
            var paletteItems = palette.items;
            if (paletteItems && paletteItems.length > 0) {
                for (var i = 0, len = paletteItems.length; i < len; i++) {
                    if (paletteItems[i].parent)
                        this.nameTable[paletteItems[i].name] = paletteItems[i];
                    else {
                        this._renderItem(paletteItems[i], content);
                        this.nameTable[paletteItems[i].name] = paletteItems[i];
                    }
                }
            }
        },
        _updateRenderSize: function (paletteItem, preSize) {
            if (!paletteItem.paletteItem || (paletteItem.type === "group" && !paletteItem.isLane) || (paletteItem._type === "group" && paletteItem.type === "bpmn")) {
                if (!paletteItem.segments) {
                    paletteItem.width = preSize.width;
                    paletteItem.height = preSize.height;
                    paletteItem.offsetX = preSize.width / 2;
                    paletteItem.offsetY = preSize.height / 2;
                }
                else {
                    var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                    ej.datavisualization.Diagram.Util._translate(paletteItem, preSize.width / 2 - bounds.center.x, preSize.height / 2 - bounds.center.y, this.nameTable);
                }
            }
            else if (paletteItem.segments && paletteItem.paletteItem && paletteItem.paletteItem.enableScale && !paletteItem.isPhase) {
                var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                var dx = (paletteItem.paletteItem.width ? paletteItem.paletteItem.width : this.model.paletteItemWidth) / bounds.width;
                var dy = (paletteItem.paletteItem.height ? paletteItem.paletteItem.height : this.model.paletteItemHeight) / bounds.height;
                ej.datavisualization.Diagram.Util.scale(paletteItem, dx, dy, { x: .5, y: .5 });
                this._updatePaletteItemWidth(paletteItem);
            }
            else if ((paletteItem.segments && paletteItem.paletteItem && paletteItem.paletteItem.enableScale && !paletteItem.isPhase) || (paletteItem.isPhase && paletteItem.paletteItem.width != paletteItem.paletteItem.height)) {
                var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                var dx = (paletteItem.paletteItem.width ? paletteItem.paletteItem.width : this.model.paletteItemWidth) / bounds.width;
                var dy = (paletteItem.paletteItem.height ? paletteItem.paletteItem.height : this.model.paletteItemHeight) / bounds.height;
                ej.datavisualization.Diagram.Util.scale(paletteItem, dx, dy, { x: .5, y: .5 });
                this._updatePaletteItemWidth(paletteItem);
            }
            else if (paletteItem._type == "node" && this._checkItemUpdate(paletteItem)) {
                paletteItem.width = preSize.width;
                paletteItem.height = preSize.height;
                paletteItem.offsetX = preSize.width / 2;
                paletteItem.offsetY = preSize.height / 2;
            }
        },
        _checkItemUpdate: function (paletteItem) {
            if (!(paletteItem.paletteItem && paletteItem.paletteItem.enableScale))
                return true;
            return false;
        },
        _updatePreviewSize: function (paletteItem, preSize) {
            if (!paletteItem.paletteItem || (paletteItem.type === "group" && !paletteItem.isLane)) {
                if (!paletteItem.segments) {
                    paletteItem.width = preSize.width;
                    paletteItem.height = preSize.height;
                    paletteItem.offsetX = preSize.width / 2;
                    paletteItem.offsetY = preSize.height / 2;
                }
                else {
                    var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                    ej.datavisualization.Diagram.Util._translate(paletteItem, preSize.width / 2 - bounds.center.x, preSize.height / 2 - bounds.center.y, this.nameTable);
                }
            }
            else if (paletteItem.segments && paletteItem.paletteItem && paletteItem.paletteItem.enableScale && !paletteItem.isPhase) {
                var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                var dx = (paletteItem.paletteItem.previewWidth ? paletteItem.paletteItem.previewWidth : this.model.previewWidth) / bounds.width;
                var dy = (paletteItem.paletteItem.previewHeight ? paletteItem.paletteItem.previewHeight : this.model.previewHeight) / bounds.height;
                ej.datavisualization.Diagram.Util.scale(paletteItem, dx, dy, { x: .5, y: .5 });
                this._updatePaletteItemWidth(paletteItem);
            }
            else if ((paletteItem.segments && paletteItem.paletteItem && paletteItem.paletteItem.enableScale && !paletteItem.isPhase) || (paletteItem.isPhase && paletteItem.paletteItem.previewWidth != paletteItem.paletteItem.previewHeight)) {
                var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                var dx = (paletteItem.paletteItem.previewWidth ? paletteItem.paletteItem.previewWidth : this.model.previewWidth) / bounds.width;
                var dy = (paletteItem.paletteItem.previewHeight ? paletteItem.paletteItem.previewHeight : this.model.previewHeight) / bounds.height;
                ej.datavisualization.Diagram.Util.scale(paletteItem, dx, dy, { x: .5, y: .5 });
                this._updatePaletteItemWidth(paletteItem);
            }
            else if (paletteItem._type == "node") {
                paletteItem.width = preSize.width;
                paletteItem.height = preSize.height;
                paletteItem.offsetX = preSize.width / 2;
                paletteItem.offsetY = preSize.height / 2;
            }
        },
        _getRenderItemSize: function (paletteItem) {
            var width = this.model.paletteItemWidth, height = this.model.paletteItemHeight;
            if (!paletteItem.segments) {
                if (paletteItem.type === "group") {
                    bounds = ej.datavisualization.Diagram.Util._getChildrenBounds(paletteItem, this);
                    if (paletteItem.paletteItem && (!paletteItem.isLane)) {
                        width = paletteItem.paletteItem.width ? paletteItem.paletteItem.width : width;
                        height = paletteItem.paletteItem.height ? paletteItem.paletteItem.height : height;
                        width = width ? width : bounds.width;
                        height = height ? height : bounds.height;
                        paletteItem.paletteItem.width = width;
                        paletteItem.paletteItem.height = height;
                    }
                    else if (paletteItem.paletteItem && paletteItem.isLane) {
                        if (!paletteItem.paletteItem.enableScale) {
                            var dx = (paletteItem.paletteItem.width ? paletteItem.paletteItem.width : this.model.paletteItemWidth) / paletteItem.width;
                            var dy = (paletteItem.paletteItem.height ? paletteItem.paletteItem.height : this.model.paletteItemHeight) / paletteItem.height;
                            ej.datavisualization.Diagram.Util.scale(paletteItem, dx, dy, { x: .5, y: .5 }, this.nameTable, false, false, false, this);
                            var x = 0.0001, y = 0.001;
                            if (paletteItem.isLane && paletteItem.paletteItem && !paletteItem.paletteItem.enableScale) {
                                if (paletteItem.orientation === "horizontal") x -= 2.2;
                                else x += .8; y -= 1;
                            }
                            ej.datavisualization.Diagram.Util._translate(paletteItem, x, y, this.nameTable, false, this);
                            width = paletteItem.width ? paletteItem.width : width;
                            height = paletteItem.height ? paletteItem.height : height;
                        }
                        else {
                            width = paletteItem.paletteItem.width ? paletteItem.paletteItem.width : width;
                            height = paletteItem.paletteItem.height ? paletteItem.paletteItem.height : height;
                            width = width ? width : bounds.width;
                            height = height ? height : bounds.height;
                            paletteItem.paletteItem.width = width;
                            paletteItem.paletteItem.height = height;
                        }
                    }
                    else {
                        width = width ? width : bounds.width;
                        height = height ? height : bounds.height;
                    }
                }
                else {
                    if (paletteItem.paletteItem) {
                        width = paletteItem.paletteItem.width ? paletteItem.paletteItem.width : width;
                        height = paletteItem.paletteItem.height ? paletteItem.paletteItem.height : height;
                        width = width ? width : paletteItem.width;
                        height = height ? height : paletteItem.height;
                        paletteItem.paletteItem.width = width;
                        paletteItem.paletteItem.height = height;
                    }
                    else {
                        var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                        width = bounds.width ? bounds.width : width;
                        height = bounds.height ? bounds.height : height;
                    }
                }
            }
            else {
                var dec, size, bounds;
                bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                if (paletteItem.paletteItem) {
                    width = paletteItem.paletteItem.width ? paletteItem.paletteItem.width : width;
                    height = paletteItem.paletteItem.height ? paletteItem.paletteItem.height : height;
                    width = width ? width : paletteItem.width;
                    height = height ? height : paletteItem.height;
                    paletteItem.paletteItem.width = width;
                    paletteItem.paletteItem.height = height;
                }
                else {
                    var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                    width = width ? width : bounds.width;
                    height = height ? height : bounds.height;
                }
            }
            return { width: width, height: height };
        },
        _renderItem: function (paletteItem, content) {

            var item = $.extend(true, {}, paletteItem);
            var itemSize = this._getRenderItemSize(item);
            this._updateRenderSize(item, itemSize)
            var svg = this._renderItemContainer(item.name, content, null, itemSize, item);
            if (item.segments) {
                this._renderConnector(item, svg, itemSize);
            }
            else if (item._type == "node" && !(item.children && item.children.length > 0))
                this._renderNode(item, svg, itemSize);
            else
                this._renderGroup(item, svg, itemSize);
            if (item.isLane && item.paletteItem && !item.paletteItem.enableScale) {
                if (item.orientation === "vertical") {
                    svg.document.setAttribute("height", itemSize.height + 2);
                }
            }
        },
        _getPreviewItemSize: function (paletteItem) {
            var width = this.model.previewWidth, height = this.model.previewHeight;
            if (!paletteItem.segments) {
                if (paletteItem.type === "group") {
                    bounds = ej.datavisualization.Diagram.Util._getChildrenBounds(paletteItem, this);
                    if (paletteItem.paletteItem && (!paletteItem.isLane)) {
                        width = paletteItem.paletteItem.previewWidth ? paletteItem.paletteItem.previewWidth : width;
                        height = paletteItem.paletteItem.previewHeight ? paletteItem.paletteItem.previewHeight : height;
                        width = width ? width : bounds.width;
                        height = height ? height : bounds.height;
                        paletteItem.paletteItem.previewWidth = width;
                        paletteItem.paletteItem.previewHeight = height;
                    }
                    else if (paletteItem.paletteItem && paletteItem.isLane) {
                        if (!paletteItem.paletteItem.enableScale) {
                            width = paletteItem.paletteItem.previewWidth ? paletteItem.paletteItem.previewWidth : width;
                            height = paletteItem.paletteItem.previewHeight ? paletteItem.paletteItem.previewHeight : height;
                        }
                        else {
                            width = paletteItem.paletteItem.previewWidth ? paletteItem.paletteItem.previewWidth : width;
                            height = paletteItem.paletteItem.previewHeight ? paletteItem.paletteItem.previewHeight : height;
                            width = width ? width : bounds.width;
                            height = height ? height : bounds.height;
                            paletteItem.paletteItem.previewWidth = width;
                            paletteItem.paletteItem.previewHeight = height;
                        }
                    }
                    else {
                        width = width ? width : bounds.width;
                        height = height ? height : bounds.height;
                    }
                }
                else {
                    if (paletteItem.paletteItem) {
                        width = paletteItem.paletteItem.previewWidth ? paletteItem.paletteItem.previewWidth : width;
                        height = paletteItem.paletteItem.previewHeight ? paletteItem.paletteItem.previewHeight : height;
                        width = width ? width : paletteItem.width;
                        height = height ? height : paletteItem.height;
                        paletteItem.paletteItem.previewWidth = width;
                        paletteItem.paletteItem.previewHeight = height;
                    }
                    else {
                        var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                        width = bounds.width ? bounds.width : width;
                        height = bounds.height ? bounds.height : height;
                    }
                }
            }
            else {
                var dec, size, bounds;
                bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                if (paletteItem.paletteItem) {
                    width = paletteItem.paletteItem.previewWidth ? paletteItem.paletteItem.previewWidth : width;
                    height = paletteItem.paletteItem.previewHeight ? paletteItem.paletteItem.previewHeight : height;
                    width = width ? width : paletteItem.width;
                    height = height ? height : paletteItem.height;
                    width = width ? width : bounds.width;
                    height = height ? height : bounds.height;
                    paletteItem.paletteItem.previewWidth = width;
                    paletteItem.paletteItem.previewHeight = height;
                }
                else {
                    var bounds = ej.datavisualization.Diagram.Util.bounds(paletteItem);
                    width = width ? width : bounds.width;
                    height = height ? height : bounds.height;
                }
            }
            return { width: width, height: height };
        },
        _renderItemContainer: function (id, content, isDrag, itemSize, item) {
            var container = document.createElement("div");
            var height = this.model.showPaletteItemText ? itemSize.height + 18 : itemSize.height;
            var width = itemSize.width;
            var mLeft = (item && item.paletteItem) ? item.paletteItem.margin.left : 4;
            var mTop = (item && item.paletteItem) ? item.paletteItem.margin.top : 4;
            var mRight = (item && item.paletteItem) ? item.paletteItem.margin.right : 4;
            var mBottom = (item && item.paletteItem) ? item.paletteItem.margin.bottom : 4;
            var attr = {
                "id": id + "_paletteItem", "draggable": "true", "height": height + "px", "width": width + "px", "class": "e-paletteItem",
                "style": "padding-left:" + mLeft + "px;padding-right:" + (mRight + 2) + "px;padding-top:" + (mTop + 1) + "px;padding-bottom:" + (mBottom + 1) + "px;height:" + (height + 1) + "px;width:" + (width + 1) +
                          "px;-ms-touch-action: none;touch-action: none;"
            };
            ej.datavisualization.Diagram.Util.attr(container, attr);
            content.appendChild(container);
            if (this.model.allowDrag)
                this._setItemDraggable(container);
            return this._renderAnchor(container, id, itemSize);
        },
        _renderAnchor: function (container, id, itemSize) {
            var anchor = document.createElement("div");
            var attr = {
                "id": id + "_anchor", "height": itemSize.height + "px", "class": "e-anchor"
            };
            ej.datavisualization.Diagram.Util.attr(anchor, attr);
            container.appendChild(anchor);
            var svg = this._renderSvg(anchor, id, itemSize);
            this._renderText(anchor, id, itemSize);
            return svg;
        },
        _renderSvg: function (anchor, id, itemSize) {
            var parent;
            var wrapper = document.createElement("div");
            var attr = {
                "id": id + "_svgWrapper", "width": this.model.paletteItemWidth + "px;",
                "height": +this.model.paletteItemHeight + "px;", "class": "e-svg-container",
                "style": "width:" + itemSize.width + "px;height:" + itemSize.height + "px;"
            };
            ej.datavisualization.Diagram.Util.attr(wrapper, attr);
            attr = {
                "id": id + "_svg", "width": itemSize.width, "height": itemSize.height,
                "version": "1.1", "xlink": "http://www.w3.org/1999/xlink"
            };
            var svg = new ej.datavisualization.Diagram.Svg(attr);
            wrapper.appendChild(svg.document);
            if (anchor.id.split("_clone").length > 1)
                parent = $(anchor).parents(".e-paletteItem")[0];
            this._renderHtmlLayer(wrapper, parent);
            anchor.appendChild(wrapper);
            return svg;
        },
        _renderText: function (anchor, text, itemSize) {
            var div = document.createElement("div");
            div.setAttribute("class", "e-text-container");
            div.setAttribute("style", "width:" + itemSize.width + "px; text-align : center;overflow: hidden;text-overflow: ellipsis;");
            var span = document.createElement("span");
            span.setAttribute("class", "e-paletteItemText");
            if (text.split("_clone").length > 1)
                span.innerHTML = text.split("_clone")[0];
            else
                span.innerHTML = text;
            div.appendChild(span);
            if (!this.model.showPaletteItemText)
                div.style.display = "none";
            anchor.appendChild(div);
        },
        _renderNode: function (node, svg, scaleSize, palette) {
            var group = ej.datavisualization.Diagram.SvgContext.renderNode(node, svg, null, true, this);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, node.width, node.height, node, isDrag, scaleSize);
        },
        _updateChildBounds: function (node, nameTable) {
            var child = null;
            if (node._type == "group") {
                var children = this._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = nameTable[children[i]];
                    if (child._type == "group")
                        this._updateChildBounds(child, nameTable);
                }
                ej.datavisualization.Diagram.Util._updateChildBounds(node, this);
            }
        },
        _getChildren: function (children) {
            if (children) {
                var children1 = [];
                for (var i = 0; i < children.length; i++) {
                    var child = children[i];
                    if (child) {
                        if (typeof (child) == "object") {
                            children1.push(child.name);
                        }
                        else if (typeof (child) == "string") {
                            children1.push(child);
                        }
                    }
                }
                return children1;
            }
        },
        _renderGroup: function (node, svg, itemSize) {
            this._updateChildBounds(node, this.nameTable);
            var group = ej.datavisualization.Diagram.SvgContext.renderGroup(node, svg, null, this.nameTable, this, false, false, true);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, node.width, node.height, node, isDrag, itemSize);

        },
        _renderConnector: function (connector, svg, itemSize) {
            var group = ej.datavisualization.Diagram.SvgContext.renderConnector(connector, svg);
            var bounds = ej.datavisualization.Diagram.Util.bounds(connector);
            var isDrag = false;
            if (svg.document.getAttribute("drag") != null) {
                isDrag = true;
            }
            this._transformItem(group, bounds.width, bounds.height, connector, isDrag, itemSize);
        },
        //#endregion
        //#region Events
        _wireEvents: function () {
            this._on(this.element, ej.eventType.click, ".e-header", this._headerClick);
            this._on(this.element, ej.eventType.mouseDown, ".e-header", this._headerMouseDown);
            this._on(this.element, "mouseover", ".e-header", this._headerMouseOver);
            this._on(this.element, "mouseout", ".e-header", this._headerMouseOut);
            this._on(this.element, ej.eventType.mouseDown, ".e-paletteItem", this._itemMouseDown);
            this._on(this.element, "mouseover", ".e-paletteItem", this._itemMouseOver);
            this._on(this.element, "mouseout", ".e-paletteItem", this._itemMouseOut);
            this._on(this.element, ej.eventType.click, ".e-anchor", this._itemClick);
        },
        _headerClick: function (evt) {
            this._expandOrCollapsePalette("", evt.currentTarget);
            var palette = this._getActivePalette(evt.currentTarget.id);
            if (palette && palette !== this.activePalette)
                this.activePalette = palette;
        },
        _headerMouseDown: function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
        },
        _headerMouseOver: function (evt) {
            $(evt.currentTarget).addClass("hover");
            $(evt.currentTarget).find(".e-header-arrow").addClass("hover");
        },
        _headerMouseOut: function (evt) {
            $(evt.currentTarget).removeClass("hover");
            $(evt.currentTarget).find(".e-header-arrow").removeClass("hover");
        },
        _itemMouseDown: function (evt) {
            if (this.model.diagramId) {
                var diagram = $("#" + this.model.diagramId).ejDiagram("instance");
                if (diagram.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                    diagram.activeTool._showAllPorts(true);
                    diagram.deactivateTool();
                }
            }
            if (!this.model.allowDrag) {
                evt.preventDefault();
                evt.stopPropagation();
            }
            if (this._selectedElement) {
                if (this._selectedElement !== evt.currentTarget) {
                    $(this._selectedElement).removeClass("selected");
                    this._selectedElement = evt.currentTarget;
                }
            }
            else
                this._selectedElement = evt.currentTarget;
            if (!($(evt.currentTarget).hasClass("selected")))
                $(evt.currentTarget).addClass("selected");

            var palette = this._getActivePalette(evt.currentTarget.parentElement.id);
            if (palette && palette !== this.activePalette) {
                this._clearHeaderSelection();
                this._updateHeaderSelection($(evt.currentTarget.parentElement).prev());
                this.activePalette = palette;
            }

            if (this.activePalette && this._selectedElement) {
                var name = this._selectedElement.id.slice(0, this._selectedElement.id.lastIndexOf('_'));
                var items = this.activePalette.items;
                for (var i = 0, len = items.length; i < len; i++) {
                    var item = items[i];
                    if (item.name === name)
                        this.selectedItem = item;
                }
            }
        },
        _itemMouseOver: function (evt) {
            $(evt.currentTarget).addClass("hover");
        },
        _itemMouseOut: function (evt) {
            $(evt.currentTarget).removeClass("hover");
        },
        _itemDraStart: function (evt) {
            var dataTransfer = evt.originalEvent.dataTransfer;
            dataTransfer.setData("Text", this.element[0].id);
            dataTransfer.effectAllowed = "copy";
        },
        _itemClick: function (evt) {
            if (evt.type === "click" || (evt.type === "touchend" && !$(".dragClone")[0])) {
                var diagram = $("#" + this.model.diagramId).ejDiagram("instance");
                var item = $.extend(true, {}, this.selectedItem);
                item.name += ej.datavisualization.Diagram.Util.randomId();
                if (item.type == "bpmn") item.children = [];
                var type = diagram.getObjectType(item);
                var swimlane;
                if (type == "group") {
                    var children = this._getChildren(item.children);
                    for (var j = 0; j < children.length; j++) {
                        var newObj = $.extend(true, {}, this.nameTable[children[j]]);
                        newObj.parent = item.name;
                        newObj.name = item.name + newObj.name;
                        item.children[j] = newObj.name;
                        diagram.nameTable[newObj.name] = newObj;
                        diagram.nodes().push(newObj);
                        diagram._nodes = $.extend(true, [], diagram.nodes());
                        if (diagram.nameTable[newObj])
                            diagram._updateQuad(diagram.nameTable[newObj]);
                    }
                    if (!item.isLane) {
                        item.height = item.width = 0;
                        diagram._updateChildBounds(item, diagram.nameTable);
                    }
                }
                if (item) {
                    if (item.isLane) {
                        var randomId = ej.datavisualization.Diagram.Util.randomId();
                        diagram._cloneGroupNode(item, randomId);
                        var obj = ej.datavisualization.Diagram.SwimLaneHelper._createDiagramLane(item, diagram.nameTable);
                        obj.isLane = item.isLane;
                        obj.orientation = item.orientation;
                        obj.height = 120;
                        var swimlane = ej.datavisualization.Diagram.SwimLaneHelper._createSwimlane(obj, diagram);
                        if (swimlane) {
                            swimlane.offsetX += 10;
                            swimlane.offsetY += 40;
                        }
                    }
                }
                var oldElement = $.extend(true, {}, this._oldItem);
                var newElement = swimlane ? swimlane : item;
                this._oldItem = $.extend(true, {}, newElement);
                var args = { oldElement: oldElement, newElement: newElement };
                this._trigger("selectionChange", args);
            }
        },
        _collectionChanged: function (args) {
            if (args.changeType == "insert") {
                this._renderPalette(args.element);
            }
            else if (args.changeType == "remove") {
                var palette = args.element;
                if (palette != null) {
                    var paletteHeader = document.getElementById(palette.name + "_header");
                    if (paletteHeader)
                        paletteHeader.parentNode.removeChild(paletteHeader);
                    var paletteContent = document.getElementById(palette.name + "_content");
                    if (paletteContent)
                        paletteContent.parentNode.removeChild(paletteContent);
                }
            }
        },
        //#endregion
        //#region Public methods
        addPaletteItem: function (paletteName, item) {
            var content, palette;
            for (var i = 0; i < this.model.palettes.length; i++) {
                palette = this.model.palettes[i];
                if (palette.name === paletteName) {
                    palette.items.push(item);
                    content = document.getElementById(paletteName + "_content");
                    this._renderItem(item, content);
                }
            }
        },
        removePaletteItem: function (paletteName, item) {
            var content, palette;
            for (var i = 0; i < this.model.palettes.length; i++) {
                palette = this.model.palettes[i];
                if (palette.name === paletteName) {
                    ej.datavisualization.Diagram.Util.removeItem(palette.items, item);
                    content = document.getElementById(paletteName + "_content");
                    content.removeChild(content.getElementsByClassName("e-paletteItem e-draggable e-js").item(item.name + "_paletteItem"));
                    this.selectedItem = null;
                }
            }
        },
        expandPalette: function (paletteName) {
            var palette = this._findPalette(paletteName);
            if (palette && !palette.expanded) {
                palette.expanded = true;
                this._expandOrCollapsePalette(paletteName);
            }
        },
        collapsePalette: function (paletteName) {
            var palette = this._findPalette(paletteName);
            if (palette && palette.expanded) {
                palette.expanded = false;
                this._expandOrCollapsePalette(paletteName);
            }
        },
        addPalette: function (palette) {
            this.model.palettes.push(palette);
            this._renderPalette(palette);
        },
        removePalette: function (paletteName) {
            var paletteHeader, paletteContent, palette;
            for (var i = 0; i < this.model.palettes.length; i++) {
                palette = this.model.palettes[i];
                if (palette.name === paletteName) {
                    ej.datavisualization.Diagram.Util.removeItem(this.model.palettes, palette);
                    paletteHeader = document.getElementById(palette.name + "_header");
                    if (paletteHeader)
                        paletteHeader.parentNode.removeChild(paletteHeader);
                    paletteContent = document.getElementById(palette.name + "_content");
                    if (paletteContent)
                        paletteContent.parentNode.removeChild(paletteContent);
                }
            }
        },
        setWidth: function (value) {
            this.model.width = value;
            this._setWidth(value);
        },
        setHeight: function (value) {
            this.model.height = value;
            this._setHeight(value);
        },
        setPaletteItemWidth: function (value) {
            this.model.paletteItemWidth = value;
            this._redrawContent();
        },
        setPaletteItemHeight: function (value) {
            this.model.paletteItemHeight = value;
            this._redrawContent();
        },
        setHeaderHeight: function (value) {
            this.model.headerHeight = value;
            this._setHeaderHeight(value);
        },
        setCssClass: function (value) {
            this._setCssClass(value);
            this.model.cssClass = value;
        },
        showPaletteItemText: function (value) {
            this._showItemText(value);
            this.model.showPaletteItemText = value;
        },
        searchPalette: function (palette, key) {
            var paletteCollection = new ej.datavisualization.Diagram.Collection();
            var paletteItems = null;
            if (key != "") {
                palette = this._getSearchPaletteItems(key);
                paletteCollection.add(palette);
            }
            return paletteCollection;
        },
        //#endregion
        //#region Helper methods
        _findPalette: function (paletteName) {
            var palette;
            for (var i = 0; i < this.model.palettes.length; i++) {
                if (this.model.palettes[i].name == paletteName) {
                    palette = this.model.palettes[i];
                    break;
                }
            }
            return palette;
        },
        _expandOrCollapsePalette: function (paletteName, header) {
            var header = header || (paletteName ? document.getElementById(paletteName + "_header") : null);
            if (header) {
                var proxy = this;
                $(header).next(".e-scontent").slideToggle(200, function () {
                    proxy._refresh();
                });
                this._updateHeaderState(header);
            }
        },
        _getSearchPaletteItems: function (key) {
            var palette = new ej.datavisualization.Diagram.Palette();
            var palettes = this.model.palettes, i, j;
            for (i = 0; i < palettes.length; i++) {
                if ((((palettes[i].name).toLocaleLowerCase()).search(key.toLocaleLowerCase())) >= 0) {
                    for (j = 0; j < palettes[i].items.length; j++) {
                        this.element.find("#" + palettes[i].items[j].name + "_paletteItem")[0].style.display = "block";
                    }
                }
                else {
                    for (j = 0; j < palettes[i].items.length; j++) {
                        if ((((palettes[i].items[j].name).toLocaleLowerCase()).search(key.toLocaleLowerCase())) >= 0) {
                            this.element.find("#" + palettes[i].items[j].name + "_paletteItem")[0].style.display = "block";
                        }
                        else
                            this.element.find("#" + palettes[i].items[j].name + "_paletteItem")[0].style.display = "none";
                    }
                }
            }
            return palette;
        },
        _addExpandedClass: function (element, expanded) {
            if (expanded)
                $(element).addClass("expanded");
            else
                $(element).addClass("collapsed");
        },
        _collapse: function (palette) {
            if (!palette.expanded)
                $(document.getElementById(palette.name + "_header")).next(".e-scontent").slideToggle(10);

        },
        _transformItem: function (group, width, height, node, isDrag, scaleSize) {
            if (!isDrag) {
                if (scaleSize && !node.segments && node.paletteItem && node.paletteItem.width && node.paletteItem.height && node.paletteItem.enableScale) {
                    var fx = (30 * scaleSize.width) / 100;
                    var fy = (30 * scaleSize.height) / 100;
                    var sx = (scaleSize.width - fx) / node.width;
                    var sy = (scaleSize.height - fy) / node.height;
                    sx = sy = Math.min(sx, sy);
                    sx = sx > 1 ? 1 : sx;
                    sy = sy > 1 ? 1 : sy;
                    var translate1 = ((scaleSize.width) / 2 - (width * sx) / 2);
                    var translate2 = ((scaleSize.height) / 2 - (height * sy) / 2);
                    var transform = "";
                    //translate
                    transform += "translate(" + translate1 + "," + translate2 + ")";
                    //scale
                    transform += "scale(" + sx + "," + sy + ")";
                    group.setAttribute("transform", transform);
                    var htmlLayer = group.parentNode.nextSibling;
                    if (!htmlLayer) htmlLayer = document.getElementById(group.id + "_svgWrapper_htmlLayer");
                    htmlLayer.style.webkitTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                    htmlLayer.style.MozTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                    htmlLayer.style.OTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                    htmlLayer.style.msTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                    htmlLayer.style.transform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                    if (node && ej.browserInfo().name === "msie") {
                        if (node._type == "group" && node.type === "html") {
                            var htmlDiv = document.getElementById(node.name + "_html");
                            htmlDiv.setAttribute("style", "zoom:" + scale + ";");
                        }
                    }
                }
                else if (!node.segments) {
                    var xd = 1;
                    var yd = 1;
                    var x = (width - (width * xd)) / 2;
                    var y = (height - (height * yd)) / 2;
                    group.childNodes[0].setAttribute("transform", "scale(" + xd + "," + yd + ") translate(" + x + "," + y + ")");
                    if (node && node.type === "image" && group.childNodes[1]) {
                        group.childNodes[1].setAttribute("transform", "scale(" + xd + "," + yd + ") translate(" + x + "," + y + ")");
                    }
                }
                else {
                    if (scaleSize && node.paletteItem && node.paletteItem.width && node.paletteItem.height || (node.isPhase && node.paletteItem && node.paletteItem.enableScale)) {
                        var fx = (10 * scaleSize.width) / 100;
                        var fy = (10 * scaleSize.height) / 100;
                        var sx = (scaleSize.width - fx) / width;
                        var sy = (scaleSize.height - fy) / height;
                        sx = sy = Math.min(sx, sy);
                        sx = sx > 1 ? 1 : sx;
                        sy = sy > 1 ? 1 : sy;
                        var translate1 = ((scaleSize.width) / 2 - (width * sx) / 2);
                        var translate2 = ((scaleSize.height) / 2 - (height * sy) / 2);
                        var transform = "";
                        //translate
                        transform += "translate(" + translate1 + "," + translate2 + ")";
                        //scale
                        transform += "scale(" + sx + "," + sy + ")";
                        group.setAttribute("transform", transform);
                        var htmlLayer = document.getElementById(group.id + "_svgWrapper_htmlLayer");
                        htmlLayer.style.webkitTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                        htmlLayer.style.MozTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                        htmlLayer.style.OTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                        htmlLayer.style.msTransform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                        htmlLayer.style.transform = "translate(" + translate1 + "px," + translate2 + "px) scale(" + sx + ", " + sy + ")";
                        if (node && ej.browserInfo().name === "msie") {
                            if (node._type == "group" && node.type === "html") {
                                var htmlDiv = document.getElementById(node.name + "_html");
                                htmlDiv.setAttribute("style", "zoom:" + scale + ";");
                            }
                        }
                    }
                }
            }
        },
        _getScaleFactor: function (width, height) {
            var fx = (30 * this.model.paletteItemWidth) / 100;
            var fy = (30 * this.model.paletteItemHeight) / 100;
            var sx = (this.model.paletteItemWidth - fx) / width;
            var sy = (this.model.paletteItemHeight - fy) / height;
            sx = sx > 1 ? 1 : sx;
            sy = sy > 1 ? 1 : sy;
            return Math.min(sx, sy);
        },
        _getTranslateFactor: function (width, height) {
            var x = (15 / 100) * this.model.paletteItemWidth;
            var y = (15 / 100) * this.model.paletteItemHeight;
            return Math.min(x, y);
        },
        _setWidth: function (val) {
            this.element.css("width", val);
        },
        _setHeight: function (val) {
            this.element.css("height", val);
        },
        _setHeaderHeight: function (val) {
            this.element.find(".e-header").css("height", val);
            this.element.find(".e-header-arrow").css("top", (val - 16) / 2);
        },
        _setCssClass: function (val) {
            this.element.removeClass(this.model.cssClass).addClass(val);
        },
        _showItemText: function (val) {
            if (val !== this.model.showPaletteItemText)
                if (val) {
                    this.element.find(".e-text-container").show();
                }
                else
                    this.element.find(".e-text-container").hide();
        },
        _setAllowDrag: function (val) {
            this.model.allowDrag = val;
            var palettes = this.model.palettes;
            var container;
            if (palettes) {
                for (var i = 0; i < palettes.length; i++) {
                    var paletteItems = palettes[i].items;
                    if (paletteItems) {
                        for (var i = 0; i < paletteItems.length; i++) {
                            container = document.getElementById(paletteItems[i].name + "_paletteItem");
                            if (this.model.allowDrag)
                                this._setItemDraggable(container);
                            else
                                $(container).ejDraggable("instance").destroy();
                        }
                    }
                }
            }
        },
        _setItemDraggable: function (container) {
            var paletteId = this.element[0].id;
            var context = this;
            $(container).ejDraggable({
                clone: true,
                cursorAt: { top: context.model.previewOffset.y, left: context.model.previewOffset.x },
                helper: function (event) {
                    var diagram = $("#" + context.model.diagramId).ejDiagram("instance");
                    var paletteItemWidth, paletteItemHeight;
                    paletteItemWidth = context.model.paletteItemWidth;
                    paletteItemHeight = context.model.paletteItemHeight;
                    diagram._selectedSymbol = context.selectedItem;
                    diagram._symbolPalette = context.model;
                    var item = $.extend(true, {}, context.selectedItem);
                    if (item.isLane) {
                        context.model.paletteItemWidth = (item.paletteItem && item.paletteItem.previewWidth) ? item.paletteItem.previewWidth : context.model.previewWidth;
                        context.model.paletteItemHeight = (item.paletteItem && item.paletteItem.previewHeight) ? item.paletteItem.previewHeight : context.model.previewHeight;
                    }
                    var previewSize = context._getPreviewItemSize(context.selectedItem);
                    var svg = context._renderItemContainer(context.selectedItem.name + "_clone", document.body, true, previewSize);
                    svg.document.setAttribute("drag", "true");
                    diagram._paletteTable = context.nameTable;
                    diagram._palNameTable = $.extend(true, {}, context.nameTable);
                    if (item.segments) {
                        var bounds = ej.datavisualization.Diagram.Util.bounds(item);
                        var delWidth = (previewSize.width - 10) / bounds.width;
                        var delHeight = (previewSize.height - 10) / bounds.height;
                        ej.datavisualization.Diagram.Util.scale(item, delWidth, delHeight, bounds.topLeft, diagram.nameTable);
                        context._renderConnector(item, svg, previewSize);
                    }
                    else {
                        if ((item.isLane && item.paletteItem && !item.paletteItem.enableScale) || item.type === "bpmn") {
                            item.width = previewSize.width;
                            item.height = previewSize.height;
                            item.offsetX = item.width / 2;
                            item.offsetY = item.height / 2;
                        }
                        else {
                            item.width = previewSize.width - 10;
                            item.height = previewSize.height - 10;
                            item.offsetX = item.width / 2 + 10;
                            item.offsetY = item.height / 2 + 1;
                        }
                        context._updatePreviewSize(item, previewSize);
                        if (item._type == "node")
                            context._renderNode(item, svg, previewSize);
                        else {
                            context._renderGroup(item, svg);
                        }
                    }
                    var helper = document.getElementById(context.selectedItem.name + "_clone_paletteItem");
                    $(helper).addClass("dragClone");
                    context.model.paletteItemWidth = paletteItemWidth;
                    context.model.paletteItemHeight = paletteItemHeight;
                    return $(helper);
                },
                drag: function () {
                    var element = this.helper[0];
                    $("#" + context.selectedItem.name + "_clone_svg").css("padding", "2px");
                    $("#" + context.selectedItem.name + "_clone_svg").siblings(".htmlLayer").css("padding", "2px");
                    if (element) {
                        element.setAttribute("paletteId", paletteId);
                    }
                },
                dragStop: function (event, ui) {
                    var diagram = $("#" + context.model.diagramId).ejDiagram("instance");
                    diagram._selectedSymbol = null;
                    $(this.helper).remove();
                }
            });
        },
        _redrawContent: function () {
            var contents = this.element.find(".e-scontent");
            for (var i = 0, len = contents.length; i < len; ++i) {
                var content = contents[i];
                $(content).empty();
                var palette = this.model.palettes[i];
                var paletteItems = palette.items;
                if (paletteItems && paletteItems.length > 0) {
                    for (var count = 0, l = paletteItems.length; count < l; count++) {
                        this._renderItem(paletteItems[count], content);
                    }
                }
            }
        },
        _getActivePalette: function (elementId) {
            var name = elementId.slice(0, elementId.lastIndexOf('_'));
            var palettes = this.model.palettes;
            for (var i = 0, len = palettes.length; i < len; i++) {
                var palette = palettes[i];
                if (palette.name === name && palette !== this.activePalette)
                    return palette;
            }
            return null;
        },
        _updateHeaderState: function (target) {
            this._clearHeaderSelection();
            this._updateExpandState(target);
            this._updateHeaderSelection(target);
        },
        _updateExpandState: function (target) {
            if (($(target).hasClass("expanded"))) {
                $(target).removeClass("expanded").addClass("collapsed");
                $(target).find(".e-header-arrow").removeClass("expanded").addClass("collapsed");
            }
            else {
                $(target).removeClass("collapsed").addClass("expanded");
                $(target).find(".e-header-arrow").removeClass("collapsed").addClass("expanded");
            }
        },
        _updateHeaderSelection: function (target) {
            if (!($(target).hasClass("selected"))) {
                $(target).addClass("selected");
                $(target).find(".e-header-arrow").addClass("selected");
            }
        },
        _clearHeaderSelection: function () {
            this.element.find(".e-header").removeClass("selected");
            this.element.find(".e-header-arrow").removeClass("selected");
        },
        _addItem: function (palette, item) {
            var content = document.getElementById(palette.name + "_content");
            this._renderItem(item, content);
        },
        _removeItem: function (palette, item) {
            var content = document.getElementById(palette.name + "_content");
            content.removeChild(document.getElementById(item.name + "_paletteItem"));
            if (item === this._selectedElement)
                this._selectedElement = null;
        },
        updateScrollerViewport: function () {
            var scroller = this.element.ejScroller("instance");
            scroller.model.touchScroll = false;
            scroller.model.width = this._setViewPortWidth();
            scroller.model.height = this._setViewPortHeight();
            this._refresh();
        },
        _refresh: function () {
            this.element.ejScroller("refresh");
        },
        _getNodeDimension: function (node) {
            var prevWidth = node._width, prevHeight = node._height;
            var width = node.width ? node.width : (node.maxWidth ? node.maxWidth : node.minWidth);
            var text = document.createElement("span");
            var w = 0, h = 0, label, i;
            if (node.labels.length) {
                ej.datavisualization.Diagram.Util.attr(text, { "id": node.name + "_label", "class": "ej-d-label", "style": "display: inline-block; position: absolute; pointer-events: all; line-height: normal;" });
                this.element[0].appendChild(text);
                for (i = 0; i < node.labels.length; i++) {
                    label = node.labels[i];
                    if (label.bold) text.style.fontWeight = "bold";
                    if (label.italic) text.style.fontStyle = "italic";
                    text.style.textDecoration = label.textDecoration;
                    text.style.fontFamily = label.fontFamily;
                    text.style.fontSize = label.fontSize + "px";
                    text.style.color = label.fontColor;
                    text.style.backgroundColor = label.fillColor;
                    text.style.borderColor = label.borderColor;
                    text.style.borderWidth = label.borderWidth;
                    text.textContent = label.text;
                    text.style.wordWrap = "break-word";
                    text.style.whiteSpace = "pre";
                    if (width) text.style.maxWidth = width - (label.margin.left + label.margin.right) - label.fontSize + "px";
                    w += text.offsetWidth + label.margin.left + label.margin.right;
                    h += text.offsetHeight + label.margin.top + label.margin.bottom;
                }
                node._width = w;
                node._height = h;
                if (node.minWidth && (node.minWidth > node._width || !node._width)) node._width = node.minWidth;
                if (node.maxWidth && (node.maxWidth < node._width || !node._width)) node._width = node.maxWidth;
                if (node.minHeight && (node.minHeight > node._height || !node._height)) node._height = node.minHeight;
                if (node.maxHeight && (node.maxHeight < node._height || !node._height)) node._height = node.maxHeight;
                if (!node.width && node.labels[0].horizontalAlignment == "left") node.offsetX += (node._width - prevWidth) * (1 - node.pivot.x);
                if (!node.width && node.labels[0].horizontalAlignment == "right") node.offsetX -= (node._width - prevWidth) * node.pivot.x;
                if (!node.height && node.labels[0].verticalAlignment == "top") node.offsetY += (node._height - prevHeight) * (1 - node.pivot.y);
                if (!node.height && node.labels[0].verticalAlignment == "bottom") node.offsetY -= (node._height - prevHeight) * node.pivot.y;
                this.element[0].removeChild(text);
            }
        },
        //#endregion
    });
    //#endregion
})(jQuery, Syncfusion);;


(function ($, ej) {
    "use strict";
    //#region Extends
    ej.datavisualization.Diagram.extend = function (d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
        function __() { this.constructor = d; }
        __.prototype = b.prototype;
        d.prototype = new __();
    }
    //#endregion

    //#region Tool
    var ToolBase = (function () {
        function ToolBase(name, diagram) {
            this.name = name;
            this.diagram = diagram;
            this.helper = null;
            this.inAction = false;
            this.selectedObject = null;
            this.startPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.currentPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.previousPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.cursor = "default";
            this._currentPossibleConnection = null;
            this._previousPossibleConnection = null;
            this._possibleConnectionPort = null;
            this.singleAction = false;
            this.svgHelper = null;
            this.single = null;
            this._centralPoint = ej.datavisualization.Diagram.Point(0, 0);
            this._enableAutoNode = null;
            this._prevTool = null;
            this.activeLabel = null;
            this._adjustLines = {
                lines: []
            }
        }
        ToolBase.prototype.abort = function (evt) {
            this._endAction();
            if (this.diagram.selectionList && this.diagram.selectionList[0])
                ej.datavisualization.Diagram.SvgContext.updateSelector(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom, this.diagram, this.diagram.model.selectedItems.constraints);
            this.diagram._removeTooltip();
        };
        ToolBase.prototype._snapTop = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var dify = dely;
            verticalsnap.top = true;
            horizontalsnap.left = horizontalsnap.right = false;
            var snapSettings = this.diagram.model.snapSettings;
            var zoomFactor = this.diagram._currZoom;
            if (this.diagram._enableSnapToObject() && !shape.rotateAngle &&
                (!this.selectedObject.isLane && !this.selectedObject.isSwimlane)) {
                var y = this.initialBounds.y - this.initialBounds.height * shape.pivot.y + dely - (shape.offsetY - shape.height * shape.pivot.y);
                ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, delx, y, this.selectedObject, ended);
            }
            if (!verticalsnap.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) {
                    var top = this.initialBounds.y - this.initialBounds.height * shape.pivot.y;
                    var actualTop = top + dely;
                    var roundedTop = ej.datavisualization.Diagram.SnapUtil._round(actualTop, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
                    dify = roundedTop - top;
                }
            }
            else {
                dify = (dely - y) + verticalsnap.offset;
            }
            return dify;
        };

        ToolBase.prototype._snapLeft = function (horizontalsnap, verticalsnap, delx, dely, shape, ended) {
            var difx = delx;
            horizontalsnap.left = true;
            verticalsnap.top = verticalsnap.bottom = false;
            var snapSettings = this.diagram.model.snapSettings;
            var zoomFactor = this.diagram._currZoom;
            if (this.diagram._enableSnapToObject() && !shape.rotateAngle &&
                (!this.selectedObject.isLane && !this.selectedObject.isSwimlane)) {
                var x = this.initialBounds.x - this.initialBounds.width * shape.pivot.x + delx - (shape.offsetX - shape.width * shape.pivot.x);
                ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, x, dely, this.selectedObject, ended);;
            }
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            if (!horizontalsnap.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines) {
                    var left = this.initialBounds.x - this.initialBounds.width * shape.pivot.x;
                    var actualLeft = left + delx;
                    var roundedLeft = ej.datavisualization.Diagram.SnapUtil._round(actualLeft, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
                    difx = roundedLeft - left;
                }
            }
            else {
                difx = (delx - x) + horizontalsnap.offset;
            }
            return difx;
        };
        ToolBase.prototype._snapBottom = function (horizontalsnap, verticalsnap, delx, dely, shape, ended, disableSnap) {
            var dify = dely;
            if (!disableSnap) {
                verticalsnap.bottom = true;
                horizontalsnap.left = horizontalsnap.right = false;
                var snapSettings = this.diagram.model.snapSettings;
                var zoomFactor = this.diagram._currZoom;
                if (this.diagram._enableSnapToObject() && !shape.rotateAngle &&
                       (!this.selectedObject.isLane && !this.selectedObject.isSwimlane)) {
                    var y = this.initialBounds.y + this.initialBounds.height * (1 - shape.pivot.y) + dely - (shape.offsetY + shape.height * (1 - shape.pivot.y));
                    ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, delx, y, this.selectedObject, ended);;
                }
                var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
                if (!verticalsnap.snapped) {
                    if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) {
                        var bottom = this.initialBounds.y + this.initialBounds.height * (1 - shape.pivot.y);
                        var actualBottom = bottom + dely;
                        var roundedBottom = ej.datavisualization.Diagram.SnapUtil._round(actualBottom, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
                        dify = roundedBottom - bottom;
                    }
                }
                else {
                    dify = (dely - y) + verticalsnap.offset;
                }
            }
            return dify;
        };

        ToolBase.prototype._snapRight = function (horizontalsnap, verticalsnap, delx, dely, shape, ended, disableSnap) {
            var difx = delx;
            if (!disableSnap) {
                horizontalsnap.right = true;
                verticalsnap.top = verticalsnap.bottom = false;
                var snapSettings = this.diagram.model.snapSettings;
                var zoomFactor = this.diagram._currZoom;
                if (this.diagram._enableSnapToObject() && !shape.rotateAngle &&
                       (!this.selectedObject.isLane && !this.selectedObject.isSwimlane)) {
                    var x = this.initialBounds.x + this.initialBounds.width * (1 - shape.pivot.x) + delx - (shape.offsetX + shape.width * (1 - shape.pivot.x));
                    ej.datavisualization.Diagram.SnapUtil._snapSize(this.diagram, horizontalsnap, verticalsnap, x, dely, this.selectedObject, ended);
                }
                if (!horizontalsnap.snapped) {
                    if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines) {
                        var right = this.initialBounds.x + this.initialBounds.width * (1 - shape.pivot.x);
                        var actualRight = right + delx;
                        var roundedRight = ej.datavisualization.Diagram.SnapUtil._round(actualRight, snapSettings.verticalGridLines.snapInterval, zoomFactor);
                        difx = roundedRight - right;
                    }
                }
                else {
                    difx = (delx - x) + horizontalsnap.offset;
                }
            }
            return difx;
        };

        ToolBase.prototype.keydown = function (evt) {
            var keycode = evt.keyCode ? evt.keyCode : evt.which;
            if (keycode === 27 && this.inAction) {

                if (this.diagram._selectedSymbol) {
                    this.diagram._documentmouseup(evt);
                    this._removeHighLighter();
                    ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
                    this.diagram._selectedSymbol = null;
                }
                else {
                    if (this.seletedObject || this.selectedObject) {
                        if (this.selectedObject) {
                            ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram);
                            ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                        }
                        if (this.seletedObject) {
                            ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.seletedObject, this.diagram._svg, this.diagram);
                            ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.seletedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                        }
                        this._removeHighLighter();
                        ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
                    }
                }
                this.abort();
            }
            else if (!this.inAction && keycode == 27) {
                this.diagram._clearSelection();
            }
        };
        ToolBase.prototype._getSwimlaneHeader = function (group) {
            var header;
            if (group && group.isSwimlane) {
                if (group.children && group.children.length > 0) {
                    header = this.diagram.nameTable[group.children[0]];
                }
            }
            return header;
        };
        ToolBase.prototype._updateNextPhases = function (seperator, dx, dy) {
            var dif = 0, index, cphase;
            var group;
            if (seperator.orientation == "horizontal") {
                dif = dx;
            }
            else {
                dif = dy;
            }
            index = this.diagram._getPhaseIndex(seperator.name);
            group = this.diagram.nameTable[seperator.parent];
            for (var i = index + 1; i < group.phases.length; i++) {
                cphase = this.diagram.nameTable[group.phases[i]];
                if (cphase)
                    cphase.offset += dif;
                ej.datavisualization.Diagram.SvgContext._updatephase(cphase, this.diagram);
            }
        };

        ToolBase.prototype._getphase = function (id) {
            return this.diagram.nameTable[id];
        };
        ToolBase.prototype._updateSelectedPhase = function (seperator, dx, dy) {
            if (typeof (seperator) === "string") {
                seperator = this._getphase(seperator);
                var stackNode = this.diagram.nameTable["phaseStack" + seperator.name];
            }
            else {
                var stackNode = this.diagram.nameTable["phaseStack" + seperator.name];
            }
            if (stackNode) {
                if (seperator.orientation == "horizontal") {
                    stackNode.width += dx;
                    stackNode.offsetX += dx / 2;
                    this._updateNextPhases(seperator, dx, 0);
                }
                else {
                    stackNode.height += dy;
                    stackNode.offsetY += dy / 2;
                    this._updateNextPhases(seperator, 0, dy);
                }
                var stack = this.diagram.nameTable[seperator.parent + "phaseStack"];
                if (stack)
                    ej.datavisualization.Diagram.SvgContext._alignOnStackContainer(stack, this.diagram);
            }
        };
        ToolBase.prototype._getLastPhase = function (group) {
            if (group.children[1])
                var phaseStack = this.diagram.nameTable[this.diagram._getChild(group.children[1])];
            if (phaseStack && phaseStack.children && phaseStack.children.length > 0) {
                return this.diagram.nameTable[this.diagram._getChild(phaseStack.children[phaseStack.children.length - 1])];
            }

        };
        ToolBase.prototype._containChil = function (node, list) {
            for (var i = 0; i < list.length; i++) {
                if (list[i] == node.name)
                    return true;
            }
        };

        ToolBase.prototype._containsSwimlane = function (list) {
            for (var i in list) {
                var child = this.diagram.nameTable[list[i]];
                if (child.isSwimlane || child.isLane)
                    return true;
            }
            return false;
        };

        ToolBase.prototype._raiseEndPointDrag = function (connector, isTarget, isSource, dragState) {
            if (isTarget || isSource) {
                var type, node, port, point;
                node = this._currentPossibleConnection ? this._currentPossibleConnection : null;
                port = this._possibleConnectionPort ? this._possibleConnectionPort : null;
                point = this.currentPoint;
                if (isTarget)
                    type = "Target";
                else type = "Source";
                return this._raiseEvent("connector" + type + "Change", { element: connector, node: node, port: port, point: point, dragState: dragState, cancel: false });
            }
        };

        ToolBase.prototype._decideSelectedItem = function (evt, node, skip) {
            var obj = null;
            if (node) {
                if (node.parent) {
                    if (this.diagram.selectionList && this.diagram.selectionList[0] && node.parent == this.diagram.selectionList[0].name && !skip) {
                        obj = node;
                        if (this.diagram.selectionList[0].type == "bpmn")
                            obj = this.diagram.selectionList[0];
                    }
                    else {
                        if (skip && this.diagram.selectionList[0] && this.diagram.selectionList[0].name == node.name) {
                            obj = node;
                        }
                        else {
                            var item = this.diagram.nameTable[node.parent];
                            if (node.isPhase) {
                                var name = (node.name).split("phaseStack")[1];
                                if (name) {
                                    node = this.diagram.nameTable[name];
                                }
                                obj = node;
                            }
                            else {
                                obj = this._decideSelectedItem(evt, item, skip);
                                if (this.diagram.selectionList && this.diagram.selectionList[0]) {
                                    if (obj && this.diagram.selectionList[0].name == obj.name && !skip)
                                        obj = node;
                                }
                            }
                        }
                    }
                } else {
                    obj = node;
                }
            }
            else {
                return null;
            }
            return obj;
        };
        ToolBase.prototype._getLanes = function (swimlane) {
            var laneStack = this.diagram.nameTable[this.diagram._getChild(swimlane.children[2])];
            var lanes = [], i, lane;
            if (laneStack) {
                for (i = 0; laneStack.children.length > 0 && i < laneStack.children.length; i++) {
                    var laneName = this.diagram._getChild(laneStack.children[i])
                    if (laneName) {
                        lanes.push(this.diagram.nameTable[laneName]);
                    }
                }
            }
            return lanes
        };
        ToolBase.prototype._childExistOnLane = function (child, lanes) {
            var lane, x1, x2, y1, y2, lBounds, cBounds;
            for (var i = 0; i < lanes.length; i++) {
                lane = lanes[i];
                lBounds = ej.datavisualization.Diagram.Util.bounds(lane);
                cBounds = ej.datavisualization.Diagram.Util.bounds(child);
                x1 = lBounds.topLeft.x;
                y1 = lBounds.topLeft.y;
                x2 = lBounds.bottomRight.x;
                y2 = lBounds.bottomRight.y;
                if ((x1 <= cBounds.center.x) && (cBounds.center.x <= x2) && (y1 <= cBounds.center.y) && (cBounds.center.y <= y2)) {
                    return lane;
                }
            }
        };
        ToolBase.prototype._collectionFromLane = function (collection) {
            for (var i = 0; i < collection.length; i++) {
                if (this._fromLane(collection[i]))
                    return true;
            }
        };
        ToolBase.prototype._fromLane = function (child) {
            if (child.parent) {
                var parent = this.diagram.nameTable[child.parent];
                return (parent && parent.isLane) ? true : false;
            }
        };
        ToolBase.prototype._isParentAsLane = function (pGroup) {
            var child, lane, laneStack;
            for (var i = 0; pGroup.children && i < pGroup.children.length > 0; i++) {
                child = this.diagram.nameTable[this.diagram._getChild(pGroup.children[i])];
                if (child) {
                    lane = this.diagram.nameTable[child.parent];
                    if (lane && lane.isLane) {
                        laneStack = this.diagram.nameTable[lane.parent];
                        this.diagram._activeSwimLane = this.diagram.nameTable[laneStack.parent];
                        return true;
                    }
                }
            }
            return null;
        };
        ToolBase.prototype._getAdjustjecentLines = function (child) {
            if (!child.segments) {
                for (var i = 0; child.inEdges && child.inEdges.length > 0 && i < child.inEdges.length ; i++) {
                    if (!this._containsCollection(this._adjustLines.lines, this.diagram.nameTable[child.inEdges[i]]))
                        this._adjustLines.lines.push(jQuery.extend(true, {}, this.diagram.nameTable[child.inEdges[i]]));
                }
                for (i = 0; child.outEdges && child.outEdges.length > 0 && i < child.outEdges.length ; i++) {
                    if (!this._containsCollection(this._adjustLines.lines, this.diagram.nameTable[child.outEdges[i]]))
                        this._adjustLines.lines.push(jQuery.extend(true, {}, this.diagram.nameTable[child.outEdges[i]]));
                }
            }
        };
        ToolBase.prototype._containsCollection = function (collection, obj) {
            for (var i = 0; collection && collection.length > 0 && i < collection.length; i++) {
                if (obj && collection[i].name === obj.name)
                    return true;
            }
        };
        ToolBase.prototype._getBottomSwimlane = function (node) {
            if (node) {
                var quads, i, j, obj;
                quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(this.diagram._spatialSearch, ej.datavisualization.Diagram.Util.bounds(node));
                for (i = 0; i < quads.length; i++) {
                    var quad = quads[i];
                    if (quad.objects.length > 0) {
                        for (j = 0; j < quad.objects.length; j++) {
                            obj = quad.objects[j];
                            if (obj && obj.isSwimlane) {
                                return obj;
                            }
                        }
                    }
                }
            }
        };
        ToolBase.prototype._updateMultipleUndoObject = function (psedoGroup) {
            var child, collection = [];
            for (var i = 0; psedoGroup.children && psedoGroup.children.length > 0 && i < psedoGroup.children.length; i++) {
                child = this.diagram.nameTable[psedoGroup.children[i]];
                if (child) {
                    collection.push(jQuery.extend(true, {}, child));
                    this._getAdjustjecentLines(child);
                }
            }
            if (!this._undoObject) {
                this._undoObject = {
                    collection: collection,
                    adjustLines: jQuery.extend(true, {}, this._adjustLines)
                };
            }
        };
        ToolBase.prototype._updateNameTable = function (node) {
            this.diagram._updateNameTable(node);
        };
        ToolBase.prototype._getUpdatedNode = function (node) {
            node = jQuery.extend(true, {}, node);
            this.diagram._updateChildren(node);
            return node;
        };
        ToolBase.prototype._updatePseudoGroup = function (psedoGroup) {
            var parent, child;
            if (psedoGroup) {
                this.diagram._isUndo = true;
                var collection = [];
                for (var i = 0; psedoGroup.children && psedoGroup.children.length > 0 && i < psedoGroup.children.length; i++) {
                    child = this.diagram.nameTable[this.diagram._getChild(psedoGroup.children[i])];
                    if (child) {
                        this.diagram._updateChildren(child);
                        collection.push(jQuery.extend(true, {}, child));
                        this._getAdjustjecentLines(child);
                        this.diagram.remove(child);
                        this.diagram._clearSelection(true);
                        this._updateNameTable(child);
                        parent = this.diagram.nameTable[child.parent];
                        child.parent = "";
                        this.diagram.nameTable[child.name] = child;
                        this.diagram.nodes().push(child);
                        this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                        if (child._type === "group") {
                            ej.datavisualization.Diagram.SvgContext.renderGroup(child, this.diagram._svg, this.diagram._diagramLayer, this.diagram.nameTable, this.diagram);
                        } else if (child.segments) {
                            ej.datavisualization.Diagram.SvgContext.renderConnector(child, this.diagram._svg, this.diagram._diagramLayer);
                        } else {
                            ej.datavisualization.Diagram.SvgContext.renderNode(child, this.diagram._svg, this.diagram._diagramLayer, undefined, this.diagram);
                        }
                    }
                }
                this._undoObject = {
                    collection: collection,
                    adjustLines: jQuery.extend(true, {}, this._adjustLines)
                };
                for (var i = 0; this._adjustLines.lines && this._adjustLines.lines.length > 0 && i < this._adjustLines.lines.length ; i++) {
                    this.diagram.add(this._adjustLines.lines[i]);
                    this.diagram._clearSelection(true);
                }
                this.diagram.nameTable[psedoGroup.name] = psedoGroup;
                this.diagram._addSelection(psedoGroup)
                this.diagram._isUndo = false;
            }
        };
        ToolBase.prototype._getParentSwimlane = function (lane) {
            var laneStack;
            if (lane && lane.isLane) {
                laneStack = this.diagram.nameTable[lane.parent];
                return this.diagram.nameTable[laneStack.parent];
            }
            return null;
        };
        ToolBase.prototype._removeHighLighter = function () {
            ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._adornerSvg, this.diagram._adornerLayer);
        };
        ToolBase.prototype._nodeHighLighter = function (overNode) {
            var source = null, childBounds = null, target = null;
            if (overNode && this.selectedObject.parent) //&& this.selectedObject.parent != overNode.name)
            {
                var parent = this.diagram.nameTable[this.selectedObject.parent];
                if (parent.container && parent.container.type == "stack" && this.selectedObject.name != overNode.name) {
                    ej.datavisualization.Diagram.SvgContext._drawStackHighlighter(overNode, this.diagram._adornerSvg, this.diagram._adornerLayer, this.diagram._currZoom, this.currentPoint, (parent.container.orientation === "vertical") ? true : false);
                }
                else if (parent.container && parent.container.type == "canvas" && this.selectedObject.parent != overNode.name && !overNode.isPhase) {
                    ej.datavisualization.Diagram.SvgContext._drawNodeHighlighter(overNode, this.diagram._adornerSvg, this.diagram._adornerLayer, this.diagram._currZoom);
                }
                else this._removeHighLighter();
            }
            else if (overNode && overNode.container && !overNode.isSwimlane) {
                if (this.selectedObject.isLane) {
                    parent = this.diagram.nameTable[overNode.parent];
                    if (overNode.name.indexOf("phaseStack") === -1)
                        ej.datavisualization.Diagram.SvgContext._drawStackHighlighter(overNode, this.diagram._adornerSvg, this.diagram._adornerLayer, this.diagram._currZoom, this.currentPoint, (parent.container.orientation === "vertical") ? true : false);
                }
                else
                    ej.datavisualization.Diagram.SvgContext._drawNodeHighlighter(overNode, this.diagram._adornerSvg, this.diagram._adornerLayer, this.diagram._currZoom);
            }
            else if (!overNode.segments) {
                ej.datavisualization.Diagram.SvgContext._drawNodeHighlighter(overNode, this.diagram._adornerSvg, this.diagram._adornerLayer, this.diagram._currZoom);
            }
            else
                this._removeHighLighter();
            if (this._nodeToHit && overNode && overNode.name != this._nodeToHit.name) {
                childBounds = ej.datavisualization.Diagram.Util.bounds(this._nodeToHit);
                target = this._getTargetNode(this._nodeToHit, childBounds);
                this._raiseEvent("mouseLeave", { element: this.diagram.getNode(this.selectedObject), source: this.diagram.getNode(this._nodeToHit), target: this.diagram.getNode(target) });
            }
            if (overNode) {
                if (this._nodeToHit && this._nodeToHit.name !== overNode.name || !this._nodeToHit) {
                    this._nodeToHit = overNode;
                    if (this.selectedObject.parent)
                        source = this.diagram.nameTable[this.selectedObject.parent];
                    if (this.selectedObject.type == "pseudoGroup") {
                        var fChild = this.diagram.nameTable[this.diagram._getChild(this.selectedObject.children[0])];
                        if (fChild && fChild.parent)
                            source = this.diagram.nameTable[fChild.parent];
                    }
                    childBounds = ej.datavisualization.Diagram.Util.bounds(overNode);
                    target = this._getTargetNode(overNode, childBounds);
                    this._raiseEvent("mouseEnter", { element: this.diagram.getNode(this.selectedObject), source: this.diagram.getNode(source), target: this.diagram.getNode(target) });
                }
            }
            else {
                this._nodeToHit = null;
            }
            if (overNode) {
                if (this.selectedObject.parent)
                    source = this.diagram.nameTable[this.selectedObject.parent];
                childBounds = ej.datavisualization.Diagram.Util.bounds(overNode);
                target = this._getTargetNode(overNode, childBounds);
                this._raiseEvent("mouseOver", { element: this.diagram.getNode(this.selectedObject), source: this.diagram.getNode(source), target: this.diagram.getNode(target) });
            }
        };

        ToolBase.prototype._doubleClick = function (element) {
            if (element && element.parent) {
                var parent = this.diagram.nameTable[element.parent];
                if (parent) {
                    var name = element.name;
                    this.diagram.updateSelectedObject(name);
                    if (!parent.isLane && !parent.isSwimlane && !parent.isPhaseStack) {
                        this.diagram._clearSelection(true);
                        this.diagram.addSelection(element, true);
                    }
                }
            }
        };

        ToolBase.prototype._itemClick = function (element, actualObject, selectedObject) {
            if (actualObject && actualObject.parent) {
                var parent = this.diagram.nameTable[actualObject.parent];
                if (parent && parent.container) {
                    if (actualObject && !this.diagram._selectedSymbol) {
                        var name = actualObject.name;
                        if (name.indexOf("_header_swimlane") != -1) {
                            if (element)
                                this.diagram.updateSelectedObject(element.parent, parent);
                        }
                        else if (name.indexOf("_Headerr_") != -1)
                            this.diagram.updateSelectedObject(actualObject.parent);
                        else if (actualObject && actualObject.isPhase) {
                            if (name.indexOf("phaseStack") != -1) {
                                var obj = this.diagram.nameTable[name.split("phaseStack")[1]];
                                if (obj && (this.diagram.selectionList.length === 0 || this.diagram.selectionList[0].name != obj.name)) {
                                    this.diagram._clearSelection(true);
                                    this.diagram.addSelection(obj, true);
                                    this.selectedObject = null;
                                }
                            }
                        }
                        else
                            this.diagram.updateSelectedObject(name);
                    }
                }
            }
            else if (actualObject) {
                this.diagram.updateSelectedObject(actualObject.name);
            }
            if (element && (element.type == "pseudoGroup" || element.name == "multipleSelection") && !this._containsChild(element, actualObject.name)) {
                this.diagram.updateSelectedObject(element.name)
            }
        };



        ToolBase.prototype._getTargetNode = function (node, childBounds) {
            var bounds = null, nodes = [];
            var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(this.diagram._spatialSearch, childBounds);
            for (var i = 0; i < quads.length; i++) {
                var quad = quads[i];
                if (quad.objects.length > 0) {
                    for (var j = 0; j < quad.objects.length; j++) {
                        var nd = quad.objects[j];
                        if (!nd.segments && nd.visible) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(nd);
                            if (nodes.indexOf(nd) == -1 && ej.datavisualization.Diagram.Geometry.intersectsRect(childBounds, bounds))
                                nodes.push(nd);

                        }
                    }
                }
            }
            var collection = [];
            collection.push(node);
            for (i = 0; i < nodes.length; i++) {
                if (nodes[i].name != node.name)
                    collection.push(nodes[i]);
            }
            return collection;
        };
        ToolBase.prototype._updateMargin = function (node, group) {
            if (node.type == "pseudoGroup") {
                var child = null;
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    this._updateMargin(child, group);
                }
            }
            else if (group && group.container && group.container.type == "canvas") {
                var groupBounds = ej.datavisualization.Diagram.Util.bounds(group); //ej.datavisualization.Diagram.SvgContext.getCanvasBoundingBox(group, this.diagram, node, true);
                var bounds1 = ej.datavisualization.Diagram.Util.bounds(node);
                var bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
                node.marginLeft = bounds.x - (groupBounds.x + group.paddingLeft);
                node.marginRight = (groupBounds.x + group.width) - (bounds.x + bounds.width) - group.paddingRight;
                node.marginTop = bounds.y - (groupBounds.y + group.paddingTop);
                node.marginBottom = groupBounds.bottom - (bounds.y + bounds.height) - group.paddingBottom;
            }
        };
        ToolBase.prototype._getNodeUnderMouse = function (evt) {
            var node = this.diagram._nodeUnderMouse; var obj, skip;
            if (this.diagram._focusedElement && (this.diagram._focusedElement.isPhase || this.diagram._focusedElement.isLane || this.diagram._focusedElement.isSwimlane)) skip = true;
            if (this.diagram._focusedElement && node && this._containsChild(this.diagram._focusedElement, node.name)) {
                if (node && (skip || !ej.datavisualization.Diagram.Util.canAllowDrop(node))) {
                    var type = this.diagram.getObjectType(node);
                    if (type == "connector" || type === "group") {
                        obj = node;
                    }
                    else {
                        if (node.parent) {
                            var parent1 = this.diagram.nameTable[node.parent];
                            if (parent1 && parent1.container && parent1.container.type == "stack" && ej.datavisualization.Diagram.Util.canAllowDrop(node)) {
                                return obj = node;
                            }
                            type = this.diagram.getObjectType(parent1);
                            if (parent1 && type != "node") {
                                obj = parent1;
                            }
                        }
                    }
                }
                if (!obj) {
                    if (node && this.selectedObject != node && ej.datavisualization.Diagram.Util.canAllowDrop(node)) return node;
                }
                else if (this.selectedObject != obj && ej.datavisualization.Diagram.Util.canAllowDrop(obj))
                    return obj;
            }
        };

        ToolBase.prototype._isChange = function (obj1, obj2) {
            var status = null;
            if (obj1._type === "group" || obj1.type === "pseudoGroup" || obj2._type === "group" || obj2.type === "pseudoGroup") {
                if ((obj1._type === "group" || obj1.type === "pseudoGroup") && !(obj2._type === "group" || obj2.type === "pseudoGroup")) {
                    status = !this._containsChild(obj1, obj2.name);
                }
                if (!(obj1._type === "group" || obj1.type === "pseudoGroup") && (obj2._type === "group" || obj2.type === "pseudoGroup")) {
                    status = !this._containsChild(obj2, obj1.name);
                }
                else if ((obj1._type === "group" || obj1.type === "pseudoGroup") && (obj2._type === "group" || obj2.type === "pseudoGroup")) {
                    if (obj1.name == obj2.name)
                        status = true;
                }
            }
            else {
                if (obj1.name == obj2.name)
                    status = true;
            }
            return status;
        };
        ToolBase.prototype._containsChild = function (node, childName) {
            var status = true;
            var child = null;
            var type;
            if (node.children) {
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    type = this.diagram.getObjectType(child);
                    if (type === "group")
                        status = this._containsChild(child, childName);
                    else {
                        if (child && child.name == childName)
                            status = false;
                    }
                }
            }
            return status;
        };

        ToolBase.prototype._findNodeUnderMouse = function (evt, skip) {
            var obj = null;
            var node = null;
            var type;
            var foreignObject = this.diagram._isForeignObject(evt.target);
            if (foreignObject)
                evt.target = foreignObject;
            var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector,.ej-d-group");
            if (parent && (foreignObject || parent.context.localName != "div")) {
                type = parent.attr("class");
                if (type === "ej-d-node" || type === "ej-d-group") {
                    node = this.diagram._findNode(parent.attr("id"));
                    if (!node) node = this.diagram._findNode(parent.attr("id").split('_parentdiv')[0]);
                } else if (type === "ej-d-connector") {
                    node = this.diagram._findConnector(parent.attr("id"));
                }
            }
            if (node && !node.parent)
                obj = node;
            else
                obj = this._decideSelectedItem(evt, node, skip);
            this.prevSelectObject = obj;

            if (skip && this.diagram.selectionList[0]) {
                if (obj && obj._type === "group" && !this._containsChild(obj, this.diagram.selectionList[0].name))
                    obj = obj;
                else if (obj && this.diagram.selectionList[0].name != obj.name && this._isChange(this.diagram.selectionList[0], obj))
                    obj = this.diagram.selectionList[0];
                else if (node && node.name == this.diagram.selectionList[0].name && !this._isChange(this.diagram.selectionList[0], obj)) {
                    obj = this.diagram.selectionList[0];

                }
            }
            this.actualObject = node;
            return obj;

        };

        ToolBase.prototype._findLabelUnderMouse = function (evt, skip, mousePosition) {
            var obj;
            var target = $(evt.target);
            var targetClass = target[0].getAttribute("class");
            if (targetClass == "ej-d-label") {
                var parent = $(evt.target).parents(".ej-d-node,.ej-d-connector,.ej-d-group");
                if (parent) {
                    var id = parent[0].getAttribute("id");
                    var type = parent[0].getAttribute("class");
                    if (type) {
                        if (type === "ej-d-node" || type === "ej-d-group") {
                            obj = this.diagram._findNode(id);
                            var curlabel = this.diagram._findLabelAtPoint(mousePosition ? mousePosition : this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                        else if (type === "ej-d-connector") {
                            obj = this.diagram._findConnector(id);
                            var curlabel = this.diagram._findLabelAtPoint(mousePosition ? mousePosition : this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                    }
                }
            }
            else {
                this.activeLabel = null;
            }
            return obj;
        };
        ToolBase.prototype._selectionContainsChild = function (name) {
            if (this.diagram.selectionList[0].children) {
                var children = this.diagram._getChildren(this.diagram.selectionList[0].children);
                var child = null;
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child && child.name == name)
                        return true;
                }
            }
            else if (this.diagram.selectionList[0]._type === "group" && this.diagram.selectionList[0].type != "pseudoGroup") {
                if (this.diagram.selectionList[0].name == name)
                    return true;
            }
            else if (this.diagram.nameTable[name] && this.diagram.nameTable[name]._type === "group") {
                children = this.diagram._getChildren(this.diagram.nameTable[name].children);
                for (var j = 0; j < children.length; j++) {
                    child = this.diagram.nameTable[children[j]];
                    if (child)
                        var status = this._selectionContainsChild(child.name);
                    if (status)
                        return status;
                }
            }
            else if (this.diagram.selectionList[0].name == name)
                return true;

            return false;
        };

        ToolBase.prototype._isSelected = function (name) {
            var child = null;
            var node = this.diagram.nameTable[name];
            if (this._selectionContainsChild(node.name))
                return true;
            else if (node._type === "group") {
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = children[i];
                    if (this._selectionContainsChild(child))
                        return true;
                }
            }
            return false;
        };
        ToolBase.prototype._isColleagueSelected = function (name) {
            var node = this.diagram.nameTable[name];
            var child = null;
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (parent) {
                    var children = this.diagram._getChildren(parent.children);
                    for (var i = 0; i < children.length; i++) {
                        child = children[i];
                        if (this._isSelected(child))
                            return true;
                    }
                }
            }
            return false;
        };
        ToolBase.prototype._isInSelection = function (name) {
            if (this.diagram.selectionList[0].type == "pseudoGroup" && this.diagram.selectionList[0].children) {
                var children = this.diagram._getChildren(this.diagram.selectionList[0].children);
                var child = null;
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child.name == name)
                        return true;
                }
            }
        };
        ToolBase.prototype._getCloneNode = function (node) {
            var obj = null;
            obj = jQuery.extend(true, {}, node);
            obj.children = [];
            obj.minHeight = 0;
            obj.minWidth = 0;
            obj.maxHeight = 0;
            obj.maxWidth = 0;
            return obj;
        };
        ToolBase.prototype._getProcessedObject = function (name, obj) {
            var node = this.diagram.nameTable[name];
            if (node) {
                if (this._isInSelection(name)) {
                    return null;
                }
                else if (this._selectionContainsChild(node.name))
                    return { add: node, remove: null };
                else {
                    if (node.parent) {
                        if (this._selectionContainsChild(node.parent)) {
                            if (this._isInSelection(node.parent))
                                return { add: node, remove: this.diagram.nameTable[node.parent] };
                            else {
                                var parent = this.diagram.nameTable[node.parent];
                                if (parent)
                                    return { add: parent, remove: this.diagram.nameTable[parent.parent] };
                            }
                        } else {
                            if (this._isColleagueSelected(node.parent) && !this._selectionContainsChild(node.parent)) {
                                if (this._isSelected(node.parent)) {
                                    return { add: node, remove: null };
                                }
                                else
                                    return { add: this.diagram.nameTable[node.parent], remove: null };
                            }
                            else if (this.diagram.selectionList[0] && this._isInSelection(node.parent)) {
                                return { add: this.diagram.nameTable[node.name], remove: this.diagram.nameTable[node.parent] };

                            }
                            else
                                return this._getProcessedObject(node.parent);
                        }
                    }
                    else {
                        return { add: node, remove: null };
                    }
                }
            }
        };

        ToolBase.prototype._measureStackBounds = function (group) {
            var child = null;
            var bounds;
            var rect = null;
            var children = this.diagram._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                child = this.diagram.nameTable[children[i]];
                bounds = ej.datavisualization.Diagram.Util.bounds(child);

                if (bounds) {
                    if (!rect)
                        rect = bounds;
                    else
                        rect = this.diagram._union(bounds, rect);
                }
            }
            rect.width += group.paddingLeft + group.paddingRight;
            rect.height += group.paddingTop + group.paddingBottom;
            return rect;
        };
        ToolBase.prototype.hasSameParent = function () {
            if (this.selectedObject.type == "pseudoGroup") {
                var list = this.selectedObject;
                var child = null;
                if (this.diagram.nameTable[this.diagram._getChild(list.children[0])]) {
                    var parent = this.diagram.nameTable[this.diagram._getChild(list.children[0])].parent;
                    var lChildren = this.diagram._getChildren(list.children);
                    for (var i = 0; i < lChildren.length; i++) {
                        child = this.diagram.nameTable[lChildren[i]];
                        if (child.parent != parent || child.parent == "")
                            return false;
                    }
                }
                return true;
            }
            return false;
        },
            ToolBase.prototype._fromDiagram = function (list) {
                var state = true;
                var child = null;
                for (var i = 0; i < list.length; i++) {
                    child = this.diagram.nameTable[this.diagram._getChild(list[i])];
                    if (child && child.parent) {
                        state = false;
                        break;
                    }
                }
                return state;
            },
            ToolBase.prototype._fromContainer = function (list) {
                var child = null, parent = null;
                for (var i = 0; i < list.length; i++) {
                    child = this.diagram.nameTable[this.diagram._getChild(list[i])];
                    if (child.parent) {
                        parent = this.diagram.nameTable[child.parent];
                        if (parent) {
                            if (parent.container)
                                return true;
                        }
                    }
                }
            };

        ToolBase.prototype._fromSameContainer = function (list) {
            var child = null;
            var parent = null;
            for (var i = 0; i < list.length; i++) {
                child = this.diagram.nameTable[this.diagram._getChild(list[i])];
                if (!parent && child.parent)
                    parent = child.parent;
                if (child.parent) {
                    if (child.parent != parent)
                        return false;
                }
            }
            return true;
        },

      ToolBase.prototype._processCtrlKey = function (evt) {
          var obj;
          if (this.diagram.selectionList.length > 0 && ej.datavisualization.Diagram.Util.canDoMultipleSelection(this.diagram)) {
              var selectionList = this.diagram.selectionList;
              var isMultipleSelection, nObj, changeType, newItems = [], oldItems = [], selectedItems = [];
              if (selectionList[0].type == "pseudoGroup") {
                  isMultipleSelection = true;
                  obj = this._findNodeUnderMouse(evt);
              }
              else {

                  obj = this._findNodeUnderMouse(evt);
              }
              var obj1 = this._findNodeUnderMouse(evt, true);
              var tempVal = this.selectedObject;
              var args = this._raiseEvent("itemClick", { element: this.selectedObject, actualObject: this.actualObject, selectedObject: obj1, cancel: false, event: evt });
              this._itemClick(this.selectedObject, this.actualObject, obj1);
              if (args.cancel)
                  this.selectedObject = tempVal;
              if (this.actualObject) {
                  if (this.actualObject.parent) {
                      if (isMultipleSelection) {

                          var mulSelection = selectionList[0];
                          var temp = this._getProcessedObject(this.actualObject.name);
                          if (temp) {
                              if (this.selectedObject) {
                                  mulSelection.children = this.diagram._getChildren(mulSelection.children);
                                  ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, this.selectedObject.name);
                              }
                              else if (temp.remove) {
                                  mulSelection.children = this.diagram._getChildren(mulSelection.children);
                                  ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, temp.remove.name);
                              }


                              if (this.selectedObject) {
                                  if (!this._hasMultipleSelection(this.selectedObject)) {
                                      mulSelection.children.push(this.selectedObject.name);
                                  }
                              } else if (temp.add) {
                                  if (!this._hasMultipleSelection(temp.add)) {
                                      mulSelection.children.push(temp.add.name);
                                  }
                              }
                          } else {
                              nObj = this.actualObject;
                              if (this.selectedObject) {
                                  if (!this._hasMultipleSelection(this.selectedObject))
                                      mulSelection.children.push(this.selectedObject.name);
                                  else {
                                      mulSelection.children = this.diagram._getChildren(mulSelection.children);
                                      ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, this.selectedObject.name);
                                  }
                              } else {
                                  if (!this._hasMultipleSelection(nObj))
                                      mulSelection.children.push(nObj.name);
                                  else {
                                      mulSelection.children = this.diagram._getChildren(mulSelection.children);
                                      ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, nObj.name);
                                  }
                              }
                          }
                          ej.datavisualization.Diagram.Util._updateGroupBounds(mulSelection, this.diagram);
                          if (mulSelection.children.length <= 1) {
                              var child = this.diagram.nameTable[this.diagram._getChild(mulSelection.children[0])];
                              if (this.diagram._hasSelection()) {
                                  this.diagram._clearSelection(true);
                              }
                              this.selectedObject = child;
                              this.diagram._addSelection(this.selectedObject);
                          } else {
                              this.selectedObject = selectionList[0];
                              ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectedObject, this.diagram._adornerSvg, this.diagram._currZoom, this.diagram, this.diagram.model.selectedItems.constraints);
                          }

                      } else {
                          if (this.diagram.selectionList[0]._type === "group" && this.diagram.selectionList[0].type !== "bpmn") {
                              selectionList = this.diagram.selectionList[0];
                              temp = this._getProcessedObject(this.actualObject.name);
                              nObj = temp.add;
                              if (temp) {
                                  if (this.diagram._hasSelection()) {
                                      this.diagram._clearSelection();
                                  }
                              }

                          } else {
                              temp = null;
                              if (obj._type === "group") {
                                  temp = this._getProcessedObject(this.actualObject.name);
                              }
                              if (temp)
                                  nObj = temp.add;
                              else
                                  nObj = obj;
                              selectionList = this.diagram.selectionList[0];
                              if (this.diagram._hasSelection()) {
                                  this.diagram._clearSelection();
                              }
                              pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                              if (selectionList.name != nObj.name)
                                  pseudoGroup.children.push(selectionList.name);
                              if (this.selectedObject)
                                  pseudoGroup.children.push(this.selectedObject.name);
                              else
                                  if (this._containsChild(pseudoGroup, nObj.name))
                                      pseudoGroup.children.push(nObj.name);

                              this._checkRepeatedChild(pseudoGroup.children);

                              this.diagram.nodes().push(pseudoGroup);
                              this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                              this.diagram.nameTable[pseudoGroup.name] = pseudoGroup;
                              ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.diagram);
                              this.selectedObject = pseudoGroup;
                              this.diagram._addSelection(this.selectedObject);
                          }
                      }
                  }
                  else {
                      if (isMultipleSelection) {
                          mulSelection = selectionList[0];
                          nObj = this.selectedObject ? this.selectedObject : this.actualObject;
                          changeType = (!this._hasMultipleSelection(nObj)) ? "insert" : "remove";
                          if (changeType == "insert") {
                              mulSelection.children.push(nObj.name);
                              newItems.push(nObj);
                          }
                          else {
                              ej.datavisualization.Diagram.Util.removeItem(mulSelection.children, nObj.name);
                              oldItems.push(nObj);
                          }
                          for (var i = 0; i < mulSelection.children.length; i++)
                              selectedItems.push(this.diagram.nameTable[mulSelection.children[i]]);
                          args = { changeType: changeType, element: this.diagram.nameTable["multipleSelection"], selectedItems: selectedItems, oldItems: oldItems, newItems: newItems, cancel: false };
                          this._raiseEvent("selectionChange", args);
                          if (!args.cancel) {
                              if (changeType == "remove") ej.datavisualization.Diagram.Util.removeItem(this.diagram.model.selectedItems.children, nObj);
                              this.diagram.updateSelection(this.selectedObject);
                              ej.datavisualization.Diagram.Util._updateGroupBounds(mulSelection, this.diagram);
                              this.selectedObject = selectionList[0];
                              if (this.diagram.model.selectedItems.children.length)
                                  ej.datavisualization.Diagram.SvgContext.updateSelector(this.selectedObject, this.diagram._adornerSvg, this.diagram._currZoom, this.diagram, this.diagram.model.selectedItems.constraints);
                              else {
                                  ej.datavisualization.Diagram.Util.clear(this.diagram.selectionList);
                                  ej.datavisualization.Diagram.SvgContext.clearSelector(this.diagram._adornerSvg, this.diagram._adornerLayer);
                              }
                          }
                      }
                      else {
                          var nObj = this.actualObject;
                          selectionList = this.diagram.selectionList[0];
                          if (nObj != selectionList) {
                              if (this.diagram._hasSelection()) {
                                  this.diagram._clearSelection(true);
                              }
                              var pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                              pseudoGroup.children.push(selectionList.name);
                              if (this.selectedObject)
                                  pseudoGroup.children.push(this.selectedObject.name);
                              else
                                  pseudoGroup.children.push(nObj.name);
                              this.diagram.nodes().push(pseudoGroup);
                              this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                              this.diagram.nameTable[pseudoGroup.name] = pseudoGroup;
                              ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.diagram, null, true);
                              this.selectedObject = pseudoGroup;
                              this.diagram._addSelection(this.selectedObject);
                          }
                      }
                  }
              }

              this.selectedObject = this.diagram.selectionList[0];
          }
          else {
              obj = this._findNodeUnderMouse(evt);
              this.selectedObject = obj;
          }
          return obj;
      };

        ToolBase.prototype._checkRepeatedChild = function (list) {
            var child = null;
            var cloneList = $.extend(true, {}, list);
            cloneList = this.diagram._getChildren(cloneList);
            for (var i = 0; i < cloneList.length; i++) {
                child = this.diagram.nameTable[cloneList.children[i]];
                if (child._type === "group") {
                    for (var j = 0; j < cloneList.children.length; j++) {
                        if (!this._containsChild(child, cloneList.children[j])) {
                            list.children = this.diagram._getChildren(list.children);
                            ej.datavisualization.Diagram.Util.removeItem(list.children, cloneList.children[j]);
                        }
                    }
                }
            }
        };
        ToolBase.prototype._hasMultipleSelection = function (node) {
            var children = this.diagram._getChildren(this.diagram.selectionList[0].children);
            for (var i = 0; i < children.length; i++) {
                if (node.name == children[i])
                    return true;
            }
            return false;
        };

        ToolBase.prototype._getNextParent = function (node) {
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (this._hasMultipleSelection(parent))
                    return node;
                else
                    return this._getNextParent(parent);
            }
            else
                return node;
        };
        ToolBase.prototype._hasMultipleSelection = function (node) {
            var children = this.diagram._getChildren(this.diagram.selectionList[0].children);
            for (var i = 0; i < children.length; i++) {
                if (node.name == children[i])
                    return true;
            }
            return false;
        };

        ToolBase.prototype._getNextParent = function (node) {
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (this._hasMultipleSelection(parent))
                    return node;
                else
                    return this._getNextParent(parent);
            }
            else
                return node;
        };
        ToolBase.prototype._findObj = function (element, group) {
            var obj;
            var id = element.getAttribute("id");
            if (group) {
                obj = this.diagram._findChildren(group, id);
            }
            else {
                var type = element.getAttribute("class");
                if (type) {
                    if (type === "ej-d-node" || type === "ej-d-group") {
                        obj = this.diagram._findNode(id);
                    }
                    else if (type === "ej-d-connector") {
                        obj = this.diagram._findConnector(id);
                    }
                }
            }
            return obj;
        };
        ToolBase.prototype.mousedown = function (evt) {
            this.startPoint = this.mousePosition(evt);
            this.currentPoint = this.startPoint;
            this.previousPoint = this.startPoint;
            this.diagramBounds = this.diagram._getDigramBounds();
        };

        ToolBase.prototype.mousemove = function (evt) {
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(this.startPoint) ||
                this instanceof ej.datavisualization.Diagram.LineTool)
                this.currentPoint = this.mousePosition(evt);
        };
        ToolBase.prototype.mouseup = function (evt) {
            this._endAction();
            if (this.diagram._pageBackgroundLayer) {
                ej.datavisualization.Diagram.PageUtil._updatePageSize(this.diagram);
            }
            if (this.singleAction) {
                this.diagram.activateTool("select");
            }
            delete this.diagramBounds;
            this._adjustLines = {
                lines: []
            }
            this._undoObject = null;
        };
        ToolBase.prototype.mousePosition = function (evt) {
            return this.diagram._mousePosition(evt);
        };
        ToolBase.prototype.updateCursor = function (cursor) {
            this.diagram._currentCursor = cursor;
            this.diagram._updateCursor();
        };
        ToolBase.prototype.nearestGridPoint = function (point) {
            var snapPt = ej.datavisualization.Diagram.Point();
            if (this.diagram.model.snap & ej.datavisualization.Diagram.Snap.Grid) {
                var magnification = this.diagram.model.magnification / 100;
                var hSpacing = this.diagram.model.grid.horizontalSpacing * magnification;
                var vSpacing = this.diagram.model.grid.verticalSpacing * magnification;
                var width = point.x % hSpacing;
                var height = point.y % vSpacing;
                if (width >= (hSpacing / 2)) {
                    snapPt.x = point.x + hSpacing - width;
                }
                else {
                    snapPt.x = point.x - width;
                }
                if (height >= (vSpacing / 2)) {
                    snapPt.y = point.y + vSpacing - height;
                }
                else {
                    snapPt.y = point.y - height;
                }
            }
            return snapPt;
        };
        ToolBase.prototype.snap = function (point) {
            var snapSettings = this.diagram.model.snapSettings;

            var zoomFactor = this.diagram._currZoom;
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines)
                point.x = ej.datavisualization.Diagram.SnapUtil._round(point.x, snapSettings.verticalGridLines.snapInterval, zoomFactor);
            if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines)
                point.y = ej.datavisualization.Diagram.SnapUtil._round(point.y, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
            return point;

        };
        ToolBase.prototype._endAction = function () {
            this.inAction = false;

            if (this.svgHelper) {
                var helper = this.helper;
                var search = this.diagram._spatialSearch;
                if (helper._type != "group") {
                    ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(search, search.quadTable[helper.name], helper);
                }
                else {
                    var length = this.helper.children.length - 1;
                    for (var i = length; i >= 0; i--) {
                        ej.datavisualization.Diagram.SpatialUtil._removeFromaQuad(search, search.quadTable[helper.children[i].name], helper.children[i]);
                    }
                }
            }
            if (this.helper) {
                if (this.svgHelper) {
                    var diagram = this.diagram;
                    var _currTool = this;
                    this.diagram._views.forEach(function (viewid) {
                        var view = diagram._views[viewid];
                        if (_currTool.helper) {
                            var element = view.svg.getElementById(_currTool.helper.name);
                            var htmlLayer = view.svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                            var label = $(htmlLayer).find("#" + _currTool.helper.name)[0];
                        }
                        if (element && element.parentNode)
                            element.parentNode.removeChild(element);
                        if (label)
                            htmlLayer.removeChild(label);

                    });
                }
                if (this.helper.gradient) {
                    var defs = this.diagram._svg.getElementsByTagName("defs")[0];
                    var hlprEle = this.diagram._svg.getElementById(this.helper.name + "_gradient");
                    if (defs && hlprEle)
                        defs.removeChild(this.diagram._svg.getElementById(this.helper.name + "_gradient"));
                }
                var htmlElement = document.getElementById(this.helper.name + "_parentdiv");
                if (htmlElement)
                    htmlElement.parentNode.removeChild(htmlElement);
            }
            if (this._previousPossibleConnection) {
                this._previousPossibleConnection = null;
                ej.datavisualization.Diagram.SvgContext._removePortHighlighter(this.diagram._adornerSvg, this.diagram._adornerLayer);
            }
            this._currentPossibleConnection = null;
            this._possibleConnectionPort = null;
            this.helper = null;
            this.svgHelper = null;
            this.selectedObject = null;
            this.startPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.currentPoint = ej.datavisualization.Diagram.Point(0, 0);
            this.previousPoint = ej.datavisualization.Diagram.Point(0, 0);
        };
        ToolBase.prototype._isResizeCorner = function (type) {
            var isResizeCorner = false;
            switch (type) {
                case "n-resize":
                case "e-resize":
                case "w-resize":
                case "s-resize":
                case "ne-resize":
                case "nw-resize":
                case "se-resize":
                case "sw-resize":
                case "targetEndPoint":
                case "sourceEndPoint":
                    isResizeCorner = true;
                    break;
            }
            return isResizeCorner;
        };
        ToolBase.prototype._initHelper = function () {
            if (this.selectedObject) {
                this.helper = $.extend(true, {}, this.selectedObject);
                this._updateHelperName(this.helper);
            }
        };
        ToolBase.prototype._updateHelperName = function (helper, child) {
            if (child) {
                helper.name += "helper";
            }
            else
                helper.name = "helper";
            helper.opacity = 0.5;
            var labels = helper.labels;
            for (var i = 0, len = labels.length; i < len; ++i) {
                labels[i].name += "helper";
            }
            if (helper.ports) {
                var ports = helper.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    ports[i].name += "helper";
                }
            }
            if (helper._type === "group") {
                var children = this.diagram._getChildren(helper.children);
                for (i = 0, len = children.length; i < len; i++) {
                    this._updateHelperName(children[i], true);
                }
            }
        };
        ToolBase.prototype._raiseEvent = function (type, args) {
            if (this.diagram.model[type]) {
                args.elementType = this.diagram.getObjectType(args.element);
                this.diagram._trigger(type, args);
            }
            return args;
        };
        ToolBase.prototype._findPort = function (pt, node) {
            var port = this.diagram._findPortAtPoint(pt, node);
            if (!port && node._type === "group") {
                var childPort;
                var children = this.diagram._getChildren(node.children);
                for (var j = 0; j < children.length; j++) {
                    var child = this.diagram.nameTable[children[j]];
                    if (child && !child.segments) {
                        childPort = this._findPort(pt, child);
                        if (childPort && child._type != "group") {
                            this._currentPossibleConnection = child;
                            port = childPort;
                        }
                        else if (childPort) {
                            return childPort;
                        }
                    }
                }
            }
            return port;
        };
        ToolBase.prototype._checkConnectionPossible = function (evt) {
            var port = null;
            var pt = ej.datavisualization.Diagram.Point(this.currentPoint.x, this.currentPoint.y);
            var node = this.diagram._findConnectableNodeUnderMouse(evt);
            if (node && !(node.segments)) {
                if (!(node.type == "pseudoGroup") && !node.isLane && !node.isSwimlane) {
                    this._showPorts(node);
                    this._currentPossibleConnection = node;
                    if (this._previousPossibleConnection == null || this._previousPossibleConnection.name !== this._currentPossibleConnection.name) {
                        this._previousPossibleConnection = this._currentPossibleConnection;
                    }
                }
                else {
                    this._currentPossibleConnection = null;
                    this._possibleConnectionPort = null;
                }
            }
            else if (this.diagram.activeTool instanceof ej.datavisualization.Diagram.LineTool) {
                this._showPorts();
                this._currentPossibleConnection = null;
                this._possibleConnectionPort = null;
            }
            else {
                var epoint = ej.datavisualization.Diagram.Point(0, 0);
                if (this._endPoint == "targetEndPoint") {
                    epoint.x = this.selectedObject.targetPoint.x - 25;
                    epoint.y = this.selectedObject.targetPoint.y - 25;
                }
                else if (this._endPoint == "sourceEndPoint") {

                    epoint.x = this.selectedObject.sourcePoint.x - 25;
                    epoint.y = this.selectedObject.sourcePoint.y - 25;
                }
                var bounds = ej.datavisualization.Diagram.Rectangle(epoint.x, epoint.y, 50, 50);
                var quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(this.diagram._spatialSearch, bounds);
                for (var i = 0; i < this._nearestNodes.length; i++)
                    this._showPort(this._nearestNodes[i], true);
                this._nearestNodes = [];
                for (var i = 0; i < quads.length; i++) {
                    var quad = quads[i];
                    if (quad.objects.length > 0) {
                        for (var j = 0; j < quad.objects.length; j++) {
                            var nd = quad.objects[j];
                            var nodebounds = ej.datavisualization.Diagram.Util.bounds(nd);
                            if (ej.datavisualization.Diagram.Geometry.intersectsRect(bounds, nodebounds)) {
                                if (!nd.segments && nd.visible) {
                                    this._showPort(nd);
                                    this._nearestNodes.push(nd);
                                }
                            }
                            else {
                                if (!nd.segments) {
                                    this._showPort(nd, true);
                                }
                            }
                        }
                    }
                }

                this._currentPossibleConnection = null;
                this._possibleConnectionPort = null;
            }
            if (this._currentPossibleConnection) {
                port = this._findPort(pt, this._currentPossibleConnection);
                if ((port && ej.datavisualization.Diagram.Util.canConnect(port, true)) || ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection)) {
                    ej.datavisualization.Diagram.SvgContext._drawPortHighlighter(port, this._currentPossibleConnection, this.diagram._adornerSvg, this.diagram._adornerLayer,
                        this.diagram._currZoom);
                }
                else {
                    ej.datavisualization.Diagram.SvgContext._removePortHighlighter(this.diagram._adornerSvg, this.diagram._adornerLayer);
                }
                this._possibleConnectionPort = port;
                if (port && ej.datavisualization.Diagram.Util.canConnect(port, true)) {
                    var connectionNode = this._currentPossibleConnection;
                    var point = ej.datavisualization.Diagram.Util._getPortPosition(this._possibleConnectionPort, ej.datavisualization.Diagram.Util.bounds(connectionNode, true));
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, connectionNode.rotateAngle, connectionNode.offsetX, connectionNode.offsetY);
                    point = ej.Matrix.transform(matrix, point);
                    this.currentPoint = point;
                }
            }
            else {
                if (this._previousPossibleConnection) {
                    this._previousPossibleConnection = null;
                    ej.datavisualization.Diagram.SvgContext._removePortHighlighter(this.diagram._adornerSvg, this.diagram._adornerLayer);
                    this._possibleConnectionPort = null;
                }
            }

        };
        ToolBase.prototype._showPorts = function (node) {
            if (node) {
                if (this._currentPossibleConnection && this._currentPossibleConnection.name !== node.name) {
                    this._showPort(this._currentPossibleConnection, true);
                }
                this._showPort(node);
            }
            else {
                if (this._currentPossibleConnection != null) {
                    this._showPort(this._currentPossibleConnection, true);
                }
            }
        };
        ToolBase.prototype._showPort = function (node, hide) {
            var ports;
            var port;
            var portShape;
            var i;
            var len;
            if (hide) {
                ports = node.ports;
                for (i = 0, len = ports.length; i < len; ++i) {
                    port = ports[i];
                    if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover ||
                        (port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect &&
                        (this.name == "endPoint" || this instanceof ej.datavisualization.Diagram.LineTool))) {
                        portShape = this.diagram._svg.getElementById(node.name + "_" + port.name);
                        if (portShape)
                            portShape.setAttribute("visibility", "hidden");
                    }
                }
            }
            else {
                if (node.visible) {
                    ports = node.ports;
                    for (i = 0, len = ports.length; i < len; ++i) {
                        port = ports[i];
                        if (port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover ||
                            (port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect &&
                            (this.name == "endPoint" && this.inAction || this instanceof ej.datavisualization.Diagram.LineTool))) {
                            portShape = this.diagram._svg.getElementById(node.name + "_" + port.name);
                            if (portShape)
                                portShape.setAttribute("visibility", "visible");
                        }
                    }
                }
            }
        };
        ToolBase.prototype._showAllPorts = function (hide) {
            var i;
            var len;
            var nodes = this.diagram.nodes();
            if (!ej.datavisualization.Diagram.Util.isPageEditable(this.diagram)) hide = true;
            if (hide) {
                for (i = 0, len = nodes.length; i < len; i++) {
                    this._showPort(nodes[i], hide);
                }
            }
            else {
                for (i = 0, len = nodes.length; i < len; ++i) {
                    this._showPort(nodes[i]);
                }
            }
        };
        ToolBase.prototype._disconnect = function (connector) {
            var args;
            var node = this.diagram.nameTable[connector.targetNode];
            var port = null;
            if (node) {
                port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.targetPort);
                args = this._raiseEvent("connectionChange", { element: connector, connection: node, port: port, cancel: false });
                if (!args.cancel) {
                    ej.datavisualization.Diagram.Util.removeItem(node.inEdges, connector.name);
                    connector.targetNode = null;
                    connector.targetPort = null;
                }
            }
            node = this.diagram.nameTable[connector.sourceNode];
            if (node) {
                port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.sourcePort);
                args = this._raiseEvent("connectionChange", { element: connector, connection: node, port: port, cancel: false });
                if (!args.cancel) {
                    ej.datavisualization.Diagram.Util.removeItem(node.outEdges, connector.name);
                    connector.sourceNode = null;
                    connector.sourcePort = null;
                }
            }
        };
        ToolBase.prototype.getLabelUnderMouse = function () {
            return this.activeLabel;
        };
        return ToolBase;
    })();

    ej.datavisualization.Diagram.ToolBase = ToolBase;
    //#endregion

    //#region SelectTool
    var SelectTool = (function (base) {
        ej.datavisualization.Diagram.extend(SelectTool, base);
        function SelectTool(diagram) {
            base.call(this, "select", diagram);
            this.cursor = "default";
            this._svgHelper = null;
        }
        SelectTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            var node = this._findNodeUnderMouse(evt);
            if (node) {
                this.selectedObject = node;
            }
            this._itemClick(this.selectedObject, this.actualObject, this.selectedObject);
            this._nodeUnderMouseDown = this.diagram._findNodeUnderMouse(evt);
        };
        SelectTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            var node = this._nodeUnderMouseDown;
            if ((!this.selectedObject && !ej.datavisualization.Diagram.Geometry.isEmptyPoint(this.startPoint)) ||
    (node && node.isLane && ej.datavisualization.Diagram.Util.canMultiSelectOnLane(node))) {
                if (ej.datavisualization.Diagram.Util.canDoMultipleSelection(this.diagram)) {
                    if (!this.inAction && (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                        this.inAction = true;
                        this._initHelper();
                    }
                    else
                        this._updateHelper();
                }
            }
        };
        SelectTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.currentPoint = this.mousePosition(evt);
                var rect = ej.datavisualization.Diagram.Geometry.rect([{ x: this.startPoint.x, y: this.startPoint.y }, { x: this.currentPoint.x, y: this.currentPoint.y }]);
                if (rect.width !== 0 || rect.height !== 0) {
                    var i;
                    var len;
                    var type;
                    var bounds;
                    var collection = [];
                    var nodes = this.diagram.nodes();
                    for (i = 0, len = nodes.length; i < len; i++) {
                        var node = this.diagram.nameTable[nodes[i].name];
                        if (node && node.visible) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(node);
                            bounds = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            if (ej.datavisualization.Diagram.Util.canSelect(node)) {
                                type = this.diagram.getObjectType(node);
                                if (ej.datavisualization.Diagram.Geometry.containsRect(rect, bounds)) {
                                    if (node.parent === "")
                                        collection.push(node);
                                }
                                else if (type === "group") {
                                    this._checkGroupChildren(collection, node, rect);
                                }
                            }
                        }
                    }
                    var connectors = this.diagram.connectors();
                    for (i = 0, len = connectors.length; i < len; i++) {
                        var connector = this.diagram.nameTable[connectors[i].name];
                        if (connector && connector.visible) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(connector);
                            bounds = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            if (ej.datavisualization.Diagram.Util.canSelect(connector) && ej.datavisualization.Diagram.Geometry.containsRect(rect, bounds)) {
                                if (connector.parent === "")
                                    collection.push(connector);
                            }
                        }
                    }
                    if (this.diagram._hasSelection()) {
                        collection = this.diagram._getChildren(collection);
                        ej.datavisualization.Diagram.Util.removeItem(collection, "multipleSelection");
                        this.diagram._clearSelection(collection.length ? true : false);
                    }
                    if (collection.length > 0) {
                        var children = collection;
                        var selection;
                        if (children.length > 1) {
                            var pseudoGroup = ej.datavisualization.Diagram.Group({ type: "pseudoGroup", "name": "multipleSelection" });
                            for (i = 0, len = children.length; i < len; i++) {
                                //children[i].parent = "multipleSelection";
                                pseudoGroup.children.push(this.diagram._getChild(children[i]));
                            }
                            this.diagram.nodes().push(pseudoGroup);
                            this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                            this.diagram.nameTable[pseudoGroup.name] = pseudoGroup;
                            selection = pseudoGroup;
                        }
                        else {
                            if (ej.datavisualization.Diagram.Util.canDoSingleSelection(this.diagram))
                                selection = collection[0];
                        }
                        if (selection) {
                            if (selection._type === "group" || selection.type === "pseudoGroup")
                                ej.datavisualization.Diagram.Util._updateGroupBounds(selection, this.diagram);
                            this.diagram._addSelection(selection);
                        }
                    }
                }
                else {
                    if (this.diagram._hasSelection() && !this.diagram._isDropped) {
                        this.diagram._clearSelection();
                    }
                }
            }
            else {
                if (this.selectedObject || this.diagram._hasSelection()) {
                    if (evt.ctrlKey || evt.shiftKey) {
                        if (ej.datavisualization.Diagram.Util.canDoMultipleSelection(this.diagram)) {
                            this._processCtrlKey(evt);
                        }
                    }
                    else if (this.diagram._hasSelection() && (!this.selectedObject || (this.diagram.selectionList[0].name != this.selectedObject.name)))
                        this.diagram._clearSelection(this.selectedObject ? true : false);
                    this.diagram._addSelection(this.selectedObject);
                }
                else if (this.diagram._hasSelection() && !this.diagram._isDropped) {
                    this.diagram._clearSelection();
                }
            }
            this.diagram._isDropped = false;
            this._nodeUnderMouseDown = null;
            if (this.selectedObject)
                this._raiseEvent("click", { element: this.diagram.getNode(this.selectedObject), actualObject: this.actualObject, event: evt });
            base.prototype.mouseup.call(this, evt);

        };
        SelectTool.prototype._endAction = function () {
            if (this._svgHelper) {
                this.diagram._adornerLayer.removeChild(this._svgHelper);
                this.selectedObject = null;
                this._svgHelper = null;
            }
            base.prototype._endAction.apply(this);
        };
        SelectTool.prototype._initHelper = function () {
            if (!this._svgHelper) {
                var g = this.diagram._adornerSvg.g();
                this.diagram._adornerLayer.appendChild(g);

                var scale = this.diagram._currZoom;
                var selectionRect = this.diagram._svg.rect({
                    "id": "helper", "x": this.startPoint.x * scale, "y": this.startPoint.y * scale,
                    "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges"
                });
                g.appendChild(selectionRect);
                g.setAttribute("pointer-events", "none");
                this._svgHelper = g;
            }
        };
        SelectTool.prototype._updateHelper = function () {
            var helper = this.diagram._adornerSvg.getElementById("helper");
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var scale = this.diagram._currZoom;
            var x = ((this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x) * scale;
            var y = ((this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y) * scale;

            if (helper) {
                this.diagram._adornerSvg.rect({
                    "id": helper.id, "x": x,
                    "y": y, "width": width * scale, "height": height * scale
                });
            }
        };
        SelectTool.prototype._checkGroupChildren = function (collection, group, bounds) {
            var child, childBounds;
            var children = this.diagram._getChildren(group.children);
            for (var i = 0; i < children.length; i++) {
                child = this.diagram.nameTable[children[i]];
                if (child && ej.datavisualization.Diagram.Util.canSelect(child)) {
                    childBounds = ej.datavisualization.Diagram.Util.bounds(child);
                    if (ej.datavisualization.Diagram.Geometry.containsRect(bounds, childBounds)) {
                        collection.push(child);
                    }
                    else if (child._type === "group") {
                        this._checkGroupChildren(collection, child, bounds);
                    }
                }
            }
        }
        return SelectTool;
    })(ToolBase);

    ej.datavisualization.Diagram.SelectTool = SelectTool;
    //#endregion

    //#region MoveTool
    var MoveTool = (function (base) {

        ej.datavisualization.Diagram.extend(MoveTool, base);

        function MoveTool(diagram) {
            base.call(this, "move", diagram);
            this.cursor = "move";
            this._isMouseDown = false;
            this.diffx = 0;
            this.diffy = 0;
            this.helper = null;
            this.hoverNode = null;
            this.stackOverNode = null;
            this.undoObject = [];
            this.activeLabel = null;
            this._canMoveLabel = false;
            this.dragState = "";
        }

        MoveTool.prototype.mousedown = function (evt) {
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            this.diffx = 0;
            this.diffy = 0;
            this._allowPan(this.selectedObject, "mousedown", evt);
            base.prototype.mousedown.call(this, evt);
            this._isMouseDown = true;
            this._canMoveLabel = false;
            this.activeLabel = null;
            if (this.diagram._currentCursor != "pointer")
                this.updateCursor(this.cursor);
            this._findLabelUnderMouse(evt);
            this.dragState = ej.datavisualization.Diagram.DragState.Starting;
        };

        MoveTool.prototype.mousemove = function (evt) {
            this._allowPan(this.selectedObject, "mousemove", evt);
            base.prototype.mousemove.call(this, evt);
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            var start = !this.inAction;
            //Initiate move action
            if (this._isMouseDown && !this.inAction &&
                (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y || this.diagram._selectedSymbol)) {
                if (!this.selectedObject) {
                    //set the selected object
                    this._updateSelectedObject();
                    // raise item click event
                    //var tempVal = this.selectedObject;
                    //var args = this._raiseEvent("itemClick",
                    //    { actualObject: this.actualObject, selectedObject: this.selectedObject, model: this.diagram.model, cancel: false });
                    this._itemClick(this.selectedObject, this.actualObject, this.diagram.model);
                    //if (args.cancel)
                    //this.selectedObject = tempVal;
                    var newbounds, oldbounds;
                    newbounds = oldbounds = ej.datavisualization.Diagram.Util.bounds(this.selectedObject);
                    var oldValue = { bounds: { x: oldbounds.x, y: oldbounds.y, width: oldbounds.width,height:oldbounds.height}, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                    var newValue = { bounds: { x: newbounds.x, y: newbounds.y, width: newbounds.width, height: newbounds.height }, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                    if (this.diagram.selectionList[0] != this.selectedObject)
                        this.diagram._clearSelection(true);
                    this._raiseDragEvent({ element: this.diagram.getNode(this.selectedObject), offset: { x: 0, y: 0 }, oldValue: oldValue, newValue:newValue, cancel: false });
                    var childTable = {};
                    if (this.selectedObject && (this.selectedObject._type === "group" || this.selectedObject.type === "pseudoGroup"))
                        childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                    var data = $.extend(true, {}, { "childTable": childTable, "node": this.selectedObject });
                    this.undoObject = jQuery.extend(true, {}, data);
                }
            }

            if (this._isMouseDown && this.selectedObject && ej.datavisualization.Diagram.Util.canSelect(this.selectedObject) && !this.diagram._isEditing) {
                this.dragState = ej.datavisualization.Diagram.DragState.Dragging;
                if (!this.inAction) {
                    this.inAction = true;
                    if (!this._canMoveLabel)
                        this.updateCursor("move");
                    this._updateSelection();
                    //Disconnect the connector, if it is being dragged
                    if (this.selectedObject.segments && !this._canMoveLabel) {
                        if (ej.datavisualization.Diagram.Util.isTargetConnected(this.selectedObject) ||
                            ej.datavisualization.Diagram.Util.isSourceConnected(this.selectedObject)) {
                            this._disconnect(this.selectedObject);
                        }
                    }
                    //disable pointer events to the selected object
                    ej.datavisualization.Diagram.SvgContext._disableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram);
                }
                if (this._canMoveLabel)
                    this._updateLabelXY(this.activeLabel, this.previousPoint, this.currentPoint);
                else {
                    if (this.diagram && !this.diagram._isEditing) {
                        this._containerMouseMove(evt);
                    }
                    //visually update the selected object
                    if (!this.helper)
                        this._updateObject();
                    //update the tooltip
                    if (!this.selectedObject.segments && !this.selectedObject.isLane)
                        this.diagram._renderTooltip(this.selectedObject, start);
                    //not needed since it is enough to update the selection for the first time
                    //this._updateSelection();
                    this.diagram._updateSelectionHandle(true);
                }
            }
            this.previousPoint = this.currentPoint;
        };

        //set the selected object
        MoveTool.prototype._updateSelectedObject = function () {
            var selectedObject = this.diagram._focusedElement;
            if (selectedObject) {
                if (this.activeLabel) {
                    if (ej.datavisualization.Diagram.Util.canMoveLabel(selectedObject))
                        this._canMoveLabel = true;
                    else {
                        this._disableLabel(selectedObject, this.activeLabel);
                        this._canMoveLabel = false;
                    }
                }
                if (this.diagram.selectionList[0] && this.diagram.selectionList[0].type == "pseudoGroup") {
                    if (this._canMoveLabel) {
                        this.selectedObject = selectedObject;
                        this.diagram._clearSelection(true);
                    }
                    else
                        this.selectedObject = this.diagram.selectionList[0];

                } else {
                    if (this.diagram.selectionList[0] != selectedObject) {
                        this.diagram._clearSelection(true);
                        this.selectedObject = this.diagram.nameTable[selectedObject.name];
                    } else {
                        this.selectedObject = this.diagram.nameTable[this.diagram.selectionList[0].name];
                    }
                }
            }
        };

        MoveTool.prototype._nodeCanHighlight = function (hoverNode) {
            var status = false;
            if ((this.diagram._getChildren(hoverNode.children)).indexOf(this.selectedObject.name) == -1) {
                if (this.selectedObject._type != "group") {
                    status = true;
                }
                else {
                    if (hoverNode._type === "group") {
                        if (!this._containsChild(this.selectedObject, hoverNode.name))
                            status = true;
                    }
                }
            }
            return status;
        },

        MoveTool.prototype._swimlanPhaseDrag = function (evt) {
            if (this.selectedObject && this.selectedObject.isPhase) {
                var swimlane = null;
                var btmNode = this._getNodeUnderMouse();
                if (btmNode)
                    swimlane = this._getSwimLaneNode(btmNode, this.selectedObject, true);
                if (swimlane) {
                    ej.datavisualization.Diagram.SvgContext._drawNodeHighlighter(swimlane, this.diagram._adornerSvg, this.diagram._adornerLayer, this.diagram._currZoom);
                }
            }
        };

        MoveTool.prototype._containerMouseMove = function (evt) {
            //#region for Container
            var overNode, gHighLight;
            var parent = this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent];
            if (parent && parent.container) {
                var container = parent.container;
                if (container.type == "canvas" || container.type == "stack") {
                    this.updateHelper();
                    this.updateHighlighter();
                }
            } else {
                if (!this.selectedObject.isSwimlane) {
                    if ((this.selectedObject.type == "pseudoGroup") && this._isParentAsLane(this.selectedObject)) {
                        this._removeHighLighter();
                        this._updatePseudoGroup(this.selectedObject);
                    }
                    else if (!this.isDiagram(evt) && !this.selectedObject.segments) {
                        overNode = this._getNodeUnderMouse();
                        //this.hoverNode = this.stackOverNode;
                        if (overNode && overNode.container && (this.selectedObject.isSwimlane || this.selectedObject.isLane)) {
                            if (this._getSwimLaneNode(overNode, this.selectedObject))
                                this._nodeHighLighter(overNode);
                        }
                        else if (overNode && overNode.container && overNode.container.type != "stack") {
                            if (this.selectedObject.name != overNode.name && this.selectedObject.type != "pseudoGroup")
                                this._nodeHighLighter(overNode);

                            if (overNode && (this.selectedObject.type == "pseudoGroup" || this.selectedObject.name == "multipleSelection")) {
                                if (this._fromDiagram(this.selectedObject.children)) {
                                    this._nodeHighLighter(overNode);
                                }
                            }
                        }
                        else if (overNode) {
                            this._removeHighLighter();
                            if (!overNode.segments)
                                this._nodeHighLighter(overNode);
                            else {
                                ej.datavisualization.Diagram.SvgContext._drawConnectorHighlighter(overNode, this.diagram, this.diagram._currZoom);
                            }
                        }
                    }
                    else if (!this.isDiagram(evt) && this.selectedObject.isPhase) {
                        ej.datavisualization.Diagram.SvgContext._disableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram);
                    }
                    else
                        this._removeHighLighter();
                }
                if (!gHighLight) {
                    if (this.selectedObject.type == "pseudoGroup") {
                        this._updateMultipleUndoObject(this.selectedObject);
                        if (this._fromDiagram(this.diagram._getChildren(this.selectedObject.children)))
                            this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                        else if (this._fromSameContainer(this.diagram._getChildren(this.selectedObject.children)) && this.hasSameParent()) {
                            this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                        }
                    }
                    else this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                }
            }
            //#endregion  
            this._swimlanPhaseDrag(evt);
        };

        MoveTool.prototype.updateHelper = function () {
            if (!this.helper) {
                this._initHelper();
            } else {
                this._updateHelperXY(this.helper, this.previousPoint, this.currentPoint);
                ej.datavisualization.Diagram.SvgContext._updateContainerHelper(this.diagram);
            }
        };

        MoveTool.prototype.updateHighlighter = function () {
            var overNode = this._getNodeUnderMouse();
            if (overNode && !overNode.segments && (!this.selectedObject.isLane || overNode.isLane)) {
                this._nodeHighLighter(overNode);
            } else {
                this._removeHighLighter();
            }
        };

        MoveTool.prototype._initHelper = function () {
            this.helper = this._getCloneNode(this.selectedObject);
            this.helper.name = "helper";
            this.startPoint = this.currentPoint;
            ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
        };

        MoveTool.prototype._removeFromContainer = function (node) {

            if (node.type == "pseudoGroup") {
                var child = null;
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child) {
                        ej.datavisualization.Diagram.SvgContext._removeFromContainer(child, this.diagram._svg, this.diagram._adornerLayer);
                        //delete this.diagram.nameTable[child.name];
                    }
                }
            } else {
                //delete this.diagram.nameTable[node.name];
                ej.datavisualization.Diagram.Util.removeItem(this.diagram.nodes(), this.diagram.nameTable[node.name]);
                this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                ej.datavisualization.Diagram.SvgContext._removeFromContainer(node, this.diagram._svg, this.diagram._adornerLayer);
            }
        };

        MoveTool.prototype._updateContainerProperties = function (node) {
            if (node && node.type == "pseudoGroup") {
                var child = null;
                if (this.helper) {
                    var diffX = this.helper.offsetX - this.selectedObject.offsetX;
                    var diffY = this.helper.offsetY - this.selectedObject.offsetY;
                }
                else {
                    var diffX = 0;
                    var diffY = 0;
                }
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child && !child.isSwimlane) {
                        oldParent = this.diagram.nameTable[this.selectedObject.parent];
                        if (oldParent) {
                            oldParent.children = this.diagram._getChildren(oldParent.children);
                            if (oldParent && oldParent.children)
                                ej.datavisualization.Diagram.Util.removeItem(oldParent.children, this.selectedObject.name);
                        }

                        newObj = this.diagram.nameTable[child.name];
                        if (newObj) {
                            var oldPar = this.diagram.nameTable[newObj.parent];
                            if (oldPar) {
                                oldPar.children = this.diagram._getChildren(oldPar.children);
                                ej.datavisualization.Diagram.Util.removeItem(oldPar.children, newObj.name);
                            }
                            newObj.parent = this._getNodeUnderMouse().name;
                            var newpar = this.diagram.nameTable[newObj.parent];
                            if (this._containsChild(newpar, newObj.name))
                                newpar.children.push(newObj.name);
                            this.diagram.updateNode(newObj.name, { offsetX: newObj.offsetX + diffX });
                            this.diagram.updateNode(newObj.name, { offsetY: newObj.offsetY + diffY });
                            this.diagram.nameTable[newObj.name] = newObj;
                        }
                    }
                }
                node.offsetX += diffX;
                node.offsetY += diffY;
                return node;
            }
            else {
                var oldParent = this.diagram.nameTable[this.selectedObject.parent];
                if (oldParent && oldParent.children) {
                    oldParent.children = this.diagram._getChildren(oldParent.children);
                    ej.datavisualization.Diagram.Util.removeItem(oldParent.children, this.selectedObject.name);
                }
                var newObj = this.selectedObject;//$.extend(true, {}, this.selectedObject);
                var overnode = this._getNodeUnderMouse();
                if (!overnode.isSwimlane) {
                    newObj.parent = overnode.name;
                    if (this.diagram._historyList && this.diagram._historyList.currentEntry && this.diagram._historyList.currentEntry.object) {
                        if (newObj && newObj.name === this.diagram._historyList.currentEntry.object.name)
                            this.diagram._historyList.currentEntry.object = newObj;
                    }
                }
                this.diagram.nameTable[newObj.name] = newObj;
                return newObj;
            }
        };

        MoveTool.prototype._updatePreviousContainer = function (node) {
            if (node._type === "group") {
                if (node.container.type == "canvas") {
                    var groupBounds = ej.datavisualization.Diagram.Util.bounds(node);
                    var newSize = this.diagram._getMinMaxSize(node, groupBounds.width, groupBounds.height);
                    node.width = newSize.width;
                    node.height = newSize.height;

                } else if (node.container.type == "stack") {

                }
            }
        };

        MoveTool.prototype._updateContainer = function (newObj, prevParent, isDrop, isExeed) {
            var bounds, groupBounds, dx = 0, dy = 0, newSize;
            if (newObj) {
                var parObj = this.diagram.nameTable[newObj.parent];
                if (!parObj && newObj.type == "pseudoGroup") {
                    var fNode = this.diagram.nameTable[this.diagram._getChild(this.selectedObject.children[0])];
                    parObj = this.diagram.nameTable[fNode.parent];
                }
                if (parObj) {
                    if (parObj.container) {
                        if (parObj.container.type == "canvas") {
                            if (isDrop) {
                                var group = this.diagram.nameTable[newObj.parent];
                                if (!group && newObj.type == "pseudoGroup") {
                                    group = parObj;
                                }
                                var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                                var grAngle = group.rotateAngle;
                                this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                                bounds = ej.datavisualization.Diagram.Util.bounds(newObj);
                                var bounds1 = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                                groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                                var tightchildBounds = this.diagram._getTightChildrenBounds(group);
                                var right = true;
                                var bottom = true;
                                var bRight = bounds1.x + bounds1.width;
                                var bBottom = bounds1.y + bounds1.height;
                                if (bRight >= groupBounds.right) {
                                    ///if exeeds right 
                                    dx = bRight - groupBounds.right + group.paddingRight;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                    widDiff = newSize.width - group.width;
                                    group.width = newSize.width;
                                    group.offsetX += (widDiff / 2);
                                    if (bRight < groupBounds.right) {
                                        right = false;
                                    }
                                }
                                if (bBottom >= groupBounds.bottom && !isExeed) {
                                    ///if exeeds bottom 
                                    dy = bBottom - groupBounds.bottom + group.paddingBottom;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                    heiDiff = newSize.height - group.height;
                                    group.height = newSize.height;
                                    group.offsetY += (heiDiff / 2);
                                    //group.offsetY += (dy / 2);
                                    //group.height += dy;
                                    bottom = true;
                                    if (bBottom < groupBounds.bottom) {
                                        bottom = false;
                                    }
                                }
                                if (bRight < groupBounds.right && right) { //if decreces from right
                                    dx = (tightchildBounds.x + tightchildBounds.width) + group.paddingRight - groupBounds.right;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                    var widDiff = newSize.width - group.width;
                                    group.width = newSize.width;
                                    group.offsetX += (widDiff / 2);
                                }
                                if (bBottom < groupBounds.bottom && bottom) { //if decreces from bottom
                                    dy = (tightchildBounds.y + tightchildBounds.height) + group.paddingBottom - groupBounds.bottom;
                                    newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                    var heiDiff = newSize.height - group.height;
                                    group.height = newSize.height;
                                    group.offsetY += (heiDiff / 2);
                                }

                                this._updateMargin(newObj, group);
                                ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                                this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                                var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                                this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                                if (group.parent && group.container && group.container.type == "canvas")
                                    this._updateMargin(group, this.diagram.nameTable[group.parent]);
                                //this is not required since updating at the end of this method
                                //ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);

                            }
                            else {
                                var vertical = false;
                                var horizontal = false;
                                group = this.diagram.nameTable[newObj.parent];
                                groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                                grAngle = group.rotateAngle;
                                if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                    this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                                if (group.container && group.container.type == "canvas") {
                                    this._updateMargin(newObj, group);
                                }
                                if (newObj.container && newObj.container.type == "stack") {
                                    if (newObj.container.orientation == "vertical")
                                        vertical = true;
                                    else
                                        horizontal = true;
                                }
                                bounds = this.diagram._getBoundsWithoutStretch(group, vertical, horizontal);// ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                                newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                                var tempWid = group.width;
                                var tempHei = group.height;
                                widDif = newSize.width - group.width;
                                heiDif = newSize.height - group.height;
                                group.offsetX += (widDif / 2);
                                group.width = newSize.width;
                                group.offsetY += (heiDif / 2);
                                group.height = newSize.height;
                                ////
                                if (group.container && group.container.type == "canvas") {
                                    this._updateMargin(newObj, group);
                                }
                                if (group.isSwimlane) {
                                    if (group.orientation == "horizontal") {
                                        if (tempWid != group.width) {
                                            if (this._getLastPhase(group)) {
                                                var lastPhase = this._getLastPhase(group).name.split("phaseStack")[1];
                                                this._updateSelectedPhase(lastPhase, group.width - tempWid, 0.0001);
                                                var phaseObj = this._getphase(lastPhase);
                                                phaseObj.offset += (group.width - tempWid);
                                                ej.datavisualization.Diagram.SvgContext._updatephase(phaseObj, this.diagram);
                                            }
                                        }
                                    }
                                    else if (group.orientation == "vertical") {
                                        if (tempHei != group.height) {
                                            if (this._getLastPhase(group)) {
                                                var lastPhase = this._getLastPhase(group).name.split("phaseStack")[1];
                                                this._updateSelectedPhase(lastPhase, .00001, group.height - tempHei);
                                                var phaseObj = this._getphase(lastPhase);
                                                phaseObj.offset += (group.height - tempHei);
                                                ej.datavisualization.Diagram.SvgContext._updatephase(phaseObj, this.diagram);
                                            }
                                        }
                                    }
                                }

                                /////
                                ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram, false);
                                if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                    this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                                groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                                this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            }
                            if (newObj.parent) {
                                this._updateContainer(this.diagram.nameTable[newObj.parent]);
                                if (!this.diagram.nameTable[newObj.parent].parent)
                                    ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[newObj.parent], this.diagram);
                            } else if (newObj.type == "pseudoGroup") {
                                fNode = this.diagram.nameTable[this.diagram._getChild(newObj.children[0])];
                                var parObj2 = this.diagram.nameTable[fNode.parent];
                                if (parObj2.parent) {
                                    this._updateContainer(this.diagram.nameTable[parObj2.name]);
                                    if (!this.diagram.nameTable[parObj2.name])
                                        ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[parObj2.name], this.diagram);
                                }
                            }

                        }//end canvas

                        else if (parObj.container.type == "stack") {
                            var tempWidth = 0;
                            if (isExeed && newObj.isLane && newObj.orientation === "horizontal") {
                                tempWidth = newObj.width;
                            }
                            group = this.diagram.nameTable[newObj.parent];
                            bounds = ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true, isExeed);
                            newObj.width += tempWidth;
                            groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                            grAngle = group.rotateAngle;
                            if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                            newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                            var widDif = newSize.width - group.width;
                            var heiDif = newSize.height - group.height;

                            group.offsetX += (widDif / 2);
                            group.width = newSize.width;
                            group.offsetY += (heiDif / 2);
                            group.height = newSize.height;


                            ej.datavisualization.Diagram.SvgContext._alignOnStack(group, this.diagram);
                            if (!((group.rotateAngle == 0) && (grAngle == 0)))
                                this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                            groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                            this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            if (group.parent)
                                this._updateContainer(group, false, false);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        }
                    }
                }
            }
        };

        MoveTool.prototype._addToContainer = function (node, addChildren) {
            if (node.type == "pseudoGroup") {
                var child = null;
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child) {
                        parent = this.diagram._svg.getElementById(child.parent);
                        if (!parent)
                            parent = this.diagram._diagramLayer;
                        if (child._type === "group") {
                            ej.datavisualization.Diagram.SvgContext.renderGroup(child, this.diagram._svg, parent, this.diagram.nameTable, this.diagram);
                        } else if (child.segments) {
                            ej.datavisualization.Diagram.SvgContext.renderConnector(child, this.diagram._svg, parent);
                        } else {
                            ej.datavisualization.Diagram.SvgContext.renderNode(child, this.diagram._svg, parent, undefined, this.diagram);
                        }
                    }
                }
            } else {
                //this.diagram._clearSelection(true);
                var overNode = this._getNodeUnderMouse();
                var parent;
                if (overNode && overNode.children && (!overNode.isSwimlane) && !addChildren) {
                    overNode.children.push(node.name);
                    //this.diagram.nameTable[node.name] = node;
                }
                if (node) {
                    parent = this.diagram._svg.getElementById(node.parent);
                    if (!parent)
                        parent = this.diagram._diagramLayer;
                    this.diagram.nodes().push(node);
                    this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                    if (node._type === "group") {
                        this.diagram._initGroupNode(node);
                        ej.datavisualization.Diagram.SvgContext.renderGroup(node, this.diagram._svg, parent, this.diagram.nameTable, this.diagram);
                    } else if (node.segments) {
                        ej.datavisualization.Diagram.SvgContext.renderConnector(node, this.diagram._svg, parent);
                    } else {
                        ej.datavisualization.Diagram.SvgContext.renderNode(node, this.diagram._svg, parent, undefined, this.diagram);
                    }
                }
            }
            this.selectedObject = node;
        };
        MoveTool.prototype._addToDiagram = function () {
            this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
            this._removeFromContainer(this.selectedObject);
            var oldParent = this.diagram.nameTable[this.selectedObject.parent];
            if (oldParent) {
                oldParent.children = this.diagram._getChildren(oldParent.children);
                ej.datavisualization.Diagram.Util.removeItem(oldParent.children, this.selectedObject.name);
            }
            this.selectedObject.marginBottom = 0;
            this.selectedObject.marginTop = 0;
            this.selectedObject.marginLeft = 0;
            this.selectedObject.marginRight = 0;
            this.selectedObject.parent = "";
            this.diagram.nameTable[this.selectedObject.name] = this.selectedObject;
            this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
            this._addToContainer(this.selectedObject);
            if (this.selectedObject.parent)
                var newParent = this.diagram.nameTable[this.selectedObject.parent];
            this._raiseGroupChange(newParent, oldParent, "ungroup");
        };
        MoveTool.prototype.isDiagram = function (evt) {
            if (evt.target.id == this.diagram._id + "_canvas_svg" || evt.target.id.match(this.selectedObject.name))
                return true;
            else
                return false;
        };
        MoveTool.prototype._allowPan = function (node, moveType, evt) {
            if (moveType === "mousedown")
                this.diagram.tools["panTool"].mousedown(evt);
            if (node && (node._type === "group" || node._type == "node") && (node.constraints & ej.datavisualization.Diagram.NodeConstraints.AllowPan)) {
                if (moveType === "mousemove")
                    this.diagram.tools["panTool"].mousemove(evt);
                else if (moveType === "mouseup")
                    this.diagram.tools["panTool"].mouseup(evt);
            }
        };
        MoveTool.prototype._checkForDropEvent = function (hoverNode, skip) {
            if (!this.diagram._selectedSymbol) {
                var source = this.diagram, target = hoverNode, childBounds = null, objectType = ej.datavisualization.Diagram.ObjectTypes.Diagram;
                if (this.selectedObject && this.selectedObject.parent) {
                    source = this.diagram.nameTable[this.selectedObject.parent];
                    if (source.isLane) objectType = ej.datavisualization.Diagram.ObjectTypes.Lane;
                    else if (source._type == "group") objectType = ej.datavisualization.Diagram.ObjectTypes.Group;
                }
                if (hoverNode && !skip) {
                    childBounds = ej.datavisualization.Diagram.Util.bounds(hoverNode);
                    target = this._getTargetNode(hoverNode, childBounds);
                }
                var args = this._raiseEvent("drop", { cancel: false, element: this.selectedObject, source: this.diagram.getNode(source), target: this.diagram.getNode(target), objectType: objectType });
                return args.cancel;
            }
            return false;
        };

        MoveTool.prototype._getSwimLaneNode = function (node, selNode, isPhase) {
            if (node.isSwimlane) {
                if (isPhase) {
                    if (this.diagram.nameTable[this.diagram._getChild(node.children[2])].container.orientation == selNode.orientation)
                        return node;
                }
                else if (this.diagram.nameTable[this.diagram._getChild(node.children[2])].container.orientation != selNode.orientation)
                    return node;
            }
            if (node.parent) {
                var parent = this.diagram.nameTable[node.parent];
                if (parent) {
                    return this._getSwimLaneNode(parent, selNode, isPhase);
                }
            }
            return null;
        };
        MoveTool.prototype._cloneGroupNode = function (node, id) {
            var child = null, child1, clnObj, node1;
            node1 = $.extend(true, {}, node);
            node1.name += id;
            if (node && node._type === "group") {
                node1.children = [];
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    child1 = $.extend(true, {}, child);
                    child1.name += id;
                    if (child1.parent)
                        child1.parent += id;
                    if (child1._type === "group")
                        this._cloneGroupNode(child1, id);
                    this.diagram.nameTable[child1.name] = child1;
                    this.diagram.nodes().push(child1);
                    this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                    node1.children.push(child1.name);
                }
            }
            return node1;
        };
        MoveTool.prototype._getSwimLaneStackIndex = function (node) {
            var index = null;
            if (node.container.type == "canvas") {
                if (node.parent)
                    var parent = this.diagram.nameTable[node.parent];
                if (parent) {
                    var bounds = ej.datavisualization.Diagram.Util.bounds(node);
                    if (parent.container.orientation === "vertical") {
                        if (this.currentPoint.y >= bounds.y && this.currentPoint.y < bounds.center.y) {
                            index = parent.children.indexOf(node.name);
                        }
                        else {
                            index = parent.children.indexOf(node.name) + 1;
                        }
                    }
                    else {
                        if (this.currentPoint.x >= bounds.x && this.currentPoint.x < bounds.center.x) {
                            index = parent.children.indexOf(node.name);
                        }
                        else {
                            index = parent.children.indexOf(node.name) + 1;
                        }
                    }
                    // return parent.children.indexOf(node.name);
                }
            }
            return index;
        };

        MoveTool.prototype._changeNodeState = function (evt, doc) {
            var newObj, pNode, mergNode = null;
            var overNode = this._getNodeUnderMouse();
            if (overNode) overNode = (this.selectedObject && !this.selectedObject.isLane || overNode.isLane) ? overNode : null;
            //#region drop fropm palette
            if (this.diagram._selectedSymbol && this.diagram._selectedSymbol.isLane && !doc) {
                this.diagram._isDragg = true;
                if (overNode)
                    mergNode = this._getSwimLaneNode(overNode, this.selectedObject);
                if (overNode && mergNode) {
                    this._addNewLane(overNode, mergNode);
                }
                else {
                    //add new stack
                    var swimlane = ej.datavisualization.Diagram.SwimLaneHelper._createSwimlane(this.selectedObject, this.diagram);
                    if (swimlane) {
                        var args;
                        args = { element: this.diagram.getNode(swimlane), cancel: false };
                        this.diagram._isUndo = true;
                        this.diagram.remove(this.selectedObject);
                        this.diagram._isUndo = false;
                        this.diagram._selectedSymbol = null;
                        this.diagram._raiseDropEvent(args);
                        if (!args.cancel) {
                            if (this.diagram.add(swimlane)) {
                                this.diagram._selectedSymbol = swimlane;
                                this.diagram._updateDroppedSymbol(swimlane);
                                this.selectedObject = swimlane;
                            }
                        }
                    }
                }

            }//#endregion
                //#region for Container 
            else if (this.selectedObject && this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent].container) {
                var container = this.diagram.nameTable[this.selectedObject.parent].container;
                this._removeHelpers();
                if (container.type == "canvas" && overNode) {
                    ej.datavisualization.Diagram.SvgContext._showNode(this.selectedObject, this.diagram._svg);
                    var checkDrop = !this._checkForDropEvent(overNode);
                    if (checkDrop && overNode) {
                        if (ej.datavisualization.Diagram.Util.canAllowDrop(overNode) || (overNode.name == this.selectedObject.parent)) {
                            if (overNode.container && overNode.container.type == "canvas") {
                                var newParent, oldParent;
                                this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                                pNode = this.selectedObject.parent;
                                if (this.selectedObject.parent)
                                    oldParent = this.diagram.nameTable[this.selectedObject.parent];
                                this._removeFromContainer(this.selectedObject);
                                newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                                this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
                                this._moveOnLane(this.diagram.nameTable[this.selectedObject.name], this.diagram.nameTable[this.diagram._getChild(newObj.parent)]);
                                this._addToContainer(newObj);
                                if (this.selectedObject.parent)
                                    newParent = this.diagram.nameTable[this.selectedObject.parent];
                                this._raiseGroupChange(newParent, oldParent, "group");
                            }
                        }
                    }
                } else if (container.type == "stack") {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                } else { //add to diagram
                    if (this.isDiagram(evt)) {
                        this._addToDiagram();
                    }
                    this.diffx = this.diffy = 0;
                }
            }
            else {///added from diagram to container
                if (this.selectedObject && !this.selectedObject.isSwimlane) {
                    if (this.selectedObject.type == "pseudoGroup" && this.hasSameParent()) {
                        this.currentPoint = this.previousPoint;
                        this.diffx = 0; this.diffy = 0;
                        this._removeHelpers();
                        if (overNode && !this._checkForDropEvent(overNode) || (overNode && overNode.name == this.selectedObject.parent)) {
                            this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                            pNode = this.selectedObject.parent;
                            newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                            this._removeFromContainer(this.selectedObject);
                            this._addToContainer(newObj);
                        } else {
                            this.diffx = 0;
                            this.diffy = 0;
                            if (!overNode && this.selectedObject.type == "pseudoGroup" && this.hasSameParent()) {
                                this._removeHelpers();
                                this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                                var checkDrop = !this._checkForDropEvent(overNode);
                                if (checkDrop) {
                                    if (this.isDiagram(evt)) {
                                        this._removeChildParent(this.selectedObject);
                                        this._addToDiagram();
                                    }
                                }
                            }
                        }
                    } else if (!this.isDiagram(evt) && this.selectedObject.type != "pseudoGroup") {
                        var overNode = this._getNodeUnderMouse();
                        if (overNode && overNode.container && ej.datavisualization.Diagram.Util.canAllowDrop(overNode) && !this._checkForDropEvent(overNode) || (overNode && overNode.container && overNode.name == this.selectedObject.parent)) {
                            if (!this.selectedObject.segments) {
                                var newParent, oldParent;
                                this._removeHelpers();
                                this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                                if (this.selectedObject.parent)
                                    oldParent = this.diagram.nameTable[this.selectedObject.parent];
                                this._removeFromContainer(this.selectedObject);
                                newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                                //this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
                                this._moveOnLane(newObj, this.diagram.nameTable[this.diagram._getChild(newObj.parent)]);
                                var parent = this.diagram.nameTable[newObj.parent];
                                if (parent)
                                    this._updateMargin(newObj, parent);
                                this._addToContainer(newObj);
                                if (this._labelUpdate)
                                    this.diagram.startLabelEdit(newObj, newObj.labels[0])
                                if (parent)
                                    ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram);
                                if (this.selectedObject.parent)
                                    newParent = this.diagram.nameTable[this.selectedObject.parent];
                                this._raiseGroupChange(newParent, oldParent, "group");
                            }
                        }
                        else if (overNode && this.selectedObject.name != overNode.name) {
                            this._checkForDropEvent(overNode, true);
                        }
                    }
                    else {
                        if ((this.selectedObject.type == "pseudoGroup" || this.selectedObject.name == "multipleSelection") && this.diagram._activeSwimLane) {
                            this._updateChildContainer(this.selectedObject);
                            this.selectedObject = null;
                        }
                        else if (!this.diagram._activeSwimLane && overNode && overNode.isLane) {
                            this.diagram._activeSwimLane = this._getParentSwimlane(overNode);
                            this._updateChildContainer(this.selectedObject);
                            this.selectedObject = null;
                        }
                        else if (!overNode && !this.diagram._activeSwimLane && (this.selectedObject.type == "pseudoGroup" || this.selectedObject.name == "multipleSelection")) {
                            this._updateChildContainer(this.selectedObject);
                        }
                        else if (overNode && (this.selectedObject.type == "pseudoGroup" || this.selectedObject.name == "multipleSelection")) {
                            if (this._fromDiagram(this.selectedObject.children)) {
                                if (ej.datavisualization.Diagram.Util.canAllowDrop(overNode)) {
                                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                                    ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._adornerSvg, this.diagram._adornerLayer);
                                    if (this._containsSwimlane(this.selectedObject.children)) {
                                        this._removeFromContainer(this.selectedObject);
                                        newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                                        parent = this.diagram.nameTable[newObj.parent];
                                        if (parent)
                                            this._updateMargin(this.selectedObject, parent);
                                        this._addToContainer(this.selectedObject);
                                    }

                                }
                            }
                        }
                        //    overNode = this.hoverNode;
                        //    if (overNode && overNode.container && this.hoverNode.allowDrop || (this.hoverNode && this.hoverNode.name == this.selectedObject.parent)) {
                        //        ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram.nameTable);
                        //        ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._svg, this.diagram._adornerLayer);
                        //        ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                        //        this.seletedObject = this.diagram.nameTable[this.selectedObject.name];
                        //        this._removeFromContainer(this.selectedObject);
                        //        newObj = this._updateContainerProperties(this.diagram.nameTable[this.selectedObject.name]);
                        //        //this._updateXY(this.diagram.nameTable[this.selectedObject.name], this.startPoint, this.currentPoint);
                        //        parent = this.diagram.nameTable[newObj.parent];
                        //        if (parent)
                        //            this._updateMargin(newObj, parent);
                        //        this._addToContainer(newObj);
                        //        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram);
                        //    }
                    }
                }
            }
            this._removeHelpers();
            this._checkPhaseDrop(evt);
            return newObj;
            //#endregion   
        };
        MoveTool.prototype._updateChildContainer = function (pseudoGroup) {
            var collection = [];

            if (this.diagram._activeSwimLane) {
                var lanes, child, bounds;
                for (var i = 0; i < pseudoGroup.children.length; i++) {
                    child = this.diagram.nameTable[this.diagram._getChild(pseudoGroup.children[i])]

                    lanes = this._getLanes(this.diagram._activeSwimLane)
                    var lane = this._childExistOnLane(child, lanes);
                    if (lane && child && !child.segments) {
                        child.parent = lane.name;
                        var parent = this.diagram.nameTable[child.parent];
                        parent.children.push(child.name);
                        this._moveOnLane(child, this.diagram.nameTable[this.diagram._getChild(child.parent)]);
                        if (parent)
                            this._updateMargin(child, parent);
                        ej.datavisualization.Diagram.Util.removeItem(this.diagram.nodes(), child)
                        this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                        this._addToContainer(child, true);
                        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram);
                        this._updateContainer(child, parent.name, true);
                        ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);
                    }
                    else if (child.segments) {
                        ej.datavisualization.Diagram.Util.removeItem(this.diagram.nodes(), child)
                        this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                    }
                    collection.push(this._getUpdatedNode(child));
                }
                ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.diagram);
                ej.datavisualization.Diagram.SvgContext.updateSelector(pseudoGroup, this.diagram._adornerSvg, this.diagram._currZoom, this.diagram, this.diagram.model.selectedItems.constraints);
            }
            else {
                for (i = 0; pseudoGroup.children && pseudoGroup.children.length > 0 && i < pseudoGroup.children.length; i++) {
                    child = this.diagram.nameTable[this.diagram._getChild(pseudoGroup.children[i])];
                    var swimlane = this._getBottomSwimlane(child);
                    if (swimlane) {
                        this.diagram._activeSwimLane = swimlane;
                        child = this.diagram.nameTable[this.diagram._getChild(pseudoGroup.children[i])]
                        lanes = this._getLanes(this.diagram._activeSwimLane)
                        var lane = this._childExistOnLane(child, lanes);
                        if (lane) {
                            child.parent = lane.name;
                            var parent = this.diagram.nameTable[child.parent];
                            parent.children.push(child.name);
                            this._moveOnLane(child, this.diagram.nameTable[this.diagram._getChild(child.parent)]);
                            if (parent)
                                this._updateMargin(child, parent);
                            this._addToContainer(child, true);
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram);
                            this._updateContainer(child, parent.name, true);
                            ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);

                        }
                    }
                    collection.push(this._getUpdatedNode(child));
                }
                this.selectedObject = pseudoGroup;
            }
            this._redoObject = { collection: collection, adjustLines: jQuery.extend(true, {}, this._adjustLines) }
            if ((this._undoObject && this._collectionFromLane(this._undoObject.collection)) || (this._redoObject && this._collectionFromLane(this._redoObject.collection))) {
                var entry = { type: "positionchanged", undoObject: jQuery.extend(true, {}, this._undoObject), redoObject: jQuery.extend(true, {}, this._redoObject), category: "internal", swimlaneMultiSelection: true };
                this.diagram.addHistoryEntry(entry);
                this._multipleUndo = true;
            }
        };

        MoveTool.prototype._moveOnLane = function (node, parent) {
            if (node && parent && parent.isLane) {
                var move = this.diagram._moveOnLane(node, parent);
                this._labelUpdate = false;
                if (this.diagram._isEditing) {
                    this._labelUpdate = true;
                    this.diagram._endEdit();
                }
                if (!move.proceedY || !move.proceedX) {
                    this.diagram._translate(node, move.proceedX ? 0 : move.x, move.proceedY ? 0 : move.y, this.diagram.nameTable);
                }
            }
        };
        MoveTool.prototype._raiseGroupChange = function (newParent, oldParent, cause) {
            if (newParent != oldParent)
                this.diagram._raiseGroupChangeEvent(this.selectedObject, oldParent, newParent, cause);
        };
        MoveTool.prototype._addNewLane = function (overNode, swimlane, updateName, index) {
            this._removeHelpers();
            var stack = this.diagram.nameTable[this.diagram._getChild(swimlane.children[2])];
            if (stack) {
                var selctObject = this._cloneGroupNode(this.selectedObject, updateName ? "" : ej.datavisualization.Diagram.Util.randomId());
                var args;
                args = { element: this.diagram.getNode(selctObject), cancel: false, target: overNode };
                var undo = this.diagram._isUndo;
                this.diagram._isUndo = true;
                this.diagram.remove(this.selectedObject);
                if (!undo)
                    this.diagram._isUndo = false;
                this.diagram._selectedSymbol = null;
                if (overNode)
                    var index = index ? index : this._getSwimLaneStackIndex(overNode);
                if (index || index === 0) {
                    stack.children.splice(index, 0, selctObject.name);
                }
                else
                    stack.children.push(selctObject.name);
                selctObject.parent = stack.name;
                this.diagram._raiseDropEvent(args);
                if (!args.cancel) {
                    this._isLane = true;

                    if (this.diagram.add(selctObject)) {
                        this.diagram._updateDroppedSymbol(selctObject);
                    }
                }
                var stackParent = this.diagram.nameTable[stack.parent];
                if (stackParent) {
                    this._updateContainer(selctObject);
                }
                ej.datavisualization.Diagram.DiagramContext.update(stackParent, this.diagram);
                this.selectedObject = stackParent;
                this.diagram.selectionList = [];
                this.diagram.selectionList.push(stackParent);
            }
        };

        MoveTool.prototype._checkPhaseDrop = function (evt) {
            var swimlane = null, nOffset, orientation;
            if (this.selectedObject && this.selectedObject.isPhase) {
                var btmNode = this._findNodeUnderMouse(evt);
                if (btmNode)
                    swimlane = this._getSwimLaneNode(btmNode, this.selectedObject, true);
                if (swimlane) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                    ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._adornerSvg, this.diagram._adornerLayer);

                    if (this.selectedObject.orientation == "vertical") {
                        nOffset = this.currentPoint.x - (swimlane.offsetX - (swimlane.width / 2));
                        orientation = "horizontal";
                    }
                    else {
                        nOffset = this.currentPoint.y - (swimlane.offsetY - (swimlane.height / 2));
                        orientation = "vertical";
                    }
                    var args;
                    args = { element: this.selectedObject, cancel: false, target: swimlane };
                    this.diagram.remove(this.selectedObject);
                    this.diagram._selectedSymbol = null;
                    this.inAction = false;
                    var label = (this.selectedObject.labels && this.selectedObject.labels.length > 0) ? this.selectedObject.labels[0] : { text: "Phase" };
                    this.diagram._addPhase(swimlane.name, {
                        name: ej.datavisualization.Diagram.Util.randomId(), offset: nOffset, orientation: orientation,
                        label: label,
                        lineColor: this.selectedObject.lineColor,
                        lineDashArray: this.selectedObject.lineDashArray,
                        lineWidth: this.selectedObject.lineWidth,
                    });
                    this.diagram._selectedSymbol = null;
                }
                else {
                    this.diagram._remove(this.selectedObject);
                    this.selectedObject = null;
                    this.diagram._selectedSymbol = null;
                }
            }
        };
        MoveTool.prototype._removeHelpers = function () {
            ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
            ej.datavisualization.Diagram.SvgContext._removeConnectorHighlighter(this.diagram._adornerLayer, this.diagram._adornerSvg);
            this._removeHighLighter();
        };

        MoveTool.prototype._removeChildParent = function (node) {
            if (node.type == "pseudoGroup") {
                var child = null, parent;
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child) {
                        if (child.parent != "") {
                            parent = this.diagram.nameTable[child.parent];
                            if (parent && parent.children) {
                                ej.datavisualization.Diagram.Util.removeItem(parent.children, child.name);
                            }
                        }
                        child.parent = "";
                    }
                }
            } else {
                ej.datavisualization.Diagram.SvgContext._removeFromContainer(node, this.diagram._svg, this.diagram._adornerLayer);
            }
        };

        MoveTool.prototype._updateNodeState = function (evt, newObj) {
            if (this.diagram.nameTable[this.selectedObject.parent])
                var container = this.diagram.nameTable[this.selectedObject.parent].container;
            var overNode = this._getNodeUnderMouse();
            if (overNode) overNode = (!this.selectedObject.isLane || overNode.isLane) ? overNode : null;
            if (container) {
                if (container.type == "canvas") {
                    if (overNode && newObj) {
                        var pNode = this.selectedObject.parent;
                        this._updateContainer(newObj, pNode, true);
                        ej.datavisualization.Diagram.DiagramContext.update(newObj, this.diagram);
                    }
                }
                else if (container.type == "stack") {
                    var evtArgs = this._checkForDropEvent(overNode);
                    if (!evtArgs) {
                        var stackOverNode = overNode;
                        if (stackOverNode) {
                            var parent = this.diagram.nameTable[stackOverNode.parent];
                            if (!parent) parent = overNode;
                            var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(parent);
                            var grAngle = parent.rotateAngle;
                            if (!(parent.rotateAngle == 0) && (grAngle == 0))
                                this.diagram._rotate(parent, -parent.rotateAngle, this.diagram.nameTable);
                            parent.children = this.diagram._getChildren(parent.children);
                            if (this.selectedObject.name != stackOverNode.name) {
                                this._swapChildren(parent.children, this.selectedObject.name, stackOverNode.name, ((parent.container.orientation === "vertical") ? true : false));
                                this._updateContainer(this.selectedObject);

                                if (!(parent.rotateAngle == 0) && (grAngle == 0))
                                    this.diagram._rotate(parent, grAngle, this.diagram.nameTable);
                                var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(parent);
                                this.diagram._translate(parent, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                                ej.datavisualization.Diagram.DiagramContext.update(parent, this.diagram);
                            }

                        } else {
                            this.diffx = this.diffy = 0;
                        }
                    }
                }
            }
            else if (this.selectedObject.type == "pseudoGroup") {
                var fNode = this.diagram.nameTable[this.diagram._getChild(this.selectedObject.children[0])];
                if (fNode && fNode.parent)
                    container = this.diagram.nameTable[fNode.parent].container;
                if (container && container.type == "canvas") {
                    if (overNode && newObj) {
                        pNode = this.selectedObject.parent;
                        this._updateContainer(newObj, pNode, true);
                        ej.datavisualization.Diagram.DiagramContext.update(newObj, this.diagram);
                    }
                }

            }
        },

       MoveTool.prototype.mouseup = function (evt, doc) {
           var skip = this.inAction;
           ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
           var clickedObject;
           if (this.inAction) {
               this.dragState = ej.datavisualization.Diagram.DragState.Completed;
               ej.datavisualization.Diagram.SvgContext._enableSelectedNode(this.selectedObject, this.diagram._svg, this.diagram);
               var newObj = this._changeNodeState(evt, doc);
               this.inAction = false;
               if (this.selectedObject) {
                   if (this._canMoveLabel)
                       this._updateLabelXY(this.activeLabel, this.previousPoint, this.currentPoint);
                   else {
                       var newbounds, oldbounds;
                       newbounds = oldbounds = ej.datavisualization.Diagram.Util.bounds(this.selectedObject);
                       var oldValue = { bounds: { x: oldbounds.x, y: oldbounds.y, width: oldbounds.width, height: oldbounds.height }, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                       var newValue = { bounds: { x: newbounds.x, y: newbounds.y, width: newbounds.width, height: newbounds.height }, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                       var endAction = this._updateXY(this.selectedObject, this.previousPoint, this.currentPoint);
                       if (!(this.selectedObject && this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent].container) || !newObj) {
                           this._raiseDragEvent({ element: this.diagram.getNode(this.selectedObject), offset: { x: 0, y: 0 },oldValue:oldValue, newValue:newValue, cancel: false });
                           this.dragState = "";
                       }
                   }
                   //#region for Container
                   if (!doc) this._updateNodeState(evt, newObj);
                   //#endregion 
                   this._updateObject();
                   var data;
                   var childTable = {};
                   if (this.selectedObject._type === "group" || this.selectedObject.type === "pseudoGroup")
                       childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                   if (!this.diagram._selectedSymbol && !this.selectedObject.isPhase && !this._isLane && !this.diagram._isDragg) {
                       var temp = { "childTable": childTable, "node": this.selectedObject };
                       data = $.extend(true, {}, temp);
                       var entry = { type: "positionchanged", undoObject: jQuery.extend(true, {}, this.undoObject), redoObject: jQuery.extend(true, {}, data), isMultipleNode: (this.selectedObject._type === "group") ? true : false, category: "internal" };
                       if (!this._multipleUndo)
                           this.diagram.addHistoryEntry(entry);
                   }
                   if (this.diagram._selectedSymbol && this.diagram._selectedSymbol.isSwimlane) this.diagram._selectedSymbol = null;
                   this.diagram._updateSelectionHandle();
                   clickedObject = this.selectedObject;
                   this._isLane = false;
                   this._multipleUndo = false;
               }
           }
           else {
               if (!this.selectedObject) {
                   if (evt.ctrlKey || evt.shiftKey) {
                       clickedObject = this._processCtrlKey(evt);
                   }
                   else {
                       var isMulSelect = true;
                       var mNode = this._findNodeUnderMouse(evt);
                       if (evt.which === 3 && this.diagram.selectionList[0] && (this.diagram.selectionList[0].type === "pseudoGroup" || this.diagram.selectionList[0]._type === "group") && this._hasMultipleSelection(mNode))
                           isMulSelect = false;
                       if (ej.datavisualization.Diagram.Util.canDoSingleSelection(this.diagram) && this._isMouseDown) {
                           if (isMulSelect)
                               this.selectedObject = mNode;
                           else
                               this.selectedObject = this.diagram.selectionList[0];
                       }
                       clickedObject = this.selectedObject;
                   }
                   if (this.selectedObject && !this.diagram._selectionContains(this.selectedObject))
                       this.diagram._clearSelection(true);
               }
               else {
                   clickedObject = this._findNodeUnderMouse(evt);
               }
           }
           if (!endAction && this.selectedObject) {
               if (this.selectedObject && this.selectedObject.type != "pseudoGroup") {
                   if (!skip) {
                       var obj1 = this._findNodeUnderMouse(evt, true);
                       var tempVal = this.selectedObject;
                       var args = this._raiseEvent("itemClick", { element: this.selectedObject, actualObject: this.actualObject, selectedObject: obj1, model: this.diagram.model, cancel: false, event: evt });
                       this._itemClick(this.selectedObject, this.actualObject, obj1);
                       if (args.cancel)
                           this.selectedObject = tempVal;
                   }
               }
               if (this.selectedObject) {
                   this._allowPan(this.selectedObject, "mouseup", evt);
                   if (!this.selectedObject.isPhase)
                       this._updateSelection();
                   this._enableLabel(this.selectedObject, this.activeLabel);
                   if (this.selectedObject.parent === "") {
                       ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                       var selectObj = this.selectedObject;
                       if (this.diagram._isDragg && !this.diagram._selectionContains(selectObj)) {
                           this.diagram._clearSelection();
                           this.diagram._addSelection(selectObj);
                       }
                   }
               }
               if (clickedObject && !skip) {
                   //raise click event
                   this._raiseEvent("click", { element: this.diagram.getNode(clickedObject), actualObject: this.diagram.getNode(this.actualObject), count: evt.originalEvent.detail, event: evt });
               }
           }
           delete this.diffx;
           delete this.diffy;
           delete this._isMouseDown;
           delete this.actualObject;
           this.diagram._removeTooltip();
           if (!this.diagram._isEditing && !this.diagram._isDragg)
               this.diagram._endEdit();
           delete this.hoverNode;

           ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
           this.diagram._activeSwimLane = null;
           base.prototype.mouseup.call(this, evt);
       };

        MoveTool.prototype._swapChildren = function (list, name1, name2, isVertical) {
            var btNode = this.diagram.nameTable[name2];
            if (btNode) {
                var bounds = ej.datavisualization.Diagram.Util.bounds(btNode);
                var currentIndex = list.indexOf(name1);
                var moveIndex = list.indexOf(name2);
                list.splice(list.indexOf(name1), 1);
                moveIndex += currentIndex > moveIndex ? 0 : -1;
                moveIndex += isVertical ? this.currentPoint.y >= bounds.y && this.currentPoint.y < bounds.center.y ? 0 : 1 :
                    this.currentPoint.x >= bounds.x && this.currentPoint.x < bounds.center.x ? 0 : 1;
                list.splice(moveIndex, 0, name1);
            }
        };

        MoveTool.prototype._updateObject = function () {
            if (this.selectedObject) {
                if (this.selectedObject.type == "pseudoGroup") {
                    var child;
                    var children = this.diagram._getChildren(this.selectedObject.children);
                    for (var i = 0, len = children.length; i < len; i++) {
                        child = this.diagram.nameTable[children[i]];
                        if (child) {
                            ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);
                        }
                    }
                }
                else {
                    if (this.selectedObject.parent) {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram);
                    }
                    ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                }
            }
        };

        MoveTool.prototype._endAction = function () {
            base.prototype._endAction.apply(this);
            this._isMouseDown = false;
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
        };

        MoveTool.prototype._updateHelperXY = function (shape, startPoint, endPoint) {
            var towardsLeft = endPoint.x < startPoint.x;
            var towardsTop = endPoint.y < startPoint.y;
            var difx = this.diffx + (endPoint.x - startPoint.x);
            var dify = this.diffy + (endPoint.y - startPoint.y);
            var offset;
            if (((this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) ||
                (this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines)) &&
                this.diagram._enableSnapToObject())
                offset = ej.datavisualization.Diagram.SnapUtil._snapPoint(this.diagram, this.helper, towardsLeft, towardsTop, ej.datavisualization.Diagram.Point(difx, dify),
                    endPoint, startPoint);
            if (!offset)
                offset = ej.datavisualization.Diagram.Point(difx, dify);
            this.diffx = difx - offset.x;
            this.diffy = dify - offset.y;
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset)) {
                var args = this._raiseDragEvent({ element: this.diagram.getNode(this.helper), offset: offset, cancel: false });
                if (!args.cancel) {
                    this.diagram._translate(shape, offset.x, offset.y, this.diagram.nameTable);
                    if (!this.inAction)
                        this._updateParent(shape);
                    if (shape._type === "group") {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(shape, this.diagram);
                    }
                }
            }
        };

        MoveTool.prototype._updateXY = function (shape, startPoint, endPoint) {
            var towardsLeft = endPoint.x < startPoint.x;
            var towardsTop = endPoint.y < startPoint.y;
            var difx = this.diffx + (endPoint.x - startPoint.x);
            var dify = this.diffy + (endPoint.y - startPoint.y);
            var offset;
            if (((this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) ||
                (this.diagram.model.snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines)) || this.diagram._enableSnapToObject())
                offset = ej.datavisualization.Diagram.SnapUtil._snapPoint(this.diagram, this.selectedObject, towardsLeft, towardsTop, ej.datavisualization.Diagram.Point(difx, dify),
                    endPoint, startPoint);

            if (!offset)
                offset = ej.datavisualization.Diagram.Point(difx, dify);

            this.diffx = difx - offset.x;
            this.diffy = dify - offset.y;
            if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset) || this.diagram._selectedSymbol && !this.inAction) {
                var args;
                if (!this.diagram._selectedSymbol) {
                    var oldbounds = ej.datavisualization.Diagram.Util.bounds(shape);
                    oldbounds = ej.datavisualization.Diagram.Rectangle(oldbounds.left, oldbounds.top, oldbounds.right - oldbounds.left, oldbounds.bottom - oldbounds.top);
                    var oldValue = { bounds: { x: oldbounds.x, y: oldbounds.y, width: oldbounds.width, height: oldbounds.height }, offsetX: shape.offsetX, offsetY: shape.offsetY, width: shape.width, height: shape.height };
                    this.diagram._translate(shape, offset.x, offset.y, this.diagram.nameTable);
                    var newbounds = ej.datavisualization.Diagram.Util.bounds(shape);
                    newbounds = ej.datavisualization.Diagram.Rectangle(newbounds.left, newbounds.top, newbounds.right - newbounds.left, newbounds.bottom - newbounds.top);
                    var newValue = { bounds: { x: newbounds.x, y: newbounds.y, width: newbounds.width, height: newbounds.height }, offsetX: shape.offsetX, offsetY: shape.offsetY, width: shape.width, height: shape.height };
                    args = this._raiseDragEvent({ element: this.diagram.getNode(this.selectedObject), offset: offset, oldValue: oldValue, newValue: newValue, cancel: false });
                }
                else {
                    args = { element: this.diagram.getNode(this.selectedObject), allowDrop: true, target: this._getNodeUnderMouse(), cancel: false };
                    if (args.target && args.target.segments) {
                        ej.datavisualization.Diagram.SvgContext._drawConnectorHighlighter(args.target, this.diagram, this.diagram._currZoom);
                    }
                    this._raiseEvent("dragOver", args);
                    if (!args.allowDrop) {
                        this.updateCursor("not-allowed");
                        if (!this.inAction) { this.diagram._removeSymbolFromDiagram(); return true; }
                    }
                    else { this.updateCursor("move"); }
                }
                if ((args && !args.cancel) || this.diagram._selectedSymbol) {
                    if (!shape.segments) {
                        if (ej.datavisualization.Diagram.Util.canAllowPan(shape)) {
                            if (this.diagram._hScrollOffset === 0) {
                                offset.x = 0;
                            }
                            if (this.diagram._vScrollOffset === 0) {
                                offset.y = 0;
                            }
                        }
                    }
                    this._updateParent(shape);
                    if (shape._type === "group" && shape.type != "bpmn" || (shape.type === "pseudoGroup")) {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(shape, this.diagram);
                    }
                }
                else {
                    this.diagram._translate(shape, -offset.x, -offset.y, this.diagram.nameTable);
                }
            }
        };

        MoveTool.prototype._updateLabelXY = function (label, startPoint, endPoint) {
            var difx = endPoint.x - startPoint.x;
            var dify = endPoint.y - startPoint.y;
            if (!this.selectedObject.segments) {
                var matrix = ej.Matrix.identity();
                ej.Matrix.rotate(matrix, -this.selectedObject.rotateAngle);
                var diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
            }
            var args = this._raiseDragEvent({ element: this.diagram.getNode(label), offset: label.offset, cancel: false });
            if (!args.cancel) {
                this.diagram._translateLabel(this.selectedObject, label, difx, dify);
            }
        };

        MoveTool.prototype._raiseDragEvent = function (args) {
            if (!this.diagram._selectedSymbol && !this.diagram._isDragg) {
                args.dragState = this.dragState;
                return this._raiseEvent("drag", args);
            }
        };

        MoveTool.prototype._updateParent = function (shape) {
            var child;
            if (shape.type == "pseudoGroup") {
                var children = this.diagram._getChildren(shape.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child && child.parent) {
                        ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[child.parent], this.diagram);
                        ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[child.parent], this.diagram);
                        this.diagram._updateAssociatedConnectorEnds(this.diagram.nameTable[child.parent], this.diagram.nameTable);
                    }
                }
            } else {
                if (shape && shape.parent) {
                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[shape.parent], this.diagram);
                    ej.datavisualization.Diagram.DiagramContext.update(this.diagram.nameTable[shape.parent], this.diagram);
                    this.diagram._updateAssociatedConnectorEnds(this.diagram.nameTable[shape.parent], this.diagram.nameTable);
                }
            }
            if (shape.parent) {
                var sParent = this.diagram.nameTable[shape.parent];
                if (sParent) {
                    this._updateParent(sParent);
                }
            }
        };
        MoveTool.prototype._updateSelection = function () {
            if (!this.diagram._selectionContains(this.selectedObject)) {
                this.diagram._clearSelection(true);
                this.diagram._addSelection(this.selectedObject);
            }
        };
        MoveTool.prototype._disableLabel = function (selectedObject, label) {
            if (selectedObject && label) {
                var htmlLayer = this.diagram._svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                var name = selectedObject._shape == "text" ? selectedObject.name : selectedObject.name + "_" + label.name;
                var text = $(htmlLayer).find("#" + name + ".ej-d-label")[0];
                if (text) text.style.pointerEvents = "none";
            }
        };
        MoveTool.prototype._enableLabel = function (selectedObject, label) {
            if (selectedObject && label) {
                var htmlLayer = this.diagram._svg.document.parentNode.getElementsByClassName("htmlLayer")[0];
                var name = selectedObject._shape == "text" ? selectedObject.name : selectedObject.name + "_" + label.name;
                var text = $(htmlLayer).find("#" + name + ".ej-d-label")[0];
                if (text) text.style.pointerEvents = "all";
            }
        };
        return MoveTool;
    })(ToolBase);

    ej.datavisualization.Diagram.MoveTool = MoveTool;
    //#endregion

    //#region ResizeTool
    var ResizeTool = (function (base) {
        ej.datavisualization.Diagram.extend(ResizeTool, base);
        function ResizeTool(diagram) {
            base.call(this, "resize", diagram);
            this.cursor = "default";//update resize cursor during action i.e in action
            this._resizeDirection = null;
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
            this.diffx = 0;
            this.diffy = 0;
            this._startPoint = ej.datavisualization.Diagram.Size(0, 0);
            this.undoObject = null;
            this.pseudoGroupConstraints = null;
            this._mouseDown = false;
            this.resizeState = "";
        }

        ResizeTool.prototype.mousedown = function (evt) {
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
            base.prototype.mousedown.call(this, evt);
            this._mouseDown = true;
            this.resizeState = ej.datavisualization.Diagram.ResizeState.Starting;
        };
        ResizeTool.prototype._createUpdateHelper = function () {
            if (!this.helper) {
                this.helper = this._getCloneNode(this.selectedObject);
                this.helper.type = null;
                this.helper.name = "helper";
                ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                this._startPoint = this.currentPoint;
                this._updateSize(this.helper, this.previousPoint, this.currentPoint, null, null, null, true);
            } else {
                var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(this.helper);
                this._updateSize(this.helper, this.previousPoint, this.currentPoint, null, null, null, true);
                var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(this.helper);
                if (this.selectedObject.container)
                    this.diagram._translate(this.helper, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                ej.datavisualization.Diagram.SvgContext._updateContainerHelper(this.diagram);
            }
        };
        ResizeTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            var start = !this.inAction;
            ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);

            if (!this.inAction && this.diagram.selectionList[0] && this._mouseDown) {
                this.selectedObject = this.diagram.nameTable[this.diagram.selectionList[0].name];
                var data;
                var childTable = {};
                var newbounds, oldbounds;
                newbounds = oldbounds = ej.datavisualization.Diagram.Util.bounds(this.selectedObject);
                var oldValue = { bounds: { x: oldbounds.x, y: oldbounds.y, width: oldbounds.width, height: oldbounds.height }, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                var newValue = { bounds: { x: newbounds.x, y: newbounds.y, width: newbounds.width, height: newbounds.height }, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                this._raiseEvent("sizeChange", { element: this.diagram.getNode(this.selectedObject), resizeState: this.resizeState, offset: { x: 0, y: 0 }, oldValue:oldValue, newValue:newValue, cancel: false });
                if (this.selectedObject._type === "group" || this.selectedObject.type === "pseudoGroup")
                    childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                var temp = { "childTable": childTable, "node": this.selectedObject };
                if (this.selectedObject && this.selectedObject.type === "pseudoGroup") {
                    if (this.diagram.model.selectedItems.getConstraints && $.isFunction(this.diagram.model.selectedItems.getConstraints)) {
                        this.pseudoGroupConstraints = this.diagram.model.selectedItems.getConstraints();
                    }
                    else this.pseudoGroupConstraints = ej.datavisualization.Diagram.Util._getPseudoGroupConstraints(this.diagram, this.selectedObject, this.selectedObject);
                }
                data = $.extend(true, {}, temp);
                this.undoObject = jQuery.extend(true, {}, data);
                //this.undoObject = ej.datavisualization.Diagram.Util.clone(this.selectedObject);
                this._resizeDirection = evt.target.getAttribute("class");
                if (!this.selectedObject.width) this.selectedObject.width = this.selectedObject._width;
                if (!this.selectedObject.height) this.selectedObject.height = this.selectedObject._height;
                this.initialBounds = { width: this.selectedObject.width, height: this.selectedObject.height, y: this.selectedObject.offsetY, x: this.selectedObject.offsetX };
                ej.datavisualization.Diagram.SvgContext._removePivotPoint(this.selectedObject, this.diagram._svg, this.diagram._currZoom);
                this.inAction = true;
            }          
            if (this.selectedObject) {
                this.resizeState = ej.datavisualization.Diagram.DragState.Resizing;
                var canvasChild = false;
                if (this.selectedObject && this.selectedObject.parent) {
                    var parNode = this.diagram.nameTable[this.selectedObject.parent];
                    if (parNode.container && parNode.container.type == "canvas")
                        canvasChild = true;
                }
                else if (this.selectedObject.type == "pseudoGroup" && this.hasSameParent() && this._fromContainer(this.diagram._getChildren(this.selectedObject.children))) {
                    canvasChild = true;
                }
                if ((this.selectedObject.type == "pseudoGroup") && this._isParentAsLane(this.selectedObject)) {

                    this._removeHighLighter();
                    this._updatePseudoGroup(this.selectedObject);
                }
                else if (this.selectedObject && canvasChild) {
                    this._createUpdateHelper();
                }
                else if (this.selectedObject && this.diagram.nameTable[this.selectedObject.name].container) {
                    this._createUpdateHelper();
                }
                else if (this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent] &&
                    this.diagram.nameTable[this.selectedObject.parent].container && this.diagram.nameTable[this.selectedObject.parent].container.type == "stack") {
                    if (!this.helper) {
                        this.helper = this._getCloneNode(this.selectedObject);
                    }
                    ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                    this._startPoint = this.currentPoint;
                    this._updateSize(this.helper, this.previousPoint, this.currentPoint);
                    ej.datavisualization.Diagram.DiagramContext.update(this.helper, this.diagram);
                }
                else {
                    if (this.selectedObject.type != "pseudoGroup")
                        this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                    else if (this.selectedObject.type == "pseudoGroup" && !this._fromContainer(this.diagram._getChildren(this.selectedObject.children)) && this._fromSameContainer(this.diagram._getChildren(this.selectedObject.children)))
                        this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                }
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                this.diagram._renderTooltip(this.selectedObject, start);
                this.diagram._updateSelectionHandle(true);
            }

            this.previousPoint = this.currentPoint;
        };
        ResizeTool.prototype._updateResizeObject = function () {
            var object = this.diagram.nameTable[this.selectedObject.name];
            var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(object);
            var grAngle = object.rotateAngle;
            this.diagram._rotate(object, -object.rotateAngle, this.diagram.nameTable);
            var dX = object.width - this.helper.width;
            var dY = object.height - this.helper.height;
            if ((!object.container && object.rotateAngle == 0)) {
                this._updateSize(object, this.startPoint, this.currentPoint);
            }
            else {
                this.diagram.updateNode(object.name, { width: this.helper.width });
                this.diagram.updateNode(object.name, { height: this.helper.height });
            }

            object.minWidth = object.width;
            object.minHeight = object.height;
            this.diagram._rotate(object, grAngle, this.diagram.nameTable);
            var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(object);
            if (!object.container && object.rotateAngle == 0) { //this.diagram._translate(object, dX / 2, dY / 2, this.diagram.nameTable);
            }
            else
                this.diagram._translate(object, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
            return object;
        };
        ResizeTool.prototype.mouseup = function (evt) {
            var undoObj = null;
            this._mouseDown = false;
            if (this.inAction) {
                this.resizeState = ej.datavisualization.Diagram.ResizeState.Completed;
                ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
                this.inAction = false;
                var canvasChild = false;
                var newbounds, oldbounds;
                newbounds = oldbounds = ej.datavisualization.Diagram.Util.bounds(this.selectedObject);
                var oldValue = { bounds: { x: oldbounds.x, y: oldbounds.y, width: oldbounds.width, height: oldbounds.height }, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                var newValue = { bounds: { x: newbounds.x, y: newbounds.y, width: newbounds.width, height: newbounds.height }, offsetX: this.selectedObject.offsetX, offsetY: this.selectedObject.offsetY, width: this.selectedObject.width, height: this.selectedObject.height };
                this._raiseEvent("sizeChange", { element: this.diagram.getNode(this.selectedObject), resizeState: this.resizeState, offset: { x: 0, y: 0 }, oldValue:oldValue,newValue:newValue, cancel: false });
                this.resizeState = "";
                if (this.selectedObject.isLane || this.selectedObject.isSwimlane) {
                    var childTable = {};
                    childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                    undoObj = $.extend(true, {}, { "childTable": childTable, "node": this.selectedObject });
                }
                if (this.selectedObject && this.selectedObject.parent) {
                    var parNode = this.diagram.nameTable[this.selectedObject.parent];
                    if (parNode.container && parNode.container.type == "canvas")
                        canvasChild = true;
                }
                else if (this.selectedObject.type == "pseudoGroup" && this.hasSameParent() && this._fromContainer(this.diagram._getChildren(this.selectedObject.children))) {
                    canvasChild = true;
                }
                if (this.selectedObject && canvasChild) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                    var object = this._updateResizeObject();
                    this._updateContainer(object, null, true);
                    ej.datavisualization.Diagram.SnapUtil._removeGuidelines(this.diagram);
                }
                else if (this.selectedObject && this.diagram.nameTable[this.selectedObject.name].container) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                    object = this.diagram.nameTable[this.selectedObject.name];
                    if (this.selectedObject.container) {
                        object.minWidth = this.helper.width;
                        object.minHeight = this.helper.height;
                    }
                    this._updateSize(this.selectedObject, this._startPoint, this.currentPoint, null, null, true);
                    if (object.isLane) {
                        var lane = this.diagram.nameTable[object.name];
                        this._updateContainer(lane, this.diagram.nameTable[lane.parent]);
                    }
                    if (this.selectedObject.isLane && this.selectedObject.children && this.selectedObject.children.length > 1) {
                        for (var j = 1; j < this.selectedObject.children.length; j++) {
                            var childname = this.diagram._getChild(this.selectedObject.children[j]);
                            if (childname) {
                                var childObj = this.diagram.nameTable[childname];
                                if (childObj)
                                    this._updateMargin(childObj, this.selectedObject);
                            }
                        }
                    }
                    if (this.selectedObject.parent) {//canvas panel
                        var parent = this.diagram.nameTable[this.selectedObject.parent];
                        if (parent._type === "group" && parent.container && parent.container.type == "stack") {
                            ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                            object = this.diagram.nameTable[this.selectedObject.name];
                            var newsize = this.diagram._getMinMaxSize(this.selectedObject, this.helper.width, this.helper.height);
                            object.height = newsize.height;
                            object.width = newsize.width;
                            if (this.selectedObject.container) {
                                object.minWidth = object.width;
                                object.minHeight = object.height;
                            }
                            this._updateContainer(this.diagram.nameTable[this.selectedObject.name]);
                        }
                    }
                    else if (this.selectedObject) {//stack panel
                        this.selectedObject.minWidth = this.selectedObject.width;
                        this.selectedObject.minHeight = this.selectedObject.height;
                    }
                }
                else if (this.selectedObject.parent && this.diagram.nameTable[this.selectedObject.parent] &&
                this.diagram.nameTable[this.selectedObject.parent].container && this.diagram.nameTable[this.selectedObject.parent].container.type == "stack") {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                    this.selectedObject.height = this.helper.height;
                    this.selectedObject.width = this.helper.width;
                    this._updateContainer(this.selectedObject);
                }
                else if ((this.selectedObject.type == "pseudoGroup" || this.selectedObject.name == "multipleSelection") && this.diagram._activeSwimLane) {
                    this._updateChildContainer(this.selectedObject);
                }
                else {
                    this._updateSize(this.selectedObject, this.previousPoint, this.currentPoint);
                }
                if (this.selectedObject.parent) {
                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram);
                }
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                this.diagram._updateSelectionHandle();

                var data;
                var childTable = {};
                if (this.selectedObject._type === "group" || this.selectedObject.type === "pseudoGroup")
                    childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                var temp = { "childTable": childTable, "node": this.selectedObject };
                data = $.extend(true, {}, temp);

                var entry = { type: "sizechanged", undoObject: undoObj ? undoObj : this.undoObject, redoObject: jQuery.extend(true, {}, data), isMultipleNode: (this.selectedObject._type === "group") ? true : false, category: "internal" };
                if (!this._multipleUndo)
                    this.diagram.addHistoryEntry(entry);
                this.diagram._removeTooltip();
                this._multipleUndo = false;
            }
            base.prototype.mouseup.call(this, evt);
        };
        ResizeTool.prototype._updateChildContainer = function (pseudoGroup) {
            var collection = [];
            var lanes = this._getLanes(this.diagram._activeSwimLane), child, bounds;
            for (var i = 0; i < pseudoGroup.children.length; i++) {
                child = this.diagram.nameTable[this.diagram._getChild(pseudoGroup.children[i])]
                var lane = this._childExistOnLane(child, lanes);
                if (lane && child && !child.segments) {
                    child.parent = lane.name;
                    var parent = this.diagram.nameTable[child.parent];
                    parent.children.push(child.name);

                    if (parent)
                        this._updateMargin(child, parent);
                    //this._addToContainer(child, true);
                    ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram, true);
                    this._updateContainer(child, parent.name, true);
                    ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);
                }
                else if (child.segments) {
                    ej.datavisualization.Diagram.Util.removeItem(this.diagram.nodes(), child)
                    this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                }
                collection.push(this._getUpdatedNode(child));
            }
            ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.diagram);
            ej.datavisualization.Diagram.SvgContext.updateSelector(pseudoGroup, this.diagram._adornerSvg, this.diagram._currZoom, this.diagram, this.diagram.model.selectedItems.constraints);
            this._redoObject = { collection: collection, adjustLines: jQuery.extend(true, {}, this._adjustLines) }
            if ((this._undoObject && this._collectionFromLane(this._undoObject.collection)) || (this._redoObject && this._collectionFromLane(this._redoObject.collection))) {
                var entry = { type: "sizechanged", undoObject: jQuery.extend(true, {}, this._undoObject), redoObject: jQuery.extend(true, {}, this._redoObject), category: "internal", swimlaneMultiSelection: true };
                this.diagram.addHistoryEntry(entry);
                this._multipleUndo = true;
            }
        };
        ResizeTool.prototype._getPhaseStack = function (group) {
            var stack;
            if (group && group.isSwimlane) {
                if (group.children && group.children.length > 0) {
                    stack = this.diagram.nameTable[this.diagram._getChild(group.children[1])];
                }
            }
            return stack;
        };
        ResizeTool.prototype._updateContainer = function (newObj, prevParent, isDrop) {
            var bounds, groupBounds, dx = 0, dy = 0, newSize;
            if (newObj) {
                var parObj = this.diagram.nameTable[newObj.parent];
                if (!parObj && newObj.type == "pseudoGroup") {
                    var fNode = this.diagram.nameTable[this.diagram._getChild(this.selectedObject.children[0])];
                    parObj = this.diagram.nameTable[fNode.parent];
                }
                if (parObj) {
                    if (parObj.container && parObj.container.type == "canvas") {
                        if (isDrop) {
                            isDrop = false;
                            var group = this.diagram.nameTable[newObj.parent];
                            if (!group && newObj.type == "pseudoGroup") {
                                group = parObj;
                            }
                            var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                            var grAngle = group.rotateAngle;
                            this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                            bounds = ej.datavisualization.Diagram.Util.bounds(newObj);
                            var bounds1 = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                            var tightchildBounds = this.diagram._getTightChildrenBounds(group);
                            var right = true;
                            var bottom = true;
                            var bRight = bounds1.x + bounds1.width;
                            var bBottom = bounds1.y + bounds1.height;
                            if (bRight >= groupBounds.right) {
                                ///if exceeds right 
                                dx = bRight - groupBounds.right + group.paddingRight;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                                if (bRight < groupBounds.right) {
                                    right = false;
                                }
                            }
                            if (bBottom >= groupBounds.bottom) {
                                ///if exceeds bottom 
                                dy = bBottom - groupBounds.bottom + group.paddingBottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                                bottom = true;
                                if (bBottom < groupBounds.bottom) {
                                    bottom = false;
                                }
                            }
                            if (bRight < groupBounds.right && right) { //if decreces from right
                                dx = (tightchildBounds.x + tightchildBounds.width) + group.paddingRight - groupBounds.right;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                var widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                            }
                            if (bBottom < groupBounds.bottom && bottom) { //if decreces from bottom
                                dy = (tightchildBounds.y + tightchildBounds.height) + group.paddingBottom - groupBounds.bottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                var heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                            }
                            this._updateMargin(newObj, group);
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                            var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                            this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            if (group.parent && group.container && group.container.type == "canvas")
                                this._updateMargin(group, this.diagram.nameTable[group.parent]);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        } else {
                            var vertical = false;
                            var horizontal = false;
                            group = this.diagram.nameTable[newObj.parent];
                            if (newObj.container && newObj.container.type == "stack") {
                                if (newObj.container.orientation == "vertical")
                                    vertical = true;
                                else
                                    horizontal = true;
                            }
                            bounds = this.diagram._getBoundsWithoutStretch(group, vertical, horizontal);
                            bounds.width += group.paddingLeft + group.paddingRight;
                            bounds.height += group.paddingTop + group.paddingBottom;
                            newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                            var tempWid = group.width;
                            var tempHei = group.height;
                            widDif = newSize.width - group.width;
                            heiDif = newSize.height - group.height;
                            group.offsetX += (widDif / 2);
                            group.width = newSize.width;
                            group.offsetY += (heiDif / 2);
                            group.height = newSize.height;
                            if (group.isSwimlane) {
                                if (group.orientation == "horizontal") {
                                    if (tempWid != group.width) {
                                        if (this._getLastPhase(group)) {
                                            var lastPhase = this._getLastPhase(group).name.split("phaseStack")[1];
                                            this._updateSelectedPhase(lastPhase, group.width - tempWid, 0.0001);
                                            var phaseObj = this._getphase(lastPhase);
                                            phaseObj.offset += (group.width - tempWid);
                                            ej.datavisualization.Diagram.SvgContext._updatephase(phaseObj, this.diagram);
                                        }
                                    }
                                }
                                else if (group.orientation == "vertical") {
                                    if (tempHei != group.height) {
                                        if (this._getLastPhase(group)) {
                                            var lastPhase = this._getLastPhase(group).name.split("phaseStack")[1];
                                            this._updateSelectedPhase(lastPhase, .00001, group.height - tempHei);
                                            var phaseObj = this._getphase(lastPhase);
                                            phaseObj.offset += (group.height - tempHei);
                                            ej.datavisualization.Diagram.SvgContext._updatephase(phaseObj, this.diagram);
                                        }
                                    }
                                }
                            }
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        }
                        if (group.parent) {
                            if (this.diagram.nameTable[group.parent])
                                isDrop = true;
                            this._updateContainer(group, null, isDrop);
                        }
                    }//end canvas

                    else if (parObj.container && parObj.container.type == "stack") {
                        var sizeState = true;
                        group = this.diagram.nameTable[newObj.parent];
                        if (group.parent)
                            var groupParent = this.diagram.nameTable[group.parent];
                        if (groupParent && groupParent.isSwimlane)
                            sizeState = true;
                        bounds = ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true, null, newObj.name);

                        newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), group.verticalAlign);
                        var widDif = newSize.width - group.width;
                        var heiDif = newSize.height - group.height;
                        group.offsetX += (widDif / 2);
                        group.width = newSize.width;
                        group.offsetY += (heiDif / 2);
                        group.height = newSize.height;
                        ej.datavisualization.Diagram.SvgContext._alignOnStack(group, this.diagram);
                        ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        if (group.parent)
                            this._updateContainer(group);

                    }
                }
            }
        };
        ResizeTool.prototype._updateSize = function (shape, startPoint, endPoint, x1, y1, updateMinMax, isHelper, phase, disableSnap) {
            var deltaWidth, deltaHeight, diff;
            var matrix = ej.Matrix.identity();
            ej.Matrix.rotate(matrix, -shape.rotateAngle);
            var horizontalsnap = { snapped: false, offset: 0, left: false, right: false };
            var verticalsnap = { snapped: false, offset: 0, top: false, bottom: false };
            var difx = x1 ? x1 : (this.currentPoint.x - this.startPoint.x);
            var dify = y1 ? y1 : (this.currentPoint.y - this.startPoint.y);
            var pivot = ej.datavisualization.Diagram.Point(0, 0);
            var trans = ej.Matrix.identity();
            var x = shape.offsetX;
            var y = shape.offsetY;
            ej.Matrix.rotate(trans, shape.rotateAngle, x, y);
            var w = shape.width;
            var h = shape.height;
            x = x - w * shape.pivot.x;
            y = y - h * shape.pivot.y;
            switch (this._resizeDirection) {
                case "n-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w / 2, y + h));
                    var deltaWidth = 1;
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify));
                    difx = diff.x; dify = diff.y;
                    dify = this._snapTop(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaHeight = (this.initialBounds.height - dify) / shape.height;
                    break;
                case "e-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x, y + h / 2));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    difx = this._snapRight(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint, disableSnap);
                    dify = 0;
                    deltaWidth = (this.initialBounds.width + difx) / shape.width;
                    deltaHeight = 1;
                    break;
                case "w-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w, y + h / 2));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    difx = this._snapLeft(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    dify = 0;
                    deltaWidth = (this.initialBounds.width - difx) / shape.width;
                    deltaHeight = 1;
                    break;
                case "s-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w / 2, y));
                    deltaWidth = 1;
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify));
                    difx = diff.x; dify = diff.y;
                    dify = this._snapBottom(horizontalsnap, verticalsnap, diff.x, diff.y, shape, (endPoint && startPoint && endPoint == startPoint) ? true : false);
                    deltaHeight = (this.initialBounds.height + dify) / shape.height;
                    break;
                case "ne-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x, y + h));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    difx = this._snapRight(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint, disableSnap);
                    dify = this._snapTop(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width + difx) / shape.width;
                    deltaHeight = (this.initialBounds.height - dify) / shape.height;
                    break;
                case "nw-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w, y + h));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    dify = this._snapTop(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    difx = this._snapLeft(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width - difx) / shape.width;
                    deltaHeight = (this.initialBounds.height - dify) / shape.height;
                    break;
                case "se-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x, y));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    dify = this._snapBottom(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint, disableSnap);
                    difx = phase ? difx : this._snapRight(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width + difx) / shape.width;
                    deltaHeight = (this.initialBounds.height + dify) / shape.height;
                    break;
                case "sw-resize":
                    pivot = ej.Matrix.transform(trans, ej.datavisualization.Diagram.Point(x + w, y));
                    diff = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(difx, dify)); difx = diff.x; dify = diff.y;
                    dify = this._snapBottom(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint, disableSnap);
                    difx = this._snapLeft(horizontalsnap, verticalsnap, difx, dify, shape, endPoint == startPoint);
                    deltaWidth = (this.initialBounds.width - difx) / shape.width;
                    deltaHeight = (this.initialBounds.height + dify) / shape.height;
                    break;
            }
            if (shape)
                if (!this.pseudoGroupConstraints) {
                    this.pseudoGroupConstraints = shape.constraints;
                }
            if (this.pseudoGroupConstraints & ej.datavisualization.Diagram.NodeConstraints.AspectRatio) {
                if (this._resizeDirection == "n-resize" || this._resizeDirection == "s-resize" || this._resizeDirection == "w-resize" || this._resizeDirection == "e-resize") {
                    var rotatedDif = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(startPoint.x - endPoint.x, startPoint.y - endPoint.y));
                    if (Math.abs(rotatedDif.x) > Math.abs(rotatedDif.y))
                        deltaHeight = deltaWidth;
                    else
                        deltaWidth = deltaHeight;
                }
                else if (startPoint != endPoint)
                    deltaHeight = deltaWidth = Math.max(deltaHeight, deltaWidth);
                else
                    deltaHeight = deltaWidth = 0;
            }
            var size = ej.datavisualization.Diagram.Size();
            if (deltaWidth < 0) deltaWidth = 1;
            if (deltaHeight < 0) deltaHeight = 1;
            size.width = deltaWidth;
            size.height = deltaHeight;
            var oldbounds = ej.datavisualization.Diagram.Util.bounds(shape);
            oldbounds = ej.datavisualization.Diagram.Rectangle(oldbounds.left, oldbounds.top, oldbounds.right - oldbounds.left, oldbounds.bottom - oldbounds.top);
            var oldValue = { bounds: { x: oldbounds.x, y: oldbounds.y, width: oldbounds.width, height: oldbounds.height }, offsetX: shape.offsetX, offsetY: shape.offsetY, width: shape.width, height: shape.height };
            if (!ej.datavisualization.Diagram.Geometry.isEmptySize(size)) {
                this.resizeState = ej.datavisualization.Diagram.ResizeState.Resizing;
                this.diagram.scale(shape, deltaWidth, deltaHeight, pivot, this.diagram.nameTable, null, updateMinMax, isHelper);
                var newbounds = ej.datavisualization.Diagram.Util.bounds(shape);
                newbounds = ej.datavisualization.Diagram.Rectangle(newbounds.left, newbounds.top, newbounds.right - newbounds.left, newbounds.bottom - newbounds.top);
                var newValue = { bounds: { x: newbounds.x, y: newbounds.y, width: newbounds.width, height: newbounds.height }, offsetX: shape.offsetX, offsetY: shape.offsetY, width: shape.width, height: shape.height };
                    var pathData = shape.pathData;
                    if (oldbounds.width != newbounds.width || oldbounds.height != newbounds.height) {
                        var args = this._raiseEvent("sizeChange", { element: this.diagram.getNode(this.selectedObject),resizeState:this.resizeState, offset: size, oldValue: oldValue, newValue: newValue });
                    }
                if (pathData && shape.pathData != pathData) {
                    shape._absoluteBounds = null;
                    shape._scaled = true;
                }
                if (args && args.cancel && !isHelper) {                   
                    this.diagram.scale(shape, 1 / deltaWidth, 1 / deltaHeight, pivot, this.diagram.nameTable);
                }
                if (shape._type === "group") {
                    ej.datavisualization.Diagram.Util._updateGroupBounds(shape, this.diagram);
                }
                this.diagram._updateAssociatedConnectorEnds(shape, this.diagram.nameTable);
           
        }
            this.pseudoGroupConstraints = null;
        };
        return ResizeTool;
    })(ToolBase);

    ej.datavisualization.Diagram.ResizeTool = ResizeTool;
    //#endregion

    //#region EndPointTool
    var ConnectionEditTool = (function (base) {
        ej.datavisualization.Diagram.extend(ConnectionEditTool, base);
        function ConnectionEditTool(diagram) {
            base.call(this, "endPoint", diagram);
            this.cursor = "move";
            this._endPoint = null;
            this._undoObject = null;
        }
        ConnectionEditTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            if (evt.ctrlKey && evt.shiftKey) {
                var parent = $(evt.target).parents(".ej-d-connector");
                if (parent.length > 0) {
                    this.selectedObject = this.diagram.nameTable[parent[0].id];
                    if (this.diagram._hasSelection())
                        this.diagram._clearSelection(true);
                    this.diagram._addSelection(this.selectedObject);
                }
            }
            this._initialValue = null;
            if (this.diagram.selectionList[0] && !this.selectedObject) this.selectedObject = this.diagram.nameTable[this.diagram.selectionList[0].name];
            this.undoObject = this._getClonedObject(this.selectedObject);
            this._endPoint = evt.target.getAttribute("class");
            this.targetid = evt.target.id;
            this._selectedSegment = "";
            this._nearestNodes = [];
            var isTarget = (this._endPoint == "targetEndPoint") ? true : false;
            var isSource = (this._endPoint == "sourceEndPoint") ? true : false;
            var dragargs = this._raiseEndPointDrag(this.selectedObject, isTarget, isSource, ej.datavisualization.Diagram.DragState.Starting);
        };
        ConnectionEditTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            var start = !this.inAction;
            if (this.selectedObject && !(evt.ctrlKey && evt.shiftKey)) {
                //  if (ej.datavisualization.Diagram.Util.canMove(this.selectedObject)) {
                var isTwo = false;
                if (this.selectedObject.segments.length == 2) isTwo = true;
                var difx = this.currentPoint.x - this.previousPoint.x;
                var dify = this.currentPoint.y - this.previousPoint.y;
                if (!this.inAction && (this.startPoint.x != this.currentPoint.x || this.startPoint.y != this.currentPoint.y)) {
                    this.inAction = true;
                    if (this.selectedObject.segments.length > 0) {
                        for (var m = 0; m < this.selectedObject.segments.length; m++) {
                            this.selectedObject.segments[m]._bridges = [];
                        }
                    }

                    (this.diagram._svg.getElementById(this.selectedObject.name + "_hitTest")).style["pointer-events"] = "none";
                    (this.diagram._svg.getElementById(this.selectedObject.name)).style["pointer-events"] = "none";
                    (this.diagram._adornerSvg.getElementById(this.diagram._adornerSvg.document.id + "handle_g")).style["pointer-events"] = "none";
                    var coll = this.targetid.split("_");
                    coll.reverse();
                    var index = Number(coll[0]);
                    this.selectedSegment = this.selectedObject.segments[index];
                    var segment = this.selectedSegment;
                    if (this.selectedSegment && this.selectedSegment.type == "orthogonal") {
                        var refresh = false;
                        if (index === 0) {
                            var editfirstsegment = true;
                        }
                        if (!this.selectedSegment.length) {
                            this._addTerminalSegment(difx, dify, coll);
                            refresh = true;
                        }
                        else if (index == 0) {
                            this.insertFirstSegment(difx, dify, coll);
                            refresh = true;
                        }
                        else {
                            this.nextSegment = this.selectedObject.segments[index + 1];
                        }
                    }
                    else
                        this.nextSegment = this.selectedObject.segments[index + 1];
                    if (refresh) {
                        ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, !(isTwo || editfirstsegment || !this.selectedObject.sourcePort), !(isTwo || !(segment.length || segment.length === 0) || !this.selectedObject.targetPort), this.diagram);
                        ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                        ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom);
                    }
                    if (!this.selectedSegment && this._endPoint == "targetEndPoint") this.selectedSegment = this.selectedObject.segments[this.selectedObject.segments.length - 1];
                    if (!this.selectedSegment && this._endPoint == "sourceEndPoint") {
                        this.selectedSegment = this.selectedObject.segments[0];
                        this.nextSegment = this.selectedObject.segments[1];
                    }
                    if (this.selectedSegment)
                        this._initialValue = { x: this.selectedSegment._endPoint.x, y: this.selectedSegment._endPoint.y };
                    this.undoObject = this._getClonedObject(this.selectedObject);
                    this.diagram._updateSelectionHandle(true);
                    this._raiseSegmentEditEvent(ej.datavisualization.Diagram.DragState.Starting);
                }
                else if (this.inAction) {
                    if (this.selectedSegment && this._endPoint == "segmentEnd") {
                        this._raiseSegmentEditEvent(ej.datavisualization.Diagram.DragState.Dragging);
                        if (difx != 0 || dify != 0) {
                            var actualdifX = this.currentPoint.x - this.startPoint.x;
                            var actualdifY = this.currentPoint.y - this.startPoint.y;
                            actualdifX = this._initialValue.x + actualdifX - this.selectedSegment._endPoint.x;
                            actualdifY = this._initialValue.y + actualdifY - this.selectedSegment._endPoint.y;
                            if (difx * actualdifX < 0) difx = 0; else difx = actualdifX;
                            if (dify * actualdifY < 0) dify = 0; else dify = actualdifY;
                            this._updateAdjacentSegments(difx, dify);
                        }
                    }
                    else if (this._endPoint != "segmentEnd") {
                        if (this._endPoint == "targetEndPoint" || this._endPoint == "sourceEndPoint")
                            this._checkConnectionPossible(evt);
                        var cancelEvent = this._updatePoints(this.selectedObject, ej.datavisualization.Diagram.DragState.Dragging);
                        if (!cancelEvent.cancel) {
                            this._disconnect(this.selectedObject);
                            this._updateConnection(this.selectedObject, cancelEvent.updateSelection);
                        }
                    }
                    this.diagram._updateSelectionHandle(true);
                    this.diagram._updateQuad(this.selectedObject);
                    ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                    //}
                }
            }
            this.previousPoint = this.currentPoint;
        };
        ConnectionEditTool.prototype._raiseSegmentEditEvent = function (state) {
            return this._raiseEvent("segmentChange", { element: this.selectedObject, dragState: state });
        };
        ConnectionEditTool.prototype._updateAdjacentSegments = function (difx, dify) {
            var current = this.selectedSegment;
            var pt = { x: difx + current._endPoint.x, y: dify + current._endPoint.y };
            pt = this.snap(pt);
            if (current.type == "orthogonal") {
                if (current._direction == "left" || current._direction == "right") {
                    dify = pt.y - current._endPoint.y;
                    current._startPoint.y += dify;
                    current._endPoint.y = current._startPoint.y;
                }
                else {
                    difx = pt.x - current._endPoint.x;
                    current._startPoint.x += difx;
                    current._endPoint.x = current._startPoint.x;
                }
                current.points[0] = current._startPoint;
                current.points[current.points.length - 1] = current._endPoint;
                var index = this.selectedObject.segments.indexOf(this.selectedSegment);
                var prev = this.selectedObject.segments[index - 1];
                if (prev) this.updatePreviousSegment(current, prev, difx, dify);
            }
            else if (current.type == "straight") {
                current.points[current.points.length - 1] = current._endPoint = pt;
                current._point = current.point = pt;
            }
            else {
                current.points[current.points.length - 1] = current._endPoint = pt;
                current._point = current.point = pt;
                ej.datavisualization.Diagram.Util._updateBezierPoints(current);
            }
            if (this.nextSegment)
                this.updateNextSegment(current, this.nextSegment, difx, dify);
        }
        ConnectionEditTool.prototype._addTerminalSegment = function (difx, dify, coll) {
            var segment = this.selectedSegment;
            this.selectedObject.segments.pop();
            var first = this.selectedObject.segments[this.selectedObject.segments.length - 1] && this.selectedObject.segments[this.selectedObject.segments.length - 1].type == "orthogonal" ? this.selectedObject.segments[this.selectedObject.segments.length - 1] : null;
            for (var i = 0; i < segment.points.length - 2; i++) {
                var seg = ej.datavisualization.Diagram.Segment({
                    type: "orthogonal",
                    length: ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]),
                    direction: ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[i], segment.points[i + 1])
                });
                if (seg.length == 0) {
                    if (first.direction == "top" || first.direction == "bottom") {
                        seg._direction = seg.direction = difx > 0 ? "right" : "left";
                    }
                    else {
                        seg._direction = seg.direction = dify > 0 ? "bottom" : "top";
                    }
                }
                first = seg;
                this.selectedObject.segments.push(seg);
            }
            var sec = Number(coll[1]);
            if (segment.points.length == 2 || sec == segment.points.length - 2) {
                if (first) {
                    first.length += 5;
                    first._length = first.length;
                }
                if (sec != 0) {
                    var length = ej.datavisualization.Diagram.Geometry.distance(segment.points[segment.points.length - 2], segment._endPoint);
                    var direction = ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[segment.points.length - 2], segment._endPoint);
                    var newsegment = ej.datavisualization.Diagram.Segment({ type: "orthogonal", length: 2 * length / 3, direction: direction });
                    this.selectedObject.segments.push(newsegment);

                }
            }
            var lastseg = ej.datavisualization.Diagram.Segment({ type: "orthogonal" });
            this.selectedObject.segments.push(lastseg);
            var index = Number(coll[0]) + (Number(coll[1]) || 0);
            this.selectedSegment = this.selectedObject.segments[index];
            this.nextSegment = this.selectedObject.segments[index + 1];
            if (index == this.selectedObject.segments.length - 2) {
                if (Number(coll[0]) == 0) {
                    this.nextSegment._direction = ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[segment.points.length - 2], segment._endPoint);;
                }
                else
                    this.nextSegment._direction = segment._direction;
            }
            else {
                lastseg._direction = ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[segment.points.length - 2], segment._endPoint);;
            }
            if (index == 0) this.insertFirstSegment(difx, dify, coll);

        },
        ConnectionEditTool.prototype.insertFirstSegment = function (difx, dify, coll) {
            var segment = this.selectedSegment;
            if (this.selectedObject.sourcePort && segment.length && this.selectedObject.segments[0].points.length > 2) {
                this.selectedObject.segments.splice(0, 1);
                var segments = [];
                var prev;
                for (var i = 0; i < segment.points.length - 1; i++) {
                    var first = ej.datavisualization.Diagram.Segment({
                        type: "orthogonal",
                        length: ej.datavisualization.Diagram.Geometry.distance(segment.points[i], segment.points[i + 1]),
                        direction: ej.datavisualization.Diagram.Util._getBezierDirection(segment.points[i], segment.points[i + 1])
                    });
                    if (first.length == 0) {
                        if (prev.direction == "top" || prev.direction == "bottom") {
                            first._direction = first.direction = difx > 0 ? "right" : "left";
                        }
                        else {
                            first._direction = first.direction = dify > 0 ? "bottom" : "top";
                        }
                    }
                    prev = first;
                    segments.push(first);
                }
                this.selectedObject.segments = segments.concat(this.selectedObject.segments);
                this.selectedSegment = this.selectedObject.segments[Number(coll[1])];
                this.nextSegment = this.selectedObject.segments[Number(coll[1]) + 1];
            }
            else {
                var segment = this.selectedObject.segments[0];
                var segments = [];
                segments.push(ej.datavisualization.Diagram.Segment({ type: "orthogonal", "direction": segment.direction, length: segment.length / 3 }));
                if (segment.direction == "bottom" || segment.direction == "top") {
                    var length = Math.max(5, Math.abs(difx));
                    var direction = difx > 0 ? "right" : "left";
                }
                else {
                    var length = Math.max(5, Math.abs(dify));
                    var direction = dify > 0 ? "bottom" : "top";
                }
                segments.push(ej.datavisualization.Diagram.Segment({ type: "orthogonal", "direction": direction, length: length }));
                var nextseg = this.selectedObject.segments[1];
                if (nextseg && nextseg.length) {
                    if (direction != nextseg.direction) {
                        length *= -1;
                    }
                    nextseg.length -= length;
                    nextseg._length = nextseg.length;
                }
                segment.length = segment._length = 2 * segment._length / 3;
                this.selectedObject.segments = segments.concat(this.selectedObject.segments);
                var index = Number(coll[0]) + Number(coll[1]);
                this.selectedSegment = this.selectedObject.segments[2];
                this.nextSegment = this.selectedObject.segments[3];
            }
        },
        ConnectionEditTool.prototype.updatePreviousSegment = function (current, prev, difx, dify) {
            prev._endPoint = prev.points[prev.points.length - 1] = current._startPoint;
            if (prev.type == "orthogonal") {
                if (prev.direction == "bottom") {
                    prev.length += dify;
                    if (prev.length < 0) {
                        prev._direction = prev.direction = "top";
                        prev.length *= -1;
                    }
                }
                else if (prev.direction == "top") {
                    prev.length -= dify;
                    if (prev.length < 0) {
                        prev._direction = prev.direction = "bottom";
                        prev.length *= -1;
                    }
                }
                else if (prev.direction == "right") {
                    prev.length += difx
                    if (prev.length < 0) {
                        prev._direction = prev.direction = "left";
                        prev.length *= -1;
                    }
                }
                else if (prev.direction == "left") {
                    prev.length -= difx;
                    if (prev.length < 0) {
                        prev._direction = prev.direction = "right";
                        prev.length *= -1;
                    }
                }
                prev._length = prev.length;
            }
            else {
                if (prev.type == "bezier") {
                    ej.datavisualization.Diagram.Util._updateBezierPoints(prev);
                }
            }
        };
        ConnectionEditTool.prototype.updateNextSegment = function (current, next, difx, dify) {
            var pt = current._endPoint;
            next._startPoint = next.points[0] = this.selectedSegment._endPoint;
            if (next) {
                if (next.type == "orthogonal") {
                    if (next.length || next.length === 0) {
                        if (next._direction == "left" || next._direction == "right") {
                            if (difx != 0) {
                                if (next._direction == "right") {
                                    next._length -= difx;
                                }
                                else {
                                    next._length += difx;
                                }

                                if (next.length || next.length === 0) {
                                    next.length = next._length;
                                    if (next.length < 0) {
                                        if (next.direction == "left") next.direction = "right";
                                        else if (next.direction == "right") next.direction = "left";
                                        next.length *= -1;
                                        next._direction = next.direction;
                                        next._length = next.length;
                                    }
                                }

                            }
                            if (dify != 0) {
                                if (current.type != "orthogonal") {
                                    next._startPoint.y = next.points[0].y = next._endPoint.y = next.points[next.points.length - 1].y = pt.y;
                                    var index = this.selectedObject.segments.indexOf(next);
                                    var segment = this.selectedObject.segments[index + 1];
                                    segment._startPoint.y = segment.points[0].y = next._endPoint.y;
                                    if (segment.length) {
                                        if (segment.direction == "bottom") {
                                            segment.length -= dify;
                                            if (segment.length < 0) {
                                                segment.direction = "top";
                                                segment.length *= -1;
                                            }
                                        }
                                        else {
                                            segment.length += dify;
                                            if (segment.length < 0) {
                                                segment.direction = "bottom";
                                                segment.length *= -1;
                                            }
                                        }
                                    }
                                }
                            }

                        }
                        else {
                            if (dify != 0) {
                                if (next._direction == "bottom") {
                                    next._length -= dify;
                                }
                                else {
                                    next._length += dify;
                                }

                                if (next.length || next.length === 0) {
                                    next.length = next._length;
                                    if (next.length < 0) {
                                        if (next.direction == "top") next.direction = "bottom";
                                        else if (next.direction == "bottom") next.direction = "top";
                                        next.length *= -1;
                                        next._length = next.length;
                                        next._direction = next.direction;
                                    }
                                }

                            }
                            if (difx != 0) {
                                if (current.type != "orthogonal") {
                                    next._startPoint.x = next.points[0].x = next._endPoint.x = next.points[next.points.length - 1].x = pt.x;
                                    var index = this.selectedObject.segments.indexOf(next);
                                    var segment = this.selectedObject.segments[index + 1];
                                    if (segment.length) {
                                        if (segment.direction == "right") {
                                            segment.length += difx;
                                            if (segment.length < 0) {
                                                segment.direction = "left";
                                                segment.length *= -1;
                                            }
                                        }
                                        else {
                                            segment.length -= difx;
                                            if (segment.length < 0) {
                                                segment.direction = "right";
                                                segment.length *= -1;
                                            }
                                        }
                                    }
                                }
                            }
                        }

                    }
                }
                else {

                    if (next.type == "bezier")
                        ej.datavisualization.Diagram.Util._updateBezierPoints(next);
                }
            }
        };
        ConnectionEditTool.prototype.mouseup = function (evt) {
            if (evt.shiftKey && evt.ctrlKey) {
                this._addOrRemoveStraightSegments(evt);
            }
            else
                if (this.inAction) {
                    this.inAction = false;
                    if (this._endPoint == "segmentEnd" && this.selectedSegment.type == "orthogonal") {
                        var args = this._raiseSegmentEditEvent(ej.datavisualization.Diagram.DragState.Completed);
                        var index = this.selectedObject.segments.indexOf(this.selectedSegment);
                        var prev = this.selectedObject.segments[index - 1];
                        var next = this.selectedObject.segments[index + 1];
                        var refresh = false;
                        if (prev && prev.type == "orthogonal" && Math.abs(prev.length) < 5) {
                            if (index != 1) {
                                refresh = this._removePrevSegment(index);
                            }
                        }
                        else if (next && next.type == "orthogonal") {
                            var length = (next.length || next.length === 0) ? next.length : ej.datavisualization.Diagram.Geometry.distance(next.points[0], next.points[1]);
                            if (Math.abs(length <= 5)) {
                                refresh = this._removeNextSegment(index);
                            }
                        }
                        if (refresh) {
                            ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                            ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom);
                        }
                    }
                    else if (this._endPoint != "segmentEnd") {
                        var cancelEvent = this._updatePoints(this.selectedObject, ej.datavisualization.Diagram.DragState.Completed);
                        if (!cancelEvent.cancel) {
                            this._disconnect(this.selectedObject);
                            var args;
                            if (this._currentPossibleConnection && !this.inAction) {
                                this._updateConnection(this.selectedObject);
                            }
                            else {
                                if (this.selectedObject.sourceNode || this.selectedObject.targetNode) {
                                    this._updateConnection(this.selectedObject);
                                }
                            }
                            ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                            if (this.selectedObject.parent)
                                ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram);
                        }
                    }

                    ej.datavisualization.Diagram.Util.updateBridging(this.selectedObject, this.diagram);
                    ej.datavisualization.Diagram.SvgContext.update(this.selectedObject, this.diagram);
                    this.diagram._updateConnectorBridging(this.selectedObject);
                    this.diagram._updateQuad(this.selectedObject);
                    this.diagram._updateSelectionHandle();

                    if (this.selectedObject) {
                        (this.diagram._svg.getElementById(this.selectedObject.name + "_hitTest")).style["pointer-events"] = "";
                        (this.diagram._svg.getElementById(this.selectedObject.name)).style["pointer-events"] = "";
                    }
                    (this.diagram._adornerSvg.getElementById(this.diagram._adornerSvg.document.id + "handle_g")).style["pointer-events"] = "";
                    this._initialValue = null;
                }
            if (this.selectedObject) {
                var entry = { type: "endpointchanged", undoObject: this.undoObject, redoObject: this._getClonedObject(this.selectedObject), isMultipleNode: (this.selectedObject._type === "group") ? true : false, category: "internal" };
                this.diagram.addHistoryEntry(entry);
                for (var i = 0; i < this._nearestNodes.length; i++)
                    this._showPort(this._nearestNodes[i], true);
            }
            delete this._nearestNodes;
            base.prototype.mouseup.call(this, evt);
        };
        ConnectionEditTool.prototype._addOrRemoveStraightSegments = function (evt) {
            if (this._endPoint == "segmentEnd") {
                var targetid = evt.target.id;
                var coll = targetid.split("_");
                coll.reverse();
                var index = Number(coll[0]);
                var segment = this.selectedObject.segments[index];
                if (segment && segment.type == "straight") {
                    var prevseg = this.selectedObject.segments[index + 1];
                    if (prevseg) {
                        this.selectedObject.segments.splice(index, 1);
                        prevseg._startPoint = prevseg.points[0] = segment._startPoint;
                        update = true;
                    }
                }
            }
            else {
                var className = evt.target.getAttribute("class");
                var targetid = evt.target.id;
                var update = false;
                var index = this._findIndex();
                if (this.selectedObject.segments && this.selectedObject.segments[index] && this.selectedObject.segments[index].type == "straight") {
                    var segment = this.selectedObject.segments[index];
                    var newseg = ej.datavisualization.Diagram.Segment({ type: "straight", point: segment._endPoint });
                    this.selectedObject.segments.splice(index + 1, 0, newseg);
                    segment._point = segment.point = segment.points[1] = segment._endPoint = this.currentPoint;
                    newseg.points[0] = newseg._startPoint = this.currentPoint;
                    newseg.points[1] = newseg._endPoint = newseg.point;
                    update = true;
                }
            }
            if (update) {
                ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                ej.datavisualization.Diagram.Util.updateBridging(this.selectedObject, this.diagram);
                ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                //ej.datavisualization.Diagram.SvgContext.clearSelector(this.diagram._svg, this.diagram._adornerLayer);
                ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom);
            }
        };
        ConnectionEditTool.prototype._findIndex = function () {
            var intersectingSegs = [];
            for (var i = 0; i < this.selectedObject.segments.length; i++) {
                var segment = this.selectedObject.segments[i];
                var rect;
                rect = {
                    x: Math.min(segment._startPoint.x, segment._endPoint.x), y: Math.min(segment._startPoint.y, segment._endPoint.y),
                    width: Math.abs(segment._startPoint.x - segment._endPoint.x), height: Math.abs(segment._startPoint.y - segment._endPoint.y)
                };
                ej.datavisualization.Diagram.Geometry.inflate(rect, this.selectedObject.lineHitPadding / 2, this.selectedObject.lineHitPadding / 2);
                if (ej.datavisualization.Diagram.Geometry.containsPoint(rect, this.currentPoint)) {
                    intersectingSegs.push(segment);
                }
            }
            if (intersectingSegs.length == 1) {
                return this.selectedObject.segments.indexOf(intersectingSegs[0]);
            }
            else {
                var ratio, min, index, seg, v, h;
                for (var i = 0; i < intersectingSegs.length; i++) {
                    seg = intersectingSegs[i];
                    var point = this.currentPoint;
                    v = (point.y - seg._startPoint.y) / (seg._endPoint.y - point.y);
                    h = (point.x - seg._startPoint.x) / (seg._endPoint.x - point.x);
                    ratio = Math.abs(v - h);
                    if (i == 0) { min = ratio; index = 0; }
                    if (ratio < min) { min = ratio; index = i; }
                }
                return this.selectedObject.segments.indexOf(intersectingSegs[index]);
            }
        }
        ConnectionEditTool.prototype._removeNextSegment = function (index) {
            var first = this.selectedObject.segments[index - 1];
            var last = this.selectedObject.segments[index + 2];
            var next = this.selectedObject.segments[index + 1];
            if (next.length || next.length === 0) {
                this.selectedObject.segments.splice(index, 2);
                if (this.selectedSegment.direction == "top" || this.selectedSegment.direction == "bottom") {
                    last._startPoint.y = this.selectedSegment._startPoint.y;
                    first._endPoint.x = last._startPoint.x;
                }
                else {
                    last._startPoint.x = this.selectedSegment._startPoint.x;
                    first._endPoint.y = last._startPoint.y;
                }
            }
            else {
                this.selectedObject.segments.splice(index + 1, 1);
                if (this.selectedSegment.direction == "top" || this.selectedSegment.direction == "bottom") {
                    first._endPoint.x = next._endPoint.x;
                }
                else {
                    first._endPoint.y = next._endPoint.y;
                }
                first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, first._endPoint);
                if (first.length)
                    first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, first._endPoint);
                this.selectedSegment.length = this.selectedSegment._length = this.selectedSegment.direction = null;
            }

            if (first && last) {
                first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, last._startPoint);
                if (first.length)
                    first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, last._startPoint);
                if (last.length || last.length === 0) {
                    last._length = last.length = ej.datavisualization.Diagram.Geometry.distance(first._endPoint, last._endPoint);
                    if (last.length)
                        last._direction = last.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._endPoint, last._endPoint);
                }
            }
            return true;
        },
        ConnectionEditTool.prototype._removePrevSegment = function (index) {
            var first = this.selectedObject.segments[index - 2];
            var next = this.selectedObject.segments[index + 1];
            var length = (next.length || next.length === 0) ? next.length : ej.datavisualization.Diagram.Geometry.distance(next.points[0], next.points[1]);
            if (length <= 5) {
                var end = this.selectedObject.segments[index + 2];
                if (next.length === length)
                    this.selectedObject.segments.splice(index - 1, 4);
                else
                    this.selectedObject.segments.splice(index - 1, 3);
                if (end) {
                    if (first.direction == "top" || first.direction == "bottom") {
                        first._endPoint.y = end._endPoint.y;
                        end._startPoint.x = first._endPoint.x;
                    }
                    else {
                        first._endPoint.x = end._endPoint.x;
                        end._startPoint.y = first._endPoint.y;
                    }
                    if (first.length || first.length === 0) {
                        first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, first._endPoint);
                        if (first.length)
                            first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, first._endPoint);
                    }
                    if (end.length || end.length === 0) {
                        end._length = end.length = ej.datavisualization.Diagram.Geometry.distance(end._startPoint, end._endPoint);
                        if (end._length)
                            end._direction = end.direction = ej.datavisualization.Diagram.Util._getBezierDirection(end._startPoint, end._endPoint);
                    }
                }
                else {
                    first._endPoint = this.selectedObject.endPoint;
                    first.direction = first.length = first._length = null;
                    first._direction = next._direction;
                }
            }
            else {

                var last = this.selectedObject.segments[index + 1];
                this.selectedObject.segments.splice(index - 1, 2);
                var segment = this.selectedSegment;
                if (segment.direction == "left" || segment.direction == "right") {
                    first._endPoint.x = last._startPoint.x;
                    last._startPoint.y = first._endPoint.y;
                }
                else {
                    first._endPoint.y = last._startPoint.y;
                    last._startPoint.x = first._endPoint.x;
                }
                if (this.selectedSegment.length || this.selectedSegment.length === 0) {
                    first._length = first.length = ej.datavisualization.Diagram.Geometry.distance(first._startPoint, first._endPoint);
                    if (first._length)
                        first._direction = first.direction = ej.datavisualization.Diagram.Util._getBezierDirection(first._startPoint, first._endPoint);
                }
                if (last.length || last.length === 0) {
                    last._length = last.length = ej.datavisualization.Diagram.Geometry.distance(last._startPoint, last._endPoint);
                    if (last.length)
                        last._direction = last.direction = ej.datavisualization.Diagram.Util._getBezierDirection(last._startPoint, last._endPoint);
                }
            }

            return true;
        },
        ConnectionEditTool.prototype._getClonedObject = function (connector) {
            if (connector) {
                var clonedObject = $.extend(true, {}, connector);
                clonedObject.targetNode = connector.targetNode;
                clonedObject.sourceNode = connector.sourceNode;
                clonedObject.targetPort = connector.targetPort;
                clonedObject.sourcePort = connector.sourcePort;
                return clonedObject;
            }
        };
        ConnectionEditTool.prototype._updateConnection = function (connector, updateSelection) {
            var connectionChange;
            if (this._endPoint == "targetEndPoint") {
                if (this._currentPossibleConnection) {
                    if (this._possibleConnectionPort && ej.datavisualization.Diagram.Util.canConnect(this._possibleConnectionPort, true)) {
                        if (connector.targetNode != this._currentPossibleConnection.name || connector.targetPort != this._possibleConnectionPort.name) {
                            connectionChange = this._raiseConnectionChangeEvent(connector, true);
                        }
                    }
                    else if (ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection)) {
                        if (connector.targetNode != this._currentPossibleConnection.name || connector.targetPort) {
                            connectionChange = this._raiseConnectionChangeEvent(connector, true);
                        }
                    }
                }
                else {
                    if (this.selectedObject.targetNode) {
                        connectionChange = this._raiseConnectionChangeEvent(connector, true);
                    }
                }
                if (connectionChange) {
                    ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                    ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                    if (connector.segments.length > 2 || connector.segments[0].points.length == 2)
                        ej.datavisualization.Diagram.Util._updatePreviousSegment(connector);
                    ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom);
                }
            }
            else if (this._endPoint == "sourceEndPoint") {
                if (this._currentPossibleConnection) {
                    if (this._possibleConnectionPort && ej.datavisualization.Diagram.Util.canConnect(this._possibleConnectionPort, true)) {
                        if (connector.sourceNode != this._currentPossibleConnection.name || connector.sourcePort != this._possibleConnectionPort.name) {
                            connectionChange = this._raiseConnectionChangeEvent(connector);
                        }
                    }
                    else if (ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection)) {
                        if (connector.sourceNode != this._currentPossibleConnection.name || connector.sourcePort) {
                            connectionChange = this._raiseConnectionChangeEvent(connector);
                        }
                    }

                }
                else {
                    if (this.selectedObject.sourceNode) {
                        connectionChange = this._raiseConnectionChangeEvent(connector);
                        //if (connectionChange)
                        {
                            ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);

                        }
                    }
                    if (connectionChange) {
                        ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom);
                    }
                }

            }

            var endPoint = this.selectedSegment._endPoint;
            this.diagram._dock(connector, this.diagram.nameTable, updateSelection);
            if (connectionChange && this._endPoint == "sourceEndPoint" && this.selectedObject.segments.length > 1) {
                if (this.selectedSegment.points.length == 2 && !this.selectedObject.sourcePort && this.selectedSegment.length) {
                    if (this.selectedSegment.direction == "right" || this.selectedSegment.direction == "left") {
                        if (this.selectedSegment._startPoint.x < endPoint.x) {
                            this.selectedSegment.direction = this.selectedSegment._direction = "right";
                        }
                        else this.selectedSegment.direction = this.selectedSegment._direction = "left";
                        this.selectedSegment.length = this.selectedSegment._length = Math.abs(this.selectedSegment._startPoint.x - endPoint.x);
                    }
                    else {
                        if (this.selectedSegment._startPoint.y < endPoint.y) {
                            this.selectedSegment.direction = this.selectedSegment._direction = "bottom";
                        }
                        else this.selectedSegment.direction = this.selectedSegment._direction = "top";
                        this.selectedSegment.length = this.selectedSegment._length = Math.abs(this.selectedSegment._startPoint.y - endPoint.y);

                    }
                    ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                    ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                    ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom);
                }
                else {
                    if (connectionChange) {
                        ej.datavisualization.Diagram.Util._updateConnectorSegments(this.selectedObject, this.selectedObject.sourcePort, this.selectedObject.targetPort, this.diagram);
                        ej.datavisualization.Diagram.DiagramContext._refreshSegments(this.selectedObject, this.diagram);
                        ej.datavisualization.Diagram.SvgContext._refreshEndPointHandles(this.diagram.selectionList[0], this.diagram._adornerSvg, this.diagram._currZoom);
                    }
                }
            }
        };
        ConnectionEditTool.prototype._raiseConnectionChangeEvent = function (connector, isTarget) {
            var args = this._raiseEvent("connectionChange", { element: this.selectedObject, endPoint: this._endPoint, connection: this._currentPossibleConnection, port: this._possibleConnectionPort, cancel: false });
            if (!args.cancel) {
                if (isTarget) {
                    if (!this._currentPossibleConnection || connector.targetNode != this._currentPossibleConnection.name) {
                        if (this._currentPossibleConnection)
                            this._currentPossibleConnection.inEdges.push(connector.name);
                        if (connector.targetNode) {
                            var node = this.diagram.nameTable[this.selectedObject.targetNode];
                            ej.datavisualization.Diagram.Util.removeItem(node.inEdges, connector.name);
                        }
                    }
                    connector.targetNode = this._currentPossibleConnection ? this._currentPossibleConnection.name : null;
                    connector.targetPort = this._possibleConnectionPort ? this._possibleConnectionPort.name : null;
                }
                else {
                    if (!this._currentPossibleConnection || connector.sourceNode != this._currentPossibleConnection.name) {
                        if (this._currentPossibleConnection)
                            this._currentPossibleConnection.outEdges.push(connector.name);
                        if (connector.sourceNode) {
                            var node = this.diagram.nameTable[this.selectedObject.sourceNode];
                            ej.datavisualization.Diagram.Util.removeItem(node.outEdges, connector.name);
                        }
                    }
                    connector.sourceNode = this._currentPossibleConnection ? this._currentPossibleConnection.name : null;
                    connector.sourcePort = this._possibleConnectionPort ? this._possibleConnectionPort.name : null;
                }

                return true;
            }
        }
        ConnectionEditTool.prototype._disconnect = function (connector) {
            var args;
            //var node = this.diagram.nameTable[connector.targetNode];
            var port = null;
            //if (this._endPoint == "targetEndPoint" && node) {
            //    port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.targetPort);
            //    args = this._raiseEvent("connectionChange", { element: connector, endPoint: this._endPoint, connection: node, port: port, cancel: false });
            //    if (!args.cancel) {
            //        ej.datavisualization.Diagram.Util.removeItem(node.inEdges, connector.name);
            //        connector.targetNode = null;
            //        connector.targetPort = null;
            //    }

            //}
            //var node = this.diagram.nameTable[connector.sourceNode];
            //if ((this._endPoint == "sourceEndPoint") && node) {
            //    port = ej.datavisualization.Diagram.Util.findPortByName(node, connector.sourcePort);
            //    args = this._raiseEvent("connectionChange", { element: connector, endPoint: this._endPoint, connection: node, port: port, cancel: false });
            //    if (!args.cancel) {
            //        ej.datavisualization.Diagram.Util.removeItem(node.outEdges, connector.name);
            //        connector.sourceNode = null;
            //        connector.sourcePort = null;
            //    }
            //}
        };
        ConnectionEditTool.prototype._endAction = function () {
            this._showPorts();
            var possibleConnection;
            if (this._currentPossibleConnection && !this._possibleConnectionPort) {
                possibleConnection = this._currentPossibleConnection;
            }
            base.prototype._endAction.apply(this);
            if (this._endPoint) {
                this._endPoint = null;
                if (possibleConnection) {
                    this.diagram._checkToolToActivate(possibleConnection, this.currentPoint);
                    this.diagram._updateCursor();
                }
            }
        };
        ConnectionEditTool.prototype._updatePoints = function (connector, dragState) {
            var cancel;
            if (this._endPoint != "segmentEnd") {
                var isTarget = (this._endPoint == "targetEndPoint") ? true : false;
                var isSource = (this._endPoint == "sourceEndPoint") ? true : false;

                var endPoint;
                if (this._endPoint) {
                    var end = this._endPoint.match("bezierpoint1") ? "bezierpoint1" : this._endPoint.match("bezierpoint2") ? "bezierpoint2" : this._endPoint;;

                    switch (end) {
                        case "targetEndPoint":
                            endPoint = this.selectedSegment._endPoint;
                            break;
                        case "sourceEndPoint":
                            endPoint = this.selectedSegment._startPoint;
                            break;
                        case "bezierpoint1":
                            endPoint = this.selectedSegment._point1;
                            break;
                        case "bezierpoint2":
                            endPoint = this.selectedSegment._point2;
                            break;
                    }

                    var ptCur = this.currentPoint;
                    if (this._possibleConnectionPort == null) {
                        ptCur = this.snap(ptCur);
                    }
                    var offset = ej.datavisualization.Diagram.Point(endPoint.x - ptCur.x, endPoint.y - ptCur.y);
                    var dragargs = this._raiseEndPointDrag(connector, isTarget, isSource, dragState);
                    cancel = dragargs ? dragargs.cancel : false;
                    if (!ej.datavisualization.Diagram.Geometry.isEmptyPoint(offset)) {
                        if (!cancel) {
                            if (isTarget || isSource) {
                                var connected = this._currentPossibleConnection ? ej.datavisualization.Diagram.Util.canConnect(this._currentPossibleConnection, false) : false;
                                if (this.selectedObject.segments.length > 1) {
                                    if (!isTarget && !connected && this.selectedSegment.type == "orthogonal" && (this.selectedSegment.length || this.selectedSegment.length === 0)) {
                                        this.selectedObject.sourcePoint = ptCur;
                                        var difx = ptCur.x - this.selectedSegment._startPoint.x;
                                        var dify = ptCur.y - this.selectedSegment._startPoint.y;
                                        if (this.selectedSegment.direction == "left" || this.selectedSegment.direction == "right") {
                                            this.selectedSegment._endPoint.y = this.selectedSegment.points[this.selectedSegment.points.length - 1].y = ptCur.y;
                                        }
                                        else {
                                            this.selectedSegment._endPoint.x = this.selectedSegment.points[this.selectedSegment.points.length - 1].x = ptCur.x;
                                        }
                                        this.selectedSegment._startPoint = this.selectedSegment.points[0] = ptCur;
                                        this.selectedSegment._length = this.selectedSegment.length = ej.datavisualization.Diagram.Geometry.distance(ptCur, this.selectedSegment._endPoint);
                                        if (this.selectedSegment._length)
                                            this.selectedSegment._direction = this.selectedSegment.direction = ej.datavisualization.Diagram.Util._getBezierDirection(ptCur, this.selectedSegment._endPoint);

                                        this.updateNextSegment(this.selectedObject.segments[0], this.selectedObject.segments[1], difx,
                                            dify);
                                        var next = this.selectedObject.segments[1];
                                        var updated = false;
                                        if (next && (next.length)) {
                                            next.length = next._length = ej.datavisualization.Diagram.Geometry.distance(this.selectedSegment._endPoint, next._endPoint);
                                            if (next.length)
                                                next.direction = next._direction = ej.datavisualization.Diagram.Util._getBezierDirection(this.selectedSegment._endPoint, next._endPoint);
                                        }
                                        if (this.selectedSegment.length == 0 || next.length === 0) {
                                            if (next.length == 0 && this.selectedObject.segments.length > 2) {
                                                this.selectedObject.segments.splice(0, 2);
                                            }
                                            else if (this.selectedObject.segments.length > 1) {
                                                this.selectedObject.segments.splice(0, 1);
                                            }
                                            this.selectedSegment = this.selectedObject.segments[0];
                                            this.selectedSegment._startPoint = this.selectedSegment.points[0] = ptCur;
                                            if (this.selectedSegment._direction == "left" || this.selectedSegment._direction == "right") {
                                                this.selectedSegment._endPoint.y = this.selectedSegment.points[this.selectedSegment.points.length - 1].y = ptCur.y;
                                            }
                                            else {
                                                this.selectedSegment._endPoint.x = this.selectedSegment.points[this.selectedSegment.points.length - 1].x = ptCur.x;
                                            }

                                            var next = this.selectedObject.segments[1];
                                            if (this.selectedObject.segments.length > 1) {
                                                this.selectedSegment._length = this.selectedSegment.length = ej.datavisualization.Diagram.Geometry.distance(ptCur, next._startPoint);
                                                ej.datavisualization.Diagram.Util._addOrthogonalPoints(this.selectedSegment, null, connector.segments[1], connector.sourcePoint, connector.targetPoint);
                                                ej.datavisualization.Diagram.Util._addOrthogonalPoints(connector.segments[1], connector.segments[0], connector.segments[2], connector.sourcePoint, connector.targetPoint);
                                                next._startPoint = next.points[0] = connector.segments[0]._endPoint;
                                            }
                                            else ej.datavisualization.Diagram.Util._addOrthogonalPoints(this.selectedSegment, null, connector.segments[1], connector.sourcePoint, connector.targetPoint);

                                            updated = true;
                                        }
                                        else ej.datavisualization.Diagram.Util._addOrthogonalPoints(this.selectedSegment, null, connector.segments[1], connector.sourcePoint, connector.targetPoint);
                                        if (next && (next.length)) {
                                            next.length = next._length = ej.datavisualization.Diagram.Geometry.distance(this.selectedSegment._endPoint, next._endPoint);
                                            if (next.length)
                                                next.direction = next._direction = ej.datavisualization.Diagram.Util._getBezierDirection(this.selectedSegment._endPoint, next._endPoint);
                                        }
                                    }
                                    else if (isTarget && !connected && this.selectedSegment.type == "orthogonal" && this.selectedSegment.points.length == 2) {
                                        var difx = ptCur.x - this.selectedSegment._endPoint.x;
                                        var dify = ptCur.y - this.selectedSegment._endPoint.y;

                                        if (this.selectedSegment._direction == "left" || this.selectedSegment._direction == "right") {
                                            this.selectedSegment._startPoint.y = (this.selectedSegment.points[0].y += dify);
                                            this.selectedSegment._endPoint.y = this.selectedSegment.points[1].y = this.selectedSegment._startPoint.y;
                                        }
                                        else {
                                            this.selectedSegment._startPoint.x = (this.selectedSegment.points[0].x += difx);
                                            this.selectedSegment._endPoint.x = this.selectedSegment.points[1].x = this.selectedSegment._startPoint.x;
                                        }
                                        var index = this.selectedObject.segments.length - 1;
                                        if (this.selectedObject.segments[index - 1])
                                            this.updatePreviousSegment(this.selectedObject.segments[index], this.selectedObject.segments[index - 1], difx,
                                                dify);
                                    }
                                }
                                if ((isSource || isTarget) && !connected) {
                                    var srclength = this.selectedSegment.points.length;
                                    this.diagram._setEndPoint(connector, ej.datavisualization.Diagram.Point(ptCur.x, ptCur.y), isTarget);
                                    if (srclength != this.selectedSegment.points.length || updated) {
                                        return { updateSelection: true, cancel: cancel };
                                    }
                                }
                            }
                            else
                                ej.datavisualization.Diagram.Util._setBezierPoint(this.selectedSegment, ej.datavisualization.Diagram.Point(ptCur.x, ptCur.y), end);
                        }
                    }
                }
            }
            return { cancel: cancel };
        };
        return ConnectionEditTool;
    })(ToolBase);

    ej.datavisualization.Diagram.ConnectionEditTool = ConnectionEditTool;
    //#endregion

    //#region ShapeTool
    var ShapeTool = (function (base) {
        ej.datavisualization.Diagram.extend(ShapeTool, base);
        function ShapeTool(diagram) {
            base.call(this, "shapeTool", diagram);
            this.cursor = "crosshair";
        }
        ShapeTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.createNode();
            this.selectedObject._type = "node";
        };
        ShapeTool.prototype.createNode = function () {
            var node = this.diagram._getNewNode(this.diagram.model.drawType);
            if (this.diagram.model.drawType.name) {
                node.name = this.diagram.model.drawType.name + ej.datavisualization.Diagram.Util.randomId();
            }
            else {
                node.name = "node" + ej.datavisualization.Diagram.Util.randomId();
            }
            node.offsetX = this.currentPoint.x;
            node.offsetY = this.currentPoint.y;
            node.width = 5;
            node.height = 5;
            if (!node.type == "text")
                node.shape = this.diagram.model.drawType.shape ? this.diagram.model.drawType.shape : node.shape;
            return node;
        };
        ShapeTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                    this._calcOffset();
                }
                else {
                    this._updateHelper();
                }
            }
        };
        ShapeTool.prototype.mouseup = function (evt) {

            if (this.inAction) {
                this.inAction = false;
                if (this.diagram.model.snapSettings.snapConstraints.snapToLines) {
                    this._calcOffset();
                }
                var node = this.createNode();
                this._updateSize(this.selectedObject, this.startPoint, this.currentPoint);
                this.diagram.add(this.selectedObject);
                //this.diagram._clearSelection(true);
                //this.diagram._addSelection(this.selectedObject);
            }
            this.diagram._toolToActivate = "selectTool";
            this.diagram._drawingTool = false;
            base.prototype.mouseup.call(this, evt);
        };
        ShapeTool.prototype._calcOffset = function () {
            var scale = this.diagram.model.magnification / 100;
            var bounds = ej.datavisualization.Diagram.Util.bounds(this.helper);
            var location = bounds.topLeft;
            location = ej.datavisualization.Diagram.Point(bounds.x + bounds.width, bounds.y + bounds.height);
            var x = location.x * scale;
            var y = location.y * scale;
            var xy = ej.datavisualization.Diagram.Point(x, y);
            var snapxy = this.snap(ej.datavisualization.Diagram.Point(x, y));
            this._mouseOffset = ej.datavisualization.Diagram.Size(xy.x - snapxy.x, xy.y - snapxy.y);
        };
        ShapeTool.prototype.getBoundingRect = function (startPoint, currentPoint) {
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var diff = ej.datavisualization.Diagram.Size(width, height);
            if (this.diagram.model.snapSettings.snapConstraints.snapToVerticalLines) {
                if (this._mouseOffset.width != 0) {
                    diff.width -= this._mouseOffset.width;
                    this._mouseOffset.width = 0;
                }
            }
            if (this.diagram.model.snapSettings.snapConstraints.snapToHorizontalLines) {
                if (this._mouseOffset.height != 0) {
                    diff.height -= this._mouseOffset.height;
                    this._mouseOffset.height = 0;
                }
            }
            var x = Math.min(this.startPoint.x, this.currentPoint.x);//(this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x;
            var y = Math.min(this.startPoint.y, this.currentPoint.y);// (this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y;
            return { size: diff, offset: ej.datavisualization.Diagram.Point(x, y) };
        };
        ShapeTool.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.svgHelper = ej.datavisualization.Diagram.SvgContext.renderNode(this.helper, this.diagram._svg, this.diagram._diagramLayer, undefined, this.diagram);
                this.svgHelper = document.getElementById(this.helper.name);
                this.svgHelper.setAttribute("pointer-events", "none");
            }
        };
        ShapeTool.prototype._updateHelper = function () {
            this._updateSize(this.helper, this.startPoint, this.currentPoint);
            ej.datavisualization.Diagram.SvgContext.update(this.helper, this.diagram);
        };
        ShapeTool.prototype._updateSize = function (shape, startPoint, currentPoint) {

            var rect = this.getBoundingRect(startPoint, currentPoint);
            var size = rect.size;
            var offset = rect.offset;
            if (!ej.datavisualization.Diagram.Geometry.isEmptySize(size)) {
                shape.offsetX = offset.x + size.width / 2;
                shape.offsetY = offset.y + size.height / 2;
                shape.width = size.width;
                shape.height = size.height;
                if (shape._shape === "path")
                    shape._scaled = true;
            }
        };
        return ShapeTool;
    })(ToolBase)

    ej.datavisualization.Diagram.ShapeTool = ShapeTool;
    //#endregion

    //#region LineTool
    var LineTool = (function (base) {
        ej.datavisualization.Diagram.extend(LineTool, base);
        function LineTool(name, diagram) {
            base.call(this, name, diagram);
            this.cursor = "crosshair";
            this._isMouseDown = false;
            this._targetPossibleConnection = null;
            this._sourcePossibleConnection = null;
            this._targetPort = null;
            this._sourcePort = null;
        }
        LineTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            if (ej.datavisualization.Diagram.Util.isPageEditable(this.diagram)) {
                this._isMouseDown = true;
                if (this.diagram._hasSelection()) {
                    this.diagram._clearSelection();
                }
                if (this._currentPossibleConnection) {
                    this._sourcePossibleConnection = this._currentPossibleConnection;
                    this._sourcePort = this._possibleConnectionPort;
                }
                this._raiseEndPointDrag(this.helper, true, false, ej.datavisualization.Diagram.DragState.Starting);
            }
        };
        LineTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            this._checkConnectionPossible(evt);
            if (this._isMouseDown) {
                if (!this.inAction) {
                    this.inAction = true;
                    this._initHelper();
                    this._renderHelper();
                }
                else {
                    this._targetPossibleConnection = this._currentPossibleConnection;
                    this._targetPort = this._possibleConnectionPort;
                    if (this.helper) {
                        var dragargs = this._raiseEndPointDrag(this.helper, true, false, ej.datavisualization.Diagram.DragState.Dragging);
                        if (!dragargs.cancel)
                            this._updateHelper();
                    }
                }
            }
            this.previousPoint = this.currentPoint;
        };
        LineTool.prototype.mouseup = function (evt) {
            this._isMouseDown = false;
            if (this.inAction) {
                this.inAction = false;
                var ptStart = this._getStartPoint();
                var ptEnd = this._getCurrentPoint();
                var distance = ej.datavisualization.Diagram.Geometry.distance(ptStart, ptEnd);
                if (distance > 0) {
                    var connector = this.createConnector(ptStart, ptEnd);
                    connector.name = "connector" + ej.datavisualization.Diagram.Util.randomId();
                    var args, dragargs;
                    if (this._sourcePossibleConnection) {
                        if (this._sourcePort && ej.datavisualization.Diagram.Util.canConnect(this._sourcePort, true)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort, cancel: false });
                            dragargs = this._raiseEndPointDrag(connector, true, false, ej.datavisualization.Diagram.DragState.Completed);
                            if (!args.cancel && !dragargs.cancel) {
                                connector.sourceNode = this._sourcePossibleConnection.name;
                                connector.sourcePort = this._sourcePort.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._sourcePossibleConnection.outEdges, this.helper.name);
                            }
                        }
                        else if (ej.datavisualization.Diagram.Util.canConnect(this._sourcePossibleConnection)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._sourcePossibleConnection, port: this._sourcePort, cancel: false });
                            dragargs = this._raiseEndPointDrag(connector, true, false, ej.datavisualization.Diagram.DragState.Completed);
                            if (!args.cancel && !dragargs.cancel) {
                                connector.sourceNode = this._sourcePossibleConnection.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._sourcePossibleConnection.outEdges, this.helper.name);
                            }
                        }
                    }
                    if (this._targetPossibleConnection) {
                        if (this._targetPort && ej.datavisualization.Diagram.Util.canConnect(this._targetPort, true)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort, cancel: false });
                            dragargs = this._raiseEndPointDrag(connector, true, false, ej.datavisualization.Diagram.DragState.Completed);
                            if (!args.cancel && !dragargs.cancel) {
                                connector.targetNode = this._targetPossibleConnection.name;
                                connector.targetPort = this._targetPort.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._targetPossibleConnection.inEdges, this.helper.name);
                            }
                        }
                        else if (ej.datavisualization.Diagram.Util.canConnect(this._targetPossibleConnection)) {
                            args = this._raiseEvent("connectionChange", { element: connector, connection: this._targetPossibleConnection, port: this._targetPort, cancel: false });
                            dragargs = this._raiseEndPointDrag(connector, true, false, ej.datavisualization.Diagram.DragState.Completed);
                            if (!args.cancel && !dragargs.cancel) {
                                connector.targetNode = this._targetPossibleConnection.name;
                                ej.datavisualization.Diagram.Util.removeItem(this._targetPossibleConnection.inEdges, this.helper.name);
                            }
                        }
                    }
                    var success = this.diagram.add(connector);
                    if (success) {
                        connector = this.diagram.nameTable[connector.name];
                        ej.datavisualization.Diagram.Util.updateBridging(connector, this.diagram);
                        ej.datavisualization.Diagram.SvgContext.update(connector, this.diagram);
                        this.diagram._updateConnectorBridging(connector);
                    }
                }
            }
            if (this.singleAction)
                this.diagram.deactivateTool();
            if (this._prevTool)
                this._prevTool.inAction = false;
            this._showPorts();
            base.prototype.mouseup.call(this, evt);
        };

        LineTool.prototype._getStartPoint = function () {
            var ptStart;
            if (this._sourcePossibleConnection && this._sourcePort) {
                ptStart = ej.datavisualization.Diagram.Util._getPortPosition(this._sourcePort, ej.datavisualization.Diagram.Util.bounds(this._sourcePossibleConnection));
            }
            else {
                ptStart = this.snap(this.startPoint);
            }
            return ptStart;
        };
        LineTool.prototype._getCurrentPoint = function () {
            var ptCur;
            if (this._targetPossibleConnection && this._targetPort) {
                ptCur = ej.datavisualization.Diagram.Util._getPortPosition(this._targetPort, ej.datavisualization.Diagram.Util.bounds(this._targetPossibleConnection));
            }
            else {
                ptCur = this.snap(this.currentPoint);
            }
            return ptCur;
        };
        LineTool.prototype._initHelper = function () {
            this.helper = this.createConnector(ej.datavisualization.Diagram.Point(0, 0), ej.datavisualization.Diagram.Point(0, 0));
            this.diagram._setZorder(this.helper);
            this.helper.name = "helper";
            this.helper._connectorType = "helper";
        };
        LineTool.prototype.createConnector = function (startPoint, endPoint) {
            return null;
        };
        LineTool.prototype._renderHelper = function () {
            if (!this.svgHelper) {
                this.svgHelper = ej.datavisualization.Diagram.SvgContext.renderConnector(this.helper, this.diagram._svg, this.diagram._diagramLayer);
                this.svgHelper = this.diagram._svg.document.getElementById(this.helper.name);
                this.svgHelper.setAttribute("pointer-events", "none");
                $(this.diagram._svg.getElementById(this.svgHelper.id)).attr("pointer-events", "none");
                $(this.diagram._svg.getElementById(this.svgHelper.id + "_hitTest")).attr("pointer-events", "none");
            }
        };
        LineTool.prototype._updateHelper = function () {
            var points = [];
            var ptStart = this._getStartPoint();
            var ptEnd = this._getCurrentPoint();
            points.push(ptStart);
            points.push(ptEnd);
            this.diagram._setEndPoint(this.helper, ptStart, false);
            this.diagram._setEndPoint(this.helper, ptEnd, true);
            this._disconnect(this.helper);
            if (this._sourcePossibleConnection) {
                if (this._sourcePort && ej.datavisualization.Diagram.Util.canConnect(this._sourcePort, true)) {
                    this.helper.sourceNode = this._sourcePossibleConnection.name;
                    this.helper.sourcePort = this._sourcePort.name;
                    this._sourcePossibleConnection.outEdges.push(this.helper.name);
                }
                else if (ej.datavisualization.Diagram.Util.canConnect(this._sourcePossibleConnection)) {
                    this.helper.sourceNode = this._sourcePossibleConnection.name;
                    this._sourcePossibleConnection.outEdges.push(this.helper.name);
                }
            }
            if (this._targetPossibleConnection) {
                if (this._targetPort && ej.datavisualization.Diagram.Util.canConnect(this._targetPort, true)) {
                    this.helper.targetNode = this._targetPossibleConnection.name;
                    this.helper.targetPort = this._targetPort.name;
                    this._targetPossibleConnection.inEdges.push(this.helper.name);
                }
                else if (ej.datavisualization.Diagram.Util.canConnect(this._targetPossibleConnection)) {
                    this.helper.targetNode = this._targetPossibleConnection.name;
                    this._targetPossibleConnection.inEdges.push(this.helper.name);
                }
            }
            this.diagram._dock(this.helper, this.diagram.nameTable);

            ej.datavisualization.Diagram.SvgContext.update(this.helper, this.diagram);
        };
        LineTool.prototype._endAction = function () {
            base.prototype._endAction.apply(this);
            this._targetPossibleConnection = null;
            this._sourcePossibleConnection = null;
            this._targetPort = null;
            this._sourcePort = null;
            this._isMouseDown = false;
        };
        LineTool.prototype._showPorts = function (node) {
        };
        LineTool.prototype._showAllPorts = function (hide) {
            base.prototype._showAllPorts.call(this, hide);
        };
        return LineTool;
    })(ToolBase);

    ej.datavisualization.Diagram.LineTool = LineTool;
    //#endregion

    //#region StraightLineTool
    var StraightLineTool = (function (base) {
        ej.datavisualization.Diagram.extend(StraightLineTool, base);
        function StraightLineTool(diagram) {
            base.call(this, "straightLine", diagram);
        }
        StraightLineTool.prototype.createConnector = function (startPoint, endPoint) {
            var connector = this.diagram._getNewConnector({ "segments": [{ "type": "straight" }], "sourcePoint": startPoint, "targetPoint": endPoint });
            return connector;
        };
        return StraightLineTool;
    })(LineTool);

    ej.datavisualization.Diagram.StraightLineTool = StraightLineTool;
    //#endregion

    //#region OrthogonalLineTool
    var OrthogonalLineTool = (function (base) {
        ej.datavisualization.Diagram.extend(OrthogonalLineTool, base);
        function OrthogonalLineTool(diagram) {
            base.call(this, "orthogonalLine", diagram);
        }
        OrthogonalLineTool.prototype.createConnector = function (startPoint, endPoint) {
            var connector = this.diagram._getNewConnector({ "segments": [{ "type": "orthogonal" }], "sourcePoint": startPoint, "targetPoint": endPoint });
            return connector;
        };
        return OrthogonalLineTool;
    })(LineTool);

    ej.datavisualization.Diagram.OrthogonalLineTool = OrthogonalLineTool;
    //#endregion

    //#region RotateTool
    var RotateTool = (function (base) {
        ej.datavisualization.Diagram.extend(RotateTool, base);
        function RotateTool(diagram) {
            base.call(this, "rotatetool", diagram);
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
            this.undoObject = null;
        }
        RotateTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
            //this.undoObject = ej.datavisualization.Diagram.Util.clone(this.selectedObject);

            var data;
            var childTable = {};
            if (this.selectedObject._type === "group" || this.selectedObject.type === "pseudoGroup")
                childTable = this.diagram._getChildTable(this.selectedObject, childTable);
            var temp = { "childTable": childTable, "node": this.selectedObject };
            data = $.extend(true, {}, temp);
            this.undoObject = jQuery.extend(true, {}, data);

            var scale = this.diagram._currZoom;
            //ej.datavisualization.Diagram.SvgContext.renderPivotPoint(this.selectedObject, this.diagram._currZoom, this.diagram._svg);

        };
        RotateTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            var start = !this.inAction;
            if (this.selectedObject) {
                if (!this.inAction) {
                    if (this.selectedObject._type === "group" || this.selectedObject.type === "pseudoGroup") {
                        var children = this.diagram._getChildren(this.selectedObject.children);
                        for (var i = 0; i < children.length; i++) {
                            var child = this.diagram.nameTable[children[i]];
                            if (!ej.datavisualization.Diagram.Util.canResize(child)) return;
                        }
                    }
                    this._initialValue = this.selectedObject.rotateAngle;
                    this.inAction = true;
                }
                {
                    var isContainer = false;
                    if (this.selectedObject.parent) {
                        var parent = this.diagram.nameTable[this.selectedObject.parent];
                        if (parent.container) {
                            isContainer = true;
                        }
                    }
                    else if (this.selectedObject.type == "pseudoGroup" && this._fromContainer(this.diagram._getChildren(this.selectedObject.children))) {
                        isContainer = true;
                    }
                    if (isContainer) {
                        if (this.selectedObject.type == "pseudoGroup" && this._isParentAsLane(this.selectedObject)) {
                            this._updatePseudoGroup(this.selectedObject);
                            this._rotate(this.selectedObject);
                            if (this.selectedObject.parent) {
                                ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram);
                            }
                        }
                        else {
                            if (!this.helper) {
                                this.helper = this._getCloneNode(this.selectedObject);
                                this.helper.name = "helper";
                            }
                            this._rotate(this.helper);
                            ej.datavisualization.Diagram.SvgContext._drawContainerHelper(this.diagram);
                        }
                    }

                    else {
                        if (this.selectedObject.type == "pseudoGroup" && this._fromContainer(this.diagram._getChildren(this.selectedObject.children))) {// && this._fromContainer(this.selectedObject.children) && this.hasSameParent()) {
                            if (this._fromContainer(this.diagram._getChildren(this.selectedObject.children))) {
                                this._rotate(this.selectedObject);
                                if (this.selectedObject.parent) {
                                    ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram);
                                }
                                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                                this.diagram._updateSelectionHandle(true);
                                this.diagram._renderTooltip(this.selectedObject, start);
                            }
                        }
                        else {
                            if (this.selectedObject.type === "pseudoGroup")
                                this._updateMultipleUndoObject(this.selectedObject);
                            this._rotate(this.selectedObject);
                            if (this.selectedObject.parent) {
                                ej.datavisualization.Diagram.Util._updateGroupBounds(this.diagram.nameTable[this.selectedObject.parent], this.diagram);
                            }
                            ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
                            this.diagram._updateSelectionHandle(true);
                            this.diagram._renderTooltip(this.selectedObject, start);
                        }
                    }
                }
            }
            this.previousPoint = this.currentPoint;
        };
        RotateTool.prototype._rotate = function (node) {
            var cancel = false;
            var args = this._raiseEvent("rotationChange", { element: this.diagram.getNode(node), cancel: false });
            if (!args.cancel) {
                var pt = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
                var crntpoint = this.currentPoint;
                var angle = ej.datavisualization.Diagram.Geometry.findAngle(ej.datavisualization.Diagram.Point(pt.x, pt.y), crntpoint) + 90;
                angle = ej.datavisualization.Diagram.SnapUtil._snapAngle(this.diagram, angle);
                this.diagram._rotate(node, angle - node.rotateAngle, this.diagram.nameTable);
            }
        };

        RotateTool.prototype._updateRotateObject = function () {
            var object = this.diagram.nameTable[this.selectedObject.name];
            ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
            if (this.helper) {
                var difAngle = this.helper.rotateAngle - this.diagram.nameTable[this.selectedObject.name].rotateAngle;
                //object.rotateAngle += difAngle;
                this.diagram._rotate(this.selectedObject, difAngle, this.diagram.nameTable);
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);
            }
            return object;
        };
        //RotateTool.prototype._updateMargin = function (node, group) {
        //    if (node.type == "pseudoGroup") {
        //        var child = null;
        //        for (var i = 0; i < node.children.length; i++) {
        //            child = this.diagram.nameTable[node.children[i]];
        //            this._updateMargin(child, group);
        //        }
        //    }
        //    else if (group && group.container && group.container.type == "canvas") {
        //        var groupBounds = ej.datavisualization.Diagram.Util.bounds(group); //ej.datavisualization.Diagram.SvgContext.getCanvasBoundingBox(group, this.diagram, node, true);
        //        var bounds1 = ej.datavisualization.Diagram.Util.bounds(node);
        //        var bounds = ej.datavisualization.Diagram.Geometry.rect([bounds1.topLeft, bounds1.topRight, bounds1.bottomRight, bounds1.bottomLeft]);
        //        node.marginLeft = bounds.x - (groupBounds.x + group.paddingLeft);
        //        node.marginRight = (groupBounds.x + group.width) - (bounds.x + bounds.width) - group.paddingRight;
        //        node.marginTop = bounds.y - (groupBounds.y + group.paddingTop);
        //        node.marginBottom = groupBounds.bottom - (bounds.y + bounds.height) - group.paddingBottom;
        //    }
        //};
        RotateTool.prototype._updateContainer = function (newObj, prevParent, isDrop) {
            var bounds, groupBounds, dx = 0, dy = 0, newSize;
            if (newObj) {
                var parObj = this.diagram.nameTable[newObj.parent];
                if (!parObj && newObj.type == "pseudoGroup") {
                    var fNode = this.diagram.nameTable[this.diagram._getChild(this.selectedObject.children[0])];
                    parObj = this.diagram.nameTable[fNode.parent];
                }
                if (parObj) {
                    if (parObj.container.type == "canvas") {
                        if (isDrop) {
                            isDrop = false;
                            var group = this.diagram.nameTable[newObj.parent];
                            if (!group && newObj.type == "pseudoGroup") {
                                group = parObj;
                            }
                            var groupBounds123 = ej.datavisualization.Diagram.Util.bounds(group);
                            var grAngle = group.rotateAngle;
                            this.diagram._rotate(group, -group.rotateAngle, this.diagram.nameTable);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                            bounds = ej.datavisualization.Diagram.Util.bounds(newObj);
                            var bounds1 = ej.datavisualization.Diagram.Geometry.rect([bounds.topLeft, bounds.topRight, bounds.bottomRight, bounds.bottomLeft]);
                            groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                            var tightchildBounds = this.diagram._getTightChildrenBounds(group);
                            var right = true;
                            var bottom = true;
                            var bRight = bounds1.x + bounds1.width;
                            var bBottom = bounds1.y + bounds1.height;
                            if (bRight >= groupBounds.right) {
                                ///if exeeds right 
                                dx = bRight - groupBounds.right + group.paddingRight;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                                if (bRight < groupBounds.right) {
                                    right = false;
                                }
                            }
                            if (bBottom >= groupBounds.bottom) {
                                ///if exeeds bottom 
                                dy = bBottom - groupBounds.bottom + group.paddingBottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                                bottom = true;
                                if (bBottom < groupBounds.bottom) {
                                    bottom = false;
                                }
                            }
                            if (bRight < groupBounds.right && right) { //if decreces from right
                                dx = (tightchildBounds.x + tightchildBounds.width) + group.paddingRight - groupBounds.right;
                                newSize = this.diagram._getMinMaxSize(group, (group.width + dx), group.height, true);
                                var widDiff = newSize.width - group.width;
                                group.width = newSize.width;
                                group.offsetX += (widDiff / 2);
                            }
                            if (bBottom < groupBounds.bottom && bottom) { //if decreces from bottom
                                dy = (tightchildBounds.y + tightchildBounds.height) + group.paddingBottom - groupBounds.bottom;
                                newSize = this.diagram._getMinMaxSize(group, (group.width), (group.height + dy), true);
                                var heiDiff = newSize.height - group.height;
                                group.height = newSize.height;
                                group.offsetY += (heiDiff / 2);
                            }
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                            this._updateMargin(newObj, group);
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            this.diagram._rotate(group, grAngle, this.diagram.nameTable);
                            var groupBounds1231 = ej.datavisualization.Diagram.Util.bounds(group);
                            this.diagram._translate(group, groupBounds123.topLeft.x - groupBounds1231.topLeft.x, groupBounds123.topLeft.y - groupBounds1231.topLeft.y, this.diagram.nameTable);
                            if (group.parent && group.container && group.container.type == "canvas")
                                this._updateMargin(group, this.diagram.nameTable[group.parent]);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        } else {
                            var vertical = false;
                            var horizontal = false;
                            group = this.diagram.nameTable[newObj.parent];
                            if (newObj.container && newObj.container.type == "stack") {
                                if (newObj.container.orientation == "vertical")
                                    vertical = true;
                                else
                                    horizontal = true;
                            }
                            bounds = this.diagram._getBoundsWithoutStretch(group, vertical, horizontal); // ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                            bounds.width += group.paddingLeft + group.paddingRight;
                            bounds.height += group.paddingTop + group.paddingBottom;
                            newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                            var tempWid = group.width;
                            var tempHei = group.height;
                            widDif = newSize.width - group.width;
                            heiDif = newSize.height - group.height;
                            group.offsetX += (widDif / 2);
                            group.width = newSize.width;
                            group.offsetY += (heiDif / 2);
                            group.height = newSize.height;
                            if (group.isSwimlane) {
                                if (group.orientation == "horizontal") {
                                    if (tempWid != group.width) {
                                        if (this._getLastPhase(group)) {
                                            var lastPhase = this._getLastPhase(group).name.split("phaseStack")[1];
                                            this._updateSelectedPhase(lastPhase, group.width - tempWid, 0.0001);
                                            var phaseObj = this._getphase(lastPhase);
                                            phaseObj.offset += (group.width - tempWid);
                                            ej.datavisualization.Diagram.SvgContext._updatephase(phaseObj, this.diagram);
                                        }
                                    }
                                }
                                else if (group.orientation == "vertical") {
                                    if (tempHei != group.height) {
                                        if (this._getLastPhase(group)) {
                                            var lastPhase = this._getLastPhase(group).name.split("phaseStack")[1];
                                            this._updateSelectedPhase(lastPhase, .00001, group.height - tempHei);
                                            var phaseObj = this._getphase(lastPhase);
                                            phaseObj.offset += (group.height - tempHei);
                                            ej.datavisualization.Diagram.SvgContext._updatephase(phaseObj, this.diagram);
                                        }
                                    }
                                }
                            }
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(group, this.diagram);
                            ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        }
                        if (group.parent) {
                            if (this.diagram.nameTable[group.parent])
                                isDrop = true;
                            this._updateContainer(group, null, isDrop);
                        }
                    }
                    else if (parObj.container.type == "stack") {
                        group = this.diagram.nameTable[newObj.parent];
                        bounds = ej.datavisualization.Diagram.SvgContext._measureStackSize(group, this.diagram, true);
                        newSize = this.diagram._getMinMaxSize(group, (bounds.width), (bounds.height), true);
                        var widDif = newSize.width - group.width;
                        var heiDif = newSize.height - group.height;
                        group.offsetX += (widDif / 2);
                        group.width = newSize.width;
                        group.offsetY += (heiDif / 2);
                        group.height = newSize.height;
                        ej.datavisualization.Diagram.SvgContext._alignOnStack(group, this.diagram);
                        ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                        if (group.parent)
                            this._updateContainer(group);
                    }
                }
            }
        };
        RotateTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                var isContainer = false;
                var canvasChild = false;
                if (this.selectedObject.parent) {
                    var parent = this.diagram.nameTable[this.selectedObject.parent];
                    if (parent.container) {
                        isContainer = true;
                    }
                }
                else if (this.selectedObject.type == "pseudoGroup" && this.hasSameParent() && this._fromContainer(this.diagram._getChildren(this.selectedObject.children))) {
                    var fNode = this.diagram.nameTable[this.diagram._getChild(this.selectedObject.children[0])];
                    parent = this.diagram.nameTable[fNode.parent];
                    if (parent.container) {
                        isContainer = true;
                    }
                }
                if ((this.selectedObject.type == "pseudoGroup" || this.selectedObject.name == "multipleSelection") && this.diagram._activeSwimLane) {
                    this._updateChildContainer(this.selectedObject);
                }
                else if (!this.diagram._activeSwimLane && (this.selectedObject.type == "pseudoGroup" || this.selectedObject.name == "multipleSelection")) {
                    this._updateChildContainer(this.selectedObject);
                }
                else if (isContainer) {
                    ej.datavisualization.Diagram.SvgContext._removeContainerHelper(this.selectedObject, this.diagram._adornerSvg, this.diagram._adornerLayer);
                    this._removeHighLighter(); //ej.datavisualization.Diagram.SvgContext._removeNodeHighlighter(this.diagram._svg, this.diagram._adornerLayer);
                    if (parent.container.type == "canvas")
                        canvasChild = true;
                    if (canvasChild) {
                        var object = this._updateRotateObject();
                        this._updateContainer(object, null, true);
                    }
                }
                if (this.selectedObject.type != "pseudoGroup")
                    this._rotate(this.selectedObject);
                else {
                    if (!this._fromContainer(this.diagram._getChildren(this.selectedObject.children)))
                        this._rotate(this.selectedObject);
                }
                ej.datavisualization.Diagram.DiagramContext.update(this.selectedObject, this.diagram);

                var data;
                var childTable = {};
                if (this.selectedObject._type === "group" || this.selectedObject.type === "pseudoGroup")
                    childTable = this.diagram._getChildTable(this.selectedObject, childTable);
                var temp = { "childTable": childTable, "node": this.selectedObject };
                data = $.extend(true, {}, temp);
                var entry = { type: "rotationchanged", undoObject: this.undoObject, redoObject: jQuery.extend(true, {}, data), isMultipleNode: (this.selectedObject._type === "group") ? true : false, category: "internal" };
                if (!this._multipleUndo)
                    this.diagram.addHistoryEntry(entry);
                if (this.selectedObject)
                    ej.datavisualization.Diagram.SvgContext._removePivotPoint(this.selectedObject, this.diagram._adornerSvg, this.diagram._currZoom);
                this.diagram._updateSelectionHandle();
                this.diagram._removeTooltip();
            }
            this._multipleUndo = false;
            //this.diagram._removeTooltip(); 
            base.prototype.mouseup.call(this, evt);
        };
        RotateTool.prototype._updateChildContainer = function (pseudoGroup) {
            var collection = [];
            if (this.diagram._activeSwimLane) {
                var lanes = this._getLanes(this.diagram._activeSwimLane), child, bounds;
                for (var i = 0; i < pseudoGroup.children.length; i++) {
                    child = this.diagram.nameTable[this.diagram._getChild(pseudoGroup.children[i])]
                    var lane = this._childExistOnLane(child, lanes);
                    if (lane && child && !child.segments) {
                        child.parent = lane.name;
                        var parent = this.diagram.nameTable[child.parent];
                        parent.children.push(child.name);

                        if (parent)
                            this._updateMargin(child, parent);
                        //this._addToContainer(child, true);
                        ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram, true);
                        this._updateContainer(child, parent.name, true);
                        ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);
                        collection.push(this._getUpdatedNode(child));
                    }
                    else if (child.segments) {
                        ej.datavisualization.Diagram.Util.removeItem(this.diagram.nodes(), child)
                        this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                        collection.push(this._getUpdatedNode(child));
                    }
                }
                ej.datavisualization.Diagram.Util._updateGroupBounds(pseudoGroup, this.diagram);
                ej.datavisualization.Diagram.SvgContext.updateSelector(pseudoGroup, this.diagram._adornerSvg, this.diagram._currZoom, this.diagram, this.diagram.model.selectedItems.constraints);
            }
            else {
                for (i = 0; pseudoGroup.children && pseudoGroup.children.length > 0 && i < pseudoGroup.children.length; i++) {
                    child = this.diagram.nameTable[this.diagram._getChild(pseudoGroup.children[i])];
                    var swimlane = this._getBottomSwimlane(child);
                    if (swimlane) {
                        this.diagram._activeSwimLane = swimlane;
                        child = this.diagram.nameTable[this.diagram._getChild(pseudoGroup.children[i])]
                        lanes = this._getLanes(this.diagram._activeSwimLane)
                        var lane = this._childExistOnLane(child, lanes);
                        if (lane) {
                            child.parent = lane.name;
                            var parent = this.diagram.nameTable[child.parent];
                            parent.children.push(child.name);
                            //this._moveOnLane(child, this.diagram.nameTable[this.diagram._getChild(child.parent)]);
                            if (parent)
                                this._updateMargin(child, parent);
                            this._addToContainer(child, true);
                            ej.datavisualization.Diagram.SvgContext._alignOnCanvas1(parent, this.diagram);
                            this._updateContainer(child, parent.name, true);
                            ej.datavisualization.Diagram.DiagramContext.update(child, this.diagram);
                        }
                    }
                    collection.push(this._getUpdatedNode(child));
                }
                this.selectedObject = pseudoGroup;
            }
            this._redoObject = { collection: collection, adjustLines: jQuery.extend(true, {}, this._adjustLines) }
            if ((this._undoObject && this._collectionFromLane(this._undoObject.collection)) || (this._redoObject && this._collectionFromLane(this._redoObject.collection))) {
                var entry = { type: "rotationchanged", undoObject: jQuery.extend(true, {}, this._undoObject), redoObject: jQuery.extend(true, {}, this._redoObject), category: "internal", swimlaneMultiSelection: true };
                this.diagram.addHistoryEntry(entry);
                this._multipleUndo = true;
            }
        };
        RotateTool.prototype._addToContainer = function (node, addChildren) {
            if (node.type == "pseudoGroup") {
                var child = null;
                var children = this.diagram._getChildren(node.children);
                for (var i = 0; i < children.length; i++) {
                    child = this.diagram.nameTable[children[i]];
                    if (child) {
                        parent = this.diagram._svg.getElementById(child.parent);
                        if (!parent)
                            parent = this.diagram._diagramLayer;
                        if (child._type === "group") {
                            ej.datavisualization.Diagram.SvgContext.renderGroup(child, this.diagram._svg, parent, this.diagram.nameTable, this.diagram);
                        } else if (child.segments) {
                            ej.datavisualization.Diagram.SvgContext.renderConnector(child, this.diagram._svg, parent);
                        } else {
                            ej.datavisualization.Diagram.SvgContext.renderNode(child, this.diagram._svg, parent, undefined, this.diagram);
                        }
                    }
                }
            } else {
                //this.diagram._clearSelection(true);
                var overNode = this._getNodeUnderMouse();
                var parent;
                if (overNode && overNode.children && (!overNode.isSwimlane) && !addChildren) {
                    overNode.children.push(node.name);
                    //this.diagram.nameTable[node.name] = node;
                }
                if (node) {
                    parent = this.diagram._svg.getElementById(node.parent);
                    if (!parent)
                        parent = this.diagram._diagramLayer;
                    this.diagram.nodes().push(node);
                    this.diagram._nodes = $.extend(true, [], this.diagram.nodes());
                    if (node._type === "group") {
                        this.diagram._initGroupNode(node);
                        ej.datavisualization.Diagram.SvgContext.renderGroup(node, this.diagram._svg, parent, this.diagram.nameTable, this.diagram);
                    } else if (node.segments) {
                        ej.datavisualization.Diagram.SvgContext.renderConnector(node, this.diagram._svg, parent);
                    } else {
                        ej.datavisualization.Diagram.SvgContext.renderNode(node, this.diagram._svg, parent, undefined, this.diagram);
                    }
                }
            }
            this.selectedObject = node;
        };
        return RotateTool;
    })(ToolBase);

    ej.datavisualization.Diagram.RotateTool = RotateTool;
    //#endregion 

    //#region PivotTool
    var PivotTool = (function (base) {
        ej.datavisualization.Diagram.extend(PivotTool, base);
        function PivotTool(diagram) {
            base.call(this, "PivotTool", diagram);
            this._mouseOffset = ej.datavisualization.Diagram.Size(0, 0);
        }
        PivotTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.selectedObject = this.diagram.selectionList[0];
        };
        PivotTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);

            if (this.selectedObject) {
                if (!this.inAction) {
                    this.inAction = true;
                }
                else {
                    var delx = this.currentPoint.x - this.previousPoint.x;
                    var dely = this.currentPoint.y - this.previousPoint.y;
                    var matrix = ej.Matrix.identity();
                    ej.Matrix.rotate(matrix, -this.selectedObject.rotateAngle);
                    var delta = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(delx, dely));
                    var difx = (this.selectedObject.width * this.selectedObject.pivot.x + delta.x) / this.selectedObject.width;
                    var dify = (this.selectedObject.height * this.selectedObject.pivot.y + delta.y) / this.selectedObject.height;
                    this.selectedObject.pivot.x = difx;
                    this.selectedObject.pivot.y = dify;
                    this.selectedObject.offsetX += delx;
                    this.selectedObject.offsetY += dely;
                    this.diagram._updateSelectionHandle();
                }
            }
            this.previousPoint = this.currentPoint;
        };
        PivotTool.prototype._rotate = function (node) {
            var pt = ej.datavisualization.Diagram.Point(node.offsetX, node.offsetY);
            var crntpoint = this.currentPoint;
            var angle = this.FindAngle(pt, crntpoint);
            this.diagram._rotate(node, angle - node.rotateAngle, pt, this.diagram.nameTable);
        };
        PivotTool.prototype.mouseup = function (evt) {
            if (this.inAction) {
                this.inAction = false;
                this.diagram._updateSelectionHandle();
            }
            base.prototype.mouseup.call(this, evt);
        };
        return PivotTool;
    })(ToolBase);
    ej.datavisualization.Diagram.PivotTool = PivotTool;
    //#endregion

    //#region PanTool
    var PanTool = (function (base) {
        ej.datavisualization.Diagram.extend(PanTool, base);
        function PanTool(diagram) {
            base.call(this, "panTool", diagram);
            this.diagram = diagram;
            this._isMouseDown = false;
            this.cursor = "pointer";
        }
        PanTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            var svgDocument = this.diagram._svg.document;
            var pt = this.diagram._svg.document.createSVGPoint();
            var originalEvent = this.diagram._isTouchEvent(evt);
            pt.x = originalEvent.clientX;
            pt.y = originalEvent.clientY;
            var mousept = pt.matrixTransform(this.diagram._svg.document.getScreenCTM().inverse());
            svgDocument.lastMouseX = mousept.x;
            svgDocument.lastMouseY = mousept.y;
            this._isMouseDown = true;
            return false;
        };

        PanTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (this._isMouseDown && this.startPoint != this.currentPoint) {
                this.inAction = true;
                var svgDocument = this.diagram._svg.document;
                var pt = this.diagram._svg.document.createSVGPoint();
                var originalEvent = this.diagram._isTouchEvent(evt);
                pt.x = originalEvent.clientX;
                pt.y = originalEvent.clientY;
                var mousept = pt.matrixTransform(this.diagram._svg.document.getScreenCTM().inverse());
                var ex = mousept.x;
                var ey = mousept.y;
                var nx, ny;

                nx = ((ex - svgDocument.lastMouseX));
                ny = ((ey - svgDocument.lastMouseY));
                if (!(this.diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableX))
                    nx = 0;
                if (!(this.diagram.model.constraints & ej.datavisualization.Diagram.DiagramConstraints.PannableY))
                    ny = 0;
                ej.datavisualization.Diagram.ZoomUtil.zoomPan(this.diagram, 1, nx, ny, ej.datavisualization.Diagram.Point(ex, ey));
                svgDocument.lastMouseX = ex;
                svgDocument.lastMouseY = ey;
                return false;
            }
        };
        PanTool.prototype.mouseup = function (evt) {
            this.inAction = false;
            document.onmousemove = null;
            document.onmouseup = null;
            this._isMouseDown = false;
            base.prototype.mouseup.call(this, evt);
        };
        return PanTool;
    })(ToolBase);
    ej.datavisualization.Diagram.PanTool = PanTool;
    //#endregion


    //#region TextToolBase
    var TextToolBase = (function (base) {
        ej.datavisualization.Diagram.extend(TextToolBase, base);
        function TextToolBase(name, diagram) {
            base.call(this, name, diagram);
            this.boundingRect = ej.datavisualization.Diagram.Rectangle();
            this.activeLabel = null;
            this._overNode = "";
            this.cursor = "crosshair";
        }
        TextToolBase.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this._findNodeUnderMouse(evt);
            var overNode = this._getNodeUnderMouse();
            if (overNode) this._overNode = (this.selectedObject && !this.selectedObject.isLane || overNode.isLane) ? overNode.name : "";
            if (this.diagram._isEditing) {
                this.diagram._endEdit();
            }
        };
        TextToolBase.prototype.createNode = function (rect) {

        };
        TextToolBase.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (ej.datavisualization.Diagram.Util.isPageEditable(this.diagram)) {
                if (!this.selectedObject && !ej.datavisualization.Diagram.Geometry.isEmptyPoint(this.startPoint) && !this.diagram._isEditing) {
                    if (!this.inAction) {
                        this.inAction = true;
                        this._initHelper();
                    }
                    else {
                        this._updateHelper();
                    }
                }
            }
        };
        TextToolBase.prototype.mouseup = function (evt) {
            this.activeLabel = null;
            var minHeight = 12;
            if (this.inAction) {
                var rect = this.getBoundingRect(this.startPoint, this.currentPoint);
                this._endAction();
                if (rect.width === 0 || rect.height === 0) {
                    rect.width = 90;
                    rect.height = minHeight;
                }
                this.selectedObject = this.createNode(rect.x + (rect.width / 2), (rect.y + (rect.height / 2)), rect.width, rect.height);
                this.diagram._startEdit(this.selectedObject);
            }
            else {
                var node = this._findNodeUnderMouse(evt);
                if (ej.datavisualization.Diagram.Util.isPageEditable(this.diagram)) {
                    if (node != null && !node.isSwimlane && !node.isLane) {
                        this.selectedObject = node;
                        this.diagram._startEdit(this.selectedObject);
                    }
                    else {
                        var rect = ej.datavisualization.Diagram.Rectangle(this.currentPoint.x - (90 / 2), this.currentPoint.y - (minHeight / 2), 90, minHeight);
                        this._endAction();
                        this.selectedObject = this.createNode(rect.x + (rect.width / 2), rect.y + (rect.height / 2), rect.width, minHeight);
                        this.diagram._startEdit(this.selectedObject);
                    }
                }
            }
            this._overNode = "";
        };
        TextToolBase.prototype.getBoundingRect = function (startPoint, currentPoint) {
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var x = (this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x;
            var y = (this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y;
            return { x: x, y: y, width: width, height: height };
        };
        TextToolBase.prototype._initHelper = function () {
            if (!this._svgHelper) {
                var g = this.diagram._adornerSvg.g();
                this.diagram._adornerLayer.appendChild(g);
                var scale = this.diagram._currZoom;
                var selectionRect = this.diagram._svg.rect({ "id": "helper", "x": this.startPoint.x * scale, "y": this.startPoint.y * scale, "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges" });
                g.appendChild(selectionRect);
                g.setAttribute("pointer-events", "none");
                this._svgHelper = g;
            }
        };
        TextToolBase.prototype._updateHelper = function () {
            var helper = this.diagram._adornerSvg.getElementById("helper");
            var width = Math.abs(this.startPoint.x - this.currentPoint.x);
            var height = Math.abs(this.startPoint.y - this.currentPoint.y);
            var scale = this.diagram._currZoom;
            var x = ((this.startPoint.x > this.currentPoint.x) ? this.currentPoint.x : this.startPoint.x) * scale;
            var y = ((this.startPoint.y > this.currentPoint.y) ? this.currentPoint.y : this.startPoint.y) * scale;
            if (helper) {
                this.diagram._adornerSvg.rect({ "id": helper.id, "x": x, "y": y, "width": width * scale, "height": height * scale });
            }
        };
        TextToolBase.prototype._endAction = function () {
            if (this._svgHelper) {
                this.diagram._adornerLayer.removeChild(this._svgHelper);
                this._svgHelper = null;
            }
            base.prototype._endAction.apply(this);
        };
        TextToolBase.prototype._findNodeUnderMouse = function (evt, skip) {
            var obj;
            var foreignObject = this.diagram._isForeignObject(evt.target);
            if (foreignObject)
                evt.target = foreignObject;
            var textElement = $(evt.target).context;
            if (textElement.getAttribute("class") == "ej-d-label") {
                var parentNode = $(textElement).parents(".ej-d-node,.ej-d-connector,.ej-d-group")[0];
                obj = this._findObj(parentNode);
                this.activeLabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
            }
            else {
                var parents = $(evt.target).parents("g");
                if (parents.length > 3) {
                    var element = parents[parents.length - 4];
                    var id = element.getAttribute("id");
                    var type = element.getAttribute("class");
                    if (type) {
                        if (type === "ej-d-node" || type === "ej-d-group") {
                            obj = this.diagram._findNode(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                        else if (type === "ej-d-connector") {
                            obj = this.diagram._findConnector(id);
                            var curlabel = this.diagram._findLabelAtPoint(this.currentPoint, obj);
                            this.activeLabel = curlabel;
                        }
                    }
                }
            }
            return obj;
        };
        return TextToolBase;
    })(ej.datavisualization.Diagram.ToolBase);
    ej.datavisualization.Diagram.TextToolBase = TextToolBase;
    //#endregion


    //#region TextTool
    var TextTool = (function (base) {
        ej.datavisualization.Diagram.extend(TextTool, base);
        function TextTool(diagram) {
            base.call(this, "textTool", diagram);
        }
        TextTool.prototype.createNode = function (x, y, width, height) {
            var node = this.diagram._getNewNode({ name: "text" + ej.datavisualization.Diagram.Util.randomId(), offsetX: x, offsetY: y, width: width, height: height, borderColor: "transparent", fillColor: "transparent", labels: [], ports: [], type: "text", parent: this._overNode });
            if (node._type = "text") node._type = "node";
            var parent = this.diagram.nameTable[node.parent];
            if (parent && parent.container)
                this._updateMargin(node, parent);
            return node;
        };
        return TextTool;
    })(ej.datavisualization.Diagram.TextToolBase);
    ej.datavisualization.Diagram.TextTool = function (diagram) {
        return new TextTool(diagram);
    };
    //#endregion


    var PhaseTool = (function (base) {
        ej.datavisualization.Diagram.extend(PhaseTool, base);
        function PhaseTool(diagram) {
            base.call(this, "phase", diagram);
            this.selectedSeperetor = null;
            this._svgHelper = null;
        }

        PhaseTool.prototype.mousedown = function (evt) {
            base.prototype.mousedown.call(this, evt);
            this.inAction = true;
        };
        PhaseTool.prototype._getSeperator = function (evt) {
            if ($(evt.target).parents("g").first()[0]) {
                var id = $(evt.target).parents("g").first()[0].id;
                return this._getphase(id);
            }
        };
        PhaseTool.prototype._removePhasehelper = function () {
            ej.datavisualization.Diagram.SvgContext._removePhasehelper(this.diagram);
        };
        PhaseTool.prototype._updatePhasehelper = function (phase) {
            var bounds = this._bounds ? jQuery.extend(true, {}, this._bounds) : this.diagram._getPhaseBounds(phase), dx, dy;
            var ptSrt = this.startPoint;
            var ptCur = this.currentPoint;
            var horizontalsnap = { snapped: false, offset: 0, left: false, right: false };
            var verticalsnap = { snapped: false, offset: 0, top: false, bottom: false };
            this.selectedObject = this.diagram.nameTable[this.selectedSeperetor.parent];
            this.initialBounds = { width: this.selectedObject.width, height: this.selectedObject.height, y: this.selectedObject.offsetY, x: this.selectedObject.offsetX };

            if (phase.orientation === "horizontal") {
                dx = ptCur.x - ptSrt.x;
                dx = this._snapRight(horizontalsnap, verticalsnap, dx, 0, this.selectedObject, false);
                dy = 0;
            }
            else {
                dx = 0;
                dy = ptCur.y - ptSrt.y;
                dy = this._snapBottom(horizontalsnap, verticalsnap, 0, dy, this.selectedObject, false);
            }
            if (bounds) {
                bounds.width += dx;
                bounds.height += dy;
                ej.datavisualization.Diagram.SvgContext._updatePhaseHelper(this.diagram, phase, bounds);
            }
        };

        PhaseTool.prototype._drawPhasehelper = function (phase) {
            var bounds = this.diagram._getPhaseBounds(phase);
            if (phase && bounds)
                ej.datavisualization.Diagram.SvgContext._phasehelper(this.diagram, phase, bounds);
        };
        PhaseTool.prototype.mousemove = function (evt) {
            base.prototype.mousemove.call(this, evt);
            if (!this.selectedSeperetor) {
                if (this.inAction) {
                    var phase = this._getSeperator(evt);
                    if (phase && !this.selectedSeperetor) {
                        this.selectedSeperetor = phase;
                    }
                }
                this._drawPhasehelper(this.selectedSeperetor);
                if (this.diagram.selectionList[0] && this.diagram.selectionList[0].type != "phase") {
                    this.diagram._clearSelection(true);
                    this.diagram.addSelection(this.selectedSeperetor, true);
                }
                else
                    this.diagram.addSelection(this.selectedSeperetor, true);
            }
            else if (this.selectedSeperetor) {
                if (!this._bounds)
                    this._bounds = this.diagram._getPhaseBounds(this.selectedSeperetor);
                this._updatePhasehelper(this.selectedSeperetor);
            }
            this.previousPoint = this.currentPoint;
        };
        PhaseTool.prototype._updateLaneMinValue = function (groupname, dx, dy) {
            var group = this.diagram.nameTable[groupname], child;
            if (group && group.children && group.children.length > 0) {
                var stack = this.diagram.nameTable[group.children[2]];
                if (stack && stack.children && stack.children.length > 0) {
                    for (var i = 0; i < stack.children.length; i++) {
                        child = this.diagram.nameTable[stack.children[i]];
                        if (child) {
                            child.minHeight = child.height + dy;
                            child.minWidth = child.width + dx;
                        }
                    }
                }
            }
        };
        PhaseTool.prototype._updateSwimlane = function (seperator, dx, dy, isVertical, isApi, updateChildren) {
            var incX = 0, incY = 0;
            dx = dx ? dx : 0.000001;
            dy = dy ? dy : 0.000001;
            dx = -dx;
            dy = -dy;
            if (isVertical) {
                dx = 0.000001;
                if (this.selectedSeperetor && this.selectedSeperetor.parent)
                    var group = this.diagram.nameTable[this.selectedSeperetor.parent];
                if (group) {
                    var outerPhasechildren = this._getOuterPhaseNodes(group, isVertical);
                    var leastOffset = this._getLeastOffset(group, isVertical);
                    var groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                    var ptCur = this.snap(this.currentPoint);
                    var diff = ptCur.y - (groupBounds.y + this.selectedSeperetor.offset);
                    if (ptCur.y < leastOffset + (20 * this.diagram._currZoom)) {
                        diff = (leastOffset + (10 * this.diagram._currZoom)) - (groupBounds.y + this.selectedSeperetor.offset);
                        if (isApi)
                            diff = dy;
                        seperator.offset += diff;
                        this._updateSelectedPhase(seperator, dx, diff);
                        incY = diff;
                    }
                    else {
                        if (isApi)
                            dy = -dy;
                        seperator.offset += dy;
                        this._updateSelectedPhase(seperator, dx, dy);
                        incY = dy;
                    }
                    this._updateLaneMinValue(seperator.parent, 0, incY);
                    if (outerPhasechildren.length && !updateChildren) {
                        for (var i in outerPhasechildren) {
                            this.diagram._translate(outerPhasechildren[i], 0, incY, this.diagram.nameTable);
                            outerPhasechildren[i].marginTop += incY;
                            ej.datavisualization.Diagram.DiagramContext.update(outerPhasechildren[i], this.diagram);
                        }
                    }
                }
            }
            else {
                dy = 0.000001;
                if (this.selectedSeperetor && this.selectedSeperetor.parent)
                    var group = this.diagram.nameTable[this.selectedSeperetor.parent];
                if (group) {
                    var outerPhasechildren = this._getOuterPhaseNodes(group, isVertical);
                    var leastOffset = this._getLeastOffset(group, isVertical);
                    var groupBounds = ej.datavisualization.Diagram.Util.bounds(group);
                    var ptCur = this.snap(this.currentPoint);
                    var diff = ptCur.x - (groupBounds.x + this.selectedSeperetor.offset);

                    if (ptCur.x < leastOffset + (20 * this.diagram._currZoom)) {
                        diff = (leastOffset + (10 * this.diagram._currZoom)) - (groupBounds.x + this.selectedSeperetor.offset);
                        if (isApi)
                            diff = -dx;
                        seperator.offset += diff;
                        this._updateSelectedPhase(seperator, diff, dy);
                        incX = diff;
                    }
                    else {
                        if (isApi)
                            dx = -dx;
                        seperator.offset += dx;
                        this._updateSelectedPhase(seperator, dx, dy);
                        incX = dx;
                    }
                    this._updateLaneMinValue(seperator.parent, incX, 0);
                    if (outerPhasechildren.length && !updateChildren) {
                        for (var i in outerPhasechildren) {
                            this.diagram._translate(outerPhasechildren[i], incX, 0, this.diagram.nameTable);
                            outerPhasechildren[i].marginLeft += incX;
                            ej.datavisualization.Diagram.DiagramContext.update(outerPhasechildren[i], this.diagram);
                        }
                    }
                }
            }
            var phase = this.diagram.nameTable[seperator.name];
            if (phase && phase.parent) {
                var group = this.diagram.nameTable[phase.parent];
                if (group) {
                    var resizeTool = this.diagram.tools["resize"];
                    resizeTool.selectedObject = group;
                    if (group.orientation === "vertical")
                        resizeTool._resizeDirection = "s-resize";
                    else
                        resizeTool._resizeDirection = "e-resize";
                    this.diagram.selectionList.push(group);
                    resizeTool.initialBounds = { width: group.width, height: group.height, y: group.offsetY, x: group.offsetX };
                    group.minWidth += incX;
                    group.minHeight += incY;
                    this.diagram.isPhaseResize = true;
                    resizeTool._updateSize(group, null, null, incX ? incX : .00001, incY ? incY : .00001, false, null, true, true);
                    this.diagram.isPhaseResize = false;
                    this._updaLaneChildMargin(group);
                    ej.datavisualization.Diagram.DiagramContext.update(group, this.diagram);
                }
            }
            return { x: incX, y: incY }
        },
        PhaseTool.prototype._updaLaneChildMargin = function (group) {
            if (group.isSwimlane) {
                var laneStack = this.diagram.nameTable[this.diagram._getChild(group.children[group.children.length - 1])];
                if (laneStack) {
                    var lanes = laneStack.children;
                    if (lanes.length > 0) {
                        for (var i = 0; i < lanes.length; i++) {
                            var lane = this.diagram.nameTable[this.diagram._getChild(lanes[i])];
                            if (lane && lane.children && lane.children.length > 1) {
                                var children = lane.children;
                                for (var j = 1; j < children.length; j++) {
                                    var child = this.diagram.nameTable[this.diagram._getChild(children[j])];
                                    if (child) {
                                        this._updateMargin(child, lane);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        },
        PhaseTool.prototype._updateResizeValue = function (phase, dx, dy) {
            if (phase.orientation === "horizontal") {
                var swimlane = this.diagram.nameTable[phase.parent];
                if (swimlane) {
                    var index = this.diagram._getPhaseIndex(phase.name);
                    if (index === 0) {
                        var stack = this.diagram.nameTable[this.diagram._getChild(swimlane.children[2])]
                        if (stack)
                            var firstlane = this.diagram.nameTable[this.diagram._getChild(stack.children[0])];
                        if (firstlane)
                            var laneHeader = this.diagram.nameTable[this.diagram._getChild(firstlane.children[0])];
                        if (laneHeader) {

                            if (dx >= (phase.offset - laneHeader.width))
                                dx -= (dx - (phase.offset - laneHeader.width) + 10);
                        }
                    }
                    else {
                        var prevPhase = this.diagram.nameTable[swimlane.phases[index - 1]];
                        if (dx >= (phase.offset - prevPhase.offset))
                            dx -= (dx - (phase.offset - prevPhase.offset) + 10);
                    }
                }
            }
            else {
                var swimlane = this.diagram.nameTable[phase.parent];
                if (swimlane) {
                    var index = this.diagram._getPhaseIndex(phase.name);
                    if (index === 0) {
                        var stack = this.diagram.nameTable[this.diagram._getChild(swimlane.children[2])]
                        if (stack)
                            var firstlane = this.diagram.nameTable[this.diagram._getChild(stack.children[0])];
                        if (firstlane)
                            var laneHeader = this.diagram.nameTable[this.diagram._getChild(firstlane.children[0])];
                        if (laneHeader) {
                            var swimlaneHeader = this._getSwimlaneHeader(swimlane)
                            if (dy >= (phase.offset - (laneHeader.height + swimlaneHeader.height)))
                                dy -= (dy - (phase.offset - (laneHeader.height + swimlaneHeader.height)) + 10);
                        }
                    }
                    else {
                        var prevPhase = this.diagram.nameTable[swimlane.phases[index - 1]];
                        if (dy >= (phase.offset - prevPhase.offset))
                            dy -= (dy - (phase.offset - prevPhase.offset) + 10);
                    }
                }
            }
            return { dx: dx, dy: dy }
        },
        PhaseTool.prototype.mouseup = function (evt) {
            var phase = this._getSeperator(evt);
            if (this.diagram.selectionList.length > 0 && phase && this.diagram.selectionList[0].name != phase.name)
                this.diagram._clearSelection(true);
            if (phase && !this.selectedSeperetor) {
                this.selectedSeperetor = phase;
                this.diagram.addSelection(this.selectedSeperetor, true);
            }
            else {
                if (this.selectedSeperetor) {
                    var ptCur = this.currentPoint;
                    var ptSrt = this.startPoint;
                    var horizontalsnap = { snapped: false, offset: 0, left: false, right: false };
                    var verticalsnap = { snapped: false, offset: 0, top: false, bottom: false };
                    this.selectedObject = this.diagram.nameTable[this.selectedSeperetor.parent];
                    this.initialBounds = { width: this.selectedObject.width, height: this.selectedObject.height, y: this.selectedObject.offsetY, x: this.selectedObject.offsetX };

                    if (this.selectedSeperetor.orientation === "horizontal") {
                        var dx = ptSrt.x - ptCur.x;
                        dx = this._snapRight(horizontalsnap, verticalsnap, dx, 0, this.selectedObject, false);
                        dy = 0;
                    }
                    else {
                        var dy = ptCur.y - ptSrt.y;
                        dy = -this._snapBottom(horizontalsnap, verticalsnap, 0, dy, this.selectedObject, false);
                        dx = 0;
                    }

                    var undoObject = jQuery.extend(true, {}, this.selectedSeperetor);
                    var vertical = this.selectedSeperetor.orientation === "vertical" ? true : false;
                    if (this.selectedSeperetor.type === "phase") {
                        var dif = this._updateResizeValue(this.selectedSeperetor, dx, dy);
                        var rValue = this._updateSwimlane(this.selectedSeperetor, dif.dx, dif.dy, vertical);
                    }
                    //if (this.selectedSeperetor.orientation == "horizontal")
                    //    this.selectedSeperetor.offset -= dx;
                    //else
                    //    this.selectedSeperetor.offset -= dy;

                    ej.datavisualization.Diagram.SvgContext._updatephase(this.selectedSeperetor, this.diagram);
                    var entry = {
                        type: "phasesizechanged", undoObject: undoObject, redoObject: jQuery.extend(true, {}, this.selectedSeperetor),
                        startPoint: this.startPoint, currentPoint: this.currentPoint,
                        modifiedValues: { dx: (rValue && (rValue.x || rValue.x === 0)) ? (-rValue.x) : dx, dy: (rValue && (rValue.y || rValue.y === 0)) ? (-rValue.y) : dy }, category: "internal"
                    };
                    this.diagram.addHistoryEntry(entry);

                    if ((this.diagram.selectionList[0] && this.diagram.selectionList[0].type != "phase") || this.inAction) {
                        this.diagram._clearSelection(true);
                        this.diagram.addSelection(this.selectedSeperetor, true);
                    }
                    else
                        this.diagram.addSelection(this.selectedSeperetor, true);
                }
                this._removePhasehelper();
            }
            this._bounds = null;
            this.inAction = false;
            this.selectedSeperetor = null;
            base.prototype.mouseup.call(this, evt);
        };

        PhaseTool.prototype._getLeastOffset = function (group, isVertical) {
            var offset;
            if (group) {
                var stack = this.diagram.nameTable[this.diagram._getChild(group.children[2])];
                var tchildren = [];
                var tchild = null;
                var bounds = null;
                var laneChild = null;
                var preOffSet;
                for (var i = 0; stack && i < stack.children.length; i++) {
                    var lane = this.diagram.nameTable[stack.children[i]];
                    if (lane && lane.children && lane.children.length > 1) {
                        for (var j = 1; j < lane.children.length; j++) {
                            laneChild = this.diagram.nameTable[lane.children[j]];
                            if (laneChild) {
                                bounds = ej.datavisualization.Diagram.Util.bounds(laneChild);
                                if (isVertical) {
                                    if ((bounds.y + bounds.height) < (group.offsetY - group.height / 2) + this.selectedSeperetor.offset) {
                                        if (offset && offset < bounds.y + bounds.height)
                                            offset = bounds.y + bounds.height;
                                        else offset = bounds.y + bounds.height;
                                        if (!preOffSet || (preOffSet && offset >= preOffSet))
                                            preOffSet = offset;
                                    }
                                }
                                else {
                                    if ((bounds.x + bounds.width) < (group.offsetX - group.width / 2) + this.selectedSeperetor.offset) {
                                        if (offset && offset < bounds.x + bounds.width)
                                            offset = bounds.x + bounds.width;
                                        else offset = bounds.x + bounds.width;
                                        if ((!preOffSet || (preOffSet && offset >= preOffSet)))
                                            preOffSet = offset;
                                    }
                                }
                            }
                        }
                    }
                    else {
                    }
                }
            }
            return preOffSet;
        };
        PhaseTool.prototype._getOuterPhaseNodes = function (group, isVertical) {
            if (group) {
                var stack = this.diagram.nameTable[this.diagram._getChild(group.children[2])];
                var tchildren = [];
                var tchild = null;
                var bounds = null;
                var laneChild = null
                for (var i = 0; stack && i < stack.children.length; i++) {
                    var lane = this.diagram.nameTable[this.diagram._getChild(stack.children[i])];
                    if (lane && lane.children && lane.children.length > 1) {
                        for (var j = 0; j < lane.children.length; j++) {
                            laneChild = this.diagram.nameTable[this.diagram._getChild(lane.children[j])];
                            if (laneChild) {
                                bounds = ej.datavisualization.Diagram.Util.bounds(laneChild);
                                if (isVertical) {
                                    if (bounds.y > ((group.offsetY - group.height / 2) + this.selectedSeperetor.offset))
                                        tchildren.push(laneChild);
                                }
                                else {
                                    if (bounds.x > ((group.offsetX - group.width / 2) + this.selectedSeperetor.offset))
                                        tchildren.push(laneChild);
                                }
                            }
                        }
                    }
                }
            }
            return tchildren;
        };
        return PhaseTool;
    })(ToolBase);
    ej.datavisualization.Diagram.PhaseTool = PhaseTool;
})(jQuery, Syncfusion);;
(function ($, ej) {
    "use strict";
    ej.datavisualization.Diagram.LayoutTypes = {
        None: "none",
        HierarchicalTree: "hierarchicaltree",
        OrganizationalChart: "organizationalchart",
        RadialTree: "radialtree"
    };
    ej.datavisualization.Diagram.ChartOrientations = {
        Horizontal: "horizontal",
        Vertical: "vertical"
    };
    ej.datavisualization.Diagram.ChartTypes = {
        Left: "left",
        Right: "right",
        Alternate: "alternate",
        Center: "center"
    };

    ej.datavisualization.Diagram.Layout = {
        doLayout: function (diagram) {
            var model = diagram.model;
            var layout = {
                diagram: diagram, _anchorX: 0, _anchorY: 0, _graphNodes: {},
                _firstLevelNodes: [], _centerNode: null, levels: [], maxLevel: 0, layoutNodes: [],
            };
            for (var prop in model.layout)
                layout[prop] = model.layout[prop];

            if (model.layout.type != "radialtree")
                ej.datavisualization.Diagram.HierarchicalLayout.doLayout(layout);
            else
                ej.datavisualization.Diagram.RadialTreeLayout.doLayout(layout);
        }
    };

    //region Hierarchical layout
    ej.datavisualization.Diagram.HierarchicalLayout = {
        doLayout: function (layout) {
            var diagram, model, nodes, node, layoutInfo, i;
            diagram = layout.diagram
            model = diagram.model;
            nodes = diagram.nodes();
            if (nodes.length > 0) {
                for (var i = 0; i < nodes.length; i++) {
                    node = nodes[i];
                    if (!node.excludeFromLayout) {
                        layoutInfo = layout._graphNodes[node.name] = this._setUpLayoutInfo(layout, node);
                        layoutInfo.tree.hasSubTree = false;
                        if (!node.inEdges || !node.inEdges.length) { layout._firstLevelNodes.push(node); }
                    }
                }

                //Update relationship(parent and children)
                for (i = 0; i < layout._firstLevelNodes.length; i++) {
                    node = layout._firstLevelNodes[i];
                    this._updateEdges(layout, node, 1);
                }

                if (layout._firstLevelNodes.length > 0) {
                    layout._rootNode = layout._firstLevelNodes[0];
                    var bounds, x = 0, y = 0, minX, maxX, minY, maxY;
                    for (i = 0; i < layout._firstLevelNodes.length; i++) {
                        bounds = this._updateTree(layout, x, y, layout._firstLevelNodes[i], 0, layout._firstLevelNodes[i - 1]);
                        if (layout.orientation == "lefttoright" || layout.orientation == "righttoleft") y = bounds.right + layout.horizontalSpacing;
                        else x = bounds.right + layout.horizontalSpacing;
                        if (i == 0) { minX = bounds.x; minY = bounds.y; maxX = bounds.right; maxY = bounds.bottom; }
                        else {
                            minX = Math.min(minX, bounds.x); minY = Math.min(minY, bounds.y); maxX = Math.max(maxX, bounds.right); maxY = Math.max(maxY, bounds.bottom);
                        }
                    }
                    this._updateAnchor(layout, { x: minX, y: minY, right: maxX, bottom: maxY });
                    for (i = 0; i < layout._firstLevelNodes.length; i++) {
                        this._updateNodes(layout, layout._firstLevelNodes[i], 0);
                    }
                    for (i = 0; i < layout._firstLevelNodes.length; i++) {
                        this._updateConnectors(layout, layout._firstLevelNodes[i], 1);
                    }
                }
            }
        },

        _updateTree: function (layout, x, y, shape, level, prev, dontupdate) {
            var dimensions, info, lev, obj, hasChild;
            dimensions = this._getDimensions(layout, shape, x, y, level);
            info = layout._graphNodes[shape.name];
            //Set maximum level of layout
            layout.maxLevel = Math.max(layout.maxLevel, level);
            lev = level;
            hasChild = this._hasChild(layout, shape);
            if (!hasChild && !info.tree.assistants.length)
                //update leaf nodes
                return this._updateLeafNode(layout, shape, prev, dimensions, level, dontupdate);
            else {
                var treeBounds, shapeBounds, levelBounds, d, asstBounds, space;
                var bottom = dimensions.y + dimensions.height + layout.verticalSpacing;
                if (info.tree.assistants.length) {
                    //Vertically place assistants
                    obj = this._setDepthSpaceForAssistants(layout, shape, bottom, dimensions.height, level, layout.verticalSpacing);
                    lev = obj.level;
                    bottom = obj.bottom;
                }
                if (!info.tree.assistants.length && info.tree.orientation != "horizontal") bottom = dimensions.y + dimensions.height + layout.verticalSpacing / 2;

                if (info.tree.orientation == "horizontal")
                    treeBounds = this._updateHorizontalTree(layout, shape, prev, dimensions.x, bottom, lev);
                else
                    treeBounds = this._updateVerticalTree(layout, shape, dimensions.x, bottom, lev, dontupdate);

                info.y = dimensions.y;
                x = info.mid;
                if (info.tree.assistants.length) {
                    //Set breadth space for assistants
                    space = x !== undefined ? x : dimensions.x;
                    asstBounds = this._setBreadthSpaceForAssistants(layout, shape, dimensions, space, bottom, level);
                    if (!hasChild) {
                        levelBounds = treeBounds = asstBounds;
                        x = (levelBounds.x + levelBounds.right) / 2 - dimensions.width / 2;
                        treeBounds = levelBounds;
                    }
                    d = asstBounds ? asstBounds.canMoveBy : undefined;
                }
                info.x = x;
                shapeBounds = { x: x, y: dimensions.y, right: x + dimensions.width, bottom: dimensions.y + dimensions.height };
                this._translateSubTree(layout, shape, shapeBounds, treeBounds, dimensions, level, d, prev != undefined, dontupdate);
                return treeBounds;
            }
        },

        _updateLeafNode: function (layout, shape, prev, dimensions, level, dontupdate) {
            var info, bounds;
            info = layout._graphNodes[shape.name];
            info.x = dimensions.x;
            info.y = dimensions.y;
            info.maxLevel = Math.max(level, info.maxLevel || 0);
            bounds = { x: dimensions.x, y: dimensions.y, right: dimensions.x + dimensions.width, bottom: dimensions.y + dimensions.height };
            info.maxLevel = Math.max(info.maxLevel || 0, level);
            this._translateSubTree(layout, shape, bounds, bounds, dimensions, level, undefined, prev != undefined, dontupdate);
            return { x: info.x, y: info.y, right: info.x + dimensions.width, bottom: info.y + dimensions.height };
        },

        _translateSubTree: function (layout, shape, shapeBounds, treeBounds, dimensions, level, assistDiff, translate, dontupdate) {
            var info = layout._graphNodes[shape.name];
            var firstChild = layout.diagram.nameTable[info.tree.children[0]];
            var firstChildInfo = firstChild ? layout._graphNodes[firstChild.name] : null;
            var hasChild = this._hasChild(layout, shape);
            var intersect = this._findIntersectingLevels(layout, shapeBounds, level);
            var treeIntersect = this._findIntersectingLevels(layout, treeBounds, level);
            var levelBounds = [], diff;
            if (intersect.length) {
                info.intersect = intersect;
                this._spaceLeftFromPrevSubTree(layout, shape, shapeBounds);
                info.canMoveBy = info.diff;
                info.fchildList = [];
                if (assistDiff !== undefined)
                    info.canMoveBy = Math.min(assistDiff, info.canMoveBy);
                if (firstChild && firstChildInfo.canMoveBy !== undefined) {
                    info.canMoveBy = Math.min(info.canMoveBy, firstChildInfo.canMoveBy);
                    if (firstChildInfo.fchildList) info.fchildList = firstChildInfo.fchildList;
                }
                info.fchildList[level] = shape.name;
                if (translate) {
                    info.x -= info.canMoveBy;
                    info.subTreeTranslation -= info.canMoveBy;
                    if (hasChild) {
                        this._shiftSubordinates(layout, treeIntersect, info.canMoveBy);
                        treeBounds.x = Math.min(treeBounds.x, info.x);
                        treeBounds.right = Math.max(treeBounds.right, info.x + dimensions.width);
                        treeBounds.bottom = Math.max(treeBounds.bottom, info.y + dimensions.height);
                        treeBounds.x -= info.canMoveBy;
                        treeBounds.right -= info.canMoveBy;
                    }
                }
            }
            else {
                if (hasChild) {
                    treeBounds.x = Math.min(treeBounds.x, shapeBounds.x);
                    treeBounds.right = Math.max(treeBounds.right, shapeBounds.x + dimensions.width);
                    treeBounds.bottom = Math.max(treeBounds.bottom, info.y + dimensions.height);
                }
            }
            if (!dontupdate) {
                shapeBounds = { x: info.x, y: dimensions.y, right: info.x + dimensions.width, bottom: dimensions.y + dimensions.height };
                levelBounds.push({ rBounds: shapeBounds });
                this._updateRearBounds(layout, shape, levelBounds, level);
            }
        },

        _updateRearBounds: function (layout, shape, levelBounds, level, intersect) {
            var bnds, index, intersect, firstLevel, lastLevel, isLastLeaf = true;
            if (shape) {
                var info = layout._graphNodes[shape.name];
                intersect = info.intersect;
                isLastLeaf = !info.tree.children.length && !info.tree.assistants.length;
            }
            firstLevel = levelBounds[0].rBounds;
            lastLevel = levelBounds[levelBounds.length - 1].rBounds;
            if (intersect && intersect.length) {
                bnds = layout.levels[intersect[0]].rBounds;
                if (bnds.y < firstLevel.y) {
                    bnds.bottom = firstLevel.y;
                    levelBounds.splice(0, 0, { rBounds: bnds });
                }
                bnds = layout.levels[intersect[intersect.length - 1]].rBounds;
                if (isLastLeaf && bnds.bottom > lastLevel.bottom) {
                    bnds.y = lastLevel.bottom;
                    levelBounds.push({ rBounds: bnds });
                }
                index = intersect[0];
                for (var i = levelBounds.length - 1; i >= 0; i--)
                    layout.levels.splice(index, 0, levelBounds[i]);
                index += levelBounds.length;
                layout.levels.splice(index, intersect.length);
            }
            else {
                index = this._findLevel(layout, levelBounds[levelBounds.length - 1].rBounds, level);
                for (var i = levelBounds.length - 1; i >= 0; i--)
                    layout.levels.splice(index, 0, levelBounds[i]);
            }
        },

        _shiftSubordinates: function (layout, intersect, diff) {
            var i;
            //Shift the sublevels by the distance diff
            if (diff != 0) {
                for (i = 0; i < intersect.length; i++) {
                    if (layout.levels[intersect[i]].rBounds) {
                        layout.levels[intersect[i]].rBounds.x -= diff;
                        layout.levels[intersect[i]].rBounds.right -= diff;
                    }
                }
            }
        },

        _setDepthSpaceForAssistants: function (layout, shape, bottom, height, lev, verticalSpacing) {
            var info = layout._graphNodes[shape.name];
            var asst, asstHeight, i;
            var max = bottom;
            //Vertically place the assistants as alternate layout(alternatively at both right and left sides of parent)
            for (i = 0; i < info.tree.assistants.length; i++) {
                asst = layout._graphNodes[info.tree.assistants[i]];
                if (asst) {
                    asst.tree.children = asst.tree.assistants = [];
                    asst.y = bottom;
                    asstHeight = layout.diagram.nameTable[info.tree.assistants[i]].height;
                    if (layout.orientation == "lefttoright" || layout.orientation == "righttoleft")
                        asstHeight = layout.diagram.nameTable[info.tree.assistants[i]].width;
                    max = bottom + asstHeight + verticalSpacing / 2;
                    layout.maxLevel = lev + 1;
                    if (i % 2 == 1 && i != info.tree.assistants.length - 1) {
                        bottom = max;
                        lev++;
                    }
                }
            }
            return { level: layout.maxLevel, bottom: bottom + asstHeight + verticalSpacing };
        },

        _setBreadthSpaceForAssistants: function (layout, shape, dimensions, space, bottom, level) {
            var asst, asstWidth, prevBounds, bounds;
            var info = layout._graphNodes[shape.name];
            var max = bottom;
            var lev = level;
            var left, diff, levelBounds, intersect, i;
            for (i = 0; i < info.tree.assistants.length; i++) {
                asst = layout._graphNodes[info.tree.assistants[i]];
                //Arrange assistants at both left and right sides of parent(like alternate layout)
                //Check - By default, distance to be left between parent and child nodes is assumed as 20. It can be modified/customized later.
                if (asst) {
                    asstWidth = layout.diagram.nameTable[info.tree.assistants[i]].width;
                    if (layout.orientation == "lefttoright" || layout.orientation == "righttoleft")
                        asstWidth = layout.diagram.nameTable[info.tree.assistants[i]].height;
                    if (i % 2 == 0) left = space + dimensions.width / 2 - 20 - asstWidth;
                    else left = space + dimensions.width / 2 + 20;
                    //Check - What will happen if update leaf node is called? Since assistants don't have children
                    bounds = this._updateTree(layout, left, asst.y, layout.diagram.nameTable[info.tree.assistants[i]], lev + 1);
                    if (!this._hasChild(layout, shape))
                        if (i == 0) levelBounds = bounds;
                        else this._uniteRects(levelBounds, bounds);
                    if (i % 2 == 0 && asst.prevBounds) {
                        if (diff === undefined) diff = asst.canMoveBy;
                        else diff = Math.min(diff, asst.canMoveBy);
                    }
                    if (i % 2 == 1 || i == info.tree.assistants.length - 1) {
                        intersect = this._findIntersectingLevels(layout, bounds, lev + 1);
                        //Update rightmost positions of known layout levels
                        this._updateRearBounds(layout, null, [{ rBounds: bounds }], lev + 1, intersect);
                        lev++;
                    }
                }
            }
            if (levelBounds)
                levelBounds.canMoveBy = diff;
            return levelBounds;
        },

        _updateHorizontalTree: function (layout, shape, prev, x, y, level) {
            var child, childInfo, childBounds, childWidth, childHeight, firstChildInfo;
            var prevBounds, bounds, actBounds, maxLevel, translateSibilingsBy;
            //Get dimensions with respect to layout orientations
            var dimensions = this._getDimensions(layout, shape, x, y, level);
            var info = layout._graphNodes[shape.name];
            var side = info.tree.type;
            var lev = level;
            var right = x;
            var bottom = y;
            var width, height;
            var prevLayoutLevels = layout.levels.slice(0, layout.levels.length);
            if (this._hasChild(layout, shape)) {
                var h = layout.orientation == "lefttoright" || layout.orientation == "righttoleft" ? true : false;
                for (var i = 0; i < info.tree.children.length; i++) {
                    child = layout.diagram.nameTable[info.tree.children[i]];
                    width = child.width ? child.width : child._width;
                    height = child.height ? child.height : child._height;
                    childWidth = h ? height : width;
                    childHeight = h ? width : height;
                    prevBounds = layout.levels[lev + 1] ? layout.levels[lev + 1].rBounds : null;
                    //Update sub tree
                    childBounds = this._updateTree(layout, right, bottom, child, lev + 1, layout.diagram.nameTable[info.tree.children[i - 1]]);
                    childInfo = layout._graphNodes[child.name];
                    actBounds = { x: childInfo.x, y: childInfo.y, right: childInfo.x + childWidth, bottom: childInfo.y + childHeight };
                    if (i == 0) {
                        //Compare with previous(right most) subtree
                        bounds = { x: Math.min(childInfo.x, childBounds.x), y: Math.min(childInfo.y, childBounds.y), right: childBounds.right, bottom: childBounds.bottom };
                        firstChildInfo = childInfo;
                    }
                        //Compare with previous subtree if level of the child is greater than the level of previous sub tree
                        //Check - what will happen if level of second child is greater than current child
                    else if (firstChildInfo && firstChildInfo.canMoveBy != undefined && childInfo.fchildList && childInfo.fchildList.length - 1 > maxLevel) {
                        var subChildInfo = layout._graphNodes[childInfo.fchildList[maxLevel]];
                        var diff;
                        while (maxLevel < childInfo.fchildList.length - 1) {
                            if (subChildInfo.canMoveBy && prevLayoutLevels.indexOf(subChildInfo.prevBounds) < 0) {
                                subChildInfo = layout._graphNodes[childInfo.fchildList[maxLevel + 1]];
                                maxLevel++;
                            }
                            else break;
                            diff = subChildInfo.canMoveBy - childInfo.canMoveBy;
                        }
                        if (diff != undefined && diff <= firstChildInfo.canMoveBy) {
                            translateSibilingsBy = translateSibilingsBy ? Math.min(translateSibilingsBy, diff) : diff;
                            if (prevLayoutLevels.indexOf(subChildInfo.prevBounds) >= 0)
                                this._translateSiblings(layout, shape, child);
                        }
                    }
                    maxLevel = maxLevel ? Math.max(childInfo.maxLevel, maxLevel) : childInfo.maxLevel;
                    this._uniteRects(bounds, childBounds);
                    info.maxLevel = Math.max(info.maxLevel || 0, childInfo.maxLevel || 0);
                    right = childBounds.right + layout.horizontalSpacing;
                }
                if (!isNaN(translateSibilingsBy)) firstChildInfo.canMoveBy = translateSibilingsBy;
                info.mid = (firstChildInfo.x + actBounds.right) / 2 - dimensions.width / 2;
                //Set parent based on the chart type
                if (side == "left") info.mid = actBounds.right - dimensions.width; else if (side == "right") info.mid = dimensions.x;
            }
            return bounds;
        },

        _updateVerticalTree: function (layout, shape, x, y, level, dontupdate) {
            //declare local variables
            var child, childInfo, childBounds, childWidth, childHeight, prevBounds, bounds, actBounds, oddBounds;
            var dimensions, info, firstChild, h, factor, right, bottom, lev, i;
            var diff, type, canMoveBy, levels = [], oddLevels = [], intersect;
            var width, height;
            //Get dimensions with respect to layout orientations
            dimensions = this._getDimensions(layout, shape, x, y, level);
            info = layout._graphNodes[shape.name];
            firstChild = layout.diagram.nameTable[info.tree.children[0]];
            h = layout.orientation == "lefttoright" || layout.orientation == "righttoleft" ? true : false;
            factor = info.tree.type == "left" || info.tree.type == "leftoffset" ? -1 : 0;
            right = x;
            bottom = y;
            lev = level;
            for (i = 0; i < info.tree.children.length; i++) {
                if (info.tree.type == "alternate") {
                    //arrange at both left and right
                    type = (i % 2 == 0 && info.tree.children.length > 2) ? "left" : "right";
                    factor = (i % 2 == 0 && info.tree.children.length > 2) ? -1 : 0;
                }
                right = x + this._findOffset(layout, shape, info, type);
                child = layout.diagram.nameTable[info.tree.children[i]];
                width = child.width ? child.width : child._width;
                height = child.height ? child.height : child._height;
                childWidth = h ? height : width;
                childHeight = h ? width : height;
                //Update sub tree
                childBounds = this._updateTree(layout, right + factor * childWidth, bottom, child, level + 1, undefined, true);
                childInfo = layout._graphNodes[child.name];
                actBounds = { x: childInfo.x, y: childInfo.y, right: childInfo.x + childWidth, bottom: childInfo.y + childHeight };
                if (i == 0) {
                    this._uniteRects(childBounds, actBounds);
                    bounds = childBounds;
                }
                else
                    this._uniteRects(bounds, childBounds);
                //Check and adjust the space left from previous subtree/sibling
                if (childInfo.prevBounds && !(info.tree.type == "alternate" && i % 2 == 1 && info.tree.children.length > 2)) {
                    canMoveBy = canMoveBy !== undefined ? Math.min(childInfo.canMoveBy, canMoveBy) : childInfo.canMoveBy;
                }
                //Max level of the subtree node
                info.maxLevel = Math.max(info.maxLevel || 0, childInfo.maxLevel || 0);
                if (!(info.tree.type == "alternate" && info.tree.children.length > 2 && i % 2 == 0)) {
                    if (info.tree.type == "alternate" && info.tree.children.length > 2)
                        //alternate - arrange children with even index(0,2,4,6,..) at the next level
                        bottom = Math.max(childBounds.bottom, prevBounds.bottom) + layout.verticalSpacing / 2;
                    else
                        // left/right - arrange next child at the nect level(bottom)
                        bottom = childBounds.bottom + layout.verticalSpacing / 2;

                    level = info.maxLevel;
                    levels.push({ rBounds: actBounds });
                    if (childInfo.levelBounds) levels = levels.concat(childInfo.levelBounds);
                }
                else {
                    if (i != 0) bottom = prevBounds.bottom + layout.verticalSpacing / 2;
                    oddLevels.push({ rBounds: actBounds });
                    if (childInfo.levelBounds) oddLevels = oddLevels.concat(childInfo.levelBounds);
                }
                prevBounds = actBounds;
            }
            //To set level bounds(right most position of levels)
            if (!dontupdate) {
                if (info.tree.type == "alternate" && info.tree.children.length > 2) {
                    oddBounds = { x: oddLevels[0].rBounds.x, y: oddLevels[0].rBounds.y, right: oddLevels[oddLevels.length - 1].rBounds.right, bottom: oddLevels[oddLevels.length - 1].rBounds.bottom };
                    intersect = this._findIntersectingLevels(layout, oddBounds, lev + 1);
                    this._updateRearBounds(layout, null, oddLevels, lev + 1, intersect);
                }
                intersect = this._findIntersectingLevels(layout, bounds, lev + 1);
                this._updateRearBounds(layout, null, levels, lev + 1, intersect);
            }
            else info.levelBounds = levels;

            if (!isNaN(canMoveBy))
                layout._graphNodes[firstChild.name].canMoveBy = canMoveBy;
            info.childBounds = bounds;
            info.mid = x;
            return bounds;
        },

        _findOffset: function (layout, shape, info, type) {
            var offset = 0;
            var space = (layout.orientation == "lefttoright" || layout.orientation == "righttoleft") ? shape.height : shape.width;
            var ch = type ? type : info.tree.type;
            offset = info.tree.offset || 20;
            if (info.tree.type == "alternate")
                if (offset >= layout.horizontalSpacing) offset = layout.horizontalSpacing / 2;
            switch (ch) {
                case "left":
                    offset = space / 2 - offset;
                    break;
                case "right":
                    offset = offset + space / 2;
                    break;
            }
            return offset;
        },

        _translateSiblings: function (layout, shape, child) {
            var i, childInfo, info, diff;
            info = layout._graphNodes[shape.name];
            diff = layout._graphNodes[child.name].diff - layout._graphNodes[child.name].canMoveBy;
            for (i = 0; i < info.tree.children.length; i++) {
                if (child.name == info.tree.children[i])
                    break;
                childInfo = layout._graphNodes[info.tree.children[i]];
                childInfo.x += diff;
            }
        },

        _uniteRects: function (rect1, rect2) {
            //Unite two rects
            rect1.x = Math.min(rect1.x, rect2.x);
            rect1.right = Math.max(rect1.right, rect2.right);
            rect1.bottom = Math.max(rect1.bottom, rect2.bottom);
        },

        _spaceLeftFromPrevSubTree: function (layout, shape, bounds) {
            var info = layout._graphNodes[shape.name];
            var dif, prevBounds, intersect, k;
            var space = layout.horizontalSpacing;
            //Find the minimum distance to move towards previous sub tree
            for (k = 0; k < info.intersect.length; k++) {
                prevBounds = layout.levels[info.intersect[k]].rBounds;
                dif = bounds.x - (prevBounds.right + space);
                if (info.diff == undefined || dif < info.diff) {
                    info.diff = dif;
                    info.prevBounds = layout.levels[info.intersect[k]];
                }
            }
        },

        _findIntersectingLevels: function (layout, bounds, level) {
            //intersecting with exact Level
            var bnds = { x: bounds.x, y: bounds.y, right: bounds.right, bottom: bounds.bottom };
            bnds.y -= layout.verticalSpacing / 2;
            bnds.bottom += layout.verticalSpacing / 2;
            var intersectingLevels = [];
            var rBounds;
            var l = level;
            rBounds = layout.levels[l] ? layout.levels[l].rBounds : null;
            //Performance - We can consider only the intersecting levels
            do {
                if (rBounds && ((bnds.y < rBounds.y && bnds.bottom > rBounds.y) || (bnds.y < rBounds.bottom && rBounds.bottom < bnds.bottom) ||
                    bnds.y > rBounds.y && bnds.bottom < rBounds.bottom || bnds.y < rBounds.y && bnds.bottom > rBounds.bottom)) {
                    intersectingLevels.splice(0, 0, l);
                }
                else if (rBounds && rBounds.bottom < bnds.y) break;
                l--;
                rBounds = layout.levels[l] ? layout.levels[l].rBounds : null;
            } while (l >= 0);

            l = level + 1;
            rBounds = layout.levels[l] ? layout.levels[l].rBounds : null;
            do {
                if (rBounds && ((bnds.y < rBounds.y && bnds.bottom > rBounds.y) || (bnds.y < rBounds.bottom && rBounds.bottom < bnds.bottom) ||
                    bnds.y > rBounds.y && bnds.bottom < rBounds.bottom || bnds.y < rBounds.y && bnds.bottom > rBounds.bottom)) {
                    intersectingLevels.push(l);
                }
                else if (rBounds && rBounds.y > bnds.bottom) break;
                l++;
                rBounds = layout.levels[l] ? layout.levels[l].rBounds : null;
            } while (l <= layout.levels.length);

            return intersectingLevels;
        },

        _findLevel: function (layout, bounds, level) {
            var bnds = bounds;
            var l = 0;
            var rBounds = layout.levels[l] ? layout.levels[l].rBounds : null;
            while (l < layout.levels.length) {
                if (rBounds && bnds.bottom < rBounds.y) {
                    return l;
                }
                else l++;
                rBounds = layout.levels[l] ? layout.levels[l].rBounds : null;
            }
            return l;
        },

        _updateEdges: function (layout, node, depth) {
            var layoutInfo = layout._graphNodes[node.name];
            //Update children collection
            if (node.outEdges && node.outEdges.length && node.isExpanded) {
                for (var j = 0; j < node.outEdges.length; j++) {
                    var edge = layout.diagram.nameTable[layout.diagram.nameTable[node.outEdges[j]].targetNode];
                    if (!edge.excludeFromLayout) {
                        layoutInfo.tree.children.push(edge.name);
                        if (edge.outEdges && edge.outEdges.length && edge.isExpanded) {
                            layoutInfo.tree.hasSubTree = true;
                        }
                        this._updateEdges(layout, edge, depth + 1);
                    }
                }
            }
            //set level info
            layoutInfo.tree.level = depth;

            //By default, orientation is horizontal for nested trees
            if (layoutInfo.tree.hasSubTree) layoutInfo.tree.orientation = "horizontal";

            //Customizing assistants and children collection
            //Performance-Instead of reading the method everytime, we can set once and can reuse that
            if (layout.getLayoutInfo && layout.type == "organizationalchart") {
                if (typeof layout.getLayoutInfo === "string") {
                    layout.getLayoutInfo = ej.util.getObject(layout.getLayoutInfo, window);
                }
                if ($.isFunction(layout.getLayoutInfo)) {
                    layout.getLayoutInfo(layout.diagram, node, layoutInfo.tree);
                }
            }
            if (layout.level && layoutInfo.tree.type != "alternate" && depth >= layout.level)
                layoutInfo.tree.hasSubTree = false;
        },

        _updateAnchor: function (layout, bounds) {
            var node, viewPort, fixedNode, width, height, mod;
            //Update anchor based on viewport
            viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(layout.diagram, true);
            if (layout.orientation == "toptobottom") {
                layout._anchorX = viewPort.width / 2 - (bounds.right - bounds.x) / 2 + layout.marginX - bounds.x;
                layout._anchorY = layout.marginY;
            }
            else if (layout.orientation == "bottomtotop") {
                layout._anchorX = viewPort.width / 2 - (bounds.right - bounds.x) / 2 + layout.marginX - bounds.x;
                layout._anchorY = viewPort.height - (bounds.x + layout.marginY);
            }
            else if (layout.orientation == "lefttoright") {
                layout._anchorX = layout.marginX;
                layout._anchorY = viewPort.height / 2 - (bounds.right - bounds.x) / 2 + layout.marginY - bounds.x;
            }
            else if (layout.orientation == "righttoleft") {
                layout._anchorX = viewPort.width - (bounds.y + layout.marginX);
                layout._anchorY = viewPort.height / 2 - (bounds.right - bounds.x) / 2 + layout.marginY - bounds.x;
            }

            //Anchor based on fixed nodes
            if (layout.fixedNode) {
                fixedNode = layout.diagram.nameTable[layout.fixedNode];
                width = fixedNode.width ? fixedNode.width : fixedNode._width;
                height = fixedNode.height ? fixedNode.height : fixedNode._height;
                layout._anchorX = fixedNode.offsetX
                layout._anchorY = fixedNode.offsetY;
                layout._anchorX += layout.orientation == "righttoleft" ? width * fixedNode.pivot.x : -width * fixedNode.pivot.x;
                layout._anchorY += layout.orientation == "bottomtotop" ? height * fixedNode.pivot.y : -height * fixedNode.pivot.y;
                node = fixedNode, mod = 0;
                while (node.inEdges.length) {
                    node = this._getParentNode(layout, node);
                    mod += layout._graphNodes[node.name].subTreeTranslation || 0;
                }
                if (layout.orientation == "lefttoright" || layout.orientation == "righttoleft") {
                    layout._anchorX -= layout._graphNodes[fixedNode.name].y;
                    layout._anchorY -= layout._graphNodes[fixedNode.name].x + mod;
                } else {
                    layout._anchorX -= layout._graphNodes[fixedNode.name].x + mod;
                    layout._anchorY -= layout._graphNodes[fixedNode.name].y;
                }
            }
        },

        _updateConnectors: function (layout, node, level) {
            var i, conn, target, info, direction, offsetDir, segments;
            var nodeWidth = 0, nodeHeight = 0, targetWidth = 0, targetHeight = 0, length = 0, offsetLen = 0;
            //Route out edges
            info = layout._graphNodes[node.name];
            if (node.outEdges.length) {
                for (i = 0; i < node.outEdges.length; i++) {
                    conn = layout.diagram.nameTable[node.outEdges[i]];
                    if (info.tree.children.indexOf(conn.targetNode) != -1) {
                        target = layout.diagram.nameTable[conn.targetNode];
                        var nwidth = node.width ? node.width : node._width;
                        var nheight = node.height ? node.height : node._height;
                        var twidth = target.width ? target.width : target._width;
                        var theight = target.height ? target.height : target._height;
                        if (layout.type == "organizationalchart") {
                            if (layout.orientation == "toptobottom" || layout.orientation == "bottomtotop") {
                                //Connector routing - vertical layouts
                                nodeHeight = layout.orientation == "toptobottom" ? nheight : -nheight;
                                targetHeight = layout.orientation == "toptobottom" ? -theight : theight;
                                direction = layout.orientation == "toptobottom" ? "bottom" : "top";
                                if (info.tree.assistants.length) {
                                    //Route inedge of child node, if the parent has assistants
                                    length = Math.abs((target.offsetY + targetHeight * (1 - target.pivot.y)) - (node.offsetY + nodeHeight * (1 - node.pivot.y))) - layout.verticalSpacing / 2;
                                    segments = [{ type: "orthogonal", length: length, direction: direction }];
                                }
                                else {
                                    //Route inedge of child node
                                    if (info.tree.orientation == "horizontal" && conn.segments.length == 1)
                                        layout.diagram._dock(conn, layout.diagram.nameTable);
                                    else {
                                        if (info.tree.orientation == "horizontal") {
                                            segments = [{ type: "orthogonal" }];
                                        }
                                        else {
                                            length = Math.abs(target.offsetY - (node.offsetY + nodeHeight * (1 - target.pivot.y)));
                                            segments = [{ type: "orthogonal", length: length, direction: direction }];
                                            if (info.tree.offset < 5) {
                                                offsetDir = info.tree.type == "left" ? "right" : "left";
                                                length = layout.verticalSpacing / 4;
                                                offsetLen = Math.abs(info.tree.offset) + layout.horizontalSpacing / 2;
                                                segments = [{ type: "orthogonal", length: length, direction: direction },
                                                 { type: "orthogonal", length: offsetLen, direction: offsetDir }];
                                            }
                                        }
                                    }
                                }
                            }
                            else {
                                //Connector routing - Horizontal layout orientation
                                nodeWidth = layout.orientation == "lefttoright" ? nwidth : -nwidth;
                                targetWidth = layout.orientation == "lefttoright" ? -twidth : twidth;
                                direction = layout.orientation == "lefttoright" ? "right" : "left";
                                if (info.tree.assistants.length) {
                                    //Route inedge of child node, if the parent has assistants
                                    length = Math.abs((target.offsetX + targetWidth * (1 - target.pivot.x)) - (node.offsetX + nodeWidth * (1 - node.pivot.x))) - layout.verticalSpacing / 2;
                                    segments = [{ type: "orthogonal", length: length, direction: direction }];
                                }
                                else {
                                    //Route inedge of child node
                                    if (info.tree.orientation == "horizontal" && conn.segments.length == 1)
                                        layout.diagram._dock(conn, layout.diagram.nameTable);
                                    else {
                                        if (info.tree.orientation == "horizontal") {
                                            segments = [{ type: "orthogonal" }];
                                        }
                                        else {
                                            length = Math.abs(target.offsetX - (node.offsetX + nodeWidth * (1 - node.pivot.x)));
                                            segments = [{ type: "orthogonal", length: length, direction: direction }];
                                            if (info.tree.offset < 5) {
                                                offsetDir = info.tree.type == "left" ? "bottom" : "top";
                                                length = layout.verticalSpacing / 4;
                                                offsetLen = Math.abs(info.tree.offset) + 10;
                                                segments = [{ type: "orthogonal", length: length, direction: direction },
                                                 { type: "orthogonal", length: offsetLen, direction: offsetDir }];
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (segments)
                        layout.diagram.updateConnector(conn.name, { segments: segments });
                    else layout.diagram._dock(conn, layout.diagram.nameTable);
                    if (layout.diagram._svg)
                        ej.datavisualization.Diagram.DiagramContext.update(conn, layout.diagram);
                    if (target && (target.isExpanded || this._hasChild(layout, target)))
                        this._updateConnectors(layout, target, level + 1);
                }
            }
            if (info.tree.assistants.length) {
                //In-Edge routing of assistant nodes
                for (i = 0; i < info.tree.assistants.length; i++) {
                    target = layout.diagram.nameTable[info.tree.assistants[i]];
                    conn = layout.diagram.nameTable[target.inEdges[0]];
                    length = Math.abs(target.offsetY - (node.offsetY + nodeHeight / 2));
                    segments = [{ type: "orthogonal", length: length, direction: direction }];
                    if (segments) { layout.diagram.updateConnector(conn.name, { segments: segments }); }
                    if (layout.diagram._svg)
                        ej.datavisualization.Diagram.DiagramContext.update(conn, layout.diagram);
                    if (target.isExpanded || this._hasChild(layout, target))
                        this._updateConnectors(layout, target, level + 1);
                }
            }
        },

        _updateNodes: function (layout, node, mod) {
            var child, i;
            var width = node.width ? node.width : node._width;
            var height = node.height ? node.height : node._height;
            var offsetX = node.offsetX;
            var offsetY = node.offsetY;
            if (!node.excludeFromLayout) {
                offsetX = layout._anchorX;
                offsetY = layout._anchorY;
                //Performance - instead of checking conditions for every node, we can make the layout related conditions once and we can reuse them
                if (layout.orientation == "lefttoright") {
                    offsetX += layout._graphNodes[node.name].y + width / 2;
                    offsetY += layout._graphNodes[node.name].x + mod + height / 2;
                }
                else if (layout.orientation == "righttoleft") {
                    offsetX -= layout._graphNodes[node.name].y + width / 2;
                    offsetY += layout._graphNodes[node.name].x + mod + height / 2;
                }
                else if (layout.orientation == "toptobottom") {
                    offsetX += layout._graphNodes[node.name].x + mod + width / 2;
                    offsetY += layout._graphNodes[node.name].y + height / 2;
                }
                else {
                    offsetX += layout._graphNodes[node.name].x + mod + width / 2;
                    offsetY -= layout._graphNodes[node.name].y + height / 2;
                }
                layout.diagram._translate(node, offsetX - (node.offsetX || 0), offsetY - (node.offsetY || 0), layout.diagram.nameTable);
            }
            if (this._hasChild(layout, node)) {
                for (i = 0; i < layout._graphNodes[node.name].tree.children.length; i++) {
                    child = layout.diagram.nameTable[layout._graphNodes[node.name].tree.children[i]];
                    this._updateNodes(layout, child, mod + (layout._graphNodes[node.name].subTreeTranslation || 0));
                }
            }
            if (layout._graphNodes[node.name].tree.assistants.length) {
                for (i = 0; i < layout._graphNodes[node.name].tree.assistants.length; i++) {
                    child = layout.diagram.nameTable[layout._graphNodes[node.name].tree.assistants[i]];
                    this._updateNodes(layout, child, mod + (layout._graphNodes[node.name].subTreeTranslation || 0));
                }
            }
            layout.diagram._updateQuad(layout.diagram.nameTable[node.name]);
            if (layout.diagram._svg)
                ej.datavisualization.Diagram.DiagramContext.update(node, layout.diagram);
        },

        _getDimensions: function (layout, shape, x, y, level) {
            var width = shape.width ? shape.width : shape._width;
            var height = shape.height ? shape.height : shape._height;
            if (layout.orientation == "lefttoright" || layout.orientation == "righttoleft") {
                if (!level) {
                    var temp = x;
                    x = y;
                    y = temp;
                }
                height = shape.width ? shape.width : shape._width;
                width = shape.height ? shape.height : shape._height;
            }
            return { x: x, y: y, width: width, height: height };
        },

        _getParentNode: function (layout, node) {
            //Return the first parent node
            if (node.inEdges && node.inEdges.length) {
                return layout.diagram.nameTable[layout.diagram.nameTable[node.inEdges[0]].sourceNode];
            }
        },

        _hasChild: function (layout, shape) {
            //Check whether the node has children
            var shape = layout._graphNodes[shape.name];
            return shape.tree.children && shape.tree.children.length;
        },

        _setUpLayoutInfo: function (layout, item) {
            //Initialize graph node info
            var info = {};
            info.subTreeTranslation = 0;
            if (layout.type == "organizationalchart")
                info.tree = { orientation: "vertical", type: "alternate", offset: 20, enableRouting: true };
            else if (layout.type == "hierarchicaltree")
                info.tree = { orientation: "horizontal", type: "center", enableRouting: true };
            info.tree.children = [];
            info.tree.assistants = [];
            info.tree.level = 0;
            return info;
        }

    };
    //end region

    //region Radialtree layout
    ej.datavisualization.Diagram.RadialTreeLayout = {
        doLayout: function (layout) {
            var diagram, nodes, node;
            diagram = layout.diagram;
            nodes = diagram.nodes();
            for (var i = 0; i < nodes.length; i++) {
                if (!nodes[i].excludeFromLayout) {
                    layout._graphNodes[nodes[i].name] = this._setUpLayoutInfo(layout, nodes[i]);
                    if (!nodes[i].inEdges || !nodes[i].inEdges.length) { layout._firstLevelNodes.push(nodes[i]); }
                }
            }
            if (layout.rootNode && diagram.nameTable[layout.rootNode])
                layout._centerNode = diagram.nameTable[layout.rootNode];
            else if (layout._firstLevelNodes.length) {
                layout._centerNode = layout._firstLevelNodes[0];
                diagram.model.layout.rootNode = layout._centerNode.name;
            }
            if (layout._centerNode) {
                this._updateEdges(layout, layout._centerNode, 0);
                this._depthFirstAllignment(layout, layout._centerNode, 0, 0);
                this._populateLevels(layout);
                this._transformToCircleLayout(layout);
                this._updateAnchor(layout);
                this._updateNodes(layout, layout._centerNode)
            }
        },

        _updateEdges: function (layout, node, depth) {
            var nodeInfo = layout._graphNodes[node.name];
            layout.layoutNodes.push(nodeInfo);
            nodeInfo.level = depth;
            nodeInfo.visited = true;
            layout.maxLevel = Math.max(layout.maxLevel, depth);
            for (var j = 0; j < node.outEdges.length; j++) {
                var edge = layout.diagram.nameTable[layout.diagram.nameTable[node.outEdges[j]].targetNode];
                if (!edge.excludeFromLayout && !edge.visited) {
                    nodeInfo.children.push(edge);
                    this._updateEdges(layout, edge, depth + 1);
                }
            }
        },

        _depthFirstAllignment: function (layout, node, x, y) {
            if (node) {
                var newValue;
                var nodeInfo = layout._graphNodes[node.name];
                if (nodeInfo.children.length) {
                    y += 300;
                    for (var i = 0; i < nodeInfo.children.length; i++) {
                        newValue = this._depthFirstAllignment(layout, nodeInfo.children[i], x, y);
                        x = newValue.x; y = newValue.y;
                    }
                    nodeInfo.children = nodeInfo.children.sort(function (obj1, obj2) { return layout._graphNodes[obj1.name].x - layout._graphNodes[obj2.name].x; });
                    var min = layout._graphNodes[nodeInfo.children[0].name].min;
                    var max = layout._graphNodes[nodeInfo.children[nodeInfo.children.length - 1].name].max;
                    nodeInfo.x = min + (max - min) / 2;
                    x = max + layout.horizontalSpacing;
                    nodeInfo.segmentOffset = max + layout.horizontalSpacing;
                    nodeInfo.x -= nodeInfo.width / 2;
                    nodeInfo.y -= nodeInfo.height / 2;
                    nodeInfo.min = min;
                    nodeInfo.max = max;
                    if (nodeInfo.x < min && nodeInfo.visited) {
                        nodeInfo.x = min;
                        x = nodeInfo.x + nodeInfo.width / 2 - (max - min) / 2;
                        nodeInfo.visited = false;
                        for (var i = 0; i < nodeInfo.children.length; i++) {
                            newValue = this._depthFirstAllignment(layout, nodeInfo.children[i], x, y);
                        }
                        nodeInfo.visited = true;
                        x = nodeInfo.x + nodeInfo.width + layout.horizontalSpacing;
                    }
                    max = layout._graphNodes[nodeInfo.children[nodeInfo.children.length - 1].name].segmentOffset;
                    x = x < max ? max : x;
                    y -= 300;
                    nodeInfo.y = y;
                }
                else {
                    nodeInfo.x = x;
                    nodeInfo.y = y;
                    nodeInfo.min = x;
                    nodeInfo.max = x + nodeInfo.width;
                    x += nodeInfo.width + layout.horizontalSpacing;
                }
                return { x: x, y: y };
            }
        },

        _populateLevels: function (layout) {
            var stages = [];
            var min = Math.min.apply(Math, layout.layoutNodes.map(function (nodeInfo) { return nodeInfo.x; }));
            var max = Math.max.apply(Math, layout.layoutNodes.map(function (nodeInfo) { return nodeInfo.x + nodeInfo.width + layout.horizontalSpacing; }));
            var full = max - min;
            layout.levels = [];
            for (var i = 0; i <= layout.maxLevel; i++) {
                stages = layout.layoutNodes.filter(function (nodeInfo) { if (nodeInfo.level == i) return nodeInfo; });
                var newlevel = {};
                stages = stages.sort(function (nodeInfo1, nodeInfo2) { return nodeInfo1.x - nodeInfo2.x; });
                newlevel.min = stages[0].x;
                newlevel.max = stages[stages.length - 1].x + stages[stages.length - 1].width + layout.horizontalSpacing;
                newlevel._actualCircumference = 0;
                newlevel.height = 0;
                for (var k = 0; k < stages.length; k++) {
                    if (stages[k].height > newlevel.height) newlevel.height = stages[k].height;
                    newlevel._actualCircumference += Math.max(stages[k].width, stages[k].height);
                    if (k != stages.length - 1) newlevel._actualCircumference += layout.horizontalSpacing;
                }
                newlevel.circumference = newlevel.max - newlevel.min;
                if (newlevel._actualCircumference < newlevel.circumference) newlevel.circumference = (newlevel.circumference + newlevel._actualCircumference) / 2;
                newlevel.radius = newlevel.circumference / (2 * Math.PI) + newlevel.height;
                newlevel.nodes = [];
                if (i > 1) {
                    if (layout.levels[i - 1].radius + layout.levels[i - 1].height >= newlevel.radius)
                        newlevel.radius = layout.levels[i - 1].radius + layout.levels[i - 1].height;
                }
                for (var j = 0; j < stages.length; j++) {
                    stages[j].ratio = Math.abs(stages[j].x + stages[j].width / 2 - min) / full;
                    newlevel.nodes.push(stages[j]);
                }
                layout.levels.push(newlevel);
            }
        },

        _transformToCircleLayout: function (layout) {
            var root = layout._graphNodes[layout._centerNode.name];
            root.x = 0;
            root.y = 0;
            //root.x = -root.width;
            //root.y = -root.height;
            for (var i = 1; i < layout.levels.length; i++) {
                for (var j = 0; j < layout.levels[i].nodes.length; j++) {
                    var nodeInfo = layout.levels[i].nodes[j];
                    nodeInfo.x = Math.cos(nodeInfo.ratio * 360 * Math.PI / 180) * (layout.levels[i].radius + layout.verticalSpacing * i);
                    nodeInfo.y = Math.sin(nodeInfo.ratio * 360 * Math.PI / 180) * (layout.levels[i].radius + layout.verticalSpacing * i);
                    //nodeInfo.x -= nodeInfo.width / 2;
                    //nodeInfo.y -= nodeInfo.height / 2;
                    layout._anchorX = Math.min(layout._anchorX, nodeInfo.x);
                    layout._anchorY = Math.min(layout._anchorY, nodeInfo.y);
                }
            }
        },

        _updateAnchor: function (layout) {
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(layout.diagram);
            layout._anchorX = viewPort.width / 2;
            layout._anchorY = viewPort.height / 2;
        },

        _updateNodes: function (layout, node) {
            var nodeInfo = layout._graphNodes[node.name];
            var offsetX = nodeInfo.x + layout._anchorX;
            var offsetY = nodeInfo.y + layout._anchorY;
            layout.diagram._translate(node, offsetX - node.offsetX || 0, offsetY - node.offsetY || 0, layout.diagram.nameTable);
            for (var i = 0; i < nodeInfo.children.length; i++) {
                var childInfo = nodeInfo.children[i];
                this._updateNodes(layout, layout.diagram.nameTable[childInfo.name]);
            }
            if (layout.diagram._svg)
                ej.datavisualization.Diagram.DiagramContext.update(node, layout.diagram);
        },

        _isValidLayout: function (layout) {
            if (layout._centerNode) {
                var maxParents = Math.max.apply(Math, layout.diagram.nodes().map(function (obj) { return obj.inEdges.length; }));
                if (maxParents < 2) return true;
            }
            return false;
        },

        _setUpLayoutInfo: function (layout, item) {
            var info = {};
            info.name = item.name;
            info.x = 0;
            info.y = 0;
            info.min = 0;
            info.max = 0;
            info.width = item.width ? item.width : item._width;
            info.height = item.height ? item.height : item._height;
            info.children = [];
            info.level = 0;
            info.ratio = 0;
            info.visited = false;
            return info;
        },

    };
    //end region
})(jQuery, Syncfusion);;

(function ($, ej) {
    "use strict";

    //#region ZoomAndPan
    ej.datavisualization.Diagram.ZoomUtil = {
        zoomPan: function (diagram, currentZoom, deltaX, deltaY, focusPoint, isZoom) {
            if (ej.datavisualization.Diagram.Util.canZooming(diagram)) {
                var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(diagram, true);
                var oVal = {
                    zoom: diagram._getCurrentZoom(), horizontalOffset: diagram._hScrollOffset,
                    verticalOffset: diagram._vScrollOffset, viewPort: viewPort
                };
                var matrix = ej.Matrix.identity();
                ej.Matrix.scale(matrix, diagram._currZoom, diagram._currZoom);
                ej.Matrix.translate(matrix, -diagram._hScrollOffset, -diagram._vScrollOffset);
                var newScale = diagram._currZoom * currentZoom;
                if (newScale > 30) {
                    currentZoom = 30 / diagram._currZoom;
                }
                else if (newScale < 0.25) {
                    currentZoom = 0.25 / diagram._currZoom;
                }
                var dMatrix = ej.Matrix.identity();
                var pivot;
                if (focusPoint) {
                    pivot = ej.Matrix.transform(matrix, focusPoint);
                }
                else {
                    pivot = ej.datavisualization.Diagram.Point($(diagram.element).width() / 2, $(diagram.element).height() / 2);
                }
                ej.Matrix.scale(dMatrix, currentZoom, currentZoom, pivot.x, pivot.y);
                newScale = diagram._currZoom * currentZoom;
                ej.Matrix.translate(dMatrix, deltaX, deltaY);
                ej.Matrix.multiply(matrix, dMatrix);
                var point = ej.Matrix.transform(matrix, ej.datavisualization.Diagram.Point(0, 0));
                this._applyTransform(diagram, newScale, point, isZoom);
                var nVal = {
                    zoom: diagram._getCurrentZoom(), horizontalOffset: diagram._hScrollOffset,
                    verticalOffset: diagram._vScrollOffset, viewPort: viewPort
                };
                diagram._raiseEvent("scrollChange", { newValues: nVal, oldValues: oVal });
            }
        },
        _applyTransform: function (diagram, currentZoom, point, isZoom) {
            diagram._currZoom = currentZoom;
            var zoomX = point.x * -1;
            var zoomY = point.y * -1;
            if (!isZoom) {
                if (diagram._scrollLimit() !== ej.datavisualization.Diagram.ScrollLimit.Infinity) {
                    if (zoomX < diagram._hScrollbar.model.minimum && !(diagram._hScrollbar.model.minimum == 0 && -diagram._hScrollOffset > point.x))
                        zoomX = diagram._hScrollOffset;
                    else if (zoomX > diagram._hScrollbar.model.maximum) {
                        zoomX = diagram._hScrollbar.model.maximum;
                    }
                    if (zoomY < diagram._vScrollbar.model.minimum && !(diagram._vScrollbar.model.minimum == 0 && -diagram._vScrollOffset > point.y))
                        zoomY = diagram._vScrollOffset;
                    else if (zoomY > diagram._vScrollbar.model.maximum) {
                        zoomY = diagram._vScrollbar.model.maximum;
                    }
                }
            }
            diagram._updateScrollOffset(zoomX, zoomY, true, isZoom);
        }
    };
    //#endregion 

    //#region PageSettings
    ej.datavisualization.Diagram.PageUtil = {
        _createPageBreaks: function (diagram) {
            this._removePageBreaks(diagram);
            var attr = { id: diagram._canvas.id + "pagebreaks", "pointer-events": "none" };
            var g = ej.datavisualization.Diagram.SvgContext._initializePageBreaks(diagram._svg, attr, diagram._pageBackgroundLayer);
            this._updatePageBreaks(g, diagram);
        },
        _removePageBreaks: function (diagram) {
            var g = diagram._svg.getElementById(diagram._canvas.id + "pagebreaks");
            if (g) { ej.datavisualization.Diagram.SvgContext._removePageBreaks(diagram._svg, g, diagram._pageBackgroundLayer); }
        },
        _updatePageBreaks: function (g, diagram) {
            var bounds = diagram._getDigramBounds();
            var pageSettings = diagram.model.pageSettings;
            var left = bounds.x;
            var right = bounds.width + bounds.x;
            var top = bounds.y;
            var bottom = bounds.height + bounds.y;
            var zoom = diagram._currZoom;
            var pHeight = diagram._pageHeight();
            var pWidth = diagram._pageWidth();
            var pMargin = diagram._pageMargin();
            var columncount = parseInt(Math.ceil((right - left) / pWidth));
            var rowcount = parseInt(Math.ceil((bottom - top) / pHeight));
            left = -diagram._canvas.clientLeft + (pMargin + left) * zoom;
            top = -diagram._canvas.clientTop + (pMargin + top) * zoom;
            var width = (pWidth * columncount - pMargin * 2) * zoom;
            var height = (pHeight * rowcount - pMargin * 2) * zoom;
            ej.datavisualization.Diagram.SvgContext._renderVPageBreakLine(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(left + width, top), diagram._svg, g);
            ej.datavisualization.Diagram.SvgContext._renderHPageBreakLine(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(left, top + height), diagram._svg, g);
            for (var column = 1; column < columncount; column++) {
                ej.datavisualization.Diagram.SvgContext._renderVPageBreakLine(ej.datavisualization.Diagram.Point(left + (pWidth * column - pMargin) * zoom, top),
                    ej.datavisualization.Diagram.Point(left + (pWidth * column - pMargin) * zoom, top + height), diagram._svg, g);
            }
            for (var row = 1; row < rowcount; row++) {
                ej.datavisualization.Diagram.SvgContext._renderHPageBreakLine(ej.datavisualization.Diagram.Point(left, top + (pHeight * row - pMargin) * zoom),
                    ej.datavisualization.Diagram.Point(left + width, top + (pHeight * row - pMargin) * zoom), diagram._svg, g);
            }
            ej.datavisualization.Diagram.SvgContext._renderVPageBreakLine(ej.datavisualization.Diagram.Point(left + width, top), ej.datavisualization.Diagram.Point(left + width, top + height), diagram._svg, g);
            ej.datavisualization.Diagram.SvgContext._renderHPageBreakLine(ej.datavisualization.Diagram.Point(left, top + height), ej.datavisualization.Diagram.Point(left + width, top + height), diagram._svg, g);
        },
        _updatePageSize: function (diagram, updatePage) {
            var swap = false;
            if (diagram._pageOrientation() == ej.datavisualization.Diagram.PageOrientations.Landscape) {
                if (diagram._pageHeight() > diagram._pageWidth()) {
                    swap = true;
                }
            }
            else {
                if (diagram._pageWidth() > diagram._pageHeight()) {
                    swap = true;
                }
            }
            if (swap) {
                var temp = diagram._pageWidth();
                diagram._pageWidth(diagram._pageHeight());
                diagram._pageHeight(temp);
            }
            var bounds = diagram._getDigramBounds();
            var zoom = diagram._currZoom;
            var left = bounds.x * zoom;
            var top = bounds.y * zoom;
            var width = bounds.width * zoom;
            var height = bounds.height * zoom;
            var pageSettings = diagram.model.pageSettings;
            if (diagram._pageBackgroundLayer && diagram._pageBackgroundLayer.firstChild) {
                ej.datavisualization.Diagram.SvgContext._updatePageBakground(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(width, height), diagram._svg, diagram._pageBackgroundLayer, diagram._pageBorderColor(), diagram._pageBackgroundColor(), diagram._pageBorderWidth());
            }
            else {
                ej.datavisualization.Diagram.SvgContext._renderPageSettingsRect(ej.datavisualization.Diagram.Point(left, top), ej.datavisualization.Diagram.Point(width, height), diagram._svg, diagram._pageBackgroundLayer, diagram._pageBorderColor(), diagram._pageBackgroundColor(), diagram._pageBorderWidth());
            }
            diagram._svg.document.style.left = 0;
            diagram._svg.document.style.top = 0;
            var check = false;
            if (!diagram._pageWidth()) {
                if (!diagram._pageHeight()) {
                    check = true;
                }
            }
            if (pageSettings.pageWidth && diagram._pageHeight()) {
                if (diagram._showPageBreak()) {
                    this._createPageBreaks(diagram);
                }
                else {
                    this._removePageBreaks(diagram);
                }
            }
            if (!updatePage)
                ej.datavisualization.Diagram.ScrollUtil._setScrollContentSize(diagram);
        }
    };
    //#endregion

    //#region Scroller
    ej.datavisualization.Diagram.ScrollUtil = {
        _transform: function (diagram, hOffset, vOffset, canScale) {
            ej.datavisualization.Diagram.SvgContext.transformView(diagram, -hOffset, -vOffset);
            if (canScale) {
                ej.datavisualization.Diagram.SvgContext.scaleContent(diagram, diagram._currZoom);
            }
            ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram);
            ej.datavisualization.Diagram.SvgContext._updateBackground(hOffset, vOffset, diagram._currZoom, diagram);
            ej.datavisualization.Diagram.SvgContext._updateGrid(hOffset, vOffset, diagram._currZoom, diagram);
        },
        _viewPort: function (diagram, update) {
            if (diagram._viewPort && !update) return diagram._viewPort;
            var element = diagram.element[0];
            var bRect = diagram.element[0].getBoundingClientRect();
            var eWidth = bRect.width;
            var eHeight = bRect.height;
            var screenX = (window.screenX < 0) ? window.screenX * -1 : window.screenX;
            if (eWidth === 0) {
                eWidth = Math.floor(((window.innerWidth - screenX) - Math.floor(bRect.left)));
            }
            var screenY = (window.screenY < 0) ? window.screenY * -1 : window.screenY;
            if (eHeight === 0) {
                eHeight = Math.floor(((window.innerHeight - screenY) - Math.floor(bRect.top)));
            }
            return ej.datavisualization.Diagram.Size(eWidth, eHeight);
        },

        _union: function (rect, rect1) {
            if (rect.width <= 0) {
                rect = rect1;
                return rect;
            }
            if (rect.width > 0) {
                var num = Math.min(rect.x, rect1.x);
                var num2 = Math.min(rect.y, rect1.y);
                if (rect1.width === Infinity || rect.width === Infinity) {
                    rect.width = Infinity;
                }
                else {
                    var num3 = Math.max(rect.x + rect.width, rect1.x + rect1.width);
                    rect.width = Math.max(num3 - num, 0);
                }
                if (rect1.height === Infinity || rect.height === Infinity) {
                    rect.height = Infinity;
                }
                else {
                    var num4 = Math.max(rect.y + rect.height, rect1.y + rect1.height);
                    rect.height = Math.max(num4 - num2, 0);
                }
                rect.x = num;
                rect.y = num2;
                return rect;
            }
        },

        _setScrollContentSize: function (diagram) {
            var scale = diagram._currZoom;
            var viewPort = this._viewPort(diagram);
            viewPort = ej.datavisualization.Diagram.Rectangle(diagram._hScrollOffset, diagram._vScrollOffset, viewPort.width, viewPort.height);
            var scrollPadding = diagram.model.scrollSettings.padding;
            var left = diagram._spatialSearch.pageLeft;
            var right = diagram._spatialSearch.pageRight;
            var top = diagram._spatialSearch.pageTop;
            var bottom = diagram._spatialSearch.pageBottom;
            if (left > right) {
                left = right = 0;
            }
            if (top > bottom) {
                top = bottom = 0;
            }

            if (diagram.model.pageSettings) {
                var pageWidth = diagram._pageWidth();
                var pageHeight = diagram._pageHeight();

                if (pageWidth > 0 && pageHeight > 0) {
                    if (diagram._multiplePage()) {
                        left = Math.floor(left / pageWidth) * pageWidth;
                        top = Math.floor(top / pageHeight) * pageHeight;
                        right = Math.ceil(right / pageWidth) * pageWidth;
                        bottom = Math.ceil(bottom / pageHeight) * pageHeight;
                    }
                    else {
                        left = 0;
                        top = 0;
                        right = pageWidth;
                        bottom = pageHeight;
                    }
                }
                else {
                    left = left < 0 ? left : 0;
                    top = top < 0 ? top : 0;
                    right = right > viewPort.width ? right : viewPort.width;
                    bottom = bottom > viewPort.height ? bottom : viewPort.height;
                }
            }
            left = left * scale;
            top = top * scale;
            right = right * scale;
            bottom = bottom * scale;

            var diagramArea = ej.datavisualization.Diagram.Rectangle(0, 0, 0, 0);
            diagramArea = this._union(diagramArea, ej.datavisualization.Diagram.Rectangle(0, 0, viewPort.width, viewPort.height));
            diagramArea = this._union(diagramArea, ej.datavisualization.Diagram.Geometry.rect([{ x: left, y: top }, { x: right, y: bottom }]));
            diagramArea.x -= scrollPadding;
            diagramArea.y -= scrollPadding;
            diagramArea.width -= viewPort.width - 2 * scrollPadding;
            diagramArea.height -= viewPort.height - 2 * scrollPadding;

            if (diagramArea.x > diagram._hScrollOffset) {
                var diff = diagramArea.x - diagram._hScrollOffset;
                diagramArea.x -= diff;
                diagramArea.width += diff;
            }
            if (diagram._hScrollOffset > diagramArea.x + diagramArea.width) {
                diagramArea.width = diagram._hScrollOffset - diagramArea.x;
            }

            var minimumX = diagramArea.x;
            var maximumX = Math.max(0, diagramArea.x + diagramArea.width);

            if (diagramArea.y > diagram._vScrollOffset) {
                var diff = diagramArea.y - diagram._vScrollOffset;
                diagramArea.y -= diff;
                diagramArea.height += diff;
            }
            if (diagram._vScrollOffset > diagramArea.y + diagramArea.height) {
                diagramArea.height = diagram._vScrollOffset - diagramArea.y;
            }
            var minimumY = diagramArea.y;
            var maximumY = Math.max(0, diagramArea.y + diagramArea.height);

            diagramArea.width = diagramArea.width < viewPort.width ? viewPort.width : diagramArea.width;
            diagramArea.height = diagramArea.height < viewPort.height ? viewPort.height : diagramArea.height;

            if (diagram._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Limited) {
                var range = this._setMinMaxValues(diagram, minimumX, minimumY, maximumX, maximumY, viewPort, scale);
                minimumX = range.minX;
                minimumY = range.minY;
                maximumX = range.maxX;
                maximumY = range.maxY;
                bottom = Math.min(bottom, (diagram.model.pageSettings.scrollableArea.y + diagram.model.pageSettings.scrollableArea.height) * scale);
                right = Math.min(right, (diagram.model.pageSettings.scrollableArea.x + diagram.model.pageSettings.scrollableArea.width) * scale);
                top = Math.max(top, diagram.model.pageSettings.scrollableArea.y * scale);
                left = Math.max(left, diagram.model.pageSettings.scrollableArea.x * scale);
            }

            if (left >= diagram._hScrollOffset && right <= viewPort.width + diagram._hScrollOffset) {
                diagram._hScrollbar._remove();
                diagram._hScrollbar.model.width = 0;
                if (diagram._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Limited) diagram._hScrollbar.model.minimum = Math.min(0, minimumX);
                else if (diagram._spatialSearch.pageLeft == null) diagram._hScrollbar.model.minimum = 0;
                if (diagram._spatialSearch.pageRight == null) diagram._hScrollbar.model.maximum = 0;
            }
            else {
                $("#" + diagram._canvas.id + "_hScrollbar").ejScrollBar({ width: viewPort.width - 18, viewportSize: viewPort.width - 18, maximum: maximumX, minimum: minimumX });
                $("#" + diagram._canvas.id + "_hScrollbar").ejScrollBar("scroll", diagram._hScrollOffset);
                var hScroll = document.getElementById(diagram._canvas.id + "_hScrollbar");
                hScroll.style.marginTop = viewPort.height - 18 + "px";
            }

            if (top >= diagram._vScrollOffset && bottom <= viewPort.height + diagram._vScrollOffset) {
                diagram._vScrollbar._remove();
                diagram._vScrollbar.model.height = 0;
                if (diagram._scrollLimit() === ej.datavisualization.Diagram.ScrollLimit.Limited) diagram._vScrollbar.model.minimum = Math.min(0, minimumY);
                else if (diagram._spatialSearch.pageTop == null) diagram._vScrollbar.model.minimum = 0;
                if (diagram._spatialSearch.pageBottom == null) diagram._vScrollbar.model.maximum = 0;
            }
            else {
                $("#" + diagram._canvas.id + "_vScrollbar").ejScrollBar({ height: viewPort.height - 18, viewportSize: viewPort.height - 18, maximum: maximumY, minimum: minimumY });
                $("#" + diagram._canvas.id + "_vScrollbar").ejScrollBar("scroll", diagram._vScrollOffset);
            }
            ej.datavisualization.Diagram.SvgContext.setSize(diagram, diagramArea.width, diagramArea.height);
        },

        _setMinMaxValues: function (diagram, minX, minY, maxX, maxY, viewPort, scale) {
            var scrollableArea = diagram.model.pageSettings.scrollableArea;
            minX = Math.max(minX, scrollableArea.x * scale);
            minY = Math.max(minY, scrollableArea.y * scale);
            maxX = Math.min((scrollableArea.x + scrollableArea.width) * scale - viewPort.width, maxX);
            maxY = Math.min((scrollableArea.y + scrollableArea.height) * scale - viewPort.height, maxY);
            maxX = maxX > 0 ? maxX : 0;
            maxY = maxY > 0 ? maxY : 0;
            return { minX: minX, minY: minY, maxX: maxX, maxY: maxY };
        },

        _initScrollbar: function (diagram) {
            var canvas = diagram._canvas;
            var hScrollbar = document.getElementById(canvas.id + "_hScrollbar");
            var vScrollbar = document.getElementById(canvas.id + "_vScrollbar");
            var viewPort = this._viewPort(diagram);
            $(hScrollbar).ejScrollBar({
                orientation: ej.ScrollBar.Orientation.Horizontal,
                width: viewPort.width - 18,
                viewportSize: viewPort.width - 18,
                scroll: function (e) {
                    if (e.source != null) {
                        if (e.source == "thumb" || e.source == "button" || e.source == "key") {
                            if (e.scrollData != null) {
                                var oVal = {
                                    zoom: diagram._getCurrentZoom(), horizontalOffset: diagram._hScrollOffset,
                                    verticalOffset: diagram._vScrollOffset, viewPort: diagram._viewPort
                                };
                                diagram._updateScrollOffset(e.scrollLeft, diagram._vScrollOffset);
                                var nVal = {
                                    zoom: diagram._getCurrentZoom(), horizontalOffset: diagram._hScrollOffset,
                                    verticalOffset: diagram._vScrollOffset, viewPort: diagram._viewPort
                                };
                                diagram._raiseEvent("scrollChange", { newValues: nVal, oldValues: oVal });
                            }
                        }
                    }
                }
            });
            diagram._hScrollbar = $(hScrollbar).ejScrollBar("instance");

            $(vScrollbar).ejScrollBar({
                orientation: ej.ScrollBar.Orientation.Vertical,
                height: viewPort.height - 18,
                viewportSize: viewPort.height - 18,
                scroll: function (e) {
                    if (e.source != null) {
                        if (e.source == "thumb" || e.source == "button" || e.source == "key") {
                            if (e.scrollData != null) {
                                var oVal = {
                                    zoom: diagram._getCurrentZoom(), horizontalOffset: diagram._hScrollOffset,
                                    verticalOffset: diagram._vScrollOffset, viewPort: diagram._viewPort
                                };
                                diagram._updateScrollOffset(diagram._hScrollOffset, e.scrollTop);
                                var nVal = {
                                    zoom: diagram._getCurrentZoom(), horizontalOffset: diagram._hScrollOffset,
                                    verticalOffset: diagram._vScrollOffset, viewPort: diagram._viewPort
                                };
                                diagram._raiseEvent("scrollChange", { newValues: nVal, oldValues: oVal });
                            }
                        }
                    }
                }
            });
            diagram._vScrollbar = $(vScrollbar).ejScrollBar("instance");

        },
    };
    //#endregion

    //#region Snapping
    ej.datavisualization.Diagram.SnapUtil = {
        _canConsider: function (nameTable, selectedObject, target) {
            var consider = false;
            if (selectedObject != target) {
                if (selectedObject._type == "group" || selectedObject._type == "pseudoGroup") {
                    consider = (target.isSwimlane && !selectedObject.isSwimlane) ? false : !this._contains(nameTable, selectedObject, target);
                }
                else if ((target._type == "group" && target.type != "bpmn") || target.type == "pseudoGroup")
                    consider = (target.isSwimlane && !selectedObject.isSwimlane) ? false : !this._contains(nameTable, target, selectedObject);
                else if (!selectedObject.segments) {
                    consider = !(selectedObject == target);
                }
            }
            return consider;
        },
        _contains: function (nameTable, group, target) {
            var name = target.name;
            var node = null;
            if (group.children.indexOf(name) >= 0) {
                node = nameTable[name];
                if (node == target)
                    return true;
            }
            else if (group.children.indexOf(target) >= 0) {
                return true;
            }
            if (group._type == "group" || group._type == "pseudoGroup") {
                var contains = false;
                for (var i = 0; i < group.children.length; i++) {
                    if (this.diagram) {
                        var child = typeof group.children[i] == "string" ? nameTable[group.children[i]] : nameTable[group.children[i].name];
                        if (child) {
                            if (typeof child == "string") child = nameTable[child];
                            if (child._type == "group")
                                contains = this._contains(nameTable, child, target);
                            if (contains) return contains;
                        }
                    }
                }
                return contains;
            }
            //return this._contains(nameTable, node, child);
            return false;
        },
        _round: function (value, snapintervals, scale) {
            if (scale > 1) scale = Math.pow(2, Math.floor(Math.log(scale) / Math.log(2)));
            else scale = Math.pow(2, Math.ceil(Math.log(scale) / Math.log(2)));
            var cutoff = 0, i;
            for (i = 0; i < snapintervals.length; i++) {
                cutoff += snapintervals[i];
            }
            cutoff /= scale;
            var quotient = Math.floor(Math.abs(value) / cutoff);
            var bal = value % cutoff;
            var prev = quotient * cutoff;
            if (prev != value) {
                if (value >= 0) {
                    for (i = 0; i < snapintervals.length ; i++) {
                        if (bal <= snapintervals[i] / scale) {
                            return prev + (bal < (snapintervals[i] / (2 * scale)) ? 0 : snapintervals[i] / scale);
                        }
                        else {
                            prev += snapintervals[i] / scale;
                            bal -= snapintervals[i] / scale;
                        }
                    }
                }
                else {
                    prev = prev * -1;
                    for (i = snapintervals.length - 1; i >= 0; i--) {
                        if (Math.abs(bal) <= snapintervals[i] / scale) {
                            return prev - (Math.abs(bal) < (snapintervals[i] / (2 * scale)) ? 0 : snapintervals[i] / scale);
                        }
                        else {
                            prev -= snapintervals[i] / scale;
                            bal += snapintervals[i] / scale;
                        }
                    }
                }
            }
            return value;
        },
        _findNodes: function (spatialSearch, node, child, viewPort, nodesInView) {
            var nodes = [], quad, nd, bounds;
            var quads;
            quads = ej.datavisualization.Diagram.SpatialUtil.findQuads(spatialSearch, nodesInView ? viewPort : child);
            for (var i = 0; i < quads.length; i++) {
                quad = quads[i];
                if (quad.objects.length > 0) {
                    for (var j = 0; j < quad.objects.length; j++) {
                        nd = quad.objects[j];
                        if (!nd.segments && nd.visible) {
                            bounds = ej.datavisualization.Diagram.Util.bounds(nd);
                            if (nodes.indexOf(nd) == -1 && ej.datavisualization.Diagram.Geometry.intersectsRect(child, bounds))
                                nodes.push(nd);
                            if (nodesInView && nodesInView.indexOf(nd) && ej.datavisualization.Diagram.Geometry.intersectsRect(viewPort, bounds)) {
                                nodesInView.push(nd);
                            }
                        }
                    }
                }
            }
            return nodes;
        },
        _snapAngle: function (diagram, angle) {
            var snapAngle = diagram._snapAngle();
            var width = angle % snapAngle;
            if (width >= (snapAngle / 2)) {
                return angle + snapAngle - width;
            }
            else {
                return angle - width;
            }
        },
        _snapPoint: function (diagram, selectedObject, towardsLeft, towardsTop, del, endPoint, startPoint) {
            var snapSettings = diagram.model.snapSettings;
            var zoomFactor = diagram._currZoom;
            var offset = ej.datavisualization.Diagram.Point();
            var bounds = ej.datavisualization.Diagram.Util.bounds(selectedObject);
            var horizontallysnapped = { snapped: false, offset: 0 };
            var verticallysnapped = { snapped: false, offset: 0 };
            this.diagram = diagram;
            if (diagram._enableSnapToObject())
                this._snapObject(diagram, selectedObject, horizontallysnapped,
                    verticallysnapped, del, startPoint == endPoint);
            //original position
            var left = bounds.x + del.x;
            var top = bounds.y + del.y;
            var right = bounds.x + bounds.width + del.x;
            var bottom = bounds.y + bounds.height + del.y;
            //snapped positions
            var roundedRight = this._round(right, snapSettings.verticalGridLines.snapInterval, zoomFactor);
            var roundedLeft = this._round(left, snapSettings.verticalGridLines.snapInterval, zoomFactor);
            var roundedTop = this._round(top, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
            var roundedBottom = this._round(bottom, snapSettings.horizontalGridLines.snapInterval, zoomFactor);
            //currentposition
            var currentright = bounds.x + bounds.width;
            var currentbottom = bounds.y + bounds.height;
            if (!horizontallysnapped.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToVerticalLines) {
                    if (Math.abs(del.x) >= 1) {
                        if (towardsLeft) {
                            if (Math.abs(roundedRight - currentright) > Math.abs(roundedLeft - bounds.x)) {
                                offset.x += roundedLeft - bounds.x;
                            } else
                                offset.x += roundedRight - currentright;
                        } else {
                            if (Math.abs(roundedRight - currentright) < Math.abs(roundedLeft - bounds.x)) {
                                offset.x += roundedRight - currentright;
                            } else
                                offset.x += roundedLeft - bounds.x;
                        }
                    }
                } else {
                    offset.x = endPoint.x - startPoint.x;
                }
            } else {
                if (diagram._enableSnapToObject())
                    offset.x = horizontallysnapped.offset;
                else
                    offset.x = endPoint.x - startPoint.x;
            }
            if (!verticallysnapped.snapped) {
                if (snapSettings.snapConstraints & ej.datavisualization.Diagram.SnapConstraints.SnapToHorizontalLines) {
                    if (Math.abs(del.y) >= 1) {
                        if (towardsTop) {
                            if (Math.abs(roundedBottom - currentbottom) > Math.abs(roundedTop - bounds.y)) {
                                offset.y += roundedTop - bounds.y;
                            }
                            else
                                offset.y += roundedBottom - currentbottom;
                        }
                        else {
                            if (Math.abs(roundedBottom - currentbottom) < Math.abs(roundedTop - bounds.y)) {
                                offset.y += roundedBottom - currentbottom;
                            }
                            else
                                offset.y += roundedTop - bounds.y;
                        }
                    }
                }
                else
                    offset.y = endPoint.y - startPoint.y;
            }
            else
                offset.y = verticallysnapped.offset;

            return offset;
        },
        _snapObject: function (diagram, selectedObject, hSnap, vSnap, del, ended) {
            var lengthX = null, lengthY = null;
            var hTarget, vTarget;
            var objectsAtLeft = [], objectsAtRight = [], objectsAtTop = [], objectsAtBottom = [];
            var bounds = ej.datavisualization.Diagram.Util.bounds(selectedObject);
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            var snapObjDistance = diagram._snapObjectDistance();
            var viewPort = diagram._viewPort;
            var hIntersectRect = ej.datavisualization.Diagram.Rectangle(hoffset / scale, (bounds.y - 5), viewPort.width / scale,
                (bounds.height + 10));
            var vIntersectRect = ej.datavisualization.Diagram.Rectangle((bounds.x - 5), voffset / scale,
                (bounds.width + 10), viewPort.height / scale);
            viewPort = ej.datavisualization.Diagram.Rectangle(hoffset / scale, voffset / scale, viewPort.width / scale,
                viewPort.height / scale);
            var nodes = this._findNodes(diagram._spatialSearch, selectedObject, vIntersectRect, viewPort);
            var i, target, targetBounds;
            var nameTable = diagram.nameTable;
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (!target.segments && this._canConsider(nameTable, selectedObject, target)) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (targetBounds.y + targetBounds.height < bounds.y + del.y) {
                        objectsAtTop.push({ object: target, distance: Math.abs(bounds.y + del.y - targetBounds.y - targetBounds.height) });
                    }
                    else if (targetBounds.y > bounds.y + del.y + bounds.height) {
                        objectsAtBottom.push({ object: target, distance: Math.abs(bounds.y + del.y + bounds.height - targetBounds.y) });
                    }
                    if (lengthX == null || lengthX > Math.abs(targetBounds.y - bounds.y - del.y)) {
                        if (Math.abs(targetBounds.x + targetBounds.width / 2 - (bounds.x + bounds.width / 2 + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "centerX");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x - (bounds.x + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "left");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x + targetBounds.width - (bounds.x + bounds.width + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "right");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x + targetBounds.width - (bounds.x + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "leftRight");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        } else if (Math.abs(targetBounds.x - (bounds.x + bounds.width + del.x)) <= snapObjDistance) {
                            hTarget = this._createSnapObject(targetBounds, bounds, "rightLeft");
                            lengthX = Math.abs(targetBounds.y - bounds.y);
                        }
                    }
                }
            }
            nodes = this._findNodes(diagram._spatialSearch, selectedObject, hIntersectRect, viewPort);
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (!target.segments && this._canConsider(nameTable, selectedObject, target)) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (targetBounds.x + targetBounds.width < bounds.x + del.x) {
                        objectsAtLeft[objectsAtLeft.length] = { object: target, distance: Math.abs((bounds.x + del.x) - targetBounds.x - targetBounds.width) };
                    }
                    if (targetBounds.x > bounds.x + del.x + bounds.width) {
                        objectsAtRight[objectsAtRight.length] = { object: target, distance: Math.abs(bounds.x + del.x + bounds.width - targetBounds.x) };
                    }
                    if (lengthY == null || lengthY > Math.abs(targetBounds.x - bounds.x - del.x)) {
                        if (Math.abs(targetBounds.y + targetBounds.height / 2 - (bounds.y + bounds.height / 2 + del.y)) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "centerY");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y - bounds.y - del.y) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "top");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y + targetBounds.height - (bounds.y + bounds.height + del.y)) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "bottom");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y + targetBounds.height - bounds.y - del.y) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "topBottom");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        } else if (Math.abs(targetBounds.y - (bounds.y + bounds.height + del.y)) <= snapObjDistance) {
                            vTarget = this._createSnapObject(targetBounds, bounds, "bottomTop");
                            lengthY = Math.abs(targetBounds.x - bounds.x);
                        }
                    }
                }
            }
            var g = this._createGuidelines(diagram, hTarget, vTarget, hSnap, vSnap, ended);
            if (!hSnap.snapped) {
                this._createHSpacingLines(diagram, selectedObject, objectsAtLeft, objectsAtRight, hSnap, vSnap, ended, g, del, snapObjDistance);
            }
            if (!vSnap.snapped) {
                this._createVSpacingLines(diagram, selectedObject, objectsAtTop, objectsAtBottom, hSnap, vSnap, ended, g, del, snapObjDistance);
            }
        },
        _snapSize: function (diagram, hSnap, vSnap, delx, dely, selectedObject, ended) {
            var lengthX = null, lengthY = null;
            var hTarget;
            var vTarget;
            var bounds = ej.datavisualization.Diagram.Util.bounds(selectedObject);
            var nameTable = diagram.nameTable;
            var sameWidth = [];
            var sameHeight = [];
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            var snapObjDistance = diagram._snapObjectDistance();
            var viewPort = diagram._viewPort;
            var hintersectedrect = ej.datavisualization.Diagram.Rectangle(hoffset / scale, (bounds.y - 5) / scale,
                viewPort.width / scale, (bounds.height + 10) / scale);
            var vintersectedrect = ej.datavisualization.Diagram.Rectangle((bounds.x - 5) / scale, voffset / scale,
                (bounds.width + 10) / scale, viewPort.height / scale);
            viewPort = ej.datavisualization.Diagram.Rectangle(hoffset / scale, voffset / scale, viewPort.width / scale,
               viewPort.height / scale);
            var nodesInView = [];
            var nodes = this._findNodes(diagram._spatialSearch, selectedObject, vintersectedrect, viewPort, nodesInView);
            var i, target, targetBounds;
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (this._canConsider(nameTable, selectedObject, target) && !target.segments) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (lengthX == null || lengthX > Math.abs(targetBounds.y - bounds.y)) {
                        if (hSnap.left) {
                            if (Math.abs(bounds.x + delx - targetBounds.x) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "left");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                            else if (Math.abs(bounds.x + delx - targetBounds.x - targetBounds.width) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "leftRight");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                        }
                        else if (hSnap.right) {
                            if (Math.abs(bounds.x + delx + bounds.width - targetBounds.x - targetBounds.width) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "right");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                            else if (Math.abs(bounds.x + delx + bounds.width - targetBounds.x) <= snapObjDistance) {
                                hTarget = this._createSnapObject(targetBounds, bounds, "rightLeft");
                                lengthX = Math.abs(targetBounds.y - bounds.y);
                            }
                        }
                    }
                }
            }
            nodes = this._findNodes(diagram._spatialSearch, selectedObject, hintersectedrect, viewPort);
            for (i = 0; i < nodes.length; i++) {
                target = nodes[i];
                if (this._canConsider(nameTable, selectedObject, target) && !target.segments) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    if (lengthY == null || lengthY > Math.abs(targetBounds.x - bounds.x)) {
                        if (vSnap.top) {
                            if (Math.abs(bounds.y + dely - targetBounds.y) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "top");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                            else if (Math.abs(bounds.y + dely - targetBounds.y - targetBounds.height) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "topBottom");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                        }
                        else if (vSnap.bottom) {
                            if (Math.abs(bounds.y + bounds.height + dely - targetBounds.y - targetBounds.height) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "bottom");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                            else if (Math.abs(bounds.y + bounds.height + dely - targetBounds.y) <= snapObjDistance) {
                                vTarget = this._createSnapObject(targetBounds, bounds, "bottomTop");
                                lengthY = Math.abs(targetBounds.x - bounds.x);
                            }
                        }
                    }
                }
            }
            for (i = 0; i < nodesInView.length; i++) {
                target = nodesInView[i];
                if (this._canConsider(nameTable, selectedObject, target)) {
                    var targetbounds = ej.datavisualization.Diagram.Util.bounds(target);
                    var del = hSnap.left ? -delx : delx;
                    var diff = Math.abs(bounds.width + del - targetbounds.width);
                    var actualDiff;
                    if (diff <= snapObjDistance) {
                        actualDiff = hSnap.left ? -targetbounds.width + bounds.width : targetbounds.width - bounds.width;
                        sameWidth[sameWidth.length] = { source: target, diff: diff, offset: actualDiff };
                    }
                    del = vSnap.top ? -dely : dely;
                    var dify = Math.abs(bounds.height + del - targetbounds.height);
                    if (dify <= snapObjDistance) {
                        actualDiff = vSnap.top ? -targetbounds.height + bounds.height : targetbounds.height - bounds.height;
                        sameHeight[sameHeight.length] = { source: target, diff: dify, offset: actualDiff };
                    }
                }
            }
            var g = this._createGuidelines(diagram, hTarget, vTarget, hSnap, vSnap, ended);
            if (!hSnap.snapped && sameWidth.length > 0 && (hSnap.left || hSnap.right)) {
                this._addSameWidthLines(diagram, sameWidth, hSnap, ended, g, selectedObject);
            }
            if (!vSnap.snapped && sameHeight.length > 0 && (vSnap.top || vSnap.bottom)) {
                this._addSameHeightLines(diagram, sameHeight, vSnap, ended, g, selectedObject);
            }
        },
        _createSnapObject: function (targetbounds, bounds, snap) {
            var snapObject;
            switch (snap) {
                case "left":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x - bounds.x, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "right":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x + targetbounds.width - bounds.x - bounds.width, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "top":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y),
                        offsetY: targetbounds.y - bounds.y, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "bottom":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height),
                        offsetY: targetbounds.y + targetbounds.height - bounds.y - bounds.height, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "topBottom":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height),
                        offsetY: targetbounds.y + targetbounds.height - bounds.y, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "bottomTop":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y),
                        offsetY: targetbounds.y - bounds.y - bounds.height, offsetX: 0, type: "sideAlign"
                    };
                    break;
                case "leftRight":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x + targetbounds.width - bounds.x, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "rightLeft":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x - bounds.x - bounds.width, offsetY: 0, type: "sideAlign"
                    };
                    break;
                case "centerX":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width / 2, Math.min(targetbounds.y, bounds.y)),
                        end: ej.datavisualization.Diagram.Point(targetbounds.x + targetbounds.width / 2, Math.max(targetbounds.y + targetbounds.height, bounds.y + bounds.height)),
                        offsetX: targetbounds.x + targetbounds.width / 2 - (bounds.x + bounds.width / 2), offsetY: 0, type: "centerAlign"
                    };
                    break;
                case "centerY":
                    snapObject = {
                        start: ej.datavisualization.Diagram.Point(Math.min(targetbounds.x, bounds.x), targetbounds.y + targetbounds.height / 2),
                        end: ej.datavisualization.Diagram.Point(Math.max(targetbounds.x + targetbounds.width, bounds.x + bounds.width), targetbounds.y + targetbounds.height / 2),
                        offsetY: targetbounds.y + targetbounds.height / 2 - (bounds.y + bounds.height / 2), offsetX: 0, type: "centerAlign"
                    };
                    break;
            }
            return snapObject;
        },
        _createGuidelines: function (diagram, hTarget, vTarget, hsnap, vsnap, ended) {
            var attr = { id: diagram._canvas.id + "guideline", "style": "pointer-events:none;" };
            var g = ej.datavisualization.Diagram.SvgContext._initializeGuidelines(diagram._adornerSvg, attr, diagram._adornerLayer);
            var scale = diagram._currZoom;
            var hOffset = diagram._hScrollOffset;
            var vOffset = diagram._vScrollOffset;
            if (hTarget) {
                hsnap.offset = hTarget.offsetX;
                hsnap.snapped = true;
                if (!ended) {
                    if (hTarget.type == "sideAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderSideAlignmentLines(hTarget.start, hTarget.end, g, diagram._adornerSvg, scale, hOffset, vOffset);
                    }
                    else if (hTarget.type == "centerAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderCenterAlignmentLines(hTarget.start, hTarget.end, g, diagram._adornerSvg, scale, hOffset, vOffset);
                    }
                }
            }
            if (vTarget) {
                vsnap.offset = vTarget.offsetY;
                vsnap.snapped = true;
                if (!ended) {
                    if (vTarget.type == "sideAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderSideAlignmentLines(vTarget.start, vTarget.end, g, diagram._adornerSvg, scale, hOffset, vOffset);
                    }
                    else if (vTarget.type == "centerAlign") {
                        ej.datavisualization.Diagram.SvgContext._renderCenterAlignmentLines(vTarget.start, vTarget.end, g, diagram._adornerSvg, scale, hOffset, vOffset);
                    }
                }
            }
            return g;
        },
        _removeGuidelines: function (diagram) {
            var g = diagram._adornerSvg.getElementById(diagram._canvas.id + "guideline");
            if (g != null) { ej.datavisualization.Diagram.SvgContext._removeGuidelines(diagram._adornerLayer, g); }
        },
        _sortByDistance: function (obj, value, asc) {
            var i, j, temp;
            if (asc) {
                for (i = 0; i < obj.length; i++) {
                    for (j = i + 1; j < obj.length; j++) {
                        if (obj[i][value] > obj[j][value]) {
                            temp = obj[i];
                            obj[i] = obj[j];
                            obj[j] = temp;
                        }
                    }
                }
            }
            else {
                for (i = 0; i < obj.length; i++) {
                    for (j = i + 1; j < obj.length; j++) {
                        if (obj[i][value] < obj[j][value]) {
                            temp = obj[i];
                            obj[i] = obj[j];
                            obj[j] = temp;
                        }
                    }
                }
            }
        },
        _createHSpacingLines: function (diagram, shape, objectsAtLeft, objectsAtRight, hSnap, vSnap, ended, g, del, snapObjDistance) {
            var top = null;
            this._sortByDistance(objectsAtLeft, "distance", true);
            this._sortByDistance(objectsAtRight, "distance", true);
            var equallySpaced = [];
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var nearestleft, nearestright;
            var targetBounds;
            if (objectsAtLeft.length > 0) {
                equallySpaced[equallySpaced.length] = objectsAtLeft[0];
                nearestleft = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[0].object);
                top = nearestleft.y;
                if (objectsAtLeft.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[1].object);
                    var equaldistance = nearestleft.x - targetBounds.x - targetBounds.width;
                    if (Math.abs(equaldistance - objectsAtLeft[0].distance) <= snapObjDistance) {
                        top = this._findEquallySpacedNodesAtLeft(objectsAtLeft, equaldistance, top, equallySpaced);

                    } else equaldistance = objectsAtLeft[0].distance;
                } else equaldistance = objectsAtLeft[0].distance;
            }
            this._sortByDistance(equallySpaced, "distance");
            equallySpaced[equallySpaced.length] = { object: shape, distance: 0 };
            top = bounds.y < top || !top ? bounds.y : top;
            if (objectsAtRight.length > 0) {
                var dist;
                nearestright = ej.datavisualization.Diagram.Util.bounds(objectsAtRight[0].object);
                top = nearestright.y < top ? nearestright.y : top;
                if (objectsAtRight.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtRight[1].object);
                    dist = targetBounds.x - nearestright.x - nearestright.width;
                }
                if (objectsAtLeft.length > 0) {
                    if (Math.abs(objectsAtRight[0].distance - objectsAtLeft[0].distance) <= snapObjDistance) {
                        var adjustablevalue = Math.abs(objectsAtRight[0].distance - objectsAtLeft[0].distance) / 2;
                        (objectsAtRight[0].distance < objectsAtLeft[0].distance) ?
                        equaldistance -= adjustablevalue : equaldistance += adjustablevalue;
                        equallySpaced[equallySpaced.length] = objectsAtRight[0];
                    }
                    else if (objectsAtLeft.length == 1) {
                        nearestleft = undefined;
                        equallySpaced.splice(0, 1);
                        equallySpaced[equallySpaced.length] = objectsAtRight[0];
                        equaldistance = dist;
                    }
                }
                else {
                    equaldistance = dist;
                    equallySpaced[equallySpaced.length] = objectsAtRight[0];
                }
                if (objectsAtRight.length > 1 && nearestright.x + nearestright.width < targetBounds.x) {
                    top = this._findEquallySpacedNodesAtRight(objectsAtRight, dist, top, equallySpaced, snapObjDistance);
                }
            }
            if (equallySpaced.length > 2) {
                this._addHSpacingLines(diagram, equallySpaced, ended, g, top);
                var delta = 0;
                if (ended)
                    delta = del.x;
                if (nearestleft)
                    hSnap.offset = equaldistance - Math.abs(bounds.x + delta - nearestleft.x - nearestleft.width) + delta;
                else if (nearestright)
                    hSnap.offset = Math.abs(bounds.x + bounds.width + delta - nearestright.x) - equaldistance + delta;
                hSnap.snapped = true;
            }
        },
        _createVSpacingLines: function (diagram, shape, objectsAtTop, objectsAtBottom, hSnap, vSnap, ended, g, del, snapObjDistance) {
            var right = null;
            this._sortByDistance(objectsAtTop, "distance", true);
            this._sortByDistance(objectsAtBottom, "distance", true);
            var equallySpaced = [];
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var nearesttop, nearestbottom;
            var targetBounds;
            if (objectsAtTop.length > 0) {
                equallySpaced[equallySpaced.length] = objectsAtTop[0];
                nearesttop = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[0].object);
                right = nearesttop.x + nearesttop.width;
                if (objectsAtTop.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[1].object);
                    var equaldistance = nearesttop.y - targetBounds.y - targetBounds.height;
                    if (Math.abs(equaldistance - objectsAtTop[0].distance) <= snapObjDistance) {
                        right = this._findEquallySpacedNodesAtTop(objectsAtTop, equaldistance, right, equallySpaced);
                    } else equaldistance = objectsAtTop[0].distance;
                } else equaldistance = objectsAtTop[0].distance;
            }
            this._sortByDistance(equallySpaced, "distance");
            equallySpaced[equallySpaced.length] = { object: shape, distance: 0 };
            right = bounds.x + bounds.width > right || !right ? bounds.x + bounds.width : right;
            var dist;
            if (objectsAtBottom.length > 0) {
                nearestbottom = ej.datavisualization.Diagram.Util.bounds(objectsAtBottom[0].object);
                right = nearestbottom.x + nearestbottom.width > right ? nearestbottom.x + nearestbottom.width : right;
                if (objectsAtBottom.length > 1) {
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtBottom[1].object);
                    dist = targetBounds.y - nearestbottom.y - nearestbottom.height;
                }

                if (objectsAtTop.length > 0) {
                    if (Math.abs(objectsAtBottom[0].distance - objectsAtTop[0].distance) <= snapObjDistance) {
                        var adjustablevalue = Math.abs(objectsAtBottom[0].distance - objectsAtTop[0].distance) / 2;
                        (objectsAtBottom[0].distance < objectsAtTop[0].distance) ?
                        equaldistance -= adjustablevalue : equaldistance += adjustablevalue;
                        equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                    }
                    else if (objectsAtTop.length == 1) {
                        nearesttop = undefined;
                        equallySpaced.splice(0, 1);
                        equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                        equaldistance = dist;
                    }
                }
                else {
                    equaldistance = dist;
                    equallySpaced[equallySpaced.length] = objectsAtBottom[0];
                }
                if (objectsAtBottom.length > 1 && targetBounds.y > nearestbottom.y + nearestbottom.height) {
                    right = this._findEquallySpacedNodesAtBottom(objectsAtBottom, dist, right, equallySpaced, snapObjDistance);
                }

            }
            if (equallySpaced.length > 2) {
                this._addVSpacingLine(diagram, equallySpaced, ended, g, right);
                var delta = 0;
                if (ended)
                    delta = del.y;
                if (nearesttop)
                    vSnap.offset = equaldistance - Math.abs(bounds.y + delta - nearesttop.y - nearesttop.height) + delta;
                else if (nearestbottom)
                    vSnap.offset = Math.abs(bounds.y + bounds.height + delta - nearestbottom.y) - equaldistance + delta;
                vSnap.snapped = true;
            }
        },
        _addHSpacingLines: function (diagram, equallySpaced, ended, g, top) {
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            if (equallySpaced.length > 2) {
                if (!ended) {
                    for (var i = 0; i < equallySpaced.length - 1; i++) {
                        var crnt = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i].object);
                        var next = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i + 1].object);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(crnt.x + crnt.width, top - 15),
                       ej.datavisualization.Diagram.Point(next.x, top - 15), g, diagram._adornerSvg, scale, hoffset, voffset);
                    }
                }

            }
        },
        _addVSpacingLine: function (diagram, equallySpaced, ended, g, right) {
            var scale = diagram._currZoom;
            var hoffset = diagram._hScrollOffset;
            var voffset = diagram._vScrollOffset;
            if (equallySpaced.length > 2) {
                if (!ended) {
                    var crnt, next;
                    for (var i = 0; i < equallySpaced.length - 1; i++) {
                        crnt = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i].object);
                        next = ej.datavisualization.Diagram.Util.bounds(equallySpaced[i + 1].object);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(right + 15, crnt.y + crnt.height),
                      ej.datavisualization.Diagram.Point(right + 15, next.y), g, diagram._adornerSvg, scale, hoffset, voffset);
                    }
                }

            }
        },
        _addSameWidthLines: function (diagram, sameWidths, hSnap, ended, g, shape) {
            var scale = diagram._currZoom;
            var hOffset = diagram._hScrollOffset;
            var vOffset = diagram._vScrollOffset;
            this._sortByDistance(sameWidths, "offset");
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var target = sameWidths[0];
            var targetBounds = ej.datavisualization.Diagram.Util.bounds(target.source);
            var sameSizes = [];
            sameSizes.push(sameWidths[0]);
            var i, crntbounds;
            for (i = 1; i < sameWidths.length; i++) {
                crntbounds = ej.datavisualization.Diagram.Util.bounds(sameWidths[i].source);
                if (crntbounds.width == targetBounds.width)
                    sameSizes.push(sameWidths[i]);
            }
            if (sameSizes.length > 0) {
                if (!ended) {
                    ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x + target.offset, bounds.y - 15),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width + target.offset, bounds.y - 15), g, diagram._adornerSvg, scale, hOffset, vOffset);
                    for (i = 0; i < sameSizes.length; i++) {
                        bounds = ej.datavisualization.Diagram.Util.bounds(sameSizes[i].source);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x, bounds.y - 15),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width, bounds.y - 15), g, diagram._adornerSvg, scale, hOffset, vOffset);
                    }
                }
            }
            hSnap.offset = target.offset;
            hSnap.snapped = true;
        },
        _addSameHeightLines: function (diagram, sameHeights, vSnap, ended, g, shape) {
            var scale = diagram._currZoom;
            var hOffset = diagram._hScrollOffset;
            var vOffset = diagram._vScrollOffset;
            this._sortByDistance(sameHeights, "offset");
            var bounds = ej.datavisualization.Diagram.Util.bounds(shape);
            var target = sameHeights[0];
            var targetBounds = ej.datavisualization.Diagram.Util.bounds(target.source);
            var sameSizes = [];
            sameSizes.push(sameHeights[0]);
            var i, crntBounds;
            for (i = 0; i < sameHeights.length; i++) {
                crntBounds = ej.datavisualization.Diagram.Util.bounds(sameHeights[i].source);
                if (crntBounds.height == targetBounds.height)
                    sameSizes.push(sameHeights[i]);
            }
            if (sameSizes.length > 0) {
                if (!ended) {
                    ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + target.offset),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + bounds.height + target.offset), g, diagram._adornerSvg, scale, hOffset, vOffset);
                    for (i = 0; i < sameSizes.length; i++) {
                        bounds = ej.datavisualization.Diagram.Util.bounds(sameSizes[i].source);
                        ej.datavisualization.Diagram.SvgContext._renderSpacingLines(ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y),
                        ej.datavisualization.Diagram.Point(bounds.x + bounds.width + 15, bounds.y + bounds.height), g, diagram._adornerSvg, scale, hOffset, vOffset);

                    }
                }
                vSnap.offset = target.offset;
                vSnap.snapped = true;
            }
        },
        _findEquallySpacedNodesAtLeft: function (objectsAtLeft, equalDistance, top, equallySpaced) {
            var prevBounds, targetBounds, dist;
            for (var i = 1; i < objectsAtLeft.length; i++) {
                prevBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[i - 1].object);
                targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtLeft[i].object);
                dist = prevBounds.x - targetBounds.x - targetBounds.width;
                if (Math.abs(dist - equalDistance) <= 1) {
                    equallySpaced[equallySpaced.length] = objectsAtLeft[i];
                    if (targetBounds.y < top)
                        top = targetBounds.y;
                }
                else
                    break;
            }
            return top;
        },
        _findEquallySpacedNodesAtRight: function (objectsAtRight, equalDistance, top, equallySpaced, snapObjDistance) {
            var actualDistance = objectsAtRight[0].distance;
            var target, targetBounds, prevBounds, dist;
            if (Math.abs(equalDistance - actualDistance) <= snapObjDistance) {
                for (var i = 0; i < objectsAtRight.length - 1; i++) {
                    target = objectsAtRight[i].object;
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtRight[i + 1].object);
                    prevBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    dist = targetBounds.x - prevBounds.x - prevBounds.width;
                    if (Math.abs(dist - equalDistance) <= 1) {
                        equallySpaced[equallySpaced.length] = objectsAtRight[i + 1];
                        if (prevBounds.y < top) {
                            top = prevBounds.y;
                        }
                    }
                    else
                        break;
                }
            }
            return top;
        },
        _findEquallySpacedNodesAtTop: function (objectsAtTop, equalDistance, right, equallySpaced) {
            var prevBounds, targetBounds, dist;
            for (var i = 1; i < objectsAtTop.length; i++) {
                prevBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[i - 1].object);
                targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtTop[i].object);
                dist = prevBounds.y - targetBounds.y - targetBounds.height;
                if (Math.abs(dist - equalDistance) <= 1) {
                    equallySpaced[equallySpaced.length] = objectsAtTop[i];
                    if (targetBounds.x + targetBounds.width > right)
                        right = targetBounds.x + targetBounds.width;
                }
                else
                    break;
            }
            return right;
        },
        _findEquallySpacedNodesAtBottom: function (objectsAtBottom, equalDistance, right, equallySpaced, snapObjDistance) {
            var actualDistance = objectsAtBottom[0].distance;
            var target, targetBounds, prevBounds, dist;
            if (Math.abs(equalDistance - actualDistance) <= snapObjDistance) {
                for (var i = 0; i < objectsAtBottom.length - 1; i++) {
                    target = objectsAtBottom[i].object;
                    targetBounds = ej.datavisualization.Diagram.Util.bounds(objectsAtBottom[i + 1].object);
                    prevBounds = ej.datavisualization.Diagram.Util.bounds(target);
                    dist = targetBounds.y - prevBounds.y - prevBounds.height;
                    if (Math.abs(dist - equalDistance) <= 1) {
                        equallySpaced[equallySpaced.length] = objectsAtBottom[i + 1];
                        if (prevBounds.x + prevBounds.width > right)
                            right = prevBounds.x + prevBounds.width;
                    }
                    else
                        break;
                }
            }
            return right;
        },

    };
    //#endregion

    //#region SpatialSearch
    ej.datavisualization.Diagram.SpatialUtil = {
        findQuads: function (spatialSearch, viewPort) {
            spatialSearch.quads = [];
            var quad = spatialSearch.parentQuad;
            this._findQuads(spatialSearch, quad, viewPort);
            return spatialSearch.quads;
        },
        _addIntoAQuad: function (spatialSearch) {
            var isAdded = false;
            while (!isAdded) {
                isAdded = this._add(spatialSearch, spatialSearch.parentQuad);
            }
        },
        _setCurrentNode: function (spatialSearch, node) {
            spatialSearch.childnode = node;
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            spatialSearch.childLeft = bounds.left;
            spatialSearch.childTop = bounds.top;
            spatialSearch.childRight = bounds.right;
            spatialSearch.childBottom = bounds.bottom;
        },
        _add: function (spatialSearch, quad) {
            if (this._isContained(spatialSearch, quad)) {
                this._selectQuad(spatialSearch, quad);
                return true;
            } else {
                var newParent;
                var isEmpty = !quad.objects.length && quad.first == null && quad.second == null && quad.third == null && quad.fourth == null;
                if (spatialSearch.childLeft < quad.left) {
                    if (spatialSearch.childTop < quad.top) {
                        newParent = ej.datavisualization.Diagram.Quad(quad.left - quad.width, quad.top - quad.height, quad.width * 2, quad.height * 2);
                        if (!isEmpty)
                            newParent.fourth = quad;
                    }
                    else {
                        newParent = ej.datavisualization.Diagram.Quad(quad.left - quad.width, quad.top, quad.width * 2, quad.height * 2);
                        if (!isEmpty)
                            newParent.second = quad;
                    }
                }
                else if (spatialSearch.childTop < quad.top) {
                    newParent = ej.datavisualization.Diagram.Quad(quad.left, quad.top - quad.height, quad.width * 2, quad.height * 2);
                    if (!isEmpty)
                        newParent.third = quad;
                }
                else {
                    newParent = ej.datavisualization.Diagram.Quad(quad.left, quad.top, quad.width * 2, quad.height * 2);
                    if (!isEmpty)
                        newParent.first = quad;
                }
                quad.parent = newParent;
                spatialSearch.parentQuad = newParent;
                return false;
            }
        },
        _isContained: function (spatialSearch, quad) {
            if (spatialSearch.childLeft >= quad.left && spatialSearch.childRight <= quad.left + quad.width &&
                spatialSearch.childTop >= quad.top && spatialSearch.childBottom <= quad.top + quad.height) {
                return true;
            }
            return false;
        },
        _selectQuad: function (spatialSearch, quad) {
            var current = quad;
            while (current !== null) {
                current = this._getQuad(spatialSearch, current);
            }
        },
        _getQuad: function (spatialSearch, quad) {
            var halfWidth = quad.width / 2;
            var halfHeight = quad.height / 2;
            if (halfWidth >= 100 && halfHeight >= 100) {
                var xCenter = quad.left + halfWidth;
                var yCenter = quad.top + halfHeight;
                if (spatialSearch.childRight <= xCenter) {
                    if (spatialSearch.childBottom <= yCenter) {
                        return quad.first ? quad.first :
                               quad.first = ej.datavisualization.Diagram.Quad(quad.left, quad.top, halfWidth, halfHeight, quad);
                    }
                    else if (spatialSearch.childTop >= yCenter) {
                        return quad.third ? quad.third :
                                quad.third = ej.datavisualization.Diagram.Quad(quad.left, yCenter, halfWidth, halfHeight, quad);
                    }
                }
                else if (spatialSearch.childLeft >= xCenter) {
                    if (spatialSearch.childBottom <= yCenter) {
                        return quad.second ? quad.second :
                               quad.second = ej.datavisualization.Diagram.Quad(xCenter, quad.top, halfWidth, halfHeight, quad);
                    }
                    else if (spatialSearch.childTop >= yCenter) {
                        return quad.fourth ? quad.fourth :
                                quad.fourth = ej.datavisualization.Diagram.Quad(xCenter, yCenter, halfWidth, halfHeight, quad);
                    }
                }
            }
            quad.objects.push(spatialSearch.childnode);
            spatialSearch.quadTable[spatialSearch.childnode.name] = quad;
            return null;
        },
        _findQuads: function (spatialSearch, quad, viewPort) {
            if (quad.first && this._isIntersect(quad.first, viewPort)) {
                this._findQuads(spatialSearch, quad.first, viewPort);
            }
            if (quad.second && this._isIntersect(quad.second, viewPort)) {
                this._findQuads(spatialSearch, quad.second, viewPort);
            }
            if (quad.third && this._isIntersect(quad.third, viewPort)) {
                this._findQuads(spatialSearch, quad.third, viewPort);
            }
            if (quad.fourth && this._isIntersect(quad.fourth, viewPort)) {
                this._findQuads(spatialSearch, quad.fourth, viewPort);
            }
            if (quad.objects.length > 0)
                spatialSearch.quads.push(quad);
        },
        _isIntersect: function (quad, viewPort) {
            if (quad.left + quad.width < viewPort.x || quad.top + quad.height < viewPort.y || quad.left > viewPort.x + viewPort.width || quad.top > viewPort.y + viewPort.height) {
                return false;
            }
            return true;
        },
        _update: function (quad) {
            if (quad.parent != null && quad.objects.length == 0 && quad.first == null &&
                quad.second == null && quad.third == null && quad.fourth == null) {
                var parent = quad.parent;
                if (parent.first == quad) {
                    parent.first = null;
                }
                else if (parent.second == quad) {
                    parent.second = null;
                }
                else if (parent.third == quad) {
                    parent.third = null;
                }
                else if (parent.fourth == quad) {
                    parent.fourth = null;
                }
                this._update(quad.parent);
            }
            else
                return;
        },
        _isWithinPageBounds: function (spatialSearch, bounds) {
            if (bounds.left >= spatialSearch.pageLeft && bounds.right <= spatialSearch.pageRight &&
                bounds.top >= spatialSearch.pageTop && bounds.bottom <= spatialSearch.pageBottom)
                return true;
            else
                return false;
        },
        _updateBounds: function (diagram, spatialSearch, node) {
            var modified = false;
            if (spatialSearch.topElement && node.name == spatialSearch.topElement.name) {
                modified = true;
                spatialSearch.pageTop = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "top");
            }
            if (spatialSearch.leftElement && node.name == spatialSearch.leftElement.name) {
                modified = true;
                spatialSearch.pageLeft = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "left");
            }
            if (spatialSearch.rightElement && node.name == spatialSearch.rightElement.name) {
                modified = true;
                spatialSearch.pageRight = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "right");
            }
            if (spatialSearch.bottomElement && node.name == spatialSearch.bottomElement.name) {
                modified = true;
                spatialSearch.pageBottom = null;
                this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "bottom");
            }
            if (modified) {
                if (diagram._pageBackgroundLayer) {
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram, true);
                }
            }
        },
        _findQuadElement: function (spatialSearch, quad, direction) {
            switch (direction) {
                case "bottom":
                    if (quad.third != null || quad.fourth != null) {
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    else {
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                    }
                    break;
                case "top":
                    if (quad.first != null || quad.second != null) {
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                    }
                    else {
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    break;
                case "left":
                    if (quad.first != null || quad.third != null) {
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                    }
                    else {
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    break;
                case "right":
                    if (quad.second != null || quad.fourth != null) {
                        if (quad.second != null) {
                            this._findQuadElement(spatialSearch, quad.second, direction);
                        }
                        if (quad.fourth != null) {
                            this._findQuadElement(spatialSearch, quad.fourth, direction);
                        }
                    }
                    else {
                        if (quad.third != null) {
                            this._findQuadElement(spatialSearch, quad.third, direction);
                        }
                        if (quad.first != null) {
                            this._findQuadElement(spatialSearch, quad.first, direction);
                        }
                    }

                    break;
            }

            var bounds, type;
            for (var i = 0; i < quad.objects.length; i++) {
                bounds = ej.datavisualization.Diagram.Util.bounds(quad.objects[i]);
                if (direction === "bottom" && (spatialSearch.pageBottom <= bounds.bottom || spatialSearch.pageBottom == null)) {
                    spatialSearch.pageBottom = bounds.bottom;
                    spatialSearch.bottomElement = quad.objects[i];
                }
                else if (direction === "top" && (spatialSearch.pageTop >= bounds.top || spatialSearch.pageTop == null)) {
                    spatialSearch.pageTop = bounds.top;
                    spatialSearch.topElement = quad.objects[i];
                }
                else if (direction === "left" && (spatialSearch.pageLeft >= bounds.left || spatialSearch.pageLeft == null)) {
                    spatialSearch.pageLeft = bounds.left;
                    spatialSearch.leftElement = quad.objects[i];
                }
                else if (direction === "right" && (spatialSearch.pageRight <= bounds.right || spatialSearch.pageRight == null)) {
                    spatialSearch.pageRight = bounds.right;
                    spatialSearch.rightElement = quad.objects[i];
                }
            }
        },
        _initializeNodes: function (diagram, spatialSearch) {
            var nodes = diagram.nodes();
            for (var i = 0; i < nodes.length; i++) {
                var node = nodes[i];
                var type = this.getObjectType(node);
                if (type != "group") {
                    this._updateQuad(diagram, spatialSearch, node);
                }
            }
        },

        _initializeConnectors: function (diagram, spatialSearch) {
            var connectors = diagram.connectors();
            for (var i = 0; i < connectors.length; i++) {
                this._updateQuad(diagram, spatialSearch, connectors[i]);
            }
        },

        _updateQuad: function (diagram, spatialSearch, node, layoutinAction) {
            this._setCurrentNode(spatialSearch, node);
            var bounds = ej.datavisualization.Diagram.Util.bounds(node);
            var quad = spatialSearch.quadTable[node.name];
            if (quad) {
                if (!this._isContained(spatialSearch, quad)) {
                    this._removeFromaQuad(spatialSearch, quad, node);
                    this._addIntoAQuad(spatialSearch);
                }
            }
            else {
                if (!spatialSearch.parentQuad)
                    spatialSearch.parentQuad = ej.datavisualization.Diagram.Quad(0, 0, 200, 200);
                this._addIntoAQuad(spatialSearch);
            }
            if (!(this._isWithinPageBounds(spatialSearch, bounds) && spatialSearch.leftElement != node &&
                spatialSearch.topElement != node && spatialSearch.rightElement != node && spatialSearch.bottomElement != node)) {
                var modified = false, endElement = false;
                if (spatialSearch.pageLeft >= spatialSearch.childLeft || spatialSearch.pageLeft == null) {
                    spatialSearch.pageLeft = spatialSearch.childLeft;
                    spatialSearch.leftElement = node;
                    modified = true;
                }
                else if (node == spatialSearch.leftElement) {
                    spatialSearch.pageLeft = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "left");
                    endElement = modified = true;
                }

                if (spatialSearch.pageTop >= spatialSearch.childTop || spatialSearch.pageTop == null) {
                    spatialSearch.pageTop = spatialSearch.childTop;
                    spatialSearch.topElement = node;
                    modified = true;
                }
                else if (node == spatialSearch.topElement) {
                    spatialSearch.pageTop = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "top");
                    endElement = modified = true;
                }

                if (spatialSearch.pageBottom <= spatialSearch.childBottom || spatialSearch.pageBottom == null) {
                    modified = true;
                    spatialSearch.pageBottom = spatialSearch.childBottom;
                    spatialSearch.bottomElement = node;
                }
                else if (node == spatialSearch.bottomElement) {
                    spatialSearch.pageBottom = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "bottom");
                    endElement = modified = true;
                }

                if (spatialSearch.pageRight <= spatialSearch.childRight || spatialSearch.pageRight == null) {
                    spatialSearch.pageRight = spatialSearch.childRight;
                    spatialSearch.rightElement = node;
                    modified = true;
                }
                else if (node == spatialSearch.rightElement) {
                    spatialSearch.pageRight = null;
                    this._findQuadElement(spatialSearch, spatialSearch.parentQuad, "right");
                    endElement = modified = true;
                }
            }
            var pageBounds = diagram.activeTool.diagramBounds;
            if (modified && diagram.model.pageSettings.multiplePage &&
                (diagram.model.pageSettings.pageWidth || diagram.model.pageSettings.pageHeight) && pageBounds) {
                modified = false;
                if (endElement || spatialSearch.pageLeft < pageBounds.x || spatialSearch.pageTop < pageBounds.y ||
                    spatialSearch.pageRight > pageBounds.x + pageBounds.width || spatialSearch.pageBottom > pageBounds.y + pageBounds.height) {
                    modified = true;
                    diagram.activeTool.diagramBounds = diagram._getDigramBounds();
                }
            }
            if (modified) {
                diagram._views.forEach(function (viewid) {
                    var view = diagram._views[viewid];
                    if (view.type == "overview") {
                        var ovw = $("#" + viewid).ejOverview("instance");
                        if (ovw)
                            ovw._updateOverview(view);
                    }
                });
                if (!diagram._layoutInAction && diagram._pageBackgroundLayer) {
                    ej.datavisualization.Diagram.PageUtil._updatePageSize(diagram, diagram.activeTool.inAction);
                }
            }
        },
        _removeFromaQuad: function (spatialSearch, quad, node) {
            if (quad) {
                var index = quad.objects.indexOf(node);
                quad.objects.splice(index, 1);
                this._update(quad);
                delete spatialSearch.quadTable[node.name];
            }
        }

    };

    ej.datavisualization.Diagram.Quad = function (left, top, width, height, parent) {
        return {
            left: left, top: top, width: width, height: height, parent: parent,
            first: null, second: null, third: null, fourth: null, objects: []
        };
    }
    ej.datavisualization.Diagram.SpatialSearch = function () {
        return {
            pageLeft: null, pageRight: null, pageTop: null, pageBottom: null,
            topElement: null, bottomElement: null, rightElement: null, leftElement: null,
            childLeft: null, childTop: null, childRight: null, childBottom: null, childNode: null,
            quads: null, parentQuad: ej.datavisualization.Diagram.Quad(0, 0, 200, 200), quadTable: {}
        };
    };
    //#endregion

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Overview elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/
(function ($, ej, undefined) {
    "use strict";
    //#region Overview widget
    ej.widget("ejOverview", "ej.datavisualization.Overview", {
        element: null,
        validTags: ["div"],
        model: null,
        //#region Initialization
        defaults: {
            sourceID: "",
            width: null,
            height: null
        },
        _canvas: null,
        _actionName: "",
        _startPoint: null,
        _currentPoint: null,
        _prevPoint: null,
        _helper: null,
        _viewPortRatio: 1,
        _resizeDirection: "",
        _inAction: false,
        _init: function () {
            if (!(ej.browserInfo().name === "msie" && Number(ej.browserInfo().version) < 9)) {
                this._wireEvents();
                this._renderCanvas();
                this._setParent(this.model.sourceID);
            }
        },
        _renderCanvas: function () {
            var canvas = document.createElement("div");
            canvas.setAttribute("id", this.element[0].id + "_canvas");
            canvas.setAttribute("class", "drawing");
            canvas.setAttribute("style", "position:relative; height:" + this.model.height + "; width:" + this.model.width + ";style:-ms-touch-action: none;touch-action: none;");
            this.element[0].setAttribute("tabindex", 0);
            this.element.css({
                overflow: "hidden", outline: "none", display: "block", height: this.model.height, width: this.model.width
            });
            this.element.append(canvas);
            this._canvas = canvas;
        },
        _renderDocument: function (view) {
            view.canvas = this._canvas;
            if (window.SVGSVGElement) {
                var attr = {
                    id: this._canvas.id + "_svg",
                    version: "1.1",
                    xlink: "http://www.w3.org/1999/xlink",
                    "style": view.style,
                    "class": "overview_svg"
                };
                var svg = new ej.datavisualization.Diagram.Svg(attr);
                this._svg = svg;
                view.svg = svg;
                var ovw = document.getElementById(this._id);
                var element = ovw;
                var eWidth = $(element).width();
                var eHeight = $(element).height();
                var bRect = element.getBoundingClientRect();
                var screenX = (window.screenX < 0) ? window.screenX * -1 : window.screenX;
                if (eWidth === 0) {
                    eWidth = Math.floor(((window.innerWidth - screenX) - Math.floor(bRect.left)));
                }
                var screenY = (window.screenY < 0) ? window.screenY * -1 : window.screenY;
                if (eHeight === 0) {
                    eHeight = Math.floor(((window.innerHeight - screenY) - Math.floor(bRect.top)));
                }
                svg.document.setAttribute("width", eWidth);
                svg.document.setAttribute("height", eHeight);
                this.width = eWidth;
                this.height = eHeight;
                var defs = svg.defs({ "id": view.canvas.id + "definition" });
                svg.appendChild(defs);
                view.diagramLayer = ej.datavisualization.Diagram.SvgContext._renderDiagramLayer(view.canvas, view.svg, view.svg);
                view.canvas.appendChild(svg.document);
                this._addOverviewRectPanel();
                this._renderHtmlLayer(view.canvas);
            }
        },
        _removeDocument: function (view) {
            var svg = document.getElementById(this._canvas.id + "_svg");
            this._canvas.removeChild(svg);
            var htmlLayer = document.getElementById(this._canvas.id + "_htmlLayer");
            this._canvas.removeChild(htmlLayer);
            //var rect = document.getElementById("overview_canvasrect");
            //this._canvas.removeChild(rect);
        },
        _renderHtmlLayer: function (canvas) {
            var div = document.createElement("div");
            var attr = { "id": canvas.id + "_htmlLayer", "class": "htmlLayer" };
            ej.datavisualization.Diagram.Util.attr(div, attr);
            var htmlLayer = div;
            div.style.pointerEvents = "none";
            div.style.position = "absolute";
            div.style.left = "0px";
            div.style.top = "0px";
            canvas.appendChild(htmlLayer);
            return htmlLayer;
        },
        _setModel: function (options) {
            for (var option in options) {
                switch (option) {
                    case "sourceID":
                        this._setParent(options[option]);
                        break;
                }
            }
        },
        _setParent: function (diagram) {
            if (this._parent) {
                var oldparent = $("#" + this._parent._id).ejDiagram("instance");
                this._parent = null;
                oldparent._setOverview(null, this._id);
            }
            if (diagram) {
                this.model.sourceID = diagram;
                this._parent = $("#" + diagram).ejDiagram("instance");
                if (this._parent)
                    this._parent._setOverview(this);
            }
        },
        _destroy: function () {
            this.element.empty().removeClass(this.model.cssClass);
        },
        //#endregion
        //#region Events
        _wireEvents: function () {
            var canvas = $(this._canvas);
            this._on(canvas, ej.eventType.mouseDown, this._mousedown);
            this._on(canvas, ej.eventType.mouseMove, this._mousemove);
            this._on(canvas, ej.eventType.mouseUp, this._mouseup);
            this._on(canvas, ej.eventType.mouseLeave, this._documentmouseup);
            this._on(canvas, "touchstart", this.handleTouchStart);
            this._on(canvas, "touchmove", this.handleTouchMove);
            this._on(canvas, "touchend", this.handleTouchEnd);
        },
        _mouseup: function (evt) {
            this._currentPoint = this._mouseposition(evt);
            var offwidth = this.width;
            var offheight = this.height;
            if (this._actionName)
                if ((this._startPoint.x != this._currentPoint.x || this._startPoint.y != this._currentPoint.y)) {
                    if (this._actionName == "pan") { }
                    else {
                        if (this._helper) {
                            var bounds = this._helper.getBBox();
                            this._resizeDirection = this._resizeDirection || "";
                            var x = bounds.x;
                            var y = bounds.y;
                            var width = bounds.width;
                            var height = bounds.height;
                            var adjust = this._resizeDirection == "topleft" || this._resizeDirection == "topright" || this._resizeDirection == "bottomleft" ||
                                this._resizeDirection == "bottomright";
                            this._renderOverviewRect(x, y, width, height);
                        }
                    }
                }
                else
                    if (evt.target.id == this._canvas.id + "overviewbackrect" || evt.target.id == "helper")
                        if (this._startPoint.x == this._currentPoint.x && this._startPoint.y == this._currentPoint.y) {
                            var g = document.getElementById(this._canvas.id + "overviewrect");
                            var bounds = g.getBBox();
                            var width = bounds.width || 100;
                            var height = bounds.height || 100 / this._viewPortRatio;
                            this.inAction = true;
                            this._actionName = "pan";
                            this._renderOverviewRect(this._currentPoint.x - width / 2, this._currentPoint.y - height / 2, width, height);
                            this.inAction = false;
                        }
            if (this._helper) {
                var g = document.getElementById(this._canvas.id + "overviewhandle");
                g.removeChild(this._helper);
                this._helper = null;
            }
            this._actionName = "";
            this._startPoint = null;
            this._currentPoint = null;
            this._prevPoint = null;
            this._helper = null;
            this._viewPortRatio = 1;
            this._resizeDirection = "";
            this._inAction = false;
            delete this._parent._viewPort;
        },
        _mousedown: function (evt) {
            if (evt.target.className.animVal == "overviewbackrect")
                this._actionName = "draw";
            if (evt.target.id == this._canvas.id + "overviewrect") this._actionName = "pan";
            if (evt.target.className.animVal == "overviewresizer") {
                this._actionName = "scale";
                switch (evt.target.id) {
                    case this._canvas.id + "left":
                        this._resizeDirection = "left";
                        break;
                    case this._canvas.id + "right":
                        this._resizeDirection = "right"; break;
                    case this._canvas.id + "top":
                        this._resizeDirection = "top"; break;
                    case this._canvas.id + "bottom":
                        this._resizeDirection = "bottom"; break;
                    case this._canvas.id + "topleft":
                        this._resizeDirection = "topleft"; break;
                    case this._canvas.id + "topright":
                        this._resizeDirection = "topright"; break;
                    case this._canvas.id + "bottomleft":
                        this._resizeDirection = "bottomleft"; break;
                    case this._canvas.id + "bottomright":
                        this._resizeDirection = "bottomright"; break;
                }
            }
            this._startPoint = this._prevPoint = this._mouseposition(evt);
            var viewPort = this._parent._viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            this._viewPortRatio = viewPort.width / viewPort.height;
        },
        _mouseposition: function (evt) {
            var e = this._parent._isTouchEvent(evt);
            if (!e) {
                e = evt;
            }
            var scrollLeft = this._canvas.scrollLeft;
            var scrollTop = this._canvas.scrollTop;
            var controlBBox = this._canvas.getBoundingClientRect();
            var layerx = (e.clientX + scrollLeft) - controlBBox.left;
            var layery = (scrollTop + e.clientY) - controlBBox.top;
            return new ej.datavisualization.Diagram.Point(layerx, layery);
        },
        _updateCursor: function (evt) {
            if (evt.target.className.animVal == "overviewresizer") {
                switch (evt.target.id) {
                    case this._canvas.id + "left":
                        this._canvas.style.cursor = "w-resize";
                        break;
                    case this._canvas.id + "right":
                        this._canvas.style.cursor = "e-resize"; break;
                    case this._canvas.id + "top":
                        this._canvas.style.cursor = "n-resize"; break;
                    case this._canvas.id + "bottom":
                        this._canvas.style.cursor = "s-resize"; break;
                    case this._canvas.id + "topleft":
                        this._canvas.style.cursor = "nw-resize"; break;
                    case this._canvas.id + "topright":
                        this._canvas.style.cursor = "ne-resize"; break;
                    case this._canvas.id + "bottomleft":
                        this._canvas.style.cursor = "sw-resize"; break;
                    case this._canvas.id + "bottomright":
                        this._canvas.style.cursor = "se-resize"; break;
                }
            }
            else this._canvas.style.cursor = "";
        },
        _mousemove: function (evt) {
            this._updateCursor(evt);
            this._currentPoint = this._mouseposition(evt);
            if (this._actionName)
                switch (this._actionName) {
                    case "draw":
                        if (!this._inAction) {
                            this._initHelper();
                            this._inAction = true;
                        }
                        this._updateHelper();
                        break;
                    case "scale":
                        if (!this._inAction) {
                            this._initHelper();
                            this._inAction = true;
                        }
                        this._updateOverviewRectangle();
                        break;
                    case "pan":
                        this._inAction = true;
                        this._translateOverviewRectangle();
                        break;
                }
            this._prevPoint = this._currentPoint;
        },
        _documentmouseup: function (evt) {
            this._inAction = false;
            this._actionName = "";
            if (this._helper) {
                var g = document.getElementById(this._canvas.id + "overviewhandle");
                g.removeChild(this._helper);
                this._helper = null;
            }
        },
        _addOverviewRectPanel: function (view) {
            var svg = this._svg;
            var ovw = svg.g({ "id": this._id + "_overviewlayer" });
            this._canvas.firstChild.appendChild(ovw);
            var rect = svg.rect({ "fill": "transparent", "width": "100%", "height": "100%", "class": "overviewbackrect", "id": this._canvas.id + "overviewbackrect" });
            ovw.appendChild(rect);
            var svgDocument = $(ovw);
            this._on(svgDocument, ej.eventType.mouseDown, this._mousedown);
            this._on(svgDocument, ej.eventType.mouseMove, this._mousemove);
            this._on(svgDocument, ej.eventType.mouseUp, this._mouseup);
            this._on(svgDocument, ej.eventType.mouseLeave, this._documentmouseup);
            this._on(svgDocument, "touchstart", this.handleTouchStart);
            this._on(svgDocument, "touchmove", this.handleTouchMove);
            this._on(svgDocument, "touchend", this.handleTouchEnd);
            var g = svg.g({ "id": this._canvas.id + "overviewhandle" });
            ovw.appendChild(g);
            var innerrect = svg.rect({ "id": this._canvas.id + "overviewrect", "fill": "transparent" });
            g.appendChild(innerrect);
            this._renderOverviewCorner("left", g);
            this._renderOverviewCorner("right", g);
            this._renderOverviewCorner("top", g);
            this._renderOverviewCorner("bottom", g);
            this._renderOverviewCorner("topleft", g);
            this._renderOverviewCorner("topright", g);
            this._renderOverviewCorner("bottomleft", g);
            this._renderOverviewCorner("bottomright", g);
        },
        _updateOverview: function (view) {
            view = view || this._parent._views[this._id];
            var width, height;
            var bounds = this._parent._getDigramBounds();
            width = bounds.width;
            height = bounds.height;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            var offwidth = this.width;
            var offheight = this.height;
            if (view.context == ej.datavisualization.Diagram.SvgContext) {
                var w = Math.max(width, viewPort.width);
                var h = Math.max(height, viewPort.height);
                this.contentWidth = w = Math.max(w, (offwidth / offheight) * h);
                this.contentHeight = h = Math.max(h, (offheight / offwidth) * w);
                var g = document.getElementById(this._canvas.id + "_diagramLayer");
                var scale = Math.min(offwidth / w, offheight / h);
                this.scale = scale;
                g.setAttribute("transform", "scale(" + scale + "," + scale + "),translate(" + -bounds.x + "," + (-bounds.y) + ")");
                var htmlLayer = document.getElementById(this._canvas.id + "_htmlLayer");
                htmlLayer.style.webkitTransform = "scale(" + scale + ") translate(" + -bounds.x + "px," + (-bounds.y) + "px)";
                htmlLayer.style.MozTransform = "scale(" + scale + ") translate(" + -bounds.x + "px," + (-bounds.y) + "px)";
                htmlLayer.style.OTransform = "scale(" + scale + ") translate(" + -bounds.x + "px," + (-bounds.y) + "px)";
                htmlLayer.style.msTransform = "scale(" + scale + ") translate(" + -bounds.x + "px," + (-bounds.y) + "px)";
                htmlLayer.style.transform = "scale(" + scale + ") translate(" + -bounds.x + "px," + (-bounds.y) + "px)";
                // if (bounds.x < 0 || bounds.y < 0) 
                {
                    var ovw = document.getElementById(this._id + "_overviewlayer");
                    ovw.setAttribute("transform", "translate(" + (-bounds.x * scale) + "," + (-bounds.y * scale) + ")");
                }
            }
            else {
                var context = view._canvas.document.getContext("2d");
                if (view.scale) {
                    context.scale(1 / view.scale.x, 1 / view.scale.y);
                }
                view.scale = { x: scale, y: scale };
                context.scale(scale, scale);
                ej.datavisualization.Diagram.CanvasContext.refreshCanvas(this._parent.model, view);
            }
            this._scrollOverviewRect(this._parent._hScrollOffset, this._parent._vScrollOffset, this._parent._currZoom);
        },
        _updateOverviewRectangle: function () {
            var difx = this._currentPoint.x - this._prevPoint.x;
            var dify = this._currentPoint.y - this._prevPoint.y;
            if (this._actionName == "scale") {
                var size = { "width": 0, "height": 0 };
                var x = 0, y = 0; var w, h;
                switch (this._resizeDirection) {
                    case "left":
                        size.width -= difx;
                        size.height -= difx / this._viewPortRatio;
                        x = difx;
                        y = difx / this._viewPortRatio;
                        y /= 2;
                        break;
                    case "right":
                        size.width += difx;
                        size.height += difx / this._viewPortRatio;
                        y = difx / this._viewPortRatio;
                        y /= -2;
                        break;
                    case "top":
                        size.height -= dify;
                        size.width -= dify * this._viewPortRatio;
                        y = dify;
                        x = dify * this._viewPortRatio;
                        x /= 2;
                        break;
                    case "bottom":
                        size.height += dify;
                        size.width += dify * this._viewPortRatio;
                        x = dify * this._viewPortRatio;
                        x /= -2;
                        break;
                    case "topleft":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = dify * this._viewPortRatio; }
                        else
                            dify = difx / this._viewPortRatio;
                        size.width -= difx;
                        size.height -= dify;
                        x = difx;
                        y = dify;
                        break;
                    case "topright":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = -dify * this._viewPortRatio; }
                        else
                            dify = -(difx / this._viewPortRatio);
                        y = dify;
                        size.width += difx;
                        size.height -= dify;
                        break;
                    case "bottomleft":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = -dify * this._viewPortRatio; }
                        else
                            dify = -difx / this._viewPortRatio;
                        x = difx;
                        size.width -= difx;
                        size.height += dify;
                        break;
                    case "bottomright":
                        if (Math.abs(dify) > Math.abs(difx)) { difx = dify * this._viewPortRatio; }
                        else
                            dify = difx / this._viewPortRatio;
                        size.width += difx;
                        size.height += dify;
                        break;
                }
                this._updateHelper(x, y, size, w, h);
            }
        },
        _initHelper: function () {
            if (!this._helper) {
                var svg = this._svg;
                var g = document.getElementById(this._canvas.id + "overviewhandle");
                var scale = this._parent._currZoom;
                var x = this._startPoint.x;
                var y = this._startPoint.y; var width = 1; var height = 1;
                if (this._actionName == "scale") {
                    var rect = document.getElementById(this._canvas.id + "overviewrect");
                    var bounds = rect.getBBox();
                    x = bounds.x; y = bounds.y; width = bounds.width; height = bounds.height;
                }
                var selectionRect = svg.rect({
                    "id": "helper", x: x, y: y, width: width, height: height,
                    "fill": "transparent", "stroke": "gray", "stroke-dasharray": "2 2", "shape-rendering": "crispEdges"
                });
                g.appendChild(selectionRect);
                this._helper = selectionRect;
            }
        },
        _updateHelper: function (difx, dify, size) {
            if (size) {
                var bounds = this._helper.getBBox();
                var x = bounds.x + difx;
                var y = bounds.y + dify;
                var width = bounds.width + size.width;
                var height = bounds.height + size.height;
            }
            else {
                var difx;
                if (this._currentPoint.x > this._startPoint.x)
                    difx = this._currentPoint.x - this._prevPoint.x;
                else
                    difx = this._prevPoint.x - this._currentPoint.x;
                var dify;
                if (this._currentPoint.y > this._startPoint.y)
                    dify = this._currentPoint.y - this._prevPoint.y;
                else dify = this._prevPoint.y - this._currentPoint.y;
                var w, h;
                if (Math.abs(dify) > Math.abs(difx)) {
                    difx = this._viewPortRatio * dify; h = true; w = false;
                }
                else { dify = difx / this._viewPortRatio; w = true; h = false; }
                var bounds = this._helper.getBBox();
                var x = ((this._startPoint.x > this._currentPoint.x) ?
                    bounds.x - difx : bounds.x);
                var y = ((this._startPoint.y > this._currentPoint.y) ? bounds.y - dify : bounds.y);
                var width = bounds.width + difx;
                var height = bounds.height + dify;
            }
            if (this._helper) {
                ej.datavisualization.Diagram.Util.attr(this._helper, {
                    "id": this._helper.id, "x": x, "y": y,
                    "width": (width < 0) ? 0 : width, "height": (height < 0) ? 0 : height
                });
            }
        },
        _renderOverviewCorner: function (name, parent) {
            var svg = this._svg;
            if (name == "top" || name == "bottom" || name == "right" || name == "left") {
                var innerrect = svg.rect({ "id": this._canvas.id + "visible" + name });
                parent.appendChild(innerrect);
                var transrect = svg.rect({ "id": this._canvas.id + name, "class": "overviewresizer", "fill": "transparent" });
                parent.appendChild(transrect);
            }
            else {
                var innerrect = svg.circle({ "id": this._canvas.id + "visible" + name });
                parent.appendChild(innerrect);
                var transrect = svg.circle({ "id": this._canvas.id + name, "class": "overviewresizer", "fill": "transparent" });
                parent.appendChild(transrect);
            }
        },
        _updateOverviewrect: function (x, y, width, height) {
            if (width && height) {
                var rect = document.getElementById(this._canvas.id + "overviewrect");
                var attr = { x: x, y: y, width: Math.max(1, width), height: Math.max(1, height) };
                ej.datavisualization.Diagram.Util.attr(rect, attr);
                this._updateOverviewCorner("top", x + 8, y - 2, Math.max(0, width - 16), 2);
                this._updateOverviewCorner("bottom", x + 8, y + height, Math.max(0, width - 16), 2);
                this._updateOverviewCorner("left", x - 2, y + 8, 2, Math.max(0, height - 16));
                this._updateOverviewCorner("right", x + width, y + 8, 2, Math.max(0, height - 16));
                this._updateOverviewCorner("topleft", x, y, 5, 5);
                this._updateOverviewCorner("topright", x + width, y, 5, 5);
                this._updateOverviewCorner("bottomleft", x, y + height, 5, 5);
                this._updateOverviewCorner("bottomright", x + width, y + height, 5, 5);
            }
        },
        _scrollOverviewRect: function (hoffset, voffset, currentzoom) {
            if (!(this._actionName)) {
                var offwidth = this.width;
                var offheight = this.height;
                var scale = Math.min(this.contentWidth / offwidth, this.contentHeight / offheight);
                var bounds = {};
                var x = bounds.x = (hoffset / currentzoom) / scale;
                var y = bounds.y = (voffset / currentzoom) / scale;
                var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
                var width = bounds.width = (viewPort.width / currentzoom) / scale;
                var height = bounds.height = (viewPort.height / currentzoom) / scale;
                var ratio = viewPort.width / viewPort.height;
                this._updateOverviewrect(x, y, width, height);
            }
        },
        _updateOverviewCorner: function (name, x, y, width, height) {
            var rectname = "visible" + name;
            var rect = document.getElementById(this._canvas.id + rectname);
            if (name == "top" || name == "bottom" || name == "right" || name == "left") {
                var attr = { x: x, y: y, width: width, height: height, fill: "#ED1C24" };
                var transattr = { x: x - 2, y: y - 2, width: width == 2 ? 4 : width, height: height == 2 ? 4 : height };
            }
            else {
                var attr = { cx: x, cy: y, "r": 4, fill: "#ED1C24" };
                var transattr = { cx: x, cy: y, "r": 6, fill: "transparent" };
            }
            ej.datavisualization.Diagram.Util.attr(rect, attr);
            var transrect = document.getElementById(this._canvas.id + name);
            ej.datavisualization.Diagram.Util.attr(transrect, transattr);
        },
        _translateOverviewRectangle: function () {
            var offwidth = this.width;
            var offheight = this.height;
            var difx = this._currentPoint.x - this._prevPoint.x;
            var dify = this._currentPoint.y - this._prevPoint.y;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            var zoom = Math.min(viewPort.width / offwidth, viewPort.height / offheight);
            var panel = document.getElementById(this._canvas.id + "overviewrect");
            var bounds = panel.getBBox();
            var x = bounds.x + difx;
            var y = bounds.y + dify;
            var width = bounds.width;
            var height = bounds.height;
            this._updateOverviewrect(x, y, width, height);
            this._updateView(this._parent._currZoom, x, y, width, height, null);
        },
        _updateView: function (zoom, x, y, width, height, focuspoint) {
            var offwidth = this.width;
            var offheight = this.height;
            var scalex = this.contentWidth / offwidth;
            var scaley = this.contentHeight / offheight;
            var hoffset = x * scalex * zoom;
            var voffset = y * scaley * zoom;
            var bounds = this._parent._getDigramBounds();
            if (zoom != 1 || this._actionName == "pan") {
                var scrollModel = $("#" + this._parent._canvas.id + "_hScrollbar").ejScrollBar("instance").model;
                var scrollwidth = -hoffset + (scrollModel.maximum);
                var delx = -hoffset + this._parent._hScrollOffset;;
                var dely = -voffset + this._parent._vScrollOffset;
                if (bounds.width * zoom < scrollwidth) {
                    var difwidth = bounds.width * zoom - scrollwidth;
                    if (Math.abs(difwidth) < Math.abs(delx)) {
                        delx = delx - difwidth;
                    }
                    //else delx = 0;
                }
                scrollModel = $("#" + this._parent._canvas.id + "_vScrollbar").ejScrollBar("instance").model;
                var scrollheight = -voffset + (scrollModel.maximum);
                if (bounds.height * zoom < scrollheight) {
                    var difwidth = bounds.height * zoom - scrollheight;
                    if (Math.abs(difwidth) < Math.abs(dely)) {
                        dely = dely - difwidth;
                    }
                    //else dely = 0;
                }
            }
            ej.datavisualization.Diagram.ZoomUtil.zoomPan(this._parent, zoom / this._parent._currZoom, delx, dely, focuspoint, false);
        },
        _renderOverviewRect: function (x, y, width, height) {
            var offwidth = this.width;
            var offheight = this.height;
            var viewPort = ej.datavisualization.Diagram.ScrollUtil._viewPort(this._parent);
            var viewwidth = (width / offwidth) * this.contentWidth;
            var viewheight = (height / offheight) * this.contentHeight;
            var zoom = Math.max(viewPort.width / viewwidth, viewPort.height / viewheight);
            if (zoom >= 0.25 && zoom <= 30) {
                this._updateView(zoom, x, y, width, height, new ej.datavisualization.Diagram.Point(0, 0));
                this._updateOverviewrect(x, y, width, height);
            }
        },
        handleTouchStart: function (evt) {
            evt.preventDefault();
        },
        handleTouchMove: function (evt) {
            evt.preventDefault();
        },
        handleTouchEnd: function (evt) {
            evt.preventDefault();
        },
        handleTouchLeave: function (evt) {
            evt.preventDefault();
        },
        //#endregion
    });
    //#endregion
})(jQuery, Syncfusion);;

(function ($, ej) {
    "use strict";

    //#region SVG
    var Canvas = (function () {
        function Canvas(attr) {
            this.document = document.createElement("canvas");
            this.id = attr.id;
            if (attr) {
                ej.datavisualization.Diagram.Util.attr(this.document, attr);
            }
            return this;
        }

        //rect
        Canvas.prototype.rect = function (x, y, width, height, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style) {
                this.setStyle(style, ctx);
                if (style.dashArray)
                    this.dashedRectangle(x, y, width, height, style.dashArray);
                else
                    ctx.strokeRect(x, y, width, height);
            }
            else ctx.strokeRect(x, y, width, height);
            ctx.fillRect(x, y, width, height);
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }
        //Rounded rect
        Canvas.prototype.roundedRect = function (x, y, width, height, radius, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            var points = [{ x: x + radius, y: y }, { x: x + width - radius, y: y },
                { x: x + width, y: y + radius }, { x: x + width, y: y + height - radius },
                { x: x + width - radius, y: y + height }, { x: x + radius, y: y + height },
                { x: x, y: y + height - radius }, { x: x, y: y + radius }
            ];
            var corners = [{ x: x + width, y: y }, { x: x + width, y: y + height }, { x: x, y: y + height }, { x: x, y: y }];
            var corner = 0;
            var point2, next;
            if (style)
                this.setStyle(style, ctx);
            //draw
            ctx.moveTo(points[0].x, points[0].y);
            var i;
            for (i = 0; i < points.length; i = i + 2) {
                point2 = points[i + 1];
                ctx.lineTo(point2.x, point2.y);
                next = points[i + 2] || points[0];
                ctx.quadraticCurveTo(corners[corner].x, corners[corner].y, next.x, next.y);
                corner++;
            }
            ctx.fill();
            if (style && style.dashArray) {
                for (i = 0; i < points.length; i = i + 2) {
                    this.dashedLine(points[i].x, points[i].y, points[i + 1].x, points[i + 1].y, style.dashArray);
                }
            }
            ctx.stroke();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }
        //ellipse
        Canvas.prototype.ellipse = function (cx, cy, rx, ry, rotation, style) {
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            var r = 4 * ((Math.sqrt(2) - 1) / 3);
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.moveTo(cx, cy - ry);
            ctx.bezierCurveTo(cx + (r * rx), cy - ry, cx + rx, cy - (r * ry), cx + rx, cy);
            ctx.bezierCurveTo(cx + rx, cy + (r * ry), cx + (r * rx), cy + ry, cx, cy + ry);
            ctx.bezierCurveTo(cx - (r * rx), cy + ry, cx - rx, cy + (r * ry), cx - rx, cy);
            ctx.bezierCurveTo(cx - rx, cy - (r * ry), cx - (r * rx), cy - ry, cx, cy - ry);
            ctx.fill();
            if (style && style.dashArray) {
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx, cy - ry),
                    new ej.datavisualization.Diagram.Point(cx + (r * rx), cy - ry),
                    new ej.datavisualization.Diagram.Point(cx + rx, cy - (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx + rx, cy), style);
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx + rx, cy),
                    new ej.datavisualization.Diagram.Point(cx + rx, cy + (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx + (r * rx), cy + ry),
                    new ej.datavisualization.Diagram.Point(cx, cy + ry), style);
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx, cy + ry),
                    new ej.datavisualization.Diagram.Point(cx - (r * rx), cy + ry),
                    new ej.datavisualization.Diagram.Point(cx - rx, cy + (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx - rx, cy), style);
                this.bezierCurve(ej.datavisualization.Diagram.Point(cx - rx, cy),
                    new ej.datavisualization.Diagram.Point(cx - rx, cy - (r * ry)),
                    new ej.datavisualization.Diagram.Point(cx - (r * rx), cy - ry),
                    new ej.datavisualization.Diagram.Point(cx, cy - ry), style);
            }
            ctx.stroke();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //circle
        Canvas.prototype.circle = function (centerx, centery, radius, rotation, style) {
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style && style.dashArray) {
                ctx.beginPath();
                ctx.arc(centerx, centery, radius, 0, 2 * Math.PI);
                ctx.fill();
                ctx.stroke();
                ctx.closePath();
            }
            else { this.ellipse(centerx, centery, radius * 2, radius * 2, rotation, style); }
            if (rotation && rotation.angle)
                ctx.restore();
        }
        Canvas.prototype.arc = function (centerx, centery, radius, angle, sweep, style) {
            var rotation = { "angle": angle, x: centerx, y: centery };
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.beginPath();
            var stAngle = 0;
            var endAngle = Math.PI;
            if (!sweep) {
                stAngle = Math.PI;
                endAngle = 0;
            }
            if (style && style.dashArray) {
                ctx.arc(centerx, centery, radius, stAngle, endAngle);
            }
            else { ctx.arc(centerx, centery, radius, stAngle, endAngle); }
            //ctx.fill();
            ctx.stroke();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //path
        Canvas.prototype.path = function (path, x, y, width, height, rotation, style) {
            var ctx = this.document.getContext("2d");
            if (style) {
                this.setStyle(style, ctx);
            }
            ctx.beginPath();
            var transX = x, transY = y;
            ctx.translate(transX, transY);
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            var shape = document.createElementNS('http://www.w3.org/2000/svg', "path");
            shape.setAttribute("d", path);
            this.document.appendChild(shape);
            var segments = this.getSegments(shape);
            var offsetX = x;
            var offsetY = y;
            var pathstr = "";
            var x0, y0, x1, y1, x2, y2, segs = segments;
            for (var x = 0, y = 0, i = 0, length = segs.length; i < length; ++i) {
                var seg = segs[i], char = seg.command;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;
                switch (char) {
                    case 'M':
                        ctx.moveTo(x, y); seg.x = x; seg.y = y;
                        break;
                    case 'L':
                        ctx.lineTo(x, y);
                        seg.x = x; seg.y = y;
                        break;
                    case 'C':
                        ctx.bezierCurveTo(x1, y1, x2, y2, x, y);
                        seg.x = x; seg.y = y; seg.x1 = x1; seg.y1 = y1; seg.x2 = x2; seg.y2 = y2;
                        break;
                    case 'Q':
                        ctx.quadraticCurveTo(x1, y1, x, y);
                        seg.x = x; seg.y = y; seg.x1 = x1; seg.y1 = y1;
                        break;
                    case 'A':
                        var curr = new ej.datavisualization.Diagram.Point(x0, y0);
                        var rx = seg.r1, ry = seg.r2;
                        var xAxisRotation = seg.angle * (Math.PI / 180.0);
                        var largeArcFlag = seg.largeArcFlag;
                        var sweepFlag = seg.sweepFlag;
                        var cp = new ej.datavisualization.Diagram.Point(x, y);

                        var currp = new ej.datavisualization.Diagram.Point(
                            Math.cos(xAxisRotation) * (curr.x - cp.x) / 2.0 + Math.sin(xAxisRotation) * (curr.y - cp.y) / 2.0,
                            -Math.sin(xAxisRotation) * (curr.x - cp.x) / 2.0 + Math.cos(xAxisRotation) * (curr.y - cp.y) / 2.0
                        );

                        var l = Math.pow(currp.x, 2) / Math.pow(rx, 2) + Math.pow(currp.y, 2) / Math.pow(ry, 2);
                        if (l > 1) {
                            rx *= Math.sqrt(l);
                            ry *= Math.sqrt(l);
                        }

                        var s = (largeArcFlag == sweepFlag ? -1 : 1) * Math.sqrt(
                            ((Math.pow(rx, 2) * Math.pow(ry, 2)) - (Math.pow(rx, 2) * Math.pow(currp.y, 2)) - (Math.pow(ry, 2) * Math.pow(currp.x, 2))) /
                            (Math.pow(rx, 2) * Math.pow(currp.y, 2) + Math.pow(ry, 2) * Math.pow(currp.x, 2))
                        );
                        if (isNaN(s)) s = 0;
                        var cpp = new ej.datavisualization.Diagram.Point(s * rx * currp.y / ry, s * -ry * currp.x / rx);

                        var centp = new ej.datavisualization.Diagram.Point(
                            (curr.x + cp.x) / 2.0 + Math.cos(xAxisRotation) * cpp.x - Math.sin(xAxisRotation) * cpp.y,
                            (curr.y + cp.y) / 2.0 + Math.sin(xAxisRotation) * cpp.x + Math.cos(xAxisRotation) * cpp.y
                        );
                        // vector magnitude
                        var m = function (v) { return Math.sqrt(Math.pow(v[0], 2) + Math.pow(v[1], 2)); }
                        // ratio between two vectors
                        var r = function (u, v) { return (u[0] * v[0] + u[1] * v[1]) / (m(u) * m(v)) }
                        // angle between two vectors
                        var a = function (u, v) { return (u[0] * v[1] < u[1] * v[0] ? -1 : 1) * Math.acos(r(u, v)); }
                        // initial angle
                        var a1 = a([1, 0], [(currp.x - cpp.x) / rx, (currp.y - cpp.y) / ry]);
                        // angle delta
                        var u = [(currp.x - cpp.x) / rx, (currp.y - cpp.y) / ry];
                        var v = [(-currp.x - cpp.x) / rx, (-currp.y - cpp.y) / ry];
                        var ad = a(u, v);
                        if (r(u, v) <= -1) ad = Math.PI;
                        if (r(u, v) >= 1) ad = 0;

                        var dir = 1 - sweepFlag ? 1.0 : -1.0;
                        var ah = a1 + dir * (ad / 2.0);
                        var halfWay = new ej.datavisualization.Diagram.Point(
                            centp.x + rx * Math.cos(ah),
                            centp.y + ry * Math.sin(ah)
                        );
                        seg.centp = centp;
                        seg.xAxisRotation = xAxisRotation;
                        seg.rx = rx;
                        seg.ry = ry;
                        seg.a1 = a1; seg.ad = ad; seg.sweepFlag = sweepFlag;
                        if (ctx != null) {
                            var r = rx > ry ? rx : ry;
                            var sx = rx > ry ? 1 : rx / ry;
                            var sy = rx > ry ? ry / rx : 1;
                            ctx.save();
                            ctx.translate(centp.x, centp.y);
                            ctx.rotate(xAxisRotation);
                            ctx.scale(sx, sy);
                            ctx.arc(0, 0, r, a1, a1 + ad, 1 - sweepFlag);
                            ctx.scale(1 / sx, 1 / sy);
                            ctx.rotate(-xAxisRotation);
                            ctx.translate(-centp.x, -centp.y);
                            ctx.restore();
                        }
                        break;
                    case 'Z':
                        ctx.closePath();
                        x = x0; y = y0;
                        break;
                }
                // Record the start of a subpath
                x0 = x, y0 = y;
            }
            ctx.fill();
            if (style && style.dashArray)
                this.drawDashedPath(segs, style);
            else
                ctx.stroke();
            ctx.translate(-transY, -transY);
            if (rotation && rotation.angle)
                ctx.restore();
        }

        Canvas.prototype.getSegments = function (shape) {
            var x0, y0, x1, y1, x2, y2, initx, inity, segs = shape.pathSegList;
            var segments = [];
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;
                switch (char) {
                    case 'M':
                        segments.push({ command: "M", x: x, y: y });
                        break;
                    case 'L':
                        segments.push({ command: "L", x0: x0, y0: y0, x: x, y: y });
                        break;
                    case 'H':
                        segments.push({ command: "L", x0: x0, y0: y0, x: x, y: y0 });
                        break;
                    case 'V':
                        segments.push({ command: "L", x0: x0, y0: y0, x: x0, y: y });
                        break;
                    case 'C':
                        segments.push({ command: "C", x0: x0, y0: y0, x1: x1, y1: y1, x2: x2, y2: y2, x: x, y: y });
                        break;
                    case 'S':
                        if (prev) {
                            if (prev.command == "C" || prev.command == "S") {
                                var ctrl = { x: prev.x2, y: prev.y2 };
                            }
                            else
                                var ctrl = { x: prev.x0, y: prev.y0 };
                            var cpt2 = new ej.datavisualization.Diagram.Point(2 * x0 - ctrl.x, 2 * y0 - ctrl.y);

                            segments.push({ command: "C", x0: x0, y0: y0, x1: cpt2.x, y1: cpt2.y, x2: x2, y2: y2, x: x, y: y });
                        }
                        break;
                    case 'Q':
                        //ctx.quadraticCurveTo(x1, y1, x, y);
                        segments.push({ command: "Q", x0: x0, y0: y0, x1: x1, y1: y1, x: x, y: y });
                        break;
                    case 'T':
                        if (prev) {
                            if (prev.command == "Q") {
                                var ctrl = { x: prev.x1, y: prev.y1 };
                            }
                            else
                                var ctrl = { x: prev.x0, y: prev.y0 };
                            var cpt2 = new ej.datavisualization.Diagram.Point(2 * x0 - ctrl.x, 2 * y0 - ctrl.y);

                            segments.push({ command: "Q", x0: x0, y0: y0, x1: cpt2.x, y1: cpt2.y, x: x, y: y });
                        }
                        break;
                    case 'A'://need to update for "A"
                        var newSeg = $.extend(true, {}, seg);
                        newSeg.command = "A";
                        segments.push(newSeg);
                        break;
                    case 'Z':
                        segments.push({ command: "L", x0: x, y0: y, x: initx, y: inity });
                        x = x0; y = y0;
                        break;
                }
                var prev = segments[segments.length - 1];
                // Record the start of a subpath
                if (char === 'M' || char === 'm') {
                    initx = x, inity = y;
                }
                x0 = x, y0 = y;
            }
            return segments;
        }

        Canvas.prototype.drawDashedPath = function (segs, style) {
            var x1, y1, x2, y2, x0, y0;
            var initial;
            for (var x = 0, y = 0, i = 0, length = segs.length; i < length; ++i) {
                var seg = segs[i], char = seg.command;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;
                switch (char) {
                    case 'M':
                        initial = { x: x, y: y };
                        break;
                    case 'L':
                        this.dashedLine(x0, y0, x, y, style.dashArray);
                        break;
                    case 'H':
                        this.dashedLine(x0, y0, x, y0, style.dashArray);
                        break;
                    case 'V':
                        this.dashedLine(x0, y0, x0, y, style.dashArray);
                        break;
                    case 'C':
                        this.bezierCurve(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x2, y2), ej.datavisualization.Diagram.Point(x, y), style);
                        break;
                    case 'Q':
                        this.quadraticCurve(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x, y), style);
                        break;
                    case 'A':
                        var ctx = this.document.getContext("2d");
                        if (ctx != null) {
                            var r = seg.rx > seg.ry ? seg.rx : seg.ry;
                            var sx = seg.rx > seg.ry ? 1 : seg.rx / seg.ry;
                            var sy = seg.rx > seg.ry ? seg.ry / seg.rx : 1;
                            ctx.save();
                            ctx.translate(seg.centp.x, seg.centp.y);
                            ctx.rotate(seg.xAxisRotation);
                            ctx.scale(sx, sy);
                            this.dashedArc(r, 0, 0, seg.a1, seg.a1 + seg.ad, 1 - seg.sweepFlag, style.dashArray);
                            ctx.scale(1 / sx, 1 / sy);
                            ctx.rotate(-seg.xAxisRotation);
                            ctx.translate(-seg.centp.x, -seg.centp.y);
                            ctx.restore();
                        }
                        break;
                    case 'Z':
                        //ctx.closePath();
                        this.dashedLine(x, y, initial.x, initial.y, style.dashArray);
                        x = x0; y = y0;
                        break;
                }
                // Record the start of a subpath
                //if (char === 'M' || char === 'm') x0 = x, y0 = y;
                x0 = x, y0 = y;
            }
            //ctx.fill();
        }
        Canvas.prototype.calculateCirclePoints = function (radius, cx, cy, stAngle, endAngle, couterClockwise, dashSize) {
            stAngle = stAngle * 180 / Math.PI;
            endAngle = endAngle * 180 / Math.PI;
            var n = (2 * Math.PI * radius) * (Math.abs(stAngle - endAngle) / 360) * 0.7;
            var alpha = Math.abs(stAngle - endAngle) / n;
            alpha = couterClockwise ? alpha * -1 : alpha;
            var i = 0;
            var points = [];
            while (i < n) {
                points.push(ej.datavisualization.Diagram.Geometry.transform({ x: cx, y: cy }, stAngle + alpha * i, radius));
                i++;
            }
            return points;
        }

        //polygon
        Canvas.prototype.polygon = function (points, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style) {
                this.setStyle(style, ctx);
            }
            ctx.moveTo(points[0].x, points[0].y);
            for (var i = 1; i < points.length; i++) {
                ctx.lineTo(points[i].x, points[i].y);
            }
            ctx.lineTo(points[0].x, points[0].y);
            ctx.fill();
            if (style && style.dashArray) {
                for (var i = 1; i < points.length; i++) {
                    this.line(points[i - 1], points[i], style);
                }
                this.line(points[i - 1], points[0], style);
            }
            ctx.stroke();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //image
        Canvas.prototype.image = function (image, x, y, width, height, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style) {
                this.setStyle(style, ctx);
            }
            if (width && height)
                ctx.drawImage(image, x, y, width, height);
            else
                ctx.drawImage(image, x, y);
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //text
        Canvas.prototype.text = function (text, x, y, rotation, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            if (style)
                this.setStyle(style, ctx);
            ctx.fillText(text.text, x, y);
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //line
        Canvas.prototype.line = function (start, end, style, dontstart, dontclose) {
            var ctx = this.document.getContext("2d");
            if (!dontstart) {
                ctx.beginPath();
                if (style) {
                    this.setStyle(style, ctx);
                }
            }
            if (style && style.dashArray) {
                this.dashedLine(start.x, start.y, end.x, end.y, style.dashArray);
            }
            else {
                if (!dontstart)
                    ctx.moveTo(start.x, start.y);
                ctx.lineTo(end.x, end.y);
                ctx.stroke();
            }
            if (!dontclose)
                ctx.closePath();
        }

        //polyline
        Canvas.prototype.polyline = function (points, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (style) {
                this.setStyle(style, ctx);
            }
            for (var i = 1; i < points.length; i++) {
                this.line(points[i - 1], points[i], style);
            }
            ctx.stroke();
        }

        //bezier curve
        Canvas.prototype.bezierCurve = function (startPoint, cpt1, cpt2, cpt3, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            ctx.moveTo(startPoint.x, startPoint.y);
            if (style) {
                this.setStyle(style, ctx);
            }
            if (style && style.dashArray) {
                var points = [];
                this.flattenCubicBezier(points, startPoint, cpt1, cpt2, cpt3);
                var dashLength = 0;
                for (var i = 0; i < style.dashArray.length; i++) {
                    dashLength += Number(style.dashArray[i]);
                }
                var dashcount = points.length / dashLength;
                var draw = true;
                var drawn = 0;
                for (var i = 0; i < dashcount; i++) {
                    for (var j = 0; j < style.dashArray.length; j++) {
                        drawn += Number(style.dashArray[j]);
                        if (drawn < points.length) {
                            if (draw) {
                                ctx.lineTo(points[drawn].x, points[drawn].y);
                            }
                            else ctx.moveTo(points[drawn].x, points[drawn].y);
                        }
                        draw = !draw;
                    }
                }


            }
            else {
                ctx.moveTo(startPoint.x, startPoint.y);
                ctx.bezierCurveTo(cpt1.x, cpt1.y, cpt2.x, cpt2.y, cpt3.x, cpt3.y);
            }
            ctx.stroke();
        }

        Canvas.prototype.quadraticCurve = function (startPoint, cpt1, cpt2, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            ctx.moveTo(startPoint.x, startPoint.y);
            if (style) {
                this.setStyle(style, ctx);
            }
            if (style && style.dashArray) {
                var points = [];
                this.flattenQuadraticBezier(points, startPoint, cpt1, cpt2);
                var dashLength = 0;
                for (var i = 0; i < style.dashArray.length; i++) {
                    dashLength += Number(style.dashArray[i]);
                }
                var dashcount = points.length / dashLength;
                var draw = true;
                var drawn = 0;
                for (var i = 0; i < dashcount; i++) {
                    for (var j = 0; j < style.dashArray.length; j++) {
                        drawn += Number(style.dashArray[j]);
                        if (drawn < points.length) {
                            if (draw) {
                                ctx.lineTo(points[drawn].x, points[drawn].y);
                            }
                            else ctx.moveTo(points[drawn].x, points[drawn].y);
                        }
                        draw = !draw;
                    }
                }

            }
            else {
                ctx.moveTo(startPoint.x, startPoint.y);
                ctx.quadraticCurveTo(cpt1.x, cpt1.y, cpt2.x, cpt2.y);
            }
            ctx.stroke();
        }

        //arrow
        Canvas.prototype.arrow = function (point, rotation, size, isOpen, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.moveTo(point.x + size.width, point.y + size.height / 2);
            ctx.lineTo(point.x, point.y);
            ctx.lineTo(point.x + size.width, point.y - size.height / 2);
            if (!isOpen) {
                ctx.closePath();
                ctx.fill();
            }
            ctx.stroke();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        //diamond
        Canvas.prototype.diamond = function (point, rotation, size, style) {
            var ctx = this.document.getContext("2d");
            ctx.beginPath();
            if (style) {
                this.setStyle(style, ctx);
            }
            if (rotation && rotation.angle) {
                this.rotateContext(rotation, ctx);
            }
            ctx.moveTo(point.x + size.width, point.y);
            ctx.lineTo(point.x + size.width / 2, point.y + size.height / 2);
            ctx.lineTo(point.x, point.y);
            ctx.lineTo(point.x + size.width / 2, point.y - size.height / 2);
            ctx.lineTo(point.x + size.width, point.y);
            ctx.stroke();
            ctx.fill();
            ctx.closePath();
            if (rotation && rotation.angle)
                ctx.restore();
        }

        Canvas.prototype.dashedLine = function (x1, y1, x2, y2, dashArray) {
            var ctx = this.document.getContext("2d");
            ctx.save();
            var dx = (x2 - x1), dy = (y2 - y1);
            var length = Math.sqrt(dx * dx + dy * dy);
            var angle = Math.atan2(dy, dx);
            ctx.beginPath();
            ctx.translate(x1, y1);
            ctx.moveTo(0, 0);
            ctx.rotate(angle);
            var dcount = dashArray.length;
            var dindex = 0, draw = true;
            var x = 0;
            while (length > x) {
                x += Number(dashArray[dindex++ % dcount]);
                if (x > length) x = length;
                draw ? ctx.lineTo(x, 0) : ctx.moveTo(x, 0);
                draw = !draw;
                ctx.stroke();
            }
            ctx.restore();
        }

        Canvas.prototype.dashedRectangle = function (x, y, width, height, dashArray) {
            this.dashedLine(x, y, x + width, y, dashArray);
            this.dashedLine(x + width, y, x + width, y + height, dashArray);
            this.dashedLine(x + width, y + height, x, y + height, dashArray);
            this.dashedLine(x, y + height, x, y, dashArray);
        }

        Canvas.prototype.dashedArc = function (radius, cx, cy, stAngle, endAngle, couterClockwise, dashArray) {
            var dashLength = 0;
            for (var i = 0; i < dashArray.length; i++) {
                dashLength += Number(dashArray[i]);
            }
            var points = this.calculateCirclePoints(radius, cx, cy, stAngle, endAngle, couterClockwise, dashLength);
            var ctx = this.document.getContext("2d");
            ctx.beginPath();

            var dashcount = points.length / dashLength;
            var draw = true;
            var drawn = 0;
            for (var i = 0; i < dashcount; i++) {
                for (var j = 0; j < dashArray.length; j++) {
                    drawn += Number(dashArray[j]);
                    if (drawn < points.length) {
                        if (draw) {
                            ctx.lineTo(points[drawn].x, points[drawn].y);
                            ctx.stroke();
                        }
                        else ctx.moveTo(points[drawn].x, points[drawn].y);
                    }
                    draw = !draw;
                }
            }
            ctx.closePath();
        }
        Canvas.prototype.absolutePath = function (path) {
            var minx, miny, maxx, maxy;
            var x0, y0, x1, y1, x2, y2, segs = path.pathSegList;
            for (var x = 0, y = 0, i = 0, length = segs.numberOfItems; i < length; ++i) {
                var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if (/[MLHVCSQTA]/.test(char)) {
                    if ('x' in seg) x = seg.x;
                    if ('y' in seg) y = seg.y;
                } else {
                    if ('x1' in seg) x1 = x + seg.x1;
                    if ('x2' in seg) x2 = x + seg.x2;
                    if ('y1' in seg) y1 = y + seg.y1;
                    if ('y2' in seg) y2 = y + seg.y2;
                    if ('x' in seg) x += seg.x;
                    if ('y' in seg) y += seg.y;
                    switch (char) {
                        case 'm': segs.replaceItem(path.createSVGPathSegMovetoAbs(x, y), i); break;
                        case 'l': segs.replaceItem(path.createSVGPathSegLinetoAbs(x, y), i); break;
                        case 'h': segs.replaceItem(path.createSVGPathSegLinetoHorizontalAbs(x), i); break;
                        case 'v': segs.replaceItem(path.createSVGPathSegLinetoVerticalAbs(y), i); break;
                        case 'c': segs.replaceItem(path.createSVGPathSegCurvetoCubicAbs(x, y, x1, y1, x2, y2), i); break;
                        case 's': segs.replaceItem(path.createSVGPathSegCurvetoCubicSmoothAbs(x, y, x2, y2), i); break;
                        case 'q': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticAbs(x, y, x1, y1), i); break;
                        case 't': segs.replaceItem(path.createSVGPathSegCurvetoQuadraticSmoothAbs(x, y), i); break;
                        case 'a': segs.replaceItem(path.createSVGPathSegArcAbs(x, y, seg.r1, seg.r2, seg.angle, seg.largeArcFlag, seg.sweepFlag), i); break;
                        case 'z': case 'Z': x = x0; y = y0; break;
                    }
                }
                if (char === 'M' || char === 'm') x0 = x, y0 = y;
            }

            Canvas.prototype._scalePathData = function (val, scaleFactor, oldOffset, newOffset) {
                if (val !== oldOffset) {
                    if (newOffset !== oldOffset) {
                        val = (((val * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)))
                            + (newOffset - Number(oldOffset)));
                    }
                    else
                        val = ((Number(val) * scaleFactor) - (Number(oldOffset) * scaleFactor - Number(oldOffset)));
                }
                else {
                    if (newOffset !== oldOffset) {
                        val = newOffset;
                    }
                }
                return val;
            }
            return path;
        };

        Canvas.prototype.setStyle = function (style, ctx) {
            if (style.fill)
                ctx.fillStyle = style.fill;
            if (style.stroke)
                ctx.strokeStyle = style.stroke == "none" ? "transparent" : style.stroke;
            if (style.font)
                ctx.font = style.font;
            if (style.lineWidth)
                ctx.lineWidth = style.lineWidth;
        }

        Canvas.prototype.rotateContext = function (rotation, ctx) {
            ctx.save();
            ctx.translate(rotation.x, rotation.y);
            ctx.rotate(rotation.angle * Math.PI / 180);
            ctx.translate(-rotation.x, -rotation.y);
        }

        Canvas.prototype.pathBounds = function (segments) {
            //var segs = shape.pathSegList;
            var x1, y1, x2, y2, x0, y0;
            var minx, miny, maxx, maxy;
            for (var x = 0, y = 0, i = 0, length = segments.length; i < length; ++i) {
                var seg = segments[i], char = seg.command;
                if (i == 0 && seg.x && seg.y) {
                    minx = maxx = seg.x;
                    miny = maxy = seg.y;
                }
                //var seg = segs.getItem(i), char = seg.pathSegTypeAsLetter;
                if ('x1' in seg) x1 = seg.x1;
                if ('x2' in seg) x2 = seg.x2;
                if ('y1' in seg) y1 = seg.y1;
                if ('y2' in seg) y2 = seg.y2;
                if ('x' in seg) x = seg.x;
                if ('y' in seg) y = seg.y;

                switch (char) {
                    case 'M':
                        x0 = x; y0 = y;
                    case 'L':
                    case 'H':
                    case 'V':
                        if (x) {
                            minx = Math.min(x, minx);
                            maxx = Math.max(x, maxx);
                        }
                        if (y) {
                            miny = Math.min(y, miny);
                            maxy = Math.max(y, maxy);
                        }
                        break;
                    case 'C':

                    case 'S':
                        var bounds = this.getBezierBounds(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x2, y2), ej.datavisualization.Diagram.Point(x, y));
                        minx = Math.min(bounds.x, minx);
                        maxx = Math.max(bounds.x + bounds.width, maxx);
                        miny = Math.min(bounds.y, miny);
                        maxy = Math.max(bounds.y + bounds.height, maxy);
                        break;
                    case 'Q':
                    case "q":
                    case 'T':
                        var bounds = this.getQuadraticBounds(ej.datavisualization.Diagram.Point(x0, y0), ej.datavisualization.Diagram.Point(x1, y1), ej.datavisualization.Diagram.Point(x, y));
                        minx = Math.min(bounds.x, minx);
                        maxx = Math.max(bounds.x + bounds.width, maxx);
                        miny = Math.min(bounds.y, miny);
                        maxy = Math.max(bounds.y + bounds.height, maxy);
                        break;
                    case 'A':
                        break;
                    case 'Z':
                        //ctx.closePath();
                        x = x0; y = y0;
                        break;
                }
            }
            return { x: minx, y: miny, width: maxx - minx, height: maxy - miny };
        };

        Canvas.prototype.flattenCubicBezier = function (points, ptStart, ptCtrl1, ptCtrl2, ptEnd, tolerance) {
            var tolerance = 1.5;

            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                              ej.datavisualization.Diagram.Geometry.distance(ptCtrl2, ptCtrl1) +
                              ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl2)) / tolerance);

            for (var i = 0; i <= max; i++) {
                var t = i / max;

                var x = (1 - t) * (1 - t) * (1 - t) * ptStart.x +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.x +
                           3 * t * t * (1 - t) * ptCtrl2.x +
                           t * t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * (1 - t) * ptStart.y +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.y +
                           3 * t * t * (1 - t) * ptCtrl2.y +
                           t * t * t * ptEnd.y;

                points.push(new ej.datavisualization.Diagram.Point(x, y));
            }
        }

        Canvas.prototype.flattenQuadraticBezier = function (points, ptStart, ptCtrl1, ptEnd, tolerance) {
            var tolerance = 1.5;

            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                             ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl1)) / tolerance);

            for (var i = 0; i <= max; i++) {
                var t = i / max;

                var x = (1 - t) * (1 - t) * ptStart.x +
                                          2 * t * (1 - t) * ptCtrl1.x +
                                          t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * ptStart.y +
                           2 * t * (1 - t) * ptCtrl1.y +
                           t * t * ptEnd.y;

                points.push(new ej.datavisualization.Diagram.Point(x, y));
            }
        }

        Canvas.prototype.getBezierBounds = function (ptStart, ptCtrl1, ptCtrl2, ptEnd, tolerance) {
            var tolerance = 3;
            var minx, miny, maxx, maxy;
            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                              ej.datavisualization.Diagram.Geometry.distance(ptCtrl2, ptCtrl1) +
                              ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl2)) / tolerance);

            for (var i = 0; i <= max; i++) {

                var t = i / max;

                var x = (1 - t) * (1 - t) * (1 - t) * ptStart.x +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.x +
                           3 * t * t * (1 - t) * ptCtrl2.x +
                           t * t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * (1 - t) * ptStart.y +
                           3 * t * (1 - t) * (1 - t) * ptCtrl1.y +
                           3 * t * t * (1 - t) * ptCtrl2.y +
                           t * t * t * ptEnd.y;
                if (i == 0) {
                    minx = maxx = x;
                    miny = maxy = y;
                }
                else {
                    minx = Math.min(x, minx);
                    miny = Math.min(y, miny);
                    maxx = Math.max(x, maxx);
                    maxy = Math.max(y, maxy);
                }
            }
            return { x: minx, y: miny, width: maxx - minx, height: maxy - miny };
        }

        Canvas.prototype.getQuadraticBounds = function (ptStart, ptCtrl1, ptEnd, tolerance) {
            var tolerance = 3;
            var minx, miny, maxx, maxy;
            var max = Number((ej.datavisualization.Diagram.Geometry.distance(ptCtrl1, ptStart) +
                              ej.datavisualization.Diagram.Geometry.distance(ptEnd, ptCtrl1)) / tolerance);

            for (var i = 0; i <= max; i++) {
                var t = i / max;


                var x = (1 - t) * (1 - t) * ptStart.x +
                           2 * t * (1 - t) * ptCtrl1.x +
                           t * t * ptEnd.x;

                var y = (1 - t) * (1 - t) * ptStart.y +
                           2 * t * (1 - t) * ptCtrl1.y +
                           t * t * ptEnd.y;

                if (i == 0) {
                    minx = maxx = x;
                    miny = maxy = y;
                }
                else {
                    minx = Math.min(x, minx);
                    miny = Math.min(y, miny);
                    maxx = Math.max(x, maxx);
                    maxy = Math.max(y, maxy);
                }
            }
            return { x: minx, y: miny, width: maxx - minx, height: maxy - miny };
        }

        return Canvas;
    })();

    ej.datavisualization.Diagram.Canvas = Canvas;
    //#endregion

    //#region Renderer
    ej.datavisualization.Diagram.CanvasContext = {
        _renderDocument: function (view, diagram) {
            var attr = {
                id: view.canvas.id + "_canvas",
                width: diagram._canvas.clientWidth,
                height: diagram._canvas.clientHeight,
                version: "1.1",
                xlink: "http://www.w3.org/1999/xlink",
                "style": view.style,
                "pointer-events": "none"
            };
            view._canvas = new ej.datavisualization.Diagram.Canvas(attr);
            diagram._view = view.canvas;
            view.canvas.appendChild(view._canvas.document);
            if (view.type == "mainview") {
                this._canvas = view.canvas;
                diagram._svg = this._svg = view._canvas;
            }
        },

        _renderGroupBackground: function (group, canvas, diagram) {
            if (!group.pivot) {
                group.pivot = {};
                group.pivot.x = .5;
                group.pivot.y = .5;
            }

            var ctx = canvas.document.getContext("2d");
            if (group.visible) {
                var rotation = { "angle": group.rotateAngle, x: group.offsetX, y: group.offsetY };
                ctx.save();
                ctx.translate(rotation.x, rotation.y);
                ctx.rotate(rotation.angle * Math.PI / 180);
                ctx.translate(-rotation.x, -rotation.y);
                ctx.fillStyle = group.type != "bpmn" ? group.fillColor : "transparent";
                ctx.strokeStyle = group.type != "bpmn" ? group.borderColor : "transparent";
                ctx.lineWidth = group.borderWidth;
                if (group.borderDashArray) {
                    if (group.borderDashArray.indexOf("[") != -1 || group.borderDashArray instanceof Array) {
                        dashArray = group.borderDashArray;
                    } else if (group.borderDashArray.indexOf(",") != -1) {
                        var dashArray = group.borderDashArray.split(",");
                    } else if (group.borderDashArray.indexOf(" ") != -1) {
                        var dashArray = group.borderDashArray.split(" ");
                    }
                    if (ctx.setLineDash) {
                        ctx.setLineDash(dashArray);
                    } else if (ctx.mozDash) {
                        ctx.mozDash = dashArray;
                    } else {
                        var style = { dashArray: dashArray };
                    }
                }
            }

            var x = group.offsetX - group.width * group.pivot.x;
            var y = group.offsetY - group.height * group.pivot.y;
            canvas.rect(x, y, group.width, group.height, null, style);
            ctx.restore();
        },

        renderGroup: function (group, canvas, diagram) {
            if (group.visible) {
                var ctx = canvas.document.getContext("2d");
                var rotation = { "angle": group.rotateAngle, x: group.offsetX, y: group.offsetY };
                ctx.save();
                this._renderGroupBackground(group, canvas, diagram);
                for (var i = 0; i < group.children.length; i++) {
                    var child = diagram.nameTable[diagram._getChild(group.children[i])];
                    if (child) {
                        if (child._type === "group") {
                            this.renderGroup(child, canvas, diagram);
                        }
                        else if (child._type === "node") {
                            this.renderNode(child, canvas);
                        }
                        else if (child.segments) {
                            this.renderConnector(child, canvas);
                        }
                    }
                }
                ctx.translate(rotation.x, rotation.y);
                ctx.rotate(rotation.angle * Math.PI / 180);
                ctx.translate(-rotation.x, -rotation.y);
                var x = group.offsetX - group.width * group.pivot.x;
                var y = group.offsetY - group.height * group.pivot.y;
                if (!group.isLane) {
                    this.renderLabels(group, canvas);
                    this.renderPorts(group, canvas, x, y);
                }
                ctx.restore();
            }
        },
        _isExportable: function (node) {
            var state = true, content;
            if (node.type === "image") {
                try {
                    var canvas = new ej.datavisualization.Diagram.Canvas({ "id": "temp_canvas", "width": 100, "height": 100 });
                    var ctx = canvas.document.getContext("2d");
                    ctx.save();
                    canvas.rect(0, 0, node.width, node.height);
                    var image = new Image();
                    image.src = node.source;
                    canvas.image(image, 0, 0, node.width, node.height);
                    ctx.restore();
                    content = canvas.document.toDataURL();
                }
                catch (e) {
                    state = false;
                }
            }
            return state;
        },

        renderNode: function (node, canvas) {
            var ctx = canvas.document.getContext("2d");
            if (node.visible && this._isExportable(node)) {
                var nodeWidth = node.width || node._width || 0;
                var nodeHeight = node.height || node._height || 0;
                var rotation = { "angle": node.rotateAngle, x: node.offsetX, y: node.offsetY };
                ctx.save();
                ctx.translate(rotation.x, rotation.y);
                ctx.rotate(rotation.angle * Math.PI / 180);
                ctx.translate(-rotation.x, -rotation.y);
                ctx.save();
                ctx.fillStyle = node.fillColor;
                ctx.strokeStyle = node.borderColor == "none" ? "transparent" : node.borderColor;
                ctx.lineWidth = node.borderWidth;
                ctx.globalAlpha = node.opacity;
                if (node.borderDashArray) {
                    if (node.borderDashArray.indexOf("[") != -1 || node.borderDashArray instanceof Array) {
                        dashArray = node.borderDashArray;
                    }
                    else
                        if (node.borderDashArray.indexOf(",") != -1) {
                            var dashArray = node.borderDashArray.split(",");
                        }
                        else if (node.borderDashArray.indexOf(" ") != -1) { var dashArray = node.borderDashArray.split(" "); }
                    if (ctx.setLineDash) {
                        ctx.setLineDash(dashArray);
                    }
                    else if (ctx.mozDash) { ctx.mozDash = dashArray; }
                    else
                    {
                        var style = { dashArray: dashArray };
                    }
                }
                var x = node.offsetX - nodeWidth * node.pivot.x;
                var y = node.offsetY - nodeHeight * node.pivot.y;
                if (node._shape == "path")
                    this._renderGradient(node, ctx, 0, 0);
                else
                    this._renderGradient(node, ctx, x, y);
                var fill = node.fillColor;
                if (node.constraints & ej.datavisualization.Diagram.NodeConstraints.Shadow) {
                    ctx.save();
                    ctx.strokeStyle = "lightgrey";
                    var t = ej.datavisualization.Diagram.Geometry.transform({ x: 0, y: 0 }, node.shadow.angle, node.shadow.distance);
                    ctx.fillStyle = fill != "none" || fill != "transparent" ? "lightgrey" : "none";
                    switch (node._shape) {
                        case "image":
                        case "rectangle":
                            if (node.cornerRadius)
                                canvas.roundedRect(x + t.x, y + t.y, nodeWidth, nodeHeight, node.cornerRadius, null);
                            else canvas.rect(x + t.x, y + t.y, nodeWidth, nodeHeight, null);
                            break;
                        case "ellipse":
                            canvas.ellipse(x + nodeWidth / 2 + t.x, y + nodeHeight / 2 + t.y, nodeWidth / 2, nodeHeight / 2, null);
                            break;
                        case "path":
                            canvas.path(node._absolutePath, x + t.x, y + t.y, nodeWidth, nodeHeight, null);
                            break;
                        case "polygon":
                            this._updatePolygonPoints(node);
                            ctx.translate(t.x, t.y);
                            canvas.polygon(node.points, null, style);
                            break;
                    }
                    ctx.restore();
                }

                switch (node._shape) {
                    case "rectangle":
                        if (node.cornerRadius)
                            canvas.roundedRect(x, y, nodeWidth, nodeHeight, node.cornerRadius, null, style);
                        else canvas.rect(x, y, nodeWidth, nodeHeight, null, style);
                        break;
                    case "ellipse":
                        canvas.ellipse(x + nodeWidth / 2, y + nodeHeight / 2, nodeWidth / 2, nodeHeight / 2, null, style);
                        break;
                    case "path":
                        canvas.path(node._absolutePath, x, y, nodeWidth, nodeHeight, null, style);
                        break;
                    case "image":
                        var rotation = { angle: node.rotateAngle, x: node.offsetX, y: node.offsetY };
                        canvas.rect(x, y, nodeWidth, nodeHeight, null, style);
                        var image = new Image();
                        image.src = node.source;
                        canvas.image(image, x, y, nodeWidth, nodeHeight, null);
                        break;
                    case "text":
                        canvas.rect(x, y, nodeWidth, nodeHeight, null, style);
                        if (node.textBlock) {
                            var font = "";
                            if (node.textBlock.italic)
                                font += "italic ";
                            if (node.textBlock.bold)
                                font += "bold ";
                            font += node.textBlock.fontSize + " ";
                            font += node.textBlock.fontFamily;
                            ctx.font = font;
                            this._renderTextElement(node, node.textBlock, canvas);
                        }
                        break;
                    case "polygon":
                        this._updatePolygonPoints(node);
                        canvas.polygon(node.points, null, style);
                        break;
                }
                ctx.restore();
                if (node.type != "text")
                    this.renderLabels(node, canvas);
                this.renderPorts(node, canvas, x, y);
                ctx.restore();
            }
        },

        renderPhases: function (phase, canvas, points) {
            var style;
            if (phase && canvas) {
                var ctx = canvas.document.getContext("2d");
                ctx.save();
                ctx.strokeStyle = phase.lineColor ? phase.lineColor : "black";
                ctx.lineWidth = phase.lineWidth ? phase.lineWidth : 1;
                if (phase.lineDashArray) {
                    if (phase.lineDashArray.indexOf("[") != -1 || phase.lineDashArray instanceof Array) {
                        dashArray = phase.lineDashArray;
                    }
                    else {
                        if (phase.lineDashArray.indexOf(",") != -1) {
                            var dashArray = phase.lineDashArray.split(",");
                        }
                        else if (phase.lineDashArray.indexOf(" ") != -1) { var dashArray = phase.lineDashArray.split(" "); }
                    }

                    if (ctx.setLineDash) { ctx.setLineDash(dashArray); }
                    else if (ctx.mozDash) { ctx.mozDash = dashArray; }
                    else
                        style = { "dashArray": dashArray }
                }
                if (points && points.length > 0) {
                    canvas.line(points[0], points[1], style);
                }
            }
        },

        renderConnector: function (connector, canvas) {
            if (connector.visible) {
                var ctx = canvas.document.getContext("2d");
                ctx.save();
                ctx.strokeStyle = connector.lineColor;
                ctx.lineWidth = connector.lineWidth;
                if (connector.lineDashArray) {
                    if (connector.lineDashArray.indexOf("[") != -1 || connector.lineDashArray instanceof Array) {
                        dashArray = connector.lineDashArray;
                    }
                    else {
                        if (connector.lineDashArray.indexOf(",") != -1) {
                            var dashArray = connector.lineDashArray.split(",");
                        }
                        else if (connector.lineDashArray.indexOf(" ") != -1) { var dashArray = connector.lineDashArray.split(" "); }
                    }
                    if (ctx.setLineDash) { ctx.setLineDash(dashArray); }
                    else if (ctx.mozDash) { ctx.mozDash = dashArray; }
                    else
                        var style = { "dashArray": dashArray }
                }
                if (connector.segments) {
                    this._renderSegments(connector, canvas, style);
                }
                ctx.restore();
                this._renderDecorators(connector, canvas);
                this.renderLabels(connector, canvas);
            }
        },

        _renderSegments: function (connector, canvas, style) {
            for (var i = 0; i < connector.segments.length; i++) {
                var segment = connector.segments[i];
                if (segment._bridges.length > 0) {
                    for (var n = 0; n < segment._bridges.length; n++) {
                        var bridge = segment._bridges[n];
                        bridge._rendered = false;
                    }
                }
                var points = segment.points;
                var diagram = canvas._diagram;
                if (i == 0) {
                    if (diagram && connector.sourceNode) var node = diagram._findNode(connector.sourceNode);
                    points = ej.datavisualization.Diagram.SvgContext._clipDecorators(connector, segment, true, node);
                    if (segment.type == "bezier" && connector.sourceDecorator && connector.sourceDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                        points[0] = ej.datavisualization.Diagram.Util._adjustPoint(points[0], segment._point1, true, connector.lineWidth);
                    }
                    var startPoint = points[0];
                    var starting = true;
                }
                else starting = false;
                if (i == connector.segments.length - 1) {
                    if (diagram && connector.targetNode) var node = diagram._findNode(connector.targetNode);
                    points = ej.datavisualization.Diagram.SvgContext._clipDecorators(connector, segment, false, node);
                    if (startPoint) {
                        points[0] = startPoint; startPoint = null;
                    }
                    var ending = true;
                }
                if (segment.type == "bezier") {
                    var startPoint = points[0];
                    var endPoint = segment._endPoint;
                    if (connector.targetDecorator && connector.targetDecorator.shape !== ej.datavisualization.Diagram.DecoratorShapes.None) {
                        endPoint = ej.datavisualization.Diagram.Util._adjustPoint(segment._endPoint, segment._point2, true, connector.lineWidth);
                    }
                    canvas.bezierCurve(startPoint, segment._point1, segment._point2, endPoint, style);
                }
                else {
                    if (connector.cornerRadius > 0) {
                        var st, end;
                        for (var j = 0; j < points.length - 1; j++) {
                            var segLength = ej.datavisualization.Diagram.Geometry.distance(points[j], points[j + 1]);
                            if (segLength > 0) {
                                var endpoint = end || st;
                                if (i < connector.segments.length - 1 || j < points.length - 2) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        end = this._adjustPoint(points[j], points[j + 1], false, segLength / 2);
                                    }
                                    else end = this._adjustPoint(points[j], points[j + 1], false, connector.cornerRadius);
                                }
                                else end = points[j + 1];

                                if (i > 0 || j > 0) {
                                    if (segLength < connector.cornerRadius * 2) {
                                        st = this._adjustPoint(points[j], points[j + 1], true, segLength / 2);
                                        if (i < connector.segments.length - 1 || j < points.length - 2)
                                            end = null;
                                    }
                                    else st = this._adjustPoint(points[j], points[j + 1], true, connector.cornerRadius);
                                }

                                if (endpoint && st) {
                                    canvas.quadraticCurve(endpoint, points[j], st, style);
                                }
                                else st = points[j];
                                if (segment._bridges.length > 0) {
                                    this._updateBridging(segment, canvas, j + 1, st ? st : points[j], end ? end : points[j + 1], connector.bridgeSpace, style);
                                }
                                if (end) {
                                    if (!(segment._bridges.length > 0)) {
                                        canvas.line(st, end, style);
                                    }
                                }
                            }
                        }
                    }
                    else {
                        for (var k = 1; k < points.length; k++) {
                            if (segment._bridges.length > 0) {
                                this._updateBridging(segment, canvas, k, points[k - 1], points[k], connector.bridgeSpace, style);
                            }
                            else
                                canvas.line(points[k - 1], points[k], style,
                                    starting && k == 1 ? false : true, ending && k == points.length - 1 ? false : true);
                        }
                    };
                }
            }
        },

        _updateBridging: function (seg, canvas, pointIndex, startPt, endPt, bridgeSpace, style, end) {
            var bridgeSP;
            var angle = this._findAngle(startPt, endPt);
            if (seg._bridges.length > 0) {
                if (seg.type === "straight") {
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (!bridge._rendered) {
                            bridgeSP = seg._bridges[n - 1] != null ? seg._bridges[n - 1].endPoint : startPt;
                            canvas.line(bridgeSP, bridge.startPoint, style);
                            var centerX = bridge.startPoint.x + ((bridge.endPoint.x - bridge.startPoint.x) / 2);
                            var centerY = bridge.startPoint.y + ((bridge.endPoint.y - bridge.startPoint.y) / 2);
                            var sweep = ej.datavisualization.Diagram.Util.sweepDirection(angle, null, canvas._diagram);
                            canvas.arc(centerX, centerY, bridgeSpace / 2, angle, sweep, style);
                            if (seg._bridges[n + 1] != null)
                                canvas.line(bridge.endPoint, seg._bridges[n + 1].startPoint, style);
                            else
                                canvas.line(bridge.endPoint, endPt, style);
                            bridge._rendered = true;
                        }
                    }
                }
                else if (seg.type === "orthogonal") {
                    var ept = startPt;
                    for (var n = 0; n < seg._bridges.length; n++) {
                        var bridge = seg._bridges[n];
                        if (bridge.segmentPointIndex === pointIndex && !bridge._rendered) {
                            bridgeSP = seg._bridges[n - 1] != null ? seg._bridges[n - 1].endPoint : startPt;
                            canvas.line(bridgeSP, bridge.startPoint, style);
                            var centerX = bridge.startPoint.x + ((bridge.endPoint.x - bridge.startPoint.x) / 2);
                            var centerY = bridge.startPoint.y + ((bridge.endPoint.y - bridge.startPoint.y) / 2);
                            var sweep = ej.datavisualization.Diagram.Util.sweepDirection(angle, null, canvas._diagram);
                            canvas.arc(centerX, centerY, bridgeSpace / 2, angle, sweep, style);
                            if (seg._bridges[n + 1] != null)
                                canvas.line(bridge.endPoint, seg._bridges[n + 1].startPoint, style);
                            else
                                ept = bridge.endPoint;
                            bridge._rendered = true;
                        }
                    }
                    canvas.line(ept, endPt, style);
                }
            }
        },

        _findAngle: function (startPt, endPt) {
            var xDiff = startPt.x - endPt.x; var yDiff = startPt.y - endPt.y;
            return Math.atan2(yDiff, xDiff) * (180 / Math.PI);
        },

        _adjustPoint: function (source, target, isStart, length) {
            var pt = isStart ? { x: source.x, y: source.y } : { x: target.x, y: target.y };
            if (source.x == target.x) {
                if (source.y < target.y && isStart || source.y > target.y && !isStart)
                    pt.y += length;
                else pt.y -= length;
            }
            else if (source.y == target.y) {
                if (source.x < target.x && isStart || source.x > target.x && !isStart)
                    pt.x += length;
                else pt.x -= length;
            }
            else {
                var angle;
                if (isStart) {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(source, target);
                    pt = ej.datavisualization.Diagram.Geometry.transform(source, angle, length);
                }
                else {
                    angle = ej.datavisualization.Diagram.Geometry.findAngle(target, source);
                    pt = ej.datavisualization.Diagram.Geometry.transform(target, angle, length);
                }
            }
            return pt;
        },

        renderLabels: function (node, canvas) {
            for (var i = 0; i < node.labels.length; i++) {
                var label = node.labels[i];
                var ctx = canvas.document.getContext("2d");
                ctx.save();
                ctx.lineWidth = label.borderWidth;
                ctx.strokeStyle = label.borderColor;
                if (label.text)
                    this._renderTextElement(node, label, canvas);
                ctx.restore();
            }
        },

        renderPorts: function (node, canvas, x, y) {
            var ctx = canvas.document.getContext("2d");
            for (var i = 0; i < node.ports.length; i++) {
                var port = node.ports[i];
                if (!(port.visibility & ej.datavisualization.Diagram.PortVisibility.Hidden || port.visibility & ej.datavisualization.Diagram.PortVisibility.Hover || port.visibility & ej.datavisualization.Diagram.PortVisibility.Connect)) {
                    var point = ej.datavisualization.Diagram.Util._getPortPosition(port, ej.datavisualization.Diagram.Util.bounds(node, true));
                    ctx.fillStyle = port.fillColor;
                    ctx.strokeStyle = port.borderColor;
                    ctx.lineWidth = port.borderWidth;
                    switch (port.shape) {
                        case "x":
                            ctx.beginPath();
                            ctx.moveTo((point.x - port.size / 2), (point.y - port.size / 2));
                            ctx.lineTo((point.x + port.size / 2), (point.y + port.size / 2));
                            ctx.moveTo((point.x + port.size / 2), (point.y - port.size / 2));
                            ctx.lineTo((point.x - port.size / 2), (point.y + port.size / 2));
                            ctx.stroke();
                            ctx.fill();
                            ctx.closePath();
                            break;
                        case "circle":
                            var rx = port.size / 2;
                            var ry = port.size / 2;
                            canvas.ellipse(point.x, point.y, rx, ry);
                            break;
                        case "square":
                            canvas.rect(point.x - port.size / 2, point.y - port.size / 2, port.size, port.size);
                            break;
                        case "path":
                            ctx.save();
                            x = node._type != "group" ? point.x - node.width * port.offset.x : 0;
                            y = node._type != "group" ? point.y - node.height * port.offset.y : 0;
                            canvas.path(port._absolutePath, x, y, port.size, port.size, null);
                            ctx.restore();
                    }
                }
            }
        },

        _renderTextElement: function (node, textElement, canvas) {
            if (textElement) {
                var text = { childNodes: [] };
                var ctx = canvas.document.getContext("2d");
                ctx.save();
                var font = "";
                if (textElement.italic)
                    font += "italic ";
                if (textElement.bold)
                    font += "bold ";
                font += textElement.fontSize + "px ";
                font += textElement.fontFamily;
                ctx.font = font;
                var bounds = this._wrapText(node, text, textElement, canvas);
                this._alignTextOnLabel(textElement, node, text, canvas, bounds);
                ctx.restore();
            }
        },

        _alignTextOnLabel: function (label, node, text, canvas, labelBounds) {
            var nodeWidth = node.width || node._width || 0;
            var nodeHeight = node.height || node._height || 0;
            var bounds = { "width": labelBounds && labelBounds.width || label.width, "height": text.childNodes.length * label.fontSize };
            var nodeBounds = ej.datavisualization.Diagram.Util.bounds(node);
            var offset, hAlign, vAlign;
            var diagram = canvas._diagram;
            if (node.segments && typeof label.offset != "object") {
                var obj = ej.datavisualization.Diagram.SvgContext._getConnectorHandlePosition(label, node, 1, diagram);
                var pt = obj.position;
                offset = {
                    x: (pt.x - nodeBounds.x) / nodeBounds.width,
                    y: (pt.y - nodeBounds.y) / nodeBounds.height
                };
                var alignment = ej.datavisualization.Diagram.Util._alignLabelOnSegments(node, label, diagram, obj);
                vAlign = alignment.vAlign; hAlign = alignment.hAlign;
            }
            else {
                hAlign = label.horizontalAlignment;
                vAlign = label.verticalAlignment;
            }
            offset = ej.datavisualization.Diagram.Util._getLabelPosition(label, nodeBounds, offset);
            var point = { x: 0, y: 0 }, tempPt = { x: offset.x, y: offset.y };
            var y = 0;
            if (vAlign == ej.datavisualization.Diagram.VerticalAlignment.Top)
                y = offset.y + bounds.height / 2;
            else if (vAlign == ej.datavisualization.Diagram.VerticalAlignment.Center)
                y = offset.y;
            else
                y = offset.y - bounds.height / 2;
            point.y = y;
            point.x = offset.x;
            tempPt.y = y;
            var ctx = canvas.document.getContext("2d");
            if (label.textAlign == "justify") {
                if (text.childNodes.length > 1) {
                    ctx.textAlign = "left";
                    bounds.width = nodeWidth - 2 * label.borderWidth;
                }
                else ctx.textAlign = "center";
            } else {
                ctx.textAlign = label.textAlign;
            }
            switch (hAlign) {
                case ej.datavisualization.Diagram.HorizontalAlignment.Left:
                    tempPt.x = offset.x + bounds.width / 2;
                    switch (ctx.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x + bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width;
                            break;
                    }
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Center:
                    tempPt.x = offset.x;
                    switch (ctx.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x + bounds.width / 2;
                            break;
                    }
                    break;
                case ej.datavisualization.Diagram.HorizontalAlignment.Right:
                    tempPt.x = offset.x - bounds.width / 2;
                    switch (ctx.textAlign) {
                        case ej.datavisualization.Diagram.TextAlign.Left:
                            point.x = offset.x - bounds.width;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Center:
                            point.x = offset.x - bounds.width / 2;
                            break;
                        case ej.datavisualization.Diagram.TextAlign.Right:
                            point.x = offset.x;
                            break;
                    }
                    break;
            }
            var rotation, labelX = nodeBounds.x, labelY = nodeBounds.y;
            if (!node.segments) {
                labelX = node.offsetX - nodeWidth * node.pivot.x;
                labelY = node.offsetY - nodeHeight * node.pivot.y;
            }
            rotation = { "angle": label.rotateAngle, x: labelX + tempPt.x, y: labelY + tempPt.y };
            if (label.textAlign == "justify") {
                var labelWidth = Math.max(nodeWidth, label.width) - 2 * label.borderWidth;
            }
            ctx.save();
            ctx.translate(rotation.x, rotation.y);
            ctx.rotate(rotation.angle * Math.PI / 180);
            ctx.translate(-rotation.x, -rotation.y);
            var bWidth = label.borderWidth || 0;
            var x = labelX + point.x + (labelBounds ? labelBounds.x : 0) - bWidth / 2;
            var y = labelY + point.y - bounds.height / 2 - bWidth / 2;
            canvas.rect(x, y, bounds.width + bWidth, bounds.height + bWidth, null,
                { fill: label.fillColor ? label.fillColor : "transparent", stroke: bWidth ? (label.borderColor || "transparent") : "transparent", lineWidth: bWidth });
            for (var i = 0; i < text.childNodes.length; i++) {
                var child = text.childNodes[i];
                var text_x = x + child.x - labelBounds.x + bWidth / 2;
                var text_y = y + child.dy * i + label.fontSize * 0.8 + bWidth / 2;
                var width = ctx.measureText(child.text).width;
                if (labelWidth > width && i < text.childNodes.length - 1) {
                    var txt = child.text;
                    if (txt[txt.length - 1] == " ") { txt = txt.slice(0, txt.length - 1); } if (txt[0] == " ") { txt = txt.slice(1, txt.length); }
                    width = ctx.measureText(txt).width;
                    var words = txt.split(" ");
                    if (words.length > 1) {
                        var extra = (((labelWidth - width) / ctx.measureText(" ").width) / (words.length - 1));
                        extra = Math.round(extra);
                        var extText = "";
                        for (var k = 0; k < extra; k++) {
                            extText += " ";
                        }
                        child.text = txt.replace(" ", extText);
                    }
                }
                canvas.text({ text: child.text }, text_x - child.x, text_y, null, { fill: label.fontColor });
                if (label.textDecoration != "none")
                    this._renderTextDecoration(text_x - child.x, text_y, ctx.measureText(child.text).width, label, canvas);
            }
            ctx.restore();
        },

        _wrapText: function (node, text, label, canvas) {
            var str = label.text; var attr = null;
            var textBBox = ej.datavisualization.Diagram.Util.bounds(node);
            var bbWidth, bbHeight;
            if (node.segments) {
                bbWidth = textBBox.width;
                bbHeight = textBBox.height;
            }
            else {
                bbWidth = textBBox.width - label.margin.left - label.margin.right;
                bbHeight = textBBox.height - label.margin.top - label.margin.bottom;
            }
            bbWidth -= 2 * label.borderWidth;
            bbWidth = bbWidth < label.width ? label.width : bbWidth;
            var eachLine = str.split('\n');
            var x, y, tspan, j, string = "", childNodes, bounds, i;
            var ctx = canvas.document.getContext("2d");
            var wrap = label.wrapping == "wrapwithoverflow" ? true : false;
            for (j = 0; j < eachLine.length; j++) {
                var txt = "";
                if (label.wrapping != "nowrap") {
                    var words = label.wrapping == "wrapwithoverflow" ? eachLine[j].split(" ") : eachLine[j];
                    for (i = 0; i < words.length; i++) {
                        txt += ((txt && wrap) ? " " : "") + words[i];
                        bounds = ctx.measureText(txt).width;
                        if (bounds >= bbWidth) {
                            text.childNodes[text.childNodes.length] = { text: txt };
                            txt = "";
                        }
                        else {
                            if (wrap) {
                                var newText = txt + (words[i + 1] || "");
                                if (ctx.measureText(newText).width > bbWidth) {
                                    text.childNodes[text.childNodes.length] = { text: txt };
                                    txt = "";
                                }
                            }
                            if (i == words.length - 1) {
                                text.childNodes[text.childNodes.length] = { text: txt };
                                txt = "";
                            }
                        }
                    }

                }
                else {
                    txt += eachLine[j];
                    text.childNodes[text.childNodes.length] = { text: txt };
                }

            }
            return this._wrapTextAlign(text, text.childNodes, label.fontSize, label.textAlign, ctx);
        },

        _wrapTextAlign: function (text, childNodes, height, textAlign, ctx) {
            var width, bounds;
            for (var i = 0; i < childNodes.length; i++) {
                var x = ctx.measureText(childNodes[i].text).width;
                width = x;
                switch (textAlign) {
                    case "left":
                        x = 0;
                        break;
                    case "center":
                        x = -x / 2;
                        break;
                    case "right":
                        x = -x;
                        break;
                    case "justify":
                        x = childNodes.length > 1 ? 0 : -x / 2;
                        break;
                }
                childNodes[i].x = Number(x);
                childNodes[i].dy = height;
                if (!bounds) {
                    bounds = {
                        x: x,
                        width: width
                    };
                } else {
                    bounds.x = Math.min(bounds.x, x);
                    bounds.width = Math.max(bounds.width, width);
                }
            }
            return bounds;
        },

        _renderTextDecoration: function (x, y, width, label, canvas) {
            var ctx = canvas.document.getContext("2d");
            ctx.save();
            ctx.strokeStyle = label.fontColor;
            ctx.lineWidth = 1;
            switch (ctx.textAlign) {
                case "center":
                    x -= (width / 2); break;
                case "right":
                    x -= width; break;
            }
            y += 2;
            switch (label.textDecoration) {
                case "overline":
                    y -= label.fontSize; break;
                case "line-through":
                    y -= label.fontSize / 2; break;
            }
            var stPt = { x: x, y: y };
            var edPt = { x: x + width, y: y };
            canvas.line(stPt, edPt, null);
            ctx.restore();
        },

        _renderGradient: function (node, ctx, x, y) {
            if (node.gradient) {
                var max, min;
                for (var i = 0; i < node.gradient.stops.length; i++) {
                    max = !max ? node.gradient.stops[i].offset : Math.max(max, node.gradient.stops[i].offset);
                    min = !min ? node.gradient.stops[i].offset : Math.min(min, node.gradient.stops[i].offset);
                }
                if (node.gradient.type == "linear") {
                    var grd = ctx.createLinearGradient(x + node.gradient.x1, y + node.gradient.y1, x + node.gradient.x2, y + node.gradient.y2);
                    for (var i = 0; i < node.gradient.stops.length; i++) {
                        var stop = node.gradient.stops[i];
                        var offset = min < 0 ? (max + stop.offset) / (2 * max) : stop.offset / max;
                        grd.addColorStop(offset, stop.color);
                    }
                }
                else if (node.gradient.type == "radial") {
                    var grd = ctx.createRadialGradient(x + node.gradient.cx, y + node.gradient.cy, 0, x + node.gradient.fx, y + node.gradient.fy, node.gradient.r);
                    for (var i = 0; i < node.gradient.stops.length; i++) {
                        var stop = node.gradient.stops[i];
                        var offset = min < 0 ? (max + stop.offset) / (2 * max) : stop.offset / max;
                        grd.addColorStop(offset, stop.color);
                    }
                }
                ctx.fillStyle = grd;
            }
        },

        _renderDecorators: function (connector, canvas) {
            var startPoint;
            var endPoint;
            if (connector.targetDecorator && connector.targetDecorator.shape) {
                var segment = connector.segments[connector.segments.length - 1];
                startPoint = segment.points[segment.points.length - 2];
                endPoint = connector.targetPoint;
                if (segment.type == "bezier") {
                    startPoint = segment._point2;
                }
                this._renderDecorator(connector.name + "_targetDecorator", endPoint,
                    startPoint, connector.targetDecorator, canvas);
            }
            if (connector.sourceDecorator && connector.sourceDecorator.shape) {
                var segment = connector.segments[0];
                startPoint = connector.sourcePoint;
                endPoint = segment.points[1];
                if (segment.type == "bezier") {
                    endPoint = segment._point1;
                }
                this._renderDecorator(connector.name + "_sourceDecorator", startPoint,
                    endPoint, connector.sourceDecorator, canvas);
            }
        },

        _renderDecorator: function (name, point1, point2, decorator, canvas) {
            var shape;
            var attr;
            var d;
            var size = new ej.datavisualization.Diagram.Size(decorator.width, decorator.height);
            var ctx = canvas.document.getContext("2d");
            ctx.fillStyle = decorator.fillColor;
            ctx.strokeStyle = decorator.borderColor == "none" ? "transparent" : decorator.borderColor;
            ctx.lineWidth = decorator.borderWidth;
            var angle = ej.datavisualization.Diagram.Geometry.findAngle(point1, point2);
            var rotation = { angle: angle, x: point1.x, y: point1.y };
            switch (decorator.shape) {
                case "arrow":
                    canvas.arrow(point1, rotation, size);
                    break;
                case "openarrow":
                    canvas.arrow(point1, rotation, size, true);
                    break;
                case "circle":
                    var rx = size.width / 2;
                    var ry = size.height / 2;
                    canvas.ellipse(point1.x + size.width / 2, point1.y, rx, ry, rotation);
                    break;
                case "diamond":
                    canvas.diamond(point1, rotation, size);
                    break;
                case "path":
                    canvas.path(decorator._absolutePath, 0, 0, size.width, size.height, rotation);
                    break;
            }
            return shape;
        },

        _updatePolygonPoints: function (node) {
            if (node.points) {
                var nodeWidth = node.width || node._width || 0;
                var nodeHeight = node.height || node._height || 0;
                var points = node.points;
                if (points.length) {
                    this._scalePoints(points, node);
                    var bounds = ej.datavisualization.Diagram.Geometry.rect(points);
                    var x = node.offsetX - nodeWidth * node.pivot.x - bounds.x;
                    var y = node.offsetY - nodeHeight * node.pivot.y - bounds.y;
                    for (var i = 0, len = points.length; i < len; ++i) {
                        var point = ej.datavisualization.Diagram.Geometry.translate(points[i], x, y);
                        points[i] = point;
                    }
                }
            }
        },

        _scalePoints: function (points, node) {
            var bounds = ej.datavisualization.Diagram.Geometry.rect(points);
            if (bounds.width > 0 && bounds.height > 0) {
                var nodeWidth = node.width || node._width || 0;
                var nodeHeight = node.height || node._height || 0;
                var scaleX = nodeWidth / bounds.width;
                var scaleY = nodeHeight / bounds.height;
                for (var i = 0, len = points.length; i < len; ++i) {
                    points[i].x = points[i].x * scaleX;
                    points[i].y = points[i].y * scaleY;
                }
            }
        },

        updateNode: function (model, canvas) {
            this.refreshCanvas(model, canvas);
        },

        updateConnector: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateGroup: function (model, canvas) { this.refreshCanvas(model, canvas); },

        refreshCanvas: function (model, canvas) {

            var ctx = canvas.document.getContext("2d");
            ctx.save();
            if (view.scale) {
                ctx.scale(1 / view.scale.x, 1 / view.scale.y);
            }
            var bounds = { x: canvas.document.offsetLeft, y: canvas.document.offsetTop, width: canvas.document.offsetWidth, height: canvas.document.offsetHeight };
            ctx.clearRect(bounds.x, bounds.y, bounds.width, bounds.height);
            if (view.scale) {
                ctx.scale(view.scale.x, view.scale.y);
            }
            ctx.restore();
            for (var i = 0; i < model.nodes.length; i++) {
                if (model.nodes[i]._type === "node")
                    this.renderNode(model.nodes[i], canvas);
                else if (model.nodes[i]._type === "group")
                    this.renderGroup(model.nodes[i], canvas);
            }
            for (var i = 0; i < model.connectors.length; i++) {
                this.renderConnector(model.connectors[i], canvas);
            }
        },

        updateViewport: function (diagram) {
            //diagram._scroller._updateViewPort();
            //diagram.model.pageSettings._updatePageSize(diagram, true);
        },

        addNodeLabel: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateLabel: function (model, canvas) { this.refreshCanvas(model, canvas); },

        setNodeShape: function (model, canvas) { this.refreshCanvas(model, canvas); },

        setLine: function (model, canvas) { this.refreshCanvas(model, canvas); },

        renderDecorators: function (model, canvas) { this.refreshCanvas(model, canvas); },

        clearDecorators: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateTargetDecoratorStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateSourceDecoratorStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        _updateNodeStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateLabelStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateConnectorStyle: function (model, canvas) { this.refreshCanvas(model, canvas); },

        updateTextBlock: function (model, label, canvas) { this.refreshCanvas(model, canvas); },

        removeChild: function (element, view, diagram) {
            this.refreshCanvas(model, canvas);
        }
    };
    //#endregion

})(jQuery, Syncfusion);;
/**
* @fileOverview PivotGrid control to visualize the data
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejPivotGrid", "ej.PivotGrid", {

        _rootCSS: "e-pivotgrid",
        element: null,
        model: null,
        validTags: ["div", "span"],
        defaults: {
            url: "",
            cssClass: "",
            jsonRecords: "",
            currentReport: "",
            layout: "Normal",
            analysisMode: "olap",
            enablePivotFieldList: true,
            enableGroupingBar: false,
            enableDeferUpdate: false,
            enableJSONRendering: false,
            enableVirtualScrolling: false,
            enableCellContext: false,
            enableRTL: false,
            enableToolTip: true,
            enableCellSelection: false,
            enableConditionalFormatting: false,
            enableColumnGrandTotal: true,
            enableRowGrandTotal: true,
            enableGrandTotal: true,
			enableCollapseByDefault : false,
            isResponsive: false,
            hyperlinkSettings: {
                enableValueCellHyperlink: false,
                enableRowHeaderHyperlink: false,
                enableColumnHeaderHyperlink: false,
                enableSummaryCellHyperlink: false
            },
            serviceMethodSettings: {
                initialize: "InitializeGrid",
                drillDown: "DrillGrid",
                exportPivotGrid: "Export",
                paging: "Paging",
                fetchMembers: "FetchMembers",
                nodeStateModified: "NodeStateModified",
                nodeDropped: "NodeDropped",
                filtering: "Filtering",
                sorting: "Sorting",
                deferUpdate:"DeferUpdate"
            },
            customObject: {},
            locale: "en-US",
            cellSelection: "",
            valueCellHyperlinkClick: "",
            rowHeaderHyperlinkClick: "",
            columnHeaderHyperlinkClick: "",
            summaryCellHyperlinkClick: "",
            beforeServiceInvoke: null,
            afterServiceInvoke: null,
            drillSuccess: null,
            cellContext: "",
            load: null,
            renderComplete: null,
            renderFailure: null,
            renderSuccess: null
        },

        dataTypes: {
            serviceMethodSettings: "data",
            customObject: "data"
        },

        observables: ["layout", "enableCellContext", "hyperlinkSettings.enableValueCellHyperlink", "hyperlinkSettings.enableRowHeaderHyperlink", "hyperlinkSettings.enableColumnHeaderHyperlink", "hyperlinkSettings.enableSummaryCellHyperlink"],
        layout: ej.util.valueFunction("layout"),
        enableCellContext: ej.util.valueFunction("enableCellContext"),
        enableValueCellHyperlink: ej.util.valueFunction("hyperlinkSettings.enableValueCellHyperlink"),
        enableRowHeaderHyperlink: ej.util.valueFunction("hyperlinkSettings.enableRowHeaderHyperlink"),
        enableColumnHeaderHyperlink: ej.util.valueFunction("hyperlinkSettings.enableColumnHeaderHyperlink"),
        enableSummaryCellHyperlink: ej.util.valueFunction("hyperlinkSettings.enableSummaryCellHyperlink"),
        locale: ej.util.valueFunction("locale"),

        getOlapReport: function () {
            return this._olapReport;
        },

        setOlapReport: function (value) {
            this._olapReport = value;
        },

        getJSONRecords: function () {
            return this._JSONRecords;
        },

        setJSONRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        _destroy: function () {
            this.element.empty().removeClass("e-pivotgrid" + this.model.cssClass);
            if (this.ogridWaitingPopup != undefined)
                this.ogridWaitingPopup._destroy();
        },

        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._olapReport = "";
            this._JSONRecords = null;
            this._drillCaption = "";
            this._drillAction = "";
            this._startDrilldown = false;
            this._pagerObj = null;
            this._seriesPageCount = null;
            this._categPageCount = null;
            this._seriesCurrentPage = 1;
            this._categCurrentPage = 1;
            this._drillHeaders = new Array();
            this._collapsedRowHeaders = new Array();
            this._isDragging = false;
            this._droppedClass = "";
            this._dataModel = "";
            this._ascdes = "";
            this._filterUpdate = new Array();
            this._pivotRow = new Array();
            this._pivotColumn = new Array();
            this._pivotCalculation = new Array();
            this._pivotFilter = new Array();
            this._pivotTableFields = new Array();
            this._schemaData = null;
            ogridWaitingPopup = null;
            this._oriX = null;
            this._oriY = null;
            this._startPosCell = null;
            _proxy = this;
            this._maxViewLoading = null;
            this.formattingArrayClone = new Array();
            this.removedItems = new Array();
            this.mdformattingArray = new Array();
            this.defaultStyleSCell = new Array();
            this.defaultStyleVCell = new Array();
            this._list = new Array();
            this._list.push("Add New");
            this.rowCount = 0;
            var i, j, k, l;
        },

        _load: function () {
            var eventArgs = { action: ej.PivotGrid.GridAction.Initialize, element: this.element, customObject: this.model.customObject };
            this._trigger("load", eventArgs);
            this.element.addClass(this.model.cssClass);
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                this.model.customObject = oclientProxy.model.customObject;
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                    this._maxViewLoading = ("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
                }
                else
                    oclientWaitingPopup.show();
            }
            else {
                if (typeof oclientWaitingPopup == 'undefined') {
                    $("#" + this._id).ejWaitingPopup({ showOnInit: true });
                    ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                }
            }

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: ej.PivotGrid.GridAction.Initialize, element: this.element, customObject: this.model.customObject });            
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (_proxy.model.enableJSONRendering == true) {
                this._renderControlSuccess(this.model.jsonRecords);
                return;
            }
            if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0 && this.layout() != "" && this.layout() != ej.PivotGrid.Layout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "currentReport": this.model.currentReport, "gridLayout": this.layout(), "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.element[0] != null && this.element.parents().find(".controlPanel").length > 0)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "currentReport": this.model.currentReport, "customObject": serializedCustomObject }), this._renderControlSuccess);
            else if (this.layout() != "" && this.layout() != ej.PivotGrid.Layout.Normal)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "gridLayout": this.layout(), "enablePivotFieldList": this.model.enablePivotFieldList, "customObject": serializedCustomObject }), this._renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGrid", "enablePivotFieldList": this.model.enablePivotFieldList, "customObject": serializedCustomObject }), this._renderControlSuccess);
        },

        _setFirst: true,

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "JsonData": this.setOlapReport(options[key]); break;
                    case "RefreshOlapGrid": this.element.renderControlFromJSON(options[key]); break;
                    case "layout": this._load(); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "enableCellContext": this._load(); break;
                    case "enableValueCellHyperlink": this._load(); break;
                    case "enableRowHeaderHyperlink": this._load(); break;
                    case "enableColumnHeaderHyperlink": this._load(); break;
                    case "enableSummaryCellHyperlink": this._load(); break;
                    case "locale": this._load(); break;
                }
            }
        },

        _wireEvents: function () {
            if (this.model.enableCellSelection) {
                this._on(this.element, "mousedown", ".value", this._initCellSelection);
            }
            if (this.enableCellContext() == true) {
                if (document.addEventListener) {
                    document.removeEventListener('contextmenu', _proxy._cellContext, false);
                    document.addEventListener('contextmenu', _proxy._cellContext, false);
                }
                else {
                    document.detachEvent('oncontextmenu', _proxy._cellContext);
                    document.attachEvent('oncontextmenu', _proxy._cellContext);
                }
            }
            this.drillDownHandler = $.proxy(this._drillDown, this);
            this.addHyperlinkHandler = $.proxy(this._addHyperlink, this);
            this.removeHyperlinkHandler = $.proxy(this._removeHyperlink, this);
            this._on(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._on(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
            this._on(this.element, "click", ".value, .colheader, .rowheader", function (e) {
                if (e.target.className.indexOf("hyperlink") != -1) {
                    var cellPos = $(e.target).attr("p");
                    var rawdata = this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: e.target.innerHTML,
                        cellPosition: cellPos,
                        cellType: e.target.className.split(' ')[0],
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    if (this.enableSummaryCellHyperlink() && e.target.className.indexOf("summary value") >= 0)
                        this._trigger("summaryCellHyperlinkClick", { element: this.element, customObject: this.model.customObject, args: e });
                    else if (this.enableValueCellHyperlink() && e.target.className.indexOf("value") >= 0)
                        this._trigger("valueCellHyperlinkClick", { element: this.element, customObject: this.model.customObject, args: e });
                    else if (this.enableRowHeaderHyperlink() && e.target.className.indexOf("rowheader") >= 0)
                        this._trigger("rowHeaderHyperlinkClick", { element: this.element, customObject: this.model.customObject, args: e });
                    else if (this.enableColumnHeaderHyperlink() && e.target.className.indexOf("colheader") >= 0)
                        this._trigger("columnHeaderHyperlinkClick", { element: this.element, customObject: this.model.customObject, args: e });
                }
            });
            this._on(this.element, "click", ".expand, .collapse", ej.proxy(this._drillDown, this));
            this._on(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                if (evt.target.className.indexOf("expand") > -1 && !$(evt.target).hasClass("header-hover-expand"))
                    $(evt.target).addClass("header-hover-expand");
                else if (evt.target.className.indexOf("collapse") > -1 && !$(evt.target).hasClass("header-hover-collapse"))
                    $(evt.target).addClass("header-hover-collapse");
                else if ((evt.target.className.indexOf("expand") < 0 || evt.target.className.indexOf("collapse") < 0) && $(evt.target).children("span").length > 0) {
                    var child = $(evt.target).children("span")[0];
                    if (!$(child).hasClass("header-hover-expand") && child.className.indexOf("expand") > -1)
                        $(child).addClass("header-hover-expand");
                    else if (!$(child).hasClass("header-hover-collapse") && child.className.indexOf("collapse") > -1)
                        $(child).addClass("header-hover-collapse");
                }
            });
            this._on(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse", function (evt) {
                this.element.find(".expand, .collapse").removeClass("header-hover-expand header-hover-collapse");
            });
            if (this.model.enableToolTip) {
                this._on(this.element, "mouseover", ".value", this._applyToolTip);
                this._on(this.element, "mouseleave", ".value", function (e) {
                    $("#" + this._id + "_gridTooltip").hide("slow");
                });
            }
            this._on(this.element, "click", ".filter", ej.proxy(this._onFilterBtnClick, this));
            this._on(this.element, "click", ".sorting", ej.proxy(this._onSortBtnClick, this));
            this._on(this.element, "click", ".removeBtn", ej.proxy(this._removePvtBtn, this));
            this._on(this.element, "mouseover", ".pvtBtn", function (evt) {
                this.element.find(evt.target.parentElement).addClass("e-btn e-select");
            });
            this._on(this.element, "mouseleave", ".pvtBtn", function (evt) {
                this.element.find(evt.target.parentElement).removeClass("e-btn e-select");
            });
            this._on(this.element, "mouseover", ".pivotButton", function (evt) {
                evt.target.title = evt.target.textContent;
            });
        },

        _unWireEvents: function () {
            this._off(this.element, "click", ".expand, .collapse");
            this._off(this.element, "click", ".value, .colheader, .rowheader");
            this._off(this.element, "mouseover", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseleave", ".colheader, .rowheader, .expand, .collapse");
            this._off(this.element, "mouseover", ".value, .rowheader, .colheader", this.addHyperlinkHandler);
            this._off(this.element, "mouseleave", ".value, .rowheader, .colheader", this.removeHyperlinkHandler);
            this._off(this.element, "mouseover", ".value");
            this._off(this.element, "mouseleave", ".value");
            this._off(this.element, "mousedown", ".value");
            this._off(this.element, "mouseup", ".value");
            this._off(this.element, "click", ".filter");
            this._off(this.element, "click", ".sorting");
            this._off(this.element, "click", ".removeBtn");
            this._off(this.element, "mouseover", ".pvtBtn");
            this._off(this.element, "mouseleave", ".pvtBtn");
        },

        _onSortBtnClick: function (args) {
            $(args.target).toggleClass("descending")
            var sortValues = this._ascdes.split("##"), count = 0;
            for (var i = 0; i < sortValues.length; i++) {
                if (sortValues[i] == $($(args.target).siblings()[1]).text()) {
                    count = 1;
                }
            }
            classNames = args.target.className;
            isDescending = new RegExp("descending");
            if (isDescending.test(classNames)) {
                if (count == 0)
                    this._ascdes += $($(args.target).siblings()[1]).text() + "##";
            }
            else {
                if (count == 1)
                    this._ascdes = this._ascdes.replace($($(args.target).siblings()[1]).text() + "##", "");
            }
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "sorting", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
           var eventArgs = JSON.stringify({ "action": "sorting", "sortedHeaders": this._ascdes, "currentReport": report, "customObject": serializedCustomObject });
		   ogridWaitingPopup = this.element.data("ejWaitingPopup");
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
            if (!this.model.enableDeferUpdate) {
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.sorting, eventArgs, this._sortingSuccess);
            }
            else {
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.hide();
            }

        },
        _deferUpdate: function () {
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            var filterValues = "";
            for(var i=0;i<this._filterUpdate.length;i++)
            {
                filterValues += this._filterUpdate[i] + "%%";
            }
            var eventArgs = JSON.stringify({ "action": "deferUpdate", "sortedHeaders": this._ascdes, "filterParams": filterValues, "currentReport": report });
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.deferUpdate, eventArgs, this._renderControlSuccess);
            this._filterUpdate = [];
			ogridWaitingPopup = this.element.data("ejWaitingPopup");
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
        },

        _dialogBtnClick: function (args) {
            this.element.find(".e-dialog, .clientDialog").hide();
            var unselectedNodes = "", enableFilterIndigator;
            if (args.model.text.toLowerCase() != "cancel") {
                var uncheckedNodes = this._dataModel == "Olap" ? unselectedNodes = this._getUnSelectedNodes() + "FILTERED" + this._getSelectedNodes(this._curFilteredAxis == "Filter" ? true : false) : this._memberTreeObj.element.find(":input:gt(0).nodecheckbox:not(:checked)"), isFiltered = false,
                filterParams = this._dataModel == "Olap" ? uncheckedNodes : this._curFilteredAxis + "::" + this._curFilteredText + "::FILTERED";
                var text = this._curFilteredText, textArr = new Array(), obj = {};
                enableFilterIndigator = this._getUnSelectedNodes() != "" ? true : false;
                if (this._dataModel == "Pivot") {
                    for (var i = 0; i < uncheckedNodes.length; i++)
                        textArr.push($(uncheckedNodes[i].parentElement).siblings("a").text());
                    obj[text] = textArr;
                    if (ej.isNullOrUndefined(this._tempFilterData))
                        this._tempFilterData = new Array();
                    for (var i = 0; i < this._tempFilterData.length; i++) {
                        if (!ej.isNullOrUndefined(this._tempFilterData[i][text])) {
                            this._tempFilterData[i][text] = textArr;
                            isFiltered = true;
                        }
                    }
                    if (!isFiltered)
                        this._tempFilterData.push(obj);
                    if (this._schemaData != null) {
                        this._schemaData._tempFilterData = this._tempFilterData;
                    }
                    if (ej.isNullOrUndefined(this._curFilteredAxis) || this._curFilteredAxis != "") {
                        for (var i = 0; i < uncheckedNodes.length; i++)
                            filterParams += "##" + $(uncheckedNodes[i].parentElement).siblings("a").text();
                    }
                }
                filteredBtn = this.model.layout == "excel" ? this.element.find(".schemaFieldTree li:contains('" + this._curFilteredText + "')") : this.element.find("." + this._curFilteredAxis + " .pivotButton:contains(" + this._curFilteredText + ") .filterBtn");
                if (this.model.layout == "Normal" && this._schemaData != null) {
                    for (var i = 0; i < this._schemaData.element.find(".fieldTable").find("li").length; i++) {
                        if (this._curFilteredText == this._schemaData.element.find(".fieldTable").find("li")[i].id) {
                            _pivotProxy._selectedTreeNode = this._schemaData.element.find(".fieldTable").find("li")[i];
                        }
                    }
                    if ($(_pivotProxy._selectedTreeNode).parents("li:eq(0)").children().children("span").hasClass("hierarchyCDB")) {
                        _pivotProxy._selectedTreeNode = $($(_pivotProxy._selectedTreeNode).parents("li:eq(0)"));
                    }
                    if (enableFilterIndigator && $($(_pivotProxy._selectedTreeNode)).find(".filter").length <= 0) {
                        var filterSpan = ej.buildTag("span.e-icon", { "display": "none" }).addClass("filter")[0].outerHTML;
                        $($(_pivotProxy._selectedTreeNode).find(".e-text")[0]).after(filterSpan);
                    }
                    else if (!enableFilterIndigator)
                        $(_pivotProxy._selectedTreeNode).find(".filter").remove();
                    if (this._curFilteredAxis == "")
                        return false;
                }

                var report;
                try {
                    report = JSON.parse(this.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.getOlapReport();
                }
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "filtering", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject),
                eventArgs = this.model.customObject != {} && filterParams != "" ? JSON.stringify({ "action": "filtering", "filterParams": filterParams, "sortedHeaders": this._ascdes, "currentReport": report, "customObject": serializedCustomObject }) :
            this.model.customObject != {} ? JSON.stringify({ "action": "filtering", "currentReport": report, "customObject": serializedCustomObject }) :
            filterParams != "" ? JSON.stringify({ "action": "filtering", "filterParams": filterParams, "sortedHeaders": this._ascdes, "currentReport": report }) : JSON.stringify({ "action": "filtering" });
			ogridWaitingPopup = this.element.data("ejWaitingPopup");
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.show();
                if (!this.model.enableDeferUpdate)
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.filtering, eventArgs, this._filterElementSuccess);
                else {
                    this._filterUpdate.push(filterParams);
                    if (!ej.isNullOrUndefined(this._tempFilterData)) {
                        for (var j = 0; j < this._tempFilterData.length; j++) {
                            for (var key in this._tempFilterData[j]) {
                                if (key == text && this._tempFilterData[j][text].length > 0)
                                    this.element.find("#pivotButton" + text).next().addClass("filtered");
                            }
                        }
                    }
                    if (!ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.hide();
                }
            }
        },

        _getUnSelectedNodes: function () {
            var treeElement = this.element.find(".editorTreeView")[0];
            var unselectedNodes = "";
            var data = $(treeElement).find(":input.nodecheckbox:not(:checked)");
            for (var i = 0; i < data.length; i++) {
                if (!($(data[i].parentElement).find('span:nth-child(1)').attr('class').indexOf("e-chk-act") > -1) && $(data[i].parentElement).attr('aria-checked') != 'mixed') {
                    var parentNode = $(data[i]).parents('li:eq(0)');
                    unselectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                }
            }
            return unselectedNodes;
        },

        _createDialog: function (title, treeViewData) {
            this.element.find(".e-dialog, .clientDialog").remove();
            var dialogContent = ej.buildTag("div#EditorDiv.editorDiv", ej.buildTag("div", ej.buildTag("div.memberEditorDiv", ej.buildTag("div#editorTreeView.editorTreeView")[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML + "</br>",
            dialogFooter = ej.buildTag("div", ej.buildTag("button#OKBtn.dialogOKBtn", "OK")[0].outerHTML + ej.buildTag("button#CancelBtn.dialogCancelBtn", "Cancel")[0].outerHTML, { "float": "right", "margin": "-5px 0 6px" })[0].outerHTML,
            ejDialog = ej.buildTag("div#clientDialog.clientDialog", dialogContent + dialogFooter, { "opacity": "1" }).attr("title", title)[0].outerHTML,
            treeViewData = JSON.parse(treeViewData);
            var selectedData;
            if (!ej.isNullOrUndefined(this._tempFilterData)) {
                for (var i = 0; i < this._tempFilterData.length; i++) {
                    if (!ej.isNullOrUndefined(this._tempFilterData[i][title]))
                        selectedData = this._tempFilterData[i][title];
                }
                if (!ej.isNullOrUndefined(selectedData)) {
                    for (var i = 0; i < selectedData.length; i++) {
                        for (var j = 0; j < treeViewData.length; j++) {
                            if (selectedData[i] == treeViewData[j].name)
                                treeViewData[j].checkedStatus = false;
                        }
                    }
                }
            }
            $(ejDialog).appendTo("#" + this._id);
            this.element.find(".editorTreeView").ejTreeView({
                showCheckbox: true,
                loadOnDemand: true,
                height: $(".memberEditorDiv").height(),
                fields: { id: "id", text: "name", isChecked: "checkedStatus", hasChild: "hasChildren", dataSource: treeViewData },
                nodeClick: ""
            });
            var treeViewElements = this.element.find(".editorTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            this.element.find(".dialogOKBtn, .dialogCancelBtn").ejButton({ type: ej.ButtonType.Button, click: ej.proxy(this._dialogBtnClick, this) });
            this._dialogOKBtnObj = this.element.find(".dialogOKBtn").data("ejButton");
            this._memberTreeObj = this.element.find(".editorTreeView").data("ejTreeView");
            var treeViewLi = this.element.find(".editorTreeView li:gt(0)"),
            uncheckedNodes = $(treeViewLi).find(":input.nodecheckbox:not(:checked)");
            if (uncheckedNodes.length > 0) {
                var firstNode = $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)");
                $(firstNode).removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
            }
            this.element.find(".clientDialog").ejDialog({ width: 265, target: "#" + this._id, enableResize: false });
            this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
            this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
            if (this._memberTreeObj.element.find(".e-plus").length == 0) {
                this._memberTreeObj.element.find(".e-item").css("padding", "0px");
            }
            this._unWireEvents();
            this._wireEvents();
        },

        _onNodeCheckChanges: function (args) {
            if (args.value == "(All)" && args.type == "nodeUncheck") {
                this._memberTreeObj.model.nodeCheck = "";
                this._memberTreeObj.model.nodeUncheck = "";
                this.element.find(".editorTreeView").ejTreeView("unCheckAll");
                this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
                this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
                this._dialogOKBtnObj.disable();
            }
            else if (args.value == "(All)" && args.type == "nodeCheck") {
                this._memberTreeObj.model.nodeCheck = "";
                this._memberTreeObj.model.nodeUncheck = "";
                this.element.find(".editorTreeView").ejTreeView("checkAll");
                $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)").removeClass("e-chk-indetericon");
                this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
                this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
                this._dialogOKBtnObj.enable();
            }
            else if (args.value != "(All)") {
                var uncheckedNodes = this._memberTreeObj.element.find(":input:gt(0).nodecheckbox:not(:checked)"),
                firstNode = $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)");
                if (uncheckedNodes.length == 0 || (uncheckedNodes.length == 1 && uncheckedNodes[0].id[uncheckedNodes[0].id.length - 1] == 0)) {
                    $(firstNode).parent().removeClass("e-chk-inact").removeClass("e-chk-ind").addClass("e-chk-act");
                    firstNode.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                }
                else if (args.type == "nodeCheck" && uncheckedNodes.length == 1 && uncheckedNodes[0].id[uncheckedNodes[0].id.length - 1] == 0)
                    firstNode.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                else if (uncheckedNodes.length > 0)
                    firstNode.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this._dialogOKBtnObj.enable();
                if (args.type == "nodeUncheck" && uncheckedNodes.length + 1 == this._memberTreeObj.element.find("li").length) {
                    firstNode.removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
                    this._dialogOKBtnObj.disable();
                }
            }
        },

        _fetchMemberSuccess: function (msg) {
            if (msg[0] != undefined && msg.length > 0) {
                this._currentMembers = msg[0].Value;
                if (msg[1] != null && msg[1] != undefined)
                    this.model.customObject = msg[1].Value;
            }
            else if (msg.d != undefined && msg.d.length > 0) {
                this._currentMembers = msg.d[0].Value;
                if (msg.d[1] != null && msg.d[1] != undefined)
                   this.model.customObject = msg.d[1].Value;
            }
            else if (msg != undefined && msg.length > 0) {
                this._currentMembers = msg.EditorTreeInfo;
                if (msg != null && msg != undefined)
                    this.model.customObject = msg.customObject;
            }
            else if (msg != undefined) {
                this._currentMembers = msg.EditorTreeInfo;
            }
            this._createDialog(this._dialogTitle, this._currentMembers);
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "fetchMembers", element: this.element, customObject: this.model.customObject });
        },
        _filterElementSuccess: function (report) {
            if (report[0] != undefined) {
                if (report[2] != null && report[2] != undefined)
                    this.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                if (report.d[2] != null && report.d[2] != undefined)
                    this.model.customObject = report.d[2].Value;
            }
            else {
                if (report.customObject != null && report.customObject != undefined)
                    this.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "filtering", element: this.element, customObject: this.model.customObject });
            this._renderControlSuccess(report);
        },
        _sortingSuccess: function (msg) {
            if (msg[0] != undefined) {
                if (msg[2] != undefined && msg.length > 0)
                    this.model.customObject = msg[2].Value;
            }
            else if (msg.d != undefined && msg.d.length > 0) {
                if (msg.d[2] != null && msg.d[2] != undefined)
                    this.model.customObject = msg.d[2].Value;
            }
            else if (msg != undefined && msg.length > 0) {
                if (msg.customObject != null && msg.customObject != undefined)
                    this.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "sorting", element: this.element, customObject: this.model.customObject });
            this._renderControlSuccess(msg)
        },
        _onFilterBtnClick: function (args) {
            this._curFilteredAxis = args.target.parentElement.attributes.tag.value.split(":")[0];
            this._curFilteredText = $($(args.target).siblings()[1]).text();
            this._dialogTitle = $($(args.target).siblings()[1]).text();
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "fetchMembers", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            eventArgs = JSON.stringify({ "action": "fetchMembers", "headerTag": this._dialogTitle, "sortedHeaders": this._ascdes, "currentReport": report, "customObject": serializedCustomObject });
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.fetchMembers, eventArgs, this._fetchMemberSuccess);
        },

        _applyToolTip: function (e) {
            {
                var toolTipInfo; var rowValue; var cellPos; var columnValue = ""; valueCell = "";
                if ($(e.target).hasClass("summary")) {
                    if ($(e.target).siblings("th").hasClass("rowheader"))
                        cellPos = $(e.target).parent("tr").find(".rowheader").last().attr("p");
                    else
                        cellPos = $(e.target).parent("tr").find(".summary").attr("p");
                }
                else
                    cellPos = $(e.target).parent("tr").find(".rowheader").last().attr("p");
                if (cellPos != null) {
                    if ((cellPos[0] == 0) && ((parseInt(cellPos.split(",")[0])) != null)) {
                        if (cellPos == this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Index) {
                            rowValue = this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Value;
                        }
                    }
                    else
                        while (cellPos[0] >= 0) {
                            if (cellPos == this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Index) {
                                toolTipInfo = this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Value;
                                cellPos = parseInt(cellPos.split(",")[0]) - 1 + "," + parseInt(cellPos.split(",")[1]);
                            }
                            if (rowValue)
                                rowValue = toolTipInfo + "-" + rowValue;
                            else
                                rowValue = toolTipInfo;
                        }
                }
                var columnHeaderCount = $(e.target).closest('tbody').prev().children('tr:last').length != 0 ? $(e.target).closest('tbody').prev().children('tr:last').children('th').attr("p")[2] : -1;
                var currentCellPos = $(e.target).attr("p");
                if ((currentCellPos != undefined) && (currentCellPos == this.getJSONRecords()[parseInt((parseInt(currentCellPos.split(",")[0]) * this.rowCount) + parseInt(currentCellPos.split(",")[1]))].Index)) {
                    var colPos = parseInt(currentCellPos.split(",")[0]);
                }
                for (var i = 0; i <= columnHeaderCount; i++) {
                    var columnHeaderPos = colPos + "," + i;
                    if (colPos != null)
                        if (columnHeaderPos == this.getJSONRecords()[parseInt((parseInt(columnHeaderPos.split(",")[0]) * this.rowCount) + parseInt(columnHeaderPos.split(",")[1]))].Index) {
                            toolTipInfo = this.getJSONRecords()[parseInt((parseInt(columnHeaderPos.split(",")[0]) * this.rowCount) + parseInt(columnHeaderPos.split(",")[1]))].Value;
                            if (columnValue == "")
                                columnValue = toolTipInfo;
                            else if (toolTipInfo != "")
                                columnValue = columnValue + "-" + toolTipInfo;
                        }
                }
                $("#" + this._id + "_gridTooltip").remove();
                var tdPos = $(e.target).position();
                $("#" + this._id).after("<div id=\"" + this._id + "_gridTooltip\" class=\"pGridTooltip\"></div>");

                $("#" + this._id + "_gridTooltip").append("<p id=\"value\" class=\"tooltipText\">" + this._getLocalizedLabels("ToolTipValue") + ": " + (($(e.target).text()) ? $(e.target).text() : "No Value") + "</p>");
                $("#" + this._id + "_gridTooltip").append("<p id=\"row\" class=\"tooltipText\">" + this._getLocalizedLabels("ToolTipRow") + ": " + ((rowValue == undefined || this.rowCount == 1) ? "No value" : rowValue) + "</p>");
                $("#" + this._id + "_gridTooltip").append("<p id=\"column\" class=\"tooltipText\">" + this._getLocalizedLabels("ToolTipColumn") + ": " + ((columnValue == undefined || columnValue == "") ? "No value" : columnValue) + "</p>");
                var containerEndPos = 0;
                if (typeof (oclientProxy) != "undefined")
                    containerEndPos = $("#" + this._id).parent().position().left + oclientProxy.element.find(".gridContainer").width();
                else
                    containerEndPos = $("#" + this._id).parent().position().left + $("#" + this._id).parent().width();
                var tooltipEndPos = tdPos.left + $("#" + this._id + "_gridTooltip").width();
                 $("#" + this._id + "_gridTooltip").css({ left: ((tdPos.left > 0 ? tdPos.left : 0) + ((containerEndPos - tooltipEndPos) < 0 ? ((containerEndPos - tooltipEndPos) - 20) : 5)), top: tdPos.top > 100 ? (tdPos.top - $("#" + this._id + "_gridTooltip").outerHeight() - 2) : tdPos.top + 30 });
                $("#" + this._id + "_gridTooltip").show("fast");
                if (this.model.enableRTL) {
                    $("#" + this._id + "_gridTooltip p").addClass("e-rtl");
                    $("#" + this._id + "_gridTooltip p").css("text-align", "right");
                }
                else {
                    $("#" + this._id + "_gridTooltip p").removeClass("e-rtl");
                    $("#" + this._id + "_gridTooltip p").css("text-align", "left");
                }
                if (typeof (oclientProxy) != "undefined" && oclientProxy.controlPlacement() == "tile")
                    $("#" + this._id + "_gridTooltip").css("position", "absolute");
            }
        },

        _applyVScrolling: function () {
            var _proxy = this;
            var oGridDiv = $(this.element).find('.virtualScrollGrid');
            if (this._seriesPageCount > 1) {
                var getMinVerticalPos = function () { return (_proxy.element.find(".vScrollPanel").position().top + 1); };
                var vScrollBar = $(this.element).find(".vScrollPanel");
                vScrollBar.height($(oGridDiv).height());
                vScrollBar.html(ej.buildTag("div.vScrollThumb")[0].outerHTML);
                var vScrollHandle = vScrollBar.find(".vScrollThumb");
                vScrollHandle.height(Math.max(Math.ceil($(oGridDiv).height() / _proxy._seriesPageCount), 15));
                var vDragRange = vScrollBar.height() - vScrollHandle.height() - 2;
                vScrollHandle.offset({ top: getMinVerticalPos() + (_proxy._seriesCurrentPage - 1) * (vDragRange / (_proxy._seriesPageCount - 1)) });

                $(vScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageY - getMinVerticalPos();
                    var initThumbPos = vScrollHandle.position().top - getMinVerticalPos();
                    _proxy.element.find(".seriesPageIndicator").removeClass("inActive").css({ top: vScrollHandle.position().top, left: vScrollHandle.position().left + 20 });
                    var currentPage = _proxy._seriesCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageY - getMinVerticalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= vDragRange)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            vScrollHandle.offset({ top: initThumbPos + movedDistance + getMinVerticalPos() });
                        else if (initThumbPos + movedDistance > vDragRange)
                            vScrollHandle.offset({ top: getMinVerticalPos() + vDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            vScrollHandle.offset({ top: getMinVerticalPos() });
                        currentPage = Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount)) == 0 ? 1 : Math.ceil((vScrollHandle.position().top - getMinVerticalPos()) / (vDragRange / _proxy._seriesPageCount));
                        _proxy.element.find(".seriesPageIndicator").css({ top: vScrollHandle.position().top });
                        _proxy.element.find(".series_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(vScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._seriesCurrentPage) {
                            _proxy._seriesCurrentPage = currentPage;
                            _proxy.refreshPagedPivotGrid("series", _proxy._seriesCurrentPage);
                        }
                        else
                            _proxy.element.find(".seriesPageIndicator").addClass("inActive");
                    });
                });
            }

            if (this._categPageCount > 1) {
                var getMinHorizontalPos = function () { return (_proxy.element.find(".hScrollPanel").position().left + 1); };
                var hScrollBar = $(this.element).find(".hScrollPanel");
                hScrollBar.width($(oGridDiv).width());
                hScrollBar.html(ej.buildTag("div.hScrollThumb")[0].outerHTML);
                var hScrollHandle = hScrollBar.find(".hScrollThumb");
                hScrollHandle.width(Math.max(Math.ceil($(oGridDiv).width() / _proxy._categPageCount), 15));
                var hDragRange = hScrollBar.width() - hScrollHandle.width() - 2;
                hScrollHandle.offset({ left: getMinHorizontalPos() + (_proxy._categCurrentPage - 1) * (hDragRange / (_proxy._categPageCount - 1)) });

                $(hScrollHandle).bind('mousedown', function (e) {
                    $(this).addClass("dragging");
                    var initCursorPos = e.pageX - getMinHorizontalPos();
                    var initThumbPos = hScrollHandle.position().left - getMinHorizontalPos();
                    _proxy.element.find(".categPageIndicator").removeClass("inActive").css({ left: hScrollHandle.position().left, top: hScrollHandle.position().top + 20 });
                    var currentPage = _proxy._categCurrentPage;
                    $(document).bind('mousemove', function (e) {
                        if (window.navigator.userAgent.indexOf('MSIE ') > -1)
                            $(document).on("selectstart", function (e) { e.preventDefault(); });
                        else
                            window.getSelection().removeAllRanges();
                        var movedDistance = (e.pageX - getMinHorizontalPos()) - initCursorPos;
                        if (movedDistance > 0 && initThumbPos + movedDistance <= hDragRange)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (movedDistance < 0 && initThumbPos + movedDistance >= 0)
                            hScrollHandle.offset({ left: initThumbPos + movedDistance + getMinHorizontalPos() });
                        else if (initThumbPos + movedDistance > hDragRange)
                            hScrollHandle.offset({ left: getMinHorizontalPos() + hDragRange });
                        else if (initThumbPos + movedDistance < 0)
                            hScrollHandle.offset({ left: getMinHorizontalPos() });
                        currentPage = Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount)) == 0 ? 1 : Math.ceil((hScrollHandle.position().left - getMinHorizontalPos()) / (hDragRange / _proxy._categPageCount));
                        _proxy.element.find(".categPageIndicator").css({ left: hScrollHandle.position().left });
                        _proxy.element.find(".categ_CurrentPage").html(currentPage);
                    });
                    $(document).bind('mouseup', function () {
                        $(document).off("selectstart");
                        $(this).unbind('mousemove').unbind('mouseup');
                        $(hScrollHandle).removeClass("dragging");
                        if (currentPage != _proxy._categCurrentPage) {
                            _proxy._categCurrentPage = currentPage;
                            _proxy.refreshPagedPivotGrid("categ", _proxy._categCurrentPage);
                        }
                        else
                            _proxy.element.find(".categPageIndicator").addClass("inActive");
                    });
                });
            }
        },

        _getLocalizedLabels: function (property) {
            return ej.PivotGrid.locale[this.locale()][property] === undefined ? ej.PivotGrid.locale["en-US"][property] : ej.PivotGrid.locale[this.locale()][property];
        },

        _drillDown: function (e) {
			if (!this.model.enableDeferUpdate) {
				if (this._dataModel != "Pivot")
					this._off(this.element, "click", ".expand, .collapse");
				if (this._dataModel == "Pivot")
					this.element.find(".colheader, .rowheader, .value").removeClass("e-droppable");
				var targetClass = e.target.className;
				this._startDrilldown = true;
				targetClass = targetClass.split(" ");
				if (targetClass[0] == "expand")
					this._drillAction = "drilldown";
				else if (targetClass[0] == "collapse")
					this._drillAction = "drillup";

				if (this.model.beforeServiceInvoke != null)
					this._trigger("beforeServiceInvoke", { action: ej.PivotGrid.GridAction.DrillDown, element: this.element, customObject: this.model.customObject });
				var serializedCustomObject = JSON.stringify(this.model.customObject);
				var cellPos = $(e.target).parent().attr("p");
				this._drillCaption = $.trim($(e.target).parent().text());

				if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
					if ($("#" + oclientProxy._id + "_maxView")[0]) {
						$("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
						this._maxViewLoading = $("#" + oclientProxy._id + "_maxView").data("ejWaitingPopup");
					}
					else
						oclientWaitingPopup.show();
				}
				else {
					ogridWaitingPopup = this.element.data("ejWaitingPopup");
					ogridWaitingPopup.show();
				}
				var headerInfo = this.model.analysisMode == "relational" ? _proxy._drillHeaders.join("||") : this.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * this.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
				var report;
				try {
					report = JSON.parse(this.getOlapReport()).Report;
				}
				catch (err) {
					report = this.getOlapReport();
				}
				if (typeof (oclientProxy) != "undefined") {
					report = !oclientProxy.model.enableDeferUpdate ? report : oclientProxy.currentReport;
					oclientProxy._olapReport = oclientProxy.currentReport;
				}

				if (this._dataModel == "Pivot") {
					_proxy = this;
					var pGridTb = $(this.element).find("table").detach();
					this._collapseMember(pGridTb,e.target); 
					
						this.element.append(pGridTb);
						_proxy._hideGrandTotal();
						if (_proxy.model.enableGroupingBar) {
							tableWidth = $("#" + _proxy._id).find(".pivotGridTable").width();
							_proxy.element.find(".groupingBarPivot .drag").width(tableWidth - 5);
							cellWidth = $("#" + _proxy._id).find(".pivotGridTable").find("th").width();
							_proxy.element.find(".groupingBarPivot .values").width(cellWidth - 2);
							_proxy.element.find(".groupingBarPivot .columns").width(tableWidth - cellWidth);
							_proxy.element.find(".groupingBarPivot .valueColumn").width(tableWidth);
						}
						if (_proxy._dataModel == "Pivot") {
							_proxy.element.find(".colheader, .rowheader, .value").addClass("e-droppable");
						}
						ogridWaitingPopup.hide();
						var eventArgs = e;
						_proxy._trigger("renderSuccess", e);
						if (this.model.enableGroupingBar)
						    _proxy._createFields("drilldown", $("table").width(), $("table th").outerWidth());
				}
				else {
					if (this.layout() != "" || this.layout() != "normal") {
						if (typeof (oclientProxy) != "undefined")
							this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "clientReports": oclientProxy.reports, "headerInfo": headerInfo, "layout": this.layout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
						else
							this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "headerInfo": headerInfo, "layout": this.layout(), "customObject": serializedCustomObject }), this._drillDownSuccess);
					}
					else {
						if (typeof (oclientProxy) != "undefined")
							this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "clientReports": oclientProxy.reports, "headerInfo": headerInfo, "customObject": serializedCustomObject }), this._drillDownSuccess);
						else
							this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": cellPos, "currentReport": report, "headerInfo": headerInfo, "customObject": serializedCustomObject }), this._drillDownSuccess);
					}
				}
			}
        },
		_collapseMember : function(pGridTb, targetCell){
	    	var clickedTh = $(targetCell).parent()[0];
	    	var positionTh = $(clickedTh).attr("p");
	    	var rowSpan = $(targetCell).attr("tag") != undefined ? (parseInt($(targetCell).attr("tag")) + clickedTh.rowSpan) : clickedTh.rowSpan;
	    	var colsSpan = $(targetCell).attr("tag") != undefined ? (parseInt($(targetCell).attr("tag")) + clickedTh.colSpan) : clickedTh.colSpan;
	    	var row = $(targetCell).closest("tr")[0];
	    	var nextRow = null, prevRow = null, isTdFRow = "false", hiddenRCnt = 0;
	    	
	    		if (clickedTh.className == "rowheader" || (clickedTh.className == "summary" && $(clickedTh).attr("role") == "rowheader")) {
	    			var isTdSameRow = !$(clickedTh).index() > 0;
	    			if ($(targetCell).attr("class").indexOf("collapse") > -1) {
	    				for (var rCount = 0; rCount < rowSpan; rCount++) {
	    					nextRow = $(row).next();
	    					if (isTdSameRow) {
	    						$(row).css("display", "none");
	    						$(row).attr("tag", $(row).attr("tag") != undefined ? parseInt($(row).attr("tag")) + 1 : 1);
	    					}
	    					else {
	    						var nextTd = clickedTh;
	    						for (tdCnt = 0; tdCnt <= $(clickedTh).nextAll().length; tdCnt++) {
	    							$(nextTd).css("display", "none").attr("hc", $(nextTd).attr("hc") == undefined ? 1 : parseInt($(nextTd).attr("hc")) + 1);
	    							nextTd = $(nextTd).next();
	    						}
	    						isTdFRow = "true";
	    					}
	    					row = nextRow;
	    					isTdSameRow = true;
	    				}
	    				var subractRSpan = isTdFRow == "true" ? (clickedTh.rowSpan - 1) : clickedTh.rowSpan;
	    				$(row).find("th").text($(row).find("th").text().replace(" Total", "")).prepend("<span class=\"expand\ e-icon\" tag=\"" + rowSpan + ":" + isTdFRow + "\">&nbsp;</span>");
	    				var prevThRowSpan = $(row).prevAll().find("th:has(span):not(.summary)");
	    				for (var rCount = 0; rCount < prevThRowSpan.length; rCount++) {
	    					if (parseInt($(prevThRowSpan[rCount]).attr("p").split(",")[0]) < parseInt(positionTh.split(",")[0]) && (parseInt($(prevThRowSpan[rCount]).attr("p").split(",")[1]) + prevThRowSpan[rCount].rowSpan + ($(prevThRowSpan[rCount]).find("span").attr("tag") != undefined ? parseInt($(prevThRowSpan[rCount]).find("span").attr("tag")) : 0)) >= (parseInt(positionTh.split(",")[1]) + rowSpan)) {
	    						prevThRowSpan[rCount].rowSpan -= subractRSpan;
	    						$(prevThRowSpan[rCount]).find("span").attr("tag", $(prevThRowSpan[rCount]).find("span").attr("tag") != undefined ? (parseInt($(prevThRowSpan[rCount]).find("span").attr("tag")) + (subractRSpan)) : subractRSpan);
	    					}
	    				}
	    			}
	    			else if ($(targetCell).attr("class").indexOf("expand") > -1) {
	    				for (var rCount = parseInt($(targetCell).attr("tag").split(":")[0]) ; rCount >= 0 ; rCount--) {
	    					prevRow = $(row).prev();
	    					if ($(row).attr("tag") == undefined || parseInt($(row).attr("tag")) <= 1)
	    						$(row).css("display", "table-row");
	    					else
	    						hiddenRCnt++;
	    					if ($(row).attr("tag") != undefined) {
	    						if ((parseInt($(row).attr("tag") - 1)) == 0)
	    							$(row).removeAttr("tag");
	    						else
	    							$(row).attr("tag", parseInt($(row).attr("tag")) - 1);
	    					}
	    					row = prevRow;
	    				}
	    				if ($(targetCell).attr("tag").split(":")[1] == "true") {
	    					var nextTd = $(pGridTb).find("th[p='" + parseInt($(clickedTh).attr("p").split(",")[0]) + "," + (parseInt($(clickedTh).attr("p").split(",")[1]) - parseInt($(targetCell).attr("tag").split(":")[0])) + "']"), ttCnt = $(nextTd).nextAll().length;
	    					for (tdCnt = 0; tdCnt <= ttCnt; tdCnt++) {
	    						if ($(nextTd).attr("hc") == undefined || parseInt($(nextTd).attr("hc")) <= 1) {
	    							$(nextTd).removeAttr("hc");
	    							if ($(nextTd).attr("ch") == undefined)
	    								$(nextTd).css("display", "");
	    						}
	    						else
	    							$(nextTd).attr("hc", (parseInt($(nextTd).attr("hc")) - 1));
	    						nextTd = $(nextTd).next();
	    					}
	    				}
	    				var prevThRowSpan = $($(targetCell).closest("tr")[0]).prevAll().find("th:has(span):not(.summary)");
	    				var rSpnCnt = ($(targetCell).attr("tag").split(":")[1] == "true" ? (parseInt($(targetCell).attr("tag").split(":")[0]) - 1) : parseInt($(targetCell).attr("tag").split(":")[0])) - hiddenRCnt;
	    				for (var rCount = 0; rCount < prevThRowSpan.length; rCount++) {
	    					if (parseInt($(prevThRowSpan[rCount]).attr("p").split(",")[0]) < parseInt(positionTh.split(",")[0]) && (parseInt($(prevThRowSpan[rCount]).attr("p").split(",")[1]) + prevThRowSpan[rCount].rowSpan + ($(prevThRowSpan[rCount]).find("span").attr("tag") != undefined ? parseInt($(prevThRowSpan[rCount]).find("span").attr("tag")) : 0)) >= (parseInt(positionTh.split(",")[1]))) {
	    						prevThRowSpan[rCount].rowSpan += rSpnCnt;
	    						if ($(prevThRowSpan[rCount]).find("span").attr("tag") != undefined)
	    							$(prevThRowSpan[rCount]).find("span").attr("tag", (parseInt($(prevThRowSpan[rCount]).find("span").attr("tag")) - rSpnCnt));
	    						if (parseInt($(prevThRowSpan[rCount]).find("span").attr("tag")) == 0)
	    							$(prevThRowSpan[rCount]).find("span").removeAttr("tag");
	    					}
	    				}
	    				$(clickedTh).text($(clickedTh).text().trim() + " Total");
	    				$(clickedTh).prepend("<span style='margin-left:10px'></span>");
	    			}
	    		}
	    		if (clickedTh.className == "colheader" || (clickedTh.className == "summary" && $(clickedTh).attr("role") == "columnheader")) {
	    			var tdPosVal = parseInt(positionTh.split(",")[0]), tbRows = $(pGridTb).find("tr");
	    			if ($(targetCell).attr("class").indexOf("collapse") > -1) {
	    				for (var cCount = 0; cCount < colsSpan; cCount++) {
	    					var thList = $(row).nextAll().find("th[p^='" + (cCount + tdPosVal) + ",']");
	    					for (var thCnt = 0; thCnt < $(thList).length; thCnt++) {
	    						$(thList[thCnt]).attr("hc", $(thList[thCnt]).attr("hc") != undefined ? parseInt($(thList[thCnt]).attr("hc")) + 1 : 1).css("display", "none");
	    					}
	    					if ($(thList[thCnt]).attr("hc") == undefined || parseInt($(thList[thCnt]).attr("hc")) < 1) {
	    						$(pGridTb).find("td[p^='" + (cCount + tdPosVal) + ",']").css("display", "none");
	    						$(pGridTb).find("th[p^='0,']:not(.summary)").parent().find("td[p^='" + (cCount + tdPosVal) + ",']").attr("ch", 1);
	    					}
	    				}
	    				$(clickedTh).attr("hc", 1).css("display", "none");
	    				$(clickedTh).next().text($(clickedTh).text().replace(" Total", "")).prepend("<span class=\"expand\ e-icon\" tag=\"" + colsSpan + "\">&nbsp;</span>");
	    				var prevThColSpan = $(row).prevAll().find("th:has(span):not(.summary)");
	    				for (var cCount = 0; cCount < prevThColSpan.length; cCount++) {
	    					if ((parseInt($(prevThColSpan[cCount]).attr("p").split(",")[1]) < parseInt(positionTh.split(",")[1]) && parseInt($(prevThColSpan[cCount]).attr("p").split(",")[0]) <= parseInt(positionTh.split(",")[0])) && (parseInt($(prevThColSpan[cCount]).attr("p").split(",")[0]) + prevThColSpan[cCount].colSpan + ($(prevThColSpan[cCount]).find("span").attr("tag") != undefined ? parseInt($(prevThColSpan[cCount]).find("span").attr("tag")) : 0)) >= (parseInt(positionTh.split(",")[0]) + colsSpan)) {
	    						prevThColSpan[cCount].colSpan -= clickedTh.colSpan;
	    						$(prevThColSpan[cCount]).find("span").attr("tag", $(prevThColSpan[cCount]).find("span").attr("tag") != undefined ? (parseInt($(prevThColSpan[cCount]).find("span").attr("tag")) + (clickedTh.colSpan)) : clickedTh.colSpan);
	    					}
	    				}
	    			}
	    			else if ($(targetCell).attr("class").indexOf("expand") > -1) {
	    				var hdnCCount = 0;
	    				for (var cCount = 1; cCount < colsSpan; cCount++) {
	    					var thList = $(row).nextAll().find("th[p^='" + (tdPosVal - cCount) + ",']");
	    					for (var thCnt = 0; thCnt < $(thList).length; thCnt++) {
	    						if ($(thList[thCnt]).attr("hc") == undefined || parseInt($(thList[thCnt]).attr("hc")) == 1) {
	    							$(thList[thCnt]).css("display", "");
	    							$(thList[thCnt]).removeAttr("hc");
	    						}
	    						else {
	    							if ((parseInt($(thList[thCnt]).attr("hc")) - 1) == 0)
	    								$(thList[thCnt]).removeAttr("hc");
	    							else
	    								$(thList[thCnt]).attr("hc", (parseInt($(thList[thCnt]).attr("hc")) - 1));
	    						}
	    					}
	    					if ($(thList).last().attr("hc") == undefined) {
	    						$(pGridTb).find("td[p^='" + (tdPosVal - cCount) + ",']").css("display", "");
	    						var rHdnTds = $(pGridTb).find("th[p^='0,']:not(.summary)").parent().find("td[p^='" + (tdPosVal - cCount) + ",']");
	    						$(rHdnTds).removeAttr("ch");
	    						for (var tdCnt = 0; tdCnt <= $(rHdnTds).length; tdCnt++) {
	    							if ($(rHdnTds[tdCnt]).attr("hc") != undefined)
	    								$(rHdnTds[tdCnt]).css("display", "none");
	    						}
	    					}
	    					else
	    						hdnCCount++;
	    				}
	    				$(clickedTh).prev().removeAttr("hc").css("display", "");
	    				var cSpnCnt = parseInt($(targetCell).attr("tag")) - hdnCCount;
	    				var prevThColSpan = $(row).prevAll().find("th:has(span):not(.summary)");
	    				for (var cCount = 0; cCount < prevThColSpan.length; cCount++) {
	    					if ((parseInt($(prevThColSpan[cCount]).attr("p").split(",")[1]) < parseInt(positionTh.split(",")[1]) && parseInt($(prevThColSpan[cCount]).attr("p").split(",")[0]) < parseInt(positionTh.split(",")[0])) && (parseInt($(prevThColSpan[cCount]).attr("p").split(",")[0]) + prevThColSpan[cCount].colSpan + ($(prevThColSpan[cCount]).find("span").attr("tag") != undefined ? parseInt($(prevThColSpan[cCount]).find("span").attr("tag")) : 0)) >= (parseInt(positionTh.split(",")[0]))) {
	    						prevThColSpan[cCount].colSpan += cSpnCnt;
	    						if ($(prevThColSpan[cCount]).find("span").attr("tag") != undefined)
	    							$(prevThColSpan[cCount]).find("span").attr("tag", (parseInt($(prevThColSpan[cCount]).find("span").attr("tag")) - cSpnCnt));
	    						if (parseInt($(prevThColSpan[cCount]).find("span").attr("tag")) == 0)
	    							$(prevThColSpan[cCount]).find("span").removeAttr("tag");
	    					}
	    				}
	    				$(clickedTh).text($(clickedTh).text().trim() + " Total");
	    				$(clickedTh).prepend("<span style='margin-left:10px'></span>");
	    			}
	    		}	
		},		
        _addHyperlink: function (e) {
            if ($(".hyperlinkValueCell")[0])
                $($(".hyperlinkValueCell")[0]).removeClass("hyperlinkValueCell");
            if ($(".hyperlinkHeaderCell")[0])
                $($(".hyperlinkHeaderCell")[0]).removeClass("hyperlinkHeaderCell");
            if ($(e.target).text() == "" || $(e.target).text() == null)
                return false;
            if (e.target.className.split(" ")[0] == "value" && this.enableValueCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if ($(e.target).hasClass("summary value") && this.enableSummaryCellHyperlink())
                $(e.target).addClass("hyperlinkValueCell");
            else if (e.target.className.split(" ")[0] == "rowheader" && this.enableRowHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
            else if (e.target.className.split(" ")[0] == "colheader" && this.enableColumnHeaderHyperlink())
                $(e.target).addClass("hyperlinkHeaderCell");
        },

        _removeHyperlink: function (e) {
            $(e.target).removeClass("hyperlinkValueCell").removeClass("hyperlinkHeaderCell");
        },

        _cellContext: function (e) {
            var currentCell = e.target || e.srcElement;
            if (_proxy.enableCellContext() == true && $(currentCell).parents("#" + _proxy._id)[0]) {
                var cellPos = $(currentCell).attr("p");
                if (cellPos) {
                    var cellPos = $(currentCell).attr("p");
                    var rawdata = _proxy.getJSONRecords()[parseInt((parseInt(cellPos.split(",")[0]) * _proxy.rowCount) + parseInt(cellPos.split(",")[1]))].Info;
                    var cellInfo = {
                        cellValue: currentCell.innerHTML,
                        cellPosition: cellPos,
                        cellType: currentCell.className.split(' ')[0],
                        role: $(currentCell).attr('role'),
                        uniqueName: rawdata.split('::')[0],
                        args: e,
                        rawdata: rawdata
                    };
                    if (cellInfo.cellType != "")
                        _proxy._trigger("cellContext", { args: cellInfo });
                    e.target ? e.preventDefault() : (window.event.returnValue = false);
                }
            }
        },

        _initCellSelection: function (e) {
            this._on(this.element, "mouseup", ".value", this._completeCellSelection);
            var targetCell;
            _oriX = e.pageX, _oriY = e.pageY;
            targetCell = e.target || window.event.srcElement;
            $(".value").addClass("selection");
            $("#" + _proxy._id).append(ej.buildTag("div.cellSelection#" + _proxy._id + "_cellSelection", "", {}));
            _startPosCell = $(targetCell)[0].attributes.getNamedItem("p").value;
            this._on(this.element, "mousemove", ".value, .cellSelection", this._cellSelection);
        },

        _cellSelection: function (e) {
            $("#" + _proxy._id + "_gridTooltip").hide();
            if (e.pageX < _oriX) {
                $("#" + _proxy._id + "_cellSelection").css({ left: e.pageX +2, width: _oriX - e.pageX });
            }
            else {
                $("#" + _proxy._id + "_cellSelection").css({ left: _oriX -6 , width: e.pageX - _oriX });
            }
            if (e.pageY < _oriY) {
                $("#" + _proxy._id + "_cellSelection").css({ top: e.pageY , height: _oriY - e.pageY - 7 });
            }
            else {
                $("#" + _proxy._id + "_cellSelection").css({ top: _oriY+7, height: e.pageY - _oriY -11});
            }
            $(".pivotGridTable").mouseleave(function (e) {
                if(!ej.isNullOrUndefined(e.toElement) && e.toElement.classList[0]=="e-pivotgrid")
                {
                 $("#" + _proxy._id + "_cellSelection").remove();
                 $(".value").removeClass("selection");
                 _proxy._off(_proxy.element, "mouseup", ".value");
                }
            });
        },

        _completeCellSelection: function (e) {
            var targetCell, endPosCell, cellInfo = [], rowHeader = [], rowLThPos, colHeader = [], curCelPos;
            _proxy._off(_proxy.element, "mousemove", ".value");
            $(".value").removeClass("selection");
            targetCell = e.target || window.event.srcElement;
            endPosCell = $(targetCell)[0].attributes.getNamedItem("p").value;
            var count = 0;
            for (var rowSelCnt = (parseInt(_startPosCell.split(",")[1]) < parseInt(endPosCell.split(",")[1]) ? parseInt(_startPosCell.split(",")[1]) : parseInt(endPosCell.split(",")[1])) ; rowSelCnt <= (parseInt(_startPosCell.split(",")[1]) > parseInt(endPosCell.split(",")[1]) ? parseInt(_startPosCell.split(",")[1]) : parseInt(endPosCell.split(",")[1])) ; rowSelCnt++) {
                for (var colSelCnt = (parseInt(_startPosCell.split(",")[0]) < parseInt(endPosCell.split(",")[0]) ? parseInt(_startPosCell.split(",")[0]) : parseInt(endPosCell.split(",")[0])) ; colSelCnt <= (parseInt(_startPosCell.split(",")[0]) > parseInt(endPosCell.split(",")[0]) ? parseInt(_startPosCell.split(",")[0]) : parseInt(endPosCell.split(",")[0])) ; colSelCnt++) {
                    cellInfo[count] = _proxy.getJSONRecords()[((colSelCnt) * _proxy.rowCount) + rowSelCnt];
                    var rowInfo, rowValue = "";
                    var tempThPos, measureColumn, measureRowCount = 0, measureColCount = 0;
                    rowLThPos = $("#" + _proxy._id).find("tbody").find('[p="' + colSelCnt + "," + rowSelCnt + '"]').parent("tr").find(".rowheader").last().attr('p');
                    if (rowLThPos == undefined) {
                        rowLThPos = $("#" + _proxy._id).find("tbody").find('[p="' + colSelCnt + "," + rowSelCnt + '"]').parent("tr").find(".summary").first().attr('p');
                    }
                    tempThPos = rowLThPos;
                    if (rowLThPos != null) {
                        while (rowLThPos[0] >= 0) {
                            if (rowLThPos == _proxy.getJSONRecords()[parseInt((parseInt(rowLThPos.split(",")[0]) * _proxy.rowCount) + parseInt(rowLThPos.split(",")[1]))].Index) {
                                rowInfo = _proxy.getJSONRecords()[parseInt((parseInt(rowLThPos.split(",")[0]) * _proxy.rowCount) + parseInt(rowLThPos.split(",")[1]))].Value;
                                rowLThPos = parseInt(rowLThPos.split(",")[0]) - 1 + "," + parseInt(rowLThPos.split(",")[1]);
                            }
                            if ((rowValue) && (rowInfo != ""))
                                rowValue = rowInfo + "##" + rowValue;
                            else if (rowInfo != "")
                                rowValue = rowInfo;
                        }
                        rowHeader[count] = rowValue;
                    }
                    var columnValue = "", headerInfo;
                    var colHeadRCnt = $("#" + _proxy._id).find("td[p*='" + parseInt(cellInfo[count].Index.split(",")[0]) + "," + parseInt(cellInfo[count].Index.split(",")[1]) + "']").closest('tbody').prev().children('tr:last').children('th').attr("p")[2];
                    measureColumn = colHeadRCnt;
                    curCelPos = cellInfo[count].Index;
                    if ((curCelPos != undefined) && (curCelPos == _proxy.getJSONRecords()[parseInt((parseInt(curCelPos.split(",")[0]) * _proxy.rowCount) + parseInt(curCelPos.split(",")[1]))].Index)) {
                        var colPos = parseInt(curCelPos.split(",")[0]);
                    }
                    for (var rCnt = 0; rCnt <= colHeadRCnt; rCnt++) {
                        if (colPos != null)
                            if ((colPos + "," + rCnt) == (_proxy.getJSONRecords()[parseInt((parseInt(colPos) * _proxy.rowCount) + rCnt)].Index)) {
                                headerInfo = _proxy.getJSONRecords()[parseInt((parseInt(colPos) * _proxy.rowCount) + rCnt)].Value;
                                if (columnValue == "")
                                    columnValue = headerInfo;
                                else if (headerInfo != "")
                                    columnValue = columnValue + "##" + headerInfo;
                            }
                        colHeader[count] = columnValue;
                    }
                    count++;
                }
            }
            var measureValue = _proxy.getJSONRecords()[parseInt((parseInt(tempThPos.split(",")[0]) * _proxy.rowCount) + parseInt(tempThPos.split(",")[1]))].Info;
            while (measureValue[0] >= 0) {
                rowInfo = _proxy.getJSONRecords()[parseInt((parseInt(measureValue.split(",")[0]) * _proxy.rowCount) + parseInt(measureValue.split(",")[1]))].Info;
                if (rowInfo.split("::")[0].indexOf("Measures") > 0 || rowInfo.split("::")[0].indexOf("MEASURES") > 0) {
                    measureRowCount++;
                }
                rowLThPos = parseInt(rowLThPos.split(",")[0]) - 1 + "," + parseInt(rowLThPos.split(",")[1]);
            }
            var columnpos = parseInt(curCelPos.split(",")[0]);
            if (measureRowCount == 0)
                for (var colRCnt = 0; colRCnt <= measureColumn; colRCnt++) {
                    if (columnpos != null)
                        var headerValue = _proxy.getJSONRecords()[parseInt((parseInt(columnpos) * _proxy.rowCount) + colRCnt)].Info;
                    if (headerValue.split("::")[0].indexOf("Measures") > 0 || headerValue.split("::")[0].indexOf("MEASURES") > 0) {
                        measureColCount++;
                    }
                }
            args = { JSONRecords: cellInfo, rowHeader: rowHeader, columnHeader: colHeader, measureCount: (measureRowCount > 0 ? ("Row:" + measureRowCount) : ("Column:" + measureColCount)) }
            _proxy._trigger("cellSelection", args);
            cellInfo = rowHeader = colHeader = [];
            $("#" + _proxy._id + "_cellSelection").remove();
            _proxy._on(_proxy.element, "mousemove", ".value", _proxy._applyToolTip);

        },

        exportPivotGrid: function (exportOption) {
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            var colorDetails;
            if (this.element.find(".summary").length > 0) {
                colorDetails = {
                    valueCellColor: this.element.find(".value").css("color"),
                    valueCellBGColor: this.element.find(".value").css("background-color"),
                    summaryCellColor: this.element.find(".summary").css("color"),
                    summaryCellBGColor: this.element.find(".summary").css("background-color")
                }
            }
            else if (this.element.find(".value").length > 0) {
                colorDetails = {
                    valueCellColor: this.element.find(".value").css("color"),
                    valueCellBGColor: this.element.find(".value").css("background-color")
                }
            }
            else return false;
            var params = {
                args: JSON.stringify({
                    exportOption: exportOption,
                    currentReport: report,
                    layout: _proxy.layout(),
                    colorSettings: JSON.stringify(colorDetails)
                })
            };
            this.doPostBack(_proxy.model.url + "/" + _proxy.model.serviceMethodSettings.exportPivotGrid, params);
        },

        _getFArrayfromMArray: function (objName) {
            var format = { "formattingArrayClone": 1, "defaultStyleSCell": 2, "defaultStyleVCell": 3, "removedItems": 4 };
            var index = format[objName];

            for (i = 0; i < this.mdformattingArray.length; i++) {
                if (this.mdformattingArray[i][0] == _proxy._id) {
                    return this.mdformattingArray[i][index];
                }
            }

            return new Array();
        },

        createConditionalDialog: function () {
            _proxy.element.find(".e-dialog, .clientDialog").remove();
            var ejDialog = ej.buildTag("div#" + this._id + "_clientDlg.clientDialog", { "opacity": "1" })[0].outerHTML;
            var dialogContent; var dialogTitle; var currentTag;
            var ddlBackColor, ddlBorderColor, ddlBorderStyle, ddlBorderWidth, ddlFont, ddlFontSize, ddlEditCondition, ddlConditionTYpe, removeBtn, okBtn, cancelBtn;
            dialogTitle = _proxy._getLocalizedLabels("ConditionalFormatting");
            var conditionLbl = ej.buildTag("label#" + this._id + "_conLbl.conditionLbl", this._getLocalizedLabels("Condition"))[0].outerHTML;
            var conditionDropDown = ej.buildTag("input#" + this._id + "_conType.conditionType", "", {}, { type: 'text', tabindex: 0, accesskey: 'f' })[0].outerHTML;
            var editcondition = ej.buildTag("label#" + this._id + "_editcond.editcondition", this._getLocalizedLabels("Editcondtion"))[0].outerHTML;
            var value1Lbl = ej.buildTag("label#" + this._id + "_value1.value1", this._getLocalizedLabels("Value1"))[0].outerHTML;
            var value2Lbl = ej.buildTag("label#" + this._id + "_value2.value2", this._getLocalizedLabels("Value2"))[0].outerHTML;
            var backcolorLbl = ej.buildTag("label#" + this._id + "_backcolorLbl.backcolorLbl", this._getLocalizedLabels("Backcolor"))[0].outerHTML;
            var bordercolorLbl = ej.buildTag("label#" + this._id + "_bordercolorLbl.bordercolorLbl", this._getLocalizedLabels("Bordercolor"))[0].outerHTML;
            var borderrangeLbl = ej.buildTag("label#" + this._id + "_borderrangeLbl.borderrangeLbl", this._getLocalizedLabels("Borderrange"))[0].outerHTML;
            var borderstyleLbl = ej.buildTag("label#" + this._id + "_borderstyleLbl.borderstyleLbl", this._getLocalizedLabels("Borderstyle"))[0].outerHTML;
            var fStyleLbl = ej.buildTag("label#" + this._id + "_fStyleLbl.fStyleLbl", this._getLocalizedLabels("Fontstyle"))[0].outerHTML;
            var fSizeLbl = ej.buildTag("label#" + this._id + "_fSizeLbl.fSizeLbl", this._getLocalizedLabels("Fontsize"))[0].outerHTML;
            var editconditionDropDown = ej.buildTag("input#" + this._id + "_editCon.editconditionDropDown", "", {}, { type: 'text', tabindex: 1, accesskey: 'e' })[0].outerHTML;
            var removeBtn = ej.buildTag("button#" + this._id + "_removeBtn.dialogremoveBtn e-icon", "", {}, { name: _proxy._getLocalizedLabels("Remove") })[0].outerHTML;
            var value1 = ej.buildTag("input#" + this._id + "_conFrom.conditionFrom", "", {}, { name: 'inputVal', type: 'text', tabindex: 2 })[0].outerHTML;
            var value2 = ej.buildTag("input#" + this._id + "_conTo.conditionTo", "", {}, { name: 'inputVal', type: 'text', tabindex: 3 })[0].outerHTML;
            var backcolor = ej.buildTag("input#" + this._id + "_backcolor.backcolor", "", {}, { type: 'text', tabindex: 4, accesskey: 'b' })[0].outerHTML;
            var bordercolor = ej.buildTag("input#" + this._id + "_bordercolor.bordercolor", "", {}, { type: 'text', tabindex: 5, accesskey: 'c' })[0].outerHTML;
            var borderrange = ej.buildTag("input#" + this._id + "_borderrange.borderrange", "", {}, { type: 'text', tabindex: 6, accesskey: 'r' })[0].outerHTML;
            var borderstyle = ej.buildTag("input#" + this._id + "_borderstyle.borderstyle", "", {}, { type: 'text', tabindex: 7, accesskey: 's' })[0].outerHTML;
            var fStyle = ej.buildTag("input#" + this._id + "_fStyle.fStyle", "", {}, { type: 'text', tabindex: 8, accesskey: 'o' })[0].outerHTML;
            var fSize = ej.buildTag("input#" + this._id + "_fSize.fSize", "", {}, { type: 'text', tabindex: 9, accesskey: 'l' })[0].outerHTML;
            var conditionDiv = ej.buildTag("div#" + this._id + "_conditionDlg.conditionDlg", "<table class='conditionformatTbl'><tr><td>" + conditionLbl + "</td><td>" + conditionDropDown + "</td><td>" + editcondition + "</td><td>" + editconditionDropDown + "</td><td>" + removeBtn + "</td></tr><tr><td>" + value1Lbl + "</td><td>" + value1 + "</td><td>" + value2Lbl + "</td><td>" + value2 + "</td></tr><tr><td>" + backcolorLbl + "</td><td>" + backcolor + "</td><td>" + borderrangeLbl + "</td><td>" + borderrange + "</td></tr><tr><td>" + bordercolorLbl + "</td><td>" + bordercolor + "</td><td>" + borderstyleLbl + "</td><td>" + borderstyle + "</td></tr><tr><td>" + fStyleLbl + "</td><td>" + fStyle + "</td><td>" + fSizeLbl + "</td><td>" + fSize + "</td></tr></table>")[0].outerHTML;
            dialogContent = conditionDiv;
            var OKBtn = ej.buildTag("button#" + this._id + "_OKBtn.dialogOKBtn", "", {}, { name: _proxy._getLocalizedLabels("OK") })[0].outerHTML;
            var CancelBtn = ej.buildTag("button#" + this._id + "_CancelBtn.dialogCancelBtn", "", {}, { name: _proxy._getLocalizedLabels("Cancel") })[0].outerHTML;
            dialogFooter = ej.buildTag("div", OKBtn + CancelBtn, { "float": "right", "margin": "20px 39px 6px" })[0].outerHTML;
            $("#" + _proxy._id).after(ejDialog);

            $("#" + this._id + "_clientDlg").append(dialogContent + dialogFooter);

            $("#" + this._id + "_clientDlg").ejDialog({ width: "auto", target: "#" + _proxy._id, enableResize: false });
            $(".e-titlebar").prepend(ej.buildTag("div", dialogTitle, { "display": "inline" })[0].outerHTML)[0];
            $("#" + this._id + "_OKBtn" + "," + "#" + this._id + "_CancelBtn" + "," + "#" + this._id + "_removeBtn").ejButton();
            $("#" + this._id + "_OKBtn" + "," + "#" + this._id + "_CancelBtn").css({ margin: "0 0 0 10px", width: "67px" });
            $("#" + this._id + "_removeBtn").css({ width: "29px", height: "29px" });
            $("#" + this._id + "_conFrom" + "," + "#" + this._id + "_conTo").css({ width: "136px", height: "26px" });
            $("#" + this._id + "_conditionDlg").css({ margin: "13px 0 0 0" });
            $("#" + this._id + "_editcond" + "," + "#" + this._id + "_value2" + "," + "#" + this._id + "_borderrangeLbl" + "," + "#" + this._id + "_borderstyleLbl" + "," + "#" + this._id + "_fSizeLbl").css({ margin: "0 0 0 15px" });

            $("#" + this._id + "_conTo").attr("disabled", "disabled");
            this.removeBtn = $("#" + this._id + "_removeBtn").data("ejButton");
            okBtn = $("#" + this._id + "_OKBtn").data("ejButton");
            cancelBtn = $("#" + this._id + "_CancelBtn").data("ejButton");
            this.removeBtn.setModel({ "enabled": false });
            okBtn.setModel({ text: _proxy._getLocalizedLabels("OK") });
            cancelBtn.setModel({ text: _proxy._getLocalizedLabels("Cancel") });

            $("#" + this._id + "_conType").ejDropDownList({
                change: "_onActiveConditionChange",
                dataSource: ["Lesser", "Greater", "Equal", "Not Equal", "Between", "Not Between"],
                selectedIndices : [0],
                width: "142px",
                height: "30px"
            });
            $("#" + this._id + "_editCon").ejDropDownList({
                change: "_onEditConditionChange",
                dataSource: this._list,
                selectedIndices: [0],
                width: "142px",
                height: "30px"
            });
            $("#" + this._id + "_backcolor").ejDropDownList({
                dataSource: ["AliceBlue", "Black", "Blue", "Brown", "Gold", "Green", "Lime", "Maroon", "Orange", "Pink", "Red", "Violet", "White", "Yellow"],
                selectedIndices: [0],
                width: "142px",
                height: "30px"
            });
            $("#" + this._id + "_bordercolor").ejDropDownList({
                dataSource: ["AliceBlue", "Black", "Blue", "Brown", "Gold", "Green", "Lime", "Maroon", "Orange", "Pink", "Red", "Violet", "White", "Yellow"],
                selectedIndices: [0],
                width: "142px",
                height: "30px"
            });
            $("#" + this._id + "_borderrange").ejDropDownList({
                dataSource: ["1", "2", "3", "4", "5", "6", "7", "8", "9"],
                selectedIndices: [0],
                width: "142px",
                height: "30px"
            });
            $("#" + this._id + "_borderstyle").ejDropDownList({
                dataSource: ["Solid", "Dashed", "Dotted", "Double", "Groove", "Inset", "Outset", "Ridge", "None"],
                selectedIndices: [0],
                width: "142px",
                height: "30px"
            });
            $("#" + this._id + "_fStyle").ejDropDownList({
                dataSource: ["Algerian", "Arial", "Bodoni MT", "Britannic Bold", "Cambria", "Calibri", "Courier New", "DejaVu Sans", "Forte", "Gerogia", "Impact", "Segoe UI", "Tahoma", "Times New Roman", "Verdana", "None"],
                selectedIndices: [0],
                width: "142px",
                height: "30px"
            });
            $("#" + this._id + "_fSize").ejDropDownList({
                dataSource: ["12", "13", "14", "15", "16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26", "27", "28", "29", "30"],
                 selectedIndices: [0],
                width: "142px",
                height: "30px"
            });
            _proxy.element.find("#" + this._id + "_conType").ejDropDownList("option", "change", ej.proxy(_proxy._onActiveConditionChange, _proxy));
            _proxy.element.find("#" + this._id + "_editCon").ejDropDownList("option", "change", ej.proxy(_proxy._onEditConditionChange, _proxy));

            this.isNewFormat = true;
            this.formattingArrayClone = this._getFArrayfromMArray("formattingArrayClone");
            this.removedItems = this._getFArrayfromMArray("removedItems");
            this.ddlEditCondition = $("#" + this._id + "_editCon").data("ejDropDownList");
            this.ddlConditionType = $("#" + this._id + "_conType").data("ejDropDownList");
            this.ddlFont = $("#" + this._id + "_fStyle").data("ejDropDownList");
            this.ddlFontSize = $("#" + this._id + "_fSize").data("ejDropDownList");
            this.ddlBackColor = $("#" + this._id + "_backcolor").data("ejDropDownList");
            this.ddlBorderColor = $("#" + this._id + "_bordercolor").data("ejDropDownList");
            this.ddlBorderStyle = $("#" + this._id + "_borderstyle").data("ejDropDownList");
            this.ddlBorderWidth = $("#" + this._id + "_borderrange").data("ejDropDownList");
            this.element.find("#" + this._id + "_conFrom" + "," + "#" + this._id + "_conTo").keypress(function (event) {
                if (event.which == 8 || event.which == 0) {
                    return true;
                }

                if (event.which < 46 || event.which > 58) {
                    return false;
                }
                if ((event.which == 46 && $(this).val().indexOf('.') != -1) || event.which == 47) {
                    return false;
                }
            });
            _proxy = this;
            $("#" + this._id + "_OKBtn").click(function () {
                _proxy._clearGrid();
                if ($("#" + _proxy._id + '_conFrom').val() == '') {
                    alert('Enter Operand1');
                }

                else if (!($("#" + _proxy._id + "_conTo").attr("disabled")) && $("#" + _proxy._id + '_conTo').val() == '') {
                    alert('Enter Operand2');
                }
                else {
                    _proxy._applyFormatting();
                }
                if (_proxy.isNewFormat) {
                    var k = _proxy.ddlEditCondition._selectedIndices[0];
                    var count = _proxy.formattingArrayClone.length / 10;
                    if (count > 0) {
                        for (i = count - 1; i < count; i++) {
                            var iterator = i;
                            for (k = 0; k < _proxy.formattingArrayClone.length - 1; k += 10) {
                                var format = "Format" + " " + i;
                                if (_proxy.formattingArrayClone[k + 9] != format && _proxy.formattingArrayClone[k + 9] == "Add New") {
                                    iterator = iterator + 1;
                                    _proxy._list.push(format);
                                }
                            }
                            if (_proxy.formattingArrayClone[k - 1] == "Add New")
                                _proxy.formattingArrayClone[k - 1] = format;
                        }
                    }
                    _proxy.isNewFormat = true;
                }
                else {
                    alert('The given value not matched')
                    _proxy.isNewFormat = true;
                }
            });
            var count = _proxy.formattingArrayClone.length / 10;
            if (count > 0) {
                _proxy.ddlEditCondition.selectItemByText(this.formattingArrayClone[this.formattingArrayClone.length - 1]);
                _proxy._onEditConditionChange();
            }
            $("#" + this._id + "_removeBtn").click(function () {
                _proxy._removeFormatting();
            });
            $("#" + this._id + "_CancelBtn").click(function () {
                _proxy.element.find(".e-dialog, .clientDialog").remove();
            });
            return false;
        },

        _onActiveConditionChange: function (args) {
            if (args.selectedText == "Between" || args.selectedText == "Not Between")
                $("#" + this._id + "_conTo").removeAttr("disabled");
            else {
                $("#" + this._id + "_conTo").attr("disabled", "disabled");
                $("#" + this._id + "_conTo").val("");
            }
        },

        _onEditConditionChange: function () {
            if (this.ddlEditCondition._selectedIndices[0] > 0) {

                this.formattingArrayClone = this._getFArrayfromMArray("formattingArrayClone");
                var i = (this.ddlEditCondition._selectedIndices[0] * 10) - 10;
                this.ddlConditionType.selectItemByText(this.formattingArrayClone[i]);
                $("#" + this._id + '_conFrom').val(this.formattingArrayClone[i + 1]);
                $("#" + this._id + '_conTo').val(this.formattingArrayClone[i + 2]);
                this.ddlBackColor.selectItemByText(this.formattingArrayClone[i + 3]);
                this.ddlBorderColor.selectItemByText(this.formattingArrayClone[i + 4]);
                this.ddlBorderStyle.selectItemByText(this.formattingArrayClone[i + 5]);
                this.ddlBorderWidth.selectItemByText(this.formattingArrayClone[i + 6]);
                this.ddlFont.selectItemByText(this.formattingArrayClone[i + 7]);
                this.ddlFontSize.selectItemByText(this.formattingArrayClone[i + 8]);
                this.removeBtn.option('enabled', true);
                _proxy._onConditionTypeChange();
            }

            else {
                this.removeBtn.option('enabled', false);
                _proxy._clearWindow();
            }
        },

        _clearWindow: function () {
            $("#" + this._id + "_clientDlg").find("#" + this._id + "_conFrom" + "," + "#" + this._id + "_conTo").val('');
            $("#" + this._id + "_conTo").disabled = true;
        },

        _onConditionTypeChange: function () {
            $("#" + this._id + "_conTo").disabled = !(this.ddlConditionType._selectedIndices[0] == 4 || this.ddlConditionType._selectedIndices[0] == 5);
        },

        _removeFormatting: function () {
            if (!this.removeBtn.option('enabled')) return;
            j = 0;

            this.removedItems = this._getFArrayfromMArray("removedItems");
            var condition = confirm('Are you sure you want to remove the selected format?');
            if (condition == true) {
                var Index = this.ddlEditCondition._selectedIndices[0];
                if (Index == 1) {
                    this.removeBtn.option('enabled', false);
                }
                _proxy.removedItems[_proxy.removedItems.length] = _proxy.ddlEditCondition._selectedValue;


                this._list.pop(this.ddlEditCondition._selectedIndices[0] - 1);
                if (Index - 1 != 0) {
                    _proxy.ddlEditCondition._selectedIndices[0] = Index - 1;
                    _proxy.ddlEditCondition.selectItemByText(this.formattingArrayClone[((Index * 10) - 10) - 1]);
                }
                else {
                    if (this._list.length > 1) {
                        _proxy.ddlEditCondition._selectedIndices[0] = Index;
                        _proxy.ddlEditCondition.selectItemByText(this.formattingArrayClone[((Index * 10) - 10) + 9]);
                    }
                    else {
                        _proxy.ddlEditCondition._selectedIndices[0] = Index - 1;
                    }
                }
                for (i = ((Index - 1) * 10) + 0; i <= ((Index - 1) * 10) + 9; i++) {
                    this.formattingArrayClone[i] = "N/A";
                }
                var fArray = this.formattingArrayClone;
                this.formattingArrayClone = new Array();
                for (i = 0; i < fArray.length; i++) {
                    if (fArray[i] != "N/A") {
                        this.formattingArrayClone[j] = fArray[i];
                        j++
                    }
                }
                for (i = 0; i < this.mdformattingArray.length; i++) {
                    if (this.mdformattingArray[i][0] == _proxy._id) {
                        this.mdformattingArray[i][1] = this.formattingArrayClone;
                        this.mdformattingArray[i][4] = this.removedItems;
                    }
                }
                _proxy._onEditConditionChange();
                _proxy._clearGrid();
                _proxy._applyFormatting();
                _proxy.element.find(".e-dialog, .clientDialog").remove();
            }
        },

        _clearGrid: function () {
            var cellCollection = $(".pivotGridTable .value");
            for (i = 0; i < cellCollection.length; i++) {            
                var isHidden = false;
                if (cellCollection[i].className == "value") {
                    if (this.defaultStyleVCell.length == 0) {
                        this.defaultStyleVCell = this.defaultStyleSCell;
                    }
					if(!$(cellCollection[i]).is(":visible")) isHidden = true;
                    $(cellCollection[i]).attr('style', this.defaultStyleVCell);

                }
                else {
                    if (this.defaultStyleSCell.length == 0) {
                        this.defaultStyleSCell = this.defaultStyleVCell;
                    }
					if(!$(cellCollection[i]).is(":visible")) isHidden = true;
                    $(cellCollection[i]).attr('style', this.defaultStyleSCell);

                }
				if(isHidden)
                    $(cellCollection[i]).hide();
            }
        },

        _applyFormatting: function () {
            if ($("#" + this._id + '_conFrom').val() != '') {

                var cellCollection = $(".pivotGridTable .value"), currentVal, currentValClone, isAvailable = false;

                this.formattingArrayClone = this._getFArrayfromMArray("formattingArrayClone");
                this.removedItems = this._getFArrayfromMArray("removedItems");

                if (this.formattingArrayClone.length == 0) {
                    isAvailable = true;
                }

                var value1 = $("#" + this._id + '_conFrom').val(), value2 = $("#" + this._id + '_conTo').val(), cdnTxt = this.ddlConditionType._selectedValue, color = this.ddlBackColor._selectedValue, bordercolor = this.ddlBorderColor._selectedValue,
                    borderrange = this.ddlBorderWidth._selectedValue, fStyle = this.ddlFont._selectedValue, fSize = this.ddlFontSize._selectedValue, borderstyle = this.ddlBorderStyle._selectedValue, editCdn = this.ddlEditCondition._selectedValue;

                currentValClone = [this.ddlConditionType._selectedValue, value1, value2, this.ddlBackColor._selectedValue, this.ddlBorderColor._selectedValue, this.ddlBorderStyle._selectedValue, this.ddlBorderWidth._selectedValue, this.ddlFont._selectedValue, this.ddlFontSize._selectedValue, this.ddlEditCondition._selectedValue];

                j = 0; l = 0;
                cdnFmtLen = this.formattingArrayClone.length + 10;
                if (!this.isDrillDown) {
                    if (this.ddlEditCondition._selectedIndices[0] == 0) {
                        for (i = this.formattingArrayClone.length; i < cdnFmtLen; i++) {
                            this.formattingArrayClone[i] = currentValClone[l];
                            l++;
                        }
                    }
                    else {

                        cdnFmtLen = (this.ddlEditCondition._selectedIndices[0] * 10) - 10;

                        if (this.ddlEditCondition._selectedIndices[0] > 0) {
                            for (i = cdnFmtLen; i < (cdnFmtLen + 10) ; i++) {
                                this.formattingArrayClone[i] = currentValClone[l];
                                l++;
                            }
                        }
                    }
                }
                this.isDrillDown = false;

                _proxy._setDefaultStyle();

                j = 0; l = 0;

                if (isAvailable) {
                    this.mdformattingArray[this.mdformattingArray.length] = new Array();
                    this.mdformattingArray[this.mdformattingArray.length - 1] = [_proxy._id, this.formattingArrayClone, this.defaultStyleSCell, this.defaultStyleVCell, this.removedItems];
                }

                for (l = 0; l < this.formattingArrayClone.length; l++) {
                    value = new Array();
                    for (i = 0; i < cellCollection.length; i++) {
                        if (value.length == 0) {
                            value[value.length] = this.formattingArrayClone[l + 1];
                            value[value.length] = this.formattingArrayClone[l + 2];
                            value[value.length] = this.formattingArrayClone[l];
                        }
                        if (this._getResult(this.formattingArrayClone[l + 1], this.formattingArrayClone[l + 2], Number($(cellCollection[i]).text().replace(/[^0-9\.]+/g, "")), this.formattingArrayClone[l])&& ($(cellCollection[i]).text() != "" || $(cellCollection[i]).text() != null)) {
                            cellCollection[i].style.backgroundColor = this.formattingArrayClone[l + 3];
                            cellCollection[i].style.borderColor = this.formattingArrayClone[l + 4];
                            cellCollection[i].style.borderStyle = this.formattingArrayClone[l + 5];
                            cellCollection[i].style.borderWidth = this.formattingArrayClone[l + 6] + "px";
                            cellCollection[i].style.fontFamily = this.formattingArrayClone[l + 7];
                            cellCollection[i].style.fontSize = this.formattingArrayClone[l + 8] + "px";
                        }
                    }
                    l += 9;
                }
                _proxy.element.find(".e-dialog, .clientDialog").remove();
            }
        },

        _getResult: function (conditionalValue1, conditionalValue2, cellValue, operator) {
            switch (operator) {
                case "Lesser":
                    return cellValue < conditionalValue1;
                    break;
                case "Greater":
                    return cellValue > conditionalValue1;
                    break;
                case "Equal":
                    if (cellValue == conditionalValue1)
                        return cellValue;
                    break;
                case "Not Equal":
                    return cellValue != conditionalValue1;
                    break;
                case "Between":
                    return (conditionalValue1 < conditionalValue2 && cellValue > conditionalValue1 && cellValue < conditionalValue2) ||
                    (conditionalValue1 > conditionalValue2 && cellValue < conditionalValue1 && cellValue > conditionalValue2) ||
                    (cellValue == conditionalValue1 && cellValue == conditionalValue2);
                    break;
                case "Not Between":
                    return !((conditionalValue1 < conditionalValue2 && cellValue > conditionalValue1 && cellValue < conditionalValue2) ||
                    (conditionalValue1 > conditionalValue2 && cellValue < conditionalValue1 && cellValue > conditionalValue2) ||
                    (cellValue == conditionalValue1 && cellValue == conditionalValue2));
                    break;
            }
        },

        _setDefaultStyle: function () {
            var cellCollection = $(".pivotGridTable .value");
            this.defaultStyleSCell = this._getFArrayfromMArray("defaultStyleSCell");
            this.defaultStyleVCell = this._getFArrayfromMArray("defaultStyleVCell");

            if (this.defaultStyleVCell.length == 0 || this.defaultStyleSCell.length == 0) {
                for (j = 0; j < cellCollection.length; j++) {
                    if (cellCollection[j].className == "value" && this.defaultStyleVCell.length == 0 && cellCollection[j].currentStyle != undefined) {
                        this.defaultStyleVCell = cellCollection[j].currentStyle;
                    }
                    else if (cellCollection[j].className == "summary value" && this.defaultStyleSCell.length == 0 && cellCollection[j].currentStyle != undefined) {
                        this.defaultStyleSCell = cellCollection[j].currentStyle;
                    }
                }
            }
        },

        _renderControlSuccess: function (msg) {
            _proxy = this;
            try {
                if (msg[0] != undefined && msg.length > 0) {
                    _proxy.setJSONRecords(msg[0].Value); _proxy.setOlapReport(msg[1].Value);
                    if (msg[2] != null && msg[2] != undefined && msg[2].Key == "PageSettings") {
                        if (_proxy.model.enableVirtualScrolling) {
                            _proxy._categPageCount = Math.ceil(JSON.parse(msg[3].Value).Column / JSON.parse(msg[2].Value).CategorialPageSize);
                            _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[3].Value).Row / JSON.parse(msg[2].Value).SeriesPageSize);
                            _proxy._categCurrentPage = JSON.parse(msg[2].Value).CategorialCurrentPage;
                            _proxy._seriesCurrentPage = JSON.parse(msg[2].Value).SeriesPageSize;
                        }
                        else if ($(".e-pivotpager")[0] != null && $(".e-pivotpager")[0] != undefined)
                            _proxy._pagerObj.initPagerProperties(JSON.parse(msg[3].Value), JSON.parse(msg[2].Value));
                        if (msg[4] != null && msg[4] != undefined)
                            _proxy.model.customObject = msg[4].Value;
                    }
                    if (msg[2] != null && msg[2] != undefined && msg[2].Key != "PageSettings" || (msg[2] != null && msg[2] != undefined && msg[2].Key == "customobject"))
                        _proxy.model.customObject = msg[2].Value;
                }
                else if (msg.d != undefined && msg.d.length > 0) {
                    _proxy.setJSONRecords(msg.d[0].Value); _proxy.setOlapReport(msg.d[1].Value);
                    if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "PageSettings") {
                        if (_proxy.model.enableVirtualScrolling) {
                            _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Column / JSON.parse(msg.d[2].Value).CategorialPageSize);
                            _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[3].Value).Row / JSON.parse(msg.d[2].Value).SeriesPageSize);
                            _proxy._categCurrentPage = JSON.parse(msg.d[2].Value).CategorialCurrentPage;
                            _proxy._seriesCurrentPage = JSON.parse(msg.d[2].Value).SeriesCurrentPage;
                        }
                        else if ($(".e-pivotpager")[0] != null && $(".e-pivotpager")[0] != undefined)
                            _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[3].Value), JSON.parse(msg.d[2].Value));
                        if (msg.d[4] != null && msg.d[4] != undefined)
                            _proxy.model.customObject = msg.d[4].Value;
                    }
                    if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Value == "PageSettings"|| (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "customobject"))
                        _proxy.model.customObject = msg.d[2].Value;
                }
                else if (!ej.isNullOrUndefined(msg) && ej.isNullOrUndefined(msg.d)) {
                    if (msg.PivotRecords != undefined) {
                        _proxy.setJSONRecords(msg.PivotRecords); _proxy.setOlapReport(msg.OlapReport);
                        if (msg.HeaderCounts != undefined && _proxy.model.enableVirtualScrolling) {
                            _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                            _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                            _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                            _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                        }
                        else if (msg.HeaderCounts != undefined && $(".e-pivotpager")[0] != null && $(".e-pivotpager")[0] != undefined)
                            _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                    }
                    else if (msg.JsonRecords != undefined && _proxy.model.enableJSONRendering == true) {
                        _proxy.setJSONRecords(JSON.stringify(msg.JsonRecords)); _proxy.setOlapReport(JSON.stringify(msg.PivotReport));
                    }
                    else if (msg.JsonRecords != undefined) {
                        _proxy.setJSONRecords(msg.JsonRecords); _proxy.setOlapReport(msg.PivotReport);
                    }
                    if (msg.customObject != null && msg.customObject != undefined)
                        _proxy.model.customObject = msg.customObject;
                }
                if (_proxy.getOlapReport() != undefined && _proxy.getOlapReport() != "" && _proxy.model.enableVirtualScrolling != true) {
                    try {
                        _proxy._dataModel = JSON.parse(_proxy.getOlapReport()).DataModel;
                    }
                    catch (err) {
                        _proxy._dataModel = "";
                    }
                }
                if (_proxy.model.enableGroupingBar == true && this._dataModel == "Pivot") {
                    report = JSON.parse(_proxy.getOlapReport());
                    this._pivotTableFields = JSON.parse(report.ItemsProperties);
                    _proxy._createGroupingBar(report);
                }
                if (_proxy.model.afterServiceInvoke != null)
                    _proxy._trigger("afterServiceInvoke", { action: ej.PivotGrid.GridAction.Initialize, element: _proxy.element, customObject: _proxy.model.customObject });
                if (_proxy.getJSONRecords() != null && _proxy.getJSONRecords().length > 0)
                    _proxy.renderControlFromJSON(_proxy.getJSONRecords());
                else
                    _proxy.renderControlFromJSON("");
				if (_proxy.model.enableConditionalFormatting && this.formattingArrayClone.length > 0) 
                    _proxy._applyFormatting();
                if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                    if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                        this._maxViewLoading.hide();
                    }
                    else
                        oclientWaitingPopup.hide();
                }
                else
                    ogridWaitingPopup.hide();
            }
            catch (err) {
            }
            var eventArgs = { action: "initialize", customObject: this.model.customObject, element: this.element };
            this._hideGrandTotal();
            this._trigger("renderSuccess", eventArgs);
        },

        _hideGrandTotal: function () {
            if ((!this.model.enableRowGrandTotal || !this.model.enableGrandTotal) && this._dataModel == "Pivot")
                $("#" + this._id).find("tr:last").hide();
            if ($("#" + this._id).find("tr:first th.summary").attr("p") != undefined && (!this.model.enableColumnGrandTotal || !this.model.enableGrandTotal) && this._dataModel == "Pivot") {
                var calcGTTh = $("#" + this._id).find("th.calc.gtot");
                for (cLen = 0; cLen < calcGTTh.length; cLen++) {
                    $("#" + this._id).find("th[p^=\"" + (parseInt($(calcGTTh[cLen]).attr("p").split(",")[0])) + ",\"]").hide();
                    $("#" + this._id).find("td[p^=\"" + (parseInt($(calcGTTh[cLen]).attr("p").split(",")[0])) + ",\"]").hide();
                }
            }
        },

        _createGroupingBar: function (report) {
            this._pivotFilter = _proxy._getLocalizedLabels("DragFieldHere");
            for (var i = 0; i < report.Filters.length; i++) {
                if (report.PivotColumns.length >= 1) {
                    for (var j = 0; j < report.PivotColumns.length; j++) {
                        if (report.Filters.length != 0 && !ej.isNullOrUndefined(report.Filters[i]) && report.Filters[i].Tag == report.PivotColumns[j].Tag) {
                            report.Filters.splice(i, 1);
                            i = -1;
                        }
                    }
                }
            }
            for (var i = 0; i < report.Filters.length; i++) {
                if (report.PivotRows.length >= 1) {
                    for (var j = 0; j < report.PivotRows.length; j++) {
                        if (report.Filters.length != 0 && !ej.isNullOrUndefined(report.Filters[i]) && report.Filters[i].Tag == report.PivotRows[j].Tag) {
                            report.Filters.splice(i, 1);
                            i = -1;
                        }
                    }
                }
            }
            for (var i = 0; i < report.Filters.length; i++) {
                if (report.PivotCalculations.length >= 1) {
                    for (var j = 0; j < report.PivotCalculations.length; j++) {
                        if (report.Filters.length != 0 && !ej.isNullOrUndefined(report.Filters[i]) && report.Filters[i].Tag == report.PivotCalculations[j].Tag) {
                            report.Filters.splice(i, 1);
                            i = -1;
                        }
                    }
                }
            }
            filterTag = _proxy._createPivotButtons(this._pivotFilter != null && report.Filters.length > 1 && report.Filters[0].Tag == report.Filters[1].Tag ? report.Filters.slice(1) : report.Filters, "filter");
            this._pivotFilter = filterTag;
            rowTag = _proxy._createPivotButtons(report.PivotRows, "row");
            this._pivotRow = rowTag;
            columnTag = _proxy._createPivotButtons(report.PivotColumns, "column");
            this._pivotColumn = columnTag;
            valueTag = _proxy._createPivotButtons(report.PivotCalculations, "values");
            this._pivotCalculation = valueTag;
        },

        _createPivotButtons: function (rows, axis) {
            if (!ej.isNullOrUndefined(rows)) {
                var rowBtns = "", tagAxis = axis == "column" ? "columns" : axis == "row" ? "rows" : axis == "filter" ? "Slicers" : "";
                if (tagAxis == "" && axis == "values") {
                    var tempAxis = this.element.find(".pivotButton:contains('Measures')").length > 0 ? this.element.find(".pivotButton:contains('Measures')").parent()[0].className.split(" ")[0] : "schemaColumn";
                    tagAxis = tempAxis == "schemaColumn" ? "columns" : "rows";
                }
                for (var i = 0; i < rows.length; i++) {
                    var l = this._ascdes.split("##"), count = 0; sort = "", filtered = "";
                    for (var j = 0; j < l.length; j++) {
                        if (l[j] == rows[i].Tag) {
                            count = 1;
                        }
                    }
                    if (count == 1) {
                        sort = "descending";
                    }
                    if (!ej.isNullOrUndefined(this._tempFilterData)) {
                        for (var j = 0; j < this._tempFilterData.length; j++) {
                            for (var key in this._tempFilterData[j]) {
                                if (key == rows[i].Tag && this._tempFilterData[j][rows[i].Tag].length > 0)
                                    filtered = "filtered";
                            }
                        }
                    }
                    rowBtns += ej.buildTag("span.pivotButton", ej.buildTag("span.dropIndicator")[0].outerHTML + ej.buildTag("button.pvtBtn#pivotButton" + rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName, (rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName))[0].outerHTML +
                      ej.buildTag("span.filter e-icon " + filtered)[0].outerHTML + ej.buildTag("span.sorting e-icon " + sort)[0].outerHTML + ej.buildTag("span.removeBtn e-icon")[0].outerHTML).attr("tag", tagAxis + ":" + rows[i].Tag)[0].outerHTML;
                }
                return rowBtns;
            }
        },

        _createFields: function (tableOlapGrid, tableWidth, cellWidth) {
		    this.element.find(".groupingBarPivot").remove();
            this.element.prepend(ej.buildTag("div#groupingBarPivot.groupingBarPivot")[0].outerHTML);
            this.element.find(".groupingBarPivot").prepend(ej.buildTag("div.valueColumn", ej.buildTag("span.values", "")[0].outerHTML + ej.buildTag("span.columns", "")[0].outerHTML, { "width":"100%"})); // Math.max(valueColumnWidth, 280) - 5 
            this.element.find(".groupingBarPivot").prepend(ej.buildTag("div.drag", ej.buildTag("span", "", { "width": "5px", "display": "inline-block" })[0].outerHTML + _proxy._getLocalizedLabels("DragFieldHere"), { "width": "100%" })[0].outerHTML);// Math.max(filterWidth, 280)
            if (this._pivotFilter != "" && this._pivotFilter != null) {
                this.element.find(".drag").text("");
            }
            this.element.find(".drag").append(this._pivotFilter);
            this.element.find(".values").append(this._pivotCalculation);
            this.element.find(".columns").append(this._pivotColumn);
            if (_proxy.element.find(".values .pivotButton").length == 0) {
                _proxy.element.find(".values").append(ej.buildTag("span", "", { "width": "5px", "display": "inline-block" })[0].outerHTML + _proxy._getLocalizedLabels("ValueArea"));
            }
            if (_proxy.element.find(".columns .pivotButton").length == 0) {
                _proxy.element.find(".columns").append(ej.buildTag("span", "", { "width": "5px", "display": "inline-block" })[0].outerHTML + _proxy._getLocalizedLabels("ColumnArea"));
            }
            if (_proxy.element.find(".rows .pivotButton").length == 0) {
                _proxy.element.find(".rows").append(ej.buildTag("span", "", { "width": "5px", "display": "inline-block" })[0].outerHTML + _proxy._getLocalizedLabels("RowArea"));
            }
            if (this.element.find(".rowheader").length == 0 && this.element.find(".colheader").length == 0 && this.element.find(".values .pivotButton").length == 0 || this.element.find(".pivotGridTable th").length == 0) {
                var rowArea = ej.buildTag("div.emptyRows", ej.buildTag("span.rows", _proxy._getLocalizedLabels("RowArea"), { "width": "5px" })[0].outerHTML, { "width": 140 })[0].outerHTML;
                this.element.find(".groupingBarPivot").append(rowArea);
                if (this.model.enableDeferUpdate && this._pivotRow != "") {
                    this.element.find(".rows").text("");
                    this.element.find(".rows").append(this._pivotRow);
                }
                this.element.find(".emptyRows,.rows").ejDroppable({
                });
            }
            this.element.find(".values").find(".filter").remove();
            this.element.find(".values").find(".sorting").remove();
            this.element.find(".drag").find(".sorting").remove();
            this.element.find(".pivotButton .pvtBtn").ejButton({ size: "normal" }).ejDraggable({
                handle: 'button', clone: true,
                cursorAt: { left: -5, top: -5 },
                dragStart: ej.proxy(function (args) {
                    this._isDragging = true;
                }, this),
                dragStop: ej.proxy(this._onPvtBtnDropped, this),
                helper: ej.proxy(function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                    else
                        return false;
                }, this)
            });

            this.element.find(".drag, .values, .columns, .grpRow, .rows").ejDroppable({
            });
               if (this.element.find(".pivotGridTable").find("thead th:eq(1)").length > 0) {
                    this.element.find(".groupingBarPivot").width($(".pivotGridTable").width());
               }
               else if (this.element.find(".pivotGridTable").find("thead").length > 0) {
                   if(!this.model.isResponsive)
                   this.element.find(".groupingBarPivot").width($(".rows").width() + 150);
               }
               else
                   this.element.find(".groupingBarPivot").width(300);
            _proxy.element.find(".rows").height((this.model.enableDeferUpdate) ? (this.element.find(".grpRow").height()) - 8 : (this.element.find(".grpRow").height()));
            if (this.element.find(".rows").width() > 140 || (this.model.enableDeferUpdate) && _proxy.element.find(".rows").width() < _proxy.element.find(".grpRow").width()) {
                if (this.enableDeferUpdate)
                    this.element.find(".rows").width(this.element.find(".colheader").length == 0 ? this.element.find(".grpRow").width() - 140 : this.element.find(".grpRow").width());
                else
                    _proxy.element.find(".rows").width((_proxy.model.enableDeferUpdate) ? (_proxy.element.find(".grpRow").width()) - 8 : (_proxy.element.find(".grpRow").width()));
            }
            if (this.element.find(".pivotGridTable thead").length > 0 || this.model.enableDeferUpdate && this.element.find(".pivotGridTable thead").length == 0) {
                if (this.model.isResponsive && this.element.find(".values .pivotButton").length == 0 && this.element.find(".columns .pivotButton").length == 0) {
                    this.element.find(".values").width(140);
                }
                else
                    this.element.find(".values").width(this.element.find(".grpRow").length > 0 ? this.model.enableDeferUpdate && this.element.find(".colheader").length == 0 ? this.element.find(".grpRow").width() : this.element.find(".grpRow").width() + 4 : 140);//this.element.find(".colheader").length==0?this.element.find(".grpRow").width()+140:
                if (this.model.enableDeferUpdate && this.element.find(".colheader").length == 0 && this.element.find(".pivotGridTable .rowheader").length != 0) {
                    var columnWidth = _proxy.element.find(".groupingBarPivot").width() - _proxy.element.find(".values").width();
                    if (columnWidth < 140)
                        this.element.find(".groupingBarPivot").width(this.element.find(".groupingBarPivot").width() + 140)
                }
                if (this.element.find(".pivotGridTable").find("thead th:eq(1)").length > 0 || this.model.enableDeferUpdate && _proxy.element.find(".columns .pivotButton").length == 0) {
                    var colWidth = this.element.find(".pivotGridTable").width() - this.element.find(".values").width() - 4;
                    if (this.element.find(".pivotGridTable").find("thead th:eq(2)").length == 0) {
                        this.element.find(".columns").css("min-width", colWidth >= 30 && this.element.find(".pivotGridTable .colheader").length == 0 ? 140 : colWidth);
                        if (_proxy.element.find(".columns .pivotButton").length == 0 || this.model.enableDeferUpdate && _proxy.element.find(".pivotGridTable .colheader").length == 0) {
                            if (_proxy.element.find(".columns .pivotButton").length == 0)
                                this.element.find(".columns").width(this.element.find(".groupingBarPivot").width() - this.element.find(".values").width() - 5);
                            this.element.find(".columns").addClass("widthSetter");
                        }
                    }
                    else {
                        if (colWidth < 140) {
                            this.element.find(".columns").css("min-width", colWidth).addClass("widthSetter");
                        }
                        else
                            this.element.find(".columns").width(colWidth);
                    }
                }
                else if (this.element.find(".pivotGridTable").find("thead th:eq(1)").length == 0 && !this.model.enableDeferUpdate) {
                    var colBtnCount = this.element.find(".columns .pivotButton").length < 1 ? 1 : this.element.find(".columns .pivotButton").length;
                    this.element.find(".columns").addClass("widthSetter");
                    this.element.find(".columns").css("min-width", (this.element.find(".groupingBarPivot").width() - this.element.find(".values").width() - 7) / colBtnCount);
                }
            }
            if (this.element.find(".drag").height() > 30) {
                var btnWidth = this.element.find(".groupingBarPivot").width() / (_proxy.element.find(".drag .pivotButton").length + 1)-5;
                _proxy.element.find(".drag .pivotButton").width(btnWidth);
            }
            if (this.element.find(".values").height() > 30) {
                var btnWidth = this.element.find(".values").width() / (_proxy.element.find(".values .pivotButton").length + 1) - 5;
                _proxy.element.find(".values .pivotButton").width(btnWidth);
            }
            if (this.element.find(".rows").height() > 30 && this.model.enableDeferUpdate && this.element.find(".pivotGridTable").find("thead th:eq(1)").length == 0) {
                var btnWidth = this.element.find(".rows").width() / (_proxy.element.find(".rows .pivotButton").length + 1) - 5;
                _proxy.element.find(".rows .pivotButton").width(btnWidth);
            }
            if (this.model.enableDeferUpdate && _proxy.element.find(".columns .pivotButton").length != 0) {
                var pvtBtnWidth = 0, columnWidth = 0;
                for (var i = 0; _proxy.element.find(".columns .pivotButton").length > i; i++) {
                    pvtBtnWidth = _proxy.element.find(".columns .pivotButton:eq(" + i + ")").width() + pvtBtnWidth + 7;
                }
                this.element.find('.pivotGridTable .colheader,.pivotGridTable tr:eq(0) .summary').each(function () {
                    columnWidth = columnWidth + $(this).width();
                });
                if (pvtBtnWidth >= columnWidth) {
                    var isempty = this.element.find(".pivotGridTable .colheader").length > 0 ? 1 : 0;
                    var colWidth = isempty == 0 ? 140 : columnWidth;
                    var btnWidth = colWidth / (_proxy.element.find(".columns .pivotButton").length + 1) - 5;
                    _proxy.element.find(".columns .pivotButton").width(btnWidth);
                }
            }
            if (this.element.find(".values").height() > this.element.find(".columns").height() || this.element.find(".columns").height() > this.element.find(".values").height()) {
                if (this.element.find(".values").height() > this.element.find(".columns").height()) {
                    this.element.find(".columns").height(this.element.find(".values").height());
                }
                else
                    this.element.find(".values").height(this.element.find(".columns").height());
            }
            this.element.find(".colheader, .rowheader, .value").addClass("e-droppable");
            this.element.find(".summary").removeClass("e-droppable");
        },

        _onPvtBtnDropped: function (args) {
            if (this._dataModel == "Pivot") {
                var buttonDrag = $(args.target).parents();
                for (var i = 0; i < buttonDrag.length; i++) {
                    if (buttonDrag[i].className.split(" ")[0] == "e-pivotschemadesigner") {
                        this.element.find(".dragClone").remove();
                        return;
                    }

                }
            }
            this._isDragging = false;
            this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
            var axis = null, axisName = "", headerTag = "", headerText, droppedClass, isFiltered = false, isSorted = false;
            if (this._dataModel == "Pivot") {
                headerText = args.element.text(), droppedClass = args.target.className.indexOf("e-droppable") >= 0 ? args.target.className.split(" ")[0] : "",
                    headerTag, isFiltered = args.element.find("~ .filterIndicator").length > 0 ? true : false,
                    isSorted = args.element.find("~ span:eq(0) span.descending").length > 0 ? true : false;
                if (droppedClass == "" && (args.target.tagName == "BUTTON" || args.target.tagName == "SPAN")) {
                    droppedClass = args.target.parentNode.parentElement.className.split(" ")[0];
                }
                for (var i = 0; i < this._pivotTableFields.length; i++) {
                    if (this._pivotTableFields[i].name == headerText) {
                        this._pivotTableFields[i].isSelected = true;
                        headerTag = this._pivotTableFields[i];
                    }
                }
            }
            this.element.find(".dragClone").remove();
            if (droppedClass == "rows" || droppedClass == "columns" || droppedClass == "values" || droppedClass == "drag") {
                if (droppedClass == "drag") {
                    this.element.find(".drag").text("");
                }
                var droppedPosition = "";
                try {
                    report = JSON.parse(this.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.getOlapReport();
                }
                $(args.element.parent()).remove();
				ogridWaitingPopup = this.element.data("ejWaitingPopup");
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.show();

                if (this._schemaData != null) {
                    this._schemaData.element.find("#pivotButton" + headerText).parent().remove();
                    this._schemaData._createPivotButton(headerText, droppedClass == "rows" ? "row" : droppedClass == "columns" ? "column" : droppedClass == "drag" ? "filter" : droppedClass == "values" ? "value" : "", isFiltered, isSorted, droppedPosition);
                }
                droppedClass = droppedClass == "rows" ? "schemaRow" : droppedClass == "columns" ? "schemaColumn" : droppedClass == "drag" ? "schemaFilter" : droppedClass == "values" ? "schemaValue" : "";
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                 eventArgs = JSON.stringify({ "action": "nodeDropped", "dropAxis": droppedClass + "::" + droppedPosition, "headerTag": JSON.stringify(headerTag), "sortedHeaders": this._ascdes, "currentReport": report, "customObject": serializedCustomObject });
                successMethod = this._renderControlSuccess;
                if (!this.model.enableDeferUpdate)
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.nodeDropped, eventArgs, this._pvtBtnDroppedSuccess);
                else
                {
                    if (!ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.hide();
                    this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._pvtBtnDroppedSuccess);
                }
            }
            else
                this._removePvtBtn(args);
        },
        _pvtBtnDroppedSuccess: function (args) {
               var report = null;
               if (!ej.isNullOrUndefined(args[0]) && args.length > 0) {
                       if (args[2] != null && args[2] != undefined)
                           this.model.customObject = args[2].Value;
                   }
               else if (!ej.isNullOrUndefined(args.d) && args.d.length > 0) {
                       if (args.d[2] != null && args.d[2] != undefined)
                           this.model.customObject = args.d[2].Value;
                   }
               else if (!ej.isNullOrUndefined(args) && !ej.isNullOrUndefined(args.OlapReport)) {
                       if (args != null && args.customObject != undefined)
                           this.model.customObject = args.customObject;
                   }
               if (this.model.afterServiceInvoke != null)
                   this._trigger("afterServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
               if (!this.model.enableDeferUpdate)
                   this._renderControlSuccess(args);
               else
                   this._deferUpdateSuccess(args);    
       },
        _deferUpdateSuccess: function (msg)
        {
            if(!ej.isNullOrUndefined(msg.PivotReport))
                _proxy.setOlapReport(msg.PivotReport);
            else
                _proxy.setOlapReport(msg.d[0].Value);
            if (this.model.enableGroupingBar) {
                report = JSON.parse(_proxy.getOlapReport());
                this._pivotTableFields = JSON.parse(report.ItemsProperties);
                _proxy._createGroupingBar(report);
                this.element.find(".grpRow .rows").text("");
                this.element.find(".grpRow .rows").children().remove();
                this.element.find(".grpRow .rows").append(_proxy._pivotRow);
                _proxy._createFields(null, $("table").width(), $("table th").outerWidth());
            }
        },
    
        _removePvtBtn: function (args) {
            if (args.type == "click") {
                var headerText = $($(args.target).siblings()[1]).text(), headerTag, report, eventArgs;
            }
            else {
                var headerText = args.element.text(), headerTag = args.element.parent().attr("tag"), eventArgs;
            }
            filterTag = "", filterItems = "", selectedElement = "", headerTag = "", uniqueName = "", report = "";
            selectedElement = _proxy.element.find(".pivotButton:contains(" + headerText + ")");
            for (var i = 0; i < this._pivotTableFields.length; i++) {
                if (this._pivotTableFields[i].name == headerText) {
                    this._pivotTableFields[i].isSelected = false;
                    headerTag = this._pivotTableFields[i];
                }
            }
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            filterTag = "schemaRow::" + headerText + "::FILTERED" + filterItems;
            var dropAxis = this._droppedClass != "" ? this._droppedClass : headerTag.pivotType == "PivotItem" ? "schemaRow" : "schemaValue", eventArgs, axisName, params;
            this._droppedClass = "";
            $(selectedElement).remove();
			ogridWaitingPopup = this.element.data("ejWaitingPopup");
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
            if (this._schemaData != null) {
                var schemaUncheck = this._schemaData._tableTreeObj.element.find("li:contains('" + headerText + "')");
                this._schemaData._tableTreeObj.uncheckNode(schemaUncheck);
            }
            else
            {
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "nodeStateModified", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "dropAxis": dropAxis + "::", "sortedHeaders": this._ascdes, "filterParams": filterTag, "currentReport": report, "customObject": serializedCustomObject });
                if (!this.model.enableDeferUpdate)
                    this.doAjaxPost("POST", this.model.url + "/" + "nodeStateModified", eventArgs, this._nodeStateModifiedSuccess);
                else {
                    this.doAjaxPost("POST", this.model.url + "/" + "nodeStateModified", eventArgs.replace("nodeStateModified", "nodeStateModifiedDeferUpdate"), this._nodeStateModifiedSuccess);
                    if (!ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.hide();
                }
            }
        },
        _nodeStateModifiedSuccess: function (report) {
            if (report[0] != undefined) {
                if (report[2] != null && report[2] != undefined)
                    this.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                if (report.d[2] != null && report.d[2] != undefined)
                    this.model.customObject = report.d[2].Value;
            }
            else {
                if (report.customObject != null && report.customObject != undefined)
                    this.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "nodeStateModified", element: this.element, customObject: this.model.customObject });
            if (!this.model.pivotControl.model.enableDeferUpdate)
                this._renderControlSuccess(report);
            else
                this._deferUpdateSuccess(report);
        },

        _drillDownSuccess: function (msg) {
            _proxy = this;
            if (msg[0] != undefined) {
                this.setJSONRecords(msg[0].Value);
                this.setOlapReport(msg[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg[1].Value;
                    if (msg[2].Value != undefined && msg[2].Value != "undefined")
                        oclientProxy.reports = msg[2].Value;
                }
                if (msg[3] != null && msg[3] != undefined && msg[3].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg[4].Value).Column / JSON.parse(msg[3].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg[4].Value).Row / JSON.parse(msg[3].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg[3].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg[3].Value).SeriesPageSize;
                    }
                    else if ($(".e-pivotpager")[0] != null && $(".e-pivotpager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg[4].Value), JSON.parse(msg[3].Value));
                    if (msg[5] != null && msg[5] != undefined)
                        _proxy.model.customObject = msg[5].Value;
                }
                if (msg[3] != null && msg[3] != undefined && msg[3].Key != "PageSettings")
                    _proxy.model.customObject = msg[3].Value;
            }
            else if (msg.d != undefined) {
                this.setJSONRecords(msg.d[0].Value);
                this.setOlapReport(msg.d[1].Value);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.d[1].Value;
                    if (msg.d[2].Value != undefined && msg.d[2].Value != "undefined")
                        oclientProxy.reports = msg.d[2].Value;
                }
                if (msg.d[3] != null && msg.d[3] != undefined && msg.d[3].Key == "PageSettings") {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.d[4].Value).Column / JSON.parse(msg.d[3].Value).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.d[4].Value).Row / JSON.parse(msg.d[3].Value).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.d[3].Value).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.d[3].Value).SeriesCurrentPage;
                    }
                    else if ($(".e-pivotpager")[0] != null && $(".e-pivotpager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.d[4].Value), JSON.parse(msg.d[3].Value));
                    if (msg.d[5] != null && msg.d[5] != undefined)
                        _proxy.model.customObject = msg.d[5].Value;
                }
                if (msg.d[3] != null && msg.d[3] != undefined && msg.d[3].Key != "PageSettings")
                    _proxy.model.customObject = msg.d[3].Value;
            }
            else {
                this.setJSONRecords(msg.PivotRecords);
                this.setOlapReport(msg.OlapReport);
                if (typeof (oclientProxy) != "undefined") {
                    oclientProxy.currentReport = msg.OlapReport;
                    if (msg.reports != undefined && msg.reports != "undefined")
                        oclientProxy.reports = msg.reports;
                }
                if (msg.length > 3) {
                    if (_proxy.model.enableVirtualScrolling) {
                        _proxy._categPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Column / JSON.parse(msg.PageSettings).CategorialPageSize);
                        _proxy._seriesPageCount = Math.ceil(JSON.parse(msg.HeaderCounts).Row / JSON.parse(msg.PageSettings).SeriesPageSize);
                        _proxy._categCurrentPage = JSON.parse(msg.PageSettings).CategorialCurrentPage;
                        _proxy._seriesCurrentPage = JSON.parse(msg.PageSettings).SeriesCurrentPage;
                    }
                    else if ($(".e-pivotpager")[0] != null && $(".e-pivotpager")[0] != undefined)
                        _proxy._pagerObj.initPagerProperties(JSON.parse(msg.HeaderCounts), JSON.parse(msg.PageSettings));
                }
                if (msg.customObject != null && msg.customObject != undefined)
                    _proxy.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: ej.PivotGrid.GridAction.DrillDown, element: this.element, customObject: this.model.customObject });
            this.renderControlFromJSON(this.getJSONRecords());
            try {
                this.model.currentReport = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                this.model.currentReport = this.getOlapReport();
            }
            this._trigger("drillSuccess", this.element);
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                if ($("#" + oclientProxy._id + "_maxView")[0] && this._maxViewLoading) {
                    this._maxViewLoading.hide();
                    oclientWaitingPopup.hide();
                }
                else if (oclientProxy.model.displaySettings.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (typeof oclientProxy != 'undefined') {
                if (oclientProxy && !oclientProxy.ogridObj._startDrilldown && !oclientProxy.ochartObj._startDrilldown || oclientProxy.model.displaySettings.mode == "gridOnly")
                    oclientWaitingPopup.hide();
            }
            else if (typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.hide();
            }
            if (typeof oclientProxy != 'undefined')
                oclientProxy.ogridObj._startDrilldown = false;
            else
                ogridWaitingPopup.hide();
            var eventArgs = { action: this._drillAction, customObject: this.model.customObject, element: this.element };
            this._trigger("renderSuccess", eventArgs);
        },

        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function (onComplete) {
                    $.proxy(onComplete, this);
                    var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, element: this.element };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    ogridWaitingPopup.hide();
                    var eventArgs = { "action": this._drillAction != "" ? this._drillAction : "initialize", "customObject": this.model.customObject, "element": this.element, "Message": msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        },

        doPostBack: function (url, params) {
            var form = $('<form>').attr({ 'action': url, 'method': 'POST', 'name': 'export' });
            var addParam = function (paramName, paramValue) {
                var input = $('<input type="hidden" title="params">').attr({
                    'id': paramName,
                    'name': paramName,
                    'value': paramValue
                }).appendTo(form);
            };
            for (var item in params)
                addParam(item, params[item]);
            form.appendTo(document.body).submit().remove();
        },

        refreshPagedPivotGrid: function (axis, pageNo) {
            if (typeof oclientWaitingPopup == 'undefined') {
                ogridWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                ogridWaitingPopup.show();
            }
            axis = axis.indexOf('categ') != -1 ? "categorical" : "series";
            var report;
            try {
                report = JSON.parse(this.getOlapReport()).Report;
            }
            catch (err) {
                report = this.getOlapReport();
            }
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.paging, JSON.stringify({ "action": "Paging", "pagingInfo": axis + ":" + pageNo, "currentReport": report, "layout": this.layout(), "customObject": null }), this._renderControlSuccess);
        },

        renderControlFromJSON: function (jsonObj) {
            _proxy = this;
            var tableOlapGrid = $("<table class=\"pivotGridTable\" cellspacing=\"0\" cellpadding=\"0\" role='grid'  aria-readonly='true'></table>");
            if (this.model.isResponsive)
                tableOlapGrid.addClass("e-table");
            this.rowCount = 0;
            if (jsonObj != null && jsonObj != "") {
                jQuery.each(jsonObj, function (index, value) {
                    if (this.Index[0] == 0)
                        _proxy.rowCount++;
                });
                var theadOlapGrid = $("<thead></thead>");
                var tbodyOlapGrid = $("<tbody></tbody>");
                var istheadOlapGrid = true; var columnHeaderHeight = 0;
                if (jsonObj[0].CSS == "none")
                    columnHeaderHeight = jsonObj[0].RowSpan - 1;
                else {
                    jQuery.each(jsonObj, function (index, value) {
                       // if (jsonObj[index].CSS == "colheader")
                        columnHeaderHeight += jsonObj[index].RowSpan;
                        return (jsonObj[index].CSS == "colheader");
                    });
                    columnHeaderHeight = columnHeaderHeight - 2;
                }
                jQuery.each(jsonObj, function (index, value) {
                    if (index < _proxy.rowCount) {
                        if (istheadOlapGrid) {
                            istheadOlapGrid = (index >= columnHeaderHeight) ? false : true;
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    if ((jsonObj[column].Index == "0,0") && (_proxy.model.enableGroupingBar) && _proxy._dataModel == "Pivot") {
                                        tRow = tRow + "<th class=" + "grpRow" + " p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='columnheader'>" + (jsonObj[column].State == 0 ? "<span style=\"margin-left: 0px\"></span>" : jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\" title=\"Collapsed\">&nbsp;</span>" : "")) + jsonObj[column].Value + ej.buildTag("div.rows", _proxy._pivotRow, {})[0].outerHTML + "</th>";
                                    }
                                    else
                                        tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + (jsonObj[column].CSS == "none" ? " p=" + jsonObj[column].Index : "\" p=" + jsonObj[column].Index) + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='columnheader'>" + (jsonObj[column].State == 0 ? "<span style=\"margin-left: 10px\"></span>" : (jsonObj[column].State == 2) ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\" title=\"Collapsed\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount) ; i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount) ; j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            if (jsonObj[i].Index != "0,0")
                                                jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(theadOlapGrid).append(tRow);
                        }
                        else {
                            var tRow = "<tr role='row'>";
                            for (var column = index; column < jsonObj.length; column = column + _proxy.rowCount) {
                                if (jsonObj[column].Span != "Block") {
                                    if (jsonObj[column].CSS == "rowheader" || jsonObj[column].CSS == "summary" || jsonObj[column].CSS == "summary") {
                                        if (_proxy.layout().toLowerCase() == "excellikelayout") {
                                            tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='rowheader'>" + (jsonObj[column].State == 2 ? "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\" class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\" class=\"collapse\ e-icon\">&nbsp;</span>" : "<span style=\"margin-left:" + (jsonObj[column].Level * 10) + "px\">&nbsp;</span>")) + jsonObj[column].Value + "</th>";
                                        }
                                        else {
                                            tRow = tRow + "<th" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='rowheader'>" + (jsonObj[column].State == 0 ? "<span style=\"margin-left: 10px\"></span>" : (jsonObj[column].State == 2) ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</th>";
                                        }
                                    }
                                    else
                                        tRow = tRow + "<td" + (jsonObj[column].CSS == "none" ? "" : " class=\"" + jsonObj[column].CSS) + "\" p=" + jsonObj[column].Index + " colspan=" + jsonObj[column].ColSpan + " rowspan=" + jsonObj[column].RowSpan + " role='gridcell'>" + (jsonObj[column].State == 2 ? "<span class=\"expand\ e-icon\">&nbsp;</span>" : (jsonObj[column].State == 1 ? "<span class=\"collapse\ e-icon\">&nbsp;</span>" : "")) + jsonObj[column].Value + "</td>";

                                    if (jsonObj[column].RowSpan > 1 && jsonObj[column].ColSpan <= 1) {
                                        for (var i = column + 1; i < column + jsonObj[column].RowSpan; i++)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan <= 1) {
                                        for (var i = column + _proxy.rowCount; i < column + (jsonObj[column].ColSpan * _proxy.rowCount) ; i = i + _proxy.rowCount)
                                            jsonObj[i].Span = "Block";
                                    }
                                    else if (jsonObj[column].ColSpan > 1 && jsonObj[column].RowSpan > 1) {
                                        for (var i = column; i < column + jsonObj[column].RowSpan; i++) {
                                            for (var j = i + _proxy.rowCount; j < i + (jsonObj[column].ColSpan * _proxy.rowCount) ; j = j + _proxy.rowCount)
                                                jsonObj[j].Span = "Block";
                                            if (jsonObj[i].Index != "0,0")
                                                jsonObj[i].Span = "Block";
                                        }
                                    }
                                }
                            }
                            tRow += "</tr>";
                            $(tbodyOlapGrid).append(tRow);
                        }
                    }
                });
               
            }
			this._unWireEvents();
            this._wireEvents();
			$(tableOlapGrid).append(theadOlapGrid);
            $(tableOlapGrid).append(tbodyOlapGrid);
			if(this._dataModel == "Pivot" && this.model.enableCollapseByDefault)
			{
				var tempReport = JSON.parse(this.getOlapReport()), rwCnt, clCnt;
				rwCnt = tempReport.Rows;
				clCnt = tempReport.Columns;				
				for (var rh = rwCnt; rh >= 0; rh--) {
					thRList = $(tableOlapGrid).find("th[p^='" + rh + ",']:has('span.collapse')[class^='rowheader']");
				for (var thcnt = 0; thcnt < thRList.length; thcnt++)
					this._collapseMember(tableOlapGrid, $(thRList[thcnt]).find('.collapse'));					
				}
				for (var ch = clCnt; ch >= 0; ch--) {
					thCList = $(tableOlapGrid).find("th[p$='," + ch + "']:has('span.collapse')[class$='colheader']");
				for (var thcnt = 0; thcnt < thCList.length; thcnt++)
					this._collapseMember(tableOlapGrid, $(thCList[thcnt]).find('.collapse'));					
				}
			}
            this.element.html("");
            if (_proxy.model.enableVirtualScrolling) {
                _proxy._createVirtualPivotGrid(tableOlapGrid);
                _proxy._applyVScrolling();
            }
            else if (this.model.enableGroupingBar && this._dataModel == "Pivot") {
                _proxy.element.append(tableOlapGrid);
                _proxy._createFields(tableOlapGrid, $("table").width(), $("table th").outerWidth());
            }
            else
                this.element.append(tableOlapGrid);
            if (_proxy.model.enableRTL) {
                this.element.find('table').addClass("e-rtl");
                this.element.find(".groupingBarPivot").addClass("e-rtl");
                this.element.find(".groupingBarPivot").width($(".pivotGridTable").width());
                this.element.find('table .rowheader,.colheader,.summary, .groupingBarPivot, .grpRow').css("text-align", "right");
            }
            else {
                this.element.find('table').removeClass("e-rtl");
                this.element.find(".groupingBarPivot").removeClass("e-rtl");
                this.element.find('table .rowheader,.colheader,.summary, .grpRow, .groupingBarPivot ').css("text-align", "left");
            }
			if (_proxy.model.enableConditionalFormatting && this.formattingArrayClone.length > 0) {
				if (_proxy._drillAction != "") _proxy.isDrillDown = true;
				_proxy._applyFormatting();
			}
          
        },

        _createVirtualPivotGrid: function (tableOlapGrid) {
            var verticalScroll = _proxy._seriesPageCount > 1 ? ej.buildTag("div.vScrollPanel")[0].outerHTML : "";
            var horizontalScroll = _proxy._categPageCount > 1 ? ej.buildTag("div.hScrollPanel")[0].outerHTML : "";
            var oGridDiv = ej.buildTag("td.virtualScrollGrid", tableOlapGrid[0].outerHTML)[0];
            var vertiScrollTd = ej.buildTag("td.virtualScrollElement", verticalScroll);
            var horiScrollTd = ej.buildTag("td.virtualScrollElement", horizontalScroll);
            var trow = ej.buildTag("tr.virtualScrollElement", oGridDiv.outerHTML + vertiScrollTd[0].outerHTML);
            var trow2 = ej.buildTag("tr.virtualScrollElement", horiScrollTd[0].outerHTML + ej.buildTag("td.virtualScrollElement")[0].outerHTML);
            var virtualOlapGrid = ej.buildTag("tbody.oGridOuterDiv.virtualScrollElement", trow[0].outerHTML + trow2[0].outerHTML);
            this.element.html(ej.buildTag("table.virtualScrollElement", virtualOlapGrid[0].outerHTML));
            var seriesPageIndicator = ej.buildTag("div.seriesPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("SeriesPage"))[0].outerHTML + ej.buildTag("span.series_CurrentPage", _proxy._seriesCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.series_pageCount", _proxy._seriesPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._seriesPageCount > 1) _proxy.element.append(seriesPageIndicator);
            var categPageIndicator = ej.buildTag("div.categPageIndicator inActive", ej.buildTag("span.axislabel", this._getLocalizedLabels("CategoricalPage"))[0].outerHTML + ej.buildTag("span.categ_CurrentPage", _proxy._categCurrentPage)[0].outerHTML + " / " + ej.buildTag("span.categ_pageCount", _proxy._categPageCount)[0].outerHTML)[0].outerHTML;
            if (_proxy._categPageCount > 1) _proxy.element.append(categPageIndicator);
        }
    });

    ej.PivotGrid.Layout = {
        Normal: "normal",
        NormalTopSummary: "normaltopsummary",
        NoSummaries: "nosummaries",
        ExcelLikeLayout: "excellikelayout"
    };

    ej.PivotGrid.locale = {};

    ej.PivotGrid.locale["en-US"] = {
        ToolTipRow: "Row",
        ToolTipColumn: "Column",
        ToolTipValue: "Value",
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page",
        DragFieldHere: "Drag field here",
        ColumnArea: "Drop column here",
        RowArea: "Drop row here",
        ValueArea: "Drop values here",
        OK: "OK",
        Cancel: "Cancel",
        Remove: "Remove",
        ConditionalFormatting: "Conditional Formatting",
        Condition: "Conditional Type",
        Value1: "Value1",
        Value2: "Value2",
        Editcondtion: "Edit Condition",
        Backcolor: "Back Color",
        Borderrange: "Border Range",
        Borderstyle: "Border Style",
        Fontsize: "Font Size",
        Fontstyle: "Font Style",
        Bordercolor: "Border Color"
    };

    ej.PivotGrid.GridAction = {
        Initialize: "initialize",
        DrillDown: "drillDown"
    };

    ej.PivotGrid.ExportOptions = {
        Excel: 'excel',
        Word: 'word',
        PDF: 'pdf',
        CSV: 'csv'
    };

    ej.PivotGrid.AnalysisMode = {
        Olap: "olap",
        Relational: "relational"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Pivot Schema Designer control to list out and analyze the data
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejPivotSchemaDesigner", "ej.PivotSchemaDesigner", {

        _rootCSS: "e-pivotschemadesigner",
        element: null,
        model: null,
        validTags: ["div", "span"],
        defaults: {
            url: "",
            cssClass: "",
            height: "630px",
            width: "415px",
            locale: "en-US",
            layout: "normal",
            pivotControl: null,
            pivotTableFields: [],
            pivotCalculations: [],
            pivotColumns: [],
            pivotRows: [],
            filters: [],
            enableWrapper: false,
            serviceMethods: {
                fetchMembers: "FetchMembers", nodeStateModified: "NodeStateModified",
                nodeDropped: "NodeDropped", removeButton: "RemoveButton", memberExpand: "MemberExpanded", filtering: "filtering"
            },
            customObject: {},
            beforeServiceInvoke: null,
            afterServiceInvoke: null
        },
        dataTypes: {
            serviceMethods: "data",
            customObject: "data",
            pivotControl: "data",
            pivotTableFieldList: "array",
            pivotCalculationList: "array",
            pivotColumnList: "array",
            pivotRowList: "array",
            filterList: "array"
        },

        locale: ej.util.valueFunction("locale"),

        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        _destroy: function () {
            this.element.empty().removeClass("e-pivotschemadesigner" + this.model.cssClass);
        },

        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._dialogTitle = "";
            this._currentMembers = new Array();
            this._memberTreeObj = null;
            this._tableTreeObj = null;
            this._dialogOKBtnObj = null;
            this._curFilteredText = "";
            this._curFilteredAxis = "";
            this._tempFilterData = null;
            this._droppedClass = "";
            this._selectedTreeNode = null;
            this._selectedMember = "";
            this._isDragging = false;
            this._dataModel = "";
            this._droppedPosition = "";
            this._currentCubeName = "";
            this._errorDialog = "";
            _pivotProxy = this;
            this._nodeDropedParams = "";
            this._contextMenuObj = null;
            this._removeButtonDeferUpdate = false;
        },

        _load: function () {
            var itemProps = "", report;
            if (this.model.pivotControl != null) {
                this.model.pivotControl._schemaData = this;
            }
            if (this.model.pivotControl != null && !jQuery.isEmptyObject(this.model.pivotControl.getOlapReport())) {
                itemProps = JSON.parse(JSON.parse(this.model.pivotControl.getOlapReport()).ItemsProperties);
                report = JSON.parse(this.model.pivotControl.getOlapReport());
                this._setTableFields(itemProps);
                this._setPivotRows(report.PivotRows);
                this._setPivotColumns(report.PivotColumns);
                this._setPivotCalculations(report.PivotCalculations);
                this._setFilters(report.Filters);
                this._dataModel = report.DataModel;
                this._currentCubeName = report.CurrentCube;
                $("#" + this._id).ejWaitingPopup({ showOnInit: true });
                pschemaDesignerWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                pschemaDesignerWaitingPopup.hide();
            }
            if (this.model.enableWrapper == false) {
                var fieldTable = ej.buildTag("table.headerTable", ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.listHeader", ej.buildTag("span.headerText", _pivotProxy._getLocalizedLabels("PivotTableFieldList"), {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.listSubhead", ej.buildTag("span.subheadText", _pivotProxy._getLocalizedLabels("ChooseFieldsToAddToReport"), {})[0].outerHTML, {})[0].outerHTML, {}).attr("valign", "top")[0].outerHTML, {})[0].outerHTML, {}).attr("width", "100%").attr("height", "10%")[0].outerHTML + ej.buildTag("table.fieldTable", ej.buildTag("tbody", ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.schemaFieldList", ej.buildTag("div.schemaFieldTree#schemaFieldTree")[0].outerHTML)[0].outerHTML).attr("valign", "top")[0].outerHTML, {})[0].outerHTML, {})[0].outerHTML, {}).attr("width", "95%").attr("height", "30%")[0].outerHTML,
                axisTable = ej.buildTag("table.axisTable", ej.buildTag("tbody", ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.pivotHeader", ej.buildTag("span.headerText", _pivotProxy._getLocalizedLabels("ReportFilter"), {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaFilter", this._createPivotButtons(this.model.filters, "filter"))[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML + ej.buildTag("td", ej.buildTag("div.rPivotHeader", ej.buildTag("span.headerText", _pivotProxy._getLocalizedLabels("ColumnLabel"), {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaColumn", this._createPivotButtons(this.model.pivotColumns, "column"), {})[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML, {})[0].outerHTML + ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.pivotHeader", ej.buildTag("span.headerText", _pivotProxy._getLocalizedLabels("RowLabel"), {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaRow", this._createPivotButtons(this.model.pivotRows, "row"), {})[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML + ej.buildTag("td", ej.buildTag("div.rPivotHeader", ej.buildTag("span.headerText", _pivotProxy._getLocalizedLabels("Values"), {})[0].outerHTML, {})[0].outerHTML + ej.buildTag("div.schemaValue", this._createPivotButtons(this.model.pivotCalculations, "values"), {})[0].outerHTML, {}).attr("width", "50%").attr("height", "50%").attr("valign", "top")[0].outerHTML, {})[0].outerHTML, {})[0].outerHTML, {}).attr("width", "100%").attr("height", "46%")[0].outerHTML,
                deferUpdate = ej.buildTag("div.deferUpdateLayout", ej.buildTag("input.chkDeferUpdate", "", {}).attr("type", "checkbox")[0].outerHTML + ej.buildTag("button.btnDeferUpdate", _pivotProxy._getLocalizedLabels("Update"), {}).attr("type", "button")[0].outerHTML)[0].outerHTML,
                htmlTag = fieldTable + ej.buildTag("div.centerDiv")[0].outerHTML + ej.buildTag("div.centerHead", _pivotProxy._getLocalizedLabels("DragFieldBetweenAreasBelow"))[0].outerHTML + axisTable + ej.buildTag("span.responsiveSchema", "")[0].outerHTML + deferUpdate;
                $(htmlTag).appendTo("#" + this._id);
            }
            else if (this.model.enableWrapper == true) {
                this._setFilters(report.Filters);
                this.element.find(".axisTable .schemaFilter").append(this._createPivotButtons(this.model.filters, "filter"));
                this.element.find(".axisTable .schemaRow").append(this._createPivotButtons(this.model.pivotRows, "row"));
                this.element.find(".axisTable .schemaColumn").append(this._createPivotButtons(this.model.pivotColumns, "column"));
                this.element.find(".axisTable .schemaValue").append(this._createPivotButtons(this.model.pivotCalculations, "values"));
                this.element.find(".pivotButton .pvtBtn").ejButton({ size: "normal" }).ejDraggable({
                    handle: 'button', clone: true,
                    cursorAt: { left: -5, top: -5 },
                    dragStart: ej.proxy(function (args) {
                        this._isDragging = true;
                    }, this),
                    dragStop: ej.proxy(this._onPvtBtnDropped, this),
                    helper: ej.proxy(function (event, ui) {
                        if (event.sender.target.className.indexOf("e-btn") > -1)
                            return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                        else
                            return false;
                    }, this)
                });
            }
            this.element.width(this.model.width).height(this.model.height);
            this.element.find(".pivotButton .pvtBtn").ejButton({ size: "normal" }).ejDraggable({
                handle: 'button', clone: true,
                cursorAt: { left: -5, top: -5 },
                dragStart: ej.proxy(function (args) {
                    this._isDragging = true;
                }, this),
                dragStop: ej.proxy(this._onPvtBtnDropped, this),
                helper: ej.proxy(function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                    else
                        return false;
                }, this)
            });
            this.element.find(".pivotButton .filterBtn").ejButton({
                size: "normal",
                contentType: "imageonly",
                prefixIcon: "filter"
            });
            this.element.find(".pivotButton input").ejToggleButton({
                size: "normal",
                contentType: "imageonly",
                defaultPrefixIcon: "ascending",
                activePrefixIcon: "descending",
                click: ej.proxy(this._onSortBtnClick, this)
            });
            this.element.find(".schemaFieldTree").ejTreeView({
                showCheckbox: true,
                fields: { id: "id", parentId: "pid", text: "name", isChecked: "isSelected", spriteCssClass: "spriteCssClass", dataSource: this.model.pivotTableFields },
                allowDragAndDrop: true,
                allowDropChild: false,
                allowDropSibling: false,
                dragAndDropAcrossControl: true,
                nodeDropped: ej.proxy(this._onNodeDropped, this),
                nodeDragStart: ej.proxy(this._onNodeDrag, this)
            });
            this._tableTreeObj = this.element.find(".schemaFieldTree").data("ejTreeView");
            this._tableTreeObj.element.find(".measureGroupCDB").parent().siblings(".e-chkbox-wrap").remove();
            this._tableTreeObj.element.find(".folderCDB").parent().siblings(".e-chkbox-wrap").remove();
            this._tableTreeObj.element.find(".dimensionCDB").parent().siblings(".e-chkbox-wrap").remove();
            this._tableTreeObj.element.find(".measureGroupCDB").parents("li").append(ej.buildTag("span.elementSeparator")[0].outerHTML);
            this._tableTreeObj.element.find(".dimensionCDB").parents("li").append(ej.buildTag("span.elementSeparator")[0].outerHTML);
            var treeViewElements = this._tableTreeObj.element.find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", this.model.pivotTableFields[i].tag);
                var checkedBoxNode = $(treeViewElements[i]).find(".e-chkbox-wrap");
                if (checkedBoxNode[0].attributes[4].value == "true" && $($(treeViewElements[i])).find(".folderCDB").length <= 0 && $(treeViewElements[i]).attr("tag").indexOf("[Measures]") == -1) {
                    var dropSpan = ej.buildTag("span.e-icon").css("display", "none").addClass("treeDrop")[0].outerHTML;
                    $($(treeViewElements[i]).find(".e-text")[0]).after(dropSpan);
                }
            }
            this._tableTreeObj.model.nodeCheck = ej.proxy(this._onCheckedStateModified, this);
            this._tableTreeObj.model.nodeUncheck = ej.proxy(this._onCheckedStateModified, this);
            this._tableTreeObj.element.find(".attributeCDB").parent().siblings("ul").remove();
            this._tableTreeObj.element.find(".attributeCDB").siblings(".e-plus").remove();
            this._tableTreeObj.element.find(".hierarchyCDB").parent().parent().siblings("ul").find(".e-chkbox-wrap").remove();
            if (this._tableTreeObj.element.find(".e-plus").length == 0) {
                this._tableTreeObj.element.find(".e-item").css("padding", "0px");
            }
            this.element.find(".schemaFilter, .schemaColumn, .schemaRow, .schemaValue").ejDroppable({
            });
            var contextTag = ej.buildTag("ul.pivotTreeContext#pivotTreeContext", ej.buildTag("li", ej.buildTag("a", "Add to Filter")[0].outerHTML)[0].outerHTML + ej.buildTag("li", ej.buildTag("a", "Add to Row")[0].outerHTML)[0].outerHTML + ej.buildTag("li", ej.buildTag("a", "Add to Column")[0].outerHTML)[0].outerHTML + ej.buildTag("li", ej.buildTag("a", "Add to Values")[0].outerHTML)[0].outerHTML)[0].outerHTML;
            $(this.element).append(contextTag);
            $("#pivotTreeContext").ejMenu({
                menuType: ej.MenuType.ContextMenu,
                openOnClick: false,
                contextMenuTarget: this._tableTreeObj.element[0],
                click: ej.proxy(this._onTreeContextClick, this),
                beforeOpen: ej.proxy(this._onContextOpen, this)
            });
            if (this.model.pivotControl != null && this.model.pivotControl.model.isResponsive == true) {
                if ($(window).width() < 750) {
                    $("#" + this._id).css("position", "absolute");
                    this.element.find(".responsiveSchema").addClass("expandSchema").removeClass("responsiveSchema");
                    $("#" + this.model.pivotControl._id).width("100%");
                    this._hideSchemaDesigner();
                }
                else if ($(window).width() > 750) {
                    $("#" + this._id).css({ "position": "static" });
                    $("#" + this.model.pivotControl._id).width(parseInt(100 - $("#" + this._id).width() / $(window).width() * 100) - 2 + "%");
                    this._showSchemaDesigner();
                }
            }
            if (this.model.pivotControl != null) {
				var deferCheck=false; 
				if(this.model.pivotControl.model.enableDeferUpdate)
					deferCheck=true;
                this.element.find(".btnDeferUpdate").ejButton({
                    size: "mini",
                    enabled: deferCheck,
                    click: ej.proxy(this.model.pivotControl._deferUpdate, this.model.pivotControl)
                });
				this.element.find(".chkDeferUpdate").ejCheckBox({
				    text: _pivotProxy._getLocalizedLabels("DeferLayoutUpdate"),
					change: this._checkedChange,
					size: "normal",
					checked: deferCheck
				});
			}
            this._unWireEvents();
            this._wireEvents();
        },

        _setFirst: true,

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "locale": this._load(); break;
                }
            }
        },

        _wireEvents: function () {
            if (this.model.layout != "excel") {
                this._on(this.element, "mouseover", ".pivotButton .pvtBtn", ej.proxy(function (evt) {
                    $(evt.target.parentNode).find("span.removePivotBtn").css("display", "inline-block");
                }, this));
                this._on(this.element, "mouseleave", ".pivotButton", ej.proxy(function (evt) {
                    this.element.find(".pivotButton > span.removePivotBtn").css("display", "none");
                }, this));
            }
            else {
                this._on(this.element, "mouseover", ".pivotButton .pvtBtn", ej.proxy(function (evt) {
                    if (this._isDragging) {
                        this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
                        $(evt.target).siblings(".dropIndicator").addClass("dropIndicatorHover");
                    }
                }, this));
                this._on(this.element, "mouseleave", ".pivotButton", ej.proxy(function (evt) {
                    if (this._isDragging)
                        $(evt.target).siblings(".dropIndicator").removeClass("dropIndicatorHover");
                }, this));
            }
            this._tableTreeObj.element.find("li").mouseover(ej.proxy(function (evt) {
                var margin = ($(evt.target).siblings("span.e-icon.filter:eq(0)").length > 0 || $(evt.target).find("span.e-icon.filter:eq(0)").length > 0 || $(evt.target).parentsUntil("li").find("span.e-icon.filter:eq(0)").length > 0) ? (navigator.userAgent.indexOf("MSIE") > 0) ? "-20px" : "-35px" : "-20px";
                $(evt.target).siblings("span.e-icon.treeDrop:eq(0)").length > 0 ? $(evt.target).siblings("span.e-icon.treeDrop:eq(0)").css({ display: "inline-block", "position": "absolute", color: "#5c5c5c", "margin-left": margin, "margin-top": "3px" }) : $(evt.target).find("span.e-icon.treeDrop:eq(0)").length > 0 ? $(evt.target).find("span.e-icon.treeDrop:eq(0)").css({ "display": "inline-block", "position": "absolute", color: "#5c5c5c" }) : $(evt.target).parentsUntil("li").find("span.e-icon.treeDrop:eq(0)").css({ color: "#5c5c5c", "display": "inline-block", "position": "absolute" });
                if ($(evt.target).parent().find(".measureGroupCDB, .dimensionCDB, .folderCDB").length > 0)
                    $(evt.target).css("cursor", "default");
            }, this))
          .mouseout(ej.proxy(function (evt) {
              $(this._tableTreeObj.element).find("span.e-icon.treeDrop").css({ display: "none" });
          }, this));
            this._on(this.element, "click", ".filterBtn", ej.proxy(this._onFilterBtnClick, this));
            this._on(this.element, "click", ".removePivotBtn", ej.proxy(this._onRemoveBtnClick, this));
            this._on(this._tableTreeObj.element, "click", ".treeDrop", ej.proxy(this._onFilterBtnClick, this));
            //this.element.find(".collapseSchema").ejButton({ type: ej.ButtonType.Button, click: ej.proxy(this._hideSchemaDesigner, this) });
            this._on(this.element, "click", ".collapseSchema", ej.proxy(this._hideSchemaDesigner, this));
            this._on(this.element, "click", ".expandSchema", ej.proxy(this._showSchemaDesigner, this));
            $(window).bind('resize', $.proxy(this._reSizeHandler, this));
        },

        _unWireEvents: function () {
            this._off(this.element, "click", ".filterBtn, .collapseSchema,.expandSchema");
            this._off(this.element, "mouseover", ".pivotButton");
            this._off(this.element, "mouseover", ".pivotButton .pvtBtn");
            this._off(this.element, "mouseleave", ".pivotButton");
            this._off(this.element, "mouseover", ".schemaFieldTree li");
            this._off(this.element, "mouseout", ".schemaFieldTree li");
            this._off(this._tableTreeObj.element, "click", ".treeDrop");
            this._off(this._tableTreeObj.element, "click", "li");
            $(window).unbind('resize', $.proxy(this._reSizeHandler, this));
        },

        _getLocalizedLabels: function (property) {
            return ej.PivotSchemaDesigner.locale[this.locale()][property] === undefined ? ej.PivotSchemaDesigner.locale["en-US"][property] : ej.PivotSchemaDesigner.locale[this.locale()][property];
        },

        _setTableFields: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotTableFields = items;
        },

        _setPivotRows: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotRows = items;
        },

        _setPivotColumns: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotColumns = items;
        },

        _setPivotCalculations: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.pivotCalculations = items;
        },

        _setFilters: function (items) {
            if (!ej.isNullOrUndefined(items))
                this.model.filters = items;
        },

        _getSortedHeaders: function () {
            var labels = this.element.find(".descending").parents("label"), headers = "";
            for (var i = 0; i < labels.length; i++)
                headers += $(labels[i]).attr("for").replace("toggleBtn", "") + "##";
            return headers;
        },

        _onContextOpen: function (args) {
            if ($(args.target.parentElement).find(".measureGroupCDB").length > 0 || $(args.target.parentElement).find(".folderCDB").length > 0 || $(args.target.parentElement).find(".dimensionCDB").length > 0)
                return false;
            else if (this._dataModel == "Olap")
                this._selectedMember = $(args.target).parents("li:eq(0)").attr("tag");
            else if (this._dataModel == "Pivot")
                this._selectedMember = args.target.text;
        },

        _onNodeDrag: function (args) {
            this._isDragging = true;
            if ($(args.dragTarget.parentElement).find(".measureGroupCDB").length > 0 || $(args.dragTarget.parentElement).find(".folderCDB").length > 0 || $(args.dragTarget.parentElement).find(".dimensionCDB").length > 0)
                return false;
        },

        _onTreeContextClick: function (args) {
            var headerTag, filterTag, filterItems = "",
            selectedPvtBtn = this.element.find(".pivotButton:contains(" + this._selectedMember + ")"),
            isFiltered = $(selectedPvtBtn).find(".filterIndicator").length > 0 ? true : false,
            isSorted = $(selectedPvtBtn).find("~ span:eq(0) span.descending").length > 0 ? true : false, report, axisName,
            params, eventArgs;
            if (this._dataModel == "Olap")
                selectedTreeNode = _pivotProxy._getNodeByUniqueName(this._selectedMember);
            else
                selectedTreeNode = $(this._tableTreeObj.element).find("div:contains(" + this._selectedMember + ")>span").parents("li")
            this._droppedClass = args.events.text == "Add to Filter" ? "schemaFilter" : args.events.text == "Add to Row" ? "schemaRow" :
            args.events.text == "Add to Column" ? "schemaColumn" : args.events.text == "Add to Values" ? "schemaValue" : "";
            for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                if (this.model.pivotTableFields[i].name == this._selectedMember) {
                    this.model.pivotTableFields[i].isSelected = true;
                    headerTag = this.model.pivotTableFields[i];
                }
            }
            if (!ej.isNullOrUndefined(this._tempFilterData)) {
                for (var i = 0; i < this._tempFilterData.length; i++) {
                    if (!ej.isNullOrUndefined(this._tempFilterData[i][this._selectedMember])) {
                        for (var j = 0; j < this._tempFilterData[i][this._selectedMember].length; j++)
                            filterItems += "##" + this._tempFilterData[i][this._selectedMember][j];
                    }
                }
            }
            if (selectedPvtBtn.length > 1)
                for (var i = 0; i < selectedElement.length; i++) {
                    if ($(selectedPvtBtn[i]).text() == this._selectedMember)
                        $(selectedPvtBtn[i]).remove();
                }
            else
                $(selectedPvtBtn).remove();
            try {
                report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
            }
            catch (err) {
                report = this.model.pivotControl.getOlapReport();
            }
            if (this._dataModel == "Olap") {
                _pivotProxy._nodeDropedParams = this._droppedClass == "schemaFilter" ? "Slicer" : this._droppedClass == "schemaColumn" ? "Categorical" : this._droppedClass == "schemaRow" ? "Series" : "";
                if (this._droppedClass == "schemaValue") {
                    var tempAxis = this.element.find(".pivotButton:contains('Measures')").parents(".e-droppable").attr("class").split(" ")[0];
                    _pivotProxy._nodeDropedParams = tempAxis == "schemaColumn" ? "Categorical" : tempAxis == "schemaRow" ? "Series" : "";
                }
             
                var _isPvtButton = false, dragNode = "";
                for (var i = 0; i < this.element.find(".pivotButton").length; i++) {
                    if ($(this.element.find(".pivotButton")[i]).attr("tag").split(":")[1] == selectedTreeNode.attr("tag").replace(/\[/g, "").replace(/\]/g, "")) {
                        _isPvtButton = true;
                        dragNode = $($(".pivotButton")[i]).attr("tag");
                        break;
                    }
                }
                if (!_isPvtButton)
                { this._tableTreeObj.checkNode(selectedTreeNode); }
                else {
                    params = this._currentCubeName + "--" + dragNode + "--" + _pivotProxy._nodeDropedParams + "--";
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "SplitButton", "nodeInfo": params, "currentReport": report, "layout": this.model.pivotControl.model.layout,"customObject":serializedCustomObject });
                    if (!this.model.pivotControl.model.enableDeferUpdate && !ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.show();
                        if (!this.model.pivotControl.model.enableDeferUpdate)
                            this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                        else 
                            this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._onDroppedSuccess);
                }
                _pivotProxy._nodeDropedParams = "";
            }
            else {
                if (selectedTreeNode.find(".e-chk-inact").length > 0)
                    this._tableTreeObj.checkNode(selectedTreeNode);
                else {
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    this._createPivotButton(this._selectedMember, this._droppedClass == "schemaRow" ? "row" : this._droppedClass == "schemaColumn" ? "column" : this._droppedClass == "schemaFilter" ? "filter" : this._droppedClass == "schemaValue" ? "value" : "", isFiltered, isSorted, "");
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropAxis": this._droppedClass + "::", "headerTag": JSON.stringify(headerTag), "sortedHeaders": this.model.pivotControl._ascdes, "currentReport": report, "customObject": serializedCustomObject });
                    if (!this.model.pivotControl.model.enableDeferUpdate && !ej.isNullOrUndefined(ogridWaitingPopup))
                            ogridWaitingPopup.show();
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._pvtNodeDroppedSuccess);
                    else {
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._pvtNodeDroppedSuccess);
                    }
                }
            }
        },

        _checkedChange: function (args) {
            var deferButton = $(".btnDeferUpdate").data("ejButton");
            if (args.isChecked) {
                _pivotProxy.model.pivotControl.model.enableDeferUpdate = true;
                deferButton.enable();
            }
            else {
                _pivotProxy.model.pivotControl.model.enableDeferUpdate = false;
                deferButton.disable();
            }
        },

        _onSortBtnClick: function (args) {
            var serializedCustomObject = JSON.stringify(this.model.customObject),
            headers = this._getSortedHeaders();
            var eventArgs = this.model.customObject != {} && headers != "" ? JSON.stringify({ "action": "sorting", "sortedHeaders": headers, "currentReport": this.model.pivotControl.getOlapReport(), "customObject": serializedCustomObject }) :
            this.model.customObject != {} ? JSON.stringify({ "action": "sorting", "currentReport": this.model.pivotControl.getOlapReport(), "customObject": serializedCustomObject }) :
            headers != "" ? JSON.stringify({ "action": "sorting", "sortedHeaders": headers, "currentReport": this.model.pivotControl.getOlapReport() }) : JSON.stringify({ "action": "sorting" });
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
            this.model.pivotControl.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.pivotControl.model.serviceMethods.sorting, eventArgs, this.model.pivotControl._renderControlSuccess);
        },

        _onFilterBtnClick: function (args) {
            $(this._tableTreeObj.element).find("span.e-icon.treeDrop").css({ display: "none" });
            this._selectedTreeNode = $($(args.target).parent()).parent();
            if (args.target.tagName.toLowerCase() == "span") {
                this.model.layout != "excel" ? this._curFilteredText = $($(args.target).parents(".pivotButton")).find(".pvtBtn").text() : this._curFilteredText = $(args.target).parent().text();
                if ($(args.target).parents("li:eq(1)").children().children("span").hasClass("hierarchyCDB")) {
                    this._curFilteredText = $($(args.target).parents("li:eq(1)")).children().find(".e-text")[0].innerHTML;
                }
                if (this._dataModel == "Pivot")
                    this._curFilteredAxis = !ej.isNullOrUndefined(this.element.find(".e-btn:contains('" + this._curFilteredText + "')").parents(".e-droppable")[0]) ? this.element.find(".e-btn:contains('" + this._curFilteredText + "')").parents(".e-droppable")[0].className.split(" ")[0] : "";
                this._dialogTitle = this._dataModel == "Olap" ? $(this.element.find(".pivotButton:contains('" + $(args.target).parent().text() + "')")[0]).attr("tag") : $(args.target).parent().text() || args.target.id.replace("filterBtn", "") || $(args.target).parents(".filterBtn")[0].id.replace("filterBtn", "");
                if (this._dataModel == "Olap")
                    for (var btnCount = 0 ; btnCount < this.element.find(".e-btn").length; btnCount++) {
                        if (!ej.isNullOrUndefined($($($(this.element.find(".e-btn"))[btnCount]).parent()[0]).attr("tag")) && $($($(this.element.find(".e-btn"))[btnCount]).parent()[0]).attr("tag").indexOf(this._curFilteredText) > -1) {
                            this._curFilteredAxis = !ej.isNullOrUndefined($($(this.element.find(".e-btn"))[0]).parents(".e-droppable")[0]) ? $($(this.element.find(".e-btn"))[0]).parents(".e-droppable")[0].className.split(" ")[0] : "";
                            this._dialogTitle = $($($(this.element.find(".e-btn"))[btnCount]).parent()[0]).attr("tag");
                        }
                    }
                var report;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                sortedHeaders = this._getSortedHeaders();
                if (this.model.beforeServiceInvoke != null)
                       this._trigger("beforeServiceInvoke", { action: "fetchMembers", element: this.element, customObject: this.model.customObject });
               var serializedCustomObject = JSON.stringify(this.model.customObject);
               eventArgs = JSON.stringify({ "action": "fetchMembers", "headerTag": this._dialogTitle || "UniqueName##" + $($(args.target).parents("li")[0]).attr("tag"), "sortedHeaders": this.model.pivotControl._ascdes, "currentReport": report, "customObject": serializedCustomObject });
                if (!ej.isNullOrUndefined(pschemaDesignerWaitingPopup))
                    pschemaDesignerWaitingPopup.show();
                this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.fetchMembers, eventArgs, this._fetchMemberSuccess);
            }
        },

        _onRemoveBtnClick: function (args) {
            var headerText = $(args.target.parentElement).text(),
            selectedTreeNode = $($(this._tableTreeObj.element).find("div:contains(" + headerText + ")>span")[1]).find(".e-chk-act"),
            headerTag;
            $(selectedTreeNode).removeClass("e-chk-act").addClass("e-chk-inact");
            $(selectedTreeNode).children("e-chk-activeicon").removeClass("e-chk-activeicon");
            for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                if (this.model.pivotTableFields[i].FieldHeader == headerText) {
                    this.model.pivotTableFields[i].IsSelected = false;
                    headerTag = this.model.pivotTableFields[i];
                }
            }
            $(args.target.parentElement).remove();
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "nodeStateModified", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            var eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "sortedHeaders": this._getSortedHeaders(), "currentReport": this.model.pivotControl.getOlapReport(), "customObject": serializedCustomObject });
            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                ogridWaitingPopup.show();
            this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs, this._nodeStateModifiedSuccess);
        },

        _onNodeCheckChanges: function (args) {
            if (args.value == "(All)" && args.type == "nodeUncheck") {
                this._memberTreeObj.model.nodeCheck = "";
                this._memberTreeObj.model.nodeUncheck = "";
                this.element.find(".editorTreeView").ejTreeView("unCheckAll");
                this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
                this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
                this._dialogOKBtnObj.disable();
            }
            else if (args.value == "(All)" && args.type == "nodeCheck") {
                this._memberTreeObj.model.nodeCheck = "";
                this._memberTreeObj.model.nodeUncheck = "";
                this.element.find(".editorTreeView").ejTreeView("checkAll");
                $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)").removeClass("e-chk-indetericon");
                this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
                this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
                this._dialogOKBtnObj.enable();
            }
            else if (args.value != "(All)") {
                var uncheckedNodes = this._memberTreeObj.element.find(":input:gt(0).nodecheckbox:not(:checked)"),
                firstNode = $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)");
                if (uncheckedNodes.length == 0 || (uncheckedNodes.length == 1 && uncheckedNodes[0].id[uncheckedNodes[0].id.length - 1] == 0)) {
                    $(firstNode).parent().removeClass("e-chk-inact").removeClass("e-chk-ind").addClass("e-chk-act");
                    firstNode.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                }
                else if (args.type == "nodeCheck" && uncheckedNodes.length == 1 && uncheckedNodes[0].id[uncheckedNodes[0].id.length - 1] == 0)
                    firstNode.removeClass("e-chk-indetericon").addClass("e-chk-activeicon");
                else if (uncheckedNodes.length > 0)
                    firstNode.removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
                this._dialogOKBtnObj.enable();
                if (args.type == "nodeUncheck" && uncheckedNodes.length + 1 == this._memberTreeObj.element.find("li").length) {
                    firstNode.removeClass("e-chk-activeicon").removeClass("e-chk-indetericon");
                    this._dialogOKBtnObj.disable();
                }
            }
        },

        _onCheckedStateModified: function (args) {
            var headerText = $(args.currentElement).find("a")[0].textContent, filterTag = "", filterItems = "", selectedElement = "", headerTag = "", uniqueName = "";
            if (this._dataModel == "Olap") {
                if (!args.currentElement[0].id.indexOf("[Measures]") > -1)
                    for (var i = 0; i < _pivotProxy.element.find(".pivotButton").length; i++) {
                        headerTag = $(_pivotProxy.element.find(".pivotButton")[i]).attr("tag"); uniqueName = "";
                        uniqueName = ((headerTag.indexOf("[Measures]") > -1) || (headerTag.indexOf("[") > -1)) ? headerTag.split(":")[1] : this._getNodeUniqueName(headerTag);
                        if ($(args.currentElement).attr("tag") == uniqueName) {
                            selectedElement = $(_pivotProxy.element.find(".pivotButton")[i]);
                            break;
                        }
                    }
                else
                    selectedElement = _pivotProxy.element.find(".pivotButton:contains(" + headerText + ")"), headerTag, report;
            }
            else
                selectedElement = _pivotProxy.element.find(".pivotButton:contains(" + headerText + ")"), headerTag, report;
            var isFiltered = selectedElement.length > 0 && $(selectedElement).siblings(".filterIndicator").length > 0 ? true : false,
            isSorted = args.currentElement.find("~ span:eq(0) span.descending").length > 0 ? true : false;
            for (var i = 0; i < _pivotProxy.model.pivotTableFields.length; i++) {
                if (_pivotProxy.model.pivotTableFields[i].name == headerText && args.type == "nodeCheck") {
                    _pivotProxy.model.pivotTableFields[i].isSelected = true;
                    headerTag = _pivotProxy.model.pivotTableFields[i];
                }
                else if (this.model.pivotTableFields[i].name == headerText && args.type == "nodeUncheck") {
                    _pivotProxy.model.pivotTableFields[i].isSelected = false;
                    headerTag = _pivotProxy.model.pivotTableFields[i];
                }
            }
            if (ej.isNullOrUndefined(headerTag))
                headerTag = $(args.currentElement).attr("tag");

            if (args.type == "nodeUncheck") {
                if (selectedElement.length > 1)
                    for (var i = 0; i < selectedElement.length; i++) {
                        if ($(selectedElement[i]).text() == headerText)
                            $(selectedElement[i]).remove();
                    }
                else
                    $(selectedElement).remove();
            }
            else {
                if (this._droppedClass != "")
                    this._createPivotButton(headerText, this._droppedClass == "schemaColumn" ? "column" : this._droppedClass == "schemaRow" ? "row" : this._droppedClass == "schemaValue" ? "value" : this._droppedClass == "schemaFilter" ? "filter" : "", isFiltered, isSorted, "");
                else
                    this._createPivotButton(headerText, ((this._dataModel == "Olap" && $(args.currentElement).attr("tag").indexOf("[Measures]") < 0) || (this._dataModel == "Pivot" && headerTag.pivotType == "PivotItem")) ? "row" : "value", isFiltered, isSorted, "");
            }
            if (!ej.isNullOrUndefined(this._tempFilterData)) {
                for (var i = 0; i < this._tempFilterData.length; i++) {
                    if (!ej.isNullOrUndefined(this._tempFilterData[i][headerText])) {
                        for (var j = 0; j < this._tempFilterData[i][headerText].length; j++)
                            filterItems += "##" + this._tempFilterData[i][headerText][j];
                    }
                }
            }
            try {
                report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
            }
            catch (err) {
                report = this.model.pivotControl.getOlapReport();
            }
            filterTag = "schemaRow::" + headerText + "::FILTERED" + filterItems;
            var dropAxis = this._droppedClass != "" ? this._droppedClass : headerTag.pivotType == "PivotItem" ? "schemaRow" : "schemaValue", eventArgs, axisName, params;
            this._droppedClass = "";
            if (!this.model.pivotControl.model.enableDeferUpdate && !ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.show();
            if (!(args.type == "nodeCheck") && !(args.type == "nodeUncheck") && dropAxis != "") {
                if (_pivotProxy._dataModel == "Pivote") {
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: "nodeStateModified", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "dropAxis": dropAxis + "::" + this._droppedPosition, "sortedHeaders": this.model.pivotControl._ascdes, "filterParams": filterTag, "currentReport": report, "customObject": serializedCustomObject });
                    _pivotProxy._droppedPosition = "";
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        _pivotProxy.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs, this._nodeStateModifiedSuccess);
                    else {
                        _pivotProxy.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + _pivotProxy.model.serviceMethods.nodeStateModified, eventArgs.replace("nodeStateModified", "nodeStateModifiedDeferUpdate"), this._nodeStateModifiedSuccess);
                    }
                }
                else {
                    if ($(args.currentElement).attr("tag").indexOf("[Measures]") == -1) {
                        var dropSpan = ej.buildTag("span.e-icon").addClass("treeDrop")[0].outerHTML;
                        $(args.currentElement.find(".e-text")[0]).after($(dropSpan).hide());
                    }
                    headerTag = $(args.currentElement).attr("tag");
                    axisName = this._nodeDropedParams == "" ? ($(args.currentElement).attr("tag").indexOf("[Measures]") >= 0 ? "Categorical" : "Series") : this._nodeDropedParams;
                    params = this._currentCubeName + "--" + headerTag + "--" + axisName + "--", report;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "currentReport": report, "customObject": serializedCustomObject });
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                    else {
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._onDroppedSuccess);
                    }
                }
            }
            else if (args.type == "nodeCheck") {
                if (this._dataModel == "Olap") {
                    if ($(args.currentElement).attr("tag").indexOf("[Measures]") == -1) {
                        var dropSpan = ej.buildTag("span.e-icon").addClass("treeDrop")[0].outerHTML;
                        $(args.currentElement.find(".e-text")[0]).after($(dropSpan).hide());
                    }
                    _pivotProxy._nodeDropedParams = dropAxis == "schemaColumn" ? "Categorical" : dropAxis == "schemaRow" ? "Series" : dropAxis == "schemaFilter" ? "Slicer" : "";
                    headerTag = $(args.currentElement).attr("tag");
                    axisName = this._nodeDropedParams == "" ? ($(args.currentElement).attr("tag").indexOf("[Measures]") >= 0 ? "Categorical" : "Series") : this._nodeDropedParams;
                    params = this._currentCubeName + "--" + headerTag + "--" + axisName + "--", report;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "currentReport": report, "customObject": serializedCustomObject });
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                    else {
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._onDroppedSuccess);
                    }
                }
                else {
                    $(args.currentElement.find(".e-text")[0]).after($(ej.buildTag("span.e-icon").addClass("treeDrop")[0].outerHTML).hide());
                    if (this.model.beforeServiceInvoke != null)
                            this._trigger("beforeServiceInvoke", { action: "nodeStateModified", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "dropAxis": dropAxis + "::", "sortedHeaders": this.model.pivotControl._ascdes, "filterParams": filterTag, "currentReport": report, "customObject": serializedCustomObject });
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs, this._nodeStateModifiedSuccess);
                    else {
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs.replace("nodeStateModified", "nodeStateModifiedDeferUpdate"), this._nodeStateModifiedSuccess);
                        if (!ej.isNullOrUndefined(ogridWaitingPopup))
                            ogridWaitingPopup.hide();
                    }
                }
            }
            else if (args.type == "nodeUncheck") {
                if (this._dataModel == "Olap") {
                    $(args.currentElement.find(".treeDrop")).remove();
                    $(args.currentElement.find(".e-icon:not(.e-chk-image, .e-plus,.e-minus)")).remove();
                    $(args.currentElement).removeClass("filter");
                    var headerTag = $(selectedElement).attr("tag"), report, eventArgs;
                    if (this.model.beforeServiceInvoke != null)
                           this._trigger("beforeServiceInvoke", { action: "removeButton", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "removeButton", "headerInfo": headerTag, "currentReport": report, "customObject": serializedCustomObject });
                    if (ej.isNullOrUndefined(headerTag))
                        return false;
                    if (headerTag.indexOf("[Measures]") >= 0 && _pivotProxy.element.find(".schemaValue .pivotButton").length == 0) {
                        var tmp = headerTag.split(":"),
                        tempAxis = tmp[0] == "Rows" ? ".schemaRow" : tmp[0] == "Columns" ? ".schemaColumn" : "";
                        _pivotProxy.element.find(tempAxis + " .pivotButton:contains('Measures')").remove();
                    }
                    else if (headerTag.indexOf("[Measures]") < 0 && headerTag.indexOf("Measures") >= 0)
                        _pivotProxy.element.find(".schemaValue .pivotButton").remove();
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", _pivotProxy.model.pivotControl.model.url + "/" + _pivotProxy.model.serviceMethods.removeButton, eventArgs, this._pvtBtnDroppedSuccess)
                    else {
                        this._removeButtonDeferUpdate = true;
                        this.doAjaxPost("POST", _pivotProxy.model.pivotControl.model.url + "/" + _pivotProxy.model.serviceMethods.removeButton, eventArgs.replace("removeButton", "removeButtonDeferUpdate"), this._onDroppedSuccess)
                    }
                }
                else {
                    if (!ej.isNullOrUndefined(this._tempFilterData)) {
                        for (var i = 0; i < this._tempFilterData.length; i++) {
                            for (var key in this._tempFilterData[i]) {
                                if (key == headerText) {
                                    this._tempFilterData[i][headerText] = [];
                                }
                            }
                        }
                    }
                    if (!ej.isNullOrUndefined(this.model.pivotControl._tempFilterData)) {
                        for (var i = 0; i < this.model.pivotControl._tempFilterData.length; i++) {
                            for (var key in this.model.pivotControl._tempFilterData[i]) {
                                if (key == headerText) {
                                    this.model.pivotControl._tempFilterData[i][headerText] = [];
                                }
                            }
                        }
                    }
                    $(args.currentElement).removeClass("filter").find(".filter").remove();
                    $(args.currentElement).removeClass("filter").find(".treeDrop").remove();
                    this.model.pivotControl._ascdes = this.model.pivotControl._ascdes.replace(headerText + "##", "");
                    if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "nodeStateModified", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeStateModified", "headerTag": JSON.stringify(headerTag), "dropAxis": dropAxis + "::", "sortedHeaders": this.model.pivotControl._ascdes, "filterParams": filterTag, "currentReport": report,"customObject":serializedCustomObject });
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeStateModified, eventArgs, this._nodeStateModifiedSuccess);
                    else {
                        this.doAjaxPost("POST", _pivotProxy.model.pivotControl.model.url + "/" + _pivotProxy.model.serviceMethods.nodeStateModified, eventArgs.replace("nodeStateModified", "nodeStateModifiedDeferUpdate"), this._nodeStateModifiedSuccess);
                        if (!ej.isNullOrUndefined(ogridWaitingPopup))
                            ogridWaitingPopup.hide();
                    }
                }
            }
        },

        _onNodeDropped: function (args) {
            if (args.dropTarget.context == undefined && args.dropTarget.className == undefined)
                return;
            var pivotDrop = false;
            if (this._dataModel == "Pivot") {
                var buttonDrag = $(args.dropTarget).parents();
                for (var i = 0; i < buttonDrag.length; i++) {
                    if (buttonDrag[i].className.split(" ")[0] == "e-pivotgrid")
                        pivotDrop = true;
                }
            }
            if ((args.dropTarget.context != undefined && args.dropTarget.context.className.indexOf("e-droppable") >= 0) || (args.dropTarget.className != undefined && args.dropTarget.className.indexOf("e-droppable") >= 0) || pivotDrop) {
                this._isDragging = false;
                this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
                var headerTag = $(args.droppedElement).attr("tag"), headerText = args.droppedElement.text(), filterTag, filterItems = "", report,
                    selectedPvtBtn = this.element.find(".pivotButton:contains(" + headerText + ")"),
                    selectedTreeNode = this._tableTreeObj.element.find("li:contains('" + headerText + "'):last"),
                    isFiltered = $(selectedPvtBtn).find(".filterIndicator").length > 0 ? true : false,
                    isSorted = $(selectedPvtBtn).find("~ span:eq(0) span.descending").length > 0 ? true : false;

                if (pivotDrop)
                    if (args.event.target.tagName == "TH" || args.event.target.tagName == "TD")
                        this._droppedClass = args.event.target.className.split(" ")[0] == "rowheader" ? "schemaRow" : args.event.target.className.split(" ")[0] == "colheader" ? "schemaColumn" : $(args.event.target).hasClass("value") ? "schemaValue" : "";
                    else
                        this._droppedClass = (args.event.target.className.split(" ")[0] == "rows" || args.event.target.className.split(" ")[0] == "emptyRows") ? "schemaRow" : args.event.target.className.split(" ")[0] == "columns" ? "schemaColumn" : args.event.target.className.split(" ")[0] == "values" ? "schemaValue" : args.event.target.className.split(" ")[0] == "drag" ? "schemaFilter" : "";
                else
                    this._droppedClass = args.event.target.className.split(" ")[0];
                if (this._dataModel == "Olap") {
                    selectedTreeNode = this._tableTreeObj.element.find("li[tag='" + headerTag + "']")
                }
                if (headerTag.split('.').length > 2) {
                    var tagName = headerTag.split('.')[0] + "." + headerTag.split('.')[1];
                    selectedTreeNode = this._tableTreeObj.element.find("li[tag='" + tagName + "']");
                }

                if (selectedTreeNode.find(".e-chk-inact").length > 0 && (this._droppedClass != "" && (this._droppedClass == "schemaRow" || this._droppedClass == "schemaColumn" || this._droppedClass == "schemaFilter" || this._droppedClass == "schemaValue"))) {
                    this._tableTreeObj.checkNode(selectedTreeNode);
                    return false;
                }
                if (ej.isNullOrUndefined(headerTag))
                    for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                        if (this.model.pivotTableFields[i].name == headerText) {
                            this.model.pivotTableFields[i].isSelected = true;
                            headerTag = this.model.pivotTableFields[i];
                        }
                    }
                if (!ej.isNullOrUndefined(this._tempFilterData))
                    for (var i = 0; i < this._tempFilterData.length; i++) {
                        if (!ej.isNullOrUndefined(this._tempFilterData[i][headerText])) {
                            for (var j = 0; j < this._tempFilterData[i][headerText].length; j++)
                                filterItems += "##" + this._tempFilterData[i][headerText][j];
                        }
                    }
                if (args.droppedElement[0].tagName.toLowerCase() == "button") {
                    if (selectedPvtBtn.length > 1)
                        for (var i = 0; i < selectedPvtBtn.length; i++) {
                            if ($(selectedPvtBtn[i]).text() == headerText)
                                $(selectedPvtBtn[i]).remove();
                        }
                    else
                        $(selectedPvtBtn).remove();
                }
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                if (this._dataModel == "Pivot") {
                    var droppedPosition = "";
                    if (pivotDrop)
                        droppedPosition = this._droppedPosition = "";
                    else
                        droppedPosition = this._droppedPosition = this._setSplitBtnTargetPos(args.event);
                    if (ej.isNullOrUndefined(headerTag) || headerTag == "undefined")
                        for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                            if (this.model.pivotTableFields[i].name == headerText) {
                                this.model.pivotTableFields[i].isSelected = true;
                                headerTag = this.model.pivotTableFields[i];
                                $(selectedPvtBtn).remove();
                                break;
                            }
                        }
                    if (selectedTreeNode.find(".e-chk-inact").length > 0 && (this._droppedClass != "" && (this._droppedClass == "schemaRow" || this._droppedClass == "schemaColumn" || this._droppedClass == "schemaFilter" || this._droppedClass == "schemaValue")))
                        this._tableTreeObj.checkNode(selectedTreeNode);
                    else {
                        this._createPivotButton(headerText, this._droppedClass == "schemaRow" ? "row" : this._droppedClass == "schemaColumn" ? "column" : this._droppedClass == "schemaFilter" ? "filter" : this._droppedClass == "schemaValue" ? "value" : "", isFiltered, isSorted, droppedPosition);
                        if (this.model.beforeServiceInvoke != null)
                            this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                        var serializedCustomObject = JSON.stringify(this.model.customObject);
                        var eventArgs = JSON.stringify({ "action": "nodeDropped", "dropAxis": this._droppedClass + "::" + droppedPosition, "headerTag": JSON.stringify(headerTag), "sortedHeaders": this.model.pivotControl._ascdes, "currentReport": report, "customObject": serializedCustomObject });
                        if (!ej.isNullOrUndefined(ogridWaitingPopup))
                            ogridWaitingPopup.show();
                        if (!this.model.pivotControl.model.enableDeferUpdate)
                            this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._pvtNodeDroppedSuccess);
                        else {
                            this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._pvtNodeDroppedSuccess);
                            if (!ej.isNullOrUndefined(ogridWaitingPopup))
                                ogridWaitingPopup.hide();
                        }
                    }
                }
                else {
                    var axis = (args.dropTarget[0] != undefined && args.dropTarget[0].className == "pivotButton") ? $(args.dropTarget).parents()[0] : (args.dropTarget[0] != undefined && args.dropTarget[0].tagName.toLowerCase() == "td") ? args.dropTarget.children(":last")[0] : args.dropTarget;
                    this._droppedClass = axis[0].className.split(" ")[0];
                    var axisName = axis[0].className.split(" ")[0] == "schemaColumn" ? "Categorical" : axis[0].className.split(" ")[0] == "schemaRow" ? "Series" : axis[0].className.split(" ")[0] == "schemaFilter" ? "Slicer" : "";
                    if (axisName == "" && args.droppedElement.parents("li:eq(0)").attr("tag").toUpperCase().indexOf("[MEASURES]") > -1)
                        axisName = (this.element.find(".schemaRow .pivotButton:contains('Measures')").length > 0 || this.element.find(".schemaRow .pivotButton:contains('MEASURES')").length > 0) ? "Series" : "Categorical";
                    var droppedPosition = this._setSplitBtnTargetPos(args.event);
                    //this._createPivotButton(headerText, this._droppedClass == "schemaRow" ? "row" : this._droppedClass == "schemaColumn" ? "column" : this._droppedClass == "schemaFilter" ? "filter" : this._droppedClass == "schemaValue" ? "value" : "", isFiltered, isSorted, droppedPosition);
                    var params = this._currentCubeName + "--" + headerTag + "--" + axisName + "--" + droppedPosition, filterParams = "";
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    var eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "filterParams": filterParams, "currentReport": report, "layout": this.model.pivotControl.model.layout ,"customObject":serializedCustomObject});
                    if (!this.model.pivotControl.model.enableDeferUpdate && !ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.show();
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                    else {
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._onDroppedSuccess);
                    }
                }
            }
        },

        _onPvtBtnDropped: function (args) {
            this._isDragging = false;
            this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
            var axis = null, axisName = "", headerTag = "", headerText, droppedClass, isFiltered = false, isSorted = false;
            if (this._dataModel == "Olap") {
                axis = args.target.className.indexOf("pvtBtn") > -1 ? $(args.target).parents(".e-droppable")[0] : args.target.className.indexOf("e-droppable") ? args.target : args.target[0].tagName.toLowerCase() == "td" ? args.target.children(":last")[0] : args.target;
                axisName = axis.className.split(" ")[0] == "schemaColumn" ? "Categorical" : axis.className.split(" ")[0] == "schemaRow" ? "Series" : axis.className.split(" ")[0] == "schemaFilter" ? "Slicer" : "";
                if (axisName == "" && args.element.parent().attr("tag").indexOf("[Measures]") > -1)
                    axisName = this.element.find(".schemaRow .pivotButton:contains('Measures')").length > 0 ? "Series" : "Categorical";
                headerTag = $(args.element.parent()[0]).attr("tag");
                droppedClass = axis.className.split(" ")[0];
            }
            else {
                headerText = args.element.text(), droppedClass = args.target.className.indexOf("e-droppable") >= 0 ? args.target.className.split(" ")[0] : (args.target.className.split(" ")[0] == "pvtBtn" ? $(args.target).parents('div.e-droppable').attr("class").split(" ")[0] : ""),
                    headerTag, isFiltered = args.element.find("~ .filterIndicator").length > 0 ? true : false,
                    isSorted = args.element.find("~ span:eq(0) span.descending").length > 0 ? true : false;
                for (var i = 0; i < this.model.pivotTableFields.length; i++) {
                    if (this.model.pivotTableFields[i].name == headerText) {
                        this.model.pivotTableFields[i].isSelected = true;
                        headerTag = this.model.pivotTableFields[i];
                    }
                }
            }
            this.element.find(".dragClone").remove();
            if (this._dataModel == "Olap") {
                if ((headerTag.split(":")[1].toLocaleUpperCase() == "MEASURES" && (droppedClass != "schemaColumn" || droppedClass != "schemaRow") && (droppedClass == "schemaFilter" || droppedClass == "schemaValue")) || (headerTag.toUpperCase().indexOf("[MEASURES]") > -1 && droppedClass != "schemaValue" && (droppedClass == "schemaRow" || droppedClass == "schemaColumn" || droppedClass == "schemaFilter"))) {
                    this._createErrorDialog();
                    return false;
                }
                else if (headerTag.toUpperCase().indexOf("[MEASURES]") == -1 && droppedClass == "schemaValue") {
                    this._createErrorDialog();
                    return false;
                }
            }
            if (droppedClass == "schemaValue" || droppedClass == "schemaRow" || droppedClass == "schemaColumn" || droppedClass == "schemaFilter") {
                var droppedPosition = this._setSplitBtnTargetPos(args.event), params, report, eventArgs;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                $(args.element.parent()).remove();
                if (!this.model.pivotControl.model.enableDeferUpdate && !ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.show();
                if (this._dataModel == "Olap") {
                    params = this._currentCubeName + "--" + headerTag + "--" + axisName + "--" + droppedPosition;
                    if (this.model.beforeServiceInvoke != null)
                   this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                   var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropType": "SplitButton", "nodeInfo": params, "currentReport": report,"customObject":serializedCustomObject });
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._onDroppedSuccess);
                    else {
                        this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._onDroppedSuccess);
                    }
                }
                else {
                    this._createPivotButton(headerText, droppedClass == "schemaRow" ? "row" : droppedClass == "schemaColumn" ? "column" : droppedClass == "schemaFilter" ? "filter" : droppedClass == "schemaValue" ? "value" : "", isFiltered, isSorted, droppedPosition);
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    eventArgs = JSON.stringify({ "action": "nodeDropped", "dropAxis": droppedClass + "::" + droppedPosition, "headerTag": JSON.stringify(headerTag), "sortedHeaders": this.model.pivotControl._ascdes, "currentReport": report,"customObject":serializedCustomObject });
                    successMethod = this.model.pivotControl._renderControlSuccess;
                    if (!this.model.pivotControl.model.enableDeferUpdate)
                       this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs, this._pvtNodeDroppedSuccess);
                    else {
                       this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.nodeDropped, eventArgs.replace("nodeDropped", "nodeDroppedDeferUpdate"), this._pvtNodeDroppedSuccess);
                        if (!ej.isNullOrUndefined(ogridWaitingPopup))
                            ogridWaitingPopup.hide();
                    }
                }
            }
            else
                this._removePvtBtn(args);
        },

        _createErrorDialog: function () {
            if (this.element.find(".errorDialog").length == 0) {
                var dialogElem = ej.buildTag("div.errorDialog#ErrorDialog", ej.buildTag("div.warningImg")[0].outerHTML + ej.buildTag("div.warningContent", "The field you are moving cannot be placed in that area of the report.")[0].outerHTML + ej.buildTag("div", ej.buildTag("button#ErrOKBtn.errOKBtn", "OK")[0].outerHTML)[0].outerHTML).attr("title", "Warning")[0].outerHTML;
                this.element.append(dialogElem);
                this.element.find(".errorDialog").ejDialog({ target: "#" + this._id, enableResize: false, width: "400px" });
                this._errorDialog = this.element.find(".errorDialog").data("ejDialog");
                $("#" + this._errorDialog._id + "_wrapper").css({ left: "50%", top: "50%" });
                this.element.find(".errOKBtn").ejButton({ click: ej.proxy(this._errOKBtnClick, this) });
            }
            else {
                this._errorDialog.open();
            }
        },

        _errOKBtnClick: function (args) {
            this._errorDialog._ejDialog.find("button.e-dialog-icon").trigger("click");
        },

        _hideSchemaDesigner: function (args) {
            $("#PivotSchemaDesigner").animate({ left: "-" + $("#" + this._id).width() + "px" }, 500, function () {
            });
            this.element.find(".collapseSchema").addClass("expandSchema").removeClass("collapseSchema");
        },

        _showSchemaDesigner: function (args) {
            $("#PivotSchemaDesigner").animate({ left: "5px" });
            this.element.find(".expandSchema").addClass("collapseSchema").removeClass("expandSchema");
        },

        _reSizeHandler: function (args) {
            if (this.model.pivotControl == null || !this.model.pivotControl.model.isResponsive) return;
            var isUpdate = true, parentElWidth = $("#" + this._id).parent().width();
            if (parentElWidth < 750 && isUpdate) {
                $("#" + this._id).css({ "position": "absolute" });
                $("#" + this.model.pivotControl._id).width("100%");
                isUpdate = false;
                this.element.find(".responsiveSchema").addClass("expandSchema").removeClass("responsiveSchema");
                this._hideSchemaDesigner();
                this._tableTreeObj.option('allowDragAndDrop', false);
            }
            else if (parentElWidth > 750) {
                $("#" + this._id).css({ "position": "static" });
                $("#" + this.model.pivotControl._id).width(parseInt(100 - $("#" + this._id).width() / $(window).width() * 100) - 4 + "%");
                this.element.find(".expandSchema, .collapseSchema").addClass("responsiveSchema").removeClass("expandSchema").removeClass("collapseSchema");
                this._tableTreeObj.option('allowDragAndDrop', true)
                isUpdate = true;
            }
        },

        _dialogBtnClick: function (args) {
            this.element.find(".e-dialog, .clientDialog").hide();
            var unselectedNodes = "", enableFilterIndigator;
            if (args.model.text.toLowerCase() != "cancel") {
                var uncheckedNodes = this._dataModel == "Olap" ? unselectedNodes = this._getUnSelectedNodes() + "FILTERED" + this._getSelectedNodes(this._curFilteredAxis == "schemaFilter" ? true : false) : this._memberTreeObj.element.find(":input:gt(0).nodecheckbox:not(:checked)"), isFiltered = false,
                filterParams = this._dataModel == "Olap" ? uncheckedNodes : this._curFilteredAxis + "::" + this._curFilteredText + "::FILTERED";
                var text = this._curFilteredText, textArr = new Array(), obj = {};
                enableFilterIndigator = this._getUnSelectedNodes() != "" ? true : false;
                if (this._dataModel == "Pivot") {
                    for (var i = 0; i < uncheckedNodes.length; i++)
                        textArr.push($(uncheckedNodes[i].parentElement).siblings("a").text());
                    obj[text] = textArr;
                    if (ej.isNullOrUndefined(this._tempFilterData))
                        this._tempFilterData = new Array();
                    for (var i = 0; i < this._tempFilterData.length; i++) {
                        if (!ej.isNullOrUndefined(this._tempFilterData[i][text])) {
                            this._tempFilterData[i][text] = textArr;
                            isFiltered = true;
                        }
                    }
                    if (!isFiltered)
                        this._tempFilterData.push(obj);
                    this.model.pivotControl._tempFilterData = this._tempFilterData;
                    if (ej.isNullOrUndefined(this._curFilteredAxis) || this._curFilteredAxis != "") {
                        for (var i = 0; i < uncheckedNodes.length; i++)
                            filterParams += "##" + $(uncheckedNodes[i].parentElement).siblings("a").text();
                    }
                }
                else {
                    obj[this._curFilteredText] = filterParams;
                    if (ej.isNullOrUndefined(this._tempFilterData))
                        this._tempFilterData = new Array();
                    for (var i = 0; i < this._tempFilterData.length; i++) {
                        if (!ej.isNullOrUndefined(this._tempFilterData[i][this._curFilteredText])) {
                            this._tempFilterData[i][this._curFilteredText] = textArr;
                            isFiltered = true;
                        }
                    }
                    if (!isFiltered)
                        this._tempFilterData.push(obj);
                }
                var sortedHeaders = this._getSortedHeaders(),
                filteredBtn = this.model.layout == "excel" ? this.element.find(".schemaFieldTree li:contains('" + this._curFilteredText + "')") : this.element.find("." + this._curFilteredAxis + " .pivotButton:contains(" + this._curFilteredText + ") .filterBtn");
                if (this.model.layout == "excel") {
                    if ($(_pivotProxy._selectedTreeNode).parents("li:eq(0)").children().children("span").hasClass("hierarchyCDB")) {
                        _pivotProxy._selectedTreeNode = $($(_pivotProxy._selectedTreeNode).parents("li:eq(0)"));
                    }
                    if (enableFilterIndigator && $($(_pivotProxy._selectedTreeNode)).find(".filter").length <= 0) {
                        var filterSpan = ej.buildTag("span.e-icon", { "display": "none" }).addClass("filter")[0].outerHTML;
                        $($(_pivotProxy._selectedTreeNode).find(".e-text")[0]).after(filterSpan);
                    }
                    else if (!enableFilterIndigator)
                        $(_pivotProxy._selectedTreeNode).find(".filter").remove();
                    if (this._curFilteredAxis == "")
                        return false;
                }
                var report;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "filtering", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject),
                eventArgs = this.model.pivotControl.model.customObject != {} && filterParams != "" ? JSON.stringify({ "action": "filtering", "filterParams": filterParams, "sortedHeaders": this.model.pivotControl._ascdes, "currentReport": report, "customObject": serializedCustomObject }) :
            this.model.pivotControl.model.customObject != {} ? JSON.stringify({ "action": "filtering", "currentReport": report, "customObject": serializedCustomObject }) :
            filterParams != "" ? JSON.stringify({ "action": "filtering", "filterParams": filterParams, "sortedHeaders": this.model.pivotControl._ascdes, "currentReport": report }) : JSON.stringify({ "action": "filtering" });
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.show();
                if (!this.model.pivotControl.model.enableDeferUpdate)
                    this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.filtering, eventArgs, this._filterElementSuccess);
                else
                {
                    this.model.pivotControl._filterUpdate.push(filterParams);
                    if (!ej.isNullOrUndefined(ogridWaitingPopup))
                        ogridWaitingPopup.hide();
                }
            }
            _pivotProxy._selectedTreeNode = null;
        },

        _onNodeExpand: function (args) {
            var selectedItem = (args.event.originalEvent != undefined && args.event.originalEvent.target != undefined) ? args.event.originalEvent.target : args.event.target; var tagInfo, dimenName = "";
            var childCount = $(selectedItem).parents("li").first().children().find("li").length;
            if (selectedItem.className.indexOf("e-plus") > -1 && childCount == 0) {
                var checkedState = $(selectedItem).parent().find("input[type=checkbox]").prop("checked"), report;
                var tagInfo = $(selectedItem).parents("li").first().attr("tag");
                this.pNode = selectedItem;
                try {
                    report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
                }
                catch (err) {
                    report = this.model.pivotControl.getOlapReport();
                }
                var pivotButtons = this.element.find(".pivotButton"), parentTag;
                for (var bt = 0; bt < pivotButtons.length > 0; bt++) {
                    if ($(pivotButtons[bt]).attr("tag").indexOf(this._curFilteredText) > -1)
                        dimenName = $(pivotButtons[bt]).attr("tag");
                }
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "memberExpanded", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                this.doAjaxPost("POST", this.model.pivotControl.model.url + "/" + this.model.serviceMethods.memberExpand, JSON.stringify({
                    "action": "memberExpanded", "checkedStatus": checkedState, "parentNode": $(selectedItem).parents("li")[0].id, "tag": tagInfo, "cubeName": this._currentCubeName + "##" + dimenName, "currentReport": report,"customObject": serializedCustomObject 
                }), this._fetchChildNodeSuccess);
            }
        },
        _nodeStateModifiedSuccess: function (report) {
            if (report[0] != undefined) {
                if (report[2] != null && report[2] != undefined)
                    this.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                if (report.d[2] != null && report.d[2] != undefined)
                    this.model.customObject = report.d[2].Value;
            }
            else {
                if (report.customObject != null && report.customObject != undefined)
                    this.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "nodeStateModified", element: this.element, customObject: this.model.customObject });
            if (!this.model.pivotControl.model.enableDeferUpdate)
                this.model.pivotControl._renderControlSuccess(report);
            else
                this.model.pivotControl._deferUpdateSuccess(report);
        },
        _pvtNodeDroppedSuccess: function (report) {
            if (report[0] != undefined) {
                if (report[2] != null && report[2] != undefined)
                    this.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                if (report.d[2] != null && report.d[2] != undefined)
                    this.model.customObject = report.d[2].Value;
            }
            else {
                if (report.customObject != null && report.customObject != undefined)
                    this.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });

            if (!this.model.pivotControl.model.enableDeferUpdate)
                this.model.pivotControl._renderControlSuccess(report);
            else
                this.model.pivotControl._deferUpdateSuccess(report);
        },
        _pvtBtnDroppedSuccess: function (report) {
            if (report[0] != undefined) {
                if (report[2] != null && report[2] != undefined)
                    this.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                if (report.d[2] != null && report.d[2] != undefined)
                    this.model.customObject = report.d[2].Value;
            }
            else {
                if (report.customObject != null && report.customObject != undefined)
                    this.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "removeButton", element: this.element, customObject: this.model.customObject });
            this.model.pivotControl._renderControlSuccess(report);
        },
        _filterElementSuccess: function (report) {
            if (report[0] != undefined) {
                if (report[2] != null && report[2] != undefined)
                    this.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                if (report.d[2] != null && report.d[2] != undefined)
                    this.model.customObject = report.d[2].Value;
            }
            else {
                if (report.customObject != null && report.customObject != undefined)
                    this.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "filtering", element: this.element, customObject: this.model.customObject });
            this.model.pivotControl._renderControlSuccess(report);
        },
        _fetchChildNodeSuccess: function (data) {
            var newDataSource; var parentNode;
            if (data.length > 1 && data[0] != undefined){
                newDataSource = JSON.parse(data[0].Value);
                if (data[1] != null && data[1] != undefined)
                this.model.customObject = data[1].Value;
            }
            else if (data.d != undefined) {
                newDataSource = JSON.parse(data.d[0].Value);
                if (data.d[1] != null && data.d[1] != undefined)
                    this.model.customObject = data.d[1].Value;
            }
            else {
                newDataSource = JSON.parse(data.ChildNodes);
                if (data.customObject != null && data.customObject != undefined)
                     this.model.customObject = data.customObject;
            }
            var mapper = { id: "id", parentId: "pid", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus" };
            if ($(this.pNode).parents("li").length > 1)
                parentNode = $(this.pNode).parents("li").first();
            else
                parentNode = $(this.pNode).parents("li");
            $($(parentNode).find('input.nodecheckbox')[0]).ejCheckBox({ checked: false })
            this._memberTreeObj._createSubNodesWhenLoadOnDemand(newDataSource, this.pNode, mapper);
            $.each($(parentNode).children().find("li"), function (index, value) {
                value.setAttribute("tag", newDataSource[index].tag);
            });
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "memberExpanded", element: this.element, customObject: this.model.customObject });
            // this._memberTreeObj._expandCollapseAction(this.pNode);
        },

        _fetchMemberSuccess: function (msg) {
            if (msg[0] != undefined && msg.length > 0) {
                this._currentMembers = msg[0].Value;
                if (msg[1] != null && msg[1] != undefined)
                    this.model.customObject = msg[1].Value;
            }
            else if (msg.d != undefined && msg.d.length > 0) {
                this._currentMembers = msg.d[0].Value;
                if (msg.d[1] != null && msg.d[1] != undefined)
                    this.model.customObject = msg.d[1].Value;
            }
            else if (msg != undefined && msg.length > 0) {
                this._currentMembers = msg.EditorTreeInfo;
               if (msg!= null && msg.customObject != undefined)
                   this.model.customObject = msg.customObject;
            }
            else if (msg != undefined) {
                this._currentMembers = msg.EditorTreeInfo;
                if (msg != null && msg.customObject != undefined)
                    this.model.customObject = msg.customObject;
            }
           if (this.model.afterServiceInvoke != null)
               this._trigger("afterServiceInvoke", { action: "fetchMembers", element: this.element, customObject: this.model.customObject });
            this._createDialog(this._dialogTitle, this._currentMembers);
        },

        _onDroppedSuccess: function (args) {
            var report = null;
            if (this.model.pivotControl.model.enableDeferUpdate) {
                report = this._removeButtonDeferUpdate == false ? (!ej.isNullOrUndefined(args.OlapReport) ? JSON.parse(args.OlapReport) : JSON.parse(args.d[0].Value)) : (!ej.isNullOrUndefined(args.OlapReport) ? report = args.OlapReport : report = args.d[0].Value);
            }
            else{
                if (!ej.isNullOrUndefined(args[0]) && args.length > 0) {
                    report = JSON.parse(args[1].Value);
                    if (args[2] != null && args[2] != undefined)
                    this.model.customObject=(args[2].Value);
                }
                else if (!ej.isNullOrUndefined(args.d) && args.d.length > 0) {
                    report = JSON.parse(args.d[1].Value);
                    if (args.d[2] != null && args.d[2] != undefined)
                        this.model.customObject = (args.d[2].Value);
                }
                else if (!ej.isNullOrUndefined(args) && !ej.isNullOrUndefined(args.OlapReport))
                    report = JSON.parse(args.OlapReport);
                if (args.customObject!= null && args.customObject != undefined)
                    this.model.customObject = (args.customObject);
            }
            if (this._removeButtonDeferUpdate == false) {
                this.element.find(".axisTable .pivotButton").remove();
                this._setPivotRows(report.PivotRows);
                this._setPivotColumns(report.PivotColumns);
                this._setPivotCalculations(report.PivotCalculations);
                this._setFilters(report.Filters);
                this.element.find(".axisTable .schemaFilter").append(this._createPivotButtons(this.model.filters, "filter"));
                this.element.find(".axisTable .schemaRow").append(this._createPivotButtons(this.model.pivotRows, "row"));
                this.element.find(".axisTable .schemaColumn").append(this._createPivotButtons(this.model.pivotColumns, "column"));
                this.element.find(".axisTable .schemaValue").append(this._createPivotButtons(this.model.pivotCalculations, "values"));
                this.element.find(".pivotButton .pvtBtn").ejButton({ size: "normal" }).ejDraggable({
                    handle: 'button', clone: true,
                    cursorAt: { left: -5, top: -5 },
                    dragStart: ej.proxy(function (args) {
                        this._isDragging = true;
                    }, this),
                    dragStop: ej.proxy(this._onPvtBtnDropped, this),
                    helper: ej.proxy(function (event, ui) {
                        if (event.sender.target.className.indexOf("e-btn") > -1)
                            return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                        else
                            return false;
                    }, this)
                });
                this._unWireEvents();
                this._wireEvents();
            }
            if(!this.model.pivotControl.model.enableDeferUpdate)
                this.model.pivotControl._renderControlSuccess(args);
            else
            {
                if (!ej.isNullOrUndefined(ogridWaitingPopup))
                    ogridWaitingPopup.hide();
                if(!ej.isNullOrUndefined(args.OlapReport))
                    this.model.pivotControl.setOlapReport(args.OlapReport);
                else
                    this.model.pivotControl.setOlapReport(args.d[0].Value);
                this._removeButtonDeferUpdate = false;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
        },

        _removePvtBtn: function (args) {
            var headerText = args.element.text(), headerTag = args.element.parent().attr("tag"), report, eventArgs;
            try {
                report = JSON.parse(this.model.pivotControl.getOlapReport()).Report;
            }
            catch (err) {
                report = this.model.pivotControl.getOlapReport();
            }
            if (this._dataModel == "Olap") {
                var uniqueName = ((headerTag.indexOf("[Measures]") > -1) || (headerTag.indexOf("[") > -1)) ? headerTag.split(":")[1] : this._getNodeUniqueName(headerTag);
                selectedTreeNode = this._getNodeByUniqueName(uniqueName);
                this._tableTreeObj.uncheckNode(selectedTreeNode);
            }
            else {
                selectedTreeNode = this._tableTreeObj.element.find("li:contains('" + headerText + "')");
                this._tableTreeObj.uncheckNode(selectedTreeNode);
            }
        },

        _createDialog: function (title, treeViewData) {
            this.element.find(".e-dialog, .clientDialog").remove();
            var dialogContent = ej.buildTag("div#EditorDiv.editorDiv", ej.buildTag("div", ej.buildTag("div.memberEditorDiv", ej.buildTag("div#editorTreeView.editorTreeView")[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML + "</br>",
            dialogFooter = ej.buildTag("div", ej.buildTag("button#OKBtn.dialogOKBtn", "OK")[0].outerHTML + ej.buildTag("button#CancelBtn.dialogCancelBtn", "Cancel")[0].outerHTML, { "float": "right", "margin": "-5px 0 6px" })[0].outerHTML,
            ejDialog = ej.buildTag("div#clientDialog.clientDialog", dialogContent + dialogFooter, { "opacity": "1" }).attr("title", title)[0].outerHTML,
            treeViewData = JSON.parse(treeViewData);
            var selectedData;
            if (!ej.isNullOrUndefined(this._tempFilterData)) {
                for (var i = 0; i < this._tempFilterData.length; i++) {
                    if (!ej.isNullOrUndefined(this._tempFilterData[i][title]))
                        selectedData = this._tempFilterData[i][title];
                }
                if (!ej.isNullOrUndefined(selectedData)) {
                    for (var i = 0; i < selectedData.length; i++) {
                        for (var j = 0; j < treeViewData.length; j++) {
                            if (selectedData[i] == treeViewData[j].name)
                                treeViewData[j].checkedStatus = false;
                        }
                    }
                }
            }
            $(ejDialog).appendTo("#" + this._id);
            this.element.find(".editorTreeView").ejTreeView({
                showCheckbox: true,
                loadOnDemand: true,
                height: $(".memberEditorDiv").height(),
                fields: { id: "id", text: "name", isChecked: "checkedStatus", hasChild: "hasChildren", dataSource: treeViewData },
                nodeClick: ej.proxy(this._onNodeExpand, this)
            });
            var treeViewElements = this.element.find(".editorTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            this.element.find(".dialogOKBtn, .dialogCancelBtn").ejButton({ type: ej.ButtonType.Button, click: ej.proxy(this._dialogBtnClick, this) });
            this._dialogOKBtnObj = this.element.find(".dialogOKBtn").data("ejButton");
            this._memberTreeObj = this.element.find(".editorTreeView").data("ejTreeView");
            var treeViewLi = this.element.find(".editorTreeView li:gt(0)"),
            uncheckedNodes = $(treeViewLi).find(":input.nodecheckbox:not(:checked)");
            if (uncheckedNodes.length > 0) {
                var firstNode = $(this._memberTreeObj.element.find("li:first")).find("div .e-chkbox-small > span > span:eq(0)");
                $(firstNode).removeClass("e-chk-activeicon").addClass("e-chk-indetericon");
            }
            this.element.find(".clientDialog").ejDialog({ width: 265, target: "#" + this._id, enableResize: false });
            this._memberTreeObj.model.nodeCheck = ej.proxy(this._onNodeCheckChanges, this);
            this._memberTreeObj.model.nodeUncheck = ej.proxy(this._onNodeCheckChanges, this);
            if (this._memberTreeObj.element.find(".e-plus").length == 0) {
                this._memberTreeObj.element.find(".e-item").css("padding", "0px");
            }
            this._unWireEvents();
            this._wireEvents();
            if (!ej.isNullOrUndefined(pschemaDesignerWaitingPopup))
                pschemaDesignerWaitingPopup.hide();
        },

        _createPivotButton: function (text, axis, isFiltered, isSorted, dropPostion) {
            var pvtAxis = axis == "row" ? ".schemaRow" : axis == "column" ? ".schemaColumn" : axis == "value" ? ".schemaValue" : ".schemaFilter";
            var sortBtn = (axis == "column" || axis == "row") && this.model.layout != "excel" ? ej.buildTag("input#toggleBtn" + text, {}).attr("type", "checkbox")[0].outerHTML : "",
                    filterBtn = (axis == "column" || axis == "row" || axis == "filter") && this.model.layout != "excel" ? ej.buildTag("button.filterBtn#filterBtn" + text, {}).addClass(isFiltered ? "filterIndicator" : "")[0].outerHTML : "",

            rowBtn = ej.buildTag("div.pivotButton", ej.buildTag("div.dropIndicator")[0].outerHTML + ej.buildTag("button.pvtBtn#pivotButton" + text, text)[0].outerHTML +
                    sortBtn + filterBtn + ej.buildTag("span.removePivotBtn", "", { "display": "none" }).addClass("e-icon")[0].outerHTML)[0].outerHTML;
            if (typeof (dropPostion) == "number" && $(this.element.find(pvtAxis + " .pivotButton")[dropPostion]).length > 0)
                axis == "row" ? $(this.element.find(".schemaRow .pivotButton")[dropPostion]).before(rowBtn) : axis == "column" ? $(this.element.find(".schemaColumn .pivotButton")[dropPostion]).before(rowBtn) :
            axis == "value" ? $(this.element.find(".schemaValue .pivotButton")[dropPostion]).before(rowBtn) : axis == "filter" ? $(this.element.find(".schemaFilter .pivotButton")[dropPostion]).before(rowBtn) : "";
            else
                axis == "row" ? this.element.find(".schemaRow").append(rowBtn) : axis == "column" ? this.element.find(".schemaColumn").append(rowBtn) :
            axis == "value" ? this.element.find(".schemaValue").append(rowBtn) : axis == "filter" ? this.element.find(".schemaFilter").append(rowBtn) : "";
            var pvtExp = (typeof (dropPostion) != "number" && dropPostion == "") ? " .pivotButton .pvtBtn:last" : " .pivotButton .pvtBtn:eq(" + dropPostion + ")";
            this.element.find(pvtAxis + pvtExp).ejButton({ size: "normal" }).ejDraggable({
                handle: 'button', clone: true,
                cursorAt: { left: -5, top: -5 },
                dragStart: ej.proxy(function (args) {
                    this._isDragging = true;
                }, this),
                dragStop: ej.proxy(this._onPvtBtnDropped, this),
                helper: ej.proxy(function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(this.element);
                    else
                        return false;
                }, this)
            });
            this.element.find(pvtAxis + " .pivotButton .filterBtn:last").ejButton({
                size: "normal",
                contentType: "imageonly",
                prefixIcon: "filter"
            });
            this.element.find(pvtAxis + " .pivotButton input:last").ejToggleButton({
                size: "normal",
                contentType: "imageonly",
                defaultPrefixIcon: "ascending",
                activePrefixIcon: "descending",
                click: ej.proxy(this._onSortBtnClick, this)
            });
            if (isSorted) {
                this.element.find(pvtAxis + " .pivotButton:contains(" + text + ") span:eq(0) span").removeClass("ascending").addClass("descending");
                this.element.find(pvtAxis + " .pivotButton:contains(" + text + ") span:eq(0) button").addClass("e-active");
            }
            this.element.find(".dropIndicator").removeClass("dropIndicatorHover");
        },

        _createPivotButtons: function (rows, axis) {
            if (!ej.isNullOrUndefined(rows)) {
                var rowBtns = "", tagAxis = axis == "column" ? "Columns" : axis == "row" ? "Rows" : axis == "filter" ? "Slicers" : "";
                if (tagAxis == "" && axis == "values") {
                    var tempAxis = this.element.find(".pivotButton:contains('Measures')").length > 0 ? this.element.find(".pivotButton:contains('Measures')").parent()[0].className.split(" ")[0] : "schemaColumn";
                    tagAxis = tempAxis == "schemaColumn" ? "Columns" : "Rows";
                }
                for (var i = 0; i < rows.length; i++) {
                    var sortBtn = (axis == "column" || axis == "row") && this.model.layout != "excel" ? ej.buildTag("input#toggleBtn" + rows[i].FieldHeader || rows[i].FieldName, {}).attr("type", "checkbox")[0].outerHTML : "",
                    filterBtn = (axis == "column" || axis == "row" || axis == "filter") && this.model.layout != "excel" ? ej.buildTag("button.filterBtn#filterBtn" + (rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName), {})[0].outerHTML : "";
                    rowBtns += ej.buildTag("div.pivotButton", ej.buildTag("div.dropIndicator")[0].outerHTML + ej.buildTag("button.pvtBtn#pivotButton" + rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName, rows[i].FieldHeader || rows[i].FieldName || rows[i].Name || rows[i].DimensionName)[0].outerHTML +
                    sortBtn + filterBtn + ej.buildTag("span.removePivotBtn", "", { "display": "none" }).addClass("e-icon")[0].outerHTML).attr("tag", tagAxis + ":" + rows[i].Tag)[0].outerHTML;
                }
                return rowBtns;
            }
        },

        _setSplitBtnTargetPos: function (event) {
            var targetPosition = ""; var AEBdiv; var targetSplitBtn; var className;
            if (event.event != undefined && event.event.type == "touchend") {
                targetSplitBtn = event.event.originalEvent.target != null ? $(event.event.originalEvent.target).parents(".pivotButton") : $(event.event.originalEvent.srcElement).parents(".pivotButton");
                className = event.event.originalEvent.target != null ? event.event.originalEvent.target.className : event.event.originalEvent.srcElement.className;
            }
            else {
                targetSplitBtn = event.originalEvent.target != null ? $(event.originalEvent.target).parents(".pivotButton") : $(event.originalEvent.srcElement).parents(".pivotButton");
                className = event.originalEvent.target != null ? event.originalEvent.target.className : event.originalEvent.srcElement.className;
            }
            this._droppedClass = targetSplitBtn.length > 0 ? $(targetSplitBtn[0]).parent()[0].className.split(" ")[0] : className.split(" ")[0];
            if (targetSplitBtn[0] || (className != undefined && className != null && jQuery.type(className) == "string" && className.indexOf("pivotButton") > -1)) {
                targetSplitBtn = targetSplitBtn[0] ? targetSplitBtn[0] : event.originalEvent.target != null ? event.originalEvent.target : event.originalEvent.srcElement;
                if (event.event != undefined && event.event.type == "touchend")
                    AEBdiv = event.target;
                else if (this._droppedClass != null && this._droppedClass != undefined && this._droppedClass != "")
                    AEBdiv = this.element.find("." + this._droppedClass)[0];
                else
                    AEBdiv = targetSplitBtn.parentNode;
                for (var i = 0; i < AEBdiv.children.length; i++) {
                    if ($(AEBdiv.childNodes[i]).attr("tag") == $(targetSplitBtn).attr("tag"))
                        targetPosition = i;
                }
            }
            return targetPosition;
        },

        _getUnSelectedNodes: function () {
            var treeElement = this.element.find(".editorTreeView")[0];
            var unselectedNodes = "";
            var data = $(treeElement).find(":input.nodecheckbox:not(:checked)");
            for (var i = 0; i < data.length; i++) {
                if (!($(data[i].parentElement).find('span:nth-child(1)').attr('class').indexOf("e-chk-act") > -1) && $(data[i].parentElement).attr('aria-checked') != 'mixed') {
                    var parentNode = $(data[i]).parents('li:eq(0)');
                    unselectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                }
            }
            return unselectedNodes;
        },

        _getSelectedNodes: function (isSlicer) {
            if (isSlicer) {
                var treeElement = this.element.find(".editorTreeView")[0].childNodes[0];
                var selectedNodes = new Array();
                var data = $(treeElement).children();
                for (var i = 0; i < data.length; i++) {
                    var parentNode = data[i];
                    var nodeInfo = { caption: $(parentNode.firstChild).find("a").text(), parentId: parentNode.parentElement.parentElement.className.indexOf("editorTreeView") > -1 ? "None" : $(parentNode).parents()[1].id, id: parentNode.id, checked: $(parentNode).find(':input.nodecheckbox')[0].checked || ($(parentNode).find('span:nth-child(1)').attr('class').indexOf("e-chk-indeter") > -1), expanded: $(parentNode.firstChild).find(".e-minus").length > 0 ? true : false, childNodes: new Array(), tag: $(parentNode).attr('tag') };
                    if ($(parentNode).find("ul:first").children().length > 0) {
                        var liElements = $(parentNode).find("ul:first").children();
                        for (var j = 0; j < liElements.length; j++) {
                            nodeInfo.childNodes.push(this._getNodeInfo(liElements[j]));
                        }
                    }
                    selectedNodes.push(nodeInfo);
                }
                return JSON.stringify(selectedNodes);
            }
            else {
                var treeElement = $(".editorTreeView");
                var selectedNodes = "";
                var data = $(treeElement).find(':input.nodecheckbox');
                for (var i = 0; i < data.length; i++) {
                    if (data[i].checked || $(data[i].parentElement).attr('aria-checked') == 'mixed') {
                        var parentNode = $(data[i]).parents('li:eq(0)');
                        selectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                    }
                }
                return selectedNodes;
            }
        },

        _getNodeInfo: function (node) {
            var childNode = { caption: $(node.firstChild).find("a").text(), parentId: node.parentElement.parentElement.className.indexOf("editorTreeView") > -1 ? "None" : $(node).parents()[1].id, id: node.id, checked: ($(node).find(':input.nodecheckbox')[0].checked || $(node).find('span:nth-child(1)').attr('class').indexOf("e-chk-indeter") > -1), expanded: $(node.firstChild).find(".e-minus").length > 0 ? true : false, childNodes: new Array(), tag: $(node).attr('tag') };
            if ($(node).find("ul:first").children().length > 0) {
                var liElements = $(node).find("ul:first").children();
                for (var i = 0; i < liElements.length; i++) {
                    childNode.childNodes.push(this._getNodeInfo(liElements[i]));
                }
            }
            return childNode;
        },

        _getNodeUniqueName: function (headerTag) {
            var uniqueName = "", selectedTreeNode = null;
            for (var i = 0; i < (headerTag.split(':')[1]).split('.').length; i++) {
                if (uniqueName != "")
                    uniqueName += ".";
                uniqueName += "[" + ((headerTag.split(':')[1]).split('.')[i]) + "]";
            }
            return uniqueName;
        },

        _getNodeByUniqueName: function (nodeUniqueName) {
            for (var i = 0; i < $(_pivotProxy._tableTreeObj.element.find("li")).length; i++) {
                if ($(_pivotProxy._tableTreeObj.element.find("li")[i]).attr("tag") == nodeUniqueName)
                    selectedTreeNode = $((_pivotProxy._tableTreeObj.element.find("li"))[i]);
            }

            return selectedTreeNode;
        },

        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: $.proxy(onComplete, this),
                error: function (msg, textStatus, errorThrown) {
                    if (_proxy.model.progressMode != "infinite") {
                        window.clearInterval(oGridTimer);
                        if (_proxy.model.progressMode == "progress")
                            $("#" + _proxy._id + "progressText")[0].innerHTML = _proxy._getLocalizedLabels("MDXqueryExecutionFailed") + "!!!";
                        $("#" + ogridProgressBar._id).show();
                        ogridProgressBar.setModel({ value: 0, text: errorThrown });
                    }
                    else
                        ogridWaitingPopup.hide();
                }
            });
        }
    });

    ej.PivotSchemaDesigner.locale = {};

    ej.PivotSchemaDesigner.locale["en-US"] = {
        PivotTableFieldList: "PivotTable Field List",
        ChooseFieldsToAddToReport: "Choose fields to add to report:",
        DragFieldBetweenAreasBelow: "Drag fields between areas below:",
        ReportFilter: "Report Filter",
        ColumnLabel: "Column Label",
        RowLabel: "Row Label",
        Values: "Values",
        DeferLayoutUpdate: "Defer Layout Update",
        Update: "Update"
    };

    ej.PivotSchemaDesigner.Layouts = {
        Excel: "excel",
        Normal: "normal"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to experience Html OLAP Client component elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejOlapClient", "ej.olap.OlapClient", {

        _rootCSS: "e-olapclient",
        element: null,
        model: null,
        validTags: ["div", "span"],
        defaults: {
            url: "",
            cssClass: "",
            title: "",
            gridLayout: "Normal",
            chartType: "column",
            clientExportMode: "chartandgrid",
            enableDeferUpdate: false,
            displaySettings: {
                mode: "chartandgrid",
                defaultView: "grid",
                controlPlacement: "tab",
                enableTogglePanel: false,
                isResponsive: false,
                enableFullScreen: false
            },
            serviceMethodSettings: {
                initialize: "InitializeClient",
                removeSplitButton: "RemoveSplitButton",
                filterElement: "FilterElement",
                nodeDropped: "NodeDropped",
		        toggleAxis: "ToggleAxis",
                fetchMemberTreeNodes: "FetchMemberTreeNodes",
                cubeChanged: "CubeChanged",
                measureGroupChanged: "MeasureGroupChanged",
                toolbarServices: "ToolbarOperations",
                memberExpand: "MemberExpanded",
                saveReport: "SaveReportToDB",
                fetchReportList: "FetchReportListFromDB",
                loadReport: "LoadReportFromDB",
                updateReport: "UpdateReport",
                exportOlapClient: "Export",
                mdxQuery: "GetMDXQuery"
            },
            customObject: {},
            enableMeasureGroups: false,
            locale: "en-US",
            renderSuccess: null,
            renderFailure: null,
            renderComplete: null,
            load: null,
            chartLoad: "",
            beforeServiceInvoke: null,
            afterServiceInvoke: null
        },

        dataTypes: {
            displaySettings: "data",
            serviceMethodSettings: "data",
            customObject: "data"
        },

        observables: ["title", "gridLayout", "displaySettings.mode", "displaySettings.defaultView", "displaySettings.controlPlacement", "displaySettings.enableTogglePanel", "locale"],
        title: ej.util.valueFunction("title"),
        gridLayout: ej.util.valueFunction("gridLayout"),
        displayMode: ej.util.valueFunction("displaySettings.mode"),
        defaultView: ej.util.valueFunction("displaySettings.defaultView"),
        controlPlacement: ej.util.valueFunction("displaySettings.controlPlacement"),
        enableTogglePanel: ej.util.valueFunction("displaySettings.enableTogglePanel"),
        locale: ej.util.valueFunction("locale"),

        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            oclientProxy = this;
            currentReport = "";
            currentCubeName = "";
            reports = "";
            reportsCount = 0;
            ogridObj = null;
            ochartObj = null;
            chartObj = null;
            this._olapReport = "";
            this._keypress = false;
            memberTreeObj = null;
            this._chartHeight = 0;
            this._chartWidth = 0;
            this._gridHeight = 0;
            this._gridWidth = 0;
            this._initStyles = new Array();
            this._toggleStyles = new Array();
            this._initToggle = true;
            this._dimensionName = "";
            this._dllSortMeasure = null;
            this._axis = "";
            this._isSorted = false;
            this._isFiltered = false;
            this._sortOrFilterTab = '';
            this._currentAxis = "";
            this._parentElwidth = 0;
            this.pNode = "";
            this.progressPos = null;
            this._selectedReport = "";
            draggedSplitBtn = null;
            isDragging = false;
            oclientWaitingPopup = null;
            currentTab = this.defaultView();
        },

        _destroy: function () {
            this.element.empty().removeClass("e-olapclient" + this.model.cssClass);
            if (this.oclientWaitingPopup != undefined)
                this.oclientWaitingPopup._destroy();
        },

        _load: function () {
            this.element.addClass(this.model.cssClass);
            $("#" + this._id).ejWaitingPopup({ showOnInit: true });
            oclientWaitingPopup = $("#" + this._id).data("ejWaitingPopup");

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "initializeClient", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({
                "action": "initializeClient", "customObject": serializedCustomObject, "clientParams": this.model.enableMeasureGroups + "-" + this.model.chartType
            }), this._renderControlSuccess);
        },

        _setFirst: true,

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "title": this.element.find(".titleText").text(this.title()); break;
                    case "currentCubeName": this.currentCubeName = options[key]; break;
                    case "gridLayout": this._renderPivotGrid(); break;
                    case "olapReport": this.currentReport = options[key]; break;
                    case "clientReports": this.reports = options[key]; break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "locale": { this.locale(); if (this.ogridObj || this.ochartObj) this._load(); break; }
                    case "displaySettings": { if (this.ogridObj || this.ochartObj) this._load(); break; }
                }
            }
        },

        _getMeasuresList: function () {
            var measureName = "";
            this.element.find(".memberEditorDiv").find("div").each(function () { measureName += $(this)[0].id + ","; });
            return measureName;
        },

        _getUnSelectedNodes: function () {
            var treeElement = this.element.find(".editorTreeView")[0];
            var unselectedNodes = "";
            var data = $(treeElement).find(":input.nodecheckbox:not(:checked)");
            for (var i = 0; i < data.length; i++) {
                if ($(data[i].parentElement).attr('aria-checked') != 'mixed') {
                    var parentNode = $(data[i]).parents('li:eq(0)');
                    unselectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                }
            }
            return unselectedNodes;
        },

        _getSelectedNodes: function (isSlicer) {
            if (isSlicer) {
                var treeElement = this.element.find(".editorTreeView")[0].childNodes[0];
                var selectedNodes = new Array();
                var data = $(treeElement).children();
                for (var i = 0; i < data.length; i++) {
                    var parentNode = data[i];
                    var nodeInfo = { caption: $(parentNode.firstChild).find("a").text(), parentId: parentNode.parentElement.parentElement.className.indexOf("editorTreeView") > -1 ? "None" : $(parentNode).parents()[1].id, id: parentNode.id, checked: ($(parentNode).find(':input.nodecheckbox')[0].checked || $(parentNode).find('span:nth-child(1)').attr('class').indexOf("e-chk-indeter") > -1), expanded: $(parentNode.firstChild).find(".e-minus").length > 0 ? true : false, childNodes: new Array(), tag: $(parentNode).attr('tag') };
                    if ($(parentNode).find("ul:first").children().length > 0) {
                        var liElements = $(parentNode).find("ul:first").children();
                        for (var j = 0; j < liElements.length; j++) {
                            nodeInfo.childNodes.push(this._getNodeInfo(liElements[j]));
                        }
                    }
                    selectedNodes.push(nodeInfo);
                }
                return JSON.stringify(selectedNodes);
            }
            else {
                var treeElement = $(".editorTreeView");
                var selectedNodes = "";
                var data = $(treeElement).find(':input.nodecheckbox');
                for (var i = 0; i < data.length; i++) {
                    if (data[i].checked || $(data[i].parentElement).attr('aria-checked') == 'mixed') {
                        var parentNode = $(data[i]).parents('li:eq(0)');
                        selectedNodes += "::" + parentNode[0].id + "||" + $(parentNode).attr('tag');
                    }
                }
                return selectedNodes;
            }
        },

        _getNodeInfo: function (node) {
            var childNode = { caption: $(node.firstChild).find("a").text(), parentId: node.parentElement.parentElement.className.indexOf("editorTreeView") > -1 ? "None" : $(node).parents()[1].id, id: node.id, checked: ($(node).find(':input.nodecheckbox')[0].checked || $(node).find('span:nth-child(1)').attr('class').indexOf("e-chk-indeter") > -1), expanded: $(node.firstChild).find(".e-minus").length > 0 ? true : false, childNodes: new Array(), tag: $(node).attr('tag') };
            if ($(node).find("ul:first").children().length > 0) {
                var liElements = $(node).find("ul:first").children();
                for (var i = 0; i < liElements.length; i++) {
                    childNode.childNodes.push(this._getNodeInfo(liElements[i]));
                }
            }
            return childNode;
        },

        _removeSplitBtn: function () {

            var dragBtn = $(document).find(".dragClone"); var btnTag;
            var clientBtns = this.element.find(".splitBtn");
            jQuery.each(clientBtns, function (index, value) {
                if ($($(value).children()[0]).text() == $(dragBtn).text()) {
                    btnTag = $(value).attr("tag");
                    $(value).remove();
                }
            });
            $(".dragClone").remove();
            oclientWaitingPopup.show();
            if (oclientProxy.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "removeSplitButton", element: this.element, customObject: oclientProxy.model.customObject });
            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.removeSplitButton, JSON.stringify({ "action": "removeSplitButton", "clientParams": btnTag, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._removeSplitButtonSuccess)
        },

        _wireEvents: function () {
            oclientProxy._wireDialogEvent();
            oclientProxy._wireEditorRemoveEvent();
            oclientProxy._wireMeasureRemoveEvent();

            this._on($(document), 'keydown', this._KeyPress);

            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejDraggable({
                handle: 'button',
                clone: true,
                cursorAt: { top: -10, left: -10 },
                dragStart: function (event, ui) {
                    event.event.preventDefault();
                    $(this.element.find(".e-txt")).unbind('touchstart');
                    $(this.element.find(".e-txt")).unbind(ej.eventType.click);
                    isDragging = true;
                    this.element.find(".e-dialog").hide();
                    draggedSplitBtn = event.event.target;
                },
                dragStop: function (event, ui) {
                    this.element.find(".targetAxis").removeClass("targetAxis");
                    isDragging = false;
                    var cssName = null; var droppedPosition;
                    if (event.event.type == "touchend")
                        droppedPosition = oclientProxy.setSplitBtnTargetPos(event);
                    else
                        droppedPosition = oclientProxy.setSplitBtnTargetPos(event.event);
                    if (oclientProxy._dropAxisClassName != undefined && oclientProxy._dropAxisClassName != "") {
                        if (oclientProxy._dropAxisClassName == "outOfAxis")
                            oclientProxy._removeSplitBtn();
                        else
                            cssName = oclientProxy._dropAxisClassName;
                        oclientProxy._dropAxisClassName = "";
                    }
                    else if (event.target != undefined) {
                        if ($(event.target).hasClass("e-btn") || $(event.target).hasClass("removeSplitBtn"))
                            cssName = $(event.target).parents("div:eq(1)").attr("class");
                        else if (jQuery.type(event.target.className) != "string")
                            oclientProxy._removeSplitBtn();
                        else {
                            if (event.target.className.indexOf("splitBtn") > -1)
                                cssName = $(event.target).parents("div:eq(0)").attr("class");
                            else
                                cssName = event.target.className;
                        }
                    }
                    else {
                        if ($(event.event.originalEvent.srcElement).hasClass("e-btn") || $(event.event.originalEvent.srcElement).hasClass("removeSplitBtn"))
                            cssName = $(event.event.originalEvent.srcElement).parents("div:eq(1)").attr("class");
                        else if (event.event.originalEvent.srcElement.className.indexOf("splitBtn") > -1)
                            cssName = $(event.event.originalEvent.srcElement).parents("div:eq(0)").attr("class");
                        else
                            cssName = event.event.originalEvent.srcElement.className;
                    }
                    if (cssName != undefined && cssName != null) {
                        var axisName = (cssName.indexOf("categoricalAxis") > cssName.indexOf("rowAxis")) ? ((cssName.indexOf("categoricalAxis") > cssName.indexOf("slicerAxis")) ? "Categorical" : "Slicer") : ((cssName.indexOf("rowAxis") > cssName.indexOf("slicerAxis")) ? "Series" : (cssName.indexOf("slicerAxis") >= 0) ? "Slicer" : null);
                        if (axisName != null)
                            oclientProxy._splitButtonDropped(axisName, droppedPosition);
                        else
                            oclientProxy._removeSplitBtn();
                        oclientProxy._setSplitBtnTitle();
                    }
                },
                helper: function (event, ui) {
                    if (event.sender.target.className.indexOf("e-btn") > -1)
                        return $(event.sender.target).clone().addClass("dragClone").appendTo(oclientProxy.element);
                    else
                        return false;
                }
            });

            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").mouseover(function () {
                if (isDragging)
                    $(this).addClass("targetAxis");
            });

            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").mouseleave(function () {
                $(this).removeClass("targetAxis");
            });

            this.element.find(".e-btn").mouseover(function (evt) {
                $(evt.target.parentNode).find("span").css("display", "inline");
            });

            this.element.find(".splitBtn").mouseover(function (evt) {
                if (isDragging)
                    $(this).addClass("dropIndicator");
            });

            this.element.find(".splitBtn").mouseleave(function (evt) {
                $(this).removeClass("dropIndicator");
            });

            this.element.find(".sortDisable, .sortEnable, .filterDisable, .filterEnable ").bind(ej.eventType.click, function (evt) {
                if (evt.target.className == "sortDisable") {
                    $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').attr("disabled", "disabled");
                    $('.measureListLbl, .orderLbl, .radioBtnAscLbl, .radioBtnDescLbl, .preserveHrchyLbl').addClass('sortFilterDisable');
                    oclientProxy._dllSortMeasure.disable();
                }
                else if (evt.target.className == "sortEnable") {
                    $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').removeAttr("disabled");
                    $('.measureListLbl, .orderLbl, .radioBtnAscLbl, .radioBtnDescLbl, .preserveHrchyLbl').removeClass('sortFilterDisable');
                    oclientProxy._dllSortMeasure.enable();
                }
                else if (evt.target.className == "filterDisable") {
                    $('.filterFrom, .filterTo').attr("disabled", "disabled");
                    $('.filterMeasureListLbl, .conditionLbl, .filterValueLbl, .filterBtw').addClass('sortFilterDisable');
                    oclientProxy._dllFilterCondition.disable();
                    oclientProxy._dllfMeasuresList.disable();
                }
                else if (evt.target.className == "filterEnable") {
                    $('.filterFrom').removeAttr("disabled");
                    $('.filterMeasureListLbl, .conditionLbl, .filterValueLbl, .filterBtw').removeClass('sortFilterDisable');
                    oclientProxy._dllFilterCondition.enable();
                    oclientProxy._dllfMeasuresList.enable();
                }
            });

            this.element.find(".filterFrom , .filterTo").keypress(function (event) {
                if (event.which == 8 || event.which == 0) {
                    return true;
                }

                if (event.which < 46 || event.which > 58) {
                    return false;
                }
                if ((event.which == 46 && $(this).val().indexOf('.') != -1) || event.which == 47) {
                    return false;
                }
            });

            this.element.find(".toggleExpandButton").click(function (evt) {
                if (oclientProxy.model.isResponsive)
                    oclientProxy._rwdToggleCollapse();
                else {
                    if (oclientProxy._initToggle) {
                        oclientProxy._initToggle = false; var items = {};
                        var tempWidth = oclientProxy.element.find(".cubeTable").width() + oclientProxy.element.find(".controlPanel").width() - 17;
                        items["controlPanelWidth"] = (tempWidth - 3);
                        items["chartOuterWidth"] = items["chartModelWidth"] = items["gridOuterWidth"] = tempWidth - 17;
                        oclientProxy._toggleStyles.push(items);
                    }
                    if (oclientProxy.displayMode() == ej.olap.OlapClient.DisplayMode.GridOnly)
                        oclientProxy.element.find(".gridContainer").width(960);
                    else if (oclientProxy.displayMode() == ej.olap.OlapClient.DisplayMode.ChartOnly)
                        oclientProxy.element.find(".chartContainer").width(954);
                    else if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile) {
                        if (oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Grid) {
                            oclientProxy.element.find(".gridContainer").width(962);
                            oclientProxy.element.find(".chartContainer").width(957);
                        }
                        else {
                            oclientProxy.element.find(".gridContainer").width(960);
                            oclientProxy.element.find(".chartContainer").width(955);
                        }
                    }
                    else if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab) {
                        if (oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Grid) {
                            //oclientProxy.element.find(".gridContainer").width(962);
                            oclientProxy.element.find(".chartContainer").width(953);
                        }
                        else {
                            oclientProxy.element.find(".gridContainer").width(960);
                            oclientProxy.element.find(".chartContainer").width(953);
                        }
                    }
                    oclientProxy._performToggleAction(oclientProxy._toggleStyles);
                    oclientProxy.element.find(".csHeader, .cubeTable").hide();
                    oclientProxy.element.find(".toggleCollapseButton").show();
                    oclientProxy.element.find(".toggleText").show();
                }

            });

            this.element.find(".toggleCollapseButton").click(function (evt) {
                if (oclientProxy.model.isResponsive)
                    oclientProxy._rwdToggleExpand();
                else {
                    oclientProxy._performToggleAction(oclientProxy._initStyles);
                    oclientProxy.element.find(".csHeader, .cubeTable").show();
                    oclientProxy.element.find(".toggleCollapseButton").hide();
                    oclientProxy.element.find(".toggleText").hide();
                    if (oclientProxy.displayMode() == ej.olap.OlapClient.DisplayMode.GridOnly)
                        oclientProxy.element.find(".gridContainer").width(571);
                    else if (oclientProxy.displayMode() == ej.olap.OlapClient.DisplayMode.ChartOnly)
                        oclientProxy.element.find(".chartContainer").width(565);
                    else if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile) {
                        if (oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Grid) {
                            oclientProxy.element.find(".gridContainer").width(574);
                            oclientProxy.element.find(".chartContainer").width(569);
                        }
                        else {
                            oclientProxy.element.find(".gridContainer").width(571);
                            oclientProxy.element.find(".chartContainer").width(566);
                        }
                    }
                    else if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab) {
                        if (oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Grid) {
                            //oclientProxy.element.find(".gridContainer").width(574);
                            oclientProxy.element.find(".chartContainer").width(564);
                        }
                        else {
                            oclientProxy.element.find(".gridContainer").width(569);
                            oclientProxy.element.find(".chartContainer").width(564);
                        }
                    }
                }
            });

            this.element.find(".maximizedView").click(function (evt) {
                oclientProxy._maxViewBtnClick();
            });

            $(document).on('click', '.winCloseBtn', function () {
                oclientProxy._maxViewClsBtnClick();
            });

            if (this.model.isResponsive)
                $(window).bind('resize', $.proxy(this._reSizeHandler, this));
        },

        _KeyPress: function (e) {
            if (e.keyCode == 9) {
                if ($(".e-dialog").length > 0) {
                    $(".clientDialog").focus();
                    $(".reportName").addClass("e-focus");
                }
            }
            if ($(".e-dialog").length > 0 && e.keyCode == 13) {
                this.element.find(".dialogOKBtn").trigger("click");
                oclientProxy._off(this.element, "click", ".dialogOKBtn");
            }
            else if ($(".e-dialog").length > 0 && e.keyCode == 27) {
                this.element.find(".dialogCancelBtn").trigger("click");
                oclientProxy._off(this.element, "click", ".dialogCancelBtn");
            }
            else if ($(".e-dialog").length == 0 && e.keyCode == 27 && $(".fullScreenView").length != 0) {
                this._maxViewClsBtnClick();
            }
            else if (e.keyCode == 9 && $(".borderFilterSortDlg").length > 0) {

            }
            else if (e.keyCode == 13) {
                if (document.activeElement.className.indexOf("newReportImg") > -1) {
                    this.element.find(".newReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("addReportImg") > -1) {
                    this.element.find(".addReportImg ").trigger("click").off("click");

                }
                else if (document.activeElement.className.indexOf("removeReportImg") > -1) {
                    this.element.find(".removeReportImg ").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("renameReportImg") > -1) {
                    this.element.find(".renameReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("saveReportImg") > -1) {
                    this.element.find(".saveReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("loadReportImg") > -1) {
                    this.element.find(".loadReportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("mdxImg") > -1) {
                    this.element.find(".mdxImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("autoExecuteImg") > -1) {
                    this.element.find(".autoExecuteImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("excelExportImg") > -1) {
                    this.element.find(".excelExportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("wordExportImg") > -1) {
                    this.element.find(".wordExportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("pdfExportImg") > -1) {
                    this.element.find(".pdfExportImg").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("maximizedView") > -1) {
                    this.element.find(".maximizedView").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("colSortFilterImg") > -1) {
                    this.element.find(".colSortFilterImg ").trigger("click").off("click");
                }
                else if (document.activeElement.className.indexOf("rowSortFilterImg") > -1) {
                    this.element.find(".rowSortFilterImg ").trigger("click").off("click");
                }
            }
        },

        _wireMeasureRemoveEvent: function () {

            this.element.find(".removeMeasure").click(function (evt) {
                $($(evt.target).parent()).remove();
            });
        },

        _wireEditorRemoveEvent: function () {

            this.element.find(".removeSplitBtn").click(function (evt) {
                oclientWaitingPopup.show();
                $($(evt.target).parent()).remove();
                oclientProxy._off(this.element, "click", ".removeSplitBtn");
                if (oclientProxy.model.beforeServiceInvoke != null)
                    oclientProxy._trigger("beforeServiceInvoke", { action: "removeSplitButton", element: this.element, customObject: oclientProxy.model.customObject });
                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.removeSplitButton, JSON.stringify({ "action": "removeSplitButton", "clientParams": $(evt.target).parent().attr("tag"), "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._removeSplitButtonSuccess);
            });
        },

        _wireDialogEvent: function () {
            oclientProxy.element.find(".newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .saveReportImg, .loadReportImg, .mdxImg, .e-txt,.colSortFilterImg, .rowSortFilterImg, .autoExecuteImg").bind(ej.eventType.click, function (evt) {
                if (evt.currentTarget.parentElement.className.indexOf("splitBtn") > -1) {
                    if ($($(evt.currentTarget).parents(".splitBtn")[0]).attr('tag').indexOf("NAMEDSET") > -1) return false;
                    oclientProxy._currentAxis = $($(evt.currentTarget).parents(".splitBtn")[0]).attr('tag').split(':')[0];
                }
                if (!isDragging) {
                    evt.preventDefault();
                    if (evt.target.innerHTML != oclientProxy._getLocalizedLabels("Cancel") && evt.target.innerHTML != oclientProxy._getLocalizedLabels("OK")) {
                        oclientProxy._off(oclientProxy.element, "click", ".newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .saveReportImg, .loadReportImg, .mdxImg, .e-txt,.colSortFilterImg, .rowSortFilterImg, .chartTypesImg, .autoExecuteImg");
                        if (evt.target.className.indexOf("loadReportImg") >= 0) {
                            oclientWaitingPopup.show();
                            if (oclientProxy.model.beforeServiceInvoke != null)
                                oclientProxy._trigger("beforeServiceInvoke", { action: "FetchingReportList", element: oclientProxy.element, customObject: oclientProxy.model.customObject });
                            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.fetchReportList, JSON.stringify({ "customObject": serializedCustomObject }), oclientProxy._fetchReportListSuccess);
                        }
                        else if (evt.target.className.indexOf("colSortFilterImg") >= 0 || evt.target.className.indexOf("rowSortFilterImg") >= 0) {
                            if (evt.target.className.indexOf("SortImg") >= 0)
                                oclientProxy._sortOrFilterTab = "sort";
                            else
                                oclientProxy._sortOrFilterTab = "filter";
                            oclientWaitingPopup.show();
                            oclientProxy._isSorted = false;
                            oclientProxy._isFiltered = false;
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "FetchSortState", "toolbarOperation": null, "clientInfo": (oclientProxy._axis = (evt.target.className.indexOf("colSortFilterImg") >= 0) ? "Column" : "Row"), "olapReport": oclientProxy.currentReport, "clientReports": "" }), oclientProxy._fetchSortState);
                        }
                        else if (evt.target.className.indexOf("mdx") >= 0) {
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.mdxQuery, JSON.stringify({ "olapReport": oclientProxy.currentReport }), oclientProxy._mdxQuery);
                        }
                        else if (evt.target.className.indexOf("autoExecute") >= 0) {
                            oclientProxy.model.enableDeferUpdate = false;
                            oclientProxy._renderControls();
                            oclientProxy.model.enableDeferUpdate = true;
                        }
                        else
                            oclientProxy._createDialogRequest(evt);
                        oclientProxy._dimensionName = $(evt.currentTarget).parent().attr("tag");
                    }
                }
            });

            this.element.find(".excelExportImg, .wordExportImg, .pdfExportImg").click(function (e) {
                if (oclientProxy.model.displaySettings.mode != oclientProxy.model.clientExportMode && oclientProxy.model.displaySettings.mode != ej.olap.OlapClient.DisplayMode.ChartAndGrid && oclientProxy.model.clientExportMode != ej.olap.OlapClient.ClientExportMode.ChartAndGrid) return;
                var exportOption = e.target.className.indexOf("excel") >= 0 ? "Excel" : e.target.className.indexOf("word") >= 0 ? "Word" : "Pdf";
                var params, chartString, legendString, mode;
                if (oclientProxy.model.displaySettings.mode != ej.olap.OlapClient.DisplayMode.GridOnly && oclientProxy.model.clientExportMode != ej.olap.OlapClient.ClientExportMode.GridOnly) {
                    var canvasElement = document.createElement('canvas');
                    var chartsvgElement = $("#" + oclientProxy.ochartObj._id + "Container_svg").clone()[0];
                    if (chartsvgElement != undefined && chartsvgElement != null) {
                        var chartsvg = ej.buildTag("div").append(chartsvgElement).html();
                        canvg(canvasElement, chartsvg);
                        chartString = canvasElement.toDataURL("image/png");
                        var legendsvgElement = $("#legend_" + oclientProxy.ochartObj._id + "Container_svg").clone()[0];
                        var legendsvg = ej.buildTag("div").append(legendsvgElement).html();
                        canvg(canvasElement, legendsvg);
                        legendString = canvasElement.toDataURL("image/png");
                    }
                    else {
                        if (oclientProxy.model.displaySettings.mode == ej.olap.OlapClient.DisplayMode.ChartOnly || oclientProxy.model.clientExportMode == ej.olap.OlapClient.ClientExportMode.ChartOnly) return;
                        oclientProxy.model.clientExportMode = ej.olap.OlapClient.ClientExportMode.GridOnly;
                    }
                }
                if (oclientProxy.model.displaySettings.mode != ej.olap.OlapClient.DisplayMode.ChartOnly && oclientProxy.model.clientExportMode != ej.olap.OlapClient.ClientExportMode.ChartOnly) {
                    var colorDetails = {
                        valueCellColor: oclientProxy.ogridObj.element.find(".value").css("color"),
                        valueCellBGColor: oclientProxy.ogridObj.element.find(".value").css("background-color"),
                        summaryCellColor: oclientProxy.ogridObj.element.find(".summary").css("color"),
                        summaryCellBGColor: oclientProxy.ogridObj.element.find(".summary").css("background-color")
                    }
                }
                if (oclientProxy.model.displaySettings.mode == ej.olap.OlapClient.DisplayMode.GridOnly || oclientProxy.model.clientExportMode == ej.olap.OlapClient.ClientExportMode.GridOnly) {
                    mode = ej.olap.OlapClient.ClientExportMode.GridOnly;
                    params = {
                        args: JSON.stringify({
                            exportMode: mode,
                            exportOption: exportOption,
                            currentReport: oclientProxy.currentReport,
                            layout: oclientProxy.gridLayout(),
                            colorSettings: JSON.stringify(colorDetails)
                        })
                    };
                }
                else if (oclientProxy.model.displaySettings.mode == ej.olap.OlapClient.DisplayMode.ChartOnly || oclientProxy.model.clientExportMode == ej.olap.OlapClient.ClientExportMode.ChartOnly) {
                    mode = ej.olap.OlapClient.ClientExportMode.ChartOnly;
                    params = {
                        args: JSON.stringify({
                            exportMode: mode,
                            exportOption: exportOption,
                            currentReport: oclientProxy.currentReport,
                            chartdata: chartString.split(',')[1],
                            legenddata: legendString.split(',')[1],
                            bgColor: $(oclientProxy.ochartObj.element).css('background-color')
                        })
                    };
                }
                else {
                    mode = ej.olap.OlapClient.ClientExportMode.ChartAndGrid;
                    params = {
                        args: JSON.stringify({
                            exportMode: mode,
                            exportOption: exportOption,
                            currentReport: oclientProxy.currentReport,
                            layout: oclientProxy.gridLayout(),
                            colorSettings: JSON.stringify(colorDetails),
                            chartdata: chartString.split(',')[1],
                            legenddata: legendString.split(',')[1],
                            bgColor: $(oclientProxy.ochartObj.element).css('background-color')
                        })
                    };
                }
                oclientProxy.doPostBack(_proxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.exportOlapClient, params);
            });
	       this.element.find(".toggleaxisImg").click(function (evt) {
	         oclientProxy.doAjaxPost("POST",oclientProxy.model.url + "/"+ oclientProxy.model.serviceMethodSettings.toggleAxis,JSON.stringify({"action": "toggleAxis","currentReport":oclientProxy.currentReport, "clientReports": oclientProxy.reports}),oclientProxy._toggleAxisSuccess);
            });

            this.element.find(".chartTypesImg").click(function (evt) {
                var chartTypesDlg = ej.buildTag("div.chartTypesDialog#chartTypesDialog", ej.buildTag("table", ej.buildTag("tbody", ej.buildTag("tr",
                     ej.buildTag("td", ej.buildTag("div.line chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Line"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.spline chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Spline"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.column chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Column"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.area chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Area"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.splinearea chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("SplineArea"), tabindex: 0 })[0].outerHTML)[0].outerHTML)[0].outerHTML +
                    ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.stepline chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StepLine"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.steparea chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StepArea"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.pie chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Pie"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.bar chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Bar"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.stackingarea chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StackingArea"), tabindex: 0 })[0].outerHTML)[0].outerHTML)[0].outerHTML +
                    ej.buildTag("tr", ej.buildTag("td", ej.buildTag("div.stackingcolumn chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StackingColumn"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.stackingbar chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("StackingBar"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                      ej.buildTag("td", ej.buildTag("div.funnel chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Funnel"), tabindex: 0 })[0].outerHTML)[0].outerHTML +
                    ej.buildTag("td", ej.buildTag("div.pyramid chartTypesIcon").attr({ "title": oclientProxy._getLocalizedLabels("Pyramid"), tabindex: 0 })[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML);

                var ele = oclientProxy.element.find('div.chartTypesDialog');
                if (ele.length == 0) {
                    $(chartTypesDlg).appendTo(oclientProxy.element);
                    $(chartTypesDlg).css("left", evt.originalEvent.clientX + 10 + "px").css("top", evt.originalEvent.clientY + 10 + "px");

                }
                $(".chartTypesIcon").click(function (e) {

                    $(chartTypesDlg).remove();
                    var selectedtype = e.target.className.split(" ")[0];
                    oclientProxy.model.chartType = selectedtype;
                    oclientProxy.ochartObj.model.type = selectedtype;
                    oclientProxy.ochartObj.model.commonSeriesOptions.type = selectedtype;
                    if (selectedtype == "funnel") {
                        oclientProxy.ochartObj.model.commonSeriesOptions.marker = {
                            dataLabel: {
                                visible: true,
                                shape: 'none',
                                font: { color: oclientProxy.element.css("color"), size: '12px', fontWeight: 'lighter' }
                            }
                        }
                    }
                    oclientProxy.ochartObj.renderControlSuccess({ "JsonRecords": JSON.stringify(oclientProxy.ochartObj.getJsonRecords()), "OlapReport": oclientProxy.ochartObj.getOlapReport() });
                    var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                    oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.updateReport, JSON.stringify({
                        "action": "chartTypeChanged", "clientParams": selectedtype, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject
                    }), oclientProxy._chartTypeChangedSuccess);
                });
            });

            $(document).click(function (e) {
                if (e.target.className.indexOf('chartTypesImg') == -1)
                    $(".chartTypesDialog").remove();
            })

            this.element.find(".dialogCancelBtn").bind(ej.eventType.click, function (e) {
                e.preventDefault();
                oclientProxy.element.find(".e-dialog").hide();
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            });

            this.element.find(".dialogOKBtn").bind(ej.eventType.click, function (e) {
                e.preventDefault();
                var loadDlgTitle = oclientProxy.element.find(".e-titlebar")[0].textContent == undefined ? oclientProxy.element.find(".e-titlebar")[0].innerText : oclientProxy.element.find(".e-titlebar")[0].textContent;
                if (loadDlgTitle == oclientProxy._getLocalizedLabels("LoadReport")) {
                    var selectedReport = oclientProxy.element.find(".reportNameList")[0].value;
                    if (selectedReport != "") {
                        oclientWaitingPopup.show();
                        if (oclientProxy.model.beforeServiceInvoke != null)
                            oclientProxy._trigger("beforeServiceInvoke", { action: "loadReport", element: this.element, customObject: oclientProxy.model.customObject });
                        var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.loadReport, JSON.stringify({
                            "reportName": selectedReport, "olapReport": currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject
                        }), oclientProxy._toolbarOperationSuccess);
                    }
                    else {
                        alert("Please select a valid report");
                        return false;
                    }
                }

                else if (oclientProxy.element.find(".sortingDlg").length > 0) {
                    if (oclientProxy.element.find(".sortEnable")[0].checked == true && oclientProxy.element.find(".measuresList")[0].value == "") {
                        alert("Please select a valid measure.");
                        return false;
                    }
                    else if (oclientProxy.element.find(".filterEnable")[0].checked == true) {
                        if (oclientProxy._axis == "Row" && $(".rowAxis ")[0].childElementCount == 0) {
                            alert("Element not found in row axis. Please add an element in row axis for filtering."); return false;
                        }
                        if (oclientProxy._axis == "Column" && $(".categoricalAxis")[0].childElementCount == 0) {
                            alert("Element not found in column axis. Please add an element in column axis for filtering."); return false;
                        }
                        if (oclientProxy.element.find(".fMeasuresList")[0].value == "") {
                            alert("Please select a valid measure."); return false;
                        }
                        else if (oclientProxy.element.find(".filterCondition")[0].value == "") {
                            alert("Please set a valid condition."); return false;
                        }
                        else if (oclientProxy.element.find(".filterFrom")[0].value == "") {
                            alert("Please set a start value."); return false;
                        }
                        else if ((oclientProxy.element.find(".filterCondition")[0].value == "Between" || oclientProxy.element.find(".filterCondition")[0].value == "NotBetween") && oclientProxy.element.find(".filterTo")[0].value == "") {
                            alert("Please set a end value."); return false;
                        }

                    }
                    var sortingDetails = null, filteringDetails = null;
                    if ((oclientProxy._isSorted == false || oclientProxy._isSorted == true) && oclientProxy.element.find(".sortEnable")[0].checked == true)
                        sortingDetails = $("li[tag*='" + oclientProxy.element.find(".measuresList")[0].value + "']").attr("tag") + "::" + ((oclientProxy.element.find(".radioBtnAsc")[0].checked == true) ? "ASC" : "DESC") + "::" + oclientProxy._axis + "::" + ((oclientProxy.element.find(".preserveHrchy")[0].checked == true) ? "PHT" : "PHF");
                    else if (oclientProxy._isSorted == true && oclientProxy.element.find(".sortDisable")[0].checked == true)
                        sortingDetails = "Disable Sorting" + "::" + " " + "::" + oclientProxy._axis + "::" + " ";
                    else
                        sortingDetails = " " + "::" + " " + "::" + oclientProxy._axis + "::" + " ";
                    if ((oclientProxy._isFiltered == false || oclientProxy._isFiltered == true) && oclientProxy.element.find(".filterEnable")[0].checked == true)
                        filteringDetails = $("li[tag*='" + oclientProxy.element.find(".fMeasuresList")[0].value + "']").attr("tag") + "::" + oclientProxy.element.find(".filterCondition")[0].value + "::" + oclientProxy.element.find(".filterFrom")[0].value + "::" + oclientProxy.element.find(".filterTo")[0].value;
                    else if (oclientProxy._isFiltered == true && oclientProxy.element.find(".filterDisable")[0].checked == true)
                        filteringDetails = "Disable Filtering";
                    else
                        filteringDetails = "";
                    oclientProxy._SortFilterDetails = sortingDetails + "||" + filteringDetails;

                    oclientWaitingPopup.show();
                    if (oclientProxy.element.find(".sortEnable")[0].checked == true || oclientProxy.element.find(".filterEnable")[0].checked == true) {
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "SortOrFilter", "clientInfo": oclientProxy._SortFilterDetails, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports }), oclientProxy._toolbarOperationSuccess);
                    }
                    else if (oclientProxy.element.find(".sortDisable")[0].checked == true || oclientProxy.element.find(".filterDisable")[0].checked == true) {
                        if (oclientProxy._isSorted == true || oclientProxy._isFiltered == true) {
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "SortOrFilter", "clientInfo": oclientProxy._SortFilterDetails, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports }), oclientProxy._toolbarOperationSuccess);
                            oclientProxy._isSorted = false;
                        }
                        else {
                            alert("Invalid operation !");
                            oclientWaitingPopup.hide();
                            return false;
                        }
                    }
                }
                else {
                    if (oclientProxy._getSelectedNodes() == 0 && args_innerHTML != oclientProxy._getLocalizedLabels("Measures") && args_innerHTML != "ToolbarButtons")
                        return false;
                    var unselectedNodes = null;
                    if (args_innerHTML != "ToolbarButtons") {
                        oclientWaitingPopup.show();
                        if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures"))
                            unselectedNodes = "Measures:" + oclientProxy._getMeasuresList();
                        else if (args_innerHTML != "ToolbarButtons" && args_innerHTML != undefined)
                            unselectedNodes = oclientProxy._getUnSelectedNodes() + "CHECKED" + oclientProxy._getSelectedNodes(oclientProxy._currentAxis == "Slicers" ? true : false);
                        if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures") && unselectedNodes != null && unselectedNodes.split(":")[1] == "") {
                            $(oclientProxy.element).find(".splitBtn").each(function (index, value) {
                                if (value.firstChild.innerHTML == oclientProxy._getLocalizedLabels("Measures"))
                                    $(value).remove();
                            });
                        }
                        $(oclientProxy).find(".e-dialog").hide();
                        if (oclientProxy.model.beforeServiceInvoke != null)
                            oclientProxy._trigger("beforeServiceInvoke", { "action": "filtering", element: this.element, customObject: oclientProxy.model.customObject });
                        var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                        oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.filterElement, JSON.stringify({ "action": "filtering", "clientParams": unselectedNodes, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._filterElementSuccess);
                    }
                    else {
                        if (oclientProxy.element.find(".e-titlebar").first().text() == oclientProxy._getLocalizedLabels("RemoveReport") && reportsCount < 2)
                            oclientProxy.element.find(".e-dialog").hide();
                        else if (oclientProxy.element.find(".reportName").val() == "") {
                            return false;
                        }
                        else {
                            var reportName = oclientProxy.element.find(".reportName").val() || reportDropTarget.selectedTextValue;
                            var operation = oclientProxy.element.find(".e-titlebar").first().attr("tag");
                            var olapReport = (operation == "New Report" ? "" : oclientProxy.currentReport);
                            var clientReports = (operation == "New Report" ? "" : oclientProxy.reports);
                            oclientProxy.element.find(".e-dialog").hide();
                            oclientWaitingPopup.show();
                            if (operation == "Save Report") {
                                if (oclientProxy.model.beforeServiceInvoke != null)
                                    oclientProxy._trigger("beforeServiceInvoke", { action: "saveReport", element: this.element, customObject: oclientProxy.model.customObject });
                                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.saveReport, JSON.stringify({
                                    "reportName": reportName, "olapReport": olapReport, "clientReports": clientReports, "customObject": serializedCustomObject
                                }), oclientProxy._toolbarOperationSuccess);

                            } else
                                if (oclientProxy.model.beforeServiceInvoke != null)
                                    oclientProxy._trigger("beforeServiceInvoke", { action: "toolbarOperation", element: this.element, customObject: oclientProxy.model.customObject });
                            var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                            oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.toolbarServices, JSON.stringify({
                                "action": "toolbarOperation", "toolbarOperation": operation, "clientInfo": reportName, "olapReport": olapReport, "clientReports": clientReports, "customObject": serializedCustomObject
                            }), oclientProxy._toolbarOperationSuccess);
                        }
                    }
                }
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            });
            this.element.find(".unCheckAll,.checkAll").click(function (evt) {
                if (evt.target.className.indexOf("checkAll") > -1)
                    oclientProxy.element.find(".editorTreeView").ejTreeView("checkAll");
                else
                    oclientProxy.element.find(".editorTreeView").ejTreeView("unCheckAll");
            });
        },

        _getLocalizedLabels: function (property) {
            return ej.olap.OlapClient.locale[this.locale()][property] === undefined ? ej.olap.OlapClient.locale["en-US"][property] : ej.olap.OlapClient.locale[this.locale()][property];
        },

        _unWireEvents: function () {
            $(this.element.find(".dialogCancelBtn, .dialogOKBtn, .newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .e-txt, .removeSplitBtn, .unCheckAll, .checkAll, .removeMeasure, .toggleCollapseButton, .toggleExpandButton, .saveReportImg, .loadReportImg,.mdxImg,.maximizedView,.colSortFilterImg, .rowSortFilterImg, .chartTypesImg, .toggleaxisImg, .autoExecuteImg")).unbind(ej.eventType.click);
            $(this.element.find(".dialogCancelBtn, .dialogOKBtn, .newReportImg, .addReportImg, .removeReportImg, .renameReportImg, .e-txt, .removeSplitBtn, .unCheckAll, .checkAll, .removeMeasure, .toggleCollapseButton, .toggleExpandButton, .saveReportImg, .loadReportImg,.mdxImg,.maximizedView,.colSortFilterImg, .rowSortFilterImg, .chartTypesImg, .toggleaxisImg,.autoExecuteImg")).unbind('click');
            $(document).find(".winCloseBtn").unbind(ej.eventType.click);
            $(document).find(".winCloseBtn").unbind('click');
            this._off($(document), 'keydown', this._KeyPress);
            if (this.model.isResponsive)
                $(window).unbind('resize', $.proxy(this._reSizeHandler, this));
        },

        _performToggleAction: function (styles) {
            this.element.find(".controlPanel").width(styles[0].controlPanelWidth);
            if ((this.controlPlacement() != "horizontal") || (this.controlPlacement() == "horizontal" && this.displayMode() == "chartOnly") ||
                this.controlPlacement() == "horizontal" && this.displayMode() == "gridOnly") {
                this.element.find(".e-olapchart").width(styles[0].chartOuterWidth);
                this.element.find(".e-pivotgrid").width(styles[0].gridOuterWidth);
                if (this.ochartObj != null || this.ochartObj != undefined) {
                    this.element.find("#" + this.ochartObj._id).ejOlapChart("option", "width", styles[0].chartOuterWidth + "px");
                    this.chartObj = this.element.find("#" + this.ochartObj._id + "Container").data("ejChart");
                    if (this.chartObj) {
                        this.element.find("#" + this.chartObj._id).ejChart("option", { "model": { size: { width: styles[0].chartOuterWidth + "px" } } });
                        this.element.find("#" + this.chartObj._id + "_svg").width(styles[0].chartOuterWidth);
                        this.chartObj.redraw();
                    }
                }
            }
        },

        _rwdToggleExpand: function () {
            oclientProxy.element.find(".csHeader, .cubeTable").show();
            oclientProxy.element.find(".toggleCollapseButton").hide();
            oclientProxy.element.find(".toggleText").hide();
            if (oclientProxy._parentElwidth < 850) {
                this.element.find(".outerTable").css({ "position": "absolute", "float": "left", "z-index": "1000", "tableLayout": "auto" });
                this.element.find(".controlPanelTD").css({ "left": "35px", "position": "relative" });
            }

            if (oclientProxy._parentElwidth > 850) {
                this.element.find(".controlPanelTD").css({ "width": "56%", "left": "inherit" });
                this.element.find(".outerTable,.controlPanelTD").css({ "position": "inherit" });
                if (currentTab == "grid")
                    $(".controlPanel")[0].style.width = $("#" + oclientProxy._id).width() * 55 / 100 + "px";
                else
                    $(".controlPanel")[0].style.width = "100%";

                if (window.navigator.userAgent.indexOf('MSIE ') > 0) {
                    oclientProxy.element.find(".outerTable")[0].style.width = $("#" + oclientProxy._id).width() * 43 / 100 + "px";
                    oclientProxy.element.find(".csHeader")[0].style.width = ($("#" + oclientProxy._id).width() * 41 / 100) + "px";
                    this.element.find("#cubeSelector_wrapper")[0].style.width = ($("#" + oclientProxy._id).width() * 39 / 100 - 60) + "px";
                    if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                        oclientProxy.element.find(".gridPanel")[0].style.width = $("#" + oclientProxy._id).width() * 56 / 100 + "px";
                }
                else {
                    this.element.find(".outerTable").css({ "width": $("#" + oclientProxy._id).width() * 42 / 100 + "px", "tableLayout": "auto" });
                    this.element.find(".csHeader")[0].style.width = ($("#" + oclientProxy._id).width() * 42 / 100) + "px";
                    this.element.find("#cubeSelector_wrapper")[0].style.width = ($("#" + oclientProxy._id).width() * 42 / 100 - 60) + "px";
                    if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                        oclientProxy.element.find(".gridPanel")[0].style.width = $("#" + oclientProxy._id).width() * 57 / 100 + "px";
                }

            }
            if (this.displayMode() != "gridonly") {
                if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                    $("#" + oclientProxy.ochartObj._id).width($(".controlPanel").width() - 7);
                else
                    $("#" + oclientProxy.ochartObj._id).width($(".controlPanel").width());
                var chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                if (chartObj != undefined && chartObj != null)
                    chartObj.redraw();
            }
        },

        _rwdToggleCollapse: function () {
            if (oclientProxy._parentElwidth < 850) {
                this.element.find(".outerTable").css({ "position": "absolute", "float": "left", "z-index": "1000" });
                this.element.find(".controlPanelTD").css({ "left": "35px", "position": "relative" });
            }
            if (currentTab == "grid") {
                $(".controlPanel")[0].style.width = $("#" + oclientProxy._id).width() * 94 / 100 + "px";
            }
            else
                $(".controlPanel")[0].style.width = "100%";
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                oclientProxy.element.find(".gridPanel")[0].style.width = $("#" + oclientProxy._id).width() * 94 / 100 + "px";
            oclientProxy.element.find(".csHeader, .cubeTable").hide();
            this.element.find(".controlPanelTD").css({ "left": "4%", "position": "relative", "width": "95%", "tableLayout": "fixed" });
            $("#" + this._id).width("98%");
            this.element.find(".outerTable").css({ "position": "absolute", "width": "0px", "tableLayout": "fixed" });
            oclientProxy.element.find(".toggleCollapseButton").show();
            oclientProxy.element.find(".toggleText").show();
            if (oclientProxy.displayMode() != "gridonly") {
                if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                    $("#" + oclientProxy.ochartObj._id).width($(".controlPanel").width() - 17);
                else
                    $("#" + oclientProxy.ochartObj._id).width($(".controlPanel").width());
                var chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                if (chartObj != undefined && chartObj != null)
                    chartObj.redraw();
            }
        },

        _splitButtonDropped: function (axis, droppedPosition) {
            if (draggedSplitBtn != null) {
                var params = this.element.find(".cubeName").html() + "--" + $(draggedSplitBtn).parent("div:eq(0)").attr("Tag") + "--" + axis + "--" + droppedPosition;
                draggedSplitBtn = null;
                oclientWaitingPopup.show();
                if (oclientProxy.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.nodeDropped, JSON.stringify({ "action": "nodeDropped", "dropType": "SplitButton", "nodeInfo": params, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._nodeDroppedSuccess);
            }
        },

        _filterElementSuccess: function (report) {
            if (report[0] != undefined) {
                oclientProxy.currentReport = report[0].Value; oclientProxy.reports = report[1].Value;
                if (report[2] != null && report[2] != undefined)
                    oclientProxy.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                oclientProxy.currentReport = report.d[0].Value; oclientProxy.reports = report.d[1].Value;
                if (report.d[2] != null && report.d[2] != undefined)
                    oclientProxy.model.customObject = report.d[2].Value;
            }
            else {
                oclientProxy.currentReport = report.UpdatedReport; oclientProxy.reports = report.ClientReports;
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "filtering", element: this.element, customObject: this.model.customObject });
            oclientProxy._renderControls();
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            oclientProxy._successAction = "Filter";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        _maxViewBtnClick: function () {
            var winWidth = $(window).width() - 150;
            var winHeight = $(window).height() - 100;
            var docDivTag = ej.buildTag("div.fullScreenView", "", { width: $(document).width(), height: $(document).height() });
            var winDivTag = ej.buildTag("div#" + oclientProxy._id + "_maxView.maximumView", "", { width: winWidth, height: winHeight });
            var maxViewCls = ej.buildTag("div#Winclose.winCloseBtn e-icon", "");
            if (oclientProxy.displayMode() == "chartonly" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                oclientProxy._fullScreenView(100, 160);
                winDivTag.append(oclientProxy.element.find("#OlapChart"));
            }
            if (oclientProxy.displayMode() == "gridOnly")
                winDivTag.append(oclientProxy.element.find("#PivotGrid").css({ "max-height": winHeight, "width": winWidth }));
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && oclientProxy.displayMode() == "chartandgrid") {
                if (currentTab == "grid")
                    winDivTag.append(oclientProxy.element.find("#PivotGrid").css({ "max-height": winHeight, "width": winWidth }));
                else if (currentTab == "chart" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                    oclientProxy._fullScreenView(100, 160);
                    winDivTag.append(oclientProxy.element.find("#OlapChart"));
                }
            }
            else if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid") {
                var hght = ($(window).height() / 2) - 50;
                oclientProxy.element.find("#PivotGrid").css({ "max-height": hght, "margin-left": "0px" });
                oclientProxy._fullScreenView(50, 160);
                if (($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]))
                    if (oclientProxy.defaultView() == "chart")
                        winDivTag.append(oclientProxy.element.find("#OlapChart")).append(oclientProxy.element.find("#PivotGrid").width(($(window).width() - 160)));
                    else
                        winDivTag.append(oclientProxy.element.find("#PivotGrid").width($(window).width() - 160)).append(oclientProxy.element.find("#OlapChart"));
                winDivTag.append(oclientProxy.element.find("#PivotGrid").width($(window).width() - 160));
            }
            winDivTag.append(maxViewCls);
            docDivTag.append(winDivTag);
            $("body").append(docDivTag);
        },

        _maxViewClsBtnClick: function () {
            var fullScreen = $(".fullScreenView");
            fullScreen.find("#PivotGrid").css("margin-left", "7px");
            fullScreen.find("#OlapChart").appendTo(oclientProxy.element.find("#chartPanel"));
            if (oclientProxy.displayMode() == "chartonly" && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]))
                oclientProxy._fullScreenViewCls(560, 790);
            if (oclientProxy.displayMode() == "gridOnly")
                fullScreen.find("#PivotGrid").css({ "width": "inherit", "max-height": '' });
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && oclientProxy.displayMode() == "chartandgrid") {
                if (currentTab == "chart" && $("#" + oclientProxy.ochartObj._id + "Container_svg")[0])
                    oclientProxy._fullScreenViewCls(547, 557);
                else
                    fullScreen.find("#PivotGrid").css({ "max-height": "550px", "width": "555px" });
            }
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                if ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0]) {
                    oclientProxy._fullScreenViewCls(275, 555);
                    fullScreen.find("#PivotGrid").css({ "max-height": "300px", "width": "555px" });
                } else
                    fullScreen.find("#PivotGrid").css({ "max-height": "275px", "width": "555px" });
            if (oclientProxy.enableTogglePanel() && (oclientProxy.element.find(".cubeTable").is(':visible')) == false)
                fullScreen.find("#PivotGrid").css("width", "950px");
            fullScreen.find("#PivotGrid").appendTo(oclientProxy.element.find(".gridContainer"));
            $(".maximumView").remove();
            $(".fullScreenView").remove();
        },

        _enableResponsive: function () {
            $(".outerPanel")[0].style.width = "100%";
            $(".e-olapclient")[0].style.width = "100%";
            oclientProxy.element.find(".outerTable").css({ "float": "left" });
            oclientProxy.element.find(".cdbHeader, .cubeBrowser").addClass("responsive");
            oclientProxy.element.find(".rowAxis, .slicerAxis, .categoricalAxis").width("99%");
            oclientProxy.element.find(".controlPanelTD").width("56%");
            if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                oclientProxy.element.find(".controlPanelTD").css("display", "inline-block");
            $(".cubeTable")[0].style.width = "98%";
            if (window.navigator.userAgent.indexOf('MSIE ') > 0) {
                if (currentTab == "grid")
                    $(".controlPanel")[0].style.width = $("#" + oclientProxy._id).width() * 55 / 100 + "px";
                else
                    $(".controlPanel")[0].style.width = "99%";
                oclientProxy.element.find(".outerTable")[0].style.width = $("#" + oclientProxy._id).width() * 39 / 100 + "px";
                oclientProxy.element.find(".csHeader")[0].style.width = ($("#" + oclientProxy._id).width() * 38 / 100) + "px";
                if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                    oclientProxy.element.find(".gridPanel")[0].style.width = $("#" + oclientProxy._id).width() * 55 / 100 + "px";
            }
            else {
                oclientProxy.element.find(".outerTable")[0].style.width = $("#" + oclientProxy._id).width() * 38 / 100 + "px";
                oclientProxy.element.find(".csHeader")[0].style.width = ($("#" + oclientProxy._id).width() * 38 / 100) + "px";
            }
        },

        _fullScreenView: function (height, width) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined && ($("#" + oclientProxy.ochartObj._id + "Container_svg")[0])) {
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = $(window).width() - width + "px";
                if (oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && oclientProxy.displayMode() == "chartandgrid")
                    var chartHeight = oclientProxy.ochartObj.model.size.height = oclientProxy.chartObj.model.size.height = ($(window).height() / 2) - height + "px";
                else
                    var chartHeight = oclientProxy.ochartObj.model.size.height = oclientProxy.chartObj.model.size.height = $(window).height() - height + "px";
                oclientProxy.element.find("#OlapChart").css({ "min-height": chartHeight, "width": chartWidth });
                oclientProxy.chartObj.redraw();
            }
        },

        _fullScreenViewCls: function (height, width) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined) {
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
                var chartHeight = oclientProxy.chartObj.model.size.height = oclientProxy.ochartObj.model.size.height = height + "px";
                if (oclientProxy.enableTogglePanel() && (oclientProxy.element.find(".cubeTable").is(':visible')) == false)
                    var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = "950px";
                else
                    var chartWidth = oclientProxy.chartObj.model.size.width = oclientProxy.ochartObj.model.size.width = width + "px";
                oclientProxy.element.find("#OlapChart").css({ "min-height": chartHeight, "width": chartWidth });
                oclientProxy.chartObj.redraw();
            }
        },

        _nodeDroppedSuccess: function (data) {
            var columnElements, rowElements, slicerElements;
            if (data[0] != undefined) {
                columnElements = data[0].Value; rowElements = data[1].Value; slicerElements = data[2].Value;
                oclientProxy.currentReport = data[3].Value; oclientProxy.reports = data[4].Value;
                if (data[5] != null && data[5] != undefined)
                    oclientProxy.model.customObject = data[5].Value;
            }
            else if (data.d != undefined) {
                columnElements = data.d[0].Value; rowElements = data.d[1].Value; slicerElements = data.d[2].Value;
                oclientProxy.currentReport = data.d[3].Value; oclientProxy.reports = data.d[4].Value;
                if (data.d[5] != null && data.d[5] != undefined)
                    oclientProxy.model.customObject = data.d[5].Value;
            }
            else {
                columnElements = data.Columns; rowElements = data.Rows; slicerElements = data.Slicers;
                oclientProxy.currentReport = data.UpdatedReport; oclientProxy.reports = data.ClientReports;
                if (data.customObject != null && data.customObject != undefined)
                    oclientProxy.model.customObject = data.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
            this.element.find(".splitBtn, .e-btn").remove();
            $(oclientProxy._createSplitButtons(columnElements, "Columns")).appendTo(".categoricalAxis");
            $(oclientProxy._createSplitButtons(rowElements, "Rows")).appendTo(".rowAxis");
            $(oclientProxy._createSplitButtons(slicerElements, "Slicers")).appendTo(".slicerAxis");
            this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
            oclientProxy._setSplitBtnTitle();
            oclientProxy._renderControls();
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            oclientProxy._successAction = "NodeDrop";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        _editorTreeInfoSuccess: function (editorTree) {

            if (editorTree[0] != undefined) {
                if (editorTree[2] != null && editorTree[2] != undefined)
                    oclientProxy.model.customObject = editorTree[2].Value;
            }
            else if (editorTree.d != undefined) {
                if (editorTree.d[2] != null && editorTree.d[2] != undefined)
                    oclientProxy.model.customObject = editorTree.d[2].Value;
            }
            else {
                if (editorTree.customObject != null && editorTree.customObject != undefined)
                    oclientProxy.model.customObject = editorTree.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "fetchMemberTreeNodes", element: this.element, customObject: this.model.customObject });
            oclientProxy._createDialog(args_className, args_innerHTML, editorTree);
            oclientWaitingPopup.hide();
            oclientProxy._successAction = "EditorTreeInfo";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        _renderControlSuccess: function (msg) {
            if (oclientProxy.model.isResponsive)
                oclientProxy.model.displaySettings.enableTogglePanel = oclientProxy.model.isResponsive;
            eventArgs = { element: this.element, customObject: this.model.customObject };
            oclientProxy._trigger("load", eventArgs);
            var cubes, columnElements, rowElements, slicerElements, cubeTreeInfo, measureGroupInfo;
            if (msg[0] != undefined) {
                cubes = msg[0].Value; columnElements = msg[1].Value; rowElements = msg[2].Value;
                slicerElements = msg[3].Value; cubeTreeInfo = msg[4].Value; oclientProxy.currentReport = msg[5].Value;
                oclientProxy.reports = msg[6].Value; reportsCount = msg[7].Value; reportList = $.parseJSON(msg[8].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(msg[9].Value);
                oclientProxy.currentCubeName = msg[10].Value;
                if (msg[11] != null && msg[11] != undefined)
                    oclientProxy.model.customObject = msg[11].Value;
            }
            else if (msg.d != undefined) {
                cubes = msg.d[0].Value; columnElements = msg.d[1].Value; rowElements = msg.d[2].Value;
                slicerElements = msg.d[3].Value; cubeTreeInfo = msg.d[4].Value; oclientProxy.currentReport = msg.d[5].Value;
                oclientProxy.reports = msg.d[6].Value; reportsCount = msg.d[7].Value; reportList = $.parseJSON(msg.d[8].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(msg.d[9].Value);
                oclientProxy.currentCubeName = msg.d[10].Value;
                if (msg.d[11] != null && msg.d[11] != undefined)
                    oclientProxy.model.customObject = msg.d[11].Value;
            }
            else {
                cubes = msg.Cubes; columnElements = msg.Columns; rowElements = msg.Rows;
                slicerElements = msg.Slicers; cubeTreeInfo = msg.CubeTreeInfo; oclientProxy.currentReport = msg.CurrentReport;
                oclientProxy.reports = msg.ClientReports; reportList = $.parseJSON(msg.ReportList); reportsCount = msg.ReportsCount;
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(msg.MeasureGroups);
                oclientProxy.currentCubeName = msg.CurrentCubeName;
                if (msg.customObject != null && msg.customObject != undefined)
                    oclientProxy.model.customObject = msg.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "initialize", element: this.element, customObject: this.model.customObject });
            var treeViewData = $.parseJSON(cubeTreeInfo);
            var reportBar = ej.buildTag("div.reportToolbar#reportToolbar", oclientProxy._reportToolbar(), { width: "410px", height: "29px" })[0].outerHTML;
            var browserPanel, htmlTag;
            if (!oclientProxy.model.isResponsive) {
                browserPanel = oclientProxy._createCubeSelector() + "<table class=cubeTable><tr><td valign=\"bottom\">" + oclientProxy._createCubeBrowser(cubeTreeInfo) + "</td><td valign=\"bottom\" style='padding-left:5px;'>" + oclientProxy._createAxisElementBuilder(columnElements, rowElements, slicerElements) + "</td></tr></table>";
                htmlTag = ej.buildTag("span.outerPanel", "", { width: "1000px", height: "600px" }).append("<div class=\"titleText\"><span style='padding-left:10px'>" + oclientProxy.title() + "</span></div>" + reportBar + "<table class=\"outerTable\"><tr><td>" + browserPanel + "</td><td class=\"controlPanelTD\">" + oclientProxy._controlPanel() + "</td></tr></table>");
            }
            else if (oclientProxy.model.isResponsive) {
                browserPanel = oclientProxy._createCubeSelector() + ej.buildTag("table.cubeTable", ej.buildTag("tbody", ej.buildTag("tr", ej.buildTag("td.cubeTabletd", oclientProxy._createCubeBrowser(cubeTreeInfo)).attr({ valign: "bottom", width: "50%" })[0].outerHTML + ej.buildTag("td.cubeTabletd", oclientProxy._createAxisElementBuilder(columnElements, rowElements, slicerElements)).attr({ valign: "bottom", width: "50%" })[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML;

                htmlTag = ej.buildTag("span.outerPanel", "").append(ej.buildTag("div.titleText", ej.buildTag("span", oclientProxy.title(), { "padding-left": "10px" })[0].outerHTML)[0].outerHTML + reportBar + ej.buildTag("table.oClientTbl", ej.buildTag("tr", ej.buildTag("td", ej.buildTag("table.outerTable", ej.buildTag("tr", ej.buildTag("td", browserPanel)[0].outerHTML)[0].outerHTML)[0].outerHTML + ej.buildTag("table.controlPanelTD", ej.buildTag("tr", ej.buildTag("td", oclientProxy._controlPanel())[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML)[0].outerHTML, { width: "100%" })[0].outerHTML);
            }
            oclientProxy.element.html(htmlTag).width($(".outerPanel").width());
            oclientProxy.element.find(".reportToolbar").ejToolbar();
            if ((oclientProxy.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Grid) || (oclientProxy.displayMode() == ej.olap.OlapClient.DisplayMode.GridOnly))
                oclientProxy.element.find(".chartTypesImg").addClass("chartTypesOnGridView");
            var reportDropDown = "<div class ='reportList' ><input type='text' id='reportList' class='reportlist' /></div>";
            if (oclientProxy.model.enableMeasureGroups) {
                var measureGroupDropdown = "<div class ='measureGroupselector' style='margin:5px 5px 0px 5px'><input type='text' id='measureGroupSelector' class='measureGroupSelector' /></div>";
                $(measureGroupDropdown).prependTo(".cubeBrowser");
                oclientProxy.element.find(".measureGroupSelector").ejDropDownList({
                    dataSource: measureGroupInfo,
                    fields: { text: "name", value: "name" },
                    height: "25px",
                    width: "100%"
                });
                oclientProxy.element.find(".measureGroupSelector").attr("tabindex", 0);
            }
            $(reportDropDown).appendTo(".reportToolbar");
            oclientProxy.element.find(".reportlist").ejDropDownList({
                dataSource: reportList,
                fields: { text: "name", value: "name" },
                height: "26px"
            });
            oclientProxy.element.find(".reportlist").attr("tabindex", 0);
            var tempddlwidth = oclientProxy.element.find(".csHeader").width() - oclientProxy.element.find(".cubeText").width() - oclientProxy.element.find(".toggleExpandButton").width() - 20;
            var cubeddlWidth = oclientProxy.enableTogglePanel() ? tempddlwidth - 25 : tempddlwidth;
            oclientProxy.element.find(".cubeSelector").ejDropDownList({
                dataSource: $.parseJSON(cubes),
                fields: { text: "name", value: "name" },
                width: "" + cubeddlWidth + "px"
            });
            ddlTarget = oclientProxy.element.find('.cubeSelector').data("ejDropDownList");
            reportDropTarget = oclientProxy.element.find('#reportList').data("ejDropDownList");
            if (oclientProxy.model.enableMeasureGroups)
                measureDropTarget = oclientProxy.element.find('#measureGroupSelector').data("ejDropDownList");
            ddlTarget.selectItemByText(oclientProxy.currentCubeName);
            reportDropTarget.selectItemByText(reportDropTarget.model.dataSource[0].name);
            this._selectedReport = reportDropTarget.currentValue;
            if (oclientProxy.model.enableMeasureGroups)
                measureDropTarget.selectItemByText(measureDropTarget.model.dataSource[0].name);
            oclientProxy.element.find(".cubeSelector").ejDropDownList("option", "change", ej.proxy(oclientProxy.cubeChanged, oclientProxy));
            oclientProxy.element.find(".cubeSelector").attr("tabindex", 0);
            oclientProxy.element.find(".reportlist").ejDropDownList("option", "change", ej.proxy(oclientProxy.reportChanged, oclientProxy));
            if (oclientProxy.model.enableMeasureGroups)
                oclientProxy.element.find(".measureGroupSelector").ejDropDownList("option", "change", ej.proxy(oclientProxy.measureGroupChanged, oclientProxy));
            oclientProxy.element.find(".cubeTreeView").ejTreeView({
                fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                allowDragAndDrop: true,
                allowDropChild: false,
                allowDropSibling: false,
                dragAndDropAcrossControl: true,
                nodeDragStart: function () {
                    isDragging = true;
                },
                nodeDropped: ej.proxy(oclientProxy.nodeDropped, oclientProxy),
                height: "496px",
            });
            oclientProxy.element.find(".cubeTreeView").attr("tabindex", 0);
            var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            var cubeName = ej.buildTag("div.cubeName", oclientProxy.currentCubeName);
            $(cubeName).prependTo(".cubeBrowser");
            oclientProxy.element.find(".categoricalAxis, .rowAxis, .slicerAxis").ejDroppable({ drop: ej.proxy(oclientProxy.onDropped, oclientProxy) });
            oclientProxy.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
            if (oclientProxy.enableTogglePanel()) {
                $(ej.buildTag("div.toggleExpandButton e-icon")[0].outerHTML).appendTo(oclientProxy.element.find(".csHeader"));
                $(oclientProxy.element.find(".outerTable").find("td")[0]).append(ej.buildTag("div.toggleCollapseButton e-icon")[0].outerHTML);
                $((ej.buildTag("div.toggleText")[0].outerHTML)).insertAfter(oclientProxy.element.find(".toggleCollapseButton"));
                oclientProxy.element.find(".toggleCollapseButton").hide();
                oclientProxy.element.find(".toggleText").hide();
            }
            oclientProxy._setSplitBtnTitle();
            oclientProxy.element.find("#clientTab").ejTab({ itemActive: ej.proxy(oclientProxy.onTabClick, oclientProxy) });
            if (oclientProxy.defaultView() == ej.olap.OlapClient.DefaultView.Chart) {
                oclientProxy.element.find("#OlapChart").ejOlapChart({ url: oclientProxy.model.url, canResize: oclientProxy.model.isResponsive, currentReport: oclientProxy.currentReport, locale: oclientProxy.locale(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: oclientProxy.model.chartType, tooltip: { visible: true } }, beforeServiceInvoke: oclientProxy.model.chartLoad, drillSuccess: ej.proxy(oclientProxy.chartDrillSuccess, oclientProxy) });
                if (oclientProxy.gridLayout() != ej.PivotGrid.Layout.Normal)
                    oclientProxy.element.find("#PivotGrid").ejPivotGrid({ url: oclientProxy.model.url, isResponsive: oclientProxy.model.isResponsive, currentReport: oclientProxy.currentReport, layout: oclientProxy.gridLayout(), locale: oclientProxy.locale(), drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
                else
                    oclientProxy.element.find("#PivotGrid").ejPivotGrid({ url: oclientProxy.model.url, isResponsive: oclientProxy.model.isResponsive, currentReport: oclientProxy.currentReport, locale: oclientProxy.locale(), drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
            }
            else {
                if (oclientProxy.gridLayout() != ej.PivotGrid.Layout.Normal)
                    oclientProxy.element.find("#PivotGrid").ejPivotGrid({ url: oclientProxy.model.url, isResponsive: oclientProxy.model.isResponsive, currentReport: oclientProxy.currentReport, locale: oclientProxy.locale(), layout: oclientProxy.gridLayout(), drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
                else
                    oclientProxy.element.find("#PivotGrid").ejPivotGrid({ url: oclientProxy.model.url, isResponsive: oclientProxy.model.isResponsive, currentReport: oclientProxy.currentReport, locale: oclientProxy.locale(), drillSuccess: ej.proxy(oclientProxy.gridDrillSuccess, oclientProxy) });
                oclientProxy.element.find("#OlapChart").ejOlapChart({ url: oclientProxy.model.url, canResize: oclientProxy.model.isResponsive, currentReport: oclientProxy.currentReport, locale: oclientProxy.locale(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: oclientProxy.model.chartType, tooltip: { visible: true } }, drillSuccess: ej.proxy(oclientProxy.chartDrillSuccess, oclientProxy), beforeServiceInvoke: oclientProxy.model.chartLoad });
            }
            if (oclientProxy.displayMode() != "chartonly")
                oclientProxy.ogridObj = oclientProxy.element.find('#PivotGrid').data("ejPivotGrid");
            if (oclientProxy.displayMode() != "gridOnly")
                oclientProxy.ochartObj = oclientProxy.element.find('#OlapChart').data("ejOlapChart");
            var items = {};
            if (oclientProxy.ochartObj != null) {
                oclientProxy.element.find('#OlapChart').width(oclientProxy.ochartObj.model.size.width);
                items["chartModelWidth"] = oclientProxy.ochartObj.model.size.width;
            }
            items["controlPanelWidth"] = oclientProxy.element.find(".controlPanel").width();
            items["chartOuterWidth"] = oclientProxy._chartWidth;
            items["gridOuterWidth"] = oclientProxy._gridWidth;
            oclientProxy._initStyles.push(items);
            oclientProxy._wireEvents();
            if (oclientProxy.model.isResponsive) {
                oclientProxy._enableResponsive();
                oclientProxy._parentElwidth = $("#" + oclientProxy._id).parent().width();
                if (oclientProxy._parentElwidth < 850) {
                    oclientProxy._rwdToggleCollapse();
                }
                else if (oclientProxy._parentElwidth > 850) {
                    oclientProxy._rwdToggleExpand();
                }
            }
            oclientProxy._trigger("renderSuccess", oclientProxy);
            oclientProxy._successAction = "ClientRender";
            oclientProxy.progressPos = $("#" + oclientProxy._id).position();
        },

        _toggleAxisSuccess: function (msg) {
            var columnElements, rowElements, slicerElements;
            if (msg[0] != undefined) {
              columnElements = msg[0].Value; rowElements = msg[1].Value;
                slicerElements = msg[2].Value;oclientProxy.currentReport = msg[3].Value; oclientProxy.reports= msg[4].Value;
            }
            else if (msg.d != undefined) {
               columnElements = msg.d[0].Value; rowElements = msg.d[1].Value;
                slicerElements = msg.d[2].Value; oclientProxy.currentReport = msg.d[3].Value; oclientProxy.reports = msg.d[4].Value
             }
            else {
              columnElements = msg.Columns; rowElements = msg.Rows;
                slicerElements = msg.Slicers; oclientProxy.currentReport = msg.CurrentReport;
                oclientProxy.reports = msg.ClientReports;
            }
            oclientProxy.element.find(".cubeTable").find(".categoricalAxis").parent().html("").html(oclientProxy._createAxisElementBuilder(columnElements, rowElements, slicerElements));
            oclientProxy.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
            oclientProxy._renderControls();
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
          },
        _measureGroupChangedSuccess: function (cubeTree) {
            var cubeTreeInfo;
            if (cubeTree[0] != undefined) {
                cubeTreeInfo = cubeTree[0].Value;
            }
            if (cubeTree.d != undefined) {
                cubeTreeInfo = cubeTree.d[0].Value;
            }
            else
                cubeTreeInfo = cubeTree.CubeTreeInfo;
            this.element.find(".e-treeview, .cubeTreeView").remove();
            var cubeTree = ej.buildTag("div#cubeTreeView.cubeTreeView")[0].outerHTML;
            this.element.find(".cubeBrowser").append(cubeTree);
            var treeViewData = $.parseJSON(cubeTreeInfo);
            this.element.find(".cubeTreeView").ejTreeView({
                fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                allowDragAndDrop: true,
                allowDropChild: false,
                allowDropSibling: false,
                dragAndDropAcrossControl: true,
                nodeDropped: ej.proxy(this.nodeDropped, this),
                height: "496px"
            });
            var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            oclientWaitingPopup.hide();
            oclientProxy._successAction = "MeasureGroupChange";
        },

        _cubeChangedSuccess: function (report) {
            var cubeTreeInfo, measureGroupInfo, chartSettings;
            if (report[0] != undefined) {
                oclientProxy.currentReport = report[0].Value; cubeTreeInfo = report[1].Value; oclientProxy.reports = report[2].Value;
                reportsCount = report[3].Value; reportList = $.parseJSON(report[4].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(report[5].Value);
                chartSettings = $.parseJSON(report[6].Value);
                if (report[7] != null && report[7] != undefined)
                    oclientProxy.model.customObject = report[7].Value;
            }
            else if (report.d != undefined) {
                oclientProxy.currentReport = report.d[0].Value; cubeTreeInfo = report.d[1].Value; oclientProxy.reports = report.d[2].Value;
                reportsCount = report.d[3].Value; reportList = $.parseJSON(report.d[4].Value);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(report.d[5].Value);
                chartSettings = $.parseJSON(report.d[6].Value);
                if (report.d[7] != null && report.d[7] != undefined)
                    oclientProxy.model.customObject = report.d[7].Value;
            }
            else {

                oclientProxy.currentReport = report.NewReport; cubeTreeInfo = report.CubeTreeInfo; oclientProxy.reports = report.ClientReports;
                reportsCount = report.ReportsCount; reportList = $.parseJSON(report.ReportList);
                if (oclientProxy.model.enableMeasureGroups) measureGroupInfo = $.parseJSON(report.MeasureGroups);
                chartSettings = $.parseJSON(report.ControlSettings);
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            oclientProxy.model.chartType = chartSettings.ChartType.toLowerCase();
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "cubeChanged", element: this.element, customObject: this.model.customObject });
            this.element.find(".splitBtn, .e-pivotgrid, .e-olapchart, .e-treeview, .cubeTreeView, .cubeName, .measureGroupselector").remove();
            var cubeTree = ej.buildTag("div#cubeTreeView.cubeTreeView")[0].outerHTML;
            var cubeName = ej.buildTag("div.cubeName", this.currentCubeName)[0].outerHTML;
            if (this.model.enableMeasureGroups) {
                var measureGroupDropdown = "<div class ='measureGroupselector' style='margin:5px 5px 0px 5px'><input type='text' id='measureGroupSelector' class='measureGroupSelector' /></div>";
                $(measureGroupDropdown).prependTo(".cubeBrowser");
                this.element.find(".measureGroupSelector").ejDropDownList({
                    dataSource: measureGroupInfo,
                    fields: { text: "name", value: "name" },
                    height: "25px",
                    width: "100%"
                });
                this.element.find(".measureGroupSelector").attr("tabindex", 0);
                measureDropTarget = oclientProxy.element.find('#measureGroupSelector').data("ejDropDownList");
                measureDropTarget.selectItemByText(measureDropTarget.model.dataSource[0].name);
                this.element.find(".measureGroupSelector").ejDropDownList("option", "change", ej.proxy(oclientProxy.measureGroupChanged, oclientProxy));
            }
            var treeViewData = $.parseJSON(cubeTreeInfo);
            $(cubeTree).appendTo(".cubeBrowser");
            this.element.find(".cubeTreeView").ejTreeView({
                fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                allowDragAndDrop: true,
                allowDropChild: false,
                allowDropSibling: false,
                dragAndDropAcrossControl: true,
                nodeDropped: ej.proxy(this.nodeDropped, this),
                height: "496px"
            });
            var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
            for (var i = 0; i < treeViewElements.length; i++) {
                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
            }
            this.element.find(".reportlist").ejDropDownList("option", "change", "");
            this.element.find(".reportlist").ejDropDownList("option", "dataSource", reportList);
            this.element.find(".reportlist").ejDropDownList("option", "value", reportList[0].name);
            $(cubeName).prependTo(".cubeBrowser");
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            oclientWaitingPopup.hide();
            oclientProxy._successAction = "CubeChange";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        _removeSplitButtonSuccess: function (report) {
            if (report[0] != undefined) {
                oclientProxy.currentReport = report[0].Value; oclientProxy.reports = report[1].Value;
                if (report[2] != null && report[2] != undefined)
                    oclientProxy.model.customObject = report[2].Value;
            }
            else if (report.d != undefined) {
                oclientProxy.currentReport = report.d[0].Value; oclientProxy.reports = report.d[1].Value;
                if (report.d[2] != null && report.d[2] != undefined)
                    oclientProxy.model.customObject = report.d[2].Value;
            }
            else {
                oclientProxy.currentReport = report.UpdatedReport; oclientProxy.reports = report.ClientReports;
                if (report.customObject != null && report.customObject != undefined)
                    oclientProxy.model.customObject = report.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "removeSplitButton", element: this.element, customObject: this.model.customObject });
            oclientProxy._renderControls();
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            oclientProxy._successAction = "ButtonRemove";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        _chartTypeChangedSuccess: function (report) {
            if (report[0] != undefined) {
                this.currentReport = report[0].Value;
                this.reports = report[1].Value;
            }
            else if (report.d != undefined) {
                this.currentReport = report.d[0].Value;
                this.reports = report.d[1].Value;
            }
            else {
                this.currentReport = report.CurrentReport;
                this.reports = report.ClientReports;
            }

        },

        _mdxQuery: function (mdxquery) {
            oclientProxy.element.find(".e-dialog").hide();
            oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            if (mdxquery.d != undefined)
                oclientProxy._createDialog("mdx", mdxquery.d, "");
            else
                oclientProxy._createDialog("mdx", mdxquery, "");
        },

        _toolbarOperationSuccess: function (report) {
            if (report != null)
                if ((report.d != undefined && report.d || report[0] != undefined && report[0]) || report.CurrentReport) {
                    var columnElements, rowElements, slicerElements, action, renamedReport, chartSettings;
                    if (report.length > 1 && report[0] != undefined) {
                        oclientProxy.currentReport = report[0].Value; oclientProxy.reports = report[1].Value; reportsCount = report[2].Value; columnElements = report[3].Value;
                        rowElements = report[4].Value; slicerElements = report[5].Value; action = report[6].Value; reportList = $.parseJSON(report[7].Value);
                        renamedReport = report[8].Value; chartSettings = $.parseJSON(report[9].Value); cubeTreeInfo = report[10].Value; var cubeSelector = report[11].Value;
                        if (report[12] != null && report[12] != undefined)
                            oclientProxy.model.customObject = report[12].Value;
                    }
                    else if (report.d != undefined) {
                        oclientProxy.currentReport = report.d[0].Value; oclientProxy.reports = report.d[1].Value; reportsCount = report.d[2].Value; columnElements = report.d[3].Value;
                        rowElements = report.d[4].Value; slicerElements = report.d[5].Value; action = report.d[6].Value; reportList = $.parseJSON(report.d[7].Value);
                        renamedReport = report.d[8].Value; chartSettings = $.parseJSON(report.d[9].Value); cubeTreeInfo = report.d[10].Value; cubeSelector = report.d[11].Value;
                        if (report.d[12] != null && report.d[12] != undefined)
                            oclientProxy.model.customObject = report.d[12].Value;
                    }
                    else {

                        oclientProxy.currentReport = report.CurrentReport; oclientProxy.reports = report.Reports; reportsCount = report.ReportsCount; columnElements = report.Columns;
                        rowElements = report.Rows; slicerElements = report.Slicers; action = report.CurrentAction; reportList = $.parseJSON(report.ReportList);
                        renamedReport = report.RenamedReport; chartSettings = $.parseJSON(report.ControlSettings); cubeTreeInfo = report.CubeTreeInfo; cubeSelector = report.CubeSelector;
                        if (report.customObject != null && report.customObject != undefined)
                            oclientProxy.model.customObject = report.customObject;
                    }
                    oclientProxy.model.chartType = chartSettings.ChartType.toLowerCase();

                    if (this.model.afterServiceInvoke != null)
                        this._trigger("afterServiceInvoke", { action: "toolbarOperation", element: this.element, customObject: this.model.customObject });
                    if (action != "Report Change") {
                        this.element.find(".reportlist").ejDropDownList("option", "change", "");
                        this.element.find(".reportlist").ejDropDownList("option", "dataSource", reportList);
                        if (action == "Load Report") {
                            ddlTarget._initValue = true;
                            ddlTarget.selectItemByText(cubeSelector);
                            ddlTarget._initValue = false;
                            reportDropTarget.selectItemByText(reportDropTarget.model.dataSource[0].name);
                            this._selectedReport = reportDropTarget.currentValue;
                            $(".cubeName").html(cubeSelector);
                            var treeViewData = $.parseJSON(cubeTreeInfo);
                            this.element.find(".cubeTreeView").ejTreeView({
                                fields: { id: "id", parentId: "pid", text: "name", spriteCssClass: "spriteCssClass", dataSource: treeViewData },
                                allowDragAndDrop: true,
                                allowDropChild: false,
                                allowDropSibling: false,
                                dragAndDropAcrossControl: true,
                                nodeDropped: ej.proxy(this.nodeDropped, this),
                                height: "496px"
                            });
                            var treeViewElements = oclientProxy.element.find(".cubeTreeView").find("li");
                            for (var i = 0; i < treeViewElements.length; i++) {
                                treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
                            }
                        }
                        else if (action != "Rename Report") {
                            reportDropTarget.selectItemByText(reportDropTarget.model.dataSource[reportList.length - 1].name);
                            this._selectedReport = reportDropTarget.currentValue;
                        }
                        else {
                            reportDropTarget.selectItemByText(renamedReport);
                            this._selectedReport = reportDropTarget.currentValue;
                        }
                        this.element.find(".reportlist").ejDropDownList("option", "change", ej.proxy(this.reportChanged, this));
                    }
                    if (action != "Rename Report")
                        this.element.find(".splitBtn, .e-pivotgrid, .e-olapchart").remove();
                    if (action == "Remove Report" || action == "Report Change" || action == "Load Report" || action == "SortOrFilter" || action == "Save Report") {
                        $(oclientProxy._createSplitButtons(columnElements, "Columns")).appendTo(".categoricalAxis");
                        $(oclientProxy._createSplitButtons(rowElements, "Rows")).appendTo(".rowAxis");
                        $(oclientProxy._createSplitButtons(slicerElements, "Slicers")).appendTo(".slicerAxis");
                        this.element.find(".categoricalAxis, .rowAxis, .slicerAxis").find("button").ejButton({ height: "20px" });
                        if (this.model.enableDeferUpdate) {
                            this.model.enableDeferUpdate = false;
                            oclientProxy._renderControls();
                            this.model.enableDeferUpdate = true;
                        }
                        else
                            oclientProxy._renderControls();
                    }
                }
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            oclientProxy._successAction = "ToolbarOperation";
            oclientProxy._trigger("renderSuccess", oclientProxy);
            oclientWaitingPopup.hide();
        },

        _fetchChildNodeSuccess: function (data) {

            var newDataSource; var parentNode;
            if (data.length > 1 && data[0] != undefined)
                newDataSource = JSON.parse(data[0].Value);
            else if (data.d != undefined)
                newDataSource = JSON.parse(data.d[0].Value);
            else
                newDataSource = JSON.parse(data.ChildNodes);
            if (data[0] != undefined) {
                if (data[2] != null && data[2] != undefined)
                    oclientProxy.model.customObject = data[2].Value;
            }
            else if (data.d != undefined) {
                if (data.d[2] != null && data.d[2] != undefined)
                    oclientProxy.model.customObject = data.d[2].Value;
            }
            else {
                if (data.customObject != null && data.customObject != undefined)
                    oclientProxy.model.customObject = data.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "memberExpanded", element: this.element, customObject: this.model.customObject });
            var mapper = { id: "id", parentId: "pid", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus" };
            if ($(this.pNode).parents("li").length > 1)
                parentNode = $(this.pNode).parents("li").first();
            else
                parentNode = $(this.pNode).parents("li");
            $($(parentNode).find('input.nodecheckbox')[0]).ejCheckBox({ checked: false })
            this.memberTreeObj._createSubNodesWhenLoadOnDemand(newDataSource, this.pNode, mapper);
            $.each($(parentNode).children().find("li"), function (index, value) {
                value.setAttribute("tag", newDataSource[index].tag);
            });
            //this.memberTreeObj._expandCollapseAction(this.pNode);
            oclientProxy._successAction = "FetchChildNode";
            oclientProxy._trigger("renderSuccess", oclientProxy);
        },

        _reportToolbar: function () {
            return ej.buildTag("ul", ej.buildTag("li.newReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("NewReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.addReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("AddReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.removeReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("RemoveReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.renameReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("RenameReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.saveReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SaveReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.loadReportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("LoadReport"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.mdxImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("MDXQuery"), tabindex: 0 })[0].outerHTML +
            (this.model.enableDeferUpdate ? (ej.buildTag("li.autoExecuteImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("DeferUpdate"), tabindex: 0 })[0].outerHTML) : "") +
            ej.buildTag("li.excelExportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ExportToExcel"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.wordExportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ExportToWord"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.pdfExportImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ExportToPdf"), tabindex: 0 })[0].outerHTML +
            (oclientProxy.model.displaySettings.enableFullScreen ? (ej.buildTag("li.maximizedView e-icon", "", {}).attr({ "title": this._getLocalizedLabels("FullScreen"), tabindex: 0 })[0].outerHTML) : "") +
			ej.buildTag("li.colSortFilterImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SortOrFilterColumn"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.rowSortFilterImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("SortOrFilterRow"), tabindex: 0 })[0].outerHTML +
	        ej.buildTag("li.toggleaxisImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ToggleAxis"), tabindex: 0 })[0].outerHTML +
            ej.buildTag("li.chartTypesImg e-icon", "", {}).attr({ "title": this._getLocalizedLabels("ChartTypes"), tabindex: 0 })[0].outerHTML);
        },

        _createCubeSelector: function () {
            return ej.buildTag("div.csHeader", ej.buildTag("span.cubeText", this._getLocalizedLabels("CubeSelector"), { "padding-left": "6px", "float": "left" })[0].outerHTML + ej.buildTag("div", "<input type='text' id='cubeSelector' class ='cubeSelector' />", { display: "inline", "float": "left", "padding-left": "5px", "position": "relative", "top": "-3px" })[0].outerHTML, { width: (oclientProxy.model.isResponsive) ? "100%" : "415px", height: "20px" })[0].outerHTML;
        },

        _createCubeBrowser: function (cubeTreeInfo) {
            return ej.buildTag("div.cdbHeader", ej.buildTag("span", this._getLocalizedLabels("CubeDimensionBrowser"), { "padding-left": "5px" })[0].outerHTML, { width: "200px", height: "30px" })[0].outerHTML + ej.buildTag("div.cubeBrowser", ej.buildTag("div.cubeTreeView.visibleHide")[0].outerHTML, { width: "200px", height: "523px" })[0].outerHTML;
        },

        _createAxisElementBuilder: function (columnDimensions, rowDimensions, slicerDimensions) {
            var AEBPanel = ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Column"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML + ej.buildTag("div.categoricalAxis", this._createSplitButtons(columnDimensions, "Columns"), { width: "200px", height: "150px" })[0].outerHTML +
            ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Row"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML + ej.buildTag("div.rowAxis", this._createSplitButtons(rowDimensions, "Rows"), { width: "200px", height: "150px" })[0].outerHTML +
            ej.buildTag("div.axisHeader", ej.buildTag("span", this._getLocalizedLabels("Slicer"), { "padding-left": "5px" })[0].outerHTML, { height: "30px" })[0].outerHTML + ej.buildTag("div.slicerAxis", this._createSplitButtons(slicerDimensions, "Slicers"), { width: "200px", height: "150px" })[0].outerHTML;
            return AEBPanel;
        },

        _createSplitButtons: function (dimensionElements, axis) {
            var splitButtons = '';
            var captionName = '';
            if (dimensionElements.indexOf("~") > -1)
                for (var i = 0; i < dimensionElements.split('~')[0].split('#').length - 1; i++) {
                    captionName = dimensionElements.split('~')[0].split('#')[i + 1].split('.')[0];
                    if (captionName == "Measures") {
                        captionName = oclientProxy._getLocalizedLabels("Measures");
                    }
                    else if (!(dimensionElements.startsWith("#Measure")))
                        captionName = dimensionElements.split('~')[i + 2];
                    else if ((dimensionElements.indexOf("#Measure")) > -1)
                        captionName = dimensionElements.split('~')[i + 1];
                    if (captionName == undefined)
                        captionName = dimensionElements.split('~')[i + 1];
                    splitButtons += ej.buildTag("div.splitBtn", ej.buildTag("button", captionName).attr({ "title": dimensionElements.split('~')[0].split('#')[i + 1].replace('.', ' - ') })[0].outerHTML + ej.buildTag("span.removeSplitBtn e-icon")[0].outerHTML, "", { "tag": axis + ':' + dimensionElements.split('~')[0].split('#')[i + 1] })[0].outerHTML;
                }
            else
                for (var i = 0; i < dimensionElements.split('#').length - 1; i++) {
                    var caption = dimensionElements.split('#')[i + 1].split('.')[0] == "Measures" ? oclientProxy._getLocalizedLabels("Measures") : dimensionElements.split('#')[i + 1].split('.')[0];
                    if (caption.split("::")[1] == "NAMEDSET") {
                        caption = caption.split("::")[0];
                    }
                    var splitBtnTitle = dimensionElements.split('#')[i + 1].replace('.', ' - ') == "Measures" ? oclientProxy._getLocalizedLabels("Measures") : dimensionElements.split('#')[i + 1].replace('.', ' - ');
                    splitButtons += ej.buildTag("div.splitBtn", ej.buildTag("button", caption).attr({ "title": splitBtnTitle })[0].outerHTML + ej.buildTag("span.removeSplitBtn e-icon")[0].outerHTML, "", { "tag": axis + ':' + dimensionElements.split('#')[i + 1] })[0].outerHTML;
                }
            return splitButtons;
        },

        _setSplitBtnTitle: function () {
            var pivotButtons = this.element.find(".splitBtn");
            for (var i = 0; i < pivotButtons.length; i++) {
                var tagValue = $(pivotButtons[i]).attr("tag").split(":")[1].split("."),
                uniqueName = "", orgName = "";
                for (var j = 0; j < tagValue.length; j++) {
                    if (tagValue[0].toUpperCase() != "[Measures]")
                        uniqueName += "[" + tagValue[j] + "]" + (j % 2 == 0 ? "." : "");
                    else
                        uniqueName += tagValue[j] + (j % 2 == 0 ? "." : "");
                }
                if (oclientProxy.element.find(".cubeTreeView").length > 0 && !(uniqueName.indexOf("Measures") >= 0)) {
                    var liElementText = oclientProxy.element.find(".cubeTreeView").find("li[tag='" + uniqueName + "'] a:eq(0)").text();
                    if (liElementText.indexOf(".") > 0)
                        $(pivotButtons[i]).find("button").attr("title", liElementText.replace('.', ' - '));
                    else
                        splBtnTitle = $(pivotButtons[i]).find("button").attr("title", $(pivotButtons[i]).text() + " - " + (oclientProxy.element.find(".cubeTreeView").find("li[tag='" + uniqueName + "'] a:eq(0)").text() || oclientProxy.element.find(".cubeTreeView").find("li[tag='" + uniqueName.toUpperCase() + "'] a:eq(0)").text()));
                }


            }
        },

        _controlPanel: function () {
            var controlTable; var gridDiv;
            if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && (this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid || this.displayMode() == "")) {
                if (oclientProxy.model.isResponsive) {
                    this._chartHeight = "542px";
                    this._chartWidth = "98%";
                    this._gridHeight = "100%";
                    this._gridWidth = "100%";
                }
                else {
                    this._chartHeight = "538px";
                    this._chartWidth = "558px";
                    this._gridHeight = "535px";
                    this._gridWidth = 555;
                }

            }
            else if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && (this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid || this.displayMode() == "")) {
                if (oclientProxy.model.isResponsive) {
                    this._chartHeight = "275px";
                    this._chartWidth = "98%";
                    this._gridHeight = "550px";
                    this._gridWidth = "100%";
                }
                else {
                    this._chartHeight = "275px";
                    this._chartWidth = "559px";
                    this._gridHeight = "550px";
                    this._gridWidth = 555;
                }
            }
            else {
                if (oclientProxy.model.isResponsive) {
                    this._chartHeight = "560px";
                    this._chartWidth = "98%";
                    this._gridHeight = "550px";
                    this._gridWidth = "100%";
                }
                else {
                    this._chartHeight = "571px";
                    this._chartWidth = "557px";
                    this._gridHeight = "572px";
                    this._gridWidth = 555;
                }
            }
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.ChartOnly) {
                if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid) {
                    if (this.defaultView() == ej.olap.OlapClient.DefaultView.Grid)
                        gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div.gridContainer", ej.buildTag("div#PivotGrid", "", { "margin": "5px 7px 7px", "max-height": "300px", "overflow": "auto", "padding": "1px" })[0].outerHTML, { "position": "relative", "left": "5px", "top": "5px", "height": "300px", "width": this.model.isResponsive ? "98%" : "574px" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
                    else
                        gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div.gridContainer", ej.buildTag("div#PivotGrid", "", { "margin": "10px 7px 7px", "max-height": "300px", "padding": "1px", "width": this.model.isResponsive ? "auto" : "554px" })[0].outerHTML, { "position": "relative", "left": "7px", "border-top": "none", "margin-top": "-7px", "overflow": "auto", "height": "297px", "width": this.model.isResponsive ? "98%" : "571px" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
                }
                else {
                    if (this.displayMode() == ej.olap.OlapClient.DisplayMode.GridOnly)
                        gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div.gridContainer", ej.buildTag("div#PivotGrid", "", { "margin": "5px 7px 7px", "min-height": "300px", "max-height": this._gridHeight, "padding": "1px" })[0].outerHTML, { "position": "relative", "left": "7px", "height": "584px", "padding-top": "2px", "width": this.model.isResponsive ? "98%" : "571px", "top": "3px" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
                    else
                        gridDiv = ej.buildTag("div#gridPanel.gridPanel", ej.buildTag("div.gridContainer", ej.buildTag("div#PivotGrid", "", { "margin": "5px 7px 7px", "min-height": "300px", "overflow": "auto", "max-height": this._gridHeight, "padding": "1px" })[0].outerHTML, { "position": "relative", "left": window.navigator.userAgent.indexOf('MSIE ') > 0 ? "0px" : "7px", "height": "548px", "padding-top": "5px", "border-top": "none" })[0].outerHTML, { width: "auto", height: "auto" })[0].outerHTML;
                }
            }
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.GridOnly) {
                var chartDiv;
                if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid) {
                    if (this.defaultView() == ej.olap.OlapClient.DefaultView.Grid)
                        chartDiv = ej.buildTag("div#chartPanel.chartPanel", ej.buildTag("div.chartContainer", ej.buildTag("div#OlapChart", "", { "min-height": "285px" })[0].outerHTML, { "position": "relative", "left": "-2px", "height": "283px", "padding": "5px 0px 5px 5px", "width": this.model.isResponsive ? "98%" : "569px", "border-top": "none", "margin-right": "-11px" })[0].outerHTML, { width: "98%", height: "auto" })[0].outerHTML;
                    else
                        chartDiv = ej.buildTag("div#chartPanel.chartPanel", ej.buildTag("div.chartContainer", ej.buildTag("div#OlapChart", "", { "min-height": "275px" })[0].outerHTML, { "position": "relative", "height": "280px", "padding": "5px 0px 5px 5px", "width": this.model.isResponsive ? "98%" : "566px" })[0].outerHTML, { width: "98%", height: "auto" })[0].outerHTML;
                }
                else {
                    if (this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartOnly)
                        chartDiv = ej.buildTag("div#chartPanel.chartPanel", ej.buildTag("div.chartContainer", ej.buildTag("div#OlapChart", "", { "min-height": "275px", "height": "576px" })[0].outerHTML, { "position": "relative", "height": "578px", "padding": "5px 0px 5px 5px", "margin-right": "-11px" })[0].outerHTML, { width: "98%", height: "auto" })[0].outerHTML;
                    else
                        chartDiv = ej.buildTag("div#chartPanel.chartPanel", ej.buildTag("div.chartContainer", ej.buildTag("div#OlapChart", "", { "min-height": "275px" })[0].outerHTML, { "position": "relative", "top": "-5px", "left": window.navigator.userAgent.indexOf('MSIE ') > 0 ? "-7px" : "0px", "height": "543px", "padding": "5px 0px 5px 5px", "border-top": "none", "margin-right": "-11px" })[0].outerHTML, { width: "98%", height: "auto" })[0].outerHTML;
                }
            }
            if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tab && this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid) {
                var tabUl; var tabDiv;
                var gridAnchor = "<a style='font: bold 12px Segoe UI' href='#gridPanel' tabindex='0'>" + this._getLocalizedLabels("Grid") + "</a>";
                var chartAnchor = "<a style='font: bold 12px Segoe UI' href='#chartPanel' tabindex='0'>" + this._getLocalizedLabels("Chart") + "</a>";
                var marginLeft = window.navigator.userAgent.indexOf('MSIE ') > 0 ? "none" : "7px";
                if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart) {
                    tabUl = ej.buildTag("ul.clientTab", ej.buildTag("li", chartAnchor)[0].outerHTML + ej.buildTag("li", gridAnchor)[0].outerHTML, { "margin-left": marginLeft })[0].outerHTML;
                    tabDiv = ej.buildTag("div#clientTab", tabUl + chartDiv + gridDiv)[0].outerHTML;
                }
                else {
                    tabUl = ej.buildTag("ul.clientTab", ej.buildTag("li", gridAnchor)[0].outerHTML + ej.buildTag("li", chartAnchor)[0].outerHTML, { "margin-left": marginLeft })[0].outerHTML;
                    tabDiv = ej.buildTag("div#clientTab", tabUl + gridDiv + chartDiv, { "height": "100%" })[0].outerHTML;
                }
                controlTable = tabDiv;
            }
            else {
                controlTable = this._createControlContainer(this.controlPlacement(), chartDiv, gridDiv);
            }

            return ej.buildTag("div.controlPanel", controlTable, { width: oclientProxy.model.isResponsive ? "100%" : "572px", height: "600px", "top": oclientProxy.model.displaySettings.mode == ej.olap.OlapClient.DisplayMode.GridOnly ? "3px" : "none" })[0].outerHTML;
        },

        _createControlContainer: function (controlPlacement, chartDiv, gridDiv) {
            var controlTable;
            if (this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartOnly)
                controlTable = "<table style='width:100%'><tr><td>" + chartDiv + "</td></tr></table>";
            else if (this.displayMode() == ej.olap.OlapClient.DisplayMode.GridOnly)
                controlTable = "<table style='width:100%'><tr><td>" + gridDiv + "</td></tr></table>";
            else {
                if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile) {
                    if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart)
                        controlTable = "<table style='width:100%'><tr><td>" + chartDiv + "</td></tr><tr><td>" + gridDiv + "</td></tr></table>";
                    else
                        controlTable = "<table style='width:100%'><tr><td>" + gridDiv + "</td></tr><tr><td>" + chartDiv + "</td></tr></table>";
                }
                else {
                    if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart)
                        controlTable = "<table style='width:100%'><tr><td valign='top'>" + chartDiv + "</td><td valign='top'>" + gridDiv + "</td></tr></table>";
                    else
                        controlTable = "<table style='width:100%'><tr><td valign='top'>" + gridDiv + "</td><td valign='top'>" + chartDiv + "</td></tr></table>";
                }
            }
            return controlTable;
        },

        _renderPivotGrid: function (tagInfo) {

            this.element.find(".e-dialog").hide();
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.ChartOnly) {
                var gridDiv;
                if (this.ochartObj != undefined && this.ochartObj._startDrilldown) {
                    var report;
                    try {
                        report = JSON.parse(this.ogridObj.getOlapReport()).Report;
                    }
                    catch (err) {
                        report = this.ogridObj.getOlapReport();
                    }
                    report = !this.model.enableDeferUpdate ? report : oclientProxy._olapReport;
                    this.ogridObj.doAjaxPost("POST", this.model.url + "/" + this.ogridObj.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillDownGrid", "cellPosition": "", "currentReport": report, "clientReports": this.reports, "headerInfo": tagInfo, layout: this.gridLayout() }), this.ogridObj._drillDownSuccess);
                }
                else {
                    if (!this.model.enableDeferUpdate) {
                        this.element.find(".e-pivotgrid, #PivotGrid").remove();
                        if (this.controlPlacement() == ej.olap.OlapClient.ControlPlacement.Tile && this.displayMode() == ej.olap.OlapClient.DisplayMode.ChartAndGrid)
                            gridDiv = ej.buildTag("div#PivotGrid", "", { "margin": "5px 7px 7px", "max-height": "280px", "overflow": "auto", "width": this.model.isResponsive ? "auto" : "563px", "padding": "1px" })[0].outerHTML;
                        else
                            gridDiv = ej.buildTag("div#PivotGrid", "", { "margin": "5px 7px 7px", "min-height": "300px", "overflow": "auto", "max-height": this._gridHeight, "padding": "1px" })[0].outerHTML;
                        $(gridDiv).appendTo(this.element.find(".gridContainer"));
                        if (this.gridLayout() != ej.PivotGrid.Layout.Normal)
                            this.element.find("#PivotGrid").ejPivotGrid({ url: this.model.url, isResponsive: oclientProxy.model.isResponsive, currentReport: this.currentReport, locale: oclientProxy.locale(), layout: this.gridLayout(), drillSuccess: ej.proxy(this.gridDrillSuccess, this) });
                        else
                            this.element.find("#PivotGrid").ejPivotGrid({ url: this.model.url, isResponsive: oclientProxy.model.isResponsive, currentReport: this.currentReport, locale: oclientProxy.locale(), drillSuccess: ej.proxy(this.gridDrillSuccess, this) });
                        this.ogridObj = this.element.find('#PivotGrid').data("ejPivotGrid");
                    }
                    else {
                        oclientWaitingPopup.hide();
                    }
                }
            }
        },

        _renderOlapChart: function (drillTag, drillAction) {

            this.element.find(".e-dialog").hide();
            if (this.displayMode() != ej.olap.OlapClient.DisplayMode.GridOnly) {
                if (this.ogridObj != undefined && this.ogridObj._startDrilldown) {
                    var report = !this.model.enableDeferUpdate ? this.ochartObj.getOlapReport() : oclientProxy._olapReport;
                    this.ochartObj.doAjaxPost("POST", this.model.url + "/" + this.ochartObj.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": drillAction, "drilledSeries": drillTag, "olapReport": report, "clientReports": this.reports }), this.ochartObj.renderControlSuccess);
                }
                else {
                    if (!this.model.enableDeferUpdate) {
                        this.element.find(".e-olapchart").remove();
                        var chartDiv = ej.buildTag("div#OlapChart", "", { "min-height": "271px", "height":"auto" })[0].outerHTML;
                        $(chartDiv).appendTo(this.element.find(".chartContainer"));
                        this.element.find("#OlapChart").ejOlapChart({ url: this.model.url, canResize: oclientProxy.model.isResponsive, currentReport: this.currentReport, customObject: this.model.customObject, locale: oclientProxy.locale(), showTooltip: true, size: { height: oclientProxy._chartHeight, width: oclientProxy._chartWidth }, commonSeriesOptions: { type: oclientProxy.model.chartType, tooltip: { visible: true } }, beforeServiceInvoke: oclientProxy.model.chartLoad, drillSuccess: ej.proxy(oclientProxy.chartDrillSuccess, oclientProxy) });
                        this.ochartObj = this.element.find('#OlapChart').data("ejOlapChart");
                        this.element.find('#OlapChart').width(this.ochartObj.model.size.width);
                    }
                    else {
                        oclientWaitingPopup.hide();
                    }
                }
            }
        },

        _renderControls: function () {

            if (this.defaultView() == ej.olap.OlapClient.DefaultView.Chart) {
                oclientProxy._renderOlapChart();
                oclientProxy._renderPivotGrid();
            }
            else {
                oclientProxy._renderPivotGrid();
                oclientProxy._renderOlapChart();
            }
        },

        _createDialogRequest: function (evt) {

            args_className = evt.target.className;
            args_innerHTML = (evt.target.innerHTML == "" ? "ToolbarButtons" : evt.target.innerHTML);
            hierarchyCaption = (evt.currentTarget.attributes == "" ? "" : evt.currentTarget.attributes.getNamedItem("title").value);
            this.element.find(".e-dialog, .clientDialog").remove();
            if (args_innerHTML != "ToolbarButtons" && args_innerHTML != undefined) {
                oclientWaitingPopup.show();
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "fetchMemberTreeNodes", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                this.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.fetchMemberTreeNodes, JSON.stringify({ "action": "fetchMemberTreeNodes", "dimensionName": $(evt.target).parent().attr("Tag"), "olapReport": oclientProxy.currentReport, "customObject": serializedCustomObject }), oclientProxy._editorTreeInfoSuccess);
            }
            else {
                oclientProxy._createDialog(args_className, args_innerHTML, "");
            }
        },

        _fetchReportListSuccess: function (reportNameList) {

            if (reportNameList[0] != undefined) {
                if (reportNameList[2] != null && reportNameList[2] != undefined)
                    oclientProxy.model.customObject = reportNameList[2].Value;
            }
            else if (reportNameList.d != undefined) {
                if (reportNameList.d[2] != null && reportNameList.d[2] != undefined)
                    oclientProxy.model.customObject = reportNameList.d[2].Value;
            }
            else {
                if (reportNameList.customObject != null && reportNameList.customObject != undefined)
                    oclientProxy.model.customObject = reportNameList.customObject;
            }
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "fetchReportList", element: this.element, customObject: this.model.customObject });
            if (this.model.afterServiceInvoke != null)
                this._trigger("afterServiceInvoke", { action: "fetchReportList", element: this.element, customObject: this.model.customObject });
            oclientProxy.element.find(".e-dialog, .clientDialog").remove();
            if (reportNameList.d != undefined)
                reportNameList = (reportNameList.d[0].Value || reportNameList[0].Value);
            else
                reportNameList = reportNameList.ReportNameList;
            oclientProxy._createDialog("LoadReport", reportNameList, "");
        },

        _fetchSortState: function (value) {

            oclientWaitingPopup.hide();
            if (value[0] != undefined) {
                oclientProxy._createDialog("SortFilterDlg", value[0].Value, "");
            }
            else if (value.d != undefined) {
                oclientProxy._createDialog("SortFilterDlg", value.d[0].Value, "");
            }
            else {
                oclientProxy._createDialog("SortFilterDlg", value.FetchSortState, "");
            }
        },

        _createDialog: function (args_className, args_innerHTML, editorTree) {

            var sortDlgField = "";
            var sortState = "", filterState = "";
            var editorTreeInfo;
            if (editorTree[0] != undefined)
                editorTreeInfo = editorTree[0].Value;
            else if (editorTree.d != undefined)
                editorTreeInfo = editorTree.d[0].Value;
            else
                editorTreeInfo = editorTree.EditorTreeInfo;
            var ejDialog = ej.buildTag("div#clientDialog.clientDialog", { "opacity": "1" })[0].outerHTML;
            var dialogContent; var dialogTitle; var currentTag; var searchText;

            if (args_className.split(" ")[0] == "newReportImg") {
                dialogTitle = this._getLocalizedLabels("NewReport");
                currentTag = "New Report";
                var newReportLabel = "<p class='dialogPara'>" + this._getLocalizedLabels("ReportName") + "</p>";
                var newReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = newReportLabel + newReport;
            }
            else if (args_className.split(" ")[0] == "addReportImg") {
                dialogTitle = this._getLocalizedLabels("AddReport");
                currentTag = "Add Report";
                var addReportLabel = "<p class='dialogPara'>" + this._getLocalizedLabels("ReportName") + "</p>";
                var addReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = addReportLabel + addReport;
            }
            else if (args_className.split(" ")[0] == "removeReportImg") {
                dialogTitle = this._getLocalizedLabels("RemoveReport");
                currentTag = "Remove Report";
                if (reportsCount > 1)
                    dialogContent = "<p>" + this._getLocalizedLabels("AreYouSureToDeleteTheReport") + "?</p>";
                else
                    dialogContent = "<p>" + this._getLocalizedLabels("CannotRemoveSingleReport") + "!</p>";
            }
            else if (args_className.split(" ")[0] == "SortFilterDlg") {

                sortDlgField = args_innerHTML.split("||");
                if (sortDlgField[1] != "")
                    sortState = sortDlgField[1].split("<<");
                if (sortDlgField[2] != "")
                    filterState = sortDlgField[2].split("<<");
                oclientProxy.element.find(".e-dialog, .clientDialog").remove();
                dialogTitle = this._getLocalizedLabels("SortingAndFiltering");

                var sortingLbl = ej.buildTag("label#sortingLbl.sortingLbl", this._getLocalizedLabels("Sorting"))[0].outerHTML;
                var measureListLbl = ej.buildTag("label#measureListLbl.measureListLbl", this._getLocalizedLabels("Measure"))[0].outerHTML;
                var orderLbl = ej.buildTag("label#orderLbl.orderLbl", this._getLocalizedLabels("Order"))[0].outerHTML;
                var measureListDropDown = ej.buildTag("input#measuresList.measuresList", "", {}, { type: 'text', accesskey: 'M' })[0].outerHTML;

                var sortDisableRadio = ej.buildTag("input#sortDisable.sortDisable", "", {}, { name: "sort", type: "radio", checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'i' })[0].outerHTML + ej.buildTag("label.sortDisableLbl", this._getLocalizedLabels("Disable"))[0].outerHTML;
                var sortEnableRadio = ej.buildTag("input#sortEnable.sortEnable", "", {}, { name: 'sort', type: 'radio', role: 'radio', tabindex: 1, accesskey: 'n' })[0].outerHTML + ej.buildTag("label.sortEnableLbl", this._getLocalizedLabels("Enable"))[0].outerHTML + "<br/>";
                var newascRadio = ej.buildTag("input#radioBtnAsc.radioBtnAsc", "", {}, { name: 'order', type: 'radio', checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'A' })[0].outerHTML + ej.buildTag("label.radioBtnAscLbl", this._getLocalizedLabels("Ascending"))[0].outerHTML + "<br/>";
                var newdescRadio = ej.buildTag("input#radioBtnDesc.radioBtnDesc", "", {}, { name: 'order', type: 'radio', role: 'radio', tabindex: 2, accesskey: 'e' })[0].outerHTML + ej.buildTag("label.radioBtnDescLbl", this._getLocalizedLabels("Descending"))[0].outerHTML;
                var preservHierarchy = ej.buildTag("input#preserveHrchy.preserveHrchy", "", {}, { type: 'checkbox', checked: 'checked', tabindex: 0, role: 'radio', accesskey: 'r' })[0].outerHTML + ej.buildTag("label.preserveHrchyLbl", this._getLocalizedLabels("PreserveHierarchy"))[0].outerHTML;

                var filter = ej.buildTag("label#filterLbl.filterLbl", this._getLocalizedLabels("Filtering"))[0].outerHTML;
                var filterDisableRadio = ej.buildTag("input#filterDisable.filterDisable", "", {}, { name: 'filter', type: 'radio', checked: 'checked', role: 'radio', tabindex: 0, accesskey: 'i' })[0].outerHTML + ej.buildTag("label.filterDisableLbl", this._getLocalizedLabels("Disable"))[0].outerHTML;
                var filterEnableRadio = ej.buildTag("input#filterEnable.filterEnable", "", {}, { name: 'filter', type: 'radio', role: 'radio', tabindex: 0, accesskey: 'n' })[0].outerHTML + ej.buildTag("label.filterDisableLbl", this._getLocalizedLabels("Enable"))[0].outerHTML + "<br/>";
                var conditionLbl = ej.buildTag("label#conditionLbl.conditionLbl", this._getLocalizedLabels("Condition"))[0].outerHTML;
                var conditionDropDown = ej.buildTag("input#filterCondition.filterCondition", "", {}, { type: 'text', tabindex: 0, accesskey: 'o' })[0].outerHTML;

                var filterFrom = ej.buildTag("div#filterfrom.filterFrmDiv", ej.buildTag("input#filterFrom.filterFrom", "", {}, { name: 'inputVal', type: 'text', tabindex: 0 })[0].outerHTML)[0].outerHTML;
                var filterbtw = ej.buildTag("div#filterbtw.filterBtw", "<p>" + this._getLocalizedLabels("and") + "</p>")[0].outerHTML;
                var filterTo = ej.buildTag("div#filterto.filterToDiv", ej.buildTag("input#filterTo.filterTo", "", {}, { name: 'inputVal', type: 'text', tabindex: 0, accesskey: 'd' })[0].outerHTML)[0].outerHTML;
                var fMeasureListLbl = ej.buildTag("label#filterMeasureListLbl.filterMeasureListLbl", this._getLocalizedLabels("Measure"))[0].outerHTML;
                var fMeasureList = ej.buildTag("input#fMeasuresList.fMeasuresList", "", {}, { type: 'text', tabindex: 0, accesskey: 'M' })[0].outerHTML;
                var fValueLbl = ej.buildTag("label#filterValueLbl.filterValueLbl", this._getLocalizedLabels("Value"), {}, { tabindex: 0, accesskey: 'a' })[0].outerHTML;

                var sortDiv = ej.buildTag("div#sortingDlg.sortingDlg", "<table class='sortReportTbl'><tr><td>" + sortingLbl + "</td><td>" + sortEnableRadio + sortDisableRadio + "</td></tr><tr><td>" + measureListLbl + "</td><td>" + measureListDropDown + "</td></tr> <tr><td>" + orderLbl + "</td><td>" + newascRadio + newdescRadio + "</td></tr><tr><td></td><td>" + preservHierarchy + "</td></tr></table>")[0].outerHTML;
                var filterDiv = ej.buildTag("div#filteringDlg.filteringDlg", "<table class='sortReportTbl'><tr><td>" + filter + "</td><td>" + filterEnableRadio + filterDisableRadio + "</td></tr><tr><td>" + fMeasureListLbl + "</td><td>" + fMeasureList + "</td></tr><tr><td>" + conditionLbl + "</td><td>" + conditionDropDown + "</td></tr><tr><td>" + fValueLbl + "</td><td>" + filterFrom + filterbtw + filterTo + "</td></tr></table>")[0].outerHTML;

                var sortTab = "<li><a style='font: bold 12px Segoe UI' href='#sortingDlg'>" + this._getLocalizedLabels("Sorting") + "</a></li>";
                var filterTab = "<li><a style='font: bold 12px Segoe UI' href='#filteringDlg'>" + this._getLocalizedLabels("Filtering") + "</a></li>";
                var sortfilterTab = ej.buildTag("div#sortfilterTab.sortfilterTab", "<ul class ='sortfiltTab'>" + (sortTab + filterTab + "</ul>" + "<div class='borderFilterSortDlg'>" + sortDiv + filterDiv + "</div>"))[0].outerHTML;

                dialogContent = sortfilterTab;
            }
            else if (args_className.split(" ")[0] == "renameReportImg") {
                dialogTitle = this._getLocalizedLabels("RenameReport");
                currentTag = "Rename Report";
                var renameReportLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ReportName") + "</p>";
                renameReport = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = renameReportLabel + renameReport;
            }
            else if (args_className.split(" ")[0] == "saveReportImg") {
                dialogTitle = this._getLocalizedLabels("SaveReport");
                currentTag = "Save Report";
                var reportNameLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ReportName") + "</p>";
                reportName = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = reportNameLabel + reportName;
            }
            else if (args_className.split(" ")[0] == "LoadReport") {
                dialogTitle = this._getLocalizedLabels("LoadReport");
                currentTag = "Load Report";
                var reportNameLabel = "<table class='loadReportTbl'><tr><td class='loadReportTd'>" + oclientProxy._getLocalizedLabels("ReportName") + "</td>";
                var reportNamesDropdown = "<td><input type='text' id='reportNameList' class='reportNameList'/></td></tr></table>";
                dialogContent = reportNameLabel + reportNamesDropdown;
            }
            else if (args_className.split(" ")[0] == "mdx") {
                dialogTitle = this._getLocalizedLabels("MDXQuery");
                currentTag = "MDX Query";
                var textarea = "<textarea readonly='readonly' style='width:460px; height:180px; resize:none; margin:3px'>" + args_innerHTML + "</textarea></br><br>";
                dialogContent = textarea;
            }
            else if (args_className.split(" ")[0] == "chartTypesImg") {
                dialogTitle = this._getLocalizedLabels("ChartTypes");
                currentTag = "Chart Types";
                var reportNameLabel = "<p class='dialogPara'>" + oclientProxy._getLocalizedLabels("ChartTypes") + "</p>";
                reportName = "<input type=text id=reportName class='reportName'/></br>";
                dialogContent = reportNameLabel + reportName;
            }
            else if (args_className.indexOf("e-txt") > -1) {

                if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures")) {
                    dialogTitle = this._getLocalizedLabels("MeasureEditor");
                    dialogContent = "<p class='editorPara'>" + this._getLocalizedLabels("Measures") + "</p>"
                }
                else {
                    dialogTitle = this._getLocalizedLabels("MemberEditor");
                    dialogContent = "<p class='editorPara'>" + hierarchyCaption + "</p>"

                }
                var memberEditor; var checkOption = "";
                $(innerDiv).appendTo(EditorDiv);
                if (args_innerHTML != oclientProxy._getLocalizedLabels("Measures")) {
                    var checkAll = ej.buildTag("div.checkAll e-icon")[0].outerHTML;
                    var unCheckAll = ej.buildTag("div.unCheckAll e-icon")[0].outerHTML;
                    checkOption = ej.buildTag("div", checkAll + unCheckAll, { "height": "19px" })[0].outerHTML;
                }
                if (args_innerHTML == oclientProxy._getLocalizedLabels("Measures")) {
                    memberEditor = oclientProxy._createMeasureEditor(editorTree);
                }
                else
                    memberEditor = ej.buildTag("div#editorTreeView.editorTreeView")[0].outerHTML;
                var memberEditorDiv = ej.buildTag("div.memberEditorDiv", memberEditor)[0].outerHTML;
                var innerDiv = ej.buildTag("div", checkOption + memberEditorDiv)[0].outerHTML;
                var EditorDiv = ej.buildTag("div#EditorDiv.editorDiv", innerDiv)[0].outerHTML;
                dialogContent += EditorDiv + "</br>";
            }
            var dialogFooter;
            if (args_className.split(" ")[0] == "mdx") {
                var cancelBtn = "<button id=CancelBtn class='dialogCancelBtn'>" + this._getLocalizedLabels("OK") + "</button>";
                dialogFooter = ej.buildTag("div", cancelBtn, { "float": "right", "margin": "-5px 0 6px" })[0].outerHTML;
                $(ejDialog).appendTo("#" + oclientProxy._id);
                $(dialogContent + dialogFooter).appendTo(this.element.find(".clientDialog"));
                this.element.find(".clientDialog").ejDialog({ width: 500, height: 300, target: "#" + oclientProxy._id, enableResize: false });
            }
            else {
                var okBtn = "<button id=OKBtn class='dialogOKBtn'>" + this._getLocalizedLabels("OK") + "</button>";
                var cancelBtn = "<button id=CancelBtn class='dialogCancelBtn'>" + this._getLocalizedLabels("Cancel") + "</button>";
                dialogFooter = ej.buildTag("div", okBtn + cancelBtn, { "float": "right", "margin": "-5px 0 6px" })[0].outerHTML;
                $(ejDialog).appendTo("#" + oclientProxy._id);
                $(dialogContent + dialogFooter).appendTo(this.element.find(".clientDialog"));
                this.element.find(".clientDialog").ejDialog({ width: args_className.split(" ")[0] == "SortFilterDlg" ? 320 : 280, target: "#" + oclientProxy._id, enableResize: false });
            }
            this.element.find(".e-titlebar").prepend(ej.buildTag("div", dialogTitle, { "display": "inline" })[0].outerHTML)[0].setAttribute("tag", currentTag)
            this.element.find(".reportName").ejMaskEdit({ width: "155px" });
            this.element.find(".e-mask").addClass("dialogInput");
            this.element.find(".dialogOKBtn, .dialogCancelBtn").ejButton();
            $(".sortfilterTab").ejTab();
            $(".reportNameList").ejDropDownList({
                dataSource: args_innerHTML.split("__"),
                width: "150px",
                height: "20px",
            });
            $(".filterFrom,.filterTo").ejMaskEdit({
                name: "mask",
                inputMode: ej.InputMode.Text,
                width: "80px",
                height: "20px",
            });
            $(".measuresList").ejDropDownList({
                dataSource: args_innerHTML.split("||")[0].split("__"),
                width: "200px",
                height: "20px",
            });
            $(".fMeasuresList").ejDropDownList({
                dataSource: args_innerHTML.split("||")[0].split("__"),
                width: "200px",
                height: "20px",
            });
            $(".filterCondition").ejDropDownList({
                change: "_onActiveConditionChange",
                dataSource: ['EqualTo', 'NotEquals', 'GreaterThan', 'GreaterThanOrEqualTo', 'LessThan', 'LessThanOrEqualTo', 'Between', 'NotBetween'],
                width: "200px",
                height: "20px",
            });
            oclientProxy.element.find(".filterCondition").ejDropDownList("option", "change", ej.proxy(oclientProxy._onActiveConditionChange, oclientProxy));
            oclientProxy._dllSortMeasure = $(".measuresList").data("ejDropDownList");
            oclientProxy._dllFilterCondition = $(".filterCondition").data("ejDropDownList");
            oclientProxy._dllfMeasuresList = $(".fMeasuresList").data("ejDropDownList");
            if (sortState != "") {
                this.element.find(".radioBtnAsc")[0].checked = sortState[0] == "ASC" ? true : false;
                this.element.find(".radioBtnDesc")[0].checked = sortState[0] == "DESC" ? true : false;
                this.element.find(".sortDisable")[0].checked = !(this.element.find(".sortEnable")[0].checked = true);
                this.element.find(".preserveHrchy")[0].checked = sortState[1] == "PH" ? true : false;
                oclientProxy._isSorted = true;
                oclientProxy._dllSortMeasure.setModel({ value: (sortState[2]) });
            }
            if (filterState != "") {
                this.element.find(".filterEnable")[0].checked = true;
                oclientProxy._dllfMeasuresList.setModel({ value: (filterState[0]) });
                oclientProxy._dllFilterCondition.setModel({ value: (filterState[1]) });
                oclientProxy.element.find(".filterFrom").val(filterState[2]);
                if (filterState[3] != undefined && filterState[3] != "" && (filterState[1] == "Between" || filterState[1] == "NotBetween"))
                    oclientProxy.element.find(".filterTo").val(filterState[3]);
                else
                    $(".filterTo").attr("disabled", "disabled");
                oclientProxy._isFiltered = true;
            }
            if (this.element.find(".sortDisable")[0] != undefined && this.element.find(".sortDisable")[0].checked == true) {
                $('.measuresList_wrapper,.radioBtnAsc, .radioBtnDesc, .preserveHrchy').attr("disabled", "disabled");
                $('.measureListLbl, .orderLbl, .radioBtnAscLbl, .radioBtnDescLbl, .preserveHrchyLbl').addClass('sortFilterDisable');
                oclientProxy._dllSortMeasure.disable();
            }
            if (this.element.find(".filterDisable")[0] != undefined && this.element.find(".filterDisable")[0].checked == true) {
                $('.filterFrom, .filterTo').attr("disabled", "disabled");
                $('.filterMeasureListLbl, .conditionLbl, .filterValueLbl, .filterBtw').addClass('sortFilterDisable');
                oclientProxy._dllFilterCondition.disable();
                oclientProxy._dllfMeasuresList.disable();
            }
            this.element.find(".e-widget-content").height("auto");
            if (editorTree != "" && editorTree != undefined && args_innerHTML != oclientProxy._getLocalizedLabels("Measures")) {
                var treeViewData = $.parseJSON(editorTreeInfo);
                this.element.find(".editorTreeView").ejTreeView({
                    showCheckbox: true,
                    loadOnDemand: true,
                    height: $(".memberEditorDiv").height(),
                    fields: { id: "id", hasChild: "hasChildren", text: "name", isChecked: "checkedStatus", dataSource: treeViewData },
                    nodeClick: ej.proxy(this._onNodeExpand, this),
                });
                var treeViewElements = this.element.find(".editorTreeView").find("li");
                for (var i = 0; i < treeViewElements.length; i++) {
                    treeViewElements[i].setAttribute("tag", treeViewData[i].tag);
                }
            }
            this.memberTreeObj = this.element.find('.editorTreeView').data("ejTreeView");
            oclientProxy._unWireEvents();
            oclientProxy._wireEvents();
            oclientWaitingPopup.hide();
        },

        _createMeasureEditor: function (editorTree) {
            var editorTreeInfo;
            if (editorTree[0] != undefined)
                editorTreeInfo = $.parseJSON(editorTree[0].Value);
            else if (editorTree.d != undefined)
                editorTreeInfo = $.parseJSON(editorTree.d[0].Value);
            else
                editorTreeInfo = $.parseJSON(editorTree.EditorTreeInfo);
            var measureEditor = "";
            for (var i = 0; i < editorTreeInfo.length; i++) {
                measureEditor += "<div id=\"" + editorTreeInfo[i].uniqueName + "\"class=measureEditor>" + editorTreeInfo[i].name + "<span class='removeMeasure e-icon'></span></div>";
            }
            return measureEditor;
        },

        _onNodeExpand: function (args) {
            var selectedItem = (args.event.originalEvent != undefined && args.event.originalEvent.target != undefined) ? args.event.originalEvent.target : args.event.target; var tagInfo;
            var childCount = $(selectedItem).parents("li").first().children().find("li").length;
            if (selectedItem.className.indexOf("e-plus") > -1 && childCount == 0) {
                var checkedState = $(selectedItem).parent().find("input[type=checkbox]").prop("checked");
                var tagInfo = $(selectedItem).parents("li").first().attr("tag");
                this.pNode = selectedItem;
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "memberExpanded", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.memberExpand, JSON.stringify({
                    "action": "memberExpanded", "checkedStatus": checkedState, "parentNode": $(selectedItem).parents("li")[0].id, "tag": tagInfo, "dimensionName": this._dimensionName, "cubeName": this.currentCubeName, "olapReport": this.currentReport, "clientReports": this.reports, "customObject": serializedCustomObject
                }), this._fetchChildNodeSuccess);
            }
        },

        _onActiveConditionChange: function (args) {
            if (args.selectedText == "Between" || args.selectedText == "NotBetween")
                $(".filterTo").removeAttr("disabled");
            else {
                $(".filterTo").attr("disabled", "disabled");
                oclientProxy.element.find(".filterTo").val("");
            }
        },

        _reSizeHandler: function (args) {
            var isUpdate = true;
            oclientProxy._parentElwidth = $("#" + oclientProxy._id).parent().width();
            if (oclientProxy._parentElwidth < 850 && isUpdate) {
                oclientProxy._rwdToggleCollapse();
            }
            else if (oclientProxy._parentElwidth > 850) {
                oclientProxy._rwdToggleExpand();
                isUpdate = true;
            }
        },

        chartDrillSuccess: function (sender) {

            if (oclientProxy.currentReport != undefined)
                currentReport = oclientProxy.currentReport;
            if (oclientProxy.reports != undefined)
                reports = oclientProxy.reports;
            if (this.ochartObj._startDrilldown && this.ogridObj != null && this.ogridObj != undefined)
                this._renderPivotGrid(this.ochartObj._selectedTagInfo);
            else if (oclientProxy.ogridObj != undefined && oclientProxy.ogridObj.element.find("table") == "block")
                oclientWaitingPopup.hide();
        },

        gridDrillSuccess: function (sender) {
            if (oclientProxy.currentReport != undefined)
                currentReport = oclientProxy.currentReport;
            if (oclientProxy.reports != undefined)
                reports = oclientProxy.reports;
            if (this.ogridObj._startDrilldown && this.ochartObj != null && this.ochartObj != undefined) {
                var masktag; var levelCaption; var levelTag;
                levelCaption = this.ogridObj._drillCaption;
                jQuery.each(this.ochartObj._labelCurrentTags, function (index, value) {
                    if (value != undefined && value.name == levelCaption) {
                        levelTag = value.tag;
                        masktag = value;
                    }
                });
                oclientProxy.ochartObj._drillAction = this.ogridObj._drillAction;
                oclientProxy.ochartObj._selectedItem = levelCaption;
                if (this.ogridObj._drillAction == "drilldown")
                    oclientProxy.ochartObj._tagCollection.push(masktag)
                else if (!ej.isNullOrUndefined(masktag)) {
                    jQuery.each(this.element.find(".rowAxis").find("button"), function (index, value) {
                        if (value.innerHTML == masktag.tag.split("[")[1].split("]")[0])
                            oclientProxy.ochartObj._dimensionIndex = index;
                    });
                    jQuery.each(oclientProxy.ochartObj._tagCollection, function (index, value) {
                        if (value != undefined && value.name == levelCaption) {
                            oclientProxy.ochartObj._selectedIndex = index + oclientProxy.ochartObj._dimensionIndex;
                            oclientProxy.ochartObj._tagCollection.splice(index, oclientProxy.ochartObj._tagCollection.length);
                            return false;
                        }
                    });
                }
                if (levelTag != null && levelTag != undefined)
                    oclientProxy._renderOlapChart(levelTag, this.ogridObj._drillAction);
                else
                    oclientWaitingPopup.hide();
            }
            else if ($("#" + oclientWaitingPopup._id + "_WaitingPopup")[0].style.display == "block")
                oclientWaitingPopup.hide();
        },

        nodeDropped: function (sender) {
            if (sender.dropTarget.prevObject != undefined)
                sender.dropTarget.prevObject.removeClass("targetAxis");
            else
                $(sender.dropTarget).removeClass("targetAxis");
            isDragging = false;
            var droppedPosition = this.setSplitBtnTargetPos(sender.event);
            sender.dropTarget.prevObject != undefined ? sender.dropTarget.prevObject.parents().find(".e-dialog").hide() : $(sender.dropTarget).parents().find(".e-dialog").hide();
            var cssName = null;
            if ($(sender.dropTarget).hasClass("splitBtn"))
                cssName = $(sender.dropTarget).parents("div:eq(0)").attr("class");
            else if (sender.event.type == "touchend")
                cssName = sender.dropTarget.context.className;
            else if (sender.event.target != undefined) {
                if (sender.event.target.className == undefined || sender.event.target.className == null)
                    return false;
                if (sender.event.target.className.indexOf("splitBtn") > -1)
                    cssName = sender.dropTarget.context.className;
                else
                    cssName = sender.event.target.className;
            }
            else {
                if (sender.event.originalEvent.srcElement.className.indexOf("splitBtn") > -1)
                    cssName = sender.dropTarget.context.className;
                else
                    cssName = sender.event.originalEvent.srcElement.className;
            }

            var axisName = (cssName.indexOf("categoricalAxis") > cssName.indexOf("rowAxis")) ? ((cssName.indexOf("categoricalAxis") > cssName.indexOf("slicerAxis")) ? "Categorical" : "Slicer") : ((cssName.indexOf("rowAxis") > cssName.indexOf("slicerAxis")) ? "Series" : (cssName.indexOf("slicerAxis") >= 0) ? "Slicer" : null);
            var tagVal = $(sender.droppedElement).attr("tag");
            if (axisName != null && tagVal != "Measures" && tagVal.lastIndexOf(".DF") != tagVal.length - ".DF".length) {
                oclientWaitingPopup.show();
                if (this.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "nodeDropped", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(this.model.customObject);
                var params = oclientProxy.currentCubeName + "--" + $(sender.droppedElement).attr("tag") + "--" + axisName + "--" + droppedPosition;
                oclientProxy.doAjaxPost("POST", oclientProxy.model.url + "/" + oclientProxy.model.serviceMethodSettings.nodeDropped, JSON.stringify({ "action": "nodeDropped", "dropType": "TreeNode", "nodeInfo": params, "olapReport": oclientProxy.currentReport, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), oclientProxy._nodeDroppedSuccess);
            }
        },

        setSplitBtnTargetPos: function (event) {
            var targetPosition = ""; var AEBdiv; var targetSplitBtn; var className;
            if (event.event != undefined && event.event.type == "touchend") {
                targetSplitBtn = event.event.originalEvent.target != null ? $(event.event.originalEvent.target).parents(".splitBtn") : $(event.event.originalEvent.srcElement).parents(".splitBtn");
                className = event.event.originalEvent.target != null ? event.event.originalEvent.target.className : event.event.originalEvent.srcElement.className;
            }
            else {
                targetSplitBtn = event.originalEvent.target != null ? $(event.originalEvent.target).parents(".splitBtn") : $(event.originalEvent.srcElement).parents(".splitBtn");
                className = event.originalEvent.target != null ? event.originalEvent.target.className : event.originalEvent.srcElement.className;
            }
            if (targetSplitBtn[0] || (className != undefined && className != null && jQuery.type(className) == "string" && className.indexOf("splitBtn") > -1)) {
                targetSplitBtn = targetSplitBtn[0] ? targetSplitBtn[0] : event.originalEvent.target != null ? event.originalEvent.target : event.originalEvent.srcElement;
                if (event.event != undefined && event.event.type == "touchend")
                    AEBdiv = event.target;
                else if (this._dropAxisClassName != null && this._dropAxisClassName != undefined && this._dropAxisClassName != "")
                    AEBdiv = this.element.find("." + this._dropAxisClassName)[0];
                else
                    AEBdiv = targetSplitBtn.parentNode;
                if (AEBdiv != undefined) {
                    for (var i = 0; i < AEBdiv.children.length; i++) {
                        if (AEBdiv.childNodes[i] == targetSplitBtn)
                            targetPosition = i;
                    }
                }
                else {
                    oclientProxy._dropAxisClassName = "";
                }
            }
            return targetPosition;
        },

        cubeChanged: function (sender) {
            this.currentCubeName = sender.text;
            oclientWaitingPopup.show();
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "cubeChanged", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.cubeChanged, JSON.stringify({ "action": "cubeChanged", "cubeName": this.currentCubeName, "customObject": serializedCustomObject, "clientParams": this.model.enableMeasureGroups }), this._cubeChangedSuccess);
        },

        measureGroupChanged: function (sender) {
            oclientWaitingPopup.show();
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: "cubeChanged", element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.measureGroupChanged, JSON.stringify({ "action": "measureGroupChanged", "measureGroupName": sender.text, "customObject": serializedCustomObject }), this._measureGroupChangedSuccess);
        },

        reportChanged: function (sender) {
            if (sender.selectedText != this._selectedReport) {
                oclientWaitingPopup.show();
                if (oclientProxy.model.beforeServiceInvoke != null)
                    this._trigger("beforeServiceInvoke", { action: "toolbarOperation", element: this.element, customObject: this.model.customObject });
                var serializedCustomObject = JSON.stringify(oclientProxy.model.customObject);
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.toolbarServices, JSON.stringify({ "action": "toolbarOperation", "toolbarOperation": "Report Change", "clientInfo": sender.text, "olapReport": null, "clientReports": this.reports, "customObject": serializedCustomObject }), this._toolbarOperationSuccess);
            }
            this._selectedReport = reportDropTarget.currentValue;
        },

        onDropped: function (sender) {
            if (sender.target.className.indexOf("e-button") > -1) {
                var columnPosInfo = this.getAxisPosition(this.element.find(".categoricalAxis"));
                var rowPosInfo = this.getAxisPosition(this.element.find(".rowAxis"));
                var slicerPosInfo = this.getAxisPosition(this.element.find(".slicerAxis"));
                this._dropAxisClassName;
                if (sender.pageX != undefined && sender.pageY != undefined) {
                    if (sender.pageX > columnPosInfo.left && sender.pageX < columnPosInfo.right && sender.pageY > columnPosInfo.top && sender.pageY < columnPosInfo.bottom)
                        this._dropAxisClassName = "categoricalAxis";
                    else if (sender.pageX > rowPosInfo.left && sender.pageX < rowPosInfo.right && sender.pageY > rowPosInfo.top && sender.pageY < rowPosInfo.bottom)
                        this._dropAxisClassName = "rowAxis";
                    else if (sender.pageX > slicerPosInfo.left && sender.pageX < slicerPosInfo.right && sender.pageY > slicerPosInfo.top && sender.pageY < slicerPosInfo.bottom)
                        this._dropAxisClassName = "slicerAxis";
                    else
                        this._dropAxisClassName = "outOfAxis";
                }
            }
        },

        getAxisPosition: function (targetElement) {
            var tempPos = targetElement.position();
            var item = {};
            item["top"] = tempPos.top;
            item["right"] = tempPos.left + $(targetElement).width();
            item["bottom"] = tempPos.top + $(targetElement).height();
            item["left"] = tempPos.left;
            return item;
        },

        onTabClick: function (args) {
            if (oclientProxy.ochartObj != null && oclientProxy.ochartObj != undefined)
                oclientProxy.chartObj = oclientProxy.element.find("#" + oclientProxy.ochartObj._id + "Container").data("ejChart");
            currentTab = "chart";
            if ($(args.activeHeader).find("a").text() == this._getLocalizedLabels("Grid")) {
                currentTab = "grid";
                oclientProxy.element.find(".chartTypesImg").addClass("chartTypesOnGridView");
                oclientProxy.element.find(".gridPanel").css("display", "inline");
            }
            else if (oclientProxy.chartObj != null && oclientProxy.chartObj != undefined) {
                currentTab = "chart";
                oclientProxy.element.find(".chartTypesImg").removeClass("chartTypesOnGridView");
                oclientProxy.chartObj.redraw();
            }
        },

        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function () {
                    var eventArgs = { "customObject": this.model.customObject, "element": this.element };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    oclientWaitingPopup = $("#" + this._id).data("ejWaitingPopup");
                    oclientWaitingPopup.hide();
                    var eventArgs = { "customObject": this.model.customObject, "element": this.element, "Message": msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        },

        doPostBack: function (url, params) {
            var form = $('<form>').attr({ 'action': url, 'method': 'POST', 'name': 'export' });
            var addParam = function (paramName, paramValue) {
                var input = $('<input type="hidden" title="params">').attr({
                    'id': paramName,
                    'name': paramName,
                    'value': paramValue
                }).appendTo(form);
            };
            for (var item in params)
                addParam(item, params[item]);
            form.appendTo(document.body).submit().remove();
        }
    });

    ej.olap.OlapClient.locale = {};

    ej.olap.OlapClient.locale["en-US"] = {
        DeferUpdate: "Defer Update",
        MDXQuery: "MDX Query",
        Column: "Column",
        Row: "Row",
        Slicer: "Slicer",
        CubeSelector: "Cube Selector",
        ReportName: "Report Name",
        NewReport: "New Report",
        CubeDimensionBrowser: "Cube Dimension Browser",
        AddReport: "Add Report",
        RemoveReport: "Remove Report",
        CannotRemoveSingleReport: "Can not remove Single Report",
        AreYouSureToDeleteTheReport: "Are you sure to delete the Report",
        RenameReport: "Rename Report",
        SaveReport: "Save Report",
        ChartTypes: "Chart Types",
	    ToggleAxis: "Toggle Axis",
        LoadReport: "Load Report",
        ExportToExcel: "Export To Excel",
        ExportToWord: "Export To Word",
        ExportToPdf: "Export To Pdf",
        FullScreen: "Full Screen",
        Grid: "Grid",
        Chart: "Chart",
        OK: "OK",
        Cancel: "Cancel",
        MeasureEditor: "Measure Editor",
        MemberEditor: "Member Editor",
        Measures: "Measures",
        SortOrFilterColumn: "Sort/Filter (Column)",
        SortOrFilterRow: "Sort/Filter (Row)",
        SortingAndFiltering: "Sorting And Filtering",
        Sorting: "Sorting",
        Measure: "<u>M</u>easure",
        Order: "Order",
        Filtering: "Filtering",
        Condition: "C<u>o</u>ndition",
        Value: "Val<u>u</u>e",
        PreserveHierarchy: "P<u>r</u>eserve  Hierarchy",
        Ascending: "<u>A</u>scending",
        Descending: "D<u>e</u>scending",
        Enable: "E<u>n</u>able",
        Disable: "D<u>i</u>sable",
        and: "<u>a</u>nd",
        Line: "Line",
        Spline: "Spline",
        Column: "Column",
        Area: "Area",
        SplineArea: "Spline Area",
        StepLine: "Step Line",
        StepArea: "Step Area",
        Pie: "Pie",
        Bar: "Bar",
        StackingArea: "Stacking Area",
        StackingColumn: "Stacking Column",
        StackingBar: "Stacking Bar",
        Pyramid: "Pyramid",
        Funnel: "Funnel"
    }

    ej.olap.OlapClient.ControlPlacement = {
        Tab: "tab",
        Tile: "tile"
    };

    ej.olap.OlapClient.DisplayMode = {
        ChartOnly: "chartonly",
        GridOnly: "gridonly",
        ChartAndGrid: "chartandgrid"
    };

    ej.olap.OlapClient.ClientExportMode = {
        ChartAndGrid: "chartandgrid",
        ChartOnly: "chartonly",
        GridOnly: "gridonly"
    };

    ej.olap.OlapClient.DefaultView = {
        Chart: "chart",
        Grid: "grid"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html OLAP Chart elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejOlapChart", "ej.olap.OlapChart", {

        _rootCSS: "e-olapchart",
        element: null,
        model: null,
        validTags: ["div", "span"],
        defaults: $.extend(ej.datavisualization.Chart.prototype.defaults, {
            url: "",
            cssClass: "",
            currentReport: "",
            customObject: {},
            isResponsive: false,
            enable3D: false,
            rotation: 0,
            serviceMethodSettings: {
                initialize: "InitializeChart",
                drillDown: "DrillChart",
                exportOlapChart: "Export"
            },
            locale: "en-US",
            drillSuccess: null,
            beforeServiceInvoke: null,
            afterServiceInvoke: null,
            renderComplete: null,
            renderFailure: null,
            renderSuccess: null
        }),

        dataTypes: {
            marker: "data",
            crossHair: "data",
            size: "data",
            serviceMethodSettings: "data",
            zooming: "data",
            customObject: "data"
        },

        observables: ["title.text", "commonSeriesOptions.type", "locale"],
        titleText: ej.util.valueFunction("title.text"),
        seriesType: ej.util.valueFunction("commonSeriesOptions.type"),
        locale: ej.util.valueFunction("locale"),

        getOlapReport: function () {
            return this._olapReport;
        },

        setOlapReport: function (value) {
            this._olapReport = value;
        },

        getJsonRecords: function () {
            return this._JSONRecords;
        },

        setJsonRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        _init: function () {
            this._initPrivateProperties();
            this._load();
        },

        _destroy: function () {

            this.element.empty().removeClass("e-olapchart" + this.model.cssClass);
            if (this.ochartWaitingPopup != undefined)
                this.ochartWaitingPopup._destroy();
        },

        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            ochartProxy = this;
            this._olapReport = "";
            this._JSONRecords = null;
            this._currentAction = "initialize";
            this._selectedItem = "";
            this._selectedIndex = -1;
            this._selectedTagInfo = null;
            this._tagCollection = new Array();
            this._labelCurrentTags = new Array();
            this._startDrilldown = false;
            this._drillAction = "";
            this._initZooming = false;
            this._dimensionIndex = -1;
            this._isDrilled = false;
            this._selectedMenuItem = "";
            ochartWaitingPopup = null;
            ochartProgressBar = null;
            oChartTimer = null;
        },

        _load: function () {
            this.element.addClass(this.model.cssClass);
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                this.model.customObject = oclientProxy.model.customObject;
                if ($("#" + oclientProxy._id + "_maxView")[0]) {
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                }
                else
                    oclientWaitingPopup.show();
            }
            else {
                this.element.ejWaitingPopup({ showOnInit: true });
                ochartWaitingPopup = this.element.data("ejWaitingPopup");
            }
            if (this.model.zooming != "")
                this._initZooming = true;
            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { action: this._currentAction, element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined)
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initialize", "currentReport": this.model.currentReport, "customObject": serializedCustomObject }), this.renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initialize" }), this.renderControlSuccess);
        },

        _setFirst: true,

        _setModel: function (options) {
            var chartObj = this.element.find("#" + this._id + "Container").data("ejChart");
            for (var key in options) {
                switch (key) {
                    case "olapReport": this.setOlapReport(options[key]); break;
                    case "jsonData": this.setJsonRecords(options[key]); break;
                    case "refreshOlapChart": this.element.renderChartFromJSON(options[key]); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "height": this.model.size.height = options[key]; break;
                    case "width": this.model.size.width = options[key]; break;
                    case "commonSeriesOptions":
                        {
                            if (chartObj) {
                                chartObj.model.commonSeriesOptions = options[key];
                                chartObj.model.type = chartObj.model.commonSeriesOptions.type = this.seriesType();
                                for (var i = 0; i < chartObj.model.series.length; i++)
                                    chartObj.model.series[i].type = chartObj.model.type;
                            } break;
                        }
                    case "title":
                        {
                            if (chartObj)
                                chartObj.model.title.text = this.titleText();
                            break;
                        }
                    case "animation": this.model.animation = options[key]; break;
                    case "crossHair": this.model.crossHair = options[key]; break;
                    case "marker": this.model.marker = options[key]; break;
                    case "zooming": this.model.zooming = options[key]; break;
                    case "legend": this.model.legend = options[key]; break;
                    case "primaryXAxis": this.model.primaryXAxis = options[key]; break;
                    case "primaryYAxis": this.model.primaryYAxis = options[key]; break;
                    case "locale": this.locale(); break;
                    default:
                        { $.extend(true, this.model, {}, options[key]); }
                }
            }
            if (chartObj)
                this.renderControlSuccess({ "JsonRecords": JSON.stringify(this.getJsonRecords()), "OlapReport": this.getOlapReport() });
        },

        _wireEvents: function () {
            window.clearInterval(oChartTimer);
            this._on(this.element, "click", ".menuList", function (evt) {
                ochartWaitingPopup = this.element.data("ejWaitingPopup");
                if (evt.target.innerHTML == ochartProxy._getLocalizedLabels("Exit")) {
                    this.element.find("#" + ochartProxy._id + "ExpandMenu, .expandMenu, .e-dialog").remove();
                }
                else if (evt.target.innerHTML.indexOf(ochartProxy._getLocalizedLabels("Expand")) > -1) {
                    if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0])
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                        else
                            oclientWaitingPopup.show();
                    }
                    else {
                        ochartWaitingPopup.show();
                    }
                    if (!this._isDrilled) {
                        this._isDrilled = true;
                        jQuery.each(evt.target.parentElement.children, function (index, value) {
                            if (value.innerHTML == evt.target.innerHTML)
                                ochartProxy._dimensionIndex = index;
                        });
                    }
                    var report = this.getOlapReport();
                    if (typeof (oclientProxy) != "undefined") {
                        report = !oclientProxy.model.enableDeferUpdate ? report : oclientProxy.currentReport;
                        oclientProxy._olapReport = oclientProxy.currentReport;
                    }
                    this._drillAction = "drilldown";
                    this._selectedItem = $.trim(evt.target.innerHTML.replace(this._getLocalizedLabels("Expand") + " - ", ""));
                    jQuery.each(this._labelCurrentTags, function (index, value) {
                        if (value.name == ochartProxy._selectedItem) {
                            ochartProxy._tagCollection.push(value);
                            ochartProxy._selectedTagInfo = value.tag;
                            ochartProxy._selectedMenuItem = value.name;
                            return false;
                        }
                    });
                    this._startDrilldown = true;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: this._drillAction, element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined) {
                        if (typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": report, "clientReports": oclientProxy.reports, "customObject": serializedCustomObject }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": report, "customObject": serializedCustomObject }), this.renderControlSuccess);
                    }
                    else {
                        if (!typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": report }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drilldown", "drilledSeries": this._selectedTagInfo, "olapReport": report, "clientReports": oclientProxy.reports }), this.renderControlSuccess);
                    }

                }
                else {
                    if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                        if ($("#" + oclientProxy._id + "_maxView")[0])
                            $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                        else
                            oclientWaitingPopup.show();
                    }
                    else {
                        ochartWaitingPopup.show();
                    }
                    var report = this.getOlapReport();
                    if (typeof (oclientProxy) != "undefined") {
                        report = !oclientProxy.model.enableDeferUpdate ? report : oclientProxy.currentReport;
                        oclientProxy._olapReport = oclientProxy.currentReport;
                    }
                    this._drillAction = "drillup";
                    this._selectedItem = $.trim(evt.target.innerHTML.replace(this._getLocalizedLabels("Collapse") + " - ", ""));
                    jQuery.each(this._tagCollection, function (index, value) {
                        if (value.name == ochartProxy._selectedItem) {
                            ochartProxy._selectedIndex = index + ochartProxy._dimensionIndex;
                            ochartProxy._selectedTagInfo = value.tag;
                            ochartProxy._tagCollection.splice(index, ochartProxy._tagCollection.length);
                            return false;
                        }
                    });
                    if (this._tagCollection.length == 0)
                        this._isDrilled = false;
                    this._startDrilldown = true;
                    if (this.model.beforeServiceInvoke != null)
                        this._trigger("beforeServiceInvoke", { action: this._drillAction, element: this.element, customObject: this.model.customObject });
                    var serializedCustomObject = JSON.stringify(this.model.customObject);
                    if (this.model.customObject != "" && this.model.customObject != null && this.model.customObject != undefined) {
                        if (typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": report, "clientReports": reports, "customObject": serializedCustomObject }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": report, "customObject": serializedCustomObject }), this.renderControlSuccess);
                    }
                    else {
                        if (!typeof (oclientProxy) != "undefined")
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": report }), this.renderControlSuccess);
                        else
                            this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.drillDown, JSON.stringify({ "action": "drillup", "drilledSeries": this._selectedTagInfo, "olapReport": report, "clientReports": reports }), this.renderControlSuccess);
                    }
                }
            });
        },

        _unWireEvents: function () {
            $(document.body).unbind("click");
            this._off(this.element, "click", ".menuList");
        },

        renderControlSuccess: function (msg) {
            ochartProxy = this;
            if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null)
                if ($("#" + oclientProxy._id + "_maxView")[0])
                    $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: true });
                else
                    oclientWaitingPopup.show();
            try {
                if (msg[0] != undefined) {
                    ochartProxy.setJsonRecords(msg[0].Value); ochartProxy.setOlapReport(msg[1].Value);
                    if (typeof (oclientProxy) != "undefined") {
                        oclientProxy.currentReport = msg[1].Value;
                        if (msg[2] != null && msg[2] != undefined && msg[2].Key == "ClientReports")
                            oclientProxy.reports = msg[2].Value;
                    }
                    if (msg[2] != null && msg[2] != undefined && !msg[2].Key == "ClientReports")
                        ochartProxy.model.customObject = msg[2].Value;
                }
                else if (msg.d != undefined) {
                    ochartProxy.setJsonRecords(msg.d[0].Value); ochartProxy.setOlapReport(msg.d[1].Value);
                    if (typeof (oclientProxy) != "undefined") {
                        oclientProxy.currentReport = msg.d[1].Value;
                        if (msg.d[2] != null && msg.d[2] != undefined && msg.d[2].Key == "ClientReports")
                            oclientProxy.reports = msg.d[2].Value;
                    }
                    if (msg.d[2] != null && msg.d[2] != undefined && !(msg.d[2].Key == "ClientReports"))
                        ochartProxy.model.customObject = msg.d[2].Value;
                }
                else {
                    ochartProxy.setJsonRecords(msg.JsonRecords); ochartProxy.setOlapReport(msg.OlapReport);
                    if (msg.customObject != null && msg.customObject != null)
                        ochartProxy.model.customObject = msg.customObject;
                    if (typeof (oclientProxy) != "undefined") {
                        if (typeof (oclientProxy.currentReport) != "undefined")
                            oclientProxy.currentReport = msg.OlapReport;
                        if (typeof (oclientProxy.reports) != "undefined" && msg.reports != undefined && msg.reports != "undefined")
                            oclientProxy.reports = msg.reports;
                    }
                }
                if (ochartProxy.model.afterServiceInvoke != null) {
                    var eventArgs;
                    if (ochartProxy._drillAction != "")
                        eventArgs = { action: ochartProxy._drillAction, element: ochartProxy.element, customObject: ochartProxy.model.customObject };
                    else
                        eventArgs = { action: ochartProxy._currentAction, element: ochartProxy.element, customObject: ochartProxy.model.customObject };
                    ochartProxy._trigger("afterServiceInvoke", eventArgs);
                }
                var htmlTag = ej.buildTag("div#" + ochartProxy._id + "Container")[0].outerHTML;
                ochartProxy.element.html(htmlTag);
                ochartProxy.renderChartFromJSON(ochartProxy.getJsonRecords());
                ochartProxy._unWireEvents();
                ochartProxy._wireEvents();
                if (ochartProxy._drillAction != "") {
                    ochartProxy.model.currentReport = ochartProxy.getOlapReport();
                    ochartProxy._trigger("drillSuccess", ochartProxy.element);
                }
                if (typeof oclientWaitingPopup != 'undefined' && oclientWaitingPopup != null) {
                    if ($("#" + oclientProxy._id + "_maxView")[0]) {
                        $("#" + oclientProxy._id + "_maxView").ejWaitingPopup({ showOnInit: false });
                        oclientWaitingPopup.hide();
                    }
                    else if (typeof oclientProxy.ogridObj != "undefined") {
                        if (oclientProxy && (oclientProxy.ogridObj._drillAction && !oclientProxy.ogridObj._startDrilldown) || oclientProxy.ochartObj._drillAction && !oclientProxy.ochartObj._startDrilldown)
                            oclientWaitingPopup.hide();
                        else if (oclientProxy && oclientProxy.ogridObj._drillAction == "" && oclientProxy.ochartObj._drillAction == "" && !oclientProxy.ogridObj._startDrilldown && !oclientProxy.ochartObj._startDrilldown)
                            oclientWaitingPopup.hide();
                        else if (oclientProxy.ochartObj._startDrilldown && !oclientProxy.ogridObj._startDrilldown && !$("#" + oclientProxy._id + "_maxView")[0])
                            oclientWaitingPopup.show();
                        else if (!oclientProxy.ochartObj._startDrilldown && !oclientProxy.ogridObj._startDrilldown && oclientProxy.ochartObj._drillAction == "" && oclientProxy.ogridObj._drillAction == "")
                            oclientWaitingPopup.hide();
                    }
                    if (oclientProxy.model.displaySettings.mode == "chartonly")
                        oclientWaitingPopup.hide();
                }
                else
                    ochartWaitingPopup.hide();
                if (typeof oclientProxy != 'undefined')
                    oclientProxy.ochartObj._startDrilldown = false;
            }
            catch (err) {
            }
            var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, "element": this.element };
            this._trigger("renderSuccess", eventArgs);
        },

        renderChartFromJSON: function (jsonData) {
            window.clearInterval(oChartTimer);
            if (jsonData != null && jsonData.chartLables.length > 0 && jsonData.points_Y.length > 0) {
                var chartSeries = new Array(); var seriesPoints = new Array(); var XValues = jsonData.chartLables; var tempArray = new Array();
                var YValues = new Array(); var pointsCount = pCnt = 0; var count = 0; var seriesName = new Array(); var points, measureRCnt = 0;
                measureRCnt = jsonData.measureInfo != "" ? parseInt(jsonData.measureInfo) : 0;
                jQuery.each(jsonData.chartLables, function (index, value) {
                    var YPoints = new Array();
                    if (measureRCnt > 1 && jsonData.addInfo.action != "DrillChart" || (measureRCnt > 1 && jsonData.addInfo.levelHash == "1")) {
                        var lblVal = (((index + 1) * measureRCnt) - measureRCnt);
                        for (var mCnt = 0; mCnt < measureRCnt; mCnt++) {
                            if (jsonData.points_Y[(lblVal + mCnt)] != undefined && jsonData.chartLables[index] == jsonData.points_Y[(lblVal + mCnt)][0].Item1)
                                for (var i = 0; i < jsonData.points_Y[index].length; i++) {
                                    points = { "xValues": jsonData.chartLables[index], "yValues": (jsonData.points_Y[lblVal + mCnt][i].Item2) };
                                    YPoints.push(points);
                                }
                        }
                    }
                    else
                        for (var i = 0; i < jsonData.points_Y[index].length; i++) {
                            points = { "xValues": jsonData.chartLables[index], "yValues": (jsonData.points_Y[index])[i].Item2 };
                            YPoints.push(points);
                        }
                    seriesPoints.push(YPoints);
                });

                if (measureRCnt > 1 && jsonData.addInfo.action != "DrillChart" || (measureRCnt > 1 && jsonData.addInfo.levelHash == "1"))
                    for (var mCnt = 0; mCnt < (measureRCnt * jsonData.seriesNames.length) ; mCnt++)
                        tempArray.push(new Array());
                else {
                    jQuery.each(jsonData.seriesNames, function (index, value) {
                        tempArray.push(new Array());
                    });
                }
                jQuery.each(jsonData.chartLables, function (index, value) {
                    for (var i = 0; i < seriesPoints[index].length; i++) {
                        tempArray[i].push((seriesPoints[index])[i]);
                    }
                });
                if (measureRCnt > 1 && jsonData.addInfo.action != "DrillChart" || (measureRCnt > 1 && jsonData.addInfo.levelHash == "1")) {
                    for (var index = 0; index < (measureRCnt * jsonData.seriesNames.length) ; index++) {
                        chartSeries[pCnt] = { dataSource: tempArray[index], xName: "xValues", yName: "yValues", name: jsonData.seriesNames[pointsCount] };
                        pointsCount++; pCnt++;
                        if (pointsCount == jsonData.seriesNames.length)
                            pointsCount = 0;
                    }
                }
                else {
                    jQuery.each(jsonData.seriesNames, function (index, value) {
                        chartSeries[pointsCount] = { dataSource: tempArray[index], xName: "xValues", yName: "yValues", name: jsonData.seriesNames[pointsCount] };
                        pointsCount++;
                    });
                }
                if (jsonData.chartLables.length > 10 && this.model.zooming == "")
                    this.model.zooming = { enable: true, type: 'x,y', enableMouseWheel: true };
                else if (jsonData.chartLables.length < 10 && this._initZooming == "")
                    this.model.zooming = this._initZooming;
                if (jQuery.inArray(this.seriesType(), ["line", "spline", "area", "splinearea", "stepline", "steparea", "stackingarea"]) > -1 && !this.model.commonSeriesOptions.marker.visible)
                    this.model.commonSeriesOptions.marker = {
                        shape: ej.olap.OlapChart.SymbolShapes.Circle,
                        size: { height: 12, width: 12 },
                        visible: true,
                        connectorLine: { height: 30, type: "line" },
                        border: { width: 3, color: 'white' }
                    };
                else if ((this.seriesType() == "pie" || this.seriesType() == "pyramid")) {
                    this.model.commonSeriesOptions.marker = {
                        dataLabel: {
                            visible: true,
                            font: { color: 'black', size: '15px', fontWeight: 'lighter' }
                        }
                    };
                }
                else if (jQuery.inArray(this.seriesType(), ["column", "stackingcolumn", "bar", "stackingcolumn"]) > -1 && this.model.commonSeriesOptions.marker) {
                    if (this.model.commonSeriesOptions.marker.dataLabel)
                        this.model.commonSeriesOptions.marker.dataLabel.visible = false;
                    this.model.commonSeriesOptions.marker.visible = false;
                }
                if (this.model.crosshair.visible) {
                    this.model.commonSeriesOptions.tooltip.format = "#point.x# : #point.y#";
                }
                else
                    this.model.commonSeriesOptions.tooltip.format = (this._getLocalizedLabels("Measure")) + " : " + jsonData.measureNames + " <br/>" + (this._getLocalizedLabels("Row")) + " : #series.name# <br/>" + (this._getLocalizedLabels("Column")) + " : #point.x# <br/>" + (this._getLocalizedLabels("Value")) + " : #point.y#";
                $("#" + ochartProxy._id + "Container").ejChart(
                        {
                            border: ochartProxy.model.border,
                            chartArea: ochartProxy.model.chartArea,
                            primaryXAxis: ochartProxy.model.primaryXAxis,
                            primaryYAxis: ochartProxy.model.primaryYAxis,
                            secondaryX: ochartProxy.model.secondaryX,
                            secondaryY: ochartProxy.model.secondaryY,
                            striplineDefault: ochartProxy.model.striplineDefault,
                            title: {
                                text: ochartProxy.titleText(),
                                textAlignment: ochartProxy.model.title.textAlignment,
                                font: {
                                    color: ochartProxy.model.title.color,
                                    fontFamily: ochartProxy.model.title.fontFamily,
                                    fontWeight: ochartProxy.model.title.fontWeight,
                                    opacity: ochartProxy.model.title.opacity,
                                    size: ochartProxy.model.title.size,
                                    fontStyle: ochartProxy.model.title.fontStyle
                                }
                            },
                            lineCap: ochartProxy.model.lineCap,
                            lineJoin: ochartProxy.model.lineJoin,
                            legendAlignment: ochartProxy.model.legendAlignment,
                            legendPosition: ochartProxy.model.legendPosition,
                            legend: ochartProxy.model.legend,
                            animation: ochartProxy.model.animation,
                            crosshair: ochartProxy.model.crosshair,
                            commonSeriesOptions: {
                                doughnutCoefficient: ochartProxy.model.commonSeriesOptions.doughnutCoefficient,
                                explodeOffset: ochartProxy.model.commonSeriesOptions.explodeOffset,
                                pyramidMode: ochartProxy.model.commonSeriesOptions.pyramidMode,
                                gapRatio: ochartProxy.model.commonSeriesOptions.gapRatio,
                                pieCoefficient: ochartProxy.model.commonSeriesOptions.pieCoefficient,
                                doughnutSize: ochartProxy.model.commonSeriesOptions.doughnutSize,
                                startAngle: ochartProxy.model.commonSeriesOptions.startAngle,
                                xAxisName: ochartProxy.model.commonSeriesOptions.xAxisName,
                                yAxisName: ochartProxy.model.commonSeriesOptions.yAxisName,
                                explodeAll: ochartProxy.model.commonSeriesOptions.explodeAll,
                                explodeIndex: ochartProxy.model.commonSeriesOptions.explodeIndex,
                                tooltipOptions: ochartProxy.model.commonSeriesOptions.tooltipOptions,
                                marker: ochartProxy.model.commonSeriesOptions.marker || ochartProxy.model.marker,
                                font: ochartProxy.model.commonSeriesOptions.font,
                                type: ochartProxy.seriesType(),
                                enableAnimation: ochartProxy.model.commonSeriesOptions.enableAnimation,
                                style: ochartProxy.model.commonSeriesOptions.style,
                                explode: ochartProxy.model.commonSeriesOptions.explode,
                                labelPosition: ochartProxy.model.commonSeriesOptions.labelPosition,
                                tooltip: ochartProxy.model.commonSeriesOptions.tooltip
                            },
                            seriesStyle: ochartProxy.model.seriesStyle,
                            pointStyle: ochartProxy.model.pointStyle,
                            textStyle: ochartProxy.model.textStyle,
                            initSeriesRender: ochartProxy.model.initSeriesRender,
                            theme: ochartProxy.model.theme,
                            canResize: ochartProxy.model.isResponsive,
                            rotation: ochartProxy.model.rotation,
                            enable3D: ochartProxy.model.enable3D,
                            zooming: ochartProxy.model.zooming,
                            margin: ochartProxy.model.margin,
                            elementSpacing: ochartProxy.model.elementSpacing,
                            seriesColors: ochartProxy.model.seriesColors,
                            seriesBorderColors: ochartProxy.model.seriesBorderColors,
                            pointColors: ochartProxy.model.pointColors,
                            pointBorderColors: ochartProxy.model.pointBorderColors,
                            series: chartSeries,
                            size: ochartProxy.model.size,
                            load: ochartProxy.model.load,
                            axesRangeCalculate: ochartProxy.model.axesRangeCalculate,
                            axesTitleRendering: ochartProxy.model.axesTitleRendering,
                            chartAreaBoundsCalculate: ochartProxy.model.chartAreaBoundsCalculate,
                            legendItemRendering: ochartProxy.model.legendItemRendering,
                            lengendBoundsCalculate: ochartProxy.model.lengendBoundsCalculate,
                            preRender: ochartProxy.model.preRender,
                            seriesRendering: ochartProxy.model.seriesRendering,
                            symbolRendering: ochartProxy.model.symbolRendering,
                            titleRendering: ochartProxy.model.titleRendering,
                            axesLabelsInitialize: ochartProxy.model.axesLabelsInitialize,
                            pointRegionMouseMove: ochartProxy.model.pointRegionMouseMove,
                            legendItemClick: ochartProxy.model.legendItemClick,
                            legendItemMouseMove: ochartProxy.model.legendItemMouseMove,
                            displayTextRendering: ochartProxy.model.displayTextRendering,
                            toolTipInitialize: ochartProxy.model.toolTipInitialize,
                            trackAxisToolTip: ochartProxy.model.trackAxisToolTip,
                            trackToolTip: ochartProxy.model.trackToolTip,
                            animationComplete: ochartProxy.model.animationComplete,
                            destroy: ochartProxy.model.destroy,
                            create: ochartProxy.model.create,
                            axesLabelRendering: ej.proxy(ochartProxy._onLabelRenders, ochartProxy),
                            pointRegionClick: ej.proxy(ochartProxy._onSeriesClick, ochartProxy)
                        });

                $("#" + ochartProxy._id + "progressContainer").hide();
                ochartProxy._labelCurrentTags.splice(0, ochartProxy._labelCurrentTags.length);
                jQuery.each(jsonData.labelTags, function (index, value) {
                    var currentTag = new Object();
                    var splitData = value.split("::");
                    currentTag = { "name": splitData[2], "state": splitData[splitData.length - 1], "tag": value };
                    ochartProxy._labelCurrentTags.push(currentTag);
                });
                jQuery.each(ochartProxy._labelCurrentTags, function (index, value) {
                    if (value.name == ochartProxy._selectedItem && ochartProxy._drillAction == "drillup") {
                        ochartProxy._selectedTagInfo = ochartProxy._labelCurrentTags[index].tag;
                        return false;
                    }
                });
            }
            if (ochartWaitingPopup != null)
                this.element.ejWaitingPopup({ showOnInit: false });
        },

        _onLabelRenders: function (args) {
            ochartProxy._trigger("axesLabelRendering", args);
            if ((args.data.axis.orientation.toLowerCase() == "horizontal" && args.data.label.Text != undefined) || ((ochartProxy.seriesType() == "bar" || ochartProxy.seriesType() == "stackingbar") && args.data.axis.orientation.toLowerCase() == "vertical" && args.data.label.Text != undefined)) {
                var splitName = args.data.label.Text.toString().split("~~");
                if (ochartProxy._tagCollection.length == 0)
                    args.data.label.Text = splitName[splitName.length - 1];
                else if (ochartProxy._drillAction == "drilldown" && splitName.indexOf(ochartProxy._selectedItem) + 1 < splitName.length)
                    args.data.label.Text = splitName[splitName.indexOf(ochartProxy._selectedItem) + 1];
                else if (ochartProxy._drillAction == "drilldown" && splitName.indexOf(ochartProxy._selectedItem) + 1 >= splitName.length)
                    args.data.label.Text = splitName[splitName.indexOf(ochartProxy._selectedItem)];
                else if (splitName.length == 1)
                    args.data.label.Text = splitName[0];
                else if (ochartProxy._drillAction == "drillup")
                    args.data.label.Text = splitName[ochartProxy._selectedIndex];
                else
                    args.data.label.Text = splitName[splitName.length - 1];
            }
        },

        _getLocalizedLabels: function (property) {

            return ej.olap.OlapChart.locale[this.locale()][property] === undefined ? ej.olap.OlapChart.locale["en-US"][property] : ej.olap.OlapChart.locale[this.locale()][property];
        },

        _onSeriesClick: function (sender) {
            this.element.find("#" + ochartProxy._id + "ExpandMenu, .expandMenu, .e-dialog").remove();
            ochartProxy = this; var tempLabel; var selectedLabel; var pointIndex;
            var selectedTags = new Array(); var menuList = new Array();
            var chartOffset = this.element.position();
            if (sender.data.region != null && sender.data.region != undefined) {
                selectedLabel = sender.model.primaryXAxis.labels[sender.data.region.Region.PointIndex];
                pointIndex = sender.data.region.Region.PointIndex;
            }
            else if (sender.data.point.x != null && sender.data.point.x != undefined && sender.data.point.x != "") {
                selectedLabel = sender.data.series.xAxis.labels[sender.data.point.X];
                pointIndex = sender.data.point.X;
            }

            if ($($("#" + ochartProxy._id + "Container_vml_XAxisLabels_0")[0]).children()[pointIndex] != undefined)
                tempLabel = $($("#" + ochartProxy._id + "Container_vml_XAxisLabels_0")[0]).children()[pointIndex].innerHTML;
            else if ($("#" + ochartProxy._id + "Container_svg_XAxisLabels_0").children()[pointIndex])
                tempLabel = $("#" + ochartProxy._id + "Container_svg_XAxisLabels_0").children()[pointIndex].textContent;
            else if (this.seriesType() == "bar" || this.seriesType() == "stackingbar") {
                if ($($("#" + ochartProxy._id + "Container_vml_YAxisLabels_0")[0]).children()[pointIndex] != undefined)
                    tempLabel = $($("#" + ochartProxy._id + "Container_vml_YAxisLabels_0")[0]).children()[pointIndex].innerHTML;
                else if ($("#" + ochartProxy._id + "Container_svg_YAxisLabels_0").children()[pointIndex])
                    tempLabel = $("#" + ochartProxy._id + "Container_svg_YAxisLabels_0").children()[pointIndex].textContent;
                else {
                    var temp = sender.model.primaryXAxis.labels[pointIndex].split("~~");
                    tempLabel = temp[temp.length - 1];
                }
            }
            else {
                var temp = sender.model.primaryXAxis.labels[pointIndex].split("~~");
                tempLabel = temp[temp.length - 1];
            }

            if (tempLabel) {
                $("#" + ochartProxy._id).find("#" + ochartProxy._id + "ExpandMenu, .expandMenu, .e-dialog").remove();
                var splitLabel = selectedLabel.split("~~");

                jQuery.each(splitLabel, function (index, value) {
                    jQuery.each(ochartProxy._labelCurrentTags, function (key, val) {
                        if ($.trim(value) == val.name) {
                            selectedTags.push(val);
                            return false;
                        }
                    });
                });

                jQuery.each(selectedTags, function (index, value) {
                    if (value.state > 1 && ochartProxy._tagCollection.length == 0)
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Expand")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    else if (value.state > 1 && ochartProxy._tagCollection.length > 0 && value.name == tempLabel) {
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Expand")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    }
                });
                if (ochartProxy._tagCollection.length > 0) {
                    jQuery.each(ochartProxy._tagCollection, function (index, value) {
                        for (var i = 0; i < menuList.length; i++) {
                            if ($.trim(menuList[i].split('-')[1].replace("</li>", "")).replace("</LI>", "") == value.name)
                                menuList.splice(i, 1);
                        }
                        menuList.push($(ej.buildTag("li.menuList", (ochartProxy._getLocalizedLabels("Collapse")) + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                    });
                }
                else if (!ochartProxy._JSONRecords.addInfo.namedsetEnabled) {
                    jQuery.each(selectedTags, function (index, value) {
                        if (value.state == 1) {
                            menuList.push($(ej.buildTag("li.menuList", ochartProxy._getLocalizedLabels("Collapse") + " - " + value.name)[0]).attr("role", "presentation")[0].outerHTML);
                            ochartProxy._tagCollection.push(value);
                        }
                    });
                }
                menuList.push($(ej.buildTag("li.menuList", ochartProxy._getLocalizedLabels("Exit"))[0]).attr("role", "presentation")[0].outerHTML);
                var expandMenu = $(ej.buildTag("div#" + ochartProxy._id + "ExpandMenu.expandMenu", menuList)[0]).attr("role", "presentation")[0].outerHTML;
                $(expandMenu).ejDialog({ width: "auto", target: "#" + ochartProxy._id, enableResize: false });
                $("#" + ochartProxy._id + "ExpandMenu_wrapper").appendTo(this.element).css({ "left": sender.data.location.x + 8 + chartOffset.left, "top": sender.data.location.y + 8 + chartOffset.top, "min-height": "auto" });
                $("#" + ochartProxy._id + "ExpandMenu").css({ "min-height": "auto" });
                this.element.find(".e-titlebar, .e-header").remove();
            }
            ochartProxy._trigger("pointRegionClick", sender);
        },

        exportOlapChart: function (exportOption) {
            var canvasElement = document.createElement('canvas');
            var chartsvgElement = $("#" + this._id + "Container_svg").clone()[0];
            var chartsvg = ej.buildTag("div").append(chartsvgElement).html();
            canvg(canvasElement, chartsvg);
            var chartString = canvasElement.toDataURL("image/png");
            var legendString = "";
            if (this.model.legend.visible) {
                var legendsvgElement = $("#legend_" + this._id + "Container_svg").clone()[0];
                var legendsvg = ej.buildTag("div").append(legendsvgElement).html();
                canvg(canvasElement, legendsvg);
                legendString = canvasElement.toDataURL("image/png");
            }
            var params = {
                args: JSON.stringify({
                    exportOption: exportOption,
                    currentReport: this.getOlapReport(),
                    chartdata: chartString.split(',')[1],
                    legenddata: this.model.legend.visible ? legendString.split(',')[1] : "",
                    bgColor: $(this.element).css('background-color')
                })
            };
            this.doPostBack(this.model.url + "/" + this.model.serviceMethodSettings.exportOlapChart, params);
        },

        doAjaxPost: function (type, url, data, onSuccess) {
            $.ajax({
                type: type,
                url: url,
                data: data,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function () {
                    var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, "element": this.element };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    ochartWaitingPopup.hide();
                    var eventArgs = { "action": this._currentAction, "customObject": this.model.customObject, "element": this.element, "Message": msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        },

        doPostBack: function (url, params) {
            var form = $('<form>').attr({ 'action': url, 'method': 'POST', 'name': 'export' });
            var addParam = function (paramName, paramValue) {
                var input = $('<input type="hidden" title="params">').attr({
                    'id': paramName,
                    'name': paramName,
                    'value': paramValue
                }).appendTo(form);
            };
            for (var item in params)
                addParam(item, params[item]);
            form.appendTo(document.body).submit().remove();
        }
    });

    ej.olap.OlapChart.locale = {};

    ej.olap.OlapChart.locale["en-US"] = {
        Measure: "Measure",
        Row: "Row",
        Column: "Column",
        Value: "Value",
        Expand: "Expand",
        Collapse: "Collapse",
        Exit: "Exit"
    };

    ej.olap.OlapChart.ChartTypes = {
        Line: 'line', Spline: 'spline', Column: 'column', Area: 'area', SplineArea: 'splinearea', StepLine: 'stepline', StepArea: 'steparea', Pie: 'pie', Bar: 'bar', StackingArea: 'stackingarea', StackingColumn: 'stackingcolumn', StackingBar: 'stackingbar', Pyramid: 'pyramid', Funnel: 'funnel'
    };

    ej.olap.OlapChart.ExportOptions = {
        Excel: 'excel',
        Word: 'word',
        PDF: 'pdf',
        CSV: 'csv',
        PNG: 'png',
        JPG: 'jpg',
        EMF: 'emf',
        GIF: 'gif',
        BMP: 'bmp'
    };

    ej.olap.OlapChart.SymbolShapes = {
        None: "none",
        LeftArrow: "leftarrow",
        RightArrow: "rightarrow",
        Circle: "circle",
        Cross: "cross",
        HorizLine: "horizline",
        VertLine: "vertline",
        Diamond: "diamond",
        Rectangle: "rectangle",
        Triangle: "triangle",
        InvertedTriangle: "invertedtriangle",
        Hexagon: "hexagon",
        Pentagon: "pentagon",
        Star: "star",
        Ellipse: "ellipse",
        Wedge: "wedge",
        Trapezoid: "trapezoid",
        UpArrow: "uparrow",
        DownArrow: "downarrow",
        Image: "image"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html OLAP Gauge elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejOlapGauge", "ej.olap.OlapGauge", {

        _rootCSS: "e-olapgauge",
        element: null,
        model: null,
        validTags: ["div", "span"],
        defaults: $.extend({}, ej.datavisualization.CircularGauge.prototype.defaults, {
            url: "",
            cssClass: "",
            rowsCount: 0,
            columnsCount: 0,
            enableTooltip: false,
            isResponsive: false,
            labelFormatSettings: {
                numberFormat: "default",
                decimalPlaces: 5,
                prefixText: "",
                suffixText: ""
            },
            showHeaderLabel: true,
            scales: ej.datavisualization.CircularGauge.prototype._defaultScaleValues(),
            customObject: {},
            locale: "en-US",
            serviceMethodSettings: {
                initialize: "InitializeGauge"
            },
            renderSuccess: null,
            renderComplete: null,
            renderFailure: null,
            beforeServiceInvoke: null,
            afterServiceInvoke: null,
            load: null
        }),

        dataTypes: {
            serviceMethodSettings: "data",
            customObject: "data",
            scales: "data"
        },

        observables: ["rowsCount", "columnsCount", "showHeaderLabel", "locale", "radius", "frameType"],
        rowsCount: ej.util.valueFunction("rowsCount"),
        columnsCount: ej.util.valueFunction("columnsCount"),
        showHeaderLabel: ej.util.valueFunction("showHeaderLabel"),
        locale: ej.util.valueFunction("locale"),
        radius: ej.util.valueFunction("radius"),
        frameType: ej.util.valueFunction("frameType"),

        getOlapReport: function () {
            return this._olapReport;
        },

        setOlapReport: function (value) {
            this._olapReport = value;
        },

        getJSONRecords: function () {
            return this._JSONRecords;
        },

        setJSONRecords: function (value) {
            this._JSONRecords = $.parseJSON(value);
        },

        _init: function () {
            this.model = $.extend(ej.datavisualization.CircularGauge.prototype.defaults, this.model);
            this._scalesInitialize();
            this._initPrivateProperties();
            this._load();
        },

        _scalesInitialize: function () {
            if (this.model.scales != null) {
                $.each(this.model.scales, ej.proxy(function (index, element) {
                    var obj = ej.datavisualization.CircularGauge.prototype._defaultScaleValues();
                    $.extend(obj, element);
                    $.extend(element, obj);
                }, this));
            }
            else {
                this.model.scales = [ej.datavisualization.CircularGauge.prototype._defaultScaleValues()];
            }
        },

        _destroy: function () {
            this._unWireEvents();
            this.element.empty().removeClass("e-olapgauge" + this.model.cssClass);
            if (this._ogaugeWaitingPopup != undefined)
                this._ogaugeWaitingPopup._destroy();
        },

        _initPrivateProperties: function () {
            this._id = this.element.attr("id");
            this._olapReport = "";
            this._JSONRecords = null;
            this._maximum = new Array();
            this._maxValue = 0;
            this._pointerValue_0 = 0;
            this._pointerValue_1 = 0;
            this._currentLayout = "";
            this._defaultArea = 0;
            this._defaultWidth = 0;
            this._gaugeObj = {};
            this._ogaugeWaitingPopup = null;
            this._ogaugeProgressBar = null;
            this._ogaugeProgressText = null;
            this._notationStr = null;
            this._ogaugeTimer = null;
        },

        _load: function () {
            this.element.addClass(this.model.cssClass);
            $("#" + this._id).ejWaitingPopup({ showOnInit: true });
            this._ogaugeWaitingPopup = $("#" + this._id).data("ejWaitingPopup");

            if (this.model.beforeServiceInvoke != null)
                this._trigger("beforeServiceInvoke", { element: this.element, customObject: this.model.customObject });
            var serializedCustomObject = JSON.stringify(this.model.customObject);
            if (this.model.customObject != null && this.model.customObject != undefined && this.model.customObject != {})
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGauge", "customObject": serializedCustomObject }), this._renderControlSuccess);
            else
                this.doAjaxPost("POST", this.model.url + "/" + this.model.serviceMethodSettings.initialize, JSON.stringify({ "action": "initializeGauge" }), this._renderControlSuccess);
        },

        _setFirst: true,

        _setModel: function (options) {
            for (var key in options) {
                switch (key) {
                    case "OlapReport": this.setOlapReport(options[key]); break;
                    case "JsonData": this.setOlapReport(options[key]); break;
                    case "RefreshOlapGauge": this.element.renderControlFromJSON(options[key]); break;
                    case "customObject": this.model.customObject = options[key]; break;
                    case "locale": this.locale(); break;
                    case "rowsCount": this.rowsCount(); break;
                    case "columnsCount": this.columnsCount(); break;
                    case "showHeaderLabel": this.showHeaderLabel(); break;
                    case "locale": this.locale(); break;
                    case "radius": this.radius(); break;
                    case "frameType": this.frameType(); break;
                    case "scales": break;
                }
            }
            if (this._gaugeObj["obj0"]) {
                this._renderControlSuccess({ "PivotRecords": JSON.stringify(this.getJSONRecords()), "OlapReport": this.getOlapReport() });
            }
        },

        _wireEvents: function () {
            if (this.model.enableTooltip) {
                this.element.find("canvas").mouseenter(ej.proxy(function (evt) {
                    this._showTooltip(evt);
                }, this)).mouseout(ej.proxy(function () {
                    $(this._tooltip).fadeOut("slow");
                }, this)).mousemove(ej.proxy(function (evt) {
                    this._showTooltip(evt);
                    this._hideTooltip();
                }, this));
            }
        },

        _getLocalizedLabels: function (property) {
            return ej.olap.OlapGauge.locale[this.locale()][property] === undefined ? ej.olap.OlapGauge.locale["en-US"][property] : ej.olap.OlapGauge.locale[this.locale()][property];
        },

        _unWireEvents: function () {
        },

        _showTooltip: function (event) {
            $(this._tooltip).remove();
            var gaugeIndexValues = event.target.parentNode.id.split("_") == "" ? event.target.parentNode.parentNode.id.split("_") : event.target.parentNode.id.split("_");
            var gaugeIndex = gaugeIndexValues[gaugeIndexValues.length - 1];
            var goal = this._numberFormatConversion(this._JSONRecords[gaugeIndex].GoalValue || 0, this.model.labelFormatSettings.numberFormat, this.model.labelFormatSettings.decimalPlaces);
            var value = this._numberFormatConversion(this._JSONRecords[gaugeIndex].MeasureValue || this._JSONRecords[gaugeIndex].Value, this.model.labelFormatSettings.numberFormat, this.model.labelFormatSettings.decimalPlaces);
            var _factor = this._setMaxScaleValue(this._JSONRecords[gaugeIndex].GoalValue || 0, this._JSONRecords[gaugeIndex].MeasureValue || this._JSONRecords[gaugeIndex].Value, gaugeIndex);
            if (this.model.labelFormatSettings.numberFormat == ej.olap.OlapGauge.NumberFormat.Notation) {
                var goalNValue = (this._JSONRecords[gaugeIndex].GoalValue || 0), measureNValue = (this._JSONRecords[gaugeIndex].MeasureValue || this._JSONRecords[gaugeIndex].Value);
                while (goalNValue >= 10) {
                    goalNValue = parseInt(goalNValue) / 10;
                }
                while (measureNValue >= 10) {
                    measureNValue = parseInt(measureNValue) / 10;
                }
                goal = (goalNValue).toFixed(1) + goal;
                value = (measureNValue).toFixed(1) + value;
            }
            var member = this._JSONRecords[gaugeIndex].MemberName || this._JSONRecords[gaugeIndex].Measure;
            if (member == undefined || member == "undefined")
                member = this._JSONRecords[gaugeIndex].MeasureCaption;
            var tooltipText = member + "<br/>" + this._getLocalizedLabels("RevenueGoal") + ": " + goal + "<br/>" + this._getLocalizedLabels("RevenueValue") + ": " + value;
            this._tooltip = ej.buildTag("div.e-olapgauge-tooltip", tooltipText);
            $(this._tooltip).appendTo('body');
            $(this._tooltip).addClass(".e-olapgauge-active");
            this._changeTooltipPos(event);
        },

        _changeTooltipPos: function (event) {
            var tooltipX = event.pageX - 8;
            var tooltipY = event.pageY + 8;
            $(this._tooltip).css({ top: tooltipY, left: tooltipX });
        },

        _hideTooltip: function () {
            window.setTimeout(ej.proxy(function () { $(this._tooltip).fadeOut(1500, "linear"); }, this), 3000);
        },

        _progressStatus: function (current_Status, completion_Status, progress_Speed) {
            if (jQuery.type(completion_Status) === "string") {
                var progressText = completion_Status.split(':');
                if (progressText != undefined) completion_Status = parseInt(progressText[0]);
            }
            this._ogaugeProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
            $("#" + this._id + "progressContainer").show();
            $("#" + this._id + "blockDiv").position = $("#" + this._id).position();
            this._ogaugeTimer = window.setInterval(ej.proxy(function () {
                this._ogaugeProgressBar.setModel({ value: current_Status, text: current_Status + " %" });
                current_Status++;
                if (current_Status >= completion_Status) {
                    window.clearInterval(this._ogaugeTimer);
                    if (progressText != undefined)
                        this._ogaugeProgressBar.setModel({ value: 100, text: progressText[1] });

                };
            }, this), progress_Speed);
        },

        _renderControlSuccess: function (msg) {
            try {
                var htmlTag = "", statusImage = "", trendImage = "", liTag = "", gaugeId = 0;
                if (msg[0] != undefined && msg.length > 0) {
                    this.setJSONRecords(msg[0].Value);
                    this.setOlapReport(msg[1].Value);
                    if (msg[2] != null && msg[2] != undefined)
                        this.model.customObject = msg[2].Value;
                }
                else if (msg.d != undefined && msg.d.length > 0) {
                    this.setJSONRecords(msg.d[0].Value);
                    this.setOlapReport(msg.d[1].Value);
                    if (msg.d[2] != null && msg.d[2] != undefined)
                        this.model.customObject = msg.d[2].Value;
                }
                else if (msg.PivotRecords && msg.OlapReport) {
                    this.setJSONRecords(msg.PivotRecords);
                    this.setOlapReport(msg.OlapReport);
                    if (msg.customObject != null && msg.customObject != undefined)
                        this.model.customObject = msg.customObject;
                }

                if (this.model.afterServiceInvoke != null)
                    this._trigger("afterServiceInvoke", { element: this.element, customObject: this.model.customObject });
                var ogaugeInfo = this.getJSONRecords();

                if (ogaugeInfo != null && ogaugeInfo.length > 0) {
                    if (this.rowsCount() < 1 && this.columnsCount() < 1) {
                        this._currentLayout = "WrapPanel";
                        for (var k = 0; k < ogaugeInfo.length; k++) {
                            statusImage = ej.buildTag("img#statusIndicator_" + k + ".kpiiconvalue")[0].outerHTML;
                            trendImage = ej.buildTag("img#trendIndicator_" + k + ".kpiiconvalue")[0].outerHTML;
                            liTag += ej.buildTag("li", ej.buildTag("div#" + this._id + "_" + k, statusImage + trendImage)[0].outerHTML)[0].outerHTML;
                        }
                        htmlTag = ej.buildTag("ul.wrapLayout", liTag)[0].outerHTML;
                    }
                    else {
                        this._currentLayout = "Table";
                        if (this.columnsCount() == undefined || this.columnsCount() < 1)
                            this.columnsCount((ogaugeInfo.length / this.rowsCount()).toFixed());
                        if (this.rowsCount() == undefined || this.rowsCount() < 1)
                            this.rowsCount((ogaugeInfo.length / this.columnsCount()).toFixed());
                        htmlTag = "<table><tbody>";
                        for (var i = 0; i < this.rowsCount() ; i++) {
                            htmlTag += "<tr>";
                            for (var j = 0; j < this.columnsCount() ; j++) {
                                statusImage = ej.buildTag("img#statusIndicator_" + gaugeId + ".kpiiconvalue")[0].outerHTML;
                                trendImage = ej.buildTag("img#trendIndicator_" + gaugeId + ".kpiiconvalue")[0].outerHTML;
                                htmlTag += "<td>" + ej.buildTag("div#" + this._id + "_" + gaugeId, statusImage + trendImage)[0].outerHTML + "</td>";
                                gaugeId++;
                            }
                            htmlTag += "</tr>";
                        }
                        htmlTag += "</tbody></table>";
                    }
                    this.element.html(htmlTag);
                    this.renderControlFromJSON(ogaugeInfo);
                    this._defaultWidth = this.element.find("canvas").width();
                    this._defaultArea = this.element.find("table").height() * this.element.find("table").width();
                }
                this._ogaugeWaitingPopup.hide();
                var eventArgs = { element: this.element, customObject: this.model.customObject };
                this._trigger("renderSuccess", eventArgs);
            }
            catch (err) {
            }
        },

        doAjaxPost: function (type, url, data, onSuccess, onComplete) {
            $.ajax({
                type: type,
                url: url,
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                data: data,
                success: $.proxy(onSuccess, this),
                complete: ej.proxy(function () {
                    var eventArgs = { element: this.element, customObject: this.model.customObject };
                    this._trigger("renderComplete", eventArgs);
                }, this),
                error: ej.proxy(function (msg, textStatus, errorThrown) {
                    this._ogaugeWaitingPopup.hide();
                    var eventArgs = { element: this.element, customObject: this.model.customObject, Message: msg };
                    this._trigger("renderFailure", eventArgs);
                }, this)
            });
        },

        progressStatus: function (msg) {

        },

        _getCalculatedValue: function (value) {
            var stringValue = value.toString();
            var factor = 1; var len; var dividend;
            if (stringValue.indexOf('.') > -1)
                len = stringValue.split('.')[0].length;
            else
                len = stringValue.length;
            dividend = parseInt(stringValue);
            for (var i = 0; i < len; i++) {
                if (dividend > this._maxValue) {
                    dividend = dividend / 10;
                    factor = factor * 10;
                }
            }
            return factor;
        },

        _findNearestUpperLimit: function (number, factor) {
            var convertedNumber = number.toString();
            var j = parseInt(convertedNumber);
            var k = j % factor;
            return j + (factor - k);
        },

        _setMaxScaleValue: function (goal, value, i) {
            var factor;
            if (value > goal) {
                factor = this._getCalculatedValue(value);
                this._maximum[i] = this._findNearestUpperLimit((value / factor), 10);
            }
            else {
                factor = this._getCalculatedValue(goal);
                this._maximum[i] = this._findNearestUpperLimit((((goal / factor) * 4) / 3), 10);
            }
            return factor;
        },

        _numberFormatConversion: function (num, type, decimal) {
            switch (type) {
                case ej.olap.OlapGauge.NumberFormat.Default:
                    return parseFloat(num).toFixed(decimal);
                    break;
                case ej.olap.OlapGauge.NumberFormat.Text:
                    return num.toString();
                    break;
                case ej.olap.OlapGauge.NumberFormat.Currency:
                    return Globalize.format(parseInt(num), "c", this.model.locale);
                    break;

                case ej.olap.OlapGauge.NumberFormat.Percentage:
                    {
                        num *= 100;
                        return parseFloat(num).toFixed(2) + "%";
                    }
                    break;
                case ej.olap.OlapGauge.NumberFormat.Fraction:
                    {
                        if (!num) {
                            num = this;
                        }
                        var whole = String(num).split('.')[0];
                        var num = parseFloat("." + String(num).split('.')[1]);
                        num = num.toFixed(4);
                        var decimal = "1";
                        for (z = 0; z < String(num).length - 2; z++) {
                            decimal += "0";
                        }
                        num = num * decimal;
                        decimal = parseInt(decimal);
                        for (z = 2; z < num + 1; z++) {
                            if (num % z == 0 && decimal % z == 0) {
                                num = num / z;
                                decimal = decimal/z;
                                z = 2;
                            }
                        }
                        if (num.toString().length == 2 &&
                                decimal.toString().length == 3) {
                            num = Math.round(Math.round(num) / 10);
                            decimal = Math.round(Math.round(decimal) / 10);
                        }
                        else if (num.toString().length == 2 &&
                                decimal.toString().length == 2) {
                            num = Math.round(num / 10);
                            decimal = Math.round(decimal/10);
                        }
                        var t = _HCF(num, decimal);
                        return ((whole == 0) ? "" : whole + " ") + num / t + "/" + decimal/t;


                    }
                    break;
                case ej.olap.OlapGauge.NumberFormat.Scientific:
                    {
                        return parseFloat(num).toExponential();
                    }
                    break;
                case ej.olap.OlapGauge.NumberFormat.Notation:
                    {
                        var count = 0;
                        var mod = parseInt(num);
                        while (mod >= 1) {
                            var mod = parseInt(mod) / 10;
                            count++;
                        }
                        return (this._notationStr = count == 1 ? "da" : count == 2 ? "h" : count == 3 ? "k" : count == 4 ? "k*da" : count == 5 ? "k*h" : count == 6 ? "M" : count == 7 ? "M*da" : count == 8 ? "M*h" : count == 9 ? "G" : count == 10 ? "G*da" : count == 11 ? "G*h" : count == 12 ? "T" : "");
                    }
                    break;
            }
            function _HCF(u, v) {
                var U = u, V = v
                while (true) {
                    if (!(U %= V)) return V
                    if (!(V %= U)) return U
                }
            }
        },

        removeImg: function () {
            this.element.find("img").remove();
        },

        renderControlFromJSON: function (jsonObj) {
            //Calculating pointer value, danger and safer regions
            this._maxValue = 99, maxGaugeCount = 0;
            var pointerCapRadius = (this.radius() / 22);
            if (this._currentLayout == "Table" && (this.rowsCount() * this.columnsCount()) <= jsonObj.length)
                maxGaugeCount = this.rowsCount() * this.columnsCount();
            else if (this._currentLayout == "WrapPanel" || (this.rowsCount() * this.columnsCount()) > jsonObj.length)
                maxGaugeCount = jsonObj.length;
            for (var i = 0; i < maxGaugeCount; i++) {
                var goal = 0.0, value = 0.0, startValue = 0.0, endValue = 0.0, dValue = 0.0, factor = 0, mark = 0.0,
                percentageValue = 0.0, customLabel = "", imgDiv, extendRanges = new Array(), extendPointers = new Array(),
                extendLabels = new Array(), extendCustomLabels = new Array(), extendTicks = new Array(), extendIndicators = new Array(),
                extendScales = new Array(), statusClass = "", trendClass = "";

                if (jsonObj[i].MeasureValue != null && jsonObj[i].MeasureValue != undefined) {
                    value = parseFloat(jsonObj[i].MeasureValue) || 0;
                    goal = parseFloat(jsonObj[i].GoalValue) || 0;
                    customLabel = this.model.labelFormatSettings.prefixText + " " + jsonObj[i].MeasureCaption + (!($.trim(jsonObj[i].MemberName)) ? "" : " - ") + jsonObj[i].MemberName + " " + this.model.labelFormatSettings.suffixText;
                }
                else {
                    value = parseFloat(jsonObj[i].Value);
                    goal = 0.0;
                    customLabel = jsonObj[i].Measure;
                    this.model.scales[0].showIndicators = false;
                }
                factor = this._setMaxScaleValue(goal, value, i);
                dValue = value;
                mark = goal / factor;
                percentageValue = (dValue / factor);
                this._pointerValue_0 = endValue + percentageValue;
                this._pointerValue_1 = mark + endValue;
                startValue = endValue;
                endValue = endValue + percentageValue;

                //Edting Model values
                if (jsonObj[i].StatusGraphic == "Traffic Signals") {
                    if (jsonObj[i].StatusValue == -1)
                        statusClass = "kpiredroad";
                    else if (jsonObj[i].StatusValue == 0)
                        statusClass = "kpiallcolor";
                    else if (jsonObj[i].StatusValue == 1)
                        statusClass = "kpigreenroad";
                }
                else {
                    if (jsonObj[i].StatusValue == -1)
                        statusClass = "kpidiamond";
                    else if (jsonObj[i].StatusValue == 0)
                        statusClass = "kpitriangle";
                    else if (jsonObj[i].StatusValue == 1)
                        statusClass = "kpicircle";
                }
                imgDiv = this.element.find("#statusIndicator_" + [i]).addClass(statusClass);
                if (imgDiv != null && imgDiv != undefined) {
                    var statusImage = $(imgDiv).css('background-image');
                    var statusIndicator = {
                        height: 15,
                        width: 15,
                        type: "image",
                        imageUrl: (!ej.isNullOrUndefined(statusImage) && statusImage != "none") ? statusImage.split("(")[1].split(")")[0].replace('"', "").replace('"', "") : "",
                        position: { x: this.radius() - 20, y: 260 }, location: { x: this.radius() - 20, y: 260 }
                    };
                }

                if (jsonObj[i].TrendGraphic != null && jsonObj[i].TrendGraphic != undefined && jsonObj[i].TrendGraphic != "" && jsonObj[i].TrendGraphic.toLowerCase() == "standard arrow") {
                    if (jsonObj[i].TrendValue == -1)
                        trendClass = "kpidownarrow";
                    else if (jsonObj[i].TrendValue == 0)
                        trendClass = "kpirightarrow";
                    else if (jsonObj[i].TrendValue == 1)
                        trendClass = "kpiuparrow";
                    imgDiv = this.element.find("#trendIndicator_" + [i]).toggleClass(trendClass);
                }
                if (!ej.isNullOrUndefined(imgDiv)) {
                    var trendImage = $(imgDiv).css('background-image');
                    var trendIndicator = {
                        height: 15,
                        width: 15,
                        type: "image",
                        imageUrl: (!ej.isNullOrUndefined(trendImage) && trendImage != "none") ? trendImage.split("(")[1].split(")")[0].replace('"', "").replace('"', "") : "",
                        position: { x: this.radius(), y: 260 }, location: { x: this.radius(), y: 260 }
                    };
                }
                if (!ej.isNullOrUndefined(this.model.scales[0])) {
                    if (this.frameType() == "fullcircle") {
                        this.model.scales[0].startAngle = 122;
                        this.model.scales[0].sweepAngle = 296;
                    }
                    else if (this.frameType() == "halfcircle") {
                        this.model.scales[0].startAngle = 180;
                        this.model.scales[0].sweepAngle = 180;
                    }
                    this.model.scales[0].indicators = this.model.scales[0].showIndicators ? [statusIndicator, trendIndicator] : this.model.scales[0].indicators;
                    if (!ej.isNullOrUndefined(this.model.scales[0].pointers[0])) {
                        var newPointerCap = ej.datavisualization.CircularGauge.prototype._defaultScaleValues().pointerCap;
                        //this.model.scales[0].pointers[0] = $.extend({}, newPointer, this.model.scales[0].pointers[0]);
                        this.model.scales[0].pointers[0].value = this._pointerValue_0;
                        this.model.scales[0].pointers[0].pointerCap = newPointerCap;
                        this.model.scales[0].pointers[0].pointerCap.radius = pointerCapRadius;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].pointers[1])) {
                        this.model.scales[0].pointers[1].value = this._pointerValue_1;
                        this.model.scales[0].pointers[1].type = this.model.scales[0].pointers[1].type || "marker";
                        this.model.scales[0].pointers[1].markerType = this.model.scales[0].pointers[1].markerType || "diamond";
                        this.model.scales[0].pointers[1].placement = this.model.scales[0].pointers[1].placement || "center";
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].ranges[0])) {
                        this.model.scales[0].ranges[0].startValue = startValue;
                        this.model.scales[0].ranges[0].endValue = mark;
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].ranges[1])) {
                        this.model.scales[0].ranges[1].startValue = mark;
                        this.model.scales[0].ranges[1].endValue = this._maximum[i];
                    }
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabels[0]))
                        this.model.scales[0].customLabels[0].value = this.showHeaderLabel() ? "X " + factor : "";
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabels[1]))
                        this.model.scales[0].customLabels[1].value = this.showHeaderLabel() ? customLabel : "";
                    if (!ej.isNullOrUndefined(this.model.scales[0].customLabels[2]))
                        this.model.scales[0].customLabels[2].value = this.showHeaderLabel() ? customLabel : "";

                    for (var j = 0; j < this.model.scales[0].ranges.length; j++)
                        extendRanges.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().ranges[0], this.model.scales[0].ranges[j]));
                    for (var j = 0; j < this.model.scales[0].pointers.length; j++)
                        extendPointers.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().pointers[0], this.model.scales[0].pointers[j]));
                    for (var j = 0; j < this.model.scales[0].labels.length; j++) {
                        if (this.model.labelFormatSettings.numberFormat == ej.olap.OlapGauge.NumberFormat.Notation && !ej.isNullOrUndefined(ej.datavisualization.CircularGauge.prototype._defaultScaleValues().labels[0])) {
                            this._numberFormatConversion(this._JSONRecords[i].MeasureValue || this._JSONRecords[i].Value, this.model.labelFormatSettings.numberFormat, this.model.labelFormatSettings.decimalPlaces);
                            this.model.scales[0].labels[j].unitText = this._notationStr;
                        }
                        extendLabels.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().labels[0], this.model.scales[0].labels[j]));
                    }
                    for (var j = 0; j < this.model.scales[0].labels.length; j++)
                        extendLabels.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().labels[0], this.model.scales[0].labels[j]));
                    for (var j = 0; j < this.model.scales[0].customLabels.length; j++)
                        extendCustomLabels.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().customLabels[0], this.model.scales[0].customLabels[j]));
                    for (var j = 0; j < this.model.scales[0].ticks.length; j++)
                        extendTicks.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().ticks[0], this.model.scales[0].ticks[j]));
                    if (this.model.scales[0].indicators) {
                        for (var j = 0; j < this.model.scales[0].indicators.length; j++)
                            extendIndicators.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues().indicators[0], this.model.scales[0].indicators[j]));
                    }
                    newScale = {
                        pointerCap: {
                            radius: pointerCapRadius || this.model.scales[0].pointerCap.radius,
                            borderWidth: this.model.scales[0].pointerCap.borderWidth,
                            interiorGradient: this.model.scales[0].pointerCap.interiorGradient,
                            borderColor: this.model.scales[0].pointerCap.borderColor,
                            backgroundColor: this.model.scales[0].pointerCap.backgroundColor
                        },
                        border: {
                            color: this.model.scales[0].border.color,
                            width: this.model.scales[0].border.width
                        },
                        majorIntervalValue: this.model.scales[0].majorIntervalValue,
                        minorIntervalValue: this.model.scales[0].minorIntervalValue,
                        minimum: this.model.scales[0].minimum,
                        backgroundColor: this.model.scales[0].backgroundColor,
                        direction: this.model.scales[0].direction,
                        showPointers: this.model.scales[0].showPointers,
                        showTicks: this.model.scales[0].showTicks,
                        showLabels: this.model.scales[0].showLabels,
                        showIndicators: this.model.scales[0].showIndicators,
                        showRanges: this.model.scales[0].showRanges,
                        showScaleBar: this.model.scales[0].showScaleBar,
                        startAngle: this.model.scales[0].startAngle,
                        sweepAngle: this.model.scales[0].sweepAngle,
                        radius: this.model.scales[0].radius,
                        size: this.model.scales[0].size,
                        maximum: this.model.scales[0].maximum || this._maximum[i],
                        pointers: extendPointers,
                        ticks: extendTicks,
                        labels: extendLabels,
                        ranges: extendRanges,
                        customLabels: extendCustomLabels,
                        indicators: extendIndicators
                    };
                    for (var j = 0; j < this.model.scales.length; j++) {
                        if (j == 0)
                            extendScales.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues(), newScale));
                        else
                            extendScales.push($.extend({}, ej.datavisualization.CircularGauge.prototype._defaultScaleValues(), this.model.scales[j]));
                    }
                }

                //Creating Gauges
                $("#" + this._id + "_" + i).ejCircularGauge({
                    backgroundColor: this.model.backgroundColor,
                    frame: {
                        frameType: this.model.frame.frameType,
                        halfCircleFrameStartAngle: this.model.halfCircleFrameStartAngle,
                        halfCircleFrameEndAngle: this.model.halfCircleFrameEndAngle
                    },

                    radius: this.radius(),
                    width: this.model.width, // Width should be 2 * radius 
                    height: this.model.height,
                    interiorGradient: this.model.interiorGradient,
                    readOnly: this.model.readOnly,
                    enableResize: this.model.isResponsive,

                    enableAnimation: true,
                    animationSpeed: this.model.animationSpeed,
                    theme: this.model.theme,
                    isRadialGradient: this.model.isRadialGradient,

                    load: this.model.load,
                    drawTicks: this.model.drawTicks,
                    drawLabels: this.model.drawLabels,
                    drawPointers: this.model.drawPointers,
                    drawRange: this.model.drawRange,
                    drawCustomLabel: this.model.drawCustomLabel,
                    drawIndicators: this.model.drawIndicators,
                    drawPointerCap: this.model.drawPointerCap,
                    renderComplete: this.model.renderComplete,
                    mouseClick: this.model.mouseClick,
                    mouseClickMove: this.model.mouseClickMove,
                    mouseClickUp: this.model.mouseClickUp,
                    scales: extendScales
                });
                this._gaugeObj["obj" + i] = $("#" + this._id + "_" + i).data("ejCircularGauge");
            }
            this.removeImg();
            this._wireEvents();
        },

        refresh: function () {
            if (this._gaugeObj == null || this._gaugeObj == undefined) {
                for (var i = 0; i < this._JSONRecords.length; i++) {
                    this._gaugeObj["obj" + i] = $("#" + this._id + "_" + i).data("ejCircularGauge");
                    this._gaugeObj["obj" + i].refresh();
                }
            }
            else {
                for (key in this._gaugeObj) {
                    this._gaugeObj[key].refresh();
                }
            }
        }
    });

    ej.olap.OlapGauge.locale = {};

    ej.olap.OlapGauge.locale["en-US"] = {
        RevenueGoal: "Revenue Goal",
        RevenueValue: "Revenue Value"
    };

    ej.olap.OlapGauge.NumberFormat = {
        Default: "default",
        Currency: "currency",
        Percentage: "percentage",
        Fraction: "fraction",
        Scientific: "scientific",
        Text: "text",
        Notation: "notation"
    };

})(jQuery, Syncfusion);;
/**
* @fileOverview Plugin to style the Html Pivot Pager elements
* @copyright Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
*  Use of this code is subject to the terms of our license.
*  A copy of the current license can be obtained at any time by e-mailing
*  licensing@syncfusion.com. Any infringement will be prosecuted under
*  applicable laws. 
* @version 12.1 
* @author <a href="mailto:licensing@syncfusion.com">Syncfusion Inc</a>
*/

(function ($, ej, undefined) {

    ej.widget("ejPivotPager", "ej.PivotPager", {

        _rootCSS: "e-pivotpager",
        element: null,
        model: null,
        _getModel: function () {
            return this.model;
        },
        _setModel: function (value) {
            this.model = value;
        },
        defaults: {
            targetControlID: "",
            categoricalCurrentPage: 1,
            seriesCurrentPage: 1,
            seriesPageCount: 0,
            categoricalPageCount: 0,
            locale: "en-US",
            mode: "both"
        },

        _init: function () {
            if ($("#" + this.model.targetControlID).hasClass('e-pivotgrid')) {
                this.targetControlName = "PivotGrid";
                this.targetControl = $("#" + this.model.targetControlID).data("ejPivotGrid");
                this.targetControl._pagerObj = this;
            }
            this._load();
            this.unwireEvents();
            this.wireEvents();
        },

        _destroy: function () {
            this.element.empty().removeClass("e-pivotpager");
        },

        wireEvents: function () {
            this._on(this.element, "keydown", ".pagerTextBox", this._pagerTextBoxClick);
            this._on(this.element, "click", ".moveNext", this._moveNextPage);
            this._on(this.element, "click", ".moveLast", this._moveLastPage);
            this._on(this.element, "click", ".moveFirst", this._moveFirstPage);
            this._on(this.element, "blur", ".pagerTextBox", this._restorePageNo);
            this._on(this.element, "click", ".movePrevious", this._movePreviousPage);
        },

        unwireEvents: function () {
            this._off(this.element, "click", ".moveNext", this._moveNextPage);
            this._off(this.element, "blur", ".pagerTextBox", this._restorePageNo);
            this._off(this.element, "click", ".moveLast", this._moveLastPage);
            this._off(this.element, "click", ".moveFirst", this._moveFirstPage);
            this._off(this.element, "click", ".movePrevious", this._movePreviousPage);
            this._off(this.element, "keydown", ".pagerTextBox", this._pagerTextBoxClick);
        },

        _getLocalizedLabels: function (property) {
            return ej.PivotPager.locale[this.model.locale][property] === undefined ? ej.PivotPager.locale["en-US"][property] : ej.PivotPager.locale[this.model.locale][property];
        },

        _load: function () {
            $(this.element).addClass("pivotPager").attr("targetControlID", this.model.targetControlID);
            var pagerContent = "";
            if (this.model.mode != "series") {
                var categoricalPager = ej.buildTag("td.categPagerTd", "");
                if (this.model.mode == "both") $(categoricalPager[0]).css("border-right", "1px solid #808080");
                $(categoricalPager).html(ej.buildTag("div.pagerDiv", ej.buildTag("span.moveFirst e-icon")[0].outerHTML + ej.buildTag("span.movePrevious e-icon")[0].outerHTML + ej.buildTag("span.pagerLabel", this._getLocalizedLabels("CategoricalPage"))[0].outerHTML + ej.buildTag("input#" + this._id + "_CategCurrentPage.pagerTextBox")[0].outerHTML + ej.buildTag("span.categPageCount")[0].outerHTML + ej.buildTag("span.moveNext e-icon")[0].outerHTML + ej.buildTag("span.moveLast e-icon")[0].outerHTML));
                pagerContent = categoricalPager[0].outerHTML;
            }
            if (this.model.mode != "categorical") {
                var seriesPager = ej.buildTag("td.seriesPagerTd", "");
                $(seriesPager).html(ej.buildTag("div.pagerDiv", ej.buildTag("span.moveFirst e-icon")[0].outerHTML + ej.buildTag("span.movePrevious e-icon")[0].outerHTML + ej.buildTag("span.pagerLabel", this._getLocalizedLabels("SeriesPage"))[0].outerHTML + ej.buildTag("input#" + this._id + "_SeriesCurrentPage.pagerTextBox")[0].outerHTML + ej.buildTag("span.seriesPageCount")[0].outerHTML + ej.buildTag("span.moveNext e-icon")[0].outerHTML + ej.buildTag("span.moveLast e-icon")[0].outerHTML));
                pagerContent += seriesPager[0].outerHTML;
            }
            var pagerTable = ej.buildTag("table", ej.buildTag("tbody", ej.buildTag("tr", pagerContent)))[0];
            $(this.element).html(pagerTable);
        },

        initPagerProperties: function (headerCounts, pageSettings) {
            this.model.categoricalPageCount = Math.ceil(headerCounts.Column / pageSettings.CategorialPageSize);
            this.model.seriesPageCount = Math.ceil(headerCounts.Row / pageSettings.SeriesPageSize);
            this.model.categoricalCurrentPage = pageSettings.CategorialCurrentPage;
            this.model.seriesCurrentPage = pageSettings.SeriesCurrentPage;
            this.initPagerControl();
        },
        initPagerControl: function (e) {
            if (this.model.mode != "series") {
                $(this.element.find(".categPageCount")[0]).html("/ " + this.model.categoricalPageCount);
                $("#" + this._id + "_CategCurrentPage")[0].value = this.model.categoricalCurrentPage;
                this._setNavigators(this.model.categoricalCurrentPage, this.model.categoricalPageCount, this.element.find(".categPagerTd")[0]);
            }
            if (this.model.mode != "categorical") {
                $(this.element.find(".seriesPageCount")[0]).html("/ " + this.model.seriesPageCount);
                $("#" + this._id + "_SeriesCurrentPage")[0].value = this.model.seriesCurrentPage;
                this._setNavigators(this.model.seriesCurrentPage, this.model.seriesPageCount, this.element.find(".seriesPagerTd")[0]);
            }
        },
		
        _moveNextPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).removeClass("disabled");
                pagerInput.value = parseInt(pagerInput.value) + 1;
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("categorical", pagerInput.value);
                    if (pagerInput.value == this.model.categoricalPageCount) {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveLast")[0]).addClass("disabled");
                    }
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("series", pagerInput.value);
                    if (pagerInput.value == this.model.seriesPageCount) {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveLast")[0]).addClass("disabled");
                    }
                }
            }
        },

        _movePreviousPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveLast")[0]).removeClass("disabled");
                pagerInput.value = parseInt(pagerInput.value) - 1;
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("categorical", pagerInput.value);
                    if (pagerInput.value == "1") {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).addClass("disabled");
                    }
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("series", pagerInput.value);
                    if (pagerInput.value == "1") {
                        $(e.target).addClass("disabled");
                        $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).addClass("disabled");
                    }
                }
            }
        },

        _moveLastPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveFirst")[0]).removeClass("disabled");
                $(e.target).addClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).addClass("disabled");
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value = this.model.categoricalPageCount;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("categorical", pagerInput.value);
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value = this.model.seriesPageCount;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("series", pagerInput.value);
                }
            }
        },

        _moveFirstPage: function (e) {
            if (!$(e.target).hasClass("disabled")) {
                var pagerInput = $($(e.target).parents('td')[0]).find(".pagerTextBox")[0];
                $($($(e.target).parents('td')[0]).find(".moveNext")[0]).removeClass("disabled");
                $($($(e.target).parents('td')[0]).find(".moveLast")[0]).removeClass("disabled");
                $(e.target).addClass("disabled");
                $($($(e.target).parents('td')[0]).find(".movePrevious")[0]).addClass("disabled");
                if (pagerInput.id.indexOf("Categ") != -1) {
                    this.model.categoricalCurrentPage = pagerInput.value = 1;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("categorical", pagerInput.value);
                }
                else if (pagerInput.id.indexOf("Series") != -1) {
                    this.model.seriesCurrentPage = pagerInput.value = 1;
                    if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid("series", pagerInput.value);
                }
            }
        },

        _pagerTextBoxClick: function (e) {
            if (e.which == 13) {
                e.preventDefault();
                var pagerTd = $(e.target).parents('td')[0];
                var pageCount = pagerTd.className.indexOf('categ') != -1 ? this.model.categoricalPageCount : this.model.seriesPageCount;
                if (parseInt(e.target.value) > pageCount || parseInt(e.target.value) < 1 || e.target.value == "") {
                    this._restorePageNo(e);
                    alert("Enter valid page number");
                }
                else {
                    if (pagerTd.className.indexOf('categ') != -1 && this.model.categoricalCurrentPage != parseInt(e.target.value)) {
                        this.model.categoricalCurrentPage = e.target.value;
                        if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid(pagerTd.className, e.target.value);
                    }
                    else if (pagerTd.className.indexOf('series') != -1 && this.model.seriesCurrentPage != parseInt(e.target.value)) {
                        this.model.seriesCurrentPage = e.target.value;
                        if (this.targetControlName == "PivotGrid") this.targetControl.refreshPagedPivotGrid(pagerTd.className, e.target.value);
                    }
                    this._setNavigators(e.target.value, pageCount, pagerTd);
                    e.target.blur();
                }
            }
            else {
                if ($.inArray(e.keyCode, [46, 8, 9, 27, 13, 190]) !== -1 || (e.keyCode == 65 && e.ctrlKey === true) || (e.keyCode >= 35 && e.keyCode <= 39)) return;
                if ((e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) && (e.keyCode < 96 || e.keyCode > 105)) e.preventDefault();
            }
        },

        _restorePageNo: function (e) { e.target.value = e.target.id.indexOf("Categ") != -1 ? this.model.categoricalCurrentPage : this.model.seriesCurrentPage; },

        _setNavigators: function (value, pageCount, pagerTd) {
            if (value == pageCount) {
                $($(pagerTd).find(".moveNext")[0]).addClass("disabled");
                $($(pagerTd).find(".moveLast")[0]).addClass("disabled");
            }
            if (value == "1") {
                $($(pagerTd).find(".movePrevious")[0]).addClass("disabled");
                $($(pagerTd).find(".moveFirst")[0]).addClass("disabled");
            }
            if (value > 1) {
                $($(pagerTd).find(".movePrevious")[0]).removeClass("disabled");
                $($(pagerTd).find(".moveFirst")[0]).removeClass("disabled");
            }
            if (value < pageCount) {
                $($(pagerTd).find(".moveNext")[0]).removeClass("disabled");
                $($(pagerTd).find(".moveLast")[0]).removeClass("disabled");
            }
        }
    })

    ej.PivotPager.locale = {};

    ej.PivotPager.locale["en-US"] = {
        SeriesPage: "Series Page",
        CategoricalPage: "Categorical Page"
    };

    ej.PivotPager.Mode = {
        Both: "both",
        Categorical: "categorical",
        Series: "series"
    };

})(jQuery, Syncfusion);

;